package ObsoleteModelica3 "Library that contains components from Modelica Standard Library 2.2.2 that have been removed from version 3.0"
  annotation(uses(Modelica(version = "3.1")), Documentation(info = "<html>
<p>
This package contains models and blocks from the Modelica Standard Library
version 2.2.2 that are no longer available in version 3.0.
The conversion script for version 3.0 changes references in existing
user models automatically to the models and blocks of package
ObsoleteModelica3. The user should <b>manually</b> replace all
references to ObsoleteModelica3 in his/her models to the models
that are recommended in the documentation of the respective model.
</p>

<p>
In most cases, this means that a model with the name
\"ObsoleteModelica3.XXX\" should be renamed to \"Modelica.XXX\" (version 3.0)
and then a manual adaptation is needed. For example, a reference to
ObsoleteModelica3.Mechanics.MultiBody.Sensors.AbsoluteSensor
should be replaced by
Modelica.Mechanics.MultiBody.Sensors.AbsoluteSensor (version 3.0).
Since the design of the component has changed (e.g., several
optional connectors, and no longer one connector where all signals
are packed together), this requires some changes at the place where
the model is used (besides the renaming of the underlying class).
</p>

<p>
The models in ObsoleteModelica3 are either not according to the Modelica Language
version 3.0 and higher, or the model was changed to get a better design.
In all cases, an automatic conversion to the new implementation
was not feasible, since too complicated.
</p>

<p>
In order to easily detect obsolete models and blocks, all of them are specially
marked in the icon layer with a red box.
</p>

<p>
Copyright &copy; 2007-2009, Modelica Association.
</p>
<p>
<i>This Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p>

</html>"));
  package Blocks "Library of basic input/output control blocks (continuous, discrete, logical, table blocks)"
    package Interfaces "Library of connectors and partial models for input/output blocks"
      connector RealSignal = Real "Real port (both input/output possible)" annotation(__Dymola_obsolete = "Connector is not valid according to Modelica 3, since input/output prefixes are missing. When using this connector, it is not possible to check for balanced models.", Documentation(info = "<html>
<p>
Connector with one signal of type Real (no icon, no input/output prefix).
</p>
</html>"));
      connector BooleanSignal = Boolean "Boolean port (both input/output possible)" annotation(__Dymola_obsolete = "Connector is not valid according to Modelica 3, since input/output prefixes are missing. When using this connector, it is not possible to check for balanced models.", Documentation(info = "<html>
<p>
Connector with one signal of type Boolean (no icon, no input/output prefix).
</p>
</html>"));
      connector IntegerSignal = Integer "Integer port (both input/output possible)" annotation(__Dymola_obsolete = "Connector is not valid according to Modelica 3, since input/output prefixes are missing. When using this connector, it is not possible to check for balanced models.", Documentation(info = "<html>
<p>
Connector with one signal of type Icon (no icon, no input/output prefix).
</p>
</html>"));
      package Adaptors "Obsolete package with components to send signals to a bus or receive signals from a bus (only for backward compatibility)"
        model AdaptorReal "Completely obsolete adaptor between 'old' and 'new' Real signal connectors (only for backward compatibility)"
          extends ObsoleteModelica3.Icons.ObsoleteBlock;
          ObsoleteModelica3.Blocks.Interfaces.RealSignal newReal "Connector of Modelica version 2.1" annotation(Hide = true, Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
          RealPort oldReal(final n = 1) "Connector of Modelica version 1.6" annotation(Placement(transformation(extent = {{ -120, -10},{ -100,10}}, rotation = 0)));
          annotation(structurallyIncomplete, __Dymola_obsolete = "Model is not balanced, so equation check will not work. This model is no longer needed", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,40},{100, -40}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -144,96},{144,46}}, lineColor = {0,0,0}, textString = ""),Text(extent = {{ -88,22},{88, -24}}, lineColor = {0,0,255}, textString = "adaptor"),Text(extent = {{ -216, -58},{36, -80}}, lineColor = {0,0,0}, textString = "port.signal")}), Documentation(info = "<html>
<p>
Completely obsolete adaptor between the Real signal connector
of version 1.6 and version &ge; 2.1 of the Modelica Standard Library.
This block is only provided for backward compatibility.
</p>
</html>
"));
        protected 
          connector RealPort "Connector with signals of type Real"
            parameter Integer n = 1 "Dimension of signal vector" annotation(Hide = true);
            replaceable type SignalType = Real "type of signal";
            SignalType signal[n] "Real signals" annotation(Hide = true);
          end RealPort;

        equation 
          newReal = oldReal.signal[1];
        end AdaptorReal;
        model AdaptorBoolean "Completely obsolete adaptor between 'old' and 'new' Boolean signal connectors (only for backward compatibility)"
          extends ObsoleteModelica3.Icons.ObsoleteBlock;
          ObsoleteModelica3.Blocks.Interfaces.BooleanSignal newBoolean "Connector of Modelica version 2.1" annotation(Hide = true, Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
          BooleanPort oldBoolean(final n = 1) "Connector of Modelica version 1.6" annotation(Placement(transformation(extent = {{ -120, -10},{ -100,10}}, rotation = 0)));
          annotation(structurallyIncomplete, __Dymola_obsolete = "Model is not balanced, so equation check will not work. This model is no longer needed", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,40},{100, -40}}, lineColor = {255,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -144,96},{144,46}}, lineColor = {0,0,0}, textString = ""),Text(extent = {{ -88,22},{88, -24}}, lineColor = {255,0,255}, textString = "adaptor"),Text(extent = {{ -216, -58},{36, -80}}, lineColor = {0,0,0}, textString = "port.signal")}), Documentation(info = "<html>
<p>
Completely obsolete adaptor between the Real signal connector
of version 1.6 and version &ge; 2.1 of the Modelica Standard Library.
This block is only provided for backward compatibility.
</p>
</html>
"));
        protected 
          connector BooleanPort "Connector with signals of type Boolean"
            parameter Integer n = 1 "Dimension of signal vector" annotation(Hide = true);
            replaceable type SignalType = Boolean "type of signal";
            SignalType signal[n] "Boolean signals" annotation(Hide = true);
          end BooleanPort;

        equation 
          newBoolean = oldBoolean.signal[1];
        end AdaptorBoolean;
        model AdaptorInteger "Completely obsolete adaptor between 'old' and 'new' Integer signal connectors (only for backward compatibility)"
          extends ObsoleteModelica3.Icons.ObsoleteBlock;
          ObsoleteModelica3.Blocks.Interfaces.IntegerSignal newInteger "Connector of Modelica version 2.1" annotation(Hide = true, Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
          IntegerPort oldInteger(final n = 1) "Connector of Modelica version 1.6" annotation(Placement(transformation(extent = {{ -120, -10},{ -100,10}}, rotation = 0)));
          annotation(structurallyIncomplete, __Dymola_obsolete = "Model is not balanced, so equation check will not work. This model is no longer needed", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,40},{100, -40}}, lineColor = {255,127,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -144,96},{144,46}}, lineColor = {0,0,0}, textString = ""),Text(extent = {{ -88,22},{88, -24}}, lineColor = {255,127,0}, textString = "adaptor"),Text(extent = {{ -216, -58},{36, -80}}, lineColor = {0,0,0}, textString = "port.signal")}), Documentation(info = "<html>
<p>
Completely obsolete adaptor between the Real signal connector
of version 1.6 and version &ge; 2.1 of the Modelica Standard Library.
This block is only provided for backward compatibility.
</p>
</html>
"));
        protected 
          connector IntegerPort "Connector with signals of type Integer"
            parameter Integer n = 1 "Dimension of signal vector" annotation(Hide = true);
            replaceable type SignalType = Integer "type of signal";
            SignalType signal[n] "Integer signals" annotation(Hide = true);
          end IntegerPort;

        equation 
          newInteger = oldInteger.signal[1];
        end AdaptorInteger;
      end Adaptors;
    end Interfaces;
    package Math "Library of mathematical functions as input/output blocks"
      package UnitConversions "Conversion blocks to convert between SI and non-SI unit signals"
        block ConvertAllUnits "Obsolete block. Use one of Modelica.Blocks.Math.UnitConversions.XXX instead"
          replaceable block ConversionBlock = Modelica.Blocks.Interfaces.PartialConversionBlock "Conversion block" annotation(choicesAllMatching = true, Documentation(info = "<html>
<p>
Internal replaceable block that is used to construct the
\"pull down menu\" of the available unit conversions.
</p>
</html>"));
          extends ConversionBlock;
          extends ObsoleteModelica3.Icons.ObsoleteBlock;
          annotation(__Dymola_obsolete = "Model is not according to Modelica Language 3.0 since replaceable base class present. " + "Use instead one of Modelica.Blocks.Math.UnitConversions.XXX", defaultComponentName = "convert", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -90,0},{30,0}}, color = {191,0,0}),Polygon(points = {{90,0},{30,20},{30, -20},{90,0}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>This block implements the Modelica.SIunits.Conversions functions as a fixed causality block to
simplify their use. The block contains a replaceable block class <b>ConversionBlock</b> that can be
changed to be any of the blocks defined in Modelica.Blocks.Math.UnitConversions, and more generally, any
blocks that extend from Modelica.Blocks.Interfaces.PartialConversionBlock.
</p>

<p
The desired conversion can be selected in the parameter menu
(the selected units are then displayed in the icon):</p>
</p>

<p>
<img src=\"../Modelica/Images/Blocks/ConvertAllUnits.png\">
</p>

</html>"));
        end ConvertAllUnits;
      end UnitConversions;
      block TwoInputs "Obsolete block. Use instead Modelica.Blocks.Math.InverseBlockConstraints"
        extends Modelica.Blocks.Interfaces.BlockIcon;
        extends ObsoleteModelica3.Icons.ObsoleteBlock;
        annotation(structurallyIncomplete, __Dymola_obsolete = "Model is not balanced, i.e., not according to Modelica Language 3.0. Use instead Modelica.Blocks.Math.InverseBlockConstraints", Window(x = 0.15, y = 0.21, width = 0.6, height = 0.6), Documentation(info = "<HTML>
<p>
This block is used to enable asignment of values to variables preliminary
defined as outputs (e.g. useful for inverse model generation).
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -95,50},{95, -50}}, lineColor = {0,0,127}, textString = "=")}));
        Modelica.Blocks.Interfaces.RealInput u1 "Connector of first Real input signal" annotation(layer = "icon", Placement(transformation(extent = {{ -139.742, -19.0044},{ -100,20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u2 "Connector of second Real input signal (u1=u2)" annotation(layer = "icon", Placement(transformation(origin = {120,0}, extent = {{ -20, -20},{20,20}}, rotation = 180)));

      equation 
        u1 = u2;
      end TwoInputs;
      block TwoOutputs "Obsolete block. Use instead Modelica.Blocks.Math.InverseBlockConstraints"
        extends Modelica.Blocks.Interfaces.BlockIcon;
        extends ObsoleteModelica3.Icons.ObsoleteBlock;
        annotation(structurallyIncomplete, __Dymola_obsolete = "Model is not balanced, i.e., not according to Modelica Language 3.0. Use instead Modelica.Blocks.Math.InverseBlockConstraints", Window(x = 0.21, y = 0.28, width = 0.6, height = 0.6), Documentation(info = "
<HTML>
<p>
This block is used to enable calculation of values preliminary defined as inputs.
(e.g. useful for inverse model generation).
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -95,50},{95, -50}}, lineColor = {0,0,127}, textString = "=")}));
        output Modelica.Blocks.Interfaces.RealOutput y1 "Connector of first Real output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        output Modelica.Blocks.Interfaces.RealOutput y2 "Connector of second Real output signal (y1=y2)" annotation(Placement(transformation(origin = { -110.366, -0.90289}, extent = {{ -10.0005, -10},{10.0005,10}}, rotation = 180)));

      equation 
        y1 = y2;
      end TwoOutputs;
    end Math;
  end Blocks;
  package Electrical "Library of electrical models (analog, digital, machines, multi-phase)"
    package Analog "Library for analog electrical models"
      package Basic "Basic electrical components such as resistor, capacitor, transformer"
        model HeatingResistor "Obsolete model. Use Modelica.Electrical.Analog.Basic.HeatingResistor instead"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          extends ObsoleteModelica3.Icons.ObsoleteModel;
          parameter Modelica.SIunits.Resistance R_ref = 1 "Resistance at temperature T_ref";
          parameter Modelica.SIunits.Temperature T_ref = 300 "Reference temperature";
          parameter Real alpha(unit = "1/K") = 0 "Temperature coefficient of resistance";
          Modelica.SIunits.Resistance R "Resistance = R_ref*(1 + alpha*(heatPort.T - T_ref));";
          annotation(__Dymola_obsolete = "Model equations depend on cardinality(..) which will become obsolete in the Modelica language. Use instead Modelica.Electrical.Analog.Basic.HeatingResistor", Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -110,20},{ -85,20}}, color = {160,160,164}),Polygon(points = {{ -95,23},{ -85,20},{ -95,17},{ -95,23}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Line(points = {{90,20},{115,20}}, color = {160,160,164}),Line(points = {{ -125,0},{ -115,0}}, color = {160,160,164}),Line(points = {{ -120, -5},{ -120,5}}, color = {160,160,164}),Text(extent = {{ -110,25},{ -90,45}}, lineColor = {160,160,164}, textString = "i"),Polygon(points = {{105,23},{115,20},{105,17},{105,23}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Line(points = {{115,0},{125,0}}, color = {160,160,164}),Text(extent = {{90,45},{110,25}}, lineColor = {160,160,164}, textString = "i"),Rectangle(extent = {{ -70,30},{70, -30}}),Line(points = {{ -96,0},{ -70,0}}),Line(points = {{70,0},{96,0}}),Line(points = {{0, -30},{0, -90}}, color = {191,0,0}),Line(points = {{ -52, -50},{48,50}}, color = {0,0,255}),Polygon(points = {{40,52},{50,42},{54,56},{40,52}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid)}), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -142,60},{143,118}}, textString = "%name"),Line(points = {{ -90,0},{ -70,0}}),Line(points = {{70,0},{90,0}}),Rectangle(extent = {{ -70,30},{70, -30}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0, -30},{0, -91}}, color = {191,0,0}),Line(points = {{ -52, -50},{48,50}}, color = {0,0,255}),Polygon(points = {{40,52},{50,42},{54,56},{40,52}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
<p>This is a model for an electrical resistor where the generated heat
is dissipated to the environment via connector <b>heatPort</b> and where
the resistance R is temperature dependent according to the following
equation:</p>
<pre>    R = R_ref*(1 + alpha*(heatPort.T - T_ref))
</pre>
<p><b>alpha</b> is the <b>temperature coefficient of resistance</b>, which
is often abbreviated as <b>TCR</b>. In resistor catalogues, it is usually
defined as <b>X [ppm/K]</b> (parts per million, similarly to per centage)
meaning <b>X*1.e-6 [1/K]</b>. Resistors are available for 1 .. 7000 ppm/K,
i.e., alpha = 1e-6 .. 7e-3 1/K;</p>
<p>When connector <b>heatPort</b> is <b>not</b> connected, the temperature
dependent behaviour is switched off by setting heatPort.T = T_ref.
Additionally, the equation <tt>heatPort.Q_flow = 0</tt> is implicitly present
due to a special rule in Modelica that flow variables of not connected
connectors are set to zero.</p>
</HTML>
", revisions = "<html>
<ul>
<li><i>  </i>
       </li>
<li><i> 2002   </i>
       by Anton Haumer<br> initially implemented<br>
       </li>
</ul>
</html>"));
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort annotation(Placement(transformation(origin = {0, -100}, extent = {{10, -10},{ -10,10}}, rotation = 270)));

        equation 
          v = R * i;
          if cardinality(heatPort) > 0 then
            R = R_ref * (1 + alpha * (heatPort.T - T_ref));
            heatPort.Q_flow =  -v * i;

          else
            R = R_ref;
            heatPort.T = T_ref;

          end if;
        end HeatingResistor;
      end Basic;
    end Analog;
  end Electrical;
  package Icons "Library of icons"
    partial block ObsoleteBlock "Icon for an obsolete block (use only for this case)"
      annotation(__Dymola_obsolete = "Only used to mark an obsolete block. Do not use otherwise.", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -102,102},{102, -102}}, lineColor = {255,0,0}, pattern = LinePattern.Dash, lineThickness = 0.5)}), Documentation(info = "<html>
<p>
This partial block is intended to provide a <u>default icon
for an obsolete block</u> that will be removed from the
corresponding library in a future release.
<p>
</html>"));
    end ObsoleteBlock;
    partial model ObsoleteModel "Icon for an obsolete model (use only for this case)"
      annotation(__Dymola_obsolete = "Only used to mark an obsolete model. Do not use otherwise.", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -102,102},{102, -102}}, lineColor = {255,0,0}, pattern = LinePattern.Dash, lineThickness = 0.5)}), Documentation(info = "<html>
<p>
This partial model is intended to provide a <u>default icon
for an obsolete model</u> that will be removed from the
corresponding library in a future release.
<p>
</html>"));
    end ObsoleteModel;
    partial class Enumeration "Obsolete class (icon for an enumeration emulated by a package). Use a real enumeration instead"
      annotation(__Dymola_obsolete = "Icon for an emulated enumeration. Emulated enumerations are no longer used (only real enumerations)", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -138,164},{138,104}}, textString = "%name"),Ellipse(extent = {{ -100,100},{100, -100}}, lineColor = {255,0,127}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -100,100},{100, -100}}, lineColor = {255,0,127}, textString = "e")}), Documentation(info = "<html>
<p>
This icon is designed for an <b>enumeration</b>
(that is emulated by a package).
</p>
</html>"));
    end Enumeration;
  end Icons;
  package Mechanics "Library of 1-dim. and 3-dim. mechanical components (multi-body, rotational, translational)"
    package MultiBody "Library to model 3-dimensional mechanical systems"
      package Forces "Components that exert forces and/or torques between frames"
        model WorldForceAndTorque "Obsolete model. Use instead Modelica.Mechanics.MultiBody.Forces.WorldForceAndTorque"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Types;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialOneFrame_b;
          extends ObsoleteModelica3.Icons.ObsoleteModel;
          Modelica.Blocks.Interfaces.RealInput load[6] "[1:6] = x-, y-, z-coordinates of force and x-, y-, z-coordiantes of torque resolved in world frame" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Real N_to_m(unit = "N/m") = world.defaultN_to_m " Force arrow scaling (length = force/N_to_m)" annotation(Dialog(group = "if animation = true", enable = animation));
          parameter Real Nm_to_m(unit = "N.m/m") = world.defaultNm_to_m " Torque arrow scaling (length = torque/Nm_to_m)" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter forceDiameter = world.defaultArrowDiameter " Diameter of force arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter torqueDiameter = forceDiameter " Diameter of torque arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color forceColor = Modelica.Mechanics.MultiBody.Types.Defaults.ForceColor " Color of force arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color torqueColor = Modelica.Mechanics.MultiBody.Types.Defaults.TorqueColor " Color of torque arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(group = "if animation = true", enable = animation));
          annotation(__Dymola_obsolete = "Based on a packed result signal which is not a good design. Use instead Modelica.Mechanics.MultiBody.Forces.WorldForceAndTorque", preferedView = "info", Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -100,10},{50,10},{50,31},{97,0},{50, -31},{50, -10},{ -100, -10},{ -100,10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -100,11},{ -94,24},{ -86,39},{ -74,59},{ -65,71},{ -52,83},{ -35,92},{ -22,95},{ -8,95},{7,91},{19,84},{32,76},{44,66},{52,58},{58,51}}, color = {0,0,0}, thickness = 0.5),Polygon(points = {{97,18},{72,77},{38,42},{97,18}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -74,62},{44,24}}, lineColor = {192,192,192}, textString = "world"),Polygon(points = {{ -100,10},{50,10},{50,31},{94,0},{50, -31},{50, -10},{ -100, -10},{ -100,10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -137, -47},{148, -108}}, textString = "%name"),Line(points = {{ -98,14},{ -92,27},{ -84,42},{ -72,62},{ -63,74},{ -50,86},{ -33,95},{ -20,98},{ -6,98},{9,94},{21,87},{34,79},{46,69},{54,61},{60,54}}, color = {0,0,0}, thickness = 0.5),Polygon(points = {{99,21},{74,80},{40,45},{99,21}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
<p>
The <b>6</b> signals of the <b>load</b> connector are interpreted
as the x-, y- and z-coordinates of a <b>force</b> and as
the x-, y-, and z-coordinates of a <b>torque</b> resolved in the
<b>world frame</b> and acting at the frame connector to which this
component is attached. The input signals are mapped to the force
and torque in the following way:
</p>
<pre>
   force  = load[1:3]
   torque = load[4:6]
</pre>
<p>
The force and torque are by default visualized as an arrow (force)
and as a double arrow (torque) acting at the connector to which
they are connected. The diameters
and colors of the arrows are fixed and can be defined via
parameters <b>forceDiameter</b>, <b>torqueDiameter</b>,
<b>forceColor</b> and <b>torqueColor</b>. The arrows
point in the directions defined by the
inPort.signal signals. The lengths of the arrows are proportional
to the length of the force and torque vectors, respectively, using parameters
<b>N_to_m</b> and <b>Nm_to_m</b> as scaling factors. For example, if N_to_m = 100 N/m,
then a force of 350 N is displayed as an arrow of length 3.5 m.
</p>
<p>
An example how to use this model is given in the
following figure:
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Forces/WorldForceAndTorque1.png\">
</p>
<p>
This leads to the following animation
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Forces/WorldForceAndTorque2.png\">
</p>
</HTML>
"));
          annotation(Coordsys(extent = [ -100, -100;100,100], grid = [1,1], component = [20,20]), Window(x = 0.18, y = 0.01, width = 0.8, height = 0.82), Documentation(info = "
An external force element exerts the inport signal
as negative force on the connector frame (the force vector
is resolved in the world frame).
"), Icon(Text(extent = [ -132,99;128,39], string = "%name"), Polygon(points = [ -100,10;49,10;49,31;100,0;49, -31;49, -10; -100, -10; -100,10], style(color = 0, gradient = 0, fillColor = 0, fillPattern = 1))), Diagram(Polygon(points = [ -90,10;40,10;40,31;91,0;40, -31;40, -10; -90, -10; -90,10], style(color = 0, gradient = 0, fillColor = 0, fillPattern = 1))));
        protected 
          SI.Position f_in_m[3] = frame_b.f / N_to_m "Force mapped from N to m for animation";
          SI.Position t_in_m[3] = frame_b.t / Nm_to_m "Torque mapped from Nm to m for animation";
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Arrow forceArrow(diameter = forceDiameter, color = forceColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = f_in_m, r_head =  -f_in_m) if world.enableAnimation and animation;
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.DoubleArrow torqueArrow(diameter = torqueDiameter, color = torqueColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = t_in_m, r_head =  -t_in_m) if world.enableAnimation and animation;

        equation 
          frame_b.f =  -Modelica.Mechanics.MultiBody.Frames.resolve2(frame_b.R, load[1:3]);
          frame_b.t =  -Modelica.Mechanics.MultiBody.Frames.resolve2(frame_b.R, load[4:6]);
        end WorldForceAndTorque;
        model FrameForceAndTorque "Obsolete model. Use instead Modelica.Mechanics.MultiBody.Forces.ForceAndTorque"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Types;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialOneFrame_b;
          extends ObsoleteModelica3.Icons.ObsoleteModel;
          Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve "If connected, the input signals are resolved in this frame" annotation(Placement(transformation(origin = {0,100}, extent = {{16, -16},{ -16,16}}, rotation = 270)));
          Modelica.Blocks.Interfaces.RealInput load[6] "[1:6] = x-, y-, z-coordinates of force and x-, y-, z-coordiantes of torque resolved in frame_b or frame_resolved (if connected)" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Real N_to_m(unit = "N/m") = world.defaultN_to_m " Force arrow scaling (length = force/N_to_m)" annotation(Dialog(group = "if animation = true", enable = animation));
          parameter Real Nm_to_m(unit = "N.m/m") = world.defaultNm_to_m " Torque arrow scaling (length = torque/Nm_to_m)" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter forceDiameter = world.defaultArrowDiameter " Diameter of force arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter torqueDiameter = forceDiameter " Diameter of torque arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color forceColor = Modelica.Mechanics.MultiBody.Types.Defaults.ForceColor " Color of force arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color torqueColor = Modelica.Mechanics.MultiBody.Types.Defaults.TorqueColor " Color of torque arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(group = "if animation = true", enable = animation));
          annotation(__Dymola_obsolete = "Based on a packed result signal which is not a good design. Use instead Modelica.Mechanics.MultiBody.Forces.ForceAndTorque", preferedView = "info", Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -100,10},{50,10},{50,31},{97,0},{50, -31},{50, -10},{ -100, -10},{ -100,10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -100,11},{ -94,24},{ -86,39},{ -74,59},{ -65,71},{ -52,83},{ -35,92},{ -22,95},{ -8,95},{7,91},{19,84},{32,76},{44,66},{52,58},{58,51}}, color = {0,0,0}),Polygon(points = {{97,18},{72,77},{38,42},{97,18}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{0,97},{0,10}}, color = {95,95,95}, pattern = LinePattern.Dot)}), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -74,62},{44,24}}, lineColor = {192,192,192}, textString = "resolve"),Polygon(points = {{ -100,10},{50,10},{50,31},{94,0},{50, -31},{50, -10},{ -100, -10},{ -100,10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -137, -47},{148, -108}}, textString = "%name"),Line(points = {{ -100,10},{ -92,26},{ -84,42},{ -76,52},{ -60,68},{ -46,76},{ -31,82},{ -17,85},{ -2,87},{14,86},{26,82},{37,75},{46,69},{54,61},{60,54}}, color = {0,0,0}),Polygon(points = {{99,21},{74,80},{40,45},{99,21}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{0,95},{0,10}}, color = {95,95,95}, pattern = LinePattern.Dot)}), Documentation(info = "<HTML>
<p>
The <b>6</b> signals of the <b>load</b> connector are interpreted
as the x-, y- and z-coordinates of a <b>force</b> and as
the x-, y-, and z-coordinates of a <b>torque</b> acting at the frame
connector to which this component is attached. If connector
<b>frame_resolve</b> is <b>not</b> connected, the force and torque coordinates
are with respect to <b>frame_b</b>. If connector
<b>frame_resolve</b> is connected, the force and torque coordinates
are with respect to <b>frame_resolve</b>. In this case the
force and torque in connector frame_resolve are set to zero,
i.e., this connector is solely used to provide the information
of the coordinate system, in which the force coordinates
are defined. The input signals are mapped to the force
and torque in the following way:
</p>
<pre>
   force  = load[1:3]
   torque = load[4:6]
</pre>
<p>
The force and torque are by default visualized as an arrow (force)
and as a double arrow (torque) acting at the connector to which
they are connected. The diameters
and colors of the arrows are fixed and can be defined via
parameters <b>forceDiameter</b>, <b>torqueDiameter</b>,
<b>forceColor</b> and <b>torqueColor</b>. The arrows
point in the directions defined by the
inPort.signal signals. The lengths of the arrows are proportional
to the length of the force and torque vectors, respectively, using parameters
<b>N_to_m</b> and <b>Nm_to_m</b> as scaling factors. For example,
if N_to_m = 100 N/m,
then a force of 350 N is displayed as an arrow of length 3.5 m.
</p>
<p>
An example how to use this model is given in the
following figure:
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Forces/FrameForceAndTorque1.png\">
</p>
<p>
This leads to the following animation
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Forces/FrameForceAndTorque2.png\">
</p>
</HTML>
"));
          annotation(Coordsys(extent = [ -100, -100;100,100], grid = [1,1], component = [20,20]), Window(x = 0.18, y = 0.01, width = 0.8, height = 0.82), Documentation(info = "
An external force element exerts the inport signal
as negative force on the connector frame (the force vector
is resolved in the world frame).
"), Icon(Text(extent = [ -132,99;128,39], string = "%name"), Polygon(points = [ -100,10;49,10;49,31;100,0;49, -31;49, -10; -100, -10; -100,10], style(color = 0, gradient = 0, fillColor = 0, fillPattern = 1))), Diagram(Polygon(points = [ -90,10;40,10;40,31;91,0;40, -31;40, -10; -90, -10; -90,10], style(color = 0, gradient = 0, fillColor = 0, fillPattern = 1))));
        protected 
          SI.Position f_in_m[3] = frame_b.f / N_to_m "Force mapped from N to m for animation";
          SI.Position t_in_m[3] = frame_b.t / Nm_to_m "Torque mapped from Nm to m for animation";
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Arrow forceArrow(diameter = forceDiameter, color = forceColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = f_in_m, r_head =  -f_in_m) if world.enableAnimation and animation;
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.DoubleArrow torqueArrow(diameter = torqueDiameter, color = torqueColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = t_in_m, r_head =  -t_in_m) if world.enableAnimation and animation;

        equation 
          if cardinality(frame_resolve) == 0 then
            frame_b.f =  -load[1:3];
            frame_b.t =  -load[4:6];
            frame_resolve.r_0 = zeros(3);
            frame_resolve.R = Modelica.Mechanics.MultiBody.Frames.nullRotation();

          else
            frame_b.f =  -Modelica.Mechanics.MultiBody.Frames.resolveRelative(load[1:3], frame_resolve.R, frame_b.R);
            frame_b.t =  -Modelica.Mechanics.MultiBody.Frames.resolveRelative(load[4:6], frame_resolve.R, frame_b.R);
            frame_resolve.f = zeros(3);
            frame_resolve.t = zeros(3);

          end if;
        end FrameForceAndTorque;
        model ForceAndTorque "Obsolete model. Use instead Modelica.Mechanics.MultiBody.Forces.ForceAndTorque"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Types;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
          extends ObsoleteModelica3.Icons.ObsoleteModel;
          Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve "If connected, the input signals are resolved in this frame" annotation(Placement(transformation(origin = {40,100}, extent = {{ -16, -16},{16,16}}, rotation = 90)));
          Modelica.Blocks.Interfaces.RealInput load[6] "[1:6] = x-, y-, z-coordinates of force and x-, y-, z-coordiantes of torque resolved in frame_b or frame_resolved (if connected)" annotation(Placement(transformation(origin = { -60,120}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Real N_to_m(unit = "N/m") = world.defaultN_to_m " Force arrow scaling (length = force/N_to_m)" annotation(Dialog(group = "if animation = true", enable = animation));
          parameter Real Nm_to_m(unit = "N.m/m") = world.defaultNm_to_m " Torque arrow scaling (length = torque/Nm_to_m)" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter forceDiameter = world.defaultArrowDiameter " Diameter of force arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter torqueDiameter = forceDiameter " Diameter of torque arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter connectionLineDiameter = forceDiameter " Diameter of line connecting frame_a and frame_b" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color forceColor = Modelica.Mechanics.MultiBody.Types.Defaults.ForceColor " Color of force arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color torqueColor = Modelica.Mechanics.MultiBody.Types.Defaults.TorqueColor " Color of torque arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color connectionLineColor = Modelica.Mechanics.MultiBody.Types.Defaults.SensorColor " Color of line connecting frame_a and frame_b" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(group = "if animation = true", enable = animation));
          SI.Position r_0[3] "Position vector from origin of frame_a to origin of frame_b resolved in world frame";
          SI.Force f_b_0[3] "frame_b.f resoved in world frame";
          SI.Torque t_b_0[3] "frame_b.t resoved in world frame";
          annotation(__Dymola_obsolete = "Based on a packed result signal which is not a good design. Use instead Modelica.Mechanics.MultiBody.Forces.ForceAndTorque", preferedView = "info", Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -98,99},{99, -98}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -59,55},{72,30}}, lineColor = {192,192,192}, textString = "resolve"),Text(extent = {{ -136, -52},{149, -113}}, textString = "%name"),Polygon(points = {{100,21},{84,55},{69,39},{100,21}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{40,100},{40,0}}, color = {95,95,95}, pattern = LinePattern.Dot),Polygon(points = {{ -95,1},{ -64,11},{ -64, -10},{ -95,1}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{ -100,20},{ -86,53},{ -70,42},{ -100,20}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -60,100},{40,100}}, color = {95,95,95}, pattern = LinePattern.Dot),Polygon(points = {{94,0},{65,12},{65, -11},{94,0}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -64,0},{ -20,0}}, color = {0,0,0}),Line(points = {{20,0},{65,0}}, color = {0,0,0}),Line(points = {{ -79,47},{ -70,61},{ -59,72},{ -45,81},{ -32,84},{ -20,85}}, color = {0,0,0}),Line(points = {{76,47},{66,60},{55,69},{49,74},{41,80},{31,84},{20,85}}, color = {0,0,0})}), Documentation(info = "<HTML>
<p>
The <b>6</b> signals of the <b>load</b> connector are interpreted
as the x-, y- and z-coordinates of a <b>force</b> and as
the x-, y-, and z-coordinates of a <b>torque</b> acting at the frame
connector to which frame_b of this component is attached. If connector
<b>frame_resolve</b> is <b>not</b> connected, the force and torque coordinates
are with respect to <b>frame_b</b>. If connector
<b>frame_resolve</b> is connected, the force and torque coordinates
are with respect to <b>frame_resolve</b>. In this case the
force and torque in connector frame_resolve are set to zero,
i.e., this connector is solely used to provide the information
of the coordinate system, in which the force/torque coordinates
are defined. The input signals are mapped to the force
and torque in the following way:
</p>
<pre>
   force  = load[1:3]
   torque = load[4:6]
</pre>
<p>
Additionally, a force and torque acts on frame_a in such a way that
the force and torque balance between frame_a and frame_b is fulfilled.
</p>
<p>
An example how to use this model is given in the
following figure:
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Forces/ForceAndTorque1.png\">
</p>
<p>
This leads to the following animation (the yellow cylinder
characterizes the line between frame_a and frame_b of the
ForceAndTorque component, i.e., the force and torque acts with
negative sign
also on the opposite side of this cylinder, but for
clarity this is not shown in the animation):
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Forces/ForceAndTorque2.png\">
</p>
</HTML>
"));
          annotation(Coordsys(extent = [ -100, -100;100,100], grid = [1,1], component = [20,20]), Window(x = 0.18, y = 0.01, width = 0.8, height = 0.82), Documentation(info = "
An external force element exerts the inport signal
as negative force on the connector frame (the force vector
is resolved in the world frame).
"), Icon(Text(extent = [ -132,99;128,39], string = "%name"), Polygon(points = [ -100,10;49,10;49,31;100,0;49, -31;49, -10; -100, -10; -100,10], style(color = 0, gradient = 0, fillColor = 0, fillPattern = 1))), Diagram(Polygon(points = [ -90,10;40,10;40,31;91,0;40, -31;40, -10; -90, -10; -90,10], style(color = 0, gradient = 0, fillColor = 0, fillPattern = 1))));
        protected 
          SI.Position f_in_m[3] = frame_b.f / N_to_m "Force mapped from N to m for animation";
          SI.Position t_in_m[3] = frame_b.t / Nm_to_m "Torque mapped from Nm to m for animation";
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Arrow forceArrow(diameter = forceDiameter, color = forceColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = f_in_m, r_head =  -f_in_m) if world.enableAnimation and animation;
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.DoubleArrow torqueArrow(diameter = torqueDiameter, color = torqueColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = t_in_m, r_head =  -t_in_m) if world.enableAnimation and animation;
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape connectionLine(shapeType = "cylinder", lengthDirection = r_0, widthDirection = {0,1,0}, length = Modelica.Math.Vectors.length(r_0), width = connectionLineDiameter, height = connectionLineDiameter, color = connectionLineColor, specularCoefficient = specularCoefficient, r = frame_a.r_0) if world.enableAnimation and animation;

        equation 
          if cardinality(frame_resolve) == 0 then
            frame_b.f =  -load[1:3];
            frame_b.t =  -load[4:6];
            f_b_0 = Modelica.Mechanics.MultiBody.Frames.resolve1(frame_b.R, frame_b.f);
            t_b_0 = Modelica.Mechanics.MultiBody.Frames.resolve1(frame_b.R, frame_b.t);
            frame_resolve.r_0 = zeros(3);
            frame_resolve.R = Modelica.Mechanics.MultiBody.Frames.nullRotation();

          else
            f_b_0 =  -Modelica.Mechanics.MultiBody.Frames.resolve1(frame_resolve.R, load[1:3]);
            t_b_0 =  -Modelica.Mechanics.MultiBody.Frames.resolve1(frame_resolve.R, load[4:6]);
            frame_b.f = Modelica.Mechanics.MultiBody.Frames.resolve2(frame_b.R, f_b_0);
            frame_b.t = Modelica.Mechanics.MultiBody.Frames.resolve2(frame_b.R, t_b_0);
            frame_resolve.f = zeros(3);
            frame_resolve.t = zeros(3);

          end if;
          r_0 = frame_b.r_0 - frame_a.r_0;
          zeros(3) = frame_a.f + Modelica.Mechanics.MultiBody.Frames.resolve2(frame_a.R, f_b_0);
          zeros(3) = frame_a.t + Modelica.Mechanics.MultiBody.Frames.resolve2(frame_a.R, t_b_0 + cross(r_0, f_b_0));
        end ForceAndTorque;
      end Forces;
      package Interfaces "Connectors and partial models for 3-dim. mechanical components"
        partial model PartialCutForceSensor "Obsolete model. Use instead instead a model from Modelica.Mechanics.MultiBody.Sensors"
          extends Modelica.Icons.RotationalSensor;
          extends ObsoleteModelica3.Icons.ObsoleteModel;
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Coordinate system with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_b "Coordinate system with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{84, -16},{116,16}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve "If connected, the output signals are resolved in this frame (cut-force/-torque are set to zero)" annotation(Placement(transformation(origin = {80, -100}, extent = {{ -16, -16},{16,16}}, rotation = 270)));
          annotation(Window(x = 0.37, y = 0.02, width = 0.6, height = 0.65), __Dymola_obsolete = "Model equations depend on cardinality(..) which will become obsolete in the Modelica language. Use instead a model from Modelica.Mechanics.MultiBody.Sensors", Documentation(info = "
<HTML>
<p>
This is a base class for 3-dim. mechanical components with two frames
and one output port in order to measure the cut-force and/or
cut-torque acting between the two frames and
to provide the measured signals as output for further processing
with the blocks of package Modelica.Blocks.
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -101,0}}, color = {0,0,0}),Line(points = {{70,0},{100,0}}, color = {0,0,0}),Line(points = {{ -80, -100},{ -80,0}}, color = {0,0,127}),Text(extent = {{ -132,76},{129,124}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -118,55},{ -82,30}}, lineColor = {128,128,128}, textString = "a"),Text(extent = {{83,55},{119,30}}, lineColor = {128,128,128}, textString = "b"),Text(extent = {{ -31, -72},{100, -97}}, lineColor = {192,192,192}, textString = "resolve"),Line(points = {{80,0},{80, -100}}, color = {95,95,95}, pattern = LinePattern.Dot)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -100,0}}, color = {0,0,0}),Line(points = {{70,0},{100,0}}, color = {0,0,0}),Line(points = {{ -80, -100},{ -80,0}}, color = {0,0,127}),Line(points = {{80,0},{80, -100}}, color = {95,95,95}, pattern = LinePattern.Dot)}));
        protected 
          outer Modelica.Mechanics.MultiBody.World world;

        equation 
          defineBranch(frame_a.R, frame_b.R);
          assert(cardinality(frame_a) > 0, "Connector frame_a of cut-force/-torque sensor object is not connected");
          assert(cardinality(frame_b) > 0, "Connector frame_b of cut-force/-torque sensor object is not connected");
          frame_a.r_0 = frame_b.r_0;
          frame_a.R = frame_b.R;
          zeros(3) = frame_a.f + frame_b.f;
          zeros(3) = frame_a.t + frame_b.t;
          if cardinality(frame_resolve) == 1 then
            frame_resolve.f = zeros(3);
            frame_resolve.t = zeros(3);

          else
            frame_resolve.r_0 = zeros(3);
            frame_resolve.R = Modelica.Mechanics.MultiBody.Frames.nullRotation();

          end if;
        end PartialCutForceSensor;
      end Interfaces;
      package Joints
        package Internal
          model RevoluteWithLengthConstraint "Obsolete model. Use instead Modelica.Mechanics.MultiBody.Joints.Internal.RevoluteWithLengthConstraint"
            import SI = Modelica.SIunits;
            import Cv = Modelica.SIunits.Conversions;
            extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
            extends ObsoleteModelica3.Icons.ObsoleteModel;
            Modelica.Mechanics.Rotational.Interfaces.Flange_a axis "1-dim. rotational flange that drives the joint" annotation(Placement(transformation(extent = {{10,90},{ -10,110}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_b bearing "1-dim. rotational flange of the drive bearing" annotation(Placement(transformation(extent = {{ -50,90},{ -70,110}}, rotation = 0)));
            Modelica.Blocks.Interfaces.RealInput position_a[3] "Position vector from frame_a to frame_a side of length constraint, resolved in frame_a of revolute joint" annotation(Placement(transformation(extent = {{ -140, -80},{ -100, -40}}, rotation = 0)));
            Modelica.Blocks.Interfaces.RealInput position_b[3] "Position vector from frame_b to frame_b side of length constraint, resolved in frame_b of revolute joint" annotation(Placement(transformation(extent = {{140, -80},{100, -40}}, rotation = 0)));
            parameter Boolean animation = true "= true, if animation shall be enabled";
            parameter SI.Position lengthConstraint = 1 "Fixed length of length constraint";
            parameter Modelica.Mechanics.MultiBody.Types.Axis n = {0,0,1} "Axis of rotation resolved in frame_a (= same as in frame_b)" annotation(Evaluate = true);
            parameter Cv.NonSIunits.Angle_deg phi_offset = 0 "Relative angle offset (angle = phi + from_deg(phi_offset))";
            parameter Cv.NonSIunits.Angle_deg phi_guess = 0 "Select the configuration such that at initial time |phi - from_deg(phi_guess)|is minimal";
            parameter SI.Distance cylinderLength = world.defaultJointLength "Length of cylinder representing the joint axis" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of cylinder representing the joint axis" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Modelica.Mechanics.MultiBody.Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinder representing the joint axis" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Modelica.Mechanics.MultiBody.Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter Boolean axisTorqueBalance = true "= true, if torque balance of flange axis with the frame_b connector (axis.tau = -e*frame_b.t) shall be defined. Otherwise this equation has to be provided outside of this joint" annotation(Dialog(tab = "Advanced"));
            final parameter Boolean positiveBranch(fixed = false) "Based on phi_guess, selection of one of the two solutions of the non-linear constraint equation";
            final parameter Real e[3](each final unit = "1") = Modelica.Math.Vectors.normalize(n) "Unit vector in direction of rotation axis, resolved in frame_a";
            SI.Angle phi "Rotation angle of revolute joint";
            Modelica.Mechanics.MultiBody.Frames.Orientation R_rel "Relative orientation object from frame_a to frame_b";
            SI.Angle angle "= phi + from_deg(phi_offset) (relative rotation angle between frame_a and frame_b)";
            SI.Torque tau "= axis.tau (driving torque in the axis)";
            annotation(structurallyIncomplete, preferedView = "info", __Dymola_obsolete = "Obsolete model that is not balanced. Use instead Modelica.Mechanics.MultiBody.Joints.Internal.RevoluteWithLengthConstraint", Window(x = 0.05, y = 0.09, width = 0.65, height = 0.69), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -30,10},{10, -10}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -100, -60},{ -30,60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{30, -60},{100,60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Text(extent = {{ -139, -168},{137, -111}}, textString = "%name"),Rectangle(extent = {{ -100,60},{ -30, -60}}, lineColor = {0,0,0}),Rectangle(extent = {{30,60},{100, -60}}, lineColor = {0,0,0}),Text(extent = {{ -142, -108},{147, -69}}, lineColor = {0,0,0}, textString = "n=%n"),Line(points = {{ -60,60},{ -60,90}}, color = {0,0,0}),Line(points = {{ -20,70},{ -60,70}}, color = {0,0,0}),Line(points = {{ -20,80},{ -20,60}}, color = {0,0,0}),Line(points = {{20,80},{20,60}}, color = {0,0,0}),Line(points = {{20,70},{41,70}}, color = {0,0,0}),Polygon(points = {{ -9,30},{10,30},{30,50},{ -29,50},{ -9,30}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{10,30},{30,50},{30, -51},{10, -31},{10,30}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -10,90},{10,50}}, lineColor = {0,0,0}, fillPattern = FillPattern.VerticalCylinder, fillColor = {192,192,192})}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100, -60},{ -30,60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -30,10},{10, -10}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{30, -60},{100,60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Line(points = {{ -60,60},{ -60,96}}, color = {0,0,0}),Line(points = {{ -20,70},{ -60,70}}, color = {0,0,0}),Line(points = {{ -20,80},{ -20,60}}, color = {0,0,0}),Line(points = {{20,80},{20,60}}, color = {0,0,0}),Line(points = {{20,70},{41,70}}, color = {0,0,0}),Polygon(points = {{ -9,30},{10,30},{30,50},{ -29,50},{ -9,30}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{10,30},{30,50},{30, -51},{10, -31},{10,30}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -10,50},{10,100}}, lineColor = {0,0,0}, fillPattern = FillPattern.VerticalCylinder, fillColor = {192,192,192})}), Documentation(info = "<HTML>
<p>
Joint where frame_b rotates around axis n which is fixed in frame_a.
The two frames coincide when \"phi + phi_offset = 0\", where
\"phi_offset\" is a parameter with a zero default
and \"phi\" is the rotation angle.
</p>
<p>
This variant of the revolute joint is designed to work together
with a length constraint in a kinematic loop. This means that the
angle of the revolute joint, phi, is computed such that the
length constraint is fulfilled.
</p>
<p>
<b>Usually, this joint should not be used by a user of the MultiBody
library. It is only provided to built-up the Modelica.Mechanics.MultiBody.Joints.Assemblies.JointXYZ
joints.</b>
</p>
</HTML>
"));
          protected 
            SI.Position r_a[3] = position_a "Position vector from frame_a to frame_a side of length constraint, resolved in frame_a of revolute joint";
            SI.Position r_b[3] = position_b "Position vector from frame_b to frame_b side of length constraint, resolved in frame_b of revolute joint";
            Real e_r_a "Projection of r_a on e";
            Real e_r_b "Projection of r_b on e";
            Real A "Coefficient A of equation: A*cos(phi) + B*sin(phi) + C = 0";
            Real B "Coefficient B of equation: A*cos(phi) + B*sin(phi) + C = 0";
            Real C "Coefficient C of equation: A*cos(phi) + B*sin(phi) + C = 0";
            Real k1 "Constant of quadratic equation";
            Real k2 "Constant of quadratic equation";
            Real k1a(start = 1);
            Real k1b;
            Real kcos_angle "= k1*cos(angle)";
            Real ksin_angle "= k1*sin(angle)";
            Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape cylinder(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = e, widthDirection = {0,1,0}, r_shape =  -e * cylinderLength / 2, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
            function selectBranch "Determine branch which is closest to initial angle=0"
              import Modelica.Math.*;
              input SI.Length L "Length of length constraint";
              input Real e[3](each final unit = "1") "Unit vector along axis of rotation, resolved in frame_a (= same in frame_b)";
              input SI.Angle angle_guess "Select the configuration such that at initial time |angle-angle_guess|is minimal (angle=0: frame_a and frame_b coincide)";
              input SI.Position r_a[3] "Position vector from frame_a to frame_a side of length constraint, resolved in frame_a of revolute joint";
              input SI.Position r_b[3] "Position vector from frame_b to frame_b side of length constraint, resolved in frame_b of revolute joint";
              output Boolean positiveBranch "Branch of the initial solution";
            protected 
              Real e_r_a "Projection of r_a on e";
              Real e_r_b "Projection of r_b on e";
              Real A "Coefficient A of equation: A*cos(phi) + B*sin(phi) + C = 0";
              Real B "Coefficient B of equation: A*cos(phi) + B*sin(phi) + C = 0";
              Real C "Coefficient C of equation: A*cos(phi) + B*sin(phi) + C = 0";
              Real k1 "Constant of quadratic equation";
              Real k2 "Constant of quadratic equation";
              Real kcos1 "k1*cos(angle1)";
              Real ksin1 "k1*sin(angle1)";
              Real kcos2 "k2*cos(angle2)";
              Real ksin2 "k2*sin(angle2)";
              SI.Angle angle1 "solution 1 of nonlinear equation";
              SI.Angle angle2 "solution 2 of nonlinear equation";
            algorithm 
              e_r_a:=e * r_a;
              e_r_b:=e * r_b;
              A:= -2 * (r_b * r_a - e_r_b * e_r_a);
              B:=2 * r_b * cross(e, r_a);
              C:=r_a * r_a + r_b * r_b - L * L - 2 * e_r_b * e_r_a;
              k1:=A * A + B * B;
              k2:=sqrt(k1 - C * C);
              kcos1:= -A * C + B * k2;
              ksin1:= -B * C - A * k2;
              angle1:=atan2(ksin1, kcos1);
              kcos2:= -A * C - B * k2;
              ksin2:= -B * C + A * k2;
              angle2:=atan2(ksin2, kcos2);
              if abs(angle1 - angle_guess) <= abs(angle2 - angle_guess) then 
                            positiveBranch:=true;

              else               positiveBranch:=false;

              end if;
            end selectBranch;

          initial equation 
            positiveBranch = selectBranch(lengthConstraint, e, Cv.from_deg(phi_offset + phi_guess), r_a, r_b);

          equation 
            Connections.branch(frame_a.R, frame_b.R);
            axis.tau = tau;
            axis.phi = phi;
            bearing.phi = 0;
            angle = Cv.from_deg(phi_offset) + phi;
            frame_b.r_0 = frame_a.r_0;
            R_rel = Modelica.Mechanics.MultiBody.Frames.planarRotation(e, angle, der(angle));
            frame_b.R = Modelica.Mechanics.MultiBody.Frames.absoluteRotation(frame_a.R, R_rel);
            zeros(3) = frame_a.f + Modelica.Mechanics.MultiBody.Frames.resolve1(R_rel, frame_b.f);
            zeros(3) = frame_a.t + Modelica.Mechanics.MultiBody.Frames.resolve1(R_rel, frame_b.t);
if axisTorqueBalance then
                          tau =  -e * frame_b.t;
            end if;
            e_r_a = e * r_a;
            e_r_b = e * r_b;
            A =  -2 * (r_b * r_a - e_r_b * e_r_a);
            B = 2 * r_b * cross(e, r_a);
            C = r_a * r_a + r_b * r_b - lengthConstraint * lengthConstraint - 2 * e_r_b * e_r_a;
            k1 = A * A + B * B;
            k1a = k1 - C * C;
            assert(k1a > 1e-10, "
Singular position of loop (either no or two analytic solutions;
the mechanism has lost one-degree-of freedom in this position).
Try first to use another Modelica.Mechanics.MultiBody.Joints.Assemblies.JointXXX component.
In most cases it is best that the joints outside of the JointXXX
component are revolute and NOT prismatic joints. If this also
lead to singular positions, it could be that this kinematic loop
cannot be solved analytically. In this case you have to build
up the loop with basic joints (NO aggregation JointXXX components)
and rely on dynamic state selection, i.e., during simulation
the states will be dynamically selected in such a way that in no
position a degree of freedom is lost.
");
            k1b = Modelica.Mechanics.MultiBody.Frames.Internal.maxWithoutEvent(k1a, 1e-12);
            k2 = sqrt(k1b);
            kcos_angle =  -A * C + (if positiveBranch then B else  -B) * k2;
            ksin_angle =  -B * C + (if positiveBranch then  -A else A) * k2;
            angle = Modelica.Math.atan2(ksin_angle, kcos_angle);
          end RevoluteWithLengthConstraint;
          model PrismaticWithLengthConstraint "Obsolete model. Use instead Modelica.Mechanics.MultiBody.Joints.Internal.PrismaticWithLengthConstraint"
            import SI = Modelica.SIunits;
            import Cv = Modelica.SIunits.Conversions;
            extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
            extends ObsoleteModelica3.Icons.ObsoleteModel;
            Modelica.Mechanics.Translational.Interfaces.Flange_a axis "1-dim. translational flange that drives the joint" annotation(Placement(transformation(extent = {{70,80},{90,60}}, rotation = 0)));
            Modelica.Mechanics.Translational.Interfaces.Flange_b bearing "1-dim. translational flange of the drive bearing" annotation(Placement(transformation(extent = {{ -30,80},{ -50,60}}, rotation = 0)));
            Modelica.Blocks.Interfaces.RealInput position_a[3] "Position vector from frame_a to frame_a side of length constraint, resolved in frame_a of revolute joint" annotation(Placement(transformation(extent = {{ -140, -80},{ -100, -40}}, rotation = 0)));
            Modelica.Blocks.Interfaces.RealInput position_b[3] "Position vector from frame_b to frame_b side of length constraint, resolved in frame_b of revolute joint" annotation(Placement(transformation(extent = {{140, -80},{100, -40}}, rotation = 0)));
            parameter Boolean animation = true "= true, if animation shall be enabled";
            parameter SI.Position length = 1 "Fixed length of length constraint";
            parameter Modelica.Mechanics.MultiBody.Types.Axis n = {1,0,0} "Axis of translation resolved in frame_a (= same as in frame_b)" annotation(Evaluate = true);
            parameter SI.Position s_offset = 0 "Relative distance offset (distance between frame_a and frame_b = s(t) + s_offset)";
            parameter SI.Position s_guess = 0 "Select the configuration such that at initial time |s(t0)-s_guess|is minimal";
            parameter Modelica.Mechanics.MultiBody.Types.Axis boxWidthDirection = {0,1,0} "Vector in width direction of box, resolved in frame_a" annotation(Evaluate = true, Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance boxWidth = world.defaultJointWidth "Width of prismatic joint box" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance boxHeight = boxWidth "Height of prismatic joint box" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Modelica.Mechanics.MultiBody.Types.Color boxColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of prismatic joint box" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Modelica.Mechanics.MultiBody.Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter Boolean axisForceBalance = true "= true, if force balance of flange axis with the frame_b connector (axis.f = -e*frame_b.f) shall be defined. Otherwise this equation has to be provided outside of this joint" annotation(Dialog(tab = "Advanced"));
            final parameter Boolean positiveBranch(fixed = false) "Selection of one of the two solutions of the non-linear constraint equation";
            final parameter Real e[3](each final unit = "1") = Modelica.Math.Vectors.normalize(n) "Unit vector in direction of translation axis, resolved in frame_a";
            SI.Position s "Relative distance between frame_a and frame_b along axis n = s + s_offset)";
            SI.Position distance "Relative distance between frame_a and frame_b along axis n";
            SI.Position r_rel_a[3] "Position vector from frame_a to frame_b resolved in frame_a";
            SI.Force f "= axis.f (driving force in the axis)";
            annotation(structurallyIncomplete, preferedView = "info", __Dymola_obsolete = "Obsolete model that is not balanced. Use instead Modelica.Mechanics.MultiBody.Joints.Internal.PrismaticWithLengthConstraint", Window(x = 0.05, y = 0.09, width = 0.65, height = 0.69), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -30, -40},{100,30}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -30,40},{100, -40}}, lineColor = {0,0,0}),Rectangle(extent = {{ -100, -60},{ -30,50}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -100,50},{ -30,60}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -30,30},{100,40}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -136, -170},{140, -113}}, textString = "%name"),Rectangle(extent = {{ -100,60},{ -30, -60}}, lineColor = {0,0,0}),Line(points = {{100, -40},{100, -60}}),Rectangle(extent = {{100,40},{90,80}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -136, -116},{153, -77}}, lineColor = {0,0,0}, textString = "n=%n")}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -30, -50},{ -30,50}}, color = {0,0,0}),Line(points = {{0, -67},{90, -67}}, color = {128,128,128}),Text(extent = {{31, -68},{68, -81}}, lineColor = {128,128,128}, textString = "s"),Line(points = {{ -100, -67},{0, -67}}, color = {128,128,128}),Polygon(points = {{ -39, -64},{ -29, -67},{ -39, -70},{ -39, -64}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{ -77, -70},{ -43, -85}}, lineColor = {128,128,128}, textString = "s_offset"),Line(points = {{ -100, -71},{ -100, -51}}, color = {128,128,128}),Line(points = {{ -30, -73},{ -30, -33}}, color = {128,128,128}),Line(points = {{100, -70},{100, -30}}, color = {128,128,128}),Polygon(points = {{90, -64},{100, -67},{90, -70},{90, -64}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -100,50},{ -30,60}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -100, -60},{ -30,50}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -30,40},{100, -40}}, lineColor = {0,0,0}),Rectangle(extent = {{ -30, -40},{100,30}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -30,30},{100,40}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -100,60},{ -30, -60}}, lineColor = {0,0,0}),Line(points = {{100, -40},{100, -60}}),Text(extent = {{42,91},{57,76}}, textString = "f"),Line(points = {{40,75},{70,75}}, color = {0,0,255}),Polygon(points = {{ -21,78},{ -31,75},{ -21,72},{ -21,78}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Line(points = {{ -8,75},{ -31,75}}, color = {0,0,255}),Text(extent = {{ -21,90},{ -6,75}}, textString = "f"),Polygon(points = {{60,78},{70,75},{60,72},{60,78}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Line(points = {{ -30,64},{70,64}}, color = {128,128,128}),Polygon(points = {{60,67},{70,64},{60,61},{60,67}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{0,63},{37,50}}, lineColor = {128,128,128}, textString = "s"),Rectangle(extent = {{100,40},{90,80}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
<p>
Joint where frame_b is translated along axis n which is fixed in frame_a.
The two frames coincide when \"s + s_offset = 0\", where
\"s_offset\" is a parameter with a zero default
and \"s\" is the relative distance.
</p>
<p>
This variant of the prismatic joint is designed to work together
with a length constraint in a kinematic loop. This means that the
relative distance \"s\" of the joint is computed such that the
length constraint is fulfilled.
</p>
<p>
<b>Usually, this joint should not be used by a user of the MultiBody
library. It is only provided to built-up the Modelica.Mechanics.MultiBody.Joints.Assemblies.JointXYZ
joints.</b>
</p>
</HTML>
"));
          protected 
            SI.Position r_a[3] = position_a "Position vector from frame_a to frame_a side of length constraint, resolved in frame_a of revolute joint";
            SI.Position r_b[3] = position_b "Position vector from frame_b to frame_b side of length constraint, resolved in frame_b of revolute joint";
            Modelica.SIunits.Position rbra[3] "= rb - ra";
            Real B "Coefficient B of equation: s*s + B*s + C = 0";
            Real C "Coefficient C of equation: s*s + B*s + C = 0";
            Real k1 "Constant of quadratic equation solution";
            Real k2 "Constant of quadratic equation solution";
            Real k1a(start = 1);
            Real k1b;
            Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape box(shapeType = "box", color = boxColor, specularCoefficient = specularCoefficient, length = if noEvent(abs(s + s_offset) > 1e-06) then s + s_offset else 1e-06, width = boxWidth, height = boxHeight, lengthDirection = e, widthDirection = boxWidthDirection, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
            function selectBranch "Determine branch which is closest to initial angle=0"
              import Modelica.Math.*;
              input SI.Length L "Length of length constraint";
              input Real e[3](each final unit = "1") "Unit vector along axis of translation, resolved in frame_a (= same in frame_b)";
              input SI.Position d_guess "Select the configuration such that at initial time |d-d_guess|is minimal (d: distance between origin of frame_a and origin of frame_b)";
              input SI.Position r_a[3] "Position vector from frame_a to frame_a side of length constraint, resolved in frame_a of prismatic joint";
              input SI.Position r_b[3] "Position vector from frame_b to frame_b side of length constraint, resolved in frame_b of prismatic joint";
              output Boolean positiveBranch "Branch of the initial solution";
            protected 
              Modelica.SIunits.Position rbra[3] "= rb - ra";
              Real B "Coefficient B of equation: d*d + B*d + C = 0";
              Real C "Coefficient C of equation: d*d + B*d + C = 0";
              Real k1 "Constant of quadratic equation solution";
              Real k2 "Constant of quadratic equation solution";
              Real d1 "solution 1 of quadratic equation";
              Real d2 "solution 2 of quadratic equation";
            algorithm 
              rbra:=r_b - r_a;
              B:=2 * e * rbra;
              C:=rbra * rbra - L * L;
              k1:=B / 2;
              k2:=sqrt(k1 * k1 - C);
              d1:= -k1 + k2;
              d2:= -k1 - k2;
              if abs(d1 - d_guess) <= abs(d2 - d_guess) then 
                            positiveBranch:=true;

              else               positiveBranch:=false;

              end if;
            end selectBranch;

          initial equation 
            positiveBranch = selectBranch(length, e, s_offset + s_guess, r_a, r_b);

          equation 
            Connections.branch(frame_a.R, frame_b.R);
            axis.f = f;
            axis.s = s;
            bearing.s = 0;
            distance = s_offset + s;
            r_rel_a = e * distance;
            frame_b.r_0 = frame_a.r_0 + Modelica.Mechanics.MultiBody.Frames.resolve1(frame_a.R, r_rel_a);
            frame_b.R = frame_a.R;
            zeros(3) = frame_a.f + frame_b.f;
            zeros(3) = frame_a.t + frame_b.t + cross(r_rel_a, frame_b.f);
if axisForceBalance then
                          f =  -e * frame_b.f;
            end if;
            rbra = r_b - r_a;
            B = 2 * e * rbra;
            C = rbra * rbra - length * length;
            k1 = B / 2;
            k1a = k1 * k1 - C;
            assert(noEvent(k1a > 1e-10), "
Singular position of loop (either no or two analytic solutions;
the mechanism has lost one-degree-of freedom in this position).
Try first to use another Modelica.Mechanics.MultiBody.Joints.Assemblies.JointXXX component.
If this also lead to singular positions, it could be that this
kinematic loop cannot be solved analytically with a fixed state
selection. In this case you have to build up the loop with
basic joints (NO aggregation JointXXX components) and rely on
dynamic state selection, i.e., during simulation the states will
be dynamically selected in such a way that in no position a
degree of freedom is lost.
");
            k1b = Modelica.Mechanics.MultiBody.Frames.Internal.maxWithoutEvent(k1a, 1e-12);
            k2 = sqrt(k1b);
            distance =  -k1 + (if positiveBranch then k2 else  -k2);
          end PrismaticWithLengthConstraint;
        end Internal;
      end Joints;
      package Sensors "Sensors to measure variables"
        model AbsoluteSensor "Obsolete model. Use instead Modelica.Mechanics.MultiBody.Sensors.AbsoluteSensor"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Frames;
          import Modelica.Mechanics.MultiBody.Types;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialAbsoluteSensor(final n_out = 3 * ((if get_r_abs then 1 else 0) + (if get_v_abs then 1 else 0) + (if get_a_abs then 1 else 0) + (if get_angles then 1 else 0) + (if get_w_abs then 1 else 0) + (if get_z_abs then 1 else 0)));
          extends ObsoleteModelica3.Icons.ObsoleteModel;
          Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve "If connected, the output signals are resolved in this frame" annotation(Placement(transformation(origin = {0,100}, extent = {{ -16, -16},{16,16}}, rotation = 270)));
          parameter Boolean animation = true "= true, if animation shall be enabled (show arrow)";
          parameter Boolean resolveInFrame_a = false "= true, if vectors are resolved in frame_a, otherwise in the world frame (if connector frame_resolve is connected, vectors are resolved in frame_resolve)";
          parameter Boolean get_r_abs = true "= true, to measure the position vector from the origin of the world frame to the origin of frame_a in [m]";
          parameter Boolean get_v_abs = false "= true, to measure the absolute velocity of the origin of frame_a in [m/s]";
          parameter Boolean get_a_abs = false "= true, to measure the absolute acceleration of the origin of frame_a in [m/s^2]";
          parameter Boolean get_angles = false "= true, to measure the 3 rotation angles to rotate the world frame into frame_a along the axes defined in 'sequence' below in [rad]";
          parameter Boolean get_w_abs = false "= true, to measure the absolute angular velocity of frame_a in [rad/s]";
          parameter Boolean get_z_abs = false "= true, to measure the absolute angular acceleration to frame_a in [rad/s^2]";
          parameter Types.RotationSequence sequence(min = {1,1,1}, max = {3,3,3}) = {1,2,3} " Angles are returned to rotate world frame around axes sequence[1], sequence[2] and finally sequence[3] into frame_a" annotation(Evaluate = true, Dialog(group = "if get_angles = true", enable = get_angles));
          parameter SI.Angle guessAngle1 = 0 " Select angles[1] such that abs(angles[1] - guessAngle1) is a minimum" annotation(Dialog(group = "if get_angles = true", enable = get_angles));
          input SI.Diameter arrowDiameter = world.defaultArrowDiameter " Diameter of arrow from world frame to frame_a" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color arrowColor = Modelica.Mechanics.MultiBody.Types.Defaults.SensorColor " Color of arrow from world frame to frame_a" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          annotation(__Dymola_obsolete = "Based on a packed result signal which is not a good design. Use instead Modelica.Mechanics.MultiBody.Sensors.AbsoluteSensor", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{19,109},{150,84}}, lineColor = {192,192,192}, textString = "resolve"),Line(points = {{ -84,0},{ -84,84},{0,84},{0,100}}, color = {95,95,95}, pattern = LinePattern.Dot),Text(extent = {{ -132,52},{ -96,27}}, lineColor = {128,128,128}, textString = "a")}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -84,0},{ -84,82},{0,82},{0,98}}, color = {95,95,95}, pattern = LinePattern.Dot)}), Documentation(info = "<HTML>
<p>
Absolute kinematic quantities of frame_a are
computed and provided at the output signal connector <b>y</b>
in packed format in the order
</p>
<ol>
<li> absolute position vector (= r_abs)</li>
<li> absolute velocity vectory (= v_abs)</li>
<li> absolute acceleration vector (= a_abs)</li>
<li> 3 angles to rotate the world frame into frame_a (= angles)</li>
<li> absolute angular velocity vector (= w_abs)</li>
<li> absolute angular acceleration vector (= z_abs)</li>
</ol>
<p>
For example, if parameters <b>get_v</b> and <b>get_w</b>
are <b>true</b> and all other get_XXX parameters are <b>false</b>, then
y contains 6 elements:
</p>
<pre>
 y[1:3] = absolute velocity
 y[4:6] = absolute angular velocity
</pre>
<p>
In the following figure the animation of an AbsoluteSensor
component is shown. The light blue coordinate system is
frame_a and the yellow arrow is the animated sensor.
</p>
<p align=\"center\">
<IMG SRC=\"../Modelica/Images/MultiBody/Sensors/AbsoluteSensor.png\">
</p>
<p>
If <b>frame_resolve</b> is connected to another frame, then the
provided absolute kinematic vectors are resolved in this frame.
If <b>frame_resolve</b> is <b>not</b> connected then the
coordinate system in which the relative quantities are
resolved is defined by parameter <b>resolveInFrame_a</b>.
If this parameter is <b>true</b>, then the
provided kinematic vectors are resolved in frame_a of this
component. Otherwise, the kinematic vectors are resolved in
the world frame. For example, if frame_resolve is not
connected and if resolveInFrame_a = <b>false</b>, and
get_v = <b>true</b>, then
</p>
<pre>
  y = <b>der</b>(frame_a.r) // resolved in world frame
</pre>
<p>
is returned, i.e., the derivative of the distance frame_a.r_0
from the origin of the world frame to the origin of frame_a,
resolved in the world frame.
</p>
<p>
Note, the cut-force and the cut-torque in frame_resolve are
always zero, whether frame_resolve is connected or not.
</p>
<p>
If <b>get_angles</b> = <b>true</b>, the 3 angles to rotate the world
frame into frame_a along the axes defined by parameter <b>sequence</b>
are returned. For example, if sequence = {3,1,2} then the world frame is
rotated around angles[1] along the z-axis, afterwards it is rotated
around angles[2] along the x-axis, and finally it is rotated around
angles[3] along the y-axis and is then identical to frame_a.
The 3 angles are returned in the range
</p>
<pre>
    -<font face=\"Symbol\">p</font> &lt;= angles[i] &lt;= <font face=\"Symbol\">p</font>
</pre>
<p>
There are <b>two solutions</b> for \"angles[1]\" in this range.
Via parameter <b>guessAngle1</b> (default = 0) the
returned solution is selected such that |angles[1] - guessAngle1| is
minimal. The transformation matrix between the world frame and
frame_a may be in a singular configuration with respect to \"sequence\", i.e.,
there is an infinite number of angle values leading to the same
transformation matrix. In this case, the returned solution is
selected by setting angles[1] = guessAngle1. Then angles[2]
and angles[3] can be uniquely determined in the above range.
</p>
<p>
Note, that parameter <b>sequence</b> has the restriction that
only values 1,2,3 can be used and that sequence[1] &ne; sequence[2]
and sequence[2] &ne; sequence[3]. Often used values are:
</p>
<pre>
  sequence = <b>{1,2,3}</b>  // Cardan angle sequence
           = <b>{3,1,3}</b>  // Euler angle sequence
           = <b>{3,2,1}</b>  // Tait-Bryan angle sequence
</pre>
<p>
Exact definition of the returned quantities:
</p>
<ol>
<li>r_abs is vector frame_a.r_0, resolved according to table below.</li>
<li>v_abs is vector <b>der</b>(frame_a.r_0), resolved according to table below.</li>
<li>a_abs is vector <b>der</b>(<b>der</b>(frame_a.r_0)), resolved according to
            table below.</li>
<li>angles is a vector of 3 angles such that
    frame_a.R = Frames.axesRotations(sequence, angles).</li>
<li>w_abs is vector Modelica.Mechanics.MultiBody.Frames.angularVelocity1(frame_a.R, <b>der</b>(frame_a.R)),
            resolved according to table below.</li>
<li>z_abs is vector <b>der</b>(w_abs) (= derivative of absolute angular
            velocity of frame_a with respect to the world frame,
            resolved according to table below).</li>
</ol>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>frame_resolve is</i></b></th>
      <th><b><i>resolveInFrame_a =</i></b></th>
      <th><b><i>vector is resolved in</i></b></th>
  </tr>
  <tr><td valign=\"top\">connected</td>
      <td valign=\"top\">true</td>
      <td valign=\"top\"><b>frame_resolve</b></td>
  </tr>
  <tr><td valign=\"top\">connected</td>
      <td valign=\"top\">false</td>
      <td valign=\"top\"><b>frame_resolve</b></td>
  </tr>
  <tr><td valign=\"top\">not connected</td>
      <td valign=\"top\">true</td>
      <td valign=\"top\"><b>frame_a</b></td>
  </tr>
  <tr><td valign=\"top\">not connected</td>
      <td valign=\"top\">false</td>
      <td valign=\"top\"><b>world frame</b></td>
  </tr>
</table><br>
</HTML>
"));
        protected 
          SI.Position r_abs[3] "Dummy or position vector from origin of the world frame to origin of frame_a (resolved in frame_resolve, frame_a or world frame)";
          SI.Velocity v_abs[3] "Dummy or velocity of origin of frame_a with respect to origin of world frame (resolved in frame_resolve, frame_a or world frame)";
          SI.Acceleration a_abs[3] "Dummy or acceleration of origin of frame_a with respect to origin of word frame (resolved in frame_resolve, frame_a or world frame)";
          SI.Angle angles[3] "Dummy or angles to rotate world frame into frame_a via 'sequence'";
          SI.AngularVelocity w_abs[3] "Dummy or angular velocity of frame_a with respect to world frame (resolved in frame_resolve, frame_a or world frame)";
          SI.AngularAcceleration z_abs[3] "Dummy or angular acceleration of frame_a with respect to world frame (resolved in frame_resolve, frame_a or world frame)";
          SI.Velocity v_abs_0[3] "Dummy or absolute velocity of origin of frame_a resolved in world frame";
          SI.AngularVelocity w_abs_0[3] "Dummy or absolute angular velocity of frame_a resolved in world frame";
          parameter Integer i1 = 1;
          parameter Integer i2 = if get_r_abs then i1 + 3 else i1;
          parameter Integer i3 = if get_v_abs then i2 + 3 else i2;
          parameter Integer i4 = if get_a_abs then i3 + 3 else i3;
          parameter Integer i5 = if get_angles then i4 + 3 else i4;
          parameter Integer i6 = if get_w_abs then i5 + 3 else i5;
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Arrow arrow(r_head = frame_a.r_0, diameter = arrowDiameter, specularCoefficient = specularCoefficient, color = arrowColor) if world.enableAnimation and animation;

        equation 
          if get_angles then
            angles = Frames.axesRotationsAngles(frame_a.R, sequence, guessAngle1);

          else
            angles = zeros(3);

          end if;
          if cardinality(frame_resolve) == 1 then
            frame_resolve.f = zeros(3);
            frame_resolve.t = zeros(3);
            if get_r_abs then
              r_abs = Frames.resolve2(frame_resolve.R, frame_a.r_0);

            else
              r_abs = zeros(3);

            end if;
            if get_v_abs or get_a_abs then
              v_abs_0 = der(frame_a.r_0);
              v_abs = Frames.resolve2(frame_resolve.R, v_abs_0);

            else
              v_abs_0 = zeros(3);
              v_abs = zeros(3);

            end if;
            if get_a_abs then
              a_abs = Frames.resolve2(frame_resolve.R, der(v_abs_0));

            else
              a_abs = zeros(3);

            end if;
            if get_w_abs or get_z_abs then
              w_abs_0 = Modelica.Mechanics.MultiBody.Frames.angularVelocity1(frame_a.R);
              w_abs = Frames.resolve2(frame_resolve.R, w_abs_0);

            else
              w_abs_0 = zeros(3);
              w_abs = zeros(3);

            end if;
            if get_z_abs then
              z_abs = Frames.resolve2(frame_resolve.R, der(w_abs_0));

            else
              z_abs = zeros(3);

            end if;

          else
            frame_resolve.r_0 = zeros(3);
            frame_resolve.R = Frames.nullRotation();
            if get_r_abs then
              if resolveInFrame_a then
                r_abs = Modelica.Mechanics.MultiBody.Frames.resolve2(frame_a.R, frame_a.r_0);

              else
                r_abs = frame_a.r_0;

              end if;

            else
              r_abs = zeros(3);

            end if;
            if get_v_abs or get_a_abs then
              v_abs_0 = der(frame_a.r_0);
              if resolveInFrame_a then
                v_abs = Modelica.Mechanics.MultiBody.Frames.resolve2(frame_a.R, v_abs_0);

              else
                v_abs = v_abs_0;

              end if;

            else
              v_abs_0 = zeros(3);
              v_abs = zeros(3);

            end if;
            if get_a_abs then
              if resolveInFrame_a then
                a_abs = Modelica.Mechanics.MultiBody.Frames.resolve2(frame_a.R, der(v_abs_0));

              else
                a_abs = der(v_abs_0);

              end if;

            else
              a_abs = zeros(3);

            end if;
            w_abs_0 = zeros(3);
            if get_w_abs or get_z_abs then
              if resolveInFrame_a then
                w_abs = Modelica.Mechanics.MultiBody.Frames.angularVelocity2(frame_a.R);

              else
                w_abs = Modelica.Mechanics.MultiBody.Frames.angularVelocity1(frame_a.R);

              end if;

            else
              w_abs = zeros(3);

            end if;
            if get_z_abs then
              z_abs = der(w_abs);

            else
              z_abs = zeros(3);

            end if;

          end if;
          frame_a.f = zeros(3);
          frame_a.t = zeros(3);
if get_r_abs then
                      y[i1:i1 + 2] = r_abs;
          end if;
if get_v_abs then
                      y[i2:i2 + 2] = v_abs;
          end if;
if get_a_abs then
                      y[i3:i3 + 2] = a_abs;
          end if;
if get_angles then
                      y[i4:i4 + 2] = angles;
          end if;
if get_w_abs then
                      y[i5:i5 + 2] = w_abs;
          end if;
if get_z_abs then
                      y[i6:i6 + 2] = z_abs;
          end if;
        end AbsoluteSensor;
        model RelativeSensor "Obsolete model. Use instead Modelica.Mechanics.MultiBody.Sensors.RelativeSensor"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Frames;
          import Modelica.Mechanics.MultiBody.Types;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialRelativeSensor(final n_out = 3 * ((if get_r_rel then 1 else 0) + (if get_v_rel then 1 else 0) + (if get_a_rel then 1 else 0) + (if get_angles then 1 else 0) + (if get_w_rel then 1 else 0) + (if get_z_rel then 1 else 0)));
          extends ObsoleteModelica3.Icons.ObsoleteModel;
          Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve "If connected, the output signals are resolved in this frame" annotation(Placement(transformation(origin = { -60, -100}, extent = {{ -16, -16},{16,16}}, rotation = 270)));
          parameter Boolean animation = true "= true, if animation shall be enabled (show arrow)";
          parameter Boolean resolveInFrame_a = true "= true, if relative vectors from frame_a to frame_b are resolved before differentiation in frame_a, otherwise in frame_b. If frame_resolve is connected, the vector and its derivatives are resolved in frame_resolve";
          parameter Boolean get_r_rel = true "= true, to measure the relative position vector from the origin of frame_a to the origin of frame_b in [m]";
          parameter Boolean get_v_rel = false "= true, to measure the relative velocity of the origin of frame_b with respect to frame_a in [m/s]";
          parameter Boolean get_a_rel = false "= true, to measure the relative acceleration of the origin of frame_b with respect to frame_a in [m/s^2]";
          parameter Boolean get_angles = false "= true, to measure the 3 rotation angles to rotate frame_a into frame_b along the axes defined in 'sequence' below in [rad]";
          parameter Boolean get_w_rel = false "= true, to measure the relative angular velocity of frame_b with respect to frame_a in [rad/s]";
          parameter Boolean get_z_rel = false "= true, to measure the relative angular acceleration of frame_b with respect to frame_a in [rad/s^2]";
          parameter Types.RotationSequence sequence(min = {1,1,1}, max = {3,3,3}) = {1,2,3} " Angles are returned to rotate frame_a around axes sequence[1], sequence[2] and finally sequence[3] into frame_b" annotation(Evaluate = true, Dialog(group = "if get_angles = true", enable = get_angles));
          parameter SI.Angle guessAngle1 = 0 " Select angles[1] such that abs(angles[1] - guessAngle1) is a minimum" annotation(Dialog(group = "if get_angles = true", enable = get_angles));
          input SI.Diameter arrowDiameter = world.defaultArrowDiameter " Diameter of relative arrow from frame_a to frame_b" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color arrowColor = Modelica.Mechanics.MultiBody.Types.Defaults.SensorColor " Color of relative arrow from frame_a to frame_b" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          SI.Position r_rel[3] "Dummy or relative position vector (resolved in frame_a, frame_b or frame_resolve)";
          SI.Velocity v_rel[3] "Dummy or relative velocity vector (resolved in frame_a, frame_b or frame_resolve";
          SI.Acceleration a_rel[3] "Dummy or relative acceleration vector (resolved in frame_a, frame_b or frame_resolve";
          SI.Angle angles[3] "Dummy or angles to rotate frame_a into frame_b via 'sequence'";
          SI.AngularVelocity w_rel[3] "Dummy or relative angular velocity vector (resolved in frame_a, frame_b or frame_resolve";
          SI.AngularAcceleration z_rel[3] "Dummy or relative angular acceleration vector (resolved in frame_a, frame_b or frame_resolve";
          Frames.Orientation R_rel "Dummy or relative orientation object from frame_a to frame_b";
        protected 
          SI.Position r_rel_ab[3] "Dummy or relative position vector resolved in frame_a or frame_b";
          SI.Velocity der_r_rel_ab[3] "Dummy or derivative of relative position vector (resolved in frame_a, frame_b or frame_resolve)";
          SI.AngularVelocity w_rel_ab[3] "Dummy or angular velocity of frame_b with respect to frame_a (resolved in frame_a or frame_b)";
          Frames.Orientation R_resolve "Dummy or relative orientation of frame_a or frame_b with respect to frame_resolve";
          parameter Integer i1 = 1;
          parameter Integer i2 = if get_r_rel then i1 + 3 else i1;
          parameter Integer i3 = if get_v_rel then i2 + 3 else i2;
          parameter Integer i4 = if get_a_rel then i3 + 3 else i3;
          parameter Integer i5 = if get_angles then i4 + 3 else i4;
          parameter Integer i6 = if get_w_rel then i5 + 3 else i5;
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Arrow arrow(r = frame_a.r_0, r_head = frame_b.r_0 - frame_a.r_0, diameter = arrowDiameter, color = arrowColor, specularCoefficient) if world.enableAnimation and animation;
          annotation(__Dymola_obsolete = "Based on a packed result signal which is not a good design. Use instead Modelica.Mechanics.MultiBody.Sensors.RelativeSensor", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -60, -94},{ -60, -76},{0, -76},{0, -76}}, color = {95,95,95}, pattern = LinePattern.Dot),Text(extent = {{ -157, -49},{ -26, -74}}, lineColor = {192,192,192}, pattern = LinePattern.Dot, textString = "resolve")}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -60, -98},{ -60, -76},{0, -76},{0, -76}}, color = {95,95,95}, pattern = LinePattern.Dot)}), Documentation(info = "<HTML>
<p>
Relative kinematic quantities between frame_a and frame_b are
determined and provided at the output signal connector <b>y</b>
in packed format in the order
</p>
<ol>
<li> relative position vector (= r_rel)</li>
<li> relative velocity vectory (= v_rel)</li>
<li> relative acceleration vector (= a_rel))</li>
<li> 3 angles to rotate frame_a into frame_b (= angles)</li>
<li> relative angular velocity vector (= w_rel)</li>
<li> relative angular acceleration vector (= z_rel)</li>
</ol>
<p>
For example, if parameters <b>get_v_rel</b> and <b>get_w_rel</b>
are <b>true</b> and all other get_XXX parameters are <b>false</b>, then
y contains 6 elements:
</p>
<pre>
 y = relative velocity
 y = relative angular velocity
</pre>
<p>
In the following figure the animation of a RelativeSensor
component is shown. The light blue coordinate system is
frame_a, the dark blue coordinate system is frame_b, and
the yellow arrow is the animated sensor.
</p>
<p align=\"center\">
<IMG SRC=\"../Modelica/Images/MultiBody/Sensors/RelativeSensor.png\">
</p>
<p>
If parameter <b>resolveInFrame_a</b> = <b>true</b>, then the
provided relative kinematic vectors of frame_b with respect to
frame_a are resolved before differentiation in frame_a. If this
parameter is <b>false</b>, the relative kinematic vectors are
resolved before differentiation in frame_b.
If <b>frame_resolve</b> is connected to another frame, then the
kinematic vector as defined above and/or its required derivatives
are resolved in frame_resolve. Note, derivatives
of relative kinematic quantities are always performed with
respect to frame_a (<b>resolveInFrame_a</b> = <b>true</b>)
or with respect to frame_b (<b>resolveInFrame_a</b> = <b>false</b>).
The resulting vector is then resolved in frame_resolve, if this
connector is connected.
</p>
<p>
For example, if frame_resolve is not
connected and if resolveInFrame_a = <b>false</b>, and
get_v = <b>true</b>, then
</p>
<pre>
  y = v_rel
    = <b>der</b>(r_rel)
</pre>
is returned (r_rel = resolve2(frame_b.R, frame_b.r_0 - frame_a.r0)), i.e.,
the derivative of the relative distance from frame_a to frame_b,
resolved in frame_b. If frame_resolve is connected, then
</p>
<pre>
  y = v_rel
    = resolve2(frame_resolve.R, <b>der</b>(r_rel))
</pre>
<p>
is returned, i.e., the previous relative velocity vector is
additionally resolved in frame_resolve.
</p>
<p>
Note, the cut-force and the cut-torque in frame_resolve are
always zero, whether frame_resolve is connected or not.
</p>
<p>
If <b>get_angles</b> = <b>true</b>, the 3 angles to rotate frame_a
into frame_b along the axes defined by parameter <b>sequence</b>
are returned. For example, if sequence = {3,1,2} then frame_a is
rotated around angles[1] along the z-axis, afterwards it is rotated
around angles[2] along the x-axis, and finally it is rotated around
angles[3] along the y-axis and is then identical to frame_b.
The 3 angles are returned in the range
</p>
<pre>
    -<font face=\"Symbol\">p</font> &lt;= angles[i] &lt;= <font face=\"Symbol\">p</font>
</pre>
<p>
There are <b>two solutions</b> for \"angles[1]\" in this range.
Via parameter <b>guessAngle1</b> (default = 0) the
returned solution is selected such that |angles[1] - guessAngle1| is
minimal. The relative transformation matrix between frame_a and
frame_b may be in a singular configuration with respect to \"sequence\", i.e.,
there is an infinite number of angle values leading to the same relative
transformation matrix. In this case, the returned solution is
selected by setting angles[1] = guessAngle1. Then angles[2]
and angles[3] can be uniquely determined in the above range.
</p>
<p>
Note, that parameter <b>sequence</b> has the restriction that
only values 1,2,3 can be used and that sequence[1] &ne; sequence[2]
and sequence[2] &ne; sequence[3]. Often used values are:
</p>
<pre>
  sequence = <b>{1,2,3}</b>  // Cardan angle sequence
           = <b>{3,1,3}</b>  // Euler angle sequence
           = <b>{3,2,1}</b>  // Tait-Bryan angle sequence
</pre>
<p>
Exact definition of the returned quantities
(r_rel_ab, R_rel_ab, w_rel_ab are defined below the enumeration):
</p>
<ol>
<li>r_rel is vector r_rel_ab, resolved according to table below.</li>
<li>v_rel is vector <b>der</b>(r_rel_ab), resolved according to table below.</li>
<li>a_rel is vector <b>der</b>(<b>der</b>(r_rel_ab)), resolved according to
            table below.</li>
<li>angles is a vector of 3 angles such that
    R_rel_ab = Frames.axesRotations(sequence, angles).</li>
<li>w_rel is vector w_rel_ab, resolved according to table below.</li>
<li>z_rel is vector <b>der</b>(w_rel_ab), resolved according to table below.</li>
</ol>
<p>
using the auxiliary quantities
</p>
<ol>
<li> r_rel_ab is vector frame_b.r_0 - frame_a.r_0, resolved either in frame_a or
     frame_b according to parameter resolveInFrame_a.</li>
<li> R_rel_ab is orientation object Frames.relativeRotation(frame_a.R, frame_b.R).</li>
<li> w_rel_ab is vector Frames.angularVelocity1(R_rel_ab, der(R_rel_ab)), resolved either
     in frame_a or frame_b according to parameter resolveInFrame_a.</li>
</ol>
<p>
and resolved in the following frame
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>frame_resolve is</i></b></th>
      <th><b><i>resolveInFrame_a =</i></b></th>
      <th><b><i>vector is resolved in</i></b></th>
  </tr>
  <tr><td valign=\"top\">connected</td>
      <td valign=\"top\">true</td>
      <td valign=\"top\"><b>frame_resolve</b></td>
  </tr>
  <tr><td valign=\"top\">connected</td>
      <td valign=\"top\">false</td>
      <td valign=\"top\"><b>frame_resolve</b></td>
  </tr>
  <tr><td valign=\"top\">not connected</td>
      <td valign=\"top\">true</td>
      <td valign=\"top\"><b>frame_a</b></td>
  </tr>
  <tr><td valign=\"top\">not connected</td>
      <td valign=\"top\">false</td>
      <td valign=\"top\"><b>frame_b</b></td>
  </tr>
</table><br>
</HTML>"));

        equation 
          if get_angles or get_w_rel or get_z_rel then
            R_rel = Modelica.Mechanics.MultiBody.Frames.relativeRotation(frame_a.R, frame_b.R);

          else
            R_rel = Modelica.Mechanics.MultiBody.Frames.nullRotation();

          end if;
          if get_angles then
            angles = Frames.axesRotationsAngles(R_rel, sequence, guessAngle1);

          else
            angles = zeros(3);

          end if;
          if cardinality(frame_resolve) == 1 then
            frame_resolve.f = zeros(3);
            frame_resolve.t = zeros(3);
            if resolveInFrame_a then
              R_resolve = Frames.relativeRotation(frame_a.R, frame_resolve.R);

            else
              R_resolve = Frames.relativeRotation(frame_b.R, frame_resolve.R);

            end if;
            if get_r_rel or get_v_rel or get_a_rel then
              if resolveInFrame_a then
                r_rel_ab = Frames.resolve2(frame_a.R, frame_b.r_0 - frame_a.r_0);

              else
                r_rel_ab = Frames.resolve2(frame_b.R, frame_b.r_0 - frame_a.r_0);

              end if;
              r_rel = Frames.resolve2(R_resolve, r_rel_ab);

            else
              r_rel_ab = zeros(3);
              r_rel = zeros(3);

            end if;
            if get_v_rel or get_a_rel then
              der_r_rel_ab = der(r_rel_ab);

            else
              der_r_rel_ab = zeros(3);

            end if;
            if get_v_rel then
              v_rel = Frames.resolve2(R_resolve, der_r_rel_ab);

            else
              v_rel = zeros(3);

            end if;
            if get_a_rel then
              a_rel = Frames.resolve2(R_resolve, der(der_r_rel_ab));

            else
              a_rel = zeros(3);

            end if;
            if get_w_rel or get_z_rel then
              if resolveInFrame_a then
                w_rel_ab = Modelica.Mechanics.MultiBody.Frames.angularVelocity1(R_rel);

              else
                w_rel_ab = Modelica.Mechanics.MultiBody.Frames.angularVelocity2(R_rel);

              end if;
              w_rel = Frames.resolve2(R_resolve, w_rel_ab);

            else
              w_rel = zeros(3);
              w_rel_ab = zeros(3);

            end if;
            if get_z_rel then
              z_rel = Frames.resolve2(R_resolve, der(w_rel_ab));

            else
              z_rel = zeros(3);

            end if;

          else
            frame_resolve.r_0 = zeros(3);
            frame_resolve.R = Frames.nullRotation();
            R_resolve = Frames.nullRotation();
            r_rel_ab = zeros(3);
            der_r_rel_ab = zeros(3);
            w_rel_ab = zeros(3);
            if get_r_rel or get_v_rel or get_a_rel then
              if resolveInFrame_a then
                r_rel = Frames.resolve2(frame_a.R, frame_b.r_0 - frame_a.r_0);

              else
                r_rel = Frames.resolve2(frame_b.R, frame_b.r_0 - frame_a.r_0);

              end if;

            else
              r_rel = zeros(3);

            end if;
            if get_v_rel or get_a_rel then
              v_rel = der(r_rel);

            else
              v_rel = zeros(3);

            end if;
            if get_a_rel then
              a_rel = der(v_rel);

            else
              a_rel = zeros(3);

            end if;
            if get_w_rel or get_z_rel then
              if resolveInFrame_a then
                w_rel = Frames.angularVelocity1(R_rel);

              else
                w_rel = Frames.angularVelocity2(R_rel);

              end if;

            else
              w_rel = zeros(3);

            end if;
            if get_z_rel then
              z_rel = der(w_rel);

            else
              z_rel = zeros(3);

            end if;

          end if;
          frame_a.f = zeros(3);
          frame_a.t = zeros(3);
          frame_b.f = zeros(3);
          frame_b.t = zeros(3);
if get_r_rel then
                      y[i1:i1 + 2] = r_rel;
          end if;
if get_v_rel then
                      y[i2:i2 + 2] = v_rel;
          end if;
if get_a_rel then
                      y[i3:i3 + 2] = a_rel;
          end if;
if get_angles then
                      y[i4:i4 + 2] = angles;
          end if;
if get_w_rel then
                      y[i5:i5 + 2] = w_rel;
          end if;
if get_z_rel then
                      y[i6:i6 + 2] = z_rel;
          end if;
        end RelativeSensor;
        model CutForceAndTorque "Obsolete model. Use instead Modelica.Mechanics.MultiBody.Sensors.CutForceAndTorque"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Types;
          extends ObsoleteModelica3.Mechanics.MultiBody.Interfaces.PartialCutForceSensor;
          Modelica.Blocks.Interfaces.RealOutput load[6] "Cut force and cut torque resolved in frame_a/frame_b or in frame_resolved, if connected" annotation(Placement(transformation(origin = { -80, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          parameter Boolean animation = true "= true, if animation shall be enabled (show force and torque arrow)";
          parameter Boolean positiveSign = true "= true, if force and torque with positive sign is returned (= frame_a.f/.t), otherwise with negative sign (= frame_b.f/.t)";
          parameter Boolean resolveInFrame_a = true "= true, if force and torque are resolved in frame_a/frame_b, otherwise in the world frame (if connector frame_resolve is connected, the force/torque is resolved in frame_resolve)";
          input Real N_to_m(unit = "N/m") = 1000 " Force arrow scaling (length = force/N_to_m)" annotation(Dialog(group = "if animation = true", enable = animation));
          input Real Nm_to_m(unit = "N.m/m") = 1000 " Torque arrow scaling (length = torque/Nm_to_m)" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter forceDiameter = world.defaultArrowDiameter " Diameter of force arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter torqueDiameter = forceDiameter " Diameter of torque arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color forceColor = Modelica.Mechanics.MultiBody.Types.Defaults.ForceColor " Color of force arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color torqueColor = Modelica.Mechanics.MultiBody.Types.Defaults.TorqueColor " Color of torque arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(group = "if animation = true", enable = animation));
          SI.Force force[3] "Cut force resolved in frame_a/frame_b or in frame_resolved, if connected";
          SI.Torque torque[3] "Cut torque resolved in frame_a/frame_b or in frame_resolved, if connected";
          annotation(__Dymola_obsolete = "Based on a packed result signal which is not a good design. Use instead Modelica.Mechanics.MultiBody.Sensors.CutForceAndTorque", preferedView = "info", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<HTML>
<p>
The cut-force and cut-torque acting at the component to which frame_b is
connected are determined and provided at the output signal connector
<b>load</b>:
</p>
<pre>
  load[1:3] = frame_a.f;
  load[4:6] = frame_a.t;
</pre>
<p>
If parameter <b>positiveSign</b> =
<b>false</b>, the negative cut-force and negative
cut-torque is provided (= frame_b.f and frame_b.t).
If <b>frame_resolve</b> is connected to another frame, then the
cut-force and cut-torque are resolved in frame_resolve.
If <b>frame_resolve</b> is <b>not</b> connected then the
coordinate system in which the cut-force and cut-torque is resolved
is defined by parameter <b>resolveInFrame_a</b>.
If this parameter is <b>true</b>, then the
cut-force and cut-torque is resolved in frame_a, otherwise it is
resolved in the world frame.
</p>
<p>
In the following figure the animation of a CutForceAndTorque
sensor is shown. The dark blue coordinate system is frame_b,
and the green arrows are the cut force and the cut torque,
respectively, acting at frame_b and
with negative sign at frame_a.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Sensors/CutForceAndTorque.png\">
</p>
</HTML>"));
        protected 
          outer Modelica.Mechanics.MultiBody.World world;
          parameter Integer csign = if positiveSign then  +1 else  -1;
          SI.Position f_in_m[3] = (frame_a.f * csign) / N_to_m "Force mapped from N to m for animation";
          SI.Position t_in_m[3] = (frame_a.t * csign) / Nm_to_m "Torque mapped from Nm to m for animation";
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Arrow forceArrow(diameter = forceDiameter, color = forceColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = f_in_m, r_head =  -f_in_m) if world.enableAnimation and animation;
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.DoubleArrow torqueArrow(diameter = torqueDiameter, color = torqueColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = t_in_m, r_head =  -t_in_m) if world.enableAnimation and animation;

        equation 
          if cardinality(frame_resolve) == 1 then
            force = Modelica.Mechanics.MultiBody.Frames.resolve2(frame_resolve.R, Modelica.Mechanics.MultiBody.Frames.resolve1(frame_a.R, frame_a.f)) * csign;
            torque = Modelica.Mechanics.MultiBody.Frames.resolve2(frame_resolve.R, Modelica.Mechanics.MultiBody.Frames.resolve1(frame_a.R, frame_a.t)) * csign;

          elseif resolveInFrame_a then
            force = frame_a.f * csign;
            torque = frame_a.t * csign;


          else
            force = Modelica.Mechanics.MultiBody.Frames.resolve1(frame_a.R, frame_a.f) * csign;
            torque = Modelica.Mechanics.MultiBody.Frames.resolve1(frame_a.R, frame_a.t) * csign;

          end if;
          load[1:3] = force;
          load[4:6] = torque;
        end CutForceAndTorque;
      end Sensors;
      package Types "Constants and types with choices, especially to build menus"
        type AngularVelocity_degs = Modelica.Icons.TypeReal(final quantity = "AngularVelocity", final unit = "deg/s") "Obsolete type. Use Modelica.SIunits.AngularVelocity instead with an appropriate displayUnit" annotation(__Dymola_obsolete = "Non SI-units should no longer be used. Use Modelica.SIunits.AngularVelocity instead with an appropriate displayUnit");
        type AngularAcceleration_degs2 = Modelica.Icons.TypeReal(final quantity = "AngularAcceleration", final unit = "deg/s2") "Obsolete type. Use Modelica.SIunits.AngularAcceleration instead with an appropriate displayUnit" annotation(__Dymola_obsolete = "Non SI-units should no longer be used. Use Modelica.SIunits.AngularAcceleration instead with an appropriate displayUnit");
        package Init "Obsolete type. This is an emulated enumeration for initialization. Initialization is now defined with start/fixed values and appropriate support in the parameter menu"
          annotation(__Dymola_obsolete = "This is an emulated enumeration for initialization. Initialization is now defined with start/fixed values and appropriate support in the parameter menu", Documentation(info = "<html>

</html>"));
          extends ObsoleteModelica3.Icons.Enumeration;
          constant Integer Free = 1;
          constant Integer PositionVelocity = 2;
          constant Integer SteadyState = 3;
          constant Integer Position = 4;
          constant Integer Velocity = 5;
          constant Integer VelocityAcceleration = 6;
          constant Integer PositionVelocityAcceleration = 7;
          type Temp "Obsolete type. This is an emulated enumeration for initialization. Initialization is now defined with start/fixed values and appropriate support in the parameter menu"
            extends Modelica.Icons.TypeInteger;
            annotation(__Dymola_obsolete = "This is an emulated enumeration for initialization. Initialization is now defined with start/fixed values and appropriate support in the parameter menu", choices(choice = Modelica.Mechanics.MultiBody.Types.Init.Free "free (no initialization)", choice = Modelica.Mechanics.MultiBody.Types.Init.PositionVelocity "initialize generalized position and velocity variables", choice = Modelica.Mechanics.MultiBody.Types.Init.SteadyState "initialize in steady state (velocity and acceleration are zero)", choice = Modelica.Mechanics.MultiBody.Types.Init.Position "initialize only generalized position variable(s)", choice = Modelica.Mechanics.MultiBody.Types.Init.Velocity "initialize only generalized velocity variable(s)", choice = Modelica.Mechanics.MultiBody.Types.Init.VelocityAcceleration "initialize generalized velocity and acceleration variables", choice = Modelica.Mechanics.MultiBody.Types.Init.PositionVelocityAcceleration "initialize generalized position, velocity and acceleration variables"), Documentation(info = "<html>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.Init.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">Free</td>
    <td valign=\"top\">No initialization</td></tr>

<tr><td valign=\"top\">PositionVelocity</td>
    <td valign=\"top\">Initialize generalized position and velocity variables</td></tr>

<tr><td valign=\"top\">SteadyState</td>
    <td valign=\"top\">Initialize in steady state (velocity and acceleration are zero)</td></tr>

<tr><td valign=\"top\">Position </td>
    <td valign=\"top\">Initialize only generalized position variable(s)</td></tr>

<tr><td valign=\"top\">Velocity</td>
    <td valign=\"top\">Initialize only generalized velocity variable(s)</td></tr>

<tr><td valign=\"top\">VelocityAcceleration</td>
    <td valign=\"top\">Initialize generalized velocity and acceleration variables</td></tr>

<tr><td valign=\"top\">PositionVelocityAcceleration</td>
    <td valign=\"top\">Initialize generalized position, velocity and acceleration variables</td></tr>

</table>

</html>"));
          end Temp;
        end Init;
      end Types;
    end MultiBody;
    package Rotational "Library to model 1-dimensional, rotational mechanical systems"
      package Interfaces "Connectors and partial models for 1D rotational mechanical components"
        partial model Rigid "Base class for the rigid connection of two rotational 1D flanges"
          extends ObsoleteModelica3.Icons.ObsoleteModel;
          Modelica.SIunits.Angle phi "Absolute rotation angle of component (= flange_a.phi = flange_b.phi)";
          Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a "(left) driving flange (flange axis directed INTO cut plane)" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_b "(right) driven flange (flange axis directed OUT OF cut plane)" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>
This is a 1D rotational component with two rigidly connected flanges,
i.e., flange_a.phi = flange_b.phi. It is used e.g. to built up components
with inertia.
</p>

</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Window(x = 0.18, y = 0.3, width = 0.61, height = 0.66));

        equation 
          flange_a.phi = phi;
          flange_b.phi = phi;
        end Rigid;
        partial model Bearing "Obsolete model. Use one of Modelica.Mechanics.Rotational.Interfaces.PartialXXX instead"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialTwoFlanges;
          extends ObsoleteModelica3.Icons.ObsoleteModel;
          Modelica.SIunits.Torque tau_support;
          Modelica.Mechanics.Rotational.Interfaces.Flange_a bearing "Flange of bearing" annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}}, rotation = 0)));
          annotation(__Dymola_obsolete = "The Rotational library has now a new improved design with optional support connectors. Use Modelica.Mechanics.Rotational.Interfaces.PartialXXX instead.", Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -20, -80},{20, -120}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid)}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -20, -80},{20, -120}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>
This is a 1D rotational component with two flanges and an additional bearing flange.
It is a superclass for the two components TwoFlangesAndBearing and TwoFlangesAndBearingH.</p>

</HTML>
"));
        end Bearing;
        partial model TwoFlangesAndBearing "Obsolete model. Use one of Modelica.Mechanics.Rotational.Interfaces.PartialXXX instead"
          extends ObsoleteModelica3.Mechanics.Rotational.Interfaces.Bearing;
          Modelica.SIunits.Angle phi_a;
          Modelica.SIunits.Angle phi_b;
          annotation(__Dymola_obsolete = "The Rotational library has now a new improved design with optional support connectors. Use Modelica.Mechanics.Rotational.Interfaces.PartialXXX instead.", Documentation(info = "<html>
<p>
This is a 1D rotational component with two flanges and an additional bearing flange.
It is used e.g. to build up equation-based parts of a drive train.</p>

</HTML>
"));

        equation 
          if cardinality(bearing) == 0 then
            bearing.phi = 0;

          else
            bearing.tau = tau_support;

          end if;
          0 = flange_a.tau + flange_b.tau + tau_support;
          phi_a = flange_a.phi - bearing.phi;
          phi_b = flange_b.phi - bearing.phi;
        end TwoFlangesAndBearing;
        partial model TwoFlangesAndBearingH "Obsolete model. Use one of Modelica.Mechanics.Rotational.Interfaces.PartialXXX instead"
          extends ObsoleteModelica3.Mechanics.Rotational.Interfaces.Bearing;
          Adapter adapter(bearingConnected = cardinality(bearing) > 1) annotation(Placement(transformation(origin = {0, -60}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        protected 
          encapsulated model Adapter
            import Modelica;
            import ObsoleteModelica3;
            import TwoFlanges = Modelica.Mechanics.Rotational.Interfaces.PartialTwoFlanges;
            extends Modelica.Mechanics.Rotational.Interfaces.PartialTwoFlanges;
            extends ObsoleteModelica3.Icons.ObsoleteModel;
            parameter Boolean bearingConnected;
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -90,10},{90, -10}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,60},{150,20}}, textString = "%name", lineColor = {0,0,255})}));

          equation 
            flange_a.phi = flange_b.phi;
            if bearingConnected then
              0 = flange_a.tau + flange_b.tau;

            else
              0 = flange_a.phi;

            end if;
          end Adapter;

        equation 
          tau_support =  -adapter.flange_b.tau;
          connect(adapter.flange_a,bearing) annotation(Line(points = {{ -6.12323e-16, -70},{0, -70},{0, -100}}, color = {0,0,0}));
          annotation(__Dymola_obsolete = "The Rotational library has now a new improved design with optional support connectors. Use Modelica.Mechanics.Rotational.Interfaces.PartialXXX instead.", Documentation(info = "<html>
<p>
This is a 1D rotational component with two flanges and an additional bearing flange.
It is used e.g. to build up parts of a drive train consisting
of several base components.</p>

</HTML>
"));
        end TwoFlangesAndBearingH;
        partial model PartialSpeedDependentTorque "Partial model of a torque acting at the flange (accelerates the flange)"
          extends ObsoleteModelica3.Icons.ObsoleteModel;
          Modelica.SIunits.AngularVelocity w = der(flange.phi) "Angular velocity at flange";
          Modelica.SIunits.Torque tau = flange.tau "accelerating torque acting at flange";
          Modelica.Mechanics.Rotational.Interfaces.Flange_b flange "Flange on which torque is acting" annotation(Placement(transformation(extent = {{110, -10},{90,10}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Interfaces.Flange_a bearing "Bearing at which the reaction torque (i.e., -flange.tau) is acting" annotation(Placement(transformation(extent = {{ -10, -130},{10, -110}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -96,96},{96, -96}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -30, -70},{30, -70}}, color = {0,0,0}),Line(points = {{ -30, -90},{ -10, -70}}, color = {0,0,0}),Line(points = {{ -10, -90},{10, -70}}, color = {0,0,0}),Rectangle(extent = {{ -20, -100},{20, -140}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{10, -90},{30, -70}}, color = {0,0,0}),Line(points = {{0, -70},{0, -110}}, color = {0,0,0}),Line(points = {{ -92,0},{ -76,36},{ -54,62},{ -30,80},{ -14,88},{10,92},{26,90},{46,80},{64,62}}, color = {0,0,0}),Text(extent = {{ -150,140},{150,100}}, lineColor = {0,0,255}, textString = "%name"),Polygon(points = {{94,16},{80,74},{50,52},{94,16}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
<p>
Partial model of torque dependent on speed that accelerates the flange.
</p>
</HTML>"));

        equation 
          if cardinality(bearing) == 0 then
            bearing.phi = 0;

          else
            bearing.tau =  -flange.tau;

          end if;
        end PartialSpeedDependentTorque;
        partial model AbsoluteSensor "Obsolete model. Use Modelica.Mechanics.Rotational.Interfaces.PartialAbsoluteSensor instead and define a meaningful name for the output signal"
          extends Modelica.Icons.RotationalSensor;
          extends ObsoleteModelica3.Icons.ObsoleteModel;
          Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a "(left) flange to be measured (flange axis directed INTO cut plane)" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput y "Sensor signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
          annotation(Window(x = 0.39, y = 0.05, width = 0.6, height = 0.6), __Dymola_obsolete = "Use Modelica.Mechanics.Rotational.Interfaces.PartialAbsoluteSensor instead and define a meaningful name for the output signal.", Documentation(info = "<html>
<p>
This is the base class of a 1D rotational component with one flange and one
output signal y in order to measure an absolute kinematic quantity in the flange
and to provide the measured signal as output signal for further processing
with the blocks of package Modelica.Blocks.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{70,0},{100,0}}, color = {0,0,127}),Text(extent = {{150,80},{ -150,120}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{70,0},{100,0}}, color = {0,0,255})}));
        end AbsoluteSensor;
        partial model RelativeSensor "Obsolete model. Use Modelica.Mechanics.Rotational.Interfaces.PartialRelativbeSensor instead and define a meaningful name for the output signal"
          extends Modelica.Icons.RotationalSensor;
          extends ObsoleteModelica3.Icons.ObsoleteModel;
          Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a "(left) driving flange (flange axis directed INTO cut plane)" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_b "(right) driven flange (flange axis directed OUT OF cut plane)" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput y "Sensor signal" annotation(Placement(transformation(origin = {0, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          annotation(Window(x = 0.37, y = 0.02, width = 0.6, height = 0.65), __Dymola_obsolete = "Use Modelica.Mechanics.Rotational.Interfaces.PartialRelativeSensor instead and define a meaningful name for the output signal.", Documentation(info = "<html>
<p>
This is a base class for 1D rotational components with two rigidly connected
flanges and one output signal y in order to measure relative kinematic quantities
between the two flanges or the cut-torque in the flange and
to provide the measured signal as output signal for further processing
with the blocks of package Modelica.Blocks.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{70,0},{90,0}}, color = {0,0,0}),Line(points = {{0, -100},{0, -70}}, color = {0,0,127}),Text(extent = {{ -150,70},{150,110}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{70,0},{90,0}}, color = {0,0,0}),Line(points = {{0, -100},{0, -70}}, color = {0,0,255})}));
        end RelativeSensor;
      end Interfaces;
      package Types "Constants and types with choices, especially to build menus"
        extends Modelica.Icons.Library;
        annotation(preferedView = "info", Documentation(info = "<HTML>
<p>
In this package <b>types</b> and <b>constants</b> are defined that are used
in library Modelica.Blocks. The types have additional annotation choices
definitions that define the menus to be built up in the graphical
user interface when the type is used as parameter in a declaration.
</p>
</HTML>"));
        package Init "Obsolete type. This is an emulated enumeration for initialization. Initialization is now defined with start/fixed values and appropriate support in the parameter menu"
          extends ObsoleteModelica3.Icons.Enumeration;
          constant Integer NoInit = 1 "no initialization (phi_start, w_start are guess values)";
          constant Integer SteadyState = 2 "steady state initialization (der(phi)=der(w)=0)";
          constant Integer InitialState = 3 "initialization with phi_start, w_start";
          constant Integer InitialAngle = 4 "initialization with phi_start";
          constant Integer InitialSpeed = 5 "initialization with w_start";
          constant Integer InitialAcceleration = 6 "initialization with a_start";
          constant Integer InitialAngleAcceleration = 7 "initialization with phi_start, a_start";
          constant Integer InitialSpeedAcceleration = 8 "initialization with w_start, a_start";
          constant Integer InitialAngleSpeedAcceleration = 9 "initialization with phi_start, w_start, a_start";
          type Temp "Obsolete type. This is an emulated enumeration for initialization. Initialization is now defined with start/fixed values and appropriate support in the parameter menu"
            extends Modelica.Icons.TypeInteger(min = 1, max = 9);
            annotation(__Dymola_obsolete = "This is an emulated enumeration for initialization. Initialization is now defined with start/fixed values and appropriate support in the parameter menu", Evaluate = true, choices(choice = Modelica.Mechanics.Rotational.Types.Init.NoInit "no initialization (phi_start, w_start are guess values)", choice = Modelica.Mechanics.Rotational.Types.Init.SteadyState "steady state initialization (der(phi)=der(w)=0)", choice = Modelica.Mechanics.Rotational.Types.Init.InitialState "initialization with phi_start, w_start", choice = Modelica.Mechanics.Rotational.Types.Init.InitialAngle "initialization with phi_start", choice = Modelica.Mechanics.Rotational.Types.Init.InitialSpeed "initialization with w_start", choice = Modelica.Mechanics.Rotational.Types.Init.InitialAcceleration "initialization with a_start", choice = Modelica.Mechanics.Rotational.Types.Init.InitialAngleAcceleration "initialization with phi_start, a_start", choice = Modelica.Mechanics.Rotational.Types.Init.InitialSpeedAcceleration "initialization with w_start, a_start", choice = Modelica.Mechanics.Rotational.Types.Init.InitialAngleSpeedAcceleration "initialization with phi_start, w_start, a_start"));
          end Temp;
          annotation(Documentation(info = "<html>
<p>
Type <b>Init</b> defines initialization of absolute rotational
quantities.
</p>

</html>"));
        end Init;
        package InitRel "Obsolete type. This is an emulated enumeration for initialization. Initialization is now defined with start/fixed values and appropriate support in the parameter menu"
          extends ObsoleteModelica3.Icons.Enumeration;
          constant Integer NoInit = 1 "no initialization (phi_rel_start, w_rel_start are guess values)";
          constant Integer SteadyState = 2 "steady state initialization (der(phi_rel)=der(w_rel)=0)";
          constant Integer InitialState = 3 "initialization with phi_rel_start, w_rel_start";
          constant Integer InitialAngle = 4 "initialization with phi_rel_start";
          constant Integer InitialSpeed = 5 "initialization with w_rel_start";
          type Temp "Obsolete type. This is an emulated enumeration for initialization. Initialization is now defined with start/fixed values and appropriate support in the parameter menu"
            extends Modelica.Icons.TypeInteger(min = 1, max = 5);
            annotation(__Dymola_obsolete = "This is an emulated enumeration for initialization. Initialization is now defined with start/fixed values and appropriate support in the parameter menu", Evaluate = true, choices(choice = Modelica.Mechanics.Rotational.Types.Init.NoInit "no initialization (phi_rel_start, w_rel_start are guess values)", choice = Modelica.Mechanics.Rotational.Types.Init.SteadyState "steady state initialization (der(phi)=der(w)=0)", choice = Modelica.Mechanics.Rotational.Types.Init.InitialState "initialization with phi_rel_start, w_rel_start", choice = Modelica.Mechanics.Rotational.Types.Init.InitialAngle "initialization with phi_rel_start", choice = Modelica.Mechanics.Rotational.Types.Init.InitialSpeed "initialization with w_rel_start"));
          end Temp;
          annotation(Documentation(info = "<html>
<p>
Type <b>Init</b> defines initialization of relative rotational
quantities.
</p>

</html>"));
        end InitRel;
      end Types;
      model GearEfficiency "Obsolete model. Use Modelica.Mechanics.Rotational.Components.LossyGear instead"
        extends ObsoleteModelica3.Mechanics.Rotational.Interfaces.TwoFlangesAndBearing;
        extends ObsoleteModelica3.Icons.ObsoleteModel;
        parameter Real eta(min = Modelica.Constants.small, max = 1) = 1 "Efficiency";
        Modelica.SIunits.Angle phi;
        Modelica.SIunits.Power power_a "Energy flowing into flange_a (= power)";
        Boolean driving_a "True, if energy is flowing INTO and not out of flange flange_a";
        annotation(__Dymola_obsolete = "This model can get stuck due when the torque direction varies, use Modelica.Mechanics.Rotational.Components.LossyGear instead.", Window(x = 0.23, y = 0.06, width = 0.69, height = 0.62), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -150,100},{150,60}}, textString = "%name", lineColor = {0,0,255}),Rectangle(extent = {{ -100,20},{100, -20}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Line(points = {{ -30, -40},{30, -40}}, color = {0,0,0}),Line(points = {{0, -40},{0, -90}}, color = {0,0,0}),Polygon(points = {{ -30, -20},{60, -20},{60, -80},{70, -80},{50, -100},{30, -80},{40, -80},{40, -30},{ -30, -30},{ -30, -20},{ -30, -20}}, lineColor = {0,0,0}, fillColor = {255,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,60},{150,20}}, lineColor = {0,0,0}, textString = "eta=%eta"),Line(points = {{30, -50},{20, -60}}, color = {0,0,0}),Line(points = {{30, -40},{10, -60}}, color = {0,0,0}),Line(points = {{20, -40},{0, -60}}, color = {0,0,0}),Line(points = {{10, -40},{ -10, -60}}, color = {0,0,0}),Line(points = {{0, -40},{ -20, -60}}, color = {0,0,0}),Line(points = {{ -10, -40},{ -30, -60}}, color = {0,0,0}),Line(points = {{ -20, -40},{ -30, -50}}, color = {0,0,0})}), Documentation(info = "<html>
<p>
THIS COMPONENT IS <b>OBSOLETE</b> and should <b>no longer be used</b>. It is only
kept for <b>backward compatibility</b> purposes. Use model
Modelica.Mechanics.Rotational.LossyGear instead which implements
gear efficiency in a much more reliable way.
</p>
<p>
This component consists of two rigidly connected flanges flange_a and flange_b without
inertia where an <b>efficency</b> coefficient <b>eta</b> reduces the driven
torque as function of the driving torque depending on the direction
of the energy flow, i.e., energy is always lost. This can be seen as a
simple model of the Coulomb friction acting between the teeth of a
gearbox.
</p>
<p>
Note, that most gearbox manufacturers provide tables of the
efficiency of a gearbox as function of the angular velocity
(efficiency becomes zero, if the angular velocity is zero).
However, such a table is practically useless for simulation purposes,
because in gearboxes always two types of friction is present:
(1) Friction in the <b>bearings</b> and (2) friction between
the teeth of the gear. (1) leads to a velocity dependent, additive
loss-torque, whereas (2) leads to a torque-dependent reduction of the
driving torque. The gearbox manufacturers measure both effects
together and determine the gear efficiency from it, although for
simulation purposes the two effects need to be separated.
Assume for example that only constant bearing friction, i.e.,
bearingTorque=const., is present, i.e.,
</p>
<pre>
   (1)  loadTorque = motorTorque - sign(w)*bearingTorque
</pre>
<p>
Gearbox manufacturers use the loss-formula
</p>
<pre>
   (2)  loadTorque = eta*motorTorque
</pre>
<p>
Comparing (1) and (2) gives a formulat for the efficiency eta:
</p>
<pre>
   eta = (1 - sign(w)*bearingTorque/motorTorque)
</pre>
<p>
When the motorTorque becomes smaller as the bearingTorque,
(2) is useless, because the efficiency is zero. To summarize,
be careful to determine the gear <b>efficiency</b> of this element
from tables of the gear manufacturers.
</p>

</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -96,20},{96, -21}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Line(points = {{ -30, -40},{30, -40}}, color = {0,0,0}),Line(points = {{0,60},{0,40}}, color = {0,0,0}),Line(points = {{ -30,40},{29,40}}, color = {0,0,0}),Line(points = {{0, -40},{0, -90}}, color = {0,0,0}),Polygon(points = {{ -30, -20},{60, -20},{60, -80},{70, -80},{50, -100},{30, -80},{40, -80},{40, -30},{ -30, -30},{ -30, -20},{ -30, -20}}, lineColor = {0,0,0}, fillColor = {255,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{16,83},{84,70}}, lineColor = {128,128,128}, textString = "rotation axis"),Polygon(points = {{12,76},{ -8,81},{ -8,71},{12,76}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{ -78,76},{ -7,76}}, color = {128,128,128}),Line(points = {{30, -50},{20, -60}}, color = {0,0,0}),Line(points = {{30, -40},{10, -60}}, color = {0,0,0}),Line(points = {{20, -40},{0, -60}}, color = {0,0,0}),Line(points = {{10, -40},{ -10, -60}}, color = {0,0,0}),Line(points = {{0, -40},{ -20, -60}}, color = {0,0,0}),Line(points = {{ -10, -40},{ -30, -60}}, color = {0,0,0}),Line(points = {{ -20, -40},{ -30, -50}}, color = {0,0,0})}));

      equation 
        phi = phi_a;
        phi = phi_b;
        power_a = flange_a.tau * der(phi);
        driving_a = power_a >= 0;
        flange_b.tau =  -(if driving_a then eta * flange_a.tau else flange_a.tau / eta);
      end GearEfficiency;
      model Gear "Obsolete model. Use Modelica.Mechanics.Rotational.Components.Gearbox instead"
        extends ObsoleteModelica3.Mechanics.Rotational.Interfaces.TwoFlangesAndBearingH;
        parameter Real ratio = 1 "transmission ratio (flange_a.phi/flange_b.phi)";
        parameter Real eta(min = Modelica.Constants.small, max = 1) = 1 "Gear efficiency";
        parameter Real friction_pos[:,2] = [0,1] "[w,tau] positive sliding friction characteristic (w>=0)";
        parameter Real peak(final min = 1) = 1 "peak*friction_pos[1,2] = maximum friction torque at zero velocity";
        parameter Real c(final unit = "N.m/rad", final min = Modelica.Constants.small) = 100000.0 "Gear elasticity (spring constant)";
        parameter Real d(final unit = "N.m.s/rad", final min = 0) = 0 "(relative) gear damping";
        parameter Modelica.SIunits.Angle b(final min = 0) = 0 "Total backlash";
        annotation(__Dymola_obsolete = "This model can get stuck due when the torque direction varies, use Modelica.Mechanics.Rotational.Components.Gearbox instead.", Documentation(info = "<html>
<p>
This component models the essential effects of a gearbox, in particular
gear <b>efficiency</b> due to friction between the teeth, <b>bearing friction</b>,
gear <b>elasticity</b> and <b>damping</b>, <b>backlash</b>.
The inertia of the gear wheels is not modeled. If necessary, inertia
has to be taken into account by connecting components of model Inertia
to the left and/or the right flange.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -40,60},{40, -60}}, lineColor = {0,0,0}, pattern = LinePattern.Solid, lineThickness = 0.25, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Polygon(points = {{ -60, -80},{ -46, -80},{ -20, -20},{20, -20},{46, -80},{60, -80},{60, -90},{ -60, -90},{ -60, -80}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -100,10},{ -60, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{60,10},{100, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Polygon(points = {{ -60,10},{ -60,20},{ -40,40},{ -40, -40},{ -60, -20},{ -60,10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {128,128,128}),Polygon(points = {{60,20},{40,40},{40, -40},{60, -20},{60,20}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,110},{150,70}}, textString = "%name=%ratio", lineColor = {0,0,255}),Text(extent = {{ -150, -160},{150, -120}}, lineColor = {0,0,0}, textString = "c=%c")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{2,29},{46,22}}, lineColor = {128,128,128}, textString = "rotation axis"),Polygon(points = {{4,25},{ -4,27},{ -4,23},{4,25}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{ -36,25},{ -3,25}}, color = {128,128,128})}));
        Modelica.Mechanics.Rotational.Components.IdealGear gearRatio(final ratio = ratio) annotation(Placement(transformation(extent = {{ -70, -10},{ -50,10}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.Rotational.GearEfficiency gearEfficiency(final eta = eta) annotation(Placement(transformation(extent = {{ -30, -10},{ -10,10}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.ElastoBacklash elastoBacklash(final b = b, final c = c, final phi_rel0 = 0, final d = d) annotation(Placement(transformation(extent = {{50, -10},{70,10}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.BearingFriction bearingFriction(final tau_pos = friction_pos, final peak = peak) annotation(Placement(transformation(extent = {{10, -10},{30,10}}, rotation = 0)));

      equation 
        connect(flange_a,gearRatio.flange_a) annotation(Line(points = {{ -100,0},{ -70,0}}, color = {0,0,0}));
        connect(gearRatio.flange_b,gearEfficiency.flange_a) annotation(Line(points = {{ -50,0},{ -30,0}}, color = {0,0,0}));
        connect(gearEfficiency.flange_b,bearingFriction.flange_a) annotation(Line(points = {{ -10,0},{10,0}}, color = {0,0,0}));
        connect(bearingFriction.flange_b,elastoBacklash.flange_a) annotation(Line(points = {{30,0},{50,0}}, color = {0,0,0}));
        connect(elastoBacklash.flange_b,flange_b) annotation(Line(points = {{70,0},{100,0}}, color = {0,0,0}));
        connect(gearEfficiency.bearing,adapter.flange_b) annotation(Line(points = {{ -20, -10},{ -20, -40},{6.12323e-16, -40},{6.12323e-16, -50}}, color = {0,0,0}));
        connect(bearingFriction.support,adapter.flange_b) annotation(Line(points = {{20, -10},{20, -40},{6.12323e-16, -40},{6.12323e-16, -50}}, color = {0,0,0}, smooth = Smooth.None));
        connect(gearRatio.support,adapter.flange_b) annotation(Line(points = {{ -60, -10},{ -60, -40},{6.12323e-16, -40},{6.12323e-16, -50}}, color = {0,0,0}, smooth = Smooth.None));
      end Gear;
    end Rotational;
  end Mechanics;
end ObsoleteModelica3;
package ModelicaTest "Library to test components of library Modelica "
  annotation(version = "1.1", versionDate = "2007-10-19", uses(Modelica(version = "3.1")), Documentation(info = "<html>
<p>
This library provides models and functions to test components of
<b>package Modelica</b> (the Modelica Standard Library).
</p>

<p>
Further development of this library should be performed in the following
way:
</p>

<ul>
<li> Functions that are added to this library to test functions of the
     Modelica Standard Library, should be called in \"ModelicaTest.testAllFunctions()\".
     The idea is that all test functions are called, when calling
     \"testAllFunctions()\".</li>

<li> Models that are added to this library should have the annotation
     (with an appropriate StopTime):
     <pre>
         <b>annotation</b>(experiment(StopTime=1.1)); </pre>
     This gives the tool vendors the possibility to automatically identify
     the models that shall be simulated and, e.g., that shall be used in an automatic
     regression test.</li>
</ul>

<p>
Copyright &copy; 1998-2007, Modelica Association.
</p>
<p>
<i>This Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense\">here</a>.</i>
</p>
</html>"), conversion(from(version = "1.0", script = "ConvertFromModelicaTest_1.0.mos")));
  function testAllFunctions "Runs all test cases for functions"
    import Modelica.Utilities.Streams.print;
    input String logFile = "ModelicaTestLog.txt" "Filename where the log of all functions is stored";
    output Boolean ok;
  protected 
    Boolean result;
    String file;
  algorithm 
    file:=Modelica.Utilities.Files.fullPathName(logFile);
    print("... testAllFunctions(..) is logged in " + file);
    if file <> "" then 
        Modelica.Utilities.Files.removeFile(file);

    else 
    end if;
    print("--- Test functions of Modelica library", logFile);
    result:=ModelicaTest.Math.Polynomials(logFile);
    result:=ModelicaTest.Math.Matrices(logFile);
    result:=ModelicaTest.Utilities.testAll(logFile);
    ok:=true;
  end testAllFunctions;
  package Fluid "Test cases for the Modelica.Fluid library"
    import SI = Modelica.SIunits;
    package TestComponents "Models to test components of the library"
      extends Modelica.Icons.Library;
      annotation(classOrder = {"TestTanks","*"});
      package Vessels "Test Vessel models"
        model TestVolume
          extends Modelica.Icons.Example;
          Modelica.Fluid.Vessels.ClosedVolume Volume(redeclare package Medium = Modelica.Media.Water.StandardWater, V = 1, use_T_start = false, h_start = 3000000.0, nPorts = 2, energyDynamics = Modelica.Fluid.Types.Dynamics.SteadyState, massDynamics = Modelica.Fluid.Types.Dynamics.SteadyState, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.0254),Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.0254)}) annotation(Placement(transformation(extent = {{ -40,14},{ -20,34}}, rotation = 0)));
          Modelica.Fluid.Sources.MassFlowSource_h FlowSource(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, m_flow = 1, h = 3000000.0) annotation(Placement(transformation(extent = {{ -82,0},{ -62,20}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT Sink(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, p = 101325, T = system.T_ambient) annotation(Placement(transformation(extent = {{60,0},{40,20}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveLinear Valve(redeclare package Medium = Modelica.Media.Water.StandardWater, dp_nominal = 10000, m_flow_nominal = 0.1) annotation(Placement(transformation(extent = {{2,0},{22,20}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 5));
          Modelica.Blocks.Sources.Step Step1(startTime = 1, height =  -0.5, offset = 1) annotation(Placement(transformation(extent = {{ -40,48},{ -20,68}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100, -100},{ -80, -80}}, rotation = 0)));

        equation 
          connect(Valve.port_b,Sink.ports[1]) annotation(Line(points = {{22,10},{40,10}}, color = {0,127,255}));
          connect(Step1.y,Valve.opening) annotation(Line(points = {{ -19,58},{12,58},{12,18}}, color = {0,0,127}));
          connect(FlowSource.ports[1],Volume.ports[1]) annotation(Line(points = {{ -62,10},{ -32,10},{ -32,14}}, color = {0,127,255}, smooth = Smooth.None));
          connect(Volume.ports[2],Valve.port_a) annotation(Line(points = {{ -28,14},{ -28,10},{2,10}}, color = {0,127,255}, smooth = Smooth.None));
        end TestVolume;
        model TestTankWithTopPorts
          extends Modelica.Icons.Example;
          import Modelica.Fluid;
          Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts tank(nTopPorts = 2, height = 10, V0 = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, level_start = 1, nPorts = 4, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.01, height = 9),Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.01, height = 6),Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.01, height = 4),Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.01, height = 2)}, crossArea = 0.2) annotation(Placement(transformation(extent = {{ -40,40},{0,80}}, rotation = 0)));
          Modelica.Fluid.Sources.MassFlowSource_T massFlowRate[2](each nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, each m_flow = 0.75) annotation(Placement(transformation(extent = {{ -82,70},{ -62,90}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 5000, Tolerance = 1e-05), experimentSetupOutput(equdistant = false));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100, -100},{ -80, -80}}, rotation = 0)));
          Modelica.Fluid.Sources.FixedBoundary Boundary_fixed(redeclare package Medium = Modelica.Media.Water.StandardWater, p = system.p_ambient, T = system.T_ambient, nPorts = 4) annotation(Placement(transformation(extent = {{100, -90},{80, -70}}, rotation = 0)));
          Modelica.Fluid.Sensors.MassFlowRate mFlow_9m(redeclare package Medium = Modelica.Media.Water.StandardWater) "Mass flow rate out of the port at a lever of 9 m" annotation(Placement(transformation(extent = {{20,30},{40,50}}, rotation = 0)));
          Modelica.Fluid.Sensors.MassFlowRate mFlow_6m(redeclare package Medium = Modelica.Media.Water.StandardWater) "Mass flow rate out of the port at a lever of 6 m" annotation(Placement(transformation(extent = {{20, -10},{40,10}}, rotation = 0)));
          Modelica.Fluid.Sensors.MassFlowRate mFlow_4m(redeclare package Medium = Modelica.Media.Water.StandardWater) "Mass flow rate out of the port at a lever of 4 m" annotation(Placement(transformation(extent = {{20, -50},{40, -30}}, rotation = 0)));
          Modelica.Fluid.Sensors.MassFlowRate mFlow_2m(redeclare package Medium = Modelica.Media.Water.StandardWater) "Mass flow rate out of the port at a lever of 2 m" annotation(Placement(transformation(extent = {{20, -90},{40, -70}}, rotation = 0)));

        equation 
          connect(massFlowRate.ports[1],tank.topPorts) annotation(Line(points = {{ -62,80},{ -40,80},{ -40,81},{ -20,81}}, color = {0,127,255}, smooth = Smooth.None));
          connect(tank.ports[1],mFlow_9m.port_a) annotation(Line(points = {{ -23,39},{ -0.5,39},{ -0.5,40},{20,40}}, color = {0,0,255}, smooth = Smooth.None));
          connect(mFlow_9m.port_b,Boundary_fixed.ports[1]) annotation(Line(points = {{40,40},{60,40},{60, -77},{80, -77}}, color = {0,127,255}, smooth = Smooth.None));
          connect(tank.ports[2],mFlow_6m.port_a) annotation(Line(points = {{ -21,39},{0,39},{0,0},{20,0}}, color = {0,0,255}, smooth = Smooth.None));
          connect(mFlow_6m.port_b,Boundary_fixed.ports[2]) annotation(Line(points = {{40,0},{60,0},{60, -79},{80, -79}}, color = {0,127,255}, smooth = Smooth.None));
          connect(tank.ports[3],mFlow_4m.port_a) annotation(Line(points = {{ -19,39},{0,39},{0, -40},{20, -40}}, color = {0,0,255}, smooth = Smooth.None));
          connect(mFlow_4m.port_b,Boundary_fixed.ports[3]) annotation(Line(points = {{40, -40},{60, -40},{60, -81},{80, -81}}, color = {0,127,255}, smooth = Smooth.None));
          connect(tank.ports[4],mFlow_2m.port_a) annotation(Line(points = {{ -17,39},{0,39},{0, -80},{20, -80}}, color = {0,0,255}, smooth = Smooth.None));
          connect(mFlow_2m.port_b,Boundary_fixed.ports[4]) annotation(Line(points = {{40, -80},{60, -80},{60, -83},{80, -83}}, color = {0,127,255}, smooth = Smooth.None));
        end TestTankWithTopPorts;
        model TestSimpleTank "Test a tank running empty and being filled through a top inlet with less water than could flow through the bottom outlet"
          import Modelica.SIunits.Conversions.from_bar;
          extends Modelica.Icons.Example;
          Modelica.Fluid.Vessels.OpenTank tank(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, height = 1, nPorts = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 0),Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 1)}, crossArea = 1, level_start = 0) annotation(Placement(transformation(extent = {{0,0},{40,40}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{70,70},{90,90}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT ambient_fixed(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, p = system.p_ambient, T = system.T_ambient, nPorts = 1) annotation(Placement(transformation(extent = {{ -40, -70},{ -20, -50}}, rotation = 0)));
          Modelica.Fluid.Pipes.StaticPipe pipe(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, length = 1, height_ab =  -1, diameter = 0.1) annotation(Placement(transformation(origin = {20, -30}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Modelica.Fluid.Sources.MassFlowSource_T flowSource(nPorts = 1, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, m_flow = 20, T = system.T_ambient, use_m_flow_in = true) annotation(Placement(transformation(extent = {{ -40,30},{ -20,50}}, rotation = 0)));
          Modelica.Blocks.Sources.TimeTable timeTable(table = [0,0;10,0;10,40;20,40;20,10;50,10;50,0;60,0;60,20;70,20;80,55;80,0;100,0]) annotation(Placement(transformation(extent = {{ -80,40},{ -60,60}})));

        equation 
          connect(pipe.port_a,tank.ports[1]) annotation(Line(points = {{20, -20},{20,0},{16,0}}, color = {0,127,255}, smooth = Smooth.None));
          connect(flowSource.ports[1],tank.ports[2]) annotation(Line(points = {{ -20,40},{0,40},{0,0},{24,0}}, color = {0,127,255}, smooth = Smooth.None));
          connect(ambient_fixed.ports[1],pipe.port_b) annotation(Line(points = {{ -20, -60},{20, -60},{20, -40}}, color = {0,127,255}, smooth = Smooth.None));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 100), experimentSetupOutput);
          connect(flowSource.m_flow_in,timeTable.y) annotation(Line(points = {{ -40,48},{ -50,48},{ -50,50},{ -59,50}}, color = {0,0,127}, smooth = Smooth.None));
        end TestSimpleTank;
        model TestMixingVolumesPressureStates "Test case where in one of the mixing volumes a pressure state appears"
          import Modelica.Fluid;
          extends Modelica.Icons.Example;
          package Medium = Modelica.Media.Water.StandardWater;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 10));
          Modelica.Fluid.Vessels.ClosedVolume mixingVolume1(V = 0.001, redeclare package Medium = Medium, p_start = system.p_ambient, use_T_start = true, T_start = system.T_ambient, nPorts = 2, energyDynamics = Modelica.Fluid.Types.Dynamics.SteadyStateInitial, massDynamics = Modelica.Fluid.Types.Dynamics.SteadyStateInitial, use_portsData = false) annotation(Placement(transformation(extent = {{ -30,38},{ -10,58}}, rotation = 0)));
          Modelica.Fluid.Sources.MassFlowSource_h flowSource2(nPorts = 1, m_flow = 1, h = 200000.0, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -100,30},{ -80,50}}, rotation = 0)));
          Modelica.Fluid.Vessels.ClosedVolume mixingVolume2(V = 0.001, p_start = system.p_ambient, use_T_start = false, h_start = 100000.0, redeclare package Medium = Medium, nPorts = 2, use_portsData = false) annotation(Placement(transformation(extent = {{10,38},{30,58}}, rotation = 0)));
          Modelica.Fluid.Sensors.Temperature Tmix_in(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -50,70},{ -30,90}}, rotation = 0)));
          Modelica.Fluid.Sensors.Temperature Tmix_out(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{30,68},{50,88}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_ph sink2(nPorts = 1, redeclare package Medium = Medium, h = Medium.h_default, p = 101325) annotation(Placement(transformation(extent = {{100,30},{80,50}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100, -100},{ -80, -80}}, rotation = 0)));
          Modelica.Fluid.Pipes.BaseClasses.WallFriction.TestWallFrictionAndGravity simpleGenericOrifice2(redeclare package Medium = Medium, diameter = 0.2, length = 1, redeclare package WallFriction = Modelica.Fluid.Pipes.BaseClasses.WallFriction.Laminar) annotation(Placement(transformation(extent = {{50,30},{70,50}}, rotation = 0)));
          Modelica.Fluid.Pipes.BaseClasses.WallFriction.TestWallFrictionAndGravity simpleGenericOrifice1(redeclare package Medium = Medium, diameter = 0.2, length = 1, redeclare package WallFriction = Modelica.Fluid.Pipes.BaseClasses.WallFriction.Laminar) annotation(Placement(transformation(extent = {{ -70,30},{ -50,50}}, rotation = 0)));

        equation 
          connect(simpleGenericOrifice2.port_b,sink2.ports[1]) annotation(Line(points = {{70,40},{80,40}}, color = {0,127,255}, smooth = Smooth.None));
          connect(flowSource2.ports[1],simpleGenericOrifice1.port_a) annotation(Line(points = {{ -80,40},{ -70,40}}, color = {0,127,255}, smooth = Smooth.None));
          connect(simpleGenericOrifice1.port_b,Tmix_in.port) annotation(Line(points = {{ -50,40},{ -40,40},{ -40,70}}, color = {0,127,255}, smooth = Smooth.None));
          connect(Tmix_out.port,simpleGenericOrifice2.port_a) annotation(Line(points = {{40,68},{40,40},{50,40}}, color = {0,127,255}, smooth = Smooth.None));
          connect(simpleGenericOrifice1.port_b,mixingVolume1.ports[1]) annotation(Line(points = {{ -50,40},{ -36,40},{ -36,38},{ -22,38}}, color = {0,127,255}, smooth = Smooth.None));
          connect(mixingVolume1.ports[2],mixingVolume2.ports[2]) annotation(Line(points = {{ -18,38},{ -18,34},{22,34},{22,38}}, color = {0,127,255}, smooth = Smooth.None));
          connect(mixingVolume2.ports[1],simpleGenericOrifice2.port_a) annotation(Line(points = {{18,38},{34,38},{34,40},{50,40}}, color = {0,127,255}, smooth = Smooth.None));
        end TestMixingVolumesPressureStates;
        model TestMixingVolumes
          extends Modelica.Icons.Example;
          package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 10));
          Modelica.Fluid.Vessels.ClosedVolume mixingVolume1(V = 0.001, use_T_start = false, h_start = 100000.0, redeclare package Medium = Medium, nPorts = 3, use_portsData = false) annotation(Placement(transformation(extent = {{ -30,40},{ -10,60}}, rotation = 0)));
          Modelica.Fluid.Sources.MassFlowSource_h flowSource2(nPorts = 1, m_flow = 1, h = 200000.0, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -98,24},{ -78,44}}, rotation = 0)));
          Modelica.Fluid.Vessels.ClosedVolume mixingVolume2(V = 0.001, use_T_start = false, h_start = 100000.0, redeclare package Medium = Medium, nPorts = 3, use_portsData = false) annotation(Placement(transformation(extent = {{10,40},{30,60}}, rotation = 0)));
          Modelica.Fluid.Sensors.Temperature Tmix_in(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -60,50},{ -40,70}}, rotation = 0)));
          Modelica.Fluid.Sensors.Temperature Tmix_out(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{40,50},{60,70}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_ph sink2(nPorts = 1, redeclare package Medium = Medium, h = Medium.h_default, p = 101325) annotation(Placement(transformation(extent = {{100,30},{80,50}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100, -100},{ -80, -80}}, rotation = 0)));

        equation 
          connect(flowSource2.ports[1],mixingVolume1.ports[2]) annotation(Line(points = {{ -78,34},{ -52,34},{ -20,34},{ -20,40}}, color = {0,127,255}));
          connect(mixingVolume2.ports[2],sink2.ports[1]) annotation(Line(points = {{20,40},{20,40},{80,40}}, color = {0,127,255}));
          connect(Tmix_in.port,mixingVolume1.ports[1]) annotation(Line(points = {{ -50,50},{ -50,36},{ -22,36},{ -22,40},{ -22.6667,40}}, color = {0,127,255}, smooth = Smooth.None));
          connect(Tmix_out.port,mixingVolume2.ports[1]) annotation(Line(points = {{50,50},{50,40},{17.3333,40}}, color = {0,127,255}, smooth = Smooth.None));
          connect(mixingVolume1.ports[3],mixingVolume2.ports[3]) annotation(Line(points = {{ -17.3333,40},{ -17.3333,36},{22.6667,36},{22.6667,40}}, color = {0,127,255}, smooth = Smooth.None));
        end TestMixingVolumes;
        model TestInitialization
          extends Modelica.Icons.Example;
          package Medium = Modelica.Media.Air.SimpleAir;
          Modelica.Fluid.Sources.Boundary_pT sou1(nPorts = 1, redeclare package Medium = Medium, p = 101330, T = 293.15) annotation(Placement(transformation(extent = {{ -90,10},{ -70,30}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT sin1(nPorts = 1, redeclare package Medium = Medium, p = 101320, T = 293.15) annotation(Placement(transformation(extent = {{90,10},{70,30}}, rotation = 0)));
          Modelica.Fluid.Pipes.StaticPipe pipe1(redeclare package Medium = Medium, length = 1, diameter = 0.25) annotation(Placement(transformation(extent = {{ -50,10},{ -30,30}}, rotation = 0)));
          Modelica.Fluid.Pipes.StaticPipe pipe2(redeclare package Medium = Medium, length = 1, diameter = 0.25) annotation(Placement(transformation(extent = {{30,10},{50,30}}, rotation = 0)));
          Modelica.Fluid.Vessels.ClosedVolume vol1(redeclare package Medium = Medium, V = 0.1, nPorts = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.25),Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.25)}) annotation(Placement(transformation(extent = {{ -10,20},{10,40}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100, -100},{ -80, -80}})));

        equation 
          connect(sou1.ports[1],pipe1.port_a) annotation(Line(points = {{ -70,20},{ -50,20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipe1.port_b,vol1.ports[1]) annotation(Line(points = {{ -30,20},{ -2,20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(vol1.ports[2],pipe2.port_a) annotation(Line(points = {{2,20},{30,20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipe2.port_b,sin1.ports[1]) annotation(Line(points = {{50,20},{70,20}}, color = {0,127,255}, smooth = Smooth.None));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -100, -8},{100, -14}}, lineColor = {0,0,255}, textString = "Problem: Medium SimpleAir missing stateSelect.prefer on pressure p (see Ticket #63)"),Text(extent = {{ -100, -28},{100, -34}}, lineColor = {0,0,255}, textString = "This leads to the bad state m without start value and results in a division by zero"),Text(extent = {{ -100, -46},{100, -52}}, lineColor = {0,0,255}, textString = "Work-around: Use Medium MoistAir.")}), experiment(StopTime = 1));
        end TestInitialization;
      end Vessels;
      package Valves "Test Valve models"
        model TestValveVaporizing "Test case for vaporizing fluid valve"
          extends Modelica.Icons.Example;
          Modelica.Fluid.Sources.Boundary_pT SourceP1(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, p = 200000, T(displayUnit = "degC") = 373.15) annotation(Placement(transformation(extent = {{ -76, -22},{ -56, -2}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveVaporizing V1(redeclare package Medium = Modelica.Media.Water.StandardWater, m_flow_nominal = 1, CvData = Modelica.Fluid.Types.CvTypes.Av, Av = 0.00024, dp_nominal = 100000, rho_nominal = 1000) annotation(Placement(transformation(extent = {{ -26, -22},{ -6, -2}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT SinkP1(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, use_p_in = true, p = 100000, T = 293.15) annotation(Placement(transformation(extent = {{46, -22},{26, -2}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant Opening(k = 1) annotation(Placement(transformation(extent = {{ -78,22},{ -58,42}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{64,60},{84,80}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp downstreamPressure(offset = 200000.0, duration = 0.6, startTime = 0.2, height =  -150000.0) annotation(Placement(transformation(extent = {{14,20},{34,40}})));

        equation 
          connect(V1.port_b,SinkP1.ports[1]) annotation(Line(points = {{ -6, -12},{26, -12}}));
          connect(Opening.y,V1.opening) annotation(Line(points = {{ -57,32},{ -16,32},{ -16, -3}}, color = {0,0,255}));
          connect(SourceP1.ports[1],V1.port_a) annotation(Line(points = {{ -56, -12},{ -26, -12}}, color = {0,127,255}));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment);
          connect(downstreamPressure.y,SinkP1.p_in) annotation(Line(points = {{35,30},{60,30},{60, -6},{48, -6}}, color = {0,0,127}, smooth = Smooth.None));
        end TestValveVaporizing;
        model TestValvesReverse "Test case for valves with reverse and zero flow"
          import Modelica.Fluid;
          extends Modelica.Icons.Example;
          package Medium = Modelica.Media.Water.StandardWater;
          Modelica.Fluid.Sources.Boundary_pT SourceP1(T = system.T_ambient, redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, nPorts = 2, p = 1000000) annotation(Placement(transformation(extent = {{ -100,26},{ -80,46}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT SourceP2(T = system.T_ambient, redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, nPorts = 2, p = 800000) annotation(Placement(transformation(extent = {{ -100, -50},{ -80, -30}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT SinkP1(nPorts = 1, T = system.T_ambient, redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, p = 100000) annotation(Placement(transformation(extent = {{82, -4},{62,16}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveIncompressible V1(dp_nominal = 900000.0, m_flow_nominal = 1.5, redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, Av = 0.1) annotation(Placement(transformation(extent = {{ -50,58},{ -30,78}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveIncompressible V2(dp_nominal = 500000.0, m_flow_nominal = 1.2, redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, Av = 0.1) annotation(Placement(transformation(extent = {{ -38,26},{ -18,46}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveIncompressible V3(dp_nominal = 300000.0, m_flow_nominal = 1.1, redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint) annotation(Placement(transformation(extent = {{ -38, -38},{ -18, -18}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveIncompressible V4(dp_nominal = 800000.0, m_flow_nominal = 1.3, redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint) annotation(Placement(transformation(extent = {{ -40, -78},{ -20, -58}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveIncompressible V5(dp_nominal = 400000.0, m_flow_nominal = 2, redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint) annotation(Placement(transformation(extent = {{30, -4},{50,16}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 4, Tolerance = 1e-06), Documentation(info = ""));
          Modelica.Fluid.Sources.Boundary_pT SinkP2(nPorts = 1, T = system.T_ambient, redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, p = 100000) annotation(Placement(transformation(extent = {{4,58},{ -16,78}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT SinkP3(nPorts = 1, T = system.T_ambient, redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, p = 100000) annotation(Placement(transformation(extent = {{26, -78},{6, -58}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp CloseLoad(duration = 1, offset = 1, startTime = 1, height =  -0.99) annotation(Placement(transformation(extent = {{8,26},{28,46}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp OpenRelief(duration = 2, height = 1, offset = 0, startTime = 1) annotation(Placement(transformation(extent = {{ -92,70},{ -72,90}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp CloseValves(duration = 2, offset = 1, startTime = 1, height =  -1) annotation(Placement(transformation(extent = {{ -96, -12},{ -76,8}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{60,68},{80,88}}, rotation = 0)));

        equation 
          connect(V1.port_b,SinkP2.ports[1]) annotation(Line(points = {{ -30,68},{ -16,68}}));
          connect(V4.port_b,SinkP3.ports[1]) annotation(Line(points = {{ -20, -68},{6, -68}}));
          connect(SourceP1.ports[1],V1.port_a) annotation(Line(points = {{ -80,38},{ -68,38},{ -68,68},{ -50,68}}, color = {0,127,255}));
          connect(SourceP1.ports[2],V2.port_a) annotation(Line(points = {{ -80,34},{ -59,34},{ -59,36},{ -38,36}}, color = {0,127,255}));
          connect(V2.port_b,V5.port_a) annotation(Line(points = {{ -18,36},{5,36},{5,6},{30,6}}, color = {0,127,255}));
          connect(V3.port_b,V5.port_a) annotation(Line(points = {{ -18, -28},{6, -28},{6,6},{30,6}}, color = {0,127,255}));
          connect(SourceP2.ports[2],V4.port_a) annotation(Line(points = {{ -80, -42},{ -60, -42},{ -60, -68},{ -40, -68}}, color = {0,127,255}));
          connect(SourceP2.ports[1],V3.port_a) annotation(Line(points = {{ -80, -38},{ -60, -38},{ -60, -28},{ -38, -28}}, color = {0,127,255}));
          connect(OpenRelief.y,V1.opening) annotation(Line(points = {{ -71,80},{ -40,80},{ -40,76}}, color = {0,0,127}));
          connect(OpenRelief.y,V4.opening) annotation(Line(points = {{ -71,80},{ -64,80},{ -64, -52},{ -30, -52},{ -30, -60}}, color = {0,0,127}));
          connect(CloseValves.y,V2.opening) annotation(Line(points = {{ -75, -2},{ -46, -2},{ -46,54},{ -28,54},{ -28,44}}, color = {0,0,127}));
          connect(CloseValves.y,V3.opening) annotation(Line(points = {{ -75, -2},{ -28, -2},{ -28, -20}}, color = {0,0,127}));
          connect(CloseLoad.y,V5.opening) annotation(Line(points = {{29,36},{40,36},{40,14}}, color = {0,0,127}));
          connect(V5.port_b,SinkP1.ports[1]) annotation(Line(points = {{50,6},{62,6}}, color = {0,127,255}));
        end TestValvesReverse;
        model TestValveLinear "Test case for linear valve"
          import Modelica.Fluid;
          extends Modelica.Icons.Example;
          Modelica.Fluid.Sources.Boundary_pT SourceP1(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, p = 200000, T(displayUnit = "degC") = 298.15) annotation(Placement(transformation(extent = {{ -76, -22},{ -56, -2}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveLinear V1(redeclare package Medium = Modelica.Media.Water.StandardWater, dp_nominal = 100000, m_flow_nominal = 2) annotation(Placement(transformation(extent = {{ -26, -22},{ -6, -2}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT SinkP1(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, use_p_in = true, p = 100000, T = 293.15) annotation(Placement(transformation(extent = {{46, -22},{26, -2}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp Opening(height = 1, duration = 0.5, offset = 0, startTime = 0.5) annotation(Placement(transformation(extent = {{ -78,22},{ -58,42}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{64,60},{84,80}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp downstreamPressure(duration = 0.6, height = 100000.0, offset = 100000.0, startTime = 2.2) annotation(Placement(transformation(extent = {{14,20},{34,40}})));

        equation 
          connect(V1.port_b,SinkP1.ports[1]) annotation(Line(points = {{ -6, -12},{26, -12}}));
          connect(SourceP1.ports[1],V1.port_a) annotation(Line(points = {{ -56, -12},{ -26, -12}}, color = {0,127,255}));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 3));
          connect(downstreamPressure.y,SinkP1.p_in) annotation(Line(points = {{35,30},{60,30},{60, -6},{48, -6}}, color = {0,0,127}, smooth = Smooth.None));
          connect(Opening.y,V1.opening) annotation(Line(points = {{ -57,32},{ -16,32},{ -16, -3}}, color = {0,0,127}, smooth = Smooth.None));
        end TestValveLinear;
        model TestValveIncompressible "Test case with different valve characteristics"
          extends Modelica.Icons.Example;
          package Medium = Modelica.Media.Water.StandardWater;
          Modelica.Fluid.Sources.Boundary_pT SourceP1(nPorts = 1, p = 1000000.0, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient) annotation(Placement(transformation(extent = {{ -100,30},{ -80,50}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveIncompressible V1(rho_nominal = 1000, dp_nominal = 900000.0, m_flow_nominal = 1.5, redeclare package Medium = Modelica.Media.Water.StandardWater, CvData = Modelica.Fluid.Types.CvTypes.Cv, Cv = 10) annotation(Placement(transformation(extent = {{ -50,30},{ -30,50}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 4, Tolerance = 1e-06), Documentation(info = ""));
          Modelica.Fluid.Sources.Boundary_pT SinkP1(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient, p = 100000) annotation(Placement(transformation(extent = {{22,30},{2,50}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp Opening(duration = 2, height = 1, offset = 0, startTime = 1) annotation(Placement(transformation(extent = {{ -92,74},{ -72,94}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT SourceP2(nPorts = 1, p = 1000000.0, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient) annotation(Placement(transformation(extent = {{ -100, -10},{ -80,10}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveIncompressible V2(rho_nominal = 1000, dp_nominal = 900000.0, m_flow_nominal = 1.5, redeclare package Medium = Modelica.Media.Water.StandardWater, CvData = Modelica.Fluid.Types.CvTypes.Cv, Cv = 10, redeclare function valveCharacteristic = Modelica.Fluid.Valves.BaseClasses.ValveCharacteristics.equalPercentage) annotation(Placement(transformation(extent = {{ -50, -10},{ -30,10}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT SinkP2(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient, p = 100000) annotation(Placement(transformation(extent = {{22, -10},{2,10}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT SourceP3(nPorts = 1, p = 1000000.0, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient) annotation(Placement(transformation(extent = {{ -100, -50},{ -80, -30}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveIncompressible V3(rho_nominal = 1000, dp_nominal = 900000.0, m_flow_nominal = 1.5, redeclare package Medium = Modelica.Media.Water.StandardWater, CvData = Modelica.Fluid.Types.CvTypes.Cv, Cv = 10, redeclare function valveCharacteristic = Modelica.Fluid.Valves.BaseClasses.ValveCharacteristics.equalPercentage(rangeability = 10)) annotation(Placement(transformation(extent = {{ -50, -50},{ -30, -30}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT SinkP3(nPorts = 1, p = 100000.0, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient) annotation(Placement(transformation(extent = {{22, -50},{2, -30}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{58,72},{78,92}}, rotation = 0)));

        equation 
          connect(V1.port_b,SinkP1.ports[1]) annotation(Line(points = {{ -30,40},{2,40}}));
          connect(Opening.y,V1.opening) annotation(Line(points = {{ -71,84},{ -40,84},{ -40,48}}, color = {0,0,255}));
          connect(SourceP1.ports[1],V1.port_a) annotation(Line(points = {{ -80,40},{ -50,40}}, color = {0,127,255}));
          connect(Opening.y,V2.opening) annotation(Line(points = {{ -71,84},{ -64,84},{ -64,20},{ -40,20},{ -40,8}}, color = {0,0,127}));
          connect(Opening.y,V3.opening) annotation(Line(points = {{ -71,84},{ -64,84},{ -64, -22},{ -40, -22},{ -40, -32}}, color = {0,0,127}));
          connect(SourceP2.ports[1],V2.port_a) annotation(Line(points = {{ -80,0},{ -50,0}}, color = {0,127,255}));
          connect(V2.port_b,SinkP2.ports[1]) annotation(Line(points = {{ -30,0},{2,0}}, color = {0,127,255}));
          connect(SourceP3.ports[1],V3.port_a) annotation(Line(points = {{ -80, -40},{ -50, -40}}, color = {0,127,255}));
          connect(V3.port_b,SinkP3.ports[1]) annotation(Line(points = {{ -30, -40},{2, -40}}, color = {0,127,255}));
        end TestValveIncompressible;
        model TestValveDiscrete "Test case for discrete valve"
          import Modelica.Fluid;
          extends Modelica.Icons.Example;
          Modelica.Fluid.Sources.Boundary_pT SourceP1(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, p = 200000, T(displayUnit = "degC") = 298.15) annotation(Placement(transformation(extent = {{ -76, -22},{ -56, -2}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveDiscrete V1(redeclare package Medium = Modelica.Media.Water.StandardWater, m_flow_nominal = 2, dp_nominal = 100000) annotation(Placement(transformation(extent = {{ -26, -22},{ -6, -2}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT SinkP1(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, use_p_in = true, p = 100000, T = 293.15) annotation(Placement(transformation(extent = {{46, -22},{26, -2}}, rotation = 0)));
          Modelica.Blocks.Sources.BooleanStep Opening(startTime = 0.5) annotation(Placement(transformation(extent = {{ -78,22},{ -58,42}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{64,60},{84,80}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp downstreamPressure(duration = 0.6, height = 100000.0, offset = 100000.0, startTime = 2.2) annotation(Placement(transformation(extent = {{14,20},{34,40}})));

        equation 
          connect(V1.port_b,SinkP1.ports[1]) annotation(Line(points = {{ -6, -12},{26, -12}}));
          connect(SourceP1.ports[1],V1.port_a) annotation(Line(points = {{ -56, -12},{ -26, -12}}, color = {0,127,255}));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 3));
          connect(downstreamPressure.y,SinkP1.p_in) annotation(Line(points = {{35,30},{60,30},{60, -6},{48, -6}}, color = {0,0,127}, smooth = Smooth.None));
          connect(Opening.y,V1.open) annotation(Line(points = {{ -57,32},{ -16,32},{ -16, -4}}, color = {255,0,255}, smooth = Smooth.None));
        end TestValveDiscrete;
        model TestValveCompressible "Test case for incompressible fluid valve"
          extends Modelica.Icons.Example;
          Modelica.Fluid.Sources.Boundary_pT SourceP1(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, p = 1000000, T(displayUnit = "degC") = 673.15) annotation(Placement(transformation(extent = {{ -76, -22},{ -56, -2}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveCompressible V1(redeclare package Medium = Modelica.Media.Water.StandardWater, m_flow_nominal = 1, CvData = Modelica.Fluid.Types.CvTypes.Av, Av = 0.00024, dp_nominal = 500000, rho_nominal = 3, p_nominal = 1000000) annotation(Placement(transformation(extent = {{ -26, -22},{ -6, -2}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT SinkP1(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, use_p_in = true, p = 100000, T = 673.15) annotation(Placement(transformation(extent = {{46, -22},{26, -2}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant Opening(k = 1) annotation(Placement(transformation(extent = {{ -78,22},{ -58,42}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{64,60},{84,80}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine downstreamPressure(amplitude = 500000.0, freqHz = 1, offset = 600000.0, phase = 2.2165681500328) annotation(Placement(transformation(extent = {{14,20},{34,40}})));

        equation 
          connect(V1.port_b,SinkP1.ports[1]) annotation(Line(points = {{ -6, -12},{26, -12}}));
          connect(Opening.y,V1.opening) annotation(Line(points = {{ -57,32},{ -16,32},{ -16, -4}}, color = {0,0,255}));
          connect(SourceP1.ports[1],V1.port_a) annotation(Line(points = {{ -56, -12},{ -26, -12}}, color = {0,127,255}));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment);
          connect(downstreamPressure.y,SinkP1.p_in) annotation(Line(points = {{35,30},{60,30},{60, -6},{48, -6}}, color = {0,0,127}, smooth = Smooth.None));
        end TestValveCompressible;
        model TestValveCoefficients "Test case to compare different choices of flow coefficient"
          extends Modelica.Icons.Example;
          Modelica.Fluid.Sources.Boundary_pT SourceP1(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient, p = 200000) annotation(Placement(transformation(extent = {{ -94,18},{ -74,38}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveIncompressible V1(rho_nominal = 1000, redeclare package Medium = Modelica.Media.Water.StandardWater, m_flow_nominal = 1, CvData = Modelica.Fluid.Types.CvTypes.Av, Av = 0.00024, dp_nominal = 100000) annotation(Placement(transformation(extent = {{ -44,18},{ -24,38}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT SinkP1(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient, p = 100000) annotation(Placement(transformation(extent = {{28,18},{8,38}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant Opening(k = 1) annotation(Placement(transformation(extent = {{ -96,62},{ -76,82}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{64,60},{84,80}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT SourceP2(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient, p = 200000) annotation(Placement(transformation(extent = {{ -94, -18},{ -74,2}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveIncompressible V2(rho_nominal = 1000, redeclare package Medium = Modelica.Media.Water.StandardWater, CvData = Modelica.Fluid.Types.CvTypes.Kv, m_flow_nominal = 1, Kv = 9, dp_nominal = 100000) annotation(Placement(transformation(extent = {{ -44, -18},{ -24,2}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT SinkP2(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient, p = 100000) annotation(Placement(transformation(extent = {{28, -18},{8,2}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT SourceP3(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient, p = 200000) annotation(Placement(transformation(extent = {{ -94, -56},{ -74, -36}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveIncompressible V3(rho_nominal = 1000, redeclare package Medium = Modelica.Media.Water.StandardWater, CvData = Modelica.Fluid.Types.CvTypes.Cv, Cv = 10, m_flow_nominal = 1, dp_nominal = 100000) annotation(Placement(transformation(extent = {{ -44, -56},{ -24, -36}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT SinkP3(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient, p = 100000) annotation(Placement(transformation(extent = {{28, -56},{8, -36}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT SourceP4(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient, p = 200000) annotation(Placement(transformation(extent = {{ -94, -88},{ -74, -68}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveIncompressible V4(rho_nominal = 1000, redeclare package Medium = Modelica.Media.Water.StandardWater, Cv = 10, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 100000, m_flow_nominal = 2.4) annotation(Placement(transformation(extent = {{ -44, -88},{ -24, -68}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT SinkP4(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient, p = 100000) annotation(Placement(transformation(extent = {{28, -88},{8, -68}}, rotation = 0)));

        equation 
          connect(V1.port_b,SinkP1.ports[1]) annotation(Line(points = {{ -24,28},{8,28}}));
          connect(Opening.y,V1.opening) annotation(Line(points = {{ -75,72},{ -34,72},{ -34,36}}, color = {0,0,255}));
          connect(SourceP1.ports[1],V1.port_a) annotation(Line(points = {{ -74,28},{ -44,28}}, color = {0,127,255}));
          connect(V2.port_b,SinkP2.ports[1]) annotation(Line(points = {{ -24, -8},{8, -8}}));
          connect(SourceP2.ports[1],V2.port_a) annotation(Line(points = {{ -74, -8},{ -44, -8}}, color = {0,127,255}));
          connect(V3.port_b,SinkP3.ports[1]) annotation(Line(points = {{ -24, -46},{8, -46}}));
          connect(SourceP3.ports[1],V3.port_a) annotation(Line(points = {{ -74, -46},{ -44, -46}}, color = {0,127,255}));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          connect(V2.opening,Opening.y) annotation(Line(points = {{ -34,0},{ -34,8},{ -62,8},{ -62,72},{ -75,72}}, color = {0,0,127}, smooth = Smooth.None));
          connect(V3.opening,Opening.y) annotation(Line(points = {{ -34, -38},{ -34, -26},{ -62, -26},{ -62,72},{ -75,72}}, color = {0,0,127}, smooth = Smooth.None));
          connect(V4.port_b,SinkP4.ports[1]) annotation(Line(points = {{ -24, -78},{8, -78}}));
          connect(SourceP4.ports[1],V4.port_a) annotation(Line(points = {{ -74, -78},{ -44, -78}}, color = {0,127,255}));
          connect(V4.opening,Opening.y) annotation(Line(points = {{ -34, -70},{ -34, -58},{ -62, -58},{ -62,72},{ -75,72}}, color = {0,0,127}, smooth = Smooth.None));
        end TestValveCoefficients;
      end Valves;
      package Utilities "Test Utility models"
        model TestRegStep "Test regStep function"
          extends Modelica.Icons.Example;
          import Modelica.Fluid.Utilities.*;
          parameter Real x_small = 0.5;
          Real x = time - 0.6;
          Real yRegStep = Modelica.Fluid.Utilities.regStep(x, 1, 0.5, x_small);
          Real yRegStep_der;
          annotation(experiment(StopTime = 1.3), experimentSetupOutput);

        equation 
          yRegStep_der = der(yRegStep);
        end TestRegStep;
        model TestRegSquare2Derivatives "Test whether regSquare2 can be differentiated"
          extends Modelica.Icons.Example;
          parameter Real x_small = 0.1;
          parameter Real k1a = 1;
          parameter Real k1b = 0.2;
          parameter Real k2a = 2;
          parameter Real k2b = 0.2;
          Real x;
          Real y;
          Real yd;
          Real ydd;
          Real k1;
          Real k2;
          Real y2;
          Real y2d;
          Real y2dd;
          annotation(experiment(StopTime = 2, NumberOfIntervals = 5000), experimentSetupOutput);

        equation 
          x = time - 1;
          k1 = k1a + k1b * time;
          k2 = k2a + k2b * time;
          y = Modelica.Fluid.Utilities.regSquare2(x, x_small, k1, k2);
          yd = der(y);
          ydd = der(yd);
          y2 = Modelica.Fluid.Utilities.regSquare2(x, x_small, k1, k2, true, 0.02);
          y2d = der(y2);
          y2dd = der(y2d);
        end TestRegSquare2Derivatives;
        model TestRegRootDerivative "Demonstrate that regRoot cannot be differentiated in a typical pressure drop equation (model fails during simulation)"
          extends Modelica.Icons.Example;
          parameter Real x_small = 0.01;
          Real x;
          Real y;
          Real yd;
          Real k1;
          Real k2;
          annotation(experiment(StopTime = 2, NumberOfIntervals = 5000), experimentSetupOutput, Documentation(info = "<html>
<p>
This test model demonstrates that the regRoot(..) function cannot
be differentiated, if multiplied with a discontinuous factor as it
is usual for pressure drop components where upstream properties
are used. In such a case, the regRoot2(..) function should be used.
In order to test this behaviour, change the line \"yd=0\" to \"yd = der(y)\".
</p>
</html>"));

        equation 
          x = time - 1;
          k1 = 1 + 0.1 * time;
          k2 = 2 + 0.2 * time;
          y = (if x >= 0 then k1 else k2) * Modelica.Fluid.Utilities.regRoot(x, x_small);
          yd = 0 "model does not translate if y=0 is replaced by y=der(y)";
        end TestRegRootDerivative;
        model TestRegRoot2Derivatives "Test whether regRoot2 can be differentiated"
          extends Modelica.Icons.Example;
          parameter Real x_small = 0.01;
          parameter Real k1a = 1;
          parameter Real k1b = 0.2;
          parameter Real k2a = 2;
          parameter Real k2b = 0.2;
          Real x;
          Real y;
          Real yd;
          Real ydd;
          Real k1;
          Real k2;
          Real y2;
          Real y2d;
          Real y2dd;
          annotation(experiment(StopTime = 2, NumberOfIntervals = 5000), experimentSetupOutput);

        equation 
          x = time - 1;
          k1 = k1a + k1b * time;
          k2 = k2a + k2b * time;
          y = Modelica.Fluid.Utilities.regRoot2(x, x_small, k1, k2);
          yd = der(y);
          ydd = der(yd);
          y2 = Modelica.Fluid.Utilities.regRoot2(x, x_small, k1, k2, true, 10);
          y2d = der(y2);
          y2dd = der(y2d);
        end TestRegRoot2Derivatives;
        model TestRegFunctions
          extends Modelica.Icons.Example;
          import Modelica.Fluid.Utilities.*;
          parameter Real delta = 0.01;
          Real x = time + 1e-09;
          Real sqrtx = sqrt(x);
          Real sqrtx_a = regRoot(x, delta);
          Real x2 = x * x;
          Real x2_a = regSquare(x, delta);
          Real x3 = x * x * x;
          Real x3_a = regPow(x, 3, delta);
          Real sqrtxerr = (sqrtx_a - sqrtx) / sqrtx;
          Real x2err = (x2_a - x2) / x2;
          Real x3err = (x3_a - x3) / x3;
        end TestRegFunctions;
      end Utilities;
      package Sources "Test Source models"
        model TestSources "Test model for models in source package"
          import Modelica.Fluid;
          package Medium = Modelica.Media.Air.MoistAir(extraPropertiesNames = {"CO2"});
          Modelica.Fluid.Sources.Boundary_pT boundary(redeclare package Medium = Medium, nPorts = 1, use_C_in = true, use_p_in = true) annotation(Placement(transformation(extent = {{ -50,40},{ -30,60}})));
          Modelica.Fluid.Sources.Boundary_ph boundary1(redeclare package Medium = Medium, nPorts = 1, use_p_in = true, use_X_in = false, use_C_in = true) annotation(Placement(transformation(extent = {{ -50,0},{ -30,20}})));
          Modelica.Fluid.Sources.MassFlowSource_T boundary2(redeclare package Medium = Medium, use_C_in = true, nPorts = 1, m_flow = 0.1) annotation(Placement(transformation(extent = {{ -50, -50},{ -30, -30}})));
          Modelica.Fluid.Sources.MassFlowSource_h boundary3(redeclare package Medium = Medium, use_C_in = true, nPorts = 1, m_flow = 0.1) annotation(Placement(transformation(extent = {{ -50, -90},{ -30, -70}})));
          Modelica.Fluid.Sources.FixedBoundary boundary4(redeclare package Medium = Medium, nPorts = 1) annotation(Placement(transformation(extent = {{80,40},{60,60}})));
          Modelica.Fluid.Sources.Boundary_pT boundary5(redeclare package Medium = Medium, nPorts = 1) annotation(Placement(transformation(extent = {{80,0},{60,20}})));
          Modelica.Fluid.Sources.Boundary_ph boundary6(nPorts = 2, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{96, -70},{76, -50}})));
          Modelica.Fluid.Pipes.StaticPipe pipe(length = 1, diameter = 0.25, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -20,40},{0,60}})));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 2));
          Modelica.Fluid.Pipes.StaticPipe pipe1(length = 1, diameter = 0.25, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{0,0},{20,20}})));
          Modelica.Fluid.Pipes.StaticPipe pipe2(length = 1, diameter = 0.25, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{0, -50},{20, -30}})));
          Modelica.Fluid.Pipes.StaticPipe pipe3(length = 1, diameter = 0.25, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{0, -90},{20, -70}})));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{60,70},{80,90}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp C(duration = 1, height = 0.001519) "substance concentration, raising to 1000 PPM CO2" annotation(Placement(transformation(extent = {{ -100,40},{ -80,60}})));
          Modelica.Blocks.Sources.Ramp P(duration = 2, height =  -100, offset = 101325 + 50) "Pressure" annotation(Placement(transformation(extent = {{ -100,0},{ -80,20}})));
          Modelica.Fluid.Sensors.TraceSubstances traceSubstance(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{0,70},{20,90}})));
          Modelica.Fluid.Sensors.TraceSubstancesTwoPort traceSubstance1(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{22,40},{42,60}})));
          Modelica.Fluid.Sensors.TraceSubstances traceSubstance2(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -22,14},{ -2,34}})));
          Modelica.Fluid.Sensors.TraceSubstances traceSubstance3(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{26, -26},{46, -6}})));
          Modelica.Fluid.Sensors.TraceSubstances traceSubstance4(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -20, -74},{0, -54}})));
          Modelica.Fluid.Fittings.TeeJunctionIdeal junction(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{26, -50},{46, -30}}, rotation = 0)));

        equation 
          connect(boundary.ports[1],pipe.port_a) annotation(Line(points = {{ -30,50},{ -20,50}}, color = {0,127,255}, smooth = Smooth.None));
          connect(boundary1.ports[1],pipe1.port_a) annotation(Line(points = {{ -30,10},{0,10}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipe1.port_b,boundary5.ports[1]) annotation(Line(points = {{20,10},{60,10}}, color = {0,127,255}, smooth = Smooth.None));
          connect(boundary2.ports[1],pipe2.port_a) annotation(Line(points = {{ -30, -40},{0, -40}}, color = {0,127,255}, smooth = Smooth.None));
          connect(boundary3.ports[1],pipe3.port_a) annotation(Line(points = {{ -30, -80},{0, -80}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipe3.port_b,boundary6.ports[2]) annotation(Line(points = {{20, -80},{46, -80},{46, -62},{76, -62}}, color = {0,127,255}, smooth = Smooth.None));
          connect(C.y,boundary.C_in[1]) annotation(Line(points = {{ -79,50},{ -70,50},{ -70,42},{ -52,42}}, color = {0,0,127}, smooth = Smooth.None));
          connect(P.y,boundary1.p_in) annotation(Line(points = {{ -79,10},{ -72,10},{ -72,18},{ -52,18}}, color = {0,0,127}, smooth = Smooth.None));
          connect(P.y,boundary.p_in) annotation(Line(points = {{ -79,10},{ -72,10},{ -72,58},{ -52,58}}, color = {0,0,127}, smooth = Smooth.None));
          connect(C.y,boundary1.C_in[1]) annotation(Line(points = {{ -79,50},{ -70,50},{ -70,2},{ -52,2}}, color = {0,0,127}, smooth = Smooth.None));
          connect(pipe.port_b,traceSubstance1.port_a) annotation(Line(points = {{0,50},{22,50}}, color = {0,127,255}, smooth = Smooth.None));
          connect(traceSubstance1.port_b,boundary4.ports[1]) annotation(Line(points = {{42,50},{60,50}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipe3.port_a,traceSubstance4.port) annotation(Line(points = {{0, -80},{ -6, -80},{ -6, -74},{ -10, -74}}, color = {0,127,255}, smooth = Smooth.None));
          connect(traceSubstance.port,pipe.port_b) annotation(Line(points = {{10,70},{6,70},{6,50},{0,50}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipe1.port_a,traceSubstance2.port) annotation(Line(points = {{0,10},{ -6,10},{ -6,14},{ -12,14}}, color = {0,127,255}, smooth = Smooth.None));
          connect(C.y,boundary2.C_in[1]) annotation(Line(points = {{ -79,50},{ -64,50},{ -64, -48},{ -50, -48}}, color = {0,0,127}, smooth = Smooth.None));
          connect(C.y,boundary3.C_in[1]) annotation(Line(points = {{ -79,50},{ -64,50},{ -64, -88},{ -50, -88}}, color = {0,0,127}, smooth = Smooth.None));
          connect(pipe2.port_b,junction.port_1) annotation(Line(points = {{20, -40},{26, -40}}, color = {0,127,255}, smooth = Smooth.None));
          connect(junction.port_3,traceSubstance3.port) annotation(Line(points = {{36, -30},{36, -26}}, color = {0,127,255}, smooth = Smooth.None));
          connect(junction.port_2,boundary6.ports[1]) annotation(Line(points = {{46, -40},{52, -40},{52, -58},{76, -58}}, color = {0,127,255}, smooth = Smooth.None));
        end TestSources;
      end Sources;
      package Sensors "Test Sensor models"
        model TestTraceSubstances
          import Modelica.Fluid;
          replaceable package Medium = Modelica.Media.Air.MoistAir(extraPropertiesNames = {"CO2"});
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 2, Tolerance = 1e-06));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100, -100},{ -80, -80}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_ph boundary_prescriberho_1(nPorts = 1, p = system.p_ambient, use_p_in = false, use_h_in = false, use_C_in = true, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -40,10},{ -20,30}}, rotation = 0)));
          Modelica.Fluid.Sensors.TraceSubstances traceSubstance(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -10,20},{10,40}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_ph boundary_prescriberho_2(nPorts = 1, p = system.p_ambient, use_h_in = false, use_C_in = true, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -40, -30},{ -20, -10}}, rotation = 0)));
          Modelica.Fluid.Sensors.TraceSubstancesTwoPort traceSubstance1(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -10, -30},{10, -10}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp C(duration = 1, height = 0.001519) "substance concentration, raising to 1000 PPM CO2" annotation(Placement(transformation(extent = {{ -80, -10},{ -60,10}})));

        equation 
          connect(boundary_prescriberho_1.ports[1],traceSubstance.port) annotation(Line(points = {{ -20,20},{0,20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(boundary_prescriberho_2.ports[1],traceSubstance1.port_a) annotation(Line(points = {{ -20, -20},{ -10, -20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(C.y,boundary_prescriberho_1.C_in[1]) annotation(Line(points = {{ -59,0},{ -50,0},{ -50,12},{ -42,12}}, color = {0,0,127}, smooth = Smooth.None));
          connect(C.y,boundary_prescriberho_2.C_in[1]) annotation(Line(points = {{ -59,0},{ -50,0},{ -50, -28},{ -42, -28}}, color = {0,0,127}, smooth = Smooth.None));
        end TestTraceSubstances;
        model TestTemperatureSensor "Test and compare case for the difference between using one port with
   and without explicit junction model and two port sensor for fluid temperature meassuring"
          import Modelica.Fluid;
          Modelica.Fluid.Sensors.Temperature Temperature(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{ -20,40},{0,60}}, rotation = 0)));
          Modelica.Fluid.Sensors.TemperatureTwoPort temperatureTwoPort(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{ -20, -20},{0,0}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100, -100},{ -80, -80}}, rotation = 0)));
          Modelica.Fluid.Vessels.OpenTank openTankCold2(nPorts = 1, level_start = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, height = 2, crossArea = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05)}) annotation(Placement(transformation(extent = {{20,0},{40,20}}, rotation = 0)));
          Modelica.Fluid.Vessels.OpenTank openTankCold1(nPorts = 1, level_start = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, height = 2, crossArea = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05)}) annotation(Placement(transformation(extent = {{20,60},{40,80}}, rotation = 0)));
          Modelica.Fluid.Vessels.OpenTank openTankHot1(nPorts = 1, level_start = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, height = 2, crossArea = 2, T_start = SI.Conversions.from_degC(80), portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05)}) annotation(Placement(transformation(extent = {{60,40},{80,60}}, rotation = 0)));
          Modelica.Fluid.Vessels.OpenTank openTankHot2(nPorts = 1, level_start = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, height = 2, crossArea = 2, T_start = SI.Conversions.from_degC(80), portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05)}) annotation(Placement(transformation(extent = {{60, -20},{80,0}}, rotation = 0)));
          Modelica.Fluid.Sources.MassFlowSource_T massFlowRate1(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, use_m_flow_in = true, T = SI.Conversions.from_degC(50)) annotation(Placement(transformation(extent = {{ -60,30},{ -40,50}}, rotation = 0)));
          Modelica.Fluid.Sources.MassFlowSource_T massFlowRate2(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, use_m_flow_in = true, T = SI.Conversions.from_degC(50)) annotation(Placement(transformation(extent = {{ -60, -20},{ -40,0}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p align = justify>In that test model the behaviour of one port temperature sensors with and without explicit junction models and two port temperature sensor are compared. Therefor each sensor is connected to two tanks with different temperatures and a flow source with changing flow direction.<p>
<p align = justify>With Modelica.Fluid stream connectors, all three sensors show the same expected result.</p>
</html>"));
          Modelica.Blocks.Sources.Sine sine annotation(Placement(transformation(extent = {{ -100,10},{ -80,30}}, rotation = 0)));
          Modelica.Fluid.Sensors.Temperature TemperatureJunction(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{ -20, -80},{0, -60}}, rotation = 0)));
          Modelica.Fluid.Vessels.OpenTank openTankCold3(nPorts = 1, level_start = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, height = 2, crossArea = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05)}) annotation(Placement(transformation(extent = {{20, -60},{40, -40}}, rotation = 0)));
          Modelica.Fluid.Vessels.OpenTank openTankHot3(nPorts = 1, level_start = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, height = 2, crossArea = 2, T_start = SI.Conversions.from_degC(80), portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05)}) annotation(Placement(transformation(extent = {{60, -80},{80, -60}}, rotation = 0)));
          Modelica.Fluid.Sources.MassFlowSource_T massFlowRate3(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, use_m_flow_in = true, T = SI.Conversions.from_degC(50)) annotation(Placement(transformation(extent = {{ -60, -90},{ -40, -70}}, rotation = 0)));
          Modelica.Fluid.Fittings.TeeJunctionIdeal junctionIdeal(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{20, -90},{40, -70}}, rotation = 0)));

        equation 
          connect(massFlowRate2.ports[1],temperatureTwoPort.port_a) annotation(Line(points = {{ -40, -10},{ -20, -10}}, color = {0,127,255}, smooth = Smooth.None));
          connect(massFlowRate1.ports[1],Temperature.port) annotation(Line(points = {{ -40,40},{ -10,40}}, color = {0,127,255}, smooth = Smooth.None));
          connect(sine.y,massFlowRate1.m_flow_in) annotation(Line(points = {{ -79,20},{ -70,20},{ -70,48},{ -60,48}}, color = {0,0,127}, smooth = Smooth.None));
          connect(sine.y,massFlowRate2.m_flow_in) annotation(Line(points = {{ -79,20},{ -70,20},{ -70, -2},{ -60, -2}}, color = {0,0,127}, smooth = Smooth.None));
          connect(massFlowRate3.ports[1],TemperatureJunction.port) annotation(Line(points = {{ -40, -80},{ -10, -80}}, color = {0,127,255}, smooth = Smooth.None));
          connect(sine.y,massFlowRate3.m_flow_in) annotation(Line(points = {{ -79,20},{ -70,20},{ -70, -72},{ -60, -72}}, color = {0,0,127}, smooth = Smooth.None));
          connect(TemperatureJunction.port,junctionIdeal.port_1) annotation(Line(points = {{ -10, -80},{20, -80}}, color = {0,127,255}, smooth = Smooth.None));
          connect(temperatureTwoPort.port_b,openTankCold2.ports[1]) annotation(Line(points = {{0, -10},{0, -6},{30, -6},{30,0}}, color = {0,127,255}));
          connect(Temperature.port,openTankCold1.ports[1]) annotation(Line(points = {{ -10,40},{6,40},{6,56},{30,56},{30,60}}, color = {0,127,255}));
          connect(Temperature.port,openTankHot1.ports[1]) annotation(Line(points = {{ -10,40},{30,40},{30,40},{70,40}}, color = {0,127,255}));
          connect(temperatureTwoPort.port_b,openTankHot2.ports[1]) annotation(Line(points = {{0, -10},{34, -10},{34, -20},{70, -20}}, color = {0,127,255}));
          connect(junctionIdeal.port_3,openTankCold3.ports[1]) annotation(Line(points = {{30, -70},{30, -65},{30, -60},{30, -60}}, color = {0,127,255}));
          connect(junctionIdeal.port_2,openTankHot3.ports[1]) annotation(Line(points = {{40, -80},{55.5, -80},{55.5, -80},{70, -80}}, color = {0,127,255}));
        end TestTemperatureSensor;
        model TestSpecificEntropy
          import Modelica.Fluid;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(Tolerance = 1e-06), experimentSetupOutput);
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100, -100},{ -80, -80}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_ph boundary_prescriberho_1(nPorts = 1, use_h_in = true, redeclare package Medium = Modelica.Media.Water.StandardWater, p = system.p_ambient) annotation(Placement(transformation(extent = {{ -40,10},{ -20,30}}, rotation = 0)));
          Modelica.Fluid.Sensors.SpecificEntropy specificEntropy(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{ -10,20},{10,40}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_ph boundary_prescriberho_2(nPorts = 1, use_h_in = true, redeclare package Medium = Modelica.Media.Water.StandardWater, p = system.p_ambient) annotation(Placement(transformation(extent = {{ -40, -30},{ -20, -10}}, rotation = 0)));
          Modelica.Fluid.Sensors.SpecificEntropyTwoPort specificEntropy1(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{ -10, -30},{10, -10}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sine1(amplitude = 1600000.0, offset = 1800000.0) annotation(Placement(transformation(extent = {{ -80, -10},{ -60,10}}, rotation = 0)));

        equation 
          connect(boundary_prescriberho_1.ports[1],specificEntropy.port) annotation(Line(points = {{ -20,20},{0,20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(sine1.y,boundary_prescriberho_1.h_in) annotation(Line(points = {{ -59,0},{ -52,0},{ -52,24},{ -42,24}}, color = {0,0,127}, smooth = Smooth.None));
          connect(sine1.y,boundary_prescriberho_2.h_in) annotation(Line(points = {{ -59,0},{ -52,0},{ -52, -16},{ -42, -16}}, color = {0,0,127}, smooth = Smooth.None));
          connect(boundary_prescriberho_2.ports[1],specificEntropy1.port_a) annotation(Line(points = {{ -20, -20},{ -10, -20}}, color = {0,127,255}, smooth = Smooth.None));
        end TestSpecificEntropy;
        model TestSpecificEnthalpy
          import Modelica.Fluid;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(Tolerance = 1e-06), experimentSetupOutput);
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100, -100},{ -80, -80}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_ph boundary_prescriberho_1(nPorts = 1, use_h_in = true, redeclare package Medium = Modelica.Media.Water.StandardWater, p = system.p_ambient) annotation(Placement(transformation(extent = {{ -40,10},{ -20,30}}, rotation = 0)));
          Modelica.Fluid.Sensors.SpecificEnthalpy specificEnthalpy(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{ -10,20},{10,40}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_ph boundary_prescriberho_2(nPorts = 1, use_h_in = true, redeclare package Medium = Modelica.Media.Water.StandardWater, p = system.p_ambient) annotation(Placement(transformation(extent = {{ -40, -30},{ -20, -10}}, rotation = 0)));
          Modelica.Fluid.Sensors.SpecificEnthalpyTwoPort specificEnthalpy1(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{ -10, -30},{10, -10}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sine1(amplitude = 1600000.0, offset = 1800000.0) annotation(Placement(transformation(extent = {{ -80, -10},{ -60,10}}, rotation = 0)));

        equation 
          connect(boundary_prescriberho_1.ports[1],specificEnthalpy.port) annotation(Line(points = {{ -20,20},{0,20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(sine1.y,boundary_prescriberho_1.h_in) annotation(Line(points = {{ -59,0},{ -52,0},{ -52,24},{ -42,24}}, color = {0,0,127}, smooth = Smooth.None));
          connect(sine1.y,boundary_prescriberho_2.h_in) annotation(Line(points = {{ -59,0},{ -52,0},{ -52, -16},{ -42, -16}}, color = {0,0,127}, smooth = Smooth.None));
          connect(boundary_prescriberho_2.ports[1],specificEnthalpy1.port_a) annotation(Line(points = {{ -20, -20},{ -10, -20}}, color = {0,127,255}, smooth = Smooth.None));
        end TestSpecificEnthalpy;
        model TestPressure
          import Modelica.Fluid;
          Modelica.Fluid.Sensors.Pressure pressure1(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{ -20,0},{0,20}}, rotation = 0)));
          Modelica.Fluid.Fittings.SimpleGenericOrifice simpleGenericOrifice(redeclare package Medium = Modelica.Media.Water.StandardWater, zeta = 2, diameter = 0.1) annotation(Placement(transformation(extent = {{20, -10},{40,10}}, rotation = 0)));
          Modelica.Fluid.Sensors.RelativePressure relativePressure(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{20,34},{40,54}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sine annotation(Placement(transformation(extent = {{ -100,0},{ -80,20}}, rotation = 0)));
          Modelica.Fluid.Sources.MassFlowSource_T massFlowRate1(nPorts = 1, use_m_flow_in = true, T = SI.Conversions.from_degC(50), redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{ -60,0},{ -40,20}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(Tolerance = 1e-06), experimentSetupOutput);
          Modelica.Fluid.Sources.Boundary_ph boundary_fixed(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, p = system.p_ambient, h = 3000000.0) annotation(Placement(transformation(extent = {{100, -10},{80,10}}, rotation = 0)));
          Modelica.Fluid.Sensors.Pressure pressure2(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{50,0},{70,20}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100, -100},{ -80, -80}}, rotation = 0)));

        equation 
          connect(sine.y,massFlowRate1.m_flow_in) annotation(Line(points = {{ -79,10},{ -70,10},{ -70,18},{ -60,18}}, color = {0,0,127}, smooth = Smooth.None));
          connect(massFlowRate1.ports[1],pressure1.port) annotation(Line(points = {{ -40,10},{ -26,10},{ -26,0},{ -10,0}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pressure1.port,simpleGenericOrifice.port_a) annotation(Line(points = {{ -10,0},{20,0}}, color = {0,127,255}, smooth = Smooth.None));
          connect(simpleGenericOrifice.port_a,relativePressure.port_a) annotation(Line(points = {{20,0},{20,44},{20,44}}, color = {0,127,255}, smooth = Smooth.None));
          connect(relativePressure.port_b,simpleGenericOrifice.port_b) annotation(Line(points = {{40,44},{40,23},{40,23},{40,0}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pressure2.port,boundary_fixed.ports[1]) annotation(Line(points = {{60,0},{80,0}}, color = {0,127,255}, smooth = Smooth.None));
          connect(simpleGenericOrifice.port_b,pressure2.port) annotation(Line(points = {{40,0},{60,0}}, color = {0,127,255}, smooth = Smooth.None));
        end TestPressure;
        model TestOnePortSensors2
          import Modelica.Fluid;
          package Medium = Modelica.Media.Water.StandardWater;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 15, Algorithm = "Euler"), experimentSetupOutput);
          Modelica.Fluid.Vessels.ClosedVolume MixingVolume1(V = 0.001, use_T_start = false, redeclare package Medium = Medium, h_start = 100000.0, nPorts = 2, use_portsData = false) annotation(Placement(transformation(extent = {{ -34,30},{ -14,50}}, rotation = 0)));
          Modelica.Fluid.Sources.MassFlowSource_h FlowSource2(nPorts = 1, m_flow = 1, h = 200000.0, redeclare package Medium = Medium, use_m_flow_in = true) annotation(Placement(transformation(extent = {{ -68,30},{ -48,50}}, rotation = 0)));
          Modelica.Fluid.Vessels.ClosedVolume MixingVolume2(V = 0.001, use_T_start = false, redeclare package Medium = Medium, h_start = 150000.0, nPorts = 2, use_portsData = false) annotation(Placement(transformation(extent = {{36,30},{56,50}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_ph Sink2(nPorts = 1, redeclare package Medium = Medium, p = 101325, h = 50000.0) annotation(Placement(transformation(extent = {{100,30},{80,50}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100, -100},{ -80, -80}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp ramp(height = 2, offset =  -1, duration = 10) annotation(Placement(transformation(extent = {{ -100,30},{ -80,50}}, rotation = 0)));
          Modelica.Fluid.Vessels.ClosedVolume MixingVolume3(V = 0.001, use_T_start = false, redeclare package Medium = Medium, h_start = 100000.0, nPorts = 2, use_portsData = false) annotation(Placement(transformation(extent = {{ -34, -30},{ -14, -10}}, rotation = 0)));
          Modelica.Fluid.Sources.MassFlowSource_h FlowSource1(nPorts = 1, m_flow = 1, h = 200000.0, redeclare package Medium = Medium, use_m_flow_in = true) annotation(Placement(transformation(extent = {{ -68, -30},{ -48, -10}}, rotation = 0)));
          Modelica.Fluid.Vessels.ClosedVolume MixingVolume4(V = 0.001, use_T_start = false, redeclare package Medium = Medium, h_start = 150000.0, nPorts = 2, use_portsData = false) annotation(Placement(transformation(extent = {{36, -30},{56, -10}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_ph Sink1(nPorts = 1, redeclare package Medium = Medium, p = 101325, h = 50000.0) annotation(Placement(transformation(extent = {{100, -30},{80, -10}}, rotation = 0)));
          Modelica.Fluid.Sensors.TemperatureTwoPort Tmix2(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{0, -30},{20, -10}}, rotation = 0)));

        equation 
          connect(ramp.y,FlowSource2.m_flow_in) annotation(Line(points = {{ -79,40},{ -74,40},{ -74,48},{ -68,48}}, color = {0,0,127}, smooth = Smooth.None));
          connect(ramp.y,FlowSource1.m_flow_in) annotation(Line(points = {{ -79,40},{ -76,40},{ -76, -12},{ -68, -12}}, color = {0,0,127}, smooth = Smooth.None));
          connect(FlowSource2.ports[1],MixingVolume1.ports[1]) annotation(Line(points = {{ -48,40},{ -36,40},{ -36,30},{ -26,30}}, color = {0,127,255}, smooth = Smooth.None));
          connect(MixingVolume1.ports[2],MixingVolume2.ports[2]) annotation(Line(points = {{ -22,30},{ -5,30},{ -5,32},{12,32},{12,30},{48,30}}, color = {0,127,255}, smooth = Smooth.None));
          connect(MixingVolume2.ports[1],Sink2.ports[1]) annotation(Line(points = {{44,30},{61,30},{61,40},{80,40}}, color = {0,127,255}, smooth = Smooth.None));
          connect(FlowSource1.ports[1],MixingVolume3.ports[1]) annotation(Line(points = {{ -48, -20},{ -36, -20},{ -36, -30},{ -26, -30}}, color = {0,127,255}, smooth = Smooth.None));
          connect(MixingVolume3.ports[2],Tmix2.port_a) annotation(Line(points = {{ -22, -30},{ -12, -30},{ -12, -20},{0, -20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(Tmix2.port_b,MixingVolume4.ports[2]) annotation(Line(points = {{20, -20},{32, -20},{32, -30},{48, -30}}, color = {0,127,255}, smooth = Smooth.None));
          connect(MixingVolume4.ports[1],Sink1.ports[1]) annotation(Line(points = {{44, -30},{61, -30},{61, -20},{80, -20}}, color = {0,127,255}, smooth = Smooth.None));
        end TestOnePortSensors2;
        model TestOnePortSensors1
          import Modelica.Fluid;
          package Medium = Modelica.Media.Water.StandardWater;
          parameter Real diameter_a = 0.1;
          parameter Real diameter_b = 0.2;
          parameter Real A_rel = (diameter_a / diameter_b) ^ 2;
          parameter Real zeta = (1 - A_rel) ^ 2;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 25, Algorithm = "Dassl"), experimentSetupOutput);
          Modelica.Fluid.Vessels.ClosedVolume volume1(V = 0.001, use_T_start = false, redeclare package Medium = Medium, h_start = 100000.0, nPorts = 2, energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, use_portsData = false, p_start = 101325) annotation(Placement(transformation(extent = {{ -30,30},{ -10,50}}, rotation = 0)));
          Modelica.Fluid.Sources.MassFlowSource_h FlowSource2(nPorts = 1, m_flow = 1, h = 200000.0, redeclare package Medium = Medium, use_m_flow_in = true) annotation(Placement(transformation(extent = {{ -68,30},{ -48,50}}, rotation = 0)));
          Modelica.Fluid.Sensors.Temperature Tmix1(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{0,60},{20,80}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_ph sink1(nPorts = 1, redeclare package Medium = Medium, h = 50000.0, p = 101325) annotation(Placement(transformation(extent = {{100,30},{80,50}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100, -100},{ -80, -80}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp ramp(height = 2, duration = 20, offset =  -1) annotation(Placement(transformation(extent = {{ -100,30},{ -80,50}}, rotation = 0)));
          Modelica.Fluid.Vessels.ClosedVolume volume2(V = 0.001, use_T_start = false, redeclare package Medium = Medium, h_start = 100000.0, nPorts = 2, energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, use_portsData = false, p_start = 101325) annotation(Placement(transformation(extent = {{ -32, -30},{ -12, -10}}, rotation = 0)));
          Modelica.Fluid.Sources.MassFlowSource_h FlowSource1(nPorts = 1, m_flow = 1, h = 200000.0, redeclare package Medium = Medium, use_m_flow_in = true) annotation(Placement(transformation(extent = {{ -68, -30},{ -48, -10}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_ph sink2(nPorts = 1, redeclare package Medium = Medium, h = 50000.0, p = 101325) annotation(Placement(transformation(extent = {{100, -30},{80, -10}}, rotation = 0)));
          Modelica.Fluid.Sensors.TemperatureTwoPort Tmix2(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{0, -30},{20, -10}}, rotation = 0)));
          Modelica.Fluid.Fittings.SimpleGenericOrifice orifice1(redeclare package Medium = Medium, diameter = diameter_a, zeta = zeta) annotation(Placement(transformation(extent = {{40,30},{60,50}}, rotation = 0)));
          Modelica.Fluid.Fittings.SimpleGenericOrifice orifice2(redeclare package Medium = Medium, zeta = zeta, diameter = diameter_a) annotation(Placement(transformation(extent = {{40, -30},{60, -10}}, rotation = 0)));

        equation 
          connect(ramp.y,FlowSource2.m_flow_in) annotation(Line(points = {{ -79,40},{ -74,40},{ -74,48},{ -68,48}}, color = {0,0,127}, smooth = Smooth.None));
          connect(ramp.y,FlowSource1.m_flow_in) annotation(Line(points = {{ -79,40},{ -76,40},{ -76, -12},{ -68, -12}}, color = {0,0,127}, smooth = Smooth.None));
          connect(orifice1.port_b,sink1.ports[1]) annotation(Line(points = {{60,40},{80,40}}, color = {0,127,255}, smooth = Smooth.None));
          connect(orifice2.port_a,Tmix2.port_b) annotation(Line(points = {{40, -20},{20, -20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(orifice2.port_b,sink2.ports[1]) annotation(Line(points = {{60, -20},{80, -20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(Tmix1.port,orifice1.port_a) annotation(Line(points = {{10,60},{10,40},{40,40}}, color = {0,127,255}, smooth = Smooth.None));
          connect(FlowSource2.ports[1],volume1.ports[1]) annotation(Line(points = {{ -48,40},{ -34,40},{ -34,30},{ -22,30}}, color = {0,127,255}, smooth = Smooth.None));
          connect(volume1.ports[2],orifice1.port_a) annotation(Line(points = {{ -18,30},{10,30},{10,40},{40,40}}, color = {0,127,255}, smooth = Smooth.None));
          connect(FlowSource1.ports[1],volume2.ports[1]) annotation(Line(points = {{ -48, -20},{ -35, -20},{ -35, -30},{ -24, -30}}, color = {0,127,255}, smooth = Smooth.None));
          connect(volume2.ports[2],Tmix2.port_a) annotation(Line(points = {{ -20, -30},{ -11, -30},{ -11, -20},{0, -20}}, color = {0,127,255}, smooth = Smooth.None));
        end TestOnePortSensors1;
        model TestFlowRate
          import Modelica.Fluid;
          Modelica.Fluid.Fittings.SimpleGenericOrifice simpleGenericOrifice(redeclare package Medium = Modelica.Media.Water.StandardWater, zeta = 2, diameter = 0.1) annotation(Placement(transformation(extent = {{ -20, -10},{0,10}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sine annotation(Placement(transformation(extent = {{ -100,0},{ -80,20}}, rotation = 0)));
          Modelica.Fluid.Sources.MassFlowSource_T massFlowRate1(nPorts = 1, use_m_flow_in = true, T = SI.Conversions.from_degC(50), redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{ -60,0},{ -40,20}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(Tolerance = 1e-06), experimentSetupOutput);
          Modelica.Fluid.Sources.Boundary_ph boundary_fixed(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, p = system.p_ambient, h = 3000000.0) annotation(Placement(transformation(extent = {{100, -10},{80,10}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100, -100},{ -80, -80}}, rotation = 0)));
          Modelica.Fluid.Sensors.MassFlowRate massFlowRate(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{10, -10},{30,10}}, rotation = 0)));
          Modelica.Fluid.Sensors.VolumeFlowRate volumeFlowRate(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{50, -10},{70,10}}, rotation = 0)));

        equation 
          connect(sine.y,massFlowRate1.m_flow_in) annotation(Line(points = {{ -79,10},{ -70,10},{ -70,18},{ -60,18}}, color = {0,0,127}, smooth = Smooth.None));
          connect(massFlowRate1.ports[1],simpleGenericOrifice.port_a) annotation(Line(points = {{ -40,10},{ -32,10},{ -32,0},{ -20,0}}, color = {0,127,255}, smooth = Smooth.None));
          connect(volumeFlowRate.port_b,boundary_fixed.ports[1]) annotation(Line(points = {{70,0},{80,0}}, color = {0,127,255}, smooth = Smooth.None));
          connect(massFlowRate.port_b,volumeFlowRate.port_a) annotation(Line(points = {{30,0},{50,0}}, color = {0,127,255}, smooth = Smooth.None));
          connect(simpleGenericOrifice.port_b,massFlowRate.port_a) annotation(Line(points = {{0,0},{10,0}}, color = {0,127,255}, smooth = Smooth.None));
        end TestFlowRate;
        model TestDensity
          import Modelica.Fluid;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(Tolerance = 1e-06), experimentSetupOutput);
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100, -100},{ -80, -80}}, rotation = 0)));
          Modelica.Fluid.Sensors.DensityTwoPort density2_1(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{ -20, -30},{0, -10}}, rotation = 0)));
          Modelica.Fluid.Fittings.SimpleGenericOrifice simpleGenericOrifice1(redeclare package Medium = Modelica.Media.Water.StandardWater, zeta = 2, diameter = 0.1) annotation(Placement(transformation(extent = {{20, -30},{40, -10}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sine1 annotation(Placement(transformation(extent = {{ -100, -20},{ -80,0}}, rotation = 0)));
          Modelica.Fluid.Sources.MassFlowSource_h massFlowRate2(nPorts = 1, use_m_flow_in = true, redeclare package Medium = Modelica.Media.Water.StandardWater, h = 3200000.0) annotation(Placement(transformation(extent = {{ -60, -20},{ -40,0}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_ph boundary_fixed1(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, p = system.p_ambient, h = 3000000.0) annotation(Placement(transformation(extent = {{100, -30},{80, -10}}, rotation = 0)));
          Modelica.Fluid.Sensors.DensityTwoPort density2_2(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{50, -30},{70, -10}}, rotation = 0)));
          Modelica.Fluid.Sensors.Density density1_1(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{ -20,50},{0,70}}, rotation = 0)));
          Modelica.Fluid.Fittings.SimpleGenericOrifice simpleGenericOrifice(redeclare package Medium = Modelica.Media.Water.StandardWater, zeta = 2, diameter = 0.1) annotation(Placement(transformation(extent = {{20,40},{40,60}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sine annotation(Placement(transformation(extent = {{ -100,50},{ -80,70}}, rotation = 0)));
          Modelica.Fluid.Sources.MassFlowSource_h massFlowRate1(nPorts = 1, use_m_flow_in = true, redeclare package Medium = Modelica.Media.Water.StandardWater, h = 3200000.0) annotation(Placement(transformation(extent = {{ -60,50},{ -40,70}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_ph boundary_fixed(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, p = system.p_ambient, h = 3000000.0) annotation(Placement(transformation(extent = {{100,40},{80,60}}, rotation = 0)));
          Modelica.Fluid.Sensors.Density density1_2(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{50,50},{70,70}}, rotation = 0)));

        equation 
          connect(sine1.y,massFlowRate2.m_flow_in) annotation(Line(points = {{ -79, -10},{ -70, -10},{ -70, -2},{ -60, -2}}, color = {0,0,127}, smooth = Smooth.None));
          connect(massFlowRate2.ports[1],density2_1.port_a) annotation(Line(points = {{ -40, -10},{ -30, -10},{ -30, -20},{ -20, -20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(density2_1.port_b,simpleGenericOrifice1.port_a) annotation(Line(points = {{0, -20},{20, -20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(simpleGenericOrifice1.port_b,density2_2.port_a) annotation(Line(points = {{40, -20},{50, -20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(density2_2.port_b,boundary_fixed1.ports[1]) annotation(Line(points = {{70, -20},{80, -20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(sine.y,massFlowRate1.m_flow_in) annotation(Line(points = {{ -79,60},{ -70,60},{ -70,68},{ -60,68}}, color = {0,0,127}, smooth = Smooth.None));
          connect(massFlowRate1.ports[1],density1_1.port) annotation(Line(points = {{ -40,60},{ -26,60},{ -26,50},{ -10,50}}, color = {0,127,255}, smooth = Smooth.None));
          connect(density1_1.port,simpleGenericOrifice.port_a) annotation(Line(points = {{ -10,50},{20,50}}, color = {0,127,255}, smooth = Smooth.None));
          connect(density1_2.port,boundary_fixed.ports[1]) annotation(Line(points = {{60,50},{80,50}}, color = {0,127,255}, smooth = Smooth.None));
          connect(simpleGenericOrifice.port_b,density1_2.port) annotation(Line(points = {{40,50},{60,50}}, color = {0,127,255}, smooth = Smooth.None));
        end TestDensity;
      end Sensors;
      package Pipes "Test Pipe models"
        model StaticPipe
          import Modelica.Fluid;
          extends Modelica.Icons.Example;
          replaceable package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater;
          Modelica.Fluid.Pipes.StaticPipe pipe1(redeclare package Medium = Medium, length = 10, diameter = 0.0254, p_a_start = 500000, p_b_start = 100000) annotation(Placement(transformation(extent = {{ -10,0},{10,20}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 10, NumberOfIntervals = 10000), Documentation(info = "<html>
Test of the StaticPipe model.
</html>"));
          Modelica.Fluid.Sources.Boundary_pT sink(nPorts = 1, redeclare package Medium = Medium, p = 200000, T = 300) annotation(Placement(transformation(extent = {{56,4},{44,16}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100,60},{ -80,80}}, rotation = 0)));
          Modelica.Blocks.Sources.TimeTable p_table(table = [0,100000.0;10,300000.0]) annotation(Placement(transformation(extent = {{ -90,20},{ -70,40}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT source(redeclare package Medium = Medium, p = system.p_ambient, T = system.T_ambient, use_p_in = true, nPorts = 1) annotation(Placement(transformation(extent = {{ -56,4},{ -44,16}}, rotation = 0)));

        equation 
          connect(pipe1.port_b,sink.ports[1]) annotation(Line(points = {{10,10},{44,10}}, color = {0,127,255}, smooth = Smooth.None));
          connect(source.ports[1],pipe1.port_a) annotation(Line(points = {{ -44,10},{ -10,10}}, color = {0,127,255}, smooth = Smooth.None));
          connect(p_table.y,source.p_in) annotation(Line(points = {{ -69,30},{ -62,30},{ -62,14.8},{ -57.2,14.8}}, color = {0,0,127}, smooth = Smooth.None));
        end StaticPipe;
        model DynamicPipeWithNominalLaminarFlow "Test NominalLaminarFlow model subject to varying nNodes and nParallel"
          extends Modelica.Icons.Example;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          Modelica.Fluid.Pipes.DynamicPipe pipeN10(nParallel = 1, length = 100, redeclare package Medium = Modelica.Media.Water.StandardWater, use_T_start = false, h_start = 3000000.0, m_flow_start = 1, diameter = 0.3, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.NominalLaminarFlow(dp_nominal = 100000, m_flow_nominal = 1), nNodes = 10, p_a_start = 200000, p_b_start = 100000) annotation(Placement(transformation(extent = {{ -10,30},{10,50}})));
          inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.SteadyStateInitial, massDynamics = Modelica.Fluid.Types.Dynamics.SteadyStateInitial, momentumDynamics = Modelica.Fluid.Types.Dynamics.SteadyStateInitial) annotation(Placement(transformation(extent = {{ -100,80},{ -80,100}})));
          Modelica.Fluid.Sources.MassFlowSource_h boundary(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, m_flow = 1, h = 3000000.0) annotation(Placement(transformation(extent = {{ -70,30},{ -50,50}})));
          Modelica.Fluid.Sources.FixedBoundary boundary1(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, p = 100000) annotation(Placement(transformation(extent = {{70,30},{50,50}})));
          Modelica.Fluid.Pipes.DynamicPipe pipeN20(nParallel = 1, length = 100, redeclare package Medium = Modelica.Media.Water.StandardWater, use_T_start = false, h_start = 3000000.0, m_flow_start = 1, diameter = 0.3, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.NominalLaminarFlow(dp_nominal = 100000, m_flow_nominal = 1), p_a_start = 200000, p_b_start = 100000, nNodes = 20) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}})));
          Modelica.Fluid.Sources.MassFlowSource_h boundary2(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, m_flow = 1, h = 3000000.0) annotation(Placement(transformation(extent = {{ -70, -10},{ -50,10}})));
          Modelica.Fluid.Sources.FixedBoundary boundary3(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, p = 100000) annotation(Placement(transformation(extent = {{70, -10},{50,10}})));
          Modelica.Fluid.Pipes.DynamicPipe pipeN10nParallel10(length = 100, redeclare package Medium = Modelica.Media.Water.StandardWater, use_T_start = false, h_start = 3000000.0, m_flow_start = 1, diameter = 0.3, nNodes = 10, nParallel = 10, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.NominalLaminarFlow(dp_nominal = 100000, m_flow_nominal = 10), p_a_start = 200000, p_b_start = 100000) annotation(Placement(transformation(extent = {{ -10, -50},{10, -30}})));
          Modelica.Fluid.Sources.MassFlowSource_h boundary4(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, h = 3000000.0, m_flow = 10) annotation(Placement(transformation(extent = {{ -70, -50},{ -50, -30}})));
          Modelica.Fluid.Sources.FixedBoundary boundary5(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, p = 100000) annotation(Placement(transformation(extent = {{70, -50},{50, -30}})));

        equation 
          assert(abs(pipeN10.port_a.p - pipeN20.port_a.p) < 1, "!!!Incorrect pressure for modified nNodes!!!");
          assert(abs(pipeN10.port_a.p - pipeN10nParallel10.port_a.p) < 1, "!!!Incorrect pressure for nParallel>1!!!");
          connect(boundary.ports[1],pipeN10.port_a) annotation(Line(points = {{ -50,40},{ -10,40}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipeN10.port_b,boundary1.ports[1]) annotation(Line(points = {{10,40},{50,40}}, color = {0,127,255}, smooth = Smooth.None));
          connect(boundary2.ports[1],pipeN20.port_a) annotation(Line(points = {{ -50,0},{ -10,0}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipeN20.port_b,boundary3.ports[1]) annotation(Line(points = {{10,0},{50,0}}, color = {0,127,255}, smooth = Smooth.None));
          connect(boundary4.ports[1],pipeN10nParallel10.port_a) annotation(Line(points = {{ -50, -40},{ -10, -40}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipeN10nParallel10.port_b,boundary5.ports[1]) annotation(Line(points = {{10, -40},{50, -40}}, color = {0,127,255}, smooth = Smooth.None));
        end DynamicPipeWithNominalLaminarFlow;
        model DynamicPipesWithTraceSubstances
          import Modelica.Fluid;
          extends Modelica.Icons.Example;
          replaceable package Medium = Modelica.Media.Air.DryAirNasa(extraPropertiesNames = {"CO2","VOC"});
          Modelica.Fluid.Pipes.DynamicPipe pipe2(redeclare package Medium = Medium, use_T_start = true, diameter = 0.01, nNodes = 5, m_flow_start = 0.1, length = 2, modelStructure = Modelica.Fluid.Types.ModelStructure.a_v_b, use_HeatTransfer = true, redeclare model HeatTransfer = Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.ConstantFlowHeatTransfer(alpha0 = 500), p_a_start = 100000, p_b_start = 100000, T_start = 280) annotation(Placement(transformation(extent = {{ -30,68},{ -10,88}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 20, Tolerance = 1e-05), experimentSetupOutput, Documentation(info = "<html>
Test of different dynamic pipe models with trace substances.
</html>"));
          Modelica.Fluid.Sources.Boundary_pT boundary2(nPorts = 1, redeclare package Medium = Medium, use_p_in = true, use_T_in = false, p = 100000, T = 300, C = {0.0015,5e-05}) annotation(Placement(transformation(extent = {{70,54},{50,74}}, rotation = 0)));
          Modelica.Fluid.Pipes.DynamicPipe pipe3(redeclare package Medium = Medium, length = 1, use_T_start = true, diameter = 0.01, nNodes = 5, m_flow_start = 0.1, p_b_start = 100000, T_start = 340, modelStructure = Modelica.Fluid.Types.ModelStructure.a_v_b, p_a_start = 100000) annotation(Placement(transformation(extent = {{20,54},{40,74}}, rotation = 0)));
          Modelica.Fluid.Pipes.DynamicPipe pipe1(redeclare package Medium = Medium, use_T_start = true, T_start = 300, diameter = 0.01, nNodes = 5, m_flow_start = 0.1, p_b_start = 100000.0, length = 0.4, modelStructure = Modelica.Fluid.Types.ModelStructure.a_v_b, p_a_start = 100000) annotation(Placement(transformation(extent = {{ -82,54},{ -62,74}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT boundary1(nPorts = 1, T = 280, redeclare package Medium = Medium, p = 150000.0) annotation(Placement(transformation(extent = {{ -108,54},{ -88,74}}, rotation = 0)));
          annotation(extent = [ -90, -86; -70, -66]);
          annotation(points = [ -22,42; -2,42; -2,32;7.8,32], style(color = 69, rgbcolor = {0,127,255}));
          Modelica.Blocks.Sources.Ramp ramp(offset = 100000.0, startTime = 5, duration = 0, height = 100000.0) annotation(Placement(transformation(extent = {{90,64},{78,76}}, rotation = 0)));
          Modelica.Fluid.Pipes.DynamicPipe pipe4(redeclare package Medium = Medium, length = 1, use_T_start = true, diameter = 0.01, nNodes = 5, m_flow_start = 0.1, p_b_start = 100000, T_start = 360, modelStructure = Modelica.Fluid.Types.ModelStructure.a_v_b, p_a_start = 100000) annotation(Placement(transformation(extent = {{ -30,38},{ -10,58}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{72, -94},{92, -74}}, rotation = 0)));
          Modelica.Fluid.Fittings.TeeJunctionVolume junction1(redeclare package Medium = Medium, V = 0.0001, p_start = 100000, T_start = 300) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 90, origin = { -42,64})));
          Modelica.Fluid.Fittings.TeeJunctionVolume junction2(redeclare package Medium = Medium, V = 1e-05, p_start = 100000, T_start = 300) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation =  -90, origin = {2,64})));
          Modelica.Thermal.HeatTransfer.Sources.FixedHeatFlow[pipe2.n] heat(each Q_flow = 200, each alpha = 10000, each T_ref = 350) annotation(Placement(transformation(extent = {{ -54,80},{ -34,100}}, rotation = 0)));
          Modelica.Fluid.Pipes.DynamicPipe pipe5(redeclare package Medium = Medium, use_T_start = true, diameter = 0.01, nNodes = 5, m_flow_start = 0.1, length = 2, use_HeatTransfer = true, p_a_start = 100000, p_b_start = 100000, T_start = 280, redeclare model HeatTransfer = Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.ConstantFlowHeatTransfer(alpha0 = 500)) annotation(Placement(transformation(extent = {{ -30,10},{ -10,30}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT boundary4(nPorts = 1, redeclare package Medium = Medium, use_p_in = true, use_T_in = false, p = 100000, T = 300, C = {0.0015,5e-05}) annotation(Placement(transformation(extent = {{68, -2},{48,18}}, rotation = 0)));
          Modelica.Fluid.Pipes.DynamicPipe pipe6(redeclare package Medium = Medium, length = 1, use_T_start = true, diameter = 0.01, nNodes = 5, m_flow_start = 0.1, p_b_start = 100000, T_start = 340, modelStructure = Modelica.Fluid.Types.ModelStructure.av_b, p_a_start = 100000) annotation(Placement(transformation(extent = {{14, -2},{34,18}}, rotation = 0)));
          Modelica.Fluid.Pipes.DynamicPipe pipe7(redeclare package Medium = Medium, use_T_start = true, T_start = 300, diameter = 0.01, nNodes = 5, m_flow_start = 0.1, p_b_start = 100000.0, length = 0.4, p_a_start = 100000) annotation(Placement(transformation(extent = {{ -74, -2},{ -54,18}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT boundary3(nPorts = 1, redeclare package Medium = Medium, p = 150000, T = 280) annotation(Placement(transformation(extent = {{ -108, -2},{ -88,18}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp ramp1(offset = 100000.0, startTime = 5, duration = 0, height = 100000.0) annotation(Placement(transformation(extent = {{96,8},{84,20}}, rotation = 0)));
          Modelica.Fluid.Pipes.DynamicPipe pipe8(redeclare package Medium = Medium, length = 1, use_T_start = true, diameter = 0.01, T_start = 360, nNodes = 5, m_flow_start = 0.1, p_b_start = 100000.0, p_a_start = 100000) annotation(Placement(transformation(extent = {{ -30, -12},{ -10,8}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Sources.FixedHeatFlow[pipe2.n] heat1(each Q_flow = 200, each alpha = 10000, each T_ref = 350) annotation(Placement(transformation(extent = {{ -72,22},{ -52,42}}, rotation = 0)));
          Modelica.Fluid.Sensors.TraceSubstances traceSubstance3(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -98,22},{ -78,42}})));
          Modelica.Fluid.Sensors.TraceSubstances traceSubstance1(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -90,80},{ -70,100}})));
          Modelica.Fluid.Sensors.TraceSubstances traceSubstance2(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{32,80},{52,100}})));
          Modelica.Fluid.Sensors.TraceSubstances traceSubstance4(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{2,22},{22,42}})));
          Modelica.Fluid.Fittings.MultiPort multiPort(redeclare package Medium = Medium, nPorts_b = 2) annotation(Placement(transformation(extent = {{ -46, -2},{ -38,18}})));

        equation 
          connect(boundary1.ports[1],pipe1.port_a) annotation(Line(points = {{ -88,64},{ -82,64}}, color = {0,127,255}, thickness = 0.5));
          connect(pipe3.port_b,boundary2.ports[1]) annotation(Line(points = {{40,64},{50,64}}, color = {0,127,255}, thickness = 0.5));
          connect(heat.port,pipe2.heatPorts) annotation(Line(points = {{ -34,90},{ -19.9,90},{ -19.9,83.2}}, color = {191,0,0}, thickness = 0.5));
          connect(ramp1.y,boundary4.p_in) annotation(Line(points = {{83.4,14},{76.7,14},{76.7,16},{70,16}}, color = {0,0,127}, thickness = 0.5));
          connect(boundary3.ports[1],pipe7.port_a) annotation(Line(points = {{ -88,8},{ -74,8}}, color = {0,127,255}, thickness = 0.5));
          connect(pipe5.port_b,pipe6.port_a) annotation(Line(points = {{ -10,20},{2,20},{2,8},{14,8}}, color = {0,127,255}, thickness = 0.5));
          connect(pipe8.port_b,pipe6.port_a) annotation(Line(points = {{ -10, -2},{2, -2},{2,8},{14,8}}, color = {0,127,255}, thickness = 0.5));
          connect(pipe6.port_b,boundary4.ports[1]) annotation(Line(points = {{34,8},{48,8}}, color = {0,127,255}, thickness = 0.5));
          connect(heat1.port,pipe5.heatPorts) annotation(Line(points = {{ -52,32},{ -19.9,32},{ -19.9,25.2}}, color = {191,0,0}, thickness = 0.5));
          connect(boundary2.p_in,ramp.y) annotation(Line(points = {{72,72},{74.7,72},{74.7,70},{77.4,70}}, color = {0,0,127}, thickness = 0.5));
          connect(junction2.port_3,pipe3.port_a) annotation(Line(points = {{12,64},{20,64}}, color = {0,127,255}, thickness = 0.5, smooth = Smooth.None));
          connect(pipe2.port_b,junction2.port_1) annotation(Line(points = {{ -10,78},{2,78},{2,74}}, color = {0,127,255}, thickness = 0.5, smooth = Smooth.None));
          connect(pipe4.port_b,junction2.port_2) annotation(Line(points = {{ -10,48},{2,48},{2,54}}, color = {0,127,255}, thickness = 0.5, smooth = Smooth.None));
          connect(pipe1.port_b,junction1.port_3) annotation(Line(points = {{ -62,64},{ -52,64}}, color = {0,127,255}, thickness = 0.5, smooth = Smooth.None));
          connect(junction1.port_1,pipe4.port_a) annotation(Line(points = {{ -42,54},{ -42,48},{ -30,48}}, color = {0,127,255}, thickness = 0.5, smooth = Smooth.None));
          connect(junction1.port_2,pipe2.port_a) annotation(Line(points = {{ -42,74},{ -42,78},{ -30,78}}, color = {0,127,255}, thickness = 0.5, smooth = Smooth.None));
          connect(traceSubstance1.port,pipe1.port_b) annotation(Line(points = {{ -80,80},{ -60,80},{ -60,64},{ -62,64}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipe7.port_a,traceSubstance3.port) annotation(Line(points = {{ -74,8},{ -80,8},{ -80,22},{ -88,22}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipe3.port_b,traceSubstance2.port) annotation(Line(points = {{40,64},{42,64},{42,80}}, color = {0,127,255}, smooth = Smooth.None));
          connect(traceSubstance4.port,pipe6.port_a) annotation(Line(points = {{12,22},{12,8},{14,8}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipe7.port_b,multiPort.port_a) annotation(Line(points = {{ -54,8},{ -46,8}}, color = {0,127,255}, smooth = Smooth.None));
          connect(multiPort.ports_b[1],pipe5.port_a) annotation(Line(points = {{ -38,10},{ -32,10},{ -32,20},{ -30,20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(multiPort.ports_b[2],pipe8.port_a) annotation(Line(points = {{ -38,6},{ -38,8},{ -32,8},{ -32, -2},{ -30, -2}}, color = {0,127,255}, smooth = Smooth.None));
        end DynamicPipesWithTraceSubstances;
        model DynamicPipesAndFittings
          import Modelica.Fluid;
          extends Modelica.Icons.Example;
          replaceable package Medium = Modelica.Media.Water.StandardWater;
          Modelica.Fluid.Pipes.DynamicPipe pipe2(redeclare package Medium = Medium, use_T_start = true, diameter = 0.01, nNodes = 5, m_flow_start = 0.1, length = 2, redeclare model HeatTransfer = Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.ConstantFlowHeatTransfer(alpha0 = 500), modelStructure = Modelica.Fluid.Types.ModelStructure.a_v_b, use_HeatTransfer = true, p_a_start = 100000, p_b_start = 100000, T_start = 280) annotation(Placement(transformation(extent = {{ -30,68},{ -10,88}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 20, Tolerance = 1e-05), experimentSetupOutput, Documentation(info = "<html>
Test of different dynamic pipe models. The first system uses explicit junctions, in the third system some of the pipe models are replaced by non-symmetric components.
</html>"));
          Modelica.Fluid.Sources.Boundary_pT boundary2(nPorts = 1, redeclare package Medium = Medium, p = 100000.0, T = 300, use_p_in = true, use_T_in = false) annotation(Placement(transformation(extent = {{70,54},{50,74}}, rotation = 0)));
          Modelica.Fluid.Pipes.DynamicPipe pipe3(redeclare package Medium = Medium, length = 1, use_T_start = true, diameter = 0.01, nNodes = 5, m_flow_start = 0.1, p_b_start = 100000, T_start = 340, modelStructure = Modelica.Fluid.Types.ModelStructure.a_v_b, p_a_start = 100000) annotation(Placement(transformation(extent = {{20,54},{40,74}}, rotation = 0)));
          Modelica.Fluid.Pipes.DynamicPipe pipe1(redeclare package Medium = Medium, use_T_start = true, T_start = 300, diameter = 0.01, nNodes = 5, m_flow_start = 0.1, p_b_start = 100000.0, length = 0.4, modelStructure = Modelica.Fluid.Types.ModelStructure.a_v_b, p_a_start = 100000) annotation(Placement(transformation(extent = {{ -82,54},{ -62,74}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT boundary1(nPorts = 1, T = 280, redeclare package Medium = Medium, p = 150000.0) annotation(Placement(transformation(extent = {{ -108,54},{ -88,74}}, rotation = 0)));
          annotation(extent = [ -90, -86; -70, -66]);
          annotation(points = [ -22,42; -2,42; -2,32;7.8,32], style(color = 69, rgbcolor = {0,127,255}));
          Modelica.Blocks.Sources.Ramp ramp(offset = 100000.0, startTime = 5, duration = 0, height = 100000.0) annotation(Placement(transformation(extent = {{90,64},{78,76}}, rotation = 0)));
          Modelica.Fluid.Pipes.DynamicPipe pipe4(redeclare package Medium = Medium, length = 1, use_T_start = true, diameter = 0.01, nNodes = 5, m_flow_start = 0.1, p_b_start = 100000, T_start = 360, modelStructure = Modelica.Fluid.Types.ModelStructure.a_v_b, p_a_start = 100000) annotation(Placement(transformation(extent = {{ -30,38},{ -10,58}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{72, -94},{92, -74}}, rotation = 0)));
          Modelica.Fluid.Fittings.TeeJunctionVolume junction1(redeclare package Medium = Medium, V = 0.0001, p_start = 100000, T_start = 300) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 90, origin = { -42,64})));
          Modelica.Fluid.Fittings.TeeJunctionVolume junction2(redeclare package Medium = Medium, V = 1e-05, p_start = 100000, T_start = 300) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation =  -90, origin = {2,64})));
          Modelica.Thermal.HeatTransfer.Sources.FixedHeatFlow[pipe2.n] heat(each Q_flow = 200, each alpha = 10000, each T_ref = 350) annotation(Placement(transformation(extent = {{ -54,80},{ -34,100}}, rotation = 0)));
          Modelica.Fluid.Pipes.DynamicPipe pipe5(redeclare package Medium = Medium, use_T_start = true, diameter = 0.01, nNodes = 5, m_flow_start = 0.1, length = 2, redeclare model HeatTransfer = Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.LocalPipeFlowHeatTransfer, use_HeatTransfer = true, p_a_start = 100000, p_b_start = 100000, T_start = 280) annotation(Placement(transformation(extent = {{ -30,10},{ -10,30}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT boundary4(nPorts = 1, redeclare package Medium = Medium, p = 100000.0, T = 300, use_p_in = true, use_T_in = false) annotation(Placement(transformation(extent = {{68, -2},{48,18}}, rotation = 0)));
          Modelica.Fluid.Pipes.DynamicPipe pipe6(redeclare package Medium = Medium, length = 1, use_T_start = true, diameter = 0.01, nNodes = 5, m_flow_start = 0.1, p_b_start = 100000, T_start = 340, modelStructure = Modelica.Fluid.Types.ModelStructure.av_b, p_a_start = 100000) annotation(Placement(transformation(extent = {{14, -2},{34,18}}, rotation = 0)));
          Modelica.Fluid.Pipes.DynamicPipe pipe7(redeclare package Medium = Medium, use_T_start = true, T_start = 300, diameter = 0.01, nNodes = 5, m_flow_start = 0.1, p_b_start = 100000.0, length = 0.4, p_a_start = 100000) annotation(Placement(transformation(extent = {{ -74, -2},{ -54,18}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT boundary3(nPorts = 1, redeclare package Medium = Medium, p = 150000, T = 280) annotation(Placement(transformation(extent = {{ -108, -2},{ -88,18}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp ramp1(offset = 100000.0, startTime = 5, duration = 0, height = 100000.0) annotation(Placement(transformation(extent = {{96,8},{84,20}}, rotation = 0)));
          Modelica.Fluid.Pipes.DynamicPipe pipe8(redeclare package Medium = Medium, length = 1, use_T_start = true, diameter = 0.01, T_start = 360, nNodes = 5, m_flow_start = 0.1, p_b_start = 100000.0, p_a_start = 100000) annotation(Placement(transformation(extent = {{ -30, -12},{ -10,8}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Sources.FixedHeatFlow[pipe2.n] heat1(each Q_flow = 200, each alpha = 10000, each T_ref = 350) annotation(Placement(transformation(extent = {{ -72,22},{ -52,42}}, rotation = 0)));
          Modelica.Fluid.Pipes.DynamicPipe pipe9(redeclare package Medium = Medium, use_T_start = true, diameter = 0.01, nNodes = 5, m_flow_start = 0.1, length = 2, redeclare model HeatTransfer = Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.ConstantFlowHeatTransfer(alpha0 = 500), use_HeatTransfer = true, p_a_start = 100000, p_b_start = 100000, T_start = 280) annotation(Placement(transformation(extent = {{ -29, -50},{ -9, -30}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT boundary5(nPorts = 1, redeclare package Medium = Medium, p = 100000.0, T = 300, use_p_in = true, use_T_in = false) annotation(Placement(transformation(extent = {{70, -62},{50, -42}}, rotation = 0)));
          Modelica.Fluid.Pipes.DynamicPipe pipe10(redeclare package Medium = Medium, length = 1, use_T_start = true, diameter = 0.01, nNodes = 5, modelStructure = Modelica.Fluid.Types.ModelStructure.av_b, m_flow_start = 0.1, p_b_start = 100000, T_start = 340, p_a_start = 100000) annotation(Placement(transformation(extent = {{14, -62},{34, -42}}, rotation = 0)));
          Modelica.Fluid.Pipes.DynamicPipe pipe11(redeclare package Medium = Medium, use_T_start = true, diameter = 0.01, nNodes = 5, modelStructure = Modelica.Fluid.Types.ModelStructure.a_vb, m_flow_start = 0.1, length = 0.4, p_b_start = 100000, T_start = 300, p_a_start = 100000) annotation(Placement(transformation(extent = {{ -74, -62},{ -54, -42}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT boundary6(nPorts = 1, redeclare package Medium = Medium, p = 150000, T = 280) annotation(Placement(transformation(extent = {{ -110, -62},{ -90, -42}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp ramp2(offset = 100000.0, startTime = 5, duration = 0, height = 100000.0) annotation(Placement(transformation(extent = {{96, -52},{84, -40}}, rotation = 0)));
          Modelica.Fluid.Pipes.DynamicPipe pipe12(redeclare package Medium = Medium, length = 1, use_T_start = true, diameter = 0.01, nNodes = 5, m_flow_start = 0.1, p_b_start = 100000, T_start = 360, p_a_start = 100000) annotation(Placement(transformation(extent = {{ -29, -72},{ -9, -52}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Sources.FixedHeatFlow[pipe2.n] heat2(each Q_flow = 200, each alpha = 10000, each T_ref = 350) annotation(Placement(transformation(extent = {{ -72, -38},{ -52, -18}}, rotation = 0)));
          Modelica.Fluid.Fittings.MultiPort muliPort11(nPorts_b = 2, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -50, -62},{ -42, -42}})));
          Modelica.Fluid.Fittings.MultiPort multiPort10(nPorts_b = 2, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{10, -62},{2, -42}})));

        equation 
          connect(boundary1.ports[1],pipe1.port_a) annotation(Line(points = {{ -88,64},{ -82,64}}, color = {0,127,255}, thickness = 0.5));
          connect(pipe3.port_b,boundary2.ports[1]) annotation(Line(points = {{40,64},{50,64}}, color = {0,127,255}, thickness = 0.5));
          connect(heat.port,pipe2.heatPorts) annotation(Line(points = {{ -34,90},{ -19.9,90},{ -19.9,83.2}}, color = {191,0,0}, thickness = 0.5));
          connect(ramp1.y,boundary4.p_in) annotation(Line(points = {{83.4,14},{76.7,14},{76.7,16},{70,16}}, color = {0,0,127}, thickness = 0.5));
          connect(boundary3.ports[1],pipe7.port_a) annotation(Line(points = {{ -88,8},{ -74,8}}, color = {0,127,255}, thickness = 0.5));
          connect(pipe7.port_b,pipe5.port_a) annotation(Line(points = {{ -54,8},{ -42,8},{ -42,20},{ -30,20}}, color = {0,127,255}, thickness = 0.5));
          connect(pipe7.port_b,pipe8.port_a) annotation(Line(points = {{ -54,8},{ -42,8},{ -42, -2},{ -30, -2}}, color = {0,127,255}, thickness = 0.5));
          connect(pipe5.port_b,pipe6.port_a) annotation(Line(points = {{ -10,20},{2,20},{2,8},{14,8}}, color = {0,127,255}, thickness = 0.5));
          connect(pipe8.port_b,pipe6.port_a) annotation(Line(points = {{ -10, -2},{2, -2},{2,8},{14,8}}, color = {0,127,255}, thickness = 0.5));
          connect(pipe6.port_b,boundary4.ports[1]) annotation(Line(points = {{34,8},{48,8}}, color = {0,127,255}, thickness = 0.5));
          connect(heat1.port,pipe5.heatPorts) annotation(Line(points = {{ -52,32},{ -19.9,32},{ -19.9,25.2}}, color = {191,0,0}, thickness = 0.5));
          connect(boundary2.p_in,ramp.y) annotation(Line(points = {{72,72},{74.7,72},{74.7,70},{77.4,70}}, color = {0,0,127}, thickness = 0.5));
          connect(ramp2.y,boundary5.p_in) annotation(Line(points = {{83.4, -46},{77.7, -46},{77.7, -44},{72, -44}}, color = {0,0,127}, thickness = 0.5));
          connect(boundary6.ports[1],pipe11.port_a) annotation(Line(points = {{ -90, -52},{ -74, -52}}, color = {0,127,255}, thickness = 0.5));
          connect(pipe10.port_b,boundary5.ports[1]) annotation(Line(points = {{34, -52},{50, -52}}, color = {0,127,255}, thickness = 0.5));
          connect(heat2.port,pipe9.heatPorts) annotation(Line(points = {{ -52, -28},{ -18.9, -28},{ -18.9, -34.8}}, color = {191,0,0}, thickness = 0.5));
          connect(pipe11.port_b,muliPort11.port_a) annotation(Line(points = {{ -54, -52},{ -50, -52}}, color = {0,127,255}, thickness = 0.5, smooth = Smooth.None));
          connect(muliPort11.ports_b[2],pipe12.port_a) annotation(Line(points = {{ -42, -54},{ -40, -54},{ -40, -62},{ -29, -62}}, color = {0,127,255}, thickness = 0.5, smooth = Smooth.None));
          connect(muliPort11.ports_b[1],pipe9.port_a) annotation(Line(points = {{ -42, -50},{ -40, -50},{ -40, -40},{ -29, -40}}, color = {0,127,255}, thickness = 0.5, smooth = Smooth.None));
          connect(multiPort10.port_a,pipe10.port_a) annotation(Line(points = {{10, -52},{14, -52}}, color = {0,127,255}, thickness = 0.5, smooth = Smooth.None));
          connect(pipe9.port_b,multiPort10.ports_b[1]) annotation(Line(points = {{ -9, -40},{0, -40},{0, -50},{2, -50}}, color = {0,127,255}, thickness = 0.5, smooth = Smooth.None));
          connect(pipe12.port_b,multiPort10.ports_b[2]) annotation(Line(points = {{ -9, -62},{0, -62},{0, -54},{2, -54}}, color = {0,127,255}, thickness = 0.5, smooth = Smooth.None));
          connect(junction2.port_3,pipe3.port_a) annotation(Line(points = {{12,64},{20,64}}, color = {0,127,255}, thickness = 0.5, smooth = Smooth.None));
          connect(pipe2.port_b,junction2.port_1) annotation(Line(points = {{ -10,78},{2,78},{2,74}}, color = {0,127,255}, thickness = 0.5, smooth = Smooth.None));
          connect(pipe4.port_b,junction2.port_2) annotation(Line(points = {{ -10,48},{2,48},{2,54}}, color = {0,127,255}, thickness = 0.5, smooth = Smooth.None));
          connect(pipe1.port_b,junction1.port_3) annotation(Line(points = {{ -62,64},{ -52,64}}, color = {0,127,255}, thickness = 0.5, smooth = Smooth.None));
          connect(junction1.port_1,pipe4.port_a) annotation(Line(points = {{ -42,54},{ -42,48},{ -30,48}}, color = {0,127,255}, thickness = 0.5, smooth = Smooth.None));
          connect(junction1.port_2,pipe2.port_a) annotation(Line(points = {{ -42,74},{ -42,78},{ -30,78}}, color = {0,127,255}, thickness = 0.5, smooth = Smooth.None));
        end DynamicPipesAndFittings;
      end Pipes;
      package Machines "Test Machine models"
        model TestWaterPumpVariableSpeed "Test pump with variable speed (starting from zero)"
          import Modelica.Fluid;
          extends Modelica.Icons.Example;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 10, Tolerance = 1e-06), Documentation(info = ""));
          Modelica.Blocks.Sources.Ramp N_pump(startTime = 1, duration = 5, height = 1500, offset = 0) annotation(Placement(transformation(extent = {{ -100,62},{ -80,82}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT Source(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient, p = 100000) annotation(Placement(transformation(extent = {{ -100,20},{ -80,40}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT Sink(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient, use_p_in = false, p = 100000) annotation(Placement(transformation(extent = {{34,26},{14,46}}, rotation = 0)));
          Modelica.Fluid.Machines.PrescribedPump pump(redeclare package Medium = Modelica.Media.Water.StandardWater, m_flow_start = 1, redeclare function flowCharacteristic = Modelica.Fluid.Machines.BaseClasses.PumpCharacteristics.quadraticFlow(V_flow_nominal = {0,0.001,0.0015}, head_nominal = {100,50,0}), N_nominal = 1500, use_N_in = true, p_a_start = 100000, p_b_start = 700000) annotation(Placement(transformation(extent = {{ -66,20},{ -34,50}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp valveOpening(height =  -1, duration = 1, offset = 1, startTime = 8) annotation(Placement(transformation(extent = {{ -40,64},{ -20,84}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveIncompressible Valve(redeclare package Medium = Modelica.Media.Water.StandardWater, m_flow_nominal = 1, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 1000000) annotation(Placement(transformation(extent = {{ -16,26},{2,46}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{64, -4},{84,16}}, rotation = 0)));

        equation 
          connect(Valve.port_b,Sink.ports[1]) annotation(Line(points = {{2,36},{14,36}}, color = {0,127,255}));
          connect(Valve.port_a,pump.port_b) annotation(Line(points = {{ -16,36},{ -26,36},{ -26,35},{ -34,35}}, color = {0,127,255}));
          connect(pump.port_a,Source.ports[1]) annotation(Line(points = {{ -66,35},{ -70,35},{ -70,30},{ -80,30}}, color = {0,127,255}));
          connect(N_pump.y,pump.N_in) annotation(Line(points = {{ -79,72},{ -50,72},{ -50,50}}, color = {0,0,127}, smooth = Smooth.None));
          connect(valveOpening.y,Valve.opening) annotation(Line(points = {{ -19,74},{ -7,74},{ -7,44}}, color = {0,0,127}, smooth = Smooth.None));
        end TestWaterPumpVariableSpeed;
        model TestWaterPumpRecirculation "Test pump with variable speed and recirculating flow"
          import Modelica.Fluid;
          extends Modelica.Icons.Example;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 10, Tolerance = 1e-06), Documentation(info = ""));
          Modelica.Blocks.Sources.Ramp N_pump(startTime = 1, height = 1500, offset = 0, duration = 1) annotation(Placement(transformation(extent = {{ -100, -8},{ -80,12}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT Source(nPorts = 1, T = system.T_ambient, p = 100000, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater) annotation(Placement(transformation(extent = {{ -100, -38},{ -80, -18}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT Sink(nPorts = 1, T = system.T_ambient, use_p_in = false, p = 100000, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater) annotation(Placement(transformation(extent = {{62, -30},{42, -10}}, rotation = 0)));
          Modelica.Fluid.Machines.PrescribedPump pump(m_flow_start = 1, redeclare function flowCharacteristic = Modelica.Fluid.Machines.BaseClasses.PumpCharacteristics.quadraticFlow(V_flow_nominal = {0,0.001,0.0015}, head_nominal = {100,50,0}), N_nominal = 1500, use_N_in = true, V = 0.1 / 1000, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, p_a_start = 100000, p_b_start = 700000, energyDynamics = Modelica.Fluid.Types.Dynamics.DynamicFreeInitial, massDynamics = Modelica.Fluid.Types.Dynamics.DynamicFreeInitial) annotation(Placement(transformation(extent = {{ -46, -40},{ -14, -10}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{80,60},{100,80}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveIncompressible V1(CvData = Modelica.Fluid.Types.CvTypes.OpPoint, m_flow_nominal = 1, dp_nominal = 800000, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater) annotation(Placement(transformation(extent = {{ -28,6},{ -50,26}})));
          Modelica.Fluid.Valves.ValveIncompressible V2(CvData = Modelica.Fluid.Types.CvTypes.OpPoint, m_flow_nominal = 1, dp_nominal = 800000, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater) annotation(Placement(transformation(extent = {{0, -30},{22, -10}})));
          Modelica.Blocks.Sources.Ramp V1_Opening(duration = 1, height =  -1, offset = 1, startTime = 5) annotation(Placement(transformation(extent = {{ -100,40},{ -80,60}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp V2_Opening(offset = 0, height = 1, duration = 1, startTime = 3) annotation(Placement(transformation(extent = {{ -100,70},{ -80,90}}, rotation = 0)));

        equation 
          connect(pump.port_a,Source.ports[1]) annotation(Line(points = {{ -46, -25},{ -46, -25},{ -80, -28}}, color = {0,127,255}));
          connect(N_pump.y,pump.N_in) annotation(Line(points = {{ -79,2},{ -30,2},{ -30, -10}}, color = {0,0,127}, smooth = Smooth.None));
          connect(pump.port_b,V1.port_a) annotation(Line(points = {{ -14, -25},{ -20,14},{ -20,16},{ -28,16}}, color = {0,127,255}, smooth = Smooth.None));
          connect(V1.port_b,pump.port_a) annotation(Line(points = {{ -50,16},{ -60,16},{ -60, -25},{ -46, -25}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pump.port_b,V2.port_a) annotation(Line(points = {{ -14, -25},{ -10.2, -25},{ -10.2, -20},{0, -20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(V2.port_b,Sink.ports[1]) annotation(Line(points = {{22, -20},{42, -20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(V1_Opening.y,V1.opening) annotation(Line(points = {{ -79,50},{ -39,50},{ -39,24}}, color = {0,0,127}, smooth = Smooth.None));
          connect(V2_Opening.y,V2.opening) annotation(Line(points = {{ -79,80},{11,80},{11, -12}}, color = {0,0,127}, smooth = Smooth.None));
        end TestWaterPumpRecirculation;
        model TestWaterPumpPowerCharacteristic "Test pump with power consumption characteristic"
          import Modelica.Fluid;
          extends Modelica.Icons.Example;
          Modelica.Fluid.Sources.Boundary_pT Source(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, p = 100000.0, T = system.T_ambient) annotation(Placement(transformation(extent = {{ -100,20},{ -80,40}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT Sink(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, p = 500000.0, T = system.T_ambient, use_p_in = true) annotation(Placement(transformation(extent = {{34,26},{14,46}}, rotation = 0)));
          Modelica.Fluid.Machines.PrescribedPump pump(redeclare package Medium = Modelica.Media.Water.StandardWater, m_flow_start = 1, redeclare function flowCharacteristic = Modelica.Fluid.Machines.BaseClasses.PumpCharacteristics.quadraticFlow(V_flow_nominal = {0,0.001,0.0015}, head_nominal = {100,50,0}), use_powerCharacteristic = true, redeclare function powerCharacteristic = Modelica.Fluid.Machines.BaseClasses.PumpCharacteristics.quadraticPower(V_flow_nominal = {0,0.001,0.0015}, W_nominal = {550,650,800}), V = 0.1 / 1000, N_nominal = 1500, energyDynamics = Modelica.Fluid.Types.Dynamics.SteadyStateInitial, massDynamics = Modelica.Fluid.Types.Dynamics.SteadyStateInitial, p_a_start = 100000, p_b_start = 700000) annotation(Placement(transformation(extent = {{ -66,20},{ -34,50}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant valveOpening(k = 1) annotation(Placement(transformation(extent = {{ -60,60},{ -40,80}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveIncompressible Valve(redeclare package Medium = Modelica.Media.Water.StandardWater, m_flow_nominal = 1, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 20000) annotation(Placement(transformation(extent = {{ -16,26},{2,46}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp downstreamPressure(startTime = 1, duration = 5, offset = 100000.0, height = 1000000.0) annotation(Placement(transformation(extent = {{4,74},{24,94}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{64, -4},{84,16}}, rotation = 0)));

        equation 
          connect(Valve.port_b,Sink.ports[1]) annotation(Line(points = {{2,36},{14,36}}, color = {0,127,255}));
          connect(Valve.port_a,pump.port_b) annotation(Line(points = {{ -16,36},{ -26,36},{ -26,35},{ -34,35}}, color = {0,127,255}));
          connect(pump.port_a,Source.ports[1]) annotation(Line(points = {{ -66,35},{ -70,35},{ -70,30},{ -80,30}}, color = {0,127,255}));
          connect(downstreamPressure.y,Sink.p_in) annotation(Line(points = {{25,84},{58,84},{58,44},{36,44}}, color = {0,0,127}));
          annotation(experiment(StopTime = 10), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
          connect(valveOpening.y,Valve.opening) annotation(Line(points = {{ -39,70},{ -7,70},{ -7,44}}, color = {0,0,127}, smooth = Smooth.None));
        end TestWaterPumpPowerCharacteristic;
        model TestWaterPumpNPSH "Test PumpNPSH"
          import Modelica.Fluid;
          extends Modelica.Icons.Example;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 10, Tolerance = 1e-06), Documentation(info = ""));
          Modelica.Fluid.Sources.Boundary_pT Source(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, p = 100000.0, T = system.T_ambient) annotation(Placement(transformation(extent = {{ -100,20},{ -80,40}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT Sink(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, p = 500000.0, T = system.T_ambient, use_p_in = true) annotation(Placement(transformation(extent = {{34,26},{14,46}}, rotation = 0)));
          Modelica.Fluid.Machines.PrescribedPump pump(redeclare package Medium = Modelica.Media.Water.StandardWater, m_flow_start = 1, redeclare function flowCharacteristic = Modelica.Fluid.Machines.BaseClasses.PumpCharacteristics.quadraticFlow(V_flow_nominal = {0,0.001,0.0015}, head_nominal = {100,50,0}), N_nominal = 1500, p_a_start = 100000, p_b_start = 700000, show_NPSHa = true) annotation(Placement(transformation(extent = {{ -66,20},{ -34,50}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant valveOpening(k = 1) annotation(Placement(transformation(extent = {{ -60,60},{ -40,80}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveIncompressible Valve(redeclare package Medium = Modelica.Media.Water.StandardWater, m_flow_nominal = 1, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 20000) annotation(Placement(transformation(extent = {{ -16,26},{2,46}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp downstreamPressure(startTime = 1, duration = 5, offset = 100000.0, height = 1000000.0) annotation(Placement(transformation(extent = {{4,74},{24,94}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{64, -4},{84,16}}, rotation = 0)));

        equation 
          connect(Valve.port_b,Sink.ports[1]) annotation(Line(points = {{2,36},{14,36}}, color = {0,127,255}));
          connect(Valve.port_a,pump.port_b) annotation(Line(points = {{ -16,36},{ -26,36},{ -26,35},{ -34,35}}, color = {0,127,255}));
          connect(pump.port_a,Source.ports[1]) annotation(Line(points = {{ -66,35},{ -70,35},{ -70,30},{ -80,30}}, color = {0,127,255}));
          connect(downstreamPressure.y,Sink.p_in) annotation(Line(points = {{25,84},{58,84},{58,44},{36,44}}, color = {0,0,127}));
          connect(valveOpening.y,Valve.opening) annotation(Line(points = {{ -39,70},{ -7,70},{ -7,44}}, color = {0,0,127}, smooth = Smooth.None));
        end TestWaterPumpNPSH;
        model TestWaterPumpDefaultLV "Test pump with default options an linear valve"
          import Modelica.Fluid;
          extends Modelica.Icons.Example;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 10, Tolerance = 1e-06), Documentation(info = ""));
          Modelica.Fluid.Sources.Boundary_pT Source(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, p = 100000.0, T = system.T_ambient) annotation(Placement(transformation(extent = {{ -100,20},{ -80,40}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT Sink(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, p = 500000.0, T = system.T_ambient, use_p_in = true) annotation(Placement(transformation(extent = {{34,26},{14,46}}, rotation = 0)));
          Modelica.Fluid.Machines.PrescribedPump pump(redeclare package Medium = Modelica.Media.Water.StandardWater, m_flow_start = 1, redeclare function flowCharacteristic = Modelica.Fluid.Machines.BaseClasses.PumpCharacteristics.quadraticFlow(V_flow_nominal = {0,0.001,0.0015}, head_nominal = {100,50,0}), N_nominal = 1500, p_a_start = 100000, p_b_start = 700000) annotation(Placement(transformation(extent = {{ -66,20},{ -34,50}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant valveOpening(k = 1) annotation(Placement(transformation(extent = {{ -60,60},{ -40,80}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveLinear Valve(redeclare package Medium = Modelica.Media.Water.StandardWater, m_flow_nominal = 1, dp_nominal = 20000) annotation(Placement(transformation(extent = {{ -16,26},{2,46}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp downstreamPressure(startTime = 1, duration = 5, offset = 100000.0, height = 1000000.0) annotation(Placement(transformation(extent = {{4,74},{24,94}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{64, -4},{84,16}}, rotation = 0)));

        equation 
          connect(Valve.port_b,Sink.ports[1]) annotation(Line(points = {{2,36},{14,36}}, color = {0,127,255}));
          connect(Valve.port_a,pump.port_b) annotation(Line(points = {{ -16,36},{ -26,36},{ -26,35},{ -34,35}}, color = {0,127,255}));
          connect(pump.port_a,Source.ports[1]) annotation(Line(points = {{ -66,35},{ -70,35},{ -70,30},{ -80,30}}, color = {0,127,255}));
          connect(downstreamPressure.y,Sink.p_in) annotation(Line(points = {{25,84},{58,84},{58,44},{36,44}}, color = {0,0,127}));
          connect(valveOpening.y,Valve.opening) annotation(Line(points = {{ -39,70},{ -7,70},{ -7,44}}, color = {0,0,127}, smooth = Smooth.None));
        end TestWaterPumpDefaultLV;
        model TestWaterPumpDefault "Test pump with default options"
          import Modelica.Fluid;
          extends Modelica.Icons.Example;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 10, Tolerance = 1e-06), Documentation(info = ""));
          Modelica.Fluid.Sources.Boundary_pT Source(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, p = 100000.0, T = system.T_ambient) annotation(Placement(transformation(extent = {{ -100,20},{ -80,40}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT Sink(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, p = 500000.0, T = system.T_ambient, use_p_in = true) annotation(Placement(transformation(extent = {{34,26},{14,46}}, rotation = 0)));
          Modelica.Fluid.Machines.PrescribedPump pump(redeclare package Medium = Modelica.Media.Water.StandardWater, m_flow_start = 1, redeclare function flowCharacteristic = Modelica.Fluid.Machines.BaseClasses.PumpCharacteristics.quadraticFlow(V_flow_nominal = {0,0.001,0.0015}, head_nominal = {100,50,0}), N_nominal = 1500, p_a_start = 100000, p_b_start = 700000) annotation(Placement(transformation(extent = {{ -66,20},{ -34,50}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant valveOpening(k = 1) annotation(Placement(transformation(extent = {{ -60,60},{ -40,80}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveIncompressible Valve(redeclare package Medium = Modelica.Media.Water.StandardWater, m_flow_nominal = 1, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 20000) annotation(Placement(transformation(extent = {{ -16,26},{2,46}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp downstreamPressure(startTime = 1, duration = 5, offset = 100000.0, height = 1000000.0) annotation(Placement(transformation(extent = {{4,74},{24,94}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{64, -4},{84,16}}, rotation = 0)));

        equation 
          connect(Valve.port_b,Sink.ports[1]) annotation(Line(points = {{2,36},{14,36}}, color = {0,127,255}));
          connect(Valve.port_a,pump.port_b) annotation(Line(points = {{ -16,36},{ -26,36},{ -26,35},{ -34,35}}, color = {0,127,255}));
          connect(pump.port_a,Source.ports[1]) annotation(Line(points = {{ -66,35},{ -70,35},{ -70,30},{ -80,30}}, color = {0,127,255}));
          connect(downstreamPressure.y,Sink.p_in) annotation(Line(points = {{25,84},{58,84},{58,44},{36,44}}, color = {0,0,127}));
          connect(valveOpening.y,Valve.opening) annotation(Line(points = {{ -39,70},{ -7,70},{ -7,44}}, color = {0,0,127}, smooth = Smooth.None));
        end TestWaterPumpDefault;
        model TestWaterPumpDCMotorHeatTransfer "Test pump with dc motor (startup transient)"
          import Modelica.Fluid;
          extends Modelica.Icons.Example;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 8, Tolerance = 1e-06), Documentation(info = ""));
          Modelica.Fluid.Sources.Boundary_pT Source(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient, p = 100000) annotation(Placement(transformation(extent = {{ -50, -60},{ -30, -40}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT Sink(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient, use_p_in = false, p = 100000) annotation(Placement(transformation(extent = {{80,0},{60,20}}, rotation = 0)));
          Modelica.Fluid.Machines.Pump pump(redeclare package Medium = Modelica.Media.Water.StandardWater, m_flow_start = 1, N_nominal = 1500, use_HeatTransfer = true, use_powerCharacteristic = true, redeclare function flowCharacteristic = Modelica.Fluid.Machines.BaseClasses.PumpCharacteristics.quadraticFlow(V_flow_nominal = {0,0.001,0.0015}, head_nominal = {100,50,0}), redeclare function powerCharacteristic = Modelica.Fluid.Machines.BaseClasses.PumpCharacteristics.quadraticPower(V_flow_nominal = {0,0.001,0.0015}, W_nominal = {550,650,800}), p_a_start = 100000, p_b_start = 100000) annotation(Placement(transformation(extent = {{ -10, -5},{20,25}}, rotation = 0)));
          Modelica.Blocks.Sources.Step valveOpening(offset = 1, startTime = 5, height =  -1) annotation(Placement(transformation(extent = {{70,40},{50,60}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveIncompressible Valve(redeclare package Medium = Modelica.Media.Water.StandardWater, m_flow_nominal = 1, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 1000000) annotation(Placement(transformation(extent = {{31,0},{49,20}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{50, -70},{70, -50}}, rotation = 0)));
          Modelica.Electrical.Machines.BasicMachines.DCMachines.DC_PermanentMagnet motor(La = 0.001, Jr = 0.1, inertiaRotor(w(fixed = true, displayUnit = "1/min", start = 0.10471975511966)), VaNominal = 400, wNominal(displayUnit = "1/min") = 157.07963267949, Ra = 10, IaNominal = 10) annotation(Placement(transformation(extent = {{ -54,28},{ -30,52}})));
          Modelica.Electrical.Analog.Sources.StepVoltage stepVoltage(startTime = 1, offset = 0.1, V = 400) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation =  -90, origin = { -80,40})));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(Placement(transformation(extent = {{ -90,2},{ -70,24}})));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor housing(C = 460 * 1) annotation(Placement(transformation(extent = {{ -1, -20},{19, -40}})));

        equation 
          connect(Valve.port_b,Sink.ports[1]) annotation(Line(points = {{49,10},{60,10}}, color = {0,127,255}));
          connect(Valve.port_a,pump.port_b) annotation(Line(points = {{31,10},{20,10}}, color = {0,127,255}));
          connect(pump.port_a,Source.ports[1]) annotation(Line(points = {{ -10,10},{ -20,10},{ -20, -50},{ -30, -50}}, color = {0,127,255}));
          connect(valveOpening.y,Valve.opening) annotation(Line(points = {{49,50},{40,50},{40,18}}, color = {0,0,127}, smooth = Smooth.None));
          connect(motor.pin_ap,stepVoltage.p) annotation(Line(points = {{ -34.8,52},{ -34,52},{ -34,60},{ -80,60},{ -80,50}}, color = {0,0,255}, smooth = Smooth.None));
          connect(stepVoltage.n,motor.pin_an) annotation(Line(points = {{ -80,30},{ -62,30},{ -62,52},{ -49.2,52}}, color = {0,0,255}, smooth = Smooth.None));
          connect(stepVoltage.n,ground.p) annotation(Line(points = {{ -80,30},{ -80,24}}, color = {0,0,255}, smooth = Smooth.None));
          connect(pump.heatPort,housing.port) annotation(Line(points = {{11,1},{20,1},{20, -20},{9, -20}}, color = {191,0,0}, smooth = Smooth.None));
          connect(pump.shaft,motor.flange) annotation(Line(points = {{5,25},{5,40},{ -30,40}}, color = {0,0,0}, smooth = Smooth.None));
        end TestWaterPumpDCMotorHeatTransfer;
        model TestWaterPumpDCMotor "Test pump with dc motor (startup transient)"
          import Modelica.Fluid;
          extends Modelica.Icons.Example;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 8, Tolerance = 1e-06), Documentation(info = ""));
          Modelica.Fluid.Sources.Boundary_pT Source(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient, p = 100000) annotation(Placement(transformation(extent = {{ -50, -60},{ -30, -40}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT Sink(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient, use_p_in = false, p = 100000) annotation(Placement(transformation(extent = {{80,0},{60,20}}, rotation = 0)));
          Modelica.Fluid.Machines.Pump pump(redeclare package Medium = Modelica.Media.Water.StandardWater, m_flow_start = 1, redeclare function flowCharacteristic = Modelica.Fluid.Machines.BaseClasses.PumpCharacteristics.quadraticFlow(V_flow_nominal = {0,0.001,0.0015}, head_nominal = {100,50,0}), N_nominal = 1500, p_a_start = 100000, p_b_start = 700000) annotation(Placement(transformation(extent = {{ -10, -5},{20,25}}, rotation = 0)));
          Modelica.Blocks.Sources.Step valveOpening(height =  -0.5, offset = 1, startTime = 5) annotation(Placement(transformation(extent = {{70,40},{50,60}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveIncompressible Valve(redeclare package Medium = Modelica.Media.Water.StandardWater, m_flow_nominal = 1, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 1000000) annotation(Placement(transformation(extent = {{31,0},{49,20}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{50, -70},{70, -50}}, rotation = 0)));
          Modelica.Electrical.Machines.BasicMachines.DCMachines.DC_PermanentMagnet motor(La = 0.001, Jr = 0.1, inertiaRotor(w(fixed = true, displayUnit = "1/min", start = 0.10471975511966)), VaNominal = 400, wNominal(displayUnit = "1/min") = 157.07963267949, IaNominal = 10, Ra = 10) annotation(Placement(transformation(extent = {{ -54,28},{ -30,52}})));
          Modelica.Electrical.Analog.Sources.StepVoltage stepVoltage(startTime = 1, offset = 0.1, V = 400) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation =  -90, origin = { -80,40})));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(Placement(transformation(extent = {{ -90,2},{ -70,24}})));

        equation 
          connect(Valve.port_b,Sink.ports[1]) annotation(Line(points = {{49,10},{60,10}}, color = {0,127,255}));
          connect(Valve.port_a,pump.port_b) annotation(Line(points = {{31,10},{20,10}}, color = {0,127,255}));
          connect(pump.port_a,Source.ports[1]) annotation(Line(points = {{ -10,10},{ -20,10},{ -20, -50},{ -30, -50}}, color = {0,127,255}));
          connect(valveOpening.y,Valve.opening) annotation(Line(points = {{49,50},{40,50},{40,18}}, color = {0,0,127}, smooth = Smooth.None));
          connect(motor.pin_ap,stepVoltage.p) annotation(Line(points = {{ -34.8,52},{ -34,52},{ -34,60},{ -80,60},{ -80,50}}, color = {0,0,255}, smooth = Smooth.None));
          connect(stepVoltage.n,motor.pin_an) annotation(Line(points = {{ -80,30},{ -62,30},{ -62,52},{ -49.2,52}}, color = {0,0,255}, smooth = Smooth.None));
          connect(stepVoltage.n,ground.p) annotation(Line(points = {{ -80,30},{ -80,24}}, color = {0,0,255}, smooth = Smooth.None));
          connect(pump.shaft,motor.flange) annotation(Line(points = {{5,25},{5,40},{ -30,40}}, color = {0,0,0}, smooth = Smooth.None));
        end TestWaterPumpDCMotor;
        model TestWaterPumpCheckValve "Test pump with check valve behaviour"
          import Modelica.Fluid;
          extends Modelica.Icons.Example;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 3, Tolerance = 1e-06), Documentation(info = ""));
          Modelica.Blocks.Sources.Ramp N_pump(startTime = 1, duration = 1, offset = 1500, height =  -1500) annotation(Placement(transformation(extent = {{ -80,0},{ -60,20}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT Source(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient, p = 100000) annotation(Placement(transformation(extent = {{ -80, -40},{ -60, -20}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT Sink(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient, use_p_in = false, p = 500000) annotation(Placement(transformation(extent = {{60, -40},{40, -20}}, rotation = 0)));
          Modelica.Fluid.Machines.PrescribedPump pump(redeclare package Medium = Modelica.Media.Water.StandardWater, m_flow_start = 1, redeclare function flowCharacteristic = Modelica.Fluid.Machines.BaseClasses.PumpCharacteristics.quadraticFlow(V_flow_nominal = {0,0.001,0.0015}, head_nominal = {100,50,0}), N_nominal = 1500, checkValve = true, V = 0.1 / 1000, use_N_in = true, energyDynamics = Modelica.Fluid.Types.Dynamics.SteadyStateInitial, massDynamics = Modelica.Fluid.Types.Dynamics.SteadyStateInitial, p_a_start = 100000, p_b_start = 700000) annotation(Placement(transformation(extent = {{ -40, -40},{ -20, -20}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{80,60},{100,80}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveIncompressible valve(redeclare package Medium = Modelica.Media.Water.StandardWater, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, m_flow_nominal = 1, dp_nominal = 800000) annotation(Placement(transformation(extent = {{0, -40},{22, -20}})));
          Modelica.Blocks.Sources.Constant valveOpening(k = 1) annotation(Placement(transformation(extent = {{ -38,32},{ -18,52}}, rotation = 0)));

        equation 
          connect(pump.port_a,Source.ports[1]) annotation(Line(points = {{ -40, -30},{ -60, -30}}, color = {0,127,255}));
          connect(N_pump.y,pump.N_in) annotation(Line(points = {{ -59,10},{ -30,10},{ -30, -20}}, color = {0,0,127}, smooth = Smooth.None));
          connect(pump.port_b,valve.port_a) annotation(Line(points = {{ -20, -30},{0, -30}}, color = {0,127,255}, smooth = Smooth.None));
          connect(valve.port_b,Sink.ports[1]) annotation(Line(points = {{22, -30},{40, -30}}, color = {0,127,255}, smooth = Smooth.None));
          connect(valveOpening.y,valve.opening) annotation(Line(points = {{ -17,42},{11,42},{11, -22}}, color = {0,0,127}, smooth = Smooth.None));
        end TestWaterPumpCheckValve;
        model TestSweptVolume "Enclosed medium with fixed quantity in an adiabatic volume with varying size"
          extends Modelica.Icons.Example;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 5));
          Modelica.Fluid.Machines.SweptVolume sweptVolume(nPorts = 1, redeclare package Medium = Modelica.Media.Air.DryAirNasa, clearance(displayUnit = "l") = 1e-05, use_portsData = false, pistonCrossArea(displayUnit = "cm2") = 0.001) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 90, origin = {10,10})));
          Modelica.Mechanics.Translational.Components.Mass mass(m = 1, s(start = 0.5, fixed = true)) annotation(Placement(transformation(extent = {{ -10,0},{ -30,20}})));
          Modelica.Fluid.Sources.Boundary_pT boundary(nPorts = 1, redeclare package Medium = Modelica.Media.Air.DryAirNasa, use_p_in = true, p = 200000) annotation(Placement(transformation(extent = {{60,0},{40,20}})));
          Modelica.Blocks.Sources.Trapezoid trapezoid(offset = 100000.0, amplitude = 100000.0, rising = 0.5, falling = 0.5, period = 2, startTime = 1) annotation(Placement(transformation(extent = {{40,40},{60,60}})));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}})));
          Modelica.Mechanics.Translational.Components.Fixed fixed(s0 = 2) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 270, origin = { -70,10})));
          Modelica.Mechanics.Translational.Components.SpringDamper springDamper(c = 100, d = 1, s_rel0 = 1.5) annotation(Placement(transformation(extent = {{ -40,0},{ -60,20}})));

        equation 
          connect(sweptVolume.ports[1],boundary.ports[1]) annotation(Line(points = {{20,10},{40,10}}, color = {0,127,255}, smooth = Smooth.None));
          connect(boundary.p_in,trapezoid.y) annotation(Line(points = {{62,18},{80,18},{80,50},{61,50}}, color = {0,0,127}, smooth = Smooth.None));
          connect(mass.flange_a,sweptVolume.flange) annotation(Line(points = {{ -10,10},{0,10}}, color = {0,127,0}, smooth = Smooth.None));
          connect(mass.flange_b,springDamper.flange_a) annotation(Line(points = {{ -30,10},{ -40,10}}, color = {0,127,0}, smooth = Smooth.None));
          connect(springDamper.flange_b,fixed.flange) annotation(Line(points = {{ -60,10},{ -70,10}}, color = {0,127,0}, smooth = Smooth.None));
        end TestSweptVolume;
        model TestControlledPump "Test controlled pump"
          import Modelica.Fluid;
          extends Modelica.Icons.Example;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 7, Tolerance = 1e-06), Documentation(info = ""));
          Modelica.Blocks.Sources.Sine massFlowRate(amplitude = 0.5, offset = 1, freqHz = 1 / 7) annotation(Placement(transformation(extent = {{ -70,60},{ -50,80}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT source(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient, p = 100000) annotation(Placement(transformation(extent = {{ -90,20},{ -70,40}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT sink(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient, use_p_in = true, p = 200000) annotation(Placement(transformation(extent = {{50,20},{30,40}}, rotation = 0)));
          Modelica.Fluid.Machines.ControlledPump pump(redeclare package Medium = Modelica.Media.Water.StandardWater, use_m_flow_set = true, m_flow_nominal = 1, p_a_nominal = 100000, p_b_nominal = 1000000) annotation(Placement(transformation(extent = {{ -50,20},{ -30,40}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{70, -90},{90, -70}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveIncompressible valve(redeclare package Medium = Modelica.Media.Water.StandardWater, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, m_flow_nominal = 1, dp_nominal = 200000) annotation(Placement(transformation(extent = {{ -10,20},{10,40}})));
          Modelica.Blocks.Sources.Ramp valveOpening(duration = 1, startTime = 3, height =  -0.5, offset = 1) annotation(Placement(transformation(extent = {{30,60},{10,80}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine pressure1(amplitude = 500000.0, freqHz = 1 / 7, offset = 1000000.0) annotation(Placement(transformation(extent = {{ -70, -40},{ -50, -20}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT source1(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient, p = 100000) annotation(Placement(transformation(extent = {{ -90, -80},{ -70, -60}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT sink1(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, T = system.T_ambient, use_p_in = true, p = 200000) annotation(Placement(transformation(extent = {{50, -80},{30, -60}}, rotation = 0)));
          Modelica.Fluid.Machines.ControlledPump pump1(redeclare package Medium = Modelica.Media.Water.StandardWater, control_m_flow = false, use_m_flow_set = false, use_p_set = true, m_flow_nominal = 1, p_a_nominal = 100000, p_b_nominal = 1000000) annotation(Placement(transformation(extent = {{ -50, -80},{ -30, -60}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveIncompressible valve1(redeclare package Medium = Modelica.Media.Water.StandardWater, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, m_flow_nominal = 1, dp_nominal = 200000) annotation(Placement(transformation(extent = {{ -10, -80},{10, -60}})));
          Modelica.Blocks.Sources.Ramp valveOpening1(duration = 1, startTime = 3, height =  -0.5, offset = 1) annotation(Placement(transformation(extent = {{30, -40},{10, -20}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp sinkPressure(duration = 1, startTime = 5, height =  -700000.0, offset = 800000.0) annotation(Placement(transformation(extent = {{90,60},{70,80}}, rotation = 0)));

        equation 
          connect(pump.port_a,source.ports[1]) annotation(Line(points = {{ -50,30},{ -70,30}}, color = {0,127,255}));
          connect(pump.port_b,valve.port_a) annotation(Line(points = {{ -30,30},{ -10,30}}, color = {0,127,255}, smooth = Smooth.None));
          connect(valve.port_b,sink.ports[1]) annotation(Line(points = {{10,30},{30,30}}, color = {0,127,255}, smooth = Smooth.None));
          connect(valveOpening.y,valve.opening) annotation(Line(points = {{9,70},{0,70},{0,38}}, color = {0,0,127}, smooth = Smooth.None));
          connect(massFlowRate.y,pump.m_flow_set) annotation(Line(points = {{ -49,70},{ -45,70},{ -45,38.2}}, color = {0,0,127}, smooth = Smooth.None));
          connect(pump1.port_a,source1.ports[1]) annotation(Line(points = {{ -50, -70},{ -70, -70}}, color = {0,127,255}));
          connect(pump1.port_b,valve1.port_a) annotation(Line(points = {{ -30, -70},{ -10, -70}}, color = {0,127,255}, smooth = Smooth.None));
          connect(valve1.port_b,sink1.ports[1]) annotation(Line(points = {{10, -70},{30, -70}}, color = {0,127,255}, smooth = Smooth.None));
          connect(valveOpening1.y,valve1.opening) annotation(Line(points = {{9, -30},{0, -30},{0, -62}}, color = {0,0,127}, smooth = Smooth.None));
          connect(pressure1.y,pump1.p_set) annotation(Line(points = {{ -49, -30},{ -35, -30},{ -35, -61.8}}, color = {0,0,127}, smooth = Smooth.None));
          connect(sinkPressure.y,sink.p_in) annotation(Line(points = {{69,70},{60,70},{60,38},{52,38}}, color = {0,0,127}, smooth = Smooth.None));
          connect(sinkPressure.y,sink1.p_in) annotation(Line(points = {{69,70},{60,70},{60, -62},{52, -62}}, color = {0,0,127}, smooth = Smooth.None));
        end TestControlledPump;
      end Machines;
      package Fittings "Test Fitting models"
        model TestWallFriction
          import Modelica.Fluid;
          extends Modelica.Icons.Example;
          replaceable package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater extends Modelica.Media.Interfaces.PartialMedium "Medium in all components" annotation(choicesAllMatching = true);
          parameter Modelica.SIunits.Length roughness = 2.5e-05;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 10, NumberOfIntervals = 10000), experimentSetupOutput, Documentation(info = "<html>
<p>
5 different wall friction models are compared.
</p>

<p>
pipe1 and pipe2 should be identical to pipe3 and pipe4 for
pipe1.WallFriction = WallFriction.QuadraticTurbulent (since the same equations).
</p>

<p>
pipe1 and pipe2 should be identical to pressureLossPipe for
pipe1.WallFriction = WallFriction.Detailed (since the same equations).
</p>
</html>"));
          Modelica.Fluid.Sources.Boundary_pT ambient_a(redeclare package Medium = Medium, p = system.p_ambient, T = system.T_ambient, use_p_in = true, nPorts = 4) annotation(Placement(transformation(extent = {{ -38,40},{ -18,60}}, rotation = 0)));
          Modelica.Blocks.Sources.TimeTable p_table(table = [0,99999.0;10,100001.0]) annotation(Placement(transformation(extent = {{ -80,40},{ -60,60}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT ambient_p1(nPorts = 1, redeclare package Medium = Medium, p = 100000.0, T = Modelica.SIunits.Conversions.from_degC(80)) annotation(Placement(transformation(extent = {{62,40},{42,60}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT ambient_p2(nPorts = 1, redeclare package Medium = Medium, p = 100000.0, T = Modelica.SIunits.Conversions.from_degC(80)) annotation(Placement(transformation(extent = {{60,10},{40,30}}, rotation = 0)));
          Modelica.Fluid.Pipes.BaseClasses.WallFriction.TestWallFrictionAndGravity pipe1(length = 1, diameter = 0.1, redeclare package Medium = Medium, roughness = roughness, port_a(m_flow(start =  -0.6)), redeclare package WallFriction = Modelica.Fluid.Pipes.BaseClasses.WallFriction.Detailed, dp_small = 0.1, show_Re = true) annotation(Placement(transformation(extent = {{0,40},{20,60}}, rotation = 0)));
          Modelica.Fluid.Pipes.BaseClasses.WallFriction.TestWallFrictionAndGravity pipe2(length = 1, diameter = 0.1, redeclare package Medium = Medium, roughness = roughness, redeclare package WallFriction = Modelica.Fluid.Pipes.BaseClasses.WallFriction.Detailed(from_dp = false, show_Re = true), port_a(m_flow(start =  -0.6))) annotation(Placement(transformation(extent = {{0,10},{20,30}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT ambient_p3(nPorts = 1, redeclare package Medium = Medium, p = 100000.0, T = Modelica.SIunits.Conversions.from_degC(80)) annotation(Placement(transformation(extent = {{60, -20},{40,0}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT ambient_p4(nPorts = 1, redeclare package Medium = Medium, p = 100000.0, T = Modelica.SIunits.Conversions.from_degC(80)) annotation(Placement(transformation(extent = {{60, -50},{40, -30}}, rotation = 0)));
          Modelica.Fluid.Fittings.BaseClasses.QuadraticTurbulent.TestWallFriction pipe3(length = 1, diameter = 0.1, redeclare package Medium = Medium, roughness = roughness, dp_small = 0.1) annotation(Placement(transformation(extent = {{0, -20},{20,0}}, rotation = 0)));
          Modelica.Fluid.Fittings.BaseClasses.QuadraticTurbulent.TestWallFriction pipe4(length = 1, diameter = 0.1, from_dp = false, redeclare package Medium = Medium, roughness = roughness) annotation(Placement(transformation(extent = {{0, -50},{20, -30}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{68,74},{88,94}}, rotation = 0)));

        equation 
          connect(p_table.y,ambient_a.p_in) annotation(Line(points = {{ -59,50},{ -52,50},{ -52,58},{ -40,58}}, color = {0,0,127}));
          connect(ambient_a.ports[1],pipe1.port_a) annotation(Line(points = {{ -18,53},{ -9,53},{ -9,50},{0,50}}, color = {0,127,255}));
          connect(pipe1.port_b,ambient_p1.ports[1]) annotation(Line(points = {{20,50},{42,50}}, color = {0,127,255}));
          connect(pipe2.port_b,ambient_p2.ports[1]) annotation(Line(points = {{20,20},{40,20}}, color = {0,127,255}));
          connect(ambient_a.ports[2],pipe2.port_a) annotation(Line(points = {{ -18,51},{ -12,51},{ -12,20},{0,20}}, color = {0,127,255}));
          connect(pipe3.port_b,ambient_p3.ports[1]) annotation(Line(points = {{20, -10},{40, -10}}, color = {0,127,255}));
          connect(pipe4.port_b,ambient_p4.ports[1]) annotation(Line(points = {{20, -40},{40, -40}}, color = {0,127,255}));
          connect(ambient_a.ports[3],pipe3.port_a) annotation(Line(points = {{ -18,49},{ -12,49},{ -12, -10},{0, -10}}, color = {0,127,255}));
          connect(ambient_a.ports[4],pipe4.port_a) annotation(Line(points = {{ -18,47},{ -12,47},{ -12, -40},{0, -40}}, color = {0,127,255}));
        end TestWallFriction;
        model TestSuddenExpansion "Test of sudden expansion models, with correct and wrong usage"
          extends Modelica.Icons.Example;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
This example shows the use of a sudden expansion / contraction model, which is connected to two boundary conditions prescribing static pressure. Notice that the prescribed static pressure on the right boundary is higher than on the left one. Still, the fluid flows from left to right.
</p>
<p>
The reason for this is that the boundary conditions model infinite reservoirs with an infinite diameter and thus zero flow velocity. The sudden expansion model does however have two ends with finite diameters, and, as explained in the <a href=\"Modelica://Modelica.Fluid.UsersGuide.Overview\">Overview</a> of the Users' Guide, the momentum balance is not fulfilled exactly for this type of connections. Using a simple <code>connect()</code>-statement, the difference of the kinetic terms is neglected, which is not reasonable in the present model: At the left boundary condition it is zero, and on the left side of the sudden expansion it has a non-zero value. It is not reasonable to neglect it in the shown model, because there is little friction and therefore these kinetic effects dominate. Consequently, only modelling these effects explicitly leads to the correct results.
</p>
<p>
To do so, two additional sudden expansions / contractions are included in the model. The diameter is set to <code>inf</code> close to the boundaries and the proper values close to the original model. These additional components now introduce <i>exact</i> momentum balances and the results are as expected.
</p>
</html>"));
          Modelica.Fluid.Sources.Boundary_pT leftBoundary1(redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, nPorts = 1, p = 100000) annotation(Placement(transformation(extent = {{ -80,20},{ -60,40}})));
          Modelica.Fluid.Sources.Boundary_pT rightBoundary1(redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, nPorts = 1, p = 110000) annotation(Placement(transformation(extent = {{80,20},{60,40}})));
          Modelica.Fluid.Fittings.AbruptAdaptor suddenExpansion1(diameter_a = 0.1, diameter_b = 0.2, redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase) annotation(Placement(transformation(extent = {{ -12,20},{8,40}})));
          Modelica.Fluid.Sources.Boundary_pT leftBoundary2(redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, nPorts = 1, p = 100000) annotation(Placement(transformation(extent = {{ -90, -40},{ -70, -20}})));
          Modelica.Fluid.Sources.Boundary_pT rightBoundary2(redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, nPorts = 1, p = 110000) annotation(Placement(transformation(extent = {{90, -40},{70, -20}})));
          Modelica.Fluid.Fittings.AbruptAdaptor suddenExpansion2(diameter_a = 0.1, diameter_b = 0.2, redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase) annotation(Placement(transformation(extent = {{ -10, -40},{10, -20}})));
          Modelica.Fluid.Fittings.AbruptAdaptor leftAdapter(diameter_a = 0.1, redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, diameter_b = Modelica.Constants.inf) annotation(Placement(transformation(extent = {{ -40, -40},{ -60, -20}})));
          Modelica.Fluid.Fittings.AbruptAdaptor rightAdapter(redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, diameter_a = 0.2, diameter_b = Modelica.Constants.inf) annotation(Placement(transformation(extent = {{40, -40},{60, -20}})));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100,80},{ -80,100}})));

        equation 
          connect(leftBoundary1.ports[1],suddenExpansion1.port_a) annotation(Line(points = {{ -60,30},{ -12,30}}, color = {0,127,255}, smooth = Smooth.None));
          connect(suddenExpansion1.port_b,rightBoundary1.ports[1]) annotation(Line(points = {{8,30},{60,30}}, color = {0,127,255}, smooth = Smooth.None));
          connect(leftAdapter.port_b,leftBoundary2.ports[1]) annotation(Line(points = {{ -60, -30},{ -70, -30}}, color = {0,127,255}, smooth = Smooth.None));
          connect(leftAdapter.port_a,suddenExpansion2.port_a) annotation(Line(points = {{ -40, -30},{ -10, -30}}, color = {0,127,255}, smooth = Smooth.None));
          connect(suddenExpansion2.port_b,rightAdapter.port_a) annotation(Line(points = {{10, -30},{40, -30}}, color = {0,127,255}, smooth = Smooth.None));
          connect(rightAdapter.port_b,rightBoundary2.ports[1]) annotation(Line(points = {{60, -30},{70, -30}}, color = {0,127,255}, smooth = Smooth.None));
        end TestSuddenExpansion;
        model TestSimpleGenericOrifice
          extends Modelica.Icons.Example;
          parameter Real diameter_a = 0.1;
          parameter Real diameter_b = 0.2;
          parameter Real A_rel = (diameter_a / diameter_b) ^ 2;
          parameter Real zeta = (1 - A_rel) ^ 2;
          replaceable package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater extends Modelica.Media.Interfaces.PartialMedium "Medium in all components" annotation(choicesAllMatching = true);
          annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 10, NumberOfIntervals = 10000), experimentSetupOutput, Documentation(info = "<html>
</html>"));
          Modelica.Fluid.Sources.Boundary_pT ambient_a(redeclare package Medium = Medium, p = system.p_ambient, T = system.T_ambient, use_p_in = true, nPorts = 2) annotation(Placement(transformation(extent = {{ -40,40},{ -20,60}}, rotation = 0)));
          Modelica.Blocks.Sources.TimeTable p_table(table = [0,99990.0;10,100010.0]) annotation(Placement(transformation(extent = {{ -80,40},{ -60,60}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT ambient_p1(nPorts = 1, redeclare package Medium = Medium, p = 100000.0, T = Modelica.SIunits.Conversions.from_degC(80)) annotation(Placement(transformation(extent = {{60,40},{40,60}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT ambient_p2(nPorts = 1, redeclare package Medium = Medium, p = 100000.0, T = Modelica.SIunits.Conversions.from_degC(80)) annotation(Placement(transformation(extent = {{60,10},{40,30}}, rotation = 0)));
          Modelica.Fluid.Fittings.SimpleGenericOrifice expansion1(redeclare package Medium = Medium, zeta = zeta, diameter = diameter_a) annotation(Placement(transformation(extent = {{0,40},{20,60}}, rotation = 0)));
          Modelica.Fluid.Fittings.SimpleGenericOrifice expansion2(redeclare package Medium = Medium, from_dp = false, zeta = zeta, diameter = diameter_a) annotation(Placement(transformation(extent = {{0,10},{20,30}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -82, -90},{ -62, -70}}, rotation = 0)));

        equation 
          connect(p_table.y,ambient_a.p_in) annotation(Line(points = {{ -59,50},{ -52,50},{ -52,58},{ -42,58}}, color = {0,0,127}));
          connect(ambient_a.ports[1],expansion1.port_a) annotation(Line(points = {{ -20,52},{ -10,52},{ -10,50},{0,50}}, color = {0,127,255}));
          connect(expansion1.port_b,ambient_p1.ports[1]) annotation(Line(points = {{20,50},{40,50}}, color = {0,127,255}));
          connect(expansion2.port_b,ambient_p2.ports[1]) annotation(Line(points = {{20,20},{40,20}}, color = {0,127,255}));
          connect(expansion2.port_a,ambient_a.ports[2]) annotation(Line(points = {{0,20},{ -10,20},{ -10,48},{ -20,48}}, color = {0,127,255}));
        end TestSimpleGenericOrifice;
        model TestSharpEdgedOrifice
          import Modelica.Fluid;
          extends Modelica.Icons.Example;
          replaceable package Medium = Modelica.Media.IdealGases.SingleGases.O2 extends Modelica.Media.Interfaces.PartialMedium "Medium in all components" annotation(choicesAllMatching = true);
          annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 10, NumberOfIntervals = 10000), experimentSetupOutput, Documentation(info = "<html>
</html>"));
          Modelica.Fluid.Sources.Boundary_pT ambient_p(redeclare package Medium = Medium, p = system.p_ambient, T = system.T_ambient, use_p_in = true, nPorts = 2) annotation(Placement(transformation(extent = {{ -40,40},{ -20,60}}, rotation = 0)));
          Modelica.Blocks.Sources.TimeTable p_table(table = [0,10000.0;10,200000.0]) annotation(Placement(transformation(extent = {{ -80,40},{ -60,60}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT ambient_p1(nPorts = 1, redeclare package Medium = Medium, p = 100000.0, T = Modelica.SIunits.Conversions.from_degC(80)) annotation(Placement(transformation(extent = {{60,40},{40,60}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT ambient_p2(nPorts = 1, redeclare package Medium = Medium, p = 100000.0, T = Modelica.SIunits.Conversions.from_degC(80)) annotation(Placement(transformation(extent = {{60,10},{40,30}}, rotation = 0)));
          Modelica.Fluid.Fittings.SharpEdgedOrifice orifice1(redeclare package Medium = Medium, diameter = 0.1, alpha = 50, leastDiameter = 0.02, length = 0.005, show_Re = true) annotation(Placement(transformation(extent = {{0,40},{20,60}}, rotation = 0)));
          Modelica.Fluid.Fittings.SharpEdgedOrifice orifice2(redeclare package Medium = Medium, diameter = 0.1, alpha = 50, from_dp = false, leastDiameter = 0.02, length = 0.005) annotation(Placement(transformation(extent = {{0,10},{20,30}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{60, -68},{80, -48}}, rotation = 0)));

        equation 
          connect(p_table.y,ambient_p.p_in) annotation(Line(points = {{ -59,50},{ -52,50},{ -52,58},{ -42,58}}, color = {0,0,127}));
          connect(ambient_p.ports[1],orifice1.port_a) annotation(Line(points = {{ -20,52},{ -10,52},{ -10,50},{0,50}}, color = {0,127,255}));
          connect(orifice1.port_b,ambient_p1.ports[1]) annotation(Line(points = {{20,50},{40,50}}, color = {0,127,255}));
          connect(ambient_p.ports[2],orifice2.port_a) annotation(Line(points = {{ -20,48},{ -12,48},{ -12,20},{0,20}}, color = {0,127,255}));
          connect(orifice2.port_b,ambient_p2.ports[1]) annotation(Line(points = {{20,20},{40,20}}, color = {0,127,255}));
        end TestSharpEdgedOrifice;
        model TestPressureLossDerivatives "Test that PressureLoss components can be differentiated"
          import Modelica.Fluid.Fittings.BaseClasses.QuadraticTurbulent;
          extends Modelica.Icons.Example;
          parameter SI.Diameter diameter_a = 0.1 "Inner diameter of pipe at port_a";
          parameter SI.Diameter diameter_b = 0.2 "Inner diameter of pipe at port_b";
          parameter QuadraticTurbulent.LossFactorData data = QuadraticTurbulent.LossFactorData.suddenExpansion(diameter_a, diameter_b) "Loss factors for both flow directions";
          parameter SI.Pressure dp_small = 0.1;
          SI.Density rho_a;
          SI.Density rho_b;
          SI.DynamicViscosity mu_a;
          SI.DynamicViscosity mu_b;
          SI.Pressure dp;
          SI.MassFlowRate m_flow1;
          SI.MassFlowRate m_flow2;
          Real der_m_flow1;
        protected 
          constant Real t2p = 1 "dummy unit constant";
          constant Real t2d = 1 "dummy unit constant";
          constant Real d2mu = 1 "dummy unit constant";
          annotation(experiment(StopTime = 3), experimentSetupOutput, Documentation(info = "<html>

</html>"));

        equation 
          dp = t2p * time - 1;
          rho_a = 0.1 + t2d * time;
          rho_b = 0.2 + 0.5 * t2d * time;
          mu_a = 0.1 * d2mu * rho_a;
          mu_b = 0.4 * d2mu * rho_b;
          m_flow1 = QuadraticTurbulent.massFlowRate_dp(dp, rho_a, rho_b, data, dp_small);
          m_flow2 = QuadraticTurbulent.massFlowRate_dp_and_Re(dp, rho_a, rho_b, mu_a, mu_b, data);
          der_m_flow1 = der(m_flow1);
        end TestPressureLossDerivatives;
        model TestMultiPortTraceSubstances "Test model for multiport and volumes with trace substances."
          import Modelica.Fluid;
          extends Modelica.Icons.Example;
          package Medium = Modelica.Media.Air.MoistAir(extraPropertiesNames = {"CO2"});
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          Modelica.Fluid.Sources.Boundary_pT source2(redeclare package Medium = Medium, X = Medium.X_default, nPorts = 3, p = 100000, T = 278.15) annotation(Placement(transformation(origin = {90, -20}, extent = {{ -10, -10},{10,10}}, rotation = 180)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100,80},{ -80,100}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT source1(T = system.T_ambient, use_p_in = true, redeclare package Medium = Medium, use_C_in = true, use_X_in = true, nPorts = 3, p = 500000) annotation(Placement(transformation(extent = {{ -68, -28},{ -48, -8}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp P(duration = 1, height =  -40, offset = 100000.0 + 20) annotation(Placement(transformation(extent = {{ -100,24},{ -80,44}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp C(duration = 1, height = 0.001519) "substance concentration, raising to 1000 PPM CO2" annotation(Placement(transformation(extent = {{ -100, -60},{ -80, -40}})));
          Modelica.Blocks.Sources.Constant X(k = 0.02) annotation(Placement(transformation(extent = {{ -100, -8},{ -80,12}})));
          Modelica.Blocks.Sources.RealExpression X2(y = 1 - X.y) "Concentration of X[2]" annotation(Placement(transformation(extent = {{ -100, -32},{ -80, -12}})));
          Modelica.Fluid.Vessels.ClosedVolume volume1(redeclare package Medium = Medium, nPorts = 3, C_start = {0.0001}, use_portsData = false, V = 1) annotation(Placement(transformation(extent = {{12,20},{32,40}})));
          Modelica.Fluid.Vessels.ClosedVolume volume2(redeclare package Medium = Medium, nPorts = 3, C_start = {0.0001}, use_portsData = false, V = 1) annotation(Placement(transformation(extent = {{12, -20},{32,0}})));
          Modelica.Fluid.Vessels.ClosedVolume volume3(redeclare package Medium = Medium, V = 1, nPorts = 3, C_start = {0.0001}, use_portsData = false) annotation(Placement(transformation(extent = {{12, -80},{32, -60}})));
          Modelica.Fluid.Sensors.TraceSubstances traceSubstance1(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{36,24},{56,44}})));
          Modelica.Fluid.Sensors.TraceSubstances traceSubstance2(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{34, -16},{54,4}})));
          Modelica.Fluid.Sensors.TraceSubstances traceSubstance3(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{36, -76},{56, -56}})));
          Modelica.Fluid.Pipes.StaticPipe pipe1(diameter = 0.1, redeclare package Medium = Medium, length = 1) annotation(Placement(transformation(extent = {{50,10},{70,30}})));
          Modelica.Fluid.Pipes.StaticPipe pipe2(diameter = 0.1, redeclare package Medium = Medium, length = 1) annotation(Placement(transformation(extent = {{50, -30},{70, -10}})));
          Modelica.Fluid.Pipes.StaticPipe pipe3(diameter = 0.1, redeclare package Medium = Medium, length = 1, nParallel = 1) annotation(Placement(transformation(extent = {{52, -90},{72, -70}})));
          Modelica.Fluid.Fittings.MultiPort multiPort(nPorts_b = 2, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -36, -10},{ -28,10}}, rotation = 0)));
          Modelica.Fluid.Sensors.TraceSubstances traceSubstance(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -54,18},{ -34,38}})));
          Modelica.Fluid.Pipes.StaticPipe pipe4(diameter = 0.1, redeclare package Medium = Medium, length = 1) annotation(Placement(transformation(extent = {{ -16,10},{4,30}})));
          Modelica.Fluid.Pipes.StaticPipe pipe5(diameter = 0.1, redeclare package Medium = Medium, length = 1) annotation(Placement(transformation(extent = {{ -16, -30},{4, -10}})));
          Modelica.Fluid.Pipes.StaticPipe pipe6(diameter = 0.1, redeclare package Medium = Medium, length = 1, nParallel = 1) annotation(Placement(transformation(extent = {{ -16, -90},{4, -70}})));

        equation 
          connect(P.y,source1.p_in) annotation(Line(points = {{ -79,34},{ -74,34},{ -74, -10},{ -70, -10}}, color = {0,0,127}));
          connect(C.y,source1.C_in[1]) annotation(Line(points = {{ -79, -50},{ -76, -50},{ -76, -26},{ -70, -26}}, color = {0,0,127}, smooth = Smooth.None));
          connect(X.y,source1.X_in[1]) annotation(Line(points = {{ -79,2},{ -76,2},{ -76, -22},{ -70, -22}}, color = {0,0,127}, smooth = Smooth.None));
          connect(X2.y,source1.X_in[2]) annotation(Line(points = {{ -79, -22},{ -70, -22}}, color = {0,0,127}, smooth = Smooth.None));
          connect(volume1.ports[3],traceSubstance1.port) annotation(Line(points = {{24.6667,20},{34,20},{34,24},{46,24}}, color = {0,127,255}, smooth = Smooth.None));
          connect(volume2.ports[3],traceSubstance2.port) annotation(Line(points = {{24.6667, -20},{34, -20},{34, -16},{44, -16}}, color = {0,127,255}, smooth = Smooth.None));
          connect(volume3.ports[3],traceSubstance3.port) annotation(Line(points = {{24.6667, -80},{25, -80},{25, -76},{46, -76}}, color = {0,127,255}, smooth = Smooth.None));
          connect(volume1.ports[2],pipe1.port_a) annotation(Line(points = {{22,20},{50,20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(volume2.ports[2],pipe2.port_a) annotation(Line(points = {{22, -20},{50, -20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(volume3.ports[2],pipe3.port_a) annotation(Line(points = {{22, -80},{52, -80}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipe1.port_b,source2.ports[1]) annotation(Line(points = {{70,20},{70, -6},{80, -6},{80, -22.6667}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipe2.port_b,source2.ports[2]) annotation(Line(points = {{70, -20},{80, -20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipe3.port_b,source2.ports[3]) annotation(Line(points = {{72, -80},{76, -80},{76, -17.3333},{80, -17.3333}}, color = {0,127,255}, smooth = Smooth.None));
          connect(source1.ports[2],traceSubstance.port) annotation(Line(points = {{ -48, -18},{ -44, -18},{ -44,18}}, color = {0,127,255}, smooth = Smooth.None));
          connect(source1.ports[1],multiPort.port_a) annotation(Line(points = {{ -48, -15.3333},{ -44, -15.3333},{ -44,0},{ -36,0}}, color = {0,127,255}, smooth = Smooth.None));
          connect(source1.ports[3],pipe6.port_a) annotation(Line(points = {{ -48, -20.6667},{ -32, -20.6667},{ -32, -80},{ -16, -80}}, color = {0,127,255}, smooth = Smooth.None));
          connect(multiPort.ports_b[1],pipe4.port_a) annotation(Line(points = {{ -28,2},{ -22,2},{ -22,20},{ -16,20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(multiPort.ports_b[2],pipe5.port_a) annotation(Line(points = {{ -28, -2},{ -22, -2},{ -22, -20},{ -16, -20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipe4.port_b,volume1.ports[1]) annotation(Line(points = {{4,20},{19.3333,20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipe5.port_b,volume2.ports[1]) annotation(Line(points = {{4, -20},{19.3333, -20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipe6.port_b,volume3.ports[1]) annotation(Line(points = {{4, -80},{19.3333, -80}}, color = {0,127,255}, smooth = Smooth.None));
        end TestMultiPortTraceSubstances;
        model TestMultiPort
          import Modelica.Fluid;
          extends Modelica.Icons.Example;
          Modelica.Fluid.Fittings.MultiPort multiPort(nPorts_b = 2, redeclare package Medium = Modelica.Media.Air.MoistAir) annotation(Placement(transformation(extent = {{ -28, -30},{ -20, -10}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          Modelica.Fluid.Sources.Boundary_pT source2(nPorts = 1, T = 278.15, p = 500000.0, redeclare package Medium = Modelica.Media.Air.MoistAir) annotation(Placement(transformation(origin = {80, -20}, extent = {{ -10, -10},{10,10}}, rotation = 180)));
          Modelica.Fluid.Sources.Boundary_pT source3(nPorts = 1, T = 283.15, p = 200000.0, redeclare package Medium = Modelica.Media.Air.MoistAir) annotation(Placement(transformation(origin = {10,70}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100,80},{ -80,100}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT source1(nPorts = 1, T = system.T_ambient, use_p_in = true, redeclare package Medium = Modelica.Media.Air.MoistAir, p = 500000) annotation(Placement(transformation(extent = {{ -60, -30},{ -40, -10}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp ramp(duration = 1, height =  -650000.0, offset = 700000.0) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 270, origin = { -80,10})));
          Modelica.Fluid.Pipes.StaticPipe pipe1(length = 1, diameter = 0.1, redeclare package Medium = Modelica.Media.Air.MoistAir) annotation(Placement(transformation(extent = {{40, -30},{60, -10}}, rotation = 0)));
          Modelica.Fluid.Pipes.StaticPipe pipe2(length = 1, diameter = 0.1, redeclare package Medium = Modelica.Media.Air.MoistAir) annotation(Placement(transformation(origin = {10,24}, extent = {{ -10, -10},{10,10}}, rotation = 90)));

        equation 
          connect(ramp.y,source1.p_in) annotation(Line(points = {{ -80, -1},{ -80, -1},{ -80, -12},{ -62, -12}}, color = {0,0,127}));
          connect(pipe1.port_b,source2.ports[1]) annotation(Line(points = {{60, -20},{70, -20}}, color = {0,127,255}));
          connect(pipe2.port_b,source3.ports[1]) annotation(Line(points = {{10,34},{10,47},{10,60}}, color = {0,127,255}));
          connect(multiPort.ports_b[1],pipe2.port_a) annotation(Line(points = {{ -20, -18},{10, -18},{10,14}}, color = {0,127,255}, smooth = Smooth.None));
          connect(multiPort.ports_b[2],pipe1.port_a) annotation(Line(points = {{ -20, -22},{35, -22},{35, -20},{40, -20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(source1.ports[1],multiPort.port_a) annotation(Line(points = {{ -40, -20},{ -28, -20}}, color = {0,127,255}, smooth = Smooth.None));
        end TestMultiPort;
        model TestJunctionVolume
          extends Modelica.Icons.Example;
          Modelica.Fluid.Fittings.TeeJunctionVolume junction(redeclare package Medium = Modelica.Media.Air.DryAirNasa, V = 2e-05, energyDynamics = Modelica.Fluid.Types.Dynamics.SteadyStateInitial, massDynamics = Modelica.Fluid.Types.Dynamics.SteadyStateInitial, p_start = 100000) annotation(Placement(transformation(extent = {{20, -30},{40, -10}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
          Modelica.Fluid.Sources.Boundary_pT source2(nPorts = 1, T = 278.15, p = 500000.0, redeclare package Medium = Modelica.Media.Air.DryAirNasa) annotation(Placement(transformation(origin = {90, -20}, extent = {{ -10, -10},{10,10}}, rotation = 180)));
          Modelica.Fluid.Sources.Boundary_pT source3(nPorts = 1, T = 283.15, p = 200000.0, redeclare package Medium = Modelica.Media.Air.DryAirNasa) annotation(Placement(transformation(origin = {30,70}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100,80},{ -80,100}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT source1(nPorts = 1, p = 500000.0, redeclare package Medium = Modelica.Media.Air.DryAirNasa, T = system.T_ambient, use_p_in = true) annotation(Placement(transformation(extent = {{ -40, -30},{ -20, -10}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp ramp(duration = 1, height =  -650000.0, offset = 700000.0) annotation(Placement(transformation(extent = {{ -90, -24},{ -70, -4}}, rotation = 0)));
          Modelica.Fluid.Pipes.StaticPipe pipe(redeclare package Medium = Modelica.Media.Air.DryAirNasa, length = 1, diameter = 0.1) annotation(Placement(transformation(extent = {{ -12, -30},{8, -10}}, rotation = 0)));
          Modelica.Fluid.Pipes.StaticPipe pipe1(redeclare package Medium = Modelica.Media.Air.DryAirNasa, length = 1, diameter = 0.1) annotation(Placement(transformation(extent = {{50, -30},{70, -10}}, rotation = 0)));
          Modelica.Fluid.Pipes.StaticPipe pipe2(redeclare package Medium = Modelica.Media.Air.DryAirNasa, length = 1, diameter = 0.1) annotation(Placement(transformation(origin = {30,24}, extent = {{ -10, -10},{10,10}}, rotation = 90)));

        equation 
          connect(ramp.y,source1.p_in) annotation(Line(points = {{ -69, -14},{ -55.5, -14},{ -55.5, -12},{ -42, -12}}, color = {0,0,127}));
          connect(source1.ports[1],pipe.port_a) annotation(Line(points = {{ -20, -20},{ -12, -20}}, color = {0,127,255}));
          connect(pipe.port_b,junction.port_1) annotation(Line(points = {{8, -20},{20, -20}}, color = {0,127,255}));
          connect(pipe1.port_b,source2.ports[1]) annotation(Line(points = {{70, -20},{80, -20}}, color = {0,127,255}));
          connect(junction.port_2,pipe1.port_a) annotation(Line(points = {{40, -20},{50, -20}}, color = {0,127,255}));
          connect(pipe2.port_b,source3.ports[1]) annotation(Line(points = {{30,34},{30,47},{30,60},{30,60}}, color = {0,127,255}));
          connect(pipe2.port_a,junction.port_3) annotation(Line(points = {{30,14},{30, -10}}, color = {0,127,255}));
        end TestJunctionVolume;
        model TestJunctionTraceSubstances
          import Modelica.Fluid;
          extends Modelica.Icons.Example;
          package Medium = Modelica.Media.Air.MoistAir(extraPropertiesNames = {"CO2"});
          Modelica.Fluid.Fittings.TeeJunctionIdeal junction(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{0, -30},{20, -10}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          Modelica.Fluid.Sources.Boundary_pT source2(nPorts = 1, T = 278.15, redeclare package Medium = Medium, p = 100000, X = Medium.X_default) annotation(Placement(transformation(origin = {90, -20}, extent = {{ -10, -10},{10,10}}, rotation = 180)));
          Modelica.Fluid.Sources.Boundary_pT source3(nPorts = 1, T = 283.15, redeclare package Medium = Medium, p = 100000, X = Medium.X_default) annotation(Placement(transformation(origin = { -30,70}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100,80},{ -80,100}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT source1(T = system.T_ambient, use_p_in = true, redeclare package Medium = Medium, use_C_in = true, use_X_in = true, nPorts = 1, p = 500000) annotation(Placement(transformation(extent = {{ -68, -28},{ -48, -8}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp ramp(duration = 1, height =  -40, offset = 100000.0 + 20) annotation(Placement(transformation(extent = {{ -100,24},{ -80,44}}, rotation = 0)));
          Modelica.Fluid.Pipes.StaticPipe pipe1(length = 1, diameter = 0.1, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -24, -30},{ -4, -10}}, rotation = 0)));
          Modelica.Fluid.Pipes.StaticPipe pipe2(length = 1, diameter = 0.1, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{56, -30},{76, -10}}, rotation = 0)));
          Modelica.Fluid.Pipes.StaticPipe pipe3(length = 1, diameter = 0.1, redeclare package Medium = Medium) annotation(Placement(transformation(origin = { -30,30}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Fluid.Fittings.TeeJunctionVolume junction1(redeclare package Medium = Medium, V = 0.1, C_start = {0.0001}, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, X_start = Medium.X_default) annotation(Placement(transformation(extent = {{30, -30},{50, -10}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT source4(nPorts = 1, T = 283.15, redeclare package Medium = Medium, p = 100000, X = Medium.X_default) annotation(Placement(transformation(origin = {10,70}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Fluid.Pipes.StaticPipe pipe4(length = 1, diameter = 0.1, redeclare package Medium = Medium) annotation(Placement(transformation(origin = {10,30}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Fluid.Sources.Boundary_pT source5(nPorts = 1, T = 283.15, redeclare package Medium = Medium, p = 100000, X = Medium.X_default) annotation(Placement(transformation(origin = {40,70}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Fluid.Pipes.StaticPipe pipe5(length = 1, diameter = 0.1, redeclare package Medium = Medium) annotation(Placement(transformation(origin = {40,30}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Fluid.Sensors.TraceSubstances traceSubstance2(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{56,4},{76,24}})));
          Modelica.Fluid.Sensors.TraceSubstances traceSubstance(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -70,16},{ -50,36}})));
          Modelica.Blocks.Sources.Ramp C(duration = 1, height = 0.001519) "substance concentration, raising to 1000 PPM CO2" annotation(Placement(transformation(extent = {{ -100, -60},{ -80, -40}})));
          Modelica.Blocks.Sources.Constant X(k = 0.02) annotation(Placement(transformation(extent = {{ -100, -8},{ -80,12}})));
          Modelica.Blocks.Sources.RealExpression X2(y = 1 - X.y) "Concentration of X[2]" annotation(Placement(transformation(extent = {{ -100, -32},{ -80, -12}})));
          Modelica.Fluid.Fittings.MultiPort multiPort(redeclare package Medium = Medium, nPorts_b = 2) annotation(Placement(transformation(extent = {{ -42, -28},{ -34, -8}})));

        equation 
          connect(ramp.y,source1.p_in) annotation(Line(points = {{ -79,34},{ -74,34},{ -74, -10},{ -70, -10}}, color = {0,0,127}));
          connect(pipe1.port_b,junction.port_1) annotation(Line(points = {{ -4, -20},{0, -20}}, color = {0,127,255}));
          connect(pipe2.port_b,source2.ports[1]) annotation(Line(points = {{76, -20},{76, -20},{80, -20}}, color = {0,127,255}));
          connect(junction.port_2,junction1.port_1) annotation(Line(points = {{20, -20},{30, -20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(junction1.port_2,pipe2.port_a) annotation(Line(points = {{50, -20},{56, -20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipe3.port_b,source3.ports[1]) annotation(Line(points = {{ -30,40},{ -30,60}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipe4.port_a,junction.port_3) annotation(Line(points = {{10,20},{10, -10}}, color = {0,127,255}, smooth = Smooth.None));
          connect(source4.ports[1],pipe4.port_b) annotation(Line(points = {{10,60},{10,40}}, color = {0,127,255}, smooth = Smooth.None));
          connect(source5.ports[1],pipe5.port_b) annotation(Line(points = {{40,60},{40,40}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipe5.port_a,junction1.port_3) annotation(Line(points = {{40,20},{40, -10}}, color = {0,127,255}, smooth = Smooth.None));
          connect(traceSubstance.port,pipe1.port_a) annotation(Line(points = {{ -60,16},{ -28,16},{ -28, -20},{ -24, -20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(C.y,source1.C_in[1]) annotation(Line(points = {{ -79, -50},{ -76, -50},{ -76, -26},{ -70, -26}}, color = {0,0,127}, smooth = Smooth.None));
          connect(pipe2.port_a,traceSubstance2.port) annotation(Line(points = {{56, -20},{54, -20},{54,4},{66,4}}, color = {0,127,255}, smooth = Smooth.None));
          connect(X.y,source1.X_in[1]) annotation(Line(points = {{ -79,2},{ -76,2},{ -76, -22},{ -70, -22}}, color = {0,0,127}, smooth = Smooth.None));
          connect(X2.y,source1.X_in[2]) annotation(Line(points = {{ -79, -22},{ -70, -22}}, color = {0,0,127}, smooth = Smooth.None));
          connect(source1.ports[1],multiPort.port_a) annotation(Line(points = {{ -48, -18},{ -42, -18}}, color = {0,127,255}, smooth = Smooth.None));
          connect(multiPort.ports_b[1],pipe3.port_a) annotation(Line(points = {{ -34, -16},{ -30, -16},{ -30,20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(multiPort.ports_b[2],pipe1.port_a) annotation(Line(points = {{ -34, -20},{ -24, -20}}, color = {0,127,255}, smooth = Smooth.None));
        end TestJunctionTraceSubstances;
        model TestJunctionIdeal
          extends Modelica.Icons.Example;
          Modelica.Fluid.Fittings.TeeJunctionIdeal junction(redeclare package Medium = Modelica.Media.Air.DryAirNasa) annotation(Placement(transformation(extent = {{20, -30},{40, -10}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
          Modelica.Fluid.Sources.Boundary_pT source2(nPorts = 1, T = 278.15, p = 500000.0, redeclare package Medium = Modelica.Media.Air.DryAirNasa) annotation(Placement(transformation(origin = {90, -20}, extent = {{ -10, -10},{10,10}}, rotation = 180)));
          Modelica.Fluid.Sources.Boundary_pT source3(nPorts = 1, T = 283.15, p = 200000.0, redeclare package Medium = Modelica.Media.Air.DryAirNasa) annotation(Placement(transformation(origin = {30,70}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100,80},{ -80,100}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT source1(nPorts = 1, p = 500000.0, redeclare package Medium = Modelica.Media.Air.DryAirNasa, T = system.T_ambient, use_p_in = true) annotation(Placement(transformation(extent = {{ -40, -30},{ -20, -10}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp ramp(duration = 1, height =  -650000.0, offset = 700000.0) annotation(Placement(transformation(extent = {{ -90, -24},{ -70, -4}}, rotation = 0)));
          Modelica.Fluid.Pipes.StaticPipe pipe1(redeclare package Medium = Modelica.Media.Air.DryAirNasa, length = 1, diameter = 0.1) annotation(Placement(transformation(extent = {{ -12, -30},{8, -10}}, rotation = 0)));
          Modelica.Fluid.Pipes.StaticPipe pipe2(redeclare package Medium = Modelica.Media.Air.DryAirNasa, length = 1, diameter = 0.1) annotation(Placement(transformation(extent = {{50, -30},{70, -10}}, rotation = 0)));
          Modelica.Fluid.Pipes.StaticPipe pipe3(redeclare package Medium = Modelica.Media.Air.DryAirNasa, length = 1, diameter = 0.1) annotation(Placement(transformation(origin = {30,24}, extent = {{ -10, -10},{10,10}}, rotation = 90)));

        equation 
          connect(ramp.y,source1.p_in) annotation(Line(points = {{ -69, -14},{ -55.5, -14},{ -55.5, -12},{ -42, -12}}, color = {0,0,127}));
          connect(source1.ports[1],pipe1.port_a) annotation(Line(points = {{ -20, -20},{ -12, -20}}, color = {0,127,255}));
          connect(pipe1.port_b,junction.port_1) annotation(Line(points = {{8, -20},{20, -20}}, color = {0,127,255}));
          connect(pipe2.port_b,source2.ports[1]) annotation(Line(points = {{70, -20},{80, -20}}, color = {0,127,255}));
          connect(junction.port_2,pipe2.port_a) annotation(Line(points = {{40, -20},{50, -20}}, color = {0,127,255}));
          connect(pipe3.port_b,source3.ports[1]) annotation(Line(points = {{30,34},{30,47},{30,60},{30,60}}, color = {0,127,255}));
          connect(pipe3.port_a,junction.port_3) annotation(Line(points = {{30,14},{30, -10}}, color = {0,127,255}));
        end TestJunctionIdeal;
      end Fittings;
    end TestComponents;
    package TestOverdeterminedInitial "Contains test cases to test overdetermined systems of initial equations"
      model DynamicPipeLumpedPressureInitialization "Steady-state initialization of a dynamic pipe using lumped pressure states"
        Modelica.Fluid.Sources.FixedBoundary source(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, use_T = false, p = 10000000, h = 2000000.0) annotation(Placement(transformation(extent = {{ -80, -10},{ -60,10}})));
        Modelica.Fluid.Pipes.DynamicPipe pipe(redeclare package Medium = Modelica.Media.Water.StandardWater, h_start = 2000000.0, diameter = 0.05, length = 200, use_T_start = false, useLumpedPressure = true, nNodes = 5, p_a_start = 10000000, p_b_start = 9900000, modelStructure = Modelica.Fluid.Types.ModelStructure.a_vb) annotation(Placement(transformation(extent = {{ -40, -10},{ -20,10}})));
        Modelica.Fluid.Valves.ValveCompressible valve(redeclare package Medium = Modelica.Media.Water.StandardWater, Av = 0.001, dp_nominal = 10000000, m_flow_nominal = 10) annotation(Placement(transformation(extent = {{0, -10},{20,10}})));
        Modelica.Fluid.Sources.FixedBoundary sink(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, p = 9500000) annotation(Placement(transformation(extent = {{60, -10},{40,10}})));
        Modelica.Blocks.Sources.Ramp ramp(offset = 1, duration = 0.1, height =  -0.5, startTime = 2) annotation(Placement(transformation(extent = {{46,30},{26,50}})));
        inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.SteadyStateInitial) annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}})));
        discrete Modelica.SIunits.MassFlowRate m_flow_initial;

      equation 
        when time > 0.1 then
                  m_flow_initial = valve.port_a.m_flow;
        
        end when;
if pipe.energyDynamics >= Modelica.Fluid.Types.Dynamics.SteadyStateInitial and pipe.massDynamics >= Modelica.Fluid.Types.Dynamics.SteadyStateInitial then
                  when time > 1 then
                      assert(abs(valve.port_a.m_flow - m_flow_initial) < 0.001, "!!!THE SIMULATION DID NOT START IN STEADY-STATE!!!");
          
          end when;
        end if;
        connect(source.ports[1],pipe.port_a) annotation(Line(points = {{ -60,0},{ -40,0}}, color = {0,127,255}, smooth = Smooth.None));
        connect(pipe.port_b,valve.port_a) annotation(Line(points = {{ -20,0},{0,0}}, color = {0,127,255}, smooth = Smooth.None));
        connect(valve.port_b,sink.ports[1]) annotation(Line(points = {{20,0},{40,0}}, color = {0,127,255}, smooth = Smooth.None));
        connect(ramp.y,valve.opening) annotation(Line(points = {{25,40},{10,40},{10,8}}, color = {0,0,127}, smooth = Smooth.None));
        annotation(Documentation(info = "<html>
All pressure states of the pipe are lumped into one.
The steady-state initial conditions become overdetermined as they are now specified nNodes times for the same pressure state.
The initial equations are consistent however and a tool shall reduce them appropriately.
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -100, -20},{100, -40}}, lineColor = {0,0,255}, textString = "Problem: pipe.medium.p[1:5] are equal and have initial equations der(medium.p)=zeros(5);"),Text(extent = {{ -76, -40},{80, -58}}, lineColor = {0,0,255}, textString = "A translator should remove consistently overdetermined initial equations.")}), experiment(StopTime = 4), experimentSetupOutput);
      end DynamicPipeLumpedPressureInitialization;
      model DynamicPipeInitialValues "Initialization of a dynamic pipe with fixed initial values and without adaptation of modelStructure to boundaries"
        Modelica.Fluid.Sources.FixedBoundary source(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, use_T = false, h = 2000000.0, p = 10000000) annotation(Placement(transformation(extent = {{ -80, -10},{ -60,10}})));
        Modelica.Fluid.Pipes.DynamicPipe pipe(redeclare package Medium = Modelica.Media.Water.StandardWater, h_start = 2000000.0, diameter = 0.05, length = 200, use_T_start = false, nNodes = 5, modelStructure = Modelica.Fluid.Types.ModelStructure.av_vb, p_a_start = 10000000, p_b_start = 9900000) annotation(Placement(transformation(extent = {{ -40, -10},{ -20,10}})));
        Modelica.Fluid.Valves.ValveCompressible valve(redeclare package Medium = Modelica.Media.Water.StandardWater, Av = 0.001, dp_nominal = 10000000, m_flow_nominal = 10) annotation(Placement(transformation(extent = {{0, -10},{20,10}})));
        Modelica.Fluid.Sources.FixedBoundary sink(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, p = 9500000) annotation(Placement(transformation(extent = {{60, -10},{40,10}})));
        Modelica.Blocks.Sources.Ramp ramp(offset = 1, duration = 0.1, height =  -0.5, startTime = 2) annotation(Placement(transformation(extent = {{46,30},{26,50}})));
        inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial) annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}})));
        discrete Modelica.SIunits.MassFlowRate m_flow_initial;

      equation 
        when time > 0.1 then
                  m_flow_initial = valve.port_a.m_flow;
        
        end when;
if pipe.energyDynamics >= Modelica.Fluid.Types.Dynamics.SteadyStateInitial and pipe.massDynamics >= Modelica.Fluid.Types.Dynamics.SteadyStateInitial then
                  when time > 1 then
                      assert(abs(valve.port_a.m_flow - m_flow_initial) < 0.001, "!!!THE SIMULATION DID NOT START IN STEADY-STATE!!!");
          
          end when;
        end if;
        connect(source.ports[1],pipe.port_a) annotation(Line(points = {{ -60,0},{ -40,0}}, color = {0,127,255}, smooth = Smooth.None));
        connect(pipe.port_b,valve.port_a) annotation(Line(points = {{ -20,0},{0,0}}, color = {0,127,255}, smooth = Smooth.None));
        connect(valve.port_b,sink.ports[1]) annotation(Line(points = {{20,0},{40,0}}, color = {0,127,255}, smooth = Smooth.None));
        connect(ramp.y,valve.opening) annotation(Line(points = {{25,40},{10,40},{10,8}}, color = {0,0,127}, smooth = Smooth.None));
        annotation(Documentation(info = "<html>
The initial values are overdetermined as the first pipe segment is directly connected to a source with fixed pressure.
The initial equations are consistent however and a tool shall reduce them appropriately.
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -100, -20},{100, -40}}, lineColor = {0,0,255}, textString = "Problem: pipe.medium[1].p is equal to source.p and  has a consistent initial value  of 100 bar;"),Text(extent = {{ -76, -36},{76, -54}}, lineColor = {0,0,255}, textString = "A translator should remove consistently overdetermined initial equations."),Text(extent = {{ -100, -64},{90, -84}}, lineColor = {0,0,255}, textString = "Work-around 2: change system.energyDynamics from FixedInitial to DynamicFreeInitial"),Text(extent = {{ -100, -54},{42, -74}}, lineColor = {0,0,255}, textString = "Work-around 1: change pipe.modelStructure from av_vb to a_vb")}), experiment(StopTime = 4), experimentSetupOutput);
      end DynamicPipeInitialValues;
    end TestOverdeterminedInitial;
    package TestCriticalCases "Collection of test cases which might be critical for the solvers"
      model IdealMixing1 "Test properties of ideal mixing"
        package Medium = Modelica.Media.IdealGases.MixtureGases.FlueGasSixComponents;
        Modelica.Fluid.Pipes.StaticPipe pipeFriction1(length = 1, diameter = 0.2, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -32, -40},{ -12, -20}}, rotation = 0)));
        Modelica.Fluid.Pipes.StaticPipe pipeFriction2(length = 1, diameter = 0.2, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{12, -40},{32, -20}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 10), experimentSetupOutput);
        Modelica.Fluid.Pipes.StaticPipe pipeFriction3(length = 1, diameter = 0.2, redeclare package Medium = Medium) annotation(Placement(transformation(origin = {0,0}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Fluid.Sources.Boundary_pT boundary1(nPorts = 1, use_p_in = true, use_T_in = true, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -68, -40},{ -48, -20}}, rotation = 0)));
        Modelica.Fluid.Sources.Boundary_pT boundary2(nPorts = 1, use_p_in = false, use_T_in = false, p = 101000, T = 320, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{66, -40},{46, -20}}, rotation = 0)));
        Modelica.Fluid.Sources.Boundary_pT boundary3(nPorts = 1, use_p_in = true, use_T_in = false, T = 340, redeclare package Medium = Medium) annotation(Placement(transformation(origin = {0,30}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
        Modelica.Blocks.Sources.Sine sine1(amplitude = 5000.0, freqHz = 2, offset = 100000.0, phase = 0.013962634015955) annotation(Placement(transformation(extent = {{ -100, -20},{ -80,0}}, rotation = 0)));
        Modelica.Blocks.Sources.Sine sine2(amplitude = 10, freqHz = 1, phase = 0.0017453292519943, offset = 300) annotation(Placement(transformation(extent = {{ -100, -58},{ -80, -38}}, rotation = 0)));
        Modelica.Blocks.Sources.Sine sine3(amplitude = 5000.0, freqHz = 2, offset = 100000.0) annotation(Placement(transformation(origin = {0,70}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
        inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -88,60},{ -68,80}}, rotation = 0)));
        Modelica.Fluid.Sensors.Temperature temperature(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -10, -60},{10, -80}}, rotation = 0)));

      equation 
        connect(pipeFriction1.port_b,pipeFriction2.port_a) annotation(Line(points = {{ -12, -30},{12, -30}}, color = {0,127,255}, smooth = Smooth.None));
        connect(pipeFriction3.port_a,pipeFriction1.port_b) annotation(Line(points = {{ -6.12323e-16, -10},{0, -10},{0, -30},{ -12, -30}}, color = {0,127,255}, smooth = Smooth.None));
        connect(boundary1.ports[1],pipeFriction1.port_a) annotation(Line(points = {{ -48, -30},{ -32, -30}}, color = {0,127,255}, smooth = Smooth.None));
        connect(boundary2.ports[1],pipeFriction2.port_b) annotation(Line(points = {{46, -30},{32, -30}}, color = {0,127,255}, smooth = Smooth.None));
        connect(boundary3.ports[1],pipeFriction3.port_b) annotation(Line(points = {{ -1.9984e-15,20},{ -1.9984e-15,10},{6.12323e-16,10}}, color = {0,127,255}, smooth = Smooth.None));
        connect(sine1.y,boundary1.p_in) annotation(Line(points = {{ -79, -10},{ -76, -10},{ -76, -22},{ -70, -22}}, color = {0,0,127}, smooth = Smooth.None));
        connect(sine2.y,boundary1.T_in) annotation(Line(points = {{ -79, -48},{ -76, -48},{ -76, -26},{ -70, -26}}, color = {0,0,127}, smooth = Smooth.None));
        connect(sine3.y,boundary3.p_in) annotation(Line(points = {{ -6.73556e-16,59},{ -6.73556e-16,50.5},{ -8,50.5},{ -8,42}}, color = {0,0,127}, smooth = Smooth.None));
        connect(temperature.port,pipeFriction3.port_a) annotation(Line(points = {{0, -60},{0, -10},{ -6.12323e-16, -10}}, color = {0,127,255}, smooth = Smooth.None));
      end IdealMixing1;
      model DynamicPipeClosingValve "This test demonstrates the importance of smooth regularization of fluid properties for reversing flow.
 A DynamicPipe model with switching port densities and viscosities would generate lots of events as the valve closes at time 2."
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 3));
        Modelica.Fluid.Sources.FixedBoundary source(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, p = 200000) annotation(Placement(transformation(extent = {{ -80, -10},{ -60,10}})));
        Modelica.Fluid.Pipes.DynamicPipe pipe(redeclare package Medium = Modelica.Media.Water.StandardWater, length = 1, diameter = 0.32, use_T_start = false, p_a_start = 200000, p_b_start = 200000) annotation(Placement(transformation(extent = {{ -40, -10},{ -20,10}})));
        Modelica.Fluid.Valves.ValveIncompressible valve(redeclare package Medium = Modelica.Media.Water.StandardWater, m_flow_nominal = 10, Av = 0.001, dp_nominal = 100000) annotation(Placement(transformation(extent = {{0, -10},{20,10}})));
        Modelica.Fluid.Sources.FixedBoundary sink(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, p = 100000) annotation(Placement(transformation(extent = {{60, -10},{40,10}})));
        Modelica.Blocks.Sources.Ramp ramp(height =  -1, offset = 1, duration = 1, startTime = 1) annotation(Placement(transformation(extent = {{46,30},{26,50}})));
        inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.SteadyStateInitial) annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}})));

      equation 
        connect(source.ports[1],pipe.port_a) annotation(Line(points = {{ -60,0},{ -40,0}}, color = {0,127,255}, smooth = Smooth.None));
        connect(pipe.port_b,valve.port_a) annotation(Line(points = {{ -20,0},{0,0}}, color = {0,127,255}, smooth = Smooth.None));
        connect(valve.port_b,sink.ports[1]) annotation(Line(points = {{20,0},{40,0}}, color = {0,127,255}, smooth = Smooth.None));
        connect(ramp.y,valve.opening) annotation(Line(points = {{25,40},{10,40},{10,8}}, color = {0,0,127}, smooth = Smooth.None));
      end DynamicPipeClosingValve;
      model DynamicPipeInitialization "Steady-state initialization of a dynamic pipe"
        Modelica.Fluid.Sources.FixedBoundary source(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, use_T = false, p = 10000000, h = 2000000.0) annotation(Placement(transformation(extent = {{ -80, -10},{ -60,10}})));
        Modelica.Fluid.Pipes.DynamicPipe pipe(redeclare package Medium = Modelica.Media.Water.StandardWater, nNodes = 5, h_start = 2000000.0, diameter = 0.05, length = 200, use_T_start = false, p_a_start = 10000000, p_b_start = 9900000, modelStructure = Modelica.Fluid.Types.ModelStructure.a_vb) annotation(Placement(transformation(extent = {{ -40, -10},{ -20,10}})));
        Modelica.Fluid.Valves.ValveCompressible valve(redeclare package Medium = Modelica.Media.Water.StandardWater, Av = 0.001, dp_nominal = 10000000, m_flow_nominal = 10) annotation(Placement(transformation(extent = {{0, -10},{20,10}})));
        Modelica.Fluid.Sources.FixedBoundary sink(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, p = 9500000) annotation(Placement(transformation(extent = {{60, -10},{40,10}})));
        Modelica.Blocks.Sources.Ramp ramp(offset = 1, duration = 0.1, height =  -0.5, startTime = 2) annotation(Placement(transformation(extent = {{46,30},{26,50}})));
        inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.SteadyStateInitial) annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}})));
        discrete Modelica.SIunits.MassFlowRate m_flow_initial;

      equation 
        when time > 0.1 then
                  m_flow_initial = valve.port_a.m_flow;
        
        end when;
if pipe.energyDynamics >= Modelica.Fluid.Types.Dynamics.SteadyStateInitial and pipe.massDynamics >= Modelica.Fluid.Types.Dynamics.SteadyStateInitial then
                  when time > 1 then
                      assert(abs(valve.port_a.m_flow - m_flow_initial) < 0.001, "!!!THE SIMULATION DID NOT START IN STEADY-STATE!!!");
          
          end when;
        end if;
        connect(source.ports[1],pipe.port_a) annotation(Line(points = {{ -60,0},{ -40,0}}, color = {0,127,255}, smooth = Smooth.None));
        connect(pipe.port_b,valve.port_a) annotation(Line(points = {{ -20,0},{0,0}}, color = {0,127,255}, smooth = Smooth.None));
        connect(valve.port_b,sink.ports[1]) annotation(Line(points = {{20,0},{40,0}}, color = {0,127,255}, smooth = Smooth.None));
        connect(ramp.y,valve.opening) annotation(Line(points = {{25,40},{10,40},{10,8}}, color = {0,0,127}, smooth = Smooth.None));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 4), experimentSetupOutput, Documentation(info = "<html>
The simulation shall start in steady-state (see system.energyDynamics on the Assumptions tab).
This requires the solution of an equation system with complex non-linear expressions for the
pipe wall friction and compressible valve characteristics. A ValveCompressible defined with Modelica.Fluid.Utilities.regRoot2 for the treatment of flow reversal
(cf. Modelica.Fluid Stream beta3) leads to wrong initial values. It works with Modelica.Fluid.Utilities.regRoot and a smooth(0, ...) expression for flow reversal.
<p>
The steady-state initial values are observed by an assertion.
</p>
</html>"));
      end DynamicPipeInitialization;
      model BranchingPipes1
        replaceable package Medium = Modelica.Media.Water.StandardWater;
        Modelica.Fluid.Sources.Boundary_pT source(nPorts = 1, redeclare package Medium = Medium, p = 200000, T = 300) annotation(Placement(transformation(extent = {{ -100,0},{ -88,12}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe1(redeclare package Medium = Medium, use_T_start = true, length = 10, diameter = 0.0254, p_b_start = 100000, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow, p_a_start = 200000, T_start = 300) annotation(Placement(transformation(extent = {{ -72, -4},{ -52,16}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valve1(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, m_flow_nominal = 1, rho_nominal = 1000, dp_nominal = 200000) annotation(Placement(transformation(extent = {{10,36},{30,56}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valve2(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, m_flow_nominal = 1, rho_nominal = 1000, dp_nominal = 200000) annotation(Placement(transformation(extent = {{8, -50},{28, -30}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 5), experimentSetupOutput(equdistant = false), Documentation(info = "<html>
Simulation starts with both valves open. At t=1, valve 1 closes; at t=2 valve 2 closes, and the simulation fails.
</html>"));
        Modelica.Fluid.Sources.Boundary_pT sink(redeclare package Medium = Medium, nPorts = 2, p = 100000, T = 300) annotation(Placement(transformation(extent = {{74, -20},{62, -8}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe2(redeclare package Medium = Medium, use_T_start = true, length = 10, diameter = 0.0254, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow, p_a_start = 200000, p_b_start = 100000, T_start = 300) annotation(Placement(transformation(extent = {{ -40,36},{ -20,56}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe3(redeclare package Medium = Medium, use_T_start = true, length = 10, diameter = 0.0254, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow, p_a_start = 200000, p_b_start = 100000, T_start = 300) annotation(Placement(transformation(extent = {{ -40, -50},{ -20, -30}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening1(offset = 0, table = [0,1;1,1;1,0;100,0]) annotation(Placement(transformation(extent = {{ -20,70},{0,90}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening2(offset = 0, table = [0,1;2,1;2,0;100,0]) annotation(Placement(transformation(extent = {{ -20, -10},{0,10}}, rotation = 0)));
        inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, massDynamics = Modelica.Fluid.Types.Dynamics.SteadyStateInitial) annotation(Placement(transformation(extent = {{ -100,60},{ -80,80}}, rotation = 0)));

      equation 
        connect(source.ports[1],pipe1.port_a) annotation(Line(points = {{ -88,6},{ -72,6}}, color = {0,127,255}));
        connect(valve2.port_b,sink.ports[2]) annotation(Line(points = {{28, -40},{46, -40},{46, -15.2},{62, -15.2}}, color = {0,127,255}));
        connect(valve1.port_b,sink.ports[1]) annotation(Line(points = {{30,46},{46,46},{46, -12.8},{62, -12.8}}, color = {0,127,255}));
        connect(pipe3.port_b,valve2.port_a) annotation(Line(points = {{ -20, -40},{8, -40}}, color = {0,127,255}));
        connect(pipe2.port_b,valve1.port_a) annotation(Line(points = {{ -20,46},{10,46}}, color = {0,127,255}));
        connect(pipe2.port_a,pipe1.port_b) annotation(Line(points = {{ -40,46},{ -46,46},{ -46,6},{ -52,6}}, color = {0,127,255}));
        connect(pipe1.port_b,pipe3.port_a) annotation(Line(points = {{ -52,6},{ -46,6},{ -46, -40},{ -40, -40}}, color = {0,127,255}));
        connect(valveOpening1.y,valve1.opening) annotation(Line(points = {{1,80},{20,80},{20,54}}, color = {0,0,127}));
        connect(valveOpening2.y,valve2.opening) annotation(Line(points = {{1,0},{18,0},{18, -32}}, color = {0,0,127}));
      end BranchingPipes1;
      model BranchingPipes2
        replaceable package Medium = Modelica.Media.Water.StandardWater;
        Modelica.Fluid.Sources.Boundary_pT source(nPorts = 1, redeclare package Medium = Medium, p = 500000.0, T = 300) annotation(Placement(transformation(extent = {{ -100,0},{ -88,12}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe1(redeclare package Medium = Medium, use_T_start = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow, length = 10, diameter = 0.0254, p_a_start = 500000.0, p_b_start = 500000.0) annotation(Placement(transformation(extent = {{ -72, -4},{ -52,16}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valveIncompressible(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 400000.0, m_flow_nominal = 1, rho_nominal = 1000) annotation(Placement(transformation(extent = {{10,36},{30,56}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valveIncompressible1(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 400000.0, m_flow_nominal = 1, rho_nominal = 1000) annotation(Placement(transformation(extent = {{8, -50},{28, -30}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 5), experimentSetupOutput(equdistant = false), Documentation(info = "<html>
Simulation starts with both valves open. At t=1, valve 1 closes; at t=2 valve 2 closes, and the simulation fails.
</html>"));
        Modelica.Fluid.Sources.Boundary_pT sink(redeclare package Medium = Medium, nPorts = 2, p = 100000, T = 300) annotation(Placement(transformation(extent = {{74, -20},{62, -8}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe2(redeclare package Medium = Medium, use_T_start = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow, length = 10, diameter = 0.0254, p_a_start = 500000.0, p_b_start = 500000.0) annotation(Placement(transformation(extent = {{ -40,36},{ -20,56}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe3(redeclare package Medium = Medium, use_T_start = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow, length = 10, diameter = 0.0254, p_a_start = 500000.0, p_b_start = 500000.0) annotation(Placement(transformation(extent = {{ -40, -50},{ -20, -30}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening1(offset = 0, table = [0,0;1,0;1,1;100,1]) annotation(Placement(transformation(extent = {{ -20,70},{0,90}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening2(offset = 0, table = [0,0;2,0;2,1;100,1]) annotation(Placement(transformation(extent = {{ -20, -10},{0,10}}, rotation = 0)));
        inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100,60},{ -80,80}}, rotation = 0)));

      equation 
        connect(source.ports[1],pipe1.port_a) annotation(Line(points = {{ -88,6},{ -72,6}}, color = {0,127,255}));
        connect(valveIncompressible1.port_b,sink.ports[2]) annotation(Line(points = {{28, -40},{46, -40},{46, -15.2},{62, -15.2}}, color = {0,127,255}));
        connect(valveIncompressible.port_b,sink.ports[1]) annotation(Line(points = {{30,46},{46,46},{46, -12.8},{62, -12.8}}, color = {0,127,255}));
        connect(pipe3.port_b,valveIncompressible1.port_a) annotation(Line(points = {{ -20, -40},{8, -40}}, color = {0,127,255}));
        connect(pipe2.port_b,valveIncompressible.port_a) annotation(Line(points = {{ -20,46},{10,46}}, color = {0,127,255}));
        connect(pipe2.port_a,pipe1.port_b) annotation(Line(points = {{ -40,46},{ -46,46},{ -46,6},{ -52,6}}, color = {0,127,255}));
        connect(pipe1.port_b,pipe3.port_a) annotation(Line(points = {{ -52,6},{ -46,6},{ -46, -40},{ -40, -40}}, color = {0,127,255}));
        connect(valveOpening1.y,valveIncompressible.opening) annotation(Line(points = {{1,80},{20,80},{20,54}}, color = {0,0,127}));
        connect(valveOpening2.y,valveIncompressible1.opening) annotation(Line(points = {{1,0},{18,0},{18, -32}}, color = {0,0,127}));
      end BranchingPipes2;
      model BranchingPipes3
        replaceable package Medium = Modelica.Media.Water.StandardWater;
        Modelica.Fluid.Sources.Boundary_pT source(nPorts = 1, redeclare package Medium = Medium, p = 500000.0, T = 300) annotation(Placement(transformation(extent = {{ -100,0},{ -88,12}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe1(redeclare package Medium = Medium, p_a_start = 500000.0, p_b_start = 500000.0, use_T_start = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow, length = 10, diameter = 0.0254) annotation(Placement(transformation(extent = {{ -80, -4},{ -60,16}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valveIncompressible(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 400000.0, m_flow_nominal = 1, rho_nominal = 1000) annotation(Placement(transformation(extent = {{10,36},{30,56}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valveIncompressible1(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 400000.0, m_flow_nominal = 1, rho_nominal = 1000) annotation(Placement(transformation(extent = {{8, -50},{28, -30}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 5), experimentSetupOutput(equdistant = false), Documentation(info = "<html>
Simulation starts with both valves open. At t=1, valve 1 closes; at t=2 valve 2 closes, and the simulation fails.
</html>"));
        Modelica.Fluid.Sources.Boundary_pT sink(redeclare package Medium = Medium, nPorts = 2, p = 100000, T = 300) annotation(Placement(transformation(extent = {{74, -20},{62, -8}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe2(redeclare package Medium = Medium, p_a_start = 500000.0, p_b_start = 500000.0, use_T_start = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow, length = 10, diameter = 0.0254) annotation(Placement(transformation(extent = {{ -40,36},{ -20,56}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe3(redeclare package Medium = Medium, p_a_start = 500000.0, p_b_start = 500000.0, use_T_start = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow, length = 10, diameter = 0.0254) annotation(Placement(transformation(extent = {{ -40, -50},{ -20, -30}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening1(offset = 0, table = [0,0;1,0;1,1;100,1]) annotation(Placement(transformation(extent = {{ -20,70},{0,90}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening2(offset = 0, table = [0,0;2,0;2,1;100,1]) annotation(Placement(transformation(extent = {{ -20, -10},{0,10}}, rotation = 0)));
        inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100,60},{ -80,80}}, rotation = 0)));
        Modelica.Fluid.Fittings.TeeJunctionIdeal splitter(redeclare package Medium = Medium) annotation(Placement(transformation(origin = { -43,6}, extent = {{ -6, -7},{6,7}}, rotation = 90)));

      equation 
        connect(source.ports[1],pipe1.port_a) annotation(Line(points = {{ -88,6},{ -80,6}}, color = {0,127,255}));
        connect(valveIncompressible1.port_b,sink.ports[2]) annotation(Line(points = {{28, -40},{46, -40},{46, -15.2},{62, -15.2}}, color = {0,127,255}));
        connect(valveIncompressible.port_b,sink.ports[1]) annotation(Line(points = {{30,46},{46,46},{46, -12.8},{62, -12.8}}, color = {0,127,255}));
        connect(pipe3.port_b,valveIncompressible1.port_a) annotation(Line(points = {{ -20, -40},{8, -40}}, color = {0,127,255}));
        connect(pipe2.port_b,valveIncompressible.port_a) annotation(Line(points = {{ -20,46},{10,46}}, color = {0,127,255}));
        connect(valveOpening1.y,valveIncompressible.opening) annotation(Line(points = {{1,80},{20,80},{20,54}}, color = {0,0,127}));
        connect(valveOpening2.y,valveIncompressible1.opening) annotation(Line(points = {{1,0},{18,0},{18, -32}}, color = {0,0,127}));
        connect(pipe1.port_b,splitter.port_3) annotation(Line(points = {{ -60,6},{ -55,6},{ -55,6},{ -50,6}}, color = {0,127,255}));
        connect(pipe2.port_a,splitter.port_2) annotation(Line(points = {{ -40,46},{ -43,46},{ -43,12}}, color = {0,127,255}));
        connect(splitter.port_1,pipe3.port_a) annotation(Line(points = {{ -43,0},{ -43, -40.3},{ -40, -40.3},{ -40, -40}}, color = {0,127,255}));
      end BranchingPipes3;
      model BranchingPipes4
        replaceable package Medium = Modelica.Media.Water.StandardWater;
        Modelica.Fluid.Sources.Boundary_pT source(nPorts = 1, redeclare package Medium = Medium, p = 500000.0, T = 300) annotation(Placement(transformation(extent = {{ -100,0},{ -88,12}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe1(redeclare package Medium = Medium, use_T_start = true, length = 10, diameter = 0.0254, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow, p_a_start = 500000, p_b_start = 500000) annotation(Placement(transformation(extent = {{ -80, -4},{ -60,16}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valveIncompressible(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 400000.0, m_flow_nominal = 1, rho_nominal = 1000) annotation(Placement(transformation(extent = {{10,36},{30,56}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valveIncompressible1(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 400000.0, m_flow_nominal = 1, rho_nominal = 1000) annotation(Placement(transformation(extent = {{8, -50},{28, -30}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 5), experimentSetupOutput(equdistant = false), Documentation(info = "<html>
Uses dynamic splitter. Simulation starts with both valves open. At t=1, valve 1 closes; at t=2 valve 2 closes. The simulation fails at t=0 due to lack of initialization of the splitter state variables.
</html>"));
        Modelica.Fluid.Sources.Boundary_pT sink(redeclare package Medium = Medium, nPorts = 2, p = 100000, T = 300) annotation(Placement(transformation(extent = {{74, -20},{62, -8}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe2(redeclare package Medium = Medium, use_T_start = true, length = 10, diameter = 0.0254, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow, p_a_start = 500000, p_b_start = 500000) annotation(Placement(transformation(extent = {{ -40,36},{ -20,56}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe3(redeclare package Medium = Medium, use_T_start = true, length = 10, diameter = 0.0254, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow, p_a_start = 500000, p_b_start = 500000) annotation(Placement(transformation(extent = {{ -40, -50},{ -20, -30}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening1(offset = 0, table = [0,0;1,0;1,1;100,1]) annotation(Placement(transformation(extent = {{ -20,70},{0,90}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening2(offset = 0, table = [0,0;2,0;2,1;100,1]) annotation(Placement(transformation(extent = {{ -20, -10},{0,10}}, rotation = 0)));
        inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100,60},{ -80,80}}, rotation = 0)));
        Modelica.Fluid.Fittings.TeeJunctionVolume splitter(redeclare package Medium = Medium, V = 0.0002, energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, p_start = 500000) annotation(Placement(transformation(origin = { -43,6}, extent = {{ -6, -7},{6,7}}, rotation = 90)));

      equation 
        connect(source.ports[1],pipe1.port_a) annotation(Line(points = {{ -88,6},{ -80,6}}, color = {0,127,255}));
        connect(valveIncompressible1.port_b,sink.ports[2]) annotation(Line(points = {{28, -40},{46, -40},{46, -15.2},{62, -15.2}}, color = {0,127,255}));
        connect(valveIncompressible.port_b,sink.ports[1]) annotation(Line(points = {{30,46},{46,46},{46, -12.8},{62, -12.8}}, color = {0,127,255}));
        connect(pipe3.port_b,valveIncompressible1.port_a) annotation(Line(points = {{ -20, -40},{8, -40}}, color = {0,127,255}));
        connect(pipe2.port_b,valveIncompressible.port_a) annotation(Line(points = {{ -20,46},{10,46}}, color = {0,127,255}));
        connect(valveOpening1.y,valveIncompressible.opening) annotation(Line(points = {{1,80},{20,80},{20,54}}, color = {0,0,127}));
        connect(valveOpening2.y,valveIncompressible1.opening) annotation(Line(points = {{1,0},{18,0},{18, -32}}, color = {0,0,127}));
        connect(pipe1.port_b,splitter.port_3) annotation(Line(points = {{ -60,6},{ -55,6},{ -55,6},{ -50,6}}, color = {0,127,255}));
        connect(pipe2.port_a,splitter.port_2) annotation(Line(points = {{ -40,46},{ -43,46},{ -43,12}}, color = {0,127,255}));
        connect(splitter.port_1,pipe3.port_a) annotation(Line(points = {{ -43,0},{ -43, -40.3},{ -40, -40.3},{ -40, -40}}, color = {0,127,255}));
      end BranchingPipes4;
      model SeriesPipes1
        replaceable package Medium = Modelica.Media.Water.StandardWater;
        Modelica.Fluid.Sources.Boundary_pT source(nPorts = 1, redeclare package Medium = Medium, p = 500000, T = 300) annotation(Placement(transformation(extent = {{ -100, -6},{ -88,6}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe1(use_T_start = true, length = 10, diameter = 0.025, redeclare package Medium = Medium, p_a_start = 500000, p_b_start = 500000) annotation(Placement(transformation(extent = {{ -76, -10},{ -56,10}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valveIncompressible(CvData = Modelica.Fluid.Types.CvTypes.OpPoint, m_flow_nominal = 1, rho_nominal = 1000, redeclare package Medium = Medium, dp_nominal = 400000) annotation(Placement(transformation(extent = {{52, -10},{72,10}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 5), experimentSetupOutput(equdistant = false), Documentation(info = "<html>
Simulation starts with the valve open. At t=1, the valve is closed, and the simulation fails.
</html>"));
        Modelica.Fluid.Sources.Boundary_pT sink(nPorts = 1, redeclare package Medium = Medium, p = 100000, T = 300) annotation(Placement(transformation(extent = {{94, -6},{82,6}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe2(use_T_start = true, length = 10, diameter = 0.025, redeclare package Medium = Medium, p_a_start = 500000, p_b_start = 500000) annotation(Placement(transformation(extent = {{ -14, -10},{6,10}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening1(offset = 0, table = [0,1;1,1;1,0;100,0]) annotation(Placement(transformation(extent = {{ -20,70},{0,90}}, rotation = 0)));
        inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial) annotation(Placement(transformation(extent = {{ -90,70},{ -70,90}}, rotation = 0)));
        Modelica.Fluid.Fittings.SimpleGenericOrifice simpleGenericOrifice(zeta = 0.4, diameter = 0.025, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -46, -10},{ -26,10}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe3(use_T_start = true, length = 10, diameter = 0.025, redeclare package Medium = Medium, p_a_start = 500000, p_b_start = 500000) annotation(Placement(transformation(extent = {{16, -10},{36,10}}, rotation = 0)));

      equation 
        connect(source.ports[1],pipe1.port_a) annotation(Line(points = {{ -88,0},{ -76,0}}, color = {0,127,255}));
        connect(valveIncompressible.port_b,sink.ports[1]) annotation(Line(points = {{72,0},{82,0}}, color = {0,127,255}));
        connect(valveOpening1.y,valveIncompressible.opening) annotation(Line(points = {{1,80},{62,80},{62,8}}, color = {0,0,127}));
        connect(pipe1.port_b,simpleGenericOrifice.port_a) annotation(Line(points = {{ -56,0},{ -46,0}}, color = {0,127,255}));
        connect(pipe2.port_a,simpleGenericOrifice.port_b) annotation(Line(points = {{ -14,0},{ -26,0}}, color = {0,127,255}));
        connect(pipe2.port_b,pipe3.port_a) annotation(Line(points = {{6,0},{16,0}}, color = {0,127,255}));
        connect(pipe3.port_b,valveIncompressible.port_a) annotation(Line(points = {{36,0},{52,0}}, color = {0,127,255}));
      end SeriesPipes1;
      model SeriesPipes12
        extends SeriesPipes1(system(energyDynamics = Modelica.Fluid.Types.Dynamics.SteadyStateInitial));

      equation 
        annotation(Documentation(info = "<html>
Same as SeriesPipes1, but with steady-state initial conditions. Equal start attributes
for pressures. Initialization fails.
</html>"));
      end SeriesPipes12;
      model SeriesPipes13
        extends SeriesPipes1(pipe2(p_a_start = 495000, p_b_start = 495000), pipe3(p_a_start = 490000, p_b_start = 490000));

      equation 
        annotation(Documentation(info = "<html>
Same as SeriesPipes1, but with steady-state initial conditions. Start attributes for
pressure in order to get positive flow rates. Initialization succeeds, then the simulation
fails for zero flow rate.
</html>"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
      end SeriesPipes13;
      model BranchingPipes12
        replaceable package Medium = Modelica.Media.Water.StandardWater;
        Modelica.Fluid.Sources.Boundary_pT source(nPorts = 1, redeclare package Medium = Medium, p = 500000.0, T = 300) annotation(Placement(transformation(extent = {{ -100,0},{ -88,12}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe1(redeclare package Medium = Medium, p_a_start = 500000.0, use_T_start = true, length = 10, diameter = 0.0254, p_b_start = 495000.0, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow) annotation(Placement(transformation(extent = {{ -78, -4},{ -58,16}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valveIncompressible(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 400000.0, m_flow_nominal = 1, rho_nominal = 1000) annotation(Placement(transformation(extent = {{10,36},{30,56}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valveIncompressible1(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 400000.0, m_flow_nominal = 1, rho_nominal = 1000) annotation(Placement(transformation(extent = {{8, -50},{28, -30}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 5), experimentSetupOutput(equdistant = false), Documentation(info = "<html>
Simulation starts with both valves open. At t=1, valve 1 closes; at t=2 valve 2 closes, and the simulation fails.
</html>"));
        Modelica.Fluid.Sources.Boundary_pT sink(redeclare package Medium = Medium, nPorts = 2, p = 100000, T = 300) annotation(Placement(transformation(extent = {{74, -20},{62, -8}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe2(redeclare package Medium = Medium, use_T_start = true, length = 10, diameter = 0.0254, p_a_start = 495000.0, p_b_start = 490000.0, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow) annotation(Placement(transformation(extent = {{ -40,36},{ -20,56}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe3(redeclare package Medium = Medium, use_T_start = true, length = 10, diameter = 0.0254, p_a_start = 495000.0, p_b_start = 490000.0, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow) annotation(Placement(transformation(extent = {{ -40, -50},{ -20, -30}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening1(offset = 0, table = [0,1;1,1;1,0;100,0]) annotation(Placement(transformation(extent = {{ -20,70},{0,90}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening2(offset = 0, table = [0,1;2,1;2,0;100,0]) annotation(Placement(transformation(extent = {{ -20, -10},{0,10}}, rotation = 0)));
        inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100,60},{ -80,80}}, rotation = 0)));

      equation 
        connect(source.ports[1],pipe1.port_a) annotation(Line(points = {{ -88,6},{ -78,6}}, color = {0,127,255}));
        connect(valveIncompressible1.port_b,sink.ports[2]) annotation(Line(points = {{28, -40},{46, -40},{46, -15.2},{62, -15.2}}, color = {0,127,255}));
        connect(valveIncompressible.port_b,sink.ports[1]) annotation(Line(points = {{30,46},{46,46},{46, -12.8},{62, -12.8}}, color = {0,127,255}));
        connect(pipe3.port_b,valveIncompressible1.port_a) annotation(Line(points = {{ -20, -40},{8, -40}}, color = {0,127,255}));
        connect(pipe2.port_b,valveIncompressible.port_a) annotation(Line(points = {{ -20,46},{10,46}}, color = {0,127,255}));
        connect(pipe2.port_a,pipe1.port_b) annotation(Line(points = {{ -40,46},{ -46,46},{ -46,6},{ -58,6}}, color = {0,127,255}));
        connect(pipe1.port_b,pipe3.port_a) annotation(Line(points = {{ -58,6},{ -46,6},{ -46, -40},{ -40, -40}}, color = {0,127,255}));
        connect(valveOpening1.y,valveIncompressible.opening) annotation(Line(points = {{1,80},{20,80},{20,54}}, color = {0,0,127}));
        connect(valveOpening2.y,valveIncompressible1.opening) annotation(Line(points = {{1,0},{18,0},{18, -32}}, color = {0,0,127}));
      end BranchingPipes12;
      model BranchingPipes13
        replaceable package Medium = Modelica.Media.Water.StandardWater;
        Modelica.Fluid.Sources.Boundary_pT source(nPorts = 1, redeclare package Medium = Medium, p = 500000.0, T = 300) annotation(Placement(transformation(extent = {{ -100,0},{ -88,12}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe1(redeclare package Medium = Medium, p_a_start = 500000.0, use_T_start = true, length = 10, diameter = 0.0254, p_b_start = 495000.0, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow) annotation(Placement(transformation(extent = {{ -78, -4},{ -58,16}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valveIncompressible(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, m_flow_nominal = 1, rho_nominal = 5, dp_nominal = 400000) annotation(Placement(transformation(extent = {{10,36},{30,56}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valveIncompressible1(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, m_flow_nominal = 1, rho_nominal = 5, dp_nominal = 400000) annotation(Placement(transformation(extent = {{8, -50},{28, -30}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 5), experimentSetupOutput(equdistant = false), Documentation(info = "<html>
Simulation starts with both valves open. At t=1, valve 1 closes; at t=2 valve 2 closes, and the simulation fails.
</html>"));
        Modelica.Fluid.Sources.Boundary_pT sink(redeclare package Medium = Medium, nPorts = 2, p = 100000, T = 300) annotation(Placement(transformation(extent = {{74, -20},{62, -8}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe2(redeclare package Medium = Medium, use_T_start = true, length = 10, diameter = 0.0254, p_a_start = 495000.0, p_b_start = 490000.0, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow) annotation(Placement(transformation(extent = {{ -34,36},{ -14,56}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe3(redeclare package Medium = Medium, use_T_start = true, length = 10, diameter = 0.0254, p_a_start = 495000.0, p_b_start = 490000.0, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow) annotation(Placement(transformation(extent = {{ -30, -50},{ -10, -30}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening1(offset = 0, table = [0,1;1,1;1,0;100,0]) annotation(Placement(transformation(extent = {{ -20,70},{0,90}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening2(offset = 0, table = [0,1;2,1;2,0.001;100,0]) annotation(Placement(transformation(extent = {{ -20, -10},{0,10}}, rotation = 0)));
        inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100,60},{ -80,80}}, rotation = 0)));
        Modelica.Fluid.Fittings.TeeJunctionIdeal junctionIdeal(redeclare package Medium = Medium) annotation(Placement(transformation(origin = { -38,6}, extent = {{ -10, -10},{10,10}}, rotation = 90)));

      equation 
        connect(source.ports[1],pipe1.port_a) annotation(Line(points = {{ -88,6},{ -78,6}}, color = {0,127,255}));
        connect(valveIncompressible1.port_b,sink.ports[2]) annotation(Line(points = {{28, -40},{46, -40},{46, -15.2},{62, -15.2}}, color = {0,127,255}));
        connect(valveIncompressible.port_b,sink.ports[1]) annotation(Line(points = {{30,46},{46,46},{46, -12.8},{62, -12.8}}, color = {0,127,255}));
        connect(pipe3.port_b,valveIncompressible1.port_a) annotation(Line(points = {{ -10, -40},{8, -40}}, color = {0,127,255}));
        connect(pipe2.port_b,valveIncompressible.port_a) annotation(Line(points = {{ -14,46},{10,46}}, color = {0,127,255}));
        connect(valveOpening1.y,valveIncompressible.opening) annotation(Line(points = {{1,80},{20,80},{20,54}}, color = {0,0,127}));
        connect(valveOpening2.y,valveIncompressible1.opening) annotation(Line(points = {{1,0},{18,0},{18, -32}}, color = {0,0,127}));
        connect(pipe1.port_b,junctionIdeal.port_3) annotation(Line(points = {{ -58,6},{ -53,6},{ -53,6},{ -48,6}}, color = {0,127,255}));
        connect(pipe2.port_a,junctionIdeal.port_2) annotation(Line(points = {{ -34,46},{ -38,46},{ -38,16}}, color = {0,127,255}));
        connect(junctionIdeal.port_1,pipe3.port_a) annotation(Line(points = {{ -38, -4},{ -38, -40},{ -30, -40}}, color = {0,127,255}));
      end BranchingPipes13;
      model BranchingPipes14
        replaceable package Medium = Modelica.Media.Water.StandardWater;
        Modelica.Fluid.Sources.Boundary_pT source(nPorts = 1, redeclare package Medium = Medium, p = 500000.0, T = 300) annotation(Placement(transformation(extent = {{ -100,0},{ -88,12}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe1(redeclare package Medium = Medium, p_a_start = 500000.0, use_T_start = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow, length = 10, diameter = 0.0254, p_b_start = 495000.0) annotation(Placement(transformation(extent = {{ -78, -4},{ -58,16}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valve1(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 400000.0, m_flow_nominal = 1, rho_nominal = 5) annotation(Placement(transformation(extent = {{10,36},{30,56}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valve2(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 400000.0, m_flow_nominal = 1, rho_nominal = 5) annotation(Placement(transformation(extent = {{8, -50},{28, -30}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 5), experimentSetupOutput(equdistant = false), Documentation(info = "<html>
Simulation starts with both valves open. At t=1, valve 1 closes; at t=2 valve 2 closes, and the simulation fails.
</html>"));
        Modelica.Fluid.Sources.Boundary_pT sink(redeclare package Medium = Medium, nPorts = 2, p = 100000, T = 300) annotation(Placement(transformation(extent = {{74, -20},{62, -8}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe2(redeclare package Medium = Medium, use_T_start = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow, length = 10, diameter = 0.0254, p_a_start = 495000.0, p_b_start = 490000.0) annotation(Placement(transformation(extent = {{ -34,36},{ -14,56}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe3(redeclare package Medium = Medium, use_T_start = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow, length = 10, diameter = 0.0254, p_a_start = 495000.0, p_b_start = 490000.0) annotation(Placement(transformation(extent = {{ -30, -50},{ -10, -30}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening1(offset = 0, table = [0,1;1,1;2,0.01;100,0.01]) annotation(Placement(transformation(extent = {{ -20,72},{0,92}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening2(offset = 0, table = [0,1;3,1;4,0.01;100,0.01]) annotation(Placement(transformation(extent = {{ -20, -10},{0,10}}, rotation = 0)));
        inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100,60},{ -80,80}}, rotation = 0)));
        Modelica.Fluid.Fittings.TeeJunctionVolume junctionIdeal(redeclare package Medium = Medium, V = 0.001, energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, p_start = 500000, T_start = 300) annotation(Placement(transformation(origin = { -38,6}, extent = {{ -10, -10},{10,10}}, rotation = 90)));

      equation 
        connect(source.ports[1],pipe1.port_a) annotation(Line(points = {{ -88,6},{ -78,6}}, color = {0,127,255}));
        connect(valve2.port_b,sink.ports[2]) annotation(Line(points = {{28, -40},{46, -40},{46, -15.2},{62, -15.2}}, color = {0,127,255}));
        connect(valve1.port_b,sink.ports[1]) annotation(Line(points = {{30,46},{46,46},{46, -12.8},{62, -12.8}}, color = {0,127,255}));
        connect(pipe3.port_b,valve2.port_a) annotation(Line(points = {{ -10, -40},{8, -40}}, color = {0,127,255}));
        connect(pipe2.port_b,valve1.port_a) annotation(Line(points = {{ -14,46},{10,46}}, color = {0,127,255}));
        connect(valveOpening1.y,valve1.opening) annotation(Line(points = {{1,82},{20,82},{20,54}}, color = {0,0,127}));
        connect(valveOpening2.y,valve2.opening) annotation(Line(points = {{1,0},{18,0},{18, -32}}, color = {0,0,127}));
        connect(pipe1.port_b,junctionIdeal.port_3) annotation(Line(points = {{ -58,6},{ -48,6},{ -48,6}}, color = {0,127,255}));
        connect(pipe2.port_a,junctionIdeal.port_2) annotation(Line(points = {{ -34,46},{ -38,46},{ -38,16}}, color = {0,127,255}));
        connect(junctionIdeal.port_1,pipe3.port_a) annotation(Line(points = {{ -38, -4},{ -38, -40},{ -30, -40}}, color = {0,127,255}));
      end BranchingPipes14;
      model BranchingPipes15
        replaceable package Medium = Modelica.Media.Air.DryAirNasa;
        Modelica.Fluid.Sources.Boundary_pT source(nPorts = 1, redeclare package Medium = Medium, p = 500000.0, T = 300) annotation(Placement(transformation(extent = {{ -100,0},{ -88,12}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe1(redeclare package Medium = Medium, use_T_start = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow(dp_small = 10), length = 10, diameter = 0.0254, p_a_start = 500000, p_b_start = 495000) annotation(Placement(transformation(extent = {{ -78, -4},{ -58,16}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valve1(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 400000.0, m_flow_nominal = 1, rho_nominal = 5) annotation(Placement(transformation(extent = {{10,36},{30,56}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valve2(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 400000.0, m_flow_nominal = 1, rho_nominal = 5) annotation(Placement(transformation(extent = {{8, -50},{28, -30}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 5), experimentSetupOutput(equdistant = false), Documentation(info = "<html>
Simulation starts with both valves open. At t=1, valve 1 closes; at t=2 valve 2 closes, and the simulation fails.
</html>"));
        Modelica.Fluid.Sources.Boundary_pT sink(redeclare package Medium = Medium, nPorts = 2, p = 100000, T = 300) annotation(Placement(transformation(extent = {{74, -20},{62, -8}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe2(redeclare package Medium = Medium, use_T_start = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow(dp_small = 10), length = 10, diameter = 0.0254, p_a_start = 495000.0, p_b_start = 490000.0) annotation(Placement(transformation(extent = {{ -34,36},{ -14,56}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe3(redeclare package Medium = Medium, use_T_start = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow(dp_small = 10), length = 10, diameter = 0.0254, p_a_start = 495000.0, p_b_start = 490000.0) annotation(Placement(transformation(extent = {{ -30, -50},{ -10, -30}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening1(offset = 0, table = [0,1;1,1;2,0.001;100,0.001]) annotation(Placement(transformation(extent = {{ -20,72},{0,92}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening2(offset = 0, table = [0,1;3,1;4,0.001;100,0.001]) annotation(Placement(transformation(extent = {{ -20, -10},{0,10}}, rotation = 0)));
        inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100,60},{ -80,80}}, rotation = 0)));
        Modelica.Fluid.Fittings.TeeJunctionVolume junctionIdeal(redeclare package Medium = Medium, V = 0.001, energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, p_start = 500000, T_start = 300) annotation(Placement(transformation(origin = { -38,6}, extent = {{ -10, -10},{10,10}}, rotation = 90)));

      equation 
        connect(source.ports[1],pipe1.port_a) annotation(Line(points = {{ -88,6},{ -78,6}}, color = {0,127,255}));
        connect(valve2.port_b,sink.ports[2]) annotation(Line(points = {{28, -40},{46, -40},{46, -15.2},{62, -15.2}}, color = {0,127,255}));
        connect(valve1.port_b,sink.ports[1]) annotation(Line(points = {{30,46},{46,46},{46, -12.8},{62, -12.8}}, color = {0,127,255}));
        connect(pipe3.port_b,valve2.port_a) annotation(Line(points = {{ -10, -40},{8, -40}}, color = {0,127,255}));
        connect(pipe2.port_b,valve1.port_a) annotation(Line(points = {{ -14,46},{10,46}}, color = {0,127,255}));
        connect(valveOpening1.y,valve1.opening) annotation(Line(points = {{1,82},{20,82},{20,54}}, color = {0,0,127}));
        connect(valveOpening2.y,valve2.opening) annotation(Line(points = {{1,0},{18,0},{18, -32}}, color = {0,0,127}));
        connect(pipe1.port_b,junctionIdeal.port_3) annotation(Line(points = {{ -58,6},{ -48,6},{ -48,6}}, color = {0,127,255}));
        connect(pipe2.port_a,junctionIdeal.port_2) annotation(Line(points = {{ -34,46},{ -38,46},{ -38,16}}, color = {0,127,255}));
        connect(junctionIdeal.port_1,pipe3.port_a) annotation(Line(points = {{ -38, -4},{ -38, -40},{ -30, -40}}, color = {0,127,255}));
      end BranchingPipes15;
      model BranchingPipes16
        replaceable package Medium = Modelica.Media.Air.DryAirNasa;
        Modelica.Fluid.Sources.Boundary_pT source(nPorts = 1, redeclare package Medium = Medium, p = 500000.0, T = 300) annotation(Placement(transformation(extent = {{ -100,0},{ -88,12}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe1(redeclare package Medium = Medium, p_a_start = 500000.0, use_T_start = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow(dp_small = 10), length = 10, diameter = 0.0254, p_b_start = 495000.0) annotation(Placement(transformation(extent = {{ -78, -4},{ -58,16}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valve1(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 400000.0, m_flow_nominal = 1, rho_nominal = 5) annotation(Placement(transformation(extent = {{10,36},{30,56}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valve2(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 400000.0, m_flow_nominal = 1, rho_nominal = 5) annotation(Placement(transformation(extent = {{8, -50},{28, -30}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 5), experimentSetupOutput(equdistant = false), Documentation(info = "<html>
Simulation starts with both valves open. At t=1, valve 1 closes; at t=2 valve 2 closes, and the simulation fails.
</html>"));
        Modelica.Fluid.Sources.Boundary_pT sink(redeclare package Medium = Medium, nPorts = 2, p = 100000, T = 300) annotation(Placement(transformation(extent = {{74, -20},{62, -8}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe2(redeclare package Medium = Medium, use_T_start = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow(dp_small = 10), length = 10, diameter = 0.0254, p_a_start = 495000.0, p_b_start = 490000.0) annotation(Placement(transformation(extent = {{ -34,36},{ -14,56}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe3(redeclare package Medium = Medium, use_T_start = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow(dp_small = 10), length = 10, diameter = 0.0254, p_a_start = 495000.0, p_b_start = 490000.0) annotation(Placement(transformation(extent = {{ -30, -50},{ -10, -30}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening1(offset = 0, table = [0,1;1,1;2,0;100,0]) annotation(Placement(transformation(extent = {{ -20,72},{0,92}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening2(offset = 0, table = [0,1;3,1;4,0;100,0]) annotation(Placement(transformation(extent = {{ -20, -12},{0,8}}, rotation = 0)));
        inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100,60},{ -80,80}}, rotation = 0)));
        Modelica.Fluid.Fittings.TeeJunctionVolume junctionIdeal(redeclare package Medium = Medium, V = 0.001, energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, p_start = 500000, T_start = 300) annotation(Placement(transformation(origin = { -38,6}, extent = {{ -10, -10},{10,10}}, rotation = 90)));

      equation 
        connect(source.ports[1],pipe1.port_a) annotation(Line(points = {{ -88,6},{ -78,6}}, color = {0,127,255}));
        connect(valve2.port_b,sink.ports[2]) annotation(Line(points = {{28, -40},{46, -40},{46, -15.2},{62, -15.2}}, color = {0,127,255}));
        connect(valve1.port_b,sink.ports[1]) annotation(Line(points = {{30,46},{46,46},{46, -12.8},{62, -12.8}}, color = {0,127,255}));
        connect(pipe3.port_b,valve2.port_a) annotation(Line(points = {{ -10, -40},{8, -40}}, color = {0,127,255}));
        connect(pipe2.port_b,valve1.port_a) annotation(Line(points = {{ -14,46},{10,46}}, color = {0,127,255}));
        connect(valveOpening1.y,valve1.opening) annotation(Line(points = {{1,82},{20,82},{20,54}}, color = {0,0,127}));
        connect(valveOpening2.y,valve2.opening) annotation(Line(points = {{1, -2},{18, -2},{18, -32}}, color = {0,0,127}));
        connect(pipe1.port_b,junctionIdeal.port_3) annotation(Line(points = {{ -58,6},{ -48,6},{ -48,6}}, color = {0,127,255}));
        connect(pipe2.port_a,junctionIdeal.port_2) annotation(Line(points = {{ -34,46},{ -38,46},{ -38,16}}, color = {0,127,255}));
        connect(junctionIdeal.port_1,pipe3.port_a) annotation(Line(points = {{ -38, -4},{ -38, -40},{ -30, -40}}, color = {0,127,255}));
      end BranchingPipes16;
      model BranchingPipes17
        replaceable package Medium = Modelica.Media.Air.DryAirNasa;
        Modelica.Fluid.Sources.Boundary_pT source(nPorts = 1, redeclare package Medium = Medium, p = 500000.0, T = 300) annotation(Placement(transformation(extent = {{ -100,0},{ -88,12}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe1(redeclare package Medium = Medium, p_a_start = 500000.0, use_T_start = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow(dp_small = 10), length = 10, diameter = 0.0254, p_b_start = 495000.0) annotation(Placement(transformation(extent = {{ -78, -4},{ -58,16}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valve1(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 400000.0, m_flow_nominal = 1, rho_nominal = 5, dp(start = 10)) annotation(Placement(transformation(extent = {{10,36},{30,56}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valve2(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 400000.0, m_flow_nominal = 1, rho_nominal = 5) annotation(Placement(transformation(extent = {{8, -50},{28, -30}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 5, Tolerance = 1e-07), experimentSetupOutput(equdistant = false), Documentation(info = "<html>
Simulation starts with both valves open. At t=1, valve 1 closes; at t=2 valve 2 closes, and the simulation fails.
</html>"));
        Modelica.Fluid.Sources.Boundary_pT sink(nPorts = 1, redeclare package Medium = Medium, T = 300, p = 100000.0) annotation(Placement(transformation(extent = {{94, -18},{82, -6}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe2(redeclare package Medium = Medium, use_T_start = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow(dp_small = 10), length = 10, diameter = 0.0254, p_a_start = 495000.0, p_b_start = 490000.0) annotation(Placement(transformation(extent = {{ -34,36},{ -14,56}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe3(redeclare package Medium = Medium, use_T_start = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow(dp_small = 10), length = 10, diameter = 0.0254, p_a_start = 495000.0, p_b_start = 490000.0) annotation(Placement(transformation(extent = {{ -30, -50},{ -10, -30}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening1(offset = 0, table = [0,1;1,1;2,0;100,0]) annotation(Placement(transformation(extent = {{ -20,72},{0,92}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening2(offset = 0, table = [0,1;3,1;4,0;100,0]) annotation(Placement(transformation(extent = {{ -18, -12},{2,8}}, rotation = 0)));
        inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100,60},{ -80,80}}, rotation = 0)));
        Modelica.Fluid.Fittings.TeeJunctionVolume junctionIdeal(redeclare package Medium = Medium, V = 0.001, energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, p_start = 500000, T_start = 300) annotation(Placement(transformation(origin = { -40,6}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Fluid.Fittings.TeeJunctionVolume junctionVolume(redeclare package Medium = Medium, V = 0.001) annotation(Placement(transformation(origin = {56, -12}, extent = {{ -10,10},{10, -10}}, rotation = 90)));

      equation 
        connect(source.ports[1],pipe1.port_a) annotation(Line(points = {{ -88,6},{ -78,6}}, color = {0,127,255}));
        connect(pipe3.port_b,valve2.port_a) annotation(Line(points = {{ -10, -40},{8, -40}}, color = {0,127,255}));
        connect(pipe2.port_b,valve1.port_a) annotation(Line(points = {{ -14,46},{10,46}}, color = {0,127,255}));
        connect(valveOpening1.y,valve1.opening) annotation(Line(points = {{1,82},{20,82},{20,54}}, color = {0,0,127}));
        connect(valveOpening2.y,valve2.opening) annotation(Line(points = {{3, -2},{18, -2},{18, -32}}, color = {0,0,127}));
        connect(pipe1.port_b,junctionIdeal.port_3) annotation(Line(points = {{ -58,6},{ -50,6},{ -50,6}}, color = {0,127,255}));
        connect(pipe2.port_a,junctionIdeal.port_2) annotation(Line(points = {{ -34,46},{ -40,46},{ -40,16}}, color = {0,127,255}));
        connect(junctionIdeal.port_1,pipe3.port_a) annotation(Line(points = {{ -40, -4},{ -40, -40},{ -30, -40}}, color = {0,127,255}));
        connect(junctionVolume.port_3,sink.ports[1]) annotation(Line(points = {{66, -12},{82, -12}}, color = {0,127,255}));
        connect(valve2.port_b,junctionVolume.port_1) annotation(Line(points = {{28, -40},{56, -40},{56, -22}}, color = {0,127,255}));
        connect(valve1.port_b,junctionVolume.port_2) annotation(Line(points = {{30,46},{56,46},{56, -2}}, color = {0,127,255}));
      end BranchingPipes17;
      model BranchingPipes18
        replaceable package Medium = Modelica.Media.Water.StandardWater;
        Modelica.Fluid.Sources.Boundary_pT source(nPorts = 1, redeclare package Medium = Medium, p = 500000.0, T = 300) annotation(Placement(transformation(extent = {{ -100,0},{ -88,12}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe1(redeclare package Medium = Medium, p_a_start = 500000.0, use_T_start = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow(dp_small = 10), length = 10, diameter = 0.0254, p_b_start = 495000.0) annotation(Placement(transformation(extent = {{ -78, -4},{ -58,16}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valve1(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 400000.0, m_flow_nominal = 1, rho_nominal = 5) annotation(Placement(transformation(extent = {{10,36},{30,56}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valve2(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, dp_nominal = 400000.0, m_flow_nominal = 1, rho_nominal = 5) annotation(Placement(transformation(extent = {{8, -50},{28, -30}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 5), experimentSetupOutput(equdistant = false), Documentation(info = "<html>
Simulation starts with both valves open. At t=1, valve 1 closes; at t=2 valve 2 closes, and the simulation fails.
</html>"));
        Modelica.Fluid.Sources.Boundary_pT sink(nPorts = 1, redeclare package Medium = Medium, T = 300, p = 100000.0) annotation(Placement(transformation(extent = {{94, -18},{82, -6}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe2(redeclare package Medium = Medium, use_T_start = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow(dp_small = 10), length = 10, diameter = 0.0254, p_a_start = 495000.0, p_b_start = 490000.0) annotation(Placement(transformation(extent = {{ -34,36},{ -14,56}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe3(redeclare package Medium = Medium, use_T_start = true, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow(dp_small = 10), length = 10, diameter = 0.0254, p_a_start = 495000.0, p_b_start = 490000.0) annotation(Placement(transformation(extent = {{ -30, -50},{ -10, -30}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening1(offset = 0, table = [0,1;1,1;2,0;100,0]) annotation(Placement(transformation(extent = {{ -20,72},{0,92}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening2(offset = 0, table = [0,1;3,1;4,0;100,0]) annotation(Placement(transformation(extent = {{ -18, -12},{2,8}}, rotation = 0)));
        inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100,60},{ -80,80}}, rotation = 0)));
        Modelica.Fluid.Fittings.TeeJunctionVolume junctionIdeal(redeclare package Medium = Medium, V = 0.001, energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, p_start = 500000, T_start = 300) annotation(Placement(transformation(origin = { -40,6}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Fluid.Fittings.TeeJunctionVolume junctionVolume(redeclare package Medium = Medium, V = 0.001) annotation(Placement(transformation(origin = {56, -12}, extent = {{ -10,10},{10, -10}}, rotation = 90)));

      equation 
        connect(source.ports[1],pipe1.port_a) annotation(Line(points = {{ -88,6},{ -78,6}}, color = {0,127,255}));
        connect(pipe3.port_b,valve2.port_a) annotation(Line(points = {{ -10, -40},{8, -40}}, color = {0,127,255}));
        connect(pipe2.port_b,valve1.port_a) annotation(Line(points = {{ -14,46},{10,46}}, color = {0,127,255}));
        connect(valveOpening1.y,valve1.opening) annotation(Line(points = {{1,82},{20,82},{20,54}}, color = {0,0,127}));
        connect(valveOpening2.y,valve2.opening) annotation(Line(points = {{3, -2},{18, -2},{18, -32}}, color = {0,0,127}));
        connect(pipe1.port_b,junctionIdeal.port_3) annotation(Line(points = {{ -58,6},{ -54,6},{ -54,6},{ -50,6}}, color = {0,127,255}));
        connect(pipe2.port_a,junctionIdeal.port_2) annotation(Line(points = {{ -34,46},{ -40,46},{ -40,16}}, color = {0,127,255}));
        connect(junctionIdeal.port_1,pipe3.port_a) annotation(Line(points = {{ -40, -4},{ -40, -40},{ -30, -40}}, color = {0,127,255}));
        connect(junctionVolume.port_3,sink.ports[1]) annotation(Line(points = {{66, -12},{82, -12}}, color = {0,127,255}));
        connect(valve2.port_b,junctionVolume.port_1) annotation(Line(points = {{28, -40},{56, -40},{56, -22}}, color = {0,127,255}));
        connect(valve1.port_b,junctionVolume.port_2) annotation(Line(points = {{30,46},{56,46},{56, -2}}, color = {0,127,255}));
      end BranchingPipes18;
      model BranchingPipes131
        replaceable package Medium = Modelica.Media.Water.StandardWater;
        Modelica.Fluid.Sources.Boundary_pT source(nPorts = 1, redeclare package Medium = Medium, p = 500000.0, T = 300) annotation(Placement(transformation(extent = {{ -100,0},{ -88,12}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe1(redeclare package Medium = Medium, p_a_start = 500000.0, use_T_start = true, length = 10, diameter = 0.0254, p_b_start = 495000.0, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow) annotation(Placement(transformation(extent = {{ -78, -4},{ -58,16}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valveIncompressible1(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, m_flow_nominal = 1, rho_nominal = 5, dp(start = 400000), dp_nominal = 400000) annotation(Placement(transformation(extent = {{8, -50},{28, -30}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 5), experimentSetupOutput(equdistant = false), Documentation(info = "<html>
Simulation starts with both valves open. At t=1, valve 1 closes; at t=2 valve 2 closes, and the simulation fails.
</html>"));
        Modelica.Fluid.Sources.Boundary_pT sink(nPorts = 1, redeclare package Medium = Medium, T = 300, p = 100000.0) annotation(Placement(transformation(extent = {{74, -20},{62, -8}}, rotation = 0)));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe3(redeclare package Medium = Medium, use_T_start = true, length = 10, diameter = 0.0254, p_a_start = 495000.0, p_b_start = 490000.0, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow) annotation(Placement(transformation(extent = {{ -30, -50},{ -10, -30}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening1(offset = 0, table = [0,1;1,1;1,0;100,0]) annotation(Placement(transformation(extent = {{ -20,70},{0,90}}, rotation = 0)));
        Modelica.Blocks.Sources.TimeTable valveOpening2(offset = 0, table = [0,1;2,1;2.01,1e-06;100,0]) annotation(Placement(transformation(extent = {{ -20, -8},{0,12}}, rotation = 0)));
        inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100,60},{ -80,80}}, rotation = 0)));

      equation 
        connect(source.ports[1],pipe1.port_a) annotation(Line(points = {{ -88,6},{ -78,6}}, color = {0,127,255}));
        connect(valveIncompressible1.port_b,sink.ports[1]) annotation(Line(points = {{28, -40},{46, -40},{46, -14},{62, -14}}, color = {0,127,255}));
        connect(pipe3.port_b,valveIncompressible1.port_a) annotation(Line(points = {{ -10, -40},{8, -40}}, color = {0,127,255}));
        connect(valveOpening2.y,valveIncompressible1.opening) annotation(Line(points = {{1,2},{18,2},{18, -32}}, color = {0,0,127}));
        connect(pipe1.port_b,pipe3.port_a) annotation(Line(points = {{ -58,6},{ -44,6},{ -44, -40},{ -30, -40}}, color = {0,127,255}));
      end BranchingPipes131;
      model LumpedPipeInitialization "Steady-state initialization of a lumped pipe"
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 4), experimentSetupOutput);
        Modelica.Fluid.Sources.FixedBoundary source(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, use_T = false, p = 10000000, h = 2000000.0) annotation(Placement(transformation(extent = {{ -80, -10},{ -60,10}})));
        ModelicaTest.Fluid.BaseClasses.LumpedPipe pipe(redeclare package Medium = Modelica.Media.Water.StandardWater, h_start = 2000000.0, diameter = 0.05, length = 200, use_T_start = false, p_a_start = 10000000, p_b_start = 9900000) annotation(Placement(transformation(extent = {{ -40, -10},{ -20,10}})));
        Modelica.Fluid.Valves.ValveCompressible valve(redeclare package Medium = Modelica.Media.Water.StandardWater, Av = 0.001, dp_nominal = 10000000, m_flow_nominal = 10) annotation(Placement(transformation(extent = {{0, -10},{20,10}})));
        Modelica.Fluid.Sources.FixedBoundary sink(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, p = 9500000) annotation(Placement(transformation(extent = {{60, -10},{40,10}})));
        Modelica.Blocks.Sources.Ramp ramp(offset = 1, duration = 0.1, height =  -0.5, startTime = 2) annotation(Placement(transformation(extent = {{46,30},{26,50}})));
        inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.SteadyState) annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}})));
        discrete Modelica.SIunits.MassFlowRate m_flow_initial;

      equation 
        when time > 0.1 then
                  m_flow_initial = valve.port_a.m_flow;
        
        end when;
if pipe.energyDynamics >= Modelica.Fluid.Types.Dynamics.SteadyStateInitial and pipe.massDynamics >= Modelica.Fluid.Types.Dynamics.SteadyStateInitial then
                  when time > 1 then
                      assert(abs(valve.port_a.m_flow - m_flow_initial) < 0.001, "!!!THE SIMULATION DID NOT START IN STEADY-STATE!!!");
          
          end when;
        end if;
        connect(source.ports[1],pipe.port_a) annotation(Line(points = {{ -60,0},{ -40,0}}, color = {0,127,255}, smooth = Smooth.None));
        connect(pipe.port_b,valve.port_a) annotation(Line(points = {{ -20,0},{0,0}}, color = {0,127,255}, smooth = Smooth.None));
        connect(valve.port_b,sink.ports[1]) annotation(Line(points = {{20,0},{40,0}}, color = {0,127,255}, smooth = Smooth.None));
        connect(ramp.y,valve.opening) annotation(Line(points = {{25,40},{10,40},{10,8}}, color = {0,0,127}, smooth = Smooth.None));
      end LumpedPipeInitialization;
    end TestCriticalCases;
    package BaseClasses "Specific models used for testing"
      extends Modelica.Fluid.Icons.BaseClassLibrary;
      model GenericStaticHead "Models two ports at different heights"
        extends Modelica.Fluid.Interfaces.PartialTwoPortTransport;
        parameter SI.Length height_ab "Height(port_b) - Height(port_a)" annotation(Dialog(group = "Static head"));
        replaceable model PressureLoss = Modelica.Fluid.Pipes.BaseClasses.FlowModels.NominalLaminarFlow extends Modelica.Fluid.Interfaces.PartialDistributedFlow "Pressure loss model" annotation(Dialog(group = "Pressure loss"), choicesAllMatching = true);
        PressureLoss pressureLoss(redeclare final package Medium = Medium, final n = 2, states = {state_a,state_b}, vs = {port_a.m_flow / Medium.density(pressureLoss.states[1]) / pressureLoss.crossAreas[1], -port_b.m_flow / Medium.density(pressureLoss.states[2]) / pressureLoss.crossAreas[2]}, final momentumDynamics = Modelica.Fluid.Types.Dynamics.SteadyState, final allowFlowReversal = allowFlowReversal, dheights = {height_ab}, dp_nominal = 1, m_flow_nominal = 1, m_flow_start = m_flow_start, p_a_start = system.p_start + 0.5 * dp_start, p_b_start = system.p_start - 0.5 * dp_start, nParallel = 1, pathLengths = {0}, crossAreas = fill(Modelica.Constants.pi / 4 * 0.0254 ^ 2, 2), dimensions = fill(0.0254, 2), roughnesses = fill(2.5e-05, 2));

      equation 
        m_flow = pressureLoss.m_flows[1];
        port_a.h_outflow = inStream(port_b.h_outflow) + system.g * height_ab;
        port_b.h_outflow = inStream(port_a.h_outflow) - system.g * height_ab;
        annotation(defaultComponentName = "staticHead", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,60},{100, -60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,127,255})}), Documentation(info = "<html>
<p>
This component is intended for early designs and later replacement by more detailed models.
It describes the static head due to the height difference between its two ports.
</p>
<p>
Per default a small regularization with a linear pressure loss of 1Pa per 1kg/s is configured.
The regularization can be changed for the PressureLoss model.
</p>
</html>", revisions = "<html>
<ul>
<li><i>8 Dec 2008</i>
    by Ruediger Franke:<br>
       Introduce small nominal pressure loss for regularization</li>
<li><i>31 Oct 2007</i>
    by <a href=\"mailto:jonas@modelon.se\">Jonas Eborn</a>:<br>
       Changed to flow-direction dependent density</li>
<li><i>2 Nov 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Added to Modelica.Fluid</li>
</ul>
</html>"));
      end GenericStaticHead;
      model LumpedPipe "Example for a composite pipe model"
        extends Modelica.Fluid.Pipes.BaseClasses.PartialStraightPipe;
        parameter Modelica.Fluid.Types.Dynamics energyDynamics = system.energyDynamics "Formulation of energy balance" annotation(Evaluate = true, Dialog(tab = "Assumptions", group = "Dynamics"));
        parameter Modelica.Fluid.Types.Dynamics massDynamics = system.massDynamics "Formulation of mass balance" annotation(Evaluate = true, Dialog(tab = "Assumptions", group = "Dynamics"));
        parameter Medium.AbsolutePressure p_a_start = system.p_start "Start value of pressure at port a" annotation(Dialog(tab = "Initialization"));
        parameter Medium.AbsolutePressure p_b_start = p_a_start "Start value of pressure at port b" annotation(Dialog(tab = "Initialization"));
        parameter Boolean use_T_start = true "Use T_start if true, otherwise h_start" annotation(Evaluate = true, Dialog(tab = "Initialization"));
        parameter Medium.Temperature T_start = if use_T_start then system.T_start else Medium.temperature_phX((p_a_start + p_b_start) / 2, h_start, X_start) "Start value of temperature" annotation(Evaluate = true, Dialog(tab = "Initialization", enable = use_T_start));
        parameter Medium.SpecificEnthalpy h_start = if use_T_start then Medium.specificEnthalpy_pTX((p_a_start + p_b_start) / 2, T_start, X_start) else Medium.h_default "Start value of specific enthalpy" annotation(Evaluate = true, Dialog(tab = "Initialization", enable = not use_T_start));
        parameter Medium.MassFraction X_start[Medium.nX] = Medium.X_default "Start value of mass fractions m_i/m" annotation(Dialog(tab = "Initialization", enable = Medium.nXi > 0));
        parameter Medium.ExtraProperty C_start[Medium.nC](quantity = Medium.extraPropertiesNames) = fill(0, Medium.nC) "Start value of trace substances" annotation(Dialog(tab = "Initialization", enable = Medium.nC > 0));
        parameter Medium.MassFlowRate m_flow_start = system.m_flow_start "Start value for mass flow rate" annotation(Evaluate = true, Dialog(tab = "Initialization"));
        parameter Boolean use_HeatTransfer = false "= true to use the HeatTransfer model" annotation(Dialog(tab = "Assumptions", group = "Heat transfer"));
        replaceable model HeatTransfer = Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.IdealHeatTransfer extends Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.PartialVesselHeatTransfer "Wall heat transfer" annotation(Dialog(tab = "Assumptions", group = "Heat transfer", enable = use_HeatTransfer), choicesAllMatching = true);
        Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort if use_HeatTransfer annotation(Placement(transformation(extent = {{ -10,44},{10,64}}, rotation = 0), iconTransformation(extent = {{ -10,35},{10,55}})));
        Modelica.Fluid.Pipes.StaticPipe staticPipe1(redeclare package Medium = Medium, allowFlowReversal = allowFlowReversal, nParallel = nParallel, length = length / 2, roughness = roughness, diameter = diameter, perimeter = perimeter, crossArea = crossArea, height_ab = height_ab / 2, m_flow_start = m_flow_start, redeclare final model FlowModel = FlowModel) annotation(Placement(transformation(extent = {{ -60, -10},{ -40,10}}, rotation = 0)));
        Modelica.Fluid.Vessels.ClosedVolume volume(redeclare package Medium = Medium, redeclare model HeatTransfer = HeatTransfer, heatTransfer(surfaceAreas = {perimeter * length}), energyDynamics = energyDynamics, massDynamics = massDynamics, p_start = (p_a_start + p_b_start) / 2, use_T_start = use_T_start, T_start = T_start, h_start = h_start, X_start = X_start, C_start = C_start, V = V, nPorts = 2, use_portsData = false, use_HeatTransfer = use_HeatTransfer) annotation(Placement(transformation(extent = {{ -10,10},{10,30}}, rotation = 0)));
        Modelica.Fluid.Pipes.StaticPipe staticPipe2(redeclare package Medium = Medium, allowFlowReversal = allowFlowReversal, nParallel = nParallel, length = length / 2, roughness = roughness, diameter = diameter, perimeter = perimeter, crossArea = crossArea, height_ab = height_ab / 2, m_flow_start = m_flow_start, redeclare final model FlowModel = FlowModel) annotation(Placement(transformation(extent = {{40, -10},{60,10}}, rotation = 0)));

      equation 
        connect(staticPipe1.port_a,port_a) annotation(Line(points = {{ -60,0},{ -80,0},{ -100,0}}, color = {0,127,255}));
        connect(staticPipe2.port_b,port_b) annotation(Line(points = {{60,0},{80,0},{100,0}}, color = {0,127,255}));
        annotation(defaultComponentName = "pipe", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -10,10},{10, -10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>
Simple pipe model consisting of one volume,
wall friction (with different friction correlations)
and gravity effect. This model is mostly used to demonstrate how
to build up more detailed models from the basic components.
Note, if the \"heatPort\" is not connected, then the pipe
is totally insulated (= no thermal flow from the fluid to the
pipe wall/environment).
</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
        connect(staticPipe1.port_b,volume.ports[1]) annotation(Line(points = {{ -40,0},{ -2,0},{ -2,10}}, color = {0,127,255}, smooth = Smooth.None));
        connect(staticPipe2.port_a,volume.ports[2]) annotation(Line(points = {{40,0},{2,0},{2,10}}, color = {0,127,255}, smooth = Smooth.None));
        connect(heatPort,volume.heatPort) annotation(Line(points = {{0,54},{0,40},{ -20,40},{ -20,20},{ -10,20}}, color = {191,0,0}, smooth = Smooth.None));
      end LumpedPipe;
    end BaseClasses;
  end Fluid;
  package Utilities "Test functions for Modelica.Utilities"
    function Strings "Test functions of Modelica.Utilities.Strings"
      import Modelica.Utilities.Streams;
      import Modelica.Utilities.Strings;
      import Modelica.Utilities.Types;
      input String logFile = "ModelicaTestLog.txt" "Filename where the log is stored";
      output Boolean ok;
    protected 
      Integer i;
      Integer j;
      Real r;
      String s;
      Boolean b;
      String svec[2];
      Types.TokenValue token;
    algorithm 
      Streams.print("... Test of Modelica.Utilities.Strings", logFile);
      i:=Strings.length("hello, world");
      assert(i == 12, "Strings.length failed");
      s:=Strings.substring("12345678", 3);
      assert(s == "3", "Strings.Advanced.substring 1 failed");
      s:=Strings.substring("12345678", 3, 5);
      assert(s == "345", "Strings.Advanced.substring 2 failed");
      s:=Strings.repeat(3, "12");
      assert(s == "121212", "Strings.Advanced.repeat failed");
      i:=Strings.compare("abc", "ABC", caseSensitive = false);
      assert(i == Modelica.Utilities.Types.Compare.Equal, "Strings.Advanced.compare 1 failed");
      i:=Strings.compare("abc", "zef");
      assert(i == Modelica.Utilities.Types.Compare.Less, "Strings.Advanced.compare 2 failed");
      b:=Strings.isEqual("abc", "ABC");
      assert(not b, "Strings.Advanced.isEqual 1 failed");
      b:=Strings.isEqual("abc", "ABC", caseSensitive = false);
      assert(b, "Strings.Advanced.isEqual 2 failed");
      i:=Strings.count("line1 line2 line1 line2", "line2");
      assert(i == 2, "Strings.Advanced.count failed");
      i:=Strings.find("x line x line x", "line");
      assert(i == 3, "Strings.Advanced.find failed");
      i:=Strings.findLast("x line x line x", "line");
      assert(i == 10, "Strings.Advanced.findLast failed");
      s:=Strings.replace("x line x line x", "line", "LINE");
      assert(s == "x LINE x LINE x", "Strings.Advanced.replace failed");
      svec:=Strings.sort({"zxy","abc"});
      assert(svec[1] == "abc" and svec[2] == "zxy", "Strings.Advanced.sort failed");
      i:=Strings.Advanced.skipWhiteSpace("  abc ", 1);
      assert(i == 3, "Strings.Advanced.skipWhiteSpace failed");
      i:=Strings.Advanced.skipLineComments(" // xx", 1);
      assert(i == 7, "Strings.Advanced.skipLineComments 1 failed");
      i:=Strings.Advanced.skipLineComments(" // xx", 1);
      assert(i == 7, "Strings.Advanced.skipLineComments 1 failed");
      (i,r):=Strings.Advanced.scanReal("  -123.47e-2", 1);
      assert(i == 13 and r ==  -1.2347, "Strings.Advanced.scanReal 1 failed");
      (i,r):=Strings.Advanced.scanReal("  123 ", 1, unsigned = true);
      assert(i == 6 and r == 123, "Strings.Advanced.scanReal 2 failed");
      (i,j):=Strings.Advanced.scanInteger("  -123", 1);
      assert(i == 7 and j ==  -123, "Strings.Advanced.scanInteger 1 failed");
      (i,j):=Strings.Advanced.scanInteger("  123", 1, unsigned = true);
      assert(i == 6 and j == 123, "Strings.Advanced.scanInteger 2 failed");
      (i,s):=Strings.Advanced.scanString(" \"string text\" x=3", 1);
      assert(i == 15 and s == "string text", "Strings.Advanced.scanString failed");
      (i,s):=Strings.Advanced.scanIdentifier(" x_12=2;", 1);
      assert(i == 6 and s == "x_12", "Strings.Advanced.scanIdentifier failed");
      (token,i):=Strings.scanToken("  -123.47e-2", 1);
      assert(i == 13 and token.tokenType == Types.TokenType.RealToken and token.real ==  -1.2347, "String.scanToken 1 failed");
      (token,i):=Strings.scanToken("  -123", 1);
      assert(i == 7 and token.tokenType == Types.TokenType.IntegerToken and token.integer ==  -123, "String.scanToken 2 failed");
      (token,i):=Strings.scanToken("  123", 1, unsigned = true);
      assert(i == 6 and token.tokenType == Types.TokenType.IntegerToken and token.integer == 123, "String.scanToken 3 failed");
      (token,i):=Strings.scanToken("  true x", 1);
      assert(i == 7 and token.tokenType == Types.TokenType.BooleanToken and token.boolean, "String.scanToken 4 failed");
      (token,i):=Strings.scanToken("  false x", 1);
      assert(i == 8 and token.tokenType == Types.TokenType.BooleanToken and not token.boolean, "String.scanToken 5 failed");
      (token,i):=Strings.scanToken(" \"string text\" x=3", 1);
      assert(i == 15 and token.tokenType == Types.TokenType.StringToken and token.string == "string text", "String.scanToken 6 failed");
      (token,i):=Strings.scanToken(" x_12=2;", 1);
      assert(i == 6 and token.tokenType == Types.TokenType.IdentifierToken and token.string == "x_12", "String.scanToken 7 failed");
      (token,i):=Strings.scanToken("  abc = 3;   ", 11);
      assert(i == 14 and token.tokenType == Types.TokenType.NoToken, "String.scanToken 8 failed");
      (r,i):=Strings.scanReal("  123 ", 1, unsigned = true);
      assert(i == 6 and r == 123, "String.scanRequiredReal 2 failed");
      (j,i):=Strings.scanInteger("  -123", 1);
      assert(i == 7 and j ==  -123, "String.scanRequiredInteger 1 failed");
      (j,i):=Strings.scanInteger("  123", 1, unsigned = true);
      assert(i == 6 and j == 123, "String.scanRequiredInteger 2 failed");
      (b,i):=Strings.scanBoolean("  true x", 1);
      assert(i == 7 and b, "String.scanRequiredBoolean 1 failed");
      (b,i):=Strings.scanBoolean("  false x", 1);
      assert(i == 8 and not b, "String.scanRequiredBoolean 2 failed");
      (s,i):=Strings.scanString(" \"string text\" x=3", 1);
      assert(i == 15 and s == "string text", "String.scanRequiredString failed");
      (s,i):=Strings.scanIdentifier(" x_12=2;", 1);
      assert(i == 6 and s == "x_12", "String.scanRequiredIdentifier failed");
      (s,i):=Strings.scanDelimiter("  &,", 1, {"&",","});
      assert(i == 4 and s == "&", "String.scanRequiredDelimiter 1 failed");
      (s,i):=Strings.scanDelimiter("  /*,", 1, {"&","/*"});
      assert(i == 5 and s == "/*", "String.scanRequiredDelimiter 2 failed");
      Strings.scanNoToken("  abc = 3;   ", 11);
      ok:=true;
    end Strings;
    function Streams "Test functions of Modelica.Utilities.Streams"
      import Modelica.Utilities.Streams;
      import Modelica.Utilities.Files;
      input String logFile = "ModelicaTestLog.txt" "Filename where the log is stored";
      output Boolean ok;
    protected 
      String file = "#testStreams.txt";
      String line1 = "this is line 1";
      String line2 = "this is line 2";
      String line3 = "this is line 3";
      String lines[3];
      String rline;
      Integer nLines;
      Boolean eof;
    algorithm 
      Streams.print("... Test of Modelica.Utilities.Streams", logFile);
      Files.remove(file);
      Streams.print(line1, file);
      Streams.print(line2, file);
      Streams.print(line3, file);
      Streams.close(file);
      lines:=Streams.readFile(file);
      assert(lines[1] == line1 and lines[2] == line2 and lines[3] == line3, "Streams.readFile failed");
      nLines:=Streams.countLines(file);
      assert(nLines == 3, "Streams.countLines failed");
      (rline,eof):=Streams.readLine(file, 1);
      assert(rline == line1 and not eof, "Streams.readLine 1 failed");
      (rline,eof):=Streams.readLine(file, 2);
      assert(rline == line2 and not eof, "Streams.readLine 2 failed");
      (rline,eof):=Streams.readLine(file, 3);
      assert(rline == line3 and not eof, "Streams.readLine 3 failed");
      (rline,eof):=Streams.readLine(file, 4);
      assert(rline == "" and eof, "Streams.readLine 4 failed");
      Files.remove(file);
      ok:=true;
    end Streams;
    function Internal "Test functions of Modelica.Utilities.Internal"
      import Modelica.Utilities.Internal.FileSystem;
      import Modelica.Utilities.System;
      import Modelica.Utilities.Types;
      import Modelica.Utilities.Streams;
      input String logFile = "ModelicaTestLog.txt" "Filename where the log is stored";
      output Boolean ok;
    protected 
      String dir1;
      String dir2;
      String dir3;
      String dir4;
      String env;
      Boolean exist;
      Modelica.Utilities.Types.FileType fileType;
    algorithm 
      Streams.print("... Test of Modelica.Utilities.Internal.FileSystem and .System", logFile);
      dir1:=System.getWorkDirectory();
      dir2:=dir1 + "/#ModelicaTest";
      fileType:=FileSystem.stat(dir2);
      if fileType > Types.FileType.NoFile then 
            FileSystem.rmdir(dir2);

      else 
      end if;
      FileSystem.mkdir(dir2);
      fileType:=FileSystem.stat(dir2);
      assert(fileType == Types.FileType.Directory, "FileSystem.mkdir/stat failed");
      Streams.print("dir2 = " + dir2);
      System.setWorkDirectory(dir2);
      Streams.print(".. 1");
      dir3:=System.getWorkDirectory();
      Streams.print(".. 2");
      assert(dir2 == dir3, "System.xxxWorkDirectory failed\n" + "set dir = " + dir1 + "\n" + "get dir = " + dir3 + "\n");
      Streams.print(".. 3");
      System.setWorkDirectory("..");
      Streams.print(".. 4");
      dir4:=dir1 + "/#ModelicaTest2";
      Streams.print(".. 5");
      FileSystem.rename(dir2, dir4);
      Streams.print(".. 6");
      FileSystem.rmdir(dir4);
      System.setEnvironmentVariable("#ModelicaTest", "/dir1/dir2/dir3", convertFromSlash = true);
      (env,exist):=System.getEnvironmentVariable("#ModelicaTest", convertToSlash = true);
      assert(exist, "System.getEnvironmentVariable 1 failed");
      assert(env == "/dir1/dir2/dir3", "System.getEnvironmentVariable 2 failed\n" + "env = " + env);
      ok:=true;
    end Internal;
    function Files "Test functions of Modelica.Utilities.Files"
      import Modelica.Utilities.Files;
      import Modelica.Utilities.Streams;
      input String logFile = "ModelicaTestLog.txt" "Filename where the log is stored";
      output Boolean ok;
    protected 
      String directory;
      String name;
      String extension;
    algorithm 
      Streams.print("... Test of Modelica.Utilities.Files", logFile);
      (directory,name,extension):=Files.splitPathName("E:/test1/test2.save.txt");
      assert(directory == "E:/test1/" and name == "test2.save" and extension == ".txt", "Files.splitPathName 1 failed");
      (directory,name,extension):=Files.splitPathName("E:/test1/test2");
      assert(directory == "E:/test1/" and name == "test2" and extension == "", "Files.splitPathName 2 failed");
      ok:=true;
    end Files;
    function testAll "Test functions of Modelica.Utilities"
      input String logFile = "ModelicaTestLog.txt" "Filename where the log is stored";
      output Boolean ok;
    protected 
      Boolean result;
    algorithm 
      result:=ModelicaTest.Utilities.Strings(logFile);
      result:=ModelicaTest.Utilities.Streams(logFile);
      result:=ModelicaTest.Utilities.Files(logFile);
      ok:=true;
    end testAll;
  end Utilities;
  package Translational
    model AllComponents
      Modelica.Mechanics.Translational.Components.Mass slidingMass annotation(Placement(transformation(extent = {{ -60,60},{ -40,80}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.Rod rod annotation(Placement(transformation(extent = {{ -28,60},{ -8,80}}, rotation = 0)));
      annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1.1));
      Modelica.Mechanics.Translational.Components.Spring spring annotation(Placement(transformation(extent = {{0,60},{20,80}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.Damper damper(d = 10) annotation(Placement(transformation(extent = {{26,60},{46,80}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.Mass slidingMass1 annotation(Placement(transformation(extent = {{54,60},{74,80}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.SpringDamper springDamper annotation(Placement(transformation(extent = {{ -30,20},{ -10,40}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.Mass slidingMass2 annotation(Placement(transformation(extent = {{0,20},{20,40}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.ElastoGap elastoGap annotation(Placement(transformation(extent = {{30,20},{50,40}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.Mass slidingMass3 annotation(Placement(transformation(extent = {{60,20},{80,40}}, rotation = 0)));
      Modelica.Mechanics.Translational.Sources.Position position annotation(Placement(transformation(extent = {{ -68, -20},{ -48,0}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine annotation(Placement(transformation(extent = {{ -100, -20},{ -80,0}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.Mass slidingMass4 annotation(Placement(transformation(extent = {{ -40, -20},{ -20,0}}, rotation = 0)));
      Modelica.Mechanics.Translational.Sources.Speed position1 annotation(Placement(transformation(extent = {{ -68, -60},{ -48, -40}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine1 annotation(Placement(transformation(extent = {{ -100, -60},{ -80, -40}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.Mass slidingMass5 annotation(Placement(transformation(extent = {{ -40, -60},{ -20, -40}}, rotation = 0)));
      Modelica.Mechanics.Translational.Sources.Accelerate position2 annotation(Placement(transformation(extent = {{ -68, -100},{ -48, -80}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine2 annotation(Placement(transformation(extent = {{ -100, -100},{ -80, -80}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.Mass slidingMass6 annotation(Placement(transformation(extent = {{ -40, -100},{ -20, -80}}, rotation = 0)));
      Modelica.Mechanics.Translational.Sources.Move position3 annotation(Placement(transformation(extent = {{32, -20},{52,0}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine3[3] annotation(Placement(transformation(extent = {{0, -20},{20,0}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.Mass slidingMass7 annotation(Placement(transformation(extent = {{60, -20},{80,0}}, rotation = 0)));
      Modelica.Mechanics.Translational.Sources.Force force annotation(Placement(transformation(extent = {{ -70,20},{ -50,40}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.RelativeStates relativeStates annotation(Placement(transformation(extent = {{ -2,82},{18,102}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine4 annotation(Placement(transformation(extent = {{ -100,20},{ -80,40}}, rotation = 0)));

    equation 
      connect(slidingMass.flange_b,rod.flange_a) annotation(Line(points = {{ -40,70},{ -28,70}}, color = {0,127,0}, smooth = Smooth.None));
      connect(rod.flange_b,spring.flange_a) annotation(Line(points = {{ -8,70},{0,70}}, color = {0,127,0}, smooth = Smooth.None));
      connect(spring.flange_b,damper.flange_a) annotation(Line(points = {{20,70},{26,70}}, color = {0,127,0}, smooth = Smooth.None));
      connect(damper.flange_b,slidingMass1.flange_a) annotation(Line(points = {{46,70},{54,70}}, color = {0,127,0}, smooth = Smooth.None));
      connect(slidingMass.flange_b,springDamper.flange_a) annotation(Line(points = {{ -40,70},{ -40,30},{ -30,30}}, color = {0,127,0}, smooth = Smooth.None));
      connect(springDamper.flange_b,slidingMass2.flange_a) annotation(Line(points = {{ -10,30},{0,30}}, color = {0,127,0}, smooth = Smooth.None));
      connect(slidingMass2.flange_b,elastoGap.flange_a) annotation(Line(points = {{20,30},{30,30}}, color = {0,127,0}, smooth = Smooth.None));
      connect(elastoGap.flange_b,slidingMass3.flange_a) annotation(Line(points = {{50,30},{60,30}}, color = {0,127,0}, smooth = Smooth.None));
      connect(sine.y,position.s_ref) annotation(Line(points = {{ -79, -10},{ -70, -10}}, color = {0,0,127}, smooth = Smooth.None));
      connect(position.flange,slidingMass4.flange_a) annotation(Line(points = {{ -48, -10},{ -40, -10}}, color = {0,127,0}, smooth = Smooth.None));
      connect(position1.flange,slidingMass5.flange_a) annotation(Line(points = {{ -48, -50},{ -40, -50}}, color = {0,127,0}, smooth = Smooth.None));
      connect(position2.flange,slidingMass6.flange_a) annotation(Line(points = {{ -48, -90},{ -40, -90}}, color = {0,127,0}, smooth = Smooth.None));
      connect(position3.flange,slidingMass7.flange_a) annotation(Line(points = {{52, -10},{60, -10}}, color = {0,127,0}, smooth = Smooth.None));
      connect(sine1.y,position1.v_ref) annotation(Line(points = {{ -79, -50},{ -70, -50}}, color = {0,0,127}, smooth = Smooth.None));
      connect(sine2.y,position2.a_ref) annotation(Line(points = {{ -79, -90},{ -70, -90}}, color = {0,0,127}, smooth = Smooth.None));
      connect(sine3.y,position3.u) annotation(Line(points = {{21, -10},{30, -10}}, color = {0,0,127}, smooth = Smooth.None));
      connect(force.flange,springDamper.flange_a) annotation(Line(points = {{ -50,30},{ -30,30}}, color = {0,127,0}, smooth = Smooth.None));
      connect(slidingMass.flange_b,relativeStates.flange_a) annotation(Line(points = {{ -40,70},{ -34,70},{ -34,92},{ -2,92}}, color = {0,127,0}, smooth = Smooth.None));
      connect(relativeStates.flange_b,damper.flange_a) annotation(Line(points = {{18,92},{26,92},{26,70}}, color = {0,127,0}, smooth = Smooth.None));
      connect(sine4.y,force.f) annotation(Line(points = {{ -79,30},{ -72,30}}, color = {0,0,127}, smooth = Smooth.None));
    end AllComponents;
  end Translational;
  package Rotational "Test models for Modelica.Mechanics.Rotational"
    package ForUsersGuide "Models used as screenshot for the users guide"
      model SimpleDriveTrain
        Modelica.Blocks.Sources.Sine sine annotation(Placement(transformation(extent = {{ -80,0},{ -60,20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Sources.Torque torque(useSupport = true) annotation(Placement(transformation(extent = {{ -44,0},{ -24,20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Fixed fixed annotation(Placement(transformation(extent = {{ -44, -20},{ -24,0}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia J1(J = 0.2) annotation(Placement(transformation(extent = {{ -10,0},{10,20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.IdealGear gear(ratio = 5, useSupport = true) annotation(Placement(transformation(extent = {{20,0},{40,20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia J2(J = 5) annotation(Placement(transformation(extent = {{52,0},{72,20}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1.1));

      equation 
        connect(sine.y,torque.tau) annotation(Line(points = {{ -59,10},{ -46,10}}, color = {0,0,127}, smooth = Smooth.None));
        connect(torque.support,fixed.flange) annotation(Line(points = {{ -34,0},{ -34, -10}}, color = {0,0,0}, smooth = Smooth.None));
        connect(torque.flange,J1.flange_a) annotation(Line(points = {{ -24,10},{ -10,10}}, color = {0,0,0}, smooth = Smooth.None));
        connect(gear.flange_b,J2.flange_a) annotation(Line(points = {{40,10},{52,10}}, color = {0,0,0}, smooth = Smooth.None));
        connect(fixed.flange,gear.support) annotation(Line(points = {{ -34, -10},{30, -10},{30,0}}, color = {0,0,0}, smooth = Smooth.None));
        connect(J1.flange_b,gear.flange_a) annotation(Line(points = {{10,10},{20,10}}, color = {0,0,0}, smooth = Smooth.None));
      end SimpleDriveTrain;
      model ConnectionPossibilities
        Modelica.Mechanics.Rotational.Components.Inertia inertia1 annotation(Placement(transformation(extent = {{ -60,40},{ -40,60}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia2 annotation(Placement(transformation(extent = {{ -20,40},{0,60}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 2), experimentSetupOutput);
        Modelica.Mechanics.Rotational.Components.Inertia inertia3 annotation(Placement(transformation(extent = {{ -60, -20},{ -40,0}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia4 annotation(Placement(transformation(extent = {{30, -20},{50,0}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Spring spring1(c = 10000.0) annotation(Placement(transformation(extent = {{ -30, -20},{ -10,0}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Spring spring2(c = 10000.0) annotation(Placement(transformation(extent = {{0, -20},{20,0}}, rotation = 0)));
        annotation(Diagram, experiment(StopTime = 1.1));

      equation 
        connect(inertia1.flange_b,inertia2.flange_a) annotation(Line(points = {{ -40,50},{ -20,50}}, color = {0,0,0}, smooth = Smooth.None));
        connect(inertia3.flange_b,spring1.flange_a) annotation(Line(points = {{ -40, -10},{ -30, -10}}, color = {0,0,0}, smooth = Smooth.None));
        connect(spring1.flange_b,spring2.flange_a) annotation(Line(points = {{ -10, -10},{0, -10}}, color = {0,0,0}, smooth = Smooth.None));
        connect(spring2.flange_b,inertia4.flange_a) annotation(Line(points = {{20, -10},{30, -10}}, color = {0,0,0}, smooth = Smooth.None));
      end ConnectionPossibilities;
      model SupportTorque1
        Modelica.Blocks.Sources.Sine sine2 annotation(Placement(transformation(extent = {{ -80,0},{ -60,20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Sources.Torque torque2(useSupport = true) annotation(Placement(transformation(extent = {{ -44,0},{ -24,20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Fixed fixed1 annotation(Placement(transformation(extent = {{ -44, -20},{ -24,0}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia3(J = 1) annotation(Placement(transformation(extent = {{ -14,0},{6,20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.IdealGear idealGear2(ratio = 1, useSupport = true) annotation(Placement(transformation(extent = {{16,0},{36,20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia4(J = 1) annotation(Placement(transformation(extent = {{74,0},{94,20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.BearingFriction bearingFriction1(useSupport = true) annotation(Placement(transformation(extent = {{46,0},{66,20}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1.1));
        annotation(Diagram, experiment(StopTime = 2), experimentSetupOutput);

      equation 
        connect(sine2.y,torque2.tau) annotation(Line(points = {{ -59,10},{ -46,10}}, color = {0,0,127}, smooth = Smooth.None));
        connect(torque2.support,fixed1.flange) annotation(Line(points = {{ -34,0},{ -34, -10}}, color = {0,0,0}, smooth = Smooth.None));
        connect(torque2.flange,inertia3.flange_a) annotation(Line(points = {{ -24,10},{ -14,10}}, color = {0,0,0}, smooth = Smooth.None));
        connect(fixed1.flange,idealGear2.support) annotation(Line(points = {{ -34, -10},{26, -10},{26,0}}, color = {0,0,0}, smooth = Smooth.None));
        connect(inertia3.flange_b,idealGear2.flange_a) annotation(Line(points = {{6,10},{16,10}}, color = {0,0,0}, smooth = Smooth.None));
        connect(idealGear2.flange_b,bearingFriction1.flange_a) annotation(Line(points = {{36,10},{46,10}}, color = {0,0,0}, smooth = Smooth.None));
        connect(bearingFriction1.flange_b,inertia4.flange_a) annotation(Line(points = {{66,10},{74,10}}, color = {0,0,0}, smooth = Smooth.None));
        connect(fixed1.flange,bearingFriction1.support) annotation(Line(points = {{ -34, -10},{56, -10},{56,0}}, color = {0,0,0}, smooth = Smooth.None));
      end SupportTorque1;
      model Directions
        Modelica.Blocks.Sources.Sine sine annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Sources.Torque torque(useSupport = true) annotation(Placement(transformation(extent = {{ -44,60},{ -24,80}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia J1(J = 0.2) annotation(Placement(transformation(extent = {{ -10,60},{10,80}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.IdealGear gear(ratio = 5, useSupport = true) annotation(Placement(transformation(extent = {{20,60},{40,80}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia J2(J = 5) annotation(Placement(transformation(extent = {{52,60},{72,80}}, rotation = 0)));
        Modelica.Blocks.Sources.Sine sine1 annotation(Placement(transformation(extent = {{ -80,10},{ -60,30}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Sources.Torque torque1(useSupport = true) annotation(Placement(transformation(extent = {{ -44,10},{ -24,30}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia J3(J = 0.2) annotation(Placement(transformation(extent = {{ -10,10},{10,30}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.IdealGear gear1(ratio = 1 / 5, useSupport = true) annotation(Placement(transformation(extent = {{40,10},{20,30}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia J4(J = 5) annotation(Placement(transformation(extent = {{52,10},{72,30}}, rotation = 0)));
        Modelica.Blocks.Sources.Sine sine2 annotation(Placement(transformation(extent = {{ -80, -50},{ -60, -30}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Sources.Torque torque2(useSupport = true) annotation(Placement(transformation(extent = {{ -44, -50},{ -24, -30}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia J5(J = 0.2) annotation(Placement(transformation(extent = {{ -10, -50},{10, -30}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.IdealGear gear2(ratio = 1 / 5, useSupport = true) annotation(Placement(transformation(extent = {{40, -50},{20, -30}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia J6(J = 5) annotation(Placement(transformation(extent = {{72, -50},{52, -30}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -23,97},{ -13,94},{ -23,91},{ -23,97}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -23,94},{ -53,94}}, color = {0,0,0}),Text(extent = {{ -18,98},{72,90}}, lineColor = {0,0,0}, textString = "(positive) axis of rotation")}), experiment(StopTime = 2), experimentSetupOutput);
        Modelica.Mechanics.Rotational.Components.Fixed fixed1 annotation(Placement(transformation(extent = {{ -10,42},{10,62}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Fixed fixed2 annotation(Placement(transformation(extent = {{ -10, -6},{10,14}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Fixed fixed3 annotation(Placement(transformation(extent = {{ -10, -66},{10, -46}}, rotation = 0)));

      equation 
        connect(sine.y,torque.tau) annotation(Line(points = {{ -59,70},{ -46,70}}, color = {0,0,127}, smooth = Smooth.None));
        connect(torque.flange,J1.flange_a) annotation(Line(points = {{ -24,70},{ -10,70}}, color = {0,0,0}, smooth = Smooth.None));
        connect(gear.flange_b,J2.flange_a) annotation(Line(points = {{40,70},{52,70}}, color = {0,0,0}, smooth = Smooth.None));
        connect(J1.flange_b,gear.flange_a) annotation(Line(points = {{10,70},{20,70}}, color = {0,0,0}, smooth = Smooth.None));
        connect(sine1.y,torque1.tau) annotation(Line(points = {{ -59,20},{ -46,20}}, color = {0,0,127}, smooth = Smooth.None));
        connect(torque1.flange,J3.flange_a) annotation(Line(points = {{ -24,20},{ -10,20}}, color = {0,0,0}, smooth = Smooth.None));
        connect(J3.flange_b,gear1.flange_b) annotation(Line(points = {{10,20},{20,20}}, color = {0,0,0}, smooth = Smooth.None));
        connect(gear1.flange_a,J4.flange_a) annotation(Line(points = {{40,20},{52,20}}, color = {0,0,0}, smooth = Smooth.None));
        connect(sine2.y,torque2.tau) annotation(Line(points = {{ -59, -40},{ -46, -40}}, color = {0,0,127}, smooth = Smooth.None));
        connect(torque2.flange,J5.flange_a) annotation(Line(points = {{ -24, -40},{ -10, -40}}, color = {0,0,0}, smooth = Smooth.None));
        connect(J5.flange_b,gear2.flange_b) annotation(Line(points = {{10, -40},{20, -40}}, color = {0,0,0}, smooth = Smooth.None));
        connect(gear2.flange_a,J6.flange_b) annotation(Line(points = {{40, -40},{52, -40}}, color = {0,0,0}, smooth = Smooth.None));
        connect(torque.support,fixed1.flange) annotation(Line(points = {{ -34,60},{ -34,52},{0,52}}, color = {0,0,0}, smooth = Smooth.None));
        connect(fixed1.flange,gear.support) annotation(Line(points = {{0,52},{30,52},{30,60}}, color = {0,0,0}, smooth = Smooth.None));
        connect(torque1.support,fixed2.flange) annotation(Line(points = {{ -34,10},{ -34,4},{0,4}}, color = {0,0,0}, smooth = Smooth.None));
        connect(fixed2.flange,gear1.support) annotation(Line(points = {{0,4},{30,4},{30,10}}, color = {0,0,0}, smooth = Smooth.None));
        connect(torque2.support,fixed3.flange) annotation(Line(points = {{ -34, -50},{ -34, -56},{0, -56}}, color = {0,0,0}, smooth = Smooth.None));
        connect(fixed3.flange,gear2.support) annotation(Line(points = {{0, -56},{30, -56},{30, -50}}, color = {0,0,0}, smooth = Smooth.None));
      end Directions;
      model Directions2
        Modelica.Mechanics.Rotational.Components.Inertia J2(J = 5) annotation(Placement(transformation(extent = {{ -60,55},{ -40,75}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -28,94},{62,86}}, lineColor = {0,0,0}, textString = "(positive) axis of rotation"),Line(points = {{ -61,65},{ -80,65}}, color = {0,0,0}),Polygon(points = {{ -68,67},{ -61,65},{ -68,63},{ -68,67}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -57,69},{ -91,73}}, lineColor = {0,0,0}, textString = "J2.flange_a.tau"),Line(points = {{ -42,45},{ -61,45}}, color = {0,0,0}),Polygon(points = {{ -49,47},{ -42,45},{ -49,43},{ -49,47}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -33,38},{ -67,42}}, lineColor = {0,0,0}, textString = "J2.w"),Line(points = {{ -9,65},{ -28,65}}, color = {0,0,0}),Polygon(points = {{ -16,67},{ -9,65},{ -16,63},{ -16,67}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -5,69},{ -39,73}}, lineColor = {0,0,0}, textString = "J4.flange_a.tau"),Line(points = {{10,45},{ -9,45}}, color = {0,0,0}),Polygon(points = {{3,47},{10,45},{3,43},{3,47}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{19,38},{ -15,42}}, lineColor = {0,0,0}, textString = "J4.w"),Line(points = {{44,65},{25,65}}, color = {0,0,0}),Polygon(points = {{37,67},{44,65},{37,63},{37,67}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{48,69},{14,73}}, lineColor = {0,0,0}, textString = "J6.flange_b.tau"),Line(points = {{63,45},{44,45}}, color = {0,0,0}),Polygon(points = {{56,47},{63,45},{56,43},{56,47}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{72,38},{38,42}}, lineColor = {0,0,0}, textString = "J6.w"),Line(points = {{ -32,90},{ -62,90}}, color = {0,0,0}),Polygon(points = {{ -32,93},{ -22,90},{ -32,87},{ -32,93}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -19,24},{71,16}}, lineColor = {0,0,0}, textString = "(positive) axis of rotation"),Line(points = {{ -23,20},{ -53,20}}, color = {0,0,0}),Polygon(points = {{ -23,23},{ -13,20},{ -23,17},{ -23,23}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{ -47, -47},{ -54, -49},{ -47, -51},{ -47, -47}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -52, -49},{ -33, -49}}, color = {0,0,0}),Line(points = {{31.5, -79.5},{31.5, -60.5}}, color = {0,0,0}),Polygon(points = {{29.5, -75.5},{31.5, -82.5},{33.5, -75.5},{29.5, -75.5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), experiment(StopTime = 2));
        Modelica.Mechanics.Rotational.Components.Inertia J4(J = 5) annotation(Placement(transformation(extent = {{ -8,55},{12,75}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia J6(J = 5) annotation(Placement(transformation(extent = {{65,55},{45,75}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia1 annotation(Placement(transformation(extent = {{ -30, -10},{ -10,10}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Sources.Torque torque1(useSupport = true) annotation(Placement(transformation(extent = {{ -60, -10},{ -40,10}}, rotation = 0)));
        Modelica.Blocks.Sources.Sine sine1 annotation(Placement(transformation(extent = {{ -90, -10},{ -70,10}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia2 annotation(Placement(transformation(extent = {{10, -10},{30,10}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Sources.Torque torque2(useSupport = true) annotation(Placement(transformation(extent = {{60, -10},{40,10}}, rotation = 0)));
        Modelica.Blocks.Sources.Sine sine2 annotation(Placement(transformation(extent = {{90, -10},{70,10}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia3 annotation(Placement(transformation(extent = {{ -80, -100},{ -60, -80}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia4 annotation(Placement(transformation(extent = {{ -6, -64},{ -26, -44}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.IdealPlanetary idealPlanetary1 annotation(Placement(transformation(extent = {{ -50, -100},{ -30, -80}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.IdealPlanetary idealPlanetary2 annotation(Placement(transformation(extent = {{60, -100},{80, -80}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia5 annotation(Placement(transformation(extent = {{0, -100},{20, -80}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia6 annotation(Placement(transformation(origin = {40, -50}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
        Modelica.Mechanics.Rotational.Components.Fixed fixed1 annotation(Placement(transformation(extent = {{ -60, -27},{ -40, -7}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Fixed fixed2 annotation(Placement(transformation(extent = {{40, -24},{60, -4}}, rotation = 0)));

      equation 
        connect(torque1.flange,inertia1.flange_a) annotation(Line(points = {{ -40,0},{ -30,0}}, color = {0,0,0}, smooth = Smooth.None));
        connect(sine1.y,torque1.tau) annotation(Line(points = {{ -69,0},{ -62,0}}, color = {0,0,127}, smooth = Smooth.None));
        connect(torque2.flange,inertia2.flange_b) annotation(Line(points = {{40,0},{30,0}}, color = {0,0,0}, smooth = Smooth.None));
        connect(sine2.y,torque2.tau) annotation(Line(points = {{69,0},{62,0}}, color = {0,0,127}, smooth = Smooth.None));
        connect(idealPlanetary1.sun,inertia3.flange_b) annotation(Line(points = {{ -50, -90},{ -60, -90}}, color = {0,0,0}, smooth = Smooth.None));
        connect(idealPlanetary1.carrier,inertia4.flange_b) annotation(Line(points = {{ -50, -86},{ -56, -86},{ -56, -54},{ -26, -54}}, color = {0,0,0}, smooth = Smooth.None));
        connect(inertia5.flange_b,idealPlanetary2.sun) annotation(Line(points = {{20, -90},{60, -90}}, color = {0,0,0}, smooth = Smooth.None));
        connect(inertia6.flange_a,idealPlanetary2.carrier) annotation(Line(points = {{40, -60},{40, -86},{60, -86}}, color = {0,0,0}, smooth = Smooth.None));
        connect(torque1.support,fixed1.flange) annotation(Line(points = {{ -50, -10},{ -50, -17}}, color = {0,0,0}, smooth = Smooth.None));
        connect(fixed2.flange,torque2.support) annotation(Line(points = {{50, -14},{50, -10}}, color = {0,0,0}, smooth = Smooth.None));
      end Directions2;
      model Directions3
        Modelica.Mechanics.Rotational.Components.Inertia J1(J = 1) annotation(Placement(transformation(extent = {{ -80,40},{ -60,60}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -60,33},{ -79,33}}, color = {0,0,0}),Polygon(points = {{ -67,35},{ -60,33},{ -67,31},{ -67,35}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{ -32,35},{ -39,33},{ -32,31},{ -32,35}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -16,33},{ -35,33}}, color = {0,0,0}),Text(extent = {{ -10,60},{14,40}}, lineColor = {0,0,0}, textString = "="),Line(points = {{40,33},{21,33}}, color = {0,0,0}),Polygon(points = {{33,35},{40,33},{33,31},{33,35}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{80,27},{61,27}}, color = {0,0,0}),Polygon(points = {{73,29},{80,27},{73,25},{73,29}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), experiment(StopTime = 2));
        Modelica.Mechanics.Rotational.Components.Inertia J2(J = 1) annotation(Placement(transformation(extent = {{ -20,40},{ -40,60}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia J3(J = 1) annotation(Placement(transformation(extent = {{20,40},{40,60}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia J4(J = 1) annotation(Placement(transformation(extent = {{60,40},{80,60}}, rotation = 0)));

      equation 
        connect(J1.flange_b,J2.flange_b) annotation(Line(points = {{ -60,50},{ -40,50}}, color = {0,0,0}, smooth = Smooth.None));
        connect(J3.flange_b,J4.flange_b) annotation(Line(points = {{40,50},{50,50},{50,34},{86,34},{86,50},{80,50}}, color = {0,0,0}, smooth = Smooth.None));
      end Directions3;
    end ForUsersGuide;
    package InitializationConversion
      model Inertia
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1.1));
        Modelica.Mechanics.Rotational.Components.Inertia inertia1(phi(start = 0.0174532925199433), w(start = 2)) annotation(Placement(transformation(extent = {{ -86,40},{ -66,60}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia2(stateSelect = StateSelect.default, a(fixed = true), phi(fixed = false, start = 0.0174532925199433), w(fixed = true, start = 2)) annotation(Placement(transformation(extent = {{0,40},{20,60}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia3(stateSelect = StateSelect.prefer, a(fixed = false), phi(fixed = true), w(fixed = true)) annotation(Placement(transformation(extent = {{40,40},{60,60}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia4(stateSelect = StateSelect.always, a(fixed = false), phi(fixed = true, start =  -0.0174532925199433), w(fixed = false, start =  -2)) annotation(Placement(transformation(extent = {{80,40},{100,60}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia5(stateSelect = StateSelect.avoid, a(fixed = false), phi(fixed = false, start = 0.383972435438752), w(fixed = true, start = 33)) annotation(Placement(transformation(extent = {{ -86,0},{ -66,20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia6(a(fixed = true, start = 44), phi(fixed = false), w(fixed = false)) annotation(Placement(transformation(extent = {{ -14,0},{6,20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia7(a(fixed = true, start = 3), phi(fixed = true, start = 0.0174532925199433), w(fixed = false, start = 2)) annotation(Placement(transformation(extent = {{22,0},{42,20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia8(a(fixed = true), phi(fixed = false), w(fixed = true)) annotation(Placement(transformation(extent = {{ -84, -40},{ -64, -20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia9(a(fixed = true, start = 33), phi(fixed = true, start = 0.191986217719376), w(fixed = true, start = 22)) annotation(Placement(transformation(extent = {{20, -40},{40, -20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Spring spring(c = 10000) annotation(Placement(transformation(extent = {{ -28,40},{ -8,60}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Fixed fixed annotation(Placement(transformation(extent = {{ -50,40},{ -30,60}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Spring spring1(c = 10000) annotation(Placement(transformation(extent = {{50,0},{70,20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia7a(a(start = 3), phi(start = 0.0174532925199433), w(start = 2)) annotation(Placement(transformation(extent = {{80,0},{100,20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Spring spring2(c = 10000) annotation(Placement(transformation(extent = {{ -40,0},{ -20,20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Fixed fixed1 annotation(Placement(transformation(extent = {{ -60,0},{ -40,20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Spring spring3(c = 10000) annotation(Placement(transformation(extent = {{ -52, -40},{ -32, -20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia8a annotation(Placement(transformation(extent = {{ -26, -40},{ -6, -20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Spring spring4(c = 10000) annotation(Placement(transformation(extent = {{52, -40},{72, -20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia9a(a(start = 33), phi(start = 0.191986217719376), w(start = 22)) annotation(Placement(transformation(extent = {{80, -40},{100, -20}}, rotation = 0)));

      equation 
        connect(spring.flange_b,inertia2.flange_a) annotation(Line(points = {{ -8,50},{0,50}}, color = {0,0,0}, smooth = Smooth.None));
        connect(fixed.flange,spring.flange_a) annotation(Line(points = {{ -40,50},{ -28,50}}, color = {0,0,0}, smooth = Smooth.None));
        connect(inertia7.flange_b,spring1.flange_a) annotation(Line(points = {{42,10},{50,10}}, color = {0,0,0}, smooth = Smooth.None));
        connect(spring1.flange_b,inertia7a.flange_a) annotation(Line(points = {{70,10},{80,10}}, color = {0,0,0}, smooth = Smooth.None));
        connect(spring2.flange_b,inertia6.flange_a) annotation(Line(points = {{ -20,10},{ -14,10}}, color = {0,0,0}, smooth = Smooth.None));
        connect(fixed1.flange,spring2.flange_a) annotation(Line(points = {{ -50,10},{ -40,10}}, color = {0,0,0}, smooth = Smooth.None));
        connect(inertia8.flange_b,spring3.flange_a) annotation(Line(points = {{ -64, -30},{ -52, -30}}, color = {0,0,0}, smooth = Smooth.None));
        connect(spring3.flange_b,inertia8a.flange_a) annotation(Line(points = {{ -32, -30},{ -26, -30}}, color = {0,0,0}, smooth = Smooth.None));
        connect(inertia9.flange_b,spring4.flange_a) annotation(Line(points = {{40, -30},{52, -30}}, color = {0,0,0}, smooth = Smooth.None));
        connect(spring4.flange_b,inertia9a.flange_a) annotation(Line(points = {{72, -30},{80, -30}}, color = {0,0,0}, smooth = Smooth.None));
      end Inertia;
      model SpringDamper
        Modelica.Mechanics.Rotational.Components.SpringDamper springDamper1(c = 10000.0, d = 10, phi_rel(start = 1), w_rel(start = 2)) annotation(Placement(transformation(extent = {{ -70,60},{ -50,80}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Fixed fixed annotation(Placement(transformation(extent = {{ -100,60},{ -80,80}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1.1));
        Modelica.Mechanics.Rotational.Components.Inertia inertia1 annotation(Placement(transformation(extent = {{ -40,60},{ -20,80}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia2 annotation(Placement(transformation(extent = {{ -40,20},{ -20,40}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.SpringDamper springDamper2(c = 10000.0, d = 10, stateSelect = StateSelect.prefer, a_rel(fixed = true), phi_rel(fixed = false, start =  -1), w_rel(fixed = true, start =  -2)) annotation(Placement(transformation(extent = {{ -70,20},{ -50,40}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.SpringDamper springDamper3(c = 10000.0, d = 10, stateSelect = StateSelect.always, a_rel(fixed = false), phi_rel(fixed = true, start = 1), w_rel(fixed = true, start = 2)) annotation(Placement(transformation(extent = {{ -70, -20},{ -50,0}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia3 annotation(Placement(transformation(extent = {{ -40, -20},{ -20,0}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.SpringDamper springDamper4(c = 10000.0, d = 10, stateSelect = StateSelect.avoid, a_rel(fixed = false), phi_rel(fixed = true), w_rel(fixed = false)) annotation(Placement(transformation(extent = {{ -70, -60},{ -50, -40}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia4 annotation(Placement(transformation(extent = {{ -40, -60},{ -20, -40}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.SpringDamper springDamper5(c = 10000.0, d = 10, stateSelect = StateSelect.never, a_rel(fixed = false), phi_rel(fixed = false), w_rel(fixed = true, start = 3)) annotation(Placement(transformation(extent = {{ -70, -100},{ -50, -80}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia5 annotation(Placement(transformation(extent = {{ -40, -100},{ -20, -80}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia6 annotation(Placement(transformation(extent = {{20,40},{40,60}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Spring spring(c = 10000.0) annotation(Placement(transformation(extent = {{48,40},{68,60}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia7 annotation(Placement(transformation(extent = {{74,40},{94,60}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.RelativeStates relativeStates annotation(Placement(transformation(extent = {{48,20},{68,40}}, rotation = 0)));

      equation 
        connect(springDamper1.flange_a,fixed.flange) annotation(Line(points = {{ -70,70},{ -90,70}}, color = {0,0,0}, smooth = Smooth.None));
        connect(springDamper1.flange_b,inertia1.flange_a) annotation(Line(points = {{ -50,70},{ -40,70}}, color = {0,0,0}, smooth = Smooth.None));
        connect(inertia2.flange_a,springDamper2.flange_b) annotation(Line(points = {{ -40,30},{ -50,30}}, color = {0,0,0}, smooth = Smooth.None));
        connect(springDamper2.flange_a,fixed.flange) annotation(Line(points = {{ -70,30},{ -80,30},{ -80,70},{ -90,70}}, color = {0,0,0}, smooth = Smooth.None));
        connect(inertia3.flange_a,springDamper3.flange_b) annotation(Line(points = {{ -40, -10},{ -50, -10}}, color = {0,0,0}, smooth = Smooth.None));
        connect(springDamper3.flange_a,fixed.flange) annotation(Line(points = {{ -70, -10},{ -80, -10},{ -80,70},{ -90,70}}, color = {0,0,0}, smooth = Smooth.None));
        connect(inertia4.flange_a,springDamper4.flange_b) annotation(Line(points = {{ -40, -50},{ -50, -50}}, color = {0,0,0}, smooth = Smooth.None));
        connect(springDamper4.flange_a,fixed.flange) annotation(Line(points = {{ -70, -50},{ -80, -50},{ -80,70},{ -90,70}}, color = {0,0,0}, smooth = Smooth.None));
        connect(inertia5.flange_a,springDamper5.flange_b) annotation(Line(points = {{ -40, -90},{ -50, -90}}, color = {0,0,0}, smooth = Smooth.None));
        connect(springDamper5.flange_a,fixed.flange) annotation(Line(points = {{ -70, -90},{ -80, -90},{ -80,70},{ -90,70}}, color = {0,0,0}, smooth = Smooth.None));
        connect(inertia6.flange_b,spring.flange_a) annotation(Line(points = {{40,50},{48,50}}, color = {0,0,0}, smooth = Smooth.None));
        connect(spring.flange_b,inertia7.flange_a) annotation(Line(points = {{68,50},{74,50}}, color = {0,0,0}, smooth = Smooth.None));
        connect(relativeStates.flange_b,spring.flange_b) annotation(Line(points = {{68,30},{68,50}}, color = {0,0,0}, smooth = Smooth.None));
        connect(spring.flange_a,relativeStates.flange_a) annotation(Line(points = {{48,50},{48,30},{48,30}}, color = {0,0,0}, smooth = Smooth.None));
      end SpringDamper;
    end InitializationConversion;
    model TestSpeed
      Modelica.Mechanics.Rotational.Sources.Speed speed1(f_crit = 5, useSupport = false) annotation(Placement(transformation(extent = {{ -20,60},{0,80}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia1 annotation(Placement(transformation(extent = {{20,60},{40,80}}, rotation = 0)));
      annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1.1));
      Modelica.Mechanics.Rotational.Sources.Speed speed2(f_crit = 5, exact = true, useSupport = false) annotation(Placement(transformation(extent = {{ -20,20},{0,40}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia2 annotation(Placement(transformation(extent = {{20,20},{40,40}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine2 annotation(Placement(transformation(extent = {{ -60,20},{ -40,40}}, rotation = 0)));
      Modelica.Blocks.Sources.Step step(startTime = 0.5) annotation(Placement(transformation(extent = {{ -60,60},{ -40,80}}, rotation = 0)));
      Modelica.Mechanics.Translational.Sources.Speed speed3(f_crit = 5, s(start = 1)) annotation(Placement(transformation(extent = {{ -20, -20},{0,0}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.Mass mass1(m = 2) annotation(Placement(transformation(extent = {{20, -20},{40,0}}, rotation = 0)));
      Modelica.Blocks.Sources.Step step1(startTime = 0.5) annotation(Placement(transformation(extent = {{ -60, -20},{ -40,0}}, rotation = 0)));
      Modelica.Mechanics.Translational.Sources.Speed speed4(exact = true, s(start = 1)) annotation(Placement(transformation(extent = {{ -20, -60},{0, -40}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.Mass mass2(m = 2) annotation(Placement(transformation(extent = {{20, -60},{40, -40}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine4 annotation(Placement(transformation(extent = {{ -60, -60},{ -40, -40}}, rotation = 0)));

    equation 
      connect(speed1.flange,inertia1.flange_a) annotation(Line(points = {{0,70},{20,70}}, color = {0,0,0}));
      connect(speed2.flange,inertia2.flange_a) annotation(Line(points = {{0,30},{20,30}}, color = {0,0,0}));
      connect(sine2.y,speed2.w_ref) annotation(Line(points = {{ -39,30},{ -22,30}}, color = {0,0,127}));
      connect(step.y,speed1.w_ref) annotation(Line(points = {{ -39,70},{ -22,70}}, color = {0,0,127}));
      connect(speed3.flange,mass1.flange_a) annotation(Line(points = {{0, -10},{20, -10}}, color = {0,127,0}));
      connect(step1.y,speed3.v_ref) annotation(Line(points = {{ -39, -10},{ -22, -10}}, color = {0,0,127}));
      connect(speed4.flange,mass2.flange_a) annotation(Line(points = {{0, -50},{20, -50}}, color = {0,127,0}));
      connect(sine4.y,speed4.v_ref) annotation(Line(points = {{ -39, -50},{ -22, -50}}, color = {0,0,127}));
    end TestSpeed;
    model TestMove
      annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1.1));
      Modelica.Mechanics.MultiBody.Parts.Body Body1(I_33 = 2) annotation(Placement(transformation(extent = {{66,36},{86,56}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Joints.Revolute ActuatedRevolute1(useAxisFlange = true) annotation(Placement(transformation(extent = {{32,0},{52,20}}, rotation = 0)));
      inner Modelica.Mechanics.MultiBody.World world(g = 0) annotation(Placement(transformation(extent = {{0,0},{20,20}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.Move move(useSupport = false) annotation(Placement(transformation(extent = {{20,40},{40,60}}, rotation = 0)));
      Modelica.Blocks.Routing.Multiplex3 Multiplex3_1 annotation(Placement(transformation(extent = {{ -56,40},{ -36,60}}, rotation = 0)));
      Modelica.Blocks.Sources.Constant Constant1 annotation(Placement(transformation(extent = {{ -100,16},{ -80,36}}, rotation = 0)));
      Modelica.Blocks.Sources.Constant Constant2(k = 0) annotation(Placement(transformation(extent = {{ -100,70},{ -80,90}}, rotation = 0)));
      Modelica.Mechanics.Translational.Sources.Move move1 annotation(Placement(transformation(extent = {{ -20, -40},{0, -20}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.Mass slidingMass(m = 2) annotation(Placement(transformation(extent = {{20, -40},{40, -20}}, rotation = 0)));

    equation 
      connect(ActuatedRevolute1.frame_b,Body1.frame_a) annotation(Line(points = {{52,10},{62,10},{62,46},{66,46}}, color = {0,0,0}, thickness = 0.5));
      connect(world.frame_b,ActuatedRevolute1.frame_a) annotation(Line(points = {{20,10},{32,10}}, color = {0,0,0}, thickness = 0.5));
      connect(move.flange,ActuatedRevolute1.axis) annotation(Line(points = {{40,50},{42,50},{42,20}}, color = {0,0,0}));
      connect(Multiplex3_1.y,move.u) annotation(Line(points = {{ -35,50},{18,50}}, color = {0,0,127}));
      connect(Constant1.y,Multiplex3_1.u3[1]) annotation(Line(points = {{ -79,26},{ -74,26},{ -74,43},{ -58,43}}, color = {0,0,127}));
      connect(Constant2.y,Multiplex3_1.u1[1]) annotation(Line(points = {{ -79,80},{ -74,80},{ -74,57},{ -58,57}}, color = {0,0,127}));
      connect(Constant2.y,Multiplex3_1.u2[1]) annotation(Line(points = {{ -79,80},{ -74,80},{ -74,50},{ -58,50}}, color = {0,0,127}));
      connect(Multiplex3_1.y,move1.u) annotation(Line(points = {{ -35,50},{ -28,50},{ -28, -30},{ -22, -30}}, color = {0,0,127}));
      connect(move1.flange,slidingMass.flange_a) annotation(Line(points = {{0, -30},{20, -30}}, color = {0,127,0}));
    end TestMove;
    model AllComponents
      Modelica.Mechanics.Rotational.Components.Inertia inertia(J = 3) annotation(Placement(transformation(extent = {{ -60,40},{ -40,60}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.IdealGear idealGear(ratio = 3, useSupport = true) annotation(Placement(transformation(extent = {{ -30,40},{ -10,60}}, rotation = 0)));
      annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -140, -200},{200,100}}, initialScale = 0.1), graphics), experiment(StopTime = 0.9));
      Modelica.Mechanics.Rotational.Components.IdealPlanetary idealPlanetary(ratio = 3) annotation(Placement(transformation(extent = {{4,40},{24,60}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Fixed fixed annotation(Placement(transformation(extent = {{ -30,10},{ -10,30}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia1(J = 2) annotation(Placement(transformation(extent = {{ -30,80},{ -10,100}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia2(J = 4) annotation(Placement(transformation(extent = {{60,40},{80,60}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Spring spring(c = 10000.0) annotation(Placement(transformation(extent = {{32,40},{52,60}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Damper damper(d = 10) annotation(Placement(transformation(extent = {{32,58},{52,78}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.SpringDamper springDamper(c = 10000.0, d = 10, phi_rel0 = 1.74532925199433e-05) annotation(Placement(transformation(extent = {{ -58,80},{ -38,100}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia3(J = 2, phi(start = 0.0174532925199433), w(start = 2)) annotation(Placement(transformation(extent = {{ -92,80},{ -72,100}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.ElastoBacklash elastoBacklash(b = 1.74532925199433e-07, c = 10000.0, d = 20) annotation(Placement(transformation(extent = {{10,80},{30,100}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia4(J = 2) annotation(Placement(transformation(extent = {{40,80},{60,100}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.BearingFriction bearingFriction(tau_pos = [0,2], peak = 1.1, useSupport = true) annotation(Placement(transformation(extent = {{64,80},{84,100}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Clutch clutch(mue_pos = [0,1], peak = 1.1, cgeo = 1.2, fn_max = 1.3) annotation(Placement(transformation(extent = {{ -86,60},{ -66,40}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia5(J = 3) annotation(Placement(transformation(extent = {{ -116,40},{ -96,60}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine annotation(Placement(transformation(extent = {{ -104,10},{ -84,30}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Brake brake(useSupport = true) annotation(Placement(transformation(extent = {{100,40},{120,60}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine1 annotation(Placement(transformation(origin = {110,90}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
      Modelica.Mechanics.Rotational.Components.LossyGear lossyGear(useSupport = true) annotation(Placement(transformation(extent = {{ -120,80},{ -100,100}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia6(J = 2, a(fixed = false), phi(fixed = true, start = 0.0174532925199433), w(fixed = true, start = 2)) annotation(Placement(transformation(extent = {{ -150,80},{ -130,100}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Gearbox gear2_1(ratio = 2, lossTable = [0,0.9,1,0,0], c = 10000.0, d = 10, b = 1.74532925199433e-07, useSupport = true) annotation(Placement(transformation(extent = {{ -80, -40},{ -60, -20}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia7(J = 3) annotation(Placement(transformation(extent = {{ -110, -40},{ -90, -20}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia8(J = 3) annotation(Placement(transformation(extent = {{ -48, -40},{ -28, -20}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.Position position(useSupport = true) annotation(Placement(transformation(extent = {{48, -10},{68,10}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Fixed fixed1 annotation(Placement(transformation(extent = {{ -80, -60},{ -60, -40}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Fixed fixed2 annotation(Placement(transformation(extent = {{48, -30},{68, -10}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia9(J = 3) annotation(Placement(transformation(extent = {{80, -10},{100,10}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine2 annotation(Placement(transformation(extent = {{18, -10},{38,10}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine3 annotation(Placement(transformation(extent = {{20, -60},{40, -40}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia10(J = 3) annotation(Placement(transformation(extent = {{80, -60},{100, -40}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Fixed fixed3 annotation(Placement(transformation(extent = {{50, -80},{70, -60}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.Speed speed(useSupport = true) annotation(Placement(transformation(extent = {{50, -60},{70, -40}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine4 annotation(Placement(transformation(extent = {{20, -120},{40, -100}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia11(J = 3) annotation(Placement(transformation(extent = {{80, -120},{100, -100}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Fixed fixed4 annotation(Placement(transformation(extent = {{50, -140},{70, -120}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.Accelerate accelerate(useSupport = true) annotation(Placement(transformation(extent = {{50, -120},{70, -100}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine5[3](each amplitude = 2) annotation(Placement(transformation(extent = {{20, -180},{40, -160}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia12(J = 3) annotation(Placement(transformation(extent = {{80, -180},{100, -160}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Fixed fixed5 annotation(Placement(transformation(extent = {{50, -200},{70, -180}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.Move move(useSupport = true) annotation(Placement(transformation(extent = {{50, -180},{70, -160}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.Torque torque(useSupport = true) annotation(Placement(transformation(extent = {{160,40},{140,60}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine6 annotation(Placement(transformation(extent = {{192,40},{172,60}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.Torque2 torque2_1 annotation(Placement(transformation(extent = {{112, -60},{132, -40}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia13(J = 3) annotation(Placement(transformation(extent = {{140, -60},{160, -40}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine7 annotation(Placement(transformation(extent = {{94, -34},{114, -14}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.LinearSpeedDependentTorque linearSpeedDependentTorque(tau_nominal = 10, w_nominal = 10, useSupport = true) annotation(Placement(transformation(extent = {{ -142, -40},{ -122, -20}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticSpeedDependentTorque(tau_nominal = 100, w_nominal = 1, useSupport = true) annotation(Placement(transformation(extent = {{6, -40},{ -16, -20}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.ConstantTorque constantTorque(tau_constant = 11, useSupport = true) annotation(Placement(transformation(extent = {{ -120, -100},{ -100, -80}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia14(J = 3) annotation(Placement(transformation(extent = {{ -84, -120},{ -64, -100}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.TorqueStep torqueStep(stepTorque = 22, offsetTorque = 11, startTime = 1, useSupport = true) annotation(Placement(transformation(extent = {{ -120, -140},{ -100, -120}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Fixed fixed6 annotation(Placement(transformation(extent = {{ -120, -162},{ -100, -142}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.ConstantSpeed constantSpeed(w_fixed = 11, useSupport = true) annotation(Placement(transformation(extent = {{ -60, -140},{ -40, -120}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia15(J = 3) annotation(Placement(transformation(extent = {{ -30, -140},{ -10, -120}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.RelativeStates relativeStates annotation(Placement(transformation(extent = {{ -60, -80},{ -40, -60}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sensors.AngleSensor angleSensor annotation(Placement(transformation(extent = {{118, -120},{138, -100}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sensors.SpeedSensor speedSensor annotation(Placement(transformation(extent = {{118, -96},{138, -76}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sensors.AccSensor accSensor annotation(Placement(transformation(extent = {{118, -146},{138, -126}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sensors.RelAngleSensor relAngleSensor annotation(Placement(transformation(extent = {{152, -20},{172,0}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sensors.RelSpeedSensor relSpeedSensor annotation(Placement(transformation(extent = {{166, -88},{186, -68}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sensors.RelAccSensor relAccSensor annotation(Placement(transformation(extent = {{160, -130},{180, -110}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sensors.TorqueSensor torqueSensor annotation(Placement(transformation(extent = {{ -26, -170},{ -6, -150}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sensors.PowerSensor powerSensor annotation(Placement(transformation(extent = {{ -58, -170},{ -38, -150}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia16(J = 3) annotation(Placement(transformation(extent = {{ -88, -170},{ -68, -150}}, rotation = 0)));

    equation 
      connect(inertia.flange_b,idealGear.flange_a) annotation(Line(points = {{ -40,50},{ -30,50}}, color = {0,0,0}, smooth = Smooth.None));
      connect(idealGear.flange_b,idealPlanetary.sun) annotation(Line(points = {{ -10,50},{4,50}}, color = {0,0,0}, smooth = Smooth.None));
      connect(idealGear.support,fixed.flange) annotation(Line(points = {{ -20,40},{ -20,20}}, color = {0,0,0}, smooth = Smooth.None));
      connect(idealPlanetary.carrier,inertia1.flange_b) annotation(Line(points = {{4,54},{ -2,54},{ -2,90},{ -10,90}}, color = {0,0,0}, smooth = Smooth.None));
      connect(idealPlanetary.ring,spring.flange_a) annotation(Line(points = {{24,50},{32,50}}, color = {0,0,0}, smooth = Smooth.None));
      connect(spring.flange_b,inertia2.flange_a) annotation(Line(points = {{52,50},{60,50}}, color = {0,0,0}, smooth = Smooth.None));
      connect(idealPlanetary.ring,damper.flange_a) annotation(Line(points = {{24,50},{28,50},{28,68},{32,68}}, color = {0,0,0}, smooth = Smooth.None));
      connect(damper.flange_b,inertia2.flange_a) annotation(Line(points = {{52,68},{60,68},{60,50}}, color = {0,0,0}, smooth = Smooth.None));
      connect(springDamper.flange_b,inertia1.flange_a) annotation(Line(points = {{ -38,90},{ -30,90}}, color = {0,0,0}, smooth = Smooth.None));
      connect(inertia3.flange_b,springDamper.flange_a) annotation(Line(points = {{ -72,90},{ -58,90}}, color = {0,0,0}, smooth = Smooth.None));
      connect(inertia1.flange_b,elastoBacklash.flange_a) annotation(Line(points = {{ -10,90},{10,90}}, color = {0,0,0}, smooth = Smooth.None));
      connect(elastoBacklash.flange_b,inertia4.flange_a) annotation(Line(points = {{30,90},{40,90}}, color = {0,0,0}, smooth = Smooth.None));
      connect(inertia4.flange_b,bearingFriction.flange_a) annotation(Line(points = {{60,90},{64,90}}, color = {0,0,0}, smooth = Smooth.None));
      connect(fixed.flange,bearingFriction.support) annotation(Line(points = {{ -20,20},{92,20},{92,74},{74,74},{74,80}}, color = {0,0,0}, smooth = Smooth.None));
      connect(clutch.flange_b,inertia.flange_a) annotation(Line(points = {{ -66,50},{ -60,50}}, color = {0,0,0}, smooth = Smooth.None));
      connect(inertia5.flange_b,clutch.flange_a) annotation(Line(points = {{ -96,50},{ -86,50}}, color = {0,0,0}, smooth = Smooth.None));
      connect(sine.y,clutch.f_normalized) annotation(Line(points = {{ -83,20},{ -76,20},{ -76,39}}, color = {0,0,127}, smooth = Smooth.None));
      connect(inertia2.flange_b,brake.flange_a) annotation(Line(points = {{80,50},{100,50}}, color = {0,0,0}, smooth = Smooth.None));
      connect(sine1.y,brake.f_normalized) annotation(Line(points = {{110,79},{110,61}}, color = {0,0,127}, smooth = Smooth.None));
      connect(fixed.flange,brake.support) annotation(Line(points = {{ -20,20},{110,20},{110,40}}, color = {0,0,0}, smooth = Smooth.None));
      connect(lossyGear.flange_b,inertia3.flange_a) annotation(Line(points = {{ -100,90},{ -92,90}}, color = {0,0,0}, smooth = Smooth.None));
      connect(inertia6.flange_b,lossyGear.flange_a) annotation(Line(points = {{ -130,90},{ -120,90}}, color = {0,0,0}, smooth = Smooth.None));
      connect(lossyGear.support,fixed.flange) annotation(Line(points = {{ -110,80},{ -112,80},{ -112,64},{ -130,64},{ -130, -6},{ -44, -6},{ -44,20},{ -20,20}}, color = {0,0,0}, smooth = Smooth.None));
      connect(inertia7.flange_b,gear2_1.flange_a) annotation(Line(points = {{ -90, -30},{ -80, -30}}, color = {0,0,0}, smooth = Smooth.None));
      connect(gear2_1.flange_b,inertia8.flange_a) annotation(Line(points = {{ -60, -30},{ -48, -30}}, color = {0,0,0}, smooth = Smooth.None));
      connect(fixed1.flange,gear2_1.support) annotation(Line(points = {{ -70, -50},{ -70, -40}}, color = {0,0,0}, smooth = Smooth.None));
      connect(fixed2.flange,position.support) annotation(Line(points = {{58, -20},{58, -10}}, color = {0,0,0}, smooth = Smooth.None));
      connect(position.flange,inertia9.flange_a) annotation(Line(points = {{68,0},{80,0}}, color = {0,0,0}, smooth = Smooth.None));
      connect(sine2.y,position.phi_ref) annotation(Line(points = {{39,0},{46,0}}, color = {0,0,127}, smooth = Smooth.None));
      connect(sine3.y,speed.w_ref) annotation(Line(points = {{41, -50},{48, -50}}, color = {0,0,127}, smooth = Smooth.None));
      connect(speed.flange,inertia10.flange_a) annotation(Line(points = {{70, -50},{80, -50}}, color = {0,0,0}, smooth = Smooth.None));
      connect(speed.support,fixed3.flange) annotation(Line(points = {{60, -60},{60, -70}}, color = {0,0,0}, smooth = Smooth.None));
      connect(sine4.y,accelerate.a_ref) annotation(Line(points = {{41, -110},{48, -110}}, color = {0,0,127}, smooth = Smooth.None));
      connect(accelerate.flange,inertia11.flange_a) annotation(Line(points = {{70, -110},{80, -110}}, color = {0,0,0}, smooth = Smooth.None));
      connect(accelerate.support,fixed4.flange) annotation(Line(points = {{60, -120},{60, -130}}, color = {0,0,0}, smooth = Smooth.None));
      connect(sine5.y,move.u) annotation(Line(points = {{41, -170},{48, -170}}, color = {0,0,127}, smooth = Smooth.None));
      connect(move.flange,inertia12.flange_a) annotation(Line(points = {{70, -170},{80, -170}}, color = {0,0,0}, smooth = Smooth.None));
      connect(move.support,fixed5.flange) annotation(Line(points = {{60, -180},{60, -190}}, color = {0,0,0}, smooth = Smooth.None));
      connect(brake.flange_b,torque.flange) annotation(Line(points = {{120,50},{140,50}}, color = {0,0,0}, smooth = Smooth.None));
      connect(fixed.flange,torque.support) annotation(Line(points = {{ -20,20},{150,20},{150,40}}, color = {0,0,0}, smooth = Smooth.None));
      connect(sine6.y,torque.tau) annotation(Line(points = {{171,50},{164.5,50},{164.5,50},{162,50}}, color = {0,0,127}, smooth = Smooth.None));
      connect(torque2_1.flange_a,inertia10.flange_b) annotation(Line(points = {{112, -50},{100, -50}}, color = {0,0,0}, smooth = Smooth.None));
      connect(torque2_1.flange_b,inertia13.flange_a) annotation(Line(points = {{132, -50},{140, -50}}, color = {0,0,0}, smooth = Smooth.None));
      connect(sine7.y,torque2_1.tau) annotation(Line(points = {{115, -24},{122, -24},{122, -46}}, color = {0,0,127}, smooth = Smooth.None));
      connect(linearSpeedDependentTorque.flange,inertia7.flange_a) annotation(Line(points = {{ -122, -30},{ -110, -30}}, color = {0,0,0}, smooth = Smooth.None));
      connect(linearSpeedDependentTorque.support,fixed1.flange) annotation(Line(points = {{ -132, -42},{ -132, -50},{ -70, -50}}, color = {0,0,0}, smooth = Smooth.None));
      connect(quadraticSpeedDependentTorque.flange,inertia8.flange_b) annotation(Line(points = {{ -16, -30},{ -28, -30}}, color = {0,0,0}, smooth = Smooth.None));
      connect(fixed1.flange,quadraticSpeedDependentTorque.support) annotation(Line(points = {{ -70, -50},{ -5, -50},{ -5, -42}}, color = {0,0,0}, smooth = Smooth.None));
      connect(constantTorque.flange,inertia14.flange_a) annotation(Line(points = {{ -100, -90},{ -92, -90},{ -92, -110},{ -84, -110}}, color = {0,0,0}, smooth = Smooth.None));
      connect(torqueStep.flange,inertia14.flange_a) annotation(Line(points = {{ -100, -130},{ -84, -130},{ -84, -110}}, color = {0,0,0}, smooth = Smooth.None));
      connect(fixed6.flange,torqueStep.support) annotation(Line(points = {{ -110, -152},{ -110, -142}}, color = {0,0,0}, smooth = Smooth.None));
      connect(fixed6.flange,constantTorque.support) annotation(Line(points = {{ -110, -152},{ -130, -152},{ -130, -102},{ -110, -102}}, color = {0,0,0}, smooth = Smooth.None));
      connect(torqueStep.support,constantSpeed.support) annotation(Line(points = {{ -110, -142},{ -50, -142}}, color = {0,0,0}, smooth = Smooth.None));
      connect(constantSpeed.flange,inertia15.flange_a) annotation(Line(points = {{ -40, -130},{ -30, -130}}, color = {0,0,0}, smooth = Smooth.None));
      connect(inertia8.flange_b,relativeStates.flange_b) annotation(Line(points = {{ -28, -30},{ -30, -30},{ -30, -70},{ -40, -70}}, color = {0,0,0}, smooth = Smooth.None));
      connect(relativeStates.flange_a,inertia7.flange_b) annotation(Line(points = {{ -60, -70},{ -90, -70},{ -90, -30}}, color = {0,0,0}, smooth = Smooth.None));
      connect(inertia11.flange_b,angleSensor.flange) annotation(Line(points = {{100, -110},{118, -110}}, color = {0,0,0}, smooth = Smooth.None));
      connect(inertia11.flange_b,speedSensor.flange) annotation(Line(points = {{100, -110},{104, -110},{104, -86},{118, -86}}, color = {0,0,0}, smooth = Smooth.None));
      connect(inertia11.flange_b,accSensor.flange) annotation(Line(points = {{100, -110},{104, -110},{104, -136},{118, -136}}, color = {0,0,0}, smooth = Smooth.None));
      connect(inertia9.flange_b,relAngleSensor.flange_a) annotation(Line(points = {{100,0},{132,0},{132, -10},{152, -10}}, color = {0,0,0}, smooth = Smooth.None));
      connect(inertia13.flange_b,relAngleSensor.flange_b) annotation(Line(points = {{160, -50},{186, -50},{186, -10},{172, -10}}, color = {0,0,0}, smooth = Smooth.None));
      connect(inertia13.flange_b,relSpeedSensor.flange_a) annotation(Line(points = {{160, -50},{160, -78},{166, -78}}, color = {0,0,0}, smooth = Smooth.None));
      connect(inertia12.flange_b,relSpeedSensor.flange_b) annotation(Line(points = {{100, -170},{198, -170},{198, -78},{186, -78}}, color = {0,0,0}, smooth = Smooth.None));
      connect(inertia13.flange_b,relAccSensor.flange_a) annotation(Line(points = {{160, -50},{160, -120}}, color = {0,0,0}, smooth = Smooth.None));
      connect(relAccSensor.flange_b,inertia12.flange_b) annotation(Line(points = {{180, -120},{188, -120},{188, -170},{100, -170}}, color = {0,0,0}, smooth = Smooth.None));
      connect(inertia15.flange_b,torqueSensor.flange_b) annotation(Line(points = {{ -10, -130},{8, -130},{8, -162},{ -6, -162},{ -6, -160}}, color = {0,0,0}, smooth = Smooth.None));
      connect(torqueSensor.flange_a,powerSensor.flange_b) annotation(Line(points = {{ -26, -160},{ -38, -160}}, color = {0,0,0}, smooth = Smooth.None));
      connect(inertia16.flange_b,powerSensor.flange_a) annotation(Line(points = {{ -68, -160},{ -58, -160}}, color = {0,0,0}, smooth = Smooth.None));
    end AllComponents;
    model TestBearingConversion "Test, whether conversion of bearing to support is correctly performed"
      Modelica.Mechanics.Rotational.Components.IdealGear idealGear(useSupport = false) annotation(Placement(transformation(extent = {{ -60,140},{ -40,160}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia annotation(Placement(transformation(extent = {{ -90,140},{ -70,160}}, rotation = 0)));
      annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -300},{300,200}}, initialScale = 0.1), graphics), experiment(StopTime = 0.5), experimentSetupOutput);
      Modelica.Mechanics.Rotational.Components.IdealGear idealGear1(useSupport = true) annotation(Placement(transformation(extent = {{ -28,140},{ -8,160}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.BearingFriction bearingFriction(useSupport = true) annotation(Placement(transformation(extent = {{0,140},{20,160}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia1 annotation(Placement(transformation(extent = {{ -90,90},{ -70,110}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.BearingFriction bearingFriction1(useSupport = false) annotation(Placement(transformation(extent = {{ -60,90},{ -40,110}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Fixed fixed annotation(Placement(transformation(extent = {{ -28,124},{ -8,144}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Brake brake(useSupport = true) annotation(Placement(transformation(extent = {{ -60,30},{ -40,50}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine annotation(Placement(transformation(extent = {{ -90,60},{ -70,80}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia2 annotation(Placement(transformation(extent = {{ -90,30},{ -70,50}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Brake brake1(useSupport = false) annotation(Placement(transformation(extent = {{16,30},{36,50}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia3 annotation(Placement(transformation(extent = {{ -20,30},{0,50}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Fixed fixed1 annotation(Placement(transformation(extent = {{ -60,10},{ -40,30}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.LossyGear lossyGear(useSupport = false) annotation(Placement(transformation(extent = {{4,90},{24,110}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia4 annotation(Placement(transformation(extent = {{ -24,90},{ -4,110}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia5 annotation(Placement(transformation(extent = {{34,90},{54,110}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.LossyGear lossyGear1(useSupport = true) annotation(Placement(transformation(extent = {{102,90},{122,110}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia6 annotation(Placement(transformation(extent = {{74,90},{94,110}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia7 annotation(Placement(transformation(extent = {{132,90},{152,110}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Fixed fixed2 annotation(Placement(transformation(extent = {{102,72},{122,92}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Gearbox gear2_1(useSupport = false) annotation(Placement(transformation(extent = {{76,30},{96,50}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia8 annotation(Placement(transformation(extent = {{50,30},{70,50}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia9 annotation(Placement(transformation(extent = {{104,30},{124,50}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.Position position(useSupport = false) annotation(Placement(transformation(extent = {{ -60, -28},{ -40, -8}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine1 annotation(Placement(transformation(extent = {{ -96, -28},{ -76, -8}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia12 annotation(Placement(transformation(extent = {{ -30, -28},{ -10, -8}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.Position position1(useSupport = true) annotation(Placement(transformation(extent = {{ -60, -58},{ -40, -38}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia13 annotation(Placement(transformation(extent = {{ -30, -58},{ -10, -38}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Fixed fixed4 annotation(Placement(transformation(extent = {{ -60, -76},{ -40, -56}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Gearbox gear2_2(useSupport = true) annotation(Placement(transformation(extent = {{66,142},{86,162}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia10 annotation(Placement(transformation(extent = {{40,142},{60,162}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia11 annotation(Placement(transformation(extent = {{94,142},{114,162}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Fixed fixed3 annotation(Placement(transformation(extent = {{66,124},{86,144}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.Speed position2(useSupport = false) annotation(Placement(transformation(extent = {{40, -30},{60, -10}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine2 annotation(Placement(transformation(extent = {{4, -30},{24, -10}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia14 annotation(Placement(transformation(extent = {{70, -30},{90, -10}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.Speed position3(useSupport = true) annotation(Placement(transformation(extent = {{40, -60},{60, -40}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia15 annotation(Placement(transformation(extent = {{70, -60},{90, -40}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Fixed fixed5 annotation(Placement(transformation(extent = {{40, -78},{60, -58}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.Accelerate position4(useSupport = false) annotation(Placement(transformation(extent = {{146, -30},{166, -10}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine3 annotation(Placement(transformation(extent = {{110, -30},{130, -10}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia16 annotation(Placement(transformation(extent = {{176, -30},{196, -10}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.Accelerate position5(useSupport = true) annotation(Placement(transformation(extent = {{146, -60},{166, -40}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia17 annotation(Placement(transformation(extent = {{176, -60},{196, -40}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Fixed fixed6 annotation(Placement(transformation(extent = {{146, -78},{166, -58}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.Move position6(useSupport = false) annotation(Placement(transformation(extent = {{ -60, -120},{ -40, -100}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine4[3] annotation(Placement(transformation(extent = {{ -96, -120},{ -76, -100}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia18 annotation(Placement(transformation(extent = {{ -30, -120},{ -10, -100}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.Move position7(useSupport = true) annotation(Placement(transformation(extent = {{ -60, -150},{ -40, -130}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia19 annotation(Placement(transformation(extent = {{ -30, -150},{ -10, -130}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Fixed fixed8 annotation(Placement(transformation(extent = {{ -60, -168},{ -40, -148}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine5 annotation(Placement(transformation(extent = {{24, -120},{44, -100}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.Torque torque(useSupport = false) annotation(Placement(transformation(extent = {{60, -120},{80, -100}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.LinearSpeedDependentTorque linearSpeedDependentTorque(tau_nominal = 1, w_nominal = 1, useSupport = false) annotation(Placement(transformation(extent = {{60, -154},{80, -134}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticSpeedDependentTorque(tau_nominal = 1, w_nominal = 1, useSupport = false) annotation(Placement(transformation(extent = {{58, -190},{78, -170}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.ConstantTorque constantTorque(tau_constant = 1, useSupport = false) annotation(Placement(transformation(extent = {{60, -224},{80, -204}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.TorqueStep torqueStep(useSupport = false) annotation(Placement(transformation(extent = {{60, -260},{80, -240}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia20 annotation(Placement(transformation(extent = {{98, -120},{118, -100}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine6 annotation(Placement(transformation(extent = {{140, -120},{160, -100}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.Torque torque1(useSupport = true) annotation(Placement(transformation(extent = {{176, -120},{196, -100}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.LinearSpeedDependentTorque linearSpeedDependentTorque1(tau_nominal = 1, w_nominal = 1, useSupport = true) annotation(Placement(transformation(extent = {{176, -154},{196, -134}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticSpeedDependentTorque1(tau_nominal = 1, w_nominal = 1, useSupport = true) annotation(Placement(transformation(extent = {{174, -190},{194, -170}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.ConstantTorque constantTorque1(tau_constant = 1, useSupport = true) annotation(Placement(transformation(extent = {{176, -224},{196, -204}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.TorqueStep torqueStep1(useSupport = true) annotation(Placement(transformation(extent = {{176, -260},{196, -240}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia21 annotation(Placement(transformation(extent = {{214, -120},{234, -100}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Fixed fixed7 annotation(Placement(transformation(extent = {{220, -280},{240, -260}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.IdealGearR2T idealGearR2T(useSupportR = false, useSupportT = false) annotation(Placement(transformation(extent = {{220,140},{240,160}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia22 annotation(Placement(transformation(extent = {{184,140},{204,160}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.Mass slidingMass annotation(Placement(transformation(extent = {{260,140},{280,160}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.IdealGearR2T idealGearR2T1(useSupportR = true, useSupportT = true) annotation(Placement(transformation(extent = {{216,100},{236,120}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia23 annotation(Placement(transformation(extent = {{180,100},{200,120}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.Mass slidingMass1 annotation(Placement(transformation(extent = {{256,100},{276,120}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Fixed fixed9 annotation(Placement(transformation(extent = {{210,82},{230,102}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.Fixed fixed10 annotation(Placement(transformation(extent = {{232,82},{252,102}}, rotation = 0)));

    equation 
      connect(inertia.flange_b,idealGear.flange_a) annotation(Line(points = {{ -70,150},{ -60,150}}, color = {0,0,0}, smooth = Smooth.None));
      connect(idealGear.flange_b,idealGear1.flange_a) annotation(Line(points = {{ -40,150},{ -28,150}}, color = {0,0,0}, smooth = Smooth.None));
      connect(idealGear1.flange_b,bearingFriction.flange_a) annotation(Line(points = {{ -8,150},{0,150}}, color = {0,0,0}, smooth = Smooth.None));
      connect(inertia1.flange_b,bearingFriction1.flange_a) annotation(Line(points = {{ -70,100},{ -60,100}}, color = {0,0,0}, smooth = Smooth.None));
      connect(idealGear1.support,fixed.flange) annotation(Line(points = {{ -18,140},{ -18,134}}, color = {0,0,0}, smooth = Smooth.None));
      connect(fixed.flange,bearingFriction.support) annotation(Line(points = {{ -18,134},{10,134},{10,140}}, color = {0,0,0}, smooth = Smooth.None));
      connect(sine.y,brake.f_normalized) annotation(Line(points = {{ -69,70},{ -50,70},{ -50,51}}, color = {0,0,127}, smooth = Smooth.None));
      connect(inertia2.flange_b,brake.flange_a) annotation(Line(points = {{ -70,40},{ -60,40}}, color = {0,0,0}, smooth = Smooth.None));
      connect(inertia3.flange_b,brake1.flange_a) annotation(Line(points = {{0,40},{16,40}}, color = {0,0,0}, smooth = Smooth.None));
      connect(sine.y,brake1.f_normalized) annotation(Line(points = {{ -69,70},{26,70},{26,51}}, color = {0,0,127}, smooth = Smooth.None));
      connect(brake.support,fixed1.flange) annotation(Line(points = {{ -50,30},{ -50,20}}, color = {0,0,0}, smooth = Smooth.None));
      connect(inertia4.flange_b,lossyGear.flange_a) annotation(Line(points = {{ -4,100},{4,100}}, color = {0,0,0}, smooth = Smooth.None));
      connect(lossyGear.flange_b,inertia5.flange_a) annotation(Line(points = {{24,100},{34,100}}, color = {0,0,0}, smooth = Smooth.None));
      connect(inertia6.flange_b,lossyGear1.flange_a) annotation(Line(points = {{94,100},{102,100}}, color = {0,0,0}, smooth = Smooth.None));
      connect(lossyGear1.flange_b,inertia7.flange_a) annotation(Line(points = {{122,100},{132,100}}, color = {0,0,0}, smooth = Smooth.None));
      connect(fixed2.flange,lossyGear1.support) annotation(Line(points = {{112,82},{112,90}}, color = {0,0,0}, smooth = Smooth.None));
      connect(inertia8.flange_b,gear2_1.flange_a) annotation(Line(points = {{70,40},{76,40}}, color = {0,0,0}, smooth = Smooth.None));
      connect(gear2_1.flange_b,inertia9.flange_a) annotation(Line(points = {{96,40},{104,40}}, color = {0,0,0}, smooth = Smooth.None));
      connect(sine1.y,position.phi_ref) annotation(Line(points = {{ -75, -18},{ -62, -18}}, color = {0,0,127}, smooth = Smooth.None));
      connect(position.flange,inertia12.flange_a) annotation(Line(points = {{ -40, -18},{ -30, -18}}, color = {0,0,0}, smooth = Smooth.None));
      connect(position1.flange,inertia13.flange_a) annotation(Line(points = {{ -40, -48},{ -30, -48}}, color = {0,0,0}, smooth = Smooth.None));
      connect(sine1.y,position1.phi_ref) annotation(Line(points = {{ -75, -18},{ -70, -18},{ -70, -48},{ -62, -48}}, color = {0,0,127}, smooth = Smooth.None));
      connect(position1.support,fixed4.flange) annotation(Line(points = {{ -50, -58},{ -50, -66}}, color = {0,0,0}, smooth = Smooth.None));
      connect(inertia10.flange_b,gear2_2.flange_a) annotation(Line(points = {{60,152},{66,152}}, color = {0,0,0}, smooth = Smooth.None));
      connect(gear2_2.flange_b,inertia11.flange_a) annotation(Line(points = {{86,152},{94,152}}, color = {0,0,0}, smooth = Smooth.None));
      connect(fixed3.flange,gear2_2.support) annotation(Line(points = {{76,134},{76,142}}, color = {0,0,0}, smooth = Smooth.None));
      connect(position2.flange,inertia14.flange_a) annotation(Line(points = {{60, -20},{70, -20}}, color = {0,0,0}, smooth = Smooth.None));
      connect(position3.flange,inertia15.flange_a) annotation(Line(points = {{60, -50},{70, -50}}, color = {0,0,0}, smooth = Smooth.None));
      connect(position3.support,fixed5.flange) annotation(Line(points = {{50, -60},{50, -68}}, color = {0,0,0}, smooth = Smooth.None));
      connect(sine2.y,position2.w_ref) annotation(Line(points = {{25, -20},{38, -20}}, color = {0,0,127}, smooth = Smooth.None));
      connect(sine2.y,position3.w_ref) annotation(Line(points = {{25, -20},{30, -20},{30, -50},{38, -50}}, color = {0,0,127}, smooth = Smooth.None));
      connect(position4.flange,inertia16.flange_a) annotation(Line(points = {{166, -20},{176, -20}}, color = {0,0,0}, smooth = Smooth.None));
      connect(position5.flange,inertia17.flange_a) annotation(Line(points = {{166, -50},{176, -50}}, color = {0,0,0}, smooth = Smooth.None));
      connect(position5.support,fixed6.flange) annotation(Line(points = {{156, -60},{156, -68}}, color = {0,0,0}, smooth = Smooth.None));
      connect(sine3.y,position4.a_ref) annotation(Line(points = {{131, -20},{144, -20}}, color = {0,0,127}, smooth = Smooth.None));
      connect(sine3.y,position5.a_ref) annotation(Line(points = {{131, -20},{136, -20},{136, -50},{144, -50}}, color = {0,0,127}, smooth = Smooth.None));
      connect(position6.flange,inertia18.flange_a) annotation(Line(points = {{ -40, -110},{ -30, -110}}, color = {0,0,0}, smooth = Smooth.None));
      connect(position7.flange,inertia19.flange_a) annotation(Line(points = {{ -40, -140},{ -30, -140}}, color = {0,0,0}, smooth = Smooth.None));
      connect(position7.support,fixed8.flange) annotation(Line(points = {{ -50, -150},{ -50, -158}}, color = {0,0,0}, smooth = Smooth.None));
      connect(sine4.y,position6.u) annotation(Line(points = {{ -75, -110},{ -62, -110}}, color = {0,0,127}, smooth = Smooth.None));
      connect(sine4.y,position7.u) annotation(Line(points = {{ -75, -110},{ -72, -110},{ -72, -140},{ -62, -140}}, color = {0,0,127}, smooth = Smooth.None));
      connect(sine5.y,torque.tau) annotation(Line(points = {{45, -110},{50.5, -110},{50.5, -110},{58, -110}}, color = {0,0,127}, smooth = Smooth.None));
      connect(torque.flange,inertia20.flange_a) annotation(Line(points = {{80, -110},{98, -110}}, color = {0,0,0}, smooth = Smooth.None));
      connect(linearSpeedDependentTorque.flange,inertia20.flange_a) annotation(Line(points = {{80, -144},{92, -144},{92, -110},{98, -110}}, color = {0,0,0}, smooth = Smooth.None));
      connect(quadraticSpeedDependentTorque.flange,inertia20.flange_a) annotation(Line(points = {{78, -180},{92, -180},{92, -110},{98, -110}}, color = {0,0,0}, smooth = Smooth.None));
      connect(constantTorque.flange,inertia20.flange_a) annotation(Line(points = {{80, -214},{92, -214},{92, -110},{98, -110}}, color = {0,0,0}, smooth = Smooth.None));
      connect(torqueStep.flange,inertia20.flange_a) annotation(Line(points = {{80, -250},{92, -250},{92, -110},{98, -110}}, color = {0,0,0}, smooth = Smooth.None));
      connect(sine6.y,torque1.tau) annotation(Line(points = {{161, -110},{174, -110}}, color = {0,0,127}, smooth = Smooth.None));
      connect(torque1.flange,inertia21.flange_a) annotation(Line(points = {{196, -110},{214, -110}}, color = {0,0,0}, smooth = Smooth.None));
      connect(linearSpeedDependentTorque1.flange,inertia21.flange_a) annotation(Line(points = {{196, -144},{208, -144},{208, -110},{214, -110}}, color = {0,0,0}, smooth = Smooth.None));
      connect(quadraticSpeedDependentTorque1.flange,inertia21.flange_a) annotation(Line(points = {{194, -180},{208, -180},{208, -110},{214, -110}}, color = {0,0,0}, smooth = Smooth.None));
      connect(constantTorque1.flange,inertia21.flange_a) annotation(Line(points = {{196, -214},{208, -214},{208, -110},{214, -110}}, color = {0,0,0}, smooth = Smooth.None));
      connect(torqueStep1.flange,inertia21.flange_a) annotation(Line(points = {{196, -250},{208, -250},{208, -110},{214, -110}}, color = {0,0,0}, smooth = Smooth.None));
      connect(torqueStep1.support,fixed7.flange) annotation(Line(points = {{186, -260},{186, -270},{230, -270}}, color = {0,0,0}, smooth = Smooth.None));
      connect(constantTorque1.support,fixed7.flange) annotation(Line(points = {{186, -224},{208, -224},{208, -228},{230, -228},{230, -270}}, color = {0,0,0}, smooth = Smooth.None));
      connect(quadraticSpeedDependentTorque1.support,fixed7.flange) annotation(Line(points = {{184, -190},{230, -190},{230, -270}}, color = {0,0,0}, smooth = Smooth.None));
      connect(linearSpeedDependentTorque1.support,fixed7.flange) annotation(Line(points = {{186, -154},{230, -154},{230, -270}}, color = {0,0,0}, smooth = Smooth.None));
      connect(torque1.support,fixed7.flange) annotation(Line(points = {{186, -120},{186, -126},{230, -126},{230, -270}}, color = {0,0,0}, smooth = Smooth.None));
      connect(inertia22.flange_b,idealGearR2T.flangeR) annotation(Line(points = {{204,150},{220,150}}, color = {0,0,0}, smooth = Smooth.None));
      connect(idealGearR2T.flangeT,slidingMass.flange_a) annotation(Line(points = {{240,150},{260,150}}, color = {0,127,0}, smooth = Smooth.None));
      connect(inertia23.flange_b,idealGearR2T1.flangeR) annotation(Line(points = {{200,110},{216,110}}, color = {0,0,0}, smooth = Smooth.None));
      connect(idealGearR2T1.flangeT,slidingMass1.flange_a) annotation(Line(points = {{236,110},{256,110}}, color = {0,127,0}, smooth = Smooth.None));
      connect(idealGearR2T1.supportR,fixed9.flange) annotation(Line(points = {{216,100},{216,96},{220,96},{220,92}}, color = {0,0,0}, smooth = Smooth.None));
      connect(fixed10.flange,idealGearR2T1.supportT) annotation(Line(points = {{242,92},{242,100},{236,100}}, color = {0,127,0}, smooth = Smooth.None));
    end TestBearingConversion;
    model TestFriction "Test friction elements"
      Modelica.Mechanics.Rotational.Components.Inertia inertia annotation(Placement(transformation(extent = {{ -8,80},{12,100}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.BearingFriction bearingFriction(useSupport = false) annotation(Placement(transformation(extent = {{22,80},{42,100}}, rotation = 0)));
      annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1.1));
      Modelica.Mechanics.Rotational.Sources.Torque torque(useSupport = false) annotation(Placement(transformation(extent = {{ -40,80},{ -20,100}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine(amplitude = 10) annotation(Placement(transformation(extent = {{ -80,80},{ -60,100}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia1 annotation(Placement(transformation(extent = {{ -8,20},{12,40}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.Torque torque1(useSupport = false) annotation(Placement(transformation(extent = {{ -40,20},{ -20,40}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine1(amplitude = 10) annotation(Placement(transformation(extent = {{ -80,20},{ -60,40}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Brake brake(useSupport = false) annotation(Placement(transformation(extent = {{20,20},{40,40}}, rotation = 0)));
      Modelica.Blocks.Sources.Step step(height = 100, startTime = 0.5) annotation(Placement(transformation(extent = {{ -18,50},{2,70}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia2 annotation(Placement(transformation(extent = {{ -8, -40},{12, -20}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.Torque torque2(useSupport = false) annotation(Placement(transformation(extent = {{ -40, -40},{ -20, -20}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine2(amplitude = 10) annotation(Placement(transformation(extent = {{ -80, -40},{ -60, -20}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Clutch clutch annotation(Placement(transformation(extent = {{24, -40},{44, -20}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia3 annotation(Placement(transformation(extent = {{58, -40},{78, -20}}, rotation = 0)));
      Modelica.Blocks.Sources.Step step1(height = 100, startTime = 0.5) annotation(Placement(transformation(extent = {{ -12, -10},{8,10}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia4 annotation(Placement(transformation(extent = {{ -6, -100},{14, -80}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.Torque torque3(useSupport = false) annotation(Placement(transformation(extent = {{ -38, -100},{ -18, -80}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine3(amplitude = 10) annotation(Placement(transformation(extent = {{ -78, -100},{ -58, -80}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia5 annotation(Placement(transformation(extent = {{60, -100},{80, -80}}, rotation = 0)));
      Modelica.Blocks.Sources.Step step2(height = 100, startTime = 0.5) annotation(Placement(transformation(extent = {{ -10, -70},{10, -50}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.OneWayClutch oneWayClutch annotation(Placement(transformation(extent = {{46, -100},{26, -80}}, rotation = 0)));

    equation 
      connect(inertia.flange_b,bearingFriction.flange_a) annotation(Line(points = {{12,90},{22,90}}, color = {0,0,0}, smooth = Smooth.None));
      connect(torque.flange,inertia.flange_a) annotation(Line(points = {{ -20,90},{ -8,90}}, color = {0,0,0}, smooth = Smooth.None));
      connect(sine.y,torque.tau) annotation(Line(points = {{ -59,90},{ -42,90}}, color = {0,0,127}, smooth = Smooth.None));
      connect(torque1.flange,inertia1.flange_a) annotation(Line(points = {{ -20,30},{ -8,30}}, color = {0,0,0}, smooth = Smooth.None));
      connect(sine1.y,torque1.tau) annotation(Line(points = {{ -59,30},{ -42,30}}, color = {0,0,127}, smooth = Smooth.None));
      connect(inertia1.flange_b,brake.flange_a) annotation(Line(points = {{12,30},{20,30}}, color = {0,0,0}, smooth = Smooth.None));
      connect(step.y,brake.f_normalized) annotation(Line(points = {{3,60},{30,60},{30,41}}, color = {0,0,127}, smooth = Smooth.None));
      connect(torque2.flange,inertia2.flange_a) annotation(Line(points = {{ -20, -30},{ -8, -30}}, color = {0,0,0}, smooth = Smooth.None));
      connect(sine2.y,torque2.tau) annotation(Line(points = {{ -59, -30},{ -42, -30}}, color = {0,0,127}, smooth = Smooth.None));
      connect(inertia2.flange_b,clutch.flange_a) annotation(Line(points = {{12, -30},{24, -30}}, color = {0,0,0}, smooth = Smooth.None));
      connect(clutch.flange_b,inertia3.flange_a) annotation(Line(points = {{44, -30},{58, -30}}, color = {0,0,0}, smooth = Smooth.None));
      connect(step1.y,clutch.f_normalized) annotation(Line(points = {{9,0},{34,0},{34, -19}}, color = {0,0,127}, smooth = Smooth.None));
      connect(torque3.flange,inertia4.flange_a) annotation(Line(points = {{ -18, -90},{ -6, -90}}, color = {0,0,0}, smooth = Smooth.None));
      connect(sine3.y,torque3.tau) annotation(Line(points = {{ -57, -90},{ -40, -90}}, color = {0,0,127}, smooth = Smooth.None));
      connect(step2.y,oneWayClutch.f_normalized) annotation(Line(points = {{11, -60},{36, -60},{36, -79}}, color = {0,0,127}, smooth = Smooth.None));
      connect(oneWayClutch.flange_b,inertia4.flange_b) annotation(Line(points = {{26, -90},{14, -90}}, color = {0,0,0}, smooth = Smooth.None));
      connect(oneWayClutch.flange_a,inertia5.flange_a) annotation(Line(points = {{46, -90},{60, -90}}, color = {0,0,0}, smooth = Smooth.None));
    end TestFriction;
  end Rotational;
  package MultiBody "Test models for Modelica.Mechanics.MultiBody"
    encapsulated model SphericalDoublePendulum "Double pendulum with two spherical joints and two bodies"
      import ObsoleteModelica3;
      import Modelica;
      import Modelica.Mechanics.MultiBody;
      extends Modelica.Icons.Example;
      annotation(experiment(StopTime = 3), Window(x = 0.29, y = 0, width = 0.71, height = 0.85), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
This example demonstrates that by using joint and body
elements animation is automatically available. Also the revolute
joints are animated. Note, that animation of every component
can be switched of by setting the first parameter <b>animation</b>
to <b>false</b> or by setting <b>enableAnimation</b> in the <b>world</b>
object to <b>false</b> to switch off animation of all components.
</p>

<p align=\"center\">
<IMG SRC=\"../Images/Examples/Elementary/DoublePendulum.png\"
ALT=\"model Examples.Elementary.DoublePendulum\">
</p>

</HTML>"), uses);
      inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -88,0},{ -68,20}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Parts.BodyBox boxBody1(width = 0.06, r = {0.4,0.0, -0.3}) annotation(Placement(transformation(extent = {{ -10,0},{10,20}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Parts.BodyBox boxBody2(width = 0.06, r = {0.3, -0.4,0}) annotation(Placement(transformation(extent = {{74,0},{94,20}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Joints.Spherical Spherical1(enforceStates = true, useQuaternions = false, angles_fixed = true, w_rel_a_fixed = true, z_rel_a_fixed = false, w_rel_a_start = {0.0523598775598299,0.0349065850398866,0.0174532925199433}) annotation(Placement(transformation(extent = {{ -52,0},{ -32,20}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Joints.Spherical Spherical2(enforceStates = true, useQuaternions = false, angles_fixed = true, w_rel_a_fixed = true, z_rel_a_fixed = false) annotation(Placement(transformation(extent = {{32,0},{52,20}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Parts.BodyBox boxBody3(width = 0.06, r = {0.4,0.0, -0.3}) annotation(Placement(transformation(extent = {{ -10, -40},{10, -20}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Parts.BodyBox boxBody4(width = 0.06, r = {0.3, -0.4,0}) annotation(Placement(transformation(extent = {{76, -40},{96, -20}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Joints.Spherical Spherical3(enforceStates = true, angles_fixed = true, w_rel_a_fixed = true, z_rel_a_fixed = false, w_rel_a_start = {0.0523598775598299,0.0349065850398866,0.0174532925199433}) annotation(Placement(transformation(extent = {{ -52, -40},{ -32, -20}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Joints.Spherical Spherical4(enforceStates = true, angles_fixed = true, w_rel_a_fixed = true, z_rel_a_fixed = false) annotation(Placement(transformation(extent = {{32, -40},{52, -20}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Parts.BodyBox boxBody5(width = 0.06, r = {0.4,0.0, -0.3}, useQuaternions = true) annotation(Placement(transformation(extent = {{ -10, -80},{10, -60}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Parts.BodyBox boxBody6(width = 0.06, r = {0.3, -0.4,0}, useQuaternions = true) annotation(Placement(transformation(extent = {{74, -80},{94, -60}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Joints.Spherical Spherical5(angles_fixed = true, w_rel_a_fixed = true, z_rel_a_fixed = false, w_rel_a_start = {0.0523598775598299,0.0349065850398866,0.0174532925199433}) annotation(Placement(transformation(extent = {{ -52, -80},{ -32, -60}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Joints.Spherical Spherical6(angles_fixed = true, w_rel_a_fixed = true, z_rel_a_fixed = false) annotation(Placement(transformation(extent = {{32, -80},{52, -60}}, rotation = 0)));

    equation 
      connect(boxBody1.frame_b,Spherical2.frame_a) annotation(Line(points = {{10,10},{32,10}}, color = {0,0,0}, thickness = 0.5));
      connect(Spherical2.frame_b,boxBody2.frame_a) annotation(Line(points = {{52,10},{74,10}}, color = {0,0,0}, thickness = 0.5));
      connect(world.frame_b,Spherical1.frame_a) annotation(Line(points = {{ -68,10},{ -52,10}}, color = {0,0,0}, thickness = 0.5));
      connect(Spherical1.frame_b,boxBody1.frame_a) annotation(Line(points = {{ -32,10},{ -10,10}}, color = {0,0,0}, thickness = 0.5));
      connect(boxBody3.frame_b,Spherical4.frame_a) annotation(Line(points = {{10, -30},{32, -30}}, color = {0,0,0}, thickness = 0.5));
      connect(Spherical4.frame_b,boxBody4.frame_a) annotation(Line(points = {{52, -30},{76, -30}}, color = {0,0,0}, thickness = 0.5));
      connect(Spherical3.frame_b,boxBody3.frame_a) annotation(Line(points = {{ -32, -30},{ -10, -30}}, color = {0,0,0}, thickness = 0.5));
      connect(world.frame_b,Spherical3.frame_a) annotation(Line(points = {{ -68,10},{ -60,10},{ -60, -30},{ -52, -30}}, color = {0,0,0}, thickness = 0.5));
      connect(boxBody5.frame_b,Spherical6.frame_a) annotation(Line(points = {{10, -70},{32, -70}}, color = {0,0,0}, thickness = 0.5));
      connect(Spherical6.frame_b,boxBody6.frame_a) annotation(Line(points = {{52, -70},{74, -70}}, color = {0,0,0}, thickness = 0.5));
      connect(Spherical5.frame_b,boxBody5.frame_a) annotation(Line(points = {{ -32, -70},{ -10, -70}}, color = {0,0,0}, thickness = 0.5));
      connect(Spherical5.frame_a,world.frame_b) annotation(Line(points = {{ -52, -70},{ -60, -70},{ -60,10},{ -68,10}}, color = {0,0,0}, thickness = 0.5));
    end SphericalDoublePendulum;
    package FourbarVariants "Test joints and assembly joints with four bar loop"
      model SphericalAndUniversal "One kinematic loop with four bars (with Spherical and Universal joint)"
        import SI = Modelica.SIunits;
        extends Modelica.Icons.Example;
        output SI.Angle j1_phi "angle of revolute joint j1";
        output SI.Position j2_s "distance of prismatic joint j2";
        output SI.AngularVelocity j1_w "axis speed of revolute joint j1";
        output SI.Velocity j2_v "axis velocity of prismatic joint j2";
        annotation(experiment(StopTime = 5), Documentation(info = "<html>
<p>
This is a second version of the \"four-bar\" mechanism, see figure:
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Loops/Fourbar2.png\" ALT=\"model Examples.Loops.Fourbar2\">
</p>
<p>
In this case
the three revolute joints on the left top-side and the two revolute
joints on the right top side have been replaced by the joint <b>UniversalSpherical</b>
that is a rod connecting a spherical and a universal joint. This joint is defined
by <b>1 constraint</b> stating that the distance between the two spherical joints is
constant. Using this joint in a kinematic loop reduces the sizes of
non-linear algebraic equations. For this loop, only one non-linear
algebraic system of equations of order 1 remains.
</p>
<p>
At the UniversalSpherical joint an additional frame_ia fixed to the rod
is present where components can be attached to the connecting rod. In this
example just a coordinate system is attached to visualize frame_ia (coordinate
system on the right in blue color).
</p>
<p>
Another feature is that the length of the connecting rod can be
automatically calculated during <b>initialization</b>. In order to do this,
another initialization condition has to be given. In this example, the
initial value of the distance of the prismatic joint j2 has been fixed
(via the \"Initialization\" menu) and the rod length of joint
\"UniversalSpherical\" is computed during initialization since parameter
<b>computeLength</b> = <b>true</b> is set in the joint parameter
menu. The main advantage is that during initialization no non-linear
system of equation is solved and therefore initialization always works.
To be precise, the following trivial non-linear equation is actually solved
for rodLength:
</p>
<pre>
   rodLength*rodLength = f(angle of revolute joint, distance of prismatic joint)
</pre>
</html>"), Diagram(graphics));
        inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -80, -80},{ -60, -60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Revolute j1(n = {1,0,0}, stateSelect = StateSelect.always, a(fixed = false), phi(fixed = true), w(fixed = true, start = 5.23598775598299)) annotation(Placement(transformation(extent = {{ -54, -40},{ -34, -20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Prismatic j2(n = {1,0,0}, boxWidth = 0.05, s_offset =  -0.2) annotation(Placement(transformation(extent = {{12, -80},{32, -60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyCylinder b1(r = {0,0.5,0.1}, diameter = 0.05) annotation(Placement(transformation(origin = { -30,0}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Mechanics.MultiBody.Parts.BodyCylinder b2(r = {0,0.2,0}, diameter = 0.05) annotation(Placement(transformation(origin = {50, -50}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation b3(r = {1.2,0,0}, animation = false) annotation(Placement(transformation(extent = {{ -32, -80},{ -12, -60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Spherical spherical(animation = false) annotation(Placement(transformation(extent = {{ -20,20},{0,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation FixedTranslation1(r = {1, -0.3, -0.1}, animation = false) annotation(Placement(transformation(extent = {{12,20},{32,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Universal Universal1(n_b = {0,1,0}, n_a = {0,0,1}) annotation(Placement(transformation(extent = {{44,20},{64,40}}, rotation = 0)));

      equation 
        j1_phi = j1.phi;
        j2_s = j2.s;
        j1_w = j1.w;
        j2_v = j2.v;
        connect(j2.frame_b,b2.frame_a) annotation(Line(points = {{32, -70},{50, -70},{50, -60}}, color = {0,0,0}, thickness = 0.5));
        connect(j1.frame_b,b1.frame_a) annotation(Line(points = {{ -34, -30},{ -30, -30},{ -30, -10}}, color = {0,0,0}, thickness = 0.5));
        connect(j1.frame_a,world.frame_b) annotation(Line(points = {{ -54, -30},{ -60, -30},{ -60, -70}}, color = {0,0,0}, thickness = 0.5));
        connect(b3.frame_a,world.frame_b) annotation(Line(points = {{ -32, -70},{ -60, -70}}, color = {0,0,0}, thickness = 0.5));
        connect(b3.frame_b,j2.frame_a) annotation(Line(points = {{ -12, -70},{12, -70}}, color = {0,0,0}, thickness = 0.5));
        connect(b1.frame_b,spherical.frame_a) annotation(Line(points = {{ -30,10},{ -30,30},{ -20,30}}, color = {0,0,0}, thickness = 0.5));
        connect(spherical.frame_b,FixedTranslation1.frame_a) annotation(Line(points = {{0,30},{12,30}}, color = {0,0,0}, thickness = 0.5));
        connect(FixedTranslation1.frame_b,Universal1.frame_a) annotation(Line(points = {{32,30},{44,30}}, color = {0,0,0}, thickness = 0.5));
        connect(b2.frame_b,Universal1.frame_b) annotation(Line(points = {{50, -40},{50, -2},{80, -2},{80,30},{64,30}}, color = {0,0,0}, thickness = 0.5));
      end SphericalAndUniversal;
      encapsulated model SphericalSpherical "One kinematic loop with four bars (with SphericalSpherical joint)"
        import ObsoleteModelica3;
        import Modelica;
        import Modelica.Mechanics.MultiBody;
        import SI = Modelica.SIunits;
        extends Modelica.Icons.Example;
        output SI.Angle j1_phi "angle of revolute joint j1";
        output SI.Position j2_s "distance of prismatic joint j2";
        output SI.AngularVelocity j1_w "axis speed of revolute joint j1";
        output SI.Velocity j2_v "axis velocity of prismatic joint j2";
        annotation(experiment(StopTime = 5), Documentation(info = "<html>
<p>
This is a third version of the \"four-bar\" mechanism. In this case
the three revolute joints on the left top-side and the two revolute
joints on the right top side have been replaced by the joint <b>SphericalSpherical</b>
that is a rod with two spherical joints on each side. This joint is defined
by <b>1 constraint</b> stating that the distance between the two spherical joints is
constant. Using this joint in a kinematic loop reduces the sizes of
non-linear algebraic equations. For this loop, only one non-linear
algebraic system of equations of order 1 remains.
</p>
<p>
The SphericalSpherical joint may be massless or may have a point mass in
the middle of the rod to approximate in an convenient way the rod
mass properties.
</p>
<p>
Another nice feature is that the <b>length</b> of the connecting rod can be
automatically calculated during <b>initialization</b>. In order to do this,
another initialization condition has to be given. In this example, the
initial value of the distance of the prismatic joint j2 has been fixed
(via the \"Initialization\" menu) and the length parameter of joint
\"SphericalSpherical\" is computed during initialization since parameter
<b>computeLength</b> = <b>true</b> is set in the joint parameter
menu (this sets \"fixed=false\" on parameter \"length\").
</p>
</html>
"), Diagram(graphics));
        inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -80, -80},{ -60, -60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Revolute j1(n = {1,0,0}, stateSelect = StateSelect.always, a(fixed = false), phi(fixed = true), w(fixed = true, start = 5.23598775598299)) annotation(Placement(transformation(extent = {{ -54, -40},{ -34, -20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Prismatic j2(n = {1,0,0}, a(fixed = false), s(fixed = true), v(fixed = false), s_offset =  -0.2) annotation(Placement(transformation(extent = {{12, -80},{32, -60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyCylinder b1(r = {0,0.5,0.1}, diameter = 0.05) annotation(Placement(transformation(origin = { -30,0}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Mechanics.MultiBody.Parts.BodyCylinder b2(r = {0,0.2,0}, diameter = 0.05) annotation(Placement(transformation(origin = {50, -50}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Mechanics.MultiBody.Joints.SphericalSpherical sphericalSpherical(computeRodLength = true, m = 1) annotation(Placement(transformation(extent = {{0,20},{ -20,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation b3(r = {1.2,0,0}, animation = false) annotation(Placement(transformation(extent = {{ -32, -80},{ -12, -60}}, rotation = 0)));

      equation 
        connect(j2.frame_b,b2.frame_a) annotation(Line(points = {{32, -70},{50, -70},{50, -60}}));
        connect(j1.frame_b,b1.frame_a) annotation(Line(points = {{ -34, -30},{ -30, -30},{ -30, -10}}));
        connect(j1.frame_a,world.frame_b) annotation(Line(points = {{ -54, -30},{ -60, -30},{ -60, -70}}));
        connect(b1.frame_b,sphericalSpherical.frame_b) annotation(Line(points = {{ -30,10},{ -30,30},{ -20,30}}));
        connect(sphericalSpherical.frame_a,b2.frame_b) annotation(Line(points = {{0,30},{50,30},{50, -40}}));
        j1_phi = j1.phi;
        j2_s = j2.s;
        j1_w = j1.w;
        j2_v = j2.v;
        connect(b3.frame_a,world.frame_b) annotation(Line(points = {{ -32, -70},{ -60, -70}}));
        connect(b3.frame_b,j2.frame_a) annotation(Line(points = {{ -12, -70},{12, -70}}));
      end SphericalSpherical;
      encapsulated model UniversalSpherical "One kinematic loop with four bars (with UniversalSpherical joint)"
        import ObsoleteModelica3;
        import Modelica;
        import Modelica.Mechanics.MultiBody;
        import SI = Modelica.SIunits;
        extends Modelica.Icons.Example;
        output SI.Angle j1_phi "angle of revolute joint j1";
        output SI.Position j2_s "distance of prismatic joint j2";
        output SI.AngularVelocity j1_w "axis speed of revolute joint j1";
        output SI.Velocity j2_v "axis velocity of prismatic joint j2";
        annotation(experiment(StopTime = 5), Documentation(info = "<html>
<p>
This is a fourth version of the \"four-bar\" mechanism. In this case
the three revolute joints on the left top-side and the two revolute
joints on the right top side have been replaced by the joint <b>UniversalSpherical</b>
that is a rod with a spherical and a universal joint on two sides. This joint is defined
by <b>1 constraint</b> stating that the distance between the two spherical joints is
constant. Using this joint in a kinematic loop reduces the sizes of
non-linear algebraic equations. For this loop, only one non-linear
algebraic system of equations of order 1 remains.
</p>
<p>
The essential difference to joint SphericalSpherical is that the
orientation of the rod can be computed by removing one degree of freedom
of one of the spherical joints (i.e., replacing it by a universal joint).
Usually, the eigenrotation of the connecting rod is of no technical
interest and by this approximation it is constrained to move in a
somewhat arbitrary way. This allows to have an additional connector,
<b>frame_ia</b>, to be fixed on the rod, where other objects can be attached.
In this example, the coordinate system of frame_ia is visualized.
</p>
<p>
Another nice feature is that the <b>length</b> of the connecting rod can be
automatically calculated during <b>initialization</b>. In order to do this,
another initialization condition has to be given. In this example, the
initial value of the distance of the prismatic joint j2 has been fixed
(via the \"Initialization\" menu) and the length parameter of joint
\"UniversalSpherical\" is computed during initialization since parameter
<b>computeLength</b> = <b>true</b> is set in the joint parameter
menu (this sets \"fixed=false\" on parameter \"length\").
</p>
</html>"), Diagram(graphics));
        inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -80, -80},{ -60, -60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Revolute j1(n = {1,0,0}, stateSelect = StateSelect.always, a(fixed = false), phi(fixed = true), w(fixed = true, start = 5.23598775598299)) annotation(Placement(transformation(extent = {{ -54, -40},{ -34, -20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Prismatic j2(n = {1,0,0}, boxWidth = 0.01, a(fixed = false), s(fixed = false), v(fixed = false), s_offset =  -0.2) annotation(Placement(transformation(extent = {{12, -80},{32, -60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyCylinder b1(r = {0,0.5,0.1}, diameter = 0.05) annotation(Placement(transformation(origin = { -30,0}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Mechanics.MultiBody.Parts.BodyCylinder b2(r = {0,0.2,0}, diameter = 0.05) annotation(Placement(transformation(origin = {50, -50}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Mechanics.MultiBody.Joints.UniversalSpherical universalSpherical(rRod_ia = { -1,0.3,0.1}, n1_a = {0,1,0.1}) annotation(Placement(transformation(extent = {{0,20},{ -20,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation b3(animation = false, r = {1,0,0}) annotation(Placement(transformation(extent = {{ -32, -80},{ -12, -60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedFrame fixedFrame4 annotation(Placement(transformation(origin = { -10,70}, extent = {{ -10, -10},{10,10}}, rotation = 90)));

      equation 
        j1_phi = j1.phi;
        j2_s = j2.s;
        j1_w = j1.w;
        j2_v = j2.v;
        connect(j2.frame_b,b2.frame_a) annotation(Line(points = {{32, -70},{50, -70},{50, -60}}));
        connect(j1.frame_b,b1.frame_a) annotation(Line(points = {{ -34, -30},{ -30, -30},{ -30, -10}}));
        connect(j1.frame_a,world.frame_b) annotation(Line(points = {{ -54, -30},{ -60, -30},{ -60, -70}}));
        connect(b1.frame_b,universalSpherical.frame_b) annotation(Line(points = {{ -30,10},{ -30,30},{ -20,30}}));
        connect(universalSpherical.frame_a,b2.frame_b) annotation(Line(points = {{0,30},{50,30},{50, -40}}));
        connect(b3.frame_a,world.frame_b) annotation(Line(points = {{ -32, -70},{ -60, -70}}));
        connect(b3.frame_b,j2.frame_a) annotation(Line(points = {{ -12, -70},{12, -70}}));
        connect(fixedFrame4.frame_a,universalSpherical.frame_ia) annotation(Line(points = {{ -10,60},{ -10,50},{ -6,50},{ -6,40}}, color = {0,0,0}));
      end UniversalSpherical;
      encapsulated model UniversalSpherical_2 "One kinematic loop with four bars (with UniversalSpherical joint)"
        import ObsoleteModelica3;
        import Modelica;
        import Modelica.Mechanics.MultiBody;
        import SI = Modelica.SIunits;
        extends Modelica.Icons.Example;
        output SI.Angle j1_phi "angle of revolute joint j1";
        output SI.Position j2_s "distance of prismatic joint j2";
        output SI.AngularVelocity j1_w "axis speed of revolute joint j1";
        output SI.Velocity j2_v "axis velocity of prismatic joint j2";
        annotation(experiment(StopTime = 5), Documentation(info = "<html>
<p>
This is a fourth version of the \"four-bar\" mechanism. In this case
the three revolute joints on the left top-side and the two revolute
joints on the right top side have been replaced by the joint <b>UniversalSpherical</b>
that is a rod with a spherical and a universal joint on two sides. This joint is defined
by <b>1 constraint</b> stating that the distance between the two spherical joints is
constant. Using this joint in a kinematic loop reduces the sizes of
non-linear algebraic equations. For this loop, only one non-linear
algebraic system of equations of order 1 remains.
</p>
<p>
The essential difference to joint SphericalSpherical is that the
orientation of the rod can be computed by removing one degree of freedom
of one of the spherical joints (i.e., replacing it by a universal joint).
Usually, the eigenrotation of the connecting rod is of no technical
interest and by this approximation it is constrained to move in a
somewhat arbitrary way. This allows to have an additional connector,
<b>frame_ia</b>, to be fixed on the rod, where other objects can be attached.
In this example, the coordinate system of frame_ia is visualized.
</p>
<p>
Another nice feature is that the <b>length</b> of the connecting rod can be
automatically calculated during <b>initialization</b>. In order to do this,
another initialization condition has to be given. In this example, the
initial value of the distance of the prismatic joint j2 has been fixed
(via the \"Initialization\" menu) and the length parameter of joint
\"UniversalSpherical\" is computed during initialization since parameter
<b>computeLength</b> = <b>true</b> is set in the joint parameter
menu (this sets \"fixed=false\" on parameter \"length\").
</p>
</html>"), Diagram(graphics));
        inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -80, -80},{ -60, -60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Revolute j1(n = {1,0,0}, stateSelect = StateSelect.always, a(fixed = false), phi(fixed = true), w(fixed = true, start = 5.23598775598299)) annotation(Placement(transformation(extent = {{ -54, -40},{ -34, -20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Prismatic j2(n = {1,0,0}, boxWidth = 0.01, a(fixed = false), s(fixed = false), v(fixed = false), s_offset =  -0.2) annotation(Placement(transformation(extent = {{12, -80},{32, -60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyCylinder b1(r = {0,0.5,0.1}, diameter = 0.05) annotation(Placement(transformation(origin = { -30,0}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Mechanics.MultiBody.Parts.BodyCylinder b2(r = {0,0.2,0}, diameter = 0.05) annotation(Placement(transformation(origin = {50, -50}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Mechanics.MultiBody.Joints.UniversalSpherical universalSpherical(n1_a = {0,1,0.1}, rRod_ia = {{ -1,0.3,0.1} * { -1,0.3,0.1},0,0}, rRod_a(start = { -1,0.3,0.1})) annotation(Placement(transformation(extent = {{0,20},{ -20,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation b3(animation = false, r = {1,0,0}) annotation(Placement(transformation(extent = {{ -32, -80},{ -12, -60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedFrame fixedFrame4 annotation(Placement(transformation(origin = { -10,70}, extent = {{ -10, -10},{10,10}}, rotation = 90)));

      equation 
        j1_phi = j1.phi;
        j2_s = j2.s;
        j1_w = j1.w;
        j2_v = j2.v;
        connect(j2.frame_b,b2.frame_a) annotation(Line(points = {{32, -70},{50, -70},{50, -60}}));
        connect(j1.frame_b,b1.frame_a) annotation(Line(points = {{ -34, -30},{ -30, -30},{ -30, -10}}));
        connect(j1.frame_a,world.frame_b) annotation(Line(points = {{ -54, -30},{ -60, -30},{ -60, -70}}));
        connect(b1.frame_b,universalSpherical.frame_b) annotation(Line(points = {{ -30,10},{ -30,30},{ -20,30}}));
        connect(universalSpherical.frame_a,b2.frame_b) annotation(Line(points = {{0,30},{50,30},{50, -40}}));
        connect(b3.frame_a,world.frame_b) annotation(Line(points = {{ -32, -70},{ -60, -70}}));
        connect(b3.frame_b,j2.frame_a) annotation(Line(points = {{ -12, -70},{12, -70}}));
        connect(fixedFrame4.frame_a,universalSpherical.frame_ia) annotation(Line(points = {{ -10,60},{ -10,50},{ -6,50},{ -6,40}}, color = {0,0,0}));
      end UniversalSpherical_2;
      encapsulated model JointUSR "One kinematic loop with four bars (using JointUSR joint)"
        import ObsoleteModelica3;
        import Modelica;
        import Modelica.Mechanics.MultiBody;
        import SI = Modelica.SIunits;
        extends Modelica.Icons.Example;
        output SI.Angle j1_phi "angle of revolute joint j1";
        output SI.Position j2_s "distance of prismatic joint j2";
        output SI.AngularVelocity j1_w "axis speed of revolute joint j1";
        output SI.Velocity j2_v "axis velocity of prismatic joint j2";
        annotation(experiment(StopTime = 2), Documentation(info = "<html>
<p>
This is a fourth version of the \"four-bar\" mechanism. In this case
the three revolute joints on the left top-side and the two revolute
joints on the right top side have been replaced by the joint <b>UniversalSpherical</b>
that is a rod with a spherical and a universal joint on two sides. This joint is defined
by <b>1 constraint</b> stating that the distance between the two spherical joints is
constant. Using this joint in a kinematic loop reduces the sizes of
non-linear algebraic equations. For this loop, only one non-linear
algebraic system of equations of order 1 remains.
</p>
<p>
The essential difference to joint SphericalSpherical is that the
orientation of the rod can be computed by removing one degree of freedom
of one of the spherical joints (i.e., replacing it by a universal joint).
Usually, the eigenrotation of the connecting rod is of no technical
interest and by this approximation it is constrained to move in a
somewhat arbitrary way. This allows to have an additional connector,
<b>frame_ia</b>, to be fixed on the rod, where other objects can be attached.
In this example, the coordinate system of frame_ia is visualized.
</p>
<p>
Another nice feature is that the <b>length</b> of the connecting rod can be
automatically calculated during <b>initialization</b>. In order to do this,
another initialization condition has to be given. In this example, the
initial value of the distance of the prismatic joint j2 has been fixed
(via the \"Initialization\" menu) and the length parameter of joint
\"UniversalSpherical\" is computed during initialization since parameter
<b>computeLength</b> = <b>true</b> is set in the joint parameter
menu (this sets \"fixed=false\" on parameter \"length\").
</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experimentSetupOutput);
        inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -80, -80},{ -60, -60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Prismatic j2(n = {1,0,0}, stateSelect = StateSelect.always, a(fixed = false), s(fixed = true), v(fixed = true, start =  -0.4), s_offset =  -0.2) annotation(Placement(transformation(extent = {{12, -80},{32, -60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyCylinder b2(r = {0,0.2,0}, diameter = 0.05) annotation(Placement(transformation(origin = {50, -50}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation b3(animation = false, r = {1.2,0,0}) annotation(Placement(transformation(extent = {{ -32, -80},{ -12, -60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedFrame fixedFrame annotation(Placement(transformation(origin = { -6,50}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUSR jointUSR(n1_a = {0,1,0}, n_b = {1,0,0}, rRod2_ib = {0,0.5,0.1}, checkTotalPower = true, rRod1_ia = { -1,0.3, -0.1}) annotation(Placement(transformation(extent = {{0,0},{ -20,20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Body Body1(m = 1, I_11 = 0.1, I_22 = 0.1, I_33 = 0.1, r_CM = {0.01,0,0}) annotation(Placement(transformation(extent = {{ -74,40},{ -54,60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Body Body2(m = 1, I_11 = 0.1, I_22 = 0.1, I_33 = 0.1, r_CM = jointUSR.eRod1_ia / 2) annotation(Placement(transformation(extent = {{20,40},{40,60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Body Body3(m = 1, I_11 = 0.1, I_22 = 0.1, I_33 = 0.1) annotation(Placement(transformation(extent = {{ -38,68},{ -18,88}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Damper damper(d = 0.1) annotation(Placement(transformation(origin = { -64,12}, extent = {{ -10, -10},{10,10}}, rotation = 90)));

      equation 
        j1_phi = jointUSR.revolute.phi;
        j2_s = j2.s;
        j1_w = der(jointUSR.revolute.phi);
        j2_v = j2.v;
        connect(j2.frame_b,b2.frame_a) annotation(Line(points = {{32, -70},{50, -70},{50, -60}}));
        connect(b3.frame_a,world.frame_b) annotation(Line(points = {{ -32, -70},{ -60, -70}}));
        connect(b3.frame_b,j2.frame_a) annotation(Line(points = {{ -12, -70},{12, -70}}));
        connect(world.frame_b,jointUSR.frame_b) annotation(Line(points = {{ -60, -70},{ -50, -70},{ -50,10},{ -20,10}}));
        connect(jointUSR.frame_a,b2.frame_b) annotation(Line(points = {{0,10},{50,10},{50, -40}}));
        connect(jointUSR.frame_ia,fixedFrame.frame_a) annotation(Line(points = {{ -2,20},{ -4,32},{ -6,32},{ -6,40}}));
        connect(jointUSR.frame_ia,Body2.frame_a) annotation(Line(points = {{ -2,20},{ -4,32},{10,32},{10,50},{20,50}}));
        connect(jointUSR.frame_ib,Body1.frame_a) annotation(Line(points = {{ -18,20},{ -18,30},{ -80,30},{ -80,50},{ -74,50}}));
        connect(Body3.frame_a,jointUSR.frame_im) annotation(Line(points = {{ -38,78},{ -48,78},{ -48,34},{ -10,34},{ -10,20}}, color = {0,0,0}));
        connect(damper.flange_b,jointUSR.axis) annotation(Line(points = {{ -64,22},{ -36,22},{ -36,18},{ -20,18}}, color = {0,0,0}));
        connect(jointUSR.bearing,damper.flange_a) annotation(Line(points = {{ -20,14},{ -54,14},{ -54,2},{ -64,2}}, color = {0,0,0}));
      end JointUSR;
      encapsulated model JointSSR "One kinematic loop with four bars (using JointSSR joint)"
        import ObsoleteModelica3;
        import Modelica;
        import Modelica.Mechanics.MultiBody;
        import SI = Modelica.SIunits;
        extends Modelica.Icons.Example;
        parameter Modelica.SIunits.Distance L = Modelica.Math.Vectors.length({ -1,0.3,0.1});
        output SI.Angle j1_phi "angle of revolute joint j1";
        output SI.Position j2_s "distance of prismatic joint j2";
        output SI.AngularVelocity j1_w "axis speed of revolute joint j1";
        output SI.Velocity j2_v "axis velocity of prismatic joint j2";
        annotation(experiment(StopTime = 1.1), Documentation(info = "<html>
<p>
This is a fourth version of the \"four-bar\" mechanism. In this case
the three revolute joints on the left top-side and the two revolute
joints on the right top side have been replaced by the joint <b>UniversalSpherical</b>
that is a rod with a spherical and a universal joint on two sides. This joint is defined
by <b>1 constraint</b> stating that the distance between the two spherical joints is
constant. Using this joint in a kinematic loop reduces the sizes of
non-linear algebraic equations. For this loop, only one non-linear
algebraic system of equations of order 1 remains.
</p>
<p>
The essential difference to joint SphericalSpherical is that the
orientation of the rod can be computed by removing one degree of freedom
of one of the spherical joints (i.e., replacing it by a universal joint).
Usually, the eigenrotation of the connecting rod is of no technical
interest and by this approximation it is constrained to move in a
somewhat arbitrary way. This allows to have an additional connector,
<b>frame_ia</b>, to be fixed on the rod, where other objects can be attached.
In this example, the coordinate system of frame_ia is visualized.
</p>
<p>
Another nice feature is that the <b>length</b> of the connecting rod can be
automatically calculated during <b>initialization</b>. In order to do this,
another initialization condition has to be given. In this example, the
initial value of the distance of the prismatic joint j2 has been fixed
(via the \"Initialization\" menu) and the length parameter of joint
\"UniversalSpherical\" is computed during initialization since parameter
<b>computeLength</b> = <b>true</b> is set in the joint parameter
menu (this sets \"fixed=false\" on parameter \"length\").
</p>
</html>"), Diagram(graphics));
        inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -80, -80},{ -60, -60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Prismatic j2(n = {1,0,0}, stateSelect = StateSelect.always, a(fixed = false), s(fixed = true), v(fixed = true, start =  -0.2), s_offset =  -0.2) annotation(Placement(transformation(extent = {{12, -80},{32, -60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyCylinder b2(r = {0,0.2,0}, diameter = 0.05) annotation(Placement(transformation(origin = {50, -50}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation b3(animation = false, r = {1.2,0,0}) annotation(Placement(transformation(extent = {{ -32, -80},{ -12, -60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Assemblies.JointSSR jointSSR(n_b = {1,0,0}, rRod2_ib = {0,0.5,0.1}, rod1Length = L, checkTotalPower = true, rod1Mass = 0.1) annotation(Placement(transformation(extent = {{ -2,0},{ -22,20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedFrame FixedFrame1 annotation(Placement(transformation(extent = {{ -40,60},{ -60,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Body Body1(r_CM = jointSSR.rRod2_ib / 2, m = 0.1) annotation(Placement(transformation(extent = {{ -30,30},{ -50,50}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Body Body2(m = 0.1) annotation(Placement(transformation(extent = {{20,30},{40,50}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Damper damper(d = 0.5) annotation(Placement(transformation(origin = { -62,16}, extent = {{ -10, -10},{10,10}}, rotation = 90)));

      equation 
        j1_phi = jointSSR.revolute.phi;
        j2_s = j2.s;
        j1_w = der(jointSSR.revolute.phi);
        j2_v = j2.v;
        connect(j2.frame_b,b2.frame_a) annotation(Line(points = {{32, -70},{50, -70},{50, -60}}));
        connect(b3.frame_a,world.frame_b) annotation(Line(points = {{ -32, -70},{ -60, -70}}));
        connect(b3.frame_b,j2.frame_a) annotation(Line(points = {{ -12, -70},{12, -70}}));
        connect(world.frame_b,jointSSR.frame_b) annotation(Line(points = {{ -60, -70},{ -50, -70},{ -50,10},{ -22,10}}));
        connect(jointSSR.frame_a,b2.frame_b) annotation(Line(points = {{ -2,10},{50,10},{50, -40}}));
        connect(FixedFrame1.frame_a,jointSSR.frame_ib) annotation(Line(points = {{ -40,70},{ -20,70},{ -20,20}}, color = {0,0,0}));
        connect(Body1.frame_a,jointSSR.frame_ib) annotation(Line(points = {{ -30,40},{ -20,40},{ -20,20}}, color = {0,0,0}));
        connect(Body2.frame_a,jointSSR.frame_im) annotation(Line(points = {{20,40},{ -12,40},{ -12,20}}, color = {0,0,0}));
        connect(jointSSR.axis,damper.flange_b) annotation(Line(points = {{ -22,18},{ -42,18},{ -42,26},{ -62,26}}, color = {0,0,0}));
        connect(jointSSR.bearing,damper.flange_a) annotation(Line(points = {{ -22,14},{ -52,14},{ -52,6},{ -62,6}}, color = {0,0,0}));
      end JointSSR;
      encapsulated model JointUSP "One kinematic loop with four bars (using JointUSP joint)"
        import ObsoleteModelica3;
        import Modelica;
        import Modelica.Mechanics.MultiBody;
        import SI = Modelica.SIunits;
        extends Modelica.Icons.Example;
        output SI.Angle revolute_phi "angle of revolute joint j1";
        output SI.Position prismatic_s "distance of prismatic joint j2";
        output SI.AngularVelocity revolute_w "axis speed of revolute joint j1";
        output SI.Velocity prismatic_v "axis velocity of prismatic joint j2";
        annotation(experiment(StopTime = 5), Documentation(info = ""), Diagram(graphics));
        inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -80, -80},{ -60, -60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Revolute revolute(n = {1,0,0}, stateSelect = StateSelect.always, a(fixed = false), phi(fixed = true), w(fixed = true, start = 5.23598775598299)) annotation(Placement(transformation(extent = {{ -54, -40},{ -34, -20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyCylinder body1(r = {0,0.5,0.1}, diameter = 0.05) annotation(Placement(transformation(origin = { -30,0}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Mechanics.MultiBody.Parts.BodyCylinder body3(r = {0,0.2,0}, diameter = 0.05) annotation(Placement(transformation(origin = {50,70}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation ground_rod(animation = false, r = {1.2,0,0}) annotation(Placement(transformation(extent = {{ -32, -80},{ -12, -60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUSP jointUSP(rRod2_ib = {0,0.2,0}, n1_a = {0,0, -1}, n_b = { -1,0,0}, rod1Diameter = 0.04, boxWidth = 0.05, rRod1_ia = {1, -0.3,0.1}) annotation(Placement(transformation(extent = {{0,20},{20,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyCylinder body2(r = {1, -0.3,0.1}, diameter = 0.05) annotation(Placement(transformation(extent = {{8,60},{28,80}}, rotation = 0)));
        Modelica.Mechanics.Translational.Components.Damper damper(d = 50) annotation(Placement(transformation(origin = {52,30}, extent = {{10, -10},{ -10,10}}, rotation = 270)));

      equation 
        revolute_phi = revolute.phi;
        prismatic_s = jointUSP.prismatic.s;
        revolute_w = revolute.w;
        prismatic_v = der(jointUSP.prismatic.s);
        connect(revolute.frame_b,body1.frame_a) annotation(Line(points = {{ -34, -30},{ -30, -30},{ -30, -10}}, color = {0,0,0}, thickness = 0.5));
        connect(revolute.frame_a,world.frame_b) annotation(Line(points = {{ -54, -30},{ -60, -30},{ -60, -70}}, color = {0,0,0}, thickness = 0.5));
        connect(ground_rod.frame_a,world.frame_b) annotation(Line(points = {{ -32, -70},{ -60, -70}}, color = {0,0,0}, thickness = 0.5));
        connect(body1.frame_b,jointUSP.frame_a) annotation(Line(points = {{ -30,10},{ -30,30},{0,30}}, color = {0,0,0}, thickness = 0.5));
        connect(ground_rod.frame_b,jointUSP.frame_b) annotation(Line(points = {{ -12, -70},{30, -70},{30,30},{20,30}}, color = {0,0,0}, thickness = 0.5));
        connect(jointUSP.frame_ib,body3.frame_a) annotation(Line(points = {{18,40},{18,48},{50,48},{50,60}}, color = {0,0,0}, thickness = 0.5));
        connect(jointUSP.frame_ia,body2.frame_a) annotation(Line(points = {{2,40},{2,70},{8,70}}, color = {0,0,0}, thickness = 0.5));
        connect(jointUSP.bearing,damper.flange_a) annotation(Line(points = {{20,34},{38,34},{38,10},{52,10},{52,20}}, color = {0,191,0}));
        connect(jointUSP.axis,damper.flange_b) annotation(Line(points = {{20,38},{38,38},{38,44},{52,44},{52,40}}, color = {0,191,0}));
      end JointUSP;
    end FourbarVariants;
    package Frames "Test functions of package Frames"
      extends Modelica.Icons.Library;
      model AngularVelocity "Test angular velocity functions"
        import SI = Modelica.SIunits;
        import Modelica.Mechanics.MultiBody.Frames.*;
        extends Modelica.Icons.Example;
        Modelica.Mechanics.MultiBody.Frames.Orientation R = body.frame_a.R;
        SI.AngularVelocity wa[3] = angularVelocity2(R);
        SI.AngularVelocity w1[3] = resolve1(R, wa);
        SI.AngularVelocity w2[3] = angularVelocity1(R);
        SI.AngularVelocity w_err[3] = w2 - w1;
        inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -60,20},{ -40,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation bar2(r = {0.8,0,0}, animation = false) annotation(Placement(transformation(extent = {{0,20},{20,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Forces.Spring spring1(width = 0.1, coilWidth = 0.005, numberOfWindings = 5, c = 20, s_unstretched = 0) annotation(Placement(transformation(origin = { -24,6}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        Modelica.Mechanics.MultiBody.Parts.BodyShape body(m = 1, I_11 = 1, I_22 = 1, I_33 = 1, r = {0.4,0,0}, r_CM = {0.2,0,0}, width = 0.05, angles_start = {0.174532925199433,0.174532925199433,0.174532925199433}, a_0(fixed = false), angles_fixed = true, r_0(fixed = true, start = {0.2, -0.5,0.1}), v_0(fixed = true), w_0_fixed = true, z_0_fixed = false) annotation(Placement(transformation(extent = {{0, -40},{20, -20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Forces.Spring spring2(c = 20, s_unstretched = 0, width = 0.1, coilWidth = 0.005, numberOfWindings = 5) annotation(Placement(transformation(origin = {42,6}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        Modelica.Mechanics.MultiBody.Forces.WorldTorque FrameTorque1(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.frame_b) annotation(Placement(transformation(extent = {{58, -42},{38, -22}}, rotation = 0)));
        Modelica.Blocks.Sources.Constant Constant1[3](k = {10,0,0}) annotation(Placement(transformation(extent = {{94, -42},{74, -22}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(bar2.frame_a,world.frame_b) annotation(Line(points = {{0,30},{ -40,30}}, color = {0,0,0}, thickness = 0.5));
        connect(spring1.frame_b,body.frame_a) annotation(Line(points = {{ -24, -4},{ -24, -30},{0, -30}}, color = {0,0,0}, thickness = 0.5));
        connect(spring2.frame_b,body.frame_b) annotation(Line(points = {{42, -4},{20, -30}}, color = {0,0,0}, thickness = 0.5));
        connect(bar2.frame_b,spring2.frame_a) annotation(Line(points = {{20,30},{42,30},{42,16}}, color = {0,0,0}, thickness = 0.5));
        connect(spring1.frame_a,world.frame_b) annotation(Line(points = {{ -24,16},{ -24,30},{ -40,30}}, color = {0,0,0}, thickness = 0.5));
        annotation(Diagram(graphics));
        connect(FrameTorque1.frame_b,body.frame_b) annotation(Line(points = {{38, -32},{30, -32},{30, -30},{20, -30}}, color = {0,0,0}, thickness = 0.5));
        connect(Constant1.y,FrameTorque1.torque) annotation(Line(points = {{73, -32},{60, -32}}, color = {0,0,255}));
      end AngularVelocity;
    end Frames;
    package Forces
      model DifferentiationOfForces
        extends Modelica.Icons.Example;
        inner Modelica.Mechanics.MultiBody.World world(gravityType = Modelica.Mechanics.MultiBody.Types.GravityTypes.UniformGravity) annotation(Placement(transformation(extent = {{ -80,0},{ -60,20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Revolute rev(n = {0,0,1}, a(fixed = false), phi(fixed = true), w(fixed = true)) annotation(Placement(transformation(extent = {{ -40,0},{ -20,20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyBox body(r = {1,0,0}) annotation(Placement(transformation(extent = {{0,0},{20,20}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Forces.FrameForceAndTorque frameForceAndTorque annotation(Placement(transformation(extent = {{ -40, -40},{ -20, -20}}, rotation = 0)));
        Modelica.Blocks.Sources.Sine sine[6] annotation(Placement(transformation(extent = {{ -80, -40},{ -60, -20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Sensors.CutForce cutForce annotation(Placement(transformation(extent = {{0, -40},{20, -20}}, rotation = 0)));
        Modelica.Blocks.Continuous.Der der1[3] annotation(Placement(transformation(extent = {{20, -70},{40, -50}}, rotation = 0)));
        Modelica.Blocks.Continuous.Der der2[3] annotation(Placement(transformation(extent = {{54, -70},{74, -50}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(world.frame_b,rev.frame_a) annotation(Line(points = {{ -60,10},{ -40,10}}, color = {95,95,95}, thickness = 0.5));
        connect(body.frame_a,rev.frame_b) annotation(Line(points = {{0,10},{ -20,10}}, color = {95,95,95}, thickness = 0.5));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
        connect(sine.y,frameForceAndTorque.load) annotation(Line(points = {{ -59, -30},{ -42, -30}}, color = {0,0,127}));
        connect(frameForceAndTorque.frame_b,cutForce.frame_a) annotation(Line(points = {{ -20, -30},{0, -30}}, color = {95,95,95}, thickness = 0.5));
        connect(cutForce.frame_b,body.frame_b) annotation(Line(points = {{20, -30},{30, -30},{30,10},{20,10}}, color = {95,95,95}, thickness = 0.5));
        connect(cutForce.force,der1.u) annotation(Line(points = {{2, -41},{2, -60},{18, -60}}, color = {0,0,127}));
        connect(frameForceAndTorque.frame_resolve,body.frame_a) annotation(Line(points = {{ -30, -20},{ -30, -14},{ -12, -14},{ -12,10},{0,10}}, color = {95,95,95}, pattern = LinePattern.Dot));
        connect(der1.y,der2.u) annotation(Line(points = {{41, -60},{52, -60}}, color = {0,0,127}));
      end DifferentiationOfForces;
      model Force
        extends Modelica.Icons.Example;
        import Modelica.Mechanics.MultiBody.*;
        inner World world annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}}, rotation = 0)));
        Joints.Revolute Revolute1(n = {0,1,0}) annotation(Placement(transformation(origin = { -20,30}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
        Parts.BodyCylinder body1(r = {1,0,0}) annotation(Placement(transformation(extent = {{0,40},{20,60}}, rotation = 0)));
        Modelica.Blocks.Sources.Constant Constant1[3](k = {0,500,0}) annotation(Placement(transformation(origin = {50,90}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
        Parts.Fixed fixed1(r = {0,0.5,0}) annotation(Placement(transformation(extent = {{ -90,0},{ -70,20}}, rotation = 0)));
        Parts.BodyCylinder body2(r = {1,0,0}) annotation(Placement(transformation(extent = {{0, -30},{20, -10}}, rotation = 0)));
        Parts.Fixed fixed3(r = {0, -0.5,0}) annotation(Placement(transformation(extent = {{ -88, -70},{ -68, -50}}, rotation = 0)));
        Parts.FixedRotation fixedRotation(n = {0,0,1}, angle = 45) annotation(Placement(transformation(extent = {{ -8, -90},{12, -70}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Forces.Force force1(N_to_m = 600, resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_b) annotation(Placement(transformation(extent = {{62,40},{42,60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Forces.Force force2(N_to_m = 600, resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve) annotation(Placement(transformation(extent = {{60, -10},{40, -30}}, rotation = 0)));
        Joints.Revolute Revolute2(n = {0,1,0}) annotation(Placement(transformation(origin = { -20, -40}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Blocks.Sources.Constant Constant2[3](k = {0,500,0}) annotation(Placement(transformation(origin = {70, -70}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
        Joints.Revolute Revolute3 annotation(Placement(transformation(extent = {{ -52,0},{ -32,20}}, rotation = 0)));
        Joints.Revolute Revolute4 annotation(Placement(transformation(extent = {{ -62, -70},{ -42, -50}}, rotation = 0)));
        Parts.Fixed fixed2(animation = false, r = {1.5,0.25,0}) annotation(Placement(transformation(extent = {{100,40},{80,60}}, rotation = 0)));
        Parts.Fixed fixed4(animation = false, r = {1.5, -0.25,0}) annotation(Placement(transformation(extent = {{100, -30},{80, -10}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(force1.frame_b,body1.frame_b) annotation(Line(points = {{42,50},{20,50}}));
        connect(Revolute1.frame_b,body1.frame_a) annotation(Line(points = {{ -20,40},{ -20,50},{0,50}}));
        connect(Revolute2.frame_b,body2.frame_a) annotation(Line(points = {{ -20, -30},{ -20, -20},{0, -20}}));
        connect(force2.frame_b,body2.frame_b) annotation(Line(points = {{40, -20},{20, -20}}));
        connect(Revolute3.frame_a,fixed1.frame_b) annotation(Line(points = {{ -52,10},{ -70,10}}));
        connect(Revolute3.frame_b,Revolute1.frame_a) annotation(Line(points = {{ -32,10},{ -20,10},{ -20,20}}));
        connect(fixedRotation.frame_b,force2.frame_resolve) annotation(Line(points = {{12, -80},{46, -80},{46, -30}}));
        connect(fixed3.frame_b,Revolute4.frame_a) annotation(Line(points = {{ -68, -60},{ -62, -60}}));
        connect(Revolute4.frame_b,Revolute2.frame_a) annotation(Line(points = {{ -42, -60},{ -20, -60},{ -20, -50}}));
        connect(fixed3.frame_b,fixedRotation.frame_a) annotation(Line(points = {{ -68, -60},{ -64, -60},{ -64, -80},{ -8, -80}}));
        connect(Constant1.y,force1.force) annotation(Line(points = {{50,79},{50,72},{58,72},{58,62}}, color = {0,0,255}));
        connect(Constant2.y,force2.force) annotation(Line(points = {{70, -59},{70, -42},{56, -42},{56, -32}}, color = {0,0,255}));
        connect(force1.frame_a,fixed2.frame_b) annotation(Line(points = {{62,50},{71.5,50},{71.5,50},{80,50}}, color = {0,0,0}, thickness = 0.5));
        connect(fixed4.frame_b,force2.frame_a) annotation(Line(points = {{80, -20},{70, -20},{70, -20},{60, -20}}, color = {0,0,0}, thickness = 0.5));
      end Force;
      model Force2
        extends Modelica.Icons.Example;
        import Modelica.Mechanics.MultiBody.*;
        inner World world annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}}, rotation = 0)));
        Joints.Revolute Revolute1(n = {0,1,0}) annotation(Placement(transformation(origin = { -20,30}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Parts.BodyCylinder body1(r = {1,0,0}) annotation(Placement(transformation(extent = {{0,40},{20,60}}, rotation = 0)));
        Modelica.Blocks.Sources.Constant Constant1[3](k = {0,500,0}) annotation(Placement(transformation(origin = {50,90}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
        Parts.Fixed fixed1(r = {0,0.5,0}) annotation(Placement(transformation(extent = {{ -90,0},{ -70,20}}, rotation = 0)));
        Parts.BodyCylinder body2(r = {1,0,0}) annotation(Placement(transformation(extent = {{0, -30},{20, -10}}, rotation = 0)));
        Parts.Fixed fixed3(r = {0, -0.5,0}) annotation(Placement(transformation(extent = {{ -88, -70},{ -68, -50}}, rotation = 0)));
        Parts.FixedRotation fixedRotation(n = {0,0,1}, angle = 45) annotation(Placement(transformation(extent = {{ -8, -90},{12, -70}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Forces.Force force1(N_to_m = 600, resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_b) annotation(Placement(transformation(extent = {{62,40},{42,60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Forces.Force force2(N_to_m = 600, resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve) annotation(Placement(transformation(extent = {{60, -10},{40, -30}}, rotation = 0)));
        Joints.Revolute Revolute2(n = {0,1,0}) annotation(Placement(transformation(origin = { -20, -40}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Blocks.Sources.Constant Constant2[3](k = {0,500,0}) annotation(Placement(transformation(origin = {70, -70}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
        Joints.Revolute Revolute3 annotation(Placement(transformation(extent = {{ -52,0},{ -32,20}}, rotation = 0)));
        Joints.Revolute Revolute4 annotation(Placement(transformation(extent = {{ -62, -70},{ -42, -50}}, rotation = 0)));
        Parts.Fixed fixed2(animation = false, r = {1.5,0.25,0}) annotation(Placement(transformation(extent = {{100,40},{80,60}}, rotation = 0)));
        Parts.Fixed fixed4(animation = false, r = {1.5, -0.25,0}) annotation(Placement(transformation(extent = {{100, -30},{80, -10}}, rotation = 0)));
        Joints.Revolute Revolute5(n = {0,1,0}) annotation(Placement(transformation(origin = { -20, -180}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Parts.BodyCylinder body3(r = {1,0,0}) annotation(Placement(transformation(extent = {{0, -170},{20, -150}}, rotation = 0)));
        Modelica.Blocks.Sources.Constant Constant3[6](k = {0,500,0,0,0,0}) annotation(Placement(transformation(origin = {50, -120}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
        Parts.Fixed fixed5(r = {0,0.5,0}) annotation(Placement(transformation(extent = {{ -90, -210},{ -70, -190}}, rotation = 0)));
        Parts.BodyCylinder body4(r = {1,0,0}) annotation(Placement(transformation(extent = {{0, -240},{20, -220}}, rotation = 0)));
        Parts.Fixed fixed6(r = {0, -0.5,0}) annotation(Placement(transformation(extent = {{ -88, -280},{ -68, -260}}, rotation = 0)));
        Parts.FixedRotation fixedRotation1(n = {0,0,1}, angle = 45) annotation(Placement(transformation(extent = {{ -8, -300},{12, -280}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Forces.ForceAndTorque force3(N_to_m = 600) annotation(Placement(transformation(extent = {{62, -170},{42, -150}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Forces.ForceAndTorque force4(N_to_m = 600) annotation(Placement(transformation(extent = {{60, -220},{40, -240}}, rotation = 0)));
        Joints.Revolute Revolute6(n = {0,1,0}) annotation(Placement(transformation(origin = { -20, -250}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Blocks.Sources.Constant Constant4[6](k = {0,500,0,0,0,0}) annotation(Placement(transformation(origin = {70, -280}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
        Joints.Revolute Revolute7 annotation(Placement(transformation(extent = {{ -52, -210},{ -32, -190}}, rotation = 0)));
        Joints.Revolute Revolute8 annotation(Placement(transformation(extent = {{ -62, -280},{ -42, -260}}, rotation = 0)));
        Parts.Fixed fixed7(animation = false, r = {1.5,0.25,0}) annotation(Placement(transformation(extent = {{100, -170},{80, -150}}, rotation = 0)));
        Parts.Fixed fixed8(animation = false, r = {1.5, -0.25,0}) annotation(Placement(transformation(extent = {{100, -240},{80, -220}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(force1.frame_b,body1.frame_b) annotation(Line(points = {{42,50},{20,50}}));
        connect(Revolute1.frame_b,body1.frame_a) annotation(Line(points = {{ -20,40},{ -20,50},{0,50}}));
        connect(Revolute2.frame_b,body2.frame_a) annotation(Line(points = {{ -20, -30},{ -20, -20},{0, -20}}));
        connect(force2.frame_b,body2.frame_b) annotation(Line(points = {{40, -20},{20, -20}}));
        connect(Revolute3.frame_a,fixed1.frame_b) annotation(Line(points = {{ -52,10},{ -70,10}}));
        connect(Revolute3.frame_b,Revolute1.frame_a) annotation(Line(points = {{ -32,10},{ -20,10},{ -20,20}}));
        connect(fixedRotation.frame_b,force2.frame_resolve) annotation(Line(points = {{12, -80},{46, -80},{46, -30}}));
        connect(fixed3.frame_b,Revolute4.frame_a) annotation(Line(points = {{ -68, -60},{ -62, -60}}));
        connect(Revolute4.frame_b,Revolute2.frame_a) annotation(Line(points = {{ -42, -60},{ -20, -60},{ -20, -50}}));
        connect(fixed3.frame_b,fixedRotation.frame_a) annotation(Line(points = {{ -68, -60},{ -64, -60},{ -64, -80},{ -8, -80}}));
        connect(Constant1.y,force1.force) annotation(Line(points = {{50,79},{50,72},{58,72},{58,62}}, color = {0,0,255}));
        connect(Constant2.y,force2.force) annotation(Line(points = {{70, -59},{70, -42},{56, -42},{56, -32}}, color = {0,0,255}));
        connect(force1.frame_a,fixed2.frame_b) annotation(Line(points = {{62,50},{71.5,50},{71.5,50},{80,50}}, color = {0,0,0}, thickness = 0.5));
        connect(fixed4.frame_b,force2.frame_a) annotation(Line(points = {{80, -20},{70, -20},{70, -20},{60, -20}}, color = {0,0,0}, thickness = 0.5));
        connect(force3.frame_b,body3.frame_b) annotation(Line(points = {{42, -160},{20, -160}}));
        connect(Revolute5.frame_b,body3.frame_a) annotation(Line(points = {{ -20, -170},{ -20, -160},{0, -160}}));
        connect(Revolute6.frame_b,body4.frame_a) annotation(Line(points = {{ -20, -240},{ -20, -230},{0, -230}}));
        connect(force4.frame_b,body4.frame_b) annotation(Line(points = {{40, -230},{20, -230}}));
        connect(Revolute7.frame_a,fixed5.frame_b) annotation(Line(points = {{ -52, -200},{ -70, -200}}));
        connect(Revolute7.frame_b,Revolute5.frame_a) annotation(Line(points = {{ -32, -200},{ -20, -200},{ -20, -190}}));
        connect(fixedRotation1.frame_b,force4.frame_resolve) annotation(Line(points = {{12, -290},{46, -290},{46, -240}}));
        connect(fixed6.frame_b,Revolute8.frame_a) annotation(Line(points = {{ -68, -270},{ -62, -270}}));
        connect(Revolute8.frame_b,Revolute6.frame_a) annotation(Line(points = {{ -42, -270},{ -20, -270},{ -20, -260}}));
        connect(fixed6.frame_b,fixedRotation1.frame_a) annotation(Line(points = {{ -68, -270},{ -64, -270},{ -64, -290},{ -8, -290}}));
        connect(Constant3.y,force3.load) annotation(Line(points = {{50, -131},{50, -138},{58, -138},{58, -148}}, color = {0,0,255}));
        connect(Constant4.y,force4.load) annotation(Line(points = {{70, -269},{70, -252},{56, -252},{56, -242}}, color = {0,0,255}));
        connect(force3.frame_a,fixed7.frame_b) annotation(Line(points = {{62, -160},{71.5, -160},{71.5, -160},{80, -160}}, color = {0,0,0}, thickness = 0.5));
        connect(fixed8.frame_b,force4.frame_a) annotation(Line(points = {{80, -230},{70, -230},{70, -230},{60, -230}}, color = {0,0,0}, thickness = 0.5));
      end Force2;
      model ForcesAndTorques
        extends Modelica.Icons.Example;
        import Modelica.Mechanics.MultiBody.*;
        inner World world annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}}, rotation = 0)));
        Joints.Revolute Revolute1(n = {0,1,0}) annotation(Placement(transformation(origin = { -20,30}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
        Parts.BodyCylinder body1(r = {1,0,0}) annotation(Placement(transformation(extent = {{0,40},{20,60}}, rotation = 0)));
        Modelica.Blocks.Sources.Constant Constant1[6](k = {0,500,0, -100,100,0}) annotation(Placement(transformation(origin = {50,90}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
        Parts.Fixed fixed1(r = {0,0.5,0}) annotation(Placement(transformation(extent = {{ -90,0},{ -70,20}}, rotation = 0)));
        Parts.BodyCylinder body2(r = {1,0,0}) annotation(Placement(transformation(extent = {{0, -30},{20, -10}}, rotation = 0)));
        Parts.Fixed fixed3(r = {0, -0.5,0}) annotation(Placement(transformation(extent = {{ -88, -70},{ -68, -50}}, rotation = 0)));
        Parts.FixedRotation fixedRotation(n = {0,0,1}, angle = 45) annotation(Placement(transformation(extent = {{ -8, -90},{12, -70}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Forces.ForceAndTorque forceAndTorque1(Nm_to_m = 120, N_to_m = 600) annotation(Placement(transformation(extent = {{62,40},{42,60}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Forces.ForceAndTorque forceAndTorque2(Nm_to_m = 120, N_to_m = 600) annotation(Placement(transformation(extent = {{60, -10},{40, -30}}, rotation = 0)));
        Joints.Revolute Revolute2(n = {0,1,0}) annotation(Placement(transformation(origin = { -20, -40}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Blocks.Sources.Constant Constant2[6](k = {0,500,0, -100,100,0}) annotation(Placement(transformation(origin = {70, -70}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
        Joints.Revolute Revolute3 annotation(Placement(transformation(extent = {{ -52,0},{ -32,20}}, rotation = 0)));
        Joints.Revolute Revolute4 annotation(Placement(transformation(extent = {{ -62, -70},{ -42, -50}}, rotation = 0)));
        Parts.Fixed fixed2(animation = false, r = {1.5,0.25,0}) annotation(Placement(transformation(extent = {{100,40},{80,60}}, rotation = 0)));
        Parts.Fixed fixed4(animation = false, r = {1.5, -0.25,0}) annotation(Placement(transformation(extent = {{100, -30},{80, -10}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(forceAndTorque1.frame_b,body1.frame_b) annotation(Line(points = {{42,50},{20,50}}));
        connect(Revolute1.frame_b,body1.frame_a) annotation(Line(points = {{ -20,40},{ -20,50},{0,50}}));
        connect(Revolute2.frame_b,body2.frame_a) annotation(Line(points = {{ -20, -30},{ -20, -20},{0, -20}}));
        connect(forceAndTorque2.frame_b,body2.frame_b) annotation(Line(points = {{40, -20},{20, -20}}));
        connect(Revolute3.frame_a,fixed1.frame_b) annotation(Line(points = {{ -52,10},{ -70,10}}));
        connect(Revolute3.frame_b,Revolute1.frame_a) annotation(Line(points = {{ -32,10},{ -20,10},{ -20,20}}));
        connect(fixedRotation.frame_b,forceAndTorque2.frame_resolve) annotation(Line(points = {{12, -80},{46, -80},{46, -30}}));
        connect(fixed3.frame_b,Revolute4.frame_a) annotation(Line(points = {{ -68, -60},{ -62, -60}}));
        connect(Revolute4.frame_b,Revolute2.frame_a) annotation(Line(points = {{ -42, -60},{ -20, -60},{ -20, -50}}));
        connect(fixed3.frame_b,fixedRotation.frame_a) annotation(Line(points = {{ -68, -60},{ -64, -60},{ -64, -80},{ -8, -80}}));
        connect(Constant1.y,forceAndTorque1.load) annotation(Line(points = {{50,79},{50,72},{58,72},{58,62}}, color = {0,0,255}));
        connect(Constant2.y,forceAndTorque2.load) annotation(Line(points = {{70, -59},{70, -42},{56, -42},{56, -32}}, color = {0,0,255}));
        connect(forceAndTorque1.frame_a,fixed2.frame_b) annotation(Line(points = {{62,50},{71.5,50},{71.5,50},{80,50}}, color = {0,0,0}, thickness = 0.5));
        connect(fixed4.frame_b,forceAndTorque2.frame_a) annotation(Line(points = {{80, -20},{70, -20},{70, -20},{60, -20}}, color = {0,0,0}, thickness = 0.5));
      end ForcesAndTorques;
      model ForceWithTwoMasses
        extends Modelica.Icons.Example;
        annotation(experiment(StopTime = 3), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
        inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -100,20},{ -80,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Revolute revolute annotation(Placement(transformation(extent = {{ -40,20},{ -20,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyBox BodyBox1(r = {0.7,0,0}) annotation(Placement(transformation(extent = {{0,20},{20,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation FixedTranslation1(r = {0, -0.9,0}, width = 0.01, animation = false) annotation(Placement(transformation(origin = { -60, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        Modelica.Mechanics.MultiBody.Forces.LineForceWithTwoMasses force(L_a = 0.3, L_b = 0.3, m_a = 1, m_b = 1) annotation(Placement(transformation(extent = {{ -20, -20},{0, -40}}, rotation = 0)));
        annotation(Diagram);
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation FixedTranslation2(width = 0.01, animation = false, r = {0,0.1,0}) annotation(Placement(transformation(origin = { -60,50}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
        Modelica.Mechanics.Translational.Components.Damper damper(d = 10) annotation(Placement(transformation(extent = {{ -20, -80},{0, -60}}, rotation = 0)));

      equation 
        connect(BodyBox1.frame_a,revolute.frame_b) annotation(Line(points = {{0,30},{ -20,30}}));
        connect(force.frame_a,FixedTranslation1.frame_b) annotation(Line(points = {{ -20, -30},{ -60, -30},{ -60, -20}}));
        connect(force.frame_b,BodyBox1.frame_b) annotation(Line(points = {{0, -30},{40, -30},{40,30},{20,30}}));
        connect(world.frame_b,FixedTranslation2.frame_a) annotation(Line(points = {{ -80,30},{ -60,30},{ -60,40}}));
        connect(FixedTranslation2.frame_b,revolute.frame_a) annotation(Line(points = {{ -60,60},{ -60,70},{ -46,70},{ -46,30},{ -40,30}}));
        connect(world.frame_b,FixedTranslation1.frame_a) annotation(Line(points = {{ -80,30},{ -60,30},{ -60,0}}));
        connect(force.flange_a,damper.flange_a) annotation(Line(points = {{ -16, -41},{ -16, -54},{ -20, -54},{ -20, -70}}, color = {0,191,0}));
        connect(force.flange_b,damper.flange_b) annotation(Line(points = {{ -4, -41},{ -4, -54},{0, -54},{0, -70}}, color = {0,191,0}));
      end ForceWithTwoMasses;
      model FrameForces
        extends Modelica.Icons.Example;
        import Modelica.Mechanics.MultiBody.*;
        inner World world annotation(Placement(transformation(extent = {{ -100,80},{ -80,100}}, rotation = 0)));
        Joints.Revolute Revolute1 annotation(Placement(transformation(extent = {{ -40,40},{ -20,60}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
        Parts.BodyCylinder BodyCylinder1(r = {1,0,0}) annotation(Placement(transformation(extent = {{0,40},{20,60}}, rotation = 0)));
        Modelica.Blocks.Sources.Constant Constant1[3](k = {0,500,0}) annotation(Placement(transformation(extent = {{100,40},{80,60}}, rotation = 0)));
        Parts.Fixed fixed1(r = {0,0.5,0}) annotation(Placement(transformation(extent = {{ -80,40},{ -60,60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Forces.WorldForce frameForce1(N_to_m = 600, resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.frame_b) annotation(Placement(transformation(extent = {{60,40},{40,60}}, rotation = 0)));
        Joints.Revolute Revolute2 annotation(Placement(transformation(extent = {{ -40,0},{ -20,20}}, rotation = 0)));
        Parts.BodyCylinder BodyCylinder2(r = {1,0,0}) annotation(Placement(transformation(extent = {{0,0},{20,20}}, rotation = 0)));
        Parts.Fixed fixed2(r = {0, -0.5,0}) annotation(Placement(transformation(extent = {{ -80,0},{ -60,20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Forces.WorldForce frameForce2(N_to_m = 600, resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.frame_resolve) annotation(Placement(transformation(extent = {{60,0},{40,20}}, rotation = 0)));
        Parts.FixedRotation FixedRotation1(n = {0,0,1}, angle = 45) annotation(Placement(transformation(extent = {{ -40, -40},{ -20, -20}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(BodyCylinder1.frame_a,Revolute1.frame_b) annotation(Line(points = {{0,50},{ -20,50}}));
        connect(fixed1.frame_b,Revolute1.frame_a) annotation(Line(points = {{ -60,50},{ -40,50}}));
        connect(frameForce1.frame_b,BodyCylinder1.frame_b) annotation(Line(points = {{40,50},{20,50}}));
        connect(Constant1.y,frameForce1.force) annotation(Line(points = {{79,50},{62,50}}, color = {0,0,255}));
        connect(BodyCylinder2.frame_a,Revolute2.frame_b) annotation(Line(points = {{0,10},{ -20,10}}));
        connect(fixed2.frame_b,Revolute2.frame_a) annotation(Line(points = {{ -60,10},{ -40,10}}));
        connect(frameForce2.frame_b,BodyCylinder2.frame_b) annotation(Line(points = {{40,10},{20,10}}));
        connect(fixed2.frame_b,FixedRotation1.frame_a) annotation(Line(points = {{ -60,10},{ -52,10},{ -52, -30},{ -40, -30}}));
        connect(FixedRotation1.frame_b,frameForce2.frame_resolve) annotation(Line(points = {{ -20, -30},{50, -30},{50,0}}));
        connect(Constant1.y,frameForce2.force) annotation(Line(points = {{79,50},{72,50},{72,10},{62,10}}, color = {0,0,255}));
      end FrameForces;
      model FrameForcesAndTorques
        extends Modelica.Icons.Example;
        import Modelica.Mechanics.MultiBody.*;
        inner World world annotation(Placement(transformation(extent = {{ -100,80},{ -80,100}}, rotation = 0)));
        Joints.Revolute Revolute1(n = {0,1,0}) annotation(Placement(transformation(origin = { -20,50}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
        Parts.BodyCylinder body1(r = {1,0,0}) annotation(Placement(transformation(extent = {{0,60},{20,80}}, rotation = 0)));
        Modelica.Blocks.Sources.Constant Constant1[6](k = {0,500,0, -100,100,0}) annotation(Placement(transformation(extent = {{100,60},{80,80}}, rotation = 0)));
        Parts.Fixed fixed1(r = {0,0.5,0}) annotation(Placement(transformation(extent = {{ -80,20},{ -60,40}}, rotation = 0)));
        Parts.BodyCylinder body2(r = {1,0,0}) annotation(Placement(transformation(extent = {{0, -30},{20, -10}}, rotation = 0)));
        Parts.Fixed fixed2(r = {0, -0.5,0}) annotation(Placement(transformation(extent = {{ -88, -70},{ -68, -50}}, rotation = 0)));
        Parts.FixedRotation FixedRotation1(n = {0,0,1}, angle = 45) annotation(Placement(transformation(extent = {{ -8, -90},{12, -70}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Forces.FrameForceAndTorque frameTorque1(Nm_to_m = 120, N_to_m = 600) annotation(Placement(transformation(extent = {{60,60},{40,80}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Forces.FrameForceAndTorque frameTorque2(Nm_to_m = 120, N_to_m = 600) annotation(Placement(transformation(extent = {{60, -10},{40, -30}}, rotation = 0)));
        Joints.Revolute Revolute2(n = {0,1,0}) annotation(Placement(transformation(origin = { -20, -40}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Blocks.Sources.Constant Constant2[6](k = {0,500,0, -100,100,0}) annotation(Placement(transformation(extent = {{100, -30},{80, -10}}, rotation = 0)));
        Joints.Revolute Revolute3 annotation(Placement(transformation(extent = {{ -52,20},{ -32,40}}, rotation = 0)));
        Joints.Revolute Revolute4 annotation(Placement(transformation(extent = {{ -62, -70},{ -42, -50}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(frameTorque1.frame_b,body1.frame_b) annotation(Line(points = {{40,70},{20,70}}));
        connect(Constant1.y,frameTorque1.load) annotation(Line(points = {{79,70},{62,70}}, color = {0,0,255}));
        connect(Revolute1.frame_b,body1.frame_a) annotation(Line(points = {{ -20,60},{ -20,70},{0,70}}));
        connect(Revolute2.frame_b,body2.frame_a) annotation(Line(points = {{ -20, -30},{ -20, -20},{0, -20}}));
        connect(frameTorque2.frame_b,body2.frame_b) annotation(Line(points = {{40, -20},{20, -20}}));
        connect(Constant2.y,frameTorque2.load) annotation(Line(points = {{79, -20},{62, -20}}, color = {0,0,255}));
        connect(Revolute3.frame_a,fixed1.frame_b) annotation(Line(points = {{ -52,30},{ -60,30}}));
        connect(Revolute3.frame_b,Revolute1.frame_a) annotation(Line(points = {{ -32,30},{ -20,30},{ -20,40}}));
        connect(FixedRotation1.frame_b,frameTorque2.frame_resolve) annotation(Line(points = {{12, -80},{50, -80},{50, -30}}));
        connect(fixed2.frame_b,Revolute4.frame_a) annotation(Line(points = {{ -68, -60},{ -62, -60}}));
        connect(Revolute4.frame_b,Revolute2.frame_a) annotation(Line(points = {{ -42, -60},{ -20, -60},{ -20, -50}}));
        connect(fixed2.frame_b,FixedRotation1.frame_a) annotation(Line(points = {{ -68, -60},{ -64, -60},{ -64, -80},{ -8, -80}}));
      end FrameForcesAndTorques;
      model FrameTorques
        import Modelica.Mechanics.MultiBody.*;
        extends Modelica.Icons.Example;
        inner World world annotation(Placement(transformation(extent = {{ -100,80},{ -80,100}}, rotation = 0)));
        Joints.Revolute Revolute1(n = {0,1,0}) annotation(Placement(transformation(origin = { -30,50}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
        Parts.BodyCylinder body1(r = {1,0,0}) annotation(Placement(transformation(extent = {{0,80},{20,100}}, rotation = 0)));
        Modelica.Blocks.Sources.Constant Constant1[3](k = {0,100,0}) annotation(Placement(transformation(extent = {{100,80},{80,100}}, rotation = 0)));
        Parts.Fixed fixed1(r = {0,0.5,0}) annotation(Placement(transformation(extent = {{ -80,20},{ -60,40}}, rotation = 0)));
        Parts.BodyCylinder body2(r = {1,0,0}) annotation(Placement(transformation(extent = {{0,0},{20,20}}, rotation = 0)));
        Parts.Fixed fixed2(r = {0, -0.5,0}) annotation(Placement(transformation(extent = {{ -80, -40},{ -60, -20}}, rotation = 0)));
        Parts.FixedRotation FixedRotation1(n = {0,0,1}, angle = 45) annotation(Placement(transformation(extent = {{0, -60},{20, -40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Forces.WorldTorque frameTorque1(Nm_to_m = 120, resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.frame_b) annotation(Placement(transformation(extent = {{60,80},{40,100}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Forces.WorldTorque frameTorque2(Nm_to_m = 120, resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.frame_resolve) annotation(Placement(transformation(extent = {{60,20},{40,0}}, rotation = 0)));
        Joints.Revolute Revolute2(n = {0,1,0}) annotation(Placement(transformation(origin = { -30, -10}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Blocks.Sources.Constant Constant2[3](k = {0,100,0}) annotation(Placement(transformation(extent = {{100,0},{80,20}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(fixed2.frame_b,FixedRotation1.frame_a) annotation(Line(points = {{ -60, -30},{ -52, -30},{ -52, -50},{0, -50}}));
        connect(frameTorque1.frame_b,body1.frame_b) annotation(Line(points = {{40,90},{20,90}}));
        connect(fixed1.frame_b,Revolute1.frame_a) annotation(Line(points = {{ -60,30},{ -30,30},{ -30,40}}));
        connect(Revolute1.frame_b,body1.frame_a) annotation(Line(points = {{ -30,60},{ -30,90},{0,90}}));
        connect(fixed2.frame_b,Revolute2.frame_a) annotation(Line(points = {{ -60, -30},{ -30, -30},{ -30, -20}}));
        connect(Revolute2.frame_b,body2.frame_a) annotation(Line(points = {{ -30,0},{ -30,10},{0,10}}));
        connect(frameTorque2.frame_b,body2.frame_b) annotation(Line(points = {{40,10},{20,10}}));
        connect(FixedRotation1.frame_b,frameTorque2.frame_resolve) annotation(Line(points = {{20, -50},{50, -50},{50,0}}));
        connect(Constant1.y,frameTorque1.torque) annotation(Line(points = {{79,90},{62,90}}, color = {0,0,127}, smooth = Smooth.None));
        connect(Constant2.y,frameTorque2.torque) annotation(Line(points = {{79,10},{62,10}}, color = {0,0,127}, smooth = Smooth.None));
      end FrameTorques;
      model LineForceWithTwoMasses "Example to demonstrate how to construct force elements with masses"
        import SI = Modelica.SIunits;
        extends Modelica.Icons.Example;
        parameter Modelica.SIunits.Mass m = 1 "Mass of point masses";
        SI.Force rod_f_diff[3] = rod1.frame_b.f - rod3.frame_b.f "Difference of cut-forces in rod1 and rod3";
        SI.Force body_f_diff[3] = bodyBox1.frame_b.f - bodyBox2.frame_b.f "Difference of cut-forces in bodyBox1 and bodyBox2";
        annotation(experiment(StopTime = 3), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Documentation(info = "<html>
<p>
With this example it is demonstrated how to use the Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUPS
joint to build up a force element with masses and inertias (note, Modelica.Mechanics.MultiBody.Forces.LineForceWithMass
is a default line force element with a point mass to approximate the mass properties of the
component):
</p>
<p align=\"center\">
<IMG SRC=\"../Images/Examples/Elementary/ForceWithMasses.png\"
ALT=\"model Examples.Elementary.ForceWithMasses\">
</p>
<p>
A JointUPS consists of
a universal, prismatic and spherical joint aggregation that approximates
a real force component, such as a hydraulic cylinder. At frame frame_ia at the
universal joint and at frame frame_ib at the spherical joint, bodies can be
attached describing the mass properties of the component. Between the 1-dimensional
mechanical translational flanges \"axis\" and \"bearing\" a one-dimensional
force law can be attached, e.g., from the Modelica.Mechanics.Translational or
the HyLib package (= library of hydraulic components). In this example, just
a linear damper is connected here.
</p>
</html>"));
        inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Revolute revolute1 annotation(Placement(transformation(extent = {{ -20,60},{0,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyBox bodyBox1(r = {0.7,0,0}) annotation(Placement(transformation(extent = {{20,60},{40,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod1(r = {0, -0.9,0}, width = 0.01, animation = false) annotation(Placement(transformation(origin = { -40,54}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUPS jointUPS(nAxis_ia = {0.7,1.2,0}, animation = true) annotation(Placement(transformation(extent = {{0,50},{20,30}}, rotation = 0)));
        annotation(Diagram);
        Modelica.Mechanics.MultiBody.Parts.Body body1(r_CM = 0.2 * jointUPS.eAxis_ia, cylinderDiameter = 0.05, animation = true, m = m, I_11 = 0, I_22 = 0, I_33 = 0) annotation(Placement(transformation(extent = {{ -14,14},{ -34,34}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Body body2(r_CM =  -0.2 * jointUPS.eAxis_ia, cylinderDiameter = 0.05, animation = true, m = m, I_11 = 0, I_22 = 0, I_33 = 0) annotation(Placement(transformation(extent = {{32,14},{52,34}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod2(r = {0,0.3,0}, width = 0.01, animation = false) annotation(Placement(transformation(origin = { -40,84}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
        Modelica.Mechanics.Translational.Components.Damper damper1(d = 3) annotation(Placement(transformation(extent = {{0,24},{20,4}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Revolute revolute2 annotation(Placement(transformation(extent = {{ -20, -40},{0, -20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyBox bodyBox2(r = {0.7,0,0}) annotation(Placement(transformation(extent = {{20, -40},{40, -20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod3(width = 0.01, r = {0, -0.9,0.3}, animation = false) annotation(Placement(transformation(origin = { -40, -46}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod4(width = 0.01, r = {0,0.3,0.3}, animation = false) annotation(Placement(transformation(origin = { -40, -16}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
        Modelica.Mechanics.Translational.Components.Damper damper2(d = 3) annotation(Placement(transformation(extent = {{0, -76},{20, -96}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Forces.LineForceWithTwoMasses lineForceWithTwoMasses(L_a = 0.2, L_b = 0.2, cylinderLength_a = 0.2, cylinderLength_b = 1.2, massDiameterFaction = 2.2, m_a = m, m_b = m) annotation(Placement(transformation(extent = {{0, -50},{20, -70}}, rotation = 0)));

      equation 
        connect(jointUPS.bearing,damper1.flange_a) annotation(Line(points = {{6,30},{6,20},{0,20},{0,14}}, color = {0,191,0}));
        connect(jointUPS.axis,damper1.flange_b) annotation(Line(points = {{14,30},{14,20},{20,20},{20,14}}, color = {0,191,0}));
        connect(jointUPS.frame_ib,body2.frame_a) annotation(Line(points = {{18,30},{18,24},{32,24}}, color = {0,0,0}, thickness = 0.5));
        connect(world.frame_b,rod2.frame_a) annotation(Line(points = {{ -60,70},{ -40,70},{ -40,74}}, color = {0,0,0}, thickness = 0.5));
        connect(world.frame_b,rod1.frame_a) annotation(Line(points = {{ -60,70},{ -40,70},{ -40,64}}, color = {0,0,0}, thickness = 0.5));
        connect(rod2.frame_b,revolute1.frame_a) annotation(Line(points = {{ -40,94},{ -40,98},{ -28,98},{ -28,70},{ -20,70}}, color = {0,0,0}, thickness = 0.5));
        connect(revolute1.frame_b,bodyBox1.frame_a) annotation(Line(points = {{0,70},{20,70}}, color = {0,0,0}, thickness = 0.5));
        connect(bodyBox1.frame_b,jointUPS.frame_b) annotation(Line(points = {{40,70},{46,70},{46,40},{20,40}}, color = {0,0,0}, thickness = 0.5));
        connect(body1.frame_a,jointUPS.frame_ia) annotation(Line(points = {{ -14,24},{2,24},{2,30}}, color = {0,0,0}, thickness = 0.5));
        connect(rod1.frame_b,jointUPS.frame_a) annotation(Line(points = {{ -40,44},{ -40,40},{0,40}}, color = {0,0,0}, thickness = 0.5));
        connect(rod4.frame_b,revolute2.frame_a) annotation(Line(points = {{ -40, -6},{ -40, -2},{ -28, -2},{ -28, -30},{ -20, -30}}, color = {0,0,0}, thickness = 0.5));
        connect(revolute2.frame_b,bodyBox2.frame_a) annotation(Line(points = {{0, -30},{20, -30}}, color = {0,0,0}, thickness = 0.5));
        connect(world.frame_b,rod4.frame_a) annotation(Line(points = {{ -60,70},{ -52,70},{ -52, -32},{ -40, -32},{ -40, -26}}, color = {0,0,0}, thickness = 0.5));
        connect(rod3.frame_a,rod4.frame_a) annotation(Line(points = {{ -40, -36},{ -40, -26}}, color = {0,0,0}, thickness = 0.5));
        connect(lineForceWithTwoMasses.frame_a,rod3.frame_b) annotation(Line(points = {{0, -60},{ -40, -60},{ -40, -56}}, color = {0,0,0}, thickness = 0.5));
        connect(lineForceWithTwoMasses.frame_b,bodyBox2.frame_b) annotation(Line(points = {{20, -60},{54, -60},{54, -30},{40, -30}}, color = {0,0,0}, thickness = 0.5));
        connect(lineForceWithTwoMasses.flange_b,damper2.flange_b) annotation(Line(points = {{16, -71},{20, -71},{20, -86}}, color = {0,191,0}));
        connect(lineForceWithTwoMasses.flange_a,damper2.flange_a) annotation(Line(points = {{4, -71},{0, -71},{0, -86}}, color = {0,191,0}));
      end LineForceWithTwoMasses;
      model Torque
        extends Modelica.Icons.Example;
        import Modelica.Mechanics.MultiBody.*;
        inner World world annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}}, rotation = 0)));
        Joints.Revolute Revolute1(n = {0,1,0}) annotation(Placement(transformation(origin = { -20,30}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
        Parts.BodyCylinder body1(r = {1,0,0}) annotation(Placement(transformation(extent = {{0,40},{20,60}}, rotation = 0)));
        Modelica.Blocks.Sources.Constant Constant1[3](k = { -100,100,0}) annotation(Placement(transformation(origin = {50,90}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
        Parts.Fixed fixed1(r = {0,0.5,0}) annotation(Placement(transformation(extent = {{ -90,0},{ -70,20}}, rotation = 0)));
        Parts.BodyCylinder body2(r = {1,0,0}) annotation(Placement(transformation(extent = {{0, -30},{20, -10}}, rotation = 0)));
        Parts.Fixed fixed3(r = {0, -0.5,0}) annotation(Placement(transformation(extent = {{ -88, -70},{ -68, -50}}, rotation = 0)));
        Parts.FixedRotation fixedRotation(n = {0,0,1}, angle = 45) annotation(Placement(transformation(extent = {{ -8, -90},{12, -70}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Forces.Torque torque1(Nm_to_m = 120, resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_b) annotation(Placement(transformation(extent = {{62,40},{42,60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Forces.Torque torque2(Nm_to_m = 120, resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve) annotation(Placement(transformation(extent = {{60, -10},{40, -30}}, rotation = 0)));
        Joints.Revolute Revolute2(n = {0,1,0}) annotation(Placement(transformation(origin = { -20, -40}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Blocks.Sources.Constant Constant2[3](k = { -100,100,0}) annotation(Placement(transformation(origin = {70, -70}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
        Joints.Revolute Revolute3 annotation(Placement(transformation(extent = {{ -52,0},{ -32,20}}, rotation = 0)));
        Joints.Revolute Revolute4 annotation(Placement(transformation(extent = {{ -62, -70},{ -42, -50}}, rotation = 0)));
        Parts.Fixed fixed2(animation = false, r = {1.5,0.25,0}) annotation(Placement(transformation(extent = {{100,40},{80,60}}, rotation = 0)));
        Parts.Fixed fixed4(animation = false, r = {1.5, -0.25,0}) annotation(Placement(transformation(extent = {{100, -30},{80, -10}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(torque1.frame_b,body1.frame_b) annotation(Line(points = {{42,50},{20,50}}));
        connect(Revolute1.frame_b,body1.frame_a) annotation(Line(points = {{ -20,40},{ -20,50},{0,50}}));
        connect(Revolute2.frame_b,body2.frame_a) annotation(Line(points = {{ -20, -30},{ -20, -20},{0, -20}}));
        connect(torque2.frame_b,body2.frame_b) annotation(Line(points = {{40, -20},{20, -20}}));
        connect(Revolute3.frame_a,fixed1.frame_b) annotation(Line(points = {{ -52,10},{ -70,10}}));
        connect(Revolute3.frame_b,Revolute1.frame_a) annotation(Line(points = {{ -32,10},{ -20,10},{ -20,20}}));
        connect(fixedRotation.frame_b,torque2.frame_resolve) annotation(Line(points = {{12, -80},{46, -80},{46, -30}}));
        connect(fixed3.frame_b,Revolute4.frame_a) annotation(Line(points = {{ -68, -60},{ -62, -60}}));
        connect(Revolute4.frame_b,Revolute2.frame_a) annotation(Line(points = {{ -42, -60},{ -20, -60},{ -20, -50}}));
        connect(fixed3.frame_b,fixedRotation.frame_a) annotation(Line(points = {{ -68, -60},{ -64, -60},{ -64, -80},{ -8, -80}}));
        connect(Constant1.y,torque1.torque) annotation(Line(points = {{50,79},{50,72},{58,72},{58,62}}, color = {0,0,255}));
        connect(Constant2.y,torque2.torque) annotation(Line(points = {{70, -59},{70, -42},{56, -42},{56, -32}}, color = {0,0,255}));
        connect(torque1.frame_a,fixed2.frame_b) annotation(Line(points = {{62,50},{80,50}}, color = {0,0,0}, thickness = 0.5));
        connect(fixed4.frame_b,torque2.frame_a) annotation(Line(points = {{80, -20},{60, -20}}, color = {0,0,0}, thickness = 0.5));
      end Torque;
      model Torque2
        extends Modelica.Icons.Example;
        import Modelica.Mechanics.MultiBody.*;
        inner World world annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}}, rotation = 0)));
        Joints.Revolute Revolute1(n = {0,1,0}) annotation(Placement(transformation(origin = { -20,30}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -300},{100,100}}, grid = {1,1}), graphics));
        Parts.BodyCylinder body1(r = {1,0,0}) annotation(Placement(transformation(extent = {{0,40},{20,60}}, rotation = 0)));
        Modelica.Blocks.Sources.Constant Constant1[3](k = { -100,100,0}) annotation(Placement(transformation(origin = {50,90}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
        Parts.Fixed fixed1(r = {0,0.5,0}) annotation(Placement(transformation(extent = {{ -90,0},{ -70,20}}, rotation = 0)));
        Parts.BodyCylinder body2(r = {1,0,0}) annotation(Placement(transformation(extent = {{0, -30},{20, -10}}, rotation = 0)));
        Parts.Fixed fixed3(r = {0, -0.5,0}) annotation(Placement(transformation(extent = {{ -88, -70},{ -68, -50}}, rotation = 0)));
        Parts.FixedRotation fixedRotation(n = {0,0,1}, angle = 45) annotation(Placement(transformation(extent = {{ -8, -90},{12, -70}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Forces.Torque torque1(Nm_to_m = 120, resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_b) annotation(Placement(transformation(extent = {{62,40},{42,60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Forces.Torque torque2(Nm_to_m = 120, resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve) annotation(Placement(transformation(extent = {{60, -10},{40, -30}}, rotation = 0)));
        Joints.Revolute Revolute2(n = {0,1,0}) annotation(Placement(transformation(origin = { -20, -40}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Blocks.Sources.Constant Constant2[3](k = { -100,100,0}) annotation(Placement(transformation(origin = {70, -70}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
        Joints.Revolute Revolute3 annotation(Placement(transformation(extent = {{ -52,0},{ -32,20}}, rotation = 0)));
        Joints.Revolute Revolute4 annotation(Placement(transformation(extent = {{ -62, -70},{ -42, -50}}, rotation = 0)));
        Parts.Fixed fixed2(animation = false, r = {1.5,0.25,0}) annotation(Placement(transformation(extent = {{100,40},{80,60}}, rotation = 0)));
        Parts.Fixed fixed4(animation = false, r = {1.5, -0.25,0}) annotation(Placement(transformation(extent = {{100, -30},{80, -10}}, rotation = 0)));
        Joints.Revolute Revolute5(n = {0,1,0}) annotation(Placement(transformation(origin = { -20, -160}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Parts.BodyCylinder body3(r = {1,0,0}) annotation(Placement(transformation(extent = {{0, -150},{20, -130}}, rotation = 0)));
        Modelica.Blocks.Sources.Constant Constant3[6](k = {0,0,0, -100,100,0}) annotation(Placement(transformation(origin = {50, -100}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
        Parts.Fixed fixed5(r = {0,0.5,0}) annotation(Placement(transformation(extent = {{ -90, -190},{ -70, -170}}, rotation = 0)));
        Parts.BodyCylinder body4(r = {1,0,0}) annotation(Placement(transformation(extent = {{0, -220},{20, -200}}, rotation = 0)));
        Parts.Fixed fixed6(r = {0, -0.5,0}) annotation(Placement(transformation(extent = {{ -88, -260},{ -68, -240}}, rotation = 0)));
        Parts.FixedRotation fixedRotation1(n = {0,0,1}, angle = 45) annotation(Placement(transformation(extent = {{ -8, -280},{12, -260}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Forces.ForceAndTorque torque3(Nm_to_m = 120) annotation(Placement(transformation(extent = {{62, -150},{42, -130}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Forces.ForceAndTorque torque4(Nm_to_m = 120) annotation(Placement(transformation(extent = {{60, -200},{40, -220}}, rotation = 0)));
        Joints.Revolute Revolute6(n = {0,1,0}) annotation(Placement(transformation(origin = { -20, -230}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Blocks.Sources.Constant Constant4[6](k = {0,0,0, -100,100,0}) annotation(Placement(transformation(origin = {70, -260}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
        Joints.Revolute Revolute7 annotation(Placement(transformation(extent = {{ -52, -190},{ -32, -170}}, rotation = 0)));
        Joints.Revolute Revolute8 annotation(Placement(transformation(extent = {{ -62, -260},{ -42, -240}}, rotation = 0)));
        Parts.Fixed fixed7(animation = false, r = {1.5,0.25,0}) annotation(Placement(transformation(extent = {{100, -150},{80, -130}}, rotation = 0)));
        Parts.Fixed fixed8(animation = false, r = {1.5, -0.25,0}) annotation(Placement(transformation(extent = {{100, -220},{80, -200}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(torque1.frame_b,body1.frame_b) annotation(Line(points = {{42,50},{20,50}}));
        connect(Revolute1.frame_b,body1.frame_a) annotation(Line(points = {{ -20,40},{ -20,50},{0,50}}));
        connect(Revolute2.frame_b,body2.frame_a) annotation(Line(points = {{ -20, -30},{ -20, -20},{0, -20}}));
        connect(torque2.frame_b,body2.frame_b) annotation(Line(points = {{40, -20},{20, -20}}));
        connect(Revolute3.frame_a,fixed1.frame_b) annotation(Line(points = {{ -52,10},{ -70,10}}));
        connect(Revolute3.frame_b,Revolute1.frame_a) annotation(Line(points = {{ -32,10},{ -20,10},{ -20,20}}));
        connect(fixedRotation.frame_b,torque2.frame_resolve) annotation(Line(points = {{12, -80},{46, -80},{46, -30}}));
        connect(fixed3.frame_b,Revolute4.frame_a) annotation(Line(points = {{ -68, -60},{ -62, -60}}));
        connect(Revolute4.frame_b,Revolute2.frame_a) annotation(Line(points = {{ -42, -60},{ -20, -60},{ -20, -50}}));
        connect(fixed3.frame_b,fixedRotation.frame_a) annotation(Line(points = {{ -68, -60},{ -64, -60},{ -64, -80},{ -8, -80}}));
        connect(Constant1.y,torque1.torque) annotation(Line(points = {{50,79},{50,72},{58,72},{58,62}}, color = {0,0,255}));
        connect(Constant2.y,torque2.torque) annotation(Line(points = {{70, -59},{70, -42},{56, -42},{56, -32}}, color = {0,0,255}));
        connect(torque1.frame_a,fixed2.frame_b) annotation(Line(points = {{62,50},{80,50}}, color = {0,0,0}, thickness = 0.5));
        connect(fixed4.frame_b,torque2.frame_a) annotation(Line(points = {{80, -20},{60, -20}}, color = {0,0,0}, thickness = 0.5));
        connect(torque3.frame_b,body3.frame_b) annotation(Line(points = {{42, -140},{20, -140}}));
        connect(Revolute5.frame_b,body3.frame_a) annotation(Line(points = {{ -20, -150},{ -20, -140},{0, -140}}));
        connect(Revolute6.frame_b,body4.frame_a) annotation(Line(points = {{ -20, -220},{ -20, -210},{0, -210}}));
        connect(torque4.frame_b,body4.frame_b) annotation(Line(points = {{40, -210},{20, -210}}));
        connect(Revolute7.frame_a,fixed5.frame_b) annotation(Line(points = {{ -52, -180},{ -70, -180}}));
        connect(Revolute7.frame_b,Revolute5.frame_a) annotation(Line(points = {{ -32, -180},{ -20, -180},{ -20, -170}}));
        connect(fixedRotation1.frame_b,torque4.frame_resolve) annotation(Line(points = {{12, -270},{46, -270},{46, -220}}));
        connect(fixed6.frame_b,Revolute8.frame_a) annotation(Line(points = {{ -68, -250},{ -62, -250}}));
        connect(Revolute8.frame_b,Revolute6.frame_a) annotation(Line(points = {{ -42, -250},{ -20, -250},{ -20, -240}}));
        connect(fixed6.frame_b,fixedRotation1.frame_a) annotation(Line(points = {{ -68, -250},{ -64, -250},{ -64, -270},{ -8, -270}}));
        connect(Constant3.y,torque3.load) annotation(Line(points = {{50, -111},{50, -118},{58, -118},{58, -128}}, color = {0,0,255}));
        connect(Constant4.y,torque4.load) annotation(Line(points = {{70, -249},{70, -232},{56, -232},{56, -222}}, color = {0,0,255}));
        connect(torque3.frame_a,fixed7.frame_b) annotation(Line(points = {{62, -140},{80, -140}}, color = {0,0,0}, thickness = 0.5));
        connect(fixed8.frame_b,torque4.frame_a) annotation(Line(points = {{80, -210},{60, -210}}, color = {0,0,0}, thickness = 0.5));
      end Torque2;
      model WorldForces
        extends Modelica.Icons.Example;
        import Modelica.Mechanics.MultiBody.*;
        inner World world annotation(Placement(transformation(extent = {{ -100,20},{ -80,40}}, rotation = 0)));
        Joints.Revolute Revolute1 annotation(Placement(transformation(extent = {{ -40,60},{ -20,80}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
        Parts.BodyCylinder BodyCylinder1(r = {1,0,0}) annotation(Placement(transformation(extent = {{0,60},{20,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Forces.WorldForce worldForce(N_to_m = 600) annotation(Placement(transformation(extent = {{60,60},{40,80}}, rotation = 0)));
        Modelica.Blocks.Sources.Constant Constant1[3](k = {0,500,0}) annotation(Placement(transformation(extent = {{100,60},{80,80}}, rotation = 0)));
        Parts.Fixed fixed2(r = {0, -0.5,0}) annotation(Placement(transformation(extent = {{ -80, -20},{ -60,0}}, rotation = 0)));
        Joints.Revolute Revolute2(n = {0,1,0}) annotation(Placement(transformation(origin = { -30,10}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Parts.BodyCylinder BodyCylinder2(r = {1,0,0}) annotation(Placement(transformation(extent = {{0,20},{20,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Forces.WorldTorque worldTorque1(Nm_to_m = 200) annotation(Placement(transformation(extent = {{60,20},{40,40}}, rotation = 0)));
        Modelica.Blocks.Sources.Constant Constant2[3](k = {100,100,100}) annotation(Placement(transformation(extent = {{100,20},{80,40}}, rotation = 0)));
        Parts.Fixed fixed1(r = {0,0.5,0}) annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}}, rotation = 0)));
        Joints.Revolute Revolute3 annotation(Placement(transformation(extent = {{ -50, -100},{ -30, -80}}, rotation = 0)));
        Parts.BodyCylinder BodyCylinder3(r = {1,0,0}) annotation(Placement(transformation(extent = {{0, -60},{20, -40}}, rotation = 0)));
        Modelica.Blocks.Sources.Constant Constant4[6](k = {0,500,0,100,100,100}) annotation(Placement(transformation(extent = {{100, -60},{80, -40}}, rotation = 0)));
        Parts.Fixed fixed3(r = {0, -1,0}) annotation(Placement(transformation(extent = {{ -80, -100},{ -60, -80}}, rotation = 0)));
        Joints.Revolute Revolute4(n = {0,1,0}) annotation(Placement(transformation(origin = { -10, -70}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        ObsoleteModelica3.Mechanics.MultiBody.Forces.WorldForceAndTorque ForceTorque(N_to_m = 600, Nm_to_m = 200) annotation(Placement(transformation(extent = {{60, -60},{40, -40}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(BodyCylinder1.frame_a,Revolute1.frame_b) annotation(Line(points = {{0,70},{ -20,70}}));
        connect(worldForce.frame_b,BodyCylinder1.frame_b) annotation(Line(points = {{40,70},{20,70}}));
        connect(fixed2.frame_b,Revolute2.frame_a) annotation(Line(points = {{ -60, -10},{ -30, -10},{ -30,0}}));
        connect(BodyCylinder2.frame_a,Revolute2.frame_b) annotation(Line(points = {{0,30},{ -30,30},{ -30,20}}));
        connect(worldTorque1.frame_b,BodyCylinder2.frame_b) annotation(Line(points = {{40,30},{20,30}}));
        connect(fixed1.frame_b,Revolute1.frame_a) annotation(Line(points = {{ -60,70},{ -40,70}}));
        connect(fixed3.frame_b,Revolute3.frame_a) annotation(Line(points = {{ -60, -90},{ -50, -90}}));
        connect(Revolute3.frame_b,Revolute4.frame_a) annotation(Line(points = {{ -30, -90},{ -10, -90},{ -10, -80}}));
        connect(ForceTorque.frame_b,BodyCylinder3.frame_b) annotation(Line(points = {{40, -50},{20, -50}}));
        connect(Revolute4.frame_b,BodyCylinder3.frame_a) annotation(Line(points = {{ -10, -60},{ -10, -50},{0, -50}}));
        connect(Constant4.y,ForceTorque.load) annotation(Line(points = {{79, -50},{62, -50}}, color = {0,0,127}, smooth = Smooth.None));
        connect(Constant1.y,worldForce.force) annotation(Line(points = {{79,70},{62,70}}, color = {0,0,127}, smooth = Smooth.None));
        connect(Constant2.y,worldTorque1.torque) annotation(Line(points = {{79,30},{62,30}}, color = {0,0,127}, smooth = Smooth.None));
      end WorldForces;
    end Forces;
    package Joints "Test MultiBody.Joints"
      model Revolute
        inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) annotation(Placement(transformation(extent = {{ -60,20},{ -40,40}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow axis(length = 0.5, color = {0,0,0}, diameter = 0.02, n = {0,0,1}) annotation(Placement(transformation(extent = {{10,60},{30,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Revolute revolute(n = {0,0,1}, cylinderDiameter = 1 / 15, a(fixed = false), phi(fixed = true, start = 0.785398163397448), w(fixed = true)) annotation(Placement(transformation(extent = {{ -22,20},{ -2,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Body body(animation = false) annotation(Placement(transformation(extent = {{20,20},{40,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedFrame frame_a(color_x = {0,128,255}) annotation(Placement(transformation(origin = { -30, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedFrame frame_a1(color_x = {0,0,200}) annotation(Placement(transformation(origin = {10, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(world.frame_b,revolute.frame_a) annotation(Line(points = {{ -40,30},{ -22,30}}, color = {0,0,0}, thickness = 0.5));
        connect(revolute.frame_b,body.frame_a) annotation(Line(points = {{ -2,30},{20,30}}, color = {0,0,0}, thickness = 0.5));
        connect(revolute.frame_b,axis.frame_a) annotation(Line(points = {{ -2,30},{4,30},{4,70},{10,70}}, color = {0,0,0}, thickness = 0.5));
        connect(frame_a.frame_a,revolute.frame_a) annotation(Line(points = {{ -30,0},{ -30,30},{ -22,30}}, color = {0,0,0}, thickness = 0.5));
        connect(frame_a1.frame_a,revolute.frame_b) annotation(Line(points = {{10,0},{12,0},{12,30},{ -2,30}}, color = {0,0,0}, thickness = 0.5));
      end Revolute;
      model RevoluteInitialization
        inner Modelica.Mechanics.MultiBody.World world(gravityType = Modelica.Mechanics.MultiBody.Types.GravityTypes.UniformGravity) annotation(Placement(transformation(extent = {{ -80,0},{ -60,20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Revolute rev(n = {0,0,1}, useAxisFlange = true, a(fixed = true), phi(fixed = true), w(fixed = true)) annotation(Placement(transformation(extent = {{ -40,0},{ -20,20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Damper damper(d = 0.1) annotation(Placement(transformation(extent = {{ -40,40},{ -20,60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyShape body(r = {1,0,0}, r_CM = {0.5,0,0}, m = 1) annotation(Placement(transformation(extent = {{0,0},{20,20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Fixed fixed(r = {1,0.2,0}, width = 0.02) annotation(Placement(transformation(origin = {50,70}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        Modelica.Mechanics.MultiBody.Forces.Spring spring(s_unstretched = 0.1, c(fixed = false) = 100) annotation(Placement(transformation(origin = {50,30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(world.frame_b,rev.frame_a) annotation(Line(points = {{ -60,10},{ -40,10}}, color = {0,0,0}, thickness = 0.5));
        connect(damper.flange_b,rev.axis) annotation(Line(points = {{ -20,50},{ -16,50},{ -16,26},{ -30,26},{ -30,20}}, color = {0,0,0}));
        connect(rev.support,damper.flange_a) annotation(Line(points = {{ -36,20},{ -36,26},{ -48,26},{ -48,50},{ -40,50}}, color = {0,0,0}));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
        connect(rev.frame_b,body.frame_a) annotation(Line(points = {{ -20,10},{0,10}}, color = {0,0,0}, thickness = 0.5));
        connect(fixed.frame_b,spring.frame_a) annotation(Line(points = {{50,60},{50,40}}, color = {0,0,0}, thickness = 0.5));
        connect(body.frame_b,spring.frame_b) annotation(Line(points = {{20,10},{50,10},{50,20}}, color = {0,0,0}, thickness = 0.5));
      end RevoluteInitialization;
      model Prismatic
        inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) annotation(Placement(transformation(extent = {{ -60,20},{ -40,40}}, rotation = 0)));
        annotation(Diagram(graphics));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow axis(length = 0.5, color = {0,0,0}, diameter = 0.02, n = {1,1,0}) annotation(Placement(transformation(extent = {{10,60},{30,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Prismatic prismatic(n = {1,1,0}, a(fixed = false), s(fixed = true, start = 0.3), v(fixed = true)) annotation(Placement(transformation(extent = {{ -20,20},{0,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Body body(animation = false) annotation(Placement(transformation(extent = {{20,20},{40,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedFrame frame_a(color_x = {0,128,255}) annotation(Placement(transformation(origin = { -30, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedFrame frame_a1(color_x = {0,0,200}) annotation(Placement(transformation(origin = {10, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(world.frame_b,prismatic.frame_a) annotation(Line(points = {{ -40,30},{ -20,30}}, color = {0,0,0}, thickness = 0.5));
        connect(prismatic.frame_b,body.frame_a) annotation(Line(points = {{0,30},{20,30}}, color = {0,0,0}, thickness = 0.5));
        connect(prismatic.frame_b,axis.frame_a) annotation(Line(points = {{0,30},{4,30},{4,70},{10,70}}, color = {0,0,0}, thickness = 0.5));
        connect(frame_a.frame_a,prismatic.frame_a) annotation(Line(points = {{ -30,0},{ -30,30},{ -20,30}}, color = {0,0,0}, thickness = 0.5));
        connect(frame_a1.frame_a,prismatic.frame_b) annotation(Line(points = {{10,0},{12,0},{12,30},{0,30}}, color = {0,0,0}, thickness = 0.5));
      end Prismatic;
      model Cylindrical
        inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) annotation(Placement(transformation(extent = {{ -60,20},{ -40,40}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow axis(length = 0.5, color = {0,0,0}, diameter = 0.02, n = {0,0,1}) annotation(Placement(transformation(extent = {{10,60},{30,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Cylindrical cylindrical(n = {0,0,1}, cylinderDiameter = 1 / 15, a(fixed = false), phi(fixed = true, start = 0.785398163397448), s(fixed = true, start = 0.3), v(fixed = true), w(fixed = true), wd(fixed = false)) annotation(Placement(transformation(extent = {{ -22,20},{ -2,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Body body(animation = false) annotation(Placement(transformation(extent = {{20,20},{40,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedFrame frame_a(color_x = {0,128,255}) annotation(Placement(transformation(origin = { -30, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedFrame frame_a1(color_x = {0,0,200}) annotation(Placement(transformation(origin = {10, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(world.frame_b,cylindrical.frame_a) annotation(Line(points = {{ -40,30},{ -22,30}}, color = {0,0,0}, thickness = 0.5));
        connect(cylindrical.frame_b,body.frame_a) annotation(Line(points = {{ -2,30},{20,30}}, color = {0,0,0}, thickness = 0.5));
        connect(cylindrical.frame_b,axis.frame_a) annotation(Line(points = {{ -2,30},{4,30},{4,70},{10,70}}, color = {0,0,0}, thickness = 0.5));
        connect(frame_a.frame_a,cylindrical.frame_a) annotation(Line(points = {{ -30,0},{ -30,30},{ -22,30}}, color = {0,0,0}, thickness = 0.5));
        connect(frame_a1.frame_a,cylindrical.frame_b) annotation(Line(points = {{10,0},{12,0},{12,30},{ -2,30}}, color = {0,0,0}, thickness = 0.5));
      end Cylindrical;
      model Universal
        inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) annotation(Placement(transformation(extent = {{ -60,20},{ -40,40}}, rotation = 0)));
        annotation(Diagram(graphics));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow axis2(length = 0.5, color = {0,0,0}, diameter = 0.02, n = {0,1,0}, animation = false) annotation(Placement(transformation(extent = {{10,60},{30,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Universal universal(n_a = {0,0,1}, cylinderDiameter = 1 / 15, cylinderLength = 0.2, a_a(fixed = false), a_b(fixed = false), phi_a(fixed = true, start = 1.5707963267949), phi_b(fixed = true, start = 0.785398163397448), w_a(fixed = true), w_b(fixed = true)) annotation(Placement(transformation(extent = {{ -24,20},{ -4,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Body body(animation = false) annotation(Placement(transformation(extent = {{20,20},{40,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedFrame frame_a(color_x = {0,128,255}) annotation(Placement(transformation(origin = { -30, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedFrame frame_a1(color_x = {0,0,200}) annotation(Placement(transformation(origin = {10, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow axis1(length = 0.5, color = {0,0,0}, diameter = 0.02, n = {0,0,1}, animation = false) annotation(Placement(transformation(extent = {{ -34,60},{ -14,80}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(world.frame_b,universal.frame_a) annotation(Line(points = {{ -40,30},{ -24,30}}, color = {0,0,0}, thickness = 0.5));
        connect(universal.frame_b,body.frame_a) annotation(Line(points = {{ -4,30},{20,30}}, color = {0,0,0}, thickness = 0.5));
        connect(universal.frame_b,axis2.frame_a) annotation(Line(points = {{ -4,30},{4,30},{4,70},{10,70}}, color = {0,0,0}, thickness = 0.5));
        connect(frame_a.frame_a,universal.frame_a) annotation(Line(points = {{ -30,0},{ -30,30},{ -24,30}}, color = {0,0,0}, thickness = 0.5));
        connect(frame_a1.frame_a,universal.frame_b) annotation(Line(points = {{10,0},{12,0},{12,30},{ -4,30}}, color = {0,0,0}, thickness = 0.5));
        connect(universal.frame_a,axis1.frame_a) annotation(Line(points = {{ -24,30},{ -24,30},{ -24,56},{ -48,56},{ -48,70},{ -34,70}}, color = {0,0,0}, thickness = 0.5));
      end Universal;
      model Planar
        inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) annotation(Placement(transformation(extent = {{ -60,20},{ -40,40}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow axis2(length = 0.5, color = {0,0,0}, diameter = 0.02, n = {0,1,0}) annotation(Placement(transformation(extent = {{10,60},{30,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Planar planar(n = {0,1,0}, n_x = {0,0,1}, cylinderLength = 1 / 10, cylinderDiameter = 1 / 15, boxWidth = 0.4 * planar.cylinderDiameter, a_x(fixed = false), a_y(fixed = false), phi(fixed = true, start = 0.785398163397448), s_x(fixed = true, start = 0.5), s_y(fixed = true, start = 0.5), v_x(fixed = true), v_y(fixed = true), w(fixed = true), wd(fixed = false)) annotation(Placement(transformation(extent = {{ -22,20},{ -2,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Body body(animation = false) annotation(Placement(transformation(extent = {{20,20},{40,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedFrame frame_a(color_x = {0,128,255}) annotation(Placement(transformation(origin = { -30, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedFrame frame_a1(color_x = {0,0,200}) annotation(Placement(transformation(origin = {10, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow axis1(length = 0.5, color = {0,0,0}, diameter = 0.02, n = {0,0,1}, r_tail = {0,0,0.5}) annotation(Placement(transformation(extent = {{ -34,60},{ -14,80}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(world.frame_b,planar.frame_a) annotation(Line(points = {{ -40,30},{ -22,30}}, color = {0,0,0}, thickness = 0.5));
        connect(planar.frame_b,body.frame_a) annotation(Line(points = {{ -2,30},{20,30}}, color = {0,0,0}, thickness = 0.5));
        connect(planar.frame_b,axis2.frame_a) annotation(Line(points = {{ -2,30},{4,30},{4,70},{10,70}}, color = {0,0,0}, thickness = 0.5));
        connect(frame_a.frame_a,planar.frame_a) annotation(Line(points = {{ -30,0},{ -30,30},{ -22,30}}, color = {0,0,0}, thickness = 0.5));
        connect(frame_a1.frame_a,planar.frame_b) annotation(Line(points = {{10,0},{12,0},{12,30},{ -2,30}}, color = {0,0,0}, thickness = 0.5));
        connect(planar.frame_a,axis1.frame_a) annotation(Line(points = {{ -22,30},{ -24,30},{ -24,56},{ -48,56},{ -48,70},{ -34,70}}, color = {0,0,0}, thickness = 0.5));
      end Planar;
      model Spherical
        inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) annotation(Placement(transformation(extent = {{ -60,20},{ -40,40}}, rotation = 0)));
        annotation(Diagram(graphics));
        Modelica.Mechanics.MultiBody.Joints.Spherical spherical(angles_start = {0.785398163397448,0.785398163397448,0.785398163397448}, angles_fixed = true, w_rel_a_fixed = true, z_rel_a_fixed = false) annotation(Placement(transformation(extent = {{ -22,20},{ -2,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Body body(animation = false) annotation(Placement(transformation(extent = {{20,20},{40,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedFrame frame_a(color_x = {0,128,255}) annotation(Placement(transformation(origin = { -30, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedFrame frame_a1(color_x = {0,0,200}) annotation(Placement(transformation(origin = {10, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(world.frame_b,spherical.frame_a) annotation(Line(points = {{ -40,30},{ -22,30}}, color = {0,0,0}, thickness = 0.5));
        connect(spherical.frame_b,body.frame_a) annotation(Line(points = {{ -2,30},{20,30}}, color = {0,0,0}, thickness = 0.5));
        connect(frame_a.frame_a,spherical.frame_a) annotation(Line(points = {{ -30,0},{ -30,30},{ -22,30}}, color = {0,0,0}, thickness = 0.5));
        connect(frame_a1.frame_a,spherical.frame_b) annotation(Line(points = {{10,0},{12,0},{12,30},{ -2,30}}, color = {0,0,0}, thickness = 0.5));
      end Spherical;
      model FreeMotion
        inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) annotation(Placement(transformation(extent = {{ -60,20},{ -40,40}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
        Modelica.Mechanics.MultiBody.Joints.FreeMotion freeMotion(angles_start = {0.785398163397448,0.785398163397448,0.785398163397448}, a_rel_a(fixed = false), angles_fixed = true, r_rel_a(fixed = true, start = {0.5,0.4,0}), v_rel_a(fixed = true), w_rel_a_fixed = true, z_rel_a_fixed = false) annotation(Placement(transformation(extent = {{ -22,20},{ -2,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Body body(animation = false) annotation(Placement(transformation(extent = {{20,20},{40,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedFrame frame_a(color_x = {0,128,255}) annotation(Placement(transformation(origin = { -30, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedFrame frame_a1(color_x = {0,0,200}) annotation(Placement(transformation(origin = {10, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(world.frame_b,freeMotion.frame_a) annotation(Line(points = {{ -40,30},{ -22,30}}, color = {0,0,0}, thickness = 0.5));
        connect(freeMotion.frame_b,body.frame_a) annotation(Line(points = {{ -2,30},{20,30}}, color = {0,0,0}, thickness = 0.5));
        connect(frame_a.frame_a,freeMotion.frame_a) annotation(Line(points = {{ -30,0},{ -30,30},{ -22,30}}, color = {0,0,0}, thickness = 0.5));
        connect(frame_a1.frame_a,freeMotion.frame_b) annotation(Line(points = {{10,0},{12,0},{12,30},{ -2,30}}, color = {0,0,0}, thickness = 0.5));
      end FreeMotion;
      model SphericalSpherical
        inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) annotation(Placement(transformation(extent = {{ -60,20},{ -40,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.SphericalSpherical sphericalSpherical(rodLength = {1,0.2,0} * {1,0.2,0}, m = 1) annotation(Placement(transformation(extent = {{ -20,20},{0,40}}, rotation = 0)));
        annotation(Diagram(graphics));
        Modelica.Mechanics.MultiBody.Joints.Prismatic Prismatic1(animation = false) annotation(Placement(transformation(extent = {{20,20},{40,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Fixed Fixed1(r = {1,0.2,0}, animation = false) annotation(Placement(transformation(extent = {{80,20},{60,40}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(world.frame_b,sphericalSpherical.frame_a) annotation(Line(points = {{ -40,30},{ -20,30}}, color = {0,0,0}));
        connect(sphericalSpherical.frame_b,Prismatic1.frame_a) annotation(Line(points = {{0,30},{20,30}}, color = {0,0,0}));
        connect(Prismatic1.frame_b,Fixed1.frame_b) annotation(Line(points = {{40,30},{60,30}}, color = {0,0,0}));
      end SphericalSpherical;
      model JointUSP
        parameter Real d = 0.015;
        parameter Real rRod1_ia[3] = {1,0.3,0};
        parameter Real rRod2_ib[3] = { -0.5,0.2,0};
        inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) annotation(Placement(transformation(extent = {{ -60,20},{ -40,40}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
        Modelica.Mechanics.MultiBody.Parts.Fixed Fixed1(animation = false, r = {1.7,0.1,0}) annotation(Placement(transformation(extent = {{40,20},{20,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUSP jointUSP(rRod1_ia = rRod1_ia, rRod2_ib = rRod2_ib) annotation(Placement(transformation(extent = {{ -20,20},{0,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow n1_a(length = 0.5, color = {0,0,200}, n = {0,0,1}, diameter = d) annotation(Placement(transformation(extent = {{ -46,60},{ -26,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow n2_ia(length = 0.5, n = jointUSP.e2_ia, diameter = d, color = {0,128,0}) annotation(Placement(transformation(extent = {{ -20,60},{0,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow rRod1(diameter = d, r_tail = {0, -0.1,0}, n = rRod1_ia, color = {0,0,0}, length = Modelica.Math.Vectors.length(rRod1_ia)) annotation(Placement(transformation(extent = {{ -20, -20},{0,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow rRod2(diameter = d, r_tail = {0, -0.1,0}, n = rRod2_ib, color = {0,0,0}, length = Modelica.Math.Vectors.length(rRod2_ib)) annotation(Placement(transformation(extent = {{40, -20},{20,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow prism(diameter = d, r_tail = {0, -0.1,0}, n = { -1,0,0}, color = {255,0,0}, length = 0.2) annotation(Placement(transformation(extent = {{0, -60},{ -20, -40}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(world.frame_b,jointUSP.frame_a) annotation(Line(points = {{ -40,30},{ -20,30}}, color = {0,0,0}));
        connect(jointUSP.frame_b,Fixed1.frame_b) annotation(Line(points = {{0,30},{20,30}}, color = {0,0,0}));
        connect(jointUSP.frame_a,n1_a.frame_a) annotation(Line(points = {{ -20,30},{ -32,30},{ -32,54},{ -50,54},{ -50,70},{ -46,70}}, color = {0,0,0}));
        connect(n2_ia.frame_a,jointUSP.frame_ia) annotation(Line(points = {{ -20,70},{ -24,70},{ -24,46},{ -18,46},{ -18,40}}, color = {0,0,0}));
        connect(jointUSP.frame_ia,rRod1.frame_a) annotation(Line(points = {{ -18,40},{ -26,40},{ -26, -10},{ -20, -10}}, color = {0,0,0}));
        connect(rRod2.frame_a,jointUSP.frame_ib) annotation(Line(points = {{40, -10},{56, -10},{56,58},{ -2,58},{ -2,40}}, color = {0,0,0}));
        connect(prism.frame_a,jointUSP.frame_b) annotation(Line(points = {{0, -50},{10, -50},{10,30},{0,30}}, color = {0,0,0}));
      end JointUSP;
      model JointUPS
        parameter Real d = 0.015;
        parameter Real rRod1_ia[3] = {1,0.3,0};
        parameter Real rRod2_ib[3] = { -0.5,0.2,0};
        inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) annotation(Placement(transformation(extent = {{ -60,20},{ -40,40}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
        Modelica.Mechanics.MultiBody.Parts.Fixed Fixed1(animation = false, r = {1.7,0.1,0}) annotation(Placement(transformation(extent = {{40,20},{20,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUPS jointUPS(nAxis_ia = rRod1_ia) annotation(Placement(transformation(extent = {{ -20,20},{0,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow n1_a(length = 0.5, color = {0,0,200}, n = {0,0,1}, diameter = d) annotation(Placement(transformation(extent = {{ -46,60},{ -26,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow n2_ia(length = 0.5, n = jointUPS.e2_ia, diameter = d, color = {0,128,0}) annotation(Placement(transformation(extent = {{ -20,60},{0,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow rRod1(diameter = d, r_tail = {0, -0.1,0}, n = rRod1_ia, color = {0,0,0}, length = Modelica.Math.Vectors.length(rRod1_ia)) annotation(Placement(transformation(extent = {{ -20, -20},{0,0}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(world.frame_b,jointUPS.frame_a) annotation(Line(points = {{ -40,30},{ -20,30}}, color = {0,0,0}));
        connect(jointUPS.frame_b,Fixed1.frame_b) annotation(Line(points = {{0,30},{20,30}}, color = {0,0,0}));
        connect(jointUPS.frame_a,n1_a.frame_a) annotation(Line(points = {{ -20,30},{ -32,30},{ -32,54},{ -50,54},{ -50,70},{ -46,70}}, color = {0,0,0}));
        connect(n2_ia.frame_a,jointUPS.frame_ia) annotation(Line(points = {{ -20,70},{ -24,70},{ -24,46},{ -18,46},{ -18,40}}, color = {0,0,0}));
        connect(jointUPS.frame_ia,rRod1.frame_a) annotation(Line(points = {{ -18,40},{ -26,40},{ -26, -10},{ -20, -10}}, color = {0,0,0}));
      end JointUPS;
      model JointSSR
        parameter Real d = 0.015;
        parameter Real rRod1_ia[3] = {1,0.3,0};
        parameter Real L = Modelica.Math.Vectors.length(rRod1_ia);
        parameter Real rRod2_ib[3] = { -0.5,0.2,0};
        inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) annotation(Placement(transformation(extent = {{ -60,20},{ -40,40}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
        Modelica.Mechanics.MultiBody.Parts.Fixed Fixed1(animation = false, r = {1.5,0.1,0}) annotation(Placement(transformation(extent = {{40,20},{20,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Assemblies.JointSSR jointSSR(rRod2_ib = rRod2_ib, rod1Mass = 1, rod1Length = L, revoluteDiameter = 0.05) annotation(Placement(transformation(extent = {{ -20,20},{0,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow rRod2(diameter = d, n = rRod2_ib, color = {0,0,0}, length = Modelica.Math.Vectors.length(rRod2_ib), r_tail = {0.05,0.1,0}) annotation(Placement(transformation(extent = {{40, -20},{20,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow prism(diameter = d, color = {255,0,0}, length = 0.3, n = {0,0,1}) annotation(Placement(transformation(extent = {{0, -60},{ -20, -40}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(world.frame_b,jointSSR.frame_a) annotation(Line(points = {{ -40,30},{ -20,30}}, color = {0,0,0}));
        connect(jointSSR.frame_b,Fixed1.frame_b) annotation(Line(points = {{0,30},{20,30}}, color = {0,0,0}));
        connect(rRod2.frame_a,jointSSR.frame_ib) annotation(Line(points = {{40, -10},{56, -10},{56,58},{ -2,58},{ -2,40}}, color = {0,0,0}));
        connect(prism.frame_a,jointSSR.frame_b) annotation(Line(points = {{0, -50},{10, -50},{10,30},{0,30}}, color = {0,0,0}));
      end JointSSR;
      model JointUSR
        parameter Real d = 0.015;
        parameter Real rRod1_ia[3] = {1,0.3,0};
        parameter Real rRod2_ib[3] = { -0.5,0.2,0};
        inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) annotation(Placement(transformation(extent = {{ -60,20},{ -40,40}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
        Modelica.Mechanics.MultiBody.Parts.Fixed Fixed1(animation = false, r = {1.5,0.1,0}) annotation(Placement(transformation(extent = {{40,20},{20,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUSR jointUSR(rRod1_ia = rRod1_ia, rRod2_ib = rRod2_ib) annotation(Placement(transformation(extent = {{ -20,20},{0,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow n1_a(length = 0.5, color = {0,0,200}, n = {0,0,1}, diameter = d) annotation(Placement(transformation(extent = {{ -46,60},{ -26,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow n2_ia(length = 0.5, n = jointUSR.e2_ia, diameter = d, color = {0,128,0}) annotation(Placement(transformation(extent = {{ -20,60},{0,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow rRod1(diameter = d, r_tail = {0, -0.1,0}, n = rRod1_ia, color = {0,0,0}, length = Modelica.Math.Vectors.length(rRod1_ia)) annotation(Placement(transformation(extent = {{ -20, -20},{0,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow rRod2(diameter = d, r_tail = {0, -0.1,0}, n = rRod2_ib, color = {0,0,0}, length = Modelica.Math.Vectors.length(rRod2_ib)) annotation(Placement(transformation(extent = {{40, -20},{20,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow prism(diameter = d, color = {255,0,0}, n = {0,0,1}, length = 0.3) annotation(Placement(transformation(extent = {{0, -60},{ -20, -40}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(world.frame_b,jointUSR.frame_a) annotation(Line(points = {{ -40,30},{ -20,30}}, color = {0,0,0}));
        connect(jointUSR.frame_b,Fixed1.frame_b) annotation(Line(points = {{0,30},{20,30}}, color = {0,0,0}));
        connect(jointUSR.frame_a,n1_a.frame_a) annotation(Line(points = {{ -20,30},{ -32,30},{ -32,54},{ -50,54},{ -50,70},{ -46,70}}, color = {0,0,0}));
        connect(n2_ia.frame_a,jointUSR.frame_ia) annotation(Line(points = {{ -20,70},{ -24,70},{ -24,46},{ -18,46},{ -18,40}}, color = {0,0,0}));
        connect(jointUSR.frame_ia,rRod1.frame_a) annotation(Line(points = {{ -18,40},{ -26,40},{ -26, -10},{ -20, -10}}, color = {0,0,0}));
        connect(rRod2.frame_a,jointUSR.frame_ib) annotation(Line(points = {{40, -10},{56, -10},{56,58},{ -2,58},{ -2,40}}, color = {0,0,0}));
        connect(prism.frame_a,jointUSR.frame_b) annotation(Line(points = {{0, -50},{10, -50},{10,30},{0,30}}, color = {0,0,0}));
      end JointUSR;
      model JointRRR
        parameter Real d = 0.015;
        parameter Real rRod1_ia[3] = {1,0.3,0};
        parameter Real rRod2_ib[3] = { -0.5,0.2,0};
        inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) annotation(Placement(transformation(extent = {{ -60,20},{ -40,40}}, rotation = 0)));
        annotation(Diagram(graphics));
        Modelica.Mechanics.MultiBody.Parts.Fixed Fixed1(animation = false, r = {1.5,0.1,0}) annotation(Placement(transformation(extent = {{40,20},{20,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Assemblies.JointRRR jointRRR(rRod1_ia = rRod1_ia, rRod2_ib = rRod2_ib) annotation(Placement(transformation(extent = {{ -20,20},{0,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow n1_a(n = {0,0,1}, diameter = d, length = 0.4, color = {255,0,0}) annotation(Placement(transformation(extent = {{ -46,60},{ -26,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow rRod1(diameter = d, r_tail = {0, -0.1,0}, n = rRod1_ia, color = {0,0,0}, length = Modelica.Math.Vectors.length(rRod1_ia)) annotation(Placement(transformation(extent = {{ -20, -20},{0,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow rRod2(diameter = d, r_tail = {0, -0.1,0}, n = rRod2_ib, color = {0,0,0}, length = Modelica.Math.Vectors.length(rRod2_ib)) annotation(Placement(transformation(extent = {{40, -20},{20,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow prism(diameter = d, n = {0,0,1}, length = 0.4, color = {255,0,0}) annotation(Placement(transformation(extent = {{0, -60},{ -20, -40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow n1_a1(n = {0,0,1}, diameter = d, length = 0.4, color = {255,0,0}) annotation(Placement(transformation(extent = {{ -10,66},{10,86}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(world.frame_b,jointRRR.frame_a) annotation(Line(points = {{ -40,30},{ -20,30}}, color = {0,0,0}));
        connect(jointRRR.frame_b,Fixed1.frame_b) annotation(Line(points = {{0,30},{20,30}}, color = {0,0,0}));
        connect(jointRRR.frame_a,n1_a.frame_a) annotation(Line(points = {{ -20,30},{ -32,30},{ -32,54},{ -50,54},{ -50,70},{ -46,70}}, color = {0,0,0}));
        connect(jointRRR.frame_ia,rRod1.frame_a) annotation(Line(points = {{ -18,40},{ -26,40},{ -26, -10},{ -20, -10}}, color = {0,0,0}));
        connect(rRod2.frame_a,jointRRR.frame_ib) annotation(Line(points = {{40, -10},{56, -10},{56,58},{ -2,58},{ -2,40}}, color = {0,0,0}));
        connect(prism.frame_a,jointRRR.frame_b) annotation(Line(points = {{0, -50},{10, -50},{10,30},{0,30}}, color = {0,0,0}));
        connect(n1_a1.frame_a,jointRRR.frame_im) annotation(Line(points = {{ -10,76},{ -10,76},{ -10,40}}, color = {0,0,0}, thickness = 0.5));
      end JointRRR;
      model JointRRP
        parameter Real d = 0.015;
        parameter Real rRod1_ia[3] = {1,0.3,0};
        parameter Real rRod2_ib[3] = { -0.5,0.2,0};
        inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) annotation(Placement(transformation(extent = {{ -60,20},{ -40,40}}, rotation = 0)));
        annotation(Diagram(graphics));
        Modelica.Mechanics.MultiBody.Parts.Fixed Fixed1(animation = false, r = {1.7,0.1,0}) annotation(Placement(transformation(extent = {{40,20},{20,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Assemblies.JointRRP jointRRP(rRod1_ia = rRod1_ia, rRod2_ib = rRod2_ib) annotation(Placement(transformation(extent = {{ -20,20},{0,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow n1_a(n = {0,0,1}, diameter = d, length = 0.4, color = {255,0,0}) annotation(Placement(transformation(extent = {{ -46,60},{ -26,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow n2_ia(diameter = d, n = jointRRP.e_im, length = 0.4, color = {255,0,0}) annotation(Placement(transformation(extent = {{ -20,60},{0,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow rRod1(diameter = d, r_tail = {0, -0.1,0}, n = rRod1_ia, color = {0,0,0}, length = Modelica.Math.Vectors.length(rRod1_ia)) annotation(Placement(transformation(extent = {{ -20, -20},{0,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow rRod2(diameter = d, r_tail = {0, -0.1,0}, n = rRod2_ib, color = {0,0,0}, length = Modelica.Math.Vectors.length(rRod2_ib)) annotation(Placement(transformation(extent = {{40, -20},{20,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow prism(diameter = d, r_tail = {0, -0.1,0}, n = { -1,0,0}, color = {255,0,0}, length = 0.4) annotation(Placement(transformation(extent = {{0, -60},{ -20, -40}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(world.frame_b,jointRRP.frame_a) annotation(Line(points = {{ -40,30},{ -20,30}}, color = {0,0,0}));
        connect(jointRRP.frame_b,Fixed1.frame_b) annotation(Line(points = {{0,30},{20,30}}, color = {0,0,0}));
        connect(jointRRP.frame_a,n1_a.frame_a) annotation(Line(points = {{ -20,30},{ -32,30},{ -32,54},{ -50,54},{ -50,70},{ -46,70}}, color = {0,0,0}));
        connect(jointRRP.frame_ia,rRod1.frame_a) annotation(Line(points = {{ -18,40},{ -26,40},{ -26, -10},{ -20, -10}}, color = {0,0,0}));
        connect(rRod2.frame_a,jointRRP.frame_ib) annotation(Line(points = {{40, -10},{56, -10},{56,58},{ -2,58},{ -2,40}}, color = {0,0,0}));
        connect(prism.frame_a,jointRRP.frame_b) annotation(Line(points = {{0, -50},{10, -50},{10,30},{0,30}}, color = {0,0,0}));
        connect(jointRRP.frame_im,n2_ia.frame_a) annotation(Line(points = {{ -10,40},{ -10,55.5},{ -20,55.5},{ -20,70}}, color = {0,0,0}, thickness = 0.5));
      end JointRRP;
      model JointSSP
        parameter Real d = 0.015;
        parameter Real rRod1_ia[3] = {1,0.3,0};
        parameter Real L = Modelica.Math.Vectors.length(rRod1_ia);
        parameter Real rRod2_ib[3] = { -0.52,0.2,0};
        inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) annotation(Placement(transformation(extent = {{ -60,20},{ -40,40}}, rotation = 0)));
        annotation(Diagram(graphics));
        Modelica.Mechanics.MultiBody.Parts.Fixed Fixed1(animation = false, r = {1.5,0.1,0}) annotation(Placement(transformation(extent = {{40,20},{20,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Assemblies.JointSSP jointSSP(rRod2_ib = rRod2_ib, rod1Mass = 1, rod1Length = L, n_b = {0,0,1}, s_guess =  -10) annotation(Placement(transformation(extent = {{ -20,20},{0,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow rRod2(diameter = d, n = rRod2_ib, color = {0,0,0}, length = Modelica.Math.Vectors.length(rRod2_ib), r_tail = {0.05,0.1,0}) annotation(Placement(transformation(extent = {{40, -20},{20,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow prism(diameter = d, color = {255,0,0}, length = 0.3, n = {0,0,1}) annotation(Placement(transformation(extent = {{0, -60},{ -20, -40}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(world.frame_b,jointSSP.frame_a) annotation(Line(points = {{ -40,30},{ -20,30}}, color = {0,0,0}));
        connect(jointSSP.frame_b,Fixed1.frame_b) annotation(Line(points = {{0,30},{20,30}}, color = {0,0,0}));
        connect(rRod2.frame_a,jointSSP.frame_ib) annotation(Line(points = {{40, -10},{56, -10},{56,58},{ -2,58},{ -2,40}}, color = {0,0,0}));
        connect(prism.frame_a,jointSSP.frame_b) annotation(Line(points = {{0, -50},{10, -50},{10,30},{0,30}}, color = {0,0,0}));
      end JointSSP;
    end Joints;
    package Parts "Test MultiBody.Parts"
      model FixedTranslation
        import SI = Modelica.SIunits;
        import Modelica.Mechanics.MultiBody.*;
        extends Modelica.Icons.Example;
        parameter SI.Length length = 0.5 "Length of axes of frames";
        parameter SI.Diameter diameter = length / 30 "Diameter of frame axes";
        annotation(experiment(StopTime = 1), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
        Modelica.Mechanics.MultiBody.Frames.Orientation R;
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation(r = {1,0.5,0}) annotation(Placement(transformation(extent = {{ -40,40},{ -20,60}}, rotation = 0)));
        Visualizers.FixedFrame frameT1(length = length, diameter = diameter) annotation(Placement(transformation(extent = {{ -60,60},{ -80,80}}, rotation = 0)));
        Visualizers.FixedFrame frameT2(length = length, diameter = diameter) annotation(Placement(transformation(extent = {{0,40},{20,60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Fixed fixed1(animation = false) annotation(Placement(transformation(extent = {{ -80,20},{ -60,40}}, rotation = 0)));
        inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, gravityType = 0) annotation(Placement(transformation(extent = {{20,0},{40,20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedArrow arrow(n = {1,0.5,0}, color = {0,0,255}, r_tail = {0.03,0.1,0}, length = 1.05, diameter = 0.02) annotation(Placement(transformation(extent = {{ -40,20},{ -20,40}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        R = Modelica.Mechanics.MultiBody.Frames.nullRotation();
        connect(frameT1.frame_a,fixedTranslation.frame_a) annotation(Line(points = {{ -60,70},{ -50,70},{ -50,50},{ -40,50}}));
        connect(fixed1.frame_b,fixedTranslation.frame_a) annotation(Line(points = {{ -60,30},{ -50,30},{ -50,50},{ -40,50}}));
        connect(fixedTranslation.frame_b,frameT2.frame_a) annotation(Line(points = {{ -20,50},{0,50}}));
        connect(arrow.frame_a,fixed1.frame_b) annotation(Line(points = {{ -40,30},{ -60,30}}, color = {0,0,0}, thickness = 0.5));
      end FixedTranslation;
      encapsulated model BodyShape "Simple spring/damper/mass system"
        import Modelica.Icons;
        import Modelica.Mechanics.MultiBody;
        import Modelica.Mechanics.Rotational;
        import Modelica;
        extends Icons.Example;
        parameter Boolean animation = true "= true, if animation shall be enabled";
        annotation(experiment(StopTime = 10), Window(x = 0.29, y = 0, width = 0.71, height = 0.85), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
This is the same example as \"FreeBody\". The only difference
is that with the <b>FreeMotion</b> joint explicitly 6 degrees of
freedom are introduced. The advantage is that <b>no non-linear system
of equations</b> occurs any more, because the FreeMotion joint is
described by a set of <b>non-redundant</b> variables. The disadvantage is
that there is an orientation for which this description is singular
(see description of FreeMotion joint). In technical systems it is
often possible to use the FreeMotion joint such that the singularity
</p>
</html>"));
        inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -60,40},{ -40,60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation bar2(r = {0.8,0,0}, animation = false) annotation(Placement(transformation(extent = {{0,40},{20,60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Forces.Spring spring1(width = 0.1, coilWidth = 0.005, numberOfWindings = 5, c = 20, s_unstretched = 0) annotation(Placement(transformation(origin = { -24,26}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        Modelica.Mechanics.MultiBody.Parts.BodyShape body(m = 1, I_11 = 1, I_22 = 1, I_33 = 1, r = {0.4,0,0}, r_CM = {0.2,0,0}, width = 0.05) annotation(Placement(transformation(extent = {{0, -20},{20,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Forces.Spring spring2(c = 20, s_unstretched = 0, width = 0.1, coilWidth = 0.005, numberOfWindings = 5) annotation(Placement(transformation(origin = {42,26}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        Modelica.Mechanics.MultiBody.Joints.FreeMotion freeMotion(arrowDiameter = 0.04, r_rel_a(start = {0.2, -0.5,0.1})) annotation(Placement(transformation(extent = {{ -30, -40},{ -10, -20}}, rotation = 0)));

      equation 
        connect(bar2.frame_a,world.frame_b) annotation(Line(points = {{0,50},{ -40,50}}, color = {0,0,0}, thickness = 0.5));
        connect(spring1.frame_b,body.frame_a) annotation(Line(points = {{ -24,16},{ -24, -10},{0, -10}}, color = {0,0,0}, thickness = 0.5));
        connect(spring2.frame_b,body.frame_b) annotation(Line(points = {{42,16},{20, -10}}, color = {0,0,0}, thickness = 0.5));
        connect(bar2.frame_b,spring2.frame_a) annotation(Line(points = {{20,50},{42,50},{42,36}}, color = {0,0,0}, thickness = 0.5));
        connect(spring1.frame_a,world.frame_b) annotation(Line(points = {{ -24,36},{ -24,50},{ -40,50}}, color = {0,0,0}, thickness = 0.5));
        connect(world.frame_b,freeMotion.frame_a) annotation(Line(points = {{ -40,50},{ -34,50},{ -34, -30},{ -30, -30}}, color = {0,0,0}, thickness = 0.5));
        connect(freeMotion.frame_b,body.frame_a) annotation(Line(points = {{ -10, -30},{ -6, -30},{ -6, -10},{0, -10}}, color = {0,0,0}, thickness = 0.5));
      end BodyShape;
      model PointMass "Test Modelica.Mechanics.MultiBody.Parts.PointMass"
        extends Modelica.Icons.Example;
        inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -60,20},{ -40,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation(r = {1,0,0}) annotation(Placement(transformation(extent = {{20,20},{40,40}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1.1));
        Modelica.Mechanics.MultiBody.Parts.PointMass pointMass(m = 5) annotation(Placement(transformation(extent = {{60,20},{80,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Revolute rev(a(fixed = false), phi(fixed = true), w(fixed = true)) annotation(Placement(transformation(extent = {{ -20,20},{0,40}}, rotation = 0)));
        annotation(experiment(StopTime = 1));

      equation 
        connect(fixedTranslation.frame_b,pointMass.frame_a) annotation(Line(points = {{40,30},{70,30}}, color = {95,95,95}, thickness = 0.5));
        connect(world.frame_b,rev.frame_a) annotation(Line(points = {{ -40,30},{ -20,30}}, color = {95,95,95}, thickness = 0.5));
        connect(rev.frame_b,fixedTranslation.frame_a) annotation(Line(points = {{0,30},{20,30}}, color = {95,95,95}, thickness = 0.5));
      end PointMass;
      model Bodies
        import SI = Modelica.SIunits;
        extends Modelica.Icons.Example;
        Modelica.Mechanics.MultiBody.Parts.Body body2(m = 1, I_11 = 1, I_22 = 1, I_33 = 1, r_0(start = {0.8,0.2,0})) annotation(Placement(transformation(extent = {{ -20,20},{0,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyShape bodyShape(m = 1, I_11 = 1, I_22 = 1, I_33 = 1, r_CM = {0.3,0,0}, r = {0.6,0,0}, width = 0.05) annotation(Placement(transformation(extent = {{ -20, -20},{0,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyBox BodyBox1(r = {1,0,0}) annotation(Placement(transformation(extent = {{ -20, -60},{0, -40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyCylinder bodyCylinder(r = {1,0,0}, innerDiameter = bodyCylinder.diameter / 2) annotation(Placement(transformation(extent = {{ -20, -100},{0, -80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Fixed Fixed3(animation = false, r = {0, -0.3,0}) annotation(Placement(transformation(extent = {{ -80, -20},{ -60,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Fixed Fixed4(animation = false, r = {0, -0.6,0}) annotation(Placement(transformation(extent = {{ -80, -60},{ -60, -40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Fixed Fixed5(animation = false, r = {0, -0.8,0}) annotation(Placement(transformation(extent = {{ -80, -100},{ -60, -80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Body body1(m = 1, I_11 = 1, I_22 = 1, I_33 = 1, r_CM = {0.3,0,0}, r_0(start = {0.8,0.2,0})) annotation(Placement(transformation(extent = {{ -20,60},{0,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Fixed Fixed1(animation = false, r = {0,0.2,0}) annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Fixed Fixed2(animation = false, r = {0,0,0}) annotation(Placement(transformation(extent = {{ -80,20},{ -60,40}}, rotation = 0)));
        inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) annotation(Placement(transformation(extent = {{40,0},{60,20}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
        connect(Fixed3.frame_b,bodyShape.frame_a) annotation(Line(points = {{ -60, -10},{ -20, -10}}));
        connect(Fixed4.frame_b,BodyBox1.frame_a) annotation(Line(points = {{ -60, -50},{ -20, -50}}));
        connect(Fixed5.frame_b,bodyCylinder.frame_a) annotation(Line(points = {{ -60, -90},{ -20, -90}}));
        connect(Fixed1.frame_b,body1.frame_a) annotation(Line(points = {{ -60,70},{ -20,70}}));
        connect(Fixed2.frame_b,body2.frame_a) annotation(Line(points = {{ -60,30},{ -20,30}}));
      end Bodies;
      package Rotor1D
        model Rotor1D_1
          annotation(Diagram(graphics), experiment(StopTime = 5, Tolerance = 1e-08));
          inner Modelica.Mechanics.MultiBody.World world(driveTrainMechanics3D = true) annotation(Placement(transformation(extent = {{ -100,20},{ -80,40}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Joints.Spherical spherical1(angles_fixed = true, w_rel_a_fixed = true, z_rel_a_fixed = false) annotation(Placement(transformation(extent = {{ -70,20},{ -50,40}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Parts.BodyCylinder bodyCylinder1(r = {0.25,0,0}, diameter = 0.05) annotation(Placement(transformation(extent = {{ -44,20},{ -24,40}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Parts.FixedRotation fixedRotation1(angle = 45, n = {0,1,0}) annotation(Placement(transformation(extent = {{ -16,20},{4,40}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Joints.Revolute revolute(n = {1,0,0}, a(fixed = false), phi(fixed = true), w(fixed = true, start = 8.72664625997165)) annotation(Placement(transformation(extent = {{14,20},{34,40}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation(r = { -0.1,0,0}) annotation(Placement(transformation(extent = {{42,20},{62,40}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Parts.BodyCylinder bodyCylinder2(diameter = 0.1, r = {0.2,0,0}) annotation(Placement(transformation(extent = {{70,20},{90,40}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Parts.Fixed fixed annotation(Placement(transformation(extent = {{ -100, -40},{ -80, -20}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Joints.Spherical spherical2(angles_fixed = true, w_rel_a_fixed = true, z_rel_a_fixed = false) annotation(Placement(transformation(extent = {{ -70, -40},{ -50, -20}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Parts.BodyCylinder bodyCylinder3(r = {0.25,0,0}, diameter = 0.05, color = {0,128,0}) annotation(Placement(transformation(extent = {{ -44, -40},{ -24, -20}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Parts.FixedRotation fixedRotation2(n = {0,1,0}, angle = 45) annotation(Placement(transformation(extent = {{ -16, -40},{4, -20}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation1(r = { -0.1,0,0}) annotation(Placement(transformation(extent = {{24, -40},{44, -20}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Parts.BodyCylinder bodyCylinder4(diameter = 0.1, r = {0.2,0,0}, color = {0,128,0}) annotation(Placement(transformation(extent = {{56, -40},{76, -20}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Parts.Rotor1D rotor1D(J = bodyCylinder4.I[1,1], n = {1,0,0}, a(fixed = false), phi(fixed = true), w(fixed = true, start = 8.72664625997165)) annotation(Placement(transformation(extent = {{28, -70},{48, -50}}, rotation = 0)));

        equation 
          connect(world.frame_b,spherical1.frame_a) annotation(Line(points = {{ -80,30},{ -70,30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(spherical1.frame_b,bodyCylinder1.frame_a) annotation(Line(points = {{ -50,30},{ -44,30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(bodyCylinder1.frame_b,fixedRotation1.frame_a) annotation(Line(points = {{ -24,30},{ -16,30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(fixedRotation1.frame_b,revolute.frame_a) annotation(Line(points = {{4,30},{14,30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(revolute.frame_b,fixedTranslation.frame_a) annotation(Line(points = {{34,30},{42,30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(fixedTranslation.frame_b,bodyCylinder2.frame_a) annotation(Line(points = {{62,30},{70,30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(spherical2.frame_b,bodyCylinder3.frame_a) annotation(Line(points = {{ -50, -30},{ -44, -30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(bodyCylinder3.frame_b,fixedRotation2.frame_a) annotation(Line(points = {{ -24, -30},{ -16, -30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(fixed.frame_b,spherical2.frame_a) annotation(Line(points = {{ -80, -30},{ -70, -30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(fixedTranslation1.frame_b,bodyCylinder4.frame_a) annotation(Line(points = {{44, -30},{56, -30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(fixedRotation2.frame_b,fixedTranslation1.frame_a) annotation(Line(points = {{4, -30},{24, -30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(rotor1D.frame_a,fixedRotation2.frame_b) annotation(Line(points = {{38, -70},{38, -76},{10, -76},{10, -30},{4, -30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        end Rotor1D_1;
        model Rotor1D_2
          Modelica.Mechanics.MultiBody.Parts.BodyShape bodyCylinder(r = {0.5,0,0}, m = 0, I_11 = 2, I_22 = 0, I_33 = 0, shapeType = "cylinder", width = 0.1, animateSphere = false, r_shape = {0.1,0,0}) annotation(Placement(transformation(extent = {{ -12,20},{8,40}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Joints.Revolute revolute(n = {1,0,0}, a(fixed = false), phi(fixed = true), w(fixed = true)) annotation(Placement(transformation(extent = {{ -46,20},{ -26,40}}, rotation = 0)));
          inner Modelica.Mechanics.MultiBody.World world(g = 0, driveTrainMechanics3D = true) annotation(Placement(transformation(extent = {{ -80,20},{ -60,40}}, rotation = 0)));
          annotation(Diagram(graphics), experiment(StopTime = 1.1));
          Modelica.Mechanics.MultiBody.Forces.Torque torque(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve) annotation(Placement(transformation(extent = {{ -12,50},{8,70}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sine[3](amplitude = {1,0,0}) annotation(Placement(transformation(extent = {{ -100,80},{ -80,100}}, rotation = 0)));
          inner Modelica.Mechanics.MultiBody.Parts.Fixed fixed annotation(Placement(transformation(extent = {{ -82, -70},{ -62, -50}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Parts.Rotor1D rotor1D(J = 2) annotation(Placement(transformation(extent = {{ -20, -20},{0,0}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Sources.Torque torque1(useSupport = true) annotation(Placement(transformation(extent = {{ -52, -20},{ -32,0}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Parts.Mounting1D mounting1D annotation(Placement(transformation(extent = {{ -66, -40},{ -46, -20}}, rotation = 0)));

        equation 
          connect(world.frame_b,revolute.frame_a) annotation(Line(points = {{ -60,30},{ -46,30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(revolute.frame_b,bodyCylinder.frame_a) annotation(Line(points = {{ -26,30},{ -12,30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(torque.frame_b,bodyCylinder.frame_b) annotation(Line(points = {{8,60},{18,60},{18,30},{8,30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(torque.frame_resolve,world.frame_b) annotation(Line(points = {{2,70},{2,84},{ -50,84},{ -50,30},{ -60,30}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(torque.frame_a,world.frame_b) annotation(Line(points = {{ -12,60},{ -50,60},{ -50,30},{ -60,30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(sine.y,torque.torque) annotation(Line(points = {{ -79,90},{ -8,90},{ -8,72}}, color = {0,0,127}, smooth = Smooth.None));
          connect(fixed.frame_b,rotor1D.frame_a) annotation(Line(points = {{ -62, -60},{ -10, -60},{ -10, -20}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(torque1.flange,rotor1D.flange_a) annotation(Line(points = {{ -32, -10},{ -20, -10}}, color = {0,0,0}, smooth = Smooth.None));
          connect(mounting1D.flange_b,torque1.support) annotation(Line(points = {{ -46, -30},{ -42, -30},{ -42, -20}}, color = {0,0,0}, smooth = Smooth.None));
          connect(mounting1D.frame_a,fixed.frame_b) annotation(Line(points = {{ -56, -40},{ -56, -60},{ -62, -60}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(sine[1].y,torque1.tau) annotation(Line(points = {{ -79,90},{ -70,90},{ -70,60},{ -92,60},{ -92, -10},{ -54, -10}}, color = {0,0,127}, smooth = Smooth.None));
        end Rotor1D_2;
        model Rotor1D_3
          Modelica.Mechanics.MultiBody.Parts.BodyShape bodyCylinder(r = {0.5,0,0}, m = 0, I_11 = 2, I_22 = 0, I_33 = 0, shapeType = "cylinder", width = 0.1, animateSphere = false, r_shape = {0.1,0,0}) annotation(Placement(transformation(extent = {{50,0},{70,20}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Joints.Revolute revolute(n = {1,0,0}, a(fixed = false), phi(fixed = true), w(fixed = true)) annotation(Placement(transformation(extent = {{16,0},{36,20}}, rotation = 0)));
          inner Modelica.Mechanics.MultiBody.World world(g = 0, driveTrainMechanics3D = true) annotation(Placement(transformation(extent = {{ -84,0},{ -64,20}}, rotation = 0)));
          annotation(Diagram(graphics), experiment(StopTime = 1.1));
          Modelica.Mechanics.MultiBody.Forces.Torque torque(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve) annotation(Placement(transformation(extent = {{50,30},{70,50}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sine1[3](amplitude = {1,0,0}) annotation(Placement(transformation(extent = {{16,60},{36,80}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Parts.Rotor1D rotor1D(J = 2) annotation(Placement(transformation(extent = {{50, -46},{70, -26}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Sources.Torque torque1(useSupport = true) annotation(Placement(transformation(extent = {{18, -46},{38, -26}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Parts.Mounting1D mounting1D annotation(Placement(transformation(extent = {{4, -66},{24, -46}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Joints.Revolute r1(n = {0,1,0}, useAxisFlange = true) annotation(Placement(transformation(extent = {{ -32,0},{ -12,20}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Sources.Position position1(useSupport = true) annotation(Placement(transformation(extent = {{ -46,50},{ -26,70}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sine2(amplitude = 2) annotation(Placement(transformation(extent = {{ -100,50},{ -80,70}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Parts.Mounting1D mounting1D1 annotation(Placement(transformation(extent = {{ -60,24},{ -40,44}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Joints.Revolute r2(n = {0,1,0}, useAxisFlange = true) annotation(Placement(transformation(extent = {{ -38, -90},{ -18, -70}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Sources.Position position2(useSupport = true) annotation(Placement(transformation(extent = {{ -52, -40},{ -32, -20}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Parts.Mounting1D mounting1D2 annotation(Placement(transformation(extent = {{ -66, -66},{ -46, -46}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Parts.Fixed fixed annotation(Placement(transformation(extent = {{ -86, -90},{ -66, -70}}, rotation = 0)));

        equation 
          connect(revolute.frame_b,bodyCylinder.frame_a) annotation(Line(points = {{36,10},{50,10}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(torque.frame_b,bodyCylinder.frame_b) annotation(Line(points = {{70,40},{80,40},{80,10},{70,10}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(sine1.y,torque.torque) annotation(Line(points = {{37,70},{54,70},{54,52}}, color = {0,0,127}, smooth = Smooth.None));
          connect(torque1.flange,rotor1D.flange_a) annotation(Line(points = {{38, -36},{50, -36}}, color = {0,0,0}, smooth = Smooth.None));
          connect(mounting1D.flange_b,torque1.support) annotation(Line(points = {{24, -56},{28, -56},{28, -46}}, color = {0,0,0}, smooth = Smooth.None));
          connect(r1.frame_a,world.frame_b) annotation(Line(points = {{ -32,10},{ -64,10}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(position1.flange,r1.axis) annotation(Line(points = {{ -26,60},{ -22,60},{ -22,20}}, color = {0,0,0}, smooth = Smooth.None));
          connect(position1.support,mounting1D1.flange_b) annotation(Line(points = {{ -36,50},{ -36,34},{ -40,34}}, color = {0,0,0}, smooth = Smooth.None));
          connect(mounting1D1.frame_a,world.frame_b) annotation(Line(points = {{ -50,24},{ -50,10},{ -64,10}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(sine2.y,position1.phi_ref) annotation(Line(points = {{ -79,60},{ -48,60}}, color = {0,0,127}, smooth = Smooth.None));
          connect(r1.frame_b,revolute.frame_a) annotation(Line(points = {{ -12,10},{16,10}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(torque.frame_resolve,revolute.frame_a) annotation(Line(points = {{64,50},{8,50},{8,10},{16,10}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(torque.frame_a,revolute.frame_a) annotation(Line(points = {{50,40},{8,40},{8,10},{16,10}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(position2.flange,r2.axis) annotation(Line(points = {{ -32, -30},{ -28, -30},{ -28, -70}}, color = {0,0,0}, smooth = Smooth.None));
          connect(position2.support,mounting1D2.flange_b) annotation(Line(points = {{ -42, -40},{ -42, -56},{ -46, -56}}, color = {0,0,0}, smooth = Smooth.None));
          connect(fixed.frame_b,r2.frame_a) annotation(Line(points = {{ -66, -80},{ -38, -80}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(fixed.frame_b,mounting1D2.frame_a) annotation(Line(points = {{ -66, -80},{ -56, -80},{ -56, -66}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(sine2.y,position2.phi_ref) annotation(Line(points = {{ -79,60},{ -70,60},{ -70,36},{ -92,36},{ -92, -30},{ -54, -30}}, color = {0,0,127}, smooth = Smooth.None));
          connect(r2.frame_b,rotor1D.frame_a) annotation(Line(points = {{ -18, -80},{60, -80},{60, -46}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(r2.frame_b,mounting1D.frame_a) annotation(Line(points = {{ -18, -80},{14, -80},{14, -66}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(sine1[1].y,torque1.tau) annotation(Line(points = {{37,70},{92,70},{92, -20},{0, -20},{0, -36},{16, -36}}, color = {0,0,127}, smooth = Smooth.None));
        end Rotor1D_3;
        model GearConstraint
          Modelica.Mechanics.MultiBody.Joints.GearConstraint gearConstraint(ratio = 10) annotation(Placement(transformation(extent = {{34,40},{54,60}}, rotation = 0)));
          inner Modelica.Mechanics.MultiBody.World world(g = 0, driveTrainMechanics3D = true) annotation(Placement(transformation(extent = {{ -62,10},{ -42,30}}, rotation = 0)));
          annotation(Diagram(graphics), experiment(StopTime = 5), experimentSetupOutput);
          Modelica.Mechanics.MultiBody.Parts.BodyCylinder cyl1(diameter = 0.1, color = {0,128,0}, r = {0.4,0,0}) annotation(Placement(transformation(extent = {{2,40},{22,60}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Parts.BodyCylinder cyl2(r = {0.4,0,0}, diameter = 0.2) annotation(Placement(transformation(extent = {{70,40},{90,60}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Forces.Torque torque1(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_b) annotation(Placement(transformation(extent = {{ -26,40},{ -6,60}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sine[3](amplitude = {2,0,0}) annotation(Placement(transformation(extent = {{ -100,60},{ -80,80}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Parts.Fixed fixed annotation(Placement(transformation(extent = {{ -48, -90},{ -28, -70}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Components.Inertia inertia1(J = cyl1.I[1,1], a(fixed = false), phi(fixed = true), w(fixed = true)) annotation(Placement(transformation(extent = {{ -20, -40},{0, -20}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Components.IdealGear idealGear(ratio = 10, useSupport = true) annotation(Placement(transformation(extent = {{12, -40},{32, -20}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Components.Inertia inertia2(J = cyl2.I[1,1]) annotation(Placement(transformation(extent = {{44, -40},{64, -20}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Sources.Torque torque2(useSupport = true) annotation(Placement(transformation(extent = {{ -48, -40},{ -28, -20}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Parts.Mounting1D mounting1D annotation(Placement(transformation(extent = {{ -20, -70},{0, -50}}, rotation = 0)));

        equation 
          connect(world.frame_b,gearConstraint.bearing) annotation(Line(points = {{ -42,20},{44,20},{44,40}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(cyl1.frame_b,gearConstraint.frame_a) annotation(Line(points = {{22,50},{34,50}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(gearConstraint.frame_b,cyl2.frame_a) annotation(Line(points = {{54,50},{70,50}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(torque1.frame_b,cyl1.frame_a) annotation(Line(points = {{ -6,50},{2,50}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(torque1.frame_a,world.frame_b) annotation(Line(points = {{ -26,50},{ -36,50},{ -36,20},{ -42,20}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(sine.y,torque1.torque) annotation(Line(points = {{ -79,70},{ -22,70},{ -22,62}}, color = {0,0,127}, smooth = Smooth.None));
          connect(inertia1.flange_b,idealGear.flange_a) annotation(Line(points = {{0, -30},{12, -30}}, color = {0,0,0}, smooth = Smooth.None));
          connect(idealGear.flange_b,inertia2.flange_a) annotation(Line(points = {{32, -30},{44, -30}}, color = {0,0,0}, smooth = Smooth.None));
          connect(torque2.flange,inertia1.flange_a) annotation(Line(points = {{ -28, -30},{ -20, -30}}, color = {0,0,0}, smooth = Smooth.None));
          connect(sine[1].y,torque2.tau) annotation(Line(points = {{ -79,70},{ -72,70},{ -72, -30},{ -50, -30}}, color = {0,0,127}, smooth = Smooth.None));
          connect(mounting1D.flange_b,idealGear.support) annotation(Line(points = {{0, -60},{22, -60},{22, -40}}, color = {0,0,0}, smooth = Smooth.None));
          connect(mounting1D.flange_b,torque2.support) annotation(Line(points = {{0, -60},{4, -60},{4, -48},{ -38, -48},{ -38, -40}}, color = {0,0,0}, smooth = Smooth.None));
          connect(fixed.frame_b,mounting1D.frame_a) annotation(Line(points = {{ -28, -80},{ -10, -80},{ -10, -70}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        end GearConstraint;
      end Rotor1D;
    end Parts;
    package Sensors "Test MultiBody.Sensors"
      encapsulated model AbsoluteSensor "Simple pendulum with one revolute joint and one body"
        import Modelica;
        import Modelica.Mechanics.MultiBody;
        import ObsoleteModelica3;
        extends Modelica.Icons.Example;
        annotation(experiment(StopTime = 5), Window(x = 0.29, y = 0, width = 0.71, height = 0.85), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = ""));
        inner Modelica.Mechanics.MultiBody.World world(gravityType = Modelica.Mechanics.MultiBody.Types.GravityTypes.UniformGravity) annotation(Placement(transformation(extent = {{ -80,0},{ -60,20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Revolute rev(n = {0,0,1}, useAxisFlange = true, a(fixed = false), phi(fixed = true), w(fixed = true)) annotation(Placement(transformation(extent = {{ -40,0},{ -20,20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Damper damper(d = 0.1) annotation(Placement(transformation(extent = {{ -40,40},{ -20,60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyBox box(r = {1,0,0}, animation = false) annotation(Placement(transformation(extent = {{0,0},{20,20}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Sensors.AbsoluteSensor sensor_frame_a1(resolveInFrame_a = true, get_r_abs = true, get_v_abs = true, get_a_abs = true, get_w_abs = true, get_z_abs = true, get_angles = true) annotation(Placement(transformation(extent = {{40,22},{60,42}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Sensors.AbsoluteSensor sensor_world1(get_r_abs = true, get_v_abs = true, get_a_abs = true, get_w_abs = true, get_z_abs = true, resolveInFrame_a = false, get_angles = true) annotation(Placement(transformation(extent = {{40, -20},{60,0}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Sensors.AbsoluteSensor sensor_frame_a2(get_r_abs = true, get_v_abs = true, get_a_abs = true, get_w_abs = true, get_z_abs = true, resolveInFrame_a = false, get_angles = true) annotation(Placement(transformation(extent = {{40,60},{60,80}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Sensors.AbsoluteSensor sensor_world2(get_r_abs = true, get_v_abs = true, get_a_abs = true, get_w_abs = true, get_z_abs = true, resolveInFrame_a = true, get_angles = true) annotation(Placement(transformation(extent = {{40, -40},{60, -60}}, rotation = 0)));
        Modelica.Blocks.Math.Add err1[18](each k2 =  -1) annotation(Placement(transformation(extent = {{80,40},{100,60}}, rotation = 0)));
        Modelica.Blocks.Math.Add err2[18](each k2 =  -1) annotation(Placement(transformation(extent = {{80, -40},{100, -20}}, rotation = 0)));

      equation 
        connect(world.frame_b,rev.frame_a) annotation(Line(points = {{ -60,10},{ -40,10}}, color = {0,0,0}, thickness = 0.5));
        connect(damper.flange_b,rev.axis) annotation(Line(points = {{ -20,50},{ -16,50},{ -16,26},{ -30,26},{ -30,20}}, color = {0,0,0}));
        connect(rev.support,damper.flange_a) annotation(Line(points = {{ -36,20},{ -36,26},{ -48,26},{ -48,50},{ -40,50}}, color = {0,0,0}));
        connect(rev.frame_b,box.frame_a) annotation(Line(points = {{ -20,10},{0,10}}, color = {0,0,0}, thickness = 0.5));
        connect(box.frame_b,sensor_frame_a1.frame_a) annotation(Line(points = {{20,10},{30,10},{30,32},{40,32}}, color = {0,0,0}, thickness = 0.5));
        connect(box.frame_b,sensor_world1.frame_a) annotation(Line(points = {{20,10},{30,10},{30, -10},{40, -10}}, color = {0,0,0}, thickness = 0.5));
        connect(box.frame_b,sensor_frame_a2.frame_a) annotation(Line(points = {{20,10},{30,10},{30,70},{40,70}}, color = {0,0,0}, thickness = 0.5));
        connect(box.frame_b,sensor_world2.frame_a) annotation(Line(points = {{20,10},{30,10},{30, -50},{40, -50}}, color = {0,0,0}, thickness = 0.5));
        connect(sensor_frame_a2.y,err1.u1) annotation(Line(points = {{61,70},{68,70},{68,56},{78,56}}, color = {0,0,255}));
        connect(sensor_frame_a1.y,err1.u2) annotation(Line(points = {{61,32},{68,32},{68,44},{78,44}}, color = {0,0,255}));
        connect(sensor_world1.y,err2.u1) annotation(Line(points = {{61, -10},{68, -10},{68, -24},{78, -24}}, color = {0,0,255}));
        connect(sensor_world2.y,err2.u2) annotation(Line(points = {{61, -50},{70, -50},{70, -36},{78, -36}}, color = {0,0,255}));
        connect(sensor_world2.frame_resolve,world.frame_b) annotation(Line(points = {{50, -60},{50, -76},{ -48, -76},{ -48,10},{ -60,10}}, color = {0,0,0}, pattern = LinePattern.Dot));
        connect(sensor_frame_a2.frame_resolve,rev.frame_b) annotation(Line(points = {{50,80},{50,80},{50,94},{ -12,94},{ -12,10},{ -20,10}}, color = {0,0,0}, pattern = LinePattern.Dot));
      end AbsoluteSensor;
      model AbsoluteSensor2
        extends Modelica.Icons.Example;
        inner Modelica.Mechanics.MultiBody.World world(animateWorld = true, animateGravity = true) annotation(Placement(transformation(extent = {{ -80, -20},{ -60,0}}, rotation = 0)));
        annotation(Diagram(graphics), experiment(StopTime = 1.1));
        ObsoleteModelica3.Mechanics.MultiBody.Sensors.AbsoluteSensor freeMotion annotation(Placement(transformation(extent = {{0, -20},{20,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedRotation fixedRotation(r = {0.5,0.5,0.1}, rotationType = Modelica.Mechanics.MultiBody.Types.RotationTypes.PlanarRotationSequence, angles = {45,45,45}, animation = false) annotation(Placement(transformation(extent = {{ -40, -20},{ -20,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedFrame frame_a(color_x = {0,128,255}) annotation(Placement(transformation(origin = { -10, -50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(world.frame_b,fixedRotation.frame_a) annotation(Line(points = {{ -60, -10},{ -40, -10}}, color = {0,0,0}, thickness = 0.5));
        connect(fixedRotation.frame_b,freeMotion.frame_a) annotation(Line(points = {{ -20, -10},{0, -10}}, color = {0,0,0}, thickness = 0.5));
        connect(frame_a.frame_a,freeMotion.frame_a) annotation(Line(points = {{ -10, -40},{ -10, -10},{0, -10}}, color = {0,0,0}, thickness = 0.5));
      end AbsoluteSensor2;
      model RelativeSensor
        import Modelica.Mechanics.MultiBody.*;
        extends Modelica.Icons.Example;
        inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -100,20},{ -80,40}}, rotation = 0)));
        Joints.Revolute revolute(a(fixed = false), phi(fixed = true), w(fixed = true)) annotation(Placement(transformation(extent = {{ -68,20},{ -48,40}}, rotation = 0)));
        Parts.BodyBox BodyBox1(r = {0.7,0,0}) annotation(Placement(transformation(extent = {{ -40,20},{ -20,40}}, rotation = 0)));
        annotation(Diagram(graphics), experiment(StopTime = 2));
        Parts.Fixed fixed(r = { -0.9,0,0.3}) annotation(Placement(transformation(extent = {{ -100, -40},{ -80, -20}}, rotation = 0)));
        Joints.Revolute revolute1(a(fixed = false), phi(fixed = true, start = 0.785398163397448), w(fixed = true)) annotation(Placement(transformation(extent = {{ -68, -40},{ -48, -20}}, rotation = 0)));
        Parts.BodyBox BodyBox2(r = {0.7,0,0}) annotation(Placement(transformation(extent = {{ -40, -40},{ -20, -20}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Sensors.RelativeSensor relative_a1(resolveInFrame_a = true, get_r_rel = true, get_v_rel = true, get_a_rel = true, get_w_rel = true, get_z_rel = true, get_angles = true) annotation(Placement(transformation(origin = {28,10}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        ObsoleteModelica3.Mechanics.MultiBody.Sensors.RelativeSensor relative_a2(resolveInFrame_a = true, get_r_rel = true, get_v_rel = true, get_a_rel = true, get_w_rel = true, get_z_rel = true, get_angles = true) annotation(Placement(transformation(origin = {50, -10}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Blocks.Math.Add err1[18](each k2 =  -1) annotation(Placement(transformation(extent = {{80,0},{100,20}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Sensors.RelativeSensor relative_b2(get_r_rel = true, get_v_rel = true, get_a_rel = true, get_w_rel = true, get_z_rel = true, resolveInFrame_a = false, get_angles = true) annotation(Placement(transformation(origin = { -2,50}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        ObsoleteModelica3.Mechanics.MultiBody.Sensors.RelativeSensor relative_b1(get_r_rel = true, get_v_rel = true, get_a_rel = true, get_w_rel = true, get_z_rel = true, resolveInFrame_a = false, get_angles = true) annotation(Placement(transformation(origin = {16,70}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Blocks.Math.Add err2[18](each k2 =  -1) annotation(Placement(transformation(extent = {{52,50},{72,70}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Sensors.RelativeSensor relative_c(get_angles = true, sequence = {3,2,1}) annotation(Placement(transformation(origin = { -70,70}, extent = {{ -10,10},{10, -10}}, rotation = 90)));

      equation 
        connect(BodyBox1.frame_a,revolute.frame_b) annotation(Line(points = {{ -40,30},{ -48,30}}));
        connect(revolute.frame_a,world.frame_b) annotation(Line(points = {{ -68,30},{ -80,30}}));
        connect(BodyBox2.frame_a,revolute1.frame_b) annotation(Line(points = {{ -40, -30},{ -48, -30}}));
        connect(revolute1.frame_a,fixed.frame_b) annotation(Line(points = {{ -68, -30},{ -80, -30}}));
        connect(BodyBox1.frame_b,relative_a2.frame_b) annotation(Line(points = {{ -20,30},{50,30},{50,0}}, color = {0,0,0}, thickness = 0.5));
        connect(BodyBox2.frame_b,relative_a2.frame_a) annotation(Line(points = {{ -20, -30},{50, -30},{50, -20}}, color = {0,0,0}, thickness = 0.5));
        connect(BodyBox1.frame_b,relative_a1.frame_b) annotation(Line(points = {{ -20,30},{28,30},{28,20}}, color = {0,0,0}, thickness = 0.5));
        connect(BodyBox2.frame_b,relative_a1.frame_a) annotation(Line(points = {{ -20, -30},{28, -30},{28,0}}, color = {0,0,0}, thickness = 0.5));
        connect(relative_a2.frame_resolve,revolute1.frame_b) annotation(Line(points = {{60, -16},{60, -52},{ -44, -52},{ -44, -30},{ -48, -30}}, color = {0,0,0}, pattern = LinePattern.Dot));
        connect(relative_a1.y,err1.u1) annotation(Line(points = {{39,10},{54,10},{54,16},{78,16}}, color = {0,0,255}));
        connect(relative_a2.y,err1.u2) annotation(Line(points = {{61, -10},{68, -10},{68,4},{78,4}}, color = {0,0,255}));
        connect(relative_b2.frame_a,BodyBox2.frame_b) annotation(Line(points = {{ -2,40},{ -2, -30},{ -20, -30}}, color = {0,0,0}, thickness = 0.5));
        connect(relative_b1.frame_a,BodyBox2.frame_b) annotation(Line(points = {{16,60},{16, -30},{ -20, -30}}, color = {0,0,0}, thickness = 0.5));
        connect(relative_b1.y,err2.u1) annotation(Line(points = {{27,70},{38,70},{38,66},{50,66}}, color = {0,0,255}));
        connect(relative_b2.y,err2.u2) annotation(Line(points = {{9,50},{28,50},{28,54},{50,54}}, color = {0,0,255}));
        connect(relative_b2.frame_b,BodyBox1.frame_b) annotation(Line(points = {{ -2,60},{ -2,68},{ -20,68},{ -20,30}}, color = {0,0,0}, thickness = 0.5));
        connect(relative_b1.frame_b,BodyBox1.frame_b) annotation(Line(points = {{16,80},{16,92},{ -20,92},{ -20,30}}, color = {0,0,0}, thickness = 0.5));
        connect(relative_b2.frame_resolve,BodyBox1.frame_a) annotation(Line(points = {{8,44},{ -32,44},{ -32,56},{ -44,56},{ -44,30},{ -40,30}}, color = {0,0,0}, pattern = LinePattern.Dot));
        connect(world.frame_b,relative_c.frame_a) annotation(Line(points = {{ -80,30},{ -70,30},{ -70,60}}, color = {0,0,0}, thickness = 0.5));
        connect(revolute.frame_b,relative_c.frame_b) annotation(Line(points = {{ -48,30},{ -46,30},{ -46,94},{ -70,94},{ -70,80}}, color = {0,0,0}, thickness = 0.5));
      end RelativeSensor;
      model Distance "Test Sensors.Distance"
        extends Modelica.Icons.Example;
        annotation(experiment(StopTime = 5), Window(x = 0.29, y = 0, width = 0.71, height = 0.85), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = ""));
        inner Modelica.Mechanics.MultiBody.World world(gravityType = Modelica.Mechanics.MultiBody.Types.GravityTypes.UniformGravity) annotation(Placement(transformation(extent = {{ -60,0},{ -40,20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Revolute rev(n = {0,0,1}, useAxisFlange = true, a(fixed = false), phi(fixed = true), w(fixed = true)) annotation(Placement(transformation(extent = {{ -20,0},{0,20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Damper damper(d = 0.1) annotation(Placement(transformation(extent = {{ -20,40},{0,60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Fixed fixed(r = {0, -1,0}) annotation(Placement(transformation(origin = { -50, -70}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Mechanics.MultiBody.Sensors.Distance Distance1 annotation(Placement(transformation(extent = {{ -20, -60},{0, -40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyBox BodyBox1(r = {0.5,0,0}) annotation(Placement(transformation(extent = {{20,0},{40,20}}, rotation = 0)));
        Modelica.Blocks.Continuous.Der Der1 annotation(Placement(transformation(extent = {{0, -100},{20, -80}}, rotation = 0)));
        Modelica.Blocks.Continuous.Der Der2 annotation(Placement(transformation(extent = {{40, -100},{60, -80}}, rotation = 0)));

      equation 
        connect(world.frame_b,rev.frame_a) annotation(Line(points = {{ -40,10},{ -20,10}}, color = {0,0,0}, thickness = 0.5));
        connect(damper.flange_b,rev.axis) annotation(Line(points = {{0,50},{4,50},{4,26},{ -10,26},{ -10,20}}, color = {0,0,0}));
        connect(rev.support,damper.flange_a) annotation(Line(points = {{ -16,20},{ -16,26},{ -28,26},{ -28,50},{ -20,50}}, color = {0,0,0}));
        connect(rev.frame_b,BodyBox1.frame_a) annotation(Line(points = {{0,10},{20,10}}, color = {0,0,0}, thickness = 0.5));
        connect(Distance1.frame_b,BodyBox1.frame_b) annotation(Line(points = {{0, -50},{60, -50},{60,10},{40,10}}, color = {0,0,0}, thickness = 0.5));
        connect(Distance1.frame_a,fixed.frame_b) annotation(Line(points = {{ -20, -50},{ -50, -50},{ -50, -60}}, color = {0,0,0}, thickness = 0.5));
        connect(Der1.y,Der2.u) annotation(Line(points = {{21, -90},{38, -90}}, color = {0,0,255}));
        connect(Distance1.distance,Der1.u) annotation(Line(points = {{ -10, -61},{ -10, -90},{ -2, -90}}, color = {0,0,127}, smooth = Smooth.None));
      end Distance;
      model Distance2 "Test Modelica.Mechanics.MultiBody.Sensors.Distance"
        extends Modelica.Icons.Example;
        inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -80,0},{ -60,20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Revolute Revolute1(a(fixed = false), phi(fixed = true, start = 0.523598775598299), w(fixed = true)) annotation(Placement(transformation(extent = {{ -40,0},{ -20,20}}, rotation = 0)));
        annotation(Diagram(graphics), experiment(StopTime = 0.5, NumberOfIntervals = 50000, Tolerance = 1e-10));
        Modelica.Mechanics.MultiBody.Parts.BodyBox BodyBox1(r = {1,0,0}, width = 0.08) annotation(Placement(transformation(extent = {{0,0},{20,20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Fixed fixed(r = {1,0,0}) annotation(Placement(transformation(origin = {70,50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        Modelica.Mechanics.MultiBody.Sensors.Distance distance1 annotation(Placement(transformation(extent = {{40,0},{60,20}}, rotation = 0)));
        Modelica.Blocks.Continuous.Der Der1 annotation(Placement(transformation(extent = {{60, -24},{80, -4}}, rotation = 0)));
        Modelica.Blocks.Logical.LessEqualThreshold LessEqualThreshold1 annotation(Placement(transformation(extent = {{60, -50},{80, -30}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Sensors.Distance distance2 annotation(Placement(transformation(extent = {{ -40,30},{ -20,50}}, rotation = 0)));

      equation 
        connect(world.frame_b,Revolute1.frame_a) annotation(Line(points = {{ -60,10},{ -40,10}}, color = {0,0,0}, thickness = 0.5));
        connect(Revolute1.frame_b,BodyBox1.frame_a) annotation(Line(points = {{ -20,10},{0,10}}, color = {0,0,0}, thickness = 0.5));
        connect(BodyBox1.frame_b,distance1.frame_a) annotation(Line(points = {{20,10},{40,10}}, color = {0,0,0}, thickness = 0.5));
        connect(distance1.frame_b,fixed.frame_b) annotation(Line(points = {{60,10},{70,10},{70,40}}, color = {0,0,0}, thickness = 0.5));
        connect(distance1.distance,Der1.u) annotation(Line(points = {{50, -1},{50, -14},{58, -14}}, color = {0,0,127}));
        connect(distance1.distance,LessEqualThreshold1.u) annotation(Line(points = {{50, -1},{50, -40},{58, -40}}, color = {0,0,127}));
        connect(Revolute1.frame_a,distance2.frame_a) annotation(Line(points = {{ -40,10},{ -46,10},{ -46,40},{ -40,40}}, color = {0,0,0}, thickness = 0.5));
        connect(Revolute1.frame_b,distance2.frame_b) annotation(Line(points = {{ -20,10},{ -10,10},{ -10,40},{ -20,40}}, color = {0,0,0}, thickness = 0.5));
      end Distance2;
      model CutForce
        extends Modelica.Icons.Example;
        annotation(experiment(StopTime = 1.1));
        inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -100,0},{ -80,20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Sensors.CutForce cutForce1a(N_to_m = 10) annotation(Placement(transformation(extent = {{ -20,20},{0,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation translate1a(r = {0, -1,0}) annotation(Placement(transformation(origin = {30,10}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
        Modelica.Mechanics.MultiBody.Parts.Body body1a annotation(Placement(transformation(extent = {{60,0},{80,20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Sensors.CutForce cutForce2a(positiveSign = false, N_to_m = 10, resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.world) annotation(Placement(transformation(extent = {{ -20, -40},{0, -20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation translate2a(r = {0, -1,0}) annotation(Placement(transformation(origin = {30, -30}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
        Modelica.Mechanics.MultiBody.Parts.Body body2a annotation(Placement(transformation(extent = {{60, -40},{80, -20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedRotation rotate1a(n = {0,0,1}, angle = 90, r = {0,0, -0.2}) annotation(Placement(transformation(extent = {{ -60,0},{ -40,20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedRotation rotate2a(n = {0,0,1}, angle = 90, r = {0,0,0.2}) annotation(Placement(transformation(extent = {{ -60, -40},{ -40, -20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Sensors.CutForce cutForce1b(N_to_m = 10, resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve) annotation(Placement(transformation(extent = {{ -20,60},{0,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation translate1b(r = {0, -1,0}) annotation(Placement(transformation(origin = {30,70}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
        Modelica.Mechanics.MultiBody.Parts.Body body1b annotation(Placement(transformation(extent = {{60,60},{80,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedRotation rotate1b(n = {0,0,1}, angle = 90, r = {0,0, -0.6}) annotation(Placement(transformation(extent = {{ -60,60},{ -40,80}}, rotation = 0)));
        Modelica.Blocks.Math.Add err1[3](each k2 =  -1) annotation(Placement(transformation(extent = {{10,30},{30,50}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Sensors.CutForce cutForce2b(positiveSign = false, N_to_m = 10, resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve) annotation(Placement(transformation(extent = {{ -20, -80},{0, -100}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation translate2b(r = {0, -1,0}) annotation(Placement(transformation(origin = {30, -90}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
        Modelica.Mechanics.MultiBody.Parts.Body body2b annotation(Placement(transformation(extent = {{60, -100},{80, -80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedRotation rotate2b(n = {0,0,1}, angle = 90, r = {0,0,0.6}) annotation(Placement(transformation(extent = {{ -60, -100},{ -40, -80}}, rotation = 0)));
        Modelica.Blocks.Math.Add err2[3](each k2 =  -1) annotation(Placement(transformation(extent = {{8, -72},{28, -52}}, rotation = 0)));

      equation 
        annotation(Diagram(graphics));
        connect(world.frame_b,rotate1a.frame_a) annotation(Line(points = {{ -80,10},{ -60,10}}, color = {0,0,0}, thickness = 0.5));
        connect(rotate1a.frame_b,cutForce1a.frame_a) annotation(Line(points = {{ -40,10},{ -20,10}}, color = {0,0,0}, thickness = 0.5));
        connect(rotate2a.frame_b,cutForce2a.frame_a) annotation(Line(points = {{ -40, -30},{ -20, -30}}, color = {0,0,0}, thickness = 0.5));
        connect(rotate2a.frame_a,world.frame_b) annotation(Line(points = {{ -60, -30},{ -70, -30},{ -70,10},{ -80,10}}, color = {0,0,0}, thickness = 0.5));
        connect(rotate1b.frame_b,cutForce1b.frame_a) annotation(Line(points = {{ -40,70},{ -20,70}}, color = {0,0,0}, thickness = 0.5));
        connect(cutForce1b.frame_resolve,rotate1b.frame_b) annotation(Line(points = {{ -2,60},{ -20,60},{ -20,46},{ -40,46},{ -40,70}}, color = {0,0,0}, pattern = LinePattern.Dot));
        connect(cutForce1b.force,err1.u1) annotation(Line(points = {{ -18,59},{ -18,46},{8,46}}, color = {0,0,255}));
        connect(cutForce1a.force,err1.u2) annotation(Line(points = {{ -18,21},{ -18,34},{8,34}}, color = {0,0,255}));
        connect(world.frame_b,rotate1b.frame_a) annotation(Line(points = {{ -80,10},{ -70,10},{ -70,70},{ -60,70}}, color = {0,0,0}, thickness = 0.5));
        connect(rotate2b.frame_b,cutForce2b.frame_a) annotation(Line(points = {{ -40, -90},{ -20, -90}}, color = {0,0,0}, thickness = 0.5));
        connect(rotate2b.frame_a,world.frame_b) annotation(Line(points = {{ -60, -90},{ -70, -90},{ -70,10},{ -80,10}}, color = {0,0,0}, thickness = 0.5));
        connect(cutForce2a.force,err2.u1) annotation(Line(points = {{ -18, -41},{ -18, -56},{6, -56}}, color = {0,0,255}));
        connect(cutForce2b.force,err2.u2) annotation(Line(points = {{ -18, -79},{ -18, -68},{6, -68}}, color = {0,0,255}));
        connect(cutForce2b.frame_resolve,world.frame_b) annotation(Line(points = {{ -2, -80},{ -2, -58},{ -74, -58},{ -74,10},{ -80,10}}, color = {0,0,0}, pattern = LinePattern.Dot));
        connect(cutForce1b.frame_b,translate1b.frame_a) annotation(Line(points = {{0,70},{20,70}}, color = {0,0,0}, thickness = 0.5));
        connect(translate1b.frame_b,body1b.frame_a) annotation(Line(points = {{40,70},{60,70}}, color = {0,0,0}, thickness = 0.5));
        connect(cutForce1a.frame_b,translate1a.frame_a) annotation(Line(points = {{0,10},{10,10},{10,10},{20,10}}, color = {0,0,0}, thickness = 0.5));
        connect(translate1a.frame_b,body1a.frame_a) annotation(Line(points = {{40,10},{50,10},{50,10},{60,10}}, color = {0,0,0}, thickness = 0.5));
        connect(cutForce2a.frame_b,translate2a.frame_a) annotation(Line(points = {{0, -30},{20, -30}}, color = {0,0,0}, thickness = 0.5));
        connect(translate2a.frame_b,body2a.frame_a) annotation(Line(points = {{40, -30},{60, -30}}, color = {0,0,0}, thickness = 0.5));
        connect(cutForce2b.frame_b,translate2b.frame_a) annotation(Line(points = {{0, -90},{20, -90}}, color = {0,0,0}, thickness = 0.5));
        connect(translate2b.frame_b,body2b.frame_a) annotation(Line(points = {{40, -90},{60, -90}}, color = {0,0,0}, thickness = 0.5));
      end CutForce;
      model CutForce2
        extends Modelica.Icons.Example;
        inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) annotation(Placement(transformation(extent = {{ -80, -20},{ -60,0}}, rotation = 0)));
        annotation(Diagram(graphics));
        Modelica.Mechanics.MultiBody.Sensors.CutForce cutForce annotation(Placement(transformation(extent = {{0, -20},{20,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedRotation fixedRotation(r = {0.5,0.5,0.1}, rotationType = Modelica.Mechanics.MultiBody.Types.RotationTypes.PlanarRotationSequence, animation = false) annotation(Placement(transformation(extent = {{ -40, -20},{ -20,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedFrame frame_b(color_x = {0,0,200}) annotation(Placement(transformation(origin = {50,30}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
        Modelica.Mechanics.MultiBody.Forces.WorldForce WorldForce1(animation = false) annotation(Placement(transformation(extent = {{60, -20},{40,0}}, rotation = 0)));
        Modelica.Blocks.Sources.Constant Constant1[3](k = {500,0,0}) annotation(Placement(transformation(extent = {{100, -20},{80,0}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(world.frame_b,fixedRotation.frame_a) annotation(Line(points = {{ -60, -10},{ -40, -10}}, color = {0,0,0}, thickness = 0.5));
        connect(Constant1.y,WorldForce1.force) annotation(Line(points = {{79, -10},{62, -10}}, color = {0,0,255}));
        connect(fixedRotation.frame_b,cutForce.frame_a) annotation(Line(points = {{ -20, -10},{0, -10}}, color = {0,0,0}, thickness = 0.5));
        connect(cutForce.frame_b,WorldForce1.frame_b) annotation(Line(points = {{20, -10},{40, -10}}, color = {0,0,0}, thickness = 0.5));
        connect(frame_b.frame_a,cutForce.frame_b) annotation(Line(points = {{50,20},{50,14},{28,14},{28, -10},{20, -10}}, color = {0,0,0}, thickness = 0.5));
      end CutForce2;
      model CutTorque
        extends Modelica.Icons.Example;
        annotation(experiment(StopTime = 1.1));
        inner Modelica.Mechanics.MultiBody.World world(n = {0,0, -1}) annotation(Placement(transformation(extent = {{ -100,0},{ -80,20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Sensors.CutTorque cutTorque1a(Nm_to_m = 10) annotation(Placement(transformation(extent = {{ -20,20},{0,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation translate1a(r = {0, -1,0}) annotation(Placement(transformation(origin = {30,10}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
        Modelica.Mechanics.MultiBody.Parts.Body body1a annotation(Placement(transformation(extent = {{60,0},{80,20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Sensors.CutTorque cutTorque2a(positiveSign = false, Nm_to_m = 10, resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.world) annotation(Placement(transformation(extent = {{ -20, -40},{0, -20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation translate2a(r = {0, -1,0}) annotation(Placement(transformation(origin = {30, -30}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
        Modelica.Mechanics.MultiBody.Parts.Body body2a annotation(Placement(transformation(extent = {{60, -40},{80, -20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedRotation rotate1a(n = {0,0,1}, angle = 90, r = {0,0, -0.2}) annotation(Placement(transformation(extent = {{ -60,0},{ -40,20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedRotation rotate2a(n = {0,0,1}, angle = 90, r = {0,0,0.2}) annotation(Placement(transformation(extent = {{ -60, -40},{ -40, -20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Sensors.CutTorque cutTorque1b(Nm_to_m = 10, resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve) annotation(Placement(transformation(extent = {{ -20,60},{0,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation translate1b(r = {0, -1,0}) annotation(Placement(transformation(origin = {30,70}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
        Modelica.Mechanics.MultiBody.Parts.Body body1b annotation(Placement(transformation(extent = {{60,60},{80,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedRotation rotate1b(n = {0,0,1}, angle = 90, r = {0,0, -0.6}) annotation(Placement(transformation(extent = {{ -60,60},{ -40,80}}, rotation = 0)));
        Modelica.Blocks.Math.Add err1[3](each k2 =  -1) annotation(Placement(transformation(extent = {{10,30},{30,50}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Sensors.CutTorque cutTorque2b(positiveSign = false, Nm_to_m = 10, resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve) annotation(Placement(transformation(extent = {{ -20, -80},{0, -100}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation translate2b(r = {0, -1,0}) annotation(Placement(transformation(origin = {30, -90}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
        Modelica.Mechanics.MultiBody.Parts.Body body2b annotation(Placement(transformation(extent = {{60, -100},{80, -80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedRotation rotate2b(n = {0,0,1}, angle = 90, r = {0,0,0.6}) annotation(Placement(transformation(extent = {{ -60, -100},{ -40, -80}}, rotation = 0)));
        Modelica.Blocks.Math.Add err2[3](each k2 =  -1) annotation(Placement(transformation(extent = {{8, -72},{28, -52}}, rotation = 0)));

      equation 
        annotation(Diagram(graphics));
        connect(world.frame_b,rotate1a.frame_a) annotation(Line(points = {{ -80,10},{ -60,10}}, color = {0,0,0}, thickness = 0.5));
        connect(rotate1a.frame_b,cutTorque1a.frame_a) annotation(Line(points = {{ -40,10},{ -20,10}}, color = {0,0,0}, thickness = 0.5));
        connect(rotate2a.frame_b,cutTorque2a.frame_a) annotation(Line(points = {{ -40, -30},{ -20, -30}}, color = {0,0,0}, thickness = 0.5));
        connect(rotate2a.frame_a,world.frame_b) annotation(Line(points = {{ -60, -30},{ -70, -30},{ -70,10},{ -80,10}}, color = {0,0,0}, thickness = 0.5));
        connect(rotate1b.frame_b,cutTorque1b.frame_a) annotation(Line(points = {{ -40,70},{ -20,70}}, color = {0,0,0}, thickness = 0.5));
        connect(cutTorque1b.frame_resolve,rotate1b.frame_b) annotation(Line(points = {{ -2,60},{ -20,60},{ -20,46},{ -40,46},{ -40,70}}, color = {0,0,0}, pattern = LinePattern.Dot));
        connect(cutTorque1b.torque,err1.u1) annotation(Line(points = {{ -18,59},{ -18,46},{8,46}}, color = {0,0,255}));
        connect(cutTorque1a.torque,err1.u2) annotation(Line(points = {{ -18,21},{ -18,34},{8,34}}, color = {0,0,255}));
        connect(world.frame_b,rotate1b.frame_a) annotation(Line(points = {{ -80,10},{ -70,10},{ -70,70},{ -60,70}}, color = {0,0,0}, thickness = 0.5));
        connect(rotate2b.frame_b,cutTorque2b.frame_a) annotation(Line(points = {{ -40, -90},{ -20, -90}}, color = {0,0,0}, thickness = 0.5));
        connect(rotate2b.frame_a,world.frame_b) annotation(Line(points = {{ -60, -90},{ -70, -90},{ -70,10},{ -80,10}}, color = {0,0,0}, thickness = 0.5));
        connect(cutTorque2a.torque,err2.u1) annotation(Line(points = {{ -18, -41},{ -18, -56},{6, -56}}, color = {0,0,255}));
        connect(cutTorque2b.torque,err2.u2) annotation(Line(points = {{ -18, -79},{ -18, -68},{6, -68}}, color = {0,0,255}));
        connect(cutTorque2b.frame_resolve,world.frame_b) annotation(Line(points = {{ -2, -80},{ -2, -58},{ -74, -58},{ -74,10},{ -80,10}}, color = {0,0,0}, pattern = LinePattern.Dot));
        connect(cutTorque1b.frame_b,translate1b.frame_a) annotation(Line(points = {{0,70},{20,70}}, color = {0,0,0}, thickness = 0.5));
        connect(translate1b.frame_b,body1b.frame_a) annotation(Line(points = {{40,70},{60,70}}, color = {0,0,0}, thickness = 0.5));
        connect(cutTorque1a.frame_b,translate1a.frame_a) annotation(Line(points = {{0,10},{10,10},{10,10},{20,10}}, color = {0,0,0}, thickness = 0.5));
        connect(translate1a.frame_b,body1a.frame_a) annotation(Line(points = {{40,10},{50,10},{50,10},{60,10}}, color = {0,0,0}, thickness = 0.5));
        connect(cutTorque2a.frame_b,translate2a.frame_a) annotation(Line(points = {{0, -30},{20, -30}}, color = {0,0,0}, thickness = 0.5));
        connect(translate2a.frame_b,body2a.frame_a) annotation(Line(points = {{40, -30},{60, -30}}, color = {0,0,0}, thickness = 0.5));
        connect(cutTorque2b.frame_b,translate2b.frame_a) annotation(Line(points = {{0, -90},{20, -90}}, color = {0,0,0}, thickness = 0.5));
        connect(translate2b.frame_b,body2b.frame_a) annotation(Line(points = {{40, -90},{60, -90}}, color = {0,0,0}, thickness = 0.5));
      end CutTorque;
      model CutTorque2
        extends Modelica.Icons.Example;
        inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) annotation(Placement(transformation(extent = {{ -80, -20},{ -60,0}}, rotation = 0)));
        annotation(Diagram(graphics));
        Modelica.Mechanics.MultiBody.Sensors.CutTorque cutTorque annotation(Placement(transformation(extent = {{0, -20},{20,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedRotation fixedRotation(r = {0.5,0.5,0.1}, rotationType = Modelica.Mechanics.MultiBody.Types.RotationTypes.PlanarRotationSequence, animation = false) annotation(Placement(transformation(extent = {{ -40, -20},{ -20,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedFrame frame_b(color_x = {0,0,200}) annotation(Placement(transformation(origin = {50,30}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
        Modelica.Mechanics.MultiBody.Forces.WorldTorque worldTorque(animation = false) annotation(Placement(transformation(extent = {{60, -20},{40,0}}, rotation = 0)));
        Modelica.Blocks.Sources.Constant Constant1[3](k = {500,0,0}) annotation(Placement(transformation(extent = {{100, -20},{80,0}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(world.frame_b,fixedRotation.frame_a) annotation(Line(points = {{ -60, -10},{ -40, -10}}, color = {0,0,0}, thickness = 0.5));
        connect(Constant1.y,worldTorque.torque) annotation(Line(points = {{79, -10},{62, -10}}, color = {0,0,255}));
        connect(fixedRotation.frame_b,cutTorque.frame_a) annotation(Line(points = {{ -20, -10},{0, -10}}, color = {0,0,0}, thickness = 0.5));
        connect(cutTorque.frame_b,worldTorque.frame_b) annotation(Line(points = {{20, -10},{40, -10}}, color = {0,0,0}, thickness = 0.5));
        connect(frame_b.frame_a,cutTorque.frame_b) annotation(Line(points = {{50,20},{50,14},{28,14},{28, -10},{20, -10}}, color = {0,0,0}, thickness = 0.5));
      end CutTorque2;
      model CutForceAndTorque1
        extends Modelica.Icons.Example;
        annotation(experiment(StopTime = 1.1));
        inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -100,0},{ -80,20}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Sensors.CutForceAndTorque cutForce1a(N_to_m = 10, Nm_to_m = 10) annotation(Placement(transformation(extent = {{ -20,20},{0,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation translate1a(r = {0, -1,0}) annotation(Placement(transformation(origin = {30,10}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
        Modelica.Mechanics.MultiBody.Parts.Body body1a annotation(Placement(transformation(extent = {{60,0},{80,20}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Sensors.CutForceAndTorque cutForce2a(positiveSign = false, resolveInFrame_a = false, N_to_m = 10, Nm_to_m = 10) annotation(Placement(transformation(extent = {{ -20, -40},{0, -20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation translate2a(r = {0, -1,0}) annotation(Placement(transformation(origin = {30, -30}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
        Modelica.Mechanics.MultiBody.Parts.Body body2a annotation(Placement(transformation(extent = {{60, -40},{80, -20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedRotation rotate1a(n = {0,0,1}, angle = 90, r = {0,0, -0.2}) annotation(Placement(transformation(extent = {{ -60,0},{ -40,20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedRotation rotate2a(n = {0,0,1}, angle = 90, r = {0,0,0.2}) annotation(Placement(transformation(extent = {{ -60, -40},{ -40, -20}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Sensors.CutForceAndTorque cutForce1b(N_to_m = 10, Nm_to_m = 10) annotation(Placement(transformation(extent = {{ -20,60},{0,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation translate1b(r = {0, -1,0}) annotation(Placement(transformation(origin = {30,70}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
        Modelica.Mechanics.MultiBody.Parts.Body body1b annotation(Placement(transformation(extent = {{60,60},{80,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedRotation rotate1b(n = {0,0,1}, angle = 90, r = {0,0, -0.6}) annotation(Placement(transformation(extent = {{ -60,60},{ -40,80}}, rotation = 0)));
        Modelica.Blocks.Math.Add err1[6](each k2 =  -1) annotation(Placement(transformation(extent = {{10,30},{30,50}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Sensors.CutForceAndTorque cutForce2b(positiveSign = false, resolveInFrame_a = false, N_to_m = 10, Nm_to_m = 10) annotation(Placement(transformation(extent = {{ -20, -80},{0, -100}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation translate2b(r = {0, -1,0}) annotation(Placement(transformation(origin = {30, -90}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
        Modelica.Mechanics.MultiBody.Parts.Body body2b annotation(Placement(transformation(extent = {{60, -100},{80, -80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedRotation rotate2b(n = {0,0,1}, angle = 90, r = {0,0,0.6}) annotation(Placement(transformation(extent = {{ -60, -100},{ -40, -80}}, rotation = 0)));
        Modelica.Blocks.Math.Add err2[6](each k2 =  -1) annotation(Placement(transformation(extent = {{8, -72},{28, -52}}, rotation = 0)));

      equation 
        annotation(Diagram(graphics));
        connect(world.frame_b,rotate1a.frame_a) annotation(Line(points = {{ -80,10},{ -60,10}}, color = {0,0,0}, thickness = 0.5));
        connect(rotate1a.frame_b,cutForce1a.frame_a) annotation(Line(points = {{ -40,10},{ -20,10}}, color = {0,0,0}, thickness = 0.5));
        connect(rotate2a.frame_b,cutForce2a.frame_a) annotation(Line(points = {{ -40, -30},{ -20, -30}}, color = {0,0,0}, thickness = 0.5));
        connect(rotate2a.frame_a,world.frame_b) annotation(Line(points = {{ -60, -30},{ -70, -30},{ -70,10},{ -80,10}}, color = {0,0,0}, thickness = 0.5));
        connect(rotate1b.frame_b,cutForce1b.frame_a) annotation(Line(points = {{ -40,70},{ -20,70}}, color = {0,0,0}, thickness = 0.5));
        connect(cutForce1b.frame_resolve,rotate1b.frame_b) annotation(Line(points = {{ -2,60},{ -20,60},{ -20,46},{ -40,46},{ -40,70}}, color = {0,0,0}, pattern = LinePattern.Dot));
        connect(cutForce1b.load,err1.u1) annotation(Line(points = {{ -18,59},{ -18,46},{8,46}}, color = {0,0,255}));
        connect(cutForce1a.load,err1.u2) annotation(Line(points = {{ -18,21},{ -18,34},{8,34}}, color = {0,0,255}));
        connect(world.frame_b,rotate1b.frame_a) annotation(Line(points = {{ -80,10},{ -70,10},{ -70,70},{ -60,70}}, color = {0,0,0}, thickness = 0.5));
        connect(rotate2b.frame_b,cutForce2b.frame_a) annotation(Line(points = {{ -40, -90},{ -20, -90}}, color = {0,0,0}, thickness = 0.5));
        connect(rotate2b.frame_a,world.frame_b) annotation(Line(points = {{ -60, -90},{ -70, -90},{ -70,10},{ -80,10}}, color = {0,0,0}, thickness = 0.5));
        connect(cutForce2a.load,err2.u1) annotation(Line(points = {{ -18, -41},{ -18, -56},{6, -56}}, color = {0,0,255}));
        connect(cutForce2b.load,err2.u2) annotation(Line(points = {{ -18, -79},{ -18, -68},{6, -68}}, color = {0,0,255}));
        connect(cutForce2b.frame_resolve,world.frame_b) annotation(Line(points = {{ -2, -80},{ -2, -58},{ -74, -58},{ -74,10},{ -80,10}}, color = {0,0,0}, pattern = LinePattern.Dot));
        connect(cutForce1b.frame_b,translate1b.frame_a) annotation(Line(points = {{0,70},{20,70}}, color = {0,0,0}, thickness = 0.5));
        connect(translate1b.frame_b,body1b.frame_a) annotation(Line(points = {{40,70},{60,70}}, color = {0,0,0}, thickness = 0.5));
        connect(cutForce1a.frame_b,translate1a.frame_a) annotation(Line(points = {{0,10},{10,10},{10,10},{20,10}}, color = {0,0,0}, thickness = 0.5));
        connect(translate1a.frame_b,body1a.frame_a) annotation(Line(points = {{40,10},{50,10},{50,10},{60,10}}, color = {0,0,0}, thickness = 0.5));
        connect(cutForce2a.frame_b,translate2a.frame_a) annotation(Line(points = {{0, -30},{20, -30}}, color = {0,0,0}, thickness = 0.5));
        connect(translate2a.frame_b,body2a.frame_a) annotation(Line(points = {{40, -30},{60, -30}}, color = {0,0,0}, thickness = 0.5));
        connect(cutForce2b.frame_b,translate2b.frame_a) annotation(Line(points = {{0, -90},{20, -90}}, color = {0,0,0}, thickness = 0.5));
        connect(translate2b.frame_b,body2b.frame_a) annotation(Line(points = {{40, -90},{60, -90}}, color = {0,0,0}, thickness = 0.5));
      end CutForceAndTorque1;
      model CutForceAndTorque2
        extends Modelica.Icons.Example;
        annotation(experiment(StopTime = 1.1));
        inner Modelica.Mechanics.MultiBody.World world(n = {0,0, -1}) annotation(Placement(transformation(extent = {{ -100,0},{ -80,20}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Sensors.CutForceAndTorque cutForce1a(N_to_m = 10, Nm_to_m = 10) annotation(Placement(transformation(extent = {{ -20,20},{0,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation translate1a(r = {0, -1,0}) annotation(Placement(transformation(origin = {30,10}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
        Modelica.Mechanics.MultiBody.Parts.Body body1a annotation(Placement(transformation(extent = {{60,0},{80,20}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Sensors.CutForceAndTorque cutForce2a(positiveSign = false, resolveInFrame_a = false, N_to_m = 10, Nm_to_m = 10) annotation(Placement(transformation(extent = {{ -20, -40},{0, -20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation translate2a(r = {0, -1,0}) annotation(Placement(transformation(origin = {30, -30}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
        Modelica.Mechanics.MultiBody.Parts.Body body2a annotation(Placement(transformation(extent = {{60, -40},{80, -20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedRotation rotate1a(n = {0,0,1}, angle = 90, r = {0,0, -0.2}) annotation(Placement(transformation(extent = {{ -60,0},{ -40,20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedRotation rotate2a(n = {0,0,1}, angle = 90, r = {0,0,0.2}) annotation(Placement(transformation(extent = {{ -60, -40},{ -40, -20}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Sensors.CutForceAndTorque cutForce1b(N_to_m = 10, Nm_to_m = 10) annotation(Placement(transformation(extent = {{ -20,60},{0,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation translate1b(r = {0, -1,0}) annotation(Placement(transformation(origin = {30,70}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
        Modelica.Mechanics.MultiBody.Parts.Body body1b annotation(Placement(transformation(extent = {{60,60},{80,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedRotation rotate1b(n = {0,0,1}, angle = 90, r = {0,0, -0.6}) annotation(Placement(transformation(extent = {{ -60,60},{ -40,80}}, rotation = 0)));
        Modelica.Blocks.Math.Add err1[6](each k2 =  -1) annotation(Placement(transformation(extent = {{10,30},{30,50}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Sensors.CutForceAndTorque cutForce2b(positiveSign = false, resolveInFrame_a = false, N_to_m = 10, Nm_to_m = 10) annotation(Placement(transformation(extent = {{ -20, -80},{0, -100}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedTranslation translate2b(r = {0, -1,0}) annotation(Placement(transformation(origin = {30, -90}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
        Modelica.Mechanics.MultiBody.Parts.Body body2b annotation(Placement(transformation(extent = {{60, -100},{80, -80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedRotation rotate2b(n = {0,0,1}, angle = 90, r = {0,0,0.6}) annotation(Placement(transformation(extent = {{ -60, -100},{ -40, -80}}, rotation = 0)));
        Modelica.Blocks.Math.Add err2[6](each k2 =  -1) annotation(Placement(transformation(extent = {{8, -72},{28, -52}}, rotation = 0)));

      equation 
        annotation(Diagram(graphics));
        connect(world.frame_b,rotate1a.frame_a) annotation(Line(points = {{ -80,10},{ -60,10}}, color = {0,0,0}, thickness = 0.5));
        connect(rotate1a.frame_b,cutForce1a.frame_a) annotation(Line(points = {{ -40,10},{ -20,10}}, color = {0,0,0}, thickness = 0.5));
        connect(rotate2a.frame_b,cutForce2a.frame_a) annotation(Line(points = {{ -40, -30},{ -20, -30}}, color = {0,0,0}, thickness = 0.5));
        connect(rotate2a.frame_a,world.frame_b) annotation(Line(points = {{ -60, -30},{ -70, -30},{ -70,10},{ -80,10}}, color = {0,0,0}, thickness = 0.5));
        connect(rotate1b.frame_b,cutForce1b.frame_a) annotation(Line(points = {{ -40,70},{ -20,70}}, color = {0,0,0}, thickness = 0.5));
        connect(cutForce1b.frame_resolve,rotate1b.frame_b) annotation(Line(points = {{ -2,60},{ -20,60},{ -20,46},{ -40,46},{ -40,70}}, color = {0,0,0}, pattern = LinePattern.Dot));
        connect(cutForce1b.load,err1.u1) annotation(Line(points = {{ -18,59},{ -18,46},{8,46}}, color = {0,0,255}));
        connect(cutForce1a.load,err1.u2) annotation(Line(points = {{ -18,21},{ -18,34},{8,34}}, color = {0,0,255}));
        connect(world.frame_b,rotate1b.frame_a) annotation(Line(points = {{ -80,10},{ -70,10},{ -70,70},{ -60,70}}, color = {0,0,0}, thickness = 0.5));
        connect(rotate2b.frame_b,cutForce2b.frame_a) annotation(Line(points = {{ -40, -90},{ -20, -90}}, color = {0,0,0}, thickness = 0.5));
        connect(rotate2b.frame_a,world.frame_b) annotation(Line(points = {{ -60, -90},{ -70, -90},{ -70,10},{ -80,10}}, color = {0,0,0}, thickness = 0.5));
        connect(cutForce2a.load,err2.u1) annotation(Line(points = {{ -18, -41},{ -18, -56},{6, -56}}, color = {0,0,255}));
        connect(cutForce2b.load,err2.u2) annotation(Line(points = {{ -18, -79},{ -18, -68},{6, -68}}, color = {0,0,255}));
        connect(cutForce2b.frame_resolve,world.frame_b) annotation(Line(points = {{ -2, -80},{ -2, -58},{ -74, -58},{ -74,10},{ -80,10}}, color = {0,0,0}, pattern = LinePattern.Dot));
        connect(cutForce1b.frame_b,translate1b.frame_a) annotation(Line(points = {{0,70},{20,70}}, color = {0,0,0}, thickness = 0.5));
        connect(translate1b.frame_b,body1b.frame_a) annotation(Line(points = {{40,70},{60,70}}, color = {0,0,0}, thickness = 0.5));
        connect(cutForce1a.frame_b,translate1a.frame_a) annotation(Line(points = {{0,10},{10,10},{10,10},{20,10}}, color = {0,0,0}, thickness = 0.5));
        connect(translate1a.frame_b,body1a.frame_a) annotation(Line(points = {{40,10},{50,10},{50,10},{60,10}}, color = {0,0,0}, thickness = 0.5));
        connect(cutForce2a.frame_b,translate2a.frame_a) annotation(Line(points = {{0, -30},{20, -30}}, color = {0,0,0}, thickness = 0.5));
        connect(translate2a.frame_b,body2a.frame_a) annotation(Line(points = {{40, -30},{60, -30}}, color = {0,0,0}, thickness = 0.5));
        connect(cutForce2b.frame_b,translate2b.frame_a) annotation(Line(points = {{0, -90},{20, -90}}, color = {0,0,0}, thickness = 0.5));
        connect(translate2b.frame_b,body2b.frame_a) annotation(Line(points = {{40, -90},{60, -90}}, color = {0,0,0}, thickness = 0.5));
      end CutForceAndTorque2;
      model CutForceAndTorque3
        extends Modelica.Icons.Example;
        inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) annotation(Placement(transformation(extent = {{ -80, -20},{ -60,0}}, rotation = 0)));
        annotation(Diagram(graphics), experiment(StopTime = 1.1), experimentSetupOutput);
        ObsoleteModelica3.Mechanics.MultiBody.Sensors.CutForceAndTorque cutForceAndTorque annotation(Placement(transformation(extent = {{0, -20},{20,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.FixedRotation fixedRotation(r = {0.5,0.5,0.1}, rotationType = Modelica.Mechanics.MultiBody.Types.RotationTypes.PlanarRotationSequence, animation = false) annotation(Placement(transformation(extent = {{ -40, -20},{ -20,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedFrame frame_b(color_x = {0,0,200}) annotation(Placement(transformation(origin = {50,30}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
        ObsoleteModelica3.Mechanics.MultiBody.Forces.WorldForceAndTorque worldForceAndTorque(animation = false) annotation(Placement(transformation(extent = {{60, -20},{40,0}}, rotation = 0)));
        Modelica.Blocks.Sources.Constant Constant1[6](k = {500,0,0,450, -250,0}) annotation(Placement(transformation(extent = {{100, -20},{80,0}}, rotation = 0)));
        annotation(experiment(StopTime = 1.1));

      equation 
        connect(world.frame_b,fixedRotation.frame_a) annotation(Line(points = {{ -60, -10},{ -40, -10}}, color = {0,0,0}, thickness = 0.5));
        connect(fixedRotation.frame_b,cutForceAndTorque.frame_a) annotation(Line(points = {{ -20, -10},{0, -10}}, color = {0,0,0}, thickness = 0.5));
        connect(cutForceAndTorque.frame_b,worldForceAndTorque.frame_b) annotation(Line(points = {{20, -10},{40, -10}}, color = {0,0,0}, thickness = 0.5));
        connect(frame_b.frame_a,cutForceAndTorque.frame_b) annotation(Line(points = {{50,20},{50,14},{28,14},{28, -10},{20, -10}}, color = {0,0,0}, thickness = 0.5));
        connect(Constant1.y,worldForceAndTorque.load) annotation(Line(points = {{79, -10},{62, -10}}, color = {0,0,127}, smooth = Smooth.None));
      end CutForceAndTorque3;
    end Sensors;
    package Visualizers "Test MultiBody.Visualizers"
      encapsulated model Shapes "Test and show models Visualizers.FixedShape"
        import Modelica.Icons;
        import Modelica.Mechanics.MultiBody;
        import Modelica.Mechanics.MultiBody.Visualizers;
        import Modelica;
        extends Icons.Example;
        Modelica.Mechanics.MultiBody.Parts.Fixed fixed(animation = false) annotation(Placement(transformation(extent = {{ -100,40},{ -80,60}}, rotation = 0)));
        annotation(experiment(StopTime = 1), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
        Modelica.Mechanics.MultiBody.Visualizers.FixedShape box(widthDirection = {1,0,0}, r_shape = {0,0,0}, lengthDirection = {0.5,1, -0.5}, length = 0.12, width = 0.08, height = 0.08, color = {155,155,155}, shapeType = "box") annotation(Placement(transformation(origin = { -60,70}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedShape sphere(r_shape = {0.2,0,0}, lengthDirection = {0.5,1, -0.5}, length = 0.1, widthDirection = { -0.5,1,0}, width = 0.1, height = 0.1, color = {155,155,155}, shapeType = "sphere") annotation(Placement(transformation(origin = { -30,70}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedShape cylinder(widthDirection = {1,0,0}, length = 0.1, width = 0.1, height = 0.1, r_shape = {0.4,0,0}, lengthDirection = {0.5,1, -0.5}, extra = 5, color = {155,155,155}, shapeType = "cylinder") annotation(Placement(transformation(origin = { -4,70}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedShape pipe(widthDirection = {1,0,0}, length = 0.1, width = 0.1, height = 0.1, r_shape = {0.8,0,0}, lengthDirection = {0.5,1, -0.5}, extra = 0.7, shapeType = "pipe", color = {155,155,155}) annotation(Placement(transformation(origin = {42,70}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedShape beam(widthDirection = {1,0,0}, length = 0.1, width = 0.1, height = 0.1, r_shape = {0, -0.2,0}, lengthDirection = {0.5,1, -0.5}, extra = 0, shapeType = "beam", color = {155,155,155}) annotation(Placement(transformation(origin = { -60,10}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
      protected 
        Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_b1 annotation(Placement(transformation(extent = {{ -61, -11},{ -59, -9}}, rotation = 0)));
      public 
        Modelica.Mechanics.MultiBody.Visualizers.FixedShape gearwheel(widthDirection = {1,0,0}, width = 0.1, height = 0.1, r_shape = {0.2, -0.2,0}, extra = 20, length = 0.04, lengthDirection = {0.5,1, -2}, shapeType = "gearwheel", color = {155,155,155}) annotation(Placement(transformation(origin = { -32,10}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedShape spring(widthDirection = {1,0,0}, length = 0.1, width = 0.1, height = 0.02, r_shape = {0.4, -0.2,0}, extra = 4, lengthDirection = {0.5,1, -0.5}, shapeType = "spring", color = {155,155,155}) annotation(Placement(transformation(origin = { -2,10}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Mechanics.MultiBody.Visualizers.FixedShape cone(widthDirection = {1,0,0}, length = 0.1, width = 0.1, height = 0.1, r_shape = {0.6,0,0}, lengthDirection = {0.5,1, -0.5}, extra = 0.5, shapeType = "cone", color = {155,155,155}) annotation(Placement(transformation(origin = {20,70}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) annotation(Placement(transformation(extent = {{40,0},{60,20}}, rotation = 0)));

      equation 
        connect(fixed.frame_b,box.frame_a) annotation(Line(points = {{ -80,50},{ -60,50},{ -60,60}}));
        connect(fixed.frame_b,sphere.frame_a) annotation(Line(points = {{ -80,50},{ -30,50},{ -30,60}}));
        connect(fixed.frame_b,cylinder.frame_a) annotation(Line(points = {{ -80,50},{ -4,50},{ -4,60}}));
        connect(fixed.frame_b,pipe.frame_a) annotation(Line(points = {{ -80,50},{42,50},{42,60}}));
        connect(fixed.frame_b,frame_b1) annotation(Line(points = {{ -80,50},{ -74,50},{ -74, -10},{ -60, -10}}));
        connect(beam.frame_a,frame_b1) annotation(Line(points = {{ -60,0},{ -60, -10}}));
        connect(frame_b1,gearwheel.frame_a) annotation(Line(points = {{ -60, -10},{ -32, -10},{ -32,0}}));
        connect(frame_b1,spring.frame_a) annotation(Line(points = {{ -60, -10},{ -2, -10},{ -2,0}}));
        connect(fixed.frame_b,cone.frame_a) annotation(Line(points = {{ -80,50},{20,50},{20,60}}));
      end Shapes;
      encapsulated model Arrow "Test and show models Visualizers.FixedShape"
        import Modelica.Icons;
        import Modelica.Mechanics.MultiBody;
        import Modelica.Mechanics.MultiBody.Visualizers;
        import Modelica;
        extends Icons.Example;
        annotation(experiment(StopTime = 1), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
        Modelica.Mechanics.MultiBody.Parts.Fixed fixed3(animation = false, r = {0.2, -0.6,0}) annotation(Placement(transformation(origin = { -30, -10}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Visualizers.SignalArrow arrow(diameter = 0.02) annotation(Placement(transformation(origin = { -30,30}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Blocks.Sources.Sine sine[3](amplitude = {0.1,0.05,0.8}, freqHz = {2,1,2}, each offset = 0.2) annotation(Placement(transformation(extent = {{12,20},{ -8,40}}, rotation = 0)));
        Visualizers.FixedArrow fixedArrow(length = 0.3, color = {0,0,255}) annotation(Placement(transformation(origin = { -70,30}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, gravityType = 0) annotation(Placement(transformation(extent = {{ -100, -40},{ -80, -20}}, rotation = 0)));

      equation 
        connect(arrow.frame_a,fixed3.frame_b) annotation(Line(points = {{ -30,20},{ -30,0}}));
        connect(fixedArrow.frame_a,fixed3.frame_b) annotation(Line(points = {{ -70,20},{ -70,0},{ -30,0}}));
        connect(sine.y,arrow.r_head) annotation(Line(points = {{ -9,30},{ -18,30}}, color = {0,0,127}, smooth = Smooth.None));
      end Arrow;
    end Visualizers;
    package InitializationConversion "Test conversion of the initialization"
      model Rotor1D
        Modelica.Mechanics.MultiBody.Parts.Rotor1D rotor1D1 annotation(Placement(transformation(extent = {{ -70,40},{ -50,60}}, rotation = 0)));
        inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -100,20},{ -80,40}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1.1), experimentSetupOutput);
        Modelica.Mechanics.MultiBody.Parts.Rotor1D rotor1D2(stateSelect = StateSelect.prefer, a(fixed = false), phi(fixed = true, start = 1.5707963267949), w(fixed = true, start = 1.74532925199433)) annotation(Placement(transformation(extent = {{ -40,40},{ -20,60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Rotor1D rotor1D3(stateSelect = StateSelect.prefer, a(fixed = false), phi(fixed = true), w(fixed = false)) annotation(Placement(transformation(extent = {{ -10,40},{10,60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Rotor1D rotor1D4(stateSelect = StateSelect.prefer, a(fixed = false), phi(fixed = false), w(fixed = true)) annotation(Placement(transformation(extent = {{20,40},{40,60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Rotor1D rotor1D5(stateSelect = StateSelect.prefer, a(fixed = true), phi(fixed = false), w(fixed = true)) annotation(Placement(transformation(extent = {{ -60,0},{ -40,20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Rotor1D rotor1D6(stateSelect = StateSelect.prefer, a(fixed = false), phi(fixed = false), w(fixed = true)) annotation(Placement(transformation(extent = {{ -8,0},{12,20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Spring spring(c = 10000.0) annotation(Placement(transformation(extent = {{ -34,0},{ -14,20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Rotor1D rotor1D7(a(start = 0.0523598775598299, fixed = true), stateSelect = StateSelect.prefer, phi(fixed = false, start = 0.0174532925199433), w(fixed = true, start = 0.0349065850398866)) annotation(Placement(transformation(extent = {{ -60, -40},{ -40, -20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Rotor1D rotor1D8(stateSelect = StateSelect.prefer, a(fixed = false), phi(fixed = false), w(fixed = true)) annotation(Placement(transformation(extent = {{ -8, -40},{12, -20}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Spring spring1(c = 10000.0) annotation(Placement(transformation(extent = {{ -34, -40},{ -14, -20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Rotor1D rotor1D9(a(start = 0.0523598775598299, fixed = true), phi(fixed = true, start = 0.0174532925199433), w(fixed = true, start = 0.0349065850398866)) annotation(Placement(transformation(extent = {{ -60, -80},{ -40, -60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Rotor1D rotor1D10(stateSelect = StateSelect.prefer, a(fixed = false), phi(fixed = false), w(fixed = true)) annotation(Placement(transformation(extent = {{ -8, -80},{12, -60}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Spring spring2(c = 10000.0) annotation(Placement(transformation(extent = {{ -34, -80},{ -14, -60}}, rotation = 0)));

      equation 
        connect(world.frame_b,rotor1D1.frame_a) annotation(Line(points = {{ -80,30},{ -60,30},{ -60,40}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(world.frame_b,rotor1D2.frame_a) annotation(Line(points = {{ -80,30},{ -30,30},{ -30,40}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(world.frame_b,rotor1D3.frame_a) annotation(Line(points = {{ -80,30},{0,30},{0,40}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(world.frame_b,rotor1D4.frame_a) annotation(Line(points = {{ -80,30},{30,30},{30,40}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(world.frame_b,rotor1D5.frame_a) annotation(Line(points = {{ -80,30},{ -74,30},{ -74, -8},{ -50, -8},{ -50,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(rotor1D5.frame_a,rotor1D6.frame_a) annotation(Line(points = {{ -50,0},{ -50, -8},{2, -8},{2,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(spring.flange_a,rotor1D5.flange_b) annotation(Line(points = {{ -34,10},{ -40,10}}, color = {0,0,0}, smooth = Smooth.None));
        connect(spring.flange_b,rotor1D6.flange_a) annotation(Line(points = {{ -14,10},{ -8,10}}, color = {0,0,0}, smooth = Smooth.None));
        connect(world.frame_b,rotor1D7.frame_a) annotation(Line(points = {{ -80,30},{ -74,30},{ -74, -48},{ -50, -48},{ -50, -40}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(rotor1D7.frame_a,rotor1D8.frame_a) annotation(Line(points = {{ -50, -40},{ -50, -48},{2, -48},{2, -40}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(spring1.flange_a,rotor1D7.flange_b) annotation(Line(points = {{ -34, -30},{ -40, -30}}, color = {0,0,0}, smooth = Smooth.None));
        connect(spring1.flange_b,rotor1D8.flange_a) annotation(Line(points = {{ -14, -30},{ -8, -30}}, color = {0,0,0}, smooth = Smooth.None));
        connect(world.frame_b,rotor1D9.frame_a) annotation(Line(points = {{ -80,30},{ -74,30},{ -74, -88},{ -50, -88},{ -50, -80}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(rotor1D9.frame_a,rotor1D10.frame_a) annotation(Line(points = {{ -50, -80},{ -50, -88},{2, -88},{2, -80}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(spring2.flange_a,rotor1D9.flange_b) annotation(Line(points = {{ -34, -70},{ -40, -70}}, color = {0,0,0}, smooth = Smooth.None));
        connect(spring2.flange_b,rotor1D10.flange_a) annotation(Line(points = {{ -14, -70},{ -8, -70}}, color = {0,0,0}, smooth = Smooth.None));
      end Rotor1D;
      model Joints
        inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Revolute revolute(a(start = 0.174532925199433, fixed = false), stateSelect = StateSelect.always, phi(fixed = true, start = 0.785398163397448), w(fixed = true, start = 0.174532925199433)) annotation(Placement(transformation(extent = {{ -48,60},{ -28,80}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 0.2), experimentSetupOutput);
        Modelica.Mechanics.MultiBody.Parts.BodyBox bodyBox(r = {1,0,0}) annotation(Placement(transformation(extent = {{ -20,60},{0,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Revolute actuatedRevolute(a(start =  -0.174532925199433, fixed = false), stateSelect = StateSelect.always, phi(fixed = true, start = 0.523598775598299), w(fixed = false, start =  -0.174532925199433)) annotation(Placement(transformation(extent = {{8,60},{28,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyBox bodyBox1(r = {1,0,0}) annotation(Placement(transformation(extent = {{36,60},{56,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Prismatic prismatic(stateSelect = StateSelect.always, a(fixed = false, start = 1), s(fixed = true, start = 1), v(fixed = true, start = 1), s_offset = 0.1) annotation(Placement(transformation(extent = {{ -48,20},{ -28,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyBox bodyBox2(r = {1,0,0}) annotation(Placement(transformation(extent = {{ -20,20},{0,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Prismatic actuatedPrismatic(stateSelect = StateSelect.always, a(fixed = false, start = 0.6), s(fixed = false, start = 0.2), v(fixed = true, start = 0.4)) annotation(Placement(transformation(extent = {{8,20},{28,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Cylindrical cylindrical(stateSelect = StateSelect.always, a(fixed = false, start = 4), phi(fixed = true, start = 1.0471975511966), s(fixed = true, start = 1), v(fixed = true, start = 3), w(fixed = true, start = 0.0349065850398866), wd(fixed = false, start = 0.0872664625997165)) annotation(Placement(transformation(extent = {{ -50, -20},{ -30,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyBox bodyBox3(r = {1,0,0}) annotation(Placement(transformation(extent = {{40,20},{60,40}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyBox bodyBox4(r = {1,0,0}) annotation(Placement(transformation(extent = {{ -20, -20},{0,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Universal universal(a_a(start = 0.523598775598299, fixed = false), a_b(start =  -0.523598775598299, fixed = false), stateSelect = StateSelect.always, phi_a(fixed = true, start =  -0.174532925199433), phi_b(fixed = true, start = 0.174532925199433), w_a(fixed = false, start = 0.349065850398866), w_b(fixed = false, start =  -0.349065850398866)) annotation(Placement(transformation(extent = {{8, -20},{28,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyBox bodyBox5(r = {1,0,0}) annotation(Placement(transformation(extent = {{38, -20},{58,0}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Planar planar(stateSelect = StateSelect.always, a_x(fixed = false, start = 2), a_y(fixed = false, start = 1), phi(fixed = true, start = 0.0523598775598299), s_x(fixed = true, start = 1), s_y(fixed = true, start = 2), v_x(fixed = true, start = 1), v_y(fixed = true, start = 2), w(fixed = true, start = 0.0174532925199433), wd(fixed = false, start = 0.0349065850398866)) annotation(Placement(transformation(extent = {{ -52, -50},{ -32, -30}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyBox bodyBox6(r = {1,0,0}) annotation(Placement(transformation(extent = {{ -20, -50},{0, -30}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Spherical spherical(sequence_start = {3,1,3}, angles_start = {0.349065850398866, -0.523598775598299,0.698131700797732}, angles_fixed = true, w_rel_a_fixed = true, z_rel_a_fixed = false, w_rel_a_start = {0.0174532925199433,0.0349065850398866,0.0523598775598299}, z_rel_a_start = {0.0349065850398866,0.0523598775598299,0.0698131700797732}) annotation(Placement(transformation(extent = {{8, -50},{28, -30}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyBox bodyBox7(r = {1,0,0}) annotation(Placement(transformation(extent = {{38, -50},{58, -30}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyBox bodyBox8(r = {1,0,0}) annotation(Placement(transformation(extent = {{ -20, -80},{0, -60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.FreeMotion freeMotion(angles_start = {0.349065850398866,0.523598775598299,0.698131700797732}, a_rel_a(fixed = false), angles_fixed = true, r_rel_a(fixed = true, start = {1,1,1}), v_rel_a(fixed = true), w_rel_a_fixed = true, z_rel_a_fixed = false) annotation(Placement(transformation(extent = {{ -56, -80},{ -36, -60}}, rotation = 0)));

      equation 
        connect(world.frame_b,revolute.frame_a) annotation(Line(points = {{ -60,70},{ -48,70}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(revolute.frame_b,bodyBox.frame_a) annotation(Line(points = {{ -28,70},{ -20,70}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(bodyBox.frame_b,actuatedRevolute.frame_a) annotation(Line(points = {{0,70},{8,70}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(actuatedRevolute.frame_b,bodyBox1.frame_a) annotation(Line(points = {{28,70},{36,70}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(bodyBox1.frame_b,prismatic.frame_a) annotation(Line(points = {{56,70},{66,70},{66,46},{ -54,46},{ -54,30},{ -48,30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(prismatic.frame_b,bodyBox2.frame_a) annotation(Line(points = {{ -28,30},{ -20,30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(bodyBox2.frame_b,actuatedPrismatic.frame_a) annotation(Line(points = {{0,30},{8,30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(actuatedPrismatic.frame_b,bodyBox3.frame_a) annotation(Line(points = {{28,30},{40,30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(bodyBox3.frame_b,cylindrical.frame_a) annotation(Line(points = {{60,30},{74,30},{74,8},{ -60,8},{ -60, -10},{ -50, -10}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(cylindrical.frame_b,bodyBox4.frame_a) annotation(Line(points = {{ -30, -10},{ -20, -10}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(bodyBox4.frame_b,universal.frame_a) annotation(Line(points = {{0, -10},{8, -10}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(universal.frame_b,bodyBox5.frame_a) annotation(Line(points = {{28, -10},{38, -10}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(bodyBox5.frame_b,planar.frame_a) annotation(Line(points = {{58, -10},{72, -10},{72, -26},{ -60, -26},{ -60, -40},{ -52, -40}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(planar.frame_b,bodyBox6.frame_a) annotation(Line(points = {{ -32, -40},{ -20, -40}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(bodyBox6.frame_b,spherical.frame_a) annotation(Line(points = {{0, -40},{8, -40}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(spherical.frame_b,bodyBox7.frame_a) annotation(Line(points = {{28, -40},{38, -40}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(freeMotion.frame_a,bodyBox7.frame_b) annotation(Line(points = {{ -56, -70},{ -60, -70},{ -60, -54},{70, -54},{70, -40},{58, -40}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        connect(freeMotion.frame_b,bodyBox8.frame_a) annotation(Line(points = {{ -36, -70},{ -20, -70}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
      end Joints;
      model Bodies
        inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.Body body(sequence_start = {3,2,1}, angles_start = {0.349065850398866,0.523598775598299,0.698131700797732}, a_0(fixed = false, start = {2,2,2}), angles_fixed = true, r_0(fixed = true, start = {1,2,3}), v_0(fixed = true, start = {1,2,3}), w_0_fixed = true, z_0_fixed = false, w_0_start = {0.0698131700797732,0.0523598775598299,0.0349065850398866}, z_0_start = {0.0523598775598299,0.0698131700797732,0.0523598775598299}) annotation(Placement(transformation(extent = {{ -42,40},{ -22,60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.PointMass pointMass(m = 2, a_0(fixed = false, start = {5,4,3}), r_0(fixed = true, start = {1,2,3}), v_0(fixed = true, start = {2,3,4})) annotation(Placement(transformation(extent = {{ -2, -2},{18,18}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyShape body1(sequence_start = {3,2,1}, angles_start = {0.349065850398866,0.523598775598299,0.698131700797732}, a_0(fixed = false, start = {2,2,2}), angles_fixed = true, r_0(fixed = true, start = {1,2,3}), v_0(fixed = true, start = {1,2,3}), w_0_fixed = true, z_0_fixed = false, w_0_start = {0.0698131700797732,0.0523598775598299,0.0349065850398866}, z_0_start = {0.0523598775598299,0.0698131700797732,0.0523598775598299}) annotation(Placement(transformation(extent = {{0,40},{20,60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyBox body2(sequence_start = {3,2,1}, angles_start = {0.349065850398866,0.523598775598299,0.698131700797732}, a_0(fixed = false, start = {2,2,2}), angles_fixed = true, r_0(fixed = true, start = {1,2,3}), v_0(fixed = true, start = {1,2,3}), w_0_fixed = true, z_0_fixed = false, w_0_start = {0.0698131700797732,0.0523598775598299,0.0349065850398866}, z_0_start = {0.0523598775598299,0.0698131700797732,0.0523598775598299}) annotation(Placement(transformation(extent = {{42,40},{62,60}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Parts.BodyCylinder body3(sequence_start = {3,2,1}, angles_start = {0.349065850398866,0.523598775598299,0.698131700797732}, a_0(fixed = false, start = {2,2,2}), angles_fixed = true, r_0(fixed = true, start = {1,2,3}), v_0(fixed = true, start = {1,2,3}), w_0_fixed = true, z_0_fixed = false, w_0_start = {0.0698131700797732,0.0523598775598299,0.0349065850398866}, z_0_start = {0.0523598775598299,0.0698131700797732,0.0523598775598299}) annotation(Placement(transformation(extent = {{ -44,0},{ -24,20}}, rotation = 0)));
        annotation(experiment(StopTime = 0.2), experimentSetupOutput);
      end Bodies;
      partial model Internal "Check whether Advanced/axisTorqueBalance,axisForceBalance parameter is correctely converted"
        ObsoleteModelica3.Mechanics.MultiBody.Joints.Internal.RevoluteWithLengthConstraint revoluteWithLengthConstraint1 annotation(Placement(transformation(extent = {{ -60,38},{ -40,58}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Joints.Internal.PrismaticWithLengthConstraint prismaticWithLengthConstraint1 annotation(Placement(transformation(extent = {{2,38},{22,58}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Joints.Internal.RevoluteWithLengthConstraint revoluteWithLengthConstraint2 annotation(Placement(transformation(extent = {{ -58,0},{ -38,20}}, rotation = 0)));
        ObsoleteModelica3.Mechanics.MultiBody.Joints.Internal.PrismaticWithLengthConstraint prismaticWithLengthConstraint2 annotation(Placement(transformation(extent = {{4,0},{24,20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Internal.RevoluteWithLengthConstraint revoluteWithLengthConstraint3 annotation(Placement(transformation(extent = {{ -58, -40},{ -38, -20}}, rotation = 0)));
        Modelica.Mechanics.MultiBody.Joints.Internal.PrismaticWithLengthConstraint prismaticWithLengthConstraint3 annotation(Placement(transformation(extent = {{4, -40},{24, -20}}, rotation = 0)));
        annotation(Diagram(graphics));
      end Internal;
    end InitializationConversion;
  end MultiBody;
  package Media
    package TestAllProperties
      model PartialMediumFunctions
        import SI = Modelica.SIunits;
        replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium model";
        parameter Real eps_h_is = 1e-10;
        Medium.ThermodynamicState state = Medium.setState_pTX(Medium.reference_p, Medium.reference_T, Medium.reference_X);
        Medium.DynamicViscosity eta = Medium.dynamicViscosity(state);
        Medium.ThermalConductivity lambda = Medium.thermalConductivity(state);
        Medium.PrandtlNumber Pr = Medium.prandtlNumber(state);
        Medium.AbsolutePressure p = Medium.pressure(state);
        Medium.Temperature T = Medium.temperature(state);
        Medium.Density d = Medium.density(state);
        Medium.SpecificEnthalpy h = Medium.specificEnthalpy(state);
        Medium.SpecificEnergy u = Medium.specificInternalEnergy(state);
        Medium.SpecificEntropy s = Medium.specificEntropy(state);
        Medium.SpecificEnergy g = Medium.specificGibbsEnergy(state);
        Medium.SpecificEnergy f = Medium.specificHelmholtzEnergy(state);
        Medium.SpecificHeatCapacity cp = Medium.specificHeatCapacityCp(state);
        Medium.SpecificHeatCapacity cp2 = Medium.heatCapacity_cp(state);
        Medium.SpecificHeatCapacity cv = Medium.specificHeatCapacityCv(state);
        Medium.SpecificHeatCapacity cv2 = Medium.heatCapacity_cv(state);
        Medium.SpecificEnthalpy h_is = Medium.isentropicEnthalpy(2 * Medium.reference_p, state);
        Medium.VelocityOfSound a = Medium.velocityOfSound(state);
        Medium.IsobaricExpansionCoefficient beta = Medium.isobaricExpansionCoefficient(state);
        Medium.IsobaricExpansionCoefficient beta2 = Medium.beta(state);
        SI.IsothermalCompressibility kappa = Medium.isothermalCompressibility(state);
        SI.IsothermalCompressibility kappa2 = Medium.kappa(state);
        Medium.DerDensityByPressure ddpT = Medium.density_derp_T(state);
        Medium.DerDensityByTemperature ddTp = Medium.density_derT_p(state);
        Medium.Density dddX[Medium.nX] = Medium.density_derX(state);
        Medium.MolarMass MM = Medium.molarMass(state);
        Medium.SpecificEnthalpy h2 = Medium.specificEnthalpy_pTX(Medium.reference_p, Medium.reference_T, Medium.reference_X);
        Medium.Density d2 = Medium.density_pTX(Medium.reference_p, Medium.reference_T, Medium.reference_X);
        Medium.ThermodynamicState state2 = Medium.setState_phX(Medium.reference_p, h, Medium.reference_X);
        Medium.ThermodynamicState state3 = Medium.setState_psX(Medium.reference_p, s, Medium.reference_X);
        Medium.ThermodynamicState state4 = if Medium.singleState then state else Medium.setState_dTX(d, T, Medium.reference_X);
        Medium.ThermodynamicState state5 = Medium.setSmoothState(0.1, state, state2, 0.001);
        Medium.BaseProperties medium(preferredMediumStates = true, p(start = Medium.reference_p), T(start = Medium.reference_T), X(start = Medium.reference_X));
        Real err_T = abs(medium.T - T);
        Real err_d = abs(medium.d - d);
        Real err_u = abs(medium.u - u);
        Medium.ThermodynamicState state_h_is = Medium.setState_phX(2 * Medium.reference_p, h_is, Medium.reference_X);
        Medium.SpecificEntropy s_is = Medium.specificEntropy(state_h_is);
        Real err_h_is = abs(s - s_is);
        constant Real eps = 1e-10;

      equation 
        medium.p = p;
        medium.h = h;
        medium.Xi = Medium.reference_X[1:Medium.nXi];
        when terminal() then
                  assert(err_T <= eps, "Error: abs(medium.T - T) > eps");
        
        end when;
        when terminal() then
                  assert(err_d <= eps, "Error: abs(medium.d - d) > eps");
        
        end when;
        when terminal() then
                  assert(err_u <= eps, "Error: abs(medium.u - u) > eps");
        
        end when;
        assert(err_h_is <= eps_h_is, "Error: entropy not constant for isentropicEnthalpy");
      end PartialMediumFunctions;
      model DryAirNasa
        extends PartialMediumFunctions(redeclare package Medium = Modelica.Media.Air.DryAirNasa, eps_h_is = 0.1);
        annotation(experiment(StopTime = 0.1), experimentSetupOutput);
      end DryAirNasa;
      model LinearColdWater
        extends PartialMediumFunctions(redeclare package Medium = Modelica.Media.CompressibleLiquids.LinearColdWater);
        annotation(experiment(StopTime = 0.1), experimentSetupOutput);
      end LinearColdWater;
      model LinearWater_pT_Ambient
        extends PartialMediumFunctions(redeclare package Medium = Modelica.Media.CompressibleLiquids.LinearWater_pT_Ambient);
        annotation(experiment(StopTime = 0.1), experimentSetupOutput);
      end LinearWater_pT_Ambient;
      model CO2
        extends PartialMediumFunctions(redeclare package Medium = Modelica.Media.IdealGases.SingleGases.CO2, eps_h_is = 0.2);
        annotation(experiment(StopTime = 0.1), experimentSetupOutput);
      end CO2;
      model FlueGasSixComponents
        extends PartialMediumFunctions(redeclare package Medium = Modelica.Media.IdealGases.MixtureGases.FlueGasSixComponents);
        annotation(experiment(StopTime = 0.1), experimentSetupOutput);
      end FlueGasSixComponents;
      model ConstantPropertyLiquidWater
        extends PartialMediumFunctions(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater);
        annotation(experiment(StopTime = 0.1), experimentSetupOutput);
      end ConstantPropertyLiquidWater;
      model SimpleAir
        extends PartialMediumFunctions(redeclare package Medium = Modelica.Media.Air.SimpleAir);
        annotation(experiment(StopTime = 0.1), experimentSetupOutput);
      end SimpleAir;
      package IncompleteMedia
        model MoistAir
          extends PartialMediumFunctions(redeclare package Medium = Modelica.Media.Air.MoistAir);
        end MoistAir;
        model Glycol47
          extends PartialMediumFunctions(redeclare package Medium = Modelica.Media.Incompressible.Examples.Glycol47);
        end Glycol47;
        model WaterIF97_pT
          extends PartialMediumFunctions(redeclare package Medium = Modelica.Media.Water.WaterIF97_pT);
        end WaterIF97_pT;
        model WaterIF97_ph
          extends PartialMediumFunctions(redeclare package Medium = Modelica.Media.Water.WaterIF97_ph);
        end WaterIF97_ph;
      end IncompleteMedia;
    end TestAllProperties;
  end Media;
  package Math "Test models for Modelica.Math"
    function Polynomials "Test functions of Modelica.Media.Incompressible.TableBased.Polynomials_Temp"
      import Modelica.Utilities.Streams;
      import Poly = Modelica.Media.Incompressible.TableBased.Polynomials_Temp;
      input String logFile = "ModelicaTestLog.txt" "Filename where the log is stored";
      output Boolean ok;
    protected 
      Real p1[4] = { -2, -3, -4, -1};
      Real p2[5];
      Real p3[4];
      Real p4[4];
      Real r;
      Real r1;
      Real r2;
      Real u[7] = {0,1,2,3,4,5,6};
      Real y[7];
    algorithm 
      Streams.print("... Test of Modelica.Media.Incompressible.TableBased.Polynomials_Temp", logFile);
      r:=Poly.evaluate(p1,  -3);
      assert(r == 38, "Polynomials.evaluate failed");
      p2:=Poly.integral(p1);
      assert(p2[1] ==  -0.5 and p2[2] ==  -1 and p2[3] ==  -2 and p2[4] ==  -1 and p2[5] == 0, "Polynomials.integral failed");
      p3:=Poly.derivative(p2);
      assert(p3[1] == p1[1] and p3[2] == p1[2] and p3[3] == p1[3] and p3[4] == p1[4], "Polynomials.derivative failed");
      r1:=Poly.derivativeValue(p2,  -3);
      r2:=Poly.evaluate(p3,  -3);
      assert(r1 == r2, "Polynomials.derivativeValue failed");
      r:=Poly.integralValue(p1, 2, 1);
      assert(r ==  -21.5, "Polynomials.integralValue failed");
      for i in 1:size(u, 1) loop
              y[i]:=Poly.evaluate(p1, u[i]) + 0.01 * i;

      end for;
      p4:=Poly.fitting(u, y, 3);
      assert(abs(p4[1] - p1[1]) <= 1e-08 and abs(p4[2] - p1[2]) <= 1e-08 and abs(p4[3] - p1[3]) <= 0.1 and abs(p4[4] - p1[4]) <= 0.1, "Polynomials.fitting failed");
      ok:=true;
    end Polynomials;
    function Matrices "Test functions of Modelica.Math.Matrices"
      import Modelica.Utilities.Streams;
      input String logFile = "ModelicaTestLog.txt" "Filename where the log is stored";
      output Boolean ok;
    protected 
      Real A1[2,3] = [1,2,3;4,5,6];
      Real A2[2,3] = [1,0,0;4,0,0];
      Real b[2] = {0, -2};
      Real A3[2,3] = [1,2,3;4,5,6];
      Real a3[2] = {0, -2};
      Real B3[1,3] = [4,3,0];
      Real b3[1] = {2};
      Real A4[2,4] = [1,2,3,4;4,5,6,7];
      Real a4[2] = {0, -2};
      Real B4[1,4] = [4,3,0, -1];
      Real b4[1] = {2};
      Real x1[3];
      Real x2[3];
      Real x3[3];
      Real e3[1] = B3 * x3 - b3;
    algorithm 
      Streams.print("... Test of Modelica.Math.Matrices", logFile);
      x1:=Modelica.Math.Matrices.leastSquares(A1, b);
      x2:=Modelica.Math.Matrices.leastSquares(A2, b);
      x3:=Modelica.Math.Matrices.equalityLeastSquares(A3, a3, B3, b3);
      ok:=true;
    end Matrices;
  end Math;
  package Blocks "Test models for Modelica.Blocks"
    model Continuous
      Modelica.Blocks.Continuous.Integrator integrator annotation(Placement(transformation(extent = {{ -60,60},{ -40,80}}, rotation = 0)));
      Modelica.Blocks.Continuous.LimIntegrator limIntegrator annotation(Placement(transformation(extent = {{ -60,20},{ -40,40}}, rotation = 0)));
      annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -200},{100,100}}), graphics), experiment(StopTime = 1.1));
      Modelica.Blocks.Continuous.Derivative derivative annotation(Placement(transformation(extent = {{ -60, -20},{ -40,0}}, rotation = 0)));
      Modelica.Blocks.Continuous.FirstOrder firstOrder annotation(Placement(transformation(extent = {{ -60, -60},{ -40, -40}}, rotation = 0)));
      Modelica.Blocks.Continuous.SecondOrder secondOrder annotation(Placement(transformation(extent = {{ -60, -100},{ -40, -80}}, rotation = 0)));
      Modelica.Blocks.Continuous.PI pI annotation(Placement(transformation(extent = {{ -60, -140},{ -40, -120}}, rotation = 0)));
      Modelica.Blocks.Continuous.PID pID annotation(Placement(transformation(extent = {{ -60, -180},{ -40, -160}}, rotation = 0)));
      Modelica.Blocks.Continuous.LimPID limPID annotation(Placement(transformation(extent = {{40,60},{60,80}}, rotation = 0)));
      Modelica.Blocks.Continuous.TransferFunction transferFunction(a = {1,1}, b = {1}) annotation(Placement(transformation(extent = {{40, -20},{60,0}}, rotation = 0)));
      Modelica.Blocks.Continuous.StateSpace stateSpace(A = [1,0;0,1], B = [1;1], C = [1,1]) annotation(Placement(transformation(extent = {{40, -60},{60, -40}}, rotation = 0)));
      Modelica.Blocks.Continuous.LowpassButterworth lowpassButterworth annotation(Placement(transformation(extent = {{40, -100},{60, -80}}, rotation = 0)));
      Modelica.Blocks.Continuous.CriticalDamping criticalDamping(normalized = false) annotation(Placement(transformation(extent = {{40, -140},{60, -120}}, rotation = 0)));
      Modelica.Blocks.Sources.Ramp ramp(offset = 0.5, startTime = 0.5) annotation(Placement(transformation(extent = {{ -100,60},{ -80,80}}, rotation = 0)));
      Modelica.Blocks.Sources.Ramp ramp1(offset = 0.5, startTime = 0.5) annotation(Placement(transformation(extent = {{0,60},{20,80}}, rotation = 0)));
      Modelica.Blocks.Sources.Constant const(k = 1.5) annotation(Placement(transformation(origin = {50,30}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
      Modelica.Blocks.Continuous.Der der1 annotation(Placement(transformation(extent = {{40, -180},{60, -160}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine annotation(Placement(transformation(extent = {{0, -180},{20, -160}}, rotation = 0)));

    equation 
      connect(ramp.y,integrator.u) annotation(Line(points = {{ -79,70},{ -62,70}}, color = {0,0,127}));
      connect(ramp.y,limIntegrator.u) annotation(Line(points = {{ -79,70},{ -72,70},{ -72,30},{ -62,30}}, color = {0,0,127}));
      connect(ramp.y,derivative.u) annotation(Line(points = {{ -79,70},{ -72,70},{ -72, -10},{ -62, -10}}, color = {0,0,127}));
      connect(ramp.y,firstOrder.u) annotation(Line(points = {{ -79,70},{ -72,70},{ -72, -50},{ -62, -50}}, color = {0,0,127}));
      connect(ramp.y,secondOrder.u) annotation(Line(points = {{ -79,70},{ -72,70},{ -72, -90},{ -62, -90}}, color = {0,0,127}));
      connect(ramp.y,pI.u) annotation(Line(points = {{ -79,70},{ -72,70},{ -72, -130},{ -62, -130}}, color = {0,0,127}));
      connect(ramp.y,pID.u) annotation(Line(points = {{ -79,70},{ -72,70},{ -72, -170},{ -62, -170}}, color = {0,0,127}));
      connect(ramp1.y,limPID.u_s) annotation(Line(points = {{21,70},{38,70}}, color = {0,0,127}));
      connect(ramp1.y,transferFunction.u) annotation(Line(points = {{21,70},{28,70},{28, -10},{38, -10}}, color = {0,0,127}));
      connect(ramp1.y,stateSpace.u[1]) annotation(Line(points = {{21,70},{28,70},{28, -50},{38, -50}}, color = {0,0,127}));
      connect(ramp1.y,lowpassButterworth.u) annotation(Line(points = {{21,70},{28,70},{28, -90},{38, -90}}, color = {0,0,127}));
      connect(ramp1.y,criticalDamping.u) annotation(Line(points = {{21,70},{28,70},{28, -130},{38, -130}}, color = {0,0,127}));
      connect(const.y,limPID.u_m) annotation(Line(points = {{50,41},{50,58}}, color = {0,0,127}));
      connect(sine.y,der1.u) annotation(Line(points = {{21, -170},{38, -170}}, color = {0,0,127}, smooth = Smooth.None));
    end Continuous;
    model Continuous_SteadyState
      extends ModelicaTest.Blocks.Continuous(derivative(initType = Modelica.Blocks.Types.Init.SteadyState), firstOrder(initType = Modelica.Blocks.Types.Init.SteadyState), secondOrder(initType = Modelica.Blocks.Types.Init.SteadyState), transferFunction(initType = Modelica.Blocks.Types.Init.SteadyState, a = {1,1}, b = {1}), stateSpace(initType = Modelica.Blocks.Types.Init.SteadyState, A = [1,0;0,1], B = [1;1], C = [1,1]), lowpassButterworth(initType = Modelica.Blocks.Types.Init.SteadyState), criticalDamping(initType = Modelica.Blocks.Types.Init.SteadyState, normalized = false));
      annotation(experiment(StopTime = 1.1));
    end Continuous_SteadyState;
    model Continuous_InitialState
      extends ModelicaTest.Blocks.Continuous(integrator(initType = Modelica.Blocks.Types.Init.InitialState), limIntegrator(initType = Modelica.Blocks.Types.Init.InitialState), derivative(initType = Modelica.Blocks.Types.Init.InitialState), firstOrder(initType = Modelica.Blocks.Types.Init.InitialState), secondOrder(initType = Modelica.Blocks.Types.Init.InitialState), pI(initType = Modelica.Blocks.Types.Init.InitialState), pID(initType = Modelica.Blocks.Types.Init.InitialState), limPID(initType = Modelica.Blocks.Types.Init.InitialState), transferFunction(initType = Modelica.Blocks.Types.Init.InitialState, a = {1,1}, b = {1}), stateSpace(initType = Modelica.Blocks.Types.Init.InitialState, A = [1,0;0,1], B = [1;1], C = [1,1]), lowpassButterworth(initType = Modelica.Blocks.Types.Init.InitialState), criticalDamping(initType = Modelica.Blocks.Types.Init.InitialState, normalized = false));
      annotation(experiment(StopTime = 1.1));
    end Continuous_InitialState;
    model Continuous_InitialOutput
      extends ModelicaTest.Blocks.Continuous(integrator(initType = Modelica.Blocks.Types.Init.InitialOutput, y_start = 2), limIntegrator(initType = Modelica.Blocks.Types.Init.InitialOutput, y_start = 0.5), derivative(initType = Modelica.Blocks.Types.Init.InitialOutput, y_start = 2), firstOrder(initType = Modelica.Blocks.Types.Init.InitialOutput, y_start = 2), secondOrder(initType = Modelica.Blocks.Types.Init.InitialOutput, y_start = 2), pI(initType = Modelica.Blocks.Types.Init.InitialOutput, y_start = 2), pID(initType = Modelica.Blocks.Types.Init.InitialOutput, y_start = 2), limPID(initType = Modelica.Blocks.Types.Init.InitialOutput, yMax = 3, y_start = 2), transferFunction(initType = Modelica.Blocks.Types.Init.InitialOutput, y_start = 2, a = {1,1}, b = {1}), stateSpace(initType = Modelica.Blocks.Types.Init.InitialOutput, y_start = {2}, A = [1,0;0,1], B = [1;1], C = [1,1]), lowpassButterworth(initType = Modelica.Blocks.Types.Init.InitialOutput, y_start = 2), criticalDamping(initType = Modelica.Blocks.Types.Init.InitialOutput, y_start = 2, normalized = false));
      annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -200},{100,100}}), graphics), experiment(StopTime = 1.1));
    end Continuous_InitialOutput;
    model Limiters
      Modelica.Blocks.Nonlinear.Limiter limiter(limitsAtInit = false) annotation(Placement(transformation(extent = {{ -20,40},{0,60}}, rotation = 0)));
      Modelica.Blocks.Nonlinear.VariableLimiter variableLimiter(limitsAtInit = false) annotation(Placement(transformation(extent = {{ -20,0},{0,20}}, rotation = 0)));
      Modelica.Blocks.Nonlinear.DeadZone deadZone(deadZoneAtInit = false) annotation(Placement(transformation(extent = {{ -20, -40},{0, -20}}, rotation = 0)));
      Modelica.Blocks.Sources.Sine sine(amplitude = 2) annotation(Placement(transformation(extent = {{ -60,40},{ -40,60}}, rotation = 0)));
      annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1.1));
      Modelica.Blocks.Sources.Constant const annotation(Placement(transformation(extent = {{ -80,16},{ -60,36}}, rotation = 0)));
      Modelica.Blocks.Sources.Constant const1(k =  -1) annotation(Placement(transformation(extent = {{ -80, -26},{ -60, -6}}, rotation = 0)));

    equation 
      connect(sine.y,limiter.u) annotation(Line(points = {{ -39,50},{ -22,50}}, color = {0,0,127}));
      connect(const.y,variableLimiter.limit1) annotation(Line(points = {{ -59,26},{ -42,26},{ -42,18},{ -22,18}}, color = {0,0,127}));
      connect(const1.y,variableLimiter.limit2) annotation(Line(points = {{ -59, -16},{ -40, -16},{ -40,2},{ -22,2}}, color = {0,0,127}));
      connect(sine.y,variableLimiter.u) annotation(Line(points = {{ -39,50},{ -32,50},{ -32,10},{ -22,10}}, color = {0,0,127}));
      connect(sine.y,deadZone.u) annotation(Line(points = {{ -39,50},{ -32,50},{ -32, -30},{ -22, -30}}, color = {0,0,127}));
    end Limiters;
    model KinematicPTP
      Modelica.Blocks.Sources.KinematicPTP kinematicPTP1a(qd_max = {3}, qdd_max = {4}, startTime = 0.5, deltaq = {1}) annotation(Placement(transformation(extent = {{ -60,60},{ -40,80}}, rotation = 0)));
      Modelica.Blocks.Continuous.Integrator integrator annotation(Placement(transformation(extent = {{ -20,60},{2,80}}, rotation = 0)));
      annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 3));
      Modelica.Blocks.Continuous.Integrator integrator1(y_start = 1) annotation(Placement(transformation(extent = {{20,60},{42,80}}, rotation = 0)));
      Modelica.Blocks.Sources.KinematicPTP kinematicPTP2a(qd_max = {3}, qdd_max = {4}, startTime = 0.5, deltaq = {4}) annotation(Placement(transformation(extent = {{ -60, -20},{ -40,0}}, rotation = 0)));
      Modelica.Blocks.Continuous.Integrator integrator2 annotation(Placement(transformation(extent = {{ -20, -20},{2,0}}, rotation = 0)));
      Modelica.Blocks.Continuous.Integrator integrator3(y_start = 1) annotation(Placement(transformation(extent = {{20, -20},{42,0}}, rotation = 0)));

    equation 
      connect(kinematicPTP1a.y[1],integrator.u) annotation(Line(points = {{ -39,70},{ -22.2,70}}, color = {0,0,127}));
      connect(integrator.y,integrator1.u) annotation(Line(points = {{3.1,70},{17.8,70}}, color = {0,0,127}));
      connect(kinematicPTP2a.y[1],integrator2.u) annotation(Line(points = {{ -39, -10},{ -22.2, -10}}, color = {0,0,127}));
      connect(integrator2.y,integrator3.u) annotation(Line(points = {{3.1, -10},{17.8, -10}}, color = {0,0,127}));
    end KinematicPTP;
  end Blocks;
end ModelicaTest;
package ModelicaReference "Modelica Reference"
  annotation(__Dymola_DocumentationClass = true, versionBuild = "$Rev: 3044 $", versionDate = "$Date:: 2009-10-08 09:31:41 +0200 #$", Documentation(info = "<html>
<p>
This package is a reference to Modelica keywords and Modelica builtin
operators and is based on the
<a href=\"http://www.modelica.org/documents/ModelicaSpec30.pdf\">Modelica Language Specification version 3.0</a> from Sept. 2007.


<dl>
<dt><b>Main Author:</b></dt>
<dd><a href=\"http://www.robotic.dlr.de/Christian.Schweiger/\">Christian.Schweiger</a><br>
    Deutsches Zentrum f&uuml;r Luft und Raumfahrt e.V. (DLR)<br>
    Institut f&uuml;r Robotik und Mechatronik<br>
    Postfach 11 16<br>
    D-82230 Wessling<br>
    Germany<br>
    email: <A HREF=\"mailto:Christian.Schweiger@dlr.de\">Christian.Schweiger@dlr.de</A><br></dd>
</dl>

<p>
<b>Copyright &copy; 2003-2004, 2008 Modelica Association and DLR.</b>
</p>
<p>
<i>The <b>ModelicaReference</b> package is <b>free</b> software;
it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p><br>
</html>", revisions = "<html>

<ul>
<li><i>Jan. 2, 2008</i>
    by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
    Adapted to Modelica language version 3.0</li>

<li><i>Sept. 30, 2004</i>
    by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
    Moved the content of \"Functions\" into \"Operators\" and updated
    \"Operators\" according to Modelica 2.1</li>

<li><i>July 10, 2003</i>
     by <a href=\"http://www.robotic.dlr.de/Christian.Schweiger/\">Christian Schweiger</a>:<br>
     Implemented.</li>
</ul>
</html>"));
  package Annotations "Annotations"
    class choices "choices"
      annotation(Documentation(info = "<html>
<p>
Define graphical layout of choices in a parameter menu
</p>

<h4><font color=\"#008000\">Examples</font></h4>

<pre><b>replaceable model</b> MyResistor=Resistor
  <b>annotation</b>(choices(
              choice(redeclare MyResistor=lib2.Resistor(a={2}) \"...\"),
              choice(redeclare MyResistor=lib2.Resistor2 \"...\")));

<b>replaceable</b> Resistor Load(R=2) constrainedby TwoPin
  <b>annotation</b>(choices(
              choice(redeclare lib2.Resistor Load(a={2}) \"...\"),
              choice(redeclare Capacitor Load(L=3) \"...\")));

<b>replaceable</b> FrictionFunction a(func=exp) constrainedby Friction
  <b>annotation</b>(choices(
             choice(redeclare ConstantFriction a(c=1) \"...\"),
             choice(redeclare TableFriction a(table=\"...\") \"...\"),
             choice(redeclare FunctionFriction a(func=exp) \"...\"))));

<b>type</b> KindOfController=Integer(min=1,max=3)
   <b>annotation</b>(choices(
                choice=1 \"P\",
                choice=2 \"PI\",
                choice=3 \"PID\"));

<b>model</b> A
  KindOfController x;
<b>end</b> A;
A a(x=3 \"PID\");
</pre>



<h4><font color=\"#008000\">Description</font></h4>

<p>
A declaration can have an annotation \"choices\" containing modifiers on choice, where each of them indicates a suitable redeclaration or modifications of the element.
This is a hint for users of the model, and can also be used by the user interface to suggest reasonable redeclaration, where the string comments on the choice declaration can be used as textual explanations of the choices.  The annotation is not restricted to replaceable elements but can also be applied to non-replaceable elements, enumeration types, and simple variables.
</p>

</html>"));
    end choices;
    class defaultComponentName "defaultComponentName"
      annotation(Documentation(info = "<html>
<p>
Default name when dragging component
</p>

<h4><font color=\"#008000\">Syntax</font></h4>

<pre>   <b>annotation</b>\"(\" defaultComponentName \"=\" STRING \")\"
</pre>


<h4><font color=\"#008000\">Description</font></h4>

<p>
When creating a component of the given class, the recommended component name is the
giving string.
</p>

</html>"));
    end defaultComponentName;
    class defaultComponentPrefixes "defaultComponentPrefixes"
      annotation(Documentation(info = "<html>
<p>
Default prefixes when dragging component
</p>

<h4><font color=\"#008000\">Examples</font></h4>

<pre><b>annotation</b>(defaultComponentPrefixes=\"inner\",
           defaultComponentName=\"world\")
</pre>

<h4><font color=\"#008000\">Syntax</font></h4>

<pre>   <b>annotation</b>\"(\" defaultComponentPrefixes \"=\" STRING \")\"
</pre>


<h4><font color=\"#008000\">Description</font></h4>

<p>
The following prefixes may be included in the string prefixes: inner, outer, replaceable, constant, parameter, discrete. In combination with defaultComponentName it can be used to make it easy for users to create inner components matching the outer declarations.
</p>

</html>"));
    end defaultComponentPrefixes;
    class derivative "derivative"
      annotation(Documentation(info = "<html>
<p>
Define derivative of function
</p>

<h4><font color=\"#008000\">Examples</font></h4>

<pre><b>function</b> foo0 <b>annotation</b>(derivative=foo1); <b>end</b> foo0;
<b>function</b> foo1 <b>annotation</b>(derivative(order=2)=foo2); <b>end</b> foo1;
<b>function</b> foo2 <b>end</b> foo2;
</pre>



<h4><font color=\"#008000\">Description</font></h4>

<p>
Derivatives of functions can be declared explicitly using the derivative annotation, whereas a function can be defined as a partial derivative of another function using the der-operator in a short function definition.
</p>

<p>
A function declaration can have an annotation derivative specifying the derivative function. This can influence simulation time and accuracy and can be applied to both functions written in Modelica and to external functions. A derivative annotation can state that it is only valid under certain restrictions on the input arguments. These restrictions are defined using the following optional attributes: order (only a restriction if order &gt; 1, the default for order is 1), noDerivative, and zeroDerivative. The given derivative-function can only be used to compute the derivative of a function call if these restrictions are satisfied. There may be multiple restrictions on the derivative, in which case they must all be satisfied. The restrictions also imply that some derivatives of some inputs are excluded from the call of the derivative (since they are not necessary). A function may supply multiple derivative functions subject to different restrictions.
</p>

<p>
The inputs to the derivative function of order 1 are constructed as follows:
</p>

<ul>
<li> First are all inputs to the original function, and after all them we will
     in order append one derivative for each input containing reals.<br>&nbsp;</li>
<li> The outputs are constructed by starting with an empty list and then in
     order appending one derivative for each output containing reals.<br>&nbsp;</li>
<li> If the Modelica function call is a nth derivative (n>=1), i.e., this
     function call has been derived from an (n-1)th derivative, an
     annotation(order=n+1)=?,  specifies the (n+1)th derivative, and the
     (n+1)th derivative call is constructed as follows:<br>&nbsp;</li>
<li> The input arguments are appended with the (n+1)th derivative,
     which are constructed in order from the nth order derivatives.<br>&nbsp;</li>
<li> The output arguments are similar to the output argument for the
     nth derivative, but each output is one higher in derivative order.</li>
</ul>

<p>
Example: Given the declarations
</p>

<pre> function foo0
   ...
   input Real x;
   input Boolean linear;
   input ...;
   output Real y;
   ...
   annotation(derivative=foo1);
 end foo0;

 function foo1
   ...
   input Real x;
   input Boolean linear;
   input ...;
   input Real der_x;
   ...
   output Real der_y;
   ...
   annotation(derivative(order=2)=foo2);
 end foo1;

 function foo2
   ...
   input Real x;
   input Boolean linear;
   input ...;
   input Real der_x;
   ...;
   input Real der_2_x;
   ...
   output Real der_2_y;
   ...
</pre>

<p>
the equation
</p>

<pre>(...,y(t),...)=foo0(...,x(t),b,...);
</pre>

<p>
implies that:
<p>

<pre>(...,d y(t)/dt,...)=foo1(...,x(t),b,..., ...,d x(t)/dt,...);
(...,d^2 y(t)/dt^2,...)=foo2(...,x(t),b,...,d x(t)/dt,..., ...,d^2 x(t)/dt^2,...);
</pre>

<p>
An input or output to the function may be any simple type (Real, Boolean, Integer, String and enumeration types) or a record, provided the record does not contain both reals and non-reals predefined types. The function must have at least one input containing reals. The output list of the derivative function may not be empty.
</p>

<ul>
<li> zeroDerivative=input_var1<br>
    The derivative function is only valid if input_var1 is independent
    of the variables the function call is  differentiated with respect to
    (i.e., that the derivative of input_var1 is \"zero\").
    The derivative of input_var1 is excluded from the argument list of the derivative-function.
    Assume that function f takes a matrix and a scalar. Since the matrix argument is
    usually a parameter expression it is then useful to define the function
    as follows (the additional derivative = f_general_der is optional and
    can be used when the derivative of the matrix is non-zero).
<pre>function f \"Simple table lookup\"
  input Real x;
  input Real y[:, 2];
  output Real z;
  annotation(derivative(zeroDerivative=y) = f_der,
             derivative=f_general_der);
algorithm
  ...
end f;

function f_der \"Derivative of simple table lookup\"
  input Real x;
  input Real y[:, 2];
  input Real x_der;
  output Real z_der;
algorithm
  ...
end f_der;

function f_general_der \"Derivative of table lookup taking into account varying tables\"
  input Real x;
  input Real y[:, 2];
  input Real x_der;
  input Real y_der[:, 2];
  output Real z_der;
algorithm
  ...
end f_general_der;

</pre></li>

<li> noDerivative(input_var2 = f(input_var1, ...) )<br>
    The derivative function is only valid if the input argument input_var2
    is computed as f(input_var1, ...). The derivative of input_var2
    is excluded from the argument list of the derivative-function.
    Assume that function fg is defined as a composition f(x, g(x)).
    When differentiating f it is useful to give the derivative under the
    assumption that the second argument is defined in this way:
<pre>function fg
  input Real x;
  output Real z;
algorithm
   z := f(x, g(x));
end fg;

function f
  input Real x;
  input Real y;
  output Real z;
  annotation(derivative(noDerivative(y = g(x))) = f_der);
algorithm
  ...
end f;

function f_der
  input Real x;
  input Real x_der;
  input Real y;
  output Real z_der;
algorithm
  ...
end f_der;
</pre>
This is useful if g represents the major computational effort of fg).</li>
</ul>

</html>"));
    end derivative;
    class Dialog "Dialog"
      annotation(Documentation(info = "<html>
<p>
Define graphical layout of parameter menu
</p>

<h4><font color=\"#008000\">Examples</font></h4>

<pre><b>model</b> BodyShape
  ...
  <b>parameter</b> Boolean animation = true;
  <b>parameter</b> Modelica.SIunits.Length length \"Length of shape\"
     <b>annotation</b>(Dialog(enable = animation, tab = \"Animation\",
                        group = \"Shape definition\"));
  ...
<b>end</b> BodyShape;
</pre>

<h4><font color=\"#008000\">Syntax</font></h4>

<pre>   <b>annotation</b>(Dialog(enable = parameter-expression, tab = \"tab\", group = \"group\"))
</pre>


<h4><font color=\"#008000\">Description</font></h4>

<p>
Defines the placement of the component or class parameter in a parameter dialog with optional tab and group specification. If enable is false, the input field may be disabled [and no input can be given]. \"Dialog\" is defined as:
</p>

<pre>   <b>record</b> Dialog
     <b>parameter</b> String  tab    = \"General\";
     <b>parameter</b> String  group  = \"Parameters\";
     <b>parameter</b> Boolean enable = <b>true</b>;
   <b>end Dialog;
</pre>

<p>
A parameter dialog is a sequence of tabs with a sequence of groups inside them.
</p>

</html>"));
    end Dialog;
    class Documentation "Documentation"
      annotation(Documentation(info = "<html>
<p>
Annotations for documentation
</p>

<h4><font color=\"#008000\">Syntax</font></h4>

<pre>documentation_annotation:
   <b>annotation</b>\"(\" Documentation \"(\" \"info\" \"=\" STRING
                            [\",\" \"revisions\" \"=\" STRING ] \")\" \")\"
</pre>

<h4><font color=\"#008000\">Description</font></h4>
<P>
The \"Documentation\" annotation can contain the \"info\" annotation giving a textual description, the \"revisions\" annotation giving a list of revisions and other annotations defined by a tool [The \"revisions\" documentation may be omitted in printed documentation]. How the tool interprets the information in \"Documentation\" is unspecified. Within a string of the \"Documentation\" annotation, the tags &lt;HTML&gt; and &lt;/HTML&gt; or &lt;html&gt; and &lt;/html&gt; define optionally begin and end of content that is HTML encoded. Links to Modelica classes may be defined with the HTML link command using scheme \"Modelica\", e.g.,
</p>

<pre>    &lt;a href=\"Modelica://MultiBody.Tutorial\"&gt;MultiBody.Tutorial&lt;/a&gt;
</pre>

<p>
Together with scheme \"Modelica\" the (URI)  fragment specifiers #diagram, #info, #text, #icon may be used to reference different layers. Example:
</p>

<pre>   &lt;a href=\"Modelica://MultiBody.Joints.Revolute#info\"&gt;Revolute&lt;/a&gt;
</pre>

</html>"));
    end Documentation;
    annotation(Documentation(info = "<html>
<p>
In this package annotations are described.
Annotations are intended for storing extra information about a model, such as graphics, documentation or versioning. A Modelica tool is free to define and use other annotations, in addition to those defined here. The only requirement is that any tool shall save files with all annotations from this chapter and all vendor-specific annotations intact. To ensure this, annotations must be represented with constructs according to the Modelica grammar. The Modelica language specification defines the semantic meaning if a tool implements any of these annotations.
</p>

</html>"));
    class DynamicSelect "DynamicSelect"
      annotation(Documentation(info = "<html>
<p>
Define schematic animation of diagram layer
</p>

<h4><font color=\"#008000\">Examples</font></h4>

<blockquote>
<p>
The level of a tank is animated by a rectangle expanding in vertical direction and its color depending on a variable overflow:
</p>

<pre><b>annotation</b>(
  Icon(graphics={Rectangle(
    extent=<b>DynamicSelect</b>({{0,0},{20,20}},{{0,0},{20,level}}),
    fillColor=<b>DynamicSelect</b>({0,0,255},
                            <b>if</b> overflow <b>then</b> {255,0,0} <b>else</b> {0,0,255}))}
);
</pre></blockquote>


<h4><font color=\"#008000\">Description</font></h4>

<p>
Any value (coordinates, color, text, etc.) in graphical annotations can be dependent on class variables using the DynamicSelect expression. DynamicSelect has the syntax of a function call with two arguments, where the first argument specifies the value of the editing state and the second argument the value of the non-editing state. The first argument must be a literal expression
and this value is used for the annotation when editing and/or browsing the diagram layer.
The second argument may contain references to variables to enable a dynamic behavior
and the actual value is used for the annotation for schematic animation
of the diagram layer, e.g., after a simulation.
</p>

</html>"));
    end DynamicSelect;
    class Evaluate "Evaluate"
      annotation(Documentation(info = "<html>
<p>
Annotation for code generation (evaluate parameter value)
</p>

<h4><font color=\"#008000\">Syntax</font></h4>

<pre>   <b>annotation</b>\"(\" Evaluate \"=\" ( <b>false</b> | <b>true</b> ) \")
</pre>


<h4><font color=\"#008000\">Description</font></h4>

<p>
Has only an effect for a declaration with the prefix parameter.
</p>

<p>
If Evaluate = true, the model developer proposes to utilize the value for the symbolic processing. In that case, it is not possible to change the parameter value after symbolic pre-processing.
</p>

<p>
If Evaluate = false, the model developer proposes to not utilize the value of the corresponding parameter for the symbolic processing.
</p<

<p>
Evaluate is for example used for axis of rotation parameters in the Modelica.Mechanics.MultiBody library in order to improve the efficiency of the generated code
</p>

</html>"));
    end Evaluate;
    class experiment "experiment"
      annotation(Documentation(info = "<html>
<p>
Define default experiment parameters
</p>

<h4><font color=\"#008000\">Examples</font></h4>

<pre><b>annotation</b>(experiment(StartTime=0, StopTime=5, Tolerance=1e-6))
</pre>


<h4><font color=\"#008000\">Syntax</font></h4>

<pre>experiment_annotation:
   <b>annotation</b>\"(\" \"experiment\" \"(\" [experimentOption] {, experimentOption}] \")\"

experimentOption:
   StartTime  \"=\" [\"+\" | \"-\"] UNSIGNED_NUMBER |
   StopTime   \"=\" [\"+\" | \"-\"] UNSIGNED_NUMBER |
   Tolerance  \"=\" UNSIGNED_NUMBER
</pre>


<h4><font color=\"#008000\">Description</font></h4>

<p>
The experiment annotation defines the default start time (StartTime) in [s], the default stop time (StopTime) in [s], and the default relative integration tolerance (Tolerance) for simulation experiments to be carried out with the model or block at hand.
</p>

</html>"));
    end experiment;
    class HideResult "HideResult"
      annotation(Documentation(info = "<html>
<p>
Annotation for code generation (hide result)
</p>

<h4><font color=\"#008000\">Syntax</font></h4>

<pre>   <b>annotation</b>\"(\" HideResult \"=\" ( <b>false</b> | <b>true</b> ) \")
</pre>


<h4><font color=\"#008000\">Description</font></h4>

<p>
HideResult = true defines that the model developer proposes to not show the simulator results of the corresponding component [e.g., it will not be possible to plot this variable].
</p>

<p>
HideResult = false defines that the developer proposes to show the corresponding component [if a variable is declared in a protected section, a tool might not include it in a simulation result. By setting HideResult = false, the modeler would like to have the variable in the simulation result, even if in the protected section].
</p>

<p>
HideResult is for example used in the connectors of the Modelica.StateGraph library to not show variables to the modeler that are of no interest to him and would confuse him.
</p>

</html>"));
    end HideResult;
    class Inline "Inline"
      annotation(Documentation(info = "<html>
<p>
Annotation for code generation (inline function body)
</p>

<h4><font color=\"#008000\">Syntax</font></h4>

<pre>   <b>annotation</b>\"(\" Inline \"=\" ( <b>false</b> | <b>true</b> ) \")
</pre>


<h4><font color=\"#008000\">Description</font></h4>

<p>
Has only an effect within a function declaration.
</p>

<p>
If \"Inline = true\", the model developer proposes to inline the function.
This means, that the body of the function is included at all places where the function is called.
</p>

<p>
If \"Inline = false\", the model developer proposes to not inline the function.
</p>

<p>
Inline = true is for example used in Modelica.Mechanics.MultiBody.Frames and in functions of Modelica.Media to have no overhead for function calls such as resolving a vector in a different coordinate system and at the same time the function can be analytically differentiated, e.g., for index reduction needed for mechanical systems.
</p>

</html>"));
    end Inline;
    class LateInline "LateInline"
      annotation(Documentation(info = "<html>
<p>
Annotation for code generation (inline function body after symbolic processing)
</p>

<h4><font color=\"#008000\">Syntax</font></h4>

<pre>   <b>annotation</b>\"(\" LateInline \"=\" ( <b>false</b> | <b>true</b> ) \")
</pre>


<h4><font color=\"#008000\">Description</font></h4>

<p>
Has only an effect within a function declaration.
</p>

<p>
If \"LateInline = true\", the model developer proposes to inline the function after all symbolic transformations have been performed, but before common subexpression elimination takes place.
</p>

<p>
If \"LateInline = false\", the model developer proposes to not inline the function after symbolic transformations have been performed.
</p>

<p>
This annotation is for example used in Modelica.Media.Water.IF97_Utilities.T_props_ph to provide in combination with common subexpression elimination the automatic caching of function calls. Furthermore, it is used in order that a tool is able to propagate specific enthalpy over connectors in the Modelica_Fluid library.
</p>

</html>"));
    end LateInline;
    class missingInnerMessage "missingInnerMessage"
      annotation(Documentation(info = "<html>
<p>
Warning message, if inner declaration is missing
</p>

<h4><font color=\"#008000\">Examples</font></h4>

<pre><b>model</b> World
  <b>annotation</b>(defaultComponentName     = \"world\",
             defaultComponentPrefixes = \"inner replaceable\",
             missingInnerMessage      = \"The World object is missing\");
  ...
<b>end</b> World;
</pre>

<h4><font color=\"#008000\">Syntax</font></h4>

<pre>   <b>annotation</b>\"(\" missingInnerMessage \"=\" STRING \")\"
</pre>


<h4><font color=\"#008000\">Description</font></h4>

<p>
When an outer component of the class does not have a corresponding inner component, the string message may be used as a diagnostic message.
</p>

</html>"));
    end missingInnerMessage;
    class PreferredView "preferredView"
      annotation(Documentation(info = "<html>
<p>
Define default view when selecting class
</p>

<h4><font color=\"#008000\">Syntax</font></h4>

<pre>preferred view_annotation:
   <b>annotation</b>\"(\" preferredView \"=\" (\"info\" | \"diagram\" | \"text\") \")\"
</pre>


<h4><font color=\"#008000\">Description</font></h4>

<p>
The preferredView annotation defines the default view when selecting the class. info means info layer, i.e., the documentation of the class, diagram means diagram layer and text means the Modelica text layer.
</p>

</html>"));
    end PreferredView;
    class smoothOrder "smoothOrder"
      annotation(Documentation(info = "<html>
<p>
Define differentiability of function body
</p>

<h4><font color=\"#008000\">Syntax</font></h4>

<pre>   <b>annotation</b>\"(\" smoothOrder \"=\" UNSIGNED_INTEGER \")\"
</pre>


<h4><font color=\"#008000\">Description</font></h4>

<p>
This annotation has only an effect within a function declaration.
</p>

<p>
smoothOrder defines the minimum number of differentations of the function, in order that all of the differentiated outputs are continuous provided all input arguments and their derivatives up to order smoothOrder are continuous.
</p>

<p>
This means that the function is at least C<sup>smoothOrder</sup>. smoothOrder = 1 means that the function can be differentiated at least once in order that all output arguments are still continuous, provided the input arguments are continuous. If a tool needs the derivative of a function, e.g., for index reduction or to compute an analytic Jacobian, the function can be differentiated analytically at least smoothOrder times.
</p>

</html>"));
    end smoothOrder;
    class version "version"
      annotation(Documentation(info = "<html>
<p>
Define version information of package
</p>

<h4><font color=\"#008000\">Examples</font></h4>

<pre><b>package</b> Modelica
  <b>annotation</b>(version=\"2.1\",
             conversion(noneFromVersion=\"2.1 Beta 1\",
                        from(version=\"1.5\",
                             script=\"convertFromModelica1_5.mos\")));
  ...
<b>end</b> Modelica;

<b>model</b> A
  <b>annotation</b>(version=\"1.0\",
     uses(Modelica(version=\"1.5\")));
  ...
<b>end</b> A;

<b>model</b> B
  <b>annotation</b>(uses(Modelica(version=\"2.1 Beta 1\")));
  ...
<b>end</b> B;
</pre>

<p>
In this example the model A uses an older version of the Modelica library and can be upgraded using the given script, and model B uses an older version of the Modelica library but no changes are required when upgrading.
</p>



<h4><font color=\"#008000\">Description</font></h4>

<p>
Version numbers are of the forms:
</p>

<ul>
<li> Main release versions:
     <pre>\"\"\" UNSIGNED_INTEGER { \".\" UNSIGNED_INTEGER } \"\"\"</pre>
     Example: <code>\"2.1\"</code><br>&nbsp;</li>

<li> Pre-release versions:
     <pre>\"\"\" UNSIGNED_INTEGER { \".\" UNSIGNED_INTEGER } \" \" {S-CHAR} \"\"\"</pre>
     Example: <code>\"2.1 Beta 1\"</code><br>&nbsp;</li>

<li> Un-ordered versions:
     <pre> \"\"\" NON-DIGIT {S-CHAR} \"\"\" </pre>
     Example: <code>\"Test 1\"</code></li>
</ul>

<p>
The main release versions are ordered using the hierarchical numerical names, and follow the corresponding pre-release versions. The pre-release versions of the same main release version are internally ordered alphabetically.
</p>

<p>
In a top-level class, the version number and the dependency to earlier versions of this class are defined using one or more of the following annotations:
</p>

<ul>
<li> <code>version = CURRENT-VERSION-NUMBER</code><br>
     Defines the version number of the model or package.
     All classes within this top-level class have this version number.<br>&nbsp;</li>

<li> <code>conversion ( noneFromVersion = VERSION-NUMBER)</code><br>
     Defines that user models using the VERSION-NUMBER can be upgraded to
     the CURRENT-VERSION-NUMBER of the current class without any changes.<br>&nbsp;</li>

<li> <code>conversion ( from (version = VERSION-NUMBER, script = \"?\") ) </code><br>
     Defines that user models using the VERSION-NUMBER can be upgraded to
     the CURRENT-VERSION-NUMBER of the current class by applying the given
     script. The semantics of the conversion script is not defined.<br>&nbsp;</li>

<li> <code>uses(IDENT (version = VERSION-NUMBER) )</code><br>
     Defines that classes within this top-level class uses version
     VERSION-NUMBER of classes within the top-level class IDENT.
     The annotations uses and conversion may contain several different sub-entries.<li>
</ul>

<p>
A top-level class, IDENT, with version VERSION-NUMBER can be stored in one
of the following ways in a directory given in the MODELICAPATH:
</p>

<ul>
<li> The file IDENT \".mo\"<br>
     Example: Modelica.mo</li>
<li> The file IDENT \" \" VERSION-NUMBER \".mo\"<br>
     Example: Modelica 2.1.mo</li>
<li> The directory IDENT<br>
     Example: Modelica</li>
<li> The directory IDENT \" \" VERSION-NUMBER<br>
     Example: Modelica 2.1</li>
</ul>

<p>
This allows a tool to access multiple versions of the same package.
</p>

</html>"));
    end version;
    class unassignedMessage "unassignedMessage"
      annotation(Documentation(info = "<html>
<p>
Error message, if variable is not assigned
</p>

<h4><font color=\"#008000\">Examples</font></h4>

<pre><b>connector</b> Frame \"Frame of a mechanical system\"
    ...
  <b>flow</b> Modelica.SIunits.Force f[3] <b>annotation</b>(unassignedMessage =
\"All Forces cannot be uniquely calculated. The reason could be that the
mechanism contains a planar loop or that joints constrain the same motion.
For planar loops, use in one revolute joint per loop the option
PlanarCutJoint=true in the Advanced menu.
\");
<b>end</b> Frame;
</pre>


<h4><font color=\"#008000\">Syntax</font></h4>

<pre>   <b>annotation</b>\"(\" unassignedMessage \"=\" STRING \")\"
</pre>


<h4><font color=\"#008000\">Description</font></h4>

<p>
When the variable to which this annotation is attached in the declaration cannot be computed due to the structure of the equations, the string message can be used as a diagnostic message. When using BLT partitioning, this means if a variable \"a\" or one of its aliases \"b = a\", \"b = -a\", cannot be assigned, the message is displayed. This annotation is used to provide library specific error messages.
</p>

</html>"));
    end unassignedMessage;
  end Annotations;
  package Classes "Classes (model, function, ...)"
    class Block "block"
      annotation(Documentation(info = "<html>
<p>
Define specialized class <i>block</i>
</p>
<h4><font color=\"#008000\">Examples</font></h4>

<pre><b>block</b> Integrator
  <b>input</b> Real u;
  <b>output</b> Real y;
<b>protected</b>
  Real x;
<b>equation
  der</b>(x) = u;
  y = x;
<b>end</b> Integrator;</pre>

<h4><font color=\"#008000\">Syntax</font></h4>

<PRE>   [ <B>encapsulated</B> ][ <B>partial </B>] <B>block</B>
   IDENT class_specifier

class_specifier :
   string_comment composition <B>end</B> IDENT
   | \"=\" base_prefix name [ array_subscripts ] [ class_modification ] comment
   | \"=\" <B>enumeration</B> \"(\" ( [enum_list] | \":\" ) \")\" comment</PRE>

<p>See Modelica Language Specification for further details.</p>

<h4><font color=\"#008000\">Description</font></h4>
<P>
A block class is the same as a model class
with the restriction that each connector component of a block must
have prefixes input and/or output for all connector variables.
The purpose is to model input/output blocks of block diagrams.
Due to the restrictions on input and output prefixes,
connections between blocks are only possible according
to block diagram semantic.
</P>
</html>"));
    end Block;
    class Class "class"
      annotation(Documentation(info = "<html>
<p>
Define class
</p>
<h4><font color=\"#008000\">Examples</font></h4>

<pre><b>class</b> MyTable
  <b>extends</b> ExternalObject;
  <b>function</b> constructor
     ...
  <b>end</b> constructor;

  <b>function</b> destructor
     ...
  <b>end</b> destructor;
<b>end</b> MyTable;</pre>

<h4><font color=\"#008000\">Syntax</font></h4>
<PRE>   [ <B>encapsulated</B> ][ <B>partial </B>] <B>class</B>
   IDENT class_specifier

class_specifier :
   string_comment composition <B>end</B> IDENT
   | \"=\" base_prefix name [ array_subscripts ] [ class_modification ] comment
   | \"=\" <B>enumeration</B> \"(\" ( [enum_list] | \":\" ) \")\" comment</PRE>

<p>See Modelica Language Specification for further details.</p>

<h4><font color=\"#008000\">Description</font></h4>

<p>
The keyword class is used to define general classes (without any restrictions).
It is identical to the keyword model.
In most cases, it is recommended to use specialized classes as <b>block</b>,
<b>connector</b>, <b>model</b>, <b>package</b>, <b>record</b>, <b>function</b>or <b>type</b>.
\"class\" should only be used to define ExternalObjects.
</p>
</html>"));
    end Class;
    class Connector "connector"
      annotation(Documentation(info = "<html>
<p>
Define specialized class <i>connector</i>
</p>
<h4><font color=\"#008000\">Examples</font></h4>

<pre><b>connector</b> flange
  Modelica.SIunits.Angle phi;
  <b>flow</b> Modelica.SIunits.Torque tau;
<b>end</b> flange;</pre>

<h4><font color=\"#008000\">Syntax</font></h4>
<PRE>   [ <B>encapsulated</B> ][ <B>partial </B>] <B>connector</B>
   IDENT class_specifier

class_specifier :
   string_comment composition <B>end</B> IDENT
   | \"=\" base_prefix name [ array_subscripts ] [ class_modification ] comment
   | \"=\" <B>enumeration</B> \"(\" ( [enum_list] | \":\" ) \")\" comment</PRE>

<p>See Modelica Language Specification for further details.</p>

<h4><font color=\"#008000\">Description</font></h4>
<P>The keyword connector is used to define connectors, which are used
in connect statements. In connectors, no equations are allowed in the
definition or in any of its components.
With respect to \"class\", it is enhanced to allow connect(..) to components
of connector classes.
</P>
</html>"));
    end Connector;
    class Function "function"
      annotation(Documentation(info = "<html>
<p>
Define specialized class <i>function</i>
</p>
<h4><font color=\"#008000\">Examples</font></h4>

<pre><b>function</b> si
  <b>input</b> Real x;
  <b>output</b> Real y;
<b>algorithm</b>
  y = <b>if abs</b>(x) &lt; Modelica.Constants.eps <b>then</b> 1 <b>else</b> Modelica.Math.sin(x)/x;
<b>end</b> si;</pre>

<p align=\"center\"><img src=\"../Images/function.png\" width=\"400\" height=\"280\" alt=\"Simulation result\"></p>

<h4><font color=\"#008000\">Syntax</font></h4>

<PRE>   [ <B>encapsulated</B> ][ <B>partial </B>] <B>function</B>
   IDENT class_specifier

class_specifier :
   string_comment composition <B>end</B> IDENT
   | \"=\" base_prefix name [ array_subscripts ] [ class_modification ] comment
   | \"=\" <B>enumeration</B> \"(\" ( [enum_list] | \":\" ) \")\" comment</PRE>

<p>See Modelica Language Specification for further details.</p>

<h4><font color=\"#008000\">Description</font></h4>

<P>The keyword function is used to define functions as known from programming
languages. Each part of a function interface must
either have causality equal to input or output. A function may not be used in connections.
In functions, no equations or initial algorithm and at most
one algorithm clause are allowed. Calling a function requires
either an algorithm clause or an external function interface.</p>

<p>
The syntax and semantics of a function have many similarities to those of the block
specialized class. A function has many of the properties of a general class,
e.g., being able to inherit other functions, or to redeclare or modify
elements of a function declaration.
</p>

<p>
Modelica functions have the following restrictions compared to a
general Modelica class:
</p>

<ul>
<li> Each input formal parameter of the function must be
     prefixed by the keyword input, and each result formal parameter
     by the keyword output. All public variables are formal parameters.</li>

<li> Input formal parameters are read-only after being bound to the
     actual arguments or default values, i.e., they may not be assigned
     values in the body of the function.</li>

<li> A function may not be used in connections, may have no equations,
     may have no initial algorithm, and can have at most one algorithm
     section, which, if present, is the body of the function.</li>

<li> A function may have zero or one external function interface, which,
     if present, is the external definition of the function. </li>

<li> For a function to be called in a simulation model, it must have
     either an algorithm section or an external function interface as
     its body, and it may not be partial.</li>

<li> A function cannot contain calls to the Modelica built-in operators
     der, initial, terminal, sample, pre, edge, change, reinit, delay,
     cardinality, to the operators of the built-in package Connections,
     and is not allowed to contain when-statements.</li>

<li> The dimension sizes not declared with (:) of each array result or
     array local variable [i.e., a non-input components] of a function must
     be either given by the input formal parameters, or given by constant
     or parameter expressions, or by expressions containing combinations
     of those</li>

<li> The local variables of a function are not automatically initialized to
     the implicit default values of the data type [(e.g., 0.0 for Real)
     for performance reasons. It is the responsibility of the user to
     provide explicit defaults or to define the values of such variables
     before they are referenced.]</li>

<li> Components of a function will inside the function behave as though
     they had discrete-time variability.</li>
</ul>

<p>
Modelica functions have the following enhancements compared to a general Modelica class:
</p>

<ul>
<li> A function may be called using the conventional positional calling syntax
     for passing arguments.</li>

<li> A function can be recursive.</li>

<li> A formal parameter or local variable may be initialized
     through an assignment (:=) of a default value in its declaration.
     Initialization through an equation is not possible.</li>

<li> A function is dynamically instantiated when it is called rather than
     being statically instantiated by an instance declaration,
     which is the case for other kinds of classes. </li>

<li> A function may have an external function interface specifier as its body.</li>

<li> A function may have a return statement in its algorithm section body.</li>

<li> A function allows dimension sizes declared with (:) to be resized
     for non-input array variables (so the actual dimension need not to be known when
     the function is translated).</li>.
</ul>


</html>"));
    end Function;
    class Model "model"
      annotation(Documentation(info = "<html>
<p>
Define specialized class <i>model</i>
</p>
<h4><font color=\"#008000\">Examples</font></h4>

<pre><b>model</b> SlidingMass
  <b>parameter</b> Modelica.SIunits.Mass m=1;
  <b>parameter</b> Modelica.SIunits.Force f=1;
  Modelica.SIunits.Position s;
  Modelica.SIunits.Velocity v;
  Modelica.SIunits.Acceleration a;
<b>equation</b>
  <b>der</b>(s) = v;
  <b>der</b>(v) = a;
  m*a = f;
<b>end</b> SlidingMass;</pre>

<h4><font color=\"#008000\">Syntax</font></h4>
<PRE>   [ <B>encapsulated</B> ][ <B>partial </B>] <B>model</B>
   IDENT class_specifier

class_specifier :
   string_comment composition <B>end</B> IDENT
   | \"=\" base_prefix name [ array_subscripts ] [ class_modification ] comment
   | \"=\" <B>enumeration</B> \"(\" ( [enum_list] | \":\" ) \")\" comment</PRE>

<p>See Modelica Language Specification for further details.</p>

<h4><font color=\"#008000\">Description</font></h4>
<P>
The keyword model is identical to the keyword class, i.e.,
no restrictions and no enhancements.
</P>
</html>"));
    end Model;
    class Package "package"
      annotation(Documentation(info = "<html>
<p>
Define specialized class <i>package</i>
</p>
<h4><font color=\"#008000\">Examples</font></h4>

<pre><b>package</b> Library
  <b>constant</b> Real k = 0.1;

  <b>type</b> X = Real(min=0);

  <b>model</b> A
    ...
  <b>end</b> A;

  <b>model</b> B
    ...
  <b>end</b> B;
<b>end</b> Library;</pre>


<h4><font color=\"#008000\">Syntax</font></h4>
<PRE>   [ <B>encapsulated</B> ][ <B>partial </B>] <B>package</B>
   IDENT class_specifier

class_specifier :
   string_comment composition <B>end</B> IDENT
   | \"=\" base_prefix name [ array_subscripts ] [ class_modification ] comment
   | \"=\" <B>enumeration</B> \"(\" ( [enum_list] | \":\" ) \")\" comment</PRE>

<p>See Modelica Language Specification for further details.</p>

<h4><font color=\"#008000\">Description</font></h4>
<P>May only contain declarations of classes and constants.
   Enhanced to allow import of elements of packages.</P>

</html>"));
    end Package;
    class Record "record"
      annotation(Documentation(info = "<html>
<p>
Define specialized class <i>record</i>
</p>
<h4><font color=\"#008000\">Examples</font></h4>

<pre>  <b>record</b> States
    Modelica.SIunits.Position s;
    Modelica.SIunits.Velocity v;
  <b>end</b> States;

  <b>record</b> System
    <b>parameter</b> Modelica.SIunits.Mass m=1;
    <b>parameter</b> Modelica.SIunits.Force f=1;
    Modelica.SIunits.Acceleration a;
    States states;
  <b>end</b> System;

  <b>model</b> SlidingMass
    System sys;
  <b>equation</b>
    <b>der</b>(sys.states.s) = sys.states.v;
    <b>der</b>(sys.states.v) = sys.a;
    sys.m*sys.a = sys.f;
  <b>end</b> SlidingMass;</pre>

<h4><font color=\"#008000\">Syntax</font></h4>
<PRE>   [ <B>encapsulated</B> ][ <B>partial </B>] <B>record</B>
   IDENT class_specifier

class_specifier :
   string_comment composition <B>end</B> IDENT
   | \"=\" base_prefix name [ array_subscripts ] [ class_modification ] comment
   | \"=\" <B>enumeration</B> \"(\" ( [enum_list] | \":\" ) \")\" comment</PRE>

<p>See Modelica Language Specification for further details.</p>

<h4><font color=\"#008000\">Description</font></h4>

<p>
The keyword record is used to define records which are generally used in
order to group variables. Only public sections are allowed in the definition
or in any of its components (i.e., equation, algorithm, initial equation,
initial algorithm and protected sections are not allowed). May not be used in
connections. The elements of a record may not have prefixes input, output, inner, outer,
or flow.  Enhanced with implicitly available record constructor function.
Additionally, record components can be used as component references in
expressions and in the left hand side of assignments, subject to
normal type compatibility rules.
</p>

</html>"));
    end Record;
    class Type "type"
      annotation(Documentation(info = "<html>
<p>
Define specialized class <i>type</i>
</p>
<h4><font color=\"#008000\">Examples</font></h4>

<pre><b>type</b> R0Plus = Real(min=0);</pre>

<h4><font color=\"#008000\">Syntax</font></h4>
<PRE>   [ <B>encapsulated</B> ][ <B>partial </B>] <B>type</B>
   IDENT class_specifier

class_specifier :
   string_comment composition <B>end</B> IDENT
   | \"=\" base_prefix name [ array_subscripts ] [ class_modification ] comment
   | \"=\" <B>enumeration</B> \"(\" ( [enum_list] | \":\" ) \")\" comment</PRE>

<p>See Modelica Language Specification for further details.</p>

<h4><font color=\"#008000\">Description</font></h4>
<p>The keyword type is used to define types, which may only be extensions to the predefined types, enumerations, array of type, or classes extending from type.
Enhanced to extend from predefined types [No other specialized class has this property].
</p>

</html>"));
    end Type;
    annotation(Documentation(info = "<html>
<p>
In this package specialized kinds of classes (earlier known as restricted classes) are
described. They have the properties of a general class, apart from restrictions.
Moreover, they have additional properties called enhancements.
</p>

</html>"));
  end Classes;
  package Operators "Operators (+, der, size, ...)"
    class ElementaryOperators "Elementary operators (+, >, or, ..)"
      annotation(Documentation(info = "<html>
<p>
Elementary operators are overloaded and operate on variables
of type Real, Integer, Boolean, and String, as well as on scalars
or arrays.
</p>
<h4><font color=\"#008000\">Syntax</font></h4>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td colspan=\"3\"><b>Arithmethic Operators (operate on Real, Integer scalars or arrays)</b></td>
  <tr><td><i>Operators</i></td>
      <td><i>Example</i></td>
      <td><i>Description</i></tr></td>
      </tr>
  <tr><td>+, -, .+, .-</td>
      <td>a + b<br>
          a .+ b</td>
      <td>addition and subtraction; element-wise on arrays</td></tr>

  <tr><td>*</td>
      <td>a * b</td>
      <td>multiplication;<br>
          scalar*array: element-wise multiplication<br>
          vector*vector: element-wise multiplication (result: scalar)<br>
          matrix*matrix: matrix product <br>
          vector*matrix: row-matrix*matrix (result: vector)<br>
          matrix*vector: matrix*column-matrix (result: vector)</td></tr>
  <tr><td>/</td>
      <td>a / b</td>
      <td>division of two scalars or an array by a scalar;<br>
          division of an array by a scalar is defined element-wise.<br>
          The result is always of real type. In order to get integer <br>
          division with truncation use the function div.</td></tr>

  <tr><td>^</td>
      <td>a^b</td>
      <td>scalar power or integer power of a square matrix</td></tr>

  <tr><td>.*, ./, .^</td>
      <td>a .* b</td>
      <td>element-wise multiplication, division and exponentation of
          scalars and arrays</td></tr>

  <tr><td>=</td>
      <td>a * b = c + d</td>
      <td>equal operator of an equation; element-wise on arrays</td></tr>
  <tr><td>:=</td>
      <td>a := c + d</td>
      <td>assignment operator; element-wise on arrays</td></tr>
</table>

&nbsp;

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td colspan=\"3\"><b>Relational Operators (operate on Real, Integer, Boolean, String scalars)</b></td>
  <tr><td><i>Operators</i></td>
      <td><i>Example</i></td>
      <td><i>Description</i></tr></td>
      </tr>
  <tr><td>==</td>
      <td>a == b</td>
      <td>equal; for strings: identical characters</td></tr>
  <tr><td>&lt;&gt;</td>
      <td>a &lt;&gt; b</td>
      <td>not equal; for strings: a is lexicographically less than b</td></tr>
  <tr><td>&lt;</td>
      <td>a &lt; b</td>
      <td>less than</td></tr>
  <tr><td>&lt;=</td>
      <td>a &lt;= b</td>
      <td>less than or equal</td></tr>
  <tr><td>&gt;</td>
      <td>a &gt; b</td>
      <td>greater than</td></tr>
  <tr><td>&gt;=</td>
      <td>a &gt;= b</td>
      <td>greater than or equal</td></tr>
</table>

&nbsp;

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td colspan=\"3\"><b>Boolean Operators (operate on scalars or element-wise on arrays)</b></td>
  <tr><td><i>Operators</i></td>
      <td><i>Example</i></td>
      <td><i>Description</i></tr></td>
      </tr>
  <tr><td><b>and</b></td>
      <td>a <b>and</b> b</td>
      <td>logical and</td></tr>
  <tr><td><b>or</b></td>
      <td>a <b>or</b> b</td>
      <td>logical or</td></tr>
  <tr><td><b>not</b></td>
      <td><b>not</b> a</td>
      <td>logical not</td></tr>
</table>

&nbsp;

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td colspan=\"3\"><b>Other Operators</b></td>
  <tr><td><i>Operators</i></td>
      <td><i>Example</i></td>
      <td><i>Description</i></tr></td>
      </tr>
  <tr><td>[..]</td>
      <td>[1,2;3,4]</td>
      <td>Matrix constructor; \",\" separates rows, \";\" separates columns</td></tr>
  <tr><td>{..}</td>
      <td>{{1,2}, {3,4}}</td>
      <td>Array constructor; every {..} adds one dimension</td></tr>
  <tr><td>\"...\"</td>
      <td>\"string value\"<br>
          \"string \"value\"\"</td>
      <td>String literal (\" is used inside a string for \")</td></tr>
  <tr><td>+</td>
      <td>\"abc\" + \"def\"</td>
      <td>Concatenation of string scalars or arrays</td></tr>
</table>

<p><span >Operator
precedence determines the order
of evaluation of operators in an expression. An operator with higher
precedence
is evaluated before an operator with lower precedence in the same
expression.</span></p>
<p><span >The
following table presents all
the expression operators in order of precedence from highest to lowest.
All operators are binary except exponentiation, the postfix
operators and
those shown as unary together with <i>expr</i>, the
conditional operator, the
array construction operator {} and concatenation operator [ ], and the
array
range constructor which is either binary or ternary</span><span
 >. Operators with the same precedence occur at
the same line of the
table: </span></p>

<table border=\"1\" cellpadding=\"0\" cellspacing=\"0\">
  <tbody>
    <tr>
      <td valign=\"top\" width=\"180\">
      <p><i>Operator Group</i></p>
      </td>
      <td valign=\"top\">
      <p><i>Operator Syntax</i></p>
      </td>
      <td valign=\"top\">
      <p><i><span>Examples</span></i></p>
      </td>
    </tr>
    <tr>
      <td valign=\"top\">
      <p><span
 >postfix array index operator</span></p>
      </td>
      <td valign=\"top\">
      <p><span class=\"CODE\"><span>[]</span></span></p>
      </td>
      <td valign=\"top\">
      <p><span class=\"CODE\"><span>arr[index]</span></span></p>
      </td>
    </tr>
    <tr>
      <td valign=\"top\">
      <p><span>postfix access operator</span></p>
      </td>
      <td valign=\"top\">
      <p><span1 class=\"CODE\"><span>. </span></span></p>
      </td>
      <td valign=\"top\">
      <p><span class=\"CODE\"><span>a.b</span></span></p>
      </td>
    </tr>
    <tr>
      <td valign=\"top\">
      <p><span>postfix function call</span></p>
      </td>
      <td valign=\"top\">
      <p><i><span>funcName</span></i><span
 >(<i>function-arguments</i>)</span></p>
      </td>
      <td valign=\"top\">
      <p><span class=\"CODE\"><span>sin(4.36)</span></span></p>
      </td>
    </tr>
    <tr>
      <td valign=\"top\">
      <p><span>array construct/concat</span></p>
      </td>
      <td

 valign=\"top\">
      {<i>expressions</i>}<br>
      [<i>expressions</i>]<br>
      [<i>expressions</i>; <i>expressions</i>...]
      </td>
      <td

 valign=\"top\">
      <p><span
 class=\"CODE\"><span
 >{2,3}&nbsp; [5,6]</span></span></p>
      <p><span
 class=\"CODE\"><span
 >[2,3; 7,8]</span></span></p>
      </td>
    </tr>
    <tr>
      <td valign=\"top\">
      <p><span>exponentiation</span></p>
      </td>
      <td valign=\"top\">
      <p><span class=\"CODE\"><span>^</span></span></p>
      </td>
      <td valign=\"top\">
      <p><span class=\"CODE\"><span>2^3</span></span></p>
      </td>
    </tr>
    <tr>
      <td valign=\"top\">
      <p>
<span>multiplicative and<br> array elementwise multiplicative</span></p>
      </td>
      <td valign=\"top\">
      <p><span class=\"CODE\"><span>*</span></span><span class=\"CODE\"><span
  >&nbsp;&nbsp;/&nbsp;
.*&nbsp;&nbsp; ./</span></span></p>
      </td>
      <td valign=\"top\">
      <p><span class=\"CODE\"><span>2*3&nbsp;&nbsp; 2/3<br>
[1,2;3,4].*[2,3;5,6]</span></span></p>
      </td>
    </tr>
    <tr>
      <td valign=\"top\">
      <p>additive and<br> array elementwise additive</p>
      </td>
      <td valign=\"top\">
      <p><spanclass=\"CODE\"><span>+</span></span><span class=\"CODE\"><span>&nbsp;&nbsp;-</span></span><span
 class=\"CODE\"><span>&nbsp;&nbsp;+</span></span><i><span>expr</span></i><span >&nbsp;
      </span><span class=\"CODE\"><span
  >-</span></span><i><span
 >expr<br>
      </span></i><span >.+&nbsp;
.</span><span class=\"CODE\"><span
  >-</span></span></p>
      </td>
      <td valign=\"top\">
      <p><span class=\"CODE\"><span>a+b</span></span><span>,</span><span class=\"CODE\"><span>&nbsp;
a-b, +a, -a</span></span></p>
      <p><span class=\"CODE\"><span
 >[1,2;3,4].+[2,3;5,6]</span></span></p>
      </td>
    </tr>
    <tr>
      <td valign=\"top\">
      <p><span >relational</span></p>
      </td>
      <td valign=\"top\">
      <p><span class=\"CODE\"><span
 >&lt;</span></span><span
 class=\"CODE\"><span
 >&nbsp;&nbsp;&lt;=</span></span><span
 class=\"CODE\"><span
 >&nbsp;&nbsp;&gt;</span></span><span
 class=\"CODE\"><span
 >&nbsp;&nbsp;&gt;=</span></span><span
 class=\"CODE\"><span
 >&nbsp;&nbsp;==</span></span><span
 class=\"CODE\"><span
 >&nbsp;&nbsp;&lt;&gt;</span></span></p>
      </td>
      <td valign=\"top\">
      <p><span class=\"CODE\"><span
 >a&lt;b</span></span><span
 >,</span><span class=\"CODE\"><span
  >&nbsp;
a&lt;=b, a&gt;b, ...</span></span></p>
      </td>
    </tr>
    <tr>
      <td valign=\"top\">
      <p><span
 class=\"CODE\"><span
 >...</span></span></p>
      </td>
      <td valign=\"top\">
      <p><span
 >&nbsp;</span></p>
      </td>
      <td valign=\"top\">
      <p><span
 >&nbsp;</span></p>
      </td>
    </tr>
    <tr>
      <td valign=\"top\">
      <p><span >unary negation</span></p>
      </td>
      <td valign=\"top\">
      <p><span class=\"CODE\"><span
 >not</span></span><span
 class=\"CODE\"><span
 >&nbsp;</span></span><i><span
 >expr</span></i></p>
      </td>
      <td valign=\"top\">
      <p><span class=\"CODE\"><span
 >not b1</span></span></p>
      </td>
    </tr>
    <tr>
      <td valign=\"top\">
      <p><span >logical and</span></p>
      </td>
      <td valign=\"top\">
      <p><span class=\"CODE\"><span
 >and</span></span></p>
      </td>
      <td valign=\"top\">
      <p><span class=\"CODE\"><span
 >b1 and b2</span></span></p>
      </td>
    </tr>
    <tr>
      <td valign=\"top\">
      <p><span >logical or</span></p>
      </td>
      <td valign=\"top\">
      <p><span class=\"CODE\"><span
 >or</span></span></p>
      </td>
      <td valign=\"top\">
      <p><span class=\"CODE\"><span
 >b1 or&nbsp; b2</span></span></p>
      </td>
    </tr>
    <tr>
      <td valign=\"top\">
      <p><span >array range</span></p>
      </td>
      <td valign=\"top\">
      <p><i><span >expr</span></i><span >&nbsp;
      </span><span class=\"CODE\"><span
  >:</span></span><span
 >&nbsp; <i>expr</i>&nbsp; </span><span
 class=\"CODE\"><span
 >:</span></span><span >&nbsp;
      <i>expr</i></span></p>
      </td>
      <td valign=\"top\">
      <p><span class=\"CODE\"><span
 >1:5</span></span><span
 >,</span><span class=\"CODE\"><span
  >
start:step:stop</span></span></p>
      </td>
    </tr>
    <tr>
      <td valign=\"top\">
      <p><span >conditional</span></p>
      </td>
      <td valign=\"top\">
      <p><span class=\"CODE\"><span
 >if</span></span><span
 >&nbsp;&nbsp;<i>expr</i>&nbsp;
      </span><span class=\"CODE\"><span
  >then</span></span><span
 >&nbsp;&nbsp;<i>expr</i>&nbsp;
      </span><span class=\"CODE\"><span
  >else</span></span><span
 >&nbsp;&nbsp;<i>expr</i></span></p>
      </td>
      <td valign=\"top\">
      <p><span class=\"CODE\"><span
 >if b then 3 else x</span></span></p>
      </td>
    </tr>
    <tr>
      <td valign=\"top\">
      <p><span >named argument</span></p>
      </td>
      <td valign=\"top\">
      <p><i>ident&nbsp;</i><span class=\"CODE\">=</span><span >&nbsp;
      <i>expr</i>&nbsp; </span></p>
      </td>
      <td valign=\"top\">
      <p><span class=\"CODE\"><span
 >x = 2.26</span></span></p>
      </td>
    </tr>
  </tbody>
</table>
<p><span >The
conditional operator may also include
elseif-clauses. Equality </span><span class=\"CODE\"><span
  >=</span></span><span
 > and assignment </span><span
 class=\"CODE\"><span
 >:=</span></span><span
 > are not expression operators since they are
allowed only in
equations and in assignment statements respectively. All binary
expression
operators are left associative.</span></p>

<p><span >
Note, the unary minus and plus in Modelica
is slightly different than in Mathematica (Mathematica is a registered trademark
of Wolfram Research Inc.) and in MATLAB (MATLAB is a registered trademark of MathWorks Inc.),
since the following expressions are illegal (whereas in
Mathematica and in MATLAB these are valid expressions):
</p>

<pre>  2*-2   // = -4 in Mathematica/MATLAB; is illegal in Modelica
  --2    // =  2 in Mathematica/MATLAB; is illegal in Modelica
  ++2    // =  2 in Mathematica/MATLAB; is illegal in Modelica
  2--2   // =  4 in Mathematica/MATLAB; is illegal in Modelica
</pre>

</html>"));
    end ElementaryOperators;
    annotation(Documentation(info = "<html>
<p>
In this chapter <b>operators</b> of Modelica are documented.
Elementary operators, such as \"+\" or \"-\" are overloaded and
operate on scalar and array variables. Other operators
have the same syntax as a
<a href=\"Modelica://ModelicaReference.Classes.Function\">Modelica function</a>
call. However, they do not behave as a Modelica function,
either because the result depends not only on the input arguments but
also on the status of the simulation (such as \"pre(..)\"), or
the function operates on input arguments of different types
(such as \"String(..)\"). Neither of these \"functions\"
can be defined with a \"standard\" Modelica function and are
therefore builtin operators of the Modelica language
(with exception of the basic mathematical functions,
sin, cos, tan, asin, acos, atan, atan2, sinh, cosh, tanh, exp,
log, log10 that are provided for convenience as built-in functions).
</p>
</html>"));
    class Abs "abs"
      annotation(Documentation(info = "<html>
<p>
Absolute value of Real or Integer variable.
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>abs</b>(v)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<P>Is expanded into &quot;noEvent(<B>if </B>v &ge; 0
<B>then</B> v <B>else</B> -v)&quot;. Argument v
needs to be an Integer or Real expression. </P>
<h4><font color=\"#008000\">Examples</font></h4>
<pre>
  <b>abs</b>({-3, 0, 3})
 = {3, 0, 3}</pre>
</html>"));
    end Abs;
    class Acos "acos"
      annotation(Documentation(info = "<html>
<p>
Trigonometric inverse cosine function
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>acos</b>(u)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>

<p>Returns the inverse of cos of u, with -1 &le; u &le; +1.
Argument u needs to be an Integer or Real expression.</p>

<p>
The acos function can also be accessed as Modelica.Math.acos.
</p

<p>
<img src=\"../Images/acos.png\">
</p>



<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>acos</b>(0)
 = 1.5707963267949</pre>
</html>"));
    end Acos;
    class Asin "asin"
      annotation(Documentation(info = "<html>
<p>
Trigonometric inverse sine function
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>asin</b>(u)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>

<p>Returns the inverse of sin of u, with -1 &le; u &le; +1.
Argument u needs to be an Integer or Real expression.</p>

<p>
The asin function can also be accessed as Modelica.Math.asin.
</p

<p>
<img src=\"../Images/asin.png\">
</p>



<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>asin</b>(0)
 = 0.0</pre>
</html>"));
    end Asin;
    class Assert "assert"
      annotation(Documentation(info = "<html>
<p>
Trigger error and print error message if assertion condition is not fulfilled
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>assert</b>(condition, message, level = AssertionLevel.error)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>The boolean expression <i>condition</i> shall be true for successful model evaluations.
Otherwise, an error occurs using the string expression <i>message</i>
as error message.</p>
<P>If the condition of an assert statement is true, message is not
evaluated and the procedure call is ignored. If the condition
evaluates to false different actions are taken depending on the level input:
</p>

<ul>
<li> level = AssertionLevel.error:<br>
     The current evaluation is aborted. The simulation may
     continue with another evaluation [e.g., with a shorter step-size,
     or by changing the values of iteration variables].
     If the simulation is aborted, message indicates the cause of the
     error. Failed assertions takes precedence over successful
     termination, such that if the model first triggers the
     end of successful analysis by reaching the stop-time
     or explicitly with terminate(), but the evaluation with
     terminal()=true triggers an assert, the analysis failed. </li>
<li> level = AssertionLevel.warning:<br>
     The current evaluation is not aborted. message indicates
     the cause of the warning [It is recommended to report the
     warning only once when the condition becomes false, and it is
     reported that the condition is no longer violated when the
     condition returns to true. The assert(..) statement shall
     have no influence on the behavior of the model.
     For example, by evaluating the condition and reporting the
     message only after accepted integrator steps. condition
     needs to be implicitly treated with noEvent(..) since
     otherwise events might be triggered that can lead to slightly
     changed simulation results].</li>
</ul>

<p>
The AssertionLevel.error case can be used to avoid evaluating a model outside its limits of validity; for instance, a function to compute the saturated liquid temperature cannot be called with a pressure lower than the triple point value.
The AssertionLevel.warning case can be used when the boundary of validity is not hard: for instance, a fluid property model based on a polynomial interpolation curve might give accurate results between temperatures of 250 K and 400 K, but still give reasonable results in the range 200 K and 500 K. When the temperature gets out of the smaller interval, but still stays in the largest one, the user should be warned, but the simulation should continue without any further action. The corresponding code would be
</p>
<pre>  <b>assert</b>(T &gt; 250 and T &lt; 400, \"Medium model outside full accuracy range\",
         AssertionLevel.warning);
  <b>assert</b>(T &gt; 200 and T &lt; 500, \"Medium model outside feasible region\");
</pre>

<h4><font color=\"#008000\">Examples</font></h4>
<pre>  <b>parameter</b> Real upperLimit=2;
  <b>parameter</b> Real lowerLimit=-2;
<b>equation</b>
  <b>assert</b>(upperLimit > lowerLimit, \"upperLimit must be greater than lowerLimit.\");
</pre>
</html>"));
    end Assert;
    class Atan "atan"
      annotation(Documentation(info = "<html>
<p>
Trigonometric inverse tangent function
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>atan</b>(u)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>

<p>Returns the inverse of tan of u, with -&infin; &lt; u &lt; &infin;.
Argument u needs to be an Integer or Real expression.</p>

<p>
The atan function can also be accessed as Modelica.Math.atan.
</p

<p>
<img src=\"../Images/atan.png\">
</p>



<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>atan</b>(1)
 = 0.785398163397448</pre>
</html>"));
    end Atan;
    class Atan2 "atan2"
      annotation(Documentation(info = "<html>
<p>
Four quadrant inverse tangent
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>atan2</b>(u1,u2)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>

<p>
Returns y = atan2(u1,u2) such that tan(y) = u1/u2 and
y is in the range -pi &lt; y &le; pi. u2 may be zero, provided
u1 is not zero. Usually u1, u2 is provided in such a form that
u1 = sin(y) and u2 = cos(y).
Arguments u1 and u2 need to be Integer or Real expressions.
</p>

<p>
The atan2 function can also be accessed as Modelica.Math.atan2.
</p

<p>
<img src=\"../Images/atan2.png\">
</p>



<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>atan2</b>(1,0)
 = 1.5707963267949</pre>
</html>"));
    end Atan2;
    class Cardinality "cardinality"
      annotation(Documentation(info = "<html>
<p>
Number of connectors in connection.
This is a deprecated operator. It should  no longer be used, since it will be removed in one of the next Modelica releases.
</p>

<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>cardinality</b>(c)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>Returns the number of (inside and outside) occurrences
of connector instance c in a connect statement as an Integer number.</p>
<p><i>[The cardinality operator allows the definition of connection dependent equations in a model.]</i></p>

<p>
Instead of the cardinality(..) operator, often conditional
connectors can be used, that are enabled/disabled via Boolean
parameters.
</p>

<h4><font color=\"#008000\">Examples</font></h4>
<PRE><B>connector</B> Pin
  Real      v;
  <B>flow</B> Real i;
<B>end</B> Pin;
<B>model</B> Resistor
   <B>Pin</B> p, n;
<B>equation</B>
   // Handle cases if pins are not connected
      <B>if cardinality</B>(p) == 0 <B>and cardinality</B>(n) == 0 <B>then</B>
         p.v = 0;
         n.v = 0;
      <B>elseif cardinality</B>(p) == 0 <B>then</B>
         p.i = 0;
      <B>elseif cardinality</B>(n) == 0 <B>then</B>
         n.i = 0;
      <B>end if</B>;
   // Equations of resistor
      ...
<B>end</B> Resistor;</PRE>
</html>"));
    end Cardinality;
    class Ceil "ceil"
      annotation(Documentation(info = "<html>
<p>
Round a Real number towards plus infinity
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>ceil</b>(x)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>Returns the smallest integer not less than <tt>x</tt>.
Result and argument shall have type Real.
<i>[Note, outside of a when clause state events are
triggered when the return value changes discontinuously.]</i></p>
<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>ceil</b>({-3.14, 3.14})
 = {-3.0, 4.0}</pre>
</html>"));
    end Ceil;
    class Change "change"
      annotation(Documentation(info = "<html>
<p>
Indicate discrete variable changing
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>change</b>(v)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<P>Is expanded into &quot;(v&lt;&gt;pre(v))&quot;.
The same restrictions as for the pre() operator apply.</P>
<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>model</b> BothEdges
  Boolean u;
  Integer i;
<b>equation</b>
  u = Modelica.Math.sin(<b>time</b>) &gt; 0.5;
  <b>when</b> <b>change</b>(u) <b>then</b>
    i = <b>pre</b>(i) + 1;
  <b>end when</b>;
<b>end</b> BothEdges;</pre>
<p align=\"center\"><img src=\"../Images/change.png\" width=\"400\" height=\"280\" alt=\"Simulation result\"></p>
</html>"));
    end Change;
    class Cos "cos"
      annotation(Documentation(info = "<html>
<p>
Trigonometric cosine function
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>cos</b>(u)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>Returns the cosine of u, with -&infin; &lt; u &lt; &infin;
Argument u needs to be an Integer or Real expression.</p>

<p>
The cosine function can also be accessed as Modelica.Math.cos.
</p>

<p>
<img src=\"../Images/cos.png\">
</p>

<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>cos</b>(3.14159265358979)
 = -1.0</pre>
</html>"));
    end Cos;
    class Cosh "cosh"
      annotation(Documentation(info = "<html>
<p>
Hyberbolic cosine function
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>cosh</b>(u)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>Returns the cosh of u, with -&infin; &lt; u &lt; &infin;.
Argument u needs to be an Integer or Real expression.</p>

<p>
The cosh function can also be accessed as Modelica.Math.cosh.
</p

<p>
<img src=\"../Images/cosh.png\">
</p>



<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>cosh</b>(1)
  = 1.54308063481524</pre>
</html>"));
    end Cosh;
    class Cross "cross"
      annotation(Documentation(info = "<html>
<p>
Return cross product of two vectors
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>cross</b>(x, y)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>
Returns the cross product of the 3-vectors x and y, i.e.
</p>
<blockquote><pre>
<b>cross</b>(x,y) = <b>vector</b>( [ x[2]*y[3]-x[3]*y[2];
                       x[3]*y[1]-x[1]*y[3];
                       x[1]*y[2]-x[2]*y[1] ] );
</pre></blockquote>
</html>"));
    end Cross;
    class Delay "delay"
      annotation(Documentation(info = "<html>
<p>
Delay expression
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>delay</b>(expr, delayTime, delayMax)
<b>delay</b>(expr, delayTime)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<P>Returns <TT>\"expr(time - delayTime)\"</TT> for <TT>time &gt; time.start + delayTime</TT>
and <TT>\"expr(time.start)\"</TT> for <TT>time &le; time.start + delayTime</TT>. The
arguments, i.e., <tt>expr</tt>, <tt>delayTime</tt> and <tt>delayMax</tt>, need to be subtypes of Real.
<TT>delayMax</TT> needs to be additionally a parameter expression. The following relation
shall hold: <TT>0 &le; delayTime &le; delayMax</tt>, otherwise an error occurs. If
<tt>delayMax</tt> is not supplied in the argument list, <tt>delayTime</tt> need to be a
parameter expression.</P>
<p><i>[The <b>delay</b> operator allows a numerical sound implementation
by interpolating in the (internal) integrator polynomials, as well as a
more simple realization by interpolating linearly in a buffer containing
past values of expression expr. Without further information, the complete
time history of the delayed signals need to be stored, because the delay
time may change during simulation. To avoid excessive storage requirements
and to enhance efficiency, the maximum allowed delay time has to be given
via delayMax. This gives an upper bound on the values of the delayed
signals which have to be stored. For realtime simulation where fixed step
size integrators are used, this information is sufficient to allocate the
necessary storage for the internal buffer before the simulation starts.
For variable step size integrators, the buffer size is dynamic during
integration. In principal, a delay operator could break algebraic loops.
For simplicity, this is not supported because the minimum delay time has
to be give as additional argument to be fixed at compile time. Furthermore,
the maximum step size of the integrator is limited by this minimum delay
time in order to avoid extrapolation in the delay buffer.]</i></p>
<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>model</b> Delay
  Real x;
  Real y;
<b>equation
  der</b>(x) = 2;
  y = <b>delay</b>(x, 1);
<b>end</b> Delay;</pre>
<p align=\"center\"><img src=\"../Images/delay.png\" width=\"400\" height=\"280\" alt=\"Simulation result\"></p>
</html>"));
    end Delay;
    class Der "der"
      annotation(Documentation(info = "<html>
<p>
Time derivative of expression or<br>
partial derivative of function
</p>

<h4><font color=\"#008000\">Syntax</font></h4>

<blockquote><pre><b>der</b>(expr) or
IDENT \"=\" <b>der</b> \"(\" name \",\" IDENT { \",\" IDENT } \")\" comment</pre>
</blockquote>

<h4><font color=\"#008000\">Description</font></h4>
<p>
The first form is the time derivative of expression expr.
If the expression expr is a scalar it needs to be a subtype of Real. The expression and all its subexpressions must be differentiable. If expr is an array, the operator is applied to all elements of the array. For Real parameters and
constants the result is a zero scalar or array of the same size as the
variable.</p>

<p>
The second form is the partial derivative of a function
and may only be used as declarations of functions.
The semantics is that a function [and only a function] can be specified in this form, defining that it is the partial derivative of the function to the right of the equal sign (looked up in the same way as a short class definition - the looked up name must be a function), and partially differentiated with respect to each IDENT in order (starting from the first one). The IDENT must be Real inputs to the function.
The comment allows a user to comment the function (in the info-layer and as one-line description, and as icon).
</p>


<h4><font color=\"#008000\">Examples</font></h4>

<blockquote>
<pre>  Real x, xdot1, xdot2;
<b>equation</b>
  xdot1 = <b>der</b>(x);
  xdot2 = <b>der</b>(x*sin(x));
</pre>
</blockquote>

<p>
The specific enthalphy can be computed from a Gibbs-function as follows:
</p>

<blockquote>
<pre><b>function</b> Gibbs
  <b>input</b> Real p,T;
  <b>output</b> Real g;
<b>algorithm</b>
  ...
<b>end</b> Gibbs;

<b>function</b> Gibbs_T=<b>der</b>(Gibbs, T);

<b>function</b> specificEnthalpy
  <b>input</b> Real p,T;
  <b>output</b> Real h;
<b>algorithm</b>
  h:=Gibbs(p,T)-T*Gibbs_T(p,T);
<b>end</b> specificEnthalpy;
</pre>
</blockquote>
</html>"));
    end Der;
    class Diagonal "diagonal"
      annotation(Documentation(info = "<html>
<p>
Returns a diagonal matrix
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>diagonal</b>(v)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>
Returns a square matrix with the elements of vector v
on the diagonal and all other elements zero.
</p>
</html>"));
    end Diagonal;
    class Div "div"
      annotation(Documentation(info = "<html>
<p>
Integer part of division of two Real numbers
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>div</b>(x, y)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<P>Returns the algebraic quotient <TT>x/y</TT> with any
fractional part discarded (also known as truncation
toward zero). <I>[Note: this is defined for / in C99;
in C89 the result for negative numbers is
implementation-defined, so the standard function
<TT>div()</TT> must be used.]</I> Result and arguments
shall have type Real or Integer. If either of the
arguments is Real the result is Real otherwise Integer.</P>

<p><i>[Note, outside of a when clause state events are triggered
when the return value changes discontinuously.]</i></p>

<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>div</b>(13,6)
 = 2</pre>
</html>"));
    end Div;
    class Edge "edge"
      annotation(Documentation(info = "<html>
<p>
Indicate rising edge
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>edge</b>(b)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<P>Is expanded into &quot;(b <B>and not pre</B>(b))&quot;
for Boolean variable b. The same restrictions as for the
<B>pre</B> operator apply (e.g., not to be used in function
classes).</P>
<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>model</b> RisingEdge
  Boolean u;
  Integer i;
<b>equation</b>
  u = Modelica.Math.sin(<b>time</b>) &gt; 0.5;
  <b>when</b> <b>edge</b>(u) <b>then</b>
    i = <b>pre</b>(i) + 1;
  <b>end when</b>;
<b>end</b> RisingEdge;</pre>
<p align=\"center\"><img src=\"../Images/edge.png\" width=\"400\" height=\"280\" alt=\"Simulation result\"></p>
</html>"));
    end Edge;
    class Exp "exp"
      annotation(Documentation(info = "<html>
<p>
Exponential, base e.
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>exp</b>(u)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>Returns the base e exponential of u, with -&infin; &lt; u &lt; &infin;
Argument u needs to be an Integer or Real expression.</p>

<p>
The exponential function can also be accessed as Modelica.Math.exp.
</p>

<p>
<img src=\"../Images/exp.png\">
</p>

<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>exp</b>(1)
 = 2.71828182845905</pre>
</html>"));
    end Exp;
    class Fill "fill"
      annotation(Documentation(info = "<html>
<p>
Return a Real, Integer, Boolean or String array with all elements equal
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>fill</b>(s, n1, n2, n3, ...)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>
Returns the n1 x n2 x n3 x ... array with all elements equal
to scalar or array expression s (ni >= 0). The returned
array has the same type as s. Recursive definition:
<p>
<blockquote><pre>
fill(s,n1,n2,n3, ...) = fill(fill(s,n2,n3, ...), n1);
fill(s,n) = {s,s,..., s}
</pre></blockquote>
<h4><font color=\"#008000\">Examples</font></h4>
<blockquote><pre>
Real    mr[2,2] = fill(-1,2,2);  // = [-1,-1;-1,-1]
Boolean vb[3]   = fill(true,3);  // = {true, true, true}
</pre></blockquote>
</html>"));
    end Fill;
    class Floor "floor"
      annotation(Documentation(info = "<html>
<p>
Round Real number towards minus infinity
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>floor</b>(x)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>Returns the largest integer not greater than <tt>x</tt>.
Result and argument shall have type Real. <i>[Note, outside
of a when clause state events are triggered when the return
value changes discontinuously.]</i></p>
<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>floor</b>({-3.14, 3.14})
 = {-4.0, 3.0}</pre>
</html>"));
    end Floor;
    class Identity "identity"
      annotation(Documentation(info = "<html>
<p>
Returns the identity matrix of the desired size
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>identity</b>(n)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>
Returns the n x n Integer identity matrix, with ones
on the diagonal and zeros at the other places.
</p>
</html>"));
    end Identity;
    class Initial "initial"
      annotation(Documentation(info = "<html>
<p>
True during initialization
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>initial</b>()</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>Returns <b>true</b> during the initialization phase and <b>false</b> otherwise.</p>
<h4><font color=\"#008000\">Examples</font></h4>
<pre>  Boolean off;
  Real x;
<b>equation</b>
  off = x &lt; -2 or <b>initial</b>();</pre>
</html>"));
    end Initial;
    class IntegerUpperCase "Integer"
      annotation(Documentation(info = "<html>
<p>
Returns ordinal number of enumeration
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>Integer</b>(E.e1)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>
Returns the ordinal number of the enumeration value E.enumvalue, where Integer(E.e1)=1, Integer(E.en)= size(E), for an enumeration type E=enumeration(e1, ...,  en).</p>

<h4><font color=\"#008000\">Examples</font></h4>
<pre>  <b>type</b> Size = enumeration(small, medium, large, xlarge);
  <b>Integer</b>(Size.large) = 3</pre>


</html>"));
    end IntegerUpperCase;
    class IntegerLowerCase "integer"
      annotation(Documentation(info = "<html>
<p>
Round Real number towards minus infinity
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>integer</b>(x)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>Returns the largest integer not greater than <tt>x</tt>.
The argument shall have type Real. The result has type Integer.</p>
<p><i>[Note, outside of a when clause state events are triggered
when the return value changes discontinuously.]</i></p>
<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>integer</b>({-3.14, 3.14})
 = {-4, 3}</pre>
</html>"));
    end IntegerLowerCase;
    class IsPresent "isPresent"
      annotation(Documentation(info = "<html>
<p>
Returns true if the formal input or output argument of a function is present
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>isPresent</b>(ident)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>
Returns true if the formal input or output argument ident is present
as an actual argument of the function call. If the argument is not
present, isPresent(ident) may return false <i>[but may also return
true e.g., for implementations that always compute all results]</i>.
isPresent() should be used for optimisation only and should not
influence the results of outputs that are present in the output
list. It can only be used in functions.
</p>
<h4><font color=\"#008000\">Examples</font></h4>
<blockquote>
<pre>
<b>function</b> eigenValues \"Compute eigenvalues and eigenvectors\"
  <b>input</b>  Real A[:, size(A, 1)];
  <b>output</b> Real eigenvalues[size(A, 1), 2];
  <b>output</b> Real eigenvectors[size(A,1), size(A,2)];
<b>algorithm</b>
  // compute eigen values
  <b>if</b> <b>isPresent</b>(eigenvectors) <b>then</b>
     // compute eigen vectors
  <b>end</b> <b>if</b>;
<b>end</b> eigenValues;
        eval = eigenValues(A);  // compute only eigenvalues
(eval, evec) = eigenValues(A);  // compute eigenvalues and eigenvectors
</pre>
</blockquote>
</html>"));
    end IsPresent;
    class Linspace "linspace"
      annotation(Documentation(info = "<html>
<p>
Return Real vector with equally spaced elements
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>linspace</b>(x1, x2, n)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>
Returns a Real vector with n equally spaced elements,
such that
</p>
<blockquote><pre>
v[i] = x1 + (x2-x1)*(i-1)/(n-1) for 1 &le; i &le; n.
</pre></blockquote>
<p>
It is required that n &ge; 2. The arguments x1 and x2 shall
be Real or Integer scalar expressions.
</p>
<h4><font color=\"#008000\">Examples</font></h4>
<blockquote><pre>
Real v[:] = linspace(1,7,4);  // = {1, 3, 5, 7}
</pre></blockquote>
</html>"));
    end Linspace;
    class Log "log"
      annotation(Documentation(info = "<html>
<p>
Natural (base e) logarithm
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>log</b>(u)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>Returns the base e logarithm of u, with u &gt; 0.
Argument u needs to be an Integer or Real expression.</p>

<p>
The natural logarithm can also be accessed as Modelica.Math.log.
</p>

<p>
<img src=\"../Images/log.png\">
</p>

<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>log</b>(1)
 = 0</pre>
</html>"));
    end Log;
    class Log10 "log10"
      annotation(Documentation(info = "<html>
<p>
Base 10 logarithm
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>log10</b>(u)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>Returns the base 10 logarithm of u, with u &gt; 0.
Argument u needs to be an Integer or Real expression.</p>

<p>
The base 10 logarithm can also be accessed as Modelica.Math.log10.
</p>

<p>
<img src=\"../Images/log10.png\">
</p>

<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>log10</b>(1)
 = 0</pre>
</html>"));
    end Log10;
    class Matrix "matrix"
      annotation(Documentation(info = "<html>
<p>
Returns the first two dimensions of an array as matrix
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>matrix</b>(A)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>
Returns <b>promote</b>(A,2), if A is a scalar or vector and
otherwise returns the elements of the first two dimensions
as a matrix. <b>size</b>(A,i) = 1 is required for
2 &lt; i &le; <b>ndims</b>(A).
</p>
<p>
Function <b>promote</b>(A,n) fills dimensions of size 1
from the right to array A upto dimension n, where
\"n &gt; <b>ndims</b>(A)\" is required. Let
C = <b>promote</b>(A,n), with nA = <b>ndims</b>(A),
then
</p>
<blockquote><pre>
<b>ndims</b>(C) = n,
<b>size</b>(C,j) = <b>size</b>(A,j) for 1 &le; j &le; nA,
<b>size</b>(C,j) = 1 for nA+1 &le; j &le; n,
C[i_1, ..., i_nA, 1, ..., 1] = A[i_1, ..., i_nA].
</pre></blockquote>
</html>"));
    end Matrix;
    class Max "max"
      annotation(Documentation(info = "<html>
<p>
Returns the largest element
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre>
<b>max</b>(A)
<b>max</b>(x,y)
<b>max</b>(e(i, ..., j) <b>for</b> i <b>in</b> u, ..., j <b>in</b> v)
</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>
The first form returns the largest element of array expression A.
</p>
<p>
The second form returns the largest element of the scalars x and y.
</p>
<p>
The third form is a reduction expression and
returns the largest value of the
scalar expression e(i, ..., j) evaluated for all
combinations of i in u, ..., j in v
</p>
<h4><font color=\"#008000\">Examples</font></h4>
<blockquote><pre>
<b>max</b>(i^2 <b>for</b> i <b>in</b> {3,7,6})  // = 49
</pre></blockquote>
</html>"));
    end Max;
    class Min "min"
      annotation(Documentation(info = "<html>
<p>
Returns the smallest element
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre>
<b>min</b>(A)
<b>min</b>(x,y)
<b>min</b>(e(i, ..., j) <b>for</b> i <b>in</b> u, ..., j <b>in</b> v)
</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>
The first form returns the smallest element of array expression A.
</p>
<p>
The second form returns the smallest element of the scalars x and y.
</p>
<p>
The third form is a reduction expression and
returns the smallest value of the
scalar expression e(i, ..., j) evaluated for all
combinations of i in u, ..., j in v
</p>
<h4><font color=\"#008000\">Examples</font></h4>
<blockquote><pre>
<b>min</b>(i^2 <b>for</b> i <b>in</b> {3,7,6})  // = 9
</pre></blockquote>
</html>"));
    end Min;
    class Mod "mod"
      annotation(Documentation(info = "<html>
<p>
Integer modulus of a division of two Real numbers
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>mod</b>(x, y)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<P>Returns the integer modulus of <TT>x/y</TT>, i.e., mod(x, y) = x - floor(x/y)*y.
Result and arguments shall have type Real or Integer. If either of the
arguments is Real the result is Real otherwise Integer. <I>[Note, outside of
a when clause state events are triggered when the return value changes
discontinuously.]</I></P>
<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>mod</b>(3,1.4)
 = 0.2
<b>mod</b>(-3,1.4)
 = 1.2
<b>mod</b>(3,-1.4)
 = -1.2</pre>
</html>"));
    end Mod;
    class Ndims "ndims"
      annotation(Documentation(info = "<html>
<p>
Return number of array dimensions
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>ndims</b>(A)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>
Returns the number of dimensions k of array expression
A, with k >= 0.
</p>
<h4><font color=\"#008000\">Examples</font></h4>
<blockquote><pre>
Real A[8,4,5];
Integer n = ndims(A);  // = 3
</pre></blockquote>
</html>"));
    end Ndims;
    class NoEvent "noEvent"
      annotation(Documentation(info = "<html>
<p>
Turn off event triggering
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>noEvent</b>(expr)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>Real elementary relations within expr are taken literally, i.e., no state or time event is triggered.</p>
<h4>smooth vs. noEvent</h4>
<p>The noEvent operator implies that real elementary expressions are taken
literally instead of generating crossing functions. The smooth operator
should be used instead of noEvent, in order to avoid events for efficiency
reasons. A tool is free to not generate events for expressions inside smooth.
However, smooth does not guarantee that no events will be generated, and thus
it can be necessary to use noEvent inside smooth. <I>[Note that smooth does
not guarantee a smooth output if any of the occurring variables change
discontinuously.]</i></p>
<p><i>[Example:</i></p>
<PRE>  Real x, y, z;
<B>equation</B>
  x = <B>if</B> time&lt;1 <B>then</B> 2 <B>else</B> time-2;
  z = smooth(0, <B>if</B> time&lt;0 <B>then</B> 0 <B>else</B> time);
  y = smooth(1, noEvent(<B>if</B> x&lt;0 <B>then</B> 0 <B>else</B> sqrt(x)*x));
  // noEvent is necessary.</PRE>
<p><i>]</i></p>
<h4><font color=\"#008000\">Examples</font></h4>
<pre>der(h)=<B>if noEvent</B>(h&gt;0) <B>then</B> -c*sqrt(h) <B>else</B> 0;</pre>
</html>"));
    end NoEvent;
    class Ones "ones"
      annotation(Documentation(info = "<html>
<p>
Returns an array with \"1\" elements
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>ones</b>(n1, n2, n3, ...)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>
Return the n1 x n2 x n3 x ... Integer array with all
elements equal to one (ni >=0 ).
</p>
</html>"));
    end Ones;
    class OuterProduct "outerProduct"
      annotation(Documentation(info = "<html>
<p>
Returns the outer product of two vectors
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>outerProduct</b>(v1,v2)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>
Returns the outer product of vectors v1 and v2 <br>
(= matrix(v)*transpose( matrix(v) ) ).
</p>
</html>"));
    end OuterProduct;
    class Pre "pre"
      annotation(Documentation(info = "<html>
<p>
Refer to left limit
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>pre</b>(y)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<P>Returns the &quot;left limit&quot; y(t<SUP>pre</SUP>)
of variable y(t) at a time instant t. At an event instant,
y(t<SUP>pre</SUP>) is the value of y after the last event
iteration at time instant t. The
<B>pre</B> operator can be applied if the following three
conditions are fulfilled simultaneously:</p>
<ul>
  <li>variable y is a subtype of a simple type</li>
  <li>y is a discrete-time expression</li>
  <li>the operator is <B>not</B> applied in a <B>function</B> class</li>
</ul>
<p>The first value of <B>pre</B>(y) is
determined in the initialization phase.</P>
<P>A new event is triggered if at least for one variable v
&quot;<B>pre</B>(v) &lt;&gt; v&quot; after the active model
equations are evaluated at an event instant. In this case
the model is at once reevaluated. This evaluation sequence
is called &quot;<I>event iteration</I>&quot;. The integration
is restarted, if for all v used in <B>pre</B>-operators the
following condition holds: &quot;<B>pre</B>(v) == v&quot;.</P>
<p><I>[If v and <B>pre</B>(v) are only used in when clauses,
the translator might mask event iteration for variable v since
v cannot change during event iteration. It is a &quot;quality
of implementation&quot; to find the minimal loops for event
iteration, i.e., not all parts of the model need to be
reevaluated.</i></p>
<p><i>The language allows mixed algebraic systems of equations
where the unknown variables are of type Real, Integer, Boolean,
or an enumeration. These systems of equations can be solved by
a global fix point iteration scheme, similarly to the event
iteration, by fixing the Boolean,  Integer, and/or enumeration
unknowns during one iteration. Again, it is a quality of
implementation to solve these systems more efficiently, e.g.,
by applying the fix point iteration scheme to a subset of the
model equations.]</I></p>
<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>model</b> Hysteresis
  Real u;
  Boolean y;
<b>equation</b>
  u = Modelica.Math.sin(<b>time</b>);
  y = u &gt; 0.5 or <b>pre</b>(y) and u &gt;= -0.5;
<b>end</b> Hysteresis;</pre>
<p align=\"center\"><img src=\"../Images/pre.png\" width=\"400\" height=\"280\" alt=\"Simulation result\"></p>
</html>"));
    end Pre;
    class Product "product"
      annotation(Documentation(info = "<html>
<p>
Returns the scalar product
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre>
<b>product</b>(A)
<b>product</b>(e(i, ..., j) <b>for</b> i <b>in</b> u, ..., j <b>in</b> v)
</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>
The first form returns the scalar product of all the elements of
array expression A:<br>
A[1,...,1]*A[2,...,1]*....*A[end,...,1]*A[end,...,end]
</p>
<p>
The second form is a reduction expression and
returns the product of the expression e(i, ..., j) evaluated for all combinations of i in u, ..., j in v:
</p>
<blockquote><pre>
e(u[1],...,v[1]) * e(u[2],...,v[1]) * ... *
e(u[end],...,v[1]) * ... * e(u[end],...,v[end])
</pre></blockquote>
<p>
The type of <b>product</b>(e(i, ..., j) <b>for</b> i <b>in</b>
u, ..., j <b>in</b> v) is the same as the type of e(i,...j).
</p>
<h4><font color=\"#008000\">Examples</font></h4>
<blockquote><pre>
{<b>product</b>(j <b>for</b> j <b>in</b> 1:i) <b>for</b> i <b>in</b> 0:4} // = {1,1,2,6,24}
</pre></blockquote>
</html>"));
    end Product;
    class Reinit "reinit"
      annotation(Documentation(info = "<html>
<p>
Reinitialize state variable
</p>
<h4><font color=\"#008000\">Examples</font></h4>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>reinit</b>(x, expr)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<P>Reinitializes state variable x with expr at an event
instant. Argument x need to be</p>
<ul>
  <li>a subtype of Real and</li>
  <li>the <B>der</B>-operator need to be applied to it.</li>
</ul>
<p>expr need to be an Integer or Real expression. The
reinit operator can only be applied once for the same
variable x. It can only be applied in the body of a
when-clause.</P>
<p>The <B>reinit</B> operator does not break the single
assignment rule, because <B>reinit</B>(x, expr) makes
the previously known state variable x unknown and
introduces the equation &quot;x = expr&quot;.</p>
<p><I>[If a higher index system is present, i.e.
constraints between state variables, some state variables
need to be redefined to non-state variables. If possible,
non-state variables should be chosen in such a way that
states with an applied <B>reinit</B> operator are not
utilized. If this is not possible, an error occurs,
because the reinit operator is applied on a non-state variable.]</i></P>
<PRE>Bouncing ball:
    <B>der</B>(h) = v;
    <B>der</B>(v) = -g;
    <B>when</B> h &lt; 0 <B>then</B>
      <B>reinit</B>(v, -e*v);
    <B>end when</B>;</PRE>
</html>"));
    end Reinit;
    class Rem "rem"
      annotation(Documentation(info = "<html>
<p>
Integer remainder of the division of two Real numbers
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>rem</b>(x, y)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<P>Returns the integer remainder of <TT>x/y</TT>,
such that <TT>div(x,y) * y + rem(x, y) = x</TT>.
Result and arguments shall have type Real or
Integer. If either of the arguments is Real the
result is Real otherwise Integer. <I>[Note, outside
of a when clause state events are triggered when
the return value changes discontinuously.]</I></P>
<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>rem</b>(3,1.4)
 = 0.2
<b>rem</b>(-3,1.4)
 = -0.2
<b>rem</b>(3,-1.4)
 = 0.2</pre>
</html>"));
    end Rem;
    class Sample "sample"
      annotation(Documentation(info = "<html>
<p>
Trigger time events
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>sample</b>(start, interval)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<P>Returns true and triggers time events at time instants
<TT>&quot;start + i*interval&quot; (i=0, 1, ...)</TT>.
During continuous integration the operator returns always
false. The starting time &quot;start&quot; and the sample
interval &quot;interval&quot; need to be parameter
expressions and need to be a subtype of Real or Integer.
</P>
<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>model</b> Sampling
  Integer i;
<b>equation</b>
  <b>when</b> <b>sample</b>(1, 0.1) <b>then</b>
    i = <b>pre</b>(i) + 1;
  <b>end when</b>;
<b>end</b> Sampling;</pre>
<p align=\"center\"><img src=\"../Images/sample.png\" width=\"400\" height=\"280\" alt=\"Simulation result\"></p>
</html>"));
    end Sample;
    class Scalar "scalar"
      annotation(Documentation(info = "<html>
<p>
Returns a one-element array as scalar
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>scalar</b>(A)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>
Returns the single element of array A.
<b>size</b>(A,i) = 1 is required for 1 &le; i &le; <b>ndims</b>(A).
</p>
<h4><font color=\"#008000\">Examples</font></h4>
<blockquote><pre>
Real A[1,1,1] = {{{3}}};
Real e = scalar(A);  // = 3
</pre></blockquote>
</html>"));
    end Scalar;
    class SemiLinear "semiLinear"
      annotation(Documentation(info = "<html>
<p>
Returns \"if x >= 0 then positiveSlope*x else negativeSlope*x\" and handle x=0 in a meaningful way
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>semiLinear</b>(x, positiveSlope, negativeSlope)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>
Returns \"if x >= 0 then positiveSlope*x else negativeSlope*x\".
In some situations, equations with the semiLinear function
become underdetermined if the first argument (x) becomes
zero, i.e., there are an infinite number of solutions.
It is recommended that the following rules are used to
transform the equations during the translation phase in
order to select one meaningful solution in such cases:
</p>
<p>
<b>Rule 1</b>: The equations
</p>
<blockquote>
<pre>
y = <b>semiLinear</b>(x, sa, s1);
y = <b>semiLinear</b>(x, s1, s2);
y = <b>semiLinear</b>(x, s2, s3);
   ...
y = <b>semiLinear</b>(x, sN, sb);
</pre>
</blockquote>
<p>
may be replaced by
</p>
<blockquote>
<pre>
s1 = <b>if</b> x >= 0 <b>then</b> sa <b>else</b> sb
s2 = s1;
s3 = s2;
   ...
sN = sN-1;
y = <b>semiLinear</b>(x, sa, sb);
</pre>
</blockquote>
<p>
<b>Rule 2</b>: The equations
</p>
<blockquote>
<pre>
x = 0;
y = 0;
y = <b>semiLinear</b>(x, sa, sb);
</pre>
</blockquote>
<p>
may be replaced by
</p>
<blockquote>
<pre>
x = 0
y = 0;
sa = sb;
</pre>
</blockquote>
<p>
<i> [For symbolic transformations, the following property is
useful (this follows from the definition):</i>
</p>
<blockquote>
<pre>
<b>semiLinear</b>(m_flow, port_h, h);
</pre>
</blockquote>
<p>
is identical to
</p>
<blockquote><pre>
-<b>semiLinear</b>(-m_flow, h, port_h);
</pre></blockquote>
<p>
<i> The semiLinear function is designed to handle reversing
flow in fluid systems, such as</i>
</p>
<blockquote><pre>
H_flow = <b>semiLinear</b>(m_flow, port.h, h);
</pre></blockquote>
<p>
<i> i.e., the enthalpy flow rate H _flow is computed from the mass flow
rate m_flow and the upstream specific enthalpy depending on the
flow direction.]</i>
</p>
</html>"));
    end SemiLinear;
    class Sign "sign"
      annotation(Documentation(info = "<html>
<p>
Signum function of a Real or Integer number
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>sign</b>(v)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>Is expanded into &quot;noEvent(<b>if</b> v &gt; 0 <b>then</b> 1 <b>else
if</b> v &lt; 0 <b>then</b> -1 <b>else</b> 0)&quot;. Argument v
needs to be an Integer or Real expression. <i>[Note, outside of a
when clause state events are triggered.]</i></p>
<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>sign</b>({-3, 0, 3})
 = {-1, 0, 1}</pre>
</html>"));
    end Sign;
    class Sin "sin"
      annotation(Documentation(info = "<html>
<p>
Trigonometric sine function
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>sin</b>(u)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>Returns the sine of u, with -&infin; &lt; u &lt; &infin;.
Argument u needs to be an Integer or Real expression.</p>

<p>
The sine function can also be accessed as Modelica.Math.sin.
</p

<p>
<img src=\"../Images/sin.png\">
</p>



<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>sin</b>(3.14159265358979)
 = 0.0</pre>
</html>"));
    end Sin;
    class Sinh "sinh"
      annotation(Documentation(info = "<html>
<p>
Hyberbolic sine function
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>sinh</b>(u)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>Returns the sinh of u, with -&infin; &lt; u &lt; &infin;.
Argument u needs to be an Integer or Real expression.</p>

<p>
The sinh function can also be accessed as Modelica.Math.sinh.
</p

<p>
<img src=\"../Images/sinh.png\">
</p>



<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>sinh</b>(1)
  = 1.1752011936438</pre>
</html>"));
    end Sinh;
    class Size "size"
      annotation(Documentation(info = "<html>
<p>
Returns dimensions of an array
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre>
   <b>size</b>(A,i)
   <b>size</b>(A)
</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>
The first form returns the size of dimension i of array expression A
where i shall be &gt; 0 and &le; <b>ndims</b>(A).
</p>
<p>
The second form returns a vector of length <b>ndims</b>(A)
containing the dimension sizes of A.
</p>
<h4><font color=\"#008000\">Examples</font></h4>
<blockquote><pre>
Real A[8,4,5];
Integer n3   = size(A,3);  // = 5
Integer n[:] = size(A);    // = {8,4,5}
</pre></blockquote>
</html>"));
    end Size;
    class Skew "skew"
      annotation(Documentation(info = "<html>
<p>
Returns the skew matrix that is associated with a vector
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>skew</b>(x)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>
Returns the 3 x 3 skew symmetric matrix associated with a
3-vector, i.e.,
</p>
<blockquote><pre>
<b>cross</b>(x,y) = <b>skew</b>(x)*y;
<b>skew</b>(x) = [ 0   , -x[3],  x[2];
            x[3],  0   , -x[1];
           -x[2],  x[1],  0   ];
</pre></blockquote>
</html>"));
    end Skew;
    class Smooth "smooth"
      annotation(Documentation(info = "<html>
<p>
Indicate smoothness of expression
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>smooth</b>(p, expr)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<P>If p&gt;=0 <B>smooth</B>(p, expr) returns expr and states
that expr is p times continuously differentiable, i.e.: expr
is continuous in all real variables appearing in the expression
and all partial derivatives with respect to all appearing real
variables <B>exist</B> and are continuous up to order p.</p>
<p>The only allowed types for expr in smooth are: real expressions,
arrays of allowed expressions, and records containing only
components of allowed expressions.</P>
<h4>smooth vs. noEvent</h4>
<p>The noEvent operator implies that real elementary expressions are taken
literally instead of generating crossing functions. The smooth operator
should be used instead of noEvent, in order to avoid events for efficiency
reasons. A tool is free to not generate events for expressions inside smooth.
However, smooth does not guarantee that no events will be generated, and thus
it can be necessary to use noEvent inside smooth. <I>[Note that smooth does
not guarantee a smooth output if any of the occurring variables change
discontinuously.]</I></p>
<h4><font color=\"#008000\">Examples</font></h4>
<PRE>  Real x, y, z;
<B>equation</B>
  x = <B>if</B> time&lt;1 <B>then</B> 2 <B>else</B> time-2;
  z = smooth(0, <B>if</B> time&lt;0 <B>then</B> 0 <B>else</B> time);
  y = smooth(1, noEvent(<B>if</B> x&lt;0 <B>then</B> 0 <B>else</B> sqrt(x)*x));
  // noEvent is necessary.</PRE>
</html>"));
    end Smooth;
    class Sqrt "sqrt"
      annotation(Documentation(info = "<html>
<p>
Square root
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>sqrt</b>(v)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>Returns the square root of v if v&gt;=0, otherwise an error occurs.
Argument v needs to be an Integer or Real expression.</p>
<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>sqrt</b>(9)
 = 3.0</pre>
</html>"));
    end Sqrt;
    class string "String"
      annotation(Documentation(info = "<html>
<p>
Convert a scalar Real, Integer or Boolean expression to a String representation
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote>
<pre>
<b>String</b>(b_expr, minimumLength=0, leftJustified=true)
<b>String</b>(i_expr, minimumLength=0, leftJustified=true)
<b>String</b>(r_expr, significantDigits=6, minimumLength=0, leftJustified=true)
<b>String</b>(r_expr, format)
<b>String</b>(e_expr, minimumLength=0, leftJustified=true)
</pre>
</blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>
The arguments have the following meaning
(the default values of the optional arguments are shown in the left column):
</p>
<table border=1 cellspacing=0 cellpadding=2>
<tr>
  <td> Boolean b_expr</td>
  <td> Boolean expression</td></tr>
<tr>
  <td> Integer i_expr</td>
  <td> Integer expression</td></tr>
<tr>
  <td> Real r_expr</td>
  <td> Real expression</td></tr>
<tr>
  <td> type e_expr = enumeration(..)</td>
  <td> Enumeration expression</td></tr>
<tr>
  <td> Integer minimumLength = 0 </td>
  <td> Minimum length of the resulting string. If necessary, <br>
       the blank character is used to fill up unused space.</td></tr>
<tr>
  <td> Boolean leftJustified = <b>true</b> </td>
  <td> if <b>true</b>, the converted result is left justified;<br>
       if <b>false</b>, it is right justified in the string.</td></tr>
<tr>
  <td> Integer significantDigits = 6 </td>
  <td> defines the number of significant digits in the result string<br>
       (e.g., \"12.3456\", \"0.0123456\", \"12345600\", \"1.23456E-10\")</td></tr>
<tr>
  <td> String format </td>
  <td> defines the string formating according to ANSI-C without \"%\" and \"*\" character<br>
       (e.g., \".6g\", \"14.5e\", \"+6f\"). In particular:<br>
&nbsp;&nbsp;&nbsp;&nbsp;
format = \"[&lt;flags&gt;] [&lt;width&gt;] [.&lt;precision&gt;] &lt;conversion&gt;\"<br>
with
<table>
<tr>
  <td>&lt;flags&gt;</td>
  <td> zero, one or more of<br>
       \"-\": left adjustment of the converted number<br>
       \"+\": number will always be printed with a sign<br>
       \"0\": padding to the field width with leading zeros</td></tr>
<tr>
  <td>&lt;width&gt;</td>
  <td> Minimum field width. The converted number will be printed in a field at<br>
       least this wide and wider if necessary. If the converted number has <br>
       fewer characters it will be padded on the left (or the right depending<br>
       on &lt;flags&gt;) with blanks or 0 (depending on &lt;flags&gt;).</td></tr>
<tr>
  <td>&lt;precision&gt;</td>
  <td> The number of digits to be printed after the decimal point for <br>
       e, E, or f conversions, or the number of significant digits for <br>
       g or G conversions.</td></tr>
<tr>
  <td> &lt;conversion&gt;</td>
  <td> = \"e\": Exponential notation using a  lower case e<br>
       = \"E\": Exponential notation using an upper case E<br>
       = \"f\": Fixed point notation<br>
       = \"g\": Either \"e\" or \"f\"<br>
       = \"G\": Same as \"g\", but with upper case E</td></tr></table>
</table>
<h4><font color=\"#008000\">Examples</font></h4>
<blockquote>
<pre>
String(2.0)   // = \"2.0\"
String(true)  // = \"true\"
String(123, minimumLength=6, leftJustified=false)  // = \"   123\"
</pre>
</blockquote>
</html>"));
    end string;
    class Sum "sum"
      annotation(Documentation(info = "<html>
<p>
Returns the scalar sum
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre>
<b>sum</b>(A)
<b>sum</b>(e(i, ..., j) <b>for</b> i <b>in</b> u, ..., j <b>in</b> v)
</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>
The first form returns the scalar sum of all the elements of
array expression A:<br>
A[1,...,1]+A[2,...,1]+....+A[end,...,1]+A[end,...,end]
</p>
<p>
The second form is a reduction expression and
returns the sum of the expression e(i, ..., j) evaluated for all combinations of i in u, ..., j in v:
</p>
<blockquote><pre>
e(u[1],...,v[1]) + e(u[2],...,v[1]) + ... +
e(u[end],...,v[1]) + ... + e(u[end],...,v[end])
</pre></blockquote>
<p>
The type of <b>sum</b>(e(i, ..., j) <b>for</b> i <b>in</b>
u, ..., j <b>in</b> v) is the same as the type of e(i,...j).
</p>
<h4><font color=\"#008000\">Examples</font></h4>
<blockquote><pre>
<b>sum</b>(i <b>for</b> i <b>in</b> 1:10)  // Gives  1+2+...+10=55
   // Read it as: compute the sum of i for i in the range 1 to 10.
</pre></blockquote>
</html>"));
    end Sum;
    class Symmetric "symmetric"
      annotation(Documentation(info = "<html>
<p>
Returns a symmetric matrix
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>symmetric</b>(A)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>
Returns a matrix where the diagonal elements and the
elements above the diagonal are identical to the
corresponding elements of matrix A and where the
elements below the diagonal are set equal to the elements
above the diagonal of A, i.e.,
<p>
<blockquote><pre>
B := <b>symmetric</b>(A)
     -> B[i,j] := A[i,j], <b>if</b> i &le; j,
        B[i,j] := A[j,i], <b>if</b> i &gt; j.
</pre></blockquote>
</html>"));
    end Symmetric;
    class Tan "tan"
      annotation(Documentation(info = "<html>
<p>
Trigonometric tangent function
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>tan</b>(u)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>Returns the tangent of u, with -&infin; &lt; u &lt; &infin;
(if u is a multiple of (2n-1)*pi/2, y = tan(u) is +/- infinity).
Argument u needs to be an Integer or Real expression.</p>

<p>
The tangent function can also be accessed as Modelica.Math.tan.
</p

<p>
<img src=\"../Images/tan.png\">
</p>

<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>tan</b>(3.14159265358979)
 = 0.0
</pre>
</html>"));
    end Tan;
    class Tanh "tanh"
      annotation(Documentation(info = "<html>
<p>
Hyberbolic tangent function
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>tanh</b>(u)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>Returns the tanh of u, with -&infin; &lt; u &lt; &infin;.
Argument u needs to be an Integer or Real expression.</p>

<p>
The tanh function can also be accessed as Modelica.Math.tanh.
</p

<p>
<img src=\"../Images/tanh.png\">
</p>



<h4><font color=\"#008000\">Examples</font></h4>
<pre><b>tanh</b>(1)
  = 0.761594155955765</pre>
</html>"));
    end Tanh;
    class Terminal "terminal"
      annotation(Documentation(info = "<html>
<p>
True after successful analysis
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>terminal</b>()</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>Returns <b>true</b> at the end of a successful analysis.</p>
<h4><font color=\"#008000\">Examples</font></h4>
<pre>  Boolean a, b;
<b>equation</b>
  a = <b>change</b>(b) or <b>terminal</b>();</pre>
</html>"));
    end Terminal;
    class Terminate "terminate"
      annotation(Documentation(info = "<html>
<p>
Successfully terminate current analysis
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>terminate</b>(message)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<P>The terminate function successfully terminates the analysis
which was carried out. The function has a string argument
indicating the reason for the success. <I>[The intention is to
give more complex stopping criteria than a fixed point in time.]</I></P>
<h4><font color=\"#008000\">Examples</font></h4>
<pre><B>model</B> ThrowingBall
  Real x(start=0);
  Real y(start=1);
<B>equation</B>
  <b>der</b>(x)= ... ;
  <b>der</b>(y)= ... ;
<B>algorithm</B>
  <B>when</B> y &lt; 0 <B>then</B>
    <b>terminate</b>(\"The ball touches the ground\");
  <B>end when;
end</B> ThrowingBall;</pre>
</html>"));
    end Terminate;
    class Transpose "transpose"
      annotation(Documentation(info = "<html>
<p>
Transpose of a matrix or permutation of the first two dimensions of an array
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>transpose</b>(A)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>
Permutes the first two dimensions of array A.
It is an error, if array A does not have at least
2 dimensions.
</p>
</html>"));
    end Transpose;
    class Vector "vector"
      annotation(Documentation(info = "<html>
<p>
Returns an array with one non-singleton dimension as vector
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>vector</b>(A)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>
Returns a 1-vector, if A is a scalar and otherwise returns a vector containing all the elements of the array, provided there is at
most one dimension size > 1.
</p>
<h4><font color=\"#008000\">Examples</font></h4>
<blockquote><pre>
Real A[1,2,1] = {{{3},{4}}};
Real v[2] = vector(A);  // = {3,4}
</pre></blockquote>
</html>"));
    end Vector;
    class Zeros "zeros"
      annotation(Documentation(info = "<html>
<p>
Returns a zero array.
</p>
<h4><font color=\"#008000\">Syntax</font></h4>
<blockquote><pre><b>zeros</b>(n1, n2, n3, ...)</pre></blockquote>
<h4><font color=\"#008000\">Description</font></h4>
<p>
Returns the n1 x n2 x n3 x ... Integer array with all
elements equal to zero (ni >= 0).
</p>
</html>"));
    end Zeros;
  end Operators;
  class BalancedModel "balanced model"
    annotation(Documentation(info = "<html>
<p>
The basic concept to count unknowns and equations.
</p>

<p>
Restrictions for model and block classes are present, in order that missing
or too many equations can be detected and localized by a Modelica
translator before using the respective model or block class.
</p>

<h4><font color=\"#008000\">Examples</font></h4>

<pre>   <b>partial model</b> BaseCorrelation
     <b>input</b> Real x;
     Real y;
   <b>end</b> BaseCorrelation;

   <b>model</b> SpecialCorrelation // correct in Modelica 2.2 and 3.0
      <b>extends</b> BaseCorrelation(x=2);
   <b>equation</b>
       y=2/x;
   <b>end</b> SpecialCorrelation;

   <b>model</b> UseCorrelation // correct according to Modelica 2.2
                        // not valid according to Modelica 3.0
      <b>replaceable model</b> Correlation=BaseCorrelation;
      Correlation correlation;
   <b>equation</b>
      correlation.y=time;
   <b>end</b> UseCorrelation;

   <b>model</b> Broken // after redeclaration, there is 1 equation too much in Modelica 2.2
      UseCorrelation example(redeclare Correlation=SpecialCorrelation);
   <b>end</b> Broken;
</pre>

<p>
In this case one can argue that both UseCorrelation (adding an acausal equation) and SpecialCorrelation (adding a default to an input) are correct, but still when combined they lead to a model with too many equations - and it is not possible to determine which model is incorrect without strict rules.
In Modelica 2.2, model Broken will work with some models. However, by just redeclaring it to model SpecialCorrelation, an error will occur and it will be very difficult in a larger model to figure out the source of this error.
In Modelica 3.0, model UseCorrelation is no longer allowed and the translator will give an error. In fact, it is guaranteed that a redeclaration cannot lead to an unbalanced model any more.
</p>


<h4><font color=\"#008000\">Description</font></h4>

<p>
The restrictions below apply after flattening \" i.e., inherited components are included \" possibly modified.
</p>

<p>
<b>Definition 1: Local Number of Unknowns</b>
</p>

<p>
The local number of unknowns of a model or block class is the sum based on the components:
</p>

<ul>
<li> For each declared component of specialized class type (Real, Integer, String, Boolean, enumeration and arrays of those, etc.) or record, not declared as outer, it is the \"number of unknown variables\" inside it (i.e., excluding parameters and constants and counting the elements after expanding all records and arrays to a set of scalars of primitive types). </li>

<li> Each declared component  of specialized class type or record declared as outer is ignored [i.e., all variables inside the component are treated as known].</li>

<li> For each declared component of specialized class connector component, it is the \"number of unknown variables\" inside it (i.e., excluding parameters and constants and counting the elements after expanding all records and arrays to a set of scalars of primitive types).</li>

<li> For each declared component of specialized class block or model, it is the \"sum of the number of inputs and flow variables\" in the (top level) public connector components of these components (and counting the elements after expanding all records and arrays to a set of scalars of primitive types).</li>
</ul>


<p>
<b>Definition 2: Local Equation Size </b>
</p>

<p>
The local equation size of a model or block class is the sum of the following numbers:
</p>

<ul>
<li>The number of equations defined locally (i.e., not in any model or block component), including binding equations, and equations generated from connect-equations. This includes the proper count for when-clauses, and algorithms, and is also used for the flat Hybrid DAE formulation.</li>

<li> The number of input and flow-variables present in each (top-level) public connector component. [This represents
the number of connection equations that will be provided when the class is used.]</li>

<li> The number of (top level) public input variables that neither are connectors nor have binding equations [i.e., top-level inputs are treated as known variables. This represents the number of binding equations that will be provided when the class is used.].
[To clarify top-level inputs without binding equation (for non-inherited inputs binding equation is identical to declaration equation, but binding equations also include the case where another model extends M and has a modifier on \"u\" giving the value):
<pre>  model M
    input Real u;
    input Real u2=2;
  end M;
</pre>
Here \"u\" and \"u2\" are top-level inputs and not connectors. The variable u2 has a binding equation, but u does not have a binding equation. In the equation count, it is assumed that an equation for u is supplied when using the model.]</li>
</ul>

<p>
<b>Definition 3: Locally Balanced</b>
</p>

<p>
A model or block class is \"locally balanced\" if the \"local number of unknowns\" is identical to the \"local equation size\" for all legal values of constants and parameters [respecting final bindings and min/max-restrictions. A tool shall verify the \"locally balanced\" property for the actual values of parameters and constants in the simulation model. It is a quality of implementation  for a tool to verify this property in general, due to arrays of (locally) undefined sizes, conditional declarations, for loops, etc.].
</p>


<p>
<b>Definition 4: Globally Balanced</b>
</p>

<p>
Similarly as locally balanced, but including all unknowns and equations from all components. The global number of unknowns is computed by expanding all unknowns (i.e., excluding parameters and constants) into a set of scalars of primitive types. This should match the global equation size defined as:
</p>

<ul>
<li> The number of equations defined (included in any model or block component), including equations generated from connect-equations.</li>
<li>The number of input and flow-variables present in each (top-level) public connector component.</li>
<li> The number of (top level) public input variables that neither are connectors nor have binding equations [i.e., top-level inputs are treated as known variables].</li>
</ul>

<p>
The following restrictions hold:
</p>

<ul>
<li>In a non-partial model or block, all non-connector inputs of model or block components must have binding equations. [E.g., if the model contains a component, firstOrder (of specialized class model) and firstOrder has  \"input Real u\" then there must be a binding equation for firstOrder.u.] </li>
<li>A component declared with the inner or outer prefix shall not be of a class having top-level public connectors containing inputs.</li>
<li>Modifiers for components shall only contain redeclarations of replaceable elements and binding equations for parameters, constants (that do not yet have binding equations), inputs and variables having a default binding equation.</li>
<li>All non-partial model and block classes must be locally balanced [this means that the local number of unknowns equals the local equation size].</li>
</ul>

<p>
Based on these restrictions, the following strong guarantee can be given for simulation models and blocks:
</p>


<p>
<b>Proposition 1: All simulation models and blocks are globally balanced.</b><br>
[Therefore the number of unknowns equal to the number of equations of a simulation model or block, provided that every used non-partial model or block class is locally balanced.]
</p>

<pre>Example 1:

connector Pin
   Real v;
   flow Real i;
end Pin;

model Capacitor
   parameter Real C;
   Pin  p, n;
   Real u;
equation
   0 = p.i + n.i;
   u = p.v - n.v;
   C*der(u) = p.i;
end Capacitor;
</pre>

<p>
Model Capacitor is a locally balanced model according to the following analysis:
</p>

<pre>Locally unknown variables: p.i, p.v, n.i, n.v, u
Local equations:  0 = p.i + n.i;
                  u = p.v - n.v;
                  C*der(u) = p.i;
                  and 2 equations corresponding to the
                  2 flow-variables p.i and n.i.
</pre>

<p>
These are 5 equations in 5 unknowns (locally balanced model). A more detailed analysis would reveal that this is structurally non-singular, i.e., that the hybrid DAE will not contain a singularity independent of actual values.
If the equation \"u = p.v - n.v\" would be missing in the Capacitor model, there would be 4 equations in 5 unknowns and the model would be locally unbalanced and thus simulation models in which this model is used would be usually  structurally singular and  thus not solvable.
If the equation \"u = p.v - n.v\" would be replaced by the equation \"u = 0\" and the equation C*der(u) = p.i would be replaced by the equation \"C*der(u) = 0\", there would be 5 equations in 5 unknowns (locally balanced), but the equations would be singular, regardless of how the equations corresponding to the flow-variables are constructed because the information that \"u\" is constant is given twice in a slightly different form.
</p>

<pre>Example 2:

connector Pin
   Real v;
   flow Real i;
end Pin;

partial model TwoPin
   Pin p,n;
end TwoPin;

model Capacitor
   parameter Real C;
   extends TwoPin;
   Real u;
equation
   0 = p.i + n.i;
   u = p.v \" n.v;
   C*der(u) = p.i;
end Capacitor;

model Circuit
   extends TwoPin;
   replaceable TwoPin t;
   Capacitor c(C=12);
equation
   connect(p, t.p);
   connect(t.n, c.p);
   connect(c.n, n);
end Circuit;
</pre>

<p>
Since t is partial we cannot check whether this is a globally balanced model, but we can check that Circuit is locally balanced.
Counting on  model Circuit results in the following balance sheet:
</p>

<pre>Locally unknown variables (8): p.i, p.v, n.i, n.v, and 2 flow variables for t (t.p.i, t.n.i)
                                                   and 2 flow variable for c (c.p.i, c.n.i).
Local equations:     p.v = t.p.v;
                       0 = p.i - t.p.i;
                   c.p.v = load.n.v;
                       0 = c.p.i+load.n.i;
                     n.v = c.n.v;
                       0 = n.i - c.n.i;
                    and 2 equation corresponding to the
                    flow variables p.i, n.i
</pre>

<p>
In total we have 8 scalar unknowns and 8 scalar equations, i.e., a locally balanced model (and this feature holds for any models used for the replaceable component \"t\").
Some more analysis reveals that this local set of equations and unknowns is structurally non-singular. However, this does not provide any guarantees for the global set of equations, and specific combinations of models that are \"locally non-singular\" may lead to a globally non-singular model.]
</p>

<pre>Example 3:

import SI = Modelica.SIunits;
partial model BaseProperties
   \"Interface of medium model for all type of media\"
   parameter Boolean preferredMediumStates=false;
   constant  Integer nXi \"Number of independent mass fractions\";
   InputAbsolutePressure     p;
   InputSpecificEnthalpy     h;
   InputMassFraction         Xi[nXi];
   SI.Temperature            T;
   SI.Density                d;
   SI.SpecificInternalEnergy u;

   connector InputAbsolutePressure = input SI.AbsolutePressure;
   connector InputSpecificEnthalpy = input SI.SpecificEnthalpy;
   connector InputMassFraction = input SI.MassFraction;
end BaseProperties;
</pre>

<p>
The use of connector here is a special design pattern. The variables p, h, Xi are marked as input to get correct equation count. Since they are connectors they should neither be given binding equations in derived classes nor when using the model. The design pattern is to give textual equations for them (as below); using connect-statements for these connectors would be possible (and would work) but is not part of the design.
This partial model defines that T,d,u can be computed from the medium model, provided p,h,Xi are given. Every medium with one or multiple substances and one or multiple phases, including incompressible media, has the property that T,d,u can be computed from p,h,Xi. A particular medium may have different \"independent variables\" from which all other intrinsic thermodynamic variables can be recursively computed. For example, a simple air model could be defined as:
</p>

<pre>model SimpleAir \"Medium model of simple air. Independent variables: p,T\"
   extends BaseProperties(nXi = 0,
        p(stateSelect = if preferredMediumStates then StateSelect.prefer
                                   else StateSelect.default),
        T(stateSelect = if preferredMediumStates then StateSelect.prefer
                                   else StateSelect.default));
   constant SI.SpecificHeatCapacity R  = 287;
   constant SI.SpecificHeatCapacity cp = 1005.45;
   constant SI.Temperature          T0 = 298.15
equation
   d = p/(R*T);
   h = cp*(T-T0);
   u = h \" p/d;
end SimpleAir;
</pre>

<p>
The local number of unknowns in model SimpleAir (after flattening) is:
</p>

<ul>
<li> 3 (T, d, u: variables defined in BaseProperties and inherited in SimpleAir), plus</li>
<li> 2+nXi (p, h, Xi: variables inside connectors defined in BaseProperties and inherited in SimpleAir)
resulting in 5+nXi unknowns.</li>
</ul>

<p>
The local equation size is:
</p>

<ul>
<li> 3 (equations defined in SimpleAir), plus</li>
<li> 2+nXi (input variables in the connectors inherited from BaseProperties)</li>
</ul>

<p>
Therefore, the model is locally balanced.
The generic medium model BaseProperties is used as a replaceable model in different components, like a dynamic volume or a fixed boundary condition:
</p>

<pre>import SI = Modelica.SIunits
connector FluidPort
  replaceable model Medium = BaseProperties;

  SI.AbsolutePressure  p;
  flow SI.MassFlowRate m_flow;

  SI.SpecificEnthalpy      h;
  flow SI.EnthalpyFlowRate H_flow;

  SI.MassFraction       Xi     [Medium.nXi] \"Independent mixture mass fractions\";
  flow SI.MassFlowRate mXi_flow[Medium.nXi] \"Independent subst. mass flow rates\";
end FluidPort;

model DynamicVolume
   parameter SI.Volume V;
   replaceable model Medium = BaseProperties;
   FluidPort port(redeclare model Medium = Medium);
   Medium    medium(preferredMediumStates=true); // No modifier for p,h,Xi
   SI.InternalEnergy U;
   SI.Mass           M;
   SI.Mass           MXi[medium.nXi];
equation
   U   = medium.u*M;
   M   = medium.d*V;
   MXi = medium.Xi*M;
   der(U)   = port.H_flow;   // Energy balance
   der(M)   = port.m_flow;   // Mass balance
   der(MXi) = port.mXi_flow; // Substance mass balance

// Equations binding to medium (inputs)
   medium.p  = port.p;
   medium.h  = port.h;
   medium.Xi = port.Xi;
end DynamicVolume;
</pre>

<p>
The local number of unknowns of DynamicVolume is:
</p>

<ul>
<li> 4+2*nXi (inside the port connector), plus</li>
<li> 2+nXi (variables U, M and MXi), plus </li>
<li> 2+nXi (the input variables in the connectors of the medium model)
</ul>

<p>
resulting in 8+4*nXi unknowns; the local equation size is
</p>

<ul>
<li> 6+3*nXi from the equation section, plus</li>
<li> 2+nXi flow variables in the port connector.</li>
</ul>

<p>
Therefore, DynamicVolume is a locally balanced model.
Note, when the DynamicVolume is used and the Medium model is redeclared to \"SimpleAir\", then a tool will try to select p,T as states, since these variables have StateSelect.prefer in the SimpleAir model (this means that the default states U,M are derived quantities). If this state selection is performed, all intrinsic medium variables are computed from medium.p and medium.T, although p and h are the input arguments to the medium model. This demonstrates that in Modelica input/output does not define the computational causality. Instead, it defines that equations have to be provided here for p,h,Xi, in order that the equation count is correct. The actual computational causality can be different as it is demonstrated with the SimpleAir model.
</p>

<pre>model FixedBoundary_pTX
   parameter SI.AbsolutePressure p \"Predefined boundary pressure\";
   parameter SI.Temperature      T \"Predefined boundary temperature\";
   parameter SI.MassFraction     Xi[medium.nXi]
                                   \"Predefined boundary mass fraction\";
   replaceable model Medium = BaseProperties;
   FluidPort port(redeclare model Medium = Medium);
   Medium medium;
equation
   port.p        = p;
   port.H_flow   = semiLinear(port.m_flow, port.h , medium.h);
   port.MXi_flow = semiLinear(port.m_flow, port.Xi, medium.Xi);

// Equations binding to medium (note: T is not an input).
   medium.p  = p;
   medium.T  = T;
   medium.Xi = Xi;
end FixedBoundary_pTX;
</pre>

<p>
The number of local variables in FixedBoundary_pTX is:
</p>

<ul>
<li> 4+2*nXi (inside the port connector), plus</li>
<li> 2+nXi (the input variables in the connectors of the medium model)</li>
</ul>

<p>
resulting in 6+3*nXi unknowns, while the local equation size is
</p>

<ul>
<li> 4+2*nXi from the equation section, plus</li>
<li> 2+nXi flow variables in the port connector.</li>
</ul>

<p>
Therefore, FixedBoundary_pTX is a locally balanced model. The predefined boundary variables p and Xi are provided via equations to the input arguments medium.p and medium.Xi, in addition there is an equation for T in the same way \" even though T is not an input. Depending on the flow direction, either the specific enthalpy in the port (port.h) or h is used to compute the enthalpy flow rate H_flow. \"h\" is provided as binding equation to the medium. With the equation \"medium.T = T\", the specific enthalpy \"h\" of the reservoir is indirectly computed via the medium equations. Again, this demonstrates, that an \"input\" just defines the number of equations have to be provided, but that it not necessarily defines the computational causality.
</p>

</html>"));
  end BalancedModel;
  class Connect "connect"
    annotation(Documentation(info = "<html>
<p>
Connect objects
</p>
<h4><font color=\"#008000\">Examples</font></h4>

<pre><b>model</b> Integrate
  Modelica.Blocks.Sources.Step step;
  Modelica.Blocks.Continuous.Integrator integrator;
<b>equation</b>
  connect(step.outPort, integrator.inPort);
<b>end</b> Integrate;</pre>

<p>Example of array use:</p>

<pre><B>connector</B> InPort = <B>input</B> Real;

<B>connector</B> OutPort = <B>output</B> Real;

<B>block</B> MatrixGain
  <B>input</B> InPort u[size(A,1)];
  <B>output</B> OutPort y[size(A,2)]
  <B>parameter</B> Real A[:,:]=[1];
<B>equation</B>
  y=A*u;
<B>end</B> MatrixGain;


  sin sinSource[5];
  MatrixGain gain(A=5*identity(5));
  MatrixGain gain2(A=ones(5,2));
  OutPort x[2];
<B>equation</B>
  <B>connect</B>(sinSource.y, gain.u); // Legal
  <B>connect</B>(gain.y, gain2.u);     // Legal
  <B>connect</B>(gain2.y, x);          // Legal</pre>

<h4><font color=\"#008000\">Syntax</font></h4>

<PRE>equation_clause :
  [ <B>initial</B> ] <B>equation</B> { equation \";\" | annotation  \";\" }

equation :
  ( simple_expression \"=\" expression
    | conditional_equation_e
    | for_clause_e
    | connect_clause
    | when_clause_e
    | IDENT function_call )
  comment

connect_clause :
  <B>connect</B> \"(\" component_reference \",\" component_reference \")\"</PRE>

<h4><font color=\"#008000\">Description</font></h4>

<P>Connections between objects are introduced by the <B>connect</B>
statement in the equation part of a class. The <B>connect</B>
construct takes two references to connectors, each of which is
either of the following forms:</P>

<UL>
  <LI>c1.c2. ... .cn, where c1 is a connector of the class, n&ge;1
      and ci+1 is a connector element of ci for i=1:(n-1).</LI>
  <LI>m.c, where m is a non-connector element in the class and c is
      a connector element of m.</LI>
</UL>

<P>There may optionally be array subscripts on any of the components;
the array subscripts shall be parameter expressions. If the connect
construct references array of connectors, the array dimensions must
match, and each corresponding pair of elements from the arrays is
connected as a pair of scalar connectors.</P>

<P>The two main tasks are to:</P>

<UL>
  <LI>Build connection sets from <B>connect </B>statements.</LI>
  <LI>Generate equations for the complete model.</LI>
</UL>

<P>Definitions:</P>

<ul>
  <li>Connection sets<br>
      A connection set is a set of variables connected by means of
      connect clauses. A connection set shall contain either only flow
      variables or only non-flow variables.</li>
  <li>Inside and outside connectors<br>
      In an element instance M, each connector element of M is called
      an outside connector with respect to M. All other connector elements
      that are hierarchically inside M, but not in one of the outside
      connectors of M, is called an inside connector with respect to M.<br>
      <i>[Example: in connect(a,b.c) 'a' is an outside connector and 'b.c'
      is an inside connector, unless 'b' is a connector.]</i></li>
</ul>
</html>"));
  end Connect;
  class Encapsulated "encapsulated"
    annotation(Documentation(info = "<html>
<p>
Break lookup in hierarchy
</p>
<h4><font color=\"#008000\">Examples</font></h4>

<pre><b>encapsulated model</b> Test
  <b>import</b> Modelica.Mechanics.Rotational;

  Rotational.Components.Inertia inertia; // lookup successful
  Modelica.Mechanics.Translational.SlidingMass slidingMass; // lookup fails
<b>equation</b>
  ...
<b>end</b> Test;</pre>

<h4><font color=\"#008000\">Syntax</font></h4>

<PRE>class_definition :
   [ <B>encapsulated</B> ]
   [ <B>partial</B> ]
   ( <B>class</B> | <B>model</B> | <B>record</B> | <B>block</B> | <B>connector</B> | <B>type</B> |
     <B>package</B> | <b>function</B> )
   IDENT class_specifier</PRE>

<h4><font color=\"#008000\">Description</font></h4>

<p>When an element, equation or algorithm is instantiated, any name is
looked up sequentially in each member of the ordered set of parents
until a match is found or a parent is encapsulated. In the latter case
the lookup stops except for the predefined types, functions and
operators defined in this specification. For these cases the lookup
continues in the global scope, where they are defined. <i>[E.g., abs is
searched upwards in the hierarchy as usual. If an encapsulated boundary
is reached, abs is searched in the global scope instead. The operator
abs cannot be redefined in the global scope, because an existing class
cannot be redefined at the same level.]</i></p>
</html>"));
  end Encapsulated;
  class Extends "extends"
    annotation(Documentation(info = "<html>
<p>
Inheritance from base class
</p>
<h4><font color=\"#008000\">Examples</font></h4>

<PRE><B>class</B> A
  <B>parameter</B> Real a, b;
<B>end</B> A;

<B>class</b> B
  <B>extends</b> A(b=2);
<B>end</B> B;

<B>class</b> C
  <B>extends</b> B(a=1);
<B>end</B> C;</PRE>

<p>From the example above we get the following instantiated
class:</P>

<PRE><B>class</b> Cinstance
  <B>parameter</b> Real a=1;
  <B>parameter</b> Real b=2;
<B>end</b> Cinstance;</PRE>

<p>The ordering of the merging rules ensures that, given
classes A and B defined above,</P>

<PRE><B>class</b> C2
  B bcomp(b=3);
<B>end</B> C2;</PRE>

<p>yields an instance with <TT>bcomp.b=3</TT>, which overrides <TT>b=2</TT>.</P>

<h4><font color=\"#008000\">Syntax</font></h4>

<PRE>class_definition :
   [ <B>encapsulated</B> ]
   [ <B>partial</b> ]
   ( <B>class</b> | <B>model</b> | <B>record</b> | <B>block</b> | <B>connector</b> | <B>type</b> |
     <B>package</B> | <b>function</b> )
   IDENT class_specifier

class_specifier :
   string_comment composition <B>end</B> IDENT
   | \"=\" base_prefix name [ array_subscripts ] [ class_modification ] comment
   | \"=\" <B>enumeration</B> \"(\" ( [enum_list] | \":\" ) \")\" comment

composition  :
   element_list
   { <B>public</b> element_list |
     <B>protected</b> element_list |
     equation_clause |
     algorithm_clause
   }
   [ <B>external</b> [ language_specification ]
              [ external_function_call ] [ annotation \";\" ]
              [ annotation  \";\" ] ]

element_list :
   { element \";\" | annotation  \";\" }

element :
   import_clause |
   extends_clause |
   [ <B>final</B> ]
   [ <B>inner</B> | <B>outer</B> ]
   ( ( class_definition | component_clause) |
     <B>replaceable</B> ( class_definition | component_clause)
        [constraining_clause comment])

extends_clause :
   <B>extends</b> name [ class_modification ]</PRE>

<h4><font color=\"#008000\">Description</font></h4>

<P>The name of the base class is looked up in the partially
instantiated parent of the extends clause. The found base
class is instantiated with a new environment and the
partially instantiated parent of the extends clause. The new
environment is the result of merging</P>

<OL>
  <LI>arguments of all parent environments that match names in the instantiated base class</LI>
  <LI>the optional class modification of the extends clause</LI>
</OL>

<P>in that order.</P>

<P>The elements of the instantiated base class become elements
of the instantiated parent class.</P>

<P>The declaration elements of the instantiated base class shall either</P>

<UL>
  <LI>not already exist in the partially instantiated parent class
      <I>[i.e., have different names]</I>.</LI>
  <LI>be exactly identical to any element of the instantiated parent
      class with the same name and the same level of protection
      (<B>public</B> or <B>protected</B>) and same contents. In this
      case, one of the elements is ignored (since they are identical
      it does not matter which one).</LI>
</UL>

<P>Otherwise the model is incorrect.</P>

<P>Equations of the instantiated base class that are syntactically
equivalent to equations in the instantiated parent class are discarded.
<I>[Note: equations that are mathematically equivalent but not
syntactically equivalent are not discarded, hence yield an
overdetermined system of equations.]</I></P>


<p><span>Since
specialized classes of different
kinds have different properties,
only specialized classes that are \"in some sense
compatible\" to
each other can be derived from each other via inheritance. The
following table
shows which kind of specialized class can be used in an extends clauses
of
another kind of specialized class:</span></p>

<table
 style=\"border: medium none ; margin-left: 23.4pt; border-collapse: collapse;\"
 border=\"1\" cellpadding=\"0\" cellspacing=\"0\">
  <tbody>
    <tr style=\"page-break-inside: avoid; height: 15pt;\">
      <td
 nowrap=\"nowrap\" valign=\"top\" width=\"110\">
      <p style=\"text-indent: 0pt;\"><span
>&nbsp;</span></p>
      </td>
      <td colspan=\"7\"
 nowrap=\"nowrap\" valign=\"top\" width=\"418\">
      <p
><b><span
>Base Class</span></b></p>
      </td>
    </tr>
    <tr style=\"page-break-inside: avoid; height: 15pt;\">
      <td
 nowrap=\"nowrap\" valign=\"top\" width=\"110\">
      <p style=\"text-indent: 0pt;\"><b><span
>Derived Class</span></b></p>
      </td>
      <td
 nowrap=\"nowrap\" valign=\"top\" width=\"66\">
      <p>
<span>package</span></p>
      </td>
      <td
 nowrap=\"nowrap\" valign=\"top\" width=\"64\">
      <p><span
>function</span></p>
      </td>
      <td
 nowrap=\"nowrap\" valign=\"top\" width=\"40\">
      <p><span
>type</span></p>
      </td>
      <td
 nowrap=\"nowrap\" valign=\"top\" width=\"64\">
      <p><span
>record</span></p>
      </td>
      <td
 nowrap=\"nowrap\" valign=\"top\" width=\"72\">
      <p><span
>connector</span></p>
      </td>
      <td
 nowrap=\"nowrap\" valign=\"top\" width=\"52\">
      <p><span
>block</span></p>
      </td>
      <td
 nowrap=\"nowrap\" valign=\"top\" width=\"60\">
      <p><span
>model</span></p>
      </td>
    </tr>
    <tr style=\"page-break-inside: avoid;\">
      <td
 nowrap=\"nowrap\" valign=\"top\" width=\"110\">
      <p style=\"text-indent: 0pt;\"><span
>package</span></p>
      </td>
      <td
 nowrap=\"nowrap\" valign=\"top\" width=\"66\">
      <p><span
>yes</span></p>
      </td>
      <td
 nowrap=\"nowrap\" valign=\"top\" width=\"64\">
      <p><span
>&nbsp;</span></p>
      </td>
      <td
 nowrap=\"nowrap\" valign=\"top\" width=\"40\">
      <p><span
>&nbsp;</span></p>
      </td>
      <td
 nowrap=\"nowrap\" valign=\"top\" width=\"64\">
      <p><span
>&nbsp;</span></p>
      </td>
      <td
 nowrap=\"nowrap\" valign=\"top\" width=\"72\">
      <p><span
>&nbsp;</span></p>
      </td>
      <td
 nowrap=\"nowrap\" valign=\"top\" width=\"52\">
      <p><span
>&nbsp;</span></p>
      </td>
      <td
 nowrap=\"nowrap\" valign=\"top\" width=\"60\">
      <p><span
>&nbsp;</span></p>
      </td>
    </tr>
    <tr style=\"page-break-inside: avoid;\">
      <td
 nowrap=\"nowrap\" valign=\"top\" width=\"110\">
      <p style=\"text-indent: 0pt;\"><span
>function</span></p>
      </td>
      <td
 nowrap=\"nowrap\" valign=\"top\" width=\"66\">
      <p><span
>&nbsp;</span></p>
      </td>
      <td
 nowrap=\"nowrap\" valign=\"top\" width=\"64\">
      <p
><span
>yes</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"40\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"64\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"72\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"52\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"60\">
      <p
><span
>&nbsp;</span></p>
      </td>
    </tr>
    <tr style=\"page-break-inside: avoid;\">
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"110\">
      <p style=\"text-indent: 0pt;\"><span
>type</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"66\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"64\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"40\">
      <p
><span
>yes</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"64\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"72\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"52\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"60\">
      <p
><span
>&nbsp;</span></p>
      </td>
    </tr>
    <tr style=\"page-break-inside: avoid;\">
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"110\">
      <p style=\"text-indent: 0pt;\"><span
>record</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"66\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"64\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"40\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"64\">
      <p
><span
>yes</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"72\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"52\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"60\">
      <p
><span
>&nbsp;</span></p>
      </td>
    </tr>
    <tr style=\"page-break-inside: avoid;\">
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"110\">
      <p style=\"text-indent: 0pt;\"><span
>connector</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"66\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"64\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td
 nowrap=\"nowrap\" valign=\"top\" width=\"40\">
      <p
><span
>yes</span></p>
      </td>
      <td
 nowrap=\"nowrap\" valign=\"top\" width=\"64\">
      <p
><span
>yes</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"72\">
      <p
><span
>&nbsp;yes</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"52\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"60\">
      <p
><span
>&nbsp;</span></p>
      </td>
    </tr>
    <tr style=\"page-break-inside: avoid;\">
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"110\">
      <p style=\"text-indent: 0pt;\"><span
>block</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"66\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"64\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"40\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"64\">
      <p
><span
>yes</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"72\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"52\">
      <p
><span
>yes</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"60\">
      <p
><span
>&nbsp;</span></p>
      </td>
    </tr>
    <tr style=\"page-break-inside: avoid;\">
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"110\">
      <p style=\"text-indent: 0pt;\"><span
>model</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"66\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"64\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"40\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"64\">
      <p
><span
>yes</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"72\">
      <p
><span
>&nbsp;</span></p>
      </td>
      <td
 nowrap=\"nowrap\" valign=\"top\" width=\"52\">
      <p
><span
>yes</span></p>
      </td>
      <td

 nowrap=\"nowrap\" valign=\"top\" width=\"60\">
      <p
><span
>yes</span></p>
      </td>
    </tr>
  </tbody>
</table>
<p><span>The
specialized classes </span><span class=\"CODE\"><span
 style=\"font-size: 9.5pt;\">package</span></span><span
>, </span><span class=\"CODE\"><span
 style=\"font-size: 9.5pt;\">function</span></span><span
>, </span><span class=\"CODE\"><span
 style=\"font-size: 9.5pt;\">type</span></span><span
> and record can only be derived from their own
kind [<i>(e.g., a
package can only be base class for packages. All other kinds of classes
can use
the import statement to use the contents of a package)</i>]<i>.</i></span></p>


</html>"));
  end Extends;
  class Flow "flow"
    annotation(Documentation(info = "<html>
<p>
Declare flow (through) variable, which have to sum up to zero in connections
</p>
<h4><font color=\"#008000\">Examples</font></h4>

<pre><b>connector</b> Pin
  Modelica.SIunits.Voltage v;
  <b>flow</b> Modelica.SIunits.Current i;
<b>end</b> Pin;

<b>model</b> A
  Pin p;
<b>end</b> A;

<b>model</b> Composition
  A a;
  A b;
  A c;
<b>equation
  connect</b>(a.p, b.p);
  <b>connect</b>(a.p, c.p);
<b>end</b> Composition;</pre>

<p>From the connect statements in model Composition, the following connect equations are derived:</p>

<pre>a.p.v = b.p.v;
a.p.v = c.p.v;
a.p.i + b.p.i + c.p.i = 0;</pre>

<h4><font color=\"#008000\">Syntax</font></h4>

<PRE>class_definition :
   [ <B>encapsulated</B> ]
   [ <B>partial</b> ]
   ( <B>class</b> | <B>model</b> | <B>record</b> | <B>block</b> | <B>connector</b> | <B>type</b> |
     <B>package</B> | <b>function</b> )
   IDENT class_specifier

class_specifier :
   string_comment composition <B>end</B> IDENT
   | \"=\" base_prefix name [ array_subscripts ] [ class_modification ] comment
   | \"=\" <B>enumeration</B> \"(\" ( [enum_list] | \":\" ) \")\" comment

base_prefix :
   type_prefix

composition  :
   element_list
   { <B>public</b> element_list |
     <B>protected</b> element_list |
     equation_clause |
     algorithm_clause
   }
   [ <B>external</b> [ language_specification ]
              [ external_function_call ] [ annotation \";\" ]
              [ annotation  \";\" ] ]

element_list :
   { element \";\" | annotation  \";\" }

element :
   import_clause |
   extends_clause |
   [ <B>final</B> ]
   [ <B>inner</B> | <B>outer</B> ]
   ( ( class_definition | component_clause) |
     <B>replaceable</B> ( class_definition | component_clause)
        [constraining_clause comment])

component_clause:
   type_prefix type_specifier [ array_subscripts ] component_list

type_prefix :
   [ <B>flow</b> ]
   [ <B>discrete</B> | <B>parameter</b> | <B>constant</b> ] [ <B>input</b> | <B>output</b> ]</PRE>

<h4><font color=\"#008000\">Description</font></h4>

<p>The flow prefix is used in order to generate equations for through variables, which sum up to zero in connections,
whereas variables without the flow prefix are identical in a connection.</p>
</html>"));
  end Flow;
  class For "for"
    annotation(Documentation(info = "<html>
<p>
Repeat equations or statements a specific number of times
</p>
<h4><font color=\"#008000\">Examples</font></h4>
<p>
<b>for</b> clauses are mostly used in algorithm sections, such as
</p>
<pre>
    <b>parameter</b> Integer np=10;
    Real p[np], x, y;
  <b>algorithm</b>
     y := p[1];
     <b>for</b> i <b>in</b> 2:np <b>loop</b>   // i shall not be declared
        y := y*x + p[i];
     <b>end for</b>;
</pre>
<p>
Other forms of the <b>for</b> condition:
</p>
<pre>
    <b>for</b> i <b>in</b> 1:10 <b>loop</b>            // i takes the values 1,2,3,...,10
    <b>for</b> r <b>in</b> 1.0 : 1.5 : 5.5 <b>loop</b> // r takes the values 1.0, 2.5, 4.0, 5.5
    <b>for</b> i <b>in</b> {1,3,6,7} <b>loop</b>       // i takes the values 1, 3, 6, 7
</pre>
<p>
In equation sections, <b>for</b> clauses are expanded at translation time
in order that symbolic transformations can be applied. Typically, a
<b>for</b> clause in an equation section is used on component arrays,
e.g., to connect elements of an array of components together:
</p>
<pre>
    <b>parameter</b> Integer nR=10 \"Number of resistances\";
    Modelica.Electrical.Analog.Basic.Resistor R[nR];
  <b>equation</b>
     <b>for</b> i <b>in</b> 1:nR-1 <b>loop</b>
        <b>connect</b>(R[i].p R[i+1].n);   // 9 connect equations
     <b>end for</b>;
</pre>
<h4><font color=\"#008000\">Syntax</font></h4>
<p>
In equation sections:
</p>
<pre>
  <b>for</b> for_indices <b>loop</b>
     { equation \";\" }
  <b>end for</b>;
  for_indices : for_index {\",\" for_index}
  for_index   : IDENT [ in expression ]
</pre>
<p>
In algorithm sections:
</p>
<pre>
  <b>for</b> for_indices <b>loop</b>
     { algorithm \";\" }
  <b>end for</b>;
  for_indices : for_index {\",\" for_index}
  for_index   : IDENT [ in expression ]
</pre>
<h4><font color=\"#008000\">Description</font></h4>
<p>A clause</p>
<pre>    <b>for</b> IDENT <b>in</b> expression <b>loop</b></pre>
<p>is one example of a for clause.</p>
<p>The expression of a for clause shall be a vector expression. It is
evaluated once for each for clause, and is evaluated in the scope
immediately enclosing the for clause. In an equation section,
the expression of a for clause shall be a parameter expression (in order
that the for clause can be expanded into equations during translation).
The loop-variable is in scope inside the loop-construct and shall not be assigned
to.</p>
<p><i>[Example:</i></p>
<p><i>The loop-variable may hide other variables as in the following example.
Using another name for the loop-variable
is, however, strongly recommended.</i></p>
<pre>    <b>constant</b> Integer j=4;
    Real x[j];
  <b>equation</b>
    <b>for</b> j <b>in</b> 1:j <b>loop</b> // The loop-variable j takes the values 1,2,3,4
      x[j]=j; // Uses the loop-variable j
    <b>end for</b>;</pre>
<p><i>]</i></p>
<h4>Several iterators</h4>
<p>The notation with several iterators is a shorthand notation for nested for-clauses (or reduction-expressions). For
for-clauses it can be expanded into the usual form by replacing each \",\" by <tt>'loop for'</tt> and adding extra <tt>'end
for'</tt>. For reduction-expressions it can be expanded into the usual form by replacing each ',' by <tt>') for'</tt> and
prepending the reduction-expression with <tt>'function-name('</tt>.</p>
<p><i>[Example:</i></p>
<pre>    Real x[4,3];
  <b>equation</b>
    <b>for</b> j, i in 1:2 <b>loop</b>
      // The loop-variable j takes the values 1,2,3,4 (due to use)
      // The loop-variable i takes the values 1,2 (given range)
      x[j,i]=j+i;
    <b>end for</b>;</pre>
<p><i>]</i></p>
</html>"));
  end For;
  class If "if"
    annotation(Documentation(info = "<html>
<p>
Declare equations or execute statements conditionally
</p>
<h4><font color=\"#008000\">Examples</font></h4>
<h4>If clause</h4>

<pre>    <b>parameter</b> Boolean linear=true;
    <b>parameter</b> Boolean quadratic=false;
    Real x, y;
  <b>equation</b>
    <b>if</b> linear <b>then</b>
      y = x + 1;
    <b>elseif</b> quadratic <b>then</b>
      y = x^2 + x + 1;
    <b>else</b>
      y = Modelica.Math.sin(x) + 1;
    <b>end if</b>;</pre>

<h4>If expression</h4>
<PRE>  Integer i;
  Integer sign_of_i=<B>if</B> i&lt;0 <B>then</B> -1 <B>else if</B> i==0 <B>then</B> 0 <B>else</B> 1;</PRE>
<h4><font color=\"#008000\">Syntax</font></h4>
<p>
In equation sections:
</p>
<PRE>   <B>if</B> expression <B>then</B>
     { equation \";\" }
   { <B>elseif </B>expression <B>then</B>
     { equation \";\" }
   }
   [ <B>else</B>
     { equation \";\" }
   ]
   <B>end if</B></pre>
<p>
In algorithm sections:
</p>
<pre>   <B>if</B> expression <B>then</B>
     { algorithm \";\" }
   { <B>elseif </B>expression <B>then</B>
     { algorithm \";\" }
   }
   [ <B>else</B>
     { algorithm \";\" }
   ]
   <B>end if</B></PRE>
<h4><font color=\"#008000\">Description</font></h4>
<h4>If clause</h4>
<P>
The expression of an <B>if</B> and <B>elseif</B>-clause must be scalar boolean expression.
One <B>if</B>-clause, and zero or more <B>elseif</B>-clauses, and an optional <B>else</B>-clause
together form a list of branches. One or zero of the bodies of these <B>if</B>-, <B>elseif</B>-
and <B>else</B>-clauses is selected, by evaluating the conditions of the <B>if</B>- and
<B>elseif</B>-clauses sequentially until a condition that evaluates to true is found. If none of
the conditions evaluate to true the body of the <B>else</B>-clause is selected (if an
<B>else</B>-clause exists, otherwise no body is selected). In an algorithm section, the selected
body is then executed. In an equation section, the equations in the body are seen as equations
that must be satisfied. The bodies that are not selected have no effect on that model evaluation.
</P>
<P>
If clauses in <B>equation </B>sections which do not have exclusively parameter expressions as
switching conditions shall have an <B>else</B> clause and each branch shall have the <I>same
number</I> of <I>equations</I>. <I>[If this condition is violated, the single assignment rule
would not hold, because the number of equations may change during simulation although the number
of unknowns remains the same].</I>
</P>

<h4>If expression</h4>

<P>An expression</P>
<pre>   <B>if</B> expression1 <B>then</B> expression2 <B>else</B> expression3</pre>
<P>is one example of if-expression. First expression1, which must be boolean expression,
is evaluated. If expression1 is true expression2 is evaluated and is the value of the
if-expression, else expression3 is evaluated and is the value of the if-expression.
The two expressions, expression2 and expression3, must be type compatible and give the
type of the if-expression. If-expressions with <B>elseif</B> are defined by replacing
<B>elseif</B> by <B>else if</B>.<I>[Note: elseif is added for symmetry with if-clauses.]</I>
</P>
</html>"));
  end If;
  class Import "import"
    annotation(Documentation(info = "<html>
<p>
Import classes
</p>
<h4><font color=\"#008000\">Examples</font></h4>

<pre><b>class</b> Lookup
  <b>import</b> SI = Modelica.SIunits; // #1
  <b>import</b> Modelica.Math.*; // #2
  <b>import</b> Modelica.Mechanics.Rotational; // #3

  SI.Torque torque; // due to #1 (Modelica.SIunits.Torque)
  Rotational.Components.Inertia inertia; // due to #3 (Modelica.Mechanics.Rotational.Components.Inertia)
<b>equation</b>
  torque = sin(<b>time</b>); // due to #2 (Modelica.Math.sin)
  ...
<b>end</b> Lookup;</pre>

<h4><font color=\"#008000\">Syntax</font></h4>

<PRE>class_definition :
   [ <B>encapsulated</B> ]
   [ <B>partial</b> ]
   ( <B>class</b> | <B>model</b> | <B>record</b> | <B>block</b> | <B>connector</b> | <B>type</b> |
     <B>package</B> | <b>function</b> )
   IDENT class_specifier

class_specifier :
   string_comment composition <B>end</B> IDENT
   | \"=\" base_prefix name [ array_subscripts ] [ class_modification ] comment
   | \"=\" <B>enumeration</B> \"(\" ( [enum_list] | \":\" ) \")\" comment

composition  :
   element_list
   { <B>public</b> element_list |
     <B>protected</b> element_list |
     equation_clause |
     algorithm_clause
   }
   [ <B>external</b> [ language_specification ]
              [ external_function_call ] [ annotation \";\" ]
              [ annotation  \";\" ] ]

element_list :
   { element \";\" | annotation  \";\" }

element :
   import_clause |
   extends_clause |
   [ <B>final</B> ]
   [ <B>inner</B> | <B>outer</B> ]
   ( ( class_definition | component_clause) |
     <B>replaceable</B> ( class_definition | component_clause)
        [constraining_clause comment])

import_clause :
   <B>import</B> ( IDENT \"=\" name | name [\".\" \"*\"] ) comment</PRE>
<h4><font color=\"#008000\">Description</font></h4>

<p>Using import statements extends the static name lookup to additional import names.
The generated import names are:</p>

<ul>
  <li><tt>C</tt> for <tt><b>import</b> A.B.C;</tt></li>
  <li><tt>D</tt> for <tt><b>import</b> D = A.B.C;</tt></li>
  <li><tt>C</tt> and all other classes in B for <tt><b>import</b> A.B.*;</tt></li>
</ul>
</html>"));
  end Import;
  class Input "input"
    annotation(Documentation(info = "<html>
<p>
Define causality and/or block diagram connection semantic
(depending on context)
</p>
<h4><font color=\"#008000\">Examples</font></h4>

<pre><b>connector</b> RealInput = <b>input</b> Real;
<b>connector</b> RealOutput = <b>output</b> Real;

<b>block</b> Integrator
  RealInput  u;
  RealOutput y;
<b>protected</b>
  Real x;
<b>equation</b>
  <b>der</b>(x) = u;
  y = x;
<b>end</b> Integrator;</pre>

<h4><font color=\"#008000\">Syntax</font></h4>

<PRE>class_definition :
   [ <B>encapsulated</B> ]
   [ <B>partial</b> ]
   ( <B>class</b> | <B>model</b> | <B>record</b> | <B>block</b> | <B>connector</b> | <B>type</b> |
     <B>package</B> | <b>function</b> )
   IDENT class_specifier

class_specifier :
   string_comment composition <B>end</B> IDENT
   | \"=\" base_prefix name [ array_subscripts ] [ class_modification ] comment
   | \"=\" <B>enumeration</B> \"(\" ( [enum_list] | \":\" ) \")\" comment

base_prefix :
   type_prefix

composition  :
   element_list
   { <B>public</b> element_list |
     <B>protected</b> element_list |
     equation_clause |
     algorithm_clause
   }
   [ <B>external</b> [ language_specification ]
              [ external_function_call ] [ annotation \";\" ]
              [ annotation  \";\" ] ]

element_list :
   { element \";\" | annotation  \";\" }

element :
   import_clause |
   extends_clause |
   [ <B>final</B> ]
   [ <B>inner</B> | <B>outer</B> ]
   ( ( class_definition | component_clause) |
     <B>replaceable</B> ( class_definition | component_clause)
        [constraining_clause comment])

component_clause:
   type_prefix type_specifier [ array_subscripts ] component_list

type_prefix :
   [ <B>flow</b> ]
   [ <B>discrete</B> | <B>parameter</b> | <B>constant</b> ] [ <B>input</b> | <B>output</b> ]</PRE>

<h4><font color=\"#008000\">Description</font></h4>

<p>
The prefixes <b>input</b> and <b>output</b> have a slightly different semantic meaning depending on the context where they are used:
</p>

<ul>
<li> In functions, these prefixes define the computational causality of the
     function body, i.e., given the variables declared as input,
     the variables declared as output are computed in the function body<br>&nbsp;</li>.

<li> In simulation models and blocks (i.e., on the top level of a model or
     block that shall be simulated), these prefixes define the interaction
     with the environment where the simulation model or block is used.
     Especially, the input prefix defines that values for such a variable
     have to be provided from the simulation environment and the output
     prefix defines that the values of the corresponding variable
     can be directly utilized in the simulation environment.<br>&nbsp;</li>

<li> In component models and blocks, the input prefix defines that a
     binding equation has to be provided for the corresponding variable
     when the component is utilized in order to guarantee a locally
     balanced model (i.e., the number of local equations is identical
     to the local number of unknowns). Example:
<pre>  <b>block</b> FirstOrder
     <b>input</b> Real u;
       ...
  <b>end</b> FirstOrder;

  <b>model</b> UseFirstOrder
     FirstOrder firstOrder(u=time); // binding equation for u
      ...
  <b>end</b> UseFirstOrder;
</pre>
     The output prefix does not have a particular effect in a model
     or block component and is ignored.<br>&nbsp;</li>

<li> In connectors, prefixes input and output define that the
     corresponding connectors can only be connected according
     to block diagram semantics (e.g., a connector with an output
     variable can only be connected to a connector where the
     corresponding variable is declared as input). There is the
     restriction that connectors which have at least one variable
     declared as input must be externally connected
     (in order to get a locally balanced model, where the number
     of local unknowns is identical to the number of unknown equations).
     Together with the block diagram semantics rule this means,
     that such connectors must be connected exactly once externally.<br>&nbsp;</li>

<li> In records, prefixes input and output are not allowed,
     since otherwise a record could not be, e.g., passed as input
     argument to a function.</li>
</ul>

</html>"));
  end Input;
  class Output "output"
    annotation(Documentation(info = "<html>
<p>
Define causality and/or block diagram connection semantic
(depending on context)
</p>

<h4><font color=\"#008000\">Examples</font></h4>

<pre><b>connector</b> RealInput = <b>input</b> Real;
<b>connector</b> RealOutput = <b>output</b> Real;

<b>block</b> Integrator
  RealInput  u;
  RealOutput y;
<b>protected</b>
  Real x;
<b>equation</b>
  <b>der</b>(x) = u;
  y = x;
<b>end</b> Integrator;</pre>

<h4><font color=\"#008000\">Syntax</font></h4>

<PRE>class_definition :
   [ <B>encapsulated</B> ]
   [ <B>partial</b> ]
   ( <B>class</b> | <B>model</b> | <B>record</b> | <B>block</b> | <B>connector</b> | <B>type</b> |
     <B>package</B> | <b>function</b> )
   IDENT class_specifier

class_specifier :
   string_comment composition <B>end</B> IDENT
   | \"=\" base_prefix name [ array_subscripts ] [ class_modification ] comment
   | \"=\" <B>enumeration</B> \"(\" ( [enum_list] | \":\" ) \")\" comment

base_prefix :
   type_prefix

composition  :
   element_list
   { <B>public</b> element_list |
     <B>protected</b> element_list |
     equation_clause |
     algorithm_clause
   }
   [ <B>external</b> [ language_specification ]
              [ external_function_call ] [ annotation \";\" ]
              [ annotation  \";\" ] ]

element_list :
   { element \";\" | annotation  \";\" }

element :
   import_clause |
   extends_clause |
   [ <B>final</B> ]
   [ <B>inner</B> | <B>outer</B> ]
   ( ( class_definition | component_clause) |
     <B>replaceable</B> ( class_definition | component_clause)
        [constraining_clause comment])

component_clause:
   type_prefix type_specifier [ array_subscripts ] component_list

type_prefix :
   [ <B>flow</b> ]
   [ <B>discrete</B> | <B>parameter</b> | <B>constant</b> ] [ <B>input</b> | <B>output</b> ]</PRE>

<h4><font color=\"#008000\">Description</font></h4>

<p>
The prefixes <b>input</b> and <b>output</b> have a slightly different semantic meaning depending on the context where they are used:
</p>

<ul>
<li> In functions, these prefixes define the computational causality of the
     function body, i.e., given the variables declared as input,
     the variables declared as output are computed in the function body<br>&nbsp;</li>.

<li> In simulation models and blocks (i.e., on the top level of a model or
     block that shall be simulated), these prefixes define the interaction
     with the environment where the simulation model or block is used.
     Especially, the input prefix defines that values for such a variable
     have to be provided from the simulation environment and the output
     prefix defines that the values of the corresponding variable
     can be directly utilized in the simulation environment.<br>&nbsp;</li>

<li> In component models and blocks, the input prefix defines that a
     binding equation has to be provided for the corresponding variable
     when the component is utilized in order to guarantee a locally
     balanced model (i.e., the number of local equations is identical
     to the local number of unknowns). Example:
<pre>  <b>block</b> FirstOrder
     <b>input</b> Real u;
       ...
  <b>end</b> FirstOrder;

  <b>model</b> UseFirstOrder
     FirstOrder firstOrder(u=time); // binding equation for u
      ...
  <b>end</b> UseFirstOrder;
</pre>
     The output prefix does not have a particular effect in a model
     or block component and is ignored.<br>&nbsp;</li>

<li> In connectors, prefixes input and output define that the
     corresponding connectors can only be connected according
     to block diagram semantics (e.g., a connector with an output
     variable can only be connected to a connector where the
     corresponding variable is declared as input). There is the
     restriction that connectors which have at least one variable
     declared as input must be externally connected
     (in order to get a locally balanced model, where the number
     of local unknowns is identical to the number of unknown equations).
     Together with the block diagram semantics rule this means,
     that such connectors must be connected exactly once externally.<br>&nbsp;</li>

<li> In records, prefixes input and output are not allowed,
     since otherwise a record could not be, e.g., passed as input
     argument to a function.</li>
</ul>

</html>"));
  end Output;
  class Partial "partial"
    annotation(Documentation(info = "<html>
<p>
Prohibit instantiation of components of the class
</p>
<h4><font color=\"#008000\">Examples</font></h4>

<pre><b>partial block</b> BaseBlock
  <b>input</b> Real u;
  <b>output</b> Real y;
<b>protected</b>
  Real x;
<b>equation</b>
  x = y;
<b>end</b> BaseBlock;

<b>block</b> Integrator
  <b>extends</b> BaseBlock;
<b>equation
  der</b>(x) = u;
<b>end</b> Integrator;

<b>block</b> Gain
  <b>extends</b> BaseBlock;
  <b>parameter</b> k = 1;
<b>equation</b>
  x = k*u;
<b>end</b> Gain;

<b>model</b> Composition
  BaseBlock block1; // Illegal
  Integrator block2; // Legal
  Gain block3; // Legal
<b>end</b> Composition;</pre>

<h4><font color=\"#008000\">Syntax</font></h4>

<PRE>class_definition :
   [ <B>encapsulated</B> ]
   [ <B>partial</b> ]
   ( <B>class</b> | <B>model</b> | <B>record</b> | <B>block</b> | <B>connector</b> | <B>type</b> |
     <B>package</B> | <b>function</b> )
   IDENT class_specifier

class_specifier :
   string_comment composition <B>end</B> IDENT
   | \"=\" base_prefix name [ array_subscripts ] [ class_modification ] comment
   | \"=\" <B>enumeration</B> \"(\" ( [enum_list] | \":\" ) \")\" comment</PRE>

</html>"));
  end Partial;
  class Time "time"
    annotation(Documentation(info = "<html>
<p>
Built-in variable time
</p>
<h4><font color=\"#008000\">Examples</font></h4>

<PRE><B>encapsulated model</B> SineSource
  <B>import</B> Modelica.Math.sin;
  <B>connector</B> OutPort=<B>output</B> Real;
  OutPort y=sin(time); // Uses the built-in variable time.
<B>end</B> SineSource;</PRE>

<h4><font color=\"#008000\">Syntax</font></h4>

<pre><b>time</b></pre>

<h4><font color=\"#008000\">Description</font></h4>

<P>All declared variables are functions of the independent
variable <B>time</B>. Time is a built-in variable available
in all classes, which is treated as an input variable. It
is implicitly defined as:</P>

<PRE><B>input</B> Real time (<B>final</B> quantity = \"Time\",
                 <B>final</B> unit     = \"s\");</PRE>

<P>The value of the <B>start</B> attribute of time is set to
the time instant at which the simulation is started.</P>

</html>"));
  end Time;
  class When "when"
    annotation(Documentation(info = "<html>
<p>
Activate equations or statements when condition becomes true
</p>
<h4><font color=\"#008000\">Examples</font></h4>
<PRE> <B>equation
   when</B> x &gt; 2 <B>then</B>
     y3 = 2*x +y1+y2; // Order of y1 and y3 equations does not matter
     y1 = sin(x);
   <B>end when</B>;
   y2 = sin(y1);</PRE>

<h4><font color=\"#008000\">Syntax</font></h4>

<p>
In equation sections:
</p>

<PRE>  <B>when</B> expression <B>then</B>
    { equation \";\" }
  { <B>elsewhen</B> expression <B>then</B>
    { equation \";\" } }
  <B>end when</B></PRE>

<p>
In algorithm sections:
</p>
<PRE>  <B>when</B> expression <B>then</B>
    { algorithm \";\" }
  { <B>elsewhen</B> expression <B>then</B>
    { algorithm \";\" } }
  <B>end when</B></PRE>

<h4><font color=\"#008000\">Description</font></h4>

<P>The expression of a when clause shall be a discrete-time Boolean scalar
or vector expression. The equations and algorithm statements within a when
clause are activated when the scalar or any one of the elements of the vector
expression becomes true. When-clauses in equation sections are allowed, provided
the equations within the when-clause have one of the following forms:</P>

<UL>
  <LI>v = expr;</LI>
  <LI> (out1, out2, out3, ...) = function_call(in1, in2, ...);</LI>
  <LI>operators <B>assert</B>(), <B>terminate</B>(), <B>reinit</B>()</LI>
  <LI><B>For</B> and <B>if</B>-clause if the equations within the <B>for</B> and <B>if</B>-clauses satisfy these requirements.</LI>
  <LI>In an equation section, the different branches of when/elsewhen must have the same set of component references on the left-hand side.</LI>
  <LI>In an equation section, the branches of an if-then-else clause inside when-clauses must have the same set of component references on the left-hand side, unless the if-then-else have exclusively parameter expressions as switching conditions.</LI>
</UL>

<P>A when clause shall not be used within a function class.</P>

<P><I>[Example:</i></P>

<P><i>Algorithms are activated when x becomes &gt; 2:</I></P>

<PRE>   <B>when</B> x &gt; 2 <B>then</B>
     y1 := sin(x);
     y3 := 2*x + y1 + y2;
   <B>end when</B>;</PRE>

<P><I>Algorithms are activated when either x becomes &gt; 2 or sample(0,2) becomes true or x becomes less than 5:</I></P>

<PRE>   <B>when</B> {x &gt; 2, sample(0,2), x &lt; 5} <B>then</B>
     y1 := sin(x);
     y3 := 2*x + y1 + y2;
   <B>end when</B>;</PRE>

<P><I>For when in equation sections the order between the equations does not matter, e.g.,</I></P>
<PRE> <B>equation
   when</B> x &gt; 2 <B>then</B>
     y3 = 2*x +y1+y2; // Order of y1 and y3 equations does not matter
     y1 = sin(x);
   <B>end when</B>;
   y2 = sin(y1);</PRE>

<P><I>The needed restrictions on equations within a when-clause becomes apparent with the following example:</I></P>

<PRE>   Real x, y;
<B>equation</B>
   x + y = 5;
   <B>when</B> condition <B>then</B>
      2*x + y = 7;         // error: not valid Modelica
   <B>end when</B>;</PRE>


<P><I>When the equations of the when-clause are not activated it is not clear which
variable to hold constant, either x or y. A corrected version of this example is:</I></P>

<PRE>   Real x, y;
<B>equation</B>
   x + y = 5;
   <B>when</B> condition <B>then</B>
      y = 7 - 2*x;        // fine
   <B>end when</B>;</PRE>

<P><I>Here, variable y is held constant when the when-clause is de-activated and x
is computed from the first equation using the value of y from the previous event instant.</i></P>

<P><i>For when in algorithm sections the order is significant and it is advisable to have only
one assignment within the when-clause and instead use several algorithms having when-clauses
with identical conditions, e.g.,</I></P>

<PRE> <B>algorithm</B>
   <B>when</B> x &gt; 2 <B>then</B>
     y1 := sin(x);
   <B>end when</B>;
 <B>equation</B>
   y2 = sin(y1);
 <B>algorithm</B>
   <B>when</B> x &gt; 2 <B>then</B>
     y3 := 2*x + y1 + y2;
   <B>end when</B>;</PRE>

<P><I>Merging the when-clauses can lead to less efficient code and different models
with different behaviour depending on the order of the assignment to y1 and y3 in the algorithm.]</I></P>

<P>A when clause</P>

<PRE> <B>algorithm
   when</B> {x&gt;1, <B>..., </B>y&gt;p} <B>then
     ...
   elsewhen</B> x &gt; y.start <B>then
     ...
   end when</B>;</PRE>

<P>is equivalent to the following special if-clause, where Boolean b1[N]
and Boolean b2 are necessary because the <B>edge</B>() operator can only
be applied to variables</P>

<PRE>   Boolean b1[N](start={x.start&gt;1,<B> ..., </B>y.start&gt;p});
   Boolean b2(start=x.start&gt;y.start);
 <B>algorithm</B>
   b1:={x&gt;1, ..., y&gt;p};
   b2:=x&gt;y.start;

   <B>if edge</B>(b1[1]) <B>or edge</B>(b1[2]) <B>or ... edge</B>(b1[N]) <B>then
     ...
   elseif edge</B>(b2) <B>then
     ...
   end if</B>;</PRE>

<P>with &quot;<B>edge</B>(A)= A <B>and not pre</B>(A)&quot; and the additional
guarantee, that the algorithms within this special if clause are only evaluated
at event instants.</P>

<P>A when-clause</P>

<PRE> <B>equation
   when</B> x&gt;2 <B>then</B>
     v1 = expr1 ;
     v2 = expr2 ;
   <B>end when</B>;</PRE>

<P>is equivalent to the following special if-expressions</P>

<PRE>   Boolean b(start=x.start&gt;2);
 <B>equation</B>
   b  = x&gt;2;
   v1 = <B>if edge</B>(b) <B>then</B> expr1 <B>else pre</B>(v1);
   v2 = <B>if edge</B>(b) <B>then</B> expr2 <B>else pre</B>(v2);</PRE>

<P>The start-values of the introduced boolean variables are defined by
the taking the start-value of the when-condition, as above where p is a
parameter variable. The start-values of the special functions
<B>initial</B>, <B>terminal</B>, and <B>sample</B> is false.</P>

<P>When clauses cannot be nested.</P>

<P><I>[Example:</i></P>

<P><i>The following when clause is invalid:</I></P>

<PRE>   <B>when</B> x &gt; 2 <B>then
     when</B> y1 &gt; 3 <B>then</B>
       y2 = sin(x);
     <B>end when</B>;
   <B>end when</B>;</PRE>

<P><I>]</I></P>
</html>"));
  end When;
  class While "while"
    annotation(Documentation(info = "<html>
<p>
Repeat statements as long as a condition is fulfilled
</p>
<h4><font color=\"#008000\">Examples</font></h4>
<pre>    Integer i;
  <b>algorithm</b>
    i := 1;
    <b>while</b> i &lt; 10 <b>loop</b>
      i := i + 1;
      <b>...
    end while</b>;</pre>

<h4><font color=\"#008000\">Syntax</font></h4>

<PRE>  <B>while</B> expression <B>loop</B>
    { algorithm \";\" }
  <B>end while</B></PRE>

<h4><font color=\"#008000\">Description</font></h4>

<P>The expression of a while clause shall be a scalar boolean expression.
The while-clause corresponds to while-statements in programming languages,
and is formally defined as follows</P>

<OL>
  <LI>The expression of the while clause is evaluated.</LI>
  <LI>If the expression of the while-clause is false, the execution
      continues after the while-clause.</LI>
  <LI>If the expression of the while-clause is true, the entire body of
      the while clause is executed (except if a break statement or return
      statement is executed), and then execution proceeds at step 1.</LI>
</OL>

</html>"));
  end While;
end ModelicaReference;
package ModelicaServices "Models and functions used in the Modelica Standard Library requiring a tool specific implementation"
  package UsersGuide "Users Guide"
    annotation(__Dymola_DocumentationClass = true);
    class ReleaseNotes "Release notes"
      annotation(Documentation(info = "<html>
<h4>Version 1.0, 2009-06-21</h4>

<p>
First version of the ModelicaServices library.
</p>

</html>
"));
    end ReleaseNotes;
    class ModelicaLicense2 "Modelica License 2"
      annotation(Documentation(info = "<html>

<p>All files in this directory (ModelicaServices) and in all
subdirectories are licensed by the <b><u>Modelica Association</u></b> under the
<b><u>Modelica License 2</u></b>.</p>

<p style=\"margin-left: 40px;\"><b>Licensor:</b><br>
Modelica Association<br>
(Ideella F&ouml;reningar 822003-8858 in Link&ouml;ping) <br>
c/o PELAB, IDA, Link&ouml;pings Universitet <br>
S-58183 Link&ouml;ping <br>
Sweden<br>
email: Board@Modelica.org<br>
web&nbsp; : http://www.Modelica.org</p>
<p style=\"margin-left: 40px;\"><b>Copyright notices of the files:</b></p>
<div style=\"margin-left: 40px;\">Copyright &copy; 1998-2009,
DLR, Dynasim.<br>
</div>
<br>
</p>

<p> <a href=\"#1.%20The%20Modelica%20License%202%7Coutline\">The
Modelica License 2</a><br>
<a href=\"#2.%20Frequently%20Asked%20Questions%7Coutline\">Frequently
Asked Questions</a><br>
<br>
</p>

<hr>
<h4><a name=\"1. The Modelica License 2|outline\"></a>The Modelica License 2</h4>

<p>
<b>Preamble.</b> The goal of this license is that Modelica related
model libraries, software, images, documents, data files etc. can be
used freely in the original or a modified form, in open source and in
commercial environments (as long as the license conditions below are
fulfilled, in particular sections 2c) and 2d). The Original Work is
provided free of charge and the use is completely at your own risk.
Developers of free Modelica packages are encouraged to utilize this
license for their work.</p>

<p>
The Modelica License applies to any Original Work that contains the
following licensing notice adjacent to the copyright notice(s) for
this Original Work:</p>
<p><b>Licensed
by the Modelica Association under the Modelica License 2</b></p>

<p><b>1. Definitions.</b></p>
<ol>
 <li>&ldquo;License&rdquo; is this Modelica License.</li>

 <li>
 &ldquo;Original Work&rdquo; is any work of authorship, including
 software, images, documents, data files, that contains the above
 licensing notice or that is packed together with a licensing notice
 referencing it.</li>

 <li>
 &ldquo;Licensor&rdquo; is the provider of the Original Work who has
 placed this licensing notice adjacent to the copyright notice(s) for
 the Original Work. The Original Work is either directly provided by
 the owner of the Original Work, or by a licensee of the owner.</li>

 <li>
 &ldquo;Derivative Work&rdquo; is any modification of the Original
 Work which represents, as a whole, an original work of authorship.
 For the matter of clarity and as examples: </li>

 <ol>
  <li>
  Derivative Work shall not include work that remains separable from
  the Original Work, as well as merely extracting a part of the
  Original Work without modifying it.</li>

  <li>
  Derivative Work shall not include (a) fixing of errors and/or (b)
  adding vendor specific Modelica annotations and/or (c) using a
  subset of the classes of a Modelica package, and/or (d) using a
  different representation, e.g., a binary representation.</li>

  <li>
  Derivative Work shall include classes that are copied from the
  Original Work where declarations, equations or the documentation
  are modified.</li>

  <li>
  Derivative Work shall include executables to simulate the models
  that are generated by a Modelica translator based on the Original
  Work (of a Modelica package).</li>
 </ol>

 <li>
 &ldquo;Modified Work&rdquo; is any modification of the Original Work
 with the following exceptions: (a) fixing of errors and/or (b)
 adding vendor specific Modelica annotations and/or (c) using a
 subset of the classes of a Modelica package, and/or (d) using a
 different representation, e.g., a binary representation.</li>

 <li>
 &quot;Source Code&quot; means the preferred form of the Original
 Work for making modifications to it and all available documentation
 describing how to modify the Original Work.</li>

 <li>
 &ldquo;You&rdquo; means an individual or a legal entity exercising
 rights under, and complying with all of the terms of, this License.</li>

 <li>
 &ldquo;Modelica package&rdquo; means any Modelica library that is
 defined with the<br>&ldquo;<FONT FACE=\"Courier New, monospace\"><FONT SIZE=2 STYLE=\"font-size: 9pt\"><b>package</b></FONT></FONT><FONT FACE=\"Courier New, monospace\"><FONT SIZE=2 STYLE=\"font-size: 9pt\">
 &lt;Name&gt; ... </FONT></FONT><FONT FACE=\"Courier New, monospace\"><FONT SIZE=2 STYLE=\"font-size: 9pt\"><b>end</b></FONT></FONT><FONT FACE=\"Courier New, monospace\"><FONT SIZE=2 STYLE=\"font-size: 9pt\">
 &lt;Name&gt;;</FONT></FONT>&ldquo; Modelica language element.</li>
</ol>

<p>
<b>2. Grant of Copyright License.</b> Licensor grants You a
worldwide, royalty-free, non-exclusive, sublicensable license, for
the duration of the copyright, to do the following:</p>

<ol>
 <li><p>
 To reproduce the Original Work in copies, either alone or as part of
 a collection.</li></p>
 <li><p>
 To create Derivative Works according to Section 1d) of this License.</li></p>
 <li><p>
 To distribute or communicate to the public copies of the <u>Original
 Work</u> or a <u>Derivative Work</u> under <u>this License</u>. No
 fee, neither as a copyright-license fee, nor as a selling fee for
 the copy as such may be charged under this License. Furthermore, a
 verbatim copy of this License must be included in any copy of the
 Original Work or a Derivative Work under this License.<br>      For
 the matter of clarity, it is permitted A) to distribute or
 communicate such copies as part of a (possible commercial)
 collection where other parts are provided under different licenses
 and a license fee is charged for the other parts only and B) to
 charge for mere printing and shipping costs.</li></p>
 <li><p>
 To distribute or communicate to the public copies of a <u>Derivative
 Work</u>, alternatively to Section 2c), under <u>any other license</u>
 of your choice, especially also under a license for
 commercial/proprietary software, as long as You comply with Sections
 3, 4 and 8 below. <br>      For the matter of clarity, no
 restrictions regarding fees, either as to a copyright-license fee or
 as to a selling fee for the copy as such apply.</li></p>
 <li><p>
 To perform the Original Work publicly.</li></p>
 <li><p>
 To display the Original Work publicly.</li></p>
</ol>

<p>
<b>3. Acceptance.</b> Any use of the Original Work or a
Derivative Work, or any action according to either Section 2a) to 2f)
above constitutes Your acceptance of this License.</p>

<p>
<b>4. Designation of Derivative Works and of Modified Works.
</b>The identifying designation of Derivative Work and of Modified
Work must be different to the corresponding identifying designation
of the Original Work. This means especially that the (root-level)
name of a Modelica package under this license must be changed if the
package is modified (besides fixing of errors, adding vendor specific
Modelica annotations, using a subset of the classes of a Modelica
package, or using another representation, e.g. a binary
representation).</p>

<p>
<b>5. Grant of Patent License.</b>
Licensor grants You a worldwide, royalty-free, non-exclusive, sublicensable license,
under patent claims owned by the Licensor or licensed to the Licensor by
the owners of the Original Work that are embodied in the Original Work
as furnished by the Licensor, for the duration of the patents,
to make, use, sell, offer for sale, have made, and import the Original Work
and Derivative Works under the conditions as given in Section 2.
For the matter of clarity, the license regarding Derivative Works covers
patent claims to the extent as they are embodied in the Original Work only.</p>

<p>
<b>6. Provision of Source Code.</b> Licensor agrees to provide
You with a copy of the Source Code of the Original Work but reserves
the right to decide freely on the manner of how the Original Work is
provided.<br>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;For the matter of clarity, Licensor might provide only a binary
representation of the Original Work. In that case, You may (a) either
reproduce the Source Code from the binary representation if this is
possible (e.g., by performing a copy of an encrypted Modelica
package, if encryption allows the copy operation) or (b) request the
Source Code from the Licensor who will provide it to You.</p>

<p>
<b>7. Exclusions from License Grant.</b> Neither the names of
Licensor, nor the names of any contributors to the Original Work, nor
any of their trademarks or service marks, may be used to endorse or
promote products derived from this Original Work without express
prior permission of the Licensor. Except as otherwise expressly
stated in this License and in particular in Sections 2 and 5, nothing
in this License grants any license to Licensor&rsquo;s trademarks,
copyrights, patents, trade secrets or any other intellectual
property, and no patent license is granted to make, use, sell, offer
for sale, have made, or import embodiments of any patent claims.<br>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;No license is granted to the trademarks of
Licensor even if such trademarks are included in the Original Work,
except as expressly stated in this License. Nothing in this License
shall be interpreted to prohibit Licensor from licensing under terms
different from this License any Original Work that Licensor otherwise
would have a right to license.</p>

<p>
<b>8. Attribution Rights.</b> You must retain in the Source
Code of the Original Work and of any Derivative Works that You
create, all author, copyright, patent, or trademark notices, as well
as any descriptive text identified therein as an &quot;Attribution
Notice&quot;. The same applies to the licensing notice of this
License in the Original Work. For the matter of clarity, &ldquo;author
notice&rdquo; means the notice that identifies the original
author(s). <br>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;You must cause the Source Code for any Derivative
Works that You create to carry a prominent Attribution Notice
reasonably calculated to inform recipients that You have modified the
Original Work. <br>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;In case the Original Work or Derivative Work is not provided in
Source Code, the Attribution Notices shall be appropriately
displayed, e.g., in the documentation of the Derivative Work.</p>

<p><b>9. Disclaimer
of Warranty. <br></b><u><b>The Original Work is provided under this
License on an &quot;as is&quot; basis and without warranty, either
express or implied, including, without limitation, the warranties of
non-infringement, merchantability or fitness for a particular
purpose. The entire risk as to the quality of the Original Work is
with You.</b></u><b> </b>This disclaimer of warranty constitutes an
essential part of this License. No license to the Original Work is
granted by this License except under this disclaimer.</p>

<p>
<b>10. Limitation of Liability.</b> Under no circumstances and
under no legal theory, whether in tort (including negligence),
contract, or otherwise, shall the Licensor, the owner or a licensee
of the Original Work be liable to anyone for any direct, indirect,
general, special, incidental, or consequential damages of any
character arising as a result of this License or the use of the
Original Work including, without limitation, damages for loss of
goodwill, work stoppage, computer failure or malfunction, or any and
all other commercial damages or losses. This limitation of liability
shall not apply to the extent applicable law prohibits such
limitation.</p>

<p>
<b>11. Termination.</b> This License conditions your rights to
undertake the activities listed in Section 2 and 5, including your
right to create Derivative Works based upon the Original Work, and
doing so without observing these terms and conditions is prohibited
by copyright law and international treaty. Nothing in this License is
intended to affect copyright exceptions and limitations. This License
shall terminate immediately and You may no longer exercise any of the
rights granted to You by this License upon your failure to observe
the conditions of this license.</p>

<p>
<b>12. Termination for Patent Action.</b> This License shall
terminate automatically and You may no longer exercise any of the
rights granted to You by this License as of the date You commence an
action, including a cross-claim or counterclaim, against Licensor,
any owners of the Original Work or any licensee alleging that the
Original Work infringes a patent. This termination provision shall
not apply for an action alleging patent infringement through
combinations of the Original Work under combination with other
software or hardware.</p>

<p>
<b>13. Jurisdiction.</b> Any action or suit relating to this
License may be brought only in the courts of a jurisdiction wherein
the Licensor resides and under the laws of that jurisdiction
excluding its conflict-of-law provisions. The application of the
United Nations Convention on Contracts for the International Sale of
Goods is expressly excluded. Any use of the Original Work outside the
scope of this License or after its termination shall be subject to
the requirements and penalties of copyright or patent law in the
appropriate jurisdiction. This section shall survive the termination
of this License.</p>

<p>
<b>14. Attorneys&rsquo; Fees.</b> In any action to enforce the
terms of this License or seeking damages relating thereto, the
prevailing party shall be entitled to recover its costs and expenses,
including, without limitation, reasonable attorneys' fees and costs
incurred in connection with such action, including any appeal of such
action. This section shall survive the termination of this License.</p>

<p>
<b>15. Miscellaneous.</b>
</p>
<ol>
 <li>If any
 provision of this License is held to be unenforceable, such
 provision shall be reformed only to the extent necessary to make it
 enforceable.</li>

 <li>No verbal
 ancillary agreements have been made. Changes and additions to this
 License must appear in writing to be valid. This also applies to
 changing the clause pertaining to written form.</li>

 <li>You may use the
 Original Work in all ways not otherwise restricted or conditioned by
 this License or by law, and Licensor promises not to interfere with
 or be responsible for such uses by You.</li>
</ol>

<p>
<br>
</p>

<hr>

<h4><a name=\"2. Frequently Asked Questions|outline\"></a>
Frequently Asked Questions</h4>
<p>This
section contains questions/answer to users and/or distributors of
Modelica packages and/or documents under Modelica License 2. Note,
the answers to the questions below are not a legal interpretation of
the Modelica License 2. In case of a conflict, the language of the
license shall prevail.</p>

<p><br>
</p>

<p><FONT COLOR=\"#008000\"><FONT SIZE=3><b>Using
or Distributing a Modelica </b></FONT></FONT><FONT COLOR=\"#008000\"><FONT SIZE=3><u><b>Package</b></u></FONT></FONT><FONT COLOR=\"#008000\"><FONT SIZE=3><b>
under the Modelica License 2</b></FONT></FONT></p>

<p><b>What are the main
differences to the previous version of the Modelica License?</b></p>
<ol>
 <li><p>
 Modelica License 1 is unclear whether the licensed Modelica package
 can be distributed under a different license. Version 2 explicitly
 allows that &ldquo;Derivative Work&rdquo; can be distributed under
 any license of Your choice, see examples in Section 1d) as to what
 qualifies as Derivative Work (so, version 2 is clearer).</p>
 <li><p>
 If You modify a Modelica package under Modelica License 2 (besides
 fixing of errors, adding vendor specific Modelica annotations, using
 a subset of the classes of a Modelica package, or using another
 representation, e.g., a binary representation), you must rename the
 root-level name of the package for your distribution. In version 1
 you could keep the name (so, version 2 is more restrictive). The
 reason of this restriction is to reduce the risk that Modelica
 packages are available that have identical names, but different
 functionality.</p>
 <li><p>
 Modelica License 1 states that &ldquo;It is not allowed to charge a
 fee for the original version or a modified version of the software,
 besides a reasonable fee for distribution and support<SPAN LANG=\"en-GB\">&ldquo;.
 Version 2 has a </SPAN>similar intention for all Original Work under
 <u>Modelica License 2</u> (to remain free of charge and open source)
 but states this more clearly as &ldquo;No fee, neither as a
 copyright-license fee, nor as a selling fee for the copy as such may
 be charged&rdquo;. Contrary to version 1, Modelica License 2 has no
 restrictions on fees for Derivative Work that is provided under a
 different license (so, version 2 is clearer and has fewer
 restrictions).</p>
 <li><p>
 Modelica License 2 introduces several useful provisions for the
 licensee (articles 5, 6, 12), and for the licensor (articles 7, 12,
 13, 14) that have no counter part in version 1.</p>
 <li><p>
 Modelica License 2 can be applied to all type of work, including
 documents, images and data files, contrary to version 1 that was
 dedicated for software only (so, version 2 is more general).</p>
</ol>

<p><b>Can I distribute a
Modelica package (under Modelica License 2) as part of my commercial
Modelica modeling and simulation environment?</b></p>
<p>Yes,
according to Section 2c). However, you are not allowed to charge a
fee for this part of your environment. Of course, you can charge for
your part of the environment.
</p>

<p><b>Can I distribute a
Modelica package (under Modelica License 2) under a different
license?</b></p>
<p>No.
The license of an unmodified Modelica package cannot be changed
according to Sections 2c) and 2d). This means that you cannot <u>sell</u>
copies of it, any distribution has to be free of charge.</p>

<p><b>Can I distribute a
Modelica package (under Modelica License 2) under a different license
when I first encrypt the package?</b></p>
<p>No.
Merely encrypting a package does not qualify for Derivative Work and
therefore the encrypted package has to stay under Modelica License 2.</p>

<p><b>Can I distribute a
Modelica package (under Modelica License 2) under a different license
when I first add classes to the package?</b></p>
<p>No.
The package itself remains unmodified, i.e., it is Original Work, and
therefore the license for this part must remain under Modelica
License 2. The newly added classes can be, however, under a different
license.
</p>

<p><b>Can
I copy a class out of a Modelica package (under Modelica License 2)
and include it </b><u><b>unmodified</b></u><b> in a Modelica package
under a </b><u><b>commercial/proprietary license</b></u><b>?</b></p>
<p>No,
according to article 2c). However, you can include model, block,
function, package, record and connector classes in your Modelica
package under <u>Modelica License 2</u>. This means that your
Modelica package could be under a commercial/proprietary license, but
one or more classes of it are under Modelica License 2.<br>Note, a
&ldquo;type&rdquo; class (e.g., type Angle = Real(unit=&rdquo;rad&rdquo;))
can be copied and included unmodified under a commercial/proprietary
license (for details, see the next question).</p>

<p><b>Can
I copy a type class or </b><u><b>part</b></u><b> of a model, block,
function, record, connector class, out of a Modelica package (under
Modelica License 2) and include it modified or unmodified in a
Modelica package under a </b><u><b>commercial/proprietary</b></u><b>
license</b></p>
<p>Yes,
according to article 2d), since this will in the end usually qualify
as Derivative Work. The reasoning is the following: A type class or
part of another class (e.g., an equation, a declaration, part of a
class description) cannot be utilized &ldquo;by its own&rdquo;. In
order to make this &ldquo;usable&rdquo;, you have to add additional
code in order that the class can be utilized. This is therefore
usually Derivative Work and Derivative Work can be provided under a
different license. Note, this only holds, if the additional code
introduced is sufficient to qualify for Derivative Work. Merely, just
copying a class and changing, say, one character in the documentation
of this class would be no Derivative Work and therefore the copied
code would have to stay under Modelica License 2.</p>

<p><b>Can
I copy a class out of a Modelica package (under Modelica License 2)
and include it in </b><u><b>modified </b></u><b>form in a
</b><u><b>commercial/proprietary</b></u><b> Modelica package?</b></p>
<p>Yes.
If the modification can be seen as a &ldquo;Derivative Work&rdquo;,
you can place it under your commercial/proprietary license. If the
modification does not qualify as &ldquo;Derivative Work&rdquo; (e.g.,
bug fixes, vendor specific annotations), it must remain under
Modelica License 2. This means that your Modelica package could be
under a commercial/proprietary license, but one or more parts of it
are under Modelica License 2.</p>

<p><b>Can I distribute a
&ldquo;save total model&rdquo; under my commercial/proprietary
license, even if classes under Modelica License 2 are included?</b></p>
<p>Your
classes of the &ldquo;save total model&rdquo; can be distributed
under your commercial/proprietary license, but the classes under
Modelica License 2 must remain under Modelica License 2. This means
you can distribute a &ldquo;save total model&rdquo;, but some parts
might be under Modelica License 2.</p>

<p><b>Can I distribute a
Modelica package (under Modelica License 2) in encrypted form?</b></p>
<p>Yes.
Note, if the encryption does not allow &ldquo;copying&rdquo; of
classes (in to unencrypted Modelica source code), you have to send
the Modelica source code of this package to your customer, if he/she
wishes it, according to article&nbsp;6.</p>

<p><b>Can I distribute an
executable under my commercial/proprietary license, if the model from
which the executable is generated uses models from a Modelica package
under Modelica License 2?</b></p>
<p>Yes,
according to article 2d), since this is seen as Derivative Work. The
reasoning is the following: An executable allows the simulation of a
concrete model, whereas models from a Modelica package (without
pre-processing, translation, tool run-time library) are not able to
be simulated without tool support. By the processing of the tool and
by its run-time libraries, significant new functionality is added (a
model can be simulated whereas previously it could not be simulated)
and functionality available in the package is removed (e.g., to build
up a new model by dragging components of the package is no longer
possible with the executable).</p>

<p><b>Is my modification to
a Modelica package (under Modelica License 2) a Derivative Work?</b></p>
<p>It
is not possible to give a general answer to it. To be regarded as &quot;an
original work of authorship&quot;, a derivative work must be
different enough from the original or must contain a substantial
amount of new material. Making minor changes or additions of little
substance to a preexisting work will not qualify the work as a new
version for such purposes.
</p>

<p><br>
</p>
<p><FONT COLOR=\"#008000\"><FONT SIZE=3><b>Using
or Distributing a Modelica </b></FONT></FONT><FONT COLOR=\"#008000\"><FONT SIZE=3><u><b>Document</b></u></FONT></FONT><FONT COLOR=\"#008000\"><FONT SIZE=3><b>
under the Modelica License 2</b></FONT></FONT></p>

<p>This
section is devoted especially for the following applications:</p>
<ol>
 <li><p>
 A Modelica tool extracts information out of a Modelica package and
 presents the result in form of a &ldquo;manual&rdquo; for this
 package in, e.g., html, doc, or pdf format.</p>
 <li><p>
 The Modelica language specification is a document defining the
 Modelica language. It will be licensed under Modelica License 2.</p>
 <li><p>
 Someone writes a book about the Modelica language and/or Modelica
 packages and uses information which is available in the Modelica
 language specification and/or the corresponding Modelica package.</p>
</ol>

<p><b>Can I sell a manual
that was basically derived by extracting information automatically
from a Modelica package under Modelica License 2 (e.g., a &ldquo;reference
guide&rdquo; of the Modelica Standard Library):</b></p>
<p>Yes.
Extracting information from a Modelica package, and providing it in a
human readable, suitable format, like html, doc or pdf format, where
the content is significantly modified (e.g. tables with interface
information are constructed from the declarations of the public
variables) qualifies as Derivative Work and there are no restrictions
to charge a fee for Derivative Work under alternative 2d).</p>

<p><b>Can
I copy a text passage out of a Modelica document (under Modelica
License 2) and use it </b><u><b>unmodified</b></u><b> in my document
(e.g. the Modelica syntax description in the Modelica Specification)?</b></p>
<p>Yes.
In case you distribute your document, the copied parts are still
under Modelica License 2 and you are not allowed to charge a license
fee for this part. You can, of course, charge a fee for the rest of
your document.</p>

<p><b>Can
I copy a text passage out of a Modelica document (under Modelica
License 2) and use it in </b><u><b>modified</b></u><b> form in my
document?</b></p>
<p>Yes,
the creation of Derivative Works is allowed. In case the content is
significantly modified this qualifies as Derivative Work and there
are no restrictions to charge a fee for Derivative Work under
alternative 2d).</p>

<p><b>Can I sell a printed
version of a Modelica document (under Modelica License 2), e.g., the
Modelica Language Specification?</b></p>
<p>No,
if you are not the copyright-holder, since article 2c) does not allow
a selling fee for a (in this case physical) copy. However, mere
printing and shipping costs may be recovered.</p>
</html>"));
    end ModelicaLicense2;
    class Contact "Contact"
      annotation(Documentation(info = "<html>
<dl>
<dt><b>Main Author:</b>
<dd>
</dl>

<table border=0 cellspacing=0 cellpadding=2>
<tr>
<td>
<a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a><br>
    Deutsches Zentrum f&uuml;r Luft und Raumfahrt e.V. (DLR)<br>
    Institut f&uuml;r Robotik und Mechatronik<br>
    Abteilung f&uuml;r Systemdynamik und Regelungstechnik<br>
    Postfach 1116<br>
    D-82230 Wessling<br>
    Germany<br>
    email: <A HREF=\"mailto:Martin.Otter@dlr.de\">Martin.Otter@dlr.de</A></td>
</tr>
</table>


<p><b>Acknowledgements:</b></p>

<p>
The design of the Animation.Shape component is from Hilding Elmqvist, Dynasim AB.
</p>

</html>
"));
    end Contact;
  end UsersGuide;
  annotation(preferredView = "info", version = "1.0", versionDate = "2009-06-21", versionBuild = 2, revisionId = "$Id:: package.mo 3123 2009-11-08 18:29:01Z #$", Documentation(info = "<html>
<p>
This package contains a set of functions and models to be used in the
Modelica Standard Library that requires a tool specific implementation.
These are:
</p>

<ul>
<li> <a href=\"Modelica://ModelicaServices.Animation.Shape\">ModelicaServices.Animation.Shape</a>.
     provides a 3-dim. visualization of
     mechanical objects. It is used in
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape\">Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape</a>
     via inheritance.</li>
</ul>

<p>
<b>Licensed by DLR and Dynasim under the Modelica License 2</b><br>
Copyright &copy; 2009, DLR and Dynasim.
</p>

<p>
<i>This Modelica package is <u>free</u> software and
the use is completely at <u>your own risk</u>;
it can be redistributed and/or modified under the terms of the
Modelica license 2, see the license conditions (including the
disclaimer of warranty)
<a href=\"Modelica://ModelicaServices.UsersGuide.ModelicaLicense2\">here</a></u>
or at
<a href=\"http://www.Modelica.org/licenses/ModelicaLicense2\">
http://www.Modelica.org/licenses/ModelicaLicense2</a>.
</p>

</html>"), uses(Modelica(version = "3.1")));
  package Animation "Models and functions for 3-dim. animation"
    model Shape "Different visual shapes with variable size; all data have to be set as modifiers (see info layer)"
      extends Modelica.Utilities.Internal.PartialModelicaServices.Animation.PartialShape;
      import T = Modelica.Mechanics.MultiBody.Frames.TransformationMatrices;
      import SI = Modelica.SIunits;
      import Modelica.Mechanics.MultiBody.Frames;
      import Modelica.Mechanics.MultiBody.Types;
      function PackShape
        input Types.ShapeType shapeType;
        output Real pack;
      algorithm 
        pack:=1.2;
      end PackShape;
      function PackMaterial
        input Real material1;
        input Real material2;
        input Real material3;
        input Types.SpecularCoefficient sp;
        output Real mat;
      algorithm 
        mat:=material1 + material2 + material3 + sp;
      end PackMaterial;
    protected 
      Real abs_n_x(final unit = "1") = Modelica.Math.Vectors.length(lengthDirection) annotation(HideResult = true);
      Real e_x[3](each final unit = "1") = noEvent(if abs_n_x < 1e-10 then {1,0,0} else lengthDirection / abs_n_x) annotation(HideResult = true);
      Real n_z_aux[3](each final unit = "1") = cross(e_x, widthDirection) annotation(HideResult = true);
      Real e_y[3](each final unit = "1") = noEvent(cross(Modelica.Math.Vectors.normalize(cross(e_x, if n_z_aux * n_z_aux > 1e-06 then widthDirection else if abs(e_x[1]) > 1e-06 then {0,1,0} else {1,0,0})), e_x)) annotation(HideResult = true);
      output Real Form annotation(HideResult = false);
    public 
      output Real rxvisobj[3](each final unit = "1") "x-axis unit vector of shape, resolved in world frame" annotation(HideResult = false);
      output Real ryvisobj[3](each final unit = "1") "y-axis unit vector of shape, resolved in world frame" annotation(HideResult = false);
      output SI.Position rvisobj[3] "position vector from world frame to shape frame, resolved in world frame" annotation(HideResult = false);
    protected 
      output SI.Length size[3] "{length,width,height} of shape" annotation(HideResult = false);
      output Real Material annotation(HideResult = false);
      output Real Extra annotation(HideResult = false);
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100, -100},{80,60}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -100,60},{ -80,100},{100,100},{80,60},{ -100,60}}, lineColor = {0,0,255}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{100,100},{100, -60},{80, -100},{80,60},{100,100}}, lineColor = {0,0,255}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Text(extent = {{ -100, -100},{80,60}}, lineColor = {0,0,0}, textString = "%shapeType"),Text(extent = {{ -132,160},{128,100}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<html>

<p>
This model is documented at
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape\">Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape</a>.
</p>


</html>
"));

    equation 
      Form = (987000 + PackShape(shapeType)) * 1e+20;
      rxvisobj = transpose(R.T) * e_x;
      ryvisobj = transpose(R.T) * e_y;
      rvisobj = r + T.resolve1(R.T, r_shape);
      size = {length,width,height};
      Material = PackMaterial(color[1] / 255.0, color[2] / 255.0, color[3] / 255.0, specularCoefficient);
      Extra = extra;
    end Shape;
  end Animation;
end ModelicaServices;
package Modelica "Modelica Standard Library (Version 3.1)"
  extends Modelica.Icons.Library;
  package UsersGuide "User's Guide"
    annotation(__Dymola_DocumentationClass = true, Documentation(info = "<html>
<p>
Package <b>Modelica</b> is a <b>standardized</b> and <b>pre-defined</b> package
that is developed together with the Modelica language from the
Modelica Association, see
<a href=\"http://www.Modelica.org\">http://www.Modelica.org</a>.
It is also called <b>Modelica Standard Library</b>.
It provides constants, types, connectors, partial models and model
components in various disciplines.
</p>
<p>
This is a short <b>User's Guide</b> for
the overall library. Some of the main sublibraries have their own
User's Guides that can be accessed by the following links:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Electrical.Digital.UsersGuide\">Digital</a>
   </td>
   <td valign=\"top\">Library for digital electrical components based on the VHDL standard
   (2-,3-,4-,9-valued logic)</td>
</tr>

<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide\">FluxTubes</a>
    </td>
   <td valign=\"top\">Library for modelling of electromagnetic devices with lumped magnetic networks</td>
</tr>

<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide\">MultiBody</a>
    </td>
   <td valign=\"top\">Library to model 3-dimensional mechanical systems</td>
</tr>

<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.Rotational.UsersGuide\">Rotational</a>
    </td>
   <td valign=\"top\">Library to model 1-dimensional mechanical systems</td>
</tr>

<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Fluid.UsersGuide\">Fluid</a></td>
    <td valign=\"top\">Library of 1-dim. thermo-fluid flow models using the Modelica.Media media description</td>

<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Media.UsersGuide\">Media</a>
    </td>
   <td valign=\"top\">Library of media property models</td>
</tr>
<tr><td valign=\"top\"><a href=\"Modelica://Modelica.SIunits.UsersGuide\">SIunits</a> </td>
   <td valign=\"top\">Library of type definitions based on SI units according to ISO 31-1992</td>
</tr>

<tr><td valign=\"top\"><a href=\"Modelica://Modelica.StateGraph.UsersGuide\">StateGraph</a>
    </td>
   <td valign=\"top\">Library to model discrete event and reactive systems by hierarchical state machines</td>
</tr>


<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Utilities.UsersGuide\">Utilities</a>
    </td>
   <td valign=\"top\">Library of utility functions especially for scripting (Files, Streams, Strings, System)</td>
</tr>
</table>

</html>"));
    class Overview "Overview of Modelica Library"
      annotation(Documentation(info = "<html>
<p>
The Modelica Standard Library consists of the following
main sub-libraries:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th>Library Components</th> <th>Description</th></tr>

<tr><td valign=\"top\">
 <img src=\"../Images/UsersGuide/Lib-Electrical.png\">
 </td>
 <td valign=\"top\">
 <a href=\"Modelica://Modelica.Electrical.Analog\">Analog</a><br>
 Analog electric and electronic components, such as
 resistor, capacitor, transformers, diodes, transistors,
 transmission lines, switches, sources, sensors.
 </td>
</tr>

<tr><td valign=\"top\">
 <img src=\"../Images/UsersGuide/Lib-Digital.png\">
 </td>
 <td valign=\"top\">
 <a href=\"Modelica://Modelica.Electrical.Digital\">Digital</a><br>
 Digital electrical components based on the VHDL standard,
 like basic logic blocks with 9-value logic, delays, gates,
 sources, converters between 2-, 3-, 4-, and 9-valued logic.
 </td>
</tr>

<tr><td valign=\"top\">
 <img src=\"../Images/UsersGuide/Lib-Machines.png\">
 </td>
 <td valign=\"top\">
 <a href=\"Modelica://Modelica.Electrical.Machines\">Machines</a><br>
            Electrical asynchronous-, synchronous-, and DC-machines
 (motors and generators) as well as 3-phase transformers.
 </td>
</tr>

<tr><td valign=\"top\">
 <img src=\"../Images/UsersGuide/Lib-FluxTubes.png\">
 </td>
 <td valign=\"top\">
 <a href=\"Modelica://Modelica.Magnetic.FluxTubes\">FluxTubes</a><br>
Based on magnetic flux tubes concepts. Especially to model electro-magnetic actuators. Nonlinear shape, force, leackage, and material models. Material data for steel, electric sheet, pure iron, Cobalt iron, Nickel iron, NdFeB, Sm2Co17, and more.
 </td>
</tr>

<tr><td valign=\"top\">
 <img src=\"../Images/UsersGuide/Lib-Translational.png\">
 </td>
 <td valign=\"top\">
 <a href=\"Modelica://Modelica.Mechanics.Translational\">Translational</a><br>
 1-dim. mechanical, translational systems, e.g.,
 sliding mass, mass with stops, spring, damper.
 </td>
</tr>

<tr><td valign=\"top\">
 <img src=\"../Images/UsersGuide/Lib-Rotational.png\">
 </td>
 <td valign=\"top\">
 <a href=\"Modelica://Modelica.Mechanics.Rotational\">Rotational</a><br>
 1-dim. mechanical, rotational systems, e.g., inertias, gears,
 planetary gears, convenient definition of speed/torque dependent friction
 (clutches, brakes, bearings, ..)
 </td>
</tr>

<tr><td valign=\"top\" width=100>
 <img src=\"../Images/UsersGuide/Lib-MultiBody1.png\"><br>
 <img src=\"../Images/UsersGuide/Lib-MultiBody2.png\">
 </td>
 <td valign=\"top\">
 <a href=\"Modelica://Modelica.Mechanics.MultiBody\">MultiBody</a>
 3-dim. mechanical systems consisting of joints, bodies, force and
 sensor elements. Joints can be driven by drive trains defined by
 1-dim. mechanical system library (Rotational).
 Every component has a default animation.
 Components can be arbitrarily connected together.
 </td>
</tr>

<tr><td valign=\"top\">
 <img src=\"../Images/UsersGuide/Lib-Fluid.png\">
 </td>
 <td valign=\"top\">
 <a href=\"Modelica://Modelica.Fluid\">Fluid</a><br>
        1-dim. thermo-fluid flow in networks of vessels, pipes,
        fluid machines, valves and fittings. All media from the
        Modelica.Media library can be used (so incompressible or compressible,
        single or multiple substance, one or two phase medium).
 </td>
</tr>

<tr><td valign=\"top\">
 <img src=\"../Images/UsersGuide/Lib-Media.png\">
 </td>
 <td valign=\"top\">
 <a href=\"Modelica://Modelica.Media\">Media</a><br>
 Large media library providing models and functions
 to compute media properties, such as h = h(p,T), d = d(p,T),
 for the following media:
 <ul>
 <li> 1240 gases and mixtures between these gases.</li>
 <li> incompressible, table based liquids (h = h(T), etc.).</li>
 <li> compressible liquids</li>
 <li> dry and moist air</li>
 <li> high precision model for water (IF97).</li>
 </ul>
 </td>
</tr>

<tr><td valign=\"top\">
 <img src=\"../Images/UsersGuide/Lib-Thermal.png\">
 </td>
 <td valign=\"top\">
 <a href=\"Modelica://Modelica.Thermal.FluidHeatFlow\">FluidHeatFlow</a>,
 <a href=\"Modelica://Modelica.Thermal.HeatTransfer\">HeatTransfer</a>
 Simple thermo-fluid pipe flow, especially to model cooling of machines
 with air or water (pipes, pumps, valves, ambient, sensors, sources) and
 lumped heat transfer with heat capacitors, thermal conductors, convection,
 body radiation, sources and sensors.
 </td>
</tr>

<tr><td valign=\"top\">
 <img src=\"../Images/UsersGuide/Lib-Blocks1.png\"><br>
 <img src=\"../Images/UsersGuide/Lib-Blocks2.png\">
 </td>
 <td valign=\"top\">
 <a href=\"Modelica://Modelica.Blocks\">Blocks</a><br>
 Input/output blocks to model block diagrams and logical networks, e.g.,
 integerator, PI, PID, transfer function, linear state space system,
 sampler, unit delay, discrete transfer function, and/or blocks,
 timer, hysteresis, nonlinear and routing blocks, sources, tables.
 </td>
</tr>

<tr><td valign=\"top\">
 <img src=\"../Images/UsersGuide/Lib-StateGraph.png\">
 </td>
 <td valign=\"top\">
 <a href=\"Modelica://Modelica.StateGraph\">StateGraph</a><br>
 Hierarchical state machines with a similar modeling power as Statecharts.
 Modelica is used as synchronous action language, i.e. deterministic
 behavior is guaranteed
 </td>
</tr>


<tr><td valign=\"top\">
 <pre>
 A = [1,2,3;
   3,4,5;
   2,1,4];
 b = {10,22,12};
 x = Matrices.solve(A,b);
 Matrices.eigenValues(A);
 </pre>
 </td>
 <td valign=\"top\">
 <a href=\"Modelica://Modelica.Math\">Math</a>,
 <a href=\"Modelica://Modelica.Utilities\">Utilities</a><br>
 Functions operating on vectors and matrices, such as for solving
 linear systems, eigen and singular values etc.,  and
 functions operating on strings, streams, files, e.g.,
 to copy and remove a file or sort a vector of strings.
 </td>
</tr>

</table>

</html>
"));
    end Overview;
    class Connectors "Connectors"
      annotation(Documentation(info = "<html>

<p>
The Modelica standard library defines the most important
<b>elementary connectors</b> in various domains. If any possible,
a user should utilize these connectors in order that components
from the Modelica Standard Library and from other libraries
can be combined without problems. The following elementary
connectors are defined (potential variables are connector variables
without the flow attribute, flow variables are connector variables
that have the flow attribute):
</p>

<table border=1 cellspacing=0 cellpadding=1>
<tr><td valign=\"top\"><b>domain</b></td>
   <td valign=\"top\"><b>pot. variables</b></td>
   <td valign=\"top\"><b>flow variables</b></td>
   <td valign=\"top\"><b>connector definition</b></td>
   <td valign=\"top\"><b>icons</b></td></tr>

<tr><td valign=\"top\"><b>electrical<br>analog</b></td>
   <td valign=\"top\">electrical potential</td>
   <td valign=\"top\">electrical current</td>
   <td valign=\"top\"><a href=\"Modelica://Modelica.Electrical.Analog.Interfaces\">Modelica.Electrical.Analog.Interfaces</a>
     <br>Pin, PositivePin, NegativePin</td>
   <td valign=\"top\"><img src=\"../Images/UsersGuide/ElectricalPins.png\"></td></tr>

<tr><td valign=\"top\"><b>electrical<br>multi-phase</b></td>
   <td colspan=\"2\">vector of electrical pins</td>
   <td valign=\"top\"><a href=\"Modelica://Modelica.Electrical.MultiPhase.Interfaces\">Modelica.Electrical.MultiPhase.Interfaces</a>
     <br>Plug, PositivePlug, NegativePlug</td>
   <td valign=\"top\"><img src=\"../Images/UsersGuide/ElectricalPlugs.png\"></td></tr>

<tr><td valign=\"top\"><b>electrical <br>sphace phasor</b></td>
   <td valign=\"top\">2 electrical potentials</td>
   <td valign=\"top\">2 electrical currents</td>
   <td valign=\"top\"><a href=\"Modelica://Modelica.Electrical.Machines.Interfaces\">Modelica.Electrical.Machines.Interfaces</a>
     <br>SpacePhasor</td>
   <td valign=\"top\"><img src=\"../Images/UsersGuide/SpacePhasor.png\"></td></tr>

<tr><td valign=\"top\"><b>electrical <br>digital</b></td>
   <td valign=\"top\">Integer (1..9)</td>
   <td valign=\"top\">---</td>
   <td valign=\"top\"><a href=\"Modelica://Modelica.Electrical.Digital.Interfaces\">Modelica.Electrical.Digital.Interfaces</a>
     <br>DigitalSignal, DigitalInput, DigitalOutput</td>
   <td valign=\"top\"><img src=\"../Images/UsersGuide/Digital.png\"></td></tr>

<tr><td valign=\"top\"><b>magnetic</b></td>
   <td valign=\"top\">magnetic potential</td>
   <td valign=\"top\">magnetic flux</td>
   <td valign=\"top\">
<a href=\"Modelica://Modelica.Magnetic.FluxTubes.Interfaces\">Modelica.Magnetic.FluxTubes.Interfaces</a>
     <br>MagneticPort, PositiveMagneticPort, <br>NegativeMagneticPort</td>
   <td valign=\"top\"><img src=\"../Images/UsersGuide/MagneticPorts.png\"></td></tr>

<tr><td valign=\"top\"><b>translational</b></td>
   <td valign=\"top\">distance</td>
   <td valign=\"top\">cut-force</td>
   <td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.Translational.Interfaces\">Modelica.Mechanics.Translational.Interfaces</a>
     <br>Flange_a, Flange_b</td>
   <td valign=\"top\"><img src=\"../Images/UsersGuide/TranslationalFlanges.png\"></td></tr>

<tr><td valign=\"top\"><b>rotational</b></td>
   <td valign=\"top\">angle</td>
   <td valign=\"top\">cut-torque</td>
   <td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.Rotational.Interfaces\">Modelica.Mechanics.Rotational.Interfaces</a>
     <br>Flange_a, Flange_b</td>
   <td valign=\"top\"><img src=\"../Images/UsersGuide/RotationalFlanges.png\"></td></tr>

<tr><td valign=\"top\"><b>3-dim.<br>mechanics</b></td>
   <td valign=\"top\">position vector<br>
    orientation object</td>
   <td valign=\"top\">cut-force vector<br>
    cut-torque vector</td>
   <td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Interfaces\">Modelica.Mechanics.MultiBody.Interfaces</a>
     <br>Frame, Frame_a, Frame_b, Frame_resolve</td>
   <td valign=\"top\"><img src=\"../Images/UsersGuide/MultiBodyFrames.png\"></td></tr>

<tr><td valign=\"top\"><b>simple<br>fluid flow</b></td>
   <td valign=\"top\">pressure<br>
    specific enthalpy</td>
   <td valign=\"top\">mass flow rate<br>
    enthalpy flow rate</td>
   <td valign=\"top\"><a href=\"Modelica://Modelica.Thermal.FluidHeatFlow.Interfaces\">Modelica.Thermal.FluidHeatFlow.Interfaces</a>
     <br>FlowPort, FlowPort_a, FlowPort_b</td>
   <td valign=\"top\"><img src=\"../Images/UsersGuide/FluidHeatFlowPorts.png\"></td></tr>

<tr><td valign=\"top\"><b>thermo<br>fluid flow</b></td>
   <td valign=\"top\">pressure<br>
                  <u>stream</u> variables:<br>
    &nbsp;&nbsp;specific enthalpy<br>
    &nbsp;&nbsp;mass fractions</td>
   <td valign=\"top\">mass flow rate</td>
   <td valign=\"top\">
<a href=\"Modelica://Modelica.Fluid.Interfaces\">Modelica.Fluid.Interfaces</a>
     <br>FluidPort, FluidPort_a, FluidPort_b</td>
   <td valign=\"top\"><img src=\"../Images/UsersGuide/FluidPorts.png\"></td></tr>

<tr><td valign=\"top\"><b>heat<br>transfer</b></td>
   <td valign=\"top\">temperature</td>
   <td valign=\"top\">heat flow rate</td>
   <td valign=\"top\"><a href=\"Modelica://Modelica.Thermal.HeatTransfer.Interfaces\">Modelica.Thermal.HeatTransfer.Interfaces</a>
     <br>HeatPort, HeatPort_a, HeatPort_b</td>
   <td valign=\"top\"><img src=\"../Images/UsersGuide/ThermalHeatPorts.png\"></td></tr>

<tr><td valign=\"top\"><b>block<br>diagram</b></td>
   <td valign=\"top\">Real variable<br>
    Integer variable<br>
    Boolean variable</td>
   <td valign=\"top\">---</td>
   <td valign=\"top\"><a href=\"Modelica://Modelica.Blocks.Interfaces\">Modelica.Blocks.Interfaces</a>
     <br>RealSignal, RealInput, RealOutput<br>
      IntegerSignal, IntegerInput, IntegerOutput<br>
      BooleanSignal, BooleanInput, BooleanOutput</td>
   <td valign=\"top\"><img src=\"../Images/UsersGuide/Signals.png\"></tr>

<tr><td valign=\"top\"><b>state<br>machine</b></td>
   <td valign=\"top\">Boolean variables<br>
    (occupied, set, <br>
     available, reset)</td>
   <td valign=\"top\">---</td>
   <td valign=\"top\"><a href=\"Modelica://Modelica.StateGraph.Interfaces\">Modelica.StateGraph.Interfaces</a>
     <br>Step_in, Step_out, Transition_in, Transition_out</td>
   <td valign=\"top\"><img src=\"../Images/UsersGuide/StateGraphPorts.png\"></td></tr>


<tr><td colspan=\"5\">&nbsp;<br><b>Connectors from other libraries</b></td></tr>

<tr><td valign=\"top\"><b>hydraulic</b></td>
   <td valign=\"top\">pressure</td>
   <td valign=\"top\">volume flow rate</td>
   <td valign=\"top\">HyLibLight.Interfaces
     <br>Port_A, Port_b</td>
   <td valign=\"top\"><img src=\"../Images/UsersGuide/HydraulicPorts.png\"></td></tr>

<tr><td valign=\"top\"><b>pneumatic</b></td>
   <td valign=\"top\">pressure</td>
   <td valign=\"top\">mass flow rate</td>
   <td valign=\"top\">PneuLibLight.Interfaces
     <br>Port_1, Port_2</td>
   <td valign=\"top\"><img src=\"../Images/UsersGuide/PneumaticPorts.png\"></td></tr>
</table>

<p>
In all domains, usually 2 connectors are defined. The variable declarations
are <b>identical</b>, only the icons are different in order that it is easy
to distinguish connectors of the same domain that are attached at the same
component.
</p>

<p>
Modelica supports also hierarchical connectors, in a similar way as hierarchical models.
As a result, it is, e.g., possible, to collect elementary connectors together.
For example, an electrical plug consisting of two electrical pins can be defined as:
</p>

<pre>   <b>connector</b> Plug
   <b>import</b> Modelica.Electrical.Analog.Interfaces;
   Interfaces.PositivePin phase;
   Interfaces.NegativePin ground;
<b>end</b> Plug;
</pre>

<p>
With one connect(..) equation, either two plugs can be connected
(and therefore implicitly also the phase and ground pins) or a
Pin connector can be directly connected to the phase or ground of
a Plug connector, such as \"connect(resistor.p, plug.phase)\".
</p>


</html>
"));
    end Connectors;
    class Conventions "Conventions"
      annotation(Documentation(info = "<html>

<p>
Note, in the html documentation of any Modelica library,
the headings \"h1, h2, h3\" should not be used,
because they are utilized from the automatically generated documentation/headings.
Additional headings in the html documentation should start with \"h4\".
</p>

<p>
In the Modelica package the following conventions are used:
</p>
<ol>
<li> Class and instance names are written in upper and lower case
  letters, e.g., \"ElectricCurrent\". An underscore is only used
  at the end of a name to characterize a lower or upper index,
  e.g., \"pin_a\".<br>&nbsp;</li>

<li> <b>Class names</b> start always with an upper case letter.<br>&nbsp;</li>

<li> <b>Instance names</b>, i.e., names of component instances and
  of variables (with the exception of constants),
  start usually with a lower case letter with only
  a few exceptions if this is common sense
  (such as \"T\" for a temperature variable).<br>&nbsp;</li>

<li> <b>Constant names</b>, i.e., names of variables declared with the
  \"constant\" prefix, follow the usual naming conventions
  (= upper and lower case letters) and start usually with an
  upper case letter, e.g. UniformGravity, SteadyState.<br>&nbsp;<li>

<li> The two connectors of a domain that have identical declarations
  and different icons are usually distinguished by \"_a\", \"_b\"
  or \"_p\", \"_n\", e.g., Flange_a/Flange_b, HeatPort_a, HeatPort_b.<br>&nbsp;</li>

<li> The <b>instance name</b> of a component is always displayed in its icon
  (= text string \"%name\") in <b>blue color</b>. A connector class has the instance
  name definition in the diagram layer and not in the icon layer.
  <b>Parameter</b> values, e.g., resistance, mass, gear ratio, are displayed
  in the icon in <b>black color</b> in a smaller font size as the instance name.
 <br>&nbsp;<li>

<li> A main package has usually the following subpackages:
  <ul>
  <li><b>UsersGuide</b> containing an overall description of the library
   and how to use it.</li>
  <li><b>Examples</b> containing models demonstrating the
   usage of the library.</li>
  <li><b>Interfaces</b> containing connectors and partial
   models.</li>
  <li><b>Types</b> containing type, enumeration and choice
   definitions.</li>
  </ul>
  </li>
</ol>

</html>
"));
    end Conventions;
    class ParameterDefaults "Parameter defaults"
      annotation(Documentation(info = "<html>

<p>
In this section the convention is summarized how default parameters are
handled in the Modelica Standard Library (since version 3.0).
</p>

<p>
Many models in this library have parameter declarations to define
constants of a model that might be changed before simulation starts.
Example:
</p>

<blockquote>
<pre>
<b>model</b> SpringDamper
<b>parameter</b> Real c(final unit=\"N.m/rad\")    = 1e5 \"Spring constant\";
<b>parameter</b> Real d(final unit=\"N.m.s/rad\")  = 0   \"Damping constant\";
<b>parameter</b> Modelica.SIunits.Angle phi_rel0 = 0   \"Unstretched spring angle\";
...
<b>end</b> SpringDamper;
</pre>
</blockquote>

<p>
In Modelica it is possible to define a default value of a parameter in
the parameter declaration. In the example above, this is performed for
all parameters. Providing default values for all parameters can lead to
errors that are difficult to detect, since a modeler may have forgotten
to provide a meaningful value (the model simulates but gives wrong
results due to wrong parameter values). In general the following basic
situations are present:
</p>

<ol>
<li> The parameter value could be anything (e.g., a spring constant or
  a resistance value) and therefore the user should provide a value in
  all cases. A Modelica translator should warn, if no value is provided.
  <br><br></li>

<li> The parameter value is not changed in &gt; 95 % of the cases
  (e.g. initialization or visualization parameters, or parameter phi_rel0
  in the example above). In this case a default parameter value should be
  provided, in order that the model or function can be conveniently
  used by a modeler.
  <br><br></li>

<li> A modeler would like to quickly utilize a model, e.g.,
  <ul>
  <li> to automatically check that the model still translates and/or simulates
    (after some changes in the library),</li>
  <li> to make a quick demo of a library by drag-and-drop of components,</li>
  <li> to implement a simple test model in order to get a better understanding
    of the desired component.</li>
  </ul>
  In all these cases, it would be not practical, if the modeler would
  have to provide explicit values for all parameters first.
  </li>
</ol>

<p>
To handle the conflicting goals of (1) and (3), the Modelica Standard Library
uses two approaches to define default parameters, as demonstrated with the
following example:
</p>

<blockquote>
<pre>
<b>model</b> SpringDamper
<b>parameter</b> Real c(final unit=\"N.m/rad\"  , start=1e5) \"Spring constant\";
<b>parameter</b> Real d(final unit=\"N.m.s/rad\", start=  0) \"Damping constant\";
<b>parameter</b> Modelica.SIunits.Angle phi_rel0 = 0       \"Unstretched spring angle\";
...
<b>end</b> SpringDamper;

SpringDamper sp1;              // warning for \"c\" and \"d\"
SpringDamper sp2(c=1e4, d=0);  // fine, no warning
</pre>
</blockquote>

<p>
Both definition forms, using a \"start\" value (for \"c\" and \"d\") and providing
a declaration equation (for \"phi_rel0\"), are valid Modelica and define the value
of the parameter. By convention, it is expected that Modelica translators will
trigger a warning message for parameters that are <b>not</b> defined by a declaration
equation, by a modifier equation or in an initial equation/algorithm section.
A Modelica translator might have options to change this behavior, especially,
that no messages are printed in such cases and/or that an error is triggered
instead of a warning.
</p>

</html>
"));
    end ParameterDefaults;
    package ReleaseNotes "Release notes"
      class VersionManagement "Version Management"
        annotation(Documentation(info = "<html>
<p>
Maintenance of the Modelica Standard Library is performed with
three branches on the subversion server of the Modelica Association
(<a href=\"https://svn.modelica.org/projects/Modelica\">https://svn.modelica.org/projects/Modelica</a>):
</p>

<h4>Released branch</h4>
<p>
Example: \"/tags/v3.0.1/Modelica\"
</p>

<p>
This branch contains the released Modelica versions (e.g., version 3.0.1),
where all available test cases and compatibility checks with other Modelica
libraries have been performed on the respective release. This version is
usually shipped with a Modelica modeling and simulation environment and
utilized by a Modelica user.
</p>

<h4>Development branch</h4>
<p>
Example: \"/trunk/Modelica\"
</p>

<p>
This branch contains the actual development version, i.e., all bug fixes
and new features based on the last Modelica release.
New features should have been tested before including them.
However, the exhaustive tests for a new version are (usually) not performed.
This version is usually only be used by the developers of the
Modelica Standard Library and is not utilized by Modelica users.
</p>


<h4>Maintenance branch</h4>
<p>
Example: \"/branches/maintenance/3.0.1/Modelica\"
</p>

<p>
This branch contains the released Modelica version (e.g., version 3.0.1)
where all bug fixes since this release date are included (up to a  new release,
when becoming available; i.e., after a new release, the previous maintenance
versions are no longer changed).
These bug fixes might be not yet tested with all test cases or with
other Modelica libraries. The goal is that a vendor may take this version at
any time for a new release of its software, in order to incorporate the latest
bug fixes, without changing the version number of the Modelica Standard Library.
</p>

<p>
Incorporation of bug fixes (subversion \"commit\") shall be performed in the following way:
</p>

<ul>
<li> One person is fixing the bug and another person is checking whether the
         fix is fine.</li>
<li> It is up to the library developer, whether he opens a new branch for
         testing and then merges it with the \"head\" maintenance branch or not.</li>
<li> Every change to the maintenance branch has to be done at the development
         branch (see above) as well.</li>
<li> Every change to the maintenance branch requires introducing a
         description of the bug fix under
         Modelica.UsersGuide.ReleaseNotes.Version_&lt;release-number&gt;_BugFixes.</li>
<li> Annotations \"version\" and \"versionDate\" must <u>not</u> be changed in a maintenance release.</li>
<li> Every change to the maintenance branch requires changing the
     \"versionBuild\" number (incrementing it by one), as well as the \"revisionId\".
     The latter one is automatically done once a commit of file Modelica\\package.mo
     is performed. Example:
         <pre>  annotation(version      = \"3.1\",
             versionDate  = \"2009-06-22\",
             versionBuild = 3,
             dateModified = \"2009-06-21 08:13:18Z\",
             revisionId   = \"$Id:: package.mo 2987 2009-10-02 07:31:34Z #$\").</pre></li>
<li> If time does not permit, a vendor makes the bug fix in its local version
         and then has to include it in the maintenance version. It would be best to make these
         changes at a new branch in order to get a unique release number.</li>
</ul>

<p>
A valid \"commit\" to the maintenance branch may contain one or
more of the following changes.
</p>

<ul>
<li> Correcting an equation.</li>
<li> Correcting attributes quantity/unit/defaultUnit in a declaration.</li>
<li> Improving/fixing the documentation.</li>
<li> Introducing a new name in the public section of a class
         (model, package, ...) or in any section of a partial class is <b>not</b> allowed.
         Since otherwise, a user might use this new name and when storing its model
         and loading it with an older build-version, an error would occur.</li>
<li> Introducing a new name in the protected section of a non-partial
         class should only be done if absolutely necessary to fix a bug.
         The problem is that this might be non-backward compatible,
         because a user might already extend from this class and already using the same name.</li>
</ul>
</html>"));
      end VersionManagement;
      class Version_3_1 "Version 3.1 (August 14, 2009)"
        annotation(Documentation(info = "<html>

<p>
Version 3.1 is backward compatible to version 3.0 and 3.0.1,
i.e., models developed with version 3.0 or 3.0.1 will work without any
changes also with version 3.1.
</p>

<p>
Version 3.1 is slightly based on the Modelica Specification 3.1. It uses
the following new language elements (compared to Modelica Specification 3.0):
</p>

<ul>
<li> Prefix <u>stream</u> and built-in operators <u>inStream(..)</u>
     and <u>actualStream(..)</u> in Modelica.Fluid.</li>
<li> Annotation <u>connectorSizing</u> in Modelica.Fluid.</li>
<li> Annotation <u>inverse</u> in Modelica.Media.</li>
<li> Annotations <u>versionBuild</u>, <u>dateModified</u>,
     <u>revisionId</u> at the root level annotation of package Modelica,
     to improve the version handling.</li>
<li> Modifiers can be used in connectors instances (so balanced models
     are less restrictive). This allowed to make the implementation
     of conditional connectors (support and heatPort) in the Rotational,
     Translational and Electrical libraries simpler.<li>
<li>
</ul>

<p>
The following <b style=\"color:blue\">new libraries</b> have been added:
</p>
<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Fluid\">Modelica.Fluid</a></td>
    <td valign=\"top\">
     Components to model 1-dim. thermo-fluid flow in networks of vessels, pipes,
     fluid machines, valves and fittings. All media from the
     Modelica.Media library can be used (so incompressible or compressible,
     single or multiple substance, one or two phase medium).
    The library is using the stream-concept from Modelica Specification 3.1.
    </td>

<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Magnetic.FluxTubes\">Modelica.Magnetic.FluxTubes</a></td>
    <td valign=\"top\">
     Components to model magnetic devices based on the magnetic flux tubes concepts.
     Especially to model
     electro-magnetic actuators. Nonlinear shape, force, leackage, and
     Material models. Material data for steel, electric sheet, pure iron,
     Cobalt iron, Nickel iron, NdFeB, Sm2Co17, and more.
    </td>


<tr><td valign=\"top\"><a href=\"Modelica://ModelicaServices\">ModelicaServices</a></td>
    <td valign=\"top\">
     New top level package that shall contain functions and models to be used in the
     Modelica Standard Library that requires a tool specific implementation.
     ModelicaServices is then used in the Modelica package.
     In this first version, the 3-dim. animation with model Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape
     was moved to ModelicaServices. Tool vendors can now provide their own implementation
     of the animation.
    </td>
</table>


<p><br>
The following <b style=\"color:blue\">new components</b> have been added
to <b style=\"color:blue\">existing</b> libraries:
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Modelica.</b></td></tr>
<tr><td valign=\"top\"> versionBuild<br>versionDate<br>dateModified<br>revisionId </td>
    <td valign=\"top\"> New annotations from Modelica 3.1 for version handling added.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.UsersGuide.ReleaseNotes.</b></td></tr>
<tr><td valign=\"top\"> VersionManagement </td>
    <td valign=\"top\"> Copied from info layer of previous ReleaseNotes (to make it more
                      visible) and adapted it to the new possibilities in
                      Modelica Specification 3.1.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Blocks.Math.</b></td></tr>
<tr><td valign=\"top\"> RectangularToPolar<br>
                      PolarToRectangular </td>
    <td valign=\"top\"> New blocks to convert between rectangular and polar form
                      of space phasors. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Blocks.Routing.</b></td></tr>
<tr><td valign=\"top\"> Replicator </td>
    <td valign=\"top\"> New block to replicate an input signal to many output signals. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Examples.</b></td></tr>
<tr><td valign=\"top\"> AmplifierWithOpAmpDetailed<br>
                      HeatingResistor<br>
                      CompareTransformers<br>
                      OvervoltageProtection<br>
                      ControlledSwitchWithArc<br>
                      SwitchWithArc<br>
                      ThyristorBehaviourTest</td>
    <td valign=\"top\"> New examples to demonstrate the usage of new components. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Basic.</b></td></tr>
<tr><td valign=\"top\"> OpAmpDetailed<br>
                      TranslationalEMF<br>
                      M_Transformer</td>
    <td valign=\"top\"> New detailed model of an operational amplifier. <br>
                      New electromotoric force from electrical energy into mechanical translattional energy.<br>
                      Generic transformer with choosable number of inductors</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Ideal.</b></td></tr>
<tr><td valign=\"top\"> OpenerWithArc<br>
                      CloserWithArc<br>
                      ControlledOpenerWithArc<br>
                      ControlledCloserWithArc</td>
    <td valign=\"top\"> New switches with simple arc model. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> ConditionalHeatPort</td>
    <td valign=\"top\"> New partial model to add a conditional HeatPort to
                      an electrical component. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Lines.</b></td></tr>
<tr><td valign=\"top\"> M_Oline</td>
    <td valign=\"top\"> New multiple line model, both the number of lines and the number of segmants choosable. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Semiconductors.</b></td></tr>
<tr><td valign=\"top\"> ZDiode<br>Thyristor</td>
    <td valign=\"top\"> Zener Diode with 3 working areas and simple thyristor model. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.MultiPhase.Ideal.</b></td></tr>
<tr><td valign=\"top\"> OpenerWithArc<br>CloserWithArc</td>
    <td valign=\"top\"> New switches with simple arc model (as in Modelica.Electrical.Analog.Ideal. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.Examples.Elementary.</b></td></tr>
<tr><td valign=\"top\"> RollingWheel<br>
                      RollingWheelSetDriving<br>
                      RollingWheelSetPulling</td>
    <td valign=\"top\"> New examples to demonstrate the usage of new components.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.Joints.</b></td></tr>
<tr><td valign=\"top\"> RollingWheel<br>
                      RollingWheelSet</td>
    <td valign=\"top\"> New joints (no mass, no inertia) that describe an
                      ideal rolling wheel and a ideal rolling wheel set consisting
                      of two wheels rolling on the plane z=0.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.Parts.</b></td></tr>
<tr><td valign=\"top\"> RollingWheel<br>
                      RollingWheelSet</td>
    <td valign=\"top\"> New ideal rolling wheel and ideal rolling wheel set consisting
                      of two wheels rolling on the plane z=0.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.Visualizers.</b></td></tr>
<tr><td valign=\"top\"> Ground</td>
    <td valign=\"top\"> New model to visualize the ground (box at z=0).</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.Rotational.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> PartialElementaryOneFlangeAndSupport2<br>
                      PartialElementaryTwoFlangesAndSupport2</td>
    <td valign=\"top\"> New partial model with one and two flanges and the support flange
                      with a much simpler implementation as previously.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.Translational.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> PartialElementaryOneFlangeAndSupport2<br>
                      PartialElementaryTwoFlangesAndSupport2</td>
    <td valign=\"top\"> New partial model with one and two flanges and the support flange
                      with a much simpler implementation as previously.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Media.IdealGases.Common.MixtureGasNasa.</b></td></tr>
<tr><td valign=\"top\"> setSmoothState</td>
    <td valign=\"top\"> Return thermodynamic state so that it smoothly approximates:
                      if x &gt; 0 then state_a else state_b.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Utilities.Internal.</b></td></tr>
<tr><td valign=\"top\"> PartialModelicaServices</td>
    <td valign=\"top\"> New package containing the interface description of
                      models and functions that require a tool dependent
                      implementation (currently only \"Shape\" for 3-dim. animation,
                      but will be extended in the future)</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Thermal.HeatTransfer.Components.</b></td></tr>
<tr><td valign=\"top\"> ThermalCollector</td>
    <td valign=\"top\"> New auxiliary model to collect the heat flows
                      from m heatports to a single heatport;
                      useful for multiphase resistors (with heatports)
                      as a junction of the m heatports.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Icons.</b></td></tr>
<tr><td valign=\"top\"> VariantLibrary<br>
                      BaseClassLibrary<br>
                      ObsoleteModel</td>
    <td valign=\"top\"> New icons (VariantLibrary and BaseClassLibrary have been moved
                      from Modelica_Fluid.Icons to this place).</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.SIunits.</b></td></tr>
<tr><td valign=\"top\"> ElectricalForceConstant </td>
    <td valign=\"top\"> New type added (#190).</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.SIunits.Conversions.</b></td></tr>
<tr><td valign=\"top\"> from_Hz<br>
                      to_Hz</td>
    <td valign=\"top\"> New functions to convert between frequency [Hz] and
                      angular velocity [1/s]. (#156) </td> </tr>

</table>



<p><br>
The following <b style=\"color:blue\">existing components</b>
have been <b style=\"color:blue\">improved</b> in a
<b style=\"color:blue\">backward compatible</b> way:
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Modelica.</b></td></tr>
<tr><td valign=\"top\"> Blocks<br>Mechanics<br>StateGraph </td>
    <td valign=\"top\"> Provided missing parameter values for examples
                      (these parameters had only start values)</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Basic</b></td></tr>
<tr><td valign=\"top\"> Resistor, Conductor, VariableResistor, VariableConductor</td>
    <td valign=\"top\"> Conditional heatport added for coupling to thermal network. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Ideal</b></td></tr>
<tr><td valign=\"top\"> Thyristors, Switches, IdealDiode</td>
    <td valign=\"top\"> Conditional heatport added for coupling to thermal network. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Semiconductors</b></td></tr>
<tr><td valign=\"top\"> Diode, ZDiode, PMOS, NMOS, NPN, PNP</td>
    <td valign=\"top\"> Conditional heatport added for coupling to thermal network. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.MultiPhase.Basic</b></td></tr>
<tr><td valign=\"top\"> Resistor, Conductor, VariableResistor, VariableConductor</td>
    <td valign=\"top\"> Conditional heatport added for coupling to thermal network (as in Modelica.Electrical.Analog). </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.MultiPhase.Ideal</b></td></tr>
<tr><td valign=\"top\"> Thyristors, Switches, IdealDiode</td>
    <td valign=\"top\"> Conditional heatport added for coupling to thermal network (as in Modelica.Electrical.Analog). </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.Visualizers.Advanced.</b></td></tr>
<tr><td valign=\"top\"> Shape </td>
    <td valign=\"top\"> New implementation by inheriting from ModelicaServices. This allows a
                      tool vendor to provide its own implementation of Shape. </td> </tr>
<tr><td colspan=\"2\"><b>Modelica.StateGraph.</b></td></tr>
<tr><td valign=\"top\"> Examples </td>
    <td valign=\"top\"> Introduced \"StateGraphRoot\" on the top level of all example models. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.StateGraph.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> StateGraphRoot<br>PartialCompositeStep<br>CompositeStepState </td>
    <td valign=\"top\"> Replaced the wrong Modelica code \"flow output Real xxx\"
                      by \"Real dummy; flow Real xxx;\".
                      As a side effect, several \"blocks\" had to be changed to \"models\". </td> </tr>
<tr><td valign=\"top\"> PartialStep </td>
    <td valign=\"top\"> Changed model by packing the protected outer connecter in to a model.
                      Otherwise, there might be differences in the sign of the flow variable
                      in Modelica 3.0 and 3.1. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Utilities.Examples.</b></td></tr>
<tr><td valign=\"top\"> expression </td>
    <td valign=\"top\"> Changed local variable \"operator\" to \"opString\" since \"operator\"
                      is a reserved keyword in Modelica 3.1 </td> </tr>
</table>


<p><br>
The following <b style=\"color:red\">uncritical errors</b> have been fixed (i.e. errors
that do <b style=\"color:red\">not</b> lead to wrong simulation results, but, e.g.,
units are wrong or errors in documentation):
</p>
<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Modelica.</b></td></tr>
<tr><td valign=\"top\"> Many models</td>
    <td valign=\"top\"> Removed wrong usages of annotations fillColor and fillPattern
                      in text annotations (#155, #185).</td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.Machines</b></td></tr>
<tr><td valign=\"top\"> All machine models</td>
    <td valign=\"top\"> The conditional heatports of the instantiated resistors
                        (which are new in Modelica.Electrical.Analog and Modelica.Electrical.MultiPhase)
                        are finally switched off until a thermal connector design for machines is implemented. </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Media.Air.MoistAir</b></td></tr>
<tr><td valign=\"top\"> saturationPressureLiquid<br>
                      sublimationPressureIce<br>
                      saturationPressure</td>
          <td valign=\"top\"> For these three functions, an error in the <code>derivative</code> annotation was corrected. However, the effect of
                            this bug was minor, as a Modelica tool was allowed to compute derivatives automatically via
                            the <code>smoothOrder</code> annotation.</td>
</tr>
<tr><td colspan=\"2\"><b>Modelica.Math.Matrices.</b></td></tr>
<tr><td valign=\"top\"> eigenValues</td>
    <td valign=\"top\"> Wrong documentation corrected (#162)</td> </tr>
</table>

</html>"));
      end Version_3_1;
      class Version_3_0_1 "Version 3.0.1 (Jan.27, 2009)"
        annotation(Documentation(info = "<html>

<p>
This Modelica package is provided under the
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica License 2</a>
and no longer under Modelica License 1.1. There are the following reasons
why the Modelica Association changes from Modelica License 1.1 to this
new license text (note, the text below is not a legal interpretation of the
Modelica License 2. In case of a conflict, the language of the license shall prevail):
</p>

<ol>
<li> The rights of licensor and licensee are much more clearly defined. For example:
         <ul>
         <li> The licensed work (Original Work) can be used in unmodified form in
                  open source and commercial software (the licensee cannot change the
                  license and the work must be provided without fees)</li>
         <li> If a model component is copied out of a Modelica package under
                  Modelica License 2 and is modified in order to adapt it to the needs
                  of the modeler, then the result can be licensed under any license
                  (including a commercial license).</li>
         <li> It is practically not possible to change the license of a
                  Modelica package under Modelica License 2 to another license, i.e., a
                  licensee cannot change the license by adding material or changing classes,
                  so the work must remain under Modelica License 2 (to be more precise,
                  if the licensee makes modifications to the Original Work \"which represents,
                  as a whole, an original work of authorship\", he/she can change the license
                  to another license. However, for a Modelica package this would
                  require a lot of changes which is usually unrealistic).</li>
         <li> If an executable is constructed using a Modelica package under
                  Modelica License 2, then this executable can be licensed under any
                  license (including a commercial license).</li>
         </ul>
         We hope that this compromise between open source contributors, commercial
         Modelica environments and Modelica users will motivate even more people to
         provide their Modelica packages freely under the Modelica License 2.<br><br></li>
<li> There are several new provisions that shall make law suites against licensors and licensees more unlikely (so the small risk is further reduced).</li>
</ol>



<p><br>
The following <b style=\"color:blue\">new components</b> have been added
to <b style=\"color:blue\">existing</b> libraries:
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Electrical.Analog.Basic.</b></td></tr>
<tr><td valign=\"top\">M_Transformer</td>
          <td valign=\"top\"> Transformer, with the possibility to
        choose the number of inductors. The inductances and the coupled inductances
        can be chosen arbitrarily.</td> </tr>

<tr><td colspan=\"2\"><b>Electrical.Analog.Lines.</b></td></tr>
<tr><td valign=\"top\">M_OLine</td>
          <td valign=\"top\"> Segmented line model that enables the use of
        multiple lines, that means, the number of segments and the number of
        single lines can be chosen by the user. The model allows to investigate
        phenomena at multiple lines like mutual magnetic or capacitive influence.</td> </tr>
<tr><td colspan=\"2\"><b>Mechanics.Translational.Components.Examples.</b></td></tr>
<tr><td valign=\"top\">Brake</td>
          <td valign=\"top\"> Demonstrates the usage of the translational brake component.</td> </tr>
<tr><td colspan=\"2\"><b>Media.Interfaces.PartialMedium.</b></td></tr>
<tr><td valign=\"top\">ThermoStates</td>
          <td valign=\"top\"> Enumeration type for independent variables to identify the independent
                                                variables of the medium (pT, ph, phX, pTX, dTX).<br>
                                                An implementation of this enumeration is provided for every medium.
                                                (This is useful for fluid libraries that do not use the
                                                PartialMedium.BaseProperties model). </td> </tr>
<tr><td valign=\"top\">setSmoothState</td>
          <td valign=\"top\"> Function that returns the thermodynamic state which smoothly approximates:
                                                if x > 0 then state_a else state_b.<br>
                                                (This is useful for pressure drop components in fluid libraries
                                                 where the upstream density and/or viscosity has to be computed
                                                 and these properties should be smooth a zero mass flow rate)<br>
                                                An implementation of this function is provided for every medium.</td> </tr>
<tr><td colspan=\"2\"><b>Media.Common.</b></td></tr>
<tr><td valign=\"top\">smoothStep</td>
          <td valign=\"top\"> Approximation of a general step, such that the characteristic
                                                is continuous and differentiable.</td> </tr>
<tr><td colspan=\"2\"><b>Media.UsersGuide.</b></td></tr>
<tr><td valign=\"top\">Future</td>
          <td valign=\"top\"> Short description of goals and changes of upcoming release of Modelica.Media. </td> </tr>
<tr><td colspan=\"2\"><b>Media.Media.Air.MoistAir.</b></td></tr>
<tr><td valign=\"top\">isentropicExponent</td>
          <td valign=\"top\"> Implemented Missing Function from interface. </td> </tr>
<tr><td valign=\"top\">isentropicEnthalpyApproximation</td>
<td valign=\"top\"> Implemented function that approximates the isentropic enthalpy change.
This is only correct as long as there is no liquid in the stream.</td> </tr>
</table>


<p><br>
The following <b style=\"color:blue\">existing components</b>
have been <b style=\"color:blue\">changed</b> (in a
<b style=\"color:blue\">backward compatible</b> way):
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Mechanics.Rotational.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> PartialFriction </td>
          <td valign=\"top\"> Improvement of friction model so that in certain situations
                                                the number of iterations is much smaller.</td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.Translational.Components.Examples.</b></td></tr>
<tr><td valign=\"top\"> Friction </td>
          <td valign=\"top\"> Added a third variant, where friction is modelled with
                                                the SupportFriction component.</td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.Translational.Components.</b></td></tr>
<tr><td valign=\"top\"> MassWithStopAndFriction </td>
          <td valign=\"top\"> Improvement of friction model so that in certain situations
                                                the number of iterations is much smaller.</td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.Translational.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> PartialFriction </td>
          <td valign=\"top\"> Improvement of friction model so that in certain situations
                                                the number of iterations is much smaller.</td>
</tr>

<tr><td colspan=\"2\"><b>Media.Examples.</b></td></tr>
<tr><td valign=\"top\"> SimpleLiquidWater <br>
                                                IdealGasH20 <br>
                                                WaterIF97 <br>
                                                MixtureGases <br>
                                                MoistAir </td>
          <td valign=\"top\"> Added equations to test the new setSmoothState(..) functions
                                                including the analytic derivatives of these functions.</td> </tr>

<tr><td colspan=\"2\"><b>Media.Interfaces.PartialLinearFluid.</b></td></tr>
<tr><td valign=\"top\"> setState_pTX <br>
                                                setState_phX <br>
                                                setState_psX <br>
                                                setState_dTX </td>
          <td valign=\"top\"> Rewritten function in one statement so that it is usually inlined.</td> </tr>

<tr><td colspan=\"2\"><b>Media.Interfaces.PartialLinearFluid.</b></td></tr>
<tr><td valign=\"top\"> consistent use of reference_d instead of density(state </td>
          <td valign=\"top\"> Change was done to achieve consistency with analytic inverse functions.</td> </tr>


<tr><td colspan=\"2\"><b>Media.Air.MoistAir.</b></td></tr>
<tr><td valign=\"top\"> T_phX </td>
          <td valign=\"top\"> Interval of nonlinear solver to compute T from p,h,X changed
                                                from 200..6000 to 240 ..400 K.</td> </tr>

</table>


<p><br>
The following <b style=\"color:red\">critical errors</b> have been fixed (i.e. errors
that can lead to wrong simulation results):
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Forces</b></td></tr>
<tr><td valign=\"top\"> WorldTorque </td>
          <td valign=\"top\"> Parameter \"ResolveInFrame\" was not propagated and therefore
                                                always the default (resolved in world frame) was used, independently
                                                of the setting of this parameter. </td>
</tr>
<tr><td valign=\"top\"> WorldForceAndTorque </td>
          <td valign=\"top\"> Parameter \"ResolveInFrame\" was not propagated and therefore
                                                always the default (resolved in world frame) was used, independently
                                                of the setting of this parameter.<br>
                                                Furthermore, internally WorldTorque was used instead of
                                                Internal.BasicWorldTorque and therefore the visualization of
                                                worldTorque was performed twice. </td>
</tr>
<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Sensors</b></td></tr>
<tr><td valign=\"top\"> AbsoluteSensor </td>
          <td valign=\"top\"> Velocity, acceleration and angular acceleration were computed
                                                  by differentiating in the resolveInFrame frame. This has been corrected, by
                                                  first transforming the vectors in to the world frame, differentiating here
                                                  and then transforming into resolveInFrame. The parameter in the Advanced menue
                                                  resolveInFrameAfterDifferentiation is then superfluous and was removed . </td>
</tr>
<tr><td valign=\"top\"> AbsoluteVelocity </td>
          <td valign=\"top\"> The velocity was computed
                                                  by differentiating in the resolveInFrame frame. This has been corrected, by
                                                  first transforming the velocity in to the world frame, differentiating here
                                                  and then transforming into resolveInFrame </td>
</tr>
<tr><td valign=\"top\"> RelativeSensor </td>
          <td valign=\"top\"> If resolveInFrame &lt;&gt; frame_resolve and
                                                   resolveInFrameAfterDifferentiation = frame_resolve, a translation
                                                error occured, since frame_resolve was not enabled in this situation.
                                                This has been corrected.</td>
</tr>
<tr><td valign=\"top\"> RelativeVelocity </td>
          <td valign=\"top\"> The velocity has have been computed
                                                  by differentiating in the resolveInFrame frame. This has been corrected, by
                                                  first transforming the relative position in to frame_a, differentiating here
                                                  and then transforming into resolveInFrame </td>
</tr>
<tr><td valign=\"top\"> TransformRelativeVector </td>
          <td valign=\"top\"> The transformation was wrong, since the parameters frame_r_in and frame_r_out
                                                have not been propagated to the submodel that performs the transformation.
                                                This has been corrected. </td>
</tr>
<tr><td colspan=\"2\"><b>Mechanics.Translational.Components.</b></td></tr>
<tr><td valign=\"top\"> SupportFriction<br>
                                                Brake </td>
          <td valign=\"top\"> The sign of the friction force was wrong and therefore friction accelerated
                                                instead of decelerated. This was fixed. </td>
<tr><td valign=\"top\"> SupportFriction</td>
          <td valign=\"top\"> The component was only correct for fixed support.
                                                This was corrected. </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> PartialSimpleMedium<br>
                                                PartialSimpleIdealGasMedium </td>
          <td valign=\"top\"> BaseProperties.p was not defined as preferred state and BaseProperties.T was
                                                always defined as preferred state. This has been fixed by
                                                Defining p,T as preferred state if parameter preferredMediumState = true.
                                                This error had the effect that mass m is selected as state instead of p
                                                and if default initialization is used then m=0 could give not the expected
                                                behavior. This means, simulation is not wrong but the numerics is not as good
                                                and if a model relies on default initial values, the result could be not
                                                as expected. </td>
</tr>

</table>


<p><br>
The following <b style=\"color:red\">uncritical errors</b> have been fixed (i.e. errors
that do <b style=\"color:red\">not</b> lead to wrong simulation results, but, e.g.,
units are wrong or errors in documentation):
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Blocks.Math.</b></td></tr>
<tr><td valign=\"top\"> InverseBlockConstraint </td>
          <td valign=\"top\"> Changed annotation preserveAspectRatio from true to false.</td>
</tr>

<tr><td colspan=\"2\"><b>Blocks.Sources.</b></td></tr>
<tr><td valign=\"top\"> RealExpression<br>
                                                IntegerExpression<br>
                                                BooleanExpression </td>
          <td valign=\"top\"> Changed annotation preserveAspectRatio from true to false.</td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Analog.Basic.</b></td></tr>
<tr><td valign=\"top\"> SaturatingInductor</td>
          <td valign=\"top\"> Replaced non-standard \"arctan\" by \"atan\" function.</td>
</tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Digital.</b></td></tr>
<tr><td valign=\"top\"> UsersGuide</td>
          <td valign=\"top\"> Removed empty documentation placeholders and added the missing
                                                  release comment for version 1.0.7</td>
</tr>


<tr><td colspan=\"2\"><b>Modelica.Mechanics.Translational.Components.</b></td></tr>
<tr><td valign=\"top\"> MassWithStopAndFriction </td>
          <td valign=\"top\"> Changed usage of reinit(..), in order that it appears
                                                only once for one variable according to the language specification
                                                (if a tool could simulate the model, there is no difference).</td>
</tr>

<tr><td colspan=\"2\"><b>Media.Interfaces.PartialSimpleMedium</b></td></tr>
<tr><td valign=\"top\"> pressure<br>
                                                temperature<br>
                                                density<br>
                                                specificEnthalpy </td>
          <td valign=\"top\"> Missing functions added.</td>
</tr>

</table>


</html>"));
      end Version_3_0_1;
      annotation(Documentation(info = "<html>

<p>
This section summarizes the changes that have been performed
on the Modelica standard library. Furthermore, it is explained
<a href=\"Modelica://Modelica.UsersGuide.ReleaseNotes.VersionManagement\">here</a>
how the versions are managed with the subversion management systems.
This is especially important for maintenance (bug-fix) releases where the
main version number is not changed.
</p>

</html>
"));
      class Version_3_0 "Version 3.0 (Feb., 2008)"
        annotation(Documentation(info = "<html>
<p>
Version 3.0 is <b>not</b> backward compatible to previous versions.
A conversion script is provided to transform models and libraries
of previous versions to the new version. Therefore, conversion
should be automatic.
</p>

<p>
The following changes are present for the whole library:
</p>

<ul>
<li> In the Modelican language version 3.0, several restrictions have been
         introduced to allow better checking, e.g., models on all levels must be balanced
         (number of equations = number of unknown variables - unknown variables that have
         to be defined when using the component). A few models of the Modelica
         Standard Library did not fulfill these new restrictions and had
         either to be moved to library ObsoleteModelica3 (e.g. Blocks.Math.TwoInputs)
         or had to be differently implemented
         (e.g. Media.Interfaces.PartialMedium.BaseProperties).
         The Modelica Standard Library version 3.0 fulfills all the restrictions of
         the Modelica Language version 3.0.<br>&nbsp;
         </li>

<li> The graphical annotations describing the layout of icon and diagram layer
         are changed from Modelica language version 1 to Modelica language version 3.
         This gives several significant improvements:<br>Especially, the coordinate systems
         of icon and diagram layers are no longer coupled and therefore the size of the
         icon layer can be changed independently of the size of the diagram layer.
         Also it can be defined that the aspect ratio of a component icon is kept when changing
         its size in a model. This flag is set so that all icons of the Modelica
         Standard Library keep its aspect ratios. This is slightly non-backward compatible:
         If the aspect ratio was not kept when using a component from the Modelica
         Standard Library, it is now resized so that the aspect ratio is maintained.<br>&nbsp; </li>

<li> All non-standard annotations removed by:<br>
         (1) Removing the annotation since without effect
                 (e.g., \"__Dymola_experimentSetupOutput\", \"Window\", \"Terminal\" removed).<br>
         (2) Renaming the annotation to a standard name (e.g., \"Hide\" renamed to \"HideResult\").<br>
         (3) Renaming the annotation to a vendor specific name
                 (e.g., \"checkBox\" renamed to \"__Dymola_checkBox\").<br>&nbsp; </li>

<li> All emulated enumerations (defined via packages and constants) have been
         replaced by \"real\" enumerations. User models are automatically correctly
         converted, provided the user models used the package constants previously.
         <b>Existing models that use directly literal values for enumerations, might give in
         some cases wrong results</b> (if the first constant of the emulated enumeration
         had value zero, whereas the first value of an enumeration is one).<br>&nbsp; </li>

<li> The operator \"cardinality\" will be removed in one of the next versions of the
         Modelica language, since it is a reflective operator and its usage significantly
         reduces the possibilities of advanced model checks (e.g. to guarantee that a model
         is \"balanced\", i.e., the number of equations and unknowns is identical,
         for all valid usages of the component). As a preparation for this change, all
         models that contain the \"cardinality(..)\" operator are rewritten: If possible
         the operator is removed. If this is not possible, it is only used in asserts to
         check that, e.g., a connector is connected at least once or is connected exactly
         once. In the next Modelica language version new language elements will be introduced
         to specifiy such a property check without the cardinality operator. Once these
         language elements are available, the cardinality operator will be removed completely
         from the Modelica Standard Library.<br>
         The changes with respect to the cardinality(..) operator are usually not backward
         compatible. This is the reason for the changes of the
         Rotational and Translational library (see below).<br>&nbsp;</li>

<li> The design of the <b>Rotational</b> and <b>Translational</b> libraries have been changed
         (especially to remove the cardinality(..) operator, see above):
         <ul>
         <li> Components have a <b>useSupport</b> flag to enable or disable a support flange.
                  If the support flange is enabled, it must be connected. If it is disabled, it must
                  not be connected and the component is then internally grounded. The grounding
                  is visualized in the icon.</li>
         <li> The relative angle/distance and the relative speed of all force/torque elements
                  (that need the relative speed) are by default defined with \"StateSelect.prefer\", i.e.,
                  to use these variables as preferred states. This improves the numerics if the
                  absolute angle or the absolute distance are continuously increasing during
                  operation (e.g. driving shaft of the wheels of a car). The effect is that relative
                  angles/distances and speeds are used as states and the size of these variables is
                  limited. Previously, the default was to use the absolute angle/distance
                  and absolute speed of every inertia/mass which has the disadvantage that the absolute
                  angle and or distance are state variables that grow in size continuously.<br>
                  A significant advantage is also, that default initialization is usually better,
                  because a default value of zero for a relative angle/distance is usually what the
                  user would like to have. Previously, say, the load was initialized to a non-zero
                  angle and then the elastically coupled motor inertia had to be explicitly
                  also initialized with this value. This is now, no longer needed. Since the default
                  nominal value of 1 is usually too large for a relative quantity, the nominal
                  values of the relative angle/distance was changed to 1e-4.</li>
         <li> The two libraries have been restructured in sublibraries to cope
                  with the growing number of components.</li>
         <li> Finally, the Translational library has been
                  made as similar as possible to the Rotational library by, e.g., adding missing
                  components.<br>&nbsp;</li>
         </ul></li>

<li> The initialization of the MultiBody, Rotational and Translational libraries have
         been signficantly simplified by removing the \"initType\" parameters and only
         using start/fixed values. This design assumes that a tool has special support
         for start/fixed values in the parameter menu.<br>&nbsp;</li>

<li> Nearly all parameters defined in the Modelica Standard Library had been
         defined with a default equation, e.g.,
         <pre>   <b>parameter</b> Modelica.SIunits.Resistance R=1; </pre>
         Physical parameters, such as a resistance, mass, gear ratio, do not have a meaningful
         default and in nearly all cases, the user of the corresponding component has to
         provide values for such parameters. If the user forgets this, a tool
         cannot provide diagnostics, since a default value is present in the library
         (such as 1 Ohm for the resistance). In most cases the model will simulate but will
         give wrong results due to wrong parameter values. To improve this situation, all physical
         parameter declarations in the Modelica Standard Library have been changed, so
         that the previous default becomes a start value. For example, the above
         declaration is changed to:
         <pre>   <b>parameter</b> Modelica.SIunits.Resistance R(start=1);  </pre>
         This is a backward compatible change and completely equivalent from the perspective
         of the Modelica language. It is, however, advised that tools will print a warning
         or optionally an error message, if the start value of a parameter is defined, but
         no value for the parameter is given via a modification. Furthermore, it is expected,
         that the input field of a parameter menu is empty, if no default equation is defined,
         but only a start value. This shows clearly to the modeler that a value has to
         be provided.</li>
</ul>

<p><br>
The following <b style=\"color:blue\">new components</b> have been added
to <b style=\"color:blue\">existing</b> libraries (note, the names in paranthesis
are the new sublibrary names that are introduced in version 3.0):
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Blocks.Examples.</b></td></tr>
<tr><td valign=\"top\">InverseModel</td>
          <td valign=\"top\"> Demonstrates the construction of an inverse model.</td> </tr>

<tr><td colspan=\"2\"><b>Blocks.Math.</b></td></tr>
<tr><td valign=\"top\">InverseBlockConstraints</td>
          <td valign=\"top\"> Construct inverse model by requiring that two inputs
                                                and two outputs are identical (replaces the previously,
                                                unbalanced, TwoInputs and TwoOutputs blocks).</td> </tr>

<tr><td colspan=\"2\"><b>Electrical.Machines.Utilities</b></td></tr>
<tr><td valign=\"top\">TransformerData</td>
          <td valign=\"top\"> A record that calculates required impedances (parameters) from nominal data of transformers.</td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Examples.Rotational3DEffects</b></td></tr>
<tr><td valign=\"top\"> GyroscopicEffects<br>
                                                ActuatedDrive<br>
                                                MovingActuatedDrive<br>
                                                GearConstraint </td>
          <td valign=\"top\"> New examples to demonstrate the usage of the Rotational library
                                                in combination with multi-body components. </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Sensors</b></td></tr>
<tr><td valign=\"top\"> AbsolutePosition<br>
                                                AbsoluteVelocity<br>
                                                AbsoluteAngles<br>
                                                AbsoluteAngularVelocity<br>
                                                RelativePosition<br>
                                                RelativeVelocity<br>
                                                RelativeAngles<br>
                                                RelativeAngularVelocity</td>
          <td valign=\"top\"> New sensors to measure one vector. </td>
</tr>
<tr><td valign=\"top\"> TransformAbsoluteVector<br>
                                                TransformRelativeVector</td>
          <td valign=\"top\"> Transform absolute and/or relative vector into another frame. </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.Rotational.(Components)</b></td></tr>
<tr><td valign=\"top\"> Disc </td>
          <td valign=\"top\"> Right flange is rotated by a fixed angle with respect to left flange</td> </tr>
<tr><td valign=\"top\"> IdealRollingWheel </td>
          <td valign=\"top\"> Simple 1-dim. model of an ideal rolling wheel without inertia</td> </tr>


<tr><td colspan=\"2\"><b>Mechanics.Translational.Sensors</b></td></tr>
<tr><td valign=\"top\">RelPositionSensor<br>RelSpeedSensor<br>RelAccSensor<br>PowerSensor</td>
          <td valign=\"top\"> Relative position sensor, i.e. distance between two flanges<br>
                                                Relative speed sensor<br>
                                                Relative acceleration sensor<br>
                                                Ideal power sensor</td> </tr>
<tr><td colspan=\"2\"><b>Mechanics.Translational(.Components)</b></td></tr>
<tr><td valign=\"top\">SupportFriction<br>Brake<br>InitializeFlange</td>
          <td valign=\"top\"> Model of friction due to support<br>
                                                Model of a brake, base on Coulomb friction<br>
                                                Initializes a flange with pre-defined postion, speed and acceleration .</td> </tr>
<tr><td colspan=\"2\"><b>Mechanics.Translational(.Sources)</b></td></tr>
<tr><td valign=\"top\">Force2<br>LinearSpeedDependentForce<br>QuadraticSpeedDependentForce<br>
                                           ConstantForce<br>ConstantSpeed<br>ForceStep</td>
          <td valign=\"top\"> Force acting on 2 flanges<br>
                                                Force linearly dependent on flange speed<br>
                                                Force quadratic dependent on flange speed<br>
                                                Constant force source<br>
                                                Constant speed source<br>
                                                Force step</td> </tr>
</tr>
</table>


<p><br>
The following <b style=\"color:blue\">existing components</b>
have been <b style=\"color:blue\">changed</b> in a
<b style=\"color:blue\">non-backward compatible</b> way
(the conversion script transforms models and libraries
of previous versions to the new version. Therefore, conversion
should be automatic):
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Blocks.Continuous.</b></td></tr>
<tr><td valign=\"top\"> CriticalDamping </td>
          <td valign=\"top\"> New parameter \"normalized\" to define whether filter is provided
                                                in normalized or non-normalized form. Default is \"normalized = true\".
                                                The previous implementation was a non-normalized filter.
                                                The conversion script automatically introduces the modifier
                                                \"normalized=false\" for existing models.</td> </tr>

<tr><td colspan=\"2\"><b>Blocks.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> RealInput<br>
                                                RealOutput</td>
          <td valign=\"top\"> Removed \"SignalType\", since extending from a replaceable class
                                                and this is not allowed in Modelica 3.<br>The conversion script
                                                removes modifiers to SignalType.</td> </tr>

<tr><td valign=\"top\"> RealSignal<br>
                                                IntegerSignal<br>
                                                BooleanSignal</td>
          <td valign=\"top\"> Moved to library ObsoleteModelica3, since these connectors
                                                are no longer allowed in Modelica 3<br>
                                                (prefixes input and/or output are required). </td> </tr>

<tr><td colspan=\"2\"><b>Blocks.Interfaces.Adaptors.</b></td></tr>
<tr><td valign=\"top\"> AdaptorReal<br>
                                                AdaptorBoolean<br>
                                                AdaptorInteger</td>
          <td valign=\"top\"> Moved to library ObsoleteModelica3, since the models are not \"balanced\".
                                                These are completely obsolete adaptors<br>between the Real, Boolean, Integer
                                                signal connectors of version 1.6 and version &ge; 2.1 of the Modelica
                                                Standard Library. </td> </tr>


<tr><td colspan=\"2\"><b>Blocks.Math.</b></td></tr>
<tr><td valign=\"top\"> ConvertAllUnits</td>
          <td valign=\"top\"> Moved to library ObsoleteModelica3, since extending from a replaceable class
                                                and this is not allowed in Modelica 3.<br> It would be possible to rewrite this
                                                model to use a replaceable component. However, the information about the
                                                conversion<br> cannot be visualized in the icon in this case.</td> </tr>

<tr><td colspan=\"2\"><b>Blocks.Math.UnitConversions.</b></td></tr>
<tr><td valign=\"top\"> TwoInputs<br>
                                                TwoOutputs</td>
          <td valign=\"top\"> Moved to library ObsoleteModelica3, since the models are not \"balanced\".
                                                A new component<br>\"InverseBlockConstraints\"
                                                is provided instead that has the same feature, but is \"balanced\".</td> </tr>

<tr><td colspan=\"2\"><b>Electrical.Analog.Baisc.</b></td></tr>
<tr><td valign=\"top\"> HeatingResistor</td>
          <td valign=\"top\"> The heatPort has to be connected; otherwise the component Resistor (without heatPort) has to be used.<br>
                                                cardinality() is only used to check whether the heatPort is connected.</td> </tr>

<tr><td colspan=\"2\"><b>Electrical.MultiPhase.Examples.</b></td></tr>
<tr><td valign=\"top\"> </td>
          <td valign=\"top\"> Changed the instance names of components used in the examples to more up-to-date style.</td> </tr>

<tr><td colspan=\"2\"><b>Electrical.Machines.</b></td></tr>
<tr><td valign=\"top\"> </td>
          <td valign=\"top\"> Moved package <tt>Machines.Examples.Utilities</tt> to <tt>Machines.Utilities</tt></td> </tr>
<tr><td valign=\"top\"> </td>
          <td valign=\"top\"> Removed all nonSIunits; especially in DCMachines<br>
                                                parameter NonSIunits.AngularVelocity_rpm rpmNominal was replaced by<br>
                                                parameter SIunits.AngularVelocity wNominal</td> </tr>
<tr><td valign=\"top\"> </td>
          <td valign=\"top\"> Changed the following component variable and parameter names to be more concise:<br>
                                                Removed suffix \"DamperCage\" from all synchronous induction machines
                                                since the user can choose whether the damper cage is present or not.<br><tt>
                                                RotorAngle ... RotorDisplacementAngle<br>
                                                J_Rotor ... Jr<br>
                                                Rr ........ Rrd (damper of synchronous induction machines)<br>
                                                Lrsigma ... Lrsigmad (damper of synchronous induction machines)<br>
                                                phi_mechanical ... phiMechanical<br>
                                                w_mechanical ..... wMechanical<br>
                                                rpm_mechanical ... rpmMechanical<br>
                                                tau_electrical ... tauElectrical<br>
                                                tau_shaft ........ tauShaft<br>
                                                TurnsRatio ....... turnsRatio    (AIMS)<br>
                                                VsNom ............ VsNominal     (AIMS)<br>
                                                Vr_Lr ............ VrLockedRotor (AIMS)<br>
                                                DamperCage ....... useDamperCage (synchronous induction machines)<br>
                                                V0 ............... VsOpenCicuit  (SMPM)<br>
                                                Ie0 .............. IeOpenCicuit  (SMEE)
                                                </tt></td> </tr>
<tr><td valign=\"top\">Interfaces.</td>
          <td valign=\"top\"> Moved as much code as possible from specific machine models to partials to reduce redundant code.</td> </tr>
<tr><td valign=\"top\">Interfaces.Adapter</td>
          <td valign=\"top\"> Removed to avoid cardinality; instead, the following solution has been implemented:</td> </tr>
<tr><td valign=\"top\">Sensors.RotorDisplacementAngle<br>Interfaces.PartialBasicMachine</td>
          <td valign=\"top\"> Introduced <tt>parameter Boolean useSupport=false \"enable / disable (=fixed stator) support\"</tt><br>
                                                The rotational support connector is only present with <tt>useSupport = true;</tt><br>
                                                otherwise the stator is fixed internally.</td> </tr>

<tr><td colspan=\"2\"><b>Electrical.Machines.Examples.</b></td></tr>
<tr><td valign=\"top\"> </td>
          <td valign=\"top\"> Changed the names of the examples to more meaninglful names.<br>
                                                Changed the instance names of components used in the examples to more up-to-date style.</td> </tr>
<tr><td valign=\"top\">SMEE_Generator</td>
          <td valign=\"top\"> Initialization of <tt>smee.phiMechanical</tt> with <tt>fixed=true</tt></td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.</b></td></tr>
<tr><td valign=\"top\"> World</td>
          <td valign=\"top\"> Changed default value of parameter driveTrainMechanics3D from false to true.<br>
                                                3-dim. effects in Rotor1D, Mounting1D and BevelGear1D are therefore taken<br>
                                                into account by default (previously this was only the case, if
                                                world.driveTrainMechanics3D was explicitly set).</td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Forces.</b></td></tr>
<tr><td valign=\"top\"> FrameForce<br>
                                                FrameTorque<br>
                                                FrameForceAndTorque</td>
          <td valign=\"top\"> Models removed, since functionality now available via Force, Torque, ForceAndTorque</td> </tr>
<tr><td valign=\"top\"> WorldForce<br>
                                                WorldTorque<br>
                                                WorldForceAndTorque<br>
                                                Force<br>
                                                Torque<br>
                                                ForceAndTorque</td>
          <td valign=\"top\"> Connector frame_resolve is optionally enabled via parameter resolveInFrame<br>.
                                                Forces and torques and be resolved in all meaningful frames defined
                                                by enumeration resolveInFrame.</td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Frames.</b></td></tr>
<tr><td valign=\"top\"> length<br>
                                                normalize</td>
          <td valign=\"top\"> Removed functions, since available also in Modelica.Math.Vectors
                                                <br>The conversion script changes the references correspondingly.</td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Joints.</b></td></tr>
<tr><td valign=\"top\"> Prismatic<br>
                                                ActuatedPrismatic<br>
                                                Revolute<br>
                                                ActuatedRevolute<br>
                                                Cylindrical<br>
                                                Universal<br>
                                                Planar<br>
                                                Spherical<br>
                                                FreeMotion</td>
          <td valign=\"top\"> Changed initialization, by replacing initial value parameters with
                                                start/fixed attributes.<br>
                                                When start/fixed attributes are properly supported
                                                in the parameter menu by a Modelica tool,<br>
                                                the initialization is considerably simplified for the
                                                user and the implementation is much simpler.<br>
                                                Replaced parameter \"enforceStates\" by the more general
                                                built-in enumeration stateSelect=StateSelection.xxx.<br>
                                                The conversion script automatically
                                                transforms from the \"old\" to the \"new\" forms.</td> </tr>
<tr><td valign=\"top\"> Revolute<br>
                                                ActuatedRevolute</td>
          <td valign=\"top\"> Parameter \"planarCutJoint\" in the \"Advanced\" menu of \"Revolute\" and of
                                                \"ActuatedRevolute\" removed.<br>
                                                A new joint \"RevolutePlanarLoopConstraint\" introduced that defines the constraints
                                                of a revolute joint<br> as cut-joint in a planar loop.
                                                This change was needed in order that the revolute joint can be
                                                properly used<br>in advanced model checking.<br>
                                                ActuatedRevolute joint removed. Flange connectors of Revolute joint<br>
                                                can be enabled with parameter useAxisFlange.</td> </tr>
<tr><td valign=\"top\"> Prismatic<br>
                                                ActuatedPrismatic</td>
          <td valign=\"top\"> ActuatedPrismatic joint removed. Flange connectors of Prismatic joint<br>
                                                can be enabled with parameter useAxisFlange.</td> </tr>
<tr><td valign=\"top\"> Assemblies</td>
          <td valign=\"top\"> Assembly joint implementation slightly changed, so that
                                                annotation \"structurallyIncomplete\" <br>could be removed
                                                (all Assembly joint models are now \"balanced\").</td> </tr>


<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Joints.Internal</b></td></tr>
<tr><td valign=\"top\"> RevoluteWithLengthConstraint<br>
                                                PrismaticWithLengthConstraint</td>
          <td valign=\"top\"> These joints should not be used by a user of the MultiBody library.
                                                They are only provided to built-up the
                                                MultiBody.Joints.Assemblies.JointXYZ joints.
                                                These two joints have been changed in a slightly not backward compatible
                                                way, in order that the usage in the Assemblies.JointXYZ joints results in
                                                balanced models (<b>no conversion is provided for this change since the
                                                user should not have used these joints and the conversion would be too
                                                complicated</b>):
                                                In releases before version 3.0 of the Modelica Standard Library,
                                                it was possible to activate the torque/force projection equation
                                                (= cut-torque/-force projected to the rotation/translation
                                                axis must be identical to
                                                the drive torque/force of flange axis) via parameter <b>axisTorqueBalance</b>.
                                                This is no longer possible, since otherwise this model would not be
                                                \"balanced\" (= same number of unknowns as equations). Instead, when
                                                using this model in version 3.0 and later versions, the torque/force
                                                projection equation must be provided in the Advanced menu of joints
                                                Joints.SphericalSpherical and Joints.UniversalSpherical
                                                via the new parameter \"constraintResidue\".</td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Parts.</b></td></tr>
<tr><td valign=\"top\"> BodyBox<br>
                                                BodyCylinder</td>
          <td valign=\"top\"> Changed unit of parameter density from g/cm3 to the SI unit kg/m3
                                                in order to allow stricter unit checking.<br>The conversion script multiplies
                                                previous density values with 1000.</td> </tr>
<tr><td valign=\"top\"> Body<br>
                                                BodyShape<br>
                                                BodyBox<br>
                                                BodyCylinder<br>
                                                PointMass
                                                Rotor1D</td>
          <td valign=\"top\"> Changed initialization, by replacing initial value parameters with
                                                start/fixed attributes.<br>
                                                When start/fixed attributes are properly supported
                                                in the parameter menu by a Modelica tool,<br>
                                                the initialization is considerably simplified for the
                                                user and the implementation is much simpler.<br>The conversion script automatically
                                                transforms from the \"old\" to the \"new\" form of initialization.</td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Sensors.</b></td></tr>
<tr><td valign=\"top\"> AbsoluteSensor<br>
                                                RelativeSensor<br>
                                                CutForceAndTorque</td>
          <td valign=\"top\"> New design of sensor components: Via Boolean parameters<br>
                                                signal connectors for the respective vectors are enabled/disabled.<br>
                                                It is not possible to automatically convert models to this new design.<br>
                                                Instead, references in existing models are changed to ObsoleteModelice3.<br>
                                                This means that these models must be manually adapted.</td> </tr>
<tr><td valign=\"top\"> CutForce<br>
                                                CutTorque</td>
          <td valign=\"top\"> Slightly new design. The force and/or torque component can be
                                                resolved in world, frame_a, or frame_resolved.<br>
                                                Existing models are automatically converted.</td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\"> </td>
          <td valign=\"top\"> Moved components to structured sub-packages (Sources, Components)</td> </tr>
<tr><td valign=\"top\"> Inertia<br>
                                                SpringDamper<br>
                                                RelativeStates</td>
          <td valign=\"top\"> Changed initialization, by replacing initial value parameters with
                                                start/fixed attributes.<br>
                                                When start/fixed attributes are properly supported
                                                in the parameter menu by a Modelica tool,<br>
                                                the initialization is considerably simplified for the
                                                user and the implementation is much simpler.<br>
                                                Parameter \"stateSelection\" in \"Inertia\" and \"SpringDamper\" replaced
                                                by the built-in enumeration<br>stateSelect=StateSelection.xxx.
                                                Introduced the \"stateSelect\" enumeration in \"RelativeStates\".<br>
                                                The conversion script automatically
                                                transforms from the \"old\" to the \"new\" forms.</td> </tr>
<tr><td valign=\"top\"> LossyGear<br>
                                                GearBox</td>
          <td valign=\"top\"> Renamed gear ratio parameter \"i\" to \"ratio\", in order to have a
                                                consistent naming convention.<br>
                                                Existing models are automatically converted. </td> </tr>
<tr><td valign=\"top\"> SpringDamper<br>
                                                ElastoBacklash<br>
                                                Clutch<br>
                                                OneWayClutch</td>
          <td valign=\"top\"> Relative quantities (phi_rel, w_rel) are used as states, if possible
                                                (due to StateSelect.prefer). <br>
                                                In most cases, relative states in drive trains are better suited as
                                                absolute states. <br> This change might give changes in the selected states
                                                of existing models.<br>
                                                This might give rise to problems if, e.g., the initialization was not
                                                completely defined in a user model,<br> since the default
                                                initialization heuristic may give different initial values.</td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.Translational.</b></td></tr>
<tr><td valign=\"top\"> </td>
          <td valign=\"top\"> Moved components to structured sub-packages (Sources, Components)</td> </tr>
<tr><td valign=\"top\"> </td>
          <td valign=\"top\"> Adaptions corresponding to Rotational</td> </tr>
<tr><td valign=\"top\"> Stop</td>
          <td valign=\"top\"> Renamed to Components.MassWithStopAndFriction to be more concise.<br>
                                                MassWithStopAndFriction is not available with a support connector, <br>
                                                since the reaction force can't be modeled in a meaningful way due to reinit of velocity v.<br>
                                                Until a sound implementation of a hard stop is available, the old model may be used.</td> </tr>
<tr><td colspan=\"" + "2\"><b>Media.</b></td></tr>
<tr><td valign=\"top\"> constant nX <br>
                                                constant nXi <br>
                                                constant reference_X<br>
                                                BaseProperties</td>
          <td valign=\"top\"> The package constant nX = nS, now always, even for single species media. This also allows to define mixtures with only 1 element. The package constant nXi=if fixedX then 0 else if reducedX or nS==1 then nS - 1 else nS. This required that all BaseProperties for single species media get an additional equation to define the composition X as {1.0} (or reference_X, which is {1.0} for single species). This will also mean that all user defined single species media need to be updated by that equation.</td> </tr>

<tr><td colspan=\"2\"><b>SIunits.</b></td></tr>
<tr><td valign=\"top\"> CelsiusTemperature </td>
          <td valign=\"top\"> Removed, since no SI unit. The conversion script changes references to
                                                SIunits.Conversions.NonSIunits.Temperature_degC </td> </tr>
<tr><td valign=\"top\"> ThermodynamicTemperature <br>
                                                TemperatureDifference</td>
          <td valign=\"top\"> Added annotation \"__Dymola_absoluteValue=true/false\"
                                                in order that unit checking is possible<br>
                                                (the unit checker needs to know for a unit that has an offset,
                                                whether it is used as absolute or as a relative number)</td> </tr>

<tr><td colspan=\"2\"><b>SIunits.Conversions.NonSIunits.</b></td></tr>
<tr><td valign=\"top\"> Temperature_degC<br>
                                                Temperature_degF<br>
                                                Temperature_degRk </td>
          <td valign=\"top\"> Added annotation \"__Dymola_absoluteValue=true\"
                                                in order that unit checking is possible<br>
                                                (the unit checker needs to know for a unit that has an offset,
                                                whether it is used as absolute or as a relative number)</td> </tr>

<tr><td colspan=\"2\"><b>StateGraph.Examples.</b></td></tr>
<tr><td valign=\"top\"> ControlledTanks </td>
          <td valign=\"top\"> The connectors of the ControlledTanks did not fulfill the new
                                                restrictions of Modelica 3. This has been fixed.</td> </tr>
<tr><td valign=\"top\"> Utilities </td>
          <td valign=\"top\"> Replacing inflow, outflow by connectors inflow1, inflow2,
                                                outflow1, outflow2 with appropriate input/output prefixes in
                                                order to fufill the restrictions of Modelica 3 to arrive
                                                at balanced models. No conversion is provided, since
                                                too difficult and since the non-backward compatible change is in
                                                an example. </td> </tr>

<tr><td colspan=\"2\"><b>Thermal.FluidHeatFlow.Sensors.</b></td></tr>
<tr><td valign=\"top\"> <br>
                                                pSensor<br>TSensor<br>dpSensor<br>dTSensor<br>m_flowSensor<br>V_flowSensor<br>H_flowSensor</td>
          <td valign=\"top\"> renamed to:<br>
                                                PressureSensor<br>TemperatureSensor<br>RelPressureSensor<br>RelTemperatureSensor<br>MassFlowSensor<br>VolumeFlowSensor<br>EnthalpyFlowSensor
                                                </td> </tr>

<tr><td colspan=\"2\"><b>Thermal.FluidHeatFlow.Sources.</b></td></tr>
<tr><td valign=\"top\"> Ambient<br>PrescribedAmbient</td>
          <td valign=\"top\"> available as one combined component Ambient<br>
                                                Boolean parameters usePressureInput and useTemperatureInput decide
                                                whether pressure and/or temperature are constant or prescribed</td> </tr>
<tr><td valign=\"top\"> ConstantVolumeFlow<br>PrescribedVolumeFlow</td>
          <td valign=\"top\"> available as one combined component VolumeFlow<br>
                                                Boolean parameter useVolumeFlowInput decides
                                                whether volume flow is constant or prescribed</td> </tr>
<tr><td valign=\"top\"> ConstantPressureIncrease<br>PrescribedPressureIncrease</td>
          <td valign=\"top\"> available as one combined component PressureIncrease<br>
                                                Boolean parameter usePressureIncreaseInput decides
                                                whether pressure increase is constant or prescribed</td> </tr>

<tr><td colspan=\"2\"><b>Thermal.FluidHeatFlow.Examples.</b></td></tr>
<tr><td valign=\"top\"> </td>
          <td valign=\"top\"> Changed the instance names of components used in the examples to more up-to-date style.</td> </tr>

<tr><td colspan=\"2\"><b>Thermal.HeatTransfer.(Components)</b></td></tr>
<tr><td valign=\"top\"> HeatCapacitor</td>
          <td valign=\"top\"> Initialization changed: SteadyStateStart removed. Instead
                                                start/fixed values for T and der_T<br>(initial temperature and its derivative).</td> </tr>

<tr><td valign=\"top\"> <br><br>HeatCapacitor<br>ThermalConductor<br>ThermalConvection<br>BodyRadiation<br><br>
                                                TemperatureSensor<br>RelTemperatureSensor<br>HeatFlowSensor<br><br>
                                                FixedTemperature<br>PrescribedTemperature<br>FixedHeatFlow<br>PrescribedHeatFlow</td>
          <td valign=\"top\"> Moved components to sub-packages:<br><br>
                                                Components.HeatCapacitor<br>Components.ThermalConductor<br>Components.ThermalConvection<br>Components.BodyRadiation<br><br>
                                                Sensors.TemperatureSensor<br>Sensors.RelTemperatureSensor<br>Sensors.HeatFlowSensor<br><br>
                                                Sources.FixedTemperature<br>Sources.PrescribedTemperature<br>Sources.FixedHeatFlow<br>Sources.PrescribedHeatFlow
                                                </tt></td> </tr>

<tr><td colspan=\"2\"><b>Thermal.FluidHeatFlow.Examples.</b></td></tr>
<tr><td valign=\"top\"> </td>
          <td valign=\"top\"> Changed the instance names of components used in the examples to more up-to-date style.</td> </tr>
</table>



<p><br>
The following <b style=\"color:blue\">existing components</b>
have been <b style=\"color:blue\">improved</b> in a
<b style=\"color:blue\">backward compatible</b> way:
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td valign=\"top\"> <b>Modelica.*</b> </td>
          <td valign=\"top\"> Parameter declarations, input and output function arguments without description
                                                strings improved<br> by providing meaningful description texts.
                                                </td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Blocks.Continuous.</b></td></tr>
<tr><td valign=\"top\"> TransferFunction </td>
          <td valign=\"top\"> Internal scaling of the controller canonical states introduced
                                                in order to enlarge the range of transfer functions where the default
                                                relative tolerance of the simulator is sufficient.</td>
</tr>

<tr><td valign=\"top\"> Butterworth<br>CriticalDamping </td>
          <td valign=\"top\"> Documentation improved and plots of the filter characteristics added.</td> </tr>

<tr><td colspan=\"2\"><b>Electrical.Analog.Basic.</b></td></tr>
<tr><td valign=\"top\"> EMF </td>
          <td valign=\"top\"> New parameter \"useSupport\" to optionally enable a support connector.</td> </tr>


<tr><td colspan=\"2\"><b>Icons.</b></td></tr>
<tr><td valign=\"top\"> TranslationalSensor<br>
                                                RotationalSensor</td>
          <td valign=\"top\"> Removed drawing from the diagram layer (kept drawing only in
                                                icon layer),<br> in order that this icon can be used in situations
                                                where components are dragged in the diagram layer.</td> </tr>

<tr><td colspan=\"2\"><b>Math.Vectors.</b></td></tr>
<tr><td valign=\"top\"> normalize</td>
          <td valign=\"top\"> Implementation changed, so that the result is awalys continuous<br>
                                                (previously, this was not the case for small vectors: normalize(eps,eps)).
                                                </td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.</b></td></tr>
<tr><td valign=\"top\"> </td>
          <td valign=\"top\"> Renamed non-standard keywords defineBranch, defineRoot, definePotentialRoot,
                                                isRooted to the standard names:<br>
                                                Connections.branch/.root/.potentialRoot/.isRooted.</td> </tr>
<tr><td valign=\"top\"> Frames </td>
          <td valign=\"top\"> Added annotation \"Inline=true\" to all one-line functions
                                                (which should be all inlined).</td> </tr>
<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Parts.</b></td></tr>
<tr><td valign=\"top\"> Mounting1D<br>
                                                Rotor1D<br>
                                                BevelGear1D</td>
          <td valign=\"top\"> Changed implementation so that no longer modifiers for connector
                                                variables are used,<br>because this violates the restrictions on
                                                \"balanced models\" of Modelica 3. </td> </tr>

<tr><td colspan=\"2\"><b>Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\"> InitializeFlange</td>
          <td valign=\"top\"> Changed implementation so that counting unknowns and
                                                equations is possible without actual values of parameters.</td> </tr>

<tr><td colspan=\"2\"><b>Thermal.FluidHeatFlow.Interfaces.Partials.</b></td></tr>
<tr><td valign=\"top\">TwoPort</td>
          <td valign=\"top\"> Introduced <tt>parameter Real tapT(final min=0, final max=1)=1</tt> <br> that defines the temperature of the heatPort
                                                betwen inlet and outlet.</td> </tr>

<tr><td colspan=\"2\"><b>StateGraph.</b></td></tr>
<tr><td valign=\"top\"> InitialStep<br>
                                                InitialStepWithSignal<br>
                                                Step<br>
                                                StepWithSignal</td>
          <td valign=\"top\"> Changed implementation so that no longer modifiers for output
                                                variables are used,<br>because this violates the restrictions on
                                                \"balanced models\" of Modelica 3. </td> </tr>

</table>


<p><br>
The following <b style=\"color:red\">critical errors</b> have been fixed (i.e. errors
that can lead to wrong simulation results):
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Electrical.Analog.Examples.</b></td></tr>
<tr><td valign=\"top\"> CauerLowPassSC </td>
          <td valign=\"top\"> Wrong calculation of Capacitor1 both in Rn and Rp corrected
                                                (C=clock/R instead of C=clock*R) </td>
</tr>


<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Parts.</b></td></tr>
<tr><td valign=\"top\"> Rotor1D </td>
          <td valign=\"top\"> The 3D reaction torque was not completely correct and gave in
                                                some situations a wrong result. This bug should not influence the
                                                movement of a multi-body system, but only the constraint torques
                                                are sometimes not correct. </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\"> ElastoBacklash </td>
          <td valign=\"top\"> If the damping torque was too large, the reaction torque
                                                could \"pull\" which is unphysical. The component was
                                                newly written by limiting the damping torque in such a case
                                                so that \"pulling\" torques can no longer occur. Furthermore,
                                                during initialization the characteristics is made continuous
                                                to reduce numerical errors. The relative angle and relative
                                                angular velocities are used as states, if possible
                                                (StateSelect.prefer), since relative quantities lead usually
                                                to better behavior.  </td>
</tr>
<tr><td valign=\"top\"> Position<br>Speed<br>Accelerate<br>Move</td>
          <td valign=\"top\"> The movement of the flange was wrongly defined as absolute;
                                                this is corrected as relative to connector support.<br>
                                                For Accelerate, it was necessary to rename
                                                RealInput a to a_ref, as well as the start values
                                                phi_start to phi.start and w_start to w.start.
                                                The conversion script performs the necessary conversion of
                                                existing models automatically.</td>
</tr>
<tr><td colspan=\"2\"><b>Media.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> PartialSimpleIdealGasMedium </td>
          <td valign=\"top\"> Inconsisteny in reference temperature corrected. This may give
                                                different results for functions:<br>
                                                specificEnthalpy, specificInternalEnergy, specificGibbsEnergy,
                                                specificHelmholtzEnergy. </td>
</tr>
<tr><td colspan=\"2\"><b>Media.Air.</b></td></tr>
<tr><td valign=\"top\"> specificEntropy </td>
          <td valign=\"top\"> Small bug in entropy computation of ideal gas mixtures corrected.</td>
</tr>
<tr><td colspan=\"2\"><b>Media.IdealGases.Common.MixtureGasNasa</b></td></tr>
<tr><td valign=\"top\"> specificEntropy </td>
          <td valign=\"top\"> Small bug in entropy computation of ideal gas mixtures corrected.</td>
</tr>
</table>


<p><br>
The following <b style=\"color:red\">uncritical errors</b> have been fixed (i.e. errors
that do <b style=\"color:red\">not</b> lead to wrong simulation results, but, e.g.,
units are wrong or errors in documentation):
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Blocks.Tables.</b></td></tr>
<tr><td valign=\"top\"> CombiTable2D</td>
          <td valign=\"top\"> Documentation improved.</td>
</tr>

<tr><td colspan=\"2\"><b>Electrica.Digital.Gates</b></td></tr>
<tr><td valign=\"top\"> AndGate<br>
                                                NandGate<br>
                                                OrGate<br>
                                                NorGate<br>
                                                XorGate<br>
                                                XnorGate</td>
          <td valign=\"top\"> The number of inputs was not correctly propagated
                                                to the included base model.<br>
                                                This gave a translation error, if the number
                                                of inputs was changed (and not the default used).</td>
</tr>

<tr><td colspan=\"2\"><b>Electrica.Digital.Sources</b></td></tr>
<tr><td valign=\"top\"> Pulse </td>
          <td valign=\"top\"> Model differently implemented, so that
                                                warning message about \"cannot properly initialize\" is gone.</td>
</tr>


<tr><td colspan=\"2\"><b>Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\"> BearingFriction<br>
                                                Clutch<br>
                                                OneWayClutch<br>
                                                Brake <br>
                                                Gear </td>
          <td valign=\"top\"> Declaration of table parameter changed from
                                                table[:,:] to table[:,2].</td>
</tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.Examples.Loops.Utilities.</b></td></tr>
<tr><td valign=\"top\"> GasForce </td>
          <td valign=\"top\"> Unit of variable \"press\" corrected (from Pa to bar)</td>
</tr>

<tr><td colspan=\"2\"><b>StateGraph.Examples.</b></td></tr>
<tr><td valign=\"top\">SimpleFriction</td>
          <td valign=\"top\"> The internal parameter k is defined and calculated with the appropriate unit.</td> </tr>


<tr><td colspan=\"2\"><b>Thermal.FluidHeatFlow.Interfaces.Partials.</b></td></tr>
<tr><td valign=\"top\">SimpleFriction</td>
          <td valign=\"top\"> The internal parameter k is defined and calculated with the appropriate unit.</td> </tr>

</table>

</html>"));
      end Version_3_0;
      class Version_2_2_2 "Version 2.2.2 (Aug. 31, 2007)"
        annotation(Documentation(info = "<html>
<p>
Version 2.2.2 is backward compatible to version 2.2.1 and 2.2 with
the following exceptions:
</p>
<ul>
<li> Removal of package Modelica.Media.Interfaces.PartialTwoPhaseMediumWithCache
         (this was not yet utilized).</li>
<li> Removal of the media packages in
         Modelica.Media.IdealGases.SingleGases that are not type compatible
         to Modelica.Media.Interfaces.PartialMedium, because a FluidConstants
         record definition is missing,
         for details, see
          <a href=\"Modelica://Modelica.Media.IdealGases\">Modelica.Media.IdealGases</a>
         (this is seen as a bug fix).
</ul>

<p>
An overview of the differences between version 2.2.2 and the previous
version 2.2.1 is given below. The exact differences (but without
differences in the documentation) are available
<a href=\"Documentation/Differences-Modelica-221-222.html\">here</a>.
This comparison file was generated automatically with Dymolas
ModelManagement.compare function.
</p>

<p>
In this version, <b>no</b> new libraries have been added. The <b>documentation</b>
of the whole library was improved. Especially, the documentation is now also
available as
<a href=\"Documentation/ModelicaStandardLibrary.pdf\">one pdf file</a>.
</p>


<p><br>
The following <b style=\"color:blue\">new components</b> have been added
to <b style=\"color:blue\">existing</b> libraries:
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Blocks.Logical.</b></td></tr>
<tr><td valign=\"top\"> TerminateSimulation</td>
          <td valign=\"top\"> Terminate a simulation by a given condition.</td>
</tr>

<tr><td colspan=\"2\"><b>Blocks.Routing.</b></td></tr>
<tr><td valign=\"top\"> RealPassThrough<br>
                   IntegerPassThrough<br>
                   BooleanPassThrough</td>
          <td valign=\"top\"> Pass a signal from input to output
                   (useful in combination with a bus due to restrictions
                   of expandable connectors).</td>
</tr>

<tr><td colspan=\"2\"><b>Blocks.Sources.</b></td></tr>
<tr><td valign=\"top\"> KinematicPTP2 </td>
          <td valign=\"top\"> Directly gives q,qd,qdd as output (and not just qdd as KinematicPTP).
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Machines.Examples.</b></td></tr>
<tr><td valign=\"top\"> TransformerTestbench </td>
          <td valign=\"top\"> Transformer Testbench
          </td>
<tr><td valign=\"top\"> Rectifier6pulse </td>
          <td valign=\"top\"> 6-pulse rectifier with 1 transformer
          </td>
</tr>
<tr><td valign=\"top\"> Rectifier12pulse </td>
          <td valign=\"top\"> 12-pulse rectifier with 2 transformers
          </td>
</tr>
<tr><td valign=\"top\"> AIMC_Steinmetz </td>
          <td valign=\"top\"> Asynchronous induction machine squirrel cage with Steinmetz connection
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Machines.BasicMachines.Components.</b></td></tr>
<tr><td valign=\"top\"> BasicAIM </td>
          <td valign=\"top\"> Partial model for asynchronous induction machine
          </td>
<tr><td valign=\"top\"> BasicSM </td>
          <td valign=\"top\"> Partial model for synchronous induction machine
          </td>
<tr><td valign=\"top\"> PartialAirGap </td>
          <td valign=\"top\"> Partial airgap model
          </td>
<tr><td valign=\"top\"> BasicDCMachine </td>
          <td valign=\"top\"> Partial model for DC machine
          </td>
<tr><td valign=\"top\"> PartialAirGapDC </td>
          <td valign=\"top\"> Partial airgap model of a DC machine
          </td>
<tr><td valign=\"top\"> BasicTransformer </td>
          <td valign=\"top\"> Partial model of threephase transformer
          </td>
<tr><td valign=\"top\"> PartialCore </td>
          <td valign=\"top\"> Partial model of transformer core with 3 windings
          </td>
<tr><td valign=\"top\"> IdealCore </td>
          <td valign=\"top\"> Ideal transformer with 3 windings
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Machines.BasicMachines.</b></td></tr>
<tr><td valign=\"top\"> Transformers </td>
          <td valign=\"top\"> Sub-Library for technical 3phase transformers
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Machines.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> Adapter </td>
          <td valign=\"top\"> Adapter to model housing of electrical machine
          </td>
</tr>

<tr><td colspan=\"2\"><b>Math.</b></td></tr>
<tr><td valign=\"top\"> Vectors </td>
          <td valign=\"top\"> New library of functions operating on vectors
          </td>
</tr>
<tr><td valign=\"top\"> atan3 </td>
          <td valign=\"top\"> Four quadrant inverse tangens (select solution that is closest to given angle y0)
          </td>
</tr>
<tr><td valign=\"top\"> asinh </td>
          <td valign=\"top\"> Inverse of sinh (area hyperbolic sine)
          </td>
</tr>
<tr><td valign=\"top\"> acosh </td>
          <td valign=\"top\"> Inverse of cosh (area hyperbolic cosine)
          </td>
</tr>

<tr><td colspan=\"2\"><b>Math.Vectors</b></td></tr>
<tr><td valign=\"top\"> isEqual </td>
          <td valign=\"top\"> Determine if two Real vectors are numerically identical
          </td>
</tr>
<tr><td valign=\"top\"> norm </td>
          <td valign=\"top\"> Return the p-norm of a vector
          </td></tr>
<tr><td valign=\"top\"> length </td>
          <td valign=\"top\"> Return length of a vector (better as norm(), if further symbolic processing is performed)
          </td></tr>
<tr><td valign=\"top\"> normalize </td>
          <td valign=\"top\"> Return normalized vector such that length = 1 and prevent zero-division for zero vector
          </td></tr>
<tr><td valign=\"top\"> reverse </td>
          <td valign=\"top\"> Reverse vector elements (e.g. v[1] becomes last element)
          </td></tr>
<tr><td valign=\"top\"> sort </td>
          <td valign=\"top\"> Sort elements of vector in ascending or descending order
          </td></tr>

<tr><td colspan=\"2\"><b>Math.Matrices</b></td></tr>
<tr><td valign=\"top\"> solve2 </td>
          <td valign=\"top\"> Solve real system of linear equations A*X=B with a B matrix
                   (Gaussian elemination with partial pivoting)
          </td>
</tr>
<tr><td valign=\"top\"> LU_solve2 </td>
          <td valign=\"top\"> Solve real system of linear equations P*L*U*X=B with a B matrix
                   and an LU decomposition (from LU(..))
          </td></tr>


<tr><td colspan=\"2\"><b>Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\"> InitializeFlange </td>
          <td valign=\"top\"> Initialize a flange according to given signals
                   (useful if initialization signals are provided by a signal bus).
          </td>
</tr>


<tr><td colspan=\"2\"><b>Media.Interfaces.PartialMedium.</b></td></tr>
<tr><td valign=\"top\"> density_pTX </td>
          <td valign=\"top\"> Return density from p, T, and X or Xi
          </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Interfaces.PartialTwoPhaseMedium.</b></td></tr>
<tr><td valign=\"top\"> BaseProperties </td>
          <td valign=\"top\"> Base properties (p, d, T, h, u, R, MM, x) of a two phase medium
          </td>
</tr>
<tr><td valign=\"top\"> molarMass </td>
          <td valign=\"top\"> Return the molar mass of the medium
          </td>
</tr>
<tr><td valign=\"top\"> saturationPressure_sat </td>
          <td valign=\"top\"> Return saturation pressure
          </td>
</tr>
<tr><td valign=\"top\"> saturationTemperature_sat </td>
          <td valign=\"top\"> Return saturation temperature
          </td>
</tr>
<tr><td valign=\"top\"> saturationTemperature_derp_sat </td>
          <td valign=\"top\"> Return derivative of saturation temperature w.r.t. pressure
          </td>
</tr>  <tr><td valign=\"top\"> setState_px </td>
          <td valign=\"top\"> Return thermodynamic state from pressure and vapour quality
          </td>
</tr>  <tr><td valign=\"top\"> setState_Tx </td>
          <td valign=\"top\"> Return thermodynamic state from temperature and vapour quality
          </td>
</tr>  <tr><td valign=\"top\"> vapourQuality </td>
          <td valign=\"top\"> Return vapour quality
          </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> PartialLinearFluid </td>
          <td valign=\"top\"> Generic pure liquid model with constant cp,
                   compressibility and thermal expansion coefficients
          </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Air.MoistAir.</b></td></tr>
<tr><td valign=\"top\"> massFraction_pTphi </td>
          <td valign=\"top\"> Return the steam mass fraction from relative humidity and T
          </td>
</tr>
<tr><td valign=\"top\"> saturationTemperature </td>
          <td valign=\"top\"> Return saturation temperature from (partial) pressure
                   via numerical inversion of function saturationPressure
          </td>
</tr>
<tr><td valign=\"top\"> enthalpyOfWater </td>
          <td valign=\"top\"> Return specific enthalpy of water (solid/liquid) near
                   atmospheric pressure from temperature
          </td>
</tr>
<tr><td valign=\"top\"> enthalpyOfWater_der </td>
          <td valign=\"top\"> Return derivative of enthalpyOfWater()\" function
          </td>
</tr>
<tr><td valign=\"top\"> PsychrometricData </td>
          <td valign=\"top\"> Model to generate plot data for psychrometric chart
          </td>
</tr>

<tr><td colspan=\"2\"><b>Media.CompressibleLiquids.</b><br>
          New sub-library for simple compressible liquid models</td></tr>
<tr><td valign=\"top\"> LinearColdWater </td>
          <td valign=\"top\"> Cold water model with linear compressibility
          </td>
<tr><td valign=\"top\"> LinerWater_pT_Ambient </td>
          <td valign=\"top\"> Liquid, linear compressibility water model at 1.01325 bar
                   and 25 degree Celsius
          </td>

<tr><td colspan=\"2\"><b>SIunits.</b></td></tr>
<tr><td valign=\"top\"> TemperatureDifference </td>
          <td valign=\"top\"> Type for temperature difference
          </td>
</table>


<p><br>
The following <b style=\"color:blue\">existing components</b>
have been <b style=\"color:blue\">improved</b>:
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Blocks.Examples.</b></td></tr>
<tr><td valign=\"top\"> BusUsage</td>
          <td valign=\"top\"> Example changed from the \"old\" to the \"new\" bus concept with
                   expandable connectors.</td> </tr>

<tr><td colspan=\"2\"><b>Blocks.Discrete.</b></td></tr>
<tr><td valign=\"top\"> ZeroOrderHold</td>
          <td valign=\"top\"> Sample output ySample moved from \"protected\" to \"public\"
                   section with new attributes (start=0, fixed=true).
          </td>
</tr>
<tr><td valign=\"top\"> TransferFunction</td>
          <td valign=\"top\"> Discrete state x with new attributes (each start=0, each fixed=0).
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.</b></td></tr>
<tr><td valign=\"top\"> Analog<br>MultiPhase</td>
          <td valign=\"top\"> Improved some icons.
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Digital.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> MISO</td>
          <td valign=\"top\"> Removed \"algorithm\" from this partial block.
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Digital.Delay.</b></td></tr>
<tr><td valign=\"top\"> DelayParams</td>
          <td valign=\"top\"> Removed \"algorithm\" from this partial block.
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Digital.Delay.</b></td></tr>
<tr><td valign=\"top\"> DelayParams</td>
          <td valign=\"top\"> Removed \"algorithm\" from this partial block.
          </td>
</tr>
<tr><td valign=\"top\"> TransportDelay</td>
          <td valign=\"top\">  If delay time is zero, an infinitely small delay is
                        introduced via pre(x) (previously \"x\" was used).
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Digital.Sources.</b></td></tr>
<tr><td valign=\"top\"> Clock<br>Step</td>
          <td valign=\"top\"> Changed if-conditions from \"xxx < time\" to \"time >= xxx\"
                   (according to the Modelica specification, in the second case
                   a time event should be triggered, i.e., this change leads
                   potentially to a faster simulation).
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Digital.Converters.</b></td></tr>
<tr><td valign=\"top\"> BooleanToLogic<br>
                   LogicToBoolean<br>
                   RealToLogic<br>
                   LogicToReal</td>
          <td valign=\"top\"> Changed from \"algorithm\" to \"equation\" section
                   to allow better symbolic preprocessing
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.</b></td></tr>
<tr><td valign=\"top\"> Machines</td>
          <td valign=\"top\"> Slightly improved documentation, typos in
                   documentation corrected
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Machines.Examples.</b></td></tr>
<tr><td valign=\"top\"> AIMS_start</td>
          <td valign=\"top\"> Changed QuadraticLoadTorque1(TorqueDirection=true) to
                   QuadraticLoadTorque1(TorqueDirection=false) since more realistic
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Machines.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> PartialBasicMachine</td>
          <td valign=\"top\"> Introduced support flange to model the
                   reaction torque to the housing
          </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Machines.Sensors.</b></td></tr>
<tr><td valign=\"top\"> Rotorangle</td>
          <td valign=\"top\"> Introduced support flange to model the
                   reaction torque to the housing
          </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Examples.Elementary.</b></td></tr>
<tr><td valign=\"top\"> PointMassesWithGravity</td>
          <td valign=\"top\"> Added two point masses connected by a line force to demonstrate
                   additionally how this works. Connections of point masses
                   with 3D-elements are demonstrated in the new example
                   PointMassesWithGravity (there is the difficulty that the orientation
                   is not defined in a PointMass object and therefore some
                   special handling is needed in case of a connection with
                   3D-elements, where the orientation of the point mass is not
                   determined by these elements.</td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Examples.Systems.</b></td></tr>
<tr><td valign=\"top\"> RobotR3</td>
          <td valign=\"top\"> Changed from the \"old\" to the \"new\" bus concept with expandable connectors.
                   Replaced the non-standard Modelica function \"constrain()\" by
                   standard Modelica components. As a result, the non-standard function
                   constrain() is no longer used in the Modelica Standard Library.</td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Frames.Orientation.</b></td></tr>
<tr><td valign=\"top\"> equalityConstraint</td>
          <td valign=\"top\"> Use a better residual for the equalityConstraint function.
                   As a result, the non-linear equation system of a kinematic
                   loop is formulated in a better way (the range where the
                   desired result is a unique solution of the non-linear
                   system of equations becomes much larger).</td>
</tr>


<tr><td colspan=\"2\"><b>Mechanics.MultiBody.</b></td></tr>
<tr><td valign=\"top\"> Visualizers.</td>
          <td valign=\"top\"> Removed (misleading) annotation \"structurallyIncomplete\"
                   in the models of this sub-library
          </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\"> Examples</td>
          <td valign=\"top\"> For all models in this sub-library:
                   <ul>
                   <li> Included a housing object in all examples to compute
                                all support torques.</li>
                   <li> Replaced initialization by modifiers via the
                                initialization menu parameters of Inertia components.</li>
                   <li> Removed \"encapsulated\" and unnecessary \"import\". </li>
                   <li> Included \"StopTime\" in the annotations.</li>
                   </ul>
          </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.Rotational.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> FrictionBase</td>
          <td valign=\"top\"> Introduced \"fixed=true\" for Boolean variables startForward,
                   startBackward, mode.
          </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.Translational.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> FrictionBase</td>
          <td valign=\"top\"> Introduced \"fixed=true\" for Boolean variables startForward,
                   startBackward, mode.
          </td>
</tr>

<tr><td colspan=\"2\"><b>Media.UsersGuide.MediumUsage.</b></td></tr>
<tr><td valign=\"top\"> TwoPhase</td>
          <td valign=\"top\"> Improved documentation and demonstrating the newly introduced functions
          </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Examples.</b></td></tr>
<tr><td valign=\"top\"> WaterIF97</td>
          <td valign=\"top\"> Provided (missing) units for variables V, dV, H_flow_ext, m, U.
          </td>
</tr>

<tr><td colspan=\"2\"><b> Media.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> PartialMedium</td>
          <td valign=\"top\"> Final modifiers are removed from nX and nXi, to allow
                   customized medium models such as mixtures of refrigerants with oil, etc.
          </td>
</tr>
<tr><td valign=\"top\"> PartialCondensingGases</td>
          <td valign=\"top\"> Included attributes \"min=1, max=2\" for input argument FixedPhase
                   for functions setDewState and setBubbleState (in order to guarantee
                   that input arguments are correct).
          </td>
</tr>

<tr><td colspan=\"2\"><b> Media.Interfaces.PartialMedium.</b></td></tr>
<tr><td valign=\"top\"> BaseProperties</td>
          <td valign=\"top\"> New Boolean parameter \"standardOrderComponents\".
                   If true, last element vector X is computed from 1-sum(Xi) (= default)
                   otherwise, no equation is provided for it in PartialMedium.
          </td>
</tr>
<tr><td valign=\"top\"> IsentropicExponent</td>
          <td valign=\"top\"> \"max\" value changed from 1.7 to 500000
          </td>
</tr>
<tr><td valign=\"top\"> setState_pTX<br>
                   setState_phX<br>
                   setState_psX<br>
                   setState_dTX<br>
                   specificEnthalpy_pTX<br>
                   temperature_phX<br>
                   density_phX<br>
                   temperature_psX<br>
                   density_psX<br>
                   specificEnthalpy_psX</td>
          <td valign=\"top\"> Introduced defaut value \"reference_X\" for input argument \"X\".
          </td>
</tr>

<tr><td colspan=\"2\"><b> Media.Interfaces.PartialSimpleMedium.</b></td></tr>
<tr><td valign=\"top\"> setState_pTX<br>
                   setState_phX<br>
                   setState_psX<br>
                   setState_dTX</td>
          <td valign=\"top\"> Introduced defaut value \"reference_X\" for input argument \"X\".
          </td>
</tr>

<tr><td colspan=\"2\"><b> Media.Interfaces.PartialSimpleIdealGasMedium.</b></td></tr>
<tr><td valign=\"top\"> setState_pTX<br>
                   setState_phX<br>
                   setState_psX<br>
                   setState_dTX</td>
          <td valign=\"top\"> Introduced defaut value \"reference_X\" for input argument \"X\".
          </td>
</tr>

<tr><td colspan=\"2\"><b> Media.Air.MoistAir.</b></td></tr>
<tr><td valign=\"top\"> setState_pTX<br>
                   setState_phX<br>
                   setState_dTX</td>
          <td valign=\"top\"> Introduced defaut value \"reference_X\" for input argument \"X\".
          </td>
</tr>

<tr><td colspan=\"2\"><b> Media.IdealGases.Common.SingleGasNasa.</b></td></tr>
<tr><td valign=\"top\"> setState_pTX<br>
                   setState_phX<br>
                   setState_psX<br>
                   setState_dTX</td>
          <td valign=\"top\"> Introduced defaut value \"reference_X\" for input argument \"X\".
          </td>
</tr>

<tr><td colspan=\"2\"><b> Media.IdealGases.Common.MixtureGasNasa.</b></td></tr>
<tr><td valign=\"top\"> setState_pTX<br>
                   setState_phX<br>
                   setState_psX<br>
                   setState_dTX<br>
                   h_TX</td>
          <td valign=\"top\"> Introduced defaut value \"reference_X\" for input argument \"X\".
          </td>
</tr>

<tr><td colspan=\"2\"><b> Media.Common.</b></td></tr>
<tr><td valign=\"top\"> IF97PhaseBoundaryProperties<br>
                   gibbsToBridgmansTables </td>
          <td valign=\"top\"> Introduced unit for variables vt, vp.
          </td>
</tr>
<tr><td valign=\"top\"> SaturationProperties</td>
          <td valign=\"top\"> Introduced unit for variable dpT.
          </td>
</tr>
<tr><td valign=\"top\"> BridgmansTables</td>
          <td valign=\"top\"> Introduced unit for dfs, dgs.
          </td>
</tr>

<tr><td colspan=\"2\"><b> Media.Common.ThermoFluidSpecial.</b></td></tr>
<tr><td valign=\"top\"> gibbsToProps_ph<br>
                   gibbsToProps_ph  <br>
                   gibbsToBoundaryProps<br>
                   gibbsToProps_dT<br>
                   gibbsToProps_pT</td>
          <td valign=\"top\"> Introduced unit for variables vt, vp.
          </td>
<tr><td valign=\"top\"> TwoPhaseToProps_ph</td>
          <td valign=\"top\"> Introduced unit for variables dht, dhd, detph.
          </td>
</tr>

<tr><td colspan=\"2\"><b> Media.</b></td></tr>
<tr><td valign=\"top\"> MoistAir</td>
          <td valign=\"top\"> Documentation of moist air model significantly improved.
          </td>
</tr>

<tr><td colspan=\"2\"><b> Media.MoistAir.</b></td></tr>
<tr><td valign=\"top\"> enthalpyOfVaporization</td>
          <td valign=\"top\"> Replaced by linear correlation since simpler and more
                   accurate in the entire region.
          </td>
</tr>

<tr><td colspan=\"2\"><b> Media.Water.IF97_Utilities.BaseIF97.Regions.</b></td></tr>
<tr><td valign=\"top\"> drhovl_dp</td>
          <td valign=\"top\"> Introduced unit for variable dd_dp.
          </td>
</tr>

<tr><td colspan=\"2\"><b> Thermal.</b></td></tr>
<tr><td valign=\"top\"> FluidHeatFlow</td>
          <td valign=\"top\"> Introduced new parameter tapT (0..1) to define the
                   temperature of the HeatPort as linear combination of the
                   flowPort_a (tapT=0) and flowPort_b (tapT=1) temperatures.
          </td>
</tr>
</table>



<p><br>
The following <b style=\"color:red\">critical errors</b> have been fixed (i.e. errors
that can lead to wrong simulation results):
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Electrical.Machines.BasicMachines.Components.</b></td></tr>
<tr><td valign=\"top\"> ElectricalExcitation</td>
          <td valign=\"top\"> Excitation voltage ve is calculated as
                   \"spacePhasor_r.v_[1]*TurnsRatio*3/2\" instead of
                   \"spacePhasor_r.v_[1]*TurnsRatio
           </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Parts.</b></td></tr>
<tr><td valign=\"top\"> FixedRotation</td>
          <td valign=\"top\"> Bug corrected that the torque balance was wrong in the
                   following cases (since vector r was not transformed
                   from frame_a to frame_b; note this special case occurs very seldomly in practice):
                   <ul><li> frame_b is in the spanning tree closer to the root
                                        (usually this is frame_a).</li>
                           <li> vector r from frame_a to frame_b is not zero.</li>
                   </ul>
           </td>
</tr>

<tr><td valign=\"top\"> PointMass</td>
         <td valign=\"top\"> If a PointMass model is connected so that no equations are present
                  to compute its orientation object, the orientation was arbitrarily
                  set to a unit rotation. In some cases this can lead to a wrong overall
                  model, depending on how the PointMass model is used. For this reason,
                  such cases lead now to an error (via an assert(..)) with an explanation
                  how to fix this.
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Interfaces.PartialPureSubstance.</b></td></tr>
<tr><td valign=\"top\"> pressure_dT<br>
                   specificEnthalpy_dT
          </td>
          <td valign=\"top\"> Changed wrong call from \"setState_pTX\" to \"setState_dTX\"
           </td>
</tr>


<tr><td colspan=\"2\"><b>Media.Interfaces.PartialTwoPhaseMedium.</b></td></tr>
<tr><td valign=\"top\"> pressure_dT<br>
                   specificEnthalpy_dT
          </td>
          <td valign=\"top\"> Changed wrong call from \"setState_pTX\" to \"setState_dTX\"
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Common.ThermoFluidSpecial.</b></td></tr>
<tr><td valign=\"top\"> gibbsToProps_dT<br>
                   helmholtzToProps_ph<br>
                   helmholtzToProps_pT<br>
                   helmholtzToProps_dT</td>
          <td valign=\"top\"> Bugs in equations corrected </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Common.</b></td></tr>
<tr><td valign=\"top\"> helmholtzToBridgmansTables<br>
                   helmholtzToExtraDerivs</td>
          <td valign=\"top\"> Bugs in equations corrected </td>
</tr>

<tr><td colspan=\"2\"><b>Media.IdealGases.Common.SingleGasNasa.</b></td></tr>
<tr><td valign=\"top\"> density_derp_T</td>
          <td valign=\"top\"> Bug in equation of partial derivative corrected </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Water.IF97_Utilities.</b></td></tr>
<tr><td valign=\"top\"> BaseIF97.Inverses.dtofps3<br>
                   isentropicExponent_props_ph<br>
                   isentropicExponent_props_pT<br>
                   isentropicExponent_props_dT</td>
          <td valign=\"top\"> Bugs in equations corrected </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Air.MoistAir.</b></td></tr>
<tr><td valign=\"top\"> h_pTX</td>
          <td valign=\"top\"> Bug in setState_phX due to wrong vector size in h_pTX corrected.
                   Furthermore, syntactical errors corrected:
                   <ul><li> In function massFractionpTphi an equation
                                        sign is used in an algorithm.</li>
                           <li> Two consecutive semicolons removed</li>
                   </ul>
          </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Water.</b></td></tr>
<tr><td valign=\"top\"> waterConstants</td>
          <td valign=\"top\"> Bug in equation of criticalMolarVolume corrected.
          </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Water.IF97_Utilities.BaseIF97.Regions.</b></td></tr>
<tr><td valign=\"top\"> region_ph<br>
                   region_ps</td>
          <td valign=\"top\"> Bug in region determination corrected.
          </td>
</tr>

<tr><td valign=\"top\"> boilingcurve_p<br>
                   dewcurve_p</td>
          <td valign=\"top\"> Bug in equation of plim corrected.
          </td>
</tr>
</table>


<p><br>
The following <b style=\"color:red\">uncritical errors</b> have been fixed (i.e. errors
that do <b style=\"color:red\">not</b> lead to wrong simulation results, but, e.g.,
units are wrong or errors in documentation):
</p>

<table border=\"1\" cellspacing=0 cellpadding=2 style=\"border-collapse:collapse;\">
<tr><td colspan=\"2\"><b>Blocks.</b></td></tr>
<tr><td valign=\"top\"> Examples</td>
          <td valign=\"top\"> Corrected typos in description texts of bus example models.
           </td>
</tr>

<tr><td colspan=\"2\"><b>Blocks.Continuous.</b></td></tr>
<tr><td valign=\"top\"> LimIntegrator</td>
          <td valign=\"top\"> removed incorrect smooth(0,..) because expression might be discontinuous.
           </td>
</tr>

<tr><td colspan=\"2\"><b>Blocks.Math.UnitConversions.</b></td></tr>
<tr><td valign=\"top\"> block_To_kWh<br>block_From_kWh</td>
          <td valign=\"top\"> Corrected unit from \"kWh\" to (syntactically correct) \"kW.h\".
           </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Analog.Examples.</b></td></tr>
<tr><td valign=\"top\"> HeatingNPN_OrGate</td>
          <td valign=\"top\"> Included start values, so that initialization is
                                                successful </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Analog.Lines.</b></td></tr>
<tr><td valign=\"top\"> OLine</td>
          <td valign=\"top\"> Corrected unit from \"Siemens/m\" to \"S/m\".
           </td>
<tr><td valign=\"top\"> TLine2</td>
          <td valign=\"top\"> Changed wrong type of parameter NL (normalized length) from
                   SIunits.Length to Real.
           </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Digital.Delay.</b></td></tr>
<tr><td valign=\"top\"> TransportDelay</td>
          <td valign=\"top\"> Syntax error corrected
                   (\":=\" in equation section is converted by Dymola silently to \"=\").
           </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.Digital</b></td></tr>
<tr><td valign=\"top\"> Converters</td>
          <td valign=\"top\"> Syntax error corrected
                   (\":=\" in equation section is converted by Dymola silently to \"=\").
           </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.MultiPhase.Basic.</b></td></tr>
<tr><td valign=\"top\"> Conductor</td>
          <td valign=\"top\"> Changed wrong type of parameter G from SIunits.Resistance to
                   SIunits.Conductance.
           </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.MultiPhase.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> Plug<br></td>
          <td valign=\"top\"> Made used \"pin\" connectors non-graphical (otherwise,
                   there are difficulties to connect to Plug).
           </td>
</tr>

<tr><td colspan=\"2\"><b>Electrical.MultiPhase.Sources.</b></td></tr>
<tr><td valign=\"top\"> SineCurrent</td>
          <td valign=\"top\"> Changed wrong type of parameter offset from SIunits.Voltage to
                   SIunits.Current.
           </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Examples.Loops.</b></td></tr>
<tr><td valign=\"top\"> EngineV6</td>
          <td valign=\"top\"> Corrected wrong crankAngleOffset of some cylinders
                   and improved the example.
           </td>
</tr>


<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Examples.Loops.Utilities.</b></td></tr>
<tr><td valign=\"top\"> GasForce</td>
          <td valign=\"top\"> Wrong units corrected:
                   \"SIunitsPosition x,y\" to \"Real x,y\";
" + "           \"SIunits.Pressure press\" to \"SIunits.Conversions.NonSIunits.Pressure_bar\"
           </td>
</tr>
<tr><td valign=\"top\"> GasForce2</td>
          <td valign=\"top\"> Wrong unit corrected: \"SIunits.Position x\" to \"Real x\".
           </td>
</tr>
<tr><td valign=\"top\"> EngineV6_analytic</td>
          <td valign=\"top\"> Corrected wrong crankAngleOffset of some cylinders.
           </td>
</tr>


<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> PartialLineForce</td>
          <td valign=\"top\"> Corrected wrong unit: \"SIunits.Position eRod_a\" to \"Real eRod_a\";
           </td>
</tr>
<tr><td valign=\"top\"> FlangeWithBearingAdaptor </td>
          <td valign=\"top\"> If includeBearingConnector = false, connector \"fr\"
                           + \"ame\" was not
                   removed. As long as the connecting element to \"frame\" determines
                   the non-flow variables, this is fine. In the corrected version, \"frame\"
                   is conditionally removed in this case.</td>
</tr>


<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Forces.</b></td></tr>
<tr><td valign=\"top\"> ForceAndTorque</td>
          <td valign=\"top\"> Corrected wrong unit: \"SIunits.Force t_b_0\" to \"SIunits.Torque t_b_0\".
           </td>
</tr>
<tr><td valign=\"top\"> LineForceWithTwoMasses</td>
          <td valign=\"top\"> Corrected wrong unit: \"SIunits.Position e_rel_0\" to \"Real e_rel_0\".
           </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Frames.</b></td></tr>
<tr><td valign=\"top\"> axisRotation</td>
          <td valign=\"top\"> Corrected wrong unit: \"SIunits.Angle der_angle\" to
                        \"SIunits.AngularVelocity der_angle\".
           </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Joints.Assemblies.</b></td></tr>
<tr><td valign=\"top\"> JointUSP<br>JointSSP</td>
          <td valign=\"top\"> Corrected wrong unit: \"SIunits.Position aux\"  to \"Real\"
           </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Sensors.</b></td></tr>
<tr><td valign=\"top\"> AbsoluteSensor</td>
          <td valign=\"top\"> Corrected wrong units: \"SIunits.Acceleration angles\" to
                   \"SIunits.Angle angles\" and
                   \"SIunits.Velocity w_abs_0\" to \"SIunits.AngularVelocity w_abs_0\"
           </td>
</tr>
<tr><td valign=\"top\"> RelativeSensor</td>
          <td valign=\"top\"> Corrected wrong units: \"SIunits.Acceleration angles\" to
                   \"SIunits.Angle angles\"
           </td>
</tr>
<tr><td valign=\"top\"> Distance</td>
          <td valign=\"top\"> Corrected wrong units: \"SIunits.Length L2\" to \"SIunits.Area L2\" and
                   SIunits.Length s_small2\" to \"SIunits.Area s_small2\"
           </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.MultiBody.Visualizers.Advanced.</b></td></tr>
<tr><td valign=\"top\"> Shape</td>
          <td valign=\"top\"> Changed \"MultiBody.Types.Color color\" to \"Real color[3]\", since
                   \"Types.Color\" is \"Integer color[3]\" and there have been backward
                   compatibility problems with models using \"color\" before it was changed
                   to \"Types.Color\".
           </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.Rotational.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> FrictionBase</td>
          <td valign=\"top\"> Rewrote equations with new variables \"unitAngularAcceleration\" and
                   \"unitTorque\" in order that the equations are correct with respect
                   to units (previously, variable \"s\" can be both a torque and an
                   angular acceleration and this lead to unit incompatibilities)
           </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\"> OneWayClutch<br>LossyGear</td>
          <td valign=\"top\"> Rewrote equations with new variables \"unitAngularAcceleration\" and
                   \"unitTorque\" in order that the equations are correct with respect
                   to units (previously, variable \"s\" can be both a torque and an
                   angular acceleration and this lead to unit incompatibilities)
           </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.Translational.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> FrictionBase</td>
          <td valign=\"top\"> Rewrote equations with new variables \"unitAngularAcceleration\" and
                   \"unitTorque\" in order that the equations are correct with respect
                   to units (previously, variable \"s\" can be both a torque and an
                   angular acceleration and this lead to unit incompatibilities)
           </td>
</tr>

<tr><td colspan=\"2\"><b>Mechanics.Translational.</b></td></tr>
<tr><td valign=\"top\"> Speed</td>
          <td valign=\"top\"> Corrected unit of v_ref from SIunits.Position to SIunits.Velocity
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Examples.Tests.Components.</b></td></tr>
<tr><td valign=\"top\"> PartialTestModel<br>PartialTestModel2</td>
          <td valign=\"top\"> Corrected unit of h_start from \"SIunits.Density\" to \"SIunits.SpecificEnthalpy\"
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Examples.SolveOneNonlinearEquation.</b></td></tr>
<tr><td valign=\"top\"> Inverse_sh_T
                   InverseIncompressible_sh_T<br>
                   Inverse_sh_TX</td>
          <td valign=\"top\"> Rewrote equations so that dimensional (unit) analysis is correct\"
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Incompressible.Examples.</b></td></tr>
<tr><td valign=\"top\"> TestGlycol</td>
          <td valign=\"top\"> Rewrote equations so that dimensional (unit) analysis is correct\"
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Interfaces.PartialTwoPhaseMedium.</b></td></tr>
<tr><td valign=\"top\"> dBubbleDensity_dPressure<br>dDewDensity_dPressure</td>
          <td valign=\"top\"> Changed wrong type of ddldp from \"DerDensityByEnthalpy\"
                   to \"DerDensityByPressure\".
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Common.ThermoFluidSpecial.</b></td></tr>
<tr><td valign=\"top\"> ThermoProperties</td>
          <td valign=\"top\"> Changed wrong units:
                   \"SIunits.DerEnergyByPressure dupT\" to \"Real dupT\" and
                   \"SIunits.DerEnergyByDensity dudT\" to \"Real dudT\"
           </td>
</tr>
<tr><td valign=\"top\"> ThermoProperties_ph</td>
          <td valign=\"top\"> Changed wrong unit from \"SIunits.DerEnergyByPressure duph\" to \"Real duph\"
           </td>
</tr>
<tr><td valign=\"top\"> ThermoProperties_pT</td>
          <td valign=\"top\"> Changed wrong unit from \"SIunits.DerEnergyByPressure dupT\" to \"Real dupT\"
           </td>
</tr>
<tr><td valign=\"top\"> ThermoProperties_dT</td>
          <td valign=\"top\">  Changed wrong unit from \"SIunits.DerEnergyByDensity dudT\" to \"Real dudT\"
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.IdealGases.Common.SingleGasNasa.</b></td></tr>
<tr><td valign=\"top\"> cp_Tlow_der</td>
          <td valign=\"top\"> Changed wrong unit from \"SIunits.Temperature dT\" to \"Real dT\".
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Water.IF97_Utilities.BaseIF97.Basic.</b></td></tr>
<tr><td valign=\"top\"> p1_hs<br>
                   h2ab_s<br>
                   p2a_hs<br>
                   p2b_hs<br>
                   p2c_hs<br>
                   h3ab_p<br>
                   T3a_ph<br>
                   T3b_ph<br>
                   v3a_ph<br>
                   v3b_ph<br>
                   T3a_ps<br>
                   T3b_ps<br>
                   v3a_ps<br>
                   v3b_ps</td>
          <td valign=\"top\"> Changed wrong unit of variables h/hstar, s, sstar from
                   \"SIunits.Enthalpy\" to \"SIunits.SpecificEnthalpy\",
                   \"SIunits.SpecificEntropy\", \"SIunits.SpecificEntropy
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Water.IF97_Utilities.BaseIF97.Transport.</b></td></tr>
<tr><td valign=\"top\"> cond_dTp</td>
          <td valign=\"top\"> Changed wrong unit of TREL, rhoREL, lambdaREL from
                   \"SIunits.Temperature\", \"SIunit.Density\", \"SIunits.ThermalConductivity\"
                   to \"Real\".
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Water.IF97_Utilities.BaseIF97.Inverses.</b></td></tr>
<tr><td valign=\"top\"> tofps5<br>tofpst5</td>
          <td valign=\"top\"> Changed wrong unit of pros from \"SIunits.SpecificEnthalpy\" to
                   \"SIunits.SpecificEntropy\".
           </td>
</tr>

<tr><td colspan=\"2\"><b>Media.Water.IF97_Utilities.</b></td></tr>
<tr><td valign=\"top\"> waterBaseProp_ph</td>
          <td valign=\"top\"> Improved calculation at the limits of the validity.
           </td>
</tr>

        <tr><td colspan=\"2\"><b>Thermal.</b></td></tr>
<tr><td valign=\"top\"> FluidHeatFlow<br>HeatTransfer</td>
          <td valign=\"top\"> Corrected wrong unit \"SIunits.Temperature\" of difference temperature
                        variables with \"SIunits.TemperatureDifference\".
           </td>
</tr>

</table>

</html>"));
      end Version_2_2_2;
      class Version_2_2_1 "Version 2.2.1 (March 24, 2006)"
        annotation(Documentation(info = "<html>

<p>
Version 2.2.1 is backward compatible to version 2.2.
</p>

<p>
In this version, <b>no</b> new libraries have been added.
The following major improvements have been made:
</p>

<ul>
<li> The <b>Documentation</b> of the Modelica standard library was
         considerably improved:<br>
         In Dymola 6, the new feature was introduced to automatically add tables
         for class content and component interface definitions (parameters and
         connectors) to the info layer. For this reason, the corresponding (partial)
         tables previously present in the Modelica Standard Library have been
         removed. The new feature of Dymola 6 has the significant advantage that
         all tables are now guaranteed to be up-to-date.<br>
         Additionally, the documentation has been improved by adding appropriate
         description texts to parameters, connector instances, function input
         and output arguments etc., in order that the automatically generated
         tables do not have empty entries. Also new User's Guides for sublibraries
         Rotational and SIunits have been added and the User's Guide on top
         level (Modelica.UsersGuide) has been improved.<br>&nbsp;</li>

<li> Initialization options have been added to the Modelica.Blocks.<b>Continuous</b>
         blocks (NoInit, SteadyState, InitialState, InitialOutput). If InitialOutput
         is selected, the block output is provided as initial condition. The states
         of the block are then initialized as close as possible to steady state.
         Furthermore, the Continuous.LimPID block has been significantly
         improved and much better documented.<br>&nbsp;</li>

<li> The Modelica.<b>Media</b> library has been significantly improved:<br>
         New functions setState_pTX, setState_phX, setState_psX, setState_dTX
         have been added to PartialMedium to compute the independent medium variables
         (= state of medium) from p,T,X, or from p,h,X or from p,s,X or from
         d,T,X. Then functions are provided for all interesting medium variables
         to compute them from its medium state. All these functions are
         implemented in a robust way for all media (with a few exceptions, if the
         generic function does not make sense for a particular medium).</li>
</ul>

<p>
The following <b>new components</b> have been added to <b>existing</b> libraries:
</p>

<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td colspan=\"2\"><b>Modelica.Blocks.Examples.</b></td></tr>
<tr><td valign=\"top\"> PID_Controller</td>
          <td valign=\"top\"> Example to demonstrate the usage of the
                   Blocks.Continuous.LimPID block.</td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Blocks.Math.</b></td></tr>
<tr><td valign=\"top\"> UnitConversions.*</td>
          <td valign=\"top\"> New package that provides blocks for unit conversions.
                   UnitConversions.ConvertAllBlocks allows to select all
                   available conversions from a menu.</td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.Machines.BasicMachines.SynchronousInductionMachines.</b></td></tr>
<tr><td valign=\"top\"> SM_ElectricalExcitedDamperCage</td>
          <td valign=\"top\"> Electrical excited synchronous induction machine with damper cage</td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.Machines.BasicMachines.Components.</b></td></tr>
<tr><td valign=\"top\"> ElectricalExcitation </td>
          <td valign=\"top\"> Electrical excitation for electrical excited synchronous
                   induction machines</td> </tr>
<tr><td valign=\"top\"> DamperCage</td>
          <td valign=\"top\"> Unsymmetrical damper cage for electrical excited synchronous
                   induction machines. At least the user has to specify the dampers
                   resistance and stray inductance in d-axis; if he omits the
                   parameters of the q-axis, the same values as for the d.axis
                   are used, assuming a symmetrical damper.</td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.Machines.Examples.</b></td></tr>
<tr><td valign=\"top\"> SMEE_Gen </td>
          <td valign=\"top\"> Test example 7: ElectricalExcitedSynchronousInductionMachine
                   as Generator</td> </tr>
<tr><td valign=\"top\"> Utilities.TerminalBox</td>
          <td valign=\"top\"> Terminal box for three-phase induction machines to choose
                   either star (wye) ? or delta ? connection</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Math.Matrices.</b></td></tr>
<tr><td valign=\"top\"> equalityLeastSquares</td>
          <td valign=\"top\"> Solve a linear equality constrained least squares problem:<br>
                  min|A*x-a|^2 subject to B*x=b</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.</b></td></tr>
<tr><td valign=\"top\"> Parts.PointMass</td>
          <td valign=\"top\"> Point mass, i.e., body where inertia tensor is neglected.</td> </tr>
<tr><td valign=\"top\"> Interfaces.FlangeWithBearing</td>
          <td valign=\"top\"> Connector consisting of 1-dim. rotational flange and its
                   3-dim. bearing frame.</td> </tr>
<tr><td valign=\"top\"> Interfaces.FlangeWithBearingAdaptor</td>
          <td valign=\"top\"> Adaptor to allow direct connections to the sub-connectors
                   of FlangeWithBearing.</td> </tr>
<tr><td valign=\"top\"> Types.SpecularCoefficient</td>
          <td valign=\"top\"> New type to define a specular coefficient.</td> </tr>
<tr><td valign=\"top\"> Types.ShapeExtra</td>
          <td valign=\"top\"> New type to define the extra data for visual shape objects and to
                   have a central place for the documentation of this data.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.Examples.Elementary</b></td></tr>
<tr><td valign=\"top\"> PointGravityWithPointMasses</td>
          <td valign=\"top\"> Example of two point masses in a central gravity field.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\">UsersGuide</td>
          <td valign=\"top\"> A User's Guide has been added by using the documentation previously
                   present in the package documentation of Rotational.</td> </tr>
<tr><td valign=\"top\">Sensors.PowerSensor</td>
          <td valign=\"top\"> New component to measure the energy flow between two connectors
                   of the Rotational library.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.Translational.</b></td></tr>
<tr><td valign=\"top\">Speed</td>
          <td valign=\"top\"> New component to move a translational flange
                   according to a reference speed</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Media.Interfaces.PartialMedium.</b></td></tr>
<tr><td valign=\"top\">specificEnthalpy_pTX</td>
          <td valign=\"top\"> New function to compute specific enthalpy from pressure, temperature
                   and mass fractions.</td> </tr>
<tr><td valign=\"top\">temperature_phX</td>
          <td valign=\"top\"> New function to compute temperature from pressure, specific enthalpy,
                   and mass fractions.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Icons.</b></td></tr>
<tr><td valign=\"top\"> SignalBus</td>
          <td valign=\"top\"> Icon for signal bus</td> </tr>
<tr><td valign=\"top\"> SignalSubBus</td>
          <td valign=\"top\"> Icon for signal sub-bus</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.SIunits.</b></td></tr>
<tr><td valign=\"top\">UsersGuide</td>
          <td valign=\"top\"> A User's Guide has been added that describes unit handling.</td> </tr>
<tr><td valign=\"top\"> Resistance<br>
                   Conductance</td>
          <td valign=\"top\"> Attribute 'min=0' removed from these types.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Thermal.FluidHeatFlow.</b></td></tr>
<tr><td valign=\"top\"> Components.Valve</td>
          <td valign=\"top\"> Simple controlled valve with either linear or
                   exponential characteristic.</td> </tr>
<tr><td valign=\"top\"> Sources. IdealPump </td>
          <td valign=\"top\"> Simple ideal pump (resp. fan)  dependent on the shaft?s speed;
                   pressure increase versus volume flow is defined as a linear
                   function. Torque * Speed = Pressure invrease * Volume flow
                   (without losses).</td> </tr>
<tr><td valign=\"top\"> Examples.PumpAndValve </td>
          <td valign=\"top\"> Test example for valves.</td> </tr>
<tr><td valign=\"top\"> Examples.PumpDropOut </td>
          <td valign=\"top\"> Drop out of 1 pump to test behavior of semiLinear.</td> </tr>
<tr><td valign=\"top\"> Examples.ParallelPumpDropOut </td>
          <td valign=\"top\"> Drop out of 2 parallel pumps to test behavior of semiLinear.</td> </tr>
<tr><td valign=\"top\"> Examples.OneMass </td>
          <td valign=\"top\"> Cooling of 1 hot mass to test behavior of semiLinear.</td> </tr>
<tr><td valign=\"top\"> Examples.TwoMass </td>
          <td valign=\"top\"> Cooling of 2 hot masses to test behavior of semiLinear.</td> </tr>
</table>

<p>
The following <b>components</b> have been improved:
</p>

<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td colspan=\"2\"><b>Modelica.</b></td></tr>
<tr><td valign=\"top\"> UsersGuide</td>
          <td valign=\"top\"> User's Guide and package description of Modelica Standard Library improved.</td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Blocks.Interfaces.</b></td></tr>
<tr><td valign=\"top\"> RealInput<br>
                   BooleanInput<br>
                   IntegerInput</td>
          <td valign=\"top\"> When dragging one of these connectors the width and height
                   is a factor of 2 larger as a standard icon. Previously,
                   these connectors have been dragged and then manually enlarged
                   by a factor of 2 in the Modelica standard library.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Blocks.</b></td></tr>
<tr><td valign=\"top\"> Continuous.*</td>
          <td valign=\"top\"> Initialization options added to all blocks
                   (NoInit, SteadyState, InitialState, InitialOutput).
                   New parameter limitsAtInit to switch off the limits
                   of LimIntegrator or LimPID during initialization</td> </tr>
<tr><td valign=\"top\"> Continuous.LimPID</td>
          <td valign=\"top\"> Option to select P, PI, PD, PID controller.
                   Documentation significantly improved.</td> </tr>
<tr><td valign=\"top\"> Nonlinear.Limiter<br>
                   Nonlinear.VariableLimiter<br>
                   Nonlinear.Deadzone</td>
          <td valign=\"top\"> New parameter limitsAtInit/deadZoneAtInit to switch off the limits
                   or the dead zone during initialization</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog. </b></td></tr>
<tr><td valign=\"top\"> Sources</td>
          <td valign=\"top\"> Icon improved (+/- added to voltage sources, arrow added to
                   current sources).</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Semiconductors. </b></td></tr>
<tr><td valign=\"top\"> Diode</td>
          <td valign=\"top\"> smooth() operator included to improve numerics.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Machines.BasicMachines.SynchronousInductionMachines. </b></td></tr>
<tr><td valign=\"top\"> SM_PermanentMagnetDamperCage<br>
                   SM_ElectricalExcitedDamperCage<br>
                   SM_ReluctanceRotorDamperCage</td>
          <td valign=\"top\"> The user can choose \"DamperCage = false\" (default: true)
                   to remove all equations for the damper cage from the model.</td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.Machines.BasicMachines.AsynchronousInductionMachines. </b></td></tr>
<tr><td valign=\"top\"> AIM_SlipRing</td>
          <td valign=\"top\"> Easier parameterization: if the user selects \"useTrunsRatio = false\"
                   (default: true, this is the same behavior as before),
                        parameter TurnsRatio is calculated internally from
                        Nominal stator voltage and Locked-rotor voltage.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Math.Matrices.</b></td></tr>
<tr><td valign=\"top\">leastSquares</td>
          <td valign=\"top\">The A matrix in the least squares problem might be rank deficient.
                  Previously, it was required that A has full rank.</td>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.</b></td></tr>
<tr><td valign=\"top\">all models</td>
          <td valign=\"top\"> <ul>
                   <li> All components with animation information have a new variable
                                <b>specularCoefficient</b> to define the reflection of ambient light.
                                The default value is world.defaultSpecularCoefficient which has
                                a default of 0.7. By changing world.defaultSpecularCoefficient, the
                                specularCoefficient of all components is changed that are not
                                explicitly set differently. Since specularCoefficient is a variable
                                (and no parameter), it can be changed during simulation. Since
                                annotation(Dialog) is set, this variable still appears in the
                                parameter menus.<br>
                                Previously, a constant specularCoefficient of 0.7 was used
                                for all components.</li>
                   <li> Variable <b>color</b> of all components is no longer a parameter
                                but an input variable. Also all parameters in package <b>Visualizers</b>,
                                with the exception of <b>shapeType</b> are no longer parameters but
                                defined as input variables with annotation(Dialog). As a result,
                                all these variables appear still in parameter menus, but they can
                                be changed during simulation (e.g., color might be used to
                                display the temperature of a part).</li>
                   <li> All menus have been changed to follow the Modelica 2.2 annotations
                                \"Dialog, group, tab, enable\" (previously, a non-standard Dymola
                                definition for menus was used). Also, the \"enable\" annotation
                                is used in all menus
                                to disable input fields if the input would be ignored.</li>
                   <li> All visual shapes are now defined with conditional declarations
                                (to remove them, if animation is switched off). Previously,
                                these (protected) objects have been defined by arrays with
                                dimension 0 or 1.</li>
                   </ul></td></tr>

<tr><td valign=\"top\">Frames.resolveRelative</td>
          <td valign=\"top\"> The derivative of this function added as function and defined via
                   an annotation. In certain situations, tools had previously
                   difficulties to differentiate the inlined function automatically.</td>


<tr><td valign=\"top\">Forces.*</td>
          <td valign=\"top\"> The scaling factors N_to_m and Nm_to_m have no longer a default
                   value of 1000 but a default value of world.defaultN_to_m (=1000)
                   and world.defaultNm_to_m (=1000). This allows to change the
                   scaling factors for all forces and torques in the world
                   object.</td>
<tr><td valign=\"top\">Interfaces.Frame.a<br>
                  Interfaces.Frame.b<br>
                  Interfaces.Frame_resolve</td>
          <td valign=\"top\"> The Frame connectors are now centered around the origin to ease
                   the usage. The shape was changed, such that the icon is a factor
                   of 1.6 larger as a standard icon (previously, the icon had a
                   standard size when dragged and then the icon was manually enlarged
                   by a factor of 1.5 in the y-direction in the MultiBody library;
                   the height of 16 allows easy positioning on the standard grid size of 2).
                   The double line width of the border in icon and diagram layer was changed
                   to a single line width and when making a connection the connection
                   line is dark grey and no longer black which looks better.</td>
<tr><td valign=\"top\">Joints.Assemblies.*</td>
          <td valign=\"top\"> When dragging an assembly joint, the icon is a factor of 2
                   larger as a standard icon. Icon texts and connectors have a
                   standard size in this enlarged icon (and are not a factor of 2
                   larger as previously). </td>
<tr><td valign=\"top\">Types.*</td>
          <td valign=\"top\"> All types have a corresponding icon now to visualize the content
                   in the package browser (previously, the types did not have an icon).</td>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\"> Inertia</td>
          <td valign=\"top\"> Initialization and state selection added.</td> </tr>
<tr><td valign=\"top\"> SpringDamper</td>
          <td valign=\"top\"> Initialization and state selection added.</td> </tr>
<tr><td valign=\"top\"> Move</td>
          <td valign=\"top\"> New implementation based solely on Modelica 2.2 language
                   (previously, the Dymola specific constrain(..) function was used).</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Mechanics.Translational.</b></td></tr>
<tr><td valign=\"top\"> Move</td>
          <td valign=\"top\"> New implementation based solely on Modelica 2.2 language
                   (previously, the Dymola specific constrain(..) function was used).</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Thermal.FluidHeatFlow.Interfaces.Partials.</b></td></tr>
<tr><td valign=\"top\"> SimpleFriction</td>
          <td valign=\"top\"> Calculates friction losses from pressure drop and volume flow.</td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Thermal.FluidHeatFlow.Components.</b></td></tr>
<tr><td valign=\"top\"> IsolatedPipe <br>
                   HeatedPipe</td>
          <td valign=\"top\"> Added geodetic height as a source of pressure change;
                   feeds friction losses as calculated by simple friction to
                   the energy balance of the medium.</td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Media.Interfaces.PartialMedium.FluidConstants.</b></td></tr>
<tr><td valign=\"top\">HCRIT0</td><td valign=\"top\">Critical specifc enthalpy of the fundamental
                  equation (base formulation of the fluid medium model). </td></tr>
<tr><td valign=\"top\">SCRIT0</td><td valign=\"top\">Critical specifc entropy of the fundamental
                  equation (base formulation of the fluid medium model). </td></tr>
<tr><td valign=\"top\">deltah</td><td valign=\"top\">Enthalpy offset (default: 0) between the
                  specific enthalpy of the fluid model and the user-visible
                  specific enthalpy in the model: deltah = h_model - h_fundamentalEquation.
</td></tr>
<tr><td valign=\"top\">deltas</td><td valign=\"top\">Entropy offset (default: 0) between the
                  specific entropy of the fluid model and the user-visible
                  specific entropy in the model: deltas = s_model - s_fundamentalEquation.</td></tr>
<tr><td valign=\"top\">T_default</td><td valign=\"top\">Default value for temperature of medium (for initialization)</td></tr>
<tr><td valign=\"top\">h_default</td><td valign=\"top\">Default value for specific enthalpy of medium (for initialization)</td></tr>
<tr><td valign=\"top\">p_default</td><td valign=\"top\">Default value for pressure of medium (for initialization)</td></tr>
<tr><td valign=\"top\">X_default</td><td valign=\"top\">Default value for mass fractions of medium (for initialization)</td></tr>
</table>
<p>
The following <b>errors</b> have been fixed:
</p>

<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td colspan=\"2\"><b>Modelica.Blocks.Tables.</b></td></tr>
<tr><td valign=\"top\">CombiTable1D<br>
                  CombiTable1Ds<br>
                  CombiTable2D</td>
          <td valign=\"top\"> Parameter \"tableOnFile\" determines now whether a table is read from
                   file or used from parameter \"table\". Previously, if \"fileName\" was not
                   \"NoName\", a table was always read from file \"fileName\", independently
                   of the setting of \"tableOnFile\". This has been corrected.<br>
                   Furthermore, the initialization of a table is now performed in a
                   when-clause and no longer in a parameter declaration, because some
                   tools evaluate the parameter declaration in some situation more than
                   once and then the table is unnecessarily read several times
                   (and occupies also more memory).</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Blocks.Sources.</b></td></tr>
<tr><td valign=\"top\">CombiTimeTable</td>
          <td valign=\"top\"> Same bug fix/improvement as for the tables from Modelica.Blocks.Tables
                   as outlined above.</td> </tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Semiconductors. </b></td></tr>
<tr><td valign=\"top\"> PMOS<br>
                   NMOS<br>
                   HeatingPMOS<br>
                   HeatingNMOS</td>
          <td valign=\"top\"> The Drain-Source-Resistance RDS had actually a resistance of
                   RDS/v, with v=Beta*(W+dW)/(L+dL). The correct formula is without
                   the divison by \"v\". This has now been corrected.<br>
                   This bug fix should not have an essential effect in most applications.
                   In the default case (Beta=1e-5), the Drain-Source-Resistance was
                   a factor of 1e5 too large and had in the default case the
                   wrong value 1e12, although it should have the value 1e7. The effect
                   was that this resistance had practically no effect.</td> </tr>

<tr><td colspan=\"2\"> <b>Modelica.Media.IdealGases.Common.SingleGasNasa.</b></td></tr>
<tr><td valign=\"top\"> dynamicViscosityLowPressure</td>
          <td valign=\"top\"> Viscosity and thermal conductivity (which needs viscosity as input)
                   were computed wrong for polar gases and gas mixtures
                   (i.e. if dipole moment not 0.0). This has been fixed in version 2.2.1.</td> </tr>
<tr><td colspan=\"2\"><b>Modelica.Utilities.Streams.</b></td></tr>
<tr><td valign=\"top\">readLine</td>
          <td valign=\"top\"> Depending on the C-implementation, the stream was not correctly closed.
                   This has been corrected by adding a \"Streams.close(..)\"
                   after reading the file content.</td> </tr>

</table>
</html>
"));
      end Version_2_2_1;
      class Version_2_2 "Version 2.2 (April 6, 2005)"
        annotation(Documentation(info = "<html>

<p>
Version 2.2 is backward compatible to version 2.1.
</p>
The following <b>new libraries</b> have been added:
</p>
<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Media\">Modelica.Media</a></td>
          <td valign=\"top\"> Property models of liquids and gases, especially
                   <ul>
                   <li>1241 detailed gas models,</li>
                   <li> moist air,</li>
                   <li> high precision water model (according to IAPWS/IF97 standard), </li>
                   <li> incompressible media defined by tables (cp(T), rho(t), eta(T), etc. are defined by tables).</li>
                   </ul>
                   The user can conveniently define mixtures of gases between the
                   1241 gas models. The models are
                   designed to work well in dynamic simulations. They
                   are based on a new standard interface for media with
                   single and multiple substances and one or multiple phases
                   with the following features:
                   <ul>
                   <li> The independent variables of a medium model do not influence the
                                definition of a fluid connector port or how the
                                balance equations have to be implemented.<br>
                                Used independent variables: \"p,T\", \"p,T,X\", \"p,h\", \"d,T\".</li>
                   <li> Optional variables, e.g., dynamic viscosity, are only computed
                                if needed.<li>
                   <li> The medium models are implemented with regards to efficient
                                dynamic simulation.</li>
                   </ul>
          </td><tr>
<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Thermal.FluidHeatFlow\">Modelica.Thermal.FluidHeatFlow</a></td>
          <td valign=\"top\"> Simple components for 1-dim., incompressible thermo-fluid flow
                   to model coolant flows, e.g., of electrical machines.
                   Components can be connected arbitrarily together (= ideal mixing
                   at connection points) and fluid may reverse direction of flow.
</td><tr>
</table>
<p>
The following <b>changes</b> have been performed in the
<b>Modelica.Mechanics.MultiBody</b> library:
</p>
<ul>
<li> Component MultiBody.World has a new parameter
         <b>driveTrainMechanics3D</b>. If set to <b>true</b>, 3D mechanical effects
         of MultiBody.Parts.Mounting1D/Rotor1D/BevelGear1D are taken into account. If set to
         <b>false</b> (= default), 3D mechanical effects in these elements
         are not taken into account and the
         frame connectors to connect to 3D parts are disabled (all
         connections to such a disabled connector are also disabled, due to the
         new feature of conditional declarations in Modelica language 2.2)</li>
<li> All references to \"MultiBody.xxx\" have
         been changed to \"Modelica.Mechanics.MultiBodys.xxx\" in order that after
         copying of a component outside of the Modelica library, the references
         still remain valid.</li>
</ul>
</html>
"));
      end Version_2_2;
      class Version_2_1 "Version 2.1 (Nov. 11, 2004)"
        annotation(Documentation(info = "<html>

<p> This is a major change with respect to previous versions of the
        Modelica Standard Library, because <b>many new libraries</b> and components
        are included and because the input/output blocks (Modelica.Blocks)
        have been considerably simplified:
</p>
<ul>
<li> An input/output connector is defined <b>without</b> a hierarchy (this is possible
         due to new features of the Modelica language). For example, the input
         signal of a block \"FirstOrder\" was previously accessed as \"FirstOrder.inPort.signal[1]\".
         Now it is accessed as \"FirstOrder.u\". This simplifies the understanding and usage
         especially for beginners.</li>
<li> De-vectorized the <b>Modelica.Blocks</b> library. All blocks in the
         Modelica.Blocks library are now scalar blocks. As a result,
         the parameters of the Blocks are scalars and no vectors any
         more. For example, a parameter \"amplitude\" that might had
         a value of \"{1}\" previously, has now a value of \"1\". This simplifies
         the understanding and usage especially for beginners.<br>
         If a vector of blocks is needed, this can be easily
         accomplished by adding a dimension to the instance. For example
         \"Constant const[3](k={1,2,3}\" defines three Constant blocks.
         An additional advantage of the new approach is that
         the implementation of Modelica.Blocks is much simpler and is easier to
         understand.
</li>
</ul>

<p>
The discussed changes of Modelica.Blocks are not backward compatible.
A script to <b>automatically</b> convert models to this new version is
provided. There might be rare cases, where this script does not convert.
In this case models have to be manually converted.
In any case you should make a back-up copy of your model
before automatic conversion is performed.
</p>
<p>
The following <b>new libraries</b> have been added:
</p>
<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Electrical.Digital\">Modelica.Electrical.Digital</a></td>
          <td valign=\"top\">Digital electrical components based on 2-,3-,4-, and 9-valued logic<br>
                  according to the VHDL standard</td><tr>
<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Electrical.Machines\">Modelica.Electrical.Machines</a></td>
          <td valign=\"top\">Asynchronous, synchronous and DC motor and generator models</td><tr>
<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Math.Matrices\">Modelica.Math.Matrices</a></td>
          <td valign=\"top\">Functions operatinng on matrices such as solve() (A*x=b), leastSquares(),<br>
                  norm(), LU(), QR(),  eigenValues(), singularValues(), exp(), ...</td><tr>
<tr><td valign=\"top\"><a href=\"Modelica://Modelica.StateGraph\">Modelica.StateGraph</a></td>
          <td valign=\"top\"> Modeling of <b>discrete event</b> and <b>reactive</b> systems in a convenient way using<br>
                   <b>hierarchical state machines</b> and <b>Modelica</b> as <b>action language</b>. <br>
                   It is based on JGraphChart and Grafcet and  has a similar modeling power as <br>
                   StateCharts. It avoids deficiences of usually used action languages. <br>
                   This library makes the ModelicaAdditions.PetriNets library obsolet.</td></tr>
<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Utilities.Files\">Modelica.Utilities.Files</a></td>
          <td valign=\"top\">Functions to operate on files and directories (copy, move, remove files etc.)</td></tr>
<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Utilities.Streams\">Modelica.Utilities.Streams</a></td>
          <td valign=\"top\">Read from files and write to files (print, readLine, readFile, error, ...)</td></tr>
<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Utilities.Strings\">Modelica.Utilities.Strings</a></td>
          <td valign=\"top\">Operations on strings (substring, find, replace, sort, scanToken, ...)</td></tr>
<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Utilities.System\">Modelica.Utilities.System</a></td>
          <td valign=\"top\">Get/set current directory, get/set environment variable, execute shell command, etc.</td></tr>
</table>
<p>
The following existing libraries outside of the Modelica standard library
have been improved and added as <b>new libraries</b>
(models using the previous libraries are automatically converted
to the new sublibraries inside package Modelica):
</p>
<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Blocks.Discrete\">Modelica.Blocks.Discrete</a></td>
          <td valign=\"top\"> Discrete input/output blocks with fixed sample period<br>
                   (from ModelicaAdditions.Blocks.Discrete)</td></tr>
<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Blocks.Logical\">Modelica.Blocks.Logical</a></td>
          <td valign=\"top\"> Logical components with Boolean input and output signals<br>
                   (from ModelicaAdditions.Blocks.Logical)</td></tr>
<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Blocks.Nonlinear\">Modelica.Blocks.Nonlinear</a></td>
          <td valign=\"top\"> Discontinuous or non-differentiable algebraic control blocks such as variable limiter,<br>
                   fixed, variable and Pade delay etc. (from ModelicaAdditions.Blocks.Nonlinear)</td></tr>
<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Blocks.Routing\">Modelica.Blocks.Routing</a></td>
          <td valign=\"top\"> Blocks to combine and extract signals, such as multiplexer<br>
                   (from ModelicaAdditions.Blocks.Multiplexer)</td></tr>
<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Blocks.Tables\">Modelica.Blocks.Tables</a></td>
          <td valign=\"top\"> One and two-dimensional interpolation in tables. CombiTimeTable is available<br>
                   in Modelica.Blocks.Sources (from ModelicaAdditions.Tables)</td></tr>
<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody\">Modelica.Mechanics.MultiBody</a></td>
          <td valign=\"top\"> Components to model the movement of 3-dimensional mechanical systems. Contains <br>
                   body, joint, force and sensor components, analytic handling of kinematic loops,<br>
                   force elements with mass, series/parallel connection of 3D force elements etc.<br>
                   (from MultiBody 1.0 where the new signal connectors are used;<br>
                   makes the ModelicaAdditions.MultiBody library obsolet)</td></tr>
</table>
<p>
As a result, the ModelicaAdditions library is obsolet, because all components
are either included in the Modelica library or are replaced by much more
powerful libraries (MultiBody, StateGraph).
</p>
<p>
The following <b>new components</b> have been added to <b>existing</b> libraries.
</p>
<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td colspan=\"2\"><b>Modelica.Blocks.Logical.</b></td></tr>
<tr><td valign=\"top\">Pre</td>
          <td valign=\"top\">y = pre(u): Breaks algebraic loops by an infinitesimal small <br>
                  time delay (event iteration continues until u = pre(u))</td></tr>
<tr><td valign=\"top\">Edge</td>
          <td valign=\"top\">y = edge(u): Output y is true, if the input u has a rising edge </td></tr>
<tr><td valign=\"top\">FallingEdge</td>
          <td valign=\"top\">y = edge(not u): Output y is true, if the input u has a falling edge </td></tr>
<tr><td valign=\"top\">Change</td>
          <td valign=\"top\">y = change(u): Output y is true, if the input u has a rising or falling edge </td></tr>
<tr><td valign=\"top\">GreaterEqual</td>
          <td valign=\"top\">Output y is true, if input u1 is greater or equal as input u2 </td></tr>
<tr><td valign=\"top\">Less</td>
          <td valign=\"top\">Output y is true, if input u1 is less as input u2 </td></tr>
<tr><td valign=\"top\">LessEqual</td>
          <td valign=\"top\">Output y is true, if input u1 is less or equal as input u2 </td></tr>
<tr><td valign=\"top\">Timer</td>
          <td valign=\"top\">Timer measuring the time from the time instant where the <br>
                  Boolean input became true </td></tr>
<tr><td colspan=\"2\"><b>Modelica.Blocks.Math.</b></td></tr>
<tr><td valign=\"top\">BooleanToReal</td>
          <td valign=\"top\">Convert Boolean to Real signal</td></tr>
<tr><td valign=\"top\">BooleanToInteger</td>
          <td valign=\"top\">Convert Boolean to Integer signal</td></tr>
<tr><td valign=\"top\">RealToBoolean</td>
          <td valign=\"top\">Convert Real to Boolean signal</td></tr>
<tr><td valign=\"top\">IntegerToBoolean</td>
          <td valign=\"top\">Convert Integer to Boolean signal</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Blocks.Sources.</b></td></tr>
<tr><td valign=\"top\">RealExpression</td>
          <td valign=\"top\">Set output signal to a time varying Real expression</td></tr>
<tr><td valign=\"top\">IntegerExpression</td>
          <td valign=\"top\">Set output signal to a time varying Integer expression</td></tr>
<tr><td valign=\"top\">BooleanExpression</td>
          <td valign=\"top\">Set output signal to a time varying Boolean expression</td></tr>
<tr><td valign=\"top\">BooleanTable</td>
          <td valign=\"top\">Generate a Boolean output signal based on a vector of time instants</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.</b></td></tr>
<tr><td valign=\"top\">Frames.from_T2</td>
          <td valign=\"top\">Return orientation object R from transformation matrix T and its derivative der(T)</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\">LinearSpeedDependentTorque</td>
          <td valign=\"top\">Linear dependency of torque versus speed (acts as load torque)</td></tr>
<tr><td valign=\"top\">QuadraticSpeedDependentTorque</td>
          <td valign=\"top\">Quadratic dependency of torque versus speed (acts as load torque)</td></tr>
<tr><td valign=\"top\">ConstantTorque</td>
          <td valign=\"top\">Constant torque, not dependent on speed (acts as load torque)</td></tr>
<tr><td valign=\"top\">ConstantSpeed</td>
          <td valign=\"top\">Constant speed, not dependent on torque (acts as load torque)</td></tr>
<tr><td valign=\"top\">TorqueStep</td>
          <td valign=\"top\">Constant torque, not dependent on speed (acts as load torque)</td></tr>
</table>
<p>
The following <b>bugs</b> have been <b>corrected</b>:
</p>
<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td colspan=\"2\"><b>Modelica.Mechanics.MultiBody.Forces.</b></td></tr>
<tr><td valign=\"top\">LineForceWithMass<br>
                  Spring</td>
          <td valign=\"top\">If mass of the line force or spring component is not zero, the<br>
                  mass was (implicitly) treated as \"mass*mass\" instead of as \"mass\"</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\">Speed</td>
          <td valign=\"top\">If parameter exact=<b>false</b>, the filter was wrong<br>
                  (position was filtered and not the speed input signal)</td></tr>
</table>
<p>
Other changes:
</p>
<ul>
<li> All connectors are now smaller in the diagram layer. This gives
         a nicer layout when connectors and components are used together
         in a diagram</li>
<li> Default instance names are defined for all connectors, according
         to a new annotation introduced in Modelica 2.1. For example,
         when dragging connector \"Flange_a\" from the Rotational library to
         the diagram layer, the default connector instance name is
         \"flange_a\" and not \"Flange_a1\".</li>
<li> The Modelica.Mechanics.Rotational connectors are changed from
         a square to a circle</li>
<li> The Modelica.Mechanics.Translational connectors are changed from a
         green to a dark green color in order that connection lines
         can be better seen, especially when printed.</li>
<li> The Modelica.Blocks connectors for Real signals are changed from
         blue to dark blue in order to distinguish them from electrical signals.</li>
</ul>
</html>
"));
      end Version_2_1;
      class Version_1_6 "Version 1.6 (June 21, 2004)"
        annotation(Documentation(info = "<html>

<p> Added 1 new library (Electrical.MultiPhase), 17 new components,
        improved 3 existing components
        in the Modelica.Electrical library and improved 3 types
        in the Modelica.SIunits library. Furthermore,
        this User's Guide has been started. The improvements
        in more detail:
</p>
<p>
<b>New components</b>
</p>
<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Basic.</b></td></tr>
<tr><td valign=\"top\">SaturatingInductor</td>
          <td valign=\"top\">Simple model of an inductor with saturation</td></tr>
<tr><td valign=\"top\">VariableResistor</td>
          <td valign=\"top\">Ideal linear electrical resistor with variable resistance</td></tr>
<tr><td valign=\"top\">VariableConductor</td>
          <td valign=\"top\">Ideal linear electrical conductor with variable conductance</td></tr>
<tr><td valign=\"top\">VariableCapacitor</td>
          <td valign=\"top\">Ideal linear electrical capacitor with variable capacitance</td></tr>
<tr><td valign=\"top\">VariableInductor</td>
          <td valign=\"top\">Ideal linear electrical inductor with variable inductance</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Semiconductors.</b></td></tr>
<tr><td valign=\"top\">HeadingDiode</td>
          <td valign=\"top\">Simple diode with heating port</td></tr>
<tr><td valign=\"top\">HeadingNMOS</td>
          <td valign=\"top\">Simple MOS Transistor with heating port</td></tr>
<tr><td valign=\"top\">HeadingPMOS</td>
          <td valign=\"top\">Simple PMOS Transistor with heating port</td></tr>
<tr><td valign=\"top\">HeadingNPN</td>
          <td valign=\"top\">Simple NPN BJT according to Ebers-Moll with heating port</td></tr>
<tr><td valign=\"top\">HeadingPNP</td>
          <td valign=\"top\">Simple PNP BJT according to Ebers-Moll with heating port</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.MultiPhase</b><br>
          A new library for multi-phase electrical circuits</td></tr>
</table>
<p>
<b>New examples</b>
</p>
<p>
The following new examples have been added to
Modelica.Electrical.Analog.Examples:
</p>
<p>
CharacteristicThyristors,
CharacteristicIdealDiodes,
HeatingNPN_OrGate,
HeatingMOSInverter,
HeatingRectifier,
Rectifier,
ShowSaturatingInductor
ShowVariableResistor
</p>
<p>
<b>Improved existing components</b>
</p>
<p>In the library Modelica.Electrical.Analog.Ideal,
a knee voltage has been introduced for the components
IdealThyristor, IdealGTOThyristor, IdealDiode in order
that the approximation of these ideal elements is improved
with not much computational effort.</p>
<p> In the Modelica.SIunits library, the following changes
        have been made:</p>
<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td valign=\"top\">Inductance</td>
          <td valign=\"top\">min=0 removed</td></tr>
<tr><td valign=\"top\">SelfInductance</td>
          <td valign=\"top\">min=0 added</td></tr>
<tr><td valign=\"top\">ThermodynamicTemperature</td>
          <td valign=\"top\">min=0 added</td></tr>
</table>
</html>
"));
      end Version_1_6;
      class Version_1_5 "Version 1.5 (Dec. 16, 2002)"
        annotation(Documentation(info = "<html>

<p> Added 55 new components. In particular, added new package
        <b>Thermal.HeatTransfer</b> for modeling of lumped
        heat transfer, added model <b>LossyGear</b> in Mechanics.Rotational
        to model gear efficiency and bearing friction according to a new
        theory in a robust way, added 10 new models in Electrical.Analog and
        added several other new models and improved existing models.
</p>
<p>
<b>New components</b>
</p>
<table border=\"1\" cellspacing=0 cellpadding=2>
<tr><td colspan=\"2\"><b>Modelica.Blocks.</b></td></tr>
<tr><td valign=\"top\">Continuous.Der</td><td valign=\"top\">Derivative of input (= analytic differentations)</td></tr>
<tr><td valign=\"top\"><b><i>Examples</i></b></td><td valign=\"top\">Demonstration examples of the components of this package</td></tr>
<tr><td valign=\"top\">Nonlinear.VariableLimiter</td><td valign=\"top\">Limit the range of a signal with variable limits</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Blocks.Interfaces.</b></td></tr>
<tr><td valign=\"top\">RealPort</td><td valign=\"top\">Real port (both input/output possible)</td></tr>
<tr><td valign=\"top\">IntegerPort</td><td valign=\"top\">Integer port (both input/output possible)</td></tr>
<tr><td valign=\"top\">BooleanPort</td><td valign=\"top\">Boolean port (both input/output possible)</td></tr>
<tr><td valign=\"top\">SIMO</td><td valign=\"top\">Single Input Multiple Output continuous control block</td></tr>
<tr><td valign=\"top\">IntegerBlockIcon</td><td valign=\"top\">Basic graphical layout of Integer block</td></tr>
<tr><td valign=\"top\">IntegerMO</td><td valign=\"top\">Multiple Integer Output continuous control block</td></tr>
<tr><td valign=\"top\">IntegerSignalSource</td><td valign=\"top\">Base class for continuous Integer signal source</td></tr>
<tr><td valign=\"top\">IntegerMIBooleanMOs</td><td valign=\"top\">Multiple Integer Input Multiple Boolean Output continuous control block with same number of inputs and outputs</td></tr>
<tr><td valign=\"top\">BooleanMIMOs</td><td valign=\"top\">Multiple Input Multiple Output continuous control block with same number of inputs and outputs of boolean type</td></tr>
<tr><td valign=\"top\"><b><i>BusAdaptors</i></b></td><td valign=\"top\">Components to send signals to the bus or receive signals from the bus</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Blocks.Math.</b></td></tr>
<tr><td valign=\"top\">RealToInteger</td><td valign=\"top\">Convert real to integer signals</td></tr>
<tr><td valign=\"top\">IntegerToReal</td><td valign=\"top\">Convert integer to real signals</td></tr>
<tr><td valign=\"top\">Max</td><td valign=\"top\">Pass through the largest signal</td></tr>
<tr><td valign=\"top\">Min</td><td valign=\"top\">Pass through the smallest signal</td></tr>
<tr><td valign=\"top\">Edge</td><td valign=\"top\">Indicates rising edge of boolean signal</td></tr>
<tr><td valign=\"top\">BooleanChange</td><td valign=\"top\">Indicates boolean signal changing</td></tr>
<tr><td valign=\"top\">IntegerChange</td><td valign=\"top\">Indicates integer signal changing</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Blocks.Sources.</b></td></tr>
<tr><td valign=\"top\">IntegerConstant</td><td valign=\"top\">Generate constant signals of type Integer</td></tr>
<tr><td valign=\"top\">IntegerStep</td><td valign=\"top\">Generate step signals of type Integer</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Basic.</b></td></tr>
<tr><td valign=\"top\">HeatingResistor</td><td valign=\"top\">Temperature dependent electrical resistor</td></tr>
<tr><td valign=\"top\">OpAmp</td><td valign=\"top\">Simple nonideal model of an OpAmp with limitation</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Ideal.</b></td></tr>
<tr><td valign=\"top\">IdealCommutingSwitch</td><td valign=\"top\">Ideal commuting switch</td></tr>
<tr><td valign=\"top\">IdealIntermediateSwitch</td><td valign=\"top\">Ideal intermediate switch</td></tr>
<tr><td valign=\"top\">ControlledIdealCommutingSwitch</td><td valign=\"top\">Controlled ideal commuting switch</td></tr>
<tr><td valign=\"top\">ControlledIdealIntermediateSwitch</td><td valign=\"top\">Controlled ideal intermediate switch</td></tr>
<tr><td valign=\"top\">IdealOpAmpLimited</td><td valign=\"top\">Ideal operational amplifier with limitation</td></tr>
<tr><td valign=\"top\">IdealOpeningSwitch</td><td valign=\"top\">Ideal opener</td></tr>
<tr><td valign=\"top\">IdealClosingSwitch</td><td valign=\"top\">Ideal closer</td></tr>
<tr><td valign=\"top\">ControlledIdealOpeningSwitch</td><td valign=\"top\">Controlled ideal opener</td></tr>
<tr><td valign=\"top\">ControlledIdealClosingSwitch</td><td valign=\"top\">Controlled ideal closer</td></tr>

<tr><td colspan=\"2\"><b>Modelica.Electrical.Analog.Lines.</b></td></tr>
<tr><td valign=\"top\">TLine1</td><td valign=\"top\">Lossless transmission line (Z0, TD)</td></tr>
<tr><td valign=\"top\">TLine2</td><td valign=\"top\">Lossless transmission line (Z0, F, NL)</td></tr>
<tr><td valign=\"top\">TLine2</td><td valign=\"top\">Lossless transmission line (Z0, F)</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Icons.</b></td></tr>
<tr><td valign=\"top\">Function</td><td valign=\"top\">Icon for a function</td></tr>
<tr><td valign=\"top\">Record</td><td valign=\"top\">Icon for a record</td></tr>
<tr><td valign=\"top\">Enumeration</td><td valign=\"top\">Icon for an enumeration</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Math.</b></td></tr>
<tr><td valign=\"top\">tempInterpol2</td><td valign=\"top\">temporary routine for vectorized linear interpolation (will be removed)</td></tr>
<tr><td colspan=\"2\"><b>Modelica.Mechanics.Rotational.</b></td></tr>
<tr><td valign=\"top\">Examples.LossyGearDemo1</td><td valign=\"top\">Example to show that gear efficiency may lead to stuck motion</td></tr>
<tr><td valign=\"top\">Examples.LossyGearDemo2</td><td valign=\"top\">Example to show combination of LossyGear and BearingFriction</td></tr>
<tr><td valign=\"top\">LossyGear</td><td valign=\"top\">Gear with mesh efficiency and bearing friction (stuck/rolling possible)</td></tr>
<tr><td valign=\"top\">Gear2</td><td valign=\"top\">Realistic model of a gearbox (based on LossyGear)</td></tr>
<tr><td colspan=\"2\"><b>Modelica.SIunits.</b></td></tr>
<tr><td valign=\"top\"><b><i>Conversions</i></b></td><td valign=\"top\">Conversion functions to/from non SI units and type definitions of non SI units</td></tr>
<tr><td valign=\"top\">EnergyFlowRate</td><td valign=\"top\">Same definition as <i>Power</i></td></tr>
<tr><td valign=\"top\">EnthalpyFlowRate</td><td valign=\"top\"><tt>Real (final quantity=\"EnthalpyFlowRate\", final unit=\"W\")</tt></td></tr>
<tr><td colspan=\"2\"><b>Modelica.</b></td></tr>
<tr><td valign=\"top\"><b><i>Thermal.HeatTransfer</i></b></td><td valign=\"top\">1-dimensional heat transfer with lumped elements</td></tr>
<tr><td colspan=\"2\"><b>ModelicaAdditions.Blocks.Discrete.</b></td></tr>
<tr><td valign=\"top\">TriggeredSampler</td><td valign=\"top\">Triggered sampling of continuous signals</td></tr>
<tr><td valign=\"top\">TriggeredMax</td><td valign=\"top\">Compute maximum, absolute value of continuous signal at trigger instants</td></tr>
<tr><td colspan=\"2\"><b>ModelicaAdditions.Blocks.Logical.Interfaces.</b></td></tr>
<tr><td valign=\"top\">BooleanMIRealMOs</td><td valign=\"top\">Multiple Boolean Input Multiple Real Output continuous control block with same number of inputs and outputs</td></tr>
<tr><td valign=\"top\">RealMIBooleanMOs</td><td valign=\"top\">Multiple Real Input Multiple Boolean Output continuous control block with same number of inputs and outputs</td></tr>
<tr><td colspan=\"2\"><b>ModelicaAdditions.Blocks.Logical.</b></td></tr>
<tr><td valign=\"top\">TriggeredTrapezoid</td><td valign=\"top\">Triggered trapezoid generator</td></tr>
<tr><td valign=\"top\">Hysteresis</td><td valign=\"top\">Transform Real to Boolean with Hysteresis</td></tr>
<tr><td valign=\"top\">OnOffController</td><td valign=\"top\">On-off controller</td></tr>
<tr><td valign=\"top\">Compare</td><td valign=\"top\">True, if signal of inPort1 is larger than signal of inPort2</td></tr>
<tr><td valign=\"top\">ZeroCrossing</td><td valign=\"top\">Trigger zero crossing of input signal</td></tr>
<tr><td colspan=\"2\"><b>ModelicaAdditions.</b></td></tr>
<tr><td valign=\"top\">Blocks.Multiplexer.Extractor</td><td valign=\"top\">Extract scalar signal out of signal vector dependent on IntegerRealInput index</td></tr>
<tr><td valign=\"top\">Tables.CombiTable1Ds</td><td valign=\"top\">Table look-up in one dimension (matrix/file) with only single input</td></tr>
</table>
<p>
<b>Package-specific Changes</b>
</p>
<ul>
<li>All example models made <b>encapsulated</b></li>
<li>Upper case constants changed to lower case (cf. Modelica.Constants)</li>
<li>Introduced Modelica.SIunits.Wavelength due to typo \"Wavelenght\"</li>
<li>Introduced ModelicaAdditions.Blocks.Logical.Interfaces.Comparison due to typo \"Comparision\"</li>
<li>Changed these components of *.Blocks to <tt>block</tt> class, which have not been already of block type</li>
<li>Changed *.Interfaces.RelativeSensor to <tt>partial</tt> models</li>
</ul>
<p>
<b>Class-specific Changes</b>
</p>
<p>
<i>Modelica.SIunits</i>
</p>
<p>Removed <tt>final</tt> from quantity attribute for <i>Mass</i> and <i>MassFlowRate</i>.</p>
<p>
<i>Modelica.Blocks.Math.Sum</i>
</p>
<p>Implemented avoiding algorithm section, which would lead to expensive function calls.</p>
<p><i>Modelica.Blocks.Sources.Step</i></p>
<pre>
block Step \"Generate step signals of type Real\"
        parameter Real height[:]={1} \"Heights of steps\";
<b> // parameter Real offset[:]={0} \"Offsets of output signals\";
// parameter SIunits.Time startTime[:]={0} \"Output = offset for time < startTime\";
// extends Interfaces.MO          (final nout=max([size(height, 1); size(offset, 1); size(startTime, 1)]));
        extends Interfaces.SignalSource(final nout=max([size(height, 1); size(offset, 1); size(startTime, 1)]));</b>
</pre>
<p><i>Modelica.Blocks.Sources.Exponentials</i></p>
<p>Replaced usage of built-in function <tt>exp</tt> by Modelica.Math.exp.</p>
<p><i>Modelica.Blocks.Sources.TimeTable</i></p>
<p>Interface definition changed from</p>
<pre>    parameter Real table[:, :]=[0, 0; 1, 1; 2, 4] \"Table matrix (time = first column)\";
</pre>
<p>to</p>
<pre>    parameter Real table[:, <b>2</b>]=[0, 0; 1, 1; 2, 4] \"Table matrix (time = first column)\";
</pre>
<p>Did the same for subfunction <i>getInterpolationCoefficients</i>.</p>
<p>Bug in <i>getInterpolationCoefficients</i> for startTime <> 0 fixed:</p>
<pre>        ...
                end if;
          end if;
          <b>// Take into account startTime \"a*(time - startTime) + b\"
          b := b - a*startTime;</b>
        end getInterpolationCoefficients;
</pre>
<p><i>Modelica.Blocks.Sources.BooleanStep</i></p>
<pre>
block BooleanStep \"Generate step signals of type Boolean\"
        parameter SIunits.Time startTime[:]={0} \"Time instants of steps\";
        <b>parameter Boolean startValue[size(startTime, 1)]=fill(false, size(startTime, 1)) \"Output before startTime\";</b>
        extends Interfaces.BooleanSignalSource(final nout=size(startTime, 1));
equation
        for i in 1:nout loop
<b>//   outPort.signal[i] = time >= startTime[i];
          outPort.signal[i] = if time >= startTime[i] then not startValue[i] else startValue[i];</b>
        end for;
end BooleanStep;
</pre>
<p>
<i>Modelica.Electrical.Analog</i></p>
<p>Corrected table of values and default for Beta by dividing them by 1000
(consistent with the values used in the NAND-example model):
</p>
<ul>
<li>Semiconductors.PMOS</li>
<li>Semiconductors.NMOS</li>
</ul>
<p>Corrected parameter defaults, unit and description for TrapezoidCurrent.
This makes the parameters consistent with their use in the model.
Models specifying parameter values are not changed.
Models not specifying parameter values did not generate trapezoids previously.
</p>
<p>Icon layer background changed from transparent to white:</p>
<ul>
<li>Basic.Gyrator</li>
<li>Basic.EMF</li>
<li>Ideal.Idle</li>
<li>Ideal.Short</li>
</ul>
<p>Basic.Transformer: Replaced invalid escape characters '\\ ' and '\\[newline]' in documentation by '|'.</p>
<p><i>Modelica.Mechanics.Rotational</i></p>
<p>Removed arrows and names documentation from flanges in diagram layer</p>
<p><i>Modelica.Mechanics.Rotational.Interfaces.FrictionBase</i></p>
<p><i>Modelica.Mechanics.Rotational.Position</i></p>
<p>Replaced <tt>reinit</tt> by <tt>initial equation</tt></p>
<p><i>Modelica.Mechanics.Rotational.RelativeStates</i></p>
<p>Bug corrected by using modifier <tt>stateSelect = StateSelect.prefer</tt> as implementation</p>
<p><i>Modelica.Mechanics.Translational.Interfaces.flange_b</i></p>
<p>Attribute <b>fillColor=7</b> added to Rectangle on Icon layer, i.e. it is now
filled with white and not transparent any more.</p>
<p><i>Modelica.Mechanics.Translational.Position</i></p>
<p>Replaced <tt>reinit</tt> by <tt>initial equation</tt></p>
<p><i>Modelica.Mechanics.Translational.RelativeStates</i></p>
<p>Bug corrected by using modifier <tt>stateSelect = StateSelect.prefer</tt> as implementation</p>
<p><i>Modelica.Mechanics.Translational.Stop</i></p>
<p>Use <tt>stateSelect = StateSelect.prefer</tt>.</p>
<p><i>Modelica.Mechanics.Translational.Examples.PreLoad</i></p>
<p>Improved documentation and coordinate system used for example.</p>
<p><i>ModelicaAdditions.Blocks.Nonlinear.PadeDelay</i></p>
<p>Replaced <tt>reinit</tt> by <tt>initial equation</tt></p>
<p><i>ModelicaAdditions.HeatFlow1D.Interfaces</i></p>
<p>Definition of connectors <i>Surface_a</i> and <i>Surface_b</i>:<br>
<tt>flow SIunits.HeatFlux q;</tt> changed to <tt>flow SIunits.HeatFlowRate q;</tt></p>
<p><i>MultiBody.Parts.InertialSystem</i></p>
<p>Icon corrected.</p>
</html>
"));
      end Version_1_5;
      class Version_1_4 "Version 1.4 (June 28, 2001)"
        annotation(Documentation(info = "<html>

<ul>
<li>Several minor bugs fixed. </li>
<li>New models:<br>
        Modelica.Blocks.Interfaces.IntegerRealInput/IntegerRealOutput,<br>
        Modelica.Blocks.Math.TwoInputs/TwoOutputs<br>
        Modelica.Electrical.Analog.Ideal.IdealOpAmp3Pin,<br>
        Modelica.Mechanics.Rotational.Move,<br>
        Modelica.Mechanics.Translational.Move.<br>
        </li>
</ul>
<hr>
<h4>Version 1.4.1beta1 (February 12, 2001)</h4>
<p> Adapted to Modelica 1.4</p>
<hr>
<h4>Version 1.3.2beta2 (June 20, 2000)</h4>
<ul>
        <li>New subpackage Modelica.Mechanics.<b>Translational</b></li>
        <li>Changes to Modelica.Mechanics.<b>Rotational</b>:<br>
           New elements:
<pre>
IdealGearR2T    Ideal gear transforming rotational in translational motion.
Position        Forced movement of a flange with a reference angle
                                   given as input signal
RelativeStates  Definition of relative state variables
</pre>
</li>
        <li>Changes to Modelica.<b>SIunits</b>:<br>
          Introduced new types:<br>
          type Temperature = ThermodynamicTemperature;<br>
          types DerDensityByEnthalpy, DerDensityByPressure,
          DerDensityByTemperature, DerEnthalpyByPressure,
          DerEnergyByDensity, DerEnergyByPressure<br>
          Attribute \"final\" removed from min and max values
          in order that these values can still be changed to narrow
          the allowed range of values.<br>
          Quantity=\"Stress\" removed from type \"Stress\", in order
          that a type \"Stress\" can be connected to a type \"Pressure\".</li>
        <li>Changes to Modelica.<b>Icons</b>:<br>
           New icons for motors and gearboxes.</li>
        <li>Changes to Modelica.<b>Blocks.Interfaces</b>:<br>
           Introduced a replaceable signal type into
           Blocks.Interfaces.RealInput/RealOutput:
<pre>
replaceable type SignalType = Real
</pre>
           in order that the type of the signal of an input/output block
           can be changed to a physical type, for example:
<pre>
Sine sin1(outPort(redeclare type SignalType=Modelica.SIunits.Torque))
</pre>
</li>
</ul>
<hr>
<h4>Version 1.3.1 (Dec. 13, 1999)</h4>
<p>
First official release of the library.
</p>
</html>
"));
      end Version_1_4;
    end ReleaseNotes;
    class ModelicaLicense2 "Modelica License 2"
      annotation(Documentation(info = "<html>

<p>All files in this directory (Modelica) and in all
subdirectories, especially all files that build package \"Modelica\" and all
files in \"Modelica\\C-sources\", \"Modelica\\help\",
\"Modelica\\Images\" are licensed by the <b><u>Modelica Association</u></b> under the
<b><u>Modelica License 2</u></b> (with exception of files
\"Modelica\\C-sources\\win32_dirent.*\").&nbsp;</p>

<p style=\"margin-left: 40px;\"><b>Licensor:</b><br>
Modelica Association<br>
(Ideella F&ouml;reningar 822003-8858 in Link&ouml;ping) <br>
c/o PELAB, IDA, Link&ouml;pings Universitet <br>
S-58183 Link&ouml;ping <br>
Sweden<br>
email: Board@Modelica.org<br>
web: <a href=\"http://www.Modelica.org\">http://www.Modelica.org</a></p>

<p style=\"margin-left: 40px;\"><b>Copyright notices of the files:</b><br>
Copyright &copy; 1998-2009,
ABB, arsenal research, T.&nbsp;B&ouml;drich, DLR, Dynasim, Fraunhofer, Modelon, TU
Hamburg-Harburg, Politecnico di Milano.<br>
<br>
</p>

<p> <a href=\"#ModelicaLicense2\">The
Modelica License 2</a><br>
<a href=\"#2.%20Frequently%20Asked%20Questions%7Coutline\">Frequently
Asked Questions</a><br>
<br>
</p>

<hr>
<h4><a name=\"ModelicaLicense2\"></a>The Modelica License 2</h4>

<p>
<b>Preamble.</b> The goal of this license is that Modelica related
model libraries, software, images, documents, data files etc. can be
used freely in the original or a modified form, in open source and in
commercial environments (as long as the license conditions below are
fulfilled, in particular sections 2c) and 2d). The Original Work is
provided free of charge and the use is completely at your own risk.
Developers of free Modelica packages are encouraged to utilize this
license for their work.</p>

<p>
The Modelica License applies to any Original Work that contains the
following licensing notice adjacent to the copyright notice(s) for
this Original Work:</p>
<p><b>Licensed
by the Modelica Association under the Modelica License 2</b></p>

<p><b>1. Definitions.</b></p>
<ol>
 <li>&ldquo;License&rdquo; is this Modelica License.</li>

 <li>
 &ldquo;Original Work&rdquo; is any work of authorship, including
 software, images, documents, data files, that contains the above
 licensing notice or that is packed together with a licensing notice
 referencing it.</li>

 <li>
 &ldquo;Licensor&rdquo; is the provider of the Original Work who has
 placed this licensing notice adjacent to the copyright notice(s) for
 the Original Work. The Original Work is either directly provided by
 the owner of the Original Work, or by a licensee of the owner.</li>

 <li>
 &ldquo;Derivative Work&rdquo; is any modification of the Original
 Work which represents, as a whole, an original work of authorship.
 For the matter of clarity and as examples: </li>

 <ol>
  <li>
  Derivative Work shall not include work that remains separable from
  the Original Work, as well as merely extracting a part of the
  Original Work without modifying it.</li>

  <li>
  Derivative Work shall not include (a) fixing of errors and/or (b)
  adding vendor specific Modelica annotations and/or (c) using a
  subset of the classes of a Modelica package, and/or (d) using a
  different representation, e.g., a binary representation.</li>

  <li>
  Derivative Work shall include classes that are copied from the
  Original Work where declarations, equations or the documentation
  are modified.</li>

  <li>
  Derivative Work shall include executables to simulate the models
  that are generated by a Modelica translator based on the Original
  Work (of a Modelica package).</li>
 </ol>

 <li>
 &ldquo;Modified Work&rdquo; is any modification of the Original Work
 with the following exceptions: (a) fixing of errors and/or (b)
 adding vendor specific Modelica annotations and/or (c) using a
 subset of the classes of a Modelica package, and/or (d) using a
 different representation, e.g., a binary representation.</li>

 <li>
 &quot;Source Code&quot; means the preferred form of the Original
 Work for making modifications to it and all available documentation
 describing how to modify the Original Work.</li>

 <li>
 &ldquo;You&rdquo; means an individual or a legal entity exercising
 rights under, and complying with all of the terms of, this License.</li>

 <li>
 &ldquo;Modelica package&rdquo; means any Modelica library that is
 defined with the<br>&ldquo;<FONT FACE=\"Courier New, monospace\"><FONT SIZE=2 STYLE=\"font-size: 9pt\"><b>package</b></FONT></FONT><FONT FACE=\"Courier New, monospace\"><FONT SIZE=2 STYLE=\"font-size: 9pt\">
 &lt;Name&gt; ... </FONT></FONT><FONT FACE=\"Courier New, monospace\"><FONT SIZE=2 STYLE=\"font-size: 9pt\"><b>end</b></FONT></FONT><FONT FACE=\"Courier New, monospace\"><FONT SIZE=2 STYLE=\"font-size: 9pt\">
 &lt;Name&gt;;</FONT></FONT>&ldquo; Modelica language element.</li>
</ol>

<p>
<b>2. Grant of Copyright License.</b> Licensor grants You a
worldwide, royalty-free, non-exclusive, sublicensable license, for
the duration of the copyright, to do the following:</p>

<ol>
 <li><p>
 To reproduce the Original Work in copies, either alone or as part of
 a collection.</li></p>
 <li><p>
 To create Derivative Works according to Section 1d) of this License.</li></p>
 <li><p>
 To distribute or communicate to the public copies of the <u>Original
 Work</u> or a <u>Derivative Work</u> under <u>this License</u>. No
 fee, neither as a copyright-license fee, nor as a selling fee for
 the copy as such may be charged under this License. Furthermore, a
 verbatim copy of this License must be included in any copy of the
 Original Work or a Derivative Work under this License.<br>      For
 the matter of clarity, it is permitted A) to distribute or
 communicate such copies as part of a (possible commercial)
 collection where other parts are provided under different licenses
 and a license fee is charged for the other parts only and B) to
 charge for mere printing and shipping costs.</li></p>
 <li><p>
 To distribute or communicate to the public copies of a <u>Derivative
 Work</u>, alternatively to Section 2c), under <u>any other license</u>
 of your choice, especially also under a license for
 commercial/proprietary software, as long as You comply with Sections
 3, 4 and 8 below. <br>      For the matter of clarity, no
 restrictions regarding fees, either as to a copyright-license fee or
 as to a selling fee for the copy as such apply.</li></p>
 <li><p>
 To perform the Original Work publicly.</li></p>
 <li><p>
 To display the Original Work publicly.</li></p>
</ol>

<p>
<b>3. Acceptance.</b> Any use of the Original Work or a
Derivative Work, or any action according to either Section 2a) to 2f)
above constitutes Your acceptance of this License.</p>

<p>
<b>4. Designation of Derivative Works and of Modified Works.
</b>The identifying designation of Derivative Work and of Modified
Work must be different to the corresponding identifying designation
of the Original Work. This means especially that the (root-level)
name of a Modelica package under this license must be changed if the
package is modified (besides fixing of errors, adding vendor specific
Modelica annotations, using a subset of the classes of a Modelica
package, or using another representation, e.g. a binary
representation).</p>

<p>
<b>5. Grant of Patent License.</b>
Licensor grants You a worldwide, royalty-free, non-exclusive, sublicensable license,
under patent claims owned by the Licensor or licensed to the Licensor by
the owners of the Original Work that are embodied in the Original Work
as furnished by the Licensor, for the duration of the patents,
to make, use, sell, offer for sale, have made, and import the Original Work
and Derivative Works under the conditions as given in Section 2.
For the matter of clarity, the license regarding Derivative Works covers
patent claims to the extent as they are embodied in the Original Work only.</p>

<p>
<b>6. Provision of Source Code.</b> Licensor agrees to provide
You with a copy of the Source Code of the Original Work but reserves
the right to decide freely on the manner of how the Original Work is
provided.<br>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;For the matter of clarity, Licensor might provide only a binary
representation of the Original Work. In that case, You may (a) either
reproduce the Source Code from the binary representation if this is
possible (e.g., by performing a copy of an encrypted Modelica
package, if encryption allows the copy operation) or (b) request the
Source Code from the Licensor who will provide it to You.</p>

<p>
<b>7. Exclusions from License Grant.</b> Neither the names of
Licensor, nor the names of any contributors to the Original Work, nor
any of their trademarks or service marks, may be used to endorse or
promote products derived from this Original Work without express
prior permission of the Licensor. Except as otherwise expressly
stated in this License and in particular in Sections 2 and 5, nothing
in this License grants any license to Licensor&rsquo;s trademarks,
copyrights, patents, trade secrets or any other intellectual
property, and no patent license is granted to make, use, sell, offer
for sale, have made, or import embodiments of any patent claims.<br>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;No license is granted to the trademarks of
Licensor even if such trademarks are included in the Original Work,
except as expressly stated in this License. Nothing in this License
shall be interpreted to prohibit Licensor from licensing under terms
different from this License any Original Work that Licensor otherwise
would have a right to license.</p>

<p>
<b>8. Attribution Rights.</b> You must retain in the Source
Code of the Original Work and of any Derivative Works that You
create, all author, copyright, patent, or trademark notices, as well
as any descriptive text identified therein as an &quot;Attribution
Notice&quot;. The same applies to the licensing notice of this
License in the Original Work. For the matter of clarity, &ldquo;author
notice&rdquo; means the notice that identifies the original
author(s). <br>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;You must cause the Source Code for any Derivative
Works that You create to carry a prominent Attribution Notice
reasonably calculated to inform recipients that You have modified the
Original Work. <br>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;In case the Original Work or Derivative Work is not provided in
Source Code, the Attribution Notices shall be appropriately
displayed, e.g., in the documentation of the Derivative Work.</p>

<p><b>9. Disclaimer
of Warranty. <br></b><u><b>The Original Work is provided under this
License on an &quot;as is&quot; basis and without warranty, either
express or implied, including, without limitation, the warranties of
non-infringement, merchantability or fitness for a particular
purpose. The entire risk as to the quality of the Original Work is
with You.</b></u><b> </b>This disclaimer of warranty constitutes an
essential part of this License. No license to the Original Work is
granted by this License except under this disclaimer.</p>

<p>
<b>10. Limitation of Liability.</b> Under no circumstances and
under no legal theory, whether in tort (including negligence),
contract, or otherwise, shall the Licensor, the owner or a licensee
of the Original Work be liable to anyone for any direct, indirect,
general, special, incidental, or consequential damages of any
character arising as a result of this License or the use of the
Original Work including, without limitation, damages for loss of
goodwill, work stoppage, computer failure or malfunction, or any and
all other commercial damages or losses. This limitation of liability
shall not apply to the extent applicable law prohibits such
limitation.</p>

<p>
<b>11. Termination.</b> This License conditions your rights to
undertake the activities listed in Section 2 and 5, including your
right to create Derivative Works based upon the Original Work, and
doing so without observing these terms and conditions is prohibited
by copyright law and international treaty. Nothing in this License is
intended to affect copyright exceptions and limitations. This License
shall terminate immediately and You may no longer exercise any of the
rights granted to You by this License upon your failure to observe
the conditions of this license.</p>

<p>
<b>12. Termination for Patent Action.</b> This License shall
terminate automatically and You may no longer exercise any of the
rights granted to You by this License as of the date You commence an
action, including a cross-claim or counterclaim, against Licensor,
any owners of the Original Work or any licensee alleging that the
Original Work infringes a patent. This termination provision shall
not apply for an action alleging patent infringement through
combinations of the Original Work under combination with other
software or hardware.</p>

<p>
<b>13. Jurisdiction.</b> Any action or suit relating to this
License may be brought only in the courts of a jurisdiction wherein
the Licensor resides and under the laws of that jurisdiction
excluding its conflict-of-law provisions. The application of the
United Nations Convention on Contracts for the International Sale of
Goods is expressly excluded. Any use of the Original Work outside the
scope of this License or after its termination shall be subject to
the requirements and penalties of copyright or patent law in the
appropriate jurisdiction. This section shall survive the termination
of this License.</p>

<p>
<b>14. Attorneys&rsquo; Fees.</b> In any action to enforce the
terms of this License or seeking damages relating thereto, the
prevailing party shall be entitled to recover its costs and expenses,
including, without limitation, reasonable attorneys' fees and costs
incurred in connection with such action, including any appeal of such
action. This section shall survive the termination of this License.</p>

<p>
<b>15. Miscellaneous.</b>
</p>
<ol>
 <li>If any
 provision of this License is held to be unenforceable, such
 provision shall be reformed only to the extent necessary to make it
 enforceable.</li>

 <li>No verbal
 ancillary agreements have been made. Changes and additions to this
 License must appear in writing to be valid. This also applies to
 changing the clause pertaining to written form.</li>

 <li>You may use the
 Original Work in all ways not otherwise restricted or conditioned by
 this License or by law, and Licensor promises not to interfere with
 or be responsible for such uses by You.</li>
</ol>

<p>
<br>
</p>

<hr>

<h4><a name=\"2. Frequently Asked Questions|outline\"></a>
Frequently Asked Questions</h4>
<p>This
section contains questions/answer to users and/or distributors of
Modelica packages and/or documents under Modelica License 2. Note,
the answers to the questions below are not a legal interpretation of
the Modelica License 2. In case of a conflict, the language of the
license shall prevail.</p>

<p><br>
</p>

<p><FONT COLOR=\"#008000\"><FONT SIZE=3><b>Using
or Distributing a Modelica </b></FONT></FONT><FONT COLOR=\"#008000\"><FONT SIZE=3><u><b>Package</b></u></FONT></FONT><FONT COLOR=\"#008000\"><FONT SIZE=3><b>
under the Modelica License 2</b></FONT></FONT></p>

<p><b>What are the main
differences to the previous version of the Modelica License?</b></p>
<ol>
 <li><p>
 Modelica License 1 is unclear whether the licensed Modelica package
 can be distributed under a different license. Version 2 explicitly
 allows that &ldquo;Derivative Work&rdquo; can be distributed under
 any license of Your choice, see examples in Section 1d) as to what
 qualifies as Derivative Work (so, version 2 is clearer).</p>
 <li><p>
 If You modify a Modelica package under Modelica License 2 (besides
 fixing of errors, adding vendor specific Modelica annotations, using
 a subset of the classes of a Modelica package, or using another
 representation, e.g., a binary representation), you must rename the
 root-level name of the package for your distribution. In version 1
 you could keep the name (so, version 2 is more restrictive). The
 reason of this restriction is to reduce the risk that Modelica
 packages are available that have identical names, but different
 functionality.</p>
 <li><p>
 Modelica License 1 states that &ldquo;It is not allowed to charge a
 fee for the original version or a modified version of the software,
 besides a reasonable fee for distribution and support<SPAN LANG=\"en-GB\">&ldquo;.
 Version 2 has a </SPAN>similar intention for all Original Work under
 <u>Modelica License 2</u> (to remain free of charge and open source)
 but states this more clearly as &ldquo;No fee, neither as a
 copyright-license fee, nor as a selling fee for the copy as such may
 be charged&rdquo;. Contrary to version 1, Modelica License 2 has no
 restrictions on fees for Derivative Work that is provided under a
 different license (so, version 2 is clearer and has fewer
 restrictions).</p>
 <li><p>
 Modelica License 2 introduces several useful provisions for the
 licensee (articles 5, 6, 12), and for the licensor (articles 7, 12,
 13, 14) that have no counter part in version 1.</p>
 <li><p>
 Modelica License 2 can be applied to all type of work, including
 documents, images and data files, contrary to version 1 that was
 dedicated for software only (so, version 2 is more general).</p>
</ol>

<p><b>Can I distribute a
Modelica package (under Modelica License 2) as part of my commercial
Modelica modeling and simulation environment?</b></p>
<p>Yes,
according to Section 2c). However, you are not allowed to charge a
fee for this part of your environment. Of course, you can charge for
your part of the environment.
</p>

<p><b>Can I distribute a
Modelica package (under Modelica License 2) under a different
license?</b></p>
<p>No.
The license of an unmodified Modelica package cannot be changed
according to Sections 2c) and 2d). This means that you cannot <u>sell</u>
copies of it, any distribution has to be free of charge.</p>

<p><b>Can I distribute a
Modelica package (under Modelica License 2) under a different license
when I first encrypt the package?</b></p>
<p>No.
Merely encrypting a package does not qualify for Derivative Work and
therefore the encrypted package has to stay under Modelica License 2.</p>

<p><b>Can I distribute a
Modelica package (under Modelica License 2) under a different license
when I first add classes to the package?</b></p>
<p>No.
The package itself remains unmodified, i.e., it is Original Work, and
therefore the license for this part must remain under Modelica
License 2. The newly added classes can be, however, under a different
license.
</p>

<p><b>Can
I copy a class out of a Modelica package (under Modelica License 2)
and include it </b><u><b>unmodified</b></u><b> in a Modelica package
under a </b><u><b>commercial/proprietary license</b></u><b>?</b></p>
<p>No,
according to article 2c). However, you can include model, block,
function, package, record and connector classes in your Modelica
package under <u>Modelica License 2</u>. This means that your
Modelica package could be under a commercial/proprietary license, but
one or more classes of it are under Modelica License 2.<br>Note, a
&ldquo;type&rdquo; class (e.g., type Angle = Real(unit=&rdquo;rad&rdquo;))
can be copied and included unmodified under a commercial/proprietary
license (for details, see the next question).</p>

<p><b>Can
I copy a type class or </b><u><b>part</b></u><b> of a model, block,
function, record, connector class, out of a Modelica package (under
Modelica License 2) and include it modified or unmodified in a
Modelica package under a </b><u><b>commercial/proprietary</b></u><b>
license</b></p>
<p>Yes,
according to article 2d), since this will in the end usually qualify
as Derivative Work. The reasoning is the following: A type class or
part of another class (e.g., an equation, a declaration, part of a
class description) cannot be utilized &ldquo;by its own&rdquo;. In
order to make this &ldquo;usable&rdquo;, you have to add additional
code in order that the class can be utilized. This is therefore
usually Derivative Work and Derivative Work can be provided under a
different license. Note, this only holds, if the additional code
introduced is sufficient to qualify for Derivative Work. Merely, just
copying a class and changing, say, one character in the documentation
of this class would be no Derivative Work and therefore the copied
code would have to stay under Modelica License 2.</p>

<p><b>Can
I copy a class out of a Modelica package (under Modelica License 2)
and include it in </b><u><b>modified </b></u><b>form in a
</b><u><b>commercial/proprietary</b></u><b> Modelica package?</b></p>
<p>Yes.
If the modification can be seen as a &ldquo;Derivative Work&rdquo;,
you can place it under your commercial/proprietary license. If the
modification does not qualify as &ldquo;Derivative Work&rdquo; (e.g.,
bug fixes, vendor specific annotations), it must remain under
Modelica License 2. This means that your Modelica package could be
under a commercial/proprietary license, but one or more parts of it
are under Modelica License 2.</p>

<p><b>Can I distribute a
&ldquo;save total model&rdquo; under my commercial/proprietary
license, even if classes under Modelica License 2 are included?</b></p>
<p>Your
classes of the &ldquo;save total model&rdquo; can be distributed
under your commercial/proprietary license, but the classes under
Modelica License 2 must remain under Modelica License 2. This means
you can distribute a &ldquo;save total model&rdquo;, but some parts
might be under Modelica License 2.</p>

<p><b>Can I distribute a
Modelica package (under Modelica License 2) in encrypted form?</b></p>
<p>Yes.
Note, if the encryption does not allow &ldquo;copying&rdquo; of
classes (in to unencrypted Modelica source code), you have to send
the Modelica source code of this package to your customer, if he/she
wishes it, according to article&nbsp;6.</p>

<p><b>Can I distribute an
executable under my commercial/proprietary license, if the model from
which the executable is generated uses models from a Modelica package
under Modelica License 2?</b></p>
<p>Yes,
according to article 2d), since this is seen as Derivative Work. The
reasoning is the following: An executable allows the simulation of a
concrete model, whereas models from a Modelica package (without
pre-processing, translation, tool run-time library) are not able to
be simulated without tool support. By the processing of the tool and
by its run-time libraries, significant new functionality is added (a
model can be simulated whereas previously it could not be simulated)
and functionality available in the package is removed (e.g., to build
up a new model by dragging components of the package is no longer
possible with the executable).</p>

<p><b>Is my modification to
a Modelica package (under Modelica License 2) a Derivative Work?</b></p>
<p>It
is not possible to give a general answer to it. To be regarded as &quot;an
original work of authorship&quot;, a derivative work must be
different enough from the original or must contain a substantial
amount of new material. Making minor changes or additions of little
substance to a preexisting work will not qualify the work as a new
version for such purposes.
</p>

<p><br>
</p>
<p><FONT COLOR=\"#008000\"><FONT SIZE=3><b>Using
or Distributing a Modelica </b></FONT></FONT><FONT COLOR=\"#008000\"><FONT SIZE=3><u><b>Document</b></u></FONT></FONT><FONT COLOR=\"#008000\"><FONT SIZE=3><b>
under the Modelica License 2</b></FONT></FONT></p>

<p>This
section is devoted especially for the following applications:</p>
<ol>
 <li><p>
 A Modelica tool extracts information out of a Modelica package and
 presents the result in form of a &ldquo;manual&rdquo; for this
 package in, e.g., html, doc, or pdf format.</p>
 <li><p>
 The Modelica language specification is a document defining the
 Modelica language. It will be licensed under Modelica License 2.</p>
 <li><p>
 Someone writes a book about the Modelica language and/or Modelica
 packages and uses information which is available in the Modelica
 language specification and/or the corresponding Modelica package.</p>
</ol>

<p><b>Can I sell a manual
that was basically derived by extracting information automatically
from a Modelica package under Modelica License 2 (e.g., a &ldquo;reference
guide&rdquo; of the Modelica Standard Library):</b></p>
<p>Yes.
Extracting information from a Modelica package, and providing it in a
human readable, suitable format, like html, doc or pdf format, where
the content is significantly modified (e.g. tables with interface
information are constructed from the declarations of the public
variables) qualifies as Derivative Work and there are no restrictions
to charge a fee for Derivative Work under alternative 2d).</p>

<p><b>Can
I copy a text passage out of a Modelica document (under Modelica
License 2) and use it </b><u><b>unmodified</b></u><b> in my document
(e.g. the Modelica syntax description in the Modelica Specification)?</b></p>
<p>Yes.
In case you distribute your document, the copied parts are still
under Modelica License 2 and you are not allowed to charge a license
fee for this part. You can, of course, charge a fee for the rest of
your document.</p>

<p><b>Can
I copy a text passage out of a Modelica document (under Modelica
License 2) and use it in </b><u><b>modified</b></u><b> form in my
document?</b></p>
<p>Yes,
the creation of Derivative Works is allowed. In case the content is
significantly modified this qualifies as Derivative Work and there
are no restrictions to charge a fee for Derivative Work under
alternative 2d).</p>

<p><b>Can I sell a printed
version of a Modelica document (under Modelica License 2), e.g., the
Modelica Language Specification?</b></p>
<p>No,
if you are not the copyright-holder, since article 2c) does not allow
a selling fee for a (in this case physical) copy. However, mere
printing and shipping costs may be recovered.</p>
</html>
"));
    end ModelicaLicense2;
    class Contact "Contact"
      annotation(Documentation(info = "<html>

<dl>
<dt>The Modelica Standard Library (this Modelica package)
 is developed by many people from different organizations (see list below).
 It is licensed under the
 <a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">Modelica License 2</a>
 by:<br>&nbsp;</dt>
<dd>Modelica Association<br>
 (Ideella F&ouml;reningar 822003-8858 in Link&ouml;ping) <br>
 c/o PELAB, IDA, Link&ouml;pings Universitet<br>
 S-58183 Link&ouml;ping<br>
 Sweden<br>
 email: <a href=\"mailto:Board@Modelica.org\">Board@Modelica.org</a><br>
 web: <a href=\"http://www.Modelica.org\">http://www.Modelica.org</a><br>&nbsp;<br>&nbsp;</dd>
</dl>

<dl>
<dt>The development of this Modelica package is organized by<br>&nbsp;</dt>
<dd><a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a><br>
 Deutsches Zentrum f&uuml;r Luft und Raumfahrt e.V. (DLR)<br>
 Institut f&uuml;r Robotik und Mechatronik<br>
 Abteilung f&uuml;r Systemdynamik und Regelungstechnik<br>
 Postfach 1116<br>
 D-82230 Wessling<br>
 Germany<br>
 email: <A HREF=\"mailto:Martin.Otter@dlr.de\">Martin.Otter@dlr.de</A><br></dd>
</dl>

<p>
Since end of 2007, the development of the sublibraries of package Modelica
is organized by personal and/or organizational <b>library officers</b> assigned by the
Modelica Association. They are responsible for the maintenance and
for the further organization of the development. Other persons may
also contribute, but the final decision for library improvements and/or changes
is performed by the responsible library officer(s). In order that a new
sublibrary or a new version of a sublibrary is ready to be released,
the responsible library officers report the changes to the members of
the Modelica Association and the library is made available for beta testing to
interested parties before a final decision. A new release of a sublibrary
is formally decided by voting of the Modelica Association members.
</p>

<p>
The following library officers are currently assigned:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><td valign=\"top\"><b>Sublibraries</b> </td>
   <td valign=\"top\"><b>Library officers</b></td>
</tr>

<tr><td valign=\"top\"> Blocks <br> Constants </td>
   <td valign=\"top\"> DLR Institute of Robotics and Mechatronics, Oberpfaffenhofen, Germany<br>
        (Martin Otter)</td>
</tr>

<tr><td valign=\"top\"> Electrical.Analog, <br> Electrical.Digital </td>
   <td valign=\"top\"> Fraunhofer Institute for Integrated Circuits, Dresden, Germany<br>
      (Christoph Clauss)</td>
</tr>

<tr><td valign=\"top\"> Electrical.Machines<br>
      Electrical.MultiPhase </td>
   <td valign=\"top\"> Anton Haumer, Consultant, St.Andrae-Woerdern, Austria, and<br>
      arsenal research, Vienna, Austria (Christian Kral)</td>
</tr>

<tr><td valign=\"top\"> Magnetic.FluxTubes </td>
   <td valign=\"top\"> Thomas Bdrich, Dresden, Germany<br>
                               (Dresden University of Technology,<br>
                               Institute of Electromechanical and Electronic Design)
</td>
</tr>

<tr><td valign=\"top\"> Fluid </td>
   <td valign=\"top\"> Politecnico di Milano (Francesco Casella), and<br>
                            R&uuml;diger Franke (ABB)</td>
</tr>

<tr><td valign=\"top\"> Icons </td>
   <td valign=\"top\"> Modelon AB, Lund, Sweden (Johan Andreasson) </td>
</tr>

<tr><td valign=\"top\"> Math </td>
   <td valign=\"top\"> DLR Institute of Robotics and Mechatronics, Oberpfaffenhofen, Germany<br>
      (Martin Otter)</td>
</tr>

<tr><td valign=\"top\"> Mechanics.MultiBody </td>
   <td valign=\"top\"> DLR Institute of Robotics and Mechatronics, Oberpfaffenhofen, Germany<br>
      (Martin Otter),<br>
       Modelon AB, Lund, Sweden (Johan Andreasson) </td>
</tr>

<tr><td valign=\"top\"> Mechanics.Rotational </td>
   <td valign=\"top\"> DLR Institute of Robotics and Mechatronics, Oberpfaffenhofen, Germany<br>
      (Martin Otter)<br>
      Anton Haumer, Consultant, St.Andrae-Woerdern, Austria,<br>
      arsenal research, Vienna, Austria (Christian Kral),<br>
      Modelon AB, Lund, Sweden (Johan Andreasson)</td>
</tr>

<tr><td valign=\"top\"> Mechanics.Translational </td>
   <td valign=\"top\"> Anton Haumer, Consultant, St.Andrae-Woerdern, Austria,<br>
      arsenal research, Vienna, Austria (Christian Kral), <br>
      DLR Institute of Robotics and Mechatronics, Oberpfaffenhofen, Germany<br>
      (Martin Otter)<br>
       Modelon AB, Lund, Sweden (Johan Andreasson)</td>
</tr>


<tr><td valign=\"top\"> Media </td>
   <td valign=\"top\"> Modelon AB, Lund, Sweden (Hubertus Tummescheit) </td>
</tr>

<tr><td valign=\"top\"> SIunits <br>
      StateGraph </td>
   <td valign=\"top\"> DLR Institute of Robotics and Mechatronics, Oberpfaffenhofen, Germany<br>
      (Martin Otter)</td>
</tr>

<tr><td valign=\"top\"> Thermal.FluidHeatFlow <br>
      Thermal.HeatTransfer </td>
   <td valign=\"top\"> Anton Haumer, Consultant, St.Andrae-Woerdern, Austria, and<br>
      arsenal research, Vienna, Austria (Christian Kral)</td>

<tr><td valign=\"top\"> Utilities </td>
   <td valign=\"top\"> DLR Institute of Robotics and Mechatronics, Oberpfaffenhofen, Germany<br>
      (Martin Otter)<br>
      Dynasim AB, Lund, Sweden (Hans Olsson)</td>
</tr>
</table>


<p>
The following people have directly contributed to the implementation
of the Modelica package (many more people have contributed to the design):
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><td valign=\"top\"><b>Peter Beater</b> </td
   <td valign=\"top\"> University of Paderborn, Germany</td>
   <td valign=\"top\"> Modelica.Mechanics.Translational </td>
</tr>

<tr><td valign=\"top\"><b>Thomas Bdrich</b> </td
   <td valign=\"top\"> Dresden University of Technology, Germany</td>
   <td valign=\"top\"> Modelica.Magnetic.FluxTubes </td>


<tr><td valign=\"top\"><b>Dag Br&uuml;ck</b> </td
   <td valign=\"top\"> Dynasim AB, Lund, Sweden</td>
   <td valign=\"top\"> Modelica.Utilities</td>
</tr>

<tr><td valign=\"top\"><b>Francesco Casella</b> </td
   <td valign=\"top\"> Politecnico di Milano, Milano, Italy</td>
   <td valign=\"top\"> Modelica.Fluid<br>
                            Modelica.Media</td>
</tr>

<tr><td valign=\"top\"><b>Christoph Clauss</b> </td
   <td valign=\"top\"> Fraunhofer Institute for Integrated Circuits,<br> Dresden, Germany</td>
   <td valign=\"top\"> Modelica.Electrical.Analog<br>
     Modelica.Electrical.Digital</td>
</tr>

<tr><td valign=\"top\"><b>Jonas Eborn</b> </td
   <td valign=\"top\"> Modelon AB, Lund, Sweden</td>
   <td valign=\"top\"> Modelica.Media</td>
</tr>

<tr><td valign=\"top\"><b>Hilding Elmqvist</b> </td
   <td valign=\"top\"> Dynasim AB, Lund, Sweden</td>
   <td valign=\"top\"> Modelica.Mechanics.MultiBody<br>
                   Modelica.Fluid<br>
     Modelica.Media<br>
     Modelica.StateGraph<br>
     Modelica.Utilities<br>
     Conversion from 1.6 to 2.0</td>
</tr>

<tr><td valign=\"top\"><b>R&uuml;diger Franke</b> </td
   <td valign=\"top\"> ABB Corporate Research,<br>Ladenburg, German</td>
   <td valign=\"top\"> Modelica.Fluid<br>
                            Modelica.Media</td>
</tr>

<tr><td valign=\"top\"><b>Manuel Gr&auml;ber</b> </td
   <td valign=\"top\"> Institut f&uuml;r Thermodynamik, <br>
     Technische Universit&auml;t Braunschweig, <br>
     Germany</td>
   <td valign=\"top\"> Modelica.Fluid</td>
</tr>

<tr><td valign=\"top\"><b>Anton Haumer</b> </td
   <td valign=\"top\"> Consultant, St.Andrae-Woerdern,<br>Austria</td>
   <td valign=\"top\"> Modelica.Electrical.Machines<br>
     Modelica.Electrical.Multiphase<br>
     Modelica.Mechanics.Rotational<br>
     Modelica.Mechanics.Translational<br>
     Modelica.Thermal.FluidHeatFlow<br>
     Modelica.Thermal.HeatTransfer<br>
     Conversion from 1.6 to 2.0<br>
     Conversion from 2.2 to 3.0</td>
</tr>

<tr><td valign=\"top\"><b>Hans-Dieter Joos</b> </td
   <td valign=\"top\"> Institute of Robotics and Mechatronics<br>
     DLR, German Aerospace Center, <br>
     Oberpfaffenhofen, Germany</td>
   <td valign=\"top\"> Modelica.Math.Matrices</td>
</tr>

<tr><td valign=\"top\"><b>Christian Kral</b> </td
   <td valign=\"top\"> arsenal research, Vienna, Austria</td>
   <td valign=\"top\"> Modelica.Electrical.Machines<br>
     Modelica.Thermal.FluidHeatFlow</td>
</tr>

<tr><td valign=\"top\"><b>Sven Erik Mattsson</b> </td
   <td valign=\"top\"> Dynasim AB, Lund, Sweden</td>
   <td valign=\"top\"> Modelica.Mechanics.MultiBody</td>
</tr>
<tr><td valign=\"top\"><b>Hans Olsson</b> </td
   <td valign=\"top\"> Dynasim AB, Lund, Sweden</td>
   <td valign=\"top\"> Modelica.Blocks<br>
     Modelica.Math.Matrices<br>
     Modelica.Utilities<br>
     Conversion from 1.6 to 2.0<br>
     Conversion from 2.2 to 3.0</td>
</tr>

<tr><td valign=\"top\"><b>Martin Otter</b> </td
   <td valign=\"top\"> Institute of Robotics and Mechatronics<br>
     DLR, German Aerospace Center, <br>
     Oberpfaffenhofen, Germany</td>
   <td valign=\"top\"> Modelica.Blocks<br>
                   Modelica.Fluid<br>
     Modelica.Mechanics.MultiBody<br>
     Modelica.Mechanics.Rotational<br>
     Modelica.Mechanics.Translational<br>
     Modelica.Math<br>
     Modelica.Media<br>
     Modelica.SIunits<br>
     Modelica.StateGraph<br>
     Modelica.Thermal.HeatTransfer<br>
     Modelica.Utilities<br>
     ModelicaReference<br>
     Conversion from 1.6 to 2.0<br>
     Conversion from 2.2 to 3.0</td>
</tr>

<tr><td valign=\"top\"><b>Katrin Pr&ouml;l&szlig;</b> </td
   <td valign=\"top\"> Modelon AB, Lund, Sweden<br>
                            until 2008:<br>
                            Department of Technical Thermodynamics,<br>
     Technical University Hamburg-Harburg,<br>Germany</td>
   <td valign=\"top\"> Modelica.Fluid<br>
                            Modelica.Media</td>
</tr>

<tr><td valign=\"top\"><b>Christoph C. Richter</b> </td
   <td valign=\"top\"> Institut f&uuml;r Thermodynamik, <br>
     Technische Universit&auml;t Braunschweig, <br>
     Germany</td>
   <td valign=\"top\"> Modelica.Fluid<br>
                            Modelica.Media</td>
</tr>

<tr><td valign=\"top\"><b>Andr&eacute; Schneider</b> </td
   <td valign=\"top\"> Fraunhofer Institute for Integrated Circuits,<br> Dresden, Germany</td>
   <td valign=\"top\"> Modelica.Electrical.Analog<br>
     Modelica.Electrical.Digital</td>
</tr>
<tr><td valign=\"top\"><b>Christian Schweiger</b> </td
   <td valign=\"top\"> Until 2006:<br>
     Institute of Robotics and Mechatronics,<br>
     DLR, German Aerospace Center,<br>
     Oberpfaffenhofen, Germany</td>
   <td valign=\"top\"> Modelica.Mechanics.Rotational<br>
     ModelicaReference<br>
     Conversion from 1.6 to 2.0</td>
</tr>

<tr><td valign=\"top\"><b>Michael Sielemann</b> </td
   <td valign=\"top\"> Institute of Robotics and Mechatronics<br>
     DLR, German Aerospace Center, <br>
     Oberpfaffenhofen, Germany</td>
   <td valign=\"top\"> Modelica.Fluid</td>
</tr>

<tr><td valign=\"top\"><b>Michael Tiller</b> </td
   <td valign=\"top\"> Emmeskay, Inc., Dearborn, MI, U.S.A, <br>
     (previously Ford Motor Company, Dearborn) </td>
   <td valign=\"top\"> Modelica.Media<br>
     Modelica.Thermal.HeatTransfer</td>
</tr>
<tr><td valign=\"top\"><b>Hubertus Tummescheit</b> </td
   <td valign=\"top\"> Modelon AB, Lund, Sweden </td>
   <td valign=\"top\"> Modelica.Media<br>
     Modelica.Thermal.HeatTransfer</td>
</tr>
<tr><td valign=\"top\"><b>Nico Walter</b> </td
   <td valign=\"top\"> Master thesis at HTWK Leipzig<br>
     (Prof. R. M&uuml;ller) and<br>
     DLR Oberpfaffenhofen, Germany</td>
   <td valign=\"top\"> Modelica.Math.Matrices</td>
</tr>

<tr><td valign=\"top\"><b>Manuel Wetter</b> </td
   <td valign=\"top\"> Lawrence Berkeley National Laboratory; U.S.A</td>
   <td valign=\"top\"> Modelica.Fluid</td>
</tr>
</table>
</html>
"));
    end Contact;
  end UsersGuide;
  annotation(preferredView = "info", version = "3.1", versionBuild = 3, versionDate = "2009-08-14", dateModified = "2009-08-26 19:32:58Z", revisionId = "$Id:: package.mo 2987 2009-10-02 07:31:34Z #$", conversion(noneFromVersion = "3.0.1", noneFromVersion = "3.0", from(version = "2.1", script = "Scripts/ConvertModelica_from_2.2.2_to_3.0.mos"), from(version = "2.2", script = "Scripts/ConvertModelica_from_2.2.2_to_3.0.mos"), from(version = "2.2.1", script = "Scripts/ConvertModelica_from_2.2.2_to_3.0.mos"), from(version = "2.2.2", script = "Scripts/ConvertModelica_from_2.2.2_to_3.0.mos")), __Dymola_classOrder = {"UsersGuide","Blocks","StateGraph","Electrical","Magnetic","Mechanics","Fluid","Media","Thermal","Math","Utilities","Constants","Icons","SIunits"}, Settings(NewStateSelection = true), Documentation(info = "<HTML>
<p>
Package <b>Modelica</b> is a <b>standardized</b> and <b>free</b> package
that is developed together with the Modelica language from the
Modelica Association, see
<a href=\"http://www.Modelica.org\">http://www.Modelica.org</a>.
It is also called <b>Modelica Standard Library</b>.
It provides model components in many domains that are based on
standardized interface definitions. Some typical examples are shown
in the next figure:
</p>

<p>
<img src=\"../Images/UsersGuide/ModelicaLibraries.png\">
</p>

<p>
For an introduction, have especially a look at:
</p>
<ul>
<li> <a href=\"Modelica://Modelica.UsersGuide.Overview\">Overview</a>
  provides an overview of the Modelica Standard Library
  inside the <a href=\"Modelica://Modelica.UsersGuide\">User's Guide</a>.</li>
<li><a href=\"Modelica://Modelica.UsersGuide.ReleaseNotes\">Release Notes</a>
 summarizes the changes of new versions of this package.</li>
<li> <a href=\"Modelica://Modelica.UsersGuide.Contact\">Contact</a>
  lists the contributors of the Modelica Standard Library.</li>
<li> <a href=\"Documentation/ModelicaStandardLibrary.pdf\">ModelicaStandardLibrary.pdf</a>
  is the complete documentation of the library in pdf format.
<li> The <b>Examples</b> packages in the various libraries, demonstrate
  how to use the components of the corresponding sublibrary.</li>
</ul>

<p>
This version of the Modelica Standard Library consists of
</p>
<ul>
<li> <b>922</b> models and blocks, and</li>
<li> <b>615</b> functions
</ul>
<p>
that are directly usable (= number of public, non-partial classes).
</p>


<p>
<b>Licensed by the Modelica Association under the Modelica License 2</b><br>
Copyright &copy; 1998-2009, ABB, arsenal research, T.&nbsp;Bdrich, DLR, Dynasim, Fraunhofer, Modelon,
TU Hamburg-Harburg, Politecnico di Milano.
</p>

<p>
<i>This Modelica package is <u>free</u> software and
the use is completely at <u>your own risk</u>;
it can be redistributed and/or modified under the terms of the
Modelica license 2, see the license conditions (including the
disclaimer of warranty)
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a></u>
or at
<a href=\"http://www.Modelica.org/licenses/ModelicaLicense2\">
http://www.Modelica.org/licenses/ModelicaLicense2</a>.
</p>

</HTML>
"));
  package Utilities "Library of utility functions dedicated to scripting (operating on files, streams, strings, system)"
    extends Modelica.Icons.Library;
    annotation(Documentation(info = "<html>
<p>
This package contains Modelica <b>functions</b> that are
especially suited for <b>scripting</b>. The functions might
be used to work with strings, read data from file, write data
to file or copy, move and remove files.
</p>
<p>
For an introduction, have especially a look at:
</p>
<ul>
<li> <a href=\"Modelica://Modelica.Utilities.UsersGuide\">Modelica.Utilities.User's Guide</a>
     discusses the most important aspects of this library.</li>
<li> <a href=\"Modelica://Modelica.Utilities.Examples\">Modelica.Utilities.Examples</a>
     contains examples that demonstrate the usage of this library.</li>
</ul>
<p>
The following main sublibraries are available:
</p>
<ul>
<li> <a href=\"Modelica://Modelica.Utilities.Files\">Files</a>
     provides functions to operate on files and directories, e.g.,
     to copy, move, remove files.</li>
<li> <a href=\"Modelica://Modelica.Utilities.Streams\">Streams</a>
     provides functions to read from files and write to files.</li>
<li> <a href=\"Modelica://Modelica.Utilities.Strings\">Strings</a>
     provides functions to operate on strings. E.g.
     substring, find, replace, sort, scanToken.</li>
<li> <a href=\"Modelica://Modelica.Utilities.System\">System</a>
     provides functions to interact with the environment.
     E.g., get or set the working directory or environment
     variables and to send a command to the default shell.</li>
</ul>

<p>
Copyright &copy; 1998-2009, Modelica Association, DLR and Dynasim.
</p>
<p>
<i>This Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p><br>
</html>
"));
    package UsersGuide "User's Guide of Utilities Library"
      annotation(__Dymola_DocumentationClass = true, Documentation(info = "<HTML>
<p>
Library <b>Modelica.Utilities</b> contains Modelica <b>functions</b> that are
especially suited for <b>scripting</b>. Currently, only a rudimentary
User's Guide is present. This will be improved in the next releases.
The User's Guide has currently the following chapters:
</p>
<ol>
<li>
<a href=\"Modelica://Modelica.Utilities.UsersGuide.ReleaseNotes\">Release Notes</a>
  summarizes the differences between different versions of this
  library.
</li>
<li>
<a href=\"Modelica://Modelica.Utilities.UsersGuide.ImplementationNotes\">ImplementationNotes</a>
  describes design decisions for this library especially for
  Modelica tool vendors.
</li>
<li>
<a href=\"Modelica://Modelica.Utilities.UsersGuide.Contact\">Contact</a> provides
  information about the authors of the library as well as acknowledgments.
</li>
</ol>
<p>
<b>Error handling</b><br>
In case of error, all functions in this library use a Modelica \"assert(..)\"
to provide an error message and to cancel all actions. This means that
functions do not return, if an error is triggered inside the function.
In the near future, an exception handling mechanism will be introduced
in Modelica that will allow to catch errors at a defined place.
</p>
</HTML>"));
      class ImplementationNotes "Implementation Notes"
        annotation(Documentation(info = "<html>
<p>
Below the major design decisions of this library are summarized.
<p>
<ul>
<li> <b>C-Function Interface</b><br>
     This library contains several interfaces to C-functions in order
     to operate with the environment. As will become clear, it is usally
     required that a Modelica tool vendor provides an implementation
     of these C-functions that are suited for his environment.
     In directory \"Modelica.Utilities\\C-Sources\" a reference implementation
     is given for Microsoft Windows Systems and for POSIX environments.
     The files \"ModelicaInternal.c\" and \"ModelicaStrings.c\" can be
     used as a basis for the integration in the vendors environment.<br>&nbsp;</li>
<li> <b>Character Encoding</b><br>
     The representation of characters is different in operating systems.
     The more modern ones (e.g. Windows-NT) use an early variant of
     Unicode (16 bit per character)
     other (e.g. Windows-ME) use 8-bit encoding. Also 32 bit per character
     and multi-byte representations are in use. This is important, since e.g.,
     Japanese Modelica users need Unicode representation. The design in this
     library is done in such a way that a basic set of calls to the operating
     system hides the actual character representation. This means, that all
     functions of this package can be used independent from the underlying
     character representation.<br>
     The C-interface of the Modelica language provides only an 8-bit
     character encoding passing mechanism of strings. As a result, the
     reference implementation in \"Modelica.Utilities\\C-Source\" needs to
     be adapted to the character representation supported in the
     Modelica vendor environment.<br>&nbsp;</li>
<li> <b>Internal String Representation</b><br>
     The design of this package was made in order that string handling
     is convenient. This is in contrast to, e.g., the C-language, where
     string handling is inconvenient, cumbersome and error prone, but on the
     other hand is in some sense \"efficient\".
     The standard reference implementation in \"Modelica.Utilities\\C-Source\"
     is based on the standard C definition of a string, i.e., a pointer to
     a sequence of characters, ended with a null terminating character.
     In order that the string handling in this package is convenient,
     some assumptions have been made, especially, that the access to
     a substring is efficient (O(1) access instead of O(n) as in standard C).
     This allows to hide string pointer arithmetic from the user.
     In such a case, a similiar efficiency as in C can be expected for
     most high level operations, such as find, sort, replace.
     The \"efficient character access\" can be reached if, e.g.,
     the number of characters
     are stored in a string, and the length of a character is fixed,
     say 16 or 32 bit (if all Unicode characters shall be represented).
     A vendor should adapt the reference implementation in this
     respect.<br>&nbsp;</li>
<li> <b>String copy = pointer copy</b><br>
     The Modelica language has no mechanism to change a character
     of a string. When a string has to be modified, the only way
     to achieve this is to generate it newly. The advantage is that
     a Modelica tool can treat a string as a constant entity and
     can replace (expensive) string copy operations by pointer
     copy operations. For example, when sorting a set of strings
     the following type of operations occur:
     <pre>
     String s[:], s_temp;
      ...
     s_temp := s[i];
     s[i]   := s[j];
     s[j]   := s_temp;
     </pre>
     Formally, three strings are copied. Due to the feature
     sketched above, a Modelica tool can replace this
     copy operation by pointer assignments, a very \"cheap\"
     operation. The Modelica.Utilities functions will perform
     efficiently, if such types of optimizations are supported
     by the tool.</li>
</ul>
</html>
"));
      end ImplementationNotes;
      class ReleaseNotes "Release notes"
        annotation(Documentation(info = "<HTML>
<h4>Version 1.0, 2004-09-29</h4>
<p>
First version implemented.
</p>
</HTML>
"));
      end ReleaseNotes;
      class Contact "Contact"
        annotation(Documentation(info = "<html>
<dl>
<dt><b>Responsible for Library:</b>
<dd>Dag Br&uuml;ck, Dynasim AB, Sweden.<br>
    email: <A HREF=\"mailto:Dag@Dynasim.se\">Dag@Dynasim.se</A><br>
</dl>
<p><b>Acknowledgements:</b></p>
<ul>
<li> This library has been designed by:<br>
     <blockquote>
     Dag Br&uuml;ck, Dynasim AB, Sweden <br>
     Hilding Elmqvist, Dynasim AB, Sweden <br>
     Hans Olsson, Dynasim AB, Sweden <br>
     Martin Otter, DLR Oberpfaffenhofen, Germany.
     </blockquote></li>
<li> The library including the C reference implementation has
     been implemented by Martin Otter and Dag Br&uuml;ck.</li>
<li> The Examples.calculator demonstration to implement a calculator
     with this library is from Hilding Elmqvist.</li>
<li> Helpful comments from Kaj Nystr&ouml;m, PELAB, Link&ouml;ping, Sweden,
     are appreciated, as well as discussions at the 34th, 36th, and 40th
     Modelica Design Meetings in Vienna, Link&ouml;ping, and Dresden. </li>
</ul>
</html>
"));
      end Contact;
    end UsersGuide;
    package Types "Type definitions used in package Modelica.Utilities"
      type Compare = enumeration(Less "String 1 is lexicographically less than string 2", Equal  "String 1 is identical to string 2", Greater  "String 1 is lexicographically greater than string 2") "Enumeration defining comparision of two strings";
      type FileType = enumeration(NoFile "No file exists", RegularFile "Regular file", Directory  "Directory", SpecialFile  "Special file (pipe, FIFO, device, etc.)") "Enumeration defining the type of a file";
      type TokenType = enumeration(RealToken, IntegerToken, BooleanToken, StringToken, IdentifierToken, DelimiterToken , NoToken ) "Enumeration defining the token type";
      record TokenValue "Value of token"
        extends Modelica.Icons.Record;
        TokenType tokenType "Type of token";
        Real real "Value if tokenType == TokenType.RealToken";
        Integer integer "Value if tokenType == TokenType.IntegerToken";
        Boolean boolean "Value if tokenType == TokenType.BooleanToken";
        String string "Value if tokenType == TokenType.StringToken/IdentifierToken/DelimiterToken";
        annotation(Documentation(info = "<html>

</html>"));
      end TokenValue;
      annotation(Documentation(info = "<html>
<p>
This package contains type definitions used in Modelica.Utilities.
</p>

</html>"));
    end Types;
    package System "Interaction with environment"
      extends Modelica.Icons.Library;
      annotation(Documentation(info = "
<HTML>
<p>
This package contains functions to interact with the environment.
</p>
</HTML>
"));
      function getWorkDirectory "Get full path name of work directory"
        extends Modelica.Icons.Function;
        output String directory "Full path name of work directory";

        external "C" directory = ModelicaInternal_getcwd(0) ;        annotation(Library = "ModelicaExternalC", Documentation(info = "<html>

</html>"));
      end getWorkDirectory;
      function setWorkDirectory "Set work directory"
        extends Modelica.Icons.Function;
        input String directory "New work directory";

        external "C" ModelicaInternal_chdir(directory) ;         annotation(Library = "ModelicaExternalC", Documentation(info = "<html>

</html>"));
      end setWorkDirectory;
      function getEnvironmentVariable "Get content of environment variable"
        extends Modelica.Icons.Function;
        input String name "Name of environment variable";
        input Boolean convertToSlash = false "True, if native directory separators in 'result' shall be changed to '/'";
        output String content "Content of environment variable (empty, if not existent)";
        output Boolean exist "= true, if environment variable exists; = false, if it does not exist";

        external "C" ModelicaInternal_getenv(name,convertToSlash,content,exist) ;         annotation(Library = "ModelicaExternalC", Documentation(info = "<html>

</html>"));
      end getEnvironmentVariable;
      function setEnvironmentVariable "Set content of local environment variable"
        extends Modelica.Icons.Function;
        input String name "Name of environment variable";
        input String content "Value of the environment variable";
        input Boolean convertFromSlash = false "True, if '/' in content shall be changed to the native directory separator";

        external "C" ModelicaInternal_setenv(name,content,convertFromSlash) ;         annotation(Library = "ModelicaExternalC", Documentation(info = "<html>

</html>"));
      end setEnvironmentVariable;
      function command "Execute command in default shell"
        extends Modelica.Icons.Function;
        input String string "String to be passed to shell";
        output Integer result "Return value from command (depends on environment)";

        external "C" result = system(string) ;        annotation(Library = "ModelicaExternalC", Documentation(info = "<html>

</html>"));
      end command;
      function exit "Terminate execution of Modelica environment"
        extends Modelica.Icons.Function;
        input Integer status = 0 "Result to be returned by environment (0 means success)";

        external "C" ModelicaInternal_exit(status) ;         annotation(Library = "ModelicaExternalC");
      end exit;
    end System;
    package Strings "Operations on strings"
      function length "Returns length of string"
        extends Modelica.Icons.Function;
        input String string;
        output Integer result "Number of characters of string";

        external "C" result = ModelicaStrings_length(string) ;        annotation(Library = "ModelicaExternalC", Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Strings.<b>length</b>(string);
</pre></blockquote>
<h4>Description</h4>
<p>
Returns the number of characters of \"string\".
</p>
</html>"));
      end length;
      function substring "Returns a substring defined by start and end index"
        extends Modelica.Icons.Function;
        input String string "String from which a substring is inquired";
        input Integer startIndex(min = 1) "Character position of substring begin (index=1 is first character in string)";
        input Integer endIndex(min = 1) "Character position of substring end";
        output String result "String containing substring string[startIndex:endIndex]";

        external "C" result = ModelicaStrings_substring(string,startIndex,endIndex) ;        annotation(Library = "ModelicaExternalC", Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
string2 = Strings.<b>substring</b>(string, startIndex, endIndex);
</pre></blockquote>
<h4>Description</h4>
<p>
This function returns
the substring from position startIndex
up to and including position endIndex of \"string\" .
</p>
<p>
If index, startIndex, or endIndex are not correct, e.g.,
if endIndex &gt; length(string), an assert is triggered.
</p>
<h4>Example</h4>
<blockquote><pre>
  string1 := \"This is line 111\";
  string2 := Strings.substring(string1,9,12); // string2 = \"line\"
</pre></blockquote>
</html>"));
      end substring;
      function repeat "Repeat a string n times"
        extends Modelica.Icons.Function;
        input Integer n(min = 0) = 1 "Number of occurences";
        input String string = " " "String that is repeated";
        output String repeatedString "String containing n concatenated strings";
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
string2 = Strings.<b>repeat</b>(n);
string2 = Strings.<b>repeat</b>(n, string=\" \");
</pre></blockquote>
<h4>Description</h4>
<p>
The first form returns a string consisting of n blanks.
</p>
<p>
The second form returns a string consisting of n substrings
defined by the optional argument \"string\".
</p>
</html>"));
      algorithm 
        repeatedString:="";
        for i in 1:n loop
                  repeatedString:=repeatedString + string;

        end for;
      end repeat;
      function compare "Compare two strings lexicographically"
        extends Modelica.Icons.Function;
        input String string1;
        input String string2;
        input Boolean caseSensitive = true "= false, if case of letters is ignored";
        output Modelica.Utilities.Types.Compare result "Result of comparison";

        external "C" result = ModelicaStrings_compare(string1,string2,caseSensitive) ;        annotation(Library = "ModelicaExternalC", Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
result = Strings.<b>compare</b>(string1, string2);
result = Strings.<b>compare</b>(string1, string2, caseSensitive=true);
</pre></blockquote>
<h4>Description</h4>
<p>
Compares two strings. If the optional argument caseSensitive=false,
upper case letters are treated as if they would be lower case letters.
The result of the comparison is returned as:
</p>
<pre>
  result = Modelica.Utilities.Types.Compare.Less     // string1 &lt; string2
         = Modelica.Utilities.Types.Compare.Equal    // string1 = string2
         = Modelica.Utilities.Types.Compare.Greater  // string1 &gt; string2
</pre>
<p>
Comparison is with regards to lexicographical order,
e.g., \"a\" &lt; \"b\";
</p>
</html>"));
      end compare;
      function isEqual "Determine whether two strings are identical"
        extends Modelica.Icons.Function;
        input String string1;
        input String string2;
        input Boolean caseSensitive = true "= false, if lower and upper case are ignored for the comparison";
        output Boolean identical "True, if string1 is identical to string2";
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Strings.<b>isEqual</b>(string1, string2);
Strings.<b>isEqual</b>(string1, string2, caseSensitive=true);
</pre></blockquote>
<h4>Description</h4>
<p>
Compare whether two strings are identical,
optionally ignoring case.
</p>
</html>"));
      algorithm 
        identical:=compare(string1, string2, caseSensitive) == Types.Compare.Equal;
      end isEqual;
      extends Modelica.Icons.Library;
      annotation(Documentation(info = "<HTML>
<h4>Library content</h4>
<p>
Package <b>Strings</b> contains functions to manipulate strings.
</p>
<p>
In the table below an example
call to every function is given using the <b>default</b> options.
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Function</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\">len = <a href=\"Modelica://Modelica.Utilities.Strings.length\">length</a>(string)</td>
      <td valign=\"top\">Returns length of string</td></tr>
  <tr><td valign=\"top\">string2 = <a href=\"Modelica://Modelica.Utilities.Strings.substring\">substring</a>(string1,startIndex,endIndex)
       </td>
      <td valign=\"top\">Returns a substring defined by start and end index</td></tr>
  <tr><td valign=\"top\">result = <a href=\"Modelica://Modelica.Utilities.Strings.repeat\">repeat</a>(n)<br>
 result = <a href=\"Modelica://Modelica.Utilities.Strings.repeat\">repeat</a>(n,string)</td>
      <td valign=\"top\">Repeat a blank or a string n times.</td></tr>
  <tr><td valign=\"top\">result = <a href=\"Modelica://Modelica.Utilities.Strings.compare\">compare</a>(string1, string2)</td>
      <td valign=\"top\">Compares two substrings with regards to alphabetical order</td></tr>
  <tr><td valign=\"top\">identical =
<a href=\"Modelica://Modelica.Utilities.Strings.isEqual\">isEqual</a>(string1,string2)</td>
      <td valign=\"top\">Determine whether two strings are identical</td></tr>
  <tr><td valign=\"top\">result = <a href=\"Modelica://Modelica.Utilities.Strings.count\">count</a>(string,searchString)</td>
      <td valign=\"top\">Count the number of occurrences of a string</td></tr>
  <tr>
<td valign=\"top\">index = <a href=\"Modelica://Modelica.Utilities.Strings.find\">find</a>(string,searchString)</td>
      <td valign=\"top\">Find first occurrence of a string in another string</td></tr>
<tr>
<td valign=\"top\">index = <a href=\"Modelica://Modelica.Utilities.Strings.findLast\">findLast</a>(string,searchString)</td>
      <td valign=\"top\">Find last occurrence of a string in another string</td></tr>
  <tr><td valign=\"top\">string2 = <a href=\"Modelica://Modelica.Utilities.Strings.replace\">replace</a>(string,searchString,replaceString)</td>
      <td valign=\"top\">Replace one or all occurrences of a string</td></tr>
  <tr><td valign=\"top\">stringVector2 = <a href=\"Modelica://Modelica.Utilities.Strings.sort\">sort</a>(stringVector1)</td>
      <td valign=\"top\">Sort vector of strings in alphabetic order</td></tr>
  <tr><td valign=\"top\">(token, index) = <a href=\"Modelica://Modelica.Utilities.Strings.scanToken\">scanToken</a>(string,startIndex)</td>
      <td valign=\"top\">Scan for a token (Real/Integer/Boolean/String/Identifier/Delimiter/NoToken)</td></tr>
  <tr><td valign=\"top\">(number, index) = <a href=\"Modelica://Modelica.Utilities.Strings.scanReal\">scanReal</a>(string,startIndex)</td>
      <td valign=\"top\">Scan for a Real constant</td></tr>
  <tr><td valign=\"top\">(number, index) = <a href=\"Modelica://Modelica.Utilities.Strings.scanInteger\">scanInteger</a>(string,startIndex)</td>
      <td valign=\"top\">Scan for an Integer constant</td></tr>
  <tr><td valign=\"top\">(boolean, index) = <a href=\"Modelica://Modelica.Utilities.Strings.scanBoolean\">scanBoolean</a>(string,startIndex)</td>
      <td valign=\"top\">Scan for a Boolean constant</td></tr>
  <tr><td valign=\"top\">(string2, index) = <a href=\"Modelica://Modelica.Utilities.Strings.scanString\">scanString</a>(string,startIndex)</td>
      <td valign=\"top\">Scan for a String constant</td></tr>
  <tr><td valign=\"top\">(identifier, index) = <a href=\"Modelica://Modelica.Utilities.Strings.scanIdentifier\">scanIdentifier</a>(string,startIndex)</td>
      <td valign=\"top\">Scan for an identifier</td></tr>
  <tr><td valign=\"top\">(delimiter, index) = <a href=\"Modelica://Modelica.Utilities.Strings.scanDelimiter\">scanDelimiter</a>(string,startIndex)</td>
      <td valign=\"top\">Scan for delimiters</td></tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Utilities.Strings.scanNoToken\">scanNoToken</a>(string,startIndex)</td>
      <td valign=\"top\">Check that remaining part of string consists solely of <br>
          white space or line comments (\"// ...\\n\").</td></tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Utilities.Strings.syntaxError\">syntaxError</a>(string,index,message)</td>
      <td valign=\"top\"> Print a \"syntax error message\" as well as a string and the <br>
           index at which scanning detected an error</td></tr>
</table>
<p>
The functions \"compare\", \"isEqual\", \"count\", \"find\", \"findLast\", \"replace\", \"sort\"
have the optional
input argument <b>caseSensitive</b> with default <b>true</b>.
If <b>false</b>, the operation is carried out without taking
into account whether a character is upper or lower case.
</p>
</HTML>"));
      function count "Count the number of non-overlapping occurrences of a string"
        extends Modelica.Icons.Function;
        input String string "String that is analyzed";
        input String searchString "String that is searched for in string";
        input Integer startIndex(min = 1) = 1 "Start search at index startIndex";
        input Boolean caseSensitive = true "= false, if lower and upper case are ignored for count";
        output Integer result "Number of occurrences of 'searchString' in 'string'";
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Strings.<b>count</b>(string, searchString)
Strings.<b>count</b>(string, searchString, startIndex=1,
                     caseSensitive=true)
</pre></blockquote>
<h4>Description</h4>
<p>
Returns the number of non-overlapping occurrences of string \"searchString\"
in \"string\". The search is started at index \"startIndex\" (default = 1).
If the optional argument \"caseSensitive\" is false,
for the counting it does not matter whether a letter is upper
or lower case.
/p>
</html>"));
      protected 
        Integer lenSearchString = length(searchString);
        Integer i = startIndex;
      algorithm 
        result:=0;
        while (i <> 0) loop
                  i:=find(string, searchString, i, caseSensitive);
          if i > 0 then 
                    result:=result + 1;
          i:=i + lenSearchString;

          else 
          end if;

        end while;
      end count;
      function find "Find first occurrence of a string within another string"
        extends Modelica.Icons.Function;
        input String string "String that is analyzed";
        input String searchString "String that is searched for in string";
        input Integer startIndex(min = 1) = 1 "Start search at index startIndex";
        input Boolean caseSensitive = true "= false, if lower and upper case are ignored for the search";
        output Integer index "Index of the beginning of the first occurrence of 'searchString' within 'string', or zero if not present";
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
index = Strings.<b>find</b>(string, searchString);
index = Strings.<b>find</b>(string, searchString, startIndex=1,
                     caseSensitive=true);
</pre></blockquote>
<h4>Description</h4>
<p>
Finds first occurence of \"searchString\" within \"string\"
and return the corresponding index.
Start search at index \"startIndex\" (default = 1).
If the optional argument \"caseSensitive\" is false, lower
and upper case are ignored for the search.
If \"searchString\" is not found, a value of \"0\" is returned.
</p>
</html>
"));
      protected 
        Integer lengthSearchString = length(searchString);
        Integer len = lengthSearchString - 1;
        Integer i = startIndex;
        Integer i_max = length(string) - lengthSearchString + 1;
      algorithm 
        index:=0;
        while (i <= i_max) loop
                  if isEqual(substring(string, i, i + len), searchString, caseSensitive) then 
                    index:=i;
          i:=i_max + 1;

          else           i:=i + 1;

          end if;

        end while;
      end find;
      function findLast "Find last occurrence of a string within another string"
        extends Modelica.Icons.Function;
        input String string "String that is analyzed";
        input String searchString "String that is searched for in string";
        input Integer startIndex(min = 0) = 0 "Start search at index startIndex. If startIndex = 0, start at length(string)";
        input Boolean caseSensitive = true "= false, if lower and upper case are ignored for the search";
        output Integer index "Index of the beginning of the last occurrence of 'searchString' within 'string', or zero if not present";
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
index = Strings.<b>findLast</b>(string, searchString);
index = Strings.<b>findLast</b>(string, searchString,
                         startIndex=length(string), caseSensitive=true,
</pre></blockquote>
<h4>Description</h4>
<p>
Finds first occurence of \"searchString\" within \"string\"
when searching from the last character of \"string\"
backwards, and return the corresponding index.
Start search at index \"startIndex\" (default = length(string)).
If the optional argument \"caseSensitive\" is false, lower
and upper case are ignored for the search.
If \"searchString\" is not found, a value of \"0\" is returned.
</p>
</html>
"));
      protected 
        Integer lenString = length(string);
        Integer lenSearchString = length(searchString);
        Integer i;
      algorithm 
        i:=if startIndex == 0 then lenString - lenSearchString + 1 else startIndex;
        index:=0;
        while (i >= 1) loop
                  if isEqual(substring(string, i, i + lenSearchString - 1), searchString, caseSensitive) then 
                    index:=i;
          i:=0;

          else           i:=i - 1;

          end if;

        end while;
      end findLast;
      function replace "Replace non-overlapping occurrences of a string from left to right"
        extends Modelica.Icons.Function;
        input String string "String to be modified";
        input String searchString "Replace non-overlapping occurrences of 'searchString' in 'string' with 'replaceString'";
        input String replaceString "String that replaces 'searchString' in 'string'";
        input Integer startIndex = 1 "Start search at index startIndex";
        input Boolean replaceAll = true "if false, replace only the first occurrence, otherwise all occurrences";
        input Boolean caseSensitive = true "= false, if lower and upper case are ignored when searching for searchString";
        output String result "Resultant string of replacement operation";
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Strings.<b>replace</b>(string, searchString, replaceString);
Strings.<b>replace</b>(string, searchString, replaceString,
                startIndex=1, replaceAll=true, caseSensitive=true);
</pre></blockquote>
<h4>Description</h4>
<p>
Search in \"string\" for \"searchString\" and replace the found
substring by \"replaceString\".
<p>
<ul>
<li> The search starts at the first character of \"string\",
     or at character position \"startIndex\",
     if this optional argument is provided.</li>
<li> If the optional argument \"replaceAll\" is <b>true</b> (default),
     all occurrences of \"searchString\" are replaced.
     If the argument is <b>false</b>, only the first occurrence
     is replaced. </li>
<li> The search for \"searchString\" distinguishes upper and lower
     case letters. If the optional argument \"caseSensitive\" is
     <b>false</b>,
     the search ignores whether letters are upper
     or lower case. </li>
</ul>
<p>
The function returns the \"string\" with the
performed replacements.
</p>
</html>"));
      protected 
        Integer lenString = length(string);
        Integer lenSearchString = length(searchString);
        Integer i = startIndex;
        Integer i_found;
      algorithm 
        result:=if startIndex == 1 then "" else substring(string, 1, startIndex - 1);
        while (i > 0) loop
                  i_found:=find(string, searchString, i, caseSensitive);
          if i_found > 0 then 
                    result:=if i_found == 1 then replaceString else result + (if i_found - 1 < i then "" else substring(string, i, i_found - 1)) + replaceString;
          i:=i_found + lenSearchString;
          if i > lenString then 
                    i:=0;
          elseif not replaceAll then
            result:=result + substring(string, i, lenString);
            i:=0;


          else 
          end if;
          elseif lenString < i then
            i:=0;


          else           result:=result + substring(string, i, lenString);
          i:=0;

          end if;

        end while;
      end replace;
      function sort "Sort vector of strings in alphabetic order"
        extends Modelica.Icons.Function;
        input String stringVector1[:] "vector of strings";
        input Boolean caseSensitive = true "= false, if lower and upper case are ignored when comparing elements of stringVector1";
        output String stringVector2[size(stringVector1, 1)] "string1 sorted in alphabetical order";
      protected 
        Integer gap;
        Integer i;
        Integer j;
        String tempString;
        Integer nStringVector1 = size(stringVector1, 1);
        Boolean swap;
      algorithm 
        stringVector2:=stringVector1;
        gap:=div(nStringVector1, 2);
        while (gap > 0) loop
                  i:=gap;
          while (i < nStringVector1) loop
                      j:=i - gap;
            if j >= 0 then 
                        swap:=compare(stringVector2[j + 1], stringVector2[j + gap + 1], caseSensitive) == Modelica.Utilities.Types.Compare.Greater;

            else             swap:=false;

            end if;
            while (swap) loop
                          tempString:=stringVector2[j + 1];
              stringVector2[j + 1]:=stringVector2[j + gap + 1];
              stringVector2[j + gap + 1]:=tempString;
              j:=j - gap;
              if j >= 0 then 
                            swap:=compare(stringVector2[j + 1], stringVector2[j + gap + 1], caseSensitive) == Modelica.Utilities.Types.Compare.Greater;

              else               swap:=false;

              end if;

            end while;
            i:=i + 1;

          end while;
          gap:=div(gap, 2);

        end while;
        annotation(Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
stringVector2 = Streams.<b>sort</b>(stringVector1);
stringVector2 = Streams.<b>sort</b>(stringVector1, caseSensitive=true);
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>sort</b>(..) sorts a string vector stringVector1
in lexicographical order and returns the result in stringVector2.
If the optional argument \"caseSensitive\" is <b>false</b>, lower
and upper case letters are not distinguished.
</p>
<h4>Example</h4>
<blockquote><pre>
  s1 = {\"force\", \"angle\", \"pressure\"};
  s2 = Strings.sort(s1);
       -> s2 = {\"angle\", \"force\", \"pressure\"};
</pre></blockquote>
</HTML>"));
      end sort;
      function scanToken "Scan for the next token and return it"
        extends Modelica.Icons.Function;
        input String string "String to be scanned";
        input Integer startIndex(min = 1) = 1 "Start scanning of string at character startIndex";
        input Boolean unsigned = false "= true, if Real and Integer tokens shall not start with a sign";
        output Types.TokenValue token "Scanned token";
        output Integer nextIndex "Index of character after the found token; = 0, if NoToken";
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
(token, nextIndex) = Strings.<b>scanToken</b>(string, startIndex, unsigned=false);
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>scanToken</b> scans the string starting at index
\"startIndex\" and returns the next token, as well as the
index directly after the token. The returned token is a record
that holds the type of the token and the value of the token:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\">token.tokenType</td>
      <td valign=\"top\">Type of the token, see below</td></tr>
  <tr><td valign=\"top\">token.real</td>
      <td valign=\"top\">Real value if tokenType == TokenType.RealToken</td></tr>
  <tr><td valign=\"top\">token.integer</td>
      <td valign=\"top\">Integer value if tokenType == TokenType.IntegerToken</td></tr>
  <tr><td valign=\"top\">token.boolean</td>
      <td valign=\"top\">Boolean value if tokenType == TokenType.BooleanToken</td></tr>
  <tr><td valign=\"top\">token.string</td>
      <td valign=\"top\">String value if tokenType == TokenType.StringToken/IdentifierToken/DelimiterToken</td></tr>
</table>
<p>
Variable token.tokenType is an enumeration (emulated as a package
with constants) that can have the following values:
</p>
<pre>
   import T = Modelica.Utilities.Types.TokenType;
</pre>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\">T.RealToken</td>
      <td valign=\"top\">Modelica Real literal (e.g., 1.23e-4)</td></tr>
  <tr><td valign=\"top\">T.IntegerToken</td>
      <td valign=\"top\">Modelica Integer literal (e.g., 123)</td></tr>
  <tr><td valign=\"top\">T.BooleanToken</td>
      <td valign=\"top\">Modelica Boolean literal (e.g., false)</td></tr>
  <tr><td valign=\"top\">T.StringToken</td>
      <td valign=\"top\">Modelica String literal (e.g., \"string 123\")</td></tr>
  <tr><td valign=\"top\">T.IdentifierToken</td>
      <td valign=\"top\">Modelica identifier (e.g., \"force_a\")</td></tr>
  <tr><td valign=\"top\">T.DelimiterToken</td>
      <td valign=\"top\">any character without white space that does not appear<br>
          as first character in the tokens above (e.g., \"&\")</td></tr>
  <tr><td valign=\"top\">T.NoToken</td>
      <td valign=\"top\">White space, line comments and no other token<br>
          until the end of the string</td></tr>
</table>
<p>
Modelica line comments (\"// ... end-of-line/end-of-string\")
as well as white space is ignored.
If \"unsigned=true\", a Real or Integer literal
is not allowed to start with a \"+\" or \"-\" sign.
</p>
<h4>Example</h4>
<blockquote><pre>
  import Modelica.Utilities.Strings.*;
  import T = Modelica.Utilities.Types.TokenType;
  (token, index) := scanToken(string);
  <b>if</b> token.tokenType == T.RealToken <b>then</b>
     realValue := token.real;
  <b>elseif</b> token.tokenType == T.IntegerToken <b>then</b>
     integerValue := token.integer;
  <b>elseif</b> token.tokenType == T.BooleanToken<b> then</b>
     booleanValue := token.boolean;
  <b>elseif</b> token.tokenType == T.Identifier <b>then</b>
     name := token.string;
  <b>else</b>
     syntaxError(string,index,\"Expected Real, Integer, Boolean or identifier token\");
  <b>end if</b>;
</pre></blockquote>
</html>"));
      protected 
        Integer startTokenIndex;
      algorithm 
        token.real:=0.0;
        token.integer:=0;
        token.boolean:=false;
        token.string:="";
        startTokenIndex:=Advanced.skipLineComments(string, startIndex);
        if startTokenIndex > length(string) then 
                token.tokenType:=Modelica.Utilities.Types.TokenType.NoToken;
        nextIndex:=startTokenIndex;

        else         (nextIndex,token.integer):=Advanced.scanInteger(string, startTokenIndex, unsigned);
        token.tokenType:=Types.TokenType.IntegerToken;
        if nextIndex == startTokenIndex then 
                (nextIndex,token.real):=Advanced.scanReal(string, startTokenIndex, unsigned);
        token.tokenType:=Types.TokenType.RealToken;

        else 
        end if;
        if nextIndex == startTokenIndex then 
                (nextIndex,token.string):=Advanced.scanString(string, startTokenIndex);
        token.tokenType:=Types.TokenType.StringToken;

        else 
        end if;
        if nextIndex == startTokenIndex then 
                (nextIndex,token.string):=Advanced.scanIdentifier(string, startTokenIndex);
        if nextIndex > startTokenIndex then 
                if token.string == "false" then 
                token.string:="";
        token.boolean:=false;
        token.tokenType:=Types.TokenType.BooleanToken;
        elseif token.string == "true" then
          token.string:="";
          token.boolean:=true;
          token.tokenType:=Types.TokenType.BooleanToken;


        else         token.tokenType:=Types.TokenType.IdentifierToken;

        end if;

        else 
        end if;

        else 
        end if;
        if nextIndex == startTokenIndex then 
                token.string:=substring(string, startTokenIndex, startTokenIndex);
        token.tokenType:=Types.TokenType.DelimiterToken;
        nextIndex:=startTokenIndex + 1;

        else 
        end if;

        end if;
      end scanToken;
      function scanReal "Scan for the next Real number and trigger an assert if not present"
        extends Modelica.Icons.Function;
        input String string "String to be scanned";
        input Integer startIndex(min = 1) = 1 "Start scanning of string at character startIndex";
        input Boolean unsigned = false "= true, if Real token shall not start with a sign";
        input String message = "" "Message used in error message if scan is not successful";
        output Real number "Value of real number";
        output Integer nextIndex "index of character after the found number";
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
             number = Strings.<b>scanReal</b>(string);
(number, nextIndex) = Strings.<b>scanReal</b>(string, startIndex=1,
                                            unsigned=false, message=\"\");
</pre></blockquote>
<h4>Description</h4>
<p>
The first form, \"scanReal(string)\", scans \"string\" for a
Real number with leading white space and returns the value.
</p>
<p>
The second form, \"scanReal(string,startIndex,unsigned)\",
scans the string starting at index
\"startIndex\", checks whether the next token is a Real literal
and returns its value as a Real number, as well as the
index directly after the Real number.
If the optional argument \"unsigned\" is <b>true</b>,
the real number shall not have a leading \"+\" or \"-\" sign.
</p>
<p>
If the required Real number with leading white space
is not present in \"string\",  an assert is triggered.
</p>
</html>"));
      algorithm 
        (nextIndex,number):=Advanced.scanReal(string, startIndex, unsigned);
        if nextIndex == startIndex then 
                nextIndex:=Advanced.skipWhiteSpace(string, startIndex);
        if unsigned then 
                syntaxError(string, nextIndex, "Expected a Real number without a sign " + message);

        else         syntaxError(string, nextIndex, "Expected a Real number " + message);

        end if;

        else 
        end if;
      end scanReal;
      function scanInteger "Scan for the next Integer number and trigger an assert if not present"
        extends Modelica.Icons.Function;
        input String string "String to be scanned";
        input Integer startIndex(min = 1) = 1 "Start scanning of string at character startIndex";
        input Boolean unsigned = false "= true, if Integer token shall not start with a sign";
        input String message = "" "Message used in error message if scan is not successful";
        output Integer number "Value of Integer number";
        output Integer nextIndex "Index of character after the found number";
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
             number = Strings.<b>scanInteger</b>(string);
(number, nextIndex) = Strings.<b>scanInteger</b>(string, startIndex=1,
                                               unsigned=false, message=\"\");
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>scanInteger</b> scans the string starting at index
\"startIndex\", checks whether the next token is an Integer literal
and returns its value as an Integer number, as well as the
index directly after the Integer number. An assert is triggered,
if the scanned string does not contain an Integer literal with optional
leading white space.
</p>
</html>"));
      algorithm 
        (nextIndex,number):=Advanced.scanInteger(string, startIndex, unsigned);
        if nextIndex == startIndex then 
                nextIndex:=Advanced.skipWhiteSpace(string, startIndex);
        if unsigned then 
                syntaxError(string, nextIndex, "Expected an Integer number without a sign " + message);

        else         syntaxError(string, nextIndex, "Expected an Integer number " + message);

        end if;

        else 
        end if;
      end scanInteger;
      function scanBoolean "Scan for the next Boolean number and trigger an assert if not present"
        extends Modelica.Icons.Function;
        input String string "String to be scanned";
        input Integer startIndex(min = 1) = 1 "Start scanning of string at character startIndex";
        input String message = "" "Message used in error message if scan is not successful";
        output Boolean number "Value of Boolean";
        output Integer nextIndex "Index of character after the found number";
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
             number = Strings.<b>scanBoolean</b>(string);
(number, nextIndex) = Strings.<b>scanBoolean</b>(string, startIndex=1, message=\"\");
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>scanBoolean</b> scans the string starting at index
\"startIndex\", checks whether the next token is a Boolean literal
(i.e., is either the string \"false\" or \"true\", if converted to lower case letters)
and returns its value as a Boolean number, as well as the
index directly after the Boolean number. An assert is triggered,
if the scanned string does not contain a Boolean literal with optional
leading white space.
</p>
</html>"));
      protected 
        String identifier;
      algorithm 
        (nextIndex,identifier):=Advanced.scanIdentifier(string, startIndex);
        if nextIndex > startIndex then 
                if identifier == "false" then 
                number:=false;
        elseif identifier == "true" then
          number:=true;


        else         nextIndex:=startIndex;

        end if;

        else 
        end if;
        if nextIndex == startIndex then 
                nextIndex:=Advanced.skipWhiteSpace(string, startIndex);
        syntaxError(string, nextIndex, "Expected a Boolean constant, i.e., \"false\" or \"true\" " + message);

        else 
        end if;
      end scanBoolean;
      function scanString "Scan for the next Modelica string and trigger an assert if not present"
        extends Modelica.Icons.Function;
        input String string "String to be scanned";
        input Integer startIndex(min = 1) = 1 "Start scanning of string at character startIndex";
        input String message = "" "Message used in error message if scan is not successful";
        output String result "Value of string";
        output Integer nextIndex "Index of character after the found string";
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
             string2 = Strings.<b>scanString</b>(string);
(string2, nextIndex) = Strings.<b>scanString</b>(string, startIndex=1, message=\"\");
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>scanString</b> scans the string starting at index
\"startIndex\", checks whether the next token is a String literal
and returns its value as a String, as well as the
index directly after the String. An assert is triggered,
if the scanned string does not contain a String literal with optional
leading white space.
</p>
</html>"));
      algorithm 
        (nextIndex,result):=Advanced.scanString(string, startIndex);
        if nextIndex == startIndex then 
                nextIndex:=Advanced.skipWhiteSpace(string, startIndex);
        syntaxError(string, nextIndex, "Expected a string enclosed in double quotes " + message);

        else 
        end if;
      end scanString;
      function scanIdentifier "Scan for the next Identifier and trigger an assert if not present"
        extends Modelica.Icons.Function;
        input String string "String to be scanned";
        input Integer startIndex(min = 1) = 1 "Start scanning of identifier at character startIndex";
        input String message = "" "Message used in error message if scan is not successful";
        output String identifier "Value of Identifier";
        output Integer nextIndex "Index of character after the found identifier";
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
             identifier = Strings.<b>scanIdentifier</b>(string);
(identifier, nextIndex) = Strings.<b>scanIdentifier</b>(string, startIndex=1, message=\"\");
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>scanIdentifier</b> scans the string starting at index
\"startIndex\", checks whether the next token is an Identifier
and returns its value as a string, as well as the
index directly after the Identifier. An assert is triggered,
if the scanned string does not contain an Identifier with optional
leading white space.
</p>
</html>"));
      algorithm 
        (nextIndex,identifier):=Advanced.scanIdentifier(string, startIndex);
        if nextIndex == startIndex then 
                nextIndex:=Advanced.skipWhiteSpace(string, startIndex);
        syntaxError(string, nextIndex, "Expected an identifier " + message);

        else 
        end if;
      end scanIdentifier;
      function scanDelimiter "Scan for the next delimiter and trigger an assert if not present"
        extends Modelica.Icons.Function;
        input String string "String to be scanned";
        input Integer startIndex(min = 1) = 1 "Start scanning of delimiters at character startIndex";
        input String requiredDelimiters[:] = {","} "Delimiters that are searched";
        input String message = "" "Message used in error message if scan is not successful";
        output String delimiter "Found delimiter";
        output Integer nextIndex "Index of character after the found delimiter";
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
             delimiter = Strings.<b>scanDelimiter</b>(string);
(delimiter, nextIndex) = Strings.<b>scanDelimiter</b>(string, startIndex=1,
                                 requiredDelimiters={\",\"}, message=\"\");
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>scanDelimiter</b> scans the string starting at index
\"startIndex\", checks whether the next token is a delimiter string
and returns its value as a string, as well as the
index directly after the delimiter. An assert is triggered,
if the scanned string does not contain a delimiter out of the
list of requiredDelimiters. Input argument requiredDelimiters is a vector
of strings. The elements may have any length, including length 0.
If an element of the requiredDelimiters is zero, white space
is treated as delimiter. The function returns delimiter=\"\" and nextIndex
is the index of the first non white space character.
</p>
</html>"));
      protected 
        Integer lenString = length(string);
        Integer lenDelimiter;
        Integer nDelimiters = size(requiredDelimiters, 1);
        Integer endIndex;
        Boolean found;
        Integer i;
        encapsulated function concatenate "Concatenate strings together"
          input String strings[:];
          output String string;
        algorithm 
          string:="{\"";
          for i in 1:size(strings, 1) loop
                      if i == 1 then 
                        string:="{\"" + strings[1] + "\"";

            else             string:=string + ", \"" + strings[i] + "\"";

            end if;

          end for;
          string:=string + "}";
        end concatenate;
      algorithm 
        nextIndex:=Advanced.skipLineComments(string, startIndex);
        found:=false;
        i:=1;
        while (not found and i <= nDelimiters) loop
                  lenDelimiter:=length(requiredDelimiters[i]);
          if lenDelimiter == 0 then 
                    found:=true;
          delimiter:="";

          else           endIndex:=nextIndex + lenDelimiter - 1;
          if endIndex <= lenString then 
                    if substring(string, nextIndex, endIndex) == requiredDelimiters[i] then 
                    found:=true;
          delimiter:=requiredDelimiters[i];

          else 
          end if;

          else 
          end if;
          i:=i + 1;

          end if;

        end while;
        if found then 
                nextIndex:=nextIndex + lenDelimiter;

        else         if size(requiredDelimiters, 1) == 1 then 
                syntaxError(string, nextIndex, "Expected delimiter \"" + requiredDelimiters[1] + "\"\n" + message);

        else         syntaxError(string, nextIndex, "Expected a delimiter out of " + concatenate(requiredDelimiters) + "\n" + message);

        end if;

        end if;
      end scanDelimiter;
      function scanNoToken "Scan string and check that it contains no more token"
        extends Modelica.Icons.Function;
        input String string "String to be scanned";
        input Integer startIndex(min = 1) = 1 "Start scanning of string at character startIndex";
        input String message = "" "Message used in error message if scan is not successful";
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Strings.<b>scanNoToken</b>(string, startIndex=1, message=\"\");
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>scanNoToken</b> scans the string starting at index
\"startIndex\" and checks whether there is no more token in the
string. An assert is triggered if this is not the case,
using the \"message\" argument as additional explanation in
the error text.
</p>
</html>"));
      protected 
        Integer nextIndex;
      algorithm 
        nextIndex:=Advanced.skipLineComments(string, startIndex);
        if nextIndex <= length(string) then 
                syntaxError(string, nextIndex, "Expected no more token " + message);

        else 
        end if;
      end scanNoToken;
      function syntaxError "Print an error message, a string and the index at which scanning detected an error"
        extends Modelica.Icons.Function;
        input String string "String that has an error at position index";
        input Integer index "Index of string at which scanning detected an error";
        input String message = "" "String printed at end of error message";
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Strings.<b>syntaxError</b>(string, index, message);
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>syntaxError</b> prints an error message in the
following form:
</p>
<pre>
   Syntax error at column &lt;index&gt; of
   &lt;string&gt;
       ^       // shows character that is wrong
   &lt;message&gt;
</pre>
<p>
where the strings withing &lt;..&gt; are the actual values of the
input arguments of the function.
</p>
<p>
If the given string is too long, only a relevant
part of the string is printed.
</p>
</html>"));
      protected 
        Integer maxIndex = 40;
        Integer maxLenString = 60;
        Integer lenString = length(string);
        String errString;
        Integer index2 = if index < 1 then 1 else if index > lenString then lenString else index;
      algorithm 
        if index2 <= maxIndex then 
                errString:=string;

        else         errString:="... " + substring(string, index2 - maxIndex, lenString);
        index2:=maxIndex + 5;

        end if;
        if length(errString) > maxLenString then 
                errString:=substring(errString, 1, maxLenString) + " ...";

        else 
        end if;
        Streams.error("Syntax error at character " + String(index) + " of\n" + errString + "\n" + repeat(index2 - 1, " ") + "*" + "\n" + message);
      end syntaxError;
      package Advanced "Advanced scanning functions"
        annotation(Documentation(info = "<html>
<h4>Library content</h4>
<p>
Package <b>Strings.Advanced</b> contains basic scanning
functions. These functions should be <b>not called</b> directly, because
it is much simpler to utilize the higher level functions \"Strings.scanXXX\".
The functions of the \"Strings.Advanced\" library provide
the basic interface in order to implement the higher level
functions in package \"Strings\".
</p>
<p>
Library \"Advanced\" provides the following functions:
</p>
<pre>
  (nextIndex, realNumber)    = <a href=\"Modelica://Modelica.Utilities.Strings.Advanced.scanReal\">scanReal</a>        (string, startIndex, unsigned=false);
  (nextIndex, integerNumber) = <a href=\"Modelica://Modelica.Utilities.Strings.Advanced.scanInteger\">scanInteger</a>     (string, startIndex, unsigned=false);
  (nextIndex, string2)       = <a href=\"Modelica://Modelica.Utilities.Strings.Advanced.scanString\">scanString</a>      (string, startIndex);
  (nextIndex, identifier)    = <a href=\"Modelica://Modelica.Utilities.Strings.Advanced.scanIdentifier\">scanIdentifier</a>  (string, startIndex);
   nextIndex                 = <a href=\"Modelica://Modelica.Utilities.Strings.Advanced.skipWhiteSpace\">skipWhiteSpace</a>  (string, startIndex);
   nextIndex                 = <a href=\"Modelica://Modelica.Utilities.Strings.Advanced.skipLineComments\">skipLineComments</a>(string, startIndex);
</pre>
<p>
All functions perform the following actions:
</p>
<ol>
<li> Scanning starts at character position \"startIndex\" of
     \"string\" (startIndex has a default of 1).
<li> First, white space is skipped, such as blanks (\" \"), tabs (\"\\t\"), or newline (\"\\n\")</li>
<li> Afterwards, the required token is scanned.</li>
<li> If successful, on return nextIndex = index of character
     directly after the found token and the token value is returned
     as second output argument.<br>
     If not successful, on return nextIndex = startIndex.
     </li>
</ol>
<p>
The following additional rules apply for the scanning:
</p>
<ul>
<li> Function <a href=\"Modelica://Modelica.Utilities.Strings.Advanced.scanReal\">scanReal</a>:<br>
     Scans a full number including one optional leading \"+\" or \"-\" (if unsigned=false)
     according to the Modelica grammar. For example, \"+1.23e-5\", \"0.123\" are
     Real numbers, but \".1\" is not.
     Note, an Integer number, such as \"123\" is also treated as a Real number.<br>&nbsp;</li>
<li> Function <a href=\"Modelica://Modelica.Utilities.Strings.Advanced.scanInteger\">scanInteger</a>:<br>
     Scans an Integer number including one optional leading \"+\"
     or \"-\" (if unsigned=false) according to the Modelica (and C/C++) grammar.
     For example, \"+123\", \"20\" are Integer numbers.
     Note, a Real number, such as \"123.4\" is not an Integer and
     scanInteger returns nextIndex = startIndex.<br>&nbsp;</li>
<li> Function <a href=\"Modelica://Modelica.Utilities.Strings.Advanced.scanString\">scanString</a>:<br>
     Scans a String according to the Modelica (and C/C++) grammar, e.g.,
     \"This is a \"string\"\" is a valid string token.<br>&nbsp;</li>
<li> Function <a href=\"Modelica://Modelica.Utilities.Strings.Advanced.scanIdentifier\">scanIdentifier</a>:<br>
     Scans a Modelica identifier, i.e., the identifier starts either
     with a letter, followed by letters, digits or \"_\".
     For example, \"w_rel\", \"T12\".<br>&nbsp;</li>
<li> Function <a href=\"Modelica://Modelica.Utilities.Strings.Advanced.skipLineComments\">skipLineComments</a><br>
     Skips white space and Modelica (C/C++) line comments iteratively.
     A line comment starts with \"//\" and ends either with an
     end-of-line (\"\\n\") or the end of the \"string\". </li>
</ul>
</html>
"));
        function scanReal "Scans a signed real number"
          extends Modelica.Icons.Function;
          input String string;
          input Integer startIndex(min = 1) = 1 "Index where scanning starts";
          input Boolean unsigned = false "= true, if number shall not start with '+' or '-'";
          output Integer nextIndex "Index after the found token (success=true) or index at which scanning failed (success=false)";
          output Real number "Value of Real number";

          external "C" ModelicaStrings_scanReal(string,startIndex,unsigned,nextIndex,number) ;           annotation(Library = "ModelicaExternalC", Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
(nextIndex, realNumber) = <b>scanReal</b>(string, startIndex=1, unsigned=false);
</pre></blockquote>
<h4>Description</h4>
<p>
Starts scanning of \"string\" at position \"startIndex\".
First skips white space and scans afterwards a number
of type Real with an optional sign according to the Modelica grammar:
</p>
<pre>
    real     ::= [sign] unsigned [fraction] [exponent]
    sign     ::= '+' | '-'
    unsigned ::= digit [unsigned]
    fraction ::= '.' [unsigned]
    exponent ::= ('e' | 'E') [sign] unsigned
    digit    ::= '0'|'1'|'2'|'3'|'4'|'5'|'6'|'7'|'8'|'9'
</pre>
<p>
If successful, the function returns nextIndex = index of character
directly after the found real number, as well as the value
in the second output argument.
</p>
<p>
If not successful, on return nextIndex = startIndex and
the second output argument is zero.
</p>
<p>
If the optional argument \"unsigned\" is <b>true</b>, the number
shall not start with '+' or '-'. The default of \"unsigned\" is <b>false</b>.
</p>
<h4>See also</h4>
<a href=\"Modelica://Modelica.Utilities.Strings.Advanced\">Strings.Advanced</a>.
</html>
"));
        end scanReal;
        function scanInteger "Scans signed integer number"
          extends Modelica.Icons.Function;
          input String string;
          input Integer startIndex(min = 1) = 1;
          input Boolean unsigned = false "= true, if number shall not start with '+' or '-'";
          output Integer nextIndex "Index after the found token (success=true) or index at which scanning failed (success=false)";
          output Integer number "Value of Integer number";

          external "C" ModelicaStrings_scanInteger(string,startIndex,unsigned,nextIndex,number) ;           annotation(Library = "ModelicaExternalC", Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
(nextIndex, integerNumber) = <b>scanInteger</b>(string, startIndex=1, unsigned=false);
</pre></blockquote>
<h4>Description</h4>
<p>
Starts scanning of \"string\" at position \"startIndex\".
First skips white space and scans afterwards a signed number
of type Integer. An Integer starts with an optional '+'
or '-', immediately
followed by a non-empty sequence of digits.
</p>
<p>
If successful, the function returns nextIndex = index of character
directly after the found Integer number, as well as the Integer value
in the second output argument.
</p>
<p>
If not successful, on return nextIndex = startIndex and
the second output argument is zero.
</p>
<p>
Note, a Real number, such as \"123.4\", is not treated
as an Integer number and scanInteger will return
nextIndex = startIndex in this case.
</p>
<p>
If the optional argument \"unsigned\" is <b>true</b>, the number
shall not start with '+' or '-'. The default of \"unsigned\" is <b>false</b>.
</p>
<h4>See also</h4>
<a href=\"Modelica://Modelica.Utilities.Strings.Advanced\">Strings.Advanced</a>.
</html>
"));
        end scanInteger;
        function scanString "Scan string"
          extends Modelica.Icons.Function;
          input String string;
          input Integer startIndex(min = 1) = 1 "Index where scanning starts";
          output Integer nextIndex "Index after the found token (success=true) or index at which scanning failed (success=false)";
          output String string2 "Value of String token";

          external "C" ModelicaStrings_scanString(string,startIndex,nextIndex,string2) ;           annotation(Library = "ModelicaExternalC", Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
(nextIndex, string2) = <b>scanString</b>(string, startIndex=1);
</pre></blockquote>
<h4>Description</h4>
<p>
Starts scanning of \"string\" at position \"startIndex\".
First skips white space and scans afterwards a string
according to the Modelica grammar, i.e., a string
enclosed in double quotes.
</p>
<p>
If successful, the function returns nextIndex = index of character
directly after the found string, as well as the string value
in the second output argument.
</p>
<p>
If not successful, on return nextIndex = startIndex and
the second output argument is an empty string.
</p>
<h4>See also</h4>
<a href=\"Modelica://Modelica.Utilities.Strings.Advanced\">Strings.Advanced</a>.
</html>
"));
        end scanString;
        function scanIdentifier "Scans simple identifiers"
          extends Modelica.Icons.Function;
          input String string;
          input Integer startIndex(min = 1) = 1 "Index where scanning starts";
          output Integer nextIndex "Index after the found token (success=true) or index at which scanning failed (success=false)";
          output String identifier "Value of identifier token";

          external "C" ModelicaStrings_scanIdentifier(string,startIndex,nextIndex,identifier) ;           annotation(Library = "ModelicaExternalC", Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
(nextIndex, identifier) = <b>scanIdentifier</b>(string, startIndex=1);
</pre></blockquote>
<h4>Description</h4>
<p>
Starts scanning of \"string\" at position \"startIndex\".
First skips white space and scans afterwards a Modelica
identifier, i.e., a sequence of characters starting with
a letter (\"a\"..\"z\" or \"A\"..\"Z\") followed by letters,
digits or underscores (\"_\").
</p>
<p>
If successful, the function returns nextIndex = index of character
directly after the found identifier, as well as the identifier
as string in the second output argument.
</p>
<p>
If not successful, on return nextIndex = startIndex and
the second output argument is an empty string.
</p>
<h4>See also</h4>
<a href=\"Modelica://Modelica.Utilities.Strings.Advanced\">Strings.Advanced</a>.
</html>
"));
        end scanIdentifier;
        function skipWhiteSpace "Scans white space"
          extends Modelica.Icons.Function;
          input String string;
          input Integer startIndex(min = 1) = 1;
          output Integer nextIndex;

          external "C" nextIndex = ModelicaStrings_skipWhiteSpace(string,startIndex) ;          annotation(Library = "ModelicaExternalC", Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
nextIndex = <b>skipWhiteSpace</b>(string, startIndex);
</pre></blockquote>
<h4>Description</h4>
<p>
Starts scanning of \"string\" at position \"startIndex\" and
skips white space. The function returns nextIndex = index of character
of the first non white space character.
</p>
<h4>See also</h4>
<a href=\"Modelica://Modelica.Utilities.Strings.Advanced\">Strings.Advanced</a>.
</html>
"));
        end skipWhiteSpace;
        function skipLineComments "Scans comments and white space"
          extends Modelica.Icons.Function;
          input String string;
          input Integer startIndex(min = 1) = 1;
          output Integer nextIndex;
          annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
nextIndex = <b>skipLineComments</b>(string, startIndex);
</pre></blockquote>
<h4>Description</h4>
<p>
Starts scanning of \"string\" at position \"startIndex\".
First skips white space and scans afterwards a Modelica (C/C++)
line comment, i.e., a sequence of characters that
starts with \"//\" and ends with an end-of-line \"\\n\" or
with the end of the string. If end-of-line is reached,
the function continues to skip white space and
scanning of line comments until end-of-string is
reached, or another token is detected.
</p>
<p>
If successful, the function returns nextIndex = index of character
directly after the found line comment.
</p>
<p>
If not successful, on return nextIndex = startIndex.
</p>
<h4>See also</h4>
<a href=\"Modelica://Modelica.Utilities.Strings.Advanced\">Strings.Advanced</a>.
</html>
"));
        protected 
          Integer lenString = length(string);
          Boolean scanning;
          Boolean lineComment;
        algorithm 
          nextIndex:=startIndex;
          scanning:=true;
          while (scanning) loop
                      nextIndex:=Advanced.skipWhiteSpace(string, nextIndex);
            if nextIndex + 1 <= lenString then 
                        if substring(string, nextIndex, nextIndex + 1) == "//" then 
                        nextIndex:=nextIndex + 2;
            if nextIndex <= lenString then 
                        lineComment:=true;
            while (lineComment) loop
                          if substring(string, nextIndex, nextIndex) == "\n" then 
                            lineComment:=false;

              else 
              end if;
              nextIndex:=nextIndex + 1;
              if nextIndex > lenString then 
                            lineComment:=false;
              scanning:=false;

              else 
              end if;

            end while;

            else             scanning:=false;

            end if;

            else             scanning:=false;

            end if;

            else             scanning:=false;

            end if;

          end while;
        end skipLineComments;
      end Advanced;
    end Strings;
    package Streams "Read from files and write to files"
      extends Modelica.Icons.Library;
      annotation(Documentation(info = "<HTML>
<h4>Library content</h4>
<p>
Package <b>Streams</b> contains functions to input and output strings
to a message window or on files. Note that a string is interpreted
and displayed as html text (e.g., with print(..) or error(..))
if it is enclosed with the Modelica html quotation, e.g.,
</p>
<center>
string = \"&lt;html&gt; first line &lt;br&gt; second line &lt;/html&gt;\".
</center>
<p>
It is a quality of implementation, whether (a) all tags of html are supported
or only a subset, (b) how html tags are interpreted if the output device
does not allow to display formatted text.
</p>
<p>
In the table below an example call to every function is given:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Function/type</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Utilities.Streams.print\">print</a>(string)<br>
          <a href=\"Modelica://Modelica.Utilities.Streams.print\">print</a>(string,fileName)</td>
      <td valign=\"top\"> Print string \"string\" or vector of strings to message window or on
           file \"fileName\".</td>
  </tr>
  <tr><td valign=\"top\">stringVector =
         <a href=\"Modelica://Modelica.Utilities.Streams.readFile\">readFile</a>(fileName)</td>
      <td valign=\"top\"> Read complete text file and return it as a vector of strings.</td>
  </tr>
  <tr><td valign=\"top\">(string, endOfFile) =
         <a href=\"Modelica://Modelica.Utilities.Streams.readLine\">readLine</a>(fileName, lineNumber)</td>
      <td valign=\"top\">Returns from the file the content of line lineNumber.</td>
  </tr>
  <tr><td valign=\"top\">lines =
         <a href=\"Modelica://Modelica.Utilities.Streams.countLines\">countLines</a>(fileName)</td>
      <td valign=\"top\">Returns the number of lines in a file.</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Utilities.Streams.error\">error</a>(string)</td>
      <td valign=\"top\"> Print error message \"string\" to message window
           and cancel all actions</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Utilities.Streams.close\">close</a>(fileName)</td>
      <td valign=\"top\"> Close file if it is still open. Ignore call if
           file is already closed or does not exist. </td>
  </tr>
</table>
<p>
Use functions <b>scanXXX</b> from package
<a href=\"Modelica://Modelica.Utilities.Strings\">Strings</a>
to parse a string.
</p>
<p>
If Real, Integer or Boolean values shall be printed
or used in an error message, they have to be first converted
to strings with the builtin operator
<a href=\"Modelica://ModelicaReference.Operators.string\">String</a>(...).
Example:
</p>
<pre>
  <b>if</b> x &lt; 0 <b>or</b> x &gt; 1 <b>then</b>
     Streams.error(\"x (= \" + String(x) + \") has to be in the range 0 .. 1\");
  <b>end if</b>;
</pre>
</HTML>
"));
      function print "Print string to terminal or file"
        extends Modelica.Icons.Function;
        input String string = "" "String to be printed";
        input String fileName = "" "File where to print (empty string is the terminal)" annotation(Dialog(__Dymola_saveSelector(filter = "Text files (*.txt)", caption = "Text file to store the output of print(..)")));

        external "C" ModelicaInternal_print(string,fileName) ;         annotation(Library = "ModelicaExternalC", Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Streams.<b>print</b>(string);
Streams.<b>print</b>(string,fileName);
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>print</b>(..) opens automatically the given file, if
it is not yet open. If the file does not exist, it is created.
If the file does exist, the given string is appended to the file.
If this is not desired, call \"Files.remove(fileName)\" before calling print
(\"remove(..)\" is silent, if the file does not exist).
The Modelica environment may close the file whenever appropriate.
This can be enforced by calling <b>Streams.close</b>(fileName).
After every call of \"print(..)\" a \"new line\" is printed automatically.
</p>
<h4>Example</h4>
<blockquote><pre>
  Streams.print(\"x = \" + String(x));
  Streams.print(\"y = \" + String(y));
  Streams.print(\"x = \" + String(y), \"mytestfile.txt\");
</pre></blockquote>
<p>
<h4>See also</h4>
<p>
<a href=\"Modelica://Modelica.Utilities.Streams\">Streams</a>,
<a href=\"Modelica://Modelica.Utilities.Streams.error\">Streams.error</a>,
<a href=\"Modelica://ModelicaReference.Operators.string\">String</a>
</p>
</HTML>"));
      end print;
      function readFile "Read content of a file and return it in a vector of strings"
        extends Modelica.Icons.Function;
        input String fileName "Name of the file that shall be read" annotation(Dialog(__Dymola_loadSelector(filter = "Text files (*.txt)", caption = "Open text file for reading")));
        output String stringVector[countLines(fileName)] "Content of file";
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
stringVector = Streams.<b>readFile</b>(fileName)
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>readFile</b>(..) opens the given file, reads the complete
content, closes the file and returns the content as a vector of strings. Lines are separated by LF or CR-LF; the returned strings do not contain the line separators.
</p>
</html>"));
      algorithm 
        for i in 1:size(stringVector, 1) loop
                  stringVector[i]:=readLine(fileName, i);

        end for;
        Streams.close(fileName);
      end readFile;
      function readLine "Reads a line of text from a file and returns it in a string"
        extends Modelica.Icons.Function;
        input String fileName "Name of the file that shall be read" annotation(Dialog(__Dymola_loadSelector(filter = "Text files (*.txt)", caption = "Open text file for reading")));
        input Integer lineNumber(min = 1) "Number of line to read";
        output String string "Line of text";
        output Boolean endOfFile "If true, end-of-file was reached when trying to read line";

        external "C" string = ModelicaInternal_readLine(fileName,lineNumber,endOfFile) ;        annotation(Library = "ModelicaExternalC", Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
(string, endOfFile) = Streams.<b>readLine</b>(fileName, lineNumber)
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>readLine</b>(..) opens the given file, reads enough of the
content to get the requested line, and returns the line as a string.
Lines are separated by LF or CR-LF; the returned string does not
contain the line separator. The file might remain open after
the call.
</p>
<p>
If lineNumber > countLines(fileName), an empty string is returned
and endOfFile=true. Otherwise endOfFile=false.
</p>
</html>"));
      end readLine;
      function countLines "Returns the number of lines in a file"
        extends Modelica.Icons.Function;
        input String fileName "Name of the file that shall be read" annotation(Dialog(__Dymola_loadSelector(filter = "Text files (*.txt)", caption = "Open text file for coutning lines")));
        output Integer numberOfLines "Number of lines in file";

        external "C" numberOfLines = ModelicaInternal_countLines(fileName) ;        annotation(Library = "ModelicaExternalC", Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
numberOfLines = Streams.<b>countLines</b>(fileName)
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>countLines</b>(..) opens the given file, reads the complete
content, closes the file and returns the number of lines. Lines are
separated by LF or CR-LF.
</p>
</html>"));
      end countLines;
      function error "Print error message and cancel all actions"
        extends Modelica.Icons.Function;
        input String string "String to be printed to error message window";

        external "C" ModelicaError(string) ;         annotation(Library = "ModelicaExternalC", Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Streams.<b>error</b>(string);
</pre></blockquote>
<h4>Description</h4>
<p>
Print the string \"string\" as error message and
cancel all actions. Line breaks are characterized
by \"\\n\" in the string.
</p>
<h4>Example</h4>
<blockquote><pre>
  Streams.error(\"x (= \" + String(x) + \")\\nhas to be in the range 0 .. 1\");
</pre></blockquote>
<h4>See also</h4>
<p>
<a href=\"Modelica://Modelica.Utilities.Streams\">Streams</a>,
<a href=\"Modelica://Modelica.Utilities.Streams.print\">Streams.print</a>,
<a href=\"Modelica://ModelicaReference.Operators.string\">String</a>
</p>
</html>"));
      end error;
      function close "Close file"
        extends Modelica.Icons.Function;
        input String fileName "Name of the file that shall be closed" annotation(Dialog(__Dymola_loadSelector(filter = "Text files (*.txt)", caption = "Close text file")));

        external "C" ModelicaStreams_closeFile(fileName) ;         annotation(Library = "ModelicaExternalC", Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Streams.<b>close</b>(fileName)
</pre></blockquote>
<h4>Description</h4>
<p>
Close file if it is open. Ignore call if
file is already closed or does not exist.
</p>
</html>"));
      end close;
    end Streams;
    package Internal "Internal components that a user should usually not directly utilize"
      partial package PartialModelicaServices "Interfaces of components requiring a tool specific implementation"
        package Animation "Models and functions for 3-dim. animation"
          partial model PartialShape "Different visual shapes with variable size; all data have to be set as modifiers"
            import SI = Modelica.SIunits;
            import Modelica.Mechanics.MultiBody.Frames;
            import Modelica.Mechanics.MultiBody.Types;
            parameter Types.ShapeType shapeType = "box" "Type of shape (box, sphere, cylinder, pipecylinder, cone, pipe, beam, gearwheel, spring)";
            input Frames.Orientation R = Frames.nullRotation() "Orientation object to rotate the world frame into the object frame" annotation(Dialog);
            input SI.Position r[3] = {0,0,0} "Position vector from origin of world frame to origin of object frame, resolved in world frame" annotation(Dialog);
            input SI.Position r_shape[3] = {0,0,0} "Position vector from origin of object frame to shape origin, resolved in object frame" annotation(Dialog);
            input Real lengthDirection[3](each final unit = "1") = {1,0,0} "Vector in length direction, resolved in object frame" annotation(Dialog);
            input Real widthDirection[3](each final unit = "1") = {0,1,0} "Vector in width direction, resolved in object frame" annotation(Dialog);
            input SI.Length length = 0 "Length of visual object" annotation(Dialog);
            input SI.Length width = 0 "Width of visual object" annotation(Dialog);
            input SI.Length height = 0 "Height of visual object" annotation(Dialog);
            input Types.ShapeExtra extra = 0.0 "Additional size data for some of the shape types" annotation(Dialog);
            input Real color[3] = {255,0,0} "Color of shape" annotation(Dialog);
            input Types.SpecularCoefficient specularCoefficient = 0.7 "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog);
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100, -100},{80,60}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -100,60},{ -80,100},{100,100},{80,60},{ -100,60}}, lineColor = {0,0,255}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{100,100},{100, -60},{80, -100},{80,60},{100,100}}, lineColor = {0,0,255}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Text(extent = {{ -100, -100},{80,60}}, lineColor = {0,0,0}, textString = "%shapeType"),Text(extent = {{ -132,160},{128,100}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<html>

<p>
This model is documented at
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape\">Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape</a>.
</p>

</html>
"));
          end PartialShape;
        end Animation;
        annotation(Documentation(info = "<html>

<p>
This package contains interfaces of a set of functions and models used in the
Modelica Standard Library that requires a <u>tool specific implementation</u>.
There is an associated package called <u>ModelicaServices</u>. A tool vendor
should provide a proper implementation of this library for the corresponding
tool. The default implementation is \"do nothing\".
In the Modelica Standard Library, the models and functions of ModelicaServices
are used.
</p>

</html>"));
      end PartialModelicaServices;
    protected 
      package FileSystem "Internal package with external functions as interface to the file system"
        extends Modelica.Icons.Library;
        annotation(Documentation(info = "<html>
<p>
Package <b>Internal.FileSystem</b> is an internal package that contains
low level functions as interface to the file system.
These functions should not be called directly in a scripting
environment since more convenient functions are provided
in packages Files and Systems.
</p>
<p>
Note, the functions in this package are direct interfaces to
functions of POSIX and of the standard C library. Errors
occuring in these functions are treated by triggering
a Modelica assert. Therefore, the functions in this package
return only for a successful operation. Furthermore, the
representation of a string is hidden by this interface,
especially if the operating system supports Unicode characters.
</p>
</html>"));
        function mkdir "Make directory (POSIX: 'mkdir')"
          extends Modelica.Icons.Function;
          input String directoryName "Make a new directory";

          external "C" ModelicaInternal_mkdir(directoryName) ;           annotation(Library = "ModelicaExternalC");
        end mkdir;
        function rmdir "Remove empty directory (POSIX function 'rmdir')"
          extends Modelica.Icons.Function;
          input String directoryName "Empty directory to be removed";

          external "C" ModelicaInternal_rmdir(directoryName) ;           annotation(Library = "ModelicaExternalC");
        end rmdir;
        function stat "Inquire file information (POSIX function 'stat')"
          extends Modelica.Icons.Function;
          input String name "Name of file, directory, pipe etc.";
          output Types.FileType fileType "Type of file";

          external "C" fileType = ModelicaInternal_stat(name) ;          annotation(Library = "ModelicaExternalC");
        end stat;
        function rename "Rename existing file or directory (C function 'rename')"
          extends Modelica.Icons.Function;
          input String oldName "Current name";
          input String newName "New name";

          external "C" ModelicaInternal_rename(oldName,newName) ;           annotation(Library = "ModelicaExternalC");
        end rename;
        function removeFile "Remove existing file (C function 'remove')"
          extends Modelica.Icons.Function;
          input String fileName "File to be removed";

          external "C" ModelicaInternal_removeFile(fileName) ;           annotation(Library = "ModelicaExternalC");
        end removeFile;
        function copyFile "Copy existing file (C functions 'fopen', 'getc', 'putc', 'fclose')"
          extends Modelica.Icons.Function;
          input String fromName "Name of file to be copied";
          input String toName "Name of copy of file";

          external "C" ModelicaInternal_copyFile(fromName,toName) ;           annotation(Library = "ModelicaExternalC");
        end copyFile;
        function readDirectory "Read names of a directory (POSIX functions opendir, readdir, closedir)"
          extends Modelica.Icons.Function;
          input String directory "Name of the directory from which information is desired";
          input Integer nNames "Number of names that are returned (inquire with getNumberOfFiles)";
          output String names[nNames] "All file and directory names in any order from the desired directory";

          external "C" ModelicaInternal_readDirectory(directory,nNames,names) ;           annotation(Library = "ModelicaExternalC");
        end readDirectory;
        function getNumberOfFiles "Get number of files and directories in a directory (POSIX functions opendir, readdir, closedir)"
          extends Modelica.Icons.Function;
          input String directory "Directory name";
          output Integer result "Number of files and directories present in 'directory'";

          external "C" result = ModelicaInternal_getNumberOfFiles(directory) ;          annotation(Library = "ModelicaExternalC");
        end getNumberOfFiles;
      end FileSystem;
    end Internal;
    package Files "Functions to work with files and directories"
      function list "List content of file or directory"
        extends Modelica.Icons.Function;
        input String name "If name is a directory, list directory content. If it is a file, list the file content";
      protected 
        Types.FileType fileType;
        function listFile "List content of file"
          input String name;
        protected 
          String file[Streams.countLines(name)] = Streams.readFile(name);
        algorithm 
          for i in 1:min(size(file, 1), 100) loop
                      Streams.print(file[i]);

          end for;
        end listFile;
        function sortDirectory "Sort directory in directories and files with alphabetic order"
          input String directory "Directory that was read (including a trailing '/')";
          input String names[:] "File and directory names of a directory in any order";
          output String orderedNames[size(names, 1)] "Names of directories followed by names of files";
          output Integer nDirectories "The first nDirectories entries in orderedNames are directories";
        protected 
          Integer nEntries = size(names, 1);
          Integer nFiles;
          Integer lenDirectory = Strings.length(directory);
          String directory2;
        algorithm 
          directory2:=if Strings.substring(directory, lenDirectory, lenDirectory) == "/" then directory else directory + "/";
          nDirectories:=0;
          nFiles:=0;
          for i in 1:nEntries loop
                      if Modelica.Utilities.Internal.FileSystem.stat(directory2 + names[i]) == Types.FileType.Directory then 
                        nDirectories:=nDirectories + 1;
            orderedNames[nDirectories]:=names[i];

            else             nFiles:=nFiles + 1;
            orderedNames[nEntries - nFiles + 1]:=names[i];

            end if;

          end for;
          if nDirectories > 0 then 
                    orderedNames[1:nDirectories]:=Strings.sort(orderedNames[1:nDirectories], caseSensitive = false);

          else 
          end if;
          if nFiles > 0 then 
                    orderedNames[nDirectories + 1:nEntries]:=Strings.sort(orderedNames[nDirectories + 1:nEntries], caseSensitive = false);

          else 
          end if;
        end sortDirectory;
        function listDirectory "List content of directory"
          input String directoryName;
          input Integer nEntries;
        protected 
          String files[nEntries];
          Integer nDirectories;
        algorithm 
          if nEntries > 0 then 
                    Streams.print("\nDirectory \"" + directoryName + "\":");
          files:=Modelica.Utilities.Internal.FileSystem.readDirectory(directoryName, nEntries);
          (files,nDirectories):=sortDirectory(directoryName, files);
          if nDirectories > 0 then 
                    Streams.print("  Subdirectories:");
          for i in 1:nDirectories loop
                      Streams.print("    " + files[i]);

          end for;
          Streams.print(" ");

          else 
          end if;
          if nDirectories < nEntries then 
                    Streams.print("  Files:");
          for i in nDirectories + 1:nEntries loop
                      Streams.print("    " + files[i]);

          end for;

          else 
          end if;

          else           Streams.print("... Directory\"" + directoryName + "\" is empty");

          end if;
        end listDirectory;
      algorithm 
        fileType:=Modelica.Utilities.Internal.FileSystem.stat(name);
        if fileType == Types.FileType.RegularFile then 
                listFile(name);
        elseif fileType == Types.FileType.Directory then
          listDirectory(name, Modelica.Utilities.Internal.FileSystem.getNumberOfFiles(name));

        elseif fileType == Types.FileType.SpecialFile then
          Streams.error("Cannot list file \"" + name + "\"\n" + "since it is not a regular file (pipe, device, ...)");


        else         Streams.error("Cannot list file or directory \"" + name + "\"\n" + "since it does not exist");

        end if;
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Files.<b>list</b>(name);
</pre></blockquote>
<h4>Description</h4>
<p>
If \"name\" is a regular file, the content of the
file is printed.
</p>
<p>
If \"name\" is a directory, the directory and file names
in the \"name\" directory are printed in sorted order.
</p>
</html>"));
      end list;
      extends Modelica.Icons.Library;
      annotation(Documentation(info = "<HTML>
<p>
This package contains functions to work with files and directories.
As a general convention of this package, '/' is used as directory
separator both for input and output arguments of all functions.
For example:
</p>
<pre>
   exist(\"Modelica/Mechanics/Rotational.mo\");
</pre>
<p>
The functions provide the mapping to the directory separator of the
underlying operating system. Note, that on Windows system the usage
of '\\' as directory separator would be inconvenient, because this
character is also the escape character in Modelica and C Strings.
</p>
<p>
In the table below an example call to every function is given:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Function/type</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Utilities.Files.list\">list</a>(name)</td>
      <td valign=\"top\"> List content of file or of directory.</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Utilities.Files.copy\">copy</a>(oldName, newName)<br>
          <a href=\"Modelica://Modelica.Utilities.Files.copy\">copy</a>(oldName, newName, replace=false)</td>
      <td valign=\"top\"> Generate a copy of a file or of a directory.</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Utilities.Files.move\">move</a>(oldName, newName)<br>
          <a href=\"Modelica://Modelica.Utilities.Files.move\">move</a>(oldName, newName, replace=false)</td>
      <td valign=\"top\"> Move a file or a directory to another place.</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Utilities.Files.remove\">remove</a>(name)</td>
      <td valign=\"top\"> Remove file or directory (ignore call, if it does not exist).</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Utilities.Files.removeFile\">removeFile</a>(name)</td>
      <td valign=\"top\"> Remove file (ignore call, if it does not exist)</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Utilities.Files.createDirectory\">createDirectory</a>(name)</td>
      <td valign=\"top\"> Create directory (if directory already exists, ignore call).</td>
  </tr>
  <tr><td valign=\"top\">result = <a href=\"Modelica://Modelica.Utilities.Files.exist\">exist</a>(name)</td>
      <td valign=\"top\"> Inquire whether file or directory exists.</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Utilities.Files.assertNew\">assertNew</a>(name,message)</td>
      <td valign=\"top\"> Trigger an assert, if a file or directory exists.</td>
  </tr>
  <tr><td valign=\"top\">fullName = <a href=\"Modelica://Modelica.Utilities.Files.fullPathName\">fullPathName</a>(name)</td>
      <td valign=\"top\"> Get full path name of file or directory name.</td>
  </tr>
  <tr><td valign=\"top\">(directory, name, extension) = <a href=\"Modelica://Modelica.Utilities.Files.splitPathName\">splitPathName</a>(name)</td>
      <td valign=\"top\"> Split path name in directory, file name kernel, file name extension.</td>
  </tr>
  <tr><td valign=\"top\">fileName = <a href=\"Modelica://Modelica.Utilities.Files.temporaryFileName\">temporaryFileName</a>()</td>
      <td valign=\"top\"> Return arbitrary name of a file that does not exist<br>
           and is in a directory where access rights allow to <br>
           write to this file (useful for temporary output of files).</td>
  </tr>
</table>
</HTML>"));
      function copy "Generate a copy of a file or of a directory"
        extends Modelica.Icons.Function;
        input String oldName "Name of file or directory to be copied";
        input String newName "Name of copy of the file or of the directory";
        input Boolean replace = false "= true, if an existing file may be replaced by the required copy";
      protected 
        function copyDirectory "Copy a directory"
          input String oldName "Old directory name without trailing '/'; existance is guaranteed";
          input String newName "New diretory name without trailing '/'; directory was already created";
          input Boolean replace "= true, if an existing newName may be replaced";
        protected 
          Integer nNames = Modelica.Utilities.Internal.FileSystem.getNumberOfFiles(oldName);
          String oldNames[nNames];
          String oldName_i;
          String newName_i;
        algorithm 
          oldNames:=Modelica.Utilities.Internal.FileSystem.readDirectory(oldName, nNames);
          for i in 1:nNames loop
                      oldName_i:=oldName + "/" + oldNames[i];
            newName_i:=newName + "/" + oldNames[i];
            Files.copy(oldName_i, newName_i, replace);

          end for;
        end copyDirectory;
        Integer lenOldName = Strings.length(oldName);
        Integer lenNewName = Strings.length(newName);
        String oldName2 = if Strings.substring(oldName, lenOldName, lenOldName) == "/" then Strings.substring(oldName, 1, lenOldName - 1) else oldName;
        String newName2 = if Strings.substring(newName, lenNewName, lenNewName) == "/" then Strings.substring(newName, 1, lenNewName - 1) else newName;
        Types.FileType oldFileType = Modelica.Utilities.Internal.FileSystem.stat(oldName2);
        Types.FileType newFileType;
      algorithm 
        if oldFileType == Types.FileType.NoFile then 
                Streams.error("It is not possible to copy the file or directory\n" + "\"" + oldName2 + "\" because it does not exist.");
        elseif oldFileType == Types.FileType.Directory then
          newFileType:=Modelica.Utilities.Internal.FileSystem.stat(newName2);
          if newFileType == Types.FileType.NoFile then 
                    createDirectory(newName2);
          elseif newFileType == Types.FileType.RegularFile or newFileType == Types.FileType.SpecialFile then
            if replace then 
                        Files.removeFile(newName2);
            Files.createDirectory(newName2);

            else             Streams.error("Directory \"" + oldName2 + "\" should be copied to\n" + "\"" + newName2 + "\" which is an existing file.\n" + "Since argument replace=false, this is not allowed");

            end if;


          else 
          end if;
          copyDirectory(oldName2, newName2, replace);


        else         if replace then 
                Files.removeFile(newName2);

        else         Files.assertNew(newName2, "File \"" + oldName2 + "\" should be copied or moved to\n" + "\"" + newName2 + "\" which is an existing file or directory.\n" + "Since argument replace=false, this is not allowed");

        end if;
        Modelica.Utilities.Internal.FileSystem.copyFile(oldName2, newName2);

        end if;
        annotation(Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Files.<b>copy</b>(oldName, newName);
Files.<b>copy</b>(oldName, newName, replace = true);
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>copy</b>(..) copies a file or a directory
to a new location. Via the optional argument <b>replace</b>
it can be defined whether an already existing file may
be replaced by the required copy.
</p>
<p>
If oldName/newName are directories, then the newName
directory may exist. In such a case the content of oldName
is copied into directory newName. If replace = <b>false</b>
it is required that the existing files
in newName are different from the existing files in
oldName.
</p>
<h4>Example</h4>
<blockquote><pre>
  copy(\"C:/test1/directory1\", \"C:/test2/directory2\");
     -> the content of directory1 is copied into directory2
        if \"C:/test2/directory2\" does not exist, it is newly
        created. If \"replace=true\", files in directory2
        may be overwritten by their copy
  copy(\"test1.txt\", \"test2.txt\")
     -> make a copy of file \"test1.txt\" with the name \"test2.txt\"
        in the current directory
</pre></blockquote>
</HTML>"));
      end copy;
      function move "Move a file or a directory to another place"
        extends Modelica.Icons.Function;
        input String oldName "Name of file or directory to be moved";
        input String newName "New name of the moved file or directory";
        input Boolean replace = false "= true, if an existing file or directory may be replaced";
      algorithm 
        if Strings.find(oldName, "/") == 0 and Strings.find(newName, "/") == 0 then 
                Modelica.Utilities.Internal.FileSystem.rename(oldName, newName);

        else         Files.copy(oldName, newName, replace);
        Files.remove(oldName);

        end if;
        annotation(Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Files.<b>move</b>(oldName, newName);
Files.<b>move</b>(oldName, newName, replace = true);
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>move</b>(..) moves a file or a directory
to a new location. Via the optional argument <b>replace</b>
it can be defined whether an already existing file may
be replaced.
</p>
<p>
If oldName/newName are directories, then the newName
directory may exist. In such a case the content of oldName
is moved into directory newName. If replace = <b>false</b>
it is required that the existing files
in newName are different from the existing files in
oldName.
</p>
<h4>Example</h4>
<blockquote><pre>
  move(\"C:/test1/directory1\", \"C:/test2/directory2\");
     -> the content of directory1 is moved into directory2.
        Afterwards directory1 is deleted.
        if \"C:/test2/directory2\" does not exist, it is newly
        created. If \"replace=true\", files in directory2
        may be overwritten
   move(\"test1.txt\", \"test2.txt\")
     -> rename file \"test1.txt\" into \"test2.txt\"
        within the current directory
</pre></blockquote>
</HTML>"));
      end move;
      function remove "Remove file or directory (ignore call, if it does not exist)"
        extends Modelica.Icons.Function;
        input String name "Name of file or directory to be removed";
      protected 
        function removeDirectory "Remove a directory, even if it is not empty"
          input String name;
        protected 
          Integer nNames = Modelica.Utilities.Internal.FileSystem.getNumberOfFiles(name);
          Integer lenName = Strings.length(name);
          String fileNames[nNames];
          String name2 = if Strings.substring(name, lenName, lenName) == "/" then Strings.substring(name, lenName - 1, lenName - 1) else name;
        algorithm 
          fileNames:=Modelica.Utilities.Internal.FileSystem.readDirectory(name2, nNames);
          for i in 1:nNames loop
                      Files.remove(name2 + "/" + fileNames[i]);

          end for;
          Modelica.Utilities.Internal.FileSystem.rmdir(name2);
        end removeDirectory;
        String fullName = Files.fullPathName(name);
        Types.FileType fileType = Modelica.Utilities.Internal.FileSystem.stat(fullName);
      algorithm 
        if fileType == Types.FileType.RegularFile or fileType == Types.FileType.SpecialFile then 
                Modelica.Utilities.Internal.FileSystem.removeFile(fullName);
        elseif fileType == Types.FileType.Directory then
          removeDirectory(fullName);


        else 
        end if;
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Files.<b>remove</b>(name);
</pre></blockquote>
<h4>Description</h4>
<p>
Removes the file or directory \"name\". If \"name\" does not exist,
the function call is ignored. If \"name\" is a directory, first
the content of the directory is removed and afterwards
the directory itself.
</p>
<p>
This function is silent, i.e., it does not print a message.
</p>
</html>"));
      end remove;
      function removeFile "Remove file (ignore call, if it does not exist)"
        extends Modelica.Icons.Function;
        input String fileName "Name of file that should be removed";
      protected 
        Types.FileType fileType = Modelica.Utilities.Internal.FileSystem.stat(fileName);
      algorithm 
        if fileType == Types.FileType.RegularFile then 
                Modelica.Utilities.Internal.FileSystem.removeFile(fileName);
        elseif fileType == Types.FileType.Directory then
          Streams.error("File \"" + fileName + "\" should be removed.\n" + "This is not possible, because it is a directory");

        elseif fileType == Types.FileType.SpecialFile then
          Streams.error("File \"" + fileName + "\" should be removed.\n" + "This is not possible, because it is a special file (pipe, device, etc.)");


        else 
        end if;
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Files.<b>removeFile</b>(fileName);
</pre></blockquote>
<h4>Description</h4>
<p>
Removes the file \"fileName\". If \"fileName\" does not exist,
the function call is ignored. If \"fileName\" exists but is
no regular file (e.g., directory, pipe, device, etc.) an
error is triggered.
</p>
<p>
This function is silent, i.e., it does not print a message.
</p>
</html>"));
      end removeFile;
      function createDirectory "Create directory (if directory already exists, ignore call)"
        extends Modelica.Icons.Function;
        input String directoryName "Name of directory to be created (if present, ignore call)";
      protected 
        function existDirectory "Inquire whether directory exists; if present and not a directory, trigger an error"
          input String directoryName;
          output Boolean exists "true if directory exists";
        protected 
          Types.FileType fileType = Modelica.Utilities.Internal.FileSystem.stat(directoryName);
        algorithm 
          if fileType == Types.FileType.RegularFile or fileType == Types.FileType.SpecialFile then 
                    Streams.error("Directory \"" + directoryName + "\" cannot be created\n" + "because this is an existing file.");
          elseif fileType == Types.FileType.Directory then
            exists:=true;


          else           exists:=false;

          end if;
        end existDirectory;
        function assertCorrectIndex "Print error, if index to last essential character in directory is wrong"
          input Integer index "Index must be > 0";
          input String directoryName "Directory name for error message";
        algorithm 
          if index < 1 then 
                    Streams.error("It is not possible to create the directory\n" + "\"" + directoryName + "\"\n" + "because this directory name is not valid");

          else 
          end if;
        end assertCorrectIndex;
        String fullName;
        Integer index;
        Integer oldIndex;
        Integer lastIndex;
        Boolean found;
        Boolean finished;
        Integer nDirectories = 0 "Number of directories that need to be generated";
      algorithm 
        if not existDirectory(directoryName) then 
                fullName:=Files.fullPathName(directoryName);
        index:=Strings.length(fullName);
        if Strings.substring(fullName, index, index) == "/" then 
                index:=index - 1;
        assertCorrectIndex(index, fullName);

        else 
        end if;
        lastIndex:=index;
        fullName:=Strings.substring(fullName, 1, index);
        found:=false;
        while (not found) loop
                  oldIndex:=index;
          index:=Strings.findLast(fullName, "/", startIndex = index);
          if index == 0 then 
                    index:=oldIndex;
          found:=true;

          else           index:=index - 1;
          assertCorrectIndex(index, fullName);
          found:=existDirectory(Strings.substring(fullName, 1, index));

          end if;

        end while;
        index:=oldIndex;
        finished:=false;
        while (not finished) loop
                  Modelica.Utilities.Internal.FileSystem.mkdir(Strings.substring(fullName, 1, index));
          if index >= lastIndex then 
                    finished:=true;
          elseif index < lastIndex then
            index:=Strings.find(fullName, "/", startIndex = index + 2);
            if index == 0 then 
                        index:=lastIndex;

            else 
            end if;


          else 
          end if;

        end while;

        else 
        end if;
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Files.<b>createDirectory</b>(directoryName);
</pre></blockquote>
<h4>Description</h4>
<p>
Creates directory \"directorName\". If this directory already exists,
the function call is ignored. If several directories in \"directoryName\"
do not exist, all of them are created. For example, assume
that directory \"E:/test1\" exists and that directory
\"E:/test1/test2/test3\" shall be created. In this case
the directories \"test2\" in \"test1\" and \"test3\" in \"test2\"
are created.
</p>
<p>
This function is silent, i.e., it does not print a message.
In case of error (e.g., \"directoryName\" is an existing regular
file), an assert is triggered.
</p>
</html>"));
      end createDirectory;
      function exist "Inquire whether file or directory exists"
        extends Modelica.Icons.Function;
        input String name "Name of file or directory";
        output Boolean result "= true, if file or directory exists";
      algorithm 
        result:=Modelica.Utilities.Internal.FileSystem.stat(name) > Types.FileType.NoFile;
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
result = Files.<b>exist</b>(name);
</pre></blockquote>
<h4>Description</h4>
<p>
Returns true, if \"name\" is an existing file or directory.
If this is not the case, the function returns false.
</p>
</html>"));
      end exist;
      function assertNew "Trigger an assert, if a file or directory exists"
        extends Modelica.Icons.Function;
        input String name "Name of file or directory";
        input String message = "This is not allowed." "Message that should be printed after the default message in a new line";
      protected 
        Types.FileType fileType = Modelica.Utilities.Internal.FileSystem.stat(name);
      algorithm 
        if fileType == Types.FileType.RegularFile then 
                Streams.error("File \"" + name + "\" already exists.\n" + message);
        elseif fileType == Types.FileType.Directory then
          Streams.error("Directory \"" + name + "\" already exists.\n" + message);

        elseif fileType == Types.FileType.SpecialFile then
          Streams.error("A special file (pipe, device, etc.) \"" + name + "\" already exists.\n" + message);


        else 
        end if;
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Files.<b>assertNew</b>(name);
Files.<b>assertNew</b>(name, message=\"This is not allowed\");
</pre></blockquote>
<h4>Description</h4>
<p>
Triggers an assert, if \"name\" is an existing file or
directory. The error message has the following structure:
</p>
<pre>
  File \"&lt;name&gt;\" already exists.
  &lt;message&gt;
</pre>
</p>
</html>"));
      end assertNew;
      function fullPathName "Get full path name of file or directory name"
        extends Modelica.Icons.Function;
        input String name "Absolute or relative file or directory name";
        output String fullName "Full path of 'name'";

        external "C" fullName = ModelicaInternal_fullPathName(name) ;        annotation(Library = "ModelicaExternalC", Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
fullName = Files.<b>fullPathName</b>(name);
</pre></blockquote>
<h4>Description</h4>
<p>
Returns the full path name of a file or directory \"name\".
</p>
</html>"));
      end fullPathName;
      function splitPathName "Split path name in directory, file name kernel, file name extension"
        extends Modelica.Icons.Function;
        input String pathName "Absolute or relative file or directory name";
        output String directory "Name of the directory including a trailing '/'";
        output String name "Name of the file without the extension";
        output String extension "Extension of the file name. Starts with '.'";
        annotation(Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
(directory, name, extension) = Files.<b>splitPathName</b>(pathName);
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>splitPathName</b>(..) splits a path name into its parts.
</p>
<h4>Example</h4>
<pre>
  (directory, name, extension) = Files.splitPathName(\"C:/user/test/input.txt\")

  -> directory = \"C:/user/test/\"
     name      = \"input\"
     extension = \".txt\"
</pre>
</HTML>"));
      protected 
        Integer lenPath = Strings.length(pathName);
        Integer i = lenPath;
        Integer indexDot = 0;
        Integer indexSlash = 0;
        String c;
      algorithm 
        while (i >= 1) loop
                  c:=Strings.substring(pathName, i, i);
          if c == "." then 
                    indexDot:=i;
          i:=0;
          elseif c == "/" then
            indexSlash:=i;
            i:=0;


          else           i:=i - 1;

          end if;

        end while;
        if indexSlash == lenPath then 
                directory:=pathName;
        name:="";
        extension:="";
        elseif indexDot > 0 then
          indexSlash:=Strings.findLast(pathName, "/", startIndex = indexDot - 1);
          if indexSlash == 0 then 
                    directory:="";
          name:=Strings.substring(pathName, 1, indexDot - 1);

          else           directory:=Strings.substring(pathName, 1, indexSlash);
          name:=Strings.substring(pathName, indexSlash + 1, indexDot - 1);

          end if;
          extension:=Strings.substring(pathName, indexDot, lenPath);


        else         extension:="";
        if indexSlash > 0 then 
                directory:=Strings.substring(pathName, 1, indexSlash);
        name:=Strings.substring(pathName, indexSlash + 1, lenPath);

        else         directory:="";
        name:=pathName;

        end if;

        end if;
      end splitPathName;
      function temporaryFileName "Return arbitrary name of a file that does not exist and is in a directory where access rights allow to write to this file (useful for temporary output of files)"
        extends Modelica.Icons.Function;
        output String fileName "Full path name of temporary file";

        external "C" fileName = ModelicaInternal_temporaryFileName(0) ;        annotation(Library = "ModelicaExternalC", Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
fileName = Files.<b>temporaryFileName</b>();
</pre></blockquote>
<h4>Description</h4>
<p>
Return arbitrary name of a file that does not exist
and is in a directory where access rights allow to
write to this file (useful for temporary output of files).
</p>
</html>"));
      end temporaryFileName;
    end Files;
    package Examples "Examples to demonstrate the usage of package Modelica.Utilities"
      annotation(Documentation(info = "<html>
<p>
This package contains quite involved examples that demonstrate how to
use the functions of package Modelica.Utilities. In particular
the following examples are present.
</p>
<ul>
<li> Function <a href=\"Modelica://Modelica.Utilities.Examples.calculator\">calculator</a>
     is an interpreter to evaluate
     expressions consisting of +,-,*,/,(),sin(), cos(), tan(), sqrt(), pi.
     For example: calculator(\"1.5*sin(pi/6)\"); <br>&nbsp;</li>
<li> Function <a href=\"Modelica://Modelica.Utilities.Examples.expression\">expression</a>
     is the basic function used in \"calculator\" to evaluate an expression.
     It is useful if the expression interpreter is used in a larger
     scan operation (such as readRealParameter below).<br>&nbsp;</li>
<li> Function <a href=\"Modelica://Modelica.Utilities.Examples.readRealParameter\">readRealParameter</a>
     reads the value of a parameter
     from file, given the file and the parameter name. The value
     on file is interpreted with the Examples.expression function
     and can therefore be an expression.<br>&nbsp;</li>
<li> Model <a href=\"Modelica://Modelica.Utilities.Examples.readRealParameterModel\">readRealParameterModel</a>
     is a test model to demonstrate the usage of \"readRealParameter\". The model
     contains 3 parameters that are read from file \"Modelica.Utilities/data/Examples_readRealParameters.txt\".
     </li>
</ul>
</html>"));
      function calculator "Interpreter to evaluate simple expressions consisting of +,-,*,/,(),sin(), cos(), tan(), sqrt(), pi"
        import Modelica.Utilities.Strings.*;
        extends Modelica.Icons.Function;
        input String string "Expression that is evaluated";
        output Real result "Value of expression";
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
result = <b>calculator</b>(expression);
</pre></blockquote>
<h4>Description</h4>
<p>
This function demonstrates how a simple expression calculator
can be implemented in form of a recursive decent parser
using basically the Strings.scanToken(..) and Strings.scanDelimiter(..)
function.
</p>
<p>
The following operations are supported (pi=3.14.. is a predefined constant):
</p>
<pre>
   +, -
   *, /
   (expression)
   sin(expression)
   cos(expression)
   tan(expression)
   sqrt(expression)
   pi
</pre>
<h4>Example</h4>
<blockquote><pre>
  calculator(\"2+3*(4-1)\");  // returns 11
  calculator(\"sin(pi/6)\");  // returns 0.5
</pre></blockquote>
</html>"));
      protected 
        Integer nextIndex;
      algorithm 
        (result,nextIndex):=expression(string, 1);
        Strings.scanNoToken(string, nextIndex);
      end calculator;
      function expression "Expression interpreter that returns with the position after the expression (expression may consist of +,-,*,/,(),sin(), cos(), tan(), sqrt(), pi"
        import Modelica.Utilities.Types;
        import Modelica.Utilities.Strings;
        import Modelica.Math;
        import Modelica.Constants;
        extends Modelica.Icons.Function;
        input String string "Expression that is evaluated";
        input Integer startIndex = 1 "Start scanning of expression at character startIndex";
        input String message = "" "Message used in error message if scan is not successful";
        output Real result "Value of expression";
        output Integer nextIndex "Index after the scanned expression";
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
             result = <b>expression</b>(string);
(result, nextIndex) = <b>expression</b>(string, startIndex=1, message=\"\");
</pre></blockquote>
<h4>Description</h4>
<p>
This function is nearly the same as Examples.<b>calculator</b>.
The essential difference is that function \"expression\" might be
used in other parsing operations: After the expression is
parsed and evaluated, the function returns with the value
of the expression as well as the position of the character
directly after the expression.
</p>
<p>
This function demonstrates how a simple expression calculator
can be implemented in form of a recursive decent parser
using basically the Strings.scanToken(..) and scanDelimiters(..)
function. There are 2 local functions (term, primary) that
implement the corresponding part of the grammar.
</p>
<p>
The following operations are supported (pi=3.14.. is a predefined constant):
</p>
<pre>
   +, -
   *, /
   (expression)
   sin(expression)
   cos(expression)
   tan(expression)
   sqrt(expression)
   pi
</pre>
<p>
The optional argument \"startIndex\" defines at which position
scanning of the expression starts.
</p>
<p>
In case of error,
the optional argument \"message\" is appended to the error
message, in order to give additional information where
the error occured.
</p>
<p>
This function parses the following grammaer
</p>
<pre>
  expression: [ add_op ] term { add_op term }
  add_op    : \"+\" | \"-\"
  term      : primary { mul_op primary }
  mul_op    : \"*\" | \"/\"
  primary   : UNSIGNED_NUMBER
              | pi
              | ( expression )
              | functionName( expression )
  function  :   sin
              | cos
              | tan
              | sqrt
</pre>
<p>
Note, in Examples.readRealParameter it is shown, how the expression
function can be used as part of another scan operation.
</p>
<h4>Example</h4>
<blockquote><pre>
  expression(\"2+3*(4-1)\");  // returns 11
  expression(\"sin(pi/6)\");  // returns 0.5
</pre></blockquote>
</html>"));
      protected 
        function term "Evaluate term of an expression"
          extends Modelica.Icons.Function;
          input String string;
          input Integer startIndex;
          input String message = "";
          output Real result;
          output Integer nextIndex;
        protected 
          Real result2;
          Boolean scanning = true;
          String opString;
        algorithm 
          (result,nextIndex):=primary(string, startIndex, message);
          while (scanning) loop
                      (opString,nextIndex):=Strings.scanDelimiter(string, nextIndex, {"*","/",""}, message);
            if opString == "" then 
                        scanning:=false;

            else             (result2,nextIndex):=primary(string, nextIndex, message);
            result:=if opString == "*" then result * result2 else result / result2;

            end if;

          end while;
        end term;
        function primary "Evaluate primary of an expression"
          extends Modelica.Icons.Function;
          input String string;
          input Integer startIndex;
          input String message = "";
          output Real result;
          output Integer nextIndex;
        protected 
          Types.TokenValue token;
          Real result2;
          String delimiter;
          String functionName;
          Real pi = Constants.pi;
        algorithm 
          (token,nextIndex):=Strings.scanToken(string, startIndex, unsigned = true);
          if token.tokenType == Types.TokenType.DelimiterToken and token.string == "(" then 
                    (result,nextIndex):=expression(string, nextIndex, message);
          (delimiter,nextIndex):=Strings.scanDelimiter(string, nextIndex, {")"}, message);
          elseif token.tokenType == Types.TokenType.RealToken then
            result:=token.real;

          elseif token.tokenType == Types.TokenType.IntegerToken then
            result:=token.integer;

          elseif token.tokenType == Types.TokenType.IdentifierToken then
            if token.string == "pi" then 
                        result:=pi;

            else             functionName:=token.string;
            (delimiter,nextIndex):=Strings.scanDelimiter(string, nextIndex, {"("}, message);
            (result,nextIndex):=expression(string, nextIndex, message);
            (delimiter,nextIndex):=Strings.scanDelimiter(string, nextIndex, {")"}, message);
            if functionName == "sin" then 
                        result:=Math.sin(result);
            elseif functionName == "cos" then
              result:=Math.cos(result);

            elseif functionName == "tan" then
              result:=Math.tan(result);

            elseif functionName == "sqrt" then
              if result < 0.0 then 
                            Strings.syntaxError(string, startIndex, "Argument of call \"sqrt(" + String(result) + ")\" is negative.\n" + "Imaginary numbers are not supported by the calculator.\n" + message);

              else 
              end if;
              result:=sqrt(result);


            else             Strings.syntaxError(string, startIndex, "Function \"" + functionName + "\" is unknown (not supported)\n" + message);

            end if;

            end if;


          else           Strings.syntaxError(string, startIndex, "Invalid primary of expression.\n" + message);

          end if;
        end primary;
        Real result2;
        String signOfNumber;
        Boolean scanning = true;
        String opString;
      algorithm 
        (signOfNumber,nextIndex):=Strings.scanDelimiter(string, startIndex, {"+","-",""}, message);
        (result,nextIndex):=term(string, nextIndex, message);
        if signOfNumber == "-" then 
                result:= -result;

        else 
        end if;
        while (scanning) loop
                  (opString,nextIndex):=Strings.scanDelimiter(string, nextIndex, {"+","-",""}, message);
          if opString == "" then 
                    scanning:=false;

          else           (result2,nextIndex):=term(string, nextIndex, message);
          result:=if opString == "+" then result + result2 else result - result2;

          end if;

        end while;
      end expression;
      function readRealParameter "Read the value of a Real parameter from file"
        import Modelica.Utilities.*;
        extends Modelica.Icons.Function;
        input String fileName "Name of file" annotation(Dialog(__Dymola_loadSelector(filter = "Text files (*.txt)", caption = "Open file in which Real parameters are present")));
        input String name "Name of parameter";
        output Real result "Actual value of parameter on file";
        annotation(Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
result = <b>readRealParameter</b>(fileName, name);
</pre></blockquote>
<h4>Description</h4>
<p>
This function demonstrates how a function can be implemented
that reads the value of a parameter from file. The function
performs the following actions:
</p>
<ol>
<li> It opens file \"fileName\" and reads the lines of the file.</li>
<li> In every line, Modelica line comments (\"// ... end-of-line\")
     are skipped </li>
<li> If a line consists of \"name = expression\" and the \"name\"
     in this line is identical to the second argument \"name\"
     of the function call, the expression calculator Examples.expression
     is used to evaluate the expression after the \"=\" character.
     The expression can optionally be terminated with a \";\".</li>
<li> The result of the expression evaluation is returned as
     the value of the parameter \"name\". </li>
</ol>
<h4>Example</h4>
<p>
On file \"test.txt\" the following lines might be present:
</p>
<blockquote><pre>
// Motor data
J        = 2.3     // inertia
w_rel0   = 1.5*2;  // relative angular velocity
phi_rel0 = pi/3
</pre></blockquote>
<p>
The function returns the value \"3.0\" when called as:
</p>
<blockquote><pre>
readRealParameter(\"test.txt\", \"w_rel0\")
</pre></blockquote>
</html>"));
      protected 
        String line;
        String identifier;
        String delimiter;
        Integer nextIndex;
        Integer iline = 1;
        Types.TokenValue token;
        String message = "in file \"" + fileName + "\" on line ";
        String message2;
        Boolean found = false;
        Boolean endOfFile = false;
      algorithm 
        (line,endOfFile):=Streams.readLine(fileName, iline);
        while (not found and not endOfFile) loop
                  (token,nextIndex):=Strings.scanToken(line);
          if token.tokenType == Types.TokenType.NoToken then 
                    iline:=iline + 1;
          elseif token.tokenType == Types.TokenType.IdentifierToken then
            if token.string == name then 
                        message2:=message + String(iline);
            (delimiter,nextIndex):=Strings.scanDelimiter(line, nextIndex, {"="}, message2);
            (result,nextIndex):=Examples.expression(line, nextIndex, message2);
            (delimiter,nextIndex):=Strings.scanDelimiter(line, nextIndex, {";",""}, message2);
            Strings.scanNoToken(line, nextIndex, message2);
            found:=true;

            else             iline:=iline + 1;

            end if;


          else           Strings.syntaxError(line, nextIndex, "Expected identifier " + message + String(iline));

          end if;
          (line,endOfFile):=Streams.readLine(fileName, iline);

        end while;
        if not found then 
                Streams.error("Parameter \"" + name + "\" not found in file \"" + fileName + "\"");

        else 
        end if;
      end readRealParameter;
      model readRealParameterModel "Demonstrate usage of Examples.readRealParameter/.expression"
        import SI = Modelica.SIunits;
        extends Modelica.Icons.Example;
        parameter String file = "Modelica/Utilities/data/Examples_readRealParameters.txt" "File on which data is present" annotation(Dialog(__Dymola_loadSelector(filter = "Text files (*.txt)", caption = "Open text file to read parameters of the form \"name = value\"")));
        parameter SI.Inertia J = readRealParameter(file, "J") "Inertia";
        parameter SI.Angle phi_rel0 = readRealParameter(file, "phi_rel0") "Relative angle";
        parameter SI.AngularVelocity w_rel0 = readRealParameter(file, "w_rel0") "Relative angular velocity";
        annotation(Documentation(info = "<html>
<p>
Model that shows the usage of Examples.readRealParameter and Examples.expression.
The model has 3 parameters and the values of these parameters are read
from a file.
</p>
</html>"));
      end readRealParameterModel;
    end Examples;
  end Utilities;
  package Thermal "Library of thermal system components to model heat transfer and simple thermo-fluid pipe flow"
    extends Modelica.Icons.Library;
    annotation(Documentation(info = "<html>
<p>
This package contains libraries to model heat transfer
and fluid heat flow.
</p>
</html>"));
    package HeatTransfer "Library of 1-dimensional heat transfer with lumped elements"
      import Modelica.SIunits.Conversions.*;
      extends Modelica.Icons.Library2;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -54, -6},{ -61, -7},{ -75, -15},{ -79, -24},{ -80, -34},{ -78, -42},{ -73, -49},{ -64, -51},{ -57, -51},{ -47, -50},{ -41, -43},{ -38, -35},{ -40, -27},{ -40, -20},{ -42, -13},{ -47, -7},{ -54, -5},{ -54, -6}}, lineColor = {128,128,128}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -75, -15},{ -79, -25},{ -80, -34},{ -78, -42},{ -72, -49},{ -64, -51},{ -57, -51},{ -47, -50},{ -57, -47},{ -65, -45},{ -71, -40},{ -74, -33},{ -76, -23},{ -75, -15},{ -75, -15}}, lineColor = {0,0,0}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Polygon(points = {{39, -6},{32, -7},{18, -15},{14, -24},{13, -34},{15, -42},{20, -49},{29, -51},{36, -51},{46, -50},{52, -43},{55, -35},{53, -27},{53, -20},{51, -13},{46, -7},{39, -5},{39, -6}}, lineColor = {160,160,164}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{18, -15},{14, -25},{13, -34},{15, -42},{21, -49},{29, -51},{36, -51},{46, -50},{36, -47},{28, -45},{22, -40},{19, -33},{17, -23},{18, -15},{18, -15}}, lineColor = {0,0,0}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Polygon(points = {{ -9, -23},{ -9, -10},{18, -17},{ -9, -23}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -41, -17},{ -9, -17}}, color = {191,0,0}, thickness = 0.5),Line(points = {{ -17, -40},{15, -40}}, color = {191,0,0}, thickness = 0.5),Polygon(points = {{ -17, -46},{ -17, -34},{ -40, -40},{ -17, -46}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
<p>
This package contains components to model <b>1-dimensional heat transfer</b>
with lumped elements. This allows especially to model heat transfer in
machines provided the parameters of the lumped elements, such as
the heat capacity of a part, can be determined by measurements
(due to the complex geometries and many materials used in machines,
calculating the lumped element parameters from some basic analytic
formulas is usually not possible).
</p>
<p>
Example models how to use this library are given in subpackage <b>Examples</b>.<br>
For a first simple example, see <b>Examples.TwoMasses</b> where two masses
with different initial temperatures are getting in contact to each
other and arriving after some time at a common temperature.<br>
<b>Examples.ControlledTemperature</b> shows how to hold a temperature
within desired limits by switching on and off an electric resistor.<br>
A more realistic example is provided in <b>Examples.Motor</b> where the
heating of an electrical motor is modelled, see the following screen shot
of this example:
</p>
<img src=\"../Images/driveWithHeatTransfer.png\" ALT=\"driveWithHeatTransfer\">
<p>
The <b>filled</b> and <b>non-filled red squares</b> at the left and
right side of a component represent <b>thermal ports</b> (connector HeatPort).
Drawing a line between such squares means that they are thermally connected.
The variables of a HeatPort connector are the temperature <b>T</b> at the port
and the heat flow rate <b>Q_flow</b> flowing into the component (if Q_flow is positive,
the heat flows into the element, otherwise it flows out of the element):
</p>
<pre>   Modelica.SIunits.Temperature  T  \"absolute temperature at port in Kelvin\";
   Modelica.SIunits.HeatFlowRate Q_flow  \"flow rate at the port in Watt\";
</pre>
<p>
Note, that all temperatures of this package, including initial conditions,
are given in Kelvin. For convenience, in subpackages <b>HeatTransfer.Celsius</b>,
 <b>HeatTransfer.Fahrenheit</b> and <b>HeatTransfer.Rankine</b> components are provided such that source and
sensor information is available in degree Celsius, degree Fahrenheit, or degree Rankine,
respectively. Additionally, in package <b>SIunits.Conversions</b> conversion
functions between the units Kelvin and Celsius, Fahrenheit, Rankine are
provided. These functions may be used in the following way:
</p>
<pre>  <b>import</b> SI=Modelica.SIunits;
  <b>import</b> Modelica.SIunits.Conversions.*;
     ...
  <b>parameter</b> SI.Temperature T = from_degC(25);  // convert 25 degree Celsius to Kelvin
</pre>

<p>
There are several other components available, such as AxialConduction (discretized PDE in
axial direction), which have been temporarily removed from this library. The reason is that
these components reference material properties, such as thermal conductivity, and currently
the Modelica design group is discussing a general scheme to describe material properties.
</p>
<p>
For technical details in the design of this library, see the following reference:<br>
<b>Michael Tiller (2001)</b>: <a href=\"http://www.amazon.de\">
Introduction to Physical Modeling with Modelica</a>.
Kluwer Academic Publishers Boston.
</p>
<p>
<b>Acknowledgements:</b><br>
Several helpful remarks from the following persons are acknowledged:
John Batteh, Ford Motors, Dearborn, U.S.A;
<a href=\"http://www.haumer.at/\">Anton Haumer</a>, Technical Consulting &amp; Electrical Engineering, Austria;
Ludwig Marvan, VA TECH ELIN EBG Elektronik GmbH, Wien, Austria;
Hans Olsson, Dynasim AB, Sweden;
Hubertus Tummescheit, Lund Institute of Technology, Lund, Sweden.
</p>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <p>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern, Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </p>
  </dd>
</dl>
<p><b>Copyright &copy; 2001-2009, Modelica Association, Michael Tiller and DLR.</b></p>
<p><i>
This Modelica package is free software; it can be redistributed and/or modified
under the terms of the Modelica license, see the license conditions
and the accompanying disclaimer in the documentation of package
Modelica in file \"Modelica/package.mo\".
</i></p>
</HTML>
", revisions = "<html>
<ul>
<li><i>July 15, 2002</i>
       by Michael Tiller, <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>
       and <a href=\"http://www.robotic.dlr.de/Nikolaus.Schuermann/\">Nikolaus Sch&uuml;rmann</a>:<br>
       Implemented.
</li>
<li><i>June 13, 2005</i>
       by <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
       Refined placing of connectors (cosmetic).<br>
       Refined all Examples; removed Examples.FrequencyInverter, introducing Examples.Motor<br>
       Introduced temperature dependent correction (1 + alpha*(T - T_ref)) in Fixed/PrescribedHeatFlow<br>
</li>
  <li> v1.1.1 2007/11/13 Anton Haumer<br>
       componentes moved to sub-packages</li>
  <li> v1.2.0 2009/08/26 Anton Haumer<br>
       added component ThermalCollector</li>

</ul>
</html>"));
      package Examples "Example models to demonstrate the usage of package Modelica.Thermal.HeatTransfer"
        extends Modelica.Icons.Library2;
        model TwoMasses "Simple conduction demo"
          extends Modelica.Icons.Example;
          parameter Modelica.SIunits.Temperature T_final_K(fixed = false) "Projected final temperature";
          HeatTransfer.Components.HeatCapacitor mass1(C = 15, T(start = 373.15, fixed = true)) annotation(Placement(transformation(extent = {{ -100,20},{ -40,80}}, rotation = 0)));
          HeatTransfer.Components.HeatCapacitor mass2(C = 15, T(start = 273.15, fixed = true)) annotation(Placement(transformation(extent = {{40,20},{100,80}}, rotation = 0)));
          HeatTransfer.Components.ThermalConductor conduction(G = 10) annotation(Placement(transformation(extent = {{ -30, -20},{30,40}}, rotation = 0)));
          annotation(Documentation(info = "<HTML>
<p>
This example demonstrates the thermal response of two masses connected by
a conducting element. The two masses have the same heat capacity but different
initial temperatures (T1=100 [degC], T2= 0 [degC]). The mass with the higher
temperature will cool off while the mass with the lower temperature heats up.
They will each asymptotically approach the calculated temperature <b>T_final_K</b>
(<b>T_final_degC</b>) that results from dividing the total initial energy in the system by the sum
of the heat capacities of each element.
</p>
<p>
Simulate for 5 s and plot the variables<br>
mass1.T, mass2.T, T_final_K or <br>
Tsensor1.T, Tsensor2.T, T_final_degC
</p>
</HTML>
"));
          HeatTransfer.Celsius.TemperatureSensor Tsensor1 annotation(Placement(transformation(extent = {{ -60, -80},{ -20, -40}}, rotation = 0)));
          HeatTransfer.Celsius.TemperatureSensor Tsensor2 annotation(Placement(transformation(extent = {{60, -80},{20, -40}}, rotation = 0)));

        equation 
          connect(mass1.port,conduction.port_a) annotation(Line(points = {{ -70,20},{ -70,10},{ -30,10}}, color = {191,0,0}));
          connect(conduction.port_b,mass2.port) annotation(Line(points = {{30,10},{70,10},{70,20}}, color = {191,0,0}));
          connect(mass1.port,Tsensor1.port) annotation(Line(points = {{ -70,20},{ -70, -60},{ -60, -60}}, color = {191,0,0}));
          connect(mass2.port,Tsensor2.port) annotation(Line(points = {{70,20},{70, -60},{60, -60}}, color = {191,0,0}));

        initial equation 
          T_final_K = (mass1.T * mass1.C + mass2.T * mass2.C) / (mass1.C + mass2.C);
        end TwoMasses;
        model ControlledTemperature "Control temperature of a resistor"
          extends Modelica.Icons.Example;
          parameter Modelica.SIunits.Temperature TAmb(displayUnit = "degC") = 293.15 "Ambient Temperature";
          parameter Modelica.SIunits.TemperatureDifference TDif = 2 "Error in Temperature";
          output Modelica.SIunits.Temperature TRes(displayUnit = "degC") = heatingResistor.heatPort.T "Resulting Temperature";
          annotation(Documentation(info = "<HTML>
<P>
A constant voltage of 10 V is applied to a
temperature dependent resistor of 10*(1+(T-20C)/(235+20C)) Ohms,
whose losses v**2/r are dissipated via a
thermal conductance of 0.1 W/K to ambient temperature 20 degree C.
The resistor is assumed to have a thermal capacity of 1 J/K,
having ambient temparature at the beginning of the experiment.
The temperature of this heating resistor is held by an OnOff-controller
at reference temperature within a given bandwith +/- 1 K
by switching on and off the voltage source.
The reference temperature starts at 25 degree C
and rises between t = 2 and 8 seconds linear to 50 degree C.
An approppriate simulating time would be 10 seconds.
</P>
</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 10));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(Placement(transformation(extent = {{ -100, -100},{ -80, -80}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.ConstantVoltage constantVoltage(V = 10) annotation(Placement(transformation(origin = { -90, -50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          HeatTransfer.Components.HeatCapacitor heatCapacitor(C = 1, T(start = TAmb, fixed = true)) annotation(Placement(transformation(extent = {{0, -60},{20, -80}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.HeatingResistor heatingResistor(R_ref = 10, T_ref = from_degC(20), alpha = 1 / (235 + 20)) annotation(Placement(transformation(origin = { -30, -50}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          HeatTransfer.Sources.FixedTemperature fixedTemperature(T = TAmb) annotation(Placement(transformation(extent = {{100, -60},{80, -40}}, rotation = 0)));
          HeatTransfer.Celsius.TemperatureSensor temperatureSensor annotation(Placement(transformation(origin = {10, -30}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          HeatTransfer.Components.ThermalConductor thermalConductor(G = 0.1) annotation(Placement(transformation(extent = {{40, -60},{60, -40}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealOpeningSwitch idealSwitch annotation(Placement(transformation(extent = {{ -70, -50},{ -50, -30}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp ramp(height = 25, duration = 6, offset = 25, startTime = 2) annotation(Placement(transformation(extent = {{40,0},{20,20}}, rotation = 0)));
          Modelica.Blocks.Logical.OnOffController onOffController(bandwidth = TDif) annotation(Placement(transformation(extent = {{0, -20},{ -20,0}}, rotation = 0)));
          Modelica.Blocks.Logical.Not logicalNot annotation(Placement(transformation(extent = {{ -30, -20},{ -50,0}}, rotation = 0)));

        equation 
          connect(constantVoltage.n,heatingResistor.n) annotation(Line(points = {{ -90, -60},{ -30, -60}}, color = {0,0,255}));
          connect(constantVoltage.n,ground.p) annotation(Line(points = {{ -90, -60},{ -90, -80}}, color = {0,0,255}));
          connect(heatingResistor.heatPort,thermalConductor.port_a) annotation(Line(points = {{ -20, -50},{40, -50}}, color = {191,0,0}));
          connect(thermalConductor.port_b,fixedTemperature.port) annotation(Line(points = {{60, -50},{80, -50}}, color = {191,0,0}));
          connect(heatingResistor.heatPort,temperatureSensor.port) annotation(Line(points = {{ -20, -50},{10, -50},{10, -40}}, color = {191,0,0}));
          connect(heatingResistor.heatPort,heatCapacitor.port) annotation(Line(points = {{ -20, -50},{10, -50},{10, -60}}, color = {191,0,0}));
          connect(constantVoltage.p,idealSwitch.p) annotation(Line(points = {{ -90, -40},{ -70, -40}}, color = {0,0,255}));
          connect(idealSwitch.n,heatingResistor.p) annotation(Line(points = {{ -50, -40},{ -30, -40}}, color = {0,0,255}));
          connect(ramp.y,onOffController.reference) annotation(Line(points = {{19,10},{10,10},{10, -4},{2, -4}}, color = {0,0,127}));
          connect(temperatureSensor.T,onOffController.u) annotation(Line(points = {{10, -20},{10, -16},{2, -16}}, color = {0,0,127}));
          connect(onOffController.y,logicalNot.u) annotation(Line(points = {{ -21, -10},{ -28, -10}}, color = {255,0,255}));
          connect(logicalNot.y,idealSwitch.control) annotation(Line(points = {{ -51, -10},{ -60, -10},{ -60, -33}}, color = {255,0,255}));
        end ControlledTemperature;
        model Motor "Second order thermal model of a motor"
          extends Modelica.Icons.Example;
          parameter Modelica.SIunits.Temperature TAmb(displayUnit = "degC") = 293.15 "Ambient temperature";
          annotation(Documentation(info = "<HTML>
<p>
This example contains a simple second order thermal model of a motor.
The periodic power losses are described by table \"lossTable\":<br>
<table>
<tr><td valign=\"top\">time</td><td valign=\"top\">winding losses</td><td valign=\"top\">core losses</td></tr>
<tr><td valign=\"top\">   0</td><td valign=\"top\">           100</td><td valign=\"top\">        500</td></tr>
<tr><td valign=\"top\"> 360</td><td valign=\"top\">           100</td><td valign=\"top\">        500</td></tr>
<tr><td valign=\"top\"> 360</td><td valign=\"top\">          1000</td><td valign=\"top\">        500</td></tr>
<tr><td valign=\"top\"> 600</td><td valign=\"top\">          1000</td><td valign=\"top\">        500</td></tr>
</table><br>
Since constant speed is assumed, the core losses keep constant
whereas the winding losses are low for 6 minutes (no-load) and high for 4 minutes (over load).
<br>
The winding losses are corrected by (1 + alpha*(T - T_ref)) because the winding's resistance is temperature dependent whereas the core losses are kept constant (alpha = 0).
</p>
<p>
The power dissipation to the environment is approximated by heat flow through
a thermal conductance between winding and core,
partially storage of the heat in the winding's heat capacity
as well as the core's heat capacity and finally by forced convection to the environment.<br>
Since constant speed is assumed, the cinvective conductance keeps constant.<br>
Using Modelica.Thermal.FluidHeatFlow it would be possible to model the coolant air flow, too
(instead of simple dissipation to a constant ambient's temperature).
</p>
<p>
Simulate for 7200 s; plot Twinding.T and Tcore.T.
</p>
</HTML>
"), experiment(StopTime = 7200), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          Modelica.Blocks.Sources.CombiTimeTable lossTable(extrapolation = Modelica.Blocks.Types.Extrapolation.Periodic, table = [0,100,500;360,100,500;360,1000,500;600,1000,500]) annotation(Placement(transformation(origin = { -40,70}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          HeatTransfer.Sources.PrescribedHeatFlow windingLosses(T_ref = from_degC(95), alpha = 0.00303) annotation(Placement(transformation(origin = { -80,10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          HeatTransfer.Components.HeatCapacitor winding(C = 2500, T(start = TAmb, fixed = true)) annotation(Placement(transformation(extent = {{ -90, -20},{ -70, -40}}, rotation = 0)));
          HeatTransfer.Celsius.TemperatureSensor Twinding annotation(Placement(transformation(origin = { -60, -50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          HeatTransfer.Components.ThermalConductor winding2core(G = 10) annotation(Placement(transformation(extent = {{ -50, -20},{ -30,0}}, rotation = 0)));
          HeatTransfer.Sources.PrescribedHeatFlow coreLosses annotation(Placement(transformation(origin = {0,10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          HeatTransfer.Components.HeatCapacitor core(C = 25000, T(start = TAmb, fixed = true)) annotation(Placement(transformation(extent = {{ -10, -20},{10, -40}}, rotation = 0)));
          HeatTransfer.Celsius.TemperatureSensor Tcore annotation(Placement(transformation(origin = { -20, -50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Blocks.Sources.Constant convectionConstant(k = 25) annotation(Placement(transformation(origin = {40,30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          HeatTransfer.Components.Convection convection annotation(Placement(transformation(extent = {{30, -20},{50,0}}, rotation = 0)));
          HeatTransfer.Sources.FixedTemperature environment(T = TAmb) annotation(Placement(transformation(origin = {80, -10}, extent = {{ -10, -10},{10,10}}, rotation = 180)));

        equation 
          connect(windingLosses.port,winding.port) annotation(Line(points = {{ -80,0},{ -80, -20}}, color = {191,0,0}));
          connect(coreLosses.port,core.port) annotation(Line(points = {{ -1.83697e-15,0},{ -1.83697e-15, -10},{0, -10},{0, -20}}, color = {191,0,0}));
          connect(winding.port,winding2core.port_a) annotation(Line(points = {{ -80, -20},{ -80, -10},{ -50, -10}}, color = {191,0,0}));
          connect(winding2core.port_b,core.port) annotation(Line(points = {{ -30, -10},{0, -10},{0, -20}}, color = {191,0,0}));
          connect(winding.port,Twinding.port) annotation(Line(points = {{ -80, -20},{ -80, -10},{ -60, -10},{ -60, -40}}, color = {191,0,0}));
          connect(core.port,Tcore.port) annotation(Line(points = {{0, -20},{0, -10},{ -20, -10},{ -20, -40}}, color = {191,0,0}));
          connect(winding2core.port_b,convection.solid) annotation(Line(points = {{ -30, -10},{30, -10}}, color = {191,0,0}));
          connect(convection.fluid,environment.port) annotation(Line(points = {{50, -10},{60, -10},{60, -10},{70, -10}}, color = {191,0,0}));
          connect(convectionConstant.y,convection.Gc) annotation(Line(points = {{40,19},{40,0}}, color = {0,0,127}));
          connect(lossTable.y[1],windingLosses.Q_flow) annotation(Line(points = {{ -40,59},{ -40,40},{ -80,40},{ -80,20}}, color = {0,0,127}));
          connect(lossTable.y[2],coreLosses.Q_flow) annotation(Line(points = {{ -40,59},{ -40,40},{1.83697e-15,40},{1.83697e-15,20}}, color = {0,0,127}));
        end Motor;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -60,10},{40, -90}}, lineColor = {135,135,135}),Polygon(points = {{ -30, -12},{ -30, -68},{28, -40},{ -30, -12}}, lineColor = {135,135,135}, fillColor = {135,135,135}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>

</html>"));
      end Examples;
      package Components "Lumped thermal components"
        extends Modelica.Icons.Library2;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,18},{ -70, -100}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Backward),Line(points = {{ -44,16},{ -44, -100}}, color = {0,127,255}),Line(points = {{ -4,16},{ -4, -100}}, color = {0,127,255}),Line(points = {{30,18},{30, -100}}, color = {0,127,255}),Line(points = {{66,18},{66, -100}}, color = {0,127,255}),Line(points = {{66, -100},{76, -80}}, color = {0,127,255}),Line(points = {{66, -100},{56, -80}}, color = {0,127,255}),Line(points = {{30, -100},{40, -80}}, color = {0,127,255}),Line(points = {{30, -100},{20, -80}}, color = {0,127,255}),Line(points = {{ -4, -100},{6, -80}}, color = {0,127,255}),Line(points = {{ -4, -100},{ -14, -80}}, color = {0,127,255}),Line(points = {{ -44, -100},{ -34, -80}}, color = {0,127,255}),Line(points = {{ -44, -100},{ -54, -80}}, color = {0,127,255}),Line(points = {{ -70, -60},{66, -60}}, color = {191,0,0}),Line(points = {{46, -70},{66, -60}}, color = {191,0,0}),Line(points = {{46, -50},{66, -60}}, color = {191,0,0}),Line(points = {{46, -30},{66, -20}}, color = {191,0,0}),Line(points = {{46, -10},{66, -20}}, color = {191,0,0}),Line(points = {{ -70, -20},{66, -20}}, color = {191,0,0})}), Documentation(info = "<html>

</html>"));
        model HeatCapacitor "Lumped thermal element storing heat"
          parameter Modelica.SIunits.HeatCapacity C "Heat capacity of element (= cp*m)";
          Modelica.SIunits.Temperature T(start = 293.15, displayUnit = "degC") "Temperature of element";
          Modelica.SIunits.TemperatureSlope der_T(start = 0) "Time derivative of temperature (= der(T))";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -129,121},{131,70}}, textString = "%name", lineColor = {0,0,255}),Polygon(points = {{0,67},{ -20,63},{ -40,57},{ -52,43},{ -58,35},{ -68,25},{ -72,13},{ -76, -1},{ -78, -15},{ -76, -31},{ -76, -43},{ -76, -53},{ -70, -65},{ -64, -73},{ -48, -77},{ -30, -83},{ -18, -83},{ -2, -85},{8, -89},{22, -89},{32, -87},{42, -81},{54, -75},{56, -73},{66, -61},{68, -53},{70, -51},{72, -35},{76, -21},{78, -13},{78,3},{74,15},{66,25},{54,33},{44,41},{36,57},{26,65},{0,67}}, lineColor = {160,160,164}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -58,35},{ -68,25},{ -72,13},{ -76, -1},{ -78, -15},{ -76, -31},{ -76, -43},{ -76, -53},{ -70, -65},{ -64, -73},{ -48, -77},{ -30, -83},{ -18, -83},{ -2, -85},{8, -89},{22, -89},{32, -87},{42, -81},{54, -75},{42, -77},{40, -77},{30, -79},{20, -81},{18, -81},{10, -81},{2, -77},{ -12, -73},{ -22, -73},{ -30, -71},{ -40, -65},{ -50, -55},{ -56, -43},{ -58, -35},{ -58, -25},{ -60, -13},{ -60, -5},{ -60,7},{ -58,17},{ -56,19},{ -52,27},{ -48,35},{ -44,45},{ -40,57},{ -58,35}}, lineColor = {0,0,0}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Text(extent = {{ -69,7},{71, -24}}, lineColor = {0,0,0}, textString = "%C")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{0,67},{ -20,63},{ -40,57},{ -52,43},{ -58,35},{ -68,25},{ -72,13},{ -76, -1},{ -78, -15},{ -76, -31},{ -76, -43},{ -76, -53},{ -70, -65},{ -64, -73},{ -48, -77},{ -30, -83},{ -18, -83},{ -2, -85},{8, -89},{22, -89},{32, -87},{42, -81},{54, -75},{56, -73},{66, -61},{68, -53},{70, -51},{72, -35},{76, -21},{78, -13},{78,3},{74,15},{66,25},{54,33},{44,41},{36,57},{26,65},{0,67}}, lineColor = {160,160,164}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -58,35},{ -68,25},{ -72,13},{ -76, -1},{ -78, -15},{ -76, -31},{ -76, -43},{ -76, -53},{ -70, -65},{ -64, -73},{ -48, -77},{ -30, -83},{ -18, -83},{ -2, -85},{8, -89},{22, -89},{32, -87},{42, -81},{54, -75},{42, -77},{40, -77},{30, -79},{20, -81},{18, -81},{10, -81},{2, -77},{ -12, -73},{ -22, -73},{ -30, -71},{ -40, -65},{ -50, -55},{ -56, -43},{ -58, -35},{ -58, -25},{ -60, -13},{ -60, -5},{ -60,7},{ -58,17},{ -56,19},{ -52,27},{ -48,35},{ -44,45},{ -40,57},{ -58,35}}, lineColor = {0,0,0}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -6, -1},{6, -12}}, lineColor = {255,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{11,13},{50, -25}}, lineColor = {0,0,0}, textString = "T"),Line(points = {{0, -12},{0, -96}}, color = {255,0,0})}), Documentation(info = "<HTML>
<p>
This is a generic model for the heat capacity of a material.
No specific geometry is assumed beyond a total volume with
uniform temperature for the entire volume.
Furthermore, it is assumed that the heat capacity
is constant (indepedent of temperature).
</p>
<p>
The temperature T [Kelvin] of this component is a <b>state</b>.
A default of T = 25 degree Celsius (= SIunits.Conversions.from_degC(25))
is used as start value for initialization.
This usually means that at start of integration the temperature of this
component is 25 degrees Celsius. You may, of course, define a different
temperature as start value for initialization. Alternatively, it is possible
to set parameter <b>steadyStateStart</b> to <b>true</b>. In this case
the additional equation '<b>der</b>(T) = 0' is used during
initialization, i.e., the temperature T is computed in such a way that
the component starts in <b>steady state</b>. This is useful in cases,
where one would like to start simulation in a suitable operating
point without being forced to integrate for a long time to arrive
at this point.
</p>
<p>
Note, that parameter <b>steadyStateStart</b> is not available in
the parameter menue of the simulation window, because its value
is utilized during translation to generate quite different
equations depending on its setting. Therefore, the value of this
parameter can only be changed before translating the model.
</p>
<p>
This component may be used for complicated geometries where
the heat capacity C is determined my measurements. If the component
consists mainly of one type of material, the <b>mass m</b> of the
component may be measured or calculated and multiplied with the
<b>specific heat capacity cp</b> of the component material to
compute C:
</p>
<pre>
   C = cp*m.
   Typical values for cp at 20 degC in J/(kg.K):
      aluminium   896
      concrete    840
      copper      383
      iron        452
      silver      235
      steel       420 ... 500 (V2A)
      wood       2500
</pre>
</HTML>
"));
          Interfaces.HeatPort_a port annotation(Placement(transformation(origin = {0, -100}, extent = {{ -10, -10},{10,10}}, rotation = 90)));

        equation 
          T = port.T;
          der_T = der(T);
          C * der(T) = port.Q_flow;
        end HeatCapacitor;
        model ThermalConductor "Lumped thermal element transporting heat without storing it"
          extends Interfaces.Element1D;
          parameter Modelica.SIunits.ThermalConductance G "Constant thermal conductance of material";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -90,70},{90, -70}}, lineColor = {0,0,0}, pattern = LinePattern.None, fillColor = {192,192,192}, fillPattern = FillPattern.Backward),Line(points = {{ -90,70},{ -90, -70}}, color = {0,0,0}, thickness = 0.5),Line(points = {{90,70},{90, -70}}, color = {0,0,0}, thickness = 0.5),Text(extent = {{ -139,134},{141,74}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -115, -76},{113, -116}}, lineColor = {0,0,0}, textString = "G=%G")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -80,0},{80,0}}, color = {255,0,0}, thickness = 0.5, arrow = {Arrow.None,Arrow.Filled}),Text(extent = {{ -26, -10},{27, -39}}, lineColor = {255,0,0}, textString = "Q_flow"),Text(extent = {{ -80,50},{80,20}}, lineColor = {0,0,0}, textString = "dT = port_a.T - port_b.T")}), Documentation(info = "<HTML>
<p>
This is a model for transport of heat without storing it.
It may be used for complicated geometries where
the thermal conductance G (= inverse of thermal resistance)
is determined by measurements and is assumed to be constant
over the range of operations. If the component consists mainly of
one type of material and a regular geometry, it may be calculated,
e.g., with one of the following equations:
</p>
<ul>
<li><p>
    Conductance for a <b>box</b> geometry under the assumption
    that heat flows along the box length:</p>
    <pre>
    G = k*A/L
    k: Thermal conductivity (material constant)
    A: Area of box
    L: Length of box
    </pre>
    </li>
<li><p>
    Conductance for a <b>cylindrical</b> geometry under the assumption
    that heat flows from the inside to the outside radius
    of the cylinder:</p>
    <pre>
    G = 2*pi*k*L/log(r_out/r_in)
    pi   : Modelica.Constants.pi
    k    : Thermal conductivity (material constant)
    L    : Length of cylinder
    log  : Modelica.Math.log;
    r_out: Outer radius of cylinder
    r_in : Inner radius of cylinder
    </pre>
    </li>
</li>
</ul>
<pre>
    Typical values for k at 20 degC in W/(m.K):
      aluminium   220
      concrete      1
      copper      384
      iron         74
      silver      407
      steel        45 .. 15 (V2A)
      wood         0.1 ... 0.2
</pre>
</HTML>
"));

        equation 
          Q_flow = G * dT;
        end ThermalConductor;
        model Convection "Lumped thermal element for heat convection"
          Modelica.SIunits.HeatFlowRate Q_flow "Heat flow rate from solid -> fluid";
          Modelica.SIunits.TemperatureDifference dT "= solid.T - fluid.T";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -62,80},{98, -80}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -90,80},{ -60, -80}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Backward),Text(extent = {{ -117, -88},{124, -128}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{100,0},{100,0}}, color = {0,127,255}),Line(points = {{ -60,20},{76,20}}, color = {191,0,0}),Line(points = {{ -60, -20},{76, -20}}, color = {191,0,0}),Line(points = {{ -34,80},{ -34, -80}}, color = {0,127,255}),Line(points = {{6,80},{6, -80}}, color = {0,127,255}),Line(points = {{40,80},{40, -80}}, color = {0,127,255}),Line(points = {{76,80},{76, -80}}, color = {0,127,255}),Line(points = {{ -34, -80},{ -44, -60}}, color = {0,127,255}),Line(points = {{ -34, -80},{ -24, -60}}, color = {0,127,255}),Line(points = {{6, -80},{ -4, -60}}, color = {0,127,255}),Line(points = {{6, -80},{16, -60}}, color = {0,127,255}),Line(points = {{40, -80},{30, -60}}, color = {0,127,255}),Line(points = {{40, -80},{50, -60}}, color = {0,127,255}),Line(points = {{76, -80},{66, -60}}, color = {0,127,255}),Line(points = {{76, -80},{86, -60}}, color = {0,127,255}),Line(points = {{56, -30},{76, -20}}, color = {191,0,0}),Line(points = {{56, -10},{76, -20}}, color = {191,0,0}),Line(points = {{56,10},{76,20}}, color = {191,0,0}),Line(points = {{56,30},{76,20}}, color = {191,0,0})}), Documentation(info = "<HTML>
<p>
This is a model of linear heat convection, e.g., the heat transfer
between a plate and the surrounding air. It may be used for complicated
solid geometries and fluid flow over the solid by determining the
convective thermal conductance Gc by measurements. The basic constitutive
equation for convection is
</p>
<pre>
   Q_flow = Gc*(solid.T - fluid.T);
   Q_flow: Heat flow rate from connector 'solid' (e.g. a plate)
      to connector 'fluid' (e.g. the surrounding air)
</pre>
<p>
Gc = G.signal[1] is an input signal to the component, since Gc is
nearly never constant in practice. For example, Gc may be a function
of the speed of a cooling fan. For simple situations,
Gc may be <i>calculated</i> according to
</p>
<pre>
   Gc = A*h
   A: Convection area (e.g. perimeter*length of a box)
   h: Heat transfer coefficient
</pre>
<p>
where the heat transfer coefficient h is calculated
from properties of the fluid flowing over the solid. Examples:
</p>
<p>
<b>Machines cooled by air</b> (empirical, very rough approximation according
to R. Fischer: Elektrische Maschinen, 10th edition, Hanser-Verlag 1999,
p. 378):
</p>
<pre>
    h = 7.8*v^0.78 [W/(m2.K)] (forced convection)
      = 12         [W/(m2.K)] (free convection)
    where
      v: Air velocity in [m/s]
</pre>
<p><b>Laminar</b> flow with constant velocity of a fluid along a
<b>flat plate</b> where the heat flow rate from the plate
to the fluid (= solid.Q_flow) is kept constant
(according to J.P.Holman: Heat Transfer, 8th edition,
McGraw-Hill, 1997, p.270):
</p>
<pre>
   h  = Nu*k/x;
   Nu = 0.453*Re^(1/2)*Pr^(1/3);
   where
      h  : Heat transfer coefficient
      Nu : = h*x/k       (Nusselt number)
      Re : = v*x*rho/mue (Reynolds number)
      Pr : = cp*mue/k    (Prandtl number)
      v  : Absolute velocity of fluid
      x  : distance from leading edge of flat plate
      rho: density of fluid (material constant
      mue: dynamic viscosity of fluid (material constant)
      cp : specific heat capacity of fluid (material constant)
      k  : thermal conductivity of fluid (material constant)
   and the equation for h holds, provided
      Re < 5e5 and 0.6 < Pr < 50
</pre>
</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -90,80},{ -60, -80}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Backward),Line(points = {{100,0},{100,0}}, color = {0,127,255}),Line(points = {{100,0},{100,0}}, color = {0,127,255}),Line(points = {{100,0},{100,0}}, color = {0,127,255}),Text(extent = {{ -35,42},{ -5,20}}, lineColor = {255,0,0}, textString = "Q_flow"),Line(points = {{ -60,20},{76,20}}, color = {191,0,0}),Line(points = {{ -60, -20},{76, -20}}, color = {191,0,0}),Line(points = {{ -34,80},{ -34, -80}}, color = {0,127,255}),Line(points = {{6,80},{6, -80}}, color = {0,127,255}),Line(points = {{40,80},{40, -80}}, color = {0,127,255}),Line(points = {{76,80},{76, -80}}, color = {0,127,255}),Line(points = {{ -34, -80},{ -44, -60}}, color = {0,127,255}),Line(points = {{ -34, -80},{ -24, -60}}, color = {0,127,255}),Line(points = {{6, -80},{ -4, -60}}, color = {0,127,255}),Line(points = {{6, -80},{16, -60}}, color = {0,127,255}),Line(points = {{40, -80},{30, -60}}, color = {0,127,255}),Line(points = {{40, -80},{50, -60}}, color = {0,127,255}),Line(points = {{76, -80},{66, -60}}, color = {0,127,255}),Line(points = {{76, -80},{86, -60}}, color = {0,127,255}),Line(points = {{56, -30},{76, -20}}, color = {191,0,0}),Line(points = {{56, -10},{76, -20}}, color = {191,0,0}),Line(points = {{56,10},{76,20}}, color = {191,0,0}),Line(points = {{56,30},{76,20}}, color = {191,0,0})}));
          Modelica.Blocks.Interfaces.RealInput Gc "Signal representing the convective thermal conductance in [W/K]" annotation(Placement(transformation(origin = {0,100}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
          Interfaces.HeatPort_a solid annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Interfaces.HeatPort_b fluid annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));

        equation 
          dT = solid.T - fluid.T;
          solid.Q_flow = Q_flow;
          fluid.Q_flow =  -Q_flow;
          Q_flow = Gc * dT;
        end Convection;
        model BodyRadiation "Lumped thermal element for radiation heat transfer"
          extends Interfaces.Element1D;
          parameter Real Gr(unit = "m2") "Net radiation conductance between two surfaces (see docu)";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{50,80},{90, -80}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Backward),Rectangle(extent = {{ -90,80},{ -50, -80}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Backward),Line(points = {{ -36,10},{36,10}}, color = {191,0,0}),Line(points = {{ -36,10},{ -26,16}}, color = {191,0,0}),Line(points = {{ -36,10},{ -26,4}}, color = {191,0,0}),Line(points = {{ -36, -10},{36, -10}}, color = {191,0,0}),Line(points = {{26, -16},{36, -10}}, color = {191,0,0}),Line(points = {{26, -4},{36, -10}}, color = {191,0,0}),Line(points = {{ -36, -30},{36, -30}}, color = {191,0,0}),Line(points = {{ -36, -30},{ -26, -24}}, color = {191,0,0}),Line(points = {{ -36, -30},{ -26, -36}}, color = {191,0,0}),Line(points = {{ -36,30},{36,30}}, color = {191,0,0}),Line(points = {{26,24},{36,30}}, color = {191,0,0}),Line(points = {{26,36},{36,30}}, color = {191,0,0}),Text(extent = {{ -132,144},{108,84}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -119, -86},{117, -125}}, lineColor = {0,0,0}, textString = "G=%G"),Rectangle(extent = {{ -50,80},{ -44, -80}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{45,80},{50, -80}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
<p>
This is a model describing the thermal radiation, i.e., electromagnetic
radiation emitted between two bodies as a result of their temperatures.
The following constitutive equation is used:
</p>
<pre>
    Q_flow = Gr*sigma*(port_a.T^4 - port_b.T^4);
</pre>
<p>
where Gr is the radiation conductance and sigma is the Stefan-Boltzmann
constant (= Modelica.Constants.sigma). Gr may be determined by
measurements and is assumed to be constant over the range of operations.
</p>
<p>
For simple cases, Gr may be analytically computed. The analytical
equations use epsilon, the emission value of a body which is in the
range 0..1. Epsilon=1, if the body absorbs all radiation (= black body).
Epsilon=0, if the body reflects all radiation and does not absorb any.
</p>
<pre>
   Typical values for epsilon:
   aluminium, polished    0.04
   copper, polished       0.04
   gold, polished         0.02
   paper                  0.09
   rubber                 0.95
   silver, polished       0.02
   wood                   0.85..0.9
</pre>
<p><b>Analytical Equations for Gr</b></p>
<p>
<b>Small convex object in large enclosure</b>
(e.g., a hot machine in a room):
</p>
<pre>
    Gr = e*A
    where
       e: Emission value of object (0..1)
       A: Surface area of object where radiation
          heat transfer takes place
</pre>
<p><b>Two parallel plates</b>:</p>
<pre>
    Gr = A/(1/e1 + 1/e2 - 1)
    where
       e1: Emission value of plate1 (0..1)
       e2: Emission value of plate2 (0..1)
       A : Area of plate1 (= area of plate2)
</pre>
<p><b>Two long cylinders in each other</b>, where radiation takes
place from the inner to the outer cylinder):
</p>
<pre>
    Gr = 2*pi*r1*L/(1/e1 + (1/e2 - 1)*(r1/r2))
    where
       pi: = Modelica.Constants.pi
       r1: Radius of inner cylinder
       r2: Radius of outer cylinder
       L : Length of the two cylinders
       e1: Emission value of inner cylinder (0..1)
       e2: Emission value of outer cylinder (0..1)
</pre>
</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -90,80},{ -56, -80}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Backward),Line(points = {{ -56,80},{ -56, -80}}, color = {0,0,0}, thickness = 1),Line(points = {{50,80},{50, -80}}, color = {0,0,0}, thickness = 1),Rectangle(extent = {{50,80},{90, -80}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Backward),Line(points = {{ -40,10},{40,10}}, color = {191,0,0}),Line(points = {{ -40,10},{ -30,16}}, color = {191,0,0}),Line(points = {{ -40,10},{ -30,4}}, color = {191,0,0}),Line(points = {{ -40, -10},{40, -10}}, color = {191,0,0}),Line(points = {{30, -16},{40, -10}}, color = {191,0,0}),Line(points = {{30, -4},{40, -10}}, color = {191,0,0}),Line(points = {{ -40, -30},{40, -30}}, color = {191,0,0}),Line(points = {{ -40, -30},{ -30, -24}}, color = {191,0,0}),Line(points = {{ -40, -30},{ -30, -36}}, color = {191,0,0}),Line(points = {{ -40,30},{40,30}}, color = {191,0,0}),Line(points = {{30,24},{40,30}}, color = {191,0,0}),Line(points = {{30,36},{40,30}}, color = {191,0,0})}));

        equation 
          Q_flow = Gr * Modelica.Constants.sigma * (port_a.T ^ 4 - port_b.T ^ 4);
        end BodyRadiation;
        model ThermalCollector
          parameter Integer m(min = 1) = 3 "number of collected heat flows";
          Interfaces.HeatPort_a port_a[m] annotation(Placement(transformation(extent = {{ -10,110},{10,90}})));
          Interfaces.HeatPort_b port_b annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}})));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -10, -90},{ -10,0},{ -100,90},{ -80,90},{0,20},{80,90},{100,90},{10,0},{10, -90},{ -10, -90}}, lineColor = {95,95,95}, smooth = Smooth.None, fillColor = {175,175,175}, fillPattern = FillPattern.Solid),Text(extent = {{ -100, -20},{100, -80}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100,80},{100,40}}, lineColor = {0,0,255}, textString = "m=%m")}), Documentation(info = "<html>
<p>
This is a model to collect the heat flows from <i>m</i> heatports to one single heatport.
</p>
</html>"));

        equation 
          port_b.Q_flow + sum(port_a.Q_flow) = 0;
          port_a.T = fill(port_b.T, m);
        end ThermalCollector;
      end Components;
      package Sources "Thermal sources"
        extends Modelica.Icons.Library2;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -100, -20},{36, -20}}, color = {191,0,0}, thickness = 0.5),Line(points = {{ -100, -60},{38, -60}}, color = {191,0,0}, thickness = 0.5),Polygon(points = {{30, -80},{30, -40},{60, -60},{30, -80}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{30, -40},{30,0},{60, -20},{30, -40}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{60,0},{80, -80}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>

</html>"));
        model FixedTemperature "Fixed temperature boundary condition in Kelvin"
          parameter Modelica.SIunits.Temperature T "Fixed temperature at port";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -121,162},{119,102}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -121, -105},{119, -151}}, lineColor = {0,0,0}, textString = "T=%T"),Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, pattern = LinePattern.None, fillColor = {159,159,223}, fillPattern = FillPattern.Backward),Text(extent = {{0,0},{ -100, -100}}, lineColor = {0,0,0}, textString = "K"),Line(points = {{ -52,0},{56,0}}, color = {191,0,0}, thickness = 0.5),Polygon(points = {{50, -20},{50,20},{90,0},{50, -20}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
<p>
This model defines a fixed temperature T at its port in Kelvin,
i.e., it defines a fixed temperature as a boundary condition.
</p>
</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -101}}, lineColor = {0,0,0}, pattern = LinePattern.None, fillColor = {159,159,223}, fillPattern = FillPattern.Backward),Line(points = {{ -52,0},{56,0}}, color = {191,0,0}, thickness = 0.5),Text(extent = {{0,0},{ -100, -100}}, lineColor = {0,0,0}, textString = "K"),Polygon(points = {{52, -20},{52,20},{90,0},{52, -20}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid)}));
          Interfaces.HeatPort_b port annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));

        equation 
          port.T = T;
        end FixedTemperature;
        model PrescribedTemperature "Variable temperature boundary condition in Kelvin"
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, pattern = LinePattern.None, fillColor = {159,159,223}, fillPattern = FillPattern.Backward),Line(points = {{ -102,0},{64,0}}, color = {191,0,0}, thickness = 0.5),Text(extent = {{0,0},{ -100, -100}}, lineColor = {0,0,0}, textString = "K"),Text(extent = {{ -125,162},{115,102}}, textString = "%name", lineColor = {0,0,255}),Polygon(points = {{50, -20},{50,20},{90,0},{50, -20}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
<p>
This model represents a variable temperature boundary condition.
The temperature in [K] is given as input signal <b>T</b>
to the model. The effect is that an instance of this model acts as
an infinite reservoir able to absorb or generate as much energy
as required to keep the temperature at the specified value.
</p>
</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, pattern = LinePattern.None, fillColor = {159,159,223}, fillPattern = FillPattern.Backward),Text(extent = {{0,0},{ -100, -100}}, lineColor = {0,0,0}, textString = "K"),Line(points = {{ -102,0},{64,0}}, color = {191,0,0}, thickness = 0.5),Polygon(points = {{52, -20},{52,20},{90,0},{52, -20}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid)}));
          Interfaces.HeatPort_b port annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput T annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));

        equation 
          port.T = T;
        end PrescribedTemperature;
        model FixedHeatFlow "Fixed heat flow boundary condition"
          parameter Modelica.SIunits.HeatFlowRate Q_flow "Fixed heat flow rate at port";
          parameter Modelica.SIunits.Temperature T_ref = 293.15 "Reference temperature";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha = 0 "Temperature coefficient of heat flow rate";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -134,120},{132,60}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -137, -52},{133, -100}}, lineColor = {0,0,0}, textString = "Q_flow=%Q_flow"),Line(points = {{ -100, -20},{48, -20}}, color = {191,0,0}, thickness = 0.5),Line(points = {{ -100,20},{46,20}}, color = {191,0,0}, thickness = 0.5),Polygon(points = {{40,0},{40,40},{70,20},{40,0}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{40, -40},{40,0},{70, -20},{40, -40}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{70,40},{90, -40}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -100,40},{0, -36}}, lineColor = {0,0,0}, textString = "Q_flow=const."),Line(points = {{ -48, -20},{60, -20}}, color = {191,0,0}, thickness = 0.5),Line(points = {{ -48,20},{60,20}}, color = {191,0,0}, thickness = 0.5),Polygon(points = {{60,0},{60,40},{90,20},{60,0}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{60, -40},{60,0},{90, -20},{60, -40}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
<p>
This model allows a specified amount of heat flow rate to be \"injected\"
into a thermal system at a given port.  The constant amount of heat
flow rate Q_flow is given as a parameter. The heat flows into the
component to which the component FixedHeatFlow is connected,
if parameter Q_flow is positive.
</p>
<p>
If parameter alpha is > 0, the heat flow is mulitplied by (1 + alpha*(port.T - T_ref))
in order to simulate temperature dependent losses (which are given an reference temperature T_ref).
</p>
</HTML>
"));
          Interfaces.HeatPort_b port annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));

        equation 
          port.Q_flow =  -Q_flow * (1 + alpha * (port.T - T_ref));
        end FixedHeatFlow;
        model PrescribedHeatFlow "Prescribed heat flow boundary condition"
          parameter Modelica.SIunits.Temperature T_ref = 293.15 "Reference temperature";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha = 0 "Temperature coefficient of heat flow rate";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -60, -20},{40, -20}}, color = {191,0,0}, thickness = 0.5),Line(points = {{ -60,20},{40,20}}, color = {191,0,0}, thickness = 0.5),Line(points = {{ -80,0},{ -60, -20}}, color = {191,0,0}, thickness = 0.5),Line(points = {{ -80,0},{ -60,20}}, color = {191,0,0}, thickness = 0.5),Polygon(points = {{40,0},{40,40},{70,20},{40,0}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{40, -40},{40,0},{70, -20},{40, -40}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{70,40},{90, -40}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -134,120},{132,60}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<HTML>
<p>
This model allows a specified amount of heat flow rate to be \"injected\"
into a thermal system at a given port.  The amount of heat
is given by the input signal Q_flow into the model. The heat flows into the
component to which the component PrescribedHeatFlow is connected,
if the input signal is positive.
</p>
<p>
If parameter alpha is > 0, the heat flow is mulitplied by (1 + alpha*(port.T - T_ref))
in order to simulate temperature dependent losses (which are given an reference temperature T_ref).
</p>
</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -60, -20},{68, -20}}, color = {191,0,0}, thickness = 0.5),Line(points = {{ -60,20},{68,20}}, color = {191,0,0}, thickness = 0.5),Line(points = {{ -80,0},{ -60, -20}}, color = {191,0,0}, thickness = 0.5),Line(points = {{ -80,0},{ -60,20}}, color = {191,0,0}, thickness = 0.5),Polygon(points = {{60,0},{60,40},{90,20},{60,0}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{60, -40},{60,0},{90, -20},{60, -40}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid)}));
          Modelica.Blocks.Interfaces.RealInput Q_flow annotation(Placement(transformation(origin = { -100,0}, extent = {{20, -20},{ -20,20}}, rotation = 180)));
          Interfaces.HeatPort_b port annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));

        equation 
          port.Q_flow =  -Q_flow * (1 + alpha * (port.T - T_ref));
        end PrescribedHeatFlow;
      end Sources;
      package Sensors "Thermal sensors"
        extends Modelica.Icons.Library2;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -60,10},{40, -90}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -50, -16},{ -36, -25}}, color = {0,0,0}),Line(points = {{ -35,0},{ -25, -14}}, color = {0,0,0}),Line(points = {{ -10,7},{ -10, -10}}, color = {0,0,0}),Line(points = {{15,0},{5, -14}}, color = {0,0,0}),Line(points = {{30, -15},{16, -25}}, color = {0,0,0}),Ellipse(extent = {{ -15, -35},{ -5, -45}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -10, -40},{ -6, -26}}, color = {0,0,0}),Polygon(points = {{ -12, -24},{ -0.5, -27},{2,1.5},{ -12, -24}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>

</html>"));
        model TemperatureSensor "Absolute temperature sensor in Kelvin"
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -20, -98},{20, -60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -12,40},{12, -68}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Line(points = {{12,0},{90,0}}, color = {0,0,255}),Line(points = {{ -94,0},{ -14,0}}, color = {191,0,0}),Polygon(points = {{ -12,40},{ -12,80},{ -10,86},{ -6,88},{0,90},{6,88},{10,86},{12,80},{12,40},{ -12,40}}, lineColor = {0,0,0}, lineThickness = 0.5),Line(points = {{ -12,40},{ -12, -64}}, color = {0,0,0}, thickness = 0.5),Line(points = {{12,40},{12, -64}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -40, -20},{ -12, -20}}, color = {0,0,0}),Line(points = {{ -40,20},{ -12,20}}, color = {0,0,0}),Line(points = {{ -40,60},{ -12,60}}, color = {0,0,0}),Text(extent = {{102, -28},{60, -78}}, lineColor = {0,0,0}, textString = "K")}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -20, -98},{20, -60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -12,40},{12, -68}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Line(points = {{12,0},{90,0}}, color = {0,0,255}),Line(points = {{ -90,0},{ -12,0}}, color = {191,0,0}),Polygon(points = {{ -12,40},{ -12,80},{ -10,86},{ -6,88},{0,90},{6,88},{10,86},{12,80},{12,40},{ -12,40}}, lineColor = {0,0,0}, lineThickness = 0.5),Line(points = {{ -12,40},{ -12, -64}}, color = {0,0,0}, thickness = 0.5),Line(points = {{12,40},{12, -64}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -40, -20},{ -12, -20}}, color = {0,0,0}),Line(points = {{ -40,20},{ -12,20}}, color = {0,0,0}),Line(points = {{ -40,60},{ -12,60}}, color = {0,0,0}),Text(extent = {{126, -20},{26, -120}}, lineColor = {0,0,0}, textString = "K"),Text(extent = {{ -132,144},{108,84}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<HTML>
<p>
This is an ideal absolute temperature sensor which returns
the temperature of the connected port in Kelvin as an output
signal.  The sensor itself has no thermal interaction with
whatever it is connected to.  Furthermore, no
thermocouple-like lags are associated with this
sensor model.
</p>
</HTML>
"));
          Modelica.Blocks.Interfaces.RealOutput T annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Interfaces.HeatPort_a port annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));

        equation 
          T = port.T;
          port.Q_flow = 0;
        end TemperatureSensor;
        model RelTemperatureSensor "Relative Temperature sensor"
          extends Modelica.Icons.TranslationalSensor;
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -90,0},{ -70,0},{ -70,0}}, color = {191,0,0}),Line(points = {{ -90,0},{ -70,0},{ -70,0}}, color = {191,0,0}),Line(points = {{70,0},{90,0},{90,0}}, color = {191,0,0}),Line(points = {{0, -30},{0, -80}}, color = {0,0,255}),Text(extent = {{ -140,94},{144,34}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{92, -62},{34, -122}}, lineColor = {0,0,0}, textString = "K")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -90,0},{ -70,0},{ -70,0}}, color = {191,0,0}),Line(points = {{ -98,0},{ -70,0},{ -70,0}}, color = {191,0,0}),Line(points = {{70,0},{94,0},{94,0}}, color = {191,0,0}),Line(points = {{0, -30},{0, -80}}, color = {0,0,255}),Text(extent = {{64, -74},{32, -102}}, lineColor = {0,0,0}, textString = "K")}), Documentation(info = "<HTML>
<p>
The relative temperature \"port_a.T - port_b.T\" is determined between
the two ports of this component and is provided as output signal in Kelvin.
</p>
</HTML>
"));
          Interfaces.HeatPort_a port_a annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Interfaces.HeatPort_b port_b annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput T_rel annotation(Placement(transformation(origin = {0, -90}, extent = {{10, -10},{ -10,10}}, rotation = 90)));

        equation 
          T_rel = port_a.T - port_b.T;
          0 = port_a.Q_flow;
          0 = port_b.Q_flow;
        end RelTemperatureSensor;
        model HeatFlowSensor "Heat flow rate sensor"
          extends Modelica.Icons.RotationalSensor;
          Modelica.Blocks.Interfaces.RealOutput Q_flow "Heat flow from port_a to port_b" annotation(Placement(transformation(origin = {0, -100}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -70,0},{ -95,0}}, color = {191,0,0}),Line(points = {{0, -70},{0, -90}}, color = {0,0,255}),Line(points = {{94,0},{69,0}}, color = {191,0,0})}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{33, -58},{88, -116}}, lineColor = {0,0,0}, textString = "Q_flow"),Line(points = {{ -70,0},{ -90,0}}, color = {191,0,0}),Line(points = {{69,0},{90,0}}, color = {191,0,0}),Line(points = {{0, -70},{0, -90}}, color = {0,0,255}),Text(extent = {{ -132,144},{108,84}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<HTML>
<p>
This model is capable of monitoring the heat flow rate flowing through
this component. The sensed value of heat flow rate is the amount that
passes through this sensor while keeping the temperature drop across the
sensor zero.  This is an ideal model so it does not absorb any energy
and it has no direct effect on the thermal response of a system it is included in.
The output signal is positive, if the heat flows from port_a
to port_b.
</p>
</HTML>
"));
          Interfaces.HeatPort_a port_a annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Interfaces.HeatPort_b port_b annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));

        equation 
          port_a.T = port_b.T;
          port_a.Q_flow + port_b.Q_flow = 0;
          Q_flow = port_a.Q_flow;
        end HeatFlowSensor;
      end Sensors;
      package Celsius "Components with Celsius input and/or output"
        extends Modelica.Icons.Library2;
        model ToKelvin "Conversion block from Celsius to Kelvin"
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -40, -50},{ -99, -99}}, lineColor = {0,0,0}, lineThickness = 1, textString = "C"),Text(extent = {{100, -47},{44, -100}}, lineColor = {0,0,0}, textString = "K"),Line(points = {{ -100,0},{ -40,0}}, color = {0,0,255}),Line(points = {{41,0},{100,0}}, color = {0,0,255})}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{112, -40},{32, -120}}, lineColor = {0,0,0}, textString = "K"),Text(extent = {{ -31, -39},{ -111, -119}}, lineColor = {0,0,0}, lineThickness = 1, textString = "C"),Line(points = {{ -41,0},{ -100,0}}, color = {0,0,255}),Line(points = {{100,0},{40,0}}, color = {0,0,255}),Text(extent = {{ -137,99},{132,49}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<HTML>
<p>
This component converts an input signal from Celsius to Kelvin
and provide is as output signal.
</p>
</HTML>
"));
          Modelica.Blocks.Interfaces.RealInput Celsius annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput Kelvin annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));

        equation 
          Kelvin = from_degC(Celsius);
        end ToKelvin;
        model FromKelvin "Conversion from Kelvin to Celsius"
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -137,99},{132,49}}, textString = "%name", lineColor = {0,0,255}),Ellipse(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -34, -42},{ -114, -122}}, lineColor = {0,0,0}, lineThickness = 1, textString = "K"),Text(extent = {{110, -39},{30, -119}}, lineColor = {0,0,0}, textString = "C"),Line(points = {{ -40,0},{ -100,0}}, color = {0,0,255}),Line(points = {{40,0},{100,0}}, color = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -42, -41},{ -101, -98}}, lineColor = {0,0,0}, lineThickness = 1, textString = "K"),Text(extent = {{100, -40},{30, -100}}, lineColor = {0,0,0}, textString = "C"),Line(points = {{ -100,0},{ -40,0}}, color = {0,0,255}),Line(points = {{40,0},{100,0}}, color = {0,0,255})}), Documentation(info = "<HTML>
<p>
This component converts an input signal from Kelvin to Celsius
and provides is as output signal.
</p>
</HTML>
"));
          Modelica.Blocks.Interfaces.RealInput Kelvin annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput Celsius annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));

        equation 
          Celsius = to_degC(Kelvin);
        end FromKelvin;
        model FixedTemperature "Fixed temperature boundary condition in degree Celsius"
          parameter Modelica.SIunits.Conversions.NonSIunits.Temperature_degC T "Fixed Temperature at the port";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -118,165},{122,105}}, textString = "%name", lineColor = {0,0,255}),Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, pattern = LinePattern.None, fillColor = {159,159,223}, fillPattern = FillPattern.Backward),Text(extent = {{0,0},{ -100, -100}}, lineColor = {0,0,0}, textString = "C"),Polygon(points = {{52, -20},{52,20},{90,0},{52, -20}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -145, -102},{135, -151}}, lineColor = {0,0,0}, textString = "T=%T"),Line(points = {{ -42,0},{66,0}}, color = {191,0,0}, thickness = 0.5)}), Documentation(info = "<HTML>
<p>
This model defines a fixed temperature T at its port in [degC],
i.e., it defines a fixed temperature as a boundary condition.
</p>
</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, pattern = LinePattern.None, fillColor = {159,159,223}, fillPattern = FillPattern.Backward),Line(points = {{ -42,0},{66,0}}, color = {191,0,0}, thickness = 0.5),Text(extent = {{0,0},{ -100, -100}}, lineColor = {0,0,0}, textString = "C"),Polygon(points = {{52, -20},{52,20},{90,0},{52, -20}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid)}));
          Interfaces.HeatPort_b port annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));

        equation 
          port.T = from_degC(T);
        end FixedTemperature;
        model PrescribedTemperature "Variable temperature boundary condition in Celsius"
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, pattern = LinePattern.None, fillColor = {159,159,223}, fillPattern = FillPattern.Backward),Line(points = {{ -102,0},{64,0}}, color = {191,0,0}, thickness = 0.5),Text(extent = {{0,0},{ -100, -100}}, lineColor = {0,0,0}, textString = "C"),Text(extent = {{ -122,163},{118,103}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -102,0},{64,0}}, color = {191,0,0}, thickness = 0.5),Polygon(points = {{52, -20},{52,20},{90,0},{52, -20}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
<p>
This model represents a variable temperature boundary condition
The temperature value in [degC] is given by the input signal
to the model. The effect is that an instance of this model acts as
an infinite reservoir able to absorb or generate as much energy
as required to keep the temperature at the specified value.
</p>
</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, pattern = LinePattern.None, fillColor = {159,159,223}, fillPattern = FillPattern.Backward),Line(points = {{ -102,0},{64,0}}, color = {191,0,0}, thickness = 0.5),Text(extent = {{0,0},{ -100, -100}}, lineColor = {0,0,0}, textString = "C"),Polygon(points = {{52, -20},{52,20},{90,0},{52, -20}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid)}));
          Interfaces.HeatPort_b port annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput T annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));

        equation 
          port.T = from_degC(T);
        end PrescribedTemperature;
        annotation(Documentation(info = "<HTML>
<p>
The components of this package are provided for the convenience of
people working mostly with Celsius units, since all models
in package HeatTransfer are based on Kelvin units.
</p>
<p>
Note, that in package SIunits.Conversions, functions are provided
to convert between the units Kelvin, degree Celsius, degree Fahrenheit,
and degree Rankine. These functions allow, e.g., a direct conversion
of units at all places where Kelvin is required as parameter.
Example:
</p>
<pre>
    <b>import</b> SIunits.Conversions.*;
    Modelica.Thermal.HeatTransfer.HeatCapacitor C(T0 = from_degC(20));
</pre>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{38,10},{ -62, -90}}, lineColor = {0,0,0}, textString = "C")}));
        model TemperatureSensor "Absolute temperature sensor in Celsius"
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -20, -98},{20, -60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -12,40},{12, -68}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Line(points = {{12,0},{90,0}}, color = {0,0,255}),Line(points = {{ -94,0},{ -12,0}}, color = {191,0,0}),Polygon(points = {{ -12,40},{ -12,80},{ -10,86},{ -6,88},{0,90},{6,88},{10,86},{12,80},{12,40},{ -12,40}}, lineColor = {0,0,0}, lineThickness = 0.5),Line(points = {{ -12,40},{ -12, -64}}, color = {0,0,0}, thickness = 0.5),Line(points = {{12,40},{12, -64}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -40, -20},{ -12, -20}}, color = {0,0,0}),Line(points = {{ -40,20},{ -12,20}}, color = {0,0,0}),Line(points = {{ -40,60},{ -12,60}}, color = {0,0,0}),Text(extent = {{102, -22},{60, -74}}, lineColor = {0,0,0}, textString = "C")}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -20, -98},{20, -60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -12,40},{12, -68}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Line(points = {{12,0},{90,0}}, color = {0,0,255}),Line(points = {{ -90,0},{ -12,0}}, color = {191,0,0}),Polygon(points = {{ -12,40},{ -12,80},{ -10,86},{ -6,88},{0,90},{6,88},{10,86},{12,80},{12,40},{ -12,40}}, lineColor = {0,0,0}, lineThickness = 0.5),Line(points = {{ -12,40},{ -12, -64}}, color = {0,0,0}, thickness = 0.5),Line(points = {{12,40},{12, -64}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -40, -20},{ -12, -20}}, color = {0,0,0}),Line(points = {{ -40,20},{ -12,20}}, color = {0,0,0}),Line(points = {{ -40,60},{ -12,60}}, color = {0,0,0}),Text(extent = {{126, -20},{26, -120}}, lineColor = {0,0,0}, textString = "C"),Text(extent = {{ -132,144},{108,84}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<HTML>
<p>
This is an ideal absolute temperature sensor which returns
the temperature of the connected port in Celsius as an output
signal.  The sensor itself has no thermal interaction with
whatever it is connected to.  Furthermore, no
thermocouple-like lags are associated with this
sensor model.
</p>
</HTML>
"));
          Modelica.Blocks.Interfaces.RealOutput T annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Interfaces.HeatPort_a port annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));

        equation 
          T = to_degC(port.T);
          port.Q_flow = 0;
        end TemperatureSensor;
      end Celsius;
      package Fahrenheit "Components with Fahrenheit input and/or output"
        extends Modelica.Icons.Library2;
        model ToKelvin "Conversion block from Fahrenheit to Kelvin"
          annotation(Diagram(graphics = {Ellipse(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -40, -50},{ -99, -99}}, lineColor = {0,0,0}, lineThickness = 1, textString = "F"),Text(extent = {{100, -47},{44, -100}}, lineColor = {0,0,0}, textString = "K"),Line(points = {{ -100,0},{ -40,0}}, color = {0,0,255}),Line(points = {{41,0},{100,0}}, color = {0,0,255})}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{112, -40},{32, -120}}, lineColor = {0,0,0}, textString = "K"),Text(extent = {{ -31, -39},{ -111, -119}}, lineColor = {0,0,0}, lineThickness = 1, textString = "F"),Line(points = {{ -41,0},{ -100,0}}, color = {0,0,255}),Line(points = {{100,0},{40,0}}, color = {0,0,255}),Text(extent = {{ -137,99},{132,49}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<HTML>
<p>
This component converts a input signal from degree Fahrenheit to Kelvin
and provides is as output signal.
</p>
</HTML>
"));
          Modelica.Blocks.Interfaces.RealInput Fahrenheit annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput Kelvin annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));

        equation 
          Kelvin = from_degF(Fahrenheit);
        end ToKelvin;
        model FromKelvin "Conversion from Kelvin to Fahrenheit"
          parameter Integer n = 1 "Number of inputs (= number of outputs)";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -137,99},{132,49}}, textString = "%name", lineColor = {0,0,255}),Ellipse(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -34, -42},{ -114, -122}}, lineColor = {0,0,0}, lineThickness = 1, textString = "K"),Text(extent = {{110, -39},{30, -119}}, lineColor = {0,0,0}, textString = "F"),Line(points = {{ -40,0},{ -100,0}}, color = {0,0,255}),Line(points = {{40,0},{100,0}}, color = {0,0,255})}), Diagram(graphics = {Ellipse(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -42, -41},{ -101, -98}}, lineColor = {0,0,0}, lineThickness = 1, textString = "K"),Text(extent = {{100, -40},{30, -100}}, lineColor = {0,0,0}, textString = "F"),Line(points = {{ -100,0},{ -40,0}}, color = {0,0,255}),Line(points = {{40,0},{100,0}}, color = {0,0,255})}), Documentation(info = "<HTML>
<p>
This component converts all input signals from Kelvin to Fahrenheit
and provides them as output signals.
</p>
</HTML>
"));
          Modelica.Blocks.Interfaces.RealInput Kelvin annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput Fahrenheit annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));

        equation 
          Fahrenheit = to_degF(Kelvin);
        end FromKelvin;
        model FixedTemperature "Fixed temperature boundary condition in Fahrenheit"
          parameter Modelica.SIunits.Conversions.NonSIunits.Temperature_degF T "Fixed Temperature at the port";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -118,165},{122,105}}, textString = "%name", lineColor = {0,0,255}),Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, pattern = LinePattern.None, fillColor = {159,159,223}, fillPattern = FillPattern.Backward),Text(extent = {{0,0},{ -100, -100}}, lineColor = {0,0,0}, textString = "F"),Polygon(points = {{52, -20},{52,20},{90,0},{52, -20}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -145, -102},{135, -151}}, lineColor = {0,0,0}, textString = "T=%T"),Line(points = {{ -42,0},{66,0}}, color = {191,0,0}, thickness = 0.5)}), Documentation(info = "<HTML>
<p>
This model defines a fixed temperature T at its port in [degF],
i.e., it defines a fixed temperature as a boundary condition.
</p>
</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, pattern = LinePattern.None, fillColor = {159,159,223}, fillPattern = FillPattern.Backward),Line(points = {{ -42,0},{66,0}}, color = {191,0,0}, thickness = 0.5),Text(extent = {{0,0},{ -100, -100}}, lineColor = {0,0,0}, textString = "F"),Polygon(points = {{52, -20},{52,20},{90,0},{52, -20}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid)}));
          Interfaces.HeatPort_b port annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));

        equation 
          port.T = from_degF(T);
        end FixedTemperature;
        model PrescribedTemperature "Variable temperature boundary condition in Fahrenheit"
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, pattern = LinePattern.None, fillColor = {159,159,223}, fillPattern = FillPattern.Backward),Line(points = {{ -102,0},{64,0}}, color = {191,0,0}, thickness = 0.5),Text(extent = {{0,0},{ -100, -100}}, lineColor = {0,0,0}, textString = "F"),Text(extent = {{ -122,163},{118,103}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -102,0},{64,0}}, color = {191,0,0}, thickness = 0.5),Polygon(points = {{52, -20},{52,20},{90,0},{52, -20}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
<p>
This model represents a variable temperature boundary condition
The temperature value in [degF] is given by the input signal
to the model. The effect is that an instance of this model acts as
an infinite reservoir able to absorb or generate as much energy
as required to keep the temperature at the specified value.
</p>
</HTML>
"), Diagram(graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, pattern = LinePattern.None, fillColor = {159,159,223}, fillPattern = FillPattern.Backward),Line(points = {{ -102,0},{64,0}}, color = {191,0,0}, thickness = 0.5),Text(extent = {{0,0},{ -100, -100}}, lineColor = {0,0,0}, textString = "F"),Polygon(points = {{52, -20},{52,20},{90,0},{52, -20}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid)}));
          Interfaces.HeatPort_b port annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput T annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));

        equation 
          port.T = from_degF(T);
        end PrescribedTemperature;
        annotation(Documentation(info = "<HTML>
<p>
The components of this package are provided for the convenience of
people working mostly with Fahrenheit units, since all models
in package HeatTransfer are based on Kelvin units.
</p>
<p>
Note, that in package SIunits.Conversions, functions are provided
to convert between the units Kelvin, degree Celsius, degree Fahrenheit
and degree Rankine. These functions allow, e.g., a direct conversion
of units at all places where Kelvin is required as parameter.
Example:
</p>
<pre>
    <b>import</b> SIunits.Conversions.*;
    Modelica.Thermal.HeatTransfer.HeatCapacitor C(T0 = from_degF(70));
</pre>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{40,10},{ -60, -90}}, lineColor = {0,0,0}, textString = "F")}));
        model TemperatureSensor "Absolute temperature sensor in Fahrenheit"
          annotation(Diagram(graphics = {Ellipse(extent = {{ -20, -98},{20, -60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -12,40},{12, -68}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Line(points = {{12,0},{90,0}}, color = {0,0,255}),Line(points = {{ -94,0},{ -12,0}}, color = {191,0,0}),Polygon(points = {{ -12,40},{ -12,80},{ -10,86},{ -6,88},{0,90},{6,88},{10,86},{12,80},{12,40},{ -12,40}}, lineColor = {0,0,0}, lineThickness = 0.5),Line(points = {{ -12,40},{ -12, -64}}, color = {0,0,0}, thickness = 0.5),Line(points = {{12,40},{12, -64}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -40, -20},{ -12, -20}}, color = {0,0,0}),Line(points = {{ -40,20},{ -12,20}}, color = {0,0,0}),Line(points = {{ -40,60},{ -12,60}}, color = {0,0,0}),Text(extent = {{102, -22},{60, -74}}, lineColor = {0,0,0}, textString = "F")}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -20, -98},{20, -60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -12,40},{12, -68}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Line(points = {{12,0},{90,0}}, color = {0,0,255}),Line(points = {{ -90,0},{ -12,0}}, color = {191,0,0}),Polygon(points = {{ -12,40},{ -12,80},{ -10,86},{ -6,88},{0,90},{6,88},{10,86},{12,80},{12,40},{ -12,40}}, lineColor = {0,0,0}, lineThickness = 0.5),Line(points = {{ -12,40},{ -12, -64}}, color = {0,0,0}, thickness = 0.5),Line(points = {{12,40},{12, -64}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -40, -20},{ -12, -20}}, color = {0,0,0}),Line(points = {{ -40,20},{ -12,20}}, color = {0,0,0}),Line(points = {{ -40,60},{ -12,60}}, color = {0,0,0}),Text(extent = {{126, -20},{26, -120}}, lineColor = {0,0,0}, textString = "F"),Text(extent = {{ -132,144},{108,84}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<HTML>
<p>
This is an ideal absolute temperature sensor which returns
the temperature of the connected port in Fahrenheit as an output
signal.  The sensor itself has no thermal interaction with
whatever it is connected to.  Furthermore, no
thermocouple-like lags are associated with this
sensor model.
</p>
</HTML>
"));
          Modelica.Blocks.Interfaces.RealOutput T annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Interfaces.HeatPort_a port annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));

        equation 
          T = to_degF(port.T);
          port.Q_flow = 0;
        end TemperatureSensor;
      end Fahrenheit;
      package Rankine "Components with Rankine input and/or output"
        extends Modelica.Icons.Library2;
        model ToKelvin "Conversion block from Rankine to Kelvin"
          parameter Integer n = 1 "Number of inputs (= number of outputs)";
          annotation(Diagram(graphics = {Ellipse(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -40, -50},{ -99, -99}}, lineColor = {0,0,0}, lineThickness = 1, textString = "Rk"),Text(extent = {{100, -47},{44, -100}}, lineColor = {0,0,0}, textString = "K"),Line(points = {{ -100,0},{ -40,0}}, color = {0,0,255}),Line(points = {{41,0},{100,0}}, color = {0,0,255})}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{112, -40},{32, -120}}, lineColor = {0,0,0}, textString = "K"),Text(extent = {{ -31, -39},{ -111, -119}}, lineColor = {0,0,0}, lineThickness = 1, textString = "Rk"),Line(points = {{ -41,0},{ -100,0}}, color = {0,0,255}),Line(points = {{100,0},{40,0}}, color = {0,0,255}),Text(extent = {{ -137,99},{132,49}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<HTML>
<p>
This component converts all input signals from degree Rankine to Kelvin
and provides them as output signals.
</p>
</HTML>
"));
          Modelica.Blocks.Interfaces.RealInput Rankine annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput Kelvin annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));

        equation 
          Kelvin = from_degRk(Rankine);
        end ToKelvin;
        model FromKelvin "Conversion from Kelvin to Rankine"
          parameter Integer n = 1 "Number of inputs (= number of outputs)";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -137,99},{132,49}}, textString = "%name", lineColor = {0,0,255}),Ellipse(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -34, -42},{ -114, -122}}, lineColor = {0,0,0}, lineThickness = 1, textString = "K"),Text(extent = {{110, -39},{30, -119}}, lineColor = {0,0,0}, textString = "Rk"),Line(points = {{ -40,0},{ -100,0}}, color = {0,0,255}),Line(points = {{40,0},{100,0}}, color = {0,0,255})}), Diagram(graphics = {Ellipse(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -42, -41},{ -101, -98}}, lineColor = {0,0,0}, lineThickness = 1, textString = "K"),Text(extent = {{100, -40},{30, -100}}, lineColor = {0,0,0}, textString = "Rk"),Line(points = {{ -100,0},{ -40,0}}, color = {0,0,255}),Line(points = {{40,0},{100,0}}, color = {0,0,255})}), Documentation(info = "<HTML>
<p>
This component converts all input signals from Kelvin to Rankine
and provides them as output signals.
</p>
</HTML>
"));
          Modelica.Blocks.Interfaces.RealInput Kelvin annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput Rankine annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));

        equation 
          Rankine = to_degRk(Kelvin);
        end FromKelvin;
        model FixedTemperature "Fixed temperature boundary condition in Rankine"
          parameter Modelica.SIunits.Conversions.NonSIunits.Temperature_degRk T "Fixed Temperature at the port";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -118,165},{122,105}}, textString = "%name", lineColor = {0,0,255}),Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, pattern = LinePattern.None, fillColor = {159,159,223}, fillPattern = FillPattern.Backward),Text(extent = {{0,0},{ -100, -100}}, lineColor = {0,0,0}, textString = "Rk"),Polygon(points = {{52, -20},{52,20},{90,0},{52, -20}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -145, -102},{135, -151}}, lineColor = {0,0,0}, textString = "T=%T"),Line(points = {{ -42,0},{66,0}}, color = {191,0,0}, thickness = 0.5)}), Documentation(info = "<HTML>
<p>
This model defines a fixed temperature T at its port in degree Rankine,
[degRk], i.e., it defines a fixed temperature as a boundary condition.
</p>
</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, pattern = LinePattern.None, fillColor = {159,159,223}, fillPattern = FillPattern.Backward),Line(points = {{ -42,0},{66,0}}, color = {191,0,0}, thickness = 0.5),Text(extent = {{0,0},{ -100, -100}}, lineColor = {0,0,0}, textString = "Rk"),Polygon(points = {{52, -20},{52,20},{90,0},{52, -20}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid)}));
          Interfaces.HeatPort_b port annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));

        equation 
          port.T = from_degRk(T);
        end FixedTemperature;
        model PrescribedTemperature "Variable temperature boundary condition in Rankine"
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, pattern = LinePattern.None, fillColor = {159,159,223}, fillPattern = FillPattern.Backward),Line(points = {{ -102,0},{64,0}}, color = {191,0,0}, thickness = 0.5),Text(extent = {{0,0},{ -100, -100}}, lineColor = {0,0,0}, textString = "Rk"),Text(extent = {{ -122,163},{118,103}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -102,0},{64,0}}, color = {191,0,0}, thickness = 0.5),Polygon(points = {{52, -20},{52,20},{90,0},{52, -20}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
<p>
This model represents a variable temperature boundary condition
The temperature value in degree Rankine, [degRk] is given by the input signal
to the model. The effect is that an instance of this model acts as
an infinite reservoir able to absorb or generate as much energy
as required to keep the temperature at the specified value.
</p>
</HTML>
"), Diagram(graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, pattern = LinePattern.None, fillColor = {159,159,223}, fillPattern = FillPattern.Backward),Line(points = {{ -102,0},{64,0}}, color = {191,0,0}, thickness = 0.5),Text(extent = {{0,0},{ -100, -100}}, lineColor = {0,0,0}, textString = "Rk"),Polygon(points = {{52, -20},{52,20},{90,0},{52, -20}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid)}));
          Interfaces.HeatPort_b port annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput T annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));

        equation 
          port.T = from_degRk(T);
        end PrescribedTemperature;
        annotation(Documentation(info = "<HTML>
<p>
The components of this package are provided for the convenience of
people working mostly with Rankine units, since all models
in package HeatTransfer are based on Kelvin units.
</p>
<p>
Note, that in package SIunits.Conversions, functions are provided
to convert between the units Kelvin, degree Celsius, degree Fahrenheit
and degree Rankine. These functions allow, e.g., a direct conversion
of units at all places where Kelvin is required as parameter.
Example:
</p>
<pre>
    <b>import</b> SIunits.Conversions.*;
    Modelica.Thermal.HeatTransfer.HeatCapacitor C(T0 = from_degRk(500));
</pre>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{40,10},{ -60, -90}}, lineColor = {0,0,0}, textString = "Rk")}));
        model TemperatureSensor "Absolute temperature sensor in Rankine"
          annotation(Diagram(graphics = {Ellipse(extent = {{ -20, -98},{20, -60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -12,40},{12, -68}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Line(points = {{12,0},{90,0}}, color = {0,0,255}),Line(points = {{ -94,0},{ -12,0}}, color = {191,0,0}),Polygon(points = {{ -12,40},{ -12,80},{ -10,86},{ -6,88},{0,90},{6,88},{10,86},{12,80},{12,40},{ -12,40}}, lineColor = {0,0,0}, lineThickness = 0.5),Line(points = {{ -12,40},{ -12, -64}}, color = {0,0,0}, thickness = 0.5),Line(points = {{12,40},{12, -64}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -40, -20},{ -12, -20}}, color = {0,0,0}),Line(points = {{ -40,20},{ -12,20}}, color = {0,0,0}),Line(points = {{ -40,60},{ -12,60}}, color = {0,0,0}),Text(extent = {{102, -22},{60, -74}}, lineColor = {0,0,0}, textString = "Rk")}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -20, -98},{20, -60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -12,40},{12, -68}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Line(points = {{12,0},{90,0}}, color = {0,0,255}),Line(points = {{ -90,0},{ -12,0}}, color = {191,0,0}),Polygon(points = {{ -12,40},{ -12,80},{ -10,86},{ -6,88},{0,90},{6,88},{10,86},{12,80},{12,40},{ -12,40}}, lineColor = {0,0,0}, lineThickness = 0.5),Line(points = {{ -12,40},{ -12, -64}}, color = {0,0,0}, thickness = 0.5),Line(points = {{12,40},{12, -64}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -40, -20},{ -12, -20}}, color = {0,0,0}),Line(points = {{ -40,20},{ -12,20}}, color = {0,0,0}),Line(points = {{ -40,60},{ -12,60}}, color = {0,0,0}),Text(extent = {{126, -20},{26, -120}}, lineColor = {0,0,0}, textString = "Rk"),Text(extent = {{ -132,144},{108,84}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<HTML>
<p>
This is an ideal absolute temperature sensor which returns
the temperature of the connected port in Rankine as an output
signal.  The sensor itself has no thermal interaction with
whatever it is connected to.  Furthermore, no
thermocouple-like lags are associated with this
sensor model.
</p>
</HTML>
"));
          Modelica.Blocks.Interfaces.RealOutput T annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Interfaces.HeatPort_a port annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));

        equation 
          T = to_degRk(port.T);
          port.Q_flow = 0;
        end TemperatureSensor;
      end Rankine;
      package Interfaces "Connectors and partial models"
        extends Modelica.Icons.Library2;
        partial connector HeatPort "Thermal port for 1-dim. heat transfer"
          Modelica.SIunits.Temperature T "Port temperature";
          flow Modelica.SIunits.HeatFlowRate Q_flow "Heat flow rate (positive if flowing from outside into the component)";
          annotation(Documentation(info = "<html>

</html>"));
        end HeatPort;
        connector HeatPort_a "Thermal port for 1-dim. heat transfer (filled rectangular icon)"
          extends HeatPort;
          annotation(defaultComponentName = "port_a", Documentation(info = "<HTML>
<p>This connector is used for 1-dimensional heat flow between components.
The variables in the connector are:</p>
<pre>
   T       Temperature in [Kelvin].
   Q_flow  Heat flow rate in [Watt].
</pre>
<p>According to the Modelica sign convention, a <b>positive</b> heat flow
rate <b>Q_flow</b> is considered to flow <b>into</b> a component. This
convention has to be used whenever this connector is used in a model
class.</p>
<p>Note, that the two connector classes <b>HeatPort_a</b> and
<b>HeatPort_b</b> are identical with the only exception of the different
<b>icon layout</b>.</p></HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -50,50},{50, -50}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -120,120},{100,60}}, lineColor = {191,0,0}, textString = "%name")}));
        end HeatPort_a;
        connector HeatPort_b "Thermal port for 1-dim. heat transfer (unfilled rectangular icon)"
          extends HeatPort;
          annotation(defaultComponentName = "port_b", Documentation(info = "<HTML>
<p>This connector is used for 1-dimensional heat flow between components.
The variables in the connector are:</p>
<pre>
   T       Temperature in [Kelvin].
   Q_flow  Heat flow rate in [Watt].
</pre>
<p>According to the Modelica sign convention, a <b>positive</b> heat flow
rate <b>Q_flow</b> is considered to flow <b>into</b> a component. This
convention has to be used whenever this connector is used in a model
class.</p>
<p>Note, that the two connector classes <b>HeatPort_a</b> and
<b>HeatPort_b</b> are identical with the only exception of the different
<b>icon layout</b>.</p></HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -50,50},{50, -50}}, lineColor = {191,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -100,120},{120,60}}, lineColor = {191,0,0}, textString = "%name")}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {191,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}));
        end HeatPort_b;
        partial model Element1D "Partial heat transfer element with two HeatPort connectors that does not store energy"
          Modelica.SIunits.HeatFlowRate Q_flow "Heat flow rate from port_a -> port_b";
          Modelica.SIunits.TemperatureDifference dT "port_a.T - port_b.T";
        public 
          HeatPort_a port_a annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          HeatPort_b port_b annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          annotation(Documentation(info = "<HTML>
<p>
This partial model contains the basic connectors and variables to
allow heat transfer models to be created that <b>do not store energy</b>,
This model defines and includes equations for the temperature
drop across the element, <b>dT</b>, and the heat flow rate
through the element from port_a to port_b, <b>Q_flow</b>.
</p>
<p>
By extending this model, it is possible to write simple
constitutive equations for many types of heat transfer components.
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          dT = port_a.T - port_b.T;
          port_a.Q_flow = Q_flow;
          port_b.Q_flow =  -Q_flow;
        end Element1D;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -60,10},{40, -90}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>

</html>"));
      end Interfaces;
    end HeatTransfer;
    package FluidHeatFlow "Simple components for 1-dimensional incompressible thermo-fluid flow models"
      extends Modelica.Icons.Library2;
      annotation(Documentation(info = "<HTML>
This package contains very simple-to-use components to model coolant flows as needed to simulate cooling e.g. of electric machines:
<ul>
<li>Components: components like different types of pipe models</li>
<li>Examples: some test examples</li>
<li>Interfaces: definition of connectors and partial models
(containing the core thermodynamic equations)</li>
<li>Media: definition of media properties</li>
<li>Sensors: various sensors for pressure, temperature, volume and enthalpy flow</li>
<li>Sources: various flow sources</li>
</ul>
<b>Variables used in connectors:</b>
<ul>
<li>Pressure p</li>
<li>flow MassFlowRate m_flow</li>
<li>SpecificEnthalpy h</li>
<li>flow EnthalpyFlowRate H_flow</li>
</ul>
EnthalpyFlowRate means the Enthalpy = cp<sub>constant</sub> * m * T that is carried by the medium's flow.<br><br>
<b>Limitations and assumptions:</b>
<ul>
<li>Splitting and mixing of coolant flows (media with the same cp) is possible.</li>
<li>Reversing the direction of flow is possible.</li>
<li>The medium is considered to be incompressible.</li>
<li>No mixtures of media is taken into consideration.</li>
<li>The medium may not change its phase.</li>
<li>Medium properties are kept constant.</li>
<li>Pressure changes are only due to pressure drop and geodetic height differnence rho*g*h (if h > 0).</li>
<li>A user-defined part (0..1) of the friction losses (V_flow*dp) are fed to the medium.</li>
<li><b>Note:</b> Connected flowPorts have the same temperature (mixing temperature)!<br>
Since mixing may occur, the outlet temperature may be different from the connector's temperature.<br>
Outlet temperature is defined by variable T of the corresponding component.</li>
</ul>
<b>Further development:</b>
<ul>
<li>Additional components like tanks (if needed)</li>
</ul>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <p>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern, Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </p>
  <p>
  Dr.Christian Kral<br>
  &Ouml;sterreichisches Forschungs- und Pr&uuml;fzentrum Arsenal Ges.m.b.H.<br>
  <a href=\"http://www.arsenal.ac.at/\">arsenal research</a><br>
  Giefinggasse 2<br>
  A-1210 Vienna, Austria
  </p>
  </dd>
</dl>
<p>
Copyright &copy; 1998-2009, Modelica Association, Anton Haumer and arsenal research.
</p>
<p>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p>
</HTML>", revisions = "<HTML>
  <ul>
  <li> v1.00 2005/02/01 Anton Haumer<br>
       first stable official release</li>
  <li> v1.10 2005/02/15 Anton Haumer<br>
       reorganisation of the package</li>
  <li> v1.11 2005/02/18 Anton Haumer<br>
       corrected usage of cv and cp</li>
  <li> v1.20 Beta 2005/02/18 Anton Haumer<br>
       introduced geodetic height in Components.Pipes<br>
       <i>new models: Components.Valve, Sources.IdealPump</i></li>
  <li> v1.30 Beta 2005/06/02 Anton Haumer<br>
       friction losses are fed to medium</li>
  <li> v1.31 Beta 2005/06/04 Anton Haumer<br>
       <i>new example: PumpAndValve</i><br>
       <i>new example: PumpDropOut</i></li>
  <li> v1.33 Beta 2005/06/07 Anton Haumer<br>
       corrected usage of simpleFlow</li>
  <li> v1.40 2005/06/13 Anton Haumer<br>
       stable release</li>
  <li> v1.42 Beta 2005/06/18 Anton Haumer<br>
       <i>new test example: ParallelPumpDropOut</i></li>
  <li> v1.43 Beta 2005/06/20 Anton Haumer<br>
       Test of mixing / semiLinear<br>
       <i>new test example: OneMass</i><br>
       <i>new test example: TwoMass</i></li>
  <li> v1.50 2005/09/07 Anton Haumer<br>
       semiLinear works fine</li>
  <li> v1.60 2007/01/23 Anton Haumer<br>
       new parameter tapT defining Temperature of heatPort </li>
  <li> v1.6.1 2007/08/12 Anton Haumer<br>
       improved documentation<br>
       removed type TemperatureDifference since this is defined in Modelica.SIunits</li>
  <li> v1.6.2 2007/08/20 Anton Haumer<br>
       improved documentation</li>
  <li> v1.6.3 2007/08/21 Anton Haumer<br>
       improved documentation</li>
  <li> v1.6.4 2007/08/24 Anton Haumer<br>
       removed redeclare type SignalType</li>
  <li> v1.6.5 2007/08/26 Anton Haumer<br>
       fixed unit bug in SimpleFriction</li>
  <li> v1.6.6 2007/11/13 Anton Haumer<br>
       replaced all nonSIunits<br>
       some renaming to be more concise</li>
  </ul>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -80,10},{ -60, -10},{ -80, -30},{ -20, -30},{0, -10},{ -20,10},{ -80,10}}, lineColor = {0,128,255}, fillColor = {0,128,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -40, -90},{ -20, -70},{0, -90},{0, -50},{ -20, -30},{ -40, -50},{ -40, -90}}, lineColor = {255,0,0}, fillColor = {255,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{ -20,10},{0, -10},{ -20, -30},{40, -30},{60, -10},{40,10},{ -20,10}}, lineColor = {255,128,0}, fillColor = {255,128,0}, fillPattern = FillPattern.Solid)}));
      package Examples "Examples that demonstrate the usage of the FluidHeatFlow components"
        extends Modelica.Icons.Library2;
        annotation(Documentation(info = "<HTML>
This package contains test examples:
<ul>
<li>1.SimpleCooling: heat is dissipated through a media flow</li>
<li>2.ParallelCooling: two heat sources dissipate through merged media flows</li>
<li>3.IndirectCooling: heat is disspated through two cooling cycles</li>
<li>4.PumpAndValve: demonstrates usage of an IdealPump and a Valve</li>
<li>5.PumpDropOut: demonstrates shutdown and restart of a pump</li>
<li>6.ParallelPumpDropOut: demonstrates shutdown and restart of a pump in a parallel circuit</li>
<li>7.OneMass: cooling of a mass (thermal capacity) by a coolant flow</li>
<li>8.TwoMass: cooling of two masses (thermal capacities) by two parallel coolant flows</li>
</ul>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <p>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern, Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </p>
  <p>
  Dr.Christian Kral<br>
  &Ouml;sterreichisches Forschungs- und Pr&uuml;fzentrum Arsenal Ges.m.b.H.<br>
  <a href=\"http://www.arsenal.ac.at/\">arsenal research</a><br>
  Giefinggasse 2<br>
  A-1210 Vienna, Austria
  </p>
  </dd>
</dl>
<p>
Copyright &copy; 1998-2009, Modelica Association, Anton Haumer and arsenal research.
</p>
<p>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p>
</HTML>", revisions = "<HTML>
  <ul>
  <li> v1.00 2005/02/01 Anton Haumer<br>
       first stable official release</li>
  <li> v1.31 Beta 2005/06/04 Anton Haumer<br>
       <i>new example: PumpAndValve</i><br>
       <i>new example: PumpDropOut</i></li>
  <li> v1.42 Beta 2005/06/18 Anton Haumer<br>
       <i>new test example: ParallelPumpDropOut</i></li>
  <li> v1.43 Beta 2005/06/20 Anton Haumer<br>
       Test of mixing / semiLinear<br>
       <i>new test example: OneMass</i><br>
       <i>new test example: TwoMass</i></li>
  </ul>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -60,10},{40, -90}}, lineColor = {135,135,135}),Polygon(points = {{ -30, -12},{ -30, -68},{28, -40},{ -30, -12}}, lineColor = {135,135,135}, fillColor = {135,135,135}, fillPattern = FillPattern.Solid)}));
        model SimpleCooling "Example: simple cooling circuit"
          extends Modelica.Icons.Example;
          annotation(Documentation(info = "<HTML>
<p>
1st test example: SimpleCooling
</p>
A prescribed heat source dissipates its heat through a thermal conductor to a coolant flow. The coolant flow is taken from an ambient and driven by a pump with prescribed mass flow.<br>
<b>Results</b>:<br>
<table>
<tr>
<td valign=\"top\"><b>output</b></td>
<td valign=\"top\"><b>explanation</b></td>
<td valign=\"top\"><b>formula</b></td>
<td valign=\"top\"><b>actual steady-state value</b></td>
</tr>
<tr>
<td valign=\"top\">dTSource</td>
<td valign=\"top\">Source over Ambient</td>
<td valign=\"top\">dtCoolant + dtToPipe</td>
<td valign=\"top\">20 K</td>
</tr>
<tr>
<td valign=\"top\">dTtoPipe</td>
<td valign=\"top\">Source over Coolant</td>
<td valign=\"top\">Losses / ThermalConductor.G</td>
<td valign=\"top\">10 K</td>
</tr>
<tr>
<td valign=\"top\">dTCoolant</td>
<td valign=\"top\">Coolant's temperature increase</td>
<td valign=\"top\">Losses * cp * massFlow</td>
<td valign=\"top\">10 K</td>
</tr>
</table>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1.0));
          parameter FluidHeatFlow.Media.Medium medium = FluidHeatFlow.Media.Medium() "Cooling medium" annotation(__Dymola_choicesAllMatching = true);
          parameter Modelica.SIunits.Temperature TAmb(displayUnit = "degC") = 293.15 "Ambient temperature";
          output Modelica.SIunits.TemperatureDifference dTSource = prescribedHeatFlow.port.T - TAmb "Source over Ambient";
          output Modelica.SIunits.TemperatureDifference dTtoPipe = prescribedHeatFlow.port.T - pipe.heatPort.T "Source over Coolant";
          output Modelica.SIunits.TemperatureDifference dTCoolant = pipe.dT "Coolant's temperature increase";
          FluidHeatFlow.Sources.Ambient ambient1(constantAmbientTemperature = TAmb, medium = medium) annotation(Placement(transformation(extent = {{ -60, -10},{ -80,10}}, rotation = 0)));
          Sources.VolumeFlow pump(medium = medium, m = 0, T0 = TAmb, useVolumeFlowInput = true) annotation(Placement(transformation(extent = {{ -40, -10},{ -20,10}}, rotation = 0)));
          FluidHeatFlow.Components.HeatedPipe pipe(medium = medium, m = 0.1, T0 = TAmb) annotation(Placement(transformation(extent = {{0, -10},{20,10}}, rotation = 0)));
          FluidHeatFlow.Sources.Ambient ambient2(constantAmbientTemperature = TAmb, medium = medium) annotation(Placement(transformation(extent = {{40, -10},{60,10}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor(T(start = TAmb), C = 0.1) annotation(Placement(transformation(origin = {40, -50}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow prescribedHeatFlow annotation(Placement(transformation(extent = {{ -30, -40},{ -10, -60}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant volumeFlow(k = 1) annotation(Placement(transformation(extent = {{ -60,10},{ -40,30}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant heatFlow(k = 10) annotation(Placement(transformation(extent = {{ -60, -60},{ -40, -40}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.Convection convection annotation(Placement(transformation(origin = {10, -30}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Blocks.Sources.Constant thermalConductance(k = 1) annotation(Placement(transformation(extent = {{ -30, -40},{ -10, -20}}, rotation = 0)));

        equation 
          connect(ambient1.flowPort,pump.flowPort_a) annotation(Line(points = {{ -60,0},{ -40,0}}, color = {255,0,0}));
          connect(pump.flowPort_b,pipe.flowPort_a) annotation(Line(points = {{ -20,0},{0,0}}, color = {255,0,0}));
          connect(pipe.flowPort_b,ambient2.flowPort) annotation(Line(points = {{20,0},{40,0}}, color = {255,0,0}));
          connect(heatFlow.y,prescribedHeatFlow.Q_flow) annotation(Line(points = {{ -39, -50},{ -30, -50}}, color = {0,0,255}));
          connect(convection.solid,prescribedHeatFlow.port) annotation(Line(points = {{10, -40},{10, -50},{ -10, -50}}, color = {191,0,0}));
          connect(convection.solid,heatCapacitor.port) annotation(Line(points = {{10, -40},{10, -50},{30, -50}}, color = {191,0,0}));
          connect(pipe.heatPort,convection.fluid) annotation(Line(points = {{10, -10},{10, -20}}, color = {191,0,0}));
          connect(thermalConductance.y,convection.Gc) annotation(Line(points = {{ -9, -30},{0, -30}}, color = {0,0,127}));
          connect(volumeFlow.y,pump.volumeFlow) annotation(Line(points = {{ -39,20},{ -30,20},{ -30,10}}, color = {0,0,127}, smooth = Smooth.None));
        end SimpleCooling;
        model ParallelCooling "Example: coolig circuit with parallel branches"
          extends Modelica.Icons.Example;
          annotation(Documentation(info = "<HTML>
<p>
2nd test example: ParallelCooling
</p>
Two prescribed heat sources dissipate their heat through thermal conductors to coolant flows. The coolant flow is taken from an ambient and driven by a pump with prescribed mass flow, then splitted into two coolant flows connected to the two heat sources, and afterwards merged. Splitting of coolant flows is determined by pressure drop characteristic of the two pipes.<br>
<b>Results</b>:<br>
<table>
<tr>
<td valign=\"top\"><b>output</b></td>
<td valign=\"top\"><b>explanation</b></td>
<td valign=\"top\"><b>formula</b></td>
<td valign=\"top\"><b>actual steady-state value</b></td>
</tr>
<tr>
<td valign=\"top\">dTSource1</td>
<td valign=\"top\">Source1 over Ambient</td>
<td valign=\"top\">dTCoolant1 + dTtoPipe1</td>
<td valign=\"top\">15 K</td>
</tr>
<tr>
<td valign=\"top\">dTtoPipe1</td>
<td valign=\"top\">Source1 over Coolant1</td>
<td valign=\"top\">Losses1 / ThermalConductor1.G</td>
<td valign=\"top\"> 5 K</td>
</tr>
<tr>
<td valign=\"top\">dTCoolant1</td>
<td valign=\"top\">Coolant's temperature increase</td>
<td valign=\"top\">Losses * cp * totalMassFlow/2</td>
<td valign=\"top\">10 K</td>
</tr>
<tr>
<td valign=\"top\">dTSource2</td>
<td valign=\"top\">Source2 over Ambient</td>
<td valign=\"top\">dTCoolant2 + dTtoPipe2</td>
<td valign=\"top\">30 K</td>
</tr>
<tr>
<td valign=\"top\">dTtoPipe2</td>
<td valign=\"top\">Source2 over Coolant2</td>
<td valign=\"top\">Losses2 / ThermalConductor2.G</td>
<td valign=\"top\">10 K</td>
</tr>
<tr>
<td valign=\"top\">dTCoolant2</td>
<td valign=\"top\">Coolant's temperature increase</td>
<td valign=\"top\">Losses * cp * totalMassFlow/2</td>
<td valign=\"top\">20 K</td>
</tr>
<tr>
<td valign=\"top\">dTmixedCoolant</td>
<td valign=\"top\">mixed Coolant's temperature increase</td>
<td valign=\"top\">(dTCoolant1+dTCoolant2)/2</td>
<td valign=\"top\">15 K</td>
</tr>
</table>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1.0));
          parameter FluidHeatFlow.Media.Medium medium = FluidHeatFlow.Media.Medium() "Cooling medium" annotation(__Dymola_choicesAllMatching = true);
          parameter Modelica.SIunits.Temperature TAmb(displayUnit = "degC") = 293.15 "Ambient temperature";
          output Modelica.SIunits.TemperatureDifference dTSource1 = prescribedHeatFlow1.port.T - TAmb "Source1 over Ambient";
          output Modelica.SIunits.TemperatureDifference dTtoPipe1 = prescribedHeatFlow1.port.T - pipe1.heatPort.T "Source1 over Coolant1";
          output Modelica.SIunits.TemperatureDifference dTCoolant1 = pipe1.dT "Coolant1's temperature increase";
          output Modelica.SIunits.TemperatureDifference dTSource2 = prescribedHeatFlow2.port.T - TAmb "Source2 over Ambient";
          output Modelica.SIunits.TemperatureDifference dTtoPipe2 = prescribedHeatFlow2.port.T - pipe2.heatPort.T "Source2 over Coolant2";
          output Modelica.SIunits.TemperatureDifference dTCoolant2 = pipe2.dT "Coolant2's temperature increase";
          output Modelica.SIunits.TemperatureDifference dTmixedCoolant = ambient2.T_port - ambient1.T_port "mixed Coolant's temperature increase";
          FluidHeatFlow.Sources.Ambient ambient1(constantAmbientTemperature = TAmb, medium = medium) annotation(Placement(transformation(extent = {{ -60, -10},{ -80,10}}, rotation = 0)));
          Sources.VolumeFlow pump(medium = medium, m = 0, T0 = TAmb, useVolumeFlowInput = true) annotation(Placement(transformation(extent = {{ -40, -10},{ -20,10}}, rotation = 0)));
          FluidHeatFlow.Components.HeatedPipe pipe1(medium = medium, m = 0.1, T0 = TAmb) annotation(Placement(transformation(extent = {{0, -20},{20,0}}, rotation = 0)));
          FluidHeatFlow.Components.HeatedPipe pipe2(medium = medium, m = 0.1, T0 = TAmb) annotation(Placement(transformation(extent = {{0,20},{20,0}}, rotation = 0)));
          FluidHeatFlow.Components.IsolatedPipe pipe3(medium = medium, m = 0.1, T0 = TAmb) annotation(Placement(transformation(extent = {{40, -10},{60,10}}, rotation = 0)));
          FluidHeatFlow.Sources.Ambient ambient2(constantAmbientTemperature = TAmb, medium = medium) annotation(Placement(transformation(extent = {{80, -10},{100,10}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor1(T(start = TAmb), C = 0.1) annotation(Placement(transformation(origin = {40, -60}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow prescribedHeatFlow1 annotation(Placement(transformation(origin = { -20, -60}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
          Modelica.Thermal.HeatTransfer.Components.Convection convection1 annotation(Placement(transformation(origin = {10, -40}, extent = {{10,10},{ -10, -10}}, rotation = 270)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor2(T(start = TAmb), C = 0.1) annotation(Placement(transformation(origin = {38,60}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow prescribedHeatFlow2 annotation(Placement(transformation(origin = { -20,60}, extent = {{10,10},{ -10, -10}}, rotation = 180)));
          Modelica.Thermal.HeatTransfer.Components.Convection convection2 annotation(Placement(transformation(origin = {10,40}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Modelica.Blocks.Sources.Constant volumeFlow(k = 1) annotation(Placement(transformation(extent = {{ -60,10},{ -40,30}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant heatFlow1(k = 5) annotation(Placement(transformation(extent = {{ -60, -70},{ -40, -50}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant heatFlow2(k = 10) annotation(Placement(transformation(extent = {{ -60,50},{ -40,70}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant thermalConductance1(k = 1) annotation(Placement(transformation(extent = {{ -30, -50},{ -10, -30}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant thermalConductance2(k = 1) annotation(Placement(transformation(extent = {{ -30,30},{ -10,50}}, rotation = 0)));

        equation 
          connect(ambient1.flowPort,pump.flowPort_a) annotation(Line(points = {{ -60,0},{ -40,0}}, color = {255,0,0}));
          connect(pump.flowPort_b,pipe1.flowPort_a) annotation(Line(points = {{ -20,0},{ -10,0},{ -10, -10},{0, -10}}, color = {255,0,0}));
          connect(pump.flowPort_b,pipe2.flowPort_a) annotation(Line(points = {{ -20,0},{ -10,0},{ -10,10},{0,10}}, color = {255,0,0}));
          connect(heatFlow2.y,prescribedHeatFlow2.Q_flow) annotation(Line(points = {{ -39,60},{ -30,60}}, color = {0,0,255}));
          connect(heatFlow1.y,prescribedHeatFlow1.Q_flow) annotation(Line(points = {{ -39, -60},{ -30, -60}}, color = {0,0,255}));
          connect(thermalConductance2.y,convection2.Gc) annotation(Line(points = {{ -9,40},{0,40}}, color = {0,0,127}));
          connect(thermalConductance1.y,convection1.Gc) annotation(Line(points = {{ -9, -40},{0, -40}}, color = {0,0,127}));
          connect(pipe1.heatPort,convection1.fluid) annotation(Line(points = {{10, -20},{10, -30},{10, -30}}, color = {191,0,0}));
          connect(convection2.fluid,pipe2.heatPort) annotation(Line(points = {{10,30},{10,20}}, color = {191,0,0}));
          connect(convection2.solid,prescribedHeatFlow2.port) annotation(Line(points = {{10,50},{10,60},{ -10,60}}, color = {191,0,0}));
          connect(convection2.solid,heatCapacitor2.port) annotation(Line(points = {{10,50},{10,60},{48,60}}, color = {191,0,0}));
          connect(convection1.solid,prescribedHeatFlow1.port) annotation(Line(points = {{10, -50},{10, -60},{ -10, -60}}, color = {191,0,0}));
          connect(convection1.solid,heatCapacitor1.port) annotation(Line(points = {{10, -50},{10, -60},{30, -60}}, color = {191,0,0}));
          connect(pipe2.flowPort_b,pipe3.flowPort_a) annotation(Line(points = {{20,10},{30,10},{30,0},{40,0}}, color = {255,0,0}));
          connect(pipe1.flowPort_b,pipe3.flowPort_a) annotation(Line(points = {{20, -10},{30, -10},{30,0},{40,0}}, color = {255,0,0}));
          connect(pipe3.flowPort_b,ambient2.flowPort) annotation(Line(points = {{60,0},{80,0}}, color = {255,0,0}));
          connect(volumeFlow.y,pump.volumeFlow) annotation(Line(points = {{ -39,20},{ -30,20},{ -30,10}}, color = {0,0,127}, smooth = Smooth.None));
        end ParallelCooling;
        model IndirectCooling "Example: indirect cooling circuit"
          extends Modelica.Icons.Example;
          annotation(Documentation(info = "<HTML>
<p>
3rd test example: IndirectCooling
</p>
A prescribed heat sources dissipates its heat through a thermal conductor to the inner coolant cycle. It is necessary to define the pressure level of the inner coolant cycle. The inner coolant cycle is coupled to the outer coolant flow through a thermal conductor.<br>
Inner coolant's temperature rise near the source is the same as temperature drop near the cooler.<br>
<b>Results</b>:<br>
<table>
<tr>
<td valign=\"top\"><b>output</b></td>
<td valign=\"top\"><b>explanation</b></td>
<td valign=\"top\"><b>formula</b></td>
<td valign=\"top\"><b>actual steady-state value</b></td>
</tr>
<tr>
<td valign=\"top\">dTSource</td>
<td valign=\"top\">Source over Ambient</td>
<td valign=\"top\">dtouterCoolant + dtCooler + dTinnerCoolant + dtToPipe</td>
<td valign=\"top\">40 K</td>
</tr>
<tr>
<td valign=\"top\">dTtoPipe</td>
<td valign=\"top\">Source over inner Coolant</td>
<td valign=\"top\">Losses / ThermalConductor.G</td>
<td valign=\"top\">10 K</td>
</tr>
<tr>
<td valign=\"top\">dTinnerColant</td>
<td valign=\"top\">inner Coolant's temperature increase</td>
<td valign=\"top\">Losses * cp * innerMassFlow</td>
<td valign=\"top\">10 K</td>
</tr>
<tr>
<td valign=\"top\">dTCooler</td>
<td valign=\"top\">Cooler's temperature rise between inner and outer pipes</td>
<td valign=\"top\">Losses * (innerGc + outerGc)</td>
<td valign=\"top\">10 K</td>
</tr>
<tr>
<td valign=\"top\">dTouterColant</td>
<td valign=\"top\">outer Coolant's temperature increase</td>
<td valign=\"top\">Losses * cp * outerMassFlow</td>
<td valign=\"top\">10 K</td>
</tr>
</table>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1.5));
          parameter FluidHeatFlow.Media.Medium outerMedium = FluidHeatFlow.Media.Medium() "Outer medium" annotation(__Dymola_choicesAllMatching = true);
          parameter FluidHeatFlow.Media.Medium innerMedium = FluidHeatFlow.Media.Medium() "Inner medium" annotation(__Dymola_choicesAllMatching = true);
          parameter Modelica.SIunits.Temperature TAmb(displayUnit = "degC") = 293.15 "Ambient temperature";
          output Modelica.SIunits.TemperatureDifference dTSource = prescribedHeatFlow.port.T - TAmb "Source over Ambient";
          output Modelica.SIunits.TemperatureDifference dTtoPipe = prescribedHeatFlow.port.T - pipe1.heatPort.T "Source over inner Coolant";
          output Modelica.SIunits.TemperatureDifference dTinnerCoolant = pipe1.dT "inner Coolant's temperature increase";
          output Modelica.SIunits.TemperatureDifference dTCooler = innerPipe.heatPort.T - outerPipe.heatPort.T "Cooler's temperature increase between inner and outer pipes";
          output Modelica.SIunits.TemperatureDifference dTouterCoolant = outerPipe.dT "outer Coolant's temperature increase";
          FluidHeatFlow.Sources.Ambient ambient1(constantAmbientTemperature = TAmb, medium = outerMedium) annotation(Placement(transformation(extent = {{ -60,60},{ -80,80}}, rotation = 0)));
          Sources.VolumeFlow outerPump(medium = outerMedium, m = 0, T0 = TAmb, useVolumeFlowInput = true) annotation(Placement(transformation(extent = {{ -40,60},{ -20,80}}, rotation = 0)));
          FluidHeatFlow.Sources.Ambient ambient2(constantAmbientTemperature = TAmb, medium = outerMedium) annotation(Placement(transformation(extent = {{40,60},{60,80}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.ThermalConductor thermalConductor(G = 1) annotation(Placement(transformation(origin = {10, -70}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor(T(start = TAmb), C = 0.05) annotation(Placement(transformation(origin = {40, -90}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow prescribedHeatFlow annotation(Placement(transformation(origin = { -20, -90}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
          FluidHeatFlow.Components.HeatedPipe pipe1(medium = innerMedium, m = 0.1, T0 = TAmb, V_flowLaminar = 1, dpLaminar = 1000, V_flowNominal = 2, dpNominal = 2000) annotation(Placement(transformation(extent = {{20, -50},{0, -30}}, rotation = 0)));
          FluidHeatFlow.Sources.AbsolutePressure absolutePressure(p = 10000, medium = innerMedium) annotation(Placement(transformation(extent = {{40, -40},{60, -20}}, rotation = 0)));
          Sources.VolumeFlow innerPump(medium = innerMedium, m = 0, T0 = TAmb, useVolumeFlowInput = true) annotation(Placement(transformation(origin = { -20, -30}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Blocks.Sources.Constant heatFlow(k = 10) annotation(Placement(transformation(extent = {{ -60, -100},{ -40, -80}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant outerVolumeFlow(k = 1) annotation(Placement(transformation(extent = {{ -60,80},{ -40,100}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant innerVolumeFlow(k = 1) annotation(Placement(transformation(extent = {{ -60, -40},{ -40, -20}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant outerGc(k = 2) annotation(Placement(transformation(extent = {{ -40,30},{ -20,50}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant innerGc(k = 2) annotation(Placement(transformation(extent = {{ -40,20},{ -20,0}}, rotation = 0)));
          FluidHeatFlow.Components.HeatedPipe outerPipe(medium = outerMedium, m = 0.1, T0 = TAmb) annotation(Placement(transformation(extent = {{0,60},{20,80}}, rotation = 0)));
          FluidHeatFlow.Components.HeatedPipe innerPipe(medium = innerMedium, m = 0.1, T0 = TAmb) annotation(Placement(transformation(extent = {{0, -10},{20, -30}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.Convection innerConvection annotation(Placement(transformation(origin = {10,10}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Modelica.Thermal.HeatTransfer.Components.Convection outerConvection annotation(Placement(transformation(origin = {10,40}, extent = {{10,10},{ -10, -10}}, rotation = 270)));

        equation 
          connect(ambient1.flowPort,outerPump.flowPort_a) annotation(Line(points = {{ -60,70},{ -40,70}}, color = {255,0,0}));
          connect(prescribedHeatFlow.port,thermalConductor.port_a) annotation(Line(points = {{ -10, -90},{10, -90},{10, -80}}, color = {191,0,0}));
          connect(heatCapacitor.port,thermalConductor.port_a) annotation(Line(points = {{30, -90},{10, -90},{10, -80}}, color = {191,0,0}));
          connect(pipe1.heatPort,thermalConductor.port_b) annotation(Line(points = {{10, -50},{10, -60}}, color = {191,0,0}));
          connect(pipe1.flowPort_b,innerPump.flowPort_a) annotation(Line(points = {{0, -40},{ -20, -40}}, color = {255,0,0}));
          connect(absolutePressure.flowPort,pipe1.flowPort_a) annotation(Line(points = {{40, -30},{40, -40},{20, -40}}, color = {255,0,0}));
          connect(heatFlow.y,prescribedHeatFlow.Q_flow) annotation(Line(points = {{ -39, -90},{ -30, -90}}, color = {0,0,255}));
          connect(innerPump.flowPort_b,innerPipe.flowPort_a) annotation(Line(points = {{ -20, -20},{0, -20}}, color = {255,0,0}));
          connect(innerPipe.flowPort_b,absolutePressure.flowPort) annotation(Line(points = {{20, -20},{40, -20},{40, -30}}, color = {255,0,0}));
          connect(outerPump.flowPort_b,outerPipe.flowPort_a) annotation(Line(points = {{ -20,70},{0,70}}, color = {255,0,0}));
          connect(outerPipe.flowPort_b,ambient2.flowPort) annotation(Line(points = {{20,70},{40,70}}, color = {255,0,0}));
          connect(outerPipe.heatPort,outerConvection.fluid) annotation(Line(points = {{10,60},{10,55},{10,50},{10,50}}, color = {191,0,0}));
          connect(outerConvection.solid,innerConvection.solid) annotation(Line(points = {{10,30},{10,20},{10,20}}, color = {191,0,0}));
          connect(innerConvection.fluid,innerPipe.heatPort) annotation(Line(points = {{10,0},{10, -5},{10, -5},{10, -10}}, color = {191,0,0}));
          connect(innerGc.y,innerConvection.Gc) annotation(Line(points = {{ -19,10},{ -9.5,10},{ -9.5,10},{0,10}}, color = {0,0,127}));
          connect(outerGc.y,outerConvection.Gc) annotation(Line(points = {{ -19,40},{0,40}}, color = {0,0,127}));
          connect(outerVolumeFlow.y,outerPump.volumeFlow) annotation(Line(points = {{ -39,90},{ -30,90},{ -30,80}}, color = {0,0,127}, smooth = Smooth.None));
          connect(innerVolumeFlow.y,innerPump.volumeFlow) annotation(Line(points = {{ -39, -30},{ -30, -30}}, color = {0,0,127}, smooth = Smooth.None));
        end IndirectCooling;
        model PumpAndValve "Example: cooling circuit with pump and valve"
          extends Modelica.Icons.Example;
          annotation(Documentation(info = "<HTML>
<p>
4th test example: PumpAndValve
</p>
The pump is running with half speed for 0.4 s,
afterwards with full speed (using a ramp of 0.1 s).<br>
The valve is half open for 0.9 s, afterwards full open (using a ramp of 0.1 s).<br>
You may try to:
<ul>
<li>drive the pump with variable speed and let the valve full open
    to regulate the volume flow rate of coolant</li>
<li>drive the pump with constant speed and throttle the valve
    to regulate the volume flow rate of coolant</li>
</ul>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 2));
          parameter FluidHeatFlow.Media.Medium medium = FluidHeatFlow.Media.Medium() "Cooling medium" annotation(__Dymola_choicesAllMatching = true);
          parameter Modelica.SIunits.Temperature TAmb(displayUnit = "degC") = 293.15 "Ambient temperature";
          output Modelica.SIunits.TemperatureDifference dTSource = prescribedHeatFlow.port.T - TAmb "Source over Ambient";
          output Modelica.SIunits.TemperatureDifference dTtoPipe = prescribedHeatFlow.port.T - pipe.heatPort.T "Source over Coolant";
          output Modelica.SIunits.TemperatureDifference dTCoolant = pipe.dT "Coolant's temperature increase";
          FluidHeatFlow.Sources.Ambient ambient1(constantAmbientTemperature = TAmb, medium = medium) annotation(Placement(transformation(extent = {{ -70, -10},{ -90,10}}, rotation = 0)));
          FluidHeatFlow.Sources.IdealPump idealPump(medium = medium, m = 0, T0 = TAmb) annotation(Placement(transformation(extent = {{ -60,10},{ -40, -10}}, rotation = 0)));
          FluidHeatFlow.Components.Valve valve(medium = medium, T0 = TAmb, LinearCharacteristic = false) annotation(Placement(transformation(extent = {{ -30, -10},{ -10,10}}, rotation = 0)));
          FluidHeatFlow.Components.HeatedPipe pipe(medium = medium, T0 = TAmb, m = 0.1) annotation(Placement(transformation(extent = {{0, -10},{20,10}}, rotation = 0)));
          FluidHeatFlow.Sources.Ambient ambient2(constantAmbientTemperature = TAmb, medium = medium) annotation(Placement(transformation(extent = {{40, -10},{60,10}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor(T(start = TAmb), C = 0.1) annotation(Placement(transformation(origin = {40, -50}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow prescribedHeatFlow annotation(Placement(transformation(extent = {{ -30, -40},{ -10, -60}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant heatFlow(k = 10) annotation(Placement(transformation(extent = {{ -60, -60},{ -40, -40}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.Convection convection annotation(Placement(transformation(origin = {10, -30}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Blocks.Sources.Constant thermalConductance(k = 1) annotation(Placement(transformation(extent = {{ -30, -40},{ -10, -20}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Sources.Speed speed(exact = true, useSupport = false) annotation(Placement(transformation(origin = { -50,30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Blocks.Sources.Ramp speedRamp(height = 0.5, offset = 0.5, duration = 0.1, startTime = 0.4) annotation(Placement(transformation(extent = {{ -80,40},{ -60,60}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp valveRamp(height = 0.5, offset = 0.5, duration = 0.1, startTime = 0.9) annotation(Placement(transformation(extent = {{12,40},{ -8,60}}, rotation = 0)));

        equation 
          connect(pipe.flowPort_b,ambient2.flowPort) annotation(Line(points = {{20,0},{40,0}}, color = {255,0,0}));
          connect(heatFlow.y,prescribedHeatFlow.Q_flow) annotation(Line(points = {{ -39, -50},{ -30, -50}}, color = {0,0,255}));
          connect(convection.solid,prescribedHeatFlow.port) annotation(Line(points = {{10, -40},{10, -50},{ -10, -50}}, color = {191,0,0}));
          connect(convection.solid,heatCapacitor.port) annotation(Line(points = {{10, -40},{10, -50},{30, -50}}, color = {191,0,0}));
          connect(pipe.heatPort,convection.fluid) annotation(Line(points = {{10, -10},{10, -20}}, color = {191,0,0}));
          connect(thermalConductance.y,convection.Gc) annotation(Line(points = {{ -9, -30},{0, -30}}, color = {0,0,127}));
          connect(ambient1.flowPort,idealPump.flowPort_a) annotation(Line(points = {{ -70,0},{ -60,0}}, color = {255,0,0}));
          connect(idealPump.flowPort_b,valve.flowPort_a) annotation(Line(points = {{ -40,0},{ -30,0}}, color = {255,0,0}));
          connect(valve.flowPort_b,pipe.flowPort_a) annotation(Line(points = {{ -10,0},{0,0}}, color = {255,0,0}));
          connect(speedRamp.y,speed.w_ref) annotation(Line(points = {{ -59,50},{ -50,50},{ -50,42}}, color = {0,0,127}));
          connect(valveRamp.y,valve.y) annotation(Line(points = {{ -9,50},{ -20,50},{ -20,9}}, color = {0,0,127}));
          connect(speed.flange,idealPump.flange_a) annotation(Line(points = {{ -50,20},{ -50,10}}, color = {0,0,0}, smooth = Smooth.None));
        end PumpAndValve;
        model PumpDropOut "Example: cooling circuit with drop out of pump"
          extends Modelica.Icons.Example;
          annotation(Documentation(info = "<HTML>
<p>
5th test example: PumpDropOut
</p>
Same as 1st test example, but with a drop out of the pump:<br>
The pump is running for 0.2 s, then shut down (using a ramp of 0.2 s) for 0.2 s,
then started again (using a ramp of 0.2 s).
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 2));
          parameter FluidHeatFlow.Media.Medium medium = FluidHeatFlow.Media.Medium() "Cooling medium" annotation(__Dymola_choicesAllMatching = true);
          parameter Modelica.SIunits.Temperature TAmb(displayUnit = "degC") = 293.15 "Ambient temperature";
          output Modelica.SIunits.TemperatureDifference dTSource = prescribedHeatFlow.port.T - TAmb "Source over Ambient";
          output Modelica.SIunits.TemperatureDifference dTtoPipe = prescribedHeatFlow.port.T - pipe.heatPort.T "Source over Coolant";
          output Modelica.SIunits.TemperatureDifference dTCoolant = pipe.dT "Coolant's temperature increase";
          FluidHeatFlow.Sources.Ambient ambient1(constantAmbientTemperature = TAmb, medium = medium) annotation(Placement(transformation(extent = {{ -60, -10},{ -80,10}}, rotation = 0)));
          Sources.VolumeFlow pump(medium = medium, m = 0, T0 = TAmb, useVolumeFlowInput = true) annotation(Placement(transformation(extent = {{ -40, -10},{ -20,10}}, rotation = 0)));
          FluidHeatFlow.Components.HeatedPipe pipe(medium = medium, T0 = TAmb, m = 0.1) annotation(Placement(transformation(extent = {{0, -10},{20,10}}, rotation = 0)));
          FluidHeatFlow.Sources.Ambient ambient2(constantAmbientTemperature = TAmb, medium = medium) annotation(Placement(transformation(extent = {{40, -10},{60,10}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor(T(start = TAmb), C = 0.1) annotation(Placement(transformation(origin = {40, -50}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow prescribedHeatFlow annotation(Placement(transformation(extent = {{ -30, -40},{ -10, -60}}, rotation = 0)));
          Utilities.DoubleRamp volumeFlow annotation(Placement(transformation(extent = {{ -60,10},{ -40,30}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant heatFlow(k = 10) annotation(Placement(transformation(extent = {{ -60, -60},{ -40, -40}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.Convection convection annotation(Placement(transformation(origin = {10, -30}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Blocks.Sources.Constant thermalConductance(k = 1) annotation(Placement(transformation(extent = {{ -30, -40},{ -10, -20}}, rotation = 0)));

        equation 
          connect(ambient1.flowPort,pump.flowPort_a) annotation(Line(points = {{ -60,0},{ -40,0}}, color = {255,0,0}));
          connect(pump.flowPort_b,pipe.flowPort_a) annotation(Line(points = {{ -20,0},{0,0}}, color = {255,0,0}));
          connect(pipe.flowPort_b,ambient2.flowPort) annotation(Line(points = {{20,0},{40,0}}, color = {255,0,0}));
          connect(heatFlow.y,prescribedHeatFlow.Q_flow) annotation(Line(points = {{ -39, -50},{ -30, -50}}, color = {0,0,255}));
          connect(convection.solid,prescribedHeatFlow.port) annotation(Line(points = {{10, -40},{10, -50},{ -10, -50}}, color = {191,0,0}));
          connect(convection.solid,heatCapacitor.port) annotation(Line(points = {{10, -40},{10, -50},{30, -50}}, color = {191,0,0}));
          connect(pipe.heatPort,convection.fluid) annotation(Line(points = {{10, -10},{10, -20}}, color = {191,0,0}));
          connect(thermalConductance.y,convection.Gc) annotation(Line(points = {{ -9, -30},{0, -30}}, color = {0,0,127}));
          connect(volumeFlow.y,pump.volumeFlow) annotation(Line(points = {{ -39,20},{ -30,20},{ -30,10}}, color = {0,0,127}, smooth = Smooth.None));
        end PumpDropOut;
        model ParallelPumpDropOut "Example: cooling circuit with parallel branches and drop out of pump"
          extends Modelica.Icons.Example;
          annotation(Documentation(info = "<HTML>
<p>
6th test example: ParallelPumpDropOut
</p>
Same as 2nd test example, but with a drop out of the pump:<br>
The pump is running for 0.2 s, then shut down (using a ramp of 0.2 s) for 0.2 s,
then started again (using a ramp of 0.2 s).
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 2));
          parameter FluidHeatFlow.Media.Medium medium = FluidHeatFlow.Media.Medium() "Cooling medium" annotation(__Dymola_choicesAllMatching = true);
          parameter Modelica.SIunits.Temperature TAmb(displayUnit = "degC") = 293.15 "Ambient temperature";
          output Modelica.SIunits.TemperatureDifference dTSource1 = prescribedHeatFlow1.port.T - TAmb "Source1 over Ambient";
          output Modelica.SIunits.TemperatureDifference dTtoPipe1 = prescribedHeatFlow1.port.T - pipe1.heatPort.T "Source1 over Coolant1";
          output Modelica.SIunits.TemperatureDifference dTCoolant1 = pipe1.dT "Coolant1's temperature increase";
          output Modelica.SIunits.TemperatureDifference dTSource2 = prescribedHeatFlow2.port.T - TAmb "Source2 over Ambient";
          output Modelica.SIunits.TemperatureDifference dTtoPipe2 = prescribedHeatFlow2.port.T - pipe2.heatPort.T "Source2 over Coolant2";
          output Modelica.SIunits.TemperatureDifference dTCoolant2 = pipe2.dT "Coolant2's temperature increase";
          output Modelica.SIunits.TemperatureDifference dTmixedCoolant = ambient2.T_port - ambient1.T_port "mixed Coolant's temperature increase";
          FluidHeatFlow.Sources.Ambient ambient1(constantAmbientTemperature = TAmb, medium = medium) annotation(Placement(transformation(extent = {{ -60, -10},{ -80,10}}, rotation = 0)));
          Sources.VolumeFlow pump(medium = medium, m = 0, T0 = TAmb, useVolumeFlowInput = true) annotation(Placement(transformation(extent = {{ -40, -10},{ -20,10}}, rotation = 0)));
          FluidHeatFlow.Components.HeatedPipe pipe1(medium = medium, m = 0.1, T0 = TAmb) annotation(Placement(transformation(extent = {{0, -20},{20,0}}, rotation = 0)));
          FluidHeatFlow.Components.HeatedPipe pipe2(medium = medium, m = 0.1, T0 = TAmb) annotation(Placement(transformation(extent = {{0,20},{20,0}}, rotation = 0)));
          FluidHeatFlow.Components.IsolatedPipe pipe3(medium = medium, T0 = TAmb, m = 0.1) annotation(Placement(transformation(extent = {{40, -10},{60,10}}, rotation = 0)));
          FluidHeatFlow.Sources.Ambient ambient2(constantAmbientTemperature = TAmb, medium = medium) annotation(Placement(transformation(extent = {{80, -10},{100,10}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor1(T(start = TAmb), C = 0.1) annotation(Placement(transformation(origin = {40, -60}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow prescribedHeatFlow1 annotation(Placement(transformation(origin = { -20, -60}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
          Modelica.Thermal.HeatTransfer.Components.Convection Convection1 annotation(Placement(transformation(origin = {10, -40}, extent = {{10,10},{ -10, -10}}, rotation = 270)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor2(T(start = TAmb), C = 0.1) annotation(Placement(transformation(origin = {38,60}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow prescribedHeatFlow2 annotation(Placement(transformation(origin = { -20,60}, extent = {{10,10},{ -10, -10}}, rotation = 180)));
          Modelica.Thermal.HeatTransfer.Components.Convection convection2 annotation(Placement(transformation(origin = {10,40}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Utilities.DoubleRamp volumeFlow annotation(Placement(transformation(extent = {{ -60,10},{ -40,30}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant heatFlow1(k = 5) annotation(Placement(transformation(extent = {{ -60, -70},{ -40, -50}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant heatFlow2(k = 10) annotation(Placement(transformation(extent = {{ -60,50},{ -40,70}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant thermalConductance1(k = 1) annotation(Placement(transformation(extent = {{ -30, -50},{ -10, -30}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant thermalConductance2(k = 1) annotation(Placement(transformation(extent = {{ -30,30},{ -10,50}}, rotation = 0)));

        equation 
          connect(ambient1.flowPort,pump.flowPort_a) annotation(Line(points = {{ -60,0},{ -40,0}}, color = {255,0,0}));
          connect(pump.flowPort_b,pipe1.flowPort_a) annotation(Line(points = {{ -20,0},{ -10,0},{ -10, -10},{0, -10}}, color = {255,0,0}));
          connect(pump.flowPort_b,pipe2.flowPort_a) annotation(Line(points = {{ -20,0},{ -10,0},{ -10,10},{0,10}}, color = {255,0,0}));
          connect(heatFlow2.y,prescribedHeatFlow2.Q_flow) annotation(Line(points = {{ -39,60},{ -30,60}}, color = {0,0,255}));
          connect(heatFlow1.y,prescribedHeatFlow1.Q_flow) annotation(Line(points = {{ -39, -60},{ -30, -60}}, color = {0,0,255}));
          connect(thermalConductance2.y,convection2.Gc) annotation(Line(points = {{ -9,40},{0,40}}, color = {0,0,127}));
          connect(thermalConductance1.y,Convection1.Gc) annotation(Line(points = {{ -9, -40},{0, -40}}, color = {0,0,127}));
          connect(pipe1.heatPort,Convection1.fluid) annotation(Line(points = {{10, -20},{10, -25},{10, -30},{10, -30}}, color = {191,0,0}));
          connect(convection2.fluid,pipe2.heatPort) annotation(Line(points = {{10,30},{10,20}}, color = {191,0,0}));
          connect(convection2.solid,prescribedHeatFlow2.port) annotation(Line(points = {{10,50},{10,60},{ -10,60}}, color = {191,0,0}));
          connect(convection2.solid,heatCapacitor2.port) annotation(Line(points = {{10,50},{10,60},{48,60}}, color = {191,0,0}));
          connect(Convection1.solid,prescribedHeatFlow1.port) annotation(Line(points = {{10, -50},{10, -60},{ -10, -60}}, color = {191,0,0}));
          connect(Convection1.solid,heatCapacitor1.port) annotation(Line(points = {{10, -50},{10, -60},{30, -60}}, color = {191,0,0}));
          connect(pipe2.flowPort_b,pipe3.flowPort_a) annotation(Line(points = {{20,10},{30,10},{30,0},{40,0}}, color = {255,0,0}));
          connect(pipe1.flowPort_b,pipe3.flowPort_a) annotation(Line(points = {{20, -10},{30, -10},{30,0},{40,0}}, color = {255,0,0}));
          connect(pipe3.flowPort_b,ambient2.flowPort) annotation(Line(points = {{60,0},{80,0}}, color = {255,0,0}));
          connect(volumeFlow.y,pump.volumeFlow) annotation(Line(points = {{ -39,20},{ -30,20},{ -30,10}}, color = {0,0,127}, smooth = Smooth.None));
        end ParallelPumpDropOut;
        model OneMass "Example: ccoling of one hot mass"
          extends Modelica.Icons.Example;
          annotation(Documentation(info = "<HTML>
<p>
7th test example: OneMass
</p>
A thermal capacity is coupled with a coolant flow.
Different inital temperatures of thermal capacity and pipe's coolant get ambient's temperature,
the time behaviour depending on coolant flow.
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1.0));
          parameter FluidHeatFlow.Media.Medium medium = FluidHeatFlow.Media.Medium() "Cooling medium" annotation(__Dymola_choicesAllMatching = true);
          parameter Modelica.SIunits.Temperature TAmb(displayUnit = "degC") = 293.15 "Ambient temperature";
          parameter Modelica.SIunits.Temperature TMass(displayUnit = "degC") = 313.15 "Inital temperature of mass";
          output Modelica.SIunits.TemperatureDifference dTMass = heatCapacitor.port.T - TAmb "Mass over Ambient";
          output Modelica.SIunits.TemperatureDifference dTtoPipe = heatCapacitor.port.T - pipe.heatPort.T "Mass over Coolant";
          output Modelica.SIunits.TemperatureDifference dTCoolant = pipe.dT "Coolant's temperature increase";
          FluidHeatFlow.Sources.Ambient ambient1(constantAmbientTemperature = TAmb, medium = medium) annotation(Placement(transformation(extent = {{ -60, -10},{ -80,10}}, rotation = 0)));
          Sources.VolumeFlow pump(medium = medium, m = 0, T0 = TAmb, useVolumeFlowInput = true) annotation(Placement(transformation(extent = {{ -40, -10},{ -20,10}}, rotation = 0)));
          FluidHeatFlow.Components.HeatedPipe pipe(medium = medium, m = 0.1, T0 = TAmb) annotation(Placement(transformation(extent = {{0, -10},{20,10}}, rotation = 0)));
          FluidHeatFlow.Sources.Ambient ambient2(constantAmbientTemperature = TAmb, medium = medium) annotation(Placement(transformation(extent = {{40, -10},{60,10}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor(C = 0.1, T(start = TMass)) annotation(Placement(transformation(origin = {10, -60}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
          Modelica.Thermal.HeatTransfer.Components.ThermalConductor thermalConductor(G = 1) annotation(Placement(transformation(origin = {10, -30}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Utilities.DoubleRamp volumeFlow(offset = 0, height_1 = 1, height_2 =  -2) annotation(Placement(transformation(extent = {{ -60,10},{ -40,30}}, rotation = 0)));

        equation 
          connect(ambient1.flowPort,pump.flowPort_a) annotation(Line(points = {{ -60,0},{ -40,0}}, color = {255,0,0}));
          connect(pump.flowPort_b,pipe.flowPort_a) annotation(Line(points = {{ -20,0},{0,0}}, color = {255,0,0}));
          connect(pipe.flowPort_b,ambient2.flowPort) annotation(Line(points = {{20,0},{40,0}}, color = {255,0,0}));
          connect(thermalConductor.port_a,heatCapacitor.port) annotation(Line(points = {{10, -40},{10, -45},{10, -50},{10, -50}}, color = {191,0,0}));
          connect(pipe.heatPort,thermalConductor.port_b) annotation(Line(points = {{10, -10},{10, -20}}, color = {191,0,0}));
          connect(volumeFlow.y,pump.volumeFlow) annotation(Line(points = {{ -39,20},{ -30,20},{ -30,10}}, color = {0,0,127}, smooth = Smooth.None));
        end OneMass;
        model TwoMass "Example: cooling of two hot masses"
          extends Modelica.Icons.Example;
          annotation(Documentation(info = "<HTML>
<p>
8th test example: TwoMass
</p>
Two thermal capacities are coupled with two parallel coolant flow.
Different inital temperatures of thermal capacities and pipe's coolants get ambient's temperature,
the time behaviour depending on coolant flow.
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1.0));
          parameter FluidHeatFlow.Media.Medium medium = FluidHeatFlow.Media.Medium() "Cooling medium" annotation(__Dymola_choicesAllMatching = true);
          parameter Modelica.SIunits.Temperature TAmb(displayUnit = "degC") = 293.15 "Ambient temperature";
          parameter Modelica.SIunits.Temperature TMass1(displayUnit = "degC") = 313.15 "Inital temperature of mass1";
          parameter Modelica.SIunits.Temperature TMass2(displayUnit = "degC") = 333.15 "Inital temperature of mass2";
          output Modelica.SIunits.TemperatureDifference dTMass1 = heatCapacitor1.port.T - TAmb "Mass1 over Ambient";
          output Modelica.SIunits.TemperatureDifference dTtoPipe1 = heatCapacitor1.port.T - pipe1.heatPort.T "Mass1 over Coolant1";
          output Modelica.SIunits.TemperatureDifference dTCoolant1 = pipe1.dT "Coolant1's temperature increase";
          output Modelica.SIunits.TemperatureDifference dTMass2 = heatCapacitor2.port.T - TAmb "Mass2 over Ambient";
          output Modelica.SIunits.TemperatureDifference dTtoPipe2 = heatCapacitor2.port.T - pipe2.heatPort.T "Mass2 over Coolant2";
          output Modelica.SIunits.TemperatureDifference dTCoolant2 = pipe2.dT "Coolant2's temperature increase";
          output Modelica.SIunits.TemperatureDifference dTmixedCoolant = ambient2.T_port - ambient1.T_port "mixed Coolant's temperature increase";
          FluidHeatFlow.Sources.Ambient ambient1(constantAmbientTemperature = TAmb, medium = medium) annotation(Placement(transformation(extent = {{ -60, -10},{ -80,10}}, rotation = 0)));
          Sources.VolumeFlow pump(medium = medium, m = 0, T0 = TAmb, useVolumeFlowInput = true) annotation(Placement(transformation(extent = {{ -40, -10},{ -20,10}}, rotation = 0)));
          FluidHeatFlow.Components.HeatedPipe pipe1(medium = medium, m = 0.1, T0 = TAmb) annotation(Placement(transformation(extent = {{0, -20},{20,0}}, rotation = 0)));
          FluidHeatFlow.Components.HeatedPipe pipe2(medium = medium, m = 0.1, T0 = TAmb) annotation(Placement(transformation(extent = {{0,20},{20,0}}, rotation = 0)));
          FluidHeatFlow.Components.IsolatedPipe pipe3(medium = medium, m = 0.1, T0 = TAmb) annotation(Placement(transformation(extent = {{40, -10},{60,10}}, rotation = 0)));
          FluidHeatFlow.Sources.Ambient ambient2(constantAmbientTemperature = TAmb, medium = medium) annotation(Placement(transformation(extent = {{80, -10},{100,10}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor1(C = 0.1, T(start = TMass1)) annotation(Placement(transformation(origin = {10, -70}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
          Modelica.Thermal.HeatTransfer.Components.ThermalConductor thermalConductor1(G = 1) annotation(Placement(transformation(origin = {10, -40}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor2(C = 0.1, T(start = TMass2)) annotation(Placement(transformation(origin = {10,70}, extent = {{10,10},{ -10, -10}}, rotation = 180)));
          Modelica.Thermal.HeatTransfer.Components.ThermalConductor thermalConductor2(G = 1) annotation(Placement(transformation(origin = {10,40}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Utilities.DoubleRamp volumeFlow(offset = 0, height_1 = 1, height_2 =  -2) annotation(Placement(transformation(extent = {{ -60,10},{ -40,30}}, rotation = 0)));

        equation 
          connect(ambient1.flowPort,pump.flowPort_a) annotation(Line(points = {{ -60,0},{ -40,0}}, color = {255,0,0}));
          connect(pump.flowPort_b,pipe1.flowPort_a) annotation(Line(points = {{ -20,0},{ -10,0},{ -10, -10},{0, -10}}, color = {255,0,0}));
          connect(pump.flowPort_b,pipe2.flowPort_a) annotation(Line(points = {{ -20,0},{ -10,0},{ -10,10},{0,10}}, color = {255,0,0}));
          connect(pipe2.flowPort_b,pipe3.flowPort_a) annotation(Line(points = {{20,10},{30,10},{30,0},{40,0}}, color = {255,0,0}));
          connect(pipe1.flowPort_b,pipe3.flowPort_a) annotation(Line(points = {{20, -10},{30, -10},{30,0},{40,0}}, color = {255,0,0}));
          connect(pipe3.flowPort_b,ambient2.flowPort) annotation(Line(points = {{60,0},{80,0}}, color = {255,0,0}));
          connect(heatCapacitor2.port,thermalConductor2.port_a) annotation(Line(points = {{10,60},{10,55},{10,55},{10,50}}, color = {191,0,0}));
          connect(thermalConductor2.port_b,pipe2.heatPort) annotation(Line(points = {{10,30},{10,20}}, color = {191,0,0}));
          connect(pipe1.heatPort,thermalConductor1.port_b) annotation(Line(points = {{10, -20},{10, -30}}, color = {191,0,0}));
          connect(thermalConductor1.port_a,heatCapacitor1.port) annotation(Line(points = {{10, -50},{10, -55},{10, -60},{10, -60}}, color = {191,0,0}));
          connect(volumeFlow.y,pump.volumeFlow) annotation(Line(points = {{ -39,20},{ -30,20},{ -30,10}}, color = {0,0,127}, smooth = Smooth.None));
        end TwoMass;
        package Utilities "Utility models for examples"
          extends Modelica.Icons.Library;
          annotation(Documentation(info = "<html>
This package contains utility components used for the test examples.<br>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <p>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern, Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </p>
  <p>
  Dr.Christian Kral<br>
  &Ouml;sterreichisches Forschungs- und Pr&uuml;fzentrum Arsenal Ges.m.b.H.<br>
  <a href=\"http://www.arsenal.ac.at/\">arsenal research</a><br>
  Giefinggasse 2<br>
  A-1210 Vienna, Austria
  </p>
  </dd>
</dl>
<p>
Copyright &copy; 1998-2009, Modelica Association, Anton Haumer and arsenal research.
</p>
<p>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p>
</HTML>", revisions = "<HTML>
  <ul>
  <li> v1.41 Beta 2005/06/17 Anton Haumer<br>
       first used</li>
  </ul>
</HTML>"));
          model DoubleRamp "Ramp going up and down"
            extends Modelica.Blocks.Interfaces.SO;
            parameter Real offset(start = 1) "Offset of ramps";
            parameter Modelica.SIunits.Time startTime(start = 0.2) "StartTime of 1st ramp";
            parameter Modelica.SIunits.Time interval(start = 0.2) "Interval between end of 1st and beginning of 2nd ramp";
            parameter Real height_1(start =  -1) "Height of ramp" annotation(Dialog(group = "Ramp 1"));
            parameter Modelica.SIunits.Time duration_1(min = Modelica.Constants.small, start = 0.2) "Duration of ramp" annotation(Dialog(group = "Ramp 1"));
            parameter Real height_2(start = 1) "Height of ramp" annotation(Dialog(group = "Ramp 2"));
            parameter Modelica.SIunits.Time duration_2(min = Modelica.Constants.small, start = 0.2) "Duration of ramp" annotation(Dialog(group = "Ramp 2"));
            annotation(Documentation(info = "<HTML>
Block generating the sum of two ramps.
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -60},{ -50, -60},{ -30,60},{10,60},{30, -20},{70, -20}}, color = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}));
            Modelica.Blocks.Math.Add add annotation(Placement(transformation(extent = {{10, -10},{30,10}}, rotation = 0)));
            Modelica.Blocks.Sources.Ramp ramp1(final height = height_1, final duration = duration_1, final startTime = startTime, final offset = offset) annotation(Placement(transformation(extent = {{ -30,10},{ -10,30}}, rotation = 0)));
            Modelica.Blocks.Sources.Ramp ramp2(final height = height_2, final duration = duration_2, final startTime = startTime + duration_1 + interval, final offset = 0) annotation(Placement(transformation(extent = {{ -30, -30},{ -10, -10}}, rotation = 0)));

          equation 
            connect(ramp1.y,add.u1) annotation(Line(points = {{ -9,20},{0,20},{0,6},{8,6}}, color = {0,0,127}));
            connect(ramp2.y,add.u2) annotation(Line(points = {{ -9, -20},{0, -20},{0, -6},{8, -6}}, color = {0,0,127}));
            connect(add.y,y) annotation(Line(points = {{31,0},{110,0}}, color = {0,0,127}));
          end DoubleRamp;
        end Utilities;
      end Examples;
      package Components "Basic components (pipes, valves)"
        extends Modelica.Icons.Library2;
        annotation(Documentation(info = "<HTML>
This package contains components:
<ul>
<li>pipe without heat exchange</li>
<li>pipe with heat exchange</li>
<li>valve (simple controlled valve)</li>
</ul>
Pressure drop is taken from partial model SimpleFriction.<br>
Thermodynamic equations are defined in partial models (package Partials).<br>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <p>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern, Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </p>
  <p>
  Dr.Christian Kral<br>
  &Ouml;sterreichisches Forschungs- und Pr&uuml;fzentrum Arsenal Ges.m.b.H.<br>
  <a href=\"http://www.arsenal.ac.at/\">arsenal research</a><br>
  Giefinggasse 2<br>
  A-1210 Vienna, Austria
  </p>
  </dd>
</dl>
<p>
Copyright &copy; 1998-2009, Modelica Association, Anton Haumer and arsenal research.
</p>
<p>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p>
</HTML>", revisions = "<HTML>
  <ul>
  <li> v1.00 2005/02/01 Anton Haumer<br>
       first stable official release</li>
  <li> v1.20 Beta 2005/02/18 Anton Haumer<br>
       introduced geodetic height in Components.Pipes<br>
       <i>new models: Components.Valve</i></li>
  <li> v1.30 Beta 2005/06/02 Anton Haumer<br>
       friction losses are fed to medium</li>
  </ul>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -56,10},{ -56, -90},{ -6, -40},{44,10},{44, -90},{ -56,10}}, lineColor = {0,0,255}, fillColor = {0,128,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -16,10},{4,10},{ -6, -10},{ -16,10}}, lineColor = {0,0,127}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid),Line(points = {{ -6, -10},{ -6, -40},{ -6, -38}}, color = {0,0,127})}));
        model IsolatedPipe "Pipe without heat exchange"
          annotation(Documentation(info = "<HTML>
Pipe without heat exchange.<br>
Thermodynamic equations are defined by Partials.TwoPortMass(Q_flow = 0).<br>
<b>Note:</b> Setting parameter m (mass of medium within pipe) to zero
leads to neglection of temperature transient cv*m*der(T).
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -90,20},{90, -20}}, lineColor = {255,0,0}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,100},{150,40}}, textString = "%name")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          extends Interfaces.Partials.TwoPort(final tapT = 1);
          extends Interfaces.Partials.SimpleFriction;
          parameter Modelica.SIunits.Length h_g(start = 0) "Geodetic height (heigth difference from flowPort_a to flowPort_b)";

        equation 
          volumeFlow = V_flow;
          dp = pressureDrop + medium.rho * Modelica.Constants.g_n * h_g;
          Q_flow = Q_friction;
        end IsolatedPipe;
        model HeatedPipe "Pipe with heat exchange"
          annotation(Documentation(info = "<HTML>
Pipe with heat exchange.<br>
Thermodynamic equations are defined by Partials.TwoPort.<br>
Q_flow is defined by heatPort.Q_flow.<br>
<b>Note:</b> Setting parameter m (mass of medium within pipe) to zero
leads to neglection of temperature transient cv*m*der(T).<br>
<b>Note:</b> Injecting heat into a pipe with zero massflow causes
temperature rise defined by storing heat in medium's mass.
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -90,20},{90, -20}}, lineColor = {255,0,0}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,100},{150,40}}, textString = "%name"),Polygon(points = {{ -10, -90},{ -10, -40},{0, -20},{10, -40},{10, -90},{ -10, -90}}, lineColor = {255,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
          extends Interfaces.Partials.TwoPort;
          extends Interfaces.Partials.SimpleFriction;
          parameter Modelica.SIunits.Length h_g(start = 0) "Geodetic height (heigth difference from flowPort_a to flowPort_b)";
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}}, rotation = 0)));

        equation 
          volumeFlow = V_flow;
          dp = pressureDrop + medium.rho * Modelica.Constants.g_n * h_g;
          Q_flow = heatPort.Q_flow + Q_friction;
          heatPort.T = T_q;
        end HeatedPipe;
        model Valve "Simple valve"
          annotation(Documentation(info = "<HTML>
Simple controlled valve.<br>
Standard characteristic Kv=<i>f </i>(y) is given at standard conditions (dp0, rho0),<br>
<tt>
<ul>
<li>either linear : Kv/Kv1 = Kv0/Kv1 + (1-Kv0/Kv1) * y/Y1</li>
<li>or exponential: Kv/Kv1 = Kv0/Kv1 * exp[ln(Kv1/Kv0) * y/Y1]</li>
</ul>
where:
<ul>
<li>Kv0 ... min. flow @ y = 0</li>
<li>Y1 .... max. valve opening</li>
<li>Kv1 ... max. flow @ y = Y1</li>
</ul>
</tt>
Flow resistance under real conditions is calculated by<br>
<tt>V_flow**2 * rho / dp = Kv(y)**2 * rho0 / dp0</tt>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -150, -60},{150, -120}}, textString = "%name"),Polygon(points = {{ -90,10},{ -60,10},{ -60,60},{0,0},{60,60},{60,10},{90,10},{90, -10},{60, -10},{60, -60},{0,0},{ -60, -60},{ -60, -10},{ -90, -10},{ -90,10}}, lineColor = {255,0,0}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Line(points = {{0,80},{0,0}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
          extends Interfaces.Partials.TwoPort(m(start = 0), final tapT = 1);
          parameter Boolean LinearCharacteristic(start = true) "Type of characteristic" annotation(Dialog(group = "Standard characteristic"), choices(choice = true "Linear", choice = false "Exponential"));
          parameter Real y1(min = small, start = 1) "Max. valve opening" annotation(Dialog(group = "Standard characteristic"));
          parameter Modelica.SIunits.VolumeFlowRate Kv1(min = small, start = 1) "Max. flow @ y = y1" annotation(Dialog(group = "Standard characteristic"));
          parameter Real kv0(min = small, max = 1 - small, start = 0.01) "Leakage flow / max.flow @ y = 0" annotation(Dialog(group = "Standard characteristic"));
          parameter Modelica.SIunits.Pressure dp0(start = 1) "Standard pressure drop" annotation(Dialog(group = "Standard characteristic"));
          parameter Modelica.SIunits.Density rho0(start = 10) "Standard medium's density" annotation(Dialog(group = "Standard characteristic"));
          parameter Real frictionLoss(min = 0, max = 1, start = 0) "Part of friction losses fed to medium";
        protected 
          constant Modelica.SIunits.VolumeFlowRate unitVolumeFlowRate = 1;
          constant Real small = Modelica.Constants.small;
          constant Modelica.SIunits.VolumeFlowRate smallVolumeFlowRate = eps * unitVolumeFlowRate;
          constant Real eps = Modelica.Constants.eps;
          Real yLim = max(min(y, y1), 0) "Limited valve opening";
          Modelica.SIunits.VolumeFlowRate Kv "Standard flow rate";
        public 
          Modelica.Blocks.Interfaces.RealInput y annotation(Placement(transformation(origin = {0,90}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        initial algorithm 
          assert(y1 > small, "Valve characteristic: y1 has to be > 0 !");
          assert(Kv1 > smallVolumeFlowRate, "Valve characteristic: Kv1 has to be > 0 !");
          assert(kv0 > small, "Valve characteristic: kv0 has to be > 0 !");
          assert(kv0 < 1 - eps, "Valve characteristic: kv0 has to be < 1 !");

        equation 
          Kv / Kv1 = if LinearCharacteristic then kv0 + ((1 - kv0) * yLim) / y1 else kv0 * exp((Modelica.Math.log(1 / kv0) * yLim) / y1);
          dp / dp0 = medium.rho / rho0 * V_flow / Kv * abs(V_flow / Kv);
          Q_flow = frictionLoss * V_flow * dp;
        end Valve;
      end Components;
      package Media "Medium properties"
        extends Modelica.Icons.Library2;
        annotation(Documentation(info = "<HTML>
This package contains definitions of medium properties.<br>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <p>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern, Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </p>
  <p>
  Dr.Christian Kral<br>
  &Ouml;sterreichisches Forschungs- und Pr&uuml;fzentrum Arsenal Ges.m.b.H.<br>
  <a href=\"http://www.arsenal.ac.at/\">arsenal research</a><br>
  Giefinggasse 2<br>
  A-1210 Vienna, Austria
  </p>
  </dd>
</dl>
<p>
Copyright &copy; 1998-2009, Modelica Association, Anton Haumer and arsenal research.
</p>
<p>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p>
</HTML>", revisions = "<HTML>
  <ul>
  <li> v1.00 2005/02/01 Anton Haumer<br>
       first stable official release</li>
  <li> v1.11 2005/02/18 Anton Haumer<br>
       corrected usage of cv and cp</li>
  </ul>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -80,8},{60, -88}}, lineColor = {0,0,127}, fillColor = {85,170,255}, fillPattern = FillPattern.Solid),Line(points = {{ -10,8},{ -10, -88},{ -10, -88}}, color = {0,0,127}),Line(points = {{ -80, -24},{60, -24}}, color = {0,0,127}),Line(points = {{ -80, -56},{60, -56}}, color = {0,0,127})}));
        record Medium "Record containing media properties"
          extends Modelica.Icons.Record;
          parameter Modelica.SIunits.Density rho = 1 "Density";
          parameter Modelica.SIunits.SpecificHeatCapacity cp = 1 "Specific heat capacity at constant pressure";
          parameter Modelica.SIunits.SpecificHeatCapacity cv = 1 "Specific heat capacity at constant volume";
          parameter Modelica.SIunits.ThermalConductivity lamda = 1 "Thermal conductivity";
          parameter Modelica.SIunits.KinematicViscosity nue = 1 "kinematic viscosity";
          annotation(Documentation(info = "<html>
Record containing (constant) medium properties.
</html>"));
        end Medium;
        record Air_30degC "Medium: properties of air at 30 degC"
          extends Medium(rho = 1.149, cp = 1007, cv = 720, lamda = 0.0264, nue = 1.63e-05);
          annotation(Documentation(info = "<html>
Medium: properties of air at 30 degC
</html>"));
        end Air_30degC;
        record Air_70degC "Medium: properties of air at 70 degC"
          extends Medium(rho = 1.015, cp = 1010, cv = 723, lamda = 0.0293, nue = 2.03e-05);
          annotation(Documentation(info = "<html>
Medium: properties of air at 70 degC
</html>"));
        end Air_70degC;
        record Water "Medium: properties of water"
          extends Medium(rho = 995.6, cp = 4177, cv = 4177, lamda = 0.615, nue = 8e-07);
          annotation(Documentation(info = "<html>
Medium: properties of water
</html>"));
        end Water;
      end Media;
      package Sources "Ideal fluid sources, e.g., ambient, volume flow"
        extends Modelica.Icons.Library2;
        annotation(Documentation(info = "<HTML>
This package contains different types of sources:
<ul>
<li>Ambient with constant or prescribed pressure and temperature</li>
<li>AbsolutePressure to define pressure level of a closed cooling cycle.</li>
<li>Constant and prescribed volume flow</li>
<li>Constant and prescribed pressure increase</li>
<li>Simple pump with mechanical flange</li>
</ul>
Thermodynamic equations are defined in partial models (package Interfaces.Partials).<br>
All fans / pumps are considered without losses, they do not change enthalpy flow.<br>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <p>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern, Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </p>
  <p>
  Dr.Christian Kral<br>
  &Ouml;sterreichisches Forschungs- und Pr&uuml;fzentrum Arsenal Ges.m.b.H.<br>
  <a href=\"http://www.arsenal.ac.at/\">arsenal research</a><br>
  Giefinggasse 2<br>
  A-1210 Vienna, Austria
  </p>
  </dd>
</dl>
<p>
Copyright &copy; 1998-2009, Modelica Association, Anton Haumer and arsenal research.
</p>
<p>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p>
</HTML>", revisions = "<HTML>
  <ul>
  <li> v1.00 2005/02/01 Anton Haumer<br>
       first stable official release</li>
  <li> v1.20 Beta 2005/02/18 Anton Haumer<br>
       <i>new model: IdealPump</i></li>
  </ul>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -60,10},{40, -90}}, lineColor = {255,0,0}),Polygon(points = {{ -40,0},{ -40, -80},{38, -50},{38, -30},{ -40,0}}, lineColor = {0,0,255})}));
        model Ambient "Ambient with constant properties"
          annotation(Documentation(info = "<HTML>
(Infinite) ambient with constant pressure and temperature.<br>
Thermodynamic equations are defined by Partials.Ambient.
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{20,80},{80,20}}, lineColor = {0,0,0}, textString = "p"),Text(extent = {{20, -20},{80, -80}}, lineColor = {0,0,0}, textString = "T")}));
          extends Interfaces.Partials.Ambient;
          parameter Boolean usePressureInput = false "enable / disable pressure input" annotation(Evaluate = true);
          parameter Modelica.SIunits.Pressure constantAmbientPressure(start = 0) "Ambient pressure" annotation(Dialog(enable = not usePressureInput));
          parameter Boolean useTemperatureInput = false "enable / disable temperature input" annotation(Evaluate = true);
          parameter Modelica.SIunits.Temperature constantAmbientTemperature(start = 293.15, displayUnit = "degC") "Ambient temperature" annotation(Dialog(enable = not useTemperatureInput));
          Modelica.Blocks.Interfaces.RealInput ambientPressure if usePressureInput annotation(Placement(transformation(extent = {{110,60},{90,80}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput ambientTemperature if useTemperatureInput annotation(Placement(transformation(extent = {{110, -60},{90, -80}}, rotation = 0)));
        protected 
          Modelica.Blocks.Sources.Constant constPressure(final k = constantAmbientPressure) if not usePressureInput annotation(Placement(transformation(extent = {{100,40},{80,60}})));
          Modelica.Blocks.Interfaces.RealInput pAmbient annotation(Placement(transformation(extent = {{62,58},{58,62}})));
          Modelica.Blocks.Sources.Constant constTemperature(final k = constantAmbientTemperature) if not useTemperatureInput annotation(Placement(transformation(extent = {{100, -60},{80, -40}})));
          Modelica.Blocks.Interfaces.RealInput TAmbient annotation(Placement(transformation(extent = {{62, -62},{58, -58}})));

        equation 
          flowPort.p = pAmbient;
          T = TAmbient;
          connect(ambientPressure,pAmbient) annotation(Line(points = {{100,70},{60,70},{60,60}}, color = {0,0,127}, smooth = Smooth.None));
          connect(ambientTemperature,TAmbient) annotation(Line(points = {{100, -70},{60, -70},{60, -60}}, color = {0,0,127}, smooth = Smooth.None));
          connect(constPressure.y,pAmbient) annotation(Line(points = {{79,50},{60,50},{60,60}}, color = {0,0,127}, smooth = Smooth.None));
          connect(constTemperature.y,TAmbient) annotation(Line(points = {{79, -50},{60, -50},{60, -60}}, color = {0,0,127}, smooth = Smooth.None));
        end Ambient;
        model AbsolutePressure "Defines absolute pressure level"
          annotation(Documentation(info = "<HTML>
AbsolutePressure to define pressure level of a closed cooling cycle.
Coolant's mass flow, temperature and enthalpy flow are not affected.<br>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -150,150},{150,90}}, lineColor = {0,0,255}, textString = "%name"),Ellipse(extent = {{ -90,90},{90, -90}}, lineColor = {255,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}));
          parameter FluidHeatFlow.Media.Medium medium = FluidHeatFlow.Media.Medium() "medium" annotation(__Dymola_choicesAllMatching = true);
          parameter Modelica.SIunits.Pressure p(start = 0) "Pressure ground";
          annotation(Icon(Ellipse(extent = [ -90,90;90, -90], style(color = 1, rgbcolor = {255,0,0}, fillColor = 7, rgbfillColor = {255,255,255}))));
          Interfaces.FlowPort_a flowPort(final medium = medium) annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));

        equation 
          flowPort.p = p;
          flowPort.H_flow = 0;
        end AbsolutePressure;
        model VolumeFlow "Enforces constant volume flow"
          annotation(Documentation(info = "<HTML>
Fan resp. pump with constant volume flow rate. Pressure increase is the response of the whole system.
Coolant's temperature and enthalpy flow are not affected.<br>
Setting parameter m (mass of medium within fan/pump) to zero
leads to neglection of temperature transient cv*m*der(T).<br>
Thermodynamic equations are defined by Partials.TwoPort.
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -90,90},{90, -90}}, lineColor = {255,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -150, -90},{150, -150}}, lineColor = {0,0,255}, textString = "%name"),Polygon(points = {{ -60,68},{90,10},{90, -10},{ -60, -68},{ -60,68}}, lineColor = {255,0,0}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -40,20},{0, -20}}, lineColor = {0,0,0}, textString = "V")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          extends Interfaces.Partials.TwoPort(final tapT = 1);
          parameter Boolean useVolumeFlowInput = false "enable / disable volume flow input" annotation(Evaluate = true);
          parameter Modelica.SIunits.VolumeFlowRate constantVolumeFlow(start = 1) "Volume flow rate" annotation(Dialog(enable = not useVolumeFlowInput));
          Modelica.Blocks.Interfaces.RealInput volumeFlow if useVolumeFlowInput annotation(Placement(transformation(origin = {0,100}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        protected 
          Modelica.Blocks.Sources.Constant constVolumeFlow(final k = constantVolumeFlow) if not useVolumeFlowInput annotation(Placement(transformation(extent = {{10, -10},{ -10,10}}, rotation = 90, origin = {30,90})));
          Modelica.Blocks.Interfaces.RealInput internalVolumeFlow annotation(Placement(transformation(extent = {{ -2, -2},{2,2}}, rotation = 270, origin = {0,60})));

        equation 
          Q_flow = 0;
          V_flow = internalVolumeFlow;
          connect(volumeFlow,internalVolumeFlow) annotation(Line(points = {{0,100},{0,60}}, color = {0,0,127}, smooth = Smooth.None));
          connect(constVolumeFlow.y,internalVolumeFlow) annotation(Line(points = {{30,79},{30,60},{0,60}}, color = {0,0,127}, smooth = Smooth.None));
        end VolumeFlow;
        model PressureIncrease "Enforces constant pressure increase"
          annotation(Documentation(info = "<HTML>
Fan resp. pump with constant pressure increase. Mass resp. volume flow is the response of the whole system.
Coolant's temperature and enthalpy flow are not affected.<br>
Setting parameter m (mass of medium within fan/pump) to zero
leads to neglection of temperature transient cv*m*der(T).<br>
Thermodynamic equations are defined by Partials.TwoPort.
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -90,90},{90, -90}}, lineColor = {255,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -150, -90},{150, -150}}, lineColor = {0,0,255}, textString = "%name"),Polygon(points = {{ -60,68},{90,10},{90, -10},{ -60, -68},{ -60,68}}, lineColor = {0,0,255}, fillColor = {255,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -40,20},{0, -20}}, lineColor = {0,0,0}, textString = "dp")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          extends Interfaces.Partials.TwoPort(final tapT = 1);
          parameter Boolean usePressureIncreaseInput = false "enable / disable pressure increase input" annotation(Evaluate = true);
          parameter Modelica.SIunits.Pressure constantPressureIncrease(start = 1) "Pressure increase" annotation(Dialog(enable = not usePressureIncreaseInput));
          Modelica.Blocks.Interfaces.RealInput pressureIncrease if usePressureIncreaseInput annotation(Placement(transformation(origin = {0,100}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        protected 
          Modelica.Blocks.Sources.Constant constPressureIncrease(final k = constantPressureIncrease) if not usePressureIncreaseInput annotation(Placement(transformation(extent = {{10, -10},{ -10,10}}, rotation = 90, origin = {30,90})));
          Modelica.Blocks.Interfaces.RealInput internalPressureIncrease annotation(Placement(transformation(extent = {{ -2, -2},{2,2}}, rotation = 270, origin = {0,60})));

        equation 
          Q_flow = 0;
          dp =  -internalPressureIncrease;
          connect(pressureIncrease,internalPressureIncrease) annotation(Line(points = {{0,100},{0,60}}, color = {0,0,127}, smooth = Smooth.None));
          connect(constPressureIncrease.y,internalPressureIncrease) annotation(Line(points = {{30,79},{30,60},{0,60}}, color = {0,0,127}, smooth = Smooth.None));
        end PressureIncrease;
        model IdealPump "Model of an ideal pump"
          annotation(Documentation(info = "<HTML>
Simple fan resp. pump where characteristic is dependent on shaft's speed, <br>
torque * speed = pressure increase * volume flow (without losses)<br>
Pressure increase versus volume flow is defined by a linear function,
from dp0(V_flow=0) to V_flow0(dp=0).<br>
The axis intersections vary with speed as follows:
<ul>
<li>dp prop. speed^2</li>
<li>V_flow prop. speed</li>
</ul>
Coolant's temperature and enthalpy flow are not affected.<br>
Setting parameter m (mass of medium within fan/pump) to zero
leads to neglection of temperature transient cv*m*der(T).<br>
Thermodynamic equations are defined by Partials.TwoPort.
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -90,90},{90, -90}}, lineColor = {255,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,150},{150,90}}, lineColor = {0,0,255}, textString = "%name"),Rectangle(extent = {{ -10, -40},{10, -100}}, lineColor = {0,0,0}, fillPattern = FillPattern.VerticalCylinder, fillColor = {175,175,175}),Polygon(points = {{ -60,68},{90,10},{90, -10},{ -60, -68},{ -60,68}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          extends Interfaces.Partials.TwoPort(final tapT = 1);
          parameter Modelica.SIunits.AngularVelocity wNominal(start = 1, displayUnit = "1/min") "Nominal speed" annotation(Dialog(group = "Pump characteristic"));
          parameter Modelica.SIunits.Pressure dp0(start = 2) "Max. pressure increase @ V_flow=0" annotation(Dialog(group = "Pump characteristic"));
          parameter Modelica.SIunits.VolumeFlowRate V_flow0(start = 2) "Max. volume flow rate @ dp=0" annotation(Dialog(group = "Pump characteristic"));
          Modelica.SIunits.AngularVelocity w = der(flange_a.phi) "Speed";
        protected 
          Modelica.SIunits.Pressure dp1;
          Modelica.SIunits.VolumeFlowRate V_flow1;
        public 
          Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}}, rotation = 0)));

        equation 
          dp1 = dp0 * sign(w / wNominal) * (w / wNominal) ^ 2;
          V_flow1 = V_flow0 * w / wNominal;
          if noEvent(abs(w) < Modelica.Constants.small) then
            dp = 0;
            flange_a.tau = 0;

          else
            dp =  -dp1 * (1 - noEvent(abs(V_flow / V_flow1)));
            flange_a.tau * w =  -dp * V_flow;

          end if;
          Q_flow = 0;
        end IdealPump;
      end Sources;
      package Sensors "Ideal sensors to measure port properties"
        extends Modelica.Icons.Library2;
        annotation(Documentation(info = "<HTML>
This package contains sensors:
<ul>
<li>pSensor: absolute pressure</li>
<li>TSensor: absolute temperature (Kelvin)</li>
<li>dpSensor: pressure drop between flowPort_a and flowPort_b</li>
<li>dTSensor: temperature difference between flowPort_a and flowPort_b</li>
<li>m_flowSensor: measures mass flow rate</li>
<li>V_flowSensor: measures volume flow rate</li>
<li>H_flowSensor: measures enthalpy flow rate</li>
</ul>
Some of the sensors do not need access to medium properties for measuring,
but it is necessary to define the medium in the connector (check of connections).<br>
Thermodynamic equations are defined in partial models (package Interfaces.Partials).<br>
All sensors are considered massless, they do not change mass flow or enthalpy flow.<br>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <p>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern, Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </p>
  <p>
  Dr.Christian Kral<br>
  &Ouml;sterreichisches Forschungs- und Pr&uuml;fzentrum Arsenal Ges.m.b.H.<br>
  <a href=\"http://www.arsenal.ac.at/\">arsenal research</a><br>
  Giefinggasse 2<br>
  A-1210 Vienna, Austria
  </p>
  </dd>
</dl>
<p>
Copyright &copy; 1998-2009, Modelica Association, Anton Haumer and arsenal research.
</p>
<p>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p>
</HTML>", revisions = "<HTML>
  <ul>
  <li> v1.00 2005/02/01 Anton Haumer<br>
       first stable official release</li>
  </ul>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -60,10},{40, -90}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -10,7},{ -10, -10}}, color = {0,0,0}),Line(points = {{ -50, -16},{ -36, -25}}, color = {0,0,0}),Line(points = {{30, -15},{16, -25}}, color = {0,0,0}),Line(points = {{ -35,0},{ -25, -14}}, color = {0,0,0}),Line(points = {{15,0},{5, -14}}, color = {0,0,0}),Ellipse(extent = {{ -15, -35},{ -5, -45}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -10, -40},{ -6, -26}}, color = {0,0,0}),Polygon(points = {{ -12, -24},{ -0.5, -27},{2,1.5},{ -12, -24}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}));
        model PressureSensor "Absolute pressure sensor"
          annotation(Documentation(info = "<HTML>
pSensor measures the absolute pressure.<br>
Thermodynamic equations are defined by Partials.AbsoluteSensor.
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          extends Interfaces.Partials.AbsoluteSensor;

        equation 
          y = flowPort.p;
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -22, -20},{20, -60}}, lineColor = {0,0,255}, textString = "p")}), Diagram);
        end PressureSensor;
        model TemperatureSensor "Absolute temperature sensor"
          annotation(Documentation(info = "<HTML>
TSensor measures the absolute temperature (Kelvin).<br>
Thermodynamic equations are defined by Partials.AbsoluteSensor.
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          extends Interfaces.Partials.AbsoluteSensor;

        equation 
          medium.cp * y = flowPort.h;
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -22, -20},{20, -60}}, lineColor = {0,0,255}, textString = "T")}));
        end TemperatureSensor;
        model RelPressureSensor "Pressure difference sensor"
          annotation(Documentation(info = "<HTML>
dpSensor measures the pressure drop between flowPort_a and flowPort_b.<br>
Thermodynamic equations are defined by Partials.RelativeSensor.
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          extends Interfaces.Partials.RelativeSensor;

        equation 
          y = flowPort_a.p - flowPort_b.p;
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -22, -20},{20, -60}}, lineColor = {0,0,255}, textString = "dp")}));
        end RelPressureSensor;
        model RelTemperatureSensor "Temperature difference sensor"
          annotation(Documentation(info = "<HTML>
dTSensor measures the temperature difference between flowPort_a and flowPort_b.<br>
Thermodynamic equations are defined by Partials.RelativeSensor.<br>
<b>Note:</b> Connected flowPorts have the same temperature (mixing temperature)!<br>
Since mixing my occur, the outlet temperature of a component may be different from the connector's temperature.<br>
Outlet temperature is defined by variable T of the corresponding component.
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          extends Interfaces.Partials.RelativeSensor;

        equation 
          medium.cp * y = flowPort_a.h - flowPort_b.h;
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -22, -20},{20, -60}}, lineColor = {0,0,255}, textString = "dT")}));
        end RelTemperatureSensor;
        model MassFlowSensor "Mass flow sensor"
          annotation(Documentation(info = "<HTML>
m_flowSensor measures the mass flow rate.<br>
Thermodynamic equations are defined by Partials.FlowSensor.
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          extends Interfaces.Partials.FlowSensor;

        equation 
          y = V_flow * medium.rho;
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -22, -20},{20, -60}}, lineColor = {0,0,255}, textString = "m")}));
        end MassFlowSensor;
        model VolumeFlowSensor "Volume flow sensor"
          annotation(Documentation(info = "<HTML>
V_flowSensor measures the volume flow rate.<br>
Thermodynamic equations are defined by Partials.FlowSensor.
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          extends Interfaces.Partials.FlowSensor;

        equation 
          y = V_flow;
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -22, -20},{20, -60}}, lineColor = {0,0,255}, textString = "V")}));
        end VolumeFlowSensor;
        model EnthalpyFlowSensor "Enthapy flow sensor"
          annotation(Documentation(info = "<HTML>
H_flowSensor measures the enthalpy flow rate.<br>
Thermodynamic equations are defined by Partials.FlowSensor.
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          extends Interfaces.Partials.FlowSensor;

        equation 
          y = flowPort_a.H_flow;
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -22, -20},{20, -60}}, lineColor = {0,0,255}, textString = "H")}));
        end EnthalpyFlowSensor;
      end Sensors;
      package Interfaces "Connectors and partial models"
        extends Modelica.Icons.Library2;
        annotation(Documentation(info = "<HTML>
This package contains connectors and partial models:
<ul>
<li>FlowPort: basic definition of the connector.</li>
<li>FlowPort_a &amp; FlowPort_b: same as FlowPort with different icons to differentiate direction of flow</li>
<li>package Partials (defining basic thermodynamic equations)</li>
</ul>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <p>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern, Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </p>
  <p>
  Dr.Christian Kral<br>
  &Ouml;sterreichisches Forschungs- und Pr&uuml;fzentrum Arsenal Ges.m.b.H.<br>
  <a href=\"http://www.arsenal.ac.at/\">arsenal research</a><br>
  Giefinggasse 2<br>
  A-1210 Vienna, Austria
  </p>
  </dd>
</dl>
<p>
Copyright &copy; 1998-2009, Modelica Association, Anton Haumer and arsenal research.
</p>
<p>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p>
</HTML>", revisions = "<HTML>
  <ul>
  <li> v1.00 2005/02/01 Anton Haumer<br>
       first stable official release</li>
  <li> v1.10 2005/02/15 Anton Haumer<br>
       moved Partials into Interfaces</li>
  <li> v1.11 2005/02/18 Anton Haumer<br>
       corrected usage of cv and cp</li>
  <li> v1.30 Beta 2005/06/02 Anton Haumer<br>
       friction losses are fed to medium</li>
  <li> v1.33 Beta 2005/06/07 Anton Haumer<br>
       corrected usage of simpleFlow</li>
  <li> v1.43 Beta 2005/06/20 Anton Haumer<br>
       Test of mixing / semiLinear</li>
  <li> v1.50 2005/09/07 Anton Haumer<br>
       semiLinear works fine</li>
  </ul>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -60,10},{40, -90}}, lineColor = {255,0,0}),Ellipse(extent = {{ -58,8},{38, -88}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid)}));
        connector FlowPort "conector flow port"
          annotation(Documentation(info = "<HTML>
Basic definition of the connector.<br>
<b>Variables:</b>
<ul>
<li>Pressure p</li>
<li>flow MassFlowRate m_flow</li>
<li>Specific Enthalpy h</li>
<li>flow EnthaplyFlowRate H_flow</li>
</ul>
If ports with different media are connected, the simulation is asserted due to the check of parameter.
</HTML>"));
          parameter FluidHeatFlow.Media.Medium medium "Medium in the connector";
          Modelica.SIunits.Pressure p;
          flow Modelica.SIunits.MassFlowRate m_flow;
          Modelica.SIunits.SpecificEnthalpy h;
          flow Modelica.SIunits.EnthalpyFlowRate H_flow;
        end FlowPort;
        connector FlowPort_a "Filled flow port (used upstream)"
          annotation(Documentation(info = "<HTML>
Same as FlowPort, but icon allows to differentiate direction of flow.
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {255,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -98,98},{98, -98}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -50,50},{50, -50}}, lineColor = {255,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -48,48},{48, -48}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -100,110},{100,50}}, lineColor = {0,0,255}, textString = "%name")}));
          extends FlowPort;
        end FlowPort_a;
        connector FlowPort_b "Hollow flow port (used downstream)"
          annotation(Documentation(info = "<HTML>
Same as FlowPort, but icon allows to differentiate direction of flow.
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {255,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -98,98},{98, -98}}, lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -50,50},{50, -50}}, lineColor = {255,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -48,48},{48, -48}}, lineColor = {0,0,255}),Text(extent = {{ -100,110},{100,50}}, lineColor = {0,0,255}, textString = "%name")}));
          extends FlowPort;
        end FlowPort_b;
        package Partials "Partial models"
          extends Modelica.Icons.Library;
          annotation(Documentation(info = "<HTML>
This package contains partial models, defining in a very compact way the basic thermodynamic equations used by the different components.<br>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <p>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern, Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </p>
  <p>
  Dr.Christian Kral<br>
  &Ouml;sterreichisches Forschungs- und Pr&uuml;fzentrum Arsenal Ges.m.b.H.<br>
  <a href=\"http://www.arsenal.ac.at/\">arsenal research</a><br>
  Giefinggasse 2<br>
  A-1210 Vienna, Austria
  </p>
  </dd>
</dl>
<p>
Copyright &copy; 1998-2009, Modelica Association, Anton Haumer and arsenal research.
</p>
<p>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p>
</HTML>", revisions = "<HTML>
  <ul>
  <li> v1.00 2005/02/01 Anton Haumer<br>
       first stable official release</li>
  <li> v1.10 2005/02/15 Anton Haumer<br>
       moved Partials into Interfaces</li>
  <li> v1.11 2005/02/18 Anton Haumer<br>
       corrected usage of cv and cp</li>
  <li> v1.30 Beta 2005/06/02 Anton Haumer<br>
       friction losses are fed to medium</li>
  <li> v1.31 Beta 2005/06/04 Anton Haumer<br>
       searching solution for problems @ m_flow=0</li>
  <li> v1.33 Beta 2005/06/07 Anton Haumer<br>
       corrected usage of simpleFlow</li>
  <li> v1.43 Beta 2005/06/20 Anton Haumer<br>
       Test of mixing / semiLinear</li>
  <li> v1.50 2005/09/07 Anton Haumer<br>
       semiLinear works fine<br>
       removed test-version of semiLinear</li>
  <li> v1.60 2007/01/23 Anton Haumer<br>
       new parameter tapT defining Temperature of heatPort</li>
  </ul>
</HTML>
"));
          partial model SimpleFriction "Simple friction model"
            annotation(Documentation(info = "<HTML>
Definition of relationship between pressure drop and volume flow rate:<br>
-V_flowLaminar &lt; VolumeFlow &lt; +V_flowLaminar: laminar i.e. linear dependency of pressure drop on volume flow.<br>
-V_flowLaminar &gt; VolumeFlow or VolumeFlow &lt; +V_flowLaminar: turbulent i.e. quadratic dependency of pressure drop on volume flow.<br>
Linear and quadratic dependency are coupled smoothly at V_flowLaminar / dpLaminar.<br>
Quadratic dependency is defined by nominal volume flow and pressure drop (V_flowNominal / dpNominal).<br>
See also sketch at diagram layer.
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -80,0},{80,0}}, color = {0,0,255}),Line(points = {{0,80},{0, -80}}, color = {0,0,255}),Line(points = {{ -40, -20},{40,20}}, color = {0,0,255}),Line(points = {{40,20},{60,40},{70,60},{74,80}}, color = {0,0,255}),Line(points = {{ -40, -20},{ -60, -40},{ -70, -60},{ -74, -80}}, color = {0,0,255}),Line(points = {{40,20},{40,0}}, color = {0,0,255}),Line(points = {{60,40},{60,0}}, color = {0,0,255}),Line(points = {{40,20},{0,20}}, color = {0,0,255}),Line(points = {{60,40},{0,40}}, color = {0,0,255}),Text(extent = {{18,0},{48, -20}}, lineColor = {0,0,255}, textString = "V_flowLaminar"),Text(extent = {{50,0},{80, -20}}, lineColor = {0,0,255}, textString = "V_flowNominal"),Text(extent = {{ -30,30},{ -4,10}}, lineColor = {0,0,255}, textString = "dpLaminar"),Text(extent = {{ -30,50},{ -4,30}}, lineColor = {0,0,255}, textString = "dpNominal"),Text(extent = {{0,20},{30,0}}, lineColor = {0,0,255}, textString = "dp ~ V_flow"),Text(extent = {{30,60},{60,40}}, lineColor = {0,0,255}, textString = "dp ~ V_flow")}));
            parameter Modelica.SIunits.VolumeFlowRate V_flowLaminar(min = Modelica.Constants.small, start = 0.1) "Laminar volume flow" annotation(Dialog(group = "Simple Friction"));
            parameter Modelica.SIunits.Pressure dpLaminar(start = 0.1) "Laminar pressure drop" annotation(Dialog(group = "Simple Friction"));
            parameter Modelica.SIunits.VolumeFlowRate V_flowNominal(start = 1) "Nominal volume flow" annotation(Dialog(group = "Simple Friction"));
            parameter Modelica.SIunits.Pressure dpNominal(start = 1) "Nominal pressure drop" annotation(Dialog(group = "Simple Friction"));
            parameter Real frictionLoss(min = 0, max = 1) = 0 "Part of friction losses fed to medium" annotation(Dialog(group = "Simple Friction"));
            Modelica.SIunits.Pressure pressureDrop;
            Modelica.SIunits.VolumeFlowRate volumeFlow;
            Modelica.SIunits.Power Q_friction;
          protected 
            parameter Modelica.SIunits.Pressure dpNomMin = dpLaminar / V_flowLaminar * V_flowNominal;
            parameter Real k(final unit = "Pa.s2/m6", fixed = false);
          initial algorithm 
            assert(V_flowNominal > V_flowLaminar, "SimpleFriction: V_flowNominal has to be > V_flowLaminar!");
            assert(dpNominal >= dpNomMin, "SimpleFriction: dpNominal has to be > dpLaminar/V_flowLaminar*V_flowNominal!");
            k:=(dpNominal - dpNomMin) / (V_flowNominal - V_flowLaminar) ^ 2;

          equation 
            if volumeFlow >  +V_flowLaminar then
              pressureDrop =  +dpLaminar / V_flowLaminar * volumeFlow + k * (volumeFlow - V_flowLaminar) ^ 2;

            elseif volumeFlow <  -V_flowLaminar then
              pressureDrop =  +dpLaminar / V_flowLaminar * volumeFlow - k * (volumeFlow + V_flowLaminar) ^ 2;


            else
              pressureDrop = dpLaminar / V_flowLaminar * volumeFlow;

            end if;
            Q_friction = frictionLoss * volumeFlow * pressureDrop;
          end SimpleFriction;
          partial model TwoPort "Partial model of two port"
            annotation(Documentation(info = "<HTML>
Partial model with two flowPorts.<br>
Possible heat exchange with the ambient is defined by Q_flow; setting this = 0 means no energy exchange.<br>
Setting parameter m (mass of medium within pipe) to zero
leads to neglection of temperature transient cv*m*der(T).<br>
Mixing rule is applied.<br>
Parameter 0 &lt; tapT &lt; 1 defines temperature of heatPort between medium's inlet and outlet temperature.
</HTML>"));
            parameter FluidHeatFlow.Media.Medium medium = FluidHeatFlow.Media.Medium() "Medium in the component" annotation(__Dymola_choicesAllMatching = true);
            parameter Modelica.SIunits.Mass m(start = 1) "Mass of medium";
            parameter Modelica.SIunits.Temperature T0(start = 293.15, displayUnit = "degC") "Initial temperature of medium" annotation(Dialog(enable = m > Modelica.Constants.small));
            parameter Real tapT(final min = 0, final max = 1) = 1 "Defines temperature of heatPort between inlet and outlet temperature";
            Modelica.SIunits.Pressure dp = flowPort_a.p - flowPort_b.p "Pressure drop a->b";
            Modelica.SIunits.VolumeFlowRate V_flow = flowPort_a.m_flow / medium.rho "Volume flow a->b";
            Modelica.SIunits.HeatFlowRate Q_flow "Heat exchange with ambient";
            output Modelica.SIunits.Temperature T(start = T0) "Outlet temperature of medium";
            output Modelica.SIunits.Temperature T_a = flowPort_a.h / medium.cp "Temperature at flowPort_a";
            output Modelica.SIunits.Temperature T_b = flowPort_b.h / medium.cp "Temperature at flowPort_b";
            output Modelica.SIunits.TemperatureDifference dT = if noEvent(V_flow >= 0) then T - T_a else T_b - T "Temperature increase of coolant in flow direction";
          protected 
            Modelica.SIunits.SpecificEnthalpy h = medium.cp * T "Medium's specific enthalpy";
            Modelica.SIunits.Temperature T_q = T - noEvent(sign(V_flow)) * (1 - tapT) * dT "Temperature relevant for heat exchange with ambient";
          public 
            Interfaces.FlowPort_a flowPort_a(final medium = medium) annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
            Interfaces.FlowPort_b flowPort_b(final medium = medium) annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));

          equation 
            flowPort_a.m_flow + flowPort_b.m_flow = 0;
            if m > Modelica.Constants.small then
              flowPort_a.H_flow + flowPort_b.H_flow + Q_flow = m * medium.cv * der(T);

            else
              flowPort_a.H_flow + flowPort_b.H_flow + Q_flow = 0;

            end if;
            flowPort_a.H_flow = semiLinear(flowPort_a.m_flow, flowPort_a.h, h);
            flowPort_b.H_flow = semiLinear(flowPort_b.m_flow, flowPort_b.h, h);
          end TwoPort;
          partial model Ambient "Partial model of ambient"
            annotation(Documentation(info = "<HTML>
<p>
Partial model of (Infinite) ambient, defines pressure and temperature.
</p>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -90,90},{90, -90}}, lineColor = {255,0,0}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,150},{150,90}}, lineColor = {0,0,255}, textString = "%name")}));
            parameter FluidHeatFlow.Media.Medium medium = FluidHeatFlow.Media.Medium() "Ambient medium" annotation(__Dymola_choicesAllMatching = true);
            output Modelica.SIunits.Temperature T "Outlet temperature of medium";
            output Modelica.SIunits.Temperature T_port = flowPort.h / medium.cp "Temperature at flowPort_a";
          protected 
            Modelica.SIunits.SpecificEnthalpy h = medium.cp * T;
          public 
            Interfaces.FlowPort_a flowPort(final medium = medium) annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));

          equation 
            flowPort.H_flow = semiLinear(flowPort.m_flow, flowPort.h, h);
          end Ambient;
          partial model AbsoluteSensor "Partial model of absolute sensor"
            annotation(Documentation(info = "<HTML>
Partial model for an absolute sensor (pressure/temperature).<br>
Pressure, mass flow, temperature and enthalpy flow of medium are not affected.
</HTML>"));
            parameter FluidHeatFlow.Media.Medium medium = FluidHeatFlow.Media.Medium() "Sensor's medium" annotation(__Dymola_choicesAllMatching = true);
            Interfaces.FlowPort_a flowPort(final medium = medium) annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
            Modelica.Blocks.Interfaces.RealOutput y annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -70,70},{70, -70}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,70},{0,40}}, color = {0,0,0}),Line(points = {{22.9,32.8},{40.2,57.3}}, color = {0,0,0}),Line(points = {{ -22.9,32.8},{ -40.2,57.3}}, color = {0,0,0}),Line(points = {{37.6,13.7},{65.8,23.9}}, color = {0,0,0}),Line(points = {{ -37.6,13.7},{ -65.8,23.9}}, color = {0,0,0}),Line(points = {{0,0},{9.02,28.6}}, color = {0,0,0}),Polygon(points = {{ -0.48,31.6},{18,26},{18,57.2},{ -0.48,31.6}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -5,5},{5, -5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{70,0},{100,0}}),Text(extent = {{ -150,130},{150,70}}, lineColor = {0,0,255}, textString = "%name")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

          equation 
            flowPort.m_flow = 0;
            flowPort.H_flow = 0;
          end AbsoluteSensor;
          partial model RelativeSensor "Partial model of relative sensor"
            annotation(Documentation(info = "<HTML>
Partial model for a relative sensor (pressure drop/temperature difference).<br>
Pressure, mass flow, temperature and enthalpy flow of medium are not affected.
</HTML>"));
            parameter FluidHeatFlow.Media.Medium medium = FluidHeatFlow.Media.Medium() "Sensor's medium" annotation(__Dymola_choicesAllMatching = true);
            Interfaces.FlowPort_a flowPort_a(final medium = medium) annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
            Interfaces.FlowPort_b flowPort_b(final medium = medium) annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
            Modelica.Blocks.Interfaces.RealOutput y annotation(Placement(transformation(origin = {0, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -70,70},{70, -70}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,70},{0,40}}, color = {0,0,0}),Line(points = {{22.9,32.8},{40.2,57.3}}, color = {0,0,0}),Line(points = {{ -22.9,32.8},{ -40.2,57.3}}, color = {0,0,0}),Line(points = {{37.6,13.7},{65.8,23.9}}, color = {0,0,0}),Line(points = {{ -37.6,13.7},{ -65.8,23.9}}, color = {0,0,0}),Line(points = {{0,0},{9.02,28.6}}, color = {0,0,0}),Polygon(points = {{ -0.48,31.6},{18,26},{18,57.2},{ -0.48,31.6}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -5,5},{5, -5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{70,0},{90,0}}, color = {0,0,0}),Line(points = {{0, -100},{0, -70}}),Text(extent = {{ -150,130},{150,70}}, lineColor = {0,0,255}, textString = "%name")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

          equation 
            flowPort_a.m_flow = 0;
            flowPort_b.m_flow = 0;
            flowPort_a.H_flow = 0;
            flowPort_b.H_flow = 0;
          end RelativeSensor;
          partial model FlowSensor "Partial model of flow sensor"
            annotation(Documentation(info = "<HTML>
Partial model for a flow sensor (mass flow/heat flow).<br>
Pressure, mass flow, temperature and enthalpy flow of medium are not affected, but mixing rule is applied.
</HTML>"));
            extends TwoPort(final m = 0, final T0 = 0, final tapT = 1);
            Modelica.Blocks.Interfaces.RealOutput y annotation(Placement(transformation(origin = {0, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -70,70},{70, -70}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,70},{0,40}}, color = {0,0,0}),Line(points = {{22.9,32.8},{40.2,57.3}}, color = {0,0,0}),Line(points = {{ -22.9,32.8},{ -40.2,57.3}}, color = {0,0,0}),Line(points = {{37.6,13.7},{65.8,23.9}}, color = {0,0,0}),Line(points = {{ -37.6,13.7},{ -65.8,23.9}}, color = {0,0,0}),Line(points = {{0,0},{9.02,28.6}}, color = {0,0,0}),Polygon(points = {{ -0.48,31.6},{18,26},{18,57.2},{ -0.48,31.6}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -5,5},{5, -5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{70,0},{90,0}}, color = {0,0,0}),Line(points = {{0, -100},{0, -70}}),Text(extent = {{ -150,130},{150,70}}, lineColor = {0,0,255}, textString = "%name")}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));

          equation 
            dp = 0;
            Q_flow = 0;
          end FlowSensor;
        end Partials;
      end Interfaces;
    end FluidHeatFlow;
  end Thermal;
  package Media "Library of media property models"
    extends Modelica.Icons.Library;
    import SI = Modelica.SIunits;
    annotation(Documentation(info = "<HTML>
<p>
This library contains <a href=\"Modelica://Modelica.Media.Interfaces\">interface</a>
definitions for media and the following <b>property</b> models for
single and multiple substance fluids with one and multiple phases:
</p>
<ul>
<li> <a href=\"Modelica://Modelica.Media.IdealGases\">Ideal gases:</a><br>
     1241 high precision gas models based on the
     NASA Glenn coefficients, plus ideal gas mixture models based
     on the same data.</li>
<li> <a href=\"Modelica://Modelica.Media.Water\">Water models:</a><br>
     ConstantPropertyLiquidWater, WaterIF97 (high precision
     water model according to the IAPWS/IF97 standard)</li>
<li> <a href=\"Modelica://Modelica.Media.Air\">Air models:</a><br>
     SimpleAir, DryAirNasa, and MoistAir</li>
<li> <a href=\"Modelica://Modelica.Media.Incompressible\">
     Incompressible media:</a><br>
     TableBased incompressible fluid models (properties are defined by tables rho(T),
     HeatCapacity_cp(T), etc.)</li>
<li> <a href=\"Modelica://Modelica.Media.CompressibleLiquids\">
     Compressible liquids:</a><br>
     Simple liquid models with linear compressibility</li>
</ul>
<p>
The following parts are useful, when newly starting with this library:
<ul>
<li> <a href=\"Modelica://Modelica.Media.UsersGuide\">Modelica.Media.UsersGuide</a>.</li>
<li> <a href=\"Modelica://Modelica.Media.UsersGuide.MediumUsage\">Modelica.Media.UsersGuide.MediumUsage</a>
     describes how to use a medium model in a component model.</li>
<li> <a href=\"Modelica://Modelica.Media.UsersGuide.MediumDefinition\">
     Modelica.Media.UsersGuide.MediumDefinition</a>
     describes how a new fluid medium model has to be implemented.</li>
<li> <a href=\"Modelica://Modelica.Media.UsersGuide.ReleaseNotes\">Modelica.Media.UsersGuide.ReleaseNotes</a>
     summarizes the changes of the library releases.</li>
<li> <a href=\"Modelica://Modelica.Media.Examples\">Modelica.Media.Examples</a>
     contains examples that demonstrate the usage of this library.</li>
</ul>
<p>
Copyright &copy; 1998-2009, Modelica Association.
</p>
<p>
<i>This Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p><br>
</HTML>"));
    package UsersGuide "User's Guide of Media Library"
      annotation(__Dymola_DocumentationClass = true, Documentation(info = "<HTML>
<p>
Library <b>Modelica.Media</b> is a <b>free</b> Modelica package providing
a standardized interface to fluid media models and specific
media models based on this interface.
A fluid medium model defines <b>algebraic</b> equations
for the intensive thermodynamic variables used in the <b>mass</b>
and <b>energy</b> balance of component models. Optionally, additional
medium properties can be computed such as dynamic viscosity or thermal
conductivity. Medium models are defined for <b>single</b> and
<b>multiple substance</b> fluids with <b>one</b> and
<b>multiple phases</b>.
</p>
<p>
A large part of the library provides specific medium models
that can be directly utilized. This library can be used in
all types of Modelica fluid libraries that may have different connectors
and design philosophies. It is particularily utilized
in the Modelica_Fluid library (the Modelica_Fluid library is currently
under development to provide 1D thermo-fluid flow components for
single and multiple substance flow with one and multiple phases).
The Modelica.Media library has the following
main features:
</p>
<ul>
<li> Balance equations and media model equations
     are decoupled.
     This means that the used medium model does usually not have an
     influence on how the balance equations are formulated.
     For example, the same balance equations are used for media
     that use pressure and temperature, or pressure and specific
     enthalpy as independent variables, as well as for
     incompressible and compressible media models.
     A Modelica tool will have enough information to
     generate as efficient code as a traditional
     (coupled) definition. This feature is described in more
     detail in section
     <a href=\"Modelica://Modelica.Media.UsersGuide.MediumDefinition.StaticStateSelection\">Static State Selection</a>.</li>
<li> Optional variables, such as dynamic viscosity, are only computed if
     needed in the corresponding component.</li>
<li> The independent variables of a medium model do not
     influence the definition of a fluid connector port.
     Especially, the media models are implemented in such a way
     that a connector may have the minimum number of independent
     medium variables in a connector and still get the same
     efficiency as if all medium variables are passed by the
     connector from one component to the next one (the latter
     approach has the restriction that a fluid port can only
     connect two components and not more). Note, the Modelica_Fluid
     library uses the first approach, i.e., having a set of
     independent medium variables in a connector.</li>
<li> The medium models are implemented with regards to
     efficient dynamic simulation. For example, two phase
     medium models trigger state events at phase boundaries
     (because the medium variables are not differentiable
     at this point).</li>
</ul>
<p>
This User's Guide has the following main parts:
</p>
<ul>
<li> <a href=\"Modelica://Modelica.Media.UsersGuide.MediumUsage\">Medium usage</a>
     describes how to use a medium model from
     this library in a component model.</li>
<li> <a href=\"Modelica://Modelica.Media.UsersGuide.MediumDefinition\">Medium definition</a>
     describes how a new fluid medium
     model has to be implemented.</li>
<li> <a href=\"Modelica://Modelica.Media.UsersGuide.ReleaseNotes\">ReleaseNotes</a>
     summarizes the changes of the library releases.</li>
<li><a href=\"Modelica://Modelica.Media.UsersGuide.Future\">Future</a>
    provides information about the upcoming version 3.1 of Modelica.Media and
    some changes to this version to enable forward compatibility.</li>
<li><a href=\"Modelica://Modelica.Media.UsersGuide.Contact\">Contact</a>
    provides information about the authors of the library as well as
    acknowledgements.</li>
</ul>
</HTML>"));
      package MediumUsage "Medium usage"
        annotation(__Dymola_DocumentationClass = true, Documentation(info = "<HTML>
<p>
Content:
</p>
<ol>
<li> <a href=\"Modelica://Modelica.Media.UsersGuide.MediumUsage.BasicUsage\">
      Basic usage of medium model</a></li>
<li> <a href=\"Modelica://Modelica.Media.UsersGuide.MediumUsage.BalanceVolume\">
      Medium model for a balance volume</a></li>
<li> <a href=\"Modelica://Modelica.Media.UsersGuide.MediumUsage.ShortPipe\">
      Medium model for a pressure loss</a></li>
<li> <a href=\"Modelica://Modelica.Media.UsersGuide.MediumUsage.OptionalProperties\">
     Optional medium properties</a></li>
<li> <a href=\"Modelica://Modelica.Media.UsersGuide.MediumUsage.Constants\">
     Constants provided by medium model</a></li>
<li> <a href=\"Modelica://Modelica.Media.UsersGuide.MediumUsage.TwoPhase\">
     Two-phase media</a></li>
<li> <a href=\"Modelica://Modelica.Media.UsersGuide.MediumUsage.Initialization\">
     Initialization</a></li>
</ol>

<p>
A good demonstration how to use the media from Modelica.Media is
given in package Modelica.Media.Examples.Tests. Under
<a href=\"Modelica://Modelica.Media.Examples.Tests.Components\">
Tests.Components</a> the most basic components of a Fluid library
are defined. Under Tests.MediaTestModels these basic components are used to test
all media models with some very simple piping networks.
</p>

</HTML>"));
        class BasicUsage "Basic usage"
          annotation(Documentation(info = "<HTML>
<h4>Basic usage of medium model</h4>
<p>
Media models in Modelica.Media are provided by packages, inheriting from the
partial package Modelica.Media.Interfaces.PartialMedium. Every package defines:
<ul>
<li> Medium <b>constants</b> (such as the number of chemical substances,
     molecular data, critical properties, etc.).
<li> A BaseProperties <b>model</b>, to compute the basic thermodynamic
     properties of the fluid;
<li> <b>setState_XXX</b> functions to compute the thermodynamic state record from
     different input arguments (such as density, temperature, and composition which
     would be setState_dTX);
<li> <b>Functions</b> to compute additional properties (such as saturation
     properties, viscosity, thermal conductivity, etc.).
</ul>
There are - as stated above - two different basic ways of using the Media library which
will be described in more details in the following section. One way is to use the model BaseProperties.
Every instance of BaseProperties for any medium model provides <b>3+nXi
equations</b> for the following <b>5+nXi variables</b> that are declared in
the medium model (nXi is the number of independent mass fractions, see
explanation below):
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Variable</b></td>
      <td valign=\"top\"><b>Unit</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">T</td>
      <td valign=\"top\">K</td>
      <td valign=\"top\">temperature</td></tr>
  <tr><td valign=\"top\">p</td>
      <td valign=\"top\">Pa</td>
      <td valign=\"top\">absolute pressure</td></tr>
  <tr><td valign=\"top\">d</td>
      <td valign=\"top\">kg/m^3</td>
      <td valign=\"top\">density</td></tr>
  <tr><td valign=\"top\">u</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific internal energy</td></tr>
  <tr><td valign=\"top\">h</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific enthalpy (h = u + p/d)</td></tr>
  <tr><td valign=\"top\">Xi[nXi]</td>
      <td valign=\"top\">kg/kg</td>
      <td valign=\"top\">independent mass fractions m_i/m</td></tr>
  <tr><td valign=\"top\">X[nX]</td>
      <td valign=\"top\">kg/kg</td>
      <td valign=\"top\">All mass fractions m_i/m. X is defined in BaseProperties by:<br>
          X = <b>if</b> reducedX <b>then</b> vector([Xi; 1-<b>sum</b>(Xi)])
          <b>else</b> Xi </td></tr>
</table>
<p>
<b>Two</b> variables out of p, d, h, or u, as well as the
<b>mass fractions</b> Xi are the <b>independent</b> variables and the
medium model basically provides equations to compute
the remaining variables, including the full mass fraction vector X
(more details to Xi and X are given further below).
</p>
<p>
In a component, the most basic usage of a medium model is as follows
</p>
<pre>
  <b>model</b> Pump
    <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium
                         \"Medium model\" <b>annotation</b> (__Dymola_choicesAllMatching = <b>true</b>);
    Medium.BaseProperties medium_a \"Medium properties at location a (e.g. port_a)\";
    // Use medium variables (medium_a.p, medium_a.T, medium_a.h, ...)
     ...
  <b>end</b> Pump;
</pre>
<p>
The second way is to use the setState_XXX functions to compute the thermodynamic state
record from which all other thermodynamic state variables can be computed (see
<a href=\"Modelica://Modelica.Media.UsersGuide.MediumDefinition.BasicDefinition\">
Basic definition of medium</a> for further details on ThermodynamicState). The setState_XXX functions
accept either X or Xi (see explanation below) and will decide internally which of these two compositions
is provided by the user. The four fundamental setState_XXX functions are provided in PartialMedium
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Function</b></td>
      <td valign=\"top\"><b>Description</b></td>
      <td valign=\"top\"><b>Short-form for<br>single component medium</b></td></tr>
  <tr><td valign=\"top\">setState_dTX</td>
      <td valign=\"top\">computes ThermodynamicState from density, temperature, and composition X or Xi</td>
      <td valign=\"top\">setState_dT</td></tr>
  <tr><td valign=\"top\">setState_phX</td>
      <td valign=\"top\">computes ThermodynamicState from pressure, specific enthalpy, and composition X or Xi</td>
      <td valign=\"top\">setState_ph</td></tr>
  <tr><td valign=\"top\">setState_psX</td>
      <td valign=\"top\">computes ThermodynamicState from pressure, specific entropy, and composition X or Xi</td>
      <td valign=\"top\">setState_ps</td></tr>
  <tr><td valign=\"top\">setState_pTX</td>
      <td valign=\"top\">computes ThermodynamicState from pressure, temperature, and composition X or Xi</td>
      <td valign=\"top\">setState_pT</td></tr>
</table>
<p>
The simple example that explained the basic usage of BaseProperties would then become
<pre>
  <b>model</b> Pump
    <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium
                         \"Medium model\" <b>annotation</b> (__Dymola_choicesAllMatching = <b>true</b>);
    Medium.ThermodynamicState state_a \"Thermodynamic state record at location a (e.g. port_a)\";
    // Compute medium variables from thermodynamic state record (pressure(state_a), temperature(state_a),
    // specificEnthalpy(state_a), ...)
    ...
  <b>end</b> Pump;
</pre>
<p>
All media models are directly or indirectly a subpackage of package
Modelica.Media.Interfaces.PartialMedium. Therefore,
a medium model in a component should inherit from this
partial package. Via the annotation \"__Dymola_choicesAllMatching = true\" it
is defined that the tool should display a selection box with
all loaded packages that inherit from PartialMedium. An example
is given in the next figure:
</p>
<IMG SRC=\"../Images/Media/UsersGuide/mediumMenu.png\" ALT=\"medium selection menu\">
<p>
A selected medium model leads, e.g., to the following equation:
</p>
<pre>
  Pump pump(<b>redeclare package</b> Medium = Modelica.Media.Water.SimpleLiquidWater);
</pre>
<p>
Usually, a medium model is associated with the variables of a
fluid connector. Therefore, equations have to be defined in a model
that relate the variables in the connector with the variables
in the medium model:
</p>
<pre>
  <b>model</b> Pump
    <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium
                         \"Medium model\" <b>annotation</b> (__Dymola_choicesAllMatching = <b>true</b>);
    Medium.BaseProperties medium_a \"Medium properties of port_a\";
    // definition of the fluid port port_a
     ...
  <b>equation</b>
    medium.p = port_a.p;
    medium.h = port_a.h;
    medium.Xi = port_a.Xi;
     ...
  <b>end</b> Pump;
</pre>
in the case of using BaseProperties or
<pre>
  <b>model</b> Pump
    <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium
                         \"Medium model\" <b>annotation</b> (__Dymola_choicesAllMatching = <b>true</b>);
    Medium.ThermodynamicState state_a \"Thermodynamic state record of medium at port_a\";
    // definition of the fluid port port_a
     ...
  <b>equation</b>
    state_a = Medium.setState_phX(port_a.p, port_a.h, port_a.Xi) // if port_a contains the variables
                                                                 // p, h, and Xi
     ...
  <b>end</b> Pump;
</pre>
in the case of using ThermodynamicState.
<p>
If a component model shall treat both single and multiple
substance fluids, equations for the mass fractions have to be
present (above: medium.Xi = port_a.Xi) in the model. According
to the Modelica semantics, the equations of the mass fractions
are ignored, if the dimension of Xi is zero, i.e., for a single-component
medium. Note, by specific techniques sketched in section
\"Medium definition\", the independent variables in the medium model
need not to be the same as the variables in the connector and still
get the same efficiency, as if the same variables would be used.
</p>

<p>
If a fluid consists of a single
substance, <b>nXi = 0</b> and the vector of mass fractions Xi is not
present. If a fluid consists of nS substances,
the medium model may define the number of independent
mass fractions <b>nXi</b> to be <b>nS</b>, <b>nS-1</b>, or zero.
In all cases, balance equations for nXi substances have to be
given in the corresponding component (see discussion below).
Note, that if nXi = nS, the constraint \"sum(Xi)=1\" between the mass
fractions is <b>not</b> present in the model; in that case, it is necessary to
provide consistent start values for Xi such that sum(Xi) = 1.
</p>

<p>
The reason for this definition of Xi is that a fluid component library
can be implemented by using only the independent mass fractions Xi and
then via the medium it is defined how Xi is interpreted:
</p>

<ul>
<li> If Xi = nS, then the constraint equation sum(X) = 1 is neglected
     during simulation. By making sure that the initial conditions of X
     fulfill this constraint, it can usually be guaranteed that small
     errors in sum(X) = 1 remain small although this constraint equation is
     not explicitly used during the simulation. This approach is usually useful
     if components of the mixture can become very small. If such a small
     quantity is computed via the equation 1 - sum(X[1:nX-1]), there might
     be large numerical errors and it is better to compute it via
     the corresponding balance equation.</li>
<li> If Xi = nS-1, then the true independent mass fractions are used
     in the fluid component and the last component of X is computed via
     X[nX] = 1 - sum(Xi). This is useful for, e.g., MoistAir, where the
     number of states should be as small as possible without introducing
     numerical problems.</li>
<li> If Xi = 0, then the reference value of composition reference_X is
     assumed. This case is useful to avoid composition states in all
     the cases when the composition will always be constant, e.g. with
     circuits having fixed composition sources.
</ul>

<p>
The full vector of mass fractions <b>X[nX]</b> is computed in
PartialMedium.BaseProperties based on Xi, reference_X, and the information whether Xi = nS or nS-1. For single-substance media, nX = 0, so there's also no X vector. For multiple-substance media, nX = nS, and X always contains the full vector of mass fractions. In order to reduce confusion for the user of a fluid component library, \"Xi\" has the annotation \"HideResult=true\", meaning, that this variable is not shown in the plot window. Only X is shown in the plot window and this vector always contains all mass fractions.
</p>
</HTML>"));
        end BasicUsage;
        class BalanceVolume "Balance volume"
          annotation(Documentation(info = "<HTML>
<p>
Fluid libraries usually have balance volume components with one fluid connector
port that fulfill the mass and energy balance and on a different grid components that
fulfill the momentum balance. A balance volume component, called junction
volume below, should be primarily implemented in the following way
(see also the implementation in
<a href=\"Modelica://Modelica.Media.Examples.Tests.Components.PortVolume\">
Modelica.Media.Examples.Tests.Components.PortVolume</a>):
</p>
<pre>
  <b>model</b> JunctionVolume
    <b>import</b> SI=Modelica.SIunits;
    <b>import</b> Modelica.Media.Examples.Tests.Components.FluidPort_a;

    <b>parameter</b> SI.Volume V = 1e-6 \"Fixed size of junction volume\";
    <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium
                           \"Medium model\" <b>annotation</b> (__Dymola_choicesAllMatching = <b>true</b>);

    FluidPort_a port(<b>redeclare package</b> Medium = Medium);
    Medium.BaseProperties medium(preferredMediumStates = <b>true</b>);

    SI.Energy U               \"Internal energy of junction volume\";
    SI.Mass   M               \"Mass of junction volume\";
    SI.Mass   MX[Medium.nXi] \"Independent substance masses of junction volume\";
  <b>equation</b>
    medium.p   = port.p;
    medium.h   = port.h;
    medium.Xi = port.Xi;

    M  = V*medium.d;                  // mass of JunctionVolume
    MX = M*medium.Xi;                // mass fractions in JunctionVolume
    U  = M*medium.u;                  // internal energy in JunctionVolume

    <b>der</b>(M)  = port.m_flow;    // mass balance
    <b>der</b>(MX) = port.mX_flow;   // substance mass balance
    <b>der</b>(U)  = port.H_flow;    // energy balance
  <b>end</b> JunctionVolume;
</pre>
<p>
Assume the Modelica.Media.Air.SimpleAir medium model is used with
the JunctionVolume model above. This medium model uses pressure p
and temperature T as independent variables. If the flag
\"preferredMediumStates\" is set to <b>true</b> in the declaration
of \"medium\", then the independent variables of this medium model
get the attribute \"stateSelect = StateSelect.prefer\", i.e., the
Modelica translator should use these variables as states, if this
is possible. Basically, this means that
constraints between the
potential states p,T and the potential states U,M are present.
A Modelica tool will therefore <b>automatically</b>
differentiate medium equations and will use the following
equations for code generation (note the equations related to X are
removed, because SimpleAir consists of a single substance only):
</p>
<pre>
    M  = V*medium.d;
    U  = M*medium.u;

    // balance equations
    <b>der</b>(M)  = port.m_flow;
    <b>der</b>(U)  = port.H_flow;

    // abbreviations introduced to get simpler terms
    p = medium.p;
    T = medium.T;
    d = medium.d;
    u = medium.u;
    h = medium.h;

    // medium equations
    d = fd(p,T);
    h = fh(p,T);
    u = h - p/d;

    // equations derived <b>automatically</b> by a Modelica tool due to index reduction
    <b>der</b>(U) = <b>der</b>(M)*u + M*<b>der</b>(u);
    <b>der</b>(M) = V*<b>der</b>(d);
    <b>der</b>(u) = <b>der</b>(h) - <b>der</b>(p)/d - p/<b>der</b>(d);
    <b>der</b>(d) = <b>der</b>(fd,p)*<b>der</b>(p) + <b>der</b>(fd,T)*<b>der</b>(T);
    <b>der</b>(h) = <b>der</b>(fh,p)*<b>der</b>(p) + <b>der</b>(fd,T)*<b>der</b>(T);
</pre>
<p>
Note, that \"der(y,x)\" is an operator that characterizes
in the example above the partial derivative of y with respect to x
(this operator will be included in one of the next Modelica language
releases).
All media models in this library are written in such a way that
at least the partial derivatives of the medium variables with
respect to the independent variables are provided, either because
the equations are directly given (= symbolic differentiation is possible)
or because the derivative of the corresponding function (such as fd above)
is provided. A Modelica tool will transform the equations above
in differential equations with p and T as states, i.e., will
generate equations to compute <b>der</b>(p) and <b>der</b>(T) as function of p and T.
</p>

<p>
Note, when preferredMediumStates = <b>false</b>, no differentiation
will take place and the Modelica translator will use the variables
appearing differentiated as states, i.e., M and U. This has the
disadvantage that for many media non-linear systems of equations are
present to compute the intrinsic properties p, d, T, u, h from
M and U.
</p>
</HTML>"));
        end BalanceVolume;
        class ShortPipe "Short pipe"
          annotation(Documentation(info = "<HTML>
<p>
Fluid libraries have components with two ports that store
neither mass nor energy and fulfill the
momentum equation between their two ports, e.g., a short pipe. In most
cases this means that an equation is present relating the pressure
drop between the two ports and the mass flow rate from one to the
other port. Since no mass or energy is stored, no differential
equations for thermodynamic variables are present. A component model of this type
has therefore usually the following structure
(see also the implementation in
<a href=\"Modelica://Modelica.Media.Examples.Tests.Components.ShortPipe\">
Modelica.Media.Examples.Tests.Components.ShortPipe</a>):
</p>
<pre>
  <b>model</b> ShortPipe
    <b>import</b> SI=Modelica.SIunits;
    <b>import</b> Modelica.Media.Examples.Tests.Components;

    // parameters defining the pressure drop equation

    <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium
                           \"Medium model\" <b>annotation</b> (__Dymola_choicesAllMatching = <b>true</b>);

    Component.FluidPort_a port_a (<b>redeclare package</b> Medium = Medium);
    Component.FluidPort_b port_b (<b>redeclare package</b> Medium = Medium);

    SI.Pressure dp = port_a.p - port_b.p \"Pressure drop\";
    Medium.BaseProperties medium_a \"Medium properties in port_a\";
    Medium.BasePropreties medium_b \"Medium properties in port_b\";
  <b>equation</b>
    // define media models of the ports
    medium_a.p   = port_a.p;
    medium_a.h   = port_a.h;
    medium_a.Xi = port_a.Xi;

    medium_b.p   = port_b.p;
    medium_b.h   = port_b.h;
    medium_b.Xi = port_b.Xi;

    // Handle reverse and zero flow (semiLinear is a built-in Modelica operator)
    port_a.H_flow   = <b>semiLinear</b>(port_a.m_flow, port_a.h, port_b.h);
    port_a.mXi_flow = <b>semiLinear</b>(port_a.m_flow, port_a.Xi, port_b.Xi);

    // Energy, mass and substance mass balance
    port_a.H_flow + port_b.H_flow = 0;
    port_a.m_flow + port_b.m_flow = 0;
    port_a.mXi_flow + port_b.mXi_flow = zeros(Medium.nXi);

    // Provide equation: port_a.m_flow = f(dp)
  <b>end</b> ShortPipe;
</pre>

<p>
The <b>semiLinear</b>(..) operator is basically defined as:
</p>
<pre>
    semiLinear(m_flow, ha, hb) = if m_flow &ge; 0 then m_flow*ha else m_flow*hb;
</pre>

<p>
that is, it computes the enthalpy flow rate either from the port_a or
from the port_b properties, depending on flow direction. The exact
details of this operator are given in
<a href=\"Modelica://ModelicaReference.Operators.SemiLinear\">
ModelicaReference.Operators.SemiLinear</a>. Especially, rules
are defined in the Modelica specification that m_flow = 0 can be treated
in a \"meaningful way\". Especially, if n fluid components (such as pipes)
are connected together and the fluid connector from above is used,
a linear system of equations appear between
medium1.h, medium2.h, medium3.h, ..., port1.h, port2.h, port3.h, ...,
port1.H_flow, port2.H_flow, port3.H_flow, .... The rules for the
semiLinear(..) operator allow the following solution of this
linear system of equations:
</p>

<ul>
<li> n = 2 (two components are connected):<br>
     The linear system of equations can be analytically solved
     with the result
     <pre>
     medium1.h = medium2.h = port1.h = port2.h
     0 = port1.H_flow + port2.H_flow
     </pre>
     Therefore, no problems with zero mass flow rate are present.</li>

<li> n &gt; 2 (more than two components are connected together):<br>
     The linear system of equations is solved numerically during simulation.
     For m_flow = 0, the linear system becomes singular and has an
     infinite number of solutions. The simulator could use the solution t
     that is closest to the solution in the previous time step
     (\"least squares solution\"). Physically, the solution is determined
     by diffusion which is usually neglected. If diffusion is included,
     the linear system is regular.</li>
</ul>

</HTML>"));
        end ShortPipe;
        class OptionalProperties "Optional properties"
          annotation(Documentation(info = "<HTML>
<p>
In some cases additional medium properties are needed.
A component that needs these optional properties has to call
one of the functions listed in the following table. They are
defined as partial functions within package
<a href=\"Modelica://Modelica.Media.Interfaces.PartialMedium\">PartialMedium</a>,
and then (optionally) implemented in actual medium packages.
If a component calls such an optional function and the
medium package does not provide a new implementation for this
function, an error message is printed at translation time,
since the function is \"partial\", i.e., incomplete.
The argument of all functions is the <b>state</b> record,
automatically defined by the BaseProperties model or specifically computed using the
setState_XXX functions, which contains the
minimum number of thermodynamic variables needed to compute all the additional
properties. In the table it is assumed that there is a declaration of the
form:
</p>
<pre>
   <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium;
   Medium.ThermodynamicState state;
</pre>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Function call</b></td>
      <td valign=\"top\"><b>Unit</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">Medium.dynamicViscosity(state)</b></td>
      <td valign=\"top\">Pa.s</td>
      <td valign=\"top\">dynamic viscosity</td></tr>
  <tr><td valign=\"top\">Medium.thermalConductivity(state)</td>
      <td valign=\"top\">W/(m.K)</td>
      <td valign=\"top\">thermal conductivity</td></tr>
  <tr><td valign=\"top\">Medium.prandtlNumber(state)</td>
      <td valign=\"top\">1</td>
      <td valign=\"top\">Prandtl number</td></tr>
  <tr><td valign=\"top\">Medium.specificEntropy(state)</td>
      <td valign=\"top\">J/(kg.K)</td>
      <td valign=\"top\">specific entropy</td></tr>
  <tr><td valign=\"top\">Medium.specificHeatCapacityCp(state)</td>
      <td valign=\"top\">J/(kg.K)</td>
      <td valign=\"top\">specific heat capacity at constant pressure</td></tr>
  <tr><td valign=\"top\">Medium.specificHeatCapacityCv(state)</td>
      <td valign=\"top\">J/(kg.K)</td>
      <td valign=\"top\">specific heat capacity at constant density</td></tr>
  <tr><td valign=\"top\">Medium.isentropicExponent(state)</td>
      <td valign=\"top\">1</td>
      <td valign=\"top\">isentropic exponent</td></tr>
  <tr><td valign=\"top\">Medium.isentropicEnthatlpy(pressure, state)</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">isentropic enthalpy</td></tr>
  <tr><td valign=\"top\">Medium.velocityOfSound(state)</td>
      <td valign=\"top\">m/s</td>
      <td valign=\"top\">velocity of sound</td></tr>
  <tr><td valign=\"top\">Medium.isobaricExpansionCoefficient(state)</td>
      <td valign=\"top\">1/K</td>
      <td valign=\"top\">isobaric expansion coefficient</td></tr>
  <tr><td valign=\"top\">Medium.isothermalCompressibility(state)</td>
      <td valign=\"top\">1/Pa</td>
      <td valign=\"top\">isothermal compressibility</td></tr>
  <tr><td valign=\"top\">Medium.density_derp_h(state)</td>
      <td valign=\"top\">kg/(m3.Pa)</td>
      <td valign=\"top\">derivative of density by pressure at constant enthalpy</td></tr>
  <tr><td valign=\"top\">Medium.density_derh_p(state)</td>
      <td valign=\"top\">kg2/(m3.J)</td>
      <td valign=\"top\">derivative of density by enthalpy at constant pressure</td></tr>
  <tr><td valign=\"top\">Medium.density_derp_T(state)</td>
      <td valign=\"top\">kg/(m3.Pa)</td>
      <td valign=\"top\">derivative of density by pressure at constant temperature</td></tr>
  <tr><td valign=\"top\">Medium.density_derT_p(state)</td>
      <td valign=\"top\">kg/(m3.K)</td>
      <td valign=\"top\">derivative of density by temperature at constant pressure</td></tr>
  <tr><td valign=\"top\">Medium.density_derX(state)</td>
      <td valign=\"top\">kg/m3</td>
      <td valign=\"top\">derivative of density by mass fraction</td></tr>
  <tr><td valign=\"top\">Medium.molarMass(state)</td>
      <td valign=\"top\">kg/mol</td>
      <td valign=\"top\">molar mass</td></tr>
</table>
<p>
There are also some short forms provided for user convenience that allow the computation of certain
thermodynamic state variables without using the ThermodynamicState record explicitly. Those short forms
are for example useful to compute consistent start values in the initial equation section. Let's
consider the function temperature_phX(p,h,X) as an exmaple. This function computes the temperature
from pressure, specific enthalpy, and composition X (or Xi) and is a short form for writing
</p>
<pre>
  temperature(setState_phX(p,h,X))
</pre>
<p>
The following functions are predefined in PartialMedium (other functions can be added in the actual
medium implementation package if they are useful)
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\">Medium.specificEnthalpy_pTX(p,T,X)</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">Specific enthalpy at p, T, X </td></tr>
  <tr><td valign=\"top\">Medium.temperature_phX(p,h,X)</td>
      <td valign=\"top\">K</td>
      <td valign=\"top\">Temperature at p, h, X</td></tr>
  <tr><td valign=\"top\">Medium.density_phX(p,h,X)</td>
      <td valign=\"top\">kg/m</td>
      <td valign=\"top\">Density at p, h, X</td></tr>
  <tr><td valign=\"top\">Medium.temperature_psX(p,s,X)</td>
      <td valign=\"top\">K</td>
      <td valign=\"top\">Temperature at p, s, X</td></tr>
  <tr><td valign=\"top\">Medium.specificEnthalpy_psX(p,s,X)</td>
      <td valign=\"top\">J/(kg.K)</td>
      <td valign=\"top\">Specific entropy at p, s, X</td></tr>
</table>
<p>
Assume for example that the dynamic viscosity eta is needed in
the pressure drop equation of a short pipe. Then, the
model of a short pipe has to be changed to:
</p>
<pre>
  <b>model</b> ShortPipe
      ...
    Medium.BaseProperties medium_a \"Medium properties in port_a\";
    Medium.BaseProperties medium_b \"Medium properties in port_b\";
      ...
    Medium.DynamicViscosity eta;
      ...
    eta = <b>if</b> port_a.m_flow &gt; 0 <b>then</b>
               Medium.dynamicViscosity(medium_a.state)
          <b>else</b>
               Medium.dynamicViscosity(medium_b.state);
    // use eta in the pressure drop equation: port_a.m_flow = f(dp, eta)
  <b>end</b> ShortPipe;
</pre>

<p>
Note, \"Medium.DynamicViscosity\" is a type defined in Modelica.Interfaces.PartialMedium
as
</p>

<pre>
  <b>import</b> SI = Modelica.SIunits;
  <b>type</b> DynamicViscosity = SI.DynamicViscosity (
                                     min=0,
                                     max=1.e8,
                                     nominal=1.e-3,
                                     start=1.e-3);
</pre>

<p>
Every medium model may modify the attributes, to provide, e.g.,
min, max, nominal, and start values adapted to the medium.
Also, other types, such as AbsolutePressure, Density, MassFlowRate,
etc. are defined in PartialMedium. Whenever possible, these medium
specific types should be used in a model in order that medium information,
e.g., about nominal or start values, are automatically utilized.
</p>

</pre>

</HTML>"));
        end OptionalProperties;
        class Constants "Constants"
          annotation(Documentation(info = "<HTML>
<p>
Every medium model provides the following <b>constants</b>. For example,
if a medium is declared as:
</p>
<pre>
   <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium;
</pre>
<p>
then constants \"Medium.mediumName\", \"Medium.nX\", etc. are defined:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Type</b></td>
      <td valign=\"top\"><b>Name</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">String</td><td valign=\"top\">mediumName</td>
      <td valign=\"top\">Unique name of the medium (is usually used to check whether
          the media in different components connected together
          are the same, by providing Medium.mediumName as quantity
          attribute of the mass flow rate in the connector)</td></tr>
  <tr><td valign=\"top\">String</td><td valign=\"top\">substanceNames[nS]</td>
      <td valign=\"top\">Names of the substances that make up the medium.
          If only one substance is present, substanceNames = {mediumName}.</td></tr>
  <tr><td valign=\"top\">String</td><td valign=\"top\">extraPropertiesNames[nC]</td>
      <td valign=\"top\">Names of the extra transported substances, outside of mass and
          energy balances.</td></tr>
  <tr><td valign=\"top\">Boolean</td><td valign=\"top\">singleState</td>
      <td valign=\"top\">= <b>true</b>, if u and d are not a function of pressure, and thus only
          a function of a single thermal variable (temperature or enthalpy) and
          of Xi for a multiple substance medium. Usually, this flag is
          <b>true</b> for incompressible media. It is used in a model to determine
          whether 1+nXi (singleState=<b>true</b>) or 2+nXi (singleState=<b>false</b>)
          initial conditions have to be provided for a volume element that
          contains mass and energy balance.</td></tr>
  <tr><td valign=\"top\">AbsolutePressure</td><td valign=\"top\">reference_p</td>
      <td valign=\"top\">Reference pressure for the medium</td></tr>
  <tr><td valign=\"top\">MassFraction</td><td valign=\"top\">reference_X[nX]</td>
      <td valign=\"top\">Reference composition for the medium</td></tr>
  <tr><td valign=\"top\">AbsolutePressure</td><td valign=\"top\">p_default</td>
      <td valign=\"top\">Default value for pressure of medium (for initialization)</td></tr>
  <tr><td valign=\"top\">Temperature</td><td valign=\"top\">T_default</td>
      <td valign=\"top\">Default value for temperature of medium (for initialization)</td></tr>
  <tr><td valign=\"top\">SpecificEnthalpy</td><td valign=\"top\">h_default</td>
      <td valign=\"top\">Default value for specific enthalpy of medium (for initialization)</td></tr>
  <tr><td valign=\"top\">MassFraction</td><td valign=\"top\">X_default[nX]</td>
      <td valign=\"top\">Default value for mass fractions of medium (for initialization)</td></tr>
  <tr><td valign=\"top\">Integer</td><td valign=\"top\">nS</td>
      <td valign=\"top\">number of substances contained in the medium.</td></tr>
  <tr><td valign=\"top\">Integer</td><td valign=\"top\">nX</td>
      <td valign=\"top\">Size of the full mass fraction vector X nX=nS.</td></tr>
  <tr><td valign=\"top\">Integer</td><td valign=\"top\">nXi</td>
      <td valign=\"top\">Number of independent mass fractions. If there is a single substance,
          then nXi = 0. </td></tr>
  <tr><td valign=\"top\">Boolean</td><td valign=\"top\">reducedX</td>
      <td valign=\"top\">= <b>true</b>, if the medium has a single substance, or if the medium model
          has multiple substances and contains the equation sum(X) = 1.
          In both cases, nXi = nS - 1 (unless fixedX = true).<br>
          = <b>false</b>, if the medium has multiple substances and does not contain the
          equation sum(X)=1, i.e., nXi = nX = nS (unless fixedX = true).
       </td></tr>
  <tr><td valign=\"top\">Boolean</td><td valign=\"top\">fixedX</td>
      <td valign=\"top\">= <b>false</b>: the composition of the medium can vary, and is
          determined by nXi independent mass fractions (see reducedX above).<br>
          = <b>true</b>: the composition of the medium is always reference_X,
          and nXi = 0.</td></tr>
  <tr><td valign=\"top\">FluidConstants</td><td valign=\"top\">fluidConstants[nS]</td>
      <td valign=\"top\">Critical, triple, molecular and other
          standard data that are provided for
          every substance of a medium.</td></tr>
</table>

<p>
The record FluidConstants that is defined in PartialMedium contains the following elements
</p>


<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Type</b></td>
      <td valign=\"top\"><b>Name</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>

  <tr><td valign=\"top\">String</td>
      <td valign=\"top\">iupacName</td>
      <td valign=\"top\">complete IUPAC name</td></tr>

  <tr><td valign=\"top\">String</td>
      <td valign=\"top\">casRegistryNumber</td>
      <td valign=\"top\">chemical abstracts sequencing number</td></tr>

  <tr><td valign=\"top\">String</td>
      <td valign=\"top\">chemicalFormula</td>
      <td valign=\"top\">Chemical formula, (brutto, nomenclature according to Hill)</td></tr>

  <tr><td valign=\"top\">String</td>
      <td valign=\"top\">structureFormula</td>
      <td valign=\"top\">Chemical structure formula</td></tr>

  <tr><td valign=\"top\">MolarMass</td>
      <td valign=\"top\">molarMass</td>
      <td valign=\"top\">molar mass</td></tr>
</table>

<p> This record is extended in the partial packages further down the hierarchy (such as
PartialTwoPhaseMedium or PartialMixtureMedium) and may contain some or all of the following
elements </p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\">Temperature</td>
      <td valign=\"top\">criticalTemperature</td>
      <td valign=\"top\">critical temperature</td></tr>

  <tr><td valign=\"top\">AbsolutePressure</td>
      <td valign=\"top\">criticalPressure</td>
      <td valign=\"top\">critical pressure</td></tr>

  <tr><td valign=\"top\">MolarVolume</td>
      <td valign=\"top\">criticalMolarVolume</td>
      <td valign=\"top\">critical molar Volume</td></tr>

  <tr><td valign=\"top\">Real</td>
      <td valign=\"top\">acentricFactor</td>
      <td valign=\"top\">Pitzer acentric factor</td></tr>

  <tr><td valign=\"top\">Temperature</td>
      <td valign=\"top\">triplePointTemperature</td>
      <td valign=\"top\">triple point temperature</td></tr>

  <tr><td valign=\"top\">AbsolutePressure</td>
      <td valign=\"top\">triplePointPressure</td>
      <td valign=\"top\">triple point pressure</td></tr>

  <tr><td valign=\"top\">Temperature</td>
      <td valign=\"top\">meltingPoint</td>
      <td valign=\"top\">melting point at 101325 Pa</td></tr>

  <tr><td valign=\"top\">Temperature</td>
      <td valign=\"top\">normalBoilingPoint</td>
      <td valign=\"top\">normal boiling point (at 101325 Pa)</td></tr>

  <tr><td valign=\"top\">DipoleMoment</td>
      <td valign=\"top\">dipoleMoment</td>
      <td valign=\"top\">dipole moment of molecule in Debye (1 debye = 3.33564e10-30 C.m)</td></tr>

  <tr><td valign=\"top\">Boolean</td>
      <td valign=\"top\">hasIdealGasHeatCapacity</td>
      <td valign=\"top\">true if ideal gas heat capacity is available</td></tr>

  <tr><td valign=\"top\">Boolean</td>
      <td valign=\"top\">hasCriticalData</td>
      <td valign=\"top\">true if critical data are known</td></tr>

  <tr><td valign=\"top\">Boolean</td>
      <td valign=\"top\">hasDipoleMoment</td>
      <td valign=\"top\">true if a dipole moment known</td></tr>

  <tr><td valign=\"top\">Boolean</td>
      <td valign=\"top\">hasFundamentalEquation</td>
      <td valign=\"top\">true if a fundamental equation</td></tr>

  <tr><td valign=\"top\">Boolean</td>
      <td valign=\"top\">hasLiquidHeatCapacity</td>
      <td valign=\"top\">true if liquid heat capacity is available</td></tr>

  <tr><td valign=\"top\">Boolean</td>
      <td valign=\"top\">hasSolidHeatCapacity</td>
      <td valign=\"top\">true if solid heat capacity is available</td></tr>

  <tr><td valign=\"top\">Boolean</td>
      <td valign=\"top\">hasAccurateViscosityData</td>
      <td valign=\"top\">true if accurate data for a viscosity function is available</td></tr>

  <tr><td valign=\"top\">Boolean</td>
      <td valign=\"top\">hasAccurateConductivityData</td>
      <td valign=\"top\">true if accurate data for thermal conductivity is available</td></tr>

  <tr><td valign=\"top\">Boolean</td>
      <td valign=\"top\">hasVapourPressureCurve</td>
      <td valign=\"top\">true if vapour pressure data, e.g. Antoine coefficents are known</td></tr>

  <tr><td valign=\"top\">Boolean</td>
      <td valign=\"top\">hasAcentricFactor</td>
      <td valign=\"top\">true if Pitzer accentric factor is known</td></tr>

  <tr><td valign=\"top\">SpecificEnthalpy</td>
      <td valign=\"top\">HCRIT0</td>
      <td valign=\"top\">Critical specific enthalpy of the fundamental equation</td></tr>

  <tr><td valign=\"top\">SpecificEntropy</td>
      <td valign=\"top\">SCRIT0</td>
      <td valign=\"top\">Critical specific entropy of the fundamental equation</td></tr>

  <tr><td valign=\"top\">SpecificEnthalpy</td>
      <td valign=\"top\">deltah</td>
      <td valign=\"top\">Difference between specific enthalpy model
          (h_m) and f.eq. (h_f) (h_m - h_f)</td></tr>

  <tr><td valign=\"top\">SpecificEntropy</td>
      <td valign=\"top\">deltas</td>
      <td valign=\"top\">Difference between specific enthalpy model (s_m) and f.eq.
          (s_f) (s_m - s_f)</td></tr>

</table>

</HTML>
"));
        end Constants;
        class TwoPhase "Two-phase media"
          annotation(Documentation(info = "<HTML>
<p>
Models for media which can exist in one-phase or two-phase conditions inherit
from <a href=\"Modelica://Modelica.Media.Interfaces.PartialTwoPhaseMedium\">
Modelica.Media.Interfaces.PartialTwoPhaseMedium</a>
(which inherits from PartialMedium). The basic usage of these
media models is the same as described in the previous sections. However, additional
functionalities are provided, which apply only to potentially two-phase media.
</p>
<p>
The following additional medium <b>constants</b> are provided:
</p>
<p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Type</b></td>
      <td valign=\"top\"><b>Name</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">Boolean</td>
      <td valign=\"top\">smoothModel</b></td>
      <td valign=\"top\">If this flag is false (default value), then events are triggered
          whenever the saturation boundary is crossed; otherwise, no events
      are generated.</td></tr>
  <tr><td valign=\"top\">Boolean</td>
      <td valign=\"top\">onePhase</b></td>
      <td valign=\"top\">If this flag is true, then the medium model assumes it will be never
          called in the two-phase region. This can be useful to speed up
      the computations in a two-phase medium, when the user is sure it will
      always work in the one-phase region. Default value: false.</td></tr>
</table>
</p>
<p>
The setState_ph(), setState_ps(), setState_dT() and setState_pT() functions have
one extra input, named <i>phase</i>. If the phase input is not specified, or if
it is given a value of zero, then the setState function will determine the phase,
based on the other input values. An input phase = 1 will force the setState
function to return a state vector corresponding to a one-phase state, while
phase = 2 will force the setState value to return a state vector corresponding
to a two-phase state, as shown in the following example;

</p>
<pre>
   <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialTwoPhaseMedium;
   Medium.ThermodynamicState state, state1, state2;
 <b>equation</b>
   // Set the state, given the pressure and the specific enthalpy
   // the phase is determined by the (p, h) values, and can be retrieved
   // from the state record
   state = Medium.setState_ph(p, h);
   phase = state1.phase;

   // Force the computation of the state with one-phase
   // equations of state, irrespective of the (p, h) values
   state1 = Medium.setState_ph(p, h, 1);

   // Force the computation of the state with 2-phase
   // equations of state, irrespective of the (p, h) values
   state2 = Medium.setState_ph(p, h, 2);
</pre>

<p> This feature can be used for the following purposes:
<ul>
<li> saving computational time, if one knows in advance the phase of the medium;
<li> unambiguously determine the phase, when the two inputs correspond to a point on the saturation boundary (the derivative functions have substantially different values on either side);
<li> get the properties of metastable states, like superheated water or subcooled vapour.
</ul>
</p>

<p>
Many additional optional functions are defined to compute properties of
saturated media, either liquid (bubble point) or vapour (dew point).
The argument to such functions is a SaturationProperties record, which can be
set starting from either the saturation pressure or the saturation temperature,
as shown in the following example.
</p>
<pre>
   <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialTwoPhaseMedium;
   Medium.SaturationProperties sat_p;
   Medium.SaturationProperties sat_T;
 <b>equation</b>
   // Set sat_p to saturation properties at pressure p
   sat_p = Medium.setSat_p(p);

   // Compute saturation properties at pressure p
   saturationTemperature_p = Medium.saturationTemperature_sat(sat_p);
   bubble_density_p =        Medium.bubbleDensity(sat_p);
   dew_enthalpy_p   =        Medium.dewEnthalpy(sat_p);

   // Set sat_T to saturation properties at temperature T
   sat_T = Medium.setSat_T(T);

   // Compute saturation properties at temperature T
   saturationTemperature_T = Medium.saturationPressure_sat(sat_T);
   bubble_density_T =        Medium.bubbleDensity(sat_T);
   dew_enthalpy_T =          Medium.dewEnthalpy(sat_T);
</pre>
</p>
<p>With reference to a model defining a pressure p, a temperature T, and a
SaturationProperties record sat, the following functions are provided:
</p>
<p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Function call</b></td>
      <td valign=\"top\"><b>Unit</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">Medium.saturationPressure(T)</b></td>
      <td valign=\"top\">Pa</td>
      <td valign=\"top\">Saturation pressure at temperature T</td></tr>
  <tr><td valign=\"top\">Medium.saturationTemperature(p)</b></td>
      <td valign=\"top\">K</td>
      <td valign=\"top\">Saturation temperature at pressure p</td></tr>
  <tr><td valign=\"top\">Medium.saturationTemperature_derp(p)</b></td>
      <td valign=\"top\">K/Pa</td>
      <td valign=\"top\">Derivative of saturation temperature with respect to pressure</td></tr>
  <tr><td valign=\"top\">Medium.saturationTemperature_sat(sat)</b></td>
      <td valign=\"top\">K</td>
      <td valign=\"top\">Saturation temperature</td></tr>
  <tr><td valign=\"top\">Medium.saturationPressure_sat(sat)</b></td>
      <td valign=\"top\">Pa</td>
      <td valign=\"top\">Saturation pressure</td></tr>
  <tr><td valign=\"top\">Medium.bubbleEnthalpy(sat)</b></td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">Specific enthalpy at bubble point</td></tr>
  <tr><td valign=\"top\">Medium.dewEnthalpy(sat)</b></td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">Specific enthalpy at dew point</td></tr>
  <tr><td valign=\"top\">Medium.bubbleEntropy(sat)</b></td>
      <td valign=\"top\">J/(kg.K)</td>
      <td valign=\"top\">Specific entropy at bubble point</td></tr>
  <tr><td valign=\"top\">Medium.dewEntropy(sat)</b></td>
      <td valign=\"top\">J/(kg.K)</td>
      <td valign=\"top\">Specific entropy at dew point</td></tr>
  <tr><td valign=\"top\">Medium.bubbleDensity(sat)</b></td>
      <td valign=\"top\">kg/m3</td>
      <td valign=\"top\">Density at bubble point</td></tr>
  <tr><td valign=\"top\">Medium.dewDensity(sat)</b></td>
      <td valign=\"top\">kg/m3</td>
      <td valign=\"top\">Density at dew point</td></tr>
  <tr><td valign=\"top\">Medium.saturationTemperature_derp_sat(sat)</b></td>
      <td valign=\"top\">K/Pa</td>
      <td valign=\"top\">Derivative of saturation temperature with respect to pressure</td></tr>
  <tr><td valign=\"top\">Medium.dBubbleDensity_dPressure(sat)</b></td>
      <td valign=\"top\">kg/(m3.Pa)</td>
      <td valign=\"top\">Derivative of density at bubble point with respect to pressure</td></tr>
  <tr><td valign=\"top\">Medium.dDewDensity_dPressure(sat)</b></td>
      <td valign=\"top\">kg/(m3.Pa)</td>
      <td valign=\"top\">Derivative of density at dew point with respect to pressure</td></tr>
  <tr><td valign=\"top\">Medium.dBubbleEnthalpy_dPressure(sat)</b></td>
      <td valign=\"top\">J/(kg.Pa)</td>
      <td valign=\"top\">Derivative of specific enthalpy at bubble point with respect to pressure</td></tr>
  <tr><td valign=\"top\">Medium.dDewEnthalpy_dPressure(sat)</b></td>
      <td valign=\"top\">J/(kg.Pa)</td>
      <td valign=\"top\">Derivative of specific enthalpy at dew point with respect to pressure</td></tr>
  <tr><td valign=\"top\">Medium.surfaceTension(sat)</b></td>
      <td valign=\"top\">N/m</td>
      <td valign=\"top\">Surface tension between liquid and vapour phase</td></tr>
</table>
</p>
<p>
Sometimes it can be necessary to compute fluid properties in the thermodynamic
plane, just inside or outside the saturation dome. In this case, it is possible
to obtain an instance of a ThermodynamicState state vector, and then use it
to call the additional functions already defined for one-phase media.
</p>
<p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Function call</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">Medium.setBubbleState(sat, phase)</b></td>
      <td valign=\"top\">Obtain the thermodynamic state vector
          corresponding to the bubble point. If phase==1 (default), the state is
      on the one-phase side; if phase==2, the state is on the two-phase
      side </td></tr>
  <tr><td valign=\"top\">Medium.setDewState(sat, phase)</b></td>
      <td valign=\"top\">Obtain the thermodynamic state vector
          corresponding to the dew point. If phase==1 (default), the state is
      on the one-phase side; if phase==2, the state is on the two-phase
      side </td></tr>
  </table>
</p>
<p>
Here are some examples:
</p>
<pre>
   <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialTwoPhaseMedium;
   Medium.SaturationProperties sat;
   Medium.ThermodynamicState   dew_1;    // dew point, one-phase side
   Medium.ThermodynamicState   bubble_2; // bubble point, two phase side
 <b>equation</b>
   // Set sat to saturation properties at pressure p
   sat = setSat_p(p);

   // Compute dew point properties, (default) one-phase side
   dew_1 = setDewState(sat);
   cpDew = Medium.specificHeatCapacityCp(dew_1);
   drho_dp_h_1 = Medium.density_derp_h(dew_1);

   // Compute bubble point properties, two-phase side
   bubble_2    = setBubbleState(sat, 2);
   drho_dp_h_2 = Medium.density_derp_h(bubble_2);
</pre>
</p>
</HTML>
"));
        end TwoPhase;
        class Initialization "Initialization"
          annotation(Documentation(info = "<HTML>

<p>
When a medium model is used in a balance volume, differential
equations for the independent medium variables are present and
therefore initial conditions have to be provided.
The following possibilities exist:
</p>

<h4>Steady state initialization</h4>
<p>
Modelica has currently no language element to define
steady state initialization. In the Modelica simulation
environment Dymola, the option
<pre>
   Advanced.DefaultSteadyStateInitialization = <b>true</b>
</pre>
can be set before translation. Then, missing initial
conditions are provided by automamtically setting appropriate
state derivatives to zero.
</p>

<h4>Explicit start values or initial equations</h4>
<p>
Explicit start values can be defined with the \"start\" and
\"fixed\" attributes. The number of independent variables nx
need to be known which can be deduced from the medium
constants (nx = nXi + <b>if</b> singleState <b>then</b> 1 <b>else</b> 2).
Then, start values or initial equations can be defined
for nx variables (= p, T, d, u, h, Xi) from Medium.BaseProperties,
e.g., in the form:
<pre>
     <b>replaceable</b> package Medium = Medium.Interfaces.PartialMedium;
     Medium.BaseProperties medium1 (p(start=1e5, fixed=<b>not</b> Medium.singleState),
                                    T(start=300, fixed=<b>true</b>));
     Medium.BaseProperties medium2;
  <b>initial equation</b>
     <b>if not</b> Medium.singleState <b>then</b>
        medium2.p = 1e5;
     <b>end if</b>;
     medium2.T = 300;
  <b>equation</b>
  </pre>
If initial conditions are not provided for the independent
medium variables, non-linear systems of equations may
occur to compute the initial values of the independent
medium variables from the provided initial conditions.
</p>

<h4>Guess values</h4>
<p>
If non-linear systems of equations occur during initialization,
e.g., in case of steady state initialization, guess values
for the iteration variables of the non-linear system of equations
have to be provided via the \"start\" attribute (and fixed=false).
Unfortunately, it is usually not known in
advance which variables are selected as iteration variables of
a non-linear system of equations. One of the following possibilies
exist:
</p>
<ul>
<li> Do not supply start values and hope that the medium specific
     types have meaningful start values, such as in \"Medium.AbsolutePressure\"
     </li>
<li> Supply start values on all variables of the BaseProperties model,
     i.e., on p, T, d, u, h, Xi.</li>
<li> Determine the iteration variables of the non-linear systems of
     equations and provide start values for these variables.
     In the Modelica simulation environment Dymola, the iteration
     variables can be determined by setting the command
     <pre>
     Advanced.OutputModelicaCode = <b>true</b>
     </pre>
     and by inspection of the file \"dsmodel.mof\" that is generated
     when this option is set (search for \"nonlinear\").</li>
</ul>
</HTML>"));
        end Initialization;
      end MediumUsage;
      package MediumDefinition "Medium definition"
        annotation(__Dymola_DocumentationClass = true, Documentation(info = "<HTML>
<p>
If a new medium model shall be introduced, copy package
<a href=\"Modelica://Modelica.Media.Interfaces.TemplateMedium\">
Modelica.Media.Interfaces.TemplateMedium</a> to the desired
location, remove the
\"partial\" keyword from the package and provide
the information that is requested in the comments of the
Modelica source.
A more detailed description for the different parts of the
TemplateMedium package is given here:
</p>
<ol>
<li> <a href=\"Modelica://Modelica.Media.UsersGuide.MediumDefinition.BasicStructure\">
      Basic structure of medium interface</a></li>
<li> <a href=\"Modelica://Modelica.Media.UsersGuide.MediumDefinition.BasicDefinition\">
      Basic definition of medium model</a></li>
<li> <a href=\"Modelica://Modelica.Media.UsersGuide.MediumDefinition.MultipleSubstances\">
      Multiple Substances</a></li>
<li> <a href=\"Modelica://Modelica.Media.UsersGuide.MediumDefinition.SpecificEnthalpyAsFunction\">
      Specific enthalpy as function</a></li>
<li> <a href=\"Modelica://Modelica.Media.UsersGuide.MediumDefinition.StaticStateSelection\">
      Static State Selection</a></li>
<li> <a href=\"Modelica://Modelica.Media.UsersGuide.MediumDefinition.TestOfMedium\">
      Test of medium model</a></li>
</ol>
</HTML>"));
        class BasicStructure "Basic structure"
          annotation(Documentation(info = "<HTML>
<p>
A medium model of Modelica.Media is essentially a <b>package</b>
that contains the following definitions:
</p>
<ul>
<li>
Definition of <b>constants</b>, such as the medium name.</li>
<li>
A <b>model</b> in the package that contains the 3 basic
thermodynamic equations that relate the 5+nXi primary medium variables.</li>
<li><b>Optional functions</b> to compute medium properties that are
only needed in certain circumstances, such as dynamic viscosity. These optional
functions need not be provided by every medium model.</li>
<li>
<b>Type</b> definitions, which are adapted to the particular
medium. For example, a type <b>Temperature</b> is defined where the attributes
<b>min</b>
and <b>max</b> define the validity region of the medium, and a suitable default
start value is given. In a device model, it is advisable to use these type
definitions, e.g., for parameters, in order that medium limits are checked as
early as possible, and that iteration variables of non-linear
systems of equations get reasonable start values.</li>
</ul>
<p>
Note, although we use the term <b>medium model</b>, it
is actually a Modelica <b>package</b> that contains all the constants and
definitions required for a complete <b>medium model</b>. The basic interface to a
medium is defined by Modelica.Media.Interfaces.PartialMedium that has the
following structure:</p>
<pre>
<b>partial package</b> PartialMedium
  <b>import</b> SI = Modelica.SIunits;
  <b>constant</b> String           mediumName = \"\";
  <b>constant</b> String           substanceNames[:] = {mediumName};
  <b>constant</b> String           extraPropertiesNames[:] = fill(\"\",0);
  <b>constant</b> Boolean          singleState = <b>false</b>;
  <b>constant</b> Boolean          reducedX = <b>true</b>;
  <b>constant</b> Boolean          fixedX = <b>false</b>;
  <b>constant</b> AbsolutePressure reference_p = 101325;
  <b>constant</b> MassFraction     reference_X[nX]=fill(1/nX,nX);
  <b>constant</b> AbsolutePressure p_default = 101325;
  <b>constant</b> Temperature      T_default = Modelica.SIunits.Conversions.from_degC(20);
  <b>constant</b> SpecificEnthalpy h_default =
                            specificEnthalpy_pTX(p_default, T_default, X_default);
  <b>constant</b> MassFraction     X_default[nX]=reference_X;
  <b>final constant</b> Integer    nS  = size(substanceNames,1);
  <b>final constant</b> Integer    nX  = nS;
  <b>final constant</b> Integer    nXi = <b>if</b> fixedX <b>then</b> 0
                                  <b>else if</b> reducedX <b>or</b> nS == 1
                                  <b>then</b> nS-1 <b>else</b> nS;
  <b>final constant</b> Integer    nC  = size(extraPropertiesNames,1);
  <b>constant</b> FluidConstants[nS] fluidConstants;

  <b>replaceable record</b> BasePropertiesRecord
    AbsolutePressure p;
    Density d;
    Temperature T;
    SpecificEnthalpy h;
    SpecificInternalEnergy u;
    MassFraction[nX] X;
    MassFraction[nXi] Xi;
    SpecificHeatCapacity R;
    MolarMass MM;
  <b>end</b> BasePropertiesRecord;

  <b>replaceable partial model</b> BaseProperties
    <b>extends</b> BasePropertiesRecord;
    ThermodynamicState state;
    <b>parameter</b> Boolean preferredMediumStates=false;
    SI.Conversions.NonSIunits.Temperature_degC T_degC =
       Modelica.SIunits.Conversions.to_degC(T)
    SI.Conversions.NonSIunits.Pressure_bar p_bar =
       Modelica.SIunits.Conversions.to_bar(p)
  <b>equation</b>
    Xi = X[1:nXi];
    <b>if</b> nX > 1 <b>then</b>
       <b>if</b> fixedX <b>then</b>
          X = reference_X;
       <b>elseif</b> reducedX <b>then</b>
          X[nX] = 1 - sum(Xi);
       <b>end if</b>;
    <b>end if</b>;
    // equations such as
    //    d = d(p,T);
    //    u = u(p,T);
    //    h = u + p/d;
    //    state.p = p;
    //    state.T = T;
    // will go here in actual media implementations, but are not present
    // in the base class since the ThermodynamicState record is still empty
   <b>end</b> BaseProperties

  <b>replaceable record</b> ThermodynamicState
     // there are no \"standard\" thermodynamic variables in the base class
     // but they will be defined here in actual media extending PartialMedium
     // Example:
     //    AbsolutePressure p \"Absolute pressure of medium\";
     //    Temperature      T \"Temperature of medium\";
  <b>end</b> ThermodynamicState;

  // optional medium properties
  <b>replaceable partial function</b> dynamicViscosity
    <b>input</b>  ThermodynamicState state;
    <b>output</b> DynamicViscosity eta;
  <b>end</b> dynamicViscosity;

  // other optional functions

  // medium specific types
  <b>type</b> AbsolutePressure = SI.AbsolutePressure (
                               min     = 0,
                               max     = 1.e8,
                               nominal = 1.e5,
                               start   = 1.e5);
  <b>type</b> DynamicViscosity = ...;
  // other type definitions
<b>end</b> PartialMedium;
</pre>
<p>
We will discuss all parts of this package in the
following paragraphs. An actual medium model should extend from PartialMedium
and has to provide implementations of the various parts.
</p>

<p>
Some of the constants at the beginning of the package do not have a value yet
(this is valid in Modelica), but a value has to be provided when extending from
package PartialMedium. A given value can be modified until the model is
translated or the <b>final</b> prefix is set.
The reason to use constants instead of parameters in the model BaseProperties
is that some of these constants are used in a context where parameters
are not allowed. For example, in connector definitions the
number of independent mass fractions nXi is used as dimension
of a vector Xi. When defining the
connector, only <i>constants</i> in packages can be accessed, but not
<i>parameters</i> in a model, because a connector cannot contain an instance
of BaseProperties.
</p>

<p>The record BasePropertiesRecord contains the variables
primarily used in balance equations. Three equations for these variables have
to be provided by every medium in model BaseProperties, plus two equations
for the gas constant and the molar mass.
</p>

<p>Optional medium properties are defined by functions, such as the function
dynamicViscosity (see code Section above) to compute the dynamic viscosity.
The argument of those functions is the ThermodynamicState record, defined
in BaseProperties, which contains the minimum number of thermodynamic variables
needed as an input to compute all the optional properties.
This construction simplifies the usage
considerably as demonstrated in the following code fragment:
</p>

<pre>
  <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium;
  Medium.BaseProperties   medium;
  Medium.DynamicViscosity eta;
  ...
  U   = m*medium.u; //Internal energy
  eta = Medium.dynamicViscosity(medium.state);
</pre>

<p>Medium is the medium package that satisfies the
requirements of a PartialMedium (when using the model above, a value for
Medium has to be provided by a redeclaration). The medium component is an
instance of the model Medium.BaseProperties and contains the core medium
equations. Variables in this model can be accessed just by dot-notation, such
as medium.u or medium.T. If an optional medium variable has to be computed, the
corresponding function from the actual Medium package is called, such as
Medium.dynamicViscosity. The medium.state vector can be given as input argument
to this function, and its fields are kept consistent to those of BaseProperties
by suitable equations, contained in BaseProperties itself (see above).
</p>

<p>If a medium model does not provide implementations of all
optional functions and one of these functions is called in a model, an error
occurs during translation since the optional functions which have not been
redeclared have the <i>partial</i> attribute. For example, if function
dynamicViscosity is not provided in the medium model when it is used, only
simple pressure drop loss models without a reference to the viscosity can be
used and not the sophisticated ones.
</p>

<p>At the bottom of the PartialMedium package type declarations
are present, that are used in all other parts of the PartialMedium package and
that should be used in all models and connectors where a medium model is
accessed. The reason is that minimum, maximum, nominal, and start
values are defined and these values can be adapted to the particular medium at
hand. For example, the nominal value of AbsolutePressure is 10<sup>5</sup>
Pa. If a simple model of water steam is used that is only valid above 100 &deg;C,
then the minimum value in the Temperature type should be set to this value. The
minimum and maximum values are also important for parameters in order to get an
early message if data outside of the validity region is given. The nominal
attribute is important as a scaling value if the variable is used as a state in
a differential equation or as an iteration variable in a non-linear system of
equations. The start attribute can be very useful to provide a meaningful
default start or guess value if the variable is used, e.g., as iteration
variable in a non-linear system of equations. Note, that all these attributes
can be set specifically for a medium in the following way:
</p>

<p>
<pre>
<b>package</b> MyMedium
  <b>extends</b> Modelica.Media.Interfaces.PartialMedium(
     ...
     Temperature(min=373));
<b>end</b> MyMedium;
</pre>
</p>

<p>
The type PartialMedium.MassFlowRate is defined as
</p>

<p>
<pre>
<b>type</b> MassFlowRate = Modelica.SIunits.MassFlowRate
     (quantity = \"MassFlowRate.\" + mediumName);
</pre>
</p>

<p>Note that the constant mediumName, that has to be
defined in every medium model, is used in the quantity attribute. For example,
if mediumName = SimpleLiquidWater, then the quantity attribute has the value
MassFlowRate.SimpleLiquidWater. This type should be used in a connector
definition of a fluid library:
</p>

<p>
<pre>
<b>connector</b> FluidPort
  <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium;
  <b>flow</b> Medium.MassFlowRate m_flow;
  ...
<b>end</b> FluidPort;
</pre></p>

<p>In the model where this connector is used, the actual
Medium has to be defined. Connectors can only be connected together, if the
corresponding attributes are either not defined or have identical values. Since
mediumName is part of the quantity attribute of MassFlowRate, it is not
possible to connect connectors with different media models together. In Dymola
this is already checked when models are connected together in the diagram layer
of the graphical user interface.</p>
</HTML>
"));
        end BasicStructure;
        class BasicDefinition "Basic definition"
          annotation(Documentation(info = "<HTML>
<p>
Let's now walk through the definition of a new medium model. Please refer to
<a href=\"Modelica://Modelica.Media.Interfaces.TemplateMedium\">
Modelica.Media.Interfaces.TemplateMedium</a> to obtain a template of the new
medium model code. For the moment being, consider a single-substance medium
model.
<p>
The new medium model is obtained by extending Modelica.Media.Interfaces.PartialMedium, and
setting the following package constants:
<ul>
<li>mediumName is a String containing the name of the medium.
<li>substancesNames is a vector of strings containing the names of the substances
    that make up the medium. In this case, it will contain only mediumName.
<li>singleState can be set to true if u and d in BaseProperties do not depend
    on pressure. In other words, density does not depend on pressure
    (incompressible fluid), and it is assumed that also u does not depend on
    pressure. This setting can be useful for fluids having high density and
    low compressibility (e.g., liquids at moderate pressure); fast states
    resulting from the low compressibility effects are automatically avoided.
<li>reducedX = true for single-substance media, which do not need mass
    fractions at all.
</ul>
It is also possible to change the default min, max, nominal, and start
attributes of Medium-defined types (see TemplateMedium).</p>
<p>
All other package constants, such as nX, nXi, nS, are automatically set
by the declarations of the base package Interfaces.PartialMedium. </p>
<p>
The second step is to provide an implementation to the BaseProperties model,
partially defined in the base class Interfaces.PartialMedium. In the case of
single-substance media, two independent state variables must be selected among
p, T, d, u, h, and three equations must be written to provide the values of
the remaining variables. Two equations must then be added to compute the molar
mass MM and the gas constant R. <p>
<p>
The third step is to consider the optional functions that are going to be
implemented, among the partial functions defined by the base class PartialMedium.
A minimal set of state variables that could be provided as an input to
<i>all</i> those functions must be selected, and included in the redeclaration
of the ThermodynamicState record. Subsequently, equations must be added to
BaseProperties in order that the instance of that record inside BaseProperties
(named \"state\") is kept updated. For example, assume that all additional
properties can be computed as a function of p and T. Then, ThermodynamicState
should be redclared as follows: </p>
<p><pre>
  <b>redeclare replaceable record</b> ThermodynamicState
    AbsolutePressure p \"Absolute pressure of medium\";
    Temperature T \"Temperature of medium\";
  <b>end</b> ThermodynamicState;
</pre></p>
and the following equations should be added to BaseProperties:
<p><pre>
  state.p = p;
  state.T = T;
</pre></p>
The additional functions can now be implemented by redeclaring the functions
defined in the base class and adding their algorithms, e.g.:
</p>
<p><pre>
    <b>redeclare function extends</b> dynamicViscosity \"Return dynamic viscosity\"
    <b>algorithm</b>
      eta := 10 - state.T*0.3 + state.p*0.2;
    <b>end</b> dynamicViscosity;
</pre></p>
</HTML>
"));
        end BasicDefinition;
        class MultipleSubstances "Multiple Substances"
          annotation(Documentation(info = "<HTML>
<p>
When writing the model of a multiple-substance medium, a fundamental issue
concerns how to consider the mass fractions of the fluid. If there are nS
substances, there are also nS mass fractions; however, one of them is redundant,
as sum(X) = 1. Therefore there are basically two options, concerning the number
of independent mass fractions nXi:
<ul>
<li> <i>Reduced-state models</i>: reducedX = <b>true</b> and nXi = nS - 1. In this
case, the number of independent mass fractions nXi is the minimum possible.
The full state vector X is provided by equations declared in the base class
Interfaces.PartialMedium.BaseProperties: the first nXi elements are equal to
Xi, and the last one is 1 - sum(Xi).
<li> <i>Full-state models</i>: reducedX = <b>false</b> and nXi = nS. In this case,
Xi = X, i.e., all the elements of the composition vector are considered as
independent variables, and the constraint sum(X) = 1 is never written explicitly.
Although this kind of model is heavier, as it provides one extra state variable,
it can be less prone to numerical and/or symbolic problems, which can be
caused by that constraint.
<li> <i>Fixed-composition models</i>: fixedX = <b>true</b> and nXi = 0. In this case X = reference_X, i.e. all the elements of the composition vector are fixed.
</ul>
<p> The medium implementor can declare the value reducedX as <b>final</b>. In
this way only one implementation must be given. For instance,
Modelica.Media.IdealGases models declare <b>final</b> reducedX = <b>false</b>, so that the
implementation can always assume nXi = nX. The same is true for Air.MoistAir,
which declares <b>final</b> reducedX = <b>true</b>, and always assumes nXi = nX - 1 = 1.</p>
<p>It is also possible to leave reducedX modifiable. In this case, the
BaseProperties model and all additional functions should check for the actual
value of reducedX, and provide the corresponding implementation.</p>

<p>If fixedX is left modifiable, then the implementation should also handle the
case fixedX = true properly.</p>
<p>Fluid connectors should always use composition vectors of size Xi, such as
in the Modelica_Fluid library: </p>
<p><pre>
<b>connector</b> FluidPort
  <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium;
  Medium.AbsolutePressure      p;
  <b>flow</b> Medium.MassFlowRate     m_flow;

  Medium.SpecificEnthalpy      h;
  <b>flow</b> Medium.EnthalpyFlowRate H_flow;

  Medium.MassFraction          Xi    [Medium.nXi];
  <b>flow</b> Medium.MassFlowRate     mX_flow[Medium.nXi];
<b>end</b> FluidPort;
</pre></p>
<p>
For further details, refer to the implementation of
<a href=\"Modelica://Modelica.Media.IdealGases.Common.MixtureGasNasa\">
      MixtureGasNasa model</a> and
<a href=\"Modelica://Modelica.Media.Air.MoistAir\">
      MoistAir model</a>.
</p>
</HTML>
"));
        end MultipleSubstances;
        class SpecificEnthalpyAsFunction "Specific enthalpy as function"
          annotation(Documentation(info = "<html>
<p>
If pressure p and specific enthalpy h are <b>not</b> used as
independent medium variables, the specific enthalpy should
be computed by a Modelica function that has as input arguments
only the independent medium variables. It should <b>not</b> be
computed by an equation. For example, if p and T are used
as independent medium variables, a function h_pT(p,T) should
be defined that is called to compute h:
</p>

<pre>
    h = h_pT(p,T);
</pre>

<p>
The reason for this rule requires a longer explanation.
In short, if h is not a computed by a Modelica function and
this function is non-linear in the independent medium variables,
then non-linear systems of equations will occur at
every connection point, if the FluidPort connectors from the
Modelica_Fluid library are used (these are the same as in
Modelica.Media.Examples.Tests.Components.FluidPort).
Only, if the above rule is fulfilled, a tool is able to
remove these non-linear system of equations in most cases.
</p>

<p>
The basic idea of the FluidPort connector is that 2 or more
components can be connected together at a point and that
automatically the mass and energy balance is fulfilled
in the connection point, i.e., the ideal mixing equations
are generated. Note, the momentum balance is only correct for
straight line connections. If \"ideal mixing\" is not sufficient,
a special component to define the mixing equations must be introduced.
</p>

<p>
The mass and momentum balance equations in a component are
derived from the partial differential equations along the
flow direction of a pipe:
</p>

<IMG SRC=\"../Images/Media/UsersGuide/BalanceEquations1.png\">

<p>
Note, F<sub>F</sub> is the fanning friction factor.
The energy balance can be given in different forms.
Usually, it is given as:
</p>

<IMG SRC=\"../Images/Media/UsersGuide/EnergyBalance1.png\">

<p>
This form describes the change of the internal energy, kinetic
energy and potential energy of a volume as function of the
in and out flowing fluid. Multiplying the momentum balance
with the flow velocity v and subtracting it from the energy
balance above, results in the following alternative form
of the energy balance:
</p>

<IMG SRC=\"../Images/Media/UsersGuide/EnergyBalance2.png\">

<p>
This form has the advantage that the kinetic and potential
energy is no longer part of the energy balance and therefore
the energy balance is substantially simpler (e.g., additional non-linear
systems of equations occur in the first form since the velocity
is present in the energy balance; in the second form this is not
the case and it is still valid also for high speeds).
</p>

<p>
Assume now that the second form of the energy balance above
is used in all components and that the following FluidPort connector
is used in all components:
</p>

<pre>
<b>connector</b> FluidPort
  <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium;
  Medium.AbsolutePressure      p;
  <b>flow</b> Medium.MassFlowRate     m_flow;

  Medium.SpecificEnthalpy      h;
  <b>flow</b> Medium.EnthalpyFlowRate H_flow;

  Medium.MassFraction          Xi    [Medium.nXi];
  <b>flow</b> Medium.MassFlowRate     mX_flow[Medium.nXi];
<b>end</b> FluidPort;
</pre>

<p>
As an example, assume that 3 components
are connected together and that the medium is a single substance
fluid. This will result in the following
connection equations:
</p>

<pre>
    p1=p2=p3;
    h1=h2=h3;
    0 = m_flow1 + m_flow2 + m_flow3;
    0 = H_flow1 + H_flow2 + H_flow3;
</pre>

<p>
These are the mass balance and the
energy balance (form 2) of an infinitesimal volume
in the connection point under the assumption that no
mass or energy is stored in this volume. In other words,
the connection equations are the equations that describe
ideal mixing. Under the assumption that the velocity
vectors of the 3 flows are identical (especially, they are
parallel), also the momentum balance is fulfilled:
</p>

<pre>
   0 = m_flow1*v1 + m_flow2*v2 + m_flow3*v3;
     = v*(m_flow1 + m_flow2 + m_flow3);
     = 0;
</pre>

<p>
With the above connector it is therefore possible to
connect components together in a nearly arbitrary fashion,
because every connection fulfills automatically the
balance equations. This approach has, however, one drawback:
If two components are connected together, then the medium
variables on both sides of the connector are identical.
However, due to the connector, only the two equations
</p>

<pre>
   p1 = p2;
   h1 = h2;
</pre>

<p>
are present. Assume, that p,T are the independent medium variables
and that the medium properties are computed at one side of the
connections. This means, the following equations are basically
present:
</p>

<pre>
    h1 = h(p1,T1);
    h2 = h(p2,T2);
    p1 = p2;
    h1 = h2;
</pre>

<p>
These equations can be solved in the following way:
</p>

<pre>
    h1 := h(p1,T1)
    p2 := p1;
    h2 := h1;
    0  := h2 - h(p2,T2);   // non-linear system of equations for T2
</pre>

<p>
This means that T2 is computed by solving a non-linear system
of equations. If h1 and h2 are provided as Modelica functions,
a Modelica translator, such as Dymola, can replace
this non-linear system of equations by the equation:
</p>

<pre>
   T2 := T1;
</pre>

<p>
because after alias substition there are two function calls
</p>

<pre>
    h1 := h(p1,T1);
    h1 := h(p1,T2);
</pre>

<p>
Since the left hand side of the function call and the first
argument are the same, the second arguments T1 and T2 must also be
identical and therefore T2 := T1. This type of analysis seems
to be only possible, if the specific enthalpy is defined as a function
of the independent medium variables.
</p>

</html>"));
        end SpecificEnthalpyAsFunction;
        class StaticStateSelection "Static State Selection"
          annotation(Documentation(info = "<html>
<p>
Without pre-caution when implementing a medium model,
it is very easy that non-linear algebraic
systems of equations occur when using the medium model.
In this section it is explained how to avoid non-linear
systems of equations that result from unnecessary
dynamic state selections.
</p>
<p>
A medium model should be implemented in such a way that
a tool is able to select states of a medium in a balance volume
statically (during translation). This is only possible if the
medium equations are written in a specific way. Otherwise,
a tool has to dynamically select states during simulation.
Since medium equations are usually non-linear, this means that
non-linear algebraic systems of equations would occur in every
balance volume.
</p>
<p>
It is assumed that medium equations in a balance volume
are defined in the following way:
</p>
<pre>
    <b>package</b> Medium = Modelica.Media.Interfaces.PartialMedium;
    Medium.BaseProperties medium;
  <b>equation</b>
     // mass balance
       <b>der</b>(M)  = port_a.m_flow + port_b.m_flow;
       <b>der</b>(MX) = port_a_mX_flow + port_b_mX_flow;
             M = V*medium.d;
            MX = M*medium.X;

     // Energy balance
     U = M*medium.u;
     <b>der</b>(U) = port_a.H_flow+port_b.H_flow;
</pre>
<p>
<b>Single Substance Media</b>
</p>
<p>
A medium consisting of a single substance
has to define two of \"p,T,d,u,h\" with
stateSelect=StateSelect.prefer if BaseProperties.preferredMediumstates = <b>true</b>
and has to provide the other three variables as function of these
states. This results in:
</p>
<ul>
<li> static state selection (no dynamic choices).</li>
<li> a linear system of equations in the two
     state derivatives.</li>
</ul>
<p>
<b>Example for a single substance medium</b>
</p>
<p>
p, T are preferred states (i.e. StateSelect.prefer is set)
and there are three equations written in the form:
</p>
<pre>
   d = fd(p,T)
   u = fu(p,T)
   h = fh(p,T)
</pre>
<p>
Index reduction leads to the equations:
</p>
<pre>
   <b>der</b>(M) = V*<b>der</b>(d)
   <b>der</b>(U) = <b>der</b>(M)*u + M*<b>der</b>(u)
   <b>der</b>(d) = <b>der</b>(fd,p)*<b>der</b>(p) + <b>der</b>(fd,T)*<b>der</b>(T)
   <b>der</b>(u) = <b>der</b>(fu,p)*<b>der</b>(p) + <b>der</b>(fu,T)*<b>der</b>(T)
</pre>
<p>
Note, that <b>der</b>(y,x) is the partial derivative of y with respect to x
and that this operator will be introduced in a future version
of the Modelica language.
The above equations imply, that if p,T are provided from the
integrator as states, all functions, such as fd(p,T)
or <b>der</b>(fd,p) can be evaluated as function of the states.
The overall system results in a linear system
of equations in <b>der</b>(p) and <b>der</b>(T) after eliminating
<b>der</b>(M), <b>der</b>(U), <b>der</b>(d), <b>der</b>(u) via tearing.
</p>
<p>
<b>Counter Example for a single substance medium</b>
</p>
<p>
An ideal gas with one substance is written in the form
</p>
<pre>
  <b>redeclare model extends</b> BaseProperties(
     T(stateSelect=if preferredMediumStates then StateSelect.prefer else StateSelect.default),
     p(stateSelect=if preferredMediumStates then StateSelect.prefer else StateSelect.default)
  <b>equation</b>
     h = h(T);
     u = h - R*T;
     p = d*R*T;
      ...
  <b>end</b> BaseProperties;
</pre>
<p>
If p, T are preferred states, these equations are <b>not</b>
written in the recommended form, because d is not a
function of p and T. If p,T would be states, it would be
necessary to solve for the density:
</p>
<pre>
   d = p/(R*T)
</pre>
<p>
If T or R are zero, this results in a division by zero.
A tool does not know that R or T cannot become zero.
Therefore, a tool must assume that p, T <b>cannot</b> always be
selected as states and has to either use another static
state selection or use dynamic state selection. The only
other choice for static state selection is d,T, because
h,u,p are given as functions of d,T.
However, as potential states only variables appearing differentiated and variables
declared with StateSelect.prefer or StateSelect.always
are used. Since \"d\" does not appear differentiated and has
StateSelect.default, it cannot be selected as a state.
As a result, the tool has to select states dynamically
during simulation. Since the equations above are non-linear
and they are utilized in the dynamic state
selection, a non-linear system of equations is present
in every balance volume.
</p>
<p>
To summarize, for single substance ideal gas media there
are the following two possibilities to get static
state selection and linear systems of equations:
</p>
<ol>
<li> Use p,T as preferred states and write the equation
     for d in the form: d = p/(T*R)</li>
<li> Use d,T as preferred states and write the equation
     for p in the form: p = d*T*R</li>
</ol>
<p>
All other settings (other/no preferred states etc.) lead
to dynamic state selection and non-linear systems of
equations for a balance volume.
</p>
<p>
<b>Multiple Substance Media</b>
</p>
<p>
A medium consisting of multiple substance
has to define two of \"p,T,d,u,h\" as well
as the mass fractions Xi with
stateSelect=StateSelect.prefer (if BaseProperties.preferredMediumStates = <b>true</b>)
and has to provide
the other three variables as functions of these
states. Only then, static selection is possible
for a tool.
</p>
<p>
<b>Example for a multiple substance medium:</b></p>
</p>
<p>
p, T and Xi are defined as preferred states and
the equations are written in the form:
</p>
<pre>
   d = fp(p,T,Xi);
   u = fu(p,T,Xi);
   h = fh(p,T,Xi);
</pre>
<p>
Since the balance equations are written in the form:
</p>
<pre>
              M = V*medium.d;
            MXi = M*medium.Xi;
</pre>
<p>
The variables M and MXi appearing differentiated in the
balance equations are provided as functions of d and Xi
and since d is given as a function of p, T and Xi, it
is possible to compute M and MXi directly from the desired
states. This means that static state selection is possible.
</p>

</html>"));
        end StaticStateSelection;
        class TestOfMedium "Test of medium"
          annotation(Documentation(info = "<html>
<p>
After implementation of a new medium model, it should
be tested. A basic test is already provided with model
Modelica.Media.Examples.Tests.Components.PartialTestModel
which might be used in the following way:
</p>

<pre>
  <b>model</b> TestOfMyMedium
     <b>extends</b> Modelica.Media.Examples.Tests.Components.PartialTestModel(
              <b>redeclare package</b> Medium = MyMedium);
  <b>end</b> TestOfMyMedium;
</pre>

<p>
It might be necessary to adapt or change initial values
depending on the validity range of the medium.
The model above should translate and simulate.
If the medium model is written according to the
suggestions given in the previous sections (and the Modelica
translator has appropriate algorithms implemented),
there should be only static state selection everywhere
and no non-linear system of equations, provided h is an independent
medium variable or is only a function of T. If h is a function
of, say h=h(p,T), one non-linear system of equations occurs that
cannot be avoided.
</p>

<p>
The test model above can be used to test the most basic
properties. Of course, more tests should be performed.
</p>

</html>"));
        end TestOfMedium;
      end MediumDefinition;
      class ReleaseNotes "Release notes"
        annotation(Documentation(info = "<HTML>
<h4>Version included in Modelica 3.0</h4>
<p> See top-level release notes for MSL. </p>
<h4>Version 1.0, 2005-03-01</h4>
<p>
Many improvements in the library, e.g., providing mixtures
of the ideal gases, table based media, test suite for all media, improved and
updated User's Guide.
</p>
<h4><font color=\"#008000\">Version 0.9, 2004-10-18</font></h4>
<ul>
<li> Changed the redeclaration/extends within packages from the
     experimental feature to the language keywords introduced
     in Modelica 2.1.</li>
<li> Re-introduced package \"Water.SaltWater\" in order to test
     substance mixtures (this medium model does not describe
     real mixing of water and salt). </li>
<li> Started to improve the documentation in
     Modelica.Media.UsersGuide.MediumDefinition.BasicStructure</li>
</ul>
<h4><font color=\"#008000\">Version 0.792, 2003-10-28</font></h4>
<p>
This is the first version made available for the public
for the Modelica'2003 conference (for evaluation).
</p>
</HTML>
"));
      end ReleaseNotes;
      class Future "Future"
        annotation(Documentation(info = "<html>

<h4>Planned changes for version 3.1</h4>

<p>Several changes are planned for version 3.1 of Modelica.Media. In order to have an easy transition, the current verison is moved
  to the ObsoleteModelica-package and current users can continue to use ot for many years. The goal for the new version is to cover many more fluids, be easier to use for users and less for implementors that want to include their own models. A beta-version of the new media libray will be available in the first quarter of 2009.</p>

<p>The main user-visible changes are:</p>
<ul>
<li>No distinction between pure fluids and mixtures on the interface level: pure fluids are mixtures with 1 component</li>
<li>Compressibility will in the future be a flag that can be set for those media where it makes sense. It will thus be possible to treat an ideal gas as incompressible, which is adequate for certain applications, and also use the same medium model for thermal-only applications, and thermal-hydraulic ones.</li>
<li>All functions will have derivatives to enable analytic Jacobians.</li>
<li>Mole-fraction based media for process applications</li>
</ul>
<p>For further information on the upcoming version, please contact the <a href=\"Modelica://Modelica.Media.UsersGuide.Contact\">Author</a></p>

<h4>Forward compatibility</h4>
<p> In order to simplify transition to the upcoming version of
Modelica.Media for Modelica 3.1, a new enumeration has been
introduced:  <a href=\"Modelica://Modelica.Media.Interfaces.PartialMedium.Choices.IndependentVariables\">IndependentVariables</a> with the
entries ph, pT, pTX, phX and dTX. Users are advised to use this enumeration to determine the independent variables of a medium. If they are going to use the function based interface of the current Modelica.Media (and not use the BaseProperties from Modelica.Media), they should not have any trouble using the next version without any need for changes in their code.</p>

</html>"));
      end Future;
      class Contact "Contact"
        annotation(Documentation(info = "<html>
<dl>
<dt><b>Main author and maintainer:</b>
<dd>Hubertus Tummescheit<br>
    Modelon AB<br>
    Ideon Science Park<br>
    SE-22730 Lund, Sweden<br>
    email: <A HREF=\"mailto:Hubertus.Tummescheit@Modelon.se\">Hubertus.Tummescheit@Modelon.se</A><br>
</dl>
<p><b>Acknowledgements:</b></p>
<p>
The development of this library has been a collaborative effort
and many have contributed:
</p>
<ul>
<li> The essential parts of the media models have been implemented
     in the ThermoFluid library by Hubertus Tummescheit with
     help from Jonas Eborn and Falko Jens Wagner. These media models
     have been converted to the Modelica.Media interface definition
     and have been improved by Hubertus Tummescheit. </li>
<li> The effort for the development of the Modelica.Media library has been
     organized by Martin Otter who also contributed to the design,
     implemented part of the generic models, contributed to the User's Guide
     and provided the generic test suite Modelica.Media.Examples.Tests.</li>
<li> The basic idea for the medium model interface based on packages
     is from Michael Tiller who also contributed to the design.</li>
<li> The first design of the medium model interface is from
     Hilding Elmqvist. The design and the implementation has been further
     improved at the Modelica design meetings in<br>
     Dearborn, Nov. 20-22, 2002<br>
     Dearborn, Sept. 2-4, 2003<br>
     Lund Jan. 28-30, 2004<br>
     Munich, May 26-28, 2004<br>
     Lund, Aug. 30-31, 2004<br>
     Dearborn, Nov. 15-17, 2004<br>
     Cremona Jan. 31 - Feb. 2, 2005.</li>
<li> Hans Olsson, Sven Erik Mattsson and Hilding Elmqvist developed
     symbolic transformation algorithms and implemented them in Dymola
     to improve the efficiency considerably (e.g., to avoid non-linear
        systems of equations).</li>
<li>Katrin Pr&ouml;&szlig; implemented the moist air model</li>
<li> R&uuml;diger Franke performed the first realistic tests of the Modelica.Media
        and Modelica_Fluid libraries and gave valuable feedback.</li>
<li> Francesco Casella has been the most relentless bug-hunter and tester of the
     water and ideal gas properties. He also contributed to the User's Guide.</li>
<li> John Batteh, Daniel Bouskela, Jonas Eborn, Andreas Idebrant, Charles Newman,
     Gerhart Schmitz, and the users of the ThermoFluid library provided
     many useful comments and feedback.</li>
</ul>
</html>"));
      end Contact;
    end UsersGuide;
    package Examples "Demonstrate usage of property models (currently: simple tests)"
      extends Modelica.Icons.Library;
      model SimpleLiquidWater "Example for Water.SimpleLiquidWater medium model"
        annotation(experiment(StopTime = 100), Documentation(info = "<html>

</html>"));
        import SI = Modelica.SIunits;
        extends Modelica.Icons.Example;
        parameter SI.Volume V = 1 "Volume";
        parameter SI.EnthalpyFlowRate H_flow_ext = 1000000.0 "Constant enthalpy flow rate into the volume";
        package Medium = Water.ConstantPropertyLiquidWater(SpecificEnthalpy(max = 1000000.0)) "Medium model";
        Medium.BaseProperties medium(T(start = 300, fixed = true));
        Medium.BaseProperties medium2;
        Medium.ThermodynamicState state;
        Real m_flow_ext2;
        Real der_p;
        Real der_T;
        SI.Mass m(start = 1.0);
        SI.InternalEnergy U;
        Medium.MassFlowRate m_flow_ext;
        Medium.DynamicViscosity eta = Medium.dynamicViscosity(medium);
        Medium.SpecificHeatCapacity cv = Medium.specificHeatCapacityCv(medium);

      equation 
        medium.p = 100000.0;
        m = medium.d * V;
        U = m * medium.u;
        der(m) = m_flow_ext;
        der(U) = H_flow_ext;
        medium2.p = (100000.0 * time) / 10;
        medium2.T = 330;
        m_flow_ext2 = time - 30;
        state = Medium.setSmoothState(m_flow_ext2, medium.state, medium2.state, 10);
        der_p = der(state.p);
        der_T = der(state.T);
      end SimpleLiquidWater;
      model IdealGasH2O "IdealGas H20 medium model"
        extends Modelica.Icons.Example;
        package Medium = IdealGases.SingleGases.H2O "Medium model";
        Medium.ThermodynamicState state "thermodynamic state record";
        Medium.ThermodynamicState state2;
        Medium.SpecificHeatCapacity cp = Medium.specificHeatCapacityCp(state);
        Medium.SpecificHeatCapacity cv = Medium.specificHeatCapacityCv(state);
        Medium.IsentropicExponent k = Medium.isentropicExponent(state);
        Medium.SpecificEntropy s = Medium.specificEntropy(state);
        Medium.VelocityOfSound a = Medium.velocityOfSound(state);
        Real beta = Medium.isobaricExpansionCoefficient(state);
        Real gamma = Medium.isothermalCompressibility(state);
        Medium.SpecificEnthalpy h_is = Medium.isentropicEnthalpyApproximation(2.0, state);
        Medium.ThermodynamicState smoothState;
        Real m_flow_ext;
        Real der_p;
        Real der_T;
        annotation(Documentation(info = "<html>
<p>An example for using ideal gas properties and how to compute
  isentropic enthalpy changes.
  The function that is implemented is approximate, but usually
   very good: the second medium record medium2
   is given to compare the approximation.
</p>
</html>"), experiment(StopTime = 1));

      equation 
        state.p = 100000.0;
        state.T = 200 + 1000 * time;
        state2.p = 200000.0;
        state2.T = 500.0;
        m_flow_ext = time - 0.5;
        smoothState = Medium.setSmoothState(m_flow_ext, state, state2, 0.1);
        der_p = der(smoothState.p);
        der_T = der(smoothState.T);
      end IdealGasH2O;
      model WaterIF97 "WaterIF97 medium model"
        extends Modelica.Icons.Example;
        package Medium = Water.StandardWater "Medium model";
        Medium.BaseProperties medium(p(start = 100000.0, stateSelect = StateSelect.prefer), h(start = 100000.0, stateSelect = StateSelect.prefer), T(start = 275.0), d(start = 999.0));
        Modelica.SIunits.Volume V(start = 0.1);
        parameter Modelica.SIunits.VolumeFlowRate dV = 0.0 "Fixed time derivative of volume";
        parameter Medium.MassFlowRate m_flow_ext = 0 "Fixed mass flow rate into volume";
        parameter Medium.EnthalpyFlowRate H_flow_ext = 10000 "Fixed enthalpy flow rate into volume";
        Modelica.SIunits.Mass m "Mass of volume";
        Modelica.SIunits.InternalEnergy U "Internal energy of volume";
        annotation(Documentation(info = "<html>

</html>"), experiment(StopTime = 1));
        Medium.ThermodynamicState state2;
        Medium.ThermodynamicState state;
        Real m_flow_ext2;
        Real der_p;
        Real der_T;

      equation 
        der(V) = dV;
        m = medium.d * V;
        U = m * medium.u;
        der(m) = m_flow_ext;
        der(U) = H_flow_ext;
        m_flow_ext2 = time - 0.5;
        state2 = Medium.setState_pT(100000.0 * (1 + time), 300 + 200 * time);
        state = Medium.setSmoothState(m_flow_ext2, medium.state, state2, 0.05);
        der_p = der(state.p);
        der_T = der(state.T);
      end WaterIF97;
      model MixtureGases "Test gas mixtures"
        extends Modelica.Icons.Example;
        parameter Modelica.SIunits.Volume V = 1 "Fixed size of volume 1 and volume 2";
        parameter Modelica.SIunits.MassFlowRate m_flow_ext = 0.01 "Fixed mass flow rate in to volume 1 and in to volume 2";
        parameter Modelica.SIunits.EnthalpyFlowRate H_flow_ext = 5000 "Fixed enthalpy flow rate in to volume and in to volume 2";
        package Medium1 = Modelica.Media.IdealGases.MixtureGases.CombustionAir "Medium model";
        Medium1.BaseProperties medium1(p(start = 100000.0, stateSelect = StateSelect.prefer), T(start = 300, stateSelect = StateSelect.prefer), X(start = {0.8,0.2}));
        Real m1(quantity = Medium1.mediumName, start = 1.0);
        SI.InternalEnergy U1;
        Medium1.SpecificHeatCapacity cp1 = Medium1.specificHeatCapacityCp(medium1.state);
        Medium1.DynamicViscosity eta1 = Medium1.dynamicViscosity(medium1.state);
        Medium1.ThermalConductivity lambda1 = Medium1.thermalConductivity(medium1.state);
        package Medium2 = Modelica.Media.IdealGases.MixtureGases.SimpleNaturalGas "Medium model";
        Medium2.BaseProperties medium2(p(start = 100000.0, stateSelect = StateSelect.prefer), T(start = 300, stateSelect = StateSelect.prefer), X(start = {0.1,0.1,0.1,0.2,0.2,0.3}));
        Real m2(quantity = Medium2.mediumName, start = 1.0);
        SI.InternalEnergy U2;
        Medium2.SpecificHeatCapacity cp2 = Medium2.specificHeatCapacityCp(medium2.state);
        Medium2.DynamicViscosity eta2 = Medium2.dynamicViscosity(medium2.state);
        Medium2.ThermalConductivity lambda2 = Medium2.thermalConductivity(medium2.state);
        Medium2.ThermodynamicState state2 = Medium2.setState_pTX(100500.0, 302, {0.3,0.2,0.2,0.1,0.1,0.1});
        Medium2.ThermodynamicState smoothState;
        Real m_flow_ext2;
        Real der_p;
        Real der_T;
        annotation(Documentation(info = "<html>

</html>"), experiment(StopTime = 1));

      equation 
        medium1.X = {0.8,0.2};
        m1 = medium1.d * V;
        U1 = m1 * medium1.u;
        der(m1) = m_flow_ext;
        der(U1) = H_flow_ext;
        medium2.X = {0.1,0.1,0.1,0.2,0.2,0.3};
        m2 = medium2.d * V;
        U2 = m2 * medium2.u;
        der(m2) = m_flow_ext;
        der(U2) = H_flow_ext;
        m_flow_ext2 = time - 0.5;
        smoothState = Medium2.setSmoothState(m_flow_ext2, medium2.state, state2, 0.2);
        der_p = der(smoothState.p);
        der_T = der(smoothState.T);
      end MixtureGases;
      model MoistAir "Ideal gas flue gas  model"
        extends Modelica.Icons.Example;
        package Medium = Air.MoistAir "Medium model";
        Medium.BaseProperties medium(T(start = 274.0), X(start = {0.95,0.05}), p(start = 100000.0));
        parameter Medium.MolarMass[2] MMx = {Medium.dryair.MM,Medium.steam.MM} "Vector of molar masses (consisting of dry air and of steam)";
        Medium.MolarMass MM = 1 / ((1 - medium.X[1]) / MMx[1] + medium.X[1] / MMx[2]) "Molar mass of gas part of mixture";
        annotation(Documentation(info = "<html>
<p>An example for using ideal gas properties and how to compute isentropic enthalpy changes.
The function that is implemented is approximate, but usually very good: the second medium record medium2
is given to compare the approximation.
</p>
</html>"), experiment(Tolerance = 1e-05));
        Medium.ThermodynamicState state1;
        Medium.ThermodynamicState state2;
        Medium.ThermodynamicState smoothState;
        Real m_flow_ext;
        Real der_p;
        Real der_T;

      equation 
        der(medium.p) = 0.0;
        der(medium.T) = 90;
        medium.X[Medium.Air] = 0.95;
        m_flow_ext = time - 0.5;
        state1.p = 100000.0 * (1 + time);
        state1.T = 300 + 10 * time;
        state1.X = {time,1 - time};
        state2.p = 100000.0 * (1 + time / 2);
        state2.T = 340 - 20 * time;
        state2.X = {0.5 * time,1 - 0.5 * time};
        smoothState = Medium.setSmoothState(m_flow_ext, state1, state2, 0.2);
        der_p = der(smoothState.p);
        der_T = der(smoothState.T);
      end MoistAir;
      package TwoPhaseWater "extension of the StandardWater package"
        extends Modelica.Media.Water.StandardWater;
        model extends BaseProperties "Make StandardWater.BaseProperties non replaceable in order that inheritance is possible in model ExtendedProperties"
        end BaseProperties;
        model ExtendedProperties "plenty of two-phase properties"
          extends BaseProperties;
          ThermodynamicState dew "dew line Properties";
          ThermodynamicState bubble "bubble line Properties";
          ThermodynamicState bubble2 "bubble line Properties, on the 2-phase side";
          DynamicViscosity eta "viscosity (McAdams mixture rules if in 2-phase)";
          DynamicViscosity eta_d "dew line viscosity";
          DynamicViscosity eta_b "bubble line viscosity";
          ThermalConductivity lambda_d "dew line thermal conductivity";
          ThermalConductivity lambda_b "bubble line thermal conductivity";
          SpecificHeatCapacity cp_d "dew line Specific heat capacity";
          SpecificHeatCapacity cp_b "bubble line Specific heat capacity";
          Real ddhp;
          Real ddhp_d;
          Real ddhp_b "derivatives";
          Real ddph;
          Real ddph_d;
          Real ddph_b "derivatives";
          Real ddhp_b2;
          Real ddph_b2 "derivatives";
          MassFraction x "steam mass fraction";
          Real dTp;
          Real dTp2;
          SpecificEntropy s_b;
          SpecificEntropy s_d;

        equation 
          eta = if phase == 1 then dynamicViscosity(state) else 1 / (x / eta_d + (1 - x) / eta_b);
          dew = setDewState(sat);
          bubble = setBubbleState(sat);
          bubble2 = setBubbleState(sat, 2);
          x = (h - bubble.h) / max(dew.h - bubble.h, 1e-06);
          eta_d = dynamicViscosity(dew);
          eta_b = dynamicViscosity(bubble);
          lambda_d = thermalConductivity(dew);
          lambda_b = thermalConductivity(bubble);
          cp_d = specificHeatCapacityCp(dew);
          cp_b = specificHeatCapacityCp(bubble);
          s_d = specificEntropy(dew);
          s_b = specificEntropy(bubble);
          ddph = density_derp_h(state);
          ddph_d = density_derp_h(dew);
          ddph_b = density_derp_h(bubble);
          ddhp = density_derh_p(state);
          ddhp_d = density_derh_p(dew);
          ddhp_b = density_derh_p(bubble);
          ddhp_b2 = density_derh_p(bubble2);
          ddph_b2 = density_derp_h(bubble2);
          dTp = saturationTemperature_derp(p);
          dTp2 = (1 / dew.d - 1 / bubble.d) / max(s_d - s_b, 1e-06);
          annotation(Documentation(info = ""));
        end ExtendedProperties;
        annotation(Documentation(info = "<html>
<h4>Example: TwoPhaseWater</h4>
The TwoPhaseWater package demonstrates how to extend the parsimonius
BaseProperties with a minimal set of properties from the standard water
package with most properties that are needed in two-phase situations.
The model also demonstrates how to compute additional&nbsp; properties
for the medium model. In this scenario, that builds a new medium model
with many more properties than the default, the standard BaseProperties
is used as a basis. For additional properties, a user has to:<br>
<ol>
<li>Declare a new variable of the wanted type, e.g. <span
style=\"color: rgb(0, 0, 153);\">\"<span style=\"color: rgb(51, 51, 255);\">DynamicViscosity
eta</span>\"</span>.</li>
<li>Compute that variable by calling the function form the package,
e.g. <span style=\"color: rgb(51, 51, 255);\">eta =
dynamicViscosity(state)</span>. Note that the instance of
ThermodynamicState is used as an input to the function. This instance
\"state\" is declared in PartialMedium and thus available in every medium
model. A user does not have to know what actual variables are required
to compute the dynamic viscosity, because the state instance is
guaranteed to contain what is needed.&nbsp;<span
style=\"color: rgb(255, 0, 0);\"></span></li>
<li><span style=\"color: rgb(255, 0, 0);\">Attention</span>: Many
properties are not well defined in the two phase region and the
functions might return undesired values if called there. It is the
user's responsibility&nbsp; to take care of such ituations. The example
uses one of several possible models to compute an averaged viscosity
for two-phase flows. </li>
</ol>
In two phase models, properties are often needed on the phase boundary
just outside the two phase dome, right on the border.. To compute the
thermodynamic state there, two auxiliary functions are provided: <b>setDewState(sat)</b> and <b>setBubbleState(sat)</b>. They take an
instance of SaturationProperties as input. By default they are in
one-phase, but with the optional phase argument set to 2, the output is
forced to be just inside the phase boundary. This is only needed when
derivatives like cv are computed with are different on both sides of
the boundaries. The ususal steps to compute properties on the phase
boundary are: <br>
<ol>
<li>Declare an instance of ThermodynamicState, e.g. \"ThermodynamicState&nbsp; dew\".</li>
<li>Compute the state, using an instance of SaturationProperties,
e.g. dew = setDewState(sat)</li>
<li>Compute properties on the phase boundary to your full desire,
e.g. \"cp_d = specificHeatCapacityCp(dew)\". <br>
</li>
</ol>
<p>The sample model TestTwoPhaseStates test the extended properties</p>

<p>
The same procedure can be used to compute properties at other state
points, e.g. when an isentropic reference state is computed.
</p>
</html>
"));
        model TestTwoPhaseStates "Test the above model"
          extends Modelica.Icons.Example;
          ExtendedProperties medium(p(start = 700.0), h(start = 800000.0));
          parameter Real dh = 80000.0 "80 kJ/second";
          parameter Real dp = 1000000.0 "10 bars per second";

        equation 
          der(medium.p) = dp;
          der(medium.h) = dh;
          annotation(experiment(StopTime = 22, NumberOfIntervals = 2500), Documentation(info = "<html>
<p> For details see the documentation of the example package TwoPhaseWater</p>
</html>
"));
        end TestTwoPhaseStates;
      end TwoPhaseWater;
      package TestOnly "examples for testing purposes: move for final version "
        extends Modelica.Icons.Library;
        model MixIdealGasAir "Ideal gas air medium model"
          extends Modelica.Icons.Example;
          package Medium = IdealGases.MixtureGases.CombustionAir "Medium model";
          Medium.BaseProperties medium(T(start = 200.0), X(start = {0.2,0.8}), p(start = 100000.0));
          Medium.BaseProperties medium2(T(start = 300.0), X(start = {0.2,0.8}), p(start = 200000.0));
          Medium.SpecificHeatCapacity cp = Medium.specificHeatCapacityCp(medium.state);
          Medium.SpecificHeatCapacity cv = Medium.specificHeatCapacityCv(medium.state);
          Medium.IsentropicExponent gamma = Medium.isentropicExponent(medium.state);
          Medium.SpecificEntropy s = Medium.specificEntropy(medium.state);
          Medium.SpecificEntropy s2 = Medium.specificEntropy(medium2.state);
          Medium.VelocityOfSound a = Medium.velocityOfSound(medium.state);
          Medium.DynamicViscosity eta = Medium.dynamicViscosity(medium.state);
          Medium.ThermalConductivity lambda = Medium.thermalConductivity(medium.state);
          Real beta = Medium.isobaricExpansionCoefficient(medium.state);
          Real gamma2 = Medium.isothermalCompressibility(medium2.state);
          Medium.SpecificEnthalpy h_is = Medium.isentropicEnthalpyApproximation(200000.0, medium);
          annotation(Documentation(info = "<html>
<p>An example for using ideal gas properties and how to compute isentropic enthalpy changes.
The function that is implemented is approximate, but usually very good: the second medium record medium2
is given to compare the approximation.
</p>
</html>"), experiment(StopTime = 1));

        equation 
          der(medium.p) = 1000.0;
          der(medium.T) = 1000;
          medium.X = {0.2,0.8};
          der(medium2.p) = 1000.0;
          der(medium2.T) = 0.0;
          der(medium2.X) = {0.0,0.0};
        end MixIdealGasAir;
        model FlueGas "Ideal gas flue gas  model"
          extends Modelica.Icons.Example;
          package Medium = IdealGases.MixtureGases.FlueGasLambdaOnePlus "Medium model";
          Medium.ThermodynamicState state(T(start = 200.0), X(start = {0.2,0.3,0.4,0.1}), p(start = 100000.0));
          Medium.BaseProperties medium2(T(start = 300.0), X(start = {0.2,0.1,0.3,0.4}), p(start = 200000.0));
          Medium.SpecificHeatCapacity cp = Medium.specificHeatCapacityCp(state);
          Medium.SpecificHeatCapacity cv = Medium.specificHeatCapacityCv(state);
          Medium.IsentropicExponent gamma = Medium.isentropicExponent(state);
          Medium.SpecificEntropy s = Medium.specificEntropy(state);
          Medium.SpecificEntropy s2 = Medium.specificEntropy(medium2.state);
          Medium.VelocityOfSound a = Medium.velocityOfSound(state);
          Real beta = Medium.isobaricExpansionCoefficient(state);
          Real gamma2 = Medium.isothermalCompressibility(medium2.state);
          Medium.SpecificEnthalpy h_is = Medium.isentropicEnthalpyApproximation(200000.0, medium2);
          parameter Medium.MolarMass[4] MMx = Medium.data.MM "Molar masses of flue gas";
          Medium.MolarMass MM = 1 / sum(state.X[j] / MMx[j] for j in 1:4) "molar mass";
          Real[4] dddX = Medium.density_derX(medium2.state);
          annotation(Documentation(info = "<html>
<p>An example for using ideal gas properties and how to compute isentropic enthalpy changes.
The function that is implemented is approximate, but usually very good: the second medium record medium2
is given to compare the approximation.
</p>
</html>"), experiment(StopTime = 1));

        equation 
          der(state.p) = 1000.0;
          der(state.T) = 1000;
          state.X = {0.2,0.2,0.4,0.2};
          der(medium2.p) = 1000.0;
          der(medium2.T) = 0.0;
          der(medium2.X[1:Medium.nX]) = {0.0,0.0,0.0,0.0};
        end FlueGas;
        package N2AsMix "air and steam mixture (no condensation!, pseudo-mixture)"
          extends IdealGases.Common.MixtureGasNasa(mediumName = "Nitrogen", data = {IdealGases.Common.SingleGasesData.N2}, fluidConstants = {IdealGases.Common.FluidData.N2}, substanceNames = {"Nitrogen"}, reference_X = {1.0});
          annotation(Documentation(info = "<html>

</html>"));
        end N2AsMix;
        model IdealGasN2 "Test IdealGas.SingleMedia.N2 medium model"
          extends Modelica.Icons.Example;
          parameter Modelica.SIunits.Volume V = 1 "Size of fixed volume";
          parameter Medium.MassFlowRate m_flow_ext = 0.01 "Mass flow rate into volume";
          parameter Medium.EnthalpyFlowRate H_flow_ext = 5000 "Enthalpy flow rate into volume";
          package Medium = IdealGases.SingleGases.N2 "Medium model";
          Medium.BaseProperties medium(preferredMediumStates = true, p(start = 100000.0), T(start = 300));
          Real m(quantity = Medium.mediumName, start = 1.0);
          SI.InternalEnergy U;
          Medium.SpecificHeatCapacity cp = Medium.specificHeatCapacityCp(medium);
          Medium.SpecificHeatCapacity cv = Medium.specificHeatCapacityCv(medium);
          Medium.IsentropicExponent gamma = Medium.isentropicExponent(medium);
          Medium.SpecificEntropy s = Medium.specificEntropy(medium);
          Medium.VelocityOfSound a = Medium.velocityOfSound(medium);

        equation 
          m = medium.d * V;
          U = m * medium.u;
          der(m) = m_flow_ext;
          der(U) = H_flow_ext;
          annotation(Documentation(info = "<html>

</html>"), experiment(StopTime = 1));
        end IdealGasN2;
        package TestMedia "Test interfaces of media"
          extends Modelica.Icons.Library;
          model TemplateMedium "Test Interfaces.TemplateMedium"
            extends Modelica.Icons.Example;
            package Medium = Interfaces.TemplateMedium "Medium model";
            Medium.ThermodynamicState medium;
            Medium.DynamicViscosity eta = Medium.dynamicViscosity(medium);
            Medium.ThermalConductivity lambda = Medium.thermalConductivity(medium);
            Medium.SpecificEntropy s = Medium.specificEntropy(medium);
            Medium.SpecificHeatCapacity cp = Medium.specificHeatCapacityCp(medium);
            Medium.SpecificHeatCapacity cv = Medium.specificHeatCapacityCv(medium);
            Medium.IsentropicExponent gamma = Medium.isentropicExponent(medium);
            Medium.VelocityOfSound a = Medium.velocityOfSound(medium);

          equation 
            medium.p = 100000.0;
            medium.T = 300 + time / 1000;
            annotation(Documentation(info = "<html>

</html>"));
          end TemplateMedium;
          annotation(Documentation(info = "<html>

</html>"));
        end TestMedia;
        annotation(Documentation(info = "<html>

</html>"));
        model IdealGasN2Mix "Test IdealGas.SingleMedia.N2 medium model"
          extends Modelica.Icons.Example;
          parameter Modelica.SIunits.Volume V = 1 "Size of volume";
          parameter Medium.MassFlowRate m_flow_ext = 0.01 "Mass flow rate flowing into volume";
          parameter Medium.EnthalpyFlowRate H_flow_ext = 5000 "Enthalpy flow rate flowing into volume";
          package Medium = N2AsMix "Medium model";
          Medium.BaseProperties medium(preferredMediumStates = true, p(start = 100000.0), T(start = 300));
          Real m(quantity = Medium.mediumName, start = 1.0);
          SI.InternalEnergy U;
          Medium.SpecificHeatCapacity cp = Medium.specificHeatCapacityCp(medium);
          Medium.SpecificHeatCapacity cv = Medium.specificHeatCapacityCv(medium);
          Medium.IsentropicExponent gamma = Medium.isentropicExponent(medium);
          Medium.SpecificEntropy s = Medium.specificEntropy(medium);
          Medium.VelocityOfSound a = Medium.velocityOfSound(medium);

        equation 
          m = medium.d * V;
          U = m * medium.u;
          medium.X = {1.0};
          der(m) = m_flow_ext;
          der(U) = H_flow_ext;
          annotation(Documentation(info = "<html>

</html>"), experiment(StopTime = 1));
        end IdealGasN2Mix;
      end TestOnly;
      annotation(Documentation(info = "<html>
<h4>Examples</h4>
Physical properties for fluids are needed in so many different variants
that a library can only provide models for the most common situations.
With the following examples we are going to demonstrate how to use the
existing packages and functions in Modelica.Media to customize these
models for advanced applications. The high level functions try to
abstract as much as possible form the fact that different media are
based on different variables, e.g. ideal gases need pressure and
temperature, while many refrigerants are based on Helmholtz functions
of density and temperature, and many water proeprties are based on
pressure and specific enthalpy. Medium properties are needed in control
volumes in the dynamic state equations and in many thermodynamic state
locations that are independent of the dynamic states of a control
volume, e.g. at a wall temperature, an isentropic reference state or at
a phase boundary. The general structure of the library is such that:<br>
<ul>
<li>Each medium has a model called BaseProperties. BaseProperties
contains the minimum set of medium properties needed in a dynamic
control volume model.</li>
<li>Each instance of BaseProperties contains a \"state\" record that is
an input to all the functions to compute properties. If these functions
need further inputs, like e.g. the molarMass, these are accessible as
constants in the package.</li>
<li>The simplest way to compute properties at any other reference
point is to declare an instance of ThermodynamicState and use that as
input to arbitrary property functions.<br>
</li>
</ul>

<p>
A small library of generic volume, pipe, pump and ambient models
is provided in Modelica.Media.Examples.Tests.Components to demonstrate
how fluid components should be implemented that are using Modelica.Media
models. This library is also used to test all media models in
Modelica.Media.Examples.Tests.MediaTestModels.
</p>
</html>
"));
      package Tests "Library to test that all media models simulate and fulfill the expected structural properties"
        extends Modelica.Icons.Library;
        package Components "Functions, connectors and models needed for the media model tests"
          extends Modelica.Icons.Library;
          connector FluidPort "Interface for quasi one-dimensional fluid flow in a piping network (incompressible or compressible, one or more phases, one or more substances)"
            replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium model" annotation(__Dymola_choicesAllMatching = true);
            Medium.AbsolutePressure p "Pressure in the connection point";
            flow Medium.MassFlowRate m_flow "Mass flow rate from the connection point into the component";
            Medium.SpecificEnthalpy h "Specific mixture enthalpy in the connection point";
            flow Medium.EnthalpyFlowRate H_flow "Enthalpy flow rate into the component (if m_flow > 0, H_flow = m_flow*h)";
            Medium.MassFraction Xi[Medium.nXi] "Independent mixture mass fractions m_i/m in the connection point";
            flow Medium.MassFlowRate mXi_flow[Medium.nXi] "Mass flow rates of the independent substances from the connection point into the component (if m_flow > 0, mX_flow = m_flow*X)";
            Medium.ExtraProperty C[Medium.nC] "properties c_i/m in the connection point";
            flow Medium.ExtraPropertyFlowRate mC_flow[Medium.nC] "Flow rates of auxiliary properties from the connection point into the component (if m_flow > 0, mC_flow = m_flow*C)";
            annotation(Documentation(info = "<html>

</html>"));
          end FluidPort;
          connector FluidPort_a "Fluid connector with filled icon"
            extends FluidPort;
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -100,100},{100, -100}}, lineColor = {0,127,255}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -88,206},{112,112}}, textString = "%name", lineColor = {0,0,255})}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -100,100},{100, -100}}, lineColor = {0,127,255}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>Modelica.Media.Examples.Tests.Components.FluidPort_a
</html>"));
          end FluidPort_a;
          connector FluidPort_b "Fluid connector with outlined icon"
            extends FluidPort;
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -100,100},{100, -100}}, lineColor = {0,127,255}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -80,80},{80, -80}}, lineColor = {0,127,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -88,192},{112,98}}, textString = "%name", lineColor = {0,0,255})}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -100,100},{100, -100}}, lineColor = {0,127,255}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -80,80},{80, -80}}, lineColor = {0,127,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>

</html>"));
          end FluidPort_b;
          model PortVolume "Fixed volume associated with a port by the finite volume method"
            import SI = Modelica.SIunits;
            replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium model" annotation(__Dymola_choicesAllMatching = true);
            parameter SI.Volume V = 1e-06 "Fixed size of junction volume";
            parameter Boolean use_p_start = true "select p_start or d_start" annotation(Evaluate = true, Dialog(group = "Initial pressure or initial density"));
            parameter Medium.AbsolutePressure p_start = 101325 "Initial pressure" annotation(Dialog(group = "Initial pressure or initial density", enable = use_p_start));
            parameter Medium.Density d_start = 1 "Initial density" annotation(Dialog(group = "Initial pressure or initial density", enable = not use_p_start));
            parameter Boolean use_T_start = true "select T_start or h_start" annotation(Evaluate = true, Dialog(group = "Initial temperature or initial specific enthalpy"));
            parameter Medium.Temperature T_start = Modelica.SIunits.Conversions.from_degC(20) "Initial temperature" annotation(Dialog(group = "Initial temperature or initial specific enthalpy", enable = use_T_start));
            parameter Medium.SpecificEnthalpy h_start = 10000.0 "Initial specific enthalpy" annotation(Dialog(group = "Initial temperature or initial specific enthalpy", enable = not use_T_start));
            parameter Medium.MassFraction X_start[Medium.nX] "Initial mass fractions m_i/m" annotation(Dialog(group = "Only for multi-substance flow", enable = Medium.nX > 0));
            FluidPort_a port(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
            Medium.BaseProperties medium(preferredMediumStates = true);
            SI.Energy U "Internal energy of port volume";
            SI.Mass m "Mass of junction volume";
            SI.Mass mXi[Medium.nXi] "Independent substance masses of junction volume";
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {170,213,255}),Text(extent = {{ -144,178},{146,116}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -130, -108},{144, -150}}, lineColor = {0,0,0}, textString = "V=%V")}), Documentation(info = "<html>
<p>
This component models the <b>volume</b> of <b>fixed size</b> that is
associated with the <b>fluid port</b> to which it is connected.
This means that all medium properties inside the volume, are identical
to the port medium properties. In particular, the specific enthalpy
inside the volume (= medium.h) is always identical to the specific enthalpy
in the port (port.h = medium.h). Usually, this model is used when
discretizing a component according to the finite volume method into
volumes in internal ports that only store energy and mass and into
transport elements that just transport energy, mass and momentum
between the internal ports without storing these quantities during the
transport.
</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

          initial equation 
if not Medium.singleState then
                          if use_p_start then
                medium.p = p_start;

              else
                medium.d = d_start;

              end if;
            end if;
            if use_T_start then
              medium.T = T_start;

            else
              medium.h = h_start;

            end if;
            medium.Xi = X_start[1:Medium.nXi];

          equation 
            medium.p = port.p;
            medium.h = port.h;
            medium.Xi = port.Xi;
            m = V * medium.d;
            mXi = m * medium.Xi;
            U = m * medium.u;
            der(m) = port.m_flow;
            der(mXi) = port.mXi_flow;
            der(U) = port.H_flow;
          end PortVolume;
          model FixedMassFlowRate "Ideal pump that produces a constant mass flow rate from a large reservoir at fixed temperature and mass fraction"
            parameter Medium.MassFlowRate m_flow "Fixed mass flow rate from an infinite reservoir to the fluid port";
            parameter Boolean use_T_ambient = true "select T_ambient or h_ambient" annotation(Evaluate = true, Dialog(group = "Ambient temperature or ambient specific enthalpy"));
            parameter Medium.Temperature T_ambient = Modelica.SIunits.Conversions.from_degC(20) "Ambient temperature" annotation(Dialog(group = "Ambient temperature or ambient specific enthalpy", enable = use_T_ambient));
            parameter Medium.SpecificEnthalpy h_ambient = 10000.0 "Ambient specific enthalpy" annotation(Dialog(group = "Ambient temperature or ambient specific enthalpy", enable = not use_T_ambient));
            parameter Medium.MassFraction X_ambient[Medium.nX] "Ambient mass fractions m_i/m of reservoir";
            replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium model" annotation(__Dymola_choicesAllMatching = true);
            Medium.BaseProperties medium "Medium in the source";
            FluidPort_b port(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{20,60},{100, -60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{38,40},{100, -40}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,127,255}),Ellipse(extent = {{ -100,80},{60, -80}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{ -60,70},{60,0},{ -60, -68},{ -60,70}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -54,32},{16, -30}}, lineColor = {255,0,0}, textString = "m"),Text(extent = {{ -142,142},{156,88}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -154, -88},{150, -132}}, lineColor = {0,0,0}, textString = "%m_flow"),Ellipse(extent = {{ -26,30},{ -18,22}}, lineColor = {255,0,0}, fillColor = {255,0,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>

</html>"));

          equation 
            if use_T_ambient then
              medium.T = T_ambient;

            else
              medium.h = h_ambient;

            end if;
            medium.Xi = X_ambient[1:Medium.nXi];
            medium.p = port.p;
            port.m_flow =  -m_flow;
            port.mXi_flow = semiLinear(port.m_flow, port.Xi, medium.Xi);
            port.H_flow = semiLinear(port.m_flow, port.h, medium.h);
          end FixedMassFlowRate;
          model FixedAmbient "Ambient pressure, temperature and mass fraction source"
            replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium model" annotation(__Dymola_choicesAllMatching = true);
            parameter Boolean use_p_ambient = true "select p_ambient or d_ambient" annotation(Evaluate = true, Dialog(group = "Ambient pressure or ambient density"));
            parameter Medium.AbsolutePressure p_ambient = 101325 "Ambient pressure" annotation(Dialog(group = "Ambient pressure or ambient density", enable = use_p_ambient));
            parameter Medium.Density d_ambient = 1 "Ambient density" annotation(Dialog(group = "Ambient pressure or ambient density", enable = not use_p_ambient));
            parameter Boolean use_T_ambient = true "select T_ambient or h_ambient" annotation(Evaluate = true, Dialog(group = "Ambient temperature or ambient specific enthalpy"));
            parameter Medium.Temperature T_ambient = Modelica.SIunits.Conversions.from_degC(20) "Ambient temperature" annotation(Dialog(group = "Ambient temperature or ambient specific enthalpy", enable = use_T_ambient));
            parameter Medium.SpecificEnthalpy h_ambient = 10000.0 "Ambient specific enthalpy" annotation(Dialog(group = "Ambient temperature or ambient specific enthalpy", enable = not use_T_ambient));
            parameter Medium.MassFraction X_ambient[Medium.nX] "Ambient mass fractions m_i/m" annotation(Dialog(group = "Only for multi-substance flow", enable = Medium.nX > 0));
            Medium.BaseProperties medium "Medium in the source";
            FluidPort_b port(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Ellipse(extent = {{ -100,80},{100, -80}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {0,127,255}),Text(extent = {{ -136,144},{132,82}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<html>
<p>
Model <b>FixedAmbient_pt</b> defines constant values for ambient conditions:
</p>
<ul>
<li> Ambient pressure.</li>
<li> Ambient temperature.</li>
<li> Ambient mass fractions (only for multi-substance flow).</li>
</ul>
<p>
Note, that ambient temperature
and mass fractions have only an effect if the mass flow
is from the ambient into the port. If mass is flowing from
the port into the ambient, the ambient definitions,
with exception of ambient pressure, do not have an effect.
</p>
</html>"));

          equation 
            if use_p_ambient or Medium.singleState then
              medium.p = p_ambient;

            else
              medium.d = d_ambient;

            end if;
            if use_T_ambient then
              medium.T = T_ambient;

            else
              medium.h = h_ambient;

            end if;
            medium.Xi = X_ambient[1:Medium.nXi];
            port.p = medium.p;
            port.H_flow = semiLinear(port.m_flow, port.h, medium.h);
            port.mXi_flow = semiLinear(port.m_flow, port.Xi, medium.Xi);
          end FixedAmbient;
          model ShortPipe "Simple pressure loss in pipe"
            replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium model" annotation(__Dymola_choicesAllMatching = true);
            parameter Medium.AbsolutePressure dp_nominal(min = 1e-10) "Nominal pressure drop";
            parameter Medium.MassFlowRate m_flow_nominal(min = 1e-10) "Nominal mass flow rate at nominal pressure drop";
            FluidPort_a port_a(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -120, -10},{ -100,10}}, rotation = 0)));
            FluidPort_b port_b(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{120, -10},{100,10}}, rotation = 0)));
            Medium.MassFlowRate m_flow "Mass flow rate from port_a to port_b (m_flow > 0 is design flow direction)";
            Modelica.SIunits.Pressure dp "Pressure drop from port_a to port_b";
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,60},{100, -60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -100,34},{100, -36}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,127,255}),Text(extent = {{ -150,140},{150,80}}, lineColor = {0,0,0}, textString = "%name"),Text(extent = {{ -136, -62},{122, -108}}, lineColor = {0,0,0}, textString = "k=%m_flow_nominal/%dp_nominal")}), Documentation(info = "<html>
<p>
Model <b>ShortPipe</b> defines a simple pipe model
with pressure loss due to friction. It is assumed that
no mass or energy is stored in the pipe.
</p>
</html>"));

          equation 
            port_a.H_flow = semiLinear(port_a.m_flow, port_a.h, port_b.h);
            port_a.mXi_flow = semiLinear(port_a.m_flow, port_a.Xi, port_b.Xi);
            port_a.H_flow + port_b.H_flow = 0;
            port_a.m_flow + port_b.m_flow = 0;
            port_a.mXi_flow + port_b.mXi_flow = zeros(Medium.nXi);
            m_flow = port_a.m_flow;
            dp = port_a.p - port_b.p;
            m_flow = m_flow_nominal / dp_nominal * dp;
          end ShortPipe;
          partial model PartialTestModel "Basic test model to test a medium"
            import SI = Modelica.SIunits;
            extends Modelica.Icons.Example;
            replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium model" annotation(__Dymola_choicesAllMatching = true);
            parameter SI.AbsolutePressure p_start = Medium.p_default "Initial value of pressure";
            parameter SI.Temperature T_start = Medium.T_default "Initial value of temperature";
            parameter SI.SpecificEnthalpy h_start = Medium.h_default "Initial value of specific enthalpy";
            parameter Real X_start[Medium.nX] = Medium.X_default "Initial value of mass fractions";
            PortVolume volume(redeclare package Medium = Medium, p_start = p_start, T_start = T_start, h_start = h_start, X_start = X_start, V = 0.1) annotation(Placement(transformation(extent = {{ -40,0},{ -20,20}}, rotation = 0)));
            FixedMassFlowRate fixedMassFlowRate(redeclare package Medium = Medium, T_ambient = 1.2 * T_start, h_ambient = 1.2 * h_start, m_flow = 1, X_ambient = 0.5 * X_start) annotation(Placement(transformation(extent = {{ -80,0},{ -60,20}}, rotation = 0)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>

</html>"));
            FixedAmbient ambient(redeclare package Medium = Medium, T_ambient = T_start, h_ambient = h_start, X_ambient = X_start, p_ambient = p_start) annotation(Placement(transformation(extent = {{60,0},{40,20}}, rotation = 0)));
            ShortPipe shortPipe(redeclare package Medium = Medium, m_flow_nominal = 1, dp_nominal = 10000.0) annotation(Placement(transformation(extent = {{0,0},{20,20}}, rotation = 0)));

          equation 
            connect(fixedMassFlowRate.port,volume.port) annotation(Line(points = {{ -59,10},{ -30,10}}, color = {0,127,255}));
            connect(volume.port,shortPipe.port_a) annotation(Line(points = {{ -30,10},{ -1,10}}, color = {0,127,255}));
            connect(shortPipe.port_b,ambient.port) annotation(Line(points = {{21,10},{39,10}}, color = {0,127,255}));
          end PartialTestModel;
          partial model PartialTestModel2 "slightly larger test model to test a medium"
            import SI = Modelica.SIunits;
            extends Modelica.Icons.Example;
            replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium model" annotation(__Dymola_choicesAllMatching = true);
            parameter SI.AbsolutePressure p_start = 100000.0 "Initial value of pressure";
            parameter SI.Temperature T_start = 300 "Initial value of temperature";
            parameter SI.SpecificEnthalpy h_start = 1 "Initial value of specific enthalpy";
            parameter Real X_start[Medium.nX] = Medium.reference_X "Initial value of mass fractions";
            PortVolume volume(redeclare package Medium = Medium, p_start = p_start, T_start = T_start, h_start = h_start, X_start = X_start, V = 0.1) annotation(Placement(transformation(extent = {{ -60,0},{ -40,20}}, rotation = 0)));
            FixedMassFlowRate fixedMassFlowRate(redeclare package Medium = Medium, T_ambient = 1.2 * T_start, h_ambient = 1.2 * h_start, m_flow = 1, X_ambient = 0.5 * X_start) annotation(Placement(transformation(extent = {{ -100,0},{ -80,20}}, rotation = 0)));
            annotation(Diagram(graphics), Documentation(info = "<html>

</html>"));
            FixedAmbient ambient(redeclare package Medium = Medium, T_ambient = T_start, h_ambient = h_start, X_ambient = X_start, p_ambient = p_start) annotation(Placement(transformation(extent = {{92,0},{72,20}}, rotation = 0)));
            ShortPipe shortPipe(redeclare package Medium = Medium, m_flow_nominal = 1, dp_nominal = 10000.0) annotation(Placement(transformation(extent = {{ -30,0},{ -10,20}}, rotation = 0)));
            PortVolume volume1(redeclare package Medium = Medium, p_start = p_start, T_start = T_start, h_start = h_start, X_start = X_start, V = 0.1) annotation(Placement(transformation(extent = {{0,0},{20,20}}, rotation = 0)));
            ShortPipe shortPipe1(redeclare package Medium = Medium, m_flow_nominal = 1, dp_nominal = 10000.0) annotation(Placement(transformation(extent = {{36,0},{56,20}}, rotation = 0)));

          equation 
            connect(fixedMassFlowRate.port,volume.port) annotation(Line(points = {{ -79,10},{ -50,10}}, color = {0,127,255}));
            connect(volume.port,shortPipe.port_a) annotation(Line(points = {{ -50,10},{ -31,10}}, color = {0,127,255}));
            connect(volume1.port,shortPipe1.port_a) annotation(Line(points = {{10,10},{35,10}}, color = {0,127,255}));
            connect(shortPipe.port_b,volume1.port) annotation(Line(points = {{ -9,10},{10,10}}, color = {0,127,255}));
            connect(shortPipe1.port_b,ambient.port) annotation(Line(points = {{57,10},{71,10}}, color = {0,127,255}));
          end PartialTestModel2;
          annotation(Documentation(info = "<html>

</html>"));
        end Components;
        package MediaTestModels "Test models to test all media"
          extends Modelica.Icons.Library;
          package Air "Test models of library Modelica.Media.Air"
            extends Modelica.Icons.Library;
            model SimpleAir "Test Modelica.Media.Air.SimpleAir"
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.Air.SimpleAir);
              annotation(Documentation(info = "<html>

</html>"));
            end SimpleAir;
            model DryAirNasa "Test Modelica.Media.Air.DryAirNasa"
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.Air.DryAirNasa);
              annotation(Documentation(info = "<html>

</html>"));
            end DryAirNasa;
            model MoistAir "Test Modelica.Media.Air.MoistAir"
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.Air.MoistAir);
              annotation(Documentation(info = "<html>

</html>"));
            end MoistAir;
            annotation(Documentation(info = "<html>

</html>"));
          end Air;
          package IdealGases "Test models of library Modelica.Media.IdealGases"
            extends Modelica.Icons.Library;
            model Air "Test single gas Modelica.Media.IdealGases.SingleGases.Air"
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.Air.DryAirNasa);
              annotation(Documentation(info = "<html>

</html>"));
            end Air;
            model Nitrogen "Test single gas Modelica.Media.IdealGases.SingleGases.N2"
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.IdealGases.SingleGases.N2);
              annotation(Documentation(info = "<html>

</html>"));
            end Nitrogen;
            model SimpleNaturalGas "Test mixture gas Modelica.Media.IdealGases.MixtureGases.SimpleNaturalGas"
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.IdealGases.MixtureGases.SimpleNaturalGas);
              annotation(Documentation(info = "<html>

</html>"));
            end SimpleNaturalGas;
            model SimpleNaturalGasFixedComposition "Test mixture gas Modelica.Media.IdealGases.MixtureGases.SimpleNaturalGas"
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.IdealGases.MixtureGases.SimpleNaturalGasFixedComposition);
              annotation(experiment(StopTime = 1));
            end SimpleNaturalGasFixedComposition;
          end IdealGases;
          package Incompressible "Test models of library Modelica.Media.Incompressible"
            extends Modelica.Icons.Library;
            model Glycol47 "Test Modelica.Media.Incompressible.Examples.Glycol47"
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.Incompressible.Examples.Glycol47(final singleState = true, final enthalpyOfT = true));
              annotation(Documentation(info = "<html>

</html>"));
            end Glycol47;
            model Essotherm650 "Test Modelica.Media.Incompressible.Examples.Essotherm65"
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.Incompressible.Examples.Essotherm650);
              annotation(Documentation(info = "<html>

</html>"));
            end Essotherm650;
            annotation(Documentation(info = "<html>

</html>"));
          end Incompressible;
          package Water "Test models of library Modelica.Media.Water"
            extends Modelica.Icons.Library;
            model ConstantPropertyLiquidWater "Test Modelica.Media.Water.ConstantPropertyLiquidWater"
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater);
              annotation(Documentation(info = "<html>

</html>"));
            end ConstantPropertyLiquidWater;
            model IdealSteam "Test Modelica.Media.Water.IdealSteam"
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.Water.IdealSteam);
              annotation(Documentation(info = "<html>

</html>"));
            end IdealSteam;
            model WaterIF97OnePhase_ph "Test Modelica.Media.Water.WaterIF97OnePhase_ph"
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.Water.WaterIF97OnePhase_ph, fixedMassFlowRate(use_T_ambient = false, h_ambient = 363755), ambient(use_T_ambient = false, h_ambient = 112570));
              annotation(Documentation(info = "<html>

</html>"));
            end WaterIF97OnePhase_ph;
            model WaterIF97_pT "Test Modelica.Media.Water.WaterIF97_pT"
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.Water.WaterIF97_pT);
              annotation(Documentation(info = "<html>

</html>"));
            end WaterIF97_pT;
            model WaterIF97_ph "Test Modelica.Media.Water.WaterIF97_ph"
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.Water.WaterIF97_ph, ambient(use_T_ambient = false, h_ambient = 112570), fixedMassFlowRate(use_T_ambient = false, h_ambient = 363755));
              annotation(Documentation(info = "<html>

</html>"));
            end WaterIF97_ph;
            annotation(Documentation(info = "<html>

</html>"));
          end Water;
          annotation(Documentation(info = "<html>

</html>"));
          package LinearFluid "Test models of library Modelica.Media.Incompressible"
            extends Modelica.Icons.Library;
            model LinearColdWater "Test Modelica.Media.Incompressible.Examples.Glycol47"
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.CompressibleLiquids.LinearColdWater);
              annotation(Documentation(info = "<html>

</html>"));
            end LinearColdWater;
            model LinearWater_pT "Test Modelica.Media.Incompressible.Examples.Essotherm65"
              extends Modelica.Media.Examples.Tests.Components.PartialTestModel(redeclare package Medium = Modelica.Media.CompressibleLiquids.LinearWater_pT_Ambient);
              annotation(Documentation(info = "<html>

</html>"));
            end LinearWater_pT;
            annotation(Documentation(info = "<html>

</html>"));
          end LinearFluid;
        end MediaTestModels;
        annotation(Documentation(info = "<html>

</html>"));
      end Tests;
      package SolveOneNonlinearEquation "Demonstrate how to solve one non-linear algebraic equation in one unknown"
        annotation(Documentation(info = "<html>
<p>
This package demonstrates how to solve one non-linear algebraic
equation in one unknown with function
Modelica.Media.Common.OneNonLinearEquation.
</p>

</html>"));
        model Inverse_sine "Solve y = A*sin(w*x) for x, given y"
          import Modelica.Utilities.Streams.print;
          extends Modelica.Icons.Example;
          parameter Real y_zero = 0.5 "Desired value of A*sin(w*x)";
          parameter Real x_min =  -1.7 "Minimum value of x_zero";
          parameter Real x_max = 1.7 "Maximum value of x_zero";
          parameter Real A = 1;
          parameter Real w = 1;
          parameter Inverse_sine_definition.f_nonlinear_Data data = Inverse_sine_definition.f_nonlinear_Data(A = A, w = w);
          Real x_zero "y_zero = A*sin(w*x_zero)";
          encapsulated package Inverse_sine_definition "Define sine as non-linear equation to be solved"
            import Modelica;
            extends Modelica.Media.Common.OneNonLinearEquation;
            record extends f_nonlinear_Data "Data for nonlinear equation"
              Real A;
              Real w;
              annotation(Documentation(info = "<html>

</html>"));
            end f_nonlinear_Data;
            function extends f_nonlinear "Non-linear equation to be solved"
            algorithm 
              y:=f_nonlinear_data.A * Modelica.Math.sin(f_nonlinear_data.w * x);
            end f_nonlinear;
            function extends solve "Solution algorithm of non-linear equation"
            end solve;
            annotation(Documentation(info = "<html>

</html>"));
          end Inverse_sine_definition;

        equation 
          x_zero = Inverse_sine_definition.solve(y_zero, x_min, x_max, f_nonlinear_data = data);
          print("x_zero = " + String(x_zero) + ", y_zero = " + String(y_zero) + ", A*sin(w*x_zero) = " + String(data.A * Modelica.Math.sin(data.w * x_zero)));
          annotation(experiment(StopTime = 0), Documentation(info = "<html>
<p>
This models solves the following non-linear equation
</p>

<pre>
   y = A*sin(w*x); -> determine x for given y
</pre>

<p>
Translate model \"Inverse_sine\"
and simulate for 0 sec. The result is printed to the
output window.
</p>

</html>"));
        end Inverse_sine;
        model Inverse_sh_T "Solve h = h_T(T), s = s_T(T) for T, if h or s is given for ideal gas NASA"
          import SI = Modelica.SIunits;
          extends Modelica.Icons.Example;
          replaceable package Medium = Modelica.Media.Air.DryAirNasa extends Modelica.Media.IdealGases.Common.SingleGasNasa "Medium model" annotation(__Dymola_choicesAllMatching = true);
          parameter SI.Temperature T_min = 300 "Vary temperature linearly from T_min (time=0) upto T_max (time=1)";
          parameter SI.Temperature T_max = 500 "Vary temperature linearly from T_min (time=0) upto T_max (time=1)";
          parameter SI.Pressure p = 100000.0 "Fixed pressure in model";
          final parameter SI.SpecificEnthalpy h_min = Medium.h_T(Medium.data, T_min) "Specific enthalpy at T_min";
          final parameter SI.SpecificEnthalpy h_max = Medium.h_T(Medium.data, T_max) "Specific enthalpy at T_max";
          final parameter SI.SpecificEntropy s_min = Medium.specificEntropy(Medium.setState_pT(p, T_min)) "Specific entropy at T_min";
          final parameter SI.SpecificEntropy s_max = Medium.specificEntropy(Medium.setState_pT(p, T_max)) "Specific entropy at T_max";
          SI.SpecificEnthalpy h1 "Pre-defined specific enthalpy";
          SI.SpecificEnthalpy h2 "Specific enthalpy computed from T (= h1 required)";
          SI.SpecificEntropy s1 "Pre-defined specific entropy";
          SI.SpecificEntropy s2 "Specific entropy computed from T (= h1 required)";
          SI.Temperature Th "Temperature computed from h1";
          SI.Temperature Ts "Temperature computed from s1";
          annotation(experiment(StopTime = 1), Documentation(info = "<html>
                               </html>"));
        protected 
          constant SI.Time timeUnit = 1.0;

        equation 
          h1 = if time < 0 then h_min else if time > 1 then h_max else h_min + time / timeUnit * (h_max - h_min);
          s1 = if time < 0 then s_min else if time > 1 then s_max else s_min + time / timeUnit * (s_max - s_min);
          Th = Medium.temperature_phX(p, h1, fill(0.0, 0));
          Ts = Medium.temperature_psX(p, s1, fill(0.0, 0));
          h2 = Medium.specificEnthalpy_pTX(p, Th, fill(0.0, 0));
          s2 = Medium.specificEntropy(Medium.setState_pT(p, Ts));
        end Inverse_sh_T;
        model InverseIncompressible_sh_T "inverse computation for incmpressible media"
          import SI = Modelica.SIunits;
          import Cv = Modelica.SIunits.Conversions;
          extends Modelica.Icons.Example;
          replaceable package Medium = Modelica.Media.Incompressible.Examples.Glycol47 "Medium model" annotation(__Dymola_choicesAllMatching = true);
          parameter SI.Temperature T_min = Medium.T_min "Vary temperature linearly from T_min (time=0) upto T_max (time=1)";
          parameter SI.Temperature T_max = Medium.T_max "Vary temperature linearly from T_min (time=0) upto T_max (time=1)";
          parameter SI.Pressure p = 100000.0 "Fixed pressure in model";
          final parameter SI.SpecificEnthalpy h_min = Medium.h_T(Medium.T_min) "Specific enthalpy at T_min";
          final parameter SI.SpecificEnthalpy h_max = Medium.h_T(Medium.T_max) "Specific enthalpy at T_max";
          final parameter SI.SpecificEntropy s_min = Medium.specificEntropy(Medium.setState_pT(p, T_min)) "Specific entropy at T_min";
          final parameter SI.SpecificEntropy s_max = Medium.specificEntropy(Medium.setState_pT(p, T_max)) "Specific entropy at T_max";
          SI.SpecificEnthalpy h1 "Pre-defined specific enthalpy";
          SI.SpecificEnthalpy h2 "Specific enthalpy computed from T (= h1 required)";
          SI.SpecificEntropy s1 "Pre-defined specific entropy";
          SI.SpecificEntropy s2 "Specific entropy computed from T (= h1 required)";
          SI.Temperature Th "Temperature computed from h1";
          SI.Temperature Ts "Temperature computed from s1";
          annotation(experiment(StopTime = 1), Documentation(info = "<html>

                               </html>"));
        protected 
          constant SI.Time timeUnit = 1.0;

        equation 
          h1 = if time < 0 then h_min else if time > 1 then h_max else h_min + time / timeUnit * (h_max - h_min);
          s1 = if time < 0 then s_min else if time > 1 then s_max else s_min + time / timeUnit * (s_max - s_min);
          Th = Medium.temperature_phX(p, h1, fill(0.0, 0));
          Ts = Medium.temperature_psX(p, s1, fill(0.0, 0));
          h2 = Medium.specificEnthalpy_pTX(p, Th, fill(0.0, 0));
          s2 = Medium.specificEntropy(Medium.setState_pT(p, Ts));
        end InverseIncompressible_sh_T;
        model Inverse_sh_TX "Solve h = h_TX(TX) for T, if h is given for ideal gas NASA"
          import SI = Modelica.SIunits;
          extends Modelica.Icons.Example;
          replaceable package Medium = Modelica.Media.IdealGases.MixtureGases.FlueGasLambdaOnePlus extends Modelica.Media.IdealGases.Common.MixtureGasNasa "Medium model" annotation(__Dymola_choicesAllMatching = true);
          parameter SI.Temperature T_min = 300 "Vary temperature linearly from T_min (time=0) upto T_max (time=1)";
          parameter SI.Temperature T_max = 500 "Vary temperature linearly from T_min (time=0) upto T_max (time=1)";
          parameter SI.Pressure p = 100000.0 "Fixed pressure in model";
          SI.SpecificEnthalpy h_min = Medium.h_TX(T_min, X) "Specific enthalpy at T_min";
          SI.SpecificEnthalpy h_max = Medium.h_TX(T_max, X) "Specific enthalpy at T_max";
          SI.SpecificEntropy s_min = Medium.specificEntropy(Medium.setState_pTX(p, T_min, Medium.reference_X)) "Specific entropy at T_min";
          SI.SpecificEntropy s_max = Medium.specificEntropy(Medium.setState_pTX(p, T_max, Medium.reference_X)) "Specific entropy at T_max";
          SI.SpecificEnthalpy h1 "Pre-defined specific enthalpy";
          SI.SpecificEnthalpy h2 "Specific enthalpy computed from T (= h1 required)";
          SI.SpecificEntropy s1 "Pre-defined specific entropy";
          SI.SpecificEntropy s2 "Specific entropy computed from T (= h1 required)";
          SI.Temperature Th "Temperature computed from h1";
          SI.Temperature Ts "Temperature computed from s1";
          SI.MassFraction[4] X "mass fraction vector";
          annotation(experiment(StopTime = 1), Documentation(info = "<html>

</html>"));
        protected 
          constant SI.Time timeUnit = 1.0;

        equation 
          X = Medium.reference_X;
          h1 = if time < 0 then h_min else if time > 1 then h_max else h_min + time / timeUnit * (h_max - h_min);
          s1 = if time < 0 then s_min else if time > 1 then s_max else s_min + time / timeUnit * (s_max - s_min);
          Th = Medium.temperature_phX(p, h1, X);
          Ts = Medium.temperature_psX(p, s1, X);
          h2 = Medium.specificEnthalpy_pTX(p, Th, X);
          s2 = Medium.specificEntropy(Medium.setState_pTX(p, Ts, X));
        end Inverse_sh_TX;
      end SolveOneNonlinearEquation;
    end Examples;
    package Interfaces "Interfaces for media models"
      annotation(Documentation(info = "<HTML>
<p>
This package provides basic interfaces definitions of media models for different
kind of media.
</p>
</HTML>"));
      extends Modelica.Icons.Library;
      import SI = Modelica.SIunits;
      partial package TemplateMedium "Template for media models"
        extends Modelica.Media.Interfaces.PartialMedium(final mediumName = "NameOfMedium", final substanceNames = {mediumName}, final singleState = false, final reducedX = true, final fixedX = true, Temperature(min = 273, max = 450, start = 323));
        constant SpecificHeatCapacity cp_const = 123456 "Constant specific heat capacity at constant pressure";
        model extends BaseProperties(final standardOrderComponents = true) "Base properties of medium"
          annotation(Documentation(info = "<html></html>"));

        equation 
          d = 1;
          h = cp_const * T;
          u = h - p / d;
          MM = 0.024;
          R = 8.3144 / MM;
          state.p = p;
          state.T = T;
          annotation(Documentation(revisions = "<html>

</html>"));
        end BaseProperties;
        redeclare replaceable record ThermodynamicState "a selction of variables that uniquely defines the thermodynamic state"
          AbsolutePressure p "Absolute pressure of medium";
          Temperature T "Temperature of medium";
          annotation(Documentation(info = "<html>

</html>"));
        end ThermodynamicState;
        function extends dynamicViscosity "Return dynamic viscosity"
        algorithm 
          eta:=10 - state.T * 0.3 + state.p * 0.2;
          annotation(Documentation(info = "<html>

</html>"));
        end dynamicViscosity;
        function extends thermalConductivity "Return thermal conductivity"
        algorithm 
          lambda:=0;
          annotation(Documentation(info = "<html>

</html>"));
        end thermalConductivity;
        function extends specificEntropy "Return specific entropy"
        algorithm 
          s:=0;
          annotation(Documentation(info = "<html>

</html>"));
        end specificEntropy;
        function extends specificHeatCapacityCp "Return specific heat capacity at constant pressure"
        algorithm 
          cp:=0;
          annotation(Documentation(info = "<html>

</html>"));
        end specificHeatCapacityCp;
        function extends specificHeatCapacityCv "Return specific heat capacity at constant volume"
        algorithm 
          cv:=0;
          annotation(Documentation(info = "<html>

</html>"));
        end specificHeatCapacityCv;
        function extends isentropicExponent "Return isentropic exponent"
          extends Modelica.Icons.Function;
        algorithm 
          gamma:=1;
          annotation(Documentation(info = "<html>

</html>"));
        end isentropicExponent;
        function extends velocityOfSound "Return velocity of sound"
          extends Modelica.Icons.Function;
        algorithm 
          a:=0;
          annotation(Documentation(info = "<html>

</html>"));
        end velocityOfSound;
        annotation(Documentation(info = "<HTML>
<p>
This package is a <b>template</b> for <b>new medium</b> models. For a new
medium model just make a copy of this package, remove the
\"partial\" keyword from the package and provide
the information that is requested in the comments of the
Modelica source.
</p>
</HTML>"));
      end TemplateMedium;
      partial package PartialMedium "Partial medium properties (base package of all media packages)"
        import SI = Modelica.SIunits;
        extends Modelica.Icons.Library;
        constant Modelica.Media.Interfaces.PartialMedium.Choices.IndependentVariables ThermoStates "Enumeration type for independent variables";
        constant String mediumName = "unusablePartialMedium" "Name of the medium";
        constant String substanceNames[:] = {mediumName} "Names of the mixture substances. Set substanceNames={mediumName} if only one substance.";
        constant String extraPropertiesNames[:] = fill("", 0) "Names of the additional (extra) transported properties. Set extraPropertiesNames=fill(\"\",0) if unused";
        constant Boolean singleState "= true, if u and d are not a function of pressure";
        constant Boolean reducedX = true "= true if medium contains the equation sum(X) = 1.0; set reducedX=true if only one substance (see docu for details)";
        constant Boolean fixedX = false "= true if medium contains the equation X = reference_X";
        constant AbsolutePressure reference_p = 101325 "Reference pressure of Medium: default 1 atmosphere";
        constant Temperature reference_T = 298.15 "Reference temperature of Medium: default 25 deg Celsius";
        constant MassFraction reference_X[nX] = fill(1 / nX, nX) "Default mass fractions of medium";
        constant AbsolutePressure p_default = 101325 "Default value for pressure of medium (for initialization)";
        constant Temperature T_default = Modelica.SIunits.Conversions.from_degC(20) "Default value for temperature of medium (for initialization)";
        constant SpecificEnthalpy h_default = specificEnthalpy_pTX(p_default, T_default, X_default) "Default value for specific enthalpy of medium (for initialization)";
        constant MassFraction X_default[nX] = reference_X "Default value for mass fractions of medium (for initialization)";
        final constant Integer nS = size(substanceNames, 1) "Number of substances" annotation(Evaluate = true);
        constant Integer nX = nS "Number of mass fractions" annotation(Evaluate = true);
        constant Integer nXi = if fixedX then 0 else if reducedX then nS - 1 else nS "Number of structurally independent mass fractions (see docu for details)" annotation(Evaluate = true);
        final constant Integer nC = size(extraPropertiesNames, 1) "Number of extra (outside of standard mass-balance) transported properties" annotation(Evaluate = true);
        replaceable record FluidConstants "critical, triple, molecular and other standard data of fluid"
          extends Modelica.Icons.Record;
          String iupacName "complete IUPAC name (or common name, if non-existent)";
          String casRegistryNumber "chemical abstracts sequencing number (if it exists)";
          String chemicalFormula "Chemical formula, (brutto, nomenclature according to Hill";
          String structureFormula "Chemical structure formula";
          MolarMass molarMass "molar mass";
          annotation(Documentation(info = "<html></html>"));
        end FluidConstants;
        replaceable record ThermodynamicState "Minimal variable set that is available as input argument to every medium function"
          extends Modelica.Icons.Record;
          annotation(Documentation(info = "<html></html>"));
        end ThermodynamicState;
        replaceable partial model BaseProperties "Base properties (p, d, T, h, u, R, MM and, if applicable, X and Xi) of a medium"
          InputAbsolutePressure p "Absolute pressure of medium";
          InputMassFraction[nXi] Xi(start = reference_X[1:nXi]) "Structurally independent mass fractions";
          InputSpecificEnthalpy h "Specific enthalpy of medium";
          Density d "Density of medium";
          Temperature T "Temperature of medium";
          MassFraction[nX] X(start = reference_X) "Mass fractions (= (component mass)/total mass  m_i/m)";
          SpecificInternalEnergy u "Specific internal energy of medium";
          SpecificHeatCapacity R "Gas constant (of mixture if applicable)";
          MolarMass MM "Molar mass (of mixture or single fluid)";
          ThermodynamicState state "thermodynamic state record for optional functions";
          parameter Boolean preferredMediumStates = false "= true if StateSelect.prefer shall be used for the independent property variables of the medium" annotation(Evaluate = true, Dialog(tab = "Advanced"));
          parameter Boolean standardOrderComponents = true "if true, and reducedX = true, the last element of X will be computed from the other ones";
          SI.Conversions.NonSIunits.Temperature_degC T_degC = Modelica.SIunits.Conversions.to_degC(T) "Temperature of medium in [degC]";
          SI.Conversions.NonSIunits.Pressure_bar p_bar = Modelica.SIunits.Conversions.to_bar(p) "Absolute pressure of medium in [bar]";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -152,164},{152,102}}, textString = "%name", lineColor = {0,0,255})}));
          connector InputAbsolutePressure = input SI.AbsolutePressure "Pressure as input signal connector";
          connector InputSpecificEnthalpy = input SI.SpecificEnthalpy "Specific enthalpy as input signal connector";
          connector InputMassFraction = input SI.MassFraction "Mass fraction as input signal connector";

        equation 
if standardOrderComponents then
                      Xi = X[1:nXi];
if fixedX then
                          X = reference_X;
            end if;
if reducedX and not fixedX then
                          X[nX] = 1 - sum(Xi);
            end if;
            for i in 1:nX loop
            assert(X[i] >=  -1e-05 and X[i] <= 1 + 1e-05, "Mass fraction X[" + String(i) + "] = " + String(X[i]) + "of substance " + substanceNames[i] + "\nof medium " + mediumName + " is not in the range 0..1");

            end for;
          end if;
          assert(p >= 0.0, "Pressure (= " + String(p) + " Pa) of medium \"" + mediumName + "\" is negative\n(Temperature = " + String(T) + " K)");
          annotation(Documentation(info = "<html>
<p>
Model <b>BaseProperties</b> is a model within package <b>PartialMedium</b>
and contains the <b>declarations</b> of the minimum number of
variables that every medium model is supposed to support.
A specific medium inherits from model <b>BaseProperties</b> and provides
the equations for the basic properties. </p>
<p>
The BaseProperties model contains the following <b>7+nXi variables</b>
(nXi is the number of independent mass fractions defined in package
PartialMedium):
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Variable</b></td>
      <td valign=\"top\"><b>Unit</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">T</td>
      <td valign=\"top\">K</td>
      <td valign=\"top\">temperature</td></tr>
  <tr><td valign=\"top\">p</td>
      <td valign=\"top\">Pa</td>
      <td valign=\"top\">absolute pressure</td></tr>
  <tr><td valign=\"top\">d</td>
      <td valign=\"top\">kg/m^3</td>
      <td valign=\"top\">density</td></tr>
  <tr><td valign=\"top\">h</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific enthalpy</td></tr>
  <tr><td valign=\"top\">u</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific internal energy</td></tr>
  <tr><td valign=\"top\">Xi[nXi]</td>
      <td valign=\"top\">kg/kg</td>
      <td valign=\"top\">independent mass fractions m_i/m</td></tr>
  <tr><td valign=\"top\">R</td>
      <td valign=\"top\">J/kg.K</td>
      <td valign=\"top\">gas constant</td></tr>
  <tr><td valign=\"top\">M</td>
      <td valign=\"top\">kg/mol</td>
      <td valign=\"top\">molar mass</td></tr>
</table>
<p>
In order to implement an actual medium model, one can extend from this
base model and add <b>5 equations</b> that provide relations among
these variables. Equations will also have to be added in order to
set all the variables within the ThermodynamicState record state.</p>
<p>
If standardOrderComponents=true, the full composition vector X[nX]
is determined by the equations contained in this base class, depending
on the independent mass fraction vector Xi[nXi]. </p>
<p>Additional <b>2 + nXi</b> equations will have to be provided
when using the BaseProperties model, in order to fully specify the
thermodynamic conditions. The input connector qualifier applied to
p, h, and nXi indirectly declares the number of missing equations,
permitting advanced equation balance checking by Modelica tools.
Please note that this doesn't mean that the additional equations
should be connection equations, nor that exactly those variables
should be supplied, in order to complete the model.
For further information, see the Modelica.Media User's guide, and
Section 4.7 (Balanced Models) of the Modelica 3.0 specification. </p>
</html>"));
        end BaseProperties;
        replaceable partial function setState_pTX "Return thermodynamic state as function of p, T and composition X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "thermodynamic state record";
          annotation(Documentation(info = "<html></html>"));
        end setState_pTX;
        replaceable partial function setState_phX "Return thermodynamic state as function of p, h and composition X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "thermodynamic state record";
          annotation(Documentation(info = "<html></html>"));
        end setState_phX;
        replaceable partial function setState_psX "Return thermodynamic state as function of p, s and composition X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "thermodynamic state record";
          annotation(Documentation(info = "<html></html>"));
        end setState_psX;
        replaceable partial function setState_dTX "Return thermodynamic state as function of d, T and composition X or Xi"
          extends Modelica.Icons.Function;
          input Density d "density";
          input Temperature T "Temperature";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "thermodynamic state record";
          annotation(Documentation(info = "<html></html>"));
        end setState_dTX;
        replaceable partial function setSmoothState "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
          extends Modelica.Icons.Function;
          input Real x "m_flow or dp";
          input ThermodynamicState state_a "Thermodynamic state if x > 0";
          input ThermodynamicState state_b "Thermodynamic state if x < 0";
          input Real x_small(min = 0) "Smooth transition in the region -x_small < x < x_small";
          output ThermodynamicState state "Smooth thermodynamic state for all x (continuous and differentiable)";
          annotation(Documentation(info = "<html>
<p>
This function is used to approximate the equation
</p>
<pre>
    state = <b>if</b> x &gt; 0 <b>then</b> state_a <b>else</b> state_b;
</pre>

<p>
by a smooth characteristic, so that the expression is continuous and differentiable:
</p>

<pre>
   state := <b>smooth</b>(1, <b>if</b> x &gt;  x_small <b>then</b> state_a <b>else</b>
                      <b>if</b> x &lt; -x_small <b>then</b> state_b <b>else</b> f(state_a, state_b));
</pre>

<p>
This is performed by applying function <b>Media.Common.smoothStep</b>(..)
on every element of the thermodynamic state record.
</p>

<p>
If <b>mass fractions</b> X[:] are approximated with this function then this can be performed
for all <b>nX</b> mass fractions, instead of applying it for nX-1 mass fractions and computing
the last one by the mass fraction constraint sum(X)=1. The reason is that the approximating function has the
property that sum(state.X) = 1, provided sum(state_a.X) = sum(state_b.X) = 1.
This can be shown by evaluating the approximating function in the abs(x) &lt; x_small
region (otherwise state.X is either state_a.X or state_b.X):
</p>

<pre>
    X[1]  = smoothStep(x, X_a[1] , X_b[1] , x_small);
    X[2]  = smoothStep(x, X_a[2] , X_b[2] , x_small);
       ...
    X[nX] = smoothStep(x, X_a[nX], X_b[nX], x_small);
</pre>

<p>
or
</p>

<pre>
    X[1]  = c*(X_a[1]  - X_b[1])  + (X_a[1]  + X_b[1])/2
    X[2]  = c*(X_a[2]  - X_b[2])  + (X_a[2]  + X_b[2])/2;
       ...
    X[nX] = c*(X_a[nX] - X_b[nX]) + (X_a[nX] + X_b[nX])/2;
    c     = (x/x_small)*((x/x_small)^2 - 3)/4
</pre>

<p>
Summing all mass fractions together results in
</p>

<pre>
    sum(X) = c*(sum(X_a) - sum(X_b)) + (sum(X_a) + sum(X_b))/2
           = c*(1 - 1) + (1 + 1)/2
           = 1
</pre>

</html>"));
        end setSmoothState;
        replaceable partial function dynamicViscosity "Return dynamic viscosity"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "thermodynamic state record";
          output DynamicViscosity eta "Dynamic viscosity";
          annotation(Documentation(info = "<html></html>"));
        end dynamicViscosity;
        replaceable partial function thermalConductivity "Return thermal conductivity"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "thermodynamic state record";
          output ThermalConductivity lambda "Thermal conductivity";
          annotation(Documentation(info = "<html></html>"));
        end thermalConductivity;
        replaceable function prandtlNumber "Return the Prandtl number"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "thermodynamic state record";
          output PrandtlNumber Pr "Prandtl number";
        algorithm 
          Pr:=(dynamicViscosity(state) * specificHeatCapacityCp(state)) / thermalConductivity(state);
        end prandtlNumber;
        replaceable partial function pressure "Return pressure"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "thermodynamic state record";
          output AbsolutePressure p "Pressure";
          annotation(Documentation(info = "<html></html>"));
        end pressure;
        replaceable partial function temperature "Return temperature"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "thermodynamic state record";
          output Temperature T "Temperature";
        end temperature;
        replaceable partial function density "Return density"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "thermodynamic state record";
          output Density d "Density";
          annotation(Documentation(info = "<html></html>"));
        end density;
        replaceable partial function specificEnthalpy "Return specific enthalpy"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "thermodynamic state record";
          output SpecificEnthalpy h "Specific enthalpy";
          annotation(Documentation(info = "<html></html>"));
        end specificEnthalpy;
        replaceable partial function specificInternalEnergy "Return specific internal energy"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "thermodynamic state record";
          output SpecificEnergy u "Specific internal energy";
          annotation(Documentation(info = "<html></html>"));
        end specificInternalEnergy;
        replaceable partial function specificEntropy "Return specific entropy"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "thermodynamic state record";
          output SpecificEntropy s "Specific entropy";
          annotation(Documentation(info = "<html></html>"));
        end specificEntropy;
        replaceable partial function specificGibbsEnergy "Return specific Gibbs energy"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "thermodynamic state record";
          output SpecificEnergy g "Specific Gibbs energy";
          annotation(Documentation(info = "<html></html>"));
        end specificGibbsEnergy;
        replaceable partial function specificHelmholtzEnergy "Return specific Helmholtz energy"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "thermodynamic state record";
          output SpecificEnergy f "Specific Helmholtz energy";
          annotation(Documentation(info = "<html></html>"));
        end specificHelmholtzEnergy;
        replaceable partial function specificHeatCapacityCp "Return specific heat capacity at constant pressure"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "thermodynamic state record";
          output SpecificHeatCapacity cp "Specific heat capacity at constant pressure";
          annotation(Documentation(info = "<html></html>"));
        end specificHeatCapacityCp;
        function heatCapacity_cp = specificHeatCapacityCp "alias for deprecated name";
        replaceable partial function specificHeatCapacityCv "Return specific heat capacity at constant volume"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "thermodynamic state record";
          output SpecificHeatCapacity cv "Specific heat capacity at constant volume";
          annotation(Documentation(info = "<html></html>"));
        end specificHeatCapacityCv;
        function heatCapacity_cv = specificHeatCapacityCv "alias for deprecated name";
        replaceable partial function isentropicExponent "Return isentropic exponent"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "thermodynamic state record";
          output IsentropicExponent gamma "Isentropic exponent";
          annotation(Documentation(info = "<html></html>"));
        end isentropicExponent;
        replaceable partial function isentropicEnthalpy "Return isentropic enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p_downstream "downstream pressure";
          input ThermodynamicState refState "reference state for entropy";
          output SpecificEnthalpy h_is "Isentropic enthalpy";
          annotation(Documentation(info = "<html>
<p>
This function computes an isentropic state transformation: 
</p>
<ol>
<li> A medium is in a particular state, refState.</li>
<li> The enhalpy at another state (h_is) shall be computed
     under the assumption that the state transformation from refState to h_is
     is performed with a change of specific entropy ds = 0 and the pressure of state h_is
     is p_downstream and the composition X upstream and downstream is assumed to be the same.</li>
</ol>

</html>"));
        end isentropicEnthalpy;
        replaceable partial function velocityOfSound "Return velocity of sound"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "thermodynamic state record";
          output VelocityOfSound a "Velocity of sound";
          annotation(Documentation(info = "<html></html>"));
        end velocityOfSound;
        replaceable partial function isobaricExpansionCoefficient "Return overall the isobaric expansion coefficient beta"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "thermodynamic state record";
          output IsobaricExpansionCoefficient beta "Isobaric expansion coefficient";
          annotation(Documentation(info = "<html>
<pre>
beta is defined as  1/v * der(v,T), with v = 1/d, at constant pressure p.
</pre>
</html>"));
        end isobaricExpansionCoefficient;
        function beta = isobaricExpansionCoefficient "alias for isobaricExpansionCoefficient for user convenience";
        replaceable partial function isothermalCompressibility "Return overall the isothermal compressibility factor"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "thermodynamic state record";
          output SI.IsothermalCompressibility kappa "Isothermal compressibility";
          annotation(Documentation(info = "<html>
<pre>

kappa is defined as - 1/v * der(v,p), with v = 1/d at constant temperature T.

</pre>
</html>"));
        end isothermalCompressibility;
        function kappa = isothermalCompressibility "alias of isothermalCompressibility for user convenience";
        replaceable partial function density_derp_h "Return density derivative wrt pressure at const specific enthalpy"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "thermodynamic state record";
          output DerDensityByPressure ddph "Density derivative wrt pressure";
          annotation(Documentation(info = "<html></html>"));
        end density_derp_h;
        replaceable partial function density_derh_p "Return density derivative wrt specific enthalpy at constant pressure"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "thermodynamic state record";
          output DerDensityByEnthalpy ddhp "Density derivative wrt specific enthalpy";
          annotation(Documentation(info = "<html></html>"));
        end density_derh_p;
        replaceable partial function density_derp_T "Return density derivative wrt pressure at const temperature"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "thermodynamic state record";
          output DerDensityByPressure ddpT "Density derivative wrt pressure";
          annotation(Documentation(info = "<html></html>"));
        end density_derp_T;
        replaceable partial function density_derT_p "Return density derivative wrt temperature at constant pressure"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "thermodynamic state record";
          output DerDensityByTemperature ddTp "Density derivative wrt temperature";
          annotation(Documentation(info = "<html></html>"));
        end density_derT_p;
        replaceable partial function density_derX "Return density derivative wrt mass fraction"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "thermodynamic state record";
          output Density[nX] dddX "Derivative of density wrt mass fraction";
          annotation(Documentation(info = "<html></html>"));
        end density_derX;
        replaceable partial function molarMass "Return the molar mass of the medium"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "thermodynamic state record";
          output MolarMass MM "Mixture molar mass";
          annotation(Documentation(info = "<html></html>"));
        end molarMass;
        replaceable function specificEnthalpy_pTX "Return specific enthalpy from p, T, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input MassFraction X[:] = reference_X "Mass fractions";
          output SpecificEnthalpy h "Specific enthalpy";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          h:=specificEnthalpy(setState_pTX(p, T, X));
        end specificEnthalpy_pTX;
        replaceable function density_pTX "Return density from p, T, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input MassFraction X[:] "Mass fractions";
          output Density d "Density";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          d:=density(setState_pTX(p, T, X));
        end density_pTX;
        replaceable function temperature_phX "Return temperature from p, h, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output Temperature T "Temperature";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          T:=temperature(setState_phX(p, h, X));
        end temperature_phX;
        replaceable function density_phX "Return density from p, h, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output Density d "Density";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          d:=density(setState_phX(p, h, X));
        end density_phX;
        replaceable function temperature_psX "Return temperature from p,s, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output Temperature T "Temperature";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          T:=temperature(setState_psX(p, s, X));
        end temperature_psX;
        replaceable function density_psX "Return density from p, s, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output Density d "Density";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          d:=density(setState_psX(p, s, X));
        end density_psX;
        replaceable function specificEnthalpy_psX "Return specific enthalpy from p, s, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output SpecificEnthalpy h "Specific enthalpy";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          h:=specificEnthalpy(setState_psX(p, s, X));
        end specificEnthalpy_psX;
        type AbsolutePressure = SI.AbsolutePressure(min = 0, max = 100000000.0, nominal = 100000.0, start = 100000.0) "Type for absolute pressure with medium specific attributes";
        type Density = SI.Density(min = 0, max = 100000.0, nominal = 1, start = 1) "Type for density with medium specific attributes";
        type DynamicViscosity = SI.DynamicViscosity(min = 0, max = 100000000.0, nominal = 0.001, start = 0.001) "Type for dynamic viscosity with medium specific attributes";
        type EnthalpyFlowRate = SI.EnthalpyFlowRate(nominal = 1000.0, min =  -100000000.0, max = 100000000.0) "Type for enthalpy flow rate with medium specific attributes";
        type MassFlowRate = SI.MassFlowRate(quantity = "MassFlowRate." + mediumName, min =  -100000.0, max = 100000.0) "Type for mass flow rate with medium specific attributes";
        type MassFraction = Real(quantity = "MassFraction", final unit = "kg/kg", min = 0, max = 1, nominal = 0.1) "Type for mass fraction with medium specific attributes";
        type MoleFraction = Real(quantity = "MoleFraction", final unit = "mol/mol", min = 0, max = 1, nominal = 0.1) "Type for mole fraction with medium specific attributes";
        type MolarMass = SI.MolarMass(min = 0.001, max = 0.25, nominal = 0.032) "Type for molar mass with medium specific attributes";
        type MolarVolume = SI.MolarVolume(min = 1e-06, max = 1000000.0, nominal = 1.0) "Type for molar volume with medium specific attributes";
        type IsentropicExponent = SI.RatioOfSpecificHeatCapacities(min = 1, max = 500000, nominal = 1.2, start = 1.2) "Type for isentropic exponent with medium specific attributes";
        type SpecificEnergy = SI.SpecificEnergy(min =  -100000000.0, max = 100000000.0, nominal = 1000000.0) "Type for specific energy with medium specific attributes";
        type SpecificInternalEnergy = SpecificEnergy "Type for specific internal energy with medium specific attributes";
        type SpecificEnthalpy = SI.SpecificEnthalpy(min =  -100000000.0, max = 100000000.0, nominal = 1000000.0) "Type for specific enthalpy with medium specific attributes";
        type SpecificEntropy = SI.SpecificEntropy(min =  -1000000.0, max = 1000000.0, nominal = 1000.0) "Type for specific entropy with medium specific attributes";
        type SpecificHeatCapacity = SI.SpecificHeatCapacity(min = 0, max = 1000000.0, nominal = 1000.0, start = 1000.0) "Type for specific heat capacity with medium specific attributes";
        type SurfaceTension = SI.SurfaceTension "Type for surface tension with medium specific attributes";
        type Temperature = SI.Temperature(min = 1, max = 10000.0, nominal = 300, start = 300) "Type for temperature with medium specific attributes";
        type ThermalConductivity = SI.ThermalConductivity(min = 0, max = 500, nominal = 1, start = 1) "Type for thermal conductivity with medium specific attributes";
        type PrandtlNumber = SI.PrandtlNumber(min = 0.001, max = 100000.0, nominal = 1.0) "Type for Prandtl number with medium specific attributes";
        type VelocityOfSound = SI.Velocity(min = 0, max = 100000.0, nominal = 1000, start = 1000) "Type for velocity of sound with medium specific attributes";
        type ExtraProperty = Real(min = 0.0, start = 1.0) "Type for unspecified, mass-specific property transported by flow";
        type CumulativeExtraProperty = Real(min = 0.0, start = 1.0) "Type for conserved integral of unspecified, mass specific property";
        type ExtraPropertyFlowRate = Real(unit = "kg/s") "Type for flow rate of unspecified, mass-specific property";
        type IsobaricExpansionCoefficient = Real(min = 0, max = 100000000.0, unit = "1/K") "Type for isobaric expansion coefficient with medium specific attributes";
        type DipoleMoment = Real(min = 0.0, max = 2.0, unit = "debye", quantity = "ElectricDipoleMoment") "Type for dipole moment with medium specific attributes";
        type DerDensityByPressure = SI.DerDensityByPressure "Type for partial derivative of density with resect to pressure with medium specific attributes";
        type DerDensityByEnthalpy = SI.DerDensityByEnthalpy "Type for partial derivative of density with resect to enthalpy with medium specific attributes";
        type DerEnthalpyByPressure = SI.DerEnthalpyByPressure "Type for partial derivative of enthalpy with resect to pressure with medium specific attributes";
        type DerDensityByTemperature = SI.DerDensityByTemperature "Type for partial derivative of density with resect to temperature with medium specific attributes";
        package Choices "Types, constants to define menu choices"
          type IndependentVariables = enumeration(T "Temperature", pT "Pressure, Temperature", ph "Pressure, Specific Enthalpy", phX "Pressure, Specific Enthalpy, Mass Fraction", pTX  "Pressure, Temperature, Mass Fractions", dTX  "Density, Temperature, Mass Fractions") "Enumeration defining the independent variables of a medium";
          annotation(Documentation(info = "<html>
<h2>Enumerations and data types for all types of fluids</h2>
<p>Note: Reference enthalpy might have to be extended with enthalpy of formation. </p>
</html>"));
          type Init = enumeration(NoInit "NoInit (no initialization)", InitialStates "InitialStates (initialize medium states)", SteadyState  "SteadyState (initialize in steady state)", SteadyMass  "SteadyMass (initialize density or pressure in steady state)") "Enumeration defining initialization for fluid flow" annotation(Evaluate = true);
          type ReferenceEnthalpy = enumeration(ZeroAt0K "The enthalpy is 0 at 0 K (default), if the enthalpy of formation is excluded", ZeroAt25C  "The enthalpy is 0 at 25 degC, if the enthalpy of formation is excluded", UserDefined  "The user-defined reference enthalpy is used at 293.15 K (25 degC)") "Enumeration defining the reference enthalpy of a medium" annotation(Evaluate = true);
          type ReferenceEntropy = enumeration(ZeroAt0K "The entropy is 0 at 0 K (default)", ZeroAt0C  "The entropy is 0 at 0 degC", UserDefined  "The user-defined reference entropy is used at 293.15 K (25 degC)") "Enumeration defining the reference entropy of a medium" annotation(Evaluate = true);
          type pd = enumeration(default "Default (no boundary condition for p or d)", p_known  "p_known (pressure p is known)", d_known  "d_known (density d is known)") "Enumeration defining whether p or d are known for the boundary condition" annotation(Evaluate = true);
          type Th = enumeration(default "Default (no boundary condition for T or h)", T_known  "T_known (temperature T is known)", h_known  "h_known (specific enthalpy h is known)") "Enumeration defining whether T or h are known as boundary condition" annotation(Evaluate = true);
        end Choices;
        annotation(Documentation(info = "<html>
<p>
<b>PartialMedium</b> is a package and contains all <b>declarations</b> for
a medium. This means that constants, models, and functions
are defined that every medium is supposed to support
(some of them are optional). A medium package
inherits from <b>PartialMedium</b> and provides the
equations for the medium. The details of this package
are described in
<a href=\"Modelica://Modelica.Media.UsersGuide\">Modelica.Media.UsersGuide</a>.
</p>
</html>
", revisions = "<html>

</html>"));
      end PartialMedium;
      partial package PartialPureSubstance "base class for pure substances of one chemical substance"
        extends PartialMedium(final reducedX = true, final fixedX = true);
        replaceable function setState_pT "Return thermodynamic state from p and T"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          output ThermodynamicState state "thermodynamic state record";
        algorithm 
          state:=setState_pTX(p, T, fill(0, 0));
        end setState_pT;
        replaceable function setState_ph "Return thermodynamic state from p and h"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          output ThermodynamicState state "thermodynamic state record";
        algorithm 
          state:=setState_phX(p, h, fill(0, 0));
        end setState_ph;
        replaceable function setState_ps "Return thermodynamic state from p and s"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          output ThermodynamicState state "thermodynamic state record";
        algorithm 
          state:=setState_psX(p, s, fill(0, 0));
        end setState_ps;
        replaceable function setState_dT "Return thermodynamic state from d and T"
          extends Modelica.Icons.Function;
          input Density d "density";
          input Temperature T "Temperature";
          output ThermodynamicState state "thermodynamic state record";
        algorithm 
          state:=setState_dTX(d, T, fill(0, 0));
        end setState_dT;
        replaceable function density_ph "Return density from p and h"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          output Density d "Density";
        algorithm 
          d:=density_phX(p, h, fill(0, 0));
        end density_ph;
        replaceable function temperature_ph "Return temperature from p and h"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          output Temperature T "Temperature";
        algorithm 
          T:=temperature_phX(p, h, fill(0, 0));
        end temperature_ph;
        replaceable function pressure_dT "Return pressure from d and T"
          extends Modelica.Icons.Function;
          input Density d "Density";
          input Temperature T "Temperature";
          output AbsolutePressure p "Pressure";
        algorithm 
          p:=pressure(setState_dTX(d, T, fill(0, 0)));
        end pressure_dT;
        replaceable function specificEnthalpy_dT "Return specific enthalpy from d and T"
          extends Modelica.Icons.Function;
          input Density d "Density";
          input Temperature T "Temperature";
          output SpecificEnthalpy h "specific enthalpy";
        algorithm 
          h:=specificEnthalpy(setState_dTX(d, T, fill(0, 0)));
        end specificEnthalpy_dT;
        replaceable function specificEnthalpy_ps "Return specific enthalpy from p and s"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          output SpecificEnthalpy h "specific enthalpy";
        algorithm 
          h:=specificEnthalpy_psX(p, s, fill(0, 0));
        end specificEnthalpy_ps;
        replaceable function temperature_ps "Return temperature from p and s"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          output Temperature T "Temperature";
        algorithm 
          T:=temperature_psX(p, s, fill(0, 0));
        end temperature_ps;
        replaceable function density_ps "Return density from p and s"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          output Density d "Density";
        algorithm 
          d:=density_psX(p, s, fill(0, 0));
        end density_ps;
        replaceable function specificEnthalpy_pT "Return specific enthalpy from p and T"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          output SpecificEnthalpy h "specific enthalpy";
        algorithm 
          h:=specificEnthalpy_pTX(p, T, fill(0, 0));
        end specificEnthalpy_pT;
        replaceable function density_pT "Return density from p and T"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          output Density d "Density";
        algorithm 
          d:=density(setState_pTX(p, T, fill(0, 0)));
        end density_pT;
        redeclare replaceable partial model extends BaseProperties(final standardOrderComponents = true)
        end BaseProperties;
      end PartialPureSubstance;
      partial package PartialLinearFluid "Generic pure liquid model with constant cp, compressibility and thermal expansion coefficients"
        import Modelica.Media.*;
        import SI = Modelica.SIunits;
        extends Interfaces.PartialPureSubstance(ThermoStates = Choices.IndependentVariables.pTX, singleState = false);
        constant SpecificHeatCapacity cp_const "Specific heat capacity at constant pressure";
        constant IsobaricExpansionCoefficient beta_const "Thermal expansion coefficient at constant pressure";
        constant SI.IsothermalCompressibility kappa_const "Isothermal compressibility";
        constant MolarMass MM_const "Molar mass";
        constant Density reference_d "Density in reference conditions";
        constant SpecificEnthalpy reference_h "Specific enthalpy in reference conditions";
        constant SpecificEntropy reference_s "Specific enthalpy in reference conditions";
        constant Boolean constantJacobian "if true, entries in thermodynamic Jacobian are constant, taken at reference conditions";
        record ThermodynamicState "a selection of variables that uniquely defines the thermodynamic state"
          AbsolutePressure p "Absolute pressure of medium";
          Temperature T "Temperature of medium";
        end ThermodynamicState;
        model extends BaseProperties(T(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default), p(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default)) "Base properties of medium"

        equation 
          d = (1 + (p - reference_p) * kappa_const - (T - reference_T) * beta_const) * reference_d;
          h = reference_h + (T - reference_T) * cp_const + ((p - reference_p) * (1 - beta_const * reference_T)) / reference_d;
          u = h - p / d;
          p = state.p;
          T = state.T;
          MM = MM_const;
          R = 8.3144 / MM;
        end BaseProperties;
        function extends setState_pTX "set the thermodynamic state record from p and T (X not needed)"
        algorithm 
          state:=ThermodynamicState(p = p, T = T);
        end setState_pTX;
        function extends setState_phX "set the thermodynamic state record from p and h (X not needed)"
        algorithm 
          state:=ThermodynamicState(p = p, T = (h - reference_h - (p - reference_p) * (1 - beta_const * reference_T) / reference_d) / cp_const + reference_T);
        end setState_phX;
        function extends setState_psX "set the thermodynamic state record from p and s (X not needed)"
        algorithm 
          state:=ThermodynamicState(p = p, T = (reference_T * cp_const) / (cp_const - s + reference_s + (p - reference_p) * ( -beta_const / reference_d)));
        end setState_psX;
        function extends setState_dTX "set the thermodynamic state record from d and T (X not needed)"
        algorithm 
          state:=ThermodynamicState(p = (d - reference_d + (state.T - reference_T) * beta_const * reference_d) / (reference_d * kappa_const) + reference_p, T = T);
        end setState_dTX;
        function extends setSmoothState "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
        algorithm 
          state:=ThermodynamicState(p = Media.Common.smoothStep(x, state_a.p, state_b.p, x_small), T = Media.Common.smoothStep(x, state_a.T, state_b.T, x_small));
        end setSmoothState;
        function extends pressure "Return the pressure from the thermodynamic state"
        algorithm 
          p:=state.p;
        end pressure;
        function extends temperature "Return the temperature from the thermodynamic state"
        algorithm 
          T:=state.T;
        end temperature;
        function extends density "Return the density from the thermodynamic state"
        algorithm 
          d:=(1 + (state.p - reference_p) * kappa_const - (state.T - reference_T) * beta_const) * reference_d;
        end density;
        function extends specificEnthalpy "Return the specific enthalpy from the thermodynamic state"
        algorithm 
          h:=reference_h + (state.T - reference_T) * cp_const + ((state.p - reference_p) * (1 - beta_const * reference_T)) / reference_d;
        end specificEnthalpy;
        function extends specificEntropy "Return the specific entropy from the thermodynamic state"
        algorithm 
          s:=reference_s + ((state.T - reference_T) * cp_const) / state.T + (state.p - reference_p) * ( -beta_const / reference_d);
        end specificEntropy;
        function extends specificInternalEnergy "Return the specific internal energy from the thermodynamic state"
        algorithm 
          u:=specificEnthalpy(state) - state.p / reference_d;
        end specificInternalEnergy;
        function extends specificGibbsEnergy "Return specific Gibbs energy from the thermodynamic state"
          extends Modelica.Icons.Function;
        algorithm 
          g:=specificEnthalpy(state) - state.T * specificEntropy(state);
        end specificGibbsEnergy;
        function extends specificHelmholtzEnergy "Return specific Helmholtz energy from the thermodynamic state"
          extends Modelica.Icons.Function;
        algorithm 
          f:=specificInternalEnergy(state) - state.T * specificEntropy(state);
        end specificHelmholtzEnergy;
        function extends velocityOfSound "Return velocity of sound from the thermodynamic state"
          extends Modelica.Icons.Function;
        algorithm 
          a:=sqrt(max(0, 1 / (kappa_const * density(state) - (beta_const * beta_const * state.T) / cp_const)));
        end velocityOfSound;
        function extends isentropicExponent "Return isentropic exponent from the thermodynamic state"
          extends Modelica.Icons.Function;
        algorithm 
          gamma:=(1 / (state.p * kappa_const) * cp_const) / specificHeatCapacityCv(state);
        end isentropicExponent;
        function extends isentropicEnthalpy "Return isentropic enthalpy"
        algorithm 
          h_is:=specificEnthalpy(setState_pTX(p_downstream, reference_T / (1 - (refState.T - reference_T) / refState.T + (refState.p - p_downstream) * ( -beta_const / (reference_d * cp_const))), reference_X));
          annotation(Documentation(info = "<html>
<p>
A minor approximation is used: the reference density is used instead of the real
one, which would require a numeric solution.
</p>
</html>"));
        end isentropicEnthalpy;
        function extends specificHeatCapacityCp "Return specific heat capacity at constant volume"
        algorithm 
          cp:=cp_const;
        end specificHeatCapacityCp;
        function extends specificHeatCapacityCv "Return specific heat capacity at constant volume from the thermodynamic state"
        algorithm 
          cv:=if constantJacobian then cp_const - (reference_T * beta_const * beta_const) / (kappa_const * reference_d) else (state.T * beta_const * beta_const) / (kappa_const * reference_d);
        end specificHeatCapacityCv;
        function extends isothermalCompressibility "Return the iso-thermal compressibility kappa"
        algorithm 
          kappa:=kappa_const;
        end isothermalCompressibility;
        function extends isobaricExpansionCoefficient "Return the iso-baric expansion coefficient"
        algorithm 
          beta:=beta_const;
        end isobaricExpansionCoefficient;
        function extends density_derp_h "Return density derivative wrt pressure at const specific enthalpy"
        algorithm 
          ddph:=if constantJacobian then kappa_const * reference_d + (beta_const * (1 - reference_T * beta_const)) / cp_const else kappa_const * density(state) + (beta_const * (1 - temperature(state) * beta_const)) / cp_const;
        end density_derp_h;
        function extends density_derh_p "Return density derivative wrt specific enthalpy at constant pressure"
        algorithm 
          ddhp:=if constantJacobian then  -(beta_const * reference_d) / cp_const else  -(beta_const * density(state)) / cp_const;
        end density_derh_p;
        function extends density_derp_T "Return density derivative wrt pressure at const temperature"
        algorithm 
          ddpT:=if constantJacobian then kappa_const * reference_d else kappa_const * density(state);
        end density_derp_T;
        function extends density_derT_p "Return density derivative wrt temperature at constant pressure"
        algorithm 
          ddTp:=if constantJacobian then  -beta_const * reference_d else  -beta_const * density(state);
        end density_derT_p;
        function extends density_derX "Returns the partial derivative of density with respect to mass fractions at constant pressure and temperature"
        algorithm 
          dddX:=fill(0, nX);
        end density_derX;
        function extends molarMass "Return molar mass"
        algorithm 
          MM:=MM_const;
        end molarMass;
        function T_ph "Return temperature from pressure and specific enthalpy"
          input SpecificEnthalpy h "Specific enthalpy";
          input AbsolutePressure p "pressure";
          output Temperature T "Temperature";
        algorithm 
          T:=(h - reference_h - (p - reference_p) * (1 - beta_const * reference_T) / reference_d) / cp_const + reference_T;
        end T_ph;
        function T_ps "Return temperature from pressure and specific entropy"
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          output Temperature T "Temperature";
        algorithm 
          T:=(reference_T * cp_const) / (s - reference_s - (p - reference_p) * ( -beta_const / reference_d) - cp_const);
        end T_ps;
        annotation(Documentation(info = "<html>
<h4>Linear Compressibility Fluid Model</h4>
<p>This linear compressibility fluid model is based on the assumptions that:
</p>
<ul>
<li>The specific heat capacity at constant pressure (cp) is constant</li>
<li>The isobaric expansion coefficient (beta) is constant</li>
<li>The isothermal compressibility (kappa) is constant</li>
<li>Pressure and temperature are used as states</li>
<li>The influence of density on specific enthalpy (h), entropy (s), inner energy (u) and heat capacity (cv) at constant volume is neglected.
</ul>
<p>
That means that the density is a linear function in temperature and in pressure.
In order to define the complete model, a number of constant reference values are needed which
are computed at the reference values of the states pressure p and temperature T. The model can
be interprested as a linearization of a full non-linear fluid model (but it is not linear in all
thermodynamic coordinates). Reference values are needed for
</p>
<ol>
<li>the density (reference_d),</li>
<li>the specific enthalpy (reference_h),</li>
<li>the specific entropy (reference_s).</li>
</ol>
<p>
Apart from that, a user needs to define the molar mass, MM_const.
Note that it is possible to define a fluid by computing the reference
values from a full non-linear fluid model by computing the package constants
using the standard functions defined in a fluid package (see example in liquids package).
</p>
<p>
In order to avoid numerical inversion of the temperature in the T_ph and T_ps functions, the density
is always taken to be the reference density in the computation of h, s, u and cv. For liquids (and this
model is intended only for liquids) the relative error of doing so is 1e-3 to 1e-4 at most. The model would
be more \"correct\" based on the other assumptions, if occurences of reference_d in the computations of h,s,u
and cv would be replaced by a call to density(state). That would require a numerical solution for T_ps, while T_ph can be solved symbolicallyfrom a quadratic function. Errors from this approximation are small because liquid density varies little.</p>
<h4>Efficiency considerations</h4>
<p>One of the main reasons to use a simple, linear fluid model is to achieve high performance
in simulations. There are a number of possible compromises and possibilities to improve performance.
Some of them can be influenced by a flag. The following rules where used in this model:</p>
<ul>
<li>All forward evaluations (using the ThermodynamicState record as input) are exactly following
the assumptions above.</li>
<li>If the flag <b>constantJacobian</b> is set to true in the package, all functions that
typically appear in thermodynamic jacobians (specificHeatCapacityCv, density_derp_h, density_derh_p,
density_derp_T, density_derT_p) are evaluated at reference conditions (that means using the reference
density) instead of the density of the current pressure and temperature. This makes it possible to evaluate
the thermodynamic jacobian at compile time.</li>
<li>For inverse functions using other inputs than the states (e.g pressure p and specific enthalpy h),
the inversion is using the reference state whenever that is necessary to achieve a symbolic inversion.</li>
<li>If <b>constantJacobian</b> is set to false, the above list of functions is computed exactly according
to the above list of assumptions</li>
</ul>
<dl> <b>Authors:</b>
<dd>Francesco Casella<br>
    Dipartimento di Elettronica e Informazione<br>
    Politecnico di Milano<br>
    Via Ponzio 34/5<br>
    I-20133 Milano, Italy<br>
    email: <A HREF=\"mailto:casella@elet.polimi.it\">casella@elet.polimi.it</A><br>
    and <br>
    Hubertus Tummescheit<br>
    Modelon AB<br>
    Ideon Science Park<br>
    SE-22730 Lund, Sweden<br>
    email: <A HREF=\"mailto:Hubertus.Tummescheit@Modelon.se\">Hubertus.Tummescheit@Modelon.se</A><br>

</dd>
</dl>
</html>"));
      end PartialLinearFluid;
      partial package PartialMixtureMedium "Base class for pure substances of several chemical substances"
        extends PartialMedium;
        redeclare replaceable record extends ThermodynamicState "thermodynamic state variables"
          AbsolutePressure p "Absolute pressure of medium";
          Temperature T "Temperature of medium";
          MassFraction X[nX] "Mass fractions (= (component mass)/total mass  m_i/m)";
        end ThermodynamicState;
        redeclare replaceable record extends FluidConstants "extended fluid constants"
          Temperature criticalTemperature "critical temperature";
          AbsolutePressure criticalPressure "critical pressure";
          MolarVolume criticalMolarVolume "critical molar Volume";
          Real acentricFactor "Pitzer acentric factor";
          Temperature triplePointTemperature "triple point temperature";
          AbsolutePressure triplePointPressure "triple point pressure";
          Temperature meltingPoint "melting point at 101325 Pa";
          Temperature normalBoilingPoint "normal boiling point (at 101325 Pa)";
          DipoleMoment dipoleMoment "dipole moment of molecule in Debye (1 debye = 3.33564e10-30 C.m)";
          Boolean hasIdealGasHeatCapacity = false "true if ideal gas heat capacity is available";
          Boolean hasCriticalData = false "true if critical data are known";
          Boolean hasDipoleMoment = false "true if a dipole moment known";
          Boolean hasFundamentalEquation = false "true if a fundamental equation";
          Boolean hasLiquidHeatCapacity = false "true if liquid heat capacity is available";
          Boolean hasSolidHeatCapacity = false "true if solid heat capacity is available";
          Boolean hasAccurateViscosityData = false "true if accurate data for a viscosity function is available";
          Boolean hasAccurateConductivityData = false "true if accurate data for thermal conductivity is available";
          Boolean hasVapourPressureCurve = false "true if vapour pressure data, e.g. Antoine coefficents are known";
          Boolean hasAcentricFactor = false "true if Pitzer accentric factor is known";
          SpecificEnthalpy HCRIT0 = 0.0 "Critical specific enthalpy of the fundamental equation";
          SpecificEntropy SCRIT0 = 0.0 "Critical specific entropy of the fundamental equation";
          SpecificEnthalpy deltah = 0.0 "Difference between specific enthalpy model (h_m) and f.eq. (h_f) (h_m - h_f)";
          SpecificEntropy deltas = 0.0 "Difference between specific enthalpy model (s_m) and f.eq. (s_f) (s_m - s_f)";
        end FluidConstants;
        constant FluidConstants[nS] fluidConstants "constant data for the fluid";
        replaceable function gasConstant "Return the gas constant of the mixture (also for liquids)"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "thermodynamic state";
          output SI.SpecificHeatCapacity R "mixture gas constant";
        end gasConstant;
        function moleToMassFractions "Return mass fractions X from mole fractions"
          extends Modelica.Icons.Function;
          input SI.MoleFraction moleFractions[:] "Mole fractions of mixture";
          input MolarMass[:] MMX "molar masses of components";
          output SI.MassFraction X[size(moleFractions, 1)] "Mass fractions of gas mixture";
          annotation(smoothOrder = 5);
        protected 
          MolarMass Mmix = moleFractions * MMX "molar mass of mixture";
        algorithm 
          for i in 1:size(moleFractions, 1) loop
                      X[i]:=(moleFractions[i] * MMX[i]) / Mmix;

          end for;
        end moleToMassFractions;
        function massToMoleFractions "Return mole fractions from mass fractions X"
          extends Modelica.Icons.Function;
          input SI.MassFraction X[:] "Mass fractions of mixture";
          input SI.MolarMass[:] MMX "molar masses of components";
          output SI.MoleFraction moleFractions[size(X, 1)] "Mole fractions of gas mixture";
          annotation(smoothOrder = 5);
        protected 
          Real invMMX[size(X, 1)] "inverses of molar weights";
          SI.MolarMass Mmix "molar mass of mixture";
        algorithm 
          for i in 1:size(X, 1) loop
                      invMMX[i]:=1 / MMX[i];

          end for;
          Mmix:=1 / (X * invMMX);
          for i in 1:size(X, 1) loop
                      moleFractions[i]:=(Mmix * X[i]) / MMX[i];

          end for;
        end massToMoleFractions;
      end PartialMixtureMedium;
      partial package PartialCondensingGases "Base class for mixtures of condensing and non-condensing gases"
        extends PartialMixtureMedium(ThermoStates = Choices.IndependentVariables.pTX);
        replaceable partial function saturationPressure "Return saturation pressure of condensing fluid"
          extends Modelica.Icons.Function;
          input Temperature Tsat "saturation temperature";
          output AbsolutePressure psat "saturation pressure";
        end saturationPressure;
        replaceable partial function enthalpyOfVaporization "Return vaporization enthalpy of condensing fluid"
          extends Modelica.Icons.Function;
          input Temperature T "temperature";
          output SpecificEnthalpy r0 "vaporization enthalpy";
        end enthalpyOfVaporization;
        replaceable partial function enthalpyOfLiquid "Return liquid enthalpy of condensing fluid"
          extends Modelica.Icons.Function;
          input Temperature T "temperature";
          output SpecificEnthalpy h "liquid enthalpy";
        end enthalpyOfLiquid;
        replaceable partial function enthalpyOfGas "Return enthalpy of non-condensing gas mixture"
          extends Modelica.Icons.Function;
          input Temperature T "temperature";
          input MassFraction[:] X "vector of mass fractions";
          output SpecificEnthalpy h "specific enthalpy";
        end enthalpyOfGas;
        replaceable partial function enthalpyOfCondensingGas "Return enthalpy of condensing gas (most often steam)"
          extends Modelica.Icons.Function;
          input Temperature T "temperature";
          output SpecificEnthalpy h "specific enthalpy";
        end enthalpyOfCondensingGas;
        replaceable partial function enthalpyOfNonCondensingGas "Return enthalpy of the non-condensing species"
          extends Modelica.Icons.Function;
          input Temperature T "temperature";
          output SpecificEnthalpy h "specific enthalpy";
        end enthalpyOfNonCondensingGas;
      end PartialCondensingGases;
      partial package PartialTwoPhaseMedium "Base class for two phase medium of one substance"
        extends PartialPureSubstance;
        constant Boolean smoothModel "true if the (derived) model should not generate state events";
        constant Boolean onePhase "true if the (derived) model should never be called with two-phase inputs";
        record FluidLimits "validity limits for fluid model"
          extends Modelica.Icons.Record;
          Temperature TMIN "minimum temperature";
          Temperature TMAX "maximum temperature";
          Density DMIN "minimum density";
          Density DMAX "maximum density";
          AbsolutePressure PMIN "minimum pressure";
          AbsolutePressure PMAX "maximum pressure";
          SpecificEnthalpy HMIN "minimum enthalpy";
          SpecificEnthalpy HMAX "maximum enthalpy";
          SpecificEntropy SMIN "minimum entropy";
          SpecificEntropy SMAX "maximum entropy";
          annotation(Documentation(info = "<html>
          <p>The minimum pressure mostly applies to the liquid state only.
          The minimum density is also arbitrary, but is reasonable for techical
          applications to limit iterations in non-linear systems. The limits in
          enthalpy and entropy are used as safeguards in inverse iterations.</p>
          </html>"));
        end FluidLimits;
        redeclare replaceable record extends FluidConstants "extended fluid constants"
          Temperature criticalTemperature "critical temperature";
          AbsolutePressure criticalPressure "critical pressure";
          MolarVolume criticalMolarVolume "critical molar Volume";
          Real acentricFactor "Pitzer acentric factor";
          Temperature triplePointTemperature "triple point temperature";
          AbsolutePressure triplePointPressure "triple point pressure";
          Temperature meltingPoint "melting point at 101325 Pa";
          Temperature normalBoilingPoint "normal boiling point (at 101325 Pa)";
          DipoleMoment dipoleMoment "dipole moment of molecule in Debye (1 debye = 3.33564e10-30 C.m)";
          Boolean hasIdealGasHeatCapacity = false "true if ideal gas heat capacity is available";
          Boolean hasCriticalData = false "true if critical data are known";
          Boolean hasDipoleMoment = false "true if a dipole moment known";
          Boolean hasFundamentalEquation = false "true if a fundamental equation";
          Boolean hasLiquidHeatCapacity = false "true if liquid heat capacity is available";
          Boolean hasSolidHeatCapacity = false "true if solid heat capacity is available";
          Boolean hasAccurateViscosityData = false "true if accurate data for a viscosity function is available";
          Boolean hasAccurateConductivityData = false "true if accurate data for thermal conductivity is available";
          Boolean hasVapourPressureCurve = false "true if vapour pressure data, e.g. Antoine coefficents are known";
          Boolean hasAcentricFactor = false "true if Pitzer accentric factor is known";
          SpecificEnthalpy HCRIT0 = 0.0 "Critical specific enthalpy of the fundamental equation";
          SpecificEntropy SCRIT0 = 0.0 "Critical specific entropy of the fundamental equation";
          SpecificEnthalpy deltah = 0.0 "Difference between specific enthalpy model (h_m) and f.eq. (h_f) (h_m - h_f)";
          SpecificEntropy deltas = 0.0 "Difference between specific enthalpy model (s_m) and f.eq. (s_f) (s_m - s_f)";
          annotation(Documentation(info = "<html></html>"));
        end FluidConstants;
        constant FluidConstants[nS] fluidConstants "constant data for the fluid";
        redeclare replaceable record extends ThermodynamicState "Thermodynamic state of two phase medium"
          FixedPhase phase(min = 0, max = 2) "phase of the fluid: 1 for 1-phase, 2 for two-phase, 0 for not known, e.g. interactive use";
          annotation(Documentation(info = "<html></html>"));
        end ThermodynamicState;
        replaceable record SaturationProperties "Saturation properties of two phase medium"
          extends Modelica.Icons.Record;
          AbsolutePressure psat "saturation pressure";
          Temperature Tsat "saturation temperature";
          annotation(Documentation(info = "<html></html>"));
        end SaturationProperties;
        type FixedPhase = Integer(min = 0, max = 2) "phase of the fluid: 1 for 1-phase, 2 for two-phase, 0 for not known, e.g. interactive use";
        redeclare replaceable partial model extends BaseProperties "Base properties (p, d, T, h, u, R, MM, sat) of two phase medium"
          SaturationProperties sat "Saturation properties at the medium pressure";
          annotation(Documentation(info = "<html></html>"));
        end BaseProperties;
        replaceable partial function setDewState "Return the thermodynamic state on the dew line"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "saturation point";
          input FixedPhase phase(min = 1, max = 2) = 1 "phase: default is one phase";
          output ThermodynamicState state "complete thermodynamic state info";
          annotation(Documentation(info = "<html></html>"));
        end setDewState;
        replaceable partial function setBubbleState "Return the thermodynamic state on the bubble line"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "saturation point";
          input FixedPhase phase(min = 1, max = 2) = 1 "phase: default is one phase";
          output ThermodynamicState state "complete thermodynamic state info";
          annotation(Documentation(info = "<html></html>"));
        end setBubbleState;
        redeclare replaceable partial function extends setState_dTX "Return thermodynamic state as function of d, T and composition X or Xi"
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          annotation(Documentation(info = "<html></html>"));
        end setState_dTX;
        redeclare replaceable partial function extends setState_phX "Return thermodynamic state as function of p, h and composition X or Xi"
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          annotation(Documentation(info = "<html></html>"));
        end setState_phX;
        redeclare replaceable partial function extends setState_psX "Return thermodynamic state as function of p, s and composition X or Xi"
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          annotation(Documentation(info = "<html></html>"));
        end setState_psX;
        redeclare replaceable partial function extends setState_pTX "Return thermodynamic state as function of p, T and composition X or Xi"
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          annotation(Documentation(info = "<html></html>"));
        end setState_pTX;
        replaceable function setSat_T "Return saturation property record from temperature"
          extends Modelica.Icons.Function;
          input Temperature T "temperature";
          output SaturationProperties sat "saturation property record";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          sat.Tsat:=T;
          sat.psat:=saturationPressure(T);
        end setSat_T;
        replaceable function setSat_p "Return saturation property record from pressure"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "pressure";
          output SaturationProperties sat "saturation property record";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          sat.psat:=p;
          sat.Tsat:=saturationTemperature(p);
        end setSat_p;
        replaceable partial function bubbleEnthalpy "Return bubble point specific enthalpy"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "saturation property record";
          output SI.SpecificEnthalpy hl "boiling curve specific enthalpy";
          annotation(Documentation(info = "<html></html>"));
        end bubbleEnthalpy;
        replaceable partial function dewEnthalpy "Return dew point specific enthalpy"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "saturation property record";
          output SI.SpecificEnthalpy hv "dew curve specific enthalpy";
          annotation(Documentation(info = "<html></html>"));
        end dewEnthalpy;
        replaceable partial function bubbleEntropy "Return bubble point specific entropy"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "saturation property record";
          output SI.SpecificEntropy sl "boiling curve specific entropy";
          annotation(Documentation(info = "<html></html>"));
        end bubbleEntropy;
        replaceable partial function dewEntropy "Return dew point specific entropy"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "saturation property record";
          output SI.SpecificEntropy sv "dew curve specific entropy";
          annotation(Documentation(info = "<html></html>"));
        end dewEntropy;
        replaceable partial function bubbleDensity "Return bubble point density"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "saturation property record";
          output Density dl "boiling curve density";
          annotation(Documentation(info = "<html></html>"));
        end bubbleDensity;
        replaceable partial function dewDensity "Return dew point density"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "saturation property record";
          output Density dv "dew curve density";
          annotation(Documentation(info = "<html></html>"));
        end dewDensity;
        replaceable partial function saturationPressure "Return saturation pressure"
          extends Modelica.Icons.Function;
          input Temperature T "temperature";
          output AbsolutePressure p "saturation pressure";
          annotation(Documentation(info = "<html></html>"));
        end saturationPressure;
        replaceable partial function saturationTemperature "Return saturation temperature"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "pressure";
          output Temperature T "saturation temperature";
          annotation(Documentation(info = "<html></html>"));
        end saturationTemperature;
        replaceable function saturationPressure_sat "Return saturation temperature"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "saturation property record";
          output AbsolutePressure p "saturation pressure";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          p:=sat.psat;
        end saturationPressure_sat;
        replaceable function saturationTemperature_sat "Return saturation temperature"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "saturation property record";
          output Temperature T "saturation temperature";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          T:=sat.Tsat;
        end saturationTemperature_sat;
        replaceable partial function saturationTemperature_derp "Return derivative of saturation temperature w.r.t. pressure"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "pressure";
          output Real dTp "derivative of saturation temperature w.r.t. pressure";
          annotation(Documentation(info = "<html></html>"));
        end saturationTemperature_derp;
        replaceable function saturationTemperature_derp_sat "Return derivative of saturation temperature w.r.t. pressure"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "saturation property record";
          output Real dTp "derivative of saturation temperature w.r.t. pressure";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          dTp:=saturationTemperature_derp(sat.psat);
        end saturationTemperature_derp_sat;
        replaceable partial function surfaceTension "Return surface tension sigma in the two phase region"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "saturation property record";
          output SurfaceTension sigma "Surface tension sigma in the two phase region";
          annotation(Documentation(info = "<html></html>"));
        end surfaceTension;
        redeclare replaceable partial function extends molarMass "Return the molar mass of the medium"
        algorithm 
          MM:=fluidConstants[1].molarMass;
        end molarMass;
        replaceable partial function dBubbleDensity_dPressure "Return bubble point density derivative"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "saturation property record";
          output DerDensityByPressure ddldp "boiling curve density derivative";
          annotation(Documentation(info = "<html></html>"));
        end dBubbleDensity_dPressure;
        replaceable partial function dDewDensity_dPressure "Return dew point density derivative"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "saturation property record";
          output DerDensityByPressure ddvdp "saturated steam density derivative";
          annotation(Documentation(info = "<html></html>"));
        end dDewDensity_dPressure;
        replaceable partial function dBubbleEnthalpy_dPressure "Return bubble point specific enthalpy derivative"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "saturation property record";
          output DerEnthalpyByPressure dhldp "boiling curve specific enthalpy derivative";
          annotation(Documentation(info = "<html></html>"));
        end dBubbleEnthalpy_dPressure;
        replaceable partial function dDewEnthalpy_dPressure "Return dew point specific enthalpy derivative"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "saturation property record";
          output DerEnthalpyByPressure dhvdp "saturated steam specific enthalpy derivative";
          annotation(Documentation(info = "<html></html>"));
        end dDewEnthalpy_dPressure;
        redeclare replaceable function specificEnthalpy_pTX "Return specific enthalpy from pressure, temperature and mass fraction"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input MassFraction X[nX] "Mass fractions";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output SpecificEnthalpy h "Specific enthalpy at p, T, X";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          h:=specificEnthalpy(setState_pTX(p, T, X, phase));
        end specificEnthalpy_pTX;
        redeclare replaceable function temperature_phX "Return temperature from p, h, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[nX] "Mass fractions";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Temperature T "Temperature";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          T:=temperature(setState_phX(p, h, X, phase));
        end temperature_phX;
        redeclare replaceable function density_phX "Return density from p, h, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[nX] "Mass fractions";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Density d "density";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          d:=density(setState_phX(p, h, X, phase));
        end density_phX;
        redeclare replaceable function temperature_psX "Return temperature from p, s, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input MassFraction X[nX] "Mass fractions";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Temperature T "Temperature";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          T:=temperature(setState_psX(p, s, X, phase));
        end temperature_psX;
        redeclare replaceable function density_psX "Return density from p, s, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input MassFraction X[nX] "Mass fractions";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Density d "Density";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          d:=density(setState_psX(p, s, X, phase));
        end density_psX;
        redeclare replaceable function specificEnthalpy_psX "Return specific enthalpy from p, s, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input MassFraction X[nX] "Mass fractions";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output SpecificEnthalpy h "specific enthalpy";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          h:=specificEnthalpy(setState_psX(p, s, X, phase));
        end specificEnthalpy_psX;
        redeclare replaceable function setState_pT "Return thermodynamic state from p and T"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output ThermodynamicState state "thermodynamic state record";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          state:=setState_pTX(p, T, fill(0, 0), phase);
        end setState_pT;
        redeclare replaceable function setState_ph "Return thermodynamic state from p and h"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output ThermodynamicState state "thermodynamic state record";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          state:=setState_phX(p, h, fill(0, 0), phase);
        end setState_ph;
        redeclare replaceable function setState_ps "Return thermodynamic state from p and s"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output ThermodynamicState state "thermodynamic state record";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          state:=setState_psX(p, s, fill(0, 0), phase);
        end setState_ps;
        redeclare replaceable function setState_dT "Return thermodynamic state from d and T"
          extends Modelica.Icons.Function;
          input Density d "density";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output ThermodynamicState state "thermodynamic state record";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          state:=setState_dTX(d, T, fill(0, 0), phase);
        end setState_dT;
        replaceable function setState_px "Return thermodynamic state from pressure and vapour quality"
          input AbsolutePressure p "Pressure";
          input MassFraction x "Vapour quality";
          output ThermodynamicState state "Thermodynamic state record";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          state:=setState_ph(p, (1 - x) * bubbleEnthalpy(setSat_p(p)) + x * dewEnthalpy(setSat_p(p)), 2);
        end setState_px;
        replaceable function setState_Tx "Return thermodynamic state from temperature and vapour quality"
          input Temperature T "Temperature";
          input MassFraction x "Vapour quality";
          output ThermodynamicState state "thermodynamic state record";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          state:=setState_ph(saturationPressure_sat(setSat_T(T)), (1 - x) * bubbleEnthalpy(setSat_T(T)) + x * dewEnthalpy(setSat_T(T)), 2);
        end setState_Tx;
        replaceable function vapourQuality "Return vapour quality"
          input ThermodynamicState state "Thermodynamic state record";
          output MassFraction x "Vapour quality";
        protected 
          constant SpecificEnthalpy eps = 1e-08;
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          x:=min(max((specificEnthalpy(state) - bubbleEnthalpy(setSat_p(pressure(state)))) / (dewEnthalpy(setSat_p(pressure(state))) - bubbleEnthalpy(setSat_p(pressure(state))) + eps), 0), 1);
        end vapourQuality;
        redeclare replaceable function density_ph "Return density from p and h"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Density d "Density";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          d:=density_phX(p, h, fill(0, 0), phase);
        end density_ph;
        redeclare replaceable function temperature_ph "Return temperature from p and h"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Temperature T "Temperature";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          T:=temperature_phX(p, h, fill(0, 0), phase);
        end temperature_ph;
        redeclare replaceable function pressure_dT "Return pressure from d and T"
          extends Modelica.Icons.Function;
          input Density d "Density";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output AbsolutePressure p "Pressure";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          p:=pressure(setState_dTX(d, T, fill(0, 0), phase));
        end pressure_dT;
        redeclare replaceable function specificEnthalpy_dT "Return specific enthalpy from d and T"
          extends Modelica.Icons.Function;
          input Density d "Density";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output SpecificEnthalpy h "specific enthalpy";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          h:=specificEnthalpy(setState_dTX(d, T, fill(0, 0), phase));
        end specificEnthalpy_dT;
        redeclare replaceable function specificEnthalpy_ps "Return specific enthalpy from p and s"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output SpecificEnthalpy h "specific enthalpy";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          h:=specificEnthalpy_psX(p, s, fill(0, 0));
        end specificEnthalpy_ps;
        redeclare replaceable function temperature_ps "Return temperature from p and s"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Temperature T "Temperature";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          T:=temperature_psX(p, s, fill(0, 0), phase);
        end temperature_ps;
        redeclare replaceable function density_ps "Return density from p and s"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Density d "Density";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          d:=density_psX(p, s, fill(0, 0), phase);
        end density_ps;
        redeclare replaceable function specificEnthalpy_pT "Return specific enthalpy from p and T"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output SpecificEnthalpy h "specific enthalpy";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          h:=specificEnthalpy_pTX(p, T, fill(0, 0), phase);
        end specificEnthalpy_pT;
        redeclare replaceable function density_pT "Return density from p and T"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Density d "Density";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          d:=density(setState_pTX(p, T, fill(0, 0), phase));
        end density_pT;
      end PartialTwoPhaseMedium;
      partial package PartialSimpleMedium "Medium model with linear dependency of u, h from temperature. All other quantities, especially density, are constant."
        extends Interfaces.PartialPureSubstance(final ThermoStates = Choices.IndependentVariables.pT, final singleState = true);
        import SI = Modelica.SIunits;
        constant SpecificHeatCapacity cp_const "Constant specific heat capacity at constant pressure";
        constant SpecificHeatCapacity cv_const "Constant specific heat capacity at constant volume";
        constant Density d_const "Constant density";
        constant DynamicViscosity eta_const "Constant dynamic viscosity";
        constant ThermalConductivity lambda_const "Constant thermal conductivity";
        constant VelocityOfSound a_const "Constant velocity of sound";
        constant Temperature T_min "Minimum temperature valid for medium model";
        constant Temperature T_max "Maximum temperature valid for medium model";
        constant Temperature T0 = reference_T "Zero enthalpy temperature";
        constant MolarMass MM_const "Molar mass";
        constant FluidConstants[nS] fluidConstants "fluid constants";
        record extends ThermodynamicState "Thermodynamic state"
          AbsolutePressure p "Absolute pressure of medium";
          Temperature T "Temperature of medium";
        end ThermodynamicState;
        redeclare replaceable model extends BaseProperties(T(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default), p(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default)) "Base properties"

        equation 
          assert(T >= T_min and T <= T_max, "
Temperature T (= " + String(T) + " K) is not
in the allowed range (" + String(T_min) + " K <= T <= " + String(T_max) + " K)
required from medium model \"" + mediumName + "\".
");
          h = specificEnthalpy_pTX(p, T, X);
          u = cv_const * (T - T0);
          d = d_const;
          R = 0;
          MM = MM_const;
          state.T = T;
          state.p = p;
          annotation(Documentation(info = "<HTML>
<p>
This is the most simple incompressible medium model, where
specific enthalpy h and specific internal energy u are only
a function of temperature T and all other provided medium
quantities are assumed to be constant.
Note that the (small) influence of the pressure term p/d is neglected.
</p>

</p>
</HTML>"));
        end BaseProperties;
        function setState_pTX "Return thermodynamic state from p, T, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "thermodynamic state record";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          state:=ThermodynamicState(p = p, T = T);
        end setState_pTX;
        function setState_phX "Return thermodynamic state from p, h, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "thermodynamic state record";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          state:=ThermodynamicState(p = p, T = T0 + h / cp_const);
        end setState_phX;
        redeclare replaceable function setState_psX "Return thermodynamic state from p, s, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "thermodynamic state record";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          state:=ThermodynamicState(p = p, T = Modelica.Math.exp(s / cp_const + Modelica.Math.log(reference_T))) "here the incompressible limit is used, with cp as heat capacity";
        end setState_psX;
        function setState_dTX "Return thermodynamic state from d, T, and X or Xi"
          extends Modelica.Icons.Function;
          input Density d "density";
          input Temperature T "Temperature";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "thermodynamic state record";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          assert(false, "pressure can not be computed from temperature and density for an incompressible fluid!");
        end setState_dTX;
        function extends setSmoothState "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
        algorithm 
          state:=ThermodynamicState(p = Media.Common.smoothStep(x, state_a.p, state_b.p, x_small), T = Media.Common.smoothStep(x, state_a.T, state_b.T, x_small));
        end setSmoothState;
        function extends dynamicViscosity "Return dynamic viscosity"
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          eta:=eta_const;
        end dynamicViscosity;
        function extends thermalConductivity "Return thermal conductivity"
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          lambda:=lambda_const;
        end thermalConductivity;
        function extends pressure "Return pressure"
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          p:=state.p;
        end pressure;
        function extends temperature "Return temperature"
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          T:=state.T;
        end temperature;
        function extends density "Return density"
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          d:=d_const;
        end density;
        function extends specificEnthalpy "Return specific enthalpy"
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          h:=cp_const * (state.T - T0);
        end specificEnthalpy;
        function extends specificHeatCapacityCp "Return specific heat capacity at constant pressure"
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          cp:=cp_const;
        end specificHeatCapacityCp;
        function extends specificHeatCapacityCv "Return specific heat capacity at constant volume"
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          cv:=cv_const;
        end specificHeatCapacityCv;
        function extends isentropicExponent "Return isentropic exponent"
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          gamma:=cp_const / cv_const;
        end isentropicExponent;
        function extends velocityOfSound "Return velocity of sound "
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          a:=a_const;
        end velocityOfSound;
        function specificEnthalpy_pTX "Return specific enthalpy from p, T, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input MassFraction X[nX] "Mass fractions";
          output SpecificEnthalpy h "Specific enthalpy";
          annotation(Documentation(info = "<html>
<p>
This function computes the specific enthalpy of the fluid, but neglects the (small) influence of the pressure term p/d.
</p>
</html>"));
        algorithm 
          h:=cp_const * (T - T0);
        end specificEnthalpy_pTX;
        function temperature_phX "Return temperature from p, h, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[nX] "Mass fractions";
          output Temperature T "Temperature";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          T:=T0 + h / cp_const;
        end temperature_phX;
        function density_phX "Return density from p, h, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[nX] "Mass fractions";
          output Density d "density";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          d:=density(setState_phX(p, h, X));
        end density_phX;
        function extends specificInternalEnergy "Return specific internal energy"
          extends Modelica.Icons.Function;
        algorithm 
          u:=cv_const * (state.T - T0);
          annotation(Documentation(info = "<html>
<p>
This function computes the specific internal energy of the fluid, but neglects the (small) influence of the pressure term p/d.
</p>
</html>"));
        end specificInternalEnergy;
        function extends specificEntropy "Return specific entropy"
          extends Modelica.Icons.Function;
        algorithm 
          s:=cv_const * Modelica.Math.log(state.T / T0);
        end specificEntropy;
        function extends specificGibbsEnergy "Return specific Gibbs energy"
          extends Modelica.Icons.Function;
        algorithm 
          g:=specificEnthalpy(state) - state.T * specificEntropy(state);
        end specificGibbsEnergy;
        function extends specificHelmholtzEnergy "Return specific Helmholtz energy"
          extends Modelica.Icons.Function;
        algorithm 
          f:=specificInternalEnergy(state) - state.T * specificEntropy(state);
        end specificHelmholtzEnergy;
        function extends isentropicEnthalpy "Return isentropic enthalpy"
        algorithm 
          h_is:=cp_const * (temperature(refState) - T0);
        end isentropicEnthalpy;
        function extends isobaricExpansionCoefficient "Returns overall the isobaric expansion coefficient beta"
        algorithm 
          beta:=0.0;
        end isobaricExpansionCoefficient;
        function extends isothermalCompressibility "Returns overall the isothermal compressibility factor"
        algorithm 
          kappa:=0;
        end isothermalCompressibility;
        function extends density_derp_T "Returns the partial derivative of density with respect to pressure at constant temperature"
        algorithm 
          ddpT:=0;
        end density_derp_T;
        function extends density_derT_p "Returns the partial derivative of density with respect to temperature at constant pressure"
        algorithm 
          ddTp:=0;
        end density_derT_p;
        function extends density_derX "Returns the partial derivative of density with respect to mass fractions at constant pressure and temperature"
        algorithm 
          dddX:=fill(0, nX);
        end density_derX;
        function extends molarMass "return the molar mass of the medium"
        algorithm 
          MM:=MM_const;
        end molarMass;
      end PartialSimpleMedium;
      partial package PartialSimpleIdealGasMedium "Medium model of Ideal gas with constant cp and cv. All other quantities, e.g. transport properties, are constant."
        extends Interfaces.PartialPureSubstance(ThermoStates = Choices.IndependentVariables.pT, final singleState = false);
        import SI = Modelica.SIunits;
        constant SpecificHeatCapacity cp_const "Constant specific heat capacity at constant pressure";
        constant SpecificHeatCapacity cv_const = cp_const - R_gas "Constant specific heat capacity at constant volume";
        constant SpecificHeatCapacity R_gas "medium specific gas constant";
        constant MolarMass MM_const "Molar mass";
        constant DynamicViscosity eta_const "Constant dynamic viscosity";
        constant ThermalConductivity lambda_const "Constant thermal conductivity";
        constant Temperature T_min "Minimum temperature valid for medium model";
        constant Temperature T_max "Maximum temperature valid for medium model";
        constant Temperature T0 = reference_T "Zero enthalpy temperature";
        record extends ThermodynamicState "Thermodynamic state of ideal gas"
          AbsolutePressure p "Absolute pressure of medium";
          Temperature T "Temperature of medium";
        end ThermodynamicState;
        record extends FluidConstants "fluid constants"
        end FluidConstants;
        redeclare replaceable model extends BaseProperties(T(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default), p(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default)) "Base properties of ideal gas"

        equation 
          assert(T >= T_min and T <= T_max, "
Temperature T (= " + String(T) + " K) is not
in the allowed range (" + String(T_min) + " K <= T <= " + String(T_max) + " K)
required from medium model \"" + mediumName + "\".
");
          h = specificEnthalpy_pTX(p, T, X);
          u = h - R * T;
          R = R_gas;
          d = p / (R * T);
          MM = MM_const;
          state.T = T;
          state.p = p;
          annotation(Documentation(info = "<HTML>
<p>
This is the most simple incompressible medium model, where
specific enthalpy h and specific internal energy u are only
a function of temperature T and all other provided medium
quantities are assumed to be constant.
</p>
</HTML>"));
        end BaseProperties;
        function setState_pTX "Return thermodynamic state from p, T, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "thermodynamic state record";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          state:=ThermodynamicState(p = p, T = T);
        end setState_pTX;
        function setState_phX "Return thermodynamic state from p, h, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "thermodynamic state record";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          state:=ThermodynamicState(p = p, T = T0 + h / cp_const);
        end setState_phX;
        redeclare replaceable function setState_psX "Return thermodynamic state from p, s, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "thermodynamic state record";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          state:=ThermodynamicState(p = p, T = Modelica.Math.exp(s / cp_const + Modelica.Math.log(reference_T)) + R_gas * Modelica.Math.log(p / reference_p));
        end setState_psX;
        function setState_dTX "Return thermodynamic state from d, T, and X or Xi"
          extends Modelica.Icons.Function;
          input Density d "density";
          input Temperature T "Temperature";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "thermodynamic state record";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          state:=ThermodynamicState(p = d * R_gas * T, T = T);
        end setState_dTX;
        function extends setSmoothState "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
        algorithm 
          state:=ThermodynamicState(p = Media.Common.smoothStep(x, state_a.p, state_b.p, x_small), T = Media.Common.smoothStep(x, state_a.T, state_b.T, x_small));
        end setSmoothState;
        function extends pressure "Return pressure of ideal gas"
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          p:=state.p;
        end pressure;
        function extends temperature "Return temperature of ideal gas"
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          T:=state.T;
        end temperature;
        function extends density "Return density of ideal gas"
        algorithm 
          d:=state.p / (R_gas * state.T);
        end density;
        function extends specificEnthalpy "Return specific enthalpy"
          extends Modelica.Icons.Function;
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          h:=cp_const * (state.T - T0);
        end specificEnthalpy;
        function extends specificInternalEnergy "Return specific internal energy"
          extends Modelica.Icons.Function;
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          u:=cp_const * (state.T - T0) - R_gas * state.T;
        end specificInternalEnergy;
        function extends specificEntropy "Return specific entropy"
          extends Modelica.Icons.Function;
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          s:=cp_const * Modelica.Math.log(state.T / T0) - R_gas * Modelica.Math.log(state.p / reference_p);
        end specificEntropy;
        function extends specificGibbsEnergy "Return specific Gibbs energy"
          extends Modelica.Icons.Function;
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          g:=cp_const * (state.T - T0) - state.T * specificEntropy(state);
        end specificGibbsEnergy;
        function extends specificHelmholtzEnergy "Return specific Helmholtz energy"
          extends Modelica.Icons.Function;
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          f:=(cp_const - R_gas) * (state.T - T0) - state.T * specificEntropy(state);
        end specificHelmholtzEnergy;
        function extends dynamicViscosity "Return dynamic viscosity"
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          eta:=eta_const;
        end dynamicViscosity;
        function extends thermalConductivity "Return thermal conductivity"
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          lambda:=lambda_const;
        end thermalConductivity;
        function extends specificHeatCapacityCp "Return specific heat capacity at constant pressure"
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          cp:=cp_const;
        end specificHeatCapacityCp;
        function extends specificHeatCapacityCv "Return specific heat capacity at constant volume"
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          cv:=cv_const;
        end specificHeatCapacityCv;
        function extends isentropicExponent "Return isentropic exponent"
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          gamma:=cp_const / cv_const;
        end isentropicExponent;
        function extends velocityOfSound "Return velocity of sound "
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          a:=sqrt(cp_const / cv_const * R_gas * state.T);
        end velocityOfSound;
        function specificEnthalpy_pTX "Return specific enthalpy from p, T, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input MassFraction X[nX] "Mass fractions";
          output SpecificEnthalpy h "Specific enthalpy at p, T, X";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          h:=cp_const * (T - T0);
        end specificEnthalpy_pTX;
        function temperature_phX "Return temperature from p, h, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[nX] "Mass fractions";
          output Temperature T "Temperature";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          T:=h / cp_const + T0;
        end temperature_phX;
        function density_phX "Return density from p, h, and X or Xi"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[nX] "Mass fractions";
          output Density d "density";
          annotation(Documentation(info = "<html></html>"));
        algorithm 
          d:=density(setState_phX(p, h, X));
        end density_phX;
        function extends isentropicEnthalpy "Return isentropic enthalpy"
        algorithm 
          h_is:=cp_const * (refState.T * (p_downstream / refState.p) ^ (R_gas / cp_const) - T0);
        end isentropicEnthalpy;
        function extends isobaricExpansionCoefficient "Returns overall the isobaric expansion coefficient beta"
        algorithm 
          beta:=1 / state.T;
        end isobaricExpansionCoefficient;
        function extends isothermalCompressibility "Returns overall the isothermal compressibility factor"
        algorithm 
          kappa:=1 / state.p;
        end isothermalCompressibility;
        function extends density_derp_T "Returns the partial derivative of density with respect to pressure at constant temperature"
        algorithm 
          ddpT:=1 / (R_gas * state.T);
        end density_derp_T;
        function extends density_derT_p "Returns the partial derivative of density with respect to temperature at constant pressure"
        algorithm 
          ddTp:= -state.p / (R_gas * state.T * state.T);
        end density_derT_p;
        function extends density_derX "Returns the partial derivative of density with respect to mass fractions at constant pressure and temperature"
        algorithm 
          dddX:=fill(0, nX);
        end density_derX;
        function extends molarMass "Returns the molar mass of the medium"
        algorithm 
          MM:=MM_const;
        end molarMass;
      end PartialSimpleIdealGasMedium;
    end Interfaces;
    package Common "data structures and fundamental functions for fluid properties"
      annotation(Documentation(info = "<HTML><h4>Package description</h4>
      <p>Package Modelica.Media.Common provides records and functions shared by many of the property sub-packages.
      High accuracy fluid property models share a lot of common structure, even if the actual models are different.
      Common data structures and computations shared by these property models are collected in this library.
      </p>

</HTML>
", revisions = "<html>
      <ul>
      <li>First implemented: <i>July, 2000</i>
      by <a href=\"http://www.control.lth.se/~hubertus/\">Hubertus Tummescheit</a>
      for the ThermoFluid Library with help from Jonas Eborn and Falko Jens Wagner
      </li>
      <li>Code reorganization, enhanced documentation, additional functions: <i>December, 2002</i>
      by <a href=\"http://www.control.lth.se/~hubertus/\">Hubertus Tummescheit</a> and move to Modelica
                            properties library.</li>
      <li>Inclusion into Modelica.Media: September 2003 </li>
      </ul>

      <address>Author: Hubertus Tummescheit, <br>
      Lund University<br>
      Department of Automatic Control<br>
      Box 118, 22100 Lund, Sweden<br>
      email: hubertus@control.lth.se
      </address>
</html>"));
      extends Modelica.Icons.Library;
    protected 
      type Rate = Real(final quantity = "Rate", final unit = "s-1");
      type MolarFlowRate = Real(final quantity = "MolarFlowRate", final unit = "mol/s");
      type MolarReactionRate = Real(final quantity = "MolarReactionRate", final unit = "mol/(m3.s)");
      type MolarEnthalpy = Real(final quantity = "MolarEnthalpy", final unit = "J/mol");
      type DerDensityByEntropy = Real(final quantity = "DerDensityByEntropy", final unit = "kg2.K/(m3.J)");
      type DerEnergyByPressure = Real(final quantity = "DerEnergyByPressure", final unit = "J/Pa");
      type DerEnergyByMoles = Real(final quantity = "DerEnergyByMoles", final unit = "J/mol");
      type DerEntropyByTemperature = Real(final quantity = "DerEntropyByTemperature", final unit = "J/K2");
      type DerEntropyByPressure = Real(final quantity = "DerEntropyByPressure", final unit = "J/(K.Pa)");
      type DerEntropyByMoles = Real(final quantity = "DerEntropyByMoles", final unit = "J/(mol.K)");
      type DerPressureByDensity = Real(final quantity = "DerPressureByDensity", final unit = "Pa.m3/kg");
      type DerPressureBySpecificVolume = Real(final quantity = "DerPressureBySpecificVolume", final unit = "Pa.kg/m3");
      type DerPressureByTemperature = Real(final quantity = "DerPressureByTemperature", final unit = "Pa/K");
      type DerVolumeByTemperature = Real(final quantity = "DerVolumeByTemperature", final unit = "m3/K");
      type DerVolumeByPressure = Real(final quantity = "DerVolumeByPressure", final unit = "m3/Pa");
      type DerVolumeByMoles = Real(final quantity = "DerVolumeByMoles", final unit = "m3/mol");
      type IsenthalpicExponent = Real(final quantity = "IsenthalpicExponent", unit = "1");
      type IsentropicExponent = Real(final quantity = "IsentropicExponent", unit = "1");
      type IsobaricVolumeExpansionCoefficient = Real(final quantity = "IsobaricVolumeExpansionCoefficient", unit = "1/K");
      type IsochoricPressureCoefficient = Real(final quantity = "IsochoricPressureCoefficient", unit = "1/K");
      type IsothermalCompressibility = Real(final quantity = "IsothermalCompressibility", unit = "1/Pa");
      type JouleThomsonCoefficient = Real(final quantity = "JouleThomsonCoefficient", unit = "K/Pa");
      constant Real MINPOS = 1e-09 "minimal value for physical variables which are always > 0.0";
      constant SI.Area AMIN = MINPOS "minimal init area";
      constant SI.Area AMAX = 100000.0 "maximal init area";
      constant SI.Area ANOM = 1.0 "nominal init area";
      constant SI.AmountOfSubstance MOLMIN =  -1.0 * MINPOS "minimal Mole Number";
      constant SI.AmountOfSubstance MOLMAX = 100000000.0 "maximal Mole Number";
      constant SI.AmountOfSubstance MOLNOM = 1.0 "nominal Mole Number";
      constant SI.Density DMIN = MINPOS "minimal init density";
      constant SI.Density DMAX = 100000.0 "maximal init density";
      constant SI.Density DNOM = 1.0 "nominal init density";
      constant SI.ThermalConductivity LAMMIN = MINPOS "minimal thermal conductivity";
      constant SI.ThermalConductivity LAMNOM = 1.0 "nominal thermal conductivity";
      constant SI.ThermalConductivity LAMMAX = 1000.0 "maximal thermal conductivity";
      constant SI.DynamicViscosity ETAMIN = MINPOS "minimal init dynamic viscosity";
      constant SI.DynamicViscosity ETAMAX = 100000000.0 "maximal init dynamic viscosity";
      constant SI.DynamicViscosity ETANOM = 100.0 "nominal init dynamic viscosity";
      constant SI.Energy EMIN =  -10000000000.0 "minimal init energy";
      constant SI.Energy EMAX = 10000000000.0 "maximal init energy";
      constant SI.Energy ENOM = 1000.0 "nominal init energy";
      constant SI.Entropy SMIN =  -1000000.0 "minimal init entropy";
      constant SI.Entropy SMAX = 1000000.0 "maximal init entropy";
      constant SI.Entropy SNOM = 1000.0 "nominal init entropy";
      constant SI.MassFlowRate MDOTMIN =  -100000.0 "minimal init mass flow rate";
      constant SI.MassFlowRate MDOTMAX = 100000.0 "maximal init mass flow rate";
      constant SI.MassFlowRate MDOTNOM = 1.0 "nominal init mass flow rate";
      constant SI.MassFraction MASSXMIN =  -1.0 * MINPOS "minimal init mass fraction";
      constant SI.MassFraction MASSXMAX = 1.0 "maximal init mass fraction";
      constant SI.MassFraction MASSXNOM = 0.1 "nominal init mass fraction";
      constant SI.Mass MMIN =  -1.0 * MINPOS "minimal init mass";
      constant SI.Mass MMAX = 100000000.0 "maximal init mass";
      constant SI.Mass MNOM = 1.0 "nominal init mass";
      constant SI.MolarMass MMMIN = 0.001 "minimal initial molar mass";
      constant SI.MolarMass MMMAX = 250.0 "maximal initial molar mass";
      constant SI.MolarMass MMNOM = 0.2 "nominal initial molar mass";
      constant SI.MoleFraction MOLEYMIN =  -1.0 * MINPOS "minimal init mole fraction";
      constant SI.MoleFraction MOLEYMAX = 1.0 "maximal init mole fraction";
      constant SI.MoleFraction MOLEYNOM = 0.1 "nominal init mole fraction";
      constant SI.MomentumFlux GMIN =  -100000000.0 "minimal init momentum flux";
      constant SI.MomentumFlux GMAX = 100000000.0 "maximal init momentum flux";
      constant SI.MomentumFlux GNOM = 1.0 "nominal init momentum flux";
      constant SI.Power POWMIN =  -100000000.0 "minimal init power or heat";
      constant SI.Power POWMAX = 100000000.0 "maximal init power or heat";
      constant SI.Power POWNOM = 1000.0 "nominal init power or heat";
      constant SI.Pressure PMIN = 10000.0 "minimal init pressure";
      constant SI.Pressure PMAX = 100000000.0 "maximal init pressure";
      constant SI.Pressure PNOM = 100000.0 "nominal init pressure";
      constant SI.Pressure COMPPMIN =  -1.0 * MINPOS "minimal init pressure";
      constant SI.Pressure COMPPMAX = 100000000.0 "maximal init pressure";
      constant SI.Pressure COMPPNOM = 100000.0 "nominal init pressure";
      constant SI.RatioOfSpecificHeatCapacities KAPPAMIN = 1.0 "minimal init isentropic exponent";
      constant SI.RatioOfSpecificHeatCapacities KAPPAMAX = 1.7 "maximal init isentropic exponent";
      constant SI.RatioOfSpecificHeatCapacities KAPPANOM = 1.2 "nominal init isentropic exponent";
      constant SI.SpecificEnergy SEMIN =  -100000000.0 "minimal init specific energy";
      constant SI.SpecificEnergy SEMAX = 100000000.0 "maximal init specific energy";
      constant SI.SpecificEnergy SENOM = 1000000.0 "nominal init specific energy";
      constant SI.SpecificEnthalpy SHMIN =  -100000000.0 "minimal init specific enthalpy";
      constant SI.SpecificEnthalpy SHMAX = 100000000.0 "maximal init specific enthalpy";
      constant SI.SpecificEnthalpy SHNOM = 1000000.0 "nominal init specific enthalpy";
      constant SI.SpecificEntropy SSMIN =  -1000000.0 "minimal init specific entropy";
      constant SI.SpecificEntropy SSMAX = 1000000.0 "maximal init specific entropy";
      constant SI.SpecificEntropy SSNOM = 1000.0 "nominal init specific entropy";
      constant SI.SpecificHeatCapacity CPMIN = MINPOS "minimal init specific heat capacity";
      constant SI.SpecificHeatCapacity CPMAX = 1000000.0 "maximal init specific heat capacity";
      constant SI.SpecificHeatCapacity CPNOM = 1000.0 "nominal init specific heat capacity";
      constant SI.Temperature TMIN = MINPOS "minimal init temperature";
      constant SI.Temperature TMAX = 100000.0 "maximal init temperature";
      constant SI.Temperature TNOM = 320.0 "nominal init temperature";
      constant SI.ThermalConductivity LMIN = MINPOS "minimal init thermal conductivity";
      constant SI.ThermalConductivity LMAX = 500.0 "maximal init thermal conductivity";
      constant SI.ThermalConductivity LNOM = 1.0 "nominal init thermal conductivity";
      constant SI.Velocity VELMIN =  -100000.0 "minimal init speed";
      constant SI.Velocity VELMAX = 100000.0 "maximal init speed";
      constant SI.Velocity VELNOM = 1.0 "nominal init speed";
      constant SI.Volume VMIN = 0.0 "minimal init volume";
      constant SI.Volume VMAX = 100000.0 "maximal init volume";
      constant SI.Volume VNOM = 0.001 "nominal init volume";
      package ThermoFluidSpecial "property records used by the ThermoFluid library"
        record FixedIGProperties "constant properties for ideal gases"
          extends Modelica.Icons.Record;
          parameter Integer nspecies(min = 1) "number of components";
          SI.MolarMass[nspecies] MM "molar mass of components";
          Real[nspecies] invMM "inverse of molar mass of components";
          SI.SpecificHeatCapacity[nspecies] R "gas constant";
          SI.SpecificEnthalpy[nspecies] Hf "enthalpy of formation at 298.15K";
          SI.SpecificEnthalpy[nspecies] H0 "H0(298.15K) - H0(0K)";
        end FixedIGProperties;
        record ThermoBaseVars
          extends Modelica.Icons.Record;
          parameter Integer n(min = 1) "discretization number";
          parameter Integer nspecies(min = 1) "number of species";
          SI.Pressure[n] p(min = PMIN, max = PMAX, nominal = PNOM, start = fill(100000.0, n)) "Pressure";
          SI.Temperature[n] T(min = TMIN, max = TMAX, nominal = TNOM) "temperature";
          SI.Density[n] d(min = DMIN, max = DMAX, nominal = DNOM) "density";
          SI.SpecificEnthalpy[n] h(min = SHMIN, max = SHMAX, nominal = SHNOM) "specific enthalpy";
          SI.SpecificEntropy[n] s(min = SSMIN, max = SSMAX, nominal = SSNOM) "specific entropy";
          SI.RatioOfSpecificHeatCapacities[n] kappa "ratio of cp/cv";
          SI.Mass[n] M(min = MMIN, max = MMAX, nominal = MNOM) "Total mass";
          SI.Energy[n] U(min = EMIN, max = EMAX, nominal = ENOM) "Inner energy";
          SI.MassFlowRate[n] dM(min = MDOTMIN, max = MDOTMAX, nominal = MDOTNOM) "Change in total mass";
          SI.Power[n] dU(min = POWMIN, max = POWMAX, nominal = POWNOM) "Change in inner energy";
          SI.Volume[n] V(min = VMIN, max = VMAX, nominal = VNOM) "Volume";
          SI.MassFraction[n,nspecies] mass_x(min = MASSXMIN, max = MASSXMAX, nominal = MASSXNOM) "mass fraction";
          SI.MoleFraction[n,nspecies] mole_y(min = MOLEYMIN, max = MOLEYMAX, nominal = MOLEYNOM) "mole fraction";
          SI.Mass[n,nspecies] M_x(min = MMIN, max = MMAX, nominal = MNOM) "component mass";
          SI.MassFlowRate[n,nspecies] dM_x(min = MDOTMIN, max = MDOTMAX, nominal = MDOTNOM) "rate of change in component mass";
          MolarFlowRate[n,nspecies] dZ(min =  -1000000.0, max = 1000000.0, nominal = 0.0) "rate of change in component moles";
          MolarFlowRate[n,nspecies] rZ(min =  -1000000.0, max = 1000000.0, nominal = 0.0) "Reaction(source) mole rates";
          SI.MolarMass[n] MM(min = MMMIN, max = MMMAX, nominal = MMNOM) "molar mass of mixture";
          SI.AmountOfSubstance[n] Moles(min = MOLMIN, max = MOLMAX, nominal = MOLNOM) "total moles";
          SI.AmountOfSubstance[n,nspecies] Moles_z(min = MOLMIN, max = MOLMAX, nominal = MOLNOM) "mole vector";
          annotation(Documentation(info = "<HTML>
                         <h4>Model description</h4>
                              <p>
                              <b>ThermoBaseVars</b> is inherited by all medium property models
                              and by all models defining the dynamic states for the conservation
                              of mass and energy. Thus it is a good choice as a restricting class
                              for any medium model or dynamic state model.
                              </p>
                              </HTML>
                              "));
        end ThermoBaseVars;
        record ThermoProperties "Thermodynamic base property data for all state models"
          extends Modelica.Icons.Record;
          parameter Integer nspecies(min = 1) "number of species";
          SI.Temperature T(min = TMIN, max = TMAX, nominal = TNOM) "temperature";
          SI.Density d(min = DMIN, max = DMAX, nominal = DNOM) "density";
          SI.Pressure p(min = PMIN, max = PMAX, nominal = PNOM) "pressure";
          SI.Volume V(min = VMIN, max = VMAX, nominal = VNOM) "Volume";
          SI.SpecificEnthalpy h(min = SHMIN, max = SHMAX, nominal = SHNOM) "specific enthalpy";
          SI.SpecificEnergy u(min = SEMIN, max = SEMAX, nominal = SENOM) "specific inner energy";
          SI.SpecificEntropy s(min = SSMIN, max = SSMAX, nominal = SSNOM) "specific entropy";
          SI.SpecificGibbsFreeEnergy g(min = SHMIN, max = SHMAX, nominal = SHNOM) "specific Gibbs free energy";
          SI.SpecificHeatCapacity cp(min = CPMIN, max = CPMAX, nominal = CPNOM) "heat capacity at constant pressure";
          SI.SpecificHeatCapacity cv(min = CPMIN, max = CPMAX, nominal = CPNOM) "heat capacity at constant volume";
          SI.SpecificHeatCapacity R(min = CPMIN, max = CPMAX, nominal = CPNOM) "gas constant";
          SI.MolarMass MM(min = MMMIN, max = MMMAX, nominal = MMNOM) "molar mass of mixture";
          SI.MassFraction[nspecies] mass_x(min = MASSXMIN, max = MASSXMAX, nominal = MASSXNOM) "mass fraction";
          SI.MoleFraction[nspecies] mole_y(min = MOLEYMIN, max = MOLEYMAX, nominal = MOLEYNOM) "mole fraction";
          SI.RatioOfSpecificHeatCapacities kappa "ratio of cp/cv";
          SI.DerDensityByTemperature ddTp "derivative of density by temperature at constant pressure";
          SI.DerDensityByPressure ddpT "derivative of density by pressure at constant temperature";
          Real dupT(unit = "m3.kg-1") "derivative of inner energy by pressure at constant T";
          Real dudT(unit = "(J.m3)/(kg2)") "derivative of inner energy by density at constant T";
          SI.SpecificHeatCapacity duTp "derivative of inner energy by temperature at constant p";
          SI.SpecificEnergy ddx[nspecies] "derivative vector of density by change in mass composition";
          SI.SpecificEnergy[nspecies] compu(min = SEMIN, max = SEMAX, nominal = SENOM) "inner energy of the components";
          SI.Pressure[nspecies] compp(min = COMPPMIN, max = COMPPMAX, nominal = COMPPNOM) "partial pressures of the components";
          SI.Velocity a(min = VELMIN, max = VELMAX, nominal = VELNOM) "speed of sound";
          SI.HeatCapacity dUTZ "derivative of inner energy by temperature at constant moles";
          SI.MolarInternalEnergy[nspecies] dUZT "derivative of inner energy by moles at constant temperature";
          SI.SpecificEnthalpy[nspecies] dHMxT(min = SEMIN, max = SEMAX, nominal = SENOM) "derivative of total enthalpy wrt component mass at constant T";
          Real dpT "derivative of pressure w.r.t. temperature";
          Real dpZ[nspecies] "derivative of pressure w.r.t. moles";
          annotation(Documentation(info = "<HTML>
        <h4>Model description</h4>
        <p>
        A base class for medium property models which work with most of the
        versions of dynamic states that are available in the ThermoFluid
        library. Currently used by all ideal gas models.
        </p>
        </HTML>
        "));
        end ThermoProperties;
        record ThermoProperties_ph "Thermodynamic property data for pressure p and specific enthalpy h as dynamic states"
          extends Modelica.Icons.Record;
          SI.Temperature T(min = 1e-09, max = 10000.0, nominal = 298.15) "temperature";
          SI.Density d(min = 1e-09, max = 10000.0, nominal = 10.0) "density";
          SI.SpecificEnergy u(min =  -100000000.0, max = 100000000.0, nominal = 1000000.0) "specific inner energy";
          SI.SpecificEntropy s(min =  -1000000.0, max = 1000000.0, nominal = 1000.0) "specific entropy";
          SI.SpecificHeatCapacity cp(min = 1.0, max = 1000000.0, nominal = 1000.0) "heat capacity at constant pressure";
          SI.SpecificHeatCapacity cv(min = 1.0, max = 1000000.0, nominal = 1000.0) "heat capacity at constant volume";
          SI.SpecificHeatCapacity R(min = 1.0, max = 1000000.0, nominal = 1000.0) "gas constant";
          SI.RatioOfSpecificHeatCapacities kappa "ratio of cp/cv";
          SI.Velocity a(min = 1.0, max = 10000.0, nominal = 300.0) "speed of sound";
          SI.DerDensityByEnthalpy ddhp "derivative of density by enthalpy at constant pressure";
          SI.DerDensityByPressure ddph "derivative of density by pressure at constant enthalpy";
          Real duph(unit = "m3/kg") "derivative of inner energy by pressure at constant enthalpy";
          Real duhp(unit = "1") "derivative of inner energy by enthalpy at constant pressure";
          annotation(Documentation(info = "<HTML>
<h4>Model description</h4>
<p>
A base class for medium property models which
use pressure and enthalpy as dynamic states.
This is the preferred model for fluids that can also be in the
two phase and liquid regions.
</p>
</HTML>
"));
        end ThermoProperties_ph;
        record ThermoProperties_pT "Thermodynamic property data for pressure p and temperature T as dynamic states"
          extends Modelica.Icons.Record;
          SI.Density d(min = 1e-09, max = 10000.0, nominal = 10.0) "density";
          SI.SpecificEnthalpy h(min =  -100000000.0, max = 100000000.0, nominal = 1000000.0) "specific enthalpy";
          SI.SpecificEnergy u(min =  -100000000.0, max = 100000000.0, nominal = 1000000.0) "specific inner energy";
          SI.SpecificEntropy s(min =  -1000000.0, max = 1000000.0, nominal = 1000.0) "specific entropy";
          SI.SpecificHeatCapacity cp(min = 1.0, max = 1000000.0, nominal = 1000.0) "heat capacity at constant pressure";
          SI.SpecificHeatCapacity cv(min = 1.0, max = 1000000.0, nominal = 1000.0) "heat capacity at constant volume";
          SI.SpecificHeatCapacity R(min = 1.0, max = 1000000.0, nominal = 1000.0) "gas constant";
          SI.RatioOfSpecificHeatCapacities kappa "ratio of cp/cv";
          SI.Velocity a(min = 1.0, max = 10000.0, nominal = 300.0) "speed of sound";
          SI.DerDensityByTemperature ddTp "derivative of density by temperature at constant pressure";
          SI.DerDensityByPressure ddpT "derivative of density by pressure at constant temperature";
          Real dupT(unit = "m3.kg-1") "derivative of inner energy by pressure at constant T";
          SI.SpecificHeatCapacity duTp "derivative of inner energy by temperature at constant p";
          annotation(Documentation(info = "<HTML>
<h4>Model description</h4>
<p>
A base class for medium property models which use pressure and temperature as dynamic states.
This is a reasonable model for fluids that can also be in the gas and
liquid regions, but never in the two-phase region.
</p>
</HTML>
"));
        end ThermoProperties_pT;
        record ThermoProperties_dT "Thermodynamic property data for density d and temperature T as dynamic states"
          extends Modelica.Icons.Record;
          SI.Pressure p(min = 1.0, max = 1000000000.0, nominal = 100000.0) "pressure";
          SI.SpecificEnthalpy h(min =  -100000000.0, max = 100000000.0, nominal = 1000000.0) "specific enthalpy";
          SI.SpecificEnergy u(min =  -100000000.0, max = 100000000.0, nominal = 1000000.0) "specific inner energy";
          SI.SpecificEntropy s(min =  -1000000.0, max = 1000000.0, nominal = 1000.0) "specific entropy";
          SI.SpecificHeatCapacity cp(min = 1.0, max = 1000000.0, nominal = 1000.0) "heat capacity at constant pressure";
          SI.SpecificHeatCapacity cv(min = 1.0, max = 1000000.0, nominal = 1000.0) "heat capacity at constant volume";
          SI.SpecificHeatCapacity R(min = 1.0, max = 1000000.0, nominal = 1000.0) "gas constant";
          SI.RatioOfSpecificHeatCapacities kappa "ratio of cp/cv";
          SI.Velocity a(min = 1.0, max = 10000.0, nominal = 300.0) "speed of sound";
          Real dudT(unit = "m5/(kg.s2)") "derivative of inner energy by density at constant T";
          annotation(Documentation(info = "<HTML>
<h4>Model description</h4>
<p>
A base class for medium property models which use density and temperature as dynamic states.
This is a reasonable model for fluids that can be in the gas, liquid
and two-phase region. The model is numerically not well suited for
liquids except if the pressure is always above approx. 80% of the
critical pressure.
</p>
</HTML>
"));
        end ThermoProperties_dT;
        record TransportProps "record with transport properties"
          extends Modelica.Icons.Record;
          SI.DynamicViscosity eta;
          SI.ThermalConductivity lam;
        end TransportProps;
        function gibbsToProps_ph "calulate property record for pressure and specific enthalpy as states from dimensionless Gibbs function"
          extends Modelica.Icons.Function;
          input GibbsDerivs g "dimensionless derivatives of Gibbs function";
          output ThermoProperties_ph pro "property record for pressure and specific enthalpy as dynamic states";
        protected 
          Real vt(unit = "m3.kg-1.K-1") "derivative of specific volume w.r.t. temperature";
          Real vp(unit = "m4.kg-2.s2") "derivative of specific volume w.r.t. pressure";
        algorithm 
          pro.T:=g.T;
          pro.R:=g.R;
          pro.d:=g.p / (pro.R * pro.T * g.pi * g.gpi);
          pro.u:=g.T * g.R * (g.tau * g.gtau - g.pi * g.gpi);
          pro.s:=pro.R * (g.tau * g.gtau - g.g);
          pro.cp:= -pro.R * g.tau * g.tau * g.gtautau;
          pro.cv:=pro.R * ( -g.tau * g.tau * g.gtautau + ((g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi)) / g.gpipi);
          pro.a:=abs(g.R * g.T * (g.gpi * g.gpi) / (((g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi)) / (g.tau * g.tau * g.gtautau) - g.gpipi)) ^ 0.5;
          vt:=g.R / g.p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
          vp:=(g.R * g.T) / (g.p * g.p) * g.pi * g.pi * g.gpipi;
          pro.kappa:= -(1 / (pro.d * g.p) * pro.cp) / (vp * pro.cp + vt * vt * g.T);
          pro.ddhp:= -(pro.d * pro.d * vt) / pro.cp;
          pro.ddph:= -(pro.d * pro.d * (vp * pro.cp - vt / pro.d + g.T * vt * vt)) / pro.cp;
          pro.duph:= -1 / pro.d + g.p / (pro.d * pro.d) * pro.ddph;
          pro.duhp:=1 + g.p / (pro.d * pro.d) * pro.ddhp;
        end gibbsToProps_ph;
        function gibbsToBoundaryProps "calulate phase boundary property record from dimensionless Gibbs function"
          extends Modelica.Icons.Function;
          input GibbsDerivs g "dimensionless derivatives of Gibbs function";
          output PhaseBoundaryProperties sat "phase boundary properties";
        protected 
          Real vt(unit = "m3.kg-1.K-1") "derivative of specific volume w.r.t. temperature";
          Real vp(unit = "m4.kg-2.s2") "derivative of specific volume w.r.t. pressure";
        algorithm 
          sat.d:=g.p / (g.R * g.T * g.pi * g.gpi);
          sat.h:=g.R * g.T * g.tau * g.gtau;
          sat.u:=g.T * g.R * (g.tau * g.gtau - g.pi * g.gpi);
          sat.s:=g.R * (g.tau * g.gtau - g.g);
          sat.cp:= -g.R * g.tau * g.tau * g.gtautau;
          sat.cv:=g.R * ( -g.tau * g.tau * g.gtautau + ((g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi)) / g.gpipi);
          vt:=g.R / g.p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
          vp:=(g.R * g.T) / (g.p * g.p) * g.pi * g.pi * g.gpipi;
          sat.pt:= -(g.p / g.T * (g.gpi - g.tau * g.gtaupi)) / (g.gpipi * g.pi);
          sat.pd:= -(g.R * g.T * g.gpi * g.gpi) / g.gpipi;
        end gibbsToBoundaryProps;
        function gibbsToProps_dT "calulate property record for density and temperature as states from dimensionless Gibbs function"
          extends Modelica.Icons.Function;
          input GibbsDerivs g "dimensionless derivatives of Gibbs function";
          output ThermoProperties_dT pro "property record for density and temperature as dynamic states";
        protected 
          Real vt(unit = "m3.kg-1.K-1") "derivative of specific volume w.r.t. temperature";
          Real vp(unit = "m4.kg-2.s2") "derivative of specific volume w.r.t. pressure";
          Modelica.SIunits.Density d;
        algorithm 
          pro.R:=g.R;
          pro.p:=g.p;
          pro.u:=g.T * g.R * (g.tau * g.gtau - g.pi * g.gpi);
          pro.h:=g.R * g.T * g.tau * g.gtau;
          pro.s:=pro.R * (g.tau * g.gtau - g.g);
          pro.cp:= -pro.R * g.tau * g.tau * g.gtautau;
          pro.cv:=pro.R * ( -g.tau * g.tau * g.gtautau + ((g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi)) / g.gpipi);
          vt:=g.R / g.p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
          vp:=(g.R * g.T) / (g.p * g.p) * g.pi * g.pi * g.gpipi;
          pro.kappa:= -(1 / (g.p / (pro.R * g.T * g.pi * g.gpi) * g.p) * pro.cp) / (vp * pro.cp + vt * vt * g.T);
          pro.a:=abs(g.R * g.T * (g.gpi * g.gpi) / (((g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi)) / (g.tau * g.tau * g.gtautau) - g.gpipi)) ^ 0.5;
          d:=g.p / (pro.R * g.T * g.pi * g.gpi);
          pro.dudT:=(pro.p - (g.T * vt) / vp) / (d * d);
        end gibbsToProps_dT;
        function gibbsToProps_pT "calulate property record for pressure and temperature as states from dimensionless Gibbs function"
          extends Modelica.Icons.Function;
          input GibbsDerivs g "dimensionless derivatives of Gibbs function";
          output ThermoProperties_pT pro "property record for pressure and temperature as dynamic states";
        protected 
          Real vt(unit = "m3.kg-1.K-1") "derivative of specific volume w.r.t. temperature";
          Real vp(unit = "m4.kg-2.s2") "derivative of specific volume w.r.t. pressure";
        algorithm 
          pro.R:=g.R;
          pro.d:=g.p / (pro.R * g.T * g.pi * g.gpi);
          pro.u:=g.T * g.R * (g.tau * g.gtau - g.pi * g.gpi);
          pro.h:=g.R * g.T * g.tau * g.gtau;
          pro.s:=pro.R * (g.tau * g.gtau - g.g);
          pro.cp:= -pro.R * g.tau * g.tau * g.gtautau;
          pro.cv:=pro.R * ( -g.tau * g.tau * g.gtautau + ((g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi)) / g.gpipi);
          vt:=g.R / g.p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
          vp:=(g.R * g.T) / (g.p * g.p) * g.pi * g.pi * g.gpipi;
          pro.kappa:= -(1 / (pro.d * g.p) * pro.cp) / (vp * pro.cp + vt * vt * g.T);
          pro.a:=abs(g.R * g.T * (g.gpi * g.gpi) / (((g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi)) / (g.tau * g.tau * g.gtautau) - g.gpipi)) ^ 0.5;
          pro.ddpT:= -pro.d * pro.d * vp;
          pro.ddTp:= -pro.d * pro.d * vt;
          pro.duTp:=pro.cp - g.p * vt;
          pro.dupT:= -g.T * vt - g.p * vp;
        end gibbsToProps_pT;
        function helmholtzToProps_ph "calulate property record for pressure and specific enthalpy as states from dimensionless Helmholtz function"
          extends Modelica.Icons.Function;
          input HelmholtzDerivs f "dimensionless derivatives of Helmholtz function";
          output ThermoProperties_ph pro "property record for pressure and specific enthalpy as dynamic states";
        protected 
          SI.Pressure p "pressure";
          DerPressureByDensity pd "derivative of pressure w.r.t. density";
          DerPressureByTemperature pt "derivative of pressure w.r.t. temperature";
          DerPressureBySpecificVolume pv "derivative of pressure w.r.t. specific volume";
        algorithm 
          pro.d:=f.d;
          pro.T:=f.T;
          pro.R:=f.R;
          pro.s:=f.R * (f.tau * f.ftau - f.f);
          pro.u:=f.R * f.T * f.tau * f.ftau;
          p:=pro.d * pro.R * pro.T * f.delta * f.fdelta;
          pd:=f.R * f.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
          pt:=f.R * f.d * f.delta * (f.fdelta - f.tau * f.fdeltatau);
          pv:= -pd * f.d * f.d;
          pro.cp:=f.R * ( -f.tau * f.tau * f.ftautau + (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) ^ 2 / (2 * f.delta * f.fdelta + f.delta * f.delta * f.fdeltadelta));
          pro.cv:=f.R * ( -f.tau * f.tau * f.ftautau);
          pro.kappa:=1 / (f.d * f.R * f.d * f.T * f.delta * f.fdelta) * ( -pv * pro.cv + pt * pt * f.T) / pro.cv;
          pro.a:=abs(f.R * f.T * (2 * f.delta * f.fdelta + f.delta * f.delta * f.fdeltadelta - ((f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) * (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau)) / (f.tau * f.tau * f.ftautau))) ^ 0.5;
          pro.ddph:=(f.d * (pro.cv * f.d + pt)) / (f.d * f.d * pd * pro.cv + f.T * pt * pt);
          pro.ddhp:= -(f.d * f.d * pt) / (f.d * f.d * pd * pro.cv + f.T * pt * pt);
          pro.duph:= -1 / pro.d + p / (pro.d * pro.d) * pro.ddph;
          pro.duhp:=1 + p / (pro.d * pro.d) * pro.ddhp;
        end helmholtzToProps_ph;
        function helmholtzToProps_pT "calulate property record for pressure and temperature as states from dimensionless Helmholtz function"
          extends Modelica.Icons.Function;
          input HelmholtzDerivs f "dimensionless derivatives of Helmholtz function";
          output ThermoProperties_pT pro "property record for pressure and temperature as dynamic states";
        protected 
          DerPressureByDensity pd "derivative of pressure w.r.t. density";
          DerPressureByTemperature pt "derivative of pressure w.r.t. temperature";
          DerPressureBySpecificVolume pv "derivative of pressure w.r.t. specific volume";
          IsobaricVolumeExpansionCoefficient alpha "isobaric volume expansion coefficient";
          IsothermalCompressibility gamma "isothermal compressibility";
          SI.Pressure p "Pressure";
        algorithm 
          pro.d:=f.d;
          pro.R:=f.R;
          pro.s:=f.R * (f.tau * f.ftau - f.f);
          pro.h:=f.R * f.T * (f.tau * f.ftau + f.delta * f.fdelta);
          pro.u:=f.R * f.T * f.tau * f.ftau;
          pd:=f.R * f.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
          pt:=f.R * f.d * f.delta * (f.fdelta - f.tau * f.fdeltatau);
          pv:= -f.d * f.d * pd;
          alpha:= -(f.d * pt) / pv;
          gamma:= -f.d / pv;
          p:=f.R * f.d * f.T * f.delta * f.fdelta;
          pro.cp:=f.R * ( -f.tau * f.tau * f.ftautau + (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) ^ 2 / (2 * f.delta * f.fdelta + f.delta * f.delta * f.fdeltadelta));
          pro.cv:=f.R * ( -f.tau * f.tau * f.ftautau);
          pro.kappa:=1 / (f.d * f.R * f.d * f.T * f.delta * f.fdelta) * ( -pv * pro.cv + pt * pt * f.T) / pro.cv;
          pro.a:=abs(f.R * f.T * (2 * f.delta * f.fdelta + f.delta * f.delta * f.fdeltadelta - ((f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) * (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau)) / (f.tau * f.tau * f.ftautau))) ^ 0.5;
          pro.ddTp:= -pt / pd;
          pro.ddpT:=1 / pd;
          pro.dupT:=(gamma * p) / f.d - (alpha * f.T) / f.d;
          pro.duTp:=pro.cp - (alpha * p) / f.d;
        end helmholtzToProps_pT;
        function helmholtzToProps_dT "calulate property record for density and temperature as states from dimensionless Helmholtz function"
          extends Modelica.Icons.Function;
          input HelmholtzDerivs f "dimensionless derivatives of Helmholtz function";
          output ThermoProperties_dT pro "property record for density and temperature as dynamic states";
        protected 
          DerPressureByTemperature pt "derivative of pressure w.r.t. temperature";
          DerPressureBySpecificVolume pv "derivative of pressure w.r.t. pressure";
        algorithm 
          pro.p:=f.R * f.d * f.T * f.delta * f.fdelta;
          pro.R:=f.R;
          pro.s:=f.R * (f.tau * f.ftau - f.f);
          pro.h:=f.R * f.T * (f.tau * f.ftau + f.delta * f.fdelta);
          pro.u:=f.R * f.T * f.tau * f.ftau;
          pv:= -f.d * f.d * f.R * f.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
          pt:=f.R * f.d * f.delta * (f.fdelta - f.tau * f.fdeltatau);
          pro.cp:=f.R * ( -f.tau * f.tau * f.ftautau + (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) ^ 2 / (2 * f.delta * f.fdelta + f.delta * f.delta * f.fdeltadelta));
          pro.cv:=f.R * ( -f.tau * f.tau * f.ftautau);
          pro.kappa:=1 / (f.d * pro.p) * ( -pv * pro.cv + pt * pt * f.T) / pro.cv;
          pro.a:=abs(f.R * f.T * (2 * f.delta * f.fdelta + f.delta * f.delta * f.fdeltadelta - ((f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) * (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau)) / (f.tau * f.tau * f.ftautau))) ^ 0.5;
          pro.dudT:=(pro.p - f.T * pt) / (f.d * f.d);
        end helmholtzToProps_dT;
        function TwoPhaseToProps_ph "compute property record for pressure and specific enthalpy as states from saturation properties"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "saturation property record";
          output ThermoProperties_ph pro "property record for pressure and specific enthalpy as dynamic states";
        protected 
          Real dht(unit = "(J/kg)/K") "derivative of specific enthalpy w.r.t. temperature";
          Real dhd(unit = "(J/kg)/(kg/m3)") "derivative of specific enthalpy w.r.t. density";
          Real detph(unit = "m4.s4/(K.s8)") "thermodynamic determinant";
        algorithm 
          pro.d:=sat.d;
          pro.T:=sat.T;
          pro.u:=sat.u;
          pro.s:=sat.s;
          pro.cv:=sat.cv;
          pro.R:=sat.R;
          pro.cp:=Modelica.Constants.inf;
          pro.kappa:= -(1 / (sat.d * sat.p) * sat.dpT * sat.dpT * sat.T) / sat.cv;
          pro.a:=Modelica.Constants.inf;
          dht:=sat.cv + sat.dpT / sat.d;
          dhd:= -(sat.T * sat.dpT) / (sat.d * sat.d);
          detph:= -sat.dpT * dhd;
          pro.ddph:=dht / detph;
          pro.ddhp:= -sat.dpT / detph;
        end TwoPhaseToProps_ph;
        function TwoPhaseToProps_dT "compute property record for density and temperature as states from saturation properties"
          extends Modelica.Icons.Function;
          input SaturationProperties sat "saturation properties";
          output ThermoProperties_dT pro "property record for density and temperature as dynamic states";
        algorithm 
          pro.p:=sat.p;
          pro.h:=sat.h;
          pro.u:=sat.u;
          pro.s:=sat.s;
          pro.cv:=sat.cv;
          pro.cp:=Modelica.Constants.inf;
          pro.R:=sat.R;
          pro.kappa:= -(1 / (sat.d * sat.p) * sat.dpT * sat.dpT * sat.T) / sat.cv;
          pro.a:=Modelica.Constants.inf;
          pro.dudT:=(sat.p - sat.T * sat.dpT) / (sat.d * sat.d);
        end TwoPhaseToProps_dT;
      end ThermoFluidSpecial;
    public 
      record SaturationProperties "properties in the two phase region"
        extends Modelica.Icons.Record;
        SI.Temp_K T "temperature";
        SI.Density d "density";
        SI.Pressure p "pressure";
        SI.SpecificEnergy u "specific inner energy";
        SI.SpecificEnthalpy h "specific enthalpy";
        SI.SpecificEntropy s "specific entropy";
        SI.SpecificHeatCapacity cp "heat capacity at constant pressure";
        SI.SpecificHeatCapacity cv "heat capacity at constant volume";
        SI.SpecificHeatCapacity R "gas constant";
        SI.RatioOfSpecificHeatCapacities kappa "isentropic expansion coefficient";
        PhaseBoundaryProperties liq "thermodynamic base properties on the boiling curve";
        PhaseBoundaryProperties vap "thermodynamic base properties on the dew curve";
        Real dpT(unit = "Pa/K") "derivative of saturation pressure wrt temperature";
        SI.MassFraction x "vapour mass fraction";
      end SaturationProperties;
      record SaturationBoundaryProperties "properties on both phase boundaries, including some derivatives"
        extends Modelica.Icons.Record;
        SI.Temp_K T "Saturation temperature";
        SI.Density dl "Liquid density";
        SI.Density dv "Vapour density";
        SI.SpecificEnthalpy hl "Liquid specific enthalpy";
        SI.SpecificEnthalpy hv "Vapour specific enthalpy";
        Real dTp "derivative of temperature wrt saturation pressure";
        Real ddldp "derivative of density along boiling curve";
        Real ddvdp "derivative of density along dew curve";
        Real dhldp "derivative of specific enthalpy along boiling curve";
        Real dhvdp "derivative of specific enthalpy along dew curve";
        SI.MassFraction x "vapour mass fraction";
      end SaturationBoundaryProperties;
      record IF97BaseTwoPhase "Intermediate property data record for IF 97"
        extends Modelica.Icons.Record;
        Integer phase = 0 "phase: 2 for two-phase, 1 for one phase, 0 if unknown";
        Integer region(min = 1, max = 5) "IF 97 region";
        SI.Pressure p "pressure";
        SI.Temperature T "temperature";
        SI.SpecificEnthalpy h "specific enthalpy";
        SI.SpecificHeatCapacity R "gas constant";
        SI.SpecificHeatCapacity cp "specific heat capacity";
        SI.SpecificHeatCapacity cv "specific heat capacity";
        SI.Density rho "density";
        SI.SpecificEntropy s "specific entropy";
        DerPressureByTemperature pt "derivative of pressure wrt temperature";
        DerPressureByDensity pd "derivative of pressure wrt density";
        Real vt "derivative of specific volume w.r.t. temperature";
        Real vp "derivative of specific volume w.r.t. pressure";
        Real x "dryness fraction";
        Real dpT "dp/dT derivative of saturation curve";
      end IF97BaseTwoPhase;
      record IF97PhaseBoundaryProperties "thermodynamic base properties on the phase boundary for IF97 steam tables"
        extends Modelica.Icons.Record;
        Boolean region3boundary "true if boundary between 2-phase and region 3";
        SI.SpecificHeatCapacity R "specific heat capacity";
        SI.Temperature T "temperature";
        SI.Density d "density";
        SI.SpecificEnthalpy h "specific enthalpy";
        SI.SpecificEntropy s "specific entropy";
        SI.SpecificHeatCapacity cp "heat capacity at constant pressure";
        SI.SpecificHeatCapacity cv "heat capacity at constant volume";
        DerPressureByTemperature dpT "dp/dT derivative of saturation curve";
        DerPressureByTemperature pt "derivative of pressure wrt temperature";
        DerPressureByDensity pd "derivative of pressure wrt density";
        Real vt(unit = "m3/(kg.K)") "derivative of specific volume w.r.t. temperature";
        Real vp(unit = "m3/(kg.Pa)") "derivative of specific volume w.r.t. pressure";
      end IF97PhaseBoundaryProperties;
      record GibbsDerivs "derivatives of dimensionless Gibbs-function w.r.t dimensionless pressure and temperature"
        extends Modelica.Icons.Record;
        SI.Pressure p "pressure";
        SI.Temperature T "temperature";
        SI.SpecificHeatCapacity R "specific heat capacity";
        Real pi(unit = "1") "dimensionless pressure";
        Real tau(unit = "1") "dimensionless temperature";
        Real g(unit = "1") "dimensionless Gibbs-function";
        Real gpi(unit = "1") "derivative of g w.r.t. pi";
        Real gpipi(unit = "1") "2nd derivative of g w.r.t. pi";
        Real gtau(unit = "1") "derivative of g w.r.t. tau";
        Real gtautau(unit = "1") "2nd derivative of g w.r.t tau";
        Real gtaupi(unit = "1") "mixed derivative of g w.r.t. pi and tau";
      end GibbsDerivs;
      record HelmholtzDerivs "derivatives of dimensionless Helmholtz-function w.r.t dimensionless pressuredensity and temperature"
        extends Modelica.Icons.Record;
        SI.Density d "density";
        SI.Temperature T "temperature";
        SI.SpecificHeatCapacity R "specific heat capacity";
        Real delta(unit = "1") "dimensionless density";
        Real tau(unit = "1") "dimensionless temperature";
        Real f(unit = "1") "dimensionless Helmholtz-function";
        Real fdelta(unit = "1") "derivative of f w.r.t. delta";
        Real fdeltadelta(unit = "1") "2nd derivative of f w.r.t. delta";
        Real ftau(unit = "1") "derivative of f w.r.t. tau";
        Real ftautau(unit = "1") "2nd derivative of f w.r.t. tau";
        Real fdeltatau(unit = "1") "mixed derivative of f w.r.t. delta and tau";
      end HelmholtzDerivs;
      record TwoPhaseTransportProps "defines properties on both phase boundaries, needed in the two phase region"
        extends Modelica.Icons.Record;
        SI.Density d_vap "density on the dew line";
        SI.Density d_liq "density on the bubble line";
        SI.DynamicViscosity eta_vap "dynamic viscosity on the dew line";
        SI.DynamicViscosity eta_liq "dynamic viscosity on the bubble line";
        SI.ThermalConductivity lam_vap "thermal conductivity on the dew line";
        SI.ThermalConductivity lam_liq "thermal conductivity on the bubble line";
        SI.SpecificHeatCapacity cp_vap "cp on the dew line";
        SI.SpecificHeatCapacity cp_liq "cp on the bubble line";
        SI.MassFraction x "steam quality";
      end TwoPhaseTransportProps;
      record PhaseBoundaryProperties "thermodynamic base properties on the phase boundary"
        extends Modelica.Icons.Record;
        SI.Density d "density";
        SI.SpecificEnthalpy h "specific enthalpy";
        SI.SpecificEnergy u "inner energy";
        SI.SpecificEntropy s "specific entropy";
        SI.SpecificHeatCapacity cp "heat capacity at constant pressure";
        SI.SpecificHeatCapacity cv "heat capacity at constant volume";
        DerPressureByTemperature pt "derivative of pressure wrt temperature";
        DerPressureByDensity pd "derivative of pressure wrt density";
      end PhaseBoundaryProperties;
      record NewtonDerivatives_ph "derivatives for fast inverse calculations of Helmholtz functions: p & h"
        extends Modelica.Icons.Record;
        SI.Pressure p "pressure";
        SI.SpecificEnthalpy h "specific enthalpy";
        DerPressureByDensity pd "derivative of pressure w.r.t. density";
        DerPressureByTemperature pt "derivative of pressure w.r.t. temperature";
        Real hd "derivative of specific enthalpy w.r.t. density";
        Real ht "derivative of specific enthalpy w.r.t. temperature";
      end NewtonDerivatives_ph;
      record NewtonDerivatives_ps "derivatives for fast inverse calculation of Helmholtz functions: p & s"
        extends Modelica.Icons.Record;
        SI.Pressure p "pressure";
        SI.SpecificEntropy s "specific entropy";
        DerPressureByDensity pd "derivative of pressure w.r.t. density";
        DerPressureByTemperature pt "derivative of pressure w.r.t. temperature";
        Real sd "derivative of specific entropy w.r.t. density";
        Real st "derivative of specific entropy w.r.t. temperature";
      end NewtonDerivatives_ps;
      record NewtonDerivatives_pT "derivatives for fast inverse calculations of Helmholtz functions:p & T"
        extends Modelica.Icons.Record;
        SI.Pressure p "pressure";
        DerPressureByDensity pd "derivative of pressure w.r.t. density";
      end NewtonDerivatives_pT;
      record ExtraDerivatives "additional thermodynamic derivatives"
        extends Modelica.Icons.Record;
        IsentropicExponent kappa "isentropic expansion coefficient";
        IsenthalpicExponent theta "isenthalpic exponent";
        IsobaricVolumeExpansionCoefficient alpha "isobaric volume expansion coefficient";
        IsochoricPressureCoefficient beta "isochoric pressure coefficient";
        IsothermalCompressibility gamma "isothermal compressibility";
        JouleThomsonCoefficient mu "Joule-Thomson coefficient";
      end ExtraDerivatives;
      record BridgmansTables "Calculates all entries in Bridgmans tables if first seven variables given"
        extends Modelica.Icons.Record;
        SI.SpecificVolume v "specific volume";
        SI.Pressure p "pressure";
        SI.Temperature T "temperature";
        SI.SpecificEntropy s "specific entropy";
        SI.SpecificHeatCapacity cp "heat capacity at constant pressure";
        IsobaricVolumeExpansionCoefficient alpha "isobaric volume expansion coefficient";
        IsothermalCompressibility gamma "isothermal compressibility";
        Real dTp = 1 "coefficient in Bridgmans table, see info for usage";
        Real dpT =  -dTp "coefficient in Bridgmans table, see info for usage";
        Real dvp = alpha * v "coefficient in Bridgmans table, see info for usage";
        Real dpv =  -dvp "coefficient in Bridgmans table, see info for usage";
        Real dsp = cp / T "coefficient in Bridgmans table, see info for usage";
        Real dps =  -dsp "coefficient in Bridgmans table, see info for usage";
        Real dup = cp - alpha * p * v "coefficient in Bridgmans table, see info for usage";
        Real dpu =  -dup "coefficient in Bridgmans table, see info for usage";
        Real dhp = cp "coefficient in Bridgmans table, see info for usage";
        Real dph =  -dhp "coefficient in Bridgmans table, see info for usage";
        Real dfp =  -s - alpha * p * v "coefficient in Bridgmans table, see info for usage";
        Real dpf =  -dfp "coefficient in Bridgmans table, see info for usage";
        Real dgp =  -s "coefficient in Bridgmans table, see info for usage";
        Real dpg =  -dgp "coefficient in Bridgmans table, see info for usage";
        Real dvT = gamma * v "coefficient in Bridgmans table, see info for usage";
        Real dTv =  -dvT "coefficient in Bridgmans table, see info for usage";
        Real dsT = alpha * v "coefficient in Bridgmans table, see info for usage";
        Real dTs =  -dsT "coefficient in Bridgmans table, see info for usage";
        Real duT = alpha * T * v - gamma * p * v "coefficient in Bridgmans table, see info for usage";
        Real dTu =  -duT "coefficient in Bridgmans table, see info for usage";
        Real dhT =  -v + alpha * T * v "coefficient in Bridgmans table, see info for usage";
        Real dTh =  -dhT "coefficient in Bridgmans table, see info for usage";
        Real dfT =  -gamma * p * v "coefficient in Bridgmans table, see info for usage";
        Real dTf =  -dfT "coefficient in Bridgmans table, see info for usage";
        Real dgT =  -v "coefficient in Bridgmans table, see info for usage";
        Real dTg =  -dgT "coefficient in Bridgmans table, see info for usage";
        Real dsv = alpha * alpha * v * v - (gamma * v * cp) / T "coefficient in Bridgmans table, see info for usage";
        Real dvs =  -dsv "coefficient in Bridgmans table, see info for usage";
        Real duv = T * alpha * alpha * v * v - gamma * v * cp "coefficient in Bridgmans table, see info for usage";
        Real dvu =  -duv "coefficient in Bridgmans table, see info for usage";
        Real dhv = T * alpha * alpha * v * v - alpha * v * v - gamma * v * cp "coefficient in Bridgmans table, see info for usage";
        Real dvh =  -dhv "coefficient in Bridgmans table, see info for usage";
        Real dfv = gamma * v * s "coefficient in Bridgmans table, see info for usage";
        Real dvf =  -dfv "coefficient in Bridgmans table, see info for usage";
        Real dgv = gamma * v * s - alpha * v * v "coefficient in Bridgmans table, see info for usage";
        Real dvg =  -dgv "coefficient in Bridgmans table, see info for usage";
        Real dus = dsv * p "coefficient in Bridgmans table, see info for usage";
        Real dsu =  -dus "coefficient in Bridgmans table, see info for usage";
        Real dhs =  -(v * cp) / T "coefficient in Bridgmans table, see info for usage";
        Real dsh =  -dhs "coefficient in Bridgmans table, see info for usage";
        Real dfs = alpha * v * s + dus "coefficient in Bridgmans table, see info for usage";
        Real dsf =  -dfs "coefficient in Bridgmans table, see info for usage";
        Real dgs = alpha * v * s - (v * cp) / T "coefficient in Bridgmans table, see info for usage";
        Real dsg =  -dgs "coefficient in Bridgmans table, see info for usage";
        Real dhu = p * alpha * v * v + gamma * v * cp * p - v * cp - p * T * alpha * alpha * v * v "coefficient in Bridgmans table, see info for usage";
        Real duh =  -dhu "coefficient in Bridgmans table, see info for usage";
        Real dfu = s * T * alpha * v - gamma * v * cp * p - gamma * v * s * p + p * T * alpha * alpha * v * v "coefficient in Bridgmans table, see info for usage";
        Real duf =  -dfu "coefficient in Bridgmans table, see info for usage";
        Real dgu = alpha * v * v * p + alpha * v * s * T - v * cp - gamma * v * s * p "coefficient in Bridgmans table, see info for usage";
        Real dug =  -dgu "coefficient in Bridgmans table, see info for usage";
        Real dfh = (s - v * alpha * p) * (v - v * alpha * T) - gamma * v * cp * p "coefficient in Bridgmans table, see info for usage";
        Real dhf =  -dfh "coefficient in Bridgmans table, see info for usage";
        Real dgh = alpha * v * s * T - v * (s + cp) "coefficient in Bridgmans table, see info for usage";
        Real dhg =  -dgh "coefficient in Bridgmans table, see info for usage";
        Real dfg = gamma * v * s * p - v * s - alpha * v * v * p "coefficient in Bridgmans table, see info for usage";
        Real dgf =  -dfg "coefficient in Bridgmans table, see info for usage";
        annotation(Documentation(info = "<HTML>
<p>
Important: the phase equilibrium conditions are not yet considered.
this means that bridgemans tables do not yet work in the two phase region.
Some derivatives are 0 or infinity anyways.
Idea: don't use the values in Bridgmans table directly, all
derivatives are calculated as the quotient of two entries in the
table. The last letter indicates which variable is held constant in
taking the derivative. The second letters are the two variables
involved in the derivative and the first letter is alwys a d to remind
of differentiation.
</p>

<pre>
Example 1: Get the derivative of specific entropy s wrt Temperature at
constant specific volume (btw identical to constant density)
constant volume  --> last letter v
Temperature      --> second letter T
Specific entropy --> second letter s
--> the needed value is dsv/dTv
Known variables:
Temperature T
pressure p
specific volume v
specific inner energy u
specific enthalpy h
specific entropy s
specific helmholtz energy f
specific gibbs enthalpy g
Not included but useful:
density d
In order to convert derivatives involving density use the following
rules:
at constant density == at constant specific volume
ddx/dyx = -d*d*dvx/dyx with y,x any of T,p,u,h,s,f,g
dyx/ddx = -1/(d*d)dyx/dvx with y,x any of T,p,u,h,s,f,g
Usage example assuming water as the medium:
model BridgmansTablesForWater
extends ThermoFluid.BaseClasses.MediumModels.Water.WaterSteamMedium_ph;
Real derOfsByTAtConstantv \"derivative of sp. entropy by temperature at constant sp. volume\"
ThermoFluid.BaseClasses.MediumModels.Common.ExtraDerivatives dpro;
ThermoFluid.BaseClasses.MediumModels.Common.BridgmansTables bt;
equation
dpro = ThermoFluid.BaseClasses.MediumModels.SteamIF97.extraDerivs_pT(p[1],T[1]);
bt.p = p[1];
bt.T = T[1];
bt.v = 1/pro[1].d;
bt.s = pro[1].s;
bt.cp = pro[1].cp;
bt.alpha = dpro.alpha;
bt.gamma = dpro.gamma;
derOfsByTAtConstantv =  bt.dsv/bt.dTv;
                ...
end BridgmansTablesForWater;
                </pre>

                </HTML>
                "));
      end BridgmansTables;
      function gibbsToBridgmansTables "calculates base coefficients for bridgemans tables from gibbs enthalpy"
        extends Modelica.Icons.Function;
        input GibbsDerivs g "dimensionless derivatives of Gibbs function";
        output SI.SpecificVolume v "specific volume";
        output SI.Pressure p = g.p "pressure";
        output SI.Temperature T = g.T "temperature";
        output SI.SpecificEntropy s "specific entropy";
        output SI.SpecificHeatCapacity cp "heat capacity at constant pressure";
        output IsobaricVolumeExpansionCoefficient alpha "isobaric volume expansion coefficient";
        output IsothermalCompressibility gamma "isothermal compressibility";
      protected 
        Real vt(unit = "m3/(kg.K)") "derivative of specific volume w.r.t. temperature";
        Real vp(unit = "m4.kg-2.s2") "derivative of specific volume w.r.t. pressure";
      algorithm 
        vt:=g.R / g.p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
        vp:=(g.R * g.T) / (g.p * g.p) * g.pi * g.pi * g.gpipi;
        v:=(g.R * g.T * g.pi * g.gpi) / g.p;
        s:=g.R * (g.tau * g.gtau - g.g);
        cp:= -g.R * g.tau * g.tau * g.gtautau;
        alpha:=vt / v;
        gamma:= -vp / v;
      end gibbsToBridgmansTables;
      function helmholtzToBridgmansTables "calculates base coefficients for Bridgmans tables from helmholtz energy"
        extends Modelica.Icons.Function;
        input HelmholtzDerivs f "dimensionless derivatives of Helmholtz function";
        output SI.SpecificVolume v = 1 / f.d "specific volume";
        output SI.Pressure p "pressure";
        output SI.Temperature T = f.T "temperature";
        output SI.SpecificEntropy s "specific entropy";
        output SI.SpecificHeatCapacity cp "heat capacity at constant pressure";
        output IsobaricVolumeExpansionCoefficient alpha "isobaric volume expansion coefficient";
        output IsothermalCompressibility gamma "isothermal compressibility";
      protected 
        DerPressureByTemperature pt "derivative of pressure w.r.t. temperature";
        DerPressureBySpecificVolume pv "derivative of pressure w.r.t. specific volume ";
        SI.SpecificHeatCapacity cv "isochoric specific heat capacity";
      algorithm 
        p:=f.R * f.d * f.T * f.delta * f.fdelta;
        pv:= -f.d * f.d * f.R * f.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
        pt:=f.R * f.d * f.delta * (f.fdelta - f.tau * f.fdeltatau);
        s:=f.R * (f.tau * f.ftau - f.f);
        alpha:= -(f.d * pt) / pv;
        gamma:= -f.d / pv;
        cp:=f.R * ( -f.tau * f.tau * f.ftautau + (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) ^ 2 / (2 * f.delta * f.fdelta + f.delta * f.delta * f.fdeltadelta));
      end helmholtzToBridgmansTables;
      function gibbsToBoundaryProps "calulate phase boundary property record from dimensionless Gibbs function"
        extends Modelica.Icons.Function;
        input GibbsDerivs g "dimensionless derivatives of Gibbs function";
        output PhaseBoundaryProperties sat "phase boundary properties";
      protected 
        Real vt "derivative of specific volume w.r.t. temperature";
        Real vp "derivative of specific volume w.r.t. pressure";
      algorithm 
        sat.d:=g.p / (g.R * g.T * g.pi * g.gpi);
        sat.h:=g.R * g.T * g.tau * g.gtau;
        sat.u:=g.T * g.R * (g.tau * g.gtau - g.pi * g.gpi);
        sat.s:=g.R * (g.tau * g.gtau - g.g);
        sat.cp:= -g.R * g.tau * g.tau * g.gtautau;
        sat.cv:=g.R * ( -g.tau * g.tau * g.gtautau + ((g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi)) / g.gpipi);
        vt:=g.R / g.p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
        vp:=(g.R * g.T) / (g.p * g.p) * g.pi * g.pi * g.gpipi;
        sat.pt:= -(g.p / g.T * (g.gpi - g.tau * g.gtaupi)) / (g.gpipi * g.pi);
        sat.pd:= -(g.R * g.T * g.gpi * g.gpi) / g.gpipi;
      end gibbsToBoundaryProps;
      function helmholtzToBoundaryProps "calulate phase boundary property record from dimensionless Helmholtz function"
        extends Modelica.Icons.Function;
        input HelmholtzDerivs f "dimensionless derivatives of Helmholtz function";
        output PhaseBoundaryProperties sat "phase boundary property record";
      protected 
        SI.Pressure p "pressure";
      algorithm 
        p:=f.R * f.d * f.T * f.delta * f.fdelta;
        sat.d:=f.d;
        sat.h:=f.R * f.T * (f.tau * f.ftau + f.delta * f.fdelta);
        sat.s:=f.R * (f.tau * f.ftau - f.f);
        sat.u:=f.R * f.T * f.tau * f.ftau;
        sat.cp:=f.R * ( -f.tau * f.tau * f.ftautau + (f.delta * f.fdelta - f.delta * f.tau * f.fdeltatau) ^ 2 / (2 * f.delta * f.fdelta + f.delta * f.delta * f.fdeltadelta));
        sat.cv:=f.R * ( -f.tau * f.tau * f.ftautau);
        sat.pt:=f.R * f.d * f.delta * (f.fdelta - f.tau * f.fdeltatau);
        sat.pd:=f.R * f.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
      end helmholtzToBoundaryProps;
      function cv2Phase "compute isochoric specific heat capacity inside the two-phase region"
        extends Modelica.Icons.Function;
        input PhaseBoundaryProperties liq "properties on the boiling curve";
        input PhaseBoundaryProperties vap "properties on the condensation curve";
        input SI.MassFraction x "vapour mass fraction";
        input SI.Temperature T "temperature";
        input SI.Pressure p "preoperties";
        output SI.SpecificHeatCapacity cv "isochoric specific heat capacity";
      protected 
        Real dpT "derivative of pressure w.r.t. temperature";
        Real dxv "derivative of vapour mass fraction w.r.t. specific volume";
        Real dvTl "derivative of liquid specific volume w.r.t. temperature";
        Real dvTv "derivative of vapour specific volume w.r.t. temperature";
        Real duTl "derivative of liquid specific inner energy w.r.t. temperature";
        Real duTv "derivative of vapour specific inner energy w.r.t. temperature";
        Real dxt "derivative of vapour mass fraction w.r.t. temperature";
      algorithm 
        dxv:=if liq.d <> vap.d then (liq.d * vap.d) / (liq.d - vap.d) else 0.0;
        dpT:=(vap.s - liq.s) * dxv;
        dvTl:=(liq.pt - dpT) / liq.pd / liq.d / liq.d;
        dvTv:=(vap.pt - dpT) / vap.pd / vap.d / vap.d;
        dxt:= -dxv * (dvTl + x * (dvTv - dvTl));
        duTl:=liq.cv + (T * liq.pt - p) * dvTl;
        duTv:=vap.cv + (T * vap.pt - p) * dvTv;
        cv:=duTl + x * (duTv - duTl) + dxt * (vap.u - liq.u);
      end cv2Phase;
      function cvdpT2Phase "compute isochoric specific heat capacity inside the two-phase region and derivative of pressure w.r.t. temperature"
        extends Modelica.Icons.Function;
        input PhaseBoundaryProperties liq "properties on the boiling curve";
        input PhaseBoundaryProperties vap "properties on the condensation curve";
        input SI.MassFraction x "vapour mass fraction";
        input SI.Temperature T "temperature";
        input SI.Pressure p "preoperties";
        output SI.SpecificHeatCapacity cv "isochoric specific heat capacity";
        output Real dpT "derivative of pressure w.r.t. temperature";
      protected 
        Real dxv "derivative of vapour mass fraction w.r.t. specific volume";
        Real dvTl "derivative of liquid specific volume w.r.t. temperature";
        Real dvTv "derivative of vapour specific volume w.r.t. temperature";
        Real duTl "derivative of liquid specific inner energy w.r.t. temperature";
        Real duTv "derivative of vapour specific inner energy w.r.t. temperature";
        Real dxt "derivative of vapour mass fraction w.r.t. temperature";
      algorithm 
        dxv:=if liq.d <> vap.d then (liq.d * vap.d) / (liq.d - vap.d) else 0.0;
        dpT:=(vap.s - liq.s) * dxv;
        dvTl:=(liq.pt - dpT) / liq.pd / liq.d / liq.d;
        dvTv:=(vap.pt - dpT) / vap.pd / vap.d / vap.d;
        dxt:= -dxv * (dvTl + x * (dvTv - dvTl));
        duTl:=liq.cv + (T * liq.pt - p) * dvTl;
        duTv:=vap.cv + (T * vap.pt - p) * dvTv;
        cv:=duTl + x * (duTv - duTl) + dxt * (vap.u - liq.u);
      end cvdpT2Phase;
      function gibbsToExtraDerivs "compute additional thermodynamic derivatives from dimensionless Gibbs function"
        extends Modelica.Icons.Function;
        input GibbsDerivs g "dimensionless derivatives of Gibbs function";
        output ExtraDerivatives dpro "additional property derivatives";
      protected 
        Real vt "derivative of specific volume w.r.t. temperature";
        Real vp "derivative of specific volume w.r.t. pressure";
        SI.Density d "density";
        SI.SpecificVolume v "specific volume";
        SI.SpecificHeatCapacity cv "isochoric heat capacity";
        SI.SpecificHeatCapacity cp "isobaric heat capacity";
      algorithm 
        d:=g.p / (g.R * g.T * g.pi * g.gpi);
        v:=1 / d;
        vt:=g.R / g.p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
        vp:=(g.R * g.T) / (g.p * g.p) * g.pi * g.pi * g.gpipi;
        cp:= -g.R * g.tau * g.tau * g.gtautau;
        cv:=g.R * ( -g.tau * g.tau * g.gtautau + ((g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi)) / g.gpipi);
        dpro.kappa:= -(1 / (d * g.p) * cp) / (vp * cp + vt * vt * g.T);
        dpro.theta:=cp / (d * g.p * ( -vp * cp + vt * v - g.T * vt * vt));
        dpro.alpha:=d * vt;
        dpro.beta:= -vt / (g.p * vp);
        dpro.gamma:= -d * vp;
        dpro.mu:= -(v - g.T * vt) / cp;
      end gibbsToExtraDerivs;
      function helmholtzToExtraDerivs "compute additional thermodynamic derivatives from dimensionless Helmholtz function"
        extends Modelica.Icons.Function;
        input HelmholtzDerivs f "dimensionless derivatives of Helmholtz function";
        output ExtraDerivatives dpro "additional property derivatives";
      protected 
        SI.Pressure p "pressure";
        SI.SpecificVolume v "specific volume";
        DerPressureByTemperature pt "derivative of pressure w.r.t. temperature";
        DerPressureBySpecificVolume pv "derivative of pressure w.r.t. specific volume";
        SI.SpecificHeatCapacity cv "isochoric specific heat capacity";
      algorithm 
        v:=1 / f.d;
        p:=f.R * f.d * f.T * f.delta * f.fdelta;
        pv:= -f.d * f.d * f.R * f.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
        pt:=f.R * f.d * f.delta * (f.fdelta - f.tau * f.fdeltatau);
        cv:=f.R * ( -f.tau * f.tau * f.ftautau);
        dpro.kappa:=1 / (f.d * p) * ( -pv * cv + pt * pt * f.T) / cv;
        dpro.theta:= -1 / (f.d * p) * ( -pv * cv + f.T * pt * pt) / (cv + pt * v);
        dpro.alpha:= -(f.d * pt) / pv;
        dpro.beta:=pt / p;
        dpro.gamma:= -f.d / pv;
        dpro.mu:=(v * pv + f.T * pt) / (pt * pt * f.T - pv * cv);
      end helmholtzToExtraDerivs;
      function Helmholtz_ph "function to calculate analytic derivatives for computing d and t given p and h"
        extends Modelica.Icons.Function;
        input HelmholtzDerivs f "dimensionless derivatives of Helmholtz function";
        output NewtonDerivatives_ph nderivs "derivatives for Newton iteration to calculate d and t from p and h";
      protected 
        SI.SpecificHeatCapacity cv "isochoric heat capacity";
      algorithm 
        cv:= -f.R * f.tau * f.tau * f.ftautau;
        nderivs.p:=f.d * f.R * f.T * f.delta * f.fdelta;
        nderivs.h:=f.R * f.T * (f.tau * f.ftau + f.delta * f.fdelta);
        nderivs.pd:=f.R * f.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
        nderivs.pt:=f.R * f.d * f.delta * (f.fdelta - f.tau * f.fdeltatau);
        nderivs.ht:=cv + nderivs.pt / f.d;
        nderivs.hd:=(nderivs.pd - (f.T * nderivs.pt) / f.d) / f.d;
      end Helmholtz_ph;
      function Helmholtz_pT "function to calculate analytic derivatives for computing d and t given p and t"
        extends Modelica.Icons.Function;
        input HelmholtzDerivs f "dimensionless derivatives of Helmholtz function";
        output NewtonDerivatives_pT nderivs "derivatives for Newton iteration to compute d and t from p and t";
      algorithm 
        nderivs.p:=f.d * f.R * f.T * f.delta * f.fdelta;
        nderivs.pd:=f.R * f.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
      end Helmholtz_pT;
      function Helmholtz_ps "function to calculate analytic derivatives for computing d and t given p and s"
        extends Modelica.Icons.Function;
        input HelmholtzDerivs f "dimensionless derivatives of Helmholtz function";
        output NewtonDerivatives_ps nderivs "derivatives for Newton iteration to compute d and t from p and s";
      protected 
        SI.SpecificHeatCapacity cv "isochoric heat capacity";
      algorithm 
        cv:= -f.R * f.tau * f.tau * f.ftautau;
        nderivs.p:=f.d * f.R * f.T * f.delta * f.fdelta;
        nderivs.s:=f.R * (f.tau * f.ftau - f.f);
        nderivs.pd:=f.R * f.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
        nderivs.pt:=f.R * f.d * f.delta * (f.fdelta - f.tau * f.fdeltatau);
        nderivs.st:=cv / f.T;
        nderivs.sd:= -nderivs.pt / (f.d * f.d);
      end Helmholtz_ps;
      function smoothStep "Approximation of a general step, such that the characteristic is continuous and differentiable"
        extends Modelica.Icons.Function;
        input Real x "Abszissa value";
        input Real y1 "Ordinate value for x > 0";
        input Real y2 "Ordinate value for x < 0";
        input Real x_small(min = 0) = 1e-05 "Approximation of step for -x_small <= x <= x_small; x_small > 0 required";
        output Real y "Ordinate value to approximate y = if x > 0 then y1 else y2";
        annotation(Documentation(revisions = "<html>
<ul>
<li><i>April 29, 2008</i>
    by <a href=\"mailto:Martin.Otter@DLR.de\">Martin Otter</a>:<br>
    Designed and implemented.</li>
<li><i>August 12, 2008</i>
    by <a href=\"mailto:Michael.Sielemann@dlr.de\">Michael Sielemann</a>:<br>
    Minor modification to cover the limit case <code>x_small -> 0</code> without division by zero.</li>
</ul>
</html>", info = "<html>
<p>
This function is used to approximate the equation
</p>
<pre>
    y = <b>if</b> x &gt; 0 <b>then</b> y1 <b>else</b> y2;
</pre>

<p>
by a smooth characteristic, so that the expression is continuous and differentiable:
</p>

<pre>
   y = <b>smooth</b>(1, <b>if</b> x &gt;  x_small <b>then</b> y1 <b>else</b>
                 <b>if</b> x &lt; -x_small <b>then</b> y2 <b>else</b> f(y1, y2));
</pre>

<p>
In the region -x_small &lt; x &lt; x_small a 2nd order polynomial is used
for a smooth transition from y1 to y2.
</p>

<p>
If <b>mass fractions</b> X[:] are approximated with this function then this can be performed
for all <b>nX</b> mass fractions, instead of applying it for nX-1 mass fractions and computing
the last one by the mass fraction constraint sum(X)=1. The reason is that the approximating function has the
property that sum(X) = 1, provided sum(X_a) = sum(X_b) = 1
(and y1=X_a[i], y2=X_b[i]).
This can be shown by evaluating the approximating function in the abs(x) &lt; x_small
region (otherwise X is either X_a or X_b):
</p>

<pre>
    X[1]  = smoothStep(x, X_a[1] , X_b[1] , x_small);
    X[2]  = smoothStep(x, X_a[2] , X_b[2] , x_small);
       ...
    X[nX] = smoothStep(x, X_a[nX], X_b[nX], x_small);
</pre>

<p>
or
</p>

<pre>
    X[1]  = c*(X_a[1]  - X_b[1])  + (X_a[1]  + X_b[1])/2
    X[2]  = c*(X_a[2]  - X_b[2])  + (X_a[2]  + X_b[2])/2;
       ...
    X[nX] = c*(X_a[nX] - X_b[nX]) + (X_a[nX] + X_b[nX])/2;
    c     = (x/x_small)*((x/x_small)^2 - 3)/4
</pre>

<p>
Summing all mass fractions together results in
</p>

<pre>
    sum(X) = c*(sum(X_a) - sum(X_b)) + (sum(X_a) + sum(X_b))/2
           = c*(1 - 1) + (1 + 1)/2
           = 1
</pre>
</html>"));
      algorithm 
        y:=smooth(1, if x > x_small then y1 else if x <  -x_small then y2 else if abs(x_small) > 0 then (x / x_small * ((x / x_small) ^ 2 - 3) * (y2 - y1)) / 4 + (y1 + y2) / 2 else (y1 + y2) / 2);
      end smoothStep;
      package OneNonLinearEquation "Determine solution of a non-linear algebraic equation in one unknown without derivatives in a reliable and efficient way"
        extends Modelica.Icons.Library;
        annotation(Documentation(info = "<html>
<p>
This function should currently only be used in Modelica.Media,
since it might be replaced in the future by another strategy,
where the tool is responsible for the solution of the non-linear
equation.
</p>

<p>
This library determines the solution of one non-linear algebraic equation \"y=f(x)\"
in one unknown \"x\" in a reliable way. As input, the desired value y of the
non-linear function has to be given, as well as an interval x_min, x_max that
contains the solution, i.e., \"f(x_min) - y\" and \"f(x_max) - y\" must
have a different sign. If possible, a smaller interval is computed by
inverse quadratic interpolation (interpolating with a quadratic polynomial
through the last 3 points and computing the zero). If this fails,
bisection is used, which always reduces the interval by a factor of 2.
The inverse quadratic interpolation method has superlinear convergence.
This is roughly the same convergence rate as a globally convergent Newton
method, but without the need to compute derivatives of the non-linear
function. The solver function is a direct mapping of the Algol 60 procedure
\"zero\" to Modelica, from:
</p>

<dl>
<dt> Brent R.P.:</dt>
<dd> <b>Algorithms for Minimization without derivatives</b>.
     Prentice Hall, 1973, pp. 58-59.</dd>
</dl>

<p>
Due to current limitations of the
Modelica language (not possible to pass a function reference to a function),
the construction to use this solver on a user-defined function is a bit
complicated (this method is from Hans Olsson, Dynasim AB). A user has to
provide a package in the following way:
</p>

<pre>
  <b>package</b> MyNonLinearSolver
    <b>extends</b> OneNonLinearEquation;

    <b>redeclare record extends</b> Data
      // Define data to be passed to user function
      ...
    <b>end</b> Data;

    <b>redeclare function extends</b> f_nonlinear
    <b>algorithm</b>
       // Compute the non-linear equation: y = f(x, Data)
    <b>end</b> f_nonlinear;

    // Dummy definition that has to be present for current Dymola
    <b>redeclare function extends</b> solve
    <b>end</b> solve;
  <b>end</b> MyNonLinearSolver;

  x_zero = MyNonLinearSolver.solve(y_zero, x_min, x_max, data=data);
</pre>
</html>"));
        replaceable record f_nonlinear_Data "Data specific for function f_nonlinear"
          extends Modelica.Icons.Record;
        end f_nonlinear_Data;
        replaceable partial function f_nonlinear "Nonlinear algebraic equation in one unknown: y = f_nonlinear(x,p,X)"
          extends Modelica.Icons.Function;
          input Real x "Independent variable of function";
          input Real p = 0.0 "disregaded variables (here always used for pressure)";
          input Real[:] X = fill(0, 0) "disregaded variables (her always used for composition)";
          input f_nonlinear_Data f_nonlinear_data "Additional data for the function";
          output Real y "= f_nonlinear(x)";
        end f_nonlinear;
        replaceable function solve "Solve f_nonlinear(x_zero)=y_zero; f_nonlinear(x_min) - y_zero and f_nonlinear(x_max)-y_zero must have different sign"
          import Modelica.Utilities.Streams.error;
          extends Modelica.Icons.Function;
          input Real y_zero "Determine x_zero, such that f_nonlinear(x_zero) = y_zero";
          input Real x_min "Minimum value of x";
          input Real x_max "Maximum value of x";
          input Real pressure = 0.0 "disregaded variables (here always used for pressure)";
          input Real[:] X = fill(0, 0) "disregaded variables (here always used for composition)";
          input f_nonlinear_Data f_nonlinear_data "Additional data for function f_nonlinear";
          input Real x_tol = 100 * Modelica.Constants.eps "Relative tolerance of the result";
          output Real x_zero "f_nonlinear(x_zero) = y_zero";
        protected 
          constant Real eps = Modelica.Constants.eps "machine epsilon";
          Real a = x_min "Current best minimum interval value";
          Real b = x_max "Current best maximum interval value";
          Real c "Intermediate point a <= c <= b";
          Real d;
          Real e "b - a";
          Real m;
          Real s;
          Real p;
          Real q;
          Real r;
          Real tol;
          Real fa "= f_nonlinear(a) - y_zero";
          Real fb "= f_nonlinear(b) - y_zero";
          Real fc;
          Boolean found = false;
        algorithm 
          fa:=f_nonlinear(x_min, pressure, X, f_nonlinear_data) - y_zero;
          fb:=f_nonlinear(x_max, pressure, X, f_nonlinear_data) - y_zero;
          fc:=fb;
          if fa > 0.0 and fb > 0.0 or fa < 0.0 and fb < 0.0 then 
                    error("The arguments x_min and x_max to OneNonLinearEquation.solve(..)\n" + "do not bracket the root of the single non-linear equation:\n" + "  x_min  = " + String(x_min) + "\n" + "  x_max  = " + String(x_max) + "\n" + "  y_zero = " + String(y_zero) + "\n" + "  fa = f(x_min) - y_zero = " + String(fa) + "\n" + "  fb = f(x_max) - y_zero = " + String(fb) + "\n" + "fa and fb must have opposite sign which is not the case");

          else 
          end if;
          c:=a;
          fc:=fa;
          e:=b - a;
          d:=e;
          while (not found) loop
                      if abs(fc) < abs(fb) then 
                        a:=b;
            b:=c;
            c:=a;
            fa:=fb;
            fb:=fc;
            fc:=fa;

            else 
            end if;
            tol:=2 * eps * abs(b) + x_tol;
            m:=(c - b) / 2;
            if abs(m) <= tol or fb == 0.0 then 
                        found:=true;
            x_zero:=b;

            else             if abs(e) < tol or abs(fa) <= abs(fb) then 
                        e:=m;
            d:=e;

            else             s:=fb / fa;
            if a == c then 
                        p:=2 * m * s;
            q:=1 - s;

            else             q:=fa / fc;
            r:=fb / fc;
            p:=s * (2 * m * q * (q - r) - (b - a) * (r - 1));
            q:=(q - 1) * (r - 1) * (s - 1);

            end if;
            if p > 0 then 
                        q:= -q;

            else             p:= -p;

            end if;
            s:=e;
            e:=d;
            if 2 * p < 3 * m * q - abs(tol * q) and p < abs(0.5 * s * q) then 
                        d:=p / q;

            else             e:=m;
            d:=e;

            end if;

            end if;
            a:=b;
            fa:=fb;
            b:=b + (if abs(d) > tol then d else if m > 0 then tol else  -tol);
            fb:=f_nonlinear(b, pressure, X, f_nonlinear_data) - y_zero;
            if fb > 0 and fc > 0 or fb < 0 and fc < 0 then 
                        c:=a;
            fc:=fa;
            e:=b - a;
            d:=e;

            else 
            end if;

            end if;

          end while;
        end solve;
      end OneNonLinearEquation;
    end Common;
    package Water "Medium models for water"
      annotation(Documentation(info = "<html>
<p>This package contains different medium models for water:</p>
<ul>
<li><b>ConstantPropertyLiquidWater</b><br>
    Simple liquid water medium (incompressible, constant data).</li>
<li><b>IdealSteam</b><br>
    Steam water medium as ideal gas from Media.IdealGases.SingleGases.H2O</li>
<li><b>WaterIF97 derived models</b><br>
    High precision water model according to the IAPWS/IF97 standard
    (liquid, steam, two phase region). Models with different independent
    variables are provided as well as models valid only
    for particular regions. The <b>WaterIF97_ph</b> model is valid
    in all regions and is the recommended one to use.</li>
</ul>
<h4>Overview of WaterIF97 derived water models</h4>
<p>
The WaterIF97 models calculate medium properties
for water in the <b>liquid</b>, <b>gas</b> and <b>two phase</b> regions
according to the IAPWS/IF97 standard, i.e., the accepted industrial standard
and best compromise between accuracy and computation time.
It has been part of the ThermoFluid Modelica library and been extended,
reorganized and documented to become part of the Modelica Standard library.</p>
<p>An important feature that distinguishes this implementation of the IF97 steam property standard
is that this implementation has been explicitly designed to work well in dynamic simulations. Computational
performance has been of high importance. This means that there often exist several ways to get the same result
from different functions if one of the functions is called often but can be optimized for that purpose.
</p>
<p>Three variable pairs can be the independent variables of the model:
</p>
<ol>
<li>Pressure <b>p</b> and specific enthalpy <b>h</b> are
    the most natural choice for general applications.
    This is the recommended choice for most general purpose
    applications, in particular for power plants.</li>
<li>Pressure <b>p</b> and temperature <b>T</b> are the most natural
    choice for applications where water is always in the same phase,
    both for liquid water and steam.</li>
<li>Density <b>d</b> and temperature <b>T</b> are explicit
    variables of the Helmholtz function in the near-critical
    region and can be the best choice for applications with
    super-critical or near-critial states.</li>
</ol>
<p>
The following quantities are always computed in Medium.Baseproperties:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Variable</b></td>
      <td valign=\"top\"><b>Unit</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">T</td>
      <td valign=\"top\">K</td>
      <td valign=\"top\">temperature</td></tr>
  <tr><td valign=\"top\">u</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific internal energy</b></td></tr>
  <tr><td valign=\"top\">d</td>
      <td valign=\"top\">kg/m^3</td>
      <td valign=\"top\">density</td></tr>
  <tr><td valign=\"top\">p</td>
      <td valign=\"top\">Pa</td>
      <td valign=\"top\">pressure</td></tr>
  <tr><td valign=\"top\">h</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific enthalpy</b></td></tr>
</table>
<p>
In some cases additional medium properties are needed.
A component that needs these optional properties has to call
one of the following functions:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Function call</b></td>
      <td valign=\"top\"><b>Unit</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">Medium.dynamicViscosity(medium.state)</b></td>
      <td valign=\"top\">Pa.s</td>
      <td valign=\"top\">dynamic viscosity</td></tr>
  <tr><td valign=\"top\">Medium.thermalConductivity(medium.state)</td>
      <td valign=\"top\">W/(m.K)</td>
      <td valign=\"top\">thermal conductivity</td></tr>
  <tr><td valign=\"top\">Medium.prandtlNumber(medium.state)</td>
      <td valign=\"top\">1</td>
      <td valign=\"top\">Prandtl number</td></tr>
  <tr><td valign=\"top\">Medium.specificEntropy(medium.state)</td>
      <td valign=\"top\">J/(kg.K)</td>
      <td valign=\"top\">specific entropy</td></tr>
  <tr><td valign=\"top\">Medium.heatCapacity_cp(medium.state)</td>
      <td valign=\"top\">J/(kg.K)</td>
      <td valign=\"top\">specific heat capacity at constant pressure</td></tr>
  <tr><td valign=\"top\">Medium.heatCapacity_cv(medium.state)</td>
      <td valign=\"top\">J/(kg.K)</td>
      <td valign=\"top\">specific heat capacity at constant density</td></tr>
  <tr><td valign=\"top\">Medium.isentropicExponent(medium.state)</td>
      <td valign=\"top\">1</td>
      <td valign=\"top\">isentropic exponent</td></tr>
  <tr><td valign=\"top\">Medium.isentropicEnthalpy(pressure, medium.state)</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">isentropic enthalpy</td></tr>
  <tr><td valign=\"top\">Medium.velocityOfSound(medium.state)</td>
      <td valign=\"top\">m/s</td>
      <td valign=\"top\">velocity of sound</td></tr>
  <tr><td valign=\"top\">Medium.isobaricExpansionCoefficient(medium.state)</td>
      <td valign=\"top\">1/K</td>
      <td valign=\"top\">isobaric expansion coefficient</td></tr>
  <tr><td valign=\"top\">Medium.isothermalCompressibility(medium.state)</td>
      <td valign=\"top\">1/Pa</td>
      <td valign=\"top\">isothermal compressibility</td></tr>
  <tr><td valign=\"top\">Medium.density_derp_h(medium.state)</td>
      <td valign=\"top\">kg/(m3.Pa)</td>
      <td valign=\"top\">derivative of density by pressure at constant enthalpy</td></tr>
  <tr><td valign=\"top\">Medium.density_derh_p(medium.state)</td>
      <td valign=\"top\">kg2/(m3.J)</td>
      <td valign=\"top\">derivative of density by enthalpy at constant pressure</td></tr>
  <tr><td valign=\"top\">Medium.density_derp_T(medium.state)</td>
      <td valign=\"top\">kg/(m3.Pa)</td>
      <td valign=\"top\">derivative of density by pressure at constant temperature</td></tr>
  <tr><td valign=\"top\">Medium.density_derT_p(medium.state)</td>
      <td valign=\"top\">kg/(m3.K)</td>
      <td valign=\"top\">derivative of density by temperature at constant pressure</td></tr>
  <tr><td valign=\"top\">Medium.density_derX(medium.state)</td>
      <td valign=\"top\">kg/m3</td>
      <td valign=\"top\">derivative of density by mass fraction</td></tr>
  <tr><td valign=\"top\">Medium.molarMass(medium.state)</td>
      <td valign=\"top\">kg/mol</td>
      <td valign=\"top\">molar mass</td></tr>
</table>
<p>More details are given in
<a href=\"Modelica://Modelica.Media.UsersGuide.MediumUsage.OptionalProperties\">
Modelica.Media.UsersGuide.MediumUsage.OptionalProperties</a>.

Many additional optional functions are defined to compute properties of
saturated media, either liquid (bubble point) or vapour (dew point).
The argument to such functions is a SaturationProperties record, which can be
set starting from either the saturation pressure or the saturation temperature.
With reference to a model defining a pressure p, a temperature T, and a
SaturationProperties record sat, the following functions are provided:
</p>
<p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Function call</b></td>
      <td valign=\"top\"><b>Unit</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">Medium.saturationPressure(T)</b></td>
      <td valign=\"top\">Pa</td>
      <td valign=\"top\">Saturation pressure at temperature T</td></tr>
  <tr><td valign=\"top\">Medium.saturationTemperature(p)</b></td>
      <td valign=\"top\">K</td>
      <td valign=\"top\">Saturation temperature at pressure p</td></tr>
  <tr><td valign=\"top\">Medium.saturationTemperature_derp(p)</b></td>
      <td valign=\"top\">K/Pa</td>
      <td valign=\"top\">Derivative of saturation temperature with respect to pressure</td></tr>
  <tr><td valign=\"top\">Medium.bubbleEnthalpy(sat)</b></td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">Specific enthalpy at bubble point</td></tr>
  <tr><td valign=\"top\">Medium.dewEnthalpy(sat)</b></td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">Specific enthalpy at dew point</td></tr>
  <tr><td valign=\"top\">Medium.bubbleEntropy(sat)</b></td>
      <td valign=\"top\">J/(kg.K)</td>
      <td valign=\"top\">Specific entropy at bubble point</td></tr>
  <tr><td valign=\"top\">Medium.dewEntropy(sat)</b></td>
      <td valign=\"top\">J/(kg.K)</td>
      <td valign=\"top\">Specific entropy at dew point</td></tr>
  <tr><td valign=\"top\">Medium.bubbleDensity(sat)</b></td>
      <td valign=\"top\">kg/m3</td>
      <td valign=\"top\">Density at bubble point</td></tr>
  <tr><td valign=\"top\">Medium.dewDensity(sat)</b></td>
      <td valign=\"top\">kg/m3</td>
      <td valign=\"top\">Density at dew point</td></tr>
  <tr><td valign=\"top\">Medium.dBubbleDensity_dPressure(sat)</b></td>
      <td valign=\"top\">kg/(m3.Pa)</td>
      <td valign=\"top\">Derivative of density at bubble point with respect to pressure</td></tr>
  <tr><td valign=\"top\">Medium.dDewDensity_dPressure(sat)</b></td>
      <td valign=\"top\">kg/(m3.Pa)</td>
      <td valign=\"top\">Derivative of density at dew point with respect to pressure</td></tr>
  <tr><td valign=\"top\">Medium.dBubbleEnthalpy_dPressure(sat)</b></td>
      <td valign=\"top\">J/(kg.Pa)</td>
      <td valign=\"top\">Derivative of specific enthalpy at bubble point with respect to pressure</td></tr>
  <tr><td valign=\"top\">Medium.dDewEnthalpy_dPressure(sat)</b></td>
      <td valign=\"top\">J/(kg.Pa)</td>
      <td valign=\"top\">Derivative of specific enthalpy at dew point with respect to pressure</td></tr>
  <tr><td valign=\"top\">Medium.surfaceTension(sat)</b></td>
      <td valign=\"top\">N/m</td>
      <td valign=\"top\">Surface tension between liquid and vapour phase</td></tr>
</table>

<p>Details on usage and some examples are given in:
<a href=\"Modelica://Modelica.Media.UsersGuide.MediumUsage.TwoPhase\">
Modelica.Media.UsersGuide.MediumUsage.TwoPhase</a>.
</p>

<p>Many further properties can be computed. Using the well-known Bridgman's Tables,
all first partial derivatives of the standard thermodynamic variables can be computed easily.
</p>
<p>
The documentation of the IAPWS/IF97 steam properties can be freely
distributed with computer implementations and are included here
(in directory Modelica\\help\\Documentation\\IF97documentation):
<ul>
<li><a href=\"Documentation/IF97documentation/IF97.pdf\">IF97.pdf</a> The standards document for the main part of the IF97.</li>
<li><a href=\"Documentation/IF97documentation/Back3.pdf\">Back3.pdf</a> The backwards equations for region 3.</li>
<li><a href=\"Documentation/IF97documentation/crits.pdf\">crits.pdf</a> The critical point data.</li>
<li><a href=\"Documentation/IF97documentation/meltsub.pdf\">meltsub.pdf</a> The melting- and sublimation line formulation (not implemented)</li>
<li><a href=\"Documentation/IF97documentation/surf.pdf\">surf.pdf</a> The surface tension standard definition</li>
<li><a href=\"Documentation/IF97documentation/thcond.pdf\">thcond.pdf</a> The thermal conductivity standard definition</li>
<li><a href=\"Documentation/IF97documentation/visc.pdf\">visc.pdf</a> The viscosity standard definition</li>
</ul>
</html>"));
      extends Modelica.Icons.Library;
      constant Interfaces.PartialTwoPhaseMedium.FluidConstants[1] waterConstants(each chemicalFormula = "H2O", each structureFormula = "H2O", each casRegistryNumber = "7732-18-5", each iupacName = "oxidane", each molarMass = 0.018015268, each criticalTemperature = 647.096, each criticalPressure = 22064000.0, each criticalMolarVolume = 1 / 322.0 * 0.018015268, each normalBoilingPoint = 373.124, each meltingPoint = 273.15, each triplePointTemperature = 273.16, each triplePointPressure = 611.657, each acentricFactor = 0.344, each dipoleMoment = 1.8, each hasCriticalData = true);
      constant Interfaces.PartialMedium.FluidConstants[1] simpleWaterConstants(each chemicalFormula = "H2O", each structureFormula = "H2O", each casRegistryNumber = "7732-18-5", each iupacName = "oxidane", each molarMass = 0.018015268);
      package ConstantPropertyLiquidWater "Water: Simple liquid water medium (incompressible, constant data)"
        import Cv = Modelica.SIunits.Conversions;
        extends Interfaces.PartialSimpleMedium(mediumName = "SimpleLiquidWater", cp_const = 4184, cv_const = 4184, d_const = 995.586, eta_const = 0.001, lambda_const = 0.598, a_const = 1484, T_min = Cv.from_degC( -1), T_max = Cv.from_degC(130), T0 = 273.15, MM_const = 0.018015268, fluidConstants = simpleWaterConstants);
        annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -90,88},{90,18}}, lineColor = {0,0,0}, textString = "liquid"),Text(extent = {{ -90, -22},{90, -90}}, lineColor = {0,0,0}, textString = "water")}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>

</html>"));
      end ConstantPropertyLiquidWater;
      package IdealSteam "Water: Steam as ideal gas from NASA source"
        extends IdealGases.SingleGases.H2O(fluidConstants = waterConstants);
        annotation(Documentation(info = "<html>

</html>"));
      end IdealSteam;
      package StandardWater = WaterIF97_ph "Water using the IF97 standard, explicit in p and h. Recommended for most applications";
      package StandardWaterOnePhase = WaterIF97_pT "Water using the IF97 standard, explicit in p and T. Recommended for one-phase applications";
      package WaterIF97OnePhase_ph "Water using the IF97 standard, explicit in p and h, and only valid outside the two-phase dome"
        extends WaterIF97_base(ThermoStates = Choices.IndependentVariables.ph, final ph_explicit = true, final dT_explicit = false, final pT_explicit = false, final smoothModel = true, final onePhase = true);
        annotation(Documentation(info = "<html>

</html>"));
      end WaterIF97OnePhase_ph;
      package WaterIF97_pT "Water using the IF97 standard, explicit in p and T"
        extends WaterIF97_base(ThermoStates = Choices.IndependentVariables.pT, final ph_explicit = false, final dT_explicit = false, final pT_explicit = true, final smoothModel = true, final onePhase = true);
      end WaterIF97_pT;
      package WaterIF97_ph "Water using the IF97 standard, explicit in p and h"
        extends WaterIF97_base(ThermoStates = Choices.IndependentVariables.ph, final ph_explicit = true, final dT_explicit = false, final pT_explicit = false, smoothModel = false, onePhase = false);
        annotation(Documentation(info = "<html>

</html>"));
      end WaterIF97_ph;
      partial package WaterIF97_base "Water: Steam properties as defined by IAPWS/IF97 standard"
        annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -94,84},{94,40}}, lineColor = {127,191,255}, textString = "IF97"),Text(extent = {{ -94,20},{94, -24}}, lineColor = {127,191,255}, textString = "water")}), Documentation(info = "<HTML>
<p>
This model calculates medium properties
for water in the <b>liquid</b>, <b>gas</b> and <b>two phase</b> regions
according to the IAPWS/IF97 standard, i.e., the accepted industrial standard
and best compromise between accuracy and computation time.
For more details see <a href=\"Modelica://Modelica.Media.Water.IF97_Utilities\">
Modelica.Media.Water.IF97_Utilities</a>. Three variable pairs can be the
independent variables of the model:
<ol>
<li>Pressure <b>p</b> and specific enthalpy <b>h</b> are the most natural choice for general applications. This is the recommended choice for most general purpose applications, in particular for power plants.</li>
<li>Pressure <b>p</b> and temperature <b>T</b> are the most natural choice for applications where water is always in the same phase, both for liquid water and steam.</li>
<li>Density <b>d</b> and temperature <b>T</b> are explicit variables of the Helmholtz function in the near-critical region and can be the best choice for applications with super-critical or near-critial states.</li>
</ol>
The following quantities are always computed:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Variable</b></td>
      <td valign=\"top\"><b>Unit</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">T</td>
      <td valign=\"top\">K</td>
      <td valign=\"top\">temperature</td></tr>
  <tr><td valign=\"top\">u</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific internal energy</b></td></tr>
  <tr><td valign=\"top\">d</td>
      <td valign=\"top\">kg/m^3</td>
      <td valign=\"top\">density</td></tr>
  <tr><td valign=\"top\">p</td>
      <td valign=\"top\">Pa</td>
      <td valign=\"top\">pressure</td></tr>
  <tr><td valign=\"top\">h</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific enthalpy</b></td></tr>
</table>
<p>
In some cases additional medium properties are needed.
A component that needs these optional properties has to call
one of the functions listed in
<a href=\"Modelica://Modelica.Media.UsersGuide.MediumUsage.OptionalProperties\">
Modelica.Media.UsersGuide.MediumUsage.OptionalProperties</a> and in
<a href=\"Modelica://Modelica.Media.UsersGuide.MediumUsage.TwoPhase\">
Modelica.Media.UsersGuide.MediumUsage.TwoPhase</a>.
</p>
</p>
<p>Many further properties can be computed. Using the well-known Bridgman's Tables, all first partial derivatives of the standard thermodynamic variables can be computed easily.</p>
</HTML>
"));
        extends Interfaces.PartialTwoPhaseMedium(mediumName = "WaterIF97", substanceNames = {"water"}, singleState = false, SpecificEnthalpy(start = 100000.0, nominal = 500000.0), Density(start = 150, nominal = 500), AbsolutePressure(start = 5000000.0, nominal = 1000000.0), Temperature(start = 500, nominal = 500), smoothModel = false, onePhase = false, fluidConstants = waterConstants);
        record extends ThermodynamicState "thermodynamic state"
          SpecificEnthalpy h "specific enthalpy";
          Density d "density";
          Temperature T "temperature";
          AbsolutePressure p "pressure";
        end ThermodynamicState;
        constant Boolean ph_explicit "true if explicit in pressure and specific enthalpy";
        constant Boolean dT_explicit "true if explicit in density and temperature";
        constant Boolean pT_explicit "true if explicit in pressure and temperature";
        redeclare replaceable model extends BaseProperties(h(stateSelect = if ph_explicit and preferredMediumStates then StateSelect.prefer else StateSelect.default), d(stateSelect = if dT_explicit and preferredMediumStates then StateSelect.prefer else StateSelect.default), T(stateSelect = if (pT_explicit or dT_explicit) and preferredMediumStates then StateSelect.prefer else StateSelect.default), p(stateSelect = if (pT_explicit or ph_explicit) and preferredMediumStates then StateSelect.prefer else StateSelect.default)) "Base properties of water"
          Integer phase(min = 0, max = 2, start = 1, fixed = false) "2 for two-phase, 1 for one-phase, 0 if not known";
          SaturationProperties sat(Tsat(start = 300.0), psat(start = 100000.0)) "saturation temperature and pressure";

        equation 
          MM = fluidConstants[1].molarMass;
          if smoothModel then
            if onePhase then
              phase = 1;
              if ph_explicit then
                assert(h < bubbleEnthalpy(sat) or h > dewEnthalpy(sat) or p > fluidConstants[1].criticalPressure, "With onePhase=true this model may only be called with one-phase states h < hl or h > hv!" + "(p = " + String(p) + ", h = " + String(h) + ")");

              else
                if dT_explicit then
                  assert(not (d < bubbleDensity(sat) and d > dewDensity(sat) and T < fluidConstants[1].criticalTemperature), "With onePhase=true this model may only be called with one-phase states d > dl or d < dv!" + "(d = " + String(d) + ", T = " + String(T) + ")");

                else
                  assert(true, "no events for pT-model");

                end if;

              end if;

            else
              phase = 0;

            end if;

          else
            if ph_explicit then
              phase = if h < bubbleEnthalpy(sat) or h > dewEnthalpy(sat) or p > fluidConstants[1].criticalPressure then 1 else 2;

            elseif dT_explicit then
              phase = if not (d < bubbleDensity(sat) and d > dewDensity(sat) and T < fluidConstants[1].criticalTemperature) then 1 else 2;


            else
              phase = 1;

            end if;

          end if;
          if dT_explicit then
            p = pressure_dT(d, T, phase);
            h = specificEnthalpy_dT(d, T, phase);
            sat.Tsat = T;
            sat.psat = saturationPressure(T);

          elseif ph_explicit then
            d = density_ph(p, h, phase);
            T = temperature_ph(p, h, phase);
            sat.Tsat = saturationTemperature(p);
            sat.psat = p;


          else
            h = specificEnthalpy_pT(p, T);
            d = density_pT(p, T);
            sat.psat = p;
            sat.Tsat = saturationTemperature(p);

          end if;
          u = h - p / d;
          R = Modelica.Constants.R / fluidConstants[1].molarMass;
          h = state.h;
          p = state.p;
          T = state.T;
          d = state.d;
          phase = state.phase;
        end BaseProperties;
        function density_ph "Computes density as a function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Density d "Density";
        algorithm 
          d:=IF97_Utilities.rho_ph(p, h, phase);
        end density_ph;
        function temperature_ph "Computes temperature as a function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Temperature T "Temperature";
        algorithm 
          T:=IF97_Utilities.T_ph(p, h, phase);
        end temperature_ph;
        function temperature_ps "Compute temperature from pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Temperature T "Temperature";
        algorithm 
          T:=IF97_Utilities.T_ps(p, s, phase);
        end temperature_ps;
        function density_ps "Computes density as a function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Density d "density";
        algorithm 
          d:=IF97_Utilities.rho_ps(p, s, phase);
        end density_ps;
        function pressure_dT "Computes pressure as a function of density and temperature"
          extends Modelica.Icons.Function;
          input Density d "Density";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output AbsolutePressure p "Pressure";
        algorithm 
          p:=IF97_Utilities.p_dT(d, T, phase);
        end pressure_dT;
        function specificEnthalpy_dT "Computes specific enthalpy as a function of density and temperature"
          extends Modelica.Icons.Function;
          input Density d "Density";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output SpecificEnthalpy h "specific enthalpy";
        algorithm 
          h:=IF97_Utilities.h_dT(d, T, phase);
        end specificEnthalpy_dT;
        function specificEnthalpy_pT "Computes specific enthalpy as a function of pressure and temperature"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output SpecificEnthalpy h "specific enthalpy";
        algorithm 
          h:=IF97_Utilities.h_pT(p, T);
        end specificEnthalpy_pT;
        function specificEnthalpy_ps "Computes specific enthalpy as a function of pressure and temperature"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output SpecificEnthalpy h "specific enthalpy";
        algorithm 
          h:=IF97_Utilities.h_ps(p, s, phase);
        end specificEnthalpy_ps;
        function density_pT "Computes density as a function of pressure and temperature"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output Density d "Density";
        algorithm 
          d:=IF97_Utilities.rho_pT(p, T);
        end density_pT;
        function extends setDewState "set the thermodynamic state on the dew line"
        algorithm 
          state:=ThermodynamicState(phase = phase, p = sat.psat, T = sat.Tsat, h = dewEnthalpy(sat), d = dewDensity(sat));
        end setDewState;
        function extends setBubbleState "set the thermodynamic state on the bubble line"
        algorithm 
          state:=ThermodynamicState(phase = phase, p = sat.psat, T = sat.Tsat, h = bubbleEnthalpy(sat), d = bubbleDensity(sat));
        end setBubbleState;
        function extends dynamicViscosity "Dynamic viscosity of water"
        algorithm 
          eta:=IF97_Utilities.dynamicViscosity(state.d, state.T, state.p, state.phase);
        end dynamicViscosity;
        function extends thermalConductivity "Thermal conductivity of water"
        algorithm 
          lambda:=IF97_Utilities.thermalConductivity(state.d, state.T, state.p, state.phase);
        end thermalConductivity;
        function extends surfaceTension "Surface tension in two phase region of water"
        algorithm 
          sigma:=IF97_Utilities.surfaceTension(sat.Tsat);
        end surfaceTension;
        function extends pressure "return pressure of ideal gas"
        algorithm 
          p:=state.p;
        end pressure;
        function extends temperature "return temperature of ideal gas"
        algorithm 
          T:=state.T;
        end temperature;
        function extends density "return density of ideal gas"
        algorithm 
          d:=state.d;
        end density;
        function extends specificEnthalpy "Return specific enthalpy"
          extends Modelica.Icons.Function;
        algorithm 
          h:=state.h;
        end specificEnthalpy;
        function extends specificInternalEnergy "Return specific internal energy"
          extends Modelica.Icons.Function;
        algorithm 
          u:=state.h - state.p / state.d;
        end specificInternalEnergy;
        function extends specificGibbsEnergy "Return specific Gibbs energy"
          extends Modelica.Icons.Function;
        algorithm 
          g:=state.h - state.T * specificEntropy(state);
        end specificGibbsEnergy;
        function extends specificHelmholtzEnergy "Return specific Helmholtz energy"
          extends Modelica.Icons.Function;
        algorithm 
          f:=state.h - state.p / state.d - state.T * specificEntropy(state);
        end specificHelmholtzEnergy;
        function extends specificEntropy "specific entropy of water"
        algorithm 
          if dT_explicit then 
                    s:=IF97_Utilities.s_dT(state.d, state.T, state.phase);
          elseif pT_explicit then
            s:=IF97_Utilities.s_pT(state.p, state.T);


          else           s:=IF97_Utilities.s_ph(state.p, state.h, state.phase);

          end if;
        end specificEntropy;
        function extends specificHeatCapacityCp "specific heat capacity at constant pressure of water"
          annotation(Documentation(info = "<html>
                                <p>In the two phase region this function returns the interpolated heat capacity between the
                                liquid and vapour state heat capacities.</p>
                          <html>"));
        algorithm 
          if dT_explicit then 
                    cp:=IF97_Utilities.cp_dT(state.d, state.T, state.phase);
          elseif pT_explicit then
            cp:=IF97_Utilities.cp_pT(state.p, state.T);


          else           cp:=IF97_Utilities.cp_ph(state.p, state.h, state.phase);

          end if;
        end specificHeatCapacityCp;
        function extends specificHeatCapacityCv "specific heat capacity at constant volume of water"
        algorithm 
          if dT_explicit then 
                    cv:=IF97_Utilities.cv_dT(state.d, state.T, state.phase);
          elseif pT_explicit then
            cv:=IF97_Utilities.cv_pT(state.p, state.T);


          else           cv:=IF97_Utilities.cv_ph(state.p, state.h, state.phase);

          end if;
        end specificHeatCapacityCv;
        function extends isentropicExponent "Return isentropic exponent"
        algorithm 
          if dT_explicit then 
                    gamma:=IF97_Utilities.isentropicExponent_dT(state.d, state.T, state.phase);
          elseif pT_explicit then
            gamma:=IF97_Utilities.isentropicExponent_pT(state.p, state.T);


          else           gamma:=IF97_Utilities.isentropicExponent_ph(state.p, state.h, state.phase);

          end if;
        end isentropicExponent;
        function extends isothermalCompressibility "Isothermal compressibility of water"
        algorithm 
          if dT_explicit then 
                    kappa:=IF97_Utilities.kappa_dT(state.d, state.T, state.phase);
          elseif pT_explicit then
            kappa:=IF97_Utilities.kappa_pT(state.p, state.T);


          else           kappa:=IF97_Utilities.kappa_ph(state.p, state.h, state.phase);

          end if;
        end isothermalCompressibility;
        function extends isobaricExpansionCoefficient "isobaric expansion coefficient of water"
        algorithm 
          if dT_explicit then 
                    beta:=IF97_Utilities.beta_dT(state.d, state.T, state.phase);
          elseif pT_explicit then
            beta:=IF97_Utilities.beta_pT(state.p, state.T);


          else           beta:=IF97_Utilities.beta_ph(state.p, state.h, state.phase);

          end if;
        end isobaricExpansionCoefficient;
        function extends velocityOfSound "Return velocity of sound as a function of the thermodynamic state record"
        algorithm 
          if dT_explicit then 
                    a:=IF97_Utilities.velocityOfSound_dT(state.d, state.T, state.phase);
          elseif pT_explicit then
            a:=IF97_Utilities.velocityOfSound_pT(state.p, state.T);


          else           a:=IF97_Utilities.velocityOfSound_ph(state.p, state.h, state.phase);

          end if;
        end velocityOfSound;
        function extends isentropicEnthalpy "compute h(p,s)"
        algorithm 
          h_is:=IF97_Utilities.isentropicEnthalpy(p_downstream, specificEntropy(refState), 0);
        end isentropicEnthalpy;
        function extends density_derh_p "density derivative by specific enthalpy"
        algorithm 
          ddhp:=IF97_Utilities.ddhp(state.p, state.h, state.phase);
        end density_derh_p;
        function extends density_derp_h "density derivative by pressure"
        algorithm 
          ddph:=IF97_Utilities.ddph(state.p, state.h, state.phase);
        end density_derp_h;
        function extends bubbleEnthalpy "boiling curve specific enthalpy of water"
        algorithm 
          hl:=IF97_Utilities.BaseIF97.Regions.hl_p(sat.psat);
        end bubbleEnthalpy;
        function extends dewEnthalpy "dew curve specific enthalpy of water"
        algorithm 
          hv:=IF97_Utilities.BaseIF97.Regions.hv_p(sat.psat);
        end dewEnthalpy;
        function extends bubbleEntropy "boiling curve specific entropy of water"
        algorithm 
          sl:=IF97_Utilities.BaseIF97.Regions.sl_p(sat.psat);
        end bubbleEntropy;
        function extends dewEntropy "dew curve specific entropy of water"
        algorithm 
          sv:=IF97_Utilities.BaseIF97.Regions.sv_p(sat.psat);
        end dewEntropy;
        function extends bubbleDensity "boiling curve specific density of water"
        algorithm 
          if ph_explicit or pT_explicit then 
                    dl:=IF97_Utilities.BaseIF97.Regions.rhol_p(sat.psat);

          else           dl:=IF97_Utilities.BaseIF97.Regions.rhol_T(sat.Tsat);

          end if;
        end bubbleDensity;
        function extends dewDensity "dew curve specific density of water"
        algorithm 
          if ph_explicit or pT_explicit then 
                    dv:=IF97_Utilities.BaseIF97.Regions.rhov_p(sat.psat);

          else           dv:=IF97_Utilities.BaseIF97.Regions.rhov_T(sat.Tsat);

          end if;
        end dewDensity;
        function extends saturationTemperature "saturation temperature of water"
        algorithm 
          T:=IF97_Utilities.BaseIF97.Basic.tsat(p);
        end saturationTemperature;
        function extends saturationTemperature_derp "derivative of saturation temperature w.r.t. pressure"
        algorithm 
          dTp:=IF97_Utilities.BaseIF97.Basic.dtsatofp(p);
        end saturationTemperature_derp;
        function extends saturationPressure "saturation pressure of water"
        algorithm 
          p:=IF97_Utilities.BaseIF97.Basic.psat(T);
        end saturationPressure;
        function extends dBubbleDensity_dPressure "bubble point density derivative"
        algorithm 
          ddldp:=IF97_Utilities.BaseIF97.Regions.drhol_dp(sat.psat);
        end dBubbleDensity_dPressure;
        function extends dDewDensity_dPressure "dew point density derivative"
        algorithm 
          ddvdp:=IF97_Utilities.BaseIF97.Regions.drhov_dp(sat.psat);
        end dDewDensity_dPressure;
        function extends dBubbleEnthalpy_dPressure "bubble point specific enthalpy derivative"
        algorithm 
          dhldp:=IF97_Utilities.BaseIF97.Regions.dhl_dp(sat.psat);
        end dBubbleEnthalpy_dPressure;
        function extends dDewEnthalpy_dPressure "dew point specific enthalpy derivative"
        algorithm 
          dhvdp:=IF97_Utilities.BaseIF97.Regions.dhv_dp(sat.psat);
        end dDewEnthalpy_dPressure;
        function extends setState_dTX "Return thermodynamic state of water as function of d and T"
        algorithm 
          state:=ThermodynamicState(d = d, T = T, phase = 0, h = specificEnthalpy_dT(d, T), p = pressure_dT(d, T));
        end setState_dTX;
        function extends setState_phX "Return thermodynamic state of water as function of p and h"
        algorithm 
          state:=ThermodynamicState(d = density_ph(p, h), T = temperature_ph(p, h), phase = 0, h = h, p = p);
        end setState_phX;
        function extends setState_psX "Return thermodynamic state of water as function of p and s"
        algorithm 
          state:=ThermodynamicState(d = density_ps(p, s), T = temperature_ps(p, s), phase = 0, h = specificEnthalpy_ps(p, s), p = p);
        end setState_psX;
        function extends setState_pTX "Return thermodynamic state of water as function of p and T"
        algorithm 
          state:=ThermodynamicState(d = density_pT(p, T), T = T, phase = 1, h = specificEnthalpy_pT(p, T), p = p);
        end setState_pTX;
        function extends setSmoothState "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
          import Modelica.Media.Common.smoothStep;
        algorithm 
          state:=ThermodynamicState(p = smoothStep(x, state_a.p, state_b.p, x_small), h = smoothStep(x, state_a.h, state_b.h, x_small), d = density_ph(smoothStep(x, state_a.p, state_b.p, x_small), smoothStep(x, state_a.h, state_b.h, x_small)), T = temperature_ph(smoothStep(x, state_a.p, state_b.p, x_small), smoothStep(x, state_a.h, state_b.h, x_small)), phase = 0);
        end setSmoothState;
      end WaterIF97_base;
      partial package WaterIF97_fixedregion "Water: Steam properties as defined by IAPWS/IF97 standard"
        annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -94,84},{94,40}}, lineColor = {127,191,255}, textString = "IF97"),Text(extent = {{ -94,20},{94, -24}}, lineColor = {127,191,255}, textString = "water")}), Documentation(info = "<HTML>
<p>
This model calculates medium properties
for water in the <b>liquid</b>, <b>gas</b> and <b>two phase</b> regions
according to the IAPWS/IF97 standard, i.e., the accepted industrial standard
and best compromise between accuracy and computation time.
For more details see <a href=\"Modelica://Modelica.Media.Water.IF97_Utilities\">
Modelica.Media.Water.IF97_Utilities</a>. Three variable pairs can be the
independent variables of the model:
<ol>
<li>Pressure <b>p</b> and specific enthalpy <b>h</b> are the most natural choice for general applications. This is the recommended choice for most general purpose applications, in particular for power plants.</li>
<li>Pressure <b>p</b> and temperature <b>T</b> are the most natural choice for applications where water is always in the same phase, both for liquid water and steam.</li>
<li>Density <b>d</b> and temperature <b>T</b> are explicit variables of the Helmholtz function in the near-critical region and can be the best choice for applications with super-critical or near-critial states.</li>
</ol>
The following quantities are always computed:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Variable</b></td>
      <td valign=\"top\"><b>Unit</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">T</td>
      <td valign=\"top\">K</td>
      <td valign=\"top\">temperature</td></tr>
  <tr><td valign=\"top\">u</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific internal energy</b></td></tr>
  <tr><td valign=\"top\">d</td>
      <td valign=\"top\">kg/m^3</td>
      <td valign=\"top\">density</td></tr>
  <tr><td valign=\"top\">p</td>
      <td valign=\"top\">Pa</td>
      <td valign=\"top\">pressure</td></tr>
  <tr><td valign=\"top\">h</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific enthalpy</b></td></tr>
</table>
<p>
In some cases additional medium properties are needed.
A component that needs these optional properties has to call
one of the functions listed in
<a href=\"Modelica://Modelica.Media.UsersGuide.MediumUsage.OptionalProperties\">
Modelica.Media.UsersGuide.MediumUsage.OptionalProperties</a> and in
<a href=\"Modelica://Modelica.Media.UsersGuide.MediumUsage.TwoPhase\">
Modelica.Media.UsersGuide.MediumUsage.TwoPhase</a>.
</p>
</p>
<p>Many further properties can be computed. Using the well-known Bridgman's Tables, all first partial derivatives of the standard thermodynamic variables can be computed easily.
</p>
</HTML>
"));
        extends Interfaces.PartialTwoPhaseMedium(mediumName = "WaterIF97", substanceNames = {"water"}, singleState = false, SpecificEnthalpy(start = 100000.0, nominal = 500000.0), Density(start = 150, nominal = 500), AbsolutePressure(start = 5000000.0, nominal = 1000000.0), Temperature(start = 500, nominal = 500), smoothModel = false, onePhase = false, fluidConstants = waterConstants);
        record extends ThermodynamicState "thermodynamic state"
          SpecificEnthalpy h "specific enthalpy";
          Density d "density";
          Temperature T "temperature";
          AbsolutePressure p "pressure";
        end ThermodynamicState;
        constant Integer Region "region of IF97, if known";
        constant Boolean ph_explicit "true if explicit in pressure and specific enthalpy";
        constant Boolean dT_explicit "true if explicit in density and temperature";
        constant Boolean pT_explicit "true if explicit in pressure and temperature";
        redeclare replaceable model extends BaseProperties(h(stateSelect = if ph_explicit then StateSelect.prefer else StateSelect.default), d(stateSelect = if dT_explicit then StateSelect.prefer else StateSelect.default), T(stateSelect = if pT_explicit or dT_explicit then StateSelect.prefer else StateSelect.default), p(stateSelect = if pT_explicit or ph_explicit then StateSelect.prefer else StateSelect.default)) "Base properties of water"
          Integer phase(min = 0, max = 2) "2 for two-phase, 1 for one-phase, 0 if not known";
          SaturationProperties sat(Tsat(start = 300.0), psat(start = 100000.0)) "saturation temperature and pressure";

        equation 
          MM = fluidConstants[1].molarMass;
          if smoothModel then
            if onePhase then
              phase = 1;
              if ph_explicit then
                assert(h < bubbleEnthalpy(sat) or h > dewEnthalpy(sat) or p > fluidConstants[1].criticalPressure, "With onePhase=true this model may only be called with one-phase states h < hl or h > hv!");

              else
                assert(not (d < bubbleDensity(sat) and d > dewDensity(sat) and T < fluidConstants[1].criticalTemperature), "With onePhase=true this model may only be called with one-phase states d > dl or d < dv!");

              end if;

            else
              phase = 0;

            end if;

          else
            if ph_explicit then
              phase = if h < bubbleEnthalpy(sat) or h > dewEnthalpy(sat) or p > fluidConstants[1].criticalPressure then 1 else 2;

            elseif dT_explicit then
              phase = if not (d < bubbleDensity(sat) and d > dewDensity(sat) and T < fluidConstants[1].criticalTemperature) then 1 else 2;


            else
              phase = 1;

            end if;

          end if;
          if dT_explicit then
            p = pressure_dT(d, T, phase, Region);
            h = specificEnthalpy_dT(d, T, phase, Region);
            sat.Tsat = T;
            sat.psat = saturationPressure(T);

          elseif ph_explicit then
            d = density_ph(p, h, phase, Region);
            T = temperature_ph(p, h, phase, Region);
            sat.Tsat = saturationTemperature(p);
            sat.psat = p;


          else
            h = specificEnthalpy_pT(p, T, Region);
            d = density_pT(p, T, Region);
            sat.psat = p;
            sat.Tsat = saturationTemperature(p);

          end if;
          u = h - p / d;
          R = Modelica.Constants.R / fluidConstants[1].molarMass;
          h = state.h;
          p = state.p;
          T = state.T;
          d = state.d;
          phase = state.phase;
        end BaseProperties;
        function density_ph "Computes density as a function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output Density d "Density";
        algorithm 
          d:=IF97_Utilities.rho_ph(p, h, phase, region);
        end density_ph;
        function temperature_ph "Computes temperature as a function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output Temperature T "Temperature";
        algorithm 
          T:=IF97_Utilities.T_ph(p, h, phase, region);
        end temperature_ph;
        function temperature_ps "Compute temperature from pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output Temperature T "Temperature";
        algorithm 
          T:=IF97_Utilities.T_ps(p, s, phase, region);
        end temperature_ps;
        function density_ps "Computes density as a function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output Density d "density";
        algorithm 
          d:=IF97_Utilities.rho_ps(p, s, phase, region);
        end density_ps;
        function pressure_dT "Computes pressure as a function of density and temperature"
          extends Modelica.Icons.Function;
          input Density d "Density";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output AbsolutePressure p "Pressure";
        algorithm 
          p:=IF97_Utilities.p_dT(d, T, phase, region);
        end pressure_dT;
        function specificEnthalpy_dT "Computes specific enthalpy as a function of density and temperature"
          extends Modelica.Icons.Function;
          input Density d "Density";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SpecificEnthalpy h "specific enthalpy";
        algorithm 
          h:=IF97_Utilities.h_dT(d, T, phase, region);
        end specificEnthalpy_dT;
        function specificEnthalpy_pT "Computes specific enthalpy as a function of pressure and temperature"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SpecificEnthalpy h "specific enthalpy";
        algorithm 
          h:=IF97_Utilities.h_pT(p, T, region);
        end specificEnthalpy_pT;
        function specificEnthalpy_ps "Computes specific enthalpy as a function of pressure and temperature"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEntropy s "Specific entropy";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SpecificEnthalpy h "specific enthalpy";
        algorithm 
          h:=IF97_Utilities.h_ps(p, s, phase, region);
        end specificEnthalpy_ps;
        function density_pT "Computes density as a function of pressure and temperature"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input FixedPhase phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output Density d "Density";
        algorithm 
          d:=IF97_Utilities.rho_pT(p, T, region);
        end density_pT;
        function extends setDewState "set the thermodynamic state on the dew line"
        algorithm 
          state.phase:=phase;
          state.p:=sat.psat;
          state.T:=sat.Tsat;
          state.h:=dewEnthalpy(sat);
          state.d:=dewDensity(sat);
        end setDewState;
        function extends setBubbleState "set the thermodynamic state on the bubble line"
        algorithm 
          state.phase:=phase;
          state.p:=sat.psat;
          state.T:=sat.Tsat;
          state.h:=bubbleEnthalpy(sat);
          state.d:=bubbleDensity(sat);
        end setBubbleState;
        function extends dynamicViscosity "Dynamic viscosity of water"
        algorithm 
          eta:=IF97_Utilities.dynamicViscosity(state.d, state.T, state.p, state.phase);
        end dynamicViscosity;
        function extends thermalConductivity "Thermal conductivity of water"
        algorithm 
          lambda:=IF97_Utilities.thermalConductivity(state.d, state.T, state.p, state.phase);
        end thermalConductivity;
        function extends surfaceTension "Surface tension in two phase region of water"
        algorithm 
          sigma:=IF97_Utilities.surfaceTension(sat.Tsat);
        end surfaceTension;
        function extends pressure "return pressure of ideal gas"
        algorithm 
          p:=state.p;
        end pressure;
        function extends temperature "return temperature of ideal gas"
        algorithm 
          T:=state.T;
        end temperature;
        function extends density "return density of ideal gas"
        algorithm 
          d:=state.d;
        end density;
        function extends specificEnthalpy "Return specific enthalpy"
          extends Modelica.Icons.Function;
        algorithm 
          h:=state.h;
        end specificEnthalpy;
        function extends specificInternalEnergy "Return specific internal energy"
          extends Modelica.Icons.Function;
        algorithm 
          u:=state.h - state.p / state.d;
        end specificInternalEnergy;
        function extends specificGibbsEnergy "Return specific Gibbs energy"
          extends Modelica.Icons.Function;
        algorithm 
          g:=state.h - state.T * specificEntropy(state);
        end specificGibbsEnergy;
        function extends specificHelmholtzEnergy "Return specific Helmholtz energy"
          extends Modelica.Icons.Function;
        algorithm 
          f:=state.h - state.p / state.d - state.T * specificEntropy(state);
        end specificHelmholtzEnergy;
        function extends specificEntropy "specific entropy of water"
        algorithm 
          if dT_explicit then 
                    s:=IF97_Utilities.s_dT(state.d, state.T, state.phase, Region);
          elseif pT_explicit then
            s:=IF97_Utilities.s_pT(state.p, state.T, Region);


          else           s:=IF97_Utilities.s_ph(state.p, state.h, state.phase, Region);

          end if;
        end specificEntropy;
        function extends specificHeatCapacityCp "specific heat capacity at constant pressure of water"
        algorithm 
          if dT_explicit then 
                    cp:=IF97_Utilities.cp_dT(state.d, state.T, Region);
          elseif pT_explicit then
            cp:=IF97_Utilities.cp_pT(state.p, state.T, Region);


          else           cp:=IF97_Utilities.cp_ph(state.p, state.h, Region);

          end if;
        end specificHeatCapacityCp;
        function extends specificHeatCapacityCv "specific heat capacity at constant volume of water"
        algorithm 
          if dT_explicit then 
                    cv:=IF97_Utilities.cv_dT(state.d, state.T, state.phase, Region);
          elseif pT_explicit then
            cv:=IF97_Utilities.cv_pT(state.p, state.T, Region);


          else           cv:=IF97_Utilities.cv_ph(state.p, state.h, state.phase, Region);

          end if;
        end specificHeatCapacityCv;
        function extends isentropicExponent "Return isentropic exponent"
        algorithm 
          if dT_explicit then 
                    gamma:=IF97_Utilities.isentropicExponent_dT(state.d, state.T, state.phase, Region);
          elseif pT_explicit then
            gamma:=IF97_Utilities.isentropicExponent_pT(state.p, state.T, Region);


          else           gamma:=IF97_Utilities.isentropicExponent_ph(state.p, state.h, state.phase, Region);

          end if;
        end isentropicExponent;
        function extends isothermalCompressibility "Isothermal compressibility of water"
        algorithm 
          if dT_explicit then 
                    kappa:=IF97_Utilities.kappa_dT(state.d, state.T, state.phase, Region);
          elseif pT_explicit then
            kappa:=IF97_Utilities.kappa_pT(state.p, state.T, Region);


          else           kappa:=IF97_Utilities.kappa_ph(state.p, state.h, state.phase, Region);

          end if;
        end isothermalCompressibility;
        function extends isobaricExpansionCoefficient "isobaric expansion coefficient of water"
        algorithm 
          if dT_explicit then 
                    beta:=IF97_Utilities.beta_dT(state.d, state.T, state.phase, Region);
          elseif pT_explicit then
            beta:=IF97_Utilities.beta_pT(state.p, state.T, Region);


          else           beta:=IF97_Utilities.beta_ph(state.p, state.h, state.phase, Region);

          end if;
        end isobaricExpansionCoefficient;
        function extends velocityOfSound "Return velocity of sound as a function of the thermodynamic state record"
        algorithm 
          if dT_explicit then 
                    a:=IF97_Utilities.velocityOfSound_dT(state.d, state.T, state.phase, Region);
          elseif pT_explicit then
            a:=IF97_Utilities.velocityOfSound_pT(state.p, state.T, Region);


          else           a:=IF97_Utilities.velocityOfSound_ph(state.p, state.h, state.phase, Region);

          end if;
        end velocityOfSound;
        function extends isentropicEnthalpy "compute h(s,p)"
        algorithm 
          h_is:=IF97_Utilities.isentropicEnthalpy(p_downstream, specificEntropy(refState), 0);
        end isentropicEnthalpy;
        function extends density_derh_p "density derivative by specific enthalpy"
        algorithm 
          ddhp:=IF97_Utilities.ddhp(state.p, state.h, state.phase, Region);
        end density_derh_p;
        function extends density_derp_h "density derivative by pressure"
        algorithm 
          ddph:=IF97_Utilities.ddph(state.p, state.h, state.phase, Region);
        end density_derp_h;
        function extends bubbleEnthalpy "boiling curve specific enthalpy of water"
        algorithm 
          hl:=IF97_Utilities.BaseIF97.Regions.hl_p(sat.psat);
        end bubbleEnthalpy;
        function extends dewEnthalpy "dew curve specific enthalpy of water"
        algorithm 
          hv:=IF97_Utilities.BaseIF97.Regions.hv_p(sat.psat);
        end dewEnthalpy;
        function extends bubbleEntropy "boiling curve specific entropy of water"
        algorithm 
          sl:=IF97_Utilities.BaseIF97.Regions.sl_p(sat.psat);
        end bubbleEntropy;
        function extends dewEntropy "dew curve specific entropy of water"
        algorithm 
          sv:=IF97_Utilities.BaseIF97.Regions.sv_p(sat.psat);
        end dewEntropy;
        function extends bubbleDensity "boiling curve specific density of water"
        algorithm 
          if ph_explicit or pT_explicit then 
                    dl:=IF97_Utilities.BaseIF97.Regions.rhol_p(sat.psat);

          else           dl:=IF97_Utilities.BaseIF97.Regions.rhol_T(sat.Tsat);

          end if;
        end bubbleDensity;
        function extends dewDensity "dew curve specific density of water"
        algorithm 
          if ph_explicit or pT_explicit then 
                    dv:=IF97_Utilities.BaseIF97.Regions.rhov_p(sat.psat);

          else           dv:=IF97_Utilities.BaseIF97.Regions.rhov_T(sat.Tsat);

          end if;
        end dewDensity;
        function extends saturationTemperature "saturation temperature of water"
        algorithm 
          T:=IF97_Utilities.BaseIF97.Basic.tsat(p);
        end saturationTemperature;
        function extends saturationTemperature_derp "derivative of saturation temperature w.r.t. pressure"
        algorithm 
          dTp:=IF97_Utilities.BaseIF97.Basic.dtsatofp(p);
        end saturationTemperature_derp;
        function extends saturationPressure "saturation pressure of water"
        algorithm 
          p:=IF97_Utilities.BaseIF97.Basic.psat(T);
        end saturationPressure;
        function extends dBubbleDensity_dPressure "bubble point density derivative"
        algorithm 
          ddldp:=IF97_Utilities.BaseIF97.Regions.drhol_dp(sat.psat);
        end dBubbleDensity_dPressure;
        function extends dDewDensity_dPressure "dew point density derivative"
        algorithm 
          ddvdp:=IF97_Utilities.BaseIF97.Regions.drhov_dp(sat.psat);
        end dDewDensity_dPressure;
        function extends dBubbleEnthalpy_dPressure "bubble point specific enthalpy derivative"
        algorithm 
          dhldp:=IF97_Utilities.BaseIF97.Regions.dhl_dp(sat.psat);
        end dBubbleEnthalpy_dPressure;
        function extends dDewEnthalpy_dPressure "dew point specific enthalpy derivative"
        algorithm 
          dhvdp:=IF97_Utilities.BaseIF97.Regions.dhv_dp(sat.psat);
        end dDewEnthalpy_dPressure;
        function extends setState_dTX "Return thermodynamic state of water as function of d, T, and optional region"
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
        algorithm 
          state:=ThermodynamicState(d = d, T = T, phase = if region == 0 then IF97_Utilities.phase_dT(d, T) else if region == 4 then 2 else 1, h = specificEnthalpy_dT(d, T, region = region), p = pressure_dT(d, T, region = region));
        end setState_dTX;
        function extends setState_phX "Return thermodynamic state of water as function of p, h, and optional region"
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
        algorithm 
          state:=ThermodynamicState(d = density_ph(p, h, region = region), T = temperature_ph(p, h, region = region), phase = if region == 0 then IF97_Utilities.phase_ph(p, h) else if region == 4 then 2 else 1, h = h, p = p);
        end setState_phX;
        function extends setState_psX "Return thermodynamic state of water as function of p, s, and optional region"
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
        algorithm 
          state:=ThermodynamicState(d = density_ps(p, s, region = region), T = temperature_ps(p, s, region = region), phase = IF97_Utilities.phase_ps(p, s), h = specificEnthalpy_ps(p, s, region = region), p = p);
        end setState_psX;
        function extends setState_pTX "Return thermodynamic state of water as function of p, T, and optional region"
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
        algorithm 
          state:=ThermodynamicState(d = density_pT(p, T, region = region), T = T, phase = 1, h = specificEnthalpy_pT(p, T, region = region), p = p);
        end setState_pTX;
        function extends setSmoothState "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
          import Modelica.Media.Common.smoothStep;
        algorithm 
          state:=ThermodynamicState(p = smoothStep(x, state_a.p, state_b.p, x_small), h = smoothStep(x, state_a.h, state_b.h, x_small), d = density_ph(smoothStep(x, state_a.p, state_b.p, x_small), smoothStep(x, state_a.h, state_b.h, x_small)), T = temperature_ph(smoothStep(x, state_a.p, state_b.p, x_small), smoothStep(x, state_a.h, state_b.h, x_small)), phase = 0);
        end setSmoothState;
      end WaterIF97_fixedregion;
      package WaterIF97_R1ph "region 1 (liquid) water according to IF97 standard"
        extends WaterIF97_fixedregion(ThermoStates = Choices.IndependentVariables.ph, final Region = 1, final ph_explicit = true, final dT_explicit = false, final pT_explicit = false, smoothModel = true, onePhase = true);
        annotation(Documentation(info = "<html>

</html>"));
      end WaterIF97_R1ph;
      package WaterIF97_R2ph "region 2 (steam) water according to IF97 standard"
        extends WaterIF97_fixedregion(ThermoStates = Choices.IndependentVariables.ph, final Region = 2, final ph_explicit = true, final dT_explicit = false, final pT_explicit = false, smoothModel = true, onePhase = true);
        annotation(Documentation(info = "<html>

</html>"));
      end WaterIF97_R2ph;
      package WaterIF97_R3ph "region 3 water according to IF97 standard"
        extends WaterIF97_fixedregion(final Region = 3, ThermoStates = Choices.IndependentVariables.ph, final ph_explicit = true, final dT_explicit = false, final pT_explicit = false, smoothModel = true, onePhase = true);
        annotation(Documentation(info = "<html>

</html>"));
      end WaterIF97_R3ph;
      package WaterIF97_R4ph "region 4 water according to IF97 standard"
        extends WaterIF97_fixedregion(final Region = 4, ThermoStates = Choices.IndependentVariables.ph, final ph_explicit = true, final dT_explicit = false, final pT_explicit = false, smoothModel = true, onePhase = false);
        annotation(Documentation(info = "<html>

</html>"));
      end WaterIF97_R4ph;
      package WaterIF97_R5ph "region 5 water according to IF97 standard"
        extends WaterIF97_fixedregion(ThermoStates = Choices.IndependentVariables.ph, final Region = 5, final ph_explicit = true, final dT_explicit = false, final pT_explicit = false, smoothModel = true, onePhase = true);
        annotation(Documentation(info = "<html>

</html>"));
      end WaterIF97_R5ph;
      package WaterIF97_R1pT "region 1 (liquid) water according to IF97 standard"
        extends WaterIF97_fixedregion(ThermoStates = Choices.IndependentVariables.pT, final Region = 1, final ph_explicit = false, final dT_explicit = false, final pT_explicit = true, smoothModel = true, onePhase = true);
        annotation(Documentation(info = "<html>

</html>"));
      end WaterIF97_R1pT;
      package WaterIF97_R2pT "region 2 (steam) water according to IF97 standard"
        extends WaterIF97_fixedregion(ThermoStates = Choices.IndependentVariables.pT, final Region = 2, final ph_explicit = false, final dT_explicit = false, final pT_explicit = true, smoothModel = true, onePhase = true);
        annotation(Documentation(info = "<html>

</html>"));
      end WaterIF97_R2pT;
      package IF97_Utilities "Low level and utility computation for high accuracy water properties according to the IAPWS/IF97 standard"
        annotation(Documentation(info = "<HTML>
      <h4>Package description:</h4>
      <p>This package provides high accuracy physical properties for water according
      to the IAPWS/IF97 standard. It has been part of the ThermoFluid Modelica library and been extended,
      reorganized and documented to become part of the Modelica Standard library.</p>
      <p>An important feature that distinguishes this implementation of the IF97 steam property standard
      is that this implementation has been explicitly designed to work well in dynamic simulations. Computational
      performance has been of high importance. This means that there often exist several ways to get the same result
      from different functions if one of the functions is called often but can be optimized for that purpose.
      </p>
      <p>
      The original documentation of the IAPWS/IF97 steam properties can freely be distributed with computer
      implementations, so for curious minds the complete standard documentation is provided with the Modelica
      properties library. The following documents are included
      (in directory Modelica\\help\\Documentation\\IF97documentation):
      <ul>
      <li><a href=\"Documentation/IF97documentation/IF97.pdf\">IF97.pdf</a> The standards document for the main part of the IF97.</li>
      <li><a href=\"Documentation/IF97documentation/Back3.pdf\">Back3.pdf</a> The backwards equations for region 3.</li>
      <li><a href=\"Documentation/IF97documentation/crits.pdf\">crits.pdf</a> The critical point data.</li>
      <li><a href=\"Documentation/IF97documentation/meltsub.pdf\">meltsub.pdf</a> The melting- and sublimation line formulation (in IF97_Utilities.BaseIF97.IceBoundaries)</li>
      <li><a href=\"Documentation/IF97documentation/surf.pdf\">surf.pdf</a> The surface tension standard definition</li>
      <li><a href=\"Documentation/IF97documentation/thcond.pdf\">thcond.pdf</a> The thermal conductivity standard definition</li>
      <li><a href=\"Documentation/IF97documentation/visc.pdf\">visc.pdf</a> The viscosity standard definition</li>
      </ul>
      </p>
      <h4>Package contents
      </h4>
      <p>
      <ul>
      <li>Package <b>BaseIF97</b> contains the implementation of the IAPWS-IF97 as described in
      <a href=\"Documentation/IF97documentation/IF97.pdf\">IF97.pdf</a>. The explicit backwards equations for region 3 from
      <a href=\"Documentation/IF97documentation/Back3.pdf\">Back3.pdf</a> are implemented as initial values for an inverse iteration of the exact
      function in IF97 for the input pairs (p,h) and (p,s).
      The low-level functions in BaseIF97 are not needed for standard simulation usage,
      but can be useful for experts and some special purposes.</li>
      <li>Function <b>water_ph</b> returns all properties needed for a dynamic control volume model and properties of general
      interest using pressure p and specific entropy enthalpy h as dynamic states in the record ThermoProperties_ph. </li>
      <li>Function <b>water_ps</b> returns all properties needed for a dynamic control volume model and properties of general
      interest using pressure p and specific entropy s as dynamic states in the record ThermoProperties_ps. </li>
      <li>Function <b>water_dT</b> returns all properties needed for a dynamic control volume model and properties of general
      interest using density d and temperature T as dynamic states in the record ThermoProperties_dT. </li>
      <li>Function <b>water_pT</b> returns all properties needed for a dynamic control volume model and properties of general
      interest using pressure p and temperature T as dynamic states in the record ThermoProperties_pT. Due to the coupling of
      pressure and temperature in the two-phase region, this model can obviously
      only be used for one-phase models or models treating both phases independently.</li>
      <li>Function <b>hl_p</b> computes the liquid specific enthalpy as a function of pressure. For overcritical pressures,
      the critical specific enthalpy is returned</li>
      <li>Function <b>hv_p</b> computes the vapour specific enthalpy as a function of pressure. For overcritical pressures,
      the critical specific enthalpy is returned</li>
      <li>Function <b>sl_p</b> computes the liquid specific entropy as a function of pressure. For overcritical pressures,
      the critical  specific entropy is returned</li>
      <li>Function <b>sv_p</b> computes the vapour  specific entropy as a function of pressure. For overcritical pressures,
      the critical  specific entropyis returned</li>
      <li>Function <b>rhol_T</b> computes the liquid density as a function of temperature. For overcritical temperatures,
      the critical density is returned</li>
      <li>Function <b>rhol_T</b> computes the vapour density as a function of temperature. For overcritical temperatures,
      the critical density is returned</li>
      <li>Function <b>dynamicViscosity</b> computes the dynamic viscosity as a function of density and temperature.</li>
      <li>Function <b>thermalConductivity</b> computes the thermal conductivity as a function of density, temperature and pressure.
      <b>Important note</b>: Obviously only two of the three
      inputs are really needed, but using three inputs speeds up the computation and the three variables
      are known in most models anyways. The inputs d,T and p have to be consistent.</li>
      <li>Function <b>surfaceTension</b> computes the surface tension between vapour
          and liquid water as a function of temperature.</li>
      <li>Function <b>isentropicEnthalpy</b> computes the specific enthalpy h(p,s,phase) in all regions.
          The phase input is needed due to discontinuous derivatives at the phase boundary.</li>
      <li>Function <b>dynamicIsentropicEnthalpy</b> computes the specific enthalpy h(p,s,,dguess,Tguess,phase) in all regions.
          The phase input is needed due to discontinuous derivatives at the phase boundary. Tguess and dguess are initial guess
          values for the density and temperature consistent with p and s. This function should be preferred in
          dynamic simulations where good guesses are often available.</li>
      </ul>
      </p>
      <h4>Version Info and Revision history
      </h4>
      <ul>
      <li>First implemented: <i>July, 2000</i>
      by Hubertus Tummescheit for the ThermoFluid Library with help from Jonas Eborn and Falko Jens Wagner
      </li>
      <li>Code reorganization, enhanced documentation, additional functions:   <i>December, 2002</i>
      by <a href=\"mailto:Hubertus.Tummescheit@modelon.se\">Hubertus Tummescheit</a> and moved to Modelica
      properties library.</li>
      </ul>
      <address>Author: Hubertus Tummescheit, <br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
      </address>
      </HTML>", revisions = "<h4>Intermediate release notes during development</h4>
<p>Currenly the Events/noEvents switch is only implmented for p-h states. Only after testing that implmentation, it will be extended to dT.</p>"));
        extends Modelica.Icons.Library;
        package BaseIF97 "Modelica Physical Property Model: the new industrial formulation IAPWS-IF97"
          extends Modelica.Icons.Library;
          annotation(Documentation(info = "<HTML>
    <h4>Version Info and Revision history
        </h4>
        <ul>
        <li>First implemented: <i>July, 2000</i>
        by Hubertus Tummescheit
        for the ThermoFluid Library with help from Jonas Eborn and Falko Jens Wagner
        </li>
      <li>Code reorganization, enhanced documentation, additional functions:   <i>December, 2002</i>
      by <a href=\"mailto:Hubertus.Tummescheit@modelon.se\">Hubertus Tummescheit</a> and moved to Modelica
      properties library.</li>
        </ul>
      <address>Author: Hubertus Tummescheit, <br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
      </address>
        <P>In September 1997, the International Association for the Properties
        of Water and Steam (<A HREF=\"http://www.iapws.org\">IAPWS</A>) adopted a
        new formulation for the thermodynamic properties of water and steam for
        industrial use. This new industrial standard is called \"IAPWS Industrial
        Formulation for the Thermodynamic Properties of Water and Steam\" (IAPWS-IF97).
        The formulation IAPWS-IF97 replaces the previous industrial standard IFC-67.
        <P>Based on this new formulation, a new steam table, titled \"<a
        href=\"http://www.springer.de/cgi-bin/search_book.pl?isbn=3-540-64339-7\">Properties
        of Water and Steam</a>\" by W. Wagner and A. Kruse, was published by
        the Springer-Verlag, Berlin - New-York - Tokyo in April 1998. This
        steam table, ref. <a href=\"#steamprop\">[1]</a> is bilingual (English /
        German) and contains a complete description of the equations of
        IAPWS-IF97. This reference is the authoritative source of information
        for this implementation. A mostly identical version has been published by the International
        Association for the Properties
        of Water and Steam (<A HREF=\"http://www.iapws.org\">IAPWS</A>) with permission granted to re-publish the
        information if credit is given to IAPWS. This document is distributed with this library as
        <a href=\"Documentation/IF97documentation/IF97.pdf\">IF97.pdf</a>.
        In addition, the equations published by <A HREF=\"http://www.iapws.org\">IAPWS</A> for
        the transport properties dynamic viscosity (standards document: <a href=\"Documentation/IF97documentation/visc.pdf\">visc.pdf</a>)
        and thermal conductivity (standards document: <a href=\"Documentation/IF97documentation/thcond.pdf\">thcond.pdf</a>)
        and equations for the surface tension (standards document: <a href=\"Documentation/IF97documentation/surf.pdf\">surf.pdf</a>)
        are also implemented in this library and included for reference.
        <P>
        The functions in BaseIF97.mo are low level functions which should
        only be used in those exceptions when the standard user level
        functions in Water.mo do not contain the wanted properties.
        </p>
<P>Based on IAPWS-IF97, Modelica functions are available for calculating
the most common thermophysical properties (thermodynamic and transport
properties). The implementation requires part of the common medium
property infrastructure of the Modelica.Thermal.Properties library in the file
Common.mo. There are a few extensions from the version of IF97 as
documented in <a href=\"Documentation/IF97documentation/IF97.pdf\">IF97.pdf</a> in order to improve performance for
dynamic simulations. Input variables for calculating the properties are
only implemented for a limited number of variable pairs which make sense as dynamic states: (p,h), (p,T), (p,s) and (d,T).
<hr size=3 width=\"70%\">
<p><a name=\"regions\"><h4>1. Structure and Regions of IAPWS-IF97</h4></a>
<P>The IAPWS Industrial Formulation 1997 consists of
a set of equations for different regions which cover the following range
of validity:
<table border=0 cellpadding=4 align=center>
<tr>
<td valign=\"top\">273,15 K &lt; <I>T</I> &lt; 1073,15 K</td>
<td valign=\"top\"><I>p</I> &lt; 100 MPa</td>
</tr>
<tr>
<td valign=\"top\">1073,15 K &lt; <I>T</I> &lt; 2273,15 K</td>
<td valign=\"top\"><I>p</I> &lt; 10 MPa</td>
</tr>
</table><br>
Figure 1 shows the 5 regions into which the entire range of validity of
IAPWS-IF97 is divided. The boundaries of the regions can be directly taken
from Fig. 1 except for the boundary between regions 2 and 3; this boundary,
which corresponds approximately to the isentropic line <nobr><I>s</I> = 5.047 kJ kg
<FONT SIZE=-1><sup>-1</sup></FONT>
K<FONT SIZE=-1><sup>-1</sup></FONT>,</nobr> is defined
by a corresponding auxiliary equation. Both regions 1 and 2 are individually
covered by a fundamental equation for the specific Gibbs free energy <nobr><I>g</I>(<I>
p</I>,<I>T </I>)</nobr>, region 3 by a fundamental equation for the specific Helmholtz
free energy <nobr><I>f </I>(<I> <FONT FACE=\"Symbol\">r</FONT></I>,<I>T
</I>)</nobr>, and the saturation curve, corresponding to region 4, by a saturation-pressure
equation <nobr><I>p</I><FONT SIZE=-1><sub>s</sub></FONT>(<I>T</I>)</nobr>. The high-temperature
region 5 is also covered by a <nobr><I>g</I>(<I> p</I>,<I>T </I>)</nobr> equation. These
5 equations, shown in rectangular boxes in Fig. 1, form the so-called <I>basic
equations</I>.
      <p>
      <img src=\"../Images/Media/Water/if97.png\" alt=\"Regions and equations of IAPWS-IF97\"></p>
      <p align=center>Figure 1: Regions and equations of IAPWS-IF97</p>
<P>In addition to these basic equations, so-called <I>backward
equations</I> are provided for regions 1, 2, and 4 in form of
<nobr><I>T </I>(<I> p</I>,<I>h </I>)</nobr> and <nobr><I>T </I>(<I>
p</I>,<I>s </I>)</nobr> for regions 1 and 2, and <nobr><I>T</I><FONT
SIZE=-1><sub>s</sub> </FONT>(<I> p </I>)</nobr> for region 4. These
backward equations, marked in grey in Fig. 1, were developed in such a
way that they are numerically very consistent with the corresponding
basic equation. Thus, properties as functions of&nbsp; <I>p</I>,<I>h
</I>and of&nbsp;<I> p</I>,<I>s </I>for regions 1 and 2, and of
<I>p</I> for region 4 can be calculated without any iteration. As a
result of this special concept for the development of the new
industrial standard IAPWS-IF97, the most important properties can be
calculated extremely quickly. All modelica functions are optimized
with regard to short computing times.
<P>The complete description of the individual equations of the new industrial
formulation IAPWS-IF97 is given in <a href=\"Documentation/IF97documentation/IF97.pdf\">IF97.pdf</a>. Comprehensive information on
IAPWS-IF97 (requirements, concept, accuracy, consistency along region boundaries,
and the increase of computing speed in comparison with IFC-67, etc.) can
be taken from <a href=\"Documentation/IF97documentation/IF97.pdf\">IF97.pdf</a> or [2].
<P><a name=\"steamprop\">[1]<I>Wagner, W., Kruse, A.</I> Properties of Water
and Steam / Zustandsgr&ouml;&szlig;en von Wasser und Wasserdampf / IAPWS-IF97.
Springer-Verlag, Berlin, 1998.
<P>[2] <I>Wagner, W., Cooper, J. R., Dittmann, A., Kijima,
J., Kretzschmar, H.-J., Kruse, A., Mare� R., Oguchi, K., Sato, H., St&ouml;cker,
I., �fner, O., Takaishi, Y., Tanishita, I., Tr&uuml;benbach, J., and Willkommen,
Th.</I> The IAPWS Industrial Formulation 1997 for the Thermodynamic Properties
of Water and Steam. ASME Journal of Engineering for Gas Turbines and Power 122 (2000), 150 - 182.
<p>
<HR size=3 width=\"90%\">
<h4>2. Calculable Properties      </h4>
<table border=\"1\" cellpadding=\"2\" cellspacing=\"0\">
       <tbody>
       <tr>
       <td valign=\"top\" bgcolor=\"#cccccc\"><br>
      </td>
      <td valign=\"top\" bgcolor=\"#cccccc\"><b>Common name</b><br>
       </td>
       <td valign=\"top\" bgcolor=\"#cccccc\"><b>Abbreviation </b><br>
       </td>
       <td valign=\"top\" bgcolor=\"#cccccc\"><b>Unit</b><br>
       </td>
       </tr>
       <tr>
       <td valign=\"top\">&nbsp;1<br>
      </td>
      <td valign=\"top\">Pressure</td>
       <td valign=\"top\">p<br>
        </td>
       <td valign=\"top\">Pa<br>
       </td>
       </tr>
       <tr>
       <td valign=\"top\">&nbsp;2<br>
      </td>
      <td valign=\"top\">Temperature</td>
       <td valign=\"top\">T<br>
       </td>
       <td valign=\"top\">K<br>
       </td>
       </tr>
       <tr>
       <td valign=\"top\">&nbsp;3<br>
      </td>
      <td valign=\"top\">Density</td>
        <td valign=\"top\">d<br>
        </td>
       <td valign=\"top\">kg/m<sup>3</sup><br>
       </td>
       </tr>
       <tr>
       <td valign=\"top\">&nbsp;4<br>
      </td>
      <td valign=\"top\">Specific volume</td>
        <td valign=\"top\">v<br>
        </td>
       <td valign=\"top\">m<sup>3</sup>/kg<br>
       </td>
       </tr>
       <tr>
       <td valign=\"top\">&nbsp;5<br>
      </td>
      <td valign=\"top\">Specific enthalpy</td>
       <td valign=\"top\">h<br>
       </td>
       <td valign=\"top\">J/kg<br>
       </td>
       </tr>
       <tr>
       <td valign=\"top\">&nbsp;6<br>
      </td>
      <td valign=\"top\">Specific entropy</td>
       <td valign=\"top\">s<br>
       </td>
       <td valign=\"top\">J/(kg K)<br>
       </td>
       </tr>
       <tr>
       <td valign=\"top\">&nbsp;7<br>
      </td>
      <td valign=\"top\">Specific internal energy<br>
       </td>
       <td valign=\"top\">u<br>
       </td>
       <td valign=\"top\">J/kg<br>
       </td>
       </tr>
       <tr>
       <td valign=\"top\">&nbsp;8<br>
      </td>
      <td valign=\"top\">Specific isobaric heat capacity</td>
       <td valign=\"top\">c<font size=\"-1\"><sub>p</sub></font><br>
       </td>
       <td valign=\"top\">J/(kg K)<br>
       </td>
       </tr>
       <tr>
       <td valign=\"top\">&nbsp;9<br>
      </td>
      <td valign=\"top\">Specific isochoric heat capacity</td>
       <td valign=\"top\">c<font size=\"-1\"><sub>v</sub></font><br>
       </td>
       <td valign=\"top\">J/(kg K)<br>
       </td>
       </tr>
       <tr>
       <td valign=\"top\">10<br>
      </td>
      <td valign=\"top\">Isentropic exponent, kappa<nobr>=       <font face=\"Symbol\">-</font>(v/p)
(dp/dv)<font size=\"-1\"><sub>s</sub> </font></nobr></td>
     <td valign=\"top\">kappa (     <font face=\"Symbol\">k</font>)<br>
     </td>
     <td valign=\"top\">1<br>
     </td>
     </tr>
     <tr>
     <td valign=\"top\">11<br>
      </td>
      <td valign=\"top\">Speed of sound<br>
     </td>
     <td valign=\"top\">a<br>
     </td>
     <td valign=\"top\">m/s<br>
     </td>
     </tr>
     <tr>
     <td valign=\"top\">12<br>
      </td>
      <td valign=\"top\">Dryness fraction<br>
     </td>
     <td valign=\"top\">x<br>
     </td>
     <td valign=\"top\">kg/kg<br>
     </td>
     </tr>
     <tr>
     <td valign=\"top\">13<br>
      </td>
      <td valign=\"top\">Specific Helmholtz free energy,     f = u - Ts</td>
     <td valign=\"top\">f<br>
     </td>
     <td valign=\"top\">J/kg<br>
     </td>
     </tr>
     <tr>
     <td valign=\"top\">14<br>
      </td>
      <td valign=\"top\">Specific Gibbs free energy,     g = h - Ts</td>
     <td valign=\"top\">g<br>
     </td>
     <td valign=\"top\">J/kg<br>
     </td>
     </tr>
     <tr>
     <td valign=\"top\">15<br>
      </td>
      <td valign=\"top\">Isenthalpic exponent, <nobr> theta     = -(v/p)(dp/dv)<font
 size=\"-1\"><sub>h</sub></font></nobr></td>
     <td valign=\"top\">theta (<font face=\"Symbol\">q</font>)<br>
     </td>
     <td valign=\"top\">1<br>
     </td>
     </tr>
     <tr>
     <td valign=\"top\">16<br>
      </td>
      <td valign=\"top\">Isobaric volume expansion coefficient,     alpha = v<font
 size=\"-1\"><sup>-1</sup></font>       (dv/dT)<font size=\"-1\"><sub>p</sub>
    </font></td>
     <td valign=\"top\">alpha  (<font face=\"Symbol\">a</font>)<br>
     </td>
       <td valign=\"top\">1/K<br>
     </td>
     </tr>
     <tr>
     <td valign=\"top\">17<br>
      </td>
      <td valign=\"top\">Isochoric pressure coefficient,     <nobr>beta = p<font
 size=\"-1\"><sup><font face=\"Symbol\">-</font>1</sup>     </font>(dp/dT)<font
 size=\"-1\"><sub>v</sub></font></nobr>     </td>
     <td valign=\"top\">beta (<font face=\"Symbol\">b</font>)<br>
     </td>
     <td valign=\"top\">1/K<br>
     </td>
     </tr>
     <tr>
     <td valign=\"top\">18<br>
      </td>
      <td valign=\"top\">Isothermal compressibility,     g<nobr>amma  = <font
 face=\"Symbol\">-</font>v        <sup><font size=\"-1\"><font face=\"Symbol\">-</font>1</font></sup>(dv/dp)<font
 size=\"-1\"><sub>T</sub></font></nobr> </td>
        <td valign=\"top\">gamma (<font face=\"Symbol\">g</font>)<br>
     </td>
     <td valign=\"top\">1/Pa<br>
     </td>
     </tr>
     <!-- <tr><td valign=\"top\">f</td><td valign=\"top\">Fugacity</td></tr> --> <tr>
     <td valign=\"top\">19<br>
      </td>
      <td valign=\"top\">Dynamic viscosity</td>
     <td valign=\"top\">eta (<font face=\"Symbol\">h</font>)<br>
     </td>
     <td valign=\"top\">Pa s<br>
     </td>
     </tr>
     <tr>
     <td valign=\"top\">20<br>
      </td>
      <td valign=\"top\">Kinematic viscosity</td>
     <td valign=\"top\">nu (<font face=\"Symbol\">n</font>)<br>
     </td>
     <td valign=\"top\">m<sup>2</sup>/s<br>
     </td>
     </tr>
     <!-- <tr><td valign=\"top\">Pr</td><td valign=\"top\">Prandtl number</td></tr> --> <tr>
     <td valign=\"top\">21<br>
      </td>
      <td valign=\"top\">Thermal conductivity</td>
     <td valign=\"top\">lambda (<font face=\"Symbol\">l</font>)<br>
     </td>
     <td valign=\"top\">W/(m K)<br>
     </td>
     </tr>
     <tr>
     <td valign=\"top\">22 <br>
      </td>
      <td valign=\"top\">Surface tension</td>
     <td valign=\"top\">sigma (<font face=\"Symbol\">s</font>)<br>
     </td>
     <td valign=\"top\">N/m<br>
     </td>
     </tr>
  </tbody>
</table>
        <p>The properties 1-11 are calculated by default with the functions for dynamic
        simulation, 2 of these variables are the dynamic states and are the inputs
        to calculate all other properties. In addition to these properties
        of general interest, the entries to the thermodynamic Jacobian matrix which render
        the mass- and energy balances explicit in the input variables to the property calculation are also calculated.
        For an explanatory example using pressure and specific enthalpy as states, see the Examples sub-package.</p>
        <p>The high-level calls to steam properties are grouped into records comprising both the properties of general interest
        and the entries to the thermodynamic Jacobian. If additional properties are
        needed the low level functions in BaseIF97 provide more choice.</p>
        <HR size=3 width=\"90%\">
        <h4>Additional functions</h4>
        <ul>
        <li>Function <b>boundaryvals_p</b> computes the temperature and the specific enthalpy and
        entropy on both phase boundaries as a function of p</li>
        <li>Function <b>boundaryderivs_p</b> is the Modelica derivative function of <b>boundaryvals_p</b></li>
        <li>Function <b>extraDerivs_ph</b> computes all entries to Bridgmans tables for all
        one-phase regions of IF97 using inputs (p,h). All 336 directional derivatives of the
        thermodynamic surface can be computed as a ratio of two entries in the return data, see package Common
        for details.</li>
        <li>Function <b>extraDerivs_pT</b> computes all entries to Bridgmans tables for all
        one-phase regions of IF97 using inputs (p,T).</li>
        </ul>
        </p>
        </HTML>"));
          record IterationData "constants for iterations internal to some functions"
            extends Modelica.Icons.Record;
            constant Integer IMAX = 50 "maximum number of iterations for inverse functions";
            constant Real DELP = 1e-06 "maximum iteration error in pressure, Pa";
            constant Real DELS = 1e-08 "maximum iteration error in specific entropy, J/{kg.K}";
            constant Real DELH = 1e-08 "maximum iteration error in specific entthalpy, J/kg";
            constant Real DELD = 1e-08 "maximum iteration error in density, kg/m^3";
          end IterationData;
          record data "constant IF97 data and region limits"
            extends Modelica.Icons.Record;
            constant SI.SpecificHeatCapacity RH2O = 461.526 "specific gas constant of water vapour";
            constant SI.MolarMass MH2O = 0.01801528 "molar weight of water";
            constant SI.Temperature TSTAR1 = 1386.0 "normalization temperature for region 1 IF97";
            constant SI.Pressure PSTAR1 = 16530000.0 "normalization pressure for region 1 IF97";
            constant SI.Temperature TSTAR2 = 540.0 "normalization temperature for region 2 IF97";
            constant SI.Pressure PSTAR2 = 1000000.0 "normalization pressure for region 2 IF97";
            constant SI.Temperature TSTAR5 = 1000.0 "normalization temperature for region 5 IF97";
            constant SI.Pressure PSTAR5 = 1000000.0 "normalization pressure for region 5 IF97";
            constant SI.SpecificEnthalpy HSTAR1 = 2500000.0 "normalization specific enthalpy for region 1 IF97";
            constant Real IPSTAR = 1e-06 "normalization pressure for inverse function in region 2 IF97";
            constant Real IHSTAR = 5e-07 "normalization specific enthalpy for inverse function in region 2 IF97";
            constant SI.Temperature TLIMIT1 = 623.15 "temperature limit between regions 1 and 3";
            constant SI.Temperature TLIMIT2 = 1073.15 "temperature limit between regions 2 and 5";
            constant SI.Temperature TLIMIT5 = 2273.15 "upper temperature limit of 5";
            constant SI.Pressure PLIMIT1 = 100000000.0 "upper pressure limit for regions 1, 2 and 3";
            constant SI.Pressure PLIMIT4A = 16529200.0 "pressure limit between regions 1 and 2, important for for two-phase (region 4)";
            constant SI.Pressure PLIMIT5 = 10000000.0 "upper limit of valid pressure in region 5";
            constant SI.Pressure PCRIT = 22064000.0 "the critical pressure";
            constant SI.Temperature TCRIT = 647.096 "the critical temperature";
            constant SI.Density DCRIT = 322.0 "the critical density";
            constant SI.SpecificEntropy SCRIT = 4412.02148223476 "the calculated specific entropy at the critical point";
            constant SI.SpecificEnthalpy HCRIT = 2087546.84511715 "the calculated specific enthalpy at the critical point";
            constant Real[5] n = array(348.05185628969,  -1.1671859879975, 0.0010192970039326, 572.54459862746, 13.91883977887) "polynomial coefficients for boundary between regions 2 and 3";
            annotation(Documentation(info = "<HTML>
 <h4>Record description</h4>
                           <p>Constants needed in the international steam properties IF97.
                           SCRIT and HCRIT are calculated from Helmholtz function for region 3.</p>
<h4>Version Info and Revision history
</h4>
<ul>
<li>First implemented: <i>July, 2000</i>
       by Hubertus Tummescheit
       </li>
</ul>
 <address>Author: Hubertus Tummescheit, <br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
 </address>
<ul>
 <li>Initial version: July 2000</li>
 <li>Documentation added: December 2002</li>
</ul>
</HTML>
"));
          end data;
          function getTstar "get normalization temperature for region 1, 2 or 5"
            extends Modelica.Icons.Function;
            input Integer region "IF 97 region";
            output SI.Temperature Tstar "normalization temperature";
          algorithm 
            if region == 1 then 
                        Tstar:=data.TSTAR1;
            elseif region == 2 then
              Tstar:=data.TSTAR2;


            else             Tstar:=data.TSTAR5;

            end if;
          end getTstar;
          function getpstar "get normalization pressure for region 1, 2 or 5"
            extends Modelica.Icons.Function;
            input Integer region "IF 97 region";
            output SI.Pressure pstar "normalization pressure";
          algorithm 
            if region == 1 then 
                        pstar:=data.PSTAR1;
            elseif region == 2 then
              pstar:=data.PSTAR2;


            else             pstar:=data.PSTAR5;

            end if;
          end getpstar;
          record critical "critical point data"
            extends Modelica.Icons.Record;
            constant SI.Pressure PCRIT = 22064000.0 "the critical pressure";
            constant SI.Temperature TCRIT = 647.096 "the critical temperature";
            constant SI.Density DCRIT = 322.0 "the critical density";
            constant SI.SpecificEnthalpy HCRIT = 2087546.84511715 "the calculated specific enthalpy at the critical point";
            constant SI.SpecificEntropy SCRIT = 4412.02148223476 "the calculated specific entropy at the critical point";
            annotation(Documentation(info = "<HTML>
 <h4>Record description</h4>
 <p>Critical point data for IF97 steam properties. SCRIT and HCRIT are calculated from helmholtz function for region 3 </p>
<h4>Version Info and Revision history
</h4>
<ul>
<li>First implemented: <i>July, 2000</i>
       by <a href=\"http://www.control.lth.se/~hubertus/\">Hubertus Tummescheit</a>
       </li>
</ul>
 <address>Author: Hubertus Tummescheit, <br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
 </address>
<ul>
 <li>Initial version: July 2000</li>
 <li>Documentation added: December 2002</li>
</ul>
</HTML>
"));
          end critical;
          record triple "triple point data"
            extends Modelica.Icons.Record;
            constant SI.Temperature Ttriple = 273.16 "the triple point temperature";
            constant SI.Pressure ptriple = 611.657 "the triple point temperature";
            constant SI.Density dltriple = 999.792520031618 "the triple point liquid density";
            constant SI.Density dvtriple = 0.00485457572477861 "the triple point vapour density";
            annotation(Documentation(info = "<HTML>
 <h4>Record description</h4>
 <p>Vapour/liquid/ice triple point data for IF97 steam properties.</p>
<h4>Version Info and Revision history
</h4>
<ul>
<li>First implemented: <i>July, 2000</i>
       by <a href=\"http://www.control.lth.se/~hubertus/\">Hubertus Tummescheit</a>
       </li>
</ul>
 <address>Author: Hubertus Tummescheit, <br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
 </address>
<ul>
 <li>Initial version: July 2000</li>
 <li>Documentation added: December 2002</li>
</ul>
</HTML>
"));
          end triple;
          package Regions "functions to find the current region for given pairs of input variables"
            extends Modelica.Icons.Library;
            annotation(Documentation(info = "<HTML><h4>Package description</h4>
 <p>Package <b>Regions</b> contains a large number of auxiliary functions which are neede to compute the current region
 of the IAPWS/IF97 for a given pair of input variables as quickly as possible. The focus of this implementation was on
 computational efficiency, not on compact code. Many of the function values calulated in these functions could be obtained
 using the fundamental functions of IAPWS/IF97, but with considerable overhead. If the region of IAPWS/IF97 is known in advance,
 the input variable mode can be set to the region, then the somewhat costly region checks are omitted.
 The checking for the phase has to be done outside the region functions because many properties are not
 differentiable at the region boundary. If the input phase is 2, the output region will be set to 4 immediately.</p>
 <h4>Package contents</h4>
 <p> The main 4 functions in this package are the functions returning the appropriate region for two input variables.
 <ul>
 <li>Function <b>region_ph</b> compute the region of IAPWS/IF97 for input pair pressure and specific enthalpy.</li>
 <li>Function <b>region_ps</b> compute the region of IAPWS/IF97 for input pair pressure and specific entropy</li>
 <li>Function <b>region_dT</b> compute the region of IAPWS/IF97 for input pair density and temperature.</li>
 <li>Function <b>region_pT</b> compute the region of IAPWS/IF97 for input pair pressure and temperature (only ine phase region).</li>
 </ul>
 <p>In addition, functions of the boiling and condensation curves compute the specific enthalpy, specific entropy, or density on these
 curves. The functions for the saturation pressure and temperature are included in the package <b>Basic</b> because they are part of
 the original <a href=\"Documentation/IF97documentation/IF97.pdf\">IAPWS/IF97 standards document</a>. These functions are also aliased to
 be used directly from package <b>Water</b>.
 </p>
 <ul>
 <li>Function <b>hl_p</b> computes the liquid specific enthalpy as a function of pressure. For overcritical pressures,
 the critical specific enthalpy is returned. An approximation is used for temperatures > 623.15 K.</li>
 <li>Function <b>hv_p</b> computes the vapour specific enthalpy as a function of pressure. For overcritical pressures,
 the critical specific enthalpy is returned. An approximation is used for temperatures > 623.15 K.</li>
 <li>Function <b>sl_p</b> computes the liquid specific entropy as a function of pressure. For overcritical pressures,
 the critical  specific entropy is returned. An approximation is used for temperatures > 623.15 K.</li>
 <li>Function <b>sv_p</b> computes the vapour  specific entropy as a function of pressure. For overcritical pressures,
 the critical  specific entropyis returned. An approximation is used for temperatures > 623.15 K.</li>
 <li>Function <b>rhol_T</b> computes the liquid density as a function of temperature. For overcritical temperatures,
 the critical density is returned. An approximation is used for temperatures > 623.15 K.</li>
 <li>Function <b>rhol_T</b> computes the vapour density as a function of temperature. For overcritical temperatures,
 the critical density is returned. An approximation is used for temperatures > 623.15 K.</li>
 </ul>
 </p>
 <p>All other functions are auxiliary functions called from the region functions to check a specific boundary.</p>
 <ul>
 <li>Function <b>boundary23ofT</b> computes the boundary pressure between regions 2 and 3 (input temperature)</li>
 <li>Function <b>boundary23ofp</b> computes the boundary temperature between regions 2 and 3 (input pressure)</li>
 <li>Function <b>hlowerofp5</b> computes the lower specific enthalpy limit of region 5 (input p, T=1073.15 K)</li>
 <li>Function <b>hupperofp5</b> computes the upper specific enthalpy limit of region 5 (input p, T=2273.15 K)</li>
 <li>Function <b>slowerofp5</b> computes the lower specific entropy limit of region 5 (input p, T=1073.15 K)</li>
 <li>Function <b>supperofp5</b> computes the upper specific entropy limit of region 5 (input p, T=2273.15 K)</li>
 <li>Function <b>hlowerofp1</b> computes the lower specific enthalpy limit of region 1 (input p, T=273.15 K)</li>
 <li>Function <b>hupperofp1</b> computes the upper specific enthalpy limit of region 1 (input p, T=623.15 K)</li>
 <li>Function <b>slowerofp1</b> computes the lower specific entropy limit of region 1 (input p, T=273.15 K)</li>
 <li>Function <b>supperofp1</b> computes the upper specific entropy limit of region 1 (input p, T=623.15 K)</li>
 <li>Function <b>hlowerofp2</b> computes the lower specific enthalpy limit of region 2 (input p, T=623.15 K)</li>
 <li>Function <b>hupperofp2</b> computes the upper specific enthalpy limit of region 2 (input p, T=1073.15 K)</li>
 <li>Function <b>slowerofp2</b> computes the lower specific entropy limit of region 2 (input p, T=623.15 K)</li>
 <li>Function <b>supperofp2</b> computes the upper specific entropy limit of region 2 (input p, T=1073.15 K)</li>
 <li>Function <b>d1n</b> computes the density in region 1 as function of pressure and temperature</li>
 <li>Function <b>d2n</b> computes the density in region 2 as function of pressure and temperature</li>
 <li>Function <b>dhot1ofp</b> computes the hot density limit of region 1 (input p, T=623.15 K)</li>
 <li>Function <b>dupper1ofT</b>computes the high pressure density limit of region 1 (input T, p=100MPa)</li>
 <li>Function <b>hl_p_R4b</b> computes a high accuracy approximation to the liquid enthalpy for temperatures > 623.15 K (input p)</li>
 <li>Function <b>hv_p_R4b</b> computes a high accuracy approximation to the vapour enthalpy for temperatures > 623.15 K (input p)</li>
 <li>Function <b>sl_p_R4b</b> computes a high accuracy approximation to the liquid entropy for temperatures > 623.15 K (input p)</li>
 <li>Function <b>sv_p_R4b</b> computes a high accuracy approximation to the vapour entropy for temperatures > 623.15 K (input p)</li>
 <li>Function <b>rhol_p_R4b</b> computes a high accuracy approximation to the liquid density for temperatures > 623.15 K (input p)</li>
 <li>Function <b>rhov_p_R4b</b> computes a high accuracy approximation to the vapour density for temperatures > 623.15 K (input p)</li>
 </ul>
 </p>
<h4>Version Info and Revision history
</h4>
 <ul>
<li>First implemented: <i>July, 2000</i>
       by <a href=\"http://www.control.lth.se/~hubertus/\">Hubertus Tummescheit</a>
       </li>
</ul>
<address>Authors: Hubertus Tummescheit, Jonas Eborn and Falko Jens Wagner<br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
 </address>
 <ul>
 <li>Initial version: July 2000</li>
 <li>Revised and extended for inclusion in Modelica.Thermal: December 2002</li>
</ul>
</HTML>
"));
            function boundary23ofT "boundary function for region boundary between regions 2 and 3 (input temperature)"
              extends Modelica.Icons.Function;
              input SI.Temperature t "temperature (K)";
              output SI.Pressure p "pressure";
            protected 
              constant Real[5] n = data.n;
            algorithm 
              p:=1000000.0 * (n[1] + t * (n[2] + t * n[3]));
            end boundary23ofT;
            function boundary23ofp "boundary function for region boundary between regions 2 and 3 (input pressure)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.Temperature t "temperature (K)";
            protected 
              constant Real[5] n = data.n;
              Real pi "dimensionless pressure";
            algorithm 
              pi:=p / 1000000.0;
              assert(p > triple.ptriple, "IF97 medium function boundary23ofp called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              t:=n[4] + ((pi - n[5]) / n[3]) ^ 0.5;
            end boundary23ofp;
            function hlowerofp5 "explicit lower specific enthalpy limit of region 5 as function of pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.SpecificEnthalpy h "specific enthalpy";
            protected 
              Real pi "dimensionless pressure";
            algorithm 
              pi:=p / data.PSTAR5;
              assert(p > triple.ptriple, "IF97 medium function hlowerofp5 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              h:=461526.0 * (9.01505286876203 + pi * ( -0.00979043490246092 + ( -2.03245575263501e-05 + 3.36540214679088e-07 * pi) * pi));
            end hlowerofp5;
            function hupperofp5 "explicit upper specific enthalpy limit of region 5 as function of pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.SpecificEnthalpy h "specific enthalpy";
            protected 
              Real pi "dimensionless pressure";
            algorithm 
              pi:=p / data.PSTAR5;
              assert(p > triple.ptriple, "IF97 medium function hupperofp5 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              h:=461526.0 * (15.9838891400332 + pi * ( -0.000489898813722568 + ( -5.01510211858761e-08 + 7.5006972718273e-08 * pi) * pi));
            end hupperofp5;
            function slowerofp5 "explicit lower specific entropy limit of region 5 as function of pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.SpecificEntropy s "specific entropy";
            protected 
              Real pi "dimensionless pressure";
            algorithm 
              pi:=p / data.PSTAR5;
              assert(p > triple.ptriple, "IF97 medium function slowerofp5 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              s:=461.526 * (18.4296209980112 + pi * ( -0.00730911805860036 + ( -1.68348072093888e-05 + 2.09066899426354e-07 * pi) * pi) - Modelica.Math.log(pi));
            end slowerofp5;
            function supperofp5 "explicit upper specific entropy limit of region 5 as function of pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.SpecificEntropy s "specific entropy";
            protected 
              Real pi "dimensionless pressure";
            algorithm 
              pi:=p / data.PSTAR5;
              assert(p > triple.ptriple, "IF97 medium function supperofp5 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              s:=461.526 * (22.7281531474243 + pi * ( -0.000656650220627603 + ( -1.96109739782049e-08 + 2.19979537113031e-08 * pi) * pi) - Modelica.Math.log(pi));
            end supperofp5;
            function hlowerofp1 "explicit lower specific enthalpy limit of region 1 as function of pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.SpecificEnthalpy h "specific enthalpy";
            protected 
              Real pi1 "dimensionless pressure";
              Real[3] o "vector of auxiliary variables";
            algorithm 
              pi1:=7.1 - p / data.PSTAR1;
              assert(p > triple.ptriple, "IF97 medium function hlowerofp1 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              o[1]:=pi1 * pi1;
              o[2]:=o[1] * o[1];
              o[3]:=o[2] * o[2];
              h:=639675.036 * (0.173379420894777 + pi1 * ( -0.022914084306349 + pi1 * ( -0.00017146768241932 + pi1 * ( -4.18695814670391e-06 + pi1 * ( -2.41630417490008e-07 + pi1 * (1.73545618580828e-11 + o[1] * pi1 * (8.43755552264362e-14 + o[2] * o[3] * pi1 * (5.35429206228374e-35 + o[1] * ( -8.12140581014818e-38 + o[1] * o[2] * ( -1.43870236842915e-44 + pi1 * (1.73894459122923e-45 + ( -7.06381628462585e-47 + 9.64504638626269e-49 * pi1) * pi1)))))))))));
            end hlowerofp1;
            function hupperofp1 "explicit upper specific enthalpy limit of region 1 as function of pressure (meets region 4 saturation pressure curve at 623.15 K)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.SpecificEnthalpy h "specific enthalpy";
            protected 
              Real pi1 "dimensionless pressure";
              Real[3] o "vector of auxiliary variables";
            algorithm 
              pi1:=7.1 - p / data.PSTAR1;
              assert(p > triple.ptriple, "IF97 medium function hupperofp1 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              o[1]:=pi1 * pi1;
              o[2]:=o[1] * o[1];
              o[3]:=o[2] * o[2];
              h:=639675.036 * (2.42896927729349 + pi1 * ( -0.00141131225285294 + pi1 * (0.00143759406818289 + pi1 * (0.000125338925082983 + pi1 * (1.23617764767172e-05 + pi1 * (3.17834967400818e-06 + o[1] * pi1 * (1.46754947271665e-08 + o[2] * o[3] * pi1 * (1.86779322717506e-17 + o[1] * ( -4.18568363667416e-19 + o[1] * o[2] * ( -9.19148577641497e-22 + pi1 * (4.27026404402408e-22 + ( -6.66749357417962e-23 + 3.49930466305574e-24 * pi1) * pi1)))))))))));
            end hupperofp1;
            function slowerofp1 "explicit lower specific entropy limit of region 1 as function of pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.SpecificEntropy s "specific entropy";
            protected 
              Real pi1 "dimensionless pressure";
              Real[3] o "vector of auxiliary variables";
            algorithm 
              pi1:=7.1 - p / data.PSTAR1;
              assert(p > triple.ptriple, "IF97 medium function slowerofp1 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              o[1]:=pi1 * pi1;
              o[2]:=o[1] * o[1];
              o[3]:=o[2] * o[2];
              s:=461.526 * ( -0.0268080988194267 + pi1 * (0.00834795890110168 + pi1 * ( -0.000486470924668433 + pi1 * ( -1.54902045012264e-05 + pi1 * ( -1.07631751351358e-06 + pi1 * (9.64159058957115e-11 + o[1] * pi1 * (4.81921078863103e-13 + o[2] * o[3] * pi1 * (2.7879623870968e-34 + o[1] * ( -4.22182957646226e-37 + o[1] * o[2] * ( -7.44601427465175e-44 + pi1 * (8.99540001407168e-45 + ( -3.65230274480299e-46 + 4.98464639687285e-48 * pi1) * pi1)))))))))));
            end slowerofp1;
            function supperofp1 "explicit upper specific entropy limit of region 1 as function of pressure (meets region 4 saturation pressure curve at 623.15 K)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.SpecificEntropy s "specific entropy";
            protected 
              Real pi1 "dimensionless pressure";
              Real[3] o "vector of auxiliary variables";
            algorithm 
              pi1:=7.1 - p / data.PSTAR1;
              assert(p > triple.ptriple, "IF97 medium function supperofp1 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              o[1]:=pi1 * pi1;
              o[2]:=o[1] * o[1];
              o[3]:=o[2] * o[2];
              s:=461.526 * (7.28316418503422 + pi1 * (0.070602197808399 + pi1 * (0.0039229343647356 + pi1 * (0.000313009170788845 + pi1 * (3.03619398631619e-05 + pi1 * (7.46739440045781e-06 + o[1] * pi1 * (3.40562176858676e-08 + o[2] * o[3] * pi1 * (4.21886233340801e-17 + o[1] * ( -9.44504571473549e-19 + o[1] * o[2] * ( -2.06859611434475e-21 + pi1 * (9.60758422254987e-22 + ( -1.49967810652241e-22 + 7.86863124555783e-24 * pi1) * pi1)))))))))));
            end supperofp1;
            function hlowerofp2 "explicit lower specific enthalpy limit of region 2 as function of pressure (meets region 4 saturation pressure curve at 623.15 K)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.SpecificEnthalpy h "specific enthalpy";
            protected 
              Real pi "dimensionless pressure";
              Real q1 "auxiliary variable";
              Real q2 "auxiliary variable";
              Real[18] o "vector of auxiliary variables";
            algorithm 
              pi:=p / data.PSTAR2;
              assert(p > triple.ptriple, "IF97 medium function hlowerofp2 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              q1:=572.54459862746 + 31.3220101646784 * ( -13.91883977887 + pi) ^ 0.5;
              q2:= -0.5 + 540.0 / q1;
              o[1]:=q1 * q1;
              o[2]:=o[1] * o[1];
              o[3]:=o[2] * o[2];
              o[4]:=pi * pi;
              o[5]:=o[4] * o[4];
              o[6]:=q2 * q2;
              o[7]:=o[6] * o[6];
              o[8]:=o[6] * o[7];
              o[9]:=o[5] * o[5];
              o[10]:=o[7] * o[7];
              o[11]:=o[9] * o[9];
              o[12]:=o[10] * o[10];
              o[13]:=o[12] * o[12];
              o[14]:=o[7] * q2;
              o[15]:=o[6] * q2;
              o[16]:=o[10] * o[6];
              o[17]:=o[13] * o[6];
              o[18]:=o[13] * o[6] * q2;
              h:=(4636975733.03507 + 3.74686560065793 * o[2] + 3.57966647812489e-06 * o[1] * o[2] + 2.81881548488163e-13 * o[3] - 76465233.2452145 * q1 - 0.00450789338787835 * o[2] * q1 - 1.55131504410292e-09 * o[1] * o[2] * q1 + o[1] * (2513837.07870341 - 4781981.98764471 * o[10] * o[11] * o[12] * o[13] * o[4] + 49.9651389369988 * o[11] * o[12] * o[13] * o[4] * o[5] * o[7] + o[15] * o[4] * (1.03746636552761e-13 - 0.00349547959376899 * o[16] - 2.55074501962569e-07 * o[8]) * o[9] + ( -242662.235426958 * o[10] * o[12] - 3.46022402653609 * o[16]) * o[4] * o[5] * pi + o[4] * (0.109336249381227 - 2248.08924686956 * o[14] - 354742.725841972 * o[17] - 24.1331193696374 * o[6]) * pi - 3.09081828396912e-19 * o[11] * o[12] * o[5] * o[7] * pi - 1.24107527851371e-08 * o[11] * o[13] * o[4] * o[5] * o[6] * o[7] * pi + 3.99891272904219 * o[5] * o[8] * pi + 0.0641817365250892 * o[10] * o[7] * o[9] * pi + pi * ( -4444.87643334512 - 75253.6156722047 * o[14] - 43051.9020511789 * o[6] - 22926.6247146068 * q2) + o[4] * ( -8.23252840892034 - 3927.0508365636 * o[15] - 239.325789467604 * o[18] - 76407.3727417716 * o[8] - 94.4508644545118 * q2) + 0.360567666582363 * o[5] * ( -0.0161221195808321 + q2) * (0.0338039844460968 + q2) + o[11] * ( -0.000584580992538624 * o[10] * o[12] * o[7] + 1332480.30241755 * o[12] * o[13] * q2) + o[9] * ( -73850273.6990986 * o[18] + 2.24425477627799e-05 * o[6] * o[7] * q2) + o[4] * o[5] * ( -208438767.026518 * o[17] - 1.24971648677697e-05 * o[6] - 8442.30378348203 * o[10] * o[6] * o[7] * q2) + o[11] * o[9] * (4.73594929247646e-22 * o[10] * o[12] * q2 - 13.6411358215175 * o[10] * o[12] * o[13] * q2 + 5.52427169406836e-10 * o[13] * o[6] * o[7] * q2) + o[11] * o[5] * (2.67174673301715e-06 * o[17] + 4.44545133805865e-18 * o[12] * o[6] * q2 - 50.2465185106411 * o[10] * o[13] * o[6] * o[7] * q2))) / o[1];
            end hlowerofp2;
            function hupperofp2 "explicit upper specific enthalpy limit of region 2 as function of pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.SpecificEnthalpy h "specific enthalpy";
            protected 
              Real pi "dimensionless pressure";
              Real[2] o "vector of auxiliary variables";
            algorithm 
              pi:=p / data.PSTAR2;
              assert(p > triple.ptriple, "IF97 medium function hupperofp2 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              o[1]:=pi * pi;
              o[2]:=o[1] * o[1] * o[1];
              h:=4160663.37647071 + pi * ( -4518.48617188327 + pi * ( -8.53409968320258 + pi * (0.109090430596056 + pi * ( -0.000172486052272327 + pi * (4.2261295097284e-15 + pi * ( -1.27295130636232e-10 + pi * ( -3.79407294691742e-25 + pi * (7.56960433802525e-23 + pi * (7.16825117265975e-32 + pi * (3.37267475986401e-21 + ( -7.5656940729795e-74 + o[1] * ( -8.00969737237617e-134 + (1.6746290980312e-65 + pi * ( -3.71600586812966e-69 + pi * (8.06630589170884e-129 + ( -1.76117969553159e-103 + 1.88543121025106e-84 * pi) * pi))) * o[1])) * o[2]))))))))));
            end hupperofp2;
            function slowerofp2 "explicit lower specific entropy limit of region 2 as function of pressure (meets region 4 saturation pressure curve at 623.15 K)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.SpecificEntropy s "specific entropy";
            protected 
              Real pi "dimensionless pressure";
              Real q1 "auxiliary variable";
              Real q2 "auxiliary variable";
              Real[40] o "vector of auxiliary variables";
            algorithm 
              pi:=p / data.PSTAR2;
              assert(p > triple.ptriple, "IF97 medium function slowerofp2 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              q1:=572.54459862746 + 31.3220101646784 * ( -13.91883977887 + pi) ^ 0.5;
              q2:= -0.5 + 540.0 / q1;
              o[1]:=pi * pi;
              o[2]:=o[1] * pi;
              o[3]:=o[1] * o[1];
              o[4]:=o[1] * o[3] * pi;
              o[5]:=q1 * q1;
              o[6]:=o[5] * q1;
              o[7]:=1 / o[5];
              o[8]:=1 / q1;
              o[9]:=o[5] * o[5];
              o[10]:=o[9] * q1;
              o[11]:=q2 * q2;
              o[12]:=o[11] * q2;
              o[13]:=o[1] * o[3];
              o[14]:=o[11] * o[11];
              o[15]:=o[3] * o[3];
              o[16]:=o[1] * o[15];
              o[17]:=o[11] * o[14];
              o[18]:=o[11] * o[14] * q2;
              o[19]:=o[3] * pi;
              o[20]:=o[14] * o[14];
              o[21]:=o[11] * o[20];
              o[22]:=o[15] * pi;
              o[23]:=o[14] * o[20] * q2;
              o[24]:=o[20] * o[20];
              o[25]:=o[15] * o[15];
              o[26]:=o[25] * o[3];
              o[27]:=o[14] * o[24];
              o[28]:=o[25] * o[3] * pi;
              o[29]:=o[20] * o[24] * q2;
              o[30]:=o[15] * o[25];
              o[31]:=o[24] * o[24];
              o[32]:=o[11] * o[31] * q2;
              o[33]:=o[14] * o[31];
              o[34]:=o[1] * o[25] * o[3] * pi;
              o[35]:=o[11] * o[14] * o[31] * q2;
              o[36]:=o[1] * o[25] * o[3];
              o[37]:=o[1] * o[25];
              o[38]:=o[20] * o[24] * o[31] * q2;
              o[39]:=o[14] * q2;
              o[40]:=o[11] * o[31];
              s:=461.526 * (9.692768600217 + 1.22151969114703e-16 * o[10] + 0.00018948987516315 * o[1] * o[11] + 1.6714766451061e-11 * o[12] * o[13] + 0.0039392777243355 * o[1] * o[14] - 1.0406965210174e-19 * o[14] * o[16] + 0.043797295650573 * o[1] * o[18] - 2.2922076337661e-06 * o[18] * o[19] - 2.0481737692309e-08 * o[2] + 3.227767723857e-05 * o[12] * o[2] + 0.0015033924542148 * o[17] * o[2] - 1.1256211360459e-11 * o[15] * o[20] + 1.0018179379511e-09 * o[11] * o[14] * o[16] * o[20] + 1.0234747095929e-13 * o[16] * o[21] - 1.9809712802088e-08 * o[22] * o[23] + 0.0021171472321355 * o[13] * o[24] - 8.9185845355421e-25 * o[26] * o[27] - 1.2790717852285e-08 * o[11] * o[3] - 4.8225372718507e-07 * o[12] * o[3] - 7.3087610595061e-29 * o[11] * o[20] * o[24] * o[30] - 0.10693031879409 * o[11] * o[24] * o[25] * o[31] + 4.2002467698208e-06 * o[24] * o[26] * o[31] - 5.5414715350778e-17 * o[20] * o[30] * o[31] + 9.436970724121e-07 * o[11] * o[20] * o[24] * o[30] * o[31] + 23.895741934104 * o[13] * o[32] + 0.040668253562649 * o[2] * o[32] - 3.0629316876232e-13 * o[26] * o[32] + 2.6674547914087e-05 * o[1] * o[33] + 8.2311340897998 * o[15] * o[33] + 1.2768608934681e-15 * o[34] * o[35] + 0.33662250574171 * o[37] * o[38] + 5.905956432427e-18 * o[4] + 0.038946842435739 * o[29] * o[4] - 4.88368302964335e-06 * o[5] - 3349017.34177133 / o[6] + 2.58538448402683e-09 * o[6] + 82839.5726841115 * o[7] - 5446.7940672972 * o[8] - 8.40318337484194e-13 * o[9] + 0.0017731742473213 * pi + 0.045996013696365 * o[11] * pi + 0.057581259083432 * o[12] * pi + 0.05032527872793 * o[17] * pi + o[8] * pi * (9.63082563787332 - 0.008917431146179 * q1) + 0.00811842799898148 * q1 + 3.3032641670203e-05 * o[1] * q2 - 4.3870667284435e-07 * o[2] * q2 + 8.0882908646985e-11 * o[14] * o[20] * o[24] * o[25] * q2 + 5.9056029685639e-26 * o[14] * o[24] * o[28] * q2 + 7.8847309559367e-10 * o[3] * q2 - 3.7826947613457e-06 * o[14] * o[24] * o[31] * o[36] * q2 + 1.2621808899101e-06 * o[11] * o[20] * o[4] * q2 + 540.0 * o[8] * (10.08665568018 - 3.3032641670203e-05 * o[1] - 6.2245802776607e-15 * o[10] - 0.015757110897342 * o[1] * o[12] - 5.0144299353183e-11 * o[11] * o[13] + 4.1627860840696e-19 * o[12] * o[16] - 0.306581069554011 * o[1] * o[17] + 9.0049690883672e-11 * o[15] * o[18] + 1.60454534363627e-05 * o[17] * o[19] + 4.3870667284435e-07 * o[2] - 9.683303171571e-05 * o[11] * o[2] + 2.57526266427144e-07 * o[14] * o[20] * o[22] - 1.40254511313154e-08 * o[16] * o[23] - 2.34560435076256e-09 * o[14] * o[20] * o[24] * o[25] - 1.24017662339842e-24 * o[27] * o[28] - 7.8847309559367e-10 * o[3] + 1.44676118155521e-06 * o[11] * o[3] + 1.90027787547159e-27 * o[29] * o[30] - 0.000960283724907132 * o[1] * o[32] - 296.320827232793 * o[15] * o[32] - 4.97975748452559e-14 * o[11] * o[14] * o[31] * o[34] + 2.21658861403112e-15 * o[30] * o[35] + 0.000200482822351322 * o[14] * o[24] * o[31] * o[36] - 19.1874828272775 * o[20] * o[24] * o[31] * o[37] - 5.47344301999018e-05 * o[30] * o[38] - 0.0090203547252888 * o[2] * o[39] - 1.38839897890111e-05 * o[21] * o[4] - 0.973671060893475 * o[20] * o[24] * o[4] - 836.35096769364 * o[13] * o[40] - 1.42338887469272 * o[2] * o[40] + 1.07202609066812e-11 * o[26] * o[40] + 1.50341259240398e-05 * o[5] - 1.8087714924605e-08 * o[6] + 18605.6518987296 * o[7] - 306.813232163376 * o[8] + 1.43632471334824e-11 * o[9] + 1.13103675106207e-18 * o[5] * o[9] - 0.017834862292358 * pi - 0.172743777250296 * o[11] * pi - 0.30195167236758 * o[39] * pi + o[8] * pi * ( -49.6756947920742 + 0.045996013696365 * q1) - 0.0003789797503263 * o[1] * q2 - 0.033874355714168 * o[11] * o[13] * o[14] * o[20] * q2 - 1.0234747095929e-12 * o[16] * o[20] * q2 + 1.78371690710842e-23 * o[11] * o[24] * o[26] * q2 + 2.558143570457e-08 * o[3] * q2 + 5.3465159397045 * o[24] * o[25] * o[31] * q2 - 0.000201611844951398 * o[11] * o[14] * o[20] * o[26] * o[31] * q2) - Modelica.Math.log(pi));
            end slowerofp2;
            function supperofp2 "explicit upper specific entropy limit of region 2 as function of pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.SpecificEntropy s "specific entropy";
            protected 
              Real pi "dimensionless pressure";
              Real[2] o "vector of auxiliary variables";
            algorithm 
              pi:=p / data.PSTAR2;
              assert(p > triple.ptriple, "IF97 medium function supperofp2 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              o[1]:=pi * pi;
              o[2]:=o[1] * o[1] * o[1];
              s:=8505.73409708683 - 461.526 * Modelica.Math.log(pi) + pi * ( -3.36563543302584 + pi * ( -0.00790283552165338 + pi * (9.15558349202221e-05 + pi * ( -1.59634706513e-07 + pi * (3.93449217595397e-18 + pi * ( -1.18367426347994e-13 + pi * (2.72575244843195e-15 + pi * (7.04803892603536e-26 + pi * (6.67637687381772e-35 + pi * (3.1377970315132e-24 + ( -7.04844558482265e-77 + o[1] * ( -7.46289531275314e-137 + (1.55998511254305e-68 + pi * ( -3.46166288915497e-72 + pi * (7.51557618628583e-132 + ( -1.64086406733212e-106 + 1.75648443097063e-87 * pi) * pi))) * o[1])) * o[2] * o[2]))))))))));
            end supperofp2;
            function d1n "density in region 1 as function of p and T"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.Temperature T "temperature (K)";
              output SI.Density d "density";
            protected 
              Real pi "dimensionless pressure";
              Real pi1 "dimensionless pressure";
              Real tau "dimensionless temperature";
              Real tau1 "dimensionless temperature";
              Real gpi "dimensionless Gibbs-derivative w.r.t. pi";
              Real[11] o "auxiliary variables";
            algorithm 
              pi:=p / data.PSTAR1;
              tau:=data.TSTAR1 / T;
              pi1:=7.1 - pi;
              tau1:=tau - 1.222;
              o[1]:=tau1 * tau1;
              o[2]:=o[1] * o[1];
              o[3]:=o[2] * o[2];
              o[4]:=o[1] * o[2];
              o[5]:=o[1] * tau1;
              o[6]:=o[2] * tau1;
              o[7]:=pi1 * pi1;
              o[8]:=o[7] * o[7];
              o[9]:=o[8] * o[8];
              o[10]:=o[3] * o[3];
              o[11]:=o[10] * o[10];
              gpi:=pi1 * (pi1 * ((9.5038934535162e-05 + o[2] * (8.4812393955936e-06 + 2.55615384360309e-09 * o[4])) / o[2] + pi1 * ((8.9701127632e-06 + (2.60684891582404e-06 + 5.7366919751696e-13 * o[2] * o[3]) * o[5]) / o[6] + pi1 * (2.02584984300585e-06 / o[3] + o[7] * pi1 * (o[8] * o[9] * pi1 * (o[7] * (o[7] * o[8] * ( -7.63737668221055e-22 / (o[1] * o[11] * o[2]) + pi1 * (pi1 * ( -5.65070932023524e-23 / (o[11] * o[3]) + (2.99318679335866e-24 * pi1) / (o[11] * o[3] * tau1)) + 3.5842867920213e-22 / (o[1] * o[11] * o[2] * tau1))) - 3.33001080055983e-19 / (o[1] * o[10] * o[2] * o[3] * tau1)) + 1.44400475720615e-17 / (o[10] * o[2] * o[3] * tau1)) + (1.01874413933128e-08 + 1.39398969845072e-09 * o[6]) / (o[1] * o[3] * tau1))))) + (0.00094368642146534 + o[5] * (0.00060003561586052 + ( -9.5322787813974e-05 + o[1] * (8.8283690661692e-06 + 1.45389992595188e-15 * o[1] * o[2] * o[3])) * tau1)) / o[5]) + ( -0.00028319080123804 + o[1] * (0.00060706301565874 + o[4] * (0.018990068218419 + tau1 * (0.032529748770505 + (0.021841717175414 + 5.283835796993e-05 * o[1]) * tau1)))) / (o[3] * tau1);
              d:=p / (data.RH2O * T * pi * gpi);
            end d1n;
            function d2n "density in region 2  as function of p and T"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.Temperature T "temperature (K)";
              output SI.Density d "density";
            protected 
              Real pi "dimensionless pressure";
              Real tau "dimensionless temperature";
              Real tau2 "dimensionless temperature";
              Real gpi "dimensionless Gibbs-derivative w.r.t. pi";
              Real[12] o "auxiliary variables";
            algorithm 
              pi:=p / data.PSTAR2;
              tau:=data.TSTAR2 / T;
              tau2:=tau - 0.5;
              o[1]:=tau2 * tau2;
              o[2]:=o[1] * tau2;
              o[3]:=o[1] * o[1];
              o[4]:=o[3] * o[3];
              o[5]:=o[4] * o[4];
              o[6]:=o[3] * o[4] * o[5] * tau2;
              o[7]:=o[3] * o[4] * tau2;
              o[8]:=o[1] * o[3] * o[4];
              o[9]:=pi * pi;
              o[10]:=o[9] * o[9];
              o[11]:=o[3] * o[5] * tau2;
              o[12]:=o[5] * o[5];
              gpi:=(1.0 + pi * ( -0.0017731742473213 + tau2 * ( -0.017834862292358 + tau2 * ( -0.045996013696365 + ( -0.057581259083432 - 0.05032527872793 * o[2]) * tau2)) + pi * (tau2 * ( -6.6065283340406e-05 + ( -0.0003789797503263 + o[1] * ( -0.007878555448671 + o[2] * ( -0.087594591301146 - 5.3349095828174e-05 * o[6]))) * tau2) + pi * (6.1445213076927e-08 + (1.31612001853305e-06 + o[1] * ( -9.683303171571e-05 + o[2] * ( -0.0045101773626444 - 0.122004760687947 * o[6]))) * tau2 + pi * (tau2 * ( -3.15389238237468e-09 + (5.116287140914e-08 + 1.92901490874028e-06 * tau2) * tau2) + pi * (1.14610381688305e-05 * o[1] * o[3] * tau2 + pi * (o[2] * ( -1.00288598706366e-10 + o[7] * ( -0.012702883392813 - 143.374451604624 * o[1] * o[5] * tau2)) + pi * ( -4.1341695026989e-17 + o[1] * o[4] * ( -8.8352662293707e-06 - 0.272627897050173 * o[8]) * tau2 + pi * (o[4] * (9.0049690883672e-11 - 65.8490727183984 * o[3] * o[4] * o[5]) + pi * (1.78287415218792e-07 * o[7] + pi * (o[3] * (1.0406965210174e-18 + o[1] * ( -1.0234747095929e-12 - 1.0018179379511e-08 * o[3]) * o[3]) + o[10] * o[9] * (( -1.29412653835176e-09 + 1.71088510070544 * o[11]) * o[6] + o[9] * ( -6.05920510335078 * o[12] * o[4] * o[5] * tau2 + o[9] * (o[3] * o[5] * (1.78371690710842e-23 + o[1] * o[3] * o[4] * (6.1258633752464e-12 - 8.4004935396416e-05 * o[7]) * tau2) + pi * ( -1.24017662339842e-24 * o[11] + pi * (8.32192847496054e-05 * o[12] * o[3] * o[5] * tau2 + pi * (o[1] * o[4] * o[5] * (1.75410265428146e-27 + (1.32995316841867e-15 - 2.26487297378904e-05 * o[1] * o[5]) * o[8]) * pi - 2.93678005497663e-14 * o[1] * o[12] * o[3] * tau2))))))))))))))))) / pi;
              d:=p / (data.RH2O * T * pi * gpi);
            end d2n;
            function dhot1ofp "density at upper temperature limit of region 1"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.Density d "density";
            protected 
              Real pi "dimensionless pressure";
              Real pi1 "dimensionless pressure";
              Real[4] o "auxiliary variables";
            algorithm 
              pi:=p / data.PSTAR1;
              pi1:=7.1 - pi;
              o[1]:=pi1 * pi1;
              o[2]:=o[1] * o[1];
              o[3]:=o[2] * o[2];
              o[4]:=o[3] * o[3];
              d:=57.4756752485113 / (0.0737412153522555 + 0.000102697173772229 * o[1] + 1.99080616601101e-06 * o[2] + 1.35549330686006e-17 * o[2] * o[4] - 3.11228834832975e-19 * o[1] * o[2] * o[4] - 7.02987180039442e-22 * o[2] * o[3] * o[4] - 5.17859076694812e-23 * o[1] * o[2] * o[3] * o[4] + 0.00145092247736023 * pi1 + 1.14683182476084e-05 * o[1] * pi1 + 1.13217858826367e-08 * o[1] * o[2] * pi1 + 3.29199117056433e-22 * o[2] * o[3] * o[4] * pi1 + 2.73712834080283e-24 * o[1] * o[2] * o[3] * o[4] * pi1);
            end dhot1ofp;
            function dupper1ofT "density at upper pressure limit of region 1"
              extends Modelica.Icons.Function;
              input SI.Temperature T "temperature (K)";
              output SI.Density d "density";
            protected 
              Real tau "dimensionless temperature";
              Real[4] o "auxiliary variables";
            algorithm 
              tau:=1386.0 / T;
              o[1]:=tau * tau;
              o[2]:=o[1] * o[1];
              o[3]:=o[2] * o[2];
              o[4]:=o[3] * o[3];
              d:=57.4756752485113 / (2.24144616859917 + 40.9288231166229 * o[1] + 106.47246463213 * o[2] + 88.4481480270751 * o[1] * o[2] + 31.3207272417546 * o[3] + 5.47811738891798 * o[1] * o[3] + 0.515626225030717 * o[2] * o[3] + 0.0274905057899089 * o[1] * o[2] * o[3] + 0.000853742979250503 * o[4] + 1.55932210492199e-05 * o[1] * o[4] + 1.6621051480279e-07 * o[2] * o[4] + 1.00606771839976e-09 * o[1] * o[2] * o[4] + 3.27598951831994e-12 * o[3] * o[4] + 5.20162317530099e-15 * o[1] * o[3] * o[4] + 3.33501889800275e-18 * o[2] * o[3] * o[4] + 5.50656040141221e-22 * o[1] * o[2] * o[3] * o[4] - 13.5354267762204 * tau - 78.3629702507642 * o[1] * tau - 109.374479648652 * o[2] * tau - 57.9035658513312 * o[1] * o[2] * tau - 14.215347150565 * o[3] * tau - 1.80906759985501 * o[1] * o[3] * tau - 0.127542214693871 * o[2] * o[3] * tau - 0.0051779458313163 * o[1] * o[2] * o[3] * tau - 0.000123304142684848 * o[4] * tau - 1.72405791469972e-06 * o[1] * o[4] * tau - 1.39155695911655e-08 * o[2] * o[4] * tau - 6.23333356847138e-11 * o[1] * o[2] * o[4] * tau - 1.44056015732082e-13 * o[3] * o[4] * tau - 1.50201626932938e-16 * o[1] * o[3] * o[4] * tau - 5.34588682252967e-20 * o[2] * o[3] * o[4] * tau - 2.73712834080283e-24 * o[1] * o[2] * o[3] * o[4] * tau);
            end dupper1ofT;
            function hl_p_R4b "explicit approximation of liquid specific enthalpy on the boundary between regions 4 and 3"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.SpecificEnthalpy h "specific enthalpy";
            protected 
              Real x "auxiliary variable";
            algorithm 
              x:=Modelica.Math.acos(p / data.PCRIT);
              h:=(1 + x * ( -0.494558695817518 + x * (1.3468000165649 + x * ( -3.88938815320975 + x * (6.67938547288793 + x * ( -6.75820241066552 + x * (3.5589197446565 + ( -0.717981855497894 - 0.000115203294561782 * x) * x))))))) * data.HCRIT;
            end hl_p_R4b;
            function hv_p_R4b "explicit approximation of vapour specific enthalpy on the boundary between regions 4 and 3"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.SpecificEnthalpy h "specific enthalpy";
            protected 
              Real x "auxiliary variable";
            algorithm 
              x:=Modelica.Math.acos(p / data.PCRIT);
              h:=(1 + x * (0.488015371865569 + x * (0.207967074625069 + x * ( -6.08412269842162 + x * (25.0888760229353 + x * ( -48.3821518026952 + x * (45.6648916483321 + ( -16.9855544296155 + 0.000661693646005769 * x) * x))))))) * data.HCRIT;
            end hv_p_R4b;
            function sl_p_R4b "explicit approximation of liquid specific entropy on the boundary between regions 4 and 3"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.SpecificEntropy s "specific entropy";
            protected 
              Real x "auxiliary variable";
            algorithm 
              x:=Modelica.Math.acos(p / data.PCRIT);
              s:=(1 + x * ( -0.361606922456481 + x * (0.996277863048665 + x * ( -2.85955481441711 + x * (4.90630115955533 + x * ( -4.97409230961421 + x * (2.62496516992045 + ( -0.531995437529902 - 8.06449743188064e-05 * x) * x))))))) * data.SCRIT;
            end sl_p_R4b;
            function sv_p_R4b "explicit approximation of vapour specific entropy on the boundary between regions 4 and 3"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.SpecificEntropy s;
            protected 
              Real x "auxiliary variable";
            algorithm 
              x:=Modelica.Math.acos(p / data.PCRIT);
              s:=(1 + x * (0.356826418266743 + x * (0.164245702781549 + x * ( -4.42535037742245 + x * (18.3244778599831 + x * ( -35.3386316259487 + x * (33.3618102581628 + ( -12.4087114905858 + 0.000481004983410923 * x) * x))))))) * data.SCRIT;
            end sv_p_R4b;
            function rhol_p_R4b "explicit approximation of liquid density on the boundary between regions 4 and 3"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.Density dl "liquid density";
            protected 
              Real x "auxiliary variable";
            algorithm 
              if p < data.PCRIT then 
                            x:=Modelica.Math.acos(p / data.PCRIT);
              dl:=(1 + x * (1.90322407909482 + x * ( -2.53148618024011 + x * ( -8.19144932384355 + x * (94.3419611677839 + x * ( -369.367683362338 + x * (796.662791059829 + x * ( -994.53853836007 + x * (673.25811770216 + ( -191.430773364052 + 0.00052536560808895 * x) * x))))))))) * data.DCRIT;

              else               dl:=data.DCRIT;

              end if;
            end rhol_p_R4b;
            function rhov_p_R4b "explicit approximation of vapour density on the boundary between regions 4 and 2"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.Density dv "vapour density";
            protected 
              Real x "auxiliary variable";
            algorithm 
              if p < data.PCRIT then 
                            x:=Modelica.Math.acos(p / data.PCRIT);
              dv:=(1 + x * ( -1.84638508033626 + x * ( -1.14478727188785 + x * (59.1870220307656 + x * ( -403.539143181161 + x * (1437.20072453324 + x * ( -3015.85354030752 + x * (3740.57903486701 + x * ( -2537.3758172539 + (725.876197580378 - 0.00111511116583323 * x) * x))))))))) * data.DCRIT;

              else               dv:=data.DCRIT;

              end if;
            end rhov_p_R4b;
            function boilingcurve_p "properties on the boiling curve"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output Common.IF97PhaseBoundaryProperties bpro "property record";
            protected 
              Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives";
              Common.HelmholtzDerivs f "dimensionless Helmholtz function and dervatives";
              SI.Pressure plim = min(p, data.PCRIT - 1e-07) "pressure limited to critical pressure - epsilon";
            algorithm 
              bpro.R:=data.RH2O;
              bpro.T:=Basic.tsat(plim);
              bpro.dpT:=Basic.dptofT(bpro.T);
              bpro.region3boundary:=bpro.T > data.TLIMIT1;
              if not bpro.region3boundary then 
                            g:=Basic.g1(p, bpro.T);
              bpro.d:=p / (bpro.R * bpro.T * g.pi * g.gpi);
              bpro.h:=if p > plim then data.HCRIT else bpro.R * bpro.T * g.tau * g.gtau;
              bpro.s:=g.R * (g.tau * g.gtau - g.g);
              bpro.cp:= -bpro.R * g.tau * g.tau * g.gtautau;
              bpro.vt:=bpro.R / p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
              bpro.vp:=(bpro.R * bpro.T) / (p * p) * g.pi * g.pi * g.gpipi;
              bpro.pt:= -(p / bpro.T * (g.gpi - g.tau * g.gtaupi)) / (g.gpipi * g.pi);
              bpro.pd:= -(bpro.R * bpro.T * g.gpi * g.gpi) / g.gpipi;

              else               bpro.d:=rhol_p_R4b(plim);
              f:=Basic.f3(bpro.d, bpro.T);
              bpro.h:=hl_p_R4b(plim);
              bpro.s:=f.R * (f.tau * f.ftau - f.f);
              bpro.cv:=bpro.R * ( -f.tau * f.tau * f.ftautau);
              bpro.pt:=bpro.R * bpro.d * f.delta * (f.fdelta - f.tau * f.fdeltatau);
              bpro.pd:=bpro.R * bpro.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);

              end if;
            end boilingcurve_p;
            function dewcurve_p "properties on the dew curve"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output Common.IF97PhaseBoundaryProperties bpro "property record";
            protected 
              Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives";
              Common.HelmholtzDerivs f "dimensionless Helmholtz function and dervatives";
              SI.Pressure plim = min(p, data.PCRIT - 1e-07) "pressure limited to critical pressure - epsilon";
            algorithm 
              bpro.R:=data.RH2O;
              bpro.T:=Basic.tsat(plim);
              bpro.dpT:=Basic.dptofT(bpro.T);
              bpro.region3boundary:=bpro.T > data.TLIMIT1;
              if not bpro.region3boundary then 
                            g:=Basic.g2(p, bpro.T);
              bpro.d:=p / (bpro.R * bpro.T * g.pi * g.gpi);
              bpro.h:=if p > plim then data.HCRIT else bpro.R * bpro.T * g.tau * g.gtau;
              bpro.s:=g.R * (g.tau * g.gtau - g.g);
              bpro.cp:= -bpro.R * g.tau * g.tau * g.gtautau;
              bpro.vt:=bpro.R / p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
              bpro.vp:=(bpro.R * bpro.T) / (p * p) * g.pi * g.pi * g.gpipi;
              bpro.pt:= -(p / bpro.T * (g.gpi - g.tau * g.gtaupi)) / (g.gpipi * g.pi);
              bpro.pd:= -(bpro.R * bpro.T * g.gpi * g.gpi) / g.gpipi;

              else               bpro.d:=rhov_p_R4b(plim);
              f:=Basic.f3(bpro.d, bpro.T);
              bpro.h:=hv_p_R4b(plim);
              bpro.s:=f.R * (f.tau * f.ftau - f.f);
              bpro.cv:=bpro.R * ( -f.tau * f.tau * f.ftautau);
              bpro.pt:=bpro.R * bpro.d * f.delta * (f.fdelta - f.tau * f.fdeltatau);
              bpro.pd:=bpro.R * bpro.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);

              end if;
            end dewcurve_p;
            function hvl_p
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input Common.IF97PhaseBoundaryProperties bpro "property record";
              output SI.SpecificEnthalpy h "specific enthalpy";
              annotation(derivative(noDerivative = bpro) = hvl_p_der, Inline = false, LateInline = true);
            algorithm 
              h:=bpro.h;
            end hvl_p;
            function hl_p "liquid specific enthalpy on the boundary between regions 4 and 3 or 1"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.SpecificEnthalpy h "specific enthalpy";
            algorithm 
              h:=hvl_p(p, boilingcurve_p(p));
            end hl_p;
            function hv_p "vapour specific enthalpy on the boundary between regions 4 and 3 or 2"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.SpecificEnthalpy h "specific enthalpy";
            algorithm 
              h:=hvl_p(p, dewcurve_p(p));
            end hv_p;
            function hvl_p_der "derivative function for the specific enthalpy along the phase boundary"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input Common.IF97PhaseBoundaryProperties bpro "property record";
              input Real p_der "derivative of pressure";
              output Real h_der "time derivative of specific enthalpy along the phase boundary";
            algorithm 
              if bpro.region3boundary then 
                            h_der:=((bpro.d * bpro.pd - bpro.T * bpro.pt) * p_der + (bpro.T * bpro.pt * bpro.pt + bpro.d * bpro.d * bpro.pd * bpro.cv) / bpro.dpT * p_der) / (bpro.pd * bpro.d * bpro.d);

              else               h_der:=(1 / bpro.d - bpro.T * bpro.vt) * p_der + bpro.cp / bpro.dpT * p_der;

              end if;
            end hvl_p_der;
            function rhovl_p
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input Common.IF97PhaseBoundaryProperties bpro "property record";
              output SI.Density rho "density";
              annotation(derivative(noDerivative = bpro) = rhovl_p_der, Inline = false, LateInline = true);
            algorithm 
              rho:=bpro.d;
            end rhovl_p;
            function rhol_p "density of saturated water"
              extends Modelica.Icons.Function;
              input SI.Pressure p "saturation pressure";
              output SI.Density rho "density of steam at the condensation point";
            algorithm 
              rho:=rhovl_p(p, boilingcurve_p(p));
            end rhol_p;
            function rhov_p "density of saturated vapour"
              extends Modelica.Icons.Function;
              input SI.Pressure p "saturation pressure";
              output SI.Density rho "density of steam at the condensation point";
            algorithm 
              rho:=rhovl_p(p, dewcurve_p(p));
            end rhov_p;
            function rhovl_p_der
              extends Modelica.Icons.Function;
              input SI.Pressure p "saturation pressure";
              input Common.IF97PhaseBoundaryProperties bpro "property record";
              input Real p_der "derivative of pressure";
              output Real d_der "time derivative of density along the phase boundary";
            algorithm 
              d_der:=if bpro.region3boundary then (p_der - (bpro.pt * p_der) / bpro.dpT) / bpro.pd else  -bpro.d * bpro.d * (bpro.vp + bpro.vt / bpro.dpT) * p_der;
            end rhovl_p_der;
            function sl_p "liquid specific entropy on the boundary between regions 4 and 3 or 1"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.SpecificEntropy s "specific entropy";
            protected 
              SI.Temperature Tsat "saturation temperature";
              SI.SpecificEnthalpy h "specific enthalpy";
            algorithm 
              if p < data.PLIMIT4A then 
                            Tsat:=Basic.tsat(p);
              (h,s):=Isentropic.handsofpT1(p, Tsat);
              elseif p < data.PCRIT then
                s:=sl_p_R4b(p);


              else               s:=data.SCRIT;

              end if;
            end sl_p;
            function sv_p "vapour specific entropy on the boundary between regions 4 and 3 or 2"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.SpecificEntropy s "specific entropy";
            protected 
              SI.Temperature Tsat "saturation temperature";
              SI.SpecificEnthalpy h "specific enthalpy";
            algorithm 
              if p < data.PLIMIT4A then 
                            Tsat:=Basic.tsat(p);
              (h,s):=Isentropic.handsofpT2(p, Tsat);
              elseif p < data.PCRIT then
                s:=sv_p_R4b(p);


              else               s:=data.SCRIT;

              end if;
            end sv_p;
            function rhol_T "density of saturated water"
              extends Modelica.Icons.Function;
              input SI.Temperature T "temperature";
              output SI.Density d "density of water at the boiling point";
            protected 
              SI.Pressure p "saturation pressure";
            algorithm 
              p:=Basic.psat(T);
              if T < data.TLIMIT1 then 
                            d:=d1n(p, T);
              elseif T < data.TCRIT then
                d:=rhol_p_R4b(p);


              else               d:=data.DCRIT;

              end if;
            end rhol_T;
            function rhov_T "density of saturated vapour"
              extends Modelica.Icons.Function;
              input SI.Temperature T "temperature";
              output SI.Density d "density of steam at the condensation point";
            protected 
              SI.Pressure p "saturation pressure";
            algorithm 
              p:=Basic.psat(T);
              if T < data.TLIMIT1 then 
                            d:=d2n(p, T);
              elseif T < data.TCRIT then
                d:=rhov_p_R4b(p);


              else               d:=data.DCRIT;

              end if;
            end rhov_T;
            function region_ph "return the current region (valid values: 1,2,3,4,5) in IF97 for given pressure and specific enthalpy"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.SpecificEnthalpy h "specific enthalpy";
              input Integer phase = 0 "phase: 2 for two-phase, 1 for one phase, 0 if not known";
              input Integer mode = 0 "mode: 0 means check, otherwise assume region=mode";
              output Integer region "region (valid values: 1,2,3,4,5) in IF97";
            protected 
              Boolean hsubcrit;
              SI.Temperature Ttest;
              constant Real[5] n = data.n;
              SI.SpecificEnthalpy hl "bubble enthalpy";
              SI.SpecificEnthalpy hv "dew enthalpy";
            algorithm 
              if mode <> 0 then 
                            region:=mode;

              else               hl:=hl_p(p);
              hv:=hv_p(p);
              if phase == 2 then 
                            region:=4;

              else               if p < triple.ptriple or p > data.PLIMIT1 or h < hlowerofp1(p) or p < 10000000.0 and h > hupperofp5(p) or p >= 10000000.0 and h > hupperofp2(p) then 
                            region:= -1;

              else               hsubcrit:=h < data.HCRIT;
              if p < data.PLIMIT4A then 
                            if hsubcrit then 
                            if phase == 1 then 
                            region:=1;

              else               if h < Isentropic.hofpT1(p, Basic.tsat(p)) then 
                            region:=1;

              else               region:=4;

              end if;

              end if;

              else               if h > hlowerofp5(p) then 
                            if p < data.PLIMIT5 and h < hupperofp5(p) then 
                            region:=5;

              else               region:= -2;

              end if;

              else               if phase == 1 then 
                            region:=2;

              else               if h > Isentropic.hofpT2(p, Basic.tsat(p)) then 
                            region:=2;

              else               region:=4;

              end if;

              end if;

              end if;

              end if;

              else               if hsubcrit then 
                            if h < hupperofp1(p) then 
                            region:=1;

              else               if h < hl or p > data.PCRIT then 
                            region:=3;

              else               region:=4;

              end if;

              end if;

              else               if h > hlowerofp2(p) then 
                            region:=2;

              else               if h > hv or p > data.PCRIT then 
                            region:=3;

              else               region:=4;

              end if;

              end if;

              end if;

              end if;

              end if;

              end if;

              end if;
            end region_ph;
            function region_ps "return the current region (valid values: 1,2,3,4,5) in IF97 for given pressure and specific entropy"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.SpecificEntropy s "specific entropy";
              input Integer phase = 0 "phase: 2 for two-phase, 1 for one phase, 0 if unknown";
              input Integer mode = 0 "mode: 0 means check, otherwise assume region=mode";
              output Integer region "region (valid values: 1,2,3,4,5) in IF97";
            protected 
              Boolean ssubcrit;
              SI.Temperature Ttest;
              constant Real[5] n = data.n;
              SI.SpecificEntropy sl "bubble entropy";
              SI.SpecificEntropy sv "dew entropy";
            algorithm 
              if mode <> 0 then 
                            region:=mode;

              else               sl:=sl_p(p);
              sv:=sv_p(p);
              if phase == 2 or phase == 0 and s > sl and s < sv and p < data.PCRIT then 
                            region:=4;

              else               region:=0;
              if p < triple.ptriple then 
                            region:= -2;

              else 
              end if;
              if p > data.PLIMIT1 then 
                            region:= -3;

              else 
              end if;
              if p < 10000000.0 and s > supperofp5(p) then 
                            region:= -5;

              else 
              end if;
              if p >= 10000000.0 and s > supperofp2(p) then 
                            region:= -6;

              else 
              end if;
              if region < 0 then 
                            assert(false, "region computation from p and s failed: function called outside the legal region");

              else               ssubcrit:=s < data.SCRIT;
              if p < data.PLIMIT4A then 
                            if ssubcrit then 
                            region:=1;

              else               if s > slowerofp5(p) then 
                            if p < data.PLIMIT5 and s < supperofp5(p) then 
                            region:=5;

              else               region:= -1;

              end if;

              else               region:=2;

              end if;

              end if;

              else               if ssubcrit then 
                            if s < supperofp1(p) then 
                            region:=1;

              else               if s < sl or p > data.PCRIT then 
                            region:=3;

              else               region:=4;

              end if;

              end if;

              else               if s > slowerofp2(p) then 
                            region:=2;

              else               if s > sv or p > data.PCRIT then 
                            region:=3;

              else               region:=4;

              end if;

              end if;

              end if;

              end if;

              end if;

              end if;

              end if;
            end region_ps;
            function region_pT "return the current region (valid values: 1,2,3,5) in IF97, given pressure and temperature"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.Temperature T "temperature (K)";
              input Integer mode = 0 "mode: 0 means check, otherwise assume region=mode";
              output Integer region "region (valid values: 1,2,3,5) in IF97, region 4 is impossible!";
            algorithm 
              if mode <> 0 then 
                            region:=mode;

              else               if p < data.PLIMIT4A then 
                            if T > data.TLIMIT2 then 
                            region:=5;
              elseif T > Basic.tsat(p) then
                region:=2;


              else               region:=1;

              end if;

              else               if T < data.TLIMIT1 then 
                            region:=1;
              elseif T < boundary23ofp(p) then
                region:=3;


              else               region:=2;

              end if;

              end if;

              end if;
            end region_pT;
            function region_dT "return the current region (valid values: 1,2,3,4,5) in IF97, given density and temperature"
              extends Modelica.Icons.Function;
              input SI.Density d "density";
              input SI.Temperature T "temperature (K)";
              input Integer phase = 0 "phase: 2 for two-phase, 1 for one phase, 0 if not known";
              input Integer mode = 0 "mode: 0 means check, otherwise assume region=mode";
              output Integer region "(valid values: 1,2,3,4,5) in IF97";
            protected 
              Boolean Tovercrit "flag if overcritical temperature";
              SI.Pressure p23 "pressure needed to know if region 2 or 3";
            algorithm 
              Tovercrit:=T > data.TCRIT;
              if mode <> 0 then 
                            region:=mode;

              else               p23:=boundary23ofT(T);
              if T > data.TLIMIT2 then 
                            if d < 20.5655874106483 then 
                            region:=5;

              else               assert(false, "out of valid region for IF97, pressure above region 5!");

              end if;
              elseif Tovercrit then
                if d > d2n(p23, T) and T > data.TLIMIT1 then 
                                region:=3;
                elseif T < data.TLIMIT1 then
                  region:=1;


                else                 region:=2;

                end if;

              elseif d > rhol_T(T) then
                if T < data.TLIMIT1 then 
                                region:=1;

                else                 region:=3;

                end if;

              elseif d < rhov_T(T) then
                if d > d2n(p23, T) and T > data.TLIMIT1 then 
                                region:=3;

                else                 region:=2;

                end if;


              else               region:=4;

              end if;

              end if;
            end region_dT;
            function hvl_dp "derivative function for the specific enthalpy along the phase boundary"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input Common.IF97PhaseBoundaryProperties bpro "property record";
              output Real dh_dp "derivative of specific enthalpy along the phase boundary";
            algorithm 
              if bpro.region3boundary then 
                            dh_dp:=(bpro.d * bpro.pd - bpro.T * bpro.pt + (bpro.T * bpro.pt * bpro.pt + bpro.d * bpro.d * bpro.pd * bpro.cv) / bpro.dpT) / (bpro.pd * bpro.d * bpro.d);

              else               dh_dp:=1 / bpro.d - bpro.T * bpro.vt + bpro.cp / bpro.dpT;

              end if;
            end hvl_dp;
            function dhl_dp "derivative of liquid specific enthalpy on the boundary between regions 4 and 3 or 1 w.r.t pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.DerEnthalpyByPressure dh_dp "specific enthalpy derivative w.r.t. pressure";
            algorithm 
              dh_dp:=hvl_dp(p, boilingcurve_p(p));
            end dhl_dp;
            function dhv_dp "derivative of vapour specific enthalpy on the boundary between regions 4 and 3 or 1 w.r.t pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.DerEnthalpyByPressure dh_dp "specific enthalpy derivative w.r.t. pressure";
            algorithm 
              dh_dp:=hvl_dp(p, dewcurve_p(p));
            end dhv_dp;
            function drhovl_dp
              extends Modelica.Icons.Function;
              input SI.Pressure p "saturation pressure";
              input Common.IF97PhaseBoundaryProperties bpro "property record";
              output Real dd_dp(unit = "kg/(m3.Pa)") "derivative of density along the phase boundary";
            algorithm 
              dd_dp:=if bpro.region3boundary then (1.0 - bpro.pt / bpro.dpT) / bpro.pd else  -bpro.d * bpro.d * (bpro.vp + bpro.vt / bpro.dpT);
            end drhovl_dp;
            function drhol_dp "derivative of density of saturated water w.r.t. pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "saturation pressure";
              output SI.DerDensityByPressure dd_dp "derivative of density of water at the boiling point";
            algorithm 
              dd_dp:=drhovl_dp(p, boilingcurve_p(p));
            end drhol_dp;
            function drhov_dp "derivative of density of saturated steam w.r.t. pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "saturation pressure";
              output SI.DerDensityByPressure dd_dp "derivative of density of water at the boiling point";
            algorithm 
              dd_dp:=drhovl_dp(p, dewcurve_p(p));
            end drhov_dp;
          end Regions;
          package Basic "Base functions as described in IAWPS/IF97"
            extends Modelica.Icons.Library;
            annotation(Documentation(info = "<HTML><h4>Package description</h4>
          <p>Package BaseIF97/Basic computes the the fundamental functions for the 5 regions of the steam tables
          as described in the standards document <a href=\"Documentation/IF97documentation/IF97.pdf\">IF97.pdf</a>. The code of these
          functions has been generated using <b><i>Mathematica</i></b> and the add-on packages \"Format\" and \"Optimize\"
          to generate highly efficient, expression-optimized C-code from a symbolic representation of the thermodynamic
          functions. The C-code has than been transformed into Modelica code. An important feature of this optimization was to
          simultaneously optimize the functions and the directional derivatives because they share many common subexpressions.</p>
          <h4>Package contents</h4>
          <p>
          <ul>
          <li>Function <b>g1</b> computes the dimensionless Gibbs function for region 1 and all derivatives up
          to order 2 w.r.t pi and tau. Inputs: p and T.</li>
          <li>Function <b>g2</b> computes the dimensionless Gibbs function  for region 2 and all derivatives up
          to order 2 w.r.t pi and tau. Inputs: p and T.</li>
          <li>Function <b>g2metastable</b> computes the dimensionless Gibbs function for metastable vapour
          (adjacent to region 2 but 2-phase at equilibrium) and all derivatives up
          to order 2 w.r.t pi and tau. Inputs: p and T.</li>
          <li>Function <b>f3</b> computes the dimensionless Helmholtz function  for region 3 and all derivatives up
          to order 2 w.r.t delta and tau. Inputs: d and T.</li>
          <li>Function <b>g5</b>computes the dimensionless Gibbs function for region 5 and all derivatives up
          to order 2 w.r.t pi and tau. Inputs: p and T.</li>
          <li>Function <b>tph1</b> computes the inverse function T(p,h) in region 1.</li>
          <li>Function <b>tph2</b> computes the inverse function T(p,h) in region 2.</li>
          <li>Function <b>tps2a</b> computes the inverse function T(p,s) in region 2a.</li>
          <li>Function <b>tps2b</b> computes the inverse function T(p,s) in region 2b.</li>
          <li>Function <b>tps2c</b> computes the inverse function T(p,s) in region 2c.</li>
          <li>Function <b>tps2</b> computes the inverse function T(p,s) in region 2.</li>
          <li>Function <b>tsat</b> computes the saturation temperature as a function of pressure.</li>
          <li>Function <b>dtsatofp</b> computes the derivative of the saturation temperature w.r.t. pressure as
          a function of pressure.</li>
          <li>Function <b>tsat_der</b> computes the Modelica derivative function of tsat.</li>
          <li>Function <b>psat</b> computes the saturation pressure as a function of temperature.</li>
          <li>Function <b>dptofT</b>  computes the derivative of the saturation pressure w.r.t. temperature as
          a function of temperature.</li>
          <li>Function <b>psat_der</b> computes the Modelica derivative function of psat.</li>
          </ul>
          </p>
          <h4>Version Info and Revision history
          </h4>
          <ul>
          <li>First implemented: <i>July, 2000</i>
          by <a href=\"http://www.control.lth.se/~hubertus/\">Hubertus Tummescheit</a>
          </li>
          </ul>
          <address>Author: Hubertus Tummescheit, <br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
          </address>
          <ul>
          <li>Initial version: July 2000</li>
          <li>Documentation added: December 2002</li>
          </ul>
          </HTML>
          "));
            function g1 "Gibbs function for region 1: g(p,T)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.Temperature T "temperature (K)";
              output Modelica.Media.Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives wrt pi and tau";
            protected 
              Real pi1 "dimensionless pressure";
              Real tau1 "dimensionless temperature";
              Real[45] o "vector of auxiliary variables";
              Real pl "auxiliary variable";
            algorithm 
              pl:=min(p, data.PCRIT - 1);
              assert(p > triple.ptriple, "IF97 medium function g1 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              assert(p <= 100000000.0, "IF97 medium function g1: the input pressure (= " + String(p) + " Pa) is higher than 100 Mpa");
              assert(T >= 273.15, "IF97 medium function g1: the temperature (= " + String(T) + " K)  is lower than 273.15 K!");
              g.p:=p;
              g.T:=T;
              g.R:=data.RH2O;
              g.pi:=p / data.PSTAR1;
              g.tau:=data.TSTAR1 / T;
              pi1:=7.1 - g.pi;
              tau1:= -1.222 + g.tau;
              o[1]:=tau1 * tau1;
              o[2]:=o[1] * o[1];
              o[3]:=o[2] * o[2];
              o[4]:=o[3] * tau1;
              o[5]:=1 / o[4];
              o[6]:=o[1] * o[2];
              o[7]:=o[1] * tau1;
              o[8]:=1 / o[7];
              o[9]:=o[1] * o[2] * o[3];
              o[10]:=1 / o[2];
              o[11]:=o[2] * tau1;
              o[12]:=1 / o[11];
              o[13]:=o[2] * o[3];
              o[14]:=1 / o[3];
              o[15]:=pi1 * pi1;
              o[16]:=o[15] * pi1;
              o[17]:=o[15] * o[15];
              o[18]:=o[17] * o[17];
              o[19]:=o[17] * o[18] * pi1;
              o[20]:=o[15] * o[17];
              o[21]:=o[3] * o[3];
              o[22]:=o[21] * o[21];
              o[23]:=o[22] * o[3] * tau1;
              o[24]:=1 / o[23];
              o[25]:=o[22] * o[3];
              o[26]:=1 / o[25];
              o[27]:=o[1] * o[2] * o[22] * tau1;
              o[28]:=1 / o[27];
              o[29]:=o[1] * o[2] * o[22];
              o[30]:=1 / o[29];
              o[31]:=o[1] * o[2] * o[21] * o[3] * tau1;
              o[32]:=1 / o[31];
              o[33]:=o[2] * o[21] * o[3] * tau1;
              o[34]:=1 / o[33];
              o[35]:=o[1] * o[3] * tau1;
              o[36]:=1 / o[35];
              o[37]:=o[1] * o[3];
              o[38]:=1 / o[37];
              o[39]:=1 / o[6];
              o[40]:=o[1] * o[22] * o[3];
              o[41]:=1 / o[40];
              o[42]:=1 / o[22];
              o[43]:=o[1] * o[2] * o[21] * o[3];
              o[44]:=1 / o[43];
              o[45]:=1 / o[13];
              g.g:=pi1 * (pi1 * (pi1 * (o[10] * ( -3.1679644845054e-05 + o[2] * ( -2.8270797985312e-06 - 8.5205128120103e-10 * o[6])) + pi1 * (o[12] * ( -2.2425281908e-06 + ( -6.5171222895601e-07 - 1.4341729937924e-13 * o[13]) * o[7]) + pi1 * ( -4.0516996860117e-07 * o[14] + o[16] * (( -1.2734301741641e-09 - 1.7424871230634e-10 * o[11]) * o[36] + o[19] * ( -6.8762131295531e-19 * o[34] + o[15] * (1.4478307828521e-20 * o[32] + o[20] * (2.6335781662795e-23 * o[30] + pi1 * ( -1.1947622640071e-23 * o[28] + pi1 * (1.8228094581404e-24 * o[26] - 9.3537087292458e-26 * o[24] * pi1))))))))) + o[8] * ( -0.00047184321073267 + o[7] * ( -0.00030001780793026 + (4.7661393906987e-05 + o[1] * ( -4.4141845330846e-06 - 7.2694996297594e-16 * o[9])) * tau1))) + o[5] * (0.00028319080123804 + o[1] * ( -0.00060706301565874 + o[6] * ( -0.018990068218419 + tau1 * ( -0.032529748770505 + ( -0.021841717175414 - 5.283835796993e-05 * o[1]) * tau1))))) + (0.14632971213167 + tau1 * ( -0.84548187169114 + tau1 * ( -3.756360367204 + tau1 * (3.3855169168385 + tau1 * ( -0.95791963387872 + tau1 * (0.15772038513228 + ( -0.016616417199501 + 0.00081214629983568 * tau1) * tau1)))))) / o[1];
              g.gpi:=pi1 * (pi1 * (o[10] * (9.5038934535162e-05 + o[2] * (8.4812393955936e-06 + 2.55615384360309e-09 * o[6])) + pi1 * (o[12] * (8.9701127632e-06 + (2.60684891582404e-06 + 5.7366919751696e-13 * o[13]) * o[7]) + pi1 * (2.02584984300585e-06 * o[14] + o[16] * ((1.01874413933128e-08 + 1.39398969845072e-09 * o[11]) * o[36] + o[19] * (1.44400475720615e-17 * o[34] + o[15] * ( -3.3300108005598e-19 * o[32] + o[20] * ( -7.6373766822106e-22 * o[30] + pi1 * (3.5842867920213e-22 * o[28] + pi1 * ( -5.6507093202352e-23 * o[26] + 2.99318679335866e-24 * o[24] * pi1))))))))) + o[8] * (0.00094368642146534 + o[7] * (0.00060003561586052 + ( -9.5322787813974e-05 + o[1] * (8.8283690661692e-06 + 1.45389992595188e-15 * o[9])) * tau1))) + o[5] * ( -0.00028319080123804 + o[1] * (0.00060706301565874 + o[6] * (0.018990068218419 + tau1 * (0.032529748770505 + (0.021841717175414 + 5.283835796993e-05 * o[1]) * tau1))));
              g.gpipi:=pi1 * (o[10] * ( -0.000190077869070324 + o[2] * ( -1.69624787911872e-05 - 5.1123076872062e-09 * o[6])) + pi1 * (o[12] * ( -2.69103382896e-05 + ( -7.8205467474721e-06 - 1.72100759255088e-12 * o[13]) * o[7]) + pi1 * ( -8.1033993720234e-06 * o[14] + o[16] * (( -7.131208975319e-08 - 9.757927889155e-09 * o[11]) * o[36] + o[19] * ( -2.8880095144123e-16 * o[34] + o[15] * (7.3260237612316e-18 * o[32] + o[20] * (2.13846547101895e-20 * o[30] + pi1 * ( -1.03944316968618e-20 * o[28] + pi1 * (1.69521279607057e-21 * o[26] - 9.2788790594118e-23 * o[24] * pi1))))))))) + o[8] * ( -0.00094368642146534 + o[7] * ( -0.00060003561586052 + (9.5322787813974e-05 + o[1] * ( -8.8283690661692e-06 - 1.45389992595188e-15 * o[9])) * tau1));
              g.gtau:=pi1 * (o[38] * ( -0.00254871721114236 + o[1] * (0.0042494411096112 + (0.018990068218419 + ( -0.021841717175414 - 0.00015851507390979 * o[1]) * o[1]) * o[6])) + pi1 * (o[10] * (0.00141552963219801 + o[2] * (4.7661393906987e-05 + o[1] * ( -1.32425535992538e-05 - 1.2358149370591e-14 * o[9]))) + pi1 * (o[12] * (0.000126718579380216 - 5.1123076872062e-09 * o[37]) + pi1 * (o[39] * (1.1212640954e-05 + (1.30342445791202e-06 - 1.4341729937924e-12 * o[13]) * o[7]) + pi1 * (3.2413597488094e-06 * o[5] + o[16] * ((1.40077319158051e-08 + 1.04549227383804e-09 * o[11]) * o[45] + o[19] * (1.9941018075704e-17 * o[44] + o[15] * ( -4.4882754268415e-19 * o[42] + o[20] * ( -1.00075970318621e-21 * o[28] + pi1 * (4.6595728296277e-22 * o[26] + pi1 * ( -7.2912378325616e-23 * o[24] + 3.8350205789908e-24 * o[41] * pi1))))))))))) + o[8] * ( -0.29265942426334 + tau1 * (0.84548187169114 + o[1] * (3.3855169168385 + tau1 * ( -1.91583926775744 + tau1 * (0.47316115539684 + ( -0.066465668798004 + 0.0040607314991784 * tau1) * tau1)))));
              g.gtautau:=pi1 * (o[36] * (0.0254871721114236 + o[1] * ( -0.033995528876889 + ( -0.037980136436838 - 0.00031703014781958 * o[2]) * o[6])) + pi1 * (o[12] * ( -0.005662118528792 + o[6] * ( -2.64851071985076e-05 - 1.97730389929456e-13 * o[9])) + pi1 * (( -0.00063359289690108 - 2.55615384360309e-08 * o[37]) * o[39] + pi1 * (pi1 * ( -2.91722377392842e-05 * o[38] + o[16] * (o[19] * ( -5.9823054227112e-16 * o[32] + o[15] * (o[20] * (3.9029628424262e-20 * o[26] + pi1 * ( -1.86382913185108e-20 * o[24] + pi1 * (2.98940751135026e-21 * o[41] - (1.61070864317613e-22 * pi1) / (o[1] * o[22] * o[3] * tau1)))) + 1.43624813658928e-17 / (o[22] * tau1))) + ( -1.68092782989661e-07 - 7.3184459168663e-09 * o[11]) / (o[2] * o[3] * tau1))) + ( -6.7275845724e-05 + ( -3.9102733737361e-06 - 1.29075569441316e-11 * o[13]) * o[7]) / (o[1] * o[2] * tau1))))) + o[10] * (0.87797827279002 + tau1 * ( -1.69096374338228 + o[7] * ( -1.91583926775744 + tau1 * (0.94632231079368 + ( -0.199397006394012 + 0.0162429259967136 * tau1) * tau1))));
              g.gtaupi:=o[38] * (0.00254871721114236 + o[1] * ( -0.0042494411096112 + ( -0.018990068218419 + (0.021841717175414 + 0.00015851507390979 * o[1]) * o[1]) * o[6])) + pi1 * (o[10] * ( -0.00283105926439602 + o[2] * ( -9.5322787813974e-05 + o[1] * (2.64851071985076e-05 + 2.4716298741182e-14 * o[9]))) + pi1 * (o[12] * ( -0.00038015573814065 + 1.53369230616185e-08 * o[37]) + pi1 * (o[39] * ( -4.4850563816e-05 + ( -5.2136978316481e-06 + 5.7366919751696e-12 * o[13]) * o[7]) + pi1 * ( -1.62067987440468e-05 * o[5] + o[16] * (( -1.12061855326441e-07 - 8.3639381907043e-09 * o[11]) * o[45] + o[19] * ( -4.1876137958978e-16 * o[44] + o[15] * (1.03230334817355e-17 * o[42] + o[20] * (2.90220313924001e-20 * o[28] + pi1 * ( -1.39787184888831e-20 * o[26] + pi1 * (2.2602837280941e-21 * o[24] - 1.22720658527705e-22 * o[41] * pi1))))))))));
            end g1;
            function g2 "Gibbs function for region 2: g(p,T)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.Temperature T "temperature (K)";
              output Modelica.Media.Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives wrt pi and tau";
            protected 
              Real tau2 "dimensionless temperature";
              Real[55] o "vector of auxiliary variables";
            algorithm 
              g.p:=p;
              g.T:=T;
              g.R:=data.RH2O;
              assert(p > triple.ptriple, "IF97 medium function g2 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              assert(p <= 100000000.0, "IF97 medium function g2: the input pressure (= " + String(p) + " Pa) is higher than 100 Mpa");
              assert(T >= 273.15, "IF97 medium function g2: the temperature (= " + String(T) + " K) is lower than 273.15 K!");
              assert(T <= 1073.15, "IF97 medium function g2: the input temperature (= " + String(T) + " K) is higher than the limit of 1073.15 K");
              g.pi:=p / data.PSTAR2;
              g.tau:=data.TSTAR2 / T;
              tau2:= -0.5 + g.tau;
              o[1]:=tau2 * tau2;
              o[2]:=o[1] * tau2;
              o[3]:= -0.05032527872793 * o[2];
              o[4]:= -0.057581259083432 + o[3];
              o[5]:=o[4] * tau2;
              o[6]:= -0.045996013696365 + o[5];
              o[7]:=o[6] * tau2;
              o[8]:= -0.017834862292358 + o[7];
              o[9]:=o[8] * tau2;
              o[10]:=o[1] * o[1];
              o[11]:=o[10] * o[10];
              o[12]:=o[11] * o[11];
              o[13]:=o[10] * o[11] * o[12] * tau2;
              o[14]:=o[1] * o[10] * tau2;
              o[15]:=o[10] * o[11] * tau2;
              o[16]:=o[1] * o[12] * tau2;
              o[17]:=o[1] * o[11] * tau2;
              o[18]:=o[1] * o[10] * o[11];
              o[19]:=o[10] * o[11] * o[12];
              o[20]:=o[1] * o[10];
              o[21]:=g.pi * g.pi;
              o[22]:=o[21] * o[21];
              o[23]:=o[21] * o[22];
              o[24]:=o[10] * o[12] * tau2;
              o[25]:=o[12] * o[12];
              o[26]:=o[11] * o[12] * o[25] * tau2;
              o[27]:=o[10] * o[12];
              o[28]:=o[1] * o[10] * o[11] * tau2;
              o[29]:=o[10] * o[12] * o[25] * tau2;
              o[30]:=o[1] * o[10] * o[25] * tau2;
              o[31]:=o[1] * o[11] * o[12];
              o[32]:=o[1] * o[12];
              o[33]:=g.tau * g.tau;
              o[34]:=o[33] * o[33];
              o[35]:= -5.3349095828174e-05 * o[13];
              o[36]:= -0.087594591301146 + o[35];
              o[37]:=o[2] * o[36];
              o[38]:= -0.007878555448671 + o[37];
              o[39]:=o[1] * o[38];
              o[40]:= -0.0003789797503263 + o[39];
              o[41]:=o[40] * tau2;
              o[42]:= -6.6065283340406e-05 + o[41];
              o[43]:=o[42] * tau2;
              o[44]:=5.7870447262208e-06 * tau2;
              o[45]:= -0.30195167236758 * o[2];
              o[46]:= -0.172743777250296 + o[45];
              o[47]:=o[46] * tau2;
              o[48]:= -0.09199202739273 + o[47];
              o[49]:=o[48] * tau2;
              o[50]:=o[1] * o[11];
              o[51]:=o[10] * o[11];
              o[52]:=o[11] * o[12] * o[25];
              o[53]:=o[10] * o[12] * o[25];
              o[54]:=o[1] * o[10] * o[25];
              o[55]:=o[11] * o[12] * tau2;
              g.g:=g.pi * ( -0.0017731742473213 + o[9] + g.pi * (tau2 * ( -3.3032641670203e-05 + ( -0.00018948987516315 + o[1] * ( -0.0039392777243355 + ( -0.043797295650573 - 2.6674547914087e-05 * o[13]) * o[2])) * tau2) + g.pi * (2.0481737692309e-08 + (4.3870667284435e-07 + o[1] * ( -3.227767723857e-05 + ( -0.0015033924542148 - 0.040668253562649 * o[13]) * o[2])) * tau2 + g.pi * (g.pi * (2.2922076337661e-06 * o[14] + g.pi * (( -1.6714766451061e-11 + o[15] * ( -0.0021171472321355 - 23.895741934104 * o[16])) * o[2] + g.pi * ( -5.905956432427e-18 + o[17] * ( -1.2621808899101e-06 - 0.038946842435739 * o[18]) + g.pi * (o[11] * (1.1256211360459e-11 - 8.2311340897998 * o[19]) + g.pi * (1.9809712802088e-08 * o[15] + g.pi * (o[10] * (1.0406965210174e-19 + ( -1.0234747095929e-13 - 1.0018179379511e-09 * o[10]) * o[20]) + o[23] * (o[13] * ( -8.0882908646985e-11 + 0.10693031879409 * o[24]) + o[21] * ( -0.33662250574171 * o[26] + o[21] * (o[27] * (8.9185845355421e-25 + (3.0629316876232e-13 - 4.2002467698208e-06 * o[15]) * o[28]) + g.pi * ( -5.9056029685639e-26 * o[24] + g.pi * (3.7826947613457e-06 * o[29] + g.pi * ( -1.2768608934681e-15 * o[30] + o[31] * (7.3087610595061e-29 + o[18] * (5.5414715350778e-17 - 9.436970724121e-07 * o[32])) * g.pi)))))))))))) + tau2 * ( -7.8847309559367e-10 + (1.2790717852285e-08 + 4.8225372718507e-07 * tau2) * tau2))))) + ( -0.00560879118302 + g.tau * (0.07145273881455 + g.tau * ( -0.4071049823928 + g.tau * (1.424081971444 + g.tau * ( -4.38395111945 + g.tau * ( -9.692768600217 + g.tau * (10.08665568018 + ( -0.2840863260772 + 0.02126846353307 * g.tau) * g.tau) + Modelica.Math.log(g.pi))))))) / (o[34] * g.tau);
              g.gpi:=(1.0 + g.pi * ( -0.0017731742473213 + o[9] + g.pi * (o[43] + g.pi * (6.1445213076927e-08 + (1.31612001853305e-06 + o[1] * ( -9.683303171571e-05 + ( -0.0045101773626444 - 0.122004760687947 * o[13]) * o[2])) * tau2 + g.pi * (g.pi * (1.14610381688305e-05 * o[14] + g.pi * (( -1.00288598706366e-10 + o[15] * ( -0.012702883392813 - 143.374451604624 * o[16])) * o[2] + g.pi * ( -4.1341695026989e-17 + o[17] * ( -8.8352662293707e-06 - 0.272627897050173 * o[18]) + g.pi * (o[11] * (9.0049690883672e-11 - 65.849072718398 * o[19]) + g.pi * (1.78287415218792e-07 * o[15] + g.pi * (o[10] * (1.0406965210174e-18 + ( -1.0234747095929e-12 - 1.0018179379511e-08 * o[10]) * o[20]) + o[23] * (o[13] * ( -1.29412653835176e-09 + 1.71088510070544 * o[24]) + o[21] * ( -6.0592051033508 * o[26] + o[21] * (o[27] * (1.78371690710842e-23 + (6.1258633752464e-12 - 8.4004935396416e-05 * o[15]) * o[28]) + g.pi * ( -1.24017662339842e-24 * o[24] + g.pi * (8.3219284749605e-05 * o[29] + g.pi * ( -2.93678005497663e-14 * o[30] + o[31] * (1.75410265428146e-27 + o[18] * (1.32995316841867e-15 - 2.26487297378904e-05 * o[32])) * g.pi)))))))))))) + tau2 * ( -3.15389238237468e-09 + (5.116287140914e-08 + 1.92901490874028e-06 * tau2) * tau2)))))) / g.pi;
              g.gpipi:=( -1.0 + o[21] * (o[43] + g.pi * (1.22890426153854e-07 + (2.6322400370661e-06 + o[1] * ( -0.00019366606343142 + ( -0.0090203547252888 - 0.244009521375894 * o[13]) * o[2])) * tau2 + g.pi * (g.pi * (4.5844152675322e-05 * o[14] + g.pi * (( -5.0144299353183e-10 + o[15] * ( -0.063514416964065 - 716.87225802312 * o[16])) * o[2] + g.pi * ( -2.48050170161934e-16 + o[17] * ( -5.3011597376224e-05 - 1.63576738230104 * o[18]) + g.pi * (o[11] * (6.303478361857e-10 - 460.94350902879 * o[19]) + g.pi * (1.42629932175034e-06 * o[15] + g.pi * (o[10] * (9.3662686891566e-18 + ( -9.2112723863361e-12 - 9.0163614415599e-08 * o[10]) * o[20]) + o[23] * (o[13] * ( -1.94118980752764e-08 + 25.6632765105816 * o[24]) + o[21] * ( -103.006486756963 * o[26] + o[21] * (o[27] * (3.389062123506e-22 + (1.16391404129682e-10 - 0.0015960937725319 * o[15]) * o[28]) + g.pi * ( -2.48035324679684e-23 * o[24] + g.pi * (0.00174760497974171 * o[29] + g.pi * ( -6.4609161209486e-13 * o[30] + o[31] * (4.0344361048474e-26 + o[18] * (3.05889228736295e-14 - 0.00052092078397148 * o[32])) * g.pi)))))))))))) + tau2 * ( -9.461677147124e-09 + (1.5348861422742e-07 + o[44]) * tau2))))) / o[21];
              g.gtau:=(0.0280439559151 + g.tau * ( -0.2858109552582 + g.tau * (1.2213149471784 + g.tau * ( -2.848163942888 + g.tau * (4.38395111945 + o[33] * (10.08665568018 + ( -0.5681726521544 + 0.06380539059921 * g.tau) * g.tau)))))) / (o[33] * o[34]) + g.pi * ( -0.017834862292358 + o[49] + g.pi * ( -3.3032641670203e-05 + ( -0.0003789797503263 + o[1] * ( -0.015757110897342 + ( -0.306581069554011 - 0.00096028372490713 * o[13]) * o[2])) * tau2 + g.pi * (4.3870667284435e-07 + o[1] * ( -9.683303171571e-05 + ( -0.0090203547252888 - 1.42338887469272 * o[13]) * o[2]) + g.pi * ( -7.8847309559367e-10 + g.pi * (1.60454534363627e-05 * o[20] + g.pi * (o[1] * ( -5.0144299353183e-11 + o[15] * ( -0.033874355714168 - 836.35096769364 * o[16])) + g.pi * (( -1.38839897890111e-05 - 0.97367106089347 * o[18]) * o[50] + g.pi * (o[14] * (9.0049690883672e-11 - 296.320827232793 * o[19]) + g.pi * (2.57526266427144e-07 * o[51] + g.pi * (o[2] * (4.1627860840696e-19 + ( -1.0234747095929e-12 - 1.40254511313154e-08 * o[10]) * o[20]) + o[23] * (o[19] * ( -2.34560435076256e-09 + 5.3465159397045 * o[24]) + o[21] * ( -19.1874828272775 * o[52] + o[21] * (o[16] * (1.78371690710842e-23 + (1.07202609066812e-11 - 0.000201611844951398 * o[15]) * o[28]) + g.pi * ( -1.24017662339842e-24 * o[27] + g.pi * (0.000200482822351322 * o[53] + g.pi * ( -4.9797574845256e-14 * o[54] + (1.90027787547159e-27 + o[18] * (2.21658861403112e-15 - 5.4734430199902e-05 * o[32])) * o[55] * g.pi)))))))))))) + (2.558143570457e-08 + 1.44676118155521e-06 * tau2) * tau2))));
              g.gtautau:=( -0.1682637354906 + g.tau * (1.429054776291 + g.tau * ( -4.8852597887136 + g.tau * (8.544491828664 + g.tau * ( -8.7679022389 + o[33] * ( -0.5681726521544 + 0.12761078119842 * g.tau) * g.tau))))) / (o[33] * o[34] * g.tau) + g.pi * ( -0.09199202739273 + ( -0.34548755450059 - 1.5097583618379 * o[2]) * tau2 + g.pi * ( -0.0003789797503263 + o[1] * ( -0.047271332692026 + ( -1.83948641732407 - 0.03360993037175 * o[13]) * o[2]) + g.pi * (( -0.00019366606343142 + ( -0.045101773626444 - 48.395221739552 * o[13]) * o[2]) * tau2 + g.pi * (2.558143570457e-08 + 2.89352236311042e-06 * tau2 + g.pi * (9.6272720618176e-05 * o[10] * tau2 + g.pi * (( -1.00288598706366e-10 + o[15] * ( -0.50811533571252 - 28435.9329015838 * o[16])) * tau2 + g.pi * (o[11] * ( -0.000138839897890111 - 23.3681054614434 * o[18]) * tau2 + g.pi * ((6.303478361857e-10 - 10371.2289531477 * o[19]) * o[20] + g.pi * (3.09031519712573e-06 * o[17] + g.pi * (o[1] * (1.24883582522088e-18 + ( -9.2112723863361e-12 - 1.823308647071e-07 * o[10]) * o[20]) + o[23] * (o[1] * o[11] * o[12] * ( -6.5676921821352e-08 + 261.979281045521 * o[24]) * tau2 + o[21] * ( -1074.49903832754 * o[1] * o[10] * o[12] * o[25] * tau2 + o[21] * ((3.389062123506e-22 + (3.6448887082716e-10 - 0.0094757567127157 * o[15]) * o[28]) * o[32] + g.pi * ( -2.48035324679684e-23 * o[16] + g.pi * (0.0104251067622687 * o[1] * o[12] * o[25] * tau2 + g.pi * (o[11] * o[12] * (4.750694688679e-26 + o[18] * (8.6446955947214e-14 - 0.0031198625213944 * o[32])) * g.pi - 1.89230784411972e-12 * o[10] * o[25] * tau2))))))))))))))));
              g.gtaupi:= -0.017834862292358 + o[49] + g.pi * ( -6.6065283340406e-05 + ( -0.0007579595006526 + o[1] * ( -0.031514221794684 + ( -0.61316213910802 - 0.00192056744981426 * o[13]) * o[2])) * tau2 + g.pi * (1.31612001853305e-06 + o[1] * ( -0.00029049909514713 + ( -0.0270610641758664 - 4.2701666240781 * o[13]) * o[2]) + g.pi * ( -3.15389238237468e-09 + g.pi * (8.0227267181813e-05 * o[20] + g.pi * (o[1] * ( -3.00865796119098e-10 + o[15] * ( -0.203246134285008 - 5018.1058061618 * o[16])) + g.pi * (( -9.7187928523078e-05 - 6.8156974262543 * o[18]) * o[50] + g.pi * (o[14] * (7.2039752706938e-10 - 2370.56661786234 * o[19]) + g.pi * (2.3177363978443e-06 * o[51] + g.pi * (o[2] * (4.1627860840696e-18 + ( -1.0234747095929e-11 - 1.40254511313154e-07 * o[10]) * o[20]) + o[23] * (o[19] * ( -3.7529669612201e-08 + 85.544255035272 * o[24]) + o[21] * ( -345.37469089099 * o[52] + o[21] * (o[16] * (3.5674338142168e-22 + (2.14405218133624e-10 - 0.004032236899028 * o[15]) * o[28]) + g.pi * ( -2.60437090913668e-23 * o[27] + g.pi * (0.0044106220917291 * o[53] + g.pi * ( -1.14534422144089e-12 * o[54] + (4.5606669011318e-26 + o[18] * (5.3198126736747e-14 - 0.00131362632479764 * o[32])) * o[55] * g.pi)))))))))))) + (1.0232574281828e-07 + o[44]) * tau2)));
            end g2;
            function g2metastable "Gibbs function for metastable part of region 2: g(p,T)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.Temperature T "temperature (K)";
              output Modelica.Media.Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives wrt pi and tau";
            protected 
              Real pi "dimensionless pressure";
              Real tau "dimensionless temperature";
              Real tau2 "dimensionless temperature";
              Real[27] o "vector of auxiliary variables";
            algorithm 
              assert(p > triple.ptriple, "IF97 medium function g2metastable called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              assert(p <= 100000000.0, "IF97 medium function g2metastable: the input pressure (= " + String(p) + " Pa) is higher than 100 Mpa");
              assert(T >= 273.15, "IF97 medium function g2metastable: the temperature (= " + String(T) + " K) is lower than 273.15 K!");
              assert(T <= 1073.15, "IF97 medium function g2metastable: the input temperature (= " + String(T) + " K) is higher than the limit of 1073.15 K");
              g.p:=p;
              g.T:=T;
              g.R:=data.RH2O;
              g.pi:=p / data.PSTAR2;
              g.tau:=data.TSTAR2 / T;
              tau2:= -0.5 + g.tau;
              o[1]:=tau2 * tau2;
              o[2]:=o[1] * tau2;
              o[3]:=o[1] * o[1];
              o[4]:=o[1] * o[3];
              o[5]:= -0.0040813178534455 * o[4];
              o[6]:= -0.072334555213245 + o[5];
              o[7]:=o[2] * o[6];
              o[8]:= -0.088223831943146 + o[7];
              o[9]:=o[1] * o[8];
              o[10]:=o[3] * o[3];
              o[11]:=o[10] * tau2;
              o[12]:=o[10] * o[3];
              o[13]:=o[1] * o[3] * tau2;
              o[14]:=g.tau * g.tau;
              o[15]:=o[14] * o[14];
              o[16]:= -0.015238081817394 * o[11];
              o[17]:= -0.106091843797284 + o[16];
              o[18]:=o[17] * o[4];
              o[19]:=0.0040195606760414 + o[18];
              o[20]:=o[19] * tau2;
              o[21]:=g.pi * g.pi;
              o[22]:= -0.0448944963879005 * o[4];
              o[23]:= -0.361672776066225 + o[22];
              o[24]:=o[2] * o[23];
              o[25]:= -0.176447663886292 + o[24];
              o[26]:=o[25] * tau2;
              o[27]:=o[3] * tau2;
              g.g:=g.pi * ( -0.0073362260186506 + o[9] + g.pi * (g.pi * (( -0.0063498037657313 - 0.086043093028588 * o[12]) * o[3] + g.pi * (o[13] * (0.007532158152277 - 0.0079238375446139 * o[2]) + o[11] * g.pi * ( -0.00022888160778447 - 0.002645650148281 * tau2))) + (0.0020097803380207 + ( -0.053045921898642 - 0.007619040908697 * o[11]) * o[4]) * tau2)) + ( -0.00560879118302 + g.tau * (0.07145273881455 + g.tau * ( -0.4071049823928 + g.tau * (1.424081971444 + g.tau * ( -4.38395111945 + g.tau * ( -9.6937268393049 + g.tau * (10.087275970006 + ( -0.2840863260772 + 0.02126846353307 * g.tau) * g.tau) + Modelica.Math.log(g.pi))))))) / (o[15] * g.tau);
              g.gpi:=(1.0 + g.pi * ( -0.0073362260186506 + o[9] + g.pi * (o[20] + g.pi * (( -0.0190494112971939 - 0.258129279085764 * o[12]) * o[3] + g.pi * (o[13] * (0.030128632609108 - 0.0316953501784556 * o[2]) + o[11] * g.pi * ( -0.00114440803892235 - 0.013228250741405 * tau2)))))) / g.pi;
              g.gpipi:=( -1.0 + o[21] * (o[20] + g.pi * (( -0.0380988225943878 - 0.516258558171528 * o[12]) * o[3] + g.pi * (o[13] * (0.090385897827324 - 0.0950860505353668 * o[2]) + o[11] * g.pi * ( -0.0045776321556894 - 0.05291300296562 * tau2))))) / o[21];
              g.gtau:=(0.0280439559151 + g.tau * ( -0.2858109552582 + g.tau * (1.2213149471784 + g.tau * ( -2.848163942888 + g.tau * (4.38395111945 + o[14] * (10.087275970006 + ( -0.5681726521544 + 0.06380539059921 * g.tau) * g.tau)))))) / (o[14] * o[15]) + g.pi * (o[26] + g.pi * (0.0020097803380207 + ( -0.371321453290494 - 0.121904654539152 * o[11]) * o[4] + g.pi * (( -0.0253992150629252 - 1.37668948845741 * o[12]) * o[2] + g.pi * ((0.052725107065939 - 0.079238375446139 * o[2]) * o[4] + o[10] * g.pi * ( -0.00205993447006023 - 0.02645650148281 * tau2)))));
              g.gtautau:=( -0.1682637354906 + g.tau * (1.429054776291 + g.tau * ( -4.8852597887136 + g.tau * (8.544491828664 + g.tau * ( -8.7679022389 + o[14] * ( -0.5681726521544 + 0.12761078119842 * g.tau) * g.tau))))) / (o[14] * o[15] * g.tau) + g.pi * ( -0.176447663886292 + o[2] * ( -1.4466911042649 - 0.448944963879005 * o[4]) + g.pi * (( -2.22792871974296 - 1.82856981808728 * o[11]) * o[27] + g.pi * (o[1] * ( -0.0761976451887756 - 20.6503423268611 * o[12]) + g.pi * ((0.316350642395634 - 0.713145379015251 * o[2]) * o[27] + o[13] * g.pi * ( -0.0164794757604818 - 0.23810851334529 * tau2)))));
              g.gtaupi:=o[26] + g.pi * (0.0040195606760414 + ( -0.742642906580988 - 0.243809309078304 * o[11]) * o[4] + g.pi * (( -0.0761976451887756 - 4.13006846537222 * o[12]) * o[2] + g.pi * ((0.210900428263756 - 0.316953501784556 * o[2]) * o[4] + o[10] * g.pi * ( -0.0102996723503012 - 0.13228250741405 * tau2))));
            end g2metastable;
            function f3 "Helmholtz function for region 3: f(d,T)"
              extends Modelica.Icons.Function;
              input SI.Density d "density";
              input SI.Temperature T "temperature (K)";
              output Modelica.Media.Common.HelmholtzDerivs f "dimensionless Helmholtz function and dervatives wrt delta and tau";
            protected 
              Real[40] o "vector of auxiliary variables";
            algorithm 
              f.T:=T;
              f.d:=d;
              f.R:=data.RH2O;
              f.tau:=data.TCRIT / T;
              f.delta:=if d == data.DCRIT and T == data.TCRIT then 1 - Modelica.Constants.eps else abs(d / data.DCRIT);
              o[1]:=f.tau * f.tau;
              o[2]:=o[1] * o[1];
              o[3]:=o[2] * f.tau;
              o[4]:=o[1] * f.tau;
              o[5]:=o[2] * o[2];
              o[6]:=o[1] * o[5] * f.tau;
              o[7]:=o[5] * f.tau;
              o[8]:= -0.64207765181607 * o[1];
              o[9]:=0.88521043984318 + o[8];
              o[10]:=o[7] * o[9];
              o[11]:= -1.1524407806681 + o[10];
              o[12]:=o[11] * o[2];
              o[13]:= -1.2654315477714 + o[12];
              o[14]:=o[1] * o[13];
              o[15]:=o[1] * o[2] * o[5] * f.tau;
              o[16]:=o[2] * o[5];
              o[17]:=o[1] * o[5];
              o[18]:=o[5] * o[5];
              o[19]:=o[1] * o[18] * o[2];
              o[20]:=o[1] * o[18] * o[2] * f.tau;
              o[21]:=o[18] * o[5];
              o[22]:=o[1] * o[18] * o[5];
              o[23]:=0.25116816848616 * o[2];
              o[24]:=0.078841073758308 + o[23];
              o[25]:=o[15] * o[24];
              o[26]:= -6.100523451393 + o[25];
              o[27]:=o[26] * f.tau;
              o[28]:=9.7944563083754 + o[27];
              o[29]:=o[2] * o[28];
              o[30]:= -1.70429417648412 + o[29];
              o[31]:=o[1] * o[30];
              o[32]:=f.delta * f.delta;
              o[33]:= -10.9153200808732 * o[1];
              o[34]:=13.2781565976477 + o[33];
              o[35]:=o[34] * o[7];
              o[36]:= -6.9146446840086 + o[35];
              o[37]:=o[2] * o[36];
              o[38]:= -2.5308630955428 + o[37];
              o[39]:=o[38] * f.tau;
              o[40]:=o[18] * o[5] * f.tau;
              f.f:= -15.732845290239 + f.tau * (20.944396974307 + ( -7.6867707878716 + o[3] * (2.6185947787954 + o[4] * ( -2.808078114862 + o[1] * (1.2053369696517 - 0.0084566812812502 * o[6])))) * f.tau) + f.delta * (o[14] + f.delta * (0.38493460186671 + o[1] * ( -0.85214708824206 + o[2] * (4.8972281541877 + ( -3.0502617256965 + o[15] * (0.039420536879154 + 0.12558408424308 * o[2])) * f.tau)) + f.delta * ( -0.2799932969871 + o[1] * (1.389979956946 + o[1] * ( -2.018991502357 + o[16] * ( -0.0082147637173963 - 0.47596035734923 * o[17]))) + f.delta * (0.0439840744735 + o[1] * ( -0.44476435428739 + o[1] * (0.90572070719733 + 0.70522450087967 * o[19])) + f.delta * (f.delta * ( -0.022175400873096 + o[1] * (0.094260751665092 + 0.16436278447961 * o[21]) + f.delta * ( -0.013503372241348 * o[1] + f.delta * ( -0.014834345352472 * o[22] + f.delta * (o[1] * (0.00057922953628084 + 0.0032308904703711 * o[21]) + f.delta * (8.0964802996215e-05 - 4.4923899061815e-05 * f.delta * o[22] - 0.00016557679795037 * f.tau))))) + (0.10770512626332 + o[1] * ( -0.32913623258954 - 0.50871062041158 * o[20])) * f.tau))))) + 1.0658070028513 * Modelica.Math.log(f.delta);
              f.fdelta:=(1.0658070028513 + f.delta * (o[14] + f.delta * (0.76986920373342 + o[31] + f.delta * ( -0.8399798909613 + o[1] * (4.169939870838 + o[1] * ( -6.056974507071 + o[16] * ( -0.0246442911521889 - 1.42788107204769 * o[17]))) + f.delta * (0.175936297894 + o[1] * ( -1.77905741714956 + o[1] * (3.6228828287893 + 2.82089800351868 * o[19])) + f.delta * (f.delta * ( -0.133052405238576 + o[1] * (0.56556450999055 + 0.98617670687766 * o[21]) + f.delta * ( -0.094523605689436 * o[1] + f.delta * ( -0.118674762819776 * o[22] + f.delta * (o[1] * (0.0052130658265276 + 0.0290780142333399 * o[21]) + f.delta * (0.00080964802996215 - 0.00049416288967996 * f.delta * o[22] - 0.0016557679795037 * f.tau))))) + (0.5385256313166 + o[1] * ( -1.6456811629477 - 2.5435531020579 * o[20])) * f.tau)))))) / f.delta;
              f.fdeltadelta:=( -1.0658070028513 + o[32] * (0.76986920373342 + o[31] + f.delta * ( -1.6799597819226 + o[1] * (8.339879741676 + o[1] * ( -12.113949014142 + o[16] * ( -0.049288582304378 - 2.85576214409538 * o[17]))) + f.delta * (0.527808893682 + o[1] * ( -5.3371722514487 + o[1] * (10.868648486368 + 8.462694010556 * o[19])) + f.delta * (f.delta * ( -0.66526202619288 + o[1] * (2.82782254995276 + 4.9308835343883 * o[21]) + f.delta * ( -0.56714163413662 * o[1] + f.delta * ( -0.83072333973843 * o[22] + f.delta * (o[1] * (0.04170452661222 + 0.232624113866719 * o[21]) + f.delta * (0.0072868322696594 - 0.0049416288967996 * f.delta * o[22] - 0.0149019118155333 * f.tau))))) + (2.1541025252664 + o[1] * ( -6.5827246517908 - 10.1742124082316 * o[20])) * f.tau))))) / o[32];
              f.ftau:=20.944396974307 + ( -15.3735415757432 + o[3] * (18.3301634515678 + o[4] * ( -28.08078114862 + o[1] * (14.4640436358204 - 0.194503669468755 * o[6])))) * f.tau + f.delta * (o[39] + f.delta * (f.tau * ( -1.70429417648412 + o[2] * (29.3833689251262 + ( -21.3518320798755 + o[15] * (0.86725181134139 + 3.2651861903201 * o[2])) * f.tau)) + f.delta * ((2.779959913892 + o[1] * ( -8.075966009428 + o[16] * ( -0.131436219478341 - 12.37496929108 * o[17]))) * f.tau + f.delta * (( -0.88952870857478 + o[1] * (3.6228828287893 + 18.3358370228714 * o[19])) * f.tau + f.delta * (0.10770512626332 + o[1] * ( -0.98740869776862 - 13.2264761307011 * o[20]) + f.delta * ((0.188521503330184 + 4.2734323964699 * o[21]) * f.tau + f.delta * ( -0.027006744482696 * f.tau + f.delta * ( -0.38569297916427 * o[40] + f.delta * (f.delta * ( -0.00016557679795037 - 0.00116802137560719 * f.delta * o[40]) + (0.00115845907256168 + 0.084003152229649 * o[21]) * f.tau)))))))));
              f.ftautau:= -15.3735415757432 + o[3] * (109.980980709407 + o[4] * ( -252.72703033758 + o[1] * (159.104479994024 - 4.2790807283126 * o[6]))) + f.delta * ( -2.5308630955428 + o[2] * ( -34.573223420043 + (185.894192367068 - 174.645121293971 * o[1]) * o[7]) + f.delta * ( -1.70429417648412 + o[2] * (146.916844625631 + ( -128.110992479253 + o[15] * (18.2122880381691 + 81.629654758002 * o[2])) * f.tau) + f.delta * (2.779959913892 + o[1] * ( -24.227898028284 + o[16] * ( -1.97154329217511 - 309.374232277 * o[17])) + f.delta * ( -0.88952870857478 + o[1] * (10.868648486368 + 458.39592557179 * o[19]) + f.delta * (f.delta * (0.188521503330184 + 106.835809911747 * o[21] + f.delta * ( -0.027006744482696 + f.delta * ( -9.6423244791068 * o[21] + f.delta * (0.00115845907256168 + 2.10007880574121 * o[21] - 0.0292005343901797 * o[21] * o[32])))) + ( -1.97481739553724 - 330.66190326753 * o[20]) * f.tau)))));
              f.fdeltatau:=o[39] + f.delta * (f.tau * ( -3.4085883529682 + o[2] * (58.766737850252 + ( -42.703664159751 + o[15] * (1.73450362268278 + 6.5303723806402 * o[2])) * f.tau)) + f.delta * ((8.339879741676 + o[1] * ( -24.227898028284 + o[16] * ( -0.39430865843502 - 37.12490787324 * o[17]))) * f.tau + f.delta * (( -3.5581148342991 + o[1] * (14.4915313151573 + 73.343348091486 * o[19])) * f.tau + f.delta * (0.5385256313166 + o[1] * ( -4.9370434888431 - 66.132380653505 * o[20]) + f.delta * ((1.1311290199811 + 25.6405943788192 * o[21]) * f.tau + f.delta * ( -0.189047211378872 * f.tau + f.delta * ( -3.08554383331418 * o[40] + f.delta * (f.delta * ( -0.0016557679795037 - 0.0128482351316791 * f.delta * o[40]) + (0.0104261316530551 + 0.75602837006684 * o[21]) * f.tau))))))));
            end f3;
            function g5 "base function for region 5: g(p,T)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.Temperature T "temperature (K)";
              output Modelica.Media.Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives wrt pi and tau";
            protected 
              Real[11] o "vector of auxiliary variables";
            algorithm 
              assert(p > triple.ptriple, "IF97 medium function g5 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              assert(p <= data.PLIMIT5, "IF97 medium function g5: input pressure (= " + String(p) + " Pa) is higher than 10 Mpa in region 5");
              assert(T <= 2273.15, "IF97 medium function g5: input temperature (= " + String(T) + " K) is higher than limit of 2273.15K in region 5");
              g.p:=p;
              g.T:=T;
              g.R:=data.RH2O;
              g.pi:=p / data.PSTAR5;
              g.tau:=data.TSTAR5 / T;
              o[1]:=g.tau * g.tau;
              o[2]:= -0.004594282089991 * o[1];
              o[3]:=0.0021774678714571 + o[2];
              o[4]:=o[3] * g.tau;
              o[5]:=o[1] * g.tau;
              o[6]:=o[1] * o[1];
              o[7]:=o[6] * o[6];
              o[8]:=o[7] * g.tau;
              o[9]:= -7.9449656719138e-06 * o[8];
              o[10]:=g.pi * g.pi;
              o[11]:= -0.013782846269973 * o[1];
              g.g:=g.pi * ( -0.00012563183589592 + o[4] + g.pi * ( -3.9724828359569e-06 * o[8] + 1.2919228289784e-07 * o[5] * g.pi)) + ( -0.024805148933466 + g.tau * (0.36901534980333 + g.tau * ( -3.1161318213925 + g.tau * ( -13.179983674201 + (6.8540841634434 - 0.32961626538917 * g.tau) * g.tau + Modelica.Math.log(g.pi))))) / o[5];
              g.gpi:=(1.0 + g.pi * ( -0.00012563183589592 + o[4] + g.pi * (o[9] + 3.8757684869352e-07 * o[5] * g.pi))) / g.pi;
              g.gpipi:=( -1.0 + o[10] * (o[9] + 7.7515369738704e-07 * o[5] * g.pi)) / o[10];
              g.gtau:=g.pi * (0.0021774678714571 + o[11] + g.pi * ( -3.5752345523612e-05 * o[7] + 3.8757684869352e-07 * o[1] * g.pi)) + (0.074415446800398 + g.tau * ( -0.73803069960666 + (3.1161318213925 + o[1] * (6.8540841634434 - 0.65923253077834 * g.tau)) * g.tau)) / o[6];
              g.gtautau:=( -0.297661787201592 + g.tau * (2.21409209881998 + ( -6.232263642785 - 0.65923253077834 * o[5]) * g.tau)) / (o[6] * g.tau) + g.pi * ( -0.027565692539946 * g.tau + g.pi * ( -0.000286018764188897 * o[1] * o[6] * g.tau + 7.7515369738704e-07 * g.pi * g.tau));
              g.gtaupi:=0.0021774678714571 + o[11] + g.pi * ( -7.1504691047224e-05 * o[7] + 1.16273054608056e-06 * o[1] * g.pi);
            end g5;
            function gibbs "Gibbs function for region 1, 2 or 5: g(p,T,region)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.Temperature T "temperature (K)";
              input Integer region "IF97 region, 1, 2 or 5";
              output Real g "dimensionless Gibbs funcion";
            protected 
              Modelica.Media.Common.GibbsDerivs gibbs "dimensionless Gibbs funcion and dervatives wrt pi and tau";
            algorithm 
              assert(region == 1 or region == 2 or region == 5, "IF97 medium function gibbs called with wrong region (= " + String(region) + ").\n" + "Only regions 1, 2 or 5 are possible");
              if region == 1 then 
                            gibbs:=g1(p, T);
              elseif region == 2 then
                gibbs:=g2(p, T);


              else               gibbs:=g5(p, T);

              end if;
              g:=gibbs.g;
            end gibbs;
            function g1pitau "derivative of g wrt pi and tau"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.Temperature T "temperature (K)";
              output Real pi "dimensionless pressure";
              output Real tau "dimensionless temperature";
              output Real gpi "dimensionless dervative of Gibbs function wrt pi";
              output Real gtau "dimensionless dervative of Gibbs function wrt tau";
            protected 
              Real pi1 "dimensionless pressure";
              Real tau1 "dimensionless temperature";
              Real[28] o "vector of auxiliary variables";
            algorithm 
              assert(p > triple.ptriple, "IF97 medium function g1pitau called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              assert(p <= 100000000.0, "IF97 medium function g1pitau: the input pressure (= " + String(p) + " Pa) is higher than 100 Mpa");
              assert(T >= 273.15, "IF97 medium function g1pitau: the temperature (= " + String(T) + " K) is lower than 273.15 K!");
              pi:=p / data.PSTAR1;
              tau:=data.TSTAR1 / T;
              pi1:=7.1 - pi;
              tau1:= -1.222 + tau;
              o[1]:=tau1 * tau1;
              o[2]:=o[1] * tau1;
              o[3]:=1 / o[2];
              o[4]:=o[1] * o[1];
              o[5]:=o[4] * o[4];
              o[6]:=o[1] * o[5];
              o[7]:=o[1] * o[4];
              o[8]:=1 / o[4];
              o[9]:=o[1] * o[4] * o[5];
              o[10]:=o[4] * tau1;
              o[11]:=1 / o[10];
              o[12]:=o[4] * o[5];
              o[13]:=o[5] * tau1;
              o[14]:=1 / o[13];
              o[15]:=pi1 * pi1;
              o[16]:=o[15] * pi1;
              o[17]:=o[15] * o[15];
              o[18]:=o[17] * o[17];
              o[19]:=o[17] * o[18] * pi1;
              o[20]:=o[15] * o[17];
              o[21]:=o[5] * o[5];
              o[22]:=o[21] * o[21];
              o[23]:=o[22] * o[5] * tau1;
              o[24]:=1 / o[23];
              o[25]:=o[22] * o[5];
              o[26]:=1 / o[25];
              o[27]:=o[1] * o[22] * o[4] * tau1;
              o[28]:=1 / o[27];
              gtau:=pi1 * (( -0.00254871721114236 + o[1] * (0.00424944110961118 + (0.018990068218419 + ( -0.021841717175414 - 0.00015851507390979 * o[1]) * o[1]) * o[7])) / o[6] + pi1 * (o[8] * (0.00141552963219801 + o[4] * (4.7661393906987e-05 + o[1] * ( -1.32425535992538e-05 - 1.2358149370591e-14 * o[9]))) + pi1 * (o[11] * (0.000126718579380216 - 5.11230768720618e-09 * o[6]) + pi1 * ((1.1212640954e-05 + (1.30342445791202e-06 - 1.4341729937924e-12 * o[12]) * o[2]) / o[7] + pi1 * (3.24135974880936e-06 * o[14] + o[16] * ((1.40077319158051e-08 + 1.04549227383804e-09 * o[10]) / o[12] + o[19] * (1.9941018075704e-17 / (o[1] * o[21] * o[4] * o[5]) + o[15] * ( -4.48827542684151e-19 / o[22] + o[20] * ( -1.00075970318621e-21 * o[28] + pi1 * (4.65957282962769e-22 * o[26] + pi1 * ( -7.2912378325616e-23 * o[24] + (3.83502057899078e-24 * pi1) / (o[1] * o[22] * o[5])))))))))))) + o[3] * ( -0.29265942426334 + tau1 * (0.84548187169114 + o[1] * (3.3855169168385 + tau1 * ( -1.91583926775744 + tau1 * (0.47316115539684 + ( -0.066465668798004 + 0.0040607314991784 * tau1) * tau1)))));
              gpi:=pi1 * (pi1 * ((9.5038934535162e-05 + o[4] * (8.4812393955936e-06 + 2.55615384360309e-09 * o[7])) * o[8] + pi1 * (o[11] * (8.9701127632e-06 + (2.60684891582404e-06 + 5.7366919751696e-13 * o[12]) * o[2]) + pi1 * (2.02584984300585e-06 / o[5] + o[16] * (o[19] * (o[15] * (o[20] * ( -7.63737668221055e-22 / (o[1] * o[22] * o[4]) + pi1 * (3.5842867920213e-22 * o[28] + pi1 * ( -5.65070932023524e-23 * o[26] + 2.99318679335866e-24 * o[24] * pi1))) - 3.33001080055983e-19 / (o[1] * o[21] * o[4] * o[5] * tau1)) + 1.44400475720615e-17 / (o[21] * o[4] * o[5] * tau1)) + (1.01874413933128e-08 + 1.39398969845072e-09 * o[10]) / (o[1] * o[5] * tau1))))) + o[3] * (0.00094368642146534 + o[2] * (0.00060003561586052 + ( -9.5322787813974e-05 + o[1] * (8.8283690661692e-06 + 1.45389992595188e-15 * o[9])) * tau1))) + o[14] * ( -0.00028319080123804 + o[1] * (0.00060706301565874 + o[7] * (0.018990068218419 + tau1 * (0.032529748770505 + (0.021841717175414 + 5.283835796993e-05 * o[1]) * tau1))));
            end g1pitau;
            function g2pitau "derivative of g wrt pi and tau"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.Temperature T "temperature (K)";
              output Real pi "dimensionless pressure";
              output Real tau "dimensionless temperature";
              output Real gpi "dimensionless dervative of Gibbs function wrt pi";
              output Real gtau "dimensionless dervative of Gibbs function wrt tau";
            protected 
              Real tau2 "dimensionless temperature";
              Real[22] o "vector of auxiliary variables";
            algorithm 
              assert(p > triple.ptriple, "IF97 medium function g2pitau called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              assert(p <= 100000000.0, "IF97 medium function g2pitau: the input pressure (= " + String(p) + " Pa) is higher than 100 Mpa");
              assert(T >= 273.15, "IF97 medium function g2pitau: the temperature (= " + String(T) + " K) is lower than 273.15 K!");
              assert(T <= 1073.15, "IF97 medium function g2pitau: the input temperature (= " + String(T) + " K) is higher than the limit of 1073.15 K");
              pi:=p / data.PSTAR2;
              tau:=data.TSTAR2 / T;
              tau2:= -0.5 + tau;
              o[1]:=tau * tau;
              o[2]:=o[1] * o[1];
              o[3]:=tau2 * tau2;
              o[4]:=o[3] * tau2;
              o[5]:=o[3] * o[3];
              o[6]:=o[5] * o[5];
              o[7]:=o[6] * o[6];
              o[8]:=o[5] * o[6] * o[7] * tau2;
              o[9]:=o[3] * o[5];
              o[10]:=o[5] * o[6] * tau2;
              o[11]:=o[3] * o[7] * tau2;
              o[12]:=o[3] * o[5] * o[6];
              o[13]:=o[3] * o[5] * tau2;
              o[14]:=o[5] * o[6] * o[7];
              o[15]:=pi * pi;
              o[16]:=o[15] * o[15];
              o[17]:=o[15] * o[16];
              o[18]:=o[5] * o[7] * tau2;
              o[19]:=o[7] * o[7];
              o[20]:=o[3] * o[5] * o[6] * tau2;
              o[21]:=o[5] * o[7];
              o[22]:=o[3] * o[7];
              gtau:=(0.0280439559151 + tau * ( -0.2858109552582 + tau * (1.2213149471784 + tau * ( -2.848163942888 + tau * (4.38395111945 + o[1] * (10.08665568018 + ( -0.5681726521544 + 0.06380539059921 * tau) * tau)))))) / (o[1] * o[2]) + pi * ( -0.017834862292358 + tau2 * ( -0.09199202739273 + ( -0.172743777250296 - 0.30195167236758 * o[4]) * tau2) + pi * ( -3.3032641670203e-05 + ( -0.0003789797503263 + o[3] * ( -0.015757110897342 + o[4] * ( -0.306581069554011 - 0.000960283724907132 * o[8]))) * tau2 + pi * (4.3870667284435e-07 + o[3] * ( -9.683303171571e-05 + o[4] * ( -0.0090203547252888 - 1.42338887469272 * o[8])) + pi * ( -7.8847309559367e-10 + (2.558143570457e-08 + 1.44676118155521e-06 * tau2) * tau2 + pi * (1.60454534363627e-05 * o[9] + pi * (( -5.0144299353183e-11 + o[10] * ( -0.033874355714168 - 836.35096769364 * o[11])) * o[3] + pi * (( -1.38839897890111e-05 - 0.973671060893475 * o[12]) * o[3] * o[6] + pi * (o[13] * (9.0049690883672e-11 - 296.320827232793 * o[14]) + pi * (2.57526266427144e-07 * o[5] * o[6] + pi * (o[4] * (4.1627860840696e-19 + ( -1.0234747095929e-12 - 1.40254511313154e-08 * o[5]) * o[9]) + o[17] * (o[14] * ( -2.34560435076256e-09 + 5.3465159397045 * o[18]) + o[15] * ( -19.1874828272775 * o[19] * o[6] * o[7] + o[15] * (o[11] * (1.78371690710842e-23 + (1.07202609066812e-11 - 0.000201611844951398 * o[10]) * o[20]) + pi * ( -1.24017662339842e-24 * o[21] + pi * (0.000200482822351322 * o[19] * o[5] * o[7] + pi * ( -4.97975748452559e-14 * o[19] * o[3] * o[5] + (1.90027787547159e-27 + o[12] * (2.21658861403112e-15 - 5.47344301999018e-05 * o[22])) * o[6] * o[7] * pi * tau2))))))))))))))));
              gpi:=(1.0 + pi * ( -0.0017731742473213 + tau2 * ( -0.017834862292358 + tau2 * ( -0.045996013696365 + ( -0.057581259083432 - 0.05032527872793 * o[4]) * tau2)) + pi * (tau2 * ( -6.6065283340406e-05 + ( -0.0003789797503263 + o[3] * ( -0.007878555448671 + o[4] * ( -0.087594591301146 - 5.3349095828174e-05 * o[8]))) * tau2) + pi * (6.1445213076927e-08 + (1.31612001853305e-06 + o[3] * ( -9.683303171571e-05 + o[4] * ( -0.0045101773626444 - 0.122004760687947 * o[8]))) * tau2 + pi * (tau2 * ( -3.15389238237468e-09 + (5.116287140914e-08 + 1.92901490874028e-06 * tau2) * tau2) + pi * (1.14610381688305e-05 * o[13] + pi * (( -1.00288598706366e-10 + o[10] * ( -0.012702883392813 - 143.374451604624 * o[11])) * o[4] + pi * ( -4.1341695026989e-17 + ( -8.8352662293707e-06 - 0.272627897050173 * o[12]) * o[3] * o[6] * tau2 + pi * ((9.0049690883672e-11 - 65.8490727183984 * o[14]) * o[6] + pi * (1.78287415218792e-07 * o[10] + pi * (o[5] * (1.0406965210174e-18 + ( -1.0234747095929e-12 - 1.0018179379511e-08 * o[5]) * o[9]) + o[17] * (( -1.29412653835176e-09 + 1.71088510070544 * o[18]) * o[8] + o[15] * ( -6.05920510335078 * o[19] * o[6] * o[7] * tau2 + o[15] * ((1.78371690710842e-23 + (6.1258633752464e-12 - 8.4004935396416e-05 * o[10]) * o[20]) * o[21] + pi * ( -1.24017662339842e-24 * o[18] + pi * (8.32192847496054e-05 * o[19] * o[5] * o[7] * tau2 + pi * ((1.75410265428146e-27 + o[12] * (1.32995316841867e-15 - 2.26487297378904e-05 * o[22])) * o[3] * o[6] * o[7] * pi - 2.93678005497663e-14 * o[19] * o[3] * o[5] * tau2))))))))))))))))) / pi;
            end g2pitau;
            function g5pitau "derivative of g wrt pi and tau"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.Temperature T "temperature (K)";
              output Real pi "dimensionless pressure";
              output Real tau "dimensionless temperature";
              output Real gpi "dimensionless dervative of Gibbs function wrt pi";
              output Real gtau "dimensionless dervative of Gibbs function wrt tau";
            protected 
              Real[3] o "vector of auxiliary variables";
            algorithm 
              assert(p > triple.ptriple, "IF97 medium function g5pitau called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              assert(p <= data.PLIMIT5, "IF97 medium function g5pitau: input pressure (= " + String(p) + " Pa) is higher than 10 Mpa in region 5");
              assert(T <= 2273.15, "IF97 medium function g5pitau: input temperature (= " + String(T) + " K) is higher than limit of 2273.15 K in region 5");
              pi:=p / data.PSTAR5;
              tau:=data.TSTAR5 / T;
              o[1]:=tau * tau;
              o[2]:=o[1] * o[1];
              o[3]:=o[2] * o[2];
              gtau:=pi * (0.0021774678714571 - 0.013782846269973 * o[1] + pi * ( -3.57523455236121e-05 * o[3] + 3.8757684869352e-07 * o[1] * pi)) + (0.074415446800398 + tau * ( -0.73803069960666 + (3.1161318213925 + o[1] * (6.8540841634434 - 0.65923253077834 * tau)) * tau)) / o[2];
              gpi:=(1.0 + pi * ( -0.00012563183589592 + (0.0021774678714571 - 0.004594282089991 * o[1]) * tau + pi * ( -7.9449656719138e-06 * o[3] * tau + 3.8757684869352e-07 * o[1] * pi * tau))) / pi;
            end g5pitau;
            function f3deltatau "1st derivatives of f wrt delta and tau"
              extends Modelica.Icons.Function;
              input SI.Density d "density";
              input SI.Temperature T "temperature (K)";
              output Real delta "dimensionless density";
              output Real tau "dimensionless temperature";
              output Real fdelta "dimensionless dervative of Helmholtz function wrt delta";
              output Real ftau "dimensionless dervative of Helmholtz function wrt tau";
            protected 
              Real[13] o "vector of auxiliary variables";
            algorithm 
              tau:=data.TCRIT / T;
              delta:=if d == data.DCRIT and T == data.TCRIT then 1 + Modelica.Constants.eps else d / data.DCRIT;
              o[1]:=tau * tau;
              o[2]:=o[1] * o[1];
              o[3]:=o[2] * o[2];
              o[4]:=o[3] * tau;
              o[5]:=o[1] * o[2] * o[3] * tau;
              o[6]:=o[2] * o[3];
              o[7]:=o[1] * o[3];
              o[8]:=o[3] * o[3];
              o[9]:=o[1] * o[2] * o[8];
              o[10]:=o[1] * o[2] * o[8] * tau;
              o[11]:=o[3] * o[8];
              o[12]:=o[1] * o[3] * o[8];
              o[13]:=o[3] * o[8] * tau;
              fdelta:=(1.0658070028513 + delta * (o[1] * ( -1.2654315477714 + o[2] * ( -1.1524407806681 + (0.88521043984318 - 0.64207765181607 * o[1]) * o[4])) + delta * (0.76986920373342 + o[1] * ( -1.70429417648412 + o[2] * (9.7944563083754 + ( -6.100523451393 + (0.078841073758308 + 0.25116816848616 * o[2]) * o[5]) * tau)) + delta * ( -0.8399798909613 + o[1] * (4.169939870838 + o[1] * ( -6.056974507071 + o[6] * ( -0.0246442911521889 - 1.42788107204769 * o[7]))) + delta * (0.175936297894 + o[1] * ( -1.77905741714956 + o[1] * (3.62288282878932 + 2.82089800351868 * o[9])) + delta * (delta * ( -0.133052405238576 + o[1] * (0.565564509990552 + 0.98617670687766 * o[11]) + delta * ( -0.094523605689436 * o[1] + delta * ( -0.118674762819776 * o[12] + delta * (o[1] * (0.00521306582652756 + 0.0290780142333399 * o[11]) + delta * (0.00080964802996215 - 0.000494162889679965 * delta * o[12] - 0.0016557679795037 * tau))))) + (0.5385256313166 + o[1] * ( -1.6456811629477 - 2.5435531020579 * o[10])) * tau)))))) / delta;
              ftau:=20.944396974307 + tau * ( -15.3735415757432 + o[2] * tau * (18.3301634515678 + o[1] * tau * ( -28.08078114862 + o[1] * (14.4640436358204 - 0.194503669468755 * o[1] * o[3] * tau)))) + delta * (( -2.5308630955428 + o[2] * ( -6.9146446840086 + (13.2781565976477 - 10.9153200808732 * o[1]) * o[4])) * tau + delta * (tau * ( -1.70429417648412 + o[2] * (29.3833689251262 + ( -21.3518320798755 + (0.867251811341388 + 3.26518619032008 * o[2]) * o[5]) * tau)) + delta * ((2.779959913892 + o[1] * ( -8.075966009428 + o[6] * ( -0.131436219478341 - 12.37496929108 * o[7]))) * tau + delta * (( -0.88952870857478 + o[1] * (3.62288282878932 + 18.3358370228714 * o[9])) * tau + delta * (0.10770512626332 + o[1] * ( -0.98740869776862 - 13.2264761307011 * o[10]) + delta * ((0.188521503330184 + 4.27343239646986 * o[11]) * tau + delta * ( -0.027006744482696 * tau + delta * ( -0.385692979164272 * o[13] + delta * (delta * ( -0.00016557679795037 - 0.00116802137560719 * delta * o[13]) + (0.00115845907256168 + 0.0840031522296486 * o[11]) * tau)))))))));
            end f3deltatau;
            function tph1 "inverse function for region 1: T(p,h)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.SpecificEnthalpy h "specific enthalpy";
              output SI.Temperature T "temperature (K)";
            protected 
              Real pi "dimensionless pressure";
              Real eta1 "dimensionless specific enthalpy";
              Real[3] o "vector of auxiliary variables";
            algorithm 
              assert(p > triple.ptriple, "IF97 medium function tph1 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              pi:=p / data.PSTAR2;
              eta1:=h / data.HSTAR1 + 1.0;
              o[1]:=eta1 * eta1;
              o[2]:=o[1] * o[1];
              o[3]:=o[2] * o[2];
              T:= -238.72489924521 - 13.391744872602 * pi + eta1 * (404.21188637945 + 43.211039183559 * pi + eta1 * (113.49746881718 - 54.010067170506 * pi + eta1 * (30.535892203916 * pi + eta1 * ( -6.5964749423638 * pi + o[1] * ( -5.8457616048039 + o[2] * (pi * (0.0093965400878363 + ( -2.5858641282073e-05 + 6.6456186191635e-08 * pi) * pi) + o[2] * o[3] * ( -0.0001528548241314 + o[1] * o[3] * ( -1.0866707695377e-06 + pi * (1.157364750534e-07 + pi * ( -4.0644363084799e-09 + pi * (8.0670734103027e-11 + pi * ( -9.3477771213947e-13 + (5.8265442020601e-15 - 1.5020185953503e-17 * pi) * pi))))))))))));
            end tph1;
            function tps1 "inverse function for region 1: T(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.SpecificEntropy s "specific entropy";
              output SI.Temperature T "temperature (K)";
            protected 
              constant SI.Pressure pstar = 1000000.0;
              constant SI.SpecificEntropy sstar = 1000.0;
              Real pi "dimensionless pressure";
              Real sigma1 "dimensionless specific entropy";
              Real[6] o "vector of auxiliary variables";
            algorithm 
              pi:=p / pstar;
              assert(p > triple.ptriple, "IF97 medium function tps1 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              sigma1:=s / sstar + 2.0;
              o[1]:=sigma1 * sigma1;
              o[2]:=o[1] * o[1];
              o[3]:=o[2] * o[2];
              o[4]:=o[3] * o[3];
              o[5]:=o[4] * o[4];
              o[6]:=o[1] * o[2] * o[4];
              T:=174.78268058307 + sigma1 * (34.806930892873 + sigma1 * (6.5292584978455 + (0.33039981775489 + o[3] * ( -1.9281382923196e-07 - 2.4909197244573e-23 * o[2] * o[4])) * sigma1)) + pi * ( -0.26107636489332 + pi * (0.00056608900654837 + pi * (o[1] * o[3] * (2.6400441360689e-13 + 7.8124600459723e-29 * o[6]) - 3.0732199903668e-31 * o[5] * pi) + sigma1 * ( -0.00032635483139717 + sigma1 * (4.4778286690632e-05 + o[1] * o[2] * ( -5.1322156908507e-10 - 4.2522657042207e-26 * o[6]) * sigma1))) + sigma1 * (0.22592965981586 + sigma1 * ( -0.064256463395226 + sigma1 * (0.0078876289270526 + o[3] * sigma1 * (3.5672110607366e-10 + 1.7332496994895e-24 * o[1] * o[4] * sigma1)))));
            end tps1;
            function tph2 "reverse function for region 2: T(p,h)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.SpecificEnthalpy h "specific enthalpy";
              output SI.Temperature T "temperature (K)";
            protected 
              Real pi "dimensionless pressure";
              Real pi2b "dimensionless pressure";
              Real pi2c "dimensionless pressure";
              Real eta "dimensionless specific enthalpy";
              Real etabc "dimensionless specific enthalpy";
              Real eta2a "dimensionless specific enthalpy";
              Real eta2b "dimensionless specific enthalpy";
              Real eta2c "dimensionless specific enthalpy";
              Real[8] o "vector of auxiliary variables";
            algorithm 
              pi:=p * data.IPSTAR;
              eta:=h * data.IHSTAR;
              etabc:=h * 0.001;
              if pi < 4.0 then 
                            eta2a:=eta - 2.1;
              o[1]:=eta2a * eta2a;
              o[2]:=o[1] * o[1];
              o[3]:=pi * pi;
              o[4]:=o[3] * o[3];
              o[5]:=o[3] * pi;
              T:=1089.8952318288 + (1.844574935579 - 0.0061707422868339 * pi) * pi + eta2a * (849.51654495535 - 4.1792700549624 * pi + eta2a * ( -107.81748091826 + (6.2478196935812 - 0.31078046629583 * pi) * pi + eta2a * (33.153654801263 - 17.344563108114 * pi + o[2] * ( -7.4232016790248 + pi * ( -200.58176862096 + 11.670873077107 * pi) + o[1] * (271.96065473796 * pi + o[1] * ( -455.11318285818 * pi + eta2a * (1.3865724283226 * o[4] + o[1] * o[2] * (3091.9688604755 * pi + o[1] * (11.765048724356 + o[2] * ( -13551.334240775 * o[5] + o[2] * ( -62.459855192507 * o[3] * o[4] * pi + o[2] * (o[4] * (235988.32556514 + 7399.9835474766 * pi) + o[1] * (19127.72923966 * o[3] * o[4] + o[1] * (o[3] * (128127984.04046 - 551966.9703006 * o[5]) + o[1] * ( -985549096.23276 * o[3] + o[1] * (2822454697.3002 * o[3] + o[1] * (o[3] * ( -3594897141.0703 + 3715408.5996233 * o[5]) + o[1] * pi * (252266.40357872 + pi * (1722734991.3197 + pi * (12848734.66465 + ( -13105236.545054 - 415351.64835634 * o[3]) * pi))))))))))))))))))));
              elseif pi < (0.00012809002730136 * etabc - 0.67955786399241) * etabc + 905.84278514723 then
                eta2b:=eta - 2.6;
                pi2b:=pi - 2.0;
                o[1]:=pi2b * pi2b;
                o[2]:=o[1] * pi2b;
                o[3]:=o[1] * o[1];
                o[4]:=eta2b * eta2b;
                o[5]:=o[4] * o[4];
                o[6]:=o[4] * o[5];
                o[7]:=o[5] * o[5];
                T:=1489.5041079516 + 0.93747147377932 * pi2b + eta2b * (743.07798314034 + o[2] * (0.00011032831789999 - 1.7565233969407e-18 * o[1] * o[3]) + eta2b * ( -97.708318797837 + pi2b * (3.3593118604916 + pi2b * ( -0.021810755324761 + pi2b * (0.00018955248387902 + (2.8640237477456e-07 - 8.1456365207833e-14 * o[2]) * pi2b))) + o[5] * (3.3809355601454 * pi2b + o[4] * ( -0.10829784403677 * o[1] + o[5] * (2.4742464705674 + (0.16844539671904 + o[1] * (0.0030891541160537 - 1.0779857357512e-05 * pi2b)) * pi2b + o[6] * ( -0.63281320016026 + pi2b * (0.73875745236695 + ( -0.046333324635812 + o[1] * ( -7.6462712454814e-05 + 2.821728163504e-07 * pi2b)) * pi2b) + o[6] * (1.1385952129658 + pi2b * ( -0.47128737436186 + o[1] * (0.0013555504554949 + (1.4052392818316e-05 + 1.2704902271945e-06 * pi2b) * pi2b)) + o[5] * ( -0.47811863648625 + (0.15020273139707 + o[2] * ( -3.1083814331434e-05 + o[1] * ( -1.1030139238909e-08 - 2.5180545682962e-11 * pi2b))) * pi2b + o[5] * o[7] * (0.0085208123431544 + pi2b * ( -0.002176411421975 + pi2b * (7.1280351959551e-05 + o[1] * ( -1.0302738212103e-06 + (7.3803353468292e-08 + 8.6934156344163e-15 * o[3]) * pi2b))))))))))));


              else               eta2c:=eta - 1.8;
              pi2c:=pi + 25.0;
              o[1]:=pi2c * pi2c;
              o[2]:=o[1] * o[1];
              o[3]:=o[1] * o[2] * pi2c;
              o[4]:=1 / o[3];
              o[5]:=o[1] * o[2];
              o[6]:=eta2c * eta2c;
              o[7]:=o[2] * o[2];
              o[8]:=o[6] * o[6];
              T:=eta2c * ((859777.2253558 + o[1] * (482.19755109255 + 1.126159740723e-12 * o[5])) / o[1] + eta2c * (( -583401318515.9 + (20825544563.171 + 31081.088422714 * o[2]) * pi2c) / o[5] + o[6] * (o[8] * (o[6] * (1.2324579690832e-07 * o[5] + o[6] * ( -1.1606921130984e-06 * o[5] + o[8] * (2.7846367088554e-05 * o[5] + ( -0.00059270038474176 * o[5] + 0.0012918582991878 * o[5] * o[6]) * o[8]))) - 10.842984880077 * pi2c) + o[4] * (7326335090218.1 + o[7] * (3.7966001272486 + ( -0.04536417267666 - 1.7804982240686e-11 * o[2]) * pi2c))))) + o[4] * ( -3236839855524.2 + pi2c * (358250899454.47 + pi2c * ( -10783068217.47 + o[1] * pi2c * (610747.83564516 + pi2c * ( -25745.72360417 + (1208.2315865936 + 1.4559115658698e-13 * o[5]) * pi2c)))));

              end if;
            end tph2;
            function tps2a "reverse function for region 2a: T(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.SpecificEntropy s "specific entropy";
              output SI.Temperature T "temperature (K)";
            protected 
              Real[12] o "vector of auxiliary variables";
              constant Real IPSTAR = 1e-06 "scaling variable";
              constant Real ISSTAR2A = 1 / 2000.0 "scaling variable";
              Real pi "dimensionless pressure";
              Real sigma2a "dimensionless specific entropy";
            algorithm 
              pi:=p * IPSTAR;
              sigma2a:=s * ISSTAR2A - 2.0;
              o[1]:=pi ^ 0.5;
              o[2]:=sigma2a * sigma2a;
              o[3]:=o[2] * o[2];
              o[4]:=o[3] * o[3];
              o[5]:=o[4] * o[4];
              o[6]:=pi ^ 0.25;
              o[7]:=o[2] * o[4] * o[5];
              o[8]:=1 / o[7];
              o[9]:=o[3] * sigma2a;
              o[10]:=o[2] * o[3] * sigma2a;
              o[11]:=o[3] * o[4] * sigma2a;
              o[12]:=o[2] * sigma2a;
              T:=(( -392359.83861984 + (515265.7382727 + o[3] * (40482.443161048 + o[2] * o[3] * ( -321.93790923902 + o[2] * (96.961424218694 - 22.867846371773 * sigma2a)))) * sigma2a) / (o[4] * o[5]) + o[6] * (( -449429.14124357 + o[3] * ( -5011.8336020166 + 0.35684463560015 * o[4] * sigma2a)) / (o[2] * o[5] * sigma2a) + o[6] * (o[8] * (44235.33584819 + o[9] * ( -13673.388811708 + o[3] * (421632.60207864 + (22516.925837475 + o[10] * (474.42144865646 - 149.31130797647 * sigma2a)) * sigma2a))) + o[6] * (( -197811.26320452 - 23554.39947076 * sigma2a) / (o[2] * o[3] * o[4] * sigma2a) + o[6] * (( -19070.616302076 + o[11] * (55375.669883164 + (3829.3691437363 - 603.91860580567 * o[2]) * o[3])) * o[8] + o[6] * ((1936.3102620331 + o[2] * (4266.064369861 + o[2] * o[3] * o[4] * ( -5978.0638872718 - 704.01463926862 * o[9]))) / (o[2] * o[4] * o[5] * sigma2a) + o[1] * ((338.36784107553 + o[12] * (20.862786635187 + (0.033834172656196 - 4.3124428414893e-05 * o[12]) * o[3])) * sigma2a + o[6] * (166.53791356412 + sigma2a * ( -139.86292055898 + o[3] * ( -0.78849547999872 + (0.072132411753872 + o[3] * ( -0.0059754839398283 + ( -1.2141358953904e-05 + 2.3227096733871e-07 * o[2]) * o[3])) * sigma2a)) + o[6] * ( -10.538463566194 + o[3] * (2.0718925496502 + ( -0.072193155260427 + 2.074988708112e-07 * o[4]) * o[9]) + o[6] * (o[6] * (o[12] * (0.21037527893619 + 0.00025681239729999 * o[3] * o[4]) + ( -0.012799002933781 - 8.2198102652018e-06 * o[11]) * o[6] * o[9]) + o[10] * ( -0.018340657911379 + 2.9036272348696e-07 * o[2] * o[4] * sigma2a))))))))))) / (o[1] * pi);
            end tps2a;
            function tps2b "reverse function for region 2b: T(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.SpecificEntropy s "specific entropy";
              output SI.Temperature T "temperature (K)";
            protected 
              Real[8] o "vector of auxiliary variables";
              constant Real IPSTAR = 1e-06 "scaling variable";
              constant Real ISSTAR2B = 1 / 785.3 "scaling variable";
              Real pi "dimensionless pressure";
              Real sigma2b "dimensionless specific entropy";
            algorithm 
              pi:=p * IPSTAR;
              sigma2b:=10.0 - s * ISSTAR2B;
              o[1]:=pi * pi;
              o[2]:=o[1] * o[1];
              o[3]:=sigma2b * sigma2b;
              o[4]:=o[3] * o[3];
              o[5]:=o[4] * o[4];
              o[6]:=o[3] * o[5] * sigma2b;
              o[7]:=o[3] * o[5];
              o[8]:=o[3] * sigma2b;
              T:=(316876.65083497 + 20.864175881858 * o[6] + pi * ( -398593.99803599 - 21.816058518877 * o[6] + pi * (223697.85194242 + ( -2784.1703445817 + 9.920743607148 * o[7]) * sigma2b + pi * ( -75197.512299157 + (2970.8605951158 + o[7] * ( -3.4406878548526 + 0.38815564249115 * sigma2b)) * sigma2b + pi * (17511.29508575 + sigma2b * ( -1423.7112854449 + (1.0943803364167 + 0.89971619308495 * o[4]) * o[4] * sigma2b) + pi * ( -3375.9740098958 + (471.62885818355 + o[4] * ( -1.9188241993679 + o[8] * (0.41078580492196 - 0.33465378172097 * sigma2b))) * sigma2b + pi * (1387.0034777505 + sigma2b * ( -406.63326195838 + sigma2b * (41.72734715961 + o[3] * (2.1932549434532 + sigma2b * ( -1.0320050009077 + (0.35882943516703 + 0.0052511453726066 * o[8]) * sigma2b)))) + pi * (12.838916450705 + sigma2b * ( -2.8642437219381 + sigma2b * (0.56912683664855 + ( -0.099962954584931 + o[4] * ( -0.0032632037778459 + 0.00023320922576723 * sigma2b)) * sigma2b)) + pi * ( -0.1533480985745 + (0.029072288239902 + 0.00037534702741167 * o[4]) * sigma2b + pi * (0.0017296691702411 + ( -0.00038556050844504 - 3.5017712292608e-05 * o[3]) * sigma2b + pi * ( -1.4566393631492e-05 + 5.6420857267269e-06 * sigma2b + pi * (4.1286150074605e-08 + ( -2.0684671118824e-08 + 1.6409393674725e-09 * sigma2b) * sigma2b)))))))))))) / (o[1] * o[2]);
            end tps2b;
            function tps2c "reverse function for region 2c: T(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.SpecificEntropy s "specific entropy";
              output SI.Temperature T "temperature (K)";
            protected 
              constant Real IPSTAR = 1e-06 "scaling variable";
              constant Real ISSTAR2C = 1 / 2925.1 "scaling variable";
              Real pi "dimensionless pressure";
              Real sigma2c "dimensionless specific entropy";
              Real[3] o "vector of auxiliary variables";
            algorithm 
              pi:=p * IPSTAR;
              sigma2c:=2.0 - s * ISSTAR2C;
              o[1]:=pi * pi;
              o[2]:=sigma2c * sigma2c;
              o[3]:=o[2] * o[2];
              T:=(909.68501005365 + 2404.566708842 * sigma2c + pi * ( -591.6232638713 + pi * (541.45404128074 + sigma2c * ( -270.98308411192 + (979.76525097926 - 469.66772959435 * sigma2c) * sigma2c) + pi * (14.399274604723 + ( -19.104204230429 + o[2] * (5.3299167111971 - 21.252975375934 * sigma2c)) * sigma2c + pi * ( -0.3114733441376 + (0.60334840894623 - 0.042764839702509 * sigma2c) * sigma2c + pi * (0.0058185597255259 + ( -0.014597008284753 + 0.0056631175631027 * o[3]) * sigma2c + pi * ( -7.6155864584577e-05 + sigma2c * (0.00022440342919332 - 1.2561095013413e-05 * o[2] * sigma2c) + pi * (6.3323132660934e-07 + ( -2.0541989675375e-06 + 3.6405370390082e-08 * sigma2c) * sigma2c + pi * ( -2.9759897789215e-09 + 1.0136618529763e-08 * sigma2c + pi * (5.9925719692351e-12 + sigma2c * ( -2.0677870105164e-11 + o[2] * ( -2.0874278181886e-11 + (1.0162166825089e-10 - 1.6429828281347e-10 * sigma2c) * sigma2c)))))))))))) / o[1];
            end tps2c;
            function tps2 "reverse function for region 2: T(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.SpecificEntropy s "specific entropy";
              output SI.Temperature T "temperature (K)";
            protected 
              Real pi "dimensionless pressure";
              constant SI.SpecificEntropy SLIMIT = 5850.0 "subregion boundary specific entropy between regions 2a and 2b";
            algorithm 
              if p < 4000000.0 then 
                            T:=tps2a(p, s);
              elseif s > SLIMIT then
                T:=tps2b(p, s);


              else               T:=tps2c(p, s);

              end if;
            end tps2;
            function tsat "region 4 saturation temperature as a function of pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.Temperature t_sat "temperature";
              annotation(derivative = tsat_der);
            protected 
              Real pi "dimensionless pressure";
              Real[20] o "vector of auxiliary variables";
            algorithm 
              assert(p > triple.ptriple, "IF97 medium function tsat called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              pi:=min(p, data.PCRIT) * data.IPSTAR;
              o[1]:=pi ^ 0.25;
              o[2]:= -3232555.0322333 * o[1];
              o[3]:=pi ^ 0.5;
              o[4]:= -724213.16703206 * o[3];
              o[5]:=405113.40542057 + o[2] + o[4];
              o[6]:= -17.073846940092 * o[1];
              o[7]:=14.91510861353 + o[3] + o[6];
              o[8]:= -4.0 * o[5] * o[7];
              o[9]:=12020.82470247 * o[1];
              o[10]:=1167.0521452767 * o[3];
              o[11]:= -4823.2657361591 + o[10] + o[9];
              o[12]:=o[11] * o[11];
              o[13]:=o[12] + o[8];
              o[14]:=o[13] ^ 0.5;
              o[15]:= -o[14];
              o[16]:= -12020.82470247 * o[1];
              o[17]:= -1167.0521452767 * o[3];
              o[18]:=4823.2657361591 + o[15] + o[16] + o[17];
              o[19]:=1 / o[18];
              o[20]:=2.0 * o[19] * o[5];
              t_sat:=0.5 * (650.17534844798 + o[20] - ( -4.0 * ( -0.23855557567849 + 1300.35069689596 * o[19] * o[5]) + (650.17534844798 + o[20]) ^ 2.0) ^ 0.5);
            end tsat;
            function dtsatofp "derivative of saturation temperature w.r.t. pressure"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output Real dtsat(unit = "K/Pa") "derivative of T w.r.t. p";
            protected 
              Real pi "dimensionless pressure";
              Real[49] o "vector of auxiliary variables";
            algorithm 
              pi:=max(Modelica.Constants.small, p * data.IPSTAR);
              o[1]:=pi ^ 0.75;
              o[2]:=1 / o[1];
              o[3]:= -4.268461735023 * o[2];
              o[4]:=sqrt(pi);
              o[5]:=1 / o[4];
              o[6]:=0.5 * o[5];
              o[7]:=o[3] + o[6];
              o[8]:=pi ^ 0.25;
              o[9]:= -3232555.0322333 * o[8];
              o[10]:= -724213.16703206 * o[4];
              o[11]:=405113.40542057 + o[10] + o[9];
              o[12]:= -4 * o[11] * o[7];
              o[13]:= -808138.758058325 * o[2];
              o[14]:= -362106.58351603 * o[5];
              o[15]:=o[13] + o[14];
              o[16]:= -17.073846940092 * o[8];
              o[17]:=14.91510861353 + o[16] + o[4];
              o[18]:= -4 * o[15] * o[17];
              o[19]:=3005.2061756175 * o[2];
              o[20]:=583.52607263835 * o[5];
              o[21]:=o[19] + o[20];
              o[22]:=12020.82470247 * o[8];
              o[23]:=1167.0521452767 * o[4];
              o[24]:= -4823.2657361591 + o[22] + o[23];
              o[25]:=2.0 * o[21] * o[24];
              o[26]:=o[12] + o[18] + o[25];
              o[27]:= -4.0 * o[11] * o[17];
              o[28]:=o[24] * o[24];
              o[29]:=o[27] + o[28];
              o[30]:=sqrt(o[29]);
              o[31]:=1 / o[30];
              o[32]:= -o[30];
              o[33]:= -12020.82470247 * o[8];
              o[34]:= -1167.0521452767 * o[4];
              o[35]:=4823.2657361591 + o[32] + o[33] + o[34];
              o[36]:=o[30];
              o[37]:= -4823.2657361591 + o[22] + o[23] + o[36];
              o[38]:=o[37] * o[37];
              o[39]:=1 / o[38];
              o[40]:= -1.72207339365771 * o[30];
              o[41]:=21592.2055343628 * o[8];
              o[42]:=o[30] * o[8];
              o[43]:= -8192.87114842946 * o[4];
              o[44]:= -0.510632954559659 * o[30] * o[4];
              o[45]:= -3100.02526152368 * o[1];
              o[46]:=pi;
              o[47]:=1295.95640782102 * o[46];
              o[48]:=2862.09212505088 + o[40] + o[41] + o[42] + o[43] + o[44] + o[45] + o[47];
              o[49]:=1 / (o[35] * o[35]);
              dtsat:=data.IPSTAR * 0.5 * ((2.0 * o[15]) / o[35] - 2.0 * o[11] * ( -3005.2061756175 * o[2] - 0.5 * o[26] * o[31] - 583.52607263835 * o[5]) * o[49] - (20953.4635664399 * (o[39] * (1295.95640782102 + 5398.05138359071 * o[2] + 0.25 * o[2] * o[30] - 0.861036696828853 * o[26] * o[31] - 0.255316477279829 * o[26] * o[31] * o[4] - 4096.43557421473 * o[5] - 0.255316477279829 * o[30] * o[5] - 2325.01894614276 / o[8] + 0.5 * o[26] * o[31] * o[8]) - 2.0 * (o[19] + o[20] + 0.5 * o[26] * o[31]) * o[48] * o[37] ^ ( -3))) / sqrt(o[39] * o[48]));
            end dtsatofp;
            function tsat_der "derivative function for tsat"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input Real der_p(unit = "Pa/s") "pressure derivatrive";
              output Real der_tsat(unit = "K/s") "temperature derivative";
            protected 
              Real dtp;
            algorithm 
              dtp:=dtsatofp(p);
              der_tsat:=dtp * der_p;
            end tsat_der;
            function psat "region 4 saturation pressure as a functionx of temperature"
              extends Modelica.Icons.Function;
              input SI.Temperature T "temperature (K)";
              output SI.Pressure p_sat "pressure";
            protected 
              Real[8] o "vector of auxiliary variables";
              Real Tlim = min(T, data.TCRIT);
              annotation(derivative = psat_der);
            algorithm 
              assert(T >= 273.16, "IF97 medium function psat: input temperature (= " + String(triple.ptriple) + " K).\n" + "lower than the triple point temperature 273.16 K");
              o[1]:= -650.17534844798 + Tlim;
              o[2]:=1 / o[1];
              o[3]:= -0.23855557567849 * o[2];
              o[4]:=o[3] + Tlim;
              o[5]:= -4823.2657361591 * o[4];
              o[6]:=o[4] * o[4];
              o[7]:=14.91510861353 * o[6];
              o[8]:=405113.40542057 + o[5] + o[7];
              p_sat:=(16000000.0 * o[8] * o[8] * o[8] * o[8] * 1) / (3232555.0322333 - 12020.82470247 * o[4] + 17.073846940092 * o[6] + ( -4.0 * ( -724213.16703206 + 1167.0521452767 * o[4] + o[6]) * o[8] + ( -3232555.0322333 + 12020.82470247 * o[4] - 17.073846940092 * o[6]) ^ 2.0) ^ 0.5) ^ 4.0;
            end psat;
            function dptofT "derivative of pressure wrt temperature along the saturation pressure curve"
              extends Modelica.Icons.Function;
              input SI.Temperature T "temperature (K)";
              output Real dpt(unit = "Pa/K") "temperature derivative of pressure";
            protected 
              Real[31] o "vector of auxiliary variables";
              Real Tlim "temeprature limited to TCRIT";
            algorithm 
              Tlim:=min(T, data.TCRIT);
              o[1]:= -650.17534844798 + Tlim;
              o[2]:=1 / o[1];
              o[3]:= -0.23855557567849 * o[2];
              o[4]:=o[3] + Tlim;
              o[5]:= -4823.2657361591 * o[4];
              o[6]:=o[4] * o[4];
              o[7]:=14.91510861353 * o[6];
              o[8]:=405113.40542057 + o[5] + o[7];
              o[9]:=o[8] * o[8];
              o[10]:=o[9] * o[9];
              o[11]:=o[1] * o[1];
              o[12]:=1 / o[11];
              o[13]:=0.23855557567849 * o[12];
              o[14]:=1.0 + o[13];
              o[15]:=12020.82470247 * o[4];
              o[16]:= -17.073846940092 * o[6];
              o[17]:= -3232555.0322333 + o[15] + o[16];
              o[18]:= -4823.2657361591 * o[14];
              o[19]:=29.83021722706 * o[14] * o[4];
              o[20]:=o[18] + o[19];
              o[21]:=1167.0521452767 * o[4];
              o[22]:= -724213.16703206 + o[21] + o[6];
              o[23]:=o[17] * o[17];
              o[24]:= -4.0 * o[22] * o[8];
              o[25]:=o[23] + o[24];
              o[26]:=sqrt(o[25]);
              o[27]:= -12020.82470247 * o[4];
              o[28]:=17.073846940092 * o[6];
              o[29]:=3232555.0322333 + o[26] + o[27] + o[28];
              o[30]:=o[29] * o[29];
              o[31]:=o[30] * o[30];
              dpt:=1000000.0 * (( -64.0 * o[10] * ( -12020.82470247 * o[14] + 34.147693880184 * o[14] * o[4] + (0.5 * ( -4.0 * o[20] * o[22] + 2.0 * o[17] * (12020.82470247 * o[14] - 34.147693880184 * o[14] * o[4]) - 4.0 * (1167.0521452767 * o[14] + 2.0 * o[14] * o[4]) * o[8])) / o[26])) / (o[29] * o[31]) + (64.0 * o[20] * o[8] * o[9]) / o[31]);
            end dptofT;
            function psat_der "derivative function for psat"
              extends Modelica.Icons.Function;
              input SI.Temperature T "temperature (K)";
              input Real der_T(unit = "K/s") "temperature derivative";
              output Real der_psat(unit = "Pa/s") "pressure";
            protected 
              Real dpt;
            algorithm 
              dpt:=dptofT(T);
              der_psat:=dpt * der_T;
            end psat_der;
            function p1_hs "pressure as a function of ehtnalpy and entropy in region 1"
              extends Modelica.Icons.Function;
              input SI.SpecificEnthalpy h "specific enthalpy";
              input SI.SpecificEntropy s "specific entropy";
              output SI.Pressure p "Pressure";
              constant Real[:] n = { -0.691997014660582, -18.361254878756, -9.28332409297335,65.9639569909906, -16.2060388912024,450.620017338667,854.68067822417,6075.23214001162,32.6487682621856, -26.9408844582931, -319.9478483343, -928.35430704332,30.3634537455249, -65.0540422444146, -4309.9131651613, -747.512324096068,730.000345529245,1142.84032569021, -436.407041874559};
              constant Real[:] I = {0,0,0,0,0,0,0,0,1,1,1,1,2,2,2,3,4,4,5};
              constant Real[:] J = {0,1,2,4,5,6,8,14,0,1,4,6,0,1,10,4,1,4,0};
              constant SI.SpecificEnthalpy hstar = 3400000.0 "normalization enthalpy";
              constant SI.Pressure pstar = 100000000.0 "normalization pressure";
              constant SI.SpecificEntropy sstar = 7600.0 "normalization entropy";
            protected 
              Real eta = h / hstar "normalized specific enthalpy";
              Real sigma = s / sstar "normalized specific entropy";
              annotation(Documentation(info = "<html>
<p>
  Equation number 1 from:<br>
  The International Association for the Properties of Water and Steam<br>
  Gaithersburg, Maryland, USA<br>
  September 2001<br>
  Supplementary Release on&nbsp; Backward Equations for Pressure as a
  Function of Enthalpy and Entropy p(h,s) to the IAPWS Industrial
  Formulation 1997 for the Thermodynamic Properties of Water and Steam<br>
  </p>
  </html>
  "));
            algorithm 
              p:=sum(n[i] * (eta + 0.05) ^ I[i] * (sigma + 0.05) ^ J[i] for i in 1:19) * pstar;
            end p1_hs;
            function h2ab_s "boundary between regions 2a and 2b"
              extends Modelica.Icons.Function;
              output SI.SpecificEnthalpy h "Enthalpy";
              input SI.SpecificEntropy s "Entropy";
            protected 
              constant Real[:] n = { -3498.98083432139,2575.60716905876, -421.073558227969,27.6349063799944};
              constant SI.SpecificEnthalpy hstar = 1000.0 "normalization enthalpy";
              constant SI.SpecificEntropy sstar = 1000.0 "normalization entropy";
              Real sigma = s / sstar "normalized specific entropy";
              annotation(Documentation(info = "<html>
  <p>
  Equation number 2 from:<br>
  The International Association for the Properties of Water and Steam<br>
  Gaithersburg, Maryland, USA<br>
  September 2001<br>
  Supplementary Release on&nbsp; Backward Equations for Pressure as a
  Function of Enthalpy and Entropy p(h,s) to the IAPWS Industrial
  Formulation 1997 for the Thermodynamic Properties of Water and Steam<br>
  </p>
  </html>
  "));
            algorithm 
              h:=(n[1] + n[2] * sigma + n[3] * sigma ^ 2 + n[4] * sigma ^ 3) * hstar;
            end h2ab_s;
            function p2a_hs "pressure as a function of enthalpy and entropy in subregion 2a"
              extends Modelica.Icons.Function;
              input SI.SpecificEnthalpy h "specific enthalpy";
              input SI.SpecificEntropy s "specific entropy";
              output SI.Pressure p "Pressure";
              constant Real[:] n = { -0.0182575361923032, -0.125229548799536,0.592290437320145,6.04769706185122,238.624965444474, -298.639090222922,0.051225081304075, -0.437266515606486,0.413336902999504, -5.16468254574773, -5.57014838445711,12.8555037824478,11.414410895329, -119.504225652714, -2847.7798596156,4317.57846408006,1.1289404080265,1974.09186206319,1516.12444706087,0.0141324451421235,0.585501282219601, -2.97258075863012,5.94567314847319, -6236.56565798905,9659.86235133332,6.81500934948134, -6332.07286824489, -5.5891922446576,0.0400645798472063};
              constant Real[:] I = {0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,2,2,2,3,3,3,3,3,4,5,5,6,7};
              constant Real[:] J = {1,3,6,16,20,22,0,1,2,3,5,6,10,16,20,22,3,16,20,0,2,3,6,16,16,3,16,3,1};
              constant SI.SpecificEnthalpy hstar = 4200000.0 "normalization enthalpy";
              constant SI.Pressure pstar = 4000000.0 "normalization pressure";
              constant SI.SpecificEntropy sstar = 12000.0 "normalization entropy";
            protected 
              Real eta = h / hstar "normalized specific enthalpy";
              Real sigma = s / sstar "normalized specific entropy";
              annotation(Documentation(info = "<html>
  <p>
  Equation number 3 from:<br>
  The International Association for the Properties of Water and Steam<br>
  Gaithersburg, Maryland, USA<br>
  September 2001<br>
  Supplementary Release on&nbsp; Backward Equations for Pressure as a
  Function of Enthalpy and Entropy p(h,s) to the IAPWS Industrial
  Formulation 1997 for the Thermodynamic Properties of Water and Steam<br>
  </p>
  </html>
  "));
            algorithm 
              p:=sum(n[i] * (eta - 0.5) ^ I[i] * (sigma - 1.2) ^ J[i] for i in 1:29) ^ 4 * pstar;
            end p2a_hs;
            function p2b_hs "pressure as a function of enthalpy and entropy in subregion 2a"
              extends Modelica.Icons.Function;
              input SI.SpecificEnthalpy h "specific enthalpy";
              input SI.SpecificEntropy s "specific entropy";
              output SI.Pressure p "Pressure";
              constant Real[:] n = {0.0801496989929495, -0.543862807146111,0.337455597421283,8.9055545115745,313.840736431485,0.797367065977789, -1.2161697355624,8.72803386937477, -16.9769781757602, -186.552827328416,95115.9274344237, -18.9168510120494, -4334.0703719484,543212633.012715,0.144793408386013,128.024559637516, -67230.9534071268,33697238.0095287, -586.63419676272, -22140322476.9889,1716.06668708389, -570817595.806302, -3121.09693178482, -2078413.8463301,3056059461577.86,3221.57004314333,326810259797.295, -1441.04158934487,410.694867802691,109077066873.024, -24796465425889.3,1888019068.65134, -123651009018773.0};
              constant Real[:] I = {0,0,0,0,0,1,1,1,1,1,1,2,2,2,3,3,3,3,4,4,5,5,6,6,6,7,7,8,8,8,8,12,14};
              constant Real[:] J = {0,1,2,4,8,0,1,2,3,5,12,1,6,18,0,1,7,12,1,16,1,12,1,8,18,1,16,1,3,14,18,10,16};
              constant SI.SpecificEnthalpy hstar = 4100000.0 "normalization enthalpy";
              constant SI.Pressure pstar = 100000000.0 "normalization pressure";
              constant SI.SpecificEntropy sstar = 7900.0 "normalization entropy";
            protected 
              Real eta = h / hstar "normalized specific enthalpy";
              Real sigma = s / sstar "normalized specific entropy";
              annotation(Documentation(info = "<html>
<p>
Equation number 4 from:<br>
The International Association for the Properties of Water and Steam<br>
Gaithersburg, Maryland, USA<br>
September 2001<br>
Supplementary Release on&nbsp; Backward Equations for Pressure as a
Function of Enthalpy and Entropy p(h,s) to the IAPWS Industrial
Formulation 1997 for the Thermodynamic Properties of Water and Steam<br>
</p>
      </html>
"));
            algorithm 
              p:=sum(n[i] * (eta - 0.6) ^ I[i] * (sigma - 1.01) ^ J[i] for i in 1:33) ^ 4 * pstar;
            end p2b_hs;
            function p2c_hs "pressure as a function of enthalpy and entropy in subregion 2c"
              extends Modelica.Icons.Function;
              input SI.SpecificEnthalpy h "specific enthalpy";
              input SI.SpecificEntropy s "specific entropy";
              output SI.Pressure p "Pressure";
              constant Real[:] n = {0.112225607199012, -3.39005953606712, -32.0503911730094, -197.5973051049, -407.693861553446,13294.3775222331,1.70846839774007,37.3694198142245,3581.44365815434,423014.446424664, -751071025.760063,52.3446127607898, -228.351290812417, -960652.417056937, -80705929.2526074,1626980172256.69,0.772465073604171,46392.9973837746, -13731788.5134128,1704703926305.12, -25110462818730.8,31774883083552.0,53.8685623675312, -55308.9094625169, -1028615.22421405,2042494187562.34,273918446.626977, -2.63963146312685e+15, -1078908541.08088, -29649262098.0124, -1.11754907323424e+15};
              constant Real[:] I = {0,0,0,0,0,0,1,1,1,1,1,2,2,2,2,2,3,3,3,3,3,4,5,5,5,5,6,6,10,12,16};
              constant Real[:] J = {0,1,2,3,4,8,0,2,5,8,14,2,3,7,10,18,0,5,8,16,18,18,1,4,6,14,8,18,7,7,10};
              constant SI.SpecificEnthalpy hstar = 3500000.0 "normalization enthalpy";
              constant SI.Pressure pstar = 100000000.0 "normalization pressure";
              constant SI.SpecificEntropy sstar = 5900.0 "normalization entropy";
            protected 
              Real eta = h / hstar "normalized specific enthalpy";
              Real sigma = s / sstar "normalized specific entropy";
              annotation(Documentation(info = "<html>
      <p>
      Equation number 5 from:<br>
      The International Association for the Properties of Water and Steam<br>
      Gaithersburg, Maryland, USA<br>
      September 2001<br>
      Supplementary Release on&nbsp; Backward Equations for Pressure as a
      Function of Enthalpy and Entropy p(h,s) to the IAPWS Industrial
      Formulation 1997 for the Thermodynamic Properties of Water and Steam<br>
      </p>
      </html>
      "));
            algorithm 
              p:=sum(n[i] * (eta - 0.7) ^ I[i] * (sigma - 1.1) ^ J[i] for i in 1:31) ^ 4 * pstar;
            end p2c_hs;
            function h3ab_p "ergion 3 a b boundary for pressure/enthalpy"
              extends Modelica.Icons.Function;
              output SI.SpecificEnthalpy h "Enthalpy";
              input SI.Pressure p "Pressure";
            protected 
              constant Real[:] n = {2014.64004206875,3.74696550136983, -0.0219921901054187,8.7513168600995e-05};
              constant SI.SpecificEnthalpy hstar = 1000 "normalization enthalpy";
              constant SI.Pressure pstar = 1000000.0 "normalization pressure";
              Real pi = p / pstar "normalized specific pressure";
              annotation(Documentation(info = "<html>
      <p>
      &nbsp;Equation number 1 from:<br>
      <div style=\"text-align: center;\">&nbsp;[1] The international Association
      for the Properties of Water and Steam<br>
      &nbsp;Vejle, Denmark<br>
      &nbsp;August 2003<br>
      &nbsp;Supplementary Release on Backward Equations for the Fucnctions
      T(p,h), v(p,h) and T(p,s), <br>
      &nbsp;v(p,s) for Region 3 of the IAPWS Industrial Formulation 1997 for
      the Thermodynamic Properties of<br>
      &nbsp;Water and Steam</div>
      </p>
      </html>"));
            algorithm 
              h:=(n[1] + n[2] * pi + n[3] * pi ^ 2 + n[4] * pi ^ 3) * hstar;
            end h3ab_p;
            annotation(Documentation(info = "<html>
       <p>
       &nbsp;Equation from:<br>
       <div style=\"text-align: center;\">&nbsp;[1] The international Association
       for the Properties of Water and Steam<br>
       &nbsp;Vejle, Denmark<br>
       &nbsp;August 2003<br>
       &nbsp;Supplementary Release on Backward Equations for the Fucnctions
       T(p,h), v(p,h) and T(p,s), <br>
       &nbsp;v(p,s) for Region 3 of the IAPWS Industrial Formulation 1997 for
       the Thermodynamic Properties of<br>
       &nbsp;Water and Steam</div>
       </p>
       </html>"));
            function T3a_ph "Region 3 a: inverse function T(p,h)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEnthalpy h "specific enthalpy";
              output SI.Temp_K T "Temperature";
            protected 
              constant Real[:] n = { -1.33645667811215e-07,4.55912656802978e-06, -1.46294640700979e-05,0.0063934131297008,372.783927268847, -7186.54377460447,573494.7521034, -2675693.29111439, -3.34066283302614e-05, -0.0245479214069597,47.8087847764996,7.64664131818904e-06,0.00128350627676972,0.0171219081377331, -8.51007304583213, -0.0136513461629781, -3.84460997596657e-06,0.00337423807911655, -0.551624873066791,0.72920227710747, -0.00992522757376041, -0.119308831407288,0.793929190615421,0.454270731799386,0.20999859125991, -0.00642109823904738, -0.023515586860454,0.00252233108341612, -0.00764885133368119,0.0136176427574291, -0.0133027883575669};
              constant Real[:] I = { -12, -12, -12, -12, -12, -12, -12, -12, -10, -10, -10, -8, -8, -8, -8, -5, -3, -2, -2, -2, -1, -1,0,0,1,3,3,4,4,10,12};
              constant Real[:] J = {0,1,2,6,14,16,20,22,1,5,12,0,2,4,10,2,0,1,3,4,0,2,0,1,1,0,1,0,3,4,5};
              constant SI.SpecificEnthalpy hstar = 2300000.0 "normalization enthalpy";
              constant SI.Pressure pstar = 100000000.0 "normalization pressure";
              constant SI.Temp_K Tstar = 760 "normalization temperature";
              Real pi = p / pstar "normalized specific pressure";
              Real eta = h / hstar "normalized specific enthalpy";
              annotation(Documentation(info = "<html>
 <p>
 &nbsp;Equation number 2 from:<br>
 <div style=\"text-align: center;\">&nbsp;[1] The international Association
 for the Properties of Water and Steam<br>
 &nbsp;Vejle, Denmark<br>
 &nbsp;August 2003<br>
 &nbsp;Supplementary Release on Backward Equations for the Fucnctions
 T(p,h), v(p,h) and T(p,s), <br>
 &nbsp;v(p,s) for Region 3 of the IAPWS Industrial Formulation 1997 for
 the Thermodynamic Properties of<br>
 &nbsp;Water and Steam</div>
 </p>
 </html>"));
            algorithm 
              T:=sum(n[i] * (pi + 0.24) ^ I[i] * (eta - 0.615) ^ J[i] for i in 1:31) * Tstar;
            end T3a_ph;
            function T3b_ph "Region 3 b: inverse function T(p,h)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEnthalpy h "specific enthalpy";
              output SI.Temp_K T "Temperature";
            protected 
              constant Real[:] n = {3.2325457364492e-05, -0.000127575556587181, -0.000475851877356068,0.00156183014181602,0.105724860113781, -85.8514221132534,724.140095480911,0.00296475810273257, -0.00592721983365988, -0.0126305422818666, -0.115716196364853,84.9000969739595, -0.0108602260086615,0.0154304475328851,0.0750455441524466,0.0252520973612982, -0.0602507901232996, -3.07622221350501, -0.0574011959864879,5.03471360939849, -0.925081888584834,3.91733882917546, -77.314600713019,9493.08762098587, -1410437.19679409,8491662.30819026,0.861095729446704,0.32334644281172,0.873281936020439, -0.436653048526683,0.286596714529479, -0.131778331276228,0.00676682064330275};
              constant Real[:] I = { -12, -12, -10, -10, -10, -10, -10, -8, -8, -8, -8, -8, -6, -6, -6, -4, -4, -3, -2, -2, -1, -1, -1, -1, -1, -1,0,0,1,3,5,6,8};
              constant Real[:] J = {0,1,0,1,5,10,12,0,1,2,4,10,0,1,2,0,1,5,0,4,2,4,6,10,14,16,0,2,1,1,1,1,1};
              constant SI.Temp_K Tstar = 860 "normalization temperature";
              constant SI.Pressure pstar = 100000000.0 "normalization pressure";
              constant SI.SpecificEnthalpy hstar = 2800000.0 "normalization enthalpy";
              Real pi = p / pstar "normalized specific pressure";
              Real eta = h / hstar "normalized specific enthalpy";
              annotation(Documentation(info = "<html>
 <p>
 &nbsp;Equation number 3 from:<br>
 <div style=\"text-align: center;\">&nbsp;[1] The international Association
 for the Properties of Water and Steam<br>
 &nbsp;Vejle, Denmark<br>
 &nbsp;August 2003<br>
 &nbsp;Supplementary Release on Backward Equations for the Fucnctions
 T(p,h), v(p,h) and T(p,s), <br>
 &nbsp;v(p,s) for Region 3 of the IAPWS Industrial Formulation 1997 for
 the Thermodynamic Properties of<br>
 &nbsp;Water and Steam</div>
 </p>
 </html>"));
            algorithm 
              T:=sum(n[i] * (pi + 0.298) ^ I[i] * (eta - 0.72) ^ J[i] for i in 1:33) * Tstar;
            end T3b_ph;
            function v3a_ph "Region 3 a: inverse function v(p,h)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEnthalpy h "specific enthalpy";
              output SI.SpecificVolume v "specific volume";
            protected 
              constant Real[:] n = {0.00529944062966028, -0.170099690234461,11.1323814312927, -2178.98123145125, -0.000506061827980875,0.556495239685324, -9.43672726094016, -0.297856807561527,93.9353943717186,0.0192944939465981,0.421740664704763, -3689141.2628233, -0.00737566847600639, -0.354753242424366, -1.99768169338727,1.15456297059049,5683.6687581596,0.00808169540124668,0.172416341519307,1.04270175292927, -0.297691372792847,0.560394465163593,0.275234661176914, -0.148347894866012, -0.0651142513478515, -2.92468715386302,0.0664876096952665,3.52335014263844, -0.0146340792313332, -2.24503486668184,1.10533464706142, -0.0408757344495612};
              constant Real[:] I = { -12, -12, -12, -12, -10, -10, -10, -8, -8, -6, -6, -6, -4, -4, -3, -2, -2, -1, -1, -1, -1,0,0,1,1,1,2,2,3,4,5,8};
              constant Real[:] J = {6,8,12,18,4,7,10,5,12,3,4,22,2,3,7,3,16,0,1,2,3,0,1,0,1,2,0,2,0,2,2,2};
              constant SI.Volume vstar = 0.0028 "normalization temperature";
              constant SI.Pressure pstar = 100000000.0 "normalization pressure";
              constant SI.SpecificEnthalpy hstar = 2100000.0 "normalization enthalpy";
              Real pi = p / pstar "normalized specific pressure";
              Real eta = h / hstar "normalized specific enthalpy";
              annotation(Documentation(info = "<html>
 <p>
 &nbsp;Equation number 4 from:<br>
 <div style=\"text-align: center;\">&nbsp;[1] The international Association
 for the Properties of Water and Steam<br>
 &nbsp;Vejle, Denmark<br>
 &nbsp;August 2003<br>
 &nbsp;Supplementary Release on Backward Equations for the Fucnctions
 T(p,h), v(p,h) and T(p,s), <br>
 &nbsp;v(p,s) for Region 3 of the IAPWS Industrial Formulation 1997 for
 the Thermodynamic Properties of<br>
 &nbsp;Water and Steam</div>
 </p>
 </html>"));
            algorithm 
              v:=sum(n[i] * (pi + 0.128) ^ I[i] * (eta - 0.727) ^ J[i] for i in 1:32) * vstar;
            end v3a_ph;
            function v3b_ph "Region 3 b: inverse function v(p,h)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEnthalpy h "specific enthalpy";
              output SI.SpecificVolume v "specific volume";
            protected 
              constant Real[:] n = { -2.25196934336318e-09,1.40674363313486e-08,2.3378408528056e-06, -3.31833715229001e-05,0.00107956778514318, -0.271382067378863,1.07202262490333, -0.853821329075382, -2.15214194340526e-05,0.00076965608822273, -0.00431136580433864,0.453342167309331, -0.507749535873652, -100.475154528389, -0.219201924648793, -3.21087965668917,607.567815637771,0.000557686450685932,0.18749904002955,0.00905368030448107,0.285417173048685,0.0329924030996098,0.239897419685483,4.82754995951394, -11.8035753702231,0.169490044091791, -0.0179967222507787,0.0371810116332674, -0.0536288335065096,1.6069710109252};
              constant Real[:] I = { -12, -12, -8, -8, -8, -8, -8, -8, -6, -6, -6, -6, -6, -6, -4, -4, -4, -3, -3, -2, -2, -1, -1, -1, -1,0,1,1,2,2};
              constant Real[:] J = {0,1,0,1,3,6,7,8,0,1,2,5,6,10,3,6,10,0,2,1,2,0,1,4,5,0,0,1,2,6};
              constant SI.Volume vstar = 0.0088 "normalization temperature";
              constant SI.Pressure pstar = 100000000.0 "normalization pressure";
              constant SI.SpecificEnthalpy hstar = 2800000.0 "normalization enthalpy";
              Real pi = p / pstar "normalized specific pressure";
              Real eta = h / hstar "normalized specific enthalpy";
              annotation(Documentation(info = "<html>
 <p>
 &nbsp;Equation number 5 from:<br>
 <div style=\"text-align: center;\">&nbsp;[1] The international Association
 for the Properties of Water and Steam<br>
 &nbsp;Vejle, Denmark<br>
 &nbsp;August 2003<br>
 &nbsp;Supplementary Release on Backward Equations for the Fucnctions
 T(p,h), v(p,h) and T(p,s), <br>
 &nbsp;v(p,s) for Region 3 of the IAPWS Industrial Formulation 1997 for
 the Thermodynamic Properties of<br>
 &nbsp;Water and Steam</div>
 </p>
 </html>"));
            algorithm 
              v:=sum(n[i] * (pi + 0.0661) ^ I[i] * (eta - 0.72) ^ J[i] for i in 1:30) * vstar;
            end v3b_ph;
            function T3a_ps "Region 3 a: inverse function T(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "specific entropy";
              output SI.Temp_K T "Temperature";
            protected 
              constant Real[:] n = {1500420082.63875, -159397258480.424,0.000502181140217975, -67.2057767855466,1450.58545404456, -8238.8953488889, -0.154852214233853,11.2305046746695, -29.7000213482822,43856513263.5495,0.00137837838635464, -2.97478527157462,9717779473494.13, -5.71527767052398e-05,28830.794977842, -74442828926270.3,12.8017324848921, -368.275545889071,6.64768904779177e+15,0.044935925195888, -4.22897836099655, -0.240614376434179, -4.74341365254924,0.72409399912611,0.923874349695897,3.99043655281015,0.0384066651868009, -0.00359344365571848, -0.735196448821653,0.188367048396131,0.000141064266818704, -0.00257418501496337,0.00123220024851555};
              constant Real[:] I = { -12, -12, -10, -10, -10, -10, -8, -8, -8, -8, -6, -6, -6, -5, -5, -5, -4, -4, -4, -2, -2, -1, -1,0,0,0,1,2,2,3,8,8,10};
              constant Real[:] J = {28,32,4,10,12,14,5,7,8,28,2,6,32,0,14,32,6,10,36,1,4,1,6,0,1,4,0,0,3,2,0,1,2};
              constant SI.Temp_K Tstar = 760 "normalization temperature";
              constant SI.Pressure pstar = 100000000.0 "normalization pressure";
              constant SI.SpecificEntropy sstar = 4400.0 "normalization entropy";
              Real pi = p / pstar "normalized specific pressure";
              Real sigma = s / sstar "normalized specific entropy";
              annotation(Documentation(info = "<html>
 <p>
 &nbsp;Equation number 6 from:<br>
 <div style=\"text-align: center;\">&nbsp;[1] The international Association
 for the Properties of Water and Steam<br>
 &nbsp;Vejle, Denmark<br>
 &nbsp;August 2003<br>
 &nbsp;Supplementary Release on Backward Equations for the Fucnctions
 T(p,h), v(p,h) and T(p,s), <br>
 &nbsp;v(p,s) for Region 3 of the IAPWS Industrial Formulation 1997 for
 the Thermodynamic Properties of<br>
 &nbsp;Water and Steam</div>
 </p>
 </html>"));
            algorithm 
              T:=sum(n[i] * (pi + 0.24) ^ I[i] * (sigma - 0.703) ^ J[i] for i in 1:33) * Tstar;
            end T3a_ps;
            function T3b_ps "Region 3 b: inverse function T(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "specific entropy";
              output SI.Temp_K T "Temperature";
            protected 
              constant Real[:] n = {0.52711170160166, -40.1317830052742,153.020073134484, -2247.99398218827, -0.193993484669048, -1.40467557893768,42.6799878114024,0.752810643416743,22.6657238616417, -622.873556909932, -0.660823667935396,0.841267087271658, -25.3717501764397,485.708963532948,880.531517490555,2650155.92794626, -0.359287150025783, -656.991567673753,2.41768149185367,0.856873461222588,0.655143675313458, -0.213535213206406,0.00562974957606348, -316955725450471.0, -0.000699997000152457,0.0119845803210767,1.93848122022095e-05, -2.15095749182309e-05};
              constant Real[:] I = { -12, -12, -12, -12, -8, -8, -8, -6, -6, -6, -5, -5, -5, -5, -5, -4, -3, -3, -2,0,2,3,4,5,6,8,12,14};
              constant Real[:] J = {1,3,4,7,0,1,3,0,2,4,0,1,2,4,6,12,1,6,2,0,1,1,0,24,0,3,1,2};
              constant SI.Temp_K Tstar = 860 "normalization temperature";
              constant SI.Pressure pstar = 100000000.0 "normalization pressure";
              constant SI.SpecificEntropy sstar = 5300.0 "normalization entropy";
              Real pi = p / pstar "normalized specific pressure";
              Real sigma = s / sstar "normalized specific entropy";
              annotation(Documentation(info = "<html>
 <p>
 &nbsp;Equation number 7 from:<br>
 <div style=\"text-align: center;\">&nbsp;[1] The international Association
 for the Properties of Water and Steam<br>
 &nbsp;Vejle, Denmark<br>
 &nbsp;August 2003<br>
 &nbsp;Supplementary Release on Backward Equations for the Fucnctions
 T(p,h), v(p,h) and T(p,s), <br>
 &nbsp;v(p,s) for Region 3 of the IAPWS Industrial Formulation 1997 for
 the Thermodynamic Properties of<br>
 &nbsp;Water and Steam</div>
 </p>
 </html>"));
            algorithm 
              T:=sum(n[i] * (pi + 0.76) ^ I[i] * (sigma - 0.818) ^ J[i] for i in 1:28) * Tstar;
            end T3b_ps;
            function v3a_ps "Region 3 a: inverse function v(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "specific entropy";
              output SI.SpecificVolume v "specific volume";
            protected 
              constant Real[:] n = {79.5544074093975, -2382.6124298459,17681.3100617787, -0.00110524727080379, -15.3213833655326,297.544599376982, -35031520.6871242,0.277513761062119, -0.523964271036888, -148011.182995403,1600148.99374266,1708023226634.27,0.000246866996006494,1.6532608479798, -0.118008384666987,2.537986423559,0.965127704669424, -28.2172420532826,0.203224612353823,1.10648186063513,0.52612794845128,0.277000018736321,1.08153340501132, -0.0744127885357893,0.0164094443541384, -0.0680468275301065,0.025798857610164, -0.000145749861944416};
              constant Real[:] I = { -12, -12, -12, -10, -10, -10, -10, -8, -8, -8, -8, -6, -5, -4, -3, -3, -2, -2, -1, -1,0,0,0,1,2,4,5,6};
              constant Real[:] J = {10,12,14,4,8,10,20,5,6,14,16,28,1,5,2,4,3,8,1,2,0,1,3,0,0,2,2,0};
              constant SI.Volume vstar = 0.0028 "normalization temperature";
              constant SI.Pressure pstar = 100000000.0 "normalization pressure";
              constant SI.SpecificEntropy sstar = 4400.0 "normalization entropy";
              Real pi = p / pstar "normalized specific pressure";
              Real sigma = s / sstar "normalized specific entropy";
              annotation(Documentation(info = "<html>
 <p>
 &nbsp;Equation number 8 from:<br>
 <div style=\"text-align: center;\">&nbsp;[1] The international Association
 for the Properties of Water and Steam<br>
 &nbsp;Vejle, Denmark<br>
 &nbsp;August 2003<br>
 &nbsp;Supplementary Release on Backward Equations for the Fucnctions
 T(p,h), v(p,h) and T(p,s), <br>
 &nbsp;v(p,s) for Region 3 of the IAPWS Industrial Formulation 1997 for
 the Thermodynamic Properties of<br>
 &nbsp;Water and Steam</div>
 </p>
 </html>"));
            algorithm 
              v:=sum(n[i] * (pi + 0.187) ^ I[i] * (sigma - 0.755) ^ J[i] for i in 1:28) * vstar;
            end v3a_ps;
            function v3b_ps "Region 3 b: inverse function v(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "Pressure";
              input SI.SpecificEntropy s "specific entropy";
              output SI.SpecificVolume v "specific volume";
            protected 
              constant Real[:] n = {5.91599780322238e-05, -0.00185465997137856,0.0104190510480013,0.0059864730203859, -0.771391189901699,1.72549765557036, -0.000467076079846526,0.0134533823384439, -0.0808094336805495,0.508139374365767,0.00128584643361683, -1.63899353915435,5.86938199318063, -2.92466667918613, -0.00614076301499537,5.76199014049172, -12.1613320606788,1.67637540957944, -7.44135838773463,0.0378168091437659,4.01432203027688,16.0279837479185,3.17848779347728, -3.58362310304853, -1159952.60446827,0.199256573577909, -0.122270624794624, -19.1449143716586, -0.0150448002905284,14.6407900162154, -3.2747778718823};
              constant Real[:] I = { -12, -12, -12, -12, -12, -12, -10, -10, -10, -10, -8, -5, -5, -5, -4, -4, -4, -4, -3, -2, -2, -2, -2, -2, -2,0,0,0,1,1,2};
              constant Real[:] J = {0,1,2,3,5,6,0,1,2,4,0,1,2,3,0,1,2,3,1,0,1,2,3,4,12,0,1,2,0,2,2};
              constant SI.Volume vstar = 0.0088 "normalization temperature";
              constant SI.Pressure pstar = 100000000.0 "normalization pressure";
              constant SI.SpecificEntropy sstar = 5300.0 "normalization entropy";
              Real pi = p / pstar "normalized specific pressure";
              Real sigma = s / sstar "normalized specific entropy";
              annotation(Documentation(info = "<html>
 <p>
 &nbsp;Equation number 9 from:<br>
 <div style=\"text-align: center;\">&nbsp;[1] The international Association
 for the Properties of Water and Steam<br>
 &nbsp;Vejle, Denmark<br>
 &nbsp;August 2003<br>
 &nbsp;Supplementary Release on Backward Equations for the Fucnctions
 T(p,h), v(p,h) and T(p,s), <br>
 &nbsp;v(p,s) for Region 3 of the IAPWS Industrial Formulation 1997 for
 the Thermodynamic Properties of<br>
 &nbsp;Water and Steam</div>
 </p>
 </html>"));
            algorithm 
              v:=sum(n[i] * (pi + 0.298) ^ I[i] * (sigma - 0.816) ^ J[i] for i in 1:31) * vstar;
            end v3b_ps;
          end Basic;
          package IceBoundaries "the melting line and sublimation line curves from IAPWS"
            annotation(Documentation(info = "<html>
<p>
The International Association for the Properties of Water and Steam<br>
Milan, Italy<br>
September 1993<br>
Release on the Pressure along the Melting and the Sublimation Curves of
Ordinary Water Substance<br>
</p>
</html>
"));
            function pmIceI_T "Melting pressure of ice I (temperature range from 273.16 to 251.165 K)"
              extends Modelica.Icons.Function;
              input SI.Temp_K T "Temperature";
              output SI.Pressure pm "Melting pressure of iceI(for T from 273.16 to 251.165 K)";
            protected 
              constant SI.Temp_K Tn = 273.16 "normalization temperature";
              constant SI.Pressure pn = 611.657 "normalization pressure";
              Real sigma = T / Tn "normalized temperature";
              annotation(Documentation(info = "<html>
  <p>
  Equation 1 from:<br>
  The International Association for the Properties of Water and Steam<br>
  Milan, Italy<br>
  September 1993<br>
  Release on the Pressure along the Melting and the Sublimation Curves of
  Ordinary Water Substance<br>
  </p>
  </html>
  "));
            algorithm 
              pm:=(1 - 626000.0 * (1 - sigma ^ ( -3)) + 197135.0 * (1 - sigma ^ 21.2)) * pn;
            end pmIceI_T;
            function pmIceIII_T "Melting pressure of ice III (temperature range from 251.165 to 256.164 K)"
              extends Modelica.Icons.Function;
              input SI.Temp_K T "Temperature";
              output SI.Pressure pm "Melting pressure of iceIII(for T from 251.165 to 256.164 K)";
            protected 
              constant SI.Temp_K Tn = 251.165 "normalization temperature";
              constant SI.Pressure pn = 209900000.0 "normalization pressure";
              Real sigma = T / Tn "normalized temperature";
              annotation(Documentation(info = "<html>
  <p>
  Equation 2 from:<br>
  The International Association for the Properties of Water and Steam<br>
  Milan, Italy<br>
  September 1993<br>
  Release on the Pressure along the Melting and the Sublimation Curves of
  Ordinary Water Substance<br>
  </p>
  </html>
  "));
            algorithm 
              pm:=(1 - 0.295252 * (1 - sigma ^ 60)) * pn;
            end pmIceIII_T;
            function pmIceV_T "Melting pressure of ice V (temperature range from 256.164 to 273.31 K)"
              extends Modelica.Icons.Function;
              input SI.Temp_K T "Temperature";
              output SI.Pressure pm "Melting pressure of iceV(for T from 256.164 to 273.31 K)";
            protected 
              constant SI.Temp_K Tn = 256.164 "normalization temperature";
              constant SI.Pressure pn = 350100000.0 "normalization pressure";
              Real sigma = T / Tn "normalized temperature";
              annotation(Documentation(info = "<html>
  <p>
  Equation 3 from:<br>
  The International Association for the Properties of Water and Steam<br>
  Milan, Italy<br>
  September 1993<br>
  Release on the Pressure along the Melting and the Sublimation Curves of
  Ordinary Water Substance<br>
  </p>
  </html>
  "));
            algorithm 
              pm:=(1 - 1.18721 * (1 - sigma ^ 8)) * pn;
            end pmIceV_T;
            function sublimationPressure_T "Sublimation pressure, valid from 190 to 273.16 K"
              extends Modelica.Icons.Function;
              input SI.Temp_K T "Temperature";
              output SI.Pressure psubl "sublimation pressure (for T from 190 to 273.16)";
            protected 
              constant SI.Temp_K Tn = 273.16 "normalization temperature";
              constant SI.Pressure pn = 611.657 "normalization pressure";
              constant Real[2] a = { -13.928169,34.7078238} "constant values";
              Real sigma = T / Tn "normalized temperature";
              annotation(Documentation(info = "<html>
  <p>
  Equation 6 from:<br>
  The International Association for the Properties of Water and Steam<br>
  Milan, Italy<br>
  September 1993<br>
  Release on the Pressure along the Melting and the Sublimation Curves of
  Ordinary Water Substance<br>
  </p>
  </html>
  "));
            algorithm 
              psubl:=Modelica.Math.exp(a[1] * (1 - sigma ^ ( -1.5)) + a[2] * (1 - sigma ^ ( -1.25))) * pn;
            end sublimationPressure_T;
          end IceBoundaries;
          package Transport "transport properties for water according to IAPWS/IF97"
            extends Modelica.Icons.Library;
            annotation(Documentation(info = "<HTML><h4>Package description</h4>
          <p></p>
          <h4>Package contents</h4>
          <p>
          <ul>
          <li>Function <b>visc_dTp</b> implements a function to compute the industrial formulation of the
          dynamic viscosity of water as a function of density and temperature.
          The details are described in the document <a href=\"Documentation/IF97documentation/visc.pdf\">visc.pdf</a>.</li>
          <li>Function <b>cond_dTp</b> implements a function to compute  the industrial formulation of the thermal conductivity of water as
          a function of density, temperature and pressure. <b>Important note</b>: Obviously only two of the three
          inputs are really needed, but using three inputs speeds up the computation and the three variables are known in most models anyways.
          The inputs d,T and p have to be consistent.
          The details are described in the document <a href=\"Documentation/IF97documentation/surf.pdf\">surf.pdf</a>.</li>
          <li>Function <b>surfaceTension</b> implements a function to compute the surface tension between vapour
          and liquid water as a function of temperature.
          The details are described in the document <a href=\"Documentation/IF97documentation/thcond.pdf\">thcond.pdf</a>.</li>
          </ul>
          </p>
          <h4>Version Info and Revision history
          </h4>
          <ul>
          <li>First implemented: <i>October, 2002</i>
          by <a href=\"http://www.control.lth.se/~hubertus/\">Hubertus Tummescheit</a>
          </li>
          </ul>
          <address>Authors: Hubertus Tummescheit and Jonas Eborn<br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
          </address>
          <ul>
          <li>Initial version: October 2002</li>
          </ul>
          </HTML>
          "));
            function visc_dTp "dynamic viscosity eta(d,T,p), industrial formulation"
              extends Modelica.Icons.Function;
              input SI.Density d "density";
              input SI.Temperature T "temperature (K)";
              input SI.Pressure p "pressure (only needed for region of validity)";
              input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
              output SI.DynamicViscosity eta "dynamic viscosity";
            protected 
              constant Real n0 = 1.0 "viscosity coefficient";
              constant Real n1 = 0.978197 "viscosity coefficient";
              constant Real n2 = 0.579829 "viscosity coefficient";
              constant Real n3 =  -0.202354 "viscosity coefficient";
              constant Real[42] nn = array(0.5132047, 0.3205656, 0.0, 0.0,  -0.7782567, 0.1885447, 0.2151778, 0.7317883, 1.241044, 1.476783, 0.0, 0.0,  -0.2818107,  -1.070786,  -1.263184, 0.0, 0.0, 0.0, 0.1778064, 0.460504, 0.2340379,  -0.4924179, 0.0, 0.0,  -0.0417661, 0.0, 0.0, 0.1600435, 0.0, 0.0, 0.0,  -0.01578386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  -0.003629481, 0.0, 0.0) "viscosity coefficients";
              constant SI.Density rhostar = 317.763 "scaling density";
              constant SI.DynamicViscosity etastar = 5.5071e-05 "scaling viscosity";
              constant SI.Temperature tstar = 647.226 "scaling temperature";
              Integer i "auxiliary variable";
              Integer j "auxiliary variable";
              Real delta "dimensionless density";
              Real deltam1 "dimensionless density";
              Real tau "dimensionless temperature";
              Real taum1 "dimensionless temperature";
              Real Psi0 "auxiliary variable";
              Real Psi1 "auxiliary variable";
              Real tfun "auxiliary variable";
              Real rhofun "auxiliary variable";
              Real Tc = T - 273.15 "Celsius temperature for region check";
            algorithm 
              delta:=d / rhostar;
              assert(d > triple.dvtriple, "IF97 medium function visc_dTp for viscosity called with too low density\n" + "d = " + String(d) + " <= " + String(triple.dvtriple) + " (triple point density)");
              assert(p <= 500000000.0 and Tc >= 0.0 and Tc <= 150 or p <= 350000000.0 and Tc > 150.0 and Tc <= 600 or p <= 300000000.0 and Tc > 600.0 and Tc <= 900, "IF97 medium function visc_dTp: viscosity computed outside the range\n" + "of validity of the IF97 formulation: p = " + String(p) + " Pa, Tc = " + String(Tc) + " K");
              deltam1:=delta - 1.0;
              tau:=tstar / T;
              taum1:=tau - 1.0;
              Psi0:=1 / (n0 + (n1 + (n2 + n3 * tau) * tau) * tau) / tau ^ 0.5;
              Psi1:=0.0;
              tfun:=1.0;
              for i in 1:6 loop
                              if i <> 1 then 
                                tfun:=tfun * taum1;

                else 
                end if;
                rhofun:=1.0;
                for j in 0:6 loop
                                  if j <> 0 then 
                                    rhofun:=rhofun * deltam1;

                  else 
                  end if;
                  Psi1:=Psi1 + nn[i + j * 6] * tfun * rhofun;

                end for;

              end for;
              eta:=etastar * Psi0 * Modelica.Math.exp(delta * Psi1);
            end visc_dTp;
            function cond_dTp "Thermal conductivity lam(d,T,p) (industrial use version) only in one-phase region"
              extends Modelica.Icons.Function;
              input SI.Density d "density";
              input SI.Temperature T "temperature (K)";
              input SI.Pressure p "pressure";
              input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
              input Boolean industrialMethod = true "if true, the industrial method is used, otherwise the scientific one";
              output SI.ThermalConductivity lambda "thermal conductivity";
            protected 
              Integer region(min = 1, max = 5) "IF97 region, valid values:1,2,3, and 5";
              constant Real n0 = 1.0 "conductivity coefficient";
              constant Real n1 = 6.978267 "conductivity coefficient";
              constant Real n2 = 2.599096 "conductivity coefficient";
              constant Real n3 =  -0.998254 "conductivity coefficient";
              constant Real[30] nn = array(1.3293046, 1.7018363, 5.2246158, 8.7127675,  -1.8525999,  -0.40452437,  -2.2156845,  -10.124111,  -9.5000611, 0.9340469, 0.2440949, 1.6511057, 4.9874687, 4.3786606, 0.0, 0.018660751,  -0.76736002,  -0.27297694,  -0.91783782, 0.0,  -0.12961068, 0.37283344,  -0.43083393, 0.0, 0.0, 0.044809953,  -0.1120316, 0.13333849, 0.0, 0.0) "conductivity coefficient";
              constant SI.ThermalConductivity lamstar = 0.4945 "scaling conductivity";
              constant SI.Density rhostar = 317.763 "scaling density";
              constant SI.Temperature tstar = 647.226 "scaling temperature";
              constant SI.Pressure pstar = 22115000.0 "scaling pressure";
              constant SI.DynamicViscosity etastar = 5.5071e-05 "scaling viscosity";
              Integer i "auxiliary variable";
              Integer j "auxiliary variable";
              Real delta "dimensionless density";
              Real tau "dimensionless temperature";
              Real deltam1 "dimensionless density";
              Real taum1 "dimensionless temperature";
              Real Lam0 "part of thermal conductivity";
              Real Lam1 "part of thermal conductivity";
              Real Lam2 "part of thermal conductivity";
              Real tfun "auxiliary variable";
              Real rhofun "auxiliary variable";
              Real dpitau "auxiliary variable";
              Real ddelpi "auxiliary variable";
              Real d2 "auxiliary variable";
              Modelica.Media.Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives wrt pi and tau";
              Modelica.Media.Common.HelmholtzDerivs f "dimensionless Helmholtz function and dervatives wrt delta and tau";
              Real Tc = T - 273.15 "Celsius temperature for region check";
              Real Chi "symmetrized compressibility";
              constant SI.Density rhostar2 = 317.7 "Reference density";
              constant SI.Temperature Tstar2 = 647.25 "Reference temperature";
              constant SI.ThermalConductivity lambdastar = 1 "Reference thermal conductivity";
              parameter Real TREL = T / Tstar2 "Relative temperature";
              parameter Real rhoREL = d / rhostar2 "Relative density";
              Real lambdaREL "Relative thermal conductivity";
              Real deltaTREL "Relative temperature increment";
              constant Real[:] C = {0.642857, -4.11717, -6.17937,0.00308976,0.0822994,10.0932};
              constant Real[:] dpar = {0.0701309,0.011852,0.00169937, -1.02};
              constant Real[:] b = { -0.39707,0.400302,1.06};
              constant Real[:] B = { -0.171587,2.39219};
              constant Real[:] a = {0.0102811,0.0299621,0.0156146, -0.00422464};
              Real Q;
              Real S;
              Real lambdaREL2 "function, part of the interpolating equation of the thermal conductivity";
              Real lambdaREL1 "function, part of the interpolating equation of the thermal conductivity";
              Real lambdaREL0 "function, part of the interpolating equation of the thermal conductivity";
            algorithm 
              assert(d > triple.dvtriple, "IF97 medium function cond_dTp called with too low density\n" + "d = " + String(d) + " <= " + String(triple.dvtriple) + " (triple point density)");
              assert(p <= 100000000.0 and Tc >= 0.0 and Tc <= 500 or p <= 70000000.0 and Tc > 500.0 and Tc <= 650 or p <= 40000000.0 and Tc > 650.0 and Tc <= 800, "IF97 medium function cond_dTp: thermal conductivity computed outside the range\n" + "of validity of the IF97 formulation: p = " + String(p) + " Pa, Tc = " + String(Tc) + " K");
              if industrialMethod == true then 
                            deltaTREL:=abs(TREL - 1) + C[4];
              Q:=2 + C[5] / deltaTREL ^ (3 / 5);
              if TREL >= 1 then 
                            S:=1 / deltaTREL;

              else               S:=C[6] / deltaTREL ^ (3 / 5);

              end if;
              lambdaREL2:=(dpar[1] / TREL ^ 10 + dpar[2]) * rhoREL ^ (9 / 5) * Modelica.Math.exp(C[1] * (1 - rhoREL ^ (14 / 5))) + dpar[3] * S * rhoREL ^ Q * Modelica.Math.exp(Q / (1 + Q) * (1 - rhoREL ^ (1 + Q))) + dpar[4] * Modelica.Math.exp(C[2] * TREL ^ (3 / 2) + C[3] / rhoREL ^ 5);
              lambdaREL1:=b[1] + b[2] * rhoREL + b[3] * Modelica.Math.exp(B[1] * (rhoREL + B[2]) ^ 2);
              lambdaREL0:=TREL ^ (1 / 2) * sum(a[i] * TREL ^ (i - 1) for i in 1:4);
              lambdaREL:=lambdaREL0 + lambdaREL1 + lambdaREL2;
              lambda:=lambdaREL * lambdastar;

              else               if p < data.PLIMIT4A then 
                            if d > data.DCRIT then 
                            region:=1;

              else               region:=2;

              end if;

              else               assert(false, "the scientific method works only for temperature up to 623.15 K");

              end if;
              tau:=tstar / T;
              delta:=d / rhostar;
              deltam1:=delta - 1.0;
              taum1:=tau - 1.0;
              Lam0:=1 / (n0 + (n1 + (n2 + n3 * tau) * tau) * tau) / tau ^ 0.5;
              Lam1:=0.0;
              tfun:=1.0;
              for i in 1:5 loop
                              if i <> 1 then 
                                tfun:=tfun * taum1;

                else 
                end if;
                rhofun:=1.0;
                for j in 0:5 loop
                                  if j <> 0 then 
                                    rhofun:=rhofun * deltam1;

                  else 
                  end if;
                  Lam1:=Lam1 + nn[i + j * 5] * tfun * rhofun;

                end for;

              end for;
              if region == 1 then 
                            g:=Basic.g1(p, T);
              dpitau:= -tstar / pstar * (data.PSTAR1 * (g.gpi - data.TSTAR1 / T * g.gtaupi)) / g.gpipi / T;
              ddelpi:= -(pstar / rhostar * data.RH2O) / data.PSTAR1 / data.PSTAR1 * T * d * d * g.gpipi;
              Chi:=delta * ddelpi;
              elseif region == 2 then
                g:=Basic.g2(p, T);
                dpitau:= -tstar / pstar * (data.PSTAR2 * (g.gpi - data.TSTAR2 / T * g.gtaupi)) / g.gpipi / T;
                ddelpi:= -(pstar / rhostar * data.RH2O) / data.PSTAR2 / data.PSTAR2 * T * d * d * g.gpipi;
                Chi:=delta * ddelpi;


              else               assert(false, "thermal conductivity can only be called in the one-phase regions below 623.15 K\n" + "(p = " + String(p) + " Pa, T = " + String(T) + " K, region = " + String(region) + ")");

              end if;
              taum1:=1 / tau - 1;
              d2:=deltam1 * deltam1;
              Lam2:=(0.0013848 * etastar) / visc_dTp(d, T, p) / (tau * tau * delta * delta) * dpitau * dpitau * max(Chi, Modelica.Constants.small) ^ 0.4678 * delta ^ 0.5 * Modelica.Math.exp( -18.66 * taum1 * taum1 - d2 * d2);
              lambda:=lamstar * (Lam0 * Modelica.Math.exp(delta * Lam1) + Lam2);

              end if;
            end cond_dTp;
            function surfaceTension "surface tension in region 4 between steam and water"
              extends Modelica.Icons.Function;
              input SI.Temperature T "temperature (K)";
              output SI.SurfaceTension sigma "surface tension in SI units";
            protected 
              Real Theta "dimensionless temperature";
            algorithm 
              Theta:=min(1.0, T / data.TCRIT);
              sigma:=0.2358 * (1 - Theta) ^ 1.256 * (1 - 0.625 * (1 - Theta));
            end surfaceTension;
          end Transport;
          package Isentropic "functions for calculating the isentropic enthalpy from pressure p and specific entropy s"
            extends Modelica.Icons.Library;
            annotation(Documentation(info = "<HTML><h4>Package description</h4>
          <p></p>
          <h4>Package contents</h4>
          <p>
          <ul>
          <li>Function <b>hofpT1</b> computes h(p,T) in region 1.</li>
          <li>Function <b>handsofpT1</b> computes (s,h)=f(p,T) in region 1, needed for two-phase properties.</li>
          <li>Function <b>hofps1</b> computes h(p,s) in region 1.</li>
          <li>Function <b>hofpT2</b> computes h(p,T) in region 2.</li>
          <li>Function <b>handsofpT2</b> computes (s,h)=f(p,T) in region 2, needed for two-phase properties.</li>
          <li>Function <b>hofps2</b> computes h(p,s) in region 2.</li>
          <li>Function <b>hofdT3</b> computes h(d,T) in region 3.</li>
          <li>Function <b>hofpsdt3</b> computes h(p,s,dguess,Tguess) in region 3, where dguess and Tguess are initial guess
          values for the density and temperature consistent with p and s.</li>
          <li>Function <b>hofps4</b> computes h(p,s) in region 4.</li>
          <li>Function <b>hofpT5</b> computes h(p,T) in region 5.</li>
          <li>Function <b>water_hisentropic</b> computes h(p,s,phase) in all regions.
          The phase input is needed due to discontinuous derivatives at the phase boundary.</li>
          <li>Function <b>water_hisentropic_dyn</b> computes h(p,s,dguess,Tguess,phase) in all regions.
          The phase input is needed due to discontinuous derivatives at the phase boundary. Tguess and dguess are initial guess
          values for the density and temperature consistent with p and s. This function should be preferred in
          dynamic simulations where good guesses are often available.</li>
          </ul>
          </p>
          <h4>Version Info and Revision history
          </h4>
          <ul>
          <li>First implemented: <i>July, 2000</i>
          by <a href=\"http://www.control.lth.se/~hubertus/\">Hubertus Tummescheit</a>
          </li>
          </ul>
          <address>Author: Hubertus Tummescheit, <br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
          </address>
          <ul>
          <li>Initial version: July 2000</li>
          <li>Documentation added: December 2002</li>
          </ul>
          </HTML>
          "));
            function hofpT1 "intermediate function for isentropic specific enthalpy in region 1"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.Temperature T "temperature (K)";
              output SI.SpecificEnthalpy h "specific enthalpy";
            protected 
              Real[13] o "vector of auxiliary variables";
              Real pi1 "dimensionless pressure";
              Real tau "dimensionless temperature";
              Real tau1 "dimensionless temperature";
            algorithm 
              tau:=data.TSTAR1 / T;
              pi1:=7.1 - p / data.PSTAR1;
              assert(p > triple.ptriple, "IF97 medium function hofpT1  called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              tau1:= -1.222 + tau;
              o[1]:=tau1 * tau1;
              o[2]:=o[1] * tau1;
              o[3]:=o[1] * o[1];
              o[4]:=o[3] * o[3];
              o[5]:=o[1] * o[4];
              o[6]:=o[1] * o[3];
              o[7]:=o[3] * tau1;
              o[8]:=o[3] * o[4];
              o[9]:=pi1 * pi1;
              o[10]:=o[9] * o[9];
              o[11]:=o[10] * o[10];
              o[12]:=o[4] * o[4];
              o[13]:=o[12] * o[12];
              h:=data.RH2O * T * tau * (pi1 * (( -0.00254871721114236 + o[1] * (0.00424944110961118 + (0.018990068218419 + ( -0.021841717175414 - 0.00015851507390979 * o[1]) * o[1]) * o[6])) / o[5] + pi1 * ((0.00141552963219801 + o[3] * (4.7661393906987e-05 + o[1] * ( -1.32425535992538e-05 - 1.2358149370591e-14 * o[1] * o[3] * o[4]))) / o[3] + pi1 * ((0.000126718579380216 - 5.11230768720618e-09 * o[5]) / o[7] + pi1 * ((1.1212640954e-05 + o[2] * (1.30342445791202e-06 - 1.4341729937924e-12 * o[8])) / o[6] + pi1 * (o[9] * pi1 * ((1.40077319158051e-08 + 1.04549227383804e-09 * o[7]) / o[8] + o[10] * o[11] * pi1 * (1.9941018075704e-17 / (o[1] * o[12] * o[3] * o[4]) + o[9] * ( -4.48827542684151e-19 / o[13] + o[10] * o[9] * (pi1 * (4.65957282962769e-22 / (o[13] * o[4]) + pi1 * ((3.83502057899078e-24 * pi1) / (o[1] * o[13] * o[4]) - 7.2912378325616e-23 / (o[13] * o[4] * tau1))) - 1.00075970318621e-21 / (o[1] * o[13] * o[3] * tau1))))) + 3.24135974880936e-06 / (o[4] * tau1)))))) + ( -0.29265942426334 + tau1 * (0.84548187169114 + o[1] * (3.3855169168385 + tau1 * ( -1.91583926775744 + tau1 * (0.47316115539684 + ( -0.066465668798004 + 0.0040607314991784 * tau1) * tau1))))) / o[2]);
            end hofpT1;
            function handsofpT1 "special function for specific enthalpy and specific entropy in region 1"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.Temperature T "temperature (K)";
              output SI.SpecificEnthalpy h "specific enthalpy";
              output SI.SpecificEntropy s "specific entropy";
            protected 
              Real[28] o "vector of auxiliary variables";
              Real pi1 "dimensionless pressure";
              Real tau "dimensionless temperature";
              Real tau1 "dimensionless temperature";
              Real g "dimensionless Gibbs energy";
              Real gtau "derivative of  dimensionless Gibbs energy w.r.t. tau";
            algorithm 
              assert(p > triple.ptriple, "IF97 medium function handsofpT1 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              tau:=data.TSTAR1 / T;
              pi1:=7.1 - p / data.PSTAR1;
              tau1:= -1.222 + tau;
              o[1]:=tau1 * tau1;
              o[2]:=o[1] * o[1];
              o[3]:=o[2] * o[2];
              o[4]:=o[3] * tau1;
              o[5]:=1 / o[4];
              o[6]:=o[1] * o[2];
              o[7]:=o[1] * tau1;
              o[8]:=1 / o[7];
              o[9]:=o[1] * o[2] * o[3];
              o[10]:=1 / o[2];
              o[11]:=o[2] * tau1;
              o[12]:=1 / o[11];
              o[13]:=o[2] * o[3];
              o[14]:=pi1 * pi1;
              o[15]:=o[14] * pi1;
              o[16]:=o[14] * o[14];
              o[17]:=o[16] * o[16];
              o[18]:=o[16] * o[17] * pi1;
              o[19]:=o[14] * o[16];
              o[20]:=o[3] * o[3];
              o[21]:=o[20] * o[20];
              o[22]:=o[21] * o[3] * tau1;
              o[23]:=1 / o[22];
              o[24]:=o[21] * o[3];
              o[25]:=1 / o[24];
              o[26]:=o[1] * o[2] * o[21] * tau1;
              o[27]:=1 / o[26];
              o[28]:=o[1] * o[3];
              g:=pi1 * (pi1 * (pi1 * (o[10] * ( -3.1679644845054e-05 + o[2] * ( -2.8270797985312e-06 - 8.5205128120103e-10 * o[6])) + pi1 * (o[12] * ( -2.2425281908e-06 + ( -6.5171222895601e-07 - 1.4341729937924e-13 * o[13]) * o[7]) + pi1 * ( -4.0516996860117e-07 / o[3] + o[15] * (o[18] * (o[14] * (o[19] * (2.6335781662795e-23 / (o[1] * o[2] * o[21]) + pi1 * ( -1.1947622640071e-23 * o[27] + pi1 * (1.8228094581404e-24 * o[25] - 9.3537087292458e-26 * o[23] * pi1))) + 1.4478307828521e-20 / (o[1] * o[2] * o[20] * o[3] * tau1)) - 6.8762131295531e-19 / (o[2] * o[20] * o[3] * tau1)) + ( -1.2734301741641e-09 - 1.7424871230634e-10 * o[11]) / (o[1] * o[3] * tau1))))) + o[8] * ( -0.00047184321073267 + o[7] * ( -0.00030001780793026 + (4.7661393906987e-05 + o[1] * ( -4.4141845330846e-06 - 7.2694996297594e-16 * o[9])) * tau1))) + o[5] * (0.00028319080123804 + o[1] * ( -0.00060706301565874 + o[6] * ( -0.018990068218419 + tau1 * ( -0.032529748770505 + ( -0.021841717175414 - 5.283835796993e-05 * o[1]) * tau1))))) + (0.14632971213167 + tau1 * ( -0.84548187169114 + tau1 * ( -3.756360367204 + tau1 * (3.3855169168385 + tau1 * ( -0.95791963387872 + tau1 * (0.15772038513228 + ( -0.016616417199501 + 0.00081214629983568 * tau1) * tau1)))))) / o[1];
              gtau:=pi1 * (( -0.00254871721114236 + o[1] * (0.00424944110961118 + (0.018990068218419 + ( -0.021841717175414 - 0.00015851507390979 * o[1]) * o[1]) * o[6])) / o[28] + pi1 * (o[10] * (0.00141552963219801 + o[2] * (4.7661393906987e-05 + o[1] * ( -1.32425535992538e-05 - 1.2358149370591e-14 * o[9]))) + pi1 * (o[12] * (0.000126718579380216 - 5.11230768720618e-09 * o[28]) + pi1 * ((1.1212640954e-05 + (1.30342445791202e-06 - 1.4341729937924e-12 * o[13]) * o[7]) / o[6] + pi1 * (3.24135974880936e-06 * o[5] + o[15] * ((1.40077319158051e-08 + 1.04549227383804e-09 * o[11]) / o[13] + o[18] * (1.9941018075704e-17 / (o[1] * o[2] * o[20] * o[3]) + o[14] * ( -4.48827542684151e-19 / o[21] + o[19] * ( -1.00075970318621e-21 * o[27] + pi1 * (4.65957282962769e-22 * o[25] + pi1 * ( -7.2912378325616e-23 * o[23] + (3.83502057899078e-24 * pi1) / (o[1] * o[21] * o[3])))))))))))) + o[8] * ( -0.29265942426334 + tau1 * (0.84548187169114 + o[1] * (3.3855169168385 + tau1 * ( -1.91583926775744 + tau1 * (0.47316115539684 + ( -0.066465668798004 + 0.0040607314991784 * tau1) * tau1)))));
              h:=data.RH2O * T * tau * gtau;
              s:=data.RH2O * (tau * gtau - g);
            end handsofpT1;
            function hofps1 "function for isentropic specific enthalpy in region 1"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.SpecificEntropy s "specific entropy";
              output SI.SpecificEnthalpy h "specific enthalpy";
            protected 
              SI.Temperature T "temperature (K)";
            algorithm 
              T:=Basic.tps1(p, s);
              h:=hofpT1(p, T);
            end hofps1;
            function hofpT2 "intermediate function for isentropic specific enthalpy in region 2"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.Temperature T "temperature (K)";
              output SI.SpecificEnthalpy h "specific enthalpy";
            protected 
              Real[16] o "vector of auxiliary variables";
              Real pi "dimensionless pressure";
              Real tau "dimensionless temperature";
              Real tau2 "dimensionless temperature";
            algorithm 
              assert(p > triple.ptriple, "IF97 medium function hofpT2 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              pi:=p / data.PSTAR2;
              tau:=data.TSTAR2 / T;
              tau2:= -0.5 + tau;
              o[1]:=tau * tau;
              o[2]:=o[1] * o[1];
              o[3]:=tau2 * tau2;
              o[4]:=o[3] * tau2;
              o[5]:=o[3] * o[3];
              o[6]:=o[5] * o[5];
              o[7]:=o[6] * o[6];
              o[8]:=o[5] * o[6] * o[7] * tau2;
              o[9]:=o[3] * o[5];
              o[10]:=o[5] * o[6] * tau2;
              o[11]:=o[3] * o[7] * tau2;
              o[12]:=o[3] * o[5] * o[6];
              o[13]:=o[5] * o[6] * o[7];
              o[14]:=pi * pi;
              o[15]:=o[14] * o[14];
              o[16]:=o[7] * o[7];
              h:=data.RH2O * T * tau * ((0.0280439559151 + tau * ( -0.2858109552582 + tau * (1.2213149471784 + tau * ( -2.848163942888 + tau * (4.38395111945 + o[1] * (10.08665568018 + ( -0.5681726521544 + 0.06380539059921 * tau) * tau)))))) / (o[1] * o[2]) + pi * ( -0.017834862292358 + tau2 * ( -0.09199202739273 + ( -0.172743777250296 - 0.30195167236758 * o[4]) * tau2) + pi * ( -3.3032641670203e-05 + ( -0.0003789797503263 + o[3] * ( -0.015757110897342 + o[4] * ( -0.306581069554011 - 0.000960283724907132 * o[8]))) * tau2 + pi * (4.3870667284435e-07 + o[3] * ( -9.683303171571e-05 + o[4] * ( -0.0090203547252888 - 1.42338887469272 * o[8])) + pi * ( -7.8847309559367e-10 + (2.558143570457e-08 + 1.44676118155521e-06 * tau2) * tau2 + pi * (1.60454534363627e-05 * o[9] + pi * (( -5.0144299353183e-11 + o[10] * ( -0.033874355714168 - 836.35096769364 * o[11])) * o[3] + pi * (( -1.38839897890111e-05 - 0.973671060893475 * o[12]) * o[3] * o[6] + pi * ((9.0049690883672e-11 - 296.320827232793 * o[13]) * o[3] * o[5] * tau2 + pi * (2.57526266427144e-07 * o[5] * o[6] + pi * (o[4] * (4.1627860840696e-19 + ( -1.0234747095929e-12 - 1.40254511313154e-08 * o[5]) * o[9]) + o[14] * o[15] * (o[13] * ( -2.34560435076256e-09 + 5.3465159397045 * o[5] * o[7] * tau2) + o[14] * ( -19.1874828272775 * o[16] * o[6] * o[7] + o[14] * (o[11] * (1.78371690710842e-23 + (1.07202609066812e-11 - 0.000201611844951398 * o[10]) * o[3] * o[5] * o[6] * tau2) + pi * ( -1.24017662339842e-24 * o[5] * o[7] + pi * (0.000200482822351322 * o[16] * o[5] * o[7] + pi * ( -4.97975748452559e-14 * o[16] * o[3] * o[5] + o[6] * o[7] * (1.90027787547159e-27 + o[12] * (2.21658861403112e-15 - 5.47344301999018e-05 * o[3] * o[7])) * pi * tau2)))))))))))))))));
            end hofpT2;
            function handsofpT2 "function for isentropic specific enthalpy and specific entropy in region 2"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.Temperature T "temperature (K)";
              output SI.SpecificEnthalpy h "specific enthalpy";
              output SI.SpecificEntropy s "specific entropy";
            protected 
              Real[22] o "vector of auxiliary variables";
              Real pi "dimensionless pressure";
              Real tau "dimensionless temperature";
              Real tau2 "dimensionless temperature";
              Real g "dimensionless Gibbs energy";
              Real gtau "derivative of  dimensionless Gibbs energy w.r.t. tau";
            algorithm 
              assert(p > triple.ptriple, "IF97 medium function handsofpT2 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              tau:=data.TSTAR2 / T;
              pi:=p / data.PSTAR2;
              tau2:=tau - 0.5;
              o[1]:=tau2 * tau2;
              o[2]:=o[1] * tau2;
              o[3]:=o[1] * o[1];
              o[4]:=o[3] * o[3];
              o[5]:=o[4] * o[4];
              o[6]:=o[3] * o[4] * o[5] * tau2;
              o[7]:=o[1] * o[3] * tau2;
              o[8]:=o[3] * o[4] * tau2;
              o[9]:=o[1] * o[5] * tau2;
              o[10]:=o[1] * o[3] * o[4];
              o[11]:=o[3] * o[4] * o[5];
              o[12]:=o[1] * o[3];
              o[13]:=pi * pi;
              o[14]:=o[13] * o[13];
              o[15]:=o[13] * o[14];
              o[16]:=o[3] * o[5] * tau2;
              o[17]:=o[5] * o[5];
              o[18]:=o[3] * o[5];
              o[19]:=o[1] * o[3] * o[4] * tau2;
              o[20]:=o[1] * o[5];
              o[21]:=tau * tau;
              o[22]:=o[21] * o[21];
              g:=pi * ( -0.0017731742473213 + tau2 * ( -0.017834862292358 + tau2 * ( -0.045996013696365 + ( -0.057581259083432 - 0.05032527872793 * o[2]) * tau2)) + pi * (tau2 * ( -3.3032641670203e-05 + ( -0.00018948987516315 + o[1] * ( -0.0039392777243355 + o[2] * ( -0.043797295650573 - 2.6674547914087e-05 * o[6]))) * tau2) + pi * (2.0481737692309e-08 + (4.3870667284435e-07 + o[1] * ( -3.227767723857e-05 + o[2] * ( -0.0015033924542148 - 0.040668253562649 * o[6]))) * tau2 + pi * (tau2 * ( -7.8847309559367e-10 + (1.2790717852285e-08 + 4.8225372718507e-07 * tau2) * tau2) + pi * (2.2922076337661e-06 * o[7] + pi * (o[2] * ( -1.6714766451061e-11 + o[8] * ( -0.0021171472321355 - 23.895741934104 * o[9])) + pi * ( -5.905956432427e-18 + o[1] * ( -1.2621808899101e-06 - 0.038946842435739 * o[10]) * o[4] * tau2 + pi * ((1.1256211360459e-11 - 8.2311340897998 * o[11]) * o[4] + pi * (1.9809712802088e-08 * o[8] + pi * ((1.0406965210174e-19 + o[12] * ( -1.0234747095929e-13 - 1.0018179379511e-09 * o[3])) * o[3] + o[15] * (( -8.0882908646985e-11 + 0.10693031879409 * o[16]) * o[6] + o[13] * ( -0.33662250574171 * o[17] * o[4] * o[5] * tau2 + o[13] * (o[18] * (8.9185845355421e-25 + o[19] * (3.0629316876232e-13 - 4.2002467698208e-06 * o[8])) + pi * ( -5.9056029685639e-26 * o[16] + pi * (3.7826947613457e-06 * o[17] * o[3] * o[5] * tau2 + pi * (o[1] * (7.3087610595061e-29 + o[10] * (5.5414715350778e-17 - 9.436970724121e-07 * o[20])) * o[4] * o[5] * pi - 1.2768608934681e-15 * o[1] * o[17] * o[3] * tau2)))))))))))))))) + ( -0.00560879118302 + tau * (0.07145273881455 + tau * ( -0.4071049823928 + tau * (1.424081971444 + tau * ( -4.38395111945 + tau * ( -9.692768600217 + tau * (10.08665568018 + ( -0.2840863260772 + 0.02126846353307 * tau) * tau) + Modelica.Math.log(pi))))))) / (o[22] * tau);
              gtau:=(0.0280439559151 + tau * ( -0.2858109552582 + tau * (1.2213149471784 + tau * ( -2.848163942888 + tau * (4.38395111945 + o[21] * (10.08665568018 + ( -0.5681726521544 + 0.06380539059921 * tau) * tau)))))) / (o[21] * o[22]) + pi * ( -0.017834862292358 + tau2 * ( -0.09199202739273 + ( -0.172743777250296 - 0.30195167236758 * o[2]) * tau2) + pi * ( -3.3032641670203e-05 + ( -0.0003789797503263 + o[1] * ( -0.015757110897342 + o[2] * ( -0.306581069554011 - 0.000960283724907132 * o[6]))) * tau2 + pi * (4.3870667284435e-07 + o[1] * ( -9.683303171571e-05 + o[2] * ( -0.0090203547252888 - 1.42338887469272 * o[6])) + pi * ( -7.8847309559367e-10 + (2.558143570457e-08 + 1.44676118155521e-06 * tau2) * tau2 + pi * (1.60454534363627e-05 * o[12] + pi * (o[1] * ( -5.0144299353183e-11 + o[8] * ( -0.033874355714168 - 836.35096769364 * o[9])) + pi * (o[1] * ( -1.38839897890111e-05 - 0.973671060893475 * o[10]) * o[4] + pi * ((9.0049690883672e-11 - 296.320827232793 * o[11]) * o[7] + pi * (2.57526266427144e-07 * o[3] * o[4] + pi * (o[2] * (4.1627860840696e-19 + o[12] * ( -1.0234747095929e-12 - 1.40254511313154e-08 * o[3])) + o[15] * (o[11] * ( -2.34560435076256e-09 + 5.3465159397045 * o[16]) + o[13] * ( -19.1874828272775 * o[17] * o[4] * o[5] + o[13] * ((1.78371690710842e-23 + o[19] * (1.07202609066812e-11 - 0.000201611844951398 * o[8])) * o[9] + pi * ( -1.24017662339842e-24 * o[18] + pi * (0.000200482822351322 * o[17] * o[3] * o[5] + pi * ( -4.97975748452559e-14 * o[1] * o[17] * o[3] + (1.90027787547159e-27 + o[10] * (2.21658861403112e-15 - 5.47344301999018e-05 * o[20])) * o[4] * o[5] * pi * tau2))))))))))))))));
              h:=data.RH2O * T * tau * gtau;
              s:=data.RH2O * (tau * gtau - g);
            end handsofpT2;
            function hofps2 "function for isentropic specific enthalpy in region 2"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.SpecificEntropy s "specific entropy";
              output SI.SpecificEnthalpy h "specific enthalpy";
            protected 
              SI.Temperature T "temperature (K)";
            algorithm 
              T:=Basic.tps2(p, s);
              h:=hofpT2(p, T);
            end hofps2;
            function hofdT3 "function for isentropic specific enthalpy in region 3"
              extends Modelica.Icons.Function;
              input SI.Density d "density";
              input SI.Temperature T "temperature (K)";
              output SI.SpecificEnthalpy h "specific enthalpy";
            protected 
              Real delta;
              Real tau "dimensionless temperature";
              Real[13] o "vector of auxiliary variables";
              Real ftau "derivative of  dimensionless Helmholtz energy w.r.t. tau";
              Real fdelta "derivative of  dimensionless Helmholtz energy w.r.t. delta";
            algorithm 
              tau:=data.TCRIT / T;
              delta:=d / data.DCRIT;
              o[1]:=tau * tau;
              o[2]:=o[1] * o[1];
              o[3]:=o[2] * o[2];
              o[4]:=o[3] * tau;
              o[5]:=o[1] * o[2] * o[3] * tau;
              o[6]:=o[2] * o[3];
              o[7]:=o[1] * o[3];
              o[8]:=o[3] * o[3];
              o[9]:=o[1] * o[2] * o[8];
              o[10]:=o[1] * o[2] * o[8] * tau;
              o[11]:=o[3] * o[8];
              o[12]:=o[3] * o[8] * tau;
              o[13]:=o[1] * o[3] * o[8];
              ftau:=20.944396974307 + tau * ( -15.3735415757432 + o[2] * tau * (18.3301634515678 + o[1] * tau * ( -28.08078114862 + o[1] * (14.4640436358204 - 0.194503669468755 * o[1] * o[3] * tau)))) + delta * (( -2.5308630955428 + o[2] * ( -6.9146446840086 + (13.2781565976477 - 10.9153200808732 * o[1]) * o[4])) * tau + delta * (tau * ( -1.70429417648412 + o[2] * (29.3833689251262 + ( -21.3518320798755 + (0.867251811341388 + 3.26518619032008 * o[2]) * o[5]) * tau)) + delta * ((2.779959913892 + o[1] * ( -8.075966009428 + o[6] * ( -0.131436219478341 - 12.37496929108 * o[7]))) * tau + delta * (( -0.88952870857478 + o[1] * (3.62288282878932 + 18.3358370228714 * o[9])) * tau + delta * (0.10770512626332 + o[1] * ( -0.98740869776862 - 13.2264761307011 * o[10]) + delta * ((0.188521503330184 + 4.27343239646986 * o[11]) * tau + delta * ( -0.027006744482696 * tau + delta * ( -0.385692979164272 * o[12] + delta * (delta * ( -0.00016557679795037 - 0.00116802137560719 * delta * o[12]) + (0.00115845907256168 + 0.0840031522296486 * o[11]) * tau)))))))));
              fdelta:=(1.0658070028513 + delta * (o[1] * ( -1.2654315477714 + o[2] * ( -1.1524407806681 + (0.88521043984318 - 0.64207765181607 * o[1]) * o[4])) + delta * (0.76986920373342 + o[1] * ( -1.70429417648412 + o[2] * (9.7944563083754 + ( -6.100523451393 + (0.078841073758308 + 0.25116816848616 * o[2]) * o[5]) * tau)) + delta * ( -0.8399798909613 + o[1] * (4.169939870838 + o[1] * ( -6.056974507071 + o[6] * ( -0.0246442911521889 - 1.42788107204769 * o[7]))) + delta * (0.175936297894 + o[1] * ( -1.77905741714956 + o[1] * (3.62288282878932 + 2.82089800351868 * o[9])) + delta * (delta * ( -0.133052405238576 + o[1] * (0.565564509990552 + 0.98617670687766 * o[11]) + delta * ( -0.094523605689436 * o[1] + delta * ( -0.118674762819776 * o[13] + delta * (o[1] * (0.00521306582652756 + 0.0290780142333399 * o[11]) + delta * (0.00080964802996215 - 0.000494162889679965 * delta * o[13] - 0.0016557679795037 * tau))))) + (0.5385256313166 + o[1] * ( -1.6456811629477 - 2.5435531020579 * o[10])) * tau)))))) / delta;
              h:=data.RH2O * T * (tau * ftau + delta * fdelta);
            end hofdT3;
            function hofps3 "isentropic specific enthalpy in region 3 h(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.SpecificEntropy s "specific entropy";
              output SI.SpecificEnthalpy h "specific enthalpy";
            protected 
              SI.Density d "density";
              SI.Temperature T "temperature (K)";
              SI.Pressure delp = IterationData.DELP "iteration accuracy";
              SI.SpecificEntropy dels = IterationData.DELS "iteration accuracy";
              Integer error "error if not 0";
            algorithm 
              (d,T,error):=Inverses.dtofps3(p = p, s = s, delp = delp, dels = dels);
              h:=hofdT3(d, T);
            end hofps3;
            function hofpsdt3 "isentropic specific enthalpy in region 3 h(p,s) with given good guess in d and T"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.SpecificEntropy s "specific entropy";
              input SI.Density dguess "good guess density, e.g. from adjacent volume";
              input SI.Temperature Tguess "good guess temperature, e.g. from adjacent volume";
              input SI.Pressure delp = IterationData.DELP "relative error in p";
              input SI.SpecificEntropy dels = IterationData.DELS "relative error in s";
              output SI.SpecificEnthalpy h "specific enthalpy";
            protected 
              SI.Density d "density";
              SI.Temperature T "temperature (K)";
              Integer error "error flag";
            algorithm 
              (d,T,error):=Inverses.dtofpsdt3(p = p, s = s, dguess = dguess, Tguess = Tguess, delp = delp, dels = dels);
              h:=hofdT3(d, T);
            end hofpsdt3;
            function hofps4 "isentropic specific enthalpy in region 4 h(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.SpecificEntropy s "specific entropy";
              output SI.SpecificEnthalpy h "specific enthalpy";
            protected 
              SI.Temp_K Tsat "saturation temperature";
              SI.MassFraction x "dryness fraction";
              SI.SpecificEntropy sl "saturated liquid specific entropy";
              SI.SpecificEntropy sv "saturated vapour specific entropy";
              SI.SpecificEnthalpy hl "saturated liquid specific enthalpy";
              SI.SpecificEnthalpy hv "saturated vapour specific enthalpy";
            algorithm 
              if p <= data.PLIMIT4A then 
                            Tsat:=Basic.tsat(p);
              (hl,sl):=handsofpT1(p, Tsat);
              (hv,sv):=handsofpT2(p, Tsat);
              elseif p < data.PCRIT then
                sl:=Regions.sl_p_R4b(p);
                sv:=Regions.sv_p_R4b(p);
                hl:=Regions.hl_p_R4b(p);
                hv:=Regions.hv_p_R4b(p);


              else 
              end if;
              x:=max(min(if sl <> sv then (s - sl) / (sv - sl) else 1.0, 1.0), 0.0);
              h:=hl + x * (hv - hl);
            end hofps4;
            function hofpT5 "specific enthalpy in region 5 h(p,T)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.Temperature T "temperature (K)";
              output SI.SpecificEnthalpy h "specific enthalpy";
            protected 
              Real[4] o "vector of auxiliary variables";
              Real tau "dimensionless temperature";
              Real pi "dimensionless pressure";
            algorithm 
              tau:=data.TSTAR5 / T;
              pi:=p / data.PSTAR5;
              assert(p > triple.ptriple, "IF97 medium function hofpT5 called with too low pressure\n" + "p = " + String(p) + " Pa <= " + String(triple.ptriple) + " Pa (triple point pressure)");
              o[1]:=tau * tau;
              o[2]:=o[1] * o[1];
              o[3]:=pi * pi;
              o[4]:=o[2] * o[2];
              h:=data.RH2O * T * tau * (6.8540841634434 + 3.1161318213925 / o[1] + 0.074415446800398 / o[2] - 3.57523455236121e-05 * o[3] * o[4] + 0.0021774678714571 * pi - 0.013782846269973 * o[1] * pi + 3.8757684869352e-07 * o[1] * o[3] * pi - 0.73803069960666 / (o[1] * tau) - 0.65923253077834 * tau);
            end hofpT5;
            function water_hisentropic "isentropic specific enthalpy from p,s (preferably use water_hisentropic_dyn in dynamic simulation!)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.SpecificEntropy s "specific entropy";
              input Integer phase = 0 "phase: 2 for two-phase, 1 for one phase, 0 if unknown";
              output SI.SpecificEnthalpy h "specific enthalpy";
            protected 
              Modelica.Media.Common.GibbsDerivs g "derivatives of dimensionless Gibbs-function w.r.t dimensionless pi and tau";
              Modelica.Media.Common.HelmholtzDerivs f "derivatives of dimensionless Helmholtz-function w.r.t dimensionless delta and tau";
              Integer region(min = 1, max = 5) "IF97 region";
              Integer error "error if not 0";
              SI.Temperature T "temperature";
              SI.Density d "density";
            algorithm 
              region:=Regions.region_ps(p = p, s = s, phase = phase);
              if region == 1 then 
                            h:=hofps1(p, s);
              elseif region == 2 then
                h:=hofps2(p, s);

              elseif region == 3 then
                (d,T,error):=Inverses.dtofps3(p = p, s = s, delp = IterationData.DELP, dels = IterationData.DELS);
                h:=hofdT3(d, T);

              elseif region == 4 then
                h:=hofps4(p, s);

              elseif region == 5 then
                (T,error):=Inverses.tofps5(p = p, s = s, relds = IterationData.DELS);
                h:=hofpT5(p, T);


              else 
              end if;
            end water_hisentropic;
            function water_hisentropic_dyn "isentropic specific enthalpy from p,s and good guesses of d and T"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.SpecificEntropy s "specific entropy";
              input SI.Density dguess "good guess density, e.g. from adjacent volume";
              input SI.Temperature Tguess "good guess temperature, e.g. from adjacent volume";
              input Integer phase "1 for one phase, 2 for two phase";
              output SI.SpecificEnthalpy h "specific enthalpy";
            protected 
              Modelica.Media.Common.GibbsDerivs g "derivatives of dimensionless Gibbs-function w.r.t dimensionless pi and tau";
              Modelica.Media.Common.HelmholtzDerivs f "derivatives of dimensionless Helmholtz-function w.r.t dimensionless delta and tau";
              Integer region(min = 1, max = 5) "IF97 region";
              Integer error "error if not 0";
              SI.Temperature T "temperature";
              SI.Density d "density";
            algorithm 
              region:=Regions.region_ps(p = p, s = s, phase = phase);
              if region == 1 then 
                            h:=hofps1(p, s);
              elseif region == 2 then
                h:=hofps2(p, s);

              elseif region == 3 then
                h:=hofpsdt3(p = p, s = s, dguess = dguess, Tguess = Tguess, delp = IterationData.DELP, dels = IterationData.DELS);

              elseif region == 4 then
                h:=hofps4(p, s);

              elseif region == 5 then
                (T,error):=Inverses.tofpst5(p = p, s = s, Tguess = Tguess, relds = IterationData.DELS);
                h:=hofpT5(p, T);


              else 
              end if;
            end water_hisentropic_dyn;
          end Isentropic;
          package Inverses "efficient inverses for selected pairs of variables"
            annotation(Documentation(info = "<HTML><h4>Package description</h4>
          <p></p>
          <h4>Package contents</h4>
          <p>
          <ul>
          <li>Function <b>fixdT</b> constrains density and temperature to allowed region</li>
          <li>Function <b>dofp13</b> computes d as a function of p at boundary between regions 1 and 3</li>
          <li>Function <b>dofp23</b> computes d as a function of p at boundary between regions 2 and 3</li>
          <li>Function <b>dofpt3</b> iteration to compute d as a function of p and T in region 3</li>
          <li>Function <b>dtofph3</b> iteration to compute d and T as a function of p and h in region 3</li>
          <li>Function <b>dtofps3</b> iteration to compute d and T as a function of p and s in region 3</li>
          <li>Function <b>dtofpsdt3</b> iteration to compute d and T as a function of p and s in region 3,
          with initial guesses</li>
          <li>Function <b>pofdt125</b> iteration to compute p as a function of p and T in regions 1, 2 and 5</li>
          <li>Function <b>tofph5</b> iteration to compute T as a function of p and h in region 5</li>
          <li>Function <b>tofps5</b> iteration to compute T as a function of p and s in region 5</li>
          <li>Function <b>tofpst5</b> iteration to compute T as a function of p and s in region 5, with initial guess in T</li>
          <li>Function <b></b></li>
          </ul>
          </p>
          <h4>Version Info and Revision history
          </h4>
          <ul>
          <li>First implemented: <i>July, 2000</i>
          by <a href=\"http://www.control.lth.se/~hubertus/\">Hubertus Tummescheit</a>
          </li>
          </ul>
          <address>Author: Hubertus Tummescheit, <br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
          </address>
          <ul>
          <li>Initial version: July 2000</li>
          <li>Documentation added: December 2002</li>
          </ul>
          </HTML>
          "));
            extends Modelica.Icons.Library;
            function fixdT "region limits for inverse iteration in region 3"
              extends Modelica.Icons.Function;
              input SI.Density din "density";
              input SI.Temperature Tin "temperature";
              output SI.Density dout "density";
              output SI.Temperature Tout "temperature";
            protected 
              SI.Temperature Tmin "approximation of minimum temperature";
              SI.Temperature Tmax "approximation of maximum temperature";
            algorithm 
              if din > 765.0 then 
                            dout:=765.0;
              elseif din < 110.0 then
                dout:=110.0;


              else               dout:=din;

              end if;
              if dout < 390.0 then 
                            Tmax:=554.3557377 + dout * 0.809344262;

              else               Tmax:=1116.85 - dout * 0.632948717;

              end if;
              if dout < data.DCRIT then 
                            Tmin:=data.TCRIT * (1.0 - ((dout - data.DCRIT) * (dout - data.DCRIT)) / 1000000.0);

              else               Tmin:=data.TCRIT * (1.0 - ((dout - data.DCRIT) * (dout - data.DCRIT)) / 1440000.0);

              end if;
              if Tin < Tmin then 
                            Tout:=Tmin;
              elseif Tin > Tmax then
                Tout:=Tmax;


              else               Tout:=Tin;

              end if;
            end fixdT;
            function dofp13 "density at the boundary between regions 1 and 3"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.Density d "density";
            protected 
              Real p2 "auxiliary variable";
              Real[3] o "vector of auxiliary variables";
            algorithm 
              p2:=7.1 - 6.04960677555959e-08 * p;
              o[1]:=p2 * p2;
              o[2]:=o[1] * o[1];
              o[3]:=o[2] * o[2];
              d:=57.4756752485113 / (0.0737412153522555 + p2 * (0.00145092247736023 + p2 * (0.000102697173772229 + p2 * (1.14683182476084e-05 + p2 * (1.99080616601101e-06 + o[1] * p2 * (1.13217858826367e-08 + o[2] * o[3] * p2 * (1.35549330686006e-17 + o[1] * ( -3.11228834832975e-19 + o[1] * o[2] * ( -7.02987180039442e-22 + p2 * (3.29199117056433e-22 + ( -5.17859076694812e-23 + 2.73712834080283e-24 * p2) * p2))))))))));
            end dofp13;
            function dofp23 "density at the boundary between regions 2 and 3"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output SI.Density d "density";
            protected 
              SI.Temperature T;
              Real[13] o "vector of auxiliary variables";
              Real taug "auxiliary variable";
              Real pi "dimensionless pressure";
              Real gpi23 "derivative of g w.r.t. pi on the boundary between regions 2 and 3";
            algorithm 
              pi:=p / data.PSTAR2;
              T:=572.54459862746 + 31.3220101646784 * ( -13.91883977887 + pi) ^ 0.5;
              o[1]:=( -13.91883977887 + pi) ^ 0.5;
              taug:= -0.5 + 540.0 / (572.54459862746 + 31.3220101646784 * o[1]);
              o[2]:=taug * taug;
              o[3]:=o[2] * taug;
              o[4]:=o[2] * o[2];
              o[5]:=o[4] * o[4];
              o[6]:=o[5] * o[5];
              o[7]:=o[4] * o[5] * o[6] * taug;
              o[8]:=o[4] * o[5] * taug;
              o[9]:=o[2] * o[4] * o[5];
              o[10]:=pi * pi;
              o[11]:=o[10] * o[10];
              o[12]:=o[4] * o[6] * taug;
              o[13]:=o[6] * o[6];
              gpi23:=(1.0 + pi * ( -0.0017731742473213 + taug * ( -0.017834862292358 + taug * ( -0.045996013696365 + ( -0.057581259083432 - 0.05032527872793 * o[3]) * taug)) + pi * (taug * ( -6.6065283340406e-05 + ( -0.0003789797503263 + o[2] * ( -0.007878555448671 + o[3] * ( -0.087594591301146 - 5.3349095828174e-05 * o[7]))) * taug) + pi * (6.1445213076927e-08 + (1.31612001853305e-06 + o[2] * ( -9.683303171571e-05 + o[3] * ( -0.0045101773626444 - 0.122004760687947 * o[7]))) * taug + pi * (taug * ( -3.15389238237468e-09 + (5.116287140914e-08 + 1.92901490874028e-06 * taug) * taug) + pi * (1.14610381688305e-05 * o[2] * o[4] * taug + pi * (o[3] * ( -1.00288598706366e-10 + o[8] * ( -0.012702883392813 - 143.374451604624 * o[2] * o[6] * taug)) + pi * ( -4.1341695026989e-17 + o[2] * o[5] * ( -8.8352662293707e-06 - 0.272627897050173 * o[9]) * taug + pi * (o[5] * (9.0049690883672e-11 - 65.8490727183984 * o[4] * o[5] * o[6]) + pi * (1.78287415218792e-07 * o[8] + pi * (o[4] * (1.0406965210174e-18 + o[2] * ( -1.0234747095929e-12 - 1.0018179379511e-08 * o[4]) * o[4]) + o[10] * o[11] * (( -1.29412653835176e-09 + 1.71088510070544 * o[12]) * o[7] + o[10] * ( -6.05920510335078 * o[13] * o[5] * o[6] * taug + o[10] * (o[4] * o[6] * (1.78371690710842e-23 + o[2] * o[4] * o[5] * (6.1258633752464e-12 - 8.4004935396416e-05 * o[8]) * taug) + pi * ( -1.24017662339842e-24 * o[12] + pi * (8.32192847496054e-05 * o[13] * o[4] * o[6] * taug + pi * (o[2] * o[5] * o[6] * (1.75410265428146e-27 + (1.32995316841867e-15 - 2.26487297378904e-05 * o[2] * o[6]) * o[9]) * pi - 2.93678005497663e-14 * o[13] * o[2] * o[4] * taug))))))))))))))))) / pi;
              d:=p / (data.RH2O * T * pi * gpi23);
            end dofp23;
            function dofpt3 "inverse iteration in region 3: (d) = f(p,T)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.Temperature T "temperature (K)";
              input SI.Pressure delp "iteration converged if (p-pre(p) < delp)";
              output SI.Density d "density";
              output Integer error = 0 "error flag: iteration failed if different from 0";
            protected 
              SI.Density dguess "guess density";
              Integer i = 0 "loop counter";
              Real dp "pressure difference";
              SI.Density deld "density step";
              Modelica.Media.Common.HelmholtzDerivs f "dimensionless Helmholtz function and dervatives wrt delta and tau";
              Modelica.Media.Common.NewtonDerivatives_pT nDerivs "derivatives needed in Newton iteration";
              Boolean found = false "flag for iteration success";
              Boolean supercritical "flag, true for supercritical states";
              Boolean liquid "flag, true for liquid states";
              SI.Density dmin "lower density limit";
              SI.Density dmax "upper density limit";
              SI.Temperature Tmax "maximum temperature";
            algorithm 
              assert(p >= data.PLIMIT4A, "BaseIF97.dofpt3: function called outside of region 3! p too low\n" + "p = " + String(p) + " Pa < " + String(data.PLIMIT4A) + " Pa");
              assert(T >= data.TLIMIT1, "BaseIF97.dofpt3: function called outside of region 3! T too low\n" + "T = " + String(T) + " K < " + String(data.TLIMIT1) + " K");
              assert(p >= Regions.boundary23ofT(T), "BaseIF97.dofpt3: function called outside of region 3! T too high\n" + "p = " + String(p) + " Pa, T = " + String(T) + " K");
              supercritical:=p > data.PCRIT;
              dmax:=dofp13(p);
              dmin:=dofp23(p);
              Tmax:=Regions.boundary23ofp(p);
              if supercritical then 
                            dguess:=dmin + (T - data.TLIMIT1) / (data.TLIMIT1 - Tmax) * (dmax - dmin);

              else               liquid:=T < Basic.tsat(p);
              if liquid then 
                            dguess:=0.5 * (Regions.rhol_p_R4b(p) + dmax);

              else               dguess:=0.5 * (Regions.rhov_p_R4b(p) + dmin);

              end if;

              end if;
              while (i < IterationData.IMAX and not found) loop
                              d:=dguess;
                f:=Basic.f3(d, T);
                nDerivs:=Modelica.Media.Common.Helmholtz_pT(f);
                dp:=nDerivs.p - p;
                if abs(dp / p) <= delp then 
                                found:=true;

                else 
                end if;
                deld:=dp / nDerivs.pd;
                d:=d - deld;
                if d > dmin and d < dmax then 
                                dguess:=d;

                else                 if d > dmax then 
                                dguess:=dmax - sqrt(Modelica.Constants.eps);

                else                 dguess:=dmin + sqrt(Modelica.Constants.eps);

                end if;

                end if;
                i:=i + 1;

              end while;
              if not found then 
                            error:=1;

              else 
              end if;
              assert(error <> 1, "error in inverse function dofpt3: iteration failed");
            end dofpt3;
            function dtofph3 "inverse iteration in region 3: (d,T) = f(p,h)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.SpecificEnthalpy h "specific enthalpy";
              input SI.Pressure delp "iteration accuracy";
              input SI.SpecificEnthalpy delh "iteration accuracy";
              output SI.Density d "density";
              output SI.Temperature T "temperature (K)";
              output Integer error "error flag: iteration failed if different from 0";
            protected 
              SI.Temperature Tguess "initial temperature";
              SI.Density dguess "initial density";
              Integer i "iteration counter";
              Real dh "Newton-error in h-direction";
              Real dp "Newton-error in p-direction";
              Real det "determinant of directional derivatives";
              Real deld "Newton-step in d-direction";
              Real delt "Newton-step in T-direction";
              Modelica.Media.Common.HelmholtzDerivs f "dimensionless Helmholtz function and dervatives wrt delta and tau";
              Modelica.Media.Common.NewtonDerivatives_ph nDerivs "derivatives needed in Newton iteration";
              Boolean found "flag for iteration success";
              Integer subregion "1 for subregion 3a, 2 for subregion 3b";
            algorithm 
              if p < data.PCRIT then 
                            subregion:=if h < Regions.hl_p(p) + 10.0 then 1 else if h > Regions.hv_p(p) - 10.0 then 2 else 0;
              assert(subregion <> 0, "inverse iteration of dt from ph called in 2 phase region: this can not work");

              else               subregion:=if h < Basic.h3ab_p(p) then 1 else 2;

              end if;
              T:=if subregion == 1 then Basic.T3a_ph(p, h) else Basic.T3b_ph(p, h);
              d:=if subregion == 1 then 1 / Basic.v3a_ph(p, h) else 1 / Basic.v3b_ph(p, h);
              i:=0;
              error:=0;
              while (i < IterationData.IMAX and not found) loop
                              f:=Basic.f3(d, T);
                nDerivs:=Modelica.Media.Common.Helmholtz_ph(f);
                dh:=nDerivs.h - h;
                dp:=nDerivs.p - p;
                if abs(dh / h) <= delh and abs(dp / p) <= delp then 
                                found:=true;

                else 
                end if;
                det:=nDerivs.ht * nDerivs.pd - nDerivs.pt * nDerivs.hd;
                delt:=(nDerivs.pd * dh - nDerivs.hd * dp) / det;
                deld:=(nDerivs.ht * dp - nDerivs.pt * dh) / det;
                T:=T - delt;
                d:=d - deld;
                dguess:=d;
                Tguess:=T;
                i:=i + 1;
                (d,T):=fixdT(dguess, Tguess);

              end while;
              if not found then 
                            error:=1;

              else 
              end if;
              assert(error <> 1, "error in inverse function dtofph3: iteration failed");
            end dtofph3;
            function dtofps3 "inverse iteration in region 3: (d,T) = f(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.SpecificEntropy s "specific entropy";
              input SI.Pressure delp "iteration accuracy";
              input SI.SpecificEntropy dels "iteration accuracy";
              output SI.Density d "density";
              output SI.Temperature T "temperature (K)";
              output Integer error "error flag: iteration failed if different from 0";
            protected 
              SI.Temperature Tguess "initial temperature";
              SI.Density dguess "initial density";
              Integer i "iteration counter";
              Real ds "Newton-error in s-direction";
              Real dp "Newton-error in p-direction";
              Real det "determinant of directional derivatives";
              Real deld "Newton-step in d-direction";
              Real delt "Newton-step in T-direction";
              Modelica.Media.Common.HelmholtzDerivs f "dimensionless Helmholtz function and dervatives wrt delta and tau";
              Modelica.Media.Common.NewtonDerivatives_ps nDerivs "derivatives needed in Newton iteration";
              Boolean found "flag for iteration success";
              Integer subregion "1 for subregion 3a, 2 for subregion 3b";
            algorithm 
              i:=0;
              error:=0;
              found:=false;
              if p < data.PCRIT then 
                            subregion:=if s < Regions.sl_p(p) + 10.0 then 1 else if s > Regions.sv_p(p) - 10.0 then 2 else 0;
              assert(subregion <> 0, "inverse iteration of dt from ps called in 2 phase region: this is illegal!");

              else               subregion:=if s < data.SCRIT then 1 else 2;

              end if;
              T:=if subregion == 1 then Basic.T3a_ps(p, s) else Basic.T3b_ps(p, s);
              d:=if subregion == 1 then 1 / Basic.v3a_ps(p, s) else 1 / Basic.v3b_ps(p, s);
              while (i < IterationData.IMAX and not found) loop
                              f:=Basic.f3(d, T);
                nDerivs:=Modelica.Media.Common.Helmholtz_ps(f);
                ds:=nDerivs.s - s;
                dp:=nDerivs.p - p;
                if abs(ds / s) <= dels and abs(dp / p) <= delp then 
                                found:=true;

                else 
                end if;
                det:=nDerivs.st * nDerivs.pd - nDerivs.pt * nDerivs.sd;
                delt:=(nDerivs.pd * ds - nDerivs.sd * dp) / det;
                deld:=(nDerivs.st * dp - nDerivs.pt * ds) / det;
                T:=T - delt;
                d:=d - deld;
                dguess:=d;
                Tguess:=T;
                i:=i + 1;
                (d,T):=fixdT(dguess, Tguess);

              end while;
              if not found then 
                            error:=1;

              else 
              end if;
              assert(error <> 1, "error in inverse function dtofps3: iteration failed");
            end dtofps3;
            function dtofpsdt3 "inverse iteration in region 3: (d,T) = f(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.SpecificEntropy s "specific entropy";
              input SI.Density dguess "guess density, e.g. from adjacent volume";
              input SI.Temperature Tguess "guess temperature, e.g. from adjacent volume";
              input SI.Pressure delp "iteration accuracy";
              input SI.SpecificEntropy dels "iteration accuracy";
              output SI.Density d "density";
              output SI.Temperature T "temperature (K)";
              output Integer error "error flag: iteration failed if different from 0";
            protected 
              Integer i "iteration counter";
              Real ds "Newton-error in s-direction";
              Real dp "Newton-error in p-direction";
              Real det "determinant of directional derivatives";
              Real deld "Newton-step in d-direction";
              Real delt "Newton-step in T-direction";
              Modelica.Media.Common.HelmholtzDerivs f "dimensionless Helmholtz function and dervatives wrt delta and tau";
              Modelica.Media.Common.NewtonDerivatives_ps nDerivs "derivatives needed in Newton iteration";
              Boolean found "flag for iteration success";
              SI.Density diter "density";
              SI.Temperature Titer "temperature (K)";
            algorithm 
              i:=0;
              error:=0;
              found:=false;
              (diter,Titer):=fixdT(dguess, Tguess);
              while (i < IterationData.IMAX and not found) loop
                              (d,T):=fixdT(diter, Titer);
                f:=Basic.f3(d, T);
                nDerivs:=Modelica.Media.Common.Helmholtz_ps(f);
                ds:=nDerivs.s - s;
                dp:=nDerivs.p - p;
                if abs(ds / s) <= dels and abs(dp / p) <= delp then 
                                found:=true;

                else 
                end if;
                det:=nDerivs.st * nDerivs.pd - nDerivs.pt * nDerivs.sd;
                delt:=(nDerivs.pd * ds - nDerivs.sd * dp) / det;
                deld:=(nDerivs.st * dp - nDerivs.pt * ds) / det;
                T:=T - delt;
                d:=d - deld;
                diter:=d;
                Titer:=T;
                i:=i + 1;

              end while;
              if not found then 
                            error:=1;

              else 
              end if;
              assert(error <> 1, "error in inverse function dtofpsdt3: iteration failed");
            end dtofpsdt3;
            function pofdt125 "inverse iteration in region 1,2 and 5: p = g(d,T)"
              extends Modelica.Icons.Function;
              input SI.Density d "density";
              input SI.Temperature T "temperature (K)";
              input SI.Pressure reldd "relative iteration accuracy of density";
              input Integer region "region in IAPWS/IF97 in which inverse should be calculated";
              output SI.Pressure p "pressure";
              output Integer error "error flag: iteration failed if different from 0";
            protected 
              Integer i "counter for while-loop";
              Modelica.Media.Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives wrt pi and tau";
              Boolean found "flag if iteration has been successful";
              Real dd "difference between density for  guessed p and the current density";
              Real delp "step in p in Newton-iteration";
              Real relerr "relative error in d";
              SI.Pressure pguess1 = 1000000.0 "initial pressure guess in region 1";
              SI.Pressure pguess2 "initial pressure guess in region 2";
              constant SI.Pressure pguess5 = 500000.0 "initial pressure guess in region 5";
            algorithm 
              i:=0;
              error:=0;
              pguess2:=42800 * d;
              found:=false;
              if region == 1 then 
                            p:=pguess1;
              elseif region == 2 then
                p:=pguess2;


              else               p:=pguess5;

              end if;
              while (i < IterationData.IMAX and not found) loop
                              if region == 1 then 
                                g:=Basic.g1(p, T);
                elseif region == 2 then
                  g:=Basic.g2(p, T);


                else                 g:=Basic.g5(p, T);

                end if;
                dd:=p / (data.RH2O * T * g.pi * g.gpi) - d;
                relerr:=dd / d;
                if abs(relerr) < reldd then 
                                found:=true;

                else 
                end if;
                delp:=dd * ( -(p * p) / (d * d * data.RH2O * T * g.pi * g.pi * g.gpipi));
                p:=p - delp;
                i:=i + 1;
                if not found then 
                                if p < triple.ptriple then 
                                p:=2.0 * triple.ptriple;

                else 
                end if;
                if p > data.PLIMIT1 then 
                                p:=0.95 * data.PLIMIT1;

                else 
                end if;

                else 
                end if;

              end while;
              if not found then 
                            error:=1;

              else 
              end if;
              assert(error <> 1, "error in inverse function pofdt125: iteration failed");
            end pofdt125;
            function tofph5 "inverse iteration in region 5: (p,T) = f(p,h)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.SpecificEnthalpy h "specific enthalpy";
              input SI.SpecificEnthalpy reldh "iteration accuracy";
              output SI.Temperature T "temperature (K)";
              output Integer error "error flag: iteration failed if different from 0";
            protected 
              Modelica.Media.Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives wrt pi and tau";
              SI.SpecificEnthalpy proh "h for current guess in T";
              constant SI.Temperature Tguess = 1500 "initial temperature";
              Integer i "iteration counter";
              Real relerr "relative error in h";
              Real dh "Newton-error in h-direction";
              Real dT "Newton-step in T-direction";
              Boolean found "flag for iteration success";
            algorithm 
              i:=0;
              error:=0;
              T:=Tguess;
              found:=false;
              while (i < IterationData.IMAX and not found) loop
                              g:=Basic.g5(p, T);
                proh:=data.RH2O * T * g.tau * g.gtau;
                dh:=proh - h;
                relerr:=dh / h;
                if abs(relerr) < reldh then 
                                found:=true;

                else 
                end if;
                dT:=dh / ( -data.RH2O * g.tau * g.tau * g.gtautau);
                T:=T - dT;
                i:=i + 1;

              end while;
              if not found then 
                            error:=1;

              else 
              end if;
              assert(error <> 1, "error in inverse function tofph5: iteration failed");
            end tofph5;
            function tofps5 "inverse iteration in region 5: (p,T) = f(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.SpecificEntropy s "specific entropy";
              input SI.SpecificEnthalpy relds "iteration accuracy";
              output SI.Temperature T "temperature (K)";
              output Integer error "error flag: iteration failed if different from 0";
            protected 
              Modelica.Media.Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives wrt pi and tau";
              SI.SpecificEntropy pros "s for current guess in T";
              parameter SI.Temperature Tguess = 1500 "initial temperature";
              Integer i "iteration counter";
              Real relerr "relative error in s";
              Real ds "Newton-error in s-direction";
              Real dT "Newton-step in T-direction";
              Boolean found "flag for iteration success";
            algorithm 
              i:=0;
              error:=0;
              T:=Tguess;
              found:=false;
              while (i < IterationData.IMAX and not found) loop
                              g:=Basic.g5(p, T);
                pros:=data.RH2O * (g.tau * g.gtau - g.g);
                ds:=pros - s;
                relerr:=ds / s;
                if abs(relerr) < relds then 
                                found:=true;

                else 
                end if;
                dT:=(ds * T) / ( -data.RH2O * g.tau * g.tau * g.gtautau);
                T:=T - dT;
                i:=i + 1;

              end while;
              if not found then 
                            error:=1;

              else 
              end if;
              assert(error <> 1, "error in inverse function tofps5: iteration failed");
            end tofps5;
            function tofpst5 "inverse iteration in region 5: (p,T) = f(p,s)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.SpecificEntropy s "specific entropy";
              input SI.Temperature Tguess "guess temperature, e.g. from adjacent volume";
              input SI.SpecificEntropy relds "iteration accuracy";
              output SI.Temperature T "temperature (K)";
              output Integer error "error flag: iteration failed if different from 0";
            protected 
              Modelica.Media.Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives wrt pi and tau";
              SI.SpecificEntropy pros "s for current guess in T";
              Integer i "iteration counter";
              Real relerr "relative error in s";
              Real ds "Newton-error in s-direction";
              Real dT "Newton-step in T-direction";
              Boolean found "flag for iteration success";
            algorithm 
              i:=0;
              error:=0;
              T:=Tguess;
              found:=false;
              while (i < IterationData.IMAX and not found) loop
                              g:=Basic.g5(p, T);
                pros:=data.RH2O * (g.tau * g.gtau - g.g);
                ds:=pros - s;
                relerr:=ds / s;
                if abs(relerr) < relds then 
                                found:=true;

                else 
                end if;
                dT:=(ds * T) / ( -data.RH2O * g.tau * g.tau * g.gtautau);
                T:=T - dT;
                i:=i + 1;

              end while;
              if not found then 
                            error:=1;

              else 
              end if;
              assert(error <> 1, "error in inverse function tofpst5: iteration failed");
            end tofpst5;
          end Inverses;
          package ByRegion "simple explicit functions for one region only"
            annotation(Documentation(info = "<HTML><h4>Package description</h4>
          <p>Package ByRegion provides fast forward calls for dynamic property calculation records for all
          one phase regions of IAPWS/IF97
          </p>
          <h4>Package contents</h4>
          <p>
          <ul>
          <li>Function <b>waterR1_pT</b> computes dynamic properties for region 1 using  (p,T) as inputs</li>
          <li>Function <b>waterR2_pT</b> computes dynamic properties for region 2 using  (p,T) as inputs</li>
          <li>Function <b>waterR3_dT</b> computes dynamic properties for region 3 using  (d,T) as inputs</li>
          <li>Function <b>waterR5_pT</b> computes dynamic properties for region 5 using  (p,T) as inputs</li>
          </ul>
          </p>
          <h4>Version Info and Revision history
          </h4>
          <ul>
          <li>First implemented: <i>July, 2000</i>
          by <a href=\"http://www.control.lth.se/~hubertus/\">Hubertus Tummescheit</a>
          </li>
          </ul>
          <address>Author: Hubertus Tummescheit, <br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
          </address>
          <ul>
          <li>Initial version: July 2000</li>
          <li>Documented and re-organized: January 2003</li>
          </ul>
          </HTML>
          "));
            extends Modelica.Icons.Library;
            function waterR1_pT "standard properties for region 1, (p,T) as inputs"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.Temperature T "temperature (K)";
              output Modelica.Media.Common.ThermoFluidSpecial.ThermoProperties_pT pro "thermodynamic property collection";
            protected 
              Modelica.Media.Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives wrt pi and tau";
            algorithm 
              g:=Basic.g1(p, T);
              pro:=Modelica.Media.Common.ThermoFluidSpecial.gibbsToProps_pT(g);
            end waterR1_pT;
            function waterR2_pT "standard properties for region 2, (p,T) as inputs"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.Temperature T "temperature (K)";
              output Modelica.Media.Common.ThermoFluidSpecial.ThermoProperties_pT pro "thermodynamic property collection";
            protected 
              Modelica.Media.Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives wrt pi and tau";
            algorithm 
              g:=Basic.g2(p, T);
              pro:=Modelica.Media.Common.ThermoFluidSpecial.gibbsToProps_pT(g);
            end waterR2_pT;
            function waterR3_dT "standard properties for region 3, (d,T) as inputs"
              extends Modelica.Icons.Function;
              input SI.Density d "density";
              input SI.Temperature T "temperature (K)";
              output Common.ThermoFluidSpecial.ThermoProperties_dT pro "thermodynamic property collection";
            protected 
              Common.HelmholtzDerivs f "dimensionless Helmholtz function and dervatives wrt delta and tau";
            algorithm 
              f:=Basic.f3(d, T);
              pro:=Modelica.Media.Common.ThermoFluidSpecial.helmholtzToProps_dT(f);
              assert(pro.p <= 100000000.0, "IF97 medium function waterR3_dT: the input pressure (= " + String(pro.p) + " Pa) is higher than 100 Mpa");
            end waterR3_dT;
            function waterR5_pT "standard properties for region 5, (p,T) as inputs"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.Temperature T "temperature (K)";
              output Modelica.Media.Common.ThermoFluidSpecial.ThermoProperties_pT pro "thermodynamic property collection";
            protected 
              Modelica.Media.Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives wrt pi and tau";
            algorithm 
              g:=Basic.g5(p, T);
              pro:=Modelica.Media.Common.ThermoFluidSpecial.gibbsToProps_pT(g);
            end waterR5_pT;
          end ByRegion;
          package TwoPhase "steam properties in the two-phase rgion and on the phase boundaries"
            extends Modelica.Icons.Library;
            annotation(Documentation(info = "<HTML><h4>Package description</h4>
          <p>Package TwoPhase provides functions to compute the steam properties
          in the two-phase region and on the phase boundaries</p>
          <h4>Package contents</h4>
          <p>
          <ul>
          <li>Function <b>WaterLiq_p</b> computes properties on the boiling boundary as a function of p</li>
          <li>Function <b>WaterVap_p</b> computes properties on the dew line boundary as a function of p</li>
          <li>Function <b>WaterSat_ph</b> computes properties on both phase boundaries and in the two
          phase region as a function of p</li>
          <li>Function <b>WaterR4_ph</b> computes dynamic simulation properties in region 4 with (p,h) as inputs</li>
          <li>Function <b>WaterR4_dT</b> computes dynamic simulation properties in region 4 with (d,T) as inputs</li>
          </ul>
          </p>
          <h4>Version Info and Revision history
          </h4>
          <ul>
          <li>First implemented: <i>July, 2000</i>
          by <a href=\"http://www.control.lth.se/~hubertus/\">Hubertus Tummescheit</a>
          </li>
          </ul>
          <address>Author: Hubertus Tummescheit, <br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
          </address>
          <ul>
          <li>Initial version: July 2000</li>
          <li>Documented and re-organized: January 2003</li>
          </ul>
          </HTML>
"));
            function waterLiq_p "properties on the liquid phase boundary of region 4"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output Modelica.Media.Common.PhaseBoundaryProperties liq "liquid thermodynamic property collection";
            protected 
              SI.Temperature Tsat "saturation temperature";
              Real dpT "derivative of saturation pressure wrt temperature";
              SI.Density dl "liquid density";
              Modelica.Media.Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives wrt pi and tau";
              Modelica.Media.Common.HelmholtzDerivs f "dimensionless Helmholtz function and dervatives wrt delta and tau";
            algorithm 
              Tsat:=Basic.tsat(p);
              dpT:=Basic.dptofT(Tsat);
              if p < data.PLIMIT4A then 
                            g:=Basic.g1(p, Tsat);
              liq:=Modelica.Media.Common.gibbsToBoundaryProps(g);

              else               dl:=Regions.rhol_p_R4b(p);
              f:=Basic.f3(dl, Tsat);
              liq:=Modelica.Media.Common.helmholtzToBoundaryProps(f);

              end if;
            end waterLiq_p;
            function waterVap_p "properties on the vapour phase boundary of region 4"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              output Modelica.Media.Common.PhaseBoundaryProperties vap "vapour thermodynamic property collection";
            protected 
              SI.Temperature Tsat "saturation temperature";
              Real dpT "derivative of saturation pressure wrt temperature";
              SI.Density dv "vapour density";
              Modelica.Media.Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives wrt pi and tau";
              Modelica.Media.Common.HelmholtzDerivs f "dimensionless Helmholtz function and dervatives wrt delta and tau";
            algorithm 
              Tsat:=Basic.tsat(p);
              dpT:=Basic.dptofT(Tsat);
              if p < data.PLIMIT4A then 
                            g:=Basic.g2(p, Tsat);
              vap:=Modelica.Media.Common.gibbsToBoundaryProps(g);

              else               dv:=Regions.rhov_p_R4b(p);
              f:=Basic.f3(dv, Tsat);
              vap:=Modelica.Media.Common.helmholtzToBoundaryProps(f);

              end if;
            end waterVap_p;
            function waterSat_ph "Water saturation properties in the 2-phase region (4) as f(p,h)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.SpecificEnthalpy h "specific enthalpy";
              output Modelica.Media.Common.SaturationProperties pro "thermodynamic property collection";
            protected 
              SI.Density dl "liquid density";
              SI.Density dv "vapour density";
              Modelica.Media.Common.GibbsDerivs gl "dimensionless Gibbs funcion and dervatives wrt pi and tau";
              Modelica.Media.Common.GibbsDerivs gv "dimensionless Gibbs funcion and dervatives wrt pi and tau";
              Modelica.Media.Common.HelmholtzDerivs fl "dimensionless Helmholtz function and dervatives wrt delta and tau";
              Modelica.Media.Common.HelmholtzDerivs fv "dimensionless Helmholtz function and dervatives wrt delta and tau";
            algorithm 
              pro.h:=h;
              pro.p:=p;
              pro.T:=Basic.tsat(p);
              pro.dpT:=Basic.dptofT(pro.T);
              if p < data.PLIMIT4A then 
                            gl:=Basic.g1(p, pro.T);
              gv:=Basic.g2(p, pro.T);
              pro.liq:=Modelica.Media.Common.gibbsToBoundaryProps(gl);
              pro.vap:=Modelica.Media.Common.gibbsToBoundaryProps(gv);

              else               dl:=Regions.rhol_p_R4b(p);
              dv:=Regions.rhov_p_R4b(p);
              fl:=Basic.f3(dl, pro.T);
              fv:=Basic.f3(dv, pro.T);
              pro.liq:=Modelica.Media.Common.helmholtzToBoundaryProps(fl);
              pro.vap:=Modelica.Media.Common.helmholtzToBoundaryProps(fv);

              end if;
              pro.x:=if h < pro.liq.h then 0.0 else if pro.vap.h <> pro.liq.h then (h - pro.liq.h) / (pro.vap.h - pro.liq.h) else 1.0;
              pro.d:=(pro.liq.d * pro.vap.d) / (pro.vap.d + pro.x * (pro.liq.d - pro.vap.d));
              pro.u:=pro.x * pro.vap.u + (1 - pro.x) * pro.liq.u;
              pro.s:=pro.x * pro.vap.s + (1 - pro.x) * pro.liq.s;
              pro.cp:=Modelica.Constants.inf;
              pro.cv:=Modelica.Media.Common.cv2Phase(pro.liq, pro.vap, pro.x, pro.T, p);
              pro.kappa:=(1 / (pro.d * p) * pro.dpT * pro.dpT * pro.T) / pro.cv;
              pro.R:=data.RH2O;
            end waterSat_ph;
            function waterR4_ph "Water/Steam properties in region 4 of IAPWS/IF97 (two-phase)"
              extends Modelica.Icons.Function;
              input SI.Pressure p "pressure";
              input SI.SpecificEnthalpy h "specific enthalpy";
              output Modelica.Media.Common.ThermoFluidSpecial.ThermoProperties_ph pro "thermodynamic property collection";
            protected 
              SI.Density dl "liquid density";
              SI.Density dv "vapour density";
              Modelica.Media.Common.PhaseBoundaryProperties liq "phase boundary property record";
              Modelica.Media.Common.PhaseBoundaryProperties vap "phase boundary property record";
              Modelica.Media.Common.GibbsDerivs gl "dimensionless Gibbs funcion and dervatives wrt pi and tau";
              Modelica.Media.Common.GibbsDerivs gv "dimensionless Gibbs funcion and dervatives wrt pi and tau";
              Modelica.Media.Common.HelmholtzDerivs fl "dimensionless Helmholtz function and dervatives wrt delta and tau";
              Modelica.Media.Common.HelmholtzDerivs fv "dimensionless Helmholtz function and dervatives wrt delta and tau";
              Real x "dryness fraction";
              Real dpT "derivative of saturation curve";
            algorithm 
              pro.T:=Basic.tsat(p);
              dpT:=Basic.dptofT(pro.T);
              dl:=Regions.rhol_p_R4b(p);
              dv:=Regions.rhov_p_R4b(p);
              if p < data.PLIMIT4A then 
                            gl:=Basic.g1(p, pro.T);
              gv:=Basic.g2(p, pro.T);
              liq:=Modelica.Media.Common.gibbsToBoundaryProps(gl);
              vap:=Modelica.Media.Common.gibbsToBoundaryProps(gv);

              else               fl:=Basic.f3(dl, pro.T);
              fv:=Basic.f3(dv, pro.T);
              liq:=Modelica.Media.Common.helmholtzToBoundaryProps(fl);
              vap:=Modelica.Media.Common.helmholtzToBoundaryProps(fv);

              end if;
              x:=if vap.h <> liq.h then (h - liq.h) / (vap.h - liq.h) else 1.0;
              pro.d:=(liq.d * vap.d) / (vap.d + x * (liq.d - vap.d));
              pro.u:=x * vap.u + (1 - x) * liq.u;
              pro.s:=x * vap.s + (1 - x) * liq.s;
              pro.cp:=Modelica.Constants.inf;
              pro.cv:=Modelica.Media.Common.cv2Phase(liq, vap, x, pro.T, p);
              pro.kappa:=(1 / (pro.d * p) * dpT * dpT * pro.T) / pro.cv;
              pro.a:=Modelica.Constants.inf;
              pro.R:=data.RH2O;
              pro.ddph:=(pro.d * ((pro.d * pro.cv) / dpT + 1.0)) / (dpT * pro.T);
              pro.ddhp:= -(pro.d * pro.d) / (dpT * pro.T);
            end waterR4_ph;
            function waterR4_dT "Water properties in region 4 as function of d and T"
              extends Modelica.Icons.Function;
              input SI.Density d "Density";
              input SI.Temperature T "temperature";
              output Modelica.Media.Common.ThermoFluidSpecial.ThermoProperties_dT pro "thermodynamic property collection";
            protected 
              SI.Density dl "liquid density";
              SI.Density dv "vapour density";
              Modelica.Media.Common.PhaseBoundaryProperties liq "phase boundary property record";
              Modelica.Media.Common.PhaseBoundaryProperties vap "phase boundary property record";
              Modelica.Media.Common.GibbsDerivs gl "dimensionless Gibbs funcion and dervatives wrt pi and tau";
              Modelica.Media.Common.GibbsDerivs gv "dimensionless Gibbs funcion and dervatives wrt pi and tau";
              Modelica.Media.Common.HelmholtzDerivs fl "dimensionless Helmholtz function and dervatives wrt delta and tau";
              Modelica.Media.Common.HelmholtzDerivs fv "dimensionless Helmholtz function and dervatives wrt delta and tau";
              Real x "dryness fraction";
              Real dpT "derivative of saturation curve";
            algorithm 
              pro.p:=Basic.psat(T);
              dpT:=Basic.dptofT(T);
              dl:=Regions.rhol_p_R4b(pro.p);
              dv:=Regions.rhov_p_R4b(pro.p);
              if pro.p < data.PLIMIT4A then 
                            gl:=Basic.g1(pro.p, T);
              gv:=Basic.g2(pro.p, T);
              liq:=Modelica.Media.Common.gibbsToBoundaryProps(gl);
              vap:=Modelica.Media.Common.gibbsToBoundaryProps(gv);

              else               fl:=Basic.f3(dl, T);
              fv:=Basic.f3(dv, T);
              liq:=Modelica.Media.Common.helmholtzToBoundaryProps(fl);
              vap:=Modelica.Media.Common.helmholtzToBoundaryProps(fv);

              end if;
              x:=if vap.d <> liq.d then (1 / d - 1 / liq.d) / (1 / vap.d - 1 / liq.d) else 1.0;
              pro.u:=x * vap.u + (1 - x) * liq.u;
              pro.h:=x * vap.h + (1 - x) * liq.h;
              pro.cp:=Modelica.Constants.inf;
              pro.cv:=Modelica.Media.Common.cv2Phase(liq, vap, x, T, pro.p);
              pro.kappa:=(1 / (d * pro.p) * dpT * dpT * T) / pro.cv;
              pro.a:=Modelica.Constants.inf;
              pro.R:=data.RH2O;
              pro.dudT:=(pro.p - T * dpT) / (d * d);
            end waterR4_dT;
          end TwoPhase;
          function extraDerivs_ph "function to calculate some extra thermophysical properties in regions 1, 2, 3 and 5 as f(p,h)"
            extends Modelica.Icons.Function;
            input SI.Pressure p "pressure";
            input SI.SpecificEnthalpy h "specific enthalpy";
            input Integer phase = 0 "phase: 2 for two-phase, 1 for one phase, 0 if unknown";
            output Modelica.Media.Common.ExtraDerivatives dpro "thermodynamic property collection";
          protected 
            SI.Density d "density";
            SI.Temperature T "temperature";
            Integer region "IF97 region";
            Integer error "error flag";
            Modelica.Media.Common.HelmholtzDerivs f "dimensionless Helmholtz function and dervatives wrt delta and tau";
            Modelica.Media.Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives wrt pi and tau";
          algorithm 
            assert(phase == 1, "extraDerivs_ph: properties are not implemented in 2 phase region");
            region:=Regions.region_ph(p = p, h = h, phase = phase);
            if region == 1 then 
                        T:=Basic.tph1(p, h);
            g:=Basic.g1(p, T);
            dpro:=Modelica.Media.Common.gibbsToExtraDerivs(g);
            elseif region == 2 then
              T:=Basic.tph2(p, h);
              g:=Basic.g2(p, T);
              dpro:=Modelica.Media.Common.gibbsToExtraDerivs(g);

            elseif region == 3 then
              (d,T,error):=Inverses.dtofph3(p = p, h = h, delp = 1e-07, delh = 1e-06);
              f:=Basic.f3(d, T);
              dpro:=Modelica.Media.Common.helmholtzToExtraDerivs(f);

            elseif region == 5 then
              (T,error):=Inverses.tofph5(p = p, h = h, reldh = 1e-07);
              g:=Basic.g5(p, T);
              dpro:=Modelica.Media.Common.gibbsToExtraDerivs(g);


            else 
            end if;
          end extraDerivs_ph;
          function extraDerivs_pT "function to calculate some extra thermophysical properties in regions 1, 2, 3 and 5 as f(p,T)"
            extends Modelica.Icons.Function;
            input SI.Pressure p "pressure";
            input SI.Temperature T "temperature";
            output Modelica.Media.Common.ExtraDerivatives dpro "thermodynamic property collection";
          protected 
            SI.Density d "density";
            Integer region "IF97 region";
            Integer error "error flag";
            Modelica.Media.Common.HelmholtzDerivs f "dimensionless Helmholtz function and dervatives wrt delta and tau";
            Modelica.Media.Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives wrt pi and tau";
          algorithm 
            region:=Regions.region_pT(p = p, T = T);
            if region == 1 then 
                        g:=Basic.g1(p, T);
            dpro:=Modelica.Media.Common.gibbsToExtraDerivs(g);
            elseif region == 2 then
              g:=Basic.g2(p, T);
              dpro:=Modelica.Media.Common.gibbsToExtraDerivs(g);

            elseif region == 3 then
              (d,error):=Inverses.dofpt3(p = p, T = T, delp = 1e-07);
              f:=Basic.f3(d, T);
              dpro:=Modelica.Media.Common.helmholtzToExtraDerivs(f);

            elseif region == 5 then
              g:=Basic.g5(p, T);
              dpro:=Modelica.Media.Common.gibbsToExtraDerivs(g);


            else 
            end if;
          end extraDerivs_pT;
        end BaseIF97;
        replaceable record iter = BaseIF97.IterationData;
        function waterBaseProp_ph "intermediate property record for water"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Integer phase = 0 "phase: 2 for two-phase, 1 for one phase, 0 if unknown";
          input Integer region = 0 "if 0, do region computation, otherwise assume the region is this input";
          output Common.IF97BaseTwoPhase aux "auxiliary record";
        protected 
          Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives wrt pi and tau";
          Common.HelmholtzDerivs f "dimensionless Helmholtz funcion and dervatives wrt delta and tau";
          Integer error "error flag for inverse iterations";
          SI.SpecificEnthalpy h_liq "liquid specific enthalpy";
          SI.Density d_liq "liquid density";
          SI.SpecificEnthalpy h_vap "vapour specific enthalpy";
          SI.Density d_vap "vapour density";
          Common.PhaseBoundaryProperties liq "phase boundary property record";
          Common.PhaseBoundaryProperties vap "phase boundary property record";
          Common.GibbsDerivs gl "dimensionless Gibbs funcion and dervatives wrt pi and tau";
          Common.GibbsDerivs gv "dimensionless Gibbs funcion and dervatives wrt pi and tau";
          Modelica.Media.Common.HelmholtzDerivs fl "dimensionless Helmholtz function and dervatives wrt delta and tau";
          Modelica.Media.Common.HelmholtzDerivs fv "dimensionless Helmholtz function and dervatives wrt delta and tau";
          SI.Temperature t1 "temperature at phase boundary, using inverse from region 1";
          SI.Temperature t2 "temperature at phase boundary, using inverse from region 2";
        algorithm 
          aux.region:=if region == 0 then if phase == 2 then 4 else BaseIF97.Regions.region_ph(p = p, h = h, phase = phase) else region;
          aux.phase:=if phase <> 0 then phase else if aux.region == 4 then 2 else 1;
          aux.p:=max(p, 611.657);
          aux.h:=max(h, 1000.0);
          aux.R:=BaseIF97.data.RH2O;
          if aux.region == 1 then 
                    aux.T:=BaseIF97.Basic.tph1(aux.p, aux.h);
          g:=BaseIF97.Basic.g1(p, aux.T);
          aux.s:=aux.R * (g.tau * g.gtau - g.g);
          aux.rho:=p / (aux.R * aux.T * g.pi * g.gpi);
          aux.vt:=aux.R / p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
          aux.vp:=(aux.R * aux.T) / (p * p) * g.pi * g.pi * g.gpipi;
          aux.cp:= -aux.R * g.tau * g.tau * g.gtautau;
          aux.cv:=aux.R * ( -g.tau * g.tau * g.gtautau + ((g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi)) / g.gpipi);
          aux.x:=0.0;
          aux.dpT:= -aux.vt / aux.vp;
          elseif aux.region == 2 then
            aux.T:=BaseIF97.Basic.tph2(aux.p, aux.h);
            g:=BaseIF97.Basic.g2(p, aux.T);
            aux.s:=aux.R * (g.tau * g.gtau - g.g);
            aux.rho:=p / (aux.R * aux.T * g.pi * g.gpi);
            aux.vt:=aux.R / p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
            aux.vp:=(aux.R * aux.T) / (p * p) * g.pi * g.pi * g.gpipi;
            aux.cp:= -aux.R * g.tau * g.tau * g.gtautau;
            aux.cv:=aux.R * ( -g.tau * g.tau * g.gtautau + ((g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi)) / g.gpipi);
            aux.x:=1.0;
            aux.dpT:= -aux.vt / aux.vp;

          elseif aux.region == 3 then
            (aux.rho,aux.T,error):=BaseIF97.Inverses.dtofph3(p = aux.p, h = aux.h, delp = 1e-07, delh = 1e-06);
            f:=BaseIF97.Basic.f3(aux.rho, aux.T);
            aux.h:=aux.R * aux.T * (f.tau * f.ftau + f.delta * f.fdelta);
            aux.s:=aux.R * (f.tau * f.ftau - f.f);
            aux.pd:=aux.R * aux.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
            aux.pt:=aux.R * aux.rho * f.delta * (f.fdelta - f.tau * f.fdeltatau);
            aux.cv:=abs(aux.R * ( -f.tau * f.tau * f.ftautau)) "can be close to neg. infinity near critical point";
            aux.cp:=(aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt) / (aux.rho * aux.rho * aux.pd);
            aux.x:=0.0;
            aux.dpT:=aux.pt;

          elseif aux.region == 4 then
            h_liq:=hl_p(p);
            h_vap:=hv_p(p);
            aux.x:=if h_vap <> h_liq then (h - h_liq) / (h_vap - h_liq) else 1.0;
            if p < BaseIF97.data.PLIMIT4A then 
                        t1:=BaseIF97.Basic.tph1(aux.p, h_liq);
            t2:=BaseIF97.Basic.tph2(aux.p, h_vap);
            gl:=BaseIF97.Basic.g1(aux.p, t1);
            gv:=BaseIF97.Basic.g2(aux.p, t2);
            liq:=Common.gibbsToBoundaryProps(gl);
            vap:=Common.gibbsToBoundaryProps(gv);
            aux.T:=t1 + aux.x * (t2 - t1);

            else             aux.T:=BaseIF97.Basic.tsat(aux.p);
            d_liq:=rhol_T(aux.T);
            d_vap:=rhov_T(aux.T);
            fl:=BaseIF97.Basic.f3(d_liq, aux.T);
            fv:=BaseIF97.Basic.f3(d_vap, aux.T);
            liq:=Common.helmholtzToBoundaryProps(fl);
            vap:=Common.helmholtzToBoundaryProps(fv);

            end if;
            aux.dpT:=if liq.d <> vap.d then ((vap.s - liq.s) * liq.d * vap.d) / (liq.d - vap.d) else BaseIF97.Basic.dptofT(aux.T);
            aux.s:=liq.s + aux.x * (vap.s - liq.s);
            aux.rho:=(liq.d * vap.d) / (vap.d + aux.x * (liq.d - vap.d));
            aux.cv:=Common.cv2Phase(liq, vap, aux.x, aux.T, p);
            aux.cp:=liq.cp + aux.x * (vap.cp - liq.cp);
            aux.pt:=liq.pt + aux.x * (vap.pt - liq.pt);
            aux.pd:=liq.pd + aux.x * (vap.pd - liq.pd);

          elseif aux.region == 5 then
            (aux.T,error):=BaseIF97.Inverses.tofph5(p = aux.p, h = aux.h, reldh = 1e-07);
            assert(error == 0, "error in inverse iteration of steam tables");
            g:=BaseIF97.Basic.g5(aux.p, aux.T);
            aux.s:=aux.R * (g.tau * g.gtau - g.g);
            aux.rho:=p / (aux.R * aux.T * g.pi * g.gpi);
            aux.vt:=aux.R / p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
            aux.vp:=(aux.R * aux.T) / (p * p) * g.pi * g.pi * g.gpipi;
            aux.cp:= -aux.R * g.tau * g.tau * g.gtautau;
            aux.cv:=aux.R * ( -g.tau * g.tau * g.gtautau + ((g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi)) / g.gpipi);
            aux.dpT:= -aux.vt / aux.vp;


          else           assert(false, "error in region computation of IF97 steam tables" + "(p = " + String(p) + ", h = " + String(h) + ")");

          end if;
        end waterBaseProp_ph;
        function waterBaseProp_ps "intermediate property record for water"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEntropy s "specific entropy";
          input Integer phase = 0 "phase: 2 for two-phase, 1 for one phase, 0 if unknown";
          input Integer region = 0 "if 0, do region computation, otherwise assume the region is this input";
          output Common.IF97BaseTwoPhase aux "auxiliary record";
        protected 
          Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives wrt pi and tau";
          Common.HelmholtzDerivs f "dimensionless Helmholtz funcion and dervatives wrt delta and tau";
          Integer error "error flag for inverse iterations";
          SI.SpecificEntropy s_liq "liquid specific entropy";
          SI.Density d_liq "liquid density";
          SI.SpecificEntropy s_vap "vapour specific entropy";
          SI.Density d_vap "vapour density";
          Common.PhaseBoundaryProperties liq "phase boundary property record";
          Common.PhaseBoundaryProperties vap "phase boundary property record";
          Common.GibbsDerivs gl "dimensionless Gibbs funcion and dervatives wrt pi and tau";
          Common.GibbsDerivs gv "dimensionless Gibbs funcion and dervatives wrt pi and tau";
          Modelica.Media.Common.HelmholtzDerivs fl "dimensionless Helmholtz function and dervatives wrt delta and tau";
          Modelica.Media.Common.HelmholtzDerivs fv "dimensionless Helmholtz function and dervatives wrt delta and tau";
          SI.Temperature t1 "temperature at phase boundary, using inverse from region 1";
          SI.Temperature t2 "temperature at phase boundary, using inverse from region 2";
        algorithm 
          aux.region:=if region == 0 then if phase == 2 then 4 else BaseIF97.Regions.region_ps(p = p, s = s, phase = phase) else region;
          aux.phase:=if phase <> 0 then phase else if aux.region == 4 then 2 else 1;
          aux.p:=p;
          aux.s:=s;
          aux.R:=BaseIF97.data.RH2O;
          if aux.region == 1 then 
                    aux.T:=BaseIF97.Basic.tps1(p, s);
          g:=BaseIF97.Basic.g1(p, aux.T);
          aux.h:=aux.R * aux.T * g.tau * g.gtau;
          aux.rho:=p / (aux.R * aux.T * g.pi * g.gpi);
          aux.vt:=aux.R / p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
          aux.vp:=(aux.R * aux.T) / (p * p) * g.pi * g.pi * g.gpipi;
          aux.cp:= -aux.R * g.tau * g.tau * g.gtautau;
          aux.cv:=aux.R * ( -g.tau * g.tau * g.gtautau + ((g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi)) / g.gpipi);
          aux.x:=0.0;
          elseif aux.region == 2 then
            aux.T:=BaseIF97.Basic.tps2(p, s);
            g:=BaseIF97.Basic.g2(p, aux.T);
            aux.h:=aux.R * aux.T * g.tau * g.gtau;
            aux.rho:=p / (aux.R * aux.T * g.pi * g.gpi);
            aux.vt:=aux.R / p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
            aux.vp:=(aux.R * aux.T) / (p * p) * g.pi * g.pi * g.gpipi;
            aux.cp:= -aux.R * g.tau * g.tau * g.gtautau;
            aux.cv:=aux.R * ( -g.tau * g.tau * g.gtautau + ((g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi)) / g.gpipi);
            aux.x:=1.0;

          elseif aux.region == 3 then
            (aux.rho,aux.T,error):=BaseIF97.Inverses.dtofps3(p = p, s = s, delp = 1e-07, dels = 1e-06);
            f:=BaseIF97.Basic.f3(aux.rho, aux.T);
            aux.h:=aux.R * aux.T * (f.tau * f.ftau + f.delta * f.fdelta);
            aux.s:=aux.R * (f.tau * f.ftau - f.f);
            aux.pd:=aux.R * aux.T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
            aux.pt:=aux.R * aux.rho * f.delta * (f.fdelta - f.tau * f.fdeltatau);
            aux.cv:=aux.R * ( -f.tau * f.tau * f.ftautau);
            aux.cp:=(aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt) / (aux.rho * aux.rho * aux.pd);
            aux.x:=0.0;

          elseif aux.region == 4 then
            s_liq:=BaseIF97.Regions.sl_p(p);
            s_vap:=BaseIF97.Regions.sv_p(p);
            aux.x:=if s_vap <> s_liq then (s - s_liq) / (s_vap - s_liq) else 1.0;
            if p < BaseIF97.data.PLIMIT4A then 
                        t1:=BaseIF97.Basic.tps1(p, s_liq);
            t2:=BaseIF97.Basic.tps2(p, s_vap);
            gl:=BaseIF97.Basic.g1(p, t1);
            gv:=BaseIF97.Basic.g2(p, t2);
            liq:=Common.gibbsToBoundaryProps(gl);
            vap:=Common.gibbsToBoundaryProps(gv);
            aux.T:=t1 + aux.x * (t2 - t1);

            else             aux.T:=BaseIF97.Basic.tsat(p);
            d_liq:=rhol_T(aux.T);
            d_vap:=rhov_T(aux.T);
            fl:=BaseIF97.Basic.f3(d_liq, aux.T);
            fv:=BaseIF97.Basic.f3(d_vap, aux.T);
            liq:=Common.helmholtzToBoundaryProps(fl);
            vap:=Common.helmholtzToBoundaryProps(fv);

            end if;
            aux.dpT:=if liq.d <> vap.d then ((vap.s - liq.s) * liq.d * vap.d) / (liq.d - vap.d) else BaseIF97.Basic.dptofT(aux.T);
            aux.h:=liq.h + aux.x * (vap.h - liq.h);
            aux.rho:=(liq.d * vap.d) / (vap.d + aux.x * (liq.d - vap.d));
            aux.cv:=Common.cv2Phase(liq, vap, aux.x, aux.T, p);
            aux.cp:=liq.cp + aux.x * (vap.cp - liq.cp);
            aux.pt:=liq.pt + aux.x * (vap.pt - liq.pt);
            aux.pd:=liq.pd + aux.x * (vap.pd - liq.pd);

          elseif aux.region == 5 then
            (aux.T,error):=BaseIF97.Inverses.tofps5(p = p, s = s, relds = 1e-07);
            assert(error == 0, "error in inverse iteration of steam tables");
            g:=BaseIF97.Basic.g5(p, aux.T);
            aux.h:=aux.R * aux.T * g.tau * g.gtau;
            aux.rho:=p / (aux.R * aux.T * g.pi * g.gpi);
            aux.vt:=aux.R / p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
            aux.vp:=(aux.R * aux.T) / (p * p) * g.pi * g.pi * g.gpipi;
            aux.cp:= -aux.R * g.tau * g.tau * g.gtautau;
            aux.cv:=aux.R * ( -g.tau * g.tau * g.gtautau + ((g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi)) / g.gpipi);


          else           assert(false, "error in region computation of IF97 steam tables" + "(p = " + String(p) + ", s = " + String(s) + ")");

          end if;
        end waterBaseProp_ps;
        function rho_props_ps "density as function of pressure and specific entropy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEntropy s "specific entropy";
          input Common.IF97BaseTwoPhase properties "auxiliary record";
          output SI.Density rho "density";
          annotation(Inline = false, LateInline = true);
        algorithm 
          rho:=properties.rho;
        end rho_props_ps;
        function rho_ps "density as function of pressure and specific entropy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEntropy s "specific entropy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.Density rho "density";
        algorithm 
          rho:=rho_props_ps(p, s, waterBaseProp_ps(p, s, phase, region));
        end rho_ps;
        function T_props_ps "temperature as function of pressure and specific entropy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEntropy s "specific entropy";
          input Common.IF97BaseTwoPhase properties "auxiliary record";
          output SI.Temperature T "temperature";
          annotation(Inline = false, LateInline = true);
        algorithm 
          T:=properties.T;
        end T_props_ps;
        function T_ps "temperature as function of pressure and specific entropy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEntropy s "specific entropy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.Temperature T "Temperature";
        algorithm 
          T:=T_props_ps(p, s, waterBaseProp_ps(p, s, phase, region));
        end T_ps;
        function h_props_ps "specific enthalpy as function or pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEntropy s "specific entropy";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.SpecificEnthalpy h "specific enthalpy";
          annotation(Inline = false, LateInline = true);
        algorithm 
          h:=aux.h;
        end h_props_ps;
        function h_ps "specific enthalpy as function or pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEntropy s "specific entropy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.SpecificEnthalpy h "specific enthalpy";
        algorithm 
          h:=h_props_ps(p, s, waterBaseProp_ps(p, s, phase, region));
        end h_ps;
        function phase_ps "phase as a function of  pressure and specific entropy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEntropy s "specific entropy";
          output Integer phase "true if in liquid or gas or supercritical region";
          annotation(InlineNoEvent = false);
        algorithm 
          phase:=if s < sl_p(p) or s > sv_p(p) or p > BaseIF97.data.PCRIT then 1 else 2;
        end phase_ps;
        function phase_ph "phase as a function of  pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          output Integer phase "true if in liquid or gas or supercritical region";
          annotation(InlineNoEvent = false);
        algorithm 
          phase:=if h < hl_p(p) or h > hv_p(p) or p > BaseIF97.data.PCRIT then 1 else 2;
        end phase_ph;
        function phase_dT "phase as a function of  pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Density rho "density";
          input SI.Temperature T "temperature";
          output Integer phase "true if in liquid or gas or supercritical region";
          annotation(InlineNoEvent = false);
        algorithm 
          phase:=if not (rho < rhol_T(T) and rho > rhov_T(T) and T < BaseIF97.data.TCRIT) then 1 else 2;
        end phase_dT;
        function rho_props_ph "density as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Common.IF97BaseTwoPhase properties "auxiliary record";
          output SI.Density rho "density";
          annotation(derivative(noDerivative = properties) = rho_ph_der, Inline = false, LateInline = true);
        algorithm 
          rho:=properties.rho;
        end rho_props_ph;
        function rho_ph "density as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.Density rho "density";
        algorithm 
          rho:=rho_props_ph(p, h, waterBaseProp_ph(p, h, phase, region));
        end rho_ph;
        function rho_ph_der "derivative function of rho_ph"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          input Real p_der "derivative of pressure";
          input Real h_der "derivative of specific enthalpy";
          output Real rho_der "derivative of density";
        algorithm 
          if aux.region == 4 then 
                    rho_der:=(aux.rho * ((aux.rho * aux.cv) / aux.dpT + 1.0)) / (aux.dpT * aux.T) * p_der + ( -(aux.rho * aux.rho) / (aux.dpT * aux.T)) * h_der;
          elseif aux.region == 3 then
            rho_der:=(aux.rho * (aux.cv * aux.rho + aux.pt)) / (aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt) * p_der + ( -(aux.rho * aux.rho * aux.pt) / (aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt)) * h_der;


          else           rho_der:=( -(aux.rho * aux.rho * (aux.vp * aux.cp - aux.vt / aux.rho + aux.T * aux.vt * aux.vt)) / aux.cp) * p_der + ( -(aux.rho * aux.rho * aux.vt) / aux.cp) * h_der;

          end if;
        end rho_ph_der;
        function T_props_ph "temperature as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Common.IF97BaseTwoPhase properties "auxiliary record";
          output SI.Temperature T "temperature";
          annotation(derivative(noDerivative = properties) = T_ph_der, Inline = false, LateInline = true);
        algorithm 
          T:=properties.T;
        end T_props_ph;
        function T_ph "temperature as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.Temperature T "Temperature";
        algorithm 
          T:=T_props_ph(p, h, waterBaseProp_ph(p, h, phase, region));
        end T_ph;
        function T_ph_der "derivative function of T_ph"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          input Real p_der "derivative of pressure";
          input Real h_der "derivative of specific enthalpy";
          output Real T_der "derivative of temperature";
        algorithm 
          if aux.region == 4 then 
                    T_der:=1 / aux.dpT * p_der;
          elseif aux.region == 3 then
            T_der:=( -aux.rho * aux.pd + aux.T * aux.pt) / (aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt) * p_der + (aux.rho * aux.rho * aux.pd) / (aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt) * h_der;


          else           T_der:=( -1 / aux.rho + aux.T * aux.vt) / aux.cp * p_der + 1 / aux.cp * h_der;

          end if;
        end T_ph_der;
        function s_props_ph "specific entropy as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Common.IF97BaseTwoPhase properties "auxiliary record";
          output SI.SpecificEntropy s "specific entropy";
          annotation(derivative(noDerivative = properties) = s_ph_der, Inline = false, LateInline = true);
        algorithm 
          s:=properties.s;
        end s_props_ph;
        function s_ph "specific entropy as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.SpecificEntropy s "specific entropy";
        algorithm 
          s:=s_props_ph(p, h, waterBaseProp_ph(p, h, phase, region));
        end s_ph;
        function s_ph_der "specific entropy as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          input Real p_der "derivative of pressure";
          input Real h_der "derivative of specific enthalpy";
          output Real s_der "derivative of entropy";
        algorithm 
          s_der:= -1 / (aux.rho * aux.T) * p_der + 1 / aux.T * h_der;
        end s_ph_der;
        function cv_props_ph "specific heat capacity at constant volume as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.SpecificHeatCapacity cv "specific heat capacity";
          annotation(Inline = false, LateInline = true);
        algorithm 
          cv:=aux.cv;
        end cv_props_ph;
        function cv_ph "specific heat capacity at constant volume as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.SpecificHeatCapacity cv "specific heat capacity";
        algorithm 
          cv:=cv_props_ph(p, h, waterBaseProp_ph(p, h, phase, region));
        end cv_ph;
        function regionAssertReal "assert function for inlining"
          extends Modelica.Icons.Function;
          input Boolean check "condition to check";
          output Real dummy "dummy output";
        algorithm 
          assert(check, "this function can not be called with two-phase inputs!");
        end regionAssertReal;
        function cp_props_ph "specific heat capacity at constant pressure as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.SpecificHeatCapacity cp "specific heat capacity";
          annotation(Inline = false, LateInline = true);
        algorithm 
          cp:=aux.cp;
        end cp_props_ph;
        function cp_ph "specific heat capacity at constant pressure as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.SpecificHeatCapacity cp "specific heat capacity";
        algorithm 
          cp:=cp_props_ph(p, h, waterBaseProp_ph(p, h, phase, region));
        end cp_ph;
        function beta_props_ph "isobaric expansion coefficient as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.RelativePressureCoefficient beta "isobaric expansion coefficient";
          annotation(Inline = false, LateInline = true);
        algorithm 
          beta:=if aux.region == 3 or aux.region == 4 then aux.pt / (aux.rho * aux.pd) else aux.vt * aux.rho;
        end beta_props_ph;
        function beta_ph "isobaric expansion coefficient as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.RelativePressureCoefficient beta "isobaric expansion coefficient";
        algorithm 
          beta:=beta_props_ph(p, h, waterBaseProp_ph(p, h, phase, region));
        end beta_ph;
        function kappa_props_ph "isothermal compressibility factor as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.IsothermalCompressibility kappa "isothermal compressibility factor";
          annotation(Inline = false, LateInline = true);
        algorithm 
          kappa:=if aux.region == 3 or aux.region == 4 then 1 / (aux.rho * aux.pd) else  -aux.vp * aux.rho;
        end kappa_props_ph;
        function kappa_ph "isothermal compressibility factor as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.IsothermalCompressibility kappa "isothermal compressibility factor";
        algorithm 
          kappa:=kappa_props_ph(p, h, waterBaseProp_ph(p, h, phase, region));
        end kappa_ph;
        function velocityOfSound_props_ph "speed of sound as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.Velocity v_sound "speed of sound";
          annotation(Inline = false, LateInline = true);
        algorithm 
          v_sound:=if aux.region == 3 then sqrt(max(0, (aux.pd * aux.rho * aux.rho * aux.cv + aux.pt * aux.pt * aux.T) / (aux.rho * aux.rho * aux.cv))) else if aux.region == 4 then sqrt(max(0, 1 / ((aux.rho * ((aux.rho * aux.cv) / aux.dpT + 1.0)) / (aux.dpT * aux.T) - (1 / aux.rho * aux.rho * aux.rho) / (aux.dpT * aux.T)))) else sqrt(max(0,  -aux.cp / (aux.rho * aux.rho * (aux.vp * aux.cp + aux.vt * aux.vt * aux.T))));
        end velocityOfSound_props_ph;
        function velocityOfSound_ph
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.Velocity v_sound "speed of sound";
        algorithm 
          v_sound:=velocityOfSound_props_ph(p, h, waterBaseProp_ph(p, h, phase, region));
        end velocityOfSound_ph;
        function isentropicExponent_props_ph "isentropic exponent as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output Real gamma "isentropic exponent";
          annotation(Inline = false, LateInline = true);
        algorithm 
          gamma:=if aux.region == 3 then 1 / (aux.rho * p) * (aux.pd * aux.cv * aux.rho * aux.rho + aux.pt * aux.pt * aux.T) / aux.cv else if aux.region == 4 then (1 / (aux.rho * p) * aux.dpT * aux.dpT * aux.T) / aux.cv else  -(1 / (aux.rho * aux.p) * aux.cp) / (aux.vp * aux.cp + aux.vt * aux.vt * aux.T);
        end isentropicExponent_props_ph;
        function isentropicExponent_ph "isentropic exponent as function of pressure and specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output Real gamma "isentropic exponent";
          annotation(Inline = false, LateInline = true);
        algorithm 
          gamma:=isentropicExponent_props_ph(p, h, waterBaseProp_ph(p, h, phase, region));
        end isentropicExponent_ph;
        function ddph_props "density derivative by pressure"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.DerDensityByPressure ddph "density derivative by pressure";
          annotation(Inline = false, LateInline = true);
        algorithm 
          ddph:=if aux.region == 3 then (aux.rho * (aux.cv * aux.rho + aux.pt)) / (aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt) else if aux.region == 4 then (aux.rho * ((aux.rho * aux.cv) / aux.dpT + 1.0)) / (aux.dpT * aux.T) else  -(aux.rho * aux.rho * (aux.vp * aux.cp - aux.vt / aux.rho + aux.T * aux.vt * aux.vt)) / aux.cp;
        end ddph_props;
        function ddph "density derivative by pressure"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.DerDensityByPressure ddph "density derivative by pressure";
        algorithm 
          ddph:=ddph_props(p, h, waterBaseProp_ph(p, h, phase, region));
        end ddph;
        function ddhp_props "density derivative by specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.DerDensityByEnthalpy ddhp "density derivative by specific enthalpy";
          annotation(Inline = false, LateInline = true);
        algorithm 
          ddhp:=if aux.region == 3 then  -(aux.rho * aux.rho * aux.pt) / (aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt) else if aux.region == 4 then  -(aux.rho * aux.rho) / (aux.dpT * aux.T) else  -(aux.rho * aux.rho * aux.vt) / aux.cp;
        end ddhp_props;
        function ddhp "density derivative by specific enthalpy"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEnthalpy h "specific enthalpy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.DerDensityByEnthalpy ddhp "density derivative by specific enthalpy";
        algorithm 
          ddhp:=ddhp_props(p, h, waterBaseProp_ph(p, h, phase, region));
        end ddhp;
        function waterBaseProp_pT "intermediate property record for water (p and T prefered states)"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.Temperature T "temperature";
          input Integer region = 0 "if 0, do region computation, otherwise assume the region is this input";
          output Common.IF97BaseTwoPhase aux "auxiliary record";
        protected 
          Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives wrt pi and tau";
          Common.HelmholtzDerivs f "dimensionless Helmholtz funcion and dervatives wrt delta and tau";
          Integer error "error flag for inverse iterations";
        algorithm 
          aux.phase:=1;
          aux.region:=if region == 0 then BaseIF97.Regions.region_pT(p = p, T = T) else region;
          aux.R:=BaseIF97.data.RH2O;
          aux.p:=p;
          aux.T:=T;
          if aux.region == 1 then 
                    g:=BaseIF97.Basic.g1(p, T);
          aux.h:=aux.R * aux.T * g.tau * g.gtau;
          aux.s:=aux.R * (g.tau * g.gtau - g.g);
          aux.rho:=p / (aux.R * T * g.pi * g.gpi);
          aux.vt:=aux.R / p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
          aux.vp:=(aux.R * T) / (p * p) * g.pi * g.pi * g.gpipi;
          aux.cp:= -aux.R * g.tau * g.tau * g.gtautau;
          aux.cv:=aux.R * ( -g.tau * g.tau * g.gtautau + ((g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi)) / g.gpipi);
          aux.x:=0.0;
          elseif aux.region == 2 then
            g:=BaseIF97.Basic.g2(p, T);
            aux.h:=aux.R * aux.T * g.tau * g.gtau;
            aux.s:=aux.R * (g.tau * g.gtau - g.g);
            aux.rho:=p / (aux.R * T * g.pi * g.gpi);
            aux.vt:=aux.R / p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
            aux.vp:=(aux.R * T) / (p * p) * g.pi * g.pi * g.gpipi;
            aux.cp:= -aux.R * g.tau * g.tau * g.gtautau;
            aux.cv:=aux.R * ( -g.tau * g.tau * g.gtautau + ((g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi)) / g.gpipi);
            aux.x:=1.0;

          elseif aux.region == 3 then
            (aux.rho,error):=BaseIF97.Inverses.dofpt3(p = p, T = T, delp = 1e-07);
            f:=BaseIF97.Basic.f3(aux.rho, T);
            aux.h:=aux.R * T * (f.tau * f.ftau + f.delta * f.fdelta);
            aux.s:=aux.R * (f.tau * f.ftau - f.f);
            aux.pd:=aux.R * T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
            aux.pt:=aux.R * aux.rho * f.delta * (f.fdelta - f.tau * f.fdeltatau);
            aux.cv:=aux.R * ( -f.tau * f.tau * f.ftautau);
            aux.x:=0.0;

          elseif aux.region == 5 then
            g:=BaseIF97.Basic.g5(p, T);
            aux.h:=aux.R * aux.T * g.tau * g.gtau;
            aux.s:=aux.R * (g.tau * g.gtau - g.g);
            aux.rho:=p / (aux.R * T * g.pi * g.gpi);
            aux.vt:=aux.R / p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
            aux.vp:=(aux.R * T) / (p * p) * g.pi * g.pi * g.gpipi;
            aux.cp:= -aux.R * g.tau * g.tau * g.gtautau;
            aux.cv:=aux.R * ( -g.tau * g.tau * g.gtautau + ((g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi)) / g.gpipi);


          else           assert(false, "error in region computation of IF97 steam tables" + "(p = " + String(p) + ", T = " + String(T) + ")");

          end if;
        end waterBaseProp_pT;
        function rho_props_pT "density as function or pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.Temperature T "temperature";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.Density rho "density";
          annotation(derivative(noDerivative = aux) = rho_pT_der, Inline = false, LateInline = true);
        algorithm 
          rho:=aux.rho;
        end rho_props_pT;
        function rho_pT "density as function or pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.Temperature T "temperature";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.Density rho "density";
        algorithm 
          rho:=rho_props_pT(p, T, waterBaseProp_pT(p, T, region));
        end rho_pT;
        function h_props_pT "specific enthalpy as function or pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.Temperature T "temperature";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.SpecificEnthalpy h "specific enthalpy";
          annotation(derivative(noDerivative = aux) = h_pT_der, Inline = false, LateInline = true);
        algorithm 
          h:=aux.h;
        end h_props_pT;
        function h_pT "specific enthalpy as function or pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.Temperature T "Temperature";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.SpecificEnthalpy h "specific enthalpy";
        algorithm 
          h:=h_props_pT(p, T, waterBaseProp_pT(p, T, region));
        end h_pT;
        function h_pT_der "derivative function of h_pT"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.Temperature T "temperature";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          input Real p_der "derivative of pressure";
          input Real T_der "derivative of temperature";
          output Real h_der "derivative of specific enthalpy";
        algorithm 
          if aux.region == 3 then 
                    h_der:=( -aux.rho * aux.pd + T * aux.pt) / (aux.rho * aux.rho * aux.pd) * p_der + (aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt) / (aux.rho * aux.rho * aux.pd) * T_der;

          else           h_der:=(1 / aux.rho - aux.T * aux.vt) * p_der + aux.cp * T_der;

          end if;
        end h_pT_der;
        function rho_pT_der "derivative function of rho_pT"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.Temperature T "temperature";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          input Real p_der "derivative of pressure";
          input Real T_der "derivative of temperature";
          output Real rho_der "derivative of density";
        algorithm 
          if aux.region == 3 then 
                    rho_der:=1 / aux.pd * p_der - aux.pt / aux.pd * T_der;

          else           rho_der:=( -aux.rho * aux.rho * aux.vp) * p_der + ( -aux.rho * aux.rho * aux.vt) * T_der;

          end if;
        end rho_pT_der;
        function s_props_pT "specific entropy as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.Temperature T "temperature";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.SpecificEntropy s "specific entropy";
          annotation(Inline = false, LateInline = true);
        algorithm 
          s:=aux.s;
        end s_props_pT;
        function s_pT "temperature as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.Temperature T "temperature";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.SpecificEntropy s "specific entropy";
          annotation(InlineNoEvent = false);
        algorithm 
          s:=s_props_pT(p, T, waterBaseProp_pT(p, T, region));
        end s_pT;
        function cv_props_pT "specific heat capacity at constant volume as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.Temperature T "temperature";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.SpecificHeatCapacity cv "specific heat capacity";
          annotation(Inline = false, LateInline = true);
        algorithm 
          cv:=aux.cv;
        end cv_props_pT;
        function cv_pT "specific heat capacity at constant volume as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.Temperature T "temperature";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.SpecificHeatCapacity cv "specific heat capacity";
          annotation(InlineNoEvent = false);
        algorithm 
          cv:=cv_props_pT(p, T, waterBaseProp_pT(p, T, region));
        end cv_pT;
        function cp_props_pT "specific heat capacity at constant pressure as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.Temperature T "temperature";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.SpecificHeatCapacity cp "specific heat capacity";
          annotation(Inline = false, LateInline = true);
        algorithm 
          cp:=if aux.region == 3 then (aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt) / (aux.rho * aux.rho * aux.pd) else aux.cp;
        end cp_props_pT;
        function cp_pT "specific heat capacity at constant pressure as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.Temperature T "temperature";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.SpecificHeatCapacity cp "specific heat capacity";
          annotation(InlineNoEvent = false);
        algorithm 
          cp:=cp_props_pT(p, T, waterBaseProp_pT(p, T, region));
        end cp_pT;
        function beta_props_pT "isobaric expansion coefficient as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.Temperature T "temperature";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.RelativePressureCoefficient beta "isobaric expansion coefficient";
          annotation(Inline = false, LateInline = true);
        algorithm 
          beta:=if aux.region == 3 then aux.pt / (aux.rho * aux.pd) else aux.vt * aux.rho;
        end beta_props_pT;
        function beta_pT "isobaric expansion coefficient as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.Temperature T "temperature";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.RelativePressureCoefficient beta "isobaric expansion coefficient";
          annotation(InlineNoEvent = false);
        algorithm 
          beta:=beta_props_pT(p, T, waterBaseProp_pT(p, T, region));
        end beta_pT;
        function kappa_props_pT "isothermal compressibility factor as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.Temperature T "temperature";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.IsothermalCompressibility kappa "isothermal compressibility factor";
          annotation(Inline = false, LateInline = true);
        algorithm 
          kappa:=if aux.region == 3 then 1 / (aux.rho * aux.pd) else  -aux.vp * aux.rho;
        end kappa_props_pT;
        function kappa_pT "isothermal compressibility factor as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.Temperature T "temperature";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.IsothermalCompressibility kappa "isothermal compressibility factor";
          annotation(InlineNoEvent = false);
        algorithm 
          kappa:=kappa_props_pT(p, T, waterBaseProp_pT(p, T, region));
        end kappa_pT;
        function velocityOfSound_props_pT "speed of sound as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.Temperature T "temperature";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.Velocity v_sound "speed of sound";
          annotation(Inline = false, LateInline = true);
        algorithm 
          v_sound:=if aux.region == 3 then sqrt(max(0, (aux.pd * aux.rho * aux.rho * aux.cv + aux.pt * aux.pt * aux.T) / (aux.rho * aux.rho * aux.cv))) else sqrt(max(0,  -aux.cp / (aux.rho * aux.rho * (aux.vp * aux.cp + aux.vt * aux.vt * aux.T))));
        end velocityOfSound_props_pT;
        function velocityOfSound_pT "speed of sound as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.Temperature T "temperature";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.Velocity v_sound "speed of sound";
        algorithm 
          v_sound:=velocityOfSound_props_pT(p, T, waterBaseProp_pT(p, T, region));
        end velocityOfSound_pT;
        function isentropicExponent_props_pT "isentropic exponent as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.Temperature T "temperature";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output Real gamma "isentropic exponent";
          annotation(Inline = false, LateInline = true);
        algorithm 
          gamma:=if aux.region == 3 then 1 / (aux.rho * p) * (aux.pd * aux.cv * aux.rho * aux.rho + aux.pt * aux.pt * aux.T) / aux.cv else  -(1 / (aux.rho * aux.p) * aux.cp) / (aux.vp * aux.cp + aux.vt * aux.vt * aux.T);
        end isentropicExponent_props_pT;
        function isentropicExponent_pT "isentropic exponent as function of pressure and temperature"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.Temperature T "temperature";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output Real gamma "isentropic exponent";
          annotation(Inline = false, LateInline = true);
        algorithm 
          gamma:=isentropicExponent_props_pT(p, T, waterBaseProp_pT(p, T, region));
        end isentropicExponent_pT;
        function waterBaseProp_dT "intermediate property record for water (d and T prefered states)"
          extends Modelica.Icons.Function;
          input SI.Density rho "density";
          input SI.Temperature T "temperature";
          input Integer phase = 0 "phase: 2 for two-phase, 1 for one phase, 0 if unknown";
          input Integer region = 0 "if 0, do region computation, otherwise assume the region is this input";
          output Common.IF97BaseTwoPhase aux "auxiliary record";
        protected 
          SI.SpecificEnthalpy h_liq "liquid specific enthalpy";
          SI.Density d_liq "liquid density";
          SI.SpecificEnthalpy h_vap "vapour specific enthalpy";
          SI.Density d_vap "vapour density";
          Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives wrt pi and tau";
          Common.HelmholtzDerivs f "dimensionless Helmholtz funcion and dervatives wrt delta and tau";
          Modelica.Media.Common.PhaseBoundaryProperties liq "phase boundary property record";
          Modelica.Media.Common.PhaseBoundaryProperties vap "phase boundary property record";
          Modelica.Media.Common.GibbsDerivs gl "dimensionless Gibbs funcion and dervatives wrt pi and tau";
          Modelica.Media.Common.GibbsDerivs gv "dimensionless Gibbs funcion and dervatives wrt pi and tau";
          Modelica.Media.Common.HelmholtzDerivs fl "dimensionless Helmholtz function and dervatives wrt delta and tau";
          Modelica.Media.Common.HelmholtzDerivs fv "dimensionless Helmholtz function and dervatives wrt delta and tau";
          Integer error "error flag for inverse iterations";
        algorithm 
          aux.region:=if region == 0 then if phase == 2 then 4 else BaseIF97.Regions.region_dT(d = rho, T = T, phase = phase) else region;
          aux.phase:=if aux.region == 4 then 2 else 1;
          aux.R:=BaseIF97.data.RH2O;
          aux.rho:=rho;
          aux.T:=T;
          if aux.region == 1 then 
                    (aux.p,error):=BaseIF97.Inverses.pofdt125(d = rho, T = T, reldd = 1e-08, region = 1);
          g:=BaseIF97.Basic.g1(aux.p, T);
          aux.h:=aux.R * aux.T * g.tau * g.gtau;
          aux.s:=aux.R * (g.tau * g.gtau - g.g);
          aux.rho:=aux.p / (aux.R * T * g.pi * g.gpi);
          aux.vt:=aux.R / aux.p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
          aux.vp:=(aux.R * T) / (aux.p * aux.p) * g.pi * g.pi * g.gpipi;
          aux.cp:= -aux.R * g.tau * g.tau * g.gtautau;
          aux.cv:=aux.R * ( -g.tau * g.tau * g.gtautau + ((g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi)) / g.gpipi);
          aux.x:=0.0;
          elseif aux.region == 2 then
            (aux.p,error):=BaseIF97.Inverses.pofdt125(d = rho, T = T, reldd = 1e-08, region = 2);
            g:=BaseIF97.Basic.g2(aux.p, T);
            aux.h:=aux.R * aux.T * g.tau * g.gtau;
            aux.s:=aux.R * (g.tau * g.gtau - g.g);
            aux.rho:=aux.p / (aux.R * T * g.pi * g.gpi);
            aux.vt:=aux.R / aux.p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
            aux.vp:=(aux.R * T) / (aux.p * aux.p) * g.pi * g.pi * g.gpipi;
            aux.cp:= -aux.R * g.tau * g.tau * g.gtautau;
            aux.cv:=aux.R * ( -g.tau * g.tau * g.gtautau + ((g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi)) / g.gpipi);
            aux.x:=1.0;

          elseif aux.region == 3 then
            f:=BaseIF97.Basic.f3(rho, T);
            aux.p:=aux.R * rho * T * f.delta * f.fdelta;
            aux.h:=aux.R * T * (f.tau * f.ftau + f.delta * f.fdelta);
            aux.s:=aux.R * (f.tau * f.ftau - f.f);
            aux.pd:=aux.R * T * f.delta * (2.0 * f.fdelta + f.delta * f.fdeltadelta);
            aux.pt:=aux.R * rho * f.delta * (f.fdelta - f.tau * f.fdeltatau);
            aux.cp:=(aux.rho * aux.rho * aux.pd * aux.cv + aux.T * aux.pt * aux.pt) / (aux.rho * aux.rho * aux.pd);
            aux.cv:=aux.R * ( -f.tau * f.tau * f.ftautau);
            aux.x:=0.0;

          elseif aux.region == 4 then
            aux.p:=BaseIF97.Basic.psat(T);
            d_liq:=rhol_T(T);
            d_vap:=rhov_T(T);
            h_liq:=hl_p(aux.p);
            h_vap:=hv_p(aux.p);
            aux.x:=if d_vap <> d_liq then (1 / rho - 1 / d_liq) / (1 / d_vap - 1 / d_liq) else 1.0;
            aux.h:=h_liq + aux.x * (h_vap - h_liq);
            if T < BaseIF97.data.TLIMIT1 then 
                        gl:=BaseIF97.Basic.g1(aux.p, T);
            gv:=BaseIF97.Basic.g2(aux.p, T);
            liq:=Common.gibbsToBoundaryProps(gl);
            vap:=Common.gibbsToBoundaryProps(gv);

            else             fl:=BaseIF97.Basic.f3(d_liq, T);
            fv:=BaseIF97.Basic.f3(d_vap, T);
            liq:=Common.helmholtzToBoundaryProps(fl);
            vap:=Common.helmholtzToBoundaryProps(fv);

            end if;
            aux.dpT:=if liq.d <> vap.d then ((vap.s - liq.s) * liq.d * vap.d) / (liq.d - vap.d) else BaseIF97.Basic.dptofT(aux.T);
            aux.s:=liq.s + aux.x * (vap.s - liq.s);
            aux.cv:=Common.cv2Phase(liq, vap, aux.x, aux.T, aux.p);
            aux.cp:=liq.cp + aux.x * (vap.cp - liq.cp);
            aux.pt:=liq.pt + aux.x * (vap.pt - liq.pt);
            aux.pd:=liq.pd + aux.x * (vap.pd - liq.pd);

          elseif aux.region == 5 then
            (aux.p,error):=BaseIF97.Inverses.pofdt125(d = rho, T = T, reldd = 1e-08, region = 5);
            g:=BaseIF97.Basic.g2(aux.p, T);
            aux.h:=aux.R * aux.T * g.tau * g.gtau;
            aux.s:=aux.R * (g.tau * g.gtau - g.g);
            aux.rho:=aux.p / (aux.R * T * g.pi * g.gpi);
            aux.vt:=aux.R / aux.p * (g.pi * g.gpi - g.tau * g.pi * g.gtaupi);
            aux.vp:=(aux.R * T) / (aux.p * aux.p) * g.pi * g.pi * g.gpipi;
            aux.cp:= -aux.R * g.tau * g.tau * g.gtautau;
            aux.cv:=aux.R * ( -g.tau * g.tau * g.gtautau + ((g.gpi - g.tau * g.gtaupi) * (g.gpi - g.tau * g.gtaupi)) / g.gpipi);


          else           assert(false, "error in region computation of IF97 steam tables" + "(rho = " + String(rho) + ", T = " + String(T) + ")");

          end if;
        end waterBaseProp_dT;
        function h_props_dT "specific enthalpy as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "density";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.SpecificEnthalpy h "specific enthalpy";
          annotation(derivative(noDerivative = aux) = h_dT_der, Inline = false, LateInline = true);
        algorithm 
          h:=aux.h;
        end h_props_dT;
        function h_dT "specific enthalpy as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "density";
          input SI.Temperature T "Temperature";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.SpecificEnthalpy h "specific enthalpy";
        algorithm 
          h:=h_props_dT(d, T, waterBaseProp_dT(d, T, phase, region));
        end h_dT;
        function h_dT_der "derivative function of h_dT"
          extends Modelica.Icons.Function;
          input SI.Density d "density";
          input SI.Temperature T "temperature";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          input Real d_der "derivative of density";
          input Real T_der "derivative of temperature";
          output Real h_der "derivative of specific enthalpy";
        algorithm 
          if aux.region == 3 then 
                    h_der:=( -d * aux.pd + T * aux.pt) / (d * d) * d_der + (aux.cv * d + aux.pt) / d * T_der;
          elseif aux.region == 4 then
            h_der:=(T * aux.dpT) / (d * d) * d_der + (aux.cv * d + aux.dpT) / d * T_der;


          else           h_der:=( -( -1 / d + T * aux.vt) / (d * d * aux.vp)) * d_der + (aux.vp * aux.cp - aux.vt / d + T * aux.vt * aux.vt) / aux.vp * T_der;

          end if;
        end h_dT_der;
        function p_props_dT "pressure as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "density";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.Pressure p "pressure";
          annotation(derivative(noDerivative = aux) = p_dT_der, Inline = false, LateInline = true);
        algorithm 
          p:=aux.p;
        end p_props_dT;
        function p_dT "pressure as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "density";
          input SI.Temperature T "Temperature";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.Pressure p "pressure";
        algorithm 
          p:=p_props_dT(d, T, waterBaseProp_dT(d, T, phase, region));
        end p_dT;
        function p_dT_der "derivative function of p_dT"
          extends Modelica.Icons.Function;
          input SI.Density d "density";
          input SI.Temperature T "temperature";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          input Real d_der "derivative of density";
          input Real T_der "derivative of temperature";
          output Real p_der "derivative of pressure";
        algorithm 
          if aux.region == 3 then 
                    p_der:=aux.pd * d_der + aux.pt * T_der;
          elseif aux.region == 4 then
            p_der:=aux.dpT * T_der;


          else           p_der:=( -1 / (d * d * aux.vp)) * d_der + ( -aux.vt / aux.vp) * T_der;

          end if;
        end p_dT_der;
        function s_props_dT "specific entropy as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "density";
          input SI.Temperature T "Temperature";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.SpecificEntropy s "specific entropy";
          annotation(Inline = false, LateInline = true);
        algorithm 
          s:=aux.s;
        end s_props_dT;
        function s_dT "temperature as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "density";
          input SI.Temperature T "Temperature";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.SpecificEntropy s "specific entropy";
        algorithm 
          s:=s_props_dT(d, T, waterBaseProp_dT(d, T, phase, region));
        end s_dT;
        function cv_props_dT "specific heat capacity at constant volume as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "density";
          input SI.Temperature T "temperature";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.SpecificHeatCapacity cv "specific heat capacity";
          annotation(Inline = false, LateInline = true);
        algorithm 
          cv:=aux.cv;
        end cv_props_dT;
        function cv_dT "specific heat capacity at constant volume as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "density";
          input SI.Temperature T "temperature";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.SpecificHeatCapacity cv "specific heat capacity";
        algorithm 
          cv:=cv_props_dT(d, T, waterBaseProp_dT(d, T, phase, region));
        end cv_dT;
        function cp_props_dT "specific heat capacity at constant pressure as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "density";
          input SI.Temperature T "temperature";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.SpecificHeatCapacity cp "specific heat capacity";
          annotation(Inline = false, LateInline = true);
        algorithm 
          cp:=aux.cp;
        end cp_props_dT;
        function cp_dT "specific heat capacity at constant pressure as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "density";
          input SI.Temperature T "temperature";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.SpecificHeatCapacity cp "specific heat capacity";
        algorithm 
          cp:=cp_props_dT(d, T, waterBaseProp_dT(d, T, phase, region));
        end cp_dT;
        function beta_props_dT "isobaric expansion coefficient as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "density";
          input SI.Temperature T "temperature";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.RelativePressureCoefficient beta "isobaric expansion coefficient";
          annotation(Inline = false, LateInline = true);
        algorithm 
          beta:=if aux.region == 3 or aux.region == 4 then aux.pt / (aux.rho * aux.pd) else aux.vt * aux.rho;
        end beta_props_dT;
        function beta_dT "isobaric expansion coefficient as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "density";
          input SI.Temperature T "temperature";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.RelativePressureCoefficient beta "isobaric expansion coefficient";
        algorithm 
          beta:=beta_props_dT(d, T, waterBaseProp_dT(d, T, phase, region));
        end beta_dT;
        function kappa_props_dT "isothermal compressibility factor as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "density";
          input SI.Temperature T "temperature";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.IsothermalCompressibility kappa "isothermal compressibility factor";
          annotation(Inline = false, LateInline = true);
        algorithm 
          kappa:=if aux.region == 3 or aux.region == 4 then 1 / (aux.rho * aux.pd) else  -aux.vp * aux.rho;
        end kappa_props_dT;
        function kappa_dT "isothermal compressibility factor as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "density";
          input SI.Temperature T "temperature";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.IsothermalCompressibility kappa "isothermal compressibility factor";
        algorithm 
          kappa:=kappa_props_dT(d, T, waterBaseProp_dT(d, T, phase, region));
        end kappa_dT;
        function velocityOfSound_props_dT "speed of sound as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "density";
          input SI.Temperature T "temperature";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.Velocity v_sound "speed of sound";
          annotation(Inline = false, LateInline = true);
        algorithm 
          v_sound:=if aux.region == 3 then sqrt(max(0, (aux.pd * aux.rho * aux.rho * aux.cv + aux.pt * aux.pt * aux.T) / (aux.rho * aux.rho * aux.cv))) else if aux.region == 4 then sqrt(max(0, 1 / ((aux.rho * ((aux.rho * aux.cv) / aux.dpT + 1.0)) / (aux.dpT * aux.T) - (1 / aux.rho * aux.rho * aux.rho) / (aux.dpT * aux.T)))) else sqrt(max(0,  -aux.cp / (aux.rho * aux.rho * (aux.vp * aux.cp + aux.vt * aux.vt * aux.T))));
        end velocityOfSound_props_dT;
        function velocityOfSound_dT "speed of sound as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "density";
          input SI.Temperature T "temperature";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.Velocity v_sound "speed of sound";
        algorithm 
          v_sound:=velocityOfSound_props_dT(d, T, waterBaseProp_dT(d, T, phase, region));
        end velocityOfSound_dT;
        function isentropicExponent_props_dT "isentropic exponent as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "density";
          input SI.Temperature T "temperature";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output Real gamma "isentropic exponent";
          annotation(Inline = false, LateInline = true);
        algorithm 
          gamma:=if aux.region == 3 then 1 / (aux.rho * aux.p) * (aux.pd * aux.cv * aux.rho * aux.rho + aux.pt * aux.pt * aux.T) / aux.cv else if aux.region == 4 then (1 / (aux.rho * aux.p) * aux.dpT * aux.dpT * aux.T) / aux.cv else  -(1 / (aux.rho * aux.p) * aux.cp) / (aux.vp * aux.cp + aux.vt * aux.vt * aux.T);
        end isentropicExponent_props_dT;
        function isentropicExponent_dT "isentropic exponent as function of density and temperature"
          extends Modelica.Icons.Function;
          input SI.Density d "density";
          input SI.Temperature T "temperature";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output Real gamma "isentropic exponent";
          annotation(Inline = false, LateInline = true);
        algorithm 
          gamma:=isentropicExponent_props_dT(d, T, waterBaseProp_dT(d, T, phase, region));
        end isentropicExponent_dT;
      protected 
        package ThermoFluidSpecial
          function water_ph "calculate the property record for dynamic simulation properties using p,h as states"
            extends Modelica.Icons.Function;
            input SI.Pressure p "pressure";
            input SI.SpecificEnthalpy h "specific enthalpy";
            input Integer phase = 0 "phase: 2 for two-phase, 1 for one phase, 0 if unknown";
            output Modelica.Media.Common.ThermoFluidSpecial.ThermoProperties_ph pro "property record for dynamic simulation";
          protected 
            Modelica.Media.Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives wrt pi and tau";
            Modelica.Media.Common.HelmholtzDerivs f "dimensionless Helmholtz funcion and dervatives wrt delta and tau";
            Integer region(min = 1, max = 5) "IF97 region";
            Integer error "error flag";
            SI.Temperature T "temperature";
            SI.Density d "density";
          algorithm 
            region:=BaseIF97.Regions.region_ph(p, h, phase);
            if region == 1 then 
                        T:=BaseIF97.Basic.tph1(p, h);
            g:=BaseIF97.Basic.g1(p, T);
            pro:=Modelica.Media.Common.ThermoFluidSpecial.gibbsToProps_ph(g);
            elseif region == 2 then
              T:=BaseIF97.Basic.tph2(p, h);
              g:=BaseIF97.Basic.g2(p, T);
              pro:=Modelica.Media.Common.ThermoFluidSpecial.gibbsToProps_ph(g);

            elseif region == 3 then
              (d,T,error):=BaseIF97.Inverses.dtofph3(p = p, h = h, delp = 1e-07, delh = 1e-06);
              f:=BaseIF97.Basic.f3(d, T);
              pro:=Modelica.Media.Common.ThermoFluidSpecial.helmholtzToProps_ph(f);

            elseif region == 4 then
              pro:=BaseIF97.TwoPhase.waterR4_ph(p = p, h = h);

            elseif region == 5 then
              (T,error):=BaseIF97.Inverses.tofph5(p = p, h = h, reldh = 1e-07);
              g:=BaseIF97.Basic.g5(p, T);
              pro:=Modelica.Media.Common.ThermoFluidSpecial.gibbsToProps_ph(g);


            else 
            end if;
          end water_ph;
          function water_dT "calculate property record for dynamic simulation properties using d and T as dynamic states"
            extends Modelica.Icons.Function;
            input SI.Density d "density";
            input SI.Temperature T "temperature";
            input Integer phase = 0 "phase: 2 for two-phase, 1 for one phase, 0 if unknown";
            output Modelica.Media.Common.ThermoFluidSpecial.ThermoProperties_dT pro "property record for dynamic simulation";
          protected 
            SI.Pressure p "pressure";
            Integer region(min = 1, max = 5) "IF97 region";
            Modelica.Media.Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives wrt pi and tau";
            Modelica.Media.Common.HelmholtzDerivs f "dimensionless Helmholtz funcion and dervatives wrt delta and tau";
            Integer error "error flag";
          algorithm 
            region:=BaseIF97.Regions.region_dT(d, T, phase);
            if region == 1 then 
                        (p,error):=BaseIF97.Inverses.pofdt125(d = d, T = T, reldd = iter.DELD, region = 1);
            g:=BaseIF97.Basic.g1(p, T);
            pro:=Modelica.Media.Common.ThermoFluidSpecial.gibbsToProps_dT(g);
            elseif region == 2 then
              (p,error):=BaseIF97.Inverses.pofdt125(d = d, T = T, reldd = iter.DELD, region = 2);
              g:=BaseIF97.Basic.g2(p, T);
              pro:=Modelica.Media.Common.ThermoFluidSpecial.gibbsToProps_dT(g);

            elseif region == 3 then
              f:=BaseIF97.Basic.f3(d, T);
              pro:=Modelica.Media.Common.ThermoFluidSpecial.helmholtzToProps_dT(f);

            elseif region == 4 then
              pro:=BaseIF97.TwoPhase.waterR4_dT(d = d, T = T);

            elseif region == 5 then
              (p,error):=BaseIF97.Inverses.pofdt125(d = d, T = T, reldd = iter.DELD, region = 5);
              g:=BaseIF97.Basic.g5(p, T);
              pro:=Modelica.Media.Common.ThermoFluidSpecial.gibbsToProps_dT(g);


            else 
            end if;
          end water_dT;
          function water_pT "calculate property record for dynamic simulation properties using p and T as dynamic states"
            extends Modelica.Icons.Function;
            input SI.Pressure p "pressure";
            input SI.Temperature T "temperature";
            output Modelica.Media.Common.ThermoFluidSpecial.ThermoProperties_pT pro "property record for dynamic simulation";
          protected 
            SI.Density d "density";
            Integer region(min = 1, max = 5) "IF97 region";
            Modelica.Media.Common.GibbsDerivs g "dimensionless Gibbs funcion and dervatives wrt pi and tau";
            Modelica.Media.Common.HelmholtzDerivs f "dimensionless Helmholtz funcion and dervatives wrt delta and tau";
            Integer error "error flag";
          algorithm 
            region:=BaseIF97.Regions.region_pT(p, T);
            if region == 1 then 
                        g:=BaseIF97.Basic.g1(p, T);
            pro:=Modelica.Media.Common.ThermoFluidSpecial.gibbsToProps_pT(g);
            elseif region == 2 then
              g:=BaseIF97.Basic.g2(p, T);
              pro:=Modelica.Media.Common.ThermoFluidSpecial.gibbsToProps_pT(g);

            elseif region == 3 then
              (d,error):=BaseIF97.Inverses.dofpt3(p = p, T = T, delp = iter.DELP);
              f:=BaseIF97.Basic.f3(d, T);
              pro:=Modelica.Media.Common.ThermoFluidSpecial.helmholtzToProps_pT(f);

            elseif region == 5 then
              g:=BaseIF97.Basic.g5(p, T);
              pro:=Modelica.Media.Common.ThermoFluidSpecial.gibbsToProps_pT(g);


            else 
            end if;
          end water_pT;
        end ThermoFluidSpecial;
      public 
        function hl_p = BaseIF97.Regions.hl_p "compute the saturated liquid specific h(p)";
        function hv_p = BaseIF97.Regions.hv_p "compute the saturated vapour specific h(p)";
        function sl_p = BaseIF97.Regions.sl_p "compute the saturated liquid specific s(p)";
        function sv_p = BaseIF97.Regions.sv_p "compute the saturated vapour specific s(p)";
        function rhol_T = BaseIF97.Regions.rhol_T "compute the saturated liquid d(T)";
        function rhov_T = BaseIF97.Regions.rhov_T "compute the saturated vapour d(T)";
        function rhol_p = BaseIF97.Regions.rhol_p "compute the saturated liquid d(p)";
        function rhov_p = BaseIF97.Regions.rhov_p "compute the saturated vapour d(p)";
        function dynamicViscosity = BaseIF97.Transport.visc_dTp "compute eta(d,T) in the one-phase region";
        function thermalConductivity = BaseIF97.Transport.cond_dTp "compute lambda(d,T,p) in the one-phase region";
        function surfaceTension = BaseIF97.Transport.surfaceTension "compute sigma(T) at saturation T";
        function isentropicEnthalpy "isentropic specific enthalpy from p,s (preferably use dynamicIsentropicEnthalpy in dynamic simulation!)"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEntropy s "specific entropy";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          input Integer region = 0 "if 0, region is unknown, otherwise known and this input";
          output SI.SpecificEnthalpy h "specific enthalpy";
        algorithm 
          h:=isentropicEnthalpy_props(p, s, waterBaseProp_ps(p, s, phase, region));
        end isentropicEnthalpy;
        function isentropicEnthalpy_props
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEntropy s "specific entropy";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          output SI.SpecificEnthalpy h "isentropic enthalpay";
          annotation(derivative(noDerivative = aux) = isentropicEnthalpy_der, Inline = false, LateInline = true);
        algorithm 
          h:=aux.h;
        end isentropicEnthalpy_props;
        function isentropicEnthalpy_der "derivative of isentropic specific enthalpy from p,s"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEntropy s "specific entropy";
          input Common.IF97BaseTwoPhase aux "auxiliary record";
          input Real p_der "pressure derivative";
          input Real s_der "entropy derivative";
          output Real h_der "specific enthalpy derivative";
        algorithm 
          h_der:=1 / aux.rho * p_der + aux.T * s_der;
        end isentropicEnthalpy_der;
        function dynamicIsentropicEnthalpy "isentropic specific enthalpy from p,s and good guesses of d and T"
          extends Modelica.Icons.Function;
          input SI.Pressure p "pressure";
          input SI.SpecificEntropy s "specific entropy";
          input SI.Density dguess "good guess density, e.g. from adjacent volume";
          input SI.Temperature Tguess "good guess temperature, e.g. from adjacent volume";
          input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
          output SI.SpecificEnthalpy h "specific enthalpy";
        algorithm 
          h:=BaseIF97.Isentropic.water_hisentropic_dyn(p, s, dguess, Tguess, 0);
        end dynamicIsentropicEnthalpy;
      end IF97_Utilities;
    end Water;
    package IdealGases "Data and models of ideal gases (single, fixed and dynamic mixtures) from NASA source"
      extends Modelica.Icons.Library;
      annotation(__Dymola_classOrder = {"Common","SingleGases","MixtureGases"}, Documentation(info = "<HTML>
<p>This package contains data for the 1241 ideal gases from </p>
<blockquote>
  <p>McBride B.J., Zehe M.J., and Gordon S. (2002): <b>NASA Glenn Coefficients
  for Calculating Thermodynamic Properties of Individual Species</b>. NASA
  report TP-2002-211556</p>
</blockquote>
<p>Medium models for some of these gases are available in package
<a href=\"Modelica://Modelica.Media.IdealGases.SingleGases\">IdealGases.SingleGases</a>
and some examples for mixtures are available in package <a href=\"Modelica://Modelica.Media.IdealGases.MixtureGases\">IdealGases.MixtureGases</a>
</p>
<h4>Using and Adapting Medium Models</h4>
<p>
The data records allow computing the ideal gas specific enthalpy, specific entropy and heat capacity of the substances listed below. From them, even the Gibbs energy and equilibrium constants for reactions can be computed. Critical data that is needed for computing the viscosity and thermal conductivity is not included. In order to add mixtures or single substance medium packages that are
subtypes of
<a href=\"Modelica://Modelica.Media.Interfaces.PartialMedium\">Interfaces.PartialMedium</a>
(i.e., can be utilized at all places where PartialMedium is defined),
a few additional steps have to be performed:
<ol>
<li>
All single gas media need to define a constant instance of record
<a href=\"Modelica://Modelica.Media.IdealGases.Common.SingleGasNasa.fluidConstants\">IdealGases.Common.SingleGasNasa.FluidConstants</a>.
For 37 ideal gases such records are provided in package
<a href=\"Modelica://Modelica.Media.IdealGases.Common.FluidData\">IdealGases.Common.FluidData</a>.
For the other gases, such a record instance has to be provided by the user, e.g. by getting
the data from a commercial or public data base. A public source of the needed data is for example the <a href=\"http://webbook.nist.gov/chemistry/\"> NIST Chemistry WebBook</a></li>

<li>When the data is available, and a user has an instance of a
<a href=\"Modelica://Modelica.Media.IdealGases.Common.SingleGasNasa.fluidConstants\">FluidConstants</a> record filled with data, a medium package has to be written. Note that only the dipole moment, the accentric factor and critical data are necessary for the viscosity and thermal conductivity functions.</li>
<ul>
<li>For single components, a new package following the pattern in
<a href=\"Modelica://Modelica.Media.IdealGases.SingleGases\">IdealGases.SingleGases</a> has to be created, pointing both to a data record for cp and to a user-defined fluidContants record.</li>
<li>For mixtures of several components, a new package following the pattern in
<a href=\"Modelica://Modelica.Media.IdealGases.MixtureGases\">IdealGases.MixtureGases</a> has to be created, building an array of data records for cp and an array of (partly) user-defined fluidContants records.</li>
</ul>
</ol>
<p>Note that many properties can computed for the full set of 1241 gases listed below, but due to the missing viscosity and thermal conductivity functions, no fully Modelica.Media-compliant media can be defined. </p>
</p>
<p>
Data records for heat capacity, specific enthalpy and specific entropy exist for the following substances and ions:
</p>
<pre>
 Ag        BaOH+           C2H4O_ethylen_o DF      In2I4    Nb      ScO2
 Ag+       Ba_OH_2         CH3CHO_ethanal  DOCl    In2I6    Nb+     Sc2O
 Ag-       BaS             CH3COOH         DO2     In2O     Nb-     Sc2O2
 Air       Ba2             OHCH2COOH       DO2-    K        NbCl5   Si
 Al        Be              C2H5            D2      K+       NbO     Si+
 Al+       Be+             C2H5Br          D2+     K-       NbOCl3  Si-
 Al-       Be++            C2H6            D2-     KAlF4    NbO2    SiBr
 AlBr      BeBr            CH3N2CH3        D2O     KBO2     Ne      SiBr2
 AlBr2     BeBr2           C2H5OH          D2O2    KBr      Ne+     SiBr3
 AlBr3     BeCl            CH3OCH3         D2S     KCN      Ni      SiBr4
 AlC       BeCl2           CH3O2CH3        e-      KCl      Ni+     SiC
 AlC2      BeF             CCN             F       KF       Ni-     SiC2
 AlCl      BeF2            CNC             F+      KH       NiCl    SiCl
 AlCl+     BeH             OCCN            F-      KI       NiCl2   SiCl2
 AlCl2     BeH+            C2N2            FCN     Kli      NiO     SiCl3
 AlCl3     BeH2            C2O             FCO     KNO2     NiS     SiCl4
 AlF       BeI             C3              FO      KNO3     O       SiF
 AlF+      BeI2            C3H3_1_propynl  FO2_FOO KNa      O+      SiFCl
 AlFCl     BeN             C3H3_2_propynl  FO2_OFO KO       O-      SiF2
 AlFCl2    BeO             C3H4_allene     F2      KOH      OD      SiF3
 AlF2      BeOH            C3H4_propyne    F2O     K2       OD-     SiF4
 AlF2-     BeOH+           C3H4_cyclo      F2O2    K2+      OH      SiH
 AlF2Cl    Be_OH_2         C3H5_allyl      FS2F    K2Br2    OH+     SiH+
 AlF3      BeS             C3H6_propylene  Fe      K2CO3    OH-     SiHBr3
 AlF4-     Be2             C3H6_cyclo      Fe+     K2C2N2   O2      SiHCl
 AlH       Be2Cl4          C3H6O_propylox  Fe_CO_5 K2Cl2    O2+     SiHCl3
 AlHCl     Be2F4           C3H6O_acetone   FeCl    K2F2     O2-     SiHF
 AlHCl2    Be2O            C3H6O_propanal  FeCl2   K2I2     O3      SiHF3
 AlHF      Be2OF2          C3H7_n_propyl   FeCl3   K2O      P       SiHI3
 AlHFCl    Be2O2           C3H7_i_propyl   FeO     K2O+     P+      SiH2
 AlHF2     Be3O3           C3H8            Fe_OH_2 K2O2     P-      SiH2Br2
 AlH2      Be4O4           C3H8O_1propanol Fe2Cl4  K2O2H2   PCl     SiH2Cl2
 AlH2Cl    Br              C3H8O_2propanol Fe2Cl6  K2SO4    PCl2    SiH2F2
 AlH2F     Br+             CNCOCN          Ga      Kr       PCl2-   SiH2I2
 AlH3      Br-             C3O2            Ga+     Kr+      PCl3    SiH3
 AlI       BrCl            C4              GaBr    li       PCl5    SiH3Br
 AlI2      BrF             C4H2_butadiyne  GaBr2   li+      PF      SiH3Cl
 AlI3      BrF3            C4H4_1_3-cyclo  GaBr3   li-      PF+     SiH3F
 AlN       BrF5            C4H6_butadiene  GaCl    liAlF4   PF-     SiH3I
 AlO       BrO             C4H6_1butyne    GaCl2   liBO2    PFCl    SiH4
 AlO+      OBrO            C4H6_2butyne    GaCl3   liBr     PFCl-   SiI
 AlO-      BrOO            C4H6_cyclo      GaF     liCl     PFCl2   SiI2
 AlOCl     BrO3            C4H8_1_butene   GaF2    liF      PFCl4   SiN
 AlOCl2    Br2             C4H8_cis2_buten GaF3    liH      PF2     SiO
 AlOF      BrBrO           C4H8_isobutene  GaH     liI      PF2-    SiO2
 AlOF2     BrOBr           C4H8_cyclo      GaI     liN      PF2Cl   SiS
 AlOF2-    C               C4H9_n_butyl    GaI2    liNO2    PF2Cl3  SiS2
 AlOH      C+              C4H9_i_butyl    GaI3    liNO3    PF3     Si2
 AlOHCl    C-              C4H9_s_butyl    GaO     liO      PF3Cl2  Si2C
 AlOHCl2   CBr             C4H9_t_butyl    GaOH    liOF     PF4Cl   Si2F6
 AlOHF     CBr2            C4H10_n_butane  Ga2Br2  liOH     PF5     Si2N
 AlOHF2    CBr3            C4H10_isobutane Ga2Br4  liON     PH      Si3
 AlO2      CBr4            C4N2            Ga2Br6  li2      PH2     Sn
 AlO2-     CCl             C5              Ga2Cl2  li2+     PH2-    Sn+
 Al_OH_2   CCl2            C5H6_1_3cyclo   Ga2Cl4  li2Br2   PH3     Sn-
 Al_OH_2Cl CCl2Br2         C5H8_cyclo      Ga2Cl6  li2F2    PN      SnBr
 Al_OH_2F  CCl3            C5H10_1_pentene Ga2F2   li2I2    PO      SnBr2
 Al_OH_3   CCl3Br          C5H10_cyclo     Ga2F4   li2O     PO-     SnBr3
 AlS       CCl4            C5H11_pentyl    Ga2F6   li2O+    POCl3   SnBr4
 AlS2      CF              C5H11_t_pentyl  Ga2I2   li2O2    POFCl2  SnCl
 Al2       CF+             C5H12_n_pentane Ga2I4   li2O2H2  POF2Cl  SnCl2
 Al2Br6    CFBr3           C5H12_i_pentane Ga2I6   li2SO4   POF3    SnCl3
 Al2C2     CFCl            CH3C_CH3_2CH3   Ga2O    li3+     PO2     SnCl4
 Al2Cl6    CFClBr2         C6D5_phenyl     Ge      li3Br3   PO2-    SnF
 Al2F6     CFCl2           C6D6            Ge+     li3Cl3   PS      SnF2
 Al2I6     CFCl2Br         C6H2            Ge-     li3F3    P2      SnF3
 Al2O      CFCl3           C6H5_phenyl     GeBr    li3I3    P2O3    SnF4
 Al2O+     CF2             C6H5O_phenoxy   GeBr2   Mg       P2O4    SnI
 Al2O2     CF2+            C6H6            GeBr3   Mg+      P2O5    SnI2
 Al2O2+    CF2Br2          C6H5OH_phenol   GeBr4   MgBr     P3      SnI3
 Al2O3     CF2Cl           C6H10_cyclo     GeCl    MgBr2    P3O6    SnI4
 Al2S      CF2ClBr         C6H12_1_hexene  GeCl2   MgCl     P4      SnO
 Al2S2     CF2Cl2          C6H12_cyclo     GeCl3   MgCl+    P4O6    SnO2
 Ar        CF3             C6H13_n_hexyl   GeCl4   MgCl2    P4O7    SnS
 Ar+       CF3+            C6H14_n_hexane  GeF     MgF      P4O8    SnS2
 B         CF3Br           C7H7_benzyl     GeF2    MgF+     P4O9    Sn2
 B+        CF3Cl           C7H8            GeF3    MgF2     P4O10   Sr
 B-        CF4             C7H8O_cresol_mx GeF4    MgF2+    Pb      Sr+
 BBr       CH+             C7H14_1_heptene GeH4    MgH      Pb+     SrBr
 BBr2      CHBr3           C7H15_n_heptyl  GeI     MgI      Pb-     SrBr2
 BBr3      CHCl            C7H16_n_heptane GeO     MgI2     PbBr    SrCl
 BC        CHClBr2         C7H16_2_methylh GeO2    MgN      PbBr2   SrCl+
 BC2       CHCl2           C8H8_styrene    GeS     MgO      PbBr3   SrCl2
 BCl       CHCl2Br         C8H10_ethylbenz GeS2    MgOH     PbBr4   SrF
 BCl+      CHCl3           C8H16_1_octene  Ge2     MgOH+    PbCl    SrF+
 BClOH     CHF             C8H17_n_octyl   H       Mg_OH_2  PbCl2   SrF2
 BCl_OH_2  CHFBr2          C8H18_n_octane  H+      MgS      PbCl3   SrH
 BCl2      CHFCl           C8H18_isooctane H-      Mg2      PbCl4   SrI
 BCl2+     CHFClBr         C9H19_n_nonyl   HAlO    Mg2F4    PbF     SrI2
 BCl2OH    CHFCl2          C10H8_naphthale HAlO2   Mn       PbF2    SrO
 BF        CHF2            C10H21_n_decyl  HBO     Mn+      PbF3    SrOH
 BFCl      CHF2Br          C12H9_o_bipheny HBO+    Mo       PbF4    SrOH+
 BFCl2     CHF2Cl          C12H10_biphenyl HBO2    Mo+      PbI     Sr_OH_2
 BFOH      CHF3            Ca              HBS     Mo-      PbI2    SrS
 BF_OH_2   CHI3            Ca+             HBS+    MoO      PbI3    Sr2
 BF2       CH2             CaBr            HCN     MoO2     PbI4    Ta
 BF2+      CH2Br2          CaBr2           HCO     MoO3     PbO     Ta+
 BF2-      CH2Cl           CaCl            HCO+    MoO3-    PbO2    Ta-
 BF2Cl     CH2ClBr         CaCl+           HCCN    Mo2O6    PbS     TaCl5
 BF2OH     CH2Cl2          CaCl2           HCCO    Mo3O9    PbS2    TaO
 BF3       CH2F            CaF             HCl     Mo4O12   Rb      TaO2
 BF4-      CH2FBr          CaF+            HD      Mo5O15   Rb+     Ti
 BH        CH2FCl          CaF2            HD+     N        Rb-     Ti+
 BHCl      CH2F2           CaH             HDO     N+       RbBO2   Ti-
 BHCl2     CH2I2           CaI             HDO2    N-       RbBr    TiCl
 BHF       CH3             CaI2            HF      NCO      RbCl    TiCl2
 BHFCl     CH3Br           CaO             HI      ND       RbF     TiCl3
 BHF2      CH3Cl           CaO+            HNC     ND2      RbH     TiCl4
 BH2       CH3F            CaOH            HNCO    ND3      RbI     TiO
 BH2Cl     CH3I            CaOH+           HNO     NF       RbK     TiO+
 BH2F      CH2OH           Ca_OH_2         HNO2    NF2      Rbli    TiOCl
 BH3       CH2OH+          CaS             HNO3    NF3      RbNO2   TiOCl2
 BH3NH3    CH3O            Ca2             HOCl    NH       RbNO3   TiO2
 BH4       CH4             Cd              HOF     NH+      RbNa    U
 BI        CH3OH           Cd+             HO2     NHF      RbO     UF
 BI2       CH3OOH          Cl              HO2-    NHF2     RbOH    UF+
 BI3       CI              Cl+             HPO     NH2      Rb2Br2  UF-
 BN        CI2             Cl-             HSO3F   NH2F     Rb2Cl2  UF2
 BO        CI3             ClCN            H2      NH3      Rb2F2   UF2+
 BO-       CI4             ClF             H2+     NH2OH    Rb2I2   UF2-
 BOCl      CN              ClF3            H2-     NH4+     Rb2O    UF3
 BOCl2     CN+             ClF5            HBOH    NO       Rb2O2   UF3+
 BOF       CN-             ClO             HCOOH   NOCl     Rb2O2H2 UF3-
 BOF2      CNN             ClO2            H2F2    NOF      Rb2SO4  UF4
 BOH       CO              Cl2             H2O     NOF3     Rn      UF4+
 BO2       CO+             Cl2O            H2O+    NO2      Rn+     UF4-
 BO2-      COCl            Co              H2O2    NO2-     S       UF5
 B_OH_2    COCl2           Co+             H2S     NO2Cl    S+      UF5+
 BS        COFCl           Co-             H2SO4   NO2F     S-      UF5-
 BS2       COF2            Cr              H2BOH   NO3      SCl     UF6
 B2        COHCl           Cr+             HB_OH_2 NO3-     SCl2    UF6-
 B2C       COHF            Cr-             H3BO3   NO3F     SCl2+   UO
 B2Cl4     COS             CrN             H3B3O3  N2       SD      UO+
 B2F4      CO2             CrO             H3B3O6  N2+      SF      UOF
 B2H       CO2+            CrO2            H3F3    N2-      SF+     UOF2
 B2H2      COOH            CrO3            H3O+    NCN      SF-     UOF3
 B2H3      CP              CrO3-           H4F4    N2D2_cis SF2     UOF4
 B2H3_db   CS              Cs              H5F5    N2F2     SF2+    UO2
 B2H4      CS2             Cs+             H6F6    N2F4     SF2-    UO2+
 B2H4_db   C2              Cs-             H7F7    N2H2     SF3     UO2-
 B2H5      C2+             CsBO2           He      NH2NO2   SF3+    UO2F
 B2H5_db   C2-             CsBr            He+     N2H4     SF3-    UO2F2
 B2H6      C2Cl            CsCl            Hg      N2O      SF4     UO3
 B2O       C2Cl2           CsF             Hg+     N2O+     SF4+    UO3-
 B2O2      C2Cl3           CsH             HgBr2   N2O3     SF4-    V
 B2O3      C2Cl4           CsI             I       N2O4     SF5     V+
 B2_OH_4   C2Cl6           Csli            I+      N2O5     SF5+    V-
 B2S       C2F             CsNO2           I-      N3       SF5-    VCl4
 B2S2      C2FCl           CsNO3           IF5     N3H      SF6     VN
 B2S3      C2FCl3          CsNa            IF7     Na       SF6-    VO
 B3H7_C2v  C2F2            CsO             I2      Na+      SH      VO2
 B3H7_Cs   C2F2Cl2         CsOH            In      Na-      SH-     V4O10
 B3H9      C2F3            CsRb            In+     NaAlF4   SN      W
 B3N3H6    C2F3Cl          Cs2             InBr    NaBO2    SO      W+
 B3O3Cl3   C2F4            Cs2Br2          InBr2   NaBr     SO-     W-
 B3O3FCl2  C2F6            Cs2CO3          InBr3   NaCN     SOF2    WCl6
 B3O3F2Cl  C2H             Cs2Cl2          InCl    NaCl     SO2     WO
 B3O3F3    C2HCl           Cs2F2           InCl2   NaF      SO2-    WOCl4
 B4H4      C2HCl3          Cs2I2           InCl3   NaH      SO2Cl2  WO2
 B4H10     C2HF            Cs2O            InF     NaI      SO2FCl  WO2Cl2
 B4H12     C2HFCl2         Cs2O+           InF2    Nali     SO2F2   WO3
 B5H9      C2HF2Cl         Cs2O2           InF3    NaNO2    SO3     WO3-
 Ba        C2HF3           Cs2O2H2         InH     NaNO3    S2      Xe
 Ba+       C2H2_vinylidene Cs2SO4          InI     NaO      S2-     Xe+
 BaBr      C2H2Cl2         Cu              InI2    NaOH     S2Cl2   Zn
 BaBr2     C2H2FCl         Cu+             InI3    NaOH+    S2F2    Zn+
 BaCl      C2H2F2          Cu-             InO     Na2      S2O     Zr
 BaCl+     CH2CO_ketene    CuCl            InOH    Na2Br2   S3      Zr+
 BaCl2     O_CH_2O         CuF             In2Br2  Na2Cl2   S4      Zr-
 BaF       HO_CO_2OH       CuF2            In2Br4  Na2F2    S5      ZrN
 BaF+      C2H3_vinyl      CuO             In2Br6  Na2I2    S6      ZrO
 BaF2      CH2Br-COOH      Cu2             In2Cl2  Na2O     S7      ZrO+
 BaH       C2H3Cl          Cu3Cl3          In2Cl4  Na2O+    S8      ZrO2
 BaI       CH2Cl-COOH      D               In2Cl6  Na2O2    Sc
 BaI2      C2H3F           D+              In2F2   Na2O2H2  Sc+
 BaO       CH3CN           D-              In2F4   Na2SO4   Sc-
 BaO+      CH3CO_acetyl    DBr             In2F6   Na3Cl3   ScO
 BaOH      C2H4            DCl             In2I2   Na3F3    ScO+
</pre></HTML>"));
      package Common "Common packages and data for the ideal gas models"
        extends Modelica.Icons.Library;
        record DataRecord "Coefficient data record for properties of ideal gases based on NASA source"
          extends Modelica.Icons.Record;
          String name "Name of ideal gas";
          SI.MolarMass MM "Molar mass";
          SI.SpecificEnthalpy Hf "Enthalpy of formation at 298.15K";
          SI.SpecificEnthalpy H0 "H0(298.15K) - H0(0K)";
          SI.Temperature Tlimit "Temperature limit between low and high data sets";
          Real alow[7] "Low temperature coefficients a";
          Real blow[2] "Low temperature constants b";
          Real ahigh[7] "High temperature coefficients a";
          Real bhigh[2] "High temperature constants b";
          SI.SpecificHeatCapacity R "Gas constant";
          annotation(Documentation(info = "<HTML>
<p>
This data record contains the coefficients for the
ideal gas equations according to:
</p>
<blockquote>
  <p>McBride B.J., Zehe M.J., and Gordon S. (2002): <b>NASA Glenn Coefficients
  for Calculating Thermodynamic Properties of Individual Species</b>. NASA
  report TP-2002-211556</p>
</blockquote>
<p>
The equations have the following structure:
</p>
<IMG SRC=\"../Images/Media/IdealGases/singleEquations.png\">
<p>
The polynomials for h(T) and s0(T) are derived via integration from the one for cp(T)  and contain the integration constants b1, b2 that define the reference specific enthalpy and entropy. For entropy differences the reference pressure p0 is arbitrary, but not for absolute entropies. It is chosen as 1 standard atmosphere (101325 Pa).
</p>
<p>
For most gases, the region of validity is from 200 K to 6000 K.
The equations are splitted into two regions that are separated
by Tlimit (usually 1000 K). In both regions the gas is described
by the data above. The two branches are continuous and in most
gases also differentiable at Tlimit.
</p>
</HTML>"));
        end DataRecord;
        partial package SingleGasNasa "Medium model of an ideal gas based on NASA source"
          annotation(Documentation(info = "<HTML>
<p>
This model calculates medium properties
for an ideal gas of a single substance, or for an ideal
gas consisting of several substances where the
mass fractions are fixed. Independent variables
are temperature <b>T</b> and pressure <b>p</b>.
Only density is a function of T and p. All other quantities
are solely a function of T. The properties
are valid in the range:
</p>
<pre>
   200 K &le; T &le; 6000 K
</pre>
<p>
The following quantities are always computed:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Variable</b></td>
      <td valign=\"top\"><b>Unit</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">h</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific enthalpy h = h(T)</td></tr>
  <tr><td valign=\"top\">u</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific internal energy u = u(T)</b></td></tr>
  <tr><td valign=\"top\">d</td>
      <td valign=\"top\">kg/m^3</td>
      <td valign=\"top\">density d = d(p,T)</td></tr>
</table>
<p>
For the other variables, see the functions in
Modelica.Media.IdealGases.Common.SingleGasNasa.
Note, dynamic viscosity and thermal conductivity are only provided
for gases that use a data record from Modelica.Media.IdealGases.FluidData.
Currently these are the following gases:
</p>
<pre>
  Ar
  C2H2_vinylidene
  C2H4
  C2H5OH
  C2H6
  C3H6_propylene
  C3H7OH
  C3H8
  C4H8_1_butene
  C4H9OH
  C4H10_n_butane
  C5H10_1_pentene
  C5H12_n_pentane
  C6H6
  C6H12_1_hexene
  C6H14_n_heptane
  C7H14_1_heptene
  C8H10_ethylbenz
  CH3OH
  CH4
  CL2
  CO
  CO2
  F2
  H2
  H2O
  He
  N2
  N2O
  NH3
  NO
  O2
  SO2
  SO3
</pre>
<p>
<b>Sources for model and literature:</b><br>
Original Data: Computer program for calculation of complex chemical
equilibrium compositions and applications. Part 1: Analysis
Document ID: 19950013764 N (95N20180) File Series: NASA Technical Reports
Report Number: NASA-RP-1311  E-8017  NAS 1.61:1311
Authors: Gordon, Sanford (NASA Lewis Research Center)
 Mcbride, Bonnie J. (NASA Lewis Research Center)
Published: Oct 01, 1994.
</p>
<p><b>Known limits of validity:</b></br>
The data is valid for
temperatures between 200K and 6000K.  A few of the data sets for
monatomic gases have a discontinuous 1st derivative at 1000K, but
this never caused problems so far.
</p>
<p>
This model has been copied from the ThermoFluid library
and adapted to the Modelica.Media package.
</p>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          extends Interfaces.PartialPureSubstance(ThermoStates = Choices.IndependentVariables.pT, mediumName = data.name, substanceNames = {data.name}, singleState = false, Temperature(min = 200, max = 6000, start = 500, nominal = 500), SpecificEnthalpy(start = if referenceChoice == ReferenceEnthalpy.ZeroAt0K then data.H0 else if referenceChoice == ReferenceEnthalpy.UserDefined then h_offset else 0, nominal = 100000.0), Density(start = 10, nominal = 10), AbsolutePressure(start = 1000000.0, nominal = 1000000.0));
          record extends ThermodynamicState "thermodynamic state variables for ideal gases"
            AbsolutePressure p "Absolute pressure of medium";
            Temperature T "Temperature of medium";
          end ThermodynamicState;
          record extends FluidConstants "Extended fluid constants"
            Temperature criticalTemperature "critical temperature";
            AbsolutePressure criticalPressure "critical pressure";
            MolarVolume criticalMolarVolume "critical molar Volume";
            Real acentricFactor "Pitzer acentric factor";
            Temperature triplePointTemperature "triple point temperature";
            AbsolutePressure triplePointPressure "triple point pressure";
            Temperature meltingPoint "melting point at 101325 Pa";
            Temperature normalBoilingPoint "normal boiling point (at 101325 Pa)";
            DipoleMoment dipoleMoment "dipole moment of molecule in Debye (1 debye = 3.33564e10-30 C.m)";
            Boolean hasIdealGasHeatCapacity = false "true if ideal gas heat capacity is available";
            Boolean hasCriticalData = false "true if critical data are known";
            Boolean hasDipoleMoment = false "true if a dipole moment known";
            Boolean hasFundamentalEquation = false "true if a fundamental equation";
            Boolean hasLiquidHeatCapacity = false "true if liquid heat capacity is available";
            Boolean hasSolidHeatCapacity = false "true if solid heat capacity is available";
            Boolean hasAccurateViscosityData = false "true if accurate data for a viscosity function is available";
            Boolean hasAccurateConductivityData = false "true if accurate data for thermal conductivity is available";
            Boolean hasVapourPressureCurve = false "true if vapour pressure data, e.g. Antoine coefficents are known";
            Boolean hasAcentricFactor = false "true if Pitzer accentric factor is known";
            SpecificEnthalpy HCRIT0 = 0.0 "Critical specific enthalpy of the fundamental equation";
            SpecificEntropy SCRIT0 = 0.0 "Critical specific entropy of the fundamental equation";
            SpecificEnthalpy deltah = 0.0 "Difference between specific enthalpy model (h_m) and f.eq. (h_f) (h_m - h_f)";
            SpecificEntropy deltas = 0.0 "Difference between specific enthalpy model (s_m) and f.eq. (s_f) (s_m - s_f)";
          end FluidConstants;
          import SI = Modelica.SIunits;
          import Modelica.Math;
          import Modelica.Media.Interfaces.PartialMedium.Choices.ReferenceEnthalpy;
          constant Boolean excludeEnthalpyOfFormation = true "If true, enthalpy of formation Hf is not included in specific enthalpy h";
          constant ReferenceEnthalpy referenceChoice = Choices.ReferenceEnthalpy.ZeroAt0K "Choice of reference enthalpy";
          constant SpecificEnthalpy h_offset = 0.0 "User defined offset for reference enthalpy, if referenceChoice = UserDefined";
          constant IdealGases.Common.DataRecord data "Data record of ideal gas substance";
          constant FluidConstants[nS] fluidConstants "constant data for the fluid";
          model extends BaseProperties(T(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default), p(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default)) "Base properties of ideal gas medium"

          equation 
            assert(T >= 200 and T <= 6000, "
Temperature T (= " + String(T) + " K) is not in the allowed range
200 K <= T <= 6000 K required from medium model \"" + mediumName + "\".
");
            MM = data.MM;
            R = data.R;
            h = h_T(data, T, excludeEnthalpyOfFormation, referenceChoice, h_offset);
            u = h - R * T;
            d = p / (R * T);
            state.T = T;
            state.p = p;
          end BaseProperties;
          function setState_pTX "Return thermodynamic state as function of p, T and composition X"
            extends Modelica.Icons.Function;
            input AbsolutePressure p "Pressure";
            input Temperature T "Temperature";
            input MassFraction X[:] = reference_X "Mass fractions";
            output ThermodynamicState state;
          algorithm 
            state:=ThermodynamicState(p = p, T = T);
          end setState_pTX;
          function setState_phX "Return thermodynamic state as function of p, h and composition X"
            extends Modelica.Icons.Function;
            input AbsolutePressure p "Pressure";
            input SpecificEnthalpy h "Specific enthalpy";
            input MassFraction X[:] = reference_X "Mass fractions";
            output ThermodynamicState state;
          algorithm 
            state:=ThermodynamicState(p = p, T = T_h(h));
          end setState_phX;
          function setState_psX "Return thermodynamic state as function of p, s and composition X"
            extends Modelica.Icons.Function;
            input AbsolutePressure p "Pressure";
            input SpecificEntropy s "Specific entropy";
            input MassFraction X[:] = reference_X "Mass fractions";
            output ThermodynamicState state;
          algorithm 
            state:=ThermodynamicState(p = p, T = T_ps(p, s));
          end setState_psX;
          function setState_dTX "Return thermodynamic state as function of d, T and composition X"
            extends Modelica.Icons.Function;
            input Density d "density";
            input Temperature T "Temperature";
            input MassFraction X[:] = reference_X "Mass fractions";
            output ThermodynamicState state;
          algorithm 
            state:=ThermodynamicState(p = d * data.R * T, T = T);
          end setState_dTX;
          function extends setSmoothState "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
          algorithm 
            state:=ThermodynamicState(p = Media.Common.smoothStep(x, state_a.p, state_b.p, x_small), T = Media.Common.smoothStep(x, state_a.T, state_b.T, x_small));
          end setSmoothState;
          function extends pressure "return pressure of ideal gas"
          algorithm 
            p:=state.p;
          end pressure;
          function extends temperature "return temperature of ideal gas"
          algorithm 
            T:=state.T;
          end temperature;
          function extends density "return density of ideal gas"
          algorithm 
            d:=state.p / (data.R * state.T);
          end density;
          function extends specificEnthalpy "Return specific enthalpy"
            extends Modelica.Icons.Function;
          algorithm 
            h:=h_T(data, state.T);
          end specificEnthalpy;
          function extends specificInternalEnergy "Return specific internal energy"
            extends Modelica.Icons.Function;
          algorithm 
            u:=h_T(data, state.T) - data.R * state.T;
          end specificInternalEnergy;
          function extends specificEntropy "Return specific entropy"
            extends Modelica.Icons.Function;
          algorithm 
            s:=s0_T(data, state.T) - data.R * Modelica.Math.log(state.p / reference_p);
          end specificEntropy;
          function extends specificGibbsEnergy "Return specific Gibbs energy"
            extends Modelica.Icons.Function;
          algorithm 
            g:=h_T(data, state.T) - state.T * specificEntropy(state);
          end specificGibbsEnergy;
          function extends specificHelmholtzEnergy "Return specific Helmholtz energy"
            extends Modelica.Icons.Function;
          algorithm 
            f:=h_T(data, state.T) - data.R * state.T - state.T * specificEntropy(state);
          end specificHelmholtzEnergy;
          function extends specificHeatCapacityCp "Return specific heat capacity at constant pressure"
          algorithm 
            cp:=cp_T(data, state.T);
          end specificHeatCapacityCp;
          function extends specificHeatCapacityCv "Compute specific heat capacity at constant volume from temperature and gas data"
          algorithm 
            cv:=cp_T(data, state.T) - data.R;
          end specificHeatCapacityCv;
          function extends isentropicExponent "Return isentropic exponent"
          algorithm 
            gamma:=specificHeatCapacityCp(state) / specificHeatCapacityCv(state);
          end isentropicExponent;
          function extends velocityOfSound "Return velocity of sound"
            extends Modelica.Icons.Function;
          algorithm 
            a:=sqrt(max(0, (data.R * state.T * cp_T(data, state.T)) / specificHeatCapacityCv(state)));
          end velocityOfSound;
          function isentropicEnthalpyApproximation "approximate method of calculating h_is from upstream properties and downstream pressure"
            extends Modelica.Icons.Function;
            input SI.Pressure p2 "downstream pressure";
            input ThermodynamicState state "properties at upstream location";
            input Boolean exclEnthForm = excludeEnthalpyOfFormation "If true, enthalpy of formation Hf is not included in specific enthalpy h";
            input ReferenceEnthalpy refChoice = referenceChoice "Choice of reference enthalpy";
            input SpecificEnthalpy h_off = h_offset "User defined offset for reference enthalpy, if referenceChoice = UserDefined";
            output SI.SpecificEnthalpy h_is "isentropic enthalpy";
          protected 
            IsentropicExponent gamma = isentropicExponent(state) "Isentropic exponent";
          algorithm 
            h_is:=h_T(data, state.T, exclEnthForm, refChoice, h_off) + (gamma / (gamma - 1.0) * state.p) / density(state) * ((p2 / state.p) ^ ((gamma - 1) / gamma) - 1.0);
          end isentropicEnthalpyApproximation;
          function extends isentropicEnthalpy "Return isentropic enthalpy"
            input Boolean exclEnthForm = excludeEnthalpyOfFormation "If true, enthalpy of formation Hf is not included in specific enthalpy h";
            input ReferenceEnthalpy refChoice = referenceChoice "Choice of reference enthalpy";
            input SpecificEnthalpy h_off = h_offset "User defined offset for reference enthalpy, if referenceChoice = UserDefined";
          algorithm 
            h_is:=isentropicEnthalpyApproximation(p_downstream, refState, exclEnthForm, refChoice, h_off);
          end isentropicEnthalpy;
          function extends isobaricExpansionCoefficient "Returns overall the isobaric expansion coefficient beta"
          algorithm 
            beta:=1 / state.T;
          end isobaricExpansionCoefficient;
          function extends isothermalCompressibility "Returns overall the isothermal compressibility factor"
          algorithm 
            kappa:=1.0 / state.p;
          end isothermalCompressibility;
          function extends density_derp_T "Returns the partial derivative of density with respect to pressure at constant temperature"
          algorithm 
            ddpT:=1 / (state.T * data.R);
          end density_derp_T;
          function extends density_derT_p "Returns the partial derivative of density with respect to temperature at constant pressure"
          algorithm 
            ddTp:= -state.p / (state.T * state.T * data.R);
          end density_derT_p;
          function extends density_derX "Returns the partial derivative of density with respect to mass fractions at constant pressure and temperature"
          algorithm 
            dddX:=fill(0, nX);
          end density_derX;
          function cp_T "Compute specific heat capacity at constant pressure from temperature and gas data"
            extends Modelica.Icons.Function;
            input IdealGases.Common.DataRecord data "Ideal gas data";
            input SI.Temperature T "Temperature";
            output SI.SpecificHeatCapacity cp "Specific heat capacity at temperature T";
            annotation(InlineNoEvent = false, smoothOrder = 2);
          algorithm 
            cp:=smooth(0, if T < data.Tlimit then data.R * 1 / (T * T) * (data.alow[1] + T * (data.alow[2] + T * (1.0 * data.alow[3] + T * (data.alow[4] + T * (data.alow[5] + T * (data.alow[6] + data.alow[7] * T)))))) else data.R * 1 / (T * T) * (data.ahigh[1] + T * (data.ahigh[2] + T * (1.0 * data.ahigh[3] + T * (data.ahigh[4] + T * (data.ahigh[5] + T * (data.ahigh[6] + data.ahigh[7] * T)))))));
          end cp_T;
          function cp_Tlow "Compute specific heat capacity at constant pressure, low T region"
            extends Modelica.Icons.Function;
            input IdealGases.Common.DataRecord data "Ideal gas data";
            input SI.Temperature T "Temperature";
            output SI.SpecificHeatCapacity cp "Specific heat capacity at temperature T";
            annotation(Inline = false, derivative(zeroDerivative = data) = cp_Tlow_der);
          algorithm 
            cp:=data.R * 1 / (T * T) * (data.alow[1] + T * (data.alow[2] + T * (1.0 * data.alow[3] + T * (data.alow[4] + T * (data.alow[5] + T * (data.alow[6] + data.alow[7] * T))))));
          end cp_Tlow;
          function cp_Tlow_der "Compute specific heat capacity at constant pressure, low T region"
            extends Modelica.Icons.Function;
            input IdealGases.Common.DataRecord data "Ideal gas data";
            input SI.Temperature T "Temperature";
            input Real dT "Temperature derivative";
            output Real cp_der "Derivative of specific heat capacity";
          algorithm 
            cp_der:=(dT * data.R) / (T * T * T) * ( -2 * data.alow[1] + T * ( -data.alow[2] + T * T * (data.alow[4] + T * (2.0 * data.alow[5] + T * (3.0 * data.alow[6] + 4.0 * data.alow[7] * T)))));
          end cp_Tlow_der;
          function h_T "Compute specific enthalpy from temperature and gas data; reference is decided by the
    refChoice input, or by the referenceChoice package constant by default"
            import Modelica.Media.Interfaces.PartialMedium.Choices;
            extends Modelica.Icons.Function;
            input IdealGases.Common.DataRecord data "Ideal gas data";
            input SI.Temperature T "Temperature";
            input Boolean exclEnthForm = excludeEnthalpyOfFormation "If true, enthalpy of formation Hf is not included in specific enthalpy h";
            input Choices.ReferenceEnthalpy refChoice = referenceChoice "Choice of reference enthalpy";
            input SI.SpecificEnthalpy h_off = h_offset "User defined offset for reference enthalpy, if referenceChoice = UserDefined";
            output SI.SpecificEnthalpy h "Specific enthalpy at temperature T";
            annotation(Inline = false, smoothOrder = 2);
          algorithm 
            h:=smooth(0, (if T < data.Tlimit then data.R * ( -data.alow[1] + T * (data.blow[1] + data.alow[2] * Math.log(T) + T * (1.0 * data.alow[3] + T * (0.5 * data.alow[4] + T * (1 / 3 * data.alow[5] + T * (0.25 * data.alow[6] + 0.2 * data.alow[7] * T)))))) / T else data.R * ( -data.ahigh[1] + T * (data.bhigh[1] + data.ahigh[2] * Math.log(T) + T * (1.0 * data.ahigh[3] + T * (0.5 * data.ahigh[4] + T * (1 / 3 * data.ahigh[5] + T * (0.25 * data.ahigh[6] + 0.2 * data.ahigh[7] * T)))))) / T) + (if exclEnthForm then  -data.Hf else 0.0) + (if refChoice == Choices.ReferenceEnthalpy.ZeroAt0K then data.H0 else 0.0) + (if refChoice == Choices.ReferenceEnthalpy.UserDefined then h_off else 0.0));
          end h_T;
          function h_T_der "derivative function for h_T"
            import Modelica.Media.Interfaces.PartialMedium.Choices;
            extends Modelica.Icons.Function;
            input IdealGases.Common.DataRecord data "Ideal gas data";
            input SI.Temperature T "Temperature";
            input Boolean exclEnthForm = excludeEnthalpyOfFormation "If true, enthalpy of formation Hf is not included in specific enthalpy h";
            input Choices.ReferenceEnthalpy refChoice = referenceChoice "Choice of reference enthalpy";
            input SI.SpecificEnthalpy h_off = h_offset "User defined offset for reference enthalpy, if referenceChoice = UserDefined";
            input Real dT "Temperature derivative";
            output Real h_der "Specific enthalpy at temperature T";
          algorithm 
            h_der:=dT * cp_T(data, T);
          end h_T_der;
          function h_Tlow "Compute specific enthalpy, low T region; reference is decided by the
    refChoice input, or by the referenceChoice package constant by default"
            import Modelica.Media.Interfaces.PartialMedium.Choices;
            extends Modelica.Icons.Function;
            input IdealGases.Common.DataRecord data "Ideal gas data";
            input SI.Temperature T "Temperature";
            input Boolean exclEnthForm = excludeEnthalpyOfFormation "If true, enthalpy of formation Hf is not included in specific enthalpy h";
            input Choices.ReferenceEnthalpy refChoice = referenceChoice "Choice of reference enthalpy";
            input SI.SpecificEnthalpy h_off = h_offset "User defined offset for reference enthalpy, if referenceChoice = UserDefined";
            output SI.SpecificEnthalpy h "Specific enthalpy at temperature T";
            annotation(Inline = false, InlineNoEvent = false, smoothOrder = 2);
          algorithm 
            h:=data.R * ( -data.alow[1] + T * (data.blow[1] + data.alow[2] * Math.log(T) + T * (1.0 * data.alow[3] + T * (0.5 * data.alow[4] + T * (1 / 3 * data.alow[5] + T * (0.25 * data.alow[6] + 0.2 * data.alow[7] * T)))))) / T + (if exclEnthForm then  -data.Hf else 0.0) + (if refChoice == Choices.ReferenceEnthalpy.ZeroAt0K then data.H0 else 0.0) + (if refChoice == Choices.ReferenceEnthalpy.UserDefined then h_off else 0.0);
          end h_Tlow;
          function h_Tlow_der "Compute specific enthalpy, low T region; reference is decided by the
    refChoice input, or by the referenceChoice package constant by default"
            import Modelica.Media.Interfaces.PartialMedium.Choices;
            extends Modelica.Icons.Function;
            input IdealGases.Common.DataRecord data "Ideal gas data";
            input SI.Temperature T "Temperature";
            input Boolean exclEnthForm = excludeEnthalpyOfFormation "If true, enthalpy of formation Hf is not included in specific enthalpy h";
            input Choices.ReferenceEnthalpy refChoice = referenceChoice "Choice of reference enthalpy";
            input SI.SpecificEnthalpy h_off = h_offset "User defined offset for reference enthalpy, if referenceChoice = UserDefined";
            input Real dT(unit = "K/s") "Temperature derivative";
            output Real h_der(unit = "J/(kg.s)") "Derivative of specific enthalpy at temperature T";
          algorithm 
            h_der:=dT * cp_Tlow(data, T);
          end h_Tlow_der;
          function s0_T "Compute specific entropy from temperature and gas data"
            extends Modelica.Icons.Function;
            input IdealGases.Common.DataRecord data "Ideal gas data";
            input SI.Temperature T "Temperature";
            output SI.SpecificEntropy s "Specific entropy at temperature T";
            annotation(InlineNoEvent = false, smoothOrder = 1);
          algorithm 
            s:=noEvent(if T < data.Tlimit then data.R * (data.blow[2] - (0.5 * data.alow[1]) / (T * T) - data.alow[2] / T + data.alow[3] * Math.log(T) + T * (data.alow[4] + T * (0.5 * data.alow[5] + T * (1 / 3 * data.alow[6] + 0.25 * data.alow[7] * T)))) else data.R * (data.bhigh[2] - (0.5 * data.ahigh[1]) / (T * T) - data.ahigh[2] / T + data.ahigh[3] * Math.log(T) + T * (data.ahigh[4] + T * (0.5 * data.ahigh[5] + T * (1 / 3 * data.ahigh[6] + 0.25 * data.ahigh[7] * T)))));
          end s0_T;
          function s0_Tlow "Compute specific entropy, low T region"
            extends Modelica.Icons.Function;
            input IdealGases.Common.DataRecord data "Ideal gas data";
            input SI.Temperature T "Temperature";
            output SI.SpecificEntropy s "Specific entropy at temperature T";
            annotation(InlineNoEvent = false, smoothOrder = 1);
          algorithm 
            s:=data.R * (data.blow[2] - (0.5 * data.alow[1]) / (T * T) - data.alow[2] / T + data.alow[3] * Math.log(T) + T * (data.alow[4] + T * (0.5 * data.alow[5] + T * (1 / 3 * data.alow[6] + 0.25 * data.alow[7] * T))));
          end s0_Tlow;
          function dynamicViscosityLowPressure "Dynamic viscosity of low pressure gases"
            extends Modelica.Icons.Function;
            input SI.Temp_K T "Gas temperature";
            input SI.Temp_K Tc "Critical temperature of gas";
            input SI.MolarMass M "Molar mass of gas";
            input SI.MolarVolume Vc "Critical molar volume of gas";
            input Real w "Acentric factor of gas";
            input DipoleMoment mu "Dipole moment of gas molecule";
            input Real k = 0.0 "Special correction for highly polar substances";
            output SI.DynamicViscosity eta "Dynamic viscosity of gas";
            annotation(smoothOrder = 2);
          protected 
            parameter Real Const1_SI = 40.785 * 10 ^ ( -9.5) "Constant in formula for eta converted to SI units";
            parameter Real Const2_SI = 131.3 / 1000.0 "Constant in formula for mur converted to SI units";
            Real mur = (Const2_SI * mu) / sqrt(Vc * Tc) "Dimensionless dipole moment of gas molecule";
            Real Fc = 1 - 0.2756 * w + 0.059035 * mur ^ 4 + k "Factor to account for molecular shape and polarities of gas";
            Real Tstar "Dimensionless temperature defined by equation below";
            Real Ov "Viscosity collision integral for the gas";
            annotation(Documentation(info = "<html>
<p>
The used formula are based on the method of Chung et al (1984, 1988) referred to in ref [1] chapter 9.
The formula 9-4.10 is the one being used. The Formula is given in non-SI units, the follwong onversion constants were used to
transform the formula to SI units:
</p>

<ul>
<li> <b>Const1_SI:</b> The factor 10^(-9.5) =10^(-2.5)*1e-7 where the
     factor 10^(-2.5) originates from the conversion of g/mol->kg/mol + cm^3/mol->m^3/mol
      and the factor 1e-7 is due to conversionfrom microPoise->Pa.s.</li>
<li>  <b>Const2_SI:</b> The factor 1/3.335641e-27 = 1e-3/3.335641e-30
      where the factor 3.335641e-30 comes from debye->C.m and
      1e-3 is due to conversion from cm^3/mol->m^3/mol</li>
</ul>

<h4>References:</h4>
<p>
[1] Bruce E. Poling, John E. Prausnitz, John P. O'Connell, \"The Properties of Gases and Liquids\" 5th Ed. Mc Graw Hill.
</p>

<h4>Author</h4>
<p>T. Skoglund, Lund, Sweden, 2004-08-31</p>

</html>"));
          algorithm 
            Tstar:=(1.2593 * T) / Tc;
            Ov:=1.16145 * Tstar ^ ( -0.14874) + 0.52487 * exp( -0.7732 * Tstar) + 2.16178 * exp( -2.43787 * Tstar);
            eta:=(Const1_SI * Fc * sqrt(M * T)) / (Vc ^ (2 / 3) * Ov);
          end dynamicViscosityLowPressure;
          redeclare replaceable function extends dynamicViscosity "dynamic viscosity"
            annotation(smoothOrder = 2);
          algorithm 
            assert(fluidConstants[1].hasCriticalData, "Failed to compute dynamicViscosity: For the species \"" + mediumName + "\" no critical data is available.");
            assert(fluidConstants[1].hasDipoleMoment, "Failed to compute dynamicViscosity: For the species \"" + mediumName + "\" no critical data is available.");
            eta:=dynamicViscosityLowPressure(state.T, fluidConstants[1].criticalTemperature, fluidConstants[1].molarMass, fluidConstants[1].criticalMolarVolume, fluidConstants[1].acentricFactor, fluidConstants[1].dipoleMoment);
          end dynamicViscosity;
          function thermalConductivityEstimate "Thermal conductivity of polyatomic gases(Eucken and Modified Eucken correlation)"
            extends Modelica.Icons.Function;
            input SpecificHeatCapacity Cp "Constant pressure heat capacity";
            input DynamicViscosity eta "Dynamic viscosity";
            input Integer method(min = 1, max = 2) = 1 "1: Eucken Method, 2: Modified Eucken Method";
            output ThermalConductivity lambda "Thermal conductivity [W/(m.k)]";
            annotation(smoothOrder = 2);
          algorithm 
            lambda:=if method == 1 then eta * (Cp - data.R + 9 / 4 * data.R) else eta * (Cp - data.R) * (1.32 + 1.77 / (Cp / Modelica.Constants.R - 1.0));
            annotation(Documentation(info = "<html>
<p>
This function provides two similar methods for estimating the
thermal conductivity of polyatomic gases.
The Eucken method (input method == 1) gives good results for low temperatures,
but it tends to give an underestimated value of the thermal conductivity
(lambda) at higher temperatures.<br>
The Modified Eucken method (input method == 2) gives good results for
high-temperatures, but it tends to give an overestimated value of the
thermal conductivity (lambda) at low temperatures.
</p>
</html>"));
          end thermalConductivityEstimate;
          redeclare replaceable function extends thermalConductivity "thermal conductivity of gas"
            input Integer method = 1 "1: Eucken Method, 2: Modified Eucken Method";
            annotation(smoothOrder = 2);
          algorithm 
            assert(fluidConstants[1].hasCriticalData, "Failed to compute thermalConductivity: For the species \"" + mediumName + "\" no critical data is available.");
            lambda:=thermalConductivityEstimate(specificHeatCapacityCp(state), dynamicViscosity(state), method = method);
          end thermalConductivity;
          function extends molarMass "return the molar mass of the medium"
          algorithm 
            MM:=data.MM;
          end molarMass;
          function T_h "Compute temperature from specific enthalpy"
            input SpecificEnthalpy h "Specific enthalpy";
            output Temperature T "Temperature";
          protected 
            package Internal "Solve h(data,T) for T with given h (use only indirectly via temperature_phX)"
              extends Modelica.Media.Common.OneNonLinearEquation;
              record extends f_nonlinear_Data "Data to be passed to non-linear function"
                extends Modelica.Media.IdealGases.Common.DataRecord;
              end f_nonlinear_Data;
              function extends f_nonlinear
              algorithm 
                y:=h_T(f_nonlinear_data, x);
              end f_nonlinear;
              function extends solve
              end solve;
            end Internal;
          algorithm 
            T:=Internal.solve(h, 200, 6000, 100000.0, {1}, data);
          end T_h;
          function T_ps "Compute temperature from pressure and specific entropy"
            input AbsolutePressure p "Pressure";
            input SpecificEntropy s "Specific entropy";
            output Temperature T "Temperature";
          protected 
            package Internal "Solve h(data,T) for T with given h (use only indirectly via temperature_phX)"
              extends Modelica.Media.Common.OneNonLinearEquation;
              record extends f_nonlinear_Data "Data to be passed to non-linear function"
                extends Modelica.Media.IdealGases.Common.DataRecord;
              end f_nonlinear_Data;
              function extends f_nonlinear
              algorithm 
                y:=s0_T(f_nonlinear_data, x) - data.R * Modelica.Math.log(p / reference_p);
              end f_nonlinear;
              function extends solve
              end solve;
            end Internal;
          algorithm 
            T:=Internal.solve(s, 200, 6000, p, {1}, data);
          end T_ps;
        end SingleGasNasa;
        partial package MixtureGasNasa "Medium model of a mixture of ideal gases based on NASA source"
          annotation(Documentation(info = "<HTML>
<p>
This model calculates the medium properties for single component ideal gases.
</p>
<p>
<b>Sources for model and literature:</b><br>
Original Data: Computer program for calculation of complex chemical
equilibrium compositions and applications. Part 1: Analysis
Document ID: 19950013764 N (95N20180) File Series: NASA Technical Reports
Report Number: NASA-RP-1311  E-8017  NAS 1.61:1311
Authors: Gordon, Sanford (NASA Lewis Research Center)
 Mcbride, Bonnie J. (NASA Lewis Research Center)
Published: Oct 01, 1994.
</p>
<p><b>Known limits of validity:</b></br>
The data is valid for
temperatures between 200 K and 6000 K.  A few of the data sets for
monatomic gases have a discontinuous 1st derivative at 1000 K, but
this never caused problems so far.
</p>
<p>
This model has been copied from the ThermoFluid library.
It has been developed by Hubertus Tummescheit.
</p>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          import Modelica.Math;
          import Modelica.Media.Interfaces.PartialMedium.Choices.ReferenceEnthalpy;
          extends Modelica.Media.Interfaces.PartialMixtureMedium(ThermoStates = Choices.IndependentVariables.pTX, substanceNames = data[:].name, reducedX = false, singleState = false, reference_X = fill(1 / nX, nX), SpecificEnthalpy(start = if referenceChoice == ReferenceEnthalpy.ZeroAt0K then 300000.0 else if referenceChoice == ReferenceEnthalpy.UserDefined then h_offset else 0, nominal = 100000.0), Density(start = 10, nominal = 10), AbsolutePressure(start = 1000000.0, nominal = 1000000.0), Temperature(start = 500, nominal = 500));
          record extends ThermodynamicState "thermodynamic state variables"
          end ThermodynamicState;
          record extends FluidConstants "fluid constants"
          end FluidConstants;
          constant Modelica.Media.IdealGases.Common.DataRecord[:] data "Data records of ideal gas substances";
          constant Boolean excludeEnthalpyOfFormation = true "If true, enthalpy of formation Hf is not included in specific enthalpy h";
          constant ReferenceEnthalpy referenceChoice = ReferenceEnthalpy.ZeroAt0K "Choice of reference enthalpy";
          constant SpecificEnthalpy h_offset = 0.0 "User defined offset for reference enthalpy, if referenceChoice = UserDefined";
          constant MolarMass[nX] MMX = data[:].MM "molar masses of components";
          redeclare replaceable model extends BaseProperties(T(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default), p(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default), Xi(each stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default), redeclare final constant Boolean standardOrderComponents = true) "Base properties (p, d, T, h, u, R, MM, X, and Xi of NASA mixture gas"
            import Modelica.Media.IdealGases.Common.SingleGasNasa;

          equation 
            assert(T >= 200 and T <= 6000, "
Temperature T (=" + String(T) + " K = 200 K) is not in the allowed range
200 K <= T <= 6000 K
required from medium model \"" + mediumName + "\".");
            MM = molarMass(state);
            h = h_TX(T, X);
            R = data.R * X;
            u = h - R * T;
            d = p / (R * T);
            state.T = T;
            state.p = p;
            state.X = if fixedX then reference_X else X;
          end BaseProperties;
          function setState_pTX "Return thermodynamic state as function of p, T and composition X"
            extends Modelica.Icons.Function;
            input AbsolutePressure p "Pressure";
            input Temperature T "Temperature";
            input MassFraction X[:] = reference_X "Mass fractions";
            output ThermodynamicState state;
          algorithm 
            state:=if size(X, 1) == nX then ThermodynamicState(p = p, T = T, X = X) else ThermodynamicState(p = p, T = T, X = cat(1, X, {1 - sum(X)}));
          end setState_pTX;
          function setState_phX "Return thermodynamic state as function of p, h and composition X"
            extends Modelica.Icons.Function;
            input AbsolutePressure p "Pressure";
            input SpecificEnthalpy h "Specific enthalpy";
            input MassFraction X[:] = reference_X "Mass fractions";
            output ThermodynamicState state;
          algorithm 
            state:=if size(X, 1) == nX then ThermodynamicState(p = p, T = T_hX(h, X), X = X) else ThermodynamicState(p = p, T = T_hX(h, X), X = cat(1, X, {1 - sum(X)}));
          end setState_phX;
          function setState_psX "Return thermodynamic state as function of p, s and composition X"
            extends Modelica.Icons.Function;
            input AbsolutePressure p "Pressure";
            input SpecificEntropy s "Specific entropy";
            input MassFraction X[:] = reference_X "Mass fractions";
            output ThermodynamicState state;
          algorithm 
            state:=if size(X, 1) == nX then ThermodynamicState(p = p, T = T_psX(p, s, X), X = X) else ThermodynamicState(p = p, T = T_psX(p, s, X), X = cat(1, X, {1 - sum(X)}));
          end setState_psX;
          function setState_dTX "Return thermodynamic state as function of d, T and composition X"
            extends Modelica.Icons.Function;
            input Density d "density";
            input Temperature T "Temperature";
            input MassFraction X[:] = reference_X "Mass fractions";
            output ThermodynamicState state;
          algorithm 
            state:=if size(X, 1) == nX then ThermodynamicState(p = d * data.R * X * T, T = T, X = X) else ThermodynamicState(p = d * data.R * cat(1, X, {1 - sum(X)}) * T, T = T, X = cat(1, X, {1 - sum(X)}));
          end setState_dTX;
          function extends setSmoothState "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
          algorithm 
            state:=ThermodynamicState(p = Media.Common.smoothStep(x, state_a.p, state_b.p, x_small), T = Media.Common.smoothStep(x, state_a.T, state_b.T, x_small), X = Media.Common.smoothStep(x, state_a.X, state_b.X, x_small));
          end setSmoothState;
          function extends pressure "Return pressure of ideal gas"
          algorithm 
            p:=state.p;
          end pressure;
          function extends temperature "Return temperature of ideal gas"
          algorithm 
            T:=state.T;
          end temperature;
          function extends density "Return density of ideal gas"
          algorithm 
            d:=state.p / (state.X * data.R * state.T);
          end density;
          function extends specificEnthalpy "Return specific enthalpy"
            extends Modelica.Icons.Function;
          algorithm 
            h:=h_TX(state.T, state.X);
          end specificEnthalpy;
          function extends specificInternalEnergy "Return specific internal energy"
            extends Modelica.Icons.Function;
          algorithm 
            u:=h_TX(state.T, state.X) - gasConstant(state) * state.T;
          end specificInternalEnergy;
          function extends specificEntropy "Return specific entropy"
          protected 
            Real[nX] Y(unit = "mol/mol") = massToMoleFractions(state.X, data.MM) "Molar fractions";
          algorithm 
            s:=s_TX(state.T, state.X) - sum((state.X[i] * Modelica.Constants.R) / MMX[i] * (if state.X[i] < Modelica.Constants.eps then Y[i] else Modelica.Math.log((Y[i] * state.p) / reference_p)) for i in 1:nX);
          end specificEntropy;
          function extends specificGibbsEnergy "Return specific Gibbs energy"
            extends Modelica.Icons.Function;
          algorithm 
            g:=h_TX(state.T, state.X) - state.T * specificEntropy(state);
          end specificGibbsEnergy;
          function extends specificHelmholtzEnergy "Return specific Helmholtz energy"
            extends Modelica.Icons.Function;
          algorithm 
            f:=h_TX(state.T, state.X) - gasConstant(state) * state.T - state.T * specificEntropy(state);
          end specificHelmholtzEnergy;
          function h_TX "Return specific enthalpy"
            import Modelica.Media.Interfaces.PartialMedium.Choices;
            extends Modelica.Icons.Function;
            input SI.Temperature T "Temperature";
            input MassFraction X[:] = reference_X "Independent Mass fractions of gas mixture";
            input Boolean exclEnthForm = excludeEnthalpyOfFormation "If true, enthalpy of formation Hf is not included in specific enthalpy h";
            input Choices.ReferenceEnthalpy refChoice = referenceChoice "Choice of reference enthalpy";
            input SI.SpecificEnthalpy h_off = h_offset "User defined offset for reference enthalpy, if referenceChoice = UserDefined";
            output SI.SpecificEnthalpy h "Specific enthalpy at temperature T";
            annotation(Inline = false, smoothOrder = 2);
          algorithm 
            h:=(if fixedX then reference_X else X) * array(SingleGasNasa.h_T(data[i], T, exclEnthForm, refChoice, h_off) for i in 1:nX);
          end h_TX;
          function h_TX_der "Return specific enthalpy derivative"
            import Modelica.Media.Interfaces.PartialMedium.Choices;
            extends Modelica.Icons.Function;
            input SI.Temperature T "Temperature";
            input MassFraction X[nX] "Independent Mass fractions of gas mixture";
            input Boolean exclEnthForm = excludeEnthalpyOfFormation "If true, enthalpy of formation Hf is not included in specific enthalpy h";
            input Choices.ReferenceEnthalpy refChoice = referenceChoice "Choice of reference enthalpy";
            input SI.SpecificEnthalpy h_off = h_offset "User defined offset for reference enthalpy, if referenceChoice = UserDefined";
            input Real dT "Temperature derivative";
            input Real dX[nX] "independent mass fraction derivative";
            output Real h_der "Specific enthalpy at temperature T";
            annotation(InlineNoEvent = false, Inline = false);
          algorithm 
            h_der:=if fixedX then dT * sum(SingleGasNasa.cp_T(data[i], T) * reference_X[i] for i in 1:nX) else dT * sum(SingleGasNasa.cp_T(data[i], T) * X[i] for i in 1:nX) + sum(SingleGasNasa.h_T(data[i], T) * dX[i] for i in 1:nX);
          end h_TX_der;
          function extends gasConstant "Return gasConstant"
          algorithm 
            R:=data.R * state.X;
          end gasConstant;
          function extends specificHeatCapacityCp "Return specific heat capacity at constant pressure"
          algorithm 
            cp:=array(SingleGasNasa.cp_T(data[i], state.T) for i in 1:nX) * state.X;
          end specificHeatCapacityCp;
          function extends specificHeatCapacityCv "Return specific heat capacity at constant volume from temperature and gas data"
          algorithm 
            cv:=array(SingleGasNasa.cp_T(data[i], state.T) for i in 1:nX) * state.X - data.R * state.X;
          end specificHeatCapacityCv;
          function MixEntropy "Return mixing entropy of ideal gases / R"
            extends Modelica.Icons.Function;
            input SI.MoleFraction x[:] "mole fraction of mixture";
            output Real smix "mixing entropy contribution, divided by gas constant";
          algorithm 
            smix:=sum(if x[i] > Modelica.Constants.eps then  -x[i] * Modelica.Math.log(x[i]) else x[i] for i in 1:size(x, 1));
          end MixEntropy;
          function s_TX "Return temperature dependent part of the entropy, expects full entropy vector"
            input Temperature T "temperature";
            input MassFraction[nX] X "mass fraction";
            output SpecificEntropy s "specific entropy";
          algorithm 
            s:=sum(SingleGasNasa.s0_T(data[i], T) * X[i] for i in 1:size(X, 1));
          end s_TX;
          function extends isentropicExponent "Return isentropic exponent"
          algorithm 
            gamma:=specificHeatCapacityCp(state) / specificHeatCapacityCv(state);
          end isentropicExponent;
          function extends velocityOfSound "Return velocity of sound"
            extends Modelica.Icons.Function;
            input ThermodynamicState state "properties at upstream location";
          algorithm 
            a:=sqrt(max(0, (gasConstant(state) * state.T * specificHeatCapacityCp(state)) / specificHeatCapacityCv(state)));
          end velocityOfSound;
          function isentropicEnthalpyApproximation "Approximate method of calculating h_is from upstream properties and downstream pressure"
            extends Modelica.Icons.Function;
            input AbsolutePressure p2 "downstream pressure";
            input ThermodynamicState state "thermodynamic state at upstream location";
            output SpecificEnthalpy h_is "isentropic enthalpy";
          protected 
            SpecificEnthalpy h "specific enthalpy at upstream location";
            SpecificEnthalpy h_component[nX] "specific enthalpy at upstream location";
            IsentropicExponent gamma = isentropicExponent(state) "Isentropic exponent";
          protected 
            MassFraction[nX] X "complete X-vector";
          algorithm 
            X:=if reducedX then cat(1, state.X, {1 - sum(state.X)}) else state.X;
            h_component:=array(SingleGasNasa.h_T(data[i], state.T, excludeEnthalpyOfFormation, referenceChoice, h_offset) for i in 1:nX);
            h:=h_component * X;
            h_is:=h + gamma / (gamma - 1.0) * state.T * gasConstant(state) * ((p2 / state.p) ^ ((gamma - 1) / gamma) - 1.0);
          end isentropicEnthalpyApproximation;
          function extends isentropicEnthalpy "Return isentropic enthalpy"
            input Boolean exact = false "flag wether exact or approximate version should be used";
          algorithm 
            h_is:=if exact then specificEnthalpy_psX(p_downstream, specificEntropy(refState), refState.X) else isentropicEnthalpyApproximation(p_downstream, refState);
          end isentropicEnthalpy;
          function gasMixtureViscosity "Return viscosities of gas mixtures at low pressures (Wilke method)"
            extends Modelica.Icons.Function;
            input MoleFraction[:] yi "Mole fractions";
            input MolarMass[:] M "Mole masses";
            input DynamicViscosity[:] eta "Pure component viscosities";
            output DynamicViscosity etam "Viscosity of the mixture";
            annotation(smoothOrder = 2);
          protected 
            Real fi[size(yi, 1),size(yi, 1)];
          algorithm 
            for i in 1:size(eta, 1) loop
                          assert(fluidConstants[i].hasDipoleMoment, "Dipole moment for " + fluidConstants[i].chemicalFormula + " not known. Can not compute viscosity.");
              assert(fluidConstants[i].hasCriticalData, "Critical data for " + fluidConstants[i].chemicalFormula + " not known. Can not compute viscosity.");
              for j in 1:size(eta, 1) loop
                              if i == 1 then 
                                fi[i,j]:=(1 + (eta[i] / eta[j]) ^ (1 / 2) * (M[j] / M[i]) ^ (1 / 4)) ^ 2 / (8 * (1 + M[i] / M[j])) ^ (1 / 2);
                elseif j < i then
                  fi[i,j]:=(eta[i] / eta[j] * M[j]) / M[i] * fi[j,i];


                else                 fi[i,j]:=(1 + (eta[i] / eta[j]) ^ (1 / 2) * (M[j] / M[i]) ^ (1 / 4)) ^ 2 / (8 * (1 + M[i] / M[j])) ^ (1 / 2);

                end if;

              end for;

            end for;
            etam:=sum((yi[i] * eta[i]) / sum(yi[j] * fi[i,j] for j in 1:size(eta, 1)) for i in 1:size(eta, 1));
            annotation(Documentation(info = "<html>

<p>
Simplification of the kinetic theory (Chapman and Enskog theory)
approach neglecting the second-order effects.<br>
<br>
This equation has been extensively tested (Amdur and Mason, 1958;
Bromley and Wilke, 1951; Cheung, 1958; Dahler, 1959; Gandhi and Saxena,
1964; Ranz and Brodowsky, 1962; Saxena and Gambhir, 1963a; Strunk, et
al., 1964; Vanderslice, et al. 1962; Wright and Gray, 1962). In most
cases, only nonpolar mixtures were compared, and very good results
obtained. For some systems containing hidrogen as one component, less
satisfactory agreement was noted. Wilke's method predicted mixture
viscosities that were larger than experimental for the H2-N2 system,
but for H2-NH3, it underestimated the viscosities. <br>
Gururaja, et al. (1967) found that this method also overpredicted in
the H2-O2 case but was quite accurate for the H2-CO2 system. <br>
Wilke's approximation has proved reliable even for polar-polar gas
mixtures of aliphatic alcohols (Reid and Belenyessy, 1960). The
principal reservation appears to lie in those cases where Mi&gt;&gt;Mj
and etai&gt;&gt;etaj.<br>
</p>

</html>
"));
          end gasMixtureViscosity;
          redeclare replaceable function extends dynamicViscosity "Return mixture dynamic viscosity"
          protected 
            DynamicViscosity[nX] etaX "component dynamic viscosities";
            annotation(smoothOrder = 2);
          algorithm 
            for i in 1:nX loop
                          etaX[i]:=SingleGasNasa.dynamicViscosityLowPressure(state.T, fluidConstants[i].criticalTemperature, fluidConstants[i].molarMass, fluidConstants[i].criticalMolarVolume, fluidConstants[i].acentricFactor, fluidConstants[i].dipoleMoment);

            end for;
            eta:=gasMixtureViscosity(massToMoleFractions(state.X, fluidConstants[:].molarMass), fluidConstants[:].molarMass, etaX);
          end dynamicViscosity;
          function mixtureViscosityChung "Return the viscosity of gas mixtures without access to component viscosities (Chung, et. al. rules)"
            extends Modelica.Icons.Function;
            import SI = Modelica.SIunits;
            input Temperature T "Temperature";
            input Temperature[:] Tc "Critical temperatures";
            input MolarVolume[:] Vcrit "Critical volumes (m3/mol)";
            input Real[:] w "Acentric factors";
            input Real[:] mu "Dipole moments (debyes)";
            input MolarMass[:] MolecularWeights "Molecular weights (kg/mol)";
            input MoleFraction[:] y "Molar Fractions";
            input Real[:] kappa = zeros(nX) "Association Factors";
            output DynamicViscosity etaMixture "Mixture viscosity (Pa.s)";
          protected 
            constant Real[size(y, 1)] Vc = Vcrit * 1000000 "Critical volumes (cm3/mol)";
            constant Real[size(y, 1)] M = MolecularWeights * 1000 "Molecular weights (g/mol)";
            Integer n = size(y, 1) "Number of mixed elements";
            Real sigmam3 "Mixture sigma^3 in ngstrm";
            Real sigma[size(y, 1),size(y, 1)];
            Real edivkm;
            Real edivk[size(y, 1),size(y, 1)];
            Real Mm;
            Real Mij[size(y, 1),size(y, 1)];
            Real wm "accentric factor";
            Real wij[size(y, 1),size(y, 1)];
            Real kappam "Correlation for highly polar substances such as alcohols and acids";
            Real kappaij[size(y, 1),size(y, 1)];
            Real mum;
            Real Vcm;
            Real Tcm;
            Real murm "Dimensionless dipole moment of the mixture";
            Real Fcm "Factor to correct for shape and polarity";
            Real omegav;
            Real Tmstar;
            Real etam "Mixture viscosity in microP";
            annotation(smoothOrder = 2);
          algorithm 
            for i in 1:n loop
                          for j in 1:n loop
                              Mij[i,j]:=(2 * M[i] * M[j]) / (M[i] + M[j]);
                if i == j then 
                                sigma[i,j]:=0.809 * Vc[i] ^ (1 / 3);
                edivk[i,j]:=Tc[i] / 1.2593;
                wij[i,j]:=w[i];
                kappaij[i,j]:=kappa[i];

                else                 sigma[i,j]:=(0.809 * Vc[i] ^ (1 / 3) * 0.809 * Vc[j] ^ (1 / 3)) ^ (1 / 2);
                edivk[i,j]:=((Tc[i] / 1.2593 * Tc[j]) / 1.2593) ^ (1 / 2);
                wij[i,j]:=(w[i] + w[j]) / 2;
                kappaij[i,j]:=(kappa[i] * kappa[j]) ^ (1 / 2);

                end if;

              end for;

            end for;
            sigmam3:=sum(sum(y[i] * y[j] * sigma[i,j] ^ 3 for j in 1:n) for i in 1:n);
            edivkm:=sum(sum(y[i] * y[j] * edivk[i,j] * sigma[i,j] ^ 3 for j in 1:n) for i in 1:n) / sigmam3;
            Mm:=(sum(sum(y[i] * y[j] * edivk[i,j] * sigma[i,j] ^ 2 * Mij[i,j] ^ (1 / 2) for j in 1:n) for i in 1:n) / (edivkm * sigmam3 ^ (2 / 3))) ^ 2;
            wm:=sum(sum(y[i] * y[j] * wij[i,j] * sigma[i,j] ^ 3 for j in 1:n) for i in 1:n) / sigmam3;
            mum:=(sigmam3 * sum(sum((y[i] * y[j] * mu[i] ^ 2 * mu[j] ^ 2) / sigma[i,j] ^ 3 for j in 1:n) for i in 1:n)) ^ (1 / 4);
            Vcm:=sigmam3 / 0.809 ^ 3;
            Tcm:=1.2593 * edivkm;
            murm:=(131.3 * mum) / (Vcm * Tcm) ^ (1 / 2);
            kappam:=sigmam3 * sum(sum(y[i] * y[j] * kappaij[i,j] for j in 1:n) for i in 1:n);
            Fcm:=1 - 0.275 * wm + 0.059035 * murm ^ 4 + kappam;
            Tmstar:=T / edivkm;
            omegav:=1.16145 * Tmstar ^ ( -0.14874) + 0.52487 * Math.exp( -0.7732 * Tmstar) + 2.16178 * Math.exp( -2.43787 * Tmstar);
            etam:=(26.69 * Fcm * (Mm * T) ^ (1 / 2)) / (sigmam3 ^ (2 / 3) * omegav);
            etaMixture:=etam * 10000000.0;
            annotation(Documentation(info = "<html>

<p>
Equation to estimate the viscosity of gas mixtures at low pressures.<br>
It is a simplification of an extension of the rigorous kinetic theory
of Chapman and Enskog to determine the viscosity of multicomponent
mixtures, at low pressures and with a factor to correct for molecule
shape and polarity.
</p>

<p>
The input argument Kappa is a special correction for highly polar substances such as
alcohols and acids.<br>
Values of kappa for a few such materials:
</p>

<table style=\"text-align: left; width: 302px; height: 200px;\" border=\"1\"
cellspacing=\"0\" cellpadding=\"2\">
<tbody>
<tr>
<td style=\"vertical-align: top;\">Compound <br>
</td>
<td style=\"vertical-align: top; text-align: center;\">Kappa<br>
</td>
<td style=\"vertical-align: top;\">Compound<br>
</td>
<td style=\"vertical-align: top;\">Kappa<br>
</td>
</tr>
<tr>
<td style=\"vertical-align: top;\">Methanol<br>
</td>
<td style=\"vertical-align: top;\">0.215<br>
</td>
<td style=\"vertical-align: top;\">n-Pentanol<br>
</td>
<td style=\"vertical-align: top;\">0.122<br>
</td>
</tr>
<tr>
<td style=\"vertical-align: top;\">Ethanol<br>
</td>
<td style=\"vertical-align: top;\">0.175<br>
</td>
<td style=\"vertical-align: top;\">n-Hexanol<br>
</td>
<td style=\"vertical-align: top;\">0.114<br>
</td>
</tr>
<tr>
<td style=\"vertical-align: top;\">n-Propanol<br>
</td>
<td style=\"vertical-align: top;\">0.143<br>
</td>
<td style=\"vertical-align: top;\">n-Heptanol<br>
</td>
<td style=\"vertical-align: top;\">0.109<br>
</td>
</tr>
<tr>
<td style=\"vertical-align: top;\">i-Propanol<br>
</td>
<td style=\"vertical-align: top;\">0.143<br>
</td>
<td style=\"vertical-align: top;\">Acetic Acid<br>
</td>
<td style=\"vertical-align: top;\">0.0916<br>
</td>
</tr>
<tr>
<td style=\"vertical-align: top;\">n-Butanol<br>
</td>
<td style=\"vertical-align: top;\">0.132<br>
</td>
<td style=\"vertical-align: top;\">Water<br>
</td>
<td style=\"vertical-align: top;\">0.076<br>
</td>
</tr>
<tr>
<td style=\"vertical-align: top;\">i-Butanol<br>
</td>
<td style=\"vertical-align: top;\">0.132</td>
<td style=\"vertical-align: top;\"><br>
</td>
<td style=\"vertical-align: top;\"><br>
</td>
</tr>
</tbody>
</table>
<p>
Chung, et al. (1984) suggest that for other alcohols not shown in the
table:<br>
&nbsp;&nbsp;&nbsp;&nbsp; <br>
&nbsp;&nbsp;&nbsp; kappa = 0.0682 + 4.704*[(number of -OH
groups)]/[molecular weight]<br>
<br>
<span style=\"font-weight: normal;\">S.I. units relation for the
debyes:&nbsp;</span><br>
&nbsp;&nbsp;&nbsp; &nbsp;&nbsp; &nbsp;&nbsp; &nbsp;&nbsp; &nbsp;&nbsp;
&nbsp;&nbsp; &nbsp;&nbsp; &nbsp;&nbsp; &nbsp;&nbsp; &nbsp;&nbsp;
&nbsp;&nbsp; &nbsp;&nbsp; &nbsp;&nbsp; &nbsp;&nbsp; &nbsp;&nbsp;
&nbsp;&nbsp; &nbsp;&nbsp; &nbsp;&nbsp; 1 debye = 3.162e-25 (J.m^3)^(1/2)<br>
</p>
<h4>References</h4>
<p>
[1] THE PROPERTIES OF GASES AND LIQUIDS, Fifth Edition,<br>
&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;&nbsp; &nbsp; Bruce E. Poling, John M.
Prausnitz, John P. O'Connell.<br>
[2] Chung, T.-H., M. Ajlan, L. L. Lee, and K. E. Starling: Ind. Eng.
Chem. Res., 27: 671 (1988).<br>
[3] Chung, T.-H., L. L. Lee, and K. E. Starling; Ing. Eng. Chem.
Fundam., 23: 3 ()1984).<br>
</p>
</html>
"));
          end mixtureViscosityChung;
          function lowPressureThermalConductivity "Return thermal conductivites of low-pressure gas mixtures (Mason and Saxena Modification)"
            extends Modelica.Icons.Function;
            input MoleFraction[:] y "Mole fraction of the components in the gass mixture";
            input Temperature T "Temperature";
            input Temperature[:] Tc "Critical temperatures";
            input AbsolutePressure[:] Pc "Critical pressures";
            input MolarMass[:] M "Molecular weights";
            input ThermalConductivity[:] lambda "Thermal conductivities of the pure gases";
            output ThermalConductivity lambdam "Thermal conductivity of the gas mixture";
          protected 
            MolarMass[size(y, 1)] gamma;
            Real[size(y, 1)] Tr "Reduced temperature";
            Real[size(y, 1),size(y, 1)] A "Mason and Saxena Modification";
            constant Real epsilon = 1.0 "Numerical constant near unity";
            annotation(smoothOrder = 2);
          algorithm 
            for i in 1:size(y, 1) loop
                          gamma[i]:=210 * ((Tc[i] * M[i] ^ 3) / Pc[i] ^ 4) ^ (1 / 6);
              Tr[i]:=T / Tc[i];

            end for;
            for i in 1:size(y, 1) loop
                          for j in 1:size(y, 1) loop
                              A[i,j]:=(epsilon * (1 + ((gamma[j] * (Math.exp(0.0464 * Tr[i]) - Math.exp( -0.2412 * Tr[i]))) / (gamma[i] * (Math.exp(0.0464 * Tr[j]) - Math.exp( -0.2412 * Tr[j])))) ^ (1 / 2) * (M[i] / M[j]) ^ (1 / 4)) ^ 2) / (8 * (1 + M[i] / M[j])) ^ (1 / 2);

              end for;

            end for;
            lambdam:=sum((y[i] * lambda[i]) / sum(y[j] * A[i,j] for j in 1:size(y, 1)) for i in 1:size(y, 1));
            annotation(Documentation(info = "<html>

<p>
This function applies the Masson and Saxena modification of the
Wassiljewa Equation for the thermal conductivity for gas mixtures of
n elements at low pressure.
</p>

<p>
For nonpolar gas mixtures errors will generally be less than 3 to 4%.
For mixtures of nonpolar-polar and polar-polar gases, errors greater
than 5 to 8% may be expected. For mixtures in which the sizes and
polarities of the constituent molecules are not greatly different, the
thermal conductivity can be estimated satisfactorily by a mole fraction
average of the pure component conductivities.
</p>

</html>
"));
          end lowPressureThermalConductivity;
          redeclare replaceable function extends thermalConductivity "Return thermal conductivity for low pressure gas mixtures"
            input Integer method = 1 "method to compute single component thermal conductivity";
          protected 
            ThermalConductivity[nX] lambdaX "component thermal conductivities";
            DynamicViscosity[nX] eta "component thermal dynamic viscosities";
            SpecificHeatCapacity[nX] cp "component heat capacity";
            annotation(smoothOrder = 2);
          algorithm 
            for i in 1:nX loop
                          assert(fluidConstants[i].hasCriticalData, "Critical data for " + fluidConstants[i].chemicalFormula + " not known. Can not compute thermal conductivity.");
              eta[i]:=SingleGasNasa.dynamicViscosityLowPressure(state.T, fluidConstants[i].criticalTemperature, fluidConstants[i].molarMass, fluidConstants[i].criticalMolarVolume, fluidConstants[i].acentricFactor, fluidConstants[i].dipoleMoment);
              cp[i]:=SingleGasNasa.cp_T(data[i], state.T);
              lambdaX[i]:=SingleGasNasa.thermalConductivityEstimate(Cp = cp[i], eta = eta[i], method = method);

            end for;
            lambda:=lowPressureThermalConductivity(massToMoleFractions(state.X, fluidConstants[:].molarMass), state.T, fluidConstants[:].criticalTemperature, fluidConstants[:].criticalPressure, fluidConstants[:].molarMass, lambdaX);
          end thermalConductivity;
          function extends isobaricExpansionCoefficient "Return isobaric expansion coefficient beta"
          algorithm 
            beta:=1 / state.T;
          end isobaricExpansionCoefficient;
          function extends isothermalCompressibility "Return isothermal compressibility factor"
          algorithm 
            kappa:=1.0 / state.p;
          end isothermalCompressibility;
          function extends density_derp_T "Return density derivative by pressure at constant temperature"
          algorithm 
            ddpT:=1 / (state.T * gasConstant(state));
          end density_derp_T;
          function extends density_derT_p "Return density derivative by temperature at constant pressure"
          algorithm 
            ddTp:= -state.p / (state.T * state.T * gasConstant(state));
          end density_derT_p;
          function density_derX "Return density derivative by mass fraction"
            extends Modelica.Icons.Function;
            input ThermodynamicState state "thermodynamic state record";
            output Density[nX] dddX "Derivative of density wrt mass fraction";
            annotation(Documentation(info = "<html></html>"));
          algorithm 
            dddX:=array( -(state.p / (state.T * gasConstant(state)) * molarMass(state)) / data[i].MM for i in 1:nX);
          end density_derX;
          function extends molarMass "Return molar mass of mixture"
          algorithm 
            MM:=1 / sum(state.X[j] / data[j].MM for j in 1:size(state.X, 1));
          end molarMass;
          function T_hX "Return temperature from specific enthalpy and mass fraction"
            input SpecificEnthalpy h "specific enthalpy";
            input MassFraction[:] X "mass fractions of composition";
            output Temperature T "temperature";
          protected 
            MassFraction[nX] Xfull = if size(X, 1) == nX then X else cat(1, X, {1 - sum(X)});
            package Internal "Solve h(data,T) for T with given h (use only indirectly via temperature_phX)"
              extends Modelica.Media.Common.OneNonLinearEquation;
              record extends f_nonlinear_Data "Data to be passed to non-linear function"
                extends Modelica.Media.IdealGases.Common.DataRecord;
              end f_nonlinear_Data;
              function extends f_nonlinear
              algorithm 
                y:=h_TX(x, X);
              end f_nonlinear;
              function extends solve
              end solve;
            end Internal;
          algorithm 
            T:=Internal.solve(h, 200, 6000, 100000.0, Xfull, data[1]);
          end T_hX;
          function T_psX "Return temperature from pressure, specific entropy and mass fraction"
            input AbsolutePressure p "pressure";
            input SpecificEntropy s "specific entropy";
            input MassFraction[:] X "mass fractions of composition";
            output Temperature T "temperature";
          protected 
            MassFraction[nX] Xfull = if size(X, 1) == nX then X else cat(1, X, {1 - sum(X)});
            package Internal "Solve h(data,T) for T with given h (use only indirectly via temperature_phX)"
              extends Modelica.Media.Common.OneNonLinearEquation;
              record extends f_nonlinear_Data "Data to be passed to non-linear function"
                extends Modelica.Media.IdealGases.Common.DataRecord;
              end f_nonlinear_Data;
              function extends f_nonlinear "note that this function always sees the complete mass fraction vector"
              protected 
                MassFraction[nX] Xfull = if size(X, 1) == nX then X else cat(1, X, {1 - sum(X)});
                Real[nX] Y(unit = "mol/mol") = massToMoleFractions(if size(X, 1) == nX then X else cat(1, X, {1 - sum(X)}), data.MM) "Molar fractions";
              algorithm 
                y:=s_TX(x, Xfull) - sum((Xfull[i] * Modelica.Constants.R) / MMX[i] * (if Xfull[i] < Modelica.Constants.eps then Y[i] else Modelica.Math.log((Y[i] * p) / reference_p)) for i in 1:nX);
              end f_nonlinear;
              function extends solve
              end solve;
            end Internal;
          algorithm 
            T:=Internal.solve(s, 200, 6000, p, Xfull, data[1]);
          end T_psX;
        end MixtureGasNasa;
        annotation(Documentation(info = "<html>

</html>"));
        package SingleGasesData "Ideal gas data based on the NASA Glenn coefficients"
          extends Modelica.Icons.Library;
          annotation(Documentation(info = "<HTML>
<p>This package contains ideal gas models for the 1241 ideal gases from </p>
<blockquote>
  <p>McBride B.J., Zehe M.J., and Gordon S. (2002): <b>NASA Glenn Coefficients
  for Calculating Thermodynamic Properties of Individual Species</b>. NASA
  report TP-2002-211556</p>
</blockquote>

<pre>
 Ag        BaOH+           C2H4O_ethylen_o DF      In2I4    Nb      ScO2
 Ag+       Ba_OH_2         CH3CHO_ethanal  DOCl    In2I6    Nb+     Sc2O
 Ag-       BaS             CH3COOH         DO2     In2O     Nb-     Sc2O2
 Air       Ba2             OHCH2COOH       DO2-    K        NbCl5   Si
 Al        Be              C2H5            D2      K+       NbO     Si+
 Al+       Be+             C2H5Br          D2+     K-       NbOCl3  Si-
 Al-       Be++            C2H6            D2-     KAlF4    NbO2    SiBr
 AlBr      BeBr            CH3N2CH3        D2O     KBO2     Ne      SiBr2
 AlBr2     BeBr2           C2H5OH          D2O2    KBr      Ne+     SiBr3
 AlBr3     BeCl            CH3OCH3         D2S     KCN      Ni      SiBr4
 AlC       BeCl2           CH3O2CH3        e-      KCl      Ni+     SiC
 AlC2      BeF             CCN             F       KF       Ni-     SiC2
 AlCl      BeF2            CNC             F+      KH       NiCl    SiCl
 AlCl+     BeH             OCCN            F-      KI       NiCl2   SiCl2
 AlCl2     BeH+            C2N2            FCN     Kli      NiO     SiCl3
 AlCl3     BeH2            C2O             FCO     KNO2     NiS     SiCl4
 AlF       BeI             C3              FO      KNO3     O       SiF
 AlF+      BeI2            C3H3_1_propynl  FO2_FOO KNa      O+      SiFCl
 AlFCl     BeN             C3H3_2_propynl  FO2_OFO KO       O-      SiF2
 AlFCl2    BeO             C3H4_allene     F2      KOH      OD      SiF3
 AlF2      BeOH            C3H4_propyne    F2O     K2       OD-     SiF4
 AlF2-     BeOH+           C3H4_cyclo      F2O2    K2+      OH      SiH
 AlF2Cl    Be_OH_2         C3H5_allyl      FS2F    K2Br2    OH+     SiH+
 AlF3      BeS             C3H6_propylene  Fe      K2CO3    OH-     SiHBr3
 AlF4-     Be2             C3H6_cyclo      Fe+     K2C2N2   O2      SiHCl
 AlH       Be2Cl4          C3H6O_propylox  Fe_CO_5 K2Cl2    O2+     SiHCl3
 AlHCl     Be2F4           C3H6O_acetone   FeCl    K2F2     O2-     SiHF
 AlHCl2    Be2O            C3H6O_propanal  FeCl2   K2I2     O3      SiHF3
 AlHF      Be2OF2          C3H7_n_propyl   FeCl3   K2O      P       SiHI3
 AlHFCl    Be2O2           C3H7_i_propyl   FeO     K2O+     P+      SiH2
 AlHF2     Be3O3           C3H8            Fe_OH_2 K2O2     P-      SiH2Br2
 AlH2      Be4O4           C3H8O_1propanol Fe2Cl4  K2O2H2   PCl     SiH2Cl2
 AlH2Cl    Br              C3H8O_2propanol Fe2Cl6  K2SO4    PCl2    SiH2F2
 AlH2F     Br+             CNCOCN          Ga      Kr       PCl2-   SiH2I2
 AlH3      Br-             C3O2            Ga+     Kr+      PCl3    SiH3
 AlI       BrCl            C4              GaBr    li       PCl5    SiH3Br
 AlI2      BrF             C4H2_butadiyne  GaBr2   li+      PF      SiH3Cl
 AlI3      BrF3            C4H4_1_3-cyclo  GaBr3   li-      PF+     SiH3F
 AlN       BrF5            C4H6_butadiene  GaCl    liAlF4   PF-     SiH3I
 AlO       BrO             C4H6_1butyne    GaCl2   liBO2    PFCl    SiH4
 AlO+      OBrO            C4H6_2butyne    GaCl3   liBr     PFCl-   SiI
 AlO-      BrOO            C4H6_cyclo      GaF     liCl     PFCl2   SiI2
 AlOCl     BrO3            C4H8_1_butene   GaF2    liF      PFCl4   SiN
 AlOCl2    Br2             C4H8_cis2_buten GaF3    liH      PF2     SiO
 AlOF      BrBrO           C4H8_isobutene  GaH     liI      PF2-    SiO2
 AlOF2     BrOBr           C4H8_cyclo      GaI     liN      PF2Cl   SiS
 AlOF2-    C               C4H9_n_butyl    GaI2    liNO2    PF2Cl3  SiS2
 AlOH      C+              C4H9_i_butyl    GaI3    liNO3    PF3     Si2
 AlOHCl    C-              C4H9_s_butyl    GaO     liO      PF3Cl2  Si2C
 AlOHCl2   CBr             C4H9_t_butyl    GaOH    liOF     PF4Cl   Si2F6
 AlOHF     CBr2            C4H10_n_butane  Ga2Br2  liOH     PF5     Si2N
 AlOHF2    CBr3            C4H10_isobutane Ga2Br4  liON     PH      Si3
 AlO2      CBr4            C4N2            Ga2Br6  li2      PH2     Sn
 AlO2-     CCl             C5              Ga2Cl2  li2+     PH2-    Sn+
 Al_OH_2   CCl2            C5H6_1_3cyclo   Ga2Cl4  li2Br2   PH3     Sn-
 Al_OH_2Cl CCl2Br2         C5H8_cyclo      Ga2Cl6  li2F2    PN      SnBr
 Al_OH_2F  CCl3            C5H10_1_pentene Ga2F2   li2I2    PO      SnBr2
 Al_OH_3   CCl3Br          C5H10_cyclo     Ga2F4   li2O     PO-     SnBr3
 AlS       CCl4            C5H11_pentyl    Ga2F6   li2O+    POCl3   SnBr4
 AlS2      CF              C5H11_t_pentyl  Ga2I2   li2O2    POFCl2  SnCl
 Al2       CF+             C5H12_n_pentane Ga2I4   li2O2H2  POF2Cl  SnCl2
 Al2Br6    CFBr3           C5H12_i_pentane Ga2I6   li2SO4   POF3    SnCl3
 Al2C2     CFCl            CH3C_CH3_2CH3   Ga2O    li3+     PO2     SnCl4
 Al2Cl6    CFClBr2         C6D5_phenyl     Ge      li3Br3   PO2-    SnF
 Al2F6     CFCl2           C6D6            Ge+     li3Cl3   PS      SnF2
 Al2I6     CFCl2Br         C6H2            Ge-     li3F3    P2      SnF3
 Al2O      CFCl3           C6H5_phenyl     GeBr    li3I3    P2O3    SnF4
 Al2O+     CF2             C6H5O_phenoxy   GeBr2   Mg       P2O4    SnI
 Al2O2     CF2+            C6H6            GeBr3   Mg+      P2O5    SnI2
 Al2O2+    CF2Br2          C6H5OH_phenol   GeBr4   MgBr     P3      SnI3
 Al2O3     CF2Cl           C6H10_cyclo     GeCl    MgBr2    P3O6    SnI4
 Al2S      CF2ClBr         C6H12_1_hexene  GeCl2   MgCl     P4      SnO
 Al2S2     CF2Cl2          C6H12_cyclo     GeCl3   MgCl+    P4O6    SnO2
 Ar        CF3             C6H13_n_hexyl   GeCl4   MgCl2    P4O7    SnS
 Ar+       CF3+            C6H14_n_hexane  GeF     MgF      P4O8    SnS2
 B         CF3Br           C7H7_benzyl     GeF2    MgF+     P4O9    Sn2
 B+        CF3Cl           C7H8            GeF3    MgF2     P4O10   Sr
 B-        CF4             C7H8O_cresol_mx GeF4    MgF2+    Pb      Sr+
 BBr       CH+             C7H14_1_heptene GeH4    MgH      Pb+     SrBr
 BBr2      CHBr3           C7H15_n_heptyl  GeI     MgI      Pb-     SrBr2
 BBr3      CHCl            C7H16_n_heptane GeO     MgI2     PbBr    SrCl
 BC        CHClBr2         C7H16_2_methylh GeO2    MgN      PbBr2   SrCl+
 BC2       CHCl2           C8H8_styrene    GeS     MgO      PbBr3   SrCl2
 BCl       CHCl2Br         C8H10_ethylbenz GeS2    MgOH     PbBr4   SrF
 BCl+      CHCl3           C8H16_1_octene  Ge2     MgOH+    PbCl    SrF+
 BClOH     CHF             C8H17_n_octyl   H       Mg_OH_2  PbCl2   SrF2
 BCl_OH_2  CHFBr2          C8H18_n_octane  H+      MgS      PbCl3   SrH
 BCl2      CHFCl           C8H18_isooctane H-      Mg2      PbCl4   SrI
 BCl2+     CHFClBr         C9H19_n_nonyl   HAlO    Mg2F4    PbF     SrI2
 BCl2OH    CHFCl2          C10H8_naphthale HAlO2   Mn       PbF2    SrO
 BF        CHF2            C10H21_n_decyl  HBO     Mn+      PbF3    SrOH
 BFCl      CHF2Br          C12H9_o_bipheny HBO+    Mo       PbF4    SrOH+
 BFCl2     CHF2Cl          C12H10_biphenyl HBO2    Mo+      PbI     Sr_OH_2
 BFOH      CHF3            Ca              HBS     Mo-      PbI2    SrS
 BF_OH_2   CHI3            Ca+             HBS+    MoO      PbI3    Sr2
 BF2       CH2             CaBr            HCN     MoO2     PbI4    Ta
 BF2+      CH2Br2          CaBr2           HCO     MoO3     PbO     Ta+
 BF2-      CH2Cl           CaCl            HCO+    MoO3-    PbO2    Ta-
 BF2Cl     CH2ClBr         CaCl+           HCCN    Mo2O6    PbS     TaCl5
 BF2OH     CH2Cl2          CaCl2           HCCO    Mo3O9    PbS2    TaO
 BF3       CH2F            CaF             HCl     Mo4O12   Rb      TaO2
 BF4-      CH2FBr          CaF+            HD      Mo5O15   Rb+     Ti
 BH        CH2FCl          CaF2            HD+     N        Rb-     Ti+
 BHCl      CH2F2           CaH             HDO     N+       RbBO2   Ti-
 BHCl2     CH2I2           CaI             HDO2    N-       RbBr    TiCl
 BHF       CH3             CaI2            HF      NCO      RbCl    TiCl2
 BHFCl     CH3Br           CaO             HI      ND       RbF     TiCl3
 BHF2      CH3Cl           CaO+            HNC     ND2      RbH     TiCl4
 BH2       CH3F            CaOH            HNCO    ND3      RbI     TiO
 BH2Cl     CH3I            CaOH+           HNO     NF       RbK     TiO+
 BH2F      CH2OH           Ca_OH_2         HNO2    NF2      Rbli    TiOCl
 BH3       CH2OH+          CaS             HNO3    NF3      RbNO2   TiOCl2
 BH3NH3    CH3O            Ca2             HOCl    NH       RbNO3   TiO2
 BH4       CH4             Cd              HOF     NH+      RbNa    U
 BI        CH3OH           Cd+             HO2     NHF      RbO     UF
 BI2       CH3OOH          Cl              HO2-    NHF2     RbOH    UF+
 BI3       CI              Cl+             HPO     NH2      Rb2Br2  UF-
 BN        CI2             Cl-             HSO3F   NH2F     Rb2Cl2  UF2
 BO        CI3             ClCN            H2      NH3      Rb2F2   UF2+
 BO-       CI4             ClF             H2+     NH2OH    Rb2I2   UF2-
 BOCl      CN              ClF3            H2-     NH4+     Rb2O    UF3
 BOCl2     CN+             ClF5            HBOH    NO       Rb2O2   UF3+
 BOF       CN-             ClO             HCOOH   NOCl     Rb2O2H2 UF3-
 BOF2      CNN             ClO2            H2F2    NOF      Rb2SO4  UF4
 BOH       CO              Cl2             H2O     NOF3     Rn      UF4+
 BO2       CO+             Cl2O            H2O+    NO2      Rn+     UF4-
 BO2-      COCl            Co              H2O2    NO2-     S       UF5
 B_OH_2    COCl2           Co+             H2S     NO2Cl    S+      UF5+
 BS        COFCl           Co-             H2SO4   NO2F     S-      UF5-
 BS2       COF2            Cr              H2BOH   NO3      SCl     UF6
 B2        COHCl           Cr+             HB_OH_2 NO3-     SCl2    UF6-
 B2C       COHF            Cr-             H3BO3   NO3F     SCl2+   UO
 B2Cl4     COS             CrN             H3B3O3  N2       SD      UO+
 B2F4      CO2             CrO             H3B3O6  N2+      SF      UOF
 B2H       CO2+            CrO2            H3F3    N2-      SF+     UOF2
 B2H2      COOH            CrO3            H3O+    NCN      SF-     UOF3
 B2H3      CP              CrO3-           H4F4    N2D2_cis SF2     UOF4
 B2H3_db   CS              Cs              H5F5    N2F2     SF2+    UO2
 B2H4      CS2             Cs+             H6F6    N2F4     SF2-    UO2+
 B2H4_db   C2              Cs-             H7F7    N2H2     SF3     UO2-
 B2H5      C2+             CsBO2           He      NH2NO2   SF3+    UO2F
 B2H5_db   C2-             CsBr            He+     N2H4     SF3-    UO2F2
 B2H6      C2Cl            CsCl            Hg      N2O      SF4     UO3
 B2O       C2Cl2           CsF             Hg+     N2O+     SF4+    UO3-
 B2O2      C2Cl3           CsH             HgBr2   N2O3     SF4-    V
 B2O3      C2Cl4           CsI             I       N2O4     SF5     V+
 B2_OH_4   C2Cl6           Csli            I+      N2O5     SF5+    V-
 B2S       C2F             CsNO2           I-      N3       SF5-    VCl4
 B2S2      C2FCl           CsNO3           IF5     N3H      SF6     VN
 B2S3      C2FCl3          CsNa            IF7     Na       SF6-    VO
 B3H7_C2v  C2F2            CsO             I2      Na+      SH      VO2
 B3H7_Cs   C2F2Cl2         CsOH            In      Na-      SH-     V4O10
 B3H9      C2F3            CsRb            In+     NaAlF4   SN      W
 B3N3H6    C2F3Cl          Cs2             InBr    NaBO2    SO      W+
 B3O3Cl3   C2F4            Cs2Br2          InBr2   NaBr     SO-     W-
 B3O3FCl2  C2F6            Cs2CO3          InBr3   NaCN     SOF2    WCl6
 B3O3F2Cl  C2H             Cs2Cl2          InCl    NaCl     SO2     WO
 B3O3F3    C2HCl           Cs2F2           InCl2   NaF      SO2-    WOCl4
 B4H4      C2HCl3          Cs2I2           InCl3   NaH      SO2Cl2  WO2
 B4H10     C2HF            Cs2O            InF     NaI      SO2FCl  WO2Cl2
 B4H12     C2HFCl2         Cs2O+           InF2    Nali     SO2F2   WO3
 B5H9      C2HF2Cl         Cs2O2           InF3    NaNO2    SO3     WO3-
 Ba        C2HF3           Cs2O2H2         InH     NaNO3    S2      Xe
 Ba+       C2H2_vinylidene Cs2SO4          InI     NaO      S2-     Xe+
 BaBr      C2H2Cl2         Cu              InI2    NaOH     S2Cl2   Zn
 BaBr2     C2H2FCl         Cu+             InI3    NaOH+    S2F2    Zn+
 BaCl      C2H2F2          Cu-             InO     Na2      S2O     Zr
 BaCl+     CH2CO_ketene    CuCl            InOH    Na2Br2   S3      Zr+
 BaCl2     O_CH_2O         CuF             In2Br2  Na2Cl2   S4      Zr-
 BaF       HO_CO_2OH       CuF2            In2Br4  Na2F2    S5      ZrN
 BaF+      C2H3_vinyl      CuO             In2Br6  Na2I2    S6      ZrO
 BaF2      CH2Br-COOH      Cu2             In2Cl2  Na2O     S7      ZrO+
 BaH       C2H3Cl          Cu3Cl3          In2Cl4  Na2O+    S8      ZrO2
 BaI       CH2Cl-COOH      D               In2Cl6  Na2O2    Sc
 BaI2      C2H3F           D+              In2F2   Na2O2H2  Sc+
 BaO       CH3CN           D-              In2F4   Na2SO4   Sc-
 BaO+      CH3CO_acetyl    DBr             In2F6   Na3Cl3   ScO
 BaOH      C2H4            DCl             In2I2   Na3F3    ScO+
</pre>

</HTML>"));
          constant IdealGases.Common.DataRecord Ag(name = "Ag", MM = 0.1078682, Hf = 2641186.18832983, H0 = 57453.7073947651, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {33520.0237,6.56281935}, ahigh = { -330992.637,982.008642,1.381179917,0.000617089999, -1.6881146e-07,2.008826848e-11, -5.627285655e-16}, bhigh = {27267.19171,14.56862733}, R = 77.0799178998074);
          constant IdealGases.Common.DataRecord Agplus(name = "Agplus", MM = 0.1078676514, Hf = 9475442.51436256, H0 = 57454.9451996319, Tlimit = 1000, alow = {3691132.75, -43169.83,202.8445385, -0.465841374,0.000558620051, -3.154880975e-07,6.57870206e-11}, blow = {337157.447, -1142.924427}, ahigh = { -53274323.5,131071.0631, -109.8820208,0.0482600276, -1.093661557e-05,1.26383591e-09, -5.852542535e-14}, bhigh = { -743912.251,844.626619}, R = 77.0803099176404);
          constant IdealGases.Common.DataRecord Agminus(name = "Agminus", MM = 0.1078687486, Hf = 1419120.27335784, H0 = 57453.4151961043, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {17665.65919,5.8696798}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = {17665.65919,5.8696798}, R = 77.0795258859617);
          constant IdealGases.Common.DataRecord Air(name = "Air", MM = 0.0289651159, Hf =  -4333.83385840345, H0 = 298609.680343105, Tlimit = 1000, alow = {10099.5016, -196.827561,5.00915511, -0.00576101373,1.06685993e-05, -7.94029797e-09,2.18523191e-12}, blow = { -176.796731, -3.921504225}, ahigh = {241521.443, -1257.8746,5.14455867, -0.000213854179,7.06522784e-08, -1.07148349e-11,6.57780015e-16}, bhigh = {6462.26319, -8.147411905}, R = 287.051224952979);
          constant IdealGases.Common.DataRecord AL(name = "AL", MM = 0.026981538, Hf = 12230585.224608, H0 = 256422.41001977, Tlimit = 1000, alow = {5006.60889,18.61304407,2.412531111,0.0001987604647, -2.432362152e-07,1.538281506e-10, -3.944375734e-14}, blow = {38874.1268,6.086585765}, ahigh = { -29208.20938,116.7751876,2.356906505,7.73723152e-05, -1.529455262e-08, -9.97167026e-13,5.053278264e-16}, bhigh = {38232.8865,6.600920155}, R = 308.154116344294);
          constant IdealGases.Common.DataRecord ALplus(name = "ALplus", MM = 0.0269809894, Hf = 33839201.1673227, H0 = 229696.098542628, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {109064.4788,3.79100578}, ahigh = { -4181.18325, -9.94855727,2.548615878, -5.87876004e-05,3.132291294e-08, -7.74889463e-12,7.27444769e-16}, bhigh = {109101.1485,3.48866729}, R = 308.160381991033);
          constant IdealGases.Common.DataRecord ALminus(name = "ALminus", MM = 0.0269820866, Hf = 10417656.616668, H0 = 250396.238814236, Tlimit = 1000, alow = {29108.01723, -383.698375,4.6551428, -0.00604582184,8.57725964e-06, -5.47626e-09,1.322714061e-12}, blow = {34906.1698, -5.9570977}, ahigh = {633981.432, -2383.438463,5.46997113, -0.001299840355,2.88830547e-07, -3.25324051e-11,1.472436088e-15}, bhigh = {47803.0654, -15.36906816}, R = 308.147850952343);
          constant IdealGases.Common.DataRecord ALBr(name = "ALBr", MM = 0.106885538, Hf = 134022.013342909, H0 = 89545.4631102666, Tlimit = 1000, alow = {8176.15864, -251.6942718,5.40329633, -0.00174721292,2.07730477e-06, -1.261267579e-09,3.16633597e-13}, blow = {1635.024429, -2.323594886}, ahigh = { -610339.56,2010.066834,1.769617961,0.001929888914, -6.64104783e-07,1.172854627e-10, -7.17874276e-15}, bhigh = { -12178.12867,22.04450764}, R = 77.7885591968485);
          constant IdealGases.Common.DataRecord ALBr2(name = "ALBr2", MM = 0.186789538, Hf =  -753051.410191935, H0 = 71727.1167510463, Tlimit = 1000, alow = {31993.7587, -711.917897,9.47825811, -0.00487553167,5.51651299e-06, -3.34005304e-09,8.36847684e-13}, blow = { -15405.91306, -17.42171366}, ahigh = { -352378.29,467.154417,7.11190819, -0.00055517092,3.16630113e-07, -5.52102833e-11,3.17672595e-15}, bhigh = { -22650.04078, -2.69561036}, R = 44.5125144000303);
          constant IdealGases.Common.DataRecord ALBr3(name = "ALBr3", MM = 0.266693538, Hf =  -1539132.39547634, H0 = 67280.7302890106, Tlimit = 1000, alow = {47189.4884, -1053.853163,13.50747168, -0.00663986993,7.26927683e-06, -4.27804949e-09,1.045573281e-12}, blow = { -46994.4033, -36.6793678}, ahigh = { -89355.2744, -26.78035134,10.02063997, -8.4587644e-06,1.904243317e-09, -2.215195785e-13,1.038219389e-17}, bhigh = { -52492.4764, -15.79666403}, R = 31.1761284594755);
          constant IdealGases.Common.DataRecord ALC(name = "ALC", MM = 0.038992238, Hf = 17497931.4857485, H0 = 232305.439867289, Tlimit = 1000, alow = {41399.2188, -585.867789,5.96244572, -0.002006064322,1.665566136e-06, -7.12164921e-10,1.271188744e-13}, blow = {83834.3611, -8.00216505}, ahigh = {1937001.487, -6749.11779,13.47525643, -0.00585080265,1.926461091e-06, -2.585922351e-10,1.222659806e-14}, bhigh = {122536.3649, -61.52244872}, R = 213.234028782857);
          constant IdealGases.Common.DataRecord ALC2(name = "ALC2", MM = 0.051002938, Hf = 13246605.048517, H0 = 240432.737423872, Tlimit = 1000, alow = {15240.87465, -264.9987059,6.2804149, -0.0001786037647,3.79190774e-06, -3.99945007e-09,1.299752388e-12}, blow = {80927.6328, -6.247291862}, ahigh = {146601.7895, -1362.764489,8.45775915, -0.000367890168,7.9031745e-08, -8.88002372e-12,4.05363395e-16}, bhigh = {87059.2423, -21.27956728}, R = 163.019471544953);
          constant IdealGases.Common.DataRecord ALCL(name = "ALCL", MM = 0.062434538, Hf =  -816974.780849664, H0 = 149326.195062099, Tlimit = 1000, alow = {23808.81392, -445.715942,5.99285399, -0.002777300261,3.101582418e-06, -1.815268376e-09,4.41466234e-13}, blow = { -5202.73603, -7.383298016}, ahigh = { -899346.122,2765.942061,1.003500696,0.002267806833, -7.2805416e-07,1.182380673e-10, -6.66049813e-15}, bhigh = { -24961.64448,26.1692548}, R = 133.171034275932);
          constant IdealGases.Common.DataRecord ALCLplus(name = "ALCLplus", MM = 0.0624339894, Hf = 13804168.4550755, H0 = 146625.821735492, Tlimit = 1000, alow = {34699.5522, -547.990003,6.10601113, -0.002674711519,2.696426573e-06, -1.463885068e-09,3.35872906e-13}, blow = {105171.951, -7.816586372}, ahigh = { -754724.082,1251.545253,4.44242491, -0.000978611643,6.34751133e-07, -1.158320344e-10,6.87002246e-15}, bhigh = {93183.327,4.047521108}, R = 133.172204433888);
          constant IdealGases.Common.DataRecord ALCL2(name = "ALCL2", MM = 0.097887538, Hf =  -2460724.38761306, H0 = 131241.006388372, Tlimit = 1000, alow = {53405.4595, -967.805798,10.06252671, -0.00553952845,5.82342056e-06, -3.30654245e-09,7.8315621e-13}, blow = { -26076.2711, -23.93364216}, ahigh = {430345.306, -1552.370585,8.76065742, -0.000946745006,2.40184488e-07, -2.427836709e-11,8.39012347e-16}, bhigh = { -21458.84709, -18.03641668}, R = 84.9390246182308);
          constant IdealGases.Common.DataRecord ALCL3(name = "ALCL3", MM = 0.133340538, Hf =  -4384854.5368851, H0 = 122999.376228706, Tlimit = 1000, alow = {77506.0097, -1440.779717,14.01744141, -0.00638163124,5.87167472e-06, -2.908872278e-09,5.99405089e-13}, blow = { -65793.4318, -44.94017799}, ahigh = { -137863.0916, -55.7920729,10.04190387, -1.682165339e-05,3.72466466e-09, -4.27552678e-13,1.982341329e-17}, bhigh = { -73434.0747, -20.45130429}, R = 62.3551706383546);
          constant IdealGases.Common.DataRecord ALF(name = "ALF", MM = 0.0459799412, Hf =  -5742948.75348818, H0 = 193391.004162485, Tlimit = 1000, alow = {30207.71686, -308.0986949,3.88641314,0.00356434369, -5.85847128e-06,4.38810992e-09, -1.256906273e-12}, blow = { -31175.72862,2.032567172}, ahigh = { -711122.905,1903.013316,2.191607179,0.001421471467, -4.17963477e-07,6.1538044e-11, -2.945403999e-15}, bhigh = { -45407.2896,16.13163743}, R = 180.828243425418);
          constant IdealGases.Common.DataRecord ALFplus(name = "ALFplus", MM = 0.0459793926, Hf = 15055300.2520525, H0 = 191666.016049112, Tlimit = 1000, alow = {36430.3812, -252.4634478,3.105310486,0.00573327043, -8.83978699e-06,6.34357959e-09, -1.645011685e-12}, blow = {83702.3745,6.76401558}, ahigh = {1505101.126, -2236.668821,3.098216724,0.00311541393, -1.168427068e-06,1.773587168e-10, -9.66584138e-15}, bhigh = {98850.8385,5.99189459}, R = 180.830400965323);
          constant IdealGases.Common.DataRecord ALFCL(name = "ALFCL", MM = 0.0814329412, Hf =  -5359132.39248198, H0 = 148996.153414142, Tlimit = 1000, alow = {41047.7948, -576.964642,6.5626427,0.00410628584, -7.21392112e-06,5.48640145e-09, -1.580656193e-12}, blow = { -51148.287, -6.041227053}, ahigh = {69706.3635, -622.448591,7.57347541, -0.0002236155999,2.382317949e-08,4.44378576e-12, -5.48697785e-16}, bhigh = { -51009.432, -10.91164505}, R = 102.102071685948);
          constant IdealGases.Common.DataRecord ALFCL2(name = "ALFCL2", MM = 0.1168859412, Hf =  -6770663.74172294, H0 = 134646.304238341, Tlimit = 1000, alow = {69556.0734, -1188.857902,11.5667259,0.000457495949, -3.40737283e-06,3.35682254e-09, -1.085329228e-12}, blow = { -91620.4926, -32.06065081}, ahigh = { -172036.0526, -119.3340759,10.08894431, -3.55099381e-05,7.83095261e-09, -8.96173423e-13,4.14533026e-17}, bhigh = { -98041.8547, -21.27046859}, R = 71.1332082767196);
          constant IdealGases.Common.DataRecord ALF2(name = "ALF2", MM = 0.0649783444, Hf =  -9722688.08683282, H0 = 178535.358312392, Tlimit = 1000, alow = {29949.51991, -219.4199915,3.41674876,0.01272953936, -1.883312206e-05,1.330702256e-08, -3.68011863e-12}, blow = { -76075.3765,8.759626916}, ahigh = { -214689.6245,30.9684015,6.81639962,0.0001820129379, -7.79367744e-08,1.472534201e-11, -8.8036953e-16}, bhigh = { -78840.0268, -7.915370474}, R = 127.957584588751);
          constant IdealGases.Common.DataRecord ALF2minus(name = "ALF2minus", MM = 0.064978893, Hf =  -13130889.826024, H0 = 174392.244570864, Tlimit = 1000, alow = {123336.9037, -1348.195504,8.61117805,0.0002245141297, -2.424673201e-06,2.21413243e-09, -6.60271396e-13}, blow = { -97084.3746, -21.92617549}, ahigh = { -159007.6965, -206.0486516,7.15528938, -6.26016769e-05,1.391935746e-08, -1.603901265e-12,7.46150852e-17}, bhigh = { -104047.9429, -11.22696648}, R = 127.956504275935);
          constant IdealGases.Common.DataRecord ALF2CL(name = "ALF2CL", MM = 0.1004313444, Hf =  -9948369.71434587, H0 = 147638.340286919, Tlimit = 1000, alow = {56774.1209, -886.982947,8.75274655,0.0081957912, -1.380480934e-05,1.032703706e-08, -2.948968416e-12}, blow = { -117793.6778, -18.5834736}, ahigh = { -211542.2988, -195.9188681,10.1457685, -5.81321935e-05,1.281071982e-08, -1.465366277e-12,6.77600245e-17}, bhigh = { -122715.311, -23.59853398}, R = 82.7876202362178);
          constant IdealGases.Common.DataRecord ALF3(name = "ALF3", MM = 0.0839767476, Hf =  -14400140.1883299, H0 = 167233.459277244, Tlimit = 1000, alow = {44102.6352, -566.749574,5.83307285,0.01603535069, -2.413263602e-05,1.71410616e-08, -4.7475432e-12}, blow = { -144334.9991, -5.461613658}, ahigh = { -249397.216, -291.1202519,10.21662932, -8.64298597e-05,1.905713108e-08, -2.181058411e-12,1.009055287e-16}, bhigh = { -147569.4276, -27.03983244}, R = 99.0092166894065);
          constant IdealGases.Common.DataRecord ALF4minus(name = "ALF4minus", MM = 0.1029756994, Hf =  -18952051.9634363, H0 = 163017.440986665, Tlimit = 1000, alow = {231201.0037, -3235.37678,20.17865547, -0.0091081935,6.67955263e-06, -2.606075106e-09,4.11006051e-13}, blow = { -221178.4563, -86.77268282}, ahigh = { -321741.161, -261.2097043,13.19792654, -8.01013541e-05,1.786204421e-08, -2.062751964e-12,9.61258177e-17}, bhigh = { -238157.7915, -42.31634322}, R = 80.742078455842);
          constant IdealGases.Common.DataRecord ALH(name = "ALH", MM = 0.027989478, Hf = 8905160.86080634, H0 = 309691.520506385, Tlimit = 1000, alow = { -37591.1403,508.900223,1.128086896,0.00398866091, -2.150790303e-07, -2.176790819e-09,1.020805902e-12}, blow = {26444.31827,16.50021856}, ahigh = {6802018.43, -21784.16933,30.32713047, -0.01503343597,4.49214236e-06, -6.17845037e-10,3.11520526e-14}, bhigh = {165830.1221, -187.6766425}, R = 297.057058370292);
          constant IdealGases.Common.DataRecord ALHCL(name = "ALHCL", MM = 0.063442478, Hf = 165850.489005174, H0 = 171597.521773976, Tlimit = 1000, alow = {35871.6976, -591.624999,6.94454238, -0.002511543148,6.47133255e-06, -5.64227714e-09,1.715950436e-12}, blow = {2750.930479, -9.90374588}, ahigh = { -168391.533, -886.230179,8.0623794, -0.000920055147,3.94981722e-07, -6.40108676e-11,3.5783541e-15}, bhigh = {3641.50485, -18.00686894}, R = 131.055284442074);
          constant IdealGases.Common.DataRecord ALHCL2(name = "ALHCL2", MM = 0.098895478, Hf =  -3552022.52018035, H0 = 138289.39681145, Tlimit = 1000, alow = {118482.8615, -2054.157118,15.74696513, -0.01476257296,2.065336978e-05, -1.42019885e-08,3.82627986e-12}, blow = { -34342.7746, -57.61184061}, ahigh = {96080.6394, -1485.540161,11.03592709, -0.00039569552,8.46584373e-08, -9.48333652e-12,4.31903353e-16}, bhigh = { -36692.1639, -32.36862171}, R = 84.0733284084031);
          constant IdealGases.Common.DataRecord ALHF(name = "ALHF", MM = 0.0469878812, Hf =  -3886416.65332209, H0 = 224996.759377182, Tlimit = 1000, alow = { -9289.5451,133.5515868,2.77244868,0.00675141503, -4.13384614e-06,5.86570407e-10,2.276158011e-13}, blow = { -23846.85845,12.28752581}, ahigh = {734216.925, -3462.16413,10.39714588, -0.001750737965,4.61568786e-07, -5.39041474e-11,2.340478515e-15}, bhigh = { -2965.279722, -37.13172313}, R = 176.949285382972);
          constant IdealGases.Common.DataRecord ALHFCL(name = "ALHFCL", MM = 0.0824408812, Hf =  -6735062.94350478, H0 = 152019.67540347, Tlimit = 1000, alow = {129037.3237, -2142.099873,14.8543442, -0.0114032693,1.564521768e-05, -1.067574855e-08,2.857255247e-12}, blow = { -58183.0307, -54.83424986}, ahigh = {40689.8713, -1502.129854,11.05214126, -0.000403158773,8.64525222e-08, -9.70065952e-12,4.42362951e-16}, bhigh = { -61298.8811, -34.03564016}, R = 100.853749729206);
          constant IdealGases.Common.DataRecord ALHF2(name = "ALHF2", MM = 0.0659862844, Hf =  -11597852.3258085, H0 = 186401.994169564, Tlimit = 1000, alow = {105018.1431, -1398.654202,9.16633484,0.00386728687, -4.83084109e-06,3.099287181e-09, -8.44070299e-13}, blow = { -86590.488, -25.57299683}, ahigh = { -4419.9473, -1612.826324,11.13336052, -0.000435181266,9.34439616e-08, -1.049431425e-11,4.78833398e-16}, bhigh = { -86069.1619, -36.76383785}, R = 126.003033442507);
          constant IdealGases.Common.DataRecord ALH2(name = "ALH2", MM = 0.028997418, Hf = 9544813.05887304, H0 = 347990.776282219, Tlimit = 1000, alow = {14551.82996, -215.3768996,5.14437023, -0.00396522203,1.340900203e-05, -1.216744854e-08,3.74310713e-12}, blow = {33110.3794, -3.608799711}, ahigh = {143291.0601, -2365.907684,9.08566719, -0.001308536612,4.77719136e-07, -7.32472189e-11,3.99790094e-15}, bhigh = {44859.5185, -32.20814285}, R = 286.731460021716);
          constant IdealGases.Common.DataRecord ALH2CL(name = "ALH2CL", MM = 0.064450418, Hf =  -1650034.38767457, H0 = 174603.227553932, Tlimit = 1000, alow = {93038.2898, -1328.188016,9.42476359, -0.00327766059,1.004231609e-05, -8.94812309e-09,2.708031665e-12}, blow = { -7647.43793, -26.82512215}, ahigh = {316692.993, -3094.037914,12.1587202, -0.000824826959,1.765027123e-07, -1.977372723e-11,9.00617628e-16}, bhigh = {2411.845642, -47.18390855}, R = 129.0057110258);
          constant IdealGases.Common.DataRecord ALH2F(name = "ALH2F", MM = 0.0479958212, Hf =  -6597571.33189754, H0 = 224286.213483936, Tlimit = 1000, alow = {89060.3929, -975.080393,5.87077232,0.00689666369, -4.04581728e-06,7.21931866e-10,7.32984674e-14}, blow = { -34253.026, -9.387539141}, ahigh = {271151.4962, -3169.62348,12.21401118, -0.000846560026,1.812341995e-07, -2.030947985e-11,9.25182582e-16}, bhigh = { -22592.17746, -49.41337333}, R = 173.233248064521);
          constant IdealGases.Common.DataRecord ALH3(name = "ALH3", MM = 0.030005358, Hf = 4295768.77569666, H0 = 346957.366747632, Tlimit = 1000, alow = {14812.07909, -28.3649534,2.507597126,0.00731592051,2.331766687e-06, -6.38920989e-09,2.456885069e-12}, blow = {14631.89428,8.3131877}, ahigh = {588545.855, -4595.78566,13.20893344, -0.001226849004,2.626580824e-07, -2.943725869e-11,1.341177648e-15}, bhigh = {39917.1695, -61.81829295}, R = 277.099576682271);
          constant IdealGases.Common.DataRecord ALI(name = "ALI", MM = 0.153886008, Hf = 437954.157599566, H0 = 63365.8454510042, Tlimit = 1000, alow = {1870.854131, -154.3184003,5.07439188, -0.00112058483,1.386302156e-06, -8.54078228e-10,2.180358492e-13}, blow = {7517.44369,0.6688267622}, ahigh = {475281.792, -1198.028588,5.43368536, -0.0001004479306, -1.005928846e-07,4.56817319e-11, -3.96020136e-15}, bhigh = {14646.27816, -3.141199746}, R = 54.0300714019432);
          constant IdealGases.Common.DataRecord ALI2(name = "ALI2", MM = 0.280790478, Hf =  -120420.586341963, H0 = 49585.1785971175, Tlimit = 1000, alow = {14234.13595, -464.657569,8.72267491, -0.00356051929,4.18868229e-06, -2.616026962e-09,6.72019551e-13}, blow = { -3846.13333, -10.73076578}, ahigh = { -335599.902,474.334794,7.10650139, -0.000552996534,3.161480314e-07, -5.51548916e-11,3.17415582e-15}, bhigh = { -9785.48885, -0.5602275944}, R = 29.6109471347529);
          constant IdealGases.Common.DataRecord ALI3(name = "ALI3", MM = 0.407694948, Hf =  -469298.02279522, H0 = 46612.220958892, Tlimit = 1000, alow = {27060.32439, -755.993681,12.71396349, -0.00546688409,6.29761892e-06, -3.86420961e-09,9.77413244e-13}, blow = { -22209.66921, -28.47883861}, ahigh = { -63574.2747, -15.7877228,10.01249733, -5.22867312e-06,1.19617352e-09, -1.40919449e-13,6.67127639e-18}, bhigh = { -26114.55525, -12.47999813}, R = 20.3938558493003);
          constant IdealGases.Common.DataRecord ALN(name = "ALN", MM = 0.040988238, Hf = 10706217.9886825, H0 = 226043.163894969, Tlimit = 1000, alow = {27166.45079, -254.349111,3.53898772,0.00529625616, -9.84570061e-06,8.45220904e-09, -2.592304568e-12}, blow = {53099.9491,5.39943522}, ahigh = {3821214.22, -10677.76595,14.49174222, -0.00372722713,7.92277096e-07, -8.08689221e-11,2.893539736e-15}, bhigh = {120523.634, -72.88585119}, R = 202.850193267639);
          constant IdealGases.Common.DataRecord ALO(name = "ALO", MM = 0.042980938, Hf = 1566252.6024909, H0 = 204465.198037325, Tlimit = 1000, alow = { -7683.3911,295.7969549,0.480810844,0.01169224855, -1.595428871e-05,1.060766814e-08, -2.647888708e-12}, blow = {5843.67217,21.60997839}, ahigh = {15657.21161,3855.74101, -5.92607978,0.00905096042, -2.930661549e-06,4.23852907e-10, -2.280655341e-14}, bhigh = { -13316.94655,68.30663436}, R = 193.445568824022);
          constant IdealGases.Common.DataRecord ALOplus(name = "ALOplus", MM = 0.0429803894, Hf = 23103397.0111029, H0 = 211485.194222089, Tlimit = 1000, alow = {28291.78513, -417.90871,5.31329987, -0.001410883046,2.526524021e-06, -2.005474816e-09,5.76255355e-13}, blow = {120364.8109, -3.41056593}, ahigh = {27108.46446, -699.865342,5.78781584, -0.000687189546,2.075047303e-07, -2.655281004e-11,1.28312966e-15}, bhigh = {121868.2226, -7.05103588}, R = 193.448037955654);
          constant IdealGases.Common.DataRecord ALOminus(name = "ALOminus", MM = 0.0429814866, Hf =  -6349752.03486796, H0 = 203453.130446167, Tlimit = 1000, alow = {20780.75267,2.859351226,1.90191237,0.00730104566, -9.13976987e-06,5.65298319e-09, -1.396787103e-12}, blow = { -33592.6773,12.96339858}, ahigh = { -69719.4829, -232.0553036,4.67278433, -4.26644704e-05,1.632506336e-08, -1.727134051e-12,8.13684064e-17}, bhigh = { -33077.1893, -2.155922595}, R = 193.443099755419);
          constant IdealGases.Common.DataRecord ALOCL(name = "ALOCL", MM = 0.078433938, Hf =  -3844822.50527826, H0 = 151797.006545814, Tlimit = 1000, alow = { -5144.62779, -58.8629707,4.45231693,0.00928823509, -1.246894823e-05,8.17807354e-09, -2.130739934e-12}, blow = { -37596.8365,2.476862818}, ahigh = { -125802.7706, -242.0565961,7.68113002, -7.2687417e-05,1.611337534e-08,1.852889749e-12,8.60727822e-17}, bhigh = { -37543.2605, -14.7581832}, R = 106.006050595088);
          constant IdealGases.Common.DataRecord ALOCL2(name = "ALOCL2", MM = 0.113886938, Hf =  -3532526.87327497, H0 = 138316.950799046, Tlimit = 1000, alow = {69026.3713, -1189.2975,11.61842739,0.0002818279805, -3.145010615e-06,3.16838065e-09, -1.032375462e-12}, blow = { -44833.2307, -31.65475132}, ahigh = { -170771.3922, -118.0013804,10.08798517, -3.51385804e-05,7.75118703e-09, -8.8724706e-13,4.10481544e-17}, bhigh = { -51249.4414, -20.5925296}, R = 73.0063705813216);
          constant IdealGases.Common.DataRecord ALOF(name = "ALOF", MM = 0.0619793412, Hf =  -9233554.14755522, H0 = 177526.943445472, Tlimit = 1000, alow = {7030.57836, -115.457543,3.37882359,0.01283929158, -1.749748388e-05,1.162605473e-08, -3.064218613e-12}, blow = { -69593.817,5.994804586}, ahigh = { -164102.0297, -327.777029,7.74451974, -9.78864944e-05,2.165743208e-08, -2.486541919e-12,1.153633944e-16}, bhigh = { -69740.8509, -17.17253748}, R = 134.149086437853);
          constant IdealGases.Common.DataRecord ALOF2(name = "ALOF2", MM = 0.0809777444, Hf =  -9553856.76314293, H0 = 173570.986746329, Tlimit = 1000, alow = {47560.8529, -612.431836,6.05022366,0.01559291479, -2.366136797e-05,1.688366194e-08, -4.6905831e-12}, blow = { -91718.2749, -4.956258224}, ahigh = { -249668.1413, -283.7821503,10.21108125, -8.41830343e-05,1.855520828e-08, -2.122974022e-12,9.8193191e-17}, bhigh = { -95218.0419, -25.22534812}, R = 102.676013781387);
          constant IdealGases.Common.DataRecord ALOF2minus(name = "ALOF2minus", MM = 0.080978293, Hf =  -12006793.2155596, H0 = 173836.474918038, Tlimit = 1000, alow = {154060.5112, -1928.054597,12.55806423, -0.000490403537, -2.341783347e-06,2.4151386e-09, -7.47180925e-13}, blow = { -109143.0174, -42.6582175}, ahigh = { -238988.1459, -288.0143725,10.2170024, -8.747062e-05,1.944846334e-08, -2.241045051e-12,1.042591053e-16}, bhigh = { -119057.6566, -26.00779629}, R = 102.675318186813);
          constant IdealGases.Common.DataRecord ALOH(name = "ALOH", MM = 0.043988878, Hf =  -4382066.09861702, H0 = 235330.644259669, Tlimit = 1000, alow = {58764.9318, -944.942269,7.82059918,0.000585888847, -4.08366681e-06,4.58722934e-09, -1.563936726e-12}, blow = { -19932.83011, -20.65043885}, ahigh = {788206.811, -2263.671626,7.82395488,0.0001821171456, -8.26372932e-08,1.265414876e-11, -6.87597253e-16}, bhigh = { -10398.08093, -22.09032458}, R = 189.013050071429);
          constant IdealGases.Common.DataRecord ALOHCL(name = "ALOHCL", MM = 0.079441878, Hf =  -4705147.32796221, H0 = 175208.823739036, Tlimit = 1000, alow = {16363.60341, -191.5959416,4.71117047,0.01388068475, -1.999018454e-05,1.43964735e-08, -4.03333045e-12}, blow = { -45680.8372,4.644207506}, ahigh = {796711.551, -2843.470502,10.9555997, -0.0001099328162, -1.344506283e-08,4.41589377e-12, -2.973546112e-16}, bhigh = { -29698.61461, -32.96113996}, R = 104.661070575396);
          constant IdealGases.Common.DataRecord ALOHCL2(name = "ALOHCL2", MM = 0.114894878, Hf =  -6311381.50475254, H0 = 149265.191786878, Tlimit = 1000, alow = {39719.2986, -775.155496,9.49405153,0.01074401359, -1.676464791e-05,1.259706368e-08, -3.61317798e-12}, blow = { -85847.9693, -19.93481642}, ahigh = {738592.976, -2863.204228,13.97062327, -0.0001160311527, -1.208248796e-08,4.25832763e-12, -2.900049181e-16}, bhigh = { -72930.48, -46.81834857}, R = 72.3659065115157);
          constant IdealGases.Common.DataRecord ALOHF(name = "ALOHF", MM = 0.0629872812, Hf =  -9116309.89717969, H0 = 211393.169324476, Tlimit = 1000, alow = { -1556.003336,288.6581999,0.770379329,0.02450406329, -3.41781101e-05,2.388932902e-08, -6.57048051e-12}, blow = { -71772.2256,24.53071801}, ahigh = {755711.313, -2936.702803,11.02455104, -0.000137307803, -7.43281616e-09,3.72998122e-12, -2.657027781e-16}, bhigh = { -53401.309, -35.27397302}, R = 132.002395429635);
          constant IdealGases.Common.DataRecord ALOHF2(name = "ALOHF2", MM = 0.0819856844, Hf =  -13923293.0645634, H0 = 188185.268110051, Tlimit = 1000, alow = {17382.63538, -130.7687299,3.31238192,0.02784734099, -3.98236321e-05,2.809344456e-08, -7.7645742e-12}, blow = { -138413.3568,10.04739945}, ahigh = {653011.992, -3034.017232,14.09719788, -0.0001663681799, -1.011564183e-09,2.993805003e-12, -2.315956955e-16}, bhigh = { -122305.2213, -51.5121069}, R = 101.413704854064);
          constant IdealGases.Common.DataRecord ALO2(name = "ALO2", MM = 0.058980338, Hf =  -655436.579559785, H0 = 226543.784811813, Tlimit = 1000, alow = {43384.8045, -473.529226,6.00171767,0.00709442088, -1.129107996e-05,8.25269168e-09, -2.327652976e-12}, blow = { -3826.1458, -4.83002248}, ahigh = {118721.6642, -833.56254,8.30930119, -0.000353866722,5.96706946e-08,4.0148977e-14, -3.51570252e-16}, bhigh = { -2033.107586, -17.15063884}, R = 140.970233164822);
          constant IdealGases.Common.DataRecord ALO2minus(name = "ALO2minus", MM = 0.0589808866, Hf =  -7673201.42318783, H0 = 180476.720741597, Tlimit = 1000, alow = {117867.8641, -1507.186304,9.52474975, -0.000520798902, -1.586902345e-06,1.700455028e-09, -5.30141977e-13}, blow = { -48254.6927, -30.81215859}, ahigh = { -187241.0758, -233.8853263,7.67607002, -7.0935703e-05,1.576717569e-08, -1.816494162e-12,8.44970796e-17}, bhigh = { -55946.4056, -17.73751567}, R = 140.968921955812);
          constant IdealGases.Common.DataRecord AL_OH_2(name = "AL_OH_2", MM = 0.060996218, Hf =  -8322822.08054276, H0 = 229973.848542544, Tlimit = 1000, alow = {4397.31691,132.7680339,1.093947024,0.03054059173, -4.37594335e-05,3.17461743e-08, -9.0042594e-12}, blow = { -63154.3739,21.01977655}, ahigh = {1669643.735, -5924.73828,15.49480264, -0.000538348808,5.41304453e-08, -1.196022328e-12, -1.082001733e-16}, bhigh = { -26975.25061, -66.17911543}, R = 136.311270970931);
          constant IdealGases.Common.DataRecord AL_OH_2CL(name = "AL_OH_2CL", MM = 0.096449218, Hf =  -8906829.33271683, H0 = 178560.670134205, Tlimit = 1000, alow = {24778.79709, -445.457721,5.9284702,0.02730162734, -4.04140537e-05,2.97421041e-08, -8.4771506e-12}, blow = { -103377.803, -3.679940779}, ahigh = {1603911.487, -5767.23224,18.17772266, -0.000369181615,1.137918702e-08,4.0830794e-12, -3.64132343e-16}, bhigh = { -71186.9985, -77.57541774}, R = 86.2056963489325);
          constant IdealGases.Common.DataRecord AL_OH_2F(name = "AL_OH_2F", MM = 0.0799946212, Hf =  -13371257.8665227, H0 = 205914.482160208, Tlimit = 1000, alow = {21328.12662, -193.2900224,3.069174664,0.0355501733, -5.18115833e-05,3.75352028e-08, -1.059220701e-11}, blow = { -129579.7992,10.01897011}, ahigh = {1559238.236, -5841.27303,18.23229919, -0.000390780542,1.610928938e-08,3.54478406e-12, -3.39345388e-16}, bhigh = { -96231.0089, -79.71351802}, R = 103.937888263917);
          constant IdealGases.Common.DataRecord AL_OH_3(name = "AL_OH_3", MM = 0.078003558, Hf =  -12982325.0113796, H0 = 225563.646724935, Tlimit = 1000, alow = { -14024.75452,369.607346, -0.297960065,0.0492985935, -6.98157228e-05,5.0148017e-08, -1.412480165e-11}, blow = { -125526.076,27.11490074}, ahigh = {2477063.616, -8968.61725,22.80320998, -0.000832029225,8.66349443e-08, -2.420287082e-12, -1.331746442e-16}, bhigh = { -70152.7864, -112.1899198}, R = 106.590932685404);
          constant IdealGases.Common.DataRecord ALS(name = "ALS", MM = 0.059046538, Hf = 3940654.55624172, H0 = 153914.341260787, Tlimit = 1000, alow = {26117.42801, -347.067389,4.43620459,0.00334016821, -8.41019967e-06,8.50818511e-09, -2.780868507e-12}, blow = {28637.79062,0.7522705491}, ahigh = {8909844.29, -25076.77454,28.93456134, -0.00983976682,2.045319809e-06, -2.08797089e-10,8.15279252e-15}, bhigh = {188576.1615, -178.6036457}, R = 140.81218445017);
          constant IdealGases.Common.DataRecord ALS2(name = "ALS2", MM = 0.091111538, Hf = 2727813.45212283, H0 = 158363.90556814, Tlimit = 1000, alow = {42523.3541, -776.46033,9.80106896, -0.00391068009,3.87569597e-06, -2.082025593e-09,4.68293589e-13}, blow = {31679.6154, -21.78097896}, ahigh = { -68525.6354, -26.71667739,7.52019386, -8.1485651e-06,1.811865152e-09, -2.086958125e-13,9.70324638e-18}, bhigh = {27583.62737, -7.873490263}, R = 91.2559724323828);
          constant IdealGases.Common.DataRecord AL2(name = "AL2", MM = 0.053963076, Hf = 9289717.32449055, H0 = 187889.993520755, Tlimit = 1000, alow = { -5281.50965, -17.27374523,4.60407701, -0.000261646777,6.30231997e-07, -3.29093859e-10,8.88836514e-14}, blow = {59007.0639,3.060188921}, ahigh = { -2320724.102,9218.70789, -9.44695187,0.00999992001, -3.154798085e-06,4.36154481e-10, -2.24115724e-14}, bhigh = {2904.589544,99.60320745}, R = 154.077058172147);
          constant IdealGases.Common.DataRecord AL2Br6(name = "AL2Br6", MM = 0.533387076, Hf =  -1766864.72808351, H0 = 71650.0093826795, Tlimit = 1000, alow = {64833.197, -1947.687811,28.86989822, -0.01365825785,1.557883731e-05, -9.486942e-09,2.385519396e-12}, blow = { -110152.6833, -102.5934782}, ahigh = { -173182.949, -43.4229978,22.0341421, -1.421187503e-05,3.23863134e-09, -3.80387976e-13,1.796525455e-17}, bhigh = { -120233.8355, -62.0105495}, R = 15.5880642297377);
          constant IdealGases.Common.DataRecord AL2C2(name = "AL2C2", MM = 0.077984476, Hf = 6988288.28445292, H0 = 207284.857565755, Tlimit = 1000, alow = {11109.87147, -350.243423,8.598856,0.001832528671,1.035743392e-06, -2.09164989e-09,7.7160687e-13}, blow = {64927.6419, -16.45080239}, ahigh = {159765.1967, -1644.007203,11.64643675, -0.000437921543,9.36964011e-08, -1.049615854e-11,4.78046893e-16}, bhigh = {72048.8981, -36.48164252}, R = 106.617014391428);
          constant IdealGases.Common.DataRecord AL2CL6(name = "AL2CL6", MM = 0.266681076, Hf =  -4863023.13404495, H0 = 128044.668606332, Tlimit = 1000, alow = {134093.5279, -3001.037953,31.43000879, -0.01700408496,1.786540089e-05, -1.015419943e-08,2.409630451e-12}, blow = { -147183.0046, -127.4445324}, ahigh = { -275297.5554, -92.4823597,22.07058224, -2.870537915e-05,6.42332569e-09, -7.4365852e-13,3.47206467e-17}, bhigh = { -162915.7014, -70.91602129}, R = 31.1775853191773);
          constant IdealGases.Common.DataRecord AL2F6(name = "AL2F6", MM = 0.1679534952, Hf =  -15673929.4282933, H0 = 155186.344701923, Tlimit = 1000, alow = {191506.1289, -3168.24025,22.45374802,0.01271532036, -2.488697302e-05,1.955252096e-08, -5.71208489e-12}, blow = { -304996.255, -93.96473101}, ahigh = { -585298.261, -545.057199,22.4068632, -0.0001627323704,3.59522504e-08, -4.12120144e-12,1.909095384e-16}, bhigh = { -321968.391, -85.89867941}, R = 49.5046083447033);
          constant IdealGases.Common.DataRecord AL2I6(name = "AL2I6", MM = 0.815389896, Hf =  -598176.452017257, H0 = 50185.7653629792, Tlimit = 1000, alow = {25875.71514, -1280.719742,26.7771582, -0.00991301847,1.168760413e-05, -7.30391853e-09,1.874532749e-12}, blow = { -58911.0555, -83.30209321}, ahigh = { -121011.2399, -24.67542774,22.0199265, -8.46202259e-06,1.957831901e-09, -2.326573151e-13,1.108910373e-17}, bhigh = { -65485.2375, -55.28536481}, R = 10.1969279246502);
          constant IdealGases.Common.DataRecord AL2O(name = "AL2O", MM = 0.069962476, Hf =  -2124157.04098294, H0 = 182619.523071196, Tlimit = 1000, alow = {7776.5307, -129.4235361,4.91250952,0.00860422345, -1.217703648e-05,8.31463487e-09, -2.237722201e-12}, blow = { -18865.12879, -0.02806368311}, ahigh = { -117107.4351, -178.3009166,7.63321536, -5.33593177e-05,1.180702791e-08, -1.355444579e-12,6.28732389e-17}, bhigh = { -19475.80149, -14.15764167}, R = 118.841877465858);
          constant IdealGases.Common.DataRecord AL2Oplus(name = "AL2Oplus", MM = 0.0699619274, Hf = 9276048.7327569, H0 = 185545.888777215, Tlimit = 1000, alow = {68289.2572, -909.850417,8.89656301, -0.000777255438, -4.03465562e-07,6.97731505e-10, -2.417262484e-13}, blow = {80850.0755, -21.76216731}, ahigh = { -110222.5105, -121.4571732,7.59159595, -3.69453916e-05,8.21852032e-09, -9.47364972e-13,4.40862135e-17}, bhigh = {76149.4358, -12.82233856}, R = 118.842809353477);
          constant IdealGases.Common.DataRecord AL2O2(name = "AL2O2", MM = 0.085961876, Hf =  -4689236.73792322, H0 = 184298.130022197, Tlimit = 1000, alow = { -19405.60042,250.8489836,3.62140379,0.01951385302, -2.560329071e-05,1.662721576e-08, -4.3123962e-12}, blow = { -51726.9728,9.923995945}, ahigh = { -194061.1656, -460.975243,10.84375637, -0.0001376042893,3.044733119e-08, -3.49619392e-12,1.622305079e-16}, bhigh = { -49630.5578, -29.4653809}, R = 96.7227844120107);
          constant IdealGases.Common.DataRecord AL2O2plus(name = "AL2O2plus", MM = 0.0859613274, Hf = 6484762.99587714, H0 = 174207.814757407, Tlimit = 1000, alow = {82920.3499, -1757.427015,15.25328567, -0.00898313133,8.9539545e-06, -4.8405862e-09,1.096696856e-12}, blow = {73116.7714, -55.1709255}, ahigh = { -165200.5184, -60.2160697,10.04626917, -1.892194088e-05,4.2533012e-09, -4.94239042e-13,2.314507373e-17}, bhigh = {63862.19, -23.45753361}, R = 96.7234016909795);
          constant IdealGases.Common.DataRecord AL2O3(name = "AL2O3", MM = 0.101961276, Hf =  -5363708.17877956, H0 = 192210.913484449, Tlimit = 1000, alow = { -7443.37432,88.2900421,5.26466264,0.02507678848, -3.43454165e-05,2.30251698e-08, -6.12252928e-12}, blow = { -68726.8595,2.202324298}, ahigh = { -277778.4969, -491.746593,13.86703888, -0.000146938194,3.25040649e-08, -3.73086735e-12,1.730444284e-16}, bhigh = { -67907.5785, -43.75559873}, R = 81.5453898399624);
          constant IdealGases.Common.DataRecord AL2S(name = "AL2S", MM = 0.086028076, Hf = 2565198.95900032, H0 = 162783.26391956, Tlimit = 1000, alow = {40946.2437, -779.717247,9.8144231, -0.00394033982,3.91246543e-06, -2.105891677e-09,4.74600108e-13}, blow = {28339.63305, -24.65067552}, ahigh = { -70434.3737, -26.76376869,7.52023382, -8.16612306e-06,1.81602604e-09, -2.091989298e-13,9.7275377e-18}, bhigh = {24227.18377, -10.66580982}, R = 96.6483546604018);
          constant IdealGases.Common.DataRecord AL2S2(name = "AL2S2", MM = 0.118093076, Hf = 1145599.09507311, H0 = 153264.878967163, Tlimit = 1000, alow = {66145.8732, -1111.319537,12.57007304, -0.001456053256, -5.85162711e-07,1.390055053e-09, -5.4964682e-13}, blow = {19144.56895, -36.00277092}, ahigh = { -137780.6456, -77.050325,10.55723566, -2.277554487e-05,5.00768285e-09, -5.71586212e-13,2.63799924e-17}, bhigh = {13133.36091, -22.56851594}, R = 70.4060922250852);
          constant IdealGases.Common.DataRecord Ar(name = "Ar", MM = 0.039948, Hf = 0, H0 = 155137.37859217, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = { -745.375,4.37967491}, ahigh = {20.10538475, -0.0599266107,2.500069401, -3.99214116e-08,1.20527214e-11, -1.819015576e-15,1.078576636e-19}, bhigh = { -744.993961,4.37918011}, R = 208.132372083709);
          constant IdealGases.Common.DataRecord Arplus(name = "Arplus", MM = 0.0399474514, Hf = 38219669.9286804, H0 = 155353.790605025, Tlimit = 1000, alow = { -57312.0917,793.079147, -1.717121217,0.01044184018, -1.180207501e-05,6.52813478e-09, -1.44755813e-12}, blow = {179057.223,29.4915095}, ahigh = { -383596.54,816.20197,2.301342628, -4.95298377e-06,1.205108477e-08, -2.185050286e-12,1.265493898e-16}, bhigh = {177181.1455,7.94750748}, R = 208.135230374171);
          constant IdealGases.Common.DataRecord B(name = "B", MM = 0.010811, Hf = 53241953.5658126, H0 = 584225.326056794, Tlimit = 1000, alow = {118.2394638, -0.0700991691,2.500236159, -4.5842137e-07,5.12318583e-10, -3.057217674e-13,7.533815325e-17}, blow = {68483.5908,4.20950192}, ahigh = { -107265.961,322.530716,2.126407232,0.0002106579339, -5.93712916e-08,7.37742799e-12, -2.282443381e-16}, bhigh = {66434.131,6.87706967}, R = 769.075201183979);
          constant IdealGases.Common.DataRecord Bplus(name = "Bplus", MM = 0.0108104514, Hf = 127868437.390135, H0 = 573281.14901844, Tlimit = 1000, alow = {0.0784979119, -0.000894748095,2.500004085, -9.57727123e-09,1.218136411e-11, -7.98675252e-15,2.113769829e-18}, blow = {165508.026,2.419053631}, ahigh = { -8911.54803,4.58779009,2.531500086, -4.9039491e-05,2.853326582e-08, -7.38217591e-12,7.12072156e-16}, bhigh = {165452.6303,2.23866978}, R = 769.11422958712);
          constant IdealGases.Common.DataRecord Bminus(name = "Bminus", MM = 0.0108115486, Hf = 50189988.3241518, H0 = 580175.720617859, Tlimit = 1000, alow = {22.01568105, -0.00474046888,2.500014238, -2.497056599e-08,2.556360708e-11, -1.41527478e-14,3.27177071e-18}, blow = {64517.9188,4.61636729}, ahigh = {21.18018248,0.0002070697496,2.499999729,1.456693631e-10, -3.88857176e-14,5.09059863e-18, -2.601566168e-22}, bhigh = {64517.8914,4.61645583}, R = 769.0361767416);
          constant IdealGases.Common.DataRecord BBr(name = "BBr", MM = 0.090715, Hf = 2656146.81144243, H0 = 99179.3859890867, Tlimit = 1000, alow = {37960.935, -497.609602,5.2670082, -3.74284019e-05, -1.13154437e-06,1.261945702e-09, -4.28881188e-13}, blow = {30381.25367, -4.35340733}, ahigh = {253685.8027, -640.59095,4.66682347,0.000417548639, -2.963714868e-07,7.51911954e-11, -4.83536832e-15}, bhigh = {31890.8434, -0.825587091}, R = 91.6548751584633);
          constant IdealGases.Common.DataRecord BBr2(name = "BBr2", MM = 0.170619, Hf = 573374.694494752, H0 = 71508.5482859471, Tlimit = 1000, alow = {65165.5071, -902.959076,8.0638178,0.00112984901, -4.19548239e-06,3.96565877e-09, -1.283029835e-12}, blow = {14704.66408, -13.36453164}, ahigh = { -419163.337,415.768847,7.14816983, -0.000568959075,3.19544177e-07, -5.55317699e-11,3.19113676e-15}, bhigh = {6118.98205, -5.50275962}, R = 48.7312198524197);
          constant IdealGases.Common.DataRecord BBr3(name = "BBr3", MM = 0.250523, Hf =  -819485.636049385, H0 = 62680.0613117359, Tlimit = 1000, alow = {39680.7369, -633.108716,8.05455157,0.0097775713, -1.614866025e-05,1.216119883e-08, -3.51653961e-12}, blow = { -23667.23308, -11.0609939}, ahigh = { -190104.6795, -150.3627687,10.10999896, -4.32309511e-05,9.40969619e-09, -1.06521847e-12,4.88304877e-17}, bhigh = { -27425.53062, -19.97496003}, R = 33.1884577463945);
          constant IdealGases.Common.DataRecord BC(name = "BC", MM = 0.0228217, Hf = 36726532.4230886, H0 = 382754.352217407, Tlimit = 1000, alow = { -39157.58,728.453806, -1.552743361,0.01552898673, -1.976857863e-05,1.272066405e-08, -3.22998278e-12}, blow = {96449.1368,32.48204466}, ahigh = { -2346280.674,6450.7513, -2.619532384,0.0033391605, -4.50802214e-07,1.351919576e-11,8.26510427e-16}, bhigh = {57866.8507,50.37884876}, R = 364.323078473558);
          constant IdealGases.Common.DataRecord BC2(name = "BC2", MM = 0.0348324, Hf = 23003259.1782364, H0 = 335544.636602703, Tlimit = 1000, alow = { -29874.22175,364.312658,2.963469415,0.00656175015, -3.53976724e-06,1.927356029e-10,3.118724866e-13}, blow = {93048.3573,10.83424193}, ahigh = {1525485.889, -5987.86238,14.08384855, -0.0037474998,1.081927223e-06, -1.378744645e-10,6.473822e-15}, bhigh = {131120.4013, -63.37215498}, R = 238.699371849198);
          constant IdealGases.Common.DataRecord BCL(name = "BCL", MM = 0.046264, Hf = 3959308.25263704, H0 = 191538.496455127, Tlimit = 1000, alow = {22024.58989, -170.0511155,3.066075869,0.00559507018, -8.46087041e-06,6.08473341e-09, -1.702889433e-12}, blow = {21974.02537,6.388978917}, ahigh = { -74212.6254,263.8090127,3.60022765,0.001018866266, -4.66618412e-07,9.84900203e-11, -6.46881817e-15}, bhigh = {19127.2648,5.235317877}, R = 179.717966453398);
          constant IdealGases.Common.DataRecord BCLplus(name = "BCLplus", MM = 0.0462634514, Hf = 26679375.6766708, H0 = 191518.741725353, Tlimit = 1000, alow = {65144.0542, -684.554101,5.52727682, -0.000445835745, -4.55772884e-07,6.37085816e-10, -2.13436612e-13}, blow = {150942.4507, -6.918246393}, ahigh = { -216942.3542,358.997516,4.03520837,0.000327445243, -8.63457845e-08,1.207061044e-11, -5.26225327e-16}, bhigh = {144547.7224,3.488222017}, R = 179.720097580095);
          constant IdealGases.Common.DataRecord BCLOH(name = "BCLOH", MM = 0.06327134, Hf =  -3698441.99601273, H0 = 196479.464477914, Tlimit = 1000, alow = { -28984.27288,689.404658, -2.288138069,0.0334942862, -4.72522598e-05,3.32698131e-08, -9.22423886e-12}, blow = { -32619.811,39.76886651}, ahigh = {702773.844, -2857.391274,10.9117584, -8.10770934e-05, -2.124440337e-08,5.40991244e-12, -3.46415384e-16}, bhigh = { -13072.21888, -35.85168799}, R = 131.409766254358);
          constant IdealGases.Common.DataRecord BCL_OH_2(name = "BCL_OH_2", MM = 0.08027868, Hf =  -10032397.8670302, H0 = 168975.162521357, Tlimit = 1000, alow = {80726.2301, -859.461982,2.705039614,0.0387879962, -5.72173435e-05,4.14585033e-08, -1.168536754e-11}, blow = { -93799.1074,7.134982695}, ahigh = {1423878.191, -5849.70407,17.92165219, -0.0002009909087, -3.39343578e-08,9.83856076e-12, -6.47267882e-16}, bhigh = { -64742.6176, -81.00399519}, R = 103.570113509589);
          constant IdealGases.Common.DataRecord BCL2(name = "BCL2", MM = 0.081717, Hf =  -745028.451852124, H0 = 140896.276172645, Tlimit = 1000, alow = {35988.7942, -360.28265,4.15768545,0.0115287745, -1.816567395e-05,1.341714418e-08, -3.84082286e-12}, blow = { -6765.10705,5.182311874}, ahigh = {350140.313, -1662.20359,8.83953472, -0.000977254411,2.467356248e-07, -2.501151018e-11,8.72295456e-16}, bhigh = {568.290944, -21.48348344}, R = 101.747151755449);
          constant IdealGases.Common.DataRecord BCL2plus(name = "BCL2plus", MM = 0.0817164514, Hf = 8227417.98599443, H0 = 157236.416656243, Tlimit = 1000, alow = {80659.5624, -1149.602601,10.16765297, -0.00358649491,2.837145924e-06, -1.225075731e-09,2.224794198e-13}, blow = {84786.1741, -29.37256994}, ahigh = {302929.3331, -1305.533159,8.92039006, -0.000728991964,1.727833266e-07, -1.523567337e-11,4.22691144e-16}, bhigh = {86534.1577, -22.67466616}, R = 101.747834830723);
          constant IdealGases.Common.DataRecord BCL2OH(name = "BCL2OH", MM = 0.09872434, Hf =  -6127338.44561534, H0 = 142333.379995247, Tlimit = 1000, alow = {20997.5517, -247.0699169,2.578761742,0.03120362393, -4.54531405e-05,3.24201177e-08, -9.03982266e-12}, blow = { -73090.2862,12.73658611}, ahigh = {587811.379, -2982.969317,14.00662004, -0.0001194189832, -1.269775742e-08,4.42289219e-12, -3.004089365e-16}, bhigh = { -58241.6807, -51.50858228}, R = 84.2190689752902);
          constant IdealGases.Common.DataRecord BF(name = "BF", MM = 0.0298094032, Hf =  -3587186.3412549, H0 = 291675.413347423, Tlimit = 1000, alow = { -52389.5473,811.847664, -1.141614903,0.01161249417, -1.175212617e-05,6.01923278e-09, -1.238293129e-12}, blow = { -17745.41998,30.05086287}, ahigh = { -374638.978,560.449391,3.60918611,0.000618721693, -1.77893877e-07,2.426601527e-11, -9.39465158e-16}, bhigh = { -18191.79292,3.71660929}, R = 278.921115737064);
          constant IdealGases.Common.DataRecord BFCL(name = "BFCL", MM = 0.0652624032, Hf =  -4277869.86550934, H0 = 169056.60011, Tlimit = 1000, alow = { -23635.12735,438.316898,0.391809572,0.01719990692, -2.139592424e-05,1.337389186e-08, -3.37116469e-12}, blow = { -36871.6659,26.68102752}, ahigh = { -108630.9561, -523.343848,7.29750474, -3.054410701e-05, -3.154598306e-08,1.052290533e-11, -7.61543226e-16}, bhigh = { -33058.9121, -11.86586595}, R = 127.400640986509);
          constant IdealGases.Common.DataRecord BFCL2(name = "BFCL2", MM = 0.1007154032, Hf =  -6384326.32517128, H0 = 131369.309754201, Tlimit = 1000, alow = {4402.55124, -157.1398929,4.01632231,0.01743396078, -2.25278906e-05,1.433673443e-08, -3.64875558e-12}, blow = { -78224.5262,6.886094746}, ahigh = { -223361.3309, -644.060296,10.47861848, -0.0001911892367,4.22488779e-08, -4.84726692e-12,2.247974725e-16}, bhigh = { -77406.2648, -28.14630564}, R = 82.5541251469666);
          constant IdealGases.Common.DataRecord BFOH(name = "BFOH", MM = 0.0468167432, Hf =  -9646798.28476407, H0 = 255744.444863478, Tlimit = 1000, alow = { -75639.5367,1354.838128, -5.64821011,0.0387027466, -5.03765918e-05,3.33959264e-08, -8.84782295e-12}, blow = { -61944.4611,58.0036282}, ahigh = {725131.809, -3215.62085,11.17909777, -0.0001883881929,2.57970808e-09,2.6651581e-12, -2.186647438e-16}, bhigh = { -37168.0808, -39.7846349}, R = 177.596121209901);
          constant IdealGases.Common.DataRecord BF_OH_2(name = "BF_OH_2", MM = 0.0638240832, Hf =  -16449739.4456894, H0 = 201886.487889261, Tlimit = 1000, alow = {13818.00676,252.9637004, -3.84455204,0.0525514085, -7.19311773e-05,4.94830991e-08, -1.345922496e-11}, blow = { -128312.3258,42.9365937}, ahigh = {1422037.64, -6257.29757,18.22460686, -0.000322139645, -7.1267399e-09,6.75869977e-12, -5.04255454e-16}, bhigh = { -91862.5419, -85.2789505}, R = 130.271702829568);
          constant IdealGases.Common.DataRecord BF2(name = "BF2", MM = 0.0488078064, Hf =  -10232521.0009848, H0 = 217428.374326612, Tlimit = 1000, alow = { -67876.5176,1085.903536, -3.023320961,0.02326503687, -2.641444147e-05,1.515620683e-08, -3.51855918e-12}, blow = { -66409.1825,44.3196831}, ahigh = { -115309.1296, -810.980012,7.60270923, -0.0002409209242,5.32847186e-08, -6.11879794e-12,2.839984178e-16}, bhigh = { -57962.2217, -16.55644047}, R = 170.351273971616);
          constant IdealGases.Common.DataRecord BF2plus(name = "BF2plus", MM = 0.0488072578, Hf = 6609394.06433934, H0 = 217436.575590608, Tlimit = 1000, alow = { -29383.56477,295.5698598,1.894274526,0.01280508319, -1.388757254e-05,7.62436675e-09, -1.706339371e-12}, blow = {35989.9244,13.84803311}, ahigh = { -176940.9966, -534.368518,7.72805291, -1.958622052e-06, -2.419680893e-08,5.73253381e-12, -3.21635423e-16}, bhigh = {39087.4505, -19.36560315}, R = 170.353188742351);
          constant IdealGases.Common.DataRecord BF2minus(name = "BF2minus", MM = 0.048808355, Hf =  -15034371.9430823, H0 = 213766.085744951, Tlimit = 1000, alow = {21097.65318,143.95646,0.2604646296,0.01817373751, -2.241565287e-05,1.369712886e-08, -3.35595268e-12}, blow = { -89718.0178,23.51692604}, ahigh = { -170992.0627, -615.621979,7.45887137, -0.0001836703248,4.06436396e-08, -4.66770095e-12,2.16626484e-16}, bhigh = { -87414.0453, -16.20604664}, R = 170.349359243925);
          constant IdealGases.Common.DataRecord BF2CL(name = "BF2CL", MM = 0.0842608064, Hf =  -10538707.5906266, H0 = 146256.528112221, Tlimit = 1000, alow = {4500.10125, -119.6284966,3.19863514,0.01743426933, -1.994070287e-05,1.12513643e-08, -2.544020371e-12}, blow = { -107677.9851,10.08195015}, ahigh = { -206681.7398, -1024.447292,10.75843981, -0.0003023235319,6.67252147e-08, -7.6499419e-12,3.5462245e-16}, bhigh = { -104693.6725, -32.29995036}, R = 98.6754382640231);
          constant IdealGases.Common.DataRecord BF2OH(name = "BF2OH", MM = 0.0658151464, Hf =  -16595227.4171345, H0 = 188343.985815399, Tlimit = 1000, alow = { -7522.52165,436.712169, -3.128973474,0.0432395768, -5.78549404e-05,3.88585813e-08, -1.038574523e-11}, blow = { -134425.9458,41.3917055}, ahigh = {576075.925, -3611.90019,14.4689764, -0.0003028293733,2.764337508e-08, -1.90549317e-13, -8.69497194e-17}, bhigh = { -113358.9041, -58.9698151}, R = 126.330676976204);
          constant IdealGases.Common.DataRecord BF3(name = "BF3", MM = 0.0678062096, Hf =  -16753627.8270302, H0 = 171831.253637867, Tlimit = 1000, alow = {3465.584,21.33198651,1.641245191,0.01993755064, -2.15011993e-05,1.145669081e-08, -2.442285789e-12}, blow = { -137945.5591,16.25533544}, ahigh = { -181976.7014, -1405.347931,11.03412258, -0.000410459105,9.03127757e-08, -1.03305736e-11,4.78055183e-16}, bhigh = { -132313.6863, -37.3838608}, R = 122.621099882274);
          constant IdealGases.Common.DataRecord BF4minus(name = "BF4minus", MM = 0.0868051614, Hf =  -20289876.4266338, H0 = 158926.52899324, Tlimit = 1000, alow = {206848.52, -2463.190805,12.06414196,0.01102766923, -1.75172076e-05,1.201346256e-08, -3.150588626e-12}, blow = { -201056.9206, -46.1082694}, ahigh = { -437324.171, -775.928941,13.58222575, -0.0002341024178,5.19691594e-08, -5.98224578e-12,2.781163206e-16}, bhigh = { -212725.1877, -49.533697}, R = 95.783152359878);
          constant IdealGases.Common.DataRecord BH(name = "BH", MM = 0.01181894, Hf = 37966789.2382904, H0 = 730954.214168106, Tlimit = 1000, alow = {20630.8255, -368.250252,6.07133787, -0.00872832107,1.458566459e-05, -1.036840401e-08,2.779462579e-12}, blow = {54604.5992, -13.00392582}, ahigh = { -1098531.663, -174.5890126,8.44242681, -0.00544019667,2.718307052e-06, -4.83981221e-10,2.868523222e-14}, bhigh = {50167.3567, -29.71030686}, R = 703.487114749715);
          constant IdealGases.Common.DataRecord BHCL(name = "BHCL", MM = 0.04727194, Hf = 2991590.93111051, H0 = 217989.932293872, Tlimit = 1000, alow = {47632.7491, -442.001652,3.97564588,0.00854169876, -1.377785182e-05,1.131521587e-08, -3.56965918e-12}, blow = {18222.78428,3.028452399}, ahigh = {349999.172, -2238.404757,8.71110605, -0.001088882906,4.18930109e-07, -6.56871126e-11,3.6200676e-15}, bhigh = {28412.40364, -25.89566225}, R = 175.885990716692);
          constant IdealGases.Common.DataRecord BHCL2(name = "BHCL2", MM = 0.08272494, Hf =  -3044831.81251023, H0 = 141984.170674527, Tlimit = 1000, alow = {44199.1481, -238.4739577,1.356682235,0.02517866897, -3.6329104e-05,2.644335581e-08, -7.62504411e-12}, blow = { -30038.77914,17.87024616}, ahigh = {411099.606, -2858.027441,11.79547506, -0.000632281797,1.269551549e-07, -1.352862344e-11,5.92165351e-16}, bhigh = { -16172.8876, -42.17900972}, R = 100.507440682338);
          constant IdealGases.Common.DataRecord BHF(name = "BHF", MM = 0.0308173432, Hf =  -2466544.3255991, H0 = 325857.324391286, Tlimit = 1000, alow = { -49083.6226,961.076021, -2.919850316,0.02149156633, -2.595677863e-05,1.693492714e-08, -4.55364884e-12}, blow = { -14669.25195,41.6070931}, ahigh = {1184821.3, -4677.43518,10.67328148, -0.001606366598,3.7045347e-07, -3.79788275e-11,1.432441145e-15}, bhigh = {18026.40937, -42.8593117}, R = 269.798468545465);
          constant IdealGases.Common.DataRecord BHFCL(name = "BHFCL", MM = 0.0662703432, Hf =  -7288880.61651083, H0 = 172242.038426625, Tlimit = 1000, alow = {576.116972,304.215625, -0.980890036,0.02803843648, -3.67673402e-05,2.499824138e-08, -6.87342164e-12}, blow = { -60502.0739,31.32842092}, ahigh = {439077.892, -3137.181709,12.00286161, -0.000715366934,1.453878608e-07, -1.56519707e-11,6.91007506e-16}, bhigh = { -42323.6834, -44.66542881}, R = 125.462938601471);
          constant IdealGases.Common.DataRecord BHF2(name = "BHF2", MM = 0.0498157464, Hf =  -14846988.4413897, H0 = 213975.454957752, Tlimit = 1000, alow = { -83535.4028,1616.659766, -8.16140567,0.0415378679, -4.92106328e-05,3.054165306e-08, -7.78681398e-12}, blow = { -97480.6546,70.3863075}, ahigh = {466087.696, -3728.10049,12.44152658, -0.000890684319,1.841732568e-07, -2.010774349e-11,8.97919659e-16}, bhigh = { -69781.4293, -51.0321804}, R = 166.904495081499);
          constant IdealGases.Common.DataRecord BH2(name = "BH2", MM = 0.01282688, Hf = 25642171.6738599, H0 = 781449.50291887, Tlimit = 1000, alow = {28125.57296, -300.0083489,4.824221, -0.0002186429819,1.485457398e-06,3.89373968e-10, -6.33162939e-13}, blow = {39919.8842, -5.04268285}, ahigh = {1360117.365, -4917.70449,9.75897103, -0.000741587064,1.269760019e-07, -1.187742442e-11,4.68255251e-16}, bhigh = {68902.9266, -41.9049012}, R = 648.206890529887);
          constant IdealGases.Common.DataRecord BH2CL(name = "BH2CL", MM = 0.04827988, Hf =  -1674522.67901246, H0 = 215967.852447024, Tlimit = 1000, alow = {16466.38117,127.7320942,0.1300268921,0.01938143193, -2.316389616e-05,1.607896164e-08, -4.66833765e-12}, blow = { -11121.12215,23.08500112}, ahigh = {1427215.437, -6265.56434,13.82183005, -0.001313843556,2.587131606e-07, -2.71384527e-11,1.172827257e-15}, bhigh = {26393.87516, -63.99725308}, R = 172.214015444943);
          constant IdealGases.Common.DataRecord BH2F(name = "BH2F", MM = 0.0318252832, Hf =  -10179225.4279139, H0 = 318549.309876997, Tlimit = 1000, alow = { -88379.2044,1705.285929, -8.06124613,0.0364318868, -4.15248255e-05,2.620135756e-08, -6.93088641e-12}, blow = { -47872.7341,68.6208767}, ahigh = {1435082.111, -6625.32017,14.09165008, -0.00142246906,2.828724718e-07, -2.992498948e-11,1.302618926e-15}, bhigh = { -793.719267, -68.013564}, R = 261.253668906865);
          constant IdealGases.Common.DataRecord BH3(name = "BH3", MM = 0.01383482, Hf = 7571229.62206953, H0 = 727129.012159175, Tlimit = 1000, alow = { -66196.3507,1262.658391, -4.6543559,0.02461795131, -2.501537437e-05,1.562330756e-08, -4.32394948e-12}, blow = {5667.58798,46.6650462}, ahigh = {1855778.95, -8002.49237,15.05692199, -0.001790456689,3.6125111e-07, -3.86603591e-11,1.698508879e-15}, bhigh = {59675.3707, -79.9404616}, R = 600.981581256569);
          constant IdealGases.Common.DataRecord BH3NH3(name = "BH3NH3", MM = 0.03086534, Hf =  -3725862.08348912, H0 = 410866.363370694, Tlimit = 1000, alow = { -181106.3598,3010.341543, -15.24271135,0.0621563775, -5.95988185e-05,3.26742936e-08, -7.75124309e-12}, blow = { -29342.78877,108.7399608}, ahigh = {4354925.96, -18242.35232,31.6299983, -0.00320824361,5.88507545e-07, -5.80853553e-11,2.3825786e-15}, bhigh = {94381.1803, -189.7865799}, R = 269.378921469843);
          constant IdealGases.Common.DataRecord BH4(name = "BH4", MM = 0.01484276, Hf = 17194273.6391345, H0 = 725665.37490332, Tlimit = 1000, alow = {27342.74578, -84.7361938,1.551871695,0.01439765491, -6.11115236e-06, -1.324417559e-10,5.13300815e-13}, blow = {30220.48074,12.50358328}, ahigh = {1354714.146, -7870.76275,18.2617061, -0.001948275329,4.07270689e-07, -4.4827109e-11,2.014115885e-15}, bhigh = {74702.4949, -96.8608689}, R = 560.170210931121);
          constant IdealGases.Common.DataRecord BI(name = "BI", MM = 0.13771547, Hf = 2367108.80774687, H0 = 66384.7133513758, Tlimit = 1000, alow = {35174.4953, -544.58193,6.02751101, -0.002387237926,2.244796277e-06, -1.110001347e-09,2.280204403e-13}, blow = {40719.1013, -7.28804984}, ahigh = {2132872.626, -6283.68496,11.23964343, -0.00325885675,7.06428215e-07, -4.23006893e-11, -4.20807847e-16}, bhigh = {77976.3481, -46.6404261}, R = 60.3742774867631);
          constant IdealGases.Common.DataRecord BI2(name = "BI2", MM = 0.26461994, Hf = 899767.130171672, H0 = 47949.6140767019, Tlimit = 1000, alow = {66409.759, -1044.411994,9.73419724, -0.00395255754,3.16556778e-06, -1.273764644e-09,1.879968132e-13}, blow = {32057.5089, -20.12407058}, ahigh = { -392402.223,442.290235,7.12962358, -0.000561963406,3.18076052e-07, -5.53707972e-11,3.1839597e-15}, bhigh = {22919.72638, -3.20179324}, R = 31.4204288611055);
          constant IdealGases.Common.DataRecord BI3(name = "BI3", MM = 0.39152441, Hf = 54658.1501776607, H0 = 43249.0837544459, Tlimit = 1000, alow = {62431.1489, -1002.408628,11.06650075,0.001609588058, -5.14725025e-06,4.73251402e-09, -1.512999796e-12}, blow = {5160.34815, -24.20687643}, ahigh = { -150617.6506, -81.6721061,10.05937867, -2.319396215e-05,5.01996673e-09, -5.65434963e-13,2.580651348e-17}, bhigh = { -426.168791, -16.17962779}, R = 21.2361523001848);
          constant IdealGases.Common.DataRecord BN(name = "BN", MM = 0.0248177, Hf = 23157923.9010867, H0 = 361077.295639806, Tlimit = 1000, alow = { -45697.0758,670.428607,0.0361508908,0.00733948751, -4.96903349e-06,1.22903138e-09,6.34028152e-14}, blow = {64854.6508,25.39869896}, ahigh = { -227693.2705, -102.5649298,4.41458681,0.0002561670989, -1.612994942e-08, -6.52605293e-13,5.74946612e-17}, bhigh = {67844.6513, -0.6778568656}, R = 335.021859398736);
          constant IdealGases.Common.DataRecord BO(name = "BO", MM = 0.0268104, Hf = 761137.618237699, H0 = 323535.046101513, Tlimit = 1000, alow = { -11662.16822,92.1757939,3.65549849, -0.00311854292,9.00832983e-06, -8.01778999e-09,2.472952292e-12}, blow = {873.828478,4.48284739}, ahigh = {17886.00589, -630.901963,4.5745284,0.0001988001643, -9.70296348e-08,1.870854291e-11, -1.030218131e-15}, bhigh = {4841.31109, -3.39889058}, R = 310.121147017575);
          constant IdealGases.Common.DataRecord BOminus(name = "BOminus", MM = 0.0268109486, Hf =  -10361105.8356958, H0 = 323525.442139709, Tlimit = 1000, alow = { -82420.2321,920.36369, -0.2302659981,0.00622990412, -3.157624391e-06,1.184578107e-10,2.819228693e-13}, blow = { -39111.4122,25.99226838}, ahigh = {212280.4369, -1262.124688,5.38316731, -0.00031810491,7.28898825e-08, -8.14170892e-12,3.71536439e-16}, bhigh = { -27068.92261, -9.77375367}, R = 310.11480138379);
          constant IdealGases.Common.DataRecord BOCL(name = "BOCL", MM = 0.0622634, Hf =  -5115954.07574916, H0 = 170371.566602531, Tlimit = 1000, alow = {70523.8064, -1315.301429,11.30087727, -0.0120711205,1.880516131e-05, -1.373823975e-08,3.85489016e-12}, blow = { -33553.991, -36.98462517}, ahigh = {155714.8487, -1446.254968,8.50944805, -0.000385900738,8.26226882e-08, -9.26102746e-12,4.21999801e-16}, bhigh = { -32035.8904, -23.72713292}, R = 133.537069931934);
          constant IdealGases.Common.DataRecord BOCL2(name = "BOCL2", MM = 0.0977164, Hf =  -3700151.78618942, H0 = 134774.20371606, Tlimit = 1000, alow = {7322.66736, -202.7726965,4.16192409,0.01733309456, -2.263908718e-05,1.45227519e-08, -3.71959312e-12}, blow = { -44144.4188,6.52042411}, ahigh = { -229752.7041, -619.456793,10.46075518, -0.0001841620739,4.07121543e-08, -4.67224655e-12,2.167237536e-16}, bhigh = { -43715.3842, -27.43570215}, R = 85.0877846502737);
          constant IdealGases.Common.DataRecord BOF(name = "BOF", MM = 0.0458088032, Hf =  -12944636.6108076, H0 = 218043.133683091, Tlimit = 1000, alow = {72268.0265, -1131.38663,8.81605452, -0.00462769457,7.80129958e-06, -5.68120584e-09,1.534732094e-12}, blow = { -67111.1601, -25.47957229}, ahigh = {193702.0297, -1739.112297,8.69565086, -0.000451889557,9.59101908e-08, -1.067806769e-11,4.84000766e-16}, bhigh = { -63301.5239, -27.02458617}, R = 181.503803181656);
          constant IdealGases.Common.DataRecord BOF2(name = "BOF2", MM = 0.0648072064, Hf =  -12849924.3565605, H0 = 179169.241277464, Tlimit = 1000, alow = {7227.79413,21.48783624,1.157840301,0.02250585296, -2.610731536e-05,1.508824929e-08, -3.51836485e-12}, blow = { -101399.5733,20.10127136}, ahigh = { -220502.8861, -1254.23869,10.9255138, -0.0003680144,8.10716494e-08, -9.28127506e-12,4.2975493e-16}, bhigh = { -96804.4758, -34.8285333}, R = 128.295485361332);
          constant IdealGases.Common.DataRecord BOH(name = "BOH", MM = 0.02781834, Hf =  -242882.141781285, H0 = 360608.289351557, Tlimit = 1000, alow = { -75214.1027,1391.444703, -5.63033018,0.02966358811, -3.83083518e-05,2.550606424e-08, -6.79585721e-12}, blow = { -8341.30028,55.1769004}, ahigh = {844749.857, -3016.982887,8.0838647, -0.0001609855388, -2.191830959e-09,3.127133119e-12, -2.376665451e-16}, bhigh = {16476.98495, -26.03227947}, R = 298.884548826422);
          constant IdealGases.Common.DataRecord BO2(name = "BO2", MM = 0.0428098, Hf =  -7220822.03140402, H0 = 251628.295390308, Tlimit = 1000, alow = { -41410.9064,719.885461, -1.477629435,0.02277415194, -2.786326934e-05,1.718462069e-08, -4.27899144e-12}, blow = { -41776.5769,32.5845801}, ahigh = { -38344.5234, -956.326114,8.20096278, -0.0002062130802,9.87228899e-09,8.15836676e-12, -7.52751966e-16}, bhigh = { -34235.6402, -22.24772278}, R = 194.21889380469);
          constant IdealGases.Common.DataRecord BO2minus(name = "BO2minus", MM = 0.0428103486, Hf =  -16689749.4266141, H0 = 224187.405939507, Tlimit = 1000, alow = {53242.7969, -732.631303,5.90247,0.0024374652, -4.87880395e-07, -8.47572476e-10,4.08078118e-13}, blow = { -83442.9368, -10.53931911}, ahigh = {119922.6372, -1715.766982,8.70103822, -0.000460124212,9.86701187e-08, -1.107288516e-11,5.05018324e-16}, bhigh = { -78257.8969, -28.39356505}, R = 194.21640495588);
          constant IdealGases.Common.DataRecord B_OH_2(name = "B_OH_2", MM = 0.04482568, Hf =  -9486607.72128833, H0 = 267071.41977545, Tlimit = 1000, alow = {1491.659222,364.818484, -2.906006389,0.0413683446, -5.78258422e-05,4.06978356e-08, -1.126499791e-11}, blow = { -53754.8366,38.8853518}, ahigh = {1557023.406, -5784.71076,14.87391235, -0.0001821804765, -3.80354164e-08,1.030332746e-11, -6.68588983e-16}, bhigh = { -18068.10301, -65.902689}, R = 185.484570451581);
          constant IdealGases.Common.DataRecord BS(name = "BS", MM = 0.042876, Hf = 6379312.55247691, H0 = 203472.921914358, Tlimit = 1000, alow = { -38394.901,698.941749, -1.21398046,0.01396391264, -1.689881284e-05,1.038889985e-08, -2.582063989e-12}, blow = {28656.85472,31.54819945}, ahigh = {1358760.165, -4364.03596,9.03430759, -0.002114548699,4.18927531e-07, -1.354787322e-11, -1.360684605e-15}, bhigh = {59130.6545, -33.36696868}, R = 193.919022296856);
          constant IdealGases.Common.DataRecord BS2(name = "BS2", MM = 0.074941, Hf = 852236.559426749, H0 = 180936.470023085, Tlimit = 1000, alow = {18202.64374, -515.067189,8.547515, -0.001626155268,1.866387194e-06, -1.277165468e-09,3.68198069e-13}, blow = {8186.83975, -17.78384854}, ahigh = {512040.251, -1938.581076,9.81487346, -0.001339731606,3.75430116e-07, -4.40038378e-11,1.887662968e-15}, bhigh = {17365.35, -27.57546712}, R = 110.94690489852);
          constant IdealGases.Common.DataRecord B2(name = "B2", MM = 0.021622, Hf = 39652691.9803903, H0 = 407229.025992045, Tlimit = 1000, alow = { -151641.8096,2630.168946, -13.81358321,0.0521622519, -6.93049474e-05,4.46941039e-08, -1.128496456e-11}, blow = {89952.5105,98.1311849}, ahigh = {1094594.495, -2602.735739,6.90945621, -0.000640594989,1.951732355e-07, -2.555902119e-11,1.053557323e-15}, bhigh = {118780.7611, -19.49045025}, R = 384.53760059199);
          constant IdealGases.Common.DataRecord B2C(name = "B2C", MM = 0.0336327, Hf = 23799237.4682972, H0 = 348562.975913322, Tlimit = 1000, alow = { -41665.1987,579.777952,1.411017091,0.01174705713, -1.066474425e-05,4.71723262e-09, -7.97001326e-13}, blow = {91968.742,17.69254387}, ahigh = {1159241.019, -4567.79881,12.28672097, -0.002500830701,6.40997985e-07, -6.93293577e-11,2.672933741e-15}, bhigh = {122244.2137, -51.99066186}, R = 247.213931679586);
          constant IdealGases.Common.DataRecord B2CL4(name = "B2CL4", MM = 0.163434, Hf =  -2998152.15928142, H0 = 132090.825654393, Tlimit = 1000, alow = {42855.5513, -728.074881,9.31576367,0.02331629623, -3.51124761e-05,2.464057658e-08, -6.73547996e-12}, blow = { -58190.2324, -16.22083878}, ahigh = { -642798.703,507.856299,14.86725022,2.026983515e-06,6.27271881e-09, -1.218822573e-12,7.24120841e-17}, bhigh = { -68200.66, -41.39110669}, R = 50.8735758777243);
          constant IdealGases.Common.DataRecord B2F4(name = "B2F4", MM = 0.0976156128, Hf =  -14731250.0403624, H0 = 181126.691651522, Tlimit = 1000, alow = { -59716.9564,818.942544, -0.643767129,0.0385896607, -4.31305319e-05,2.404463223e-08, -5.39355935e-12}, blow = { -179004.1225,35.5217997}, ahigh = {38558.6589, -2928.842655,17.92063736, -0.000952399827,1.614708845e-07, -1.428613504e-11,5.21873967e-16}, bhigh = { -161305.1924, -71.8480325}, R = 85.1756369858081);
          constant IdealGases.Common.DataRecord B2H(name = "B2H", MM = 0.02262994, Hf = 35186239.866301, H0 = 447000.610695389, Tlimit = 1000, alow = {87755.7974, -1343.931564,9.83209915, -0.00560713775,5.73065105e-06, -2.266436197e-09,1.97235208e-13}, blow = {100990.8698, -32.8246148}, ahigh = {617554.575, -2627.897466,9.08490507, -0.000538941833,1.0507147e-07, -1.092459556e-11,4.68485466e-16}, bhigh = {109953.7555, -31.6402991}, R = 367.410253849546);
          constant IdealGases.Common.DataRecord B2H2(name = "B2H2", MM = 0.02363788, Hf = 19235140.334074, H0 = 447300.646250848, Tlimit = 1000, alow = {142161.7322, -2138.011262,13.17732399, -0.00894265807,1.127804935e-05, -5.7878862e-09,1.000813871e-12}, blow = {63723.1925, -53.4698274}, ahigh = {1240614.592, -5368.96502,13.79292454, -0.001136749748,2.245617156e-07, -2.361530467e-11,1.022604572e-15}, bhigh = {85065.1149, -64.354613}, R = 351.743557374858);
          constant IdealGases.Common.DataRecord B2H3(name = "B2H3", MM = 0.02464582, Hf = 14244716.5888577, H0 = 484407.538479142, Tlimit = 1000, alow = {94251.4716, -1075.866284,6.38399939,0.01167858327, -1.429661252e-05,1.128776882e-08, -3.68132506e-12}, blow = {46353.3508, -14.00002845}, ahigh = {1771506.902, -7872.26093,17.89447015, -0.001709304785,3.40957543e-07, -3.61454368e-11,1.575671409e-15}, bhigh = {87377.1189, -90.5462946}, R = 337.358302543799);
          constant IdealGases.Common.DataRecord B2H3_db(name = "B2H3_db", MM = 0.02464582, Hf = 14339479.3518739, H0 = 483296.031538005, Tlimit = 1000, alow = {60640.1484, -752.446633,5.58448384,0.01080468494, -8.64903281e-06,4.74833331e-09, -1.309096028e-12}, blow = {44917.923, -8.87368682}, ahigh = {1497731.017, -7219.31414,17.50463766, -0.0015796988,3.16410347e-07, -3.36722515e-11,1.472919654e-15}, bhigh = {83243.4967, -87.7954926}, R = 337.358302543799);
          constant IdealGases.Common.DataRecord B2H4(name = "B2H4", MM = 0.02565376, Hf = 8231232.34176978, H0 = 481015.609407744, Tlimit = 1000, alow = {40001.323, -307.1548876,1.763071673,0.02646361882, -3.066168202e-05,2.163781936e-08, -6.49183541e-12}, blow = {25810.33714,10.1526713}, ahigh = {2292078.842, -10598.32517,22.68612155, -0.002363586598,4.76217307e-07, -5.0901004e-11,2.23391159e-15}, bhigh = {86385.9198, -124.6794566}, R = 324.103445264944);
          constant IdealGases.Common.DataRecord B2H4_db(name = "B2H4_db", MM = 0.02565376, Hf = 8183301.31723381, H0 = 449394.084921665, Tlimit = 1000, alow = { -17814.96923,571.604804, -2.763085348,0.03125348696, -2.709524346e-05,1.341728634e-08, -3.004352798e-12}, blow = {21581.38307,36.7587294}, ahigh = {1886854.256, -10100.45614,22.47110812, -0.002319968667,4.73187996e-07, -5.11067303e-11,2.262693782e-15}, bhigh = {82334.87, -123.7455797}, R = 324.103445264944);
          constant IdealGases.Common.DataRecord B2H5(name = "B2H5", MM = 0.0266617, Hf = 9556175.67521951, H0 = 458364.957973423, Tlimit = 1000, alow = {38309.6545, -40.6614805, -1.422523787,0.0367459527, -3.85198971e-05,2.44449444e-08, -6.82172294e-12}, blow = {30089.56452,28.02425895}, ahigh = {2696691.012, -13200.19835,27.35260413, -0.002961879012,5.98499943e-07, -6.41374855e-11,2.821187823e-15}, bhigh = {106452.4901, -155.9654379}, R = 311.850782208186);
          constant IdealGases.Common.DataRecord B2H5_db(name = "B2H5_db", MM = 0.0266617, Hf = 10320070.2130772, H0 = 437613.35548746, Tlimit = 1000, alow = {14244.46027,420.45582, -3.70045125,0.036720531, -3.19350373e-05,1.695013434e-08, -4.25703814e-12}, blow = {30466.84683,42.4549582}, ahigh = {2571729.448, -13568.29858,27.81376038, -0.00319236503,6.56116526e-07, -7.12733394e-11,3.16949265e-15}, bhigh = {110637.0527, -159.7214457}, R = 311.850782208186);
          constant IdealGases.Common.DataRecord B2H6(name = "B2H6", MM = 0.02766964, Hf = 1322749.41054528, H0 = 431242.07615278, Tlimit = 1000, alow = { -10528.44558,1041.795556, -8.96051886,0.0549248088, -5.30519705e-05,3.011904756e-08, -7.6887683e-12}, blow = { -925.937435,68.1259243}, ahigh = {2835765.414, -15676.00163,32.2612233, -0.00373860973,7.71880646e-07, -8.41445454e-11,3.75222338e-15}, bhigh = {93583.7855, -192.0223395}, R = 300.490790628284);
          constant IdealGases.Common.DataRecord B2O(name = "B2O", MM = 0.0376214, Hf = 5124690.14975519, H0 = 313199.880918839, Tlimit = 1000, alow = { -56995.4676,1018.000465, -2.636481947,0.02746009279, -3.63243272e-05,2.406890217e-08, -6.38412972e-12}, blow = {17038.74394,38.5534611}, ahigh = { -162872.322, -387.628981,7.78772352, -0.0001146822108,2.528097612e-08, -2.893783941e-12,1.339210475e-16}, bhigh = {22630.35354, -19.08513011}, R = 221.003790395892);
          constant IdealGases.Common.DataRecord B2O2(name = "B2O2", MM = 0.0536208, Hf =  -8536080.88652165, H0 = 249839.950168591, Tlimit = 1000, alow = {81743.9169, -1732.702797,16.05560926, -0.02160057288,3.56685457e-05, -2.660198794e-08,7.53183324e-12}, blow = { -48996.329, -61.7270239}, ahigh = {460578.966, -2990.079203,12.56764079, -0.000785097495,1.672537624e-07, -1.86769478e-11,8.48619067e-16}, bhigh = { -40157.4815, -48.7441371}, R = 155.060573508788);
          constant IdealGases.Common.DataRecord B2O3(name = "B2O3", MM = 0.0696202, Hf =  -11999136.3282496, H0 = 207105.538335138, Tlimit = 1000, alow = {73796.1191, -1263.620592,10.72681512,0.000384138372,5.97605838e-06, -6.55289135e-09,2.123951064e-12}, blow = { -96281.8314, -30.88078011}, ahigh = {390503.53, -3691.34821,15.55502598, -0.000970764551,2.068887872e-07, -2.310858356e-11,1.050136734e-15}, bhigh = { -82630.5441, -63.9086344}, R = 119.426143561782);
          constant IdealGases.Common.DataRecord B2_OH_4(name = "B2_OH_4", MM = 0.08965136, Hf =  -13998533.1064693, H0 = 216874.222543863, Tlimit = 1000, alow = { -17287.17249,849.591348, -8.71595957,0.0936236261, -0.0001278784165,8.81367116e-08, -2.400605571e-11}, blow = { -156433.7133,71.2016931}, ahigh = {2983925.656, -12074.69577,31.6261057, -0.000516368244, -4.23209397e-08,1.67197728e-11, -1.156366726e-15}, bhigh = { -82809.7697, -165.3010498}, R = 92.7422852257902);
          constant IdealGases.Common.DataRecord B2S(name = "B2S", MM = 0.053687, Hf = 11590528.5264589, H0 = 245547.4323393, Tlimit = 1000, alow = {66386.093, -920.118569,8.69983134,0.000764558901, -3.70902603e-06,3.64237398e-09, -1.197766071e-12}, blow = {77703.758, -22.51712444}, ahigh = { -121150.1483, -64.7339845,7.54682561, -1.82054145e-05,3.92395117e-09, -4.40386238e-13,2.003651718e-17}, bhigh = {72585.3187, -13.94971229}, R = 154.869372473783);
          constant IdealGases.Common.DataRecord B2S2(name = "B2S2", MM = 0.085752, Hf = 1612989.92443325, H0 = 169898.218117362, Tlimit = 1000, alow = { -70615.4997,718.294478,1.919797788,0.02036407461, -2.309813846e-05,1.311616725e-08, -2.996394781e-12}, blow = {11008.29019,18.55402586}, ahigh = { -164153.5014, -774.286564,11.07731737, -0.0002313278554,5.12561429e-08, -5.89408123e-12,2.738673834e-16}, bhigh = {17330.03515, -33.59766998}, R = 96.959511148428);
          constant IdealGases.Common.DataRecord B2S3(name = "B2S3", MM = 0.117817, Hf = 150691.810180195, H0 = 169229.033161598, Tlimit = 1000, alow = { -34312.685,558.761537,3.41407741,0.02398333891, -2.841617637e-05,1.690005199e-08, -4.06007071e-12}, blow = { -3027.690575,18.46767864}, ahigh = { -190585.5086, -934.834248,13.69333613, -0.000276664725,6.1101558e-08, -7.00804857e-12,3.24952462e-16}, bhigh = {2906.43376, -39.46870479}, R = 70.5710720863712);
          constant IdealGases.Common.DataRecord B3H7_C2v(name = "B3H7_C2v", MM = 0.03948858, Hf = 4457469.45066143, H0 = 375258.846988167, Tlimit = 1000, alow = {108600.9469, -792.15368, -0.957736804,0.0565611929, -6.4050345e-05,4.28132296e-08, -1.227580455e-11}, blow = {24306.38819,21.02582666}, ahigh = {3598592.14, -18637.01968,39.9399883, -0.00427541016,8.70535882e-07, -9.38510502e-11,4.14786531e-15}, bhigh = {127215.4092, -237.9769856}, R = 210.553836071087);
          constant IdealGases.Common.DataRecord B3H7_Cs(name = "B3H7_Cs", MM = 0.03948858, Hf = 4034541.68268395, H0 = 356642.907898942, Tlimit = 1000, alow = {52981.6933,407.174537, -9.63573053,0.07944312, -9.30645682e-05,6.10514735e-08, -1.685927691e-11}, blow = {17070.69083,68.7396656}, ahigh = {3281576.45, -18110.96595,39.6383246, -0.00417842411,8.52682906e-07, -9.20960024e-11,4.07649677e-15}, bhigh = {121362.2564, -235.7090217}, R = 210.553836071087);
          constant IdealGases.Common.DataRecord B3H9(name = "B3H9", MM = 0.04150446, Hf = 3346840.31547453, H0 = 417543.921785755, Tlimit = 1000, alow = {84832.149, -520.616304, -0.466570121,0.0534698386, -4.15579682e-05,1.944135439e-08, -4.54122735e-12}, blow = {18051.011,21.10804198}, ahigh = {3863999.5, -22294.33407,48.7504615, -0.00541844926,1.125767651e-06, -1.233226406e-10,5.52034818e-15}, bhigh = {142218.7123, -295.8240102}, R = 200.327193752189);
          constant IdealGases.Common.DataRecord B3N3H6(name = "B3N3H6", MM = 0.08050074, Hf =  -6360189.98086229, H0 = 201505.576222032, Tlimit = 1000, alow = { -155262.1082,4010.21127, -33.824913,0.1624575582, -0.0002065528647,1.368862991e-07, -3.6734709e-11}, blow = { -80512.534,199.5000751}, ahigh = {3739245.17, -19412.38951,44.4507003, -0.00321030805,5.71746542e-07, -5.48669214e-11,2.191943197e-15}, bhigh = {47584.2616, -264.1156625}, R = 103.284417012813);
          constant IdealGases.Common.DataRecord B3O3CL3(name = "B3O3CL3", MM = 0.1867902, Hf =  -8758393.32577405, H0 = 130905.052834678, Tlimit = 1000, alow = { -42844.528,1005.623966, -4.55651963,0.0811796714, -0.0001038385803,6.62142233e-08, -1.695748504e-11}, blow = { -204090.7277,54.66113803}, ahigh = { -741405.981, -2527.447455,26.87263667, -0.000746338936,1.64633411e-07, -1.886231553e-11,8.73792631e-16}, bhigh = { -192350.9858, -118.5636255}, R = 44.5123566439781);
          constant IdealGases.Common.DataRecord B3O3FCL2(name = "B3O3FCL2", MM = 0.1703356032, Hf =  -11059388.9099516, H0 = 137658.472800124, Tlimit = 1000, alow = { -37191.2007,867.374589, -4.09577846,0.0781556397, -9.76703517e-05,6.09761009e-08, -1.532880319e-11}, blow = { -233138.3552,51.98088158}, ahigh = { -716236.083, -2851.210308,27.10953133, -0.000840091964,1.852318684e-07, -2.121694694e-11,9.82729156e-16}, bhigh = { -220272.9252, -121.0679365}, R = 48.8122966884236);
          constant IdealGases.Common.DataRecord B3O3F2CL(name = "B3O3F2CL", MM = 0.1538810064, Hf =  -13860169.4705319, H0 = 144663.669160926, Tlimit = 1000, alow = { -52770.1554,1104.904696, -5.85418105,0.079539431, -9.59266077e-05,5.79985631e-08, -1.416698348e-11}, blow = { -264040.0115,60.71247222}, ahigh = { -681078.749, -3370.00902,27.48953579, -0.000990544256,2.18290844e-07, -2.49957348e-11,1.157532106e-15}, bhigh = { -247211.4785, -126.1501548}, R = 54.0318275433374);
          constant IdealGases.Common.DataRecord B3O3F3(name = "B3O3F3", MM = 0.1374264096, Hf =  -17337999.9152652, H0 = 154269.751074105, Tlimit = 1000, alow = { -72515.4978,1400.996228, -7.9067162,0.0821716185, -9.63315266e-05,5.66819426e-08, -1.349024305e-11}, blow = { -295346.1728,70.0153683}, ahigh = { -662982.291, -3784.94687,27.79526578, -0.001112096232,2.450806704e-07, -2.80648498e-11,1.299750466e-15}, bhigh = { -274886.5475, -131.5605015}, R = 60.5012677272186);
          constant IdealGases.Common.DataRecord B4H4(name = "B4H4", MM = 0.04727576, Hf = 6899738.89367405, H0 = 315946.311598164, Tlimit = 1000, alow = {217179.1579, -2598.282557,9.84544061,0.0302630914, -4.12833416e-05,3.089368344e-08, -9.35338426e-12}, blow = {50791.4348, -37.7075721}, ahigh = {1937200.886, -10781.85932,28.79625241, -0.002401547707,4.83775678e-07, -5.17049023e-11,2.269138182e-15}, bhigh = {98365.9539, -156.1814117}, R = 175.871778687429);
          constant IdealGases.Common.DataRecord B4H10(name = "B4H10", MM = 0.0533234, Hf = 1239605.87659452, H0 = 290408.601101955, Tlimit = 1000, alow = { -32966.0482,2440.532922, -26.85332563,0.1406499347, -0.0001586710574,9.87459413e-08, -2.601426018e-11}, blow = { -3091.924018,158.9905532}, ahigh = {4030474.13, -25606.29762,56.8723366, -0.00617856786,1.280621032e-06, -1.400254585e-10,6.25886656e-15}, bhigh = {150600.9087, -352.07142}, R = 155.925391104093);
          constant IdealGases.Common.DataRecord B4H12(name = "B4H12", MM = 0.05533928, Hf = 3401491.92400046, H0 = 393006.811798057, Tlimit = 1000, alow = {71521.703, -159.0351575, -4.71977166,0.088112289, -8.60494688e-05,5.12353551e-08, -1.37986525e-11}, blow = {21941.86547,41.878974}, ahigh = {6158360.8, -32020.8673,66.604436, -0.00740636928,1.512993908e-06, -1.635619071e-10,7.24538514e-15}, bhigh = {205458.6275, -414.210686}, R = 150.245395314142);
          constant IdealGases.Common.DataRecord B5H9(name = "B5H9", MM = 0.06312646, Hf = 1159893.96522473, H0 = 254293.952171562, Tlimit = 1000, alow = {89984.7372,104.2533357, -11.95549834,0.1007310686, -9.86668021e-05,5.29553518e-08, -1.232119583e-11}, blow = {8374.34178,76.6309048}, ahigh = {3169017.62, -22825.03203,55.1200898, -0.00556341142,1.157837073e-06, -1.270332301e-10,5.69426203e-15}, bhigh = {133445.2571, -337.106046}, R = 131.711361606528);
          constant IdealGases.Common.DataRecord Ba(name = "Ba", MM = 0.137327, Hf = 1347149.50446744, H0 = 45128.9841036358, Tlimit = 1000, alow = {2222.563526, -34.0797785,2.706751118, -0.000638289449,1.063003846e-06, -9.10262427e-10,3.148062219e-13}, blow = {21665.49702,5.10254588}, ahigh = { -19265792.28,60065.0104, -66.3396413,0.0350756593, -7.80760183e-06,8.0851268e-10, -3.199486918e-14}, bhigh = { -358966.372,500.75834}, R = 60.5450639713967);
          constant IdealGases.Common.DataRecord Baplus(name = "Baplus", MM = 0.1373264514, Hf = 5054011.96145668, H0 = 45129.1643876265, Tlimit = 1000, alow = { -54231.6755,674.468078, -0.89402725,0.00879664295, -1.222812901e-05,8.38795387e-09, -2.037964358e-12}, blow = {79417.7464,26.07063698}, ahigh = {8794971.85, -19518.17883,14.85542861, -0.00094042335, -7.03125779e-07,1.667412753e-10, -1.07011731e-14}, bhigh = {214680.0732, -92.2826419}, R = 60.5453058404741);
          constant IdealGases.Common.DataRecord BaBr(name = "BaBr", MM = 0.217231, Hf =  -346749.20706529, H0 = 46955.1905575171, Tlimit = 1000, alow = { -691.255744, -62.4550016,4.79667765, -0.000683800494,1.04394768e-06, -7.4412243e-10,2.172187003e-13}, blow = { -10113.49165,5.19645487}, ahigh = { -1056014.042,1034.740207,6.73572938, -0.0037840263,1.952196896e-06, -3.34783188e-10,1.893189803e-14}, bhigh = { -19502.33645, -5.06927964}, R = 38.274795033858);
          constant IdealGases.Common.DataRecord BaBr2(name = "BaBr2", MM = 0.297135, Hf =  -1388307.47303414, H0 = 52116.9131876083, Tlimit = 1000, alow = { -5143.37406, -38.7051846,7.16244697, -0.000369671531,4.68971527e-07, -3.107902475e-10,8.36358205e-14}, blow = { -51533.1826,0.9741407}, ahigh = { -8928.25121, -0.753655889,7.00065821, -2.94982171e-07,7.09239211e-11, -8.6700381e-15,4.22174231e-19}, bhigh = { -51726.6949,1.910185706}, R = 27.9821360660979);
          constant IdealGases.Common.DataRecord BaCL(name = "BaCL", MM = 0.17278, Hf =  -788811.673804839, H0 = 57165.8698923487, Tlimit = 1000, alow = { -2661.946867, -77.2499634,4.74610605, -0.000352354888,3.99919835e-07, -1.936413749e-10,4.12404853e-14}, blow = { -17363.28141,3.904455084}, ahigh = { -1075410.089,1209.303069,6.2725586, -0.00328811031,1.717751087e-06, -2.910607065e-10,1.632658438e-14}, bhigh = { -27791.73737, -3.574185216}, R = 48.1217270517421);
          constant IdealGases.Common.DataRecord BaCLplus(name = "BaCLplus", MM = 0.1727794514, Hf = 2018166.09078549, H0 = 55957.0592547905, Tlimit = 1000, alow = { -895.155669, -133.4692684,4.93541612, -0.000757405451,8.355724e-07, -4.68525851e-10,1.095343103e-13}, blow = {41251.5599,1.896278378}, ahigh = { -18291.98157, -2.931616688,4.50228071,3.67178849e-05,2.396147871e-09,5.4794586e-14,1.174342316e-18}, bhigh = {40551.0015,4.503765564}, R = 48.1218798452557);
          constant IdealGases.Common.DataRecord BaCL2(name = "BaCL2", MM = 0.208233, Hf =  -2397801.43877291, H0 = 70109.507138638, Tlimit = 1000, alow = { -1157.501999, -205.4518828,7.83751405, -0.001867700704,2.335378734e-06, -1.531373341e-09,4.08852509e-13}, blow = { -61156.6528, -5.978149218}, ahigh = { -22165.98768, -4.00248381,7.00342674, -1.515820808e-06,3.61184921e-10, -4.38675906e-14,2.125803293e-18}, bhigh = { -62190.9527, -1.132439694}, R = 39.9286952596371);
          constant IdealGases.Common.DataRecord BaF(name = "BaF", MM = 0.1563254032, Hf =  -2040575.50129511, H0 = 59760.6774635845, Tlimit = 1000, alow = {25303.17292, -471.69951,6.18503622, -0.0033666788,4.01183456e-06, -2.506578601e-09,6.47725816e-13}, blow = { -37318.7214, -6.22071538}, ahigh = {1868380.434, -7071.0996,14.66959284, -0.0069959502,2.366313845e-06, -3.2877343e-10,1.647896238e-14}, bhigh = {3571.78068, -66.6239092}, R = 53.1869538142986);
          constant IdealGases.Common.DataRecord BaFplus(name = "BaFplus", MM = 0.1563248546, Hf = 857593.72265554, H0 = 58862.7702456216, Tlimit = 1000, alow = {28407.19922, -486.357182,5.94605839, -0.002429844077,2.445346729e-06, -1.316757471e-09,2.97488632e-13}, blow = {17306.39076, -5.95120693}, ahigh = { -40544.2367, -17.01997704,4.51305534,2.632753508e-05,2.612088236e-09, -9.50219793e-14,6.51517616e-18}, bhigh = {14744.8657,2.783167711}, R = 53.1871404664016);
          constant IdealGases.Common.DataRecord BaF2(name = "BaF2", MM = 0.1753238064, Hf =  -4631450.22728642, H0 = 76641.1434699492, Tlimit = 1000, alow = {34883.9552, -764.845956,9.79775038, -0.00575645751,6.7756038e-06, -4.24361475e-09,1.093687692e-12}, blow = { -95903.5645, -20.82035292}, ahigh = { -55086.3366, -16.91114228,7.01356488, -5.73190977e-06,1.321203959e-09, -1.565528167e-13,7.44498302e-18}, bhigh = { -99835.297, -4.38958149}, R = 47.4235197759202);
          constant IdealGases.Common.DataRecord BaH(name = "BaH", MM = 0.13833494, Hf = 1514694.01005993, H0 = 63115.6886322429, Tlimit = 1000, alow = { -37652.6888,698.668012, -1.313524351,0.01460648882, -1.802974829e-05,1.133031115e-08, -2.865589059e-12}, blow = {20974.78889,32.308026}, ahigh = { -6755466.57,17307.00212, -11.36812458,0.00561774474, -3.18078885e-07, -1.052342502e-10,1.111101879e-14}, bhigh = { -89540.7038,118.2377538}, R = 60.1039187930396);
          constant IdealGases.Common.DataRecord BaI(name = "BaI", MM = 0.26423147, Hf =  -38745.1010282764, H0 = 39348.5492095245, Tlimit = 1000, alow = { -4186.28554,10.33094613,4.42890023,0.0003026019976, -2.591429408e-07,1.7416054e-10, -3.37006276e-14}, blow = { -2636.169733,8.20192468}, ahigh = { -3533859.66,7394.258,1.285151488, -0.002303429302,2.214839289e-06, -4.68109681e-10,3.011371472e-14}, bhigh = { -53402.2959,37.1044387}, R = 31.4666228061328);
          constant IdealGases.Common.DataRecord BaI2(name = "BaI2", MM = 0.39113594, Hf =  -737441.731895054, H0 = 40370.7084549684, Tlimit = 1000, alow = { -4241.59273, -18.62214382,7.07855675, -0.0001793922821,2.281437677e-07, -1.514647881e-10,4.08149836e-14}, blow = { -36703.5149,3.16313897}, ahigh = { -6048.2248, -0.362272945,7.00031745, -1.425793502e-07,3.43334942e-11, -4.20175498e-15,2.047703474e-19}, bhigh = { -36796.5075,3.61547532}, R = 21.257243709182);
          constant IdealGases.Common.DataRecord BaO(name = "BaO", MM = 0.1533264, Hf =  -769263.427563681, H0 = 58790.3192144341, Tlimit = 1000, alow = {37643.985, -507.15553,5.39284728, -0.000411932469, -6.52789662e-07,9.43058842e-10, -3.43660303e-13}, blow = { -12755.52614, -3.75221376}, ahigh = {13184816.98, -38542.5325,46.8674161, -0.02188646633,5.33567745e-06, -5.1523043e-10,1.4330834e-14}, bhigh = {230690.2396, -302.8332772}, R = 54.2272694069645);
          constant IdealGases.Common.DataRecord BaOplus(name = "BaOplus", MM = 0.1533258514, Hf = 3337367.32147701, H0 = 61762.4876270539, Tlimit = 1000, alow = {356641.097, -3779.24671,16.73544137, -0.0130661161,2.686157445e-06,4.04407795e-09, -2.016508813e-12}, blow = {79832.6849, -72.8511855}, ahigh = { -331767.923,1594.253495,3.22724781,0.00057226415, -1.147415658e-07,1.382873231e-11, -6.38443703e-16}, bhigh = {50252.4761,14.09092365}, R = 54.2274634321711);
          constant IdealGases.Common.DataRecord BaOH(name = "BaOH", MM = 0.15433434, Hf =  -1453056.48114347, H0 = 72728.9921348677, Tlimit = 1000, alow = {37762.3151, -889.139022,9.531718, -0.0054588259,4.94178301e-06, -1.809327537e-09,2.050048305e-13}, blow = { -24418.52965, -24.89410158}, ahigh = {2637336.694, -8365.4104,15.99803975, -0.00509844605,1.594657431e-06, -2.180478542e-10,1.084610688e-14}, bhigh = {23935.50647, -74.9070579}, R = 53.8731172854985);
          constant IdealGases.Common.DataRecord BaOHplus(name = "BaOHplus", MM = 0.1543337914, Hf = 1308976.544718, H0 = 73237.2340332462, Tlimit = 1000, alow = {27976.32165, -768.07694,9.05297844, -0.0043685779,3.52349726e-06, -8.38636087e-10, -6.47545747e-14}, blow = {26232.80802, -22.64561641}, ahigh = {876673.843, -2335.8196,7.97316606,0.0001038679898, -6.31948578e-08,1.028729445e-11, -5.74180857e-16}, bhigh = {37725.1747, -19.30805001}, R = 53.8733087846632);
          constant IdealGases.Common.DataRecord Ba_OH_2(name = "Ba_OH_2", MM = 0.17134168, Hf =  -3540682.40722281, H0 = 101485.668869361, Tlimit = 1000, alow = {57028.149, -1577.384797,16.59935325, -0.01039598275,9.6648896e-06, -3.67362144e-09,4.62578499e-13}, blow = { -68351.589, -58.519117}, ahigh = {1762908.112, -4676.20542,13.95135494,0.0002051842928, -1.25726553e-07,2.048917625e-11, -1.144045777e-15}, bhigh = { -45459.6441, -49.2945719}, R = 48.5256827177135);
          constant IdealGases.Common.DataRecord BaS(name = "BaS", MM = 0.169392, Hf = 229475.506517427, H0 = 56414.2049211297, Tlimit = 1000, alow = {13770.75147, -329.350056,5.81154584, -0.00284897376,3.57633602e-06, -2.325649347e-09,6.17280531e-13}, blow = {4964.43245, -3.497962615}, ahigh = {6104262.07, -22276.17479,35.9751351, -0.0215779374,7.31987235e-06, -1.089038636e-09,5.88410195e-14}, bhigh = {140413.8006, -214.1776331}, R = 49.0842070463776);
          constant IdealGases.Common.DataRecord Ba2(name = "Ba2", MM = 0.274654, Hf = 1296046.43296657, H0 = 41357.9885965615, Tlimit = 1000, alow = { -12941.77083, -727.633008,14.64866569, -0.0396633165,5.89871106e-05, -4.23530056e-08,1.189594362e-11}, blow = {43867.1223, -41.4567367}, ahigh = {216011.4547,195.0676877,2.253699771,0.0001507750613, -4.742515e-08,7.04895196e-12, -3.54744017e-16}, bhigh = {41677.6297,23.75906459}, R = 30.2725319856984);
          constant IdealGases.Common.DataRecord Be(name = "Be", MM = 0.009012182, Hf = 35951337.8669006, H0 = 687672.308437624, Tlimit = 1000, alow = { -0.000411290152,5.36496736e-06,2.499999972,7.56920369e-11, -1.097852652e-13,8.00211024e-17, -2.303022777e-20}, blow = {38222.6459,2.146172983}, ahigh = { -692628.584,2466.773005, -0.977661334,0.002458939515, -9.04795042e-07,1.587880407e-10, -9.415600603e-15}, bhigh = {23002.12917,26.23234754}, R = 922.581456965694);
          constant IdealGases.Common.DataRecord Beplus(name = "Beplus", MM = 0.0090116334, Hf = 136457096.446023, H0 = 687714.171772678, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {147152.8569,2.839228698}, ahigh = { -94781.3598,276.8325398,2.191388413,0.0001648824289, -4.28016682e-08,4.54235047e-12, -6.270417825e-17}, bhigh = {145385.0986,5.05550384}, R = 922.637620833533);
          constant IdealGases.Common.DataRecord Beplusplus(name = "Beplusplus", MM = 0.0090110848, Hf = 332146638.77095, H0 = 687756.040205059, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {359227.916,2.145990203}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = {359227.916,2.145990203}, R = 922.693791539949);
          constant IdealGases.Common.DataRecord BeBr(name = "BeBr", MM = 0.088916182, Hf = 1489563.59822108, H0 = 100893.985753909, Tlimit = 1000, alow = {37462.8806, -472.233907,5.03940904,0.000579885322, -1.937634343e-06,1.793372625e-09, -5.67229558e-13}, blow = {17231.35679, -2.573912886}, ahigh = {821723.814, -2850.509348,8.01726602, -0.002105663212,6.84913634e-07, -9.98912932e-11,5.19245684e-15}, bhigh = {32293.7287, -23.26810811}, R = 93.5090982651504);
          constant IdealGases.Common.DataRecord BeBr2(name = "BeBr2", MM = 0.168820182, Hf =  -1386460.41146905, H0 = 76042.4544501439, Tlimit = 1000, alow = { -21186.98678,209.0382611,3.73117131,0.01115501549, -1.542821761e-05,1.052354444e-08, -2.850006057e-12}, blow = { -30904.56277,9.45891772}, ahigh = { -104133.1811, -154.3443092,7.61446541, -4.55670063e-05,1.003180548e-08, -1.146879501e-12,5.30179538e-17}, bhigh = { -29845.27608, -11.4936098}, R = 49.2504622462734);
          constant IdealGases.Common.DataRecord BeCL(name = "BeCL", MM = 0.044465182, Hf = 1274999.99887553, H0 = 199281.923550881, Tlimit = 1000, alow = {20161.38947, -141.3988978,2.898223773,0.00606992551, -9.16203847e-06,6.61262066e-09, -1.859126853e-12}, blow = {6626.69144,7.90018105}, ahigh = {280884.5146, -1103.100122,5.67118983, -0.000544300045,1.459036606e-07, -1.230971735e-11,9.90080476e-17}, bhigh = {12246.49804, -8.38237137}, R = 186.988372160492);
          constant IdealGases.Common.DataRecord BeCL2(name = "BeCL2", MM = 0.079918182, Hf =  -4523865.92077382, H0 = 151165.350583175, Tlimit = 1000, alow = { -22299.57657,152.8781667,3.71499471,0.0106777655, -1.397766942e-05,9.06669356e-09, -2.350292311e-12}, blow = { -45904.2823,6.682555572}, ahigh = { -115408.746, -248.4300053,7.68567214, -7.44496037e-05,1.649479787e-08, -1.895997075e-12,8.80497395e-17}, bhigh = { -44687.4728, -15.00337407}, R = 104.037301549227);
          constant IdealGases.Common.DataRecord BeF(name = "BeF", MM = 0.0280105852, Hf =  -6091439.8532452, H0 = 310993.324052366, Tlimit = 1000, alow = { -46644.9723,788.717885, -1.463363201,0.01381790494, -1.591299964e-05,9.35293152e-09, -2.229211181e-12}, blow = { -25226.27476,31.976343}, ahigh = { -185794.1543,47.7608277,4.26998602,0.0002310668473, -6.65721492e-08,1.152149888e-11, -6.33558644e-16}, bhigh = { -22579.5096, -0.218478727}, R = 296.833212895531);
          constant IdealGases.Common.DataRecord BeF2(name = "BeF2", MM = 0.0470089884, Hf =  -16945445.1821388, H0 = 231440.079233868, Tlimit = 1000, alow = {7490.27024, -287.8053102,5.43471282,0.00390323346, -2.214865601e-06, -1.965806119e-10,4.04236227e-13}, blow = { -95916.1564, -5.61661692}, ahigh = { -64820.1807, -768.455606,8.06286084, -0.0002227502549,4.89159866e-08, -5.58788672e-12,2.583409761e-16}, bhigh = { -93954.9549, -21.25660266}, R = 176.869834535729);
          constant IdealGases.Common.DataRecord BeH(name = "BeH", MM = 0.010020122, Hf = 34156480.7294761, H0 = 863073.72305447, Tlimit = 1000, alow = { -1615.149125, -59.3528608,4.5072691, -0.00526418961,1.145319967e-05, -9.24788359e-09,2.700364753e-12}, blow = {40301.9304, -3.48511705}, ahigh = { -2424081.636,6597.39846, -3.62631648,0.00457963435, -1.163329056e-06,1.297785691e-10, -5.29188581e-15}, bhigh = { -2489.846221,52.1601869}, R = 829.777521670894);
          constant IdealGases.Common.DataRecord BeHplus(name = "BeHplus", MM = 0.0100195734, Hf = 117591728.106907, H0 = 862706.490078709, Tlimit = 1000, alow = { -32206.1102,252.0229596,3.20771957, -0.00219128729,6.90253449e-06, -5.76952263e-09,1.656414969e-12}, blow = {139253.066,3.30704477}, ahigh = {462234.159, -1760.169629,5.37990555,2.742462444e-05, -1.16720817e-07,3.55717791e-11, -2.569695765e-15}, bhigh = {151585.9734, -13.69222641}, R = 829.822954338555);
          constant IdealGases.Common.DataRecord BeH2(name = "BeH2", MM = 0.011028062, Hf = 14608088.2570301, H0 = 835967.643272227, Tlimit = 1000, alow = {90653.149, -1320.65722,9.90130806, -0.01255772231,2.205704068e-05, -1.633609388e-08,4.50510324e-12}, blow = {24645.61869, -36.3582675}, ahigh = {607426.201, -3407.53425,9.79237036, -0.000825461482,1.915656246e-07, -1.962774813e-11,8.92990866e-16}, bhigh = {37931.052, -42.5465358}, R = 753.937727227141);
          constant IdealGases.Common.DataRecord BeI(name = "BeI", MM = 0.135916652, Hf = 1526334.74962288, H0 = 66872.6669341443, Tlimit = 1000, alow = {35300.5481, -519.095026,5.71450524, -0.00140907364,7.66477212e-07, -2.601945105e-11, -8.23096064e-14}, blow = {26379.03657, -5.14099784}, ahigh = { -532820.662,1139.497861,3.62917166,0.0001704033769,1.067048688e-07, -3.58945788e-11,2.72276956e-15}, bhigh = {15854.74307,9.37095127}, R = 61.1733137746801);
          constant IdealGases.Common.DataRecord BeI2(name = "BeI2", MM = 0.262821122, Hf =  -246401.242438954, H0 = 51139.5313197088, Tlimit = 1000, alow = {359.272618, -48.6605901,5.24157118,0.00783030506, -1.169525374e-05,8.38958391e-09, -2.356636003e-12}, blow = { -9333.22092,2.896318281}, ahigh = { -93923.7284, -89.5921438,7.56555479, -2.577727326e-05,5.61426533e-09, -6.35966385e-13,2.917080336e-17}, bhigh = { -9814.76862, -9.03204281}, R = 31.63547867359);
          constant IdealGases.Common.DataRecord BeN(name = "BeN", MM = 0.023018882, Hf = 18549988.6571381, H0 = 379046.25428811, Tlimit = 1000, alow = { -42477.8363,759.01889, -1.561469235,0.01496945721, -1.83560888e-05,1.146861461e-08, -2.897470581e-12}, blow = {46830.0332,32.56973759}, ahigh = { -59096.9616, -307.8649431,4.72795915, -3.92815883e-05,2.006201069e-08, -2.300018068e-12,1.066101831e-16}, bhigh = {51560.9571, -3.031380625}, R = 361.202251264853);
          constant IdealGases.Common.DataRecord BeO(name = "BeO", MM = 0.025011582, Hf = 5155223.84789575, H0 = 347363.233561156, Tlimit = 1000, alow = { -48694.5832,721.347362, -0.378415732,0.00879220385, -7.07015126e-06,2.25017562e-09, -2.799117872e-14}, blow = {11014.66639,25.74081184}, ahigh = { -35034724.5,105585.9473, -116.5011722,0.0647736948, -1.649656058e-05,2.0055584e-09, -9.42689179e-14}, bhigh = { -657001.8,864.089803}, R = 332.424874204279);
          constant IdealGases.Common.DataRecord BeOH(name = "BeOH", MM = 0.026019522, Hf =  -3832419.40416892, H0 = 415013.38879323, Tlimit = 1000, alow = { -38530.2452,449.805929,1.532096459,0.01299912603, -1.708979996e-05,1.169968182e-08, -3.16318371e-12}, blow = { -15590.46567,15.4554293}, ahigh = {855033.031, -2646.537838,8.20478463,1.110814513e-05, -4.265364e-08,7.92651192e-12, -4.64541952e-16}, bhigh = {3121.216271, -25.71641308}, R = 319.547453638849);
          constant IdealGases.Common.DataRecord BeOHplus(name = "BeOHplus", MM = 0.0260189734, Hf = 29208835.7336958, H0 = 367074.590268039, Tlimit = 1000, alow = {71061.1916, -806.89816,5.60310781,0.00493207086, -7.72284716e-06,5.91088981e-09, -1.695201901e-12}, blow = {94407.8242, -10.88099852}, ahigh = {813487.439, -2814.653683,8.38114501, -6.94207328e-05, -2.369535806e-08,5.67810638e-12, -3.58325356e-16}, bhigh = {107312.9932, -29.01093335}, R = 319.55419117343);
          constant IdealGases.Common.DataRecord Be_OH_2(name = "Be_OH_2", MM = 0.043026862, Hf =  -14848036.4661499, H0 = 353868.66000128, Tlimit = 1000, alow = {7863.08983, -739.714225,10.43303334,0.00451424953, -6.95060153e-06,5.77703004e-09, -1.727199382e-12}, blow = { -75856.7696, -33.0146661}, ahigh = {1722889.403, -5242.47719,14.86877785,3.93149836e-05, -8.92027631e-08,1.630912925e-11, -9.50537114e-16}, bhigh = { -46428.2384, -64.7680402}, R = 193.2390979384);
          constant IdealGases.Common.DataRecord BeS(name = "BeS", MM = 0.041077182, Hf = 6015390.88051366, H0 = 213746.551552636, Tlimit = 1000, alow = { -7390.57908,282.8134792,0.65520194,0.01076012495, -1.398380157e-05,8.81841889e-09, -2.110588578e-12}, blow = {27515.93294,19.80493561}, ahigh = { -16557083.61,54314.6612, -63.5185925,0.0396409199, -1.078978696e-05,1.394030343e-09, -6.93695504e-14}, bhigh = { -312155.9628,480.3454935}, R = 202.410963829018);
          constant IdealGases.Common.DataRecord Be2(name = "Be2", MM = 0.018024364, Hf = 35371196.3429056, H0 = 545823.308938945, Tlimit = 1000, alow = { -158087.381,2035.567774, -3.69158063,0.01028818435, -9.60385451e-06,4.70751942e-09, -9.37468879e-13}, blow = {65269.6837,48.7932499}, ahigh = {101003.4484,141.4207488,2.292070182,0.0001543331824, -5.801043e-08,1.003129697e-11, -5.68281834e-16}, bhigh = {75501.3973,12.38758616}, R = 461.290728482847);
          constant IdealGases.Common.DataRecord Be2CL4(name = "Be2CL4", MM = 0.159836364, Hf =  -5127775.38533096, H0 = 141653.5225989, Tlimit = 1000, alow = {121615.2589, -2000.824823,17.23384275,0.00581044828, -1.362970089e-05,1.156602362e-08, -3.5522558e-12}, blow = { -92064.7547, -61.37959774}, ahigh = { -335492.008, -231.6199283,16.17047912, -6.73440595e-05,1.472073032e-08, -1.672352806e-12,7.68877397e-17}, bhigh = { -103107.5197, -50.47009374}, R = 52.0186507746135);
          constant IdealGases.Common.DataRecord Be2F4(name = "Be2F4", MM = 0.0940179768, Hf =  -18418816.2832281, H0 = 210965.058758848, Tlimit = 1000, alow = { -6327.70187, -69.0253372,5.47093723,0.03097341347, -4.09431006e-05,2.668113685e-08, -6.94290956e-12}, blow = { -210597.9269, -0.2073546195}, ahigh = { -368183.495, -943.919148,16.70099855, -0.0002797974015,6.17806172e-08, -7.08308019e-12,3.28279104e-16}, bhigh = { -208904.5725, -60.9266576}, R = 88.4349172678645);
          constant IdealGases.Common.DataRecord Be2O(name = "Be2O", MM = 0.034023764, Hf =  -1088468.72438922, H0 = 328899.118862922, Tlimit = 1000, alow = { -426.367357, -115.9556261,4.08267777,0.01015885064, -1.326015538e-05,8.48895536e-09, -2.168039077e-12}, blow = { -5362.25812,1.253737886}, ahigh = { -141264.6378, -349.864459,7.76093268, -0.0001044953518,2.313342334e-08, -2.657722263e-12,1.233820439e-16}, bhigh = { -5172.38549, -18.54887619}, R = 244.372492120507);
          constant IdealGases.Common.DataRecord Be2OF2(name = "Be2OF2", MM = 0.0720205704, Hf =  -16725415.9930952, H0 = 230279.68131727, Tlimit = 1000, alow = { -27834.91921,178.2881461,4.09425136,0.02270142496, -2.672688189e-05,1.572575618e-08, -3.73607058e-12}, blow = { -148008.1903,7.35349477}, ahigh = { -228182.6506, -1130.355396,13.83400563, -0.000331608064,7.30492101e-08, -8.36270438e-12,3.87218498e-16}, bhigh = { -143139.0587, -47.3915023}, R = 115.445794914171);
          constant IdealGases.Common.DataRecord Be2O2(name = "Be2O2", MM = 0.050023164, Hf =  -8228893.89803492, H0 = 218477.423779112, Tlimit = 1000, alow = { -30317.1953,1163.353177, -8.73315595,0.0537200293, -7.27943919e-05,4.89492825e-08, -1.312147056e-11}, blow = { -55469.3688,69.5328264}, ahigh = { -396057.664, -907.494883,10.67160422, -0.0002670905218,5.87762999e-08, -6.71866409e-12,3.105949075e-16}, bhigh = { -48611.1433, -36.1654689}, R = 166.212437102139);
          constant IdealGases.Common.DataRecord Be3O3(name = "Be3O3", MM = 0.075034746, Hf =  -13643287.7110026, H0 = 201583.050604316, Tlimit = 1000, alow = { -27380.72346,874.38326, -6.59960612,0.0653302114, -8.76230574e-05,5.81724479e-08, -1.5406005e-11}, blow = { -128467.9012,58.3720312}, ahigh = { -589080.936, -1392.823588,17.03450699, -0.000412733074,9.10752915e-08, -1.043465231e-11,4.83305377e-16}, bhigh = { -121904.5534, -70.0061451}, R = 110.808291401426);
          constant IdealGases.Common.DataRecord Be4O4(name = "Be4O4", MM = 0.100046328, Hf =  -16485308.5062752, H0 = 158265.668681014, Tlimit = 1000, alow = { -68932.5564,1687.565069, -14.79616594,0.1007319774, -0.0001291057622,8.24313676e-08, -2.111283987e-11}, blow = { -207287.7151,100.2507262}, ahigh = { -884133.846, -2726.394165,24.02941774, -0.000811467546,1.794255972e-07, -2.059359498e-11,9.55290533e-16}, bhigh = { -192371.0587, -113.662075}, R = 83.1062185510697);
          constant IdealGases.Common.DataRecord Br(name = "Br", MM = 0.079904, Hf = 1400055.0660793, H0 = 77560.9231077293, Tlimit = 1000, alow = { -3700.29391,61.4521542,2.092120721,0.00137681887, -2.445566658e-06,2.050975161e-09, -5.144249091e-13}, blow = {12425.08647,8.9961662}, ahigh = { -4789717.4,16920.51999, -20.24085357,0.01395620355, -3.65623056e-06,4.489781e-10, -2.122507526e-14}, bhigh = { -92070.5496,166.1695929}, R = 104.055766920304);
          constant IdealGases.Common.DataRecord Brplus(name = "Brplus", MM = 0.0799034514, Hf = 15743087.5883291, H0 = 77561.5307150549, Tlimit = 1000, alow = {39811.8742, -475.158147,4.73457898, -0.00516932479,5.90585752e-06, -2.885129283e-09,5.05489429e-13}, blow = {152905.2046, -5.76928322}, ahigh = {1741149.829, -5455.46567,8.47066335, -0.002683465574,6.59045456e-07, -7.9537703e-11,3.735898818e-15}, bhigh = {185178.1643, -36.3790903}, R = 104.056481344935);
          constant IdealGases.Common.DataRecord Brminus(name = "Brminus", MM = 0.0799045486, Hf =  -2740776.03637198, H0 = 77560.3905983395, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = { -27084.92743,5.41955617}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = { -27084.92743,5.41955617}, R = 104.055052505484);
          constant IdealGases.Common.DataRecord BrCL(name = "BrCL", MM = 0.115357, Hf = 128202.995917023, H0 = 81547.8297805941, Tlimit = 1000, alow = {16532.91583, -357.876928,5.70062216, -0.002201257188,2.415146969e-06, -1.371032976e-09,3.22780342e-13}, blow = {2252.619279, -4.155659669}, ahigh = { -128486.3067, -1138.379626,7.9640408, -0.00343805667,1.524999514e-06, -2.761469114e-10,1.694450407e-14}, bhigh = {5938.26426, -19.25044389}, R = 72.0760075244675);
          constant IdealGases.Common.DataRecord BrF(name = "BrF", MM = 0.0989024032, Hf =  -595045.652035278, H0 = 91212.2224346516, Tlimit = 1000, alow = {38361.4319, -518.461197,5.45832705, -0.00056568431, -4.12131197e-07,7.85991348e-10, -3.03616152e-13}, blow = { -5595.53991, -4.9011295}, ahigh = {3771388.47, -12674.9343,20.8055593, -0.0102635936,3.30820359e-06, -4.90319225e-10,2.64827508e-14}, bhigh = {70626.0972, -113.02536}, R = 84.0674415482758);
          constant IdealGases.Common.DataRecord BrF3(name = "BrF3", MM = 0.1368992096, Hf =  -1867067.02505315, H0 = 107463.147836903, Tlimit = 1000, alow = {111645.5127, -1959.42433,15.26220126, -0.0079993623,6.98203026e-06, -3.24275796e-09,6.16475146e-13}, blow = { -23453.41551, -55.226336}, ahigh = { -188653.224, -82.4523297,10.06180694, -2.477319092e-05,5.47857686e-09, -6.28265526e-13,2.910628214e-17}, bhigh = { -33868.0037, -22.98937369}, R = 60.7342586147408);
          constant IdealGases.Common.DataRecord BrF5(name = "BrF5", MM = 0.174896016, Hf =  -2451742.52568452, H0 = 109633.961016013, Tlimit = 1000, alow = {221136.6805, -4024.80922,27.34900212, -0.01832254979,1.721754649e-05, -8.75350863e-09,1.860196959e-12}, blow = { -35374.4767, -124.4296466}, ahigh = { -375764.764, -157.5896472,16.11869838, -4.7764978e-05,1.059755365e-08, -1.218527723e-12,5.65753545e-17}, bhigh = { -56672.668, -55.4083931}, R = 47.539516280348);
          constant IdealGases.Common.DataRecord BrO(name = "BrO", MM = 0.0959034, Hf = 1311736.60162205, H0 = 94480.4876573719, Tlimit = 1000, alow = {14554.31116,122.5152439, -0.3058728672,0.02117309202, -3.49426119e-05,2.622639719e-08, -7.50805954e-12}, blow = {13891.49932,25.27867438}, ahigh = { -2392612.795,6932.79313, -2.513183892,0.00344654456, -7.56953684e-07,6.51429134e-11, -1.715768736e-15}, bhigh = { -30844.99064,53.6104836}, R = 86.6963215068496);
          constant IdealGases.Common.DataRecord OBrO(name = "OBrO", MM = 0.1119028, Hf = 1357915.31579192, H0 = 101826.138398682, Tlimit = 1000, alow = {34702.1323, -340.134182,3.93225162,0.01215938586, -1.901501702e-05,1.398631495e-08, -3.99289976e-12}, blow = {18759.55788,6.36123518}, ahigh = { -162450.466, -145.3689372,7.10582213, -4.14136207e-05,8.98183326e-09, -1.013712869e-12,4.63513314e-17}, bhigh = {16500.75386, -9.11181792}, R = 74.300839657274);
          constant IdealGases.Common.DataRecord BrOO(name = "BrOO", MM = 0.1119028, Hf = 965123.303438341, H0 = 114837.698431139, Tlimit = 1000, alow = { -44176.199,495.963716,3.23988495,0.00594150211, -3.40269677e-06, -3.44477816e-11,4.81195795e-13}, blow = {8815.22198,16.07446878}, ahigh = {6172.48348, -658.504966,7.48395533, -0.0001920530746,4.22694617e-08, -4.83739261e-12,2.23974132e-16}, bhigh = {14603.5347, -9.85038024}, R = 74.300839657274);
          constant IdealGases.Common.DataRecord BrO3(name = "BrO3", MM = 0.1279022, Hf = 1726480.74075348, H0 = 102428.58215105, Tlimit = 1000, alow = {106390.1881, -1501.928813,9.70365743,0.00813300076, -1.531857845e-05,1.199117619e-08, -3.51274787e-12}, blow = {32331.3599, -27.3512543}, ahigh = { -282285.1113, -235.8973461,10.17525279, -6.97845105e-05,1.535701661e-08, -1.75445495e-12,8.10410015e-17}, bhigh = {24011.4014, -25.88197197}, R = 65.0064815147824);
          constant IdealGases.Common.DataRecord Br2(name = "Br2", MM = 0.159808, Hf = 193419.603524229, H0 = 60855.0072587105, Tlimit = 1000, alow = {7497.04754, -235.0884557,5.49193432, -0.002227573303,2.932401703e-06, -1.954889514e-09,5.31230789e-13}, blow = {3521.47505, -1.96415157}, ahigh = { -4311698.57,11112.68634, -5.55577561,0.00363051659, -2.754164226e-07, -6.21750676e-11,7.37534162e-15}, bhigh = { -70365.8416,78.7847802}, R = 52.0278834601522);
          constant IdealGases.Common.DataRecord BrBrO(name = "BrBrO", MM = 0.1758074, Hf = 955591.175343017, H0 = 74726.6497314675, Tlimit = 1000, alow = {16174.98037, -246.4406776,5.91285845,0.00498035428, -8.09995825e-06,6.07277626e-09, -1.753715313e-12}, blow = {19740.12093,2.009704963}, ahigh = { -85371.0175, -64.8704607,7.04715935, -1.843088186e-05,3.99226786e-09, -4.50068795e-13,2.055867395e-17}, bhigh = {18215.66655, -3.162186125}, R = 47.2930718502179);
          constant IdealGases.Common.DataRecord BrOBr(name = "BrOBr", MM = 0.1758074, Hf = 612255.263430322, H0 = 70526.3714724181, Tlimit = 1000, alow = {64050.6891, -1062.722566,9.90786629, -0.00447049247,3.91370107e-06, -1.805444867e-09,3.36613255e-13}, blow = {16429.21537, -23.50319399}, ahigh = { -96866.6025, -39.5739527,7.02943438, -1.171881908e-05,2.576958549e-09, -2.941186078e-13,1.357198784e-17}, bhigh = {10769.42321, -5.69777938}, R = 47.2930718502179);
          constant IdealGases.Common.DataRecord C(name = "C", MM = 0.0120107, Hf = 59670127.4696729, H0 = 544172.696012722, Tlimit = 1000, alow = {649.503147, -0.964901086,2.504675479, -1.281448025e-05,1.980133654e-08, -1.606144025e-11,5.314483411e-15}, blow = {85457.6311,4.747924288}, ahigh = { -128913.6472,171.9528572,2.646044387, -0.000335306895,1.74209274e-07, -2.902817829e-11,1.642182385e-15}, bhigh = {84105.9785,4.130047418}, R = 692.255405596676);
          constant IdealGases.Common.DataRecord Cplus(name = "Cplus", MM = 0.0120101514, Hf = 150659589.686771, H0 = 553638.482858759, Tlimit = 1000, alow = {2258.535929, -1.574575687,2.50363773, -5.20287837e-06,4.51690839e-09, -2.181431053e-12,4.495047033e-16}, blow = {216895.1913,4.345699505}, ahigh = {12551.12551, -34.1187467,2.543383218, -2.805120849e-05,9.75164197e-09, -1.736855394e-12,1.246191931e-16}, bhigh = {217100.1786,4.063913515}, R = 692.287026456636);
          constant IdealGases.Common.DataRecord Cminus(name = "Cminus", MM = 0.0120112486, Hf = 48980273.0417219, H0 = 517751.00217308, Tlimit = 1000, alow = {4.67129153, -0.001986169369,2.500008638, -1.976750928e-08,2.478947477e-11, -1.610664044e-14,4.23650681e-18}, blow = {70012.1855,4.879570141}, ahigh = {4.25317572,0.000577818648,2.499999424,2.836136231e-10, -7.32725342e-14,9.47850781e-18, -4.83048732e-22}, bhigh = {70012.1717,4.879624211}, R = 692.22378762521);
          constant IdealGases.Common.DataRecord CBr(name = "CBr", MM = 0.0919147, Hf = 5335727.75627838, H0 = 104609.121283103, Tlimit = 1000, alow = {4324.44688, -159.6366559,4.85785659, -0.000438943916,4.40886363e-07, -2.281330618e-10,5.44561822e-14}, blow = {58476.7725,0.1351169646}, ahigh = {1021862.102, -3243.48436,8.38822656, -0.002242155508,6.98835537e-07, -9.71913437e-11,4.71697624e-15}, bhigh = {78059.2406, -25.18386915}, R = 90.458566475221);
          constant IdealGases.Common.DataRecord CBr2(name = "CBr2", MM = 0.1718187, Hf = 1959177.4294649, H0 = 70986.5631622169, Tlimit = 1000, alow = {72481.9784, -1128.26123,9.82230338, -0.0038727481,2.904819596e-06, -1.061380159e-09,1.297474642e-13}, blow = {44377.716, -23.61121795}, ahigh = { -744143.141,421.542783,8.79679774, -0.002726562507,1.336438546e-06, -2.304854483e-10,1.337493953e-14}, bhigh = {33646.1814, -15.99565135}, R = 48.3909609373136);
          constant IdealGases.Common.DataRecord CBr3(name = "CBr3", MM = 0.2517227, Hf = 933566.976677113, H0 = 61903.2570364135, Tlimit = 1000, alow = {85635.1626, -1275.764123,11.17190031,0.002341705749, -6.54067141e-06,5.73671714e-09, -1.777823046e-12}, blow = {32432.1589, -27.66468801}, ahigh = { -192649.8949, -132.4072178,10.09808665, -3.89504082e-05,8.55044501e-09, -9.74749368e-13,4.49424559e-17}, bhigh = {25415.76129, -18.66868947}, R = 33.0302829264107);
          constant IdealGases.Common.DataRecord CBr4(name = "CBr4", MM = 0.3316267, Hf = 239727.380213957, H0 = 61434.4803961804, Tlimit = 1000, alow = {102308.3739, -1735.600709,16.51317914, -0.003147664747,2.924135486e-07,1.360605681e-09, -6.43175865e-13}, blow = {15005.02543, -55.3647676}, ahigh = { -205978.5072, -104.8517002,13.0777255, -3.086857556e-05,6.77530981e-09, -7.72183754e-13,3.55921824e-17}, bhigh = {5615.89394, -32.8378763}, R = 25.0717810116013);
          constant IdealGases.Common.DataRecord CCL(name = "CCL", MM = 0.0474637, Hf = 9114566.66884377, H0 = 197943.122849673, Tlimit = 1000, alow = {17070.70049, -14.63803497,2.334108251,0.00720209913, -1.034120045e-05,7.2281067e-09, -1.985123755e-12}, blow = {51233.503,12.00820386}, ahigh = {590050.763, -2075.31146,6.88001723, -0.001294796228,3.88313798e-07, -5.00698163e-11,2.218229619e-15}, bhigh = {63589.2205, -16.10664472}, R = 175.175386663914);
          constant IdealGases.Common.DataRecord CCL2(name = "CCL2", MM = 0.0829167, Hf = 2688724.31006058, H0 = 137761.85014599, Tlimit = 1000, alow = {75096.2315, -1034.728559,8.06170454,0.001487134213, -4.56481711e-06,3.95488581e-09, -1.166197091e-12}, blow = {30524.09293, -17.39691692}, ahigh = { -6437455.17,19584.13353, -16.04038625,0.01238833906, -3.013367274e-06,3.47346892e-10, -1.553272793e-14}, bhigh = { -99612.4216,155.6949315}, R = 100.274998884423);
          constant IdealGases.Common.DataRecord CCL2Br2(name = "CCL2Br2", MM = 0.2427247, Hf = 41198.9385505472, H0 = 77008.6933880235, Tlimit = 1000, alow = {101849.5619, -1740.854039,15.21338304,0.00112825651, -5.87540549e-06,5.65988697e-09, -1.821576512e-12}, blow = {6918.57219, -50.15385819}, ahigh = { -255334.7012, -159.8005477,13.1186093, -4.71744387e-05,1.036912808e-08, -1.183321687e-12,5.46061269e-17}, bhigh = { -2588.63698, -34.61705319}, R = 34.2547421008245);
          constant IdealGases.Common.DataRecord CCL3(name = "CCL3", MM = 0.1183697, Hf = 600897.020098894, H0 = 121652.754040941, Tlimit = 1000, alow = {34144.7165, -554.796973,6.84554439,0.01242006495, -2.036896881e-05,1.554979993e-08, -4.56894011e-12}, blow = {9388.56882, -7.143598465}, ahigh = { -542757.254,680.902062,9.04772362,0.0001324277766,6.30126304e-08, -2.6566392e-11,2.105668053e-15}, bhigh = {386.621615, -15.28745849}, R = 70.2415567497425);
          constant IdealGases.Common.DataRecord CCL3Br(name = "CCL3Br", MM = 0.1982737, Hf =  -216871.93006435, H0 = 90705.6709992298, Tlimit = 1000, alow = {104966.8463, -1793.828482,14.89250892,0.00239712831, -7.78961547e-06,7.01661711e-09, -2.196069746e-12}, blow = {910.134931, -50.58348139}, ahigh = { -277232.4186, -184.0310333,13.13673812, -5.44391001e-05,1.197658923e-08, -1.367818632e-12,6.31616479e-17}, bhigh = { -8895.85769, -36.69528209}, R = 41.9343160489767);
          constant IdealGases.Common.DataRecord CCL4(name = "CCL4", MM = 0.1538227, Hf =  -621494.746874161, H0 = 111547.333390975, Tlimit = 1000, alow = {109338.5626, -1861.731846,14.49632467,0.00394948516, -1.010805379e-05,8.64551417e-09, -2.642368852e-12}, blow = { -4948.00623, -51.8028475}, ahigh = { -304307.8255, -216.8170491,13.16132386, -6.43112489e-05,1.416482497e-08, -1.61934332e-12,7.48397435e-17}, bhigh = { -15123.2007, -39.968443}, R = 54.0523082743964);
          constant IdealGases.Common.DataRecord CF(name = "CF", MM = 0.0310091032, Hf = 7813834.48715795, H0 = 292337.025728625, Tlimit = 1000, alow = { -45827.8668,797.263589, -1.364936319,0.01312359375, -1.457012892e-05,8.25327668e-09, -1.896351845e-12}, blow = {24382.5962,32.4806528}, ahigh = { -132980.71, -121.4340891,4.45241362,0.0001293786948, -3.67055059e-08,6.59021239e-12, -3.73126161e-16}, bhigh = {28159.34108, -0.664046873}, R = 268.130037375605);
          constant IdealGases.Common.DataRecord CFplus(name = "CFplus", MM = 0.0310085546, Hf = 36943485.8792161, H0 = 280487.082103466, Tlimit = 1000, alow = { -59752.132,927.686365, -1.866581982,0.01384340912, -1.51342886e-05,8.51855634e-09, -1.959577655e-12}, blow = {132351.941,34.1201319}, ahigh = { -80537.6278, -339.027509,4.7083735, -3.96314831e-05,2.497996218e-08, -5.23679043e-12,5.31394913e-16}, bhigh = {138133.7068, -3.92051473}, R = 268.134781103277);
          constant IdealGases.Common.DataRecord CFBr3(name = "CFBr3", MM = 0.2707211032, Hf =  -443260.605034355, H0 = 67387.0776395388, Tlimit = 1000, alow = {51492.4428, -936.479664,10.44260886,0.01205370047, -1.876290291e-05,1.344488618e-08, -3.73054576e-12}, blow = { -12432.46293, -23.63948551}, ahigh = { -272915.8956, -332.816465,13.24822519, -9.92707754e-05,2.193640054e-08, -2.515424945e-12,1.165680878e-16}, bhigh = { -17301.85479, -36.2681843}, R = 30.7123157438434);
          constant IdealGases.Common.DataRecord CFCL(name = "CFCL", MM = 0.0664621032, Hf = 388879.869212445, H0 = 164039.076030925, Tlimit = 1000, alow = {15419.97945, -82.7403981,2.501701822,0.01390738086, -1.8957347e-05,1.261934581e-08, -3.33345382e-12}, blow = {2411.755114,13.31517094}, ahigh = { -268163.0077, -2.328091539,6.83463383,0.0001565270633, -5.96775516e-08,9.82092446e-12, -5.05889721e-16}, bhigh = {301.7669604, -9.005085129}, R = 125.100946248719);
          constant IdealGases.Common.DataRecord CFCLBr2(name = "CFCLBr2", MM = 0.2262701032, Hf =  -773411.942298526, H0 = 77332.4568846531, Tlimit = 1000, alow = {58987.9974, -1050.714861,10.39349401,0.01273690622, -1.999097478e-05,1.43820423e-08, -4.00013035e-12}, blow = { -18377.99602, -24.17479131}, ahigh = { -296081.1596, -353.12762,13.26346169, -0.0001053860298,2.32908508e-08, -2.670979639e-12,1.237844667e-16}, bhigh = { -23875.94306, -36.84977271}, R = 36.7457825068955);
          constant IdealGases.Common.DataRecord CFCL2(name = "CFCL2", MM = 0.1019151032, Hf =  -1030269.28004916, H0 = 129685.92078117, Tlimit = 1000, alow = {26113.98792, -335.35659,4.09467308,0.01960244711, -2.762775193e-05,1.880124302e-08, -5.0474103e-12}, blow = { -12512.98871,6.870882561}, ahigh = { -265311.8323, -446.972417,10.33334881, -0.0001333673368,2.948755634e-08, -3.3833062e-12,1.568746348e-16}, bhigh = { -13931.60489, -25.56817972}, R = 81.5823341088468);
          constant IdealGases.Common.DataRecord CFCL2Br(name = "CFCL2Br", MM = 0.1818191032, Hf =  -1292493.45016019, H0 = 92017.6851911785, Tlimit = 1000, alow = {65299.5922, -1135.235485,10.09526872,0.01401380147, -2.191705096e-05,1.573404132e-08, -4.36934034e-12}, blow = { -25044.88145, -24.51093103}, ahigh = { -321545.505, -388.412962,13.28990796, -0.0001160048581,2.564498787e-08, -2.941639768e-12,1.363542881e-16}, bhigh = { -30974.21144, -38.71356695}, R = 45.7293642618737);
          constant IdealGases.Common.DataRecord CFCL3(name = "CFCL3", MM = 0.1373681032, Hf =  -2065253.8208739, H0 = 116938.238395942, Tlimit = 1000, alow = {74001.74, -1252.282466,10.01327229,0.01475107601, -2.315635251e-05,1.664207025e-08, -4.62292996e-12}, blow = { -30205.08601, -27.08387087}, ahigh = { -345046.478, -417.782145,13.31196736, -0.0001248759531,2.761391353e-08, -3.16820764e-12,1.468834601e-16}, bhigh = { -36740.6719, -41.56886057}, R = 60.5269477143075);
          constant IdealGases.Common.DataRecord CF2(name = "CF2", MM = 0.0500075064, Hf =  -3731439.80640474, H0 = 206987.165430828, Tlimit = 1000, alow = { -37970.2627,873.180003, -3.46019157,0.02746253741, -3.4746219e-05,2.204470693e-08, -5.62175085e-12}, blow = { -27467.97157,44.5677807}, ahigh = { -108642.8547, -585.498914,7.01864895,0.000392918615, -2.603822675e-07,6.14219639e-11, -4.17283326e-15}, bhigh = { -21529.45157, -13.56143285}, R = 166.26447904629);
          constant IdealGases.Common.DataRecord CF2plus(name = "CF2plus", MM = 0.0500069578, Hf = 18984184.2168611, H0 = 206811.200980516, Tlimit = 1000, alow = { -28923.73827,604.935461, -0.972186666,0.01808478341, -1.980065782e-05,1.101640055e-08, -2.500457119e-12}, blow = {110275.567,32.4759147}, ahigh = { -64889.6607, -1064.283593,7.7857021, -0.0003158347626,7.13080408e-08, -8.53318359e-12,4.25163882e-16}, bhigh = {117844.6465, -18.19424707}, R = 166.266303046333);
          constant IdealGases.Common.DataRecord CF2Br2(name = "CF2Br2", MM = 0.2098155064, Hf =  -1811114.94817525, H0 = 77591.9629551269, Tlimit = 1000, alow = {12614.04028, -281.5725114,5.09803213,0.02502548939, -3.43885319e-05,2.297615553e-08, -6.08016143e-12}, blow = { -46427.6375,3.094167118}, ahigh = { -332766.93, -618.435249,13.46170049, -0.0001849172548,4.09259134e-08, -4.69981043e-12,2.180811799e-16}, bhigh = { -47151.8661, -41.0536924}, R = 39.6275382246963);
          constant IdealGases.Common.DataRecord CF2CL(name = "CF2CL", MM = 0.0854605064, Hf =  -3217860.64211761, H0 = 145473.898104587, Tlimit = 1000, alow = {8914.55835, -17.68816959,1.607015025,0.02473626554, -3.27073894e-05,2.12635183e-08, -5.50983111e-12}, blow = { -34273.0625,19.2968069}, ahigh = { -280234.6654, -686.031485,10.51143635, -0.0002046954963,4.52889718e-08, -5.20022903e-12,2.41297145e-16}, bhigh = { -33081.7948, -28.74899785}, R = 97.2902262137777);
          constant IdealGases.Common.DataRecord CF2CLBr(name = "CF2CLBr", MM = 0.1653645064, Hf =  -2630552.40492648, H0 = 93903.1194665121, Tlimit = 1000, alow = {26966.99365, -467.883227,5.24480942,0.02530121349, -3.50844352e-05,2.354353958e-08, -6.24616113e-12}, blow = { -51983.8273,0.8532887803}, ahigh = { -359282.884, -653.928359,13.48831524, -0.0001956099469,4.32977597e-08, -4.97264437e-12,2.30757505e-16}, bhigh = { -53651.4117, -42.24913908}, R = 50.2796650926296);
          constant IdealGases.Common.DataRecord CF2CL2(name = "CF2CL2", MM = 0.1209135064, Hf =  -4059099.88563527, H0 = 123072.222806699, Tlimit = 1000, alow = {38412.5875, -613.949982,5.26966719,0.02574105783, -3.58737648e-05,2.41160013e-08, -6.4022369e-12}, blow = { -57845.4133, -1.957454856}, ahigh = { -382744.855, -693.37748,13.51791274, -0.0002075114435,4.59402161e-08, -5.2768746e-12,2.449037044e-16}, bhigh = { -60215.2777, -44.79580046}, R = 68.7637985825544);
          constant IdealGases.Common.DataRecord CF3(name = "CF3", MM = 0.0690059096, Hf =  -6773332.93205369, H0 = 166528.780891543, Tlimit = 1000, alow = { -29783.07106,715.367883, -3.49818538,0.0359545799, -4.50797443e-05,2.82180845e-08, -7.09804702e-12}, blow = { -60599.9703,45.0259264}, ahigh = { -299730.5557, -1046.989457,10.77923191, -0.0003116087076,6.89135143e-08, -7.91122564e-12,3.67059302e-16}, bhigh = { -54253.044, -34.1703879}, R = 120.489274733073);
          constant IdealGases.Common.DataRecord CF3plus(name = "CF3plus", MM = 0.069005361, Hf = 6138906.80174835, H0 = 167251.932788237, Tlimit = 1000, alow = { -35874.6354,374.35067,0.778741393,0.01929435768, -1.89281577e-05,9.36390876e-09, -1.885759656e-12}, blow = {47755.8172,22.24044005}, ahigh = { -29109.19531, -1996.658183,11.44279678, -0.000565339304,1.232297113e-07, -1.399758046e-11,6.44301795e-16}, bhigh = {59023.458, -40.8034999}, R = 120.490232635693);
          constant IdealGases.Common.DataRecord CF3Br(name = "CF3Br", MM = 0.1489099096, Hf =  -4356996.80258217, H0 = 96998.4337429213, Tlimit = 1000, alow = { -5439.48901,124.361519,0.923194195,0.0348104438, -4.54964549e-05,2.932836552e-08, -7.54800036e-12}, blow = { -80233.9676,22.32999096}, ahigh = { -383408.851, -971.90933,13.72477396, -0.0002901710612,6.4218922e-08, -7.37568877e-12,3.42315425e-16}, bhigh = { -77653.0631, -47.1736721}, R = 55.83558557207);
          constant IdealGases.Common.DataRecord CF3CL(name = "CF3CL", MM = 0.1044589096, Hf =  -6741406.76651291, H0 = 132020.82094106, Tlimit = 1000, alow = {14994.09978, -136.5768572,1.45291237,0.0340476507, -4.47377764e-05,2.888115112e-08, -7.43417133e-12}, blow = { -85471.6664,17.27327095}, ahigh = { -406604.212, -1022.626507,13.76252628, -0.0003052722979,6.75596604e-08, -7.75931525e-12,3.60119229e-16}, bhigh = { -84105.5937, -49.13411939}, R = 79.5956231195429);
          constant IdealGases.Common.DataRecord CF4(name = "CF4", MM = 0.0880043128, Hf =  -10603116.7145254, H0 = 144655.558289866, Tlimit = 1000, alow = {9817.4585,116.3343483, -1.288338636,0.0395956691, -4.99624421e-05,3.125339346e-08, -7.84170032e-12}, blow = { -113850.2297,29.38656548}, ahigh = { -416445.678, -1414.797167,14.05124837, -0.000419909386,9.27892161e-08, -1.06457583e-11,4.93709968e-16}, bhigh = { -109469.1149, -54.87105}, R = 94.4780060824474);
          constant IdealGases.Common.DataRecord CHplus(name = "CHplus", MM = 0.0130180914, Hf = 125254229.202908, H0 = 662777.95529996, Tlimit = 1000, alow = {30196.07105, -461.814131,6.22564119, -0.0077757118,1.094489485e-05, -6.67548791e-09,1.565232409e-12}, blow = {197249.1928, -14.31512811}, ahigh = { -7102094.67,18283.54883, -13.12691402,0.00619171736, -2.909421253e-07, -1.134243575e-10,1.105962085e-14}, bhigh = {75412.9604,124.3984829}, R = 638.68594439274);
          constant IdealGases.Common.DataRecord CHBr3(name = "CHBr3", MM = 0.25273064, Hf = 66236.5275536041, H0 = 62938.9495472334, Tlimit = 1000, alow = {43465.7686, -499.963442,5.62022211,0.02079346321, -2.921336328e-05,2.066942448e-08, -5.78122091e-12}, blow = {2627.830062,1.247904015}, ahigh = {627423.495, -3378.7111,14.87852612, -0.000593274527,1.082421577e-07, -1.061012665e-11,4.31915245e-16}, bhigh = {18772.34659, -53.1190965}, R = 32.8985515962766);
          constant IdealGases.Common.DataRecord CHCL(name = "CHCL", MM = 0.04847164, Hf = 6129357.29015977, H0 = 210441.466391482, Tlimit = 1000, alow = { -269991.2334,4351.19973, -22.75911813,0.0755062176, -9.07199797e-05,5.25697186e-08, -1.189769182e-11}, blow = {14168.6268,152.0980812}, ahigh = { -954806.19,2174.413794,4.86764537,0.000832164186, -1.536948638e-07,1.529236537e-11, -6.59615936e-16}, bhigh = {18801.2181,2.674761385}, R = 171.532714799829);
          constant IdealGases.Common.DataRecord CHCLBr2(name = "CHCLBr2", MM = 0.20827964, Hf = 48012.3741331606, H0 = 73416.8543790454, Tlimit = 1000, alow = {38153.8025, -408.417511,4.63924402,0.02325990646, -3.22298886e-05,2.254671151e-08, -6.25650774e-12}, blow = {1483.809044,6.175235281}, ahigh = {604286.53, -3438.24101,14.94162631, -0.000622842797,1.153724426e-07, -1.147291377e-11,4.73348286e-16}, bhigh = {18224.60958, -54.07258924}, R = 39.9197540383688);
          constant IdealGases.Common.DataRecord CHCL2(name = "CHCL2", MM = 0.08392464, Hf = 1141500.27929819, H0 = 152517.782620217, Tlimit = 1000, alow = {56385.5613, -656.245542,6.06881802,0.0097441226, -1.275310131e-05,8.55238672e-09, -2.2315827e-12}, blow = {13304.47798, -4.533535456}, ahigh = {884939.371, -3526.96973,11.86372153, -0.000500237612,6.1338604e-08, -1.885433405e-12, -1.234221421e-16}, bhigh = {30711.36475, -40.88905858}, R = 99.0706900857722);
          constant IdealGases.Common.DataRecord CHCL2Br(name = "CHCL2Br", MM = 0.16382864, Hf =  -274677.248129509, H0 = 89910.3783074803, Tlimit = 1000, alow = {32940.2265, -327.530117,3.77323524,0.02536466143, -3.47266699e-05,2.404666618e-08, -6.62173093e-12}, blow = { -5425.56508,9.433412004}, ahigh = {592487.644, -3509.47561,14.99349593, -0.000643306257,1.198581809e-07, -1.198476716e-11,4.96994807e-16}, bhigh = {11972.51405, -56.02687781}, R = 50.7510286357746);
          constant IdealGases.Common.DataRecord CHCL3(name = "CHCL3", MM = 0.11937764, Hf =  -860295.110541639, H0 = 119833.948803143, Tlimit = 1000, alow = {33953.3329, -304.7428785,2.923672263,0.02830547858, -3.71242469e-05,2.551365915e-08, -6.98765955e-12}, blow = { -12350.63157,11.15556408}, ahigh = {613605.274, -3715.08717,15.10777247,0.0002362584336,1.297140438e-07, -1.267494791e-11,5.25902231e-16}, bhigh = {6203.31345, -59.92576539}, R = 69.6484869360795);
          constant IdealGases.Common.DataRecord CHF(name = "CHF", MM = 0.0320170432, Hf = 3398190.12393999, H0 = 311746.963567204, Tlimit = 1000, alow = { -78685.8583,1300.218466, -3.94768525,0.02114995848, -2.239962738e-05,1.283155181e-08, -2.904778136e-12}, blow = {5824.38972,47.8545377}, ahigh = {3994085.42, -7962.7562,6.75559509,0.00494983657, -2.101763812e-06,3.34823295e-10, -1.88682505e-14}, bhigh = {67061.3494, -23.81240379}, R = 259.688939670731);
          constant IdealGases.Common.DataRecord CHFBr2(name = "CHFBr2", MM = 0.1918250432, Hf =  -912289.642064838, H0 = 74857.5668768571, Tlimit = 1000, alow = { -18201.9798,436.224967, -0.373385417,0.033886314, -4.37129853e-05,2.889569295e-08, -7.68391511e-12}, blow = { -24656.11586,33.2047879}, ahigh = {593575.163, -3742.65658,15.17184031, -0.000715803903,1.360636972e-07, -1.385897355e-11,5.84420911e-16}, bhigh = { -2347.676027, -57.6578343}, R = 43.3440381990749);
          constant IdealGases.Common.DataRecord CHFCL(name = "CHFCL", MM = 0.0674700432, Hf =  -1232320.30181952, H0 = 165198.575121114, Tlimit = 1000, alow = { -71130.0851,1377.872745, -6.48461724,0.0389097249, -4.80073522e-05,3.073300019e-08, -7.98285341e-12}, blow = { -17517.93745,63.73895867}, ahigh = {662216.358, -3809.00798,12.23880469, -0.000746718275,1.43476785e-07, -1.475406631e-11,6.27378068e-16}, bhigh = {10189.87746, -46.58365563}, R = 123.23205389618);
          constant IdealGases.Common.DataRecord CHFCLBr(name = "CHFCLBr", MM = 0.1473740432, Hf =  -1560654.74628981, H0 = 93552.3359516542, Tlimit = 1000, alow = { -19592.40614,490.172327, -1.259408503,0.0361770847, -4.65424496e-05,3.065240659e-08, -8.1239036e-12}, blow = { -31399.00705,37.10450945}, ahigh = {580011.751, -3823.33785,15.22809062, -0.000737390631,1.407097155e-07, -1.438245319e-11,6.08386715e-16}, bhigh = { -8552.2499, -59.03677705}, R = 56.417479085625);
          constant IdealGases.Common.DataRecord CHFCL2(name = "CHFCL2", MM = 0.1029230432, Hf =  -2768087.60353483, H0 = 129167.984026341, Tlimit = 1000, alow = { -17349.59015,492.49132, -1.854635884,0.0378278628, -4.86334304e-05,3.19719763e-08, -8.45905799e-12}, blow = { -37887.4927,38.01530544}, ahigh = {564011.302, -3887.0833,15.27760246, -0.000757648742,1.452498792e-07, -1.490796515e-11,6.3289985e-16}, bhigh = { -14847.36185, -61.67535516}, R = 80.7833867080992);
          constant IdealGases.Common.DataRecord CHF2(name = "CHF2", MM = 0.0510154464, Hf =  -4682895.41420145, H0 = 214052.816756299, Tlimit = 1000, alow = { -146969.0117,2553.397313, -13.0262763,0.0543812846, -6.71341879e-05,4.28765173e-08, -1.110824216e-11}, blow = { -41793.7216,99.3993025}, ahigh = {552680.655, -3696.00917,12.08610573, -0.00064985881,1.12084622e-07, -9.81414701e-12,3.34919239e-16}, bhigh = { -9437.06842, -47.044162}, R = 162.979501047745);
          constant IdealGases.Common.DataRecord CHF2Br(name = "CHF2Br", MM = 0.1309194464, Hf =  -3223356.12931526, H0 = 100592.420470241, Tlimit = 1000, alow = { -77145.1016,1422.544046, -6.68439365,0.0473838915, -5.83058875e-05,3.69572047e-08, -9.49548528e-12}, blow = { -58785.0093,66.0863909}, ahigh = {576770.684, -4177.71608,15.50150581, -0.000849014729,1.657177491e-07, -1.727828054e-11,7.43562723e-16}, bhigh = { -29661.47348, -63.4938014}, R = 63.5083039886731);
          constant IdealGases.Common.DataRecord CHF2CL(name = "CHF2CL", MM = 0.0864684464, Hf =  -5583539.66216282, H0 = 143029.006705965, Tlimit = 1000, alow = { -72405.7941,1365.510973, -7.15322208,0.0485498797, -5.94878167e-05,3.75096619e-08, -9.58952442e-12}, blow = { -65659.328,66.5721223}, ahigh = {562949.373, -4298.67305,15.58191398, -0.000878927848,1.720248689e-07, -1.797910869e-11,7.7533917e-16}, bhigh = { -36340.6716, -66.1126148}, R = 96.1561395649176);
          constant IdealGases.Common.DataRecord CHF3(name = "CHF3", MM = 0.0700138496, Hf =  -9902326.5248366, H0 = 165208.541825416, Tlimit = 1000, alow = { -109513.226,2042.273879, -11.66213079,0.0578580777, -6.85713506e-05,4.21211838e-08, -1.053196888e-11}, blow = { -93954.7005,89.3592065}, ahigh = {568523.202, -4728.3617,15.86728516, -0.000738234865,1.970841706e-07, -2.062115571e-11,8.97059964e-16}, bhigh = { -59231.9637, -71.6127322}, R = 118.754675646345);
          constant IdealGases.Common.DataRecord CHI3(name = "CHI3", MM = 0.39373205, Hf = 535576.415483576, H0 = 43574.2708778724, Tlimit = 1000, alow = {52529.2359, -724.216647,8.01862334,0.01470995487, -2.163843469e-05,1.594571452e-08, -4.60143796e-12}, blow = {26781.86915, -8.60505484}, ahigh = {629420.543, -3184.06157,14.79622178, -0.00057495555,1.062025541e-07, -1.052804696e-11,4.32972002e-16}, bhigh = {41036.098, -49.0135739}, R = 21.1170820358668);
          constant IdealGases.Common.DataRecord CH2(name = "CH2", MM = 0.01402658, Hf = 27830341.8937474, H0 = 0, Tlimit = 1000, alow = {32189.2173, -287.7601815,4.20358382,0.00345540596, -6.74619334e-06,7.65457164e-09, -2.870328419e-12}, blow = {0,47336.2471}, ahigh = {10027.417,2550418.031, -7971.62539,12.28924487, -0.001699122922,2.991728605e-07, -2.767007492e-11}, bhigh = {1.05134174e-15,0}, R = 592.765449596409);
          constant IdealGases.Common.DataRecord CH2Br2(name = "CH2Br2", MM = 0.17383458, Hf =  -84965.8336103208, H0 = 72565.5792995847, Tlimit = 1000, alow = {4797.30801,361.385924, -1.819592338,0.0347704834, -4.49854618e-05,3.068623685e-08, -8.43484634e-12}, blow = { -4481.49706,38.2712359}, ahigh = {1528284.441, -6673.41495,16.70213316, -0.001166351237,2.122634493e-07, -2.07547224e-11,8.4285777e-16}, bhigh = {36007.0664, -74.4806549}, R = 47.8297931286169);
          constant IdealGases.Common.DataRecord CH2CL(name = "CH2CL", MM = 0.04947958, Hf = 2409074.61219356, H0 = 221908.108355002, Tlimit = 1000, alow = { -31885.8563,633.316321, -1.164065495,0.0216058608, -2.545462163e-05,1.693887757e-08, -4.6600786e-12}, blow = {10201.4254,32.30835289}, ahigh = {1662438.334, -6441.12572,13.59753722, -0.00114053681,2.087760159e-07, -2.052347186e-11,8.37577227e-16}, bhigh = {52129.0612, -61.48586271}, R = 168.038451417736);
          constant IdealGases.Common.DataRecord CH2CLBr(name = "CH2CLBr", MM = 0.12938358, Hf =  -347803.021063415, H0 = 94221.7474582169, Tlimit = 1000, alow = { -13079.48755,645.586038, -3.57734991,0.0383368723, -4.86425205e-05,3.26162233e-08, -8.85287739e-12}, blow = { -9402.27253,47.48806758}, ahigh = {1525016.309, -6823.33416,16.82769102, -0.001219735119,2.244964495e-07, -2.219063248e-11,9.10470513e-16}, bhigh = {33202.3992, -76.37571062}, R = 64.2621884477149);
          constant IdealGases.Common.DataRecord CH2CL2(name = "CH2CL2", MM = 0.08493258, Hf =  -1118534.25387525, H0 = 139570.433395524, Tlimit = 1000, alow = { -25098.41179,868.766738, -5.09466921,0.0415004999, -5.19977215e-05,3.44594426e-08, -9.27029252e-12}, blow = { -16389.7884,53.9689032}, ahigh = {1529279.337, -6976.95476,16.94154931, -0.001265053995,2.344766734e-07, -2.333227421e-11,9.63283473e-16}, bhigh = {28063.18171, -79.4945351}, R = 97.8949656303859);
          constant IdealGases.Common.DataRecord CH2F(name = "CH2F", MM = 0.0330249832, Hf =  -962907.378556971, H0 = 336987.302388696, Tlimit = 1000, alow = { -85697.1253,1392.226749, -4.38205259,0.02645916948, -2.848145663e-05,1.732706028e-08, -4.44206144e-12}, blow = { -11694.44586,50.494992}, ahigh = {2535399.502, -9358.43902,17.00366206, -0.003062919929,7.61286908e-07, -9.66455498e-11,4.8447681e-15}, bhigh = {52283.9131, -87.0755157}, R = 251.76309552218);
          constant IdealGases.Common.DataRecord CH2FBr(name = "CH2FBr", MM = 0.1129289832, Hf =  -1903851.37550765, H0 = 102919.424851423, Tlimit = 1000, alow = { -92561.0953,1811.985865, -9.69834687,0.0503584867, -6.07027268e-05,3.8782508e-08, -1.012615992e-11}, blow = { -35375.0962,81.4039605}, ahigh = {1539975.462, -7231.55136,17.14566439, -0.001350184394,2.538022824e-07, -2.558977386e-11,1.069310946e-15}, bhigh = {15096.04224, -80.6205371}, R = 73.6256695526503);
          constant IdealGases.Common.DataRecord CH2FCL(name = "CH2FCL", MM = 0.0684779832, Hf =  -3880079.22523045, H0 = 164320.800265625, Tlimit = 1000, alow = { -107134.0332,2081.328396, -11.55723697,0.0544148211, -6.51862274e-05,4.13315988e-08, -1.071935103e-11}, blow = { -42647.5621,90.35923547}, ahigh = {1536120.418, -7378.93973,17.25797178, -0.001395645457,2.639247747e-07, -2.675653563e-11,1.123582778e-15}, bhigh = {9813.71046, -83.13642553}, R = 121.418178682576);
          constant IdealGases.Common.DataRecord CH2F2(name = "CH2F2", MM = 0.0520233864, Hf =  -8694166.8218661, H0 = 205548.460797623, Tlimit = 1000, alow = { -181991.75,3174.42789, -17.14256906,0.0641135383, -7.31359212e-05,4.42638416e-08, -1.103877216e-11}, blow = { -70270.5895,120.7331926}, ahigh = {1546609.496, -7876.88333,17.68770469, -0.001581298721,3.068917255e-07, -3.18340546e-11,1.363827177e-15}, bhigh = { -9800.13596, -89.0742036}, R = 159.821814290813);
          constant IdealGases.Common.DataRecord CH2I2(name = "CH2I2", MM = 0.26783552, Hf = 438964.928923542, H0 = 49472.2581978671, Tlimit = 1000, alow = {27381.33757, -89.7256804,1.387869408,0.02733482611, -3.61940877e-05,2.53918761e-08, -7.14558516e-12}, blow = {13387.67276,22.41870165}, ahigh = {1512892.716, -6442.9124,16.59026137, -0.00113555152,2.07375025e-07, -2.033902231e-11,8.2822837e-16}, bhigh = {50582.1903, -71.2523514}, R = 31.0432014394506);
          constant IdealGases.Common.DataRecord CH3(name = "CH3", MM = 0.01503452, Hf = 9754753.72675682, H0 = 0, Tlimit = 1000, alow = { -28761.88806,509.326866,0.2002143949,0.01363605829, -1.433989346e-05,1.013556725e-08, -3.027331936e-12}, blow = {0,14082.71825}, ahigh = {10366.34,2760802.663, -9336.53117,14.87729606, -0.001439429774,2.444477951e-07, -2.224555778e-11}, bhigh = {8.39506576e-16,0}, R = 553.025437459926);
          constant IdealGases.Common.DataRecord CH3Br(name = "CH3Br", MM = 0.09493852, Hf =  -397520.41637051, H0 = 111784.500116497, Tlimit = 1000, alow = { -71155.8577,1524.705928, -8.23044521,0.0423997321, -4.94697989e-05,3.19443334e-08, -8.53123708e-12}, blow = { -12517.53591,70.4812777}, ahigh = {2524874.348, -10118.76098,18.65902163, -0.00179767369,3.29852011e-07, -3.25091203e-11,1.330179925e-15}, bhigh = {55405.7639, -97.7866446}, R = 87.5774343227596);
          constant IdealGases.Common.DataRecord CH3CL(name = "CH3CL", MM = 0.05048752, Hf =  -1621588.85997965, H0 = 206311.262664516, Tlimit = 1000, alow = { -98419.711,1983.700841, -10.84512305,0.0477798005, -5.51551626e-05,3.50617614e-08, -9.23610396e-12}, blow = { -19946.89506,83.99658331}, ahigh = {2522463.305, -10301.15447,18.82725852, -0.001872291294,3.47337286e-07, -3.45888722e-11,1.428941079e-15}, bhigh = {51114.1741, -100.6571389}, R = 164.683707973773);
          constant IdealGases.Common.DataRecord CH3F(name = "CH3F", MM = 0.0340329232, Hf =  -6984413.25780678, H0 = 297806.595702599, Tlimit = 1000, alow = { -202982.1878,3447.33113, -17.68994275,0.059452758, -6.46952825e-05,3.85941804e-08, -9.62654153e-12}, blow = { -45779.2622,122.8382176}, ahigh = {2561903.188, -10860.52758,19.2944692, -0.002070973131,3.92912453e-07, -3.99450342e-11,1.681447081e-15}, bhigh = {35635.0851, -106.1158456}, R = 244.306724730598);
          constant IdealGases.Common.DataRecord CH3I(name = "CH3I", MM = 0.14193899, Hf = 96980.8225350906, H0 = 76200.5844905618, Tlimit = 1000, alow = { -45164.6274,1086.208429, -5.66317627,0.0368317171, -4.3213935e-05,2.833483666e-08, -7.68438369e-12}, blow = { -4303.83065,56.8856269}, ahigh = {2511915.982, -9960.28999,18.56907132, -0.001768191331,3.24220626e-07, -3.19294201e-11,1.305407821e-15}, bhigh = {60669.2995, -95.9077704}, R = 58.577787540971);
          constant IdealGases.Common.DataRecord CH2OH(name = "CH2OH", MM = 0.03103392, Hf =  -573565.956218228, H0 = 379616.883719491, Tlimit = 1000, alow = { -156007.6238,2685.446279, -13.4202242,0.0575713947, -7.28444999e-05,4.83664886e-08, -1.293492601e-11}, blow = { -15968.2041,99.630337}, ahigh = {2250349.506, -8173.18606,15.99639179, -0.000870413372,6.06918395e-08,4.40834946e-12, -5.7023095e-16}, bhigh = {46453.1343, -78.3515845}, R = 267.915622647735);
          constant IdealGases.Common.DataRecord CH2OHplus(name = "CH2OHplus", MM = 0.0310333714, Hf = 23084826.6779033, H0 = 327035.04460363, Tlimit = 1000, alow = { -107708.0841,2252.082711, -11.88167865,0.0460231696, -4.87973688e-05,2.876413471e-08, -7.1002265e-12}, blow = {74844.4777,90.2792857}, ahigh = {2603333.487, -10099.5381,17.30843898, -0.000694639032,3.009715083e-08,5.22148598e-12, -5.09018379e-16}, bhigh = {146540.2426, -92.2395528}, R = 267.920358791569);
          constant IdealGases.Common.DataRecord CH3O(name = "CH3O", MM = 0.03103392, Hf = 418896.484878481, H0 = 364183.770532372, Tlimit = 1000, alow = {86571.1766, -663.168525,2.257455672,0.02266283789, -2.970566403e-05,2.199341353e-08, -6.58804338e-12}, blow = {4174.10213,8.1747779}, ahigh = {2101188.243, -8841.9688,18.22645731, -0.001743485034,3.34043427e-07, -3.43067316e-11,1.473897771e-15}, bhigh = {53095.8206, -94.2250059}, R = 267.915622647735);
          constant IdealGases.Common.DataRecord CH4(name = "CH4", MM = 0.01604246, Hf =  -4650159.63885838, H0 = 624355.740952447, Tlimit = 1000, alow = { -176685.0998,2786.18102, -12.0257785,0.0391761929, -3.61905443e-05,2.026853043e-08, -4.97670549e-12}, blow = { -23313.1436,89.0432275}, ahigh = {3730042.76, -13835.01485,20.49107091, -0.001961974759,4.72731304e-07, -3.72881469e-11,1.623737207e-15}, bhigh = {75320.6691, -121.9124889}, R = 518.279116793809);
          constant IdealGases.Common.DataRecord CH3OH(name = "CH3OH", MM = 0.03204186, Hf =  -6271171.52375049, H0 = 356885.55533293, Tlimit = 1000, alow = { -241664.2886,4032.14719, -20.46415436,0.0690369807, -7.59893269e-05,4.59820836e-08, -1.158706744e-11}, blow = { -44332.6117,140.014219}, ahigh = {3411570.76, -13455.00201,22.61407623, -0.002141029179,3.73005054e-07, -3.49884639e-11,1.366073444e-15}, bhigh = {56360.8156, -127.7814279}, R = 259.487807511799);
          constant IdealGases.Common.DataRecord CH3OOH(name = "CH3OOH", MM = 0.04804126, Hf =  -2893346.26110972, H0 = 0, Tlimit = 1000, alow = { -149797.4156,2656.222273, -13.77060625,0.0658867383, -7.75180165e-05,4.9688007e-08, -1.31436764e-11}, blow = {0, -30584.14201}, ahigh = {13918.692,3060740.61, -12829.59627,25.41021168, -0.002394481095,4.44342991e-07, -4.4166595e-11}, bhigh = {1.819673372e-15,0}, R = 173.069399095694);
          constant IdealGases.Common.DataRecord CI(name = "CI", MM = 0.13891517, Hf = 4104669.15888308, H0 = 68344.6883446927, Tlimit = 1000, alow = {104301.1064, -1715.427168,12.88874952, -0.01828504834,2.135468356e-05, -1.286980869e-08,3.16704766e-12}, blow = {75507.847, -44.9681132}, ahigh = { -240822.9894,344.738704,4.9769687, -0.000844481539,5.06123972e-07, -1.047700525e-10,6.74084314e-15}, bhigh = {64513.4202,1.090913159}, R = 59.8528727999973);
          constant IdealGases.Common.DataRecord CI2(name = "CI2", MM = 0.26581964, Hf = 1762073.08459225, H0 = 47562.8249289631, Tlimit = 1000, alow = {60282.8981, -1043.205435,10.17465942, -0.00553567171,5.63031901e-06, -3.108249657e-09,7.20246956e-13}, blow = {59648.9655, -23.10319208}, ahigh = { -702865.553,308.2750393,9.04642705, -0.002946736644,1.424347233e-06, -2.45318341e-10,1.425270291e-14}, bhigh = {50211.3421, -15.60349377}, R = 31.2786218505149);
          constant IdealGases.Common.DataRecord CI3(name = "CI3", MM = 0.39272411, Hf = 1033763.75593543, H0 = 42858.2064900472, Tlimit = 1000, alow = {85967.8425, -1347.703793,12.72103659, -0.002442876357,2.511609978e-07,1.024658583e-09, -4.87269296e-13}, blow = {53107.1651, -32.384929}, ahigh = { -153651.1918, -83.5704437,10.06203125, -2.466447336e-05,5.41905475e-09, -6.181417e-13,2.851262022e-17}, bhigh = {45825.8953, -14.94825345}, R = 21.1712797566719);
          constant IdealGases.Common.DataRecord CI4(name = "CI4", MM = 0.51962858, Hf = 515644.000951603, H0 = 43022.1717211936, Tlimit = 1000, alow = {89190.4387, -1651.906475,17.96124325, -0.00851811425,8.50868225e-06, -4.59909773e-09,1.039492403e-12}, blow = {36893.9215, -58.0595312}, ahigh = { -144676.277, -52.8036517,13.03985589, -1.606214814e-05,3.56755047e-09, -4.10536698e-13,1.907272766e-17}, bhigh = {28177.46618, -28.10414436}, R = 16.0007981085259);
          constant IdealGases.Common.DataRecord CN(name = "CN", MM = 0.0260174, Hf = 16861160.3004143, H0 = 333319.393944053, Tlimit = 1000, alow = {3949.14857, -139.1590572,4.93083532, -0.00630467051,1.256836472e-05, -9.8783005e-09,2.843137221e-12}, blow = {52284.5538, -2.763115585}, ahigh = { -2228006.27,5040.73339, -0.2121897722,0.001354901134,1.325929798e-07, -6.93700637e-11,5.49495227e-15}, bhigh = {17844.96132,32.82563919}, R = 319.573516185322);
          constant IdealGases.Common.DataRecord CNplus(name = "CNplus", MM = 0.0260168514, Hf = 69143297.7935216, H0 = 333710.788692901, Tlimit = 1000, alow = { -830290.957,8775.6875, -29.7744356,0.0497689706, -1.302225951e-05, -2.058325353e-08,1.126843895e-11}, blow = {170386.0539,203.9918818}, ahigh = { -7153463.08,18572.50421, -10.84534159,0.00610668143, -1.191208566e-06,1.184848778e-10, -4.79983873e-15}, bhigh = {92426.4496,113.5340573}, R = 319.580254818998);
          constant IdealGases.Common.DataRecord CNminus(name = "CNminus", MM = 0.0260179486, Hf = 2455424.32964911, H0 = 333273.93075102, Tlimit = 1000, alow = { -46065.4139,429.417475,2.32878188, -0.0001235303004,4.47846277e-06, -4.40315129e-09,1.349001191e-12}, blow = {4362.07834,11.42928617}, ahigh = {351796.472, -1630.477359,5.60987575, -0.000397560597,8.85614708e-08, -9.72287232e-12,4.43420569e-16}, bhigh = {16479.76581, -11.75502699}, R = 319.566777835821);
          constant IdealGases.Common.DataRecord CNN(name = "CNN", MM = 0.0400241, Hf = 15827565.2919116, H0 = 259285.280618427, Tlimit = 1000, alow = { -73576.9236,965.243866, -1.704121157,0.02037239025, -2.183423906e-05,1.176082777e-08, -2.551355221e-12}, blow = {70217.2983,35.2815091}, ahigh = { -181714.8765, -672.986349,7.85794834, -6.13688072e-05, -1.088178985e-08,4.45665581e-12, -2.836496278e-16}, bhigh = {77234.8898, -19.66012324}, R = 207.736638675198);
          constant IdealGases.Common.DataRecord CO(name = "CO", MM = 0.0280101, Hf =  -3946262.09831454, H0 = 309570.619169514, Tlimit = 1000, alow = {14890.45326, -292.2285939,5.72452717, -0.00817623503,1.456903469e-05, -1.087746302e-08,3.027941827e-12}, blow = { -13031.31878, -7.85924135}, ahigh = {461919.725, -1944.704863,5.91671418, -0.000566428283,1.39881454e-07, -1.787680361e-11,9.62093557e-16}, bhigh = { -2466.261084, -13.87413108}, R = 296.838354736327);
          constant IdealGases.Common.DataRecord COplus(name = "COplus", MM = 0.0280095514, Hf = 44548703.6254354, H0 = 309576.682474108, Tlimit = 1000, alow = { -21787.86658,128.8857032,3.76905755, -0.00343173013,8.19394575e-06, -6.46381469e-09,1.803727574e-12}, blow = {148234.5898,3.99054707}, ahigh = {231684.7506, -1057.646148,4.55425778,0.000449552032, -2.489507047e-07,5.26756642e-11, -3.28951027e-15}, bhigh = {155505.0724, -3.87346264}, R = 296.844168664551);
          constant IdealGases.Common.DataRecord COCL(name = "COCL", MM = 0.0634631, Hf =  -252115.008564032, H0 = 182007.323940999, Tlimit = 1000, alow = {25131.7574, -596.918967,8.32767135, -0.00705613259,1.313150734e-05, -1.037059653e-08,3.033665179e-12}, blow = { -705.277032, -15.80716775}, ahigh = {344372.024, -1793.14347,8.3927559, -0.000537476959,9.11355571e-08, -3.111441728e-12, -2.040435218e-16}, bhigh = {6914.47015, -19.98919104}, R = 131.012698717838);
          constant IdealGases.Common.DataRecord COCL2(name = "COCL2", MM = 0.0989161, Hf =  -2219052.30796604, H0 = 130197.429943154, Tlimit = 1000, alow = {93193.2145, -1577.971273,12.08353907, -0.00480901561,7.68847732e-06, -5.8588412e-09,1.687786559e-12}, blow = { -20542.52334, -38.3476732}, ahigh = { -25458.81891, -1305.958516,10.92922584, -0.000360121016,7.78701765e-08, -8.79245203e-12,4.02869661e-16}, bhigh = { -22198.4734, -32.3330345}, R = 84.0558008251437);
          constant IdealGases.Common.DataRecord COFCL(name = "COFCL", MM = 0.0824615032, Hf =  -5208404.47157893, H0 = 144355.821056631, Tlimit = 1000, alow = {72621.739, -1019.738175,7.29091199,0.00742069171, -7.90254808e-06,4.20942765e-09, -9.3144129e-13}, blow = { -48043.8646, -13.16923671}, ahigh = { -53168.8791, -1581.009678,11.12696501, -0.00043730008,9.46419071e-08, -1.069291346e-11,4.90175664e-16}, bhigh = { -45996.6723, -35.25879824}, R = 100.828528190109);
          constant IdealGases.Common.DataRecord COF2(name = "COF2", MM = 0.0660069064, Hf =  -9695955.08872387, H0 = 168678.243039125, Tlimit = 1000, alow = {52633.9315, -461.860339,2.774114516,0.01831931082, -2.130172554e-05,1.266924542e-08, -3.101675983e-12}, blow = { -75642.551,9.46718146}, ahigh = { -40738.0685, -1974.009812,11.40363654, -0.000543711147,1.175232744e-07, -1.326564192e-11,6.07677213e-16}, bhigh = { -69077.9541, -40.09695}, R = 125.963667341331);
          constant IdealGases.Common.DataRecord COHCL(name = "COHCL", MM = 0.06447104, Hf =  -2547064.17020727, H0 = 170716.681474349, Tlimit = 1000, alow = {6332.94687,81.2036559,1.374648391,0.01650970564, -1.692917241e-05,9.68404683e-09, -2.37446939e-12}, blow = { -21203.5658,19.3837965}, ahigh = {831895.285, -4416.87084,12.70661114, -0.000936140863,1.855335611e-07, -1.958378539e-11,8.51204683e-16}, bhigh = {4330.49644, -51.45071542}, R = 128.964446672491);
          constant IdealGases.Common.DataRecord COHF(name = "COHF", MM = 0.0480164432, Hf =  -7801293.39109399, H0 = 217629.509884231, Tlimit = 1000, alow = { -45858.285,1048.202675, -4.77657422,0.03080719931, -3.4158959e-05,2.034926806e-08, -5.05419462e-12}, blow = { -50859.8373,52.3224741}, ahigh = {857885.316, -4791.95912,12.93975218, -0.001018285299,2.021278474e-07, -2.136698091e-11,9.29958891e-16}, bhigh = { -18789.33107, -55.2744914}, R = 173.158848217229);
          constant IdealGases.Common.DataRecord COS(name = "COS", MM = 0.0600751, Hf =  -2358714.34254791, H0 = 165486.149835789, Tlimit = 1000, alow = {85478.7643, -1319.464821,9.73525724, -0.00687083096,1.082331416e-05, -7.70559734e-09,2.078570344e-12}, blow = { -11916.57685, -29.91988593}, ahigh = {195909.8567, -1756.167688,8.71043034, -0.000413942496,1.015243648e-07, -1.159609663e-11,5.69105386e-16}, bhigh = { -8927.09669, -26.36328016}, R = 138.401301038201);
          constant IdealGases.Common.DataRecord CO2(name = "CO2", MM = 0.0440095, Hf =  -8941478.54440518, H0 = 212805.621513537, Tlimit = 1000, alow = {49436.5054, -626.411601,5.30172524,0.002503813816, -2.127308728e-07, -7.68998878e-10,2.849677801e-13}, blow = { -45281.9846, -7.04827944}, ahigh = {117696.2419, -1788.791477,8.29152319, -9.22315678e-05,4.86367688e-09, -1.891053312e-12,6.33003659e-16}, bhigh = { -39083.5059, -26.52669281}, R = 188.924482214067);
          constant IdealGases.Common.DataRecord CO2plus(name = "CO2plus", MM = 0.0440089514, Hf = 21465814.5433567, H0 = 240089.542328882, Tlimit = 1000, alow = { -73830.3098,1086.211742, -2.771112737,0.02318463595, -2.570240315e-05,1.450335497e-08, -3.33447042e-12}, blow = {107178.4918,40.5488521}, ahigh = { -169505.1682, -806.646973,8.00282846, -0.0001577214041,2.566759314e-08, -2.404195965e-12,1.6774468e-16}, bhigh = {115438.9478, -21.33567772}, R = 188.926837279745);
          constant IdealGases.Common.DataRecord COOH(name = "COOH", MM = 0.04501744, Hf =  -4731499.61437168, H0 = 240203.552223316, Tlimit = 1000, alow = { -11283.80671,377.517943, -0.599255041,0.02181894272, -2.425918417e-05,1.451245206e-08, -3.59623338e-12}, blow = { -28410.42565,29.34561769}, ahigh = {929318.873, -4483.03057,12.42199567, -0.000746313964,1.332996131e-07, -1.28271055e-11,5.1379979e-16}, bhigh = { -851.823268, -50.6806551}, R = 184.694465078423);
          constant IdealGases.Common.DataRecord CP(name = "CP", MM = 0.042984461, Hf = 12101159.184013, H0 = 202750.128703487, Tlimit = 1000, alow = { -45239.4011,775.882814, -1.453947907,0.01397713706, -1.624489706e-05,9.51408388e-09, -2.210281109e-12}, blow = {57926.3467,33.1164792}, ahigh = { -5449937.15,16883.45926, -15.956922,0.01136028761, -2.84654133e-06,3.37936404e-10, -1.554457397e-14}, bhigh = { -45545.1774,145.1324059}, R = 193.429714054109);
          constant IdealGases.Common.DataRecord CS(name = "CS", MM = 0.0440757, Hf = 6319810.64396028, H0 = 197577.758265893, Tlimit = 1000, alow = { -49248.4412,816.69681, -1.542998408,0.01380324735, -1.574407905e-05,9.16971493e-09, -2.169700595e-12}, blow = {28651.82876,33.08541327}, ahigh = { -971957.476,2339.201284,1.709390402,0.001577178949, -4.14633591e-07,4.50475708e-11, -5.94545773e-16}, bhigh = {16810.20727,18.7404822}, R = 188.640724934601);
          constant IdealGases.Common.DataRecord CS2(name = "CS2", MM = 0.0761407, Hf = 1532688.82476783, H0 = 140059.429451003, Tlimit = 1000, alow = {16135.60482, -464.948147,6.29793879,0.001888896706,3.031927747e-07, -1.737645373e-09,7.79398939e-13}, blow = {14777.61119, -9.30338213}, ahigh = { -1390419.724,3354.9755,3.019247723,0.002876437543, -9.07681272e-07,1.374091042e-10, -6.99957557e-15}, bhigh = { -10138.98046,15.65113703}, R = 109.198785931834);
          constant IdealGases.Common.DataRecord C2(name = "C2", MM = 0.0240214, Hf = 34571562.1071212, H0 = 423335.942118278, Tlimit = 1000, alow = {555963.451, -9980.12644,66.8162037, -0.1743432724,0.0002448523051, -1.70346758e-07,4.68452773e-11}, blow = {144586.9634, -344.82297}, ahigh = { -968926.793,3561.09299, -0.506413893,0.002945154879, -7.13944119e-07,8.67065725e-11, -4.07690681e-15}, bhigh = {76817.9683,33.3998524}, R = 346.127702798338);
          constant IdealGases.Common.DataRecord C2plus(name = "C2plus", MM = 0.0240208514, Hf = 83459803.260762, H0 = 361565.202472382, Tlimit = 1000, alow = { -99134.2384,1347.170609, -3.47675316,0.01676429424, -1.865908025e-05,1.091134647e-08, -2.434913818e-12}, blow = {233545.48,44.0664462}, ahigh = {3836292.81, -6242.06245,2.779245639,0.00606586586, -2.452799858e-06,3.8829425e-10, -2.190639912e-14}, bhigh = {285744.7553,0.729738349}, R = 346.135607832785);
          constant IdealGases.Common.DataRecord C2minus(name = "C2minus", MM = 0.0240219486, Hf = 20013651.3488335, H0 = 361174.03065295, Tlimit = 1000, alow = { -118192.866,1438.18971, -3.19613135,0.01465548163, -1.545537278e-05,9.06123561e-09, -2.135962274e-12}, blow = {49653.1779,42.2560888}, ahigh = {4478136.25, -11541.45714,13.10143499, -0.001862700578,4.00693125e-08,3.7102136e-11, -3.33726687e-15}, bhigh = {132535.6168, -69.759644}, R = 346.119798124953);
          constant IdealGases.Common.DataRecord C2CL(name = "C2CL", MM = 0.0594744, Hf = 8980049.75249855, H0 = 181264.981235624, Tlimit = 1000, alow = {47258.8941, -898.022361,9.01687715, -0.00633378283,1.08706005e-05, -8.08906892e-09,2.248275223e-12}, blow = {67022.157, -23.54893403}, ahigh = {213736.8408, -1630.519518,8.62349025, -0.000425351786,9.04001864e-08, -1.007541495e-11,4.57076475e-16}, bhigh = {71710.9328, -24.13014598}, R = 139.799174098436);
          constant IdealGases.Common.DataRecord C2CL2(name = "C2CL2", MM = 0.0949268, Hf = 2387102.48317651, H0 = 0, Tlimit = 1000, alow = {38711.6908, -933.687073,11.19377861, -0.0037462882,6.77047256e-06, -4.91038835e-09,1.299068749e-12}, blow = {0,29481.53198}, ahigh = {14592.612,296197.8472, -2044.286552,11.87529315, -0.000511501145,1.072661556e-07, -1.183411171e-11}, bhigh = {5.32649834e-16,0}, R = 87.588246943961);
          constant IdealGases.Common.DataRecord C2CL3(name = "C2CL3", MM = 0.1303804, Hf = 1459357.32671475, H0 = 123870.290319711, Tlimit = 1000, alow = {46750.1604, -885.101964,9.03227055,0.01242122796, -1.554614347e-05,9.51388712e-09, -2.341473263e-12}, blow = {24958.63399, -17.79070154}, ahigh = { -220402.8219, -1072.926248,13.78367377, -0.0003093322915,6.77752895e-08, -7.72730182e-12,3.56669022e-16}, bhigh = {24310.20015, -43.42161815}, R = 63.7708735362064);
          constant IdealGases.Common.DataRecord C2CL4(name = "C2CL4", MM = 0.1658322, Hf =  -145930.645556171, H0 = 0, Tlimit = 1000, alow = {37462.583, -848.177563,10.0915611,0.01845463613, -2.390899444e-05,1.514828257e-08, -3.84111932e-12}, blow = {0, -1598.288471}, ahigh = {19605.643, -300131.5659, -1132.407598,16.8307247, -0.000328879249,7.22090703e-08, -8.24533269e-12}, bhigh = {3.81011278e-16,0}, R = 50.1378622487068);
          constant IdealGases.Common.DataRecord C2CL6(name = "C2CL6", MM = 0.2367376, Hf =  -626009.55657234, H0 = 0, Tlimit = 1000, alow = {159345.1813, -2606.697136,20.28011847,0.01769364822, -3.133105185e-05,2.389395238e-08, -6.8930383e-12}, blow = {0, -9038.00764}, ahigh = {27129.39, -557738.632, -497.649124,22.37041989, -0.0001477759098,3.25754658e-08, -3.72709921e-12}, bhigh = {1.723801704e-16,0}, R = 35.121045410615);
          constant IdealGases.Common.DataRecord C2F(name = "C2F", MM = 0.0430198032, Hf = 8225223.16884983, H0 = 240982.878322419, Tlimit = 1000, alow = {12497.97213, -348.387675,5.75508435,0.000852868538,2.353546435e-06, -2.804737097e-09,9.08357564e-13}, blow = {42815.2531, -6.43720096}, ahigh = {289877.6676, -2016.644658,8.87504571, -0.000516684113,1.092094306e-07, -1.212216787e-11,5.48228768e-16}, bhigh = {52412.8792, -27.7308958}, R = 193.270805106798);
          constant IdealGases.Common.DataRecord C2FCL(name = "C2FCL", MM = 0.0784728032, Hf = 430294.288250939, H0 = 176832.054344148, Tlimit = 1000, alow = {26976.75344, -741.440318,9.6296043, -0.000386554493,3.003347997e-06, -2.710191661e-09,7.73758689e-13}, blow = {5500.62039, -25.15774889}, ahigh = {347145.489, -2348.4611,12.06875617, -0.000580469823,1.21273434e-07, -1.334214186e-11,5.99255661e-16}, bhigh = {15020.61224, -42.52445687}, R = 105.953549012507);
          constant IdealGases.Common.DataRecord C2FCL3(name = "C2FCL3", MM = 0.1493788032, Hf =  -1111268.7773897, H0 = 125398.74198162, Tlimit = 1000, alow = {12372.82217, -470.396184,7.58162179,0.02305727066, -2.792144322e-05,1.679699826e-08, -4.07757244e-12}, blow = { -20313.33976, -9.274637513}, ahigh = { -282950.5545, -1473.853145,17.07879074, -0.000426521339,9.35719985e-08, -1.067932796e-11,4.93326161e-16}, bhigh = { -17384.33314, -61.97054796}, R = 55.6603200848245);
          constant IdealGases.Common.DataRecord C2F2(name = "C2F2", MM = 0.0620182064, Hf =  -2332632.56707146, H0 = 213911.684488831, Tlimit = 1000, alow = {17763.13572, -611.328094,8.6636068,0.000915638121,2.363808425e-06, -2.753750054e-09,8.71393597e-13}, blow = { -16496.11017, -21.65150307}, ahigh = {419389.094, -2715.359989,12.30403413, -0.000664930554,1.385254301e-07, -1.520813764e-11,6.81982103e-16}, bhigh = { -4179.22194, -46.7038623}, R = 134.065018687803);
          constant IdealGases.Common.DataRecord C2F2CL2(name = "C2F2CL2", MM = 0.1329242064, Hf =  -2541576.20458812, H0 = 134934.256790116, Tlimit = 1000, alow = { -115043.3214,2032.113595, -11.19750271,0.083424514, -0.0001163698616,7.83309289e-08, -2.070004069e-11}, blow = { -52082.5681,88.88322147}, ahigh = { -870256.566,726.472448,15.30689121,0.0003045597224, -7.04546804e-08,8.28592434e-12, -3.89858107e-16}, bhigh = { -52088.8871, -49.98300753}, R = 62.5504731243594);
          constant IdealGases.Common.DataRecord C2F3(name = "C2F3", MM = 0.0810166096, Hf =  -2816472.99395259, H0 = 174823.003207974, Tlimit = 1000, alow = { -26147.75016,210.1767714,2.32862508,0.02368943293, -2.416431591e-05,1.224916093e-08, -2.488670434e-12}, blow = { -30285.66585,16.9960879}, ahigh = { -124260.3837, -2137.292446,14.55051371, -0.000609389198,1.331427672e-07, -1.515124358e-11,6.98411206e-16}, bhigh = { -19771.46179, -54.2103987}, R = 102.626758155528);
          constant IdealGases.Common.DataRecord C2F3CL(name = "C2F3CL", MM = 0.1164696096, Hf =  -4423471.51131861, H0 = 147061.100821274, Tlimit = 1000, alow = { -8122.27031, -157.4007092,5.04181029,0.02621692321, -2.843001445e-05,1.537643979e-08, -3.36073067e-12}, blow = { -63540.4995,2.792301872}, ahigh = { -216272.4659, -2195.103372,17.59605428, -0.000628261729,1.37416661e-07, -1.565004037e-11,7.2183099e-16}, bhigh = { -55151.095, -69.21931824}, R = 71.3874806359787);
          constant IdealGases.Common.DataRecord C2F4(name = "C2F4", MM = 0.1000150128, Hf =  -6594010.05445854, H0 = 163281.026945987, Tlimit = 1000, alow = { -9991.53007, -129.1088427,4.50422905,0.0259202964, -2.63030872e-05,1.316489777e-08, -2.625017169e-12}, blow = { -80904.4708,3.27424147}, ahigh = { -162991.5758, -2603.903955,17.88488423, -0.000739703801,1.61446034e-07, -1.835820392e-11,8.45764107e-16}, bhigh = { -70063.7166, -74.5416586}, R = 83.1322395231449);
          constant IdealGases.Common.DataRecord C2F6(name = "C2F6", MM = 0.1380118192, Hf =  -9738296.38498092, H0 = 146886.260303712, Tlimit = 1000, alow = { -37953.717,799.82764, -4.67156181,0.0750145099, -9.8121161e-05,6.31980759e-08, -1.622597311e-11}, blow = { -167521.1878,50.537592}, ahigh = { -1011551.484, -942.214071,22.02553906, -0.000131451875,1.8660453e-08, -3.46711861e-12,2.488311205e-16}, bhigh = { -165743.4402, -93.0216115}, R = 60.2446373665365);
          constant IdealGases.Common.DataRecord C2H(name = "C2H", MM = 0.02502934, Hf = 22621470.7219607, H0 = 417457.352051632, Tlimit = 1000, alow = {13436.69487, -506.797072,7.77210741, -0.00651233982,1.030117855e-05, -5.88014767e-09,1.226901861e-12}, blow = {68922.6999, -18.71881626}, ahigh = {3922334.57, -12047.51703,17.5617292, -0.00365544294,6.98768543e-07, -6.82516201e-11,2.719262793e-15}, bhigh = {143326.6627, -95.6163438}, R = 332.189022962651);
          constant IdealGases.Common.DataRecord C2HCL(name = "C2HCL", MM = 0.06048204, Hf = 3743259.98263286, H0 = 0, Tlimit = 1000, alow = {132978.5007, -2174.542126,14.97980854, -0.01290343389,1.602596678e-05, -9.15204951e-09,2.021223607e-12}, blow = {0,36048.0132}, ahigh = {11787.625,1152145.326, -4461.193,12.81366744, -0.000679734676,1.151382756e-07, -1.046641954e-11}, bhigh = {3.94986052e-16,0}, R = 137.47009856149);
          constant IdealGases.Common.DataRecord C2HCL3(name = "C2HCL3", MM = 0.13138744, Hf =  -133193.857799497, H0 = 0, Tlimit = 1000, alow = {39592.3125, -517.875358,5.15502877,0.02816922486, -3.62474172e-05,2.39207427e-08, -6.36240196e-12}, blow = {0, -1534.346259}, ahigh = {16604.624,604925.466, -4236.99232,18.46273557, -0.000813719506,1.551211545e-07, -1.584630634e-11}, bhigh = {6.70093249e-16,0}, R = 63.2820914997659);
          constant IdealGases.Common.DataRecord C2HF(name = "C2HF", MM = 0.0440277432, Hf = 946955.282504691, H0 = 259970.354328768, Tlimit = 1000, alow = {91611.6911, -1537.245636,11.33665074, -0.00462472931,5.87798546e-06, -2.690992345e-09,3.52471953e-13}, blow = {10859.0282, -40.2399438}, ahigh = {1234347.179, -4819.58196,13.02337752, -0.000749543582,1.285452942e-07, -1.184559365e-11,4.53678476e-16}, bhigh = {32368.1171, -56.4079182}, R = 188.846200047792);
          constant IdealGases.Common.DataRecord C2HFCL2(name = "C2HFCL2", MM = 0.1149337432, Hf =  -1467349.755646, H0 = 141466.635883482, Tlimit = 1000, alow = {31173.99823, -136.9505457,1.176399869,0.040807766, -5.4181378e-05,3.61360066e-08, -9.62586235e-12}, blow = { -21151.42871,21.4643316}, ahigh = {407124.352, -3808.20452,18.2114405, -0.000727896606,1.381532849e-07, -1.405342513e-11,5.92000972e-16}, bhigh = { -2676.849943, -75.67353567}, R = 72.3414357568753);
          constant IdealGases.Common.DataRecord C2HF2CL(name = "C2HF2CL", MM = 0.0984791464, Hf =  -3388067.54726298, H0 = 154984.629314374, Tlimit = 1000, alow = {170443.7675, -1667.236861,4.7887885,0.0444483001, -7.32219777e-05,5.62934755e-08, -1.654638029e-11}, blow = { -32918.0976, -5.78859364}, ahigh = {583037.47, -3810.31741,17.99003171, -0.000588821086,1.002861284e-07, -9.1497881e-12,3.45998516e-16}, bhigh = { -22058.75409, -75.90948533}, R = 84.4287578024742);
          constant IdealGases.Common.DataRecord C2HF3(name = "C2HF3", MM = 0.0820245496, Hf =  -5986012.75343059, H0 = 174673.461419409, Tlimit = 1000, alow = { -14109.69036,368.762492, -1.144797568,0.0395630602, -4.60200136e-05,2.785149385e-08, -6.90376008e-12}, blow = { -62264.1033,32.8966169}, ahigh = {691741.378, -5260.30164,19.15276614, -0.001072493938,2.099276744e-07, -2.195126633e-11,9.4727024e-16}, bhigh = { -32476.4021, -87.5045695}, R = 101.365652607009);
          constant IdealGases.Common.DataRecord C2H2_vinylidene(name = "C2H2_vinylidene", MM = 0.02603728, Hf = 15930556.8016321, H0 = 417638.401553465, Tlimit = 1000, alow = { -14660.42239,278.9475593,1.276229776,0.01395015463, -1.475702649e-05,9.47629811e-09, -2.567602217e-12}, blow = {47361.1018,16.58225704}, ahigh = {1940838.725, -6892.71815,13.39582494, -0.000936896867,1.470804368e-07, -1.220040365e-11,4.12239166e-16}, bhigh = {91071.1293, -63.3750293}, R = 319.329515218179);
          constant IdealGases.Common.DataRecord C2H2CL2(name = "C2H2CL2", MM = 0.09694328, Hf = 35175.207605932, H0 = 153517.366030941, Tlimit = 1000, alow = { -12037.24514,462.903119, -1.318184584,0.039304496, -4.85152902e-05,3.17841146e-08, -8.47693841e-12}, blow = { -3251.80686,34.89218118}, ahigh = {1561121.185, -7358.09635,20.11869185, -0.001310978834,2.411885716e-07, -2.38416674e-11,9.78541575e-16}, bhigh = {41222.009, -95.50252712}, R = 85.7663573999147);
          constant IdealGases.Common.DataRecord C2H2FCL(name = "C2H2FCL", MM = 0.0804886832, Hf =  -2050996.40641158, H0 = 167344.183859129, Tlimit = 1000, alow = {226015.4427, -2557.428772,9.2362412,0.02879408894, -5.16397547e-05,4.25219103e-08, -1.311612713e-11}, blow = { -8180.62784, -32.34787581}, ahigh = {1527690.968, -6689.71223,19.5055383, -0.001040920895,1.780587502e-07, -1.633098515e-11,6.21346556e-16}, bhigh = {17052.66451, -91.94078301}, R = 103.299888499108);
          constant IdealGases.Common.DataRecord C2H2F2(name = "C2H2F2", MM = 0.0640340864, Hf =  -5253452.0114587, H0 = 194891.247796424, Tlimit = 1000, alow = {57029.3581, -676.28604,4.11741171,0.01822618762, -6.86133154e-06, -3.29575674e-09,2.180965597e-12}, blow = { -38386.5078,1.487528685}, ahigh = { -822719.275, -2915.718833,18.30163995, -0.001039630129,2.546725128e-07, -3.157578705e-11,1.55154096e-15}, bhigh = { -31100.28589, -84.008347}, R = 129.844469835366);
          constant IdealGases.Common.DataRecord CH2CO_ketene(name = "CH2CO_ketene", MM = 0.04203668, Hf =  -1179353.2457844, H0 = 0, Tlimit = 1000, alow = {35495.9809, -406.306283,3.71892192,0.01583501817, -1.726195691e-05,1.157376959e-08, -3.30584263e-12}, blow = {0, -5209.99258}, ahigh = {11795.805,2013564.915, -8200.88746,17.59694074, -0.001464544521,2.695886969e-07, -2.66567484e-11}, bhigh = {1.094204522e-15,0}, R = 197.790881677621);
          constant IdealGases.Common.DataRecord O_CH_2O(name = "O_CH_2O", MM = 0.05803608, Hf =  -3652900.05803287, H0 = 0, Tlimit = 1000, alow = { -229245.9698,3724.09805, -18.93769993,0.0751174414, -8.08385542e-05,4.35823319e-08, -9.36453933e-12}, blow = {0, -44544.8601}, ahigh = {13682.443,267806.3593, -4436.61748,17.81696797, -0.000709717378,1.272621878e-07, -1.237226678e-11}, bhigh = {5.02505752e-16,0}, R = 143.26384552506);
          constant IdealGases.Common.DataRecord HO_CO_2OH(name = "HO_CO_2OH", MM = 0.09003488, Hf =  -8127961.07464129, H0 = 0, Tlimit = 1000, alow = {30725.13274, -391.617469,3.17838864,0.0374128975, -4.00975396e-05,2.288662646e-08, -5.38677155e-12}, blow = {0, -87979.4255}, ahigh = {17321.662,1805560.696, -9240.33315,26.25742604, -0.001418458557,2.526840574e-07, -2.521005198e-11}, bhigh = {1.040036111e-15,0}, R = 92.3472325392115);
          constant IdealGases.Common.DataRecord C2H3_vinyl(name = "C2H3_vinyl", MM = 0.02704522, Hf = 11080953.1961655, H0 = 389044.385662235, Tlimit = 1000, alow = { -33478.9687,1064.104103, -6.40385706,0.0393451548, -4.76004609e-05,3.17007135e-08, -8.63340643e-12}, blow = {30391.22649,58.0922618}, ahigh = {2718080.093, -10309.56829,18.36579807, -0.001580131153,2.680594939e-07, -2.439003999e-11,9.20909639e-16}, bhigh = {97650.5559, -97.6008686}, R = 307.428521564994);
          constant IdealGases.Common.DataRecord CH2BrminusCOOH(name = "CH2BrminusCOOH", MM = 0.13894802, Hf =  -2760024.93594367, H0 = 0, Tlimit = 1000, alow = { -78324.8479,1648.865353, -9.34953363,0.0687963544, -8.37488077e-05,5.40603842e-08, -1.423159923e-11}, blow = {0, -55411.8762}, ahigh = {16862.437,2486349.85, -11402.56346,27.56020307, -0.00184318701,3.26539274e-07, -3.122663159e-11}, bhigh = {1.223452495e-15,0}, R = 59.8387224229608);
          constant IdealGases.Common.DataRecord C2H3CL(name = "C2H3CL", MM = 0.06249792, Hf = 352011.714949874, H0 = 0, Tlimit = 1000, alow = { -16888.95457,854.510455, -6.51496755,0.0494468193, -6.11775667e-05,4.06729385e-08, -1.101392611e-11}, blow = {0, -2069.321321}, ahigh = {11819.647,2456176.938, -10474.52327,21.78736181, -0.001816891849,3.29636995e-07, -3.21439569e-11}, bhigh = {1.302254061e-15,0}, R = 133.035979437396);
          constant IdealGases.Common.DataRecord CH2CLminusCOOH(name = "CH2CLminusCOOH", MM = 0.09449672, Hf =  -4525024.78392901, H0 = 0, Tlimit = 1000, alow = { -112250.6518,2168.288658, -12.27791286,0.0751438825, -9.08315877e-05,5.81236419e-08, -1.518247503e-11}, blow = {0, -63143.1455}, ahigh = {16513.941,2472883.176, -11522.75963,27.66275606, -0.001886430963,3.36177605e-07, -3.23127225e-11}, bhigh = {1.273275883e-15,0}, R = 87.9868846241436);
          constant IdealGases.Common.DataRecord C2H3F(name = "C2H3F", MM = 0.0460436232, Hf =  -3042766.62571594, H0 = 246199.738686073, Tlimit = 1000, alow = { -45791.3496,1411.541724, -10.21500877,0.0577581657, -7.08067749e-05,4.64085076e-08, -1.240455865e-11}, blow = { -24027.88827,78.6082839}, ahigh = {2478832.34, -10759.32307,21.94796688, -0.00186911411,3.39528067e-07, -3.31538293e-11,1.345081799e-15}, bhigh = {45634.5555, -118.0410277}, R = 180.578143554958);
          constant IdealGases.Common.DataRecord CH3CN(name = "CH3CN", MM = 0.04105192, Hf = 1618194.71537507, H0 = 294594.942209768, Tlimit = 1000, alow = { -99659.8838,1739.278534, -7.89842082,0.0429489432, -4.49997388e-05,2.717105086e-08, -7.02611759e-12}, blow = { -1461.161333,68.52508274}, ahigh = {2923231.393, -12337.92258,23.24477222, -0.002411565845,4.62215717e-07, -4.74060124e-11,2.010639467e-15}, bhigh = {80585.6555, -129.2249102}, R = 202.535520872105);
          constant IdealGases.Common.DataRecord CH3CO_acetyl(name = "CH3CO_acetyl", MM = 0.04304462, Hf =  -232317.070054283, H0 = 302859.381729935, Tlimit = 1000, alow = { -71938.9413,1464.465167, -6.63227613,0.0410846838, -4.22625664e-05,2.485766819e-08, -6.29255848e-12}, blow = { -9309.37081,64.2289762}, ahigh = {2485388.15, -11207.14204,22.77525438, -0.00231426055,4.53618917e-07, -4.74263555e-11,2.044663903e-15}, bhigh = {63800.8841, -121.5350925}, R = 193.159377408838);
          constant IdealGases.Common.DataRecord C2H4(name = "C2H4", MM = 0.02805316, Hf = 1871446.92433936, H0 = 374955.584326329, Tlimit = 1000, alow = { -116360.5836,2554.85151, -16.09746428,0.0662577932, -7.88508186e-05,5.12522482e-08, -1.370340031e-11}, blow = { -6176.19107,109.3338343}, ahigh = {3408763.67, -13748.47903,23.65898074, -0.002423804419,4.43139566e-07, -4.35268339e-11,1.775410633e-15}, bhigh = {88204.2938, -137.1278108}, R = 296.382724798205);
          constant IdealGases.Common.DataRecord C2H4O_ethylen_o(name = "C2H4O_ethylen_o", MM = 0.04405256, Hf =  -1194816.37389518, H0 = 245856.653960633, Tlimit = 1000, alow = { -172823.3345,3816.6788, -26.29851977,0.1014103162, -0.0001240578373,8.03404035e-08, -2.120942544e-11}, blow = { -24375.19333,165.4885056}, ahigh = {3151809.957, -14236.46316,27.08080476, -0.002606238456,4.85389193e-07, -4.85214476e-11,2.011778721e-15}, bhigh = {76625.6144, -156.3952401}, R = 188.739814439842);
          constant IdealGases.Common.DataRecord CH3CHO_ethanal(name = "CH3CHO_ethanal", MM = 0.04405256, Hf =  -3772538.98524853, H0 = 292757.469713451, Tlimit = 1000, alow = { -137390.4369,2559.937679, -13.40470172,0.0592212862, -6.24000605e-05,3.70332441e-08, -9.34269741e-12}, blow = { -33187.3131,100.7417652}, ahigh = {3321176.59, -14497.19957,27.08421279, -0.002879320054,5.55630992e-07, -5.73267488e-11,2.443965239e-15}, bhigh = {65077.5564, -153.6236027}, R = 188.739814439842);
          constant IdealGases.Common.DataRecord CH3COOH(name = "CH3COOH", MM = 0.06005196, Hf =  -7197917.27031058, H0 = 226427.097466927, Tlimit = 1000, alow = { -32191.9198,1196.329795, -8.70582402,0.0569625759, -5.75788716e-05,3.35211522e-08, -8.61443823e-12}, blow = { -58401.1287,72.8241392}, ahigh = {2103514.223, -14678.22192,33.8280283, -0.00569485868,1.343221353e-06, -1.606041158e-10,7.65279425e-15}, bhigh = {29242.28407, -193.527885}, R = 138.454631622348);
          constant IdealGases.Common.DataRecord OHCH2COOH(name = "OHCH2COOH", MM = 0.07605136, Hf =  -7665872.11589642, H0 = 223625.705049851, Tlimit = 1000, alow = { -313897.858,5693.06877, -36.8516029,0.1563502811, -0.0002039487993,1.340232412e-07, -3.50913026e-11}, blow = { -98016.4009,226.9492355}, ahigh = {1946628.253, -9804.0202,28.44111243, -0.000787640475,6.41627595e-08,1.069321262e-12, -3.23717828e-16}, bhigh = { -16946.5047, -147.4109363}, R = 109.32706528851);
          constant IdealGases.Common.DataRecord C2H5(name = "C2H5", MM = 0.0290611, Hf = 4083060.86142644, H0 = 419296.791931482, Tlimit = 1000, alow = { -141131.2551,2714.285088, -15.34977725,0.0645167258, -7.25914396e-05,4.59911601e-08, -1.218367535e-11}, blow = {598.141884,109.096652}, ahigh = {4169220.4, -16629.82142,27.95442134, -0.003051715761,5.68516004e-07, -5.6828636e-11,2.355648561e-15}, bhigh = {113701.0087, -163.9357995}, R = 286.103141312614);
          constant IdealGases.Common.DataRecord C2H5Br(name = "C2H5Br", MM = 0.1089651, Hf =  -583673.121026824, H0 = 124526.109736053, Tlimit = 1000, alow = { -137417.2662,2861.429418, -18.24108956,0.085662306, -0.0001047174473,6.90607457e-08, -1.862276022e-11}, blow = { -21984.80205,125.8435579}, ahigh = {2378649.403, -12670.33647,27.77558646, -0.002010898783,4.43608341e-07, -5.34819809e-11,2.63983585e-15}, bhigh = {64140.6205, -152.8277085}, R = 76.3039909108513);
          constant IdealGases.Common.DataRecord C2H6(name = "C2H6", MM = 0.03006904, Hf =  -2788633.8905399, H0 = 395476.343774194, Tlimit = 1000, alow = { -186204.4161,3406.19186, -19.51705092,0.0756583559, -8.20417322e-05,5.0611358e-08, -1.319281992e-11}, blow = { -27029.3289,129.8140496}, ahigh = {5025782.13, -20330.22397,33.2255293, -0.00383670341,7.23840586e-07, -7.3191825e-11,3.065468699e-15}, bhigh = {111596.395, -203.9410584}, R = 276.512718729963);
          constant IdealGases.Common.DataRecord CH3N2CH3(name = "CH3N2CH3", MM = 0.05808244, Hf = 2560143.13448264, H0 = 284509.380115574, Tlimit = 1000, alow = { -373849.232,5880.45313, -29.86398524,0.1087380861, -0.0001167950177,6.91689489e-08, -1.719950055e-11}, blow = { -11899.84084,194.8246321}, ahigh = {4993357.09, -21609.96161,39.6444992, -0.00419645011,8.02336198e-07, -8.21226002e-11,3.47723744e-15}, bhigh = {144996.261, -237.2109745}, R = 143.149495785645);
          constant IdealGases.Common.DataRecord C2H5OH(name = "C2H5OH", MM = 0.04606844, Hf =  -5100020.75173373, H0 = 315659.180124181, Tlimit = 1000, alow = { -234279.1392,4479.18055, -27.44817302,0.1088679162, -0.0001305309334,8.4373464e-08, -2.234559017e-11}, blow = { -50222.29,176.4829211}, ahigh = {4694817.65, -19297.98213,34.4758404, -0.00323616598,5.78494772e-07, -5.56460027e-11,2.2262264e-15}, bhigh = {86016.2271, -203.4801732}, R = 180.480867161988);
          constant IdealGases.Common.DataRecord CH3OCH3(name = "CH3OCH3", MM = 0.04606844, Hf =  -3996445.28879207, H0 = 311588.128445417, Tlimit = 1000, alow = { -269310.3242,4300.70971, -21.52788028,0.0813183339, -8.29567132e-05,4.80191151e-08, -1.188699808e-11}, blow = { -44102.3709,146.7666934}, ahigh = {4933577.19, -20830.94065,36.2905061, -0.00410835164,7.90322031e-07, -8.13143563e-11,3.45816611e-15}, bhigh = {101330.1012, -218.5447466}, R = 180.480867161988);
          constant IdealGases.Common.DataRecord CH3O2CH3(name = "CH3O2CH3", MM = 0.06206784, Hf =  -2021981.1097019, H0 = 276365.924768769, Tlimit = 1000, alow = { -228578.4757,3820.14257, -19.76647823,0.0884074386, -9.64128456e-05,5.90720083e-08, -1.526491225e-11}, blow = { -34920.1696,138.6769151}, ahigh = {5316368.47, -22212.67874,40.3433509, -0.00461274809,8.7929872e-07, -9.06822119e-11,3.86566489e-15}, bhigh = {116159.6028, -239.5296055}, R = 133.957811323868);
          constant IdealGases.Common.DataRecord CCN(name = "CCN", MM = 0.0380281, Hf = 21157945.6244198, H0 = 290272.8245692, Tlimit = 1000, alow = { -16962.81385,98.3789163,3.81266294,0.00534689423, -2.473598508e-06, -3.73056422e-10,4.48175686e-13}, blow = {94800.7257,5.553165572}, ahigh = {79486.7489, -1344.786906,8.30998646, -0.0002220105361,1.753683113e-08,2.545998719e-12, -2.645649117e-16}, bhigh = {102318.7495, -22.5979394}, R = 218.640216050762);
          constant IdealGases.Common.DataRecord CNC(name = "CNC", MM = 0.0380281, Hf = 18010748.867285, H0 = 298637.349749264, Tlimit = 1000, alow = { -70751.9271,1007.523898, -1.576789967,0.02052532634, -2.278935009e-05,1.283362343e-08, -2.933174091e-12}, blow = {76133.2485,34.87094132}, ahigh = { -90313.1356, -831.32365,8.11473595, -0.0002447691991,5.39750877e-08, -6.18398486e-12,2.865172411e-16}, bhigh = {84518.336, -21.02937255}, R = 218.640216050762);
          constant IdealGases.Common.DataRecord OCCN(name = "OCCN", MM = 0.05402754, Hf = 3886906.56653995, H0 = 251618.915094043, Tlimit = 1000, alow = {24288.01276, -552.586462,8.58783817, -0.00337938777,1.119841826e-05, -1.008408077e-08,3.086448824e-12}, blow = {25996.20072, -16.59592359}, ahigh = {935913.168, -4441.08229,13.68959297, -0.001647530917,3.81987344e-07, -3.945161e-11,1.509598839e-15}, bhigh = {49512.2278, -54.1708968}, R = 153.893218162441);
          constant IdealGases.Common.DataRecord C2N2(name = "C2N2", MM = 0.0520348, Hf = 5940255.36756171, H0 = 244358.640755802, Tlimit = 1000, alow = {108240.4484, -1928.137871,15.53891898, -0.01821159329,2.77884084e-05, -1.899434373e-08,4.94967772e-12}, blow = {44490.9759, -60.90964741}, ahigh = {793442.372, -3997.37627,13.1449743, -0.0008747782,2.059156733e-07, -2.200469389e-11,9.97448577e-16}, bhigh = {58636.323, -54.73201251}, R = 159.786758092661);
          constant IdealGases.Common.DataRecord C2O(name = "C2O", MM = 0.0400208, Hf = 7272185.11374085, H0 = 262006.706512614, Tlimit = 1000, alow = { -3959.92942, -111.7516348,4.59396006,0.00371060202, -7.01476018e-07, -1.371129839e-09,7.123854e-13}, blow = {34101.0974,0.46228056}, ahigh = { -634805.659,1184.133091,4.87917334,0.001757538773, -3.95755227e-07,3.98917994e-11, -1.546043135e-15}, bhigh = {24898.03938,0.980904059}, R = 207.753768040619);
          constant IdealGases.Common.DataRecord C3(name = "C3", MM = 0.0360321, Hf = 23311121.0837004, H0 = 336065.508255139, Tlimit = 1000, alow = { -43546.1448,666.018322,1.451033157,0.00743451312, -3.81015299e-06, -2.336961396e-11,4.40705453e-13}, blow = {96351.702,20.25173297}, ahigh = {4508098.93, -14610.33761,22.81974644, -0.00854434061,2.146069341e-06, -2.103867761e-10,6.35158906e-15}, bhigh = {191197.6065, -127.1869723}, R = 230.751801865559);
          constant IdealGases.Common.DataRecord C3H3_1_propynl(name = "C3H3_1_propynl", MM = 0.03905592, Hf = 11521940.8478919, H0 = 317493.481141911, Tlimit = 1000, alow = { -65058.5935,1350.858921, -5.82543393,0.0375661048, -3.73490334e-05,2.117676603e-08, -5.13911325e-12}, blow = {46565.1053,57.8147755}, ahigh = {4550654.87, -16405.74172,27.12605991, -0.00447460038,1.037712415e-06, -1.250211369e-10,6.02658205e-15}, bhigh = {153408.7662, -156.5931809}, R = 212.886343478786);
          constant IdealGases.Common.DataRecord C3H3_2_propynl(name = "C3H3_2_propynl", MM = 0.03905592, Hf = 8495511.05184566, H0 = 338745.060928023, Tlimit = 1000, alow = {61885.7832, -890.957867,6.34755882,0.01633173115, -1.949975695e-05,1.417349778e-08, -4.19986632e-12}, blow = {42717.8583, -12.31400729}, ahigh = {2989723.833, -11189.54446,22.22225052, -0.002068106902,4.12188364e-07, -4.43898059e-11,1.970824701e-15}, bhigh = {106187.8289, -118.6744583}, R = 212.886343478786);
          constant IdealGases.Common.DataRecord C3H4_allene(name = "C3H4_allene", MM = 0.04006386, Hf = 4765392.05158964, H0 = 314611.947026572, Tlimit = 1000, alow = { -16451.55745,962.945781, -7.53232668,0.0551821911, -6.73358512e-05,4.53270905e-08, -1.251837614e-11}, blow = {17724.94269,61.5196976}, ahigh = {3479355.1, -14304.12453,27.02534756, -0.002557412369,4.70664675e-07, -4.65168807e-11,1.908219044e-15}, bhigh = {107231.2354, -154.8846158}, R = 207.530477592524);
          constant IdealGases.Common.DataRecord C3H4_propyne(name = "C3H4_propyne", MM = 0.04006386, Hf = 4615131.94185483, H0 = 325243.648515146, Tlimit = 1000, alow = { -35638.844,832.81391, -4.07375944,0.0411392961, -4.47044495e-05,2.847458197e-08, -7.69529824e-12}, blow = {17102.06236,45.1672095}, ahigh = {3710441.42, -14891.45507,27.32397127, -0.00264526477,4.85830035e-07, -4.79412848e-11,1.964338121e-15}, bhigh = {110489.8462, -156.7992462}, R = 207.530477592524);
          constant IdealGases.Common.DataRecord C3H4_cyclo(name = "C3H4_cyclo", MM = 0.04006386, Hf = 6916457.87500256, H0 = 283888.34725361, Tlimit = 1000, alow = { -19695.20627,1505.379338, -14.18206573,0.0764263296, -9.76558366e-05,6.61200382e-08, -1.811251145e-11}, blow = {26256.31782,96.0463189}, ahigh = {3168399.58, -13710.44699,26.64303646, -0.00242040805,4.42799413e-07, -4.3518202e-11,1.775948955e-15}, bhigh = {113368.3702, -152.2619086}, R = 207.530477592524);
          constant IdealGases.Common.DataRecord C3H5_allyl(name = "C3H5_allyl", MM = 0.0410718, Hf = 3983131.97863254, H0 = 310157.407272143, Tlimit = 1000, alow = { -43159.9614,1441.600907, -11.97014426,0.0731979646, -9.06635785e-05,6.07705945e-08, -1.658826363e-11}, blow = {12321.5746,85.6317324}, ahigh = {4094570.59, -16766.76186,31.23006342, -0.002885449982,5.21134354e-07, -5.05828422e-11,2.039932554e-15}, bhigh = {118572.0481, -182.3070197}, R = 202.437487521852);
          constant IdealGases.Common.DataRecord C3H6_propylene(name = "C3H6_propylene", MM = 0.04207974, Hf = 475288.107768727, H0 = 322020.953551519, Tlimit = 1000, alow = { -191246.2174,3542.07424, -21.14878626,0.0890148479, -0.0001001429154,6.26795939e-08, -1.637870781e-11}, blow = { -15299.61824,140.7641382}, ahigh = {5017620.34, -20860.84035,36.4415634, -0.00388119117,7.27867719e-07, -7.3212045e-11,3.052176369e-15}, bhigh = {126124.5355, -219.5715757}, R = 197.588483198803);
          constant IdealGases.Common.DataRecord C3H6_cyclo(name = "C3H6_cyclo", MM = 0.04207974, Hf = 1266642.80720366, H0 = 271151.413958356, Tlimit = 1000, alow = { -156578.777,4111.12987, -32.3344746,0.1306337881, -0.0001645563833,1.095708326e-07, -2.956394783e-11}, blow = { -12452.71686,193.1559109}, ahigh = {4785000.67, -20421.18175,36.3149578, -0.00356131944,6.47624124e-07, -6.3284301e-11,2.568705857e-15}, bhigh = {126827.4126, -222.3729099}, R = 197.588483198803);
          constant IdealGases.Common.DataRecord C3H6O_propylox(name = "C3H6O_propylox", MM = 0.05807914, Hf =  -1613660.25736607, H0 = 248031.444680483, Tlimit = 1000, alow = { -229280.8804,4495.75054, -29.41117945,0.1213113827, -0.0001440060464,9.20205175e-08, -2.416278343e-11}, blow = { -33176.9721,184.6878218}, ahigh = {4789729.99, -21068.95971,39.5464773, -0.00391092998,7.32553151e-07, -7.35708597e-11,3.0606185e-15}, bhigh = {112034.454, -237.2004192}, R = 143.15762940016);
          constant IdealGases.Common.DataRecord C3H6O_acetone(name = "C3H6O_acetone", MM = 0.05807914, Hf =  -3738857.01475607, H0 = 278812.5478442, Tlimit = 1000, alow = { -227780.2525,4215.28001, -24.15785316,0.0990748332, -0.0001084940903,6.58335596e-08, -1.676046146e-11}, blow = { -47262.4394,160.7926432}, ahigh = {5001601.92, -21701.55542,39.6449399, -0.00417994505,7.96242953e-07, -8.12255805e-11,3.42878096e-15}, bhigh = {101514.5028, -236.8533477}, R = 143.15762940016);
          constant IdealGases.Common.DataRecord C3H6O_propanal(name = "C3H6O_propanal", MM = 0.05807914, Hf =  -3202526.75917722, H0 = 298339.059428222, Tlimit = 1000, alow = { -265578.1702,4250.64045, -21.09249262,0.0919425612, -0.0001004653044,6.13331482e-08, -1.576298535e-11}, blow = { -44503.7105,145.6678605}, ahigh = {4830933.49, -20754.51152,39.2485518, -0.004095514,7.88169216e-07, -8.10768408e-11,3.43710543e-15}, bhigh = {99449.3788, -231.6690627}, R = 143.15762940016);
          constant IdealGases.Common.DataRecord C3H7_n_propyl(name = "C3H7_n_propyl", MM = 0.04308768, Hf = 2332453.26738409, H0 = 344879.51080216, Tlimit = 1000, alow = { -189533.7073,3949.51726, -26.06216089,0.1121920441, -0.0001365292213,9.02366272e-08, -2.44105699e-11}, blow = { -7227.87744,167.3705556}, ahigh = {5646512.94, -22910.87136,39.8727518, -0.00410623287,7.56255777e-07, -7.47826302e-11,3.068983677e-15}, bhigh = {148300.6853, -240.378119}, R = 192.966342119139);
          constant IdealGases.Common.DataRecord C3H7_i_propyl(name = "C3H7_i_propyl", MM = 0.04308768, Hf = 2165352.13778045, H0 = 343652.918885398, Tlimit = 1000, alow = { -295206.3445,5294.4323, -31.05287013,0.1143871563, -0.0001291752393,8.05784376e-08, -2.093908432e-11}, blow = { -14768.15514,198.808236}, ahigh = {5807002.52, -24112.19997,40.852884, -0.00451785133,8.49942717e-07, -8.57351434e-11,3.58338396e-15}, bhigh = {154650.405, -248.7098372}, R = 192.966342119139);
          constant IdealGases.Common.DataRecord C3H8(name = "C3H8", MM = 0.04409562, Hf =  -2373931.92339738, H0 = 334301.184562095, Tlimit = 1000, alow = { -243314.4337,4656.27081, -29.39466091,0.1188952745, -0.0001376308269,8.81482391e-08, -2.342987994e-11}, blow = { -35403.3527,184.1749277}, ahigh = {6420731.68, -26597.91134,45.3435684, -0.00502066392,9.47121694e-07, -9.57540523e-11,4.00967288e-15}, bhigh = {145558.2459, -281.8374734}, R = 188.555507327032);
          constant IdealGases.Common.DataRecord C3H8O_1propanol(name = "C3H8O_1propanol", MM = 0.06009502, Hf =  -4246608.12160475, H0 = 291517.649881804, Tlimit = 1000, alow = { -261697.3337,5192.37666, -32.9648116,0.1354568128, -0.0001593156164,1.01949816e-07, -2.688552974e-11}, blow = { -56128.5435,208.5024431}, ahigh = {6308672.12, -26422.10376,47.1511259, -0.00464251193,8.59346536e-07, -8.68209182e-11,3.64222401e-15}, bhigh = {125500.3155, -285.9463804}, R = 138.355424459464);
          constant IdealGases.Common.DataRecord C3H8O_2propanol(name = "C3H8O_2propanol", MM = 0.06009502, Hf =  -4537813.61583705, H0 = 287291.27638197, Tlimit = 1000, alow = { -338651.024,6106.048, -37.9141804,0.1530494531, -0.0001864354461,1.213257738e-07, -3.22043349e-11}, blow = { -62799.5935,233.432261}, ahigh = {6001074.75, -25058.7683,46.2209612, -0.00427246631,7.69367877e-07, -7.45058484e-11,2.998959935e-15}, bhigh = {114851.8732, -279.6132222}, R = 138.355424459464);
          constant IdealGases.Common.DataRecord CNCOCN(name = "CNCOCN", MM = 0.08004498, Hf = 3092011.51652483, H0 = 214226.52613568, Tlimit = 1000, alow = {113105.2075, -1978.834961,16.26886206, -0.00833080344,1.884127045e-05, -1.530681289e-08,4.4181253e-12}, blow = {36802.6174, -59.6332728}, ahigh = {700052.244, -5086.00201,19.4675349, -0.001302852727,2.75360075e-07, -3.056290852e-11,1.382140838e-15}, bhigh = {55393.3888, -86.2755086}, R = 103.872497688175);
          constant IdealGases.Common.DataRecord C3O2(name = "C3O2", MM = 0.0680309, Hf =  -1376403.95761338, H0 = 221737.225290273, Tlimit = 1000, alow = {157987.3382, -2529.493506,18.01761578, -0.01786032042,2.978671986e-05, -2.182900022e-08,6.01379722e-12}, blow = { -1121.054014, -72.7772117}, ahigh = {696869.989, -4624.73319,16.63905725, -0.001175486554,2.478106444e-07, -2.745165984e-11,1.239566766e-15}, bhigh = {12525.80069, -72.7596878}, R = 122.216110620321);
          constant IdealGases.Common.DataRecord C4(name = "C4", MM = 0.0480428, Hf = 21520472.2039515, H0 = 273042.828477941, Tlimit = 1000, alow = {39037.805, -894.828078,10.50952925, -0.00655289446,1.243940464e-05, -8.64534137e-09,2.263638846e-12}, blow = {126642.5869, -30.77594475}, ahigh = {920068.513, -1530.3118,6.0500692,0.00525274367, -1.779154772e-06,2.589873632e-10, -1.385553481e-14}, bhigh = {133438.9611, -7.26114882}, R = 173.063851399169);
          constant IdealGases.Common.DataRecord C4H2_butadiyne(name = "C4H2_butadiyne", MM = 0.05005868, Hf = 8989449.98150171, H0 = 287539.743357196, Tlimit = 1000, alow = {246754.2569, -3897.85564,23.66080456, -0.02208077805,2.78110114e-05, -1.57734001e-08,3.42316546e-12}, blow = {70869.0782, -110.917356}, ahigh = {2328179.913, -8925.18609,21.14326883, -0.001368871276,2.327503159e-07, -2.124517624e-11,8.05331302e-16}, bhigh = {105778.8416, -108.8313574}, R = 166.094511481326);
          constant IdealGases.Common.DataRecord C4H4_1_3minuscyclo(name = "C4H4_1_3minuscyclo", MM = 0.05207456, Hf = 7393245.37739733, H0 = 232427.004664082, Tlimit = 1000, alow = { -27784.28049,1768.176915, -17.57895171,0.0938351292, -0.0001195524281,7.97808619e-08, -2.1527511e-11}, blow = {38116.2712,112.8478124}, ahigh = {2991498.949, -14167.81502,30.01978876, -0.002579200423,4.78999045e-07, -4.77532971e-11,1.974923662e-15}, bhigh = {127466.692, -172.7532057}, R = 159.66475760909);
          constant IdealGases.Common.DataRecord C4H6_butadiene(name = "C4H6_butadiene", MM = 0.05409044, Hf = 2033631.08157375, H0 = 279716.711492826, Tlimit = 1000, alow = { -91811.3053,3312.57053, -29.85828611,0.1479201147, -0.0002056618326,1.466496826e-07, -4.14528573e-11}, blow = { -2077.309444,178.0687329}, ahigh = { -23619031.88,56513.2337, -32.7573832,0.02293070572, -2.297106441e-06,4.29259621e-11,4.23676604e-15}, bhigh = { -367135.862,301.3437302}, R = 153.714260782497);
          constant IdealGases.Common.DataRecord C4H6_1butyne(name = "C4H6_1butyne", MM = 0.05409044, Hf = 3054144.13341803, H0 = 296170.635698286, Tlimit = 1000, alow = { -55970.3997,1433.191711, -9.69121072,0.0715000239, -8.1579678e-05,5.29036497e-08, -1.435655372e-11}, blow = {11849.87013,76.6022536}, ahigh = {6364402.7, -23920.87731,40.7375041, -0.00317672649,1.199856984e-07,3.20180251e-11, -2.854392633e-15}, bhigh = {163433.5546, -246.0284791}, R = 153.714260782497);
          constant IdealGases.Common.DataRecord C4H6_2butyne(name = "C4H6_2butyne", MM = 0.05409044, Hf = 2693636.80532087, H0 = 307632.919976247, Tlimit = 1000, alow = { -265075.6405,4490.72869, -24.00723889,0.0981957955, -0.0001079717182,6.67555577e-08, -1.740766886e-11}, blow = { -5328.36371,159.5035268}, ahigh = {3981304.33, -18730.32899,36.5456149, -0.002369686378,3.99021181e-07, -3.69870727e-11,1.442072006e-15}, bhigh = {126146.0214, -215.5866205}, R = 153.714260782497);
          constant IdealGases.Common.DataRecord C4H6_cyclo(name = "C4H6_cyclo", MM = 0.05409044, Hf = 2896999.91347824, H0 = 232158.658720469, Tlimit = 1000, alow = { -204670.7734,4919.47057, -37.5327816,0.1497293031, -0.0001860139375,1.219909019e-07, -3.25407359e-11}, blow = { -3915.95054,223.3282138}, ahigh = {4517367.82, -20973.11985,40.0803917, -0.00394979398,7.44848499e-07, -7.52975417e-11,3.153253502e-15}, bhigh = {140646.9848, -243.4836999}, R = 153.714260782497);
          constant IdealGases.Common.DataRecord C4H8_1_butene(name = "C4H8_1_butene", MM = 0.05610632, Hf =  -9624.58418231672, H0 = 305134.965187523, Tlimit = 1000, alow = { -272149.2014,5100.07925, -31.8378625,0.1317754442, -0.0001527359339,9.71476111e-08, -2.56020447e-11}, blow = { -25230.96386,200.6932108}, ahigh = {6257948.61, -26603.76305,47.6492005, -0.00438326711,7.12883844e-07, -5.99102084e-11,2.051753504e-15}, bhigh = {156925.2657, -291.3869761}, R = 148.191362399102);
          constant IdealGases.Common.DataRecord C4H8_cis2_buten(name = "C4H8_cis2_buten", MM = 0.05610632, Hf =  -131892.449905822, H0 = 299431.507894298, Tlimit = 1000, alow = { -277387.0877,5382.38404, -33.751881,0.1322980623, -0.0001490975922,9.277722e-08, -2.408282948e-11}, blow = { -27158.84347,211.4462085}, ahigh = {6461018.35, -27753.76432,48.6353236, -0.00486238635,8.4126261e-07, -7.63389037e-11,2.861702826e-15}, bhigh = {163085.6187, -300.3105998}, R = 148.191362399102);
          constant IdealGases.Common.DataRecord C4H8_isobutene(name = "C4H8_isobutene", MM = 0.05610632, Hf =  -304778.499106696, H0 = 303174.401742977, Tlimit = 1000, alow = { -232720.5032,3941.99424, -22.24581184,0.1012790864, -0.0001073194065,6.45469691e-08, -1.646330345e-11}, blow = { -22337.66063,147.9597621}, ahigh = {6484970.99, -27325.04764,48.3632108, -0.00476800405,8.23387584e-07, -7.449253e-11,2.782303056e-15}, bhigh = {159594.1773, -298.2986237}, R = 148.191362399102);
          constant IdealGases.Common.DataRecord C4H8_cyclo(name = "C4H8_cyclo", MM = 0.05610632, Hf = 506181.834773694, H0 = 241223.502093882, Tlimit = 1000, alow = { -304765.5983,6519.48273, -46.6298759,0.1743593052, -0.0002090964176,1.343528679e-07, -3.5427274e-11}, blow = { -27000.31171,273.8348195}, ahigh = {4456213.81, -23010.18492,44.9244846, -0.003080145176,5.31716526e-07, -5.08107938e-11,2.048919092e-15}, bhigh = {135548.7603, -277.1801965}, R = 148.191362399102);
          constant IdealGases.Common.DataRecord C4H9_n_butyl(name = "C4H9_n_butyl", MM = 0.05711426, Hf = 1164857.95316266, H0 = 346620.966462666, Tlimit = 1000, alow = { -223956.0407,4676.5541, -29.85424449,0.1345493704, -0.000160066035,1.045338096e-07, -2.812951048e-11}, blow = { -15252.97704,190.1651559}, ahigh = {7198686.94, -29592.41524,52.4204281, -0.00544157244,9.91775837e-07, -9.46445587e-11,3.72948704e-15}, bhigh = {183456.6472, -321.420917}, R = 145.576113566034);
          constant IdealGases.Common.DataRecord C4H9_i_butyl(name = "C4H9_i_butyl", MM = 0.05711426, Hf = 1003602.25274739, H0 = 320707.998317758, Tlimit = 1000, alow = { -239946.1281,4697.44454, -30.8747256,0.1391655831, -0.0001670233968,1.092423088e-07, -2.936209962e-11}, blow = { -16381.51036,193.6662372}, ahigh = {6752936.06, -28374.23721,51.4068761, -0.00498239901,8.80150663e-07, -8.12573869e-11,3.099140009e-15}, bhigh = {174344.848, -314.9940745}, R = 145.576113566034);
          constant IdealGases.Common.DataRecord C4H9_s_butyl(name = "C4H9_s_butyl", MM = 0.05711426, Hf = 1243122.12046519, H0 = 307062.526941608, Tlimit = 1000, alow = { -335106.289,6312.94595, -40.0302526,0.1563875047, -0.0001842331788,1.182727848e-07, -3.131156589e-11}, blow = { -22160.45659,248.1295714}, ahigh = {7224744.35, -30352.91099,52.8855183, -0.00565208355,1.060032692e-06, -1.066129835e-10,4.44381096e-15}, bhigh = {188349.4279, -325.571988}, R = 145.576113566034);
          constant IdealGases.Common.DataRecord C4H9_t_butyl(name = "C4H9_t_butyl", MM = 0.05711426, Hf = 905203.008845777, H0 = 297818.355696108, Tlimit = 1000, alow = { -472346.195,8090.19877, -46.8367534,0.1575300095, -0.0001686559436,9.98104013e-08, -2.487608823e-11}, blow = { -33193.6741,289.4838706}, ahigh = {7151064.95, -31712.2441,54.4251032, -0.00639233116,1.241097612e-06, -1.2872483e-10,5.51261874e-15}, bhigh = {193450.6997, -339.932529}, R = 145.576113566034);
          constant IdealGases.Common.DataRecord C4H10_n_butane(name = "C4H10_n_butane", MM = 0.0581222, Hf =  -2164233.28779709, H0 = 330832.022875941, Tlimit = 1000, alow = { -317587.254,6176.33182, -38.9156212,0.1584654284, -0.0001860050159,1.199676349e-07, -3.20167055e-11}, blow = { -45403.6339,237.9488665}, ahigh = {7682322.45, -32560.5151,57.3673275, -0.00619791681,1.180186048e-06, -1.221893698e-10,5.25063525e-15}, bhigh = {177452.656, -358.791876}, R = 143.051570656307);
          constant IdealGases.Common.DataRecord C4H10_isobutane(name = "C4H10_isobutane", MM = 0.0581222, Hf =  -2322520.48270712, H0 = 308599.812119982, Tlimit = 1000, alow = { -383446.933,7000.03964, -44.400269,0.1746183447, -0.0002078195348,1.339792433e-07, -3.55168163e-11}, blow = { -50340.1889,265.8966497}, ahigh = {7528018.92, -32025.1706,57.00161, -0.00606001309,1.143975809e-06, -1.157061835e-10,4.84604291e-15}, bhigh = {172850.0802, -357.617689}, R = 143.051570656307);
          constant IdealGases.Common.DataRecord C4N2(name = "C4N2", MM = 0.0760562, Hf = 6958012.6275044, H0 = 234029.586016656, Tlimit = 1000, alow = {158780.2866, -2987.184206,23.48081602, -0.02607502448,4.04283003e-05, -2.804912444e-08,7.39765205e-12}, blow = {75052.9947, -101.757825}, ahigh = {1167686.152, -6198.64418,20.62070093, -0.001518619449,3.16236168e-07, -3.4699228e-11,1.555154128e-15}, bhigh = {96674.0939, -96.69734738}, R = 109.320108025381);
          constant IdealGases.Common.DataRecord C5(name = "C5", MM = 0.0600535, Hf = 17499801.5436236, H0 = 269623.435769772, Tlimit = 1000, alow = { -12008.01119, -555.370291,11.23828271, -0.00434788452,1.73898749e-05, -1.707945418e-08,5.57454191e-12}, blow = {126240.4627, -32.6230899}, ahigh = {217205.5356, -2958.510027,15.61080967, -0.000820036192,1.776898025e-07, -2.009853583e-11,9.22267777e-16}, bhigh = {139520.8064, -64.3307718}, R = 138.451081119335);
          constant IdealGases.Common.DataRecord C5H6_1_3cyclo(name = "C5H6_1_3cyclo", MM = 0.06610114, Hf = 2031735.00487284, H0 = 204769.978248484, Tlimit = 1000, alow = { -188625.9728,4738.22087, -38.8895801,0.1667438533, -0.0002141149581,1.438132328e-07, -3.90647811e-11}, blow = { -5667.02201,227.9898557}, ahigh = {4428478.19, -21235.47976,43.0981901, -0.00391478346,7.31184701e-07, -7.32724119e-11,3.044403403e-15}, bhigh = {138254.2782, -260.5959678}, R = 125.78409389006);
          constant IdealGases.Common.DataRecord C5H8_cyclo(name = "C5H8_cyclo", MM = 0.06811702, Hf = 497672.975124279, H0 = 218112.580380058, Tlimit = 1000, alow = { -263111.4588,5987.75349, -45.222446,0.1804626339, -0.0002177216062,1.402022671e-07, -3.6990943e-11}, blow = { -23795.04749,266.0136401}, ahigh = {4569848.1, -24060.18294,48.8100646, -0.00341305498,6.04700166e-07, -5.92749578e-11,2.44786539e-15}, bhigh = {141398.3093, -298.9533527}, R = 122.061593416741);
          constant IdealGases.Common.DataRecord C5H10_1_pentene(name = "C5H10_1_pentene", MM = 0.0701329, Hf =  -303423.927999555, H0 = 309127.38529278, Tlimit = 1000, alow = { -534054.813,9298.91738, -56.6779245,0.2123100266, -0.000257129829,1.666834304e-07, -4.43408047e-11}, blow = { -47906.8218,339.60364}, ahigh = {3744014.97, -21044.85321,47.3612699, -0.00042442012, -3.89897505e-08,1.367074243e-11, -9.31319423e-16}, bhigh = {115409.1373, -278.6177449}, R = 118.553089919282);
          constant IdealGases.Common.DataRecord C5H10_cyclo(name = "C5H10_cyclo", MM = 0.0701329, Hf =  -1099341.39326907, H0 = 214212.402453057, Tlimit = 1000, alow = { -414111.971,8627.5928, -62.0295998,0.2259910921, -0.000268230333,1.706289935e-07, -4.46405092e-11}, blow = { -49315.2214,358.391623}, ahigh = {7501938.73, -35058.6485,63.2248075, -0.00694035658,1.337306593e-06, -1.377905033e-10,5.86735764e-15}, bhigh = {195492.5511, -402.65509}, R = 118.553089919282);
          constant IdealGases.Common.DataRecord C5H11_pentyl(name = "C5H11_pentyl", MM = 0.07114084, Hf = 643933.920375413, H0 = 343290.857965692, Tlimit = 1000, alow = { -465371.592,8564.22042, -52.9524289,0.2094288859, -0.0002561602906,1.692755961e-07, -4.59678811e-11}, blow = { -36417.0427,319.686224}, ahigh = {5697252.9, -28917.06175,58.1102968, -0.00359399501,4.41996763e-07, -1.509664551e-11, -6.62696443e-16}, bhigh = {171700.955, -352.247712}, R = 116.873402113329);
          constant IdealGases.Common.DataRecord C5H11_t_pentyl(name = "C5H11_t_pentyl", MM = 0.07114084, Hf = 458245.924563162, H0 = 276124.417423241, Tlimit = 1000, alow = { -515218.198,9144.32783, -56.0239789,0.1998204194, -0.0002239112591,1.375455547e-07, -3.52383305e-11}, blow = { -40362.6661,340.2811}, ahigh = {8602108.26, -38052.0577,66.4969037, -0.00753386063,1.451612787e-06, -1.495593207e-10,6.36802233e-15}, bhigh = {228185.8805, -416.940855}, R = 116.873402113329);
          constant IdealGases.Common.DataRecord C5H12_n_pentane(name = "C5H12_n_pentane", MM = 0.07214878, Hf =  -2034130.02964153, H0 = 335196.243096557, Tlimit = 1000, alow = { -276889.4625,5834.28347, -36.1754148,0.1533339707, -0.0001528395882,8.191092e-08, -1.792327902e-11}, blow = { -46653.7525,226.5544053}, ahigh = { -2530779.286, -8972.59326,45.3622326, -0.002626989916,3.135136419e-06, -5.31872894e-10,2.886896868e-14}, bhigh = {14846.16529, -251.6550384}, R = 115.240645787774);
          constant IdealGases.Common.DataRecord C5H12_i_pentane(name = "C5H12_i_pentane", MM = 0.07214878, Hf =  -2130320.15232967, H0 = 305036.342956873, Tlimit = 1000, alow = { -423190.339,6497.1891, -36.8112697,0.1532424729, -0.0001548790714,8.74989712e-08, -2.07054771e-11}, blow = { -51554.1659,230.9518218}, ahigh = {11568885.94, -45562.4687,74.9544363, -0.00784541558,1.444393314e-06, -1.464370213e-10,6.230285e-15}, bhigh = {254492.7135, -480.198578}, R = 115.240645787774);
          constant IdealGases.Common.DataRecord CH3C_CH3_2CH3(name = "CH3C_CH3_2CH3", MM = 0.07214878, Hf =  -2327412.88210279, H0 = 321266.69362947, Tlimit = 1000, alow = { -8973222.27,128922.5617, -719.934483,2.056862183, -0.002953159699,2.158893146e-06, -6.26831877e-10}, blow = { -639493.202,4020.80636}, ahigh = {16847055.2, -59794.3057,86.8545148, -0.0096219176,1.653363091e-06, -1.674727926e-10,7.37211936e-15}, bhigh = {345849.682, -576.046697}, R = 115.240645787774);
          constant IdealGases.Common.DataRecord C6D5_phenyl(name = "C6D5_phenyl", MM = 0.08213471, Hf = 3844172.57941253, H0 = 193816.682374601, Tlimit = 1000, alow = {201283.7008, -1979.349332,2.6282675,0.0595186526, -6.08128045e-05,3.38139165e-08, -8.12542908e-12}, blow = {46972.4749,0.335470098}, ahigh = {1411628.125, -13625.69472,40.2898494, -0.00349098927,7.37961213e-07, -8.19224405e-11,3.70533097e-15}, bhigh = {108807.3731, -229.3621057}, R = 101.229699356094);
          constant IdealGases.Common.DataRecord C6D6(name = "C6D6", MM = 0.084148812, Hf = 691125.360153629, H0 = 193997.819006643, Tlimit = 1000, alow = {276291.1236, -2865.868902,5.39507043,0.0593931517, -6.02363118e-05,3.38139857e-08, -8.30604637e-12}, blow = {20470.83778, -20.11790696}, ahigh = {1758057.871, -15721.21558,44.6816249, -0.00400336192,8.4454344e-07, -9.36055698e-11,4.22847553e-15}, bhigh = {89620.4536, -261.5332268}, R = 98.8067662797188);
          constant IdealGases.Common.DataRecord C6H2(name = "C6H2", MM = 0.07408008, Hf = 9044266.69085671, H0 = 264682.678528425, Tlimit = 1000, alow = {290372.2964, -4929.7515,31.8932321, -0.03119447315,4.32576368e-05, -2.732517022e-08,6.67444611e-12}, blow = {101189.8682, -153.0593012}, ahigh = {2592848.577, -10833.61978,28.47459495, -0.001876727704,3.41213428e-07, -3.33739289e-11,1.356853889e-15}, bhigh = {141851.7294, -149.4853494}, R = 112.236271883076);
          constant IdealGases.Common.DataRecord C6H5_phenyl(name = "C6H5_phenyl", MM = 0.0771039, Hf = 4373319.63752806, H0 = 183578.690572072, Tlimit = 1000, alow = { -121127.8245,3529.04558, -31.16903422,0.146755063, -0.0001831398296,1.192576957e-07, -3.14926586e-11}, blow = {24209.72928,186.8799946}, ahigh = {3670279.23, -18946.01209,41.8058182, -0.00350391415,6.56182174e-07, -6.59471445e-11,2.748094212e-15}, bhigh = {147674.4628, -247.5301142}, R = 107.834649090383);
          constant IdealGases.Common.DataRecord C6H5O_phenoxy(name = "C6H5O_phenoxy", MM = 0.0931033, Hf = 512334.149272904, H0 = 174087.212805561, Tlimit = 1000, alow = { -129226.4217,3406.74068, -29.11367361,0.1459180378, -0.0001780202758,1.138615885e-07, -2.967142152e-11}, blow = { -10550.26758,177.0682011}, ahigh = {3678640.34, -19729.80806,45.4118441, -0.00375106439,7.11445078e-07, -7.23332885e-11,3.045637067e-15}, bhigh = {116359.5961, -268.1058539}, R = 89.3037303726076);
          constant IdealGases.Common.DataRecord C6H6(name = "C6H6", MM = 0.07811184, Hf = 1061042.73052587, H0 = 181735.457774391, Tlimit = 1000, alow = { -167734.0902,4404.50004, -37.1737791,0.1640509559, -0.0002020812374,1.307915264e-07, -3.4442841e-11}, blow = { -10354.55401,216.9853345}, ahigh = {4538575.72, -22605.02547,46.940073, -0.00420667683,7.90799433e-07, -7.9683021e-11,3.32821208e-15}, bhigh = {139146.4686, -286.8751333}, R = 106.443171739393);
          constant IdealGases.Common.DataRecord C6H5OH_phenol(name = "C6H5OH_phenol", MM = 0.09411124, Hf =  -1024309.10484231, H0 = 185915.093670002, Tlimit = 1000, alow = { -120941.8144,3378.29227, -29.91846148,0.1567802942, -0.0001970937198,1.291815064e-07, -3.42435126e-11}, blow = { -27793.87017,179.9708977}, ahigh = {4462081.57, -21655.21026,48.1501505, -0.00356828243,6.32717573e-07, -6.03990572e-11,2.399168678e-15}, bhigh = {111372.6204, -286.0454446}, R = 88.3472792410343);
          constant IdealGases.Common.DataRecord C6H10_cyclo(name = "C6H10_cyclo", MM = 0.0821436, Hf =  -55999.4935697973, H0 = 210251.121207252, Tlimit = 1000, alow = { -375028.221,7643.87675, -55.1043476,0.2166231405, -0.0002545452198,1.607607304e-07, -4.18524841e-11}, blow = { -36610.7301,320.240946}, ahigh = {7510128.98, -35568.5821,67.0303494, -0.00704535187,1.3581664e-06, -1.400074492e-10,5.9645517e-15}, bhigh = {206027.4332, -423.819766}, R = 101.218743760926);
          constant IdealGases.Common.DataRecord C6H12_1_hexene(name = "C6H12_1_hexene", MM = 0.08415948, Hf =  -498458.403022452, H0 = 311788.998696285, Tlimit = 1000, alow = { -666883.165,11768.64939, -72.7099833,0.2709398396, -0.00033332464,2.182347097e-07, -5.85946882e-11}, blow = { -62157.8054,428.682564}, ahigh = {733290.696, -14488.48641,46.7121549,0.00317297847, -5.24264652e-07,4.28035582e-11, -1.472353254e-15}, bhigh = {66977.4041, -262.3643854}, R = 98.7942415994015);
          constant IdealGases.Common.DataRecord C6H12_cyclo(name = "C6H12_cyclo", MM = 0.08415948, Hf =  -1465075.5921971, H0 = 208471.036180357, Tlimit = 1000, alow = { -567998.704,10342.38704, -68.0004125,0.2387797658, -0.0002511890049,1.425293184e-07, -3.40783319e-11}, blow = { -64046.3516,393.480821}, ahigh = {5225149.47, -33641.9458,71.7460747, -0.00669897912,1.318443254e-06, -1.390794789e-10,6.06010224e-15}, bhigh = {173253.7609, -454.681417}, R = 98.7942415994015);
          constant IdealGases.Common.DataRecord C6H13_n_hexyl(name = "C6H13_n_hexyl", MM = 0.08516742, Hf = 294713.635801108, H0 = 340306.187506913, Tlimit = 1000, alow = { -1427278.22,22488.28093, -129.749224,0.427979733, -0.000555601318,3.79125694e-07, -1.048462404e-10}, blow = { -106026.1015,749.718676}, ahigh = {5967938.62, -32990.2316,68.6907344, -0.00422500906,5.49652382e-07, -2.292851471e-11, -5.08634189e-16}, bhigh = {190697.8683, -418.5362}, R = 97.6250307922912);
          constant IdealGases.Common.DataRecord C6H14_n_hexane(name = "C6H14_n_hexane", MM = 0.08617536, Hf =  -1936980.59398882, H0 = 333065.043186359, Tlimit = 1000, alow = { -581592.67,10790.97724, -66.3394703,0.2523715155, -0.0002904344705,1.802201514e-07, -4.61722368e-11}, blow = { -72715.4457,393.828354}, ahigh = { -3106625.684, -7346.08792,46.9413176,0.001693963977,2.068996667e-06, -4.21214168e-10,2.452345845e-14}, bhigh = {523.750312, -254.9967718}, R = 96.4831710595697);
          constant IdealGases.Common.DataRecord C7H7_benzyl(name = "C7H7_benzyl", MM = 0.09113048, Hf = 2309874.80807739, H0 = 203607.969583832, Tlimit = 1000, alow = { -183676.4826,4102.46566, -32.024061,0.1588249575, -0.0001894466924,1.203649671e-07, -3.136589637e-11}, blow = {5266.33323,193.875172}, ahigh = {5297322.16, -25999.09398,55.0975079, -0.00497766147,9.46315243e-07, -9.63900886e-11,4.06447042e-15}, bhigh = {173838.2912, -334.382955}, R = 91.2370043480513);
          constant IdealGases.Common.DataRecord C7H8(name = "C7H8", MM = 0.09213842, Hf = 544506.840903067, H0 = 194710.794910527, Tlimit = 1000, alow = { -287796.222,6133.94152, -45.7470676,0.1936895724, -0.0002304305304,1.459301178e-07, -3.7907961e-11}, blow = { -23084.02499,269.3915042}, ahigh = {6184538.35, -29902.84056,59.8200597, -0.00569698396,1.080748416e-06, -1.098702235e-10,4.62474022e-15}, bhigh = {178204.7857, -369.808225}, R = 90.2389253039069);
          constant IdealGases.Common.DataRecord C7H8O_cresol_mx(name = "C7H8O_cresol_mx", MM = 0.10813782, Hf =  -1223420.26129249, H0 = 201949.179297308, Tlimit = 1000, alow = { -244141.7503,5080.87784, -37.8904804,0.186494507, -0.0002269511868,1.458989279e-07, -3.82240335e-11}, blow = { -40936.5783,228.1352234}, ahigh = {6017373.45, -28498.82774,60.8154307, -0.00499680711,9.13427995e-07, -8.97944411e-11,3.66775697e-15}, bhigh = {147221.84, -367.485014}, R = 76.8877345594723);
          constant IdealGases.Common.DataRecord C7H14_1_heptene(name = "C7H14_1_heptene", MM = 0.09818606, Hf =  -639194.606647828, H0 = 313588.303675695, Tlimit = 1000, alow = { -744940.284,13321.79893, -82.8169438,0.3108065994, -0.000378677992,2.446841042e-07, -6.48876387e-11}, blow = { -72178.8501,485.667149}, ahigh = { -1927608.174, -9125.02442,47.4817797,0.00606766053, -8.68485908e-07,5.81399526e-11, -1.473979569e-15}, bhigh = {26009.14656, -256.2880707}, R = 84.6807785137727);
          constant IdealGases.Common.DataRecord C7H15_n_heptyl(name = "C7H15_n_heptyl", MM = 0.099194, Hf = 44256.7090751457, H0 = 338155.533600823, Tlimit = 1000, alow = { -1671733.521,26400.1025, -152.6867707,0.502712141, -0.000652101403,4.44348811e-07, -1.227815006e-10}, blow = { -127375.4623,878.393332}, ahigh = {5444527.57, -34568.2929,76.3865195, -0.003298972,2.343496957e-07,2.467674021e-11, -3.162012849e-15}, bhigh = {193907.9354, -464.142466}, R = 83.820311712402);
          constant IdealGases.Common.DataRecord C7H16_n_heptane(name = "C7H16_n_heptane", MM = 0.10020194, Hf =  -1874015.61287137, H0 = 331540.487140269, Tlimit = 1000, alow = { -612743.289,11840.85437, -74.871886,0.2918466052, -0.000341679549,2.159285269e-07, -5.65585273e-11}, blow = { -80134.0894,440.721332}, ahigh = {9135632.47, -39233.1969,78.8978085, -0.00465425193,2.071774142e-06, -3.4425393e-10,1.976834775e-14}, bhigh = {205070.8295, -485.110402}, R = 82.9771559313123);
          constant IdealGases.Common.DataRecord C7H16_2_methylh(name = "C7H16_2_methylh", MM = 0.10020194, Hf =  -1942078.16734886, H0 = 308576.85988914, Tlimit = 1000, alow = { -710477.777,11912.5112, -73.4533944,0.2902952369, -0.000346276768,2.260184498e-07, -6.12881392e-11}, blow = { -82021.477,432.004229}, ahigh = {1289912.969, -1784.340963,10.83537673,0.0527060924, -1.886832314e-05,2.432255843e-09, -1.135553789e-13}, bhigh = { -16375.29884, -29.8186241}, R = 82.9771559313123);
          constant IdealGases.Common.DataRecord C8H8_styrene(name = "C8H8_styrene", MM = 0.10414912, Hf = 1423919.85645198, H0 = 201057.867795714, Tlimit = 1000, alow = { -268693.052,6167.99947, -48.3605494,0.2182873229, -0.0002738561832,1.810084981e-07, -4.86775027e-11}, blow = { -11406.39978,281.7679014}, ahigh = { -6629183.62,15145.94166,1.609822364,0.033833186, -1.093737395e-05,1.338825116e-09, -6.03253492e-14}, bhigh = { -89973.2415,43.1128279}, R = 79.8323788045449);
          constant IdealGases.Common.DataRecord C8H10_ethylbenz(name = "C8H10_ethylbenz", MM = 0.106165, Hf = 281825.460368295, H0 = 209862.007252861, Tlimit = 1000, alow = { -469494,9307.16836, -65.2176947,0.2612080237, -0.000318175348,2.051355473e-07, -5.40181735e-11}, blow = { -40738.7021,378.090436}, ahigh = {5551564.1, -28313.80598,60.6124072,0.001042112857, -1.327426719e-06,2.166031743e-10, -1.142545514e-14}, bhigh = {164224.1062, -369.176982}, R = 78.3165073235059);
          constant IdealGases.Common.DataRecord C8H16_1_octene(name = "C8H16_1_octene", MM = 0.11221264, Hf =  -744924.992407273, H0 = 315026.898930459, Tlimit = 1000, alow = { -928190.522,16409.74476, -101.5939534,0.374800141, -0.00045908294,2.96533534e-07, -7.84044521e-11}, blow = { -89524.2608,590.759427}, ahigh = { -4409336.07, -4383.6788,49.3915426,0.00791233963, -7.88866951e-07,9.97021235e-12,1.913144872e-15}, bhigh = { -11226.19342, -257.7650649}, R = 74.0956811995512);
          constant IdealGases.Common.DataRecord C8H17_n_octyl(name = "C8H17_n_octyl", MM = 0.11322058, Hf =  -144143.405730654, H0 = 336537.756651662, Tlimit = 1000, alow = { -1934340.995,30549.7983, -176.7903454,0.580159665, -0.000751741401,5.11246903e-07, -1.410193662e-10}, blow = { -149889.4706,1013.724329}, ahigh = {5632173.39, -38211.4367,86.379275, -0.00360893158,2.544260445e-07,2.908638837e-11, -3.67954974e-15}, bhigh = {210313.547, -526.242283}, R = 73.4360484639807);
          constant IdealGases.Common.DataRecord C8H18_n_octane(name = "C8H18_n_octane", MM = 0.11422852, Hf =  -1827477.06089512, H0 = 330740.51909278, Tlimit = 1000, alow = { -698664.715,13385.01096, -84.1516592,0.327193666, -0.000377720959,2.339836988e-07, -6.01089265e-11}, blow = { -90262.2325,493.922214}, ahigh = {6365406.95, -31053.64657,69.6916234,0.01048059637, -4.12962195e-06,5.54322632e-10, -2.651436499e-14}, bhigh = {150096.8785, -416.989565}, R = 72.7880567830171);
          constant IdealGases.Common.DataRecord C8H18_isooctane(name = "C8H18_isooctane", MM = 0.11422852, Hf =  -1961068.91693948, H0 = 281628.440953275, Tlimit = 1000, alow = { -168875.8565,3126.903227, -21.23502828,0.1489151508, -0.0001151180135,4.47321617e-08, -5.55488207e-12}, blow = { -44680.6062,141.7455793}, ahigh = {13527650.32, -46633.7034,77.9531318,0.01423729984, -5.07359391e-06,7.24823297e-10, -3.81919011e-14}, bhigh = {254117.8017, -493.388719}, R = 72.7880567830171);
          constant IdealGases.Common.DataRecord C9H19_n_nonyl(name = "C9H19_n_nonyl", MM = 0.12724716, Hf =  -291008.459442238, H0 = 335284.496722756, Tlimit = 1000, alow = { -2194880.612,34685.6578, -200.9261419,0.658050311, -0.000852593001,5.79463896e-07, -1.597637099e-10}, blow = { -172319.4608,1149.114017}, ahigh = {5277361.74, -40257.0196,94.5729672, -0.002940301447,6.97810699e-09,6.80525024e-11, -5.90709533e-15}, bhigh = {216532.0614, -575.44495}, R = 65.3411203833547);
          constant IdealGases.Common.DataRecord C10H8_naphthale(name = "C10H8_naphthale", MM = 0.12817052, Hf = 1174841.14131705, H0 = 161605.617266747, Tlimit = 1000, alow = { -260284.5316,6237.40957, -52.2609504,0.2397692776, -0.0002912244803,1.854944401e-07, -4.81661927e-11}, blow = { -11147.0088,297.2139517}, ahigh = {5906172.11, -31632.2924,70.3034203, -0.00601886554,1.142052144e-06, -1.161605689e-10,4.89284402e-15}, bhigh = {196256.7046, -434.784895}, R = 64.8703929733608);
          constant IdealGases.Common.DataRecord C10H21_n_decyl(name = "C10H21_n_decyl", MM = 0.14127374, Hf =  -408710.068835157, H0 = 334273.022006779, Tlimit = 1000, alow = { -2446511.152,38700.813, -224.4388176,0.734362497, -0.00095135256,6.46297033e-07, -1.781502862e-10}, blow = { -194163.3889,1280.987834}, ahigh = {4967237.76, -42424.6844,102.8853417, -0.00232484818, -2.2842339e-07,1.056127364e-10, -8.0680659e-15}, bhigh = {223542.989, -625.519116}, R = 58.853627008105);
          constant IdealGases.Common.DataRecord C12H9_o_bipheny(name = "C12H9_o_bipheny", MM = 0.15319986, Hf = 2791973.83078549, H0 = 173559.864871939, Tlimit = 1000, alow = { -359584.082,7661.37856, -58.7329046,0.2697557882, -0.000322975668,2.0329071e-07, -5.23131672e-11}, blow = {14584.14642,339.268711}, ahigh = {6736469.42, -36321.9435,82.102396, -0.00750234286,1.456251733e-06, -1.507145019e-10,6.43647043e-15}, bhigh = {255552.4521, -504.246297}, R = 54.2720600397415);
          constant IdealGases.Common.DataRecord C12H10_biphenyl(name = "C12H10_biphenyl", MM = 0.1542078, Hf = 1181068.66189648, H0 = 173684.619065962, Tlimit = 1000, alow = { -367103.405,8128.41259, -63.9099457,0.2901422744, -0.000350959074,2.230989996e-07, -5.78847029e-11}, blow = { -16792.63066,363.346419}, ahigh = {7480385.36, -39280.8723,86.6148222, -0.00794639857,1.531868544e-06, -1.576450171e-10,6.70060273e-15}, bhigh = {243805.0641, -538.138149}, R = 53.9173245451916);
          constant IdealGases.Common.DataRecord Ca(name = "Ca", MM = 0.040078, Hf = 4436349.11921753, H0 = 154634.163381406, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {20638.92786,4.38454833}, ahigh = {7547341.24, -21486.42662,25.30849567, -0.01103773705,2.293249636e-06, -1.209075383e-10, -4.015333268e-15}, bhigh = {158586.2323, -160.9512955}, R = 207.457258346225);
          constant IdealGases.Common.DataRecord Caplus(name = "Caplus", MM = 0.0400774514, Hf = 19308306.8151385, H0 = 154636.280090405, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {92324.1779,5.07767498}, ahigh = {3747070.82, -11747.07738,16.72546969, -0.00833479771,2.394593294e-06, -2.988243468e-10,1.356563002e-14}, bhigh = {166432.9088, -95.8282126}, R = 207.460098123904);
          constant IdealGases.Common.DataRecord CaBr(name = "CaBr", MM = 0.119982, Hf =  -207270.098848161, H0 = 82146.6469970496, Tlimit = 1000, alow = {569.383262, -125.7513537,5.00150833, -0.001039868753,1.377594927e-06, -8.94901396e-10,2.395658928e-13}, blow = { -3728.10398,1.784864847}, ahigh = {2783236.402, -8458.20373,14.3175656, -0.00543078168,1.522218659e-06, -1.831824807e-10,7.86428866e-15}, bhigh = {49312.7068, -65.370011}, R = 69.297661315864);
          constant IdealGases.Common.DataRecord CaBr2(name = "CaBr2", MM = 0.199886, Hf =  -1937090.64166575, H0 = 78060.4944818547, Tlimit = 1000, alow = {1572.504318, -223.8551236,8.38845789, -0.001944164774,2.397926088e-06, -1.55657235e-09,4.12430958e-13}, blow = { -47721.0661, -10.74965233}, ahigh = { -22198.46426, -4.46315533,7.50375511, -1.641654626e-06,3.87942806e-10, -4.68342426e-14,2.259325541e-18}, bhigh = { -48854.3682, -5.59032425}, R = 41.5960697597631);
          constant IdealGases.Common.DataRecord CaCL(name = "CaCL", MM = 0.075531, Hf =  -1373911.17554382, H0 = 127035.45564073, Tlimit = 1000, alow = {6395.33526, -224.9042269,5.28327839, -0.001447983237,1.643742771e-06, -9.43010428e-10,2.237516133e-13}, blow = { -12701.69, -1.391964289}, ahigh = {1629182.545, -4766.22302,9.65892977, -0.002523044131,5.83354216e-07, -4.10172699e-11,8.81317764e-17}, bhigh = {16625.99241, -33.98731174}, R = 110.080258436933);
          constant IdealGases.Common.DataRecord CaCLplus(name = "CaCLplus", MM = 0.0755304514, Hf = 6185459.03460601, H0 = 123938.263660371, Tlimit = 1000, alow = {2285.284542, -200.2872721,4.93699019, -0.000386097303,8.68666188e-08,1.47020568e-10, -7.68931587e-14}, blow = {55882.7795, -0.6359815341}, ahigh = {177028.0396, -589.137239,5.1033688, -0.0002356761168,5.48373287e-08,2.137973112e-14, -5.57288731e-16}, bhigh = {58534.1725, -2.191253644}, R = 110.081057982397);
          constant IdealGases.Common.DataRecord CaCL2(name = "CaCL2", MM = 0.110984, Hf =  -4372193.08188568, H0 = 133866.350104519, Tlimit = 1000, alow = {11068.0203, -424.910825,9.13471186, -0.00349718531,4.24218463e-06, -2.719673231e-09,7.13826863e-13}, blow = { -58503.4676, -18.08678294}, ahigh = { -35957.6878, -8.79649563,7.50725691, -3.129465934e-06,7.32288648e-10, -8.77640322e-14,4.21048604e-18}, bhigh = { -60667.7966, -8.553523451}, R = 74.9159518489152);
          constant IdealGases.Common.DataRecord CaF(name = "CaF", MM = 0.0590764032, Hf =  -4678751.15998937, H0 = 154581.347295023, Tlimit = 1000, alow = {31842.354, -491.823509,5.70112089, -0.001465138218,9.09293081e-07, -1.367585222e-10, -5.10632338e-14}, blow = { -31976.9411, -5.99666226}, ahigh = {519588.267, -1512.321567,5.86927625, -0.000388252448, -2.484582298e-08,3.74991596e-11, -3.47097645e-15}, bhigh = { -24888.14584, -8.6146637}, R = 140.740998937457);
          constant IdealGases.Common.DataRecord CaFplus(name = "CaFplus", MM = 0.0590758546, Hf = 4412371.34468809, H0 = 152399.115695569, Tlimit = 1000, alow = {41583.408, -562.766166,5.68752215, -0.001264710572,6.66908648e-07, -6.92675282e-11, -3.97871462e-14}, blow = {33051.195, -6.97448501}, ahigh = { -139867.0845,219.1526375,4.21432434,0.0002142798048, -5.7786726e-08,9.98575631e-12, -6.05962188e-16}, bhigh = {28416.05897,2.740342729}, R = 140.742305909867);
          constant IdealGases.Common.DataRecord CaF2(name = "CaF2", MM = 0.0780748064, Hf =  -10129111.3672233, H0 = 164093.79146408, Tlimit = 1000, alow = {59093.5229, -1019.733042,10.13165309, -0.00550350644,5.62931998e-06, -3.115401465e-09,7.20552594e-13}, blow = { -91926.0819, -26.3950595}, ahigh = { -82804.0944, -31.15950071,7.02357737, -9.52100002e-06,2.118160895e-09, -2.440713888e-13,1.135135468e-17}, bhigh = { -97294.6677, -7.54624334}, R = 106.493661443136);
          constant IdealGases.Common.DataRecord CaH(name = "CaH", MM = 0.04108594, Hf = 5583640.16984886, H0 = 211875.522380649, Tlimit = 1000, alow = { -45137.8223,762.942921, -1.280874223,0.01318774659, -1.481595334e-05,8.53657322e-09, -1.989958945e-12}, blow = {23003.78814,30.53421525}, ahigh = { -2696952.529,8607.05975, -7.02745482,0.00746791631, -2.318610699e-06,3.42307242e-10, -1.892679792e-14}, bhigh = { -27738.19107,78.4582201}, R = 202.367817311713);
          constant IdealGases.Common.DataRecord CaI(name = "CaI", MM = 0.16698247, Hf = 72961.6707669973, H0 = 59995.0401979322, Tlimit = 1000, alow = { -826.179018, -85.089759,4.86417169, -0.000786316965,1.100493506e-06, -7.34803061e-10,2.010892767e-13}, blow = {523.675344,3.58539168}, ahigh = {1771071.309, -5683.64373,11.53857476, -0.0041940202,1.291359961e-06, -1.730043579e-10,8.38076235e-15}, bhigh = {35832.932, -44.0591757}, R = 49.7924842050785);
          constant IdealGases.Common.DataRecord CaI2(name = "CaI2", MM = 0.29388694, Hf =  -882378.45819212, H0 = 54616.2241847154, Tlimit = 1000, alow = { -1097.851401, -145.9821761,8.08934184, -0.001305332923,1.624199331e-06, -1.061194001e-09,2.825551217e-13}, blow = { -32726.9298, -6.98171494}, ahigh = { -16237.27104, -2.859530722,7.50243219, -1.071178707e-06,2.544559095e-10, -3.083631004e-14,1.491836724e-18}, bhigh = { -33463.4117, -3.56730574}, R = 28.2913966847251);
          constant IdealGases.Common.DataRecord CaO(name = "CaO", MM = 0.0560774, Hf = 677729.495304704, H0 = 159656.260810951, Tlimit = 1000, alow = {38897.3307, -483.567735,5.07771325,0.000307623525, -1.159759897e-06,8.49343334e-10, -1.495333366e-13}, blow = {5937.64348, -3.95532073}, ahigh = { -49131061.7,149586.595, -168.1654149,0.0938195026, -2.455529428e-05,3.07498072e-09, -1.485914237e-13}, bhigh = { -946151.172,1235.694769}, R = 148.267787022936);
          constant IdealGases.Common.DataRecord CaOplus(name = "CaOplus", MM = 0.0560768514, Hf = 12665431.8897798, H0 = 163402.719147673, Tlimit = 1000, alow = {109806.0332, -1459.992448,9.88077794, -0.0091575646,8.70758256e-06, -4.39041108e-09,9.26485466e-13}, blow = {91500.5735, -30.09947701}, ahigh = {939784.313, -2993.362243,8.33619182, -0.002303295087,7.37396753e-07, -1.052888279e-10,5.25713841e-15}, bhigh = {102809.8401, -24.61547836}, R = 148.269237527127);
          constant IdealGases.Common.DataRecord CaOH(name = "CaOH", MM = 0.05708534, Hf =  -3035934.44481543, H0 = 193564.529877548, Tlimit = 1000, alow = {46200.289, -928.567282,9.17582877, -0.0039628288,2.505447308e-06,3.85206821e-11, -3.35277847e-13}, blow = { -17980.09717, -25.3370485}, ahigh = {1979972.994, -5598.88099,11.51348706, -0.001668264707,3.31257391e-07, -1.789056647e-11, -3.58071641e-16}, bhigh = {13401.96822, -46.4608426}, R = 145.649863870479);
          constant IdealGases.Common.DataRecord CaOHplus(name = "CaOHplus", MM = 0.0570847914, Hf = 6533055.31742733, H0 = 194407.629910337, Tlimit = 1000, alow = {48843.4266, -983.24151,9.61018295, -0.00520436066,4.26304982e-06, -1.197590878e-09,9.02841189e-15}, blow = {47950.5634, -28.27077334}, ahigh = {863761.537, -2347.302046,7.9819112,0.0001003160947, -6.24007417e-08,1.019540904e-11, -5.69892528e-16}, bhigh = {58305.8012, -21.52181937}, R = 145.651263604337);
          constant IdealGases.Common.DataRecord Ca_OH_2(name = "Ca_OH_2", MM = 0.07409268, Hf =  -8075546.3157764, H0 = 223939.544365246, Tlimit = 1000, alow = {83892.5754, -1791.902135,16.21891031, -0.00784085717,5.09511161e-06, -6.95548755e-11, -6.13215264e-13}, blow = { -66004.0563, -60.7091392}, ahigh = {1721854.884, -4702.21767,13.96947691,0.0001983791405, -1.243050592e-07,2.033402404e-11, -1.137157819e-15}, bhigh = { -44437.6135, -52.8744486}, R = 112.2171852874);
          constant IdealGases.Common.DataRecord CaS(name = "CaS", MM = 0.072143, Hf = 1683812.88551904, H0 = 129743.883675478, Tlimit = 1000, alow = {23209.90615, -451.844608,6.17233375, -0.00359885202,4.82373171e-06, -3.63173354e-09,1.212849867e-12}, blow = {15545.97691, -7.686936462}, ahigh = { -15683532.14,52938.5581, -63.2246724,0.0402772638, -1.114855081e-05,1.464389344e-09, -7.37607897e-14}, bhigh = { -317202.267,479.9606012}, R = 115.249878713112);
          constant IdealGases.Common.DataRecord Ca2(name = "Ca2", MM = 0.080156, Hf = 4263752.38285344, H0 = 140639.952093418, Tlimit = 1000, alow = { -85822.2862,158.818896,11.03952055, -0.0333319676,5.34593881e-05, -4.01157324e-08,1.160486682e-11}, blow = {37703.508, -23.97744561}, ahigh = {240596.6267,57.7580382,2.347436675,0.0001199275034, -4.32915031e-08,7.01530269e-12, -3.70566032e-16}, bhigh = {40818.754,18.95399607}, R = 103.728629173112);
          constant IdealGases.Common.DataRecord Cd(name = "Cd", MM = 0.112411, Hf = 994564.588874754, H0 = 55131.8643193282, Tlimit = 1000, alow = { -0.0001081751543,1.816433041e-06,2.499999989,3.129989231e-11, -4.60071016e-14,3.40704874e-17, -9.989497436e-21}, blow = {12700.99766,5.93154976}, ahigh = { -269975.7467,786.600114,1.628169079,0.000459412329, -1.150420443e-07,1.074836707e-11,8.790199555e-17}, bhigh = {7675.14826,12.20006052}, R = 73.9649322575193);
          constant IdealGases.Common.DataRecord Cdplus(name = "Cdplus", MM = 0.1124104514, Hf = 8769240.89996137, H0 = 55132.1333809714, Tlimit = 1000, alow = {0.000409834494, -4.34358162e-06,2.500000019, -4.38903681e-11,5.5639692e-14, -3.63822826e-17,9.607881995e-21}, blow = {117812.9439,6.62468945}, ahigh = {14848.80812, -46.6915368,2.557883006, -3.6247017e-05,1.21374757e-08, -2.072802814e-12,1.420665367e-16}, bhigh = {118107.0109,6.21641448}, R = 73.9652932307325);
          constant IdealGases.Common.DataRecord CL(name = "CL", MM = 0.035453, Hf = 3421459.39694807, H0 = 176898.654556737, Tlimit = 1000, alow = {22762.15854, -216.8413293,2.745185115,0.002451101694, -5.45801199e-06,4.41798688e-09, -1.288134004e-12}, blow = {15013.57068,3.102963457}, ahigh = { -169793.293,608.172646,2.12866409,0.0001307367034, -2.644883596e-08,2.842504775e-12, -1.252911731e-16}, bhigh = {9934.3874,8.844772103}, R = 234.52097142696);
          constant IdealGases.Common.DataRecord CLplus(name = "CLplus", MM = 0.0354524514, Hf = 38891517.5270503, H0 = 180138.03694263, Tlimit = 1000, alow = {103469.7859, -1293.758873,8.18670269, -0.0099160146,9.20847237e-06, -4.50742624e-09,9.18212788e-13}, blow = {171475.878, -27.66417931}, ahigh = {40564.0948, -49.6016572,3.10165363, -0.000586873829,2.252039316e-07, -3.29970302e-11,1.708780842e-15}, bhigh = {165298.2337,2.574942598}, R = 234.524600462466);
          constant IdealGases.Common.DataRecord CLminus(name = "CLminus", MM = 0.0354535486, Hf =  -6599000.13506688, H0 = 174804.166147701, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = { -28883.89093,4.200642023}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = { -28883.89093,4.200642023}, R = 234.517342503763);
          constant IdealGases.Common.DataRecord CLCN(name = "CLCN", MM = 0.0614704, Hf = 2183164.58002551, H0 = 173563.894166949, Tlimit = 1000, alow = {72740.4343, -1297.344947,11.07676527, -0.01108430216,1.614009477e-05, -1.088916772e-08,2.830952246e-12}, blow = {20843.88986, -35.97370046}, ahigh = {346757.312, -1957.85737,8.80780615, -0.000438836278,1.024761895e-07, -1.110675026e-11,4.98617942e-16}, bhigh = {25819.23944, -26.36885363}, R = 135.259767302637);
          constant IdealGases.Common.DataRecord CLF(name = "CLF", MM = 0.0544514032, Hf =  -1022948.71622335, H0 = 163597.381821007, Tlimit = 1000, alow = {33522.1081, -368.83119,4.27228862,0.002549434508, -4.45683089e-06,3.41385412e-09, -9.8386852e-13}, blow = { -5839.36969,0.2318014199}, ahigh = {3045867.173, -9979.32826,16.84162254, -0.00746585062,2.336213612e-06, -3.36586546e-10,1.763082415e-14}, bhigh = {54471.208, -87.07987094}, R = 152.69527526152);
          constant IdealGases.Common.DataRecord CLF3(name = "CLF3", MM = 0.0924482096, Hf =  -1780456.33022189, H0 = 148498.808786017, Tlimit = 1000, alow = {128517.5171, -2140.445721,14.93096474, -0.00604247104,3.71896861e-06, -8.0532011e-10, -7.98560053e-14}, blow = { -11384.5964, -55.94838353}, ahigh = { -229495.6235, -122.0741,10.09124644, -3.64972475e-05,8.05891075e-09, -9.23084995e-13,4.27256949e-17}, bhigh = { -22828.40447, -25.11605479}, R = 89.93653891162);
          constant IdealGases.Common.DataRecord CLF5(name = "CLF5", MM = 0.130445016, Hf =  -1824523.52184924, H0 = 137455.585117947, Tlimit = 1000, alow = {245970.3939, -4315.34807,27.27005972, -0.01670193839,1.421668208e-05, -6.42647974e-09,1.183099799e-12}, blow = { -10714.23726, -126.7466837}, ahigh = { -426932.745, -205.805739,16.15465278, -6.21268818e-05,1.376637959e-08, -1.581335709e-12,7.33642249e-17}, bhigh = { -33614.666, -57.57531175}, R = 63.7392846040204);
          constant IdealGases.Common.DataRecord CLO(name = "CLO", MM = 0.0514524, Hf = 1975051.01802831, H0 = 185066.488638042, Tlimit = 1000, alow = { -16872.68145,257.3812247,2.17584612,0.00643206113, -8.5682495e-06,5.76497125e-09, -1.545771209e-12}, blow = {9829.51898,13.86010503}, ahigh = {409376.052, -1765.985112,7.08790063, -0.001828450169,7.1038181e-07, -1.209332942e-10,7.07644104e-15}, bhigh = {21518.91784, -16.68645548}, R = 161.595416345982);
          constant IdealGases.Common.DataRecord CLO2(name = "CLO2", MM = 0.0674518, Hf = 1556667.13119591, H0 = 160123.910703643, Tlimit = 1000, alow = { -11272.77696,390.303203, -0.384301853,0.02108677947, -2.793137755e-05,1.841289286e-08, -4.83977915e-12}, blow = {9756.93367,29.132627}, ahigh = { -163379.3802, -316.148067,7.00978726,0.0002837971144, -1.179925338e-07,2.920383252e-11, -2.024030202e-15}, bhigh = {11849.46452, -10.91168827}, R = 123.265383577607);
          constant IdealGases.Common.DataRecord CL2(name = "CL2", MM = 0.070906, Hf = 0, H0 = 129482.836431332, Tlimit = 1000, alow = {34628.1517, -554.712652,6.20758937, -0.002989632078,3.17302729e-06, -1.793629562e-09,4.26004359e-13}, blow = {1534.069331, -9.438331107}, ahigh = {6092569.42, -19496.27662,28.54535795, -0.01449968764,4.46389077e-06, -6.35852586e-10,3.32736029e-14}, bhigh = {121211.7724, -169.0778824}, R = 117.26048571348);
          constant IdealGases.Common.DataRecord CL2O(name = "CL2O", MM = 0.0869054, Hf = 909034.421336304, H0 = 134577.160912901, Tlimit = 1000, alow = {77988.554, -1182.537879,9.52651466, -0.002596163176,8.69678131e-07,4.48409827e-10, -3.044511967e-13}, blow = {13767.29572, -24.84690718}, ahigh = { -127174.3461, -67.4196322,7.05002245, -1.988084386e-05,4.36620905e-09, -4.97858068e-13,2.295679186e-17}, bhigh = {7387.2893, -8.797477254}, R = 95.672673965024);
          constant IdealGases.Common.DataRecord Co(name = "Co", MM = 0.0589332, Hf = 7269953.0994414, H0 = 107914.859535881, Tlimit = 1000, alow = { -2598.939184,246.1989844, -0.610605837,0.01393005772, -2.210012979e-05,1.623755261e-08, -4.534904351e-12}, blow = {49846.1376,22.57584199}, ahigh = {1381841.305, -3756.03668,6.65713065, -0.001269246675,1.464092329e-07,6.57494657e-12, -1.102384178e-15}, bhigh = {74944.4291, -22.58500836}, R = 141.082988875541);
          constant IdealGases.Common.DataRecord Coplus(name = "Coplus", MM = 0.0589326514, Hf = 20243503.0269146, H0 = 106757.575818148, Tlimit = 1000, alow = {102849.416, -874.473126,4.27950028,0.002225857835, -7.45727457e-06,7.27922195e-09, -2.347541963e-12}, blow = {147489.5959, -5.67901922}, ahigh = {2907386.174, -8619.70575,11.88134934, -0.00351064742,5.74800468e-07, -2.534065135e-11,2.976607469e-16}, bhigh = {197741.9221, -60.9653344}, R = 141.084302207384);
          constant IdealGases.Common.DataRecord Cominus(name = "Cominus", MM = 0.0589337486, Hf = 6081648.9789689, H0 = 107014.607925348, Tlimit = 1000, alow = {34594.9376, -135.5041712,1.116330898,0.00904276183, -1.454296726e-05,9.99494063e-09, -2.595633259e-12}, blow = {43370.3782,12.70494975}, ahigh = { -574139.417,1763.109207,1.415561988,0.000377686969, -7.53417998e-08,7.99570154e-12, -3.49044e-16}, bhigh = {30834.93114,16.34360353}, R = 141.081675568148);
          constant IdealGases.Common.DataRecord Cr(name = "Cr", MM = 0.0519961, Hf = 7644419.48530755, H0 = 119190.24696083, Tlimit = 1000, alow = {1335.658217, -21.02424026,2.631908173, -0.000424626325,7.43919416e-07, -6.76393163e-10,2.507855625e-13}, blow = {47158.6664,6.00542545}, ahigh = { -11202207.89,34011.6369, -36.5706217,0.02110296902, -5.51818014e-06,7.17360171e-10, -3.505127367e-14}, bhigh = { -168899.344,286.4481267}, R = 159.905685234085);
          constant IdealGases.Common.DataRecord Crplus(name = "Crplus", MM = 0.0519955514, Hf = 20319944.6789596, H0 = 119191.50452552, Tlimit = 1000, alow = {181.9187467, -2.188843517,2.510676511, -2.706791825e-05,3.76849263e-08, -2.736784742e-11,8.115389932e-15}, blow = {126338.0825,6.50627617}, ahigh = {3342330.79, -10642.61051,15.57884307, -0.00770897148,2.158300274e-06, -2.36810811e-10,8.952805604e-15}, bhigh = {193299.767, -86.0435667}, R = 159.9073723834);
          constant IdealGases.Common.DataRecord Crminus(name = "Crminus", MM = 0.0519966486, Hf = 6289317.42343102, H0 = 119188.989422676, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {38586.2787,6.56683537}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = {38586.2787,6.56683537}, R = 159.903998120371);
          constant IdealGases.Common.DataRecord CrN(name = "CrN", MM = 0.0660028, Hf = 7651323.8832292, H0 = 132991.160981049, Tlimit = 1000, alow = { -8239.12922,300.8144202,0.514787492,0.01129636791, -1.515183504e-05,1.010997229e-08, -2.68777575e-12}, blow = {58456.2812,22.98028696}, ahigh = {1110672.49, -3690.47854,8.59905668, -0.002125587223,5.28235848e-07, -4.92113915e-11,1.404331106e-15}, bhigh = {82548.0458, -27.99968411}, R = 125.971504239214);
          constant IdealGases.Common.DataRecord CrO(name = "CrO", MM = 0.0679955, Hf = 2744024.50162143, H0 = 144849.556220632, Tlimit = 1000, alow = {9373.33411,136.9743818,1.621443428,0.00881409596, -1.23284536e-05,8.49796094e-09, -2.315804197e-12}, blow = {20909.48871,17.81935787}, ahigh = {1092367.332, -3749.75865,9.00787021, -0.002545445236,6.92805168e-07, -6.39083195e-11,1.659741645e-15}, bhigh = {44470.9821, -29.42600453}, R = 122.279739100382);
          constant IdealGases.Common.DataRecord CrO2(name = "CrO2", MM = 0.0839949, Hf =  -1286307.08531113, H0 = 127315.694167146, Tlimit = 1000, alow = {35486.299, -229.8628537,2.286289393,0.01616929338, -2.34519891e-05,1.631365714e-08, -4.44426962e-12}, blow = { -12789.1284,14.42954956}, ahigh = { -432710.914,191.5584657,7.18824737, -0.000569484619,3.54636613e-07, -5.65512306e-11,2.908946349e-15}, bhigh = { -17433.39545, -10.0642472}, R = 98.9878194985648);
          constant IdealGases.Common.DataRecord CrO3(name = "CrO3", MM = 0.0999943, Hf =  -3220554.41160146, H0 = 130408.153264736, Tlimit = 1000, alow = {41830.2006, -505.934885,4.43271567,0.01995079387, -2.920597649e-05,2.03933028e-08, -5.58008663e-12}, blow = { -37697.024,0.865382728}, ahigh = { -628331.401,692.815818,8.9712746,0.000682336564, -2.235048825e-07,3.36678579e-11, -1.614026492e-15}, bhigh = { -47238.802, -19.45305345}, R = 83.1494595191926);
          constant IdealGases.Common.DataRecord CrO3minus(name = "CrO3minus", MM = 0.0999948486, Hf =  -6328834.47357907, H0 = 134244.305461152, Tlimit = 1000, alow = {187345.5703, -2300.722991,13.43953022, -0.001663085846, -1.443973096e-06,2.045898933e-09, -6.8375553e-13}, blow = { -66301.1158, -49.306442}, ahigh = { -649968.203,452.201762,9.9805759, -0.000421413673,2.631277945e-07, -4.59454365e-11,2.659097549e-15}, bhigh = { -83500.1973, -24.648109}, R = 83.1490033377579);
          constant IdealGases.Common.DataRecord Cs(name = "Cs", MM = 0.13290545, Hf = 575597.163246503, H0 = 46630.352630385, Tlimit = 1000, alow = {54.6658407, -0.827934604,2.50494221, -1.49462069e-05,2.425976774e-08, -2.013172322e-11,6.704271991e-15}, blow = {8459.32139,6.848825772}, ahigh = {6166040.9, -18961.75522,24.83229903, -0.01251977234,3.30901739e-06, -3.35401202e-10,9.626500908e-15}, bhigh = {128511.1231, -152.2942188}, R = 62.559300615588);
          constant IdealGases.Common.DataRecord Csplus(name = "Csplus", MM = 0.1329049014, Hf = 3449096.48305868, H0 = 46630.5451094522, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {54387.3782,6.182757992}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = {54387.3782,6.182757992}, R = 62.5595588455852);
          constant IdealGases.Common.DataRecord Csminus(name = "Csminus", MM = 0.1329059986, Hf = 186577.191858968, H0 = 46630.1601529067, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {2237.029001,6.182770382}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = {2237.029001,6.182770382}, R = 62.5590423877226);
          constant IdealGases.Common.DataRecord CsBO2(name = "CsBO2", MM = 0.17571525, Hf =  -3909176.38622715, H0 = 82326.9124336106, Tlimit = 1000, alow = {41936.0958, -666.42374,8.13483588,0.002902960302, -7.88146792e-07, -8.32769454e-10,4.44171911e-13}, blow = { -81223.2007, -11.21622595}, ahigh = {89755.7989, -1656.134283,11.16468934, -0.000447636699,9.62109181e-08, -1.081470115e-11,4.93844674e-16}, bhigh = { -76104.7967, -30.36616012}, R = 47.3178736620754);
          constant IdealGases.Common.DataRecord CsBr(name = "CsBr", MM = 0.21280945, Hf =  -971897.042166125, H0 = 48898.7918534633, Tlimit = 1000, alow = {1639.263647, -69.4747797,4.86005872, -0.000858778015,1.37517755e-06, -9.89176698e-10,2.895433432e-13}, blow = { -25895.50471,4.46535569}, ahigh = { -152832.9963,1259.712657,1.704268408,0.002776255318, -1.228389838e-06,2.698111786e-10, -1.98353448e-14}, bhigh = { -33278.4203,24.84116659}, R = 39.0700318994293);
          constant IdealGases.Common.DataRecord CsCL(name = "CsCL", MM = 0.16835845, Hf =  -1438768.75796849, H0 = 60175.9044467326, Tlimit = 1000, alow = { -25381.65292,297.2839326,2.667813848,0.00558327241, -8.52900509e-06,6.59519139e-09, -1.989715823e-12}, blow = { -31892.5145,15.11365709}, ahigh = { -3674923.48,11861.52941, -10.63287842,0.00980411792, -3.27779746e-06,5.53423339e-10, -3.4012315e-14}, bhigh = { -104865.8658,111.4096064}, R = 49.3855342574133);
          constant IdealGases.Common.DataRecord CsF(name = "CsF", MM = 0.1519038532, Hf =  -2397667.18438976, H0 = 63494.8738746016, Tlimit = 1000, alow = {18436.85799, -404.240939,6.38317886, -0.00467432337,6.63013401e-06, -4.76090553e-09,1.374569797e-12}, blow = { -43184.8959, -7.22659323}, ahigh = { -1850863.231,5625.2988, -2.250022212,0.00410924354, -1.250243837e-06,1.941483152e-10, -1.071166179e-14}, bhigh = { -80798.2779,51.3555945}, R = 54.7350960811572);
          constant IdealGases.Common.DataRecord CsH(name = "CsH", MM = 0.13391339, Hf = 865858.940618261, H0 = 66058.4128293668, Tlimit = 1000, alow = {16205.44411, -70.8701628,2.480847135,0.00702198599, -1.007126309e-05,7.09063903e-09, -1.950395735e-12}, blow = {13427.77328,9.89424717}, ahigh = { -911214.665,3576.47275, -1.258058765,0.00431485515, -1.407820502e-06,2.154598897e-10, -1.254525606e-14}, bhigh = { -9158.71727,39.0880003}, R = 62.0884289465004);
          constant IdealGases.Common.DataRecord CsI(name = "CsI", MM = 0.25980992, Hf =  -586274.276979108, H0 = 40607.0984510522, Tlimit = 1000, alow = { -4072.68565,23.30073667,4.38727769,0.000341208429, -2.175588153e-07,8.906787e-11, -2.182639837e-15}, blow = { -19787.66885,8.07465782}, ahigh = {4511259.05, -13417.06362,19.80984712, -0.00835985555,2.348201245e-06, -2.874285248e-10,1.20808775e-14}, bhigh = {65734.3913, -102.1032592}, R = 32.0021344835486);
          constant IdealGases.Common.DataRecord CsLi(name = "CsLi", MM = 0.13984645, Hf = 1159459.70741481, H0 = 73946.274646228, Tlimit = 1000, alow = {1368.709568, -74.5129706,4.84562093, -0.000557366721,4.95332676e-07,4.36906891e-10, -4.60814577e-13}, blow = {18505.77392,2.112012017}, ahigh = {7481630.23, -28852.97839,46.3110499, -0.02784288108,8.78741349e-06, -1.264901197e-09,6.72018628e-14}, bhigh = {194513.7819, -285.7846194}, R = 59.4542943349652);
          constant IdealGases.Common.DataRecord CsNO2(name = "CsNO2", MM = 0.17891095, Hf =  -1175667.71625772, H0 = 89319.1389347606, Tlimit = 1000, alow = { -71060.1044,1272.448257, -2.447362349,0.03064441948, -3.69922594e-05,2.259121812e-08, -5.58063591e-12}, blow = { -33133.7092,49.54693336}, ahigh = { -163350.3154, -846.887958,10.62853461, -0.0002508799215,5.54125577e-08, -6.35559195e-12,2.946880031e-16}, bhigh = { -24030.40739, -24.43488888}, R = 46.4726837569193);
          constant IdealGases.Common.DataRecord CsNO3(name = "CsNO3", MM = 0.19491035, Hf =  -1634014.76627588, H0 = 84980.6590568433, Tlimit = 1000, alow = { -26772.19779,901.169269, -3.26534082,0.042983777, -5.38675138e-05,3.38452752e-08, -8.56522366e-12}, blow = { -44053.0414,51.18862689}, ahigh = { -314751.7676, -1367.011655,14.01012848, -0.000401864713,8.8536147e-08, -1.013466457e-11,4.69175873e-16}, bhigh = { -35490.2667, -45.00692791}, R = 42.6579296584301);
          constant IdealGases.Common.DataRecord CsNa(name = "CsNa", MM = 0.15589522, Hf = 807640.728176271, H0 = 68649.500606882, Tlimit = 1000, alow = {25217.38609, -429.536249,7.22309474, -0.008460463,1.443905157e-05, -1.143624855e-08,3.18372193e-12}, blow = {15790.99265, -8.8387903}, ahigh = {3879556.79, -17801.67288,34.913558, -0.02320080793,7.86197584e-06, -1.171011257e-09,6.32392737e-14}, bhigh = {119577.6556, -200.1754433}, R = 53.3337199177755);
          constant IdealGases.Common.DataRecord CsO(name = "CsO", MM = 0.14890485, Hf = 252425.075476051, H0 = 66049.6820620685, Tlimit = 1000, alow = {8683.95881,425.027817, -3.4840553,0.0380196983, -6.65192756e-05,5.17347219e-08, -1.512567066e-11}, blow = {1969.690015,42.4079614}, ahigh = {837554.435, -2418.205772,8.90806535, -0.00337818004,1.312755917e-06, -2.150508925e-10,1.21913898e-14}, bhigh = {18094.93447, -24.6085858}, R = 55.8374827952212);
          constant IdealGases.Common.DataRecord CsOH(name = "CsOH", MM = 0.14991279, Hf =  -1707659.49989991, H0 = 78943.8246062928, Tlimit = 1000, alow = {9386.96079, -500.935426,8.30135198, -0.00323559684,2.612406777e-06, -4.95061341e-10, -1.038364927e-13}, blow = { -30257.22427, -17.42194837}, ahigh = {896717.113, -2323.978587,7.95964487,0.0001101149275, -6.46601397e-08,1.045968201e-11, -5.82251417e-16}, bhigh = { -17362.58234, -18.64239624}, R = 55.4620589744211);
          constant IdealGases.Common.DataRecord CsRb(name = "CsRb", MM = 0.21837325, Hf = 510489.869981786, H0 = 50244.8537080434, Tlimit = 1000, alow = { -4910.92268, -9.20901555,5.19337789, -0.00439024699,1.254790436e-05, -1.414878514e-08,5.10144131e-12}, blow = {12004.98171,5.36154293}, ahigh = { -13286933.78,34145.7947, -23.73696406,0.0070704654,3.86345097e-07, -3.158132986e-10,2.667747207e-14}, bhigh = { -212400.8997,223.9970251}, R = 38.074590179887);
          constant IdealGases.Common.DataRecord Cs2(name = "Cs2", MM = 0.2658109, Hf = 411587.083148208, H0 = 41492.4030579634, Tlimit = 1000, alow = { -46741.5873,595.201951,1.895333975,0.00408206581,2.531487119e-06, -9.08513903e-09,4.29017993e-12}, blow = {8857.28257,23.91592727}, ahigh = { -25927395.9,75398.1891, -74.840868,0.0369286679, -8.08249724e-06,8.17185073e-10, -3.0892859e-14}, bhigh = { -471504.572,586.093375}, R = 31.279650307794);
          constant IdealGases.Common.DataRecord Cs2Br2(name = "Cs2Br2", MM = 0.4256189, Hf =  -1329426.08751632, H0 = 51955.235070623, Tlimit = 1000, alow = { -6321.28518, -12.13153529,10.04967112, -0.0001102100044,1.365293456e-07, -8.85695707e-11,2.339447704e-14}, blow = { -70997.7171, -7.53942479}, ahigh = { -7545.84665, -0.2087093009,10.00018042, -8.03426073e-08,1.923604258e-11, -2.34469814e-15,1.139365834e-19}, bhigh = { -71058.8354, -7.25179304}, R = 19.5350159497146);
          constant IdealGases.Common.DataRecord Cs2CO3(name = "Cs2CO3", MM = 0.3258198, Hf =  -2475134.8414062, H0 = 65002.3755462375, Tlimit = 1000, alow = { -46731.3,943.654906, -0.2374670436,0.0416848461, -5.09437221e-05,3.132078394e-08, -7.78042167e-12}, blow = { -103916.2411,40.5585398}, ahigh = { -303714.9985, -1517.0784,17.11941656, -0.000444970012,9.79832524e-08, -1.121256063e-11,5.18973478e-16}, bhigh = { -94204.1477, -56.8200626}, R = 25.5186210291701);
          constant IdealGases.Common.DataRecord Cs2CL2(name = "Cs2CL2", MM = 0.3367169, Hf =  -1914541.2600318, H0 = 62202.7347008719, Tlimit = 1000, alow = { -10779.56357, -53.4610695,10.21710683, -0.000478968703,5.90894653e-07, -3.82141652e-10,1.006995985e-13}, blow = { -80295.1517, -12.07856486}, ahigh = { -16251.43966, -0.900613735,10.00077173, -3.41619408e-07,8.14496808e-11, -9.89747528e-15,4.79833081e-19}, bhigh = { -80565.1322, -10.81977337}, R = 24.6927671287066);
          constant IdealGases.Common.DataRecord Cs2F2(name = "Cs2F2", MM = 0.3038077064, Hf =  -2935601.98511146, H0 = 63818.6576296802, Tlimit = 1000, alow = { -9942.67748, -253.6723355,11.00684073, -0.002185640152,2.664791874e-06, -1.708229209e-09,4.47115198e-13}, blow = { -109058.2601, -20.00699602}, ahigh = { -36779.4642, -4.35575348,10.00366687, -1.603814186e-06,3.79146974e-10, -4.57868743e-14,2.209385787e-18}, bhigh = { -110345.6448, -14.15086452}, R = 27.3675480405786);
          constant IdealGases.Common.DataRecord Cs2I2(name = "Cs2I2", MM = 0.51961984, Hf =  -873779.371087909, H0 = 43464.2815024153, Tlimit = 1000, alow = { -4449.28697, -5.32179366,10.02183919, -4.85337485e-05,6.0193384e-08, -3.90822639e-11,1.032981014e-14}, blow = { -57578.2338, -5.27126709}, ahigh = { -4983.37601, -0.0946117988,10.00008229, -3.68096204e-08,8.84466091e-12, -1.081178562e-15,5.26614198e-20}, bhigh = { -57605.0104, -5.14486489}, R = 16.0010672417743);
          constant IdealGases.Common.DataRecord Cs2O(name = "Cs2O", MM = 0.2818103, Hf =  -506920.240317689, H0 = 49997.6579990157, Tlimit = 1000, alow = {19105.33804, -496.218153,8.7596063, -0.00351162279,4.01647983e-06, -2.450899853e-09,6.17240489e-13}, blow = { -16776.64956, -11.59258822}, ahigh = { -41191.4741, -10.79255391,7.00850047, -3.54295578e-06,8.08169912e-10, -9.49941993e-14,4.48912254e-18}, bhigh = { -19343.7785, -1.20397045}, R = 29.503790315684);
          constant IdealGases.Common.DataRecord Cs2Oplus(name = "Cs2Oplus", MM = 0.2818097514, Hf = 1006707.27180522, H0 = 51267.6581567007, Tlimit = 1000, alow = {683.705609, -243.5575007,7.78303747, -0.001414552725,1.470799739e-06, -8.21882133e-10,1.911749489e-13}, blow = {33241.9487, -4.75042356}, ahigh = { -31530.85844, -5.77155903,7.00448263, -1.848241331e-06,4.1808823e-10, -4.88237627e-14,2.295419936e-18}, bhigh = {31962.2731, -0.0543435069}, R = 29.5038477508128);
          constant IdealGases.Common.DataRecord Cs2O2(name = "Cs2O2", MM = 0.2978097, Hf =  -829620.741030262, H0 = 58515.5419719371, Tlimit = 1000, alow = {22745.0518, -566.791257,10.01728332,0.002547081016, -4.90921669e-06,3.84878891e-09, -1.124566008e-12}, blow = { -29473.3436, -18.41275291}, ahigh = { -119003.2094, -92.6107526,10.06915777, -2.766227147e-05,6.11075296e-09, -7.00359548e-13,3.24374252e-17}, bhigh = { -32556.1074, -17.24868948}, R = 27.9187413976106);
          constant IdealGases.Common.DataRecord Cs2O2H2(name = "Cs2O2H2", MM = 0.29982558, Hf =  -2177932.91686453, H0 = 79837.5942439601, Tlimit = 1000, alow = { -10859.81041, -688.306842,16.43543354, -0.00403922058,2.331117156e-06,8.43965163e-10, -6.85930973e-13}, blow = { -79394.7623, -49.0619681}, ahigh = {1802857.941, -4656.35475,16.93566597,0.0002117402535, -1.272249096e-07,2.066555839e-11, -1.152390667e-15}, bhigh = { -51908.1369, -58.4971916}, R = 27.7310294872105);
          constant IdealGases.Common.DataRecord Cs2SO4(name = "Cs2SO4", MM = 0.3618735, Hf =  -3088515.27951066, H0 = 66288.8163957847, Tlimit = 1000, alow = {61536.3787, -638.137433,6.89477209,0.0396018291, -5.52986201e-05,3.73633252e-08, -9.97324183e-12}, blow = { -133976.031, -1.284432588}, ahigh = { -522339.752, -944.415525,19.70476453, -0.000282122599,6.24079228e-08, -7.16346407e-12,3.32266719e-16}, bhigh = { -136417.9983, -68.20406795}, R = 22.9761836663917);
          constant IdealGases.Common.DataRecord Cu(name = "Cu", MM = 0.063546, Hf = 5309539.54615554, H0 = 97526.6421175212, Tlimit = 1000, alow = {77.1313315, -1.169236206,2.506987803, -2.116434879e-05,3.44171471e-08, -2.862608999e-11,9.559250991e-15}, blow = {39839.8121,5.73081322}, ahigh = {2308090.411, -8503.261,14.67859102, -0.00846713652,2.887821016e-06, -4.27065918e-10,2.304265084e-14}, bhigh = {92075.3562, -78.5470156}, R = 130.841783904573);
          constant IdealGases.Common.DataRecord Cuplus(name = "Cuplus", MM = 0.0635454514, Hf = 17138594.5650864, H0 = 97527.4840836208, Tlimit = 1000, alow = { -0.002452340093,2.606893531e-05,2.49999989,2.351922485e-10, -2.669362382e-13,1.510315123e-16, -3.278224814e-20}, blow = {130240.0621,5.07594077}, ahigh = { -2181443.016,7217.85819, -6.94115475,0.00620824892, -2.139340497e-06,3.56643144e-10, -2.081198501e-14}, bhigh = {85164.5666,71.1680067}, R = 130.842913486645);
          constant IdealGases.Common.DataRecord Cuminus(name = "Cuminus", MM = 0.0635465486, Hf = 3347445.81234424, H0 = 97525.800165959, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {24838.64954,5.07596603}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = {24838.64954,5.07596603}, R = 130.840654342005);
          constant IdealGases.Common.DataRecord CuCL(name = "CuCL", MM = 0.098999, Hf = 920110.304144486, H0 = 95669.4007010172, Tlimit = 1000, alow = {14698.47736, -339.316725,5.72345959, -0.002417296132,2.858019864e-06, -1.757900548e-09,4.45504729e-13}, blow = {11317.14015, -4.527352594}, ahigh = { -25771.2241, -7.10634606,4.50562318,5.39454827e-05,5.38641796e-10, -6.34848277e-14,3.006622062e-18}, bhigh = {9566.19287,2.680067564}, R = 83.9854139940808);
          constant IdealGases.Common.DataRecord CuF(name = "CuF", MM = 0.0825444032, Hf =  -152039.381393214, H0 = 110038.847552053, Tlimit = 1000, alow = {37613.8541, -546.81049,5.83235172, -0.001695312068,1.21815088e-06, -3.71328528e-10,2.002276589e-14}, blow = {58.6514072, -7.15693009}, ahigh = {509415.483, -1415.00987,5.63234938, -0.000162912841, -1.156611499e-07,5.06603408e-11, -4.15320511e-15}, bhigh = {6305.5066, -7.40777361}, R = 100.727265298103);
          constant IdealGases.Common.DataRecord CuF2(name = "CuF2", MM = 0.1015428064, Hf =  -2628842.05650633, H0 = 118727.366589703, Tlimit = 1000, alow = {65733.1128, -896.243084,7.91756094,0.001345667904, -4.16240001e-06,3.71618109e-09, -1.155537597e-12}, blow = { -29168.87325, -15.87076153}, ahigh = { -1650355.082,3775.36212,3.88911655,0.000682191264,1.875979026e-07, -5.67736811e-11,3.82324873e-15}, bhigh = { -59533.717,15.22659664}, R = 81.8814477831883);
          constant IdealGases.Common.DataRecord CuO(name = "CuO", MM = 0.0795454, Hf = 3850254.06874565, H0 = 122583.317199989, Tlimit = 1000, alow = {4689.76224, -118.4808464,4.56615524,0.000430905802, -8.3098864e-07,6.94663802e-10, -2.108693366e-13}, blow = {36151.9068,1.733847344}, ahigh = {358228.017, -913.936641,5.13689867,6.24057724e-05, -1.558613495e-07,5.23680312e-11, -4.02674562e-15}, bhigh = {41507.9286, -2.63407096}, R = 104.524862531334);
          constant IdealGases.Common.DataRecord Cu2(name = "Cu2", MM = 0.127092, Hf = 3818808.4222453, H0 = 78133.5882667674, Tlimit = 1000, alow = { -852.918348, -97.2004493,4.8822337, -0.00073713694,1.000575401e-06, -6.39198944e-10,1.668655797e-13}, blow = {57493.0702,1.105391325}, ahigh = { -86993.995,320.910387,3.97380288,0.000508080967, -1.707470385e-07,3.21910819e-11, -1.958830868e-15}, bhigh = {55060.9019,6.91450217}, R = 65.4208919522865);
          constant IdealGases.Common.DataRecord Cu3CL3(name = "Cu3CL3", MM = 0.296997, Hf =  -870614.854695502, H0 = 96713.7378492039, Tlimit = 1000, alow = {4487.88532, -873.664444,19.36189091, -0.00713744786,8.56045487e-06, -5.41997874e-09,1.40517535e-12}, blow = { -31626.8996, -59.76797945}, ahigh = { -91885.5968, -15.7193643,16.0129417, -5.57299237e-06,1.302748507e-09, -1.560175322e-13,7.48082757e-18}, bhigh = { -36087.6074, -40.13211395}, R = 27.9951379980269);
          constant IdealGases.Common.DataRecord D(name = "D", MM = 0.002014102, Hf = 110083912.334132, H0 = 3077017.94645951, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {25921.287,0.591714338}, ahigh = {60.5001921, -0.1810766064,2.500210817, -1.220711706e-07,3.71517217e-11, -5.66068021e-15,3.393920393e-19}, bhigh = {25922.43752,0.590212537}, R = 4128.12856548477);
          constant IdealGases.Common.DataRecord Dplus(name = "Dplus", MM = 0.0020135534, Hf = 764978136.661287, H0 = 3077856.29127094, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {184512.0037, -0.1018414521}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = {184512.0037, -0.1018414521}, R = 4129.25328923484);
          constant IdealGases.Common.DataRecord Dminus(name = "Dminus", MM = 0.0020146506, Hf = 70857312.9256259, H0 = 3076180.05821953, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {16423.73393, -0.1010243437}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = {16423.73393, -0.1010243437}, R = 4127.00445427113);
          constant IdealGases.Common.DataRecord DBr(name = "DBr", MM = 0.081918102, Hf =  -452116.139116602, H0 = 105814.268011239, Tlimit = 1000, alow = { -19182.82458,202.0175399,3.044629038, -0.00148815745,6.8708978e-06, -6.59116028e-09,2.093769216e-12}, blow = { -6560.0744,8.00869004}, ahigh = {665400.044, -2594.092228,6.8858794, -0.001103284901,2.894201105e-07, -3.152037514e-11,1.011011776e-15}, bhigh = {10378.1314, -19.73703653}, R = 101.497371118291);
          constant IdealGases.Common.DataRecord DCL(name = "DCL", MM = 0.037467102, Hf =  -2496777.11929788, H0 = 231165.570264815, Tlimit = 1000, alow = {10464.21033, -231.3813446,5.42069326, -0.00750801265,1.398672495e-05, -1.066249569e-08,3.011167635e-12}, blow = { -11284.03374, -6.711842349}, ahigh = {411728.49, -1764.535217,5.6688771, -0.000347596417,5.88268803e-08,3.76025313e-13, -5.16460056e-16}, bhigh = { -1541.258092, -12.75659404}, R = 221.913933989344);
          constant IdealGases.Common.DataRecord DF(name = "DF", MM = 0.0210125052, Hf =  -13145866.871695, H0 = 411093.485416484, Tlimit = 1000, alow = {57213.7075, -731.17338,7.22087087, -0.00942396935,1.208025139e-05, -6.94181269e-09,1.538525476e-12}, blow = { -30692.30024, -19.32760992}, ahigh = {800117.28, -2438.386832,5.62066445, -0.0002020416838,1.714418979e-08,2.697462563e-12, -2.88829741e-16}, bhigh = { -18574.47029, -14.73004444}, R = 395.691609394581);
          constant IdealGases.Common.DataRecord DOCL(name = "DOCL", MM = 0.053466502, Hf =  -1487635.86591096, H0 = 193108.088499973, Tlimit = 1000, alow = {68528.3429, -767.344455,5.93397014,0.002868820667, -5.20716637e-06,4.92757274e-09, -1.709526276e-12}, blow = { -6824.03505, -7.757833983}, ahigh = {604306.633, -2646.500381,8.61247421, -0.000553226824,1.086845232e-07, -1.13738129e-11,4.90429552e-16}, bhigh = {4845.79518, -25.88861801}, R = 155.508059981182);
          constant IdealGases.Common.DataRecord DO2(name = "DO2", MM = 0.034012902, Hf = 190731.858163705, H0 = 295927.233730306, Tlimit = 1000, alow = { -21114.79735,602.337175, -1.294877674,0.0181294797, -2.161807666e-05,1.391729127e-08, -3.69774028e-12}, blow = { -2976.943856,32.7283448}, ahigh = { -1267224.927,2799.947016,2.325174609,0.00272632507, -6.31450732e-07,6.7292892e-11, -2.765192818e-15}, bhigh = { -19594.11733,17.89922833}, R = 244.450532330349);
          constant IdealGases.Common.DataRecord DO2minus(name = "DO2minus", MM = 0.0340134506, Hf =  -3081002.01983036, H0 = 296350.11509241, Tlimit = 1000, alow = {104870.5051, -890.623677,5.66655356,0.001904603784, -1.21526321e-06,6.57751333e-10, -2.147376147e-13}, blow = { -8942.37725, -7.7972192}, ahigh = {552766.192, -2796.895783,8.77220667, -0.000629111663,1.272331622e-07, -1.364440687e-11,6.00519944e-16}, bhigh = {2512.066767, -28.90020381}, R = 244.446589608877);
          constant IdealGases.Common.DataRecord D2(name = "D2", MM = 0.004028204, Hf = 0, H0 = 2127276.324635, Tlimit = 1000, alow = {21257.90482, -299.6945907,5.13031498, -0.00417297089,5.01434572e-06, -2.126389969e-09,2.386536969e-13}, blow = {394.49859, -11.64191209}, ahigh = {821516.856, -2365.623159,5.34297451,6.92814599e-05, -8.52367102e-08,2.456447415e-11, -1.960597698e-15}, bhigh = {14342.14587, -17.12600356}, R = 2064.06428274238);
          constant IdealGases.Common.DataRecord D2plus(name = "D2plus", MM = 0.0040276554, Hf = 372069647.517511, H0 = 2147899.74335937, Tlimit = 1000, alow = { -96409.5909,1243.052385, -2.557714366,0.01343064234, -1.285600289e-05,6.46342167e-09, -1.337616868e-12}, blow = {173096.6171,33.5631492}, ahigh = {925595.135, -4505.21994,11.03203365, -0.00470608903,1.83806846e-06, -3.135924623e-10,1.857684975e-14}, bhigh = {205829.889, -52.8391224}, R = 2064.34542538073);
          constant IdealGases.Common.DataRecord D2minus(name = "D2minus", MM = 0.0040287526, Hf = 58370578.7741844, H0 = 2162915.38974123, Tlimit = 1000, alow = { -4365.33206,311.2987057,0.548195003,0.00995698811, -1.167000612e-05,6.97565641e-09, -1.682718179e-12}, blow = {25978.97625,14.42223161}, ahigh = { -57988.0519, -312.2959355,4.73038828,5.6959008e-05,2.01897543e-08, -2.311492448e-12,1.070266527e-16}, bhigh = {28512.00896, -9.15752792}, R = 2063.78321667109);
          constant IdealGases.Common.DataRecord D2O(name = "D2O", MM = 0.020027604, Hf =  -12443325.7218387, H0 = 497314.456587019, Tlimit = 1000, alow = {6958.27847, -12.80889437,3.59587887,0.001502093683,3.59467505e-07,5.3404172e-10, -5.18194127e-13}, blow = { -31019.44566,2.895556576}, ahigh = {1544193.253, -5474.2389,10.17542424, -0.000961941554,2.036545675e-07, -2.050566442e-11,8.51077069e-16}, bhigh = {2983.24898, -44.6501157}, R = 415.15060912928);
          constant IdealGases.Common.DataRecord D2O2(name = "D2O2", MM = 0.036027004, Hf =  -4005328.89162807, H0 = 321026.944122248, Tlimit = 1000, alow = {29577.11324, -68.9303757,2.043905473,0.01570281822, -1.935478714e-05,1.384941336e-08, -4.1041849e-12}, blow = { -18025.02679,13.47156482}, ahigh = {1147867.936, -5225.76093,13.11088701, -0.001179811896,2.729336904e-07, -2.961433535e-11,1.310306129e-15}, bhigh = {12195.80532, -56.905382}, R = 230.784441581654);
          constant IdealGases.Common.DataRecord D2S(name = "D2S", MM = 0.036093204, Hf =  -665126.598347988, H0 = 279513.92178982, Tlimit = 1000, alow = {3988.38648, -66.2079256,4.39335445, -0.002325113084,1.189503465e-05, -1.146524521e-08,3.62619045e-12}, blow = { -3787.38555,0.9238754375}, ahigh = {423581.463, -2823.776231,8.96282184, -0.000646923123,1.634615644e-07, -1.797991376e-11,8.16093957e-16}, bhigh = {12046.03509, -31.91047376}, R = 230.361150536816);
          constant IdealGases.Common.DataRecord eminus(name = "eminus", MM = 5.48579903e-07, Hf = 0, H0 = 11297220270.2074, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = { -745.375, -11.72081224}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = { -745.375, -11.72081224}, R = 15156355.4452705);
          constant IdealGases.Common.DataRecord F(name = "F", MM = 0.0189984032, Hf = 4178245.88542262, H0 = 343105.677428722, Tlimit = 1000, alow = {1137.409088, -145.3392797,4.07740361, -0.00430336014,5.72889774e-06, -3.8193129e-09,1.018322509e-12}, blow = {9311.11012, -3.55898265}, ahigh = {14735.06226,81.4992736,2.444371819,2.120210026e-05, -4.54691862e-09,5.10952873e-13, -2.333894647e-17}, bhigh = {8388.37465,5.47871064}, R = 437.640569708511);
          constant IdealGases.Common.DataRecord Fplus(name = "Fplus", MM = 0.0189978546, Hf = 93000834.5258101, H0 = 353236.307009108, Tlimit = 1000, alow = { -38716.8019,321.881566,2.200920452, -0.0002455492688,7.85835506e-07, -6.43598792e-10,1.839793564e-13}, blow = {209883.0937,7.81699924}, ahigh = {16496.35664,133.7351478,2.332522942,0.0001215277877, -4.8010377e-08,9.02722515e-12, -5.47066494e-16}, bhigh = {211074.5327,6.62581709}, R = 437.653207431117);
          constant IdealGases.Common.DataRecord Fminus(name = "Fminus", MM = 0.0189989518, Hf =  -13426639.2528034, H0 = 326198.416904242, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = { -31425.72443,3.26488271}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = { -31425.72443,3.26488271}, R = 437.627932715741);
          constant IdealGases.Common.DataRecord FCN(name = "FCN", MM = 0.0450158032, Hf = 762573.39777956, H0 = 225008.980846086, Tlimit = 1000, alow = {39844.5412, -698.865536,7.05883966, -0.001404863382,3.96465251e-06, -3.045609294e-09,7.9230459e-13}, blow = {6172.62757, -15.05643174}, ahigh = {398187.355, -2302.971079,9.02708139, -0.000522871026,1.191059272e-07, -1.280287231e-11,5.73380719e-16}, bhigh = {15884.49451, -29.84971982}, R = 184.701180673369);
          constant IdealGases.Common.DataRecord FCO(name = "FCO", MM = 0.0470085032, Hf =  -3816716.92962987, H0 = 220980.339573969, Tlimit = 1000, alow = {11326.62744, -53.979185,2.966927601,0.00755995935, -6.21177358e-06,2.40378155e-09, -3.36950775e-13}, blow = { -22403.69579,10.92652142}, ahigh = { -60858.5158, -1022.397533,7.52732256, -0.0001057942328, -1.365311093e-09,2.484612871e-12, -9.9979168e-17}, bhigh = { -18050.98416, -16.30331278}, R = 176.871660104251);
          constant IdealGases.Common.DataRecord FO(name = "FO", MM = 0.0349978032, Hf = 3114826.93290875, H0 = 268245.408043211, Tlimit = 1000, alow = { -39121.8244,796.703694, -1.634777767,0.01601810071, -2.095210771e-05,1.382740666e-08, -3.66452495e-12}, blow = {8375.52523,33.8332572}, ahigh = { -1597940.503,4377.37638, -0.489750764,0.002682336321, -6.90080485e-07,7.24647968e-11, -2.726912632e-15}, bhigh = { -16442.48211,35.5992361}, R = 237.571254186606);
          constant IdealGases.Common.DataRecord FO2_FOO(name = "FO2_FOO", MM = 0.0509972032, Hf = 498066.529264099, H0 = 220714.299877527, Tlimit = 1000, alow = {5821.564, -234.7363967,5.43733876,0.002165855252,3.67147219e-07, -2.071530827e-09,9.43106685e-13}, blow = {2694.856027, -1.168202057}, ahigh = { -1213166.895,2493.397189,4.46506574,0.000941610404, -6.42647226e-08, -1.085643277e-11,1.216995394e-15}, bhigh = { -15968.00286,8.65519318}, R = 163.037803610375);
          constant IdealGases.Common.DataRecord FO2_OFO(name = "FO2_OFO", MM = 0.0509972032, Hf = 7423936.53462157, H0 = 206646.273496034, Tlimit = 1000, alow = { -77341.064,1663.800209, -9.61281516,0.0498915687, -6.94621713e-05,4.7077878e-08, -1.258890729e-11}, blow = {36970.9273,77.9594304}, ahigh = { -621186.734,936.947459,6.40155875,0.0002136110846, -4.33797765e-08,4.66731224e-12, -2.059767432e-16}, bhigh = {36383.5727, -6.15026433}, R = 163.037803610375);
          constant IdealGases.Common.DataRecord F2(name = "F2", MM = 0.0379968064, Hf = 0, H0 = 232259.151126975, Tlimit = 1000, alow = {10181.76308,22.74241183,1.97135304,0.00815160401, -1.14896009e-05,7.95865253e-09, -2.167079526e-12}, blow = { -958.6943,11.30600296}, ahigh = { -2941167.79,9456.5977, -7.73861615,0.00764471299, -2.241007605e-06,2.915845236e-10, -1.425033974e-14}, bhigh = { -60710.0561,84.2383508}, R = 218.820284854256);
          constant IdealGases.Common.DataRecord F2O(name = "F2O", MM = 0.0539962064, Hf = 453735.579468412, H0 = 202079.400155786, Tlimit = 1000, alow = {30829.19995, -229.9506259,2.603805825,0.01586111264, -2.345633734e-05,1.679619314e-08, -4.70356033e-12}, blow = {3055.185332,10.50933022}, ahigh = { -188537.4518, -210.0729689,7.15123916,0.0001327687906,1.804705706e-08, -1.416973671e-12,6.48938939e-17}, bhigh = {1449.129965, -12.57858336}, R = 153.982521260975);
          constant IdealGases.Common.DataRecord F2O2(name = "F2O2", MM = 0.0699956064, Hf = 274302.931105116, H0 = 196847.398124692, Tlimit = 1000, alow = {56122.7046, -998.595223,8.98064939,0.00642190087, -9.88851602e-06,6.85806101e-09, -1.842137493e-12}, blow = {5298.71259, -22.39292195}, ahigh = { -219056.3532, -391.092081,10.29107144, -0.0001163345202,2.570922521e-08, -2.949199983e-12,1.367386916e-16}, bhigh = {827.802094, -27.56752555}, R = 118.785627093303);
          constant IdealGases.Common.DataRecord FS2F(name = "FS2F", MM = 0.1021268064, Hf =  -3294286.89547272, H0 = 142913.506399432, Tlimit = 1000, alow = {114446.228, -1853.247564,13.94098518, -0.00404368937,1.370970316e-06,6.7119031e-10, -4.62495496e-13}, blow = { -33510.6833, -48.49215357}, ahigh = { -207709.7921, -109.4539831,10.08136946, -3.23953791e-05,7.12522527e-09, -8.13478741e-13,3.75502241e-17}, bhigh = { -43494.4606, -23.46916058}, R = 81.4132184593603);
          constant IdealGases.Common.DataRecord Fe(name = "Fe", MM = 0.055845, Hf = 7439717.07404423, H0 = 122666.792013609, Tlimit = 1000, alow = {67908.2266, -1197.218407,9.84339331, -0.01652324828,1.917939959e-05, -1.149825371e-08,2.832773807e-12}, blow = {54669.9594, -33.8394626}, ahigh = { -1954923.682,6737.1611, -5.48641097,0.00437880345, -1.116286672e-06,1.544348856e-10, -8.023578182e-15}, bhigh = {7137.37006,65.0497986}, R = 148.884806159907);
          constant IdealGases.Common.DataRecord Feplus(name = "Feplus", MM = 0.0558444514, Hf = 21205649.2509478, H0 = 124197.638729065, Tlimit = 1000, alow = { -56912.3162,184.713439,4.19697212, -0.00597827597,1.054267912e-05, -8.05980432e-09,2.256925874e-12}, blow = {140120.6571, -0.360254258}, ahigh = { -817645.009,1925.359408,1.717387154,0.000338533898, -9.81353312e-08,2.228179208e-11, -1.483964439e-15}, bhigh = {128635.2466,15.00256262}, R = 148.886268761877);
          constant IdealGases.Common.DataRecord Fe_CO_5(name = "Fe_CO_5", MM = 0.1958955, Hf =  -3715501.37700968, H0 = 169199.389470406, Tlimit = 1000, alow = {379780.571, -7285.92816,54.0023618, -0.0693540075,0.0001026705717, -7.20737313e-08,1.958981996e-11}, blow = { -58545.8048, -260.4377836}, ahigh = {1116600.852, -8067.07473,36.5294241, -0.0020471903,4.44196775e-07, -4.93250713e-11,2.235704865e-15}, bhigh = { -48606.6125, -175.4566113}, R = 42.4434047744844);
          constant IdealGases.Common.DataRecord FeCL(name = "FeCL", MM = 0.091298, Hf = 2749676.88229753, H0 = 113662.435102631, Tlimit = 1000, alow = {11173.40353, -54.0214429,3.48605792,0.00687983714, -1.273679557e-05,1.025321859e-08, -3.051544011e-12}, blow = {29286.8199,9.428962979}, ahigh = {528870.022, -1282.897413,5.79844169, -0.0002896589776,3.34390381e-08, -1.469606582e-13, -1.213444602e-16}, bhigh = {37261.9673, -4.075134191}, R = 91.069596267169);
          constant IdealGases.Common.DataRecord FeCL2(name = "FeCL2", MM = 0.126751, Hf =  -1112423.57062272, H0 = 112640.831236046, Tlimit = 1000, alow = {23011.21607, -585.804406,9.32290348, -0.003006298824,2.590788666e-06, -1.080178662e-09,2.341239608e-13}, blow = { -16210.41276, -18.16752393}, ahigh = {164412.3697,692.800269,4.63827014,0.002754339782, -8.62421825e-07,1.170827576e-10, -5.93806195e-15}, bhigh = { -22197.21855,11.16897701}, R = 65.5968946990556);
          constant IdealGases.Common.DataRecord FeCL3(name = "FeCL3", MM = 0.162204, Hf =  -6529458.509038, H0 = 112291.145717738, Tlimit = 1000, alow = {4284.7721, -574.170537,12.20667256, -0.00468043412,5.60950047e-06, -3.5495995e-09,9.19854349e-13}, blow = { -127568.8456, -28.87584081}, ahigh = { -59157.4059, -10.33382228,10.00849967, -3.65765187e-06,8.54587364e-10, -1.023070527e-13,4.90406364e-18}, bhigh = { -130501.205, -15.98494171}, R = 51.2593524204089);
          constant IdealGases.Common.DataRecord FeO(name = "FeO", MM = 0.0718444, Hf = 3494218.06014108, H0 = 123002.363440992, Tlimit = 1000, alow = {15692.82213, -64.6018888,2.45892547,0.00701604736, -1.021405947e-05,7.17929787e-09, -1.978966365e-12}, blow = {29645.72665,13.26115545}, ahigh = { -119597.148, -362.486478,5.51888075, -0.000997885689,4.37691383e-07, -6.79062946e-11,3.63929268e-15}, bhigh = {30379.85806, -3.63365542}, R = 115.728880747838);
          constant IdealGases.Common.DataRecord Fe_OH_2(name = "Fe_OH_2", MM = 0.08985968, Hf =  -3678357.18978746, H0 = 158129.441369032, Tlimit = 1000, alow = {444302.72, -6795.14089,38.9472621, -0.0597300568,7.04616543e-05, -4.08785951e-08,9.36876634e-12}, blow = { -9051.42086, -193.1304058}, ahigh = {1612519.19, -6533.24199,18.42922816, -0.002073249635,4.26587436e-07, -4.56406313e-11,1.990105746e-15}, bhigh = { -2992.568633, -84.4594059}, R = 92.5272825365058);
          constant IdealGases.Common.DataRecord Fe2CL4(name = "Fe2CL4", MM = 0.253502, Hf =  -1701644.95743623, H0 = 117747.398442616, Tlimit = 1000, alow = {1501.308814, -661.709651,18.26924882, -0.00418149466,4.19653505e-06, -2.188670747e-09,5.3951947e-13}, blow = { -53400.5758, -49.3564068}, ahigh = {140245.0973,693.606441,13.1380041,0.00275433307, -8.62399601e-07,1.170782491e-10, -5.93777808e-15}, bhigh = { -59741.7273, -17.52491511}, R = 32.7984473495278);
          constant IdealGases.Common.DataRecord Fe2CL6(name = "Fe2CL6", MM = 0.324408, Hf =  -2017143.84355503, H0 = 124682.048531479, Tlimit = 1000, alow = { -10345.44447, -823.48766,25.20090594, -0.00684579679,8.25582075e-06, -5.24896566e-09,1.365242237e-12}, blow = { -81318.4719, -80.10768938}, ahigh = { -99991.18, -14.57716085,22.0120846, -5.22948113e-06,1.226831608e-09, -1.473195806e-13,7.07827233e-18}, bhigh = { -85514.9619, -61.43757198}, R = 25.6296762102044);
          constant IdealGases.Common.DataRecord Ga(name = "Ga", MM = 0.069723, Hf = 3901151.70029976, H0 = 93959.1669893722, Tlimit = 1000, alow = {238794.789, -3121.634631,16.30171272, -0.02347922342,1.932327565e-05, -7.31631187e-09,8.857387735e-13}, blow = {47327.1738, -75.4717276}, ahigh = { -55441.8652,880.862442,1.760717716,0.000299325946, -5.61082731e-08,2.682832239e-12,3.132134914e-16}, bhigh = {26843.51822,12.52212023}, R = 119.250060955495);
          constant IdealGases.Common.DataRecord Gaplus(name = "Gaplus", MM = 0.0697224514, Hf = 12287124.316458, H0 = 88887.1213727864, Tlimit = 1000, alow = {0.000409834494, -4.34358162e-06,2.500000019, -4.38903681e-11,5.5639692e-14, -3.63822826e-17,9.607881995e-21}, blow = {102289.9726,5.21509046}, ahigh = {35666.0714, -110.8845546,2.635517951, -8.33897352e-05,2.734243614e-08, -4.55567374e-12,3.035050101e-16}, bhigh = {102989.743,4.25707541}, R = 119.2509992556);
          constant IdealGases.Common.DataRecord GaBr(name = "GaBr", MM = 0.149627, Hf =  -120084.476732141, H0 = 66379.1895847675, Tlimit = 1000, alow = { -2185.275415, -76.7223297,4.77080567, -0.000449788106,5.5798536e-07, -3.065156457e-10,7.04294604e-14}, blow = { -3138.004818,2.948144294}, ahigh = {335980.503, -840.683668,5.14508473, -3.265526e-05, -7.67102275e-08,3.63512382e-11, -3.35817783e-15}, bhigh = {2035.179445, -0.427789408}, R = 55.5679924077874);
          constant IdealGases.Common.DataRecord GaBr2(name = "GaBr2", MM = 0.229531, Hf =  -649938.143431606, H0 = 61102.6310171611, Tlimit = 1000, alow = {7081.68162, -370.57661,8.41842536, -0.003001930915,3.59547767e-06, -2.276883735e-09,5.91322177e-13}, blow = { -18211.16655, -9.51257205}, ahigh = { -330229.436,476.672265,7.1047254, -0.000552277121,3.159876015e-07, -5.51363715e-11,3.17329323e-15}, bhigh = { -23656.8362, -1.162714892}, R = 36.2237431980865);
          constant IdealGases.Common.DataRecord GaBr3(name = "GaBr3", MM = 0.309435, Hf =  -946766.4065151, H0 = 61522.8949537059, Tlimit = 1000, alow = {6363.80734, -514.204045,11.96065138, -0.00413463765,4.93397833e-06, -3.111868526e-09,8.04363148e-13}, blow = { -35704.155, -25.56896053}, ahigh = { -51030.6375, -9.39520969,10.00768765, -3.2959543e-06,7.67983814e-10, -9.17514726e-14,4.39118694e-18}, bhigh = { -38334.0215, -14.10315914}, R = 26.8698498876986);
          constant IdealGases.Common.DataRecord GaCL(name = "GaCL", MM = 0.105176, Hf =  -661951.776070586, H0 = 91362.2499429528, Tlimit = 1000, alow = {6417.143, -225.7949147,5.31167369, -0.001562280975,1.836579731e-06, -1.100421873e-09,2.725812445e-13}, blow = { -8593.89233, -1.695911735}, ahigh = { -486484.224,1452.870183,2.719542959,0.001144285628, -3.43145286e-07,5.47044048e-11, -3.019622103e-15}, bhigh = { -18950.84035,15.70926145}, R = 79.0529398341827);
          constant IdealGases.Common.DataRecord GaCL2(name = "GaCL2", MM = 0.140629, Hf =  -1571360.8786239, H0 = 96092.4915913503, Tlimit = 1000, alow = {22254.81851, -604.621844,9.22038891, -0.00455440079,5.32485518e-06, -3.30860557e-09,8.46223556e-13}, blow = { -25645.50386, -16.85409262}, ahigh = { -344540.53,471.443716,7.10877548, -0.000553943356,3.16363824e-07, -5.51802393e-11,3.17535303e-15}, bhigh = { -32309.3616, -3.74484074}, R = 59.1234524884625);
          constant IdealGases.Common.DataRecord GaCL3(name = "GaCL3", MM = 0.176082, Hf =  -2456950.47193921, H0 = 98677.0197975943, Tlimit = 1000, alow = {47849.484, -1162.678125,14.08954213, -0.00810613429,9.21943016e-06, -5.59971055e-09,1.4048526e-12}, blow = { -49159.0973, -42.82179383}, ahigh = { -94669.1731, -25.61468254,10.02010048, -8.35407307e-06,1.901439055e-09, -2.231168493e-13,1.052949609e-17}, bhigh = { -55182.0143, -18.64958383}, R = 47.2193182721687);
          constant IdealGases.Common.DataRecord GaF(name = "GaF", MM = 0.0887214032, Hf =  -2621785.52874827, H0 = 102355.335606324, Tlimit = 1000, alow = {36703.9801, -533.657222,5.75929192, -0.001506128492,9.50903845e-07, -1.83094113e-10, -3.135959193e-14}, blow = { -26470.7569, -6.58932188}, ahigh = { -298339.2279,722.515787,3.63955743,0.000554673676, -1.46490145e-07,2.177385396e-11, -1.031058874e-15}, bhigh = { -34085.2871,7.55396}, R = 93.7143879617991);
          constant IdealGases.Common.DataRecord GaF2(name = "GaF2", MM = 0.1077198064, Hf =  -4796819.25050322, H0 = 112473.82821141, Tlimit = 1000, alow = {68425.3953, -889.761774,7.70745549,0.00207215089, -5.28998193e-06,4.56412742e-09, -1.406942753e-12}, blow = { -59198.5615, -13.41142116}, ahigh = {410025.136, -1723.259472,8.95132922, -0.001059205682,2.745755154e-07, -2.907681311e-11,1.084834142e-15}, bhigh = { -53765.716, -21.13835528}, R = 77.1861023322448);
          constant IdealGases.Common.DataRecord GaF3(name = "GaF3", MM = 0.1267182096, Hf =  -7271859.23719049, H0 = 120132.828960046, Tlimit = 1000, alow = {95932.667, -1355.507494,10.97760197,0.00346157172, -8.46955195e-06,7.22246893e-09, -2.213328653e-12}, blow = { -106147.9087, -32.1265185}, ahigh = { -209696.176, -143.1211274,10.10568378, -4.18493682e-05,9.16475945e-09, -1.042661957e-12,4.79916345e-17}, bhigh = { -113667.8134, -23.9253933}, R = 65.6138689636284);
          constant IdealGases.Common.DataRecord GaH(name = "GaH", MM = 0.07073094, Hf = 3030122.37643102, H0 = 122621.07643416, Tlimit = 1000, alow = { -43918.762,625.445712,0.327686289,0.0064966415, -3.80297001e-06,2.852103704e-10,3.62680913e-13}, blow = {21712.60108,22.24032244}, ahigh = {3257993.99, -11090.99502,18.01126796, -0.00816771911,2.601293566e-06, -3.81345935e-10,2.028049649e-14}, bhigh = {93697.7824, -98.2090022}, R = 117.550706946635);
          constant IdealGases.Common.DataRecord GaI(name = "GaI", MM = 0.19662747, Hf = 228203.729621299, H0 = 51412.5620392715, Tlimit = 1000, alow = { -3661.85503, -32.0692418,4.59996192, -8.89947546e-05,1.162792633e-07, -1.812351457e-11, -7.58461964e-15}, blow = {4198.64937,4.92896465}, ahigh = {1498356.654, -4551.96724,9.80300535, -0.002943735609,8.65804051e-07, -1.106638204e-10,5.00783686e-15}, bhigh = {32918.1191, -32.2735977}, R = 42.2854039672076);
          constant IdealGases.Common.DataRecord GaI2(name = "GaI2", MM = 0.32353194, Hf =  -89496.4651712594, H0 = 44727.9950165044, Tlimit = 1000, alow = { -983.13959, -215.2621259,7.84431876, -0.001819797849,2.210706958e-06, -1.416110855e-09,3.71376293e-13}, blow = { -4534.09023, -4.25051681}, ahigh = { -320287.975,479.650233,7.10233396, -0.000551265783,3.157543452e-07, -5.51087194e-11,3.17197773e-15}, bhigh = { -9180.91961,0.727520488}, R = 25.6990762643095);
          constant IdealGases.Common.DataRecord GaI3(name = "GaI3", MM = 0.45043641, Hf =  -257255.491402216, H0 = 44673.9196771416, Tlimit = 1000, alow = { -5663.47265, -242.5937116,10.95451119, -0.002059175489,2.499117622e-06, -1.596502506e-09,4.16764406e-13}, blow = { -15767.22832, -16.46129942}, ahigh = { -31640.4013, -4.20866283,10.00352001, -1.53265018e-06,3.61153154e-10, -4.35097887e-14,2.095704875e-18}, bhigh = { -17000.58453, -10.90291552}, R = 18.4587031940868);
          constant IdealGases.Common.DataRecord GaO(name = "GaO", MM = 0.0857224, Hf = 1712779.93849916, H0 = 104116.391981559, Tlimit = 1000, alow = {73255.4391, -980.426951,7.84412276, -0.00717107285,7.87278051e-06, -2.401464112e-09, -2.594919458e-13}, blow = {21405.81724, -17.99742951}, ahigh = {2937313.804, -11183.14112,18.59366773, -0.00685475345,1.665399251e-06, -1.887560878e-10,7.97268692e-15}, bhigh = {85302.4808, -99.0404048}, R = 96.9929913301541);
          constant IdealGases.Common.DataRecord GaOH(name = "GaOH", MM = 0.08673034, Hf =  -1656058.24905102, H0 = 122028.900151896, Tlimit = 1000, alow = {69631.0187, -1247.828695,10.22001611, -0.00592253439,4.6433874e-06, -1.215936978e-09, -2.945243614e-14}, blow = { -12753.97794, -32.17063}, ahigh = {842905.101, -2372.324595,8.00907109,8.77723071e-05, -5.94306891e-08,9.84194344e-12, -5.53155279e-16}, bhigh = { -3751.78825, -21.71053719}, R = 95.8657835308843);
          constant IdealGases.Common.DataRecord Ga2Br2(name = "Ga2Br2", MM = 0.299254, Hf =  -457683.746917334, H0 = 69483.8097402207, Tlimit = 1000, alow = { -11171.83241, -74.7147162,10.30246084, -0.000665792985,8.20051585e-07, -5.29702152e-10,1.394553192e-13}, blow = { -19132.97856, -13.17110316}, ahigh = { -18853.73131, -1.263621802,10.00108034, -4.77497566e-07,1.137227648e-10, -1.380820486e-14,6.69024243e-19}, bhigh = { -19510.53843, -11.41667594}, R = 27.7839962038937);
          constant IdealGases.Common.DataRecord Ga2Br4(name = "Ga2Br4", MM = 0.459062, Hf =  -905803.647437601, H0 = 66639.7262243444, Tlimit = 1000, alow = { -13155.45939, -413.655302,17.62849229, -0.00351465216,4.26693773e-06, -2.726515882e-09,7.11892449e-13}, blow = { -52831.0181, -44.2752069}, ahigh = { -57421.6721, -7.16013737,16.00598947, -2.60818227e-06,6.14645595e-10, -7.40543865e-14,3.56712663e-18}, bhigh = { -54933.8976, -34.7927166}, R = 18.1118715990433);
          constant IdealGases.Common.DataRecord Ga2Br6(name = "Ga2Br6", MM = 0.61887, Hf =  -1088578.75159565, H0 = 66494.2007206683, Tlimit = 1000, alow = { -6725.11689, -806.752655,25.11671424, -0.00663651457,7.9775557e-06, -5.05968521e-09,1.313546388e-12}, blow = { -83706.352, -75.8527872}, ahigh = { -95261.73, -14.4468188,22.01192546, -5.14500674e-06,1.204348449e-09, -1.443806473e-13,6.92827074e-18}, bhigh = { -87822.233, -57.6589512}, R = 13.4349249438493);
          constant IdealGases.Common.DataRecord Ga2CL2(name = "Ga2CL2", MM = 0.210352, Hf =  -1050490.06427322, H0 = 92646.7302426409, Tlimit = 1000, alow = { -9246.1337, -259.2159934,11.02847129, -0.002231983927,2.720723456e-06, -1.743798624e-09,4.56366604e-13}, blow = { -28340.59821, -20.67773272}, ahigh = { -36689.542, -4.43458068,10.00373051, -1.630793333e-06,3.85375332e-10, -4.65252773e-14,2.244498791e-18}, bhigh = { -29656.33085, -14.69535817}, R = 39.5264699170913);
          constant IdealGases.Common.DataRecord Ga2CL4(name = "Ga2CL4", MM = 0.281258, Hf =  -2141545.58448115, H0 = 97118.6312922655, Tlimit = 1000, alow = {16373.43702, -1140.637354,20.30139827, -0.0089977462,1.067254867e-05, -6.70029062e-09,1.725749144e-12}, blow = { -71623.8501, -66.663619}, ahigh = { -112711.0409, -21.38517542,16.01738077, -7.41556464e-06,1.721694751e-09, -2.051361953e-13,9.7973115e-18}, bhigh = { -77468.5338, -41.4728087}, R = 29.5617262442313);
          constant IdealGases.Common.DataRecord Ga2CL6(name = "Ga2CL6", MM = 0.352164, Hf =  -2732998.06908145, H0 = 103246.77990936, Tlimit = 1000, alow = {60894.6984, -2104.456302,29.59653878, -0.01537809796,1.779096808e-05, -1.095619429e-08,2.779706493e-12}, blow = { -111840.0535, -112.1945814}, ahigh = { -189844.5426, -44.1365524,22.03505047, -1.470071128e-05,3.36951281e-09, -3.9754628e-13,1.884231868e-17}, bhigh = { -122696.9998, -67.45332883}, R = 23.6096591360843);
          constant IdealGases.Common.DataRecord Ga2F2(name = "Ga2F2", MM = 0.1774428064, Hf =  -3416488.70585041, H0 = 96638.7781387119, Tlimit = 1000, alow = {29781.3065, -1013.97552,13.74933053, -0.00772805096,9.06400733e-06, -5.64119322e-09,1.443106716e-12}, blow = { -70860.8337, -41.0983395}, ahigh = { -87732.3493, -19.8377587,10.0159403, -6.74440283e-06,1.556129063e-09, -1.845316975e-13,8.78088402e-18}, bhigh = { -76073.6863, -19.08325521}, R = 46.8571939808995);
          constant IdealGases.Common.DataRecord Ga2F4(name = "Ga2F4", MM = 0.2154396128, Hf =  -6150229.01675026, H0 = 105148.917163297, Tlimit = 1000, alow = {142207.9998, -2798.829941,23.95904235, -0.01293904774,1.223101326e-05, -6.25120982e-09,1.334919195e-12}, blow = { -149601.371, -97.3415364}, ahigh = { -270436.609, -104.9957196,16.07903965, -3.17889592e-05,7.04958272e-09, -8.10237952e-13,3.76054053e-17}, bhigh = { -164416.9589, -48.9659173}, R = 38.5930511661224);
          constant IdealGases.Common.DataRecord Ga2F6(name = "Ga2F6", MM = 0.2534364192, Hf =  -7961065.14355298, H0 = 118891.657699053, Tlimit = 1000, alow = {240904.5572, -4142.3798,31.9577372, -0.0129791429,8.95928219e-06, -2.733621068e-09,1.554030994e-13}, blow = { -227278.524, -139.8403732}, ahigh = { -436866.48, -212.8208368,22.15886373, -6.34628791e-05,1.399738556e-08, -1.601732039e-12,7.40758442e-17}, bhigh = { -249429.8707, -77.8763583}, R = 32.8069344818142);
          constant IdealGases.Common.DataRecord Ga2I2(name = "Ga2I2", MM = 0.39325494, Hf = 34382.14151868, H0 = 54666.6928074699, Tlimit = 1000, alow = { -8960.25267, -31.7205721,10.12930491, -0.0002860191165,3.53533301e-07, -2.289632389e-10,6.04008536e-14}, blow = { -1233.174068, -9.80196838}, ahigh = { -12190.56501, -0.528755447,10.00045392, -2.011599344e-07,4.79946781e-11, -5.83478108e-15,2.829565069e-19}, bhigh = { -1393.258606, -9.05261491}, R = 21.1427019836038);
          constant IdealGases.Common.DataRecord Ga2I4(name = "Ga2I4", MM = 0.64706388, Hf =  -246139.510986149, H0 = 50356.4083966486, Tlimit = 1000, alow = { -17694.95991, -135.4871509,16.54625333, -0.001199014713,1.473754848e-06, -9.50479657e-10,2.499380837e-13}, blow = { -23334.05795, -33.2571289}, ahigh = { -31712.9976, -2.287068836,16.00194812, -8.58896438e-07,2.041966602e-10, -2.476145769e-14,1.198551944e-18}, bhigh = { -24019.40505, -30.08672521}, R = 12.8495381321547);
          constant IdealGases.Common.DataRecord Ga2I6(name = "Ga2I6", MM = 0.90087282, Hf =  -352208.009783223, H0 = 48367.9571995523, Tlimit = 1000, alow = { -21108.91458, -369.540416,23.46811836, -0.00318917821,3.89033904e-06, -2.494830378e-09,6.53200728e-13}, blow = { -43011.8809, -60.148115}, ahigh = { -60156.9815, -6.32920923,22.0053304, -2.332008629e-06,5.51386848e-10, -6.65944535e-14,3.21367043e-18}, bhigh = { -44886.9938, -51.6100875}, R = 9.22935159704341);
          constant IdealGases.Common.DataRecord Ga2O(name = "Ga2O", MM = 0.1554454, Hf =  -639822.452127885, H0 = 78243.5183028896, Tlimit = 1000, alow = {56971.9718, -807.408599,7.73306154,0.001478450874, -4.09774343e-06,3.58115892e-09, -1.107073202e-12}, blow = { -9512.21682, -12.56149299}, ahigh = { -119387.4986, -87.1934161,7.0646782, -2.571524542e-05,5.65110849e-09, -6.44819054e-13,2.975374645e-17}, bhigh = { -13936.94796, -6.9428013}, R = 53.4880543264709);
          constant IdealGases.Common.DataRecord Ge(name = "Ge", MM = 0.07264, Hf = 5063325.99118943, H0 = 101852.753303965, Tlimit = 1000, alow = { -20592.15242, -143.2022103,4.50600233,0.00154718784, -8.51829655e-06,8.24382446e-09, -2.566167305e-12}, blow = {43630.7086, -6.224648889}, ahigh = { -856541.384,3917.95866, -1.809888212,0.002276482224, -5.36562755e-07,5.98495809e-11, -2.541700646e-15}, bhigh = {19565.18798,38.41408752}, R = 114.461343612335);
          constant IdealGases.Common.DataRecord Geplus(name = "Geplus", MM = 0.0726394514, Hf = 15624903.0399478, H0 = 85432.7074391974, Tlimit = 1000, alow = { -345366.643,4008.44971, -15.22395737,0.0362603002, -3.35136726e-05,1.431059219e-08, -2.236976459e-12}, blow = {115705.8042,109.0158182}, ahigh = { -2244860.57,5165.53114, -0.386751651,0.000852873094, -1.386040003e-07,1.155945775e-11, -3.784413134e-16}, bhigh = {100682.3985,29.65845411}, R = 114.462208066731);
          constant IdealGases.Common.DataRecord Geminus(name = "Geminus", MM = 0.0726405486, Hf = 3378313.41763848, H0 = 96102.5231023654, Tlimit = 1000, alow = {2989.142308,86.5741723,2.191868507,0.000590079527, -6.37580058e-07,3.66113215e-10, -8.68977845e-14}, blow = {28356.9519,9.417623637}, ahigh = {13520.40246, -0.504851878,2.500140247, -3.65294416e-09, -6.7167222e-12,1.378119925e-15, -8.47521294e-20}, bhigh = {28817.39757,7.577953867}, R = 114.460479170996);
          constant IdealGases.Common.DataRecord GeBr(name = "GeBr", MM = 0.152544, Hf = 900973.614170338, H0 = 64664.0903608139, Tlimit = 1000, alow = { -56807.7463,917.336126, -2.224809366,0.02171239242, -3.035490868e-05,2.011776305e-08, -5.20291504e-12}, blow = {11041.92901,41.07865983}, ahigh = {225675.0476, -731.071546,6.26328381, -0.001198085447,4.00037777e-07, -5.42993788e-11,2.510481953e-15}, bhigh = {19428.63606, -6.106507844}, R = 54.5054017201594);
          constant IdealGases.Common.DataRecord GeBr2(name = "GeBr2", MM = 0.232448, Hf =  -262264.66564565, H0 = 61060.8092992841, Tlimit = 1000, alow = { -1649.285325, -229.6262116,7.90014468, -0.001936685504,2.345776175e-06, -1.496279368e-09,3.9014535e-13}, blow = { -8316.62421, -6.918156136}, ahigh = { -26366.55245, -3.99050902,7.00332744, -1.445749787e-06,3.40159136e-10, -4.09345703e-14,1.969987842e-18}, bhigh = { -9484.99925, -1.673617098}, R = 35.7691698788546);
          constant IdealGases.Common.DataRecord GeBr3(name = "GeBr3", MM = 0.312352, Hf =  -381080.665403135, H0 = 59386.4998463272, Tlimit = 1000, alow = {2725.568677, -518.621326,11.99957754, -0.00425102453,5.10362972e-06, -3.23370625e-09,8.38835886e-13}, blow = { -14779.88251, -25.34555678}, ahigh = { -54339.267, -9.28032795,10.00765019, -3.29732215e-06,7.71292926e-10, -9.24157185e-14,4.43287623e-18}, bhigh = { -17426.97092, -13.66946296}, R = 26.6189171191476);
          constant IdealGases.Common.DataRecord GeBr4(name = "GeBr4", MM = 0.392256, Hf =  -741862.457170827, H0 = 61090.7621553271, Tlimit = 1000, alow = {5431.94039, -677.77488,15.60015305, -0.00550778888,6.59476429e-06, -4.17014614e-09,1.080116643e-12}, blow = { -35576.064, -42.09122927}, ahigh = { -69777.5791, -11.79873115,13.00953955, -4.02493875e-06,9.20356721e-10, -1.076853369e-13,5.0409548e-18}, bhigh = { -39041.4816, -26.8943607}, R = 21.1965451133953);
          constant IdealGases.Common.DataRecord GeCL(name = "GeCL", MM = 0.108093, Hf = 638615.960330456, H0 = 88804.2241403236, Tlimit = 1000, alow = {10697.24739, -22.41238266,1.98737107,0.01368270905, -2.358294414e-05,1.78675602e-08, -5.10250325e-12}, blow = {7440.68339,15.05837189}, ahigh = { -378419.325,1429.309204,3.19485464,0.000811993912, -2.652459954e-07,5.15195788e-11, -3.53421457e-15}, bhigh = { -2087.046335,13.8503382}, R = 76.9196155162684);
          constant IdealGases.Common.DataRecord GeCL2(name = "GeCL2", MM = 0.143546, Hf =  -1191255.7647026, H0 = 92491.5427807114, Tlimit = 1000, alow = {18385.36673, -582.720534,9.16140466, -0.00446478603,5.24487355e-06, -3.26806721e-09,8.36757542e-13}, blow = { -19757.98598, -17.3432163}, ahigh = { -48901.2892, -11.26210418,7.00906353, -3.83918073e-06,8.86563357e-10, -1.05200054e-13,5.00842222e-18}, bhigh = { -22752.54424, -4.656641298}, R = 57.9220040962479);
          constant IdealGases.Common.DataRecord GeCL3(name = "GeCL3", MM = 0.178999, Hf =  -1491352.40420338, H0 = 98097.9614411254, Tlimit = 1000, alow = {19358.59579, -834.202,13.1270253, -0.00651139002,7.69627748e-06, -4.81854173e-09,1.238399959e-12}, blow = { -30972.18753, -35.52547184}, ahigh = { -75744.9982, -15.7921342,10.01278835, -5.44173831e-06,1.260936724e-09, -1.500137106e-13,7.15640081e-18}, bhigh = { -35251.3172, -17.19719944}, R = 46.4498237420321);
          constant IdealGases.Common.DataRecord GeCL4(name = "GeCL4", MM = 0.214452, Hf =  -2331524.0706545, H0 = 98561.0999197956, Tlimit = 1000, alow = {65028.7041, -1652.48276,18.81327, -0.0115242681,1.31080998e-05, -7.96195611e-09,1.99750504e-12}, blow = { -55700.5239, -67.54438135}, ahigh = { -137626.121, -36.037695,13.0281667, -1.16495696e-05,2.63619338e-09, -3.07292672e-13,1.43980875e-17}, bhigh = { -64262.8178, -33.18158305}, R = 38.7707832055658);
          constant IdealGases.Common.DataRecord GeF(name = "GeF", MM = 0.0916384032, Hf =  -770342.864289456, H0 = 99784.6937603557, Tlimit = 1000, alow = {50170.9782, -439.555252,2.990864513,0.01252766738, -2.337593226e-05,1.841631179e-08, -5.39849226e-12}, blow = { -7093.5164,7.064248931}, ahigh = { -515048.058,1659.711777,2.999610299,0.000819851168, -2.271982948e-07,3.62418386e-11, -2.133840473e-15}, bhigh = { -20515.29487,13.7315423}, R = 90.7313059771867);
          constant IdealGases.Common.DataRecord GeF2(name = "GeF2", MM = 0.1106368064, Hf =  -5188146.86248934, H0 = 106542.166061655, Tlimit = 1000, alow = {75430.9708, -1108.456469,9.07889827, -0.001445630636, -6.57532232e-07,1.475344103e-09, -5.81122027e-13}, blow = { -65106.8555, -22.00457134}, ahigh = { -127532.7716, -71.6259135,7.05300237, -2.101910441e-05,4.6078937e-09, -5.24639467e-13,2.416218951e-17}, bhigh = { -71126.3763, -8.483841439}, R = 75.1510484669955);
          constant IdealGases.Common.DataRecord GeF3(name = "GeF3", MM = 0.1296352096, Hf =  -6220014.11104287, H0 = 113226.854380772, Tlimit = 1000, alow = {111412.1569, -1776.90803,13.52166477, -0.002983622949, -3.61274975e-08,1.621528436e-09, -7.19901178e-13}, blow = { -90382.6296, -45.68760023}, ahigh = { -206958.0129, -111.0233803,10.08234122, -3.27167626e-05,7.1838967e-09, -8.19038812e-13,3.77632244e-17}, bhigh = { -99997.4909, -23.0346594}, R = 64.1374517436658);
          constant IdealGases.Common.DataRecord GeF4(name = "GeF4", MM = 0.1486336128, Hf =  -8007273.57412387, H0 = 116346.663949219, Tlimit = 1000, alow = {111981.8968, -1636.85963,12.31371005,0.01025446325, -1.893783072e-05,1.476849261e-08, -4.32517411e-12}, blow = { -137426.5128, -41.04202391}, ahigh = { -325303.268, -253.8279448,13.18792173, -7.46005855e-05,1.637384721e-08, -1.866482733e-12,8.6055305e-17}, bhigh = { -146617.6526, -41.22237561}, R = 55.9393790096987);
          constant IdealGases.Common.DataRecord GeH4(name = "GeH4", MM = 0.07667176, Hf = 1184177.85114102, H0 = 0, Tlimit = 1000, alow = { -256839.8191, -41.3281145,7.75511628,0.002129640783,7.25505361e-07, -5.09631401e-10,1.431608261e-13}, blow = {7881.53804, -20.30107999}, ahigh = { -3809805.88,10931.67093, -5.14967541,0.00948593137, -1.649459859e-06,1.419674126e-10, -5.00463975e-15}, bhigh = { -61585.1852,71.68961273}, R = 108.442430433317);
          constant IdealGases.Common.DataRecord GeI(name = "GeI", MM = 0.19954447, Hf = 1057253.65378454, H0 = 50139.8009175599, Tlimit = 1000, alow = { -74383.0822,1033.759595, -1.520491112,0.01555125152, -1.683415215e-05,8.38355781e-09, -1.550875391e-12}, blow = {19129.20413,39.59395473}, ahigh = { -213314.8437, -295.3079008,6.87660736, -0.002102579132,8.59629938e-07, -1.44769276e-10,8.40104279e-15}, bhigh = {24523.26136, -8.549783749}, R = 41.66726344258);
          constant IdealGases.Common.DataRecord GeO(name = "GeO", MM = 0.0886394, Hf =  -425254.536921504, H0 = 99076.7762417164, Tlimit = 1000, alow = { -6781.22563,279.1946972,0.619895286,0.01111378013, -1.501027092e-05,1.006783003e-08, -2.687101932e-12}, blow = { -6711.84743,21.56441027}, ahigh = { -1044508.485,2734.866048,1.298071298,0.001832638634, -5.06098635e-07,6.37495802e-11, -2.172371872e-15}, bhigh = { -23621.04658,23.50906051}, R = 93.8010861986882);
          constant IdealGases.Common.DataRecord GeO2(name = "GeO2", MM = 0.1046388, Hf =  -1014652.28959048, H0 = 107589.173423243, Tlimit = 1000, alow = { -3824.17383,156.3451633,1.694198285,0.01756205059, -2.418270032e-05,1.632388154e-08, -4.36895786e-12}, blow = { -14775.3488,15.56926515}, ahigh = { -172734.7526, -296.1154443,7.72072625, -8.82493339e-05,1.949874702e-08, -2.235834829e-12,1.036128507e-16}, bhigh = { -13879.83288, -16.69845695}, R = 79.4587858423453);
          constant IdealGases.Common.DataRecord GeS(name = "GeS", MM = 0.104705, Hf = 883674.227591806, H0 = 87303.471658469, Tlimit = 1000, alow = {36610.1843, -565.96535,6.15293641, -0.002746908039,2.733435196e-06, -1.460209187e-09,3.27870947e-13}, blow = {12741.75754, -7.70565946}, ahigh = { -2351367.441,7211.58186, -4.36252631,0.00545173775, -1.721002774e-06,2.632908084e-10, -1.39437404e-14}, bhigh = { -35833.1725,65.05566891}, R = 79.4085478248412);
          constant IdealGases.Common.DataRecord GeS2(name = "GeS2", MM = 0.13677, Hf = 868741.909775536, H0 = 95546.0334868758, Tlimit = 1000, alow = {55436.836, -1024.826697,10.57594552, -0.00528788391,5.29580308e-06, -2.87252953e-09,6.51902755e-13}, blow = {17355.79668, -29.91719536}, ahigh = { -89709.8036, -33.64377,7.52543372, -1.026368137e-05,2.282230718e-09, -2.62874877e-13,1.222218614e-17}, bhigh = {11952.32103, -11.35170616}, R = 60.7916355926007);
          constant IdealGases.Common.DataRecord Ge2(name = "Ge2", MM = 0.14528, Hf = 3245449.67648679, H0 = 73624.2359581498, Tlimit = 1000, alow = {216197.8929, -3079.621733,18.88047728, -0.02604825861,2.27915135e-05, -9.02835127e-09,1.146806387e-12}, blow = {70324.0255, -79.01095968}, ahigh = {1969461.258, -5248.6871,10.58277303, -0.00333655151,1.052745838e-06, -1.484534793e-10,7.47543176e-15}, bhigh = {89256.6599, -37.50233948}, R = 57.2306718061674);
          constant IdealGases.Common.DataRecord H(name = "H", MM = 0.00100794, Hf = 216281552.473361, H0 = 6148608.05206659, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {25473.70801, -0.446682853}, ahigh = {60.7877425, -0.1819354417,2.500211817, -1.226512864e-07,3.73287633e-11, -5.68774456e-15,3.410210197e-19}, bhigh = {25474.86398, -0.448191777}, R = 8248.97513740897);
          constant IdealGases.Common.DataRecord Hplus(name = "Hplus", MM = 0.0010073914, Hf = 1524974233.45087, H0 = 6151956.42924885, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {184021.4877, -1.140646644}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = {184021.4877, -1.140646644}, R = 8253.46732163884);
          constant IdealGases.Common.DataRecord Hminus(name = "Hminus", MM = 0.0010084886, Hf = 137861080.43264, H0 = 6145263.31780052, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {15976.15494, -1.139013868}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = {15976.15494, -1.139013868}, R = 8244.48784051699);
          constant IdealGases.Common.DataRecord HALO(name = "HALO", MM = 0.043988878, Hf = 41399.4191895506, H0 = 225857.476974066, Tlimit = 1000, alow = {26686.71568, -466.321459,5.38380021,0.002902425002, -2.24367761e-07, -1.516294323e-09,6.95173766e-13}, blow = {1235.914473, -6.50883752}, ahigh = {78474.6705, -1515.62885,8.57277623, -0.000414261144,8.9350817e-08, -1.007036446e-11,4.60801725e-16}, bhigh = {6663.24189, -26.72948642}, R = 189.013050071429);
          constant IdealGases.Common.DataRecord HALO2(name = "HALO2", MM = 0.059988278, Hf =  -5925716.87088601, H0 = 199715.751133913, Tlimit = 1000, alow = {3544.59891,115.9819104,0.863797069,0.02460875785, -3.42316012e-05,2.373182987e-08, -6.46760883e-12}, blow = { -44495.0614,20.12317065}, ahigh = {698570.544, -2864.935042,10.86721494, -5.3683127e-05, -2.836877559e-08,6.29164119e-12, -3.88903311e-16}, bhigh = { -27643.92495, -37.7588634}, R = 138.601611468161);
          constant IdealGases.Common.DataRecord HBO(name = "HBO", MM = 0.02781834, Hf =  -7571307.45400337, H0 = 329527.139290123, Tlimit = 1000, alow = {63609.7503, -800.155759,6.21881613, -0.000780167998,3.141286759e-06, -2.031853478e-09,3.73855289e-13}, blow = { -22402.8291, -13.26952393}, ahigh = {886186.023, -3913.06805,9.8886448, -0.000822226914,1.621530554e-07, -1.703550237e-11,7.37274002e-16}, bhigh = { -3161.852029, -40.3681272}, R = 298.884548826422);
          constant IdealGases.Common.DataRecord HBOplus(name = "HBOplus", MM = 0.0278177914, Hf = 42247053.768618, H0 = 326935.696268109, Tlimit = 1000, alow = {65244.7147, -704.733736,5.29427187,0.001308494598,2.088343622e-06, -2.564100339e-09,8.19898125e-13}, blow = {143929.3436, -6.81674923}, ahigh = {18360.83606, -1212.975446,6.82290196,0.00091422725, -2.665355457e-07,3.29704981e-11, -1.530684082e-15}, bhigh = {146361.7504, -16.99646159}, R = 298.890443185939);
          constant IdealGases.Common.DataRecord HBO2(name = "HBO2", MM = 0.04381774, Hf =  -12785005.6392685, H0 = 249170.815290793, Tlimit = 1000, alow = {6225.08747,75.6615369,1.253406833,0.01748006535, -1.982688351e-05,1.22965646e-08, -3.153609847e-12}, blow = { -68785.8878,17.67793507}, ahigh = {1049369.185, -4479.14548,11.97755861, -0.00047357434,6.08020714e-08, -3.64156544e-12,6.15597317e-17}, bhigh = { -42211.4947, -49.1136682}, R = 189.751274255587);
          constant IdealGases.Common.DataRecord HBS(name = "HBS", MM = 0.04388394, Hf = 1144108.75595947, H0 = 211609.509082366, Tlimit = 1000, alow = {56499.6621, -546.640995,3.63645839,0.00996365767, -1.395090386e-05,1.032834167e-08, -3.047186722e-12}, blow = {7919.86053,1.182455314}, ahigh = { -202183.6183, -505.202662,6.41521828,0.001049432932, -3.68314669e-07,5.34296831e-11, -2.154140451e-15}, bhigh = {6485.47562, -13.31379864}, R = 189.465029803614);
          constant IdealGases.Common.DataRecord HBSplus(name = "HBSplus", MM = 0.0438833914, Hf = 25737731.9976231, H0 = 230975.539415579, Tlimit = 1000, alow = {148950.9163, -1847.661736,11.4745802, -0.00947682837,1.110746739e-05, -6.17555861e-09,1.336853512e-12}, blow = {143782.2395, -41.13512498}, ahigh = {661065.836, -2791.498527,9.13126764, -0.000506379529,7.75121314e-08, -4.19113293e-12,2.687149095e-17}, bhigh = {151073.4114, -30.81426166}, R = 189.467398365205);
          constant IdealGases.Common.DataRecord HCN(name = "HCN", MM = 0.02702534, Hf = 4924358.39845123, H0 = 341733.462002698, Tlimit = 1000, alow = {90982.8693, -1238.657512,8.72130787, -0.00652824294,8.87270083e-06, -4.80888667e-09,9.3178985e-13}, blow = {20989.1545, -27.46678076}, ahigh = {1236889.278, -4446.73241,9.73887485, -0.000585518264,1.07279144e-07, -1.013313244e-11,3.34824798e-16}, bhigh = {42215.1377, -40.05774072}, R = 307.654667804364);
          constant IdealGases.Common.DataRecord HCO(name = "HCO", MM = 0.02901804, Hf = 1461085.93137235, H0 = 344249.64608223, Tlimit = 1000, alow = { -11898.51887,215.1536111,2.730224028,0.001806516108,4.98430057e-06, -5.81456792e-09,1.869689894e-12}, blow = {2905.75564,11.3677254}, ahigh = {694960.612, -3656.22338,9.60473117, -0.001117129278,2.875328019e-07, -3.62624774e-11,1.808329595e-15}, bhigh = {25437.0444, -35.8247372}, R = 286.527691050119);
          constant IdealGases.Common.DataRecord HCOplus(name = "HCOplus", MM = 0.0290174914, Hf = 28707995.0681057, H0 = 311742.730455328, Tlimit = 1000, alow = {157344.2506, -1867.692159,10.99235423, -0.01211637888,1.659091514e-05, -1.016592642e-08,2.391234771e-12}, blow = {108493.028, -40.7826162}, ahigh = {1219060.653, -4714.29489,10.21192493, -0.000885451707,1.667408026e-07, -1.683285548e-11,7.04005178e-16}, bhigh = {127798.9027, -43.5115846}, R = 286.533108096312);
          constant IdealGases.Common.DataRecord HCCN(name = "HCCN", MM = 0.03903604, Hf = 15637616.6229976, H0 = 303968.614644313, Tlimit = 1000, alow = {2994.114377, -440.466068,6.94003641,0.00384630496, -1.264728529e-06, -3.63923513e-10,2.748929104e-13}, blow = {73708.7864, -13.15302591}, ahigh = {939562.031, -4091.06775,12.72233302, -0.000687440531,1.231169968e-07, -1.186956238e-11,4.76061035e-16}, bhigh = {95851.6482, -52.48695794}, R = 212.99476073905);
          constant IdealGases.Common.DataRecord HCCO(name = "HCCO", MM = 0.04102874, Hf = 4303522.36017972, H0 = 284312.75247546, Tlimit = 1000, alow = {69596.127, -1164.594402,9.45661626, -0.002331240632,5.1618736e-06, -3.52616997e-09,8.59914323e-13}, blow = {25350.03992, -27.26355351}, ahigh = {1093922.002, -4498.22821,12.46446433, -0.00063433174,1.108549019e-07, -1.125488678e-11,5.68915194e-16}, bhigh = {46522.803, -50.9907043}, R = 202.649947329604);
          constant IdealGases.Common.DataRecord HCL(name = "HCL", MM = 0.03646094, Hf =  -2531750.41565028, H0 = 236968.76712449, Tlimit = 1000, alow = {20625.88287, -309.3368855,5.27541885, -0.00482887422,6.1957946e-06, -3.040023782e-09,4.91679003e-13}, blow = { -10677.82299, -7.309305408}, ahigh = {915774.951, -2770.550211,5.97353979, -0.000362981006,4.73552919e-08,2.810262054e-12, -6.65610422e-16}, bhigh = {5674.95805, -16.42825822}, R = 228.037785092759);
          constant IdealGases.Common.DataRecord HD(name = "HD", MM = 0.003022042, Hf = 106981.306017587, H0 = 2815679.59677596, Tlimit = 1000, alow = {25191.20338, -276.1004999,4.64444129, -0.002082376844,1.418070803e-06,2.839893835e-10, -3.20233103e-13}, blow = {391.361643, -9.39539612}, ahigh = {845583.0, -1956.578537,4.40437387,0.000575168109, -2.131983152e-07,4.03612668e-11, -2.727170705e-15}, bhigh = {12272.54163, -10.84742878}, R = 2751.27612389239);
          constant IdealGases.Common.DataRecord HDplus(name = "HDplus", MM = 0.0030214934, Hf = 495381726.797748, H0 = 2850908.09730049, Tlimit = 1000, alow = { -80700.7305,879.643258,0.0596893216,0.00541818101, -2.021515155e-06, -4.94526165e-10,4.09293565e-13}, blow = {174499.2497,19.34281193}, ahigh = {1340083.03, -5730.06957,12.13836576, -0.00524333812,1.976302344e-06, -3.30657353e-10,1.937902763e-14}, bhigh = {213534.8051, -61.2875839}, R = 2751.77566166453);
          constant IdealGases.Common.DataRecord HDO(name = "HDO", MM = 0.019021442, Hf =  -12894946.5030044, H0 = 521817.588803204, Tlimit = 1000, alow = { -27377.95356,431.05039,1.558479899,0.00576496915, -4.85512936e-06,3.38299017e-09, -1.040863879e-12}, blow = { -32732.2645,14.87751891}, ahigh = {1711376.798, -5322.8723,9.12435152, -0.000340066415,4.15234352e-08, -4.78076857e-14, -1.46803517e-16}, bhigh = {3245.22468, -37.7460068}, R = 437.110498772911);
          constant IdealGases.Common.DataRecord HDO2(name = "HDO2", MM = 0.035020842, Hf =  -4004519.22315289, H0 = 323663.405922679, Tlimit = 1000, alow = { -32164.7665,749.233446, -1.957676212,0.02412802791, -2.915946684e-05,1.930016324e-08, -5.22698631e-12}, blow = { -21510.59175,36.7240674}, ahigh = {1313180.619, -5175.63712,12.16490975, -0.000613669948,1.227560176e-07, -1.079624939e-11,3.87401372e-16}, bhigh = {13032.0967, -50.8527687}, R = 237.414965636748);
          constant IdealGases.Common.DataRecord HF(name = "HF", MM = 0.0200063432, Hf =  -13660667.3827329, H0 = 429818.828660302, Tlimit = 1000, alow = { -3192.09897,59.8680772,3.055113902,0.001684673783, -3.28739483e-06,3.095923617e-09, -9.76469161e-13}, blow = { -34184.4316,3.29490412}, ahigh = {725708.904, -1484.797741,3.85552747,0.000713898985, -2.106757333e-07,3.050092453e-11, -1.639495583e-15}, bhigh = { -23554.5666, -3.20385683}, R = 415.591790907596);
          constant IdealGases.Common.DataRecord HI(name = "HI", MM = 0.12791241, Hf = 206070.701036749, H0 = 67675.2161889531, Tlimit = 1000, alow = {18728.8173, -343.178884,5.95671243, -0.0085434396,1.454780274e-05, -1.049104164e-08,2.839734003e-12}, blow = {3682.95072, -8.14975609}, ahigh = {472492.145, -1923.465741,5.75804897, -0.000406626638,9.47433205e-08, -1.033534431e-11,4.61161479e-16}, bhigh = {13948.57037, -11.82487652}, R = 65.0012926814529);
          constant IdealGases.Common.DataRecord HNC(name = "HNC", MM = 0.02702534, Hf = 7192439.42906916, H0 = 370049.886513916, Tlimit = 1000, alow = {48706.2064, -989.145625,9.72215389, -0.01113593916,1.668862707e-05, -1.113940941e-08,2.893600868e-12}, blow = {26646.79758, -31.04826344}, ahigh = {1198791.66, -3918.94186,9.11802009, -0.00034177611,3.31480968e-08, -5.70157474e-13, -7.78945539e-17}, bhigh = {46588.103, -34.68575882}, R = 307.654667804364);
          constant IdealGases.Common.DataRecord HNCO(name = "HNCO", MM = 0.04302474, Hf =  -2743921.96210831, H0 = 254879.541398739, Tlimit = 1000, alow = {75424.6008, -955.093777,6.72570587,0.0047056875, -4.95947551e-06,3.69425512e-09, -1.164859121e-12}, blow = { -10681.49742, -13.65584762}, ahigh = {1253216.926, -5021.09154,12.47789314, -0.000689165525,1.097738448e-07, -9.3064038e-12,3.24260695e-16}, bhigh = {14531.55559, -53.06419819}, R = 193.248628579743);
          constant IdealGases.Common.DataRecord HNO(name = "HNO", MM = 0.03101404, Hf = 3289888.22481689, H0 = 320560.623511158, Tlimit = 1000, alow = { -68547.6486,955.16272, -0.600072021,0.00799517675, -6.54707916e-07, -3.6705134e-09,1.783392519e-12}, blow = {6435.35126,30.48166179}, ahigh = { -5795614.98,19454.57427, -21.52568374,0.01797428992, -4.97604067e-06,6.39792417e-10, -3.142619368e-14}, bhigh = { -110419.2372,181.8650338}, R = 268.087356564962);
          constant IdealGases.Common.DataRecord HNO2(name = "HNO2", MM = 0.04701344, Hf =  -1668712.64897868, H0 = 246680.204639354, Tlimit = 1000, alow = {8591.98506,120.3644046,0.941297912,0.01942891839, -2.253174194e-05,1.384587594e-08, -3.47355046e-12}, blow = { -11063.37202,20.73967331}, ahigh = {878790.413, -3990.45503,11.87349269, -0.000488190061,7.13363679e-08, -5.37630334e-12,1.581778986e-16}, bhigh = {12463.43241, -46.08874688}, R = 176.853087117216);
          constant IdealGases.Common.DataRecord HNO3(name = "HNO3", MM = 0.06301284, Hf =  -2125167.96576698, H0 = 188475.71383864, Tlimit = 1000, alow = {9202.86901,109.3774496, -0.452104245,0.02984914503, -3.1906355e-05,1.720931528e-08, -3.78264983e-12}, blow = { -17640.48507,27.46644879}, ahigh = { -94978.0964, -2733.024468,14.49426995, -0.000782186805,1.702693665e-07, -1.930543961e-11,8.87045512e-16}, bhigh = { -4882.51778, -59.28392985}, R = 131.948853598727);
          constant IdealGases.Common.DataRecord HOCL(name = "HOCL", MM = 0.05246034, Hf =  -1443757.93218267, H0 = 194902.015503521, Tlimit = 1000, alow = { -9739.30743,354.756952,0.1539514254,0.01617051795, -2.179693631e-05,1.509103049e-08, -4.12538351e-12}, blow = { -11763.23791,24.73257759}, ahigh = {853045.781, -2847.760552,7.94832904, -0.0001048782013, -1.482405043e-08,4.59167827e-12, -3.060073987e-16}, bhigh = {7250.96495, -22.4983169}, R = 158.490623583454);
          constant IdealGases.Common.DataRecord HOF(name = "HOF", MM = 0.0360057432, Hf =  -2691187.91582116, H0 = 280189.605973749, Tlimit = 1000, alow = { -36968.883,780.900666, -2.077685317,0.02038690173, -2.586919999e-05,1.713797538e-08, -4.55128187e-12}, blow = { -16317.20064,36.450525}, ahigh = {881201.823, -3120.013169,8.22371007, -0.0002298036315,1.459115709e-08,1.095883303e-12, -1.404445608e-16}, bhigh = {6300.54671, -25.90150427}, R = 230.920716003996);
          constant IdealGases.Common.DataRecord HO2(name = "HO2", MM = 0.03300674, Hf = 364168.045677943, H0 = 0, Tlimit = 1000, alow = { -75988.8254,1329.383918, -4.67738824,0.02508308202, -3.006551588e-05,1.895600056e-08, -4.82856739e-12}, blow = {0, -5873.35096}, ahigh = {10002.162, -1810669.724,4963.19203, -1.039498992,0.00456014853, -1.061859447e-06,1.144567878e-10}, bhigh = { -4.76306416e-15,0}, R = 251.902247843925);
          constant IdealGases.Common.DataRecord HO2minus(name = "HO2minus", MM = 0.0330072886, Hf =  -2966710.81307781, H0 = 0, Tlimit = 1000, alow = {110383.9835, -1047.963653,6.36001399,0.002942520461, -6.28414134e-06,5.43825424e-09, -1.64730582e-12}, blow = {0, -7417.74159}, ahigh = {10245.32,793330.6, -2503.312417,7.54896233,8.30839015e-05, -5.96973091e-08,9.955377e-12}, bhigh = { -5.60647728e-16,0}, R = 251.898061084605);
          constant IdealGases.Common.DataRecord HPO(name = "HPO", MM = 0.047981101, Hf =  -1185232.47309394, H0 = 209777.241251717, Tlimit = 1000, alow = { -38163.1412,792.764187, -1.889940652,0.01800907425, -1.857631793e-05,1.018768867e-08, -2.355583619e-12}, blow = { -11576.4124,36.9292591}, ahigh = {384245.945, -2434.951707,8.58217392, -0.000405844857, -1.66948874e-08,2.253640566e-11, -1.943063011e-15}, bhigh = {5761.63212, -26.49097544}, R = 173.286394574397);
          constant IdealGases.Common.DataRecord HSO3F(name = "HSO3F", MM = 0.1000695432, Hf =  -7525966.20227202, H0 = 150037.399191406, Tlimit = 1000, alow = {6896.24213, -93.0581075,1.570331788,0.0378082075, -4.87272078e-05,3.160796011e-08, -8.18978293e-12}, blow = { -91802.4082,17.16315977}, ahigh = {554404.708, -3974.42432,17.78869264, -0.000440932203,5.94441139e-08, -3.92701581e-12,8.891284e-17}, bhigh = { -71534.3827, -76.13127849}, R = 83.0869386840571);
          constant IdealGases.Common.DataRecord H2(name = "H2", MM = 0.00201588, Hf = 0, H0 = 4200697.46215052, Tlimit = 1000, alow = {40783.2321, -800.918604,8.21470201, -0.01269714457,1.753605076e-05, -1.20286027e-08,3.36809349e-12}, blow = {2682.484665, -30.43788844}, ahigh = {560812.801, -837.150474,2.975364532,0.001252249124, -3.74071619e-07,5.9366252e-11, -3.6069941e-15}, bhigh = {5339.82441, -2.202774769}, R = 4124.48756870449);
          constant IdealGases.Common.DataRecord H2plus(name = "H2plus", MM = 0.0020153314, Hf = 741650941.378673, H0 = 4258904.01945804, Tlimit = 1000, alow = { -31208.8606,230.4622909,3.33556442, -0.002419056763,7.00602234e-06, -5.61001066e-09,1.564169746e-12}, blow = {177410.4638, -0.827852376}, ahigh = {1672225.964, -6595.18499,12.79321925, -0.00550934526,2.030669412e-06, -3.35102748e-10,1.946089104e-14}, bhigh = {218999.9548, -67.9271078}, R = 4125.61030905389);
          constant IdealGases.Common.DataRecord H2minus(name = "H2minus", MM = 0.0020164286, Hf = 116626178.58128, H0 = 4275178.40205202, Tlimit = 1000, alow = { -97535.6567,1221.166236, -2.264588838,0.01237202227, -1.12710002e-05,5.36723995e-09, -1.04942016e-12}, blow = {21213.99948,30.50556136}, ahigh = {95992.7562, -914.468288,5.14941881, -0.0001016559478,5.44691956e-08, -6.15517445e-12,2.822451181e-16}, bhigh = {32341.0518, -14.4078098}, R = 4123.36543927219);
          constant IdealGases.Common.DataRecord HBOH(name = "HBOH", MM = 0.02882628, Hf =  -1690275.81776074, H0 = 379533.39799655, Tlimit = 1000, alow = { -61596.4419,1223.483035, -5.46037051,0.0355538868, -4.67502014e-05,3.23600885e-08, -8.98605332e-12}, blow = { -12636.59051,54.2235229}, ahigh = {1534611.049, -5753.66643,12.70893665, -0.000705962825,1.027679375e-07, -7.67092674e-12,2.211854768e-16}, bhigh = {27793.70419, -56.0886008}, R = 288.433748648802);
          constant IdealGases.Common.DataRecord HCOOH(name = "HCOOH", MM = 0.04602538, Hf =  -8225244.41949203, H0 = 237426.589416535, Tlimit = 1000, alow = { -29062.79097,765.837888, -3.32841413,0.02817542991, -2.370050804e-05,1.166063663e-08, -2.79137317e-12}, blow = { -50064.4347,43.8709423}, ahigh = {487233.645, -7632.23808,21.32788153, -0.00440254654,1.102001695e-06, -1.364343517e-10,6.64842975e-15}, bhigh = { -5781.43191, -111.1790688}, R = 180.64971978504);
          constant IdealGases.Common.DataRecord H2F2(name = "H2F2", MM = 0.0400126864, Hf =  -14243576.9571323, H0 = 346625.814156782, Tlimit = 1000, alow = {52592.1471, -991.354489,10.43577115, -0.002407796033, -6.37695616e-07,2.7357849e-09, -1.10434859e-12}, blow = { -65724.6083, -30.38432132}, ahigh = {1464995.601, -3335.07492,9.18748704,0.001051127249, -3.27860557e-07,4.45604623e-11, -2.281370136e-15}, bhigh = { -48254.4209, -26.39128168}, R = 207.795895453798);
          constant IdealGases.Common.DataRecord H2O(name = "H2O", MM = 0.01801528, Hf =  -13423382.8172529, H0 = 549760.647628014, Tlimit = 1000, alow = { -39479.6083,575.573102,0.931782653,0.00722271286, -7.34255737e-06,4.95504349e-09, -1.336933246e-12}, blow = { -33039.7431,17.24205775}, ahigh = {1034972.096, -2412.698562,4.64611078,0.002291998307, -6.83683048e-07,9.42646893e-11, -4.82238053e-15}, bhigh = { -13842.86509, -7.97814851}, R = 461.523329085088);
          constant IdealGases.Common.DataRecord H2Oplus(name = "H2Oplus", MM = 0.0180147314, Hf = 54488837.1746692, H0 = 551463.453959686, Tlimit = 1000, alow = { -1753.89272,224.9850054,1.989400675,0.00611789516, -7.09543664e-06,5.54765947e-09, -1.704344789e-12}, blow = {115958.5952,11.35409642}, ahigh = {622871.426, -2864.257487,7.71756556, -0.000902780167,6.17743686e-07, -1.201457479e-10,7.40770994e-15}, bhigh = {134208.6651, -26.3661792}, R = 461.537383788026);
          constant IdealGases.Common.DataRecord H2O2(name = "H2O2", MM = 0.03401468, Hf =  -3994745.79799075, H0 = 328059.384947911, Tlimit = 1000, alow = { -92795.3358,1564.748385, -5.97646014,0.0327074452, -3.93219326e-05,2.509255235e-08, -6.46504529e-12}, blow = { -24940.04728,58.7717418}, ahigh = {1489428.027, -5170.82178,11.2820497, -8.04239779e-05, -1.818383769e-08,6.94726559e-12, -4.8278319e-16}, bhigh = {14182.51038, -46.5085566}, R = 244.437754522459);
          constant IdealGases.Common.DataRecord H2S(name = "H2S", MM = 0.03408088, Hf =  -604444.48617524, H0 = 292199.643905909, Tlimit = 1000, alow = {9543.80881, -68.7517508,4.05492196, -0.0003014557336,3.76849775e-06, -2.239358925e-09,3.086859108e-13}, blow = { -3278.45728,1.415194691}, ahigh = {1430040.22, -5284.02865,10.16182124, -0.000970384996,2.154003405e-07, -2.1696957e-11,9.31816307e-16}, bhigh = {29086.96214, -43.49160391}, R = 243.962949313515);
          constant IdealGases.Common.DataRecord H2SO4(name = "H2SO4", MM = 0.09807848, Hf =  -7470871.50004772, H0 = 168320.196234689, Tlimit = 1000, alow = { -41291.5005,668.158989, -2.632753507,0.0541538248, -7.06750223e-05,4.68461142e-08, -1.236791238e-11}, blow = { -93156.6012,39.61096201}, ahigh = {1437877.914, -6614.90253,21.57662058, -0.000480625597,3.010775121e-08,2.334842469e-12, -2.946330375e-16}, bhigh = { -52590.9295, -102.3603724}, R = 84.7736628871084);
          constant IdealGases.Common.DataRecord H2BOH(name = "H2BOH", MM = 0.02983422, Hf =  -9708110.28409658, H0 = 356044.367843369, Tlimit = 1000, alow = { -86867.6678,1820.335089, -10.32373881,0.0492280106, -5.97861991e-05,3.94606273e-08, -1.068442257e-11}, blow = { -44152.3815,79.8617729}, ahigh = {2294795.193, -9382.92356,17.98228329, -0.001506116214,2.635641095e-07, -2.483163637e-11,9.73675912e-16}, bhigh = {20397.38953, -94.4820121}, R = 278.689102647899);
          constant IdealGases.Common.DataRecord HB_OH_2(name = "HB_OH_2", MM = 0.04583362, Hf =  -14060393.30954, H0 = 265253.584595762, Tlimit = 1000, alow = { -20670.44022,953.445916, -8.08895652,0.0597242529, -8.07133632e-05,5.61211835e-08, -1.553607566e-11}, blow = { -82642.7081,65.3957164}, ahigh = {2122674.609, -8615.94894,19.6617952, -0.00081484115,8.95508204e-08, -3.34483051e-12, -6.91957814e-17}, bhigh = { -27887.90377, -99.8355961}, R = 181.405527209066);
          constant IdealGases.Common.DataRecord H3BO3(name = "H3BO3", MM = 0.06183302, Hf =  -16243108.3262632, H0 = 219299.6234051, Tlimit = 1000, alow = {25689.01843,113.8029495, -4.04509658,0.0592452168, -8.14802841e-05,5.65859329e-08, -1.54927705e-11}, blow = { -122170.205,41.3222014}, ahigh = {2297369.132, -8933.57179,21.93496552, -0.000309478349, -5.06405299e-08,1.482296684e-11, -9.7644209e-16}, bhigh = { -69702.9847, -112.2292829}, R = 134.466535841206);
          constant IdealGases.Common.DataRecord H3B3O3(name = "H3B3O3", MM = 0.08345502, Hf =  -14424069.2411313, H0 = 186966.835548059, Tlimit = 1000, alow = { -198528.4188,4104.13209, -28.27215617,0.1269639253, -0.0001558505781,9.96630407e-08, -2.60367765e-11}, blow = { -164849.4375,176.3917407}, ahigh = {1286220.713, -10932.58784,32.1351867, -0.002595402269,5.35376647e-07, -5.83356738e-11,2.600757913e-15}, bhigh = { -87528.5689, -177.0847815}, R = 99.6281829421406);
          constant IdealGases.Common.DataRecord H3B3O6(name = "H3B3O6", MM = 0.13145322, Hf =  -17220480.4948863, H0 = 180268.334240881, Tlimit = 1000, alow = { -20670.31503,708.339154, -7.12382095,0.1025049979, -0.0001295004602,8.32273759e-08, -2.145746724e-11}, blow = { -277804.9216,60.6359763}, ahigh = {1952433.352, -11549.32896,38.9267611, -0.00111385228,1.280317726e-07, -5.72846482e-12, -2.214529572e-17}, bhigh = { -212092.7136, -207.5566801}, R = 63.2504247518623);
          constant IdealGases.Common.DataRecord H3F3(name = "H3F3", MM = 0.0600190296, Hf =  -14723276.8655093, H0 = 254303.04524617, Tlimit = 1000, alow = {98885.6265, -1380.21288,8.98045847,0.01871609057, -3.127862508e-05,2.589436165e-08, -8.04037227e-12}, blow = { -101366.1491, -25.85302557}, ahigh = {2515790.373, -8789.42332,20.23933445, -0.001131704646,1.692779921e-07, -1.308662105e-11,3.978322e-16}, bhigh = { -54717.1365, -99.0778764}, R = 138.530596969199);
          constant IdealGases.Common.DataRecord H3Oplus(name = "H3Oplus", MM = 0.0190226714, Hf = 31436173.5754948, H0 = 528129.13542732, Tlimit = 1000, alow = { -64476.4015,1181.817922, -3.80189306,0.02220628313, -2.445343237e-05,1.573297747e-08, -4.15883641e-12}, blow = {65306.1332,42.8272313}, ahigh = {2955126.2, -9185.66941,13.41398696, -0.000559033921,1.138387119e-08,7.25992721e-12, -6.13373436e-16}, bhigh = {129053.4257, -70.2182818}, R = 437.082249131423);
          constant IdealGases.Common.DataRecord H4F4(name = "H4F4", MM = 0.0800253728, Hf =  -14831950.0987067, H0 = 270590.179618632, Tlimit = 1000, alow = {131510.9024, -1840.36156,12.64093152,0.0249540736, -4.17039461e-05,3.45252384e-08, -1.07203436e-11}, blow = { -136400.3454, -41.2291353}, ahigh = {3354037.99, -11719.2204,27.65243565, -0.001508934621,2.257027572e-07, -1.744867255e-11,5.30435174e-16}, bhigh = { -74202.1316, -138.8603332}, R = 103.897947726899);
          constant IdealGases.Common.DataRecord H5F5(name = "H5F5", MM = 0.100031716, Hf =  -14897154.0386251, H0 = 280362.46024211, Tlimit = 1000, alow = {164136.1897, -2300.510396,16.30140545,0.03119205422, -5.21292635e-05,4.31561124e-08, -1.340031413e-11}, blow = { -171434.5408, -56.8215738}, ahigh = {4192290.73, -14649.03043,35.0655492, -0.001886170471,2.821289954e-07, -2.181090984e-11,6.63047416e-16}, bhigh = { -93687.042, -178.8592044}, R = 83.1183581815191);
          constant IdealGases.Common.DataRecord H6F6(name = "H6F6", MM = 0.1200380592, Hf =  -15041438.8822441, H0 = 286030.057706898, Tlimit = 1000, alow = {195688.6618, -2753.90988,19.75625055,0.0380771701, -6.34121888e-05,5.21923329e-08, -1.612185649e-11}, blow = { -207926.941, -71.1360798}, ahigh = {5024520.32, -17428.17868,42.148705, -0.002084564791,2.930051519e-07, -2.052079159e-11,5.2069373e-16}, bhigh = { -115449.7653, -216.3947001}, R = 69.2652984845993);
          constant IdealGases.Common.DataRecord H7F7(name = "H7F7", MM = 0.1400444024, Hf =  -14993094.604401, H0 = 291530.780954655, Tlimit = 1000, alow = {229386.7477, -3220.80786,23.62235212,0.0436680188, -7.29799034e-05,6.04178642e-08, -1.876025627e-11}, blow = { -241863.7479, -88.3881226}, ahigh = {5868789.83, -20508.63473,49.8917611, -0.002640634895,3.94979633e-07, -3.053515088e-11,9.28260178e-16}, bhigh = { -133017.7809, -259.2385142}, R = 59.3702558439423);
          constant IdealGases.Common.DataRecord He(name = "He", MM = 0.004002602, Hf = 0, H0 = 1548349.7984561, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = { -745.375,0.928723974}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = { -745.375,0.928723974}, R = 2077.26673798694);
          constant IdealGases.Common.DataRecord Heplus(name = "Heplus", MM = 0.0040020534, Hf = 594325271.371941, H0 = 1548562.04567385, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {285323.3739,1.621665557}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = {285323.3739,1.621665557}, R = 2077.55148894315);
          constant IdealGases.Common.DataRecord Hg(name = "Hg", MM = 0.20059, Hf = 305997.307941572, H0 = 30895.9968094122, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {6636.90008,6.80020154}, ahigh = {51465.7351, -168.1269855,2.718343098, -0.0001445026192,5.15897766e-08, -9.47248501e-12,7.034797406e-16}, bhigh = {7688.68493,5.27123609}, R = 41.4500822573409);
          constant IdealGases.Common.DataRecord Hgplus(name = "Hgplus", MM = 0.2005894514, Hf = 5357425.92892898, H0 = 30896.0813080922, Tlimit = 1000, alow = {0.000409834494, -4.34358162e-06,2.500000019, -4.38903681e-11,5.5639692e-14, -3.63822826e-17,9.607881995e-21}, blow = {128503.7483,7.4933445}, ahigh = { -12299.84728,27.32269908,2.48418216, -4.42679761e-06,7.48968586e-09, -2.549887287e-12,2.819873366e-16}, bhigh = {128318.8257,7.62524457}, R = 41.4501956208052);
          constant IdealGases.Common.DataRecord HgBr2(name = "HgBr2", MM = 0.360398, Hf =  -253363.270606385, H0 = 43447.477510974, Tlimit = 1000, alow = { -1991.826537, -190.2186083,8.24640126, -0.001607089392,1.947654549e-06, -1.242873546e-09,3.24181831e-13}, blow = { -12307.23096, -8.7166593}, ahigh = { -22436.4929, -3.31265227,7.50276517, -1.202360213e-06,2.830526999e-10, -3.4076867e-14,1.640497921e-18}, bhigh = { -13274.83366, -4.3685556}, R = 23.0702501123758);
          constant IdealGases.Common.DataRecord I(name = "I", MM = 0.12690447, Hf = 841262.723054594, H0 = 48835.37987275, Tlimit = 1000, alow = {169.8199675, -2.716437233,2.517385557, -5.73069207e-05,1.031716184e-07, -9.67064193e-11,3.706471651e-14}, blow = {12107.5009,7.40582313}, ahigh = { -778586.057,2303.279568,0.002886686091,0.001180878463, -2.264074866e-07,1.963511339e-11, -6.243525941e-16}, bhigh = { -2616.792742,25.58922997}, R = 65.5175660873096);
          constant IdealGases.Common.DataRecord Iplus(name = "Iplus", MM = 0.1269039214, Hf = 8836220.47001614, H0 = 48835.590985922, Tlimit = 1000, alow = { -801.496901,8.13905261,2.47017476,4.16139382e-05,7.13339786e-09, -7.10595014e-11,4.873766102e-14}, blow = {134079.4213,7.90342009}, ahigh = { -778838.533,2404.962651, -0.1791751142,0.001227311979, -1.80149403e-07,9.92398396e-12, -9.775286439e-17}, bhigh = {118853.1631,27.10544347}, R = 65.5178493168297);
          constant IdealGases.Common.DataRecord Iminus(name = "Iminus", MM = 0.1269050186, Hf =  -1533395.40190572, H0 = 48835.1687614031, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = { -24149.70936,6.11346538}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = { -24149.70936,6.11346538}, R = 65.5172828602383);
          constant IdealGases.Common.DataRecord IF5(name = "IF5", MM = 0.221896486, Hf =  -3790055.51264115, H0 = 90331.0474236171, Tlimit = 1000, alow = {202618.453, -3598.18317,25.30349738, -0.01348602458,1.105687229e-05, -4.70793951e-09,7.86441147e-13}, blow = { -87001.4725, -111.2719941}, ahigh = { -362050.626, -163.9236093,16.12272743, -4.91443155e-05,1.086015052e-08, -1.244672691e-12,5.76359362e-17}, bhigh = { -106164.2293, -53.9877336}, R = 37.4700480835916);
          constant IdealGases.Common.DataRecord IF7(name = "IF7", MM = 0.2598932924, Hf =  -3699595.28820837, H0 = 92820.1446725756, Tlimit = 1000, alow = {299985.3564, -5501.55413,36.2713398, -0.02095765818,1.76197268e-05, -7.82914233e-09,1.405939372e-12}, blow = { -93313.0766, -175.421615}, ahigh = { -561442.264, -264.6288326,22.19879891, -7.98422147e-05,1.768835964e-08, -2.031523755e-12,9.42376739e-17}, bhigh = { -122524.5322, -87.7401073}, R = 31.9918683672807);
          constant IdealGases.Common.DataRecord I2(name = "I2", MM = 0.25380894, Hf = 245933.023478212, H0 = 39857.2327672934, Tlimit = 1000, alow = { -5087.96877, -12.4958521,4.50421909,0.0001370962533, -1.390523014e-07,1.174813853e-10, -2.337541043e-14}, blow = {6213.46981,5.58383694}, ahigh = { -5632594.16,17939.6156, -17.23055169,0.0124421408, -3.33276858e-06,4.12547794e-10, -1.960461713e-14}, bhigh = { -106850.5292,160.0531883}, R = 32.7587830436548);
          constant IdealGases.Common.DataRecord In(name = "In", MM = 0.114818, Hf = 2096361.19772161, H0 = 53986.7355292724, Tlimit = 1000, alow = {51495.1805, -917.799902,8.93868795, -0.02224112602,3.82388194e-05, -2.890116948e-08,8.047481221e-12}, blow = {32390.3162, -27.64567028}, ahigh = { -1683608.899,2210.473186,3.47221937, -0.001082267422,3.47969998e-07, -5.15809241e-11,3.183043089e-15}, bhigh = {10959.65206,2.557189088}, R = 72.4143601177516);
          constant IdealGases.Common.DataRecord Inplus(name = "Inplus", MM = 0.1148174514, Hf = 60935.2055344437, H0 = 53976.3592070116, Tlimit = 1000, alow = {0.2150488908, -0.002523854672,2.500011877, -2.869063919e-08,3.7538704e-11, -2.525942419e-14,6.84293266e-18}, blow = {96.1093017,5.9632544}, ahigh = {45369.9514, -144.5142103,2.681812441, -0.000115748006,3.94771481e-08, -6.87924362e-12,4.81920492e-16}, bhigh = {1004.26144,4.68375918}, R = 72.4147061149626);
          constant IdealGases.Common.DataRecord InBr(name = "InBr", MM = 0.194722, Hf =  -277913.533139553, H0 = 51787.2710839042, Tlimit = 1000, alow = { -2540.851435, -52.9470471,4.71743809, -0.000434466949,6.51351364e-07, -4.40399515e-10,1.234046209e-13}, blow = { -7607.59428,4.258881}, ahigh = {794837.442, -2318.183711,7.03288958, -0.001245920568,3.27669335e-07, -3.003771106e-11,5.85319277e-16}, bhigh = {6957.62729, -12.7350266}, R = 42.6991916681217);
          constant IdealGases.Common.DataRecord InBr2(name = "InBr2", MM = 0.274626, Hf =  -545210.792131845, H0 = 52523.5811612884, Tlimit = 1000, alow = { -66.198789, -236.589731,7.92654516, -0.001994547959,2.420414153e-06, -1.548924922e-09,4.05814839e-13}, blow = { -18953.55588, -5.33886754}, ahigh = { -321716.203,479.26821,7.10264948, -0.000551401978,3.157862421e-07, -5.51125451e-11,3.17216139e-15}, bhigh = { -23709.23659,0.1193263347}, R = 30.275618477493);
          constant IdealGases.Common.DataRecord InBr3(name = "InBr3", MM = 0.35453, Hf =  -723739.223761036, H0 = 56166.5670042028, Tlimit = 1000, alow = { -4990.1153, -280.5930452,11.1035606, -0.002379995808,2.887825281e-06, -1.844500707e-09,4.81439587e-13}, blow = { -32505.0491, -18.71748987}, ahigh = { -35057.831, -4.85872132,10.00406131, -1.767620958e-06,4.16401606e-10, -5.01552887e-14,2.415414123e-18}, bhigh = { -33931.7924, -12.29069868}, R = 23.452097142696);
          constant IdealGases.Common.DataRecord InCL(name = "InCL", MM = 0.150271, Hf =  -480118.472626122, H0 = 64896.8663281671, Tlimit = 1000, alow = {6430.71303, -217.8267726,5.4188309, -0.0020480051,2.702295936e-06, -1.795319798e-09,4.84746909e-13}, blow = { -8959.84107, -1.202999253}, ahigh = { -274575.1636,894.098691,3.26811839,0.000918027778, -3.083262431e-07,5.74545881e-11, -3.64527575e-15}, bhigh = { -15616.0343,12.66880374}, R = 55.3298507363364);
          constant IdealGases.Common.DataRecord InCL2(name = "InCL2", MM = 0.185724, Hf =  -1084853.95533157, H0 = 75145.829295083, Tlimit = 1000, alow = {9935.16487, -417.210186,8.58849491, -0.00334838561,3.99786664e-06, -2.525301853e-09,6.54449033e-13}, blow = { -24264.80269, -11.64521725}, ahigh = { -332871.984,475.769161,7.10544369, -0.000552578696,3.160567738e-07, -5.51445368e-11,3.17368038e-15}, bhigh = { -29950.95917, -2.294555511}, R = 44.7678921410265);
          constant IdealGases.Common.DataRecord InCL3(name = "InCL3", MM = 0.221177, Hf =  -1671481.33395425, H0 = 82769.7861893416, Tlimit = 1000, alow = {26147.43044, -841.460456,13.1178479, -0.00643528247,7.55487475e-06, -4.70507856e-09,1.204213663e-12}, blow = { -43264.6958, -35.0008094}, ahigh = { -71140.4363, -16.28425068,10.01309622, -5.54452888e-06,1.279877732e-09, -1.518261505e-13,7.22656277e-18}, bhigh = { -47589.7417, -16.69761011}, R = 37.5919376788726);
          constant IdealGases.Common.DataRecord InF(name = "InF", MM = 0.1338164032, Hf =  -1445413.15096414, H0 = 68878.7830160421, Tlimit = 1000, alow = {31901.2332, -529.525426,6.17288626, -0.002986035832,3.23377803e-06, -1.856975575e-09,4.46502497e-13}, blow = { -21871.76282, -7.59121662}, ahigh = { -468900.12,1321.019051,2.881329353,0.001041591078, -3.090754867e-07,4.92664656e-11, -2.699156425e-15}, bhigh = { -33083.6981,13.96688303}, R = 62.1334290951859);
          constant IdealGases.Common.DataRecord InF2(name = "InF2", MM = 0.1528148064, Hf =  -2991770.46236797, H0 = 82379.2948901056, Tlimit = 1000, alow = {75224.8021, -1088.616881,9.48170521, -0.002893229367,1.516224649e-06, -7.98951781e-11, -1.466781026e-13}, blow = { -51243.3615, -21.4666609}, ahigh = {431896.034, -1688.47242,8.92586301, -0.001049188835,2.72393419e-07, -2.882959902e-11,1.073493779e-15}, bhigh = { -46736.9194, -19.32464363}, R = 54.4088115273102);
          constant IdealGases.Common.DataRecord InF3(name = "InF3", MM = 0.1718132096, Hf =  -5023359.77547561, H0 = 94989.4017927711, Tlimit = 1000, alow = {107160.8336, -1660.071901,13.95972813, -0.00501869326,3.22636753e-06, -7.82338597e-10, -3.7269998e-14}, blow = { -97952.1499, -46.0635347}, ahigh = { -165685.5506, -79.6588929,10.05913347, -2.350816639e-05,5.16351293e-09, -5.88804881e-13,2.715090697e-17}, bhigh = { -106868.2627, -21.32968218}, R = 48.3925073011383);
          constant IdealGases.Common.DataRecord InH(name = "InH", MM = 0.11582594, Hf = 1856379.52949054, H0 = 74975.4674989039, Tlimit = 1000, alow = { -51528.2872,781.379259, -0.843338534,0.01044471274, -9.76107482e-06,4.5406352e-09, -8.1699069e-13}, blow = {21100.48908,29.39346222}, ahigh = {779740.666, -3646.88208,9.54296735, -0.00348128172,1.304805603e-06, -2.182950701e-10,1.274009748e-14}, bhigh = {46432.9546, -36.5932923}, R = 71.7841961826513);
          constant IdealGases.Common.DataRecord InI(name = "InI", MM = 0.24172247, Hf = 109287.001742122, H0 = 42491.3910568596, Tlimit = 1000, alow = { -468.634874, -56.8553099,4.78184969, -0.000661936831,1.027666742e-06, -7.27059207e-10,2.077991102e-13}, blow = {2095.570254,4.88551513}, ahigh = {1529221.772, -4690.37368,10.04593462, -0.00314043411,9.46083439e-07, -1.253862848e-10,5.94980564e-15}, bhigh = {31524.78143, -32.9769749}, R = 34.3967691543116);
          constant IdealGases.Common.DataRecord InI2(name = "InI2", MM = 0.36862694, Hf =  -107047.637375608, H0 = 40454.8755986201, Tlimit = 1000, alow = { -4513.8906, -118.372687,7.47311008, -0.00103447409,1.271465534e-06, -8.2278203e-10,2.178307019e-13}, blow = { -6278.53604, -0.66672975}, ahigh = { -313067.1665,481.38107,7.10091418, -0.000550655992,3.156120735e-07, -5.50917054e-11,3.17116283e-15}, bhigh = { -10430.1275,2.141662307}, R = 22.5552478611574);
          constant IdealGases.Common.DataRecord InI3(name = "InI3", MM = 0.49553141, Hf =  -212773.422374981, H0 = 42080.2507756269, Tlimit = 1000, alow = { -8728.39717, -126.5914859,10.5065912, -0.001106109314,1.354335766e-06, -8.70945346e-10,2.28518635e-13}, blow = { -15082.69063, -12.11411745}, ahigh = { -21967.42567, -2.151167521,10.001822, -8.0022147e-07,1.897343164e-10, -2.29625246e-14,1.109838196e-18}, bhigh = { -15724.06335, -9.17088869}, R = 16.7789000499484);
          constant IdealGases.Common.DataRecord InO(name = "InO", MM = 0.1308174, Hf = 1116006.80796286, H0 = 69120.0788274343, Tlimit = 1000, alow = { -115472.7169,1753.668703, -6.96503381,0.02961698958, -3.123332334e-05,1.43724472e-08, -2.235519898e-12}, blow = {8188.67959,66.0862126}, ahigh = { -805213.095,1693.064203,4.76086358, -0.00073407184,3.22959776e-07, -4.58015038e-11,1.954611725e-15}, bhigh = {4005.51981,3.50074246}, R = 63.5578447515392);
          constant IdealGases.Common.DataRecord InOH(name = "InOH", MM = 0.13182534, Hf =  -944032.035115555, H0 = 81447.4591910781, Tlimit = 1000, alow = {61234.6302, -1195.863129,10.41427981, -0.0068700025,6.21412878e-06, -2.408203181e-09,3.17764178e-13}, blow = { -10798.5217, -31.8989084}, ahigh = {852512.893, -2350.317172,7.98451514,9.91581562e-05, -6.21238545e-08,1.016169575e-11, -5.68255764e-16}, bhigh = { -1536.21361, -20.43107643}, R = 63.0718798070234);
          constant IdealGases.Common.DataRecord In2Br2(name = "In2Br2", MM = 0.389444, Hf =  -504065.321329896, H0 = 55039.8593892832, Tlimit = 1000, alow = { -9221.1883, -36.1667703,10.14728771, -0.00032557752,4.02232748e-07, -2.604080237e-10,6.86768955e-14}, blow = { -26448.83479, -10.00360073}, ahigh = { -12907.81804, -0.607540124,10.00052163, -2.312058331e-07,5.51731572e-11, -6.70862019e-15,3.25384165e-19}, bhigh = { -26631.36878, -9.14994947}, R = 21.3495958340609);
          constant IdealGases.Common.DataRecord In2Br4(name = "In2Br4", MM = 0.549252, Hf =  -794732.987772461, H0 = 57893.4933327507, Tlimit = 1000, alow = { -14255.55379, -286.6009341,17.13866292, -0.002473506655,3.017273071e-06, -1.93490043e-09,5.06587425e-13}, blow = { -55937.5306, -38.3710598}, ahigh = { -44534.6371, -4.91152398,16.00413764, -1.810579975e-06,4.28171574e-10, -5.17199802e-14,2.496142837e-18}, bhigh = { -57391.7541, -31.7490639}, R = 15.1378092387465);
          constant IdealGases.Common.DataRecord In2Br6(name = "In2Br6", MM = 0.70906, Hf =  -886643.485741686, H0 = 60780.0863114546, Tlimit = 1000, alow = { -17589.40752, -485.903448,23.91654505, -0.00414206149,5.03384932e-06, -3.21910365e-09,8.41022007e-13}, blow = { -79888.4446, -64.3787188}, ahigh = { -69450.3948, -8.40236817,22.00703884, -3.068251799e-06,7.23596573e-10, -8.72287058e-14,4.20346991e-18}, bhigh = { -82357.5985, -53.221928}, R = 11.726048571348);
          constant IdealGases.Common.DataRecord In2CL2(name = "In2CL2", MM = 0.300542, Hf =  -772528.881154714, H0 = 67291.410185598, Tlimit = 1000, alow = { -11428.48747, -141.6015827,10.56853314, -0.00124425344,1.526076784e-06, -9.82640281e-10,2.58076699e-13}, blow = { -30263.27087, -15.6510496}, ahigh = { -26166.06332, -2.401955489,10.00203984, -8.97519003e-07,2.130755553e-10, -2.5811409e-14,1.248404822e-18}, bhigh = { -30980.16735, -12.34946505}, R = 27.6649253681682);
          constant IdealGases.Common.DataRecord In2CL4(name = "In2CL4", MM = 0.371448, Hf =  -1559103.44381986, H0 = 79301.8807477763, Tlimit = 1000, alow = {9814.08308, -848.895901,19.2341036, -0.00681567424,8.12922073e-06, -5.12507685e-09,1.324324755e-12}, blow = { -70276.8814, -55.82665695}, ahigh = { -85037.7179, -15.52464614,16.01269668, -5.44154987e-06,1.267594997e-09, -1.514110365e-13,7.24540454e-18}, bhigh = { -74619.2174, -36.91150615}, R = 22.3839460705132);
          constant IdealGases.Common.DataRecord In2CL6(name = "In2CL6", MM = 0.442354, Hf =  -1994646.88688245, H0 = 86861.3418212563, Tlimit = 1000, alow = {30046.63159, -1585.621359,27.9648947, -0.01245506052,1.475341213e-05, -9.2526892e-09,2.381233885e-12}, blow = { -104882.83, -97.8840464}, ahigh = { -149940.0484, -29.88680816,22.02425362, -1.033652301e-05,2.397911922e-09, -2.855307516e-13,1.363051661e-17}, bhigh = { -113011.0233, -62.9398568}, R = 18.7959688394363);
          constant IdealGases.Common.DataRecord In2F2(name = "In2F2", MM = 0.2676328064, Hf =  -1988673.61277276, H0 = 67405.4135689099, Tlimit = 1000, alow = {7255.7171, -644.826565,12.47539512, -0.00524608962,6.28361621e-06, -3.97434234e-09,1.029559356e-12}, blow = { -63848.9245, -31.08004586}, ahigh = { -64097.1544, -11.63740672,10.00956511, -4.11408691e-06,9.60882832e-10, -1.150013486e-13,5.51143748e-18}, bhigh = { -67142.7785, -16.61718474}, R = 31.0667145475929);
          constant IdealGases.Common.DataRecord In2F4(name = "In2F4", MM = 0.3056296128, Hf =  -4203742.2821353, H0 = 81023.1501232331, Tlimit = 1000, alow = {122521.3611, -2444.998374,23.42425457, -0.0129142712,1.308700476e-05, -7.18167393e-09,1.648386404e-12}, blow = { -146694.2111, -89.4356005}, ahigh = { -220597.7486, -78.2208377,16.05923872, -2.39402324e-05,5.32958279e-09, -6.14460957e-13,2.859099598e-17}, bhigh = { -159571.6347, -44.6957458}, R = 27.2044057636551);
          constant IdealGases.Common.DataRecord In2F6(name = "In2F6", MM = 0.3436264192, Hf =  -5703868.7670264, H0 = 97684.869743566, Tlimit = 1000, alow = {182549.0583, -3552.11171,32.7542523, -0.01868064635,1.892443162e-05, -1.038916311e-08,2.386637271e-12}, blow = { -223964.876, -136.3626758}, ahigh = { -317096.638, -116.7440973,22.08847456, -3.57773393e-05,7.96895904e-09, -9.1916817e-13,4.27849415e-17}, bhigh = { -242666.1118, -71.5532856}, R = 24.1962536505691);
          constant IdealGases.Common.DataRecord In2I2(name = "In2I2", MM = 0.48344494, Hf =  -57531.9373494736, H0 = 45627.6903011954, Tlimit = 1000, alow = { -6681.92501, -14.49103222,10.05929122, -0.0001314921486,1.628373589e-07, -1.056087811e-10,2.788963879e-14}, blow = { -6279.6026, -7.29113098}, ahigh = { -8148.1573, -0.2444544443,10.00021086, -9.37678925e-08,2.24298818e-11, -2.732299303e-15,1.327135699e-19}, bhigh = { -6352.6515, -6.94772542}, R = 17.1983845771558);
          constant IdealGases.Common.DataRecord In2I4(name = "In2I4", MM = 0.73725388, Hf =  -270114.795462317, H0 = 45396.8163585657, Tlimit = 1000, alow = { -15010.28064, -97.447979,16.3938395, -0.000865938021,1.065671801e-06, -6.87925268e-10,1.810240886e-13}, blow = { -28303.88443, -29.86294828}, ahigh = { -25056.99251, -1.642155234,16.00140149, -6.18706628e-07,1.472308565e-10, -1.786593333e-14,8.65233878e-19}, bhigh = { -28796.55333, -27.57790375}, R = 11.2776239305787);
          constant IdealGases.Common.DataRecord In2I6(name = "In2I6", MM = 0.99106282, Hf =  -322603.20289283, H0 = 45494.7144521071, Tlimit = 1000, alow = { -22326.40607, -200.0786874,22.80417927, -0.001761331313,2.161510393e-06, -1.392400447e-09,3.65817188e-13}, blow = { -44125.4768, -52.7380203}, ahigh = { -43119.0585, -3.39035059,22.00288128, -1.268348706e-06,3.0121219e-10, -3.64967671e-14,1.765530622e-18}, bhigh = { -45138.2027, -48.0686837}, R = 8.3894500249742);
          constant IdealGases.Common.DataRecord In2O(name = "In2O", MM = 0.2456354, Hf =  -141525.639219754, H0 = 52095.520433944, Tlimit = 1000, alow = {50656.535, -807.036367,8.64206727, -0.001533035446,2.866419382e-07,4.98113497e-10, -2.554320291e-13}, blow = { -1924.908459, -14.90718759}, ahigh = { -92327.8636, -52.0112728,7.03872362, -1.543821179e-05,3.39970087e-09, -3.8854571e-13,1.79512527e-17}, bhigh = { -6270.9268, -4.42182041}, R = 33.8488344920968);
          constant IdealGases.Common.DataRecord K(name = "K", MM = 0.0390983, Hf = 2276313.80392498, H0 = 158508.886575631, Tlimit = 1000, alow = {9.66514393, -0.1458059455,2.500865861, -2.601219276e-06,4.18730658e-09, -3.43972211e-12,1.131569009e-15}, blow = {9959.49349,5.03582226}, ahigh = { -3566422.36,10852.89825, -10.54134898,0.00800980135, -2.696681041e-06,4.71529415e-10, -2.97689735e-14}, bhigh = { -58753.3701,97.3855124}, R = 212.655588606154);
          constant IdealGases.Common.DataRecord Kplus(name = "Kplus", MM = 0.0390977514, Hf = 13146728.6376986, H0 = 158511.110692673, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {61075.1686,4.34740444}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = {61075.1686,4.34740444}, R = 212.658572482509);
          constant IdealGases.Common.DataRecord Kminus(name = "Kminus", MM = 0.0390988486, Hf = 880284.950385981, H0 = 158506.662521003, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {3394.15071,4.34744653}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = {3394.15071,4.34744653}, R = 212.652604813534);
          constant IdealGases.Common.DataRecord KALF4(name = "KALF4", MM = 0.1420734508, Hf =  -13428665.3224587, H0 = 150776.87547799, Tlimit = 1000, alow = {119932.7246, -2156.457352,18.15348004,0.003115762809, -9.51199167e-06,8.43175623e-09, -2.615583061e-12}, blow = { -222254.5809, -68.2329997}, ahigh = { -343513.537, -242.0680834,16.1802925, -7.19381768e-05,1.585712708e-08, -1.814017733e-12,8.38839118e-17}, bhigh = { -233965.8121, -52.338987}, R = 58.5223484977814);
          constant IdealGases.Common.DataRecord KBO2(name = "KBO2", MM = 0.0819081, Hf =  -8155765.99872296, H0 = 172030.641658151, Tlimit = 1000, alow = {42963.0865, -720.961643,8.34963935,0.002407820401, -1.680003427e-07, -1.231930486e-09,5.47995164e-13}, blow = { -78685.2347, -14.76077068}, ahigh = {88907.266, -1671.432436,11.1748393, -0.000451376026,9.69900346e-08, -1.09002769e-11,4.9768458e-16}, bhigh = { -73753.3608, -32.7474937}, R = 101.509765212476);
          constant IdealGases.Common.DataRecord KBr(name = "KBr", MM = 0.1190023, Hf =  -1506280.78616968, H0 = 84948.9631713, Tlimit = 1000, alow = {9203.30919, -213.7880361,5.58850915, -0.002753854319,4.01367261e-06, -2.856323127e-09,8.13520974e-13}, blow = { -21883.87013, -1.707501588}, ahigh = {1562614.367, -4384.89478,9.0454646, -0.0020456809,4.40192385e-07, -1.448178289e-11, -2.27333876e-15}, bhigh = {5315.34201, -28.64647026}, R = 69.8681622119909);
          constant IdealGases.Common.DataRecord KCN(name = "KCN", MM = 0.0651157, Hf = 1220842.28534747, H0 = 191295.401876967, Tlimit = 1000, alow = {17986.27669, -630.055897,9.988695, -0.01076486132,1.75101498e-05, -1.265592431e-08,3.46976281e-12}, blow = {10580.25371, -25.94060289}, ahigh = {361566.387, -1749.018011,8.68070267, -0.000440235004,9.24949971e-08, -1.021916855e-11,4.60476719e-16}, bhigh = {18137.34856, -22.81619241}, R = 127.687669793921);
          constant IdealGases.Common.DataRecord KCL(name = "KCL", MM = 0.0745513, Hf =  -2878217.83121153, H0 = 132594.857500808, Tlimit = 1000, alow = {9058.35151, -245.6801212,5.68069619, -0.002900127425,4.13098306e-06, -2.907340629e-09,8.22385087e-13}, blow = { -25973.04884, -3.677976854}, ahigh = { -212294.5722,934.61589,2.866264958,0.001468386693, -5.83426078e-07,1.255777709e-10, -9.150148e-15}, bhigh = { -32737.8764,14.01864636}, R = 111.526854662494);
          constant IdealGases.Common.DataRecord KF(name = "KF", MM = 0.0580967032, Hf =  -5653416.93950028, H0 = 162747.857954184, Tlimit = 1000, alow = {14357.04906, -339.184823,5.72790672, -0.002518371562,3.26591564e-06, -2.21011086e-09,6.21204205e-13}, blow = { -39142.5442, -5.81099962}, ahigh = { -1483743.237,4550.04804, -1.081464319,0.00350396588, -1.093902537e-06,1.771028824e-10, -1.029032783e-14}, bhigh = { -69633.2981,40.8863592}, R = 143.114351452562);
          constant IdealGases.Common.DataRecord KH(name = "KH", MM = 0.04010624, Hf = 3126673.20596496, H0 = 219295.201943638, Tlimit = 1000, alow = {223.778215,179.3554623,1.130735415,0.00996611256, -1.341218171e-05,9.03452994e-09, -2.40988777e-12}, blow = {13382.50358,15.52739897}, ahigh = { -3752276.52,11727.78444, -10.14137678,0.0088545489, -2.517188074e-06,3.34407236e-10, -1.701157835e-14}, bhigh = { -60251.033,101.030459}, R = 207.311181501931);
          constant IdealGases.Common.DataRecord KI(name = "KI", MM = 0.16600277, Hf =  -773817.671837645, H0 = 61710.5545889385, Tlimit = 1000, alow = { -457.281174, -63.3176973,4.82329242, -0.000787666832,1.336737911e-06, -1.000530992e-09,3.004503956e-13}, blow = { -16503.40769,3.55259482}, ahigh = {3293747.78, -10028.24194,16.29302044, -0.00669639675,2.001412089e-06, -2.677673023e-10,1.273359646e-14}, bhigh = {46763.4867, -78.5912337}, R = 50.0863449447259);
          constant IdealGases.Common.DataRecord KLi(name = "KLi", MM = 0.0460393, Hf = 3707752.35505318, H0 = 221487.359712246, Tlimit = 1000, alow = { -3426.04369, -19.86063081,4.48760705,0.000524953555, -1.207990165e-06,1.705161047e-09, -7.87999421e-13}, blow = {19278.68247,1.912285942}, ahigh = {12112968.43, -40810.0487,56.8379212, -0.03128695515,8.9750622e-06, -1.189303691e-09,5.88012918e-14}, bhigh = {273879.2595, -366.96936}, R = 180.595100273028);
          constant IdealGases.Common.DataRecord KNO2(name = "KNO2", MM = 0.0851038, Hf =  -2261913.02856042, H0 = 180281.092031143, Tlimit = 1000, alow = { -72462.3693,1226.722811, -2.258926457,0.03019643903, -3.64025995e-05,2.219135205e-08, -5.47165423e-12}, blow = { -30772.6896,45.37172549}, ahigh = { -168941.4093, -851.597315,10.63209586, -0.000252325481,5.57364441e-08, -6.39318592e-12,2.964484049e-16}, bhigh = { -21877.01436, -27.55200432}, R = 97.6980111346379);
          constant IdealGases.Common.DataRecord KNO3(name = "KNO3", MM = 0.1011032, Hf =  -3123866.38602932, H0 = 157434.304749998, Tlimit = 1000, alow = { -25961.12303,820.060661, -2.886015624,0.042072889, -5.27032309e-05,3.30856173e-08, -8.36586037e-12}, blow = { -43350.5447,46.03187331}, ahigh = { -318368.287, -1375.234449,14.01587531, -0.000404066008,8.90084778e-08, -1.018771993e-11,4.71598202e-16}, bhigh = { -35138.5322, -48.04256439}, R = 82.2374761629701);
          constant IdealGases.Common.DataRecord KNa(name = "KNa", MM = 0.06208807, Hf = 2132524.44471216, H0 = 170308.514985246, Tlimit = 1000, alow = {25424.05292, -411.920741,6.84108795, -0.00625048138,8.86101314e-06, -4.92241535e-09,6.49321247e-13}, blow = {16525.9989, -9.13616691}, ahigh = {6260326.62, -25635.93614,43.7333545, -0.0267265511,8.23270579e-06, -1.12850359e-09,5.66326489e-14}, bhigh = {169727.4909, -266.4117219}, R = 133.914164186453);
          constant IdealGases.Common.DataRecord KO(name = "KO", MM = 0.0550977, Hf = 1174882.32721148, H0 = 172078.217421054, Tlimit = 1000, alow = {14625.62908, -338.476565,5.71660764, -0.002363265083,2.848716276e-06, -1.739858233e-09,4.43100652e-13}, blow = {8141.83538, -4.02210152}, ahigh = {696010.338, -3304.83529,10.05743444, -0.004331112,1.747281632e-06, -3.012370548e-10,1.79082787e-14}, bhigh = {26049.72496, -34.4878152}, R = 150.904157523817);
          constant IdealGases.Common.DataRecord KOH(name = "KOH", MM = 0.05610564, Hf =  -4135056.65384086, H0 = 208083.358464497, Tlimit = 1000, alow = {17706.84196, -615.320522,8.68407572, -0.00396284951,3.40865059e-06, -9.60197222e-10,8.49405497e-15}, blow = { -26779.03261, -21.74495666}, ahigh = {891727.195, -2334.179072,7.97257871,0.0001038863156, -6.31589347e-08,1.027938106e-11, -5.73668582e-16}, bhigh = { -14436.96469, -20.76401416}, R = 148.193158477472);
          constant IdealGases.Common.DataRecord K2(name = "K2", MM = 0.0781966, Hf = 1618309.86257715, H0 = 137360.56299123, Tlimit = 1000, alow = {15241.69293, -330.178936,7.07079595, -0.00976707246,2.021535863e-05, -1.886092452e-08,6.11297464e-12}, blow = {15334.02849, -9.1010358}, ahigh = { -27344707.45,65621.8001, -44.7635044,0.00893885915,2.984557092e-06, -1.064158914e-09,8.33493693e-14}, bhigh = { -422624.383,386.714251}, R = 106.327794303077);
          constant IdealGases.Common.DataRecord K2plus(name = "K2plus", MM = 0.0781960514, Hf = 6709555.29859402, H0 = 138896.156590331, Tlimit = 1000, alow = {51960.3657, -611.338253,7.26499054, -0.00581063482,6.5674965e-06, -2.378020865e-09, -1.318637581e-13}, blow = {64798.2027, -10.42370517}, ahigh = {11079507.39, -41774.8382,64.4865984, -0.0402499803,1.360587923e-05, -2.361920107e-09,1.67343061e-13}, bhigh = {317761.205, -410.50631}, R = 106.328540266932);
          constant IdealGases.Common.DataRecord K2Br2(name = "K2Br2", MM = 0.2380046, Hf =  -2263587.31301832, H0 = 88031.0968779595, Tlimit = 1000, alow = { -10930.40504, -48.5665148,10.19754923, -0.00043631571,5.38717366e-07, -3.4861165e-10,9.19070295e-14}, blow = { -67580.7324, -12.94944706}, ahigh = { -15892.29976, -0.810137946,10.00069442, -3.074491521e-07,7.33102416e-11, -8.90894478e-15,4.31924543e-19}, bhigh = { -67825.9544, -11.80425726}, R = 34.9340811059954);
          constant IdealGases.Common.DataRecord K2CO3(name = "K2CO3", MM = 0.1382055, Hf =  -5872770.35284413, H0 = 141208.83032875, Tlimit = 1000, alow = { -44074.067,706.981544,0.693148247,0.0396783132, -4.8509583e-05,2.976635787e-08, -7.37476153e-12}, blow = { -103391.3232,29.80651689}, ahigh = { -326426.316, -1521.168973,17.12283562, -0.000446458071,9.83337963e-08, -1.125478371e-11,5.21006912e-16}, bhigh = { -94882.751, -62.1522063}, R = 60.1602107007319);
          constant IdealGases.Common.DataRecord K2C2N2(name = "K2C2N2", MM = 0.1302314, Hf =  -64254.8571235509, H0 = 190324.622172533, Tlimit = 1000, alow = {4627.78948, -972.981062,19.9666591, -0.01953015081,3.27533046e-05, -2.39398512e-08,6.59654824e-12}, blow = { -777.441126, -67.55613632}, ahigh = {726959.95, -3492.10868,18.35676822, -0.000878547514,1.845532919e-07, -2.038717568e-11,9.18542099e-16}, bhigh = {15826.00773, -67.28484052}, R = 63.8438348969603);
          constant IdealGases.Common.DataRecord K2CL2(name = "K2CL2", MM = 0.1491026, Hf =  -4127316.78052562, H0 = 133890.676621333, Tlimit = 1000, alow = { -13285.26456, -124.1832429,10.5003573, -0.001097776004,1.348871402e-06, -8.69721512e-10,2.28658408e-13}, blow = { -76443.6315, -17.90150052}, ahigh = { -26146.11331, -2.096229095,10.00178456, -7.86480371e-07,1.869281152e-10, -2.266272348e-14,1.096790704e-18}, bhigh = { -77071.8933, -14.99720802}, R = 55.7634273312471);
          constant IdealGases.Common.DataRecord K2F2(name = "K2F2", MM = 0.1161934064, Hf =  -7400381.03401365, H0 = 155995.306115752, Tlimit = 1000, alow = { -1611.833856, -513.564489,11.99922511, -0.00428010955,5.16538754e-06, -3.2858149e-09,8.54964875e-13}, blow = { -103924.8602, -30.39925079}, ahigh = { -57408.9253, -9.04618211,10.00750685, -3.25076484e-06,7.63006459e-10, -9.16564953e-14,4.40504844e-18}, bhigh = { -106541.3546, -18.7403762}, R = 71.5571757262811);
          constant IdealGases.Common.DataRecord K2I2(name = "K2I2", MM = 0.33200554, Hf =  -1261772.47524243, H0 = 64627.0782108034, Tlimit = 1000, alow = { -8977.75244, -27.76460169,10.11331014, -0.0002508421994,3.102352114e-07, -2.010096209e-10,5.30443715e-14}, blow = { -53262.0839, -10.32092509}, ahigh = { -11796.34729, -0.472476155,10.00040695, -1.807689054e-07,4.32056097e-11, -5.25969943e-15,2.553419144e-19}, bhigh = { -53402.1023, -9.6644405}, R = 25.043172472363);
          constant IdealGases.Common.DataRecord K2O(name = "K2O", MM = 0.094196, Hf =  -786518.557051255, H0 = 147121.93723725, Tlimit = 1000, alow = {23920.44068, -544.535839,8.82640323, -0.00348142943,3.83454207e-06, -2.268494189e-09,5.56921225e-13}, blow = { -8234.29168, -16.63099064}, ahigh = { -46114.6458, -13.63119524,7.01053044, -4.32365826e-06,9.74778842e-10, -1.135283694e-13,5.3258606e-18}, bhigh = { -11072.22244, -5.76871872}, R = 88.2677820714255);
          constant IdealGases.Common.DataRecord K2Oplus(name = "K2Oplus", MM = 0.0941954514, Hf = 3910914.12084894, H0 = 150093.45769747, Tlimit = 1000, alow = {7201.10273, -333.65264,8.05073035, -0.001869353797,1.921291874e-06, -1.064020951e-09,2.457514799e-13}, blow = {43899.8636, -10.7320415}, ahigh = { -37800.335, -8.84953739,7.00684789, -2.815510856e-06,6.3550015e-10, -7.40855199e-14,3.47833496e-18}, bhigh = {42145.1968, -4.41827661}, R = 88.2682961483212);
          constant IdealGases.Common.DataRecord K2O2(name = "K2O2", MM = 0.1101954, Hf =  -1738423.75453059, H0 = 147826.524519172, Tlimit = 1000, alow = {48108.7061, -1003.601504,11.54229002, -0.000454633252, -1.516606209e-06,1.79942609e-09, -6.12833729e-13}, blow = { -20571.5192, -31.8119979}, ahigh = { -147607.3073, -102.193204,10.07664713, -3.076517991e-05,6.81525281e-09, -7.82865445e-13,3.6325061e-17}, bhigh = { -25920.56324, -21.62287184}, R = 75.4520787619084);
          constant IdealGases.Common.DataRecord K2O2H2(name = "K2O2H2", MM = 0.11221128, Hf =  -5712438.17912067, H0 = 199489.15117981, Tlimit = 1000, alow = {8174.78837, -1130.63068,18.15303256, -0.00773743108,6.83761401e-06, -2.054691111e-09,7.67287751e-14}, blow = { -75749.6517, -63.9175947}, ahigh = {1773523.196, -4665.29247,16.94308128,0.0002085297143, -1.264714762e-07,2.057506565e-11, -1.148042134e-15}, bhigh = { -50512.6331, -63.3477392}, R = 74.0965792387361);
          constant IdealGases.Common.DataRecord K2SO4(name = "K2SO4", MM = 0.1742592, Hf =  -6288627.06244491, H0 = 127877.776324005, Tlimit = 1000, alow = {62714.9231, -815.204992,7.45541372,0.0384990978, -5.40063098e-05,3.6546568e-08, -9.76022357e-12}, blow = { -130469.2568, -10.3463477}, ahigh = { -544612.564, -959.066723,19.71591285, -0.0002866586009,6.34248064e-08, -7.28146254e-12,3.37788693e-16}, bhigh = { -133787.3384, -73.99103551}, R = 47.7132455560453);
          constant IdealGases.Common.DataRecord Kr(name = "Kr", MM = 0.0838, Hf = 0, H0 = 73954.9880668258, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = { -745.375,5.49095651}, ahigh = {264.3639057, -0.791005082,2.500920585, -5.32816411e-07,1.620730161e-10, -2.467898017e-14,1.47858504e-18}, bhigh = { -740.348894,5.48439815}, R = 99.2180429594272);
          constant IdealGases.Common.DataRecord Krplus(name = "Krplus", MM = 0.0837994514, Hf = 16192873.525184, H0 = 73955.472219237, Tlimit = 1000, alow = { -5650.40286,69.3074081,2.157028132,0.000871122893, -1.18160973e-06,7.86219863e-10, -1.832589387e-13}, blow = {162116.4118,8.81824226}, ahigh = { -221656.7015,1166.16784,0.486965532,0.001429223599, -3.94962861e-07,4.98285351e-11, -2.406719258e-15}, bhigh = {155600.2861,20.59230986}, R = 99.2186924985049);
          constant IdealGases.Common.DataRecord Li(name = "Li", MM = 0.006941, Hf = 22950583.4894108, H0 = 892872.496758392, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {18413.90197,2.447622965}, ahigh = {1125610.652, -3463.53673,6.56661192, -0.002260983356,5.92228916e-07, -6.2816351e-11,2.884948238e-15}, bhigh = {40346.374, -26.55918195}, R = 1197.87811554531);
          constant IdealGases.Common.DataRecord Liplus(name = "Liplus", MM = 0.0069404514, Hf = 98800407.7083517, H0 = 892943.072838173, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {81727.2455,1.754357228}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = {81727.2455,1.754357228}, R = 1197.9728004435);
          constant IdealGases.Common.DataRecord Liminus(name = "Liminus", MM = 0.0069415486, Hf = 13465976.1656066, H0 = 892801.931834058, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {10496.98659,1.754594332}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = {10496.98659,1.754594332}, R = 1197.78344561327);
          constant IdealGases.Common.DataRecord LiALF4(name = "LiALF4", MM = 0.1099161508, Hf =  -16897315.0031378, H0 = 177720.824990898, Tlimit = 1000, alow = {176152.936, -2999.300134,20.7973957, -0.001742914656, -4.26664209e-06,5.36282977e-09, -1.866817087e-12}, blow = { -211794.7828, -87.6272589}, ahigh = { -402070.093, -278.3355146,16.20791185, -8.31634157e-05,1.83690569e-08, -2.104932808e-12,9.74725302e-17}, bhigh = { -227869.6416, -55.9730484}, R = 75.6437697234208);
          constant IdealGases.Common.DataRecord LiBO2(name = "LiBO2", MM = 0.0497508, Hf =  -13112397.7101876, H0 = 269899.659904966, Tlimit = 1000, alow = {65189.8757, -1060.320845,9.54804996, -5.46957607e-05,2.658687599e-06, -2.931603694e-09,9.66302973e-13}, blow = { -75062.0411, -24.90883998}, ahigh = {85544.0452, -1731.920898,11.2146209, -0.000465916217,9.99987569e-08, -1.122880592e-11,5.12352946e-16}, bhigh = { -71547.4382, -36.1239729}, R = 167.122377931611);
          constant IdealGases.Common.DataRecord LiBr(name = "LiBr", MM = 0.086845, Hf =  -1740605.56163279, H0 = 105626.230640797, Tlimit = 1000, alow = {38056.2047, -612.961499,6.57736955, -0.00421989176,5.39142337e-06, -3.69314481e-09,1.049158761e-12}, blow = { -16374.87617, -11.28654012}, ahigh = {63801.9142, -259.7050764,4.65676887,0.0001022867706, -5.17825234e-08,2.058203991e-11, -1.942190308e-15}, bhigh = { -17933.63752, -0.2290806613}, R = 95.7392135413668);
          constant IdealGases.Common.DataRecord LiCL(name = "LiCL", MM = 0.042394, Hf =  -4570927.27744492, H0 = 213712.058310138, Tlimit = 1000, alow = {49643.995, -718.734662,6.78514703, -0.00452214546,5.7484837e-06, -3.96625567e-09,1.137911398e-12}, blow = { -20910.14703, -14.06383901}, ahigh = { -235276.9705,612.004692,3.63429373,0.000681097332, -2.174238799e-07,4.22040361e-11, -2.848628426e-15}, bhigh = { -28623.58494,5.618511135}, R = 196.123791102515);
          constant IdealGases.Common.DataRecord LiF(name = "LiF", MM = 0.0259394032, Hf =  -13143898.931337, H0 = 340335.740646493, Tlimit = 1000, alow = {29125.3732, -253.1413159,3.53972798,0.00369591704, -4.82946615e-06,2.944090711e-09, -6.83879474e-13}, blow = { -40648.4966,2.325294408}, ahigh = { -378424.649,766.806246,3.5854734,0.000603108435, -1.588764206e-07,2.569397177e-11, -1.406798386e-15}, bhigh = { -47564.6945,4.38516329}, R = 320.534436968079);
          constant IdealGases.Common.DataRecord LiH(name = "LiH", MM = 0.00794894, Hf = 17519833.3362687, H0 = 1092737.39643273, Tlimit = 1000, alow = { -49137.3157,775.609219, -1.011102377,0.01145479597, -1.151038734e-05,5.87506896e-09, -1.196789735e-12}, blow = {12048.5891,25.68801877}, ahigh = { -2633686.357,6996.42917, -3.23353306,0.00403393598, -9.09957964e-07,8.77590987e-11, -2.889490251e-15}, bhigh = { -29900.43016,49.719845}, R = 1045.98499925776);
          constant IdealGases.Common.DataRecord LiI(name = "LiI", MM = 0.13384547, Hf =  -637077.130813617, H0 = 69409.2373839772, Tlimit = 1000, alow = {40719.7637, -666.728947,7.06425753, -0.00548004102,6.90579016e-06, -4.53716852e-09,1.226373296e-12}, blow = { -8235.48702, -12.96236234}, ahigh = {1616342.632, -4877.71708,9.98718655, -0.002889137815,8.03731377e-07, -9.02817261e-11,3.078321954e-15}, bhigh = {19377.18764, -37.2927631}, R = 62.1199357736948);
          constant IdealGases.Common.DataRecord LiN(name = "LiN", MM = 0.0209477, Hf = 15978842.5459597, H0 = 429600.76762604, Tlimit = 1000, alow = {37649.592, -488.345763,5.18737345,0.0002275252171, -1.416337564e-06,1.450236258e-09, -4.79001433e-13}, blow = {41619.1531, -5.952157244}, ahigh = { -59934.8998, -40.3113455,4.52962421,8.74415693e-05,2.557428364e-09, -2.9057518e-13,1.336125922e-17}, bhigh = {38948.3113, -1.214952896}, R = 396.915747313548);
          constant IdealGases.Common.DataRecord LiNO2(name = "LiNO2", MM = 0.0529465, Hf =  -3815761.34399819, H0 = 252496.236767303, Tlimit = 1000, alow = { -31337.73859,538.994081, -0.02059877081,0.02610777454, -3.20923504e-05,1.974344044e-08, -4.89219916e-12}, blow = { -28382.14963,27.10019529}, ahigh = { -219667.116, -861.004271,10.63956465, -0.0002554392587,5.64447854e-08, -6.47614005e-12,3.003536591e-16}, bhigh = { -23135.47535, -32.38277103}, R = 157.035347001218);
          constant IdealGases.Common.DataRecord LiNO3(name = "LiNO3", MM = 0.0689459, Hf =  -4519263.29194339, H0 = 201829.622936244, Tlimit = 1000, alow = {19672.05518,90.5279526, -0.51097704,0.0375660595, -4.78273865e-05,3.029277805e-08, -7.70801315e-12}, blow = { -39075.6399,27.46873033}, ahigh = { -355940.245, -1447.644737,14.06684061, -0.000423637128,9.32082451e-08, -1.065894872e-11,4.93078062e-16}, bhigh = { -34348.2746, -52.75382697}, R = 120.594147005116);
          constant IdealGases.Common.DataRecord LiO(name = "LiO", MM = 0.0229404, Hf = 3178423.78511273, H0 = 408105.917943889, Tlimit = 1000, alow = {36270.2976, -349.936323,4.39493318,0.001079712984, -8.72403881e-07,5.60796297e-10, -2.054288457e-13}, blow = {9533.33053, -0.905814961}, ahigh = {1612392.133, -5551.31234,11.20573851, -0.00343722688,9.13319466e-07, -1.027902258e-10,3.822991e-15}, bhigh = {42015.7547, -48.5735458}, R = 362.437969695385);
          constant IdealGases.Common.DataRecord LiOF(name = "LiOF", MM = 0.0419388032, Hf =  -2194817.04237092, H0 = 258170.528814709, Tlimit = 1000, alow = {55101.1069, -577.068817,4.42495493,0.01102425422, -1.710493381e-05,1.234361201e-08, -3.453693e-12}, blow = { -9278.98698,0.1238830687}, ahigh = { -183356.3281, -211.0049301,7.15681817, -6.24946201e-05,1.376567002e-08, -1.574099067e-12,7.27716714e-17}, bhigh = { -12545.30277, -12.72130479}, R = 198.252486136753);
          constant IdealGases.Common.DataRecord LiOH(name = "LiOH", MM = 0.02394834, Hf =  -9562249.4085185, H0 = 473392.352037761, Tlimit = 1000, alow = {4574.19012, -103.0949027,4.27240737,0.00846521923, -1.386148524e-05,1.101099795e-08, -3.29124821e-12}, blow = { -28487.28855, -0.877574578}, ahigh = {850075.137, -2430.540791,8.05531462,6.89568088e-05, -5.52720746e-08,9.36805403e-12, -5.31378568e-16}, bhigh = { -13658.94396, -24.57598093}, R = 347.183646131632);
          constant IdealGases.Common.DataRecord LiON(name = "LiON", MM = 0.0369471, Hf = 4869448.48174823, H0 = 305869.175117939, Tlimit = 1000, alow = { -9412.67267,100.93323,3.006187607,0.0098904622, -1.128294343e-05,6.41029174e-09, -1.466806373e-12}, blow = {19783.48318,10.16334813}, ahigh = { -97406.2914, -511.156094,7.37849387, -0.0001508777884,3.32991498e-08, -3.81751429e-12,1.769558455e-16}, bhigh = {22110.06635, -14.5410504}, R = 225.037201837221);
          constant IdealGases.Common.DataRecord Li2(name = "Li2", MM = 0.013882, Hf = 15552514.0469673, H0 = 696954.041204437, Tlimit = 1000, alow = {6778.48158, -224.6205832,5.29603744, -0.001272412017,1.205843729e-06, -9.81854459e-11, -2.416702607e-13}, blow = {25736.38186, -6.86925758}, ahigh = {37676454, -118574.7185,148.2167789, -0.0837853211,2.424919798e-05, -3.27582024e-09,1.652000081e-13}, bhigh = {772307.201, -1021.697298}, R = 598.939057772655);
          constant IdealGases.Common.DataRecord Li2plus(name = "Li2plus", MM = 0.0138814514, Hf = 51983859.5552047, H0 = 719601.986287976, Tlimit = 1000, alow = { -10400.56453,26.75405642,4.24727175,0.001057450777, -1.281715096e-06,1.067477239e-09, -2.759413508e-13}, blow = {85298.1131,0.529818996}, ahigh = {12799310.73, -34928.6139,38.4478536, -0.01380635397,2.53139425e-06, -2.197815991e-10,7.08766925e-15}, bhigh = {311796.445, -250.9543641}, R = 598.962728061707);
          constant IdealGases.Common.DataRecord Li2Br2(name = "Li2Br2", MM = 0.17369, Hf =  -2854703.9207784, H0 = 97590.3966837469, Tlimit = 1000, alow = {27863.12863, -1005.641437,13.70963066, -0.0076336633,8.94290592e-06, -5.56113509e-09,1.421736784e-12}, blow = { -57628.4584, -41.2184376}, ahigh = { -89009.8336, -19.88916335,10.01596418, -6.74913405e-06,1.556299963e-09, -1.844693069e-13,8.77487992e-18}, bhigh = { -62799.8996, -19.43063525}, R = 47.8696067706834);
          constant IdealGases.Common.DataRecord Li2F2(name = "Li2F2", MM = 0.0518788064, Hf =  -18029001.4536649, H0 = 265349.782604096, Tlimit = 1000, alow = {144316.6185, -2466.874678,17.0286329, -0.0114554121,1.086098792e-05, -5.56975428e-09,1.193819664e-12}, blow = { -102607.0133, -70.0038661}, ahigh = { -218893.1683, -92.5981453,10.06970757, -2.803632163e-05,6.21754952e-09, -7.14626702e-13,3.3168573e-17}, bhigh = { -115661.1527, -27.29853181}, R = 160.26721848404);
          constant IdealGases.Common.DataRecord Li2I2(name = "Li2I2", MM = 0.26769094, Hf =  -1355299.32017871, H0 = 65966.5657717067, Tlimit = 1000, alow = {10148.72266, -712.042101,12.7039296, -0.00568553681,6.77034038e-06, -4.26330489e-09,1.100658186e-12}, blow = { -43130.6386, -33.0395098}, ahigh = { -69735.8706, -13.16825508,10.01074855, -4.60020378e-06,1.070510527e-09, -1.277717768e-13,6.11061224e-18}, bhigh = { -46774.7005, -17.21894034}, R = 31.0599678868474);
          constant IdealGases.Common.DataRecord Li2O(name = "Li2O", MM = 0.0298814, Hf =  -5600103.07415315, H0 = 428095.069173466, Tlimit = 1000, alow = {26366.01597, -179.8629279,4.05111522,0.01189981375, -1.730095793e-05,1.20558455e-08, -3.29273388e-12}, blow = { -20619.07963,1.60599561}, ahigh = {726148.704, -9543.78372,28.87491643, -0.01959494099,8.08633984e-06, -1.370211764e-09,8.11172719e-14}, bhigh = {29907.26399, -156.4517822}, R = 278.249078021779);
          constant IdealGases.Common.DataRecord Li2Oplus(name = "Li2Oplus", MM = 0.0298808514, Hf = 14694865.3879387, H0 = 436021.34442528, Tlimit = 1000, alow = {108104.4623, -1261.961355,9.69201129, -0.001793477428,3.3803269e-07,4.14339515e-10, -1.990530222e-13}, blow = {57549.7604, -29.15719946}, ahigh = { -130216.6086, -146.6759528,7.61072945, -4.46942669e-05,9.94716064e-09, -1.147038806e-12,5.33924147e-17}, bhigh = {50987.6231, -15.28011047}, R = 278.254186559089);
          constant IdealGases.Common.DataRecord Li2O2(name = "Li2O2", MM = 0.0458808, Hf =  -6089649.43941693, H0 = 295287.178950672, Tlimit = 1000, alow = {139261.973, -1764.036747,10.29649198,0.00760594995, -1.538519312e-05,1.238199941e-08, -3.68543157e-12}, blow = { -26380.56746, -34.3771056}, ahigh = { -293923.2153, -235.9107937,10.17463402, -6.93162436e-05,1.521202834e-08, -1.733841909e-12,7.99315462e-17}, bhigh = { -36184.8191, -29.04221044}, R = 181.218984847692);
          constant IdealGases.Common.DataRecord Li2O2H2(name = "Li2O2H2", MM = 0.04789668, Hf =  -15387287.803664, H0 = 325263.462937306, Tlimit = 1000, alow = {189441.3579, -2779.117524,15.30234911,0.01070738317, -2.510075145e-05,2.224910575e-08, -6.94935958e-12}, blow = { -77027.7692, -65.1411694}, ahigh = {1215377.492, -3982.05744,15.85877373,0.000750914568, -2.585218184e-07,3.6468417e-11, -1.904139406e-15}, bhigh = { -67387.4838, -66.472084}, R = 173.591823065816);
          constant IdealGases.Common.DataRecord Li2SO4(name = "Li2SO4", MM = 0.1099446, Hf =  -9475826.9164652, H0 = 178749.461092223, Tlimit = 1000, alow = {100631.8702, -1543.457217,9.67005491,0.0339075738, -4.82348381e-05,3.26736892e-08, -8.70251851e-12}, blow = { -120193.9538, -29.11716635}, ahigh = { -603875.087, -1064.550976,19.79557349, -0.000318897825,7.06241112e-08, -8.11443907e-12,3.76683021e-16}, bhigh = { -126890.8595, -80.14316674}, R = 75.6241961860792);
          constant IdealGases.Common.DataRecord Li3plus(name = "Li3plus", MM = 0.0208224514, Hf = 36335339.2674986, H0 = 638224.997850157, Tlimit = 1000, alow = { -6873.22229, -282.0780467,7.92269748, -0.001687949474,1.771718993e-06, -9.9725354e-10,2.332934849e-13}, blow = {90279.6365, -16.19319856}, ahigh = { -43550.9806, -6.05643275,7.00472289, -1.953320933e-06,4.42915223e-10, -5.18198319e-14,2.439894871e-18}, bhigh = {88799.0287, -10.66880792}, R = 399.303225171653);
          constant IdealGases.Common.DataRecord Li3Br3(name = "Li3Br3", MM = 0.260535, Hf =  -3165176.24503426, H0 = 99935.9279943194, Tlimit = 1000, alow = {64319.6625, -1823.743601,22.43492856, -0.0127941826,1.459200355e-05, -8.88468815e-09,2.23370805e-12}, blow = { -94806.7767, -82.0686442}, ahigh = { -158467.0201, -40.4136094,16.03177633, -1.322713911e-05,3.014221791e-09, -3.54029216e-13,1.672030783e-17}, bhigh = { -104247.0448, -44.0547816}, R = 31.9130711804556);
          constant IdealGases.Common.DataRecord Li3CL3(name = "Li3CL3", MM = 0.127182, Hf =  -7674884.18958658, H0 = 193113.380824331, Tlimit = 1000, alow = {98674.29, -2324.074851,23.78484564, -0.014835224,1.634495612e-05, -9.67536652e-09,2.377033859e-12}, blow = { -110383.9177, -94.74114379}, ahigh = { -200593.0356, -59.3940019,16.04590929, -1.885890854e-05,4.25351737e-09, -4.95556094e-13,2.325408214e-17}, bhigh = { -122492.7071, -48.44413629}, R = 65.3745970341715);
          constant IdealGases.Common.DataRecord Li3F3(name = "Li3F3", MM = 0.0778182096, Hf =  -19591775.933123, H0 = 263906.534287574, Tlimit = 1000, alow = {182387.4443, -3163.78624,22.87811928, -0.00754490389,3.44679023e-06,2.690705228e-10, -5.25084096e-13}, blow = { -171244.6093, -99.7365499}, ahigh = { -361866.499, -193.7051395,16.14457133, -5.7756285e-05,1.274057038e-08, -1.458168875e-12,6.74485003e-17}, bhigh = { -188208.1974, -56.3361288}, R = 106.844812322693);
          constant IdealGases.Common.DataRecord Li3I3(name = "Li3I3", MM = 0.40153641, Hf =  -1525284.61117636, H0 = 67821.2319525395, Tlimit = 1000, alow = {36117.6469, -1392.033487,21.10791892, -0.01046866587,1.222589173e-05, -7.58413543e-09,1.93519918e-12}, blow = { -71530.8458, -70.9388203}, ahigh = { -126669.6511, -27.83891914,16.02228068, -9.39936493e-06,2.163919735e-09, -2.561731922e-13,1.217394107e-17}, bhigh = { -78695.5068, -40.9172406}, R = 20.7066452578983);
          constant IdealGases.Common.DataRecord Mg(name = "Mg", MM = 0.024305, Hf = 6052252.6229171, H0 = 254985.723102242, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {16946.58761,3.63433014}, ahigh = { -536483.155,1973.709576, -0.36337769,0.002071795561, -7.73805172e-07,1.359277788e-10, -7.766898397e-15}, bhigh = {4829.18811,23.39104998}, R = 342.088952890352);
          constant IdealGases.Common.DataRecord Mgplus(name = "Mgplus", MM = 0.0243044514, Hf = 36661884.908869, H0 = 254991.478639176, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {106422.3354,4.32744346}, ahigh = { -19147.58821,48.7734792,2.457662661,1.218104674e-05,1.897261686e-09, -1.580433756e-12,2.135732238e-16}, bhigh = {106102.2394,4.64644286}, R = 342.096674521113);
          constant IdealGases.Common.DataRecord MgBr(name = "MgBr", MM = 0.104209, Hf = 59143.7879645712, H0 = 92008.5117408285, Tlimit = 1000, alow = {7361.41914, -239.5789881,5.36056042, -0.001667141829,1.981137765e-06, -1.201637202e-09,3.032148099e-13}, blow = {591.563444, -1.421771179}, ahigh = {24776.04216, -641.768752,6.01993209, -0.001391004302,6.44333096e-07, -1.197734078e-10,7.42164424e-15}, bhigh = {2824.060334, -6.26443992}, R = 79.7865059639762);
          constant IdealGases.Common.DataRecord MgBr2(name = "MgBr2", MM = 0.184113, Hf =  -1666059.53409048, H0 = 80153.3894944952, Tlimit = 1000, alow = {21484.77999, -515.45289,9.27325875, -0.00347004904,3.92184079e-06, -2.377174864e-09,5.96702556e-13}, blow = { -36524.4385, -17.91077763}, ahigh = { -43268.9901, -12.83036001,7.5100243, -4.1523324e-06,9.42649918e-10, -1.103873928e-13,5.20110431e-18}, bhigh = { -39198.8602, -7.40916803}, R = 45.1596139327478);
          constant IdealGases.Common.DataRecord MgCL(name = "MgCL", MM = 0.059758, Hf =  -915440.392918103, H0 = 156683.841494026, Tlimit = 1000, alow = {20439.9528, -407.215516,5.8803723, -0.002594175042,2.945953528e-06, -1.750648628e-09,4.33795933e-13}, blow = { -5851.44495, -6.02354575}, ahigh = {1041328.453, -3380.15833,8.63777547, -0.002447789643,7.84196944e-07, -1.12640938e-10,5.81062073e-15}, bhigh = {13271.88977, -27.03802395}, R = 139.135714046655);
          constant IdealGases.Common.DataRecord MgCLplus(name = "MgCLplus", MM = 0.0597574514, Hf = 10816045.8128239, H0 = 159239.254303272, Tlimit = 1000, alow = {8182.38512, -262.225376,5.41986343, -0.001774129606,2.185811127e-06, -1.418143432e-09,3.91889858e-13}, blow = {77704.0371, -3.78093884}, ahigh = { -12683919.21,34788.2454, -30.0422295,0.01481739497, -2.470965605e-06,1.424433718e-10,2.789613105e-16}, bhigh = { -148701.374,255.2015117}, R = 139.136991374435);
          constant IdealGases.Common.DataRecord MgCL2(name = "MgCL2", MM = 0.095211, Hf =  -4192476.66761194, H0 = 145997.878396404, Tlimit = 1000, alow = {36378.2468, -730.784496,9.66103051, -0.00366021294,3.61081935e-06, -1.928769286e-09,4.30991235e-13}, blow = { -46469.1457, -23.60112274}, ahigh = { -68352.1701, -24.90899393,7.5187832, -7.56451826e-06,1.67929377e-09, -1.931706275e-13,8.97165782e-18}, bhigh = { -50326.8691, -10.53268382}, R = 87.3268004747351);
          constant IdealGases.Common.DataRecord MgF(name = "MgF", MM = 0.0433034032, Hf =  -5363707.58037789, H0 = 207122.473921403, Tlimit = 1000, alow = {38230.0162, -480.331039,5.06846894,0.00052120293, -1.874026347e-06,1.759241129e-09, -5.60940319e-13}, blow = { -26591.14296, -3.76896921}, ahigh = { -169588.3782,358.875763,3.93600165,0.000481304414, -1.658385409e-07,3.33605229e-11, -2.270104205e-15}, bhigh = { -31693.2399,4.4105659}, R = 192.005047769548);
          constant IdealGases.Common.DataRecord MgFplus(name = "MgFplus", MM = 0.0433028546, Hf = 11936113.2141159, H0 = 207131.263812802, Tlimit = 1000, alow = {641329.384, -8518.53261,48.1421691, -0.1163071535,0.0001622560562, -1.06348459e-07,2.634809398e-11}, blow = {102430.848, -245.0074295}, ahigh = { -10568523.62,20771.22379, -6.11952356,0.002245834831, -9.51573861e-08, -2.263502341e-11,2.284524246e-15}, bhigh = { -83315.4988,87.2710238}, R = 192.007480264361);
          constant IdealGases.Common.DataRecord MgF2(name = "MgF2", MM = 0.0623018064, Hf =  -11805405.0516262, H0 = 202601.509159452, Tlimit = 1000, alow = {43384.2955, -661.651177,7.45344852,0.00352081405, -6.95576458e-06,5.61992348e-09, -1.688028906e-12}, blow = { -86871.8367, -15.4547679}, ahigh = { -124441.9584, -86.8734375,7.56358123, -2.499267978e-05,5.44012914e-09, -6.15824576e-13,2.822792666e-17}, bhigh = { -90600.1944, -14.2079699}, R = 133.454750037553);
          constant IdealGases.Common.DataRecord MgF2plus(name = "MgF2plus", MM = 0.0623012578, Hf = 9352815.40977171, H0 = 199274.692653155, Tlimit = 1000, alow = {78322.2026, -1176.632752,10.25829767, -0.00375527289,3.016621585e-06, -1.327098871e-09,2.465289749e-13}, blow = {74132.2855, -29.90883551}, ahigh = { -150231.687,77.2528956,7.27156936,0.000223752977, -1.003070865e-07,1.996274309e-11, -1.233278721e-15}, bhigh = {66991.175, -10.94528006}, R = 133.455925186795);
          constant IdealGases.Common.DataRecord MgH(name = "MgH", MM = 0.02531294, Hf = 9077811.74371685, H0 = 342990.739123942, Tlimit = 1000, alow = { -49586.7915,750.027865, -0.64420475,0.00982630101, -8.78982244e-06,3.82335352e-09, -6.00372576e-13}, blow = {23022.79383,26.57165344}, ahigh = { -100574.8598,1952.890106, -1.317191549,0.0056036658, -2.13733498e-06,3.3248805e-10, -1.824672746e-14}, bhigh = {15985.82755,34.3123316}, R = 328.467258248153);
          constant IdealGases.Common.DataRecord MgI(name = "MgI", MM = 0.15120947, Hf = 404778.331674597, H0 = 64421.3421289024, Tlimit = 1000, alow = {2943.889099, -169.0248574,5.14725183, -0.001321186997,1.623056505e-06, -1.005114222e-09,2.567337785e-13}, blow = {6845.89027,0.859313225}, ahigh = { -2370562.811,6916.45248, -3.18389449,0.00405155114, -9.77429075e-07,1.0233294e-10, -3.79030487e-15}, bhigh = { -38185.5111,59.9310438}, R = 54.9864502534134);
          constant IdealGases.Common.DataRecord MgI2(name = "MgI2", MM = 0.27811394, Hf =  -617394.543401888, H0 = 54993.5432938025, Tlimit = 1000, alow = {15947.39709, -416.041272,9.01063832, -0.003090359024,3.62126426e-06, -2.260138392e-09,5.80905555e-13}, blow = { -20804.414, -14.12175736}, ahigh = { -33402.9918, -9.34351178,7.50746854, -3.14770789e-06,7.24133647e-10, -8.56766495e-14,4.0696917e-18}, bhigh = { -22945.55505, -5.24127259}, R = 29.8959196363908);
          constant IdealGases.Common.DataRecord MgN(name = "MgN", MM = 0.0383117, Hf = 7535452.61630259, H0 = 234624.95791103, Tlimit = 1000, alow = {37595.3864, -485.316428,5.16305424,0.0002591539493, -1.518993975e-06,1.522840476e-09, -4.99253145e-13}, blow = {36072.9461, -3.813908776}, ahigh = { -60185.891, -40.1086478,4.52949895,4.76049091e-05,2.547271298e-09, -2.894058118e-13,1.330639131e-17}, bhigh = {33412.857,0.7925250209}, R = 217.0217453154);
          constant IdealGases.Common.DataRecord MgO(name = "MgO", MM = 0.0403044, Hf = 800441.31658082, H0 = 221045.518603428, Tlimit = 1000, alow = {351365.974, -5287.19716,33.8206006, -0.0840048963,0.000121001616, -7.63079502e-08,1.701022862e-11}, blow = {27906.79519, -162.4886199}, ahigh = { -15867383.67,34204.681, -17.74087677,0.00700496305, -1.104138249e-06,8.95748853e-11, -3.052513649e-15}, bhigh = { -230050.4434,173.8984472}, R = 206.291918500213);
          constant IdealGases.Common.DataRecord MgOH(name = "MgOH", MM = 0.04131234, Hf =  -3205555.04723286, H0 = 269272.667682344, Tlimit = 1000, alow = {38398.5162, -736.738364,7.92066446, -0.000595094059, -2.112941162e-06,3.22828211e-09, -1.214159329e-12}, blow = { -13923.26188, -19.16078109}, ahigh = {664866.475, -1770.750355,7.26999927,0.000533684276, -1.980894443e-07,3.025677088e-11, -1.554849476e-15}, bhigh = { -6149.11456, -16.71027009}, R = 201.258800639228);
          constant IdealGases.Common.DataRecord MgOHplus(name = "MgOHplus", MM = 0.0413117914, Hf = 14905414.3897522, H0 = 246621.355664572, Tlimit = 1000, alow = {117022.4573, -1735.933343,11.64059613, -0.00844987622,7.35171374e-06, -2.790223071e-09,3.51498213e-13}, blow = {81188.0767, -42.7117437}, ahigh = {829633.954, -2459.700177,8.11873202,3.5005791e-05, -4.67057475e-08,8.31235826e-12, -4.80283622e-16}, bhigh = {88016.6171, -24.38155217}, R = 201.261473255793);
          constant IdealGases.Common.DataRecord Mg_OH_2(name = "Mg_OH_2", MM = 0.05831968, Hf =  -9465000.63100483, H0 = 293752.606324315, Tlimit = 1000, alow = {52458.9467, -1289.056383,13.89327642, -0.000780669367, -4.15125723e-06,6.10947304e-09, -2.274138833e-12}, blow = { -62950.8915, -50.1535334}, ahigh = {1713709.254, -4730.00535,14.48925967,0.0001907819857, -1.226834131e-07,2.015343753e-11, -1.128993279e-15}, bhigh = { -38877.2467, -58.4049812}, R = 142.567174579833);
          constant IdealGases.Common.DataRecord MgS(name = "MgS", MM = 0.05637, Hf = 2140310.64395955, H0 = 163812.506652475, Tlimit = 1000, alow = { -9565.78809,144.3637798,1.813794717,0.01147168775, -2.220170412e-05,1.995344981e-08, -6.09068874e-12}, blow = {12765.01517,14.61333093}, ahigh = {26507943.28, -77113.5586,84.6377168, -0.0364425068,8.40308442e-06, -9.53988217e-10,4.26465803e-14}, bhigh = {507893.117, -583.4656096}, R = 147.498172786943);
          constant IdealGases.Common.DataRecord Mg2(name = "Mg2", MM = 0.04861, Hf = 5894122.91709525, H0 = 196299.423986834, Tlimit = 1000, alow = {4545.19559,411.585004,0.484119617,0.00489196965, -6.39553684e-06,4.29976455e-09, -1.164624418e-12}, blow = {31816.4179,26.40432143}, ahigh = {30382.24994,59.4524046,2.352706666,0.0001378537924, -5.89569204e-08,1.104045317e-11, -6.55886829e-16}, bhigh = {33510.3656,15.88177377}, R = 171.044476445176);
          constant IdealGases.Common.DataRecord Mg2F4(name = "Mg2F4", MM = 0.1246036128, Hf =  -13790683.603678, H0 = 169684.598422816, Tlimit = 1000, alow = {151195.6137, -3122.595912,25.17715088, -0.01558723757,1.552195619e-05, -8.40872905e-09,1.911618525e-12}, blow = { -195307.8883, -108.5103537}, ahigh = { -298061.0166, -120.1299868,16.09110864, -3.68745629e-05,8.22031705e-09, -9.48886978e-13,4.41979533e-17}, bhigh = { -211734.969, -53.0865547}, R = 66.7273750187763);
          constant IdealGases.Common.DataRecord Mn(name = "Mn", MM = 0.054938049, Hf = 5140335.4349187, H0 = 112807.573490642, Tlimit = 1000, alow = {0.1034061359, -0.001551537349,2.500009148, -2.723162066e-08,4.33389743e-11, -3.51109389e-14,1.136032201e-17}, blow = {33219.3519,6.649325463}, ahigh = {5855.15582,883.858844, -0.0364866258,0.002703720687, -1.324971998e-06,2.87260329e-10, -1.92363357e-14}, bhigh = {28678.03487,22.92541198}, R = 151.342687833709);
          constant IdealGases.Common.DataRecord Mnplus(name = "Mnplus", MM = 0.0549375004, Hf = 18309375.5754494, H0 = 112808.699974999, Tlimit = 1000, alow = {345.80177, -4.25115133,2.521281028, -5.56508728e-05,8.03716221e-08, -6.09355097e-11,1.900014268e-14}, blow = {120253.3602,6.683468162}, ahigh = {647131.41, -2403.796253,5.93771575, -0.002341014594,7.46416564e-07, -9.07596973e-11,4.467879847e-15}, bhigh = {134990.2108, -17.02666341}, R = 151.344199125594);
          constant IdealGases.Common.DataRecord Mo(name = "Mo", MM = 0.09594, Hf = 6863664.79049406, H0 = 64596.9147383782, Tlimit = 1000, alow = {76.4636791, -1.159269043,2.506929462, -2.099249725e-05,3.41477943e-08, -2.841269591e-11,9.492443321e-15}, blow = {78458.998,7.60183566}, ahigh = {5573271, -16623.65811,21.35147077, -0.01003069377,2.409784357e-06, -1.811267352e-10,1.034189087e-15}, bhigh = {184264.6473, -127.5326434}, R = 86.6632478632479);
          constant IdealGases.Common.DataRecord Moplus(name = "Moplus", MM = 0.0959394514, Hf = 14061086.5323335, H0 = 64597.2841158022, Tlimit = 1000, alow = {129.8236623, -1.560279908,2.507600281, -1.923789063e-05,2.673316651e-08, -1.937174292e-11,5.729735412e-15}, blow = {161510.3759,7.44254346}, ahigh = {12988911.2, -39482.7623,48.6659978, -0.02605352326,7.21543192e-06, -8.71916496e-10,3.78842304e-14}, bhigh = {411894.857, -321.679103}, R = 86.6637434201547);
          constant IdealGases.Common.DataRecord Mominus(name = "Mominus", MM = 0.0959405486, Hf = 6048794.13833162, H0 = 64596.5453651784, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {69051.2369,7.48565954}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = {69051.2369,7.48565954}, R = 86.6627523120084);
          constant IdealGases.Common.DataRecord MoO(name = "MoO", MM = 0.1119394, Hf = 3198206.55640463, H0 = 91595.3006716134, Tlimit = 1000, alow = { -28011.52706,513.988348,1.075385931,0.00868104847, -1.111118984e-05,7.23434933e-09, -1.893138381e-12}, blow = {39413.7408,22.72230239}, ahigh = {1573131.992, -5241.48358,11.02656868, -0.00390299662,1.147334134e-06, -1.358975691e-10,5.77526858e-15}, bhigh = {74489.72, -42.5361293}, R = 74.276546059743);
          constant IdealGases.Common.DataRecord MoO2(name = "MoO2", MM = 0.1279388, Hf =  -121605.634881678, H0 = 83688.6464465823, Tlimit = 1000, alow = {32471.8322, -190.4783783,2.120771647,0.01650280086, -2.381696822e-05,1.652371586e-08, -4.49445243e-12}, blow = { -1862.932837,16.40582056}, ahigh = {309614.3654, -1932.750274,9.42867318, -0.001630508855,5.75276017e-07, -7.59045747e-11,3.46133778e-15}, bhigh = {7327.72518, -25.33315948}, R = 64.9878848324355);
          constant IdealGases.Common.DataRecord MoO3(name = "MoO3", MM = 0.1439382, Hf =  -2531727.99854382, H0 = 91655.8981562921, Tlimit = 1000, alow = {59773.8536, -768.455783,5.88184444,0.01686119817, -2.582485043e-05,1.850382718e-08, -5.15224985e-12}, blow = { -41558.7239, -6.52916216}, ahigh = { -409759.727,237.9066513,9.3111008,0.000657933891, -2.895307725e-07,5.69263726e-11, -3.48965731e-15}, bhigh = { -49237.3872, -21.14864892}, R = 57.7641793491929);
          constant IdealGases.Common.DataRecord MoO3minus(name = "MoO3minus", MM = 0.1439387486, Hf =  -4552231.46910143, H0 = 94207.3147911194, Tlimit = 1000, alow = {182161.7352, -2224.65697,13.20405967, -0.001249800352, -1.864166983e-06,2.27657947e-09, -7.36656457e-13}, blow = { -69389.86, -46.9937721}, ahigh = { -488109.872,18.48536991,10.42644941, -0.000626555791,3.020554347e-07, -4.73319589e-11,2.527517727e-15}, bhigh = { -83378.155, -27.01522825}, R = 57.763959190069);
          constant IdealGases.Common.DataRecord Mo2O6(name = "Mo2O6", MM = 0.2878764, Hf =  -3992848.36478433, H0 = 89495.002021701, Tlimit = 1000, alow = {156837.5811, -2159.930184,15.22500503,0.031941989, -5.06387715e-05,3.68937182e-08, -1.037872908e-11}, blow = { -130993.1161, -54.089185}, ahigh = { -631223.816, -664.282755,22.4938357, -0.0001968470329,4.33674061e-08, -4.95977799e-12,2.293212733e-16}, bhigh = { -143057.325, -86.6962854}, R = 28.8820896745965);
          constant IdealGases.Common.DataRecord Mo3O9(name = "Mo3O9", MM = 0.4318146, Hf =  -4404740.71279665, H0 = 94448.2168967886, Tlimit = 1000, alow = {148338.6187, -1863.895133,17.38405871,0.0623301232, -9.32844135e-05,6.61363879e-08, -1.830607518e-11}, blow = { -224894.5631, -58.2666947}, ahigh = { -923029.054, -1076.171759,34.8000726, -0.00031896338,7.02839704e-08, -8.03964264e-12,3.71787953e-16}, bhigh = { -235742.2166, -144.8769485}, R = 19.254726449731);
          constant IdealGases.Common.DataRecord Mo4O12(name = "Mo4O12", MM = 0.5757528, Hf =  -4560163.27319641, H0 = 95342.7408429451, Tlimit = 1000, alow = {223487.6437, -2996.203728,26.82458827,0.0759734572, -0.0001155247012,8.25647257e-08, -2.296764551e-11}, blow = { -308461.9476, -106.5236386}, ahigh = { -1225283.388, -1368.141197,47.0168781, -0.00040529102,8.92847239e-08, -1.021091158e-11,4.72109169e-16}, bhigh = { -325647.777, -204.7486535}, R = 14.4410448372982);
          constant IdealGases.Common.DataRecord Mo5O15(name = "Mo5O15", MM = 0.719691, Hf =  -4625746.93444826, H0 = 95759.6273956462, Tlimit = 1000, alow = {276890.3251, -3757.9331,34.0819776,0.0948541496, -0.00014426721,1.0311639e-07, -2.868600013e-11}, blow = { -391350.703, -142.6449143}, ahigh = { -1535389.516, -1710.367122,59.2712578, -0.000506683629,1.116223706e-07, -1.276562233e-11,5.90231891e-16}, bhigh = { -412897.238, -265.1287167}, R = 11.5528358698386);
          constant IdealGases.Common.DataRecord N(name = "N", MM = 0.0140067, Hf = 33746706.9331106, H0 = 442461.679053596, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {56104.6378,4.193905036}, ahigh = {88765.0138, -107.12315,2.362188287,0.0002916720081, -1.7295151e-07,4.01265788e-11, -2.677227571e-15}, bhigh = {56973.5133,4.865231506}, R = 593.606773901062);
          constant IdealGases.Common.DataRecord Nplus(name = "Nplus", MM = 0.0140061514, Hf = 134378643.372369, H0 = 508099.891023597, Tlimit = 1000, alow = {5237.07921,2.299958315,2.487488821,2.737490756e-05, -3.134447576e-08,1.850111332e-11, -4.447350984e-15}, blow = {225628.4738,5.076830786}, ahigh = {290497.0374, -855.790861,3.47738929, -0.000528826719,1.352350307e-07, -1.389834122e-11,5.046166279e-16}, bhigh = {231080.9984, -1.994146545}, R = 593.630024590481);
          constant IdealGases.Common.DataRecord Nminus(name = "Nminus", MM = 0.0140072486, Hf = 33806606.7450249, H0 = 463927.298327525, Tlimit = 1000, alow = {1445.682471,7.33520511,2.476680939,4.22786918e-05, -4.42629332e-08,2.490985431e-11, -5.83160809e-15}, blow = {56176.25,5.145753977}, ahigh = {2404.189576,0.2954965336,2.499789368,8.30756497e-08, -1.82994277e-11,2.100136461e-15, -9.75498671e-20}, bhigh = {56214.1389,5.006484157}, R = 593.583525032889);
          constant IdealGases.Common.DataRecord NCO(name = "NCO", MM = 0.0420168, Hf = 3137964.83787437, H0 = 242718.269834923, Tlimit = 1000, alow = {11365.03036, -244.4613367,4.6713761,0.002309387548,2.798649599e-06, -4.54635738e-09,1.692880931e-12}, blow = {15776.49188, -0.2171476903}, ahigh = {108944.5289, -1735.459316,8.65561033, -0.000405322926,7.59971641e-08, -7.25380415e-12,3.24487241e-16}, bhigh = {23657.92776, -26.1953297}, R = 197.884465261514);
          constant IdealGases.Common.DataRecord ND(name = "ND", MM = 0.016020802, Hf = 22204805.7893731, H0 = 539798.194871892, Tlimit = 1000, alow = {22901.55757, -395.738851,6.17901033, -0.00884780697,1.44158297e-05, -1.006647227e-08,2.654403586e-12}, blow = {43559.1343, -11.80414072}, ahigh = {543965.796, -2084.583507,5.83408972, -0.000418939336,9.76892953e-08, -1.056317654e-11,4.68307921e-16}, bhigh = {54666.3341, -14.80810939}, R = 518.979761437661);
          constant IdealGases.Common.DataRecord ND2(name = "ND2", MM = 0.018034904, Hf = 10248824.7234363, H0 = 552391.351792058, Tlimit = 1000, alow = {19352.22164, -213.0631713,4.8440176, -0.002516949288,7.61638154e-06, -5.46450177e-09,1.292736999e-12}, blow = {22119.98374, -3.171712417}, ahigh = {1631308.357, -6564.63749,12.80547961, -0.003094456779,9.18166023e-07, -1.241119284e-10,6.22629986e-15}, bhigh = {61089.2958, -61.24502693}, R = 461.021139896281);
          constant IdealGases.Common.DataRecord ND3(name = "ND3", MM = 0.020049006, Hf =  -2730913.74205784, H0 = 510449.246212007, Tlimit = 1000, alow = {10451.2037,161.0166943,0.857496323,0.01319688794, -1.153090144e-05,7.14249556e-09, -2.109194351e-12}, blow = { -8220.9489,16.75921299}, ahigh = {2599516.958, -10134.20124,17.98028169, -0.0035826098,1.009922e-06, -1.537638609e-10,9.10617565e-15}, bhigh = {53972.0566, -98.10988569}, R = 414.707442354);
          constant IdealGases.Common.DataRecord NF(name = "NF", MM = 0.0330051032, Hf = 7059226.52591494, H0 = 264750.119005839, Tlimit = 1000, alow = { -35049.2775,667.450299, -1.201665982,0.01452074253, -1.822873148e-05,1.160136864e-08, -2.973416333e-12}, blow = {23954.14002,30.89260431}, ahigh = {800298.733, -3237.69658,8.70340887, -0.002701025798,9.15004211e-07, -1.36525663e-10,7.23462441e-15}, bhigh = {46428.1945, -30.19933248}, R = 251.91474026356);
          constant IdealGases.Common.DataRecord NF2(name = "NF2", MM = 0.0520035064, Hf = 661906.270997142, H0 = 203478.683122029, Tlimit = 1000, alow = {15118.31104,91.9638994,0.494730179,0.02001847323, -2.767712684e-05,1.872924867e-08, -5.02031804e-12}, blow = {2839.279606,22.7051567}, ahigh = { -194501.0078, -353.603407,7.26349436, -8.4023804e-05,2.326721111e-08, -2.667577562e-12,1.236070614e-16}, bhigh = {3435.44059, -13.36102511}, R = 159.882911279998);
          constant IdealGases.Common.DataRecord NF3(name = "NF3", MM = 0.0710019096, Hf =  -1854879.68903867, H0 = 166960.453131249, Tlimit = 1000, alow = {87571.4928, -903.183289,4.02741727,0.02314439555, -3.41510647e-05,2.409483651e-08, -6.63346419e-12}, blow = { -12372.32074,0.3026430713}, ahigh = { -349626.876, -497.372867,10.36866128,8.90068765e-05,5.88265436e-08, -3.157737664e-12,1.714329953e-16}, bhigh = { -17131.83352, -30.98920858}, R = 117.102089885199);
          constant IdealGases.Common.DataRecord NH(name = "NH", MM = 0.01501464, Hf = 23778925.169035, H0 = 572847.767245835, Tlimit = 1000, alow = {13596.5132, -190.0296604,4.51849679, -0.002432776899,2.377587464e-06, -2.592797084e-10, -2.659680792e-13}, blow = {42809.7219, -3.886561616}, ahigh = {1958141.991, -5782.8613,9.33574202, -0.002292910311,6.07609248e-07, -6.64794275e-11,2.384234783e-15}, bhigh = {78989.1234, -41.169704}, R = 553.757665851462);
          constant IdealGases.Common.DataRecord NHplus(name = "NHplus", MM = 0.0150140914, Hf = 110948299.808539, H0 = 632413.493899471, Tlimit = 1000, alow = {4253.65685, -245.8222206,6.70891949, -0.0103848943,1.509008623e-05, -9.58051219e-09,2.333206758e-12}, blow = {200107.7797, -13.95057632}, ahigh = {1405709.438, -4136.21571,7.63201448, -0.001228325778,2.721187746e-07, -2.010098289e-11,3.71719018e-17}, bhigh = {225897.596, -27.86785234}, R = 553.777899607032);
          constant IdealGases.Common.DataRecord NHF(name = "NHF", MM = 0.0340130432, Hf = 3292854.43062031, H0 = 294878.877524255, Tlimit = 1000, alow = { -51106.5982,961.225643, -2.706446594,0.0203656268, -2.425558952e-05,1.551553017e-08, -4.05845826e-12}, blow = {7909.62834,40.99317124}, ahigh = {901390.272, -3463.39705,8.70580486, -0.000401896341,2.322774501e-08,6.28048733e-12, -6.28309569e-16}, bhigh = {33370.6534, -29.00483634}, R = 244.449517530969);
          constant IdealGases.Common.DataRecord NHF2(name = "NHF2", MM = 0.0530114464, Hf =  -1942976.60212493, H0 = 203869.913649442, Tlimit = 1000, alow = { -56261.1342,1205.756556, -6.01752942,0.0376002769, -4.6191986e-05,2.94247557e-08, -7.59873236e-12}, blow = { -18970.14374,59.01714907}, ahigh = {739427.899, -4004.47177,12.2132232, -0.000697043773,1.271073981e-07, -1.247136898e-11,5.08651759e-16}, bhigh = {9134.19598, -48.67843963}, R = 156.842956844883);
          constant IdealGases.Common.DataRecord NH2(name = "NH2", MM = 0.01602258, Hf = 11804260.7994468, H0 = 620241.371863957, Tlimit = 1000, alow = { -31182.40659,475.424339,1.372395176,0.00630642972, -5.98789356e-06,4.49275234e-09, -1.414073548e-12}, blow = {19289.39662,15.40126885}, ahigh = {2111053.74, -6880.62723,11.32305924, -0.001829236741,5.64389009e-07, -7.88645248e-11,4.07859345e-15}, bhigh = {65037.7856, -53.59155744}, R = 518.922171086055);
          constant IdealGases.Common.DataRecord NH2F(name = "NH2F", MM = 0.0350209832, Hf =  -2141573.22687617, H0 = 288540.614131016, Tlimit = 1000, alow = { -109237.476,1844.91978, -7.6738716,0.0322953344, -3.38810867e-05,1.97187155e-08, -4.81020515e-12}, blow = { -18783.1896,68.61483739}, ahigh = {1927205.34, -7500.44716,13.9558958, -0.00118480442,2.05067867e-07, -1.90876131e-11,7.38923621e-16}, bhigh = {35529.2734, -67.31185491}, R = 237.41400841082);
          constant IdealGases.Common.DataRecord NH3(name = "NH3", MM = 0.01703052, Hf =  -2697510.11713089, H0 = 589713.115042876, Tlimit = 1000, alow = { -76812.2615,1270.951578, -3.89322913,0.02145988418, -2.183766703e-05,1.317385706e-08, -3.33232206e-12}, blow = { -12648.86413,43.66014588}, ahigh = {2452389.535, -8040.89424,12.71346201, -0.000398018658,3.55250275e-08,2.53092357e-12, -3.32270053e-16}, bhigh = {43861.9196, -64.62330602}, R = 488.210107501122);
          constant IdealGases.Common.DataRecord NH2OH(name = "NH2OH", MM = 0.03302992, Hf =  -1513779.02217141, H0 = 340169.700683502, Tlimit = 1000, alow = { -56175.8667,1209.290057, -6.17959906,0.0405311644, -5.19010554e-05,3.59454458e-08, -9.93368164e-12}, blow = { -12658.88352,57.27932928}, ahigh = {4878285.05, -15336.04636,22.2723999, -0.002514583678,3.33958973e-07, -1.881744532e-11,1.918174365e-16}, bhigh = {89230.2071, -126.9053624}, R = 251.725465880632);
          constant IdealGases.Common.DataRecord NH4plus(name = "NH4plus", MM = 0.0180379114, Hf = 35752750.4542461, H0 = 553211.055244456, Tlimit = 1000, alow = { -266831.5752,3763.02069, -15.71327725,0.0454882021, -4.37996212e-05,2.464478293e-08, -5.96153233e-12}, blow = {58232.8472,111.2087156}, ahigh = {4141889, -14420.72042,20.11893564, -0.001971492619,3.112721421e-07, -2.602979969e-11,8.89434213e-16}, bhigh = {166419.6236, -120.1535761}, R = 460.944275399867);
          constant IdealGases.Common.DataRecord NO(name = "NO", MM = 0.0300061, Hf = 3041758.50910315, H0 = 305908.132013157, Tlimit = 1000, alow = { -11439.16503,153.6467592,3.43146873, -0.002668592368,8.48139912e-06, -7.68511105e-09,2.386797655e-12}, blow = {9098.21441,6.72872549}, ahigh = {223901.8716, -1289.651623,5.43393603, -0.00036560349,9.88096645e-08, -1.416076856e-11,9.38018462e-16}, bhigh = {17503.17656, -8.50166909}, R = 277.092724479356);
          constant IdealGases.Common.DataRecord NOCL(name = "NOCL", MM = 0.0654591, Hf = 805064.964229572, H0 = 173612.041717653, Tlimit = 1000, alow = {23088.35209, -549.598384,7.73046336, -0.0050739109,1.062996184e-05, -8.7932497e-09,2.648180166e-12}, blow = {7389.89839, -13.18393021}, ahigh = { -613341.333, -391.929883,9.13891722, -0.002605664613,1.295687247e-06, -2.215378352e-10,1.280394898e-14}, bhigh = {4517.32842, -23.07323335}, R = 127.01781723244);
          constant IdealGases.Common.DataRecord NOF(name = "NOF", MM = 0.0490045032, Hf =  -1326408.71257726, H0 = 218760.936239835, Tlimit = 1000, alow = {47550.2426, -725.390417,7.21399636, -0.002532427181,6.3777439e-06, -5.51830588e-09,1.681935713e-12}, blow = { -5609.72252, -12.89663616}, ahigh = {1889069.274, -6731.02266,14.19018767, -0.00369312462,9.93857514e-07, -1.080748188e-10,4.21035443e-15}, bhigh = {32099.0078, -63.70266962}, R = 169.667509250456);
          constant IdealGases.Common.DataRecord NOF3(name = "NOF3", MM = 0.0870013096, Hf =  -2149392.93281627, H0 = 157441.917403046, Tlimit = 1000, alow = {148836.0135, -2241.049812,13.02355027,0.00546397668, -8.64186525e-06,5.91365903e-09, -1.577009169e-12}, blow = { -13283.42568, -48.77320739}, ahigh = { -278562.5217, -1252.321663,13.90824337, -0.00035666875,7.78501106e-08, -8.85041197e-12,4.07598003e-16}, bhigh = { -20256.51446, -51.06881859}, R = 95.5672051171055);
          constant IdealGases.Common.DataRecord NO2(name = "NO2", MM = 0.0460055, Hf = 743237.634630642, H0 = 221890.317462043, Tlimit = 1000, alow = { -56420.3878,963.308572, -2.434510974,0.01927760886, -1.874559328e-05,9.14549773e-09, -1.777647635e-12}, blow = { -1547.925037,40.6785121}, ahigh = {721300.157, -3832.6152,11.13963285, -0.002238062246,6.54772343e-07, -7.6113359e-11,3.32836105e-15}, bhigh = {25024.97403, -43.0513004}, R = 180.727782547739);
          constant IdealGases.Common.DataRecord NO2minus(name = "NO2minus", MM = 0.0460060486, Hf =  -4348028.07646471, H0 = 221210.238864113, Tlimit = 1000, alow = { -12820.67858,699.013818, -2.812596273,0.02412894252, -2.831606689e-05,1.670509365e-08, -3.98333013e-12}, blow = { -28099.15579,40.6327151}, ahigh = {132571.0335, -1557.032129,8.12672192, -0.000272862678, -4.7075418e-08,2.826729008e-11, -2.353985481e-15}, bhigh = { -17157.95217, -22.28576043}, R = 180.725627455865);
          constant IdealGases.Common.DataRecord NO2CL(name = "NO2CL", MM = 0.0814585, Hf = 153452.371452948, H0 = 149828.919020115, Tlimit = 1000, alow = {8508.37034, -180.5383762,3.78538856,0.01414934934, -1.423946765e-05,7.02822618e-09, -1.374688214e-12}, blow = {915.624647,6.958904458}, ahigh = { -108677.3327, -1452.231167,11.05656962, -0.000400009928,9.10154304e-08, -1.036656913e-11,4.78166481e-16}, bhigh = {6294.26732, -35.21239681}, R = 102.070035662331);
          constant IdealGases.Common.DataRecord NO2F(name = "NO2F", MM = 0.0650039032, Hf =  -1676822.38502872, H0 = 174552.302883867, Tlimit = 1000, alow = {56678.5695, -653.825195,4.47277152,0.01368870672, -1.460533236e-05,7.77922794e-09, -1.689355106e-12}, blow = { -11021.79443,0.329207431}, ahigh = { -100857.7842, -1704.722752,11.22954945, -0.000468521597,1.047692566e-07, -1.189150595e-11,5.47030712e-16}, bhigh = { -6891.71918, -38.49788492}, R = 127.907273112794);
          constant IdealGases.Common.DataRecord NO3(name = "NO3", MM = 0.0620049, Hf = 1147135.14577074, H0 = 176742.709043963, Tlimit = 1000, alow = {34053.9841,226.6670652, -3.79308163,0.041707327, -5.70991327e-05,3.83415811e-08, -1.021969284e-11}, blow = {7088.1122,42.73091713}, ahigh = { -394387.271, -824.426353,10.61325843, -0.0002448749816,5.40606032e-08, -6.19546675e-12,2.870000149e-16}, bhigh = {8982.01173, -34.44666597}, R = 134.093789361808);
          constant IdealGases.Common.DataRecord NO3minus(name = "NO3minus", MM = 0.0620054486, Hf =  -5012132.61119766, H0 = 173744.489286704, Tlimit = 1000, alow = {92048.1361, -391.117115, -0.2354356764,0.02836042108, -3.46132408e-05,2.08178746e-08, -5.02160127e-12}, blow = { -35764.115,22.99942308}, ahigh = { -311000.5758, -1369.087552,11.01342913, -0.000403687882,8.90208647e-08, -1.01973348e-11,4.72333079e-16}, bhigh = { -33643.2109, -38.78432657}, R = 134.092602952315);
          constant IdealGases.Common.DataRecord NO3F(name = "NO3F", MM = 0.0810033032, Hf = 185177.633595564, H0 = 178315.641824345, Tlimit = 1000, alow = {64728.3203, -821.313431,6.19491744,0.01805438628, -1.99669324e-05,1.124482018e-08, -2.680013077e-12}, blow = {4206.66179, -7.016104301}, ahigh = { -341179.33, -2353.908798,16.28114887, -0.001910415273,4.69087356e-07, -5.68604014e-11,2.720906921e-15}, bhigh = {9760.58398, -65.58153684}, R = 102.643616637105);
          constant IdealGases.Common.DataRecord N2(name = "N2", MM = 0.0280134, Hf = 0, H0 = 309498.454311151, Tlimit = 1000, alow = {22103.71497, -381.846182,6.08273836, -0.00853091441,1.384646189e-05, -9.62579362e-09,2.519705809e-12}, blow = {710.846086, -10.76003744}, ahigh = {587712.406, -2239.249073,6.06694922, -0.00061396855,1.491806679e-07, -1.923105485e-11,1.061954386e-15}, bhigh = {12832.10415, -15.86640027}, R = 296.803386950531);
          constant IdealGases.Common.DataRecord N2plus(name = "N2plus", MM = 0.0280128514, Hf = 53886282.4938985, H0 = 309540.07059774, Tlimit = 1000, alow = { -34740.4747,269.6222703,3.16491637, -0.002132239781,6.7304764e-06, -5.63730497e-09,1.621756e-12}, blow = {179000.4424,6.832974166}, ahigh = { -2845599.002,7058.89303, -2.884886385,0.003068677059, -4.36165231e-07,2.102514545e-11,5.41199647e-16}, bhigh = {134038.8483,50.90897022}, R = 296.809199509051);
          constant IdealGases.Common.DataRecord N2minus(name = "N2minus", MM = 0.0280139486, Hf = 5289624.9691841, H0 = 309641.533360992, Tlimit = 1000, alow = { -81462.2711,906.360079, -0.1520054079,0.00602319084, -2.897138445e-06, -4.12910668e-11,3.20698977e-13}, blow = {12188.08548,26.38068855}, ahigh = {216963.7706, -1275.098516,5.3910957, -0.000319890751,7.31105135e-08, -8.20201737e-12,3.7400447e-16}, bhigh = {24249.64308, -9.014934294}, R = 296.797574619666);
          constant IdealGases.Common.DataRecord NCN(name = "NCN", MM = 0.0400241, Hf = 12503880.736856, H0 = 254351.178415005, Tlimit = 1000, alow = { -56346.807,732.380458, -0.782140184,0.01838552441, -1.950836491e-05,1.035712021e-08, -2.208158483e-12}, blow = {55397.897,29.05308985}, ahigh = { -164188.0975, -776.784075,7.99998187, -0.0001659081508,2.983403318e-08, -3.120157047e-12,1.99269872e-16}, bhigh = {61844.2448, -21.4910882}, R = 207.736638675198);
          constant IdealGases.Common.DataRecord N2D2_cis(name = "N2D2_cis", MM = 0.032041604, Hf = 6331060.39260706, H0 = 321707.89577201, Tlimit = 1000, alow = { -27437.33656,714.980883, -2.22324762,0.02088722282, -1.821711897e-05,8.84407994e-09, -1.918010649e-12}, blow = {20111.15649,36.37100195}, ahigh = {879807.471, -5299.36204,13.55007485, -0.001316635227,2.755816197e-07, -3.036294387e-11,1.365324117e-15}, bhigh = {53563.1189, -62.71215875}, R = 259.48988071883);
          constant IdealGases.Common.DataRecord N2F2(name = "N2F2", MM = 0.0660102064, Hf = 944907.604470087, H0 = 194951.003819312, Tlimit = 1000, alow = {15438.9315, -218.363513,3.89028425,0.0167401774, -2.05639309e-05,1.25869211e-08, -3.11049829e-12}, blow = {7052.0387,5.265866442}, ahigh = { -182488.386, -953.402996,10.6979548, -0.00027599687,6.0554397e-08, -6.91121508e-12,3.19258723e-16}, bhigh = {9283.46696, -32.46968772}, R = 125.95737013178);
          constant IdealGases.Common.DataRecord N2F4(name = "N2F4", MM = 0.1040070128, Hf =  -211524.198299059, H0 = 171253.558010081, Tlimit = 1000, alow = {116291.4512, -1538.660418,9.05403305,0.02862113563, -4.33228699e-05,3.067642499e-08, -8.45547411e-12}, blow = {2865.277526, -24.76493006}, ahigh = { -518859.471, -670.225651,16.50109262, -0.0002006404436,4.43675251e-08, -5.08980888e-12,2.359374159e-16}, bhigh = { -5281.4889, -60.40513435}, R = 79.9414556399989);
          constant IdealGases.Common.DataRecord N2H2(name = "N2H2", MM = 0.03002928, Hf = 7055072.78229781, H0 = 332907.115988129, Tlimit = 1000, alow = { -150400.5163,2346.687716, -9.40543029,0.032842998, -3.121920401e-05,1.72128319e-08, -4.01453722e-12}, blow = {13193.84041,78.3238263}, ahigh = {6217567.87, -17539.52096,20.22730509, -0.000975729766, -4.20841674e-07,1.117921171e-10, -7.62710221e-15}, bhigh = {137415.2574, -119.9559168}, R = 276.878832925731);
          constant IdealGases.Common.DataRecord NH2NO2(name = "NH2NO2", MM = 0.06202808, Hf =  -419164.997530151, H0 = 196102.362027005, Tlimit = 1000, alow = { -45730.3506,1201.365987, -8.10598411,0.054027152, -6.43807445e-05,4.02509792e-08, -1.02515419e-11}, blow = { -9615.78516,68.67353357}, ahigh = {1654040.575, -8125.22088,20.21742772, -0.001244291821,2.122804183e-07, -1.948359653e-11,7.43935136e-16}, bhigh = {42308.2258, -101.6190179}, R = 134.043678282481);
          constant IdealGases.Common.DataRecord N2H4(name = "N2H4", MM = 0.03204516, Hf = 2970183.32877726, H0 = 357286.529385405, Tlimit = 1000, alow = { -166075.6354,3035.416736, -17.36889823,0.0715983402, -8.8667993e-05,5.79897028e-08, -1.530037218e-11}, blow = { -3731.92723,119.0002218}, ahigh = {3293486.7, -11998.50628,21.04406814, -0.001399381724,1.933173351e-07, -1.318016127e-11,3.16640017e-16}, bhigh = {83484.337, -115.5751024}, R = 259.461085543027);
          constant IdealGases.Common.DataRecord N2O(name = "N2O", MM = 0.0440128, Hf = 1854006.10731424, H0 = 217685.196124764, Tlimit = 1000, alow = {42882.2597, -644.011844,6.03435143,0.0002265394436,3.47278285e-06, -3.62774864e-09,1.137969552e-12}, blow = {11794.05506, -10.0312857}, ahigh = {343844.804, -2404.557558,9.12563622, -0.000540166793,1.315124031e-07, -1.4142151e-11,6.38106687e-16}, bhigh = {21986.32638, -31.47805016}, R = 188.910316998691);
          constant IdealGases.Common.DataRecord N2Oplus(name = "N2Oplus", MM = 0.0440122514, Hf = 30286033.1975655, H0 = 241373.859824858, Tlimit = 1000, alow = { -56241.4708,669.621161,0.0878145619,0.01524476027, -1.527290811e-05,7.82723739e-09, -1.646739623e-12}, blow = {155729.5192,25.62354785}, ahigh = { -29835.53254, -1179.455967,8.30018669, -0.0002887267217,5.70510501e-08, -5.95888512e-12,2.835725557e-16}, bhigh = {164602.1769, -22.87356617}, R = 188.912671711223);
          constant IdealGases.Common.DataRecord N2O3(name = "N2O3", MM = 0.0760116, Hf = 1139702.29543912, H0 = 225240.029153445, Tlimit = 1000, alow = { -92044.4417,929.552015,3.20366481,0.01356473078, -6.26296607e-06, -1.402915559e-09,1.43162093e-12}, blow = {3313.62208,18.44430953}, ahigh = {778388.186, -4483.02466,16.66668024, -0.002062143878,5.30954171e-07, -6.19045122e-11,2.692956658e-15}, bhigh = {33609.1245, -67.39212388}, R = 109.384251877345);
          constant IdealGases.Common.DataRecord N2O4(name = "N2O4", MM = 0.092011, Hf = 120756.420427992, H0 = 181948.147504103, Tlimit = 1000, alow = { -38047.5144,561.282889, -0.2083648324,0.0388708782, -4.42241226e-05,2.49881231e-08, -5.67910238e-12}, blow = { -3310.79473,29.6392484}, ahigh = { -458284.376, -1604.749805,16.74102133, -0.000509138508,1.14363467e-07, -1.316288176e-11,5.97631662e-16}, bhigh = {4306.90052, -65.6945038}, R = 90.3638912738694);
          constant IdealGases.Common.DataRecord N2O5(name = "N2O5", MM = 0.1080104, Hf = 123136.290579426, H0 = 192550.208128106, Tlimit = 1000, alow = {40078.2817, -876.967512,10.55932981,0.01394613859, -8.88434692e-06,8.50043115e-10,7.79155091e-13}, blow = {3038.962037, -23.8683186}, ahigh = { -53255.7896, -3109.277389,20.36088958, -0.000995990114,2.401398635e-07, -3.057161911e-11,1.495915511e-15}, bhigh = {13369.57281, -82.98623341}, R = 76.9784391132706);
          constant IdealGases.Common.DataRecord N3(name = "N3", MM = 0.0420201, Hf = 10375986.7301601, H0 = 227769.710210114, Tlimit = 1000, alow = {33374.0679, -296.5683604,3.31427915,0.00672168536, -4.18112639e-06,8.61844236e-10,6.88335253e-14}, blow = {52988.4062,5.312776486}, ahigh = {252926.4658, -2362.876591,9.13526713, -0.000621287085,1.324094351e-07, -1.47898964e-11,6.72123047e-16}, bhigh = {64126.9539, -31.35825973}, R = 197.868924633687);
          constant IdealGases.Common.DataRecord N3H(name = "N3H", MM = 0.04302804, Hf = 6832753.71130082, H0 = 254419.745821562, Tlimit = 1000, alow = {3242.57606,66.9266489,1.766142217,0.01487411419, -1.53908644e-05,9.17230355e-09, -2.337205474e-12}, blow = {33920.697,15.13752057}, ahigh = {1170469.241, -5102.45199,12.7828891, -0.000840948716,1.592142834e-07, -1.512289051e-11,6.10290663e-16}, bhigh = {64283.4447, -55.13119108}, R = 193.233807535737);
          constant IdealGases.Common.DataRecord Na(name = "Na", MM = 0.02298977, Hf = 4675992.84377356, H0 = 269573.292816761, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {12183.82949,4.24402818}, ahigh = {952572.338, -2623.807254,5.16259662, -0.001210218586,2.306301844e-07, -1.249597843e-11,7.22677119e-16}, bhigh = {29129.63564, -15.19717061}, R = 361.659642527959);
          constant IdealGases.Common.DataRecord Naplus(name = "Naplus", MM = 0.0229892214, Hf = 26514291.9542286, H0 = 269579.725740516, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {72565.3707,3.55084508}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = {72565.3707,3.55084508}, R = 361.668272941162);
          constant IdealGases.Common.DataRecord Naminus(name = "Naminus", MM = 0.0229903186, Hf = 2107557.9178794, H0 = 269566.860200015, Tlimit = 1000, alow = {0,0,2.500000001,0,0,0,0}, blow = {5082.19967,3.55091679}, ahigh = {0,0,2.500000001,0,0,0,0}, bhigh = {5082.19967,3.55091679}, R = 361.651012526638);
          constant IdealGases.Common.DataRecord NaALF4(name = "NaALF4", MM = 0.1259649208, Hf =  -14748880.3565381, H0 = 165194.141891605, Tlimit = 1000, alow = {131535.9536, -2394.685695,19.09539612,0.001044339733, -6.94975413e-06,6.76909814e-09, -2.176199098e-12}, blow = { -215051.2366, -75.0510213}, ahigh = { -357117.793, -249.6543879,16.18634511, -7.4490493e-05,1.644510872e-08, -1.883696066e-12,8.71987745e-17}, bhigh = { -227953.3538, -53.7066514}, R = 66.006249574842);
          constant IdealGases.Common.DataRecord NaBO2(name = "NaBO2", MM = 0.06579957, Hf =  -9626952.42537299, H0 = 207944.671978859, Tlimit = 1000, alow = {50468.6736, -869.99706,8.94220824,0.001092655824,1.440608828e-06, -2.254633535e-09,8.12331285e-13}, blow = { -73783.1836, -19.51815135}, ahigh = {85958.1405, -1691.902222,11.18847517, -0.000456420231,9.80450516e-08, -1.101655008e-11,5.02917276e-16}, bhigh = { -69492.9206, -34.1635313}, R = 126.360582599552);
          constant IdealGases.Common.DataRecord NaBr(name = "NaBr", MM = 0.10289377, Hf =  -1418247.98527647, H0 = 95449.1025064005, Tlimit = 1000, alow = { -14668.27136,76.2228287,3.92116347,0.001793780908, -2.457391648e-06,1.739334522e-09, -4.79670138e-13}, blow = { -19264.91755,6.40438286}, ahigh = {897851.302, -2697.899721,7.54537716, -0.001566692184,4.47039274e-07, -4.83281021e-11,1.4239613e-15}, bhigh = { -1750.862247, -18.56478949}, R = 80.8063695207203);
          constant IdealGases.Common.DataRecord NaCN(name = "NaCN", MM = 0.04900717, Hf = 1923514.45717025, H0 = 247432.610371095, Tlimit = 1000, alow = {24461.6541, -753.601997,10.44966241, -0.01168769048,1.849279385e-05, -1.319387974e-08,3.58976656e-12}, blow = {12978.20493, -29.88519681}, ahigh = {366557.029, -1782.094358,8.69962263, -0.0004463297,9.36222931e-08, -1.03307132e-11,4.65046681e-16}, bhigh = {20109.26347, -24.19276094}, R = 169.658276533821);
          constant IdealGases.Common.DataRecord NaCL(name = "NaCL", MM = 0.05844277, Hf =  -3106370.28327028, H0 = 164521.907500278, Tlimit = 1000, alow = {43623.7835, -758.303446,8.259173, -0.00964091514,1.358854616e-05, -9.66703225e-09,2.74626129e-12}, blow = { -19504.09477, -19.36687551}, ahigh = {331449.876, -896.831565,5.27728738, -0.0001475674008, -1.491128988e-08,2.465673596e-11, -2.730355213e-15}, bhigh = { -17362.77667, -3.99828856}, R = 142.26690487121);
          constant IdealGases.Common.DataRecord NaF(name = "NaF", MM = 0.0419881732, Hf =  -7029524.42332023, H0 = 219707.367502238, Tlimit = 1000, alow = {39598.8744, -653.462607,6.9411732, -0.00530781493,6.97972066e-06, -4.82042573e-09,1.364849175e-12}, blow = { -33529.4089, -14.03212229}, ahigh = { -1092926.912,3293.30364,0.413591984,0.00263499447, -8.38429563e-07,1.417053025e-10, -8.60027016e-15}, bhigh = { -57728.8463,29.09489906}, R = 198.019379418965);
          constant IdealGases.Common.DataRecord NaH(name = "NaH", MM = 0.02399771, Hf = 5868689.34577508, H0 = 363830.757184748, Tlimit = 1000, alow = { -32222.0641,623.762201, -0.921627751,0.01360765851, -1.659249878e-05,1.033284544e-08, -2.589726392e-12}, blow = {13073.81654,26.41418597}, ahigh = { -4756184.75,14520.47626, -13.27563485,0.01055828277, -2.990041189e-06,3.90532288e-10, -1.923931194e-14}, bhigh = { -76329.2227,122.207006}, R = 346.469392287847);
          constant IdealGases.Common.DataRecord NaI(name = "NaI", MM = 0.14989424, Hf =  -604678.872250195, H0 = 66394.279059689, Tlimit = 1000, alow = {12288.68506, -285.71928,5.961057, -0.00379253258,5.56688131e-06, -4.04501368e-09,1.172998933e-12}, blow = { -10882.50556, -3.99073992}, ahigh = {2281549.408, -7093.15663,13.04994968, -0.00501349233,1.581735155e-06, -2.285837754e-10,1.19588995e-14}, bhigh = {32538.9362, -56.4002332}, R = 55.4689226217098);
          constant IdealGases.Common.DataRecord NaLi(name = "NaLi", MM = 0.02993077, Hf = 5967047.28946165, H0 = 333874.470987549, Tlimit = 1000, alow = { -6569.99276, -5.25394043,4.32938426,0.001097919189, -1.965726597e-06,2.086472026e-09, -8.19598811e-13}, blow = {20162.24293,1.413253322}, ahigh = {10916648.6, -34800.1064,46.1431192, -0.02260951051,5.68532437e-06, -6.4581433e-10,2.696508992e-14}, bhigh = {239443.1755, -296.1780126}, R = 277.790113652272);
          constant IdealGases.Common.DataRecord NaNO2(name = "NaNO2", MM = 0.06899527, Hf =  -2410205.49669564, H0 = 214612.11761328, Tlimit = 1000, alow = { -69344.6061,1115.43267, -1.816284973,0.02916366016, -3.50578852e-05,2.128602855e-08, -5.22621199e-12}, blow = { -27072.75053,41.21853253}, ahigh = { -176555.7495, -863.521295,10.6410594, -0.0002559465044,5.65446563e-08, -6.48671032e-12,3.008171243e-16}, bhigh = { -18684.35672, -29.21304474}, R = 120.50785510369);
          constant IdealGases.Common.DataRecord NaNO3(name = "NaNO3", MM = 0.08499467, Hf =  -3359371.95826515, H0 = 181202.997787979, Tlimit = 1000, alow = { -20438.52507,691.709341, -2.397485012,0.0410038643, -5.14170258e-05,3.22862247e-08, -8.1646916e-12}, blow = { -39064.0132,41.74749459}, ahigh = { -322559.219, -1396.782195,14.03092409, -0.000409816177,9.02384572e-08, -1.032543966e-11,4.77867155e-16}, bhigh = { -31388.89875, -49.59121431}, R = 97.8234517529158);
          constant IdealGases.Common.DataRecord NaO(name = "NaO", MM = 0.03898917, Hf = 2731664.12621761, H0 = 250149.387637644, Tlimit = 1000, alow = {18577.48013, -337.149732,5.64456002, -0.003136926368,6.33077539e-06, -5.42946247e-09,1.68718377e-12}, blow = {13203.32678, -4.99613115}, ahigh = {256974.4011, -2269.334161,9.22439762, -0.0036512691,1.446811119e-06, -2.443068386e-10,1.428508328e-14}, bhigh = {24132.39357, -29.89159486}, R = 213.25080785254);
          constant IdealGases.Common.DataRecord NaOH(name = "NaOH", MM = 0.03999711, Hf =  -4775345.0186776, H0 = 284962.363530765, Tlimit = 1000, alow = {34420.3674, -792.321818,8.9979323, -0.00407984452,3.065783937e-06, -5.11918934e-10, -1.541016409e-13}, blow = { -20869.51091, -25.1059009}, ahigh = {875378.776, -2342.514649,7.97846989,0.0001016451512, -6.26853195e-08,1.022715136e-11, -5.71328641e-16}, bhigh = { -9509.90171, -22.02310401}, R = 207.87681910018);
          constant IdealGases.Common.DataRecord NaOHplus(name = "NaOHplus", MM = 0.0399965614, Hf = 17098029.1070722, H0 = 292396.910900445, Tlimit = 1000, alow = {22780.39363, -667.219422,8.92159312, -0.00448126327,3.95188392e-06, -1.173341234e-09,2.041806631e-14}, blow = {83633.8219, -22.58912262}, ahigh = {881541.365, -2363.159755,8.05320393,7.27127974e-05, -5.89741789e-08,1.007487881e-11, -5.47440621e-16}, bhigh = {95844.4165, -20.79484012}, R = 207.879670375864);
          constant IdealGases.Common.DataRecord Na2(name = "Na2", MM = 0.04597954, Hf = 3095705.72041391, H0 = 226255.525827357, Tlimit = 1000, alow = {6848.62868, -153.0836599,5.32523039, -0.001944906088,2.657477888e-06, -9.09684112e-10, -2.44875673e-13}, blow = {16491.70574, -2.653564394}, ahigh = {19299407.58, -62692.8012,82.6768211, -0.0456513781,1.259515667e-05, -1.560445735e-09,7.02467717e-14}, bhigh = {409082.08, -550.997089}, R = 180.82982126398);
          constant IdealGases.Common.DataRecord Na2Br2(name = "Na2Br2", MM = 0.20578754, Hf =  -2336625.69172069, H0 = 95202.7853581417, Tlimit = 1000, alow = { -13849.46731, -157.3241177,10.63192589, -0.001383418339,1.697156537e-06, -1.092992547e-09,2.87098802e-13}, blow = { -60103.9182, -18.61473856}, ahigh = { -30217.30975, -2.65923278,10.00225819, -9.93549063e-07,2.358676253e-10, -2.857194816e-14,1.381909879e-18}, bhigh = { -60900.3992, -14.94517863}, R = 40.4031847603601);
          constant IdealGases.Common.DataRecord Na2CL2(name = "Na2CL2", MM = 0.11688554, Hf =  -4828670.70640218, H0 = 159979.814440691, Tlimit = 1000, alow = { -10829.5525, -313.9528641,11.24411332, -0.002697665795,3.28635948e-06, -2.105377639e-09,5.50805661e-13}, blow = { -69386.7543, -25.11011582}, ahigh = { -44121.0523, -5.39349128,10.00453407, -1.981155641e-06,4.6801882e-10, -5.64894068e-14,2.724721177e-18}, bhigh = { -70980.6341, -17.87230397}, R = 71.133452435605);
          constant IdealGases.Common.DataRecord Na2F2(name = "Na2F2", MM = 0.0839763464, Hf =  -9932115.55105236, H0 = 198655.106052578, Tlimit = 1000, alow = {23515.80802, -1000.737062,13.75845076, -0.00783834698,9.2762887e-06, -5.81361297e-09,1.49535482e-12}, blow = { -98358.3188, -43.8114809}, ahigh = { -90307.5069, -18.93938035,10.01535518, -6.53967545e-06,1.516333603e-09, -1.804877459e-13,8.61347492e-18}, bhigh = { -103489.6664, -21.78860202}, R = 99.0096897094823);
          constant IdealGases.Common.DataRecord Na2I2(name = "Na2I2", MM = 0.29978848, Hf =  -1190405.75541795, H0 = 67534.5096649478, Tlimit = 1000, alow = { -13050.63983, -89.9250668,10.36367976, -0.000800007019,9.84877798e-07, -6.35937312e-10,1.673774775e-13}, blow = { -45514.6549, -14.87509548}, ahigh = { -22312.36397, -1.515570558,10.00129413, -5.71503538e-07,1.360287207e-10, -1.650916036e-14,7.99614968e-19}, bhigh = { -45969.2175, -12.76523965}, R = 27.7344613108549);
          constant IdealGases.Common.DataRecord Na2O(name = "Na2O", MM = 0.06197894, Hf =  -267184.788897648, H0 = 232501.072138375, Tlimit = 1000, alow = {39011.4929, -726.620789,9.62371078, -0.00355641864,3.47070435e-06, -1.835177736e-09,4.06213471e-13}, blow = { -459.307325, -23.49565832}, ahigh = { -66005.2516, -25.69021634,7.51938542, -7.81163524e-06,1.73501589e-09, -1.996634558e-13,9.27643355e-18}, bhigh = { -4297.33965, -10.63530214}, R = 134.14995480723);
          constant IdealGases.Common.DataRecord Na2Oplus(name = "Na2Oplus", MM = 0.0619783914, Hf = 8403476.69946142, H0 = 235671.944206025, Tlimit = 1000, alow = {26479.69755, -596.906216,9.29412953, -0.003080702005,3.080460653e-06, -1.669452593e-09,3.78978414e-13}, blow = {63473.1025, -20.25274459}, ahigh = { -57409.0933, -19.89178143,7.51529526, -6.25833323e-06,1.407321045e-09, -1.635833952e-13,7.66244513e-18}, bhigh = {60329.7747, -9.42652674}, R = 134.151142231807);
          constant IdealGases.Common.DataRecord Na2O2(name = "Na2O2", MM = 0.07797834, Hf =  -1589291.91106146, H0 = 199609.981438435, Tlimit = 1000, alow = {73824.5892, -1355.12534,12.55579861, -0.002112046045,4.92035352e-08,1.003950603e-09, -4.44773207e-13}, blow = { -10588.58918, -40.2181178}, ahigh = { -173229.5239, -113.7561118,10.08529143, -3.42209894e-05,7.57772189e-09, -8.70125673e-13,4.03606139e-17}, bhigh = { -17803.00574, -23.8678919}, R = 106.62540392627);
          constant IdealGases.Common.DataRecord Na2O2H2(name = "Na2O2H2", MM = 0.07999422, Hf =  -7800563.59071943, H0 = 241844.235746033, Tlimit = 1000, alow = {108941.4289, -2634.822704,23.06608858, -0.01689310193,1.67348319e-05, -7.82116601e-09,1.47503447e-12}, blow = { -65931.3733, -98.0298758}, ahigh = {1675713.839, -4704.92117,16.97343934,0.0001961487616, -1.236951748e-07,2.025311778e-11, -1.132991623e-15}, bhigh = { -48562.4557, -68.1348098}, R = 103.93840955009);
          constant IdealGases.Common.DataRecord Na2SO4(name = "Na2SO4", MM = 0.14204214, Hf =  -7322702.27694401, H0 = 147644.1991088, Tlimit = 1000, alow = {83442.8321, -1210.880769,8.742353,0.0360079663, -5.11764596e-05,3.48156904e-08, -9.32193122e-12}, blow = { -121738.7045, -20.61505785}, ahigh = { -575448.311, -981.706034,19.73310397, -0.000293639552,6.49870817e-08, -7.4624794e-12,3.46249166e-16}, bhigh = { -127059.7762, -76.67871596}, R = 58.5352487649088);
          constant IdealGases.Common.DataRecord Na3CL3(name = "Na3CL3", MM = 0.17532831, Hf =  -5205517.06110667, H0 = 168125.484127464, Tlimit = 1000, alow = { -12542.1667, -550.216046,18.16110649, -0.00465651526,5.64643822e-06, -3.60476032e-09,9.40555624e-13}, blow = { -111927.0172, -52.9010496}, ahigh = { -71602.9797, -9.57349381,16.00799614, -3.47835697e-06,8.19094369e-10, -9.86321003e-14,4.74900535e-18}, bhigh = { -114725.2948, -40.3134216}, R = 47.4223016237366);
          constant IdealGases.Common.DataRecord Na3F3(name = "Na3F3", MM = 0.1259645196, Hf =  -10701548.2635953, H0 = 203249.884025279, Tlimit = 1000, alow = {52470.3035, -1775.597273,22.50927609, -0.01333033615,1.555790239e-05, -9.64601146e-09,2.460254271e-12}, blow = { -158073.6389, -85.8012237}, ahigh = { -155399.5698, -35.5197019,16.02841159, -1.198056094e-05,2.75724873e-09, -3.26330097e-13,1.550483212e-17}, bhigh = { -167214.1773, -47.5378068}, R = 66.0064598063215);
          constant IdealGases.Common.DataRecord Nb(name = "Nb", MM = 0.09290638, Hf = 7783244.7997651, H0 = 89919.5405094892, Tlimit = 1000, alow = {78896.6067, -1212.813914,10.34579819, -0.01676630056,1.979119979e-05, -1.218224409e-08,3.058098336e-12}, blow = {91653.1514, -35.9474285}, ahigh = { -1096553.196,2546.650713,2.236054882, -0.001280029198,8.46423799e-07, -1.486269508e-10,8.714309406e-15}, bhigh = {68791.2455,13.9816903}, R = 89.4930143656442);
          constant IdealGases.Common.DataRecord Nbplus(name = "Nbplus", MM = 0.0929058314, Hf = 15000185.1767509, H0 = 92449.2560969644, Tlimit = 1000, alow = {131444.7859, -2000.135035,15.05024212, -0.02996583942,3.72986863e-05, -2.269869569e-08,5.449089902e-12}, blow = {176005.4029, -62.2459552}, ahigh = { -1077639.646,2159.046421,2.310604767, -0.000536399176,5.05791509e-07, -1.032401533e-10,6.62924128e-15}, bhigh = {151794.5546,12.10678502}, R = 89.4935428132878);
          constant IdealGases.Common.DataRecord Nbminus(name = "Nbminus", MM = 0.0929069286, Hf = 6792324.5070013, H0 = 93144.9153513401, Tlimit = 1000, alow = { -72209.2485,525.950125,3.47046839, -0.00495050553,7.40185903e-06, -5.01630207e-09,1.314100719e-12}, blow = {71788.2887,5.15551007}, ahigh = {111745.8019,134.0072834,2.391474129,4.52481343e-05, -1.025345e-08,1.195577988e-12, -5.60633045e-17}, bhigh = {74739.9675,9.67531561}, R = 89.4924859242414);
          constant IdealGases.Common.DataRecord NbCL5(name = "NbCL5", MM = 0.27017138, Hf =  -2603273.52216212, H0 = 97505.3797334122, Tlimit = 1000, alow = {73482.7797, -1919.172996,22.90799567, -0.01404352774,1.638106985e-05, -1.018941843e-08,2.612266639e-12}, blow = { -79741.1881, -84.39637087}, ahigh = { -156381.6638, -44.5052128,16.0354463, -1.489949737e-05,3.42077792e-09, -4.04112442e-13,1.917266128e-17}, bhigh = { -89628.1512, -43.74582607}, R = 30.774806717129);
          constant IdealGases.Common.DataRecord NbO(name = "NbO", MM = 0.10890578, Hf = 1937350.85502349, H0 = 80621.1203849786, Tlimit = 1000, alow = { -6797.83436,283.9767438,0.56457084,0.01134928619, -1.549821141e-05,1.047624988e-08, -2.762937835e-12}, blow = {23179.93893,23.65708974}, ahigh = {553225.878, -1287.669306,4.98006604,0.0001116014163,4.03183868e-08,1.04877371e-11, -1.893595022e-15}, bhigh = {32684.5779, -1.868958549}, R = 76.3455530092159);
          constant IdealGases.Common.DataRecord NbOCL3(name = "NbOCL3", MM = 0.21526478, Hf =  -3494765.84139774, H0 = 95840.1090972708, Tlimit = 1000, alow = { -110848.2359,117.3066983,11.65558433,0.001287422064, -1.659778281e-06,1.102107826e-09, -2.949903264e-13}, blow = { -95040.2392, -23.86450092}, ahigh = { -151555.1427,53.0554523,12.28915282, -0.0001527366571,6.77857644e-08, -9.36517065e-12,4.67003693e-16}, bhigh = { -94907.6812, -27.45263834}, R = 38.6243955002765);
          constant IdealGases.Common.DataRecord NbO2(name = "NbO2", MM = 0.12490518, Hf =  -1611356.326455, H0 = 85299.5208045015, Tlimit = 1000, alow = {17390.46582,38.8102944,0.902429912,0.01909523319, -2.667013583e-05,1.813901051e-08, -4.87236588e-12}, blow = { -25285.18558,22.89323379}, ahigh = { -685185.99,911.224932,6.33919984, -8.2300803e-05,1.984439605e-07, -3.26117149e-11,1.602430854e-15}, bhigh = { -33250.2732, -3.55955655}, R = 66.5662705101582);
          constant IdealGases.Common.DataRecord Ne(name = "Ne", MM = 0.0201797, Hf = 0, H0 = 307111.998691755, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = { -745.375,3.35532272}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = { -745.375,3.35532272}, R = 412.021586049347);
          constant IdealGases.Common.DataRecord Neplus(name = "Neplus", MM = 0.0201791514, Hf = 103421888.49428, H0 = 312412.245442591, Tlimit = 1000, alow = {72815.5148, -869.569799,6.10864697, -0.00584135693,5.04104417e-06, -2.293759207e-09,4.33906568e-13}, blow = {254599.689, -16.73449355}, ahigh = { -111274.2658,476.569797,2.196650531,0.0001102593151, -2.287564425e-08,2.510218183e-12, -1.126646096e-16}, bhigh = {247253.6944,7.46614054}, R = 412.032787463996);
          constant IdealGases.Common.DataRecord Ni(name = "Ni", MM = 0.0586934, Hf = 7328193.71513662, H0 = 116282.461060358, Tlimit = 1000, alow = { -32358.1055,601.526462, -1.079270657,0.01089505519, -1.369578748e-05,8.31772579e-09, -2.019206968e-12}, blow = {48138.1081,27.188292}, ahigh = { -493826.221,1092.909991,2.410485014, -1.599071827e-05, -1.047414069e-08,4.62479521e-12, -4.448865218e-17}, bhigh = {43360.7217,9.6771956}, R = 141.659402931164);
          constant IdealGases.Common.DataRecord Niplus(name = "Niplus", MM = 0.0586928514, Hf = 19978490.4810401, H0 = 105733.029695674, Tlimit = 1000, alow = { -89693.8603,1173.6015, -3.41062041,0.01390739137, -1.501714923e-05,7.8963379e-09, -1.648686761e-12}, blow = {134558.95,40.3149516}, ahigh = { -3961999.32,10170.84853, -6.02933129,0.002770858029, -8.9020777e-08, -5.54100058e-11,5.235342833e-15}, bhigh = {73403.9512,71.375031}, R = 141.660727016578);
          constant IdealGases.Common.DataRecord Niminus(name = "Niminus", MM = 0.0586939486, Hf = 5311695.06288763, H0 = 105754.633110508, Tlimit = 1000, alow = { -84376.2475,1135.476552, -3.38061583,0.01423003786, -1.582586302e-05,8.60884041e-09, -1.875316029e-12}, blow = {31243.0759,39.980613}, ahigh = { -543342.48,1182.64533,2.12644124,3.73045594e-05,6.95360843e-09, -1.945719381e-12,1.271571579e-16}, bhigh = {28547.59122,10.43462235}, R = 141.658078870502);
          constant IdealGases.Common.DataRecord NiCL(name = "NiCL", MM = 0.0941464, Hf = 1933201.90681747, H0 = 100534.072465862, Tlimit = 1000, alow = { -23579.97085,220.549163,2.714260287,0.00445359847, -2.849162229e-06, -1.691898007e-10,5.15669926e-13}, blow = {19572.2949,14.23643068}, ahigh = { -3905769.19,9961.53399, -3.83943234,0.002767979391, -7.00595938e-08, -5.74140366e-11,5.3076358e-15}, bhigh = { -45053.9705,67.69402548}, R = 88.3142849859368);
          constant IdealGases.Common.DataRecord NiCL2(name = "NiCL2", MM = 0.1295994, Hf =  -570457.887922321, H0 = 109618.917988818, Tlimit = 1000, alow = {71099.7653, -1218.958288,12.08750596, -0.00867440314,1.043608104e-05, -6.29849301e-09,1.485857653e-12}, blow = { -5006.99871, -34.99472036}, ahigh = {158588.9817, -1161.488738,9.60801538, -0.000946122451,2.608172043e-07, -3.26463725e-11,1.525950893e-15}, bhigh = { -4578.98554, -22.12397347}, R = 64.1551735579023);
          constant IdealGases.Common.DataRecord NiO(name = "NiO", MM = 0.0746928, Hf = 3977143.71398582, H0 = 118707.144463723, Tlimit = 1000, alow = {23206.30462, -190.7136094,3.19017862,0.00529138349, -8.08021043e-06,5.83981276e-09, -1.639499596e-12}, blow = {35767.1112,7.84126567}, ahigh = { -72340.9194, -80.3555427,4.55922102,2.30515451e-05,5.13983282e-09, -5.85347491e-13,2.697077329e-17}, bhigh = {34611.4385,1.209414814}, R = 111.315575262944);
          constant IdealGases.Common.DataRecord NiS(name = "NiS", MM = 0.0907584, Hf = 3938136.85565193, H0 = 101572.163017418, Tlimit = 1000, alow = { -12724.88974,143.2801381,2.471951556,0.00550527112, -3.41835528e-06, -3.099052241e-10,6.26815467e-13}, blow = {41177.1608,15.17981835}, ahigh = { -735812.898,1178.231095,4.83786558, -0.000393120139,1.341974852e-07, -1.569591708e-11,6.67704506e-16}, bhigh = {32755.6594,3.690652283}, R = 91.6110464706297);
          constant IdealGases.Common.DataRecord O(name = "O", MM = 0.0159994, Hf = 15574021.7133143, H0 = 420353.450754403, Tlimit = 1000, alow = { -7953.6113,160.7177787,1.966226438,0.00101367031, -1.110415423e-06,6.5175075e-10, -1.584779251e-13}, blow = {28403.62437,8.40424182}, ahigh = {261902.0262, -729.872203,3.31717727, -0.000428133436,1.036104594e-07, -9.43830433e-12,2.725038297e-16}, bhigh = {33924.2806, -0.667958535}, R = 519.673987774542);
          constant IdealGases.Common.DataRecord Oplus(name = "Oplus", MM = 0.0159988514, Hf = 98056240.9623981, H0 = 387367.058112684, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {187935.2842,4.39337676}, ahigh = { -216651.3208,666.545615,1.702064364,0.000471499281, -1.427131823e-07,2.016595903e-11, -9.107157762e-16}, bhigh = {183719.1966,10.05690382}, R = 519.691807375622);
          constant IdealGases.Common.DataRecord Ominus(name = "Ominus", MM = 0.0159999486, Hf = 6365407.44887143, H0 = 410675.81929607, Tlimit = 1000, alow = { -5695.85711,109.9287334,2.184719661,0.00053263598, -5.29887844e-07,2.870216236e-10, -6.52469274e-14}, blow = {10932.87498,6.72986386}, ahigh = {9769.36318,7.15960478,2.494961726,1.968240938e-06, -4.30417485e-10,4.91208308e-14, -2.271600083e-18}, bhigh = {11495.54438,4.83703644}, R = 519.656169395444);
          constant IdealGases.Common.DataRecord OD(name = "OD", MM = 0.018013502, Hf = 1952535.8256268, H0 = 0, Tlimit = 1000, alow = {21186.91536, -278.598236,5.45621012, -0.00614811983,9.11767056e-06, -5.52781271e-09,1.239794711e-12}, blow = {0,4464.87825}, ahigh = {8999.108,783247.316, -2532.992554,5.95212465, -0.000374359528,4.95952762e-08,3.45445473e-12}, bhigh = { -7.3806268e-16,0}, R = 461.568883163307);
          constant IdealGases.Common.DataRecord ODminus(name = "ODminus", MM = 0.0180140506, Hf =  -8191266.8769788, H0 = 0, Tlimit = 1000, alow = {56061.2832, -751.415615,7.54418847, -0.01092083064,1.503688938e-05, -9.35166584e-09,2.246466046e-12}, blow = {0, -15157.0737}, ahigh = {8642.104,302946.7029, -1079.684654,4.21141738,0.000626083007, -2.411665054e-07,4.24842554e-11}, bhigh = { -2.387099102e-15, -11889.35343}, R = 461.554826541899);
          constant IdealGases.Common.DataRecord OH(name = "OH", MM = 0.01700734, Hf = 2191889.26663429, H0 = 518194.262006875, Tlimit = 1000, alow = { -1998.85899,93.0013616,3.050854229,0.001529529288, -3.157890998e-06,3.31544618e-09, -1.138762683e-12}, blow = {2991.214235,4.67411079}, ahigh = {1017393.379, -2509.957276,5.11654786,0.000130529993, -8.28432226e-08,2.006475941e-11, -1.556993656e-15}, bhigh = {20196.40206, -11.01282337}, R = 488.875509044918);
          constant IdealGases.Common.DataRecord OHplus(name = "OHplus", MM = 0.0170067914, Hf = 76393787.1902162, H0 = 505862.793142744, Tlimit = 1000, alow = {60316.3086, -757.35203,7.30775293, -0.00950688167,1.202555795e-05, -6.8290261e-09,1.501588659e-12}, blow = {158926.2158, -19.50106996}, ahigh = {504072.91, -1380.052958,4.1254622,0.000833194884, -3.44285629e-07,6.79285395e-11, -4.36387213e-15}, bhigh = {164383.9235, -3.99705849}, R = 488.891279045147);
          constant IdealGases.Common.DataRecord OHminus(name = "OHminus", MM = 0.0170078886, Hf =  -8540519.01539383, H0 = 506006.606840075, Tlimit = 1000, alow = {29108.80827, -321.690494,4.85102905, -0.002579035357,2.004980024e-06, -7.95685296e-11, -2.320495634e-13}, blow = { -16888.86234, -7.1215913}, ahigh = {471133.117, -857.233669,3.16618121,0.001233581296, -3.99924458e-07,6.23908167e-11, -3.35434322e-15}, bhigh = { -12248.49139,1.48773626}, R = 488.859740062032);
          constant IdealGases.Common.DataRecord O2(name = "O2", MM = 0.0319988, Hf = 0, H0 = 271263.422378339, Tlimit = 1000, alow = { -34255.6342,484.700097,1.119010961,0.00429388924, -6.83630052e-07, -2.0233727e-09,1.039040018e-12}, blow = { -3391.45487,18.4969947}, ahigh = { -1037939.022,2344.830282,1.819732036,0.001267847582, -2.188067988e-07,2.053719572e-11, -8.19346705e-16}, bhigh = { -16890.10929,17.38716506}, R = 259.836993887271);
          constant IdealGases.Common.DataRecord O2plus(name = "O2plus", MM = 0.0319982514, Hf = 36621639.7687281, H0 = 290988.150683759, Tlimit = 1000, alow = { -86072.0545,1051.875934, -0.543238047,0.00657116654, -3.27426375e-06,5.94064534e-11,3.23878479e-13}, blow = {134554.4668,29.0270975}, ahigh = {73846.5488, -845.955954,4.98516416, -0.000161101089,6.42708399e-08, -1.504939874e-11,1.578465409e-15}, bhigh = {144632.1044, -5.81123065}, R = 259.841448711163);
          constant IdealGases.Common.DataRecord O2minus(name = "O2minus", MM = 0.0319993486, Hf =  -1500903.0840084, H0 = 292181.291465415, Tlimit = 1000, alow = {18838.74344,114.9551768,1.518876821,0.00801611138, -9.85057103e-06,6.04419621e-09, -1.486439845e-12}, blow = { -7101.53876,15.0121038}, ahigh = { -56552.0805, -236.7815862,4.67583367, -2.1972453e-05,1.71150928e-08, -1.757645062e-12,8.24817279e-17}, bhigh = { -5960.17775, -2.436885556}, R = 259.832539216127);
          constant IdealGases.Common.DataRecord O3(name = "O3", MM = 0.0479982, Hf = 2954277.45207112, H0 = 215972.786479493, Tlimit = 1000, alow = { -12823.14507,589.821664, -2.547496763,0.02690121526, -3.52825834e-05,2.312290922e-08, -6.04489327e-12}, blow = {13483.68701,38.5221858}, ahigh = { -38696624.8,102334.4994, -89.615516,0.0370614497, -4.13763874e-06, -2.725018591e-10,5.24818811e-14}, bhigh = { -651791.818,702.910952}, R = 173.224662591514);
          constant IdealGases.Common.DataRecord P(name = "P", MM = 0.030973761, Hf = 10218326.4086011, H0 = 200086.389250566, Tlimit = 1000, alow = {50.4086657, -0.763941865,2.504563992, -1.381689958e-05,2.245585515e-08, -1.866399889e-11,6.227063395e-15}, blow = {37324.2191,5.359303481}, ahigh = {1261794.642, -4559.83819,8.91807931, -0.00438140146,1.454286224e-06, -2.030782763e-10,1.021022887e-14}, bhigh = {65417.2396, -39.15974795}, R = 268.435983605607);
          constant IdealGases.Common.DataRecord Pplus(name = "Pplus", MM = 0.0309732124, Hf = 43148687.4122233, H0 = 262867.793461423, Tlimit = 1000, alow = { -73169.0811,962.797914, -0.369393805,0.00476677834, -4.57476858e-06,2.371262331e-09, -5.1313149e-13}, blow = {154940.6849,23.76640762}, ahigh = { -559424.936,1722.576545,0.843038109,0.000628736894, -6.31719546e-08, -1.810842484e-12,4.31811257e-16}, bhigh = {149049.3431,18.45275207}, R = 268.440738165086);
          constant IdealGases.Common.DataRecord Pminus(name = "Pminus", MM = 0.0309743096, Hf = 7710479.04163778, H0 = 217853.604717634, Tlimit = 1000, alow = { -10089.49093,182.6468403,1.962456304,0.000919737754, -9.21499863e-07,5.01287236e-10, -1.142677121e-13}, blow = {27030.82432,9.478137822}, ahigh = {15434.88016,9.49500934,2.493170861,2.700167711e-06, -5.94921039e-10,6.82376629e-14, -3.16695991e-18}, bhigh = {27975.72693,6.246793872}, R = 268.431229214549);
          constant IdealGases.Common.DataRecord PCL(name = "PCL", MM = 0.066426761, Hf = 2026519.28189002, H0 = 139870.014134815, Tlimit = 1000, alow = {34888.617, -560.311934,6.26456848, -0.00318336437,3.49532684e-06, -2.091034291e-09,5.4187586e-13}, blow = {17746.53764, -8.074572148}, ahigh = { -347168.151,993.555793,3.39943027,0.00040197117,8.29558839e-08, -3.090212484e-11,2.106384858e-15}, bhigh = {8433.59115,10.64902238}, R = 125.167505909253);
          constant IdealGases.Common.DataRecord PCL2(name = "PCL2", MM = 0.101879761, Hf =  -532906.16769311, H0 = 120227.578861321, Tlimit = 1000, alow = {51900.845, -1060.9042,10.5854994, -0.00687991753,7.62077318e-06, -4.53011547e-09,1.11667166e-12}, blow = { -3220.33118, -27.53518359}, ahigh = { -83498.3397, -26.2524947,7.02036181, -8.38876868e-06,1.89678474e-09, -2.21487992e-13,1.04165355e-17}, bhigh = { -8742.99368, -6.23402319}, R = 81.6106351093619);
          constant IdealGases.Common.DataRecord PCL2minus(name = "PCL2minus", MM = 0.1018803096, Hf =  -3497096.29268736, H0 = 122234.090658869, Tlimit = 1000, alow = {49348.3277, -960.159054,9.8847697, -0.00495020295,4.94621787e-06, -2.678675201e-09,6.07672361e-13}, blow = { -39980.724, -24.13131695}, ahigh = { -85649.8676, -31.8555594,7.02449496, -1.002251172e-05,2.253747944e-09, -2.619657192e-13,1.227059658e-17}, bhigh = { -45038.1143, -6.721522361}, R = 81.6101956564922);
          constant IdealGases.Common.DataRecord PCL3(name = "PCL3", MM = 0.137332761, Hf =  -2108018.49385377, H0 = 116013.585425549, Tlimit = 1000, alow = {77177.4547, -1617.65086,15.3608047, -0.0101107705,1.10340834e-05, -6.47612102e-09,1.57915114e-12}, blow = { -29558.9364, -52.44338688}, ahigh = { -133307.693, -41.4588321,10.0318606, -1.30194616e-05,2.92291689e-09, -3.39186924e-13,1.58642553e-17}, bhigh = { -38003.3668, -20.50737678}, R = 60.5425241541601);
          constant IdealGases.Common.DataRecord PCL5(name = "PCL5", MM = 0.208238761, Hf =  -1805619.6559871, H0 = 111917.041227498, Tlimit = 1000, alow = {102907.02, -2515.70746,24.3011497, -0.0156461069,1.70987303e-05, -1.00601143e-08,2.46015879e-12}, blow = { -37225.8237, -98.1633517}, ahigh = { -225590.689, -69.3854282,16.0535963, -2.20215696e-05,4.97139229e-09, -5.79986005e-13,2.72597077e-17}, bhigh = { -50342.3422, -48.7245487}, R = 39.9275906179638);
          constant IdealGases.Common.DataRecord PF(name = "PF", MM = 0.0499721642, Hf =  -959429.990026327, H0 = 177921.191574088, Tlimit = 1000, alow = {22278.66728, -172.7508706,3.085366934,0.00548971823, -8.24797254e-06,5.86888291e-09, -1.617252113e-12}, blow = { -5809.27965,7.70971885}, ahigh = { -1132572.282,3200.03855,0.778050237,0.001969518902, -4.36581897e-07,5.24364857e-11, -2.618755712e-15}, bhigh = { -27667.73009,27.59312302}, R = 166.382067559123);
          constant IdealGases.Common.DataRecord PFplus(name = "PFplus", MM = 0.0499716156, Hf = 18040601.4329463, H0 = 188704.085044631, Tlimit = 1000, alow = { -17791.38157,390.999127,1.257434156,0.00786136375, -9.34231268e-06,5.63091565e-09, -1.370669924e-12}, blow = {105487.3882,19.01676576}, ahigh = { -40115.2518, -159.7504413,4.62139748, -2.378072183e-05,1.431831329e-08, -2.300109157e-12,1.694069882e-16}, bhigh = {107863.3756, -0.0407234395}, R = 166.383894140097);
          constant IdealGases.Common.DataRecord PFminus(name = "PFminus", MM = 0.0499727128, Hf =  -3282711.52011583, H0 = 190416.778814537, Tlimit = 1000, alow = {7705.21095, -115.2432706,4.25495231,0.001163521526, -1.627947259e-06,1.084382815e-09, -2.809490132e-13}, blow = { -20355.58966,2.217923793}, ahigh = { -130379.3388,271.5526386,4.13761981,0.0002728610614, -7.64441625e-08,1.182073001e-11, -6.08390664e-16}, bhigh = { -22907.99827,3.7485261}, R = 166.380241018254);
          constant IdealGases.Common.DataRecord PFCL(name = "PFCL", MM = 0.0854251642, Hf =  -3314996.48437316, H0 = 136147.013692249, Tlimit = 1000, alow = {54901.8853, -865.867948,7.94446765,0.00095037791, -3.33909021e-06,3.023977817e-09, -9.45037754e-13}, blow = { -31328.43217, -14.40426886}, ahigh = { -128214.3211, -97.3904355,7.07266166, -2.903614134e-05,6.40850449e-09, -7.33899377e-13,3.39672317e-17}, bhigh = { -36006.4926, -7.626493035}, R = 97.3304772412717);
          constant IdealGases.Common.DataRecord PFCLminus(name = "PFCLminus", MM = 0.0854257128, Hf =  -6195665.09487762, H0 = 137462.042927197, Tlimit = 1000, alow = {89343.3685, -1266.972513,9.91219453, -0.0038779283,3.036692411e-06, -1.29655938e-09,2.32422658e-13}, blow = { -58945.0571, -26.1705046}, ahigh = { -123156.515, -97.6586111,7.07402957, -2.997070692e-05,6.68528062e-09, -7.7222622e-13,3.5993823e-17}, bhigh = { -65588.1722, -8.208878059}, R = 97.3298521894218);
          constant IdealGases.Common.DataRecord PFCL2(name = "PFCL2", MM = 0.1208781642, Hf =  -4235052.23948462, H0 = 122997.897084211, Tlimit = 1000, alow = {82665.6056, -1512.138928,13.03404207, -0.002903247162,7.83665297e-07,6.45834066e-10, -3.73764182e-13}, blow = { -56442.3787, -41.00905668}, ahigh = { -186458.9813, -114.8004379,10.08599754, -3.44745652e-05,7.62797201e-09, -8.75309743e-13,4.05778835e-17}, bhigh = { -64504.7219, -21.74054563}, R = 68.7839036523025);
          constant IdealGases.Common.DataRecord PFCL4(name = "PFCL4", MM = 0.1917841642, Hf =  -3311099.44686455, H0 = 113324.580737204, Tlimit = 1000, alow = {141207.1573, -2882.47108,23.87345233, -0.01239510207,1.141290085e-05, -5.70548382e-09,1.195229703e-12}, blow = { -66134.7865, -98.5969355}, ahigh = { -295437.588, -132.5154294,16.1000538, -4.03535917e-05,8.97113074e-09, -1.03328881e-12,4.80446835e-17}, bhigh = { -81357.194, -50.6240392}, R = 43.3532770272385);
          constant IdealGases.Common.DataRecord PF2(name = "PF2", MM = 0.0689705674, Hf =  -7439458.2840564, H0 = 160669.21612711, Tlimit = 1000, alow = {55247.6572, -628.05178,5.13743792,0.00912067335, -1.46877376e-05,1.08126845e-08, -3.06494449e-12}, blow = { -59775.4147, -1.689589472}, ahigh = { -170846.712, -169.205154,7.1254916, -4.99085285e-05,1.09733523e-08, -1.25287026e-12,5.78479114e-17}, bhigh = { -63382.9584, -10.41715673}, R = 120.551016374559);
          constant IdealGases.Common.DataRecord PF2minus(name = "PF2minus", MM = 0.068971116, Hf =  -10284565.309919, H0 = 160133.018581286, Tlimit = 1000, alow = {140479.3057, -1699.945753,10.36823971, -0.00359958769,1.9619258e-06, -3.82932987e-10, -3.37554952e-14}, blow = { -78104.3451, -32.17375182}, ahigh = { -165397.6056, -164.0604822,7.12409152, -5.0154314e-05,1.117302152e-08, -1.289308601e-12,6.00470667e-17}, bhigh = { -87002.2332, -11.09074866}, R = 120.550057505232);
          constant IdealGases.Common.DataRecord PF2CL(name = "PF2CL", MM = 0.1044235674, Hf =  -7039375.30868152, H0 = 132884.92574522, Tlimit = 1000, alow = {83975.9016, -1318.075883,10.17203515,0.00557123122, -1.100920478e-05,8.72638318e-09, -2.567714099e-12}, blow = { -83816.5617, -27.66012951}, ahigh = { -240928.1185, -203.348467,10.15150769, -6.04836731e-05,1.333942001e-08, -1.526772528e-12,7.06333918e-17}, bhigh = { -91011.1059, -24.17825549}, R = 79.6225622914315);
          constant IdealGases.Common.DataRecord PF2CL3(name = "PF2CL3", MM = 0.1753295674, Hf =  -5011961.04588119, H0 = 115013.333455587, Tlimit = 1000, alow = {184209.9543, -3334.40239,23.88510873, -0.01030943175,7.35650422e-06, -2.498621785e-09,2.513997646e-13}, blow = { -92795.4989, -102.7991502}, ahigh = { -365527.254, -202.969416,16.15238015, -6.11879657e-05,1.355610375e-08, -1.55712964e-12,7.22435997e-17}, bhigh = { -110494.1533, -53.87308191}, R = 47.4219615282071);
          constant IdealGases.Common.DataRecord PF3(name = "PF3", MM = 0.0879689706, Hf =  -10883383.0095995, H0 = 147057.455734284, Tlimit = 1000, alow = {84809.8448, -1102.340691,7.16071511,0.01438278338, -2.318137222e-05,1.702244315e-08, -4.81099195e-12}, blow = { -111183.7495, -14.57652362}, ahigh = { -295704.0675, -299.7689931,10.22297023, -8.89026345e-05,1.958901359e-08, -2.240491075e-12,1.035952523e-16}, bhigh = { -117374.3464, -27.77419177}, R = 94.5159633367359);
          constant IdealGases.Common.DataRecord PF3CL2(name = "PF3CL2", MM = 0.1588749706, Hf =  -7062301.32891682, H0 = 119887.883711747, Tlimit = 1000, alow = {182249.7206, -3089.610245,20.67710388, -0.001168732391, -5.02054074e-06,5.80457335e-09, -1.965367906e-12}, blow = { -122812.0667, -86.27977355}, ahigh = { -422383.805, -320.796522,16.24013, -9.62365531e-05,2.129279712e-08, -2.443515401e-12,1.13290435e-16}, bhigh = { -139264.2151, -55.31285745}, R = 52.3334290391995);
          constant IdealGases.Common.DataRecord PF4CL(name = "PF4CL", MM = 0.1424203738, Hf =  -9583666.70850572, H0 = 128637.529246535, Tlimit = 1000, alow = {180424.0323, -2741.096618,16.80192838,0.00991931525, -2.025636171e-05,1.617979147e-08, -4.77261152e-12}, blow = { -153238.2587, -66.79119965}, ahigh = { -478114.004, -426.481283,16.31812121, -0.0001271374903,2.806715254e-08, -3.2151663e-12,1.488512118e-16}, bhigh = { -168048.8362, -57.35844805}, R = 58.3797934112711);
          constant IdealGases.Common.DataRecord PF5(name = "PF5", MM = 0.125965777, Hf =  -12648673.6155329, H0 = 131286.523958011, Tlimit = 1000, alow = {185577.8473, -2436.979828,12.01012406,0.02339343049, -3.81394441e-05,2.796136412e-08, -7.87070488e-12}, blow = { -181456.6568, -44.8648874}, ahigh = { -566872.784, -667.791791,16.49789569, -0.000199009944,4.3950071e-08, -5.03684954e-12,2.332945909e-16}, bhigh = { -194432.1623, -62.6172104}, R = 66.005800924802);
          constant IdealGases.Common.DataRecord PH(name = "PH", MM = 0.031981701, Hf = 7215129.17652504, H0 = 270407.881056733, Tlimit = 1000, alow = {22736.33198, -397.267406,6.23369766, -0.0091817846,1.523328123e-05, -1.085888585e-08,2.929760547e-12}, blow = {28527.68404, -10.95191197}, ahigh = {781473.065, -3038.451204,7.46748102, -0.001837522255,7.1659477e-07, -1.142128853e-10,6.17541056e-15}, bhigh = {45362.6018, -24.6729814}, R = 259.975915602488);
          constant IdealGases.Common.DataRecord PH2(name = "PH2", MM = 0.032989641, Hf = 3623970.02137732, H0 = 302566.82696244, Tlimit = 1000, alow = {15552.68372, -184.1602025,4.89589604, -0.0034954366,1.053418945e-05, -8.37756292e-09,2.27076615e-12}, blow = {14098.39468, -2.210564792}, ahigh = {1127884.913, -4715.23825,10.214983, -0.00116757382,2.150542671e-07, -1.624213739e-11,3.76622524e-16}, bhigh = {41830.7463, -42.3162325}, R = 252.032812360704);
          constant IdealGases.Common.DataRecord PH2minus(name = "PH2minus", MM = 0.0329901896, Hf =  -280853.735984591, H0 = 301910.42006015, Tlimit = 1000, alow = {69506.8449, -771.916291,7.38353762, -0.00852294721,1.474476507e-05, -9.84716081e-09,2.413107871e-12}, blow = {1582.29585, -17.61306419}, ahigh = {1382525.815, -5213.40067,10.21694832, -0.001116316349,2.215004171e-07, -2.338005187e-11,1.015580932e-15}, bhigh = {29906.83733, -43.78711585}, R = 252.028621260182);
          constant IdealGases.Common.DataRecord PH3(name = "PH3", MM = 0.033997581, Hf = 159981.970481959, H0 = 298157.154181058, Tlimit = 1000, alow = { -6384.32534,405.756741, -0.1565680086,0.01338380613, -8.27539143e-06,3.024360831e-09, -6.42176463e-13}, blow = { -2159.842124,23.85561888}, ahigh = {1334801.106, -6725.46352,14.45857073, -0.001639736883,3.40921857e-07, -3.73627208e-11,1.672947506e-15}, bhigh = {39103.2571, -71.9878119}, R = 244.560693891721);
          constant IdealGases.Common.DataRecord PN(name = "PN", MM = 0.044980461, Hf = 3812484.38071811, H0 = 193464.113229075, Tlimit = 1000, alow = { -51032.0384,820.292668, -1.392772765,0.01287989789, -1.401425371e-05,7.77563346e-09, -1.75153933e-12}, blow = {15732.2652,32.51070633}, ahigh = { -249562.5593,176.043883,4.14412196,0.0002478018097, -5.6748963e-08,4.26364512e-12,3.063920924e-16}, bhigh = {17703.17267,1.325517397}, R = 184.846304709949);
          constant IdealGases.Common.DataRecord PO(name = "PO", MM = 0.046973161, Hf =  -593055.404553251, H0 = 199903.791869574, Tlimit = 1000, alow = { -68457.5406,1141.295708, -2.77955606,0.01678458047, -1.974879516e-05,1.19260232e-08, -2.927460912e-12}, blow = { -9847.74504,41.84328297}, ahigh = { -336666.744,622.935584,3.56560546,0.000651662072, -2.061770841e-07,3.18441323e-11, -1.573691908e-15}, bhigh = { -8939.79039,6.954859188}, R = 177.004736811304);
          constant IdealGases.Common.DataRecord POminus(name = "POminus", MM = 0.0469737096, Hf =  -2981818.87682978, H0 = 186872.743812424, Tlimit = 1000, alow = {54343.4632, -412.362355,3.74648388,0.00385863647, -5.96334304e-06,4.27828636e-09, -1.139722989e-12}, blow = { -15558.10937,3.404436468}, ahigh = { -848.029478,347.054487,2.877890167,0.001622182445, -4.87019099e-07,6.64575227e-11, -3.39570376e-15}, bhigh = { -19872.0954,10.75444355}, R = 177.00266959542);
          constant IdealGases.Common.DataRecord POCL3(name = "POCL3", MM = 0.153332161, Hf =  -3706984.86405601, H0 = 115539.726854825, Tlimit = 1000, alow = {47937.8816, -1270.38828,13.6771646,0.0010826721, -1.73610406e-06,8.46089064e-10, -1.17466133e-13}, blow = { -65075.7188, -43.14705941}, ahigh = { -218593.797, -508.026968,13.3778465, -0.000151073105,3.34098678e-08, -3.83532976e-12,1.77932988e-16}, bhigh = { -70093.1509, -39.84746301}, R = 54.2252319785671);
          constant IdealGases.Common.DataRecord POFCL2(name = "POFCL2", MM = 0.1368775642, Hf =  -5799995.90612236, H0 = 120210.993643617, Tlimit = 1000, alow = {46940.6156, -1069.2426,10.71284279,0.0094911833, -1.306412424e-05,8.42690602e-09, -2.140294969e-12}, blow = { -92748.9499, -28.15077719}, ahigh = { -270764.7503, -655.650227,13.48668421, -0.0001942765035,4.29118449e-08, -4.9218448e-12,2.282079808e-16}, bhigh = { -96543.9214, -41.64963494}, R = 60.7438629449252);
          constant IdealGases.Common.DataRecord POF2CL(name = "POF2CL", MM = 0.1204229674, Hf =  -8491794.06618741, H0 = 123700.323298959, Tlimit = 1000, alow = {52264.3816, -969.10377,7.96363598,0.01755599796, -2.398693997e-05,1.572376747e-08, -4.079746e-12}, blow = { -120265.6893, -15.93967075}, ahigh = { -334708.659, -817.610481,13.6066432, -0.0002420426112,5.34371272e-08, -6.12646135e-12,2.839570116e-16}, bhigh = { -123340.1194, -44.96103437}, R = 69.0439056561564);
          constant IdealGases.Common.DataRecord POF3(name = "POF3", MM = 0.1039683706, Hf =  -12042123.8957072, H0 = 136256.333712322, Tlimit = 1000, alow = {29660.3071, -427.670575,3.6691461,0.0283949378, -3.768318e-05,2.44910454e-08, -6.34197287e-12}, blow = { -150112.453,5.160417996}, ahigh = { -369060.242, -1005.55254,13.7452994, -0.000297103711,6.55452664e-08, -7.50998733e-12,3.47896991e-16}, bhigh = { -149976.173, -48.82214181}, R = 79.9711676928021);
          constant IdealGases.Common.DataRecord PO2(name = "PO2", MM = 0.062972561, Hf =  -4470633.43985645, H0 = 166943.599451196, Tlimit = 1000, alow = { -63726.9822,1036.741044, -2.877797967,0.02278134083, -2.567920328e-05,1.465060412e-08, -3.3879967e-12}, blow = { -39935.4472,44.2530938}, ahigh = {492621.099, -2605.465745,9.51760561, -0.001180371565,2.532912819e-07, -1.789964539e-11,1.800381054e-16}, bhigh = { -20288.84763, -29.69743125}, R = 132.033251752299);
          constant IdealGases.Common.DataRecord PO2minus(name = "PO2minus", MM = 0.0629731096, Hf =  -9490141.97958552, H0 = 168544.146341473, Tlimit = 1000, alow = {27024.23135,29.39956284,1.162686114,0.01599897972, -1.979229263e-05,1.208847527e-08, -2.95664048e-12}, blow = { -72859.4627,19.63118182}, ahigh = {1628679.152, -5989.19998,13.69318836, -0.00362384379,9.48819834e-07, -1.040308759e-10,4.01903366e-15}, bhigh = { -36859.595, -59.5679622}, R = 132.032101524172);
          constant IdealGases.Common.DataRecord PS(name = "PS", MM = 0.063038761, Hf = 2386330.46737705, H0 = 152542.909274502, Tlimit = 1000, alow = { -768.97868, -46.3782407,4.10464917,0.001555262273, -2.315757288e-06,1.661425178e-09, -4.6312384e-13}, blow = {17078.75808,4.230652171}, ahigh = { -270272.9081,888.354822,3.16919012,0.001022480817, -3.80374048e-07,7.01986188e-11, -4.26912231e-15}, bhigh = {11215.10462,11.47334049}, R = 131.894597357331);
          constant IdealGases.Common.DataRecord P2(name = "P2", MM = 0.061947522, Hf = 2324548.18773865, H0 = 143736.290210285, Tlimit = 1000, alow = {30539.2251, -324.617759,4.02246381,0.00323209479, -5.51105245e-06,4.19557293e-09, -1.21503218e-12}, blow = {17969.1087,1.645350331}, ahigh = { -780693.649,2307.91087,1.41174313,0.00210823742, -7.36085662e-07,1.25936012e-10, -7.07975249e-15}, bhigh = {1329.82474,21.69741365}, R = 134.217991802804);
          constant IdealGases.Common.DataRecord P2O3(name = "P2O3", MM = 0.109945722, Hf =  -6227120.63321573, H0 = 146389.333820556, Tlimit = 1000, alow = { -66457.5389,758.090055,0.988843677,0.0285987873, -3.23114211e-05,1.83452517e-08, -4.2124893e-12}, blow = { -88200.3674,26.89914458}, ahigh = { -217534.411, -1369.902,14.0114296, -0.000402385696,8.86834578e-08, -1.01565728e-11,4.70435606e-16}, bhigh = { -79227.57, -47.39487232}, R = 75.6234244384697);
          constant IdealGases.Common.DataRecord P2O4(name = "P2O4", MM = 0.125945122, Hf =  -7413980.09047147, H0 = 137564.517981093, Tlimit = 1000, alow = { -43760.9126,490.510283,0.964364937,0.0388607358, -4.67550922e-05,2.81262563e-08, -6.8244061e-12}, blow = { -116899.779,23.79582767}, ahigh = { -367011.486, -1638.42373,17.2118861, -0.000482658015,1.0644619e-07, -1.21961007e-11,5.65065458e-16}, bhigh = { -109043.593, -67.14276593}, R = 66.0166258761495);
          constant IdealGases.Common.DataRecord P2O5(name = "P2O5", MM = 0.141944522, Hf =  -7921195.81761669, H0 = 155341.711601945, Tlimit = 1000, alow = { -29991.5922, -45.9450659,6.98748683,0.0288330297, -3.10450953e-05,1.64386868e-08, -3.4679423e-12}, blow = { -138190.14, -3.400572711}, ahigh = { -324708.016, -2016.97508,20.4869934, -0.000591064115,1.30199302e-07, -1.49068294e-11,6.90357341e-16}, bhigh = { -130629.016, -80.31642952}, R = 58.5755045904484);
          constant IdealGases.Common.DataRecord P3(name = "P3", MM = 0.092921283, Hf = 2259977.40474591, H0 = 129201.853573201, Tlimit = 1000, alow = {46933.5895, -864.358932,8.73424753,1.09453456e-05, -1.99857332e-06,2.09675051e-09, -6.92288971e-13}, blow = {27748.4734, -20.63587261}, ahigh = { -125351.306, -83.0673462,7.56195677, -2.47440395e-05,5.45727693e-09, -6.24466057e-13,2.88772422e-17}, bhigh = {23087.2228, -12.28310621}, R = 89.4786612018691);
          constant IdealGases.Common.DataRecord P3O6(name = "P3O6", MM = 0.188917683, Hf =  -8340569.31028526, H0 = 124071.403098883, Tlimit = 1000, alow = {201449.3813, -3053.488073,16.02114098,0.043168899, -6.37654426e-05,4.42286792e-08, -1.200020334e-11}, blow = { -177650.3811, -65.5626789}, ahigh = { -878796.81, -1651.988071,27.72258397, -0.00048593267,1.067867708e-07, -1.218015776e-11,5.61491196e-16}, bhigh = { -190916.5548, -122.0665277}, R = 44.011083917433);
          constant IdealGases.Common.DataRecord P4(name = "P4", MM = 0.123895044, Hf = 475402.38978405, H0 = 113875.80604112, Tlimit = 1000, alow = {120514.185, -2345.71179,17.7394655, -0.0145631497,1.58759409e-05, -9.31471031e-09,2.27149167e-12}, blow = {16088.4648, -70.88590647}, ahigh = { -185870.586, -62.8454689,10.0484488, -1.98766885e-05,4.48240844e-09, -5.22610367e-13,2.45579129e-17}, bhigh = {3848.21092, -24.77297797}, R = 67.1089959014018);
          constant IdealGases.Common.DataRecord P4O6(name = "P4O6", MM = 0.219891444, Hf =  -7303603.86373196, H0 = 112771.659273837, Tlimit = 1000, alow = {376089.475, -5685.83262,29.1422545,0.0225153212, -4.51026963e-05,3.58831269e-08, -1.05757206e-11}, blow = { -168856.248, -145.1359851}, ahigh = { -1008997.24, -887.275399,28.6606483, -0.000263601894,5.81094072e-08, -6.64808696e-12,3.07439193e-16}, bhigh = { -199713.89, -128.1853821}, R = 37.8117122192349);
          constant IdealGases.Common.DataRecord P4O7(name = "P4O7", MM = 0.235890844, Hf =  -8412569.1839061, H0 = 111798.81572682, Tlimit = 1000, alow = {321858.696, -4871.44473,24.54417068,0.0401550955, -6.52419715e-05,4.75655418e-08, -1.332975491e-11}, blow = { -218451.7427, -118.1504792}, ahigh = { -1102947.375, -1511.077128,32.109795, -0.000437180376,9.51663961e-08, -1.075204176e-11,4.91102568e-16}, bhigh = { -242909.654, -147.3140859}, R = 35.2471162466993);
          constant IdealGases.Common.DataRecord P4O8(name = "P4O8", MM = 0.251890244, Hf =  -9139750.87498824, H0 = 110946.480324978, Tlimit = 1000, alow = {271932.6942, -4118.60039,20.28714235,0.0568541699, -8.40073963e-05,5.82338906e-08, -1.57864716e-11}, blow = { -260453.9703, -94.0716207}, ahigh = { -1173177.311, -2200.632995,35.6301013, -0.000648984974,1.429623989e-07, -1.635821725e-11,7.57107076e-16}, bhigh = { -278384.3144, -167.963696}, R = 33.0083129380747);
          constant IdealGases.Common.DataRecord P4O9(name = "P4O9", MM = 0.267889644, Hf =  -9757671.37530706, H0 = 110199.687301089, Tlimit = 1000, alow = {219991.6395, -3336.46494,15.86413106,0.0740245233, -0.0001034835052,6.94443832e-08, -1.84073345e-11}, blow = { -301874.2692, -69.9796767}, ahigh = { -1233696.783, -2917.032808,39.1775711, -0.000873640298,1.937914895e-07, -2.230510118e-11,1.037248818e-15}, bhigh = { -312963.7516, -189.717642}, R = 31.0369295201273);
          constant IdealGases.Common.DataRecord P4O10(name = "P4O10", MM = 0.283889044, Hf =  -10237180.7169846, H0 = 109537.700229108, Tlimit = 1000, alow = {167014.2268, -2540.2433,11.36853403,0.0913781216, -0.0001231990626,8.08103223e-08, -2.106787011e-11}, blow = { -341015.191, -46.4343134}, ahigh = { -1337201.132, -3516.38233,42.6040331, -0.001037406404,2.287613089e-07, -2.620219946e-11,1.213529528e-15}, bhigh = { -345951.505, -211.5498906}, R = 29.2877522952242);
          constant IdealGases.Common.DataRecord Pb(name = "Pb", MM = 0.2072, Hf = 942084.942084942, H0 = 29910.3667953668, Tlimit = 1000, alow = {1213.382285, -19.06116019,2.619299546, -0.000382951961,6.68818045e-07, -6.06123108e-10,2.240022429e-13}, blow = {22820.96238,6.2013692}, ahigh = { -9084313.07,26726.7318, -26.26244039,0.01358282305, -2.685523566e-06,2.3524328e-10, -7.324114532e-15}, bhigh = { -148165.0666,215.4011624}, R = 40.1277606177606);
          constant IdealGases.Common.DataRecord Pbplus(name = "Pbplus", MM = 0.2071994514, Hf = 4425670.63669359, H0 = 29910.4459887581, Tlimit = 1000, alow = {9.48668904, -0.1134955793,2.500549799, -1.382464681e-06,1.906022991e-09, -1.368396828e-12,4.003528117e-16}, blow = {109543.8901,7.5388559}, ahigh = {1320690.183, -4096.04801,7.38910151, -0.002807751909,7.83099165e-07, -9.31060091e-11,4.016371727e-15}, bhigh = {135434.7306, -27.22020908}, R = 40.1278668636475);
          constant IdealGases.Common.DataRecord Pbminus(name = "Pbminus", MM = 0.2072005486, Hf = 742671.431324579, H0 = 29910.2876023949, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {17762.2614,8.2351321}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = {17762.2614,8.2351321}, R = 40.1276543724364);
          constant IdealGases.Common.DataRecord PbBr(name = "PbBr", MM = 0.287104, Hf = 225776.450345519, H0 = 35339.5354993313, Tlimit = 1000, alow = { -2393.487988, -45.7659121,4.69260828, -0.000384529063,5.8882135e-07, -4.08479289e-10,1.202515201e-13}, blow = {6662.43477,5.99170551}, ahigh = { -2581831.67,7143.49076, -2.65819364,0.00310772226, -4.63138154e-07,1.095529211e-11,1.508555088e-15}, bhigh = { -39704.4098,59.5923865}, R = 28.9597915737851);
          constant IdealGases.Common.DataRecord PbBr2(name = "PbBr2", MM = 0.367008, Hf =  -283122.065459064, H0 = 40930.8162219897, Tlimit = 1000, alow = { -6172.76636, -67.8320763,7.27315614, -0.000599059612,7.35863461e-07, -4.74360481e-10,1.246922291e-13}, blow = { -14278.90796, -0.698799215}, ahigh = { -13204.21479, -1.143074947,7.00097254, -4.28447161e-07,1.018044861e-10, -1.234016791e-14,5.97133765e-19}, bhigh = { -14622.1375,0.886858265}, R = 22.6547432208562);
          constant IdealGases.Common.DataRecord PbBr3(name = "PbBr3", MM = 0.446912, Hf =  -232732.522286267, H0 = 44681.3757518259, Tlimit = 1000, alow = { -11644.2425, -155.5151665,10.62317321, -0.001361947117,1.668738791e-06, -1.073686019e-09,2.81825034e-13}, blow = { -14782.08304, -14.43758579}, ahigh = { -27879.30766, -2.633265111,10.00223199, -9.80776141e-07,2.326232632e-10, -2.81599446e-14,1.361283735e-18}, bhigh = { -15569.81182, -10.81765474}, R = 18.6042710869254);
          constant IdealGases.Common.DataRecord PbBr4(name = "PbBr4", MM = 0.526816, Hf =  -346298.348949159, H0 = 49109.1557583672, Tlimit = 1000, alow = { -11764.58199, -256.7166983,14.02223292, -0.002224117068,2.716201994e-06, -1.743340164e-09,4.56735975e-13}, blow = { -24621.25551, -28.8202075}, ahigh = { -38802.1603, -4.38293377,13.00369823, -1.620061335e-06,3.83413088e-10, -4.63396718e-14,2.237427066e-18}, bhigh = { -25923.27065, -22.87710636}, R = 15.7824971147421);
          constant IdealGases.Common.DataRecord PbCL(name = "PbCL", MM = 0.242653, Hf = 36344.5661088056, H0 = 40333.8017663083, Tlimit = 1000, alow = { -2902.700749, -82.9345971,4.71482448, -0.0001931473228,9.10557902e-08,5.41938523e-11, -3.2849697e-14}, blow = {125.4460217,4.323194455}, ahigh = { -385428.553,692.21975,4.50997124, -0.00068534213,5.0873955e-07, -9.77275381e-11,5.80747158e-15}, bhigh = { -5269.68472,6.646491015}, R = 34.2648638178799);
          constant IdealGases.Common.DataRecord PbCL2(name = "PbCL2", MM = 0.278106, Hf =  -631225.040092626, H0 = 50349.6112992888, Tlimit = 1000, alow = {2830.580143, -313.4256659,8.21282729, -0.002585096173,3.109456077e-06, -1.972978208e-09,5.12354248e-13}, blow = { -21675.69042, -9.219412244}, ahigh = { -31491.92744, -5.56547758,7.00459954, -1.986025455e-06,4.65171959e-10, -5.57913701e-14,2.67814357e-18}, bhigh = { -23274.39309, -2.140739285}, R = 29.896773172819);
          constant IdealGases.Common.DataRecord PbCL3(name = "PbCL3", MM = 0.313559, Hf =  -566571.225829908, H0 = 58220.9185512136, Tlimit = 1000, alow = {862.358897, -525.315275,12.03336337, -0.00433518498,5.21569835e-06, -3.31002238e-09,8.59697061e-13}, blow = { -21805.76379, -26.94206962}, ahigh = { -56644.1279, -9.32951479,10.00771099, -3.32972757e-06,7.79933081e-10, -9.3546061e-14,4.4905897e-18}, bhigh = { -24485.08643, -15.0749087}, R = 26.5164514493285);
          constant IdealGases.Common.DataRecord PbCL4(name = "PbCL4", MM = 0.349012, Hf =  -938163.621881196, H0 = 67188.0565711208, Tlimit = 1000, alow = {17394.05796, -890.213932,16.37565058, -0.00708899226,8.43256734e-06, -5.30530939e-09,1.368671294e-12}, blow = { -38882.1562, -48.50133784}, ahigh = { -82666.7956, -16.39486405,13.01336711, -5.7161414e-06,1.32936839e-09, -1.585922492e-13,7.58176361e-18}, bhigh = { -43439.8422, -28.74559263}, R = 23.8228828808179);
          constant IdealGases.Common.DataRecord PbF(name = "PbF", MM = 0.2261984032, Hf =  -437084.380797256, H0 = 40973.370584784, Tlimit = 1000, alow = {26848.64255, -475.172564,6.01964437, -0.002718531008,2.942882954e-06, -1.687543009e-09,4.08612208e-13}, blow = { -10790.38653, -4.98304884}, ahigh = { -473420.275,1027.785283,3.88865873, -0.0001649813448,2.93116326e-07, -5.76645965e-11,3.24748619e-15}, bhigh = { -20261.65076,9.31948389}, R = 36.7574301249533);
          constant IdealGases.Common.DataRecord PbF2(name = "PbF2", MM = 0.2451968064, Hf =  -1808454.99788695, H0 = 51275.5862712574, Tlimit = 1000, alow = {59205.5228, -1052.529292,10.28969284, -0.00587634627,6.10233313e-06, -3.42479264e-09,8.02402542e-13}, blow = { -49990.5, -25.25115478}, ahigh = { -85058.9977, -30.74244334,7.02334604, -9.45518359e-06,2.108489027e-09, -2.434212002e-13,1.133872429e-17}, bhigh = { -55522.3677, -5.49607322}, R = 33.9093812928226);
          constant IdealGases.Common.DataRecord PbF3(name = "PbF3", MM = 0.2641952096, Hf =  -1853071.49868928, H0 = 58801.3954663317, Tlimit = 1000, alow = {85530.9087, -1731.036694,15.5659615, -0.01022225919,1.089984998e-05, -6.26999882e-09,1.502616508e-12}, blow = { -53003.3567, -53.3573348}, ahigh = { -145970.4258, -48.2039643,10.03685845, -1.501095764e-05,3.36247499e-09, -3.89602071e-13,1.820152103e-17}, bhigh = { -62068.6959, -20.06978661}, R = 31.4709415533627);
          constant IdealGases.Common.DataRecord PbF4(name = "PbF4", MM = 0.2831936128, Hf =  -2824658.24384582, H0 = 69303.2120532346, Tlimit = 1000, alow = {130996.3563, -2307.299499,19.4520144, -0.01024522729,9.39587496e-06, -4.62689415e-09,9.45088143e-13}, blow = { -88041.4746, -75.2464582}, ahigh = { -213261.6322, -85.7862195,13.06428551, -2.575563598e-05,5.69332459e-09, -6.52621955e-13,3.02234196e-17}, bhigh = { -100290.8979, -35.9024028}, R = 29.3596734678897);
          constant IdealGases.Common.DataRecord PbI(name = "PbI", MM = 0.33410447, Hf = 325958.895431719, H0 = 30945.7817191132, Tlimit = 1000, alow = { -2717.33406, -15.42541996,4.5653279, -0.0001061215818,2.414841342e-07, -1.876070301e-10,6.46061427e-14}, blow = {11818.64978,7.66980267}, ahigh = { -3901070.25,11479.76718, -8.17610902,0.0065234547, -1.522611682e-06,1.626692202e-10, -6.55332888e-15}, bhigh = { -61544.3369,99.3840497}, R = 24.885844837694);
          constant IdealGases.Common.DataRecord PbI2(name = "PbI2", MM = 0.46100894, Hf =  -22239.5470248364, H0 = 33073.9139245326, Tlimit = 1000, alow = { -5904.98672, -48.3330703,7.19543803, -0.000429860009,5.29141207e-07, -3.41640282e-10,8.99134192e-14}, blow = { -3107.780155,1.324773678}, ahigh = { -10883.39477, -0.816441837,7.0006973, -3.079876763e-07,7.33171814e-11, -8.89919388e-15,4.3107125e-19}, bhigh = { -3352.09638,2.458605463}, R = 18.035381266142);
          constant IdealGases.Common.DataRecord PbI3(name = "PbI3", MM = 0.58791341, Hf = 37004.4493456953, H0 = 35830.1267528495, Tlimit = 1000, alow = { -10149.96331, -59.380782,10.24073798, -0.000530469453,6.53862303e-07, -4.22589572e-10,1.113029818e-13}, blow = { -113.8482403, -8.9755405}, ahigh = { -16242.29573, -1.002945584,10.00085839, -3.79667974e-07,9.04675172e-11, -1.098840054e-14,5.32538829e-19}, bhigh = { -413.825893, -7.57941711}, R = 14.1423411314942);
          constant IdealGases.Common.DataRecord PbI4(name = "PbI4", MM = 0.71481788, Hf =  -57674.0735696203, H0 = 38500.0736131559, Tlimit = 1000, alow = { -12170.96972, -72.2631744,13.29294204, -0.00064546609,7.95576185e-07, -5.14163283e-10,1.354188031e-13}, blow = { -8528.14868, -20.10457498}, ahigh = { -19584.77744, -1.223385179,13.00104732, -4.63329569e-07,1.104213978e-10, -1.341397771e-14,6.50170611e-19}, bhigh = { -8893.1932, -18.40570345}, R = 11.6315948895962);
          constant IdealGases.Common.DataRecord PbO(name = "PbO", MM = 0.2231994, Hf = 305274.601992658, H0 = 40152.9215580329, Tlimit = 1000, alow = {34240.25, -419.805011,4.72030641,0.001451061751, -3.20603546e-06,2.713956273e-09, -8.33327561e-13}, blow = {9253.18647,0.448289121}, ahigh = {242699.685, -110.7731354,3.18068449,0.001991815065, -1.079782025e-06,2.540990941e-10, -1.834652493e-14}, bhigh = {8339.39154,10.50920926}, R = 37.2513187759465);
          constant IdealGases.Common.DataRecord PbO2(name = "PbO2", MM = 0.2391988, Hf = 569203.24851128, H0 = 51216.5362033589, Tlimit = 1000, alow = {70710.5829, -1076.757462,9.40171173, -0.001003861584, -1.233815617e-06,1.858386979e-09, -6.83462121e-13}, blow = {19996.45981, -25.03976748}, ahigh = { -130779.667, -73.1436145,7.55408341, -2.143421561e-05,4.696464e-09, -5.34497218e-13,2.460760535e-17}, bhigh = {14133.86101, -12.52684909}, R = 34.7596727073882);
          constant IdealGases.Common.DataRecord PbS(name = "PbS", MM = 0.239265, Hf = 534743.121643366, H0 = 39412.8393204188, Tlimit = 1000, alow = {15298.40503, -343.974635,5.68063064, -0.002232931917,2.504608209e-06, -1.470427328e-09,3.57752719e-13}, blow = {15785.51202, -2.628900926}, ahigh = {2105441.649, -5520.41358,9.36924926, -0.001410262019, -1.642683362e-07,1.451447577e-10, -1.393564925e-14}, bhigh = {50093.3453, -32.17417657}, R = 34.7500553779282);
          constant IdealGases.Common.DataRecord PbS2(name = "PbS2", MM = 0.27133, Hf = 899454.557181292, H0 = 51675.8375410017, Tlimit = 1000, alow = {24463.7588, -665.485546,9.92020118, -0.00492419438,5.71690629e-06, -3.52948883e-09,8.97113835e-13}, blow = {30443.1013, -22.95765552}, ahigh = { -54167.003, -13.43600342,7.51070036, -4.49731863e-06,1.032444494e-09, -1.219580043e-13,5.78580465e-18}, bhigh = {27012.29533, -8.714618097}, R = 30.6433936534847);
          constant IdealGases.Common.DataRecord Rb(name = "Rb", MM = 0.0854678, Hf = 946555.310889013, H0 = 72511.8465667772, Tlimit = 1000, alow = {13.52856616, -0.2042232679,2.501213823, -3.6506199e-06,5.88472267e-09, -4.84227472e-12,1.596211946e-15}, blow = {8985.56921,6.20700548}, ahigh = { -1138274.064,3804.04194, -2.750899258,0.0038914607, -1.632296823e-06,3.51189314e-10, -2.521064422e-14}, bhigh = { -14664.54849,42.5344237}, R = 97.2819237186402);
          constant IdealGases.Common.DataRecord Rbplus(name = "Rbplus", MM = 0.0854672514, Hf = 5734700.95236969, H0 = 72512.3120081992, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {58203.2736,5.52050692}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = {58203.2736,5.52050692}, R = 97.2825481550469);
          constant IdealGases.Common.DataRecord Rbminus(name = "Rbminus", MM = 0.0854683486, Hf = 325483.39187169, H0 = 72511.3811313303, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {2600.405796,5.52052617}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = {2600.405796,5.52052617}, R = 97.2812992902498);
          constant IdealGases.Common.DataRecord RbBO2(name = "RbBO2", MM = 0.1282776, Hf =  -5293030.91108658, H0 = 111731.884600273, Tlimit = 1000, alow = {41986.5775, -679.944833,8.18515101,0.002772536733, -6.17577047e-07, -9.42613625e-10,4.72241424e-13}, blow = { -80203.3959, -12.39198887}, ahigh = {91544.634, -1668.328341,11.17268757, -0.000450553353,9.68133059e-08, -1.088037652e-11,4.96773366e-16}, bhigh = { -75079.0325, -31.31951149}, R = 64.816242274567);
          constant IdealGases.Common.DataRecord RbBr(name = "RbBr", MM = 0.1653718, Hf =  -1158062.47498062, H0 = 62399.5384944712, Tlimit = 1000, alow = { -5703.8597,20.92281597,4.37060737,0.000449460478, -4.83543884e-07,3.5260686e-10, -9.88653832e-14}, blow = { -24491.12515,6.4274317}, ahigh = {1609092.474, -4466.77459,9.06409206, -0.001995333358,4.01663352e-07, -4.1630914e-12, -3.092443341e-15}, bhigh = {4427.46769, -27.57714089}, R = 50.2774475454703);
          constant IdealGases.Common.DataRecord RbCL(name = "RbCL", MM = 0.1209208, Hf =  -1846852.47699321, H0 = 83171.1417721351, Tlimit = 1000, alow = { -5740.80958, -15.28835285,4.5152009,0.0001067270745, -6.38594559e-09, -1.301381692e-11,1.641459742e-14}, blow = { -28142.43986,4.190115141}, ahigh = { -1143043.004,4003.86188, -1.096156782,0.00403358128, -1.454151464e-06,2.719263762e-10, -1.81046398e-14}, bhigh = { -52976.8604,43.11317822}, R = 68.7596509450814);
          constant IdealGases.Common.DataRecord RbF(name = "RbF", MM = 0.1044662032, Hf =  -3192533.8031238, H0 = 91781.9802605787, Tlimit = 1000, alow = {32873.6019, -608.090746,7.40662236, -0.00727786472,1.013930727e-05, -7.1413725e-09,2.014332982e-12}, blow = { -38498.3413, -13.75750778}, ahigh = { -854481.115,2633.756505,1.216216506,0.002136494067, -6.72842608e-07,1.134905135e-10, -6.82562444e-15}, bhigh = { -58101.7911,25.88118896}, R = 79.5900659286141);
          constant IdealGases.Common.DataRecord RbH(name = "RbH", MM = 0.08647574, Hf = 1379856.43141071, H0 = 101995.149159753, Tlimit = 1000, alow = {8908.27061,46.498704,1.833967623,0.00844427914, -1.167538647e-05,8.01038942e-09, -2.162578321e-12}, blow = {13282.48009,12.79544976}, ahigh = { -2937703.53,9379.81726, -7.54256346,0.0074932268, -2.159228887e-06,2.871300173e-10, -1.459981256e-14}, bhigh = { -45955.1883,83.6566432}, R = 96.1480294935898);
          constant IdealGases.Common.DataRecord RbI(name = "RbI", MM = 0.21237227, Hf =  -652066.646930882, H0 = 49249.0191869212, Tlimit = 1000, alow = { -769.30528, -29.68608844,4.64874584, -0.0002962415236,5.97813534e-07, -4.35125525e-10,1.313617336e-13}, blow = { -17866.10717,5.81240983}, ahigh = {4297424.85, -12969.11161,19.58730402, -0.00845842559,2.463480035e-06, -3.19157573e-10,1.458455589e-14}, bhigh = {64329.4804, -100.9499336}, R = 39.1504597092643);
          constant IdealGases.Common.DataRecord RbK(name = "RbK", MM = 0.1245661, Hf = 963450.971010572, H0 = 86766.2229129755, Tlimit = 1000, alow = {24946.95429, -455.306066,7.68618995, -0.01108112244,2.093398586e-05, -1.824445366e-08,5.61764536e-12}, blow = {15161.3616, -10.63563675}, ahigh = { -4175391.07,5918.58851,9.20264942, -0.010609336,4.96482606e-06, -8.71669879e-10,5.22188796e-14}, bhigh = { -32727.6904, -13.72401997}, R = 66.7474698172296);
          constant IdealGases.Common.DataRecord RbLi(name = "RbLi", MM = 0.0924088, Hf = 1776684.94775389, H0 = 111202.872453706, Tlimit = 1000, alow = { -2263.413044, -28.8849691,4.55582654,0.000362566753, -9.59800222e-07,1.631010244e-09, -8.15982598e-13}, blow = {18534.5525,2.831743182}, ahigh = {10855151.94, -38051.0697,55.1655504, -0.03128845612,9.26165025e-06, -1.264671408e-09,6.43166205e-14}, bhigh = {254396.4131, -352.075871}, R = 89.9748941659236);
          constant IdealGases.Common.DataRecord RbNO2(name = "RbNO2", MM = 0.1314733, Hf =  -1427132.1781685, H0 = 119950.005058061, Tlimit = 1000, alow = { -71994.2003,1265.802595, -2.415715742,0.0305464142, -3.68400405e-05,2.247716481e-08, -5.54752743e-12}, blow = { -30373.81667,48.14256904}, ahigh = { -164499.4453, -849.944344,10.63080575, -0.0002517893988,5.56142012e-08, -6.3788056e-12,2.957679896e-16}, bhigh = { -21285.90982, -25.68367325}, R = 63.240764474612);
          constant IdealGases.Common.DataRecord RbNO3(name = "RbNO3", MM = 0.1474727, Hf =  -2135801.96877117, H0 = 110986.758905208, Tlimit = 1000, alow = { -26646.69172,880.763479, -3.159459457,0.0427122902, -5.35102921e-05,3.36096975e-08, -8.50329051e-12}, blow = { -43535.9546,49.43052899}, ahigh = { -314741.3867, -1371.457846,14.01319759, -0.000403027502,8.87833229e-08, -1.016221214e-11,4.70425409e-16}, bhigh = { -35043.3994, -46.19379241}, R = 56.379736724153);
          constant IdealGases.Common.DataRecord RbNa(name = "RbNa", MM = 0.10845757, Hf = 1212182.12799715, H0 = 98334.5653051235, Tlimit = 1000, alow = {29744.55567, -508.279059,7.73403352, -0.0101300684,1.745455325e-05, -1.39872547e-08,3.96810677e-12}, blow = {16823.79436, -12.4008529}, ahigh = {4297114.27, -18310.13588,34.543942, -0.02230134993,7.33987453e-06, -1.064784996e-09,5.61759846e-14}, bhigh = {124261.0951, -199.366198}, R = 76.6610574070579);
          constant IdealGases.Common.DataRecord RbO(name = "RbO", MM = 0.1014672, Hf = 517303.365028305, H0 = 101876.507876437, Tlimit = 1000, alow = {190008.1316, -3428.22899,24.80251273, -0.0505165458,6.47633876e-05, -4.19801242e-08,1.093889043e-11}, blow = {20838.9845, -109.755915}, ahigh = {617075.688, -2175.08962,8.29041221, -0.002996487663,1.233144275e-06, -2.115810631e-10,1.24260743e-14}, bhigh = {18281.64827, -21.17386042}, R = 81.9424602235993);
          constant IdealGases.Common.DataRecord RbOH(name = "RbOH", MM = 0.10247514, Hf =  -2322514.51425195, H0 = 114771.270378357, Tlimit = 1000, alow = {12138.32721, -544.141159,8.45981464, -0.00356152656,2.99287239e-06, -7.2802674e-10, -4.55384718e-14}, blow = { -27872.62486, -19.13421991}, ahigh = {895858.313, -2332.339,7.97119248,0.0001044439355, -6.32825775e-08,1.029358824e-11, -5.74327779e-16}, bhigh = { -15155.68947, -19.50051629}, R = 81.1364785644596);
          constant IdealGases.Common.DataRecord Rb2Br2(name = "Rb2Br2", MM = 0.3307436, Hf =  -1668366.16944364, H0 = 65599.3524893603, Tlimit = 1000, alow = { -8005.5254, -21.11968088,10.08629404, -0.000191193609,2.36605949e-07, -1.53372238e-10,4.04872506e-14}, blow = { -69273.136, -9.62243463}, ahigh = { -10145.35336, -0.360155032,10.00031058, -1.380745805e-07,3.30206795e-11, -4.02158182e-15,1.953006729e-19}, bhigh = { -69379.6088, -9.12256236}, R = 25.1387237727351);
          constant IdealGases.Common.DataRecord Rb2CL2(name = "Rb2CL2", MM = 0.2418416, Hf =  -2556936.17640638, H0 = 85947.0579089784, Tlimit = 1000, alow = { -11250.39522, -67.5695399,10.27381915, -0.000603184899,7.43332478e-07, -4.80337216e-10,1.264973638e-13}, blow = { -77067.575, -13.63652575}, ahigh = { -18188.75133, -1.13790358,10.00097318, -4.3021564e-07,1.024725787e-10, -1.244291058e-14,6.02891239e-19}, bhigh = { -77408.9767, -12.04843385}, R = 34.3798254725407);
          constant IdealGases.Common.DataRecord Rb2F2(name = "Rb2F2", MM = 0.2089324064, Hf =  -4091817.84066217, H0 = 90410.8573939251, Tlimit = 1000, alow = { -8178.3471, -333.473354,11.31540799, -0.002842914926,3.45495958e-06, -2.209373419e-09,5.77207327e-13}, blow = { -104223.0504, -23.68991587}, ahigh = { -43765.7439, -5.76145204,10.00482691, -2.104159548e-06,4.96241627e-10, -5.98217497e-14,2.882751742e-18}, bhigh = { -105917.6154, -16.03254014}, R = 39.7950329643071);
          constant IdealGases.Common.DataRecord Rb2I2(name = "Rb2I2", MM = 0.42474454, Hf =  -1019333.83769924, H0 = 52307.9543294424, Tlimit = 1000, alow = { -5839.66355, -9.5275379,10.03904495, -8.668754e-05,1.074384368e-07, -6.97212223e-11,1.842065721e-14}, blow = { -55027.8096, -7.17411332}, ahigh = { -6799.83464, -0.1642527699,10.000142, -6.32142073e-08,1.51281117e-11, -1.843027189e-15,8.95124418e-20}, bhigh = { -55075.7962, -6.94804539}, R = 19.5752298546322);
          constant IdealGases.Common.DataRecord Rb2O(name = "Rb2O", MM = 0.186935, Hf =  -582712.819964159, H0 = 76046.6418808677, Tlimit = 1000, alow = {19712.76138, -462.750886,8.58829313, -0.003085559728,3.45198014e-06, -2.068570368e-09,5.1324805e-13}, blow = { -12848.54711, -12.64891828}, ahigh = { -38463.6009, -10.82831019,7.00842189, -3.47622812e-06,7.86978506e-10, -9.19568385e-14,4.32520152e-18}, bhigh = { -15253.73768, -3.23038878}, R = 44.4778773370423);
          constant IdealGases.Common.DataRecord Rb2O2(name = "Rb2O2", MM = 0.2029344, Hf =  -1063636.66288219, H0 = 83017.6254001293, Tlimit = 1000, alow = {32605.5323, -757.816472,10.70815124,0.001148004329, -3.29124367e-06,2.853106637e-09, -8.72159068e-13}, blow = { -24753.17258, -24.3427646}, ahigh = { -131856.7396, -96.4773779,10.07222726, -2.894921706e-05,6.4056391e-09, -7.35141939e-13,3.40856171e-17}, bhigh = { -28821.79733, -19.10948823}, R = 40.9712301117997);
          constant IdealGases.Common.DataRecord Rb2O2H2(name = "Rb2O2H2", MM = 0.20495028, Hf =  -3117829.35841805, H0 = 114210.036697681, Tlimit = 1000, alow = { -4577.07653, -841.7631,17.04083828, -0.00535788254,3.95177877e-06, -2.051775094e-10, -4.08541691e-13}, blow = { -76949.8217, -54.3541278}, ahigh = {1792711.652, -4659.4066,16.93822422,0.0002106246492, -1.26961747e-07,2.063382927e-11, -1.150861507e-15}, bhigh = { -50241.1763, -60.2705492}, R = 40.5682392822298);
          constant IdealGases.Common.DataRecord Rb2SO4(name = "Rb2SO4", MM = 0.2669982, Hf =  -4107114.02548781, H0 = 87348.7087178865, Tlimit = 1000, alow = {60948.5493, -688.97059,7.03424218,0.0393394547, -5.49892051e-05,3.71634108e-08, -9.91986338e-12}, blow = { -131187.7657, -4.450168488}, ahigh = { -530289.831, -951.543545,19.71013517, -0.0002842916301,6.28914294e-08, -7.2193243e-12,3.34871819e-16}, bhigh = { -133871.3151, -70.56246882}, R = 31.1405545056109);
          constant IdealGases.Common.DataRecord Rn(name = "Rn", MM = 0.2220176, Hf = 0, H0 = 27914.1293302873, Tlimit = 1000, alow = {3.38943209e-06, -1.311675533e-07,2.500000001, -2.978593139e-12,4.33705073e-15, -3.18204022e-18,9.24778703e-22}, blow = { -745.374999,6.95244198}, ahigh = {27301.90029, -82.8467262,2.598178483, -5.81372985e-05,1.819136527e-08, -2.866656182e-12,1.789322176e-16}, bhigh = { -220.280934,6.25500571}, R = 37.4496075986769);
          constant IdealGases.Common.DataRecord Rnplus(name = "Rnplus", MM = 0.2220170514, Hf = 4699054.67810388, H0 = 27914.1983055811, Tlimit = 1000, alow = { -0.0969714897,0.001106742047,2.499994937,1.189388239e-08, -1.515895754e-11,9.95893481e-15, -2.640751817e-18}, blow = {124730.476,8.3387617}, ahigh = { -19982.85319,59.3067566,2.432476003,3.71602592e-05, -1.012057848e-08,1.192256661e-12, -3.18452198e-17}, bhigh = {124352.8478,8.82199779}, R = 37.4497001359599);
          constant IdealGases.Common.DataRecord S(name = "S", MM = 0.032065, Hf = 8644004.3661313, H0 = 207622.797442694, Tlimit = 1000, alow = { -317.484182, -192.4704923,4.68682593, -0.0058413656,7.53853352e-06, -4.86358604e-09,1.256976992e-12}, blow = {33235.9218, -5.718523969}, ahigh = { -485424.479,1438.830408,1.258504116,0.000379799043,1.630685864e-09, -9.54709585e-12,8.041466646e-16}, bhigh = {23349.9527,15.59554855}, R = 259.300545766412);
          constant IdealGases.Common.DataRecord Splus(name = "Splus", MM = 0.0320644514, Hf = 39997454.2524061, H0 = 193280.337863507, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {153502.6117,5.43622334}, ahigh = {1346218.684, -4056.87151,7.15343655, -0.002523562352,6.42953961e-07, -6.43167216e-11,2.141387919e-15}, bhigh = {179282.3835, -27.86935079}, R = 259.304982214665);
          constant IdealGases.Common.DataRecord Sminus(name = "Sminus", MM = 0.0320655486, Hf = 2194520.53909347, H0 = 201615.293742394, Tlimit = 1000, alow = { -2596.051473, -142.2398653,4.00782567, -0.00360885591,4.23623e-06, -2.520987604e-09,6.07947976e-13}, blow = {8197.79307, -2.582377345}, ahigh = {2730.311692,141.4072078,2.403340775,3.69357753e-05, -7.94408044e-09,8.95220838e-13, -4.09966282e-17}, bhigh = {6931.1957,6.574986902}, R = 259.296109469962);
          constant IdealGases.Common.DataRecord SCL(name = "SCL", MM = 0.067518, Hf = 2317382.03145828, H0 = 145430.122337747, Tlimit = 1000, alow = {16130.51454, -560.624955,8.06493113, -0.0091863264,1.224205395e-05, -8.20956202e-09,2.205087197e-12}, blow = {19977.3928, -16.9335441}, ahigh = { -94051.2345,362.976085,4.06995936,0.0003034029742, -8.24830139e-08,1.252700734e-11, -6.41746365e-16}, bhigh = {15231.11069,6.014528409}, R = 123.144524423117);
          constant IdealGases.Common.DataRecord SCL2(name = "SCL2", MM = 0.102971, Hf =  -170659.700303969, H0 = 120862.427285352, Tlimit = 1000, alow = {57923.1183, -1062.447681,10.37694738, -0.00613022902,6.463486e-06, -3.67903805e-09,8.73193165e-13}, blow = {1262.475821, -26.91769492}, ahigh = { -234103.9963,428.756958,6.47141539,0.000317947674, -9.74748308e-08,1.39331857e-11, -6.39479517e-16}, bhigh = { -7210.4737, -2.774273151}, R = 80.7457633702693);
          constant IdealGases.Common.DataRecord SCL2plus(name = "SCL2plus", MM = 0.1029704514, Hf = 8753805.91951061, H0 = 120940.005901538, Tlimit = 1000, alow = {49348.7687, -960.164102,9.88479278, -0.00495025717,4.94628697e-06, -2.678720604e-09,6.07684403e-13}, blow = {111281.2037, -23.42664942}, ahigh = { -233940.3605,551.628326,6.12521216,0.000677105632, -2.674382769e-07,4.93593888e-11, -2.955831247e-15}, bhigh = {102678.7381,0.1365611954}, R = 80.7461935628652);
          constant IdealGases.Common.DataRecord SD(name = "SD", MM = 0.034079102, Hf = 4063806.96298864, H0 = 272731.452841686, Tlimit = 1000, alow = { -32955.7499,164.7104687,4.97898455, -0.00803158571,1.653766397e-05, -1.345197981e-08,4.00043153e-12}, blow = {14358.62329, -1.997334017}, ahigh = {245673.249, -1180.034105,5.28469005, -0.0002384319882,6.00996029e-08, -7.13116364e-12,4.0358587e-16}, bhigh = {22655.45124, -8.350483655}, R = 243.975677528123);
          constant IdealGases.Common.DataRecord SF(name = "SF", MM = 0.0510634032, Hf = 302478.096485351, H0 = 185457.948482368, Tlimit = 1000, alow = {12064.56774, -291.027077,5.48010157, -0.001975880235,2.47368599e-06, -1.610904623e-09,4.31003854e-13}, blow = {1991.337574, -4.544138975}, ahigh = {763458.075, -2423.63843,7.35219871, -0.001598026079,4.86851502e-07, -6.47800053e-11,2.98029731e-15}, bhigh = {15774.61591, -19.02584835}, R = 162.826436918721);
          constant IdealGases.Common.DataRecord SFplus(name = "SFplus", MM = 0.0510628546, Hf = 19477372.06999, H0 = 173596.209405809, Tlimit = 1000, alow = {65783.0367, -698.669997,5.62129224, -0.000690927411, -1.28239278e-07,4.02132042e-10, -1.424033453e-13}, blow = {122175.1247, -6.682377715}, ahigh = { -937494.39,2377.21424,2.052929113,0.001081609837, -1.535468024e-07,8.2226919e-12,3.25974688e-17}, bhigh = {102684.9122,18.87705768}, R = 162.828186264385);
          constant IdealGases.Common.DataRecord SFminus(name = "SFminus", MM = 0.0510639518, Hf =  -4530532.49592015, H0 = 173803.763460391, Tlimit = 1000, alow = {50198.7928, -532.154931,4.90484901,0.000991530742, -2.270841545e-06,1.844749756e-09, -5.3570572e-13}, blow = { -26113.88646, -3.637468065}, ahigh = {206892.3125, -1324.372811,6.64621124, -0.001641781588,6.78388312e-07, -1.198331099e-10,7.12173954e-15}, bhigh = { -21509.94816, -14.70239183}, R = 162.824687610644);
          constant IdealGases.Common.DataRecord SF2(name = "SF2", MM = 0.0700618064, Hf =  -4184715.59705603, H0 = 157694.663721945, Tlimit = 1000, alow = {68708.7113, -843.241825,6.2694964,0.00648414245, -1.145126533e-05,8.77029156e-09, -2.542611601e-12}, blow = { -32299.807, -8.80002578}, ahigh = { -170261.2287, -149.7038204,7.11093414, -4.4077416e-05,9.68220831e-09, -1.104468959e-12,5.09530878e-17}, bhigh = { -37042.8949, -10.9525396}, R = 118.67338892935);
          constant IdealGases.Common.DataRecord SF2plus(name = "SF2plus", MM = 0.0700612578, Hf = 10077122.8374949, H0 = 160347.963378985, Tlimit = 1000, alow = {131164.8923, -1552.127914,9.80342472, -0.002488118587,7.50156513e-07,3.135384495e-10, -1.986281446e-13}, blow = {91377.5839, -27.92300566}, ahigh = { -384753.102,532.789302,6.2819089,0.000452279012, -1.426421096e-07,2.079708115e-11, -9.69170989e-16}, bhigh = {78842.1381, -4.21611053}, R = 118.674318176457);
          constant IdealGases.Common.DataRecord SF2minus(name = "SF2minus", MM = 0.070062355, Hf =  -5634908.50400333, H0 = 173058.342101118, Tlimit = 1000, alow = {52845.7248, -1032.342905,10.06404631, -0.00520950325,5.16838137e-06, -2.783420867e-09,6.28641008e-13}, blow = { -44233.0378, -26.9926697}, ahigh = { -93766.1895, -36.3899527,7.02794282, -1.142143365e-05,2.566295898e-09, -2.981125977e-13,1.395700722e-17}, bhigh = { -49670.5033, -8.483087899}, R = 118.672459696794);
          constant IdealGases.Common.DataRecord SF3(name = "SF3", MM = 0.0890602096, Hf =  -5660231.40147651, H0 = 152124.782333771, Tlimit = 1000, alow = {128400.8253, -2084.871223,14.23421102, -0.00401503219,9.15225804e-07,1.109809721e-09, -5.98860272e-13}, blow = { -52395.1734, -51.87194724}, ahigh = { -241128.346, -141.8950375,10.10590548, -4.23152691e-05,9.3361206e-09, -1.068727665e-12,4.9443426e-17}, bhigh = { -63584.4586, -24.89561796}, R = 93.357875950923);
          constant IdealGases.Common.DataRecord SF3plus(name = "SF3plus", MM = 0.089059661, Hf = 4419323.76095615, H0 = 139335.226079515, Tlimit = 1000, alow = {215763.3857, -2564.488498,13.23216044, -0.0001749580869, -3.71295871e-06,3.58068842e-09, -1.084376153e-12}, blow = {58760.8842, -50.24130184}, ahigh = { -377826.075, -162.1770252,9.96026597,0.0001237729568, -6.44220789e-08,1.300856025e-11, -7.91510698e-16}, bhigh = {44154.2681, -26.7390077}, R = 93.3584510275645);
          constant IdealGases.Common.DataRecord SF3minus(name = "SF3minus", MM = 0.0890607582, Hf =  -8871743.54866429, H0 = 154084.61905504, Tlimit = 1000, alow = {154650.0675, -2469.768526,16.57548496, -0.01020137434,9.36800846e-06, -4.7256726e-09,1.009661431e-12}, blow = { -85001.579, -65.32166474}, ahigh = { -225386.0964, -130.8890242,10.0997238, -4.05234553e-05,9.06456522e-09, -1.049320641e-12,4.89920126e-17}, bhigh = { -98002.272, -25.24368023}, R = 93.3573008813662);
          constant IdealGases.Common.DataRecord SF4(name = "SF4", MM = 0.1080586128, Hf =  -7033220.03037966, H0 = 142356.833957061, Tlimit = 1000, alow = {163618.3265, -2531.316261,15.59106568,0.00332670431, -1.054935525e-05,9.39136305e-09, -2.914816694e-12}, blow = { -81155.5892, -61.31532958}, ahigh = { -377712.516, -293.7595659,13.21902007, -8.74766945e-05,1.929905265e-08, -2.209423506e-12,1.022335559e-16}, bhigh = { -94831.7534, -42.43061278}, R = 76.944093437409);
          constant IdealGases.Common.DataRecord SF4plus(name = "SF4plus", MM = 0.1080580642, Hf = 3850818.21593469, H0 = 152747.017283695, Tlimit = 1000, alow = {230958.7324, -3383.77767,21.01756367, -0.01106529555,9.03809728e-06, -4.05591264e-09,7.71257518e-13}, blow = {64253.7515, -89.38224908}, ahigh = { -418917.275,40.0288964,12.83873264,0.0001340235036, -4.81813518e-08,7.57459596e-12, -3.69667078e-16}, bhigh = {44699.4423, -37.60378598}, R = 76.9444840748868);
          constant IdealGases.Common.DataRecord SF4minus(name = "SF4minus", MM = 0.1080591614, Hf =  -8212763.34650511, H0 = 171020.399941768, Tlimit = 1000, alow = {100552.3355, -2267.351994,19.7928456, -0.01163761842,1.161792139e-05, -6.28889547e-09,1.426371272e-12}, blow = { -98955.9743, -79.16366078}, ahigh = {320882.208, -1710.416599,14.94425485, -0.001057207329,2.74284692e-07, -2.905761211e-11,1.084482543e-15}, bhigh = { -100523.1225, -51.64256578}, R = 76.9437028038976);
          constant IdealGases.Common.DataRecord SF5(name = "SF5", MM = 0.127057016, Hf =  -7104395.45502942, H0 = 148055.413169785, Tlimit = 1000, alow = {222417.685, -4043.99175,27.07196786, -0.01744176604,1.604198715e-05, -7.99724302e-09,1.667968061e-12}, blow = { -92200.7827, -123.2426279}, ahigh = { -389242.637, -182.5826536,16.13765057, -5.54483423e-05,1.231433706e-08, -1.417173409e-12,6.5848894e-17}, bhigh = { -113567.1054, -55.77524201}, R = 65.4389050030893);
          constant IdealGases.Common.DataRecord SF5plus(name = "SF5plus", MM = 0.1270564674, Hf = 1358797.46645624, H0 = 128662.037710644, Tlimit = 1000, alow = {377713.562, -5646.07512,30.4940427, -0.02176488368,1.941993792e-05, -9.55620114e-09,2.000077302e-12}, blow = {45922.0481, -148.9889627}, ahigh = { -981434.232,720.881739,15.64386818, -0.0002452519972,2.241690996e-07, -3.99003589e-11,2.279087545e-15}, bhigh = {9090.95319, -55.31460631}, R = 65.4391875529195);
          constant IdealGases.Common.DataRecord SF5minus(name = "SF5minus", MM = 0.1270575646, Hf =  -9480917.49430557, H0 = 149931.356389307, Tlimit = 1000, alow = {209503.1134, -3930.08114,27.02127215, -0.01792973203,1.718122152e-05, -9.00056019e-09,1.988018147e-12}, blow = { -129181.6581, -123.1672857}, ahigh = { -373321.92, -179.3729521,16.13699748, -5.57753997e-05,1.249477046e-08, -1.448104799e-12,6.7674623e-17}, bhigh = { -149855.4401, -56.31439391}, R = 65.4386224556991);
          constant IdealGases.Common.DataRecord SF6(name = "SF6", MM = 0.1460554192, Hf =  -8348885.69475278, H0 = 115983.830608868, Tlimit = 1000, alow = {330952.674, -4737.68505,22.47738068,0.01046954309, -2.560641961e-05,2.153716967e-08, -6.51609896e-12}, blow = { -125536.0583, -109.1760145}, ahigh = { -730672.65, -636.705655,19.47442853, -0.0001894325671,4.17872283e-08, -4.78374495e-12,2.213516129e-16}, bhigh = { -151060.9837, -81.47574587}, R = 56.9268298673303);
          constant IdealGases.Common.DataRecord SF6minus(name = "SF6minus", MM = 0.1460559678, Hf =  -9187409.82112749, H0 = 119753.504519245, Tlimit = 1000, alow = {498580.921, -6934.44574,34.4131833, -0.0198403236,1.497871222e-05, -6.13323465e-09,1.045159581e-12}, blow = { -129706.9066, -174.7984495}, ahigh = { -681807.351, -594.742996,19.45009842, -0.0001820212649,4.05707957e-08, -4.68382093e-12,2.182263999e-16}, bhigh = { -165890.2452, -79.65845887}, R = 56.9266160447845);
          constant IdealGases.Common.DataRecord SH(name = "SH", MM = 0.03307294, Hf = 4297631.50781273, H0 = 275092.235525478, Tlimit = 1000, alow = {6389.43468, -374.796092,7.54814577, -0.01288875477,1.907786343e-05, -1.265033728e-08,3.23515869e-12}, blow = {17429.02395, -17.60761843}, ahigh = {1682631.601, -5177.15221,9.19816852, -0.002323550224,6.54391478e-07, -8.46847042e-11,3.86474155e-15}, bhigh = {48992.1449, -37.70400275}, R = 251.398031139657);
          constant IdealGases.Common.DataRecord SHminus(name = "SHminus", MM = 0.0330734886, Hf =  -2617628.18694609, H0 = 261420.985991783, Tlimit = 1000, alow = {38780.7076, -574.25906,6.89854446, -0.01001352412,1.486460572e-05, -9.7503689e-09,2.446909813e-12}, blow = { -8735.38898, -16.15966489}, ahigh = {1198715.402, -3894.84682,7.66042233, -0.00135523759,3.34237024e-07, -3.35072231e-11,9.05508478e-16}, bhigh = {13174.77387, -28.18370616}, R = 251.393861124163);
          constant IdealGases.Common.DataRecord SN(name = "SN", MM = 0.0460717, Hf = 5803743.14383884, H0 = 203880.321325239, Tlimit = 1000, alow = { -68354.1235,1147.567483, -2.877802574,0.0172486432, -2.058999904e-05,1.26136964e-08, -3.139030141e-12}, blow = {25641.43612,42.24006964}, ahigh = { -483728.446,1058.07559,3.086198804,0.000911136078, -2.764061722e-07,4.15737011e-11, -2.128351755e-15}, bhigh = {23793.45477,10.33222139}, R = 180.468096467029);
          constant IdealGases.Common.DataRecord SO(name = "SO", MM = 0.0480644, Hf = 99040.1627816013, H0 = 183048.285217333, Tlimit = 1000, alow = { -33427.57,640.38625, -1.006641228,0.01381512705, -1.704486364e-05,1.06129493e-08, -2.645796205e-12}, blow = { -3371.29219,30.93861963}, ahigh = { -1443410.557,4113.87436, -0.538369578,0.002794153269, -6.63335226e-07,7.83822119e-11, -3.56050907e-15}, bhigh = { -27088.38059,36.15358329}, R = 172.98607701334);
          constant IdealGases.Common.DataRecord SOminus(name = "SOminus", MM = 0.0480649486, Hf =  -2204685.3078295, H0 = 196965.02910647, Tlimit = 1000, alow = {8420.19697, -69.1658668,3.83766661,0.002166482062, -2.811222562e-06,1.793426453e-09, -4.52179595e-13}, blow = { -13541.62531,4.316141603}, ahigh = {176715.6147, -663.398736,5.17727981, -0.0002853461125,7.21442153e-08, -3.67646949e-12, -2.910092894e-16}, bhigh = { -9984.43801, -3.951456757}, R = 172.984102598208);
          constant IdealGases.Common.DataRecord SOF2(name = "SOF2", MM = 0.0860612064, Hf =  -6796933.68788286, H0 = 146696.119286564, Tlimit = 1000, alow = {61145.7965, -908.111967,6.88362326,0.01192459695, -1.67100641e-05,1.110723646e-08, -2.910788226e-12}, blow = { -67429.3584, -11.25458745}, ahigh = { -251055.5779, -607.443961,10.45046709, -0.0001796384979,3.96416973e-08, -4.54305524e-12,2.104969923e-16}, bhigh = { -70720.7171, -29.04334504}, R = 96.6111485976102);
          constant IdealGases.Common.DataRecord SO2(name = "SO2", MM = 0.0640638, Hf =  -4633037.69055223, H0 = 164650.348558781, Tlimit = 1000, alow = { -53108.4214,909.031167, -2.356891244,0.02204449885, -2.510781471e-05,1.446300484e-08, -3.36907094e-12}, blow = { -41137.5208,40.45512519}, ahigh = { -112764.0116, -825.226138,7.61617863, -0.000199932761,5.65563143e-08, -5.45431661e-12,2.918294102e-16}, bhigh = { -33513.0869, -16.55776085}, R = 129.78424632944);
          constant IdealGases.Common.DataRecord SO2minus(name = "SO2minus", MM = 0.0640643486, Hf =  -6378057.030615, H0 = 167791.154283273, Tlimit = 1000, alow = {94609.0166, -856.269105,5.36007422,0.00760957674, -1.092005659e-05,7.20254773e-09, -1.85013822e-12}, blow = { -45800.9481, -3.929956604}, ahigh = { -179340.0005, -366.316876,7.27444105, -0.0001101944663,2.443328132e-08, -2.809735019e-12,1.305160431e-16}, bhigh = { -49730.7261, -12.61421156}, R = 129.783134952534);
          constant IdealGases.Common.DataRecord SO2CL2(name = "SO2CL2", MM = 0.1349698, Hf =  -2628759.91518103, H0 = 118757.707279703, Tlimit = 1000, alow = {6821.59239, -584.95692,8.59675691,0.01197228675, -1.3337607e-05,7.126129e-09, -1.496150016e-12}, blow = { -42307.8381, -16.52458363}, ahigh = { -237663.9109, -964.774041,13.71469904, -0.0002850327057,6.29350962e-08, -7.21784426e-12,3.34684022e-16}, bhigh = { -41900.7299, -44.81942575}, R = 61.6024621804285);
          constant IdealGases.Common.DataRecord SO2FCL(name = "SO2FCL", MM = 0.1185152032, Hf =  -4695363.84341279, H0 = 124042.997042256, Tlimit = 1000, alow = {36845.613, -803.835895,7.52618688,0.01620141782, -1.97059527e-05,1.16625033e-08, -2.759241465e-12}, blow = { -65035.9819, -12.99745646}, ahigh = { -291540.2438, -1107.134423,13.81845158, -0.000325851446,7.184741e-08, -8.2306252e-12,3.81292562e-16}, bhigh = { -65524.7269, -47.09166446}, R = 70.1553199547651);
          constant IdealGases.Common.DataRecord SO2F2(name = "SO2F2", MM = 0.1020606064, Hf =  -7446555.79471454, H0 = 132180.784299161, Tlimit = 1000, alow = {55331.5353, -807.463968,5.28717216,0.02294547775, -2.891025299e-05,1.785631907e-08, -4.4189195e-12}, blow = { -88994.2882, -4.124256607}, ahigh = { -340390.445, -1309.288764,13.96601541, -0.000384008815,8.45668878e-08, -9.67824687e-12,4.48001428e-16}, bhigh = { -89019.9414, -51.0927005}, R = 81.4660258573576);
          constant IdealGases.Common.DataRecord SO3(name = "SO3", MM = 0.0800632, Hf =  -4944843.57357687, H0 = 145990.904685299, Tlimit = 1000, alow = { -39528.5529,620.857257, -1.437731716,0.02764126467, -3.144958662e-05,1.792798e-08, -4.12638666e-12}, blow = { -51841.0617,33.91331216}, ahigh = { -216692.3781, -1301.022399,10.96287985, -0.000383710002,8.46688904e-08, -9.70539929e-12,4.49839754e-16}, bhigh = { -43982.8399, -36.55217314}, R = 103.848859401073);
          constant IdealGases.Common.DataRecord S2(name = "S2", MM = 0.06413, Hf = 2005301.73085919, H0 = 142399.968813348, Tlimit = 1000, alow = {35280.9178, -422.215658,4.67743349,0.001724046361, -3.86220821e-06,3.33615634e-09, -9.93066154e-13}, blow = {16547.67715, -0.7957279032}, ahigh = { -15881.28788,631.548088,2.449628069,0.001986240565, -6.50792724e-07,1.002813651e-10, -5.59699005e-15}, bhigh = {10855.08427,14.58544515}, R = 129.650272883206);
          constant IdealGases.Common.DataRecord S2minus(name = "S2minus", MM = 0.0641305486, Hf =  -579012.433397459, H0 = 149649.663218381, Tlimit = 1000, alow = {10255.58251, -616.612825,8.18205284, -0.00831120358,9.72027416e-06, -5.77547678e-09,1.393690695e-12}, blow = { -3063.559874, -19.06068041}, ahigh = {483020.403, -1319.171302,6.03177848, -0.000796556069,2.28224169e-07, -2.504364698e-11,7.28055078e-16}, bhigh = {2660.25849, -9.010480319}, R = 129.649163799606);
          constant IdealGases.Common.DataRecord S2CL2(name = "S2CL2", MM = 0.135036, Hf =  -123937.320418259, H0 = 122343.382505406, Tlimit = 1000, alow = {79749.7474, -1636.770024,15.83744686, -0.01186906072,1.413008027e-05, -8.48490393e-09,1.981743887e-12}, blow = {3276.87242, -52.93745205}, ahigh = {632881.6, -3442.7058,15.1963735, -0.002960877176,6.88828437e-07, -7.99069253e-11,3.69246606e-15}, bhigh = {15266.72656, -54.53315445}, R = 61.5722622115584);
          constant IdealGases.Common.DataRecord S2F2(name = "S2F2", MM = 0.1021268064, Hf =  -3930535.12735712, H0 = 134322.147960557, Tlimit = 1000, alow = {125290.4041, -1941.243874,13.25382183, -0.001341525124, -2.772465632e-06,3.65816929e-09, -1.299531209e-12}, blow = { -40672.0825, -45.60828617}, ahigh = { -259175.7719, -105.9653069,10.06588551, -2.274618148e-05,5.41234314e-09, -9.91138714e-13,9.62186144e-17}, bhigh = { -51476.7518, -23.7566614}, R = 81.4132184593603);
          constant IdealGases.Common.DataRecord S2O(name = "S2O", MM = 0.0801294, Hf =  -699312.549451263, H0 = 138882.320346839, Tlimit = 1000, alow = {10927.0331, -95.2309987,3.14452543,0.011768542, -1.58026684e-05,1.03764504e-08, -2.70862226e-12}, blow = { -7500.4719,11.04169896}, ahigh = { -144213.979, -327.643013,7.24428611, -9.7765383e-05,2.1627127e-08, -2.48278222e-12,1.15177875e-16}, bhigh = { -7438.55393, -10.85180744}, R = 103.763063245201);
          constant IdealGases.Common.DataRecord S3(name = "S3", MM = 0.096195, Hf = 1504634.6275794, H0 = 124479.733873902, Tlimit = 1000, alow = {72453.9574, -1162.146759,9.95541368, -0.00415802622,3.24177839e-06, -1.264648239e-09,1.777450535e-13}, blow = {21462.75475, -25.87525865}, ahigh = { -111780.5401, -51.9790839,7.03876084, -1.546804022e-05,3.40834041e-09, -3.89686236e-13,1.800860389e-17}, bhigh = {15254.06485, -7.610045099}, R = 86.4335152554707);
          constant IdealGases.Common.DataRecord S4(name = "S4", MM = 0.12826, Hf = 1057479.59613285, H0 = 111338.944331826, Tlimit = 1000, alow = {119866.4135, -2040.786521,15.10235054, -0.0070411043,5.350405e-06, -2.002348038e-09,2.569940995e-13}, blow = {24109.09409, -55.03153238}, ahigh = { -206833.3537, -96.9615143,10.0724321, -2.895047053e-05,6.38780619e-09, -7.31176743e-13,3.38226529e-17}, bhigh = {13211.0953, -23.44872237}, R = 64.825136441603);
          constant IdealGases.Common.DataRecord S5(name = "S5", MM = 0.160325, Hf = 829523.492905037, H0 = 118842.06455637, Tlimit = 1000, alow = {136137.0439, -2955.476536,26.33358816, -0.0412580653,7.05681403e-05, -5.61109533e-08,1.6594572e-11}, blow = {26753.07046, -106.9711067}, ahigh = { -4038495.16,8601.80388,7.4437809,0.001533393812, -2.532718676e-07,2.145210795e-11, -7.21310268e-16}, bhigh = { -46869.2542,11.04229196}, R = 51.8601091532824);
          constant IdealGases.Common.DataRecord S6(name = "S6", MM = 0.19239, Hf = 526613.545402568, H0 = 118442.637351214, Tlimit = 1000, alow = {97803.0721, -2568.47013,24.67025557, -0.01619983175,1.712334526e-05, -9.74629674e-09,2.486964867e-12}, blow = {20378.88389, -101.4410309}, ahigh = {3686845.06, -7695.0179,18.60721995,0.002290382548, -1.219834021e-06,2.060780798e-10, -1.190354318e-14}, bhigh = {60324.8648, -74.90245718}, R = 43.2167576277353);
          constant IdealGases.Common.DataRecord S7(name = "S7", MM = 0.224455, Hf = 498498.001826647, H0 = 117058.5106146, Tlimit = 1000, alow = {123365.5613, -3200.54364,30.15678887, -0.02197472483,2.487892075e-05, -1.506049557e-08,3.76892599e-12}, blow = {23900.05896, -127.5978171}, ahigh = { -272639.6041, -73.7321522,19.05771987, -2.394575885e-05,5.4426713e-09, -6.37966348e-13,3.008212993e-17}, bhigh = {7308.45883, -61.59112428}, R = 37.0429351094874);
          constant IdealGases.Common.DataRecord S8(name = "S8", MM = 0.25652, Hf = 394811.796351162, H0 = 123082.496491502, Tlimit = 1000, alow = {314562.5719, -6116.51016,48.7532754, -0.0624179465,7.42183175e-05, -3.72644931e-08,5.79942988e-12}, blow = {35738.9084, -228.8701977}, ahigh = { -8727921.13,12216.27968,22.09959617, -0.00349406483,1.397604162e-06, -2.169815281e-10,1.212304364e-14}, bhigh = { -86581.1082, -64.35742508}, R = 32.4125682208015);
          constant IdealGases.Common.DataRecord Sc(name = "Sc", MM = 0.04495591, Hf = 8401570.76121916, H0 = 155758.364139442, Tlimit = 1000, alow = { -3700.80594,169.2506026,1.842242597,0.001364835821, -1.580085847e-06,9.61311115e-10, -2.392381918e-13}, blow = {43852.1524,10.72781921}, ahigh = {8810382.65, -27112.32975,34.7658866, -0.01861104581,5.2902839e-06, -6.58540806e-10,2.997850429e-14}, bhigh = {216246.0097, -222.5618519}, R = 184.947251651674);
          constant IdealGases.Common.DataRecord Scplus(name = "Scplus", MM = 0.0449553614, Hf = 22625671.1173942, H0 = 159311.854625642, Tlimit = 1000, alow = { -5884.93016,147.9044408,2.009576456,0.000738956697, -6.61796482e-07,6.84107376e-10, -2.164125532e-13}, blow = {120843.9139,10.2657062}, ahigh = {1973658.531, -4954.3841,6.36062735, -0.000716878592,2.464991123e-08,9.63237379e-12, -8.544709642e-16}, bhigh = {154342.3764, -22.61925782}, R = 184.949508602994);
          constant IdealGases.Common.DataRecord Scminus(name = "Scminus", MM = 0.0449564586, Hf = 7842228.65810876, H0 = 137854.007922234, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {41657.4639,8.27042072}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = {41657.4639,8.27042072}, R = 184.944994755437);
          constant IdealGases.Common.DataRecord ScO(name = "ScO", MM = 0.06095531, Hf =  -903371.568449082, H0 = 144156.53041548, Tlimit = 1000, alow = { -3136.784613,227.1981685,0.885075732,0.01052430774, -1.431954596e-05,9.65235107e-09, -2.586210633e-12}, blow = { -8550.79877,20.12706847}, ahigh = {1224192.443, -3918.91572,8.67984881, -0.001995893306,4.06698985e-07, -1.504459913e-11, -8.4729989e-16}, bhigh = {16773.22594, -29.51656272}, R = 136.402751458405);
          constant IdealGases.Common.DataRecord ScOplus(name = "ScOplus", MM = 0.0609547614, Hf = 9206991.43283005, H0 = 143993.115523868, Tlimit = 1000, alow = {45885.6027, -336.711437,3.5272064,0.0039888503, -5.56898543e-06,3.65028119e-09, -9.3564915e-13}, blow = {68383.5079,4.3398127}, ahigh = { -1814357.141,5853.42532, -3.65710121,0.00573436451, -2.092878161e-06,3.69564502e-10, -2.214038347e-14}, bhigh = {29553.64078,56.6088941}, R = 136.403979099162);
          constant IdealGases.Common.DataRecord ScO2(name = "ScO2", MM = 0.07695471, Hf =  -5375248.20767956, H0 = 139720.271832614, Tlimit = 1000, alow = {47947.0096, -429.835859,3.4172065,0.01361338544, -2.047465023e-05,1.455906041e-08, -4.03808769e-12}, blow = { -48610.4953,7.67001333}, ahigh = { -190390.2542, -231.558882,7.17210491, -6.85920964e-05,1.510998557e-08, -1.727951588e-12,7.9889805e-17}, bhigh = { -51129.7475, -10.90764597}, R = 108.043705187116);
          constant IdealGases.Common.DataRecord Sc2O(name = "Sc2O", MM = 0.10591122, Hf =  -217580.422546355, H0 = 110562.827998771, Tlimit = 1000, alow = {25500.02615, -307.7608607,4.67121504,0.00869614225, -1.283873116e-05,8.98896681e-09, -2.461994147e-12}, blow = { -2614.951944,2.92001718}, ahigh = { -135088.0396, -185.4540692,7.13829408, -5.52943214e-05,1.221589315e-08, -1.400490463e-12,6.48885811e-17}, bhigh = { -4240.23514, -9.37271095}, R = 78.5041660364218);
          constant IdealGases.Common.DataRecord Sc2O2(name = "Sc2O2", MM = 0.12191062, Hf =  -4024022.02531658, H0 = 107732.861993483, Tlimit = 1000, alow = {3468.44549,261.7988314, -0.3052099624,0.0307610407, -4.18347836e-05,2.794218425e-08, -7.41267205e-12}, blow = { -61440.1321,29.15248555}, ahigh = { -297734.8677, -580.367792,10.43318377, -0.0001734165462,3.83608979e-08, -4.40307803e-12,2.042208922e-16}, bhigh = { -59648.6628, -28.55828369}, R = 68.2013757292023);
          constant IdealGases.Common.DataRecord Si(name = "Si", MM = 0.0280855, Hf = 16022502.7149241, H0 = 268831.176229727, Tlimit = 1000, alow = {98.3614081,154.6544523,1.87643667,0.001320637995, -1.529720059e-06,8.95056277e-10, -1.95287349e-13}, blow = {52635.1031,9.69828888}, ahigh = { -616929.885,2240.683927, -0.444861932,0.001710056321, -4.10771416e-07,4.55888478e-11, -1.889515353e-15}, bhigh = {39535.5876,26.79668061}, R = 296.041444873689);
          constant IdealGases.Common.DataRecord Siplus(name = "Siplus", MM = 0.0280849514, Hf = 44241060.8907089, H0 = 261454.787491639, Tlimit = 1000, alow = { -43297.9188,679.589449,0.2257046144,0.00411860049, -4.2348816e-06,2.327995626e-09, -5.318388059e-13}, blow = {145203.9813,19.3465051}, ahigh = {59193.9023, -48.5673095,2.556312024, -3.50339716e-05,1.190298787e-08, -2.082923821e-12,1.471452049e-16}, bhigh = {149143.1392,5.24426714}, R = 296.047227626678);
          constant IdealGases.Common.DataRecord Siminus(name = "Siminus", MM = 0.0280860486, Hf = 10995406.7372795, H0 = 220658.594174761, Tlimit = 1000, alow = { -794.014667,5.56741842,2.499837183, -9.48139446e-05,3.17124693e-07, -4.19132318e-10,2.03592438e-13}, blow = {36364.4338,5.27011984}, ahigh = { -6162070.1,18833.10402, -18.9930245,0.01111021657, -2.535790208e-06,2.699962923e-10, -1.105062911e-14}, bhigh = { -83140.8931,159.5298253}, R = 296.035662346607);
          constant IdealGases.Common.DataRecord SiBr(name = "SiBr", MM = 0.1079895, Hf = 1621985.66527301, H0 = 92926.821589136, Tlimit = 1000, alow = {7370.28335, -505.106296,8.28059104, -0.0100538392,1.35705092e-05, -9.1154499e-09,2.447477685e-12}, blow = {21844.05475, -16.62032657}, ahigh = {1317799.94, -3669.79502,8.51168612, -0.001987602041,5.03541489e-07, -4.49012019e-11,8.23567007e-16}, bhigh = {43328.5155, -24.67075472}, R = 76.9933373152019);
          constant IdealGases.Common.DataRecord SiBr2(name = "SiBr2", MM = 0.1878935, Hf =  -271430.358155019, H0 = 70876.1133301578, Tlimit = 1000, alow = {23866.16151, -641.569347,9.34587911, -0.00479290414,5.58253847e-06, -3.45531238e-09,8.80033894e-13}, blow = { -5014.75702, -17.40767596}, ahigh = { -51473.4394, -12.77719997,7.01020224, -4.29633186e-06,9.87765197e-10, -1.168133105e-13,5.54668138e-18}, bhigh = { -8319.54225, -3.61182661}, R = 44.2509826045073);
          constant IdealGases.Common.DataRecord SiBr3(name = "SiBr3", MM = 0.2677975, Hf =  -586263.874756112, H0 = 65203.8947338941, Tlimit = 1000, alow = {39728.0686, -1066.475023,13.80836232, -0.00764111472,8.77586583e-06, -5.37243085e-09,1.356453455e-12}, blow = { -16517.95623, -37.8405537}, ahigh = { -88880.2474, -22.67879074,10.01791167, -7.48115075e-06,1.70924269e-09, -2.011596627e-13,9.51553415e-18}, bhigh = { -22030.26938, -15.37574366}, R = 31.0476087342115);
          constant IdealGases.Common.DataRecord SiBr4(name = "SiBr4", MM = 0.3477015, Hf =  -1195853.33971812, H0 = 64174.9345343635, Tlimit = 1000, alow = {68210.8674, -1520.370776,18.1045336, -0.00973468842,1.072367466e-05, -6.34385829e-09,1.5572282e-12}, blow = { -46165.8521, -59.7612063}, ahigh = { -127140.9281, -37.6808795,13.02910642, -1.194958251e-05,2.693843051e-09, -3.137193207e-13,1.471642663e-17}, bhigh = { -54089.9883, -29.4038433}, R = 23.9126722202809);
          constant IdealGases.Common.DataRecord SiC(name = "SiC", MM = 0.0400962, Hf = 18329570.8820287, H0 = 229874.925803443, Tlimit = 1000, alow = { -6223.33089,314.1790457,0.389313083,0.0118750754, -1.639277197e-05,1.131808223e-08, -3.045324231e-12}, blow = {86062.2773,23.10166717}, ahigh = { -62688.0603,720.983692,2.162879732,0.002201299585, -6.56946659e-07,9.17711026e-11, -4.96916674e-15}, bhigh = {83212.2585,16.01675317}, R = 207.363091764307);
          constant IdealGases.Common.DataRecord SiC2(name = "SiC2", MM = 0.0521069, Hf = 12116647.4689533, H0 = 224258.380368051, Tlimit = 1000, alow = { -41196.2899,686.974718,0.02361343691,0.01532939217, -1.435588838e-05,6.37522874e-09, -1.059204957e-12}, blow = {71308.9071,28.28850316}, ahigh = {7026893.09, -24661.48437,39.1545303, -0.02002884068,6.30735369e-06, -8.84838351e-10,4.53054985e-14}, bhigh = {226732.5155, -236.6622024}, R = 159.5656621292);
          constant IdealGases.Common.DataRecord SiCL(name = "SiCL", MM = 0.0635385, Hf = 2240581.99359444, H0 = 155561.100749939, Tlimit = 1000, alow = {12485.08319, -204.4170347,4.97707085, -0.000516075626,2.931980374e-07, -3.66661523e-12, -3.39303713e-14}, blow = {16865.28019, -0.226395724}, ahigh = {376729.456, -1147.956509,5.73540843, -0.000578489081,1.517304648e-07, -1.291391394e-11,1.55462842e-16}, bhigh = {23062.78401, -6.098515228}, R = 130.857228294656);
          constant IdealGases.Common.DataRecord SiCL2(name = "SiCL2", MM = 0.0989915, Hf =  -1647305.9505109, H0 = 126569.392321563, Tlimit = 1000, alow = {53055.5097, -1015.842786,10.32848185, -0.00621226752,6.7158254e-06, -3.90880164e-09,9.46173215e-13}, blow = { -16502.18093, -26.49925756}, ahigh = { -80508.7157, -26.61025523,7.02041472, -8.33582472e-06,1.871092411e-09, -2.171552767e-13,1.015849895e-17}, bhigh = { -21812.94885, -6.638764238}, R = 83.9917770717688);
          constant IdealGases.Common.DataRecord SiCL3(name = "SiCL3", MM = 0.1344445, Hf =  -2501193.20611851, H0 = 116906.731030276, Tlimit = 1000, alow = {86917.0846, -1677.556857,15.1151906, -0.008937031,9.09735353e-06, -5.01457996e-09,1.155962016e-12}, blow = { -34774.773, -50.87215506}, ahigh = { -147718.0114, -53.3520098,10.04043521, -1.63512106e-05,3.64194011e-09, -4.20060059e-13,1.955201587e-17}, bhigh = { -43605.7165, -20.06632444}, R = 61.8431546102667);
          constant IdealGases.Common.DataRecord SiCL4(name = "SiCL4", MM = 0.1698975, Hf =  -3897644.16780706, H0 = 114512.797421975, Tlimit = 1000, alow = {117164.6285, -2185.821461,19.08250496, -0.00963633572,8.83684769e-06, -4.3600424e-09,8.94023243e-13}, blow = { -72128.0211, -73.01532221}, ahigh = { -210033.0761, -84.7610771,13.06364027, -2.55402042e-05,5.65381847e-09, -6.48875267e-13,3.008025543e-17}, bhigh = { -83722.6813, -35.92584781}, R = 48.938165658706);
          constant IdealGases.Common.DataRecord SiF(name = "SiF", MM = 0.0470839032, Hf =  -535908.968566565, H0 = 200835.388685448, Tlimit = 1000, alow = {14782.23768,13.57998649,2.324182643,0.00713285118, -1.022951215e-05,7.14839647e-09, -1.964689785e-12}, blow = { -3995.38065,12.31128121}, ahigh = { -365363.788,872.286004,3.37721827,0.000750815243, -2.308898619e-07,3.7667818e-11, -2.143080403e-15}, bhigh = { -10116.97664,8.96813282}, R = 176.588418438512);
          constant IdealGases.Common.DataRecord SiFCL(name = "SiFCL", MM = 0.0825369032, Hf =  -4577667.70197891, H0 = 143364.768257988, Tlimit = 1000, alow = {41769.5464, -592.657538,6.40977071,0.00464168261, -7.97846191e-06,6.00872837e-09, -1.720953157e-12}, blow = { -43982.9607, -6.259737448}, ahigh = { -128469.8945, -124.7550173,7.09289385, -3.70731177e-05,8.17517301e-09, -9.35643521e-13,4.32854434e-17}, bhigh = { -47233.0972, -8.337392708}, R = 100.73641822801);
          constant IdealGases.Common.DataRecord SiF2(name = "SiF2", MM = 0.0660823064, Hf =  -8971207.88144888, H0 = 169553.252759955, Tlimit = 1000, alow = {40532.3665, -399.763756,4.05530388,0.01153423049, -1.757586754e-05,1.260369111e-08, -3.51710522e-12}, blow = { -70477.7046,3.87985442}, ahigh = { -167520.0202, -184.621209,7.13698635, -5.45076986e-05,1.199012961e-08, -1.369470187e-12,6.32488999e-17}, bhigh = { -72874.0102, -11.17933348}, R = 125.819942628395);
          constant IdealGases.Common.DataRecord SiF3(name = "SiF3", MM = 0.0850807096, Hf =  -11711662.7456995, H0 = 153014.168090577, Tlimit = 1000, alow = {57553.6998, -717.959577,5.37939694,0.01813065142, -2.738373356e-05,1.94651947e-08, -5.39079056e-12}, blow = { -117763.1169, -3.44290204}, ahigh = { -290328.3702, -342.787694,10.25536335, -0.000101979669,2.250326132e-08, -2.577117181e-12,1.192923177e-16}, bhigh = { -121808.8967, -27.25261142}, R = 97.7245257954454);
          constant IdealGases.Common.DataRecord SiF4(name = "SiF4", MM = 0.1040791128, Hf =  -15524536.638825, H0 = 147634.934489949, Tlimit = 1000, alow = {10696.70784, -95.6963864,2.513988593,0.0330978103, -4.57304408e-05,3.091834452e-08, -8.26930405e-12}, blow = { -195625.2529,11.33722022}, ahigh = { -384589.232, -656.997991,13.4892572, -5.10047136e-05,5.34632958e-08, -4.95959811e-12,2.299314375e-16}, bhigh = { -195728.4479, -46.7389465}, R = 79.8860768152128);
          constant IdealGases.Common.DataRecord SiH(name = "SiH", MM = 0.02909344, Hf = 12670767.3619895, H0 = 314335.809034614, Tlimit = 1000, alow = { -6426.6763,74.1725121,3.9734916, -0.00414940888,1.022918384e-05, -8.59238636e-09,2.567093743e-12}, blow = {42817.458,2.24693715}, ahigh = {404208.649, -2364.796524,7.62749914, -0.002496591233,1.10843641e-06, -1.943991955e-10,1.136251507e-14}, bhigh = {57047.3768, -24.48054429}, R = 285.785111695283);
          constant IdealGases.Common.DataRecord SiHplus(name = "SiHplus", MM = 0.0290928914, Hf = 39448508.2703055, H0 = 297469.951714734, Tlimit = 1000, alow = { -43071.5717,388.495824,2.546166863, -0.000668632598,5.18627702e-06, -4.81856974e-09,1.44570264e-12}, blow = {134907.9707,9.02795009}, ahigh = {171169.2033, -1045.807287,4.83864824,0.0001277940891, -6.88387245e-08,1.41884348e-11, -7.85531696e-16}, bhigh = {143172.145, -7.55329914}, R = 285.790500699425);
          constant IdealGases.Common.DataRecord SiHBr3(name = "SiHBr3", MM = 0.26880544, Hf =  -1126919.15758848, H0 = 66306.057645262, Tlimit = 1000, alow = {132281.1122, -2205.365507,17.75084194, -0.01047794569,1.286821723e-05, -7.74312426e-09,1.819060891e-12}, blow = { -28349.92682, -63.3119954}, ahigh = {219997.3526, -2024.458552,14.35781007, -0.000503701584,1.054049755e-07, -1.160828765e-11,5.21725768e-16}, bhigh = { -28420.86214, -45.3643717}, R = 30.9311894878318);
          constant IdealGases.Common.DataRecord SiHCL(name = "SiHCL", MM = 0.06454644, Hf = 851261.711722598, H0 = 165236.270195537, Tlimit = 1000, alow = {59096.7305, -878.006403,7.99968793, -0.00466314127,8.594986e-06, -6.60807355e-09,1.868076396e-12}, blow = {9567.57034, -16.90014311}, ahigh = {101215.289, -1163.239598,7.37636655,0.0001874151313, -1.661351093e-07,3.96441404e-11, -2.64058008e-15}, bhigh = {11410.64087, -15.07678798}, R = 128.813796702033);
          constant IdealGases.Common.DataRecord SiHCL3(name = "SiHCL3", MM = 0.13545244, Hf =  -3663440.83576494, H0 = 119241.912511875, Tlimit = 1000, alow = {165956.5586, -2552.498122,17.50117375, -0.00788572866,8.09256749e-06, -3.9804206e-09,7.05917332e-13}, blow = { -49513.166, -67.58519053}, ahigh = {172864.4072, -2116.727327,14.41802307, -0.000525524962,1.098826178e-07, -1.209327059e-11,5.43218705e-16}, bhigh = { -51303.5194, -50.36360893}, R = 61.3829621673851);
          constant IdealGases.Common.DataRecord SiHF(name = "SiHF", MM = 0.0480918432, Hf =  -3382219.48207633, H0 = 212722.788716071, Tlimit = 1000, alow = {21925.98671, -61.7522908,2.127372739,0.01123538312, -1.322503985e-05,8.30730739e-09, -2.010545971e-12}, blow = { -20169.94845,13.67184876}, ahigh = {4049752.24, -9295.52682,10.71754366,0.00186812422, -1.129820372e-06,1.9602501e-10, -1.144682011e-14}, bhigh = {41332.252, -47.363176}, R = 172.887363984419);
          constant IdealGases.Common.DataRecord SiHF3(name = "SiHF3", MM = 0.0860886496, Hf =  -13948505.4717364, H0 = 157333.563285444, Tlimit = 1000, alow = {83258.7939, -817.859446,4.04171724,0.02726104628, -3.78783148e-05,2.630589305e-08, -7.2895825e-12}, blow = { -141614.6961,1.382806532}, ahigh = {78201.008, -2592.742261,14.75104214, -0.00065310472,1.372428937e-07, -1.516371051e-11,6.83249715e-16}, bhigh = { -133661.1887, -58.5145095}, R = 96.5803510524574);
          constant IdealGases.Common.DataRecord SiHI3(name = "SiHI3", MM = 0.40980685, Hf =  -181731.95494414, H0 = 46693.7631716015, Tlimit = 1000, alow = {111384.0054, -1974.457175,17.91396249, -0.01219208519,1.613169233e-05, -1.038096942e-08,2.613287761e-12}, blow = { -2256.377425, -59.9545513}, ahigh = {227942.4252, -1898.603142,14.28134804, -0.000477602695,1.003097496e-07, -1.107859145e-11,4.9903382e-16}, bhigh = { -1614.547674, -41.2294307}, R = 20.2887579844017);
          constant IdealGases.Common.DataRecord SiH2(name = "SiH2", MM = 0.03010138, Hf = 9080398.63953081, H0 = 332812.05047742, Tlimit = 1000, alow = { -20638.63564,330.58622,2.099271145,0.00354253937,3.37887667e-06, -5.38384562e-09,2.081191273e-12}, blow = {30117.84298,12.8233357}, ahigh = {4624039.37, -11434.3611,12.6488087,0.00091148995, -8.76661154e-07,1.646297357e-10, -9.96509037e-15}, bhigh = {107247.5101, -66.0607807}, R = 276.215641940668);
          constant IdealGases.Common.DataRecord SiH2Br2(name = "SiH2Br2", MM = 0.18990938, Hf =  -1002436.00395094, H0 = 75152.8018257971, Tlimit = 1000, alow = {165152.8003, -2468.145554,16.08639427, -0.0094138409,1.407423872e-05, -9.24075145e-09,2.255641666e-12}, blow = { -12764.82614, -59.4537432}, ahigh = {566585.144, -3973.91962,15.67387912, -0.000994380229,2.084856022e-07, -2.299521351e-11,1.034736283e-15}, bhigh = { -2990.69018, -61.9047652}, R = 43.7812603042567);
          constant IdealGases.Common.DataRecord SiH2CL2(name = "SiH2CL2", MM = 0.10100738, Hf =  -3172976.07362947, H0 = 132514.554877079, Tlimit = 1000, alow = {189428.1754, -2650.960252,15.49874174, -0.00653313075,9.30609826e-06, -5.63969867e-09,1.213664264e-12}, blow = { -27209.12007, -60.06331649}, ahigh = {540020.458, -4062.33729,15.73118788, -0.001015038389,2.127060243e-07, -2.345079226e-11,1.054871984e-15}, bhigh = { -18230.98225, -65.44540499}, R = 82.3154902146754);
          constant IdealGases.Common.DataRecord SiH2F2(name = "SiH2F2", MM = 0.0680981864, Hf =  -11612291.630721, H0 = 176042.955528695, Tlimit = 1000, alow = {127218.8427, -1241.72431,4.89943589,0.02123882008, -2.7257087e-05,1.860857446e-08, -5.22660433e-12}, blow = { -89804.6692, -5.1118155}, ahigh = {472653.408, -4388.51769,15.96032597, -0.001102993658,2.315838331e-07, -2.556974559e-11,1.151499711e-15}, bhigh = { -73161.8446, -70.9898927}, R = 122.095351426275);
          constant IdealGases.Common.DataRecord SiH2I2(name = "SiH2I2", MM = 0.28391032, Hf =  -134105.727470562, H0 = 52542.175994166, Tlimit = 1000, alow = {153996.3319, -2414.436989,16.84409354, -0.0121203758,1.808453226e-05, -1.207740133e-08,3.040799276e-12}, blow = {5073.07325, -60.9872082}, ahigh = {594239.083, -3924.81448,15.63850289, -0.000980570634,2.054871384e-07, -2.265587199e-11,1.019168663e-15}, bhigh = {15136.27352, -59.4081252}, R = 29.2855574957613);
          constant IdealGases.Common.DataRecord SiH3(name = "SiH3", MM = 0.03110932, Hf = 6569007.80859241, H0 = 330350.808053664, Tlimit = 1000, alow = {4341.14282,227.7185085,0.650825035,0.01221438558, -4.34760427e-06, -1.774916828e-09,1.184191367e-12}, blow = {22599.93826,19.68347482}, ahigh = {605632.122, -4721.25406,13.29129523, -0.001256824868,2.68828594e-07, -3.010741582e-11,1.370945857e-15}, bhigh = {49744.2064, -61.405031}, R = 267.266272615409);
          constant IdealGases.Common.DataRecord SiH3Br(name = "SiH3Br", MM = 0.11101332, Hf =  -704789.299157975, H0 = 106017.710307196, Tlimit = 1000, alow = {152106.5525, -1987.744479,11.31460743, -0.001806099407,7.76926247e-06, -6.23347817e-09,1.585331846e-12}, blow = { -924.886888, -38.4646369}, ahigh = {927928.578, -5917.93579,16.99209298, -0.001487507364,3.123451161e-07, -3.44907147e-11,1.553431675e-15}, bhigh = {22415.60128, -79.4861159}, R = 74.8961656132796);
          constant IdealGases.Common.DataRecord SiH3CL(name = "SiH3CL", MM = 0.06656232, Hf =  -2130905.29296455, H0 = 171879.615974924, Tlimit = 1000, alow = {163338.8301, -2030.302118,10.68016138,0.000494237014,4.30517476e-06, -3.74832727e-09,8.89350421e-13}, blow = { -8180.71498, -36.88956824}, ahigh = {908778.094, -5958.94956,17.02076016, -0.001498450667,3.146813781e-07, -3.47516846e-11,1.565281681e-15}, bhigh = {14937.93206, -81.27277864}, R = 124.912593190862);
          constant IdealGases.Common.DataRecord SiH3F(name = "SiH3F", MM = 0.0501077232, Hf =  -7515009.18325501, H0 = 218162.416926579, Tlimit = 1000, alow = {126783.629, -1177.593516,4.49918896,0.01658543011, -1.678598921e-05,1.019153931e-08, -2.803920232e-12}, blow = { -40103.9297, -4.48108092}, ahigh = {862431.317, -6103.37057,17.12733597, -0.001540724131,3.23962872e-07, -3.58105689e-11,1.61415138e-15}, bhigh = { -12614.87616, -83.9922721}, R = 165.931945596762);
          constant IdealGases.Common.DataRecord SiH3I(name = "SiH3I", MM = 0.15801379, Hf =  -13239.3508186849, H0 = 76437.8159653028, Tlimit = 1000, alow = {145938.4799, -1982.982746,11.96516737, -0.00393750595,1.087991733e-05, -8.43313592e-09,2.196078188e-12}, blow = {8063.20831, -40.6455332}, ahigh = {936114.918, -5859.72029,16.95376698, -0.00147349713,3.094475885e-07, -3.41746247e-11,1.539334807e-15}, bhigh = {31274.99231, -78.0170008}, R = 52.6186480306561);
          constant IdealGases.Common.DataRecord SiH4(name = "SiH4", MM = 0.03211726, Hf = 1080415.95080029, H0 = 328016.804671382, Tlimit = 1000, alow = {78729.9329, -552.608705,2.498944303,0.01442118274, -8.46710731e-06,2.726164641e-09, -5.43675437e-13}, blow = {6269.66906,4.96546183}, ahigh = {1290378.74, -7813.39978,18.28851664, -0.001975620946,4.15650215e-07, -4.59674561e-11,2.072777131e-15}, bhigh = {47668.8795, -98.0169746}, R = 258.878621650788);
          constant IdealGases.Common.DataRecord SiI(name = "SiI", MM = 0.15498997, Hf = 1696583.46278795, H0 = 64140.4085696642, Tlimit = 1000, alow = {94920.8134, -1573.960062,12.37908109, -0.0159386482,1.692049005e-05, -9.21555832e-09,2.046186848e-12}, blow = {37797.3567, -40.6640785}, ahigh = {852940.279, -2442.039471,7.72297752, -0.001985601243,6.54286663e-07, -9.49415829e-11,4.89004137e-15}, bhigh = {45781.6094, -17.33975081}, R = 53.6452262039924);
          constant IdealGases.Common.DataRecord SiI2(name = "SiI2", MM = 0.28189444, Hf = 328016.402168131, H0 = 48927.9426724415, Tlimit = 1000, alow = {11633.52403, -451.448217,8.70797813, -0.00358066493,4.25371443e-06, -2.673478266e-09,6.89155898e-13}, blow = {11262.47142, -11.55818335}, ahigh = {10021.68907,14.33017148,6.70133055,0.00039607699, -2.073866165e-07,4.5534571e-11, -3.01679828e-15}, bhigh = {9007.3681,0.2887336913}, R = 29.4949840089077);
          constant IdealGases.Common.DataRecord SiN(name = "SiN", MM = 0.0420922, Hf = 9590100.7074945, H0 = 207547.645406988, Tlimit = 1000, alow = { -14646.72152,137.4993497,3.67850858, -0.0061584992,2.309417067e-05, -2.294461481e-08,7.39566568e-12}, blow = {46732.1299,6.494564115}, ahigh = { -2932685.132,5853.68859,1.321451677,0.001258329284, -3.77388636e-07,6.88776104e-11, -4.18984259e-15}, bhigh = {6527.14881,25.53145732}, R = 197.529993680539);
          constant IdealGases.Common.DataRecord SiO(name = "SiO", MM = 0.0440849, Hf =  -2242092.37176448, H0 = 197689.118042686, Tlimit = 1000, alow = { -47227.7105,806.313764, -1.636976133,0.01454275546, -1.723202046e-05,1.04239734e-08, -2.559365273e-12}, blow = { -16665.85903,33.557957}, ahigh = { -176513.4162, -31.9917709,4.47744193,4.59176471e-06,3.55814315e-08, -1.327012559e-11,1.613253297e-15}, bhigh = { -13508.4236, -0.838695733}, R = 188.601357834542);
          constant IdealGases.Common.DataRecord SiO2(name = "SiO2", MM = 0.0600843, Hf =  -5360359.97756486, H0 = 175462.192286504, Tlimit = 1000, alow = { -33629.4878,473.407892,0.2309770671,0.01850230806, -2.242786671e-05,1.364981554e-08, -3.35193503e-12}, blow = { -42264.8749,22.95803206}, ahigh = { -146403.1193, -626.144106,7.96456371, -0.0001854119096,4.09521467e-08, -4.69720676e-12,2.17805428e-16}, bhigh = { -37918.3477, -20.45285414}, R = 138.380109279795);
          constant IdealGases.Common.DataRecord SiS(name = "SiS", MM = 0.0601505, Hf = 1798728.30649787, H0 = 148495.972602057, Tlimit = 1000, alow = {35994.4929, -423.97233,4.65401442,0.001588470782, -3.31025436e-06,2.706096479e-09, -8.11351782e-13}, blow = {14115.15571, -1.183201858}, ahigh = { -2102323.897,6228.83618, -3.004120882,0.00449549993, -1.368821364e-06,1.998097253e-10, -9.8820358e-15}, bhigh = { -27955.38166,54.05828786}, R = 138.227811905138);
          constant IdealGases.Common.DataRecord SiS2(name = "SiS2", MM = 0.0922155, Hf = 76155.4944667653, H0 = 132333.685768661, Tlimit = 1000, alow = {43977.4566, -743.001312,7.94158836,0.00203808373, -4.6463376e-06,3.84801436e-09, -1.157260149e-12}, blow = {2801.077853, -17.36888325}, ahigh = { -126648.9071, -97.7423522,7.57280368, -2.905413928e-05,6.40558784e-09, -7.32927334e-13,3.38981978e-17}, bhigh = { -1244.672263, -13.52710463}, R = 90.1634974597546);
          constant IdealGases.Common.DataRecord Si2(name = "Si2", MM = 0.056171, Hf = 10329093.7316409, H0 = 183299.63860355, Tlimit = 1000, alow = {12375.96221, -102.4904376,4.35484852,0.001281063335, -2.531991623e-06,2.265694244e-09, -7.00129014e-13}, blow = {69069.4285,3.2511252}, ahigh = {1370060.657, -4207.06004,9.33743289, -0.002749217168,9.58634596e-07, -1.372449748e-10,6.7650281e-15}, bhigh = {95108.8454, -31.6838519}, R = 148.020722436845);
          constant IdealGases.Common.DataRecord Si2C(name = "Si2C", MM = 0.0681817, Hf = 8126724.67832277, H0 = 169874.702449484, Tlimit = 1000, alow = { -4553.3662,131.4796415,2.469106923,0.0127652068, -1.656910776e-05,1.065289663e-08, -2.739192976e-12}, blow = {64700.4992,14.6883898}, ahigh = { -125382.9442, -341.427779,7.25436533, -0.0001017635503,2.250902158e-08, -2.584074852e-12,1.198884876e-16}, bhigh = {66080.0938, -11.46216579}, R = 121.945800706055);
          constant IdealGases.Common.DataRecord Si2F6(name = "Si2F6", MM = 0.1701614192, Hf =  -14006054.3171586, H0 = 156876.289146512, Tlimit = 1000, alow = {15807.6745, -297.5292881,7.5125915,0.0455279623, -6.42187684e-05,4.35740873e-08, -1.166762282e-11}, blow = { -288670.7444, -7.72178477}, ahigh = { -918630.314,157.3948683,21.41256129, -0.0002681898667,7.21228011e-08, -9.18425266e-12,4.53723836e-16}, bhigh = { -296617.6753, -78.4902247}, R = 48.8622628977227);
          constant IdealGases.Common.DataRecord Si2N(name = "Si2N", MM = 0.0701777, Hf = 5663907.48058144, H0 = 168460.75035232, Tlimit = 1000, alow = {23182.71938, -367.484379,5.4096586,0.0081763894, -1.220312934e-05,8.5806091e-09, -2.354927059e-12}, blow = {48092.7803, -3.042205654}, ahigh = { -280502.6986,250.2366876,7.0924561,0.0002853120795, -9.79210181e-08,1.552154244e-11, -8.03626045e-16}, bhigh = {43420.1479, -10.0957091}, R = 118.477408065525);
          constant IdealGases.Common.DataRecord Si3(name = "Si3", MM = 0.0842565, Hf = 7451856.90124798, H0 = 148591.420246509, Tlimit = 1000, alow = { -11142.08177,157.5785843,2.486135003,0.01631637255, -2.208240021e-05,1.372008287e-08, -3.2623307e-12}, blow = {73282.5385,15.88081347}, ahigh = { -1699395.561,4697.81538,2.618198124,0.001959082075, -2.581160603e-07,6.10344486e-12,6.08630924e-16}, bhigh = {42779.1681,25.86540384}, R = 98.6804816245631);
          constant IdealGases.Common.DataRecord Sn(name = "Sn", MM = 0.11871, Hf = 2537275.71392469, H0 = 52352.0596411423, Tlimit = 1000, alow = { -124869.2263,1618.84119, -4.60239735,0.01045433308,2.99826555e-06, -1.068699386e-08,4.32342131e-12}, blow = {27483.64008,48.0506723}, ahigh = { -5145695.64,11405.75108, -4.17963206,0.002236390679, -3.60321977e-07,2.440237836e-11, -2.937628285e-16}, bhigh = { -42150.1357,59.8145093}, R = 70.0401988038076);
          constant IdealGases.Common.DataRecord Snplus(name = "Snplus", MM = 0.1187094514, Hf = 8558290.14470536, H0 = 52206.6939650603, Tlimit = 1000, alow = { -5571.29778,122.2323189,1.566361415,0.00339706141, -6.31292229e-06,5.58545208e-09, -1.68902134e-12}, blow = {120902.412,11.62634765}, ahigh = {4622916.85, -11859.58712,12.37026473, -0.002773624217,3.09851349e-07, -5.36295144e-12, -8.663474691e-16}, bhigh = {199432.2977, -68.3710828}, R = 70.0405224853057);
          constant IdealGases.Common.DataRecord Snminus(name = "Snminus", MM = 0.1187105486, Hf = 1512047.16949644, H0 = 54674.5767460803, Tlimit = 1000, alow = {272279.6, -3369.9358,17.43906405, -0.02810527618,2.790684767e-05, -1.442400675e-08,3.070159757e-12}, blow = {37532.2775, -80.0785697}, ahigh = { -64477.1322,743.05551,1.921380256,0.0002361623997, -5.28066836e-08,6.10233938e-12, -2.843422486e-16}, bhigh = {16457.39414,12.65436865}, R = 70.0398751253012);
          constant IdealGases.Common.DataRecord SnBr(name = "SnBr", MM = 0.198614, Hf = 380859.45603029, H0 = 50279.0337035657, Tlimit = 1000, alow = {25641.93542, -525.912233,7.68741069, -0.00965420542,1.482863248e-05, -9.94223698e-09,2.475051091e-12}, blow = {10204.83889, -11.26550396}, ahigh = {1815578.541, -6728.26458,13.32464516, -0.0046081023,1.236439537e-06, -1.528863587e-10,6.9715135e-15}, bhigh = {49255.8309, -56.4096278}, R = 41.8624668955864);
          constant IdealGases.Common.DataRecord SnBr2(name = "SnBr2", MM = 0.278518, Hf =  -427170.208029643, H0 = 52403.8230922238, Tlimit = 1000, alow = { -4995.75306, -136.6498896,7.54356385, -0.001181772507,1.442445852e-06, -9.25421726e-10,2.423731634e-13}, blow = { -15755.06969, -3.59558402}, ahigh = { -19408.94091, -2.335607932,7.00196926, -8.62224187e-07,2.039869794e-10, -2.464780359e-14,1.189850894e-18}, bhigh = { -16448.28122, -0.434935475}, R = 29.8525481297439);
          constant IdealGases.Common.DataRecord SnBr3(name = "SnBr3", MM = 0.358422, Hf =  -442820.044528517, H0 = 53792.6075966319, Tlimit = 1000, alow = { -8050.41647, -289.7962523,11.14443355, -0.002475383807,3.01005135e-06, -1.92569532e-09,5.03260509e-13}, blow = { -20700.66649, -18.95965383}, ahigh = { -38930.0453, -4.99084417,10.00418437, -1.82499691e-06,4.30564666e-10, -5.19188793e-14,2.502455595e-18}, bhigh = { -22172.99473, -12.29854098}, R = 23.1974376572867);
          constant IdealGases.Common.DataRecord SnBr4(name = "SnBr4", MM = 0.438326, Hf =  -739670.888790535, H0 = 57133.3162988278, Tlimit = 1000, alow = { -5832.84092, -427.019847,14.67466525, -0.00360426323,4.36668629e-06, -2.78587513e-09,7.26508787e-13}, blow = { -40829.1645, -34.4710436}, ahigh = { -51768.1292, -7.42612695,13.00619502, -2.692587978e-06,6.3367598e-10, -7.62708719e-14,3.671112e-18}, bhigh = { -43001.6634, -24.71455118}, R = 18.9686945332926);
          constant IdealGases.Common.DataRecord SnCL(name = "SnCL", MM = 0.154163, Hf = 224819.288674974, H0 = 62557.9159720555, Tlimit = 1000, alow = {33515.4973, -683.220862,8.39391773, -0.01164434373,1.812159499e-05, -1.26199917e-08,3.29521347e-12}, blow = {6051.79089, -16.67834578}, ahigh = {774160.568, -3602.04848,9.74905982, -0.002596198909,6.36771216e-07, -6.75429557e-11,2.484167602e-15}, bhigh = {24424.47662, -32.25454098}, R = 53.9329930009146);
          constant IdealGases.Common.DataRecord SnCL2(name = "SnCL2", MM = 0.189616, Hf =  -1068729.86984221, H0 = 72154.8656231542, Tlimit = 1000, alow = {9524.6659, -432.984564,8.64472028, -0.00345840961,4.11785203e-06, -2.592639269e-09,6.6923636e-13}, blow = { -24329.21127, -12.99143851}, ahigh = { -39035.8849, -7.94796268,7.00648762, -2.776594714e-06,6.46134142e-10, -7.1190674e-15,3.68813118e-18}, bhigh = { -26545.29653, -3.368081174}, R = 43.8490000843811);
          constant IdealGases.Common.DataRecord SnCL3(name = "SnCL3", MM = 0.225069, Hf =  -1299034.76267278, H0 = 77994.0595995006, Tlimit = 1000, alow = {17600.76813, -814.280548,13.06326984, -0.00639559619,7.57462457e-06, -4.74971146e-09,1.222182656e-12}, blow = { -34134.3096, -34.5767967}, ahigh = { -74829.664, -15.28523914,10.01240305, -5.28558124e-06,1.226095111e-09, -1.459895108e-13,6.96887978e-18}, bhigh = { -38308.7827, -16.63063137}, R = 36.9418800456749);
          constant IdealGases.Common.DataRecord SnCL4(name = "SnCL4", MM = 0.260522, Hf =  -1836566.46655561, H0 = 86265.3864165023, Tlimit = 1000, alow = {42459.6787, -1280.574318,17.68445328, -0.00957477952,1.115620323e-05, -6.90722927e-09,1.759652856e-12}, blow = { -55040.0904, -58.27352999}, ahigh = { -107833.6862, -25.54305872,13.02040391, -8.5951259e-06,1.976570888e-09, -2.337929994e-13,1.110287624e-17}, bhigh = { -61635.6203, -30.72702622}, R = 31.9146636368522);
          constant IdealGases.Common.DataRecord SnF(name = "SnF", MM = 0.1377084032, Hf =  -689987.595470136, H0 = 66343.881620145, Tlimit = 1000, alow = {60985.0352, -986.030629,8.99473121, -0.01223012156,1.840583578e-05, -1.267926607e-08,3.29133102e-12}, blow = { -7882.61448, -22.21921734}, ahigh = {25897.15476, -1484.478214,7.37851893, -0.00132424444,2.847634043e-07, -2.360371345e-11,6.11678371e-16}, bhigh = { -4776.68897, -16.81202516}, R = 60.3773757213968);
          constant IdealGases.Common.DataRecord SnF2(name = "SnF2", MM = 0.1567068064, Hf =  -3260591.94707704, H0 = 78127.1042481024, Tlimit = 1000, alow = {70423.9185, -1137.505955,10.08659352, -0.00472285593,4.13570292e-06, -1.921143972e-09,3.64076028e-13}, blow = { -57566.7082, -25.71566624}, ahigh = { -102775.8772, -44.5175162,7.03326876, -1.329882797e-05,2.934271938e-09, -3.35841592e-13,1.553358669e-17}, bhigh = { -63621.2588, -6.81008452}, R = 53.0575039528085);
          constant IdealGases.Common.DataRecord SnF3(name = "SnF3", MM = 0.1757052096, Hf =  -3680200.94835025, H0 = 84230.4051979572, Tlimit = 1000, alow = {109555.5529, -1950.557979,15.41407832, -0.00854651877,7.80055182e-06, -3.82594222e-09,7.78774776e-13}, blow = { -70567.9492, -54.9463948}, ahigh = { -182931.9689, -75.5735868,10.05671455, -2.275132668e-05,5.03469582e-09, -5.77653096e-13,2.67721417e-17}, bhigh = { -80918.9759, -21.91673972}, R = 47.3205775681224);
          constant IdealGases.Common.DataRecord SnF4(name = "SnF4", MM = 0.1947036128, Hf =  -5263213.67263299, H0 = 96567.2014484572, Tlimit = 1000, alow = {139044.1119, -2168.784371,16.91802917, -0.002318608529, -2.045847976e-06,3.4300879e-09, -1.28922886e-12}, blow = { -115356.3396, -63.7060475}, ahigh = { -263569.2764, -144.5469969,13.10690681, -4.23775579e-05,9.28687195e-09, -1.057062694e-12,4.86710868e-17}, bhigh = { -127153.9967, -38.039219}, R = 42.7032240461847);
          constant IdealGases.Common.DataRecord SnI(name = "SnI", MM = 0.24561447, Hf = 703236.344340787, H0 = 41435.3519155447, Tlimit = 1000, alow = {26702.12235, -500.602635,7.49067315, -0.00874634655,1.289688397e-05, -8.21134493e-09,1.936120475e-12}, blow = {21772.49172, -9.35480685}, ahigh = { -55549.9185, -1530.457453,7.93260139, -0.002108885464,7.8098036e-07, -1.348104887e-10,8.2071847e-15}, bhigh = {27448.64953, -16.26822892}, R = 33.8517189154206);
          constant IdealGases.Common.DataRecord SnI2(name = "SnI2", MM = 0.37251894, Hf =  -21654.5687583026, H0 = 39953.0370187352, Tlimit = 1000, alow = { -5739.25489, -94.3502445,7.37795666, -0.000825834894,1.01168704e-06, -6.5084516e-10,1.7081866e-13}, blow = { -2622.6445, -0.960581496}, ahigh = { -15594.5339, -1.594789414,7.00135116, -5.93532424e-07,1.407432564e-10, -1.703457321e-14,8.23361171e-19}, bhigh = { -3100.608319,1.235038509}, R = 22.3195953472862);
          constant IdealGases.Common.DataRecord SnI3(name = "SnI3", MM = 0.49942341, Hf =  -16053.592682009, H0 = 40665.6147736447, Tlimit = 1000, alow = { -10699.78099, -138.769354,10.55637633, -0.001216435658,1.490873499e-06, -9.59449963e-10,2.518810438e-13}, blow = { -3314.22988, -12.49558209}, ahigh = { -25175.3487, -2.34828289,10.0019913, -8.75276484e-07,2.07646498e-10, -2.514060336e-14,1.215482266e-18}, bhigh = { -4017.05561, -9.26390846}, R = 16.648142304743);
          constant IdealGases.Common.DataRecord SnI4(name = "SnI4", MM = 0.62632788, Hf =  -189763.296182824, H0 = 42702.8044799794, Tlimit = 1000, alow = { -12831.46793, -145.9885093,13.58634885, -0.001283554646,1.574557147e-06, -1.013990168e-09,2.66337043e-13}, blow = { -17511.78045, -23.67973168}, ahigh = { -28018.40706, -2.474699366,13.00210215, -9.25096234e-07,2.196515817e-10, -2.661071614e-14,1.287167546e-18}, bhigh = { -18250.83832, -20.27484281}, R = 13.274951132624);
          constant IdealGases.Common.DataRecord SnO(name = "SnO", MM = 0.1347094, Hf = 162654.231998658, H0 = 65875.9299647983, Tlimit = 1000, alow = {25477.88022, -229.3583066,3.42159353,0.00471330004, -7.36597132e-06,5.37973887e-09, -1.519277855e-12}, blow = {2853.047679,6.67396527}, ahigh = { -2555955.548,7870.0535, -5.43976873,0.00630781918, -2.06887137e-06,3.30707488e-10, -1.848169716e-14}, bhigh = { -48416.6391,71.7595246}, R = 61.7215428173535);
          constant IdealGases.Common.DataRecord SnO2(name = "SnO2", MM = 0.1507088, Hf = 77501.6455575255, H0 = 78268.0241631544, Tlimit = 1000, alow = {47076.7408, -611.456867,6.11743391,0.00751401466, -1.238371787e-05,9.22091599e-09, -2.632417597e-12}, blow = {2981.068066, -8.14492641}, ahigh = { -157355.8926, -142.7316788,7.60570076, -4.19800853e-05,9.21874243e-09, -1.051372983e-12,4.84957927e-17}, bhigh = { -523.453187, -14.28360169}, R = 55.1691208476214);
          constant IdealGases.Common.DataRecord SnS(name = "SnS", MM = 0.150775, Hf = 736853.006134969, H0 = 61688.6884430443, Tlimit = 1000, alow = {27248.77813, -492.412638,6.22165473, -0.00340940696,3.98491368e-06, -2.470702874e-09,6.33934146e-13}, blow = {14524.95307, -6.791285229}, ahigh = { -1797985.129,6141.73632, -4.01950488,0.00599382918, -2.188735204e-06,3.899542e-10, -2.40656363e-14}, bhigh = { -26113.49815,62.56820705}, R = 55.1448980268612);
          constant IdealGases.Common.DataRecord SnS2(name = "SnS2", MM = 0.18284, Hf = 818451.07744476, H0 = 74573.9170859768, Tlimit = 1000, alow = {38771.4541, -848.695394,10.4208399, -0.00568690267,6.37382977e-06, -3.8251581e-09,9.50248096e-13}, blow = {20060.27952, -27.23724769}, ahigh = { -67640.8405, -19.70189679,7.51533387, -6.33255918e-06,1.434207625e-09, -1.676383081e-13,7.8869119e-18}, bhigh = {15650.53629, -9.922744838}, R = 45.4740319404944);
          constant IdealGases.Common.DataRecord Sn2(name = "Sn2", MM = 0.23742, Hf = 1774676.67846011, H0 = 47903.0283885098, Tlimit = 1000, alow = { -132966.6654,1639.179787, -1.902868972,0.01290431677, -1.120542242e-05,4.50299386e-09, -6.05623777e-13}, blow = {40974.5271,44.3540803}, ahigh = { -4275783.04,12364.21656, -8.43833617,0.00708805629, -1.712012737e-06,1.805262952e-10, -7.05865681e-15}, bhigh = { -30014.1872,100.5138538}, R = 35.0200994019038);
          constant IdealGases.Common.DataRecord Sr(name = "Sr", MM = 0.08762, Hf = 1831773.56767861, H0 = 70730.7464049304, Tlimit = 1000, alow = {4.19064984, -0.0630443758,2.500373027, -1.115455943e-06,1.785248643e-09, -1.456209589e-12,4.750132981e-16}, blow = {18558.52648,5.55577284}, ahigh = {14894144.1, -43753.3505,51.3726628, -0.02592566025,6.58299e-06, -6.9496118e-10,2.417779662e-14}, bhigh = {297754.5522, -345.489077}, R = 94.8923989956631);
          constant IdealGases.Common.DataRecord Srplus(name = "Srplus", MM = 0.0876194514, Hf = 8173599.77216201, H0 = 70731.1892619314, Tlimit = 1000, alow = {11.27287678, -0.134695187,2.500651495, -1.635163061e-06,2.249493149e-09, -1.610827539e-12,4.698612333e-16}, blow = {85389.8118,6.24725924}, ahigh = {3145095.058, -9514.75689,13.50086948, -0.00605971712,1.594068746e-06, -1.718800946e-10,6.322256169e-15}, bhigh = {145799.1907, -72.3641693}, R = 94.8929931328011);
          constant IdealGases.Common.DataRecord SrBr(name = "SrBr", MM = 0.167524, Hf =  -381543.408705618, H0 = 60332.3822258303, Tlimit = 1000, alow = { -754.981242, -73.5953201,4.82803581, -0.000716058465,1.028527433e-06, -6.88837072e-10,1.880257878e-13}, blow = { -8686.16845,4.15115309}, ahigh = {3009976.114, -9193.19224,15.26033745, -0.00600925395,1.692611858e-06, -2.039438429e-10,8.80777584e-15}, bhigh = {49228.0613, -70.6825383}, R = 49.6315274229364);
          constant IdealGases.Common.DataRecord SrBr2(name = "SrBr2", MM = 0.247428, Hf =  -1643814.97243643, H0 = 66017.7869925797, Tlimit = 1000, alow = { -4632.45866, -72.4493864,7.80021716, -0.000677212531,8.53778024e-07, -5.63235174e-10,1.510568795e-13}, blow = { -50822.3833, -5.28507476}, ahigh = { -11861.73059, -1.404383013,7.50121557, -5.41598319e-07,1.296969275e-10, -1.580911652e-14,7.68158227e-19}, bhigh = { -51185.7521, -3.55203711}, R = 33.6036018558934);
          constant IdealGases.Common.DataRecord SrCL(name = "SrCL", MM = 0.123073, Hf =  -1038959.6580891, H0 = 79628.496908339, Tlimit = 1000, alow = {3555.34632, -172.9385697,5.21345102, -0.001542910668,2.025835568e-06, -1.327076633e-09,3.54717596e-13}, blow = { -15882.88066,0.4818144784}, ahigh = {2070217.324, -6199.89173,11.50315072, -0.00367568546,9.46173235e-07, -9.20376575e-11,2.707242192e-15}, bhigh = {22714.71632, -45.61058514}, R = 67.5572383869736);
          constant IdealGases.Common.DataRecord SrCLplus(name = "SrCLplus", MM = 0.1230724514, Hf = 3316031.79556071, H0 = 77862.7701893651, Tlimit = 1000, alow = {1689.043529, -174.9284676,5.06532864, -0.000985732784,1.071655138e-06, -5.98812713e-10,1.395650932e-13}, blow = {48611.9026,0.2492001711}, ahigh = { -21324.75783, -4.05794827,4.50314939,3.82620238e-05,2.67695151e-09,5.6676105e-14,1.613288103e-18}, bhigh = {47693.2603,3.637722496}, R = 67.5575395258601);
          constant IdealGases.Common.DataRecord SrCL2(name = "SrCL2", MM = 0.158526, Hf =  -3058261.11805004, H0 = 90938.3823473752, Tlimit = 1000, alow = {1913.170011, -268.1156951,8.0813343, -0.002393329295,2.976397494e-06, -1.943902862e-09,5.17430771e-13}, blow = { -59101.1185, -8.457827543}, ahigh = { -25927.16099, -5.26378585,7.00447524, -1.970397814e-06,4.6796712e-10, -5.67021588e-14,2.742892387e-18}, bhigh = { -60453.9953, -2.192202775}, R = 52.4486330318055);
          constant IdealGases.Common.DataRecord SrF(name = "SrF", MM = 0.1066184032, Hf =  -2847095.61285195, H0 = 87068.5709162825, Tlimit = 1000, alow = {27250.08615, -485.001972,6.1126972, -0.003008469209,3.3684658e-06, -1.987813072e-09,4.88416109e-13}, blow = { -35368.9039, -6.68373918}, ahigh = {870234.049, -2589.81877,7.20026422, -0.001173446296,1.996165066e-07,1.103842667e-11, -2.343183859e-15}, bhigh = { -21336.71699, -16.67390309}, R = 77.9834601762259);
          constant IdealGases.Common.DataRecord SrFplus(name = "SrFplus", MM = 0.1066178546, Hf = 1964664.48125284, H0 = 85415.4497308746, Tlimit = 1000, alow = {40096.242, -594.494608,6.18233033, -0.00271615975,2.641369127e-06, -1.383123292e-09,3.052935853e-13}, blow = {26971.49351, -8.31365945}, ahigh = { -47520.8206, -25.26802345,4.51929528,2.55642675e-05,3.38254183e-09, -1.511394636e-13,9.53749118e-18}, bhigh = {23838.61417,1.892197325}, R = 77.9838614385325);
          constant IdealGases.Common.DataRecord SrF2(name = "SrF2", MM = 0.1256168064, Hf =  -6247522.26625625, H0 = 105635.554511279, Tlimit = 1000, alow = {41707.5965, -850.013241,10.02806064, -0.00609848519,7.05626133e-06, -4.35937431e-09,1.11130831e-12}, blow = { -92178.5985, -23.25100053}, ahigh = { -61262.6543, -19.64060822,7.01555775, -6.51252678e-06,1.490473613e-09, -1.75643704e-13,8.31709385e-18}, bhigh = { -96567.0125, -5.40230876}, R = 66.1891687767028);
          constant IdealGases.Common.DataRecord SrH(name = "SrH", MM = 0.08862794, Hf = 2473566.51863961, H0 = 98423.8717496988, Tlimit = 1000, alow = { -43177.9689,767.64297, -1.594398638,0.01500711077, -1.826296898e-05,1.136757659e-08, -2.855588359e-12}, blow = {21796.85693,33.2467423}, ahigh = { -226982.5428,1345.209458,1.152194421,0.003109555289, -1.188080968e-06,2.073091147e-10, -1.300839287e-14}, bhigh = {17355.47062,21.12916944}, R = 93.8132151102689);
          constant IdealGases.Common.DataRecord SrI(name = "SrI", MM = 0.21452447, Hf =  -36600.374773097, H0 = 47934.5036955458, Tlimit = 1000, alow = { -2467.498501, -25.35495809,4.60329848, -0.0001613676212,2.694294049e-07, -1.564795762e-10,3.82648228e-14}, blow = { -2175.538291,6.41148188}, ahigh = {2201136.857, -7082.2307,13.32597928, -0.00530586442,1.63607115e-06, -2.205540591e-10,1.078496824e-14}, bhigh = {42190.7717, -55.308332}, R = 38.7576857782238);
          constant IdealGases.Common.DataRecord SrI2(name = "SrI2", MM = 0.34142894, Hf =  -814866.021609065, H0 = 48916.9371524277, Tlimit = 1000, alow = { -4391.84969, -37.5726735,7.6569935, -0.00035617213,4.50868901e-07, -2.983230473e-10,8.01865866e-14}, blow = { -35533.0346, -2.598281578}, ahigh = { -8090.38192, -0.734826323,7.50064006, -2.863515042e-07,6.87655697e-11, -8.39880736e-15,4.08696816e-19}, bhigh = { -35721.0642, -1.693119087}, R = 24.3519837539255);
          constant IdealGases.Common.DataRecord SrO(name = "SrO", MM = 0.1036194, Hf =  -137114.198692523, H0 = 87243.4022972532, Tlimit = 1000, alow = {42248.9167, -591.351787,5.964363, -0.002105712736,2.028373972e-06, -1.197885492e-09,3.46628992e-13}, blow = {101.8053721, -7.51142747}, ahigh = { -51729330.5,151082.1, -161.4513515,0.0851605179, -2.091669402e-05,2.452307103e-09, -1.108841292e-13}, bhigh = { -968782.952,1197.846971}, R = 80.2404955056679);
          constant IdealGases.Common.DataRecord SrOH(name = "SrOH", MM = 0.10462734, Hf =  -1855019.21390719, H0 = 105581.065140335, Tlimit = 1000, alow = {46400.0656, -993.868554,9.78494896, -0.00577437762,5.12616411e-06, -1.833418356e-09,1.92601341e-13}, blow = { -20227.21196, -27.40670981}, ahigh = {2545648.379, -7416.19598,13.81956204, -0.003096481934,7.72389421e-07, -7.92128032e-11,2.740829111e-15}, bhigh = {22313.02058, -61.4878113}, R = 79.4674890903276);
          constant IdealGases.Common.DataRecord SrOHplus(name = "SrOHplus", MM = 0.1046267914, Hf = 2964533.27918837, H0 = 106135.042959943, Tlimit = 1000, alow = {38573.6694, -903.024233,9.45110325, -0.00504251628,4.19010189e-06, -1.196920603e-09,1.606318743e-14}, blow = {39950.6772, -26.0148611}, ahigh = {867969.566, -2340.609538,7.97682975,0.0001023752569, -6.28603233e-08,1.024851502e-11, -5.72368392e-16}, bhigh = {50731.5772, -20.27046917}, R = 79.4679057700703);
          constant IdealGases.Common.DataRecord Sr_OH_2(name = "Sr_OH_2", MM = 0.12163468, Hf =  -4905628.66610082, H0 = 139860.778192535, Tlimit = 1000, alow = {72050.903, -1789.465226,17.29429139, -0.01169280781,1.106702894e-05, -4.49014609e-09,6.60399746e-13}, blow = { -66053.9404, -63.8868122}, ahigh = {1750013.518, -4681.6015,13.95548402,0.000203501956, -1.253496115e-07,2.044549432e-11, -1.142004855e-15}, bhigh = { -44271.7024, -50.5145685}, R = 68.3560971262472);
          constant IdealGases.Common.DataRecord SrS(name = "SrS", MM = 0.119685, Hf = 871882.984500982, H0 = 79676.7765384133, Tlimit = 1000, alow = {12564.40807, -315.0436559,5.72172251, -0.002630479718,3.38469975e-06, -2.296256846e-09,6.51208668e-13}, blow = {12772.9516, -3.693874943}, ahigh = { -13794329.47,39213.3158, -36.0286638,0.01818513386, -3.27585816e-06,2.324312746e-10, -3.6145557e-15}, bhigh = { -241024.6179,299.1679642}, R = 69.4696244307975);
          constant IdealGases.Common.DataRecord Sr2(name = "Sr2", MM = 0.17524, Hf = 1755137.73681808, H0 = 64854.6964163433, Tlimit = 1000, alow = { -110885.9753,592.760401,8.43968855, -0.02652801112,4.43582597e-05, -3.39991549e-08,9.96554668e-12}, blow = {31576.15033, -7.03396665}, ahigh = {209844.5682,103.865013,2.30933233,0.0001330517507, -4.42643339e-08,6.71141915e-12, -3.374128e-16}, bhigh = {36366.4269,21.68088517}, R = 47.4461994978315);
          constant IdealGases.Common.DataRecord Ta(name = "Ta", MM = 0.1809479, Hf = 4324552.19430565, H0 = 34262.0058038806, Tlimit = 1000, alow = { -11509.07339,47.8073043,3.18558839, -0.00536652816,1.288379705e-05, -1.045798666e-08,3.050617695e-12}, blow = {92997.9763,5.33605661}, ahigh = {1689726.898, -5986.85466,9.56503967, -0.002511649459,6.44303117e-07, -7.18923725e-11,3.11335207e-15}, bhigh = {130671.0983, -43.3509627}, R = 45.9495357503458);
          constant IdealGases.Common.DataRecord Taplus(name = "Taplus", MM = 0.1809473514, Hf = 8564255.42020948, H0 = 35025.8511714253, Tlimit = 1000, alow = {286971.2865, -3084.920994,14.30679704, -0.01984772164,1.951445133e-05, -8.97094603e-09,1.501974665e-12}, blow = {201382.8712, -63.0642783}, ahigh = {3656142.13, -12540.73524,18.65022579, -0.00794327466,2.151786937e-06, -2.816764844e-10,1.413722944e-14}, bhigh = {263687.6455, -106.5864286}, R = 45.9496750611184);
          constant IdealGases.Common.DataRecord Taminus(name = "Taminus", MM = 0.1809484486, Hf = 4119789.00492215, H0 = 35458.3255598026, Tlimit = 1000, alow = {187398.2301, -1681.268679,6.48004015, -0.0002254355782,2.028434876e-06, -4.67980143e-09,1.997027996e-12}, blow = {97934.9602, -20.49618155}, ahigh = { -4235467.48,11010.56361, -4.73691107,0.002503562129, -4.82185169e-07,4.88640803e-11, -2.030547835e-15}, bhigh = {15746.99408,64.918027}, R = 45.949396440418);
          constant IdealGases.Common.DataRecord TaCL5(name = "TaCL5", MM = 0.3582129, Hf =  -2135140.86176126, H0 = 75008.7447995312, Tlimit = 1000, alow = {63064.8329, -1771.221442,22.55083356, -0.01360238041,1.613009806e-05, -1.016307761e-08,2.631917453e-12}, blow = { -87927.3711, -80.97864282}, ahigh = { -142644.4545, -38.9434032,16.0314237, -1.333676355e-05,3.084307611e-09, -3.66393006e-13,1.745841795e-17}, bhigh = { -97012.9717, -42.56986372}, R = 23.2109787224301);
          constant IdealGases.Common.DataRecord TaO(name = "TaO", MM = 0.1969473, Hf = 1231470.06838885, H0 = 44509.9018874592, Tlimit = 1000, alow = { -13957.38049,394.523699, -0.0630169641,0.01308595441, -1.86233973e-05,1.366155583e-08, -3.92455123e-12}, blow = {26451.95419,27.41671469}, ahigh = {6106591.12, -19841.35246,28.15797433, -0.01330045882,3.87680749e-06, -5.06887277e-10,2.443666035e-14}, bhigh = {152581.4433, -165.8307106}, R = 42.2167351367599);
          constant IdealGases.Common.DataRecord TaO2(name = "TaO2", MM = 0.2129467, Hf =  -815518.653259243, H0 = 50243.084302316, Tlimit = 1000, alow = {15163.56303,70.5841476,0.691851699,0.02020605733, -2.928317413e-05,2.081417374e-08, -5.75110669e-12}, blow = { -22121.99468,24.846625}, ahigh = {1297565.964, -4149.21743,10.41680373, -0.001045478838,2.754904271e-07, -3.29704313e-11,1.356249074e-15}, bhigh = {3418.98567, -33.9264279}, R = 39.0448501902119);
          constant IdealGases.Common.DataRecord Ti(name = "Ti", MM = 0.047867, Hf = 9881546.78588589, H0 = 157501.848872919, Tlimit = 1000, alow = { -45701.794,660.809202,0.429525749,0.00361502991, -3.54979281e-06,1.759952494e-09, -3.052720871e-13}, blow = {52709.4793,20.26149738}, ahigh = { -170478.6714,1073.852803,1.181955014,0.0002245246352,3.091697848e-07, -5.74002728e-11,2.927371014e-15}, bhigh = {49780.6991,17.40431368}, R = 173.699458917417);
          constant IdealGases.Common.DataRecord Tiplus(name = "Tiplus", MM = 0.0478664514, Hf = 23766625.595312, H0 = 165038.952521975, Tlimit = 1000, alow = {170745.7044, -1727.524602,9.61588533, -0.0108965506,8.20180965e-06, -2.871464413e-09,3.420382976e-13}, blow = {144789.7558, -34.6314366}, ahigh = { -768546.308,2545.8681,0.342386278,0.000709990136,2.706231875e-08, -2.3716601e-11,1.895443077e-15}, bhigh = {119882.1489,24.8479915}, R = 173.701449696353);
          constant IdealGases.Common.DataRecord Timinus(name = "Timinus", MM = 0.0478675486, Hf = 9593217.68986516, H0 = 157990.417750367, Tlimit = 1000, alow = { -3006.48499,204.0911689,1.822638976,0.001245254812, -1.309239865e-06,7.37214322e-10, -1.724319779e-13}, blow = {53467.7205,12.05926588}, ahigh = {23411.1764,2.580413872,2.497754577,9.76907204e-07, -2.280024955e-10,2.717202291e-14, -1.295670294e-18}, bhigh = {54546.6218,7.99982395}, R = 173.697468184113);
          constant IdealGases.Common.DataRecord TiCL(name = "TiCL", MM = 0.08332, Hf = 1810500.79212674, H0 = 116186.101776284, Tlimit = 1000, alow = { -17141.77839,310.3259047,0.892093598,0.01383436793, -1.885370354e-05,1.212085912e-08, -3.032162427e-12}, blow = {15580.66453,22.44778384}, ahigh = { -963322.542,2868.829781,1.973820676,0.001752668011, -4.25543436e-07,5.0491034e-11, -2.304720715e-15}, bhigh = { -1811.011561,23.19450287}, R = 99.7896303408546);
          constant IdealGases.Common.DataRecord TiCL2(name = "TiCL2", MM = 0.118773, Hf =  -1997339.46267249, H0 = 114043.343184057, Tlimit = 1000, alow = {13243.54656, -576.282932,9.64004729, -0.00447512095,5.39879487e-06, -3.52580049e-09,9.74600455e-13}, blow = { -27920.66683, -22.18347625}, ahigh = { -3190012.95,10157.63826, -4.84079477,0.00682200634, -1.70075327e-06,2.118106588e-10, -1.041245457e-14}, bhigh = { -94809.8298,78.23195948}, R = 70.003047830736);
          constant IdealGases.Common.DataRecord TiCL3(name = "TiCL3", MM = 0.154226, Hf =  -3496946.04022668, H0 = 99691.0183756306, Tlimit = 1000, alow = {124328.2268, -2320.728253,17.79895274, -0.01160889348,1.006643154e-05, -4.81194069e-09,1.006186828e-12}, blow = { -56096.0251, -67.32754065}, ahigh = { -94087.9231, -125.5351109,10.66923972, -0.0002637456897,7.95916935e-08, -1.176740054e-11,6.41622348e-16}, bhigh = { -67691.1468, -23.69164216}, R = 53.9109618352288);
          constant IdealGases.Common.DataRecord TiCL4(name = "TiCL4", MM = 0.189679, Hf =  -4023429.05645854, H0 = 113947.991079666, Tlimit = 1000, alow = {81871.968, -1758.32385,18.92512121, -0.01133495876,1.251952037e-05, -7.42268133e-09,1.825440187e-12}, blow = { -86729.2311, -67.69594291}, ahigh = { -143256.2278, -43.1677485,13.03337752, -1.371365738e-05,3.093403483e-09, -3.60424516e-13,1.691384523e-17}, bhigh = { -95889.47, -32.47541011}, R = 43.8344360735769);
          constant IdealGases.Common.DataRecord TiO(name = "TiO", MM = 0.0638664, Hf = 775112.030739168, H0 = 150205.976851678, Tlimit = 1000, alow = { -11681.5246,454.256565, -0.1139144613,0.01275432333, -1.727656935e-05,1.187369403e-08, -3.23657937e-12}, blow = {2924.306353,27.02903947}, ahigh = {2330644.03, -7415.79386,12.81799311, -0.00434455595,1.186303111e-06, -1.367644275e-10,5.70321225e-15}, bhigh = {51448.4136, -57.9399424}, R = 130.185386995353);
          constant IdealGases.Common.DataRecord TiOplus(name = "TiOplus", MM = 0.0638658514, Hf = 10730632.9748545, H0 = 144257.232903655, Tlimit = 1000, alow = {36912.5625, -149.2825538,2.624977257,0.00581862713, -7.52966211e-06,4.74415435e-09, -1.186916989e-12}, blow = {82415.5117,10.95428726}, ahigh = {342132.953, -2161.85106,8.02517566, -0.002708700692,1.004583805e-06, -1.50576685e-10,8.04565811e-15}, bhigh = {93626.4699, -22.61587887}, R = 130.186505272206);
          constant IdealGases.Common.DataRecord TiOCL(name = "TiOCL", MM = 0.0993194, Hf =  -2459358.39322429, H0 = 122259.105471841, Tlimit = 1000, alow = {35458.5651, -629.528714,7.3701763,0.00330201914, -6.12145875e-06,4.73588025e-09, -1.374690357e-12}, blow = { -27970.87903, -12.94470159}, ahigh = { -127305.9873, -111.111885,7.58288808, -3.3128718e-05,7.31401409e-09, -8.37882226e-13,3.87927506e-17}, bhigh = { -31393.56857, -12.47104274}, R = 83.7144807560255);
          constant IdealGases.Common.DataRecord TiOCL2(name = "TiOCL2", MM = 0.1347724, Hf =  -4047950.47057112, H0 = 123986.884554998, Tlimit = 1000, alow = {32890.5246, -738.918859,10.55976857,0.001577742341, -3.9071821e-06,3.29057653e-09, -9.94947175e-13}, blow = { -64484.107, -24.17469934}, ahigh = { -132295.1633, -96.9121205,10.07240769, -2.897248531e-05,6.40177504e-09, -7.3384526e-13,3.39928267e-17}, bhigh = { -68474.0964, -19.75043469}, R = 61.6926907883216);
          constant IdealGases.Common.DataRecord TiO2(name = "TiO2", MM = 0.0798658, Hf =  -3824290.24688916, H0 = 142134.555717216, Tlimit = 1000, alow = { -1710.545601,272.1435528,0.596137896,0.01925463599, -2.665500165e-05,1.811109197e-08, -4.87671047e-12}, blow = { -39122.4177,24.08605889}, ahigh = {154629.9764, -1046.25688,7.78898583, -0.0001546805714, -7.05993595e-08,3.100244802e-11, -2.49472543e-15}, bhigh = { -32663.3675, -15.9153466}, R = 104.105537038382);
          constant IdealGases.Common.DataRecord U(name = "U", MM = 0.23802891, Hf = 2247626.1391946, H0 = 27304.6412723564, Tlimit = 1000, alow = {69657.3775, -1070.351517,8.07584231, -0.01060034069,9.25654801e-06, -3.21989976e-09,4.058048809e-13}, blow = {68665.137, -22.40521678}, ahigh = { -4092498.96,12748.88349, -12.18707506,0.00725810568, -7.78777507e-07, -3.84435385e-11,7.066508567e-15}, bhigh = { -16993.72664,115.5026301}, R = 34.9305132725264);
          constant IdealGases.Common.DataRecord UF(name = "UF", MM = 0.2570273132, Hf =  -191616.744488461, H0 = 36521.6555514303, Tlimit = 1000, alow = {172553.3463, -1698.985561,5.90292643,0.01841977309, -4.63317431e-05,4.3766572e-08, -1.451876566e-11}, blow = {2086.455917, -11.86592944}, ahigh = { -5325439.37,15339.67086, -11.75044398,0.00964721093, -2.49877819e-06,3.155572896e-10, -1.544694839e-14}, bhigh = { -105719.6525,122.0828149}, R = 32.3485932155789);
          constant IdealGases.Common.DataRecord UFplus(name = "UFplus", MM = 0.2570267646, Hf = 2167318.90885732, H0 = 36958.6685448244, Tlimit = 1000, alow = {1622640.597, -19173.26611,87.5946122, -0.1699915707,0.0001861251683, -1.036176702e-07,2.324287412e-11}, blow = {161670.9643, -480.6891249}, ahigh = {539509.184, -2923.962095,10.08511948, -0.002425945123,5.92872548e-07, -6.79584554e-11,3.152666445e-15}, bhigh = {82672.9122, -33.07089704}, R = 32.3486622606773);
          constant IdealGases.Common.DataRecord UFminus(name = "UFminus", MM = 0.2570278618, Hf =  -605689.950925001, H0 = 35622.6244730018, Tlimit = 1000, alow = {3692.32786, -149.1979606,4.03398852,0.002857429445, -5.17170256e-06,3.98537911e-09, -9.47035544e-13}, blow = { -19152.81695,6.059243838}, ahigh = { -4311143.56,19748.17938, -27.60304763,0.02276806261, -6.88874172e-06,9.48351106e-10, -4.9106881e-14}, bhigh = { -138080.129,224.0758537}, R = 32.3485241707753);
          constant IdealGases.Common.DataRecord UF2(name = "UF2", MM = 0.2760257164, Hf =  -1938358.21523476, H0 = 54894.85616638, Tlimit = 1000, alow = { -38824.9202,445.493086,4.71800919,0.00164891148,8.69624301e-06, -1.207439921e-08,4.53679391e-12}, blow = { -68553.2933,11.59725423}, ahigh = { -471677.682,322.423686,8.21409063, -0.000140741378,6.54117805e-09,1.818915497e-12, -2.287963719e-16}, bhigh = { -69952.3505, -9.648963156}, R = 30.1220919138968);
          constant IdealGases.Common.DataRecord UF2plus(name = "UF2plus", MM = 0.2760251678, Hf = 255214.144280651, H0 = 52303.6617097928, Tlimit = 1000, alow = {5439.01016, -150.7096256,6.87348519,0.001530973969, -3.31669431e-06,2.949453335e-09, -8.28170654e-13}, blow = {7256.01491, -2.205491833}, ahigh = { -3779149.61,12987.37504, -10.27930641,0.01048838051, -2.651047812e-06,3.116973593e-10, -1.417216291e-14}, bhigh = { -74334.402,118.9131079}, R = 30.1221517815521);
          constant IdealGases.Common.DataRecord UF2minus(name = "UF2minus", MM = 0.276026265, Hf =  -2457132.82031331, H0 = 47652.4906062834, Tlimit = 1000, alow = {806049.173, -9404.25466,44.4839277, -0.0686855877,6.77780992e-05, -3.33882768e-08,6.60485162e-12}, blow = { -36033.0613, -226.1594249}, ahigh = {10382318.55, -28401.13518,34.9853337, -0.01053216396,1.92462047e-06, -1.69874957e-10,5.70985889e-15}, bhigh = {100157.2794, -209.4039075}, R = 30.1220320464793);
          constant IdealGases.Common.DataRecord UF3(name = "UF3", MM = 0.2950241196, Hf =  -3596175.79213005, H0 = 63191.657093246, Tlimit = 1000, alow = {30710.27207, -634.078947,11.14030799, -0.0002937554125, -1.865390371e-06,2.37181947e-09, -7.44675407e-13}, blow = { -127183.8141, -23.48962529}, ahigh = { -3828876.42,12968.91783, -7.26555429,0.01048289752, -2.649840687e-06,3.115594509e-10, -1.4165794e-14}, bhigh = { -211361.5913,105.4464687}, R = 28.1823466205846);
          constant IdealGases.Common.DataRecord UF3plus(name = "UF3plus", MM = 0.295023571, Hf =  -965161.62771279, H0 = 63245.3533687313, Tlimit = 1000, alow = { -398415.589,3480.89244, -2.519603507,0.02283749282, -2.16884478e-05,1.11126579e-08, -2.350882012e-12}, blow = { -55508.892,58.53815975}, ahigh = { -1196316.388,3972.78507,4.53590711,0.00413341615, -1.097960689e-06,1.319641252e-10, -6.10112918e-15}, bhigh = { -61995.4174,21.37623045}, R = 28.1823990260087);
          constant IdealGases.Common.DataRecord UF3minus(name = "UF3minus", MM = 0.2950246682, Hf =  -4021496.84037845, H0 = 65615.9673633691, Tlimit = 1000, alow = { -231518.73,2534.197822, -3.18071459,0.02932741422, -2.85691226e-05,1.345804152e-08, -2.465442912e-12}, blow = { -158038.7746,59.73605435}, ahigh = { -191005.3149, -506.918131,11.81482258, -0.0001711879404, -9.38483918e-08,2.242009537e-11, -1.42819528e-15}, bhigh = { -143862.8681, -27.74680411}, R = 28.1822942153554);
          constant IdealGases.Common.DataRecord UF4(name = "UF4", MM = 0.3140225228, Hf =  -5114784.1361142, H0 = 69716.5980477882, Tlimit = 1000, alow = { -50078.5238, -969.349355,20.69596355, -0.02564852637,4.19586012e-05, -3.20101339e-08,9.42207686e-12}, blow = { -193162.6334, -71.72362935}, ahigh = { -1230291.173,3876.46379,7.62162591,0.00409452522, -1.088530094e-06,1.308042117e-10, -6.04439087e-15}, bhigh = { -221414.292,6.270300658}, R = 26.4773110089796);
          constant IdealGases.Common.DataRecord UF4plus(name = "UF4plus", MM = 0.3140219742, Hf =  -2042976.40518432, H0 = 64687.5622374837, Tlimit = 1000, alow = {38489.5633, -1343.282151,16.46334809, -0.00719607986,1.147165041e-05, -8.70587798e-09,2.501076195e-12}, blow = { -74050.482, -52.95285535}, ahigh = { -1617630.159,3765.93477,9.16613853,0.002426641067, -5.6953203e-07,6.0210966e-11, -2.438874329e-15}, bhigh = { -106191.0645, -4.162641042}, R = 26.4773572651464);
          constant IdealGases.Common.DataRecord UF4minus(name = "UF4minus", MM = 0.3140230714, Hf =  -5503846.76608191, H0 = 69666.13918674, Tlimit = 1000, alow = {72031.7969, -1374.343862,16.40877729, -0.00436306627,2.460407959e-06, -1.498144731e-10, -1.239417075e-13}, blow = { -204517.4023, -51.72250795}, ahigh = { -4639200.11,16268.24426, -9.59529105,0.01441556535, -4.00817385e-06,5.14023079e-10, -2.511687821e-14}, bhigh = { -312514.0933,127.4038784}, R = 26.4772647529745);
          constant IdealGases.Common.DataRecord UF5(name = "UF5", MM = 0.333020926, Hf =  -5854959.04842929, H0 = 70875.0716764267, Tlimit = 1000, alow = {161576.9193, -2976.611537,25.3754242, -0.0193010567,2.561852107e-05, -1.747402896e-08,4.73205938e-12}, blow = { -223908.9616, -102.4264836}, ahigh = { -1695373.88,3735.42535,12.1874804,0.002418608203, -5.67852616e-07,6.00276561e-11, -2.430740809e-15}, bhigh = { -264510.7342, -18.83137475}, R = 24.9668154487085);
          constant IdealGases.Common.DataRecord UF5plus(name = "UF5plus", MM = 0.3330203774, Hf =  -2563258.28666844, H0 = 70903.0425836038, Tlimit = 1000, alow = {162651.9713, -2915.370389,24.36168077, -0.01382199656,1.339849041e-05, -7.07857269e-09,1.57340404e-12}, blow = { -92264.0485, -98.30243935}, ahigh = { -262847.8263, -117.4047323,16.08988291, -3.66551671e-05,8.22149497e-09, -9.53716052e-13,4.46014508e-17}, bhigh = { -107629.3931, -47.63374165}, R = 24.966856577708);
          constant IdealGases.Common.DataRecord UF5minus(name = "UF5minus", MM = 0.3330214746, Hf =  -6874725.54059732, H0 = 75649.6650261364, Tlimit = 1000, alow = { -314213.0934,2020.611008,7.50849182,0.01640034396, -1.562795472e-05,7.99024601e-09, -1.671674534e-12}, blow = { -290764.7496,4.690020575}, ahigh = { -1331518.187,3905.51686,10.58727667,0.00411250727, -1.09327793e-06,1.314215436e-10, -6.07577827e-15}, bhigh = { -304950.6955, -7.977511665}, R = 24.9667743198444);
          constant IdealGases.Common.DataRecord UF6(name = "UF6", MM = 0.3520193292, Hf =  -6103760.41816513, H0 = 75630.369674598, Tlimit = 1000, alow = {191567.406, -3426.39161,28.0039589, -0.01326392086,1.107123764e-05, -4.82175411e-09,8.32513245e-13}, blow = { -246104.5194, -121.1612947}, ahigh = { -340902.792, -145.8950046,19.10902533, -4.35845113e-05,9.61760466e-09, -1.100915551e-12,5.09264508e-17}, bhigh = { -264364.5561, -65.79130396}, R = 23.6193620926882);
          constant IdealGases.Common.DataRecord UF6minus(name = "UF6minus", MM = 0.3520198778, Hf =  -7645324.36866837, H0 = 78775.6565717437, Tlimit = 1000, alow = {156923.0858, -3027.27994,27.01442427, -0.01136860992,8.29857616e-06, -2.582889593e-09,2.790817248e-13}, blow = { -313530.7376, -112.8934794}, ahigh = {2699676.848, -6574.45967,22.41731997,0.000605779971, -4.51749171e-07,7.82182047e-11, -4.56033472e-15}, bhigh = { -285566.7984, -91.82896336}, R = 23.6193252834542);
          constant IdealGases.Common.DataRecord UO(name = "UO", MM = 0.25402831, Hf = 120020.437879542, H0 = 37548.1339068075, Tlimit = 1000, alow = {1007249.615, -12871.90666,59.9312392, -0.1003445164,8.52934555e-05, -2.634105066e-08, -5.35064979e-13}, blow = {66274.4252, -322.6786779}, ahigh = { -2458660.003,3942.65216,5.01603072, -0.000545063947,2.096321351e-07, -2.654652526e-11,1.361108472e-15}, bhigh = { -26651.49645,5.580598689}, R = 32.7304936996983);
          constant IdealGases.Common.DataRecord UOplus(name = "UOplus", MM = 0.2540277614, Hf = 2287041.08479381, H0 = 34737.2545085933, Tlimit = 1000, alow = {15628.34007, -122.4921454,3.32414616,0.002467496097, -6.85185674e-07,5.55207319e-11, -5.89301203e-14}, blow = {69529.9974,9.583869879}, ahigh = { -106371.4748, -1793.716133,8.02444905, -0.001565116442,5.06603391e-07, -7.37397747e-11,4.07476643e-15}, bhigh = {77890.8584, -21.1528425}, R = 32.7305643846846);
          constant IdealGases.Common.DataRecord UOF(name = "UOF", MM = 0.2730267132, Hf =  -1985823.06341151, H0 = 51297.0171887195, Tlimit = 1000, alow = {386.124847,51.622869,4.88896252,0.00726563356, -1.134176875e-05,8.50904175e-09, -2.354655485e-12}, blow = { -67198.1338,8.269521875}, ahigh = { -3805936.01,12952.07909, -10.25373631,0.01047841671, -2.648894949e-06,3.114551623e-10, -1.416111895e-14}, bhigh = { -147897.6701,118.4570601}, R = 30.452961552921);
          constant IdealGases.Common.DataRecord UOF2(name = "UOF2", MM = 0.2920251164, Hf =  -3819912.06013949, H0 = 61009.9440063095, Tlimit = 1000, alow = { -91066.1831, -101.4150169,13.71057013, -0.01659931694,3.082352584e-05, -2.490530206e-08,7.58061764e-12}, blow = { -137468.6016, -34.70029665}, ahigh = { -1198944.06,3852.17341,4.63910975,0.00408773844, -1.087067018e-06,1.306398384e-10, -6.03690152e-15}, bhigh = { -161263.7583,20.18237189}, R = 28.4717701768203);
          constant IdealGases.Common.DataRecord UOF3(name = "UOF3", MM = 0.3110235196, Hf =  -4856989.72522334, H0 = 63748.1082636427, Tlimit = 1000, alow = {124925.6374, -2111.719498,18.44931536, -0.01041493583,1.469002962e-05, -1.049770051e-08,2.922353602e-12}, blow = { -174384.4024, -66.49285435}, ahigh = { -1658296.049,3714.43468,9.20242554,0.002412862293, -5.66624172e-07,5.98906211e-11, -2.42453481e-15}, bhigh = { -210549.1714, -5.670532802}, R = 26.7326149826002);
          constant IdealGases.Common.DataRecord UOF4(name = "UOF4", MM = 0.3300219228, Hf =  -5410584.59344265, H0 = 68912.7976924811, Tlimit = 1000, alow = {143716.4405, -2496.730363,21.04827038, -0.00469256133,8.74171865e-07,1.517555623e-09, -7.76879974e-13}, blow = { -206128.4178, -82.46219615}, ahigh = { -299619.0288, -167.7276501,16.12494871, -4.98441446e-05,1.098253497e-08, -1.255803231e-12,5.80448098e-17}, bhigh = { -219545.926, -50.24484645}, R = 25.1936960110336);
          constant IdealGases.Common.DataRecord UO2(name = "UO2", MM = 0.27002771, Hf =  -1769521.73908374, H0 = 49847.0953221801, Tlimit = 1000, alow = { -112965.0727,427.073027,8.41369401, -0.00976428,2.199903691e-05, -1.907665954e-08,6.02988991e-12}, blow = { -62514.433, -13.00704863}, ahigh = { -1190542.635,3832.18635,2.153236312,0.00408235191, -1.085924847e-06,1.305134065e-10, -6.03121575e-15}, bhigh = { -83676.1801,25.90742388}, R = 30.7911806532744);
          constant IdealGases.Common.DataRecord UO2plus(name = "UO2plus", MM = 0.2700271614, Hf = 190699.034619412, H0 = 42871.6835002066, Tlimit = 1000, alow = {44880.2338, -648.923642,6.87424356,0.002281949316,1.859218752e-07, -1.849509036e-09,8.14996533e-13}, blow = {7891.74023, -10.28257233}, ahigh = { -1589957.42,3657.74607,3.74678542,0.002394401448, -5.62410144e-07,5.93945164e-11, -2.401047056e-15}, bhigh = { -20485.58817,14.64015614}, R = 30.7912432100988);
          constant IdealGases.Common.DataRecord UO2minus(name = "UO2minus", MM = 0.2700282586, Hf =  -2124592.5295909, H0 = 51021.9895926107, Tlimit = 1000, alow = {63419.7131, -719.61185,8.30824162,0.000962645545, -3.58373398e-06,3.47355458e-09, -1.02279312e-12}, blow = { -67181.6987, -16.0603012}, ahigh = { -273825.164,3228.53442, -0.2805114576,0.00660731071, -2.070291015e-06,2.851708013e-10, -1.466578192e-14}, bhigh = { -88934.3618,41.45760316}, R = 30.7911180967043);
          constant IdealGases.Common.DataRecord UO2F(name = "UO2F", MM = 0.2890261132, Hf =  -3452749.75659189, H0 = 54679.3430705139, Tlimit = 1000, alow = {83217.9239, -1327.082127,12.4012539, -0.00408415101,7.33159348e-06, -5.98740488e-09,1.787869209e-12}, blow = { -115752.6888, -34.15298625}, ahigh = { -1614277.202,3712.77186,6.20330882,0.002412618055, -5.66588802e-07,5.98882352e-11, -2.424485602e-15}, bhigh = { -147835.7384,8.736139132}, R = 28.7671999873816);
          constant IdealGases.Common.DataRecord UO2F2(name = "UO2F2", MM = 0.3080245164, Hf =  -4396507.2742502, H0 = 61904.0757627174, Tlimit = 1000, alow = {99598.743, -1530.010701,13.58992177,0.0053706558, -1.141444341e-05,9.30119704e-09, -2.781220538e-12}, blow = { -158031.0734, -41.96260275}, ahigh = { -263655.2036, -203.2835065,13.15092994, -6.00648683e-05,1.321137913e-08, -1.508668985e-12,6.96618275e-17}, bhigh = { -166444.4818, -35.66258175}, R = 26.9928903620219);
          constant IdealGases.Common.DataRecord UO3(name = "UO3", MM = 0.28602711, Hf =  -2794278.52485731, H0 = 52948.6348339498, Tlimit = 1000, alow = {66376.7704, -758.264658,7.11284471,0.01322149697, -2.106191042e-05,1.545856318e-08, -4.37856629e-12}, blow = { -94133.6935, -8.588030428}, ahigh = { -1097362.721,2808.784061,5.96612147,0.002861871152, -1.05284381e-06,1.849985929e-10, -1.102849619e-14}, bhigh = { -117336.0017,6.672828312}, R = 29.0688249795623);
          constant IdealGases.Common.DataRecord UO3minus(name = "UO3minus", MM = 0.2860276586, Hf =  -4563036.05178692, H0 = 53935.6301258063, Tlimit = 1000, alow = {100014.3902, -1448.443754,12.17532415, -0.002944956748,5.675228e-06, -4.91671042e-09,1.525185713e-12}, blow = { -151925.3376, -34.71731375}, ahigh = { -1632938.634,3667.49541,6.24003065,0.002396952522, -5.62949023e-07,5.94541313e-11, -2.403730982e-15}, bhigh = { -184592.7385,7.032059012}, R = 29.0687692256626);
          constant IdealGases.Common.DataRecord V(name = "V", MM = 0.0509415, Hf = 10154138.8455385, H0 = 155218.515355849, Tlimit = 1000, alow = { -55353.7602,559.333851,2.675543482, -0.00624304963,1.565902337e-05, -1.372845314e-08,4.16838881e-12}, blow = {58206.6436,9.52456749}, ahigh = {1200390.3, -5027.0053,10.58830594, -0.0050443261,1.488547375e-06, -1.785922508e-10,8.113013866e-15}, bhigh = {91707.4091, -47.6833632}, R = 163.21608119117);
          constant IdealGases.Common.DataRecord Vplus(name = "Vplus", MM = 0.0509409514, Hf = 23041293.4729759, H0 = 155038.545275383, Tlimit = 1000, alow = {75688.3446, -841.527382,7.55923271, -0.01441722656,2.038356397e-05, -1.289073883e-08,3.065656561e-12}, blow = {144447.8191, -19.91067645}, ahigh = {2347072.054, -9021.19719,14.77349798, -0.00689189688,1.968884877e-06, -2.539798544e-10,1.226783122e-14}, bhigh = {195835.1444, -78.5559293}, R = 163.217838919279);
          constant IdealGases.Common.DataRecord Vminus(name = "Vminus", MM = 0.0509420486, Hf = 9037446.97460008, H0 = 154651.475873312, Tlimit = 1000, alow = { -3799.27356,231.3840448,1.72560819,0.001429275357, -1.506038188e-06,8.49181517e-10, -1.987980413e-13}, blow = {53474.0292,12.61900982}, ahigh = {26001.0043,2.096334097,2.498006548,8.99127884e-07, -2.139749508e-10,2.581021334e-14, -1.240812796e-18}, bhigh = {54700.0708,7.97790024}, R = 163.21432350092);
          constant IdealGases.Common.DataRecord VCL4(name = "VCL4", MM = 0.1927535, Hf =  -2734365.29038383, H0 = 113010.129517752, Tlimit = 1000, alow = {77198.3471, -1702.85404,18.82697965, -0.01130896832,1.266950765e-05, -7.62501934e-09,1.907582465e-12}, blow = { -58637.4854, -65.59171106}, ahigh = { -1717776.251,4550.44105,8.1644648,0.002224875998, -4.09411178e-07,3.2717875e-11, -8.8586908e-16}, bhigh = { -96906.2006,4.348293098}, R = 43.135258244338);
          constant IdealGases.Common.DataRecord VN(name = "VN", MM = 0.0649482, Hf = 8052571.12591265, H0 = 135488.650955685, Tlimit = 1000, alow = { -15817.37285,486.816542, -1.045200388,0.01685261043, -2.334616543e-05,1.58871005e-08, -4.27908088e-12}, blow = {59814.814,31.44995263}, ahigh = {1018619.667, -3932.30557,9.85682361, -0.00328922171,1.005181767e-06, -1.243211436e-10,5.48677656e-15}, bhigh = {85573.1834, -35.52830762}, R = 128.016973526595);
          constant IdealGases.Common.DataRecord VO(name = "VO", MM = 0.0669409, Hf = 2219610.22334627, H0 = 131057.485035307, Tlimit = 1000, alow = { -13116.19784,374.781697,0.0930083486,0.01244977714, -1.688540028e-05,1.142443381e-08, -3.04058932e-12}, blow = {15237.8992,25.36811755}, ahigh = {2986190.283, -10113.44974,17.18161749, -0.00787670503,2.562279547e-06, -3.54740035e-10,1.770268056e-14}, bhigh = {79612.5488, -87.8999301}, R = 124.206157969194);
          constant IdealGases.Common.DataRecord VO2(name = "VO2", MM = 0.0829403, Hf =  -2805604.21170408, H0 = 128073.433541957, Tlimit = 1000, alow = { -6678.58586,391.159758, -1.028549847,0.02401523419, -3.33737881e-05,2.283623543e-08, -6.1991431e-12}, blow = { -30746.09276,32.7791238}, ahigh = {121063.2401, -1627.832993,9.2527131, -0.001572703139,5.23143016e-07, -6.47607114e-11,2.847226026e-15}, bhigh = { -20973.06345, -25.47380687}, R = 100.246466434291);
          constant IdealGases.Common.DataRecord V4O10(name = "V4O10", MM = 0.36376, Hf =  -7766561.70552012, H0 = 101154.698152628, Tlimit = 1000, alow = {338573.916, -5353.92926,29.93220131,0.0581883652, -9.69634016e-05,7.27475699e-08, -2.091628357e-11}, blow = { -318934.937, -146.6398396}, ahigh = { -1360273.27, -1341.69286,40.9913859, -0.000393226553,8.62849412e-08, -9.83594409e-12,4.53563729e-16}, bhigh = { -348461.612, -190.7911348}, R = 22.8570266109523);
          constant IdealGases.Common.DataRecord W(name = "W", MM = 0.18384, Hf = 4630349.90208877, H0 = 33814.8716275022, Tlimit = 1000, alow = {159522.3922, -2673.843928,20.60469727, -0.0625231523,0.0001105654838, -8.45351161e-08,2.336187771e-11}, blow = {113964.8616, -90.118369}, ahigh = { -8048745.96,14657.00424, -0.2508531501, -0.002596486992,1.409225475e-06, -2.233011706e-10,1.262640862e-14}, bhigh = { -3091.130919,39.5582219}, R = 45.2266753698869);
          constant IdealGases.Common.DataRecord Wplus(name = "Wplus", MM = 0.1838394514, Hf = 8854687.89535346, H0 = 33840.7504625528, Tlimit = 1000, alow = { -196928.4929,2670.137332, -11.31686913,0.0330818373, -3.6290355e-05,2.066142971e-08, -4.808285562e-12}, blow = {182095.0862,85.5210448}, ahigh = {6387743.4, -20618.11463,27.59291576, -0.01244535845,3.27120049e-06, -4.06546372e-10,1.912595872e-14}, bhigh = {324517.443, -171.6919194}, R = 45.2268103319634);
          constant IdealGases.Common.DataRecord Wminus(name = "Wminus", MM = 0.1838405486, Hf = 4168783.94802614, H0 = 33710.8872182728, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {91429.8137,8.46116967}, ahigh = {0,0,2.5,0,0,0,0}, bhigh = {91429.8137,8.46116967}, R = 45.2265404086158);
          constant IdealGases.Common.DataRecord WCL6(name = "WCL6", MM = 0.396558, Hf =  -1244993.16619511, H0 = 77645.5272620903, Tlimit = 1000, alow = {33393.9167, -1697.36608,25.60612352, -0.01425247881,1.739682668e-05, -1.120537864e-08,2.95152289e-12}, blow = { -56730.5924, -97.41466299}, ahigh = { -151673.5473, -34.5137925,19.02867176, -1.242486969e-05,2.917675103e-09, -3.50598144e-13,1.685355887e-17}, bhigh = { -65357.7979, -58.94914959}, R = 20.966597572108);
          constant IdealGases.Common.DataRecord WO(name = "WO", MM = 0.1998394, Hf = 2010292.82013457, H0 = 46332.7602064458, Tlimit = 1000, alow = { -19337.58411,493.669084, -0.411614822,0.01307976507, -1.689145619e-05,1.092748066e-08, -2.820593541e-12}, blow = {45110.179,30.02592661}, ahigh = {1262156.956, -4177.26312,9.35828647, -0.00288761222,8.89393396e-07, -8.9553187e-11,2.504359614e-15}, bhigh = {73133.8064, -31.4488229}, R = 41.6057694328546);
          constant IdealGases.Common.DataRecord WOCL4(name = "WOCL4", MM = 0.3416514, Hf =  -1678591.10192436, H0 = 65187.4366679018, Tlimit = 1000, alow = {26588.12693, -933.691324,13.21377385,0.01270930811, -1.964716509e-05,1.405106974e-08, -3.89938214e-12}, blow = { -67922.9079, -35.94869683}, ahigh = { -304902.8285, -353.942586,16.26432703, -0.0001058245461,2.340548846e-08, -2.685852021e-12,1.245412684e-16}, bhigh = { -72725.4674, -49.91761253}, R = 24.3361274094003);
          constant IdealGases.Common.DataRecord WO2(name = "WO2", MM = 0.2158388, Hf = 134645.596621182, H0 = 49642.8260349854, Tlimit = 1000, alow = {3120.918919,241.3883468, -0.3024119184,0.02324333417, -3.37881268e-05,2.380934019e-08, -6.42159288e-12}, blow = {1442.005265,29.57655179}, ahigh = { -753740.668,3204.64305,0.67019656,0.00448823887, -8.85832846e-07,6.24517175e-11, -9.0466139e-16}, bhigh = { -17694.2023,34.0572401}, R = 38.5216745089391);
          constant IdealGases.Common.DataRecord WO2CL2(name = "WO2CL2", MM = 0.2867448, Hf =  -2341915.18032759, H0 = 68018.0320619589, Tlimit = 1000, alow = {430.015448, -243.8547856,7.97630699,0.01715005722, -2.487975176e-05,1.737644163e-08, -4.7719284e-12}, blow = { -82328.124, -7.844771421}, ahigh = { -240479.173, -306.8379644,13.22814502, -9.10022071e-05,2.00658102e-08, -2.296837607e-12,1.062829386e-16}, bhigh = { -83674.9306, -34.91556626}, R = 28.9960689784087);
          constant IdealGases.Common.DataRecord WO3(name = "WO3", MM = 0.2318382, Hf =  -1379087.71289632, H0 = 57229.3220012923, Tlimit = 1000, alow = {7262.46146,34.2939109,1.573061955,0.02754971099, -3.99482731e-05,2.809371537e-08, -7.77754573e-12}, blow = { -40017.3327,18.57409963}, ahigh = {1732203.64, -6284.71978,16.81358864, -0.00347208936,8.0799358e-07, -6.58937872e-11,1.142928957e-15}, bhigh = { -2473.075565, -74.0747409}, R = 35.8632529065529);
          constant IdealGases.Common.DataRecord WO3minus(name = "WO3minus", MM = 0.2318387486, Hf =  -2805725.81127191, H0 = 59155.9395606572, Tlimit = 1000, alow = {163099.9684, -1967.609682,12.01970914,0.001681260779, -5.85936006e-06,5.02861633e-09, -1.434012198e-12}, blow = { -70092.0399, -39.3690104}, ahigh = {463783.251, -1347.089334,9.77363756,0.001071395459, -3.84240366e-07,5.7375319e-11, -3.157605982e-15}, bhigh = { -72395.1557, -23.91116343}, R = 35.8631680433423);
          constant IdealGases.Common.DataRecord Xe(name = "Xe", MM = 0.131293, Hf = 0, H0 = 47203.0344344329, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = { -745.375,6.164454205}, ahigh = {4025.22668, -12.09507521,2.514153347, -8.24810208e-06,2.530232618e-09, -3.89233323e-13,2.360439138e-17}, bhigh = { -668.580073,6.063710715}, R = 63.3276107637117);
          constant IdealGases.Common.DataRecord Xeplus(name = "Xeplus", MM = 0.1312924514, Hf = 8961309.04293558, H0 = 47203.231670332, Tlimit = 1000, alow = {100.292362, -1.218753648,2.506016493, -1.547411334e-05,2.191372741e-08, -1.623684074e-11,4.92913267e-15}, blow = {140766.5368,7.516712465}, ahigh = { -12416.83887, -150.0654643,2.964678293, -0.000469339666,1.959138719e-07, -3.037761925e-11,1.637361082e-15}, bhigh = {141496.6808,4.565685735}, R = 63.3278753754765);
          constant IdealGases.Common.DataRecord Zn(name = "Zn", MM = 0.06539, Hf = 1994188.71387062, H0 = 94776.3878268848, Tlimit = 1000, alow = {0,0,2.5,0,0,0,0}, blow = {14938.05072,5.11886101}, ahigh = { -175559.1489,498.413924,1.969386292,0.0002608808787, -5.62719508e-08,2.723336049e-12,4.266685808e-16}, bhigh = {11737.73458,8.96108565}, R = 127.152041596574);
          constant IdealGases.Common.DataRecord Znplus(name = "Znplus", MM = 0.0653894514, Hf = 15950586.9168372, H0 = 94777.182975418, Tlimit = 1000, alow = {0.000409834494, -4.34358162e-06,2.500000019, -4.38903681e-11,5.5639692e-14, -3.63822826e-17,9.607881995e-21}, blow = {124697.9918,5.8119955}, ahigh = { -343617.946,956.735524,1.511478952,0.000461346796, -8.78680098e-08,7.55856778e-13,1.168827311e-15}, bhigh = {118532.1933,13.0074267}, R = 127.153108368179);
          constant IdealGases.Common.DataRecord Zr(name = "Zr", MM = 0.091224, Hf = 6569747.11698676, H0 = 74712.9154608436, Tlimit = 1000, alow = {67158.9996, -943.598174,6.35975618, -0.000979011973, -7.60822415e-06,9.30871743e-09, -3.124675586e-12}, blow = {75880.1947, -16.65770522}, ahigh = {6006771.84, -15669.60605,17.9698235, -0.00676340965,1.733678968e-06, -2.064699786e-10,9.33409261e-15}, bhigh = {173463.6249, -105.1117377}, R = 91.1434710163992);
          constant IdealGases.Common.DataRecord Zrplus(name = "Zrplus", MM = 0.0912234514, Hf = 13661468.3272113, H0 = 81907.5784277989, Tlimit = 1000, alow = {173984.2193, -2224.598466,14.00787829, -0.02378785396,2.641058912e-05, -1.442565487e-08,3.135982142e-12}, blow = {159821.0714, -58.1672881}, ahigh = {729813.716, -2017.117556,5.0374983, -0.000550337195,1.023753499e-07, -1.261537793e-11,7.092401042e-16}, bhigh = {162088.4945, -9.82064086}, R = 91.1440191354128);
          constant IdealGases.Common.DataRecord Zrminus(name = "Zrminus", MM = 0.0912245486, Hf = 6061442.96119871, H0 = 84949.6996031176, Tlimit = 1000, alow = {30466.62367, -807.427721,9.21300611, -0.01614342054,1.908653551e-05, -1.138888914e-08,2.745019116e-12}, blow = {69030.3403, -28.62644371}, ahigh = {84718.6116,317.543934,2.251491246,0.0001018645389, -2.285208242e-08,2.647293502e-12, -1.235813604e-16}, bhigh = {64223.376,10.81261057}, R = 91.1429229039781);
          constant IdealGases.Common.DataRecord ZrN(name = "ZrN", MM = 0.1052307, Hf = 6779124.34299116, H0 = 84223.2352345846, Tlimit = 1000, alow = {22591.09156, -180.2590198,3.130058377,0.00542770928, -8.26461484e-06,5.95843822e-09, -1.670274535e-12}, blow = {85788.8149,8.47072478}, ahigh = { -72557.9089, -81.267966,4.5599244,1.29116165e-05,5.20383077e-09, -5.92743667e-13,2.731548854e-17}, bhigh = {84686.4839,1.493633264}, R = 79.0118473031159);
          constant IdealGases.Common.DataRecord ZrO(name = "ZrO", MM = 0.1072234, Hf = 782690.233661682, H0 = 83658.1194030408, Tlimit = 1000, alow = { -509176.14,8652.77009, -52.9474015,0.1728961761, -0.0002457230895,1.672135156e-07, -4.42301238e-11}, blow = { -30951.29818,313.2576719}, ahigh = {464809.831,344.231447,4.81577918, -0.000466063314,2.140489079e-07, -2.054364483e-11,4.08466776e-16}, bhigh = {7317.3407,1.502933548}, R = 77.543446673021);
          constant IdealGases.Common.DataRecord ZrOplus(name = "ZrOplus", MM = 0.1072228514, Hf = 6720713.95780844, H0 = 88265.830244466, Tlimit = 1000, alow = {10329.11549,73.0466122,2.606345299,0.00509930029, -6.25033876e-06,3.79746002e-09, -9.1984169e-13}, blow = {85332.3378,12.67311019}, ahigh = { -493716.656,669.990076,4.57535347, -0.00069232379,4.28060094e-07, -7.54766022e-11,4.41427986e-15}, bhigh = {80188.8066,2.928943704}, R = 77.5438434199298);
          constant IdealGases.Common.DataRecord ZrO2(name = "ZrO2", MM = 0.1232228, Hf =  -2572922.68151673, H0 = 97451.6323277835, Tlimit = 1000, alow = {36376.49, -262.0658297,3.69286687,0.01214524156, -1.822445342e-05,1.299215204e-08, -3.61590011e-12}, blow = { -38019.9088,8.2908576}, ahigh = {2854363.887, -8738.58989,16.21315114, -0.00441792283,8.95909692e-07, -4.05466711e-11, -2.147732083e-15}, bhigh = {15275.10023, -74.8199549}, R = 67.4751101257235);
        end SingleGasesData;
        package FluidData "Critical data, dipole moments and related data"
          extends Modelica.Icons.Library;
          import Modelica.Media.Interfaces.PartialMedium;
          import Modelica.Media.IdealGases.Common.SingleGasesData;
          constant SingleGasNasa.FluidConstants N2(chemicalFormula = "N2", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "7727-37-9", meltingPoint = 63.15, normalBoilingPoint = 77.35, criticalTemperature = 126.2, criticalPressure = 3398000.0, criticalMolarVolume = 9.01e-05, acentricFactor = 0.037, dipoleMoment = 0.0, molarMass = SingleGasesData.N2.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants O2(chemicalFormula = "O2", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "7782-44-7", meltingPoint = 54.36, normalBoilingPoint = 90.17, criticalTemperature = 154.58, criticalPressure = 5043000.0, criticalMolarVolume = 7.337e-05, acentricFactor = 0.022, dipoleMoment = 0.0, molarMass = SingleGasesData.O2.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants CL2(chemicalFormula = "CL2", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "7782-50-5", meltingPoint = 172.19, normalBoilingPoint = 239.12, criticalTemperature = 417.0, criticalPressure = 7700000.0, criticalMolarVolume = 0.000124, acentricFactor = 0.069, dipoleMoment = 0.0, molarMass = SingleGasesData.CL2.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants F2(chemicalFormula = "F2", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "7782-41-4", meltingPoint = 53.48, normalBoilingPoint = 84.95, criticalTemperature = 144.3, criticalPressure = 5215000.0, criticalMolarVolume = 6.62e-05, acentricFactor = 0.051, dipoleMoment = 0.0, molarMass = SingleGasesData.F2.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants CO2(chemicalFormula = "CO2", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "124-38-9", meltingPoint = 216.58, normalBoilingPoint =  -1.0, criticalTemperature = 304.12, criticalPressure = 7374000.0, criticalMolarVolume = 9.407e-05, acentricFactor = 0.225, dipoleMoment = 0.0, molarMass = SingleGasesData.CO2.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants CO(chemicalFormula = "CO", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "630-08-0", meltingPoint = 68.15, normalBoilingPoint = 81.66, criticalTemperature = 132.85, criticalPressure = 3494000.0, criticalMolarVolume = 9.31e-05, acentricFactor = 0.045, dipoleMoment = 0.1, molarMass = SingleGasesData.CO.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants H2(chemicalFormula = "H2", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "800000-51-5", meltingPoint = 13.56, normalBoilingPoint = 20.38, criticalTemperature = 33.25, criticalPressure = 1297000.0, criticalMolarVolume = 6.5e-05, acentricFactor =  -0.216, dipoleMoment = 0.0, molarMass = SingleGasesData.H2.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants H2O(chemicalFormula = "H2O", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "7732-18-5", meltingPoint = 273.15, normalBoilingPoint = 373.15, criticalTemperature = 647.14, criticalPressure = 22064000.0, criticalMolarVolume = 5.595e-05, acentricFactor = 0.344, dipoleMoment = 1.8, molarMass = SingleGasesData.H2O.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants N2O(chemicalFormula = "N2O", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "10024-97-2", meltingPoint = 182.33, normalBoilingPoint = 184.67, criticalTemperature = 309.6, criticalPressure = 7255000.0, criticalMolarVolume = 9.7e-05, acentricFactor = 0.142, dipoleMoment = 0.2, molarMass = SingleGasesData.N2O.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants NO(chemicalFormula = "NO", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "10102-43-9", meltingPoint = 109.51, normalBoilingPoint = 121.38, criticalTemperature = 180.0, criticalPressure = 6480000.0, criticalMolarVolume = 5.8e-05, acentricFactor = 0.582, dipoleMoment = 0.2, molarMass = SingleGasesData.NO.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants NO2(chemicalFormula = "NO2", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "10102-44-0", meltingPoint = 261.95, normalBoilingPoint = 294.0, criticalTemperature = 431.35, criticalPressure = 10133000.0, criticalMolarVolume = 8.25e-05, acentricFactor = 0.849, dipoleMoment = 0.32, molarMass = SingleGasesData.NO2.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants NH3(chemicalFormula = "NH3", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "7664-41-7", meltingPoint = 195.41, normalBoilingPoint = 239.82, criticalTemperature = 405.4, criticalPressure = 11353000.0, criticalMolarVolume = 7.247e-05, acentricFactor = 0.257, dipoleMoment = 1.5, molarMass = SingleGasesData.NH3.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants SO2(chemicalFormula = "SO2", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "7446-09-5", meltingPoint = 197.67, normalBoilingPoint = 263.13, criticalTemperature = 430.8, criticalPressure = 7884000.0, criticalMolarVolume = 0.000122, acentricFactor = 0.245, dipoleMoment = 1.6, molarMass = SingleGasesData.SO2.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants SO3(chemicalFormula = "SO3", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "7446-11-9", meltingPoint = 289.95, normalBoilingPoint = 317.9, criticalTemperature = 490.9, criticalPressure = 8210000.0, criticalMolarVolume = 0.0001265, acentricFactor = 0.422, dipoleMoment = 0.0, molarMass = SingleGasesData.SO3.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants Ar(chemicalFormula = "Ar", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "7440-37-1", meltingPoint = 83.8, normalBoilingPoint = 87.27, criticalTemperature = 150.86, criticalPressure = 4898000.0, criticalMolarVolume = 7.457e-05, acentricFactor =  -0.002, dipoleMoment = 0.0, molarMass = SingleGasesData.Ar.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants He(chemicalFormula = "He", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "7440-59-7", meltingPoint = 2.15, normalBoilingPoint = 4.3, criticalTemperature = 5.19, criticalPressure = 227000.0, criticalMolarVolume = 5.73e-05, acentricFactor =  -0.39, dipoleMoment = 0.0, molarMass = SingleGasesData.He.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants Ne(chemicalFormula = "Ne", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "7440-01-9", meltingPoint = 24.56, normalBoilingPoint = 27.07, criticalTemperature = 44.4, criticalPressure = 2760000.0, criticalMolarVolume = 4.17e-05, acentricFactor =  -0.016, dipoleMoment = 0.0, molarMass = SingleGasesData.Ne.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants CH4(chemicalFormula = "CH4", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "74-82-8", meltingPoint = 90.69, normalBoilingPoint = 111.66, criticalTemperature = 190.56, criticalPressure = 4599000.0, criticalMolarVolume = 9.86e-05, acentricFactor = 0.011, dipoleMoment = 0.0, molarMass = SingleGasesData.CH4.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants C2H6(chemicalFormula = "C2H6", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "74-84-0", meltingPoint = 90.35, normalBoilingPoint = 184.55, criticalTemperature = 305.32, criticalPressure = 4872000.0, criticalMolarVolume = 0.0001455, acentricFactor = 0.099, dipoleMoment = 0.0, molarMass = SingleGasesData.C2H6.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants C3H8(chemicalFormula = "C3H8", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "74-98-6", meltingPoint = 91.45, normalBoilingPoint = 231.02, criticalTemperature = 369.83, criticalPressure = 4248000.0, criticalMolarVolume = 0.0002, acentricFactor = 0.152, dipoleMoment = 0.0, molarMass = SingleGasesData.C3H8.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants C4H10_n_butane(chemicalFormula = "C4H10", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "106-97-8", meltingPoint = 134.79, normalBoilingPoint = 272.66, criticalTemperature = 425.12, criticalPressure = 3796000.0, criticalMolarVolume = 0.000255, acentricFactor = 0.2, dipoleMoment = 0.0, molarMass = SingleGasesData.C4H10_n_butane.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants C5H12_n_pentane(chemicalFormula = "C5H12", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "109-66-0", meltingPoint = 143.43, normalBoilingPoint = 309.22, criticalTemperature = 469.7, criticalPressure = 3370000.0, criticalMolarVolume = 0.000311, acentricFactor = 0.252, dipoleMoment = 0.0, molarMass = SingleGasesData.C5H12_n_pentane.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants C6H14_n_hexane(chemicalFormula = "C6H14", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "110-54-3", meltingPoint = 177.84, normalBoilingPoint = 341.88, criticalTemperature = 507.6, criticalPressure = 3025000.0, criticalMolarVolume = 0.000368, acentricFactor = 0.3, dipoleMoment = 0.0, molarMass = SingleGasesData.C6H14_n_hexane.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants C7H16_n_heptane(chemicalFormula = "C7H16", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "142-82-5", meltingPoint = 182.59, normalBoilingPoint = 371.57, criticalTemperature = 540.2, criticalPressure = 2740000.0, criticalMolarVolume = 0.000428, acentricFactor = 0.35, dipoleMoment = 0.0, molarMass = SingleGasesData.C7H16_n_heptane.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants C2H4(chemicalFormula = "C2H4", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "74-85-1", meltingPoint = 103.99, normalBoilingPoint = 169.42, criticalTemperature = 282.34, criticalPressure = 5041000.0, criticalMolarVolume = 0.0001311, acentricFactor = 0.087, dipoleMoment = 0.0, molarMass = SingleGasesData.C2H4.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants C3H6_propylene(chemicalFormula = "C3H6", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "115-07-1", meltingPoint = 87.89, normalBoilingPoint = 225.46, criticalTemperature = 364.9, criticalPressure = 4600000.0, criticalMolarVolume = 0.0001846, acentricFactor = 0.142, dipoleMoment = 0.4, molarMass = SingleGasesData.C3H6_propylene.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants C4H8_1_butene(chemicalFormula = "C4H8", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "106-98-9", meltingPoint = 87.79, normalBoilingPoint = 266.92, criticalTemperature = 419.5, criticalPressure = 4020000.0, criticalMolarVolume = 0.0002408, acentricFactor = 0.194, dipoleMoment = 0.3, molarMass = SingleGasesData.C4H8_1_butene.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants C5H10_1_pentene(chemicalFormula = "C5H10", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "109-67-1", meltingPoint = 106.95, normalBoilingPoint = 303.11, criticalTemperature = 464.8, criticalPressure = 3560000.0, criticalMolarVolume = 0.0002984, acentricFactor = 0.237, dipoleMoment = 0.4, molarMass = SingleGasesData.C5H10_1_pentene.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants C6H12_1_hexene(chemicalFormula = "C6H12", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "592-41-6", meltingPoint = 133.34, normalBoilingPoint = 336.63, criticalTemperature = 504.0, criticalPressure = 3143000.0, criticalMolarVolume = 0.0003551, acentricFactor = 0.281, dipoleMoment = 0.4, molarMass = SingleGasesData.C6H12_1_hexene.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants C7H14_1_heptene(chemicalFormula = "C7H14", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "592-76-7", meltingPoint = 153.45, normalBoilingPoint = 366.79, criticalTemperature = 537.3, criticalPressure = 2920000.0, criticalMolarVolume = 0.000409, acentricFactor = 0.343, dipoleMoment = 0.3, molarMass = SingleGasesData.C7H14_1_heptene.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants C2H2_vinylidene(chemicalFormula = "C2H2", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "74-86-2", meltingPoint = 192.35, normalBoilingPoint = 188.4, criticalTemperature = 308.3, criticalPressure = 6114000.0, criticalMolarVolume = 0.0001122, acentricFactor = 0.189, dipoleMoment = 0.0, molarMass = SingleGasesData.C2H2_vinylidene.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants C6H6(chemicalFormula = "C6H6", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "71-43-2", meltingPoint = 278.68, normalBoilingPoint = 353.24, criticalTemperature = 562.05, criticalPressure = 4895000.0, criticalMolarVolume = 0.000256, acentricFactor = 0.21, dipoleMoment = 0.0, molarMass = SingleGasesData.C6H6.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants C8H18_n_octane(chemicalFormula = "C8H18", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "111-65-9", meltingPoint = 216.39, normalBoilingPoint = 398.82, criticalTemperature = 568.7, criticalPressure = 2490000.0, criticalMolarVolume = 0.000492, acentricFactor = 0.399, dipoleMoment = 0.0, molarMass = SingleGasesData.C8H18_n_octane.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants C8H10_ethylbenz(chemicalFormula = "C8H10", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "100-41-4", meltingPoint = 178.18, normalBoilingPoint = 409.36, criticalTemperature = 617.15, criticalPressure = 3609000.0, criticalMolarVolume = 0.000374, acentricFactor = 0.304, dipoleMoment = 0.4, molarMass = SingleGasesData.C8H10_ethylbenz.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants CH3OH(chemicalFormula = "CH3OH", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "67-56-1", meltingPoint = 175.49, normalBoilingPoint = 337.69, criticalTemperature = 512.64, criticalPressure = 8097000.0, criticalMolarVolume = 0.000118, acentricFactor = 0.565, dipoleMoment = 1.7, molarMass = SingleGasesData.CH3OH.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants C2H5OH(chemicalFormula = "C2H5OH", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "64-17-5", meltingPoint = 159.05, normalBoilingPoint = 351.8, criticalTemperature = 513.92, criticalPressure = 6148000.0, criticalMolarVolume = 0.000167, acentricFactor = 0.649, dipoleMoment = 1.7, molarMass = SingleGasesData.C2H5OH.MM, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants C3H7OH(chemicalFormula = "C3H7OH", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "71-23-8", meltingPoint = 147.0, normalBoilingPoint = 370.93, criticalTemperature = 536.78, criticalPressure = 5175000.0, criticalMolarVolume = 0.000219, acentricFactor = 0.629, dipoleMoment = 1.7, molarMass = 0.0601, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          constant SingleGasNasa.FluidConstants C4H9OH(chemicalFormula = "C4H9OH", iupacName = "unknown", structureFormula = "unknown", casRegistryNumber = "71-36-3", meltingPoint = 183.35, normalBoilingPoint = 390.88, criticalTemperature = 563.05, criticalPressure = 4423000.0, criticalMolarVolume = 0.000275, acentricFactor = 0.589, dipoleMoment = 1.8, molarMass = 0.07412, hasDipoleMoment = true, hasIdealGasHeatCapacity = true, hasCriticalData = true, hasAcentricFactor = true);
          annotation(Documentation(info = "<html>
<p>
This package contains FluidConstants data records for the following 37 gases
(see also the description in
<a href=\"Modelica://Modelica.Media.IdealGases\">Modelica.Media.IdealGases</a>):
</p>
<pre>
Argon             Methane          Methanol       Carbon Monoxide  Carbon Dioxide
Acetylene         Ethylene         Ethanol        Ethane           Propylene
Propane           1-Propanol       1-Butene       N-Butane         1-Pentene
N-Pentane         Benzene          1-Hexene       N-Hexane         1-Heptane
N-Heptane         Ethylbenzene     N-Octane       Chlorine         Fluorine
Hydrogen          Steam            Helium         Ammonia          Nitric Oxide
Nitrogen Dioxide  Nitrogen         Nitrous        Oxide            Neon Oxygen
Sulfur Dioxide    Sulfur Trioxide
</pre>

</html>"));
        end FluidData;
      end Common;
      package SingleGases "Media models of ideal gases from NASA tables"
        extends Modelica.Icons.Library;
        annotation(Documentation(info = "<HTML>
<p>
This package contains medium
models for the following 37 gases
(see also the description in
<a href=\"Modelica://Modelica.Media.IdealGases\">Modelica.Media.IdealGases</a>):
</p>
<pre>
Argon             Methane          Methanol       Carbon Monoxide  Carbon Dioxide
Acetylene         Ethylene         Ethanol        Ethane           Propylene
Propane           1-Propanol       1-Butene       N-Butane         1-Pentene
N-Pentane         Benzene          1-Hexene       N-Hexane         1-Heptane
N-Heptane         Ethylbenzene     N-Octane       Chlorine         Fluorine
Hydrogen          Steam            Helium         Ammonia          Nitric Oxide
Nitrogen Dioxide  Nitrogen         Nitrous        Oxide            Neon Oxygen
Sulfur Dioxide    Sulfur Trioxide
</pre>
</HTML>"));
        package Ar "Ideal gas \"Ar\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Argon", data = Common.SingleGasesData.Ar, fluidConstants = {Common.FluidData.Ar});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/Ar.png\"></HTML>"));
        end Ar;
        package CH4 "Ideal gas \"CH4\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Methane", data = Common.SingleGasesData.CH4, fluidConstants = {Common.FluidData.CH4});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/CH4.png\"></HTML>"));
        end CH4;
        package CH3OH "Ideal gas \"CH3OH\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Methanol", data = Common.SingleGasesData.CH3OH, fluidConstants = {Common.FluidData.CH3OH});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/CH3OH.png\"></HTML>"));
        end CH3OH;
        package CO "Ideal gas \"CO\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Carbon Monoxide", data = Common.SingleGasesData.CO, fluidConstants = {Common.FluidData.CO});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/CO.png\"></HTML>"));
        end CO;
        package CO2 "Ideal gas \"CO2\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Carbon Dioxide", data = Common.SingleGasesData.CO2, fluidConstants = {Common.FluidData.CO2});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/CO2.png\"></HTML>"));
        end CO2;
        package C2H2_vinylidene "Ideal gas \"C2H2_vinylidene\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Acetylene", data = Common.SingleGasesData.C2H2_vinylidene, fluidConstants = {Common.FluidData.C2H2_vinylidene});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/C2H2_vinylidene.png\"></HTML>"));
        end C2H2_vinylidene;
        package C2H4 "Ideal gas \"C2H4\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Ethylene", data = Common.SingleGasesData.C2H4, fluidConstants = {Common.FluidData.C2H4});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/C2H4.png\"></HTML>"));
        end C2H4;
        package C2H5OH "Ideal gas \"C2H5OH\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Ethanol", data = Common.SingleGasesData.C2H5OH, fluidConstants = {Common.FluidData.C2H5OH});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/C2H5OH.png\"></HTML>"));
        end C2H5OH;
        package C2H6 "Ideal gas \"C2H6\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Ethane", data = Common.SingleGasesData.C2H6, fluidConstants = {Common.FluidData.C2H6});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/C2H6.png\"></HTML>"));
        end C2H6;
        package C3H6_propylene "Ideal gas \"C3H6_propylene\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Propylene", data = Common.SingleGasesData.C3H6_propylene, fluidConstants = {Common.FluidData.C3H6_propylene});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/C3H6_propylene.png\"></HTML>"));
        end C3H6_propylene;
        package C3H8 "Ideal gas \"C3H8\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Propane", data = Common.SingleGasesData.C3H8, fluidConstants = {Common.FluidData.C3H8});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/C3H8.png\"></HTML>"));
        end C3H8;
        package C3H8O_1propanol "Ideal gas \"C3H8O_1propanol\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "1-Propanol", data = Common.SingleGasesData.C3H8O_1propanol, fluidConstants = {Common.FluidData.C3H7OH});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/C3H8O_1propanol.png\"></HTML>"));
        end C3H8O_1propanol;
        package C4H8_1_butene "Ideal gas \"C4H8_1_butene\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "1-Butene", data = Common.SingleGasesData.C4H8_1_butene, fluidConstants = {Common.FluidData.C4H8_1_butene});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/C4H8_1_butene.png\"></HTML>"));
        end C4H8_1_butene;
        package C4H10_n_butane "Ideal gas \"C4H10_n_butane\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "N-Butane", data = Common.SingleGasesData.C4H10_n_butane, fluidConstants = {Common.FluidData.C4H10_n_butane});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/C4H10_n_butane.png\"></HTML>"));
        end C4H10_n_butane;
        package C5H10_1_pentene "Ideal gas \"C5H10_1_pentene\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "1-Pentene", data = Common.SingleGasesData.C5H10_1_pentene, fluidConstants = {Common.FluidData.C5H10_1_pentene});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/C5H10_1_pentene.png\"></HTML>"));
        end C5H10_1_pentene;
        package C5H12_n_pentane "Ideal gas \"C5H12_n_pentane\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "N-Pentane", data = Common.SingleGasesData.C5H12_n_pentane, fluidConstants = {Common.FluidData.C5H12_n_pentane});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/C5H12_n_pentane.png\"></HTML>"));
        end C5H12_n_pentane;
        package C6H6 "Ideal gas \"C6H6\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Benzene", data = Common.SingleGasesData.C6H6, fluidConstants = {Common.FluidData.C6H6});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/C6H6.png\"></HTML>"));
        end C6H6;
        package C6H12_1_hexene "Ideal gas \"C6H12_1_hexene\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "1-Hexene", data = Common.SingleGasesData.C6H12_1_hexene, fluidConstants = {Common.FluidData.C6H12_1_hexene});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/C6H12_1_hexene.png\"></HTML>"));
        end C6H12_1_hexene;
        package C6H14_n_hexane "Ideal gas \"C6H14_n_hexane\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "N-Hexane", data = Common.SingleGasesData.C6H14_n_hexane, fluidConstants = {Common.FluidData.C6H14_n_hexane});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/C6H14_n_hexane.png\"></HTML>"));
        end C6H14_n_hexane;
        package C7H14_1_heptene "Ideal gas \"C7H14_1_heptene\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "1-Heptene", data = Common.SingleGasesData.C7H14_1_heptene, fluidConstants = {Common.FluidData.C7H14_1_heptene});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/C7H14_1_heptene.png\"></HTML>"));
        end C7H14_1_heptene;
        package C7H16_n_heptane "Ideal gas \"C7H16_n_heptane\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "N-Heptane", data = Common.SingleGasesData.C7H16_n_heptane, fluidConstants = {Common.FluidData.C7H16_n_heptane});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/C7H16_n_heptane.png\"></HTML>"));
        end C7H16_n_heptane;
        package C8H10_ethylbenz "Ideal gas \"C8H10_ethylbenz\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Ethylbenzene", data = Common.SingleGasesData.C8H10_ethylbenz, fluidConstants = {Common.FluidData.C8H10_ethylbenz});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/C8H10_ethylbenz.png\"></HTML>"));
        end C8H10_ethylbenz;
        package C8H18_n_octane "Ideal gas \"C8H18_n_octane\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "N-Octane", data = Common.SingleGasesData.C8H18_n_octane, fluidConstants = {Common.FluidData.C8H18_n_octane});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/C8H18_n_octane.png\"></HTML>"));
        end C8H18_n_octane;
        package CL2 "Ideal gas \"Cl2\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Chlorine", data = Common.SingleGasesData.CL2, fluidConstants = {Common.FluidData.CL2});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/CL2.png\"></HTML>"));
        end CL2;
        package F2 "Ideal gas \"F2\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Fluorine", data = Common.SingleGasesData.F2, fluidConstants = {Common.FluidData.F2});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/F2.png\"></HTML>"));
        end F2;
        package H2 "Ideal gas \"H2\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Hydrogen", data = Common.SingleGasesData.H2, fluidConstants = {Common.FluidData.H2});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/H2.png\"></HTML>"));
        end H2;
        package H2O "Ideal gas \"H2O\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "IdealGasSteam", data = Common.SingleGasesData.H2O, fluidConstants = {Common.FluidData.H2O});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/H2O.png\"></HTML>"));
        end H2O;
        package He "Ideal gas \"He\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Helium", data = Common.SingleGasesData.He, fluidConstants = {Common.FluidData.He});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/He.png\"></HTML>"));
        end He;
        package NH3 "Ideal gas \"NH3\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "IdealGasAmmonia", data = Common.SingleGasesData.NH3, fluidConstants = {Common.FluidData.NH3});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/NH3.png\"></HTML>"));
        end NH3;
        package NO "Ideal gas \"NO\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Nitric Oxide", data = Common.SingleGasesData.NO, fluidConstants = {Common.FluidData.NO});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/NO.png\"></HTML>"));
        end NO;
        package NO2 "Ideal gas \"NO2\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Nitrogen Dioxide", data = Common.SingleGasesData.NO2, fluidConstants = {Common.FluidData.NO2});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/NO2.png\"></HTML>"));
        end NO2;
        package N2 "Ideal gas \"N2\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Nitrogen", data = Common.SingleGasesData.N2, fluidConstants = {Common.FluidData.N2});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/N2.png\"></HTML>"));
        end N2;
        package N2O "Ideal gas \"N2O\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Nitrous Oxide", data = Common.SingleGasesData.N2O, fluidConstants = {Common.FluidData.N2O});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/N2O.png\"></HTML>"));
        end N2O;
        package Ne "Ideal gas \"Ne\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Neon", data = Common.SingleGasesData.Ne, fluidConstants = {Common.FluidData.Ne});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/Ne.png\"></HTML>"));
        end Ne;
        package O2 "Ideal gas \"O2\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Oxygen", data = Common.SingleGasesData.O2, fluidConstants = {Common.FluidData.O2});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/O2.png\"></HTML>"));
        end O2;
        package SO2 "Ideal gas \"SO2\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Sulfur Dioxide", data = Common.SingleGasesData.SO2, fluidConstants = {Common.FluidData.SO2});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/SO2.png\"></HTML>"));
        end SO2;
        package SO3 "Ideal gas \"SO3\" from NASA Glenn coefficients"
          extends Common.SingleGasNasa(mediumName = "Sulfur Trioxide", data = Common.SingleGasesData.SO3, fluidConstants = {Common.FluidData.SO3});
          annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/SO3.png\"></HTML>"));
        end SO3;
      end SingleGases;
      package MixtureGases "Medium models consisting of mixtures of ideal gases"
        extends Modelica.Icons.Library;
        package CombustionAir "Air as mixture of N2 and O2"
          extends Common.MixtureGasNasa(mediumName = "CombustionAirN2O2", data = {Common.SingleGasesData.N2,Common.SingleGasesData.O2}, fluidConstants = {Common.FluidData.N2,Common.FluidData.O2}, substanceNames = {"Nitrogen","Oxygen"}, reference_X = {0.768,0.232});
          annotation(Documentation(info = "<html>

</html>"));
        end CombustionAir;
        package AirSteam "air and steam mixture (no condensation!, pseudo-mixture)"
          extends Common.MixtureGasNasa(mediumName = "MoistAir", data = {Common.SingleGasesData.H2O,Common.SingleGasesData.Air}, fluidConstants = {Common.FluidData.H2O,Common.FluidData.N2}, substanceNames = {"Water","Air"}, reference_X = {0.0,1.0});
          annotation(Documentation(info = "<html>

</html>"));
        end AirSteam;
        package simpleMoistAir = AirSteam(reference_X = {0.03,0.97}) "moist air without condensation";
        package FlueGasLambdaOnePlus "simple flue gas for over0stochiometric O2-fuel ratios"
          extends Common.MixtureGasNasa(mediumName = "FlueGasLambda1plus", data = {Common.SingleGasesData.N2,Common.SingleGasesData.O2,Common.SingleGasesData.H2O,Common.SingleGasesData.CO2}, fluidConstants = {Common.FluidData.N2,Common.FluidData.O2,Common.FluidData.H2O,Common.FluidData.CO2}, substanceNames = {"Nitrogen","Oxygen","Water","Carbondioxide"}, reference_X = {0.768,0.232,0.0,0.0});
          annotation(Documentation(info = "<html>

</html>"));
        end FlueGasLambdaOnePlus;
        package FlueGasSixComponents "simplest flue gas for over-and understochiometric combustion of hydrocarbons"
          extends Common.MixtureGasNasa(mediumName = "FlueGasSixComponents", data = {Common.SingleGasesData.N2,Common.SingleGasesData.H2,Common.SingleGasesData.CO,Common.SingleGasesData.O2,Common.SingleGasesData.H2O,Common.SingleGasesData.CO2}, fluidConstants = {Common.FluidData.N2,Common.FluidData.H2,Common.FluidData.CO,Common.FluidData.O2,Common.FluidData.H2O,Common.FluidData.CO2}, substanceNames = {"Nitrogen","Hydrogen,","Carbonmonoxide","Oxygen","Water","Carbondioxide"}, reference_X = {0.768,0.0,0.0,0.232,0.0,0.0});
          annotation(Documentation(info = "<html>

</html>"));
        end FlueGasSixComponents;
        package SimpleNaturalGas "simple natural gas mix with 6 components"
          extends Common.MixtureGasNasa(mediumName = "SimpleNaturalGas", data = {Common.SingleGasesData.CH4,Common.SingleGasesData.C2H6,Common.SingleGasesData.C3H8,Common.SingleGasesData.C4H10_n_butane,Common.SingleGasesData.N2,Common.SingleGasesData.CO2}, fluidConstants = {Common.FluidData.CH4,Common.FluidData.C2H6,Common.FluidData.C3H8,Common.FluidData.C4H10_n_butane,Common.FluidData.N2,Common.FluidData.CO2}, substanceNames = {"Methane","Ethane","Propane","N-Butane,","Nitrogen","Carbondioxide"}, reference_X = {0.92,0.048,0.005,0.002,0.015,0.01});
          annotation(Documentation(info = "<html>

</html>"));
        end SimpleNaturalGas;
        package SimpleNaturalGasFixedComposition "Same as SimpleNaturalGas but with fixed composition"
          extends SimpleNaturalGas(fixedX = true);
          annotation(Documentation(info = "<html>

</html>"));
        end SimpleNaturalGasFixedComposition;
        annotation(Documentation(info = "<html>

</html>"));
      end MixtureGases;
    end IdealGases;
    package Incompressible "Medium model for T-dependent properties, defined by tables or polynomials"
      import SI = Modelica.SIunits;
      import Cv = Modelica.SIunits.Conversions;
      import Modelica.Constants;
      import Modelica.Math;
      package Common "Common data structures"
        record BaseProps_Tpoly "Fluid state record"
          extends Modelica.Icons.Record;
          annotation(Documentation(info = "<html></html>"));
          SI.Temperature T "temperature";
          SI.Pressure p "pressure";
        end BaseProps_Tpoly;
      end Common;
      package TableBased "Incompressible medium properties based on tables"
        import Poly = Modelica.Media.Incompressible.TableBased.Polynomials_Temp;
        extends Modelica.Media.Interfaces.PartialMedium(ThermoStates = if enthalpyOfT then Choices.IndependentVariables.T else Choices.IndependentVariables.pT, final reducedX = true, final fixedX = true, mediumName = "tableMedium", redeclare record ThermodynamicState = Common.BaseProps_Tpoly, singleState = true);
        constant Boolean enthalpyOfT = true "true if enthalpy is approximated as a function of T only, (p-dependence neglected)";
        constant Boolean densityOfT = size(tableDensity, 1) > 1 "true if density is a function of temperature";
        constant Temperature T_min "Minimum temperature valid for medium model";
        constant Temperature T_max "Maximum temperature valid for medium model";
        constant Temperature T0 = 273.15 "reference Temperature";
        constant SpecificEnthalpy h0 = 0 "reference enthalpy at T0, reference_p";
        constant SpecificEntropy s0 = 0 "reference entropy at T0, reference_p";
        constant MolarMass MM_const = 0.1 "Molar mass";
        constant Integer npol = 2 "degree of polynomial used for fitting";
        constant Integer neta = size(tableViscosity, 1) "number of data points for viscosity";
        constant Real[:,:] tableDensity "Table for rho(T)";
        constant Real[:,:] tableHeatCapacity "Table for Cp(T)";
        constant Real[:,:] tableViscosity "Table for eta(T)";
        constant Real[:,:] tableVaporPressure "Table for pVap(T)";
        constant Real[:,:] tableConductivity "Table for lambda(T)";
        constant Boolean TinK "true if T[K],Kelvin used for table temperatures";
        constant Boolean hasDensity = not size(tableDensity, 1) == 0 "true if table tableDensity is present";
        constant Boolean hasHeatCapacity = not size(tableHeatCapacity, 1) == 0 "true if table tableHeatCapacity is present";
        constant Boolean hasViscosity = not size(tableViscosity, 1) == 0 "true if table tableViscosity is present";
        constant Boolean hasVaporPressure = not size(tableVaporPressure, 1) == 0 "true if table tableVaporPressure is present";
        final constant Real invTK[neta] = if size(tableViscosity, 1) > 0 then invertTemp(tableViscosity[:,1], TinK) else fill(0, 0);
        annotation(__Dymola_keepConstant = true, Documentation(info = "<HTML>
<p>
This is the base package for medium models of incompressible fluids based on
tables. The minimal data to provide for a useful medium description is tables
of density and heat capacity as functions of temperature.
</p>

<p>It should be noted that incompressible media only have 1 state per control volume (usually T),
but have both T and p as inputs for fully correct properties. The error of using only T-dependent
properties is small, therefore a Boolean flag enthalpyOfT exists. If it is true, the
enumeration Choices.independentVariables  is set to  Choices.independentVariables.T otherwise
it is set to Choices.independentVariables.pT. </p>

<h4>Using the package TableBased</h4>
<p>
To implement a new medium model, create a package that <b>extends</b> TableBased
and provides one or more of the constant tables:
</p>

<pre>
tableDensity        = [T, d];
tableHeatCapacity   = [T, Cp];
tableConductivity   = [T, lam];
tableViscosity      = [T, eta];
tableVaporPressure  = [T, pVap];
</pre>

<p>
The table data is used to fit constant polynomials of order <b>npol</b>, the
temperature data points do not need to be same for different properties. Properties
like enthalpy, inner energy and entropy are calculated consistently from integrals
and derivatives of d(T) and Cp(T). The minimal
data for a useful medium model is thus density and heat capacity. Transport
properties and vapor pressure are optional, if the data tables are empty the corresponding
function calls can not be used.
</p>
</HTML>"));
        final constant Real poly_rho[:] = if hasDensity then Poly.fitting(tableDensity[:,1], tableDensity[:,2], npol) else zeros(npol + 1) annotation(__Dymola_keepConstant = true);
        final constant Real poly_Cp[:] = if hasHeatCapacity then Poly.fitting(tableHeatCapacity[:,1], tableHeatCapacity[:,2], npol) else zeros(npol + 1) annotation(__Dymola_keepConstant = true);
        final constant Real poly_eta[:] = if hasViscosity then Poly.fitting(invTK, Math.log(tableViscosity[:,2]), npol) else zeros(npol + 1) annotation(__Dymola_keepConstant = true);
        final constant Real poly_pVap[:] = if hasVaporPressure then Poly.fitting(tableVaporPressure[:,1], tableVaporPressure[:,2], npol) else zeros(npol + 1) annotation(__Dymola_keepConstant = true);
        final constant Real poly_lam[:] = if size(tableConductivity, 1) > 0 then Poly.fitting(tableConductivity[:,1], tableConductivity[:,2], npol) else zeros(npol + 1) annotation(__Dymola_keepConstant = true);
        function invertTemp "function to invert temperatures"
          input Real[:] table "table temperature data";
          input Boolean Tink "flag for Celsius or Kelvin";
          output Real invTable[size(table, 1)] "inverted temperatures";
        algorithm 
          for i in 1:size(table, 1) loop
                      invTable[i]:=if TinK then 1 / table[i] else 1 / Cv.from_degC(table[i]);

          end for;
        end invertTemp;
        model extends BaseProperties(final standardOrderComponents = true, p_bar = Cv.to_bar(p), T_degC(start = T_start - 273.15) = Cv.to_degC(T), T(start = T_start, stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default)) "Base properties of T dependent medium"
          annotation(Documentation(info = "<html>
<p>
Note that the inner energy neglects the pressure dependence, which is only
true for an incompressible medium with d = constant. The neglected term is
p-reference_p)/rho*(T/rho)*(partial rho /partial T). This is very small for
liquids due to proportionality to 1/d^2, but can be problematic for gases that are
modeled incompressible.
</p>
<p>It should be noted that incompressible media only have 1 state per control volume (usually T),
but have both T and p as inputs for fully correct properties. The error of using only T-dependent
properties is small, therefore a Boolean flag enthalpyOfT exists. If it is true, the
enumeration Choices.independentVariables  is set to  Choices.independentVariables.T otherwise
it is set to Choices.independentVariables.pT. </p>
<p>
Enthalpy is never a function of T only (h = h(T) + (p-reference_p)/d), but the
error is also small and non-linear systems can be avoided. In particular,
non-linear systems are small and local as opposed to large and over all volumes.
</p>

<p>
Entropy is calculated as
</p>
<pre>
  s = s0 + integral(Cp(T)/T,dt)
</pre>
<p>
which is only exactly true for a fluid with constant density d=d0.
</p>
</html>
      "));
          SI.SpecificHeatCapacity cp "specific heat capacity";
          parameter SI.Temperature T_start = 298.15 "initial temperature";

        equation 
          assert(hasDensity, "Medium " + mediumName + " can not be used without assigning tableDensity.");
          assert(T >= T_min and T <= T_max, "Temperature T (= " + String(T) + " K) is not in the allowed range (" + String(T_min) + " K <= T <= " + String(T_max) + " K) required from medium model \"" + mediumName + "\".");
          R = Modelica.Constants.R;
          cp = Poly.evaluate(poly_Cp, if TinK then T else T_degC);
          h = if enthalpyOfT then h_T(T) else h_pT(p, T, densityOfT);
          if singleState then
            u = h_T(T) - reference_p / d;

          else
            u = h - p / d;

          end if;
          d = Poly.evaluate(poly_rho, if TinK then T else T_degC);
          state.T = T;
          state.p = p;
          MM = MM_const;
        end BaseProperties;
        function extends setState_pTX "Returns state record, given pressure and temperature"
        algorithm 
          state:=ThermodynamicState(p = p, T = T);
        end setState_pTX;
        function extends setState_dTX "Returns state record, given pressure and temperature"
        algorithm 
          assert(false, "for incompressible media with d(T) only, state can not be set from density and temperature");
        end setState_dTX;
        function setState_pT "returns state record as function of p and T"
          input AbsolutePressure p "pressure";
          input Temperature T "temperature";
          output ThermodynamicState state "thermodynamic state";
        algorithm 
          state.T:=T;
          state.p:=p;
        end setState_pT;
        function extends setState_phX "Returns state record, given pressure and specific enthalpy"
        algorithm 
          state:=ThermodynamicState(p = p, T = T_ph(p, h));
        end setState_phX;
        function setState_ph "returns state record as function of p and h"
          input AbsolutePressure p "pressure";
          input SpecificEnthalpy h "specific enthalpy";
          output ThermodynamicState state "thermodynamic state";
        algorithm 
          state:=ThermodynamicState(p = p, T = T_ph(p, h));
        end setState_ph;
        function extends setState_psX "Returns state record, given pressure and specific entropy"
        algorithm 
          state:=ThermodynamicState(p = p, T = T_ps(p, s));
        end setState_psX;
        function setState_ps "returns state record as function of p and s"
          input AbsolutePressure p "pressure";
          input SpecificEntropy s "specific entropy";
          output ThermodynamicState state "thermodynamic state";
        algorithm 
          state:=ThermodynamicState(p = p, T = T_ps(p, s));
        end setState_ps;
        function extends setSmoothState "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
        algorithm 
          state:=ThermodynamicState(p = Media.Common.smoothStep(x, state_a.p, state_b.p, x_small), T = Media.Common.smoothStep(x, state_a.T, state_b.T, x_small));
        end setSmoothState;
        function extends specificHeatCapacityCv "Specific heat capacity at constant volume (or pressure) of medium"
          annotation(smoothOrder = 2);
        algorithm 
          assert(hasHeatCapacity, "Specific Heat Capacity, Cv, is not defined for medium " + mediumName + ".");
          cv:=Poly.evaluate(poly_Cp, if TinK then state.T else state.T - 273.15);
        end specificHeatCapacityCv;
        function extends specificHeatCapacityCp "Specific heat capacity at constant volume (or pressure) of medium"
          annotation(smoothOrder = 2);
        algorithm 
          assert(hasHeatCapacity, "Specific Heat Capacity, Cv, is not defined for medium " + mediumName + ".");
          cp:=Poly.evaluate(poly_Cp, if TinK then state.T else state.T - 273.15);
        end specificHeatCapacityCp;
        function extends dynamicViscosity "Return dynamic viscosity as a function of the thermodynamic state record"
          annotation(smoothOrder = 2);
        algorithm 
          assert(size(tableViscosity, 1) > 0, "DynamicViscosity, eta, is not defined for medium " + mediumName + ".");
          eta:=Math.exp(Poly.evaluate(poly_eta, 1 / state.T));
        end dynamicViscosity;
        function extends thermalConductivity "Return thermal conductivity as a function of the thermodynamic state record"
          annotation(smoothOrder = 2);
        algorithm 
          assert(size(tableConductivity, 1) > 0, "ThermalConductivity, lambda, is not defined for medium " + mediumName + ".");
          lambda:=Poly.evaluate(poly_lam, if TinK then state.T else Cv.to_degC(state.T));
        end thermalConductivity;
        function s_T "compute specific entropy"
          input Temperature T "temperature";
          output SpecificEntropy s "specific entropy";
          annotation(smoothOrder = 2);
        algorithm 
          s:=s0 + (if TinK then Poly.integralValue(poly_Cp[1:npol], T, T0) else Poly.integralValue(poly_Cp[1:npol], Cv.to_degC(T), Cv.to_degC(T0))) + Modelica.Math.log(T / T0) * Poly.evaluate(poly_Cp, if TinK then 0 else Modelica.Constants.T_zero);
        end s_T;
        function extends specificEntropy "Return specific entropy
 as a function of the thermodynamic state record"
        protected 
          Integer npol = size(poly_Cp, 1) - 1;
          annotation(smoothOrder = 2);
        algorithm 
          assert(hasHeatCapacity, "Specific Entropy, s(T), is not defined for medium " + mediumName + ".");
          s:=s_T(state.T);
        end specificEntropy;
        function h_T "Compute specific enthalpy from temperature"
          import Modelica.SIunits.Conversions.to_degC;
          extends Modelica.Icons.Function;
          input SI.Temperature T "Temperature";
          output SI.SpecificEnthalpy h "Specific enthalpy at p, T";
          annotation(derivative = h_T_der);
        algorithm 
          h:=h0 + Poly.integralValue(poly_Cp, if TinK then T else Cv.to_degC(T), if TinK then T0 else Cv.to_degC(T0));
        end h_T;
        function h_T_der "Compute specific enthalpy from temperature"
          import Modelica.SIunits.Conversions.to_degC;
          extends Modelica.Icons.Function;
          input SI.Temperature T "Temperature";
          input Real dT "temperature derivative";
          output Real dh "derivative of Specific enthalpy at T";
          annotation(smoothOrder = 1);
        algorithm 
          dh:=Poly.evaluate(poly_Cp, if TinK then T else Cv.to_degC(T)) * dT;
        end h_T_der;
        function h_pT "Compute specific enthalpy from pressure and temperature"
          import Modelica.SIunits.Conversions.to_degC;
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input Boolean densityOfT = false "include or neglect density derivative dependence of enthalpy" annotation(Evaluate);
          output SI.SpecificEnthalpy h "Specific enthalpy at p, T";
          annotation(smoothOrder = 2);
        algorithm 
          h:=h0 + Poly.integralValue(poly_Cp, if TinK then T else Cv.to_degC(T), if TinK then T0 else Cv.to_degC(T0)) + (p - reference_p) / Poly.evaluate(poly_rho, if TinK then T else Cv.to_degC(T)) * (if densityOfT then 1 + T / Poly.evaluate(poly_rho, if TinK then T else Cv.to_degC(T)) * Poly.derivativeValue(poly_rho, if TinK then T else Cv.to_degC(T)) else 1.0);
        end h_pT;
        function density_T "Return density as function of temperature"
          input Temperature T "temperature";
          output Density d "density";
          annotation(smoothOrder = 2);
        algorithm 
          d:=Poly.evaluate(poly_rho, if TinK then T else Cv.to_degC(T));
        end density_T;
        function extends temperature "Return temperature as a function of the thermodynamic state record"
          annotation(smoothOrder = 2);
        algorithm 
          T:=state.T;
        end temperature;
        function extends pressure "Return pressure as a function of the thermodynamic state record"
          annotation(smoothOrder = 2);
        algorithm 
          p:=state.p;
        end pressure;
        function extends density "Return density as a function of the thermodynamic state record"
          annotation(smoothOrder = 2);
        algorithm 
          d:=Poly.evaluate(poly_rho, if TinK then state.T else Cv.to_degC(state.T));
        end density;
        function extends specificEnthalpy "Return specific enthalpy as a function of the thermodynamic state record"
          annotation(smoothOrder = 2);
        algorithm 
          h:=if enthalpyOfT then h_T(state.T) else h_pT(state.p, state.T);
        end specificEnthalpy;
        function extends specificInternalEnergy "Return specific internal energy as a function of the thermodynamic state record"
          annotation(smoothOrder = 2);
        algorithm 
          u:=if enthalpyOfT then h_T(state.T) else h_pT(state.p, state.T) - (if singleState then reference_p / density(state) else state.p / density(state));
        end specificInternalEnergy;
        function T_ph "Compute temperature from pressure and specific enthalpy"
          input AbsolutePressure p "pressure";
          input SpecificEnthalpy h "specific enthalpy";
          output Temperature T "temperature";
          annotation(Inline = false, LateInline = true, inverse = h_pT(p, T));
        protected 
          package Internal "Solve h(T) for T with given h (use only indirectly via temperature_phX)"
            extends Modelica.Media.Common.OneNonLinearEquation;
            record extends f_nonlinear_Data "superfluous record, fix later when better structure of inverse functions exists"
              constant Real[5] dummy = {1,2,3,4,5};
            end f_nonlinear_Data;
            function extends f_nonlinear "p is smuggled in via vector"
            algorithm 
              y:=if singleState then h_T(x) else h_pT(p, x);
            end f_nonlinear;
            function extends solve
            end solve;
          end Internal;
        algorithm 
          T:=Internal.solve(h, T_min, T_max, p, {1}, Internal.f_nonlinear_Data());
        end T_ph;
        function T_ps "Compute temperature from pressure and specific enthalpy"
          input AbsolutePressure p "pressure";
          input SpecificEntropy s "specific entropy";
          output Temperature T "temperature";
        protected 
          package Internal "Solve h(T) for T with given h (use only indirectly via temperature_phX)"
            extends Modelica.Media.Common.OneNonLinearEquation;
            record extends f_nonlinear_Data "superfluous record, fix later when better structure of inverse functions exists"
              constant Real[5] dummy = {1,2,3,4,5};
            end f_nonlinear_Data;
            function extends f_nonlinear "p is smuggled in via vector"
            algorithm 
              y:=s_T(x);
            end f_nonlinear;
            function extends solve
            end solve;
          end Internal;
        algorithm 
          T:=Internal.solve(s, T_min, T_max, p, {1}, Internal.f_nonlinear_Data());
        end T_ps;
        package Polynomials_Temp "Temporary Functions operating on polynomials (including polynomial fitting); only to be used in Modelica.Media.Incompressible.TableBased"
          extends Modelica.Icons.Library;
          annotation(Documentation(info = "<HTML>
<p>
This package contains functions to operate on polynomials,
in particular to determine the derivative and the integral
of a polynomial and to use a polynomial to fit a given set
of data points.
</p>
<p>

<p><b>Copyright &copy; 2004-2009, Modelica Association and DLR.</b></p>

<p><i>
This package is <b>free</b> software. It can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package
Modelica in file \"Modelica/package.mo\".
</i>
</p>

</HTML>
", revisions = "<html>
<ul>
<li><i>Oct. 22, 2004</i> by Martin Otter (DLR):<br>
       Renamed functions to not have abbrevations.<br>
       Based fitting on LAPACK<br>
       New function to return the polynomial of an indefinite integral<li>
<li><i>Sept. 3, 2004</i> by Jonas Eborn (Scynamics):<br>
       polyderval, polyintval added<li>
<li><i>March 1, 2004</i> by Martin Otter (DLR):<br>
       first version implemented
</li>
</ul>
</html>"));
          function evaluate "Evaluate polynomial at a given abszissa value"
            extends Modelica.Icons.Function;
            input Real p[:] "Polynomial coefficients (p[1] is coefficient of highest power)";
            input Real u "Abszissa value";
            output Real y "Value of polynomial at u";
            annotation(derivative(zeroDerivative = p) = evaluate_der);
          algorithm 
            y:=p[1];
            for j in 2:size(p, 1) loop
                          y:=p[j] + u * y;

            end for;
          end evaluate;
          function derivative "Derivative of polynomial"
            extends Modelica.Icons.Function;
            input Real p1[:] "Polynomial coefficients (p1[1] is coefficient of highest power)";
            output Real p2[size(p1, 1) - 1] "Derivative of polynomial p1";
          protected 
            Integer n = size(p1, 1);
          algorithm 
            for j in 1:n - 1 loop
                          p2[j]:=p1[j] * (n - j);

            end for;
          end derivative;
          function derivativeValue "Value of derivative of polynomial at abszissa value u"
            extends Modelica.Icons.Function;
            input Real p[:] "Polynomial coefficients (p[1] is coefficient of highest power)";
            input Real u "Abszissa value";
            output Real y "Value of derivative of polynomial at u";
            annotation(derivative(zeroDerivative = p) = derivativeValue_der);
          protected 
            Integer n = size(p, 1);
          algorithm 
            y:=p[1] * (n - 1);
            for j in 2:size(p, 1) - 1 loop
                          y:=p[j] * (n - j) + u * y;

            end for;
          end derivativeValue;
          function secondDerivativeValue "Value of 2nd derivative of polynomial at abszissa value u"
            extends Modelica.Icons.Function;
            input Real p[:] "Polynomial coefficients (p[1] is coefficient of highest power)";
            input Real u "Abszissa value";
            output Real y "Value of 2nd derivative of polynomial at u";
          protected 
            Integer n = size(p, 1);
          algorithm 
            y:=p[1] * (n - 1) * (n - 2);
            for j in 2:size(p, 1) - 2 loop
                          y:=p[j] * (n - j) * (n - j - 1) + u * y;

            end for;
          end secondDerivativeValue;
          function integral "Indefinite integral of polynomial p(u)"
            extends Modelica.Icons.Function;
            input Real p1[:] "Polynomial coefficients (p1[1] is coefficient of highest power)";
            output Real p2[size(p1, 1) + 1] "Polynomial coefficients of indefinite integral of polynomial p1 (polynomial p2 + C is the indefinite integral of p1, where C is an arbitrary constant)";
          protected 
            Integer n = size(p1, 1) + 1 "degree of output polynomial";
          algorithm 
            for j in 1:n - 1 loop
                          p2[j]:=p1[j] / (n - j);

            end for;
          end integral;
          function integralValue "Integral of polynomial p(u) from u_low to u_high"
            extends Modelica.Icons.Function;
            input Real p[:] "Polynomial coefficients";
            input Real u_high "High integrand value";
            input Real u_low = 0 "Low integrand value, default 0";
            output Real integral = 0.0 "Integral of polynomial p from u_low to u_high";
            annotation(derivative(zeroDerivative = p) = integralValue_der);
          protected 
            Integer n = size(p, 1) "degree of integrated polynomial";
            Real y_low = 0 "value at lower integrand";
          algorithm 
            for j in 1:n loop
                          integral:=u_high * (p[j] / (n - j + 1) + integral);
              y_low:=u_low * (p[j] / (n - j + 1) + y_low);

            end for;
            integral:=integral - y_low;
          end integralValue;
          function fitting "Computes the coefficients of a polynomial that fits a set of data points in a least-squares sense"
            extends Modelica.Icons.Function;
            input Real u[:] "Abscissa data values";
            input Real y[size(u, 1)] "Ordinate data values";
            input Integer n(min = 1) "Order of desired polynomial that fits the data points (u,y)";
            output Real p[n + 1] "Polynomial coefficients of polynomial that fits the date points";
            annotation(Documentation(info = "<HTML>
<p>
Polynomials.fitting(u,y,n) computes the coefficients of a polynomial
p(u) of degree \"n\" that fits the data \"p(u[i]) - y[i]\"
in a least squares sense. The polynomial is
returned as a vector p[n+1] that has the following definition:
</p>
<pre>
  p(u) = p[1]*u^n + p[2]*u^(n-1) + ... + p[n]*u + p[n+1];
</pre>
</HTML>"));
          protected 
            Real V[size(u, 1),n + 1] "Vandermonde matrix";
          algorithm 
            V[:,n + 1]:=ones(size(u, 1));
            for j in n: -1:1 loop
                          V[:,j]:=array(u[i] * V[i,j + 1] for i in 1:size(u, 1));

            end for;
            p:=Modelica.Math.Matrices.leastSquares(V, y);
          end fitting;
          function evaluate_der "Evaluate polynomial at a given abszissa value"
            extends Modelica.Icons.Function;
            input Real p[:] "Polynomial coefficients (p[1] is coefficient of highest power)";
            input Real u "Abszissa value";
            input Real du "Abszissa value";
            output Real dy "Value of polynomial at u";
          protected 
            Integer n = size(p, 1);
          algorithm 
            dy:=p[1] * (n - 1);
            for j in 2:size(p, 1) - 1 loop
                          dy:=p[j] * (n - j) + u * dy;

            end for;
            dy:=dy * du;
          end evaluate_der;
          function integralValue_der "Time derivative of integral of polynomial p(u) from u_low to u_high, assuming only u_high as time-dependent (Leibnitz rule)"
            extends Modelica.Icons.Function;
            input Real p[:] "Polynomial coefficients";
            input Real u_high "High integrand value";
            input Real u_low = 0 "Low integrand value, default 0";
            input Real du_high "High integrand value";
            input Real du_low = 0 "Low integrand value, default 0";
            output Real dintegral = 0.0 "Integral of polynomial p from u_low to u_high";
          algorithm 
            dintegral:=evaluate(p, u_high) * du_high;
          end integralValue_der;
          function derivativeValue_der "time derivative of derivative of polynomial"
            extends Modelica.Icons.Function;
            input Real p[:] "Polynomial coefficients (p[1] is coefficient of highest power)";
            input Real u "Abszissa value";
            input Real du "delta of abszissa value";
            output Real dy "time-derivative of derivative of polynomial w.r.t. input variable at u";
          protected 
            Integer n = size(p, 1);
          algorithm 
            dy:=secondDerivativeValue(p, u) * du;
          end derivativeValue_der;
        end Polynomials_Temp;
        annotation(Documentation(info = "<HTML>
<h4>Table based media</h4>
<p>
This is the base package for medium models of incompressible fluids based on
tables. The minimal data to provide for a useful medium description is tables
of density and heat capacity as functions of temperature.
</p>
<h4>Using the package TableBased</h4>
<p>
To implement a new medium model, create a package that <b>extends</b> TableBased
and provides one or more of the constant tables:
<pre>
tableDensity        = [T, d];
tableHeatCapacity   = [T, Cp];
tableConductivity   = [T, lam];
tableViscosity      = [T, eta];
tableVaporPressure  = [T, pVap];
</pre>
The table data is used to fit constant polynomials of order <b>npol</b>, the
temperature data points do not need to be same for different properties. Properties
like enthalpy, inner energy and entropy are calculated consistently from integrals
and derivatives of d(T) and Cp(T). The minimal
data for a useful medium model is thus density and heat capacity. Transport
properties and vapor pressure are optional, if the data tables are empty the corresponding
function calls can not be used.
</HTML>"));
      end TableBased;
      package Examples "Examples for incompressible media"
        package Glycol47 "1,2-Propylene glycol, 47% mixture with water"
          extends TableBased(mediumName = "Glycol-Water 47%", T_min = Cv.from_degC( -30), T_max = Cv.from_degC(100), TinK = false, T0 = 273.15, tableDensity = [ -30,1066; -20,1062; -10,1058;0,1054;20,1044;40,1030;60,1015;80,999;100,984], tableHeatCapacity = [ -30,3450; -20,3490; -10,3520;0,3560;20,3620;40,3690;60,3760;80,3820;100,3890], tableConductivity = [ -30,0.397; -20,0.396; -10,0.395;0,0.395;20,0.394;40,0.393;60,0.392;80,0.391;100,0.39], tableViscosity = [ -30,0.16; -20,0.0743; -10,0.0317;0,0.019;20,0.00626;40,0.00299;60,0.00162;80,0.0011;100,0.00081], tableVaporPressure = [0,500;20,1900.0;40,5300.0;60,16000.0;80,37000.0;100,80000.0]);
          annotation(Documentation(info = "<html>

</html>"));
        end Glycol47;
        package Essotherm650 "Essotherm thermal oil"
          extends TableBased(mediumName = "Essotherm 650", T_min = Cv.from_degC(0), T_max = Cv.from_degC(320), TinK = false, T0 = 273.15, tableDensity = [0,909;20,897;40,884;60,871;80,859;100,846;150,813;200,781;250,748;300,715;320,702], tableHeatCapacity = [0,1770;20,1850;40,1920;60,1990;80,2060;100,2130;150,2310;200,2490;250,2670;300,2850;320,2920], tableConductivity = [0,0.1302;20,0.1288;40,0.1274;60,0.126;80,0.1246;100,0.1232;150,0.1197;200,0.1163;250,0.1128;300,0.1093;320,0.1079], tableViscosity = [0,14370;20,1917;40,424;60,134;80,54.5;100,26.64;150,7.47;200,3.22;250,1.76;300,1.1;320,0.94], tableVaporPressure = [160,3;180,10;200,40;220,100;240,300;260,600;280,1600;300,3000.0;320,5500.0]);
          annotation(Documentation(info = "<html>

</html>"));
        end Essotherm650;
        model TestGlycol "Test Glycol47 Medium model"
          extends Modelica.Icons.Example;
          package Medium = Glycol47 "Medium model (Glycol47)";
          Medium.BaseProperties medium;
          Medium.DynamicViscosity eta = Medium.dynamicViscosity(medium.state);
          Medium.ThermalConductivity lambda = Medium.thermalConductivity(medium.state);
          Medium.SpecificEntropy s = Medium.specificEntropy(medium.state);
          Medium.SpecificHeatCapacity cv = Medium.specificHeatCapacityCv(medium.state);
        protected 
          constant Modelica.SIunits.Time timeUnit = 1;
          constant Modelica.SIunits.Temperature Ta = 1;

        equation 
          medium.p = 100000.0;
          medium.T = Medium.T_min + time / timeUnit * Ta;
        end TestGlycol;
        annotation(Documentation(info = "<HTML>

<p>
This package provides a few examples of how to construct medium models for
incompressible fluids. The package contains:
</p>
<ul>
<li><b>Glycol47</b>, a model of 47% glycol water mixture, based on tables of
density and heat capacity as functions of temperature.</li>
<li><b>Essotherm650</b>, a medium model for thermal oil, also based on tables.</li>
</ul>

</HTML>"));
      end Examples;
      annotation(Documentation(info = "<HTML>
<h4>Incompressible media package</h4>
<p>
This package provides a structure and examples of how to create simple
medium models of incompressible fluids, meaning fluids with very little
pressure influence on density. The medium properties is typically described
in terms of tables, functions or polynomial coefficients.
</p>
<h4>Definitions</h4>
<p>
The common meaning of <em>incompressible</em> is that properties like density
and enthalpy are independent of pressure. Thus properties are conveniently
described as functions of temperature, e.g. as polynomials density(T) and cp(T).
However, enthalpy can not be independent of pressure since h = u - p/d. For liquids
it is anyway
common to neglect this dependence since for constant density the neglected term
is (p - p0)/d, which in comparison with cp is very small for most liquids. For
water, the equivalent change of temperature to increasing pressure 1 bar is
0.025 Kelvin.
</p>
<p>
Two boolean flags are used to choose how enthalpy and inner energy is calculated:
<ul>
<li><b>enthalpyOfT</b>=true, means assuming that enthalpy is only a function
of temperature, neglecting the pressure dependent term.</li>
<li><b>singleState</b>=true, means also neglect the pressure influence on inner
energy, which makes all medium properties pure functions of temperature.</li>
</ul>
The default setting for both these flags is true, which enables the simulation tool
to choose temperature as the only medium state and avoids non-linear equation
systems, see the section about
<a href=\"Modelica://Modelica.Media.UsersGuide.MediumDefinition.StaticStateSelection\">Static
state selection</a> in the Modelica.Media User's Guide.

</p>
<h4>Contents</h4>
<p>
Currently, the package contains the following parts:
</p>
<ol>
<li> <a href=\"Modelica://Modelica.Media.Incompressible.TableBased\">
      Table based medium models</a></li>
<li> <a href=\"Modelica://Modelica.Media.Incompressible.Examples\">
      Example medium models</a></li>
</ol>

<p>
A few examples are given in the Examples package. The model
<a href=\"Modelica://Modelica.Media.Incompressible.Examples.Glycol47\">
Examples.Glycol47</a> shows how the medium models can be used. For more
realistic examples of how to implement volume models with medium properties
look in the <a href=\"Modelica://Modelica.Media.UsersGuide.MediumUsage\">Medium
usage section</a> of the User's Guide.
</p>

</HTML>"));
    end Incompressible;
    package CompressibleLiquids "compressible liquid models"
      package Common "base classes for compressible liquids"
        extends Icons.Library;
        partial package LinearWater_pT "base class for liquid, linear compressibility water models"
          extends Media.Interfaces.PartialLinearFluid(constantJacobian = true, reference_d = Modelica.Media.Water.StandardWater.density(state), reference_h = Modelica.Media.Water.StandardWater.specificEnthalpy(state), reference_s = Modelica.Media.Water.StandardWater.specificEntropy(state), cp_const = Modelica.Media.Water.StandardWater.specificHeatCapacityCp(state), beta_const = Modelica.Media.Water.StandardWater.isobaricExpansionCoefficient(state), kappa_const = Modelica.Media.Water.StandardWater.isothermalCompressibility(state), MM_const = Modelica.Media.Water.StandardWater.molarMass(state));
          constant Modelica.Media.Water.StandardWater.ThermodynamicState state = Modelica.Media.Water.StandardWater.setState_pT(reference_p, reference_T);
        end LinearWater_pT;
      end Common;
      package LinearColdWater "cold water model with linear compressibility"
        extends Media.Interfaces.PartialLinearFluid(constantJacobian = true, reference_p = 101325, reference_T = 278.15, reference_d = 997.05, reference_h = 104929, reference_s = 100.0, cp_const = 4181.9, beta_const = 0.00025713, kappa_const = 4.5154e-10, MM_const = 0.018015268);
        function extends dynamicViscosity "Dynamic viscosity of water"
        algorithm 
          eta:=0.0015;
        end dynamicViscosity;
        function extends thermalConductivity "Thermal conductivity of water"
        algorithm 
          lambda:=0.572;
        end thermalConductivity;
      end LinearColdWater;
      package LinearWater_pT_Ambient "liquid, linear compressibility water model at 1.01325 bar and 25 degree Celsius"
        extends Modelica.Media.CompressibleLiquids.Common.LinearWater_pT(reference_p = 101325, reference_T = 298.15);
        annotation(Documentation(info = "<html>
<h4>Water model with linear compressibility at ambient conditions</h4>
</html>"));
        function extends dynamicViscosity "Dynamic viscosity of water"
        algorithm 
          eta:=0.00089;
        end dynamicViscosity;
        function extends thermalConductivity "Thermal conductivity of water"
        algorithm 
          lambda:=0.608;
        end thermalConductivity;
      end LinearWater_pT_Ambient;
      annotation(Documentation(info = "<html>

<h4>Fluid models with linear compressibility, using PartialLinearFluid as base class.</h4>
<p>The linear compressibility fluid models contained in this package are based on the assumptions that:
</p>
<ul>
<li>The specific heat capacity at constant pressure (cp) is constant</li>
<li>The isobaric expansion coefficient (beta) is constant</li>
<li>The isothermal compressibility (kappa) is constant</li>
<li>Pressure and temperature are used as states</li>
</ul>
<p>
This results in models that are only valid for small temperature ranges, but sufficient to model compressibility and e.g. the \"water hammer\" effect. ANother advantage id that only 3 values need to be measured to have an initial model. Hydraulic fluids can often be approximated by this type of model.
</p>
<p>
That means that the density is a linear function in temperature and in pressure.
In order to define the complete model, a number of constant reference values are needed which
are computed at the reference values of the states pressure p and temperature T. The model can
be interprested as a linearization of a full non-linear fluid model (but it is not linear in all
thermodynamic coordinates). Reference values are needed for
</p>
<ol>
<li>the density (reference_d),</li>
<li>the specific enthalpy (reference_h),</li>
<li>the specific entropy (reference_s).</li>
</ol>
<p>
Apart from that, a user needs to define the molar mass, MM_const.
Note that it is possible to define a fluid by computing the reference
values from a full non-linear fluid model by computing the package constants
using the standard functions defined in a fluid package (see example in Common, LinearWater_pT).
</p>

</html>"));
    end CompressibleLiquids;
    package Air "Medium models for air"
      package SimpleAir "Air: Simple dry air model (0..100 degC)"
        extends Interfaces.PartialSimpleIdealGasMedium(mediumName = "SimpleAir", cp_const = 1005.45, MM_const = 0.0289651159, R_gas = Constants.R / 0.0289651159, eta_const = 1.82e-05, lambda_const = 0.026, T_min = Cv.from_degC(0), T_max = Cv.from_degC(100));
        import SI = Modelica.SIunits;
        import Cv = Modelica.SIunits.Conversions;
        import Modelica.Constants;
        constant FluidConstants[nS] fluidConstants = FluidConstants(iupacName = {"simple air"}, casRegistryNumber = {"not a real substance"}, chemicalFormula = {"N2, O2"}, structureFormula = {"N2, O2"}, molarMass = Modelica.Media.IdealGases.Common.SingleGasesData.N2.MM) "constant data for the fluid";
        annotation(Documentation(info = "<html>
                              <h4>Simple Ideal gas air model for low temperatures</h4>
                              <p>This model demonstrats how to use the PartialSimpleIdealGas base class to build a
                              simple ideal gas model with a limited temperature validity range.</p>
                              </html>"));
      end SimpleAir;
      package DryAirNasa "Air: Detailed dry air model as ideal gas (200..6000 K)"
        extends IdealGases.Common.SingleGasNasa(mediumName = "Air", data = IdealGases.Common.SingleGasesData.Air, fluidConstants = {IdealGases.Common.FluidData.N2});
        annotation(Documentation(info = "<HTML>
      <IMG SRC=\"../Images/Media/IdealGases/SingleGases/Air.png\"></HTML>"));
        import Cv = Modelica.SIunits.Conversions;
        function dynamicViscosity "Simple polynomial for dry air (moisture influence small), valid from 73.15 K to 373.15 K"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          output DynamicViscosity eta "Dynamic viscosity";
        algorithm 
          eta:=Incompressible.TableBased.Polynomials_Temp.evaluate({ -4.96717436974791e-11,5.06626785714286e-08,1.72937731092437e-05}, Cv.to_degC(state.T));
          annotation(Documentation(info = "<html>
Dynamic viscosity is computed from temperature using a second order polynomial with a range of validity between 73 and 373 K.
</html>"));
        end dynamicViscosity;
        function thermalConductivity "Simple polynomial for dry air (moisture influence small), valid from 73.15 K to 373.15 K"
          extends Modelica.Icons.Function;
          input ThermodynamicState state "Thermodynamic state record";
          input Integer method = 1 "Dummy for compatibility reasons";
          output ThermalConductivity lambda "Thermal conductivity";
        algorithm 
          lambda:=Incompressible.TableBased.Polynomials_Temp.evaluate({ -4.8737307422969e-08,7.67803133753502e-05,0.0241814385504202}, Cv.to_degC(state.T));
          annotation(Documentation(info = "<html>
Thermal conductivity is computed from temperature using a second order polynomial with a range of validity between 73 and 373 K.
</html>"));
        end thermalConductivity;
        annotation(Documentation(info = "<html>
Ideal gas medium model for dry air based on the package <a href=Modelica://Modelica.Media.IdealGases>IdealGases</a> with additional functions for dynamic viscosity and thermal conductivity in a limited temperature range.
</html>"));
      end DryAirNasa;
      package MoistAir "Air: Moist air model (240 ... 400 K)"
        extends Interfaces.PartialCondensingGases(mediumName = "Moist air", substanceNames = {"water","air"}, final reducedX = true, final singleState = false, reference_X = {0.01,0.99}, fluidConstants = {IdealGases.Common.FluidData.H2O,IdealGases.Common.FluidData.N2});
        constant Integer Water = 1 "Index of water (in substanceNames, massFractions X, etc.)";
        constant Integer Air = 2 "Index of air (in substanceNames, massFractions X, etc.)";
        constant Real k_mair = steam.MM / dryair.MM "ratio of molar weights";
        constant IdealGases.Common.DataRecord dryair = IdealGases.Common.SingleGasesData.Air;
        constant IdealGases.Common.DataRecord steam = IdealGases.Common.SingleGasesData.H2O;
        constant SI.MolarMass[2] MMX = {steam.MM,dryair.MM} "Molar masses of components";
        import Modelica.Media.Interfaces;
        import Modelica.Math;
        import SI = Modelica.SIunits;
        import Cv = Modelica.SIunits.Conversions;
        import Modelica.Constants;
        import Modelica.Media.IdealGases.Common.SingleGasNasa;
        record extends ThermodynamicState "ThermodynamicState record for moist air"
        end ThermodynamicState;
        redeclare replaceable model extends BaseProperties(T(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default), p(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default), Xi(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default), redeclare final constant Boolean standardOrderComponents = true) "Moist air base properties record"
          MassFraction x_water "Mass of total water/mass of dry air";
          Real phi "Relative humidity";
          annotation(Documentation(info = "<html>
This model computes thermodynamic properties of moist air from three independent (thermodynamic or/and numerical) state variables. Preferred numerical states are temperature T, pressure p and the reduced composition vector Xi, which contains the water mass fraction only. As an EOS the <b>ideal gas law</b> is used and associated restrictions apply. The model can also be used in the <b>fog region</b>, when moisture is present in its liquid state. However, it is assumed that the liquid water volume is negligible compared to that of the gas phase. Computation of thermal properties is based on property data of <a href=Modelica://Modelica.Media.Air.DryAirNasa> dry air</a> and water (source: VDI-W&auml;rmeatlas), respectively. Besides the standard thermodynamic variables <b>absolute and relative humidity</b>, x_water and phi, respectively, are given by the model. Upper case X denotes absolute humidity with respect to mass of moist air while absolute humidity with respect to mass of dry air only is denoted by a lower case x throughout the model. See <a href=Modelica://Modelica.Media.Air.MoistAir>package description</a> for further information.
</html>"));
        protected 
          MassFraction X_liquid "Mass fraction of liquid or solid water";
          MassFraction X_steam "Mass fraction of steam water";
          MassFraction X_air "Mass fraction of air";
          MassFraction X_sat "Steam water mass fraction of saturation boundary in kg_water/kg_moistair";
          MassFraction x_sat "Steam water mass content of saturation boundary in kg_water/kg_dryair";
          AbsolutePressure p_steam_sat "Partial saturation pressure of steam";

        equation 
          assert(T >= 200.0 and T <= 423.15, "
Temperature T is not in the allowed range
200.0 K <= (T =" + String(T) + " K) <= 423.15 K
required from medium model \"" + mediumName + "\".");
          MM = 1 / (Xi[Water] / MMX[Water] + (1.0 - Xi[Water]) / MMX[Air]);
          p_steam_sat = min(saturationPressure(T), 0.999 * p);
          X_sat = min((p_steam_sat * k_mair) / max(100 * Constants.eps, p - p_steam_sat) * (1 - Xi[Water]), 1.0) "Water content at saturation with respect to actual water content";
          X_liquid = max(Xi[Water] - X_sat, 0.0);
          X_steam = Xi[Water] - X_liquid;
          X_air = 1 - Xi[Water];
          h = specificEnthalpy_pTX(p, T, Xi);
          R = dryair.R * X_air / (1 - X_liquid) + (steam.R * X_steam) / (1 - X_liquid);
          u = h - R * T;
          d = p / (R * T);
          state.p = p;
          state.T = T;
          state.X = X;
          x_sat = (k_mair * p_steam_sat) / max(100 * Constants.eps, p - p_steam_sat);
          x_water = Xi[Water] / max(X_air, 100 * Constants.eps);
          phi = (p / p_steam_sat * Xi[Water]) / (Xi[Water] + k_mair * X_air);
        end BaseProperties;
        function setState_pTX "Return thermodynamic state as function of pressure p, temperature T and composition X"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "Thermodynamic state";
          annotation(smoothOrder = 2);
        algorithm 
          state:=if size(X, 1) == nX then ThermodynamicState(p = p, T = T, X = X) else ThermodynamicState(p = p, T = T, X = cat(1, X, {1 - sum(X)}));
          annotation(Documentation(info = "<html>
The <a href=Modelica://Modelica.Media.Air.MoistAir.ThermodynamicState>thermodynamic state record</a> is computed from pressure p, temperature T and composition X.
</html>"));
        end setState_pTX;
        function setState_phX "Return thermodynamic state as function of pressure p, specific enthalpy h and composition X"
          extends Modelica.Icons.Function;
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "Thermodynamic state";
          annotation(smoothOrder = 2);
        algorithm 
          state:=if size(X, 1) == nX then ThermodynamicState(p = p, T = T_phX(p, h, X), X = X) else ThermodynamicState(p = p, T = T_phX(p, h, X), X = cat(1, X, {1 - sum(X)}));
          annotation(Documentation(info = "<html>
The <a href=Modelica://Modelica.Media.Air.MoistAir.ThermodynamicState>thermodynamic state record</a> is computed from pressure p, specific enthalpy h and composition X.
</html>"));
        end setState_phX;
        function setState_dTX "Return thermodynamic state as function of density d, temperature T and composition X"
          extends Modelica.Icons.Function;
          input Density d "density";
          input Temperature T "Temperature";
          input MassFraction X[:] = reference_X "Mass fractions";
          output ThermodynamicState state "Thermodynamic state";
          annotation(smoothOrder = 2);
        algorithm 
          state:=if size(X, 1) == nX then ThermodynamicState(p = d * {steam.R,dryair.R} * X * T, T = T, X = X) else ThermodynamicState(p = d * {steam.R,dryair.R} * cat(1, X, {1 - sum(X)}) * T, T = T, X = cat(1, X, {1 - sum(X)}));
          annotation(Documentation(info = "<html>
The <a href=Modelica://Modelica.Media.Air.MoistAir.ThermodynamicState>thermodynamic state record</a> is computed from density d, temperature T and composition X.
</html>"));
        end setState_dTX;
        function extends setSmoothState "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
        algorithm 
          state:=ThermodynamicState(p = Media.Common.smoothStep(x, state_a.p, state_b.p, x_small), T = Media.Common.smoothStep(x, state_a.T, state_b.T, x_small), X = Media.Common.smoothStep(x, state_a.X, state_b.X, x_small));
        end setSmoothState;
        function Xsaturation "Return absolute humitity per unit mass of moist air at saturation as a function of the thermodynamic state record"
          input ThermodynamicState state "Thermodynamic state record";
          output MassFraction X_sat "Steam mass fraction of sat. boundary";
          annotation(smoothOrder = 2);
        algorithm 
          X_sat:=k_mair / (state.p / min(saturationPressure(state.T), 0.999 * state.p) - 1 + k_mair);
          annotation(Documentation(info = "<html>
Absolute humidity per unit mass of moist air at saturation is computed from pressure and temperature in the state record. Note, that unlike X_sat in the BaseProperties model this mass fraction refers to mass of moist air at saturation.
</html>"));
        end Xsaturation;
        function xsaturation "Return absolute humitity per unit mass of dry air at saturation as a function of the thermodynamic state record"
          input ThermodynamicState state "Thermodynamic state record";
          output MassFraction x_sat "Absolute humidity per unit mass of dry air";
          annotation(smoothOrder = 2);
        algorithm 
          x_sat:=(k_mair * saturationPressure(state.T)) / max(100 * Constants.eps, state.p - saturationPressure(state.T));
          annotation(Documentation(info = "<html>
Absolute humidity per unit mass of dry air at saturation is computed from pressure and temperature in the thermodynamic state record.
</html>"));
        end xsaturation;
        function xsaturation_pT "Return absolute humitity per unit mass of dry air at saturation as a function of pressure p and temperature T"
          input AbsolutePressure p "Pressure";
          input SI.Temperature T "Temperature";
          output MassFraction x_sat "Absolute humidity per unit mass of dry air";
          annotation(smoothOrder = 2);
        algorithm 
          x_sat:=(k_mair * saturationPressure(T)) / max(100 * Constants.eps, p - saturationPressure(T));
          annotation(Documentation(info = "<html>
Absolute humidity per unit mass of dry air at saturation is computed from pressure and temperature.
</html>"));
        end xsaturation_pT;
        function massFraction_pTphi "Return steam mass fraction as a function of relative humidity phi and temperature T"
          input AbsolutePressure p "Pressure";
          input Temperature T "Temperature";
          input Real phi "Relative humidity (0 ... 1.0)";
          output MassFraction X_steam "Absolute humidity, steam mass fraction";
          annotation(smoothOrder = 2);
        protected 
          constant Real k = 0.621964713077499 "Ratio of molar masses";
          AbsolutePressure psat = saturationPressure(T) "Saturation pressure";
        algorithm 
          X_steam:=(phi * k) / (k * phi + p / psat - phi);
          annotation(Documentation(info = "<html>
Absolute humidity per unit mass of moist air is computed from temperature, pressure and relative humidity.
</html>"));
        end massFraction_pTphi;
        function relativeHumidity_pTX "Return relative humidity as a function of pressure p, temperature T and composition X"
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input SI.MassFraction[:] X "Composition";
          output Real phi "Relative humidity";
        protected 
          SI.Pressure p_steam_sat "Saturation pressure";
          SI.MassFraction X_air "Dry air mass fraction";
          annotation(smoothOrder = 2);
        algorithm 
          p_steam_sat:=min(saturationPressure(T), 0.999 * p);
          X_air:=1 - X[Water];
          phi:=max(0.0, min(1.0, (p / p_steam_sat * X[Water]) / (X[Water] + k_mair * X_air)));
          annotation(Documentation(info = "<html>
Relative humidity is computed from pressure, temperature and composition with 1.0 as the upper limit at saturation. Water mass fraction is the first entry in the composition vector.
</html>"));
        end relativeHumidity_pTX;
        function relativeHumidity "Return relative humidity as a function of the thermodynamic state record"
          input ThermodynamicState state "Thermodynamic state";
          output Real phi "Relative humidity";
          annotation(smoothOrder = 2);
        algorithm 
          phi:=relativeHumidity_pTX(state.p, state.T, state.X);
          annotation(Documentation(info = "<html>
Relative humidity is computed from the thermodynamic state record with 1.0 as the upper limit at saturation.
</html>"));
        end relativeHumidity;
        function extends gasConstant "Return ideal gas constant as a function from thermodynamic state, only valid for phi<1"
          annotation(smoothOrder = 2);
        algorithm 
          R:=dryair.R * (1 - state.X[Water]) + steam.R * state.X[Water];
          annotation(Documentation(info = "<html>
The ideal gas constant for moist air is computed from <a href=Modelica://Modelica.Media.Air.MoistAir.ThermodynamicState>thermodynamic state</a> assuming that all water is in the gas phase.
</html>"));
        end gasConstant;
        function gasConstant_X "Return ideal gas constant as a function from composition X"
          input SI.MassFraction X[:] "Gas phase composition";
          output SI.SpecificHeatCapacity R "Ideal gas constant";
          annotation(smoothOrder = 2);
        algorithm 
          R:=dryair.R * (1 - X[Water]) + steam.R * X[Water];
          annotation(Documentation(info = "<html>
The ideal gas constant for moist air is computed from the gas phase composition. The first entry in composition vector X is the steam mass fraction of the gas phase.
</html>"));
        end gasConstant_X;
        function saturationPressureLiquid "Return saturation pressure of water as a function of temperature T in the range of 273.16 to 373.16 K"
          extends Modelica.Icons.Function;
          input SI.Temperature Tsat "saturation temperature";
          output SI.AbsolutePressure psat "saturation pressure";
          annotation(Inline = false, smoothOrder = 5, derivative = saturationPressureLiquid_der, Documentation(info = "<html>
Saturation pressure of water above the triple point temperature is computed from temperature. It's range of validity is between
273.16 and 373.16 K. Outside these limits a less accurate result is returned.
</html>"));
        algorithm 
          psat:=611.657 * Math.exp(17.2799 - 4102.99 / (Tsat - 35.719));
        end saturationPressureLiquid;
        function saturationPressureLiquid_der "Time derivative of saturationPressureLiquid"
          extends Modelica.Icons.Function;
          input SI.Temperature Tsat "Saturation temperature";
          input Real dTsat(unit = "K/s") "Saturation temperature derivative";
          output Real psat_der(unit = "Pa/s") "Saturation pressure";
          annotation(Inline = false, smoothOrder = 5, Documentation(info = "<html>
Derivative function of <a href=Modelica://Modelica.Media.Air.MoistAir.saturationPressureLiquid>saturationPressureLiquid</a>
</html>"));
        algorithm 
          psat_der:=(611.657 * Math.exp(17.2799 - 4102.99 / (Tsat - 35.719)) * 4102.99 * dTsat) / (Tsat - 35.719) / (Tsat - 35.719);
        end saturationPressureLiquid_der;
        function sublimationPressureIce "Return sublimation pressure of water as a function of temperature T between 223.16 and 273.16 K"
          extends Modelica.Icons.Function;
          input SI.Temperature Tsat "sublimation temperature";
          output SI.AbsolutePressure psat "sublimation pressure";
          annotation(Inline = false, smoothOrder = 5, derivative = sublimationPressureIce_der, Documentation(info = "<html>
Sublimation pressure of water below the triple point temperature is computed from temperature. It's range of validity is between
 223.16 and 273.16 K. Outside of these limits a less accurate result is returned.
</html>"));
        algorithm 
          psat:=611.657 * Math.exp(22.5159 * (1.0 - 273.16 / Tsat));
        end sublimationPressureIce;
        function sublimationPressureIce_der "Derivative function for 'sublimationPressureIce'"
          extends Modelica.Icons.Function;
          input SI.Temperature Tsat "Sublimation temperature";
          input Real dTsat(unit = "K/s") "Time derivative of sublimation temperature";
          output Real psat_der(unit = "Pa/s") "Sublimation pressure";
          annotation(Inline = false, smoothOrder = 5, Documentation(info = "<html>
Derivative function of <a href=Modelica://Modelica.Media.Air.MoistAir.sublimationPressureIce>saturationPressureIce</a>
</html>"));
        algorithm 
          psat_der:=(611.657 * Math.exp(22.5159 * (1.0 - 273.16 / Tsat)) * 22.5159 * 273.16 * dTsat) / Tsat / Tsat;
        end sublimationPressureIce_der;
        function extends saturationPressure "Return saturation pressure of water as a function of temperature T between 223.16 and 373.16 K"
          annotation(Inline = false, smoothOrder = 5, derivative = saturationPressure_der, Documentation(info = "<html>
Saturation pressure of water in the liquid and the solid region is computed using an Antoine-type correlation. It's range of validity is between 223.16 and 373.16 K. Outside of these limits a (less accurate) result is returned. Functions for the
<a href=Modelica://Modelica.Media.Air.MoistAir.sublimationPressureIce>solid</a> and the <a href=\"Modelica://Modelica.Media.Air.MoistAir.saturationPressureLiquid\"> liquid</a> region, respectively, are combined using the first derivative continuous <a href=Modelica://Modelica.Media.Air.MoistAir.Utilities.spliceFunction>spliceFunction</a>.
</html>"));
        algorithm 
          psat:=Utilities.spliceFunction(saturationPressureLiquid(Tsat), sublimationPressureIce(Tsat), Tsat - 273.16, 1.0);
        end saturationPressure;
        function saturationPressure_der "Derivative function for 'saturationPressure'"
          input Temperature Tsat "Saturation temperature";
          input Real dTsat(unit = "K/s") "Time derivative of saturation temperature";
          output Real psat_der(unit = "Pa/s") "Saturation pressure";
          annotation(Inline = false, smoothOrder = 5, Documentation(info = "<html>
Derivative function of <a href=Modelica://Modelica.Media.Air.MoistAir.saturationPressure>saturationPressure</a>
</html>"));
        algorithm 
          psat_der:=Utilities.spliceFunction_der(saturationPressureLiquid(Tsat), sublimationPressureIce(Tsat), Tsat - 273.16, 1.0, saturationPressureLiquid_der(Tsat = Tsat, dTsat = dTsat), sublimationPressureIce_der(Tsat = Tsat, dTsat = dTsat), dTsat, 0);
        end saturationPressure_der;
        function saturationTemperature "Return saturation temperature of water as a function of (partial) pressure p"
          input SI.Pressure p "Pressure";
          input SI.Temperature T_min = 200 "Lower boundary of solution";
          input SI.Temperature T_max = 400 "Upper boundary of solution";
          output SI.Temperature T "Saturation temperature";
        protected 
          package Internal
            extends Modelica.Media.Common.OneNonLinearEquation;
            record extends f_nonlinear_Data
            end f_nonlinear_Data;
            function extends f_nonlinear
            algorithm 
              y:=saturationPressure(x);
            end f_nonlinear;
            function extends solve
            end solve;
          end Internal;
        algorithm 
          T:=Internal.solve(p, T_min, T_max);
          annotation(Documentation(info = "<html>
 Computes saturation temperature from (partial) pressure via numerical inversion of the function <a href=Modelica://Modelica.Media.Air.MoistAir.saturationPressure>saturationPressure</a>. Therefore additional inputs are required (or the defaults are used) for upper and lower temperature bounds.
</html>"));
        end saturationTemperature;
        function extends enthalpyOfVaporization "Return enthalpy of vaporization of water as a function of temperature T, 0 - 130 degC"
          annotation(smoothOrder = 2);
        algorithm 
          r0:=(2405900 - 2500500) / (40 - 0) * (T - 273.16) + 2500500;
          annotation(Documentation(info = "<html>
Enthalpy of vaporization of water is computed from temperature in the region of 0 to 130 C.
</html>"));
        end enthalpyOfVaporization;
        function HeatCapacityOfWater "Return specific heat capacity of water (liquid only) as a function of temperature T"
          extends Modelica.Icons.Function;
          input Temperature T "Temperature";
          output SpecificHeatCapacity cp_fl "Specific heat capacity of liquid";
          annotation(Documentation(info = "<html>
The specific heat capacity of water (liquid and solid) is calculated using a
                 polynomial approach and data from VDI-Waermeatlas 8. Edition (Db1)
</html>"));
          annotation(smoothOrder = 2);
        algorithm 
          cp_fl:=1000.0 * (4.2166 - (T - 273.15) * (0.0033166 + (T - 273.15) * (0.00010295 - (T - 273.15) * (1.3819e-06 + (T - 273.15) * 7.3221e-09))));
        end HeatCapacityOfWater;
        function extends enthalpyOfLiquid "Return enthalpy of liquid water as a function of temperature T(use enthalpyOfWater instead)"
          annotation(Inline = false, smoothOrder = 5, Documentation(info = "<html>
Specific enthalpy of liquid water is computed from temperature using a polynomial approach. Kept for compatibility reasons, better use <a href=Modelica://Modelica.Media.Air.MoistAir.enthalpyOfWater>enthalpyOfWater</a> instead.
</html>"));
        algorithm 
          h:=(T - 273.15) * 1000.0 * (4.2166 - 0.5 * (T - 273.15) * (0.0033166 + 0.333333 * (T - 273.15) * (0.00010295 - 0.25 * (T - 273.15) * (1.3819e-06 + 0.2 * (T - 273.15) * 7.3221e-09))));
        end enthalpyOfLiquid;
        function extends enthalpyOfGas "Return specific enthalpy of gas (air and steam) as a function of temperature T and composition X"
          annotation(Inline = false, smoothOrder = 5, Documentation(info = "<html>
Specific enthalpy of moist air is computed from temperature, provided all water is in the gaseous state. The first entry in the composition vector X must be the mass fraction of steam. For a function that also covers the fog region please refer to <a href=Modelica://Modelica.Media.Air.MoistAir.h_pTX>h_pTX</a>.
</html>"));
        algorithm 
          h:=SingleGasNasa.h_Tlow(data = steam, T = T, refChoice = 3, h_off = 46479.819 + 2501014.5) * X[Water] + SingleGasNasa.h_Tlow(data = dryair, T = T, refChoice = 3, h_off = 25104.684) * (1.0 - X[Water]);
        end enthalpyOfGas;
        function extends enthalpyOfCondensingGas "Return specific enthalpy of steam as a function of temperature T"
          annotation(Inline = false, smoothOrder = 5, Documentation(info = "<html>
Specific enthalpy of steam is computed from temperature.
</html>"));
        algorithm 
          h:=SingleGasNasa.h_Tlow(data = steam, T = T, refChoice = 3, h_off = 46479.819 + 2501014.5);
        end enthalpyOfCondensingGas;
        function extends enthalpyOfNonCondensingGas "Return specific enthalpy of dry air as a function of temperature T"
          annotation(Inline = false, smoothOrder = 1, Documentation(info = "<html>
Specific enthalpy of dry air is computed from temperature.
</html>"));
        algorithm 
          h:=SingleGasNasa.h_Tlow(data = dryair, T = T, refChoice = 3, h_off = 25104.684);
        end enthalpyOfNonCondensingGas;
        function enthalpyOfWater "Computes specific enthalpy of water (solid/liquid) near atmospheric pressure from temperature T"
          input SIunits.Temperature T "Temperature";
          output SIunits.SpecificEnthalpy h "Specific enthalpy of water";
          annotation(derivative = enthalpyOfWater_der, Documentation(info = "<html>
Specific enthalpy of water (liquid and solid) is computed from temperature using constant properties as follows:<br>
<ul>
<li>  heat capacity of liquid water:4200 J/kg
<li>  heat capacity of solid water: 2050 J/kg
<li>  enthalpy of fusion (liquid=>solid): 333000 J/kg
</ul>
Pressure is assumed to be around 1 bar. This function is usually used to determine the specific enthalpy of the liquid or solid fraction of moist air.
</html>"));
        algorithm 
          h:=Utilities.spliceFunction(4200 * (T - 273.15), 2050 * (T - 273.15) - 333000, T - 273.16, 0.1);
        end enthalpyOfWater;
        function enthalpyOfWater_der "Derivative function of enthalpyOfWater"
          input SIunits.Temperature T "Temperature";
          input Real dT(unit = "K/s") "Time derivative of temperature";
          output Real dh(unit = "J/(kg.s)") "Time derivative of specific enthalpy";
        algorithm 
          dh:=Utilities.spliceFunction_der(4200 * (T - 273.15), 2050 * (T - 273.15) - 333000, T - 273.16, 0.1, 4200 * dT, 2050 * dT, dT, 0);
          annotation(Documentation(info = "<html>
Derivative function for <a href=Modelica://Modelica.Media.Air.MoistAir.enthalpyOfWater>enthalpyOfWater</a>.

</html>"));
        end enthalpyOfWater_der;
        function extends pressure "Returns pressure of ideal gas as a function of the thermodynamic state record"
          annotation(smoothOrder = 2);
        algorithm 
          p:=state.p;
          annotation(Documentation(info = "<html>
Pressure is returned from the thermodynamic state record input as a simple assignment.
</html>"));
        end pressure;
        function extends temperature "Return temperature of ideal gas as a function of the thermodynamic state record"
          annotation(smoothOrder = 2);
        algorithm 
          T:=state.T;
          annotation(Documentation(info = "<html>
Temperature is returned from the thermodynamic state record input as a simple assignment.
</html>"));
        end temperature;
        function T_phX "Return temperature as a function of pressure p, specific enthalpy h and composition X"
          input AbsolutePressure p "Pressure";
          input SpecificEnthalpy h "Specific enthalpy";
          input MassFraction[:] X "Mass fractions of composition";
          output Temperature T "Temperature";
        protected 
          package Internal "Solve h(data,T) for T with given h (use only indirectly via temperature_phX)"
            extends Modelica.Media.Common.OneNonLinearEquation;
            record extends f_nonlinear_Data "Data to be passed to non-linear function"
              extends Modelica.Media.IdealGases.Common.DataRecord;
            end f_nonlinear_Data;
            function extends f_nonlinear
            algorithm 
              y:=h_pTX(p, x, X);
            end f_nonlinear;
            function extends solve
            end solve;
          end Internal;
        algorithm 
          T:=Internal.solve(h, 240, 400, p, X[1:nXi], steam);
          annotation(Documentation(info = "<html>
Temperature is computed from pressure, specific enthalpy and composition via numerical inversion of function <a href=Modelica://Modelica.Media.Air.MoistAir.h_pTX>h_pTX</a>.
</html>"));
        end T_phX;
        function extends density "Returns density of ideal gas as a function of the thermodynamic state record"
          annotation(smoothOrder = 2);
        algorithm 
          d:=state.p / (gasConstant(state) * state.T);
          annotation(Documentation(info = "<html>
Density is computed from pressure, temperature and composition in the thermodynamic state record applying the ideal gas law.
</html>"));
        end density;
        function extends specificEnthalpy "Return specific enthalpy of moist air as a function of the thermodynamic state record"
          annotation(smoothOrder = 2);
        algorithm 
          h:=h_pTX(state.p, state.T, state.X);
          annotation(Documentation(info = "<html>
Specific enthalpy of moist air is computed from the thermodynamic state record. The fog region is included for both, ice and liquid fog.
</html>"));
        end specificEnthalpy;
        function h_pTX "Return specific enthalpy of moist air as a function of pressure p, temperature T and composition X"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input SI.MassFraction X[:] "Mass fractions of moist air";
          output SI.SpecificEnthalpy h "Specific enthalpy at p, T, X";
          annotation(derivative = h_pTX_der, Inline = false, Documentation(info = "<html>
Specific enthalpy of moist air is computed from pressure, temperature and composition with X[1] as the total water mass fraction. The fog region is included for both, ice and liquid fog.
</html>"));
        protected 
          SI.AbsolutePressure p_steam_sat "Partial saturation pressure of steam";
          SI.MassFraction X_sat "Absolute humidity per unit mass of moist air";
          SI.MassFraction X_liquid "mass fraction of liquid water";
          SI.MassFraction X_steam "mass fraction of steam water";
          SI.MassFraction X_air "mass fraction of air";
        algorithm 
          p_steam_sat:=saturationPressure(T);
          X_sat:=min((p_steam_sat * k_mair) / max(100 * Constants.eps, p - p_steam_sat) * (1 - X[Water]), 1.0);
          X_liquid:=max(X[Water] - X_sat, 0.0);
          X_steam:=X[Water] - X_liquid;
          X_air:=1 - X[Water];
          h:={SingleGasNasa.h_Tlow(data = steam, T = T, refChoice = 3, h_off = 46479.819 + 2501014.5),SingleGasNasa.h_Tlow(data = dryair, T = T, refChoice = 3, h_off = 25104.684)} * {X_steam,X_air} + enthalpyOfWater(T) * X_liquid;
        end h_pTX;
        function h_pTX_der "Derivative function of h_pTX"
          extends Modelica.Icons.Function;
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input SI.MassFraction X[:] "Mass fractions of moist air";
          input Real dp(unit = "Pa/s") "Pressure derivative";
          input Real dT(unit = "K/s") "Temperature derivative";
          input Real dX[:](each unit = "1/s") "Composition derivative";
          output Real h_der(unit = "J/(kg.s)") "Time derivative of specific enthalpy";
          annotation(Inline = false, smoothOrder = 1, Documentation(info = "<html>
Derivative function for <a href=Modelica://Modelica.Media.Air.MoistAir.h_pTX>h_pTX</a>.
</html>"));
        protected 
          SI.AbsolutePressure p_steam_sat "Partial saturation pressure of steam";
          SI.MassFraction X_sat "Absolute humidity per unit mass of moist air";
          SI.MassFraction X_liquid "Mass fraction of liquid water";
          SI.MassFraction X_steam "Mass fraction of steam water";
          SI.MassFraction X_air "Mass fraction of air";
          SI.MassFraction x_sat "Absolute humidity per unit mass of dry air at saturation";
          Real dX_steam(unit = "1/s") "Time deriveative of steam mass fraction";
          Real dX_air(unit = "1/s") "Time derivative of dry air mass fraction";
          Real dX_liq(unit = "1/s") "Time derivative of liquid/solid water mass fraction";
          Real dps(unit = "Pa/s") "Time derivative of saturation pressure";
          Real dx_sat(unit = "1/s") "Time derivative of abolute humidity per unit mass of dry air";
        algorithm 
          p_steam_sat:=saturationPressure(T);
          x_sat:=(p_steam_sat * k_mair) / max(100 * Modelica.Constants.eps, p - p_steam_sat);
          X_sat:=min(x_sat * (1 - X[Water]), 1.0);
          X_liquid:=Utilities.spliceFunction(X[Water] - X_sat, 0.0, X[Water] - X_sat, 1e-06);
          X_steam:=X[Water] - X_liquid;
          X_air:=1 - X[Water];
          dX_air:= -dX[Water];
          dps:=saturationPressure_der(Tsat = T, dTsat = dT);
          dx_sat:=(k_mair * (dps * (p - p_steam_sat) - p_steam_sat * (dp - dps))) / (p - p_steam_sat) / (p - p_steam_sat);
          dX_liq:=Utilities.spliceFunction_der(X[Water] - X_sat, 0.0, X[Water] - X_sat, 1e-06, (1 + x_sat) * dX[Water] - (1 - X[Water]) * dx_sat, 0.0, (1 + x_sat) * dX[Water] - (1 - X[Water]) * dx_sat, 0.0);
          dX_steam:=dX[Water] - dX_liq;
          h_der:=X_steam * Modelica.Media.IdealGases.Common.SingleGasNasa.h_Tlow_der(data = steam, T = T, refChoice = 3, h_off = 46479.819 + 2501014.5, dT = dT) + dX_steam * Modelica.Media.IdealGases.Common.SingleGasNasa.h_Tlow(data = steam, T = T, refChoice = 3, h_off = 46479.819 + 2501014.5) + X_air * Modelica.Media.IdealGases.Common.SingleGasNasa.h_Tlow_der(data = dryair, T = T, refChoice = 3, h_off = 25104.684, dT = dT) + dX_air * Modelica.Media.IdealGases.Common.SingleGasNasa.h_Tlow(data = dryair, T = T, refChoice = 3, h_off = 25104.684) + X_liquid * enthalpyOfWater_der(T = T, dT = dT) + dX_liq * enthalpyOfWater(T);
        end h_pTX_der;
        function extends isentropicExponent "Return isentropic exponent (only for gas fraction!)"
        algorithm 
          gamma:=specificHeatCapacityCp(state) / specificHeatCapacityCv(state);
        end isentropicExponent;
        function isentropicEnthalpyApproximation "Approximate calculation of h_is from upstream properties, downstream pressure, gas part only"
          extends Modelica.Icons.Function;
          input AbsolutePressure p2 "downstream pressure";
          input ThermodynamicState state "thermodynamic state at upstream location";
          output SpecificEnthalpy h_is "isentropic enthalpy";
        protected 
          SpecificEnthalpy h "specific enthalpy at upstream location";
          IsentropicExponent gamma = isentropicExponent(state) "Isentropic exponent";
        protected 
          MassFraction[nX] X "complete X-vector";
        algorithm 
          X:=if reducedX then cat(1, state.X, {1 - sum(state.X)}) else state.X;
          h:={SingleGasNasa.h_Tlow(data = steam, T = state.T, refChoice = 3, h_off = 46479.819 + 2501014.5),SingleGasNasa.h_Tlow(data = dryair, T = state.T, refChoice = 3, h_off = 25104.684)} * X;
          h_is:=h + gamma / (gamma - 1.0) * state.T * gasConstant(state) * ((p2 / state.p) ^ ((gamma - 1) / gamma) - 1.0);
        end isentropicEnthalpyApproximation;
        function extends specificInternalEnergy "Return specific internal energy of moist air as a function of the thermodynamic state record"
          extends Modelica.Icons.Function;
          output SI.SpecificInternalEnergy u "Specific internal energy";
          annotation(smoothOrder = 2);
        algorithm 
          u:=specificInternalEnergy_pTX(state.p, state.T, state.X);
          annotation(Documentation(info = "<html>
Specific internal energy is determined from the thermodynamic state record, assuming that the liquid or solid water volume is negligible.
</html>"));
        end specificInternalEnergy;
        function specificInternalEnergy_pTX "Return specific internal energy of moist air as a function of pressure p, temperature T and composition X"
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input SI.MassFraction X[:] "Mass fractions of moist air";
          output SI.SpecificInternalEnergy u "Specific internal energy";
        protected 
          SI.AbsolutePressure p_steam_sat "Partial saturation pressure of steam";
          SI.MassFraction X_liquid "Mass fraction of liquid water";
          SI.MassFraction X_steam "Mass fraction of steam water";
          SI.MassFraction X_air "Mass fraction of air";
          SI.MassFraction X_sat "Absolute humidity per unit mass of moist air";
          Real R_gas "Ideal gas constant";
        algorithm 
          p_steam_sat:=saturationPressure(T);
          X_sat:=min((p_steam_sat * k_mair) / max(100 * Constants.eps, p - p_steam_sat) * (1 - X[Water]), 1.0);
          X_liquid:=max(X[Water] - X_sat, 0.0);
          X_steam:=X[Water] - X_liquid;
          X_air:=1 - X[Water];
          R_gas:=(dryair.R * X_air) / (1 - X_liquid) + (steam.R * X_steam) / (1 - X_liquid);
          u:=X_steam * SingleGasNasa.h_Tlow(data = steam, T = T, refChoice = 3, h_off = 46479.819 + 2501014.5) + X_air * SingleGasNasa.h_Tlow(data = dryair, T = T, refChoice = 3, h_off = 25104.684) + enthalpyOfWater(T) * X_liquid - R_gas * T;
          annotation(derivative = specificInternalEnergy_pTX_der, Documentation(info = "<html>
Specific internal energy is determined from pressure p, temperature T and composition X, assuming that the liquid or solid water volume is negligible.
</html>"));
        end specificInternalEnergy_pTX;
        function specificInternalEnergy_pTX_der "Derivative function for specificInternalEnergy_pTX"
          input SI.Pressure p "Pressure";
          input SI.Temperature T "Temperature";
          input SI.MassFraction X[:] "Mass fractions of moist air";
          input Real dp(unit = "Pa/s") "Pressure derivative";
          input Real dT(unit = "K/s") "Temperature derivative";
          input Real dX[:](each unit = "1/s") "Mass fraction derivatives";
          output Real u_der(unit = "J/(kg.s)") "Specific internal energy derivative";
        protected 
          SI.AbsolutePressure p_steam_sat "Partial saturation pressure of steam";
          SI.MassFraction X_liquid "Mass fraction of liquid water";
          SI.MassFraction X_steam "Mass fraction of steam water";
          SI.MassFraction X_air "Mass fraction of air";
          SI.MassFraction X_sat "Absolute humidity per unit mass of moist air";
          SI.SpecificHeatCapacity R_gas "Ideal gas constant";
          SI.MassFraction x_sat "Absolute humidity per unit mass of dry air at saturation";
          Real dX_steam(unit = "1/s") "Time deriveative of steam mass fraction";
          Real dX_air(unit = "1/s") "Time derivative of dry air mass fraction";
          Real dX_liq(unit = "1/s") "Time derivative of liquid/solid water mass fraction";
          Real dps(unit = "Pa/s") "Time derivative of saturation pressure";
          Real dx_sat(unit = "1/s") "Time derivative of abolute humidity per unit mass of dry air";
          Real dR_gas(unit = "J/(kg.K.s)") "Time derivative of ideal gas constant";
        algorithm 
          p_steam_sat:=saturationPressure(T);
          x_sat:=(p_steam_sat * k_mair) / max(100 * Modelica.Constants.eps, p - p_steam_sat);
          X_sat:=min(x_sat * (1 - X[Water]), 1.0);
          X_liquid:=Utilities.spliceFunction(X[Water] - X_sat, 0.0, X[Water] - X_sat, 1e-06);
          X_steam:=X[Water] - X_liquid;
          X_air:=1 - X[Water];
          R_gas:=(steam.R * X_steam) / (1 - X_liquid) + (dryair.R * X_air) / (1 - X_liquid);
          dX_air:= -dX[Water];
          dps:=saturationPressure_der(Tsat = T, dTsat = dT);
          dx_sat:=(k_mair * (dps * (p - p_steam_sat) - p_steam_sat * (dp - dps))) / (p - p_steam_sat) / (p - p_steam_sat);
          dX_liq:=Utilities.spliceFunction_der(X[Water] - X_sat, 0.0, X[Water] - X_sat, 1e-06, (1 + x_sat) * dX[Water] - (1 - X[Water]) * dx_sat, 0.0, (1 + x_sat) * dX[Water] - (1 - X[Water]) * dx_sat, 0.0);
          dX_steam:=dX[Water] - dX_liq;
          dR_gas:=(steam.R * (dX_steam * (1 - X_liquid) + dX_liq * X_steam) + dryair.R * (dX_air * (1 - X_liquid) + dX_liq * X_air)) / (1 - X_liquid) / (1 - X_liquid);
          u_der:=X_steam * SingleGasNasa.h_Tlow_der(data = steam, T = T, refChoice = 3, h_off = 46479.819 + 2501014.5, dT = dT) + dX_steam * SingleGasNasa.h_Tlow(data = steam, T = T, refChoice = 3, h_off = 46479.819 + 2501014.5) + X_air * SingleGasNasa.h_Tlow_der(data = dryair, T = T, refChoice = 3, h_off = 25104.684, dT = dT) + dX_air * SingleGasNasa.h_Tlow(data = dryair, T = T, refChoice = 3, h_off = 25104.684) + X_liquid * enthalpyOfWater_der(T = T, dT = dT) + dX_liq * enthalpyOfWater(T) - dR_gas * T - R_gas * dT;
          annotation(Documentation(info = "<html>
Derivative function for <a href=Modelica://Modelica.Media.Air.MoistAir.specificInternalEnergy_pTX>specificInternalEnergy_pTX</a>.
</html>"));
        end specificInternalEnergy_pTX_der;
        function extends specificEntropy "Return specific entropy from thermodynamic state record, only valid for phi<1"
          annotation(Inline = false, smoothOrder = 2, Documentation(info = "<html>
Specific entropy is calculated from the thermodynamic state record, assuming ideal gas behavior and including entropy of mixing. Liquid or solid water is not taken into account, the entire water content X[1] is assumed to be in the vapor state (relative humidity below 1.0).
</html>"));
        protected 
          MoleFraction[2] Y = massToMoleFractions(state.X, {steam.MM,dryair.MM}) "molar fraction";
        algorithm 
          s:=SingleGasNasa.s0_Tlow(dryair, state.T) * (1 - state.X[Water]) + SingleGasNasa.s0_Tlow(steam, state.T) * state.X[Water] - (state.X[Water] * Modelica.Constants.R) / MMX[Water] * (if state.X[Water] < Modelica.Constants.eps then state.X[Water] else Modelica.Math.log((Y[Water] * state.p) / reference_p)) + ((1 - state.X[Water]) * Modelica.Constants.R) / MMX[Air] * (if 1 - state.X[Water] < Modelica.Constants.eps then 1 - state.X[Water] else Modelica.Math.log((Y[Air] * state.p) / reference_p));
        end specificEntropy;
        function extends specificGibbsEnergy "Return specific Gibbs energy as a function of the thermodynamic state record, only valid for phi<1"
          extends Modelica.Icons.Function;
          annotation(smoothOrder = 2);
        algorithm 
          g:=h_pTX(state.p, state.T, state.X) - state.T * specificEntropy(state);
          annotation(Documentation(info = "<html>
The Gibbs Energy is computed from the thermodynamic state record for moist air with a water content below saturation.
</html>"));
        end specificGibbsEnergy;
        function extends specificHelmholtzEnergy "Return specific Helmholtz energy as a function of the thermodynamic state record, only valid for phi<1"
          extends Modelica.Icons.Function;
          annotation(smoothOrder = 2);
        algorithm 
          f:=h_pTX(state.p, state.T, state.X) - gasConstant(state) * state.T - state.T * specificEntropy(state);
          annotation(Documentation(info = "<html>
The Specific Helmholtz Energy is computed from the thermodynamic state record for moist air with a water content below saturation.
</html>"));
        end specificHelmholtzEnergy;
        function extends specificHeatCapacityCp "Return specific heat capacity at constant pressure as a function of the thermodynamic state record"
          annotation(Inline = false, smoothOrder = 2, Documentation(info = "<html>
The specific heat capacity at constant pressure <b>cp</b> is computed from temperature and composition for a mixture of steam (X[1]) and dry air. All water is assumed to be in the vapor state.
</html>"));
        protected 
          Real dT(unit = "s/K") = 1.0;
        algorithm 
          cp:=h_pTX_der(state.p, state.T, state.X, 0.0, 1.0, zeros(size(state.X, 1))) * dT "Definition of cp: dh/dT @ constant p";
        end specificHeatCapacityCp;
        function extends specificHeatCapacityCv "Return specific heat capacity at constant volume as a function of the thermodynamic state record"
          annotation(Inline = false, smoothOrder = 2, Documentation(info = "<html>
The specific heat capacity at constant density <b>cv</b> is computed from temperature and composition for a mixture of steam (X[1]) and dry air. All water is assumed to be in the vapor state.
</html>"));
        algorithm 
          cv:=SingleGasNasa.cp_Tlow(dryair, state.T) * (1 - state.X[Water]) + SingleGasNasa.cp_Tlow(steam, state.T) * state.X[Water] - gasConstant(state);
        end specificHeatCapacityCv;
        function extends dynamicViscosity "Return dynamic viscosity as a function of the thermodynamic state record, valid from 73.15 K to 373.15 K"
          import Modelica.Media.Incompressible.TableBased.Polynomials_Temp;
          annotation(smoothOrder = 2);
        algorithm 
          eta:=Polynomials_Temp.evaluate({ -4.96717436974791e-11,5.06626785714286e-08,1.72937731092437e-05}, Cv.to_degC(state.T));
          annotation(Documentation(info = "<html>
Dynamic viscosity is computed from temperature using a simple polynomial for dry air, assuming that moisture influence is small. Range of  validity is from 73.15 K to 373.15 K.
</html>"));
        end dynamicViscosity;
        function extends thermalConductivity "Return thermal conductivity as a function of the thermodynamic state record, valid from 73.15 K to 373.15 K"
          import Modelica.Media.Incompressible.TableBased.Polynomials_Temp;
          annotation(smoothOrder = 2);
        algorithm 
          lambda:=Polynomials_Temp.evaluate({ -4.8737307422969e-08,7.67803133753502e-05,0.0241814385504202}, Cv.to_degC(state.T));
          annotation(Documentation(info = "<html>
Thermal conductivity is computed from temperature using a simple polynomial for dry air, assuming that moisture influence is small. Range of  validity is from 73.15 K to 373.15 K.
</html>"));
        end thermalConductivity;
        package Utilities "utility functions"
          function spliceFunction "Spline interpolation of two functions"
            input Real pos "Returned value for x-deltax >= 0";
            input Real neg "Returned value for x+deltax <= 0";
            input Real x "Function argument";
            input Real deltax = 1 "Region around x with spline interpolation";
            output Real out;
            annotation(derivative = spliceFunction_der);
          protected 
            Real scaledX;
            Real scaledX1;
            Real y;
          algorithm 
            scaledX1:=x / deltax;
            scaledX:=scaledX1 * Modelica.Math.asin(1);
            if scaledX1 <=  -0.999999999 then 
                        y:=0;
            elseif scaledX1 >= 0.999999999 then
              y:=1;


            else             y:=(Modelica.Math.tanh(Modelica.Math.tan(scaledX)) + 1) / 2;

            end if;
            out:=pos * y + (1 - y) * neg;
          end spliceFunction;
          function spliceFunction_der "Derivative of spliceFunction"
            input Real pos;
            input Real neg;
            input Real x;
            input Real deltax = 1;
            input Real dpos;
            input Real dneg;
            input Real dx;
            input Real ddeltax = 0;
            output Real out;
          protected 
            Real scaledX;
            Real scaledX1;
            Real dscaledX1;
            Real y;
          algorithm 
            scaledX1:=x / deltax;
            scaledX:=scaledX1 * Modelica.Math.asin(1);
            dscaledX1:=(dx - scaledX1 * ddeltax) / deltax;
            if scaledX1 <=  -0.99999999999 then 
                        y:=0;
            elseif scaledX1 >= 0.9999999999 then
              y:=1;


            else             y:=(Modelica.Math.tanh(Modelica.Math.tan(scaledX)) + 1) / 2;

            end if;
            out:=dpos * y + (1 - y) * dneg;
            if abs(scaledX1) < 1 then 
                        out:=out + ((pos - neg) * dscaledX1 * Modelica.Math.asin(1)) / 2 / (Modelica.Math.cosh(Modelica.Math.tan(scaledX)) * Modelica.Math.cos(scaledX)) ^ 2;

            else 
            end if;
          end spliceFunction_der;
        end Utilities;
        annotation(Documentation(info = "<html>
<h4>Thermodynamic Model</h4>
<p>This package provides a full thermodynamic model of moist air including the fog region and temperatures below zero degC.
The governing assumptions in this model are:</p>
<ul>
<li>the perfect gas law applies</li>
<li>water volume other than that of steam is neglected</li></ul>
<p>All extensive properties are expressed in terms of the total mass in order to comply with other media in this libary. However, for moist air it is rather common to express the absolute humidity in terms of mass of dry air only, which has advantages when working with charts. In addition, care must be taken, when working with mass fractions with respect to total mass, that all properties refer to the same water content when being used in mathematical operations (which is always the case if based on dry air only). Therefore two absolute humidities are computed in the <b>BaseProperties</b> model: <b>X</b> denotes the absolute humidity in terms of the total mass while <b>x</b> denotes the absolute humitity per unit mass of dry air. In addition, the relative humidity <b>phi</b> is also computed.</p>
<p>At the triple point temperature of water of 0.01C or 273.16 K and a relative humidity greater than 1 fog may be present as liquid and as ice resulting in a specific enthalpy somewhere between those of the two isotherms for solid and liquid fog, respectively. For numerical reasons a coexisting mixture of 50% solid and 50% liquid fog is assumed in the fog region at the triple point in this model.

<h4>Range of validity</h4>
<p>From the assumptions mentioned above it follows that the <b>pressure</b> should be in the region around <b>atmospheric</b> conditions or below (a few bars may still be fine though). Additionally a very high water content at low temperatures would yield incorrect densities, because the volume of the liquid or solid phase would not be negligible anymore. The model does not provide information on limits for water drop size in the fog region or transport information for the actual condensation or evaporation process in combination with surfaces. All excess water which is not in its vapour state is assumed to be still present in the air regarding its energy but not in terms of its spatial extent.<br><br>
The thermodynamic model may be used for <b>temperatures</b> ranging from <b>240 - 400 K</b>. This holds for all functions unless otherwise stated in their description. However, although the model works at temperatures above the saturation temperature it is questionable to use the term \"relative humidity\" in this region. Please note, that although several functions compute pure water properties, they are designed to be used within the moist air medium model where properties are dominated by air and steam in their vapor states, and not for pure liquid water applications.</p>

<h4>Transport Properties</h4>
<p>Several additional functions that are not needed to describe the thermodynamic system, but are required to model transport processes, like heat and mass transfer, may be called. They usually neglect the moisture influence unless otherwise stated.</p>

<h4>Application</h4>
<p>The model's main area of application is all processes that involve moist air cooling under near atmospheric pressure with possible moisture condensation. This is the case in all domestic and industrial air conditioning applications. Another large domain of moist air applications covers all processes that deal with dehydration of bulk material using air as a transport medium. Engineering tasks involving moist air are often performed (or at least visualized) by using charts that contain all relevant thermodynamic data for a moist air system. These so called psychrometric charts can be generated from the medium properties in this package. The model <a href=\"Modelica://Modelica.Media.Air.MoistAir.PsychrometricData\">PsychrometricData</a> may be used for this purpose in order to obtain data for figures like those below (the plotting itself is not part of the model though).</p>

<p><img src=\"../../Modelica/Images/Media/Air/Mollier.png\">
<img src=\"../../Modelica/Images/Media/Air/PsycroChart.png\"></p>
<p>
<b>Legend:</b> blue - constant specific enthalpy, red - constant temperature, black - constant relative humidity</p>

</html>"));
        model PsychrometricData "Produces plot data for psychrometric charts"
          extends Modelica.Icons.Example;
          package Medium = Modelica.Media.Air.MoistAir "Used medium package";
          parameter SIunits.Pressure p_const = 100000.0 "Pressure";
          parameter Integer n_T = 11 "Number of isotherms";
          parameter SIunits.Temperature T_min = 253.15 "Lowest isotherm";
          parameter SIunits.Temperature T_step = 10 "Temperature step between two isotherms";
          parameter Integer n_h = 16 "Number of lines with constant specific enthalpy";
          parameter SIunits.SpecificEnthalpy h_min =  -20000.0 "Lowest line of constant enthalpy";
          parameter SIunits.SpecificEnthalpy h_step = 10000.0 "Enthalpy step between two lines of constant enthalpy";
          parameter Integer n_phi = 10 "Number of lines with constant relative humidity";
          parameter Real phi_min = 0.1 "Lowest line of constant humidity";
          parameter Real phi_step = 0.1 "Step between two lines of constant humidity";
          parameter SIunits.MassFraction x_min = 0.0 "Minimum diagram absolute humidity";
          parameter SIunits.MassFraction x_max = 0.03 "Maximum diagram absolute humidity";
          parameter SIunits.Time t = 1 "Simulation time";
          final parameter SIunits.Temperature[n_T] T_const = array(T_min - T_step + i * T_step for i in 1:n_T) "Constant temperatures";
          final parameter SIunits.SpecificEnthalpy[n_h] h_const = array((i - 1) * h_step + h_min for i in 1:n_h) "Constant enthalpies";
          final parameter Real[n_phi] phi_const = array((i - 1) * phi_step + phi_min for i in 1:n_phi) "Constant relative humidities";
          final parameter Real diagSlope = Medium.enthalpyOfVaporization(273.15) "Rotation of diagram that zero degrees isotherm becomes horizontal outside the fog region";
          final parameter SIunits.MassFraction x_start = x_min "Initial absolute humidity in kg water/kg dry air";
          SIunits.MassFraction x(start = x_start) "Absolute humidity in kg water/kg dry air";
          SIunits.SpecificEnthalpy[n_T] hx_T "h_1+x for const T";
          SIunits.SpecificEnthalpy[n_h] hx_h(start = h_const) "Const h_1+x";
          SIunits.SpecificEnthalpy[n_phi] hx_phi "h_1+x for const phi";
          SIunits.SpecificEnthalpy[n_T] y_T "Chart enthalpy for const T";
          SIunits.SpecificEnthalpy[n_h] y_h "Chart enthalpy for const h";
          SIunits.SpecificEnthalpy[n_phi] y_phi "Chart enthalpy for const phi";
          Medium.BaseProperties[n_T] medium_T "Medium properties for const T";
          Medium.BaseProperties[n_phi] medium_phi "Medium properties for const phi";
        protected 
          SIunits.Pressure[n_phi] ps_phi "Saturation pressure for constant-phi-lines";
          SIunits.Temperature[n_phi] T_phi(each start = 290);
          Boolean[n_T] fog(start = fill(false, n_T)) "Triggers events at intersection of isotherms with phi=1";
          SIunits.Pressure[n_T] pd "Steam partial pressure along isotherms";

        initial equation 
          x = x_min;

        equation 
          der(x) = (x_max - x_min) / t;
          for i in 1:n_T loop
          medium_T[i].T = T_const[i];
          medium_T[i].p = p_const;
          medium_T[i].Xi = {x / (1 + x)};
          hx_T[i] = medium_T[i].h * (medium_T[i].x_water + 1);
          y_T[i] = hx_T[i] - diagSlope * x;
          pd[i] = (medium_T[i].Xi[1] * medium_T[i].MM) / MMX[1] * p_const;
          fog[i] = pd[i] >= Medium.saturationPressure(T_const[i]);

          end for;
          for i in 1:n_h loop
          der(hx_h[i]) = 0.0;
          y_h[i] = hx_h[i] - diagSlope * x;

          end for;
          for i in 1:n_phi loop
          medium_phi[i].p = p_const;
          ps_phi[i] = (p_const * x) / phi_const[i] / (Medium.k_mair + x);
          T_phi[i] = if x < 5e-06 then 200 else Medium.saturationTemperature(ps_phi[i]);
          medium_phi[i].T = T_phi[i];
          medium_phi[i].Xi = {x / (1 + x)};
          hx_phi[i] = medium_phi[i].h * (medium_phi[i].x_water + 1);
          y_phi[i] = hx_phi[i] - diagSlope * x;

          end for;
          annotation(experiment(StopTime = 1), Documentation(info = "<html>
<p>This model produces psychrometric data from the moist air model in this library to be plotted in charts. The two most common chart varieties are the Mollier Diagram and the Psycrometric Chart. The first is widely used in some European countries while the second is more common in the Anglo-American world. Specific enthalpy is plotted over absolute humidity in the Mollier Diagram, it is the other way round in the Psychrometric Chart.<br>
It must be noted that the relationship of both axis variables is not right-angled, the absolute humidity follows a slope which equals the enthalpy of vaporization at 0C. For better reading and in oder to reduce the fog region the humidity axis is rotated to obtain a right-angled plot. Both charts usually contain additional information as isochores or auxiliary scales for e.g. heat ratios. Those information are omitted in this model and the charts below. Other important features of psychrometric chart data are that all mass specific variables (like absolute humidity, specific enthalpy etc.) are expressed in terms of kg dry air and that their baseline of 0 enthalpy is found at 0C and zero humidity. </p>

<p><img src=\"../../Modelica/Images/Media/Air/Mollier.png\">
<img src=\"../../Modelica/Images/Media/Air/PsycroChart.png\"></p>
<p>
<b>Legend:</b> blue - constant specific enthalpy, red - constant temperature, black - constant relative humidity</p>

<p>The model provides data for lines of constant specific enthalpy, temperature and relative humidity in a Mollier Diagram or Psychrometric Chart as they were used for the figures above. For limitations and ranges of validity please refer to the <a href=Modelica://Modelica.Media.Air.MoistAir>MoistAir package description</a>. Absolute humidity <b>x</b> is increased with time in this model. The specific enthalpies adjusted for plotting are then obtained from:</p>
<ul>
<li><b>y_h</b>: constant specific enthalpy</li>
<li><b>y_T</b>: constant temperature</li>
<li><b>y_phi</b>: constant relative humidity</li>
</ul>
</html>"));
        end PsychrometricData;
      end MoistAir;
      annotation(Documentation(info = "<html>
  <p>This package contains different medium models for air:</p>
<ul>
<li><b>SimpleAir</b><br>
    Simple dry air medium in a limited temperature range.</li>
<li><b>DryAirNasa</b><br>
    Dry air as an ideal gas from Media.IdealGases.MixtureGases.Air.</li>
<li><b>MoistAir</b><br>
    Moist air as an ideal gas mixture of steam and dry air with fog below and above the triple point temperature.</li>
</ul>
</html>"));
    end Air;
  end Media;
  package Mechanics "Library of 1-dim. and 3-dim. mechanical components (multi-body, rotational, translational)"
    extends Modelica.Icons.Library2;
    annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -5, -40},{45, -70}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Ellipse(extent = {{ -90, -50},{ -80, -60}}, lineColor = {0,0,0}),Line(points = {{ -85, -55},{ -60, -21}}, color = {0,0,0}, thickness = 0.5),Ellipse(extent = {{ -65, -16},{ -55, -26}}, lineColor = {0,0,0}),Line(points = {{ -60, -21},{9, -55}}, color = {0,0,0}, thickness = 0.5),Ellipse(extent = {{4, -50},{14, -60}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -10, -34},{72, -34},{72, -76},{ -10, -76}}, color = {0,0,0})}), Documentation(info = "<HTML>
<p>
This package contains components to model the movement
of 1-dim. rotational, 1-dim. translational, and
3-dim. <b>mechanical systems</b>.
</p>
</HTML>
"));
    package MultiBody "Library to model 3-dimensional mechanical systems"
      annotation(Documentation(info = "<HTML>
<p>
Library <b>MultiBody</b> is a <b>free</b> Modelica package providing
3-dimensional mechanical components to model in a convenient way
<b>mechanical systems</b>, such as robots, mechanisms, vehicles.
Typical animations generated with this library are shown
in the next figure:
</p>
<p>
<img src=\"../Images/MultiBody/MultiBody.png\">
</p>
<p>
For an introduction, have especially a look at:
</p>
<ul>
<li> <a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide\">MultiBody.UsersGuide</a>
     discusses the most important aspects how to use this library.</li>
<li> <a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples\">MultiBody.Examples</a>
     contains examples that demonstrate the usage of this library.</li>
</ul>

<p>
Copyright &copy; 1998-2009, Modelica Association and DLR.
</p>
<p>
<i>This Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p><br>
</HTML>"));
      import SI = Modelica.SIunits;
      extends Modelica.Icons.Library;
      package UsersGuide "User's Guide of MultiBody Library"
        annotation(__Dymola_DocumentationClass = true, Documentation(info = "<HTML>
<p>
Library <b>MultiBody</b> is a <b>free</b> Modelica package providing
3-dimensional mechanical components to model in a convenient way
<b>mechanical systems</b>, such as robots, mechanisms, vehicles.
This package contains the User's Guide for the MultiBody library.
</p>
<ol>
<li>
<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial\">Tutorial</a>
gives an introduction into the most important aspects of the library.
</li>
<li>
<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Upgrade\">Upgrade</a> describes
  how to upgrade from former versions, especially from the \"old\"
  ModelicaAdditions.MultiBody library.
</li>

<li>
<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Literature\">Literature</a> provides
  references that have been used to design and implement this library.
</li>
<li>
<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Contact\">Contact</a> provides
  information about the author of the library as well as
  acknowledgments.
</li>
</ol>
</HTML>"));
        package Tutorial "Tutorial"
          annotation(__Dymola_DocumentationClass = true, Documentation(info = "<HTML>
<p>
This tutorial provides an introduction into the
MultiBody library.
</p>
<ol>
<li>
<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.OverView\">Overview of
MultiBody library</a> summarizes the most important aspects.
</li>
<li>
<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.FirstExample\">A first example</a>
  describes in detail all the steps to build a simple pendulum model.
</li>
<li>
<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures\">Loop structures</a>
  explains how to model kinematic loops, especially by analytically
  solving non-linear equations.
</li>
</ol>
</HTML>"));
          class OverView "Overview of MultiBody library"
            annotation(Documentation(info = "<HTML>
<p>
Library <b>MultiBody</b> is a <b>free</b> Modelica package providing
3-dimensional mechanical components to model in a convenient way
<b>mechanical systems</b>, such as robots, mechanisms, vehicles.
A basic feature is that all components have <b>animation</b> information
with appropriate default sizes and colors. A typical screenshot of the
animation of a double pendulum is shown in the figure below, together
with its schematic.
</p>
<IMG SRC=\"../Images/MultiBody/Examples/Elementary/DoublePendulumSmall.png\"> &nbsp;&nbsp;
<IMG SRC=\"../Images/MultiBody/Tutorial/DoublePendulumDiagramSmall.png\"
  ALT=\"double pendulum (diagram layer)\">
<p>
Note, that all components - the coordinate system of the world frame,
the gravity acceleration vector, the revolute joints and the
bodies - are visualized in the animation.<br>
This library replaces the long available ModelicaAdditions.MultiBody
library, since it is much more easier to use and more powerful.
The main features of the library are:
</p>
<ul>
<li>About <b>60 main components</b>, i.e., joint, force, part,
    body, sensor and visualizer components that are ready to use
    and have useful default animation properties. One-dimensional
    force laws can be defined with components of the Modelica.Mechanics.Rotational
    and of the Modelica.Mechanics.Translational library and can be
    connected via available flange connectors to MultiBody
    components.
<li>About <b>75 functions</b> to operate in a convenient way on
    orientation objects, e.g., to transform vector quantities between
    frames, or compute the orientation object of a planar rotation.
    The basic idea is to hide the actual definition of an <b>orientation</b>
    by providing essentially an <b>Orientation</b> type together with
    <b>functions</b> operating on instances of this type. Orientation
    objects based on a 3x3 transformation matrix and on quaternions
    are provided. As a side effect, the equations in all other
    components are simpler and easier to understand.</li>
<li> <b>A World model</b> has to be present in every model on top
     level. Here the gravity field is defined (currently: no gravity,
     uniform gravity, point gravity), the visualization of the
     world coordinate system and default settings for animation.
     If a world model is not present, it is automatically provided
     together with a warning message.
<li><b>Built-in animation properties</b> of all components, such as
    joints, forces, bodies, sensors. This allows an easy visual
    check of the constructed model. Animation of every component
    can be switched off via a parameter. The animation of a complete
    system can be switched off via one parameter in the <b>world</b>
    model. If animation is switched off, all equations related
    to animation are removed from the generated code. This is especially
    important for real-time simulation.</li>
<li><b>Automatic handling of kinematic loops</b>.
    Components can be connected together in a nearly arbitrary fashion.
    It does not matter whether components are flipped. This does not
    influence the efficiency. If kinematic loop structures occur,
    this is automatically handeled in an efficient way by a new
    technique to transform a certain class of overdetermined sets of
    differential algebraic equations symbolically to a system where
    the number of equations and unknowns are the same (the user need
    <b>not</b> cut loops with special cut-joints to construct a
    tree-structure). </li>
<li><b>Automatic state selection from joints and bodies</b>.
    Most joints and all bodies have potential states. A Modelica
    translator, such as Dymola, will use the generalized coordinates
    of joints as states if possible. If this is not possible,
    states are selected from body coordinates. As a consequence,
    strange joints with 6 degrees of freedom are not necessary
    to define a body moving freely in space. An advanced user
    may select states manually from the <b>Advanced</b> menu of the
    corresponding components or use a Modelica parameter modification
    to set the \"stateSelect\" attribute directly. </li>
<li> <b>Analytic solution of kinematic loops</b>. The non-linear
    equations occuring in kinematic loops are solved <b>analytically</b>
    for a large class of mechanisms, such as a 4 bar mechanism,
    a slider-crank mechanism or a MacPherson suspension. This is performed
    by constructing such loops with assembly joints JointXXX,
    available in the Modelica.Mechanics.MultiBody.Joints package. Assembly joints consist
    of 3 joints that have together 6 degrees of freedom, i.e., no
    constraints.They do not have potential states. When the motion
    of the two frame connectors are provided, a non-linear system of
    equation is solved analytically to compute the motion of the 3 joints.
    Analytic loop handling is especially important for real-time simulation.</li>
<li> <b>Line force components may have mass</b>.
     Masses of line force components are located on the line on which
     the force is acting. They approximate the mass properties of
     a real physical device by one or two point masses. For example,
     a spring has often significant mass that has to be taken into
     account. If masses are set to zero, the additional code to handle
     these point masses is removed. If the masses are taken into
     account, the calculation overhead is small (the reason is that
     the occuring kinematic loops are analytically solved).<br>
     Note, in this Beta-release, not all provided line force
     components have already an optional mass. This will be fixed
     in the next release.</li>
<li> <b>Force components may be connected directly together</b>, e.g.,
     3-dimensional springs in series connection. Usually,
     multi-body programs have the restriction that force components
     can only be connected between two bodies. Such restrictions are
     not present in the Modelica multi-body library, since it is
     a fully object-oriented, equation based library. Usually, if
     force components are connected directly together, non-linear
     systems of equations occur. The advantage is often, that this
     may avoid stiff systems that would occur if a small mass has
     to be put in between the two force elements.</li>
<li><b>Initialization definition is available via menus</b>.
    Initialization of states in joints and bodies can be
    performed in the parameter menu, <b>without</b> typing Modelica
    statements. For non-standard initialization, the usual
    Modelica commands can be used.</li>
<li><b>Multi-body specific error messages</b>. Annotations
    and assert statements have been introduced that provide
    in many cases warning or error messages that are related
    to the library components (and not to specific equations
    as it is usual in Modelica libraries). This requires
    appropriate tool support, as it is, e.g., available in Dymola.</li>
<li><b>Inverse models</b> of mechanical systems can be easily
    defined by using motion generators, e.g.,
    Modelica.Mechanics.Rotational.Position. Also, non-standard
    inverse models can be generated, e.g., when elasticity is present
    it might be necessary to differentiate equations several times.</li>
</ul>
</HTML>
"));
          end OverView;
          class FirstExample "A first example"
            annotation(Documentation(info = "<HTML>
<p>
As a first example it shall be demonstrated how to build up, simulate
and animate a <b>simple pendulum</b>.
</p>
<p>
A simple pendulum consisting of a <b>body</b> and a <b>revolute</b> joint
with <b>linear damping</b> in the joint, is first build-up as
Modelica composition diagram, resulting in:
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/FirstExample/PenulumSchematic1.png\"
ALT=\"Modelica composition diagram of simple pendulum\">
</p>
<p>
In the following figure the location of the used
model components is shown. Drag these components in the diagram layer
and connect them according to the figure:
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/FirstExample/PenulumSchematic2.png\">
</p>
<p>
Every model that uses model components from the MultiBody library
must have an instance of the Modelica.Mechanics.MultiBody.World model on highest level.
The reason is that in the world object the gravity field is defined
(uniform gravity or point gravity), as well as the default sizes of
animation shapes and this information is reported to all used
components. If the World object is missing, a warning message is
printed and an instance of the World object with default settings is
automatically utilized (this feature is defined with annotations and
is, e.g., supported by Dymola).
</p>
<p>
In a second step the parameters of the dragged components need to be
defined. Some parameters are vectors that have to be defined with respect
to a local coordinate system of the corresponding component. The easiest
way to perform this is to define a <b>reference configuration</b> of your
multi-body model: In this configuration, the relative coordinates of
all joints are zero. This means that all coordinate systems on all
components are parallel to each other. Therefore, this just means
that all vectors are resolved in the world frame in this configuration.
</p>
<p>
The reference configuration for the simple pendulum shall be defined
in the following way: The y-axis of the world frame is directed
upwards, i.e., the opposite direction of the gravity acceleration.
The x-axis of the world frame is orthogonal to it. The revolute joint
is placed in the origin of the world frame. The rotation axis of the revolute
joint is directed along the z-axis of the world frame. The body is
placed on the x-axis of the world frame (i.e., the rotation angle of the
revolute joint is zero, when the body is on the x-axis).
In the following figures the definition of this reference configuration
is shown in the parameter menus of the revolute joint and the body:
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/FirstExample/ActuatedRevolute.png\">
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/FirstExample/Body.png\">
</p>
<p>
Translate and simulate the model, e.g., with Dymola.
Automatically, all defined components are visualized in
an animation using default absolute or relative sizes
of the components. For example, a body is visualized as
a sphere and as a cylinder. The default size of the sphere is defined
as parameter in the world object. You may change this size
in the \"Animation\" parameter menu of the body (see parameter menu
above). The default size of the cylinder is defined relatively
to the size of the sphere (half of the sphere size).
With default settings, the following animation is defined:
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/FirstExample/Pendulum.png\">
</p>
<p>
The world coordinate system is visualized as coordinate system
with axes labels. The direction of the gravity acceleration
vector is shown as green arrow. The red cylinder represents
the rotation axis of the revolute joint and the light blue
shapes represent the body. The center of mass of the body is
in the middle of the light blue sphere.
</p>
</HTML>
"));
          end FirstExample;
          package LoopStructures "Loop structures"
            class Introduction "Introduction"
              annotation(__Dymola_DocumentationClass = true, Documentation(info = "<HTML>
<p>
In principal, now special action is needed, if
loop structures occur (contrary to the ModelicaAdditions.MultiBody library).
An example is presented in the figure below. It is available as
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Fourbar1\">
MultiBody.Examples.Loops.Fourbar1</a>
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/LoopStructures/Fourbar1a.png\">
</p>
<p>
This mechanism consists of 6 revolute joints, 1 prismatic joint and forms a
kinematical loop.
It has has one degree of freedom. In the next figure the default animation
is shown. Note, that the axes of the revolute joints are represented by the
red cylinders and that the axis of the prismatic joint is represented by the
red box on the lower right side.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/LoopStructures/Fourbar1b.png\" width=\"205\" height=\"107\">
</p>
<p>
Whenever loop structures occur, non-linear algebraic
equations are present on \"position level\". It is then usually not possible by
structural analysis to select states during translation (which is possible for
non-loop structures). In the example above, Dymola detects a non-linear
algebraic loop of 57 equations and reduces this to a system of 7 coupled
algebraic equations. Note, that this is performed without using any
\"cut-joints\" as it is usually done in multi-body programs, but by just
appropriate symbolic equation manipulation. Via the dynamic dummy derivative
method the generalized coordinates on position and velocity level from one of
the 7 joints are dynamically selected as states during simulation. Whenever,
these two states are no longer appropriate, states from one of the other
joints are selected during simulation.
</p>
<p>
The efficiency of loop structures can usually be
enhanced, if states are statically fixed at translation time. For this
mechanism, the generalized coordinates of joint j1 (i.e., the
rotation angle of the revolute joint and its derivative) can always be used as
states. This can be stated by setting parameter \"enforceStates = <b>true</b>\"
in the \"Advanced\" menu of the desired joint. This flag sets the attribute
stateSelect of the generalized coordinates of the coresponding joint to
\"StateSelect.always\". When setting this flag to <b>true</b> for joint j1 in
the four bar mechanism, Dymola detects a non-linear algebraic loop of 40
equations and reduces this to a system of 5 coupled non-linear algebraic
equations.
</p>
<p>
In many mechanisms it is possible to solve the non-linear algebraic
equations analytically. For a certain class of systems this can
be performed also with the MultiBody library. This technique
is described in section
\"<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.AnalyticLoopHandling\">Analytic loop handling</a>\".
</p>
</HTML>"));
            end Introduction;
            class PlanarLoops "Planar loops"
              annotation(Documentation(info = "<HTML>
<p>
In the figure below, the model of a V6 engine is
shown that has a simple combustion model. It is available as
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Loops.EngineV6\">
MultiBody.Examples.Loops.EngineV6</a>.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/LoopStructures/EngineV6_1.png\">
</p>
<p>
The Modelica schematic of one cylinder
is given in the figure below. Connecting 6 instances of this
cylinder appropriately together results in the engine schematic displayed
above.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/LoopStructures/EngineV6_2.png\">
</p>
<p>
In the next figure the animation of the engine is
shown. Every cylinder consists essentially of 1 prismatic and 2 revolute
joints that form a planar loop, since the axes of the two revolute joints are
parallel to each other and the axis of the prismatic joint is orthogonal to
the revolute joint axes. All 6 cylinders together form a coupled set of 6
loops that have together 1 degree of freedom.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/LoopStructures/EngineV6_3.png\" width=\"303\" height=\"136\">
</p>
<p>
All planar loops, and especially the engine, result in
a DAE (= Differential-Algebraic Equation system)
that does not have a unique solution. The reason is that, e.g., the cut
forces in direction of the axes of the revolute joints cannot be uniquely
computed. Any value fulfills the DAE equations. This is a structural property
that is determined by the symbolic algorithms. Since they detect that the DAE is
structurally singular, a further processing is not possible. Without
additional information it is also impossible that the symbolic algorithms
could be enhanced because if the axes of rotations of the revolute joints are
only slightly changed such that they are no longer parallel to each other, the
planar loop can no longer move and has 0 degrees of freedom. Algorithms based
on pure structural information cannot distinguish these two cases.
</p>
<p>
The usual remedy is to remove superfluous constraints,
e.g., along the axis of rotation of <b>one</b> revolute joint. Since this is
not easy for an inexperienced modeler, the special joint:
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.RevolutePlanarLoopConstraint\">
RevolutePlanarLoopConstraint</a> is provided that removes these constraints.
Exactly one revolute joint in a every planar loop must be replaced by this
joint type. In the engine example, this special joinst is used for
the revolute joint B2 in the cylinder model above. The icon of the joint is
slightly different to other revolute joints to visualize this case.
</p>
<p>
If a modeler is not aware of the problems with planar
loops and models them without special consideration, a Modelica
translator, such as Dymola, displays an error
message and points out that a planar loop may be the reason and suggests to
use the RevolutePlanarLoopConstraint joint.
This error message is due to an annotation in
the Frame connector.
</p>
<pre>
  <b>connector</b> Frame
     ...
     <b>flow</b> SI.Force f[3] <b>annotation</b>(unassignedMessage=\"..\");
  <b>end</b> Frame;
</pre>
<p>
If no assignment can be found for some
forces in a connector, the \"unassignedMessage\" is displayed. In most cases the
reason for this is a planar loop or two joints that constrain the same motion.
Both cases are discussed in the error message.
</p>
<p>
Note, that the non-linear algebraic equations occurring
in planar loops can be solved analytically in most cases and therefore it is
highly recommended to use the techniques discussed in section
\"<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.AnalyticLoopHandling\">Analytic loop handling</a>\"
for such systems.
</p>
</HTML>"));
            end PlanarLoops;
            class AnalyticLoopHandling "Analytic loop handling"
              annotation(Documentation(info = "<HTML>
It is well known that the non-linear
algebraic equations of most mechanical loops in technical devices can be
solved analytically. It is, however, difficult to perform this fully
automatically and therefore none of the commercial, general purpose multi-body
programs, such as MSC ADAMS, LMS DADS, SIMPACK, have this feature.
These programs solve loop structures with pure numerical methods. Multi-body
programs that are designed for real-time simulation of the dynamics of
specific vehicles, such as ve-DYNA, usually contain manual implementations
of a particular multi-body system (the vehicle) where the occurring loops are
either analytically solved, if this is possible, or are treated by table
look-up where the tables are constructed in a pre-processing phase. Without
these features the required real-time capability would be difficult to
achieve.
</p>
<p>
In a series of papers and dissertations
Prof. Hiller and his group in Duisburg, Germany,
have developed systematic methods to handle mechanical
loops analytically. The \"characteristic pair of joints\" method
basically cuts a loop at two joints and uses geometric
invariants to reduce the number of algebraic
equations, often down to one equation that can be solved analytically. Also
several multi-body codes have been developed that are based on this method,
e.g., MOBILE. Besides the very desired feature to solve non-linear
algebraic equations analytically, i.e., efficiently and in a robust way, there
are several drawbacks: It is difficult to apply this method automatically.
Even if this would be possible in a good way, there is always the problem that
it cannot be guaranteed that the statically selected states lead to no
singularity during simulation. Therefore, the \"characteristic pair of joints\"
method is usually manually applied which requires know-how and experience.
</p>
<p>
In the MultiBody library the \"characteristic pair of
joints\" method is supported in a restricted form such that it can be applied
also by non-specialists. The idea is to provide aggregations of joints in package
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies\">
MultiBody.Joints.Assemblies</a>.
as one object that either have <b>6</b> degrees of freedom or
<b>3</b> degrees of freedom (for usage in planar loops).
</p>
<p>
As an example, a variant of the four bar mechanism is given in
the figure below.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/LoopStructures/FourbarAnalytic1.png\">
</p>
<p>
Here, the mechanism is modeled with one revolute
joint, two spherical joints and one prismatic joint.
In the figure below, the two spherical joints
and the prismatic joint are collected together in an assembly object
called \"jointSSP\" from
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies.JointSSP\">
MultiBody.Joints.Assemblies.JointSSP</a>.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/LoopStructures/FourbarAnalytic2.png\">
</p>
<p>
The JointSSP joint aggregation has a frame at the
left side of the left spherical joint (frame_a) and a frame at the right side
of the prismatic joint (frame_b). JointSSP, as all other objects from the
Joints.Assemblies package, has the property, that the <b>generalized
coordinates, and all other frames
defined in the assembly, can be calculated given the movement of frame_a and
of frame_b</b>. This is performed by <b>analytically</b> solving non-linear
systems of equations (details are given in section xxx). From a
structural point of view, the equations in an assembly object are written in
the form
</p>
<p align=\"center\">
  <b>q</b> = <b>
  f</b><sub>1</sub>(<b>r</b><sup>a</sup>, <b>R</b><sup>a</sup>,<b> r</b><sup>b</sup>,
  <b>R</b><sup>b</sup>)
</p>
<p>
where<b> r</b><sup>a</sup>, <b>R</b><sup>a</sup>,<b>
r</b><sup>b</sup>, <b>R</b><sup>b</sup><b> </b>are the variables defining the
position and orientation of the frame_a and frame_b connector,
<b>q</b> are the generalized positional coordinates inside the
assembly, e.g., the angle of a revolute joint. Given angle
<span style=\"font-family:Symbol\">j</span> of revolute joint j1 from the four
bar mechanism, frame_a and frame_b of the assembly object can be computed by a
forward recursion
</p>
<p align=\"center\">
(<b>r</b><sup>a</sup>,
<b>R</b><sup>a</sup>,<b> r</b><sup>b</sup>, <b>R</b><sup>b</sup>) = <b>f</b>(<span style=\"font-family:Symbol\">j)</span>
</p>
<p>
Since this is a structural property, the
symbolic algorithms can automatically select <span style=\"font-family:Symbol\">
j</span> and its derivative as states and then all positional variables can be
computed in a forwards sequence. It is now understandable that a Modelica
translator, such as Dymola, can
transform the equations of the four bar mechanism to a recursive sequence of
statements that has no non-linear algebraic loops anymore(remember,
the previous \"straightforward\" solution with 6 revolute joints and 1
prismatic joint has a nonlinear system of equations of order 5).
</p>
<p>
The aggregated joint
objects consist of a combination of either a revolute or prismatic joint and
of a rod that has either two spherical joints at its two ends or a spherical
and a universal joint, respectively. For all combinations, analytic solutions
can be determined. For planar loops, combinations of 1, 2 or 3 revolute joints
with parallel axes and of 2 or 1 prismatic joint with axes that are orthogonal
to the revolute joints can be treated analytically. The currently supported
combinations are listed in the table below.
The missing combinations (such as JointSUP
or Joint RPP) will be added in one of the next releases.
</p>
<div align=\"center\">
    <table border=\"1\" cellspacing=\"0\" cellpadding=\"0\">
      <tr>
        <td colspan=\"2\"> <b>3-dimensional Loops:</b></td>
      </tr>
      <tr>
        <td valign=\"top\">JointSSR</td>
        <td valign=\"top\">Spherical - Spherical - Revolute</td>
      </tr>
      <tr>
        <td valign=\"top\">JointSSP</td>
        <td valign=\"top\">Spherical - Spherical - Prismatic</td>
      </tr>
      <tr>
        <td valign=\"top\">JointUSR</td>
        <td valign=\"top\">Universal - Spherical - Revolute</td>
      </tr>
      <tr>
        <td valign=\"top\">JointUSP</td>
        <td valign=\"top\">Universal - Spherical - Prismatic</td>
      </tr>
      <tr>
        <td valign=\"top\">JointUPS</td>
        <td valign=\"top\">Universal - Prismatic - Spherical</td>
      </tr>
      <tr>
        <td colspan=\"2\"><b>Planar Loops:</b></td>
      </tr>
      <tr>
        <td valign=\"top\">JointRRR</td>
        <td valign=\"top\">Revolute - Revolute - Revolute</td>
      </tr>
      <tr>
        <td valign=\"top\">JointRRP</td>
        <td valign=\"top\">Revolute - Revolute - Prismatic</td>
      </tr>
</table>
</div>
<p>
On first view this seems to be quite restrictive. However, mechanical devices are usually built up with rods connected by spherical joints on each end,
and additionally with revolute and prismatic joints.
Therefore, the combinations of the above table occur frequently.
The universal joint is usually not present in actual devices but is used
(a) if two JointXXX components can be connected such that a revolute
and a universal joint together form a spherical joint
and (b) if the orientation of the connecting rod between two
spherical joints is needed, e.g., since a body shall be attached.
In this case one of the spherical joints might be replaced by a
universal joint. This approximation is fine as long as the mass
and inertia of the rod is not significant.
</p>
<p>
Let us discuss item (a) in more detail: The
MacPherson suspension in the next figure
is from the Modelica VehicleDynamics library.
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/LoopStructures/MacPherson1.png\">
</p>
It has three frame
connectors. The lower left one (frame_C) is fixed in the vehicle chassis. The
upper left one (frame_S) is driven by the steering mechanism, i.e., the
movement of both frames are given. The frame connector on the right (frame_U)
drives the wheel. The three frames are connected by a mechanism consisting
essentially of two rods with spherical joints on both ends. These are built up
by a jointUPS and a jointSSR assembly.
As can be seen, the universal joint from the jointUPS
assembly is connected to the revolute joint of the jointSSR assembly.
Therefore, we have 3 revolute joints connected together at one point and if
the axes of rotations are chosen appropriately, this describes a spherical
joint. In other words, the two connected assemblies define the desired two
rods with spherical joints on each ends.
</p>
<p>
The movement of the chassis, frame_C, is computed
somewhere else. When the generalized coordinates of revolute joint
\"innerJoint\" (lower left part in figure) are used as states, then frame_a and
frame_b of the jointUPS joint can be calculated. After the non-linear loop
with jointUPS is (analytically) solved, all frames on this assembly are
known, especially,
the one connected to frame_b of the jointSSR assembly. Since frame_b of
jointSSR is connected to frame_S which is computed from the steering
mechanism, again the two required frame movements of the jointSSR assembly are
calculated, meaning in turn that also all other frames on the jointSSR
assembly can be computed, especially, the one connected to frame_U that drives
the wheel. From this analysis it is clear that a tool is able to solve these
coupled loops analytically.
</p>
<p>
Another example is the model of the V6 engine,
see next figure for an animation view and the original definition
of one cylinder with elementary joints.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/LoopStructures/EngineV6_3.png\" width=\"303\" height=\"136\">
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/LoopStructures/EngineV6_2.png\">
</p>

<p>
It is sufficient to rewrite the basic cylinder model
by replacing the joints with a JointRRP object that has two
revolute and one prismatic joint, see next figure.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/LoopStructures/EngineV6_4.png\">
</p>
<p>
Since 6
cylinders are connected together, 6 coupled loops with 6 JointRRP objects are
present. This model is available as
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Loops.EngineV6_analytic\">
MultiBody.Examples.Loops.EngineV6_analytic</a>.
</p>
<p>
The composition diagram of the connected 6 cylinders is
shown in the next figure
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/LoopStructures/EngineV6_1.png\">
</p>
<p>
It can be seen that the revolute joint
of the crank shaft (joint \"bearing\" in left part of figure) might be
selected as degree of freedom. Then the 4 connector frames of all cylinders
can be computed. As a result the computations of the cylinders are decoupled
from each other. Within one cylinder
the position of frame_a and frame_b of the jointRRP assembly
can be computed and therefore the generalized coordinates of the two revolute
and the prismatic joint in the jointRRP object can be determined. From this
analysis it is not surprising that a Modelica translator, such as Dymola,
is able to transform the DAE
equations into a sequential evaluation without any non-linear loop.
Compare this nice result with the model using only elementary joints
that leads to a DAE with 6 algebraic loops and 5 non-linear equations per
loop. Additionally, a linear system of equations of order 43 is present.
The simulation time is about 5 times faster with the analytic loop handling.
</p>
</HTML>
"));
            end AnalyticLoopHandling;
            annotation(Documentation(info = "<HTML>
<p>
The MultiBody library has the feature that all components
can be connected together in a nearly arbitrary fashion.
Therefore, kinematic loop structures pose in principal
no problems. In this section several examples are given,
the special treatment of planar loops is discussed and it is explained
how a kinematic loop structure can be modeled such that the
occuring non-linear algebraic equation systems are solved
analytically. There are the following sub-chapters:
</p>
<ol>
<li>
<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.Introduction\">Introduction</a>
</li>
<li>
<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.PlanarLoops\">Planar loops</a>.
</li>
<li>
<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.AnalyticLoopHandling\">Analytic loop handling</a>.
</li>
</ol>
</HTML>"));
          end LoopStructures;
        end Tutorial;
        class Upgrade "Upgrade from Former Versions"
          annotation(Documentation(info = "<HTML>
<p>
If different versions of the MultiBody library are not
compatible to each other, corresponding conversion scripts are
provided. As a result, models build with an older version
of the MultiBody library are automatically converted to the
new version when the model is loaded. The user is prompted
whether automatic conversion shall take place or not.
Problems are not to be expected. Still one should first make
a copy of such a model as backup before the conversion
is performed.
</p>
<p>
<b><font color=\"#008000\">Upgrade from ModelicaAdditions.MultiBody</font></b>
</p>
<p>
There is now also a conversion script from the \"old\"
<b>ModelicaAdditions.MultiBody</b> library to the
\"new\" Modelica.Mechanics.MultiBody library. This script is also
automatically invoked. Since the differences between the \"old\" and the
\"new\" MultiBody library are so large, not everything is
converted and it might be that some pieces have to
be adapted manually. Still, this script is useful, since
many class names, parameters and modifiers are
automatically converted.
</p>
<p>
Components from the following sublibraries
are automatically converted
to the Modelica.Mechanics.MultiBody library:
</p>
<ul>
<li> ModelicaAdditions.MultiBody.Parts</li>
<li> ModelicaAdditions.MultiBody.Joints</li>
<li> ModelicaAdditions.MultiBody.Forces</li>
<li> Part of ModelicaAdditions.MultiBody.Interfaces</li>
</ul>
<p>
Models using the ModelicaAdditions.MultiBody library
that are programmed with <b>equations</b> are only partly converted:
The Frame connectors will be converted to the \"new\"
Frame connectors of the MultiBody library, but the equations
that reference variables of the Frame connectors will
<b>not</b> be converted. For a manual conversion, the following
table might be helpful showing how the <b>variables</b> of the
\"old\" and the \"new\" <b>Frame connectors</b> are
related to each other (resolve2 and angularVelocity2 are
functions from library Modelica.Mechanics.MultiBody.Frames):
</p>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>ModelicaAdditions.MultiBody.<br>
           Interfaces.Frame_a</b></th>
    <th><b>MultiBody.Interfaces.Frame_a</b></th></tr>
<tr>
  <td valign=\"top\">frame_a.<b>r0</b></td>
  <td valign=\"top\">= frame_a.r_0 (is converted)</td>
</tr>
<tr>
  <td valign=\"top\">frame_a.<b>S</b></td>
  <td valign=\"top\">= transpose(frame_a.R)</td>
</tr>
<tr>
  <td valign=\"top\">frame_a.<b>v</b></td>
  <td valign=\"top\">= resolve2(frame_a.R, <b>der</b>(frame_a.r_0))</td>
</tr>
<tr>
  <td valign=\"top\">frame_a.<b>w</b></td>
  <td valign=\"top\">= angularVelocity2(frame_a.R)</td>
</tr>
<tr>
  <td valign=\"top\">frame_a.<b>a</b></td>
  <td valign=\"top\">= resolve2(frame_a.R, <b>der</b>(v_0)); v_0 = der(r_0)</td>
</tr>
<tr>
  <td valign=\"top\">frame_a.<b>z</b></td>
  <td valign=\"top\">= <b>der</b>(w);  w = angulaVelocity2(frame_a.R)</td>
</tr>
<tr>
  <td valign=\"top\">frame_a.<b>f</b></td>
  <td valign=\"top\">= frame_a.f (no conversion needed)</td>
</tr>
<tr>
  <td valign=\"top\">frame_a.<b>t</b></td>
  <td valign=\"top\">= frame_a.t (no conversion needed)</td>
</tr>
</table>
<p>
<b><font color=\"#008000\">Upgrade from MultiBody 0.99 (and earlier) to 1.0 (and later)</font></b>
<p>
The conversion from MultiBody 0.99 to 1.0 does not work in some rare
cases, where own components are implemented using functions of the
MultiBody.Frames package. In this case, the conversion has to be
performed manually. The changes in 1.0 with regards to 0.99 are:
</p>
<p>
The definition of the Modelica.Mechanics.MultiBody.Frames.Orientation object has changed.
In 0.99 this was just an alias type for a transformation matrix
(now Modelica.Mechanics.MultiBody.Frames.TransformationMatrices.Orientation).
In 1.0 the orientation object is a record holding the
transformation matrix from frame 1 to frame 2 and the angular
velocity of the transformation matrix resolved in frame 2.
The reason is that this allows to compute the angular velocity
in many cases by standard recursive formulas and not by
differentiation of the transformation matrix. This is usually
much more efficient. As a consequence, the following
calls in 0.99 should be changed:
</p>
<pre>
   Frames.angularVelocity1(T,der(T)) -> Frames.angularVelocity1(T)
   Frames.angularVelocity2(T,der(T)) -> Frames.angularVelocity2(T)
   Frames.from_T(T)                  -> Frames.from_T2(T,der(T))
</pre>
</HTML>
"));
        end Upgrade;
        class Literature "Literature"
          annotation(Documentation(info = "<html>
<ul>
<li>
Technical details of this library are described in the
20 page paper:
<dl>
<dt>Otter M., Elmqvist H., and Mattsson S.E.:</dt>
<dd> <b>The New Modelica MultiBody Library</b>.
     Modelica 2003 Conference, Link&ouml;ping, Sweden,
     pp. 311-330, Nov. 3-4, 2003.
     Download from:
     <a href=\"http://www.modelica.org/Conference2003/papers/h37_Otter_multibody.pdf\">http://www.modelica.org/Conference2003/papers/h37_Otter_multibody.pdf</a>
     </dd>
</dl>
</li>
<li>
The method how to describe drive trains with 1-dimensional mechanics
and to mount them on 3-dimensional components without neglecting
dynamical effects is described in:
<dl>
<dt>Schweiger C., and Otter M.:</dt>
<dd> <b>Modelling 3-dim. Mechanical Effects of 1-dim. Powertrains</b>.
     Modelica 2003 Conference, Link&ouml;ping, Sweden,
     pp. 149-158, Nov. 3-4, 2003.
     Download from:
<a href=\"http://www.modelica.org/Conference2003/papers/h06_Schweiger_powertrains_v5.pdf\">http://www.modelica.org/Conference2003/papers/h06_Schweiger_powertrains_v5.pdf</a>
     </dd>
</dl>
</li>
<li> The method to solve a certain class of kinematic loops
     analytically is based on:
<dl>
<dt>Woernle C.:</dt>
<dd><b>Ein systematisches Verfahren zur Aufstellung der geometrischen
    Schliessbedingungen in kinematischen Schleifen mit Anwendung
    bei der R&uuml;ckw&auml;rtstransformation f&uuml;r
    Industrieroboter.</b><br>
    Fortschritt-Berichte VDI, Reihe 18, Nr. 59, Duesseldorf: VDI-Verlag 1988,
    ISBN 3-18-145918-6.<br>&nbsp;</dd>
<dt>Hiller M., and Woernle C.:</dt
<dd><b>A Systematic Approach for Solving the Inverse Kinematic
    Problem of Robot Manipulators</b>.<br>
    Proceedings 7th World Congress Th. Mach. Mech., Sevilla 1987. </dd>
</dl>
</li>
</ul>
</html>
"));
        end Literature;
        class Contact "Contact"
          annotation(Documentation(info = "<html>
<dl>
<dt><b>Library Officer:</b>
<dd><a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a><br>
    Deutsches Zentrum f&uuml;r Luft und Raumfahrt e.V. (DLR)<br>
    Institut f&uuml;r Robotik und Mechatronik<br>
    Abteilung f&uuml;r Entwurfsorientierte Regelungstechnik<br>
    Postfach 1116<br>
    D-82230 Wessling<br>
    Germany<br>
    email: <A HREF=\"mailto:Martin.Otter@dlr.de\">Martin.Otter@dlr.de</A><br>
</dl>
<p><b>Acknowledgements:</b></p>
<ul>
<li> The central idea to handle a certain class of overdetermined, consistent
     set of differential algebraic equations (i.e., there are more equations than
     unknowns) with symbolic transformation algorithms was developed together
     with Hilding Elmqvist and Sven Erik Mattsson from Dynasim AB, Lund, Sweden.
     The MultiBody library is heavily relying on this feature which is a
     prerequiste for a truely \"object-oriented\" multi-body systems library,
     where components can be connected together in any meaningful way.<li>
<li> The Examples.Loops.EngineV6 demo of a six cylinder V6 engine with
     6 planar loops and 1 degree of freedom is from Hilding Elmqvist and
     Sven Erik Mattsson.</li>
<li> Modelica.Mechanics.MultiBody.Forces.LineForceWithMass is based on model
     \"RelativeDistance\" from the Modelica VehicleDynamics library of
     Johan Andreasson from Royal Institute of Technology, Stockholm, Sweden.</li>
<li> The 1-dim. components (Parts.Rotor1D, Parts.BevelGear1D, Mounting1D) and
     Joints.GearConstraints are from Christian Schweiger.</li>
<li> The design of this library is based on work carried out
     in the EU RealSim project (Real-time Simulation for Design of
     Multi-physics Systems) funded by the European Commission within
     the Information Societies Technology (IST) programme under
     contract number IST 1999-11979.
     </li>
</ul>
</html>
"));
        end Contact;
      end UsersGuide;
      model World "World coordinate system + gravity field + default animation definition"
        import SI = Modelica.SIunits;
        import Modelica.Mechanics.MultiBody.Types.GravityTypes;
        import Modelica.Mechanics.MultiBody.Types;
        Interfaces.Frame_b frame_b "Coordinate system fixed in the origin of the world frame" annotation(Placement(transformation(extent = {{84, -16},{116,16}}, rotation = 0)));
        annotation(defaultComponentName = "world", defaultComponentPrefixes = "inner", missingInnerMessage = "No \"world\" component is defined. A default world
component with the default gravity field will be used
(g=9.81 in negative y-axis). If this is not desired,
drag Modelica.Mechanics.MultiBody.World into the top level of your model.", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -100, -118},{ -100,61}}, color = {0,0,0}, thickness = 0.5),Polygon(points = {{ -100,100},{ -120,60},{ -80,60},{ -100,100},{ -100,100}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -119, -100},{59, -100}}, color = {0,0,0}, thickness = 0.5),Polygon(points = {{99, -100},{59, -80},{59, -120},{99, -100}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -140,165},{140,103}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{95, -113},{144, -162}}, lineColor = {0,0,0}, textString = "%label1"),Text(extent = {{ -170,127},{ -119,77}}, lineColor = {0,0,0}, textString = "%label2"),Line(points = {{ -56,78},{ -56, -26}}, color = {0,0,255}),Polygon(points = {{ -68, -26},{ -56, -66},{ -44, -26},{ -68, -26}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{2,78},{2, -26}}, color = {0,0,255}),Polygon(points = {{ -10, -26},{2, -66},{14, -26},{ -10, -26}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{66,80},{66, -26}}, color = {0,0,255}),Polygon(points = {{54, -26},{66, -66},{78, -26},{54, -26}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<HTML>
<p>
Model <b>World</b> represents a global coordinate system fixed in
ground. This model serves several purposes:
<ul>
<li> It is used as <b>inertial system</b> in which
     the equations of all elements of the MultiBody library
     are defined.</li>
<li> It is the world frame of an <b>animation window</b> in which
     all elements of the MultiBody library are visualized.</li>
<li> It is used to define the <b>gravity field</b> in which a
     multi-body model is present. Default is a uniform gravity
     field where the gravity acceleration vector g is the
     same at every position. Additionally, a point gravity field
     can be selected.</li>
<li> It is used to define <b>default settings</b> of animation properties
     (e.g. the diameter of a sphere representing by default
     the center of mass of a body, or the diameters of the cylinders
     representing a revolute joint).</li>
<li> It is used to define a <b>visual representation</b> of the
     world model (= 3 coordinate axes with labels) and of the defined
     gravity field.<br>
    <IMG SRC=\"../Images/MultiBody/world.png\" ALT=\"MultiBodys.World\">
</li>
</ul>
<p>
Since the gravity field function is required from all bodies with mass
and the default settings of animation properties are required
from nearly every component, exactly one instance of model World needs
to be present in every model on the top level. The basic declaration
needs to be:
</p>
<pre>
    <b>inner</b> Modelica.Mechanics.MultiBody.World world
</pre>
<p>
Note, it must be an <b>inner</b> declaration with instance name <b>world</b>
in order that this world object can be accessed from all objects in the
model. When dragging the \"World\" object from the package browser into
the diagram layer, this declaration is automatically generated
(this is defined via annotations in model World).
</p>
<p>
All vectors and tensors of a mechanical system are resolved in a
frame that is local to the corresponding component. Usually,
if all relative joint coordinates vanish, the local frames
of all components are parallel to each other, as well as to the
world frame (this holds as long as a Parts.FixedRotation,
component is <b>not</b> used). In this \"reference configuration\"
it is therefore
alternatively possible to resolve all vectors in the world
frame, since all frames are parallel to each other.
This is often very convenient. In order to give some visual
support in such a situation, in the icon of a World instance
two axes of the world frame are shown and the labels
of these axes can be set via parameters.
</p>
</HTML>
"));
        parameter Boolean enableAnimation = true "= true, if animation of all components is enabled";
        parameter Boolean animateWorld = true "= true, if world coordinate system shall be visualized" annotation(Dialog(enable = enableAnimation));
        parameter Boolean animateGravity = true "= true, if gravity field shall be visualized (acceleration vector or field center)" annotation(Dialog(enable = enableAnimation));
        parameter Types.AxisLabel label1 = "x" "Label of horizontal axis in icon";
        parameter Types.AxisLabel label2 = "y" "Label of vertical axis in icon";
        parameter Types.GravityTypes gravityType = GravityTypes.UniformGravity "Type of gravity field" annotation(Evaluate = true);
        parameter SI.Acceleration g = 9.81 "Constant gravity acceleration" annotation(Dialog(enable = gravityType == GravityTypes.UniformGravity));
        parameter Types.Axis n = {0, -1,0} "Direction of gravity resolved in world frame (gravity = g*n/length(n))" annotation(Evaluate = true, Dialog(enable = gravityType == Modelica.Mechanics.MultiBody.Types.GravityTypes.UniformGravity));
        parameter Real mue(unit = "m3/s2", min = 0) = 398600000000000.0 "Gravity field constant (default = field constant of earth)" annotation(Dialog(enable = gravityType == Types.GravityTypes.PointGravity));
        parameter Boolean driveTrainMechanics3D = true "= true, if 3-dim. mechanical effects of Parts.Mounting1D/Rotor1D/BevelGear1D shall be taken into account";
        parameter SI.Distance axisLength = nominalLength / 2 "Length of world axes arrows" annotation(Dialog(tab = "Animation", group = "if animateWorld = true", enable = enableAnimation and animateWorld));
        parameter SI.Distance axisDiameter = axisLength / defaultFrameDiameterFraction "Diameter of world axes arrows" annotation(Dialog(tab = "Animation", group = "if animateWorld = true", enable = enableAnimation and animateWorld));
        parameter Boolean axisShowLabels = true "= true, if labels shall be shown" annotation(Dialog(tab = "Animation", group = "if animateWorld = true", enable = enableAnimation and animateWorld));
        input Types.Color axisColor_x = Modelica.Mechanics.MultiBody.Types.Defaults.FrameColor "Color of x-arrow" annotation(Dialog(tab = "Animation", group = "if animateWorld = true", enable = enableAnimation and animateWorld));
        input Types.Color axisColor_y = axisColor_x annotation(Dialog(tab = "Animation", group = "if animateWorld = true", enable = enableAnimation and animateWorld));
        input Types.Color axisColor_z = axisColor_x "Color of z-arrow" annotation(Dialog(tab = "Animation", group = "if animateWorld = true", enable = enableAnimation and animateWorld));
        parameter SI.Position gravityArrowTail[3] = {0,0,0} "Position vector from origin of world frame to arrow tail, resolved in world frame" annotation(Dialog(tab = "Animation", group = "if animateGravity = true and gravityType = UniformGravity", enable = enableAnimation and animateGravity and gravityType == GravityTypes.UniformGravity));
        parameter SI.Length gravityArrowLength = axisLength / 2 "Length of gravity arrow" annotation(Dialog(tab = "Animation", group = "if animateGravity = true and gravityType = UniformGravity", enable = enableAnimation and animateGravity and gravityType == GravityTypes.UniformGravity));
        parameter SI.Diameter gravityArrowDiameter = gravityArrowLength / defaultWidthFraction "Diameter of gravity arrow" annotation(Dialog(tab = "Animation", group = "if animateGravity = true and gravityType = UniformGravity", enable = enableAnimation and animateGravity and gravityType == GravityTypes.UniformGravity));
        input Types.Color gravityArrowColor = {0,230,0} "Color of gravity arrow" annotation(Dialog(tab = "Animation", group = "if animateGravity = true and gravityType = UniformGravity", enable = enableAnimation and animateGravity and gravityType == GravityTypes.UniformGravity));
        parameter SI.Diameter gravitySphereDiameter = 12742000 "Diameter of sphere representing gravity center (default = mean diameter of earth)" annotation(Dialog(tab = "Animation", group = "if animateGravity = true and gravityType = PointGravity", enable = enableAnimation and animateGravity and gravityType == GravityTypes.PointGravity));
        input Types.Color gravitySphereColor = {0,230,0} "Color of gravity sphere" annotation(Dialog(tab = "Animation", group = "if animateGravity = true and gravityType = PointGravity", enable = enableAnimation and animateGravity and gravityType == GravityTypes.PointGravity));
        parameter SI.Length nominalLength = 1 "\"Nominal\" length of multi-body system" annotation(Dialog(tab = "Defaults"));
        parameter SI.Length defaultAxisLength = nominalLength / 5 "Default for length of a frame axis (but not world frame)" annotation(Dialog(tab = "Defaults"));
        parameter SI.Length defaultJointLength = nominalLength / 10 "Default for the fixed length of a shape representing a joint" annotation(Dialog(tab = "Defaults"));
        parameter SI.Length defaultJointWidth = nominalLength / 20 "Default for the fixed width of a shape representing a joint" annotation(Dialog(tab = "Defaults"));
        parameter SI.Length defaultForceLength = nominalLength / 10 "Default for the fixed length of a shape representing a force (e.g. damper)" annotation(Dialog(tab = "Defaults"));
        parameter SI.Length defaultForceWidth = nominalLength / 20 "Default for the fixed width of a shape represening a force (e.g. spring, bushing)" annotation(Dialog(tab = "Defaults"));
        parameter SI.Length defaultBodyDiameter = nominalLength / 9 "Default for diameter of sphere representing the center of mass of a body" annotation(Dialog(tab = "Defaults"));
        parameter Real defaultWidthFraction = 20 "Default for shape width as a fraction of shape length (e.g., for Parts.FixedTranslation)" annotation(Dialog(tab = "Defaults"));
        parameter SI.Length defaultArrowDiameter = nominalLength / 40 "Default for arrow diameter (e.g., of forces, torques, sensors)" annotation(Dialog(tab = "Defaults"));
        parameter Real defaultFrameDiameterFraction = 40 "Default for arrow diameter of a coordinate system as a fraction of axis length" annotation(Dialog(tab = "Defaults"));
        parameter Real defaultSpecularCoefficient(min = 0) = 0.7 "Default reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Defaults"));
        parameter Real defaultN_to_m(unit = "N/m", min = 0) = 1000 "Default scaling of force arrows (length = force/defaultN_to_m)" annotation(Dialog(tab = "Defaults"));
        parameter Real defaultNm_to_m(unit = "N.m/m", min = 0) = 1000 "Default scaling of torque arrows (length = torque/defaultNm_to_m)" annotation(Dialog(tab = "Defaults"));
      protected 
        parameter Integer ndim = if enableAnimation and animateWorld then 1 else 0;
        parameter Integer ndim2 = if enableAnimation and animateWorld and axisShowLabels then 1 else 0;
        parameter SI.Length headLength = min(axisLength, axisDiameter * Types.Defaults.FrameHeadLengthFraction);
        parameter SI.Length headWidth = axisDiameter * Types.Defaults.FrameHeadWidthFraction;
        parameter SI.Length lineLength = max(0, axisLength - headLength);
        parameter SI.Length lineWidth = axisDiameter;
        parameter SI.Length scaledLabel = Modelica.Mechanics.MultiBody.Types.Defaults.FrameLabelHeightFraction * axisDiameter;
        parameter SI.Length labelStart = 1.05 * axisLength;
        Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape x_arrowLine(shapeType = "cylinder", length = lineLength, width = lineWidth, height = lineWidth, lengthDirection = {1,0,0}, widthDirection = {0,1,0}, color = axisColor_x, specularCoefficient = 0) if enableAnimation and animateWorld;
        Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape x_arrowHead(shapeType = "cone", length = headLength, width = headWidth, height = headWidth, lengthDirection = {1,0,0}, widthDirection = {0,1,0}, color = axisColor_x, r = {lineLength,0,0}, specularCoefficient = 0) if enableAnimation and animateWorld;
        Modelica.Mechanics.MultiBody.Visualizers.Internal.Lines x_label(lines = scaledLabel * {[0,0;1,1],[0,1;1,0]}, diameter = axisDiameter, color = axisColor_x, r_lines = {labelStart,0,0}, n_x = {1,0,0}, n_y = {0,1,0}, specularCoefficient = 0) if enableAnimation and animateWorld and axisShowLabels;
        Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape y_arrowLine(shapeType = "cylinder", length = lineLength, width = lineWidth, height = lineWidth, lengthDirection = {0,1,0}, widthDirection = {1,0,0}, color = axisColor_y, specularCoefficient = 0) if enableAnimation and animateWorld;
        Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape y_arrowHead(shapeType = "cone", length = headLength, width = headWidth, height = headWidth, lengthDirection = {0,1,0}, widthDirection = {1,0,0}, color = axisColor_y, r = {0,lineLength,0}, specularCoefficient = 0) if enableAnimation and animateWorld;
        Modelica.Mechanics.MultiBody.Visualizers.Internal.Lines y_label(lines = scaledLabel * {[0,0;1,1.5],[0,1.5;0.5,0.75]}, diameter = axisDiameter, color = axisColor_y, r_lines = {0,labelStart,0}, n_x = {0,1,0}, n_y = { -1,0,0}, specularCoefficient = 0) if enableAnimation and animateWorld and axisShowLabels;
        Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape z_arrowLine(shapeType = "cylinder", length = lineLength, width = lineWidth, height = lineWidth, lengthDirection = {0,0,1}, widthDirection = {0,1,0}, color = axisColor_z, specularCoefficient = 0) if enableAnimation and animateWorld;
        Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape z_arrowHead(shapeType = "cone", length = headLength, width = headWidth, height = headWidth, lengthDirection = {0,0,1}, widthDirection = {0,1,0}, color = axisColor_z, r = {0,0,lineLength}, specularCoefficient = 0) if enableAnimation and animateWorld;
        Modelica.Mechanics.MultiBody.Visualizers.Internal.Lines z_label(lines = scaledLabel * {[0,0;1,0],[0,1;1,1],[0,1;1,0]}, diameter = axisDiameter, color = axisColor_z, r_lines = {0,0,labelStart}, n_x = {0,0,1}, n_y = {0,1,0}, specularCoefficient = 0) if enableAnimation and animateWorld and axisShowLabels;
        parameter SI.Length gravityHeadLength = min(gravityArrowLength, gravityArrowDiameter * Types.Defaults.ArrowHeadLengthFraction);
        parameter SI.Length gravityHeadWidth = gravityArrowDiameter * Types.Defaults.ArrowHeadWidthFraction;
        parameter SI.Length gravityLineLength = max(0, gravityArrowLength - gravityHeadLength);
        Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape gravityArrowLine(shapeType = "cylinder", length = gravityLineLength, width = gravityArrowDiameter, height = gravityArrowDiameter, lengthDirection = n, widthDirection = {0,1,0}, color = gravityArrowColor, r_shape = gravityArrowTail, specularCoefficient = 0) if enableAnimation and animateGravity and gravityType == GravityTypes.UniformGravity;
        Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape gravityArrowHead(shapeType = "cone", length = gravityHeadLength, width = gravityHeadWidth, height = gravityHeadWidth, lengthDirection = n, widthDirection = {0,1,0}, color = gravityArrowColor, r_shape = gravityArrowTail + Modelica.Math.Vectors.normalize(n) * gravityLineLength, specularCoefficient = 0) if enableAnimation and animateGravity and gravityType == GravityTypes.UniformGravity;
        parameter Integer ndim_pointGravity = if enableAnimation and animateGravity and gravityType == 2 then 1 else 0;
        Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape gravitySphere(shapeType = "sphere", r_shape = { -gravitySphereDiameter / 2,0,0}, lengthDirection = {1,0,0}, length = gravitySphereDiameter, width = gravitySphereDiameter, height = gravitySphereDiameter, color = gravitySphereColor, specularCoefficient = 0) if enableAnimation and animateGravity and gravityType == GravityTypes.PointGravity;
        function gravityAcceleration = gravityAccelerationTypes(gravityType = gravityType, g = g * Modelica.Math.Vectors.normalize(n), mue = mue);
      protected 
        function gravityAccelerationTypes "Gravity field acceleration depending on field type and position"
          import Modelica.Mechanics.MultiBody.Types.GravityTypes;
          extends Modelica.Icons.Function;
          input SI.Position r[3] "Position vector from world frame to actual point, resolved in world frame";
          input GravityTypes gravityType "Type of gravity field";
          input SI.Acceleration g[3] "Constant gravity acceleration, resolved in world frame, if gravityType=1";
          input Real mue(unit = "m3/s2") "Field constant of point gravity field, if gravityType=2";
          output SI.Acceleration gravity[3] "Gravity acceleration at point r, resolved in world frame";
        algorithm 
          gravity:=if gravityType == GravityTypes.UniformGravity then g else if gravityType == GravityTypes.PointGravity then  -mue / (r * r) * r / Modelica.Math.Vectors.length(r) else zeros(3);
        end gravityAccelerationTypes;

      equation 
        Connections.root(frame_b.R);
        assert(Modelica.Math.Vectors.length(n) > 1e-10, "Parameter n of World object is wrong (lenght(n) > 0 required)");
        frame_b.r_0 = zeros(3);
        frame_b.R = Frames.nullRotation();
      end World;
      package Examples "Examples that demonstrate the usage of the MultiBody library"
        extends Modelica.Icons.Library;
        annotation(Documentation(info = "<html>
<p>
This package contains example models to demonstrate the usage of the
MultiBody package. Open the models and
simulate them according to the provided description in the models.
</p>

</HTML>
"));
        package Systems "Examples of complete system models including 3-dimensional mechanics"
          extends Modelica.Icons.Library;
          annotation(Documentation(info = "<html>
<p>
This package contains complete <b>system models</b> where components
from different domains are used, including 3-dimensional mechanics.
</p>
<h4>Content</h4>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Model</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3\">RobotR3</a><br>
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.oneAxis\">RobotR3.oneAxis</a><br>
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.fullRobot\">RobotR3.fullRobot</a>
</b></td>
      <td valign=\"top\"> 6 degree of freedom robot with path planning,
           controllers, motors, brakes, gears and mechanics.
           \"oneAxis\" models only one drive train. \"fullRobot\" is
           the complete, detailed robot model.<br>
      <img src=\"../Images/MultiBody/Examples/Systems/r3_fullRobot_small.png\">
      </td>
  </tr>
</table>
</html>"));
          package RobotR3 "Library to demonstrate robot system models based on the Manutec r3 robot"
            model oneAxis "Model of one axis of robot (controller, motor, gearbox) with simple load"
              import SI = Modelica.SIunits;
              extends Modelica.Icons.Example;
              parameter SI.Mass mLoad(min = 0) = 15 "Mass of load";
              parameter Real kp = 5 "Gain of position controller of axis 2";
              parameter Real ks = 0.5 "Gain of speed controller of axis 2";
              parameter SI.Time Ts = 0.05 "Time constant of integrator of speed controller of axis 2";
              parameter Real startAngle(unit = "deg") = 0 "Start angle of axis 2";
              parameter Real endAngle(unit = "deg") = 120 "End angle of axis 2";
              parameter SI.Time swingTime = 0.5 "Additional time after reference motion is in rest before simulation is stopped";
              parameter SI.AngularVelocity refSpeedMax = 3 "Maximum reference speed";
              parameter SI.AngularAcceleration refAccMax = 10 "Maximum reference acceleration";
              annotation(Documentation(info = "<HTML>
<p>
With this model one axis of the r3 robot is checked.
The mechanical structure is replaced by a simple
load inertia.
</p>
</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), experiment(StopTime = 1.6), __Dymola_Commands(file = "Mechanics/MultiBody/Examples/Systems/oneAxisPlot.mos" "Plot result"));
              Components.AxisType1 axis(w = 5500, ratio = 210, c = 8, cd = 0.01, Rv0 = 0.5, Rv1 = 0.1 / 130, kp = kp, ks = ks, Ts = Ts) annotation(Placement(transformation(extent = {{20,0},{40,20}}, rotation = 0)));
              Modelica.Mechanics.Rotational.Components.Inertia load(J = 1.3 * mLoad) annotation(Placement(transformation(extent = {{54,0},{74,20}}, rotation = 0)));
              Components.PathPlanning1 pathPlanning(swingTime = swingTime, angleBegDeg = startAngle, angleEndDeg = endAngle, speedMax = refSpeedMax, accMax = refAccMax) annotation(Placement(transformation(extent = {{ -60,0},{ -40,20}}, rotation = 0)));
            protected 
              Components.ControlBus controlBus annotation(Placement(transformation(extent = {{ -32,10},{8,50}}, rotation = 0)));

            equation 
              connect(axis.flange,load.flange_a) annotation(Line(points = {{40,10},{54,10}}, color = {128,128,128}, thickness = 0.5));
              connect(pathPlanning.controlBus,controlBus) annotation(Line(points = {{ -40,10},{ -15,10},{ -15,28},{ -12,28},{ -12,30}}, color = {255,204,51}, thickness = 0.5));
              connect(controlBus.axisControlBus1,axis.axisControlBus) annotation(Text(string = "%first", index =  -1, extent = [ -6,3; -6,3], style(color = 0, rgbcolor = {0,0,0})), Line(points = {{ -12,30},{ -12,29},{ -9,29},{ -9,10},{20,10}}, color = {255,204,51}, thickness = 0.5));
            end oneAxis;
            model fullRobot "6 degree of freedom robot with path planning, controllers, motors, brakes, gears and mechanics"
              annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {0.5,0.5}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {0.5,0.5}), graphics = {Rectangle(extent = {{ -99.5,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Bitmap(extent = {{ -75.5,98.25},{87, -96.75}}, fileName = "../../../../Images/MultiBody/Examples/Systems/robot_kr15.bmp"),Text(extent = {{ -111.5,130},{108.5,100}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -104.5, -104},{115, -128}}, lineColor = {0,0,0}, textString = "mLoad=%mLoad")}), experiment(StopTime = 2), __Dymola_Commands(file = "Mechanics/MultiBody/Examples/Systems/Run.mos" "Simulate", file = "Mechanics/MultiBody/Examples/Systems/fullRobotPlot.mos" "Plot result of axis 3 + animate"), Documentation(info = "<HTML>
<p>
This is a detailed model of the robot. For animation CAD data
is used. Translate and simulate with the default settings
(default simulation time = 3 s). Use command script \"Scripts\\ExamplesfullRobotPlot.mos\"
to plot variables.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Systems/r3_fullRobot.png\" ALT=\"model Examples.Loops.Systems.RobotR3.fullRobot\">
</p>
</HTML>"));
              import SI = Modelica.SIunits;
              parameter SI.Mass mLoad(min = 0) = 15 "Mass of load";
              parameter SI.Position rLoad[3] = {0.1,0.25,0.1} "Distance from last flange to load mass";
              parameter SI.Acceleration g = 9.81 "Gravity acceleration";
              parameter SI.Time refStartTime = 0 "Start time of reference motion";
              parameter SI.Time refSwingTime = 0.5 "Additional time after reference motion is in rest before simulation is stopped";
              parameter Real startAngle1(unit = "deg") =  -60 "Start angle of axis 1" annotation(Dialog(tab = "Reference", group = "startAngles"));
              parameter Real startAngle2(unit = "deg") = 20 "Start angle of axis 2" annotation(Dialog(tab = "Reference", group = "startAngles"));
              parameter Real startAngle3(unit = "deg") = 90 "Start angle of axis 3" annotation(Dialog(tab = "Reference", group = "startAngles"));
              parameter Real startAngle4(unit = "deg") = 0 "Start angle of axis 4" annotation(Dialog(tab = "Reference", group = "startAngles"));
              parameter Real startAngle5(unit = "deg") =  -110 "Start angle of axis 5" annotation(Dialog(tab = "Reference", group = "startAngles"));
              parameter Real startAngle6(unit = "deg") = 0 "Start angle of axis 6" annotation(Dialog(tab = "Reference", group = "startAngles"));
              parameter Real endAngle1(unit = "deg") = 60 "End angle of axis 1" annotation(Dialog(tab = "Reference", group = "endAngles"));
              parameter Real endAngle2(unit = "deg") =  -70 "End angle of axis 2" annotation(Dialog(tab = "Reference", group = "endAngles"));
              parameter Real endAngle3(unit = "deg") =  -35 "End angle of axis 3" annotation(Dialog(tab = "Reference", group = "endAngles"));
              parameter Real endAngle4(unit = "deg") = 45 "End angle of axis 4" annotation(Dialog(tab = "Reference", group = "endAngles"));
              parameter Real endAngle5(unit = "deg") = 110 "End angle of axis 5" annotation(Dialog(tab = "Reference", group = "endAngles"));
              parameter Real endAngle6(unit = "deg") = 45 "End angle of axis 6" annotation(Dialog(tab = "Reference", group = "endAngles"));
              parameter SI.AngularVelocity refSpeedMax[6] = {3,1.5,5,3.1,3.1,4.1} "Maximum reference speeds of all joints" annotation(Dialog(tab = "Reference", group = "Limits"));
              parameter SI.AngularAcceleration refAccMax[6] = {15,15,15,60,60,60} "Maximum reference accelerations of all joints" annotation(Dialog(tab = "Reference", group = "Limits"));
              parameter Real kp1 = 5 "Gain of position controller" annotation(Dialog(tab = "Controller", group = "Axis 1"));
              parameter Real ks1 = 0.5 "Gain of speed controller" annotation(Dialog(tab = "Controller", group = "Axis 1"));
              parameter SI.Time Ts1 = 0.05 "Time constant of integrator of speed controller" annotation(Dialog(tab = "Controller", group = "Axis 1"));
              parameter Real kp2 = 5 "Gain of position controller" annotation(Dialog(tab = "Controller", group = "Axis 2"));
              parameter Real ks2 = 0.5 "Gain of speed controller" annotation(Dialog(tab = "Controller", group = "Axis 2"));
              parameter SI.Time Ts2 = 0.05 "Time constant of integrator of speed controller" annotation(Dialog(tab = "Controller", group = "Axis 2"));
              parameter Real kp3 = 5 "Gain of position controller" annotation(Dialog(tab = "Controller", group = "Axis 3"));
              parameter Real ks3 = 0.5 "Gain of speed controller" annotation(Dialog(tab = "Controller", group = "Axis 3"));
              parameter SI.Time Ts3 = 0.05 "Time constant of integrator of speed controller" annotation(Dialog(tab = "Controller", group = "Axis 3"));
              parameter Real kp4 = 5 "Gain of position controller" annotation(Dialog(tab = "Controller", group = "Axis 4"));
              parameter Real ks4 = 0.5 "Gain of speed controller" annotation(Dialog(tab = "Controller", group = "Axis 4"));
              parameter SI.Time Ts4 = 0.05 "Time constant of integrator of speed controller" annotation(Dialog(tab = "Controller", group = "Axis 4"));
              parameter Real kp5 = 5 "Gain of position controller" annotation(Dialog(tab = "Controller", group = "Axis 5"));
              parameter Real ks5 = 0.5 "Gain of speed controller" annotation(Dialog(tab = "Controller", group = "Axis 5"));
              parameter SI.Time Ts5 = 0.05 "Time constant of integrator of speed controller" annotation(Dialog(tab = "Controller", group = "Axis 5"));
              parameter Real kp6 = 5 "Gain of position controller" annotation(Dialog(tab = "Controller", group = "Axis 6"));
              parameter Real ks6 = 0.5 "Gain of speed controller" annotation(Dialog(tab = "Controller", group = "Axis 6"));
              parameter SI.Time Ts6 = 0.05 "Time constant of integrator of speed controller" annotation(Dialog(tab = "Controller", group = "Axis 6"));
              Components.MechanicalStructure mechanics(mLoad = mLoad, rLoad = rLoad, g = g) annotation(Placement(transformation(extent = {{35, -35},{95,25}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Components.PathPlanning6 pathPlanning(naxis = 6, angleBegDeg = {startAngle1,startAngle2,startAngle3,startAngle4,startAngle5,startAngle6}, angleEndDeg = {endAngle1,endAngle2,endAngle3,endAngle4,endAngle5,endAngle6}, speedMax = refSpeedMax, accMax = refAccMax, startTime = refStartTime, swingTime = refSwingTime) annotation(Placement(transformation(extent = {{ -5,50},{ -25,70}}, rotation = 0)));
              RobotR3.Components.AxisType1 axis1(w = 4590, ratio =  -105, c = 43, cd = 0.005, Rv0 = 0.4, Rv1 = 0.13 / 160, kp = kp1, ks = ks1, Ts = Ts1) annotation(Placement(transformation(extent = {{ -25, -75},{ -5, -55}}, rotation = 0)));
              RobotR3.Components.AxisType1 axis2(w = 5500, ratio = 210, c = 8, cd = 0.01, Rv1 = 0.1 / 130, Rv0 = 0.5, kp = kp2, ks = ks2, Ts = Ts2) annotation(Placement(transformation(extent = {{ -25, -55},{ -5, -35}}, rotation = 0)));
              RobotR3.Components.AxisType1 axis3(w = 5500, ratio = 60, c = 58, cd = 0.04, Rv0 = 0.7, Rv1 = 0.2 / 130, kp = kp3, ks = ks3, Ts = Ts3) annotation(Placement(transformation(extent = {{ -25, -35},{ -5, -15}}, rotation = 0)));
              RobotR3.Components.AxisType2 axis4(k = 0.2365, w = 6250, D = 0.55, J = 0.00016, ratio =  -99, Rv0 = 21.8, Rv1 = 9.8, peak = 26.7 / 21.8, kp = kp4, ks = ks4, Ts = Ts4) annotation(Placement(transformation(extent = {{ -25, -15},{ -5,5}}, rotation = 0)));
              RobotR3.Components.AxisType2 axis5(k = 0.2608, w = 6250, D = 0.55, J = 0.00018, ratio = 79.2, Rv0 = 30.1, Rv1 = 0.03, peak = 39.6 / 30.1, kp = kp5, ks = ks5, Ts = Ts5) annotation(Placement(transformation(extent = {{ -25,5},{ -5,25}}, rotation = 0)));
              RobotR3.Components.AxisType2 axis6(k = 0.0842, w = 7400, D = 0.27, J = 4.3e-05, ratio =  -99, Rv0 = 10.9, Rv1 = 3.92, peak = 16.8 / 10.9, kp = kp6, ks = ks6, Ts = Ts6) annotation(Placement(transformation(extent = {{ -25,25},{ -5,45}}, rotation = 0)));
            protected 
              Components.ControlBus controlBus annotation(Placement(transformation(origin = { -80, -10}, extent = {{ -20, -20},{20,20}}, rotation = 90)));

            equation 
              connect(axis2.flange,mechanics.axis2) annotation(Line(points = {{ -5, -45},{25, -45},{25, -21.5},{33.5, -21.5}}, color = {0,0,0}));
              connect(axis1.flange,mechanics.axis1) annotation(Line(points = {{ -5, -65},{30, -65},{30, -30.5},{33.5, -30.5}}, color = {0,0,0}));
              connect(axis3.flange,mechanics.axis3) annotation(Line(points = {{ -5, -25},{15, -25},{15, -12.5},{33.5, -12.5}}, color = {0,0,0}));
              connect(axis4.flange,mechanics.axis4) annotation(Line(points = {{ -5, -5},{15, -5},{15, -3.5},{33.5, -3.5}}, color = {0,0,0}));
              connect(axis5.flange,mechanics.axis5) annotation(Line(points = {{ -5,15},{10,15},{10,5.5},{33.5,5.5}}, color = {0,0,0}));
              connect(axis6.flange,mechanics.axis6) annotation(Line(points = {{ -5,35},{20,35},{20,14.5},{33.5,14.5}}, color = {0,0,0}));
              connect(controlBus,pathPlanning.controlBus) annotation(Line(points = {{ -80, -10},{ -80,60},{ -25,60}}, color = {255,204,51}, thickness = 0.5));
              connect(controlBus.axisControlBus1,axis1.axisControlBus) annotation(Text(string = "%first", index =  -1, extent = [ -6,3; -6,3], style(color = 0, rgbcolor = {0,0,0})), Line(points = {{ -80, -10},{ -80, -14.5},{ -79, -14.5},{ -79, -17},{ -65, -17},{ -65, -65},{ -25, -65}}, color = {255,204,51}, thickness = 0.5));
              connect(controlBus.axisControlBus2,axis2.axisControlBus) annotation(Text(string = "%first", index =  -1, extent = [ -6,3; -6,3], style(color = 0, rgbcolor = {0,0,0})), Line(points = {{ -80, -10},{ -79, -10},{ -79, -15},{ -62.5, -15},{ -62.5, -45},{ -25, -45}}, color = {255,204,51}, thickness = 0.5));
              connect(controlBus.axisControlBus3,axis3.axisControlBus) annotation(Text(string = "%first", index =  -1, extent = [ -6,3; -6,3], style(color = 0, rgbcolor = {0,0,0})), Line(points = {{ -80, -10},{ -77, -10},{ -77, -12.5},{ -61, -12.5},{ -61, -25},{ -25, -25}}, color = {255,204,51}, thickness = 0.5));
              connect(controlBus.axisControlBus4,axis4.axisControlBus) annotation(Text(string = "%first", index =  -1, extent = [ -6,3; -6,3], style(color = 0, rgbcolor = {0,0,0})), Line(points = {{ -80, -10},{ -60.5, -10},{ -60.5, -5},{ -25, -5}}, color = {255,204,51}, thickness = 0.5));
              connect(controlBus.axisControlBus5,axis5.axisControlBus) annotation(Text(string = "%first", index =  -1, extent = [ -6,3; -6,3], style(color = 0, rgbcolor = {0,0,0})), Line(points = {{ -80, -10},{ -77, -10},{ -77, -7},{ -63, -7},{ -63,15},{ -25,15}}, color = {255,204,51}, thickness = 0.5));
              connect(controlBus.axisControlBus6,axis6.axisControlBus) annotation(Text(string = "%first", index =  -1, extent = [ -6,3; -6,3], style(color = 0, rgbcolor = {0,0,0})), Line(points = {{ -80, -10},{ -79, -10},{ -79, -5},{ -65, -5},{ -65,35},{ -25,35}}, color = {255,204,51}, thickness = 0.5));
            end fullRobot;
            extends Modelica.Icons.Library;
            import SI = Modelica.SIunits;
            package Components "Library of components of the robot"
              extends Modelica.Icons.Library;
              expandable connector AxisControlBus "Data bus for one robot axis"
                extends Modelica.Icons.SignalSubBus;
                annotation(defaultComponentPrefixes = "protected", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -20,2},{22, -2}}, lineColor = {255,204,51}, lineThickness = 0.5)}), Documentation(info = "<html>
<p>
Signal bus that is used to communicate all signals for <b>one</b> axis.
This is an expandable connector which is \"empty\".
The actual signal content is defined by connecting to an instance
of this connector. The signals that are usually used
(and are by default listed as choices in the menu that defines
the connection to this bus) are defined
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Components.InternalConnectors.AxisControlBus\">here</a>.
</p>

</html>"));
              end AxisControlBus;
              expandable connector ControlBus "Data bus for all axes of robot"
                extends Modelica.Icons.SignalBus;
                annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -20,2},{22, -2}}, lineColor = {255,204,51}, lineThickness = 0.5)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
Signal bus that is used to communicate <b>all signals</b> of the robot.
This is an expandable connector which is \"empty\".
The actual signal content is defined by connecting to an instance
of this connector. The sub-buses that are usually used
(and are by default listed as choices in the menu that defines
the connection to this bus) are defined
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Components.InternalConnectors.ControlBus\">here</a>.
</p>
</html>"));
              end ControlBus;
              model PathPlanning1 "Generate reference angles for fastest kinematic movement"
                import SI = Modelica.SIunits;
                import Cv = Modelica.SIunits.Conversions;
                parameter Real angleBegDeg(unit = "deg") = 0 "Start angle";
                parameter Real angleEndDeg(unit = "deg") = 1 "End angle";
                parameter SI.AngularVelocity speedMax = 3 "Maximum axis speed";
                parameter SI.AngularAcceleration accMax = 2.5 "Maximum axis acceleration";
                parameter SI.Time startTime = 0 "Start time of movement";
                parameter SI.Time swingTime = 0.5 "Additional time after reference motion is in rest before simulation is stopped";
                final parameter SI.Angle angleBeg = Cv.from_deg(angleBegDeg) "Start angles";
                final parameter SI.Angle angleEnd = Cv.from_deg(angleEndDeg) "End angles";
                ControlBus controlBus annotation(Placement(transformation(origin = {100,0}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
                Modelica.Blocks.Sources.KinematicPTP2 path(q_end = {angleEnd}, qd_max = {speedMax}, qdd_max = {accMax}, startTime = startTime, q_begin = {angleBeg}) annotation(Placement(transformation(extent = {{ -50, -10},{ -30,10}}, rotation = 0)));
                PathToAxisControlBus pathToAxis1(final nAxis = 1, final axisUsed = 1) annotation(Placement(transformation(extent = {{0, -10},{20,10}}, rotation = 0)));
                annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,150},{150,110}}, textString = "%name", lineColor = {0,0,255}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,88},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,78},{ -80, -82}}, color = {192,192,192}),Line(points = {{ -90,0},{82,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -42,55},{29,12}}, lineColor = {192,192,192}, textString = "w"),Line(points = {{ -80,0},{ -41,69},{26,69},{58,0}}, color = {0,0,0}),Text(extent = {{ -73, -44},{82, -69}}, lineColor = {0,0,0}, textString = "1 axis")}), Documentation(info = "<html>
<p>
Given
</p>
<ul>
<li> start and end angle of an axis</li>
<li> maximum speed of the axis </li>
<li> maximum acceleration of the axis </li>
</ul>

<p>
this component computes the fastest movement under the
given constraints. This means, that:
</p>

<ol>
<li> The axis accelerates with the maximum acceleration
     until the maximum speed is reached.</li>
<li> Drives with the maximum speed as long as possible.</li>
<li> Decelerates with the negative of the maximum acceleration
     until rest.</li>
</ol>

<p>
The acceleration, constant velocity and deceleration
phase are determined in such a way that the movement
starts form the start angles and ends at the end angles.
The output of this block are the computed angles, angular velocities
and angular acceleration and this information is stored as reference
motion on the controlBus of the r3 robot.
</p>

</html>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
                Blocks.Logical.TerminateSimulation terminateSimulation(condition = time >= path.endTime + swingTime) annotation(Placement(transformation(extent = {{ -50, -30},{30, -24}}, rotation = 0)));

              equation 
                connect(path.q,pathToAxis1.q) annotation(Line(points = {{ -29,8},{ -2,8}}, color = {0,0,127}));
                connect(path.qd,pathToAxis1.qd) annotation(Line(points = {{ -29,3},{ -2,3}}, color = {0,0,127}));
                connect(path.qdd,pathToAxis1.qdd) annotation(Line(points = {{ -29, -3},{ -2, -3}}, color = {0,0,127}));
                connect(path.moving,pathToAxis1.moving) annotation(Line(points = {{ -29, -8},{ -2, -8}}, color = {255,0,255}));
                connect(pathToAxis1.axisControlBus,controlBus.axisControlBus1) annotation(Text(string = "%second", index = 1, extent = [6,3;6,3], style(color = 0, rgbcolor = {0,0,0})), Line(points = {{20,0},{100,0}}, color = {255,204,51}, thickness = 0.5));
              end PathPlanning1;
              model PathPlanning6 "Generate reference angles for fastest kinematic movement"
                import SI = Modelica.SIunits;
                import Cv = Modelica.SIunits.Conversions;
                parameter Integer naxis = 6 "number of driven axis";
                parameter Real angleBegDeg[naxis](unit = "deg") = zeros(naxis) "Start angles";
                parameter Real angleEndDeg[naxis](unit = "deg") = ones(naxis) "End angles";
                parameter SI.AngularVelocity speedMax[naxis] = fill(3, naxis) "Maximum axis speed";
                parameter SI.AngularAcceleration accMax[naxis] = fill(2.5, naxis) "Maximum axis acceleration";
                parameter SI.Time startTime = 0 "Start time of movement";
                parameter SI.Time swingTime = 0.5 "Additional time after reference motion is in rest before simulation is stopped";
                final parameter SI.Angle angleBeg[:] = Cv.from_deg(angleBegDeg) "Start angles";
                final parameter SI.Angle angleEnd[:] = Cv.from_deg(angleEndDeg) "End angles";
                ControlBus controlBus annotation(Placement(transformation(origin = {100,0}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
                Modelica.Blocks.Sources.KinematicPTP2 path(q_end = angleEnd, qd_max = speedMax, qdd_max = accMax, startTime = startTime, q_begin = angleBeg) annotation(Placement(transformation(extent = {{ -90, -80},{ -70, -60}}, rotation = 0)));
                PathToAxisControlBus pathToAxis1(nAxis = naxis, axisUsed = 1) annotation(Placement(transformation(extent = {{ -10,70},{10,90}}, rotation = 0)));
                PathToAxisControlBus pathToAxis2(nAxis = naxis, axisUsed = 2) annotation(Placement(transformation(extent = {{ -10,40},{10,60}}, rotation = 0)));
                PathToAxisControlBus pathToAxis3(nAxis = naxis, axisUsed = 3) annotation(Placement(transformation(extent = {{ -10,10},{10,30}}, rotation = 0)));
                PathToAxisControlBus pathToAxis4(nAxis = naxis, axisUsed = 4) annotation(Placement(transformation(extent = {{ -10, -20},{10,0}}, rotation = 0)));
                PathToAxisControlBus pathToAxis5(nAxis = naxis, axisUsed = 5) annotation(Placement(transformation(extent = {{ -10, -50},{10, -30}}, rotation = 0)));
                PathToAxisControlBus pathToAxis6(nAxis = naxis, axisUsed = 6) annotation(Placement(transformation(extent = {{ -10, -80},{10, -60}}, rotation = 0)));
                annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,150},{150,110}}, textString = "%name", lineColor = {0,0,255}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,88},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,78},{ -80, -82}}, color = {192,192,192}),Line(points = {{ -90,0},{82,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -42,55},{29,12}}, lineColor = {192,192,192}, textString = "w"),Line(points = {{ -80,0},{ -41,69},{26,69},{58,0}}, color = {0,0,0}),Text(extent = {{ -70, -43},{85, -68}}, lineColor = {0,0,0}, textString = "6 axes")}), Documentation(info = "<html>
<p>
Given
</p>
<ul>
<li> start and end angles of every axis</li>
<li> maximum speed of every axis </li>
<li> maximum acceleration of every axis </li>
</ul>

<p>
this component computes the fastest movement under the
given constraints. This means, that:
</p>

<ol>
<li> Every axis accelerates with the maximum acceleration
     until the maximum speed is reached.</li>
<li> Drives with the maximum speed as long as possible.</li>
<li> Decelerates with the negative of the maximum acceleration
     until rest.</li>
</ol>

<p>
The acceleration, constant velocity and deceleration
phase are determined in such a way that the movement
starts form the start angles and ends at the end angles.
The output of this block are the computed angles, angular velocities
and angular acceleration and this information is stored as reference
motion on the controlBus of the r3 robot.
</p>

</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
                Blocks.Logical.TerminateSimulation terminateSimulation(condition = time >= path.endTime + swingTime) annotation(Placement(transformation(extent = {{ -50, -100},{30, -94}}, rotation = 0)));

              equation 
                connect(path.q,pathToAxis1.q) annotation(Line(points = {{ -69, -62},{ -60, -62},{ -60,88},{ -12,88}}, color = {0,0,127}));
                connect(path.qd,pathToAxis1.qd) annotation(Line(points = {{ -69, -67},{ -59, -67},{ -59,83},{ -12,83}}, color = {0,0,127}));
                connect(path.qdd,pathToAxis1.qdd) annotation(Line(points = {{ -69, -73},{ -58, -73},{ -58,77},{ -12,77}}, color = {0,0,127}));
                connect(path.moving,pathToAxis1.moving) annotation(Line(points = {{ -69, -78},{ -57, -78},{ -57,72},{ -12,72}}, color = {255,0,255}));
                connect(path.q,pathToAxis2.q) annotation(Line(points = {{ -69, -62},{ -60, -62},{ -60,58},{ -12,58}}, color = {0,0,127}));
                connect(path.qd,pathToAxis2.qd) annotation(Line(points = {{ -69, -67},{ -59, -67},{ -59,53},{ -12,53}}, color = {0,0,127}));
                connect(path.qdd,pathToAxis2.qdd) annotation(Line(points = {{ -69, -73},{ -58, -73},{ -58,47},{ -12,47}}, color = {0,0,127}));
                connect(path.moving,pathToAxis2.moving) annotation(Line(points = {{ -69, -78},{ -57, -78},{ -57,42},{ -12,42}}, color = {255,0,255}));
                connect(path.q,pathToAxis3.q) annotation(Line(points = {{ -69, -62},{ -60, -62},{ -60,28},{ -12,28}}, color = {0,0,127}));
                connect(path.qd,pathToAxis3.qd) annotation(Line(points = {{ -69, -67},{ -59, -67},{ -59,23},{ -12,23}}, color = {0,0,127}));
                connect(path.qdd,pathToAxis3.qdd) annotation(Line(points = {{ -69, -73},{ -58, -73},{ -58,17},{ -12,17}}, color = {0,0,127}));
                connect(path.moving,pathToAxis3.moving) annotation(Line(points = {{ -69, -78},{ -57, -78},{ -57,12},{ -12,12}}, color = {255,0,255}));
                connect(path.q,pathToAxis4.q) annotation(Line(points = {{ -69, -62},{ -60, -62},{ -60, -2},{ -12, -2}}, color = {0,0,127}));
                connect(path.qd,pathToAxis4.qd) annotation(Line(points = {{ -69, -67},{ -59, -67},{ -59, -7},{ -12, -7}}, color = {0,0,127}));
                connect(path.qdd,pathToAxis4.qdd) annotation(Line(points = {{ -69, -73},{ -58, -73},{ -58, -13},{ -12, -13}}, color = {0,0,127}));
                connect(path.moving,pathToAxis4.moving) annotation(Line(points = {{ -69, -78},{ -57, -78},{ -57, -18},{ -12, -18}}, color = {255,0,255}));
                connect(path.q,pathToAxis5.q) annotation(Line(points = {{ -69, -62},{ -60, -62},{ -60, -32},{ -12, -32}}, color = {0,0,127}));
                connect(path.qd,pathToAxis5.qd) annotation(Line(points = {{ -69, -67},{ -59, -67},{ -59, -37},{ -12, -37}}, color = {0,0,127}));
                connect(path.qdd,pathToAxis5.qdd) annotation(Line(points = {{ -69, -73},{ -58, -73},{ -58, -43},{ -12, -43}}, color = {0,0,127}));
                connect(path.moving,pathToAxis5.moving) annotation(Line(points = {{ -69, -78},{ -57, -78},{ -57, -48},{ -12, -48}}, color = {255,0,255}));
                connect(path.q,pathToAxis6.q) annotation(Line(points = {{ -69, -62},{ -12, -62}}, color = {0,0,127}));
                connect(path.qd,pathToAxis6.qd) annotation(Line(points = {{ -69, -67},{ -12, -67}}, color = {0,0,127}));
                connect(path.qdd,pathToAxis6.qdd) annotation(Line(points = {{ -69, -73},{ -12, -73}}, color = {0,0,127}));
                connect(path.moving,pathToAxis6.moving) annotation(Line(points = {{ -69, -78},{ -12, -78}}, color = {255,0,255}));
                connect(pathToAxis1.axisControlBus,controlBus.axisControlBus1) annotation(Text(string = "%second", index = 1, extent = [6,3;6,3], style(color = 0, rgbcolor = {0,0,0})), Line(points = {{10,80},{80,80},{80,7},{98,7}}, color = {255,204,51}, thickness = 0.5));
                connect(pathToAxis2.axisControlBus,controlBus.axisControlBus2) annotation(Text(string = "%second", index = 1, extent = [6,3;6,3], style(color = 0, rgbcolor = {0,0,0})), Line(points = {{10,50},{77,50},{77,5},{97,5}}, color = {255,204,51}, thickness = 0.5));
                connect(pathToAxis3.axisControlBus,controlBus.axisControlBus3) annotation(Text(string = "%second", index = 1, extent = [6,3;6,3], style(color = 0, rgbcolor = {0,0,0})), Line(points = {{10,20},{75,20},{75,3},{96,3}}, color = {255,204,51}, thickness = 0.5));
                connect(pathToAxis4.axisControlBus,controlBus.axisControlBus4) annotation(Text(string = "%second", index = 1, extent = [6,3;6,3], style(color = 0, rgbcolor = {0,0,0})), Line(points = {{10, -10},{73, -10},{73,0},{100,0}}, color = {255,204,51}, thickness = 0.5));
                connect(pathToAxis5.axisControlBus,controlBus.axisControlBus5) annotation(Text(string = "%second", index = 1, extent = [6,3;6,3], style(color = 0, rgbcolor = {0,0,0})), Line(points = {{10, -40},{75, -40},{75, -3},{100, -3},{100,0}}, color = {255,204,51}, thickness = 0.5));
                connect(pathToAxis6.axisControlBus,controlBus.axisControlBus6) annotation(Text(string = "%second", index = 1, extent = [6,3;6,3], style(color = 0, rgbcolor = {0,0,0})), Line(points = {{10, -70},{78, -70},{78, -6},{98, -6}}, color = {255,204,51}, thickness = 0.5));
              end PathPlanning6;
              model PathToAxisControlBus "Map path planning to one axis control bus"
                extends Blocks.Interfaces.BlockIcon;
                parameter Integer nAxis = 6 "Number of driven axis";
                parameter Integer axisUsed = 1 "Map path planning of axisUsed to axisControlBus";
                Blocks.Interfaces.RealInput q[nAxis] annotation(Placement(transformation(extent = {{ -140,60},{ -100,100}}, rotation = 0)));
                Blocks.Interfaces.RealInput qd[nAxis] annotation(Placement(transformation(extent = {{ -140,10},{ -100,50}}, rotation = 0)));
                Blocks.Interfaces.RealInput qdd[nAxis] annotation(Placement(transformation(extent = {{ -140, -50},{ -100, -10}}, rotation = 0)));
                AxisControlBus axisControlBus annotation(Placement(transformation(origin = {100,0}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
                Blocks.Routing.RealPassThrough q_axisUsed annotation(Placement(transformation(extent = {{ -40,50},{ -20,70}}, rotation = 0)));
                Blocks.Routing.RealPassThrough qd_axisUsed annotation(Placement(transformation(extent = {{ -40,10},{ -20,30}}, rotation = 0)));
                Blocks.Routing.RealPassThrough qdd_axisUsed annotation(Placement(transformation(extent = {{ -40, -30},{ -20, -10}}, rotation = 0)));
                Blocks.Interfaces.BooleanInput moving[nAxis] annotation(Placement(transformation(extent = {{ -140, -100},{ -100, -60}}, rotation = 0)));
                Blocks.Routing.BooleanPassThrough motion_ref_axisUsed annotation(Placement(transformation(extent = {{ -40, -70},{ -20, -50}}, rotation = 0)));
                annotation(defaultComponentName = "pathToAxis1", Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}, initialScale = 0.1), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}, initialScale = 0.1), graphics = {Text(extent = {{ -100,98},{ -24,68}}, lineColor = {0,0,0}, textString = "q"),Text(extent = {{ -94,46},{ -18,16}}, lineColor = {0,0,0}, textString = "qd"),Text(extent = {{ -96, -16},{ -20, -46}}, lineColor = {0,0,0}, textString = "qdd"),Text(extent = {{ -2,20},{80, -18}}, lineColor = {0,0,0}, textString = "%axisUsed"),Text(extent = {{2,52},{76,28}}, lineColor = {0,0,0}, textString = "axis"),Text(extent = {{ -94, -70},{32, -96}}, lineColor = {0,0,0}, textString = "moving")}));

              equation 
                connect(q_axisUsed.u,q[axisUsed]) annotation(Line(points = {{ -42,60},{ -60,60},{ -60,80},{ -120,80}}, color = {0,0,127}));
                connect(qd_axisUsed.u,qd[axisUsed]) annotation(Line(points = {{ -42,20},{ -80,20},{ -80,30},{ -120,30}}, color = {0,0,127}));
                connect(qdd_axisUsed.u,qdd[axisUsed]) annotation(Line(points = {{ -42, -20},{ -80, -20},{ -80, -30},{ -120, -30}}, color = {0,0,127}));
                connect(motion_ref_axisUsed.u,moving[axisUsed]) annotation(Line(points = {{ -42, -60},{ -60, -60},{ -60, -80},{ -120, -80}}, color = {255,0,255}));
                connect(motion_ref_axisUsed.y,axisControlBus.motion_ref) annotation(Text(string = "%second", index = 1, extent = [6,3;6,3], style(color = 0, rgbcolor = {0,0,0})), Line(points = {{ -19, -60},{44, -60},{44, -8},{102, -8},{102,0},{100,0}}, color = {255,0,255}));
                connect(qdd_axisUsed.y,axisControlBus.acceleration_ref) annotation(Text(string = "%second", index = 1, extent = [6,3;6,3], style(color = 0, rgbcolor = {0,0,0})), Line(points = {{ -19, -20},{40, -20},{40, -4},{98, -4},{98,0},{100,0}}, color = {0,0,127}));
                connect(qd_axisUsed.y,axisControlBus.speed_ref) annotation(Text(string = "%second", index = 1, extent = [6,3;6,3], style(color = 0, rgbcolor = {0,0,0})), Line(points = {{ -19,20},{20,20},{20,0},{100,0}}, color = {0,0,127}));
                connect(q_axisUsed.y,axisControlBus.angle_ref) annotation(Text(string = "%second", index = 1, extent = [6,3;6,3], style(color = 0, rgbcolor = {0,0,0})), Line(points = {{ -19,60},{40,60},{40,4},{96,4}}, color = {0,0,127}));
              end PathToAxisControlBus;
              model GearType1 "Motor inertia and gearbox model for r3 joints 1,2,3 "
                extends Modelica.Mechanics.Rotational.Interfaces.PartialTwoFlanges;
                parameter Real i =  -105 "gear ratio";
                parameter Real c(unit = "N.m/rad") = 43 "Spring constant";
                parameter Real d(unit = "N.m.s/rad") = 0.005 "Damper constant";
                parameter SI.Torque Rv0 = 0.4 "Viscous friction torque at zero velocity";
                parameter Real Rv1(unit = "N.m.s/rad") = 0.13 / 160 "Viscous friction coefficient (R=Rv0+Rv1*abs(qd))";
                parameter Real peak = 1 "Maximum static friction torque is peak*Rv0 (peak >= 1)";
                SI.AngularAcceleration a_rel = der(spring.w_rel) "Relative angular acceleration of spring";
                constant SI.AngularVelocity unitAngularVelocity = 1;
                constant SI.Torque unitTorque = 1;
                annotation(Documentation(info = "<html>
<p>
Models the gearbox used in the first three joints with all its effects,
like elasticity and friction.
Coulomb friction is approximated by a friction element acting
at the \"motor\"-side. In reality, bearing friction should be
also incorporated at the driven side of the gearbox. However,
this would require considerable more effort for the measurement
of the friction parameters.
Default values for all parameters are given for joint 1.
Model relativeStates is used to define the relative angle
and relative angular velocity across the spring (=gear elasticity)
as state variables. The reason is, that a default initial
value of zero of these states makes always sense.
If the absolute angle and the absolute angular velocity of model
Jmotor would be used as states, and the load angle (= joint angle of
robot) is NOT zero, one has always to ensure that the initial values
of the motor angle and of the joint angle are modified correspondingly.
Otherwise, the spring has an unrealistic deflection at initial time.
Since relative quantities are used as state variables, this simplifies
the definition of initial values considerably.
</p>
</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100,10},{ -60, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Polygon(points = {{ -60,10},{ -60,20},{ -40,40},{ -40, -40},{ -60, -20},{ -60,10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {128,128,128}),Rectangle(extent = {{ -40,60},{40, -60}}, lineColor = {0,0,0}, pattern = LinePattern.Solid, lineThickness = 0.25, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Polygon(points = {{60,20},{40,40},{40, -40},{60, -20},{60,20}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Rectangle(extent = {{60,10},{100, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Polygon(points = {{ -60, -90},{ -50, -90},{ -20, -30},{20, -30},{48, -90},{60, -90},{60, -100},{ -60, -100},{ -60, -90}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{0,128},{0,68}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -36,40},{36, -30}}, textString = "1", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{72,30},{130,22}}, lineColor = {0,0,0}, textString = "flange of joint axis"),Text(extent = {{ -128,26},{ -70,18}}, lineColor = {0,0,0}, textString = "flange of motor axis")}));
                Modelica.Mechanics.Rotational.Components.IdealGear gear(ratio = i, useSupport = false) annotation(Placement(transformation(extent = {{50, -10},{70,10}}, rotation = 0)));
                Modelica.Mechanics.Rotational.Components.SpringDamper spring(c = c, d = d) annotation(Placement(transformation(extent = {{0, -10},{20,10}}, rotation = 0)));
                Modelica.Mechanics.Rotational.Components.BearingFriction bearingFriction(tau_pos = [0,Rv0 / unitTorque;1,(Rv0 + Rv1 * unitAngularVelocity) / unitTorque], useSupport = false) annotation(Placement(transformation(extent = {{ -60, -10},{ -40,10}}, rotation = 0)));

              equation 
                connect(spring.flange_b,gear.flange_a) annotation(Line(points = {{20,0},{50,0}}, color = {128,128,128}, thickness = 0.5));
                connect(bearingFriction.flange_b,spring.flange_a) annotation(Line(points = {{ -40,0},{0,0}}, color = {128,128,128}, thickness = 0.5));
                connect(gear.flange_b,flange_b) annotation(Line(points = {{70,0},{100,0}}, color = {128,128,128}, thickness = 0.5));
                connect(bearingFriction.flange_a,flange_a) annotation(Line(points = {{ -60,0},{ -100,0}}, color = {128,128,128}, thickness = 0.5));

              initial equation 
                spring.w_rel = 0;
                a_rel = 0;
              end GearType1;
              model GearType2 "Motor inertia and gearbox model for r3 joints 4,5,6  "
                extends Modelica.Mechanics.Rotational.Interfaces.PartialTwoFlanges;
                parameter Real i =  -99 "Gear ratio";
                parameter SI.Torque Rv0 = 21.8 "Viscous friction torque at zero velocity";
                parameter Real Rv1 = 9.8 "Viscous friction coefficient in [Nms/rad] (R=Rv0+Rv1*abs(qd))";
                parameter Real peak = 26.7 / 21.8 "Maximum static friction torque is peak*Rv0 (peak >= 1)";
                annotation(Documentation(info = "<html>
<p>
The elasticity and damping in the gearboxes of the outermost
three joints of the robot is neglected.
Default values for all parameters are given for joint 4.
</p>
</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100,10},{ -60, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Polygon(points = {{ -60,10},{ -60,20},{ -40,40},{ -40, -40},{ -60, -20},{ -60,10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {128,128,128}),Rectangle(extent = {{ -40,60},{40, -60}}, lineColor = {0,0,0}, pattern = LinePattern.Solid, lineThickness = 0.25, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Polygon(points = {{60,20},{40,40},{40, -40},{60, -20},{60,20}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Rectangle(extent = {{60,10},{100, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Polygon(points = {{ -60, -90},{ -50, -90},{ -20, -30},{20, -30},{48, -90},{60, -90},{60, -100},{ -60, -100},{ -60, -90}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{0,128},{0,68}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -36,40},{38, -30}}, textString = "2", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));
                constant SI.AngularVelocity unitAngularVelocity = 1;
                constant SI.Torque unitTorque = 1;
                Modelica.Mechanics.Rotational.Components.IdealGear gear(ratio = i, useSupport = false) annotation(Placement(transformation(extent = {{ -28, -10},{ -8,10}}, rotation = 0)));
                Modelica.Mechanics.Rotational.Components.BearingFriction bearingFriction(tau_pos = [0,Rv0 / unitTorque;1,(Rv0 + Rv1 * unitAngularVelocity) / unitTorque], peak = peak, useSupport = false) annotation(Placement(transformation(extent = {{30, -10},{50,10}}, rotation = 0)));

              equation 
                connect(gear.flange_b,bearingFriction.flange_a) annotation(Line(points = {{ -8,0},{30,0}}, color = {128,128,128}, thickness = 0.5));
                connect(bearingFriction.flange_b,flange_b) annotation(Line(points = {{50,0},{100,0}}, color = {128,128,128}, thickness = 0.5));
                connect(gear.flange_a,flange_a) annotation(Line(points = {{ -28,0},{ -100,0}}, color = {128,128,128}, thickness = 0.5));
              end GearType2;
              model Motor "Motor model including current controller of r3 motors "
                extends Modelica.Icons.MotorIcon;
                parameter SI.Inertia J(min = 0) = 0.0013 "Moment of inertia of motor";
                parameter Real k = 1.1616 "Gain of motor";
                parameter Real w = 4590 "Time constant of motor";
                parameter Real D = 0.6 "Damping constant of motor";
                parameter SI.AngularVelocity w_max = 315 "Maximum speed of motor";
                parameter SI.Current i_max = 9 "Maximum current of motor";
                annotation(Documentation(info = "<html>
<p>
Default values are given for the motor of joint 1.
The input of the motor is the desired current
(the actual current is proportional to the torque
produced by the motor).
</p>
</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{0,120},{0,60}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{80, -102},{80, -10}}, color = {255,204,51}, thickness = 0.5)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
                Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_motor annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
                Modelica.Electrical.Analog.Sources.SignalVoltage Vs annotation(Placement(transformation(origin = { -90,0}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
                Modelica.Electrical.Analog.Ideal.IdealOpAmp diff annotation(Placement(transformation(extent = {{ -64,15},{ -44,35}}, rotation = 0)));
                Modelica.Electrical.Analog.Ideal.IdealOpAmp power annotation(Placement(transformation(extent = {{16,15},{36,35}}, rotation = 0)));
                Electrical.Analog.Basic.EMF emf(k = k, useSupport = false) annotation(Placement(transformation(extent = {{46, -10},{66,10}}, rotation = 0)));
                Modelica.Electrical.Analog.Basic.Inductor La(L = 250 / (2 * D * w)) annotation(Placement(transformation(origin = {56,30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
                Modelica.Electrical.Analog.Basic.Resistor Ra(R = 250) annotation(Placement(transformation(origin = {56,60}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
                Modelica.Electrical.Analog.Basic.Resistor Rd2(R = 100) annotation(Placement(transformation(extent = {{ -86,22},{ -71,38}}, rotation = 0)));
                Modelica.Electrical.Analog.Basic.Capacitor C(C = (0.004 * D) / w) annotation(Placement(transformation(extent = {{ -14,36},{6,56}}, rotation = 0)));
                Modelica.Electrical.Analog.Ideal.IdealOpAmp OpI annotation(Placement(transformation(extent = {{ -14,10},{6,30}}, rotation = 0)));
                Modelica.Electrical.Analog.Basic.Resistor Rd1(R = 100) annotation(Placement(transformation(extent = {{ -63,37},{ -48,53}}, rotation = 0)));
                Modelica.Electrical.Analog.Basic.Resistor Ri(R = 10) annotation(Placement(transformation(extent = {{ -37,17},{ -22,33}}, rotation = 0)));
                Modelica.Electrical.Analog.Basic.Resistor Rp1(R = 200) annotation(Placement(transformation(extent = {{17,38},{32,54}}, rotation = 0)));
                Modelica.Electrical.Analog.Basic.Resistor Rp2(R = 50) annotation(Placement(transformation(origin = {11,72}, extent = {{ -8, -7},{8,7}}, rotation = 90)));
                Modelica.Electrical.Analog.Basic.Resistor Rd4(R = 100) annotation(Placement(transformation(extent = {{ -55, -15},{ -40,1}}, rotation = 0)));
                Modelica.Electrical.Analog.Sources.SignalVoltage hall2 annotation(Placement(transformation(origin = { -70, -50}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
                Modelica.Electrical.Analog.Basic.Resistor Rd3(R = 100) annotation(Placement(transformation(origin = { -70, -22}, extent = {{ -8, -7},{8,7}}, rotation = 90)));
                Modelica.Electrical.Analog.Basic.Ground g1 annotation(Placement(transformation(extent = {{ -100, -37},{ -80, -17}}, rotation = 0)));
                Modelica.Electrical.Analog.Basic.Ground g2 annotation(Placement(transformation(extent = {{ -80, -91},{ -60, -71}}, rotation = 0)));
                Modelica.Electrical.Analog.Basic.Ground g3 annotation(Placement(transformation(extent = {{ -34, -27},{ -14, -7}}, rotation = 0)));
                Modelica.Electrical.Analog.Sensors.CurrentSensor hall1 annotation(Placement(transformation(origin = {16, -50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
                Modelica.Electrical.Analog.Basic.Ground g4 annotation(Placement(transformation(extent = {{6, -84},{26, -64}}, rotation = 0)));
                Modelica.Electrical.Analog.Basic.Ground g5 annotation(Placement(transformation(origin = {11,93}, extent = {{ -10, -10},{10,10}}, rotation = 180)));
                Modelica.Mechanics.Rotational.Sensors.AngleSensor phi annotation(Placement(transformation(origin = {76, -40}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
                Modelica.Mechanics.Rotational.Sensors.SpeedSensor speed annotation(Placement(transformation(origin = {55, -40}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
                Modelica.Mechanics.Rotational.Components.Inertia Jmotor(J = J) annotation(Placement(transformation(extent = {{70, -10},{90,10}}, rotation = 0)));
                Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Components.AxisControlBus axisControlBus annotation(Placement(transformation(extent = {{60, -120},{100, -80}}, rotation = 0)));
                Blocks.Math.Gain convert1(k = 1) annotation(Placement(transformation(extent = {{ -30, -56},{ -42, -44}}, rotation = 0)));
                Blocks.Math.Gain convert2(k = 1) annotation(Placement(transformation(extent = {{ -30, -101},{ -42, -89}}, rotation = 0)));

              initial equation 
                der(C.v) = 0;
                der(La.i) = 0;

              equation 
                connect(La.n,emf.p) annotation(Line(points = {{56,20},{56,15},{56,10}}));
                connect(Ra.n,La.p) annotation(Line(points = {{56,50},{56,40}}));
                connect(Rd2.n,diff.n1) annotation(Line(points = {{ -71,30},{ -64,30}}));
                connect(C.n,OpI.p2) annotation(Line(points = {{6,46},{6,20}}));
                connect(OpI.p2,power.p1) annotation(Line(points = {{6,20},{16,20}}));
                connect(Vs.p,Rd2.p) annotation(Line(points = {{ -90,10},{ -90,30},{ -86,30}}));
                connect(diff.n1,Rd1.p) annotation(Line(points = {{ -64,30},{ -68,30},{ -68,45},{ -63,45}}));
                connect(Rd1.n,diff.p2) annotation(Line(points = {{ -48,45},{ -44,45},{ -44,25}}));
                connect(diff.p2,Ri.p) annotation(Line(points = {{ -44,25},{ -37,25}}));
                connect(Ri.n,OpI.n1) annotation(Line(points = {{ -22,25},{ -14,25}}));
                connect(OpI.n1,C.p) annotation(Line(points = {{ -14,25},{ -14,46}}));
                connect(power.n1,Rp1.p) annotation(Line(points = {{16,30},{11,30},{11,46},{17,46}}));
                connect(power.p2,Rp1.n) annotation(Line(points = {{36,25},{36,46},{32,46}}));
                connect(Rp1.p,Rp2.p) annotation(Line(points = {{17,46},{11,46},{11,64}}));
                connect(power.p2,Ra.p) annotation(Line(points = {{36,25},{42,25},{42,80},{56,80},{56,70}}));
                connect(Rd3.p,hall2.p) annotation(Line(points = {{ -70, -30},{ -70, -60}}));
                connect(Rd3.n,diff.p1) annotation(Line(points = {{ -70, -14},{ -70,20},{ -64,20}}));
                connect(Rd3.n,Rd4.p) annotation(Line(points = {{ -70, -14},{ -70, -7},{ -55, -7}}));
                connect(Vs.n,g1.p) annotation(Line(points = {{ -90, -10},{ -90, -17}}));
                connect(g2.p,hall2.n) annotation(Line(points = {{ -70, -71},{ -70, -40}}));
                connect(Rd4.n,g3.p) annotation(Line(points = {{ -40, -7},{ -24, -7}}));
                connect(g3.p,OpI.p1) annotation(Line(points = {{ -24, -7},{ -24,15},{ -14,15}}));
                connect(g5.p,Rp2.n) annotation(Line(points = {{11,83},{11,81.5},{11,81.5},{11,80}}));
                connect(emf.n,hall1.p) annotation(Line(points = {{56, -10},{56, -24},{16, -24},{16, -40}}));
                connect(hall1.n,g4.p) annotation(Line(points = {{16, -60},{16, -62},{16, -62},{16, -64}}));
                connect(emf.flange,phi.flange) annotation(Line(points = {{66,0},{66, -30},{76, -30}}, pattern = LinePattern.Dot));
                connect(emf.flange,speed.flange) annotation(Line(points = {{66,0},{66, -30},{55, -30}}, pattern = LinePattern.Dot));
                connect(OpI.n2,power.n2) annotation(Line(points = {{ -4,10},{ -4,4},{26,4},{26,15}}));
                connect(OpI.p1,OpI.n2) annotation(Line(points = {{ -14,15},{ -14,10},{ -4,10}}));
                connect(OpI.p1,diff.n2) annotation(Line(points = {{ -14,15},{ -54,15}}));
                connect(Jmotor.flange_b,flange_motor) annotation(Line(points = {{90,0},{100,0}}, color = {128,128,128}, thickness = 0.5));
                connect(phi.phi,axisControlBus.motorAngle) annotation(Line(points = {{76, -51},{76, -100},{80, -100}}, color = {0,0,127}));
                connect(speed.w,axisControlBus.motorSpeed) annotation(Line(points = {{55, -51},{55, -95},{80, -95},{80, -100}}, color = {0,0,127}));
                connect(hall1.i,axisControlBus.current) annotation(Line(points = {{6, -50},{ -10, -50},{ -10, -95},{80, -95},{80, -100}}, color = {0,0,127}));
                connect(hall1.i,convert1.u) annotation(Line(points = {{6, -50},{ -28.8, -50}}, color = {0,0,127}));
                connect(convert1.y,hall2.v) annotation(Line(points = {{ -42.6, -50},{ -63, -50}}, color = {0,0,127}));
                connect(convert2.u,axisControlBus.current_ref) annotation(Line(points = {{ -28.8, -95},{80, -95},{80, -100}}, color = {0,0,127}));
                connect(convert2.y,Vs.v) annotation(Line(points = {{ -42.6, -95},{ -108, -95},{ -108,1.28588e-15},{ -97,1.28588e-15}}, color = {0,0,127}));
                connect(emf.flange,Jmotor.flange_a) annotation(Line(points = {{66,0},{70,0}}, color = {0,0,0}, smooth = Smooth.None));
              end Motor;
              model Controller "P-PI cascade controller for one axis"
                parameter Real kp = 10 "Gain of position controller";
                parameter Real ks = 1 "Gain of speed controller";
                parameter SI.Time Ts = 0.01 "Time constant of integrator of speed controller";
                parameter Real ratio = 1 "Gear ratio of gearbox";
                annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100, -100},{100,100}}, lineColor = {0,0,0}, pattern = LinePattern.Solid, lineThickness = 0.25, fillColor = {235,235,235}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -30,54},{30,24}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{ -30,40},{ -60,50},{ -60,30},{ -30,40}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Line(points = {{ -31, -41},{ -78, -41},{ -78,39},{ -30,39}}, color = {0,0,255}),Rectangle(extent = {{ -30, -26},{30, -56}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{60, -32},{30, -42},{60, -52},{60, -32}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{30,39},{76,39},{76, -41},{30, -41}}, color = {0,0,255}),Text(extent = {{ -100,150},{100,110}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<html>
<p>
This controller has an inner PI-controller to control the motor speed,
and an outer P-controller to control the motor position of one axis.
The reference signals are with respect to the gear-output, and the
gear ratio is used in the controller to determine the motor
reference signals. All signals are communicated via the
\"axisControlBus\".
</p>
</html>"));
                Modelica.Blocks.Math.Gain gain1(k = ratio) annotation(Placement(transformation(extent = {{ -70,0},{ -50,20}}, rotation = 0)));
                Modelica.Blocks.Continuous.PI PI(k = ks, T = Ts) annotation(Placement(transformation(extent = {{60,0},{80,20}}, rotation = 0)));
                Modelica.Blocks.Math.Feedback feedback1 annotation(Placement(transformation(extent = {{ -46,0},{ -26,20}}, rotation = 0)));
                Modelica.Blocks.Math.Gain P(k = kp) annotation(Placement(transformation(extent = {{ -16,0},{4,20}}, rotation = 0)));
                Modelica.Blocks.Math.Add3 add3(k3 =  -1) annotation(Placement(transformation(extent = {{20,0},{40,20}}, rotation = 0)));
                Modelica.Blocks.Math.Gain gain2(k = ratio) annotation(Placement(transformation(extent = {{ -60,40},{ -40,60}}, rotation = 0)));
                Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Components.AxisControlBus axisControlBus annotation(Placement(transformation(extent = {{ -20, -120},{20, -80}}, rotation = 0)));

              equation 
                connect(gain1.y,feedback1.u1) annotation(Line(points = {{ -49,10},{ -44,10}}, color = {0,0,127}));
                connect(feedback1.y,P.u) annotation(Line(points = {{ -27,10},{ -18,10}}, color = {0,0,127}));
                connect(P.y,add3.u2) annotation(Line(points = {{5,10},{18,10}}, color = {0,0,127}));
                connect(gain2.y,add3.u1) annotation(Line(points = {{ -39,50},{10,50},{10,18},{18,18}}, color = {0,0,127}));
                connect(add3.y,PI.u) annotation(Line(points = {{41,10},{58,10}}, color = {0,0,127}));
                connect(gain2.u,axisControlBus.speed_ref) annotation(Line(points = {{ -62,50},{ -90,50},{ -90, -100},{0, -100}}, color = {0,0,127}));
                connect(gain1.u,axisControlBus.angle_ref) annotation(Line(points = {{ -72,10},{ -80,10},{ -80, -100},{0, -100}}, color = {0,0,127}));
                connect(feedback1.u2,axisControlBus.motorAngle) annotation(Line(points = {{ -36,2},{ -36, -100},{0, -100}}, color = {0,0,127}));
                connect(add3.u3,axisControlBus.motorSpeed) annotation(Line(points = {{18,2},{0,2},{0, -100}}, color = {0,0,127}));
                connect(PI.y,axisControlBus.current_ref) annotation(Line(points = {{81,10},{90,10},{90, -100},{0, -100}}, color = {0,0,127}));
              end Controller;
              model AxisType1 "Axis model of the r3 joints 1,2,3 "
                extends AxisType2(redeclare GearType1 gear(c = c, d = cd));
                parameter Real c(unit = "N.m/rad") = 43 "Spring constant" annotation(Dialog(group = "Gear"));
                parameter Real cd(unit = "N.m.s/rad") = 0.005 "Damper constant" annotation(Dialog(group = "Gear"));
              end AxisType1;
              model AxisType2 "Axis model of the r3 joints 4,5,6 "
                parameter Real kp = 10 "Gain of position controller" annotation(Dialog(group = "Controller"));
                parameter Real ks = 1 "Gain of speed controller" annotation(Dialog(group = "Controller"));
                parameter SI.Time Ts = 0.01 "Time constant of integrator of speed controller" annotation(Dialog(group = "Controller"));
                parameter Real k = 1.1616 "Gain of motor" annotation(Dialog(group = "Motor"));
                parameter Real w = 4590 "Time constant of motor" annotation(Dialog(group = "Motor"));
                parameter Real D = 0.6 "Damping constant of motor" annotation(Dialog(group = "Motor"));
                parameter SI.Inertia J(min = 0) = 0.0013 "Moment of inertia of motor" annotation(Dialog(group = "Motor"));
                parameter Real ratio =  -105 "Gear ratio" annotation(Dialog(group = "Gear"));
                parameter SI.Torque Rv0 = 0.4 "Viscous friction torque at zero velocity in [Nm]" annotation(Dialog(group = "Gear"));
                parameter Real Rv1(unit = "N.m.s/rad") = 0.13 / 160 "Viscous friction coefficient in [Nms/rad]" annotation(Dialog(group = "Gear"));
                parameter Real peak = 1 "Maximum static friction torque is peak*Rv0 (peak >= 1)" annotation(Dialog(group = "Gear"));
                annotation(Documentation(info = "<HTML>
<p>
The axis model consists of the <b>controller</b>, the <b>motor</b> including current
controller and the <b>gearbox</b> including gear elasticity and bearing friction.
The only difference to the axis model of joints 4,5,6 (= model axisType2) is
that elasticity and damping in the gear boxes are not neglected.
</p>
<p>
The input signals of this component are the desired angle and desired angular
velocity of the joint. The reference signals have to be \"smooth\" (position
has to be differentiable at least 2 times). Otherwise, the gear elasticity
leads to significant oscillations.
</p>
<p>
Default values of the parameters are given for the axis of joint 1.
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,50},{100, -50}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {160,160,164}),Text(extent = {{ -150,57},{150,97}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
                Modelica.Mechanics.Rotational.Interfaces.Flange_b flange annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
                replaceable GearType2 gear(Rv0 = Rv0, Rv1 = Rv1, peak = peak, i = ratio) annotation(Placement(transformation(extent = {{0, -10},{20,10}}, rotation = 0)));
                Motor motor(J = J, k = k, w = w, D = D) annotation(Placement(transformation(extent = {{ -30, -10},{ -10,10}}, rotation = 0)));
                RobotR3.Components.Controller controller(kp = kp, ks = ks, Ts = Ts, ratio = ratio) annotation(Placement(transformation(extent = {{ -70, -10},{ -50,10}}, rotation = 0)));
                Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Components.AxisControlBus axisControlBus annotation(Placement(transformation(origin = { -100,0}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
                Modelica.Mechanics.Rotational.Sensors.AngleSensor angleSensor annotation(Placement(transformation(extent = {{30,60},{50,80}}, rotation = 0)));
                Modelica.Mechanics.Rotational.Sensors.SpeedSensor speedSensor annotation(Placement(transformation(origin = {40,50}, extent = {{10,10},{ -10, -10}}, rotation = 180)));
                Modelica.Mechanics.Rotational.Sensors.AccSensor accSensor annotation(Placement(transformation(extent = {{30,20},{50,40}}, rotation = 0)));
                Modelica.Mechanics.Rotational.Components.InitializeFlange initializeFlange(stateSelect = StateSelect.prefer) annotation(Placement(transformation(extent = {{ -40, -60},{ -20, -40}}, rotation = 0)));
                Blocks.Sources.Constant const(k = 0) annotation(Placement(transformation(extent = {{ -65, -65},{ -55, -55}}, rotation = 0)));

              equation 
                connect(gear.flange_b,flange) annotation(Line(points = {{20,0},{100,0}}, color = {0,0,0}));
                connect(gear.flange_b,angleSensor.flange) annotation(Line(points = {{20,0},{20,70},{30,70}}, color = {0,0,0}));
                connect(gear.flange_b,speedSensor.flange) annotation(Line(points = {{20,0},{24,0},{24,50},{30,50}}, color = {0,0,0}));
                connect(motor.flange_motor,gear.flange_a) annotation(Line(points = {{ -10,0},{0,0}}, color = {0,0,0}));
                connect(gear.flange_b,accSensor.flange) annotation(Line(points = {{20,0},{28,0},{28,30},{30,30}}, color = {0,0,0}));
                connect(controller.axisControlBus,axisControlBus) annotation(Line(points = {{ -60, -10},{ -60, -20},{ -95, -20},{ -95, -4},{ -100, -4},{ -100,0}}, color = {255,204,51}, thickness = 0.5));
                connect(motor.axisControlBus,axisControlBus) annotation(Line(points = {{ -12, -10},{ -12, -20},{ -95, -20},{ -95, -5},{ -100, -5},{ -100,0}}, color = {255,204,51}, thickness = 0.5));
                connect(angleSensor.phi,axisControlBus.angle) annotation(Text(string = "%second", index = 1, extent = [6,3;6,3], style(color = 0, rgbcolor = {0,0,0})), Line(points = {{51,70},{70,70},{70,84},{ -98,84},{ -98,9},{ -100,9},{ -100,0}}, color = {0,0,127}));
                connect(speedSensor.w,axisControlBus.speed) annotation(Text(string = "%second", index = 1, extent = [6,3;6,3], style(color = 0, rgbcolor = {0,0,0})), Line(points = {{51,50},{74,50},{74,87},{ -100,87},{ -100,0}}, color = {0,0,127}));
                connect(accSensor.a,axisControlBus.acceleration) annotation(Text(string = "%second", index = 1, extent = [6,3;6,3], style(color = 0, rgbcolor = {0,0,0})), Line(points = {{51,30},{77,30},{77,90},{ -102,90},{ -102,0},{ -100,0}}, color = {0,0,127}));
                connect(axisControlBus.angle_ref,initializeFlange.phi_start) annotation(Text(string = "%first", index =  -1, extent = [ -6,3; -6,3], style(color = 0, rgbcolor = {0,0,0})), Line(points = {{ -100,0},{ -100, -7},{ -97, -7},{ -97, -42},{ -42, -42}}, color = {0,0,0}));
                connect(axisControlBus.speed_ref,initializeFlange.w_start) annotation(Text(string = "%first", index =  -1, extent = [ -6,3; -6,3], style(color = 0, rgbcolor = {0,0,0})), Line(points = {{ -100,0},{ -99,0},{ -99, -50},{ -42, -50}}, color = {0,0,127}));
                connect(initializeFlange.flange,flange) annotation(Line(points = {{ -20, -50},{80, -50},{80,0},{100,0}}, color = {0,0,0}));
                connect(const.y,initializeFlange.a_start) annotation(Line(points = {{ -54.5, -60},{ -48, -60},{ -48, -58},{ -42, -58}}, color = {0,0,127}));
              end AxisType2;
              model MechanicalStructure "Model of the mechanical part of the r3 robot (without animation)"
                parameter Boolean animation = true "= true, if animation shall be enabled";
                parameter SI.Mass mLoad(min = 0) = 15 "Mass of load";
                parameter SI.Position rLoad[3] = {0,0.25,0} "Distance from last flange to load mass>";
                parameter SI.Acceleration g = 9.81 "Gravity acceleration";
                SI.Angle q[6] "Joint angles";
                SI.AngularVelocity qd[6] "Joint speeds";
                SI.AngularAcceleration qdd[6] "Joint accelerations";
                SI.Torque tau[6] "Joint driving torques";
                annotation(Documentation(info = "<HTML>
<p>
This model contains the mechanical components of the r3 robot
(multibody system).
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -200, -200},{200,200}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -200,200},{200, -200}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -200,280},{200,200}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -200, -150},{ -140, -190}}, textString = "1", lineColor = {0,0,255}),Text(extent = {{ -200, -30},{ -140, -70}}, textString = "3", lineColor = {0,0,255}),Text(extent = {{ -200, -90},{ -140, -130}}, textString = "2", lineColor = {0,0,255}),Text(extent = {{ -200,90},{ -140,50}}, textString = "5", lineColor = {0,0,255}),Text(extent = {{ -200,28},{ -140, -12}}, textString = "4", lineColor = {0,0,255}),Text(extent = {{ -198,150},{ -138,110}}, textString = "6", lineColor = {0,0,255}),Bitmap(extent = {{ -130,195},{195, -195}}, fileName = "../../../../Images/MultiBody/Examples/Systems/robot_kr15.bmp")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -200, -200},{200,200}}, grid = {2,2}), graphics));
                Modelica.Mechanics.Rotational.Interfaces.Flange_a axis1 annotation(Placement(transformation(extent = {{ -220, -180},{ -200, -160}}, rotation = 0)));
                Modelica.Mechanics.Rotational.Interfaces.Flange_a axis2 annotation(Placement(transformation(extent = {{ -220, -120},{ -200, -100}}, rotation = 0)));
                Modelica.Mechanics.Rotational.Interfaces.Flange_a axis3 annotation(Placement(transformation(extent = {{ -220, -60},{ -200, -40}}, rotation = 0)));
                Modelica.Mechanics.Rotational.Interfaces.Flange_a axis4 annotation(Placement(transformation(extent = {{ -220,0},{ -200,20}}, rotation = 0)));
                Modelica.Mechanics.Rotational.Interfaces.Flange_a axis5 annotation(Placement(transformation(extent = {{ -220,60},{ -200,80}}, rotation = 0)));
                Modelica.Mechanics.Rotational.Interfaces.Flange_a axis6 annotation(Placement(transformation(extent = {{ -220,120},{ -200,140}}, rotation = 0)));
                inner Modelica.Mechanics.MultiBody.World world(g = g * Modelica.Math.Vectors.length({0, -1,0}), n = {0, -1,0}, animateWorld = false, animateGravity = false, enableAnimation = animation) annotation(Placement(transformation(extent = {{ -100, -200},{ -80, -180}}, rotation = 0)));
                Modelica.Mechanics.MultiBody.Joints.Revolute r1(n = {0,1,0}, useAxisFlange = true, animation = animation) annotation(Placement(transformation(origin = { -70, -160}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
                Modelica.Mechanics.MultiBody.Joints.Revolute r2(n = {1,0,0}, useAxisFlange = true, animation = animation) annotation(Placement(transformation(extent = {{ -50, -110},{ -30, -90}}, rotation = 0)));
                Modelica.Mechanics.MultiBody.Joints.Revolute r3(n = {1,0,0}, useAxisFlange = true, animation = animation) annotation(Placement(transformation(origin = { -50, -36}, extent = {{ -10, -10},{10,10}}, rotation = 180)));
                Modelica.Mechanics.MultiBody.Joints.Revolute r4(n = {0,1,0}, useAxisFlange = true, animation = animation) annotation(Placement(transformation(origin = { -70,10}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
                Modelica.Mechanics.MultiBody.Joints.Revolute r5(n = {1,0,0}, useAxisFlange = true, animation = animation) annotation(Placement(transformation(extent = {{ -60,70},{ -40,90}}, rotation = 0)));
                Modelica.Mechanics.MultiBody.Joints.Revolute r6(n = {0,1,0}, useAxisFlange = true, animation = animation) annotation(Placement(transformation(origin = { -60,130}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
                Modelica.Mechanics.MultiBody.Parts.BodyShape b0(r = {0,0.351,0}, shapeType = "0", r_shape = {0,0,0}, lengthDirection = {1,0,0}, widthDirection = {0,1,0}, length = 0.225, width = 0.3, height = 0.3, color = {0,0,255}, animation = animation, animateSphere = false, r_CM = {0,0,0}, m = 1) annotation(Placement(transformation(origin = { -30, -170}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
                Modelica.Mechanics.MultiBody.Parts.BodyShape b1(r = {0,0.324,0.3}, I_22 = 1.16, shapeType = "1", lengthDirection = {1,0,0}, widthDirection = {0,1,0}, length = 0.25, width = 0.15, height = 0.2, animation = animation, animateSphere = false, color = {255,0,0}, r_CM = {0,0,0}, m = 1) annotation(Placement(transformation(origin = { -70, -118}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
                Modelica.Mechanics.MultiBody.Parts.BodyShape b2(r = {0,0.65,0}, r_CM = {0.172,0.205,0}, m = 56.5, I_11 = 2.58, I_22 = 0.64, I_33 = 2.73, I_21 =  -0.46, shapeType = "2", r_shape = {0,0,0}, lengthDirection = {1,0,0}, widthDirection = {0,1,0}, length = 0.5, width = 0.2, height = 0.15, animation = animation, animateSphere = false, color = {255,178,0}) annotation(Placement(transformation(origin = { -16, -70}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
                Modelica.Mechanics.MultiBody.Parts.BodyShape b3(r = {0,0.414, -0.155}, r_CM = {0.064, -0.034,0}, m = 26.4, I_11 = 0.279, I_22 = 0.245, I_33 = 0.413, I_21 =  -0.07, shapeType = "3", r_shape = {0,0,0}, lengthDirection = {1,0,0}, widthDirection = {0,1,0}, length = 0.15, width = 0.15, height = 0.15, animation = animation, animateSphere = false, color = {255,0,0}) annotation(Placement(transformation(origin = { -86, -22}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
                Modelica.Mechanics.MultiBody.Parts.BodyShape b4(r = {0,0.186,0}, r_CM = {0,0,0}, m = 28.7, I_11 = 1.67, I_22 = 0.081, I_33 = 1.67, shapeType = "4", r_shape = {0,0,0}, lengthDirection = {1,0,0}, widthDirection = {0,1,0}, length = 0.73, width = 0.1, height = 0.1, animation = animation, animateSphere = false, color = {255,178,0}) annotation(Placement(transformation(origin = { -70,50}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
                Modelica.Mechanics.MultiBody.Parts.BodyShape b5(r = {0,0.125,0}, r_CM = {0,0,0}, m = 5.2, I_11 = 1.25, I_22 = 0.81, I_33 = 1.53, shapeType = "5", r_shape = {0,0,0}, lengthDirection = {1,0,0}, widthDirection = {0,1,0}, length = 0.225, width = 0.075, height = 0.1, animation = animation, animateSphere = false, color = {0,0,255}) annotation(Placement(transformation(origin = { -20,98}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
                Modelica.Mechanics.MultiBody.Parts.BodyShape b6(r = {0,0,0}, r_CM = {0.05,0.05,0.05}, m = 0.5, shapeType = "6", r_shape = {0,0,0}, lengthDirection = {1,0,0}, widthDirection = {0,1,0}, animation = animation, animateSphere = false, color = {0,0,255}) annotation(Placement(transformation(origin = { -60,160}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
                Modelica.Mechanics.MultiBody.Parts.BodyShape load(r_CM = rLoad, m = mLoad, r_shape = {0,0,0}, widthDirection = {1,0,0}, width = 0.05, height = 0.05, color = {255,0,0}, lengthDirection = rLoad, length = Modelica.Math.Vectors.length(rLoad), animation = animation) annotation(Placement(transformation(origin = { -60,188}, extent = {{ -10, -10},{10,10}}, rotation = 90)));

              equation 
                connect(r6.frame_b,b6.frame_a) annotation(Line(points = {{ -60,140},{ -60,150}}, color = {95,95,95}, thickness = 0.5));
                q = {r1.phi,r2.phi,r3.phi,r4.phi,r5.phi,r6.phi};
                qd = der(q);
                qdd = der(qd);
                tau = {r1.axis.tau,r2.axis.tau,r3.axis.tau,r4.axis.tau,r5.axis.tau,r6.axis.tau};
                connect(load.frame_a,b6.frame_b) annotation(Line(points = {{ -60,178},{ -60,170}}, color = {95,95,95}, thickness = 0.5));
                connect(world.frame_b,b0.frame_a) annotation(Line(points = {{ -80, -190},{ -30, -190},{ -30, -180}}, color = {95,95,95}, thickness = 0.5));
                connect(b0.frame_b,r1.frame_a) annotation(Line(points = {{ -30, -160},{ -30, -146},{ -48, -146},{ -48, -180},{ -70, -180},{ -70, -170}}, color = {95,95,95}, thickness = 0.5));
                connect(b1.frame_b,r2.frame_a) annotation(Line(points = {{ -70, -108},{ -70, -100},{ -50, -100}}, color = {95,95,95}, thickness = 0.5));
                connect(r1.frame_b,b1.frame_a) annotation(Line(points = {{ -70, -150},{ -70, -128}}, color = {95,95,95}, thickness = 0.5));
                connect(r2.frame_b,b2.frame_a) annotation(Line(points = {{ -30, -100},{ -16, -100},{ -16, -80}}, color = {95,95,95}, thickness = 0.5));
                connect(b2.frame_b,r3.frame_a) annotation(Line(points = {{ -16, -60},{ -16, -36},{ -40, -36}}, color = {95,95,95}, thickness = 0.5));
                connect(r2.axis,axis2) annotation(Line(points = {{ -40, -90},{ -42, -90},{ -42, -80},{ -160, -80},{ -160, -110},{ -210, -110}}, color = {0,0,0}));
                connect(r1.axis,axis1) annotation(Line(points = {{ -80, -160},{ -160, -160},{ -160, -170},{ -210, -170}}, color = {0,0,0}));
                connect(r3.frame_b,b3.frame_a) annotation(Line(points = {{ -60, -36},{ -88, -36},{ -86, -32}}, color = {95,95,95}, thickness = 0.5));
                connect(b3.frame_b,r4.frame_a) annotation(Line(points = {{ -86, -12},{ -86, -8},{ -70, -8},{ -70,0}}, color = {95,95,95}, thickness = 0.5));
                connect(r3.axis,axis3) annotation(Line(points = {{ -50, -46},{ -50, -50},{ -210, -50}}, color = {0,0,0}));
                connect(r4.axis,axis4) annotation(Line(points = {{ -80,10},{ -210,10}}, color = {0,0,0}));
                connect(r4.frame_b,b4.frame_a) annotation(Line(points = {{ -70,20},{ -70,40}}, color = {95,95,95}, thickness = 0.5));
                connect(b4.frame_b,r5.frame_a) annotation(Line(points = {{ -70,60},{ -70,80},{ -60,80}}, color = {95,95,95}, thickness = 0.5));
                connect(r5.axis,axis5) annotation(Line(points = {{ -50,90},{ -50,94},{ -160,94},{ -160,70},{ -210,70}}, color = {0,0,0}));
                connect(r5.frame_b,b5.frame_a) annotation(Line(points = {{ -40,80},{ -20,80},{ -20,88}}, color = {95,95,95}, thickness = 0.5));
                connect(b5.frame_b,r6.frame_a) annotation(Line(points = {{ -20,108},{ -20,116},{ -60,116},{ -60,120}}, color = {95,95,95}, thickness = 0.5));
                connect(r6.axis,axis6) annotation(Line(points = {{ -70,130},{ -210,130}}, color = {0,0,0}));
              end MechanicalStructure;
              annotation(Documentation(info = "<html>
<p>
This library contains the different components
of the r3 robot. Usually, there is no need to
use this library directly.
</p>
</html>"));
              package InternalConnectors "Internal models that should not be used"
                expandable connector AxisControlBus "Data bus for one robot axis"
                  extends Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Components.AxisControlBus;
                  import SI = Modelica.SIunits;
                  Boolean motion_ref "= true, if reference motion is not in rest";
                  SI.Angle angle_ref "Reference angle of axis flange";
                  SI.Angle angle "Angle of axis flange";
                  SI.AngularVelocity speed_ref "Reference speed of axis flange";
                  SI.AngularVelocity speed "Speed of axis flange";
                  SI.AngularAcceleration acceleration_ref "Reference acceleration of axis flange";
                  SI.AngularAcceleration acceleration "Acceleration of axis flange";
                  SI.Current current_ref "Reference current of motor";
                  SI.Current current "Current of motor";
                  SI.Angle motorAngle "Angle of motor flange";
                  SI.AngularVelocity motorSpeed "Speed of motor flange";
                end AxisControlBus;
                expandable connector ControlBus "Data bus for all axes of robot"
                  extends Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Components.ControlBus;
                  Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Components.AxisControlBus axisControlBus1 "Bus of axis 1";
                  Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Components.AxisControlBus axisControlBus2 "Bus of axis 2";
                  Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Components.AxisControlBus axisControlBus3 "Bus of axis 3";
                  Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Components.AxisControlBus axisControlBus4 "Bus of axis 4";
                  Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Components.AxisControlBus axisControlBus5 "Bus of axis 5";
                  Modelica.Mechanics.MultiBody.Examples.Systems.RobotR3.Components.AxisControlBus axisControlBus6 "Bus of axis 6";
                end ControlBus;
                annotation(Documentation(info = "<html>
<p>
This package contains the \"actual\" default bus definitions needed for the
robot example.
The bus definitions in this package are the default definitions shown in the
bus menu when connecting a signal to an expandable connector (here: ControlBus
or AxisControlBus). Usually, the connectors of this package should not be
utilized by a user.
</p>
</html>"));
              end InternalConnectors;
            end Components;
            annotation(Documentation(info = "<HTML>
<p>
This package contains models of the robot r3 of the company Manutec.
These models are used to demonstrate in which way complex
robot models might be built up by testing first the component
models individually before composing them together.
Furthermore, it is shown how CAD data can be used
for animation.
</p>
<IMG SRC=\"../Images/MultiBody/Examples/Systems/robot_kr15.bmp\"
ALT=\"model Examples.Systems.RobotR3\">
<p>
The following models are available:
</p>
<pre>
   <b>oneAxis</b>   Test one axis (controller, motor, gearbox).
   <b>fullRobot</b> Test complete robot model.
</pre>
<p>
The r3 robot is no longer manufactured. In fact the company
Manutec does no longer exist.
The parameters of this robot have been determined by measurements
in the laboratory of DLR. The measurement procedure is described in:
</p>
<pre>
   Tuerk S. (1990): Zur Modellierung der Dynamik von Robotern mit
       rotatorischen Gelenken. Fortschrittberichte VDI, Reihe 8, Nr. 211,
       VDI-Verlag 1990.
</pre>
<p>
The robot model is described in detail in
</p>
<pre>
   Otter M. (1995): Objektorientierte Modellierung mechatronischer
       Systeme am Beispiel geregelter Roboter. Dissertation,
       Fortschrittberichte VDI, Reihe 20, Nr. 147, VDI-Verlag 1995.
       This report can be downloaded as compressed postscript file
       from: <a href=\"http://www.robotic.dlr.de/Martin.Otter/publications.html\">http://www.robotic.dlr.de/Martin.Otter/publications.html</a>.
</pre>
<p>
The path planning is performed in a simple way by using essentially
the Modelica.Mechanics.Rotational.KinematicPTP block. A user defines
a path by start and end angle of every axis. A path is planned such
that all axes are moving as fast as possible under the given
restrictions of maximum joint speeds and maximum joint accelerations.
The actual r3 robot from Manutec had a different path planning strategy.
Todays path planning algorithms from robot companies are much
more involved.
</p>
<p>
In order to get a nice animation, CAD data from a KUKA robot
is used, since CAD data of the original r3 robot was not available.
The KUKA CAD data was derived from public data of KUKA available at:
<a href=\"http://www.kuka-roboter.de/english/produkte/cad/low_payloads.html\">
http://www.kuka-roboter.de/english/produkte/cad/low_payloads.html</a>.
Since dimensions of the corresponding KUKA robot are similar but not
identical to the r3 robot, the data of the r3 robot (such as arm lengths) have been modified, such that it matches the CAD data.
</p>
<p>
In this model, a simplified P-PI cascade controller for every
axes is used. The parameters have been manually adjusted by
simulations. The original r3 controllers are more complicated.
The reason to use simplified controllers is to have a simpler demo.
</p>
</HTML>
"));
          end RobotR3;
        end Systems;
        package Loops "Examples with kinematic loops"
          extends Modelica.Icons.Library;
          annotation(Documentation(info = "<html>
<p>
This package contains different examples to show how
mechanical systems with kinematic loops can be modeled.
</p>
<h4>Content</h4>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Model</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Engine1a\">Engine1a</a><br>
             <a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Engine1b\">Engine1b</a><br>
             <a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Engine1b_analytic\">Engine1b_analytic</a></td>
      <td valign=\"top\"> Model of one cylinder engine (Engine1a: simple, without combustion; Engine1b: with combustion;
           Engine1b_analytic: same as Engine1b but analytic loop handling)<br>
      <img src=\"../Images/MultiBody/Examples/Loops/Engine.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Loops.EngineV6\">EngineV6</a><br>
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Loops.EngineV6_analytic\">EngineV6_analytic</a></td>
      <td valign=\"top\"> V6 engine with 6 cylinders, 6 planar loops and 1 degree-of-freedom.
           Second version with analytic handling of kinematic loops and CAD data
           animation.<br>
      <img src=\"../Images/MultiBody/Examples/Loops/EngineV6_small.png\">
      <img src=\"../Images/MultiBody/Examples/Loops/EngineV6_CAD_smaller.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Fourbar1\">Fourbar1</a></td>
      <td valign=\"top\"> One kinematic loop with four bars (with only revolute joints;
           5 non-linear equations)<br>
      <img src=\"../Images/MultiBody/Examples/Loops/Fourbar1_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Fourbar2\">Fourbar2</a></td>
      <td valign=\"top\"> One kinematic loop with four bars (with UniversalSpherical
           joint; 1 non-linear equation) <br>
      <img src=\"../Images/MultiBody/Examples/Loops/Fourbar2_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Fourbar_analytic\">Fourbar_analytic</a></td>
      <td valign=\"top\"> One kinematic loop with four bars (with JointSSP joint;
           analytic solution of non-linear algebraic loop)  <br>
      <img src=\"../Images/MultiBody/Examples/Loops/Fourbar_analytic_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Loops.PlanarLoops_analytic\">PlanarLoops_analytic</a></td>
      <td valign=\"top\"> Mechanism with three planar kinematic loops and one
           degree-of-freedom with analytic loop handling
           (with JointRRR joints) <br>
      <img src=\"../Images/MultiBody/Examples/Loops/PlanarLoops_small.png\">
      </td>
  </tr>
</table>
</html>"));
          package Utilities "Utility models for Examples.Loops"
            extends Modelica.Icons.Library;
            model Cylinder "Cylinder with rod and crank of a combustion engine"
              import SI = Modelica.SIunits;
              parameter Boolean animation = true "= true, if animation shall be enabled";
              parameter SI.Length cylinderTopPosition = 0.42 "Length from crank shaft to end of cylinder.";
              parameter SI.Length pistonLength = 0.1 "Length of cylinder";
              parameter SI.Length rodLength = 0.2 "Length of rod";
              parameter SI.Length crankLength = 0.2 "Length of crank shaft in x direction";
              parameter SI.Length crankPinOffset = 0.1 "Offset of crank pin from center axis";
              parameter SI.Length crankPinLength = 0.1 "Offset of crank pin from center axis";
              parameter SI.Angle cylinderInclination = 0 "Inclination of cylinder";
              parameter SI.Angle crankAngleOffset = 0 "Offset for crank angle";
              parameter SI.Length cylinderLength = cylinderTopPosition - pistonLength + rodLength - crankPinOffset "Maximum length of cylinder volume";
              Modelica.Mechanics.MultiBody.Parts.BodyCylinder Piston(diameter = 0.1, r = {0,pistonLength,0}, color = {180,180,180}, animation = animation) annotation(Placement(transformation(origin = {14.5,69.5}, extent = {{10.5, -30.5},{ -10.5,30.5}}, rotation = 270)));
              Modelica.Mechanics.MultiBody.Parts.BodyBox Rod(widthDirection = {1,0,0}, height = 0.06, color = {0,0,200}, width = 0.02, r_shape = {0, -0.02,0}, r = {0,rodLength,0}, animation = animation) annotation(Placement(transformation(origin = {14,8}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
              Modelica.Mechanics.MultiBody.Joints.Revolute B2(n = {1,0,0}, cylinderLength = 0.02, animation = animation, cylinderDiameter = 0.055) annotation(Placement(transformation(extent = {{4,25},{24,45}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.BodyBox Crank4(height = 0.05, widthDirection = {1,0,0}, width = 0.02, r = {0, -crankPinOffset,0}, animation = animation) annotation(Placement(transformation(origin = {40.5, -74}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
              Modelica.Mechanics.MultiBody.Parts.BodyCylinder Crank3(r_shape = { -0.01,0,0}, length = 0.12, diameter = 0.03, r = {crankPinLength,0,0}, color = {180,180,180}, animation = animation) annotation(Placement(transformation(extent = {{4.5, -60},{24.5, -40}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.BodyCylinder Crank1(diameter = 0.05, r_shape = { -0.01,0,0}, length = 0.12, r = {crankLength - crankPinLength,0,0}, color = {180,180,180}, animation = animation) annotation(Placement(transformation(extent = {{ -50, -100},{ -30, -80}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.BodyBox Crank2(height = 0.05, widthDirection = {1,0,0}, width = 0.02, r = {0,crankPinOffset,0}, animation = animation) annotation(Placement(transformation(origin = { -10, -76}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
              Joints.RevolutePlanarLoopConstraint B1(n = {1,0,0}, cylinderLength = 0.02, animation = animation, cylinderDiameter = 0.055) annotation(Placement(transformation(extent = {{4, -27},{24, -7}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation Mid(r = {crankPinLength / 2,0,0}, animation = false) annotation(Placement(transformation(extent = {{ -6, -46},{14, -26}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Joints.Prismatic Cylinder(useAxisFlange = true, s(start =  -0.3), n = {0, -1,0}, boxWidth = 0.02) annotation(Placement(transformation(origin = {14,99}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation Mounting(r = {crankLength,0,0}, animation = false) annotation(Placement(transformation(extent = {{0,120},{20,140}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedRotation CylinderInclination(r = {crankLength - crankPinLength / 2,0,0}, n_y = {0,cos(cylinderInclination),sin(cylinderInclination)}, animation = false, rotationType = Modelica.Mechanics.MultiBody.Types.RotationTypes.TwoAxesVectors) annotation(Placement(transformation(extent = {{ -60,30},{ -40,50}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedRotation CrankAngle1(n_y = {0,cos(crankAngleOffset),sin(crankAngleOffset)}, animation = false, rotationType = Modelica.Mechanics.MultiBody.Types.RotationTypes.TwoAxesVectors) annotation(Placement(transformation(extent = {{ -90, -100},{ -70, -80}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedRotation CrankAngle2(n_y = {0,cos( -crankAngleOffset),sin( -crankAngleOffset)}, animation = false, rotationType = Modelica.Mechanics.MultiBody.Types.RotationTypes.TwoAxesVectors) annotation(Placement(transformation(extent = {{60, -100},{80, -80}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation CylinderTop(r = {0,cylinderTopPosition,0}, animation = false) annotation(Placement(transformation(origin = { -30,71}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
              GasForce gasForce(L = cylinderLength, d = 0.1) annotation(Placement(transformation(origin = {50,107}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
              Interfaces.Frame_a cylinder_a annotation(Placement(transformation(extent = {{ -116,114},{ -84,146}}, rotation = 0)));
              Interfaces.Frame_a cylinder_b annotation(Placement(transformation(extent = {{84,114},{116,146}}, rotation = 0)));
              Interfaces.Frame_a crank_a annotation(Placement(transformation(extent = {{ -116, -106},{ -84, -74}}, rotation = 0)));
              Interfaces.Frame_a crank_b annotation(Placement(transformation(extent = {{84, -106},{116, -74}}, rotation = 0)));

            equation 
              connect(B1.frame_a,Mid.frame_b) annotation(Line(points = {{4, -17},{ -6, -17},{ -6, -29},{22, -29},{22, -36},{14, -36}}, color = {95,95,95}, thickness = 0.5));
              annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -150},{100,150}}, grid = {1,1}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -150},{100,150}}, grid = {1,1}), graphics = {Polygon(points = {{ -60, -50},{ -60,100},{60,100},{60, -52},{100, -52},{100,150},{ -100,150},{ -100, -50},{ -60, -50}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -58,89},{58,13}}, lineColor = {0,0,0}, fillPattern = FillPattern.VerticalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -60,81},{60,75}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -60,67},{60,61}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -60,55},{60,49}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{ -60,11},{ -42,23},{38,23},{56,11},{ -60,11}}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Ellipse(extent = {{ -6,41},{2,33}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -40, -129},{40, -49}}, lineColor = {192,192,192}),Line(points = {{0, -90},{26, -58},{ -2,37}}, color = {0,0,0}, thickness = 1),Text(extent = {{ -150, -124},{164, -186}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -100, -90},{100, -91}}, color = {0,0,0}, thickness = 0.5)}));
              connect(Rod.frame_a,B1.frame_b) annotation(Line(points = {{14, -2},{14, -9},{30, -9},{30, -17},{24, -17}}, color = {95,95,95}, thickness = 0.5));
              connect(Cylinder.frame_b,Piston.frame_b) annotation(Line(points = {{14,89},{14,80},{14.5,80}}, color = {95,95,95}, thickness = 0.5));
              connect(Crank1.frame_a,CrankAngle1.frame_b) annotation(Line(points = {{ -50, -90},{ -70, -90}}, color = {95,95,95}, thickness = 0.5));
              connect(B2.frame_a,Piston.frame_a) annotation(Line(points = {{4,35},{ -6,35},{ -6,49},{14.5,49},{14.5,59}}, color = {95,95,95}, thickness = 0.5));
              connect(Rod.frame_b,B2.frame_b) annotation(Line(points = {{14,18},{14,23},{32,23},{32,35},{24,35}}, color = {95,95,95}, thickness = 0.5));
              connect(Crank4.frame_b,CrankAngle2.frame_a) annotation(Line(points = {{40.5, -84},{40.5, -90},{60, -90}}, color = {95,95,95}, thickness = 0.5));
              connect(Cylinder.support,gasForce.flange_b) annotation(Line(points = {{20,103},{34,103},{34,117},{50,117}}, color = {0,191,0}));
              connect(Cylinder.axis,gasForce.flange_a) annotation(Line(points = {{20,91},{50,91},{50,97}}, color = {0,191,0}));
              connect(CylinderInclination.frame_b,CylinderTop.frame_a) annotation(Line(points = {{ -40,40},{ -30,40},{ -30,61}}));
              connect(Crank1.frame_b,Crank2.frame_a) annotation(Line(points = {{ -30, -90},{ -10, -90},{ -10, -86}}, color = {95,95,95}, thickness = 0.5));
              connect(Crank3.frame_b,Crank4.frame_a) annotation(Line(points = {{24.5, -50},{40.5, -50},{40.5, -64}}, color = {95,95,95}, thickness = 0.5));
              connect(Crank3.frame_a,Crank2.frame_b) annotation(Line(points = {{4.5, -50},{ -10, -50},{ -10, -66}}, color = {95,95,95}, thickness = 0.5));
              connect(Crank2.frame_b,Mid.frame_a) annotation(Line(points = {{ -10, -66},{ -10, -36},{ -6, -36}}, color = {95,95,95}, thickness = 0.5));
              connect(CylinderTop.frame_b,Cylinder.frame_a) annotation(Line(points = {{ -30,81},{ -30,120},{14,120},{14,109}}, color = {95,95,95}, thickness = 0.5));
              connect(CylinderInclination.frame_a,cylinder_a) annotation(Line(points = {{ -60,40},{ -80,40},{ -80,130},{ -100,130}}, color = {95,95,95}, thickness = 0.5));
              connect(Mounting.frame_a,cylinder_a) annotation(Line(points = {{0,130},{ -100,130}}, color = {95,95,95}, thickness = 0.5));
              connect(Mounting.frame_b,cylinder_b) annotation(Line(points = {{20,130},{100,130}}, color = {95,95,95}, thickness = 0.5));
              connect(CrankAngle1.frame_a,crank_a) annotation(Line(points = {{ -90, -90},{ -100, -90}}, color = {95,95,95}, thickness = 0.5));
              connect(CrankAngle2.frame_b,crank_b) annotation(Line(points = {{80, -90},{100, -90}}, color = {95,95,95}, thickness = 0.5));
            end Cylinder;
            model GasForce "Simple gas force computation for combustion engine"
              import SI = Modelica.SIunits;
              extends Modelica.Mechanics.Translational.Interfaces.PartialCompliant;
              parameter SI.Length L "Length of cylinder";
              parameter SI.Diameter d "Diameter of cylinder";
              parameter SIunits.Volume k0 = 0.01 "Volume V = k0 + k1*(1-x), with x = 1 + s_rel/L";
              parameter SIunits.Volume k1 = 1 "Volume V = k0 + k1*(1-x), with x = 1 + s_rel/L";
              parameter SIunits.HeatCapacity k = 1 "Gas constant (p*V = k*T)";
              constant Real pi = Modelica.Constants.pi;
              Real x "Normalized position of cylinder";
              Real y "Normalized relative movement (= -s_rel/L)";
              SI.Density dens;
              Modelica.SIunits.Conversions.NonSIunits.Pressure_bar press "cylinder pressure";
              SI.Volume V;
              SI.Temperature T;
              SI.Velocity v_rel;
              annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -90,50},{90, -50}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -40,50},{ -20, -50}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{ -90,10},{ -40, -10}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{60,2},{54,2},{0,2},{0,10},{ -20,0},{0, -10},{0, -4},{60, -4},{60,2}}, lineColor = {255,0,0}, fillColor = {255,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -100,120},{100,60}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -90,50},{90, -50}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -40,50},{ -20, -50}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{ -90,10},{ -40, -10}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{60,2},{54,2},{0,2},{0,10},{ -20,0},{0, -10},{0, -4},{60, -4},{60,2}}, lineColor = {255,0,0}, fillColor = {255,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -100,120},{100,60}}, textString = "%name", lineColor = {0,0,255})}));
            protected 
              constant SI.Mass unitMass = 1;
              Modelica.SIunits.Pressure p;

            equation 
              y =  -s_rel / L;
              x = 1 + s_rel / L;
              v_rel = der(s_rel);
              press = p / 100000.0;
              p = (if v_rel < 0 then if x < 0.987 then 177.4132 * x ^ 4 - 287.2189 * x ^ 3 + 151.8252 * x ^ 2 - 24.9973 * x + 2.4 else 2836360 * x ^ 4 - 10569296 * x ^ 3 + 14761814 * x ^ 2 - 9158505 * x + 2129670 else if x > 0.93 then  -3929704 * x ^ 4 + 14748765 * x ^ 3 - 20747000 * x ^ 2 + 12964477 * x - 3036495 else 145.93 * x ^ 4 - 131.707 * x ^ 3 + 17.3438 * x ^ 2 + 17.9272 * x + 2.4) * 100000.0;
              f =  -(100000.0 * press * pi * d ^ 2) / 4;
              V = k0 + k1 * (1 - x);
              dens = unitMass / V;
              p / 100000.0 * V = k * T;
            end GasForce;
            model GasForce2 "Rough approximation of gas force in a cylinder"
              import SI = Modelica.SIunits;
              extends Modelica.Mechanics.Translational.Interfaces.PartialCompliant;
              parameter SI.Length L "Length of cylinder";
              parameter SI.Length d "diameter of cylinder";
              parameter SIunits.Volume k0 = 0.01 "Volume V = k0 + k1*(1-x), with x = 1 - s_rel/L";
              parameter SIunits.Volume k1 = 1 "Volume V = k0 + k1*(1-x), with x = 1 - s_rel/L";
              parameter SIunits.HeatCapacity k = 1 "Gas constant (p*V = k*T)";
              constant Real pi = Modelica.Constants.pi;
              Real x "Normalized position of cylinder (= 1 - s_rel/L)";
              SI.Density dens;
              Modelica.SIunits.AbsolutePressure press "Cylinder pressure";
              SI.Volume V;
              SI.Temperature T;
              SI.Velocity v_rel;
              annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -90,50},{90, -50}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -13,4},{ -16,4},{ -65,4},{ -65,15},{ -90,0},{ -65, -15},{ -65, -4},{ -13, -4},{ -13,4}}, lineColor = {255,0,0}, fillColor = {255,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -100,120},{100,60}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -135,44},{ -99,19}}, lineColor = {128,128,128}, textString = "a"),Text(extent = {{97,40},{133,15}}, lineColor = {128,128,128}, textString = "b"),Polygon(points = {{12,4},{70,4},{65,4},{65,15},{90,0},{65, -15},{65, -4},{12, -4},{12,4}}, lineColor = {255,0,0}, fillColor = {255,0,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -90,50},{90, -50}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -100,120},{100,60}}, textString = "%name", lineColor = {0,0,255}),Polygon(points = {{12,5},{70,5},{65,5},{65,16},{90,1},{65, -14},{65, -3},{12, -3},{12,5}}, lineColor = {255,0,0}, fillColor = {255,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{ -13,5},{ -16,5},{ -65,5},{ -65,16},{ -90,1},{ -65, -14},{ -65, -3},{ -13, -3},{ -13,5}}, lineColor = {255,0,0}, fillColor = {255,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>
The gas force in a cylinder is computed as function of the relative
distance of the two flanges. It is required that s_rel = flange_b.s - flange_a.s
is in the range
</p>
<pre>
    0 &le; s_rel &le; L
</pre>
<p>
where the parameter L is the length
of the cylinder. If this assumption is not fulfilled, an error occurs.
</p>
</html>"));
            protected 
              Modelica.SIunits.SpecificHeatCapacity R_air = Modelica.Constants.R / 0.0289651159;

            equation 
              x = 1 - s_rel / L;
              v_rel = der(s_rel);
              press = 100000.0 * (if v_rel < 0 then if x < 0.987 then 177.4132 * x ^ 4 - 287.2189 * x ^ 3 + 151.8252 * x ^ 2 - 24.9973 * x + 2.4 else 2836360 * x ^ 4 - 10569296 * x ^ 3 + 14761814 * x ^ 2 - 9158505 * x + 2129670 else if x > 0.93 then  -3929704 * x ^ 4 + 14748765 * x ^ 3 - 20747000 * x ^ 2 + 12964477 * x - 3036495 else 145.93 * x ^ 4 - 131.707 * x ^ 3 + 17.3438 * x ^ 2 + 17.9272 * x + 2.4);
              f =  -(press * pi * d ^ 2) / 4;
              V = k0 + k1 * (1 - x);
              dens = press / (R_air * T);
              press * V = k * T;
              assert(s_rel >=  -1e-12, "flange_b.s - flange_a.s (= " + String(s_rel) + ") >= 0 required for GasForce component.\n" + "Most likely, the component has to be flipped.");
              assert(s_rel <= L + 1e-12, " flange_b.s - flange_a.s (= " + String(s_rel) + " <= L (" + String(L) + ") required for GasForce component.\n" + "Most likely, parameter L is not correct.");
            end GasForce2;
            model CylinderBase "One cylinder with analytic handling of kinematic loop"
              import SI = Modelica.SIunits;
              import Cv = Modelica.SIunits.Conversions;
              parameter Boolean animation = true "= true, if animation shall be enabled";
              parameter SI.Length cylinderTopPosition = 0.42 "Length from crank shaft to end of cylinder.";
              parameter SI.Length crankLength = 0.14 "Length of crank shaft in x direction";
              parameter SI.Length crankPinOffset = 0.05 "Offset of crank pin from center axis";
              parameter SI.Length crankPinLength = 0.1 "Offset of crank pin from center axis";
              parameter Cv.NonSIunits.Angle_deg cylinderInclination = 0 "Inclination of cylinder";
              parameter Cv.NonSIunits.Angle_deg crankAngleOffset = 0 "Offset for crank angle";
              parameter SI.Length pistonLength = 0.1 " Length of cylinder" annotation(Dialog(group = "Piston"));
              parameter SI.Length pistonCenterOfMass = pistonLength / 2 " Distance from frame_a to center of mass of piston" annotation(Dialog(group = "Piston"));
              parameter SI.Mass pistonMass(min = 0) = 6 " Mass of piston" annotation(Dialog(group = "Piston"));
              parameter SI.Inertia pistonInertia_11(min = 0) = 0.0088 " Inertia 11 of piston with respect to center of mass frame, parallel to frame_a" annotation(Dialog(group = "Piston"));
              parameter SI.Inertia pistonInertia_22(min = 0) = 0.0076 " Inertia 22 of piston with respect to center of mass frame, parallel to frame_a" annotation(Dialog(group = "Piston"));
              parameter SI.Inertia pistonInertia_33(min = 0) = 0.0088 " Inertia 33 of piston with respect to center of mass frame, parallel to frame_a" annotation(Dialog(group = "Piston"));
              parameter SI.Length rodLength = 0.175 " Length of rod" annotation(Dialog(group = "Rod"));
              parameter SI.Length rodCenterOfMass = rodLength / 2 " Distance from frame_a to center of mass of piston" annotation(Dialog(group = "Rod"));
              parameter SI.Mass rodMass(min = 0) = 1 " Mass of rod" annotation(Dialog(group = "Rod"));
              parameter SI.Inertia rodInertia_11(min = 0) = 0.006 " Inertia 11 of rod with respect to center of mass frame, parallel to frame_a" annotation(Dialog(group = "Rod"));
              parameter SI.Inertia rodInertia_22(min = 0) = 0.0005 " Inertia 22 of rod with respect to center of mass frame, parallel to frame_a" annotation(Dialog(group = "Rod"));
              parameter SI.Inertia rodInertia_33(min = 0) = 0.006 " Inertia 33 of rod with respect to center of mass frame, parallel to frame_a" annotation(Dialog(group = "Rod"));
              final parameter SI.Length cylinderLength = cylinderTopPosition - pistonLength + rodLength - crankPinOffset "Maximum length of cylinder volume";
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation Mid(animation = false, r = {crankLength - crankPinLength / 2,crankPinOffset,0}) annotation(Placement(transformation(extent = {{ -44, -30},{ -24, -10}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation Mounting(r = {crankLength,0,0}, animation = false) annotation(Placement(transformation(extent = {{ -3,90},{17,110}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedRotation CylinderInclination(r = {crankLength - crankPinLength / 2,0,0}, animation = false, rotationType = Modelica.Mechanics.MultiBody.Types.RotationTypes.RotationAxis, n = {1,0,0}, angle = cylinderInclination) annotation(Placement(transformation(extent = {{ -44,30},{ -24,50}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedRotation CrankAngle(animation = false, rotationType = Modelica.Mechanics.MultiBody.Types.RotationTypes.RotationAxis, n = {1,0,0}, angle = crankAngleOffset) annotation(Placement(transformation(extent = {{ -84, -80},{ -64, -60}}, rotation = 0)));
              Joints.Assemblies.JointRRP jointRRP(n_a = {1,0,0}, n_b = {0, -1,0}, rRod1_ia = {0,rodLength,0}, animation = false, rRod2_ib =  -{0,pistonLength,0}, s_offset =  -cylinderTopPosition) annotation(Placement(transformation(origin = {0,12}, extent = {{ -20,20},{20, -20}}, rotation = 90)));
              Modelica.Mechanics.MultiBody.Parts.BodyShape Rod(animation = animation, r = {0,rodLength,0}, r_CM = {0,rodLength / 2,0}, shapeType = "2", lengthDirection = {1,0,0}, widthDirection = {0,0, -1}, length = rodLength / 1.75, width = rodLength / 1.75, height = rodLength / 1.75, color = {155,155,155}, extra = 1, r_shape = {0,0,0}, animateSphere = false, m = rodMass, I_11 = rodInertia_11, I_22 = rodInertia_22, I_33 = rodInertia_33) annotation(Placement(transformation(origin = {49,9}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
              Modelica.Mechanics.MultiBody.Parts.BodyShape Piston(animation = animation, r = {0,pistonLength,0}, r_CM = {0,pistonLength / 2,0}, shapeType = "3", length = 0.08, width = 0.08, height = 0.08, extra = 1, lengthDirection = {1,0,0}, widthDirection = {0,0, -1}, color = {180,180,180}, animateSphere = false, m = pistonMass, I_11 = pistonInertia_11, I_22 = pistonInertia_22, I_33 = pistonInertia_33) annotation(Placement(transformation(origin = {50,50}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
              GasForce gasForce(L = cylinderLength, d = 0.1) annotation(Placement(transformation(origin = { -1,70}, extent = {{ -10, -10},{10,10}}, rotation = 180)));
              annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -60, -61},{ -60,64},{60,64},{60, -61},{100, -61},{100,114},{ -100,114},{ -100, -61},{ -60, -61}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -58,63},{58, -13}}, lineColor = {0,0,0}, fillPattern = FillPattern.VerticalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -60,55},{60,49}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -60,41},{60,35}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -60,29},{60,23}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{ -57, -13},{ -39, -1},{41, -1},{59, -13},{ -57, -13}}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Ellipse(extent = {{ -6,15},{2,7}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -41, -139},{39, -59}}, lineColor = {192,192,192}),Line(points = {{ -1, -99},{25, -67},{ -2,10}}, color = {0,0,0}, thickness = 1),Text(extent = {{ -156,178},{158,116}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -100, -99},{100, -100}}, color = {0,0,0}, thickness = 0.5)}));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation Crank(animation = false, r = {crankLength,0,0}) annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}}, rotation = 0)));
              Interfaces.Frame_a cylinder_a annotation(Placement(transformation(extent = {{ -116,84},{ -84,116}}, rotation = 0)));
              Interfaces.Frame_a cylinder_b annotation(Placement(transformation(extent = {{84,84},{116,116}}, rotation = 0)));
              Interfaces.Frame_a crank_a annotation(Placement(transformation(extent = {{ -116, -116},{ -84, -84}}, rotation = 0)));
              Interfaces.Frame_a crank_b annotation(Placement(transformation(extent = {{84, -116},{116, -84}}, rotation = 0)));

            equation 
              connect(jointRRP.frame_ia,Rod.frame_a) annotation(Line(points = {{20, -4},{49, -4},{49, -1}}, color = {95,95,95}, thickness = 0.5));
              connect(Mid.frame_b,jointRRP.frame_a) annotation(Line(points = {{ -24, -20},{ -1.22465e-15, -20},{ -1.22465e-15, -8}}, color = {95,95,95}, thickness = 0.5));
              connect(gasForce.flange_a,jointRRP.axis) annotation(Line(points = {{9,70},{16,70},{16,32}}, color = {0,191,0}));
              connect(jointRRP.bearing,gasForce.flange_b) annotation(Line(points = {{8,32},{8,52},{ -20,52},{ -20,70},{ -11,70}}, color = {0,191,0}));
              connect(jointRRP.frame_ib,Piston.frame_b) annotation(Line(points = {{20,28},{30,28},{30,70},{50,70},{50,60}}, color = {95,95,95}, thickness = 0.5));
              connect(jointRRP.frame_b,CylinderInclination.frame_b) annotation(Line(points = {{1.22465e-15,32},{1,32},{1,40},{ -24,40}}, color = {95,95,95}, thickness = 0.5));
              connect(CrankAngle.frame_b,Mid.frame_a) annotation(Line(points = {{ -64, -70},{ -56, -70},{ -56, -20},{ -44, -20}}, color = {95,95,95}, thickness = 0.5));
              connect(cylinder_a,CylinderInclination.frame_a) annotation(Line(points = {{ -100,100},{ -70,100},{ -70,40},{ -44,40}}, color = {95,95,95}, thickness = 0.5));
              connect(cylinder_a,Mounting.frame_a) annotation(Line(points = {{ -100,100},{ -3,100}}, color = {95,95,95}, thickness = 0.5));
              connect(cylinder_b,Mounting.frame_b) annotation(Line(points = {{100,100},{17,100}}, color = {95,95,95}, thickness = 0.5));
              connect(CrankAngle.frame_a,crank_a) annotation(Line(points = {{ -84, -70},{ -89.5, -70},{ -89.5, -100},{ -100, -100}}, color = {95,95,95}, thickness = 0.5));
              connect(crank_a,Crank.frame_a) annotation(Line(points = {{ -100, -100},{ -10, -100}}, color = {95,95,95}, thickness = 0.5));
              connect(Crank.frame_b,crank_b) annotation(Line(points = {{10, -100},{100, -100}}, color = {95,95,95}, thickness = 0.5));
            end CylinderBase;
            model Cylinder_analytic_CAD "One cylinder with analytic handling of kinematic loop and CAD visualization"
              extends CylinderBase;
              Visualizers.FixedShape CrankShape(animation = animation, shapeType = "1", lengthDirection = {1,0,0}, extra = 1, widthDirection = {0,1,0}, length = crankPinOffset / 0.5, width = crankPinOffset / 0.5, height = crankPinOffset / 0.5, r_shape = {crankLength - crankPinLength / 2 - 0.002,0,0}) annotation(Placement(transformation(origin = { -10, -70}, extent = {{10,10},{ -10, -10}}, rotation = 180)));

            equation 
              annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
              connect(CrankShape.frame_a,CrankAngle.frame_b) annotation(Line(points = {{ -20, -70},{ -64, -70}}, color = {95,95,95}, thickness = 0.5));
            end Cylinder_analytic_CAD;
            model EngineV6_analytic "V6 engine with analytic loop handling"
              import SI = Modelica.SIunits;
              parameter Boolean animation = true "= true, if animation shall be enabled";
              replaceable model Cylinder = Cylinder_analytic_CAD extends CylinderBase "Cylinder type" annotation(choices(choice(redeclare model Cylinder = Modelica.Mechanics.MultiBody.Examples.Loops.Utilities.Cylinder_analytic_CAD "Analytic loop handling + CAD animation"), choice(redeclare model Cylinder = Modelica.Mechanics.MultiBody.Examples.Loops.Utilities.Cylinder_analytic "Analytic loop handling + standard animation")));
              Cylinder cylinder1(crankAngleOffset =  -30, cylinderInclination =  -30, animation = animation) annotation(Placement(transformation(extent = {{ -90, -10},{ -70,10}}, rotation = 0)));
              Cylinder cylinder2(crankAngleOffset = 90, cylinderInclination = 30, animation = animation) annotation(Placement(transformation(extent = {{ -60, -10},{ -40,10}}, rotation = 0)));
              Cylinder cylinder3(cylinderInclination =  -30, animation = animation, crankAngleOffset = 210) annotation(Placement(transformation(extent = {{ -30, -10},{ -10,10}}, rotation = 0)));
              Cylinder cylinder4(cylinderInclination = 30, animation = animation, crankAngleOffset = 210) annotation(Placement(transformation(extent = {{0, -10},{20,10}}, rotation = 0)));
              Cylinder cylinder5(cylinderInclination =  -30, animation = animation, crankAngleOffset = 90) annotation(Placement(transformation(extent = {{30, -10},{50,10}}, rotation = 0)));
              Cylinder cylinder6(cylinderInclination = 30, animation = animation, crankAngleOffset =  -30) annotation(Placement(transformation(extent = {{60, -10},{80,10}}, rotation = 0)));
              Joints.Revolute bearing(useAxisFlange = true, n = {1,0,0}, cylinderLength = 0.02, cylinderDiameter = 0.06, animation = true) annotation(Placement(transformation(extent = {{ -90, -40},{ -70, -60}}, rotation = 0)));
              Parts.BodyShape crank(animation = false, r_CM = {(6 * 0.1) / 2,0,0}, I_22 = 1e-05, I_33 = 1e-05, m = 6 * 30, I_11 = 0.1) annotation(Placement(transformation(extent = {{ -50, -60},{ -30, -40}}, rotation = 0)));
              Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_b annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
              Interfaces.Frame_a frame_a annotation(Placement(transformation(origin = {0, -101}, extent = {{ -16, -16},{16,16}}, rotation = 270)));

            equation 
              connect(cylinder1.crank_b,cylinder2.crank_a) annotation(Line(points = {{ -70, -10},{ -60, -10}}, color = {95,95,95}, thickness = 0.5));
              connect(cylinder2.cylinder_a,cylinder1.cylinder_b) annotation(Line(points = {{ -60,10},{ -70,10}}, color = {95,95,95}, thickness = 0.5));
              connect(cylinder3.cylinder_a,cylinder2.cylinder_b) annotation(Line(points = {{ -30,10},{ -40,10}}, color = {95,95,95}, thickness = 0.5));
              connect(cylinder3.crank_a,cylinder2.crank_b) annotation(Line(points = {{ -30, -10},{ -40, -10}}, color = {95,95,95}, thickness = 0.5));
              connect(cylinder3.cylinder_b,cylinder4.cylinder_a) annotation(Line(points = {{ -10,10},{0,10}}, color = {95,95,95}, thickness = 0.5));
              connect(cylinder3.crank_b,cylinder4.crank_a) annotation(Line(points = {{ -10, -10},{0, -10}}, color = {95,95,95}, thickness = 0.5));
              connect(cylinder4.cylinder_b,cylinder5.cylinder_a) annotation(Line(points = {{20,10},{30,10}}, color = {95,95,95}, thickness = 0.5));
              connect(cylinder4.crank_b,cylinder5.crank_a) annotation(Line(points = {{20, -10},{30, -10}}, color = {95,95,95}, thickness = 0.5));
              connect(cylinder5.cylinder_b,cylinder6.cylinder_a) annotation(Line(points = {{50,10},{60,10}}, color = {95,95,95}, thickness = 0.5));
              connect(cylinder5.crank_b,cylinder6.crank_a) annotation(Line(points = {{50, -10},{60, -10}}, color = {95,95,95}, thickness = 0.5));
              annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -137,160},{137,104}}, lineColor = {0,0,255}, textString = "%name"),Bitmap(extent = {{ -97,75},{99, -39}}, fileName = "../../../../Images/MultiBody/Examples/Loops/EngineV6_CAD.png")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
              connect(bearing.frame_b,crank.frame_a) annotation(Line(points = {{ -70, -50},{ -50, -50}}, color = {95,95,95}, thickness = 0.5));
              connect(crank.frame_b,cylinder1.crank_a) annotation(Line(points = {{ -30, -50},{ -24, -50},{ -24, -26},{ -94, -26},{ -94, -10},{ -90, -10}}, color = {95,95,95}, thickness = 0.5));
              connect(bearing.axis,flange_b) annotation(Line(points = {{ -80, -60},{ -80, -66},{90, -66},{90,0},{110,0}}, color = {0,0,0}));
              connect(frame_a,bearing.frame_a) annotation(Line(points = {{0, -101},{0, -81},{ -98, -81},{ -98, -50},{ -90, -50}}, color = {95,95,95}, thickness = 0.5));
              connect(bearing.frame_a,cylinder1.cylinder_a) annotation(Line(points = {{ -90, -50},{ -98, -50},{ -98,10},{ -90,10}}, color = {95,95,95}, thickness = 0.5));
            end EngineV6_analytic;
            partial model Engine1bBase "Model of one cylinder engine with gas force"
              import SI = Modelica.SIunits;
              extends Modelica.Icons.Example;
              annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -120},{150,120}}, grid = {1,1}), graphics), Documentation(info = "<html>
<p>
This is a model of the mechanical part of one cylinder of an engine.
The combustion is not modelled. The \"inertia\" component at the lower
left part is the output inertia of the engine driving the gearbox.
The angular velocity of the output inertia has a start value of 10 rad/s
in order to demonstrate the movement of the engine.
</p>
<p>
The engine is modeled solely by revolute and prismatic joints.
Since this results in a <b>planar</b> loop there is the well known
difficulty that the cut-forces perpendicular to the loop cannot be
uniquely computed, as well as the cut-torques within the plane.
This ambiguity is resolved by using the option <b>planarCutJoint</b>
in the <b>Advanced</b> menu of one revolute joint in every planar loop
(here: joint B1). This option sets the cut-force in direction of the
axis of rotation, as well as the cut-torques perpendicular to the axis
of rotation at this joint to zero and makes the problem mathematically
well-formed.
</p>
<p>
An animation of this example is shown in the figure below.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Loops/Engine.png\" ALT=\"model Examples.Loops.Engine\">
</p>
</html>"));
              Modelica.Mechanics.MultiBody.Parts.BodyCylinder Piston(diameter = 0.1, r = {0, -0.1,0}) annotation(Placement(transformation(origin = {120,53}, extent = {{ -10,30},{10, -30}}, rotation = 270)));
              Modelica.Mechanics.MultiBody.Parts.BodyBox Rod2(widthDirection = {1,0,0}, width = 0.02, height = 0.06, color = {0,0,200}, r = {0,0.2,0}) annotation(Placement(transformation(origin = {120,10}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
              Modelica.Mechanics.MultiBody.Joints.Revolute Bearing(useAxisFlange = true, n = {1,0,0}, cylinderLength = 0.02, cylinderDiameter = 0.05) annotation(Placement(transformation(extent = {{ -50, -80},{ -30, -100}}, rotation = 0)));
              inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -90, -100},{ -70, -80}}, rotation = 0)));
              Modelica.Mechanics.Rotational.Components.Inertia Inertia(stateSelect = StateSelect.always, J = 0.1, w(fixed = true), phi(fixed = true, start = 0.001, displayUnit = "rad")) annotation(Placement(transformation(extent = {{ -68, -120},{ -48, -100}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.BodyBox Crank4(height = 0.05, widthDirection = {1,0,0}, width = 0.02, r = {0, -0.1,0}) annotation(Placement(transformation(origin = {75.5, -75}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
              Modelica.Mechanics.MultiBody.Parts.BodyCylinder Crank3(r = {0.1,0,0}, diameter = 0.03) annotation(Placement(transformation(extent = {{41.5, -71},{61.5, -51}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.BodyCylinder Crank1(diameter = 0.05, r = {0.1,0,0}) annotation(Placement(transformation(extent = {{ -16, -100},{4, -80}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.BodyBox Crank2(height = 0.05, widthDirection = {1,0,0}, width = 0.02, r = {0,0.1,0}) annotation(Placement(transformation(origin = {30, -76}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation Mid(r = {0.05,0,0}) annotation(Placement(transformation(extent = {{30, -53},{50, -33}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation cylPosition(animation = false, r = {0.15,0.55,0}) annotation(Placement(transformation(extent = {{ -40.5,100},{ -20.5,120}}, rotation = 0)));
              Utilities.GasForce2 gasForce(d = 0.1, L = 0.35) annotation(Placement(transformation(origin = {119,97}, extent = {{10, -10},{ -10,10}}, rotation = 90)));

            equation 
              connect(world.frame_b,Bearing.frame_a) annotation(Line(points = {{ -70, -90},{ -50, -90}}, color = {95,95,95}, thickness = 0.5));
              connect(Crank2.frame_a,Crank1.frame_b) annotation(Line(points = {{30, -86},{30, -90},{4, -90}}, color = {95,95,95}, thickness = 0.5));
              connect(Crank2.frame_b,Crank3.frame_a) annotation(Line(points = {{30, -66},{30, -61},{41.5, -61}}, color = {95,95,95}, thickness = 0.5));
              connect(Bearing.frame_b,Crank1.frame_a) annotation(Line(points = {{ -30, -90},{ -16, -90}}, color = {95,95,95}, thickness = 0.5));
              connect(world.frame_b,cylPosition.frame_a) annotation(Line(points = {{ -70, -90},{ -60, -90},{ -60,110},{ -40.5,110}}, color = {95,95,95}, thickness = 0.5));
              connect(Crank3.frame_b,Crank4.frame_a) annotation(Line(points = {{61.5, -61},{75, -61},{75, -65},{75.5, -65}}, color = {95,95,95}, thickness = 0.5));
              connect(Inertia.flange_b,Bearing.axis) annotation(Line(points = {{ -48, -110},{ -40, -110},{ -40, -100}}, color = {0,0,0}));
              connect(Mid.frame_a,Crank2.frame_b) annotation(Line(points = {{30, -43},{23, -43},{23, -61},{30, -61},{30, -66}}, color = {95,95,95}, thickness = 0.5));
            end Engine1bBase;
          end Utilities;
          model PlanarLoops_analytic "Mechanism with three planar kinematic loops and one degree-of-freedom with analytic loop handling (with JointRRR joints)"
            import SI = Modelica.SIunits;
            extends Modelica.Icons.Example;
            parameter SI.Length rh[3] = {0.5,0,0} "Position vector from 'lower left' revolute to 'lower right' revolute joint for all the 3 loops";
            parameter SI.Length rv[3] = {0,0.5,0} "Position vector from 'lower left' revolute to 'upper left' revolute joint in the first loop";
            parameter SI.Length r1b[3] = {0.1,0.5,0} "Position vector from 'lower right' revolute to 'upper right' revolute joint in the first loop";
            final parameter SI.Length r1a[3] = r1b + rh - rv "Position vector from 'upper left' revolute to 'upper right' revolute joint in the first loop";
            parameter SI.Length r2b[3] = {0.1,0.6,0} "Position vector from 'lower right' revolute to 'upper right' revolute joint in the second loop";
            final parameter SI.Length r2a[3] = r2b + rh - r1b "Position vector from 'upper left' revolute to 'upper right' revolute joint in the second loop";
            parameter SI.Length r3b[3] = {0,0.55,0} "Position vector from 'lower right' revolute to 'upper right' revolute joint in the third loop";
            final parameter SI.Length r3a[3] = r3b + rh - r2b "Position vector from 'upper left' revolute to 'upper right' revolute joint in the third loop";
            annotation(experiment(StopTime = 5), Documentation(info = "<html>
<p>
It is demonstrated how the Modelica.Mechanics.MultiBody.Joints.Assemblies.JointRRR joint can be
used to solve the non-linear equations of coupled planar loops analytically.
In the mechanism below no non-linear equation occurs any more from the tool
view, since these equations are solved analytically in the JointRRR joints.
For more details, see
<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.AnalyticLoopHandling\">
MultiBody.UsersGuide.Tutorial.LoopStructures.AnalyticLoopHandling</a>.
</p>

<p>
In the following figure the parameter vectors of this example are visualized in the
animation view.
</p>

<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Loops/PlanarLoops2.png\" ALT=\"model Examples.Loops.PlanarLoops2\">
</p>

</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
            inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -94, -90},{ -74, -70}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Assemblies.JointRRR jointRRR1(rRod1_ia = r1a, rRod2_ib = r1b, checkTotalPower = true) annotation(Placement(transformation(origin = { -20,0}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Joints.Revolute rev(useAxisFlange = true, w(fixed = true)) annotation(Placement(transformation(origin = { -56, -10}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod1(r = rv) annotation(Placement(transformation(origin = { -56,30}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod2(r = rh) annotation(Placement(transformation(extent = {{ -50, -60},{ -30, -40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body1(m = 1, cylinderColor = {155,155,155}, r_CM = jointRRR1.rRod1_ia / 2) annotation(Placement(transformation(origin = {6,60}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Modelica.Mechanics.Rotational.Sources.Position position(useSupport = true) annotation(Placement(transformation(extent = {{ -90, -20},{ -70,0}}, rotation = 0)));
            Modelica.Blocks.Sources.Sine sine(amplitude = 0.7, freqHz = 1) annotation(Placement(transformation(origin = { -96,30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Joints.Assemblies.JointRRR jointRRR2(rRod1_ia = r2a, rRod2_ib = r2b, checkTotalPower = true) annotation(Placement(transformation(origin = {30,0}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod3(r = rh) annotation(Placement(transformation(extent = {{0, -60},{20, -40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body2(m = 1, cylinderColor = {155,155,155}, r_CM = jointRRR2.rRod1_ia / 2) annotation(Placement(transformation(origin = {55,61}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Joints.Assemblies.JointRRR jointRRR3(rRod1_ia = r3a, rRod2_ib = r3b, checkTotalPower = true) annotation(Placement(transformation(origin = {80,0}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod4(r = rh) annotation(Placement(transformation(extent = {{40, -60},{60, -40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body3(m = 1, cylinderColor = {155,155,155}, r_CM = jointRRR3.rRod1_ia / 2) annotation(Placement(transformation(origin = {108,62}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Parts.Mounting1D mounting1D annotation(Placement(transformation(extent = {{ -100, -50},{ -80, -30}})));

          equation 
            connect(world.frame_b,rev.frame_a) annotation(Line(points = {{ -74, -80},{ -56, -80},{ -56, -20}}, color = {95,95,95}, thickness = 0.5));
            connect(rod1.frame_a,rev.frame_b) annotation(Line(points = {{ -56,20},{ -56,0}}, color = {0,0,0}, thickness = 0.5));
            connect(rod1.frame_b,jointRRR1.frame_a) annotation(Line(points = {{ -56,40},{ -56,50},{ -20,50},{ -20,20}}, color = {95,95,95}, thickness = 0.5));
            connect(rod2.frame_a,world.frame_b) annotation(Line(points = {{ -50, -50},{ -56, -50},{ -56, -80},{ -74, -80}}, color = {95,95,95}, thickness = 0.5));
            connect(rod2.frame_b,jointRRR1.frame_b) annotation(Line(points = {{ -30, -50},{ -20, -50},{ -20, -20}}, color = {95,95,95}, thickness = 0.5));
            connect(jointRRR1.frame_ia,body1.frame_a) annotation(Line(points = {{3.55271e-15,16},{6,16},{6,50}}, color = {95,95,95}, thickness = 0.5));
            connect(rod3.frame_a,rod2.frame_b) annotation(Line(points = {{0, -50},{ -30, -50}}, color = {95,95,95}, thickness = 0.5));
            connect(rod3.frame_b,jointRRR2.frame_b) annotation(Line(points = {{20, -50},{30, -50},{30, -20}}, color = {95,95,95}, thickness = 0.5));
            connect(jointRRR2.frame_ia,body2.frame_a) annotation(Line(points = {{50,16},{55,16},{55,51}}, color = {95,95,95}, thickness = 0.5));
            connect(jointRRR1.frame_im,jointRRR2.frame_a) annotation(Line(points = {{0, -3.67394e-15},{12, -3.67394e-15},{12,25},{30,25},{30,20}}, color = {95,95,95}, thickness = 0.5));
            connect(rod3.frame_b,rod4.frame_a) annotation(Line(points = {{20, -50},{40, -50}}, color = {95,95,95}, thickness = 0.5));
            connect(rod4.frame_b,jointRRR3.frame_b) annotation(Line(points = {{60, -50},{80, -50},{80, -20}}, color = {95,95,95}, thickness = 0.5));
            connect(jointRRR2.frame_im,jointRRR3.frame_a) annotation(Line(points = {{50, -3.67394e-15},{60, -3.67394e-15},{60,26},{80,26},{80,20}}, color = {95,95,95}, thickness = 0.5));
            connect(jointRRR3.frame_ia,body3.frame_a) annotation(Line(points = {{100,16},{108,16},{108,52}}, color = {95,95,95}, thickness = 0.5));
            connect(sine.y,position.phi_ref) annotation(Line(points = {{ -96,19},{ -96, -10},{ -92, -10}}, color = {0,0,127}));
            connect(mounting1D.flange_b,position.support) annotation(Line(points = {{ -80, -40},{ -80, -20}}, color = {0,0,0}, smooth = Smooth.None));
            connect(mounting1D.frame_a,world.frame_b) annotation(Line(points = {{ -90, -50},{ -90, -57},{ -70, -57},{ -70, -80},{ -74, -80}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(position.flange,rev.axis) annotation(Line(points = {{ -70, -10},{ -66, -10}}, color = {0,0,0}, smooth = Smooth.None));
          end PlanarLoops_analytic;
          model Fourbar_analytic "One kinematic loop with four bars (with JointSSP joint; analytic solution of non-linear algebraic loop)"
            import SI = Modelica.SIunits;
            extends Modelica.Icons.Example;
            output SI.Angle j1_phi "angle of revolute joint j1";
            output SI.Position j2_s "distance of prismatic joint j2";
            output SI.AngularVelocity j1_w "axis speed of revolute joint j1";
            output SI.Velocity j2_v "axis velocity of prismatic joint j2";
            annotation(experiment(StopTime = 5), Documentation(info = "<html>
<p>
This is a third version of the \"four-bar\" mechanism, see figure:
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Loops/Fourbar_analytic.png\" ALT=\"model Examples.Loops.Fourbar_analytic\">
</p>
<p>
In this case
the three revolute joints on the left top-side and the two revolute
joints on the right top side have been replaced by the assembly joint
<b>Joints.Assemblies.JointSSP</b>
which consists of two spherical joints and one prismatic joint.
Since JointSSP solves the non-linear constraint equation internally
analytically, no non-linear equation appears any more and a Modelica
translator, such as Dymola, can transform the system into state space
form without solving a system of equations. For more details, see
<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.AnalyticLoopHandling\">
MultiBody.UsersGuide.Tutorial.LoopStructures.AnalyticLoopHandling</a>.
</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
            inner Modelica.Mechanics.MultiBody.World world(animateGravity = false) annotation(Placement(transformation(extent = {{ -80, -60},{ -60, -40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute j1(useAxisFlange = true, n = {1,0,0}, stateSelect = StateSelect.always, phi(fixed = true), w(displayUnit = "deg/s", start = 5.23598775598299, fixed = true)) annotation(Placement(transformation(extent = {{ -54, -40},{ -34, -20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.BodyCylinder b1(r = {0,0.5,0.1}, diameter = 0.05) annotation(Placement(transformation(origin = { -30, -8}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation b3(r = {1.2,0,0}, animation = false) annotation(Placement(transformation(extent = {{ -32, -60},{ -12, -40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Assemblies.JointSSP jointSSP(rod1Length = sqrt({ -1,0.3,0.1} * { -1,0.3,0.1}), n_b = {1,0,0}, s_offset =  -0.2, rRod2_ib = {0,0.2,0}, rod1Color = {0,128,255}, rod2Color = {0,128,255}, checkTotalPower = true) annotation(Placement(transformation(extent = {{ -20,0},{20,40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.BodyCylinder b2(r = {0,0.2,0}, diameter = 0.05, animation = false) annotation(Placement(transformation(origin = {50,10}, extent = {{10, -10},{ -10,10}}, rotation = 90)));

          equation 
            j1_phi = j1.phi;
            j2_s = jointSSP.prismatic.distance;
            j1_w = j1.w;
            j2_v = der(jointSSP.prismatic.distance);
            connect(j1.frame_b,b1.frame_a) annotation(Line(points = {{ -34, -30},{ -30, -30},{ -30, -18}}, color = {95,95,95}, thickness = 0.5));
            connect(j1.frame_a,world.frame_b) annotation(Line(points = {{ -54, -30},{ -60, -30},{ -60, -50}}, color = {95,95,95}, thickness = 0.5));
            connect(b3.frame_a,world.frame_b) annotation(Line(points = {{ -32, -50},{ -60, -50}}, color = {95,95,95}, thickness = 0.5));
            connect(b1.frame_b,jointSSP.frame_a) annotation(Line(points = {{ -30,2},{ -30,20},{ -20,20}}, color = {95,95,95}, thickness = 0.5));
            connect(b3.frame_b,jointSSP.frame_b) annotation(Line(points = {{ -12, -50},{30, -50},{30,20},{20,20}}, color = {95,95,95}, thickness = 0.5));
            connect(b2.frame_a,jointSSP.frame_ib) annotation(Line(points = {{50,20},{50,48},{16,48},{16,40}}, color = {95,95,95}, thickness = 0.5));
          end Fourbar_analytic;
          model Fourbar2 "One kinematic loop with four bars (with UniversalSpherical joint; 1 non-linear equation)"
            import SI = Modelica.SIunits;
            extends Modelica.Icons.Example;
            output SI.Angle j1_phi "angle of revolute joint j1";
            output SI.Position j2_s "distance of prismatic joint j2";
            output SI.AngularVelocity j1_w "axis speed of revolute joint j1";
            output SI.Velocity j2_v "axis velocity of prismatic joint j2";
            annotation(experiment(StopTime = 5), Documentation(info = "<html>
<p>
This is a second version of the \"four-bar\" mechanism, see figure:
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Loops/Fourbar2.png\" ALT=\"model Examples.Loops.Fourbar2\">
</p>
<p>
In this case
the three revolute joints on the left top-side and the two revolute
joints on the right top side have been replaced by the joint <b>UniversalSpherical</b>
that is a rod connecting a spherical and a universal joint. This joint is defined
by <b>1 constraint</b> stating that the distance between the two spherical joints is
constant. Using this joint in a kinematic loop reduces the sizes of
non-linear algebraic equations. For this loop, only one non-linear
algebraic system of equations of order 1 remains.
</p>
<p>
At the UniversalSpherical joint an additional frame_ia fixed to the rod
is present where components can be attached to the connecting rod. In this
example just a coordinate system is attached to visualize frame_ia (coordinate
system on the right in blue color).
</p>
<p>
Another feature is that the length of the connecting rod can be
automatically calculated during <b>initialization</b>. In order to do this,
another initialization condition has to be given. In this example, the
initial value of the distance of the prismatic joint j2 has been fixed
(via the \"Initialization\" menu) and the rod length of joint
\"UniversalSpherical\" is computed during initialization since parameter
<b>computeLength</b> = <b>true</b> is set in the joint parameter
menu. The main advantage is that during initialization no non-linear
system of equation is solved and therefore initialization always works.
To be precise, the following trivial non-linear equation is actually solved
for rodLength:
</p>
<pre>
   rodLength*rodLength = f(angle of revolute joint, distance of prismatic joint)
</pre>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
            inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -80, -80},{ -60, -60}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute j1(useAxisFlange = true, n = {1,0,0}, stateSelect = StateSelect.always, phi(fixed = true), w(displayUnit = "deg/s", start = 5.23598775598299, fixed = true)) annotation(Placement(transformation(extent = {{ -54, -40},{ -34, -20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Prismatic j2(n = {1,0,0}, boxWidth = 0.05, s(fixed = true, start =  -0.2)) annotation(Placement(transformation(extent = {{12, -80},{32, -60}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.BodyCylinder b1(r = {0,0.5,0.1}, diameter = 0.05) annotation(Placement(transformation(origin = { -30,0}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Parts.BodyCylinder b2(r = {0,0.2,0}, diameter = 0.05) annotation(Placement(transformation(origin = {50, -50}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Joints.UniversalSpherical universalSpherical(n1_a = {0,1,0}, computeRodLength = true, rRod_ia = { -1,0.3,0.1}) annotation(Placement(transformation(extent = {{0,18},{ -20,38}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation b3(r = {1.2,0,0}, animation = false) annotation(Placement(transformation(extent = {{ -32, -80},{ -12, -60}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Visualizers.FixedFrame fixedFrame(color_x = {0,0,255}) annotation(Placement(transformation(origin = { -6,70}, extent = {{ -10, -10},{10,10}}, rotation = 90)));

          equation 
            j1_phi = j1.phi;
            j2_s = j2.s;
            j1_w = j1.w;
            j2_v = j2.v;
            connect(j2.frame_b,b2.frame_a) annotation(Line(points = {{32, -70},{50, -70},{50, -60}}, color = {95,95,95}, thickness = 0.5));
            connect(j1.frame_b,b1.frame_a) annotation(Line(points = {{ -34, -30},{ -30, -30},{ -30, -10}}, color = {95,95,95}, thickness = 0.5));
            connect(j1.frame_a,world.frame_b) annotation(Line(points = {{ -54, -30},{ -60, -30},{ -60, -70}}, color = {95,95,95}, thickness = 0.5));
            connect(b1.frame_b,universalSpherical.frame_b) annotation(Line(points = {{ -30,10},{ -30,28},{ -20,28}}, color = {95,95,95}, thickness = 0.5));
            connect(universalSpherical.frame_a,b2.frame_b) annotation(Line(points = {{0,28},{50,28},{50, -40}}, color = {95,95,95}, thickness = 0.5));
            connect(b3.frame_a,world.frame_b) annotation(Line(points = {{ -32, -70},{ -60, -70}}, color = {95,95,95}, thickness = 0.5));
            connect(b3.frame_b,j2.frame_a) annotation(Line(points = {{ -12, -70},{12, -70}}, color = {95,95,95}, thickness = 0.5));
            connect(fixedFrame.frame_a,universalSpherical.frame_ia) annotation(Line(points = {{ -6,60},{ -6,49},{ -6,49},{ -6,38}}, color = {95,95,95}, thickness = 0.5));
          end Fourbar2;
          model Fourbar1 "One kinematic loop with four bars (with only revolute joints; 5 non-linear equations)"
            import SI = Modelica.SIunits;
            extends Modelica.Icons.Example;
            output SI.Angle j1_phi "angle of revolute joint j1";
            output SI.Position j2_s "distance of prismatic joint j2";
            output SI.AngularVelocity j1_w "axis speed of revolute joint j1";
            output SI.Velocity j2_v "axis velocity of prismatic joint j2";
            annotation(experiment(StopTime = 5), Documentation(info = "<html>
<p>
This is a simple kinematic loop consisting of 6 revolute joints, 1 prismatic joint
and 4 bars that is often used as basic constructing unit in mechanisms.
This example demonstrates that usually no particular knowledge
of the user is needed to handle kinematic loops.
Just connect the joints and bodies together according
to the real system. In particular <b>no</b> cut-joints or a spanning tree has
to be determined. In this case, the initial condition of the angular velocity
of revolute joint j1 is set to 300 deg/s in order to drive this loop.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Loops/Fourbar1.png\" ALT=\"model Examples.Loops.Fourbar1\">
</p>
</html>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
            inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -100, -80},{ -80, -60}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute j1(n = {1,0,0}, stateSelect = StateSelect.always, phi(fixed = true), w(displayUnit = "deg/s", start = 5.23598775598299, fixed = true)) annotation(Placement(transformation(extent = {{ -54, -40},{ -34, -20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Prismatic j2(n = {1,0,0}, s(start =  -0.2), boxWidth = 0.05) annotation(Placement(transformation(extent = {{10, -80},{30, -60}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.BodyCylinder b1(r = {0,0.5,0.1}, diameter = 0.05) annotation(Placement(transformation(origin = { -30,2}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Parts.BodyCylinder b2(r = {0,0.2,0}, diameter = 0.05) annotation(Placement(transformation(origin = {50, -50}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Parts.BodyCylinder b3(r = { -1,0.3,0.1}, diameter = 0.05) annotation(Placement(transformation(extent = {{38,20},{18,40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute rev(n = {0,1,0}) annotation(Placement(transformation(origin = {50, -22}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Joints.Revolute rev1 annotation(Placement(transformation(extent = {{60,0},{80,20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute j3(n = {1,0,0}) annotation(Placement(transformation(extent = {{ -60,40},{ -40,60}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute j4(n = {0,1,0}) annotation(Placement(transformation(extent = {{ -32,60},{ -12,80}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute j5(n = {0,0,1}) annotation(Placement(transformation(extent = {{0,70},{20,90}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation b0(animation = false, r = {1.2,0,0}) annotation(Placement(transformation(extent = {{ -40, -80},{ -20, -60}}, rotation = 0)));

          equation 
            connect(j2.frame_b,b2.frame_a) annotation(Line(points = {{30, -70},{50, -70},{50, -60}}, color = {95,95,95}, thickness = 0.5));
            connect(j1.frame_b,b1.frame_a) annotation(Line(points = {{ -34, -30},{ -30, -30},{ -30, -8}}, color = {95,95,95}, thickness = 0.5));
            connect(rev.frame_a,b2.frame_b) annotation(Line(points = {{50, -32},{50, -40}}, color = {95,95,95}, thickness = 0.5));
            connect(rev.frame_b,rev1.frame_a) annotation(Line(points = {{50, -12},{50,10},{60,10}}, color = {95,95,95}, thickness = 0.5));
            connect(rev1.frame_b,b3.frame_a) annotation(Line(points = {{80,10},{90,10},{90,30},{38,30}}, color = {95,95,95}, thickness = 0.5));
            connect(world.frame_b,j1.frame_a) annotation(Line(points = {{ -80, -70},{ -66, -70},{ -66, -30},{ -54, -30}}, color = {95,95,95}, thickness = 0.5));
            connect(b1.frame_b,j3.frame_a) annotation(Line(points = {{ -30,12},{ -30,28},{ -72,28},{ -72,50},{ -60,50}}, color = {95,95,95}, thickness = 0.5));
            connect(j3.frame_b,j4.frame_a) annotation(Line(points = {{ -40,50},{ -34,50},{ -42,70},{ -32,70}}, color = {95,95,95}, thickness = 0.5));
            connect(j4.frame_b,j5.frame_a) annotation(Line(points = {{ -12,70},{0,70},{0,80}}, color = {95,95,95}, thickness = 0.5));
            connect(j5.frame_b,b3.frame_b) annotation(Line(points = {{20,80},{30,80},{30,54},{4,54},{4,30},{18,30}}, color = {95,95,95}, thickness = 0.5));
            connect(b0.frame_a,world.frame_b) annotation(Line(points = {{ -40, -70},{ -80, -70}}, color = {95,95,95}, thickness = 0.5));
            connect(b0.frame_b,j2.frame_a) annotation(Line(points = {{ -20, -70},{10, -70}}, color = {95,95,95}, thickness = 0.5));
            j1_phi = j1.phi;
            j2_s = j2.s;
            j1_w = j1.w;
            j2_v = j2.v;
          end Fourbar1;
          model EngineV6_analytic "V6 engine with 6 cylinders, 6 planar loops, 1 degree-of-freedom and analytic handling of kinematic loops"
            import Cv = Modelica.SIunits.Conversions;
            import SI = Modelica.SIunits;
            extends Modelica.Icons.Example;
            parameter Boolean animation = true "= true, if animation shall be enabled";
            output Modelica.SIunits.Conversions.NonSIunits.AngularVelocity_rpm engineSpeed_rpm = Modelica.SIunits.Conversions.to_rpm(load.w) "Engine speed";
            output Modelica.SIunits.Torque engineTorque = filter.u "Torque generated by engine";
            output Modelica.SIunits.Torque filteredEngineTorque = filter.y "Filtered torque generated by engine";
            inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) annotation(Placement(transformation(extent = {{ -80, -20},{ -60,0}}, rotation = 0)));
            Utilities.EngineV6_analytic engine(redeclare model Cylinder = Modelica.Mechanics.MultiBody.Examples.Loops.Utilities.Cylinder_analytic_CAD) annotation(Placement(transformation(extent = {{ -40,0},{0,40}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia load(phi(start = 0, fixed = true), w(start = 10, fixed = true), stateSelect = StateSelect.always, J = 1) annotation(Placement(transformation(extent = {{40,10},{60,30}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque load2(tau_nominal =  -100, w_nominal = 200, useSupport = false) annotation(Placement(transformation(extent = {{90,10},{70,30}}, rotation = 0)));
            Rotational.Sensors.TorqueSensor torqueSensor annotation(Placement(transformation(extent = {{12,10},{32,30}}, rotation = 0)));
            Blocks.Continuous.CriticalDamping filter(n = 2, initType = Modelica.Blocks.Types.Init.SteadyState, f = 5) annotation(Placement(transformation(extent = {{30, -20},{50,0}}, rotation = 0)));

          equation 
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<HTML>
<p>
This is a similar model as the example \"EngineV6\". However, the cylinders
have been built up with component Modelica.Mechanics.MultiBody.Joints.Assemblies.JointRRR that
solves the non-linear system of equations in an aggregation of 3 revolution
joints <b>analytically</b> and only one body is used that holds the total
mass of the crank shaft:
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Loops/EngineV6_CAD_small.png\">
</p>
<p>
This model is about 20 times faster as the EngineV6 example and <b>no</b> linear or
non-linear system of equations occur. In contrast, the \"EngineV6\" example
leads to 6 systems of nonlinear equations (every system has dimension = 5, with
Evaluate=false and dimension=1 with Evaluate=true) and a linear system of equations
of about 40. This shows the power of the analytic loop handling.
</p>

<p>
Simulate for 5 s, and plot the variables <b>engineSpeed_rpm</b>,
<b>engineTorque</b>, and <b>filteredEngineTorque</b>. Note, the result file has
a size of about 50 Mbyte (for 5000 output intervalls).
</p>
</HTML>
"));
            connect(world.frame_b,engine.frame_a) annotation(Line(points = {{ -60, -10},{ -20, -10},{ -20, -0.2}}, color = {95,95,95}, thickness = 0.5));
            connect(load2.flange,load.flange_b) annotation(Line(points = {{70,20},{60,20}}, color = {0,0,0}));
            connect(torqueSensor.flange_a,engine.flange_b) annotation(Line(points = {{12,20},{2,20}}, color = {0,0,0}));
            connect(torqueSensor.flange_b,load.flange_a) annotation(Line(points = {{32,20},{40,20}}, color = {0,0,0}));
            connect(torqueSensor.tau,filter.u) annotation(Line(points = {{14,9},{14, -10},{28, -10}}, color = {0,0,127}));
          end EngineV6_analytic;
          model EngineV6 "V6 engine with 6 cylinders, 6 planar loops and 1 degree-of-freedom"
            import Cv = Modelica.SIunits.Conversions;
            extends Modelica.Icons.Example;
            parameter Boolean animation = true "= true, if animation shall be enabled";
            output Modelica.SIunits.Conversions.NonSIunits.AngularVelocity_rpm engineSpeed_rpm = Modelica.SIunits.Conversions.to_rpm(load.w) "Engine speed";
            output Modelica.SIunits.Torque engineTorque = filter.u "Torque generated by engine";
            output Modelica.SIunits.Torque filteredEngineTorque = filter.y "Filtered torque generated by engine";
            Modelica.Mechanics.MultiBody.Joints.Revolute bearing(useAxisFlange = true, n = {1,0,0}, cylinderLength = 0.02, cylinderDiameter = 0.06, animation = animation) annotation(Placement(transformation(extent = {{ -90,20},{ -70,0}}, rotation = 0)));
            inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) annotation(Placement(transformation(extent = {{ -120,20},{ -100,40}}, rotation = 0)));
            Utilities.Cylinder cylinder1(crankAngleOffset = Cv.from_deg( -30), cylinderInclination = Cv.from_deg( -30), animation = animation) annotation(Placement(transformation(extent = {{ -50,4},{ -30,34}}, rotation = 0)));
            Utilities.Cylinder cylinder2(crankAngleOffset = Cv.from_deg(90), cylinderInclination = Cv.from_deg(30), animation = animation) annotation(Placement(transformation(extent = {{ -20,4},{0,34}}, rotation = 0)));
            Utilities.Cylinder cylinder3(cylinderInclination = Cv.from_deg( -30), animation = animation, crankAngleOffset = Cv.from_deg(210)) annotation(Placement(transformation(extent = {{10,4},{30,34}}, rotation = 0)));
            Utilities.Cylinder cylinder4(cylinderInclination = Cv.from_deg(30), animation = animation, crankAngleOffset = Cv.from_deg(210)) annotation(Placement(transformation(extent = {{39,4},{59,34}}, rotation = 0)));
            Utilities.Cylinder cylinder5(cylinderInclination = Cv.from_deg( -30), animation = animation, crankAngleOffset = Cv.from_deg(90)) annotation(Placement(transformation(extent = {{70,4},{90,34}}, rotation = 0)));
            Utilities.Cylinder cylinder6(cylinderInclination = Cv.from_deg(30), animation = animation, crankAngleOffset = Cv.from_deg( -30)) annotation(Placement(transformation(extent = {{100,4},{120,34}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia load(phi(start = 0, fixed = true), w(start = 10, fixed = true), stateSelect = StateSelect.always, J = 1) annotation(Placement(transformation(extent = {{ -44, -30},{ -24, -10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque load2(tau_nominal =  -100, w_nominal = 200, useSupport = false) annotation(Placement(transformation(extent = {{6, -30},{ -14, -10}}, rotation = 0)));
            Rotational.Sensors.TorqueSensor torqueSensor annotation(Placement(transformation(extent = {{ -72, -30},{ -52, -10}}, rotation = 0)));
            Blocks.Continuous.CriticalDamping filter(n = 2, initType = Modelica.Blocks.Types.Init.SteadyState, f = 5) annotation(Placement(transformation(extent = {{ -54, -60},{ -34, -40}}, rotation = 0)));

          equation 
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -120, -100},{120,100}}, grid = {1,1}), graphics), Documentation(info = "<HTML>
<p>
This is a V6 engine with 6 cylinders. It is hierarchically built
up by using instances of one cylinder. For more details on the
modeling of one cylinder, see example
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Engine1b\">Engine1b</a>.
An animation of the engine is shown in the figure below.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Loops/EngineV6.png\" ALT=\"model Examples.Loops.EngineV6\">
</p>

<p>
Simulate for 5 s, and plot the variables <b>engineSpeed_rpm</b>,
<b>engineTorque</b>, and <b>filteredEngineTorque</b>. Note, the result file has
a size of about 50 Mbyte (for 5000 output intervalls).
</p>
</HTML>
"));
            connect(bearing.frame_b,cylinder1.crank_a) annotation(Line(points = {{ -70,10},{ -50,10}}, color = {95,95,95}, thickness = 0.5));
            connect(cylinder1.crank_b,cylinder2.crank_a) annotation(Line(points = {{ -30,10},{ -20,10}}, color = {95,95,95}, thickness = 0.5));
            connect(cylinder2.crank_b,cylinder3.crank_a) annotation(Line(points = {{0,10},{10,10}}, color = {95,95,95}, thickness = 0.5));
            connect(cylinder3.crank_b,cylinder4.crank_a) annotation(Line(points = {{30,10},{39,10}}, color = {95,95,95}, thickness = 0.5));
            connect(cylinder4.crank_b,cylinder5.crank_a) annotation(Line(points = {{59,10},{70,10}}, color = {95,95,95}, thickness = 0.5));
            connect(cylinder5.crank_b,cylinder6.crank_a) annotation(Line(points = {{90,10},{100,10}}, color = {95,95,95}, thickness = 0.5));
            connect(cylinder5.cylinder_b,cylinder6.cylinder_a) annotation(Line(points = {{90,32},{100,32}}, color = {95,95,95}, thickness = 0.5));
            connect(cylinder4.cylinder_b,cylinder5.cylinder_a) annotation(Line(points = {{59,32},{70,32}}, color = {95,95,95}, thickness = 0.5));
            connect(cylinder4.cylinder_a,cylinder3.cylinder_b) annotation(Line(points = {{39,32},{30,32}}, color = {95,95,95}, thickness = 0.5));
            connect(cylinder3.cylinder_a,cylinder2.cylinder_b) annotation(Line(points = {{10,32},{0,32}}, color = {95,95,95}, thickness = 0.5));
            connect(cylinder2.cylinder_a,cylinder1.cylinder_b) annotation(Line(points = {{ -20,32},{ -30,32}}, color = {95,95,95}, thickness = 0.5));
            connect(world.frame_b,cylinder1.cylinder_a) annotation(Line(points = {{ -100,30},{ -59,30},{ -59,32},{ -50,32}}, color = {95,95,95}, thickness = 0.5));
            connect(world.frame_b,bearing.frame_a) annotation(Line(points = {{ -100,30},{ -96,30},{ -96,10},{ -90,10}}, color = {95,95,95}, thickness = 0.5));
            connect(load2.flange,load.flange_b) annotation(Line(points = {{ -14, -20},{ -24, -20}}, color = {0,0,0}));
            connect(torqueSensor.flange_b,load.flange_a) annotation(Line(points = {{ -52, -20},{ -44, -20}}, color = {0,0,0}));
            connect(torqueSensor.tau,filter.u) annotation(Line(points = {{ -70, -31},{ -70, -50},{ -56, -50}}, color = {0,0,127}));
            connect(torqueSensor.flange_a,bearing.axis) annotation(Line(points = {{ -72, -20},{ -80, -20},{ -80,0}}, color = {0,0,0}));
          end EngineV6;
          model Engine1b_analytic "Model of one cylinder engine with gas force and analytic loop handling"
            import SI = Modelica.SIunits;
            extends Modelica.Icons.Example;
            extends Utilities.Engine1bBase;
            annotation(experiment(StopTime = 0.5), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -120},{150,120}}, grid = {1,1}), graphics), Documentation(info = "<html>
<p>
This is the same model as
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Engine1b#diagram\">Loops.Engine1b</a>.
The only difference is that the central part of
the planar kinematic loop has been replaced by the
assembly joint \"Modelica.Mechanics.MultiBody.Joints.Assemblies.<b>JointRRP</b>\".
The advantage of using JointRRP is, that the
non-linear algebraic equation of this loop is solved analytically, and
not numerically as in
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Engine1b#diagram\">Loops.Engine1b</a>.
</p>
<p>
An animation of this example is shown in the figure below.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Loops/Engine.png\" ALT=\"model Examples.Loops.Engine\">
</p>
</html>"));
            Joints.Assemblies.JointRRP jointRRP(n_a = {1,0,0}, n_b = {0, -1,0}, animation = false, rRod1_ia = {0,0.2,0}, rRod2_ib = {0, -0.1,0}) annotation(Placement(transformation(origin = {30,54}, extent = {{ -20,20},{20, -20}}, rotation = 90)));

          equation 
            connect(Mid.frame_b,jointRRP.frame_a) annotation(Line(points = {{50, -43},{58, -43},{58, -30},{30, -30},{30,34}}, color = {95,95,95}, thickness = 0.5));
            connect(jointRRP.frame_b,cylPosition.frame_b) annotation(Line(points = {{30,74},{30,110},{ -20.5,110}}, color = {95,95,95}, thickness = 0.5));
            connect(jointRRP.axis,gasForce.flange_b) annotation(Line(points = {{46,74},{46,82},{119,82},{119,87}}, color = {0,127,0}));
            connect(jointRRP.bearing,gasForce.flange_a) annotation(Line(points = {{38,74},{38,114},{119,114},{119,107}}, color = {0,127,0}));
            connect(jointRRP.frame_ib,Piston.frame_a) annotation(Line(points = {{50,70},{120,70},{120,63}}, color = {95,95,95}, thickness = 0.5));
            connect(jointRRP.frame_ia,Rod2.frame_a) annotation(Line(points = {{50,38},{80,38},{80, -10},{120, -10},{120,0}}, color = {95,95,95}, thickness = 0.5));
          end Engine1b_analytic;
          model Engine1b "Model of one cylinder engine with gas force and preparation for assembly joint JointRRP"
            import SI = Modelica.SIunits;
            extends Modelica.Icons.Example;
            extends Utilities.Engine1bBase;
            annotation(experiment(StopTime = 0.5), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -120},{150,120}}, grid = {1,1}), graphics = {Rectangle(extent = {{3,117},{88, -23}}, lineColor = {255,0,0}, lineThickness = 0.5),Text(extent = {{65, -30},{145, -39}}, lineColor = {255,0,0}, lineThickness = 0.5, textString = "jointRRP in model"),Text(extent = {{66, -34},{141, -52}}, lineColor = {255,0,0}, lineThickness = 0.5, textString = "Loops.Engine1b_analytic")}), Documentation(info = "<html>
<p>
This is a model of the mechanical part of one cylinder of an engine.
It is similiar to
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Engine1a#diagram\">Loops.Engine1a</a>.
The difference is that a simple
model for the gas force in the cylinder is added and that the
model is restructured in such a way, that the central part of
the planar kinematic loop can be easily replaced by the
assembly joint \"Modelica.Mechanics.MultiBody.Joints.Assemblies.<b>JointRRP</b>\".
This exchange of the kinematic loop is shown in
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Engine1b_analytic#diagram\">Loops.Engine1b_analytic</a>.
The advantage of using JointRRP is, that the
non-linear algebraic equation of this loop is solved analytically, and
not numerically as in this model (Engine1b).
</p>
<p>
An animation of this example is shown in the figure below.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Loops/Engine.png\" ALT=\"model Examples.Loops.Engine\">
</p>
</html>"));
            Joints.RevolutePlanarLoopConstraint B2(n = {1,0,0}, cylinderLength = 0.02, cylinderDiameter = 0.05) annotation(Placement(transformation(extent = {{40,20},{60,40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute B1(n = {1,0,0}, cylinderLength = 0.02, cylinderDiameter = 0.05) annotation(Placement(transformation(extent = {{40, -20},{60,0}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Prismatic Cylinder(useAxisFlange = true, boxWidth = 0.02, n = {0, -1,0}) annotation(Placement(transformation(origin = {50,97}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Parts.FixedTranslation Rod1(r = {0,0.2,0}, animation = false) annotation(Placement(transformation(origin = {70,10}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Parts.FixedTranslation Rod3(r = {0, -0.1,0}, animation = false) annotation(Placement(transformation(origin = {50,58}, extent = {{10, -10},{ -10,10}}, rotation = 90)));

          equation 
            connect(B1.frame_b,Rod1.frame_a) annotation(Line(points = {{60, -10},{70, -10},{70,0}}, color = {95,95,95}, thickness = 0.5));
            connect(Rod1.frame_b,B2.frame_b) annotation(Line(points = {{70,20},{70,30},{60,30}}, color = {95,95,95}, thickness = 0.5));
            connect(Cylinder.frame_b,Rod3.frame_a) annotation(Line(points = {{50,87},{50,68}}, color = {95,95,95}, thickness = 0.5));
            connect(B2.frame_a,Rod3.frame_b) annotation(Line(points = {{40,30},{30,30},{30,44},{50,44},{50,48}}, color = {95,95,95}, thickness = 0.5));
            connect(cylPosition.frame_b,Cylinder.frame_a) annotation(Line(points = {{ -20.5,110},{50,110},{50,107}}, color = {95,95,95}, thickness = 0.5));
            connect(gasForce.flange_a,Cylinder.support) annotation(Line(points = {{119,107},{119,112},{70,112},{70,101},{56,101}}, color = {0,127,0}));
            connect(Cylinder.axis,gasForce.flange_b) annotation(Line(points = {{56,89},{71,89},{71,80},{119,80},{119,87}}, color = {0,127,0}));
            connect(Piston.frame_a,Rod3.frame_a) annotation(Line(points = {{120,63},{120,75},{50,75},{50,68}}, color = {95,95,95}, thickness = 0.5));
            connect(B1.frame_b,Rod2.frame_a) annotation(Line(points = {{60, -10},{120, -10},{120,0}}, color = {95,95,95}, thickness = 0.5));
            connect(Mid.frame_b,B1.frame_a) annotation(Line(points = {{50, -43},{57, -43},{57, -28},{30, -28},{30, -10},{40, -10}}, color = {95,95,95}, thickness = 0.5));
          end Engine1b;
          model Engine1a "Model of one cylinder engine"
            import SI = Modelica.SIunits;
            extends Modelica.Icons.Example;
            annotation(experiment(StopTime = 5), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -120},{150,120}}, grid = {1,1}), graphics), Documentation(info = "<html>
<p>
This is a model of the mechanical part of one cylinder of an engine.
The combustion is not modelled. The \"inertia\" component at the lower
left part is the output inertia of the engine driving the gearbox.
The angular velocity of the output inertia has a start value of 10 rad/s
in order to demonstrate the movement of the engine.
</p>
<p>
The engine is modeled solely by revolute and prismatic joints.
Since this results in a <b>planar</b> loop there is the well known
difficulty that the cut-forces perpendicular to the loop cannot be
uniquely computed, as well as the cut-torques within the plane.
This ambiguity is resolved by using the option <b>planarCutJoint</b>
in the <b>Advanced</b> menu of one revolute joint in every planar loop
(here: joint B1). This option sets the cut-force in direction of the
axis of rotation, as well as the cut-torques perpendicular to the axis
of rotation at this joint to zero and makes the problem mathematically
well-formed.
</p>
<p>
An animation of this example is shown in the figure below.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Loops/Engine.png\" ALT=\"model Examples.Loops.Engine\">
</p>
</html>"));
            Modelica.Mechanics.MultiBody.Parts.BodyCylinder Piston(diameter = 0.1, r = {0, -0.1,0}) annotation(Placement(transformation(origin = {90.5,66.5}, extent = {{ -10.5,30.5},{10.5, -30.5}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Parts.BodyBox Rod(widthDirection = {1,0,0}, width = 0.02, height = 0.06, r = {0, -0.2,0}, color = {0,0,200}) annotation(Placement(transformation(origin = {90,5}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Joints.Revolute B2(n = {1,0,0}, cylinderLength = 0.02, cylinderDiameter = 0.05) annotation(Placement(transformation(extent = {{80,22},{100,42}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute Bearing(useAxisFlange = true, n = {1,0,0}, cylinderLength = 0.02, cylinderDiameter = 0.05) annotation(Placement(transformation(extent = {{ -10, -80},{10, -100}}, rotation = 0)));
            inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -50, -100},{ -30, -80}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Inertia Inertia(stateSelect = StateSelect.always, phi(fixed = true, start = 0), w(fixed = true, start = 10), J = 1) annotation(Placement(transformation(extent = {{ -28, -120},{ -8, -100}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.BodyBox Crank4(height = 0.05, widthDirection = {1,0,0}, width = 0.02, r = {0, -0.1,0}) annotation(Placement(transformation(origin = {115.5, -75}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Parts.BodyCylinder Crank3(r = {0.1,0,0}, diameter = 0.03) annotation(Placement(transformation(extent = {{81.5, -71},{101.5, -51}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.BodyCylinder Crank1(diameter = 0.05, r = {0.1,0,0}) annotation(Placement(transformation(extent = {{24, -100},{44, -80}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.BodyBox Crank2(r = {0,0.1,0}, height = 0.05, widthDirection = {1,0,0}, width = 0.02) annotation(Placement(transformation(origin = {70, -76}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Joints.RevolutePlanarLoopConstraint B1(n = {1,0,0}, cylinderLength = 0.02, cylinderDiameter = 0.05) annotation(Placement(transformation(extent = {{80, -30},{100, -10}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation Mid(r = {0.05,0,0}) annotation(Placement(transformation(extent = {{70, -53},{90, -33}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Prismatic Cylinder(boxWidth = 0.02, n = {0, -1,0}, s(start = 0.15)) annotation(Placement(transformation(origin = {90,96}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation cylPosition(animation = false, r = {0.15,0.45,0}) annotation(Placement(transformation(extent = {{ -0.5,100},{19.5,120}}, rotation = 0)));

          equation 
            connect(world.frame_b,Bearing.frame_a) annotation(Line(points = {{ -30, -90},{ -10, -90}}, color = {95,95,95}, thickness = 0.5));
            connect(Crank2.frame_a,Crank1.frame_b) annotation(Line(points = {{70, -86},{70, -90},{44, -90}}, color = {95,95,95}, thickness = 0.5));
            connect(Crank2.frame_b,Crank3.frame_a) annotation(Line(points = {{70, -66},{70, -61},{81.5, -61}}, color = {95,95,95}, thickness = 0.5));
            connect(Bearing.frame_b,Crank1.frame_a) annotation(Line(points = {{10, -90},{24, -90}}, color = {95,95,95}, thickness = 0.5));
            connect(cylPosition.frame_b,Cylinder.frame_a) annotation(Line(points = {{19.5,110},{90,110},{90,106}}, color = {95,95,95}, thickness = 0.5));
            connect(world.frame_b,cylPosition.frame_a) annotation(Line(points = {{ -30, -90},{ -20, -90},{ -20,110},{ -0.5,110}}, color = {95,95,95}, thickness = 0.5));
            connect(Crank3.frame_b,Crank4.frame_a) annotation(Line(points = {{101.5, -61},{115, -61},{115, -65},{115.5, -65}}, color = {95,95,95}, thickness = 0.5));
            connect(B1.frame_a,Mid.frame_b) annotation(Line(points = {{80, -20},{70, -20},{70, -32},{98, -32},{98, -43},{90, -43}}, color = {95,95,95}, thickness = 0.5));
            connect(B1.frame_b,Rod.frame_b) annotation(Line(points = {{100, -20},{112, -20},{112, -9},{90, -9},{90, -5}}, color = {95,95,95}, thickness = 0.5));
            connect(Rod.frame_a,B2.frame_b) annotation(Line(points = {{90,15},{90,21},{110,21},{110,32},{100,32}}, color = {95,95,95}, thickness = 0.5));
            connect(B2.frame_a,Piston.frame_b) annotation(Line(points = {{80,32},{70,32},{70,46},{90.5,46},{90.5,56}}, color = {95,95,95}, thickness = 0.5));
            connect(Inertia.flange_b,Bearing.axis) annotation(Line(points = {{ -8, -110},{0, -110},{0, -100}}, color = {0,0,0}));
            connect(Mid.frame_a,Crank2.frame_b) annotation(Line(points = {{70, -43},{63, -43},{63, -61},{70, -61},{70, -66}}, color = {95,95,95}, thickness = 0.5));
            connect(Cylinder.frame_b,Piston.frame_a) annotation(Line(points = {{90,86},{90,77},{90.5,77}}, color = {95,95,95}, thickness = 0.5));
          end Engine1a;
        end Loops;
        package Elementary "Elementary examples to demonstrate various features of the MultiBody library"
          extends Modelica.Icons.Library;
          annotation(Documentation(info = "<HTML>
<p>
This package contains elementary example models to demonstrate
the usage of the MultiBody library
</p>
<h4>Content</h4>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Model</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.DoublePendulum\">DoublePendulum</a></td>
      <td valign=\"top\"> Simple double pendulum with two revolute joints and two bodies.<br>
      <img src=\"../Images/MultiBody/Examples/Elementary/DoublePendulumSmall.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.ForceAndTorque\">ForceAndTorque</a></td>
      <td valign=\"top\"> Demonstrates usage of Forces.ForceAndTorque element.<br>
      <img src=\"../Images/MultiBody/Examples/Elementary/ForceAndTorque_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.FreeBody\">FreeBody</a></td>
      <td valign=\"top\"> Free flying body attached by two springs to environment.<br>
      <img src=\"../Images/MultiBody/Examples/Elementary/FreeBody_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.InitSpringConstant\">InitSpringConstant</a></td>
      <td valign=\"top\"> Determine spring constant such that system is in steady state
           at given position.<br>
      <img src=\"../Images/MultiBody/Examples/Elementary/InitSpringConstant_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.LineForceWithTwoMasses\">LineForceWithTwoMasses</a></td>
      <td valign=\"top\"> Demonstrates a line force with two point masses using a
           Joints.Assemblies.JointUPS and alternatively a
           Forces.LineForceWithTwoMasses component.<br>
      <img src=\"../Images/MultiBody/Examples/Elementary/LineForceWithTwoMasses_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.Pendulum\">Pendulum</a></td>
      <td valign=\"top\"> Simple pendulum with one revolute joint and one body. <br>
      <img src=\"../Images/MultiBody/Examples/Elementary/Pendulum_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.PendulumWithSpringDamper\">PendulumWithSpringDamper</a></td>
      <td valign=\"top\"> Simple spring/damper/mass system <br>
      <img src=\"../Images/MultiBody/Examples/Elementary/PendulumWithSpringDamper_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.PointGravity\">PointGravity</a></td>
      <td valign=\"top\"> Two bodies in a point gravity field <br>
      <img src=\"../Images/MultiBody/Examples/Elementary/PointGravity_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.PointGravityWithPointMasses\">PointGravityWithPointMasses</a></td>
      <td valign=\"top\"> Two point masses in a point gravity field (rotation of bodies is neglected) <br>
      <img src=\"../Images/MultiBody/Examples/Elementary/PointGravityWithPointMasses_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.PointGravityWithPointMasses2\">PointGravityWithPointMasses2</a></td>
      <td valign=\"top\"> Rigidly connected point masses in a point gravity field <br>
      <img src=\"../Images/MultiBody/Examples/Elementary/PointGravityWithPointMasses2_small.png\">
      </td>
  </tr>

  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.RollingWheel\">RollingWheel</a></td>
      <td valign=\"top\"> Single wheel rolling on ground starting from an initial speed <br>
      <img src=\"../Images/MultiBody/Examples/Elementary/RollingWheel.png\">
      </td>
  </tr>


  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.RollingWheelSetDriving\">RollingWheelSetDriving</a></td>
      <td valign=\"top\"> Rolling wheel set that is driven by torques driving the wheels <br>
      <img src=\"../Images/MultiBody/Examples/Elementary/RollingWheelSetDriving.png\">
      </td>
  </tr>


  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.RollingWheelSetPulling\">RollingWheelSetPulling</a></td>
      <td valign=\"top\"> Rolling wheel set that is pulled by a force <br>
      <img src=\"../Images/MultiBody/Examples/Elementary/RollingWheelSetPulling.png\">
      </td>
  </tr>


  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.SpringDamperSystem\">SpringDamperSystem</a></td>
      <td valign=\"top\"> Spring/damper system with a prismatic joint and
           attached on free flying body <br>
      <img src=\"../Images/MultiBody/Examples/Elementary/SpringDamperSystem_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.SpringMassSystem\">SpringMassSystem</a></td>
      <td valign=\"top\"> Mass attached via a prismatic joint and a spring to the world frame <br>
      <img src=\"../Images/MultiBody/Examples/Elementary/SpringMassSystem_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.SpringWithMass\">SpringWithMass</a></td>
      <td valign=\"top\"> Point mass hanging on a spring <br>
      <img src=\"../Images/MultiBody/Examples/Elementary/SpringWithMass_small.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.ThreeSprings\">ThreeSprings</a></td>
      <td valign=\"top\"> 3-dimensional springs in series and parallel connection<br>
      <img src=\"../Images/MultiBody/Examples/Elementary/ThreeSprings_small.png\">
      </td>
  </tr>
</table>
</HTML>"));
          model ThreeSprings "3-dim. springs in series and parallel connection"
            extends Modelica.Icons.Example;
            parameter Boolean animation = true "= true, if animation shall be enabled";
            annotation(experiment(StopTime = 10), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
This example demonstrates that <b>3-dimensional line force</b> elements
(here: Modelica.Mechanics.MultiBody.Forces.Spring elements) can be connected together
in <b>series</b> without having a body with mass at the
connection point (as usually required by multi-body programs).
This is advantageous since stiff systems can be avoided, say, due to
a stiff spring and a small mass at the connection point.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Elementary/ThreeSprings.png\"
ALT=\"model Examples.Elementary.ThreeSprings\">
</p>
</html>"));
            inner Modelica.Mechanics.MultiBody.World world(animateWorld = animation) annotation(Placement(transformation(extent = {{ -60,20},{ -40,40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body1(animation = animation, r_CM = {0, -0.2,0}, m = 0.8, I_11 = 0.1, I_22 = 0.1, I_33 = 0.1, sphereDiameter = 0.2, r_0(start = {0.5, -0.3,0}, fixed = true), v_0(fixed = true), angles_fixed = true, w_0_fixed = true) annotation(Placement(transformation(origin = {30, -70}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation bar1(animation = animation, r = {0.3,0,0}) annotation(Placement(transformation(extent = {{ -20,20},{0,40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Forces.Spring spring1(lineForce(r_rel_0(start = { -0.2, -0.2,0.2})), s_unstretched = 0.1, width = 0.1, coilWidth = 0.005, numberOfWindings = 5, c = 20, animation = animation) annotation(Placement(transformation(origin = {30,10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation bar2(animation = animation, r = {0,0,0.3}) annotation(Placement(transformation(origin = { -30, -10}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Forces.Spring spring2(s_unstretched = 0.1, width = 0.1, coilWidth = 0.005, numberOfWindings = 5, c = 40, animation = animation) annotation(Placement(transformation(origin = {30, -38}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Forces.Spring spring3(s_unstretched = 0.1, width = 0.1, coilWidth = 0.005, numberOfWindings = 5, c = 20, animation = animation) annotation(Placement(transformation(extent = {{ -20, -42},{0, -22}}, rotation = 0)));

          equation 
            connect(world.frame_b,bar1.frame_a) annotation(Line(points = {{ -40,30},{ -20,30}}, color = {95,95,95}, thickness = 0.5));
            connect(world.frame_b,bar2.frame_a) annotation(Line(points = {{ -40,30},{ -30,30},{ -30,0}}, color = {95,95,95}, thickness = 0.5));
            connect(bar1.frame_b,spring1.frame_a) annotation(Line(points = {{0,30},{30,30},{30,20}}, color = {95,95,95}, thickness = 0.5));
            connect(bar2.frame_b,spring3.frame_a) annotation(Line(points = {{ -30, -20},{ -30, -32},{ -20, -32}}, color = {95,95,95}, thickness = 0.5));
            connect(spring2.frame_b,body1.frame_a) annotation(Line(points = {{30, -48},{30, -51},{30, -60}}, color = {0,0,0}, thickness = 0.5));
            connect(spring3.frame_b,spring1.frame_b) annotation(Line(points = {{0, -32},{30, -14},{30,0}}, color = {95,95,95}, thickness = 0.5));
            connect(spring2.frame_a,spring1.frame_b) annotation(Line(points = {{30, -28},{30, -21},{30, -21},{30, -14},{30,0},{30,0}}, color = {95,95,95}, thickness = 0.5));
          end ThreeSprings;
          model SpringWithMass "Point mass hanging on a spring"
            extends Modelica.Icons.Example;
            annotation(experiment(StopTime = 5), Documentation(info = "<html>
<p>
This example shows that a force component may have a mass.
The 3-dimensional spring as used in this example, has an optional
point mass between the two points where the spring is attached.
In the animation, this point mass is represented by a small,
light blue, sphere.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Elementary/SpringWithMass.png\"
ALT=\"model Examples.Elementary.SpringWithMass\">
</p>
</html>"));
            inner Modelica.Mechanics.MultiBody.World world(animateGravity = false) annotation(Placement(transformation(extent = {{ -40,40},{ -20,60}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Forces.Spring spring(s_unstretched = 0.2, m = 0.5, c = 40, width = 0.1, massDiameter = 0.07) annotation(Placement(transformation(origin = {10,30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Parts.Body body(r_0(start = {0, -0.3,0}, fixed = true), v_0(fixed = true), angles_fixed = true, w_0_fixed = true, r_CM = {0,0,0}, m = 1) annotation(Placement(transformation(origin = {10, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));

          equation 
            connect(world.frame_b,spring.frame_a) annotation(Line(points = {{ -20,50},{10,50},{10,40}}, color = {95,95,95}, thickness = 0.5));
            connect(body.frame_a,spring.frame_b) annotation(Line(points = {{10,0},{10,10},{10,20}}, color = {95,95,95}, thickness = 0.5));
          end SpringWithMass;
          model SpringMassSystem "Mass attached with a spring to the world frame"
            extends Modelica.Icons.Example;
            parameter Boolean animation = true "= true, if animation shall be enabled";
            annotation(experiment(StopTime = 5), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
This example shows the two different ways how force laws
can be utilized:
</p>
<ul>
<li>In the left system a body is attached via a prismatic
    joint to the world frame. The prismatic joint has two
    1-dimensional translational flanges (called \"support\" and \"axis\")
    that allows to connect elements from the Modelica.Mechanics.Translational
    library between the support and the axis connector. The effect is
    that the force generated by the 1-dimensional elements acts as driving
    force in the axis of the prismatic joint. In the example a simple
    spring is used.<br>
    The advantage of this approach is that the many elements from the
    Translational library can be easily used here and that this implementation
    is usually more efficient as when using 3-dimensional springs.</li>
<li>In the right system the same model is defined. The difference is
    that a 3-dimensional spring from the Modelica.Mechanics.MultiBody.Forces library is used.
    This has the advantage to get a nice animation of the force component.</li>
</ul>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Elementary/SpringMassSystem.png\"
ALT=\"model Examples.Elementary.SpringMassSystem\">
</p>
</html>"));
            inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -80,20},{ -60,40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Prismatic p1(useAxisFlange = true, n = {0, -1,0}, animation = animation, boxWidth = 0.05, s(fixed = true, start = 0.1), v(fixed = true)) annotation(Placement(transformation(origin = { -20, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.Mechanics.Translational.Components.Spring spring1(c = 30, s_rel0 = 0.1) annotation(Placement(transformation(origin = {10, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Parts.Body body1(m = 1, sphereDiameter = 0.2, animation = animation, r_CM = {0,0,0}) annotation(Placement(transformation(origin = { -20, -50}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation bar1(animation = animation, r = {0.3,0,0}) annotation(Placement(transformation(extent = {{ -46,20},{ -26,40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation bar2(animation = animation, r = {0.3,0,0}) annotation(Placement(transformation(extent = {{0,20},{20,40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body2(m = 1, sphereDiameter = 0.2, animation = animation, r_CM = {0,0,0}) annotation(Placement(transformation(origin = {50, -50}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Joints.Prismatic p2(useAxisFlange = true, n = {0, -1,0}, animation = animation, boxWidth = 0.05, stateSelect = StateSelect.always, s(fixed = true, start = 0.1), v(fixed = true)) annotation(Placement(transformation(origin = {50, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Forces.Spring spring2(c = 30, s_unstretched = 0.1, width = 0.1) annotation(Placement(transformation(origin = {90, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));

          equation 
            connect(body1.frame_a,p1.frame_b) annotation(Line(points = {{ -20, -40},{ -20, -35},{ -20, -35},{ -20, -30},{ -20, -20},{ -20, -20}}, color = {95,95,95}, thickness = 0.5));
            connect(world.frame_b,bar1.frame_a) annotation(Line(points = {{ -60,30},{ -46,30}}, color = {95,95,95}, thickness = 0.5));
            connect(bar1.frame_b,p1.frame_a) annotation(Line(points = {{ -26,30},{ -20,30},{ -20,0}}, color = {95,95,95}, thickness = 0.5));
            connect(spring1.flange_b,p1.axis) annotation(Line(points = {{10, -20},{10, -30},{ -8, -30},{ -8, -18},{ -14, -18}}, color = {0,191,0}));
            connect(bar1.frame_b,bar2.frame_a) annotation(Line(points = {{ -26,30},{0,30}}, color = {95,95,95}, thickness = 0.5));
            connect(bar2.frame_b,p2.frame_a) annotation(Line(points = {{20,30},{50,30},{50,0}}, color = {95,95,95}, thickness = 0.5));
            connect(p2.frame_b,body2.frame_a) annotation(Line(points = {{50, -20},{50, -40}}, color = {95,95,95}, thickness = 0.5));
            connect(bar2.frame_b,spring2.frame_a) annotation(Line(points = {{20,30},{90,30},{90,0}}, color = {95,95,95}, thickness = 0.5));
            connect(body2.frame_a,spring2.frame_b) annotation(Line(points = {{50, -40},{90, -40},{90, -20}}, color = {95,95,95}, thickness = 0.5));
            connect(spring1.flange_a,p1.support) annotation(Line(points = {{10,0},{ -8,0},{ -8, -6},{ -14, -6}}, color = {0,127,0}, smooth = Smooth.None));
          end SpringMassSystem;
          model SpringDamperSystem "Simple spring/damper/mass system"
            extends Modelica.Icons.Example;
            parameter Boolean animation = true "= true, if animation shall be enabled";
            annotation(experiment(StopTime = 10), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
This example demonstrates:
</p>
<ul>
<li>The animation of spring and damper components</li>
<li>A body can be freely moving without any connection to a joint.
    In this case body coordinates are used automatically as
    states (whenever joints are present, it is first tried to
    use the generalized coordinates of the joints as states).</li>
<li>If a body is freely moving, the initial position and velocity of the body
    can be defined with the \"Initialization\" menu as shown with the
    body \"body1\" in the left part (click on \"Initialization\").</li>
</ul>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Elementary/SpringDamperSystem.png\"
ALT=\"model Examples.Elementary.SpringDamperSystem\">
</p>
</html>"));
            inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -80,20},{ -60,40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body1(m = 1, animation = animation, r_CM = {0, -0.2,0}, cylinderDiameter = 0.05, sphereDiameter = 0.15, I_11 = 0.1, I_22 = 0.1, I_33 = 0.1, r_0(start = {0.3, -0.2,0}, fixed = true), v_0(fixed = true), angles_fixed = true, w_0_fixed = true, w_0_start(displayUnit = "deg/s") = {0,0,0.0349065850398866}) annotation(Placement(transformation(origin = { -20, -50}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation bar1(animation = animation, r = {0.3,0,0}) annotation(Placement(transformation(extent = {{ -46,20},{ -26,40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation bar2(animation = animation, r = {0.6,0,0}) annotation(Placement(transformation(extent = {{0,20},{20,40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body2(m = 1, animation = animation, cylinderDiameter = 0.05, sphereDiameter = 0.15, r_CM = {0,0,0}) annotation(Placement(transformation(origin = {50, -50}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Joints.Prismatic p2(useAxisFlange = true, n = {0, -1,0}, animation = animation, boxWidth = 0.05, stateSelect = StateSelect.always, v(fixed = true), s(fixed = true, start = 0.1)) annotation(Placement(transformation(origin = {50, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Forces.Spring spring2(c = 30, s_unstretched = 0.1, coilWidth = 0.01, width = 0.1) annotation(Placement(transformation(origin = {82, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Forces.Spring spring1(s_unstretched = 0.1, coilWidth = 0.01, c = 30, numberOfWindings = 10, width = 0.1) annotation(Placement(transformation(origin = { -6, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Forces.Damper damper1(d = 2) annotation(Placement(transformation(origin = { -30, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));

          equation 
            connect(world.frame_b,bar1.frame_a) annotation(Line(points = {{ -60,30},{ -46,30}}, color = {95,95,95}, thickness = 0.5));
            connect(bar1.frame_b,bar2.frame_a) annotation(Line(points = {{ -26,30},{0,30}}, color = {95,95,95}, thickness = 0.5));
            connect(bar2.frame_b,p2.frame_a) annotation(Line(points = {{20,30},{50,30},{50,0}}, color = {95,95,95}, thickness = 0.5));
            connect(p2.frame_b,body2.frame_a) annotation(Line(points = {{50, -20},{50, -40}}, color = {95,95,95}, thickness = 0.5));
            connect(bar2.frame_b,spring2.frame_a) annotation(Line(points = {{20,30},{82,30},{82,0}}, color = {95,95,95}, thickness = 0.5));
            connect(body2.frame_a,spring2.frame_b) annotation(Line(points = {{50, -40},{82, -40},{82, -20}}, color = {95,95,95}, thickness = 0.5));
            connect(damper1.frame_a,bar1.frame_b) annotation(Line(points = {{ -30,0},{ -30,10},{ -20,10},{ -20,30},{ -26,30}}, color = {95,95,95}, thickness = 0.5));
            connect(spring1.frame_a,bar1.frame_b) annotation(Line(points = {{ -6,0},{ -6,10},{ -20,10},{ -20,30},{ -26,30}}, color = {95,95,95}, thickness = 0.5));
            connect(damper1.frame_b,body1.frame_a) annotation(Line(points = {{ -30, -20},{ -30, -28},{ -20, -28},{ -20, -40}}, color = {95,95,95}, thickness = 0.5));
            connect(spring1.frame_b,body1.frame_a) annotation(Line(points = {{ -6, -20},{ -6, -28},{ -20, -28},{ -20, -40}}, color = {95,95,95}, thickness = 0.5));
          end SpringDamperSystem;
          model RollingWheelSetPulling "Rolling wheel set that is pulled by a force"
            extends Modelica.Icons.Example;
            Modelica.Mechanics.MultiBody.Forces.WorldForce force(animation = false) annotation(Placement(transformation(extent = {{ -20,60},{0,80}})));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 3), Documentation(info = "<html>

</html>"));
            Modelica.Mechanics.MultiBody.Visualizers.Ground ground(length = 3) annotation(Placement(transformation(extent = {{ -20, -40},{0, -20}})));
            inner Modelica.Mechanics.MultiBody.World world(label2 = "z", n = {0,0, -1}) annotation(Placement(transformation(extent = {{ -60,0},{ -40,20}})));
            Modelica.Mechanics.MultiBody.Parts.RollingWheelSet wheelSet(wheelRadius = 0.1, wheelMass = 0.5, wheel_I_axis = 0.01, wheel_I_long = 0.02, wheelDistance = 0.5, x(start = 0.1, fixed = true), y(start = 0.1, fixed = true), phi(fixed = true), theta1(fixed = true), theta2(fixed = true), der_theta1(fixed = true), der_theta2(fixed = true)) annotation(Placement(transformation(extent = {{ -20,0},{0,20}})));
            Modelica.Mechanics.MultiBody.Parts.Body body(m = 0.01, r_CM = {0,0,0}, animation = false) annotation(Placement(transformation(extent = {{42,20},{62,40}})));
            Modelica.Blocks.Sources.CombiTimeTable combiTimeTable(table = [0,1,0,0;1,1,0,0;2,0,2,0;3,0,2,0]) annotation(Placement(transformation(extent = {{ -80,70},{ -60,90}})));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation(r = {0.2,0,0}, animation = true, width = 0.04) annotation(Placement(transformation(extent = {{0,20},{20,40}})));
            Modelica.Mechanics.MultiBody.Visualizers.FixedShape shape(final lengthDirection = {0,1,0}, final widthDirection = {1,0,0}, final shapeType = "pipe", final r_shape = {0, -wheelSet.wheelWidth,0}, final length = 2 * wheelSet.wheelWidth, final width = 2 * wheelSet.wheelRadius, final height = 2 * wheelSet.wheelRadius, final color = {0,128,255}, final extra = 0.8) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0, origin = {52,54})));

          equation 
            connect(combiTimeTable.y,force.force) annotation(Line(points = {{ -59,80},{ -42,80},{ -42,70},{ -22,70}}, color = {0,0,127}, smooth = Smooth.None));
            connect(fixedTranslation.frame_a,wheelSet.frameMiddle) annotation(Line(points = {{0,30},{ -6,30},{ -6,14},{ -10,14},{ -10,10}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedTranslation.frame_b,body.frame_a) annotation(Line(points = {{20,30},{42,30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(force.frame_b,fixedTranslation.frame_b) annotation(Line(points = {{0,70},{34,70},{34,30},{20,30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(shape.frame_a,fixedTranslation.frame_b) annotation(Line(points = {{42,54},{34,54},{34,30},{20,30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          end RollingWheelSetPulling;
          model RollingWheelSetDriving "Rolling wheel set that is driven by torques driving the wheels"
            extends Modelica.Icons.Example;
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 3), Documentation(info = "<html>

</html>"));
            Modelica.Mechanics.MultiBody.Visualizers.Ground ground(length = 3, groundColor = {0,255,0}) annotation(Placement(transformation(extent = {{ -20, -60},{0, -40}})));
            inner Modelica.Mechanics.MultiBody.World world(label2 = "z", n = {0,0, -1}) annotation(Placement(transformation(extent = {{ -80, -60},{ -60, -40}})));
            Modelica.Mechanics.MultiBody.Parts.RollingWheelSet wheelSet(wheelRadius = 0.1, wheelMass = 0.5, wheel_I_axis = 0.01, wheel_I_long = 0.02, wheelDistance = 0.5, x(start = 0.1, fixed = true), y(start = 0.1, fixed = true), phi(fixed = true), theta1(fixed = true), theta2(fixed = true), der_theta1(fixed = true), der_theta2(fixed = true)) annotation(Placement(transformation(extent = {{ -20, -20},{0,0}})));
            Modelica.Mechanics.MultiBody.Parts.Body body(m = 0.01, r_CM = {0,0,0}, animation = false) annotation(Placement(transformation(extent = {{40,56},{60,76}})));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation(r = {0.2,0,0}, animation = true, width = 0.04) annotation(Placement(transformation(extent = {{0,56},{20,76}})));
            Modelica.Blocks.Sources.Sine sine1(freqHz = 1, amplitude = 2) annotation(Placement(transformation(extent = {{ -80,20},{ -60,40}})));
            Modelica.Blocks.Sources.Sine sine2(freqHz = 1, amplitude = 2, phase = 1.5707963267949) annotation(Placement(transformation(extent = {{60,20},{40,40}})));
            Modelica.Mechanics.Rotational.Sources.Torque2 torque1 annotation(Placement(transformation(extent = {{ -40,4},{ -20,24}})));
            Modelica.Mechanics.Rotational.Sources.Torque2 torque2 annotation(Placement(transformation(extent = {{24,4},{4,24}})));
            Modelica.Mechanics.MultiBody.Visualizers.FixedShape shape(final lengthDirection = {0,1,0}, final widthDirection = {1,0,0}, final shapeType = "pipe", final r_shape = {0, -wheelSet.wheelWidth,0}, final length = 2 * wheelSet.wheelWidth, final width = 2 * wheelSet.wheelRadius, final height = 2 * wheelSet.wheelRadius, final color = {0,128,255}, final extra = 0.8) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0, origin = {50,90})));

          equation 
            connect(fixedTranslation.frame_a,wheelSet.frameMiddle) annotation(Line(points = {{0,66},{ -8,66},{ -8, -6},{ -10, -6},{ -10, -10}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedTranslation.frame_b,body.frame_a) annotation(Line(points = {{20,66},{40,66}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(wheelSet.axis1,torque1.flange_a) annotation(Line(points = {{ -20,0},{ -40,0},{ -40,14}}, color = {0,0,0}, smooth = Smooth.None));
            connect(torque1.flange_b,wheelSet.support) annotation(Line(points = {{ -20,14},{ -10,14},{ -10, -1.8}}, color = {0,0,0}, smooth = Smooth.None));
            connect(wheelSet.axis2,torque2.flange_a) annotation(Line(points = {{0,0},{24,0},{24,14}}, color = {0,0,0}, smooth = Smooth.None));
            connect(wheelSet.support,torque2.flange_b) annotation(Line(points = {{ -10, -1.8},{ -10,14},{4,14}}, color = {0,0,0}, smooth = Smooth.None));
            connect(sine1.y,torque1.tau) annotation(Line(points = {{ -59,30},{ -30,30},{ -30,18}}, color = {0,0,127}, smooth = Smooth.None));
            connect(sine2.y,torque2.tau) annotation(Line(points = {{39,30},{14,30},{14,18}}, color = {0,0,127}, smooth = Smooth.None));
            connect(shape.frame_a,fixedTranslation.frame_b) annotation(Line(points = {{40,90},{28,90},{28,66},{20,66}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          end RollingWheelSetDriving;
          model RollingWheel "Single wheel rolling on ground starting from an initial speed"
            extends Modelica.Icons.Example;
            Modelica.Mechanics.MultiBody.Parts.RollingWheel wheel1(wheelRadius = 0.3, wheelMass = 2, wheel_I_axis = 0.06, wheel_I_long = 0.12, hollowFraction = 0.6, x(start = 0.2), y(start = 0.2), der_angles(start = {0,5,1})) annotation(Placement(transformation(extent = {{ -20,0},{0,20}})));
            inner Modelica.Mechanics.MultiBody.World world(label2 = "z", n = {0,0, -1}) annotation(Placement(transformation(extent = {{ -60,0},{ -40,20}})));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 4), Documentation(info = "<html>

</html>"));
            Modelica.Mechanics.MultiBody.Visualizers.Ground ground(length = 4) annotation(Placement(transformation(extent = {{ -20, -40},{0, -20}})));
          end RollingWheel;
          model PointGravityWithPointMasses2 "Rigidly connected point masses in a point gravity field"
            extends Modelica.Icons.Example;
            model PointMass = Modelica.Mechanics.MultiBody.Parts.PointMass(m = 1, sphereColor = {255,0,0}) "Point mass used at all places of this example";
            PointMass pointMass1(r_0(start = {3,0,0}, fixed = true), v_0(start = {0,0, -1}, fixed = true)) annotation(Placement(transformation(extent = {{60, -10},{80,10}}, rotation = 0)));
            PointMass pointMass2 annotation(Placement(transformation(extent = {{ -60, -10},{ -40,10}}, rotation = 0)));
            PointMass pointMass3(r_0(start = {2,1,0}, fixed = true), v_0(start = {0,0, -1}, fixed = true)) annotation(Placement(transformation(extent = {{60,20},{80,40}}, rotation = 0)));
            PointMass pointMass4 annotation(Placement(transformation(extent = {{ -50, -40},{ -30, -20}}, rotation = 0)));
            PointMass pointMass5 annotation(Placement(transformation(extent = {{0,80},{20,100}}, rotation = 0)));
            PointMass pointMass6 annotation(Placement(transformation(extent = {{0, -100},{20, -80}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation(r = {1,0,0}) annotation(Placement(transformation(extent = {{20, -10},{40,10}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation1(r = { -1,0,0}) annotation(Placement(transformation(extent = {{0, -10},{ -20,10}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation2(r = {0,1,0}) annotation(Placement(transformation(extent = {{20,20},{40,40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation3(r = {0, -1,0}) annotation(Placement(transformation(extent = {{0, -40},{ -20, -20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation4(r = {0,0,1}) annotation(Placement(transformation(origin = {10,60}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation5(r = {0,0, -1}) annotation(Placement(transformation(origin = {10, -60}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 3, Tolerance = 1e-06), Documentation(info = "<html>
<p>
This model demonstrates the usage of model Parts.PointMass in a
point gravity field. 6 point masses are connected rigidly together.
Translating such a model results in an error, because point masses do
not define an orientation object. The example demonstrates that in such
a case (when the orientation object is not defined by an object that
is connected to a point mass), a \"MultiBody.Joints.FreeMotion\" joint
has to be used, to define the the degrees of freedom of this structure.
</p>

<p>
In order to demonstrate that this approach is correct, in model
\"referenceSystem\", the same system is again provided, but this time
modeled with a generic body (Parts.Body) where the inertia tensor is
set to zero. In this case, no FreeMotion object is needed because every
body provides its absolute translational and rotational position and
velocity as potential states.
</p>

<p>
The two systems should move exactly in the same way. The system with the PointMasses
object visulizes the point masses in \"red\", whereas the \"referenceSystem\" shows
its bodies in \"blue\".
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Elementary/PointGravityWithPointMasses2.png\">
</p>
</html>"));
            inner World world(gravityType = Modelica.Mechanics.MultiBody.Types.GravityTypes.PointGravity, mue = 5) annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}}, rotation = 0)));
            Joints.FreeMotion freeMotion annotation(Placement(transformation(extent = {{ -40,60},{ -20,80}}, rotation = 0)));
            model SystemWithStandardBodies "For comparison purposes, an equivalent model with Bodies instead of PointMasses"
              extends Modelica.Icons.Example;
              model PointMass = Modelica.Mechanics.MultiBody.Parts.Body(m = 1, I_11 = 0, I_22 = 0, I_33 = 0) "Body used all places of the comparision model with zero inertia tensor";
              PointMass pointMass1(r_0(start = {3,0,0}, fixed = true), v_0(start = {0,0, -1}, fixed = true), angles_fixed = true, w_0_fixed = true, r_CM = {0,0,0}) annotation(Placement(transformation(extent = {{40, -20},{60,0}}, rotation = 0)));
              PointMass pointMass2(r_CM = {0,0,0}) annotation(Placement(transformation(extent = {{ -60, -20},{ -80,0}}, rotation = 0)));
              PointMass pointMass3(r_CM = {0,0,0}) annotation(Placement(transformation(extent = {{40,10},{60,30}}, rotation = 0)));
              PointMass pointMass4(r_CM = {0,0,0}) annotation(Placement(transformation(extent = {{ -50, -50},{ -70, -30}}, rotation = 0)));
              PointMass pointMass5(r_CM = {0,0,0}) annotation(Placement(transformation(extent = {{0,60},{20,80}}, rotation = 0)));
              PointMass pointMass6(r_CM = {0,0,0}) annotation(Placement(transformation(extent = {{2, -102},{22, -82}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation(r = {1,0,0}) annotation(Placement(transformation(extent = {{0, -20},{20,0}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation1(r = { -1,0,0}) annotation(Placement(transformation(extent = {{ -20, -20},{ -40,0}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation2(r = {0,1,0}) annotation(Placement(transformation(extent = {{0,10},{20,30}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation3(r = {0, -1,0}) annotation(Placement(transformation(extent = {{ -20, -50},{ -40, -30}}, rotation = 0)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation4(r = {0,0,1}) annotation(Placement(transformation(origin = { -10,50}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
              Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation5(r = {0,0, -1}) annotation(Placement(transformation(origin = { -10, -70}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
              annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
In order to compare the results of the \"PointMass\" example where
6 point masses are rigidly connected together, in this comparision model,
an equivalent system is setup, with the only difference that the
point masses are replaced by Bodies with zero inertia.
</p>
</html>"));

            equation 
              connect(fixedTranslation1.frame_a,fixedTranslation.frame_a) annotation(Line(points = {{ -20, -10},{0, -10}}, color = {95,95,95}, thickness = 0.5));
              connect(fixedTranslation1.frame_a,fixedTranslation2.frame_a) annotation(Line(points = {{ -20, -10},{ -10, -10},{ -10,20},{0,20}}, color = {95,95,95}, thickness = 0.5));
              connect(fixedTranslation3.frame_a,fixedTranslation.frame_a) annotation(Line(points = {{ -20, -40},{ -10, -40},{ -10, -10},{0, -10}}, color = {95,95,95}, thickness = 0.5));
              connect(fixedTranslation1.frame_a,fixedTranslation4.frame_a) annotation(Line(points = {{ -20, -10},{ -10, -10},{ -10,40}}, color = {95,95,95}, thickness = 0.5));
              connect(fixedTranslation5.frame_a,fixedTranslation.frame_a) annotation(Line(points = {{ -10, -60},{ -10, -10},{0, -10}}, color = {95,95,95}, thickness = 0.5));
              connect(fixedTranslation2.frame_b,pointMass3.frame_a) annotation(Line(points = {{20,20},{40,20}}, color = {95,95,95}, thickness = 0.5));
              connect(fixedTranslation3.frame_b,pointMass4.frame_a) annotation(Line(points = {{ -40, -40},{ -50, -40}}, color = {95,95,95}, thickness = 0.5));
              connect(pointMass5.frame_a,fixedTranslation4.frame_b) annotation(Line(points = {{0,70},{ -10,70},{ -10,60}}, color = {95,95,95}, thickness = 0.5));
              connect(fixedTranslation5.frame_b,pointMass6.frame_a) annotation(Line(points = {{ -10, -80},{ -10, -92},{2, -92}}, color = {95,95,95}, thickness = 0.5));
              connect(fixedTranslation.frame_b,pointMass1.frame_a) annotation(Line(points = {{20, -10},{40, -10}}, color = {95,95,95}, thickness = 0.5));
              connect(fixedTranslation1.frame_b,pointMass2.frame_a) annotation(Line(points = {{ -40, -10},{ -60, -10}}, color = {95,95,95}, thickness = 0.5));
            end SystemWithStandardBodies;
            SystemWithStandardBodies referenceSystem annotation(Placement(transformation(extent = {{60, -60},{80, -40}}, rotation = 0)));

          equation 
            connect(fixedTranslation1.frame_a,fixedTranslation.frame_a) annotation(Line(points = {{0,0},{20,0}}, color = {95,95,95}, thickness = 0.5));
            connect(fixedTranslation1.frame_a,fixedTranslation2.frame_a) annotation(Line(points = {{0,0},{10,0},{10,30},{20,30}}, color = {95,95,95}, thickness = 0.5));
            connect(fixedTranslation3.frame_a,fixedTranslation.frame_a) annotation(Line(points = {{0, -30},{10, -30},{10,0},{20,0}}, color = {95,95,95}, thickness = 0.5));
            connect(fixedTranslation1.frame_a,fixedTranslation4.frame_a) annotation(Line(points = {{0,0},{10,0},{10,50}}, color = {95,95,95}, thickness = 0.5));
            connect(fixedTranslation5.frame_a,fixedTranslation.frame_a) annotation(Line(points = {{10, -50},{10,0},{20,0}}, color = {95,95,95}, thickness = 0.5));
            connect(fixedTranslation2.frame_b,pointMass3.frame_a) annotation(Line(points = {{40,30},{70,30}}, color = {95,95,95}, thickness = 0.5));
            connect(fixedTranslation3.frame_b,pointMass4.frame_a) annotation(Line(points = {{ -20, -30},{ -40, -30}}, color = {95,95,95}, thickness = 0.5));
            connect(pointMass5.frame_a,fixedTranslation4.frame_b) annotation(Line(points = {{10,90},{10,70}}, color = {95,95,95}, thickness = 0.5));
            connect(fixedTranslation5.frame_b,pointMass6.frame_a) annotation(Line(points = {{10, -70},{10, -90}}, color = {95,95,95}, thickness = 0.5));
            connect(fixedTranslation.frame_b,pointMass1.frame_a) annotation(Line(points = {{40,0},{70,0}}, color = {95,95,95}, thickness = 0.5));
            connect(fixedTranslation1.frame_b,pointMass2.frame_a) annotation(Line(points = {{ -20,0},{ -50,0}}, color = {95,95,95}, thickness = 0.5));
            connect(world.frame_b,freeMotion.frame_a) annotation(Line(points = {{ -60,70},{ -40,70}}, color = {95,95,95}, thickness = 0.5));
            connect(freeMotion.frame_b,fixedTranslation1.frame_a) annotation(Line(points = {{ -20,70},{ -10,70},{ -10,20},{10,20},{10,0},{0,0}}, color = {95,95,95}, thickness = 0.5));
          end PointGravityWithPointMasses2;
          model PointGravityWithPointMasses "Two point masses in a point gravity field (rotation of bodies is neglected)"
            import SI = Modelica.SIunits;
            extends Modelica.Icons.Example;
            inner Modelica.Mechanics.MultiBody.World world(mue = 1, gravitySphereDiameter = 0.1, gravityType = Modelica.Mechanics.MultiBody.Types.GravityTypes.PointGravity) annotation(Placement(transformation(extent = {{ -20, -20},{0,0}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.PointMass body1(m = 1, sphereDiameter = 0.1, r_0(start = {0,0.6,0}, fixed = true), v_0(start = {1,0,0}, fixed = true)) annotation(Placement(transformation(extent = {{ -20,20},{0,40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.PointMass body2(m = 1, sphereDiameter = 0.1, r_0(start = {0.6,0.6,0}, fixed = true), v_0(start = {0.6,0,0}, fixed = true)) annotation(Placement(transformation(extent = {{20,20},{40,40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.PointMass body3(m = 1, sphereDiameter = 0.1, r_0(start = {0,0.8,0}, fixed = true), v_0(start = {0.6,0,0}, fixed = true)) annotation(Placement(transformation(extent = {{ -20,60},{0,80}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.PointMass body4(m = 1, sphereDiameter = 0.1, r_0(start = {0.3,0.8,0}, fixed = true), v_0(start = {0.6,0,0}, fixed = true)) annotation(Placement(transformation(extent = {{20,60},{40,80}}, rotation = 0)));
            Forces.Spring spring(showMass = false, c = 10) annotation(Placement(transformation(extent = {{0,60},{20,80}}, rotation = 0)));

          equation 
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 2), Documentation(info = "<HTML>
<p>
This model demonstrates the usage of model Parts.PointMass in a
point gravity field. The PointMass model has the feature that
that rotation is not taken into account and can therefore also not be
calculated. This example demonstrates two cases where this does not matter:
If a PointMass is not connected (body1, body2), the orientation object in
these point masses is set to a unit rotation.
If a PointMass is connected by a line force element, such as
the used Forces.LineForceWithMass component, then the orientation object
is set to a unit rotation within the line force element.
These are the two cases where the rotation is automatically set to
a default value, when the physical system does not provide the equations.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Elementary/PointGravityWithPointMasses.png\">
</p>
</HTML>"));
            connect(spring.frame_a,body3.frame_a) annotation(Line(points = {{0,70},{ -10,70}}, color = {95,95,95}, thickness = 0.5));
            connect(spring.frame_b,body4.frame_a) annotation(Line(points = {{20,70},{30,70}}, color = {95,95,95}, thickness = 0.5));
          end PointGravityWithPointMasses;
          model PointGravity "Two point masses in a point gravity field"
            import SI = Modelica.SIunits;
            extends Modelica.Icons.Example;
            inner Modelica.Mechanics.MultiBody.World world(mue = 1, gravitySphereDiameter = 0.1, gravityType = Modelica.Mechanics.MultiBody.Types.GravityTypes.PointGravity) annotation(Placement(transformation(extent = {{ -20, -20},{0,0}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body1(m = 1, sphereDiameter = 0.1, I_11 = 0.1, I_22 = 0.1, I_33 = 0.1, r_0(start = {0,0.6,0}, fixed = true), v_0(start = {1,0,0}, fixed = true), angles_fixed = true, w_0_fixed = true, r_CM = {0,0,0}) annotation(Placement(transformation(extent = {{ -20,20},{0,40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body2(m = 1, sphereDiameter = 0.1, I_11 = 0.1, I_22 = 0.1, I_33 = 0.1, r_0(start = {0.6,0.6,0}, fixed = true), v_0(start = {0.6,0,0}, fixed = true), angles_fixed = true, w_0_fixed = true, r_CM = {0,0,0}) annotation(Placement(transformation(extent = {{20,20},{40,40}}, rotation = 0)));

          equation 
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 5), Documentation(info = "<HTML>
<p>
This model demonstrates a point gravity field. Two bodies
are placed in the gravity field. The initial positions and velocities of
these bodies are selected such that one body rotates on a circle and
the other body rotates on an ellipse around the center of the
point gravity field.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Elementary/PointGravity.png\"
ALT=\"model Examples.Elementary.PointGravity\">
</p>
</HTML>"));
          end PointGravity;
          model PendulumWithSpringDamper "Simple spring/damper/mass system"
            extends Modelica.Icons.Example;
            parameter Boolean animation = true "= true, if animation shall be enabled";
            annotation(experiment(StopTime = 10), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
A body is attached on a revolute and prismatic joint.
A 3-dim. spring and a 3-dim. damper are connected between the body
and a point fixed in the world frame:
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Elementary/PendulumWithSpringDamper.png\"
ALT=\"model Examples.Elementary.PendulumWithSpringDamper\">
</p>
</html>"));
            inner Modelica.Mechanics.MultiBody.World world(axisLength = 0.6) annotation(Placement(transformation(extent = {{ -80,20},{ -60,40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body1(m = 1, animation = animation, I_11 = 1, I_22 = 1, I_33 = 1, r_CM = {0,0,0}, cylinderDiameter = 0.05, sphereDiameter = 0.2) annotation(Placement(transformation(origin = {72,30}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation bar1(animation = animation, r = {0.3,0,0}) annotation(Placement(transformation(extent = {{ -46,20},{ -26,40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Forces.Spring spring1(coilWidth = 0.01, numberOfWindings = 5, c = 20, s_unstretched = 0.2) annotation(Placement(transformation(extent = {{0, -46},{20, -26}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Forces.Damper damper1(d = 1, length_a = 0.1, diameter_a = 0.08, animation = false) annotation(Placement(transformation(extent = {{0, -20},{20,0}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute revolute(phi(fixed = true), w(fixed = true)) annotation(Placement(transformation(extent = {{ -12,20},{8,40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Prismatic prismatic(boxWidth = 0.04, boxColor = {255,65,65}, s(fixed = true, start = 0.5), v(fixed = true)) annotation(Placement(transformation(extent = {{20,20},{40,40}}, rotation = 0)));

          equation 
            connect(world.frame_b,bar1.frame_a) annotation(Line(points = {{ -60,30},{ -46,30}}, color = {95,95,95}, thickness = 0.5));
            connect(revolute.frame_a,bar1.frame_b) annotation(Line(points = {{ -12,30},{ -26,30}}, color = {95,95,95}, thickness = 0.5));
            connect(prismatic.frame_a,revolute.frame_b) annotation(Line(points = {{20,30},{8,30}}, color = {95,95,95}, thickness = 0.5));
            connect(damper1.frame_a,bar1.frame_b) annotation(Line(points = {{0, -10},{ -18, -10},{ -18,30},{ -26,30}}, color = {95,95,95}, thickness = 0.5));
            connect(damper1.frame_b,prismatic.frame_b) annotation(Line(points = {{20, -10},{44, -10},{44,30},{40,30}}, color = {95,95,95}, thickness = 0.5));
            connect(spring1.frame_a,bar1.frame_b) annotation(Line(points = {{0, -36},{ -18, -36},{ -18,30},{ -26,30}}, color = {95,95,95}, thickness = 0.5));
            connect(spring1.frame_b,prismatic.frame_b) annotation(Line(points = {{20, -36},{44, -36},{44,30},{40,30}}, color = {95,95,95}, thickness = 0.5));
            connect(body1.frame_a,prismatic.frame_b) annotation(Line(points = {{62,30},{40,30}}, color = {95,95,95}, thickness = 0.5));
          end PendulumWithSpringDamper;
          model Pendulum "Simple pendulum with one revolute joint and one body"
            extends Modelica.Icons.Example;
            annotation(experiment(StopTime = 5), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
This simple model demonstrates that by just dragging components
default animation is defined that shows the structure of the
assembled system.
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Elementary/Pendulum.png\"
ALT=\"model Examples.Elementary.Pendulum\">
</p>
</html>"));
            inner Modelica.Mechanics.MultiBody.World world(gravityType = Modelica.Mechanics.MultiBody.Types.GravityTypes.UniformGravity) annotation(Placement(transformation(extent = {{ -60,0},{ -40,20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute rev(n = {0,0,1}, useAxisFlange = true, phi(fixed = true), w(fixed = true)) annotation(Placement(transformation(extent = {{ -20,0},{0,20}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Damper damper(d = 0.1) annotation(Placement(transformation(extent = {{ -20,40},{0,60}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body(m = 1.0, r_CM = {0.5,0,0}) annotation(Placement(transformation(extent = {{20,0},{40,20}}, rotation = 0)));

          equation 
            connect(world.frame_b,rev.frame_a) annotation(Line(points = {{ -40,10},{ -20,10}}, color = {95,95,95}, thickness = 0.5));
            connect(damper.flange_b,rev.axis) annotation(Line(points = {{0,50},{4,50},{4,26},{ -10,26},{ -10,20}}, color = {0,0,0}));
            connect(rev.support,damper.flange_a) annotation(Line(points = {{ -16,20},{ -16,26},{ -28,26},{ -28,50},{ -20,50}}, color = {0,0,0}));
            connect(body.frame_a,rev.frame_b) annotation(Line(points = {{20,10},{0,10}}, color = {95,95,95}, thickness = 0.5));
          end Pendulum;
          model LineForceWithTwoMasses "Demonstrate line force with two point masses using a JointUPS and alternatively a LineForceWithTwoMasses component"
            import SI = Modelica.SIunits;
            extends Modelica.Icons.Example;
            parameter Modelica.SIunits.Mass m = 1 "Mass of point masses";
            SI.Force rod_f_diff[3] = rod1.frame_b.f - rod3.frame_b.f "Difference of cut-forces in rod1 and rod3";
            SI.Force body_f_diff[3] = bodyBox1.frame_b.f - bodyBox2.frame_b.f "Difference of cut-forces in bodyBox1 and bodyBox2";
            annotation(experiment(StopTime = 3), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Documentation(info = "<html>
<p>
It is demonstrated how to implement line force components
that shall have mass properties. Two alternative implementations
are given:
</p>
<ul>
<li> With
     <a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUPS\">JointUPS</a>:<br>
     Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUPS is an aggregation
     of a universal, a prismatic and a spherical joint that approximates
     a real force component, such as a hydraulic cylinder. At the two
     frames of the prismatic joint (frame_ia, frame_ib of jointUPS)
     two bodies are attached. The parameters are selected such that
     the center of masses of the two bodies are located on the line
     connecting frame_a and frame_b of the jointUPS component.
     Both bodies have the same mass and the inertia tensor is set to zero,
     i.e., the two bodies are treated as point masses.</li>
<li> With <a href=\"Modelica://Modelica.Mechanics.MultiBody.Forces.LineForceWithTwoMasses\">
     LineForceWithTwoMasses</a>:<br>
     Modelica.Mechanics.MultiBody.Forces.LineForceWithTwoMasses is a line force component
     with the built-in property that two point masses are located
     on the line on which the line force is acting.
     The parameters are selected in such a way that the same
     system as with the jointUPS component is described.</li>
</ul>
<p>
In both cases, a linear 1-dimensional translational damper from the
Modelica.Mechanics.Translational library is used as
line force between the two attachment points. Simulate
this system and plot the differences of the cut forces at both sides
of the line force component (\"rod_f_diff\" and \"body_f_diff\").
Both vectors should be zero
(depending on the choosen relative tolerance of the integration,
the difference is in the order of 1.e-10 ... 1.e-15).
</p>
<p>
Note, that the implementation with the LineForceWithTwoMasses
component is simpler and more convenient.
An animation of this simulation is shown in the figure below.
The system on the left side in the front is the animation with
the LineForceWithTwoMasses component whereas the system on the right
side in the back is the animation with the JointUPS component.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Elementary/LineForceWithTwoMasses2.png\">
</p>
</html>"));
            inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute revolute1(phi(fixed = true), w(fixed = true)) annotation(Placement(transformation(extent = {{ -20,60},{0,80}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.BodyBox bodyBox1(r = {0.7,0,0}) annotation(Placement(transformation(extent = {{20,60},{40,80}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod1(r = {0, -0.9,0}, width = 0.01, animation = false) annotation(Placement(transformation(origin = { -40,54}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUPS jointUPS(nAxis_ia = {0.7,1.2,0}, animation = true) annotation(Placement(transformation(extent = {{0,50},{20,30}}, rotation = 0)));
            annotation(Diagram);
            Modelica.Mechanics.MultiBody.Parts.Body body1(r_CM = 0.2 * jointUPS.eAxis_ia, cylinderDiameter = 0.05, animation = true, m = m, I_11 = 0, I_22 = 0, I_33 = 0) annotation(Placement(transformation(extent = {{ -14,14},{ -34,34}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Body body2(r_CM =  -0.2 * jointUPS.eAxis_ia, cylinderDiameter = 0.05, animation = true, m = m, I_11 = 0, I_22 = 0, I_33 = 0) annotation(Placement(transformation(extent = {{32,14},{52,34}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod2(r = {0,0.3,0}, width = 0.01, animation = false) annotation(Placement(transformation(origin = { -40,84}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
            Modelica.Mechanics.Translational.Components.Damper damper1(d = 3) annotation(Placement(transformation(extent = {{0,24},{20,4}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute revolute2(phi(fixed = true), w(fixed = true)) annotation(Placement(transformation(extent = {{ -20, -40},{0, -20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.BodyBox bodyBox2(r = {0.7,0,0}) annotation(Placement(transformation(extent = {{20, -40},{40, -20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod3(width = 0.01, r = {0, -0.9,0.3}, animation = false) annotation(Placement(transformation(origin = { -40, -46}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod4(width = 0.01, r = {0,0.3,0.3}, animation = false) annotation(Placement(transformation(origin = { -40, -16}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
            Modelica.Mechanics.Translational.Components.Damper damper2(d = 3) annotation(Placement(transformation(extent = {{0, -76},{20, -96}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Forces.LineForceWithTwoMasses lineForceWithTwoMasses(L_a = 0.2, L_b = 0.2, cylinderLength_a = 0.2, cylinderLength_b = 1.2, massDiameterFaction = 2.2, m_a = m, m_b = m) annotation(Placement(transformation(extent = {{0, -50},{20, -70}}, rotation = 0)));

          equation 
            connect(jointUPS.bearing,damper1.flange_a) annotation(Line(points = {{6,30},{6,20},{0,20},{0,14}}, color = {0,191,0}));
            connect(jointUPS.axis,damper1.flange_b) annotation(Line(points = {{14,30},{14,20},{20,20},{20,14}}, color = {0,191,0}));
            connect(jointUPS.frame_ib,body2.frame_a) annotation(Line(points = {{18,30},{18,24},{32,24}}, color = {95,95,95}, thickness = 0.5));
            connect(world.frame_b,rod2.frame_a) annotation(Line(points = {{ -60,70},{ -40,70},{ -40,74}}, color = {95,95,95}, thickness = 0.5));
            connect(world.frame_b,rod1.frame_a) annotation(Line(points = {{ -60,70},{ -40,70},{ -40,64}}, color = {95,95,95}, thickness = 0.5));
            connect(rod2.frame_b,revolute1.frame_a) annotation(Line(points = {{ -40,94},{ -40,98},{ -28,98},{ -28,70},{ -20,70}}, color = {95,95,95}, thickness = 0.5));
            connect(revolute1.frame_b,bodyBox1.frame_a) annotation(Line(points = {{0,70},{20,70}}, color = {95,95,95}, thickness = 0.5));
            connect(bodyBox1.frame_b,jointUPS.frame_b) annotation(Line(points = {{40,70},{46,70},{46,40},{20,40}}, color = {95,95,95}, thickness = 0.5));
            connect(body1.frame_a,jointUPS.frame_ia) annotation(Line(points = {{ -14,24},{2,24},{2,30}}, color = {95,95,95}, thickness = 0.5));
            connect(rod1.frame_b,jointUPS.frame_a) annotation(Line(points = {{ -40,44},{ -40,40},{0,40}}, color = {95,95,95}, thickness = 0.5));
            connect(rod4.frame_b,revolute2.frame_a) annotation(Line(points = {{ -40, -6},{ -40, -2},{ -28, -2},{ -28, -30},{ -20, -30}}, color = {95,95,95}, thickness = 0.5));
            connect(revolute2.frame_b,bodyBox2.frame_a) annotation(Line(points = {{0, -30},{20, -30}}, color = {95,95,95}, thickness = 0.5));
            connect(world.frame_b,rod4.frame_a) annotation(Line(points = {{ -60,70},{ -52,70},{ -52, -32},{ -40, -32},{ -40, -26}}, color = {95,95,95}, thickness = 0.5));
            connect(rod3.frame_a,rod4.frame_a) annotation(Line(points = {{ -40, -36},{ -40, -26}}, color = {0,0,0}, thickness = 0.5));
            connect(lineForceWithTwoMasses.frame_a,rod3.frame_b) annotation(Line(points = {{0, -60},{ -40, -60},{ -40, -56}}, color = {95,95,95}, thickness = 0.5));
            connect(lineForceWithTwoMasses.frame_b,bodyBox2.frame_b) annotation(Line(points = {{20, -60},{54, -60},{54, -30},{40, -30}}, color = {95,95,95}, thickness = 0.5));
            connect(lineForceWithTwoMasses.flange_b,damper2.flange_b) annotation(Line(points = {{16, -71},{20, -71},{20, -86}}, color = {0,191,0}));
            connect(lineForceWithTwoMasses.flange_a,damper2.flange_a) annotation(Line(points = {{4, -71},{0, -71},{0, -86}}, color = {0,191,0}));
          end LineForceWithTwoMasses;
          model InitSpringConstant "Determine spring constant such that system is in steady state at given position"
            extends Modelica.Icons.Example;
            inner Modelica.Mechanics.MultiBody.World world(gravityType = Modelica.Mechanics.MultiBody.Types.GravityTypes.UniformGravity) annotation(Placement(transformation(extent = {{ -80,0},{ -60,20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute rev(useAxisFlange = true, n = {0,0,1}, phi(fixed = true), w(fixed = true), a(fixed = true)) annotation(Placement(transformation(extent = {{ -40,0},{ -20,20}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Damper damper(d = 0.1) annotation(Placement(transformation(extent = {{ -40,40},{ -20,60}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.BodyShape body(r = {1,0,0}, r_CM = {0.5,0,0}, m = 1) annotation(Placement(transformation(extent = {{0,0},{20,20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.Fixed fixed(r = {1,0.2,0}, width = 0.02) annotation(Placement(transformation(origin = {50,70}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Forces.Spring spring(s_unstretched = 0.1, c(fixed = false) = 100) annotation(Placement(transformation(origin = {50,30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            annotation(Documentation(info = "<html>
<p>
This example demonstrates a non-standard type of initialization
by calculating a spring constant such
that a simple pendulum is at a defined position in steady state.
</p>
<p>
The goal is that the pendulum should be in steady state
when the rotation angle of the pendulum is zero. The spring
constant of the spring shall be calculated during initialization
such that this goal is reached.
</p>
<p>
The pendulum has one degree of freedom, i.e., two states.
Therefore, two additional equations have to be provided
for initialization. However, parameter \"c\" of the spring
component is defined with attribute \"fixed = <b>false</b>\", i.e.,
the value of this parameter is computed during initialization.
Therefore, there is one additional equation required during
initialization. The 3 initial equations are the rotational
angle of the revolute joint and its first and second
derivative. The latter ones are zero, in order to initialize
in steady state. By setting the start values of phi, w, a to zero and
their fixed attributes to true, the required
3 initial equations are defined.
</p>
<p>
After translation, this model is initialized in steady-state.
The spring constant is computed as c = 49.05 N/m.
An animation of this simulation is shown in the figure below.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Elementary/InitSpringConstant.png\"
ALT=\"model Examples.Elementary.InitSpringConstant\">
</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

          equation 
            connect(world.frame_b,rev.frame_a) annotation(Line(points = {{ -60,10},{ -40,10}}, color = {95,95,95}, thickness = 0.5));
            connect(damper.flange_b,rev.axis) annotation(Line(points = {{ -20,50},{ -16,50},{ -16,26},{ -30,26},{ -30,20}}, color = {0,0,0}));
            connect(rev.support,damper.flange_a) annotation(Line(points = {{ -36,20},{ -36,26},{ -48,26},{ -48,50},{ -40,50}}, color = {0,0,0}));
            connect(rev.frame_b,body.frame_a) annotation(Line(points = {{ -20,10},{0,10}}, color = {95,95,95}, thickness = 0.5));
            connect(fixed.frame_b,spring.frame_a) annotation(Line(points = {{50,60},{50,40}}, color = {95,95,95}, thickness = 0.5));
            connect(body.frame_b,spring.frame_b) annotation(Line(points = {{20,10},{50,10},{50,20}}, color = {95,95,95}, thickness = 0.5));
          end InitSpringConstant;
          model FreeBody "Free flying body attached by two springs to environment"
            extends Modelica.Icons.Example;
            parameter Boolean animation = true "= true, if animation shall be enabled";
            annotation(experiment(StopTime = 10), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
This example demonstrates:
</p>
<ul>
<li>The animation of spring and damper components</li>
<li>A body can be freely moving without any connection to a joint.
    In this case body coordinates are used automatically as
    states (whenever joints are present, it is first tried to
    use the generalized coordinates of the joints as states).</li>
<li>If a body is freely moving, the initial position and velocity of the body
    can be defined with the \"Initialization\" menu as shown with the
    body \"body1\" in the left part (click on \"Initialization\").</li>
</ul>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Elementary/FreeBody.png\"
ALT=\"model Examples.Elementary.FreeBody\">
</p>
</html>"));
            inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -60,20},{ -40,40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation bar2(r = {0.8,0,0}, animation = false) annotation(Placement(transformation(extent = {{0,20},{20,40}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Forces.Spring spring1(width = 0.1, coilWidth = 0.005, numberOfWindings = 5, c = 20, s_unstretched = 0) annotation(Placement(transformation(origin = { -20,6}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Parts.BodyShape body(m = 1, I_11 = 1, I_22 = 1, I_33 = 1, r = {0.4,0,0}, r_CM = {0.2,0,0}, width = 0.05, r_0(start = {0.2, -0.5,0.1}, fixed = true), v_0(fixed = true), angles_fixed = true, w_0_fixed = true, angles_start = {0.174532925199433,0.174532925199433,0.174532925199433}) annotation(Placement(transformation(extent = {{0, -40},{20, -20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Forces.Spring spring2(c = 20, s_unstretched = 0, width = 0.1, coilWidth = 0.005, numberOfWindings = 5) annotation(Placement(transformation(origin = {40,6}, extent = {{ -10, -10},{10,10}}, rotation = 270)));

          equation 
            connect(bar2.frame_a,world.frame_b) annotation(Line(points = {{0,30},{ -40,30}}, color = {95,95,95}, thickness = 0.5));
            connect(spring1.frame_b,body.frame_a) annotation(Line(points = {{ -20, -4},{ -20, -30},{0, -30}}, color = {95,95,95}, thickness = 0.5));
            connect(bar2.frame_b,spring2.frame_a) annotation(Line(points = {{20,30},{40,30},{40,16}}, color = {95,95,95}, thickness = 0.5));
            connect(spring1.frame_a,world.frame_b) annotation(Line(points = {{ -20,16},{ -20,30},{ -40,30}}, color = {95,95,95}, thickness = 0.5));
            connect(body.frame_b,spring2.frame_b) annotation(Line(points = {{20, -30},{40, -30},{40, -4}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          end FreeBody;
          model ForceAndTorque "Demonstrate usage of ForceAndTorque element"
            extends Modelica.Icons.Example;
            inner World world(animateGravity = false) annotation(Placement(transformation(extent = {{ -90,30},{ -70,50}}, rotation = 0)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Documentation(info = "<HTML>
In this example the usage of the general force element
\"<a href=\"Modelica://Modelica.Mechanics.MultiBody.Forces.ForceAndTorque\">ForceAndTorque</a>\"
is shown. A \"ForceAndTorque\" element is connected
between a body and a fixed point in the world system. The force and torque
is defined by the \"Constant\" block. The two vectors are resolved in the
coordinate system defined by the \"fixedRotation\" component that is
fixed in the world system:
</p>
<p>
The animation view at time = 0 is shown in the figure below.
The yellow line is directed from frame_a to frame_b of the
forceAndTorque component. The green arrow characterizes the
force acting at the body whereas the green double arrow characterizes
the torque acting at the body. The lengths of the two vectors
are proportional to the lengths of the force and torque vectors
(constant scaling factors are defined as parameters in the
forceAndTorque component):
</p>
<IMG SRC=\"../Images/MultiBody/Examples/Elementary/ForceAndTorque.png\">
</HTML>
"));
            Parts.BodyCylinder body(r = {1,0,0}) annotation(Placement(transformation(extent = {{0,30},{20,50}}, rotation = 0)));
            Parts.Fixed fixed1(r = {0, -0.5,0}, width = 0.03) annotation(Placement(transformation(extent = {{ -90, -10},{ -70,10}}, rotation = 0)));
            Parts.FixedRotation fixedRotation(n = {0,0,1}, angle = 30) annotation(Placement(transformation(extent = {{ -8, -30},{12, -10}}, rotation = 0)));
            Forces.ForceAndTorque forceAndTorque(Nm_to_m = 120, N_to_m = 1200, resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve) annotation(Placement(transformation(extent = {{60,50},{40,30}}, rotation = 0)));
            Joints.Revolute revolute2(n = {0,1,0}, phi(fixed = true), w(fixed = true)) annotation(Placement(transformation(origin = { -20,20}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Modelica.Blocks.Sources.Constant torque[3](k = { -100,100,0}) annotation(Placement(transformation(origin = {40, -10}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
            Joints.Revolute revolute1(phi(fixed = true), w(fixed = true)) annotation(Placement(transformation(extent = {{ -60, -10},{ -40,10}}, rotation = 0)));
            Parts.Fixed fixed2(width = 0.03, r = {1.5,0.25,0}) annotation(Placement(transformation(extent = {{100,30},{80,50}}, rotation = 0)));
            Modelica.Blocks.Sources.Constant force[3](k = {0,1000,0}) annotation(Placement(transformation(origin = {80, -10}, extent = {{10, -10},{ -10,10}}, rotation = 270)));

          equation 
            connect(revolute2.frame_b,body.frame_a) annotation(Line(points = {{ -20,30},{ -20,40},{0,40}}, color = {95,95,95}, thickness = 0.5));
            connect(forceAndTorque.frame_b,body.frame_b) annotation(Line(points = {{40,40},{20,40}}, color = {95,95,95}, thickness = 0.5));
            connect(fixed1.frame_b,revolute1.frame_a) annotation(Line(points = {{ -70,0},{ -60,0}}, color = {95,95,95}, thickness = 0.5));
            connect(revolute1.frame_b,revolute2.frame_a) annotation(Line(points = {{ -40,0},{ -20,0},{ -20,10}}, color = {95,95,95}, thickness = 0.5));
            connect(fixed2.frame_b,forceAndTorque.frame_a) annotation(Line(points = {{80,40},{60,40}}, color = {95,95,95}, thickness = 0.5));
            connect(fixedRotation.frame_a,fixed1.frame_b) annotation(Line(points = {{ -8, -20},{ -65, -20},{ -65,0},{ -70,0}}, color = {95,95,95}, thickness = 0.5));
            connect(forceAndTorque.frame_resolve,fixedRotation.frame_b) annotation(Line(points = {{42,30},{42,30},{20,30},{20, -20},{12, -20}}, color = {95,95,95}, pattern = LinePattern.Dot));
            connect(force.y,forceAndTorque.force) annotation(Line(points = {{80,1},{80,10},{58,10},{58,28}}, color = {0,0,127}, smooth = Smooth.None));
            connect(torque.y,forceAndTorque.torque) annotation(Line(points = {{40,1},{40,10},{50,10},{50,28}}, color = {0,0,127}, smooth = Smooth.None));
          end ForceAndTorque;
          model DoublePendulum "Simple double pendulum with two revolute joints and two bodies"
            extends Modelica.Icons.Example;
            annotation(experiment(StopTime = 3), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
This example demonstrates that by using joint and body
elements animation is automatically available. Also the revolute
joints are animated. Note, that animation of every component
can be switched of by setting the first parameter <b>animation</b>
to <b>false</b> or by setting <b>enableAnimation</b> in the <b>world</b>
object to <b>false</b> to switch off animation of all components.
</p>

<table border=0 cellspacing=0 cellpadding=0><tr><td valign=\"top\">
<IMG SRC=\"../Images/MultiBody/Examples/Elementary/DoublePendulum.png\"
ALT=\"model Examples.Elementary.DoublePendulum\">
</td></tr></table>

</HTML>"));
            inner Modelica.Mechanics.MultiBody.World world annotation(Placement(transformation(extent = {{ -88,0},{ -68,20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute revolute1(useAxisFlange = true, phi(fixed = true), w(fixed = true)) annotation(Placement(transformation(extent = {{ -48,0},{ -28,20}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Components.Damper damper(d = 0.1) annotation(Placement(transformation(extent = {{ -48,40},{ -28,60}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.BodyBox boxBody1(r = {0.5,0,0}, width = 0.06) annotation(Placement(transformation(extent = {{ -10,0},{10,20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.Revolute revolute2(phi(fixed = true), w(fixed = true)) annotation(Placement(transformation(extent = {{32,0},{52,20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.BodyBox boxBody2(r = {0.5,0,0}, width = 0.06) annotation(Placement(transformation(extent = {{74,0},{94,20}}, rotation = 0)));

          equation 
            connect(damper.flange_b,revolute1.axis) annotation(Line(points = {{ -28,50},{ -24,50},{ -24,28},{ -38,28},{ -38,20}}, color = {0,0,0}));
            connect(revolute1.support,damper.flange_a) annotation(Line(points = {{ -44,20},{ -44,28},{ -58,28},{ -58,50},{ -48,50}}, color = {0,0,0}));
            connect(revolute1.frame_b,boxBody1.frame_a) annotation(Line(points = {{ -28,10},{ -10,10}}, color = {95,95,95}, thickness = 0.5));
            connect(revolute2.frame_b,boxBody2.frame_a) annotation(Line(points = {{52,10},{74,10}}, color = {95,95,95}, thickness = 0.5));
            connect(boxBody1.frame_b,revolute2.frame_a) annotation(Line(points = {{10,10},{32,10}}, color = {95,95,95}, thickness = 0.5));
            connect(world.frame_b,revolute1.frame_a) annotation(Line(points = {{ -68,10},{ -48,10}}, color = {95,95,95}, thickness = 0.5));
          end DoublePendulum;
        end Elementary;
        package Rotational3DEffects "Demonstrates the usage of 1-dim. rotational elements with all 3-dim. effects included"
          model GyroscopicEffects "Demonstrates that a cylindrical body can be replaced by Rotor1D model"
            extends Modelica.Icons.Example;
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 5, Tolerance = 1e-08), Documentation(info = "<html>
<p>
This example consists of a body that is attached to the world system
with a spherical joint. On this body, a \"rotor\", i.e., a body with rotational
symmetry is present. Two kinds of models are shown:
</p>

<ul>
<li> In the upper part of the diagram layer, only multi-body components are used.</li>
<li> In the lower part of the diagram layer, the same model is implemented,
     but by a different modeling of the cylindrical body:
     The cylindrical body is included, but it is rigidly attached to its mount.
     This part takes into account the movement of the center of mass and of
     the inertia tensor of the cylindrical body. Note, since the cylindrical body
     has rotational symmetry, its center of mass and its inertia tensor is independent
     of the angle of the inertia and can therefore be rigidly attached to its mount.
     Additionally, with a \"MultiBody.Parts.Rotor1D\" model, a primarily 1-dim.
     inertia is included that takes into account the additional effects when the
     cylindrical body is moving relatively to its mounts</li>
</ul>

<p>
The simulation reveals that both the kinematic movement and the reaction forces on the
environment (object \"world\" and \"fixed\" respectively) are identical for both models.
</p>

<p>
A typical usage scenario is to model a complete drive train of a vehicle, including
the automatic gearbox, with elements of the \"Mechanics.Rotational\" library, but using
the \"Rotor1D\" model instead of the \"Rotational.Components.Inertia\" component.
This drive train model can be mounted on a 3-dim. multi-body model of the vehicle.
Additionally, one rigid body has to be fixed to the vehicle that has the mass, center
of mass and inertia tensor of the complete drive train. Both models together, give
exactly the same effect, as if every part of the drive train would have been modelled
solely with mult-body components. One benefit of this modeling is that the simulation
is much faster.
</p>

</html>"));
            inner World world(driveTrainMechanics3D = true) annotation(Placement(transformation(extent = {{ -100,20},{ -80,40}}, rotation = 0)));
            Joints.Spherical spherical1(angles_fixed = true, w_rel_a_fixed = true, z_rel_a_fixed = false) annotation(Placement(transformation(extent = {{ -70,20},{ -50,40}}, rotation = 0)));
            Parts.BodyCylinder bodyCylinder1(r = {0.25,0,0}, diameter = 0.05) annotation(Placement(transformation(extent = {{ -44,20},{ -24,40}}, rotation = 0)));
            Parts.FixedRotation fixedRotation1(angle = 45, n = {0,1,0}) annotation(Placement(transformation(extent = {{ -16,20},{4,40}}, rotation = 0)));
            Joints.Revolute revolute(n = {1,0,0}, a(fixed = false), phi(fixed = true), w(fixed = true, start = 10)) annotation(Placement(transformation(extent = {{14,20},{34,40}}, rotation = 0)));
            Parts.FixedTranslation fixedTranslation(r = { -0.1,0,0}) annotation(Placement(transformation(extent = {{42,20},{62,40}}, rotation = 0)));
            Parts.BodyCylinder bodyCylinder2(diameter = 0.1, r = {0.2,0,0}) annotation(Placement(transformation(extent = {{70,20},{90,40}}, rotation = 0)));
            Parts.Fixed fixed annotation(Placement(transformation(extent = {{ -100, -40},{ -80, -20}}, rotation = 0)));
            Joints.Spherical spherical2(angles_fixed = true, w_rel_a_fixed = true, z_rel_a_fixed = false) annotation(Placement(transformation(extent = {{ -70, -40},{ -50, -20}}, rotation = 0)));
            Parts.BodyCylinder bodyCylinder3(r = {0.25,0,0}, diameter = 0.05, color = {0,128,0}) annotation(Placement(transformation(extent = {{ -44, -40},{ -24, -20}}, rotation = 0)));
            Parts.FixedRotation fixedRotation2(n = {0,1,0}, angle = 45) annotation(Placement(transformation(extent = {{ -16, -40},{4, -20}}, rotation = 0)));
            Parts.FixedTranslation fixedTranslation1(r = { -0.1,0,0}) annotation(Placement(transformation(extent = {{24, -40},{44, -20}}, rotation = 0)));
            Parts.BodyCylinder bodyCylinder4(diameter = 0.1, r = {0.2,0,0}, color = {0,128,0}) annotation(Placement(transformation(extent = {{56, -40},{76, -20}}, rotation = 0)));
            Parts.Rotor1D rotor1D(J = bodyCylinder4.I[1,1], n = {1,0,0}, a(fixed = false), phi(fixed = true), w(fixed = true, start = 10)) annotation(Placement(transformation(extent = {{28, -70},{48, -50}}, rotation = 0)));

          equation 
            connect(world.frame_b,spherical1.frame_a) annotation(Line(points = {{ -80,30},{ -70,30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(spherical1.frame_b,bodyCylinder1.frame_a) annotation(Line(points = {{ -50,30},{ -44,30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(bodyCylinder1.frame_b,fixedRotation1.frame_a) annotation(Line(points = {{ -24,30},{ -16,30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedRotation1.frame_b,revolute.frame_a) annotation(Line(points = {{4,30},{14,30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(revolute.frame_b,fixedTranslation.frame_a) annotation(Line(points = {{34,30},{42,30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedTranslation.frame_b,bodyCylinder2.frame_a) annotation(Line(points = {{62,30},{70,30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(spherical2.frame_b,bodyCylinder3.frame_a) annotation(Line(points = {{ -50, -30},{ -44, -30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(bodyCylinder3.frame_b,fixedRotation2.frame_a) annotation(Line(points = {{ -24, -30},{ -16, -30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixed.frame_b,spherical2.frame_a) annotation(Line(points = {{ -80, -30},{ -70, -30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedTranslation1.frame_b,bodyCylinder4.frame_a) annotation(Line(points = {{44, -30},{56, -30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixedRotation2.frame_b,fixedTranslation1.frame_a) annotation(Line(points = {{4, -30},{24, -30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(rotor1D.frame_a,fixedRotation2.frame_b) annotation(Line(points = {{38, -70},{38, -76},{10, -76},{10, -30},{4, -30}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          end GyroscopicEffects;
          model ActuatedDrive
            extends Modelica.Icons.Example;
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1.1));
            Parts.BodyShape bodyCylinder(r = {0.5,0,0}, m = 0, I_11 = 2, I_22 = 0, I_33 = 0, shapeType = "cylinder", width = 0.1, animateSphere = false, r_shape = {0.1,0,0}, r_CM = {0,0,0}) annotation(Placement(transformation(extent = {{8,0},{28,20}}, rotation = 0)));
            Joints.Revolute revolute(n = {1,0,0}, a(fixed = false), phi(fixed = true), w(fixed = true)) annotation(Placement(transformation(extent = {{ -26,0},{ -6,20}}, rotation = 0)));
            inner World world(g = 0, driveTrainMechanics3D = true) annotation(Placement(transformation(extent = {{ -60,0},{ -40,20}}, rotation = 0)));
            Forces.Torque torque annotation(Placement(transformation(extent = {{8,30},{28,50}}, rotation = 0)));
            Blocks.Sources.Sine sine[3](amplitude = {1,0,0}, freqHz = {1,1,1}) annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}}, rotation = 0)));
            inner Parts.Fixed fixed annotation(Placement(transformation(extent = {{ -62, -90},{ -42, -70}}, rotation = 0)));
            Parts.Rotor1D rotor1D(J = 2, phi(fixed = true), w(fixed = true)) annotation(Placement(transformation(extent = {{0, -40},{20, -20}}, rotation = 0)));
            Rotational.Sources.Torque torque1(useSupport = true) annotation(Placement(transformation(extent = {{ -32, -40},{ -12, -20}}, rotation = 0)));
            Parts.Mounting1D mounting1D annotation(Placement(transformation(extent = {{ -46, -60},{ -26, -40}}, rotation = 0)));

          equation 
            connect(world.frame_b,revolute.frame_a) annotation(Line(points = {{ -40,10},{ -26,10}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(revolute.frame_b,bodyCylinder.frame_a) annotation(Line(points = {{ -6,10},{8,10}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(torque.frame_b,bodyCylinder.frame_b) annotation(Line(points = {{28,40},{38,40},{38,10},{28,10}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(torque.frame_resolve,world.frame_b) annotation(Line(points = {{22,50},{22,64},{ -30,64},{ -30,10},{ -40,10}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
            connect(torque.frame_a,world.frame_b) annotation(Line(points = {{8,40},{ -30,40},{ -30,10},{ -40,10}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(sine.y,torque.torque) annotation(Line(points = {{ -59,70},{12,70},{12,52}}, color = {0,0,127}, smooth = Smooth.None));
            connect(fixed.frame_b,rotor1D.frame_a) annotation(Line(points = {{ -42, -80},{10, -80},{10, -40}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(torque1.flange,rotor1D.flange_a) annotation(Line(points = {{ -12, -30},{0, -30}}, color = {0,0,0}, smooth = Smooth.None));
            connect(mounting1D.flange_b,torque1.support) annotation(Line(points = {{ -26, -50},{ -22, -50},{ -22, -40}}, color = {0,0,0}, smooth = Smooth.None));
            connect(mounting1D.frame_a,fixed.frame_b) annotation(Line(points = {{ -36, -60},{ -36, -80},{ -42, -80}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(sine[1].y,torque1.tau) annotation(Line(points = {{ -59,70},{ -50,70},{ -50,40},{ -72,40},{ -72, -30},{ -34, -30}}, color = {0,0,127}, smooth = Smooth.None));
          end ActuatedDrive;
          model MovingActuatedDrive
            extends Modelica.Icons.Example;
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1.1));
            Parts.BodyShape bodyCylinder(r = {0.5,0,0}, m = 0, I_11 = 2, I_22 = 0, I_33 = 0, shapeType = "cylinder", width = 0.1, animateSphere = false, r_shape = {0.1,0,0}, r_CM = {0,0,0}) annotation(Placement(transformation(extent = {{50,10},{70,30}}, rotation = 0)));
            Joints.Revolute revolute(n = {1,0,0}, a(fixed = false), phi(fixed = true), w(fixed = true)) annotation(Placement(transformation(extent = {{16,10},{36,30}}, rotation = 0)));
            inner World world(g = 0, driveTrainMechanics3D = true) annotation(Placement(transformation(extent = {{ -84,10},{ -64,30}}, rotation = 0)));
            Forces.Torque torque annotation(Placement(transformation(extent = {{50,40},{70,60}}, rotation = 0)));
            Blocks.Sources.Sine sine1[3](amplitude = {1,0,0}, freqHz = {1,1,1}) annotation(Placement(transformation(extent = {{16,70},{36,90}}, rotation = 0)));
            Parts.Rotor1D rotor1D(J = 2, phi(fixed = true), w(fixed = true)) annotation(Placement(transformation(extent = {{50, -36},{70, -16}}, rotation = 0)));
            Rotational.Sources.Torque torque1(useSupport = true) annotation(Placement(transformation(extent = {{18, -36},{38, -16}}, rotation = 0)));
            Parts.Mounting1D mounting1D annotation(Placement(transformation(extent = {{4, -56},{24, -36}}, rotation = 0)));
            Joints.Revolute r1(useAxisFlange = true, n = {0,1,0}) annotation(Placement(transformation(extent = {{ -32,10},{ -12,30}}, rotation = 0)));
            Rotational.Sources.Position position1(useSupport = true) annotation(Placement(transformation(extent = {{ -46,60},{ -26,80}}, rotation = 0)));
            Blocks.Sources.Sine sine2(amplitude = 2, freqHz = 1) annotation(Placement(transformation(extent = {{ -100,60},{ -80,80}}, rotation = 0)));
            Parts.Mounting1D mounting1D1 annotation(Placement(transformation(extent = {{ -60,34},{ -40,54}}, rotation = 0)));
            Joints.Revolute r2(useAxisFlange = true, n = {0,1,0}) annotation(Placement(transformation(extent = {{ -38, -80},{ -18, -60}}, rotation = 0)));
            Rotational.Sources.Position position2(useSupport = true) annotation(Placement(transformation(extent = {{ -52, -30},{ -32, -10}}, rotation = 0)));
            Parts.Mounting1D mounting1D2 annotation(Placement(transformation(extent = {{ -66, -56},{ -46, -36}}, rotation = 0)));
            Parts.Fixed fixed annotation(Placement(transformation(extent = {{ -86, -80},{ -66, -60}}, rotation = 0)));

          equation 
            connect(revolute.frame_b,bodyCylinder.frame_a) annotation(Line(points = {{36,20},{50,20}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(torque.frame_b,bodyCylinder.frame_b) annotation(Line(points = {{70,50},{80,50},{80,20},{70,20}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(sine1.y,torque.torque) annotation(Line(points = {{37,80},{54,80},{54,62}}, color = {0,0,127}, smooth = Smooth.None));
            connect(torque1.flange,rotor1D.flange_a) annotation(Line(points = {{38, -26},{50, -26}}, color = {0,0,0}, smooth = Smooth.None));
            connect(mounting1D.flange_b,torque1.support) annotation(Line(points = {{24, -46},{28, -46},{28, -36}}, color = {0,0,0}, smooth = Smooth.None));
            connect(r1.frame_a,world.frame_b) annotation(Line(points = {{ -32,20},{ -64,20}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(position1.flange,r1.axis) annotation(Line(points = {{ -26,70},{ -22,70},{ -22,30}}, color = {0,0,0}, smooth = Smooth.None));
            connect(position1.support,mounting1D1.flange_b) annotation(Line(points = {{ -36,60},{ -36,44},{ -40,44}}, color = {0,0,0}, smooth = Smooth.None));
            connect(mounting1D1.frame_a,world.frame_b) annotation(Line(points = {{ -50,34},{ -50,20},{ -64,20}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(sine2.y,position1.phi_ref) annotation(Line(points = {{ -79,70},{ -48,70}}, color = {0,0,127}, smooth = Smooth.None));
            connect(r1.frame_b,revolute.frame_a) annotation(Line(points = {{ -12,20},{16,20}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(torque.frame_resolve,revolute.frame_a) annotation(Line(points = {{64,60},{8,60},{8,20},{16,20}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
            connect(torque.frame_a,revolute.frame_a) annotation(Line(points = {{50,50},{8,50},{8,20},{16,20}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(position2.flange,r2.axis) annotation(Line(points = {{ -32, -20},{ -28, -20},{ -28, -60}}, color = {0,0,0}, smooth = Smooth.None));
            connect(position2.support,mounting1D2.flange_b) annotation(Line(points = {{ -42, -30},{ -42, -46},{ -46, -46}}, color = {0,0,0}, smooth = Smooth.None));
            connect(fixed.frame_b,r2.frame_a) annotation(Line(points = {{ -66, -70},{ -38, -70}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(fixed.frame_b,mounting1D2.frame_a) annotation(Line(points = {{ -66, -70},{ -56, -70},{ -56, -56}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(sine2.y,position2.phi_ref) annotation(Line(points = {{ -79,70},{ -70,70},{ -70,46},{ -92,46},{ -92, -20},{ -54, -20}}, color = {0,0,127}, smooth = Smooth.None));
            connect(r2.frame_b,rotor1D.frame_a) annotation(Line(points = {{ -18, -70},{60, -70},{60, -36}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(r2.frame_b,mounting1D.frame_a) annotation(Line(points = {{ -18, -70},{14, -70},{14, -56}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(sine1[1].y,torque1.tau) annotation(Line(points = {{37,80},{92,80},{92, -10},{0, -10},{0, -26},{16, -26}}, color = {0,0,127}, smooth = Smooth.None));
          end MovingActuatedDrive;
          model GearConstraint
            extends Modelica.Icons.Example;
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 5));
            Joints.GearConstraint gearConstraint(ratio = 10) annotation(Placement(transformation(extent = {{34,40},{54,60}}, rotation = 0)));
            inner World world(driveTrainMechanics3D = true, g = 0) annotation(Placement(transformation(extent = {{ -62,10},{ -42,30}}, rotation = 0)));
            Parts.BodyCylinder cyl1(diameter = 0.1, color = {0,128,0}, r = {0.4,0,0}) annotation(Placement(transformation(extent = {{2,40},{22,60}}, rotation = 0)));
            Parts.BodyCylinder cyl2(r = {0.4,0,0}, diameter = 0.2) annotation(Placement(transformation(extent = {{70,40},{90,60}}, rotation = 0)));
            Forces.Torque torque1 annotation(Placement(transformation(extent = {{ -26,40},{ -6,60}}, rotation = 0)));
            Blocks.Sources.Sine sine[3](amplitude = {2,0,0}, freqHz = {1,1,1}) annotation(Placement(transformation(extent = {{ -100,60},{ -80,80}}, rotation = 0)));
            Parts.Fixed fixed annotation(Placement(transformation(extent = {{ -48, -90},{ -28, -70}}, rotation = 0)));
            Rotational.Components.Inertia inertia1(J = cyl1.I[1,1], a(fixed = false), phi(fixed = true), w(fixed = true)) annotation(Placement(transformation(extent = {{ -20, -40},{0, -20}}, rotation = 0)));
            Rotational.Components.IdealGear idealGear(ratio = 10, useSupport = true) annotation(Placement(transformation(extent = {{12, -40},{32, -20}}, rotation = 0)));
            Rotational.Components.Inertia inertia2(J = cyl2.I[1,1]) annotation(Placement(transformation(extent = {{44, -40},{64, -20}}, rotation = 0)));
            Rotational.Sources.Torque torque2(useSupport = true) annotation(Placement(transformation(extent = {{ -48, -40},{ -28, -20}}, rotation = 0)));
            Parts.Mounting1D mounting1D annotation(Placement(transformation(extent = {{ -20, -70},{0, -50}}, rotation = 0)));

          equation 
            connect(world.frame_b,gearConstraint.bearing) annotation(Line(points = {{ -42,20},{44,20},{44,40}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(cyl1.frame_b,gearConstraint.frame_a) annotation(Line(points = {{22,50},{34,50}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(gearConstraint.frame_b,cyl2.frame_a) annotation(Line(points = {{54,50},{70,50}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(torque1.frame_b,cyl1.frame_a) annotation(Line(points = {{ -6,50},{2,50}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(torque1.frame_a,world.frame_b) annotation(Line(points = {{ -26,50},{ -36,50},{ -36,20},{ -42,20}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
            connect(sine.y,torque1.torque) annotation(Line(points = {{ -79,70},{ -22,70},{ -22,62}}, color = {0,0,127}, smooth = Smooth.None));
            connect(inertia1.flange_b,idealGear.flange_a) annotation(Line(points = {{0, -30},{12, -30}}, color = {0,0,0}, smooth = Smooth.None));
            connect(idealGear.flange_b,inertia2.flange_a) annotation(Line(points = {{32, -30},{44, -30}}, color = {0,0,0}, smooth = Smooth.None));
            connect(torque2.flange,inertia1.flange_a) annotation(Line(points = {{ -28, -30},{ -20, -30}}, color = {0,0,0}, smooth = Smooth.None));
            connect(sine[1].y,torque2.tau) annotation(Line(points = {{ -79,70},{ -72,70},{ -72, -30},{ -50, -30}}, color = {0,0,127}, smooth = Smooth.None));
            connect(mounting1D.flange_b,idealGear.support) annotation(Line(points = {{0, -60},{22, -60},{22, -40}}, color = {0,0,0}, smooth = Smooth.None));
            connect(mounting1D.flange_b,torque2.support) annotation(Line(points = {{0, -60},{4, -60},{4, -48},{ -38, -48},{ -38, -40}}, color = {0,0,0}, smooth = Smooth.None));
            connect(fixed.frame_b,mounting1D.frame_a) annotation(Line(points = {{ -28, -80},{ -10, -80},{ -10, -70}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          end GearConstraint;
          annotation(Documentation(info = "<html>
<p>
This library demonstrates the usage of elements of the Mechanics.Rotational library
by taking into account all 3-dim. effects. The reason for this type of modeling is
to speedup the simulation drastically. This is possible if moving bodies have
rotational symmetry. A typical application area are drive trains, driving joints of
a multi-body system.
</p>
</html>"));
        end Rotational3DEffects;
      end Examples;
      package Visualizers "3-dimensional visual objects used for animation"
        extends Modelica.Icons.Library;
        annotation(Documentation(info = "<HTML>
<p>
Package <b>Visualizers</b> contains components to visualize
3-dimensional shapes. These components are the basis for the
animation features of the MultiBody library.
<p>
<h4>Content</h4>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Visualizers.FixedShape\">FixedShape</a><br>
             <a href=\"Modelica://Modelica.Mechanics.MultiBody.Visualizers.FixedShape2\">FixedShape2</a></td>
      <td valign=\"top\">Animation shape of a part with fixed sizes. FixedShape2 has additionally
          a frame_b for easier connection to further visual objects.
          The following shape types are supported:<br>&nbsp;<br>
      <IMG SRC=\"../Images/MultiBody/FixedShape.png\" ALT=\"model Visualizers.FixedShape\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Visualizers.FixedFrame\">FixedFrame</a></td>
      <td valign=\"top\">Visualizing a coordinate system including axes labels with fixed sizes:<br>
      <IMG SRC=\"../Images/MultiBody/FixedFrame2.png\"
       ALT=\"model Visualizers.FixedFrame\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Visualizers.FixedArrow\">FixedArrow</a>,<br>
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Visualizers.SignalArrow\">SignalArrow</a></td>
      <td valign=\"top\">Visualizing an arrow. Model \"FixedArrow\" provides
      a fixed sized arrow, model \"SignalArrow\" provides
      an arrow with dynamically varying length that is defined
      by an input signal vector:<br>
      <IMG SRC=\"../Images/MultiBody/Visualizers/Arrow.png\" \">
      </td>
  </tr>
<tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Visualizers.Advanced\">Advanced</a></td>
      <td valign=\"top\"> <b>Package</b> that contains components to visualize
          3-dimensional shapes where all parts of the shape
          can vary dynamically. Basic knowledge of Modelica is
          needed in order to utilize the components of this package.
      </td>
  </tr>
</table>
<p>
The colors of the visualization components are declared with
the predefined type <b>MultiBody.Types.Color</b>.
This is a vector with 3 elements,
{r, g, b}, and specifies the color of the shape.
{r,g,b} are the \"red\", \"green\" and \"blue\" color parts.
Note, r g, b are given as Integer[3] in the ranges 0 .. 255,
respectively.
</p>
</HTML>"));
        model FixedShape "Animation shape of a part with fixed shape type and dynamically varying shape definition"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Types;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialVisualizer;
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Types.ShapeType shapeType = "box" "Type of shape" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Position r_shape[3] = {0,0,0} "Vector from frame_a to shape origin, resolved in frame_a" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Axis lengthDirection = {1,0,0} "Vector in length direction of shape, resolved in frame_a" annotation(Evaluate = true, Dialog(group = "if animation = true", enable = animation));
          input Types.Axis widthDirection = {0,1,0} "Vector in width direction of shape, resolved in frame_a" annotation(Evaluate = true, Dialog(group = "if animation = true", enable = animation));
          input SI.Distance length(start = 1) "Length of shape" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Distance width(start = 0.1) "Width of shape" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Distance height(start = 0.1) "Height of shape" annotation(Dialog(group = "if animation = true", enable = animation));
          input Modelica.Mechanics.MultiBody.Types.Color color = {0,128,255} "Color of shape" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.ShapeExtra extra = 0.0 "Additional data for cylinder, cone, pipe, gearwheel and spring" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(group = "if animation = true", enable = animation));
          annotation(Documentation(info = "<HTML>
<p>
Model <b>FixedShape</b> defines a visual shape that is
shown at the location of its frame_a.
All describing data such as size and color can vary dynamically by
providing appropriate expressions in the input fields of the
parameter menu. The only exception is parameter shapeType
that cannot be changed during simulation.
The following shapes are currently supported via
parameter <b>shapeType</b> (e.g., shapeType=\"box\"):<br>&nbsp;
</p>
<IMG SRC=\"../Images/MultiBody/Shape.png\" ALT=\"model Visualizers.FixedShape\">
<p>&nbsp;<br>
The dark blue arrows in the figure above are directed along
variable <b>lengthDirection</b>. The light blue arrows are directed
along variable <b>widthDirection</b>. The <b>coordinate systems</b>
in the figure represent frame_a of the FixedShape component.
</p>
<p>
Additionally external shapes are specified as DXF-files
(only 3-dim.Face is supported). External shapes must be named \"1\", \"2\"
etc.. The corresponding definitions should be in files \"1.dxf\",
\"2.dxf\" etc.Since the DXF-files contain color and dimensions for
the individual faces, the corresponding information in the model
is currently ignored. The DXF-files must be found in the current directory.
</p>
<p>
The sizes of any of the above components are specified by the
<b>length</b>, <b>width</b> and <b>height</b> variables.
Via variable <b>extra</b> additional data can be defined:
</p>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>shapeType</b></th><th>Meaning of parameter <b>extra</b></th></tr>
<tr>
  <td valign=\"top\">\"cylinder\"</td>
  <td valign=\"top\">if extra &gt; 0, a black line is included in the
      cylinder to show the rotation of it.</td>
</tr>
<tr>
  <td valign=\"top\">\"cone\"</td>
  <td valign=\"top\">extra = diameter-left-side / diameter-right-side, i.e.,<br>
      extra = 1: cylinder<br>
      extra = 0: \"real\" cone.</td>
</tr>
<tr>
  <td valign=\"top\">\"pipe\"</td>
  <td valign=\"top\">extra = outer-diameter / inner-diameter, i.e, <br>
      extra = 1: cylinder that is completely hollow<br>
      extra = 0: cylinder without a hole.</td>
</tr>
<tr>
  <td valign=\"top\">\"gearwheel\"</td>
  <td valign=\"top\">extra is the number of teeth of the gear.</td>
</tr>
<tr>
  <td valign=\"top\">\"spring\"</td>
  <td valign=\"top\">extra is the number of windings of the spring.
      Additionally, \"height\" is <b>not</b> the \"height\" but
      2*coil-width.</td>
</tr>
</table>
<p>
Parameter <b>color</b> is a vector with 3 elements,
{r, g, b}, and specifies the color of the shape.
{r,g,b} are the \"red\", \"green\" and \"blue\" color parts.
Note, r g, b are given as Integer[3] in the ranges 0 .. 255,
respectively. The predefined type
<b>MultiBody.Types.Color</b> contains a temporary menu
definition of the colors used in the MultiBody library
(this will be replaced by a color editor).
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -100,33},{ -100, -61},{ -10, -41},{50, -61},{50,33},{ -10,11},{ -100,33}}, lineColor = {0,127,255}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -98,34},{ -64,46},{0,30},{74,56},{50,32},{ -10,12},{ -98,34}}, lineColor = {255,255,255}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Polygon(points = {{72,54},{50,32},{50, -60},{72, -32},{72,54}}, lineColor = {255,255,255}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Text(extent = {{ -131, -65},{134, -104}}, lineColor = {0,0,0}, textString = "%shapeType"),Text(extent = {{ -131,118},{129,58}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
        protected 
          Advanced.Shape vis(shapeType = shapeType, r_shape = r_shape, lengthDirection = lengthDirection, widthDirection = widthDirection, length = length, width = width, height = height, color = color, extra = extra, specularCoefficient = specularCoefficient, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;

        equation 
          frame_a.f = zeros(3);
          frame_a.t = zeros(3);
        end FixedShape;
        model FixedShape2 "Animation shape of a part with fixed shape type and dynamically varying shape definition with two frames"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Frames;
          import Modelica.Mechanics.MultiBody.Types;
          Interfaces.Frame_a frame_a "Coordinate system a (all shape definition vectors are resolved in this frame)" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
          Interfaces.Frame_b frame_b "Coordinate system b" annotation(Placement(transformation(extent = {{84, -16},{116,16}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Types.ShapeType shapeType = "box" "Type of shape" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Position r[3] = {1,0,0} "Vector from frame_a to frame_b resolved in frame_a";
          input SI.Position r_shape[3] = {0,0,0} "Vector from frame_a to shape origin, resolved in frame_a" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Axis lengthDirection = r - r_shape "Vector in length direction of shape, resolved in frame_a" annotation(Evaluate = true, Dialog(group = "if animation = true", enable = animation));
          input Types.Axis widthDirection = {0,1,0} "Vector in width direction of shape, resolved in frame_a" annotation(Evaluate = true, Dialog(group = "if animation = true", enable = animation));
          input SI.Length length = Modelica.Math.Vectors.length(r - r_shape) "Length of shape" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Distance width = 0.1 "Width of shape" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Distance height = width "Height of shape" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.ShapeExtra extra = 0.0 "Additional data for cylinder, cone, pipe, gearwheel and spring" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color color = {0,128,255} "Color of shape" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(group = "if animation = true", enable = animation));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -123,128},{137,68}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{129, -106},{ -131, -56}}, lineColor = {0,0,0}, textString = "%=r"),Polygon(points = {{ -100,50},{ -100, -44},{ -10, -24},{79, -43},{80,49},{ -10,28},{ -100,50}}, lineColor = {0,127,255}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -101,50},{ -65,62},{ -1,46},{100,70},{79,49},{ -11,28},{ -101,50}}, lineColor = {255,255,255}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Text(extent = {{ -86,15},{ -50, -10}}, lineColor = {0,0,0}, textString = "a"),Text(extent = {{37,15},{73, -10}}, lineColor = {0,0,0}, textString = "b"),Polygon(points = {{100,70},{78,48},{78, -44},{100, -16},{100,70}}, lineColor = {255,255,255}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Documentation(info = "<HTML>
<p>
Model <b>FixedShape2</b> defines a visual shape that is
shown at the location of its frame_a. This model is identical
to <b>FixedShape</b> with the only difference that an
additional frame_b is present which is parallel to frame_a.
This makes it more convenient to connect several visual
shapes together when building up more complex visual
objects. All describing data such as size and color can vary dynamically by
providing appropriate expressions in the input fields of the
parameter menu. The only exception is parameter shapeType
that cannot be changed during simulation.
The following shapes are currently supported via
parameter <b>shapeType</b> (e.g., shapeType=\"box\"):<br>&nbsp;
</p>
<IMG SRC=\"../Images/MultiBody/Shape.png\" ALT=\"model Visualizers.FixedShape2\">
<p>&nbsp;<br>
The dark blue arrows in the figure above are directed along
variable <b>lengthDirection</b>. The light blue arrows are directed
along variable <b>widthDirection</b>. The <b>coordinate systems</b>
in the figure represent frame_a of the FixedShape component.
</p>
<p>
Additionally external shapes are specified as DXF-files
(only 3-dim.Face is supported). External shapes must be named \"1\", \"2\"
etc.. The corresponding definitions should be in files \"1.dxf\",
\"2.dxf\" etc.Since the DXF-files contain color and dimensions for
the individual faces, the corresponding information in the model
is currently ignored. The DXF-files must be found in the current directory.
</p>
<p>
The sizes of any of the above components are specified by the
<b>length</b>, <b>width</b> and <b>height</b> variables.
Via variable <b>extra</b> additional data can be defined:
</p>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>shapeType</b></th><th>Meaning of parameter <b>extra</b></th></tr>
<tr>
  <td valign=\"top\">\"cylinder\"</td>
  <td valign=\"top\">if extra &gt; 0, a black line is included in the
      cylinder to show the rotation of it.</td>
</tr>
<tr>
  <td valign=\"top\">\"cone\"</td>
  <td valign=\"top\">extra = diameter-left-side / diameter-right-side, i.e.,<br>
      extra = 1: cylinder<br>
      extra = 0: \"real\" cone.</td>
</tr>
<tr>
  <td valign=\"top\">\"pipe\"</td>
  <td valign=\"top\">extra = outer-diameter / inner-diameter, i.e, <br>
      extra = 1: cylinder that is completely hollow<br>
      extra = 0: cylinder without a hole.</td>
</tr>
<tr>
  <td valign=\"top\">\"gearwheel\"</td>
  <td valign=\"top\">extra is the number of teeth of the gear.</td>
</tr>
<tr>
  <td valign=\"top\">\"spring\"</td>
  <td valign=\"top\">extra is the number of windings of the spring.
      Additionally, \"height\" is <b>not</b> the \"height\" but
      2*coil-width.</td>
</tr>
</table>
<p>
Parameter <b>color</b> is a vector with 3 elements,
{r, g, b}, and specifies the color of the shape.
{r,g,b} are the \"red\", \"green\" and \"blue\" color parts.
Note, r g, b are given as Integer[3] in the ranges 0 .. 255,
respectively. The predefined type
<b>MultiBody.Types.Color</b> contains a temporary menu
definition of the colors used in the MultiBody library
(this will be replaced by a color editor).
</p>
<p>
In the following figure the relationships between
frame_a and frame_b are shown. The origin of frame_b
with respect to frame_a is specified via parameter
vector <b>r</b>.
</p>
<IMG SRC=\"../Images/MultiBody/FixedTranslation.png\" ALT=\"Parts.FixedTranslation\">
</HTML>
"));
        protected 
          outer MultiBody.World world;
          Advanced.Shape shape(shapeType = shapeType, r_shape = r_shape, lengthDirection = lengthDirection, widthDirection = widthDirection, length = length, width = width, height = height, extra = extra, color = color, specularCoefficient = specularCoefficient, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;

        equation 
          Connections.branch(frame_a.R, frame_b.R);
          assert(cardinality(frame_a) > 0 or cardinality(frame_b) > 0, "Neither connector frame_a nor frame_b of
MultiBody.Visualizers.FixedShape2 object is connected");
          frame_b.r_0 = frame_a.r_0 + Frames.resolve1(frame_a.R, r);
          frame_b.R = frame_a.R;
          zeros(3) = frame_a.f + frame_b.f;
          zeros(3) = frame_a.t + frame_b.t + cross(r, frame_b.f);
        end FixedShape2;
        model FixedFrame "Visualizing a coordinate system including axes labels (visualization data may vary dynamically)"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Types;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialVisualizer;
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Boolean showLabels = true "= true, if labels shall be shown" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Distance length = 0.5 "Length of axes arrows" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Distance diameter = length / world.defaultFrameDiameterFraction "Diameter of axes arrows" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color color_x = Modelica.Mechanics.MultiBody.Types.Defaults.FrameColor "Color of x-arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color color_y = color_x "Color of y-arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color color_z = color_x "Color of z-arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(group = "if animation = true", enable = animation));
        protected 
          parameter Boolean animation2 = world.enableAnimation and animation;
          parameter Boolean showLabels2 = world.enableAnimation and animation and showLabels;
          SI.Length headLength = min(length, diameter * Types.Defaults.FrameHeadLengthFraction);
          SI.Length headWidth = diameter * Types.Defaults.FrameHeadWidthFraction;
          SI.Length lineLength = max(0, length - headLength);
          SI.Length lineWidth = diameter;
          SI.Length scaledLabel = Modelica.Mechanics.MultiBody.Types.Defaults.FrameLabelHeightFraction * diameter;
          SI.Length labelStart = 1.05 * length;
          Visualizers.Advanced.Shape x_arrowLine(shapeType = "cylinder", length = lineLength, width = lineWidth, height = lineWidth, lengthDirection = {1,0,0}, widthDirection = {0,1,0}, color = color_x, specularCoefficient = specularCoefficient, r = frame_a.r_0, R = frame_a.R) if animation2;
          Visualizers.Advanced.Shape x_arrowHead(shapeType = "cone", length = headLength, width = headWidth, height = headWidth, lengthDirection = {1,0,0}, widthDirection = {0,1,0}, color = color_x, specularCoefficient = specularCoefficient, r = frame_a.r_0 + Frames.resolve1(frame_a.R, {lineLength,0,0}), R = frame_a.R) if animation2;
          Visualizers.Internal.Lines x_label(lines = scaledLabel * {[0,0;1,1],[0,1;1,0]}, diameter = diameter, color = color_x, specularCoefficient = specularCoefficient, r_lines = {labelStart,0,0}, n_x = {1,0,0}, n_y = {0,1,0}, r = frame_a.r_0, R = frame_a.R) if showLabels2;
          Visualizers.Advanced.Shape y_arrowLine(shapeType = "cylinder", length = lineLength, width = lineWidth, height = lineWidth, lengthDirection = {0,1,0}, widthDirection = {1,0,0}, color = color_y, specularCoefficient = specularCoefficient, r = frame_a.r_0, R = frame_a.R) if animation2;
          Visualizers.Advanced.Shape y_arrowHead(shapeType = "cone", length = headLength, width = headWidth, height = headWidth, lengthDirection = {0,1,0}, widthDirection = {1,0,0}, color = color_y, specularCoefficient = specularCoefficient, r = frame_a.r_0 + Frames.resolve1(frame_a.R, {0,lineLength,0}), R = frame_a.R) if animation2;
          Visualizers.Internal.Lines y_label(lines = scaledLabel * {[0,0;1,1.5],[0,1.5;0.5,0.75]}, diameter = diameter, color = color_y, specularCoefficient = specularCoefficient, r_lines = {0,labelStart,0}, n_x = {0,1,0}, n_y = { -1,0,0}, r = frame_a.r_0, R = frame_a.R) if showLabels2;
          Visualizers.Advanced.Shape z_arrowLine(shapeType = "cylinder", length = lineLength, width = lineWidth, height = lineWidth, lengthDirection = {0,0,1}, widthDirection = {0,1,0}, color = color_z, specularCoefficient = specularCoefficient, r = frame_a.r_0, R = frame_a.R) if animation2;
          Visualizers.Advanced.Shape z_arrowHead(shapeType = "cone", length = headLength, width = headWidth, height = headWidth, lengthDirection = {0,0,1}, widthDirection = {0,1,0}, color = color_z, specularCoefficient = specularCoefficient, r = frame_a.r_0 + Frames.resolve1(frame_a.R, {0,0,lineLength}), R = frame_a.R) if animation2;
          Visualizers.Internal.Lines z_label(lines = scaledLabel * {[0,0;1,0],[0,1;1,1],[0,1;1,0]}, diameter = diameter, color = color_z, specularCoefficient = specularCoefficient, r_lines = {0,0,labelStart}, n_x = {0,0,1}, n_y = {0,1,0}, r = frame_a.r_0, R = frame_a.R) if showLabels2;

        equation 
          frame_a.f = zeros(3);
          frame_a.t = zeros(3);
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,127,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -2,92},{ -14,52},{12,52},{ -2,92},{ -2,92}}, lineColor = {0,191,0}, fillColor = {0,191,0}, fillPattern = FillPattern.Solid),Line(points = {{ -2, -18},{ -2,52}}, color = {0,191,0}, thickness = 0.5),Text(extent = {{16,93},{67,43}}, lineColor = {0,191,0}, textString = "y"),Text(extent = {{43,11},{92, -38}}, lineColor = {0,0,0}, textString = "x"),Polygon(points = {{98, -70},{74, -44},{64, -60},{98, -70}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -2, -18},{72, -54}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -72, -54},{ -2, -18}}, thickness = 0.5, color = {0,0,255}),Polygon(points = {{ -98, -68},{ -66, -60},{ -78, -46},{ -98, -68}}, lineColor = {0,0,255}),Text(extent = {{ -87,13},{ -38, -36}}, textString = "z", lineColor = {0,0,255}),Text(extent = {{ -138,164},{146,102}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<HTML>
<p>
Model <b>FixedFrame</b> visualizes the three axes of
its coordinate system <b>frame_a</b> together with appropriate axes
labels. A typical example is shown in the following figure:
<br>&nbsp;
</p>
<IMG SRC=\"../Images/MultiBody/FixedFrame.png\" ALT=\"model Visualizers.FixedFrame\">
<p>
The sizes of the axes, the axes colors and the specular coefficient
(= reflection factor for
ambient light) can vary dynamically by
providing appropriate expressions in the input fields of the
parameter menu.
</p>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
        end FixedFrame;
        model FixedArrow "Visualizing an arrow with dynamically varying size in frame_a"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Types;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialVisualizer;
          parameter Boolean animation = true "= true, if animation shall be enabled";
          input SI.Position r_tail[3] = {0,0,0} " Vector from frame_a to arrow tail, resolved in frame_a" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Axis n = {1,0,0} " Vector in arrow direction, resolved in frame_a" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Length length = 0.1 " Length of complete arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter diameter = world.defaultArrowDiameter " Diameter of arrow line" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color color = {0,0,255} " Color of arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(group = "if animation = true", enable = animation));
        protected 
          SI.Length headLength = min(length, diameter * Types.Defaults.ArrowHeadLengthFraction);
          SI.Length headWidth = diameter * Types.Defaults.ArrowHeadWidthFraction;
          SI.Length lineLength = max(0, length - headLength);
          Visualizers.Advanced.Shape arrowLine(shapeType = "cylinder", length = lineLength, width = diameter, height = diameter, lengthDirection = n, widthDirection = {0,1,0}, color = color, specularCoefficient = specularCoefficient, r_shape = r_tail, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          Visualizers.Advanced.Shape arrowHead(shapeType = "cone", length = headLength, width = headWidth, height = headWidth, lengthDirection = n, widthDirection = {0,1,0}, color = color, specularCoefficient = specularCoefficient, r_shape = r_tail + Modelica.Math.Vectors.normalize(n) * lineLength, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,28},{20, -30}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Polygon(points = {{20,60},{100,0},{20, -60},{20,60}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{ -146,124},{142,62}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -118, -74},{120, -112}}, lineColor = {0,0,0}, textString = "%length")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
Model <b>FixedArrow</b> defines an arrow that is
shown at the location of its frame_a.
<br>&nbsp;
</p>
<IMG SRC=\"../Images/MultiBody/Visualizers/Arrow.png\" ALT=\"model Visualizers.FixedArrow\">
<p>
The direction of the arrow specified with vector
<b>n</b> is with respect to frame_a, i.e., the local frame to which the
arrow component is attached. The direction and length of the arrow, its diameter
and color can vary dynamically by
providing appropriate expressions in the input fields of the
parameter menu.
</p>
</html>"));

        equation 
          frame_a.f = zeros(3);
          frame_a.t = zeros(3);
        end FixedArrow;
        model SignalArrow "Visualizing an arrow with dynamically varying size in frame_a based on input signal"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Types;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialVisualizer;
          parameter Boolean animation = true "= true, if animation shall be enabled";
          input SI.Position r_tail[3] = {0,0,0} "Vector from frame_a to arrow tail, resolved in frame_a" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter diameter = world.defaultArrowDiameter "Diameter of arrow line" annotation(Dialog(group = "if animation = true", enable = animation));
          input Modelica.Mechanics.MultiBody.Types.Color color = {0,0,255} "Color of arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(group = "if animation = true", enable = animation));
          Modelica.Blocks.Interfaces.RealInput r_head[3](each final quantity = "Position", each final unit = "m") "Position vector from origin of frame_a to head of arrow, resolved in frame_a" annotation(Placement(transformation(origin = {0, -120}, extent = {{ -20, -20},{20,20}}, rotation = 90)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,28},{20, -30}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Polygon(points = {{20,60},{100,0},{20, -60},{20,60}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{ -146,124},{142,62}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{0, -102},{0, -30}}, color = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<HTML>
<p>
Model <b>SignalArrow</b> defines an arrow that is dynamically visualized
at the location where its frame_a is attached. The
position vector from the tail to the head of the arrow,
resolved in frame_a, is defined via the signal vector of
the connector <b>r_head</b> (Real r_head[3]):<br>&nbsp;
</p>
<IMG SRC=\"../Images/MultiBody/Visualizers/Arrow.png\" ALT=\"model Visualizers.SignalArrow\">
<p>
The tail of the arrow is defined with parameter <b>r_tail</b>
with respect to frame_a (vector from the origin of frame_a to the arrow tail).
</p>
</HTML>"));
        protected 
          Visualizers.Advanced.Arrow arrow(R = frame_a.R, r = frame_a.r_0, r_tail = r_tail, r_head = r_head, diameter = diameter, color = color, specularCoefficient = specularCoefficient) if world.enableAnimation and animation;

        equation 
          frame_a.f = zeros(3);
          frame_a.t = zeros(3);
        end SignalArrow;
        model Ground "Visualize ground (box in z=0)"
          parameter Boolean animation = true "= true, if animation of ground shall be enabled";
          parameter Modelica.SIunits.Position length = 10 "Length and width of box (center is at x=y=0)" annotation(Dialog(enable = animation));
          parameter Modelica.SIunits.Position height = 0.02 "Height of box (upper surface is at z=0, lower surface is at z=-height)" annotation(Dialog(enable = animation));
          parameter Modelica.Mechanics.MultiBody.Types.Color groundColor = {0,255,0} "Color of box" annotation(Dialog(enable = animation));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -20,60},{ -100, -32},{20, -92},{100,8},{ -20,60}}, lineColor = {0,0,0}, smooth = Smooth.None, fillColor = {0,255,0}, fillPattern = FillPattern.Solid),Text(extent = {{2,82},{44,54}}, lineColor = {0,0,0}, textString = "z"),Polygon(points = {{100,8},{100, -4},{20, -104},{20, -92},{100,8}}, lineColor = {0,0,0}, smooth = Smooth.None, fillColor = {0,255,0}, fillPattern = FillPattern.Solid),Polygon(points = {{ -100, -32},{ -100, -44},{20, -104},{20, -92},{ -100, -32}}, lineColor = {0,0,0}, smooth = Smooth.None, fillColor = {0,255,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -11, -7},{ -1, -12}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -6, -10},{ -6,88}}, color = {0,0,0}, smooth = Smooth.None),Polygon(points = {{ -6,102},{ -14,72},{2,72},{ -6,102}}, lineColor = {0,0,0}, smooth = Smooth.None, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -146, -96},{154, -136}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
This shape visualizes the x-y plane by a box
</p>

</html>"));
          Modelica.Mechanics.MultiBody.Visualizers.FixedShape ground(lengthDirection = {1,0,0}, widthDirection = {0,1,0}, animation = animation, r_shape = { -length / 2,0, -height}, length = length, height = height, color = groundColor, width = length) annotation(Placement(transformation(extent = {{ -20,0},{0,20}})));
          Modelica.Mechanics.MultiBody.Parts.Fixed fixed annotation(Placement(transformation(extent = {{ -60,0},{ -40,20}})));

        equation 
          connect(fixed.frame_b,ground.frame_a) annotation(Line(points = {{ -40,10},{ -20,10}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        end Ground;
        package Advanced "Visualizers that require basic knowledge about Modelica in order to use them"
          extends Modelica.Icons.Library;
          annotation(Documentation(info = "<HTML>
<p>
Package <b>Visualizers.Advanced</b> contains components to visualize
3-dimensional shapes with dynamical sizes. None of the components
has a frame connector. The position and orientation is set via
modifiers. Basic knowledge of Modelica
is needed in order to utilize the components of this package.
These components have also to be used for models,
where the forces and torques in the frame connector are set via
equations (in this case, the models of the Visualizers package cannot be used,
since they all have frame connectors).
<p>
<h4>Content</h4>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Visualizers.Advanced.Arrow\">Arrow</a></td>
      <td valign=\"top\">Visualizing an arrow where all parts of the arrow can vary dynamically:<br>
      <IMG SRC=\"../Images/MultiBody/Visualizers/Arrow.png\" ALT=\"model Visualizers.Advanced.Arrow\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Visualizers.Advanced.DoubleArrow\">DoubleArrow</a></td>
      <td valign=\"top\">Visualizing a double arrow where all parts of the arrow can vary dynamically:<br>
      <IMG SRC=\"../Images/MultiBody/Visualizers/DoubleArrow.png\" ALT=\"model Visualizers.Advanced.DoubleArrow\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape\">Shape</a></td>
      <td valign=\"top\">Animation shape of a part with dynamically varying sizes.
      The following shape types are supported:<br>&nbsp;<br>
      <IMG SRC=\"../Images/MultiBody/FixedShape.png\" ALT=\"model Visualizers.Advanced.Shape\">
      </td>
  </tr>
</table>
</HTML>"));
          model Arrow "Visualizing an arrow with variable size; all data have to be set as modifiers (see info layer)"
            import SI = Modelica.SIunits;
            import Modelica.Mechanics.MultiBody.Types;
            import Modelica.Mechanics.MultiBody.Frames;
            input Frames.Orientation R = Frames.nullRotation() "Orientation object to rotate the world frame into the arrow frame." annotation(Dialog);
            input SI.Position r[3] = {0,0,0} "Position vector from origin of world frame to origin of arrow frame, resolved in world frame" annotation(Dialog);
            input SI.Position r_tail[3] = {0,0,0} "Position vector from origin of arrow frame to arrow tail, resolved in arrow frame" annotation(Dialog);
            input SI.Position r_head[3] = {0,0,0} "Position vector from arrow tail to the head of the arrow, resolved in arrow frame" annotation(Dialog);
            input SI.Diameter diameter = world.defaultArrowDiameter "Diameter of arrow line" annotation(Dialog);
            input Modelica.Mechanics.MultiBody.Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.ArrowColor "Color of arrow" annotation(Dialog);
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Material property describing the reflecting of ambient light (= 0 means, that light is completely absorbed)" annotation(Dialog);
            annotation(Documentation(info = "<HTML>
<p>
Model <b>Arrow</b> defines an arrow that is dynamically
visualized at the defined location (see variables below).
</p>
<IMG SRC=\"../Images/MultiBody/Visualizers/Arrow.png\" ALT=\"model Visualizers.Advanced.Arrow\">
<p>
The variables under heading <b>Parameters</b> below
are declared as (time varying) <b>input</b> variables.
If the default equation is not appropriate, a corresponding
modifier equation has to be provided in the
model where an <b>Arrow</b> instance is used, e.g., in the form
</p>
<pre>
    Visualizers.Advanced.Arrow arrow(diameter = sin(time));
</pre>

<p>
Variable <b>color</b> is an Integer vector with 3 elements,
{r, g, b}, and specifies the color of the shape.
{r,g,b} are the \"red\", \"green\" and \"blue\" color parts.
Note, r g, b are given in the range 0 .. 255.
The predefined type <b>MultiBody.Types.Color</b> contains
a menu definition of the colors used in the MultiBody
library (will be replaced by a color editor).
</p>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,28},{20, -30}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Polygon(points = {{20,60},{100,0},{20, -60},{20,60}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{ -146,124},{142,62}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          protected 
            outer Modelica.Mechanics.MultiBody.World world;
            SI.Length length = Modelica.Math.Vectors.length(r_head) "Length of arrow";
            Visualizers.Advanced.Shape arrowLine(length = noEvent(max(0, length - diameter * Types.Defaults.ArrowHeadLengthFraction)), width = diameter, height = diameter, lengthDirection = r_head, widthDirection = {0,1,0}, shapeType = "cylinder", color = color, specularCoefficient = specularCoefficient, r_shape = r_tail, r = r, R = R) if world.enableAnimation;
            Visualizers.Advanced.Shape arrowHead(length = noEvent(max(0, min(length, diameter * Types.Defaults.ArrowHeadLengthFraction))), width = noEvent(max(0, diameter * MultiBody.Types.Defaults.ArrowHeadWidthFraction)), height = noEvent(max(0, diameter * MultiBody.Types.Defaults.ArrowHeadWidthFraction)), lengthDirection = r_head, widthDirection = {0,1,0}, shapeType = "cone", color = color, specularCoefficient = specularCoefficient, r = arrowLine.rvisobj + arrowLine.rxvisobj * arrowLine.length, R = R) if world.enableAnimation;
          end Arrow;
          model DoubleArrow "Visualizing a double arrow with variable size; all data have to be set as modifiers (see info layer)"
            import SI = Modelica.SIunits;
            import Modelica.Mechanics.MultiBody.Types;
            import Modelica.Mechanics.MultiBody.Frames;
            input Frames.Orientation R = Frames.nullRotation() "Orientation object to rotate the world frame into the arrow frame." annotation(Dialog);
            input SI.Position r[3] = {0,0,0} "Position vector from origin of world frame to origin of arrow frame, resolved in world frame" annotation(Dialog);
            input SI.Position r_tail[3] = {0,0,0} "Position vector from origin of arrow frame to double arrow tail, resolved in arrow frame" annotation(Dialog);
            input SI.Position r_head[3] = {0,0,0} "Position vector from double arrow tail to the head of the double arrow, resolved in arrow frame" annotation(Dialog);
            input SI.Diameter diameter = world.defaultArrowDiameter "Diameter of arrow line" annotation(Dialog);
            input Modelica.Mechanics.MultiBody.Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.ArrowColor "Color of double arrow" annotation(Dialog);
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Material property describing the reflecting of ambient light (= 0 means, that light is completely absorbed)" annotation(Dialog);
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,28},{0, -28}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Polygon(points = {{40,60},{100,0},{40, -60},{40,60}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{ -146,124},{142,62}}, textString = "%name", lineColor = {0,0,255}),Polygon(points = {{0,60},{60,0},{0, -60},{0,60}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<HTML>
<p>
Model <b>DoubleArrow</b> defines a double arrow that is dynamically
visualized at the defined location (see variables below).
</p>
<IMG SRC=\"../Images/MultiBody/Visualizers/DoubleArrow.png\" ALT=\"model Visualizers.Advanced.DoubleArrow\">
<p>
The variables under heading <b>Parameters</b> below
are declared as (time varying) <b>input</b> variables.
If the default equation is not appropriate, a corresponding
modifier equation has to be provided in the
model where an <b>Arrow</b> instance is used, e.g., in the form
</p>
<pre>
    Visualizers.Advanced.DoubleArrow doubleArrow(diameter = sin(time));
</pre>
<p>
Variable <b>color</b> is an Integer vector with 3 elements,
{r, g, b}, and specifies the color of the shape.
{r,g,b} are the \"red\", \"green\" and \"blue\" color parts.
Note, r g, b are given in the range 0 .. 255.
The predefined type <b>MultiBody.Types.Color</b> contains
a menu definition of the colors used in the MultiBody
library (will be replaced by a color editor).
</p>
</HTML>"));
          protected 
            outer Modelica.Mechanics.MultiBody.World world;
            SI.Length length = Modelica.Math.Vectors.length(r_head) "Length of arrow";
            SI.Length headLength = noEvent(max(0, min(length, diameter * MultiBody.Types.Defaults.ArrowHeadLengthFraction)));
            SI.Length headWidth = noEvent(max(0, diameter * MultiBody.Types.Defaults.ArrowHeadWidthFraction));
            Visualizers.Advanced.Shape arrowLine(length = noEvent(max(0, length - 1.5 * diameter * MultiBody.Types.Defaults.ArrowHeadLengthFraction)), width = diameter, height = diameter, lengthDirection = r_head, widthDirection = {0,1,0}, shapeType = "cylinder", color = color, specularCoefficient = specularCoefficient, r_shape = r_tail, r = r, R = R) if world.enableAnimation;
            Visualizers.Advanced.Shape arrowHead1(length = headLength, width = headWidth, height = headWidth, lengthDirection = r_head, widthDirection = {0,1,0}, shapeType = "cone", color = color, specularCoefficient = specularCoefficient, r = arrowLine.rvisobj + arrowLine.rxvisobj * arrowLine.length, R = R) if world.enableAnimation;
            Visualizers.Advanced.Shape arrowHead2(length = headLength, width = headWidth, height = headWidth, lengthDirection = r_head, widthDirection = {0,1,0}, shapeType = "cone", color = color, specularCoefficient = specularCoefficient, r = arrowLine.rvisobj + arrowLine.rxvisobj * (arrowLine.length + 0.5 * arrowHead1.length), R = R) if world.enableAnimation;
          end DoubleArrow;
          model Shape "Different visual shapes with variable size; all data have to be set as modifiers (see info layer)"
            extends Modelica.Utilities.Internal.PartialModelicaServices.Animation.PartialShape;
            extends ModelicaServices.Animation.Shape;
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100, -100},{80,60}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -100,60},{ -80,100},{100,100},{80,60},{ -100,60}}, lineColor = {0,0,255}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{100,100},{100, -60},{80, -100},{80,60},{100,100}}, lineColor = {0,0,255}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Text(extent = {{ -100, -100},{80,60}}, lineColor = {0,0,0}, textString = "%shapeType"),Text(extent = {{ -132,160},{128,100}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<HTML>
<p>
Model <b>Shape</b> defines a visual shape that is
shown at the location of its reference coordinate system, called
'object frame' below. All describing variables such
as size and color can vary dynamically (with the only exception
of parameter shapeType). The default equations in the
declarations should be modified by providing appropriate equations.
Model <b>Shape</b> is usually used as a basic building block to
implement simpler to use graphical components.
</p>
<p>
The following shapes are supported via
parameter <b>shapeType</b> (e.g., shapeType=\"box\"):<br>&nbsp;
</p>
<IMG SRC=\"../Images/MultiBody/Shape.png\" ALT=\"model Visualizers.FixedShape\">
<p>&nbsp;<br>
The dark blue arrows in the figure above are directed along
variable <b>lengthDirection</b>. The light blue arrows are directed
along variable <b>widthDirection</b>. The <b>coordinate systems</b>
in the figure represent frame_a of the Shape component.
</p>
<p>
Additionally, external shapes are specified as DXF-files
(only 3-dim.Face is supported). External shapes must be named \"1\", \"2\"
etc.. The corresponding definitions should be in files \"1.dxf\",
\"2.dxf\" etc.Since the DXF-files contain color and dimensions for
the individual faces, the corresponding information in the model
is currently ignored. The DXF-files must be found either in the current
directory or in the directory where the Shape instance is stored
that references the DXF file.
</p>

<p>
Via input variable <b>extra</b> additional sizing data is defined
according to:
</p>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>shapeType</b></th><th>Meaning of variable <b>extra</b></th></tr>
<tr>
  <td valign=\"top\">\"cylinder\"</td>
  <td valign=\"top\">if extra &gt; 0, a black line is included in the
      cylinder to show the rotation of it.</td>
</tr>
<tr>
  <td valign=\"top\">\"cone\"</td>
  <td valign=\"top\">extra = diameter-left-side / diameter-right-side, i.e.,<br>
      extra = 1: cylinder<br>
      extra = 0: \"real\" cone.</td>
</tr>
<tr>
  <td valign=\"top\">\"pipe\"</td>
  <td valign=\"top\">extra = outer-diameter / inner-diameter, i.e, <br>
      extra = 1: cylinder that is completely hollow<br>
      extra = 0: cylinder without a hole.</td>
</tr>
<tr>
  <td valign=\"top\">\"gearwheel\"</td>
  <td valign=\"top\">extra is the number of teeth of the gear.</td>
</tr>
<tr>
  <td valign=\"top\">\"spring\"</td>
  <td valign=\"top\">extra is the number of windings of the spring.
      Additionally, \"height\" is <b>not</b> the \"height\" but
      2*coil-width.</td>
</tr>
</table>

<p>
Parameter <b>color</b> is an Integer vector with 3 elements,
{r, g, b}, and specifies the color of the shape.
{r,g,b} are the \"red\", \"green\" and \"blue\" color parts.
Note, r g, b are given in the range 0 .. 255.
The predefined type <b>MultiBody.Types.Color</b> contains
a menu definition of the colors used in the MultiBody
library (will be replaced by a color editor).
</p>

<p>
The variables under heading <b>Parameters</b> below
are declared as (time varying) <b>input</b> variables.
If the default equation is not appropriate, a corresponding
modifier equation has to be provided in the
model where a <b>Shape</b> instance is used, e.g., in the form
</p>
<pre>
    Visualizers.Advanced.Shape shape(length = sin(time));
</pre>
</HTML>
"));
          end Shape;
        end Advanced;
        package Internal "Visualizers that will be replaced by improved versions in the future (don't use them)"
          extends Modelica.Icons.Library;
          model FixedLines "Visualizing a set of lines as cylinders (e.g., used to display characters)"
            import SI = Modelica.SIunits;
            import Modelica.Mechanics.MultiBody;
            import Modelica.Mechanics.MultiBody.Types;
            extends Modelica.Mechanics.MultiBody.Interfaces.PartialVisualizer;
            parameter Boolean animation = true "= true, if animation shall be enabled";
            input Real scale(min = 0) = 1 "The 'lines' are visualized 'scale' times bigger" annotation(Dialog(group = "if animation = true", enable = animation));
            input SI.Position lines[:,2,2] = {[0,0;1,1],[0,1;1,0]} "List of start and end points of cylinders resolved along n_x and n_y" annotation(Dialog(group = "if animation = true", enable = animation));
            input SI.Distance diameter(min = 0) = 0.05 "Diameter of the cylinders defined by lines" annotation(Dialog(group = "if animation = true", enable = animation));
            input SI.Position r_lines[3] = {0,0,0} "Position vector from origin of frame_a to the origin of the 'lines' frame, resolved in frame_a" annotation(Dialog(group = "if animation = true", enable = animation));
            input Real n_x[3](each final unit = "1") = {1,0,0} "Vector in direction of x-axis of 'lines' frame, resolved in frame_a." annotation(Dialog(group = "if animation = true", enable = animation));
            input Real n_y[3](each final unit = "1") = {0,1,0} "Vector in direction of y-axis of 'lines' frame, resolved in frame_a." annotation(Dialog(group = "if animation = true", enable = animation));
            input MultiBody.Types.Color color = {0,128,255} " Color of cylinders" annotation(Dialog(group = "if animation = true", enable = animation));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(group = "if animation = true", enable = animation));
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {128,128,128}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -140,164},{148,102}}, textString = "%name", lineColor = {0,0,255}),Polygon(points = {{ -82, -24},{ -20,46},{ -10,38},{ -72, -32},{ -82, -24}}, lineColor = {0,127,255}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -24, -34},{ -82,40},{ -72,46},{ -14, -26},{ -24, -34}}, lineColor = {0,127,255}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Polygon(points = {{42, -18},{10,40},{20,48},{50, -6},{42, -18}}, lineColor = {0,127,255}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Polygon(points = {{10, -68},{84,48},{96,42},{24, -72},{10, -68}}, lineColor = {0,127,255}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<HTML>
<p>
With model <b>FixedLines</b> a set of lines is defined
that are located relatively to frame_a. Every line
is represented by a cylinder. This allows to define simple shaped
3-dimensional characters. An example is shown in the
following figure:<br>&nbsp;
</p>
<IMG SRC=\"../Images/MultiBody/FixedLines.png\" ALT=\"model Visualizers.FixedLines\">
<p>&nbsp;<br>
The two letters \"x\" and \"y\" are constructed with 4 lines
by providing the following data for parameter <b>lines</b>
</p>
<pre>
   lines = {[0, 0; 1, 1],[0, 1; 1, 0],[1.5, -0.5; 2.5, 1],[1.5, 1; 2, 0.25]}
</pre>
<p>
Via parameter vectors <b>n_x</b> and <b>n_y</b> a two-dimensional
coordinate system is defined. The points defined with parameter
<b>lines</b> are with respect to this coordinate system. For example
\"[0, 0; 1, 1]\" defines a line that starts at {0,0} and ends at {1,1}.
The diameter and color of all line cylinders are identical.
</p>
</HTML>"));
          protected 
            Lines x_label(lines = scale * lines, diameter = scale * diameter, color = color, specularCoefficient = specularCoefficient, r_lines = r_lines, n_x = n_x, n_y = n_y, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;

          equation 
            frame_a.f = zeros(3);
            frame_a.t = zeros(3);
          end FixedLines;
          model Lines "Visualizing a set of lines as cylinders with variable size, e.g., used to display characters (no Frame connector)"
            import SI = Modelica.SIunits;
            import Modelica.Mechanics.MultiBody;
            import Modelica.Mechanics.MultiBody.Types;
            import Modelica.Mechanics.MultiBody.Frames;
            import T = Modelica.Mechanics.MultiBody.Frames.TransformationMatrices;
            input Modelica.Mechanics.MultiBody.Frames.Orientation R = Frames.nullRotation() "Orientation object to rotate the world frame into the object frame" annotation(Dialog);
            input SI.Position r[3] = {0,0,0} "Position vector from origin of world frame to origin of object frame, resolved in world frame" annotation(Dialog);
            input SI.Position r_lines[3] = {0,0,0} "Position vector from origin of object frame to the origin of 'lines' frame, resolved in object frame" annotation(Dialog);
            input Real n_x[3](each final unit = "1") = {1,0,0} "Vector in direction of x-axis of 'lines' frame, resolved in object frame" annotation(Dialog);
            input Real n_y[3](each final unit = "1") = {0,1,0} "Vector in direction of y-axis of 'lines' frame, resolved in object frame" annotation(Dialog);
            input SI.Position lines[:,2,2] = zeros(0, 2, 2) "List of start and end points of cylinders resolved in an x-y frame defined by n_x, n_y, e.g., {[0,0;1,1], [0,1;1,0], [2,0; 3,1]}" annotation(Dialog);
            input SI.Length diameter(min = 0) = 0.05 "Diameter of the cylinders defined by lines" annotation(Dialog);
            input Modelica.Mechanics.MultiBody.Types.Color color = {0,128,255} "Color of cylinders" annotation(Dialog);
            input Types.SpecularCoefficient specularCoefficient = 0.7 "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog);
          protected 
            parameter Integer n = size(lines, 1) "Number of cylinders";
            T.Orientation R_rel = T.from_nxy(n_x, n_y);
            T.Orientation R_lines = T.absoluteRotation(R.T, R_rel);
            Modelica.SIunits.Position r_abs[3] = r + T.resolve1(R.T, r_lines);
            Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape cylinders[n](each shapeType = "cylinder", lengthDirection = array(T.resolve1(R_rel, vector([lines[i,2,:] - lines[i,1,:];0])) for i in 1:n), length = array(Modelica.Math.Vectors.length(lines[i,2,:] - lines[i,1,:]) for i in 1:n), r = array(r_abs + T.resolve1(R_lines, vector([lines[i,1,:];0])) for i in 1:n), each width = diameter, each height = diameter, each widthDirection = {0,1,0}, each color = color, each R = R, each specularCoefficient = specularCoefficient);
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {128,128,128}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -24, -34},{ -82,40},{ -72,46},{ -14, -26},{ -24, -34}}, lineColor = {0,127,255}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -82, -24},{ -20,46},{ -10,38},{ -72, -32},{ -82, -24}}, lineColor = {0,127,255}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Polygon(points = {{42, -18},{10,40},{20,48},{50, -6},{42, -18}}, lineColor = {0,127,255}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Polygon(points = {{10, -68},{84,48},{96,42},{24, -72},{10, -68}}, lineColor = {0,127,255}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -140,164},{148,102}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<HTML>
<p>
With model <b>Lines</b> a set of dynamic lines is defined
that are located relatively to frame_a. Every line
is represented by a cylinder. This allows, e.g., to define simple shaped
3-dimensional characters. Note, if the lines are fixed relatively to frame_a,
it is more convenient to use model <b>Visualizers.FixedLines</b>.
An example for dynamic lines is shown in the following figure:<br>&nbsp;
</p>
<IMG SRC=\"../Images/MultiBody/FixedLines.png\" ALT=\"model Visualizers.FixedLines\">
<p>&nbsp;<br>
The two letters \"x\" and \"y\" are constructed with 4 lines
by providing the following data for input variable <b>lines</b>
</p>
<pre>
   lines = {[0, 0; 1, 1],[0, 1; 1, 0],[1.5, -0.5; 2.5, 1],[1.5, 1; 2, 0.25]}
</pre>
<p>
Via vectors <b>n_x</b> and <b>n_y</b> a two-dimensional
coordinate system is defined. The points defined with variable
<b>lines</b> are with respect to this coordinate system. For example
\"[0, 0; 1, 1]\" defines a line that starts at {0,0} and ends at {1,1}.
The diameter and color of all line cylinders are identical
and are defined by parameters.
</p>

</HTML>
"));
          end Lines;
          annotation(Documentation(info = "<html>
<p>
This package contains components to construct 3-dim. fonts
with \"cylinder\" elements for the animation window.
This is just a temporary hack until 3-dim. fonts are supported in
Modelica tools. The components are used to construct the \"x\", \"y\",
\"z\" labels of coordinates systems in the animation.
</p>
</html>"));
        end Internal;
      end Visualizers;
      package Types "Constants and types with choices, especially to build menus"
        extends Modelica.Icons.Library;
        annotation(Documentation(info = "<HTML>
<p>
In this package <b>types</b> and <b>constants</b> are defined that are used in the
MultiBody library. The types have additional annotation choices
definitions that define the menus to be built up in the graphical
user interface when the type is used as parameter in a declaration.
</p>
</HTML>"));
        type Axis = Modelica.Icons.TypeReal[3](each final unit = "1") "Axis vector with choices for menus" annotation(preferedView = "text", Evaluate = true, choices(choice = {1,0,0} "{1,0,0} \"x axis\"", choice = {0,1,0} "{0,1,0} \"y axis\"", choice = {0,0,1} "{0,0,1} \"z axis\"", choice = { -1,0,0} "{-1,0,0} \"negative x axis\"", choice = {0, -1,0} "{0,-1,0} \"negative y axis\"", choice = {0,0, -1} "{0,0,-1} \"negative z axis\""), Documentation(info = "<html>
</html>"));
        type AxisLabel = Modelica.Icons.TypeString "Label of axis with choices for menus" annotation(preferedView = "text", choices(choice = "x" "x", choice = "y" "y", choice = "z" "z"));
        type RotationSequence = Modelica.Icons.TypeInteger[3](min = {1,1,1}, max = {3,3,3}) "Sequence of planar frame rotations with choices for menus" annotation(preferedView = "text", Evaluate = true, choices(choice = {1,2,3} "{1,2,3} \"Cardan/Tait-Bryan angles\"", choice = {3,1,3} "{3,1,3} \"Euler angles\"", choice = {3,2,1} "{3,2,1}"));
        type Color = Modelica.Icons.TypeInteger[3](each min = 0, each max = 255) "RGB representation of color (will be improved with a color editor)" annotation(Dialog(colorSelector), choices(choice = {0,0,0} "{0,0,0}       \"black\"", choice = {155,0,0} "{155,0,0}     \"dark red\"", choice = {255,0,0} "{255,0,0 }    \"red\"", choice = {255,65,65} "{255,65,65}   \"light red\"", choice = {0,128,0} "{0,128,0}     \"dark green\"", choice = {0,180,0} "{0,180,0}     \"green\"", choice = {0,230,0} "{0,230,0}     \"light green\"", choice = {0,0,200} "{0,0,200}     \"dark blue\"", choice = {0,0,255} "{0,0,255}     \"blue\"", choice = {0,128,255} "{0,128,255}   \"light blue\"", choice = {255,255,0} "{255,255,0}   \"yellow\"", choice = {255,0,255} "{255,0,255}   \"pink\"", choice = {100,100,100} "{100,100,100} \"dark grey\"", choice = {155,155,155} "{155,155,155} \"grey\"", choice = {255,255,255} "{255,255,255} \"white\""), Documentation(info = "<html>
<p>
Type <b>Color</b> is an Integer vector with 3 elements,
{r, g, b}, and specifies the color of a shape.
{r,g,b} are the \"red\", \"green\" and \"blue\" color parts.
Note, r g, b are given in the range 0 .. 255.
</p>
</html>"));
        type SpecularCoefficient = Modelica.Icons.TypeReal "Reflection of ambient light (= 0: light is completely absorbed)" annotation(min = 0, choices(choice = 0 "\"0.0 (dull)\"", choice = 0.7 "\"0.7 (medium)\"", choice = 1 "\"1.0 (glossy)\""), Documentation(info = "<html>
<p>
Type <b>SpecularCoefficient</b> defines the reflection of
ambient light on shape surfaces. If value = 0, the light
is completely absorbed. Often, 0.7 is a reasonable value.
It might be that from some viewing directions, a body is no
longer visible, if the SpecularCoefficient value is too high.
In the following image, the different values of SpecularCoefficient
are shown for a cylinder:
</p>

<p>
<img src=\"../Images/MultiBody/Visualizers/SpecularCoefficient.png\"
</p>
</html>"));
        type ShapeType = Modelica.Icons.TypeString "Type of shape (box, sphere, cylinder, pipecylinder, cone, pipe, beam, gearwheel, spring, dxf-file)" annotation(choices(choice = "box" "\"box\"", choice = "sphere" "\"sphere\"", choice = "cylinder" "\"cylinder\"", choice = "pipecylinder" "\"pipecylinder\"", choice = "cone" "\"cone\"", choice = "pipe" "\"pipe\"", choice = "beam" "\"beam\"", choice = "gearwheel" "\"gearwheel\"", choice = "spring" "\"spring\"", choice = "1" "File \"1.dxf\" in current directory", choice = "2" "File \"2.dxf\" in current directory", choice = "3" "File \"3.dxf\" in current directory", choice = "4" "File \"4.dxf\" in current directory", choice = "5" "File \"5.dxf\" in current directory", choice = "6" "File \"6.dxf\" in current directory", choice = "7" "File \"7.dxf\" in current directory", choice = "8" "File \"8.dxf\" in current directory", choice = "9" "File \"9.dxf\" in current directory"), Documentation(info = "<html>
<p>
Type <b>ShapeType</b> is used to define the shape of the
visual object as parameter String. Usually, \"shapeType\" is used
as instance name. The following
values for shapeType are possible, e.g., shapeType=\"box\":
</p>
<IMG SRC=\"../Images/MultiBody/Shape.png\" ALT=\"model Visualizers.FixedShape\">
<p>&nbsp;<br>
The dark blue arrows in the figure above are directed along
variable <b>lengthDirection</b>. The light blue arrows are directed
along variable <b>widthDirection</b>. The <b>coordinate systems</b>
in the figure represent frame_a of the Shape component.
</p>
<p>
Additionally, external shapes are specified as DXF-files
(only 3-dim.Face is supported). External shapes must be named \"1\", \"2\"
etc.. The corresponding definitions should be in files \"1.dxf\",
\"2.dxf\" etc.Since the DXF-files contain color and dimensions for
the individual faces, the corresponding information in the model
is currently ignored. The DXF-files must be found either in the current
directory or in the directory where the Shape instance is stored
that references the DXF file.
</p>
</html>"));
        type ShapeExtra = Modelica.Icons.TypeReal "Reflection of ambient light (= 0: light is completely absorbed)" annotation(min = 0, Documentation(info = "<html>
<p>
This type is used in shapes of visual objects to define
extra data depending on the shape type. Usually, input
variable <b>extra</b> is used as instance name:
</p>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>shapeType</b></th><th>Meaning of variable <b>extra</b></th></tr>
<tr>
  <td valign=\"top\">\"cylinder\"</td>
  <td valign=\"top\">if extra &gt; 0, a black line is included in the
      cylinder to show the rotation of it.</td>
</tr>
<tr>
  <td valign=\"top\">\"cone\"</td>
  <td valign=\"top\">extra = diameter-left-side / diameter-right-side, i.e.,<br>
      extra = 1: cylinder<br>
      extra = 0: \"real\" cone.</td>
</tr>
<tr>
  <td valign=\"top\">\"pipe\"</td>
  <td valign=\"top\">extra = outer-diameter / inner-diameter, i.e, <br>
      extra = 1: cylinder that is completely hollow<br>
      extra = 0: cylinder without a hole.</td>
</tr>
<tr>
  <td valign=\"top\">\"gearwheel\"</td>
  <td valign=\"top\">extra is the number of teeth of the gear.</td>
</tr>
<tr>
  <td valign=\"top\">\"spring\"</td>
  <td valign=\"top\">extra is the number of windings of the spring.
      Additionally, \"height\" is <b>not</b> the \"height\" but
      2*coil-width.</td>
</tr>
</table>
</html>"));
        type ResolveInFrameA = enumeration(world "Resolve in world frame", frame_a  "Resolve in frame_a", frame_resolve  "Resolve in frame_resolve (frame_resolve must be connected)") "Enumeration to define the frame in which an absolute vector is resolved (world, frame_a, frame_resolve)" annotation(Documentation(info = "<html>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameA.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vector in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vector in frame_a</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vector in frame_resolve (frame_resolve must be connected)</td></tr>
</table>
</html>"));
        type ResolveInFrameB = enumeration(world "Resolve in world frame", frame_b  "Resolve in frame_b", frame_resolve  "Resolve in frame_resolve (frame_resolve must be connected)") "Enumeration to define the frame in which an absolute vector is resolved (world, frame_b, frame_resolve)" annotation(Documentation(info = "<html>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vector in world frame</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve vector in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vector in frame_resolve (frame_resolve must be connected)</td></tr>
</table>
</html>"));
        type ResolveInFrameAB = enumeration(world "Resolve in world frame", frame_a "Resolve in frame_a", frame_b  "Resolve in frame_b", frame_resolve  "Resolve in frame_resolve (frame_resolve must be connected)") "Enumeration to define the frame in which a relative vector is resolved (world, frame_a, frame_b, frame_resolve)" annotation(Documentation(info = "<html>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vector in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vector in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve vector in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vector in frame_resolve (frame_resolve must be connected)</td></tr>
</table>
</html>"));
        type RotationTypes = enumeration(RotationAxis "Rotating frame_a around an angle with a fixed axis", TwoAxesVectors  "Resolve two vectors of frame_b in frame_a", PlanarRotationSequence  "Planar rotation sequence") "Enumeration defining in which way the fixed orientation of frame_b with respect to frame_a is specified" annotation(Evaluate = true, Documentation(info = "<html>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.RotationTypes.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">RotationAxis</td>
    <td valign=\"top\">frame_b is defined by rotating the coordinate system along
        an axis fixed in frame_a and with a fixed angle.</td></tr>

<tr><td valign=\"top\">TwoAxesVectors</td>
    <td valign=\"top\">frame_b is defined by resolving two vectors of frame_b in frame_a.</td></tr>

<tr><td valign=\"top\">PlanarRotationSequence</td>
    <td valign=\"top\">frame_b is defined by rotating the coordinate system along
        3 consecutive axes vectors with fixed rotation angles
        (e.g. Cardan or Euler angle sequence rotation).</td></tr>
</table>
</html>"));
        type GravityTypes = enumeration(NoGravity "No gravity field", UniformGravity  "Uniform gravity field", PointGravity  "Point gravity field") "Enumeration defining the type of the gravity field" annotation(Documentation(info = "<html>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.GravityTypes.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">NoGravity</td>
    <td valign=\"top\">No gravity field</td></tr>

<tr><td valign=\"top\">UniformGravity</td>
    <td valign=\"top\">Gravity field is described by a vector of constant gravity acceleration</td></tr>

<tr><td valign=\"top\">PointGravity</td>
    <td valign=\"top\">Central gravity field. The gravity acceleration vector is directed to
        the field center and the gravity is proportional to 1/r^2, where
        r is the distance to the field center.</td></tr>
</table>
</html>"));
        type Init = enumeration(Free, PositionVelocity, SteadyState, Position, Velocity, VelocityAcceleration , PositionVelocityAcceleration );
        package Defaults "Default settings of the MultiBody library via constants"
          annotation(Documentation(info = "<html>
<p>
This package contains constants used as default setting
in the MultiBody library.
</p>
</html>"));
          extends Modelica.Icons.Library;
          constant Types.Color BodyColor = {0,128,255} "Default color for body shapes that have mass (light blue)";
          constant Types.Color RodColor = {155,155,155} "Default color for massless rod shapes (grey)";
          constant Types.Color JointColor = {255,0,0} "Default color for elementary joints (red)";
          constant Types.Color ForceColor = {0,128,0} "Default color for force arrow (dark green)";
          constant Types.Color TorqueColor = {0,128,0} "Default color for torque arrow (dark green)";
          constant Types.Color SpringColor = {0,0,255} "Default color for a spring (blue)";
          constant Types.Color SensorColor = {255,255,0} "Default color for sensors (yellow)";
          constant Types.Color FrameColor = {0,0,0} "Default color for frame axes and labels (black)";
          constant Types.Color ArrowColor = {0,0,255} "Default color for arrows and double arrows (blue)";
          constant Real FrameHeadLengthFraction = 5.0 "Frame arrow head length / arrow diameter";
          constant Real FrameHeadWidthFraction = 3.0 "Frame arrow head width / arrow diameter";
          constant Real FrameLabelHeightFraction = 3.0 "Height of frame label / arrow diameter";
          constant Real ArrowHeadLengthFraction = 4.0 "Arrow head length / arrow diameter";
          constant Real ArrowHeadWidthFraction = 3.0 "Arrow head width / arrow diameter";
          constant SI.Diameter BodyCylinderDiameterFraction = 3 "Default for body cylinder diameter as a fraction of body sphere diameter";
          constant Real JointRodDiameterFraction = 2 "Default for rod diameter as a fraction of joint sphere diameter attached to rod";
        end Defaults;
      end Types;
      package Sensors "Sensors to measure variables"
        extends Modelica.Icons.Library;
        model AbsoluteSensor "Measure absolute kinematic quantities of frame connector"
          import SI = Modelica.SIunits;
          Blocks.Interfaces.RealOutput r[3](each final quantity = "Position", each final unit = "m") if get_r "Absolute position vector frame_a.r_0 resolved in frame defined by resolveInFrame" annotation(Placement(transformation(origin = { -100, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Blocks.Interfaces.RealOutput v[3](each final quantity = "Velocity", each final unit = "m/s") if get_v "Absolute velocity vector" annotation(Placement(transformation(origin = { -60, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Blocks.Interfaces.RealOutput a[3](each final quantity = "Acceleration", each final unit = "m/s2") if get_a "Absolute acceleration vector" annotation(Placement(transformation(origin = { -20, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Blocks.Interfaces.RealOutput angles[3](each final quantity = "Angles", each final unit = "rad", each displayUnit = "deg") if get_angles "Angles to rotate world frame into frame_a via 'sequence'" annotation(Placement(transformation(origin = {20, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Blocks.Interfaces.RealOutput w[3](each final quantity = "AngularVelocity", each final unit = "1/s") if get_w "Absolute angular velocity vector" annotation(Placement(transformation(origin = {60, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Blocks.Interfaces.RealOutput z[3](each final quantity = "AngularAcceleration", each final unit = "1/s2") if get_z "Absolute angular acceleration vector" annotation(Placement(transformation(origin = {100, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          extends Modelica.Mechanics.MultiBody.Sensors.Internal.PartialAbsoluteSensor;
          Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve "If resolveInFrame = Types.ResolveInFrameA.frame_resolve, the output signals are resolved in this frame" annotation(Placement(transformation(extent = {{84, -16},{116,16}}), iconTransformation(extent = {{84, -16},{116,16}})));
          parameter Boolean animation = true "= true, if animation shall be enabled (show arrow)";
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameA resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_a "Frame in which vectors are resolved (1: world, 2: frame_a, 3: frame_resolve)";
          parameter Boolean get_r = false "= true, to measure the absolute position vector of the origin of frame_a" annotation(HideResult = true, choices(__Dymola_checkBox = true));
          parameter Boolean get_v = false "= true, to measure the absolute velocity of the origin of frame_a" annotation(HideResult = true, choices(__Dymola_checkBox = true));
          parameter Boolean get_a = false "= true, to measure the absolute acceleration of the origin of frame_a" annotation(HideResult = true, choices(__Dymola_checkBox = true));
          parameter Boolean get_w = false "= true, to measure the absolute angular velocity of frame_a" annotation(HideResult = true, choices(__Dymola_checkBox = true));
          parameter Boolean get_z = false "= true, to measure the absolute angular acceleration of frame_a" annotation(HideResult = true, choices(__Dymola_checkBox = true));
          parameter Boolean get_angles = false "= true, to measure the 3 rotation angles" annotation(HideResult = true, choices(__Dymola_checkBox = true), Dialog(group = "3 angles to rotate the world frame into frame_a along the axes defined in \"sequence\""));
          parameter Types.RotationSequence sequence(min = {1,1,1}, max = {3,3,3}) = {1,2,3} "If get_angles=true: Angles are returned to rotate world frame around axes sequence[1], sequence[2] and finally sequence[3] into frame_a" annotation(HideResult = true, Evaluate = true, Dialog(group = "3 angles to rotate the world frame into frame_a along the axes defined in \"sequence\"", enable = get_angles));
          parameter SI.Angle guessAngle1 = 0 "If get_angles=true: Select angles[1] such that abs(angles[1] - guessAngle1) is a minimum" annotation(HideResult = true, Dialog(group = "3 angles to rotate the world frame into frame_a along the axes defined in \"sequence\"", enable = get_angles));
          input SI.Diameter arrowDiameter = world.defaultArrowDiameter "Diameter of absolute arrow from world frame to frame_a" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color arrowColor = Modelica.Mechanics.MultiBody.Types.Defaults.SensorColor "Color of absolute arrow from world frame to frame_b" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(visible = get_r, points = {{ -84,0},{ -84, -60},{ -100, -60},{ -100, -100}}, color = {0,0,127}, smooth = Smooth.None),Line(visible = get_a, points = {{ -20, -67},{ -20, -100}}, color = {0,0,127}, smooth = Smooth.None),Line(visible = get_w, points = {{60, -36},{60, -100}}, color = {0,0,127}, smooth = Smooth.None),Line(visible = get_z, points = {{86,0},{86, -60},{100, -60},{100, -100}}, color = {0,0,127}, smooth = Smooth.None),Line(visible = get_v, points = {{ -60, -36},{ -60, -100}}, color = {0,0,127}, smooth = Smooth.None),Line(visible = get_angles, points = {{20, -67},{20, -100}}, color = {0,0,127}, smooth = Smooth.None),Line(points = {{95,0},{95,0},{70,0}}, color = {0,0,0}, pattern = LinePattern.Dot, smooth = Smooth.None),Text(extent = {{ -132,76},{129,124}}, textString = "%name", lineColor = {0,0,255}),Text(visible = get_r, extent = {{ -130, -74},{ -95, -90}}, lineColor = {0,0,0}, textString = "r"),Text(visible = get_v, extent = {{ -95, -74},{ -60, -90}}, lineColor = {0,0,0}, textString = "v"),Text(visible = get_a, extent = {{ -55, -74},{ -20, -90}}, lineColor = {0,0,0}, textString = "a"),Text(visible = get_angles, extent = {{ -71, -35},{96, -54}}, lineColor = {0,0,0}, textString = "angles"),Text(visible = get_w, extent = {{59, -68},{105, -85}}, lineColor = {0,0,0}, textString = "w"),Text(visible = get_z, extent = {{107, -68},{153, -84}}, lineColor = {0,0,0}, textString = "z"),Text(extent = {{60,52},{191,27}}, lineColor = {95,95,95}, textString = "resolve")}), Documentation(info = "<html>
<p>
Absolute kinematic quantities of frame_a are
determined and provided at the conditional output signal connectors.
For example, if parameter \"get_r = <b>true</b>\", the connector
\"r\" is enabled and contains the absolute vector from the world frame
to the origin of frame_a. The following quantities can be provided
as output signals:
</p>

<ol>
<li> Absolute position vector (= r) </li>
<li> Absolute velocity vector (= v)</li>
<li> Absolute acceleration vector (= a)</li>
<li> Three angles to rotate world frame into frame_a (= angles)</li>
<li> Absolute angular velocity vector (= w)</li>
<li> Absolute angular acceleration vector (= z)</li>
</ol>

<p>
Via parameter <b>resolveInFrame</b> it is defined, in which frame
a vector is resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>resolveInFrame =<br>Types.ResolveInFrameA.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vectors in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vectors in frame_a</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vectors in frame_resolve</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameA.frame_resolve, the conditional connector
\"frame_resolve\" is enabled and the vectors are resolved in the frame, to
which frame_resolve is connected. Note, if this connector is enabled, it must
be connected.
</p>

<p>
In the following figure the animation of an AbsoluteSensor
component is shown. The light blue coordinate system is
frame_a and the yellow arrow is the animated sensor.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Sensors/AbsoluteSensor.png\">
</p>

<p>
Velocity, acceleration, angular velocity and angular acceleration are
determined by differentiating them in the world frame and then transforming
them in to the frame defined by <b>resolveInFrame</b>.
</p>
<p>
For example, if resolveInFrame = <b>Types.ResolveInFrameA.frame_a</b>, then
</p>
<pre>
   v0 = <b>der</b>(frame_a.r0);
   v  = resolve2(frame_a.R, v0);
</pre>
is returned, i.e., the derivative of the absolute distance from the
world frame to the origin of frame_a, resolved in frame_a.
</p>

<p>
The cut-force and the cut-torque in frame_resolve are
always zero, whether frame_resolve is connected or not.
</p>

<p>
If <b>get_angles</b> = <b>true</b>, the 3 angles to rotate the world
frame into frame_a along the axes defined by parameter <b>sequence</b>
are returned. For example, if sequence = {3,1,2} then the world frame is
rotated around angles[1] along the z-axis, afterwards it is rotated
around angles[2] along the x-axis, and finally it is rotated around
angles[3] along the y-axis and is then identical to frame_a.
The 3 angles are returned in the range
</p>
<pre>
    -<font face=\"Symbol\">p</font> &lt;= angles[i] &lt;= <font face=\"Symbol\">p</font>
</pre>
<p>
There are <b>two solutions</b> for \"angles[1]\" in this range.
Via parameter <b>guessAngle1</b> (default = 0) the
returned solution is selected such that |angles[1] - guessAngle1| is
minimal. The absolute transformation matrix of frame_a
may be in a singular configuration with respect to \"sequence\", i.e.,
there is an infinite number of angle values leading to the same absolute
transformation matrix. In this case, the returned solution is
selected by setting angles[1] = guessAngle1. Then angles[2]
and angles[3] can be uniquely determined in the above range.
</p>
<p>
The parameter <b>sequence</b> has the restriction that
only values 1,2,3 can be used and that sequence[1] &ne; sequence[2]
and sequence[2] &ne; sequence[3]. Often used values are:
</p>
<pre>
  sequence = <b>{1,2,3}</b>  // Cardan or Tait-Bryan angle sequence
           = <b>{3,1,3}</b>  // Euler angle sequence
           = <b>{3,2,1}</b>
</pre>
</html>"));
        protected 
          AbsolutePosition position(resolveInFrame = resolveInFrame) if get_r annotation(Placement(transformation(extent = {{10,10},{ -10,30}}, rotation = 90, origin = { -80, -60})));
        protected 
          AbsoluteVelocity velocity(resolveInFrame = resolveInFrame) if get_v annotation(Placement(transformation(extent = {{10, -10},{ -10,10}}, rotation = 90, origin = { -60, -60})));
          Modelica.Mechanics.MultiBody.Sensors.AbsoluteAngles absoluteAngles(sequence = sequence, guessAngle1 = guessAngle1) if get_angles annotation(Placement(transformation(extent = {{ -10,10},{10, -10}}, rotation =  -90, origin = {20, -60})));
          AbsoluteAngularVelocity angularVelocity(resolveInFrame = resolveInFrame) if get_w annotation(Placement(transformation(extent = {{ -10, -10},{10, -30}}, rotation =  -90, origin = {70, -60})));
        protected 
          Blocks.Continuous.Der der1[3] if get_a annotation(Placement(transformation(extent = {{ -6, -6},{6,6}}, rotation =  -90, origin = { -20, -34})));
        protected 
          Blocks.Continuous.Der der2[3] if get_z annotation(Placement(transformation(extent = {{ -6, -6},{6,6}}, rotation =  -90, origin = {100, -46})));
        protected 
          Modelica.Mechanics.MultiBody.Sensors.TansformAbsoluteVector transformVector_a(frame_r_in = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.world, frame_r_out = resolveInFrame) if get_a annotation(Placement(transformation(extent = {{ -10, -70},{ -30, -50}})));
          Modelica.Mechanics.MultiBody.Sensors.TansformAbsoluteVector transformVector_z(frame_r_in = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.world, frame_r_out = resolveInFrame) if get_z annotation(Placement(transformation(extent = {{90, -80},{110, -60}})));
        protected 
          outer Modelica.Mechanics.MultiBody.World world;
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Arrow arrow(r_head = frame_a.r_0, diameter = arrowDiameter, color = arrowColor, specularCoefficient) if world.enableAnimation and animation;
        protected 
          AbsoluteVelocity absoluteVelocity(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.world) if get_a annotation(Placement(transformation(extent = {{10, -10},{ -10,10}}, rotation = 90, origin = { -20, -12})));
          AbsoluteAngularVelocity absoluteAngularVelocity(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.world) if get_z annotation(Placement(transformation(extent = {{40, -10},{60,10}}, rotation = 0)));
          Internal.ZeroForceAndTorque zeroForce1 annotation(Placement(transformation(extent = {{ -60,20},{ -40,40}})));
          Internal.ZeroForceAndTorque zeroForce2 if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve annotation(Placement(transformation(extent = {{80,20},{60,40}})));

        equation 
          connect(zeroForce1.frame_a,frame_a) annotation(Line(points = {{ -60,30},{ -80,30},{ -80,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(absoluteAngles.angles,angles) annotation(Line(points = {{20, -71},{20, -90},{20, -90},{20, -110}}, color = {0,0,127}, smooth = Smooth.None));
          connect(angularVelocity.frame_a,frame_a) annotation(Line(points = {{50, -50},{50, -40},{20, -40},{20,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(angularVelocity.w,w) annotation(Line(points = {{50, -71},{50, -94},{60, -94},{60, -110}}, color = {0,0,127}, smooth = Smooth.None));
          connect(frame_resolve,position.frame_resolve) annotation(Line(points = {{100,0},{114,0},{114, -90},{ -80, -90},{ -80, -60},{ -89.9, -60}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(frame_resolve,zeroForce2.frame_a) annotation(Line(points = {{100,0},{90,0},{90,30},{80,30}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(angularVelocity.frame_resolve,frame_resolve) annotation(Line(points = {{60.1, -60},{66, -60},{66, -90},{114, -90},{114,0},{100,0}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(transformVector_a.frame_a,frame_a) annotation(Line(points = {{ -10, -60},{0, -60},{0, -40},{20, -40},{20,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(transformVector_a.frame_resolve,frame_resolve) annotation(Line(points = {{ -30, -59.9},{ -40, -59.9},{ -40, -90},{114, -90},{114,0},{100,0}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(transformVector_a.r_out,a) annotation(Line(points = {{ -20, -71},{ -20, -110}}, color = {0,0,127}, smooth = Smooth.None));
          connect(der2.y,transformVector_z.r_in) annotation(Line(points = {{100, -52.6},{100, -58}}, color = {0,0,127}, smooth = Smooth.None));
          connect(transformVector_z.r_out,z) annotation(Line(points = {{100, -81},{100, -110}}, color = {0,0,127}, smooth = Smooth.None));
          connect(transformVector_z.frame_a,frame_a) annotation(Line(points = {{90, -70},{80, -70},{80, -40},{20, -40},{20,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(transformVector_z.frame_resolve,frame_resolve) annotation(Line(points = {{110, -69.9},{110, -70},{114, -70},{114,0},{100,0}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(frame_a,position.frame_a) annotation(Line(points = {{ -100,0},{ -80,0},{ -80, -40},{ -100, -40},{ -100, -50}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(absoluteAngles.frame_a,frame_a) annotation(Line(points = {{20, -50},{20,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(position.r,r) annotation(Line(points = {{ -100, -71},{ -100, -110}}, color = {0,0,127}, smooth = Smooth.None));
          connect(velocity.frame_a,frame_a) annotation(Line(points = {{ -60, -50},{ -60, -40},{ -80, -40},{ -80,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(velocity.frame_resolve,frame_resolve) annotation(Line(points = {{ -50, -60},{ -40, -60},{ -40, -90},{114, -90},{114,0},{100,0}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(velocity.v,v) annotation(Line(points = {{ -60, -71},{ -60, -110}}, color = {0,0,127}, smooth = Smooth.None));
          connect(der1.y,transformVector_a.r_in) annotation(Line(points = {{ -20, -40.6},{ -20, -48}}, color = {0,0,127}, smooth = Smooth.None));
          connect(absoluteVelocity.v,der1.u) annotation(Line(points = {{ -20, -23},{ -20, -26.8}}, color = {0,0,127}, smooth = Smooth.None));
          connect(absoluteVelocity.frame_a,frame_a) annotation(Line(points = {{ -20, -2},{ -20,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(absoluteAngularVelocity.frame_a,frame_a) annotation(Line(points = {{40,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(absoluteAngularVelocity.w,der2.u) annotation(Line(points = {{61,0},{80,0},{80, -30},{100, -30},{100, -38.8}}, color = {0,0,127}, smooth = Smooth.None));
        end AbsoluteSensor;
        model RelativeSensor "Measure relative kinematic quantities between two frame connectors"
          import SI = Modelica.SIunits;
          extends Modelica.Mechanics.MultiBody.Sensors.Internal.PartialRelativeSensor;
          Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve or resolveInFrameAfterDifferentiation == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve "If resolveInFrame = Types.ResolveInFrameAB.frame_resolve, the output signals are resolved in this frame" annotation(Placement(transformation(extent = {{84,64},{116,96}}), iconTransformation(extent = {{84,64},{116,96}})));
          parameter Boolean animation = true "= true, if animation shall be enabled (show arrow)";
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a "Frame in which vectors are resolved before differentiation (1: world, 2: frame_a, 3: frame_b, 4: frame_resolve)";
          parameter Boolean get_r_rel = false "= true, to measure the relative position vector from the origin of frame_a to frame_b" annotation(HideResult = true, choices(__Dymola_checkBox = true));
          parameter Boolean get_v_rel = false "= true, to measure the relative velocity of the origin of frame_b with respect to frame_a" annotation(HideResult = true, choices(__Dymola_checkBox = true));
          parameter Boolean get_a_rel = false "= true, to measure the relative acceleration of the origin of frame_b with respect to frame_a" annotation(HideResult = true, choices(__Dymola_checkBox = true));
          parameter Boolean get_w_rel = false "= true, to measure the relative angular velocity of frame_b with respect to frame_a" annotation(HideResult = true, choices(__Dymola_checkBox = true));
          parameter Boolean get_z_rel = false "= true, to measure the relative angular acceleration of frame_b with respect to frame_a" annotation(HideResult = true, choices(__Dymola_checkBox = true));
          parameter Boolean get_angles = false "= true, to measure the 3 rotation angles" annotation(HideResult = true, choices(__Dymola_checkBox = true), Dialog(group = "3 angles to rotate frame_a into frame_b along the axes defined in \"sequence\""));
          parameter Types.RotationSequence sequence(min = {1,1,1}, max = {3,3,3}) = {1,2,3} "If get_angles=true: Angles are returned to rotate frame_a around axes sequence[1], sequence[2] and finally sequence[3] into frame_b" annotation(HideResult = true, Evaluate = true, Dialog(group = "3 angles to rotate frame_a into frame_b along the axes defined in \"sequence\"", enable = get_angles));
          parameter SI.Angle guessAngle1 = 0 "If get_angles=true: Select angles[1] such that abs(angles[1] - guessAngle1) is a minimum" annotation(HideResult = true, Dialog(group = "3 angles to rotate frame_a into frame_b along the axes defined in \"sequence\"", enable = get_angles));
          input SI.Diameter arrowDiameter = world.defaultArrowDiameter "Diameter of relative arrow from frame_a to frame_b" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color arrowColor = Modelica.Mechanics.MultiBody.Types.Defaults.SensorColor "Color of relative arrow from frame_a to frame_b" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB resolveInFrameAfterDifferentiation = resolveInFrame "Frame in which vectors are resolved after differentiation (1: world, 2: frame_a, 3: frame_b, 4: frame_resolve)" annotation(Dialog(tab = "Advanced", group = "if get_v_rel or get_a_rel or get_z_rel", enable = get_v_rel or get_a_rel or get_z_rel));
          Blocks.Interfaces.RealOutput r_rel[3](each final quantity = "Position", each final unit = "m") if get_r_rel "Relative position vector frame_b.r_0 - frame_a.r_0 resolved in frame defined by resolveInFrame" annotation(Placement(transformation(origin = { -100, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Blocks.Interfaces.RealOutput v_rel[3](each final quantity = "Velocity", each final unit = "m/s") if get_v_rel "Relative velocity vector" annotation(Placement(transformation(origin = { -60, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Blocks.Interfaces.RealOutput a_rel[3](each final quantity = "Acceleration", each final unit = "m/s2") if get_a_rel "Relative acceleration vector" annotation(Placement(transformation(origin = { -20, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Blocks.Interfaces.RealOutput angles[3](each final quantity = "Angles", each final unit = "rad", each displayUnit = "deg") if get_angles "Angles to rotate frame_a into frame_b via 'sequence'" annotation(Placement(transformation(origin = {20, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Blocks.Interfaces.RealOutput w_rel[3](each final quantity = "AngularVelocity", each final unit = "1/s") if get_w_rel "Relative angular velocity vector" annotation(Placement(transformation(origin = {60, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Blocks.Interfaces.RealOutput z_rel[3](each final quantity = "AngularAcceleration", each final unit = "1/s2") if get_z_rel "Relative angular acceleration vector" annotation(Placement(transformation(origin = {100, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(visible = get_r_rel, points = {{ -84,0},{ -84, -60},{ -100, -60},{ -100, -100}}, color = {0,0,127}, smooth = Smooth.None),Line(visible = get_a_rel, points = {{ -20, -67},{ -20, -100}}, color = {0,0,127}, smooth = Smooth.None),Line(visible = get_w_rel, points = {{60, -36},{60, -100}}, color = {0,0,127}, smooth = Smooth.None),Line(visible = get_z_rel, points = {{86,0},{86, -60},{100, -60},{100, -100}}, color = {0,0,127}, smooth = Smooth.None),Line(visible = get_v_rel, points = {{ -60, -36},{ -60, -100}}, color = {0,0,127}, smooth = Smooth.None),Line(visible = get_angles, points = {{20, -67},{20, -100}}, color = {0,0,127}, smooth = Smooth.None),Text(extent = {{ -132,90},{129,138}}, textString = "%name", lineColor = {0,0,255}),Text(visible = get_r_rel, extent = {{ -130, -74},{ -95, -90}}, lineColor = {0,0,0}, textString = "r"),Text(visible = get_v_rel, extent = {{ -95, -74},{ -60, -90}}, lineColor = {0,0,0}, textString = "v"),Text(visible = get_a_rel, extent = {{ -55, -74},{ -20, -90}}, lineColor = {0,0,0}, textString = "a"),Text(visible = get_angles, extent = {{ -71, -35},{96, -54}}, lineColor = {0,0,0}, textString = "angles"),Text(visible = get_w_rel, extent = {{63, -73},{103, -90}}, lineColor = {0,0,0}, textString = "w"),Text(visible = get_z_rel, extent = {{103, -71},{149, -87}}, lineColor = {0,0,0}, textString = "z")}), Documentation(info = "<html>
<p>
Relative kinematic quantities between frame_a and frame_b are
determined and provided at the conditional output signal connectors.
For example, if parameter \"get_r_rel = <b>true</b>\", the connector
\"r_rel\" is enabled and contains the relative vector from
frame_a to frame_b. The following quantities can be provided
as output signals:
</p>

<ol>
<li> Relative position vector (= r_rel) </li>
<li> Relative velocity vector (= v_rel)</li>
<li> Relative acceleration vector (= a_rel)</li>
<li> Three angles to rotate frame_a into frame_b (= angles)</li>
<li> Relative angular velocity vector (= w_rel)</li>
<li> Relative angular acceleration vector (= z_rel)</li>
</ol>

<p>
Via parameter <b>resolveInFrame</b> it is defined, in which frame
a vector is resolved (before differentiation):
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>resolveInFrame =<br>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vectors in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vectors in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve vectors in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vectors in frame_resolve</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameAB.frame_resolve, the conditional connector
\"frame_resolve\" is enabled and the vectors are resolved in the frame, to
which frame_resolve is connected. Note, if this connector is enabled, it must
be connected.
</p>

<p>
In the following figure the animation of a RelativeSensor
component is shown. The light blue coordinate system is
frame_a, the dark blue coordinate system is frame_b, and
the yellow arrow is the animated sensor.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Sensors/RelativeSensor.png\">
</p>
<p>
Note, derivatives
of relative kinematic quantities are always performed with
respect to the frame, in which the vector to be differentiated
is resolved. After differentiation, it is possible via parameter
<b>resolveInFrameAfterDifferentiation</b> (in the \"Advanced\" menu)
to resolve the differentiated
vector in another frame.
</p>
<p>
For example, if resolveInFrame = <b>Types.ResolveInFrameAB.frame_b</b>, then
</p>
<pre>
   r_rel = resolve2(frame_b.R, frame_b.r_0 - frame_a.r0);
   v_rel = <b>der</b>(r_rel);
</pre>
is returned (r_rel = resolve2(frame_b.R, frame_b.r_0 - frame_a.r0)), i.e.,
the derivative of the relative distance from frame_a to frame_b,
resolved in frame_b. If
<b>resolveInFrameAfterDifferentiation</b> = Types.ResolveInFrameAB.world, then
v_rel is additionally transformed to:
</p>
<pre>
   v_rel = resolve1(frame_b.R, <b>der</b>(r_rel))
</pre>


<p>
The cut-force and the cut-torque in frame_resolve are
always zero, whether frame_resolve is connected or not.
</p>

<p>
If <b>get_angles</b> = <b>true</b>, the 3 angles to rotate frame_a
into frame_b along the axes defined by parameter <b>sequence</b>
are returned. For example, if sequence = {3,1,2} then frame_a is
rotated around angles[1] along the z-axis, afterwards it is rotated
around angles[2] along the x-axis, and finally it is rotated around
angles[3] along the y-axis and is then identical to frame_b.
The 3 angles are returned in the range
</p>
<pre>
    -<font face=\"Symbol\">p</font> &lt;= angles[i] &lt;= <font face=\"Symbol\">p</font>
</pre>
<p>
There are <b>two solutions</b> for \"angles[1]\" in this range.
Via parameter <b>guessAngle1</b> (default = 0) the
returned solution is selected such that |angles[1] - guessAngle1| is
minimal. The relative transformation matrix between frame_a and
frame_b may be in a singular configuration with respect to \"sequence\", i.e.,
there is an infinite number of angle values leading to the same relative
transformation matrix. In this case, the returned solution is
selected by setting angles[1] = guessAngle1. Then angles[2]
and angles[3] can be uniquely determined in the above range.
</p>
<p>
The parameter <b>sequence</b> has the restriction that
only values 1,2,3 can be used and that sequence[1] &ne; sequence[2]
and sequence[2] &ne; sequence[3]. Often used values are:
</p>
<pre>
  sequence = <b>{1,2,3}</b>  // Cardan or Tait-Bryan angle sequence
           = <b>{3,1,3}</b>  // Euler angle sequence
           = <b>{3,2,1}</b>
</pre>
</html>"));
        protected 
          RelativePosition relativePosition(resolveInFrame = resolveInFrame) if get_r_rel or get_v_rel or get_a_rel annotation(Placement(transformation(extent = {{ -80, -10},{ -60,10}})));
        protected 
          Blocks.Continuous.Der der1[3] if get_v_rel or get_a_rel annotation(Placement(transformation(extent = {{ -10, -10},{0,0}}, rotation =  -90, origin = { -55, -30})));
          Blocks.Continuous.Der der2[3] if get_a_rel annotation(Placement(transformation(extent = {{0,0},{10,10}}, rotation =  -90, origin = { -25, -40})));
          Modelica.Mechanics.MultiBody.Sensors.RelativeAngles relativeAngles(sequence = sequence, guessAngle1 = guessAngle1) if get_angles annotation(Placement(transformation(extent = {{10, -25},{30, -5}})));
          RelativeAngularVelocity relativeAngularVelocity(resolveInFrame = resolveInFrame) if get_w_rel or get_z_rel annotation(Placement(transformation(extent = {{50, -40},{70, -20}})));
        protected 
          Blocks.Continuous.Der der3[3] if get_z_rel annotation(Placement(transformation(extent = {{ -10, -10},{0,0}}, rotation =  -90, origin = {105, -60})));
          Internal.ZeroForceAndTorque zeroForce1 annotation(Placement(transformation(extent = {{ -81,40},{ -61,60}})));
          Internal.ZeroForceAndTorque zeroForce2 annotation(Placement(transformation(extent = {{70,20},{50,40}})));
          Internal.ZeroForceAndTorque zeroForce3 if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve annotation(Placement(transformation(extent = {{70,50},{50,70}})));
        protected 
          Modelica.Mechanics.MultiBody.Sensors.TansformRelativeVector transformVector_v_rel(frame_r_in = resolveInFrame, frame_r_out = resolveInFrameAfterDifferentiation) if get_v_rel annotation(Placement(transformation(extent = {{ -70, -64},{ -50, -44}})));
          Modelica.Mechanics.MultiBody.Sensors.TansformRelativeVector transformVector_a_rel(frame_r_in = resolveInFrame, frame_r_out = resolveInFrameAfterDifferentiation) if get_a_rel annotation(Placement(transformation(extent = {{ -30, -80},{ -10, -60}})));
          Modelica.Mechanics.MultiBody.Sensors.TansformRelativeVector transformVector_z_rel(frame_r_in = resolveInFrame, frame_r_out = resolveInFrameAfterDifferentiation) if get_z_rel annotation(Placement(transformation(extent = {{90, -95},{110, -75}})));
        protected 
          outer Modelica.Mechanics.MultiBody.World world;
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Arrow arrow(r = frame_a.r_0, r_head = frame_b.r_0 - frame_a.r_0, diameter = arrowDiameter, color = arrowColor, specularCoefficient) if world.enableAnimation and animation;

        equation 
          connect(relativePosition.frame_a,frame_a) annotation(Line(points = {{ -80,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativePosition.frame_b,frame_b) annotation(Line(points = {{ -60,0},{100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativePosition.r_rel,r_rel) annotation(Line(points = {{ -70, -11},{ -70, -15},{ -80, -15},{ -80, -80},{ -100, -80},{ -100, -110}}, color = {0,0,127}, smooth = Smooth.None));
          connect(zeroForce1.frame_a,frame_a) annotation(Line(points = {{ -81,50},{ -90,50},{ -90,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(zeroForce2.frame_a,frame_b) annotation(Line(points = {{70,30},{90,30},{90,0},{100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativePosition.r_rel,der1.u) annotation(Line(points = {{ -70, -11},{ -70, -15},{ -60, -15},{ -60, -19}}, color = {0,0,127}, smooth = Smooth.None));
          connect(der2.u,der1.y) annotation(Line(points = {{ -20, -39},{ -20, -35},{ -60, -35},{ -60, -30.5}}, color = {0,0,127}, smooth = Smooth.None));
          connect(relativeAngles.frame_a,frame_a) annotation(Line(points = {{10, -15},{0, -15},{0,30},{ -90,30},{ -90,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativeAngles.frame_b,frame_b) annotation(Line(points = {{30, -15},{40, -15},{40,0},{100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativeAngles.angles,angles) annotation(Line(points = {{20, -26},{20, -110}}, color = {0,0,127}, smooth = Smooth.None));
          connect(relativeAngularVelocity.frame_b,frame_b) annotation(Line(points = {{70, -30},{80, -30},{80,0},{100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativeAngularVelocity.frame_a,frame_a) annotation(Line(points = {{50, -30},{0, -30},{0,30},{ -90,30},{ -90,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativeAngularVelocity.w_rel,w_rel) annotation(Line(points = {{60, -41},{60, -110}}, color = {0,0,127}, smooth = Smooth.None));
          connect(relativeAngularVelocity.w_rel,der3.u) annotation(Line(points = {{60, -41},{60, -43},{100, -43},{100, -49}}, color = {0,0,127}, smooth = Smooth.None));
          connect(der1.y,transformVector_v_rel.r_in) annotation(Line(points = {{ -60, -30.5},{ -60, -42}}, color = {0,0,127}, smooth = Smooth.None));
          connect(transformVector_v_rel.r_out,v_rel) annotation(Line(points = {{ -60, -65},{ -60, -110}}, color = {0,0,127}, smooth = Smooth.None));
          connect(transformVector_v_rel.frame_a,frame_a) annotation(Line(points = {{ -70, -54},{ -90, -54},{ -90,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(transformVector_v_rel.frame_b,frame_b) annotation(Line(points = {{ -50, -54},{ -36, -54},{ -36,0},{100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(transformVector_v_rel.frame_resolve,frame_resolve) annotation(Line(points = {{ -50, -45.9},{ -47, -45.9},{ -47, -47},{ -42, -47},{ -42,80},{100,80}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(frame_resolve,relativePosition.frame_resolve) annotation(Line(points = {{100,80},{ -50,80},{ -50,8.1},{ -60,8.1}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(frame_resolve,zeroForce3.frame_a) annotation(Line(points = {{100,80},{80,80},{80,60},{70,60}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(relativeAngularVelocity.frame_resolve,frame_resolve) annotation(Line(points = {{70, -21.9},{77, -21.9},{77,80},{100,80}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(der2.y,transformVector_a_rel.r_in) annotation(Line(points = {{ -20, -50.5},{ -20, -58}}, color = {0,0,127}, smooth = Smooth.None));
          connect(transformVector_a_rel.frame_a,frame_a) annotation(Line(points = {{ -30, -70},{ -90, -70},{ -90,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(transformVector_a_rel.frame_b,frame_b) annotation(Line(points = {{ -10, -70},{6, -70},{6,0},{100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(transformVector_a_rel.frame_resolve,frame_resolve) annotation(Line(points = {{ -10, -61.9},{ -10,80},{100,80}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(transformVector_a_rel.r_out,a_rel) annotation(Line(points = {{ -20, -81},{ -20, -110}}, color = {0,0,127}, smooth = Smooth.None));
          connect(der3.y,transformVector_z_rel.r_in) annotation(Line(points = {{100, -60.5},{100, -73}}, color = {0,0,127}, smooth = Smooth.None));
          connect(transformVector_z_rel.r_out,z_rel) annotation(Line(points = {{100, -96},{100, -110}}, color = {0,0,127}, smooth = Smooth.None));
          connect(transformVector_z_rel.frame_a,frame_a) annotation(Line(points = {{90, -85},{ -90, -85},{ -90,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(transformVector_z_rel.frame_b,frame_b) annotation(Line(points = {{110, -85},{119, -85},{119,0},{100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(transformVector_z_rel.frame_resolve,frame_resolve) annotation(Line(points = {{110, -76.9},{116, -76.9},{116,80},{100,80}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
        end RelativeSensor;
        model AbsolutePosition "Measure absolute position vector of the origin of a frame connector"
          extends Internal.PartialAbsoluteSensor;
          Blocks.Interfaces.RealOutput r[3](each final quantity = "Position", each final unit = "m") "Absolute position vector resolved in frame defined by resolveInFrame" annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0, origin = {110,0})));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve "Coordinate system in which output vector r is optionally resolved" annotation(Placement(transformation(extent = {{ -16, -16},{16,16}}, rotation =  -90, origin = {0, -100}), iconTransformation(extent = {{ -16, -16},{16,16}}, rotation =  -90, origin = {0, -101})));
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameA resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_a "Frame in which output vector r shall be resolved (1: world, 2: frame_a, 3:frame_resolve)";
        protected 
          Internal.BasicAbsolutePosition position(resolveInFrame = resolveInFrame) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}})));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{70,0},{100,0}}, color = {0,0,127}, smooth = Smooth.None),Text(extent = {{ -127,95},{134,143}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{62,46},{146,16}}, lineColor = {0,0,0}, textString = "r"),Text(extent = {{15, -67},{146, -92}}, lineColor = {95,95,95}, textString = "resolve"),Line(points = {{0, -96},{0, -96},{0, -70},{0, -70}}, color = {0,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot)}), Documentation(info = "<html>
<p>
The absolute position vector of the origin of frame_a is
determined and provided at the output signal connector <b>r</b>.
</p>

<p>
Via parameter <b>resolveInFrame</b> it is defined, in which frame
the position vector is resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>resolveInFrame =<br>Types.ResolveInFrameA.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vector in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vector in frame_a</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vector in frame_resolve</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameA.frame_resolve, the conditional connector
\"frame_resolve\" is enabled and r is resolved in the frame, to
which frame_resolve is connected. Note, if this connector is enabled, it must
be connected.
</p>

<p>
Example: If resolveInFrame = Types.ResolveInFrameA.frame_a, the output vector is
computed as:
</p>

<pre>
    r = MultiBody.Frames.resolve2(frame_a.R, frame_b.r_0);
</pre>
</html>"));
          Modelica.Mechanics.MultiBody.Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve annotation(Placement(transformation(extent = {{20, -40},{40, -20}})));

        equation 
          connect(position.frame_resolve,frame_resolve) annotation(Line(points = {{0, -10},{0, -100}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve,position.frame_resolve) annotation(Line(points = {{20, -30},{0, -30},{0, -10}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(position.r,r) annotation(Line(points = {{11,0},{110,0}}, color = {0,0,127}, smooth = Smooth.None));
          connect(position.frame_a,frame_a) annotation(Line(points = {{ -10,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        end AbsolutePosition;
        model AbsoluteVelocity "Measure absolute velocity vector of origin of frame connector"
          extends Internal.PartialAbsoluteSensor;
          Blocks.Interfaces.RealOutput v[3](each final quantity = "Velocity", each final unit = "m/s") "Absolute velocity vector resolved in frame defined by resolveInFrame" annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0, origin = {110,0})));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve "Coordinate system in which output vector v is optionally resolved" annotation(Placement(transformation(extent = {{ -16, -16},{16,16}}, rotation =  -90, origin = {0, -100}), iconTransformation(extent = {{ -16, -16},{16,16}}, rotation =  -90, origin = {0, -100})));
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameA resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_a "Frame in which output vector v shall be resolved (1: world, 2: frame_a, 3: frame_resolve)";
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{70,0},{100,0}}, color = {0,0,127}, smooth = Smooth.None),Text(extent = {{ -130,72},{131,120}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{58,48},{142,18}}, lineColor = {0,0,0}, textString = "v"),Text(extent = {{15, -67},{146, -92}}, lineColor = {95,95,95}, textString = "resolve"),Line(points = {{0, -70},{0, -95}}, color = {95,95,95}, smooth = Smooth.None, pattern = LinePattern.Dot)}), Documentation(info = "<html>
<p>
The absolute velocity vector of the origin of frame_a is
determined and provided at the output signal connector <b>v</b>.
</p>

<p>
Via parameter <b>resolveInFrame</b> it is defined, in which frame
the velocity vector is resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>resolveInFrame =<br>Types.ResolveInFrameA.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vector in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vector in frame_a</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vector in frame_resolve</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameA.frame_resolve, the conditional connector
\"frame_resolve\" is enabled and v is resolved in the frame, to
which frame_resolve is connected. Note, if this connector is enabled, it must
be connected.
</p>

<p>
Example: If resolveInFrame = Types.ResolveInFrameA.frame_a, the output vector is
computed as:
</p>

<pre>
    v0 = der(frame_a.r_0);
    v  = MultiBody.Frames.resolve2(frame_a.R, v0);
</pre>

</html>"));
        protected 
          Internal.BasicAbsolutePosition position(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.world) annotation(Placement(transformation(extent = {{ -60, -10},{ -40,10}})));
          Blocks.Continuous.Der der1[3] annotation(Placement(transformation(extent = {{ -20, -20},{0,0}}, rotation = 0, origin = {10,10})));
          TansformAbsoluteVector tansformAbsoluteVector(frame_r_in = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.world, frame_r_out = resolveInFrame) annotation(Placement(transformation(extent = {{10, -10},{ -10,10}}, rotation = 90, origin = {50,0})));
          Modelica.Mechanics.MultiBody.Interfaces.ZeroPosition zeroPosition annotation(Placement(transformation(extent = {{ -60, -60},{ -80, -40}})));
          Modelica.Mechanics.MultiBody.Interfaces.ZeroPosition zeroPosition1 if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve annotation(Placement(transformation(extent = {{60, -60},{80, -40}})));

        equation 
          connect(position.r,der1.u) annotation(Line(points = {{ -39,0},{ -12,0}}, color = {0,0,127}, smooth = Smooth.None));
          connect(position.frame_a,frame_a) annotation(Line(points = {{ -60,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(der1.y,tansformAbsoluteVector.r_in) annotation(Line(points = {{11,0},{19.5,0},{19.5,7.34788e-16},{38,7.34788e-16}}, color = {0,0,127}, smooth = Smooth.None));
          connect(tansformAbsoluteVector.r_out,v) annotation(Line(points = {{61, -6.73556e-16},{71.5, -6.73556e-16},{71.5,0},{110,0}}, color = {0,0,127}, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve,position.frame_resolve) annotation(Line(points = {{ -60, -50},{ -50, -50},{ -50, -10}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(tansformAbsoluteVector.frame_a,frame_a) annotation(Line(points = {{50,10},{50,20},{ -70,20},{ -70,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(tansformAbsoluteVector.frame_resolve,zeroPosition1.frame_resolve) annotation(Line(points = {{49.9, -10},{50, -10},{50, -50},{60, -50}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(tansformAbsoluteVector.frame_resolve,frame_resolve) annotation(Line(points = {{49.9, -10},{50, -10},{50, -50},{0, -50},{0, -100}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
        end AbsoluteVelocity;
        model AbsoluteAngles "Measure absolute angles between frame connector and the world frame"
          extends Modelica.Icons.RotationalSensor;
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Coordinate system a from which the angles shall be determined" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput angles[3](each final quantity = "Angles", each final unit = "rad", each displayUnit = "deg") "Angles to rotate world frame into frame_a via 'sequence'" annotation(Placement(transformation(origin = {110,0}, extent = {{ -10, -10},{10,10}}, rotation = 0)));
          parameter MultiBody.Types.RotationSequence sequence(min = {1,1,1}, max = {3,3,3}) = {1,2,3} "Angles are returned to rotate world frame around axes sequence[1], sequence[2] and finally sequence[3] into frame_a" annotation(Evaluate = true);
          parameter SI.Angle guessAngle1 = 0 "Select angles[1] such that abs(angles[1] - guessAngle1) is a minimum";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -108,43},{ -72,18}}, lineColor = {128,128,128}, textString = "a"),Line(points = {{ -70,0},{ -96,0},{ -96,0}}, color = {0,0,0}, smooth = Smooth.None),Text(extent = {{ -132,76},{129,124}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{70,0},{100,0}}, color = {0,0,127}, smooth = Smooth.None),Text(extent = {{62, -22},{172, -44}}, lineColor = {0,0,0}, textString = "angles")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
This model determines the 3 angles to rotate the world frame
into frame_a along the axes defined by parameter <b>sequence</b>.
For example, if sequence = {3,1,2} then the world frame is
rotated around angles[1] along the z-axis, afterwards it is rotated
around angles[2] along the x-axis, and finally it is rotated around
angles[3] along the y-axis and is then identical to frame_a.
The 3 angles are returned in the range
</p>
<pre>
    -<font face=\"Symbol\">p</font> &lt;= angles[i] &lt;= <font face=\"Symbol\">p</font>
</pre>
<p>
There are <b>two solutions</b> for \"angles[1]\" in this range.
Via parameter <b>guessAngle1</b> (default = 0) the
returned solution is selected such that |angles[1] - guessAngle1| is
minimal. The transformation matrix between the world frame and
frame_a may be in a singular configuration with respect to \"sequence\", i.e.,
there is an infinite number of angle values leading to the same relative
transformation matrix. In this case, the returned solution is
selected by setting angles[1] = guessAngle1. Then angles[2]
and angles[3] can be uniquely determined in the above range.
</p>
<p>
The parameter <b>sequence</b> has the restriction that
only values 1,2,3 can be used and that sequence[1] &ne; sequence[2]
and sequence[2] &ne; sequence[3]. Often used values are:
</p>
<pre>
  sequence = <b>{1,2,3}</b>  // Cardan or Tait-Bryan angle sequence
           = <b>{3,1,3}</b>  // Euler angle sequence
           = <b>{3,2,1}</b>
</pre>
</html>"));

        equation 
          frame_a.f = zeros(3);
          frame_a.t = zeros(3);
          angles = MultiBody.Frames.axesRotationsAngles(frame_a.R, sequence, guessAngle1);
        end AbsoluteAngles;
        model AbsoluteAngularVelocity "Measure absolute angular velocity of frame connector"
          extends Internal.PartialAbsoluteSensor;
          Blocks.Interfaces.RealOutput w[3](each final quantity = "AngularVelocity", each final unit = "1/s") "Absolute angular velocity vector of frame_a with respect to world frame, resolved in frame defined by resolveInFrame" annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0, origin = {110,0})));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve "Coordinate system in which w is optionally resolved" annotation(Placement(transformation(extent = {{ -16, -16},{16,16}}, rotation =  -90, origin = {0, -100}), iconTransformation(extent = {{ -16, -16},{16,16}}, rotation =  -90, origin = {0, -101})));
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameA resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_a "Frame in which output vector w shall be resolved (1: world, 2: frame_a, 3: frame_resolve)";
        protected 
          Internal.BasicAbsoluteAngularVelocity angularVelocity(resolveInFrame = resolveInFrame) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}})));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{70,0},{100,0}}, color = {0,0,127}, smooth = Smooth.None),Text(extent = {{ -127,77},{134,125}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{66,48},{150,18}}, lineColor = {0,0,0}, textString = "w"),Line(points = {{0, -70},{0, -96}}, color = {95,95,95}, smooth = Smooth.None, pattern = LinePattern.Dot),Text(extent = {{15, -67},{146, -92}}, lineColor = {95,95,95}, textString = "resolve")}), Documentation(info = "<html>
<p>
The absolute angular velocity of frame_a with respect to the
world frame is
determined and provided at the output signal connector <b>w</b>.
</p>

<p>
Via parameter <b>resolveInFrame</b> it is defined, in which frame
the angular velocity is resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>resolveInFrame =<br>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vector in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vector in frame_a</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vector in frame_resolve</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameA.frame_resolve, the conditional connector
\"frame_resolve\" is enabled and w is resolved in the frame, to
which frame_resolve is connected. Note, if this connector is enabled, it must
be connected.
</p>

<p>
Example: If resolveInFrame = Types.ResolveInFrameA.frame_a, the output vector is
computed as:
</p>

<pre>
    w = MultiBody.Frames.angularVelocity2(frame_a.R);
</pre>

</html>"));
          Modelica.Mechanics.MultiBody.Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve annotation(Placement(transformation(extent = {{40, -40},{60, -20}})));

        equation 
          connect(angularVelocity.frame_resolve,frame_resolve) annotation(Line(points = {{0, -10},{0, -100}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve,angularVelocity.frame_resolve) annotation(Line(points = {{40, -30},{0, -30},{0, -10}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(angularVelocity.w,w) annotation(Line(points = {{11,0},{110,0}}, color = {0,0,127}, smooth = Smooth.None));
          connect(angularVelocity.frame_a,frame_a) annotation(Line(points = {{ -10,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        end AbsoluteAngularVelocity;
        model RelativePosition "Measure relative position vector between the origins of two frame connectors"
          extends Internal.PartialRelativeSensor;
          Blocks.Interfaces.RealOutput r_rel[3] "Relative position vector resolved in frame defined by resolveInFrame" annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation =  -90, origin = {0, -110})));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve "Coordinate system in which r_rel is optionally resolved" annotation(Placement(transformation(extent = {{84,64},{116,96}}), iconTransformation(extent = {{84,65},{116,97}})));
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a "Frame in which output vector r_rel shall be resolved (1: world, 2: frame_a, 3: frame_b, 4: frame_resolve)";
        protected 
          Internal.BasicRelativePosition relativePosition(resolveInFrame = resolveInFrame) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}})));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{0, -70},{0, -100}}, color = {0,0,127}, smooth = Smooth.None),Text(extent = {{ -127,95},{134,143}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{18, -80},{102, -110}}, lineColor = {0,0,0}, textString = "r_rel")}), Documentation(info = "<html>
<p>
The relative position vector between the origins of frame_a and frame_b are
determined and provided at the output signal connector <b>r_rel</b>.
</p>

<p>
Via parameter <b>resolveInFrame</b> it is defined, in which frame
the position vector is resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>resolveInFrame =<br>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vector in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vector in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve vector in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vector in frame_resolve</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameAB.frame_resolve, the conditional connector
\"frame_resolve\" is enabled and r_rel is resolved in the frame, to
which frame_resolve is connected. Note, if this connector is enabled, it must
be connected.
</p>

<p>
Example: If resolveInFrame = Types.ResolveInFrameAB.frame_a, the output vector is
computed as:
</p>

<pre>
    r_rel = MultiBody.Frames.resolve2(frame_a.R, frame_b.r_0 - frame_a.r_0);
</pre>
</html>"));
          Modelica.Mechanics.MultiBody.Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve annotation(Placement(transformation(extent = {{52,20},{72,40}})));

        equation 
          connect(relativePosition.frame_a,frame_a) annotation(Line(points = {{ -10,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativePosition.frame_b,frame_b) annotation(Line(points = {{10,0},{100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativePosition.frame_resolve,frame_resolve) annotation(Line(points = {{10,8.1},{26,8.1},{26,8},{36,8},{36,80},{100,80}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve,relativePosition.frame_resolve) annotation(Line(points = {{52,30},{36,30},{36,8.1},{10,8.1}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(relativePosition.r_rel,r_rel) annotation(Line(points = {{0, -11},{0, -110}}, color = {0,0,127}, smooth = Smooth.None));
        end RelativePosition;
        model RelativeVelocity "Measure relative velocity vector between the origins of two frame connectors"
          extends Internal.PartialRelativeSensor;
          Modelica.Blocks.Interfaces.RealOutput v_rel[3](each final quantity = "Velocity", each final unit = "m/s") "Relative velocity vector resolved in frame defined by resolveInFrame" annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation =  -90, origin = {0, -110})));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve "Coordinate system in which v_rel is optionally resolved" annotation(Placement(transformation(extent = {{84,64},{116,96}}), iconTransformation(extent = {{84,65},{116,97}})));
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a "Frame in which output vector v_rel shall be resolved (1: world, 2: frame_a, 3: frame_b, 4: frame_resolve)";
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{0, -70},{0, -100}}, color = {0,0,127}, smooth = Smooth.None),Text(extent = {{ -127,95},{134,143}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{18, -80},{102, -110}}, lineColor = {0,0,0}, textString = "v_rel")}), Documentation(info = "<html>
<p>
The relative velocity vector between the origins of frame_a and of frame_b are
determined and provided at the output signal connector <b>v_rel</b>.
This vector is defined as:
</p>

<pre>
    r_rel = MultiBody.Frames.resolve2(frame_a.R, frame_b.r_0 - frame_a.r_0);
    v_rel = <b>der</b>(r_rel);
</pre>

<p>
Via parameter <b>resolveInFrame</b> it is defined, in which frame
the velocity vector is resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>resolveInFrame =<br>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vector in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vector in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve vector in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vector in frame_resolve</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameAB.frame_resolve, the conditional connector
\"frame_resolve\" is enabled and v_rel is resolved in the frame, to
which frame_resolve is connected. Note, if this connector is enabled, it must
be connected.
</p>

<p>
Example: If resolveInFrame = Types.ResolveInFrameAB.frame_b, the output vector is
computed as:
</p>

<pre>
    r_rel   = MultiBody.Frames.resolve2(frame_a.R, frame_b.r_0 - frame_a.r_0);
    v_rel_a = <b>der</b>(r_rel);
    v_rel   = MultiBody.Frames.resolveRelative(frame_a.R, frame_b.R, v_rel_a);
</pre>

</html>"));
        protected 
          RelativePosition relativePosition(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}})));
          Modelica.Mechanics.MultiBody.Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve annotation(Placement(transformation(extent = {{50, -60},{70, -40}})));
          Modelica.Blocks.Continuous.Der der_r_rel[3] annotation(Placement(transformation(extent = {{ -20, -20},{0,0}}, rotation =  -90, origin = {10, -40})));
          TansformRelativeVector tansformRelativeVector(frame_r_in = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a, frame_r_out = resolveInFrame) annotation(Placement(transformation(extent = {{ -10, -80},{10, -60}})));

        equation 
          connect(relativePosition.frame_a,frame_a) annotation(Line(points = {{ -10,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativePosition.frame_b,frame_b) annotation(Line(points = {{10,0},{100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativePosition.r_rel,der_r_rel.u) annotation(Line(points = {{0, -11},{0, -18},{3.98072e-15, -18}}, color = {0,0,127}, smooth = Smooth.None));
          connect(der_r_rel.y,tansformRelativeVector.r_in) annotation(Line(points = {{ -2.4431e-16, -41},{ -2.4431e-16, -50},{0, -50},{0, -58}}, color = {0,0,127}, smooth = Smooth.None));
          connect(tansformRelativeVector.r_out,v_rel) annotation(Line(points = {{0, -81},{0, -110}}, color = {0,0,127}, smooth = Smooth.None));
          connect(tansformRelativeVector.frame_a,frame_a) annotation(Line(points = {{ -10, -70},{ -70, -70},{ -70,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(tansformRelativeVector.frame_b,frame_b) annotation(Line(points = {{10, -70},{80, -70},{80,0},{100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(tansformRelativeVector.frame_resolve,frame_resolve) annotation(Line(points = {{10, -61.9},{35, -61.9},{35,80},{100,80}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve,tansformRelativeVector.frame_resolve) annotation(Line(points = {{50, -50},{35, -50},{35, -61.9},{10, -61.9}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
        end RelativeVelocity;
        model RelativeAngles "Measure relative angles between two frame connectors"
          extends Modelica.Icons.RotationalSensor;
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Coordinate system a" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_b "Coordinate system b" annotation(Placement(transformation(extent = {{84, -16},{116,16}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput angles[3](each final quantity = "Angles", each final unit = "rad", displayUnit = "deg") "Angles to rotate frame_a into frame_b via 'sequence'" annotation(Placement(transformation(origin = {0, -110}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          parameter MultiBody.Types.RotationSequence sequence(min = {1,1,1}, max = {3,3,3}) = {1,2,3} "Angles are returned to rotate frame_a around axes sequence[1], sequence[2] and finally sequence[3] into frame_b" annotation(Evaluate = true);
          parameter SI.Angle guessAngle1 = 0 "Select angles[1] such that abs(angles[1] - guessAngle1) is a minimum";
          Modelica.Mechanics.MultiBody.Frames.Orientation R_rel "Relative orientation object from frame_a to frame_b";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -108,43},{ -72,18}}, lineColor = {128,128,128}, textString = "a"),Text(extent = {{72,41},{108,16}}, lineColor = {128,128,128}, textString = "b"),Line(points = {{ -70,0},{ -96,0},{ -96,0}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{96,0},{70,0},{70,0}}, color = {0,0,0}, smooth = Smooth.None),Text(extent = {{ -132,90},{129,138}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{0, -70},{0, -100}}, color = {0,0,127}, smooth = Smooth.None),Text(extent = {{0, -76},{110, -98}}, lineColor = {0,0,0}, textString = "angles")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
This model determines the 3 angles to rotate frame_a
into frame_b along the axes defined by parameter <b>sequence</b>.
For example, if sequence = {3,1,2} then frame_a is
rotated around angles[1] along the z-axis, afterwards it is rotated
around angles[2] along the x-axis, and finally it is rotated around
angles[3] along the y-axis and is then identical to frame_b.
The 3 angles are returned in the range
</p>
<pre>
    -<font face=\"Symbol\">p</font> &lt;= angles[i] &lt;= <font face=\"Symbol\">p</font>
</pre>
<p>
There are <b>two solutions</b> for \"angles[1]\" in this range.
Via parameter <b>guessAngle1</b> (default = 0) the
returned solution is selected such that |angles[1] - guessAngle1| is
minimal. The relative transformation matrix between frame_a and
frame_b may be in a singular configuration with respect to \"sequence\", i.e.,
there is an infinite number of angle values leading to the same relative
transformation matrix. In this case, the returned solution is
selected by setting angles[1] = guessAngle1. Then angles[2]
and angles[3] can be uniquely determined in the above range.
</p>
<p>
The parameter <b>sequence</b> has the restriction that
only values 1,2,3 can be used and that sequence[1] &ne; sequence[2]
and sequence[2] &ne; sequence[3]. Often used values are:
</p>
<pre>
  sequence = <b>{1,2,3}</b>  // Cardan or Tait-Bryan angle sequence
           = <b>{3,1,3}</b>  // Euler angle sequence
           = <b>{3,2,1}</b>
</pre>
</html>"));

        equation 
          frame_a.f = zeros(3);
          frame_a.t = zeros(3);
          frame_b.f = zeros(3);
          frame_b.t = zeros(3);
          R_rel = Modelica.Mechanics.MultiBody.Frames.relativeRotation(frame_a.R, frame_b.R);
          angles = MultiBody.Frames.axesRotationsAngles(R_rel, sequence, guessAngle1);
        end RelativeAngles;
        model RelativeAngularVelocity "Measure relative angular velocity between two frame connectors"
          extends Internal.PartialRelativeSensor;
          Blocks.Interfaces.RealOutput w_rel[3](each final quantity = "AngularVelocity", each final unit = "1/s") "Relative angular velocity vector between frame_a and frame_b resolved in frame defined by resolveInFrame" annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation =  -90, origin = {0, -110})));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve "Coordinate system in which w_rel is optionally resolved" annotation(Placement(transformation(extent = {{84,64},{116,96}}), iconTransformation(extent = {{84,65},{116,97}})));
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a "Frame in which output vector w_rel shall be resolved (1: world, 2: frame_a, 3: frame_b, 4: frame_resolve)";
        protected 
          Internal.BasicRelativeAngularVelocity relativeAngularVelocity(resolveInFrame = resolveInFrame) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}})));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{0, -70},{0, -100}}, color = {0,0,127}, smooth = Smooth.None),Text(extent = {{ -127,95},{134,143}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{14, -78},{98, -108}}, lineColor = {0,0,0}, textString = "w_rel")}), Documentation(info = "<html>
<p>
The relative angular velocity between frame_a and frame_b is
determined and provided at the output signal connector <b>w_rel</b>.
</p>

<p>
Via parameter <b>resolveInFrame</b> it is defined, in which frame
the angular velocity is resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>resolveInFrame =<br>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vector in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vector in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve vector in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vector in frame_resolve</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameAB.frame_resolve, the conditional connector
\"frame_resolve\" is enabled and w_rel is resolved in the frame, to
which frame_resolve is connected. Note, if this connector is enabled, it must
be connected.
</p>

<p>
Example: If resolveInFrame = Types.ResolveInFrameAB.frame_a, the output vector is
computed as:
</p>

<pre>
    // Relative orientation object from frame_a to frame_b
    R_rel = MultiBody.Frames.relativeRotation(frame_a.R, frame_b.R);

    // Angular velocity resolved in frame_a
    w_rel = MultiBody.Frames.angularVelocity1(R_rel);
</pre>

</html>"));
          Modelica.Mechanics.MultiBody.Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve annotation(Placement(transformation(extent = {{52,20},{72,40}})));

        equation 
          connect(relativeAngularVelocity.frame_a,frame_a) annotation(Line(points = {{ -10,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativeAngularVelocity.frame_b,frame_b) annotation(Line(points = {{10,0},{100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(relativeAngularVelocity.frame_resolve,frame_resolve) annotation(Line(points = {{10,8.1},{26,8.1},{26,8},{34,8},{34,80},{100,80}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve,relativeAngularVelocity.frame_resolve) annotation(Line(points = {{52,30},{34,30},{34,8.1},{10,8.1}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(relativeAngularVelocity.w_rel,w_rel) annotation(Line(points = {{0, -11},{0, -110}}, color = {0,0,127}, smooth = Smooth.None));
        end RelativeAngularVelocity;
        model Distance "Measure the distance between the origins of two frame connectors"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Frames;
          import Modelica.Mechanics.MultiBody.Types;
          extends Interfaces.PartialTwoFrames;
          extends Modelica.Icons.TranslationalSensor;
          Modelica.Blocks.Interfaces.RealOutput distance "Distance between the origin of frame_a and the origin of frame_b" annotation(Placement(transformation(origin = {0, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          parameter Boolean animation = true "= true, if animation shall be enabled (show arrow)";
          input SI.Diameter arrowDiameter = world.defaultArrowDiameter "Diameter of relative arrow from frame_a to frame_b" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color arrowColor = Modelica.Mechanics.MultiBody.Types.Defaults.SensorColor "Color of relative arrow from frame_a to frame_b" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Position s_small(min = sqrt(Modelica.Constants.small)) = 1e-10 "Prevent zero-division if distance between frame_a and frame_b is zero" annotation(Dialog(tab = "Advanced"));
        protected 
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Arrow arrow(r = frame_a.r_0, r_head = frame_b.r_0 - frame_a.r_0, diameter = arrowDiameter, color = arrowColor, specularCoefficient = specularCoefficient) if world.enableAnimation and animation;
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{0, -60},{0, -100}}, color = {0,0,255}),Line(points = {{ -70,0},{ -101,0}}, color = {0,0,0}),Line(points = {{70,0},{100,0}}, color = {0,0,0}),Text(extent = {{ -128,30},{133,78}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -70,0},{ -101,0}}, color = {0,0,0}),Line(points = {{70,0},{100,0}}, color = {0,0,0}),Line(points = {{0, -60},{0, -100}}, color = {0,0,255}),Text(extent = {{ -22,70},{20,46}}, textString = "s", lineColor = {0,0,255}),Line(points = {{ -98,40},{88,40}}, color = {0,0,255}),Polygon(points = {{102,40},{87,46},{87,34},{102,40}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
<p>
The <b>distance</b> between the origins of frame_a
and of frame_b are determined and provided at the
output signal connector <b>distance</b>. This
distance is always positive. <b>Derivatives</b> of this
signal can be easily obtained by connecting the
block
<a href=\"Modelica://Modelica.Blocks.Continuous.Der\">Modelica.Blocks.Continuous.Der</a>
to \"distance\" (this block performs analytic differentiation
of the input signal using the der(..) operator).
</p>
<p>
In the following figure the animation of a Distance
sensor is shown. The light blue coordinate system is
frame_a, the dark blue coordinate system is frame_b, and
the yellow arrow is the animated sensor.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Sensors/Distance.png\">
</p>
<p>
If the distance is smaller as parameter <b>s_small</b> (in the \"advanced\" menu),
it is approximated such that its derivative is
finite for zero distance. Without such an approximation, the derivative would
be infinite and a division by zero would occur. The approximation is performed
in the following way: If distance > s_small, it is computed as sqrt(r*r) where
r is the position vector from the origin of frame_a to the origin of frame_b.
If the distance becomes smaller as s_small, the \"sqrt()\" function is approximated
by a second order polynomial, such that the function value and its first derivative
are identical for sqrt() and the polynomial at s_small. Futhermore, the polynomial
passes through zero. The effect is, that the distance function is continuous and
differentiable everywhere. The derivative at zero distance is 3/(2*s_small).
</p>
</HTML>"));
        protected 
          SI.Position r_rel_0[3] = frame_b.r_0 - frame_a.r_0 "Position vector from frame_a to frame_b resolved in world frame";
          SI.Area L2 = r_rel_0 * r_rel_0;
          SI.Area s_small2 = s_small ^ 2;

        equation 
          frame_a.f = zeros(3);
          frame_b.f = zeros(3);
          frame_a.t = zeros(3);
          frame_b.t = zeros(3);
          distance = smooth(1, if noEvent(L2 > s_small2) then sqrt(L2) else L2 / (2 * s_small) * (3 - L2 / s_small2));
        end Distance;
        model CutForce "Measure cut force vector"
          import SI = Modelica.SIunits;
          Modelica.Blocks.Interfaces.RealOutput force[3](final quantity = "Force", final unit = "N") "Cut force resolved in frame defined by resolveInFrame" annotation(Placement(transformation(origin = { -80, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          parameter Boolean animation = true "= true, if animation shall be enabled (show arrow)";
          parameter Boolean positiveSign = true "= true, if force with positive sign is returned (= frame_a.f), otherwise with negative sign (= frame_b.f)";
          input Real N_to_m(unit = "N/m") = 1000 " Force arrow scaling (length = force/N_to_m)" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter forceDiameter = world.defaultArrowDiameter " Diameter of force arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color forceColor = Modelica.Mechanics.MultiBody.Types.Defaults.ForceColor " Color of force arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(group = "if animation = true", enable = animation));
          extends Modelica.Mechanics.MultiBody.Sensors.Internal.PartialCutForceSensor;
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -190, -70},{ -74, -96}}, lineColor = {0,0,0}, textString = "force"),Line(points = {{ -80, -100},{ -80,0}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<HTML>
<p>
The cut-force acting between the two frames to which this
model is connected, is determined and provided at the output signal connector
<b>force</b> (= frame_a.f). If parameter <b>positiveSign</b> =
<b>false</b>, the negative cut-force is provided (= frame_b.f).

<p>
Via parameter <b>resolveInFrame</b> it is defined, in which frame
the force vector is resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>resolveInFrame =<br>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vector in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vector in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve vector in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vector in frame_resolve</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameAB.frame_resolve, the conditional connector
\"frame_resolve\" is enabled and output force is resolved in the frame, to
which frame_resolve is connected. Note, if this connector is enabled, it must
be connected.
</p>

<p>
In the following figure the animation of a CutForce
sensor is shown. The dark blue coordinate system is frame_b,
and the green arrow is the cut force acting at frame_b and
with negative sign at frame_a.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Sensors/CutForce.png\">
</p>
</HTML>"));
        protected 
          SI.Position f_in_m[3] = (frame_a.f * (if positiveSign then  +1 else  -1)) / N_to_m "Force mapped from N to m for animation";
          Visualizers.Advanced.Arrow forceArrow(diameter = forceDiameter, color = forceColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = f_in_m, r_head =  -f_in_m) if world.enableAnimation and animation;
          Internal.BasicCutForce cutForce(resolveInFrame = resolveInFrame, positiveSign = positiveSign) annotation(Placement(transformation(extent = {{ -50, -10},{ -30,10}})));
          Modelica.Mechanics.MultiBody.Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve annotation(Placement(transformation(extent = {{0, -40},{20, -20}})));

        equation 
          connect(cutForce.frame_a,frame_a) annotation(Line(points = {{ -50,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(cutForce.frame_b,frame_b) annotation(Line(points = {{ -30,0},{100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(cutForce.frame_resolve,frame_resolve) annotation(Line(points = {{ -32, -10},{ -32, -60},{80, -60},{80, -100}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(cutForce.force,force) annotation(Line(points = {{ -48, -11},{ -48, -60},{ -80, -60},{ -80, -110}}, color = {0,0,127}, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve,cutForce.frame_resolve) annotation(Line(points = {{0, -30},{ -32, -30},{ -32, -10}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
        end CutForce;
        model CutTorque "Measure cut torque vector"
          import SI = Modelica.SIunits;
          Modelica.Blocks.Interfaces.RealOutput torque[3] "Cut torque resolved in frame defined by resolveInFrame" annotation(Placement(transformation(origin = { -80, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          parameter Boolean animation = true "= true, if animation shall be enabled (show arrow)";
          parameter Boolean positiveSign = true "= true, if torque with positive sign is returned (= frame_a.t), otherwise with negative sign (= frame_b.t)";
          input Real Nm_to_m(unit = "N.m/m") = 1000 "Torque arrow scaling (length = torque/Nm_to_m)" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter torqueDiameter = world.defaultArrowDiameter "Diameter of torque arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color torqueColor = Modelica.Mechanics.MultiBody.Types.Defaults.TorqueColor "Color of torque arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(group = "if animation = true", enable = animation));
          extends Modelica.Mechanics.MultiBody.Sensors.Internal.PartialCutForceSensor;
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -180, -72},{ -64, -98}}, lineColor = {0,0,0}, textString = "torque"),Line(points = {{ -80, -100},{ -80,0}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<HTML>
<p>
The cut-torque acting between the two frames to which this
model is connected, is determined and provided at the output signal connector
<b>torque</b> (= frame_a.t). If parameter <b>positiveSign</b> =
<b>false</b>, the negative cut-torque is provided (= frame_b.t).
</p>

<p>
Via parameter <b>resolveInFrame</b> it is defined, in which frame
the torque vector is resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>resolveInFrame =<br>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vector in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vector in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve vector in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vector in frame_resolve</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameAB.frame_resolve, the conditional connector
\"frame_resolve\" is enabled and output torque is resolved in the frame, to
which frame_resolve is connected. Note, if this connector is enabled, it must
be connected.
</p>


<p>
In the following figure the animation of a CutTorque
sensor is shown. The dark blue coordinate system is frame_b,
and the green arrow is the cut torque acting at frame_b and
with negative sign at frame_a.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Sensors/CutTorque.png\">
</p>
</HTML>"));
        protected 
          SI.Position t_in_m[3] = (frame_a.t * (if positiveSign then  +1 else  -1)) / Nm_to_m "Torque mapped from Nm to m for animation";
          Visualizers.Advanced.DoubleArrow torqueArrow(diameter = torqueDiameter, color = torqueColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = t_in_m, r_head =  -t_in_m) if world.enableAnimation and animation;
          Internal.BasicCutTorque cutTorque(resolveInFrame = resolveInFrame, positiveSign = positiveSign) annotation(Placement(transformation(extent = {{ -62, -10},{ -42,10}})));
          Modelica.Mechanics.MultiBody.Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve annotation(Placement(transformation(extent = {{ -20, -40},{0, -20}})));

        equation 
          connect(cutTorque.frame_a,frame_a) annotation(Line(points = {{ -62,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(cutTorque.frame_b,frame_b) annotation(Line(points = {{ -42,0},{100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(cutTorque.torque,torque) annotation(Line(points = {{ -60, -11},{ -60, -80},{ -80, -80},{ -80, -110}}, color = {0,0,127}, smooth = Smooth.None));
          connect(cutTorque.frame_resolve,frame_resolve) annotation(Line(points = {{ -44, -10},{ -44, -74},{80, -74},{80, -100}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve,cutTorque.frame_resolve) annotation(Line(points = {{ -20, -30},{ -44, -30},{ -44, -10}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
        end CutTorque;
        model CutForceAndTorque "Measure cut force and cut torque vector"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Types;
          Modelica.Blocks.Interfaces.RealOutput force[3](final quantity = "Force", final unit = "N") "Cut force resolved in frame defined by resolveInFrame" annotation(Placement(transformation(origin = { -80, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Modelica.Blocks.Interfaces.RealOutput torque[3] "Cut torque resolved in frame defined by resolveInFrame" annotation(Placement(transformation(origin = {0, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          parameter Boolean animation = true "= true, if animation shall be enabled (show force and torque arrow)";
          parameter Boolean positiveSign = true "= true, if force and torque with positive sign is returned (= frame_a.f/.t), otherwise with negative sign (= frame_b.f/.t)";
          input Real N_to_m(unit = "N/m") = 1000 "Force arrow scaling (length = force/N_to_m)" annotation(Dialog(group = "if animation = true", enable = animation));
          input Real Nm_to_m(unit = "N.m/m") = 1000 "Torque arrow scaling (length = torque/Nm_to_m)" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter forceDiameter = world.defaultArrowDiameter "Diameter of force arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter torqueDiameter = forceDiameter " Diameter of torque arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color forceColor = Modelica.Mechanics.MultiBody.Types.Defaults.ForceColor "Color of force arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color torqueColor = Modelica.Mechanics.MultiBody.Types.Defaults.TorqueColor "Color of torque arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(group = "if animation = true", enable = animation));
          extends Modelica.Mechanics.MultiBody.Sensors.Internal.PartialCutForceSensor;
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -80, -100},{ -80,0}}, color = {0,0,127}),Line(points = {{0, -100},{0, -70}}, color = {0,0,127}),Text(extent = {{ -188, -70},{ -72, -96}}, lineColor = {0,0,0}, textString = "force"),Text(extent = {{ -56, -70},{60, -96}}, lineColor = {0,0,0}, textString = "torque")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<HTML>
<p>
The cut-force and cut-torque acting between the two frames to which this
model is connected, are determined and provided at the output signal connectors
<b>force</b> (= frame_a.f) and <b>torque</b> (= frame_a.t).
If parameter <b>positiveSign</b> =
<b>false</b>, the negative cut-force and cut-torque is provided
(= frame_b.f, frame_b.t).

<p>
Via parameter <b>resolveInFrame</b> it is defined, in which frame
the two vectors are resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>resolveInFrame =<br>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve vectors in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve vectors in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve vectors in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve vectors in frame_resolve</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameAB.frame_resolve, the conditional connector
\"frame_resolve\" is enabled and the output vectors force and torque are resolved in the frame, to
which frame_resolve is connected. Note, if this connector is enabled, it must
be connected.
</p>


<p>
In the following figure the animation of a CutForceAndTorque
sensor is shown. The dark blue coordinate system is frame_b,
and the green arrows are the cut force and the cut torque,
respectively, acting at frame_b and
with negative sign at frame_a.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Sensors/CutForceAndTorque.png\">
</p>
</HTML>"));
        protected 
          parameter Integer csign = if positiveSign then  +1 else  -1;
          SI.Position f_in_m[3] = (frame_a.f * csign) / N_to_m "Force mapped from N to m for animation";
          SI.Position t_in_m[3] = (frame_a.t * csign) / Nm_to_m "Torque mapped from Nm to m for animation";
          Visualizers.Advanced.Arrow forceArrow(diameter = forceDiameter, color = forceColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = f_in_m, r_head =  -f_in_m) if world.enableAnimation and animation;
          Visualizers.Advanced.DoubleArrow torqueArrow(diameter = torqueDiameter, color = torqueColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = t_in_m, r_head =  -t_in_m) if world.enableAnimation and animation;
          Internal.BasicCutForce cutForce(resolveInFrame = resolveInFrame, positiveSign = positiveSign) annotation(Placement(transformation(extent = {{ -60, -10},{ -40,10}})));
          Internal.BasicCutTorque cutTorque(resolveInFrame = resolveInFrame, positiveSign = positiveSign) annotation(Placement(transformation(extent = {{ -2, -10},{18,10}})));
          Modelica.Mechanics.MultiBody.Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve annotation(Placement(transformation(extent = {{60,30},{80,50}})));

        equation 
          connect(cutForce.frame_a,frame_a) annotation(Line(points = {{ -60,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(cutForce.frame_b,cutTorque.frame_a) annotation(Line(points = {{ -40,0},{ -2,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(cutTorque.frame_b,frame_b) annotation(Line(points = {{18,0},{100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(cutForce.force,force) annotation(Line(points = {{ -58, -11},{ -58, -20},{ -80, -20},{ -80, -110}}, color = {0,0,127}, smooth = Smooth.None));
          connect(cutTorque.torque,torque) annotation(Line(points = {{0, -11},{0, -110}}, color = {0,0,127}, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve,cutTorque.frame_resolve) annotation(Line(points = {{60,40},{32,40},{32, -20},{16, -20},{16, -10}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve,cutForce.frame_resolve) annotation(Line(points = {{60,40},{ -26,40},{ -26, -20},{ -42, -20},{ -42, -10}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(cutForce.frame_resolve,frame_resolve) annotation(Line(points = {{ -42, -10},{ -42, -70},{80, -70},{80, -100}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(cutTorque.frame_resolve,frame_resolve) annotation(Line(points = {{16, -10},{16, -70},{80, -70},{80, -100}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
        end CutForceAndTorque;
        model Power "Measure power flowing from frame_a to frame_b"
          import SI = Modelica.SIunits;
          extends Modelica.Icons.RotationalSensor;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
          Modelica.Blocks.Interfaces.RealOutput power(quantity = "Power", unit = "W") "Power at frame_a as output signal" annotation(Placement(transformation(origin = { -80, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -70,0},{ -101,0}}, color = {0,0,0}),Line(points = {{70,0},{100,0}}, color = {0,0,0}),Line(points = {{ -80,0},{ -80, -100}}, color = {0,0,127}),Text(extent = {{ -60, -92},{16, -114}}, lineColor = {0,0,0}, textString = "power"),Text(extent = {{ -128,126},{126,68}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<HTML>
<p>
This component provides the power flowing from frame_a to frame_b
as output signal <b>power</b>.
</p>
</HTML>"));

        equation 
          Connections.branch(frame_a.R, frame_b.R);
          frame_a.r_0 = frame_b.r_0;
          frame_a.R = frame_b.R;
          zeros(3) = frame_a.f + frame_b.f;
          zeros(3) = frame_a.t + frame_b.t;
          power = frame_a.f * Frames.resolve2(frame_a.R, der(frame_a.r_0)) + frame_a.t * Frames.angularVelocity2(frame_a.R);
        end Power;
        annotation(Documentation(info = "<html>
<p>
Package <b>Sensors</b> contains <b>ideal measurement</b>
components to determine absolute and relative kinematic
quantities, as well as cut-forces, cut-torques and power. All
measured quantities can be provided in every desired
coordinate system.
</p>

</html>"));
        model TansformAbsoluteVector "Transform absolute vector in to another frame"
          extends Modelica.Icons.RotationalSensor;
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Coordinate system from which absolute kinematic quantities are measured" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve if frame_r_in == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve or frame_r_out == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve "Coordinate system in which r_in or r_out is optionally resolved" annotation(Placement(transformation(extent = {{84, -16},{116,16}}), iconTransformation(extent = {{84, -15},{116,17}})));
          Blocks.Interfaces.RealInput r_in[3] "Input vector resolved in frame defined by frame_r_in" annotation(Placement(transformation(extent = {{ -20, -20},{20,20}}, rotation =  -90, origin = {0,120})));
          Blocks.Interfaces.RealOutput r_out[3] "Input vector r_in resolved in frame defined by frame_r_out" annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation =  -90, origin = {0, -110})));
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameA frame_r_in = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_a "Frame in which vector r_in is resolved (1: world, 2: frame_a, 3: frame_resolve)";
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameA frame_r_out = frame_r_in "Frame in which vector r_in shall be resolved and provided as r_out (1: world, 2: frame_a, 3: frame_resolve)";
        protected 
          Internal.BasicTransformAbsoluteVector basicTransformVector(frame_r_in = frame_r_in, frame_r_out = frame_r_out) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}})));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{0, -70},{0, -100}}, color = {0,0,127}, smooth = Smooth.None),Line(points = {{0,100},{0,70}}, color = {0,0,127}, smooth = Smooth.None),Text(extent = {{ -104,124},{ -18,96}}, lineColor = {0,0,0}, textString = "r_in"),Text(extent = {{ -124, -76},{2, -104}}, lineColor = {0,0,0}, textString = "r_out"),Line(points = {{95,0},{95,0},{70,0},{70,0}}, color = {0,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot),Text(extent = {{58,47},{189,22}}, lineColor = {95,95,95}, textString = "resolve"),Line(points = {{ -70,0},{ -96,0},{ -96,0}}, color = {0,0,0}, smooth = Smooth.None),Text(extent = {{ -116,45},{ -80,20}}, lineColor = {95,95,95}, textString = "a")}), Documentation(info = "<html>
<p>
The input vector \"Real r_in[3]\" is assumed to be an absolute kinematic quantity
of frame_a that is defined to be resolved in the frame defined
with parameter \"frame_r_in\". This model resolves vector r_in in the
coordinate system defined with parameter \"frame_r_out\" and returns the
transformed output vector as \"Real r_out[3]\";
</p>
</html>"));
          Modelica.Mechanics.MultiBody.Interfaces.ZeroPosition zeroPosition if not (frame_r_in == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve or frame_r_out == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve) annotation(Placement(transformation(extent = {{40,18},{60,38}})));

        equation 
          connect(basicTransformVector.frame_a,frame_a) annotation(Line(points = {{ -10,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicTransformVector.frame_resolve,frame_resolve) annotation(Line(points = {{10,0},{100,0}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve,basicTransformVector.frame_resolve) annotation(Line(points = {{40,28},{32,28},{32,0},{10,0}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(basicTransformVector.r_out,r_out) annotation(Line(points = {{0, -11},{0, -110}}, color = {0,0,127}, smooth = Smooth.None));
          connect(basicTransformVector.r_in,r_in) annotation(Line(points = {{0,12},{0,120}}, color = {0,0,127}, smooth = Smooth.None));
        end TansformAbsoluteVector;
        model TansformRelativeVector "Transform relative vector in to another frame"
          extends Internal.PartialRelativeSensor;
          Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve if frame_r_in == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve or frame_r_out == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve "Coordinate system in which r_in or r_out is optionally resolved" annotation(Placement(transformation(extent = {{84,64},{116,96}}), iconTransformation(extent = {{84,65},{116,97}})));
          Blocks.Interfaces.RealInput r_in[3] "Input vector resolved in frame defined by frame_r_in" annotation(Placement(transformation(extent = {{ -20, -20},{20,20}}, rotation =  -90, origin = {0,120})));
          Blocks.Interfaces.RealOutput r_out[3] "Input vector r_in resolved in frame defined by frame_r_out" annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation =  -90, origin = {0, -110})));
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB frame_r_in = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a "Frame in which vector r_in is resolved (1: world, 2: frame_a, 3: frame_b, 4: frame_resolve)";
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB frame_r_out = frame_r_in "Frame in which vector r_in shall be resolved and provided as r_out (1: world, 2: frame_a, 3: frame_b, 4: frame_resolve)";
        protected 
          Modelica.Mechanics.MultiBody.Sensors.Internal.BasicTransformRelativeVector basicTransformVector(frame_r_in = frame_r_in, frame_r_out = frame_r_out) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}})));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{0, -70},{0, -100}}, color = {0,0,127}, smooth = Smooth.None),Line(points = {{0,100},{0,70}}, color = {0,0,127}, smooth = Smooth.None),Text(extent = {{ -104,124},{ -18,96}}, lineColor = {0,0,0}, textString = "r_in"),Text(extent = {{ -124, -76},{2, -104}}, lineColor = {0,0,0}, textString = "r_out")}), Documentation(info = "<html>
<p>
The input vector \"Real r_in[3]\" is assumed to be a relative kinematic quantity
between frame_a and frame_b
that is defined to be resolved in the frame defined
with parameter \"frame_r_in\". This model resolves vector r_in in the
coordinate system defined with parameter \"frame_r_out\" and returns the
transformed output vector as \"Real r_out[3]\";
</p>
</html>"));
          Modelica.Mechanics.MultiBody.Interfaces.ZeroPosition zeroPosition if not (frame_r_in == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve or frame_r_out == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve) annotation(Placement(transformation(extent = {{48,10},{68,30}})));

        equation 
          connect(basicTransformVector.frame_a,frame_a) annotation(Line(points = {{ -10,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicTransformVector.frame_b,frame_b) annotation(Line(points = {{10,0},{100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicTransformVector.frame_resolve,frame_resolve) annotation(Line(points = {{10,8.1},{20,8.1},{20,8},{32,8},{32,80},{100,80}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve,basicTransformVector.frame_resolve) annotation(Line(points = {{48,20},{32,20},{32,8.1},{10,8.1}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(basicTransformVector.r_out,r_out) annotation(Line(points = {{0, -11},{0, -110}}, color = {0,0,127}, smooth = Smooth.None));
          connect(basicTransformVector.r_in,r_in) annotation(Line(points = {{0,12},{0,120}}, color = {0,0,127}, smooth = Smooth.None));
        end TansformRelativeVector;
        package Internal "Internal package, should not be used by user"
          partial model PartialAbsoluteSensor "Partial absolute sensor model for sensors defined by components"
            extends Modelica.Icons.RotationalSensor;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Coordinate system at which the kinematic quantities are measured" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -108,43},{ -72,18}}, lineColor = {128,128,128}, textString = "a"),Line(points = {{ -70,0},{ -96,0},{ -96,0}}, color = {0,0,0}, smooth = Smooth.None)}));

          equation 
            assert(cardinality(frame_a) > 0, "Connector frame_a must be connected at least once");
          end PartialAbsoluteSensor;
          model PartialAbsoluteBaseSensor "Partial absolute sensor models for sensors defined by equations (frame_resolve must be connected exactly once)"
            extends Modelica.Icons.RotationalSensor;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Coordinate system from which kinematic quantities are measured" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve "Coordinate system in which vector is optionally resolved" annotation(Placement(transformation(extent = {{ -16, -16},{16,16}}, rotation =  -90, origin = {10, -100}), iconTransformation(extent = {{ -16, -16},{16,16}}, rotation =  -90, origin = {0, -100})));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -108,43},{ -72,18}}, lineColor = {95,95,95}, textString = "a"),Line(points = {{ -70,0},{ -96,0},{ -96,0}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{0,15},{0, -15}}, color = {0,0,127}, smooth = Smooth.None, origin = {85,0}, rotation = 90),Line(points = {{0, -95},{0, -95},{0, -70},{0, -70}}, color = {0,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot),Text(extent = {{0, -75},{131, -100}}, lineColor = {95,95,95}, textString = "resolve")}));

          equation 
            assert(cardinality(frame_a) > 0, "Connector frame_a must be connected at least once");
            assert(cardinality(frame_resolve) == 1, "Connector frame_resolve must be connected exactly once");
            frame_a.f = zeros(3);
            frame_a.t = zeros(3);
            frame_resolve.f = zeros(3);
            frame_resolve.t = zeros(3);
          end PartialAbsoluteBaseSensor;
          partial model PartialRelativeSensor "Partial relative sensor model for sensors defined by components"
            extends Modelica.Icons.RotationalSensor;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Coordinate system a" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_b "Coordinate system b" annotation(Placement(transformation(extent = {{84, -16},{116,16}}, rotation = 0)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -108,43},{ -72,18}}, lineColor = {128,128,128}, textString = "a"),Text(extent = {{72,41},{108,16}}, lineColor = {128,128,128}, textString = "b"),Line(points = {{ -70,0},{ -96,0},{ -96,0}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{96,0},{70,0},{70,0}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{60,36},{60,36},{60,80},{95,80}}, color = {0,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot)}));

          equation 
            assert(cardinality(frame_a) > 0, "Connector frame_a must be connected at least once");
            assert(cardinality(frame_b) > 0, "Connector frame_b must be connected at least once");
          end PartialRelativeSensor;
          model PartialRelativeBaseSensor "Partial relative sensor models for sensors defined by equations (frame_resolve must be connected exactly once)"
            extends Modelica.Icons.RotationalSensor;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Coordinate system a (measurement is between frame_a and frame_b)" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_b "Coordinate system b (measurement is between frame_a and frame_b)" annotation(Placement(transformation(extent = {{84, -16},{116,16}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve "Coordinate system in which vector is optionally resolved" annotation(Placement(transformation(extent = {{84,64},{116,96}}), iconTransformation(extent = {{84,65},{116,97}})));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -108,43},{ -72,18}}, lineColor = {128,128,128}, textString = "a"),Text(extent = {{72,41},{108,16}}, lineColor = {128,128,128}, textString = "b"),Line(points = {{ -70,0},{ -96,0},{ -96,0}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{96,0},{70,0},{70,0}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{0, -70},{0, -100}}, color = {0,0,127}, smooth = Smooth.None),Line(points = {{60,36},{60,36},{60,80},{95,80}}, color = {0,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot)}));

          equation 
            assert(cardinality(frame_a) > 0, "Connector frame_a must be connected at least once");
            assert(cardinality(frame_b) > 0, "Connector frame_b must be connected at least once");
            assert(cardinality(frame_resolve) == 1, "Connector frame_resolve must be connected exactly once");
            frame_a.f = zeros(3);
            frame_a.t = zeros(3);
            frame_b.f = zeros(3);
            frame_b.t = zeros(3);
            frame_resolve.f = zeros(3);
            frame_resolve.t = zeros(3);
          end PartialRelativeBaseSensor;
          model BasicAbsolutePosition "Measure absolute position vector (same as Sensors.AbsolutePosition, but frame_resolve is not conditional and must be connected)"
            import Modelica.Mechanics.MultiBody.Types.ResolveInFrameA;
            extends Modelica.Mechanics.MultiBody.Sensors.Internal.PartialAbsoluteBaseSensor;
            Modelica.Blocks.Interfaces.RealOutput r[3](each final quantity = "Position", each final unit = "m") "Absolute position vector frame_a.r_0 resolved in frame defined by resolveInFrame" annotation(Placement(transformation(origin = {110,0}, extent = {{ -10, -10},{10,10}}, rotation = 0)));
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameA resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_a "Frame in which output vector r is resolved (1: world, 2: frame_a, 3: frame_resolve)";
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{61,47},{145,17}}, lineColor = {0,0,0}, textString = "r"),Text(extent = {{ -127,75},{134,123}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

          equation 
            if resolveInFrame == ResolveInFrameA.world then
              r = frame_a.r_0;

            elseif resolveInFrame == ResolveInFrameA.frame_a then
              r = Frames.resolve2(frame_a.R, frame_a.r_0);


            elseif resolveInFrame == ResolveInFrameA.frame_resolve then
              r = Frames.resolve2(frame_resolve.R, frame_a.r_0);


            else
              assert(false, "Wrong value for parameter resolveInFrame");
              r = zeros(3);

            end if;
          end BasicAbsolutePosition;
          model BasicAbsoluteAngularVelocity "Measure absolute angular velocity"
            import Modelica.Mechanics.MultiBody.Frames;
            import Modelica.Mechanics.MultiBody.Types.ResolveInFrameA;
            extends Modelica.Mechanics.MultiBody.Sensors.Internal.PartialAbsoluteBaseSensor;
            Modelica.Blocks.Interfaces.RealOutput w[3](final quantity = "AngularVelocity", final unit = "rad/s") "Absolute angular velocity vector" annotation(Placement(transformation(origin = {110,0}, extent = {{ -10, -10},{10,10}}, rotation = 0)));
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameA resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_a "Frame in which output vector w is resolved (1: world, 2: frame_a, 3: frame_resolve)";
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{62,48},{146,18}}, lineColor = {0,0,0}, textString = "w"),Text(extent = {{ -130,76},{131,124}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

          equation 
            if resolveInFrame == ResolveInFrameA.world then
              w = Frames.angularVelocity1(frame_a.R);

            elseif resolveInFrame == ResolveInFrameA.frame_a then
              w = Frames.angularVelocity2(frame_a.R);


            elseif resolveInFrame == ResolveInFrameA.frame_resolve then
              w = Frames.resolveRelative(Frames.angularVelocity1(frame_a.R), frame_a.R, frame_resolve.R);


            else
              assert(false, "Wrong value for parameter resolveInFrame");
              w = zeros(3);

            end if;
          end BasicAbsoluteAngularVelocity;
          model BasicRelativePosition "Measure relative position vector (same as Sensors.RelativePosition, but frame_resolve is not conditional and must be connected)"
            import Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB;
            extends Modelica.Mechanics.MultiBody.Sensors.Internal.PartialRelativeBaseSensor;
            Modelica.Blocks.Interfaces.RealOutput r_rel[3](each final quantity = "Position", each final unit = "m") "Relative position vector frame_b.r_0 - frame_a.r_0 resolved in frame defined by resolveInFrame" annotation(Placement(transformation(origin = {0, -110}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a "Frame in which output vector r_rel is resolved (1: world, 2: frame_a, 3: frame_b, 4: frame_resolve)";
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{12, -76},{96, -106}}, lineColor = {0,0,0}, textString = "r_rel"),Text(extent = {{ -127,95},{134,143}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

          equation 
            if resolveInFrame == ResolveInFrameAB.frame_a then
              r_rel = Frames.resolve2(frame_a.R, frame_b.r_0 - frame_a.r_0);

            elseif resolveInFrame == ResolveInFrameAB.frame_b then
              r_rel = Frames.resolve2(frame_b.R, frame_b.r_0 - frame_a.r_0);


            elseif resolveInFrame == ResolveInFrameAB.world then
              r_rel = frame_b.r_0 - frame_a.r_0;


            elseif resolveInFrame == ResolveInFrameAB.frame_resolve then
              r_rel = Frames.resolve2(frame_resolve.R, frame_b.r_0 - frame_a.r_0);


            else
              assert(false, "Wrong value for parameter resolveInFrame");
              r_rel = zeros(3);

            end if;
          end BasicRelativePosition;
          model BasicRelativeAngularVelocity "Measure relative angular velocity"
            import Modelica.Mechanics.MultiBody.Frames;
            import Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB;
            extends Modelica.Mechanics.MultiBody.Sensors.Internal.PartialRelativeBaseSensor;
            Modelica.Blocks.Interfaces.RealOutput w_rel[3](final quantity = "AngularVelocity", final unit = "rad/s") "Relative angular velocity vector" annotation(Placement(transformation(origin = {0, -110}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a "Frame in which output vector w_rel is resolved (1: world, 2: frame_a, 3: frame_b, 4: frame_resolve)";
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{12, -76},{96, -106}}, lineColor = {0,0,0}, textString = "w_rel"),Text(extent = {{ -132,90},{129,138}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          protected 
            Modelica.Mechanics.MultiBody.Frames.Orientation R_rel "Relative orientation object from frame_a to frame_b";

          equation 
            R_rel = Frames.relativeRotation(frame_a.R, frame_b.R);
            if resolveInFrame == ResolveInFrameAB.frame_a then
              w_rel = Frames.angularVelocity1(R_rel);

            elseif resolveInFrame == ResolveInFrameAB.frame_b then
              w_rel = Frames.angularVelocity2(R_rel);


            elseif resolveInFrame == ResolveInFrameAB.world then
              w_rel = Frames.resolve1(frame_a.R, Frames.angularVelocity1(R_rel));


            elseif resolveInFrame == ResolveInFrameAB.frame_resolve then
              w_rel = Frames.resolveRelative(Frames.angularVelocity1(R_rel), frame_a.R, frame_resolve.R);


            else
              assert(false, "Wrong value for parameter resolveInFrame");
              w_rel = zeros(3);

            end if;
          end BasicRelativeAngularVelocity;
          model BasicTransformAbsoluteVector "Transform absolute vector in to another frame"
            import Modelica.Mechanics.MultiBody.Frames;
            import Modelica.Mechanics.MultiBody.Types.ResolveInFrameA;
            extends Modelica.Icons.RotationalSensor;
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameA frame_r_in = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_a "Frame in which vector r_in is resolved (1: world, 2: frame_a, 3: frame_resolve)";
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameA frame_r_out = frame_r_in "Frame in which vector r_out (= r_in in other frame) is resolved (1: world, 2: frame_a, 3: frame_resolve)";
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Coordinate system from which absolute kinematic quantities are measured" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve "Coordinate system in which vector is optionally resolved" annotation(Placement(transformation(extent = {{ -16, -16},{16,16}}, rotation = 0, origin = {100,0}), iconTransformation(extent = {{ -16, -16},{16,16}}, rotation = 0, origin = {100,0})));
            Blocks.Interfaces.RealInput r_in[3] "Input vector resolved in frame defined by frame_r_in" annotation(Placement(transformation(extent = {{ -20, -20},{20,20}}, rotation =  -90, origin = {0,120})));
            Blocks.Interfaces.RealOutput r_out[3] "Input vector r_in resolved in frame defined by frame_r_out" annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation =  -90, origin = {0, -110})));
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -128, -84},{ -2, -112}}, lineColor = {0,0,0}, textString = "r_out"),Text(extent = {{ -108,137},{ -22,109}}, lineColor = {0,0,0}, textString = "r_in"),Line(points = {{0,100},{0,70}}, color = {0,0,127}, smooth = Smooth.None),Line(points = {{0, -70},{0, -100}}, color = {0,0,127}, smooth = Smooth.None),Text(extent = {{58,47},{189,22}}, lineColor = {95,95,95}, textString = "resolve"),Text(extent = {{ -116,45},{ -80,20}}, lineColor = {95,95,95}, textString = "a"),Line(points = {{ -70,0},{ -96,0},{ -96,0}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{95,0},{95,0},{70,0},{70,0}}, color = {0,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
          protected 
            Modelica.Mechanics.MultiBody.Frames.Orientation R1 "Orientation object from world frame to frame in which r_in is resolved";

          equation 
            assert(cardinality(frame_a) > 0, "Connector frame_a must be connected at least once");
            assert(cardinality(frame_resolve) == 1, "Connector frame_resolve must be connected exactly once");
            frame_a.f = zeros(3);
            frame_a.t = zeros(3);
            frame_resolve.f = zeros(3);
            frame_resolve.t = zeros(3);
            if frame_r_out == frame_r_in then
              r_out = r_in;
              R1 = Frames.nullRotation();

            else
              if frame_r_in == ResolveInFrameA.world then
                R1 = Frames.nullRotation();

              elseif frame_r_in == ResolveInFrameA.frame_a then
                R1 = frame_a.R;


              elseif frame_r_in == ResolveInFrameA.frame_resolve then
                R1 = frame_resolve.R;


              else
                assert(false, "Wrong value for parameter frame_r_in");
                R1 = Frames.nullRotation();

              end if;
              if frame_r_out == ResolveInFrameA.world then
                r_out = Frames.resolve1(R1, r_in);

              elseif frame_r_out == ResolveInFrameA.frame_a then
                r_out = Frames.resolveRelative(r_in, R1, frame_a.R);


              elseif frame_r_out == ResolveInFrameA.frame_resolve then
                r_out = Frames.resolveRelative(r_in, R1, frame_resolve.R);


              else
                assert(false, "Wrong value for parameter frame_r_out");
                r_out = zeros(3);

              end if;

            end if;
          end BasicTransformAbsoluteVector;
          model BasicTransformRelativeVector "Transform relative vector in to another frame"
            import Modelica.Mechanics.MultiBody.Frames;
            import Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB;
            extends Modelica.Mechanics.MultiBody.Sensors.Internal.PartialRelativeBaseSensor;
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB frame_r_in = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a "Frame in which vector r_in is resolved (1: world, 2: frame_a, 3: frame_b, 4: frame_resolve)";
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB frame_r_out = frame_r_in "Frame in which vector r_out (= r_in in other frame) is resolved (1: world, 2: frame_a, 3: frame_b, 4: frame_resolve)";
            Blocks.Interfaces.RealInput r_in[3] "Input vector resolved in frame defined by frame_r_in" annotation(Placement(transformation(extent = {{ -20, -20},{20,20}}, rotation =  -90, origin = {0,120})));
            Blocks.Interfaces.RealOutput r_out[3] "Input vector r_in resolved in frame defined by frame_r_out" annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation =  -90, origin = {0, -110})));
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -128, -92},{ -2, -120}}, lineColor = {0,0,0}, textString = "r_out"),Text(extent = {{ -108,144},{ -22,116}}, lineColor = {0,0,0}, textString = "r_in"),Line(points = {{0,100},{0,70}}, color = {0,0,127}, smooth = Smooth.None)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          protected 
            Modelica.Mechanics.MultiBody.Frames.Orientation R1 "Orientation object from world frame to frame in which r_in is resolved";

          equation 
            if frame_r_out == frame_r_in then
              r_out = r_in;
              R1 = Frames.nullRotation();

            else
              if frame_r_in == ResolveInFrameAB.world then
                R1 = Frames.nullRotation();

              elseif frame_r_in == ResolveInFrameAB.frame_a then
                R1 = frame_a.R;


              elseif frame_r_in == ResolveInFrameAB.frame_b then
                R1 = frame_b.R;


              else
                R1 = frame_resolve.R;

              end if;
              if frame_r_out == ResolveInFrameAB.world then
                r_out = Frames.resolve1(R1, r_in);

              elseif frame_r_out == ResolveInFrameAB.frame_a then
                r_out = Frames.resolveRelative(r_in, R1, frame_a.R);


              elseif frame_r_out == ResolveInFrameAB.frame_b then
                r_out = Frames.resolveRelative(r_in, R1, frame_b.R);


              else
                r_out = Frames.resolveRelative(r_in, R1, frame_resolve.R);

              end if;

            end if;
          end BasicTransformRelativeVector;
          model ZeroForceAndTorque "Set force and torque to zero"
            extends Modelica.Blocks.Interfaces.BlockIcon;
            Interfaces.Frame_a frame_a annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}})));
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -74,24},{80, -20}}, lineColor = {0,0,0}, textString = "f = t = 0")}));

          equation 
            frame_a.f = zeros(3);
            frame_a.t = zeros(3);
          end ZeroForceAndTorque;
          partial model PartialCutForceSensor "Base model to measure the cut force and/or torque between two frames, defined by components"
            extends Modelica.Icons.RotationalSensor;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Coordinate system a" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_b "Coordinate system b" annotation(Placement(transformation(extent = {{84, -16},{116,16}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_resolve "Output vectors are optionally resolved in this frame (cut-force/-torque are set to zero)" annotation(Placement(transformation(origin = {80, -100}, extent = {{ -16, -16},{16,16}}, rotation = 270)));
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameA resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_a "Frame in which output vector(s) is/are resolved (1: world, 2: frame_a, 3: frame_resolve)";
            annotation(Documentation(info = "
<HTML>
<p>
This is a base class for 3-dim. mechanical components with two frames
and one output port in order to measure the cut-force and/or
cut-torque acting between the two frames and
to provide the measured signals as output for further processing
with the blocks of package Modelica.Blocks.
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -101,0}}, color = {0,0,0}),Line(points = {{70,0},{100,0}}, color = {0,0,0}),Text(extent = {{ -132,76},{129,124}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -118,55},{ -82,30}}, lineColor = {128,128,128}, textString = "a"),Text(extent = {{83,55},{119,30}}, lineColor = {128,128,128}, textString = "b"),Text(extent = {{70, -66},{201, -91}}, lineColor = {95,95,95}, textString = "resolve"),Line(points = {{80,0},{80, -100}}, color = {95,95,95}, pattern = LinePattern.Dot)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
          protected 
            outer Modelica.Mechanics.MultiBody.World world;

          equation 
            assert(cardinality(frame_a) > 0, "Connector frame_a of cut-force/-torque sensor object is not connected");
            assert(cardinality(frame_b) > 0, "Connector frame_b of cut-force/-torque sensor object is not connected");
          end PartialCutForceSensor;
          partial model PartialCutForceBaseSensor "Base model to measure the cut force and/or torque between two frames, defined by equations (frame_resolve must be connected exactly once)"
            extends Modelica.Icons.RotationalSensor;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Coordinate system a" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_b "Coordinate system b" annotation(Placement(transformation(extent = {{84, -16},{116,16}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_resolve frame_resolve "The output vector is optionally resolved in this frame (cut-force/-torque are set to zero)" annotation(Placement(transformation(origin = {80, -100}, extent = {{ -16, -16},{16,16}}, rotation = 270)));
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameA resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameA.frame_a "Frame in which output vector is resolved (1: world, 2: frame_a, 3: frame_resolve)";
            annotation(Documentation(info = "
<HTML>
<p>
This is a base class for 3-dim. mechanical components with two frames
and one output port in order to measure the cut-force and/or
cut-torque acting between the two frames and
to provide the measured signals as output for further processing
with the blocks of package Modelica.Blocks.
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -101,0}}, color = {0,0,0}),Line(points = {{70,0},{100,0}}, color = {0,0,0}),Text(extent = {{ -132,76},{129,124}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -118,55},{ -82,30}}, lineColor = {128,128,128}, textString = "a"),Text(extent = {{83,55},{119,30}}, lineColor = {128,128,128}, textString = "b"),Text(extent = {{70, -66},{201, -91}}, lineColor = {95,95,95}, textString = "resolve"),Line(points = {{80,0},{80, -100}}, color = {95,95,95}, pattern = LinePattern.Dot)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
          protected 
            outer Modelica.Mechanics.MultiBody.World world;

          equation 
            Connections.branch(frame_a.R, frame_b.R);
            assert(cardinality(frame_a) > 0, "Connector frame_a of cut-force/-torque sensor object is not connected");
            assert(cardinality(frame_b) > 0, "Connector frame_b of cut-force/-torque sensor object is not connected");
            frame_a.r_0 = frame_b.r_0;
            frame_a.R = frame_b.R;
            zeros(3) = frame_a.f + frame_b.f;
            zeros(3) = frame_a.t + frame_b.t;
            frame_resolve.f = zeros(3);
            frame_resolve.t = zeros(3);
          end PartialCutForceBaseSensor;
          model BasicCutForce "Measure cut force vector (frame_resolve must be connected)"
            import SI = Modelica.SIunits;
            import Modelica.Mechanics.MultiBody.Types.ResolveInFrameA;
            import Modelica.Mechanics.MultiBody.Frames;
            extends Modelica.Mechanics.MultiBody.Sensors.Internal.PartialCutForceBaseSensor;
            Modelica.Blocks.Interfaces.RealOutput force[3](final quantity = "Force", final unit = "N") "Cut force resolved in frame defined by resolveInFrame" annotation(Placement(transformation(origin = { -80, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
            parameter Boolean positiveSign = true "= true, if force with positive sign is returned (= frame_a.f), otherwise with negative sign (= frame_b.f)";
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -190, -70},{ -74, -96}}, lineColor = {0,0,0}, textString = "force"),Line(points = {{ -80, -100},{ -80,0}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Documentation(info = "<HTML>

</HTML>"));
          protected 
            parameter Integer csign = if positiveSign then  +1 else  -1;

          equation 
            if resolveInFrame == ResolveInFrameA.world then
              force = Frames.resolve1(frame_a.R, frame_a.f) * csign;

            elseif resolveInFrame == ResolveInFrameA.frame_a then
              force = frame_a.f * csign;


            elseif resolveInFrame == ResolveInFrameA.frame_resolve then
              force = Frames.resolveRelative(frame_a.f, frame_a.R, frame_resolve.R) * csign;


            else
              assert(false, "Wrong value for parameter resolveInFrame");
              force = zeros(3);

            end if;
          end BasicCutForce;
          model BasicCutTorque "Measure cut torque vector (frame_resolve must be connected)"
            import SI = Modelica.SIunits;
            import Modelica.Mechanics.MultiBody.Types.ResolveInFrameA;
            import Modelica.Mechanics.MultiBody.Frames;
            extends Modelica.Mechanics.MultiBody.Sensors.Internal.PartialCutForceBaseSensor;
            Modelica.Blocks.Interfaces.RealOutput torque[3](final quantity = "Torque", final unit = "N.m") "Cut torque resolved in frame defined by resolveInFrame" annotation(Placement(transformation(origin = { -80, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
            parameter Boolean positiveSign = true "= true, if torque with positive sign is returned (= frame_a.t), otherwise with negative sign (= frame_b.t)";
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -190, -70},{ -74, -96}}, lineColor = {0,0,0}, textString = "torque"),Line(points = {{ -80, -100},{ -80,0}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<HTML>

</HTML>"));
          protected 
            parameter Integer csign = if positiveSign then  +1 else  -1;

          equation 
            if resolveInFrame == ResolveInFrameA.world then
              torque = Frames.resolve1(frame_a.R, frame_a.t) * csign;

            elseif resolveInFrame == ResolveInFrameA.frame_a then
              torque = frame_a.t * csign;


            elseif resolveInFrame == ResolveInFrameA.frame_resolve then
              torque = Frames.resolveRelative(frame_a.t, frame_a.R, frame_resolve.R) * csign;


            else
              assert(false, "Wrong value for parameter resolveInFrame");
              torque = zeros(3);

            end if;
          end BasicCutTorque;
        end Internal;
      end Sensors;
      package Parts "Rigid components such as bodies with mass and inertia and massless rods"
        import SI = Modelica.SIunits;
        extends Modelica.Icons.Library;
        annotation(Documentation(info = "<HTML>
<p>
Package <b>Parts</b> contains <b>rigid components</b> of a
multi-body system. These components may be used to build up
more complicated structures. For example, a part may be built up of
a \"Body\" and of several \"FixedTranslation\" components.
</p>
<h4>Content</h4>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Model</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Parts.Fixed\">Fixed</a></td>
      <td valign=\"top\">Frame fixed in world frame at a given position.
          It is visualized with a shape, see <b>shapeType</b> below
         (the frames on the two
          sides do not belong to the component):<br>&nbsp;<br>
      <IMG SRC=\"../Images/MultiBody/Fixed.png\" ALT=\"model Parts.Fixed\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Parts.FixedTranslation\">FixedTranslation</a></td>
      <td valign=\"top\">Fixed translation of frame_b with respect to frame_a.
          It is visualized with a shape, see <b>shapeType</b> below
          (the frames on the two sides do not belong to the component):<br>&nbsp;<br>
      <IMG SRC=\"../Images/MultiBody/FixedTranslation.png\" ALT=\"model Parts.FixedTranslation\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Parts.FixedRotation\">FixedRotation</a></td>
      <td valign=\"top\">Fixed translation and fixed rotation of frame_b with respect to frame_a
          It is visualized with a shape, see <b>shapeType</b>  below
          (the frames on the two sides do not belong to the component):<br>&nbsp;<br>
      <IMG SRC=\"../Images/MultiBody/FixedRotation.png\" ALT=\"model Parts.FixedRotation\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Parts.Body\">Body</a></td>
      <td valign=\"top\">Rigid body with mass, inertia tensor and one frame connector.
          It is visualized with a cylinder and a sphere at the
          center of mass:<br>&nbsp;<br>
      <IMG SRC=\"../Images/MultiBody/Body.png\" ALT=\"model Parts.Body\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Parts.BodyShape\">BodyShape</a></td>
      <td valign=\"top\">Rigid body with mass, inertia tensor, different shapes
          (see <b>shapeType</b> below)
          for animation, and two frame connectors:<br>&nbsp;<br>
      <IMG SRC=\"../Images/MultiBody/BodyShape.png\" ALT=\"model Parts.BodyShape\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Parts.Fixed\">Fixed</a>
BodyBox</b></td>
      <td valign=\"top\">Rigid body with box shape (mass and animation properties are computed
          from box data and from density):<br>&nbsp;<br>
      <IMG SRC=\"../Images/MultiBody/BodyBox.png\" ALT=\"model Parts.BodyBox\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Parts.BodyCylinder\">BodyCylinder</a></td>
      <td valign=\"top\">Rigid body with cylinder shape (mass and animation properties
          are computed from cylinder data and from density):<br>&nbsp;<br>
      <IMG SRC=\"../Images/MultiBody/BodyCylinder.png\" ALT=\"model Parts.BodyCylinder\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Parts.PointMass\">PointMass</a></td>
      <td valign=\"top\">Rigid body where inertia tensor and rotation is neglected:<br>&nbsp;<br>
      <IMG SRC=\"../Images/MultiBody/Parts/PointMass.png\" ALT=\"model Parts.PointMass\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Parts.Mounting1D\">Mounting1D</a></td>
      <td valign=\"top\"> Propagate 1-dim. support torque to 3-dim. system
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Parts.Rotor1D\">Rotor1D</a></td>
      <td valign=\"top\">1D inertia attachable on 3-dim. bodies (without neglecting dynamic effects)<br>
      <IMG SRC=\"../Images/MultiBody/Parts/Rotor1D.png\" ALT=\"model Parts.Rotor1D\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Parts.BevelGear1D\">BevelGear1D</a></td>
      <td valign=\"top\">1D gearbox with arbitrary shaft directions (3D bearing frame)
      </td>
  </tr>
</table>
<p>
Components <b>Fixed</b>, <b>FixedTranslation</b>, <b>FixedRotation</b>
and <b>BodyShape</b> are visualized according to parameter
<b>shapeType</b>, that may have the following values (e.g., shapeType = \"box\"): <br>&nbsp;<br>
</p>
<IMG SRC=\"../Images/MultiBody/FixedShape.png\" ALT=\"model Visualizers.FixedShape\">
<p>
All the details of the visualization shape parameters are
given in
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Visualizers.FixedShape\">Visualizers.FixedShape</a>
</p>
<p>
Colors in all animation parts are defined via parameter <b>color</b>.
This is an Integer vector with 3 elements, {r, g, b}, and specifies the
color of the shape. {r,g,b} are the \"red\", \"green\" and \"blue\" color parts,
given in the ranges 0 .. 255, respectively. The predefined type
<b>MultiBody.Types.Color</b> contains a menu
definition of the colors used in the MultiBody library
(this will be replaced by a color editor).
</p>
</HTML>
"));
        model Fixed "Frame fixed in the world frame at a given position"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Types;
          Interfaces.Frame_b frame_b "Coordinate system fixed in the world frame" annotation(Placement(transformation(extent = {{84, -16},{116,16}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter SI.Position r[3] = {0,0,0} "Position vector from world frame to frame_b, resolved in world frame";
          parameter Types.ShapeType shapeType = "cylinder" " Type of shape" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Position r_shape[3] = {0,0,0} " Vector from world frame to shape origin, resolved in world frame" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Position lengthDirection[3] = r - r_shape " Vector in length direction of shape, resolved in world frame" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Position widthDirection[3] = {0,1,0} " Vector in width direction of shape, resolved in world frame" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Length length = Modelica.Math.Vectors.length(r - r_shape) " Length of shape" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance width = length / world.defaultWidthFraction " Width of shape" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance height = width " Height of shape" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Types.ShapeExtra extra = 0.0 " Additional parameter for cone, pipe etc. (see docu of Visualizers.Advanced.Shape)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor " Color of shape" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -90,90},{90, -90}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{150,170},{ -142,106}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{0,100},{0, -100}}, color = {0,0,0}),Line(points = {{0, -80},{ -100, -20}}, color = {0,0,0}),Line(points = {{0, -40},{ -100,20}}, color = {0,0,0}),Line(points = {{0,0},{ -100,60}}, color = {0,0,0}),Line(points = {{0,40},{ -100,100}}, color = {0,0,0}),Line(points = {{0,0},{100,0}}, color = {0,0,0}),Text(extent = {{ -146, -104},{154, -148}}, lineColor = {0,0,0}, textString = "r=%r")}), Documentation(info = "<html>
<p>
Element consisting of a frame (frame_b) that is fixed in the world
frame at a given position defined by parameter vector <b>r</b>
(vector from origin of world frame to frame_b, resolved in the
world frame).
</p>
<p>
By default, this component is visualized by a cylinder connecting the
world frame and frame_b of this components, as shown in the figure below.
Note, that the visualized world frame on the left side and
Fixed.frame_b on the right side are not part of the
component animation and that the animation may be switched off via parameter
animation = <b>false</b>.
</p>
<IMG SRC=\"../Images/MultiBody/Fixed.png\" ALT=\"Parts.Fixed\">
</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{0,100},{0, -100}}, color = {0,0,0}),Line(points = {{0, -80},{ -100, -20}}, color = {0,0,0}),Line(points = {{0, -40},{ -100,20}}, color = {0,0,0}),Line(points = {{0,0},{ -100,60}}, color = {0,0,0}),Line(points = {{0,40},{ -100,100}}, color = {0,0,0}),Line(points = {{0,0},{100,0}}, color = {0,0,0})}));
        protected 
          outer Modelica.Mechanics.MultiBody.World world;
          Visualizers.Advanced.Shape shape(shapeType = shapeType, color = color, specularCoefficient = specularCoefficient, length = length, width = width, height = height, lengthDirection = lengthDirection, widthDirection = widthDirection, extra = extra, r_shape = r_shape, r = zeros(3), R = Frames.nullRotation()) if world.enableAnimation and animation;

        equation 
          Connections.root(frame_b.R);
          frame_b.r_0 = r;
          frame_b.R = Frames.nullRotation();
        end Fixed;
        model FixedTranslation "Fixed translation of frame_b with respect to frame_a"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Types;
          Interfaces.Frame_a frame_a "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
          Interfaces.Frame_b frame_b "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{84, -16},{116,16}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter SI.Position r[3](start = {0,0,0}) "Vector from frame_a to frame_b resolved in frame_a";
          parameter Types.ShapeType shapeType = "cylinder" " Type of shape" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Position r_shape[3] = {0,0,0} " Vector from frame_a to shape origin, resolved in frame_a" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Types.Axis lengthDirection = r - r_shape " Vector in length direction of shape, resolved in frame_a" annotation(Evaluate = true, Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Types.Axis widthDirection = {0,1,0} " Vector in width direction of shape, resolved in frame_a" annotation(Evaluate = true, Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Length length = Modelica.Math.Vectors.length(r - r_shape) " Length of shape" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance width = length / world.defaultWidthFraction " Width of shape" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance height = width " Height of shape." annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Types.ShapeExtra extra = 0.0 " Additional parameter depending on shapeType (see docu of Visualizers.Advanced.Shape)." annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor " Color of shape" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -99,5},{101, -5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -131,101},{129,41}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{127, -72},{ -133, -22}}, lineColor = {0,0,0}, textString = "%=r"),Text(extent = {{ -89,38},{ -53,13}}, lineColor = {128,128,128}, textString = "a"),Text(extent = {{57,39},{93,14}}, lineColor = {128,128,128}, textString = "b")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,5},{100, -5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -95,20},{ -58,20}}, color = {128,128,128}, arrow = {Arrow.None,Arrow.Filled}),Line(points = {{ -94,18},{ -94,50}}, color = {128,128,128}, arrow = {Arrow.None,Arrow.Filled}),Text(extent = {{ -72,35},{ -58,24}}, lineColor = {128,128,128}, textString = "x"),Text(extent = {{ -113,57},{ -98,45}}, lineColor = {128,128,128}, textString = "y"),Line(points = {{ -100, -4},{ -100, -69}}, color = {128,128,128}),Line(points = {{ -100, -63},{90, -63}}, color = {128,128,128}),Text(extent = {{ -22, -39},{16, -63}}, lineColor = {128,128,128}, textString = "r"),Polygon(points = {{88, -59},{88, -68},{100, -63},{88, -59}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{100, -3},{100, -68}}, color = {128,128,128}),Line(points = {{69,20},{106,20}}, color = {128,128,128}, arrow = {Arrow.None,Arrow.Filled}),Line(points = {{70,18},{70,50}}, color = {128,128,128}, arrow = {Arrow.None,Arrow.Filled}),Text(extent = {{92,35},{106,24}}, lineColor = {128,128,128}, textString = "x"),Text(extent = {{51,57},{66,45}}, lineColor = {128,128,128}, textString = "y")}), Documentation(info = "<HTML>
<p>
Component for a <b>fixed translation</b> of frame_b with respect
to frame_a, i.e., the relationship between connectors frame_a and frame_b
remains constant and frame_a is always <b>parallel</b> to frame_b.
</p>
<p>
By default, this component is visualized by a cylinder connecting
frame_a and frame_b, as shown in the figure below. Note, that the
two visualized frames are not part of the component animation and that
the animation may be switched off via parameter animation = <b>false</b>.
</p>
<IMG SRC=\"../Images/MultiBody/FixedTranslation.png\" ALT=\"Parts.FixedTranslation\">
</HTML>"));
        protected 
          outer Modelica.Mechanics.MultiBody.World world;
          Visualizers.Advanced.Shape shape(shapeType = shapeType, color = color, specularCoefficient = specularCoefficient, r_shape = r_shape, lengthDirection = lengthDirection, widthDirection = widthDirection, length = length, width = width, height = height, extra = extra, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;

        equation 
          Connections.branch(frame_a.R, frame_b.R);
          assert(cardinality(frame_a) > 0 or cardinality(frame_b) > 0, "Neither connector frame_a nor frame_b of FixedTranslation object is connected");
          frame_b.r_0 = frame_a.r_0 + Frames.resolve1(frame_a.R, r);
          frame_b.R = frame_a.R;
          zeros(3) = frame_a.f + frame_b.f;
          zeros(3) = frame_a.t + frame_b.t + cross(r, frame_b.f);
        end FixedTranslation;
        model FixedRotation "Fixed translation followed by a fixed rotation of frame_b with respect to frame_a"
          import Modelica.Mechanics.MultiBody.Frames;
          import Modelica.Mechanics.MultiBody.Types;
          import SI = Modelica.SIunits;
          import Cv = Modelica.SIunits.Conversions;
          Interfaces.Frame_a frame_a "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
          Interfaces.Frame_b frame_b "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{84, -16},{116,16}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter SI.Position r[3] = {0,0,0} "Vector from frame_a to frame_b resolved in frame_a";
          parameter Modelica.Mechanics.MultiBody.Types.RotationTypes rotationType = Modelica.Mechanics.MultiBody.Types.RotationTypes.RotationAxis "Type of rotation description" annotation(Evaluate = true);
          parameter Types.Axis n = {1,0,0} " Axis of rotation in frame_a (= same as in frame_b)" annotation(Evaluate = true, Dialog(group = "if rotationType = RotationAxis", enable = rotationType == Modelica.Mechanics.MultiBody.Types.RotationTypes.RotationAxis));
          parameter Cv.NonSIunits.Angle_deg angle = 0 " Angle to rotate frame_a around axis n into frame_b" annotation(Dialog(group = "if rotationType = RotationAxis", enable = rotationType == Modelica.Mechanics.MultiBody.Types.RotationTypes.RotationAxis));
          parameter Types.Axis n_x = {1,0,0} " Vector along x-axis of frame_b resolved in frame_a" annotation(Evaluate = true, Dialog(group = "if rotationType = TwoAxesVectors", enable = rotationType == Types.RotationTypes.TwoAxesVectors));
          parameter Types.Axis n_y = {0,1,0} " Vector along y-axis of frame_b resolved in frame_a" annotation(Evaluate = true, Dialog(group = "if rotationType = TwoAxesVectors", enable = rotationType == Types.RotationTypes.TwoAxesVectors));
          parameter Types.RotationSequence sequence(min = {1,1,1}, max = {3,3,3}) = {1,2,3} " Sequence of rotations" annotation(Evaluate = true, Dialog(group = "if rotationType = PlanarRotationSequence", enable = rotationType == Types.RotationTypes.PlanarRotationSequence));
          parameter Cv.NonSIunits.Angle_deg angles[3] = {0,0,0} " Rotation angles around the axes defined in 'sequence'" annotation(Dialog(group = "if rotationType = PlanarRotationSequence", enable = rotationType == Types.RotationTypes.PlanarRotationSequence));
          parameter Types.ShapeType shapeType = "cylinder" " Type of shape" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Position r_shape[3] = {0,0,0} " Vector from frame_a to shape origin, resolved in frame_a" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Types.Axis lengthDirection = r - r_shape " Vector in length direction of shape, resolved in frame_a" annotation(Evaluate = true, Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Types.Axis widthDirection = {0,1,0} " Vector in width direction of shape, resolved in frame_a" annotation(Evaluate = true, Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Length length = Modelica.Math.Vectors.length(r - r_shape) " Length of shape" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance width = length / world.defaultWidthFraction " Width of shape" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance height = width " Height of shape." annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Types.ShapeExtra extra = 0.0 " Additional parameter depending on shapeType (see docu of Visualizers.Advanced.Shape)." annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor " Color of shape" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          final parameter Frames.Orientation R_rel = if rotationType == 1 then Frames.planarRotation(Modelica.Math.Vectors.normalize(n), Cv.from_deg(angle), 0) else if rotationType == 2 then Frames.from_nxy(n_x, n_y) else Frames.axesRotations(sequence, Cv.from_deg(angles), zeros(3)) "Fixed rotation object from frame_a to frame_b";
        protected 
          outer Modelica.Mechanics.MultiBody.World world;
          parameter Frames.Orientation R_rel_inv = Frames.from_T(transpose(R_rel.T), zeros(3)) "Inverse of R_rel (rotate from frame_b to frame_a)";
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape shape(shapeType = shapeType, color = color, specularCoefficient = specularCoefficient, r_shape = r_shape, lengthDirection = lengthDirection, widthDirection = widthDirection, length = length, width = width, height = height, extra = extra, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          annotation(Documentation(info = "<HTML>
<p>
Component for a <b>fixed translation</b> and <b>fixed rotation</b> of frame_b with respect
to frame_a, i.e., the relationship between connectors frame_a and frame_b
remains constant. There are several possibilities to define the
orientation of frame_b with respect to frame_a:
</p>
<ul>
<li><b>Planar rotation</b> along axis 'n' (that is fixed and resolved
    in frame_a) with a fixed angle 'angle'.</li>
<li><b>Vectors n_x</b> and <b>n_y</b> that are directed along the corresponding axes
    direction of frame_b and are resolved in frame_a (if n_y is not
    orthogonal to n_x, the y-axis of frame_b is selected such that it is
    orthogonal to n_x and in the plane of n_x and n_y).</li>
<li><b>Sequence</b> of <b>three planar axes rotations</b>.
    For example, \"sequence = {1,2,3}\" and \"angles = {90, 45, -90}\"
    means to rotate frame_a around the x axis with 90 degrees, around the new
    y axis with 45 degrees and around the new z axis around -90 degrees to
    arrive at frame_b. Note, that sequence={1,2,3}
    is the Cardan angle sequence and sequence = {3,1,3} is the Euler angle
    sequence.</li>
</ul>
<p>
By default, this component is visualized by a cylinder connecting
frame_a and frame_b, as shown in the figure below. In this figure
frame_b is rotated along the z-axis of frame_a with 60 degree. Note, that the
two visualized frames are not part of the component animation and that
the animation may be switched off via parameter animation = <b>false</b>.
</p>
<IMG SRC=\"../Images/MultiBody/FixedRotation.png\" ALT=\"Parts.FixedRotation\">
</HTML>", revisions = "<HTML><p><b>Release Notes:</b></p>
<ul>
  <li><i>July 28, 2003</i><br>
         Bug fixed: if rotationType = PlanarRotationSequence, then 'angles'
         was used with unit [rad] instead of [deg].</li>
</ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -136,79},{132,139}}, textString = "%name", lineColor = {0,0,255}),Rectangle(extent = {{ -100,5},{100, -4}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{80,20},{129,50}}, color = {0,0,0}),Line(points = {{80,20},{57,59}}, color = {0,0,0}),Polygon(points = {{144,60},{117,59},{132,37},{144,60}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{43,80},{46,50},{68,65},{43,80}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -144, -52},{143, -89}}, lineColor = {0,0,0}, textString = "r=%r"),Text(extent = {{ -117,51},{ -81,26}}, lineColor = {128,128,128}, textString = "a"),Text(extent = {{84, -24},{120, -49}}, lineColor = {128,128,128}, textString = "b")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -100, -1},{ -100, -66}}, color = {128,128,128}),Line(points = {{100,0},{100, -65}}, color = {128,128,128}),Line(points = {{ -100, -60},{89, -60}}, color = {128,128,128}),Text(extent = {{ -22, -36},{16, -60}}, lineColor = {128,128,128}, textString = "r"),Rectangle(extent = {{ -100,5},{100, -5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{69,29},{97,45}}, color = {128,128,128}, arrow = {Arrow.None,Arrow.Filled}),Line(points = {{70,27},{55,54}}, color = {128,128,128}, arrow = {Arrow.None,Arrow.Filled}),Text(extent = {{95,42},{109,31}}, lineColor = {128,128,128}, textString = "x"),Text(extent = {{42,70},{57,58}}, lineColor = {128,128,128}, textString = "y"),Line(points = {{ -95,22},{ -58,22}}, color = {128,128,128}, arrow = {Arrow.None,Arrow.Filled}),Line(points = {{ -94,20},{ -94,52}}, color = {128,128,128}, arrow = {Arrow.None,Arrow.Filled}),Text(extent = {{ -72,37},{ -58,26}}, lineColor = {128,128,128}, textString = "x"),Text(extent = {{ -113,59},{ -98,47}}, lineColor = {128,128,128}, textString = "y"),Polygon(points = {{88, -56},{88, -65},{100, -60},{88, -56}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}));

        equation 
          Connections.branch(frame_a.R, frame_b.R);
          assert(cardinality(frame_a) > 0 or cardinality(frame_b) > 0, "Neither connector frame_a nor frame_b of FixedRotation object is connected");
          frame_b.r_0 = frame_a.r_0 + Frames.resolve1(frame_a.R, r);
          if rooted(frame_a.R) then
            frame_b.R = Frames.absoluteRotation(frame_a.R, R_rel);
            zeros(3) = frame_a.f + Frames.resolve1(R_rel, frame_b.f);
            zeros(3) = frame_a.t + Frames.resolve1(R_rel, frame_b.t) - cross(r, frame_a.f);

          else
            frame_a.R = Frames.absoluteRotation(frame_b.R, R_rel_inv);
            zeros(3) = frame_b.f + Frames.resolve1(R_rel_inv, frame_a.f);
            zeros(3) = frame_b.t + Frames.resolve1(R_rel_inv, frame_a.t) + cross(Frames.resolve1(R_rel_inv, r), frame_b.f);

          end if;
        end FixedRotation;
        model Body "Rigid body with mass, inertia tensor and one frame connector (12 potential states)"
          import SI = Modelica.SIunits;
          import C = Modelica.Constants;
          import Modelica.Math.*;
          import Modelica.Mechanics.MultiBody.Types;
          import Modelica.Mechanics.MultiBody.Frames;
          Interfaces.Frame_a frame_a "Coordinate system fixed at body" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled (show cylinder and sphere)";
          parameter SI.Position r_CM[3](start = {0,0,0}) "Vector from frame_a to center of mass, resolved in frame_a";
          parameter SI.Mass m(min = 0, start = 1) "Mass of rigid body";
          parameter SI.Inertia I_11(min = 0) = 0.001 " (1,1) element of inertia tensor" annotation(Dialog(group = "Inertia tensor (resolved in center of mass, parallel to frame_a)"));
          parameter SI.Inertia I_22(min = 0) = 0.001 " (2,2) element of inertia tensor" annotation(Dialog(group = "Inertia tensor (resolved in center of mass, parallel to frame_a)"));
          parameter SI.Inertia I_33(min = 0) = 0.001 " (3,3) element of inertia tensor" annotation(Dialog(group = "Inertia tensor (resolved in center of mass, parallel to frame_a)"));
          parameter SI.Inertia I_21(min =  -C.inf) = 0 " (2,1) element of inertia tensor" annotation(Dialog(group = "Inertia tensor (resolved in center of mass, parallel to frame_a)"));
          parameter SI.Inertia I_31(min =  -C.inf) = 0 " (3,1) element of inertia tensor" annotation(Dialog(group = "Inertia tensor (resolved in center of mass, parallel to frame_a)"));
          parameter SI.Inertia I_32(min =  -C.inf) = 0 " (3,2) element of inertia tensor" annotation(Dialog(group = "Inertia tensor (resolved in center of mass, parallel to frame_a)"));
          SI.Position r_0[3](start = {0,0,0}, stateSelect = if enforceStates then StateSelect.always else StateSelect.avoid) "Position vector from origin of world frame to origin of frame_a" annotation(Dialog(tab = "Initialization", __Dymola_initialDialog = true));
          SI.Velocity v_0[3](start = {0,0,0}, stateSelect = if enforceStates then StateSelect.always else StateSelect.avoid) "Absolute velocity of frame_a, resolved in world frame (= der(r_0))" annotation(Dialog(tab = "Initialization", __Dymola_initialDialog = true));
          SI.Acceleration a_0[3](start = {0,0,0}) "Absolute acceleration of frame_a resolved in world frame (= der(v_0))" annotation(Dialog(tab = "Initialization", __Dymola_initialDialog = true));
          parameter Boolean angles_fixed = false "= true, if angles_start are used as initial values, else as guess values" annotation(Evaluate = true, choices(__Dymola_checkBox = true), Dialog(tab = "Initialization"));
          parameter SI.Angle angles_start[3] = {0,0,0} "Initial values of angles to rotate frame_a around 'sequence_start' axes into frame_b" annotation(Dialog(tab = "Initialization"));
          parameter Types.RotationSequence sequence_start = {1,2,3} "Sequence of rotations to rotate frame_a into frame_b at initial time" annotation(Evaluate = true, Dialog(tab = "Initialization"));
          parameter Boolean w_0_fixed = false "= true, if w_0_start are used as initial values, else as guess values" annotation(Evaluate = true, choices(__Dymola_checkBox = true), Dialog(tab = "Initialization"));
          parameter SI.AngularVelocity w_0_start[3] = {0,0,0} "Initial or guess values of angular velocity of frame_a resolved in world frame" annotation(Dialog(tab = "Initialization"));
          parameter Boolean z_0_fixed = false "= true, if z_0_start are used as initial values, else as guess values" annotation(Evaluate = true, choices(__Dymola_checkBox = true), Dialog(tab = "Initialization"));
          parameter SI.AngularAcceleration z_0_start[3] = {0,0,0} "Initial values of angular acceleration z_0 = der(w_0)" annotation(Dialog(tab = "Initialization"));
          parameter SI.Diameter sphereDiameter = world.defaultBodyDiameter "Diameter of sphere" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color sphereColor = Modelica.Mechanics.MultiBody.Types.Defaults.BodyColor "Color of sphere" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Diameter cylinderDiameter = sphereDiameter / Types.Defaults.BodyCylinderDiameterFraction "Diameter of cylinder" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color cylinderColor = sphereColor "Color of cylinder" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Boolean enforceStates = false " = true, if absolute variables of body object shall be used as states (StateSelect.always)" annotation(Evaluate = true, Dialog(tab = "Advanced"));
          parameter Boolean useQuaternions = true " = true, if quaternions shall be used as potential states otherwise use 3 angles as potential states" annotation(Evaluate = true, Dialog(tab = "Advanced"));
          parameter Types.RotationSequence sequence_angleStates = {1,2,3} " Sequence of rotations to rotate world frame into frame_a around the 3 angles used as potential states" annotation(Evaluate = true, Dialog(tab = "Advanced", enable = not useQuaternions));
          final parameter SI.Inertia I[3,3] = [I_11,I_21,I_31;I_21,I_22,I_32;I_31,I_32,I_33] "inertia tensor";
          final parameter Frames.Orientation R_start = Modelica.Mechanics.MultiBody.Frames.axesRotations(sequence_start, angles_start, zeros(3)) "Orientation object from world frame to frame_a at initial time";
          final parameter SI.AngularAcceleration z_a_start[3] = Frames.resolve2(R_start, z_0_start) "Initial values of angular acceleration z_a = der(w_a), i.e., time derivative of angular velocity resolved in frame_a";
          SI.AngularVelocity w_a[3](start = Frames.resolve2(R_start, w_0_start), fixed = fill(w_0_fixed, 3), stateSelect = if enforceStates then if useQuaternions then StateSelect.always else StateSelect.never else StateSelect.avoid) "Absolute angular velocity of frame_a resolved in frame_a";
          SI.AngularAcceleration z_a[3](start = Frames.resolve2(R_start, z_0_start), fixed = fill(z_0_fixed, 3)) "Absolute angular acceleration of frame_a resolved in frame_a";
          SI.Acceleration g_0[3] "Gravity acceleration resolved in world frame";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,30},{ -3, -31}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,127,255}),Text(extent = {{131, -123},{ -129, -73}}, lineColor = {0,0,0}, textString = "m=%m"),Text(extent = {{ -128,132},{132,72}}, textString = "%name", lineColor = {0,0,255}),Ellipse(extent = {{ -20,60},{100, -60}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {0,127,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Documentation(info = "<HTML>
<p>
<b>Rigid body</b> with mass and inertia tensor.
All parameter vectors have to be resolved in frame_a.
The <b>inertia tensor</b> has to be defined with respect to a
coordinate system that is parallel to frame_a with the
origin at the center of mass of the body.
</p>
<p>
By default, this component is visualized by a <b>cylinder</b> located
between frame_a and the center of mass and by a <b>sphere</b> that has
its center at the center of mass. If the cylinder length is smaller as
the radius of the sphere, e.g., since frame_a is located at the
center of mass, the cylinder is not displayed. Note, that
the animation may be switched off via parameter animation = <b>false</b>.
</p>
<IMG SRC=\"../Images/MultiBody/Body.png\" ALT=\"Parts.Body\">
<p>
<b>States of Body Components</b>
</p>
<p>
Every body has potential states. If possible a tool will select
the states of joints and not the states of bodies because this is
usually the most efficient choice. In this case the position, orientation,
velocity and angular velocity of frame_a of the body will be computed
by the component that is connected to frame_a. However, if a body is moving
freely in space, variables of the body have to be used as states. The potential
states of the body are:
</p>
<ul>
<li> The <b>position vector</b> frame_a.r_0 from the origin of the
     world frame to the origin of frame_a of the body, resolved in
     the world frame and the <b>absolute velocity</b> v_0 of the origin of
     frame_a, resolved in the world frame (= der(frame_a.r_0)).</li>
</li>
<li> If parameter <b>useQuaternions</b> in the \"Advanced\" menu
     is <b>true</b> (this is the default), then <b>4 quaternions</b>
     are potential states. Additionally, the coordinates of the
     absolute angular velocity vector of the
     body are 3 potential states.<br>
     If <b>useQuaternions</b> in the \"Advanced\" menu
     is <b>false</b>, then <b>3 angles</b> and the derivatives of
     these angles are potential states. The orientation of frame_a
     is computed by rotating the world frame along the axes defined
     in parameter vector \"sequence_angleStates\" (default = {1,2,3}, i.e.,
     the Cardan angle sequence) around the angles used as potential states.
     For example, the default is to rotate the x-axis of the world frame
     around angles[1], the new y-axis around angles[2] and the new z-axis
     around angles[3], arriving at frame_a.
 </li>
</ul>
<p>
The quaternions have the slight disadvantage that there is a
non-linear constraint equation between the 4 quaternions.
Therefore, at least one non-linear equation has to be solved
during simulation. A tool might, however, analytically solve this
simple constraint equation. Using the 3 angles as states has the
disadvantage that there is a singular configuration in which a
division by zero will occur. If it is possible to determine in advance
for an application class that this singular configuration is outside
of the operating region, the 3 angles might be used as potential
states by setting <b>useQuaternions</b> = <b>false</b>.
</p>
<p>
In text books about 3-dimensional mechanics often 3 angles and the
angular velocity are used as states. This is not the case here, since
3 angles and their derivatives are used as potential states
(if useQuaternions = false). The reason
is that for real-time simulation the discretization formula of the
integrator might be \"inlined\" and solved together with the body equations.
By appropriate symbolic transformation the performance is
drastically increased if angles and their
derivatives are used as states, instead of angles and the angular
velocity.
</p>
<p>
Whether or not variables of the body are used as states is usually
automatically selected by the Modelica translator. If parameter
<b>enforceStates</b> is set to <b>true</b> in the \"Advanced\" menu,
then body variables are forced to be used as states according
to the setting of parameters \"useQuaternions\" and
\"sequence_angleStates\".
</p>
</HTML>"));
        protected 
          outer Modelica.Mechanics.MultiBody.World world;
          parameter Frames.Quaternions.Orientation Q_start = Frames.to_Q(R_start) "Quaternion orientation object from world frame to frame_a at initial time";
          Frames.Quaternions.Orientation Q(start = Q_start, stateSelect = if enforceStates then if useQuaternions then StateSelect.prefer else StateSelect.never else StateSelect.avoid) "Quaternion orientation object from world frame to frame_a (dummy value, if quaternions are not used as states)";
          parameter SI.Angle phi_start[3] = if sequence_start[1] == sequence_angleStates[1] and sequence_start[2] == sequence_angleStates[2] and sequence_start[3] == sequence_angleStates[3] then angles_start else Frames.axesRotationsAngles(R_start, sequence_angleStates) "Potential angle states at initial time";
          SI.Angle phi[3](start = phi_start, stateSelect = if enforceStates then if useQuaternions then StateSelect.never else StateSelect.always else StateSelect.avoid) "Dummy or 3 angles to rotate world frame into frame_a of body";
          SI.AngularVelocity phi_d[3](stateSelect = if enforceStates then if useQuaternions then StateSelect.never else StateSelect.always else StateSelect.avoid) "= der(phi)";
          SI.AngularAcceleration phi_dd[3] "= der(phi_d)";
          Visualizers.Advanced.Shape cylinder(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = if Modelica.Math.Vectors.length(r_CM) > sphereDiameter / 2 then Modelica.Math.Vectors.length(r_CM) - (if cylinderDiameter > 1.1 * sphereDiameter then sphereDiameter / 2 else 0) else 0, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = r_CM, widthDirection = {0,1,0}, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          Visualizers.Advanced.Shape sphere(shapeType = "sphere", color = sphereColor, specularCoefficient = specularCoefficient, length = sphereDiameter, width = sphereDiameter, height = sphereDiameter, lengthDirection = {1,0,0}, widthDirection = {0,1,0}, r_shape = r_CM - ({1,0,0} * sphereDiameter) / 2, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation and sphereDiameter > 0;

        initial equation 
if angles_fixed then
                      if not Connections.isRoot(frame_a.R) then
              zeros(3) = Frames.Orientation.equalityConstraint(frame_a.R, R_start);

            elseif useQuaternions then
              zeros(3) = Frames.Quaternions.Orientation.equalityConstraint(Q, Q_start);


            else
              phi = phi_start;

            end if;
          end if;

        equation 
          if enforceStates then
            Connections.root(frame_a.R);

          else
            Connections.potentialRoot(frame_a.R);

          end if;
          r_0 = frame_a.r_0;
          if not Connections.isRoot(frame_a.R) then
            Q = {0,0,0,1};
            phi = zeros(3);
            phi_d = zeros(3);
            phi_dd = zeros(3);

          elseif useQuaternions then
            frame_a.R = Frames.from_Q(Q, Frames.Quaternions.angularVelocity2(Q, der(Q)));
            {0} = Frames.Quaternions.orientationConstraint(Q);
            phi = zeros(3);
            phi_d = zeros(3);
            phi_dd = zeros(3);


          else
            phi_d = der(phi);
            phi_dd = der(phi_d);
            frame_a.R = Frames.axesRotations(sequence_angleStates, phi, phi_d);
            Q = {0,0,0,1};

          end if;
          g_0 = world.gravityAcceleration(frame_a.r_0 + Frames.resolve1(frame_a.R, r_CM));
          v_0 = der(frame_a.r_0);
          a_0 = der(v_0);
          w_a = Frames.angularVelocity2(frame_a.R);
          z_a = der(w_a);
          frame_a.f = m * (Frames.resolve2(frame_a.R, a_0 - g_0) + cross(z_a, r_CM) + cross(w_a, cross(w_a, r_CM)));
          frame_a.t = I * z_a + cross(w_a, I * w_a) + cross(r_CM, frame_a.f);
        end Body;
        model BodyShape "Rigid body with mass, inertia tensor, different shapes for animation, and two frame connectors (12 potential states)"
          import SI = Modelica.SIunits;
          import C = Modelica.Constants;
          import Modelica.Mechanics.MultiBody.Types;
          Interfaces.Frame_a frame_a "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
          Interfaces.Frame_b frame_b "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{84, -16},{116,16}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled (show shape between frame_a and frame_b and optionally a sphere at the center of mass)";
          parameter Boolean animateSphere = true "= true, if mass shall be animated as sphere provided animation=true";
          parameter SI.Position r[3](start = {0,0,0}) "Vector from frame_a to frame_b resolved in frame_a";
          parameter SI.Position r_CM[3](start = {0,0,0}) "Vector from frame_a to center of mass, resolved in frame_a";
          parameter SI.Mass m(min = 0, start = 1) "Mass of rigid body";
          parameter SI.Inertia I_11(min = 0) = 0.001 " (1,1) element of inertia tensor" annotation(Dialog(group = "Inertia tensor (resolved in center of mass, parallel to frame_a)"));
          parameter SI.Inertia I_22(min = 0) = 0.001 " (2,2) element of inertia tensor" annotation(Dialog(group = "Inertia tensor (resolved in center of mass, parallel to frame_a)"));
          parameter SI.Inertia I_33(min = 0) = 0.001 " (3,3) element of inertia tensor" annotation(Dialog(group = "Inertia tensor (resolved in center of mass, parallel to frame_a)"));
          parameter SI.Inertia I_21(min =  -C.inf) = 0 " (2,1) element of inertia tensor" annotation(Dialog(group = "Inertia tensor (resolved in center of mass, parallel to frame_a)"));
          parameter SI.Inertia I_31(min =  -C.inf) = 0 " (3,1) element of inertia tensor" annotation(Dialog(group = "Inertia tensor (resolved in center of mass, parallel to frame_a)"));
          parameter SI.Inertia I_32(min =  -C.inf) = 0 " (3,2) element of inertia tensor" annotation(Dialog(group = "Inertia tensor (resolved in center of mass, parallel to frame_a)"));
          SI.Position r_0[3](start = {0,0,0}, stateSelect = if enforceStates then StateSelect.always else StateSelect.avoid) "Position vector from origin of world frame to origin of frame_a" annotation(Dialog(tab = "Initialization", __Dymola_initialDialog = true));
          SI.Velocity v_0[3](start = {0,0,0}, stateSelect = if enforceStates then StateSelect.always else StateSelect.avoid) "Absolute velocity of frame_a, resolved in world frame (= der(r_0))" annotation(Dialog(tab = "Initialization", __Dymola_initialDialog = true));
          SI.Acceleration a_0[3](start = {0,0,0}) "Absolute acceleration of frame_a resolved in world frame (= der(v_0))" annotation(Dialog(tab = "Initialization", __Dymola_initialDialog = true));
          parameter Boolean angles_fixed = false "= true, if angles_start are used as initial values, else as guess values" annotation(Evaluate = true, choices(__Dymola_checkBox = true), Dialog(tab = "Initialization"));
          parameter SI.Angle angles_start[3] = {0,0,0} "Initial values of angles to rotate frame_a around 'sequence_start' axes into frame_b" annotation(Dialog(tab = "Initialization"));
          parameter Types.RotationSequence sequence_start = {1,2,3} "Sequence of rotations to rotate frame_a into frame_b at initial time" annotation(Evaluate = true, Dialog(tab = "Initialization"));
          parameter Boolean w_0_fixed = false "= true, if w_0_start are used as initial values, else as guess values" annotation(Evaluate = true, choices(__Dymola_checkBox = true), Dialog(tab = "Initialization"));
          parameter SI.AngularVelocity w_0_start[3] = {0,0,0} "Initial or guess values of angular velocity of frame_a resolved in world frame" annotation(Dialog(tab = "Initialization"));
          parameter Boolean z_0_fixed = false "= true, if z_0_start are used as initial values, else as guess values" annotation(Evaluate = true, choices(__Dymola_checkBox = true), Dialog(tab = "Initialization"));
          parameter SI.AngularAcceleration z_0_start[3] = {0,0,0} "Initial values of angular acceleration z_0 = der(w_0)" annotation(Dialog(tab = "Initialization"));
          parameter Types.ShapeType shapeType = "cylinder" " Type of shape" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Position r_shape[3] = {0,0,0} " Vector from frame_a to shape origin, resolved in frame_a" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Types.Axis lengthDirection = r - r_shape " Vector in length direction of shape, resolved in frame_a" annotation(Evaluate = true, Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Types.Axis widthDirection = {0,1,0} " Vector in width direction of shape, resolved in frame_a" annotation(Evaluate = true, Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Length length = Modelica.Math.Vectors.length(r - r_shape) " Length of shape" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance width = length / world.defaultWidthFraction " Width of shape" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance height = width " Height of shape." annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Types.ShapeExtra extra = 0.0 " Additional parameter depending on shapeType (see docu of Visualizers.Advanced.Shape)." annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.BodyColor " Color of shape" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Diameter sphereDiameter = 2 * width " Diameter of sphere" annotation(Dialog(tab = "Animation", group = "if animation = true and animateSphere = true", enable = animation and animateSphere));
          input Types.Color sphereColor = color " Color of sphere of mass" annotation(Dialog(tab = "Animation", group = "if animation = true and animateSphere = true", enable = animation and animateSphere));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Boolean enforceStates = false " = true, if absolute variables of body object shall be used as states (StateSelect.always)" annotation(Dialog(tab = "Advanced"));
          parameter Boolean useQuaternions = true " = true, if quaternions shall be used as potential states otherwise use 3 angles as potential states" annotation(Dialog(tab = "Advanced"));
          parameter Types.RotationSequence sequence_angleStates = {1,2,3} " Sequence of rotations to rotate world frame into frame_a around the 3 angles used as potential states" annotation(Evaluate = true, Dialog(tab = "Advanced", enable = not useQuaternions));
          annotation(Documentation(info = "<HTML>
<p>
<b>Rigid body</b> with mass and inertia tensor and <b>two frame connectors</b>.
All parameter vectors have to be resolved in frame_a.
The <b>inertia tensor</b> has to be defined with respect to a
coordinate system that is parallel to frame_a with the
origin at the center of mass of the body. The coordinate system <b>frame_b</b>
is always parallel to <b>frame_a</b>.
</p>
<p>
By default, this component is visualized by any <b>shape</b> that can be
defined with Modelica.Mechanics.MultiBody.Visualizers.FixedShape. This shape is placed
between frame_a and frame_b (default: length(shape) = Frames.length(r)).
Additionally a <b>sphere</b> may be visualized that has
its center at the center of mass.
Note, that
the animation may be switched off via parameter animation = <b>false</b>.
</p>
<IMG SRC=\"../Images/MultiBody/BodyShape.png\" ALT=\"Parts.BodyShape\">
<p>
The following shapes can be defined via parameter <b>shapeType</b>,
e.g., shapeType=\"cone\":
</p>
<IMG SRC=\"../Images/MultiBody/FixedShape.png\" ALT=\"Visualizers.FixedShape\">
<p>
A BodyShape component has potential states. For details of these
states and of the \"Advanced\" menu parameters, see model
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Parts.Body\">MultiBody.Parts.Body</a>.
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -134,126},{126,66}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{122, -123},{ -138, -73}}, lineColor = {0,0,0}, textString = "%=r"),Rectangle(extent = {{ -100,31},{101, -30}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,127,255}),Ellipse(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {0,127,255}),Text(extent = {{ -50,24},{55, -27}}, lineColor = {0,0,0}, textString = "%m"),Text(extent = {{55,12},{91, -13}}, lineColor = {0,0,0}, textString = "b"),Text(extent = {{ -92,13},{ -56, -12}}, lineColor = {0,0,0}, textString = "a")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -100,9},{ -100,43}}, color = {128,128,128}),Line(points = {{100,0},{100,44}}, color = {128,128,128}),Line(points = {{ -100,40},{90,40}}, color = {135,135,135}),Polygon(points = {{90,44},{90,36},{100,40},{90,44}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{ -22,68},{20,40}}, lineColor = {128,128,128}, textString = "r"),Line(points = {{ -100, -10},{ -100, -90}}, color = {128,128,128}),Line(points = {{ -100, -84},{ -10, -84}}, color = {128,128,128}),Polygon(points = {{ -10, -80},{ -10, -88},{0, -84},{ -10, -80}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{ -82, -66},{ -56, -84}}, lineColor = {128,128,128}, textString = "r_CM"),Line(points = {{0, -46},{0, -90}}, color = {128,128,128})}));
          FixedTranslation frameTranslation(r = r, animation = false) annotation(Placement(transformation(extent = {{ -20, -20},{20,20}}, rotation = 0)));
          Body body(r_CM = r_CM, m = m, I_11 = I_11, I_22 = I_22, I_33 = I_33, I_21 = I_21, I_31 = I_31, I_32 = I_32, animation = false, sequence_start = sequence_start, angles_fixed = angles_fixed, angles_start = angles_start, w_0_fixed = w_0_fixed, w_0_start = w_0_start, z_0_fixed = z_0_fixed, z_0_start = z_0_start, useQuaternions = useQuaternions, enforceStates = enforceStates, sequence_angleStates = sequence_angleStates) annotation(Placement(transformation(extent = {{ -27.3333, -70.3333},{13, -30}}, rotation = 0)));
        protected 
          outer Modelica.Mechanics.MultiBody.World world;
          Visualizers.Advanced.Shape shape1(shapeType = shapeType, color = color, specularCoefficient = specularCoefficient, length = length, width = width, height = height, lengthDirection = lengthDirection, widthDirection = widthDirection, r_shape = r_shape, extra = extra, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          Visualizers.Advanced.Shape shape2(shapeType = "sphere", color = sphereColor, specularCoefficient = specularCoefficient, length = sphereDiameter, width = sphereDiameter, height = sphereDiameter, lengthDirection = {1,0,0}, widthDirection = {0,1,0}, r_shape = r_CM - ({1,0,0} * sphereDiameter) / 2, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation and animateSphere;

        equation 
          r_0 = frame_a.r_0;
          v_0 = der(r_0);
          a_0 = der(v_0);
          connect(frame_a,frameTranslation.frame_a) annotation(Line(points = {{ -100,0},{ -20,0}}, color = {95,95,95}, thickness = 0.5));
          connect(frame_b,frameTranslation.frame_b) annotation(Line(points = {{100,0},{20,0}}, color = {95,95,95}, thickness = 0.5));
          connect(frame_a,body.frame_a) annotation(Line(points = {{ -100,0},{ -60,0},{ -60, -50.1666},{ -27.3333, -50.1666}}, color = {95,95,95}, thickness = 0.5));
        end BodyShape;
        model BodyBox "Rigid body with box shape. Mass and animation properties are computed from box data and density (12 potential states)"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Types;
          Interfaces.Frame_a frame_a "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
          Interfaces.Frame_b frame_b "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{84, -16},{116,16}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled (show box between frame_a and frame_b)";
          parameter SI.Position r[3](start = {0.1,0,0}) "Vector from frame_a to frame_b resolved in frame_a";
          parameter SI.Position r_shape[3] = {0,0,0} "Vector from frame_a to box origin, resolved in frame_a";
          parameter Modelica.Mechanics.MultiBody.Types.Axis lengthDirection = r - r_shape "Vector in length direction of box, resolved in frame_a" annotation(Evaluate = true);
          parameter Modelica.Mechanics.MultiBody.Types.Axis widthDirection = {0,1,0} "Vector in width direction of box, resolved in frame_a" annotation(Evaluate = true);
          parameter SI.Length length = Modelica.Math.Vectors.length(r - r_shape) "Length of box";
          parameter SI.Distance width = length / world.defaultWidthFraction "Width of box";
          parameter SI.Distance height = width "Height of box";
          parameter SI.Distance innerWidth = 0 "Width of inner box surface (0 <= innerWidth <= width)";
          parameter SI.Distance innerHeight = innerWidth "Height of inner box surface (0 <= innerHeight <= height)";
          parameter SI.Density density = 7700 "Density of cylinder (e.g., steel: 7700 .. 7900, wood : 400 .. 800)";
          input Modelica.Mechanics.MultiBody.Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.BodyColor "Color of box" annotation(Dialog(enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(enable = animation));
          SI.Position r_0[3](start = {0,0,0}, stateSelect = if enforceStates then StateSelect.always else StateSelect.avoid) "Position vector from origin of world frame to origin of frame_a" annotation(Dialog(tab = "Initialization", __Dymola_initialDialog = true));
          SI.Velocity v_0[3](start = {0,0,0}, stateSelect = if enforceStates then StateSelect.always else StateSelect.avoid) "Absolute velocity of frame_a, resolved in world frame (= der(r_0))" annotation(Dialog(tab = "Initialization", __Dymola_initialDialog = true));
          SI.Acceleration a_0[3](start = {0,0,0}) "Absolute acceleration of frame_a resolved in world frame (= der(v_0))" annotation(Dialog(tab = "Initialization", __Dymola_initialDialog = true));
          parameter Boolean angles_fixed = false "= true, if angles_start are used as initial values, else as guess values" annotation(Evaluate = true, choices(__Dymola_checkBox = true), Dialog(tab = "Initialization"));
          parameter SI.Angle angles_start[3] = {0,0,0} "Initial values of angles to rotate frame_a around 'sequence_start' axes into frame_b" annotation(Dialog(tab = "Initialization"));
          parameter Types.RotationSequence sequence_start = {1,2,3} "Sequence of rotations to rotate frame_a into frame_b at initial time" annotation(Evaluate = true, Dialog(tab = "Initialization"));
          parameter Boolean w_0_fixed = false "= true, if w_0_start are used as initial values, else as guess values" annotation(Evaluate = true, choices(__Dymola_checkBox = true), Dialog(tab = "Initialization"));
          parameter SI.AngularVelocity w_0_start[3] = {0,0,0} "Initial or guess values of angular velocity of frame_a resolved in world frame" annotation(Dialog(tab = "Initialization"));
          parameter Boolean z_0_fixed = false "= true, if z_0_start are used as initial values, else as guess values" annotation(Evaluate = true, choices(__Dymola_checkBox = true), Dialog(tab = "Initialization"));
          parameter SI.AngularAcceleration z_0_start[3] = {0,0,0} "Initial values of angular acceleration z_0 = der(w_0)" annotation(Dialog(tab = "Initialization"));
          parameter Boolean enforceStates = false " = true, if absolute variables of body object shall be used as states (StateSelect.always)" annotation(Dialog(tab = "Advanced"));
          parameter Boolean useQuaternions = true " = true, if quaternions shall be used as potential states otherwise use 3 angles as potential states" annotation(Dialog(tab = "Advanced"));
          parameter Types.RotationSequence sequence_angleStates = {1,2,3} " Sequence of rotations to rotate world frame into frame_a around the 3 angles used as potential states" annotation(Evaluate = true, Dialog(tab = "Advanced", enable = not useQuaternions));
          final parameter SI.Mass mo(min = 0) = density * length * width * height "Mass of box without hole";
          final parameter SI.Mass mi(min = 0) = density * length * innerWidth * innerHeight "Mass of hole of box";
          final parameter SI.Mass m(min = 0) = mo - mi "Mass of box";
          final parameter Frames.Orientation R = Frames.from_nxy(r, widthDirection) "Orientation object from frame_a to coordinates system spanned by r and widthDirection";
          final parameter SI.Position r_CM[3] = (Modelica.Math.Vectors.normalize(r) * length) / 2 "Position vector from origin of frame_a to center of mass, resolved in frame_a";
          final parameter SI.Inertia I[3,3] = Frames.resolveDyade1(R, diagonal({mo * (width * width + height * height) - mi * (innerWidth * innerWidth + innerHeight * innerHeight),mo * (length * length + height * height) - mi * (length * length + innerHeight * innerHeight),mo * (length * length + width * width) - mi * (length * length + innerWidth * innerWidth)} / 12)) "Inertia tensor of body box with respect to center of mass, parallel to frame_a";
          Body body(animation = false, r_CM = r_CM, m = m, I_11 = I[1,1], I_22 = I[2,2], I_33 = I[3,3], I_21 = I[2,1], I_31 = I[3,1], I_32 = I[3,2], sequence_start = sequence_start, angles_fixed = angles_fixed, angles_start = angles_start, w_0_fixed = w_0_fixed, w_0_start = w_0_start, z_0_fixed = z_0_fixed, z_0_start = z_0_start, useQuaternions = useQuaternions, enforceStates = enforceStates, sequence_angleStates = sequence_angleStates) annotation(Placement(transformation(extent = {{ -30, -80},{10, -40}}, rotation = 0)));
          FixedTranslation frameTranslation(r = r, animation = animation, shapeType = "box", r_shape = r_shape, lengthDirection = lengthDirection, widthDirection = widthDirection, length = length, width = width, height = height, color = color, specularCoefficient = specularCoefficient) annotation(Placement(transformation(extent = {{ -30, -20},{10,20}}, rotation = 0)));
          annotation(Documentation(info = "<HTML>
<p>
<b>Rigid body</b> with <b>box</b> shape.
The mass properties of the body (mass, center of mass,
inertia tensor) are computed
from the box data. Optionally, the box may be hollow.
The (outer) box shape is by default used in the animation.
The hollow part is not shown in the animation.
The two connector frames <b>frame_a</b> and <b>frame_b</b>
are always parallel to each other. Example of component
animation (note, that
the animation may be switched off via parameter animation = <b>false</b>):
</p>
<IMG SRC=\"../Images/MultiBody/BodyBox.png\" ALT=\"Parts.BodyBox\">
<p>
A BodyBox component has potential states. For details of these
states and of the \"Advanced\" menu parameters, see model
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Parts.Body\">MultiBody.Parts.Body</a>.</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{100,40},{100, -30},{90, -40},{90,30},{100,40}}, lineColor = {0,95,191}, fillColor = {0,95,191}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -100,30},{90, -40}}, lineColor = {0,127,255}, pattern = LinePattern.None, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -100,30},{ -90,40},{100,40},{90,30},{ -100,30}}, lineColor = {0,95,191}, fillColor = {0,95,191}, fillPattern = FillPattern.Solid),Text(extent = {{ -129,100},{131,40}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{130, -98},{ -130, -48}}, lineColor = {0,0,0}, textString = "%=r"),Text(extent = {{52,8},{88, -17}}, lineColor = {0,0,0}, textString = "b"),Text(extent = {{ -87,12},{ -51, -13}}, lineColor = {0,0,0}, textString = "a")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
        protected 
          outer Modelica.Mechanics.MultiBody.World world;

        equation 
          r_0 = frame_a.r_0;
          v_0 = der(r_0);
          a_0 = der(v_0);
          assert(innerWidth <= width, "parameter innerWidth is greater as parameter width");
          assert(innerHeight <= height, "parameter innerHeight is greater as paraemter height");
          connect(frameTranslation.frame_a,frame_a) annotation(Line(points = {{ -30,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5));
          connect(frameTranslation.frame_b,frame_b) annotation(Line(points = {{10,0},{100,0}}, color = {95,95,95}, thickness = 0.5));
          connect(frame_a,body.frame_a) annotation(Line(points = {{ -100,0},{ -70,0},{ -70, -60},{ -30, -60}}, color = {95,95,95}, thickness = 0.5));
        end BodyBox;
        model BodyCylinder "Rigid body with cylinder shape. Mass and animation properties are computed from cylinder data and density (12 potential states)"
          import SI = Modelica.SIunits;
          import NonSI = Modelica.SIunits.Conversions.NonSIunits;
          import Modelica.Mechanics.MultiBody.Types;
          Interfaces.Frame_a frame_a "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
          Interfaces.Frame_b frame_b "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{84, -16},{116,16}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled (show cylinder between frame_a and frame_b)";
          parameter SI.Position r[3](start = {0.1,0,0}) "Vector from frame_a to frame_b, resolved in frame_a";
          parameter SI.Position r_shape[3] = {0,0,0} "Vector from frame_a to cylinder origin, resolved in frame_a";
          parameter Modelica.Mechanics.MultiBody.Types.Axis lengthDirection = r - r_shape "Vector in length direction of cylinder, resolved in frame_a" annotation(Evaluate = true);
          parameter SI.Length length = Modelica.Math.Vectors.length(r - r_shape) "Length of cylinder";
          parameter SI.Distance diameter = length / world.defaultWidthFraction "Diameter of cylinder";
          parameter SI.Distance innerDiameter = 0 "Inner diameter of cylinder (0 <= innerDiameter <= Diameter)";
          parameter SI.Density density = 7700 "Density of cylinder (e.g., steel: 7700 .. 7900, wood : 400 .. 800)";
          input Modelica.Mechanics.MultiBody.Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.BodyColor "Color of cylinder" annotation(Dialog(enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(enable = animation));
          SI.Position r_0[3](start = {0,0,0}, stateSelect = if enforceStates then StateSelect.always else StateSelect.avoid) "Position vector from origin of world frame to origin of frame_a" annotation(Dialog(tab = "Initialization", __Dymola_initialDialog = true));
          SI.Velocity v_0[3](start = {0,0,0}, stateSelect = if enforceStates then StateSelect.always else StateSelect.avoid) "Absolute velocity of frame_a, resolved in world frame (= der(r_0))" annotation(Dialog(tab = "Initialization", __Dymola_initialDialog = true));
          SI.Acceleration a_0[3](start = {0,0,0}) "Absolute acceleration of frame_a resolved in world frame (= der(v_0))" annotation(Dialog(tab = "Initialization", __Dymola_initialDialog = true));
          parameter Boolean angles_fixed = false "= true, if angles_start are used as initial values, else as guess values" annotation(Evaluate = true, choices(__Dymola_checkBox = true), Dialog(tab = "Initialization"));
          parameter SI.Angle angles_start[3] = {0,0,0} "Initial values of angles to rotate frame_a around 'sequence_start' axes into frame_b" annotation(Dialog(tab = "Initialization"));
          parameter Types.RotationSequence sequence_start = {1,2,3} "Sequence of rotations to rotate frame_a into frame_b at initial time" annotation(Evaluate = true, Dialog(tab = "Initialization"));
          parameter Boolean w_0_fixed = false "= true, if w_0_start are used as initial values, else as guess values" annotation(Evaluate = true, choices(__Dymola_checkBox = true), Dialog(tab = "Initialization"));
          parameter SI.AngularVelocity w_0_start[3] = {0,0,0} "Initial or guess values of angular velocity of frame_a resolved in world frame" annotation(Dialog(tab = "Initialization"));
          parameter Boolean z_0_fixed = false "= true, if z_0_start are used as initial values, else as guess values" annotation(Evaluate = true, choices(__Dymola_checkBox = true), Dialog(tab = "Initialization"));
          parameter SI.AngularAcceleration z_0_start[3] = {0,0,0} "Initial values of angular acceleration z_0 = der(w_0)" annotation(Dialog(tab = "Initialization"));
          parameter Boolean enforceStates = false " = true, if absolute variables of body object shall be used as states (StateSelect.always)" annotation(Dialog(tab = "Advanced"));
          parameter Boolean useQuaternions = true " = true, if quaternions shall be used as potential states otherwise use 3 angles as potential states" annotation(Dialog(tab = "Advanced"));
          parameter Types.RotationSequence sequence_angleStates = {1,2,3} " Sequence of rotations to rotate world frame into frame_a around the 3 angles used as potential states" annotation(Evaluate = true, Dialog(tab = "Advanced", enable = not useQuaternions));
          constant Real pi = Modelica.Constants.pi;
          final parameter SI.Distance radius = diameter / 2 "Radius of cylinder";
          final parameter SI.Distance innerRadius = innerDiameter / 2 "Inner-Radius of cylinder";
          final parameter SI.Mass mo(min = 0) = density * pi * length * radius * radius "Mass of cylinder without hole";
          final parameter SI.Mass mi(min = 0) = density * pi * length * innerRadius * innerRadius "Mass of hole of cylinder";
          final parameter SI.Inertia I22 = (mo * (length * length + 3 * radius * radius) - mi * (length * length + 3 * innerRadius * innerRadius)) / 12 "Inertia with respect to axis through center of mass, perpendicular to cylinder axis";
          final parameter SI.Mass m(min = 0) = mo - mi "Mass of cylinder";
          final parameter Frames.Orientation R = Frames.from_nxy(r, {0,1,0}) "Orientation object from frame_a to frame spanned by cylinder axis and axis perpendicular to cylinder axis";
          final parameter SI.Position r_CM[3] = (Modelica.Math.Vectors.normalize(r) * length) / 2 "Position vector from frame_a to center of mass, resolved in frame_a";
          final parameter SI.Inertia I[3,3] = Frames.resolveDyade1(R, diagonal({(mo * radius * radius - mi * innerRadius * innerRadius) / 2,I22,I22})) "Inertia tensor of cylinder with respect to center of mass, resolved in frame parallel to frame_a";
          Body body(r_CM = r_CM, m = m, I_11 = I[1,1], I_22 = I[2,2], I_33 = I[3,3], I_21 = I[2,1], I_31 = I[3,1], I_32 = I[3,2], animation = false, sequence_start = sequence_start, angles_fixed = angles_fixed, angles_start = angles_start, w_0_fixed = w_0_fixed, w_0_start = w_0_start, z_0_fixed = z_0_fixed, z_0_start = z_0_start, useQuaternions = useQuaternions, enforceStates = enforceStates, sequence_angleStates = sequence_angleStates) annotation(Placement(transformation(extent = {{ -30, -80},{10, -40}}, rotation = 0)));
          FixedTranslation frameTranslation(r = r, animation = animation, shapeType = "pipecylinder", r_shape = r_shape, lengthDirection = lengthDirection, length = length, width = diameter, height = diameter, extra = innerDiameter / diameter, color = color, specularCoefficient = specularCoefficient, widthDirection = {0,1,0}) annotation(Placement(transformation(extent = {{ -30, -20},{10,20}}, rotation = 0)));
          annotation(Documentation(info = "<HTML>
<p>
<b>Rigid body</b> with <b>cylinder</b> shape.
The mass properties of the body (mass, center of mass,
inertia tensor) are computed
from the cylinder data. Optionally, the cylinder may be hollow.
The cylinder shape is by default used in the animation.
The two connector frames <b>frame_a</b> and <b>frame_b</b>
are always parallel to each other. Example of component
animation (note, that
the animation may be switched off via parameter animation = <b>false</b>):
</p>
<IMG SRC=\"../Images/MultiBody/BodyCylinder.png\" ALT=\"Parts.BodyCylinder\">
<p>
A BodyCylinder component has potential states. For details of these
states and of the \"Advanced\" menu parameters, see model
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Parts.Body\">MultiBody.Parts.Body</a>.</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -129,100},{131,40}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{130, -98},{ -130, -48}}, lineColor = {0,0,0}, textString = "%=r"),Rectangle(extent = {{ -100,40},{100, -40}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,127,255}),Text(extent = {{ -87,13},{ -51, -12}}, lineColor = {0,0,0}, textString = "a"),Text(extent = {{51,12},{87, -13}}, lineColor = {0,0,0}, textString = "b")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
        protected 
          outer Modelica.Mechanics.MultiBody.World world;

        equation 
          r_0 = frame_a.r_0;
          v_0 = der(r_0);
          a_0 = der(v_0);
          assert(innerDiameter < diameter, "parameter innerDiameter is greater as parameter diameter.");
          connect(frameTranslation.frame_a,frame_a) annotation(Line(points = {{ -30,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5));
          connect(frameTranslation.frame_b,frame_b) annotation(Line(points = {{10,0},{100,0}}, color = {95,95,95}, thickness = 0.5));
          connect(frame_a,body.frame_a) annotation(Line(points = {{ -100,0},{ -70,0},{ -70, -60},{ -30, -60}}, color = {95,95,95}, thickness = 0.5));
        end BodyCylinder;
        model PointMass "Rigid body where body rotation and inertia tensor is neglected (6 potential states)"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Types;
          Interfaces.Frame_a frame_a "Coordinate system fixed at center of mass point" annotation(Placement(transformation(extent = {{ -16, -16},{16,16}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled (show sphere)";
          parameter SI.Mass m(min = 0) "Mass of mass point";
          input SI.Diameter sphereDiameter = world.defaultBodyDiameter "Diameter of sphere" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color sphereColor = Modelica.Mechanics.MultiBody.Types.Defaults.BodyColor "Color of sphere" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter StateSelect stateSelect = StateSelect.avoid "Priority to use frame_a.r_0, v_0 (= der(frame_a.r_0)) as states" annotation(Dialog(tab = "Advanced"));
          SI.Position r_0[3](start = {0,0,0}, stateSelect = stateSelect) "Position vector from origin of world frame to origin of frame_a, resolved in world frame" annotation(Dialog(group = "Initialization", __Dymola_initialDialog = true));
          SI.Velocity v_0[3](start = {0,0,0}, stateSelect = stateSelect) "Absolute velocity of frame_a, resolved in world frame (= der(r_0))" annotation(Dialog(group = "Initialization", __Dymola_initialDialog = true));
          SI.Acceleration a_0[3](start = {0,0,0}) "Absolute acceleration of frame_a resolved in world frame (= der(v_0))" annotation(Dialog(group = "Initialization", __Dymola_initialDialog = true));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{132, -102},{ -129, -56}}, lineColor = {0,0,0}, textString = "m=%m"),Text(extent = {{ -128,110},{132,55}}, textString = "%name", lineColor = {0,0,255}),Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {0,127,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Documentation(info = "<HTML>
<p>
<b>Rigid body</b> where the inertia tensor is neglected.
This body is
solely defined by its mass.
By default, this component is visualized by a <b>sphere</b> that has
its center at frame_a. Note, that
the animation may be switched off via parameter animation = <b>false</b>.
</p>

<p>
Every PointMass has potential states. If possible a tool will select
the states of joints and not the states of PointMasss because this is
usually the most efficient choice. In this case the position and
velocity of frame_a of the body will be computed
by the component that is connected to frame_a. However, if a PointMass is moving
freely in space, variables of the PointMass have to be used as states. The potential
states are: The <b>position vector</b> frame_a.r_0 from the origin of the
world frame to the origin of frame_a of the body, resolved in
the world frame and the <b>absolute velocity</b> v_0 of the origin of
frame_a, resolved in the world frame (= der(frame_a.r_0)).
</p>

<p>
Whether or not variables of the body are used as states is usually
automatically selected by the Modelica translator. If parameter
<b>enforceStates</b> is set to <b>true</b> in the \"Advanced\" menu,
then PointMass variables frame_a.r_0 and der(frame_a.r_0)
are forced to be used as states.
</p>
</HTML>"));
        protected 
          outer Modelica.Mechanics.MultiBody.World world;
          Visualizers.Advanced.Shape sphere(shapeType = "sphere", color = sphereColor, specularCoefficient = specularCoefficient, length = sphereDiameter, width = sphereDiameter, height = sphereDiameter, lengthDirection = {1,0,0}, widthDirection = {0,1,0}, r_shape =  -({1,0,0} * sphereDiameter) / 2, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;

        equation 
          Connections.potentialRoot(frame_a.R, 10000);
          if Connections.isRoot(frame_a.R) then
            assert(cardinality(frame_a) == 0, "
A Modelica.Mechanics.MultiBody.Parts.PointMass model is connected in
a way, so that no equations are present to compute frame_a.R
(the orientation object in the connector). Setting frame_a.R to
an arbitrary value in the PointMass model, might lead to a wrong
overall model, depending on how the PointMass model is used.
   You can avoid this message, by providing equations that
compute the orientation object, e.g., by using the
Modelica.Mechanics.MultiBody.Joints.FreeMotion joint.
   If a PointMass model is not connected at all, the
orientation object is set to a unit rotation. But this is
the only case where this is done.
");
            frame_a.R = Frames.nullRotation();

          else
            frame_a.t = zeros(3);

          end if;
          r_0 = frame_a.r_0;
          v_0 = der(r_0);
          a_0 = der(v_0);
          frame_a.f = m * Frames.resolve2(frame_a.R, a_0 - world.gravityAcceleration(r_0));
        end PointMass;
        model Mounting1D "Propagate 1-dim. support torque to 3-dim. system (provided world.driveTrainMechanics3D=true)"
          parameter Modelica.SIunits.Angle phi0 = 0 "Fixed offset angle of housing";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n = {1,0,0} "Axis of rotation = axis of support torque (resolved in frame_a)";
          Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_b "(right) flange fixed in housing" annotation(Placement(transformation(extent = {{110,10},{90, -10}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a if world.driveTrainMechanics3D "Frame in which housing is fixed (connector is removed, if world.driveTrainMechanics3D=false)" annotation(Placement(transformation(origin = {0, -100}, extent = {{ -20, -20},{20,20}}, rotation = 90)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -80, -60},{80, -100}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -146,60},{154,20}}, lineColor = {0,0,255}, textString = "%name"),Line(points = {{80, -60},{40, -100}}, color = {0,0,0}),Line(points = {{40, -60},{0, -100}}, color = {0,0,0}),Line(points = {{0, -60},{ -40, -100}}, color = {0,0,0}),Line(points = {{ -40, -60},{ -80, -100}}, color = {0,0,0}),Line(points = {{0, -60},{0,0}}, color = {0,0,0}),Line(points = {{0,0},{90,0}}, color = {0,0,0})}), Documentation(info = "<html>
<p>This component is used to acquire support torques from a 1-dim.-rotational
mechanical system (e.g., components from Modelica.Mechanics.Rotational)
and to propagate them to a carrier body.</p>
<p>The 1-dim. support torque at <tt>flange_b</tt> is transformed into 3-dim. space under
consideration of the rotation axis, parameter <tt>n</tt>, which has to be given in the
local coordinate system of <tt>frame_a</tt>.</p>
<p>All components of a 1-dim.-rotational mechanical system that are connected to <b>a</b> common
<b>Mounting1D</b> element need to have the same axis of rotation
along parameter vector <tt>n</tt>. This means that, e.g., bevel
gears where the axis of rotation of <tt>flange_a</tt> and
<tt>flange_b</tt> are different cannot be described properly by
connecting to the <b>Mounting1D</b> component. In this case, a combination of several
<b>Mounting1D</b> components or the component <b>BevelGear1D</b> should be used.</p>
<p><b>Reference</b><br>
<span style=\"font-variant:small-caps\">Schweiger</span>, Christian ;
<span style=\"font-variant:small-caps\">Otter</span>, Martin:
<a href=\"http://www.modelica.org/Conference2003/papers/h06_Schweiger_powertrains_v5.pdf\">Modelling
3D Mechanical Effects of 1-dim. Powertrains</a>. In: <i>Proceedings of the 3rd International
Modelica Conference</i>. Link&ouml;ping : The Modelica Association and Link&ouml;ping University,
November 3-4, 2003, pp. 149-158</p>
</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
        protected 
          outer Modelica.Mechanics.MultiBody.World world;
          encapsulated model Housing
            import Modelica;
            input Modelica.SIunits.Torque t[3];
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,110},{150,150}}, lineColor = {0,0,255}, textString = "%name")}));

          equation 
            frame_a.f = zeros(3);
            frame_a.t = t;
          end Housing;
          Housing housing(t =  -n * flange_b.tau) if world.driveTrainMechanics3D annotation(Placement(transformation(extent = {{20, -60},{40, -40}}, rotation = 0)));

        equation 
          flange_b.phi = phi0;
          connect(housing.frame_a,frame_a) annotation(Line(points = {{20, -50},{0, -50},{0, -100}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        end Mounting1D;
        model Rotor1D "1D inertia attachable on 3-dim. bodies (3D dynamic effects are taken into account if world.driveTrainMechanics3D=true)"
          import SI = Modelica.SIunits;
          import Cv = Modelica.SIunits.Conversions;
          parameter Boolean animation = true "= true, if animation shall be enabled (show rotor as cylinder)";
          parameter SI.Inertia J(min = 0, start = 1) "Moment of inertia of rotor around its axis of rotation";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n = {1,0,0} "Axis of rotation resolved in frame_a";
          parameter SI.Position r_center[3] = zeros(3) "Position vector from origin of frame_a to center of cylinder" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance cylinderLength = 2 * world.defaultJointLength "Length of cylinder representing the rotor" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance cylinderDiameter = 2 * world.defaultJointWidth "Diameter of cylinder representing the rotor" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Modelica.Mechanics.MultiBody.Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor "Color of cylinder representing the rotor" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Modelica.Mechanics.MultiBody.Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter StateSelect stateSelect = StateSelect.default "Priority to use rotor angle (phi) and rotor speed (w) as states" annotation(Dialog(tab = "Advanced"));
          parameter Boolean exact = true "= true, if exact calculations; false if influence of bearing on rotor acceleration is neglected to avoid an algebraic loop" annotation(Dialog(tab = "Advanced"));
          SI.Angle phi(start = 0, stateSelect = stateSelect) "Rotation angle of rotor with respect to frame_a (= flange_a.phi = flange_b.phi)";
          SI.AngularVelocity w(start = 0, stateSelect = stateSelect) "Angular velocity of rotor with respect to frame_a";
          SI.AngularAcceleration a(start = 0) "Angular acceleration of rotor with respect to frame_a";
          Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a "(left) driving flange (flange axis directed INTO cut plane)" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_b "(right) driven flange (flange axis directed OUT OF cut plane)" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a if world.driveTrainMechanics3D "Frame in which rotor housing is fixed (connector is removed, if world.driveTrainMechanics3D=false)" annotation(Placement(transformation(origin = {0, -100}, extent = {{ -20, -20},{20,20}}, rotation = 90)));
          annotation(Documentation(info = "<html>
<p>This component is used to model the gyroscopic torques exerted by a 1-dim.
inertia (so called <i>rotor</i>) on its 3-dim. carrier body. Gyroscopic torques
appear, if the vector of the carrier body's angular velocity is not parallel
to the vector of the rotor's axis. The axis of rotation of the rotor is defined by
the parameter <tt>n</tt>, which has to be given in the local coordinate system
of <tt>frame_a</tt>. The default animation of this component is
shown in the figure below. </p>
      <IMG SRC=\"../Images/MultiBody/Parts/Rotor1D.png\" ALT=\"model Parts.Rotor1D\">
<p>This component is a replacement for
<a href=\"Modelica://Modelica.Mechanics.Rotational.Components.Inertia\">Modelica.Mechanics.Rotational.Components.Inertia</a>
for the case, that a 1-dim.-rotational mechanical system should be attached with a 3-dim.
carrier body.</p>
<p>The Boolean parameter <tt>exact</tt> was introduced due to performance
reasons. If <tt>exact</tt> is set to <b>false</b>, the influence of the carrier body
motion on the angular velocity of the rotor is neglected. This influence is usually
negligible if the 1-dim.-rotational mechanical system accelerates much faster as the base body (this is,
e.g., the case in vehicle powertrains). The essential advantage is
that an algebraic loop is removed since then there is only an
action on acceleration level from the powertrain to the base body
but not vice versa.</p>
<p><b>Reference</b><br>
<span style=\"font-variant:small-caps\">Schweiger</span>, Christian ;
<span style=\"font-variant:small-caps\">Otter</span>, Martin:
<a href=\"http://www.modelica.org/Conference2003/papers/h06_Schweiger_powertrains_v5.pdf\">Modelling
3D Mechanical Effects of 1-dim. Powertrains</a>. In: <i>Proceedings of the 3rd International
Modelica Conference</i>. Link&ouml;ping : The Modelica Association and Link&ouml;ping University,
November 3-4, 2003, pp. 149-158</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -80, -25},{ -60, -25}}, color = {0,0,0}),Line(points = {{60, -25},{80, -25}}, color = {0,0,0}),Line(points = {{ -70, -25},{ -70, -70}}, color = {0,0,0}),Line(points = {{70, -25},{70, -70}}, color = {0,0,0}),Line(points = {{ -80,25},{ -60,25}}, color = {0,0,0}),Line(points = {{60,25},{80,25}}, color = {0,0,0}),Line(points = {{ -70,45},{ -70,25}}, color = {0,0,0}),Line(points = {{70,45},{70,25}}, color = {0,0,0}),Line(points = {{ -70, -70},{70, -70}}, color = {0,0,0}),Rectangle(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -100,10},{ -50, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{50,10},{100, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Text(extent = {{ -148,112},{152,72}}, textString = "%name=%J", lineColor = {0,0,255}),Line(points = {{0, -70},{0, -100}}, color = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          encapsulated model RotorWith3DEffects "1D inertia attachable on 3-dim. bodies (3D dynamic effects are taken into account)"
            import Modelica;
            import Modelica.Mechanics.MultiBody.Frames;
            import Modelica.Mechanics.MultiBody.Types;
            import SI = Modelica.SIunits;
            import Cv = Modelica.SIunits.Conversions;
            parameter Boolean animation = true "= true, if animation shall be enabled (show rotor as cylinder)";
            parameter SI.Inertia J(min = 0) = 1 "Moment of inertia of rotor around its axis of rotation";
            parameter Types.Axis n = {1,0,0} "Axis of rotation resolved in frame_a";
            parameter SI.Position r_center[3] = zeros(3) "Position vector from origin of frame_a to center of cylinder" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance cylinderLength = 2 * world.defaultJointLength "Length of cylinder representing the rotor" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance cylinderDiameter = 2 * world.defaultJointWidth "Diameter of cylinder representing the rotor" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor "Color of cylinder representing the rotor" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter StateSelect stateSelect = StateSelect.default "Priority to use rotor angle (phi) and rotor speed (w) as states" annotation(Dialog(tab = "Advanced"));
            parameter Boolean exact = true "= true, if exact calculations; false if influence of bearing on rotor acceleration is neglected to avoid an algebraic loop" annotation(Dialog(tab = "Advanced"));
            SI.AngularVelocity w_a[3] "Angular velocity of frame_a, resolved in frame_a";
            SI.Angle phi(start = 0, final stateSelect = stateSelect) "Rotation angle of rotor with respect to frame_a (= flange_a.phi = flange_b.phi)" annotation(Dialog(__Dymola_initialDialog = true));
            SI.AngularVelocity w(start = 0, stateSelect = stateSelect) "Angular velocity of rotor with respect to frame_a" annotation(Dialog(__Dymola_initialDialog = true));
            SI.AngularAcceleration a(start = 0) "Angular acceleration of rotor with respect to frame_a" annotation(Dialog(__Dymola_initialDialog = true));
            Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a "(left) driving flange (flange axis directed INTO cut plane)" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_b "(right) driven flange (flange axis directed OUT OF cut plane)" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Frame in which rotor housing is fixed" annotation(Placement(transformation(origin = {0, -100}, extent = {{ -20, -20},{20,20}}, rotation = 90)));
            annotation(Documentation(info = "<html>
<p>
This component is used to model the gyroscopic torques exerted by a 1-dim.
inertia (so called <i>rotor</i>) on its 3-dim. carrier body. Gyroscopic torques
appear, if the vector of the carrier body's angular velocity is not parallel
to the vector of the rotor's axis. The axis of rotation of the rotor is defined by
the parameter <tt>n</tt>, which has to be given in the local coordinate system
of <tt>frame_a</tt>. The default animation of this component is
shown in the figure below. </p>
      <IMG SRC=\"../Images/MultiBody/Parts/Rotor1D.png\" ALT=\"model Parts.Rotor1D\">
<p>This component is a replacement for
<a href=\"Modelica://Modelica.Mechanics.Rotational.Components.Inertia\">Modelica.Mechanics.Rotational.Components.Inertia</a>
for the case, that a 1-dim.-rotational mechanical system should be attached with a 3-dim.
carrier body.</p>
<p>The Boolean parameter <tt>exact</tt> was introduced due to performance
reasons. If <tt>exact</tt> is set to <b>false</b>, the influence of the carrier body
motion on the angular velocity of the rotor is neglected. This influence is usually
negligible if the 1-dim.-rotational mechanical system accelerates much faster as the base body (this is,
e.g., the case in vehicle powertrains). The essential advantage is
that an algebraic loop is removed since then there is only an
action on acceleration level from the powertrain to the base body
but not vice versa.</p>
<p><b>Reference</b><br>
<span style=\"font-variant:small-caps\">Schweiger</span>, Christian ;
<span style=\"font-variant:small-caps\">Otter</span>, Martin:
<a href=\"http://www.modelica.org/Conference2003/papers/h06_Schweiger_powertrains_v5.pdf\">Modelling
3D Mechanical Effects of 1-dim. Powertrains</a>. In: <i>Proceedings of the 3rd International
Modelica Conference</i>. Link&ouml;ping : The Modelica Association and Link&ouml;ping University,
November 3-4, 2003, pp. 149-158</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -80, -25},{ -60, -25}}, color = {0,0,0}),Line(points = {{60, -25},{80, -25}}, color = {0,0,0}),Line(points = {{ -70, -25},{ -70, -70}}, color = {0,0,0}),Line(points = {{70, -25},{70, -70}}, color = {0,0,0}),Line(points = {{ -80,25},{ -60,25}}, color = {0,0,0}),Line(points = {{60,25},{80,25}}, color = {0,0,0}),Line(points = {{ -70,45},{ -70,25}}, color = {0,0,0}),Line(points = {{70,45},{70,25}}, color = {0,0,0}),Line(points = {{ -70, -70},{70, -70}}, color = {0,0,0}),Rectangle(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -100,10},{ -50, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{50,10},{100, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Text(extent = {{ -148,112},{152,72}}, textString = "%name=%J", lineColor = {0,0,255}),Line(points = {{0, -70},{0, -100}}, color = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          protected 
            outer Modelica.Mechanics.MultiBody.World world;
            parameter Real e[3](each final unit = "1") = Modelica.Math.Vectors.normalize(n) "Unit vector in direction of rotor axis, resolved in frame_a";
            parameter SI.Inertia nJ[3] = J * e;
            Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape cylinder(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = n, widthDirection = {0,1,0}, extra = 1, r_shape = r_center - e * cylinderLength / 2, r = frame_a.r_0, R = Frames.absoluteRotation(frame_a.R, Frames.planarRotation(e, phi, 0))) if world.enableAnimation and animation;

          equation 
            phi = flange_a.phi;
            phi = flange_b.phi;
            w = der(phi);
            a = der(w);
            w_a = Modelica.Mechanics.MultiBody.Frames.angularVelocity2(frame_a.R);
            if exact then
              J * a = flange_a.tau + flange_b.tau - nJ * der(w_a);

            else
              J * a = flange_a.tau + flange_b.tau;

            end if;
            frame_a.f = zeros(3);
            frame_a.t = cross(w_a, nJ * w) - e * nJ * der(w_a);
          end RotorWith3DEffects;
        protected 
          outer Modelica.Mechanics.MultiBody.World world;
          parameter Real e[3](each final unit = "1") = Modelica.Math.Vectors.normalize(n) "Unit vector in direction of rotor axis, resolved in frame_a";
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape cylinder(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = n, widthDirection = {0,1,0}, extra = 1, r_shape = r_center - e * cylinderLength / 2, r = zeros(3), R = Modelica.Mechanics.MultiBody.Frames.planarRotation(e, phi, 0)) if world.enableAnimation and animation and not world.driveTrainMechanics3D;
          Modelica.Mechanics.Rotational.Components.Inertia inertia(J = J) if not world.driveTrainMechanics3D annotation(Placement(transformation(extent = {{ -20, -20},{20,20}}, rotation = 0)));
          RotorWith3DEffects rotorWith3DEffects(animation = animation, J = J, n = n, r_center = r_center, cylinderLength = cylinderLength, cylinderDiameter = cylinderDiameter, cylinderColor = cylinderColor, specularCoefficient = specularCoefficient, exact = exact) if world.driveTrainMechanics3D annotation(Placement(transformation(extent = {{ -20, -80},{20, -40}}, rotation = 0)));

        equation 
          phi = flange_a.phi;
          w = der(phi);
          a = der(w);
          connect(inertia.flange_a,flange_a) annotation(Line(points = {{ -20,0},{ -100,0}}, color = {0,0,0}, smooth = Smooth.None));
          connect(inertia.flange_b,flange_b) annotation(Line(points = {{20,0},{100,0}}, color = {0,0,0}, smooth = Smooth.None));
          connect(rotorWith3DEffects.flange_b,flange_b) annotation(Line(points = {{20, -60},{60, -60},{60,0},{100,0}}, color = {0,0,0}, smooth = Smooth.None));
          connect(rotorWith3DEffects.flange_a,flange_a) annotation(Line(points = {{ -20, -60},{ -60, -60},{ -60,0},{ -100,0}}, color = {0,0,0}, smooth = Smooth.None));
          connect(rotorWith3DEffects.frame_a,frame_a) annotation(Line(points = {{0, -80},{0, -100}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        end Rotor1D;
        model BevelGear1D "1D gearbox with arbitrary shaft directions and 3-dim. bearing frame (3D dynamic effects are taken into account provided world.driveTrainMechanics3D=true)"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialTwoFlanges;
          parameter Real ratio(start = 1) "Gear speed ratio";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n_a = {1,0,0} "Axis of rotation of flange_a, resolved in frame_a";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n_b = {1,0,0} "Axis of rotation of flange_b, resolved in frame_a";
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a if world.driveTrainMechanics3D "Bearing frame" annotation(Placement(transformation(origin = {0, -100}, extent = {{ -20, -20},{20,20}}, rotation = 90)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -40,20},{ -20, -20}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -40,140},{ -20,20}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{20,100},{40,60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{20,60},{40, -60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{40,10},{100, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -20,90},{20,70}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -100,10},{ -40, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Text(extent = {{ -150,178},{150,138}}, textString = "%name=%ratio", lineColor = {0,0,255}),Line(points = {{ -80,20},{ -60,20}}, color = {0,0,0}),Line(points = {{ -80, -20},{ -60, -20}}, color = {0,0,0}),Line(points = {{ -10,100},{10,100}}, color = {0,0,0}),Line(points = {{ -10,60},{10,60}}, color = {0,0,0}),Line(points = {{60,20},{80,20}}, color = {0,0,0}),Line(points = {{60, -20},{80, -20}}, color = {0,0,0}),Line(points = {{ -70, -20},{ -70, -70},{70, -70},{70, -20}}, color = {0,0,0}),Line(points = {{0,60},{0, -70},{0, -100}}, color = {0,0,0})}), Documentation(info = "<html>
<p>This component is used to model a 1-dim. gearbox
with non-parallel axes (defined by parameters <tt>n_a</tt>, <tt>n_b</tt>).
A 3-dim. <tt>bearing</tt> frame is necessary to reflect the
correct support torque, as the axes of rotation of <tt>flange_a</tt> and
<tt>flange_b</tT> and the direction of the support torque vector
are different in general.</p>
<p>Note: The name BevelGear1D is kept only for simplicity. Regardless,
this component could be used to model any kind of gearbox with non-parallel axes.</p>
<p><b>Reference</b><br>
<span style=\"font-variant:small-caps\">Schweiger</span>, Christian ;
<span style=\"font-variant:small-caps\">Otter</span>, Martin:
<a href=\"http://www.modelica.org/Conference2003/papers/h06_Schweiger_powertrains_v5.pdf\">Modelling
3D Mechanical Effects of 1-dim. Powertrains</a>. In: <i>Proceedings of the 3rd International
Modelica Conference</i>. Link&ouml;ping : The Modelica Association and Link&ouml;ping University,
November 3-4, 2003, pp. 149-158</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
        protected 
          outer World world;
          parameter Real e_a[3](each final unit = "1") = Modelica.Math.Vectors.normalize(n_a) "Unit vector in direction of flange_a rotation axis";
          parameter Real e_b[3](each final unit = "1") = Modelica.Math.Vectors.normalize(n_b) "Unit vector in direction of flange_b rotation axis";
          encapsulated model Housing
            import Modelica;
            input Modelica.SIunits.Torque t[3];
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,110},{150,150}}, lineColor = {0,0,255}, textString = "%name")}));

          equation 
            frame_a.f = zeros(3);
            frame_a.t = t;
          end Housing;
          Housing housing(t =  -flange_a.tau * e_a - flange_b.tau * e_b) if world.driveTrainMechanics3D annotation(Placement(transformation(extent = {{20, -60},{40, -40}}, rotation = 0)));

        equation 
          flange_a.phi = ratio * flange_b.phi;
          0 = ratio * flange_a.tau + flange_b.tau;
          connect(housing.frame_a,frame_a) annotation(Line(points = {{20, -50},{0, -50},{0, -100}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        end BevelGear1D;
        model RollingWheel "Ideal rolling wheel on flat surface z=0 (5 positional, 3 velocity degrees of freedom)"
          import SI = Modelica.SIunits;
          parameter Boolean animation = true "= true, if animation of wheel shall be enabled";
          parameter SI.Radius wheelRadius "Radius of wheel";
          parameter SI.Mass wheelMass "Mass of wheel";
          parameter SI.Inertia wheel_I_axis "Inertia along the wheel axis";
          parameter SI.Inertia wheel_I_long "Inertia perpendicular to the wheel axis";
          parameter StateSelect stateSelect = StateSelect.always "Priority to use generalized coordinates as states" annotation(HideResult = true, Evaluate = true);
          SI.Position x(start = 0, fixed = true, stateSelect = stateSelect) "x-coordinate of wheel axis";
          SI.Position y(start = 0, fixed = true, stateSelect = stateSelect) "y-coordinate of wheel axis";
          SI.Angle angles[3](start = {0,0,0}, fixed = true, stateSelect = stateSelect) "Angles to rotate world-frame in to frame_a around z-, y-, x-axis" annotation(Dialog(group = "Initialization", __Dymola_initialDialog = true));
          SI.AngularVelocity der_angles[3](start = {0,0,0}, fixed = true, stateSelect = stateSelect) "Derivative of angles" annotation(Dialog(group = "Initialization", __Dymola_initialDialog = true));
          parameter SI.Distance width = 0.035 "Width of wheel" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Real hollowFraction = 0.8 "1.0: Completely hollow, 0.0: rigid cylinder" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Modelica.Mechanics.MultiBody.Types.Color wheelColor = {30,30,30} "Color of wheel" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          Modelica.Mechanics.MultiBody.Parts.Body body(final r_CM = {0,0,0}, final m = wheelMass, final I_11 = wheel_I_long, final I_22 = wheel_I_axis, final I_33 = wheel_I_long, final I_21 = 0, final I_31 = 0, final I_32 = 0, animation = false) annotation(Placement(transformation(extent = {{20, -10},{40,10}})));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Frame fixed in wheel center point. y-axis: along wheel axis, z-Axis: upwards, " annotation(Placement(transformation(extent = {{ -16, -16},{16,16}})));
          Modelica.Mechanics.MultiBody.Joints.RollingWheel rollingWheel(wheelRadius = wheelRadius, stateSelect = StateSelect.avoid) annotation(Placement(transformation(extent = {{ -60, -60},{ -40, -40}})));
          Modelica.Mechanics.MultiBody.Visualizers.FixedShape fixedShape(final animation = animation, final r_shape = {0, -width,0}, final lengthDirection = {0,1,0}, final widthDirection = {1,0,0}, final length = 2 * width, final width = 2 * wheelRadius, final height = 2 * wheelRadius, final color = wheelColor, final extra = hollowFraction, final shapeType = "pipe") if animation annotation(Placement(transformation(extent = {{20,20},{40,40}})));
          annotation(defaultComponentName = "wheel", Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -80},{100, -100}}, lineColor = {0,0,0}, fillColor = {175,175,175}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -80,80},{80, -80}}, lineColor = {0,0,0}, fillColor = {215,215,215}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,122},{150,82}}, textString = "%name", lineColor = {0,0,255})}));

        equation 
          rollingWheel.x = x;
          rollingWheel.y = y;
          rollingWheel.angles = angles;
          rollingWheel.der_angles = der_angles;
          connect(body.frame_a,frame_a) annotation(Line(points = {{20,0},{0,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(rollingWheel.frame_a,frame_a) annotation(Line(points = {{ -50, -50},{ -25, -50},{ -25,0},{0,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(fixedShape.frame_a,frame_a) annotation(Line(points = {{20,30},{0,30},{0,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        end RollingWheel;
        model RollingWheelSet "Ideal rolling wheel set consisting of two ideal rolling wheels connected together by an axis"
          import SI = Modelica.SIunits;
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frameMiddle "Frame fixed in middle of axis connecting both wheels (y-axis: along wheel axis, z-Axis: upwards)" annotation(Placement(transformation(extent = {{ -16,16},{16, -16}}), iconTransformation(extent = {{ -16, -16},{16,16}})));
          parameter Boolean animation = true "= true, if animation of wheel set shall be enabled";
          parameter SI.Radius wheelRadius "Radius of one wheel";
          parameter SI.Mass wheelMass "Mass of one wheel";
          parameter SI.Inertia wheel_I_axis "Inertia along one wheel axis";
          parameter SI.Inertia wheel_I_long "Inertia perpendicular to one wheel axis";
          parameter SI.Distance wheelDistance "Distance between the two wheels";
          parameter StateSelect stateSelect = StateSelect.always "Priority to use the generalized coordinates as states";
          Modelica.SIunits.Position x(start = 0, fixed = true, stateSelect = stateSelect) "x coordinate of center between wheels";
          Modelica.SIunits.Position y(start = 0, fixed = true, stateSelect = stateSelect) "y coordinate of center between wheels";
          Modelica.SIunits.Angle phi(start = 0, fixed = true, stateSelect = stateSelect) "Orientation angle of wheel axis along z-axis";
          Modelica.SIunits.Angle theta1(start = 0, fixed = true, stateSelect = stateSelect) "Angle of wheel 1";
          Modelica.SIunits.Angle theta2(start = 0, fixed = true, stateSelect = stateSelect) "Angle of wheel 2";
          Modelica.SIunits.AngularVelocity der_theta1(start = 0, fixed = true, stateSelect = stateSelect) "Derivative of theta 1";
          Modelica.SIunits.AngularVelocity der_theta2(start = 0, fixed = true, stateSelect = stateSelect) "Derivative of theta 2";
          parameter SI.Distance wheelWidth = 0.01 "Width of one wheel" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Real hollowFraction = 0.8 "1.0: Completely hollow, 0.0: rigid cylinder" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Modelica.Mechanics.MultiBody.Types.Color wheelColor = {30,30,30} "Color of wheels" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          annotation(defaultComponentName = "wheelSet", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -80},{100, -100}}, lineColor = {0,0,0}, fillColor = {175,175,175}, fillPattern = FillPattern.Solid),Text(extent = {{ -146, -98},{154, -138}}, textString = "%name", lineColor = {0,0,255}),Ellipse(extent = {{42,80},{118, -80}}, lineColor = {0,0,0}, fillColor = {215,215,215}, fillPattern = FillPattern.Sphere),Rectangle(extent = {{ -62,2},{64, -6}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -118,80},{ -42, -80}}, lineColor = {0,0,0}, fillColor = {215,215,215}, fillPattern = FillPattern.Sphere),Line(points = {{86,24},{64,24},{64,10},{56,10}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{86, -24},{64, -24},{64, -12},{56, -12}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{ -96,100},{ -80,100},{ -80,4}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{100,100},{80,100},{80, -2}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{0,72},{0,40},{ -20,40},{ -20,2}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{0,40},{20,40},{20,2}}, color = {0,0,0}, smooth = Smooth.None)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{0, -106},{0, -78}}, color = {0,0,255}, smooth = Smooth.None),Polygon(points = {{0, -60},{ -6, -78},{6, -78},{0, -60}}, lineColor = {0,0,255}, smooth = Smooth.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{12, -68},{30, -80}}, lineColor = {0,0,255}, textString = "x"),Line(points = {{6, -100},{ -26, -100}}, color = {0,0,255}, smooth = Smooth.None),Polygon(points = {{ -22, -94},{ -22, -106},{ -40, -100},{ -22, -94}}, lineColor = {0,0,255}, smooth = Smooth.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -46, -80},{ -28, -92}}, lineColor = {0,0,255}, textString = "y")}), Documentation(info = "<html>
<p>
Two wheels are connected by an axis and can rotate around this axis.
The wheels are rolling on the x-y plane. The coordinate system attached
to the center of the wheel axis (frameMiddle) is constrained so that it
is always parallel to the x-y plane. If all generalized coordinates are zero,
frameMiddle is parallel to the world frame.
</p>
</html>"));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame1 "Frame fixed in center point of left wheel (y-axis: along wheel axis, z-Axis: upwards)" annotation(Placement(transformation(extent = {{ -96,16},{ -64, -16}}), iconTransformation(extent = {{ -96,16},{ -64, -16}})));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame2 "Frame fixed in center point of right wheel (y-axis: along wheel axis, z-Axis: upwards)" annotation(Placement(transformation(extent = {{64,16},{96, -16}})));
          Modelica.Mechanics.MultiBody.Parts.Body body2(final r_CM = {0,0,0}, final I_21 = 0, final I_31 = 0, final I_32 = 0, animation = false, final m = wheelMass, final I_11 = wheel_I_long, final I_22 = wheel_I_axis, final I_33 = wheel_I_long) annotation(Placement(transformation(extent = {{10, -10},{ -10,10}}, rotation =  -90, origin = {60,30})));
          Modelica.Mechanics.MultiBody.Visualizers.FixedShape shape2(final animation = animation, final lengthDirection = {0,1,0}, final widthDirection = {1,0,0}, final color = wheelColor, final extra = hollowFraction, final shapeType = "pipe", final r_shape = {0, -wheelWidth,0}, final length = 2 * wheelWidth, final width = 2 * wheelRadius, final height = 2 * wheelRadius) if animation annotation(Placement(transformation(extent = {{10, -10},{ -10,10}}, rotation = 90, origin = {60, -38})));
          Modelica.Mechanics.MultiBody.Parts.Body body1(final r_CM = {0,0,0}, final I_21 = 0, final I_31 = 0, final I_32 = 0, animation = false, final m = wheelMass, final I_11 = wheel_I_long, final I_22 = wheel_I_axis, final I_33 = wheel_I_long) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 90, origin = { -60,30})));
          Modelica.Mechanics.MultiBody.Visualizers.FixedShape shape1(final animation = animation, final lengthDirection = {0,1,0}, final widthDirection = {1,0,0}, final color = wheelColor, final extra = hollowFraction, final shapeType = "pipe", final r_shape = {0, -wheelWidth,0}, final length = 2 * wheelWidth, final width = 2 * wheelRadius, final height = 2 * wheelRadius) if animation annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation =  -90, origin = { -60, -40})));
          Modelica.Mechanics.Rotational.Interfaces.Flange_a axis1 "1-dim. rotational flange that drives the left wheel" annotation(Placement(transformation(extent = {{ -110,90},{ -90,110}})));
          Modelica.Mechanics.Rotational.Interfaces.Flange_a axis2 "1-dim. rotational flange that drives the right wheel" annotation(Placement(transformation(extent = {{90,90},{110,110}})));
          Modelica.Mechanics.MultiBody.Joints.RollingWheelSet wheelSetJoint(animation = false, wheelRadius = wheelRadius, wheelDistance = wheelDistance, stateSelect = StateSelect.default, x(fixed = false), y(fixed = false), phi(fixed = false), theta1(fixed = false), theta2(fixed = false), der_theta1(fixed = false), der_theta2(fixed = false)) annotation(Placement(transformation(extent = {{ -10, -42},{10, -22}})));
          Modelica.Mechanics.Rotational.Interfaces.Flange_b support "Support of 1D axes" annotation(Placement(transformation(extent = {{ -10,70},{10,90}}), iconTransformation(extent = {{ -10,72},{10,92}})));

        equation 
          wheelSetJoint.x = x;
          wheelSetJoint.y = y;
          wheelSetJoint.phi = phi;
          wheelSetJoint.theta1 = theta1;
          wheelSetJoint.theta2 = theta2;
          der_theta1 = der(theta1);
          der_theta2 = der(theta2);
          connect(body2.frame_a,frame2) annotation(Line(points = {{60,20},{60,0},{80,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(body1.frame_a,frame1) annotation(Line(points = {{ -60,20},{ -60,0},{ -80,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(shape1.frame_a,frame1) annotation(Line(points = {{ -60, -30},{ -60,0},{ -80,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(shape2.frame_a,frame2) annotation(Line(points = {{60, -28},{60,0},{80,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(wheelSetJoint.frame2,frame2) annotation(Line(points = {{8, -32},{40, -32},{40,0},{80,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(wheelSetJoint.frame1,frame1) annotation(Line(points = {{ -8, -32},{ -40, -32},{ -40,0},{ -80,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(wheelSetJoint.axis1,axis1) annotation(Line(points = {{ -10, -22},{ -20, -22},{ -20,50},{ -80,50},{ -80,100},{ -100,100}}, color = {0,0,0}, smooth = Smooth.None));
          connect(wheelSetJoint.axis2,axis2) annotation(Line(points = {{10, -22},{24, -22},{24,50},{80,50},{80,100},{100,100}}, color = {0,0,0}, smooth = Smooth.None));
          connect(wheelSetJoint.support,support) annotation(Line(points = {{0, -24},{0, -14},{16, -14},{16,58},{0,58},{0,80}}, color = {0,0,0}, smooth = Smooth.None));
          connect(wheelSetJoint.frameMiddle,frameMiddle) annotation(Line(points = {{0, -32},{ -4, -32},{ -4,0},{0,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
        end RollingWheelSet;
      end Parts;
      package MultiBody "Library to model 3-dimensional mechanical systems"
        annotation(Documentation(info = "<HTML>
<p>
Library <b>MultiBody</b> is a <b>free</b> Modelica package providing
3-dimensional mechanical components to model in a convenient way
<b>mechanical systems</b>, such as robots, mechanisms, vehicles.
Typical animations generated with this library are shown
in the next figure:
</p>
<p>
<img src=\"../Images/MultiBody/MultiBody.png\">
</p>
<p>
For an introduction, have especially a look at:
</p>
<ul>
<li> <a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide\">MultiBody.UsersGuide</a>
     discusses the most important aspects how to use this library.</li>
<li> <a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples\">MultiBody.Examples</a>
     contains examples that demonstrate the usage of this library.</li>
</ul>

<p>
Copyright &copy; 1998-2009, Modelica Association and DLR.
</p>
<p>
<i>This Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p><br>
</HTML>"));
        import SI = Modelica.SIunits;
        extends Modelica.Icons.Library;
        package UsersGuide "User's Guide of MultiBody Library"
          annotation(__Dymola_DocumentationClass = true, Documentation(info = "<HTML>
<p>
Library <b>MultiBody</b> is a <b>free</b> Modelica package providing
3-dimensional mechanical components to model in a convenient way
<b>mechanical systems</b>, such as robots, mechanisms, vehicles.
This package contains the User's Guide for the MultiBody library.
</p>
<ol>
<li>
<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial\">Tutorial</a>
gives an introduction into the most important aspects of the library.
</li>
<li>
<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Upgrade\">Upgrade</a> describes
  how to upgrade from former versions, especially from the \"old\"
  ModelicaAdditions.MultiBody library.
</li>

<li>
<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Literature\">Literature</a> provides
  references that have been used to design and implement this library.
</li>
<li>
<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Contact\">Contact</a> provides
  information about the author of the library as well as
  acknowledgments.
</li>
</ol>
</HTML>"));
          package Tutorial "Tutorial"
            annotation(__Dymola_DocumentationClass = true, Documentation(info = "<HTML>
<p>
This tutorial provides an introduction into the
MultiBody library.
</p>
<ol>
<li>
<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.OverView\">Overview of
MultiBody library</a> summarizes the most important aspects.
</li>
<li>
<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.FirstExample\">A first example</a>
  describes in detail all the steps to build a simple pendulum model.
</li>
<li>
<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures\">Loop structures</a>
  explains how to model kinematic loops, especially by analytically
  solving non-linear equations.
</li>
</ol>
</HTML>"));
            class OverView "Overview of MultiBody library"
              annotation(Documentation(info = "<HTML>
<p>
Library <b>MultiBody</b> is a <b>free</b> Modelica package providing
3-dimensional mechanical components to model in a convenient way
<b>mechanical systems</b>, such as robots, mechanisms, vehicles.
A basic feature is that all components have <b>animation</b> information
with appropriate default sizes and colors. A typical screenshot of the
animation of a double pendulum is shown in the figure below, together
with its schematic.
</p>
<IMG SRC=\"../Images/MultiBody/Examples/Elementary/DoublePendulumSmall.png\"> &nbsp;&nbsp;
<IMG SRC=\"../Images/MultiBody/Tutorial/DoublePendulumDiagramSmall.png\"
  ALT=\"double pendulum (diagram layer)\">
<p>
Note, that all components - the coordinate system of the world frame,
the gravity acceleration vector, the revolute joints and the
bodies - are visualized in the animation.<br>
This library replaces the long available ModelicaAdditions.MultiBody
library, since it is much more easier to use and more powerful.
The main features of the library are:
</p>
<ul>
<li>About <b>60 main components</b>, i.e., joint, force, part,
    body, sensor and visualizer components that are ready to use
    and have useful default animation properties. One-dimensional
    force laws can be defined with components of the Modelica.Mechanics.Rotational
    and of the Modelica.Mechanics.Translational library and can be
    connected via available flange connectors to MultiBody
    components.
<li>About <b>75 functions</b> to operate in a convenient way on
    orientation objects, e.g., to transform vector quantities between
    frames, or compute the orientation object of a planar rotation.
    The basic idea is to hide the actual definition of an <b>orientation</b>
    by providing essentially an <b>Orientation</b> type together with
    <b>functions</b> operating on instances of this type. Orientation
    objects based on a 3x3 transformation matrix and on quaternions
    are provided. As a side effect, the equations in all other
    components are simpler and easier to understand.</li>
<li> <b>A World model</b> has to be present in every model on top
     level. Here the gravity field is defined (currently: no gravity,
     uniform gravity, point gravity), the visualization of the
     world coordinate system and default settings for animation.
     If a world model is not present, it is automatically provided
     together with a warning message.
<li><b>Built-in animation properties</b> of all components, such as
    joints, forces, bodies, sensors. This allows an easy visual
    check of the constructed model. Animation of every component
    can be switched off via a parameter. The animation of a complete
    system can be switched off via one parameter in the <b>world</b>
    model. If animation is switched off, all equations related
    to animation are removed from the generated code. This is especially
    important for real-time simulation.</li>
<li><b>Automatic handling of kinematic loops</b>.
    Components can be connected together in a nearly arbitrary fashion.
    It does not matter whether components are flipped. This does not
    influence the efficiency. If kinematic loop structures occur,
    this is automatically handeled in an efficient way by a new
    technique to transform a certain class of overdetermined sets of
    differential algebraic equations symbolically to a system where
    the number of equations and unknowns are the same (the user need
    <b>not</b> cut loops with special cut-joints to construct a
    tree-structure). </li>
<li><b>Automatic state selection from joints and bodies</b>.
    Most joints and all bodies have potential states. A Modelica
    translator, such as Dymola, will use the generalized coordinates
    of joints as states if possible. If this is not possible,
    states are selected from body coordinates. As a consequence,
    strange joints with 6 degrees of freedom are not necessary
    to define a body moving freely in space. An advanced user
    may select states manually from the <b>Advanced</b> menu of the
    corresponding components or use a Modelica parameter modification
    to set the \"stateSelect\" attribute directly. </li>
<li> <b>Analytic solution of kinematic loops</b>. The non-linear
    equations occuring in kinematic loops are solved <b>analytically</b>
    for a large class of mechanisms, such as a 4 bar mechanism,
    a slider-crank mechanism or a MacPherson suspension. This is performed
    by constructing such loops with assembly joints JointXXX,
    available in the Modelica.Mechanics.MultiBody.Joints package. Assembly joints consist
    of 3 joints that have together 6 degrees of freedom, i.e., no
    constraints.They do not have potential states. When the motion
    of the two frame connectors are provided, a non-linear system of
    equation is solved analytically to compute the motion of the 3 joints.
    Analytic loop handling is especially important for real-time simulation.</li>
<li> <b>Line force components may have mass</b>.
     Masses of line force components are located on the line on which
     the force is acting. They approximate the mass properties of
     a real physical device by one or two point masses. For example,
     a spring has often significant mass that has to be taken into
     account. If masses are set to zero, the additional code to handle
     these point masses is removed. If the masses are taken into
     account, the calculation overhead is small (the reason is that
     the occuring kinematic loops are analytically solved).<br>
     Note, in this Beta-release, not all provided line force
     components have already an optional mass. This will be fixed
     in the next release.</li>
<li> <b>Force components may be connected directly together</b>, e.g.,
     3-dimensional springs in series connection. Usually,
     multi-body programs have the restriction that force components
     can only be connected between two bodies. Such restrictions are
     not present in the Modelica multi-body library, since it is
     a fully object-oriented, equation based library. Usually, if
     force components are connected directly together, non-linear
     systems of equations occur. The advantage is often, that this
     may avoid stiff systems that would occur if a small mass has
     to be put in between the two force elements.</li>
<li><b>Initialization definition is available via menus</b>.
    Initialization of states in joints and bodies can be
    performed in the parameter menu, <b>without</b> typing Modelica
    statements. For non-standard initialization, the usual
    Modelica commands can be used.</li>
<li><b>Multi-body specific error messages</b>. Annotations
    and assert statements have been introduced that provide
    in many cases warning or error messages that are related
    to the library components (and not to specific equations
    as it is usual in Modelica libraries). This requires
    appropriate tool support, as it is, e.g., available in Dymola.</li>
<li><b>Inverse models</b> of mechanical systems can be easily
    defined by using motion generators, e.g.,
    Modelica.Mechanics.Rotational.Position. Also, non-standard
    inverse models can be generated, e.g., when elasticity is present
    it might be necessary to differentiate equations several times.</li>
</ul>
</HTML>
"));
            end OverView;
            class FirstExample "A first example"
              annotation(Documentation(info = "<HTML>
<p>
As a first example it shall be demonstrated how to build up, simulate
and animate a <b>simple pendulum</b>.
</p>
<p>
A simple pendulum consisting of a <b>body</b> and a <b>revolute</b> joint
with <b>linear damping</b> in the joint, is first build-up as
Modelica composition diagram, resulting in:
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/FirstExample/PenulumSchematic1.png\"
ALT=\"Modelica composition diagram of simple pendulum\">
</p>
<p>
In the following figure the location of the used
model components is shown. Drag these components in the diagram layer
and connect them according to the figure:
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/FirstExample/PenulumSchematic2.png\">
</p>
<p>
Every model that uses model components from the MultiBody library
must have an instance of the Modelica.Mechanics.MultiBody.World model on highest level.
The reason is that in the world object the gravity field is defined
(uniform gravity or point gravity), as well as the default sizes of
animation shapes and this information is reported to all used
components. If the World object is missing, a warning message is
printed and an instance of the World object with default settings is
automatically utilized (this feature is defined with annotations and
is, e.g., supported by Dymola).
</p>
<p>
In a second step the parameters of the dragged components need to be
defined. Some parameters are vectors that have to be defined with respect
to a local coordinate system of the corresponding component. The easiest
way to perform this is to define a <b>reference configuration</b> of your
multi-body model: In this configuration, the relative coordinates of
all joints are zero. This means that all coordinate systems on all
components are parallel to each other. Therefore, this just means
that all vectors are resolved in the world frame in this configuration.
</p>
<p>
The reference configuration for the simple pendulum shall be defined
in the following way: The y-axis of the world frame is directed
upwards, i.e., the opposite direction of the gravity acceleration.
The x-axis of the world frame is orthogonal to it. The revolute joint
is placed in the origin of the world frame. The rotation axis of the revolute
joint is directed along the z-axis of the world frame. The body is
placed on the x-axis of the world frame (i.e., the rotation angle of the
revolute joint is zero, when the body is on the x-axis).
In the following figures the definition of this reference configuration
is shown in the parameter menus of the revolute joint and the body:
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/FirstExample/ActuatedRevolute.png\">
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/FirstExample/Body.png\">
</p>
<p>
Translate and simulate the model, e.g., with Dymola.
Automatically, all defined components are visualized in
an animation using default absolute or relative sizes
of the components. For example, a body is visualized as
a sphere and as a cylinder. The default size of the sphere is defined
as parameter in the world object. You may change this size
in the \"Animation\" parameter menu of the body (see parameter menu
above). The default size of the cylinder is defined relatively
to the size of the sphere (half of the sphere size).
With default settings, the following animation is defined:
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/FirstExample/Pendulum.png\">
</p>
<p>
The world coordinate system is visualized as coordinate system
with axes labels. The direction of the gravity acceleration
vector is shown as green arrow. The red cylinder represents
the rotation axis of the revolute joint and the light blue
shapes represent the body. The center of mass of the body is
in the middle of the light blue sphere.
</p>
</HTML>
"));
            end FirstExample;
            package LoopStructures "Loop structures"
              class Introduction "Introduction"
                annotation(__Dymola_DocumentationClass = true, Documentation(info = "<HTML>
<p>
In principal, now special action is needed, if
loop structures occur (contrary to the ModelicaAdditions.MultiBody library).
An example is presented in the figure below. It is available as
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Loops.Fourbar1\">
MultiBody.Examples.Loops.Fourbar1</a>
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/LoopStructures/Fourbar1a.png\">
</p>
<p>
This mechanism consists of 6 revolute joints, 1 prismatic joint and forms a
kinematical loop.
It has has one degree of freedom. In the next figure the default animation
is shown. Note, that the axes of the revolute joints are represented by the
red cylinders and that the axis of the prismatic joint is represented by the
red box on the lower right side.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/LoopStructures/Fourbar1b.png\" width=\"205\" height=\"107\">
</p>
<p>
Whenever loop structures occur, non-linear algebraic
equations are present on \"position level\". It is then usually not possible by
structural analysis to select states during translation (which is possible for
non-loop structures). In the example above, Dymola detects a non-linear
algebraic loop of 57 equations and reduces this to a system of 7 coupled
algebraic equations. Note, that this is performed without using any
\"cut-joints\" as it is usually done in multi-body programs, but by just
appropriate symbolic equation manipulation. Via the dynamic dummy derivative
method the generalized coordinates on position and velocity level from one of
the 7 joints are dynamically selected as states during simulation. Whenever,
these two states are no longer appropriate, states from one of the other
joints are selected during simulation.
</p>
<p>
The efficiency of loop structures can usually be
enhanced, if states are statically fixed at translation time. For this
mechanism, the generalized coordinates of joint j1 (i.e., the
rotation angle of the revolute joint and its derivative) can always be used as
states. This can be stated by setting parameter \"enforceStates = <b>true</b>\"
in the \"Advanced\" menu of the desired joint. This flag sets the attribute
stateSelect of the generalized coordinates of the coresponding joint to
\"StateSelect.always\". When setting this flag to <b>true</b> for joint j1 in
the four bar mechanism, Dymola detects a non-linear algebraic loop of 40
equations and reduces this to a system of 5 coupled non-linear algebraic
equations.
</p>
<p>
In many mechanisms it is possible to solve the non-linear algebraic
equations analytically. For a certain class of systems this can
be performed also with the MultiBody library. This technique
is described in section
\"<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.AnalyticLoopHandling\">Analytic loop handling</a>\".
</p>
</HTML>"));
              end Introduction;
              class PlanarLoops "Planar loops"
                annotation(Documentation(info = "<HTML>
<p>
In the figure below, the model of a V6 engine is
shown that has a simple combustion model. It is available as
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Loops.EngineV6\">
MultiBody.Examples.Loops.EngineV6</a>.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/LoopStructures/EngineV6_1.png\">
</p>
<p>
The Modelica schematic of one cylinder
is given in the figure below. Connecting 6 instances of this
cylinder appropriately together results in the engine schematic displayed
above.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/LoopStructures/EngineV6_2.png\">
</p>
<p>
In the next figure the animation of the engine is
shown. Every cylinder consists essentially of 1 prismatic and 2 revolute
joints that form a planar loop, since the axes of the two revolute joints are
parallel to each other and the axis of the prismatic joint is orthogonal to
the revolute joint axes. All 6 cylinders together form a coupled set of 6
loops that have together 1 degree of freedom.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/LoopStructures/EngineV6_3.png\" width=\"303\" height=\"136\">
</p>
<p>
All planar loops, and especially the engine, result in
a DAE (= Differential-Algebraic Equation system)
that does not have a unique solution. The reason is that, e.g., the cut
forces in direction of the axes of the revolute joints cannot be uniquely
computed. Any value fulfills the DAE equations. This is a structural property
that is determined by the symbolic algorithms. Since they detect that the DAE is
structurally singular, a further processing is not possible. Without
additional information it is also impossible that the symbolic algorithms
could be enhanced because if the axes of rotations of the revolute joints are
only slightly changed such that they are no longer parallel to each other, the
planar loop can no longer move and has 0 degrees of freedom. Algorithms based
on pure structural information cannot distinguish these two cases.
</p>
<p>
The usual remedy is to remove superfluous constraints,
e.g., along the axis of rotation of <b>one</b> revolute joint. Since this is
not easy for an inexperienced modeler, the special joint:
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.RevolutePlanarLoopConstraint\">
RevolutePlanarLoopConstraint</a> is provided that removes these constraints.
Exactly one revolute joint in a every planar loop must be replaced by this
joint type. In the engine example, this special joinst is used for
the revolute joint B2 in the cylinder model above. The icon of the joint is
slightly different to other revolute joints to visualize this case.
</p>
<p>
If a modeler is not aware of the problems with planar
loops and models them without special consideration, a Modelica
translator, such as Dymola, displays an error
message and points out that a planar loop may be the reason and suggests to
use the RevolutePlanarLoopConstraint joint.
This error message is due to an annotation in
the Frame connector.
</p>
<pre>
  <b>connector</b> Frame
     ...
     <b>flow</b> SI.Force f[3] <b>annotation</b>(unassignedMessage=\"..\");
  <b>end</b> Frame;
</pre>
<p>
If no assignment can be found for some
forces in a connector, the \"unassignedMessage\" is displayed. In most cases the
reason for this is a planar loop or two joints that constrain the same motion.
Both cases are discussed in the error message.
</p>
<p>
Note, that the non-linear algebraic equations occurring
in planar loops can be solved analytically in most cases and therefore it is
highly recommended to use the techniques discussed in section
\"<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.AnalyticLoopHandling\">Analytic loop handling</a>\"
for such systems.
</p>
</HTML>"));
              end PlanarLoops;
              class AnalyticLoopHandling "Analytic loop handling"
                annotation(Documentation(info = "<HTML>
It is well known that the non-linear
algebraic equations of most mechanical loops in technical devices can be
solved analytically. It is, however, difficult to perform this fully
automatically and therefore none of the commercial, general purpose multi-body
programs, such as MSC ADAMS, LMS DADS, SIMPACK, have this feature.
These programs solve loop structures with pure numerical methods. Multi-body
programs that are designed for real-time simulation of the dynamics of
specific vehicles, such as ve-DYNA, usually contain manual implementations
of a particular multi-body system (the vehicle) where the occurring loops are
either analytically solved, if this is possible, or are treated by table
look-up where the tables are constructed in a pre-processing phase. Without
these features the required real-time capability would be difficult to
achieve.
</p>
<p>
In a series of papers and dissertations
Prof. Hiller and his group in Duisburg, Germany,
have developed systematic methods to handle mechanical
loops analytically. The \"characteristic pair of joints\" method
basically cuts a loop at two joints and uses geometric
invariants to reduce the number of algebraic
equations, often down to one equation that can be solved analytically. Also
several multi-body codes have been developed that are based on this method,
e.g., MOBILE. Besides the very desired feature to solve non-linear
algebraic equations analytically, i.e., efficiently and in a robust way, there
are several drawbacks: It is difficult to apply this method automatically.
Even if this would be possible in a good way, there is always the problem that
it cannot be guaranteed that the statically selected states lead to no
singularity during simulation. Therefore, the \"characteristic pair of joints\"
method is usually manually applied which requires know-how and experience.
</p>
<p>
In the MultiBody library the \"characteristic pair of
joints\" method is supported in a restricted form such that it can be applied
also by non-specialists. The idea is to provide aggregations of joints in package
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies\">
MultiBody.Joints.Assemblies</a>.
as one object that either have <b>6</b> degrees of freedom or
<b>3</b> degrees of freedom (for usage in planar loops).
</p>
<p>
As an example, a variant of the four bar mechanism is given in
the figure below.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/LoopStructures/FourbarAnalytic1.png\">
</p>
<p>
Here, the mechanism is modeled with one revolute
joint, two spherical joints and one prismatic joint.
In the figure below, the two spherical joints
and the prismatic joint are collected together in an assembly object
called \"jointSSP\" from
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies.JointSSP\">
MultiBody.Joints.Assemblies.JointSSP</a>.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/LoopStructures/FourbarAnalytic2.png\">
</p>
<p>
The JointSSP joint aggregation has a frame at the
left side of the left spherical joint (frame_a) and a frame at the right side
of the prismatic joint (frame_b). JointSSP, as all other objects from the
Joints.Assemblies package, has the property, that the <b>generalized
coordinates, and all other frames
defined in the assembly, can be calculated given the movement of frame_a and
of frame_b</b>. This is performed by <b>analytically</b> solving non-linear
systems of equations (details are given in section xxx). From a
structural point of view, the equations in an assembly object are written in
the form
</p>
<p align=\"center\">
  <b>q</b> = <b>
  f</b><sub>1</sub>(<b>r</b><sup>a</sup>, <b>R</b><sup>a</sup>,<b> r</b><sup>b</sup>,
  <b>R</b><sup>b</sup>)
</p>
<p>
where<b> r</b><sup>a</sup>, <b>R</b><sup>a</sup>,<b>
r</b><sup>b</sup>, <b>R</b><sup>b</sup><b> </b>are the variables defining the
position and orientation of the frame_a and frame_b connector,
<b>q</b> are the generalized positional coordinates inside the
assembly, e.g., the angle of a revolute joint. Given angle
<span style=\"font-family:Symbol\">j</span> of revolute joint j1 from the four
bar mechanism, frame_a and frame_b of the assembly object can be computed by a
forward recursion
</p>
<p align=\"center\">
(<b>r</b><sup>a</sup>,
<b>R</b><sup>a</sup>,<b> r</b><sup>b</sup>, <b>R</b><sup>b</sup>) = <b>f</b>(<span style=\"font-family:Symbol\">j)</span>
</p>
<p>
Since this is a structural property, the
symbolic algorithms can automatically select <span style=\"font-family:Symbol\">
j</span> and its derivative as states and then all positional variables can be
computed in a forwards sequence. It is now understandable that a Modelica
translator, such as Dymola, can
transform the equations of the four bar mechanism to a recursive sequence of
statements that has no non-linear algebraic loops anymore(remember,
the previous \"straightforward\" solution with 6 revolute joints and 1
prismatic joint has a nonlinear system of equations of order 5).
</p>
<p>
The aggregated joint
objects consist of a combination of either a revolute or prismatic joint and
of a rod that has either two spherical joints at its two ends or a spherical
and a universal joint, respectively. For all combinations, analytic solutions
can be determined. For planar loops, combinations of 1, 2 or 3 revolute joints
with parallel axes and of 2 or 1 prismatic joint with axes that are orthogonal
to the revolute joints can be treated analytically. The currently supported
combinations are listed in the table below.
The missing combinations (such as JointSUP
or Joint RPP) will be added in one of the next releases.
</p>
<div align=\"center\">
    <table border=\"1\" cellspacing=\"0\" cellpadding=\"0\">
      <tr>
        <td colspan=\"2\"> <b>3-dimensional Loops:</b></td>
      </tr>
      <tr>
        <td valign=\"top\">JointSSR</td>
        <td valign=\"top\">Spherical - Spherical - Revolute</td>
      </tr>
      <tr>
        <td valign=\"top\">JointSSP</td>
        <td valign=\"top\">Spherical - Spherical - Prismatic</td>
      </tr>
      <tr>
        <td valign=\"top\">JointUSR</td>
        <td valign=\"top\">Universal - Spherical - Revolute</td>
      </tr>
      <tr>
        <td valign=\"top\">JointUSP</td>
        <td valign=\"top\">Universal - Spherical - Prismatic</td>
      </tr>
      <tr>
        <td valign=\"top\">JointUPS</td>
        <td valign=\"top\">Universal - Prismatic - Spherical</td>
      </tr>
      <tr>
        <td colspan=\"2\"><b>Planar Loops:</b></td>
      </tr>
      <tr>
        <td valign=\"top\">JointRRR</td>
        <td valign=\"top\">Revolute - Revolute - Revolute</td>
      </tr>
      <tr>
        <td valign=\"top\">JointRRP</td>
        <td valign=\"top\">Revolute - Revolute - Prismatic</td>
      </tr>
</table>
</div>
<p>
On first view this seems to be quite restrictive. However, mechanical devices are usually built up with rods connected by spherical joints on each end,
and additionally with revolute and prismatic joints.
Therefore, the combinations of the above table occur frequently.
The universal joint is usually not present in actual devices but is used
(a) if two JointXXX components can be connected such that a revolute
and a universal joint together form a spherical joint
and (b) if the orientation of the connecting rod between two
spherical joints is needed, e.g., since a body shall be attached.
In this case one of the spherical joints might be replaced by a
universal joint. This approximation is fine as long as the mass
and inertia of the rod is not significant.
</p>
<p>
Let us discuss item (a) in more detail: The
MacPherson suspension in the next figure
is from the Modelica VehicleDynamics library.
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/LoopStructures/MacPherson1.png\">
</p>
It has three frame
connectors. The lower left one (frame_C) is fixed in the vehicle chassis. The
upper left one (frame_S) is driven by the steering mechanism, i.e., the
movement of both frames are given. The frame connector on the right (frame_U)
drives the wheel. The three frames are connected by a mechanism consisting
essentially of two rods with spherical joints on both ends. These are built up
by a jointUPS and a jointSSR assembly.
As can be seen, the universal joint from the jointUPS
assembly is connected to the revolute joint of the jointSSR assembly.
Therefore, we have 3 revolute joints connected together at one point and if
the axes of rotations are chosen appropriately, this describes a spherical
joint. In other words, the two connected assemblies define the desired two
rods with spherical joints on each ends.
</p>
<p>
The movement of the chassis, frame_C, is computed
somewhere else. When the generalized coordinates of revolute joint
\"innerJoint\" (lower left part in figure) are used as states, then frame_a and
frame_b of the jointUPS joint can be calculated. After the non-linear loop
with jointUPS is (analytically) solved, all frames on this assembly are
known, especially,
the one connected to frame_b of the jointSSR assembly. Since frame_b of
jointSSR is connected to frame_S which is computed from the steering
mechanism, again the two required frame movements of the jointSSR assembly are
calculated, meaning in turn that also all other frames on the jointSSR
assembly can be computed, especially, the one connected to frame_U that drives
the wheel. From this analysis it is clear that a tool is able to solve these
coupled loops analytically.
</p>
<p>
Another example is the model of the V6 engine,
see next figure for an animation view and the original definition
of one cylinder with elementary joints.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/LoopStructures/EngineV6_3.png\" width=\"303\" height=\"136\">
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/LoopStructures/EngineV6_2.png\">
</p>

<p>
It is sufficient to rewrite the basic cylinder model
by replacing the joints with a JointRRP object that has two
revolute and one prismatic joint, see next figure.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/LoopStructures/EngineV6_4.png\">
</p>
<p>
Since 6
cylinders are connected together, 6 coupled loops with 6 JointRRP objects are
present. This model is available as
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Loops.EngineV6_analytic\">
MultiBody.Examples.Loops.EngineV6_analytic</a>.
</p>
<p>
The composition diagram of the connected 6 cylinders is
shown in the next figure
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Tutorial/LoopStructures/EngineV6_1.png\">
</p>
<p>
It can be seen that the revolute joint
of the crank shaft (joint \"bearing\" in left part of figure) might be
selected as degree of freedom. Then the 4 connector frames of all cylinders
can be computed. As a result the computations of the cylinders are decoupled
from each other. Within one cylinder
the position of frame_a and frame_b of the jointRRP assembly
can be computed and therefore the generalized coordinates of the two revolute
and the prismatic joint in the jointRRP object can be determined. From this
analysis it is not surprising that a Modelica translator, such as Dymola,
is able to transform the DAE
equations into a sequential evaluation without any non-linear loop.
Compare this nice result with the model using only elementary joints
that leads to a DAE with 6 algebraic loops and 5 non-linear equations per
loop. Additionally, a linear system of equations of order 43 is present.
The simulation time is about 5 times faster with the analytic loop handling.
</p>
</HTML>
"));
              end AnalyticLoopHandling;
              annotation(Documentation(info = "<HTML>
<p>
The MultiBody library has the feature that all components
can be connected together in a nearly arbitrary fashion.
Therefore, kinematic loop structures pose in principal
no problems. In this section several examples are given,
the special treatment of planar loops is discussed and it is explained
how a kinematic loop structure can be modeled such that the
occuring non-linear algebraic equation systems are solved
analytically. There are the following sub-chapters:
</p>
<ol>
<li>
<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.Introduction\">Introduction</a>
</li>
<li>
<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.PlanarLoops\">Planar loops</a>.
</li>
<li>
<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.AnalyticLoopHandling\">Analytic loop handling</a>.
</li>
</ol>
</HTML>"));
            end LoopStructures;
          end Tutorial;
          class Upgrade "Upgrade from Former Versions"
            annotation(Documentation(info = "<HTML>
<p>
If different versions of the MultiBody library are not
compatible to each other, corresponding conversion scripts are
provided. As a result, models build with an older version
of the MultiBody library are automatically converted to the
new version when the model is loaded. The user is prompted
whether automatic conversion shall take place or not.
Problems are not to be expected. Still one should first make
a copy of such a model as backup before the conversion
is performed.
</p>
<p>
<b><font color=\"#008000\">Upgrade from ModelicaAdditions.MultiBody</font></b>
</p>
<p>
There is now also a conversion script from the \"old\"
<b>ModelicaAdditions.MultiBody</b> library to the
\"new\" Modelica.Mechanics.MultiBody library. This script is also
automatically invoked. Since the differences between the \"old\" and the
\"new\" MultiBody library are so large, not everything is
converted and it might be that some pieces have to
be adapted manually. Still, this script is useful, since
many class names, parameters and modifiers are
automatically converted.
</p>
<p>
Components from the following sublibraries
are automatically converted
to the Modelica.Mechanics.MultiBody library:
</p>
<ul>
<li> ModelicaAdditions.MultiBody.Parts</li>
<li> ModelicaAdditions.MultiBody.Joints</li>
<li> ModelicaAdditions.MultiBody.Forces</li>
<li> Part of ModelicaAdditions.MultiBody.Interfaces</li>
</ul>
<p>
Models using the ModelicaAdditions.MultiBody library
that are programmed with <b>equations</b> are only partly converted:
The Frame connectors will be converted to the \"new\"
Frame connectors of the MultiBody library, but the equations
that reference variables of the Frame connectors will
<b>not</b> be converted. For a manual conversion, the following
table might be helpful showing how the <b>variables</b> of the
\"old\" and the \"new\" <b>Frame connectors</b> are
related to each other (resolve2 and angularVelocity2 are
functions from library Modelica.Mechanics.MultiBody.Frames):
</p>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>ModelicaAdditions.MultiBody.<br>
           Interfaces.Frame_a</b></th>
    <th><b>MultiBody.Interfaces.Frame_a</b></th></tr>
<tr>
  <td valign=\"top\">frame_a.<b>r0</b></td>
  <td valign=\"top\">= frame_a.r_0 (is converted)</td>
</tr>
<tr>
  <td valign=\"top\">frame_a.<b>S</b></td>
  <td valign=\"top\">= transpose(frame_a.R)</td>
</tr>
<tr>
  <td valign=\"top\">frame_a.<b>v</b></td>
  <td valign=\"top\">= resolve2(frame_a.R, <b>der</b>(frame_a.r_0))</td>
</tr>
<tr>
  <td valign=\"top\">frame_a.<b>w</b></td>
  <td valign=\"top\">= angularVelocity2(frame_a.R)</td>
</tr>
<tr>
  <td valign=\"top\">frame_a.<b>a</b></td>
  <td valign=\"top\">= resolve2(frame_a.R, <b>der</b>(v_0)); v_0 = der(r_0)</td>
</tr>
<tr>
  <td valign=\"top\">frame_a.<b>z</b></td>
  <td valign=\"top\">= <b>der</b>(w);  w = angulaVelocity2(frame_a.R)</td>
</tr>
<tr>
  <td valign=\"top\">frame_a.<b>f</b></td>
  <td valign=\"top\">= frame_a.f (no conversion needed)</td>
</tr>
<tr>
  <td valign=\"top\">frame_a.<b>t</b></td>
  <td valign=\"top\">= frame_a.t (no conversion needed)</td>
</tr>
</table>
<p>
<b><font color=\"#008000\">Upgrade from MultiBody 0.99 (and earlier) to 1.0 (and later)</font></b>
<p>
The conversion from MultiBody 0.99 to 1.0 does not work in some rare
cases, where own components are implemented using functions of the
MultiBody.Frames package. In this case, the conversion has to be
performed manually. The changes in 1.0 with regards to 0.99 are:
</p>
<p>
The definition of the Modelica.Mechanics.MultiBody.Frames.Orientation object has changed.
In 0.99 this was just an alias type for a transformation matrix
(now Modelica.Mechanics.MultiBody.Frames.TransformationMatrices.Orientation).
In 1.0 the orientation object is a record holding the
transformation matrix from frame 1 to frame 2 and the angular
velocity of the transformation matrix resolved in frame 2.
The reason is that this allows to compute the angular velocity
in many cases by standard recursive formulas and not by
differentiation of the transformation matrix. This is usually
much more efficient. As a consequence, the following
calls in 0.99 should be changed:
</p>
<pre>
   Frames.angularVelocity1(T,der(T)) -> Frames.angularVelocity1(T)
   Frames.angularVelocity2(T,der(T)) -> Frames.angularVelocity2(T)
   Frames.from_T(T)                  -> Frames.from_T2(T,der(T))
</pre>
</HTML>
"));
          end Upgrade;
          class Literature "Literature"
            annotation(Documentation(info = "<html>
<ul>
<li>
Technical details of this library are described in the
20 page paper:
<dl>
<dt>Otter M., Elmqvist H., and Mattsson S.E.:</dt>
<dd> <b>The New Modelica MultiBody Library</b>.
     Modelica 2003 Conference, Link&ouml;ping, Sweden,
     pp. 311-330, Nov. 3-4, 2003.
     Download from:
     <a href=\"http://www.modelica.org/Conference2003/papers/h37_Otter_multibody.pdf\">http://www.modelica.org/Conference2003/papers/h37_Otter_multibody.pdf</a>
     </dd>
</dl>
</li>
<li>
The method how to describe drive trains with 1-dimensional mechanics
and to mount them on 3-dimensional components without neglecting
dynamical effects is described in:
<dl>
<dt>Schweiger C., and Otter M.:</dt>
<dd> <b>Modelling 3-dim. Mechanical Effects of 1-dim. Powertrains</b>.
     Modelica 2003 Conference, Link&ouml;ping, Sweden,
     pp. 149-158, Nov. 3-4, 2003.
     Download from:
<a href=\"http://www.modelica.org/Conference2003/papers/h06_Schweiger_powertrains_v5.pdf\">http://www.modelica.org/Conference2003/papers/h06_Schweiger_powertrains_v5.pdf</a>
     </dd>
</dl>
</li>
<li> The method to solve a certain class of kinematic loops
     analytically is based on:
<dl>
<dt>Woernle C.:</dt>
<dd><b>Ein systematisches Verfahren zur Aufstellung der geometrischen
    Schliessbedingungen in kinematischen Schleifen mit Anwendung
    bei der R&uuml;ckw&auml;rtstransformation f&uuml;r
    Industrieroboter.</b><br>
    Fortschritt-Berichte VDI, Reihe 18, Nr. 59, Duesseldorf: VDI-Verlag 1988,
    ISBN 3-18-145918-6.<br>&nbsp;</dd>
<dt>Hiller M., and Woernle C.:</dt
<dd><b>A Systematic Approach for Solving the Inverse Kinematic
    Problem of Robot Manipulators</b>.<br>
    Proceedings 7th World Congress Th. Mach. Mech., Sevilla 1987. </dd>
</dl>
</li>
</ul>
</html>
"));
          end Literature;
          class Contact "Contact"
            annotation(Documentation(info = "<html>
<dl>
<dt><b>Library Officer:</b>
<dd><a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a><br>
    Deutsches Zentrum f&uuml;r Luft und Raumfahrt e.V. (DLR)<br>
    Institut f&uuml;r Robotik und Mechatronik<br>
    Abteilung f&uuml;r Entwurfsorientierte Regelungstechnik<br>
    Postfach 1116<br>
    D-82230 Wessling<br>
    Germany<br>
    email: <A HREF=\"mailto:Martin.Otter@dlr.de\">Martin.Otter@dlr.de</A><br>
</dl>
<p><b>Acknowledgements:</b></p>
<ul>
<li> The central idea to handle a certain class of overdetermined, consistent
     set of differential algebraic equations (i.e., there are more equations than
     unknowns) with symbolic transformation algorithms was developed together
     with Hilding Elmqvist and Sven Erik Mattsson from Dynasim AB, Lund, Sweden.
     The MultiBody library is heavily relying on this feature which is a
     prerequiste for a truely \"object-oriented\" multi-body systems library,
     where components can be connected together in any meaningful way.<li>
<li> The Examples.Loops.EngineV6 demo of a six cylinder V6 engine with
     6 planar loops and 1 degree of freedom is from Hilding Elmqvist and
     Sven Erik Mattsson.</li>
<li> Modelica.Mechanics.MultiBody.Forces.LineForceWithMass is based on model
     \"RelativeDistance\" from the Modelica VehicleDynamics library of
     Johan Andreasson from Royal Institute of Technology, Stockholm, Sweden.</li>
<li> The 1-dim. components (Parts.Rotor1D, Parts.BevelGear1D, Mounting1D) and
     Joints.GearConstraints are from Christian Schweiger.</li>
<li> The design of this library is based on work carried out
     in the EU RealSim project (Real-time Simulation for Design of
     Multi-physics Systems) funded by the European Commission within
     the Information Societies Technology (IST) programme under
     contract number IST 1999-11979.
     </li>
</ul>
</html>
"));
          end Contact;
        end UsersGuide;
        model World "World coordinate system + gravity field + default animation definition"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Types.GravityTypes;
          import Modelica.Mechanics.MultiBody.Types;
          Interfaces.Frame_b frame_b "Coordinate system fixed in the origin of the world frame" annotation(Placement(transformation(extent = {{84, -16},{116,16}}, rotation = 0)));
          annotation(defaultComponentName = "world", defaultComponentPrefixes = "inner", missingInnerMessage = "No \"world\" component is defined. A default world
component with the default gravity field will be used
(g=9.81 in negative y-axis). If this is not desired,
drag Modelica.Mechanics.MultiBody.World into the top level of your model.", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -100, -118},{ -100,61}}, color = {0,0,0}, thickness = 0.5),Polygon(points = {{ -100,100},{ -120,60},{ -80,60},{ -100,100},{ -100,100}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -119, -100},{59, -100}}, color = {0,0,0}, thickness = 0.5),Polygon(points = {{99, -100},{59, -80},{59, -120},{99, -100}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -140,165},{140,103}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{95, -113},{144, -162}}, lineColor = {0,0,0}, textString = "%label1"),Text(extent = {{ -170,127},{ -119,77}}, lineColor = {0,0,0}, textString = "%label2"),Line(points = {{ -56,78},{ -56, -26}}, color = {0,0,255}),Polygon(points = {{ -68, -26},{ -56, -66},{ -44, -26},{ -68, -26}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{2,78},{2, -26}}, color = {0,0,255}),Polygon(points = {{ -10, -26},{2, -66},{14, -26},{ -10, -26}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{66,80},{66, -26}}, color = {0,0,255}),Polygon(points = {{54, -26},{66, -66},{78, -26},{54, -26}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<HTML>
<p>
Model <b>World</b> represents a global coordinate system fixed in
ground. This model serves several purposes:
<ul>
<li> It is used as <b>inertial system</b> in which
     the equations of all elements of the MultiBody library
     are defined.</li>
<li> It is the world frame of an <b>animation window</b> in which
     all elements of the MultiBody library are visualized.</li>
<li> It is used to define the <b>gravity field</b> in which a
     multi-body model is present. Default is a uniform gravity
     field where the gravity acceleration vector g is the
     same at every position. Additionally, a point gravity field
     can be selected.</li>
<li> It is used to define <b>default settings</b> of animation properties
     (e.g. the diameter of a sphere representing by default
     the center of mass of a body, or the diameters of the cylinders
     representing a revolute joint).</li>
<li> It is used to define a <b>visual representation</b> of the
     world model (= 3 coordinate axes with labels) and of the defined
     gravity field.<br>
    <IMG SRC=\"../Images/MultiBody/world.png\" ALT=\"MultiBodys.World\">
</li>
</ul>
<p>
Since the gravity field function is required from all bodies with mass
and the default settings of animation properties are required
from nearly every component, exactly one instance of model World needs
to be present in every model on the top level. The basic declaration
needs to be:
</p>
<pre>
    <b>inner</b> Modelica.Mechanics.MultiBody.World world
</pre>
<p>
Note, it must be an <b>inner</b> declaration with instance name <b>world</b>
in order that this world object can be accessed from all objects in the
model. When dragging the \"World\" object from the package browser into
the diagram layer, this declaration is automatically generated
(this is defined via annotations in model World).
</p>
<p>
All vectors and tensors of a mechanical system are resolved in a
frame that is local to the corresponding component. Usually,
if all relative joint coordinates vanish, the local frames
of all components are parallel to each other, as well as to the
world frame (this holds as long as a Parts.FixedRotation,
component is <b>not</b> used). In this \"reference configuration\"
it is therefore
alternatively possible to resolve all vectors in the world
frame, since all frames are parallel to each other.
This is often very convenient. In order to give some visual
support in such a situation, in the icon of a World instance
two axes of the world frame are shown and the labels
of these axes can be set via parameters.
</p>
</HTML>
"));
          parameter Boolean enableAnimation = true "= true, if animation of all components is enabled";
          parameter Boolean animateWorld = true "= true, if world coordinate system shall be visualized" annotation(Dialog(enable = enableAnimation));
          parameter Boolean animateGravity = true "= true, if gravity field shall be visualized (acceleration vector or field center)" annotation(Dialog(enable = enableAnimation));
          parameter Types.AxisLabel label1 = "x" "Label of horizontal axis in icon";
          parameter Types.AxisLabel label2 = "y" "Label of vertical axis in icon";
          parameter Types.GravityTypes gravityType = GravityTypes.UniformGravity "Type of gravity field" annotation(Evaluate = true);
          parameter SI.Acceleration g = 9.81 "Constant gravity acceleration" annotation(Dialog(enable = gravityType == GravityTypes.UniformGravity));
          parameter Real[3] n = {0.0, -1.0,0.0} "Direction of gravity resolved in world frame (gravity = g*n/length(n))" annotation(Evaluate = true, Dialog(enable = gravityType == Modelica.Mechanics.MultiBody.Types.GravityTypes.UniformGravity));
          parameter Real mue(unit = "m3/s2", min = 0) = 398600000000000.0 "Gravity field constant (default = field constant of earth)" annotation(Dialog(enable = gravityType == Types.GravityTypes.PointGravity));
          parameter Boolean driveTrainMechanics3D = true "= true, if 3-dim. mechanical effects of Parts.Mounting1D/Rotor1D/BevelGear1D shall be taken into account";
          parameter SI.Distance axisLength = nominalLength / 2 "Length of world axes arrows" annotation(Dialog(tab = "Animation", group = "if animateWorld = true", enable = enableAnimation and animateWorld));
          parameter SI.Distance axisDiameter = axisLength / defaultFrameDiameterFraction "Diameter of world axes arrows" annotation(Dialog(tab = "Animation", group = "if animateWorld = true", enable = enableAnimation and animateWorld));
          parameter Boolean axisShowLabels = true "= true, if labels shall be shown" annotation(Dialog(tab = "Animation", group = "if animateWorld = true", enable = enableAnimation and animateWorld));
          annotation(Dialog(tab = "Animation", group = "if animateWorld = true", enable = enableAnimation and animateWorld));
          parameter SI.Position gravityArrowTail[3] = {0,0,0} "Position vector from origin of world frame to arrow tail, resolved in world frame" annotation(Dialog(tab = "Animation", group = "if animateGravity = true and gravityType = UniformGravity", enable = enableAnimation and animateGravity and gravityType == GravityTypes.UniformGravity));
          parameter SI.Length gravityArrowLength = axisLength / 2 "Length of gravity arrow" annotation(Dialog(tab = "Animation", group = "if animateGravity = true and gravityType = UniformGravity", enable = enableAnimation and animateGravity and gravityType == GravityTypes.UniformGravity));
          parameter SI.Diameter gravityArrowDiameter = gravityArrowLength / defaultWidthFraction "Diameter of gravity arrow" annotation(Dialog(tab = "Animation", group = "if animateGravity = true and gravityType = UniformGravity", enable = enableAnimation and animateGravity and gravityType == GravityTypes.UniformGravity));
          parameter SI.Diameter gravitySphereDiameter = 12742000 "Diameter of sphere representing gravity center (default = mean diameter of earth)" annotation(Dialog(tab = "Animation", group = "if animateGravity = true and gravityType = PointGravity", enable = enableAnimation and animateGravity and gravityType == GravityTypes.PointGravity));
          parameter SI.Length nominalLength = 1 "\"Nominal\" length of multi-body system" annotation(Dialog(tab = "Defaults"));
          parameter SI.Length defaultAxisLength = nominalLength / 5 "Default for length of a frame axis (but not world frame)" annotation(Dialog(tab = "Defaults"));
          parameter SI.Length defaultJointLength = nominalLength / 10 "Default for the fixed length of a shape representing a joint" annotation(Dialog(tab = "Defaults"));
          parameter SI.Length defaultJointWidth = nominalLength / 20 "Default for the fixed width of a shape representing a joint" annotation(Dialog(tab = "Defaults"));
          parameter SI.Length defaultForceLength = nominalLength / 10 "Default for the fixed length of a shape representing a force (e.g. damper)" annotation(Dialog(tab = "Defaults"));
          parameter SI.Length defaultForceWidth = nominalLength / 20 "Default for the fixed width of a shape represening a force (e.g. spring, bushing)" annotation(Dialog(tab = "Defaults"));
          parameter SI.Length defaultBodyDiameter = nominalLength / 9 "Default for diameter of sphere representing the center of mass of a body" annotation(Dialog(tab = "Defaults"));
          parameter Real defaultWidthFraction = 20 "Default for shape width as a fraction of shape length (e.g., for Parts.FixedTranslation)" annotation(Dialog(tab = "Defaults"));
          parameter SI.Length defaultArrowDiameter = nominalLength / 40 "Default for arrow diameter (e.g., of forces, torques, sensors)" annotation(Dialog(tab = "Defaults"));
          parameter Real defaultFrameDiameterFraction = 40 "Default for arrow diameter of a coordinate system as a fraction of axis length" annotation(Dialog(tab = "Defaults"));
          parameter Real defaultSpecularCoefficient(min = 0) = 0.7 "Default reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Defaults"));
          parameter Real defaultN_to_m(unit = "N/m", min = 0) = 1000 "Default scaling of force arrows (length = force/defaultN_to_m)" annotation(Dialog(tab = "Defaults"));
          parameter Real defaultNm_to_m(unit = "N.m/m", min = 0) = 1000 "Default scaling of torque arrows (length = torque/defaultNm_to_m)" annotation(Dialog(tab = "Defaults"));
        protected 
          parameter Integer ndim = if enableAnimation and animateWorld then 1 else 0;
          parameter Integer ndim2 = if enableAnimation and animateWorld and axisShowLabels then 1 else 0;
          parameter SI.Length headLength = min(axisLength, axisDiameter * Types.Defaults.FrameHeadLengthFraction);
          parameter SI.Length headWidth = axisDiameter * Types.Defaults.FrameHeadWidthFraction;
          parameter SI.Length lineLength = max(0, axisLength - headLength);
          parameter SI.Length lineWidth = axisDiameter;
          parameter SI.Length scaledLabel = Modelica.Mechanics.MultiBody.Types.Defaults.FrameLabelHeightFraction * axisDiameter;
          parameter SI.Length labelStart = 1.05 * axisLength;
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape x_arrowLine(shapeType = "cylinder", length = lineLength, width = lineWidth, height = lineWidth, lengthDirection = {1,0,0}, widthDirection = {0,1,0}, color = {1,2,3}, specularCoefficient = 0) if enableAnimation and animateWorld;
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape x_arrowHead(shapeType = "cone", length = headLength, width = headWidth, height = headWidth, lengthDirection = {1,0,0}, widthDirection = {0,1,0}, color = {1,2,3}, r = {lineLength,0,0}, specularCoefficient = 0) if enableAnimation and animateWorld;
          Modelica.Mechanics.MultiBody.Visualizers.Internal.Lines x_label(lines = scaledLabel * {[0,0;1,1],[0,1;1,0]}, diameter = axisDiameter, color = {1,2,3}, r_lines = {labelStart,0,0}, n_x = {1,0,0}, n_y = {0,1,0}, specularCoefficient = 0) if enableAnimation and animateWorld and axisShowLabels;
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape y_arrowLine(shapeType = "cylinder", length = lineLength, width = lineWidth, height = lineWidth, lengthDirection = {0,1,0}, widthDirection = {1,0,0}, color = {1,2,3}, specularCoefficient = 0) if enableAnimation and animateWorld;
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape y_arrowHead(shapeType = "cone", length = headLength, width = headWidth, height = headWidth, lengthDirection = {0,1,0}, widthDirection = {1,0,0}, color = {1,2,3}, r = {0,lineLength,0}, specularCoefficient = 0) if enableAnimation and animateWorld;
          Modelica.Mechanics.MultiBody.Visualizers.Internal.Lines y_label(lines = scaledLabel * {[0,0;1,1.5],[0,1.5;0.5,0.75]}, diameter = axisDiameter, color = {1,2,3}, r_lines = {0,labelStart,0}, n_x = {0,1,0}, n_y = { -1,0,0}, specularCoefficient = 0) if enableAnimation and animateWorld and axisShowLabels;
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape z_arrowLine(shapeType = "cylinder", length = lineLength, width = lineWidth, height = lineWidth, lengthDirection = {0,0,1}, widthDirection = {0,1,0}, color = {1,2,3}, specularCoefficient = 0) if enableAnimation and animateWorld;
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape z_arrowHead(shapeType = "cone", length = headLength, width = headWidth, height = headWidth, lengthDirection = {0,0,1}, widthDirection = {0,1,0}, color = {1,2,3}, r = {0,0,lineLength}, specularCoefficient = 0) if enableAnimation and animateWorld;
          Modelica.Mechanics.MultiBody.Visualizers.Internal.Lines z_label(lines = scaledLabel * {[0,0;1,0],[0,1;1,1],[0,1;1,0]}, diameter = axisDiameter, color = {1,2,3}, r_lines = {0,0,labelStart}, n_x = {0,0,1}, n_y = {0,1,0}, specularCoefficient = 0) if enableAnimation and animateWorld and axisShowLabels;
          parameter SI.Length gravityHeadLength = min(gravityArrowLength, gravityArrowDiameter * Types.Defaults.ArrowHeadLengthFraction);
          parameter SI.Length gravityHeadWidth = gravityArrowDiameter * Types.Defaults.ArrowHeadWidthFraction;
          parameter SI.Length gravityLineLength = max(0, gravityArrowLength - gravityHeadLength);
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape gravityArrowLine(shapeType = "cylinder", length = gravityLineLength, width = gravityArrowDiameter, height = gravityArrowDiameter, lengthDirection = n, widthDirection = {0,1,0}, color = {0,1,2}, r_shape = gravityArrowTail, specularCoefficient = 0) if enableAnimation and animateGravity and gravityType == GravityTypes.UniformGravity;
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape gravityArrowHead(shapeType = "cone", length = gravityHeadLength, width = gravityHeadWidth, height = gravityHeadWidth, lengthDirection = n, widthDirection = {0,1,0}, color = {0,1,2}, r_shape = gravityArrowTail + Modelica.Math.Vectors.normalize(n) * gravityLineLength, specularCoefficient = 0) if enableAnimation and animateGravity and gravityType == GravityTypes.UniformGravity;
          parameter Integer ndim_pointGravity = if enableAnimation and animateGravity and gravityType == 2 then 1 else 0;
          Modelica.Mechanics.MultiBody.Visualizers.Advanced.Shape gravitySphere(shapeType = "sphere", r_shape = { -gravitySphereDiameter / 2,0,0}, lengthDirection = {1,0,0}, length = gravitySphereDiameter, width = gravitySphereDiameter, height = gravitySphereDiameter, color = {0,1,2}, specularCoefficient = 0) if enableAnimation and animateGravity and gravityType == GravityTypes.PointGravity;
          function gravityAcceleration = gravityAccelerationTypes(gravityType = gravityType, g = g * Modelica.Math.Vectors.normalize(n), mue = mue);
        protected 
          function gravityAccelerationTypes "Gravity field acceleration depending on field type and position"
            import Modelica.Mechanics.MultiBody.Types.GravityTypes;
            extends Modelica.Icons.Function;
            input SI.Position r[3] "Position vector from world frame to actual point, resolved in world frame";
            input GravityTypes gravityType "Type of gravity field";
            input SI.Acceleration g[3] "Constant gravity acceleration, resolved in world frame, if gravityType=1";
            input Real mue(unit = "m3/s2") "Field constant of point gravity field, if gravityType=2";
            output SI.Acceleration gravity[3] "Gravity acceleration at point r, resolved in world frame";
          algorithm 
            gravity:=if gravityType == GravityTypes.UniformGravity then g else if gravityType == GravityTypes.PointGravity then  -mue / (r * r) * r / Modelica.Math.Vectors.length(r) else zeros(3);
          end gravityAccelerationTypes;

        equation 
          Connections.root(frame_b.R);
          assert(Modelica.Math.Vectors.length(n) > 1e-10, "Parameter n of World object is wrong (lenght(n) > 0 required)");
          frame_b.r_0 = zeros(3);
          frame_b.R = Frames.nullRotation();
        end World;
      end MultiBody;
      package Joints "Components that constrain the motion between two frames"
        import SI = Modelica.SIunits;
        extends Modelica.Icons.Library;
        annotation(Documentation(info = "<HTML>
<p>
This package contains <b>joint components</b>,
that is, idealized, massless elements that constrain
the motion between frames. In subpackage <b>Assemblies</b>
aggregation joint components are provided to handle
kinematic loops analytically (this means that non-linear systems
of equations occuring in these joint aggregations are analytically
solved, i.e., robustly and efficiently).
</p>
<h4>Content</h4>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Model</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.Prismatic\">Prismatic</a>
      <td valign=\"top\">Prismatic joint and actuated prismatic joint
          (1 translational degree-of-freedom, 2 potential states)<br>
      <IMG SRC=\"../Images/MultiBody/Joints/Prismatic.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.Revolute\">Revolute</a>
 </td>
      <td valign=\"top\">Revolute and actuated revolute joint
          (1 rotational degree-of-freedom, 2 potential states)<br>
      <IMG SRC=\"../Images/MultiBody/Joints/Revolute.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.Cylindrical\">Cylindrical</a></td>
      <td valign=\"top\">Cylindrical joint (2 degrees-of-freedom, 4 potential states)<br>
      <IMG SRC=\"../Images/MultiBody/Joints/Cylindrical.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.Universal\">Universal</a></td>
      <td valign=\"top\">Universal joint (2 degrees-of-freedom, 4 potential states)<br>
      <IMG SRC=\"../Images/MultiBody/Joints/Universal.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.Planar\">Planar</a></td>
      <td valign=\"top\">Planar joint (3 degrees-of-freedom, 6 potential states)<br>
      <IMG SRC=\"../Images/MultiBody/Joints/Planar.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.Spherical\">Spherical</a></td>
      <td valign=\"top\">Spherical joint (3 constraints and no potential states, or 3 degrees-of-freedom and 3 states)<br>
      <IMG SRC=\"../Images/MultiBody/Joints/Spherical.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.FreeMotion\">FreeMotion</a></td>
      <td valign=\"top\">Free motion joint (6 degrees-of-freedom, 12 potential states)<br>
      <IMG SRC=\"../Images/MultiBody/Joints/FreeMotion.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.SphericalSpherical\">SphericalSpherical</a></td>
      <td valign=\"top\">Spherical - spherical joint aggregation (1 constraint,
          no potential states) with an optional point mass in the middle<br>
      <IMG SRC=\"../Images/MultiBody/Joints/SphericalSpherical.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.UniversalSpherical\">UniversalSpherical</a></td>
      <td valign=\"top\">Universal - spherical joint aggregation (1 constraint, no potential states)<br>
      <IMG SRC=\"../Images/MultiBody/Joints/UniversalSpherical.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.GearConstraint\">GearConstraint</a></td>
      <td valign=\"top\">Ideal 3-dim. gearbox (arbitrary shaft directions)
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies\">MultiBody.Joints.Assemblies</a></td>
      <td valign=\"top\"><b>Package</b> of joint aggregations for analytic loop handling.
      </td>
  </tr>
</table>
</HTML>"));
        model Prismatic "Prismatic joint (1 translational degree-of-freedom, 2 potential states, optional axis flange)"
          import SI = Modelica.SIunits;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialElementaryJoint;
          Modelica.Mechanics.Translational.Interfaces.Flange_a axis if useAxisFlange "1-dim. translational flange that drives the joint" annotation(Placement(transformation(extent = {{90,50},{70,70}}, rotation = 0)));
          Modelica.Mechanics.Translational.Interfaces.Flange_b support if useAxisFlange "1-dim. translational flange of the drive drive support (assumed to be fixed in the world frame, NOT in the joint)" annotation(Placement(transformation(extent = {{ -30,50},{ -50,70}}, rotation = 0)));
          parameter Boolean useAxisFlange = false "= true, if axis flange is enabled" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n = {1,0,0} "Axis of translation resolved in frame_a (= same as in frame_b)" annotation(Evaluate = true);
          constant SI.Position s_offset = 0 "Relative distance offset (distance between frame_a and frame_b = s_offset + s)" annotation(Evaluate = false);
          parameter Types.Axis boxWidthDirection = {0,1,0} "Vector in width direction of box, resolved in frame_a" annotation(Evaluate = true, Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance boxWidth = world.defaultJointWidth "Width of prismatic joint box" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance boxHeight = boxWidth "Height of prismatic joint box" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color boxColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of prismatic joint box" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter StateSelect stateSelect = StateSelect.prefer "Priority to use distance s and v=der(s) as states" annotation(Dialog(tab = "Advanced"));
          final parameter Real e[3](each final unit = "1") = Modelica.Math.Vectors.normalize(n) "Unit vector in direction of prismatic axis n";
          SI.Position s(start = 0, final stateSelect = stateSelect) "Relative distance between frame_a and frame_b" annotation(unassignedMessage = "
The relative distance s of a prismatic joint cannot be determined.
Possible reasons:
- A non-zero mass might be missing on either side of the parts
  connected to the prismatic joint.
- Too many StateSelect.always are defined and the model
  has less degrees of freedom as specified with this setting
  (remove all StateSelect.always settings).
");
          SI.Velocity v(start = 0, final stateSelect = stateSelect) "First derivative of s (relative velocity)";
          SI.Acceleration a(start = 0) "Second derivative of s (relative acceleration)";
          SI.Force f "Actuation force in direction of joint axis";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100, -50},{ -30,41}}, pattern = LinePattern.None, fillColor = {192,192,192}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{ -100,40},{ -30,50}}, pattern = LinePattern.None, fillColor = {0,0,0}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{ -30, -30},{100,20}}, pattern = LinePattern.None, fillColor = {192,192,192}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{ -30,20},{100,30}}, pattern = LinePattern.None, fillColor = {0,0,0}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -30, -50},{ -30,50}}, color = {0,0,0}),Line(points = {{100, -30},{100,21}}, color = {0,0,0}),Text(extent = {{60,12},{96, -13}}, lineColor = {128,128,128}, textString = "b"),Text(extent = {{ -95,13},{ -60, -9}}, lineColor = {128,128,128}, textString = "a"),Text(visible = useAxisFlange, extent = {{ -150, -155},{150, -95}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -150, -93},{150, -54}}, lineColor = {0,0,0}, textString = "n=%n"),Rectangle(visible = useAxisFlange, extent = {{90,30},{100,70}}, pattern = LinePattern.None, fillColor = {192,192,192}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(visible = not useAxisFlange, extent = {{ -150,55},{150,115}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Documentation(info = "<HTML>
<p>
Joint where frame_b is translated along axis n which is fixed in frame_a.
The two frames coincide when the relative distance \"s = 0\".
</p>

<p>
Optionally, two additional 1-dimensional mechanical flanges
(flange \"axis\" represents the driving flange and
flange \"support\" represents the bearing) can be enabled via
parameter <b>useAxisFlange</b>. The enabled axis flange can be
driven with elements of the
<a href=\"Modelica://Modelica.Mechanics.Translational\">Modelica.Mechanics.Translational</a>
library.

</p>

<p>
In the \"Advanced\" menu it can be defined via parameter <b>stateSelect</b>
that the relative distance \"s\" and its derivative shall be definitely
used as states by setting stateSelect=StateSelect.always.
Default is StateSelect.prefer to use the relative distance and its
derivative as preferred states. The states are usually selected automatically.
In certain situations, especially when closed kinematic loops are present,
it might be slightly more efficient, when using the StateSelect.always setting.
</p>

<p>
In the following figure the animation of a prismatic
joint is shown. The light blue coordinate system is
frame_a and the dark blue coordinate system is
frame_b of the joint. The black arrow is parameter
vector \"n\" defining the translation axis
(here: n = {1,1,0}).
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Joints/Prismatic.png\">
</p>

</HTML>
"));
        protected 
          Visualizers.Advanced.Shape box(shapeType = "box", color = boxColor, specularCoefficient = specularCoefficient, length = if noEvent(abs(s + s_offset) > 1e-06) then s + s_offset else 1e-06, width = boxWidth, height = boxHeight, lengthDirection = e, widthDirection = boxWidthDirection, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          Translational.Components.Fixed fixed annotation(Placement(transformation(extent = {{ -50,30},{ -30,50}})));
          Translational.Interfaces.InternalSupport internalAxis(f = f) annotation(Placement(transformation(extent = {{70,50},{90,30}})));
          Translational.Sources.ConstantForce constantForce(f_constant = 0) if not useAxisFlange annotation(Placement(transformation(extent = {{40,30},{60,50}})));

        equation 
          v = der(s);
          a = der(v);
          frame_b.r_0 = frame_a.r_0 + Frames.resolve1(frame_a.R, e * (s_offset + s));
          frame_b.R = frame_a.R;
          zeros(3) = frame_a.f + frame_b.f;
          zeros(3) = frame_a.t + frame_b.t + cross(e * (s_offset + s), frame_b.f);
          f =  -e * frame_b.f;
          s = internalAxis.s;
          connect(fixed.flange,support) annotation(Line(points = {{ -40,40},{ -40,60}}, color = {0,127,0}, smooth = Smooth.None));
          connect(internalAxis.flange,axis) annotation(Line(points = {{80,40},{80,60}}, color = {0,127,0}, smooth = Smooth.None));
          connect(constantForce.flange,internalAxis.flange) annotation(Line(points = {{60,40},{80,40}}, color = {0,127,0}, smooth = Smooth.None));
        end Prismatic;
        model Revolute "Revolute joint (1 rotational degree-of-freedom, 2 potential states, optional axis flange)"
          import SI = Modelica.SIunits;
          Modelica.Mechanics.Rotational.Interfaces.Flange_a axis if useAxisFlange "1-dim. rotational flange that drives the joint" annotation(Placement(transformation(extent = {{10,90},{ -10,110}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Interfaces.Flange_b support if useAxisFlange "1-dim. rotational flange of the drive support (assumed to be fixed in the world frame, NOT in the joint)" annotation(Placement(transformation(extent = {{ -70,90},{ -50,110}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Coordinate system fixed to the joint with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_b "Coordinate system fixed to the joint with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{84, -16},{116,16}}, rotation = 0)));
          parameter Boolean useAxisFlange = false "= true, if axis flange is enabled" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
          parameter Boolean animation = true "= true, if animation shall be enabled (show axis as cylinder)";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n = {0,0,1} "Axis of rotation resolved in frame_a (= same as in frame_b)" annotation(Evaluate = true);
          constant SI.Angle phi_offset = 0 "Relative angle offset (angle = phi_offset + phi)";
          parameter SI.Distance cylinderLength = world.defaultJointLength "Length of cylinder representing the joint axis" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of cylinder representing the joint axis" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Modelica.Mechanics.MultiBody.Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinder representing the joint axis" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Modelica.Mechanics.MultiBody.Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter StateSelect stateSelect = StateSelect.prefer "Priority to use joint angle phi and w=der(phi) as states" annotation(Dialog(tab = "Advanced"));
          SI.Angle phi(start = 0, final stateSelect = stateSelect) "Relative rotation angle from frame_a to frame_b" annotation(unassignedMessage = "
The rotation angle phi of a revolute joint cannot be determined.
Possible reasons:
- A non-zero mass might be missing on either side of the parts
  connected to the revolute joint.
- Too many StateSelect.always are defined and the model
  has less degrees of freedom as specified with this setting
  (remove all StateSelect.always settings).
");
          SI.AngularVelocity w(start = 0, stateSelect = stateSelect) "First derivative of angle phi (relative angular velocity)";
          SI.AngularAcceleration a(start = 0) "Second derivative of angle phi (relative angular acceleration)";
          SI.Torque tau "Driving torque in direction of axis of rotation";
          SI.Angle angle "= phi_offset + phi";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100, -60},{ -30,60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{30, -60},{100,60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -100,59},{ -30, -60}}, lineColor = {0,0,0}),Rectangle(extent = {{30,60},{100, -60}}, lineColor = {0,0,0}),Text(extent = {{ -90,14},{ -54, -11}}, lineColor = {128,128,128}, textString = "a"),Text(extent = {{51,11},{87, -14}}, lineColor = {128,128,128}, textString = "b"),Line(visible = useAxisFlange, points = {{ -20,80},{ -20,60}}, color = {0,0,0}),Line(visible = useAxisFlange, points = {{20,80},{20,60}}, color = {0,0,0}),Rectangle(visible = useAxisFlange, extent = {{ -10,100},{10,50}}, lineColor = {0,0,0}, fillPattern = FillPattern.VerticalCylinder, fillColor = {192,192,192}),Polygon(visible = useAxisFlange, points = {{ -10,30},{10,30},{30,50},{ -30,50},{ -10,30}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -30,11},{30, -10}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(visible = useAxisFlange, points = {{10,30},{30,50},{30, -50},{10, -30},{10,30}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -150, -117},{150, -77}}, lineColor = {0,0,0}, textString = "n=%n"),Text(visible = useAxisFlange, extent = {{ -150, -175},{150, -115}}, textString = "%name", lineColor = {0,0,255}),Line(visible = useAxisFlange, points = {{ -20,70},{ -60,70},{ -60,60}}, color = {0,0,0}, smooth = Smooth.None),Line(visible = useAxisFlange, points = {{20,70},{50,70},{50,60}}, color = {0,0,0}, smooth = Smooth.None),Line(visible = useAxisFlange, points = {{ -90,100},{ -30,100}}, color = {0,0,0}),Line(visible = useAxisFlange, points = {{ -30,100},{ -50,80}}, color = {0,0,0}),Line(visible = useAxisFlange, points = {{ -49,100},{ -70,80}}, color = {0,0,0}),Line(visible = useAxisFlange, points = {{ -70,100},{ -90,80}}, color = {0,0,0}),Text(visible = not useAxisFlange, extent = {{ -150,70},{150,130}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Documentation(info = "<html>

<p>
Joint where frame_b rotates around axis n which is fixed in frame_a.
The two frames coincide when the rotation angle \"phi = 0\".
</p>

<p>
Optionally, two additional 1-dimensional mechanical flanges
(flange \"axis\" represents the driving flange and
flange \"support\" represents the bearing) can be enabled via
parameter <b>useAxisFlange</b>. The enabled axis flange can be
driven with elements of the
<a href=\"Modelica://Modelica.Mechanics.Rotational\">Modelica.Mechanics.Rotational</a>
library.

</p>

<p>
In the \"Advanced\" menu it can be defined via parameter <b>stateSelect</b>
that the rotation angle \"phi\" and its derivative shall be definitely
used as states by setting stateSelect=StateSelect.always.
Default is StateSelect.prefer to use the joint angle and its
derivative as preferred states. The states are usually selected automatically.
In certain situations, especially when closed kinematic loops are present,
it might be slightly more efficient, when using the StateSelect.always setting.
</p>
<p>
If a <b>planar loop</b> is present, e.g., consisting of 4 revolute joints
where the joint axes are all parallel to each other, then there is no
longer a unique mathematical solution and the symbolic algorithms will
fail. Usually, an error message will be printed pointing out this
situation. In this case, one revolute joint of the loop has to be replaced
by a Joints.RevolutePlanarLoopConstraint joint. The
effect is that from the 5 constraints of a usual revolute joint,
3 constraints are removed and replaced by appropriate known
variables (e.g., the force in the direction of the axis of rotation is
treated as known with value equal to zero; for standard revolute joints,
this force is an unknown quantity).
</p>

<p>
In the following figure the animation of a revolute
joint is shown. The light blue coordinate system is
frame_a and the dark blue coordinate system is
frame_b of the joint. The black arrow is parameter
vector \"n\" defining the translation axis
(here: n = {0,0,1}, phi.start = 45<sup>o</sup>).
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Joints/Revolute.png\">
</p>

</html>
"));
        protected 
          outer Modelica.Mechanics.MultiBody.World world;
          parameter Real e[3](each final unit = "1") = Modelica.Math.Vectors.normalize(n) "Unit vector in direction of rotation axis, resolved in frame_a (= same as in frame_b)";
          Frames.Orientation R_rel "Relative orientation object from frame_a to frame_b or from frame_b to frame_a";
          Visualizers.Advanced.Shape cylinder(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = e, widthDirection = {0,1,0}, r_shape =  -e * cylinderLength / 2, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
        protected 
          Modelica.Mechanics.Rotational.Components.Fixed fixed "support flange is fixed to ground" annotation(Placement(transformation(extent = {{ -70,70},{ -50,90}})));
          Rotational.Interfaces.InternalSupport internalAxis(tau = tau) annotation(Placement(transformation(extent = {{ -10,90},{10,70}})));
          Rotational.Sources.ConstantTorque constantTorque(tau_constant = 0) if not useAxisFlange annotation(Placement(transformation(extent = {{40,70},{20,90}})));

        equation 
          Connections.branch(frame_a.R, frame_b.R);
          assert(cardinality(frame_a) > 0, "Connector frame_a of revolute joint is not connected");
          assert(cardinality(frame_b) > 0, "Connector frame_b of revolute joint is not connected");
          angle = phi_offset + phi;
          w = der(phi);
          a = der(w);
          frame_b.r_0 = frame_a.r_0;
          if rooted(frame_a.R) then
            R_rel = Frames.planarRotation(e, phi_offset + phi, w);
            frame_b.R = Frames.absoluteRotation(frame_a.R, R_rel);
            frame_a.f =  -Frames.resolve1(R_rel, frame_b.f);
            frame_a.t =  -Frames.resolve1(R_rel, frame_b.t);

          else
            R_rel = Frames.planarRotation( -e, phi_offset + phi, w);
            frame_a.R = Frames.absoluteRotation(frame_b.R, R_rel);
            frame_b.f =  -Frames.resolve1(R_rel, frame_a.f);
            frame_b.t =  -Frames.resolve1(R_rel, frame_a.t);

          end if;
          tau =  -frame_b.t * e;
          phi = internalAxis.phi;
          connect(fixed.flange,support) annotation(Line(points = {{ -60,80},{ -60,100}}, color = {0,0,0}, smooth = Smooth.None));
          connect(internalAxis.flange,axis) annotation(Line(points = {{0,80},{0,100}}, color = {0,0,0}, smooth = Smooth.None));
          connect(constantTorque.flange,internalAxis.flange) annotation(Line(points = {{20,80},{0,80}}, color = {0,0,0}, smooth = Smooth.None));
        end Revolute;
        model RevolutePlanarLoopConstraint "Revolute joint that is described by 2 positional constraints for usage in a planar loop (the ambiguous cut-force perpendicular to the loop and the ambiguous cut-torques are set arbitrarily to zero)"
          import SI = Modelica.SIunits;
          import Cv = Modelica.SIunits.Conversions;
          import T = Modelica.Mechanics.MultiBody.Frames.TransformationMatrices;
          import Modelica.Mechanics.MultiBody.Types;
          Interfaces.Frame_a frame_a "Coordinate system fixed to the joint with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
          Interfaces.Frame_b frame_b "Coordinate system fixed to the joint with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{84, -16},{116,16}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled (show axis as cylinder)";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n = {0,0,1} "Axis of rotation resolved in frame_a (= same as in frame_b)" annotation(Evaluate = true);
          parameter SI.Distance cylinderLength = world.defaultJointLength "Length of cylinder representing the joint axis" annotation(Dialog(group = "if animation = true", enable = animation));
          parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of cylinder representing the joint axis" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinder representing the joint axis" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(group = "if animation = true", enable = animation));
          annotation(defaultComponentName = "revolute", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -146,70},{143,109}}, lineColor = {0,0,0}, textString = "n=%n"),Text(extent = {{ -137, -125},{139, -68}}, textString = "%name", lineColor = {0,0,255}),Rectangle(extent = {{ -20,10},{20, -10}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -100, -60},{ -20,60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{20, -60},{100,60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -100,59},{ -20, -60}}, lineColor = {0,0,0}),Rectangle(extent = {{20,60},{100, -60}}, lineColor = {0,0,0}),Text(extent = {{ -90,14},{ -54, -11}}, lineColor = {128,128,128}, textString = "a"),Text(extent = {{51,11},{87, -14}}, lineColor = {128,128,128}, textString = "b"),Line(points = {{ -91, -76},{ -33,15},{30, -49},{87,61}}, color = {255,0,0}, thickness = 0.5, smooth = Smooth.None)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100, -60},{ -20,60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -20,10},{20, -10}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{20, -60},{100,60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192})}), Documentation(info = "<HTML>
<p>
Joint where frame_b rotates around axis n which is fixed in frame_a and
where this joint is used in a planar loop providing 2 constraint equations
on position level.
</p>

<p>
If a <b>planar loop</b> is present, e.g., consisting of 4 revolute joints
where the joint axes are all parallel to each other, then there is no
unique mathematical solution if all revolute joints are modelled with
Joints.Revolute and the symbolic algorithms will
fail. The reason is that, e.g., the cut-forces in the revolute joints perpendicular
to the planar loop are not uniquely defined when 3-dim. descriptions of revolute
joints are used. Usually, an error message will be printed pointing out this
situation. In this case, <b>one</b> revolute joint in the loop has to be replaced by
model Joints.RevolutePlanarLoopCutJoint. The
effect is that from the 5 constraints of a 3-dim. revolute joint,
3 constraints are removed and replaced by appropriate known
variables (e.g., the force in the direction of the axis of rotation is
treated as known with value equal to zero; for standard revolute joints,
this force is an unknown quantity).
</p>


</HTML>
"));
        protected 
          outer Modelica.Mechanics.MultiBody.World world;
          parameter Real e[3](each final unit = "1") = Modelica.Math.Vectors.normalize(n) "Unit vector in direction of rotation axis, resolved in frame_a (= same as in frame_b)";
          parameter Real nnx_a[3](each final unit = "1") = if abs(e[1]) > 0.1 then {0,1,0} else if abs(e[2]) > 0.1 then {0,0,1} else {1,0,0} "Arbitrary vector that is not aligned with rotation axis n" annotation(Evaluate = true);
          parameter Real ey_a[3](each final unit = "1") = Modelica.Math.Vectors.normalize(cross(e, nnx_a)) "Unit vector orthogonal to axis n of revolute joint, resolved in frame_a" annotation(Evaluate = true);
          parameter Real ex_a[3](each final unit = "1") = cross(ey_a, e) "Unit vector orthogonal to axis n of revolute joint and to ey_a, resolved in frame_a" annotation(Evaluate = true);
          Real ey_b[3](each final unit = "1") "ey_a, resolved in frame_b";
          Real ex_b[3](each final unit = "1") "ex_a, resolved in frame_b";
          Frames.Orientation R_rel "Dummy or relative orientation object from frame_a to frame_b";
          Modelica.SIunits.Position r_rel_a[3] "Position vector from origin of frame_a to origin of frame_b, resolved in frame_a";
          SI.Force f_c[2] "Dummy or constraint forces in direction of ex_a, ey_a";
          Visualizers.Advanced.Shape cylinder(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = e, widthDirection = {0,1,0}, r_shape =  -e * cylinderLength / 2, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;

        equation 
          assert(cardinality(frame_a) > 0, "Connector frame_a of revolute joint is not connected");
          assert(cardinality(frame_b) > 0, "Connector frame_b of revolute joint is not connected");
          R_rel = Frames.relativeRotation(frame_a.R, frame_b.R);
          r_rel_a = Frames.resolve2(frame_a.R, frame_b.r_0 - frame_a.r_0);
          0 = ex_a * r_rel_a;
          0 = ey_a * r_rel_a;
          frame_a.t = zeros(3);
          frame_b.t = zeros(3);
          frame_a.f = [ex_a,ey_a] * f_c;
          frame_b.f =  -Frames.resolve2(R_rel, frame_a.f);
          ex_b = Frames.resolve2(R_rel, ex_a);
          ey_b = Frames.resolve2(R_rel, ey_a);
          assert(noEvent(abs(e * r_rel_a) <= 1e-10 and abs(e * ex_b) <= 1e-10 and abs(e * ey_b) <= 1e-10), "
The MultiBody.Joints.RevolutePlanarLoopConstraint joint is used as cut-joint of a
planar loop. However, the revolute joint is not part of a planar loop where the
axis of the revolute joint (parameter n) is orthogonal to the possible movements.
Either use instead joint MultiBody.Joints.Revolute or correct the
definition of the axes vectors n in the revolute joints of the planar loop.
");
        end RevolutePlanarLoopConstraint;
        model Cylindrical "Cylindrical joint (2 degrees-of-freedom, 4 potential states)"
          import SI = Modelica.SIunits;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
          parameter Boolean animation = true "= true, if animation shall be enabled (show cylinder)";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n = {1,0,0} "Cylinder axis resolved in frame_a (= same as in frame_b)" annotation(Evaluate = true);
          parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of cylinder" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinder" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter StateSelect stateSelect = StateSelect.prefer "Priority to use joint coordinates (phi, s, w, v) as states" annotation(Dialog(tab = "Advanced"));
          Prismatic prismatic(n = n, animation = false, stateSelect = StateSelect.never) annotation(Placement(transformation(extent = {{ -70, -25},{ -15,25}}, rotation = 0)));
          Revolute revolute(n = n, animation = false, stateSelect = StateSelect.never) annotation(Placement(transformation(extent = {{10, -25},{65,25}}, rotation = 0)));
          SI.Position s(start = 0, stateSelect = stateSelect) "Relative distance between frame_a and frame_b";
          SI.Angle phi(start = 0, stateSelect = stateSelect) "Relative rotation angle from frame_a to frame_b";
          SI.Velocity v(start = 0, stateSelect = stateSelect) "First derivative of s (relative velocity)";
          SI.AngularVelocity w(start = 0, stateSelect = stateSelect) "First derivative of angle phi (relative angular velocity)";
          SI.Acceleration a(start = 0) "Second derivative of s (relative acceleration)";
          SI.AngularAcceleration wd(start = 0) "Second derivative of angle phi (relative angular acceleration)";
          annotation(Documentation(info = "<HTML>
<p>
Joint where frame_b rotates around and translates along axis n
which is fixed in frame_a. The two frames coincide when
\"phi=revolute.phi=0\" and \"s=prismatic.s=0\". This joint
has the following potential states;
<ul>
<li> The relative angle phi [rad] around axis n, </li>
<li> the relative distance s [m] along axis n, </li>
<li> the relative angular velocity w [rad/s] (= der(phi))
     and </li>
<li> the relative velocity v [m/s] (= der(s)).</li>
</ul>
<p>
They are used as candidates for automatic selection of states
from the tool. This may be enforced by setting \"stateSelect=StateSelect.<b>always</b>\"
in the <b>Advanced</b> menu. The states are usually selected automatically.
In certain situations, especially when closed kinematic loops are present,
it might be slightly more efficient, when using the \"StateSelect.always\" setting.
</p>
<p>
In the following figure the animation of a cylindrical
joint is shown. The light blue coordinate system is
frame_a and the dark blue coordinate system is
frame_b of the joint. The black arrow is parameter
vector \"n\" defining the cylinder axis
(here: n = {0,0,1}).
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Joints/Cylindrical.png\">
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100, -50},{0,50}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{0, -30},{100,30}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Line(points = {{0, -50},{0,50}}, color = {0,0,0}),Text(extent = {{ -126,116},{134,56}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -118, -65},{120, -102}}, lineColor = {0,0,0}, textString = "n=%n")}));
        protected 
          Visualizers.Advanced.Shape cylinder(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = prismatic.s, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = prismatic.n, widthDirection = {0,1,0}, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation annotation(Placement(transformation(extent = {{ -20,40},{0,60}}, rotation = 0)));

        equation 
          phi = revolute.phi;
          w = der(phi);
          wd = der(w);
          s = prismatic.s;
          v = der(s);
          a = der(v);
          connect(frame_a,prismatic.frame_a) annotation(Line(points = {{ -100,0},{ -70,0}}, color = {95,95,95}, thickness = 0.5));
          connect(prismatic.frame_b,revolute.frame_a) annotation(Line(points = {{ -15,0},{10,0}}, color = {95,95,95}, thickness = 0.5));
          connect(revolute.frame_b,frame_b) annotation(Line(points = {{65,0},{100,0}}, color = {95,95,95}, thickness = 0.5));
        end Cylindrical;
        model Universal "Universal joint (2 degrees-of-freedom, 4 potential states)"
          import SI = Modelica.SIunits;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n_a = {1,0,0} "Axis of revolute joint 1 resolved in frame_a" annotation(Evaluate = true);
          parameter Modelica.Mechanics.MultiBody.Types.Axis n_b = {0,1,0} "Axis of revolute joint 2 resolved in frame_b" annotation(Evaluate = true);
          parameter SI.Distance cylinderLength = world.defaultJointLength "Length of cylinders representing the joint axes" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of cylinders representing the joint axes" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinders representing the joint axes" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter StateSelect stateSelect = StateSelect.prefer "Priority to use joint coordinates (phi_a, phi_b, w_a, w_b) as states" annotation(Dialog(tab = "Advanced"));
          Modelica.Mechanics.MultiBody.Joints.Revolute revolute_a(n = n_a, stateSelect = StateSelect.never, cylinderDiameter = cylinderDiameter, cylinderLength = cylinderLength, cylinderColor = cylinderColor, specularCoefficient = specularCoefficient, animation = animation) annotation(Placement(transformation(extent = {{ -60, -25},{ -10,25}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Joints.Revolute revolute_b(n = n_b, stateSelect = StateSelect.never, animation = animation, cylinderDiameter = cylinderDiameter, cylinderLength = cylinderLength, cylinderColor = cylinderColor, specularCoefficient = specularCoefficient) annotation(Placement(transformation(origin = {35,45}, extent = {{ -25, -25},{25,25}}, rotation = 90)));
          SI.Angle phi_a(start = 0, stateSelect = stateSelect) "Relative rotation angle from frame_a to intermediate frame";
          SI.Angle phi_b(start = 0, stateSelect = stateSelect) "Relative rotation angle from intermediate frame to frame_b";
          SI.AngularVelocity w_a(start = 0, stateSelect = stateSelect) "First derivative of angle phi_a (relative angular velocity a)";
          SI.AngularVelocity w_b(start = 0, stateSelect = stateSelect) "First derivative of angle phi_b (relative angular velocity b)";
          SI.AngularAcceleration a_a(start = 0) "Second derivative of angle phi_a (relative angular acceleration a)";
          SI.AngularAcceleration a_b(start = 0) "Second derivative of angle phi_b (relative angular acceleration b)";
          annotation(Documentation(info = "<HTML>
<p>
Joint where frame_a rotates around axis n_a which is fixed in frame_a
and frame_b rotates around axis n_b which is fixed in frame_b.
The two frames coincide when
\"revolute_a.phi=0\" and \"revolute_b.phi=0\". This joint
has the following potential states;
<ul>
<li> The relative angle phi_a = revolute_a.phi [rad] around axis n_a, </li>
<li> the relative angle phi_b = revolute_b.phi [rad] around axis n_b, </li>
<li> the relative angular velocity w_a (= der(phi_a))  and </li>
<li> the relative angular velocity w_b (= der(phi_b)).</li>
</ul>
<p>
They are used as candidates for automatic selection of states
from the tool. This may be enforced by setting \"stateSelect=StateSelect.<b>always</b>\"
in the <b>Advanced</b> menu. The states are usually selected automatically.
In certain situations, especially when closed kinematic loops are present,
it might be slightly more efficient, when using the \"StateSelect.always\" setting.
</p>

<p>
In the following figure the animation of a universal
joint is shown. The light blue coordinate system is
frame_a and the dark blue coordinate system is
frame_b of the joint
(here: n_a = {0,0,1}, n_b = {0,1,0}, phi_a.start = 90<sup>o</sup>,
phi_b.start = 45<sup>o</sup>).
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Joints/Universal.png\">
</p>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,15},{ -65, -15}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {235,235,235}),Ellipse(extent = {{ -80, -80},{80,80}}, lineColor = {160,160,164}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -60, -60},{60,60}}, lineColor = {160,160,164}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -134, -70},{126, -130}}, textString = "%name", lineColor = {0,0,255}),Rectangle(extent = {{12,82},{80, -82}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{56,15},{100, -15}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {235,235,235}),Line(points = {{12,78},{12, -78}}, color = {0,0,0}, thickness = 0.5),Ellipse(extent = {{ -52, -40},{80,40}}, lineColor = {160,160,164}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -32, -20},{60,26}}, lineColor = {160,160,164}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -22, -54},{ -60,0},{ -22,50},{40,52},{ -22, -54}}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{12,78},{12, -20}}, color = {0,0,0}, thickness = 0.5),Line(points = {{32,38},{ -12, -36}}, color = {0,0,0}, thickness = 0.5)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          phi_a = revolute_a.phi;
          phi_b = revolute_b.phi;
          w_a = der(phi_a);
          w_b = der(phi_b);
          a_a = der(w_a);
          a_b = der(w_b);
          connect(frame_a,revolute_a.frame_a) annotation(Line(points = {{ -100,0},{ -60,0}}, color = {95,95,95}, thickness = 0.5));
          connect(revolute_b.frame_b,frame_b) annotation(Line(points = {{35,70},{35,90},{70,90},{70,0},{100,0}}, color = {95,95,95}, thickness = 0.5));
          connect(revolute_a.frame_b,revolute_b.frame_a) annotation(Line(points = {{ -10,0},{35,0},{35,20}}, color = {95,95,95}, thickness = 0.5));
        end Universal;
        model Planar "Planar joint (3 degrees-of-freedom, 6 potential states)"
          import SI = Modelica.SIunits;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n = {0,0,1} "Axis orthogonal to unconstrained plane, resolved in frame_a (= same as in frame_b)" annotation(Evaluate = true);
          parameter Modelica.Mechanics.MultiBody.Types.Axis n_x = {1,0,0} "Vector in direction of x-axis of plane, resolved in frame_a (n_x shall be orthogonal to n)" annotation(Evaluate = true);
          parameter SI.Distance cylinderLength = world.defaultJointLength "Length of revolute cylinder" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of revolute cylinder" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of revolute cylinder" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance boxWidth = 0.3 * cylinderDiameter "Width of prismatic joint boxes" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance boxHeight = boxWidth "Height of prismatic joint boxes" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color boxColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of prismatic joint boxes" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter StateSelect stateSelect = StateSelect.prefer "Priority to use joint coordinates (s_x, s_y, phi, v_x, v_y, w) as states" annotation(Dialog(tab = "Advanced"));
          Prismatic prismatic_x(stateSelect = StateSelect.never, n = cross(cross(n, n_x), n), animation = false) annotation(Placement(transformation(extent = {{ -69, -20},{ -29,20}}, rotation = 0)));
          Prismatic prismatic_y(stateSelect = StateSelect.never, n = cross(n, n_x), animation = false) annotation(Placement(transformation(origin = {0,50}, extent = {{ -20, -20},{20,20}}, rotation = 90)));
          Revolute revolute(stateSelect = StateSelect.never, n = n, animation = false) annotation(Placement(transformation(extent = {{41, -20},{81,20}}, rotation = 0)));
          SI.Position s_x(start = 0, stateSelect = stateSelect) "Relative distance along first prismatic joint starting at frame_a";
          SI.Position s_y(start = 0, stateSelect = stateSelect) "Relative distance along second prismatic joint starting at first prismatic joint";
          SI.Angle phi(start = 0, stateSelect = stateSelect) "Relative rotation angle from frame_a to frame_b";
          SI.Velocity v_x(start = 0, stateSelect = stateSelect) "First derivative of s_x (relative velocity in s_x direction)";
          SI.Velocity v_y(start = 0, stateSelect = stateSelect) "First derivative of s_y (relative velocity in s_y direction)";
          SI.AngularVelocity w(start = 0, stateSelect = stateSelect) "First derivative of angle phi (relative angular velocity)";
          SI.Acceleration a_x(start = 0) "Second derivative of s_x (relative acceleration in s_x direction)";
          SI.Acceleration a_y(start = 0) "Second derivative of s_y (relative acceleration in s_y direction)";
          SI.AngularAcceleration wd(start = 0) "Second derivative of angle phi (relative angular acceleration)";
          annotation(Documentation(info = "<HTML>
<p>
Joint where frame_b can move in a plane and can rotate around an
axis orthogonal to the plane. The plane is defined by
vector n which is perpendicular to the plane and by vector n_x,
which points in the direction of the x-axis of the plane.
frame_a and frame_b coincide when s_x=prismatic_x.s=0,
s_y=prismatic_y.s=0 and phi=revolute.phi=0. This joint has the following
potential states:
<ul>
<li> the relative distance s_x = prismatic_x.s [m] along axis n_x, </li>
<li> the relative distance s_y = prismatic_y.s [m] along axis n_y = cross(n,n_x), </li>
<li> the relative angle phi = revolute.phi [rad] around axis n, </li>
<li> the relative velocity v_x (= der(s_x)).</li>
<li> the relative velocity v_y (= der(s_y)).</li>
<li> the relative angular velocity w (= der(phi))</li>
</ul>
<p>
They are used as candidates for automatic selection of states
from the tool. This may be enforced by setting \"stateSelect=StateSelect.<b>always</b>\"
in the <b>Advanced</b> menu. The states are usually selected automatically.
In certain situations, especially when closed kinematic loops are present,
it might be slightly more efficient, when using the \"StateSelect.always\" setting.
</p>
<p>
In the following figure the animation of a planar
joint is shown. The light blue coordinate system is
frame_a and the dark blue coordinate system is
frame_b of the joint. The black arrows are parameter
vectors \"n\" and \"n_x\"
(here: n = {0,1,0}, n_x = {0,0,1}, s_x.start = 0.5,
s_y.start = 0.5, phi.start = 45<sup>o</sup>).
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Joints/Planar.png\">
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -30, -60},{ -10,60}}, lineColor = {0,0,0}, pattern = LinePattern.None, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{10, -60},{30,60}}, lineColor = {0,0,0}, pattern = LinePattern.None, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -100, -10},{ -30,10}}, lineColor = {0,0,0}, pattern = LinePattern.None, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{100, -10},{30,10}}, lineColor = {0,0,0}, pattern = LinePattern.None, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -131, -74},{133, -112}}, lineColor = {0,0,0}, textString = "n=%n"),Text(extent = {{ -124,127},{136,67}}, textString = "%name", lineColor = {0,0,255})}));
        protected 
          parameter Integer ndim = if world.enableAnimation and animation then 1 else 0;
          parameter Real e[3](each final unit = "1") = Modelica.Math.Vectors.normalize(n);
        protected 
          Visualizers.Advanced.Shape box_x[ndim](each shapeType = "box", each color = boxColor, each length = prismatic_x.s, each width = boxWidth, each height = boxWidth, each lengthDirection = prismatic_x.e, each widthDirection = {0,1,0}, each r = frame_a.r_0, each R = frame_a.R) annotation(Placement(transformation(extent = {{ -80,30},{ -60,50}}, rotation = 0)));
          Visualizers.Advanced.Shape box_y[ndim](each shapeType = "box", each color = boxColor, each length = prismatic_y.s, each width = boxWidth, each height = boxWidth, each lengthDirection = prismatic_y.e, each widthDirection = {1,0,0}, each r = prismatic_y.frame_a.r_0, each R = prismatic_y.frame_a.R) annotation(Placement(transformation(extent = {{ -46,69},{ -26,89}}, rotation = 0)));
          Visualizers.Advanced.Shape cylinder[ndim](each shapeType = "cylinder", each color = cylinderColor, each length = cylinderLength, each width = cylinderDiameter, each height = cylinderDiameter, each lengthDirection = n, each widthDirection = {0,1,0}, each r_shape =  -e * cylinderLength / 2, each r = revolute.frame_b.r_0, each R = revolute.frame_b.R) annotation(Placement(transformation(extent = {{50,30},{70,50}}, rotation = 0)));

        equation 
          s_x = prismatic_x.s;
          s_y = prismatic_y.s;
          phi = revolute.phi;
          v_x = der(s_x);
          v_y = der(s_y);
          w = der(phi);
          a_x = der(v_x);
          a_y = der(v_y);
          wd = der(w);
          connect(frame_a,prismatic_x.frame_a) annotation(Line(points = {{ -100,0},{ -69,0}}, color = {95,95,95}, thickness = 0.5));
          connect(prismatic_x.frame_b,prismatic_y.frame_a) annotation(Line(points = {{ -29,0},{ -1.22465e-15,0},{ -1.22465e-15,30}}, color = {95,95,95}, thickness = 0.5));
          connect(prismatic_y.frame_b,revolute.frame_a) annotation(Line(points = {{1.22465e-15,70},{0,80},{30,80},{30,0},{41,0}}, color = {95,95,95}, thickness = 0.5));
          connect(revolute.frame_b,frame_b) annotation(Line(points = {{81,0},{100,0}}, color = {95,95,95}, thickness = 0.5));
        end Planar;
        model Spherical "Spherical joint (3 constraints and no potential states, or 3 degrees-of-freedom and 3 states)"
          import Modelica.Mechanics.MultiBody.Frames;
          import SI = Modelica.SIunits;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
          parameter Boolean animation = true "= true, if animation shall be enabled (show sphere)";
          parameter SI.Distance sphereDiameter = world.defaultJointLength "Diameter of sphere representing the spherical joint" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color sphereColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of sphere representing the spherical joint" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(group = "if animation = true", enable = animation));
          parameter Boolean angles_fixed = false "= true, if angles_start are used as initial values, else as guess values" annotation(Evaluate = true, choices(__Dymola_checkBox = true), Dialog(tab = "Initialization"));
          parameter SI.Angle angles_start[3] = {0,0,0} "Initial values of angles to rotate frame_a around 'sequence_start' axes into frame_b" annotation(Dialog(tab = "Initialization"));
          parameter Types.RotationSequence sequence_start = {1,2,3} "Sequence of rotations to rotate frame_a into frame_b at initial time" annotation(Evaluate = true, Dialog(tab = "Initialization"));
          parameter Boolean w_rel_a_fixed = false "= true, if w_rel_a_start are used as initial values, else as guess values" annotation(Evaluate = true, choices(__Dymola_checkBox = true), Dialog(tab = "Initialization"));
          parameter SI.AngularVelocity w_rel_a_start[3] = {0,0,0} "Initial values of angular velocity of frame_b with respect to frame_a, resolved in frame_a" annotation(Dialog(tab = "Initialization"));
          parameter Boolean z_rel_a_fixed = false "= true, if z_rel_a_start are used as initial values, else as guess values" annotation(Evaluate = true, choices(__Dymola_checkBox = true), Dialog(tab = "Initialization"));
          parameter SI.AngularAcceleration z_rel_a_start[3] = {0,0,0} "Initial values of angular acceleration z_rel_a = der(w_rel_a)" annotation(Dialog(tab = "Initialization"));
          parameter Boolean enforceStates = false " = true, if relative variables of spherical joint shall be used as states (StateSelect.always)" annotation(Dialog(tab = "Advanced"));
          parameter Boolean useQuaternions = true " = true, if quaternions shall be used as states otherwise use 3 angles as states (provided enforceStates=true)" annotation(Dialog(tab = "Advanced", enable = enforceStates));
          parameter Types.RotationSequence sequence_angleStates = {1,2,3} " Sequence of rotations to rotate frame_a into frame_b around the 3 angles used as states" annotation(Evaluate = true, Dialog(tab = "Advanced", enable = enforceStates and not useQuaternions));
          final parameter Frames.Orientation R_rel_start = Frames.axesRotations(sequence_start, angles_start, zeros(3)) "Orientation object from frame_a to frame_b at initial time";
          annotation(Documentation(info = "<html>
<p>
Joint with <b>3 constraints</b> that define that the origin of
frame_a and the origin of frame_b coincide. By default this joint
defines only the 3 constraints without any potential states.
If parameter <b>enforceStates</b> is set to <b>true</b>
in the \"Advanced\" menu, three states are introduced.
Depending on parameter <b>useQuaternions</b> these are either
quaternions and the relative angular velocity or 3 angles
and the angle derivatves. In the latter case the orientation
of frame_b is computed by rotating frame_a along the axes defined
in parameter vector \"sequence_angleStates\" (default = {1,2,3}, i.e.,
the Cardan angle sequence) around the angles used as states.
For example, the default is to rotate the x-axis of frame_a
around angles[1], the new y-axis around angles[2] and the new z-axis
around angles[3], arriving at frame_b. If angles are used
as states there is the slight disadvantage that
a singular configuration is present leading to a divison by zero.
</p>
<p>
If this joint is used in a <b>chain</b> structure, a Modelica translator
has to select orientation coordinates of a body as states, if the
default setting is used. It is usually better to use relative coordinates
in the spherical joint as states, and therefore in this situation
parameter enforceStates might be set to <b>true</b>.
</p>
<p>
If this joint is used in a <b>loop</b> structure, the default
setting results in a <b>cut-joint</b> that
breaks the loop in independent kinematic pieces, hold together
by the constraints of this joint. As a result, a Modelica translator
will first try to select 3 generalized coordinates in the joints of
the remaining parts of the loop and their first derivative as states
and if this is not possible, e.g., because there are only spherical
joints in the loop, will select coordinates from a body of the loop
as states.
</p>
<p>
In the following figure the animation of a spherical
joint is shown. The light blue coordinate system is
frame_a and the dark blue coordinate system is
frame_b of the joint.
(here: angles_start = {45, 45, 45}<sup>o</sup>).
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Joints/Spherical.png\">
</p>
</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -70, -70},{70,70}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -49, -50},{51,50}}, lineColor = {128,128,128}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{30,70},{71, -68}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -100,10},{ -68, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{23,10},{100, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Ellipse(extent = {{ -24,25},{26, -25}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {160,160,164}),Text(extent = {{ -100, -132},{100, -72}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -70, -70},{70,70}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -49, -50},{51,50}}, lineColor = {128,128,128}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{30,70},{71, -68}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -100,10},{ -68, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{23,10},{100, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Ellipse(extent = {{ -24,25},{26, -25}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {160,160,164})}));
        protected 
          Visualizers.Advanced.Shape sphere(shapeType = "sphere", color = sphereColor, specularCoefficient = specularCoefficient, length = sphereDiameter, width = sphereDiameter, height = sphereDiameter, lengthDirection = {1,0,0}, widthDirection = {0,1,0}, r_shape = { -0.5,0,0} * sphereDiameter, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          parameter Frames.Quaternions.Orientation Q_start = Modelica.Mechanics.MultiBody.Frames.to_Q(R_rel_start) "Quaternion orientation object from frame_a to frame_b at initial time";
          Frames.Quaternions.Orientation Q(start = Q_start, stateSelect = if enforceStates and useQuaternions then StateSelect.prefer else StateSelect.never) "Quaternion orientation object from frame_a to frame_b (dummy value, if quaternions are not used as states)";
          parameter SI.Angle phi_start[3] = if sequence_start[1] == sequence_angleStates[1] and sequence_start[2] == sequence_angleStates[2] and sequence_start[3] == sequence_angleStates[3] then angles_start else Frames.axesRotationsAngles(R_rel_start, sequence_angleStates) "Potential angle states at initial time";
          SI.Angle phi[3](start = phi_start, stateSelect = if enforceStates and not useQuaternions then StateSelect.always else StateSelect.never) "Dummy or 3 angles to rotate frame_a into frame_b";
          SI.AngularVelocity phi_d[3](stateSelect = if enforceStates and not useQuaternions then StateSelect.always else StateSelect.never) "= der(phi)";
          SI.AngularAcceleration phi_dd[3] "= der(phi_d)";
          SI.AngularVelocity w_rel[3](start = Frames.resolve2(R_rel_start, w_rel_a_start), fixed = fill(w_rel_a_fixed, 3), stateSelect = if enforceStates and useQuaternions then StateSelect.always else StateSelect.never) "Dummy or relative angular velocity of frame_b with respect to frame_a, resolved in frame_b";
          Frames.Orientation R_rel "Dummy or relative orientation object to rotate from frame_a to frame_b";
          Frames.Orientation R_rel_inv "Dummy or relative orientation object to rotate from frame_b to frame_a";

        initial equation 
if angles_fixed then
                      if not enforceStates then
              zeros(3) = Frames.Orientation.equalityConstraint(Frames.absoluteRotation(frame_a.R, R_rel_start), frame_b.R);

            elseif useQuaternions then
              zeros(3) = Frames.Quaternions.Orientation.equalityConstraint(Q, Q_start);


            else
              phi = phi_start;

            end if;
          end if;
if z_rel_a_fixed then
                      der(w_rel) = Frames.resolve2(R_rel_start, z_rel_a_start);
          end if;

        equation 
          zeros(3) = frame_a.t;
          zeros(3) = frame_b.t;
          if enforceStates then
            Connections.branch(frame_a.R, frame_b.R);
            frame_b.r_0 = frame_a.r_0;
            if rooted(frame_a.R) then
              R_rel_inv = Frames.nullRotation();
              frame_b.R = Frames.absoluteRotation(frame_a.R, R_rel);
              zeros(3) = frame_a.f + Frames.resolve1(R_rel, frame_b.f);

            else
              R_rel_inv = Frames.inverseRotation(R_rel);
              frame_a.R = Frames.absoluteRotation(frame_b.R, R_rel_inv);
              zeros(3) = frame_b.f + Frames.resolve2(R_rel, frame_a.f);

            end if;
            if useQuaternions then
              {0} = Frames.Quaternions.orientationConstraint(Q);
              w_rel = Frames.Quaternions.angularVelocity2(Q, der(Q));
              R_rel = Frames.from_Q(Q, w_rel);
              phi = zeros(3);
              phi_d = zeros(3);
              phi_dd = zeros(3);

            else
              phi_d = der(phi);
              phi_dd = der(phi_d);
              R_rel = Frames.axesRotations(sequence_angleStates, phi, phi_d);
              w_rel = Frames.angularVelocity2(R_rel);
              Q = zeros(4);

            end if;

          else
            frame_b.r_0 = frame_a.r_0;
            zeros(3) = frame_a.f + Frames.resolveRelative(frame_b.f, frame_b.R, frame_a.R);
            if w_rel_a_fixed or z_rel_a_fixed then
              w_rel = Frames.angularVelocity2(frame_b.R) - Frames.resolve2(frame_b.R, Frames.angularVelocity1(frame_a.R));

            else
              w_rel = zeros(3);

            end if;
            R_rel = Frames.nullRotation();
            R_rel_inv = Frames.nullRotation();
            Q = zeros(4);
            phi = zeros(3);
            phi_d = zeros(3);
            phi_dd = zeros(3);

          end if;
        end Spherical;
        model FreeMotion "Free motion joint (6 degrees-of-freedom, 12 potential states)"
          import Modelica.Math.*;
          import SI = Modelica.SIunits;
          extends Interfaces.PartialTwoFrames;
          parameter Boolean animation = true "= true, if animation shall be enabled (show arrow from frame_a to frame_b)";
          SI.Position r_rel_a[3](start = {0,0,0}, stateSelect = if enforceStates then StateSelect.always else StateSelect.prefer) "Position vector from origin of frame_a to origin of frame_b, resolved in frame_a" annotation(Dialog(group = "Initialization", __Dymola_initialDialog = true));
          SI.Velocity v_rel_a[3](start = {0,0,0}, stateSelect = if enforceStates then StateSelect.always else StateSelect.prefer) "= der(r_rel_a), i.e., velocity of origin of frame_b with respect to origin of frame_a, resolved in frame_a" annotation(Dialog(group = "Initialization", __Dymola_initialDialog = true));
          SI.Acceleration a_rel_a[3](start = {0,0,0}) "= der(v_rel_a)" annotation(Dialog(group = "Initialization", __Dymola_initialDialog = true));
          parameter Boolean angles_fixed = false "= true, if angles_start are used as initial values, else as guess values" annotation(Evaluate = true, choices(__Dymola_checkBox = true), Dialog(group = "Initialization"));
          parameter SI.Angle angles_start[3] = {0,0,0} "Initial values of angles to rotate frame_a around 'sequence_start' axes into frame_b" annotation(Dialog(group = "Initialization"));
          parameter Types.RotationSequence sequence_start = {1,2,3} "Sequence of rotations to rotate frame_a into frame_b at initial time" annotation(Evaluate = true, Dialog(group = "Initialization"));
          parameter Boolean w_rel_a_fixed = false "= true, if w_rel_a_start are used as initial values, else as guess values" annotation(Evaluate = true, choices(__Dymola_checkBox = true), Dialog(group = "Initialization"));
          parameter SI.AngularVelocity w_rel_a_start[3] = {0,0,0} "Initial values of angular velocity of frame_b with respect to frame_a, resolved in frame_a" annotation(Dialog(group = "Initialization"));
          parameter Boolean z_rel_a_fixed = false "= true, if z_rel_a_start are used as initial values, else as guess values" annotation(Evaluate = true, choices(__Dymola_checkBox = true), Dialog(group = "Initialization"));
          parameter SI.AngularAcceleration z_rel_a_start[3] = {0,0,0} "Initial values of angular acceleration z_rel_a = der(w_rel_a)" annotation(Dialog(group = "Initialization"));
          parameter SI.Length arrowDiameter = world.defaultArrowDiameter "Diameter of arrow from frame_a to frame_b" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color arrowColor = Modelica.Mechanics.MultiBody.Types.Defaults.SensorColor "Color of arrow" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Boolean enforceStates = true " = true, if relative variables between frame_a and frame_b shall be used as states" annotation(Dialog(tab = "Advanced"));
          parameter Boolean useQuaternions = true " = true, if quaternions shall be used as states otherwise use 3 angles as states" annotation(Dialog(tab = "Advanced"));
          parameter Types.RotationSequence sequence_angleStates = {1,2,3} " Sequence of rotations to rotate frame_a into frame_b around the 3 angles used as states" annotation(Evaluate = true, Dialog(tab = "Advanced", enable = not useQuaternions));
          final parameter Frames.Orientation R_rel_start = Modelica.Mechanics.MultiBody.Frames.axesRotations(sequence_start, angles_start, zeros(3)) "Orientation object from frame_a to frame_b at initial time";
          annotation(Documentation(info = "<HTML>
<p>
Joint which does not constrain the motion between frame_a and frame_b.
Such a joint is only meaningful if the <b>relative</b> distance and orientation
between frame_a and frame_b, and their derivatives, shall be used
as <b>states</b>.
</p>
<p>
Note, that <b>bodies</b> such as Parts.Body, Parts.BodyShape,
have potential states describing the distance
and orientation, and their derivatives, between the <b>world frame</b> and
a <b>body fixed frame</b>.
Therefore, if these potential state variables are suited,
a FreeMotion joint is not needed.
</p>
The states of the FreeMotion object are:
</p>
<ul>
<li> The <b>relative position vector</b> r_rel_a from the origin of
     frame_a to the origin of frame_b, resolved in
     frame_a and the <b>relative velocity</b> v_rel_a of the origin of
     frame_b with respect to the origin of frame_a, resolved in frame_a
     (= der(r_rel_a)).</li>
</li>
<li> If parameter <b>useQuaternions</b> in the \"Advanced\" menu
     is <b>true</b> (this is the default), then <b>4 quaternions</b>
     are states. Additionally, the coordinates of the
     relative angular velocity vector are 3 potential states.<br>
     If <b>useQuaternions</b> in the \"Advanced\" menu
     is <b>false</b>, then <b>3 angles</b> and the derivatives of
     these angles are potential states. The orientation of frame_b
     is computed by rotating frame_a along the axes defined
     in parameter vector \"sequence_angleStates\" (default = {1,2,3}, i.e.,
     the Cardan angle sequence) around the angles used as states.
     For example, the default is to rotate the x-axis of frame_a
     around angles[1], the new y-axis around angles[2] and the new z-axis
     around angles[3], arriving at frame_b.
 </li>
</ul>
<p>
The quaternions have the slight disadvantage that there is a
non-linear constraint equation between the 4 quaternions.
Therefore, at least one non-linear equation has to be solved
during simulation. A tool might, however, analytically solve this
simple constraint equation. Using the 3 angles as states has the
disadvantage that there is a singular configuration in which a
division by zero will occur. If it is possible to determine in advance
for an application class that this singular configuration is outside
of the operating region, the 3 angles might be used as
states by setting <b>useQuaternions</b> = <b>false</b>.
</p>
<p>
In text books about 3-dimensional mechanics often 3 angles and the
angular velocity are used as states. This is not the case here, since
3 angles and their derivatives are used as states
(if useQuaternions = false). The reason
is that for real-time simulation the discretization formula of the
integrator might be \"inlined\" and solved together with the model equations.
By appropriate symbolic transformation the performance is
drastically increased if angles and their
derivatives are used as states, instead of angles and the angular
velocity.
</p>
<p>
If parameter
<b>enforceStates</b> is set to <b>true</b> (= the default)
in the \"Advanced\" menu,
then FreeMotion variables are forced to be used as states according
to the setting of parameters \"useQuaternions\" and
\"sequence_angleStates\".
</p>
<p>
In the following figure the animation of a FreeMotion
joint is shown. The light blue coordinate system is
frame_a and the dark blue coordinate system is
frame_b of the joint.
(here: r_rel_a_start = {0.5, 0, 0.5}, angles_start = {45, 45, 45}<sup>o</sup>).
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Joints/FreeMotion.png\">
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -86,31},{ -74,61},{ -49,83},{ -17,92},{19,88},{40,69},{59,48}}, color = {160,160,164}, thickness = 0.5),Polygon(points = {{90,0},{50,20},{50, -20},{90,0}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{69,58},{49,40},{77,28},{69,58}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{136, -35},{ -150, -98}}, lineColor = {0,0,255}, textString = "%name"),Rectangle(extent = {{ -70, -5},{ -90,5}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{50, -5},{30,5}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{11, -5},{ -9,5}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -30, -5},{ -50,5}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -86,31},{ -74,61},{ -49,83},{ -17,92},{19,88},{40,69},{59,48}}, color = {160,160,164}, thickness = 0.5),Polygon(points = {{90,0},{50,20},{50, -20},{90,0}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{69,58},{49,40},{77,28},{69,58}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{50, -5},{30,5}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{11, -5},{ -9,5}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -30, -5},{ -50,5}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -70, -5},{ -90,5}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid)}));
        protected 
          Visualizers.Advanced.Arrow arrow(r_head = r_rel_a, diameter = arrowDiameter, color = arrowColor, specularCoefficient = specularCoefficient, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          parameter Frames.Quaternions.Orientation Q_start = Frames.to_Q(R_rel_start) "Quaternion orientation object from frame_a to frame_b at initial time";
          Frames.Quaternions.Orientation Q(start = Q_start, stateSelect = if enforceStates then if useQuaternions then StateSelect.prefer else StateSelect.never else StateSelect.default) "Quaternion orientation object from frame_a to frame_b (dummy value, if quaternions are not used as states)";
          parameter SI.Angle phi_start[3] = if sequence_start[1] == sequence_angleStates[1] and sequence_start[2] == sequence_angleStates[2] and sequence_start[3] == sequence_angleStates[3] then angles_start else Frames.axesRotationsAngles(R_rel_start, sequence_angleStates) "Potential angle states at initial time";
          SI.Angle phi[3](start = phi_start, stateSelect = if enforceStates then if useQuaternions then StateSelect.never else StateSelect.always else StateSelect.prefer) "Dummy or 3 angles to rotate frame_a into frame_b";
          SI.AngularVelocity phi_d[3](stateSelect = if enforceStates then if useQuaternions then StateSelect.never else StateSelect.always else StateSelect.prefer) "= der(phi)";
          SI.AngularAcceleration phi_dd[3] "= der(phi_d)";
          SI.AngularVelocity w_rel_b[3](start = Frames.resolve2(R_rel_start, w_rel_a_start), fixed = fill(w_rel_a_fixed, 3), stateSelect = if enforceStates then if useQuaternions then StateSelect.always else StateSelect.avoid else StateSelect.prefer) "Dummy or relative angular velocity of frame_b with respect to frame_a, resolved in frame_b";
          Frames.Orientation R_rel "Dummy or relative orientation object to rotate from frame_a to frame_b";
          Frames.Orientation R_rel_inv "Dummy or relative orientation object to rotate from frame_b to frame_a";

        initial equation 
if angles_fixed then
                      if not enforceStates then
              zeros(3) = Frames.Orientation.equalityConstraint(Frames.absoluteRotation(frame_a.R, R_rel_start), frame_b.R);

            elseif useQuaternions then
              zeros(3) = Frames.Quaternions.Orientation.equalityConstraint(Q, Q_start);


            else
              phi = phi_start;

            end if;
          end if;
if z_rel_a_fixed then
                      der(w_rel_b) = Frames.resolve2(R_rel_start, z_rel_a_start);
          end if;

        equation 
          der(r_rel_a) = v_rel_a;
          der(v_rel_a) = a_rel_a;
          frame_b.r_0 = frame_a.r_0 + Frames.resolve1(frame_a.R, r_rel_a);
          frame_a.f = zeros(3);
          frame_a.t = zeros(3);
          frame_b.f = zeros(3);
          frame_b.t = zeros(3);
          if enforceStates then
            Connections.branch(frame_a.R, frame_b.R);
            if rooted(frame_a.R) then
              R_rel_inv = Frames.nullRotation();
              frame_b.R = Frames.absoluteRotation(frame_a.R, R_rel);

            else
              R_rel_inv = Frames.inverseRotation(R_rel);
              frame_a.R = Frames.absoluteRotation(frame_b.R, R_rel_inv);

            end if;
            if useQuaternions then
              {0} = Frames.Quaternions.orientationConstraint(Q);
              w_rel_b = Frames.Quaternions.angularVelocity2(Q, der(Q));
              R_rel = Frames.from_Q(Q, w_rel_b);
              phi = zeros(3);
              phi_d = zeros(3);
              phi_dd = zeros(3);

            else
              phi_d = der(phi);
              phi_dd = der(phi_d);
              R_rel = Frames.axesRotations(sequence_angleStates, phi, phi_d);
              w_rel_b = Frames.angularVelocity2(R_rel);
              Q = zeros(4);

            end if;

          else
            if w_rel_a_fixed or z_rel_a_fixed then
              w_rel_b = Frames.angularVelocity2(frame_b.R) - Frames.resolve2(frame_b.R, Frames.angularVelocity1(frame_a.R));

            else
              w_rel_b = zeros(3);

            end if;
            R_rel = Frames.nullRotation();
            R_rel_inv = Frames.nullRotation();
            Q = zeros(4);
            phi = zeros(3);
            phi_d = zeros(3);
            phi_dd = zeros(3);

          end if;
        end FreeMotion;
        model SphericalSpherical "Spherical - spherical joint aggregation (1 constraint, no potential states) with an optional point mass in the middle"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Types;
          extends Interfaces.PartialTwoFrames;
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Boolean showMass = true "= true, if mass shall be shown (provided animation = true and m > 0)";
          parameter Boolean computeRodLength = false "= true, if rodLength shall be computed during initialization (see info)";
          parameter SI.Length rodLength(min = Modelica.Constants.eps, fixed = not computeRodLength, start = 1) "Distance between the origins of frame_a and frame_b (if computeRodLength=true, guess value)";
          parameter SI.Mass m(min = 0) = 0 "Mass of rod (= point mass located in middle of rod)";
          parameter SI.Diameter sphereDiameter = world.defaultJointLength "Diameter of spheres respresenting the spherical joints" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color sphereColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of spheres respresenting the spherical joints" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Diameter rodDiameter = sphereDiameter / Types.Defaults.JointRodDiameterFraction "Diameter of rod connecting the two spherical joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color rodColor = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor "Color of rod connecting the two spherical joints" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Diameter massDiameter = sphereDiameter "Diameter of sphere representing the mass point" annotation(Dialog(tab = "Animation", group = "if animation = true and showMass = true and m > 0", enable = animation and showMass and m > 0));
          input Types.Color massColor = Modelica.Mechanics.MultiBody.Types.Defaults.BodyColor "Color of sphere representing the mass point" annotation(Dialog(tab = "Animation", group = "if animation = true and showMass = true and m > 0", enable = animation and showMass and m > 0));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Boolean kinematicConstraint = true "= false, if no constraint shall be defined, due to analytically solving a kinematic loop (\"false\" should not be used by user, but only by MultiBody.Joints.Assemblies joints)" annotation(Dialog(tab = "Advanced"));
          Real constraintResidue = rRod_0 * rRod_0 - rodLength * rodLength "Constraint equation of joint in residue form: Either length constraint (= default) or equation to compute rod force (for analytic solution of loops in combination with Internal.RevoluteWithLengthConstraint/PrismaticWithLengthConstraint)" annotation(Dialog(tab = "Advanced", enable = not kinematicConstraint));
          parameter Boolean checkTotalPower = false "= true, if total power flowing into this component shall be determined (must be zero)" annotation(Dialog(tab = "Advanced"));
          SI.Force f_rod "Constraint force in direction of the rod (positive on frame_a, when directed from frame_a to frame_b)";
          SI.Position rRod_0[3] "Position vector from frame_a to frame_b resolved in world frame";
          SI.Position rRod_a[3] "Position vector from frame_a to frame_b resolved in frame_a";
          Real eRod_a[3](each final unit = "1") "Unit vector in direction from frame_a to frame_b, resolved in frame_a";
          SI.Position r_CM_0[3] "Dummy if m==0, or position vector from world frame to mid-point of rod, resolved in world frame";
          SI.Velocity v_CM_0[3] "First derivative of r_CM_0";
          SI.Force f_CM_a[3] "Dummy if m==0, or inertial force acting at mid-point of rod due to mass oint acceleration, resolved in frame_a";
          SI.Force f_CM_e[3] "Dummy if m==0, or projection of f_CM_a onto eRod_a, resolved in frame_a";
          SI.Force f_b_a1[3] "Force acting at frame_b, but without force in rod, resolved in frame_a";
          SI.Power totalPower "Total power flowing into this element, if checkTotalPower=true (otherwise dummy)";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -95, -40},{ -15,40}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -84, -30},{ -24,30}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{15, -40},{95,40}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{25, -29},{85,30}}, lineColor = {128,128,128}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -149,111},{141,51}}, textString = "%name", lineColor = {0,0,255}),Rectangle(extent = {{ -40,40},{41, -41}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -51,6},{48, -4}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Ellipse(extent = {{ -68,15},{ -39, -13}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{39,14},{68, -14}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Text(extent = {{ -130, -58},{137, -98}}, lineColor = {0,0,0}, textString = "%rodLength")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -98, -40},{ -18,40}}, lineColor = {0,0,0}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -88, -30},{ -28,30}}, lineColor = {160,160,164}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{18, -40},{98,40}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Ellipse(extent = {{29, -30},{89,29}}, lineColor = {192,192,192}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -56, -60},{46, -60}}, color = {0,0,255}),Polygon(points = {{56, -60},{41, -54},{41, -66},{56, -60}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -37, -63},{33, -79}}, textString = "rodLength", lineColor = {0,0,255}),Rectangle(extent = {{ -40,41},{40, -40}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -51,6},{48, -4}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Ellipse(extent = {{ -71,15},{ -42, -13}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{42,14},{71, -14}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -56, -71},{ -56,1}}, color = {0,0,255}),Line(points = {{56, -72},{56,0}}, color = {0,0,255}),Polygon(points = {{11,1},{ -1,4},{ -1, -2},{11,1}}, lineColor = {0,0,255}),Line(points = {{ -56,1},{ -1,1}}, color = {0,0,255}),Text(extent = {{ -32, -4},{4, -29}}, lineColor = {0,0,0}, textString = "eRod_a")}), Documentation(info = "<html>
<p>
Joint that has a spherical joint on each of its two ends.
The rod connecting the two spherical joints is approximated by a
point mass that is located in the middle of the rod. When the mass
is set to zero (default), special code for a massless body is generated.
In the following default animation figure, the two spherical joints are
represented by two red spheres, the connecting rod by a grey cylinder
and the point mass in the middle of the rod by a light blue sphere:
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Joints/SphericalSpherical.png\" ALT=\"model Joints.SphericalSpherical\">
</p>
This joint introduces <b>one constraint</b> defining that the distance between
the origin of frame_a and the origin of frame_b is constant (= rodLength).
It is highly recommended to use this joint in loops
whenever possible, because this enhances the efficiency
considerably due to smaller systems of non-linear algebraic
equations.
</p>
<p>
It is sometimes desirable to <b>compute</b> the <b>rodLength</b>
of the connecting rod during initialization. For this, parameter
<b>computeLength</b> has to be set to <b>true</b> and instead <b>one</b> other,
easier to determine, position variable in the same loop
needs to have a fixed attribute of <b>true</b>. For example,
if a loop consists of one Revolute joint, one Prismatic joint and
a SphericalSpherical joint, one may fix the start values of the revolute
joint angle and of the relative distance of the prismatic joint
in order to compute the rodLength of the rod.
</p>
<p>
It is not possible to connect other components, such as a body with mass
properties or a special visual shape object to the rod connecting
the two spherical joints. If this is needed, use instead joint Joints.<b>UniversalSpherical</b>
that has this property.
</p>
</html>"));
        protected 
          Visualizers.Advanced.Shape shape_rod(shapeType = "cylinder", color = rodColor, specularCoefficient = specularCoefficient, length = rodLength, width = rodDiameter, height = rodDiameter, lengthDirection = eRod_a, widthDirection = {0,1,0}, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          Visualizers.Advanced.Shape shape_a(shapeType = "sphere", color = sphereColor, specularCoefficient = specularCoefficient, length = sphereDiameter, width = sphereDiameter, height = sphereDiameter, lengthDirection = eRod_a, widthDirection = {0,1,0}, r_shape =  -eRod_a * sphereDiameter / 2, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          Visualizers.Advanced.Shape shape_b(shapeType = "sphere", color = sphereColor, specularCoefficient = specularCoefficient, length = sphereDiameter, width = sphereDiameter, height = sphereDiameter, lengthDirection = eRod_a, widthDirection = {0,1,0}, r_shape = eRod_a * (rodLength - sphereDiameter / 2), r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          Visualizers.Advanced.Shape shape_mass(shapeType = "sphere", color = massColor, specularCoefficient = specularCoefficient, length = massDiameter, width = massDiameter, height = massDiameter, lengthDirection = eRod_a, widthDirection = {0,1,0}, r_shape = eRod_a * (rodLength / 2 - sphereDiameter / 2), r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation and showMass and m > 0;

        equation 
          if kinematicConstraint then
            rRod_0 = transpose(frame_b.R.T) * frame_b.R.T * frame_b.r_0 - transpose(frame_a.R.T) * frame_a.R.T * frame_a.r_0;

          else
            rRod_0 = frame_b.r_0 - frame_a.r_0;

          end if;
          rRod_a = Frames.resolve2(frame_a.R, rRod_0);
          eRod_a = rRod_a / rodLength;
          constraintResidue = 0;
          frame_a.t = zeros(3);
          frame_b.t = zeros(3);
          if m > 0 then
            r_CM_0 = frame_a.r_0 + rRod_0 / 2;
            v_CM_0 = der(r_CM_0);
            f_CM_a = m * Frames.resolve2(frame_a.R, der(v_CM_0) - world.gravityAcceleration(r_CM_0));
            f_CM_e = f_CM_a * eRod_a * eRod_a;
            frame_a.f = (f_CM_a - f_CM_e) / 2 + f_rod * eRod_a;
            f_b_a1 = (f_CM_a + f_CM_e) / 2;
            frame_b.f = Frames.resolveRelative(f_b_a1 - f_rod * eRod_a, frame_a.R, frame_b.R);

          else
            r_CM_0 = zeros(3);
            v_CM_0 = zeros(3);
            f_CM_a = zeros(3);
            f_CM_e = zeros(3);
            f_b_a1 = zeros(3);
            frame_a.f = f_rod * eRod_a;
            frame_b.f =  -Frames.resolveRelative(frame_a.f, frame_a.R, frame_b.R);

          end if;
          if checkTotalPower then
            totalPower = frame_a.f * Frames.resolve2(frame_a.R, der(frame_a.r_0)) + frame_b.f * Frames.resolve2(frame_b.R, der(frame_b.r_0)) + ( -m) * (der(v_CM_0) - world.gravityAcceleration(r_CM_0)) * v_CM_0 + frame_a.t * Frames.angularVelocity2(frame_a.R) + frame_b.t * Frames.angularVelocity2(frame_b.R);

          else
            totalPower = 0;

          end if;
        end SphericalSpherical;
        model UniversalSpherical "Universal - spherical joint aggregation (1 constraint, no potential states)"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Types;
          extends Interfaces.PartialTwoFrames;
          Interfaces.Frame_a frame_ia "Coordinate system at the origin of frame_a, fixed at the rod connecting the universal with the spherical joint" annotation(Placement(transformation(origin = { -40,100}, extent = {{ -16, -16},{16,16}}, rotation = 270)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Boolean showUniversalAxes = true " = true, if universal joint shall be visualized with two cylinders, otherwise with a sphere (provided animation=true)";
          parameter Boolean computeRodLength = false "= true, if distance between frame_a and frame_b shall be computed during initialization (see info)";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n1_a = {0,0,1} "Axis 1 of universal joint resolved in frame_a (axis 2 is orthogonal to axis 1 and to rod)" annotation(Evaluate = true);
          parameter SI.Position rRod_ia[3] = {1,0,0} "Vector from origin of frame_a to origin of frame_b, resolved in frame_ia (if computeRodLength=true, rRod_ia is only an axis vector along the connecting rod)" annotation(Evaluate = true);
          parameter SI.Diameter sphereDiameter = world.defaultJointLength "Diameter of spheres representing the universal and the spherical joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color sphereColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of spheres representing the universal and the spherical joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Types.ShapeType rodShapeType = "cylinder" "Shape type of rod connecting the universal and the spherical joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance rodWidth = sphereDiameter / Types.Defaults.JointRodDiameterFraction "Width of rod shape in direction of axis 2 of universal joint." annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance rodHeight = rodWidth "Height of rod shape in direction that is orthogonal to rod and to axis 2" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Types.ShapeExtra rodExtra = 0.0 "Additional parameter depending on rodShapeType" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color rodColor = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor "Color of rod shape connecting the universal and the spherical joints" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter SI.Distance cylinderLength = world.defaultJointLength "Length of cylinders representing the two universal joint axes" annotation(Dialog(tab = "Animation", group = "if animation = true and showUniversalAxes", enable = animation and showUniversalAxes));
          parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of cylinders representing the two universal joint axes" annotation(Dialog(tab = "Animation", group = "if animation = true and showUniversalAxes", enable = animation and showUniversalAxes));
          input Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinders representing the two universal joint axes" annotation(Dialog(tab = "Animation", group = "if animation = true and showUniversalAxes", enable = animation and showUniversalAxes));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Boolean kinematicConstraint = true "= false, if no constraint shall be defined, due to analytically solving a kinematic loop" annotation(Dialog(tab = "Advanced"));
          Real constraintResidue = rRod_0 * rRod_0 - rodLength * rodLength "Constraint equation of joint in residue form: Either length constraint (= default) or equation to compute rod force (for analytic solution of loops in combination with Internal.RevoluteWithLengthConstraint/PrismaticWithLengthConstraint)" annotation(Dialog(tab = "Advanced", enable = not kinematicConstraint));
          parameter Boolean checkTotalPower = false "= true, if total power flowing into this component shall be determined (must be zero)" annotation(Dialog(tab = "Advanced"));
          SI.Force f_rod "Constraint force in direction of the rod (positive, if rod is pressed)";
          final parameter SI.Distance rodLength(fixed = not computeRodLength) = Modelica.Math.Vectors.length(rRod_ia) "Length of rod (distance between origin of frame_a and origin of frame_b)";
          final parameter Real eRod_ia[3](each final unit = "1") = Modelica.Math.Vectors.normalize(rRod_ia) "Unit vector from origin of frame_a to origin of frame_b, resolved in frame_ia";
          final parameter Real e2_ia[3](each final unit = "1") = Modelica.Math.Vectors.normalize(cross(n1_a, eRod_ia)) "Unit vector in direction of axis 2 of universal joint, resolved in frame_ia (orthogonal to n1_a and eRod_ia; note: frame_ia is parallel to frame_a when the universal joint angles are zero)";
          final parameter Real e3_ia[3](each final unit = "1") = cross(eRod_ia, e2_ia) "Unit vector perpendicular to eRod_ia and e2_ia, resolved in frame_ia";
          SI.Power totalPower "Total power flowing into this element, if checkTotalPower=true (otherwise dummy)";
          SI.Force f_b_a1[3] "frame_b.f without f_rod part, resolved in frame_a (needed for analytic loop handling)";
          Real eRod_a[3](each final unit = "1") "Unit vector in direction of rRod_a, resolved in frame_a (needed for analytic loop handling)";
          SI.Position rRod_0[3](start = rRod_ia) "Position vector from origin of frame_a to origin of frame_b resolved in world frame";
          SI.Position rRod_a[3](start = rRod_ia) "Position vector from origin of frame_a to origin of frame_b resolved in frame_a";
          annotation(Documentation(info = "<html>
<p>
This component consists of a <b>universal joint</b> at frame_a and
a <b>spherical joint</b> at frame_b that are connected together with
a <b>rigid rod</b>, see default aimation figure (the arrows are not
part of the default animation):
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Joints/UniversalSpherical.png\" ALT=\"model Joints.UniversalSpherical\">
</p>
<p>
This joint aggregation has no mass and no inertia and introduces the constraint
that the distance between the origin of frame_a and the origin of frame_b is constant
(= Frames.length(rRod_ia)). The universal joint is defined in the following way:
<p>
<ul>
<li> The rotation <b>axis</b> of revolute joint <b>1</b> is along parameter
     vector n1_a which is fixed in frame_a.<li>
<li> The rotation <b>axis</b> of revolute joint <b>2</b> is perpendicular to
     axis 1 and to the line connecting the universal and the spherical joint.
</ul>
<p>
The definition of axis 2 of the universal joint is performed according
to the most often occuring case. In a future release, axis 2 might
be explicitly definable via a parameter. However, the treatment is much more
complicated and the number of operations is considerably higher,
if axis 2 is not orthogonal to axis 1 and to the connecting rod.
</p>
<p>
Note, there is a <b>singularity</b> when axis 1 and the connecting rod are parallel
to other. Therefore, if possible n1_a should be selected in such a way that it
is perpendicular to rRod_ia in the initial configuration (i.e., the
distance to the singularity is as large as possible).
</p>
<p>
An additional <b>frame_ia</b> is present. It is <b>fixed</b> in the connecting
<b>rod</b> at the origin of <b>frame_a</b>. The placement of frame_ia on the rod
is implicitly defined by the universal joint (frame_a and frame_ia coincide
when the angles of the two revolute joints of the universal joint are zero)
and by parameter vector <b>rRod_ia</b>, the position vector
from the origin of frame_a to the origin of frame_b, resolved in frame_<b>ia</b>.
</p>
<p>
The easiest way to define the parameters of this joint is by moving the
MultiBody system in a <b>reference configuration</b> where <b>all frames</b>
of all components are <b>parallel</b> to other (alternatively,
at least frame_a and frame_ia of the UniversalSpherical joint
should be parallel to other when defining an instance of this
component). Since frame_a and frame_ia are parallel to other,
vector <b>rRod_ia</b> from frame_a to frame_b resolved in frame_<b>ia</b> can be resolved
in frame_<b>a</b> (or the <b>world frame</b>, if all frames are parallel to other).
</p>
<p>
This joint aggregation can be used in cases where
in reality a rod with spherical joints at end are present.
Such a system has an additional degree of freedom to rotate
the rod along its axis. In practice this rotation is usually
of no interested and is mathematically removed by replacing one
of the spherical joints by a universal joint. Still, in most
cases the Joints.SphericalSpherical joint aggregation can be used instead
of the UniversalSpherical joint
since the rod is animated and its mass properties are approximated by
a point mass in the middle of the rod. The SphericalSpherical joint
has the advantage that it does not have a singular configuration.
</p>
<p>
In the public interface of the UniversalSpherical joint, the following
(final) <b>parameters</b> are provided:
</p>
<pre>
  <b>parameter</b> Real rodLength(unit=\"m\")  \"Length of rod\";
  <b>parameter</b> Real eRod_ia[3] \"Unit vector along rod, resolved in frame_ia\";
  <b>parameter</b> Real e2_ia  [3] \"Unit vector along axis 2, resolved in frame_ia\";
</pre>
<p>
This allows a more convenient definition of data which is related to the rod.
For example, if a box shall be connected at frame_ia directing from
the origin of frame_a to the middle of the rod, this might be defined as:
</p>
<pre>
    Modelica.Mechanics.MultiBody.Joints.UniversalSpherical jointUS(rRod_ia={1.2, 1, 0.2});
    Modelica.Mechanics.MultiBody.Visualizers.FixedShape    shape(shapeType       = \"box\",
                                              lengthDirection = jointUS.eRod_ia,
                                              widthDirection  = jointUS.e2_ia,
                                              length          = jointUS.rodLength/2,
                                              width           = jointUS.rodLength/10);
  <b>equation</b>
    <b>connect</b>(jointUS.frame_ia, shape.frame_a);
</pre>
</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -122, -49},{138, -107}}, lineColor = {0,0,255}, textString = "%name"),Ellipse(extent = {{ -100, -40},{ -19,40}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -90, -30},{ -29,29}}, lineColor = {160,160,164}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -60,41},{ -9, -44}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -60,40},{ -60, -40}}, color = {0,0,0}, thickness = 0.5),Ellipse(extent = {{ -83, -17},{ -34,21}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -74, -12},{ -40,15}}, lineColor = {160,160,164}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -72, -20},{ -89,3},{ -69,25},{ -45,27},{ -72, -20}}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -60,40},{ -60, -10}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -49,20},{ -69, -15}}, color = {0,0,0}, thickness = 0.5),Ellipse(extent = {{44,14},{73, -14}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{20, -40},{100,40}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{30, -30},{90,30}}, lineColor = {192,192,192}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -22,45},{40, -43}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{46,14},{75, -14}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Rectangle(extent = {{ -36, -8},{48,8}}, lineColor = {0,0,0}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Text(extent = {{ -105,118},{ -67,86}}, lineColor = {128,128,128}, textString = "ia"),Text(extent = {{ -24,103},{167,64}}, lineColor = {0,0,0}, textString = "%rRod_ia"),Line(points = {{ -40,101},{ -40,60},{ -60,1}}, color = {128,128,128}, thickness = 0.5)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -60, -70},{46, -70}}, color = {0,0,255}),Polygon(points = {{60, -70},{45, -64},{45, -76},{60, -70}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -56, -71},{56, -90}}, textString = "rRod", lineColor = {0,0,255}),Ellipse(extent = {{ -100, -40},{ -19,40}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -90, -30},{ -29,29}}, lineColor = {160,160,164}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -60,41},{ -19, -41}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -60,40},{ -60, -40}}, color = {0,0,0}, thickness = 0.5),Ellipse(extent = {{ -83, -17},{ -34,21}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -74, -12},{ -40,15}}, lineColor = {160,160,164}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -72, -20},{ -89,3},{ -69,25},{ -45,27},{ -72, -20}}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -60,40},{ -60, -10}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -49,20},{ -69, -15}}, color = {0,0,0}, thickness = 0.5),Polygon(points = {{7, -1},{ -5,2},{ -5, -4},{7, -1}}, lineColor = {0,0,255}),Line(points = {{ -50,19},{ -30,57}}, color = {0,0,255}),Text(extent = {{ -34,78},{8,62}}, lineColor = {0,0,0}, textString = "e2"),Polygon(points = {{ -25,64},{ -33,56},{ -27,53},{ -25,64}}, lineColor = {0,0,255}),Line(points = {{ -60,41},{ -60,65}}, color = {0,0,255}),Polygon(points = {{ -60,75},{ -64,63},{ -56,63},{ -60,75}}, lineColor = {0,0,255}),Text(extent = {{ -93,82},{ -64,62}}, lineColor = {0,0,0}, textString = "n1"),Line(points = {{ -60, -40},{ -60, -72}}, color = {0,0,255}),Ellipse(extent = {{20, -40},{100,40}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{30, -30},{90,30}}, lineColor = {192,192,192}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -22,45},{40, -43}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{45,14},{74, -14}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -36, -8},{48,8}}, lineColor = {0,0,0}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Text(extent = {{ -31, -7},{0, -28}}, lineColor = {0,0,0}, textString = "eRod"),Line(points = {{ -60,0},{ -5,0}}, color = {0,0,255}),Polygon(points = {{7,0},{ -5,3},{ -5, -3},{7,0}}, lineColor = {0,0,255}),Line(points = {{60, -1},{60, -72}}, color = {0,0,255}),Line(points = {{ -40,100},{ -40,70},{ -60,0}}, color = {128,128,128}, thickness = 0.5),Text(extent = {{ -23,30},{26,10}}, textString = " eRod*e2 = 0;  n1*e2 = 0", lineColor = {0,0,255})}));
        protected 
          SI.Force f_b_a[3] "frame_b.f resolved in frame_a";
          SI.Force f_ia_a[3] "frame_ia.f resolved in frame_a";
          SI.Torque t_ia_a[3] "frame_ia.t resolved in frame_a";
          Real n2_a[3](each final unit = "1") "Vector in direction of axis 2 of the universal joint (e2_ia), resolved in frame_a";
          Real length2_n2_a(start = 1, unit = "m2") "Square of length of vector n2_a";
          SI.Length length_n2_a "Length of vector n2_a";
          Real e2_a[3](each final unit = "1") "Unit vector in direction of axis 2 of the universal joint (e2_ia), resolved in frame_a";
          Real e3_a[3](each final unit = "1") "Unit vector perpendicular to eRod_ia and e2_a, resolved in frame_a";
          Real der_rRod_a_L[3](each unit = "1/s") "= der(rRod_a)/rodLength";
          SI.AngularVelocity w_rel_ia1[3];
          Frames.Orientation R_rel_ia1;
          Frames.Orientation R_rel_ia2;
          Frames.Orientation R_rel_ia "Rotation from frame_a to frame_ia";
          Visualizers.Advanced.Shape rodShape(shapeType = rodShapeType, color = rodColor, specularCoefficient = specularCoefficient, length = rodLength, width = rodWidth, height = rodHeight, lengthDirection = eRod_ia, widthDirection = e2_ia, r = frame_ia.r_0, R = frame_ia.R) if world.enableAnimation and animation;
          Visualizers.Advanced.Shape sphericalShape_b(shapeType = "sphere", color = sphereColor, specularCoefficient = specularCoefficient, length = sphereDiameter, width = sphereDiameter, height = sphereDiameter, lengthDirection = {1,0,0}, widthDirection = {0,1,0}, r_shape = { -0.5,0,0} * sphereDiameter, r = frame_b.r_0, R = frame_b.R) if world.enableAnimation and animation;
          Visualizers.Advanced.Shape sphericalShape_a(shapeType = "sphere", color = sphereColor, specularCoefficient = specularCoefficient, length = sphereDiameter, width = sphereDiameter, height = sphereDiameter, lengthDirection = {1,0,0}, widthDirection = {0,1,0}, r_shape = { -0.5,0,0} * sphereDiameter, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation and not showUniversalAxes;
          Visualizers.Advanced.Shape universalShape1(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = n1_a, widthDirection = {0,1,0}, r_shape =  -n1_a * cylinderLength / 2, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation and showUniversalAxes;
          Visualizers.Advanced.Shape universalShape2(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = e2_ia, widthDirection = {0,1,0}, r_shape =  -e2_ia * cylinderLength / 2, r = frame_ia.r_0, R = frame_ia.R) if world.enableAnimation and animation and showUniversalAxes;

        equation 
          Connections.branch(frame_a.R, frame_ia.R);
          if kinematicConstraint then
            rRod_0 = transpose(frame_b.R.T) * frame_b.R.T * frame_b.r_0 - transpose(frame_a.R.T) * frame_a.R.T * frame_a.r_0;

          else
            rRod_0 = frame_b.r_0 - frame_a.r_0;

          end if;
          rRod_a = Frames.resolve2(frame_a.R, rRod_0);
          constraintResidue = 0;
          eRod_a = rRod_a / rodLength;
          n2_a = cross(n1_a, eRod_a);
          length2_n2_a = n2_a * n2_a;
          assert(length2_n2_a > 1e-10, "
A Modelica.Mechanics.MultiBody.Joints.UniversalSpherical joint (consisting of
a universal joint and a spherical joint connected together
by a rigid rod) is in the singular configuration of the
universal joint. This means that axis 1 of the universal
joint defined via parameter \"n1_a\" is parallel to vector
\"rRod_ia\" that is directed from the origin of frame_a to the
origin of frame_b.
   You may try to use another \"n1_a\" vector. If this fails,
use instead Modelica.Mechanics.MultiBody.Joints.SphericalSpherical, if this is
possible, because this joint aggregation does not have a
singular configuration.
");
          length_n2_a = sqrt(length2_n2_a);
          e2_a = n2_a / length_n2_a;
          e3_a = cross(eRod_a, e2_a);
          der_rRod_a_L = (Frames.resolve2(frame_a.R, der(rRod_0)) - cross(frame_a.R.w, rRod_a)) / rodLength;
          w_rel_ia1 = {(e3_a * cross(n1_a, der_rRod_a_L)) / length_n2_a, -e3_a * der_rRod_a_L,e2_a * der_rRod_a_L};
          R_rel_ia1 = Frames.from_T(transpose([eRod_a,e2_a,e3_a]), w_rel_ia1);
          R_rel_ia2 = Frames.from_T([eRod_ia,e2_ia,e3_ia], zeros(3));
          R_rel_ia = Frames.absoluteRotation(R_rel_ia1, R_rel_ia2);
          frame_ia.r_0 = frame_a.r_0;
          frame_ia.R = Frames.absoluteRotation(frame_a.R, R_rel_ia);
          f_ia_a = Frames.resolve1(R_rel_ia, frame_ia.f);
          t_ia_a = Frames.resolve1(R_rel_ia, frame_ia.t);
          f_b_a1 =  -e2_a * (n1_a * t_ia_a) / (rodLength * n1_a * e3_a) + e3_a * (e2_a * t_ia_a) / rodLength;
          f_b_a =  -f_rod * eRod_a + f_b_a1;
          frame_b.f = Frames.resolveRelative(f_b_a, frame_a.R, frame_b.R);
          frame_b.t = zeros(3);
          zeros(3) = frame_a.f + f_b_a + f_ia_a;
          zeros(3) = frame_a.t + t_ia_a + cross(rRod_a, f_b_a);
          if checkTotalPower then
            totalPower = frame_a.f * Frames.resolve2(frame_a.R, der(frame_a.r_0)) + frame_b.f * Frames.resolve2(frame_b.R, der(frame_b.r_0)) + frame_ia.f * Frames.resolve2(frame_ia.R, der(frame_ia.r_0)) + frame_a.t * Frames.angularVelocity2(frame_a.R) + frame_b.t * Frames.angularVelocity2(frame_b.R) + frame_ia.t * Frames.angularVelocity2(frame_ia.R);

          else
            totalPower = 0;

          end if;
        end UniversalSpherical;
        model GearConstraint "Ideal 3-dim. gearbox (arbitrary shaft directions)"
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
          Interfaces.Frame_a bearing "Coordinate system fixed in the bearing" annotation(Placement(transformation(origin = {0, -100}, extent = {{ -16, -16},{16,16}}, rotation = 90)));
          parameter Real ratio(start = 2) "Gear speed ratio";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n_a = {1,0,0} "Axis of rotation of shaft a (same coordinates in frame_a, frame_b, bearing)";
          parameter Modelica.Mechanics.MultiBody.Types.Axis n_b = {1,0,0} "Axis of rotation of shaft b (same coordinates in frame_a, frame_b, bearing)";
          parameter Modelica.SIunits.Position r_a[3] = {0,0,0} "Vector from frame bearing to frame_a resolved in bearing";
          parameter Modelica.SIunits.Position r_b[3] = {0,0,0} "Vector from frame bearing to frame_b resolved in bearing";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -98,98},{98, -98}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -40,20},{ -20, -20}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -40,140},{ -20,20}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{20,100},{40,60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{20,60},{40, -60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{40,10},{100, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -20,90},{20,70}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -100,10},{ -40, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Text(extent = {{ -156,186},{152,130}}, textString = "%name=%ratio", lineColor = {0,0,255}),Line(points = {{ -80,20},{ -60,20}}, color = {0,0,0}),Line(points = {{ -80, -20},{ -60, -20}}, color = {0,0,0}),Line(points = {{ -10,60},{10,60}}, color = {0,0,0}),Line(points = {{60,20},{80,20}}, color = {0,0,0}),Line(points = {{60, -20},{80, -20}}, color = {0,0,0}),Line(points = {{ -70, -20},{ -70, -70},{70, -70},{70, -20}}, color = {0,0,0}),Line(points = {{0,60},{0, -70},{0, -100}}, color = {0,0,0}),Line(points = {{ -10,100},{10,100}}, color = {0,0,0})}), Documentation(info = "<html>
<p>This ideal massless joint provides a gear constraint between
frames <tt>frame_a</tt> and <tt>frame_b</tt>. The axes of rotation
of <tt>frame_a</tt> and <tt>frame_b</tt> may be arbitrary.</p>
<p><b>Reference</b><br>
<span style=\"font-variant:small-caps\">Schweiger</span>, Christian ;
<span style=\"font-variant:small-caps\">Otter</span>, Martin:
<a href=\"http://www.modelica.org/Conference2003/papers/h06_Schweiger_powertrains_v5.pdf\">Modelling
3D Mechanical Effects of 1-dim. Powertrains</a>. In: <i>Proceedings of the 3rd International
Modelica Conference</i>. Link&ouml;ping : The Modelica Association and Link&ouml;ping University,
November 3-4, 2003, pp. 149-158</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          Modelica.Mechanics.MultiBody.Joints.Revolute actuatedRevolute_a(useAxisFlange = true, n = n_a, animation = false) annotation(Placement(transformation(extent = {{ -40, -10},{ -60,10}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Joints.Revolute actuatedRevolute_b(useAxisFlange = true, n = n_b, animation = false) annotation(Placement(transformation(extent = {{40, -10},{60,10}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Components.IdealGear idealGear(ratio = ratio) annotation(Placement(transformation(extent = {{ -10,30},{10,50}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation1(animation = false, r = r_b) annotation(Placement(transformation(extent = {{10, -10},{30,10}}, rotation = 0)));
          Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation2(animation = false, r = r_a) annotation(Placement(transformation(origin = { -20,0}, extent = {{ -10, -10},{10,10}}, rotation = 180)));

        equation 
          assert(cardinality(bearing) > 0, "Connector bearing of component is not connected");
          connect(actuatedRevolute_a.axis,idealGear.flange_a) annotation(Line(points = {{ -50,10},{ -50,40},{ -10,40}}, color = {0,0,0}));
          connect(idealGear.flange_b,actuatedRevolute_b.axis) annotation(Line(points = {{10,40},{50,40},{50,10}}, color = {0,0,0}));
          connect(actuatedRevolute_a.frame_a,fixedTranslation2.frame_b) annotation(Line(points = {{ -40,0},{ -35,0},{ -35,1.22465e-15},{ -30,1.22465e-15}}, color = {95,95,95}, thickness = 0.5));
          connect(fixedTranslation2.frame_a,bearing) annotation(Line(points = {{ -10, -1.22465e-15},{ -4, -1.22465e-15},{ -4,0},{0,0},{0, -100}}, color = {95,95,95}, thickness = 0.5));
          connect(fixedTranslation1.frame_a,bearing) annotation(Line(points = {{10,0},{0,0},{0, -100}}, color = {95,95,95}, thickness = 0.5));
          connect(fixedTranslation1.frame_b,actuatedRevolute_b.frame_a) annotation(Line(points = {{30,0},{40,0}}, color = {95,95,95}, thickness = 0.5));
          connect(frame_a,actuatedRevolute_a.frame_b) annotation(Line(points = {{ -100,0},{ -60,0}}, color = {95,95,95}, thickness = 0.5));
          connect(actuatedRevolute_b.frame_b,frame_b) annotation(Line(points = {{60,0},{100,0}}, color = {95,95,95}, thickness = 0.5));
        end GearConstraint;
        model RollingWheel "Joint (no mass, no inertia) that describes an ideal rolling wheel (rolling on the plane z=0)"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Frames;
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -80},{100, -100}}, lineColor = {0,0,0}, fillColor = {175,175,175}, fillPattern = FillPattern.Solid),Text(extent = {{ -154,124},{146,84}}, lineColor = {0,0,255}, textString = "%name"),Ellipse(extent = {{ -80,80},{80, -80}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Frame fixed in wheel center point. x-Axis: upwards, y-axis: along wheel axis" annotation(Placement(transformation(extent = {{ -16, -16},{16,16}})));
          parameter SI.Radius wheelRadius "Wheel radius";
          parameter StateSelect stateSelect = StateSelect.always "Priority to use generalized coordinates as states" annotation(HideResult = true, Evaluate = true);
          SI.Position x(start = 0, stateSelect = stateSelect) "x-coordinate of wheel axis";
          SI.Position y(start = 0, stateSelect = stateSelect) "y-coordinate of wheel axis";
          SI.Position z;
          SI.Angle angles[3](start = {0,0,0}, stateSelect = stateSelect) "Angles to rotate world-frame in to frame_a around z-, y-, x-axis" annotation(Dialog(group = "Initialization", __Dymola_initialDialog = true));
          SI.AngularVelocity der_angles[3](start = {0,0,0}, stateSelect = stateSelect) "Derivative of angles" annotation(Dialog(group = "Initialization", __Dymola_initialDialog = true));
          SI.Position r_road_0[3] "Position vector from world frame to contact point on road, resolved in world frame";
          SI.Force f_wheel_0[3] "Contact force acting on wheel, resolved in world frame";
          SI.Force f_n "Contact force acting on wheel in normal direction";
          SI.Force f_lat "Contact force acting on wheel in lateral direction";
          SI.Force f_long "Contact force acting on wheel in longitudinal direction";
          SI.Position err "|r_road_0 - frame_a.r_0| - wheelRadius (must be zero; used for checking)";
        protected 
          Real e_axis_0[3] "Unit vector along wheel axis, resolved in world frame";
          SI.Position delta_0[3](start = {0,0, -wheelRadius}) "Distance vector from wheel center to contact point";
          Real e_n_0[3] "Unit vector in normal direction of road at contact point, resolved in world frame";
          Real e_lat_0[3] "Unit vector in lateral direction of wheel at contact point, resolved in world frame";
          Real e_long_0[3] "Unit vector in longitudinal direction of wheel at contact point, resolved in world frame";
          SI.Position s "Road surface parameter 1";
          SI.Position w "Road surface parameter 2";
          Real e_s_0[3] "Road heading at (s,w), resolved in world frame (unit vector)";
          SI.Velocity v_0[3] "Velocity of wheel center, resolved in world frame";
          SI.AngularVelocity w_0[3] "Angular velocity of wheel, resolved in world frame";
          SI.Velocity vContact_0[3] "Velocity of wheel contact point, resolved in world frame";
          Real aux[3];

        equation 
          Connections.root(frame_a.R);
          frame_a.r_0 = {x,y,z};
          der_angles = der(angles);
          frame_a.R = Frames.axesRotations({3,2,1}, angles, der_angles);
          r_road_0 = {s,w,0};
          e_n_0 = {0,0,1};
          e_s_0 = {1,0,0};
          e_axis_0 = Frames.resolve1(frame_a.R, {0,1,0});
          aux = cross(e_n_0, e_axis_0);
          e_long_0 = aux / Modelica.Math.Vectors.length(aux);
          e_lat_0 = cross(e_long_0, e_n_0);
          delta_0 = r_road_0 - frame_a.r_0;
          0 = delta_0 * e_axis_0;
          0 = delta_0 * e_long_0;
          0 = wheelRadius - delta_0 * cross(e_long_0, e_axis_0);
          err = Modelica.Math.Vectors.length(delta_0) - wheelRadius;
          v_0 = der(frame_a.r_0);
          w_0 = Frames.angularVelocity1(frame_a.R);
          vContact_0 = v_0 + cross(w_0, delta_0);
          0 = vContact_0 * e_long_0;
          0 = vContact_0 * e_lat_0;
          f_wheel_0 = f_n * e_n_0 + f_lat * e_lat_0 + f_long * e_long_0;
          zeros(3) = frame_a.f + Frames.resolve2(frame_a.R, f_wheel_0);
          zeros(3) = frame_a.t + Frames.resolve2(frame_a.R, cross(delta_0, f_wheel_0));
          assert(abs(e_n_0 * e_axis_0) < 0.99, "Wheel lays nearly on the ground (which is a singularity)");
        end RollingWheel;
        model RollingWheelSet "Joint (no mass, no inertia) that describes an ideal rolling wheel set (two ideal rolling wheels connected together by an axis)"
          import SI = Modelica.SIunits;
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frameMiddle "Frame fixed in middle of axis connecting both wheels (y-axis: along wheel axis, z-Axis: upwards)" annotation(Placement(transformation(extent = {{ -16,16},{16, -16}}), iconTransformation(extent = {{ -16, -16},{16,16}})));
          parameter Boolean animation = true "= true, if animation of wheel set shall be enabled";
          parameter SI.Radius wheelRadius "Radius of one wheel";
          parameter SI.Distance wheelDistance "Distance between the two wheels";
          parameter StateSelect stateSelect = StateSelect.default "Priority to use the generalized coordinates as states";
          Modelica.SIunits.Position x(start = 0, stateSelect = stateSelect) "x coordinate for center between wheels";
          Modelica.SIunits.Position y(start = 0, stateSelect = stateSelect) "y coordinate for center between wheels";
          Modelica.SIunits.Angle phi(start = 0, stateSelect = stateSelect) "Orientation angle of wheel axis along z-axis";
          Modelica.SIunits.Angle theta1(start = 0, stateSelect = stateSelect) "Angle of wheel 1";
          Modelica.SIunits.Angle theta2(start = 0, stateSelect = stateSelect) "Angle of wheel 2";
          Modelica.SIunits.AngularVelocity der_theta1(start = 0, stateSelect = stateSelect) "Derivative of theta 1";
          Modelica.SIunits.AngularVelocity der_theta2(start = 0, stateSelect = stateSelect) "Derivative of theta 2";
          annotation(defaultComponentName = "wheelSet", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -80},{100, -100}}, lineColor = {0,0,0}, fillColor = {175,175,175}, fillPattern = FillPattern.Solid),Text(extent = {{ -146, -98},{154, -138}}, textString = "%name", lineColor = {0,0,255}),Ellipse(extent = {{42,80},{118, -80}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -62,2},{64, -6}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -118,80},{ -42, -80}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{86,24},{64,24},{64,10},{56,10}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{86, -24},{64, -24},{64, -12},{56, -12}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{ -96,100},{ -80,100},{ -80,4}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{100,100},{80,100},{80, -2}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{0,72},{0,40},{ -20,40},{ -20,2}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{0,40},{20,40},{20,2}}, color = {0,0,0}, smooth = Smooth.None)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -68,24},{ -68,52}}, color = {0,0,255}, smooth = Smooth.None),Polygon(points = {{ -68,70},{ -74,52},{ -62,52},{ -68,70}}, lineColor = {0,0,255}, smooth = Smooth.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -56,62},{ -38,50}}, lineColor = {0,0,255}, textString = "x"),Line(points = {{ -62,30},{ -94,30}}, color = {0,0,255}, smooth = Smooth.None),Polygon(points = {{ -90,36},{ -90,24},{ -108,30},{ -90,36}}, lineColor = {0,0,255}, smooth = Smooth.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -114,50},{ -96,38}}, lineColor = {0,0,255}, textString = "y")}));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame1 "Frame fixed in center point of left wheel (y-axis: along wheel axis, z-Axis: upwards)" annotation(Placement(transformation(extent = {{ -96,16},{ -64, -16}}), iconTransformation(extent = {{ -96,16},{ -64, -16}})));
          Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame2 "Frame fixed in center point of right wheel (y-axis: along wheel axis, z-Axis: upwards)" annotation(Placement(transformation(extent = {{64,16},{96, -16}})));
          Modelica.Mechanics.MultiBody.Parts.Fixed fixed(r = {0,0,wheelRadius}, animation = animation) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 90, origin = {0, -90})));
          Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod1(r = {0,wheelDistance / 2,0}, animation = animation) annotation(Placement(transformation(extent = {{ -8, -10},{ -28,10}})));
          Modelica.Mechanics.MultiBody.Joints.Prismatic prismatic1(animation = animation) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 90, origin = {0, -66})));
          Modelica.Mechanics.MultiBody.Joints.Prismatic prismatic2(n = {0,1,0}, animation = animation) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 180, origin = { -24, -50})));
          Modelica.Mechanics.MultiBody.Joints.Revolute revolute(animation = animation) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 90, origin = {0, -22})));
          Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod2(r = {0, -wheelDistance / 2,0}, animation = animation) annotation(Placement(transformation(extent = {{12, -10},{32,10}})));
          Modelica.Mechanics.MultiBody.Joints.Revolute revolute1(n = {0,1,0}, useAxisFlange = true, animation = animation) annotation(Placement(transformation(extent = {{ -34, -10},{ -54,10}})));
          Modelica.Mechanics.MultiBody.Joints.Revolute revolute2(n = {0,1,0}, useAxisFlange = true, animation = animation) annotation(Placement(transformation(extent = {{40, -10},{60,10}})));
          Modelica.Mechanics.MultiBody.Joints.Internal.RollingConstraintVerticalWheel rolling1(radius = wheelRadius) annotation(Placement(transformation(extent = {{ -70, -60},{ -50, -40}})));
          Modelica.Mechanics.MultiBody.Joints.Internal.RollingConstraintVerticalWheel rolling2(radius = wheelRadius, lateralSlidingConstraint = false) annotation(Placement(transformation(extent = {{54, -60},{74, -40}})));
          Modelica.Mechanics.Rotational.Interfaces.Flange_a axis1 "1-dim. rotational flange that drives the joint" annotation(Placement(transformation(extent = {{ -110,90},{ -90,110}})));
          Modelica.Mechanics.Rotational.Interfaces.Flange_a axis2 "1-dim. rotational flange that drives the joint" annotation(Placement(transformation(extent = {{90,90},{110,110}})));
          Modelica.Mechanics.MultiBody.Parts.Mounting1D mounting1D annotation(Placement(transformation(extent = {{ -10,38},{10,58}})));
          Modelica.Mechanics.Rotational.Interfaces.Flange_b support "Support of 1D axes" annotation(Placement(transformation(extent = {{ -10,70},{10,90}}), iconTransformation(extent = {{ -10,70},{10,90}})));

        equation 
          prismatic1.s = x;
          prismatic2.s = y;
          revolute.phi = phi;
          revolute1.phi = theta1;
          revolute2.phi = theta2;
          der_theta1 = der(theta1);
          der_theta2 = der(theta2);
          connect(revolute.frame_b,frameMiddle) annotation(Line(points = {{6.12323e-16, -12},{0, -12},{0,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(rod1.frame_a,frameMiddle) annotation(Line(points = {{ -8,0},{0,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(rod2.frame_a,frameMiddle) annotation(Line(points = {{12,0},{0,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(rod1.frame_b,revolute1.frame_a) annotation(Line(points = {{ -28,0},{ -34,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(revolute1.frame_b,frame1) annotation(Line(points = {{ -54,0},{ -80,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(revolute2.frame_a,rod2.frame_b) annotation(Line(points = {{40,0},{32,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(revolute2.frame_b,frame2) annotation(Line(points = {{60,0},{80,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(prismatic1.frame_a,fixed.frame_b) annotation(Line(points = {{ -6.12323e-16, -76},{6.12323e-16, -76},{6.12323e-16, -80}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(prismatic1.frame_b,prismatic2.frame_a) annotation(Line(points = {{6.12323e-16, -56},{6.12323e-16, -50},{ -14, -50}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(prismatic2.frame_b,revolute.frame_a) annotation(Line(points = {{ -34, -50},{ -40, -50},{ -40, -36},{ -6.12323e-16, -36},{ -6.12323e-16, -32}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(rolling1.frame_a,revolute1.frame_b) annotation(Line(points = {{ -60, -48},{ -60,0},{ -54,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(rolling2.frame_a,revolute2.frame_b) annotation(Line(points = {{64, -48},{64,0},{60,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(revolute1.axis,axis1) annotation(Line(points = {{ -44,10},{ -44,100},{ -100,100}}, color = {0,0,0}, smooth = Smooth.None));
          connect(revolute2.axis,axis2) annotation(Line(points = {{50,10},{50,100},{100,100}}, color = {0,0,0}, smooth = Smooth.None));
          connect(frameMiddle,mounting1D.frame_a) annotation(Line(points = {{0,0},{0,38}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(mounting1D.flange_b,support) annotation(Line(points = {{10,48},{16,48},{16,80},{0,80}}, color = {0,0,0}, smooth = Smooth.None));
        end RollingWheelSet;
        package Assemblies "Joint aggregations for analytic loop handling"
          import SI = Modelica.SIunits;
          extends Modelica.Icons.Library;
          annotation(Documentation(info = "<HTML>
<p>
The joints in this package are mainly designed to be used
in <b>kinematic loop</b> structures. Every component consists of
<b>3 elementary joints</b>. These joints are combined in such a
way that the kinematics of the 3 joints between frame_a and
frame_b are computed from the movement of frame_a and frame_b,
i.e., there are <b>no constraints</b> between frame_a and frame_b.
This requires to solve a <b>non-linear system of equations</b> which
is performed <b>analytically</b> (i.e., when a mathematical
solution exists, it is computed efficiently and reliably).
A detailed description how to use these joints is provided in
<a href=\"Modelica://Modelica.Mechanics.MultiBody.UsersGuide.Tutorial.LoopStructures.AnalyticLoopHandling\">MultiBody.UsersGuide.Tutorial.LoopStructures.AnalyticLoopHandling</a>.
</p>
<p>
The assembly joints in this package are named <b>JointXYZ</b> where
<b>XYZ</b> are the first letters of the elementary joints used in the
component, in particular:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>P</b></td><td valign=\"top\">Prismatic joint</td></tr>
  <tr><td valign=\"top\"><b>R</b></td><td valign=\"top\">Revolute joint</td></tr>
  <tr><td valign=\"top\"><b>S</b></td><td valign=\"top\">Spherical joint</td></tr>
  <tr><td valign=\"top\"><b>U</b></td><td valign=\"top\">Universal joint</td></tr>
</table>
<p>
For example, JointUSR is an assembly joint consisting
of a universal, a spherical and a revolute joint.
</p>
<p> This package contains the following models:
</p>
<h4>Content</h4>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Model</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUPS\">JointUPS</a></td>
      <td valign=\"top\"> Universal - prismatic - spherical joint aggregation<br>
     <img src=\"../Images/MultiBody/Joints/JointUPS.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUSR\">JointUSR</a></td>
      <td valign=\"top\"> Universal - spherical - revolute joint aggregation<br>
     <img src=\"../Images/MultiBody/Joints/JointUSR.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUSP\">JointUSP</a></td>
      <td valign=\"top\"> Universal - spherical - prismatic joint aggregation<br>
     <img src=\"../Images/MultiBody/Joints/JointUSP.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies.JointSSR\">JointSSR</a></td>
      <td valign=\"top\"> Spherical - spherical - revolute joint aggregation
           with an optional mass point at the rod connecting
           the two spherical joints<br>
     <img src=\"../Images/MultiBody/Joints/JointSSR.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies.JointSSP\">JointSSP</a></td>
      <td valign=\"top\"> Spherical - spherical - prismatic joint aggregation
           with an optional mass point at the rod connecting
           the two spherical joints<br>
     <img src=\"../Images/MultiBody/Joints/JointSSP.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies.JointRRR\">JointRRR</a></td>
      <td valign=\"top\"> Revolute - revolute - revolute joint aggregation for planar loops<br>
     <img src=\"../Images/MultiBody/Joints/JointRRR.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies.JointRRP\">JointRRP</a></td>
      <td valign=\"top\"> Revolute - revolute - prismatic joint aggregation for planar loops<br>
     <img src=\"../Images/MultiBody/Joints/JointRRP.png\">
      </td>
  </tr>
</table>
<p>
Note, no component of this package has potential states, since the
components are designed in such a way that the generalized coordinates
of the used elementary joints are computed from the frame_a and frame_b
coordinates. Still, it is possible to use the components in a
tree structure. In this case states are selected from bodies that are
connected to the frame_a or frame_b side of the component.
In most cases this gives a less efficient solution, as if elementary
joints of package Modelica.Mechanics.MultiBody.Joints would be used directly.
</p>
<p>
The analytic handling of kinematic loops by using joint aggregations
with 6 degrees of freedom as provided in this package, is a <b>new</b>
methodology. It is based on a more general method for solving
non-linear equations of kinematic loops developed by Woernle and
Hiller. An automatic application of this more general method
is difficult, and a manual application is only suited for
specialists in this field. The method introduced here is a
compromize: It can be quite easily applied by an end user, but
for a smaller class of kinematic loops. The method of the \"characteristic
pair of joints\" from Woernle and Hiller is described in:
</p>
<dl>
<dt>Woernle C.:</dt>
<dd><b>Ein systematisches Verfahren zur Aufstellung der geometrischen
    Schliessbedingungen in kinematischen Schleifen mit Anwendung
    bei der R&uuml;ckw&auml;rtstransformation f&uuml;r
    Industrieroboter.</b><br>
    Fortschritt-Berichte VDI, Reihe 18, Nr. 59, Duesseldorf: VDI-Verlag 1988,
    ISBN 3-18-145918-6.<br>&nbsp;</dd>
<dt>Hiller M., and Woernle C.:</dt
<dd><b>A Systematic Approach for Solving the Inverse Kinematic
    Problem of Robot Manipulators</b>.<br>
    Proceedings 7th World Congress Th. Mach. Mech., Sevilla 1987. </dd>
</dl>
</HTML>"));
          model JointUPS "Universal - prismatic - spherical joint aggregation (no constraints, no potential states)"
            import SI = Modelica.SIunits;
            import Modelica.Mechanics.MultiBody.Types;
            extends Interfaces.PartialTwoFramesDoubleSize;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_ia "Coordinate system at origin of frame_a fixed at prismatic joint" annotation(Placement(transformation(origin = { -80,100}, extent = {{ -8, -8},{8,8}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_ib "Coordinate system at origin of frame_b fixed at prismatic joint" annotation(Placement(transformation(origin = {80,100}, extent = {{ -8,8},{8, -8}}, rotation = 270)));
            Modelica.Mechanics.Translational.Interfaces.Flange_a axis "1-dim. translational flange that drives the prismatic joint" annotation(Placement(transformation(extent = {{45,95},{35,105}}, rotation = 0)));
            Modelica.Mechanics.Translational.Interfaces.Flange_b bearing "1-dim. translational flange of the drive bearing of the prismatic joint" annotation(Placement(transformation(extent = {{ -35,95},{ -45,105}}, rotation = 0)));
            parameter Boolean animation = true "= true, if animation shall be enabled";
            parameter Boolean showUniversalAxes = true " = true, if universal joint shall be visualized with two cylinders, otherwise with a sphere (provided animation=true)";
            parameter Modelica.Mechanics.MultiBody.Types.Axis n1_a = {0,0,1} "Axis 1 of universal joint resolved in frame_a (axis 2 is orthogonal to axis 1 and to line from universal to spherical joint)" annotation(Evaluate = true);
            parameter SI.Position nAxis_ia[3] = {1,0,0} "Axis vector along line from origin of frame_a to origin of frame_b, resolved in frame_ia" annotation(Evaluate = true);
            parameter SI.Position s_offset = 0 "Relative distance offset (distance between frame_a and frame_b = s(t) + s_offset)";
            parameter SI.Diameter sphereDiameter = world.defaultJointLength "Diameter of spheres representing the spherical joints" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color sphereColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of spheres representing the spherical joints" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter axisDiameter = sphereDiameter / Types.Defaults.JointRodDiameterFraction "Diameter of cylinder on the connecting line from frame_a to frame_b" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color axisColor = Modelica.Mechanics.MultiBody.Types.Defaults.SensorColor "Color of cylinder on the connecting line from frame_a to frame_b" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance cylinderLength = world.defaultJointLength "Length of cylinders representing the two universal joint axes" annotation(Dialog(tab = "Animation", group = "if animation = true and showUniversalAxes", enable = animation and showUniversalAxes));
            parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of cylinders representing the two universal joint axes" annotation(Dialog(tab = "Animation", group = "if animation = true and showUniversalAxes", enable = animation and showUniversalAxes));
            input Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinders representing the two universal joint axes" annotation(Dialog(tab = "Animation", group = "if animation = true and showUniversalAxes", enable = animation and showUniversalAxes));
            parameter Boolean checkTotalPower = false "= true, if total power flowing into this component shall be determined (must be zero)" annotation(Dialog(tab = "Advanced"));
            final parameter Real eAxis_ia[3](each final unit = "1") = Modelica.Math.Vectors.normalize(nAxis_ia) "Unit vector from origin of frame_a to origin of frame_b, resolved in frame_ia";
            final parameter Real e2_ia[3](each final unit = "1") = Modelica.Math.Vectors.normalize(cross(n1_a, eAxis_ia)) "Unit vector in direction of second rotation axis of universal joint, resolved in frame_ia";
            final parameter Real e3_ia[3](each final unit = "1") = cross(eAxis_ia, e2_ia) "Unit vector perpendicular to eAxis_ia and e2_ia, resolved in frame_ia";
            SI.Position s "Relative distance between frame_a and frame_b along axis nAxis = s + s_offset";
            SI.Force f "= axis.f (driving force in the axis; = -bearing.f)";
            SI.Length axisLength "Distance between frame_a and frame_b";
            SI.Power totalPower "Total power flowing into this element, if checkTotalPower=true (otherwise dummy)";
            annotation(Documentation(info = "<html>
<p>
This component consists of a <b>universal</b> joint at frame_a,
a <b>spherical</b> joint at frame_b and a <b>prismatic</b> joint along the
line connecting the origin of frame_a and the origin of frame_b,
see the default animation in the following figure (the axes vectors
are not part of the default animation):
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Joints/JointUPS.png\" ALT=\"model Joints.Assemblies.JointUPS\">
</p>
<p>
This joint aggregation has no mass and no inertia and
introduces neither constraints nor potential state variables.
It is especially useful to build up more complicated force elements
where the mass and/or inertia of the force element shall be taken
into account.
</p>
<p>
The universal joint is defined in the following way:
<p>
<ul>
<li> The rotation <b>axis</b> of revolute joint <b>1</b> is along parameter
     vector n1_a which is fixed in frame_a.<li>
<li> The rotation <b>axis</b> of revolute joint <b>2</b> is perpendicular to
     axis 1 and to the line connecting the universal and the spherical joint.
</ul>
<p>
The definition of axis 2 of the universal joint is performed according
to the most often occuring case. In a future release, axis 2 might
be explicitly definable via a parameter. However, the treatment is much more
complicated and the number of operations is considerably higher,
if axis 2 is not orthogonal to axis 1 and to the connecting rod.
</p>
<p>
Note, there is a <b>singularity</b> when axis 1 and the connecting line are parallel
to each other. Therefore, if possible n1_a should be selected in such a way that it
is perpendicular to nAxis_ia in the initial configuration (i.e., the
distance to the singularity is as large as possible).
</p>
<p>
An additional <b>frame_ia</b> is present. It is <b>fixed</b> on the line
connecting the universal and the spherical joint at the
origin of <b>frame_a</b>. The placement of frame_ia on this line
is implicitly defined by the universal joint (frame_a and frame_ia coincide
when the angles of the two revolute joints of the universal joint are zero)
and by parameter vector <b>nAxis_ia</b>, an axis vector directed
along the line from the origin of frame_a to the spherical joint,
resolved in frame_<b>ia</b>.
</p
<p>
An additional <b>frame_ib</b> is present. It is <b>fixed</b> in the line
connecting the prismatic and the spherical joint at the
origin of <b>frame_b</b>.
It is always parallel to <b>frame_ia</b>.
</p>
<p>
Note, this joint aggregation can be used in cases where
in reality a rod with spherical joints at each end are present.
Such a system has an additional degree of freedom to rotate
the rod along its axis. In practice this rotation is usually
of no interested and is mathematically removed by replacing one
of the spherical joints by a universal joint.
</p>
<p>
The easiest way to define the parameters of this joint is by moving the
MultiBody system in a <b>reference configuration</b> where <b>all frames</b>
of all components are <b>parallel</b> to each other (alternatively,
at least frame_a, frame_ia and frame_ib of the JointUSP joint
should be parallel to each other when defining an instance of this
component).
</p>
</html> "), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}, initialScale = 0.2), graphics = {Text(extent = {{ -140, -50},{140, -75}}, lineColor = {0,0,255}, textString = "%name"),Ellipse(extent = {{ -100, -40},{ -19,40}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -90, -30},{ -29,29}}, lineColor = {160,160,164}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -60,41},{ -9, -44}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -60,40},{ -60, -40}}, color = {0,0,0}, thickness = 0.5),Ellipse(extent = {{ -83, -17},{ -34,21}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -74, -12},{ -40,15}}, lineColor = {160,160,164}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -72, -20},{ -89,3},{ -69,25},{ -45,27},{ -72, -20}}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -60,40},{ -60, -10}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -49,20},{ -69, -15}}, color = {0,0,0}, thickness = 0.5),Ellipse(extent = {{44,14},{73, -14}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{20, -40},{100,40}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{30, -30},{90,30}}, lineColor = {192,192,192}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -22,45},{40, -43}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{45,14},{74, -14}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Text(extent = {{ -98,84},{ -60,65}}, lineColor = {128,128,128}, textString = "ia"),Line(points = {{ -40,0},{ -40,90},{ -80,90},{ -80,97}}, color = {95,95,95}, thickness = 0.5),Text(extent = {{61,86},{109,64}}, lineColor = {128,128,128}, textString = "ib"),Rectangle(extent = {{ -35, -13},{ -6,14}}, pattern = LinePattern.None, fillColor = {192,192,192}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{ -35,14},{ -6,18}}, pattern = LinePattern.None, fillColor = {0,0,0}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{ -6, -7},{46,6}}, pattern = LinePattern.None, fillColor = {192,192,192}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{ -6,6},{46,10}}, pattern = LinePattern.None, fillColor = {0,0,0}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -6, -13},{ -6,18}}, color = {0,0,0}),Line(points = {{60, -1},{60,90},{80,90},{80,97}}, color = {95,95,95}, thickness = 0.5),Line(points = {{60,90},{40,90},{40,95}}, color = {95,95,95}, thickness = 0.5),Line(points = {{ -30,70},{10,70}}, color = {0,0,0}),Polygon(points = {{30,70},{10,76},{10,63},{30,70}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{ -40,90},{ -40,90},{ -40,95}}, color = {95,95,95}, thickness = 0.5)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}, initialScale = 0.2), graphics = {Line(points = {{ -60, -70},{46, -70}}, color = {0,0,255}),Polygon(points = {{60, -70},{45, -64},{45, -76},{60, -70}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -62, -73},{65, -90}}, textString = "rAxis", lineColor = {0,0,255}),Ellipse(extent = {{ -100, -40},{ -19,40}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -90, -30},{ -29,29}}, lineColor = {160,160,164}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -60,41},{ -19, -41}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -60,40},{ -60, -40}}, color = {0,0,0}, thickness = 0.5),Ellipse(extent = {{ -83, -17},{ -34,21}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -74, -12},{ -40,15}}, lineColor = {160,160,164}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -72, -20},{ -89,3},{ -69,25},{ -45,27},{ -72, -20}}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -60,40},{ -60, -10}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -49,20},{ -69, -15}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -40,0},{ -40,90},{ -80,90},{ -80,99}}, color = {95,95,95}, thickness = 0.5),Polygon(points = {{7, -1},{ -5,2},{ -5, -4},{7, -1}}, lineColor = {0,0,255}),Line(points = {{ -50,19},{ -30,57}}, color = {0,0,255}),Text(extent = {{ -24,74},{7,53}}, lineColor = {0,0,0}, textString = "e2"),Polygon(points = {{ -25,64},{ -33,56},{ -27,53},{ -25,64}}, lineColor = {0,0,255}),Line(points = {{ -60,41},{ -60,65}}, color = {0,0,255}),Polygon(points = {{ -60,75},{ -64,63},{ -56,63},{ -60,75}}, lineColor = {0,0,255}),Text(extent = {{ -96,82},{ -65,61}}, lineColor = {0,0,0}, textString = "n1"),Line(points = {{ -60, -40},{ -60, -72}}, color = {0,0,255}),Ellipse(extent = {{20, -40},{100,40}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{30, -30},{90,30}}, lineColor = {192,192,192}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -22,45},{40, -43}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{45,14},{74, -14}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {128,128,128}),Line(points = {{60,0},{60, -74}}, color = {0,0,255}),Rectangle(extent = {{ -35,14},{ -6,18}}, pattern = LinePattern.None, fillColor = {0,0,0}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{ -35, -13},{ -6,14}}, pattern = LinePattern.None, fillColor = {192,192,192}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{ -6,6},{46,10}}, pattern = LinePattern.None, fillColor = {0,0,0}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{ -6, -7},{46,6}}, pattern = LinePattern.None, fillColor = {192,192,192}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -6, -13},{ -6,18}}, color = {0,0,0}),Text(extent = {{ -40, -2},{ -1, -16}}, lineColor = {0,0,0}, textString = "nAxis"),Line(points = {{ -61,1},{ -2,1}}, color = {0,0,255}),Polygon(points = {{10,1},{ -2,4},{ -2, -2},{10,1}}, lineColor = {0,0,255}),Line(points = {{60, -1},{60,90},{80,90},{80,99}}, color = {95,95,95}, thickness = 0.5),Text(extent = {{ -24,117},{ -9,102}}, textString = "f", lineColor = {0,0,255}),Polygon(points = {{ -26,103},{ -36,100},{ -26,97},{ -26,103}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Polygon(points = {{26,103},{36,100},{26,97},{26,103}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Line(points = {{14,100},{36,100}}, color = {0,0,255}),Text(extent = {{12,116},{27,101}}, textString = "f", lineColor = {0,0,255}),Polygon(points = {{30,93},{40,90},{30,87},{30,93}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{ -40,90},{40,90}}, color = {128,128,128}),Line(points = {{ -25,100},{ -10,100}}, color = {0,0,255}),Text(extent = {{ -18,90},{19,77}}, lineColor = {128,128,128}, textString = "s"),Line(points = {{60,90},{40,90},{40,98}}, color = {95,95,95}, thickness = 0.5),Line(points = {{ -40,90},{ -40,96},{ -40,98}}, color = {135,135,135}, thickness = 0.5)}));
          protected 
            SI.Force f_c_a[3] "frame_ia.f resolved in frame_a";
            SI.Torque t_cd_a[3] "frame_ia.t + frame_ib.t resolved in frame_a";
            SI.Force f_bd_a[3] "frame_b.f + frame_ib.f resolved in frame_a";
            SI.Position rAxis_0[3] "Position vector from origin of frame_a to origin of frame_b resolved in world frame";
            SI.Position rAxis_a[3] "Position vector from origin of frame_a to origin of frame_b resolved in frame_a";
            Real eAxis_a[3](each final unit = "1") "Unit vector in direction of rAxis_a, resolved in frame_a";
            Real e2_a[3](each final unit = "1") "Unit vector in direction of second rotation axis of universal joint, resolved in frame_a";
            Real e3_a[3](each final unit = "1") "Unit vector perpendicular to eAxis_a and e2_a, resolved in frame_a";
            Real n2_a[3](each final unit = "1") "Vector in direction of second rotation axis of universal joint, resolved in frame_a";
            Real length2_n2_a(unit = "m2") "Square of length of vector n2_a";
            SI.Length length_n2_a "Length of vector n2_a";
            Real der_rAxis_a_L[3](each unit = "1/s") "= der(rAxis_a)/axisLength";
            SI.AngularVelocity w_rel_ia1[3];
            Frames.Orientation R_ia1_a;
            Frames.Orientation R_ia2_a;
            Frames.Orientation R_ia_a "Rotation from frame_a to frame_ia";
            Visualizers.Advanced.Shape axisCylinder(shapeType = "cylinder", color = axisColor, specularCoefficient = specularCoefficient, length = axisLength, width = axisDiameter, height = axisDiameter, lengthDirection = eAxis_ia, widthDirection = e2_ia, r = frame_ia.r_0, R = frame_ia.R) if world.enableAnimation and animation;
            Visualizers.Advanced.Shape sphericalShape_b(shapeType = "sphere", color = sphereColor, specularCoefficient = specularCoefficient, length = sphereDiameter, width = sphereDiameter, height = sphereDiameter, lengthDirection = {1,0,0}, widthDirection = {0,1,0}, r_shape = { -0.5,0,0} * sphereDiameter, r = frame_b.r_0, R = frame_b.R) if world.enableAnimation and animation;
            Visualizers.Advanced.Shape sphericalShape_a(shapeType = "sphere", color = sphereColor, specularCoefficient = specularCoefficient, length = sphereDiameter, width = sphereDiameter, height = sphereDiameter, lengthDirection = {1,0,0}, widthDirection = {0,1,0}, r_shape = { -0.5,0,0} * sphereDiameter, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
            Visualizers.Advanced.Shape universalShape1(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = n1_a, widthDirection = {0,1,0}, r_shape =  -n1_a * cylinderLength / 2, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation and showUniversalAxes;
            Visualizers.Advanced.Shape universalShape2(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = e2_ia, widthDirection = {0,1,0}, r_shape =  -e2_ia * cylinderLength / 2, r = frame_ia.r_0, R = frame_ia.R) if world.enableAnimation and animation and showUniversalAxes;

          equation 
            Connections.branch(frame_a.R, frame_ia.R);
            Connections.branch(frame_ia.R, frame_ib.R);
            axisLength = s + s_offset;
            bearing.s = 0;
            axis.s = s;
            axis.f = f;
            rAxis_0 = frame_b.r_0 - frame_a.r_0;
            rAxis_a = Frames.resolve2(frame_a.R, rAxis_0);
            axisLength = sqrt(rAxis_0 * rAxis_0);
            assert(axisLength > 1e-15, "
Distance between frame_a and frame_b of a JointUPS joint
became zero. This is not allowed. If this occurs during
initialization, the initial conditions are probably wrong.");
            eAxis_a = rAxis_a / axisLength;
            n2_a = cross(n1_a, eAxis_a);
            length2_n2_a = n2_a * n2_a;
            assert(noEvent(length2_n2_a > 1e-10), "
A Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUPS joint (consisting of
a universal, prismatic and spherical joint) is in the singular
configuration of the universal joint. This means that axis 1 of
the universal joint defined via parameter \"n1_a\" is parallel to vector
\"eAxis_ia\" that is directed from the origin of frame_a to the
origin of frame_b. You may try to use another \"n1_a\" vector.
");
            length_n2_a = sqrt(length2_n2_a);
            e2_a = n2_a / length_n2_a;
            e3_a = cross(eAxis_a, e2_a);
            der_rAxis_a_L = (Frames.resolve2(frame_a.R, der(rAxis_0)) - cross(frame_a.R.w, rAxis_a)) / axisLength;
            w_rel_ia1 = {(e3_a * cross(n1_a, der_rAxis_a_L)) / length_n2_a, -e3_a * der_rAxis_a_L,e2_a * der_rAxis_a_L};
            R_ia1_a = Frames.from_T(transpose([eAxis_a,e2_a,e3_a]), w_rel_ia1);
            R_ia2_a = Frames.from_T([eAxis_ia,e2_ia,e3_ia], zeros(3));
            R_ia_a = Frames.absoluteRotation(R_ia1_a, R_ia2_a);
            frame_ia.r_0 = frame_a.r_0;
            frame_ib.r_0 = frame_b.r_0;
            frame_ia.R = Frames.absoluteRotation(frame_a.R, R_ia_a);
            frame_ib.R = frame_ia.R;
            f_c_a = Frames.resolve1(R_ia_a, frame_ia.f);
            t_cd_a = Frames.resolve1(R_ia_a, frame_ia.t + frame_ib.t);
            f_bd_a =  -eAxis_a * f - e2_a * (n1_a * t_cd_a) / (axisLength * n1_a * e3_a) + e3_a * (e2_a * t_cd_a) / axisLength;
            zeros(3) = frame_b.f + Frames.resolveRelative(frame_ib.f, frame_ib.R, frame_b.R) - Frames.resolveRelative(f_bd_a, frame_a.R, frame_b.R);
            zeros(3) = frame_b.t;
            zeros(3) = frame_a.f + f_c_a + f_bd_a;
            zeros(3) = frame_a.t + t_cd_a + cross(rAxis_a, f_bd_a);
            if checkTotalPower then
              totalPower = frame_a.f * Frames.resolve2(frame_a.R, der(frame_a.r_0)) + frame_b.f * Frames.resolve2(frame_b.R, der(frame_b.r_0)) + frame_ia.f * Frames.resolve2(frame_ia.R, der(frame_ia.r_0)) + frame_ib.f * Frames.resolve2(frame_ib.R, der(frame_ib.r_0)) + frame_a.t * Frames.angularVelocity2(frame_a.R) + frame_b.t * Frames.angularVelocity2(frame_b.R) + frame_ia.t * Frames.angularVelocity2(frame_ia.R) + frame_ib.t * Frames.angularVelocity2(frame_ib.R) + axis.f * der(axis.s) + bearing.f * der(bearing.s);

            else
              totalPower = 0;

            end if;
          end JointUPS;
          model JointUSR "Universal - spherical - revolute joint aggregation (no constraints, no potential states)"
            import SI = Modelica.SIunits;
            import Cv = Modelica.SIunits.Conversions;
            import Modelica.Mechanics.MultiBody.Types;
            extends Interfaces.PartialTwoFramesDoubleSize;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_ia "Coordinate system at origin of frame_a fixed at connecting rod of universal and spherical joint" annotation(Placement(transformation(origin = { -80,100}, extent = {{ -8, -8},{8,8}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_ib "Coordinate system at origin of frame_b fixed at connecting rod of spherical and revolute joint" annotation(Placement(transformation(origin = {80,100}, extent = {{ -8,8},{8, -8}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_im "Coordinate system at origin of spherical joint fixed at connecting rod of spherical and revolute joint" annotation(Placement(transformation(origin = {0,100}, extent = {{8, -8},{ -8,8}}, rotation = 270)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_a axis "1-dim. rotational flange that drives the revolute joint" annotation(Placement(transformation(extent = {{105,85},{95,75}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_b bearing "1-dim. rotational flange of the drive bearing of the revolute joint" annotation(Placement(transformation(extent = {{95,45},{105,35}}, rotation = 0)));
            parameter Boolean animation = true "= true, if animation shall be enabled";
            parameter Boolean showUniversalAxes = true " = true, if universal joint shall be visualized with two cylinders, otherwise with a sphere (provided animation=true)";
            parameter Modelica.Mechanics.MultiBody.Types.Axis n1_a = {0,0,1} "Axis 1 of universal joint fixed and resolved in frame_a (axis 2 is orthogonal to axis 1 and to rod 1)" annotation(Evaluate = true);
            parameter Modelica.Mechanics.MultiBody.Types.Axis n_b = {0,0,1} "Axis of revolute joint fixed and resolved in frame_b" annotation(Evaluate = true);
            parameter SI.Position rRod1_ia[3] = {1,0,0} "Vector from origin of frame_a to spherical joint, resolved in frame_ia" annotation(Evaluate = true);
            parameter SI.Position rRod2_ib[3] = { -1,0,0} "Vector from origin of frame_ib to spherical joint, resolved in frame_ib";
            parameter Cv.NonSIunits.Angle_deg phi_offset = 0 "Relative angle offset of revolute joint (angle = phi(t) + from_deg(phi_offset))";
            parameter Cv.NonSIunits.Angle_deg phi_guess = 0 "Select the configuration such that at initial time |phi(t0) - from_deg(phi_guess)|is minimal";
            parameter SI.Diameter sphereDiameter = world.defaultJointLength "Diameter of the spheres representing the universal and the spherical joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color sphereColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of the spheres representing the universal and the spherical joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter rod1Diameter = sphereDiameter / Types.Defaults.JointRodDiameterFraction "Diameter of rod 1 connecting the universal and the spherical joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color rod1Color = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor "Color of rod 1 connecting the universal and the spherical joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter rod2Diameter = rod1Diameter "Diameter of rod 2 connecting the revolute and the spherical joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color rod2Color = rod1Color "Color of rod 2 connecting the revolute and the spherical joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter revoluteDiameter = world.defaultJointWidth "Diameter of cylinder representing the revolute joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance revoluteLength = world.defaultJointLength "Length of cylinder representing the revolute joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color revoluteColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinder representing the revolute joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance cylinderLength = world.defaultJointLength "Length of cylinders representing the two universal joint axes" annotation(Dialog(tab = "Animation", group = "if animation = true and showUniversalAxes", enable = animation and showUniversalAxes));
            parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of cylinders representing the two universal joint axes" annotation(Dialog(tab = "Animation", group = "if animation = true and showUniversalAxes", enable = animation and showUniversalAxes));
            input Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinders representing the two universal joint axes" annotation(Dialog(tab = "Animation", group = "if animation = true and showUniversalAxes", enable = animation and showUniversalAxes));
            parameter Boolean checkTotalPower = false "= true, if total power flowing into this component shall be determined (must be zero)" annotation(Dialog(tab = "Advanced"));
            final parameter Real eRod1_ia[3](each final unit = "1") = rod1.eRod_ia "Unit vector from origin of frame_a to origin of spherical joint, resolved in frame_ia";
            final parameter Real e2_ia[3](each final unit = "1") = rod1.e2_ia "Unit vector in direction of axis 2 of universal joint, resolved in frame_ia";
            final parameter SI.Distance rod1Length = rod1.rodLength "Length of rod 1 (= distance between universal and spherical joint";
            SI.Power totalPower "Total power flowing into this element, if checkTotalPower=true (otherwise dummy)";
            SI.Position aux "Denominator used to compute force in rod connecting universal and spherical joint";
            SI.Force f_rod "Constraint force in direction of the rod (positive, if rod is pressed)";
            annotation(Documentation(info = "<html>
<p>
This component consists of a <b>universal</b> joint at frame_a, a <b>revolute</b>
joint at frame_b and a <b>spherical</b> joint which is connected via <b>rod1</b>
to the universal and via <b>rod2</b> to the revolute joint, see the default
animation in the following figure (the axes vectors are not part of the
default animation):
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Joints/JointUSR.png\" ALT=\"model Joints.Assemblies.JointUSR\">
</p>
<p>
This joint aggregation has no mass and no inertia and
introduces neither constraints nor potential state variables.
It should be used in kinematic loops whenever possible since
the non-linear system of equations introduced by this joint aggregation
is solved <b>analytically</b> (i.e., a solution is always computed, if a
unique solution exists).
</p>
<p>
The universal joint is defined in the following way:
<p>
<ul>
<li> The rotation <b>axis</b> of revolute joint <b>1</b> is along parameter
     vector n1_a which is fixed in frame_a.<li>
<li> The rotation <b>axis</b> of revolute joint <b>2</b> is perpendicular to
     axis 1 and to the line connecting the universal and the spherical joint
     (= rod 1).
</ul>
<p>
The definition of axis 2 of the universal joint is performed according
to the most often occuring case. In a future release, axis 2 might
be explicitly definable via a parameter. However, the treatment is much more
complicated and the number of operations is considerably higher,
if axis 2 is not orthogonal to axis 1 and to the connecting rod.
</p>
<p>
Note, there is a <b>singularity</b> when axis 1 and the connecting rod are parallel
to each other. Therefore, if possible n1_a should be selected in such a way that it
is perpendicular to rRod1_ia in the initial configuration (i.e., the
distance to the singularity is as large as possible).
</p>
<p>
The rest of this joint aggregation is defined by the following parameters:
</p>
<ul>
<li> The position of the spherical joint with respect to the universal
     joint is defined by vector <b>rRod1_ia</b>. This vector is directed from
     frame_a to the spherical joint and is resolved in frame_ia
     (it is most simple to select frame_ia such that it is parallel to
     frame_a in the reference or initial configuration).</li>
<li> The position of the spherical joint with respect to the revolute
     joint is defined by vector <b>rRod2_ib</b>. This vector is directed from
     the inner frame of the revolute joint (frame_ib or revolute.frame_a)
     to the spherical joint and is resolved in frame_ib (note, that frame_ib
     and frame_b are parallel to each other).</li>
<li> The axis of rotation of the revolute joint is defined by axis
     vector <b>n_b</b>. It is fixed and resolved in frame_b.</li>
<li> When specifying this joint aggregation with the definitions above, <b>two</b>
     different <b>configurations</b> are possible. Via parameter <b>phi_guess</b>
     a guess value for revolute.phi(t0) at the initial time t0 is given. The configuration
     is selected that is closest to phi_guess (|revolute.phi - phi_guess|is minimal).
</ul>
<p>
An additional <b>frame_ia</b> is present. It is <b>fixed</b> in the rod
connecting the universal and the spherical joint at the
origin of <b>frame_a</b>. The placement of frame_ia on the rod
is implicitly defined by the universal joint (frame_a and frame_ia coincide
when the angles of the two revolute joints of the universal joint are zero)
and by parameter vector <b>rRod1_ia</b>, the position vector
from the origin of frame_a to the spherical joint, resolved in frame_<b>ia</b>.
</p
<p>
An additional <b>frame_ib</b> is present. It is <b>fixed</b> in the rod
connecting the revolute and the spherical joint at the side of the revolute
joint that is connected to this rod (= rod2.frame_a = revolute.frame_a).
</p>
<p>
An additional <b>frame_im</b> is present. It is <b>fixed</b> in the rod
connecting the revolute and the spherical joint at the side of the spherical
joint that is connected to this rod (= rod2.frame_b).
It is always parallel to <b>frame_ib</b>.
</p>
<p>
The easiest way to define the parameters of this joint is by moving the
MultiBody system in a <b>reference configuration</b> where <b>all frames</b>
of all components are <b>parallel</b> to each other (alternatively,
at least frame_a and frame_ia of the JointUSR joint
should be parallel to each other when defining an instance of this
component).
</p>
<p>
In the public interface of the JointUSR joint, the following
(final) <b>parameters</b> are provided:
</p>
<pre>
  <b>parameter</b> Real rod1Length(unit=\"m\")  \"Length of rod 1\";
  <b>parameter</b> Real eRod1_ia[3] \"Unit vector along rod 1, resolved in frame_ia\";
  <b>parameter</b> Real e2_ia  [3]  \"Unit vector along axis 2, resolved in frame_ia\";
</pre>
<p>
This allows a more convenient definition of data which is related to rod 1.
For example, if a box shall be connected at frame_ia directing from
the origin of frame_a to the middle of rod 1, this might be defined as:
</p>
<pre>
    Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUSP jointUSR(rRod1_ia={1.2, 1, 0.2});
    Modelica.Mechanics.MultiBody.Visualizers.FixedShape     shape(shapeType       = \"box\",
                                               lengthDirection = jointUSR.eRod1_ia,
                                               widthDirection  = jointUSR.e2_ia,
                                               length          = jointUSR.rod1Length/2,
                                               width           = jointUSR.rod1Length/10);
  <b>equation</b>
    <b>connect</b>(jointUSP.frame_ia, shape.frame_a);
</pre>
</html> "), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}, initialScale = 0.2), graphics = {Text(extent = {{ -140, -41},{140, -66}}, lineColor = {0,0,255}, textString = "%name"),Ellipse(extent = {{ -100, -30},{ -40,30}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -93, -22},{ -48,23}}, lineColor = {160,160,164}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -70,40},{ -39, -33}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -70,28},{ -70, -30}}, color = {0,0,0}, thickness = 0.5),Ellipse(extent = {{ -89, -18},{ -48,18}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -84, -12},{ -53,13}}, lineColor = {160,160,164}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -81, -17},{ -92, -1},{ -83,16},{ -57,24},{ -81, -17}}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -70,30},{ -70, -10}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -61,16},{ -79, -15}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -50,0},{ -50,80},{ -80,80},{ -80,100}}, color = {95,95,95}, thickness = 0.5),Ellipse(extent = {{ -40, -30},{20,30}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -33, -22},{12,23}}, lineColor = {192,192,192}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -44,31},{ -14, -30}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -23,10},{ -3, -10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{19,6},{61, -6}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -50,5},{ -21, -5}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{60, -30},{76,30}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{85, -30},{100,30}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{76,10},{85, -10}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{60,30},{76, -30}}, lineColor = {0,0,0}),Rectangle(extent = {{85,30},{100, -30}}, lineColor = {0,0,0}),Text(extent = {{40,109},{77,91}}, lineColor = {128,128,128}, textString = "ib"),Text(extent = {{ -124,109},{ -95,92}}, lineColor = {128,128,128}, textString = "ia"),Line(points = {{60,30},{60,80},{80,80},{80,100}}, color = {95,95,95}, thickness = 0.5),Text(extent = {{ -43,108},{ -10,92}}, lineColor = {128,128,128}, textString = "im"),Line(points = {{19,6},{19,80},{0,80},{0,100}}, color = {95,95,95}, thickness = 0.5),Line(points = {{80,80},{101,80}}, color = {128,128,128}, thickness = 0.5),Line(points = {{90,30},{90,40},{95,40}}, color = {95,95,95}, thickness = 0.5)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}, initialScale = 0.2), graphics));
            Modelica.Mechanics.MultiBody.Joints.Internal.RevoluteWithLengthConstraint revolute(animation = animation, lengthConstraint = rod1Length, n = n_b, phi_offset = phi_offset, phi_guess = phi_guess, cylinderDiameter = revoluteDiameter, cylinderLength = revoluteLength, cylinderColor = revoluteColor, specularCoefficient = specularCoefficient) annotation(Placement(transformation(extent = {{75, -20},{35,20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.UniversalSpherical rod1(animation = animation, showUniversalAxes = showUniversalAxes, rRod_ia = rRod1_ia, n1_a = n1_a, sphereDiameter = sphereDiameter, sphereColor = sphereColor, rodWidth = rod1Diameter, rodHeight = rod1Diameter, rodColor = rod1Color, cylinderLength = cylinderLength, cylinderDiameter = cylinderDiameter, cylinderColor = cylinderColor, specularCoefficient = specularCoefficient, kinematicConstraint = false, constraintResidue = rod1.f_rod - f_rod) annotation(Placement(transformation(extent = {{ -92, -20},{ -52,20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod2(animation = animation, width = rod2Diameter, height = rod2Diameter, color = rod2Color, specularCoefficient = specularCoefficient, r = rRod2_ib) annotation(Placement(transformation(extent = {{15, -20},{ -25,20}}, rotation = 0)));
            Sensors.RelativePosition relativePosition(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a) annotation(Placement(transformation(extent = {{60, -70},{40, -90}}, rotation = 0)));
            Modelica.Blocks.Sources.Constant position_b[3](k = rRod2_ib) annotation(Placement(transformation(extent = {{ -20, -50},{0, -30}}, rotation = 0)));

          equation 
            aux = cross(revolute.e, rRod2_ib) * Frames.resolveRelative(rod1.eRod_a, rod1.frame_a.R, rod1.frame_b.R);
            f_rod = ( -revolute.tau - revolute.e * (frame_ib.t + frame_im.t + cross(rRod2_ib, frame_im.f) - cross(rRod2_ib, Frames.resolveRelative(rod1.f_b_a1, rod1.frame_a.R, rod1.frame_b.R)))) / noEvent(if abs(aux) < 1e-10 then 1e-10 else aux);
            if checkTotalPower then
              totalPower = frame_a.f * Frames.resolve2(frame_a.R, der(frame_a.r_0)) + frame_b.f * Frames.resolve2(frame_b.R, der(frame_b.r_0)) + frame_ia.f * Frames.resolve2(frame_ia.R, der(frame_ia.r_0)) + frame_ib.f * Frames.resolve2(frame_ib.R, der(frame_ib.r_0)) + frame_im.f * Frames.resolve2(frame_im.R, der(frame_im.r_0)) + frame_a.t * Frames.angularVelocity2(frame_a.R) + frame_b.t * Frames.angularVelocity2(frame_b.R) + frame_ia.t * Frames.angularVelocity2(frame_ia.R) + frame_ib.t * Frames.angularVelocity2(frame_ib.R) + frame_im.t * Frames.angularVelocity2(frame_im.R) + axis.tau * der(axis.phi) + bearing.tau * der(bearing.phi);

            else
              totalPower = 0;

            end if;
            connect(revolute.frame_b,rod2.frame_a) annotation(Line(points = {{35,0},{15,0}}, color = {95,95,95}, thickness = 0.5));
            connect(rod2.frame_b,rod1.frame_b) annotation(Line(points = {{ -25,0},{ -52,0}}, color = {95,95,95}, thickness = 0.5));
            connect(revolute.frame_a,frame_b) annotation(Line(points = {{75,0},{100,0}}, color = {95,95,95}, thickness = 0.5));
            connect(rod2.frame_a,frame_ib) annotation(Line(points = {{15,0},{26,0},{26,70},{80,70},{80,100}}, color = {95,95,95}, thickness = 0.5));
            connect(rod1.frame_a,frame_a) annotation(Line(points = {{ -92,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5));
            connect(relativePosition.frame_b,frame_a) annotation(Line(points = {{40, -80},{ -96, -80},{ -96,0},{ -100,0}}, color = {95,95,95}, pattern = LinePattern.Dot));
            connect(relativePosition.frame_a,frame_b) annotation(Line(points = {{60, -80},{96, -80},{96,0},{100,0}}, color = {95,95,95}, pattern = LinePattern.Dot));
            connect(position_b.y,revolute.position_b) annotation(Line(points = {{1, -40},{20, -40},{20, -12},{31, -12}}, color = {0,0,127}));
            connect(rod2.frame_b,frame_im) annotation(Line(points = {{ -25,0},{ -40,0},{ -40,80},{0,80},{0,100}}, color = {95,95,95}, thickness = 0.5));
            connect(rod1.frame_ia,frame_ia) annotation(Line(points = {{ -80,20},{ -80,100}}, color = {95,95,95}, thickness = 0.5));
            connect(revolute.axis,axis) annotation(Line(points = {{55,20},{55,60},{90,60},{90,80},{100,80}}, color = {0,0,0}));
            connect(relativePosition.r_rel,revolute.position_a) annotation(Line(points = {{50, -69},{50, -40},{90, -40},{90, -12},{79, -12}}, color = {0,0,127}, smooth = Smooth.None));
            connect(revolute.bearing,bearing) annotation(Line(points = {{67,20},{67,40},{100,40}}, color = {0,0,0}, smooth = Smooth.None));
          end JointUSR;
          model JointUSP "Universal - spherical - prismatic joint aggregation (no constraints, no potential states)"
            import SI = Modelica.SIunits;
            import Modelica.Mechanics.MultiBody.Types;
            extends Interfaces.PartialTwoFramesDoubleSize;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_ia "Coordinate system at origin of frame_a fixed at connecting rod of universal and spherical joint" annotation(Placement(transformation(origin = { -80,100}, extent = {{ -8, -8},{8,8}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_ib "Coordinate system at origin of frame_b fixed at connecting rod of spherical and prismatic joint" annotation(Placement(transformation(origin = {80,100}, extent = {{ -8,8},{8, -8}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_im "Coordinate system at origin of spherical joint fixed at connecting rod of spherical and prismatic joint" annotation(Placement(transformation(origin = {0,100}, extent = {{8, -8},{ -8,8}}, rotation = 270)));
            Modelica.Mechanics.Translational.Interfaces.Flange_a axis "1-dim. translational flange that drives the prismatic joint" annotation(Placement(transformation(extent = {{95,75},{105,85}}, rotation = 0)));
            Modelica.Mechanics.Translational.Interfaces.Flange_b bearing "1-dim. translational flange of the drive bearing of the prismatic joint" annotation(Placement(transformation(extent = {{105,35},{95,45}}, rotation = 0)));
            parameter Boolean animation = true "= true, if animation shall be enabled";
            parameter Boolean showUniversalAxes = true " = true, if universal joint shall be visualized with two cylinders, otherwise with a sphere (provided animation=true)";
            parameter Modelica.Mechanics.MultiBody.Types.Axis n1_a = {0,0,1} "Axis 1 of universal joint fixed and resolved in frame_a (axis 2 is orthogonal to axis 1 and to rod 1)" annotation(Evaluate = true);
            parameter Modelica.Mechanics.MultiBody.Types.Axis n_b = { -1,0,0} "Axis of prismatic joint fixed and resolved in frame_b" annotation(Evaluate = true);
            parameter SI.Position rRod1_ia[3] = {1,0,0} "Vector from origin of frame_a to spherical joint, resolved in frame_ia" annotation(Evaluate = true);
            parameter SI.Position rRod2_ib[3] = { -1,0,0} "Vector from origin of frame_ib to spherical joint, resolved in frame_ib (frame_ib is parallel to frame_b)" annotation(Evaluate = true);
            parameter SI.Position s_offset = 0 "Relative distance offset of prismatic joint (distance between the prismatic joint frames = s(t) + s_offset)";
            parameter SI.Position s_guess = 0 "Select the configuration such that at initial time |s(t0)-s_guess|is minimal";
            parameter SI.Diameter sphereDiameter = world.defaultJointLength "Diameter of the spheres representing the universal and the spherical joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color sphereColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of the spheres representing the universal and the spherical joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter rod1Diameter = sphereDiameter / Types.Defaults.JointRodDiameterFraction "Diameter of rod 1 connecting the universal and the spherical joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color rod1Color = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor "Color of rod 1 connecting the universal and the spherical joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter rod2Diameter = rod1Diameter "Diameter of rod 2 connecting the prismatic and the spherical joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color rod2Color = rod1Color "Color of rod 2 connecting the prismatic and the spherical joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter Types.Axis boxWidthDirection = {0,1,0} "Vector in width direction of prismatic joint, resolved in frame_b" annotation(Evaluate = true, Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance boxWidth = world.defaultJointWidth "Width of prismatic joint box" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance boxHeight = boxWidth "Height of prismatic joint box" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color boxColor = sphereColor "Color of prismatic joint box" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance cylinderLength = world.defaultJointLength "Length of cylinders representing the two universal joint axes" annotation(Dialog(tab = "Animation", group = "if animation = true and showUniversalAxes", enable = animation and showUniversalAxes));
            parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of cylinders representing the two universal joint axes" annotation(Dialog(tab = "Animation", group = "if animation = true and showUniversalAxes", enable = animation and showUniversalAxes));
            input Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinders representing the two universal joint axes" annotation(Dialog(tab = "Animation", group = "if animation = true and showUniversalAxes", enable = animation and showUniversalAxes));
            parameter Boolean checkTotalPower = false "= true, if total power flowing into this component shall be determined (must be zero)" annotation(Dialog(tab = "Advanced"));
            final parameter Real eRod1_ia[3](each final unit = "1") = rod1.eRod_ia "Unit vector from origin of frame_a to origin of spherical joint, resolved in frame_ia";
            final parameter Real e2_ia[3](each final unit = "1") = rod1.e2_ia "Unit vector in direction of axis 2 of universal joint, resolved in frame_ia";
            final parameter SI.Distance rod1Length = rod1.rodLength "Length of rod 1 (= distance between universal and spherical joint";
            SI.Force f_rod "Constraint force in direction of the rod (positive, if rod is pressed)";
            SI.Power totalPower "Total power flowing into this element, if checkTotalPower=true (otherwise dummy)";
            annotation(Documentation(info = "<html>
<p>
This component consists of a <b>universal</b> joint at frame_a, a <b>prismatic</b>
joint at frame_b and a <b>spherical</b> joint which is connected via <b>rod1</b>
to the universal and via <b>rod2</b> to the prismatic joint, see the default
animation in the following figure (the axes vectors are not part of the
default animation):
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Joints/JointUSP.png\" ALT=\"model Joints.Assemblies.JointUSP\">
</p>
<p>
This joint aggregation has no mass and no inertia and
introduces neither constraints nor potential state variables.
It should be used in kinematic loops whenever possible since
the non-linear system of equations introduced by this joint aggregation
is solved <b>analytically</b> (i.e., a solution is always computed, if a
unique solution exists).
</p>
<p>
The universal joint is defined in the following way:
<p>
<ul>
<li> The rotation <b>axis</b> of revolute joint <b>1</b> is along parameter
     vector n1_a which is fixed in frame_a.<li>
<li> The rotation <b>axis</b> of revolute joint <b>2</b> is perpendicular to
     axis 1 and to the line connecting the universal and the spherical joint
     (= rod 1).
</ul>
<p>
The definition of axis 2 of the universal joint is performed according
to the most often occuring case. In a future release, axis 2 might
be explicitly definable via a parameter. However, the treatment is much more
complicated and the number of operations is considerably higher,
if axis 2 is not orthogonal to axis 1 and to the connecting rod.
</p>
<p>
Note, there is a <b>singularity</b> when axis 1 and the connecting rod are parallel
to each other. Therefore, if possible n1_a should be selected in such a way that it
is perpendicular to rRod1_ia in the initial configuration (i.e., the
distance to the singularity is as large as possible).
</p>
<p>
The rest of this joint aggregation is defined by the following parameters:
</p>
<ul>
<li> The position of the spherical joint with respect to the universal
     joint is defined by vector <b>rRod1_ia</b>. This vector is directed from
     frame_a to the spherical joint and is resolved in frame_ia
     (it is most simple to select frame_ia such that it is parallel to
     frame_a in the reference or initial configuration).</li>
<li> The position of the spherical joint with respect to the prismatic
     joint is defined by vector <b>rRod2_ib</b>. This vector is directed from
     the inner frame of the prismatic joint (frame_ib or prismatic.frame_a)
     to the spherical joint and is resolved in frame_ib (note, that frame_ib
     and frame_b are parallel to each other).</li>
<li> The axis of translation of the prismatic joint is defined by axis
     vector <b>n_b</b>. It is fixed and resolved in frame_b.</li>
<li> The two frames of the prismatic joint, i.e., frame_b and frame_ib,
     are parallel to each other.
     The distance between the origins of these two frames along axis n_b
     is equal to \"prismatic.s(t) + s_offset\", where \"prismatic.s(t)\" is
     a time varying variable and \"s_offset\" is a fixed, constant offset
     parameter.</li>
<li> When specifying this joint aggregation with the definitions above, <b>two</b>
     different <b>configurations</b> are possible. Via parameter <b>s_guess</b>
     a guess value for prismatic.s(t0) at the initial time t0 is given. The configuration
     is selected that is closest to s_guess (|prismatic.s - s_guess|is minimal).
</ul>
<p>
An additional <b>frame_ia</b> is present. It is <b>fixed</b> in the rod
connecting the universal and the spherical joint at the
origin of <b>frame_a</b>. The placement of frame_ia on the rod
is implicitly defined by the universal joint (frame_a and frame_ia coincide
when the angles of the two revolute joints of the universal joint are zero)
and by parameter vector <b>rRod1_ia</b>, the position vector
from the origin of frame_a to the spherical joint, resolved in frame_<b>ia</b>.
</p
<p>
An additional <b>frame_ib</b> is present. It is <b>fixed</b> in the rod
connecting the prismatic and the spherical joint at the side of the prismatic
joint that is connected to this rod (= rod2.frame_a = prismatic.frame_a).
It is always parallel to <b>frame_b</b>.
</p>
<p>
An additional <b>frame_im</b> is present. It is <b>fixed</b> in the rod
connecting the prismatic and the spherical joint at the side of the spherical
joint that is connected to this rod (= rod2.frame_b).
It is always parallel to <b>frame_b</b>.
</p>
<p>
The easiest way to define the parameters of this joint is by moving the
MultiBody system in a <b>reference configuration</b> where <b>all frames</b>
of all components are <b>parallel</b> to each other (alternatively,
at least frame_a and frame_ia of the JointUSP joint
should be parallel to each other when defining an instance of this
component).
</p>
<p>
In the public interface of the JointUSP joint, the following
(final) <b>parameters</b> are provided:
</p>
<pre>
  <b>parameter</b> Real rod1Length(unit=\"m\")  \"Length of rod 1\";
  <b>parameter</b> Real eRod1_ia[3] \"Unit vector along rod 1, resolved in frame_ia\";
  <b>parameter</b> Real e2_ia  [3]  \"Unit vector along axis 2, resolved in frame_ia\";
</pre>
<p>
This allows a more convenient definition of data which is related to rod 1.
For example, if a box shall be connected at frame_ia directing from
the origin of frame_a to the middle of rod 1, this might be defined as:
</p>
<pre>
    Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUSP jointUSP(rRod1_ia={1.2, 1, 0.2});
    Modelica.Mechanics.MultiBody.Visualizers.FixedShape     shape(shapeType       = \"box\",
                                               lengthDirection = jointUSP.eRod1_ia,
                                               widthDirection  = jointUSP.e2_ia,
                                               length          = jointUSP.rod1Length/2,
                                               width           = jointUSP.rod1Length/10);
  <b>equation</b>
    <b>connect</b>(jointUSP.frame_ia, shape.frame_a);
</pre>
</html> "), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}, initialScale = 0.2), graphics = {Rectangle(extent = {{50,20},{80, -20}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{80,30},{100, -30}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -140, -45},{140, -70}}, lineColor = {0,0,255}, textString = "%name"),Ellipse(extent = {{ -100, -30},{ -40,30}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -93, -22},{ -48,23}}, lineColor = {160,160,164}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -70,40},{ -39, -33}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -70,28},{ -70, -30}}, color = {0,0,0}, thickness = 0.5),Ellipse(extent = {{ -89, -18},{ -48,18}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -84, -12},{ -53,13}}, lineColor = {160,160,164}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -81, -17},{ -92, -1},{ -83,16},{ -57,24},{ -81, -17}}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -70,30},{ -70, -10}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -61,16},{ -79, -15}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -50,0},{ -50,80},{ -80,80},{ -80,100}}, color = {95,95,95}, thickness = 0.5),Ellipse(extent = {{ -40, -30},{20,30}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -33, -22},{12,23}}, lineColor = {192,192,192}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -44,31},{ -14, -30}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -23,10},{ -3, -10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{19,6},{50, -6}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -50,5},{ -21, -5}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Text(extent = {{37,109},{68,90}}, lineColor = {128,128,128}, textString = "ib"),Text(extent = {{ -124,110},{ -93,90}}, lineColor = {128,128,128}, textString = "ia"),Line(points = {{50,6},{50,80},{80,80},{80,100}}, color = {95,95,95}, thickness = 0.5),Text(extent = {{ -44,111},{ -8,91}}, lineColor = {128,128,128}, textString = "im"),Line(points = {{19,6},{19,80},{0,80},{0,100}}, color = {95,95,95}, thickness = 0.5),Rectangle(extent = {{80,24},{100,30}}, pattern = LinePattern.None, fillColor = {0,0,0}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{50,14},{80,20}}, pattern = LinePattern.None, fillColor = {0,0,0}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{95,80},{79,80}}, color = {135,135,135}, thickness = 0.5),Line(points = {{95,40},{90,40},{90,30}}, color = {135,135,135}, thickness = 0.5)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}, initialScale = 0.2), graphics = {Line(points = {{ -78,30},{ -50,30}}, color = {128,128,128}, arrow = {Arrow.None,Arrow.Filled}),Text(extent = {{ -76,39},{ -49,32}}, lineColor = {128,128,128}, textString = "rRod1_ia"),Text(extent = {{ -27,40},{0,33}}, lineColor = {128,128,128}, textString = "rRod2_ib"),Line(points = {{3,30},{ -43,30}}, color = {128,128,128}, arrow = {Arrow.None,Arrow.Filled})}));
            Modelica.Mechanics.MultiBody.Joints.Internal.PrismaticWithLengthConstraint prismatic(animation = animation, length = rod1.rodLength, n = n_b, s_offset = s_offset, s_guess = s_guess, boxWidthDirection = boxWidthDirection, boxWidth = boxWidth, boxHeight = boxHeight, boxColor = boxColor, specularCoefficient = specularCoefficient) annotation(Placement(transformation(extent = {{76, -20},{36,20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.UniversalSpherical rod1(animation = animation, showUniversalAxes = showUniversalAxes, rRod_ia = rRod1_ia, n1_a = n1_a, sphereDiameter = sphereDiameter, sphereColor = sphereColor, rodWidth = rod1Diameter, rodHeight = rod1Diameter, rodColor = rod1Color, specularCoefficient = specularCoefficient, cylinderLength = cylinderLength, cylinderDiameter = cylinderDiameter, cylinderColor = cylinderColor, kinematicConstraint = false, constraintResidue = rod1.f_rod - f_rod) annotation(Placement(transformation(extent = {{ -92, -20},{ -52,20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod2(animation = animation, r = rRod2_ib, width = rod2Diameter, height = rod2Diameter, specularCoefficient = specularCoefficient, color = rod2Color) annotation(Placement(transformation(extent = {{0,20},{ -40, -20}}, rotation = 0)));
            Sensors.RelativePosition relativePosition(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a) annotation(Placement(transformation(extent = {{50, -70},{30, -90}}, rotation = 0)));
            Modelica.Blocks.Sources.Constant position_b[3](k = rRod2_ib) annotation(Placement(transformation(extent = {{ -20, -60},{0, -40}}, rotation = 0)));
          protected 
            Real aux "Denominator used to compute force in rod connecting universal and spherical joint";

          equation 
            aux = prismatic.e * Frames.resolveRelative(rod1.eRod_a, rod1.frame_a.R, rod1.frame_b.R);
            f_rod = ( -prismatic.f - prismatic.e * (frame_ib.f + frame_im.f - Frames.resolveRelative(rod1.f_b_a1, rod1.frame_a.R, rod1.frame_b.R))) / noEvent(if abs(aux) < 1e-10 then 1e-10 else aux);
            if checkTotalPower then
              totalPower = frame_a.f * Frames.resolve2(frame_a.R, der(frame_a.r_0)) + frame_b.f * Frames.resolve2(frame_b.R, der(frame_b.r_0)) + frame_ia.f * Frames.resolve2(frame_ia.R, der(frame_ia.r_0)) + frame_ib.f * Frames.resolve2(frame_ib.R, der(frame_ib.r_0)) + frame_im.f * Frames.resolve2(frame_im.R, der(frame_im.r_0)) + frame_a.t * Frames.angularVelocity2(frame_a.R) + frame_b.t * Frames.angularVelocity2(frame_b.R) + frame_ia.t * Frames.angularVelocity2(frame_ia.R) + frame_ib.t * Frames.angularVelocity2(frame_ib.R) + frame_im.t * Frames.angularVelocity2(frame_im.R) + axis.f * der(axis.s) + bearing.f * der(bearing.s);

            else
              totalPower = 0;

            end if;
            connect(prismatic.frame_b,rod2.frame_a) annotation(Line(points = {{36,0},{0,0}}, color = {95,95,95}, thickness = 0.5));
            connect(rod2.frame_b,rod1.frame_b) annotation(Line(points = {{ -40,0},{ -52,0}}, color = {0,0,0}, thickness = 0.5));
            connect(prismatic.frame_a,frame_b) annotation(Line(points = {{76,0},{100,0}}, color = {95,95,95}, thickness = 0.5));
            connect(rod2.frame_a,frame_ib) annotation(Line(points = {{0,0},{7,0},{7,70},{80,70},{80,100}}, color = {95,95,95}, thickness = 0.5));
            connect(rod1.frame_a,frame_a) annotation(Line(points = {{ -92,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5));
            connect(relativePosition.frame_b,frame_a) annotation(Line(points = {{30, -80},{ -97, -80},{ -97,0},{ -100,0}}, color = {95,95,95}, pattern = LinePattern.Dot));
            connect(relativePosition.frame_a,frame_b) annotation(Line(points = {{50, -80},{95, -80},{95,0},{100,0}}, color = {95,95,95}, pattern = LinePattern.Dot));
            connect(rod2.frame_b,frame_im) annotation(Line(points = {{ -40,0},{ -46,0},{ -46,80},{0,80},{0,100}}, color = {95,95,95}, thickness = 0.5));
            connect(rod1.frame_ia,frame_ia) annotation(Line(points = {{ -80,20},{ -80,100}}, color = {95,95,95}, thickness = 0.5));
            connect(position_b.y,prismatic.position_b) annotation(Line(points = {{1, -50},{10, -50},{10, -12},{32, -12}}, color = {0,0,127}));
            connect(prismatic.axis,axis) annotation(Line(points = {{40,14},{40,56},{90,56},{90,80},{100,80}}, color = {0,191,0}));
            connect(prismatic.bearing,bearing) annotation(Line(points = {{64,14},{64,40},{100,40}}, color = {0,191,0}));
            connect(relativePosition.r_rel,prismatic.position_a) annotation(Line(points = {{40, -69},{40, -50},{90, -50},{90, -12},{80, -12}}, color = {0,0,127}, smooth = Smooth.None));
          end JointUSP;
          model JointSSR "Spherical - spherical - revolute joint aggregation with mass (no constraints, no potential states)"
            import SI = Modelica.SIunits;
            import Cv = Modelica.SIunits.Conversions;
            import Modelica.Mechanics.MultiBody.Types;
            extends Interfaces.PartialTwoFramesDoubleSize;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_ib "Coordinate system at origin of frame_b fixed at connecting rod of spherical and revolute joint" annotation(Placement(transformation(origin = {80,100}, extent = {{ -8,8},{8, -8}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_im "Coordinate system at origin of spherical joint in the middle fixed at connecting rod of spherical and revolute joint" annotation(Placement(transformation(origin = {0,100}, extent = {{8, -8},{ -8,8}}, rotation = 270)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_a axis "1-dim. rotational flange that drives the revolute joint" annotation(Placement(transformation(extent = {{105,85},{95,75}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_b bearing "1-dim. rotational flange of the drive bearing of the revolute joint" annotation(Placement(transformation(extent = {{95,45},{105,35}}, rotation = 0)));
            parameter Boolean animation = true "= true, if animation shall be enabled";
            parameter Boolean showMass = true "= true, if point mass on rod 1 shall be shown (provided animation = true and rod1Mass > 0)";
            parameter SI.Length rod1Length(min = Modelica.Constants.eps, start = 1) "Distance between the origins of the two spherical joints ";
            parameter SI.Mass rod1Mass(min = 0) = 0 "Mass of rod 1 (= point mass located in middle of rod connecting the two spherical joints)";
            parameter Modelica.Mechanics.MultiBody.Types.Axis n_b = {0,0,1} "Axis of revolute joint fixed and resolved in frame_b";
            parameter SI.Position rRod2_ib[3] = {1,0,0} "Vector from origin of frame_ib to spherical joint in the middle, resolved in frame_ib";
            parameter Cv.NonSIunits.Angle_deg phi_offset = 0 "Relative angle offset of revolute joint (angle = phi(t) + from_deg(phi_offset))";
            parameter Cv.NonSIunits.Angle_deg phi_guess = 0 "Select the configuration such that at initial time |phi(t0) - from_deg(phi_guess)|is minimal";
            parameter SI.Diameter sphereDiameter = world.defaultJointLength "Diameter of the spheres representing the two spherical joints" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color sphereColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of the spheres representing the two spherical joints" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter rod1Diameter = sphereDiameter / Types.Defaults.JointRodDiameterFraction "Diameter of rod 1 connecting the two spherical joints" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color rod1Color = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor "Color of rod 1 connecting the two spherical joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter rod2Diameter = rod1Diameter "Diameter of rod 2 connecting the revolute joint and spherical joint 2" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color rod2Color = rod1Color "Color of rod 2 connecting the revolute joint and spherical joint 2" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter revoluteDiameter = world.defaultJointWidth "Diameter of cylinder representing the revolute joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance revoluteLength = world.defaultJointLength "Length of cylinder representing the revolute joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color revoluteColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinder representing the revolute joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter Boolean checkTotalPower = false "= true, if total power flowing into this component shall be determined (must be zero)" annotation(Dialog(tab = "Advanced"));
            SI.Position aux "Denominator used to compute force in rod connecting universal and spherical joint";
            SI.Force f_rod "Constraint force in direction of the rod (positive, if rod is pressed)";
            SI.Power totalPower "Total power flowing into this element, if checkTotalPower=true (otherwise dummy)";
            annotation(Documentation(info = "<html>
<p>
This component consists of a <b>spherical</b> joint 1 at frame_a, a <b>revolute</b>
joint at frame_b and a <b>spherical</b> joint 2 which is connected via rod 1
to the spherical joint 1 and via rod 2 to the revolute joint, see the default
animation in the following figure (the axes vectors are not part of the
default animation):
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Joints/JointSSR.png\" ALT=\"model Joints.Assemblies.JointSSR\">
</p>
<p>
Besides an optional point mass in the middle of rod 1,
this joint aggregation has no mass and no inertia,
and introduces neither constraints nor potential state variables.
It should be used in kinematic loops whenever possible since
the non-linear system of equations introduced by this joint aggregation
is solved <b>analytically</b> (i.e., a solution is always computed, if a
unique solution exists).
</p>
<p>
An additional <b>frame_ib</b> is present. It is <b>fixed</b> in rod 2
connecting the revolute and the spherical joint at the side of the revolute
joint that is connected to this rod (= rod2.frame_a = revolute.frame_a).
</p>
<p>
An additional <b>frame_im</b> is present. It is <b>fixed</b> in rod 2
connecting the revolute and the spherical joint at the side of spherical
joint 2 that is connected to this rod (= rod2.frame_b).
It is always parallel to <b>frame_ib</b>.
</p>
<p>
The easiest way to define the parameters of this joint is by moving the
MultiBody system in a <b>reference configuration</b> where <b>all frames</b>
of all components are <b>parallel</b> to each other (alternatively,
at least frame_b and frame_ib of the JointSSR joint
should be parallel to each other when defining an instance of this
component).
</p>
</html> "), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}, initialScale = 0.2), graphics = {Text(extent = {{ -141, -41},{139, -66}}, lineColor = {0,0,255}, textString = "%name"),Ellipse(extent = {{ -100, -30},{ -40,30}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -93, -22},{ -48,23}}, lineColor = {160,160,164}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -63,33},{ -39, -33}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -40, -30},{20,30}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -33, -22},{12,23}}, lineColor = {192,192,192}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -44,31},{ -19, -30}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -23,10},{ -3, -10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{19,6},{61, -6}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{60, -30},{76,30}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{85, -30},{100,30}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{76,10},{85, -10}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{60,30},{76, -30}}, lineColor = {0,0,0}),Rectangle(extent = {{85,30},{100, -30}}, lineColor = {0,0,0}),Text(extent = {{88,112},{127,92}}, lineColor = {128,128,128}, textString = "ib"),Ellipse(extent = {{ -80,11},{ -60, -9}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -62,6},{ -21, -5}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Line(points = {{80,80},{100,80}}, color = {95,95,95}, thickness = 0.5),Line(points = {{19,6},{19,80},{0,80},{0,100}}, color = {95,95,95}, thickness = 0.5),Text(extent = {{ -47,111},{ -8,92}}, lineColor = {128,128,128}, textString = "im"),Line(points = {{68,30},{68,80},{80,80},{80,98}}, color = {95,95,95}, thickness = 0.5),Line(points = {{90,30},{90,40},{95,40}}, color = {95,95,95}, thickness = 0.5)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}, initialScale = 0.2), graphics));
            Modelica.Mechanics.MultiBody.Joints.Internal.RevoluteWithLengthConstraint revolute(animation = animation, lengthConstraint = rod1Length, n = n_b, phi_offset = phi_offset, phi_guess = phi_guess, cylinderDiameter = revoluteDiameter, cylinderLength = revoluteLength, cylinderColor = revoluteColor, specularCoefficient = specularCoefficient) annotation(Placement(transformation(extent = {{75, -20},{35,20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.SphericalSpherical rod1(animation = animation, showMass = showMass, m = rod1Mass, rodLength = rod1Length, rodDiameter = rod1Diameter, sphereDiameter = sphereDiameter, rodColor = rod1Color, specularCoefficient = specularCoefficient, kinematicConstraint = false, sphereColor = sphereColor, constraintResidue = rod1.f_rod - f_rod) annotation(Placement(transformation(extent = {{ -89, -20},{ -49,20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod2(animation = animation, width = rod2Diameter, height = rod2Diameter, color = rod2Color, specularCoefficient = specularCoefficient, r = rRod2_ib) annotation(Placement(transformation(extent = {{15, -20},{ -25,20}}, rotation = 0)));
            Sensors.RelativePosition relativePosition(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a) annotation(Placement(transformation(extent = {{60, -70},{40, -90}}, rotation = 0)));
            Modelica.Blocks.Sources.Constant position_b[3](k = rRod2_ib) annotation(Placement(transformation(extent = {{ -20, -50},{0, -30}}, rotation = 0)));

          equation 
            aux = cross(revolute.e, rRod2_ib) * Frames.resolveRelative(rod1.eRod_a, rod1.frame_a.R, rod1.frame_b.R);
            f_rod = ( -revolute.tau - revolute.e * (frame_ib.t + frame_im.t + cross(rRod2_ib, frame_im.f) - cross(rRod2_ib, Frames.resolveRelative(rod1.f_b_a1, rod1.frame_a.R, rod1.frame_b.R)))) / noEvent(if abs(aux) < 1e-10 then 1e-10 else aux);
            if checkTotalPower then
              totalPower = frame_a.f * Frames.resolve2(frame_a.R, der(frame_a.r_0)) + frame_b.f * Frames.resolve2(frame_b.R, der(frame_b.r_0)) + frame_ib.f * Frames.resolve2(frame_ib.R, der(frame_ib.r_0)) + frame_im.f * Frames.resolve2(frame_im.R, der(frame_im.r_0)) + frame_a.t * Frames.angularVelocity2(frame_a.R) + frame_b.t * Frames.angularVelocity2(frame_b.R) + frame_ib.t * Frames.angularVelocity2(frame_ib.R) + frame_im.t * Frames.angularVelocity2(frame_im.R) + axis.tau * der(axis.phi) + bearing.tau * der(bearing.phi) + ( -rod1Mass) * (der(rod1.v_CM_0) - world.gravityAcceleration(rod1.r_CM_0)) * rod1.v_CM_0;

            else
              totalPower = 0;

            end if;
            connect(revolute.frame_b,rod2.frame_a) annotation(Line(points = {{35,0},{15,0}}, color = {95,95,95}, thickness = 0.5));
            connect(rod2.frame_b,rod1.frame_b) annotation(Line(points = {{ -25,0},{ -49,0}}, color = {95,95,95}, thickness = 0.5));
            connect(revolute.frame_a,frame_b) annotation(Line(points = {{75,0},{100,0}}, color = {95,95,95}, thickness = 0.5));
            connect(rod2.frame_a,frame_ib) annotation(Line(points = {{15,0},{26,0},{26,70},{80,70},{80,100}}, color = {95,95,95}, thickness = 0.5));
            connect(rod1.frame_a,frame_a) annotation(Line(points = {{ -89,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5));
            connect(relativePosition.frame_b,frame_a) annotation(Line(points = {{40, -80},{ -95, -80},{ -95,0},{ -100,0}}, color = {95,95,95}, pattern = LinePattern.Dot));
            connect(relativePosition.frame_a,frame_b) annotation(Line(points = {{60, -80},{96, -80},{96,0},{100,0}}, color = {95,95,95}, pattern = LinePattern.Dot));
            connect(position_b.y,revolute.position_b) annotation(Line(points = {{1, -40},{20, -40},{20, -12},{31, -12}}, color = {0,0,127}));
            connect(revolute.axis,axis) annotation(Line(points = {{55,20},{55,60},{90,60},{90,80},{100,80}}, color = {0,0,0}));
            connect(rod2.frame_b,frame_im) annotation(Line(points = {{ -25,0},{ -35,0},{ -35,60},{0,60},{0,100}}, color = {95,95,95}, thickness = 0.5));
            connect(relativePosition.r_rel,revolute.position_a) annotation(Line(points = {{50, -69},{50, -50},{90, -50},{90, -12},{79, -12}}, color = {0,0,127}, smooth = Smooth.None));
            connect(revolute.bearing,bearing) annotation(Line(points = {{67,20},{67,40},{100,40}}, color = {0,0,0}, smooth = Smooth.None));
          end JointSSR;
          model JointSSP "Spherical - spherical - prismatic joint aggregation with mass (no constraints, no potential states)"
            import SI = Modelica.SIunits;
            import Cv = Modelica.SIunits.Conversions;
            import Modelica.Mechanics.MultiBody.Types;
            extends Interfaces.PartialTwoFramesDoubleSize;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_ib "Coordinate system at origin of frame_b fixed at connecting rod of spherical and prismatic joint" annotation(Placement(transformation(origin = {80,100}, extent = {{ -8,8},{8, -8}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_im "Coordinate system at origin of spherical joint in the middle fixed at connecting rod of spherical and prismatic joint" annotation(Placement(transformation(origin = {0,100}, extent = {{8, -8},{ -8,8}}, rotation = 270)));
            Modelica.Mechanics.Translational.Interfaces.Flange_a axis "1-dim. translational flange that drives the prismatic joint" annotation(Placement(transformation(extent = {{95,75},{105,85}}, rotation = 0)));
            Modelica.Mechanics.Translational.Interfaces.Flange_b bearing "1-dim. translational flange of the drive bearing of the prismatic joint" annotation(Placement(transformation(extent = {{105,35},{95,45}}, rotation = 0)));
            parameter Boolean animation = true "= true, if animation shall be enabled";
            parameter Boolean showMass = true "= true, if point mass on rod 1 shall be shown (provided animation = true and rod1Mass > 0)";
            parameter SI.Length rod1Length(min = Modelica.Constants.eps, start = 1) "Distance between the origins of the two spherical joints ";
            parameter SI.Mass rod1Mass(min = 0) = 0 "Mass of rod 1 (= point mass located in middle of rod connecting the two spherical joints)";
            parameter Modelica.Mechanics.MultiBody.Types.Axis n_b = {0,0,1} "Axis of prismatic joint fixed and resolved in frame_b";
            parameter SI.Position rRod2_ib[3] = {1,0,0} "Vector from origin of frame_ib to spherical joint in the middle, resolved in frame_ib";
            parameter SI.Position s_offset = 0 "Relative distance offset of prismatic joint (distance between frame_b and frame_ib = s(t) + s_offset)";
            parameter SI.Position s_guess = 0 "Select the configuration such that at initial time |s(t0)-s_guess|is minimal";
            parameter SI.Diameter sphereDiameter = world.defaultJointLength "Diameter of the spheres representing the two spherical joints" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color sphereColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of the spheres representing the two spherical joints" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter rod1Diameter = sphereDiameter / Types.Defaults.JointRodDiameterFraction "Diameter of rod 1 connecting the two spherical joints" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color rod1Color = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor "Color of rod 1 connecting the two spherical joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter rod2Diameter = rod1Diameter "Diameter of rod 2 connecting the revolute joint and spherical joint 2" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color rod2Color = rod1Color "Color of rod 2 connecting the revolute joint and spherical joint 2" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter Types.Axis boxWidthDirection = {0,1,0} "Vector in width direction of prismatic joint box, resolved in frame_b" annotation(Evaluate = true, Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance boxWidth = world.defaultJointWidth "Width of prismatic joint box" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance boxHeight = boxWidth "Height of prismatic joint box" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color boxColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of prismatic joint box" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter Boolean checkTotalPower = false "= true, if total power flowing into this component shall be determined (must be zero)" annotation(Dialog(tab = "Advanced"));
            Real aux "Denominator used to compute force in rod connecting universal and spherical joint";
            SI.Force f_rod "Constraint force in direction of the rod (positive, if rod is pressed)";
            SI.Power totalPower "Total power flowing into this element, if checkTotalPower=true (otherwise dummy)";
            annotation(Documentation(info = "<html>
<p>
This component consists of a <b>spherical</b> joint 1 at frame_a, a <b>prismatic</b>
joint at frame_b and a <b>spherical</b> joint 2 which is connected via rod 1
to the spherical joint 1 and via rod 2 to the prismatic joint, see the default
animation in the following figure (the axes vectors are not part of the
default animation):
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Joints/JointSSP.png\" ALT=\"model Joints.Assemblies.JointSSP\">
</p>
<p>
Besides an optional point mass in the middle of rod 1,
this joint aggregation has no mass and no inertia,
and introduces neither constraints nor potential state variables.
It should be used in kinematic loops whenever possible since
the non-linear system of equations introduced by this joint aggregation
is solved <b>analytically</b> (i.e., a solution is always computed, if a
unique solution exists).
</p>
<p>
An additional <b>frame_ib</b> is present. It is <b>fixed</b> in rod 2
connecting the prismatic and the spherical joint at the side of the prismatic
joint that is connected to this rod (= rod2.frame_a = prismatic.frame_a).
</p>
<p>
An additional <b>frame_im</b> is present. It is <b>fixed</b> in rod 2
connecting the prismatic and the spherical joint at the side of spherical
joint 2 that is connected to this rod (= rod2.frame_b).
It is always parallel to <b>frame_ib</b>.
</p>
<p>
The easiest way to define the parameters of this joint is by moving the
MultiBody system in a <b>reference configuration</b> where <b>all frames</b>
of all components are <b>parallel</b> to each other (alternatively,
at least frame_b and frame_ib of the JointSSP joint
should be parallel to each other when defining an instance of this
component).
</p>
</html> "), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}, initialScale = 0.2), graphics = {Text(extent = {{ -140, -40},{140, -65}}, lineColor = {0,0,255}, textString = "%name"),Ellipse(extent = {{ -100, -30},{ -40,30}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -93, -22},{ -48,23}}, lineColor = {160,160,164}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -63,33},{ -39, -33}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -40, -30},{20,30}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -33, -22},{12,23}}, lineColor = {192,192,192}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -44,31},{ -19, -30}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -23,10},{ -3, -10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{19,6},{61, -6}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Text(extent = {{89,115},{132,92}}, lineColor = {128,128,128}, textString = "ib"),Ellipse(extent = {{ -80,11},{ -60, -9}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -62,6},{ -21, -5}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Line(points = {{19,6},{19,80},{0,80},{0,100}}, color = {95,95,95}, thickness = 0.5),Text(extent = {{ -49,114},{ -11,92}}, lineColor = {128,128,128}, textString = "im"),Rectangle(extent = {{50,20},{80, -20}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{80,30},{100, -30}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{50,14},{80,20}}, pattern = LinePattern.None, fillColor = {0,0,0}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{80,24},{100,30}}, pattern = LinePattern.None, fillColor = {0,0,0}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{50,6},{50,80},{80,80},{80,100}}, color = {95,95,95}, thickness = 0.5),Line(points = {{101,80},{80,80}}, color = {95,95,95}, thickness = 0.5),Line(points = {{99,40},{90,40},{90,30}}, color = {95,95,95}, thickness = 0.5)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}, initialScale = 0.2), graphics));
            Modelica.Mechanics.MultiBody.Joints.Internal.PrismaticWithLengthConstraint prismatic(animation = animation, length = rod1Length, n = n_b, s_offset = s_offset, s_guess = s_guess, boxWidthDirection = boxWidthDirection, boxWidth = boxWidth, boxHeight = boxHeight, specularCoefficient = specularCoefficient, boxColor = boxColor) annotation(Placement(transformation(extent = {{75, -20},{35,20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Joints.SphericalSpherical rod1(animation = animation, showMass = showMass, m = rod1Mass, rodLength = rod1Length, rodDiameter = rod1Diameter, sphereDiameter = sphereDiameter, rodColor = rod1Color, kinematicConstraint = false, specularCoefficient = specularCoefficient, sphereColor = sphereColor, constraintResidue = rod1.f_rod - f_rod) annotation(Placement(transformation(extent = {{ -89, -20},{ -49,20}}, rotation = 0)));
            Modelica.Mechanics.MultiBody.Parts.FixedTranslation rod2(animation = animation, width = rod2Diameter, height = rod2Diameter, specularCoefficient = specularCoefficient, color = rod2Color, r = rRod2_ib) annotation(Placement(transformation(extent = {{15, -20},{ -25,20}}, rotation = 0)));
            Sensors.RelativePosition relativePosition(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_a) annotation(Placement(transformation(extent = {{60, -70},{40, -90}}, rotation = 0)));
            Modelica.Blocks.Sources.Constant position_b[3](k = rRod2_ib) annotation(Placement(transformation(extent = {{ -20, -50},{0, -30}}, rotation = 0)));

          equation 
            aux = prismatic.e * Frames.resolveRelative(rod1.eRod_a, rod1.frame_a.R, rod1.frame_b.R);
            f_rod = ( -prismatic.f - prismatic.e * (frame_ib.f + frame_im.f)) / noEvent(if abs(aux) < 1e-10 then 1e-10 else aux);
            if checkTotalPower then
              totalPower = frame_a.f * Frames.resolve2(frame_a.R, der(frame_a.r_0)) + frame_b.f * Frames.resolve2(frame_b.R, der(frame_b.r_0)) + frame_ib.f * Frames.resolve2(frame_ib.R, der(frame_ib.r_0)) + frame_im.f * Frames.resolve2(frame_im.R, der(frame_im.r_0)) + frame_a.t * Frames.angularVelocity2(frame_a.R) + frame_b.t * Frames.angularVelocity2(frame_b.R) + frame_ib.t * Frames.angularVelocity2(frame_ib.R) + frame_im.t * Frames.angularVelocity2(frame_im.R) + axis.f * der(axis.s) + bearing.f * der(bearing.s) + ( -rod1Mass) * (der(rod1.v_CM_0) - world.gravityAcceleration(rod1.r_CM_0)) * rod1.v_CM_0;

            else
              totalPower = 0;

            end if;
            connect(prismatic.frame_b,rod2.frame_a) annotation(Line(points = {{35,0},{15,0}}, color = {95,95,95}, thickness = 0.5));
            connect(rod2.frame_b,rod1.frame_b) annotation(Line(points = {{ -25,0},{ -49,0}}, color = {95,95,95}, thickness = 0.5));
            connect(prismatic.frame_a,frame_b) annotation(Line(points = {{75,0},{100,0}}, color = {95,95,95}, thickness = 0.5));
            connect(rod2.frame_a,frame_ib) annotation(Line(points = {{15,0},{26,0},{26,70},{80,70},{80,100}}, color = {95,95,95}, thickness = 0.5));
            connect(rod1.frame_a,frame_a) annotation(Line(points = {{ -89,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5));
            connect(relativePosition.frame_b,frame_a) annotation(Line(points = {{40, -80},{ -95, -80},{ -95,0},{ -100,0}}, color = {95,95,95}, pattern = LinePattern.Dot));
            connect(relativePosition.frame_a,frame_b) annotation(Line(points = {{60, -80},{96, -80},{96,0},{100,0}}, color = {95,95,95}, pattern = LinePattern.Dot));
            connect(position_b.y,prismatic.position_b) annotation(Line(points = {{1, -40},{20, -40},{20, -12},{31, -12}}, color = {0,0,127}));
            connect(prismatic.axis,axis) annotation(Line(points = {{39,14},{40,14},{40,60},{90,60},{90,80},{100,80}}, color = {0,0,0}));
            connect(prismatic.bearing,bearing) annotation(Line(points = {{63,14},{63,40},{100,40}}, color = {0,0,0}));
            connect(rod2.frame_b,frame_im) annotation(Line(points = {{ -25,0},{ -35,0},{ -35,60},{0,60},{0,100}}, color = {95,95,95}, thickness = 0.5));
            connect(relativePosition.r_rel,prismatic.position_a) annotation(Line(points = {{50, -69},{50, -50},{90, -50},{90, -12},{79, -12}}, color = {0,0,127}, smooth = Smooth.None));
          end JointSSP;
          model JointRRR "Planar revolute - revolute - revolute joint aggregation (no constraints, no potential states)"
            import SI = Modelica.SIunits;
            import Cv = Modelica.SIunits.Conversions;
            import Modelica.Mechanics.MultiBody.Types;
            extends Interfaces.PartialTwoFramesDoubleSize;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_ia "Coordinate system at origin of frame_a fixed at connecting rod of left and middle revolute joint" annotation(Placement(transformation(origin = { -80,100}, extent = {{ -8, -8},{8,8}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_ib "Coordinate system at origin of frame_b fixed at connecting rod of middle and right revolute joint" annotation(Placement(transformation(origin = {80,100}, extent = {{ -8,8},{8, -8}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_im "Coordinate system at origin of revolute joint in the middle fixed at connecting rod of middle and right revolute joint" annotation(Placement(transformation(origin = {0,100}, extent = {{8, -8},{ -8,8}}, rotation = 270)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_a axis "1-dim. rotational flange that drives the right revolute joint at frame_b" annotation(Placement(transformation(extent = {{105,85},{95,75}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_b bearing "1-dim. rotational flange of the drive bearing of the right revolute joint at frame_b" annotation(Placement(transformation(extent = {{95,45},{105,35}}, rotation = 0)));
            parameter Boolean animation = true "= true, if animation shall be enabled";
            parameter Modelica.Mechanics.MultiBody.Types.Axis n_a = {0,0,1} "Axes of revolute joints resolved in frame_a (all axes are parallel to each other)" annotation(Evaluate = true);
            final parameter Real n_b[3](each final unit = "1", fixed = false) = {0,0,1} "Axis of revolute joint fixed and resolved in frame_b" annotation(Evaluate = true);
            parameter SI.Position rRod1_ia[3] = {1,0,0} "Vector from origin of frame_a to revolute joint in the middle, resolved in frame_ia" annotation(Evaluate = true);
            parameter SI.Position rRod2_ib[3] = { -1,0,0} "Vector from origin of frame_ib to revolute joint in the middle, resolved in frame_ib";
            parameter Cv.NonSIunits.Angle_deg phi_offset = 0 "Relative angle offset of revolute joint at frame_b (angle = phi(t) + from_deg(phi_offset))";
            parameter Cv.NonSIunits.Angle_deg phi_guess = 0 "Select the configuration such that at initial time |phi(t0) - from_deg(phi_guess)|is minimal";
            parameter SI.Distance cylinderLength = world.defaultJointLength "Length of cylinders representing the revolute joints" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of cylinders representing the revolute joints" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinders representing the revolute joints" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter rodDiameter = 1.1 * cylinderDiameter "Diameter of the two rods connecting the revolute joints" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color rodColor = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor "Color of the two rods connecting the revolute joint" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter Boolean checkTotalPower = false "= true, if total power flowing into this component shall be determined (must be zero)" annotation(Dialog(tab = "Advanced"));
            final parameter Real e_a[3](each final unit = "1") = Modelica.Math.Vectors.normalize(n_a) "Unit vector along axes of rotations, resolved in frame_a";
            final parameter Real e_ia[3](each final unit = "1") = jointUSR.e2_ia "Unit vector along axes of rotations, resolved in frame_ia";
            final parameter Real e_b[3](each final unit = "1") = jointUSR.revolute.e "Unit vector along axes of rotations, resolved in frame_b, frame_ib and frame_im";
            SI.Power totalPower = jointUSR.totalPower "Total power flowing into this element, if checkTotalPower=true (otherwise dummy)";
            annotation(Documentation(info = "<html>
<p>
This component consists of <b>3 revolute</b> joints with parallel
axes of rotation that are connected together by two rods, see the default
animation in the following figure (the axes vectors are not part of the
default animation):
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Joints/JointRRR.png\" ALT=\"model Joints.Assemblies.JointRRR\">
</p>
<p>
This joint aggregation introduces neither constraints nor state variables and
should therefore be used in kinematic loops whenever possible to
avoid non-linear systems of equations. It is only meaningful to
use this component in <b>planar loops</b>. Basically, the position
and orientation of the 3 revolute joints as well as of frame_ia, frame_ib, and
frame_im are calculated by solving analytically a non-linear equation,
given the position and orientation at frame_a and at frame_b.
</p>
<p>
Connector <b>frame_a</b> is the \"left\" side of the first revolute joint
whereas <b>frame_ia</b> is the \"right side of this revolute joint, fixed in rod 1.
Connector <b>frame_b</b> is the \"right\" side of the third revolute joint
whereas <b>frame_ib</b> is the \"left\" side of this revolute joint, fixed in rod 2.
Finally, connector <b>frame_im</b> is the connector at the \"right\" side
of the revolute joint in the middle, fixed in rod 2.
</p>
<p>
The easiest way to define the parameters of this joint is by moving the
MultiBody system in a <b>reference configuration</b> where <b>all frames</b>
of all components are <b>parallel</b> to each other (alternatively,
at least frame_a, frame_ia, frame_im, frame_ib, frame_b of the JointRRR joint
should be parallel to each other when defining an instance of this
component).
</p>
<p>
Basically, the JointRRR model consists internally of a universal -
spherical - revolute joint aggregation (= JointUSR). In a planar
loop this will behave as if 3 revolute joints with parallel axes
are connected by rigid rods.
</p>
</html> "), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}, initialScale = 0.2), graphics = {Rectangle(extent = {{ -90,90},{90, -90}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -140, -55},{140, -80}}, lineColor = {0,0,255}, textString = "%name"),Text(extent = {{36,114},{71,92}}, lineColor = {128,128,128}, textString = "ib"),Text(extent = {{ -126,115},{ -87,90}}, lineColor = {128,128,128}, textString = "ia"),Ellipse(extent = {{ -100,25},{ -50, -25}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -85,10},{ -65, -10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{50,25},{100, -25}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Ellipse(extent = {{65,10},{85, -10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -26,80},{24,30}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -10,66},{10,46}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{ -71,9},{ -24,45},{ -19,39},{ -66,3},{ -71,9}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{54,12},{5,47},{10,52},{59,18},{54,12}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{100, -4},{83, -4},{84,3},{100,3},{100, -4}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{80,24},{80,80},{80,80},{80,100}}, color = {95,95,95}, thickness = 0.5),Text(extent = {{ -128, -29},{136, -47}}, lineColor = {0,0,0}, textString = "n_a=%n_a"),Line(points = {{0,57},{0,86},{0,86},{0,100}}, color = {95,95,95}, thickness = 0.5),Text(extent = {{ -46,114},{ -7,91}}, lineColor = {128,128,128}, textString = "im"),Line(points = {{ -80,100},{ -80,8}}, color = {95,95,95}, thickness = 0.5),Line(points = {{80,80},{101,80}}, color = {95,95,95}, thickness = 0.5),Line(points = {{100,40},{93,40},{93,3}}, color = {95,95,95}, thickness = 0.5)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}, initialScale = 0.2), graphics));
            JointUSR jointUSR(animation = false, n1_a = n_a, n_b = n_b, phi_offset = phi_offset, rRod2_ib = rRod2_ib, showUniversalAxes = false, rRod1_ia = rRod1_ia, checkTotalPower = checkTotalPower, phi_guess = phi_guess) annotation(Placement(transformation(extent = {{ -30, -20},{10,20}}, rotation = 0)));
          protected 
            Visualizers.Advanced.Shape shape_rev1(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = e_a, widthDirection = {0,1,0}, r_shape =  -e_a * cylinderLength / 2, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
            Visualizers.Advanced.Shape shape_rev2(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = e_b, widthDirection = {0,1,0}, r_shape =  -e_b * cylinderLength / 2, r = frame_im.r_0, R = frame_im.R) if world.enableAnimation and animation;
            Visualizers.Advanced.Shape shape_rev3(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = e_b, widthDirection = {0,1,0}, r_shape =  -e_b * cylinderLength / 2, r = frame_b.r_0, R = frame_b.R) if world.enableAnimation and animation;
            Visualizers.Advanced.Shape shape_rod1(shapeType = "cylinder", color = rodColor, specularCoefficient = specularCoefficient, length = Modelica.Math.Vectors.length(rRod1_ia), width = rodDiameter, height = rodDiameter, lengthDirection = rRod1_ia, widthDirection = e_ia, r = frame_ia.r_0, R = frame_ia.R) if world.enableAnimation and animation;
            Visualizers.Advanced.Shape shape_rod2(shapeType = "cylinder", color = rodColor, specularCoefficient = specularCoefficient, length = Modelica.Math.Vectors.length(rRod2_ib), width = rodDiameter, height = rodDiameter, lengthDirection = rRod2_ib, widthDirection = e_b, r = frame_ib.r_0, R = frame_ib.R) if world.enableAnimation and animation;

          initial equation 
            n_b = Frames.resolve2(frame_b.R, Frames.resolve1(frame_a.R, n_a));

          equation 
            connect(jointUSR.frame_a,frame_a) annotation(Line(points = {{ -30,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5));
            connect(jointUSR.frame_b,frame_b) annotation(Line(points = {{10,0},{100,0}}, color = {95,95,95}, thickness = 0.5));
            connect(jointUSR.frame_ia,frame_ia) annotation(Line(points = {{ -26,20},{ -26,70},{ -80,70},{ -80,100}}, color = {95,95,95}, thickness = 0.5));
            connect(jointUSR.frame_im,frame_im) annotation(Line(points = {{ -10,20},{ -10,70},{0,70},{0,100}}, color = {95,95,95}, thickness = 0.5));
            connect(jointUSR.frame_ib,frame_ib) annotation(Line(points = {{6,20},{6,50},{80,50},{80,100}}, color = {95,95,95}, thickness = 0.5));
            connect(jointUSR.axis,axis) annotation(Line(points = {{10,16},{86,16},{86,80},{100,80}}, color = {0,0,0}));
            connect(jointUSR.bearing,bearing) annotation(Line(points = {{10,8},{94,8},{94,40},{100,40}}, color = {0,0,0}));
          end JointRRR;
          model JointRRP "Planar revolute - revolute - prismatic joint aggregation (no constraints, no potential states)"
            import SI = Modelica.SIunits;
            import Cv = Modelica.SIunits.Conversions;
            import Modelica.Mechanics.MultiBody.Types;
            extends Interfaces.PartialTwoFramesDoubleSize;
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_ia "Coordinate system at origin of frame_a fixed at connecting rod of revolute joints" annotation(Placement(transformation(origin = { -80,100}, extent = {{ -8, -8},{8,8}}, rotation = 90)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_ib "Coordinate system at origin of frame_b fixed at connecting rod of revolute and prismatic joint" annotation(Placement(transformation(origin = {80,100}, extent = {{ -8,8},{8, -8}}, rotation = 270)));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_im "Coordinate system at origin of revolute joint in the middle fixed at connecting rod of revolute and prismatic joint" annotation(Placement(transformation(origin = {0,100}, extent = {{8, -8},{ -8,8}}, rotation = 270)));
            Modelica.Mechanics.Translational.Interfaces.Flange_a axis "1-dim. translational flange that drives the prismatic joint" annotation(Placement(transformation(extent = {{95,75},{105,85}}, rotation = 0)));
            Modelica.Mechanics.Translational.Interfaces.Flange_b bearing "1-dim. translational flange of the drive bearing of the prismatic joint" annotation(Placement(transformation(extent = {{105,35},{95,45}}, rotation = 0)));
            parameter Boolean animation = true "= true, if animation shall be enabled";
            parameter Modelica.Mechanics.MultiBody.Types.Axis n_a = {0,0,1} "Axes of the two revolute joints resolved in frame_a (both axes are parallel to each other)" annotation(Evaluate = true);
            parameter Modelica.Mechanics.MultiBody.Types.Axis n_b = { -1,0,0} "Axis of prismatic joint fixed and resolved in frame_b (must be orthogonal to revolute joint axes)" annotation(Evaluate = true);
            parameter SI.Position rRod1_ia[3] = {1,0,0} "Vector from origin of frame_a to revolute joint in the middle, resolved in frame_ia" annotation(Evaluate = true);
            parameter SI.Position rRod2_ib[3] = { -1,0,0} "Vector from origin of frame_ib to revolute joint in the middle, resolved in frame_ib (frame_ib is parallel to frame_b)";
            parameter SI.Position s_offset = 0 "Relative distance offset of prismatic joint (distance between the prismatic joint frames = s(t) + s_offset)";
            parameter SI.Position s_guess = 0 "Select the configuration such that at initial time |s(t0)-s_guess|is minimal";
            parameter SI.Distance cylinderLength = world.defaultJointLength "Length of cylinders representing the revolute joints" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of cylinders representing the revolute joints" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinders representing the revolute joints" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter Types.Axis boxWidthDirection = {0,1,0} "Vector in width direction of prismatic joint, resolved in frame_b" annotation(Evaluate = true, Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance boxWidth = world.defaultJointWidth "Width of prismatic joint box" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance boxHeight = boxWidth "Height of prismatic joint box" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color boxColor = cylinderColor "Color of prismatic joint box" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Diameter rodDiameter = 1.1 * cylinderDiameter "Diameter of the two rods connecting the joints" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color rodColor = Modelica.Mechanics.MultiBody.Types.Defaults.RodColor "Color of the two rods connecting the joints" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter Boolean checkTotalPower = false "= true, if total power flowing into this component shall be determined (must be zero)" annotation(Dialog(tab = "Advanced"));
            final parameter Real e_a[3](each final unit = "1") = Modelica.Math.Vectors.normalize(n_a) "Unit vector along axes of rotations, resolved in frame_a";
            final parameter Real e_ia[3](each final unit = "1") = jointUSP.e2_ia "Unit vector along axes of rotations, resolved in frame_ia";
            final parameter Real e_im[3](each final unit = "1", each fixed = false) "Unit vector along axes of rotations, resolved in frame_im";
            final parameter Real e_b[3](each final unit = "1") = jointUSP.prismatic.e "Unit vector along axes of translation of the prismatic joint, resolved in frame_b and frame_ib";
            SI.Power totalPower = jointUSP.totalPower "Total power flowing into this element, if checkTotalPower=true (otherwise dummy)";
            annotation(Documentation(info = "<html>
<p>
This component consists of <b>2 revolute</b> joints with parallel
axes of rotation that and a <b>prismatic</b> joint with a translational
axis that is orthogonal to the revolute joint axes, see the default
animation in the following figure (the axes vectors are not part of the
default animation):
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Joints/JointRRP.png\" ALT=\"model Joints.Assemblies.JointRRP\">
</p>
<p>
This joint aggregation introduces neither constraints nor state variables and
should therefore be used in kinematic loops whenever possible to
avoid non-linear systems of equations. It is only meaningful to
use this component in <b>planar loops</b>. Basically, the position
and orientation of the 3 joints as well as of frame_ia, frame_ib, and
frame_im are calculated by solving analytically a non-linear equation,
given the position and orientation at frame_a and at frame_b.
</p>
<p>
Connector <b>frame_a</b> is the \"left\" side of the first revolute joint
whereas <b>frame_ia</b> is the \"right side of this revolute joint, fixed in rod 1.
Connector <b>frame_b</b> is the \"right\" side of the prismatic joint
whereas <b>frame_ib</b> is the \"left\" side of this prismatic joint, fixed in rod 2.
Finally, connector <b>frame_im</b> is the connector at the \"right\" side
of the revolute joint in the middle, fixed in rod 2. The frames
frame_b, frame_ib, frame_im are always parallel to each other.
</p>
<p>
The easiest way to define the parameters of this joint is by moving the
MultiBody system in a <b>reference configuration</b> where <b>all frames</b>
of all components are <b>parallel</b> to each other (alternatively,
at least frame_a, frame_ia, frame_im, frame_ib, frame_b of the JointRRP joint
should be parallel to each other when defining an instance of this
component).
</p>
<p>
Basically, the JointRRP model consists internally of a universal -
spherical - prismatic joint aggregation (= JointUSP). In a planar
loop this will behave as if 2 revolute joints with parallel axes
and 1 prismatic joint are connected by rigid rods.
</p>
</html> "), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}, initialScale = 0.2), graphics = {Rectangle(extent = {{ -90,90},{90, -90}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -139, -53},{141, -78}}, lineColor = {0,0,255}, textString = "%name"),Text(extent = {{26,124},{68,93}}, lineColor = {128,128,128}, textString = "ib"),Text(extent = {{ -134,128},{ -94,94}}, lineColor = {128,128,128}, textString = "ia"),Ellipse(extent = {{ -100,25},{ -50, -25}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -85,10},{ -65, -10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -26,80},{24,30}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -10,66},{10,46}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{ -71,9},{ -24,45},{ -19,39},{ -66,3},{ -71,9}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{54,5},{5,47},{8,53},{58,11},{54,5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -128, -29},{139, -47}}, lineColor = {0,0,0}, textString = "n_a=%n_a"),Line(points = {{0,57},{0,86},{0,86},{0,100}}, color = {95,95,95}, thickness = 0.5),Text(extent = {{ -55,126},{ -15,92}}, lineColor = {128,128,128}, textString = "im"),Line(points = {{ -80,100},{ -80,8}}, color = {95,95,95}, thickness = 0.5),Line(points = {{80,80},{101,80}}, color = {95,95,95}, thickness = 0.5),Line(points = {{100,40},{93,40},{93,3}}, color = {95,95,95}, thickness = 0.5),Rectangle(extent = {{80,15},{100,21}}, pattern = LinePattern.None, fillColor = {0,0,0}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{53,5},{80,11}}, pattern = LinePattern.None, fillColor = {0,0,0}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{53,5},{80, -15}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{80,15},{100, -21}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{80,100},{80,80},{57,11}}, color = {95,95,95}, thickness = 0.5)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}, initialScale = 0.2), graphics));
            JointUSP jointUSP(animation = false, showUniversalAxes = false, n1_a = n_a, n_b = n_b, s_offset = s_offset, s_guess = s_guess, rRod1_ia = rRod1_ia, rRod2_ib = rRod2_ib, checkTotalPower = checkTotalPower) annotation(Placement(transformation(extent = {{ -30, -20},{10,20}}, rotation = 0)));
          protected 
            Visualizers.Advanced.Shape shape_rev1(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = e_a, widthDirection = {0,1,0}, r_shape =  -e_a * cylinderLength / 2, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
            Visualizers.Advanced.Shape shape_rev2(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = e_im, widthDirection = {0,1,0}, r_shape =  -e_im * cylinderLength / 2, r = frame_im.r_0, R = frame_im.R) if world.enableAnimation and animation;
            Visualizers.Advanced.Shape shape_prism(shapeType = "box", color = boxColor, specularCoefficient = specularCoefficient, length = jointUSP.prismatic.distance, width = boxWidth, height = boxHeight, lengthDirection = e_b, widthDirection = e_im, r = frame_b.r_0, R = frame_b.R) if world.enableAnimation and animation;
            Visualizers.Advanced.Shape shape_rod1(shapeType = "cylinder", color = rodColor, specularCoefficient = specularCoefficient, length = Modelica.Math.Vectors.length(rRod1_ia), width = rodDiameter, height = rodDiameter, lengthDirection = rRod1_ia, widthDirection = e_ia, r = frame_ia.r_0, R = frame_ia.R) if world.enableAnimation and animation;
            Visualizers.Advanced.Shape shape_rod2(shapeType = "cylinder", color = rodColor, specularCoefficient = specularCoefficient, length = Modelica.Math.Vectors.length(rRod2_ib), width = rodDiameter, height = rodDiameter, lengthDirection = rRod2_ib, widthDirection = e_b, r = frame_ib.r_0, R = frame_ib.R) if world.enableAnimation and animation;

          initial equation 
            e_im = Frames.resolve2(frame_im.R, Frames.resolve1(frame_a.R, e_a));

          equation 
            connect(jointUSP.frame_a,frame_a) annotation(Line(points = {{ -30,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5));
            connect(jointUSP.frame_b,frame_b) annotation(Line(points = {{10,0},{100,0}}, color = {95,95,95}, thickness = 0.5));
            connect(jointUSP.frame_ia,frame_ia) annotation(Line(points = {{ -26,20},{ -26,70},{ -80,70},{ -80,100}}, color = {95,95,95}, thickness = 0.5));
            connect(jointUSP.frame_im,frame_im) annotation(Line(points = {{ -10,20},{ -10,70},{0,70},{0,100}}, color = {95,95,95}, thickness = 0.5));
            connect(jointUSP.frame_ib,frame_ib) annotation(Line(points = {{6,20},{6,50},{80,50},{80,100}}, color = {95,95,95}, thickness = 0.5));
            connect(jointUSP.axis,axis) annotation(Line(points = {{10,16},{86,16},{86,80},{100,80}}, color = {0,0,0}));
            connect(jointUSP.bearing,bearing) annotation(Line(points = {{10,8},{94,8},{94,40},{100,40}}, color = {0,0,0}));
          end JointRRP;
        end Assemblies;
        package Internal "Components used for analytic solution of kinematic loops (use only if you know what you are doing)"
          extends Modelica.Icons.Library;
          model RevoluteWithLengthConstraint "Revolute joint where the rotation angle is computed from a length constraint (1 degree-of-freedom, no potential state)"
            import SI = Modelica.SIunits;
            import Cv = Modelica.SIunits.Conversions;
            extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
            Modelica.Mechanics.Rotational.Interfaces.Flange_a axis "1-dim. rotational flange that drives the joint" annotation(Placement(transformation(extent = {{10,90},{ -10,110}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_b bearing "1-dim. rotational flange of the drive bearing" annotation(Placement(transformation(extent = {{ -50,90},{ -70,110}}, rotation = 0)));
            Modelica.Blocks.Interfaces.RealInput position_a[3](each final quantity = "Position", each final unit = "m") "Position vector from frame_a to frame_a side of length constraint, resolved in frame_a of revolute joint" annotation(Placement(transformation(extent = {{ -140, -80},{ -100, -40}}, rotation = 0)));
            Modelica.Blocks.Interfaces.RealInput position_b[3](each final quantity = "Position", each final unit = "m") "Position vector from frame_b to frame_b side of length constraint, resolved in frame_b of revolute joint" annotation(Placement(transformation(extent = {{140, -80},{100, -40}}, rotation = 0)));
            parameter Boolean animation = true "= true, if animation shall be enabled";
            parameter SI.Position lengthConstraint(start = 1) "Fixed length of length constraint";
            parameter Modelica.Mechanics.MultiBody.Types.Axis n = {0,0,1} "Axis of rotation resolved in frame_a (= same as in frame_b)" annotation(Evaluate = true);
            parameter Cv.NonSIunits.Angle_deg phi_offset = 0 "Relative angle offset (angle = phi + from_deg(phi_offset))";
            parameter Cv.NonSIunits.Angle_deg phi_guess = 0 "Select the configuration such that at initial time |phi - from_deg(phi_guess)|is minimal";
            parameter SI.Distance cylinderLength = world.defaultJointLength "Length of cylinder representing the joint axis" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance cylinderDiameter = world.defaultJointWidth "Diameter of cylinder representing the joint axis" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color cylinderColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of cylinder representing the joint axis" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            final parameter Boolean positiveBranch(fixed = false) "Based on phi_guess, selection of one of the two solutions of the non-linear constraint equation";
            final parameter Real e[3](each final unit = "1") = Modelica.Math.Vectors.normalize(n) "Unit vector in direction of rotation axis, resolved in frame_a";
            SI.Angle phi "Rotation angle of revolute joint";
            Frames.Orientation R_rel "Relative orientation object from frame_a to frame_b";
            SI.Angle angle "= phi + from_deg(phi_offset) (relative rotation angle between frame_a and frame_b)";
            SI.Torque tau "= axis.tau (driving torque in the axis)";
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -30,10},{10, -10}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -100, -60},{ -30,60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{30, -60},{100,60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Text(extent = {{ -139, -168},{137, -111}}, textString = "%name", lineColor = {0,0,255}),Rectangle(extent = {{ -100,60},{ -30, -60}}, lineColor = {0,0,0}),Rectangle(extent = {{30,60},{100, -60}}, lineColor = {0,0,0}),Text(extent = {{ -142, -108},{147, -69}}, lineColor = {0,0,0}, textString = "n=%n"),Line(points = {{ -60,60},{ -60,90}}, color = {0,0,0}),Line(points = {{ -20,70},{ -60,70}}, color = {0,0,0}),Line(points = {{ -20,80},{ -20,60}}, color = {0,0,0}),Line(points = {{20,80},{20,60}}, color = {0,0,0}),Line(points = {{20,70},{41,70}}, color = {0,0,0}),Polygon(points = {{ -9,30},{10,30},{30,50},{ -29,50},{ -9,30}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{10,30},{30,50},{30, -51},{10, -31},{10,30}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -10,90},{10,50}}, lineColor = {0,0,0}, fillPattern = FillPattern.VerticalCylinder, fillColor = {192,192,192})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100, -60},{ -30,60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -30,10},{10, -10}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{30, -60},{100,60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Line(points = {{ -60,60},{ -60,96}}, color = {0,0,0}),Line(points = {{ -20,70},{ -60,70}}, color = {0,0,0}),Line(points = {{ -20,80},{ -20,60}}, color = {0,0,0}),Line(points = {{20,80},{20,60}}, color = {0,0,0}),Line(points = {{20,70},{41,70}}, color = {0,0,0}),Polygon(points = {{ -9,30},{10,30},{30,50},{ -29,50},{ -9,30}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{10,30},{30,50},{30, -51},{10, -31},{10,30}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -10,50},{10,100}}, lineColor = {0,0,0}, fillPattern = FillPattern.VerticalCylinder, fillColor = {192,192,192})}), Documentation(info = "<html>
<p>
Joint where frame_b rotates around axis n which is fixed in frame_a.
The two frames coincide when \"phi + phi_offset = 0\", where
\"phi_offset\" is a parameter with a zero default
and \"phi\" is the rotation angle.
</p>
<p>
This variant of the revolute joint is designed to work together
with a length constraint in a kinematic loop. This means that the
angle of the revolute joint, phi, is computed such that the
length constraint is fulfilled.
</p>
<p>
<b>Usually, this joint should not be used by a user of the MultiBody
library. It is only provided to built-up the Modelica.Mechanics.MultiBody.Joints.Assemblies.JointXYZ
joints.</b>
</p>

<p>
In releases before version 3.0 of the Modelica Standard Library, it was possible
to activate the torque projection equation (= cut-torque projected to the rotation
axis must be identical to the drive torque of flange axis) via parameter
<b>axisTorqueBalance</b>. This is no longer possible, since otherwise this
model would not be \"balanced\" (= same number of unknowns as equations).
Instead, when using this model in version 3.0 and later versions,
the force in the length constraint component (Joints.SphericalSpherical or
Joints.UniversalSpherical) must be calculated such that the driving torque
in direction of the rotation
axis is (RC shall be the name of the instance of RevoluteWithLenghtConstraint):
</p>
<pre>
    0 = RC.axis.tau + RC.e*RC.frame_b.t;
</pre>
<p>
If this equation is used, usually the force in the length constraint
and the second derivative of the revolute angle will be part of a linear
algebraic system of equations. In some cases it is possible to solve
this system of equations locally, i.e., provide the rod force directly
as function of the revolute constraint torque. In any case, this projection
equation or an equivalent one has to be provided via variable \"constraintResidue\" in the \"Advanced\"
menu of \"Joints.SphericalSpherical\" or \"Joints.UniversalSpherical\".
</p>

</html>
"));
          protected 
            SI.Position r_a[3] = position_a "Position vector from frame_a to frame_a side of length constraint, resolved in frame_a of revolute joint";
            SI.Position r_b[3] = position_b "Position vector from frame_b to frame_b side of length constraint, resolved in frame_b of revolute joint";
            Real e_r_a "Projection of r_a on e";
            Real e_r_b "Projection of r_b on e";
            Real A "Coefficient A of equation: A*cos(phi) + B*sin(phi) + C = 0";
            Real B "Coefficient B of equation: A*cos(phi) + B*sin(phi) + C = 0";
            Real C "Coefficient C of equation: A*cos(phi) + B*sin(phi) + C = 0";
            Real k1 "Constant of quadratic equation";
            Real k2 "Constant of quadratic equation";
            Real k1a(start = 1);
            Real k1b;
            Real kcos_angle "= k1*cos(angle)";
            Real ksin_angle "= k1*sin(angle)";
            Visualizers.Advanced.Shape cylinder(shapeType = "cylinder", color = cylinderColor, specularCoefficient = specularCoefficient, length = cylinderLength, width = cylinderDiameter, height = cylinderDiameter, lengthDirection = e, widthDirection = {0,1,0}, r_shape =  -e * cylinderLength / 2, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
            function selectBranch "Determine branch which is closest to initial angle=0"
              import Modelica.Math.*;
              input SI.Length L "Length of length constraint";
              input Real e[3](each final unit = "1") "Unit vector along axis of rotation, resolved in frame_a (= same in frame_b)";
              input SI.Angle angle_guess "Select the configuration such that at initial time |angle-angle_guess|is minimal (angle=0: frame_a and frame_b coincide)";
              input SI.Position r_a[3] "Position vector from frame_a to frame_a side of length constraint, resolved in frame_a of revolute joint";
              input SI.Position r_b[3] "Position vector from frame_b to frame_b side of length constraint, resolved in frame_b of revolute joint";
              output Boolean positiveBranch "Branch of the initial solution";
            protected 
              Real e_r_a "Projection of r_a on e";
              Real e_r_b "Projection of r_b on e";
              Real A "Coefficient A of equation: A*cos(phi) + B*sin(phi) + C = 0";
              Real B "Coefficient B of equation: A*cos(phi) + B*sin(phi) + C = 0";
              Real C "Coefficient C of equation: A*cos(phi) + B*sin(phi) + C = 0";
              Real k1 "Constant of quadratic equation";
              Real k2 "Constant of quadratic equation";
              Real k1a;
              Real k1b;
              Real kcos1 "k1*cos(angle1)";
              Real ksin1 "k1*sin(angle1)";
              Real kcos2 "k2*cos(angle2)";
              Real ksin2 "k2*sin(angle2)";
              SI.Angle angle1 "solution 1 of nonlinear equation";
              SI.Angle angle2 "solution 2 of nonlinear equation";
            algorithm 
              e_r_a:=e * r_a;
              e_r_b:=e * r_b;
              A:= -2 * (r_b * r_a - e_r_b * e_r_a);
              B:=2 * r_b * cross(e, r_a);
              C:=r_a * r_a + r_b * r_b - L * L - 2 * e_r_b * e_r_a;
              k1:=A * A + B * B;
              k1a:=k1 - C * C;
              assert(k1a > 1e-10, "
Singular position of loop (either no or two analytic solutions;
the mechanism has lost one-degree-of freedom in this position).
Try first to use another Modelica.Mechanics.MultiBody.Joints.Assemblies.JointXXX component.
In most cases it is best that the joints outside of the JointXXX
component are revolute and NOT prismatic joints. If this also
lead to singular positions, it could be that this kinematic loop
cannot be solved analytically. In this case you have to build
up the loop with basic joints (NO aggregation JointXXX components)
and rely on dynamic state selection, i.e., during simulation
the states will be dynamically selected in such a way that in no
position a degree of freedom is lost.
");
              k1b:=max(k1a, 1e-12);
              k2:=sqrt(k1b);
              kcos1:= -A * C + B * k2;
              ksin1:= -B * C - A * k2;
              angle1:=atan2(ksin1, kcos1);
              kcos2:= -A * C - B * k2;
              ksin2:= -B * C + A * k2;
              angle2:=atan2(ksin2, kcos2);
              if abs(angle1 - angle_guess) <= abs(angle2 - angle_guess) then 
                            positiveBranch:=true;

              else               positiveBranch:=false;

              end if;
            end selectBranch;

          initial equation 
            positiveBranch = selectBranch(lengthConstraint, e, Cv.from_deg(phi_offset + phi_guess), r_a, r_b);

          equation 
            Connections.branch(frame_a.R, frame_b.R);
            axis.tau = tau;
            axis.phi = phi;
            bearing.phi = 0;
            angle = Cv.from_deg(phi_offset) + phi;
            frame_b.r_0 = frame_a.r_0;
            R_rel = Frames.planarRotation(e, angle, der(angle));
            frame_b.R = Frames.absoluteRotation(frame_a.R, R_rel);
            zeros(3) = frame_a.f + Frames.resolve1(R_rel, frame_b.f);
            zeros(3) = frame_a.t + Frames.resolve1(R_rel, frame_b.t);
            e_r_a = e * r_a;
            e_r_b = e * r_b;
            A =  -2 * (r_b * r_a - e_r_b * e_r_a);
            B = 2 * r_b * cross(e, r_a);
            C = r_a * r_a + r_b * r_b - lengthConstraint * lengthConstraint - 2 * e_r_b * e_r_a;
            k1 = A * A + B * B;
            k1a = k1 - C * C;
            assert(k1a > 1e-10, "
Singular position of loop (either no or two analytic solutions;
the mechanism has lost one-degree-of freedom in this position).
Try first to use another Modelica.Mechanics.MultiBody.Joints.Assemblies.JointXXX component.
In most cases it is best that the joints outside of the JointXXX
component are revolute and NOT prismatic joints. If this also
lead to singular positions, it could be that this kinematic loop
cannot be solved analytically. In this case you have to build
up the loop with basic joints (NO aggregation JointXXX components)
and rely on dynamic state selection, i.e., during simulation
the states will be dynamically selected in such a way that in no
position a degree of freedom is lost.
");
            k1b = Frames.Internal.maxWithoutEvent(k1a, 1e-12);
            k2 = sqrt(k1b);
            kcos_angle =  -A * C + (if positiveBranch then B else  -B) * k2;
            ksin_angle =  -B * C + (if positiveBranch then  -A else A) * k2;
            angle = Modelica.Math.atan2(ksin_angle, kcos_angle);
          end RevoluteWithLengthConstraint;
          model PrismaticWithLengthConstraint "Prismatic joint where the translational distance is computed from a length constraint (1 degree-of-freedom, no potential state)"
            import SI = Modelica.SIunits;
            import Cv = Modelica.SIunits.Conversions;
            extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
            Modelica.Mechanics.Translational.Interfaces.Flange_a axis "1-dim. translational flange that drives the joint" annotation(Placement(transformation(extent = {{70,80},{90,60}}, rotation = 0)));
            Modelica.Mechanics.Translational.Interfaces.Flange_b bearing "1-dim. translational flange of the drive bearing" annotation(Placement(transformation(extent = {{ -30,80},{ -50,60}}, rotation = 0)));
            Modelica.Blocks.Interfaces.RealInput position_a[3] "Position vector from frame_a to frame_a side of length constraint, resolved in frame_a of revolute joint" annotation(Placement(transformation(extent = {{ -140, -80},{ -100, -40}}, rotation = 0)));
            Modelica.Blocks.Interfaces.RealInput position_b[3] "Position vector from frame_b to frame_b side of length constraint, resolved in frame_b of revolute joint" annotation(Placement(transformation(extent = {{140, -80},{100, -40}}, rotation = 0)));
            parameter Boolean animation = true "= true, if animation shall be enabled";
            parameter SI.Position length(start = 1) "Fixed length of length constraint";
            parameter Modelica.Mechanics.MultiBody.Types.Axis n = {1,0,0} "Axis of translation resolved in frame_a (= same as in frame_b)" annotation(Evaluate = true);
            parameter SI.Position s_offset = 0 "Relative distance offset (distance between frame_a and frame_b = s(t) + s_offset)";
            parameter SI.Position s_guess = 0 "Select the configuration such that at initial time |s(t0)-s_guess|is minimal";
            parameter Types.Axis boxWidthDirection = {0,1,0} "Vector in width direction of box, resolved in frame_a" annotation(Evaluate = true, Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance boxWidth = world.defaultJointWidth "Width of prismatic joint box" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            parameter SI.Distance boxHeight = boxWidth "Height of prismatic joint box" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.Color boxColor = Modelica.Mechanics.MultiBody.Types.Defaults.JointColor "Color of prismatic joint box" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
            final parameter Boolean positiveBranch(fixed = false) "Selection of one of the two solutions of the non-linear constraint equation";
            final parameter Real e[3](each final unit = "1") = Modelica.Math.Vectors.normalize(n) "Unit vector in direction of translation axis, resolved in frame_a";
            SI.Position s "Relative distance between frame_a and frame_b along axis n = s + s_offset)";
            SI.Position distance "Relative distance between frame_a and frame_b along axis n";
            SI.Position r_rel_a[3] "Position vector from frame_a to frame_b resolved in frame_a";
            SI.Force f "= axis.f (driving force in the axis)";
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -30, -40},{100,30}}, pattern = LinePattern.None, fillColor = {192,192,192}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{ -30,40},{100, -40}}, lineColor = {0,0,0}),Rectangle(extent = {{ -100, -60},{ -30,50}}, pattern = LinePattern.None, fillColor = {192,192,192}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{ -100,50},{ -30,60}}, pattern = LinePattern.None, fillColor = {0,0,0}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{ -30,30},{100,40}}, pattern = LinePattern.None, fillColor = {0,0,0}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -136, -170},{140, -113}}, textString = "%name", lineColor = {0,0,255}),Rectangle(extent = {{ -100,60},{ -30, -60}}, lineColor = {0,0,0}),Line(points = {{100, -40},{100, -60}}, color = {0,0,255}),Rectangle(extent = {{100,40},{90,80}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -136, -116},{153, -77}}, lineColor = {0,0,0}, textString = "n=%n")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -30, -50},{ -30,50}}, color = {0,0,0}),Line(points = {{0, -67},{90, -67}}, color = {128,128,128}),Text(extent = {{31, -68},{68, -81}}, lineColor = {128,128,128}, textString = "s"),Line(points = {{ -100, -67},{0, -67}}, color = {128,128,128}),Polygon(points = {{ -39, -64},{ -29, -67},{ -39, -70},{ -39, -64}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{ -77, -70},{ -43, -85}}, lineColor = {128,128,128}, textString = "s_offset"),Line(points = {{ -100, -71},{ -100, -51}}, color = {128,128,128}),Line(points = {{ -30, -73},{ -30, -33}}, color = {128,128,128}),Line(points = {{100, -70},{100, -30}}, color = {128,128,128}),Polygon(points = {{90, -64},{100, -67},{90, -70},{90, -64}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -100,50},{ -30,60}}, pattern = LinePattern.None, fillColor = {0,0,0}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{ -100, -60},{ -30,50}}, pattern = LinePattern.None, fillColor = {192,192,192}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{ -30,40},{100, -40}}, lineColor = {0,0,0}),Rectangle(extent = {{ -30, -40},{100,30}}, pattern = LinePattern.None, fillColor = {192,192,192}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{ -30,30},{100,40}}, pattern = LinePattern.None, fillColor = {0,0,0}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{ -100,60},{ -30, -60}}, lineColor = {0,0,0}),Line(points = {{100, -40},{100, -60}}, color = {0,0,255}),Text(extent = {{42,91},{57,76}}, textString = "f", lineColor = {0,0,255}),Line(points = {{40,75},{70,75}}, color = {0,0,255}),Polygon(points = {{ -21,78},{ -31,75},{ -21,72},{ -21,78}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Line(points = {{ -8,75},{ -31,75}}, color = {0,0,255}),Text(extent = {{ -21,90},{ -6,75}}, textString = "f", lineColor = {0,0,255}),Polygon(points = {{60,78},{70,75},{60,72},{60,78}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Line(points = {{ -30,64},{70,64}}, color = {128,128,128}),Polygon(points = {{60,67},{70,64},{60,61},{60,67}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{0,63},{37,50}}, lineColor = {128,128,128}, textString = "s"),Rectangle(extent = {{100,40},{90,80}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
<p>
Joint where frame_b is translated along axis n which is fixed in frame_a.
The two frames coincide when \"s + s_offset = 0\", where
\"s_offset\" is a parameter with a zero default
and \"s\" is the relative distance.
</p>
<p>
This variant of the prismatic joint is designed to work together
with a length constraint in a kinematic loop. This means that the
relative distance \"s\" of the joint is computed such that the
length constraint is fulfilled.
</p>
<p>
<b>Usually, this joint should not be used by a user of the MultiBody
library. It is only provided to built-up the Modelica.Mechanics.MultiBody.Joints.Assemblies.JointXYZ
joints.</b>
</p>

<p>
In releases before version 3.0 of the Modelica Standard Library, it was possible
to activate the force projection equation (= cut-force projected to the translation
axis must be identical to the driving force of flange axis) via parameter
<b>axisForceBalance</b>. This is no longer possible, since otherwise this
model would not be \"balanced\" (= same number of unknowns as equations).
Instead, when using this model in version 3.0 and later versions,
the force in the length constraint component (Joints.SphericalSpherical or
Joints.UniversalSpherical) must be calculated such that the driving force
in direction of the translation
axis is (RC shall be the name of the instance of PrismaticWithLenghtConstraint):
</p>
<pre>
    0 = RC.axis.f + RC.e*RC.frame_b.f;
</pre>
<p>
If this equation is used, usually the force in the length constraint
and the second derivative of the prismatic distance will be part of a linear
algebraic system of equations. In some cases it is possible to solve
this system of equations locally, i.e., provide the rod force directly
as function of the prismatic constraint force. In any case, this projection
equation or an equivalent one has to be provided via variable \"constraintResidue\" in the \"Advanced\"
menu of \"Joints.SphericalSpherical\" or \"Joints.UniversalSpherical\".
</p>

</HTML>
"));
          protected 
            SI.Position r_a[3] = position_a "Position vector from frame_a to frame_a side of length constraint, resolved in frame_a of revolute joint";
            SI.Position r_b[3] = position_b "Position vector from frame_b to frame_b side of length constraint, resolved in frame_b of revolute joint";
            Modelica.SIunits.Position rbra[3] "= rb - ra";
            Real B "Coefficient B of equation: s*s + B*s + C = 0";
            Real C "Coefficient C of equation: s*s + B*s + C = 0";
            Real k1 "Constant of quadratic equation solution";
            Real k2 "Constant of quadratic equation solution";
            Real k1a(start = 1);
            Real k1b;
            Visualizers.Advanced.Shape box(shapeType = "box", color = boxColor, specularCoefficient = specularCoefficient, length = if noEvent(abs(s + s_offset) > 1e-06) then s + s_offset else 1e-06, width = boxWidth, height = boxHeight, lengthDirection = e, widthDirection = boxWidthDirection, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
            function selectBranch "Determine branch which is closest to initial angle=0"
              import Modelica.Math.*;
              input SI.Length L "Length of length constraint";
              input Real e[3](each final unit = "1") "Unit vector along axis of translation, resolved in frame_a (= same in frame_b)";
              input SI.Position d_guess "Select the configuration such that at initial time |d-d_guess|is minimal (d: distance between origin of frame_a and origin of frame_b)";
              input SI.Position r_a[3] "Position vector from frame_a to frame_a side of length constraint, resolved in frame_a of prismatic joint";
              input SI.Position r_b[3] "Position vector from frame_b to frame_b side of length constraint, resolved in frame_b of prismatic joint";
              output Boolean positiveBranch "Branch of the initial solution";
            protected 
              Modelica.SIunits.Position rbra[3] "= rb - ra";
              Real B "Coefficient B of equation: d*d + B*d + C = 0";
              Real C "Coefficient C of equation: d*d + B*d + C = 0";
              Real k1 "Constant of quadratic equation solution";
              Real k2 "Constant of quadratic equation solution";
              Real k1a;
              Real k1b;
              Real d1 "solution 1 of quadratic equation";
              Real d2 "solution 2 of quadratic equation";
            algorithm 
              rbra:=r_b - r_a;
              B:=2 * e * rbra;
              C:=rbra * rbra - L * L;
              k1:=B / 2;
              k1a:=k1 * k1 - C;
              assert(noEvent(k1a > 1e-10), "
Singular position of loop (either no or two analytic solutions;
the mechanism has lost one-degree-of freedom in this position).
Try first to use another Modelica.Mechanics.MultiBody.Joints.Assemblies.JointXXX component.
If this also lead to singular positions, it could be that this
kinematic loop cannot be solved analytically with a fixed state
selection. In this case you have to build up the loop with
basic joints (NO aggregation JointXXX components) and rely on
dynamic state selection, i.e., during simulation the states will
be dynamically selected in such a way that in no position a
degree of freedom is lost.
");
              k1b:=max(k1a, 1e-12);
              k2:=sqrt(k1b);
              d1:= -k1 + k2;
              d2:= -k1 - k2;
              if abs(d1 - d_guess) <= abs(d2 - d_guess) then 
                            positiveBranch:=true;

              else               positiveBranch:=false;

              end if;
            end selectBranch;

          initial equation 
            positiveBranch = selectBranch(length, e, s_offset + s_guess, r_a, r_b);

          equation 
            Connections.branch(frame_a.R, frame_b.R);
            axis.f = f;
            axis.s = s;
            bearing.s = 0;
            distance = s_offset + s;
            r_rel_a = e * distance;
            frame_b.r_0 = frame_a.r_0 + Frames.resolve1(frame_a.R, r_rel_a);
            frame_b.R = frame_a.R;
            zeros(3) = frame_a.f + frame_b.f;
            zeros(3) = frame_a.t + frame_b.t + cross(r_rel_a, frame_b.f);
            rbra = r_b - r_a;
            B = 2 * e * rbra;
            C = rbra * rbra - length * length;
            k1 = B / 2;
            k1a = k1 * k1 - C;
            assert(noEvent(k1a > 1e-10), "
Singular position of loop (either no or two analytic solutions;
the mechanism has lost one-degree-of freedom in this position).
Try first to use another Modelica.Mechanics.MultiBody.Joints.Assemblies.JointXXX component.
If this also lead to singular positions, it could be that this
kinematic loop cannot be solved analytically with a fixed state
selection. In this case you have to build up the loop with
basic joints (NO aggregation JointXXX components) and rely on
dynamic state selection, i.e., during simulation the states will
be dynamically selected in such a way that in no position a
degree of freedom is lost.
");
            k1b = Frames.Internal.maxWithoutEvent(k1a, 1e-12);
            k2 = sqrt(k1b);
            distance =  -k1 + (if positiveBranch then k2 else  -k2);
          end PrismaticWithLengthConstraint;
          annotation(Documentation(info = "<HTML>
<p>
The models in this package should not be used by the user.
They are designed to build up other models in the MultiBody library
and some of them cannot be used in an arbitrary way and require
particular knowledge how to set the options in the parameter menu.
Don't use the models of this package.
</p>
</HTML>"));
          model RollingConstraintVerticalWheel "Rolling constraint for wheel that is always perpendicular to x-y plane"
            import SI = Modelica.SIunits;
            import Modelica.Mechanics.MultiBody.Frames;
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -60},{100, -80}}, lineColor = {0,0,0}, fillColor = {175,175,175}, fillPattern = FillPattern.Solid),Text(extent = {{ -148, -86},{152, -126}}, lineColor = {0,0,255}, textString = "%name"),Line(points = {{0, -60},{0,4}}, color = {0,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot),Line(visible = lateralSlidingConstraint, points = {{ -98, -30},{ -16, -30}}, color = {0,0,0}, smooth = Smooth.None),Polygon(visible = lateralSlidingConstraint, points = {{ -40, -16},{ -40, -42},{ -6, -30},{ -40, -16}}, lineColor = {0,0,0}, smooth = Smooth.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
            Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a "Frame fixed in wheel center point. x-Axis: upwards, y-axis: along wheel axis" annotation(Placement(transformation(extent = {{ -16,4},{16,36}}), iconTransformation(extent = {{ -16,4},{16,36}})));
            parameter SI.Radius radius "Wheel radius";
            parameter Boolean lateralSlidingConstraint = true "= true, if lateral sliding constraint taken into account, = false if lateral force = 0 (needed to avoid overconstraining if two ideal rolling wheels are connect on one axis)" annotation(choices(__Dymola_checkBox = true), HideResult = true, Evaluate = true);
            SI.Force f_wheel_0[3] "Contact force acting on wheel, resolved in world frame";
            SI.Force f_lat "Contact force acting on wheel in lateral direction";
            SI.Force f_long "Contact force acting on wheel in longitudinal direction";
          protected 
            Real e_axis_0[3] "Unit vector along wheel axis, resolved in world frame";
            SI.Position rContact_0[3] "Distance vector from wheel center to contact point, resolved in world frame";
            Real e_n_0[3] "Unit vector in normal direction of road at contact point, resolved in world frame";
            Real e_lat_0[3] "Unit vector in lateral direction of wheel at contact point, resolved in world frame";
            Real e_long_0[3] "Unit vector in longitudinal direction of wheel at contact point, resolved in world frame";
            SI.Velocity v_0[3] "Velocity of wheel center, resolved in world frame";
            SI.AngularVelocity w_0[3] "Angular velocity of wheel, resolved in world frame";
            SI.Velocity vContact_0[3] "Velocity of wheel contact point, resolved in world frame";
            Real aux[3];

          equation 
            e_n_0 = {0,0,1};
            e_axis_0 = Frames.resolve1(frame_a.R, {0,1,0});
            aux = cross(e_n_0, e_axis_0);
            e_long_0 = aux / Modelica.Math.Vectors.length(aux);
            e_lat_0 = cross(e_long_0, e_n_0);
            rContact_0 = {0,0, -radius};
            v_0 = der(frame_a.r_0);
            w_0 = Frames.angularVelocity1(frame_a.R);
            vContact_0 = v_0 + cross(w_0, rContact_0);
            0 = vContact_0 * e_long_0;
            if lateralSlidingConstraint then
              0 = vContact_0 * e_lat_0;
              f_wheel_0 = f_lat * e_lat_0 + f_long * e_long_0;

            else
              0 = f_lat;
              f_wheel_0 = f_long * e_long_0;

            end if;
            zeros(3) = frame_a.f + Frames.resolve2(frame_a.R, f_wheel_0);
            zeros(3) = frame_a.t + Frames.resolve2(frame_a.R, cross(rContact_0, f_wheel_0));
          end RollingConstraintVerticalWheel;
        end Internal;
      end Joints;
      package Interfaces "Connectors and partial models for 3-dim. mechanical components"
        extends Modelica.Icons.Library;
        annotation(Documentation(info = "<html>
<p>
This package contains connectors and partial models (i.e. models
that are only used to build other models) of the MultiBody library.
</p>
</html>"));
        connector Frame "Coordinate system fixed to the component with one cut-force and cut-torque (no icon)"
          import SI = Modelica.SIunits;
          SI.Position r_0[3] "Position vector from world frame to the connector frame origin, resolved in world frame";
          Frames.Orientation R "Orientation object to rotate the world frame into the connector frame";
          flow SI.Force f[3] "Cut-force resolved in connector frame" annotation(unassignedMessage = "All Forces cannot be uniquely calculated.
The reason could be that the mechanism contains
a planar loop or that joints constrain the
same motion. For planar loops, use for one
revolute joint per loop the joint
Joints.RevolutePlanarLoopConstraint instead of
Joints.Revolute.");
          flow SI.Torque t[3] "Cut-torque resolved in connector frame";
          annotation(Documentation(info = "<html>
<p>
Basic definition of a coordinate system that is fixed to a mechanical
component. In the origin of the coordinate system the cut-force
and the cut-torque is acting. This component has no icon definition
and is only used by inheritance from frame connectors to define
different icons.
</p>
</html>"));
        end Frame;
        connector Frame_a "Coordinate system fixed to the component with one cut-force and cut-torque (filled rectangular icon)"
          extends Frame;
          annotation(defaultComponentName = "frame_a", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}, initialScale = 0.16), graphics = {Rectangle(extent = {{ -10,10},{10, -10}}, lineColor = {95,95,95}, lineThickness = 0.5),Rectangle(extent = {{ -30,100},{30, -100}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}, initialScale = 0.16), graphics = {Text(extent = {{ -140, -50},{140, -88}}, lineColor = {0,0,0}, textString = "%name"),Rectangle(extent = {{ -12,40},{12, -40}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>
Basic definition of a coordinate system that is fixed to a mechanical
component. In the origin of the coordinate system the cut-force
and the cut-torque is acting.
This component has a filled rectangular icon.
</p>
</html>"));
        end Frame_a;
        connector Frame_b "Coordinate system fixed to the component with one cut-force and cut-torque (non-filled rectangular icon)"
          extends Frame;
          annotation(defaultComponentName = "frame_b", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}, initialScale = 0.16), graphics = {Rectangle(extent = {{ -10,10},{10, -10}}, lineColor = {95,95,95}, lineThickness = 0.5),Rectangle(extent = {{ -30,100},{30, -100}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}, initialScale = 0.16), graphics = {Text(extent = {{ -140, -50},{140, -88}}, lineColor = {0,0,0}, textString = "%name"),Rectangle(extent = {{ -12,40},{12, -40}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>
Basic definition of a coordinate system that is fixed to a mechanical
component. In the origin of the coordinate system the cut-force
and the cut-torque is acting. This component has a non-filled rectangular icon.
</p>
</html>"));
        end Frame_b;
        connector Frame_resolve "Coordinate system fixed to the component used to express in which
coordinate system a vector is resolved (non-filled rectangular icon)"
          extends Frame;
          annotation(defaultComponentName = "frame_resolve", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}, initialScale = 0.16), graphics = {Rectangle(extent = {{ -10,10},{10, -10}}, lineColor = {95,95,95}, pattern = LinePattern.Dot),Rectangle(extent = {{ -30,100},{30, -100}}, lineColor = {95,95,95}, pattern = LinePattern.Dot, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}, initialScale = 0.16), graphics = {Text(extent = {{ -140, -50},{140, -88}}, lineColor = {0,0,0}, textString = "%name"),Rectangle(extent = {{ -12,40},{12, -40}}, lineColor = {95,95,95}, pattern = LinePattern.Dot, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>
Basic definition of a coordinate system that is fixed to a mechanical
component. In the origin of the coordinate system the cut-force
and the cut-torque is acting. This coordinate system is used to
express in which coordinate system a vector is resolved.
A component that uses a Frame_resolve connector has to set the
cut-force and cut-torque of this frame to zero. When connecting
from a Frame_resolve connector to another frame connector,
by default the connecting line has line style \"dotted\".
This component has a non-filled rectangular icon.
</p>
</html>"));
        end Frame_resolve;
        connector FlangeWithBearing "Connector consisting of 1-dim. rotational flange and its bearing frame"
          parameter Boolean includeBearingConnector = false "= true, if bearing frame connector is present, otherwise not present";
          Modelica.Mechanics.Rotational.Interfaces.Flange_a flange "1-dim. rotational flange";
          Modelica.Mechanics.MultiBody.Interfaces.Frame bearingFrame if includeBearingConnector "3-dim. frame in which the 1-dim. shaft is mounted";
          annotation(defaultComponentName = "flange", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -20,1},{20, -1}}, lineColor = {135,135,135}, lineThickness = 0.5),Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -100,25},{100, -24}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {175,175,175}),Line(points = {{ -80,60},{80,60}}, color = {0,0,0}),Line(points = {{ -80, -60},{80, -60}}, color = {0,0,0}),Line(points = {{0,100},{0,60}}, color = {0,0,0}),Line(points = {{0, -60},{0, -100}}, color = {0,0,0}),Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {135,135,135}),Rectangle(extent = {{ -100,25},{100, -24}}, lineColor = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -50, -40},{50, -40}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -50,40},{50,40}}, color = {0,0,0}, thickness = 0.5),Text(extent = {{ -158, -66},{158, -124}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "%name"),Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {255,255,255}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -60,15},{60, -15}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {175,175,175}),Line(points = {{0,60},{0,40}}, color = {0,0,0}),Line(points = {{0, -40},{0, -60}}, color = {0,0,0}),Line(points = {{ -50,40},{50,40}}, color = {0,0,0}),Line(points = {{ -50, -40},{50, -40}}, color = {0,0,0}),Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {135,135,135}),Rectangle(extent = {{ -60,15},{60, -15}}, lineColor = {0,0,0})}), Documentation(info = "<html>
<p>
This hierarchical connector models a 1-dim. rotational flange
connector and its optional bearing defined by a 3-dim. frame connector.
If a connection to the subconnectors should be clearly visible,
connect first an  instance of
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Interfaces.FlangeWithBearingAdaptor\">FlangeWithBearingAdaptor</a>
to the FlangeWithBearing connector.
</p>
</html>"));
        end FlangeWithBearing;
        model FlangeWithBearingAdaptor "Adaptor to allow direct connections to the sub-connectors of FlangeWithBearing"
          parameter Boolean includeBearingConnector = false "= true, if bearing frame connector is present, otherwise not present";
          Modelica.Mechanics.MultiBody.Interfaces.FlangeWithBearing flangeAndFrame(includeBearingConnector = includeBearingConnector) "Compound connector consisting of 1-dim. rotational flange and 3-dim. frame mounting" annotation(Placement(transformation(extent = {{ -130, -30},{ -70,30}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Interfaces.Flange_b flange "1-dim. rotational flange" annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          Frame_a frame if includeBearingConnector "3-dim. frame in which the 1-dim. shaft is mounted" annotation(Placement(transformation(origin = {0, -100}, extent = {{ -16, -16},{16,16}}, rotation = 90)));
          annotation(defaultComponentName = "adaptor", Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,30},{20, -100}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -100, -10},{ -100, -40},{0, -40},{0, -100}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -90,0},{0,0}}, color = {0,0,0}),Text(extent = {{ -216,88},{86,36}}, lineColor = {0,0,255}, textString = "%name")}), Documentation(info = "<html>
<p>
Adaptor object to make a more visible connection to the flange and frame
subconnectors of a
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Interfaces.FlangeWithBearing\">FlangeWithBearing</a>
connector.
</p>
</html>"));

        equation 
          connect(flange,flangeAndFrame.flange) annotation(Line(points = {{0,0},{ -100,0}}, color = {0,0,0}));
          connect(frame,flangeAndFrame.bearingFrame) annotation(Line(points = {{0, -100},{0, -40},{ -100, -40},{ -100,0}}, color = {0,0,0}, thickness = 0.5));
        end FlangeWithBearingAdaptor;
        partial model PartialTwoFrames "Base model for components providing two frame connectors + outer world + assert to guarantee that the component is connected"
          Interfaces.Frame_a frame_a "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
          Interfaces.Frame_b frame_b "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{84, -16},{116,16}}, rotation = 0)));
        protected 
          outer Modelica.Mechanics.MultiBody.World world;

        equation 
          assert(cardinality(frame_a) > 0, "Connector frame_a of component is not connected");
          assert(cardinality(frame_b) > 0, "Connector frame_b of component is not connected");
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -136, -25},{ -100, -50}}, lineColor = {128,128,128}, textString = "a"),Text(extent = {{100, -25},{136, -50}}, lineColor = {128,128,128}, textString = "b")}), Documentation(info = "<HTML>
<p>
This partial model provides two frame connectors, access to the world
object and an assert to check that both frame connectors are connected.
Therefore, inherit from this partial model if the two frame connectors are
needed and if the two frame connectors should be connected for a correct model.
</p>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
        end PartialTwoFrames;
        partial model PartialTwoFramesDoubleSize "Base model for components providing two frame connectors + outer world + assert to guarantee that the component is connected (default icon size is factor 2 larger as usual)"
          Interfaces.Frame_a frame_a "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{ -108, -8},{ -92,8}})));
          Interfaces.Frame_b frame_b "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{92, -8},{108,8}})));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, initialScale = 0.2)), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, initialScale = 0.2), graphics = {Text(extent = {{ -117, -13},{ -106, -23}}, lineColor = {128,128,128}, textString = "a"),Text(extent = {{110, -15},{122, -25}}, lineColor = {128,128,128}, textString = "b")}), Documentation(info = "<HTML>
<p>
This partial model provides two frame connectors, access to the world
object and an assert to check that both frame connectors are connected.
Therefore, inherit from this partial model if the two frame connectors are
needed and if the two frame connectors should be connected for a correct model.
</p>
<p>
When dragging \"PartialTwoFrames\", the default size is a factor of two
larger as usual. This partial model is used by the Joint.Assemblies
joint aggregation models.
</p>
</HTML>"));
        protected 
          outer Modelica.Mechanics.MultiBody.World world;

        equation 
          assert(cardinality(frame_a) > 0, "Connector frame_a of component is not connected");
          assert(cardinality(frame_b) > 0, "Connector frame_b of component is not connected");
        end PartialTwoFramesDoubleSize;
        partial model PartialOneFrame_a "Base model for components providing one frame_a connector + outer world + assert to guarantee that the component is connected"
          Interfaces.Frame_a frame_a "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
        protected 
          outer Modelica.Mechanics.MultiBody.World world;

        equation 
          assert(cardinality(frame_a) > 0, "Connector frame_a of component is not connected");
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
This partial model provides one frame_a connector, access to the world
object and an assert to check that the frame_a connector is connected.
Therefore, inherit from this partial model if the frame_a connector is
needed and if this connector should be connected for a correct model.
</p>
</html>"));
        end PartialOneFrame_a;
        partial model PartialOneFrame_b "Base model for components providing one frame_b connector + outer world + assert to guarantee that the component is connected"
          Interfaces.Frame_b frame_b "Coordinate system fixed to the component with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{84, -16},{116,16}}, rotation = 0)));
        protected 
          outer Modelica.Mechanics.MultiBody.World world;

        equation 
          assert(cardinality(frame_b) > 0, "Connector frame_b of component is not connected");
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{94, -20},{130, -45}}, lineColor = {128,128,128}, textString = "b")}), Documentation(info = "<HTML>
<p>
This partial model provides one frame_b connector, access to the world
object and an assert to check that the frame_b connector is connected.
Therefore, inherit from this partial model if the frame_b connector is
needed and if this connector should be connected for a correct model.
</p>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
        end PartialOneFrame_b;
        partial model PartialElementaryJoint "Base model for elementary joints (has two frames + outer world + assert to guarantee that the joint is connected)"
          Interfaces.Frame_a frame_a "Coordinate system fixed to the joint with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
          Interfaces.Frame_b frame_b "Coordinate system fixed to the joint with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{84, -16},{116,16}}, rotation = 0)));
        protected 
          outer Modelica.Mechanics.MultiBody.World world;

        equation 
          Connections.branch(frame_a.R, frame_b.R);
          assert(cardinality(frame_a) > 0, "Connector frame_a of joint object is not connected");
          assert(cardinality(frame_b) > 0, "Connector frame_b of joint object is not connected");
          annotation(Documentation(info = "<HTML>
<p>
All <b>elementary joints</b> should inherit from this base model, i.e.,
joints that are directly defined by equations, provided they compute
either the rotation object of frame_b from the rotation object of frame_a
and from relative quantities (or vice versa), or there is a constraint
equation between the rotation objects of the two frames.
In other cases, a joint object should inherit from
<b>Interfaces.PartialTwoFrames</b> (e.g., joint Spherical, because there
is no constraint between the rotation objects of frame_a and frame_b
or joint Cylindrical because it is not an elementary joint).
</p>
<p>
This partial model provides two frame connectors, a \"Connections.branch\"
between frame_a and frame_b, access to the world
object and an assert to check that both frame connectors are connected.
</p>
</HTML>
 "), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
        end PartialElementaryJoint;
        partial model PartialForce "Base model for force elements (provide frame_b.f and frame_b.t in subclasses)"
          import SI = Modelica.SIunits;
          Interfaces.Frame_a frame_a "Coordinate system fixed to the joint with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
          Interfaces.Frame_b frame_b "Coordinate system fixed to the joint with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{84, -16},{116,16}}, rotation = 0)));
          SI.Position r_rel_b[3] "Position vector from origin of frame_a to origin of frame_b, resolved in frame_b";
        protected 
          outer Modelica.Mechanics.MultiBody.World world;

        equation 
          assert(cardinality(frame_a) > 0, "Connector frame_a of force object is not connected");
          assert(cardinality(frame_b) > 0, "Connector frame_b of force object is not connected");
          r_rel_b = Frames.resolve2(frame_b.R, frame_b.r_0 - frame_a.r_0);
          zeros(3) = frame_a.f + Frames.resolveRelative(frame_b.f, frame_b.R, frame_a.R);
          zeros(3) = frame_a.t + Frames.resolveRelative(frame_b.t + cross(r_rel_b, frame_b.f), frame_b.R, frame_a.R);
          annotation(Documentation(info = "<HTML>
<p>
All <b>3-dimensional force</b> and <b>torque elements</b>
should be based on this superclass.
This model defines frame_a and frame_b, computes the relative
translation and rotation between the two frames and calculates
the cut-force and cut-torque at frame_a by a force and torque
balance from the cut-force and cut-torque at frame_b.
As a result, in a subclass, only the relationship between
the cut-force and cut-torque at frame_b has to be defined as
a function of the following relative quantities:
</p>
<pre>
  r_rel_b[3]: Position vector from origin of frame_a to origin
              of frame_b, resolved in frame_b
  R_rel     : Relative orientation object to rotate from frame_a to frame_b
</pre>
<p>
Assume that force f = {100,0,0} should be applied on the body
to which this force element is attached at frame_b, then
the definition should be:
</p>
<pre>
   <b>model</b> Constant_x_Force
      extends Modelica.Mechanics.MultiBody.Interfaces.PartialForce;
   <b>equation</b>
      frame_b.f = {-100, 0, 0};
      frame_b.t = zeros(3);
   <b>end</b> Constant_x_Force;
</pre>
<p>
Note, that frame_b.f and frame_b.t are flow variables and therefore
the negative value of frame_b.f and frame_b.t is acting at the part
to which this force element is connected.
</p>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -136,42},{ -100,17}}, lineColor = {128,128,128}, textString = "a"),Text(extent = {{102,44},{138,19}}, lineColor = {128,128,128}, textString = "b")}));
        end PartialForce;
        partial model PartialLineForce "Base model for line force elements"
          import SI = Modelica.SIunits;
          parameter SI.Position s_small = 1e-06 " Prevent zero-division if relative distance s=0" annotation(Dialog(tab = "Advanced"));
          Interfaces.Frame_a frame_a "Coordinate system fixed to the force element with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
          Interfaces.Frame_b frame_b "Coordinate system fixed to the force element with one cut-force and cut-torque" annotation(Placement(transformation(extent = {{84, -16},{116,16}}, rotation = 0)));
          SI.Force f "Line force acting on frame_a and on frame_b (positive, if acting on frame_b and directed from frame_a to frame_b)";
          SI.Position s "(Guarded) distance between the origin of frame_a and the origin of frame_b (>= s_small))";
          Real e_a[3](each final unit = "1") "Unit vector on the line connecting the origin of frame_a with the origin of frame_b resolved in frame_a (directed from frame_a to frame_b)";
          Modelica.SIunits.Position r_rel_a[3] "Position vector from origin of frame_a to origin of frame_b, resolved in frame_a";
        protected 
          outer Modelica.Mechanics.MultiBody.World world;

        equation 
          Connections.potentialRoot(frame_a.R, 100);
          Connections.potentialRoot(frame_b.R, 100);
          assert(cardinality(frame_a) > 0, "Connector frame_a of line force object is not connected");
          assert(cardinality(frame_b) > 0, "Connector frame_b of line force object is not connected");
          r_rel_a = Frames.resolve2(frame_a.R, frame_b.r_0 - frame_a.r_0);
          s = noEvent(max(Modelica.Math.Vectors.length(r_rel_a), s_small));
          e_a = r_rel_a / s;
          frame_a.f =  -e_a * f;
          frame_b.f =  -Frames.resolve2(Frames.relativeRotation(frame_a.R, frame_b.R), frame_a.f);
          if Connections.isRoot(frame_a.R) then
            frame_a.R = Frames.nullRotation();

          else
            frame_a.t = zeros(3);

          end if;
          if Connections.isRoot(frame_b.R) then
            frame_b.R = Frames.nullRotation();

          else
            frame_b.t = zeros(3);

          end if;
          annotation(Documentation(info = "<HTML>
<p>
All <b>line force</b> elements should be based on this base model.
This model defines frame_a and frame_b, computes the relative
distance <b>s</b> and provides the force and torque
balance of the cut-forces and cut-torques at frame_a and
frame_b, respectively. In sub-models, only the line force <b>f</b>,
acting at frame_b on the line from frame_a to frame_b, as a function
of the relative distance <b>s</b> and its derivative <b>der</b>(<b>s</b>)
has to be defined. Example:
</p>
<pre>
   <b>model</b> Spring
      <b>parameter</b> Real c \"spring constant\",
      <b>parameter</b> Real s_unstretched \"unstretched spring length\";
      <b>extends</b> Modelica.Mechanics.MultiBody.Interfaces.PartialLineForce;
   <b>equation</b>
      f = c*(s-s_unstretched);
   <b>end</b> Spring;
</pre>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -136,44},{ -100,19}}, lineColor = {128,128,128}, textString = "a"),Text(extent = {{100,42},{136,17}}, lineColor = {128,128,128}, textString = "b")}));
        end PartialLineForce;
        partial model PartialAbsoluteSensor "Base model to measure an absolute frame variable"
          extends Modelica.Icons.RotationalSensor;
          parameter Integer n_out = 1 "Number of output signals";
          Interfaces.Frame_a frame_a "Coordinate system from which absolute quantities are provided as output signals" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput y[n_out] "Measured data as signal vector" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        protected 
          outer Modelica.Mechanics.MultiBody.World world;
          annotation(Documentation(info = "
<HTML>
<p>
This is the base class of a 3-dim. mechanics component with one frame and one
output port in order to measure an absolute quantity in the frame connector
and to provide the measured signal as output for further processing
with the blocks of package Modelica.Blocks.
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -100,0}}, color = {0,0,0}),Line(points = {{70,0},{100,0}}, color = {0,0,255}),Text(extent = {{ -132, -125},{131, -79}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          assert(cardinality(frame_a) > 0, "Connector frame_a of absolute sensor object is not connected");
        end PartialAbsoluteSensor;
        partial model PartialRelativeSensor "Base model to measure a relative variable between two frames"
          extends Modelica.Icons.RotationalSensor;
          parameter Integer n_out = 1 "Number of output signals";
          Interfaces.Frame_a frame_a "Coordinate system a" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
          Interfaces.Frame_b frame_b "Coordinate system b" annotation(Placement(transformation(extent = {{84, -16},{116,16}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput y[n_out] "Measured data as signal vector" annotation(Placement(transformation(origin = {0, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
        protected 
          outer Modelica.Mechanics.MultiBody.World world;
          annotation(Documentation(info = "
<HTML>
<p>
This is a base class for 3-dim. mechanical components with two frames
and one output port in order to measure relative quantities
between the two frames or the cut-forces/torques in the frame and
to provide the measured signals as output for further processing
with the blocks of package Modelica.Blocks.
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -101,0}}, color = {0,0,0}),Line(points = {{70,0},{100,0}}, color = {0,0,0}),Line(points = {{0, -100},{0, -70}}, color = {0,0,127}),Text(extent = {{ -132,76},{129,124}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -118,52},{ -82,27}}, lineColor = {128,128,128}, textString = "a"),Text(extent = {{85,53},{121,28}}, lineColor = {128,128,128}, textString = "b")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          assert(cardinality(frame_a) > 0, "Connector frame_a of relative sensor object is not connected");
          assert(cardinality(frame_b) > 0, "Connector frame_b of relative sensor object is not connected");
        end PartialRelativeSensor;
        partial model PartialVisualizer "Base model for visualizers (has a frame_a on the left side + outer world + assert to guarantee that the component is connected)"
          Interfaces.Frame_a frame_a "Coordinate system in which visualization data is resolved" annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}}, rotation = 0)));
        protected 
          outer Modelica.Mechanics.MultiBody.World world;

        equation 
          assert(cardinality(frame_a) > 0, "Connector frame_a of visualizer object is not connected");
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
This partial model provides one frame_a connector, access to the world
object and an assert to check that the frame_a connector is connected.
It is used by inheritance from all visualizer objects.
</p>
</html>"));
        end PartialVisualizer;
        model ZeroPosition "Set absolute position vector of frame_resolve to a zero vector and the orientation object to a null rotation"
          extends Modelica.Blocks.Interfaces.BlockIcon;
          Interfaces.Frame_resolve frame_resolve annotation(Placement(transformation(extent = {{ -116, -16},{ -84,16}})));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -74,24},{80, -20}}, lineColor = {0,0,0}, textString = "r = 0")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          Connections.root(frame_resolve.R);
          frame_resolve.R = Modelica.Mechanics.MultiBody.Frames.nullRotation();
          frame_resolve.r_0 = zeros(3);
        end ZeroPosition;
      end Interfaces;
      package Frames "Functions to transform rotational frame quantities"
        extends Modelica.Icons.Library;
        annotation(Documentation(info = "<HTML>
<p>
Package <b>Frames</b> contains type definitions and
functions to transform rotational frame quantities. The basic idea is to
hide the actual definition of an <b>orientation</b> in this package
by providing essentially type <b>Orientation</b> together with
<b>functions</b> operating on instances of this type.
</p>
<h4>Content</h4>
<p>In the table below an example is given for every function definition.
The used variables have the following declaration:
</p>
<pre>
   Frames.Orientation R, R1, R2, R_rel, R_inv;
   Real[3,3]   T, T_inv;
   Real[3]     v1, v2, w1, w2, n_x, n_y, n_z, e, e_x, res_ori, phi;
   Real[6]     res_equal;
   Real        L, angle;
</pre>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Function/type</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\"><b>Orientation R;</b></td>
      <td valign=\"top\">New type defining an orientation object that describes<br>
          the rotation of frame 1 into frame 2.
      </td>
  </tr>
  <tr><td valign=\"top\">res_ori = <b>orientationConstraint</b>(R);</td>
      <td valign=\"top\">Return the constraints between the variables of an orientation object<br>
      (shall be zero).</td>
  </tr>
  <tr><td valign=\"top\">w1 = <b>angularVelocity1</b>(R);</td>
      <td valign=\"top\">Return angular velocity resolved in frame 1 from
          orientation object R.
     </td>
  </tr>
  <tr><td valign=\"top\">w2 = <b>angularVelocity2</b>(R);</td>
      <td valign=\"top\">Return angular velocity resolved in frame 2 from
          orientation object R.
     </td>
  </tr>
  <tr><td valign=\"top\">v1 = <b>resolve1</b>(R,v2);</td>
      <td valign=\"top\">Transform vector v2 from frame 2 to frame 1.
      </td>
  </tr>
  <tr><td valign=\"top\">v2 = <b>resolve2</b>(R,v1);</td>
      <td valign=\"top\">Transform vector v1 from frame 1 to frame 2.
     </td>
  </tr>
  <tr><td valign=\"top\">v2 = <b>resolveRelative</b>(v1,R1,R2);</td>
      <td valign=\"top\">Transform vector v1 from frame 1 to frame 2
          using absolute orientation objects R1 of frame 1 and R2 of frame 2.
      </td>
  </tr>
  <tr><td valign=\"top\">D1 = <b>resolveDyade1</b>(R,D2);</td>
      <td valign=\"top\">Transform second order tensor D2 from frame 2 to frame 1.
      </td>
  </tr>
  <tr><td valign=\"top\">D2 = <b>resolveDyade2</b>(R,D1);</td>
      <td valign=\"top\">Transform second order tensor D1 from frame 1 to frame 2.
     </td>
  </tr>
  <tr><td valign=\"top\">R = <b>nullRotation</b>()</td>
      <td valign=\"top\">Return orientation object R that does not rotate a frame.
  </tr>
  <tr><td valign=\"top\">R_inv = <b>inverseRotation</b>(R);</td>
      <td valign=\"top\">Return inverse orientation object.
      </td>
  </tr>
  <tr><td valign=\"top\">R_rel = <b>relativeRotation</b>(R1,R2);</td>
      <td valign=\"top\">Return relative orientation object from two absolute
          orientation objects.
      </td>
  </tr>
  <tr><td valign=\"top\">R2 = <b>absoluteRotation</b>(R1,R_rel);</td>
      <td valign=\"top\">Return absolute orientation object from another
          absolute<br> and a relative orientation object.
      </td>
  </tr>
  <tr><td valign=\"top\">R = <b>planarRotation</b>(e, angle, der_angle);</td>
      <td valign=\"top\">Return orientation object of a planar rotation.
      </td>
  </tr>
  <tr><td valign=\"top\">angle = <b>planarRotationAngle</b>(e, v1, v2);</td>
      <td valign=\"top\">Return angle of a planar rotation, given the rotation axis<br>
        and the representations of a vector in frame 1 and frame 2.
      </td>
  </tr>
  <tr><td valign=\"top\">R = <b>axisRotation</b>(axis, angle, der_angle);</td>
      <td valign=\"top\">Return orientation object R to rotate around angle along axis of frame 1.
      </td>
  </tr>
  <tr><td valign=\"top\">R = <b>axesRotations</b>(sequence, angles, der_angles);</td>
      <td valign=\"top\">Return rotation object to rotate in sequence around 3 axes. Example:<br>
          R = axesRotations({1,2,3},{pi/2,pi/4,-pi}, zeros(3));
      </td>
  </tr>
  <tr><td valign=\"top\">angles = <b>axesRotationsAngles</b>(R, sequence);</td>
      <td valign=\"top\">Return the 3 angles to rotate in sequence around 3 axes to<br>
          construct the given orientation object.
      </td>
  </tr>
  <tr><td valign=\"top\">phi = <b>smallRotation</b>(R);</td>
      <td valign=\"top\">Return rotation angles phi valid for a small rotation R.
      </td>
  </tr>
  <tr><td valign=\"top\">R = <b>from_nxy</b>(n_x, n_y);</td>
      <td valign=\"top\">Return orientation object from n_x and n_y vectors.
      </td>
  </tr>
  <tr><td valign=\"top\">R = <b>from_nxz</b>(n_x, n_z);</td>
      <td valign=\"top\">Return orientation object from n_x and n_z vectors.
      </td>
  </tr>
  <tr><td valign=\"top\">R = <b>from_T</b>(T,w);</td>
      <td valign=\"top\">Return orientation object R from transformation matrix T and
          its angular velocity w.
      </td>
  </tr>
  <tr><td valign=\"top\">R = <b>from_T2</b>(T,der(T));</td>
      <td valign=\"top\">Return orientation object R from transformation matrix T and
          its derivative der(T).
      </td>
  </tr>
  <tr><td valign=\"top\">R = <b>from_T_inv</b>(T_inv,w);</td>
      <td valign=\"top\">Return orientation object R from inverse transformation matrix T_inv and
          its angular velocity w.
      </td>
  </tr>
  <tr><td valign=\"top\">R = <b>from_Q</b>(Q,w);</td>
      <td valign=\"top\">Return orientation object R from quaternion orientation object Q
          and its angular velocity w.
      </td>
  </tr>
  <tr><td valign=\"top\">T = <b>to_T</b>(R);</td>
      <td valign=\"top\">Return transformation matrix T from orientation object R.
  </tr>
  <tr><td valign=\"top\">T_inv = <b>to_T_inv</b>(R);</td>
      <td valign=\"top\">Return inverse transformation matrix T_inv from orientation object R.
      </td>
  </tr>
  <tr><td valign=\"top\">Q = <b>to_Q</b>(R);</td>
      <td valign=\"top\">Return quaternione orientation object Q from orientation object R.
      </td>
  </tr>
  <tr><td valign=\"top\">exy = <b>to_exy</b>(R);</td>
      <td valign=\"top\">Return [e_x, e_y] matrix of an orientation object R, <br>
          with e_x and e_y vectors of frame 2, resolved in frame 1.
  </tr>
  <tr><td valign=\"top\">L = <b>length</b>(n_x);</td>
      <td valign=\"top\">Return length L of a vector n_x.
      </td>
  </tr>
  <tr><td valign=\"top\">e_x = <b>normalize</b>(n_x);</td>
      <td valign=\"top\">Return normalized vector e_x of n_x such that length of e_x is one.
      </td>
  </tr>
  <tr><td valign=\"top\">e = <b>axis</b>(i);</td>
      <td valign=\"top\">Return unit vector e directed along axis i
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Frames.Quaternions\">Quaternions</a></td>
      <td valign=\"top\"><b>Package</b> with functions to transform rotational frame quantities based
          on quaternions (also called Euler parameters).
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Frames.TransformationMatrices\">TransformationMatrices</a></td>
      <td valign=\"top\"><b>Package</b> with functions to transform rotational frame quantities based
          on transformation matrices.
      </td>
  </tr>
</table>
</HTML>"));
        record Orientation "Orientation object defining rotation from a frame 1 into a frame 2"
          import SI = Modelica.SIunits;
          extends Modelica.Icons.Record;
          Real T[3,3] "Transformation matrix from world frame to local frame";
          SI.AngularVelocity w[3] "Absolute angular velocity of local frame, resolved in local frame";
          encapsulated function equalityConstraint "Return the constraint residues to express that two frames have the same orientation"
            import Modelica;
            import Modelica.Mechanics.MultiBody.Frames;
            extends Modelica.Icons.Function;
            input Frames.Orientation R1 "Orientation object to rotate frame 0 into frame 1";
            input Frames.Orientation R2 "Orientation object to rotate frame 0 into frame 2";
            output Real residue[3] "The rotation angles around x-, y-, and z-axis of frame 1 to rotate frame 1 into frame 2 for a small rotation (should be zero)";
            annotation(Inline = true);
          algorithm 
            residue:={Modelica.Math.atan2(cross(R1.T[1,:], R1.T[2,:]) * R2.T[2,:], R1.T[1,:] * R2.T[1,:]),Modelica.Math.atan2( -cross(R1.T[1,:], R1.T[2,:]) * R2.T[1,:], R1.T[2,:] * R2.T[2,:]),Modelica.Math.atan2(R1.T[2,:] * R2.T[1,:], R1.T[3,:] * R2.T[3,:])};
          end equalityConstraint;
          annotation(Documentation(info = "<html>
<p>
This object describes the <b>rotation</b> from a <b>frame 1</b> into a <b>frame 2</b>.
An instance of this type should never be directly accessed but
only with the access functions provided
in package Modelica.Mechanics.MultiBody.Frames. As a consequence, it is not necessary to know
the internal representation of this object as described in the next paragraphs.
</p>
<p>
\"Orientation\" is defined to be a record consisting of two
elements: \"Real T[3,3]\", the transformation matrix to rotate frame 1
into frame 2 and \"Real w[3]\", the angular velocity of frame 2 with
respect to frame 1, resolved in frame 2. Element \"T\"
has the following interpretation:
</p>
<pre>
   Orientation R;
   <b>R.T</b> = [<b>e</b><sub>x</sub>, <b>e</b><sub>y</sub>, <b>e</b><sub>z</sub>];
       e.g., <b>R.T</b> = [1,0,0; 0,1,0; 0,0,1]
</pre>
<p>
where <b>e</b><sub>x</sub>,<b>e</b><sub>y</sub>,<b>e</b><sub>z</sub>
are unit vectors in the direction of the x-axis, y-axis, and z-axis
of frame 1, resolved in frame 2, respectively. Therefore, if <b>v</b><sub>1</sub>
is vector <b>v</b> resolved in frame 1 and <b>v</b><sub>2</sub> is
vector <b>v</b> resolved in frame 2, the following relationship holds:
</p>
<pre>
    <b>v</b><sub>2</sub> = <b>R.T</b> * <b>v</b><sub>1</sub>
</pre>
</p>
The <b>inverse</b> orientation
<b>R_inv.T</b> = <b>R.T</b><sup>T</sup> describes the rotation
from frame 2 into frame 1.
</p>
<p>
Since the orientation is described by 9 variables, there are
6 constraints between these variables. These constraints
are defined in function <b>Frames.orientationConstraint</b>.
</p>
<p>
R.w is the angular velocity of frame 2 with respect to frame 1, resolved
in frame 2. Formally, R.w is defined as:<br>
<b>skew</b>(R.w) = R.T*<b>der</b>(transpose(R.T))
with
</p>
<pre>
             |   0   -w[3]  w[2] |
   <b>skew</b>(w) = |  w[3]   0   -w[1] |
             | -w[2]  w[1]     0 |
</pre>

</html>
"));
        end Orientation;
        function orientationConstraint "Return residues of orientation constraints (shall be zero)"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          output Real residue[6] "Residues of constraints between elements of orientation object (shall be zero)";
          annotation(Inline = true);
        algorithm 
          residue:={R.T[:,1] * R.T[:,1] - 1,R.T[:,2] * R.T[:,2] - 1,R.T[:,3] * R.T[:,3] - 1,R.T[:,1] * R.T[:,2],R.T[:,1] * R.T[:,3],R.T[:,2] * R.T[:,3]};
        end orientationConstraint;
        function angularVelocity1 "Return angular velocity resolved in frame 1 from orientation object"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          output Modelica.SIunits.AngularVelocity w[3] "Angular velocity of frame 2 with respect to frame 1 resolved in frame 1";
          annotation(Inline = true);
        algorithm 
          w:=resolve1(R, R.w);
        end angularVelocity1;
        function angularVelocity2 "Return angular velocity resolved in frame 2 from orientation object"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          output Modelica.SIunits.AngularVelocity w[3] "Angular velocity of frame 2 with respect to frame 1 resolved in frame 2";
          annotation(Inline = true);
        algorithm 
          w:=R.w;
        end angularVelocity2;
        function resolve1 "Transform vector from frame 2 to frame 1"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          input Real v2[3] "Vector in frame 2";
          output Real v1[3] "Vector in frame 1";
          annotation(derivative(noDerivative = R) = Internal.resolve1_der, __Dymola_InlineAfterIndexReduction = true);
        algorithm 
          v1:=transpose(R.T) * v2;
        end resolve1;
        function resolve2 "Transform vector from frame 1 to frame 2"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          input Real v1[3] "Vector in frame 1";
          output Real v2[3] "Vector in frame 2";
          annotation(derivative(noDerivative = R) = Internal.resolve2_der, __Dymola_InlineAfterIndexReduction = true);
        algorithm 
          v2:=R.T * v1;
        end resolve2;
        function resolveRelative "Transform vector from frame 1 to frame 2 using absolute orientation objects of frame 1 and of frame 2"
          extends Modelica.Icons.Function;
          input Real v1[3] "Vector in frame 1";
          input Orientation R1 "Orientation object to rotate frame 0 into frame 1";
          input Orientation R2 "Orientation object to rotate frame 0 into frame 2";
          output Real v2[3] "Vector in frame 2";
          annotation(derivative(noDerivative = R1, noDerivative = R2) = Internal.resolveRelative_der, __Dymola_InlineAfterIndexReduction = true);
        algorithm 
          v2:=resolve2(R2, resolve1(R1, v1));
        end resolveRelative;
        function resolveDyade1 "Transform second order tensor from frame 2 to frame 1"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          input Real D2[3,3] "Second order tensor resolved in frame 2";
          output Real D1[3,3] "Second order tensor resolved in frame 1";
          annotation(Inline = true);
        algorithm 
          D1:=transpose(R.T) * D2 * R.T;
        end resolveDyade1;
        function resolveDyade2 "Transform second order tensor from frame 1 to frame 2"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          input Real D1[3,3] "Second order tensor resolved in frame 1";
          output Real D2[3,3] "Second order tensor resolved in frame 2";
          annotation(Inline = true);
        algorithm 
          D2:=R.T * D1 * transpose(R.T);
        end resolveDyade2;
        function nullRotation "Return orientation object that does not rotate a frame"
          extends Modelica.Icons.Function;
          output Orientation R "Orientation object such that frame 1 and frame 2 are identical";
          annotation(Inline = true);
        algorithm 
          R:=Orientation(T = identity(3), w = zeros(3));
        end nullRotation;
        function inverseRotation "Return inverse orientation object"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          output Orientation R_inv "Orientation object to rotate frame 2 into frame 1";
          annotation(Inline = true);
        algorithm 
          R_inv:=Orientation(T = transpose(R.T), w =  -resolve1(R, R.w));
        end inverseRotation;
        function relativeRotation "Return relative orientation object"
          extends Modelica.Icons.Function;
          input Orientation R1 "Orientation object to rotate frame 0 into frame 1";
          input Orientation R2 "Orientation object to rotate frame 0 into frame 2";
          output Orientation R_rel "Orientation object to rotate frame 1 into frame 2";
          annotation(Inline = true);
        algorithm 
          R_rel:=Orientation(T = R2.T * transpose(R1.T), w = R2.w - resolve2(R2, resolve1(R1, R1.w)));
        end relativeRotation;
        function absoluteRotation "Return absolute orientation object from another absolute and a relative orientation object"
          extends Modelica.Icons.Function;
          input Orientation R1 "Orientation object to rotate frame 0 into frame 1";
          input Orientation R_rel "Orientation object to rotate frame 1 into frame 2";
          output Orientation R2 "Orientation object to rotate frame 0 into frame 2";
          annotation(Inline = true);
        algorithm 
          R2:=Orientation(T = R_rel.T * R1.T, w = resolve2(R_rel, R1.w) + R_rel.w);
        end absoluteRotation;
        function planarRotation "Return orientation object of a planar rotation"
          import Modelica.Math;
          extends Modelica.Icons.Function;
          input Real e[3](each final unit = "1") "Normalized axis of rotation (must have length=1)";
          input Modelica.SIunits.Angle angle "Rotation angle to rotate frame 1 into frame 2 along axis e";
          input Modelica.SIunits.AngularVelocity der_angle "= der(angle)";
          output Orientation R "Orientation object to rotate frame 1 into frame 2";
          annotation(Inline = true);
        algorithm 
          R:=Orientation(T = [e] * transpose([e]) + (identity(3) - [e] * transpose([e])) * Math.cos(angle) - skew(e) * Math.sin(angle), w = e * der_angle);
        end planarRotation;
        function planarRotationAngle "Return angle of a planar rotation, given the rotation axis and the representations of a vector in frame 1 and frame 2"
          extends Modelica.Icons.Function;
          input Real e[3](each final unit = "1") "Normalized axis of rotation to rotate frame 1 around e into frame 2 (must have length=1)";
          input Real v1[3] "A vector v resolved in frame 1 (shall not be parallel to e)";
          input Real v2[3] "Vector v resolved in frame 2, i.e., v2 = resolve2(planarRotation(e,angle),v1)";
          output Modelica.SIunits.Angle angle "Rotation angle to rotate frame 1 into frame 2 along axis e in the range: -pi <= angle <= pi";
          annotation(Inline = true, Documentation(info = "<HTML>
<p>
A call to this function of the form
</p>
<pre>
    Real[3]                e, v1, v2;
    Modelica.SIunits.Angle angle;
  <b>equation</b>
    angle = <b>planarRotationAngle</b>(e, v1, v2);
</pre>
<p>
computes the rotation angle \"<b>angle</b>\" of a planar
rotation along unit vector <b>e</b>, rotating frame 1 into frame 2, given
the coordinate representations of a vector \"v\" in frame 1 (<b>v1</b>)
and in frame 2 (<b>v2</b>). Therefore, the result of this function
fulfills the following equation:
</p>
<pre>
    v2 = <b>resolve2</b>(<b>planarRotation</b>(e,angle), v1)
</pre>
<p>
The rotation angle is returned in the range
</p>
<pre>
    -<font face=\"Symbol\">p</font> &lt;= angle &lt;= <font face=\"Symbol\">p</font>
</pre>
<p>
This function makes the following assumptions on the input arguments
</p>
<ul>
<li> Vector <b>e</b> has length 1, i.e., length(e) = 1</li>
<li> Vector \"v\" is not parallel to <b>e</b>, i.e.,
     length(cross(e,v1)) &ne; 0</li>
</ul>
<p>
The function does not check the above assumptions. If these
assumptions are violated, a wrong result will be returned
and/or a division by zero will occur.
</p>
</HTML>"));
        algorithm 
          angle:=Modelica.Math.atan2( -cross(e, v1) * v2, v1 * v2 - e * v1 * e * v2);
        end planarRotationAngle;
        function axisRotation "Return rotation object to rotate around an angle along one frame axis"
          import Modelica.Math.*;
          extends Modelica.Icons.Function;
          input Integer axis(min = 1, max = 3) "Rotate around 'axis' of frame 1";
          input Modelica.SIunits.Angle angle "Rotation angle to rotate frame 1 into frame 2 along 'axis' of frame 1";
          input Modelica.SIunits.AngularVelocity der_angle "= der(angle)";
          output Orientation R "Orientation object to rotate frame 1 into frame 2";
          annotation(Inline = true);
        algorithm 
          R:=Orientation(T = if axis == 1 then [1,0,0;0,cos(angle),sin(angle);0, -sin(angle),cos(angle)] else if axis == 2 then [cos(angle),0, -sin(angle);0,1,0;sin(angle),0,cos(angle)] else [cos(angle),sin(angle),0; -sin(angle),cos(angle),0;0,0,1], w = if axis == 1 then {der_angle,0,0} else if axis == 2 then {0,der_angle,0} else {0,0,der_angle});
        end axisRotation;
        function axesRotations "Return fixed rotation object to rotate in sequence around fixed angles along 3 axes"
          import TM = Modelica.Mechanics.MultiBody.Frames.TransformationMatrices;
          extends Modelica.Icons.Function;
          input Integer sequence[3](min = {1,1,1}, max = {3,3,3}) = {1,2,3} "Sequence of rotations from frame 1 to frame 2 along axis sequence[i]";
          input Modelica.SIunits.Angle angles[3] "Rotation angles around the axes defined in 'sequence'";
          input Modelica.SIunits.AngularVelocity der_angles[3] "= der(angles)";
          output Orientation R "Orientation object to rotate frame 1 into frame 2";
          annotation(Inline = true);
        algorithm 
          R:=Orientation(T = TM.axisRotation(sequence[3], angles[3]) * TM.axisRotation(sequence[2], angles[2]) * TM.axisRotation(sequence[1], angles[1]), w = Frames.axis(sequence[3]) * der_angles[3] + TM.resolve2(TM.axisRotation(sequence[3], angles[3]), Frames.axis(sequence[2]) * der_angles[2]) + TM.resolve2(TM.axisRotation(sequence[3], angles[3]) * TM.axisRotation(sequence[2], angles[2]), Frames.axis(sequence[1]) * der_angles[1]));
        end axesRotations;
        function axesRotationsAngles "Return the 3 angles to rotate in sequence around 3 axes to construct the given orientation object"
          import SI = Modelica.SIunits;
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          input Integer sequence[3](min = {1,1,1}, max = {3,3,3}) = {1,2,3} "Sequence of rotations from frame 1 to frame 2 along axis sequence[i]";
          input SI.Angle guessAngle1 = 0 "Select angles[1] such that |angles[1] - guessAngle1| is a minimum";
          output SI.Angle angles[3] "Rotation angles around the axes defined in 'sequence' such that R=Frames.axesRotation(sequence,angles); -pi < angles[i] <= pi";
          annotation(Documentation(info = "<HTML>
<p>
A call to this function of the form
</p>
<pre>
    Frames.Orientation     R;
    <b>parameter</b> Integer      sequence[3] = {1,2,3};
    Modelica.SIunits.Angle angles[3];
  <b>equation</b>
    angle = <b>axesRotationAngles</b>(R, sequence);
</pre>
<p>
computes the rotation angles \"<b>angles</b>[1:3]\" to rotate frame 1
into frame 2 along axes <b>sequence</b>[1:3], given the orientation
object <b>R</b> from frame 1 to frame 2. Therefore, the result of
this function fulfills the following equation:
</p>
<pre>
    R = <b>axesRotation</b>(sequence, angles)
</pre>
<p>
The rotation angles are returned in the range
</p>
<pre>
    -<font face=\"Symbol\">p</font> &lt;= angles[i] &lt;= <font face=\"Symbol\">p</font>
</pre>
<p>
There are <b>two solutions</b> for \"angles[1]\" in this range.
Via the third argument <b>guessAngle1</b> (default = 0) the
returned solution is selected such that |angles[1] - guessAngle1| is
minimal. The orientation object R may be in a singular configuration, i.e.,
there is an infinite number of angle values leading to the same R. The returned solution is
selected by setting angles[1] = guessAngle1. Then angles[2]
and angles[3] can be uniquely determined in the above range.
</p>
<p>
Note, that input argument <b>sequence</b> has the restriction that
only values 1,2,3 can be used and that sequence[1] &ne; sequence[2]
and sequence[2] &ne; sequence[3]. Often used values are:
</p>
<pre>
  sequence = <b>{1,2,3}</b>  // Cardan angle sequence
           = <b>{3,1,3}</b>  // Euler angle sequence
           = <b>{3,2,1}</b>  // Tait-Bryan angle sequence
</pre>
</HTML>"));
        protected 
          Real e1_1[3](each final unit = "1") "First rotation axis, resolved in frame 1";
          Real e2_1a[3](each final unit = "1") "Second rotation axis, resolved in frame 1a";
          Real e3_1[3](each final unit = "1") "Third rotation axis, resolved in frame 1";
          Real e3_2[3](each final unit = "1") "Third rotation axis, resolved in frame 2";
          Real A "Coefficient A in the equation A*cos(angles[1])+B*sin(angles[1]) = 0";
          Real B "Coefficient B in the equation A*cos(angles[1])+B*sin(angles[1]) = 0";
          SI.Angle angle_1a "Solution 1 for angles[1]";
          SI.Angle angle_1b "Solution 2 for angles[1]";
          TransformationMatrices.Orientation T_1a "Orientation object to rotate frame 1 into frame 1a";
        algorithm 
          assert(sequence[1] <> sequence[2] and sequence[2] <> sequence[3], "input argument 'sequence[1:3]' is not valid");
          e1_1:=if sequence[1] == 1 then {1,0,0} else if sequence[1] == 2 then {0,1,0} else {0,0,1};
          e2_1a:=if sequence[2] == 1 then {1,0,0} else if sequence[2] == 2 then {0,1,0} else {0,0,1};
          e3_1:=R.T[sequence[3],:];
          e3_2:=if sequence[3] == 1 then {1,0,0} else if sequence[3] == 2 then {0,1,0} else {0,0,1};
          A:=e2_1a * e3_1;
          B:=cross(e1_1, e2_1a) * e3_1;
          if abs(A) <= 1e-12 and abs(B) <= 1e-12 then 
                    angles[1]:=guessAngle1;

          else           angle_1a:=Modelica.Math.atan2(A,  -B);
          angle_1b:=Modelica.Math.atan2( -A, B);
          angles[1]:=if abs(angle_1a - guessAngle1) <= abs(angle_1b - guessAngle1) then angle_1a else angle_1b;

          end if;
          T_1a:=TransformationMatrices.planarRotation(e1_1, angles[1]);
          angles[2]:=planarRotationAngle(e2_1a, TransformationMatrices.resolve2(T_1a, e3_1), e3_2);
          angles[3]:=planarRotationAngle(e3_2, e2_1a, TransformationMatrices.resolve2(R.T, TransformationMatrices.resolve1(T_1a, e2_1a)));
        end axesRotationsAngles;
        function smallRotation "Return rotation angles valid for a small rotation and optionally residues that should be zero"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          input Boolean withResidues = false "= false/true, if 'angles'/'angles and residues' are returned in phi";
          output Modelica.SIunits.Angle phi[if withResidues then 6 else 3] "The rotation angles around x-, y-, and z-axis of frame 1 to rotate frame 1 into frame 2 for a small rotation + optionally 3 residues that should be zero";
          annotation(Inline = true);
        algorithm 
          phi:=if withResidues then {R.T[2,3], -R.T[1,3],R.T[1,2],R.T[1,1] - 1,R.T[2,2] - 1,R.T[1,1] * R.T[2,2] - R.T[2,1] * R.T[1,2] - 1} else {R.T[2,3], -R.T[1,3],R.T[1,2]};
        end smallRotation;
        function from_nxy "Return fixed orientation object from n_x and n_y vectors"
          extends Modelica.Icons.Function;
          input Real n_x[3](each final unit = "1") "Vector in direction of x-axis of frame 2, resolved in frame 1";
          input Real n_y[3](each final unit = "1") "Vector in direction of y-axis of frame 2, resolved in frame 1";
          output Orientation R "Orientation object to rotate frame 1 into frame 2";
          annotation(Documentation(info = "<html>
<p>
It is assumed that the two input vectors n_x and n_y are
resolved in frame 1 and are directed along the x and y axis
of frame 2 (i.e., n_x and n_y are orthogonal to each other)
The function returns the orientation object R to rotate from
frame 1 to frame 2.
</p>
<p>
The function is robust in the sense that it returns always
an orientation object R, even if n_y is not orthogonal to n_x.
This is performed in the following way:
</p>
<p>
If n_x and n_y are not orthogonal to each other, first a unit
vector e_y is determined that is orthogonal to n_x and is lying
in the plane spanned by n_x and n_y. If n_x and n_y are parallel
or nearly parallel to each other, a vector e_y is selected
arbitrarily such that e_x and e_y are orthogonal to each other.
</p>
</html>"));
        protected 
          Real abs_n_x = sqrt(n_x * n_x);
          Real e_x[3](each final unit = "1") = if abs_n_x < 1e-10 then {1,0,0} else n_x / abs_n_x;
          Real n_z_aux[3](each final unit = "1") = cross(e_x, n_y);
          Real n_y_aux[3](each final unit = "1") = if n_z_aux * n_z_aux > 1e-06 then n_y else if abs(e_x[1]) > 1e-06 then {0,1,0} else {1,0,0};
          Real e_z_aux[3](each final unit = "1") = cross(e_x, n_y_aux);
          Real e_z[3](each final unit = "1") = e_z_aux / sqrt(e_z_aux * e_z_aux);
        algorithm 
          R:=Orientation(T = {e_x,cross(e_z, e_x),e_z}, w = zeros(3));
        end from_nxy;
        function from_nxz "Return fixed orientation object from n_x and n_z vectors"
          extends Modelica.Icons.Function;
          input Real n_x[3](each final unit = "1") "Vector in direction of x-axis of frame 2, resolved in frame 1";
          input Real n_z[3](each final unit = "1") "Vector in direction of z-axis of frame 2, resolved in frame 1";
          output Orientation R "Orientation object to rotate frame 1 into frame 2";
          annotation(Documentation(info = "<html>
<p>
It is assumed that the two input vectors n_x and n_z are
resolved in frame 1 and are directed along the x and z axis
of frame 2 (i.e., n_x and n_z are orthogonal to each other)
The function returns the orientation object R to rotate from
frame 1 to frame 2.
</p>
<p>
The function is robust in the sense that it returns always
an orientation object R, even if n_z is not orthogonal to n_x.
This is performed in the following way:
</p>
<p>
If n_x and n_z are not orthogonal to each other, first a unit
vector e_z is determined that is orthogonal to n_x and is lying
in the plane spanned by n_x and n_z. If n_x and n_z are parallel
or nearly parallel to each other, a vector e_z is selected
arbitrarily such that n_x and e_z are orthogonal to each other.
</p>
</html>"));
        protected 
          Real abs_n_x = sqrt(n_x * n_x);
          Real e_x[3](each final unit = "1") = if abs_n_x < 1e-10 then {1,0,0} else n_x / abs_n_x;
          Real n_y_aux[3](each final unit = "1") = cross(n_z, e_x);
          Real n_z_aux[3](each final unit = "1") = if n_y_aux * n_y_aux > 1e-06 then n_z else if abs(e_x[1]) > 1e-06 then {0,0,1} else {1,0,0};
          Real e_y_aux[3](each final unit = "1") = cross(n_z_aux, e_x);
          Real e_y[3](each final unit = "1") = e_y_aux / sqrt(e_y_aux * e_y_aux);
        algorithm 
          R:=Orientation(T = {e_x,e_y,cross(e_x, e_y)}, w = zeros(3));
        end from_nxz;
        function from_T "Return orientation object R from transformation matrix T"
          extends Modelica.Icons.Function;
          input Real T[3,3] "Transformation matrix to transform vector from frame 1 to frame 2 (v2=T*v1)";
          input Modelica.SIunits.AngularVelocity w[3] "Angular velocity from frame 2 with respect to frame 1, resolved in frame 2 (skew(w)=T*der(transpose(T)))";
          output Orientation R "Orientation object to rotate frame 1 into frame 2";
          annotation(Inline = true);
        algorithm 
          R:=Orientation(T = T, w = w);
        end from_T;
        function from_T2 "Return orientation object R from transformation matrix T and its derivative der(T)"
          extends Modelica.Icons.Function;
          input Real T[3,3] "Transformation matrix to transform vector from frame 1 to frame 2 (v2=T*v1)";
          input Real der_T[3,3] "= der(T)";
          output Orientation R "Orientation object to rotate frame 1 into frame 2";
          annotation(Inline = true, Documentation(info = "<html>
<p>
Computes the orientation object from a transformation matrix T and
the derivative der(T) of the transformation matrix.
Usually, it is more efficient to use function \"from_T\" instead, where
the angular velocity has to be given as input argument. Only if this
is not possible or too difficult to compute, use function from_T2(..).
</p>
</html>"));
        algorithm 
          R:=Orientation(T = T, w = {T[3,:] * der_T[2,:], -T[3,:] * der_T[1,:],T[2,:] * der_T[1,:]});
        end from_T2;
        function from_T_inv "Return orientation object R from inverse transformation matrix T_inv"
          extends Modelica.Icons.Function;
          input Real T_inv[3,3] "Inverse transformation matrix to transform vector from frame 2 to frame 1 (v1=T_inv*v2)";
          input Modelica.SIunits.AngularVelocity w[3] "Angular velocity from frame 1 with respect to frame 2, resolved in frame 1 (skew(w)=T_inv*der(transpose(T_inv)))";
          output Orientation R "Orientation object to rotate frame 1 into frame 2";
          annotation(Inline = true);
        algorithm 
          R:=Orientation(T = transpose(T_inv), w =  -w);
        end from_T_inv;
        function from_Q "Return orientation object R from quaternion orientation object Q"
          extends Modelica.Icons.Function;
          input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
          input Modelica.SIunits.AngularVelocity w[3] "Angular velocity from frame 2 with respect to frame 1, resolved in frame 2";
          output Orientation R "Orientation object to rotate frame 1 into frame 2";
          annotation(Inline = true);
        algorithm 
          R:=Orientation([2 * (Q[1] * Q[1] + Q[4] * Q[4]) - 1,2 * (Q[1] * Q[2] + Q[3] * Q[4]),2 * (Q[1] * Q[3] - Q[2] * Q[4]);2 * (Q[2] * Q[1] - Q[3] * Q[4]),2 * (Q[2] * Q[2] + Q[4] * Q[4]) - 1,2 * (Q[2] * Q[3] + Q[1] * Q[4]);2 * (Q[3] * Q[1] + Q[2] * Q[4]),2 * (Q[3] * Q[2] - Q[1] * Q[4]),2 * (Q[3] * Q[3] + Q[4] * Q[4]) - 1], w = w);
        end from_Q;
        function to_T "Return transformation matrix T from orientation object R"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          output Real T[3,3] "Transformation matrix to transform vector from frame 1 to frame 2 (v2=T*v1)";
          annotation(Inline = true);
        algorithm 
          T:=R.T;
        end to_T;
        function to_T_inv "Return inverse transformation matrix T_inv from orientation object R"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          output Real T_inv[3,3] "Inverse transformation matrix to transform vector from frame 2 into frame 1 (v1=T_inv*v2)";
          annotation(Inline = true);
        algorithm 
          T_inv:=transpose(R.T);
        end to_T_inv;
        function to_Q "Return quaternion orientation object Q from orientation object R"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          input Quaternions.Orientation Q_guess = Quaternions.nullRotation() "Guess value for output Q (there are 2 solutions; the one closer to Q_guess is used";
          output Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
          annotation(Inline = true);
        algorithm 
          Q:=Quaternions.from_T(R.T, Q_guess);
        end to_Q;
        function to_vector "Map rotation object into vector"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          output Real vec[9] "Elements of R in one vector";
          annotation(Inline = true);
        algorithm 
          vec:={R.T[1,1],R.T[2,1],R.T[3,1],R.T[1,2],R.T[2,2],R.T[3,2],R.T[1,3],R.T[2,3],R.T[3,3]};
        end to_vector;
        function to_exy "Map rotation object into e_x and e_y vectors of frame 2, resolved in frame 1"
          extends Modelica.Icons.Function;
          input Orientation R "Orientation object to rotate frame 1 into frame 2";
          output Real exy[3,2] "= [e_x, e_y] where e_x and e_y are axes unit vectors of frame 2, resolved in frame 1";
          annotation(Inline = true);
        algorithm 
          exy:=[R.T[1,:],R.T[2,:]];
        end to_exy;
        function axis "Return unit vector for x-, y-, or z-axis"
          extends Modelica.Icons.Function;
          input Integer axis(min = 1, max = 3) "Axis vector to be returned";
          output Real e[3](each final unit = "1") "Unit axis vector";
          annotation(Inline = true);
        algorithm 
          e:=if axis == 1 then {1,0,0} else if axis == 2 then {0,1,0} else {0,0,1};
        end axis;
        package Quaternions "Functions to transform rotational frame quantities based on quaternions (also called Euler parameters)"
          extends Modelica.Icons.Library;
          annotation(Documentation(info = "<HTML>
<p>
Package <b>Frames.Quaternions</b> contains type definitions and
functions to transform rotational frame quantities with quaternions.
Functions of this package are currently only utilized in
MultiBody.Parts.Body components, when quaternions shall be used
as parts of the body states.
Some functions are also used in a new Modelica package for
B-Spline interpolation that is able to interpolate paths consisting of
position vectors and orientation objects.
</p>
<h4>Content</h4>
<p>In the table below an example is given for every function definition.
The used variables have the following declaration:
</p>
<pre>
   Quaternions.Orientation Q, Q1, Q2, Q_rel, Q_inv;
   Real[3,3]   T, T_inv;
   Real[3]     v1, v2, w1, w2, n_x, n_y, n_z, res_ori, phi;
   Real[6]     res_equal;
   Real        L, angle;
</pre>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Function/type</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\"><b>Orientation Q;</b></td>
      <td valign=\"top\">New type defining a quaternion object that describes<br>
          the rotation of frame 1 into frame 2.
      </td>
  </tr>
  <tr><td valign=\"top\"><b>der_Orientation</b> der_Q;</td>
      <td valign=\"top\">New type defining the first time derivative
         of Frames.Quaternions.Orientation.
      </td>
  </tr>
  <tr><td valign=\"top\">res_ori = <b>orientationConstraint</b>(Q);</td>
      <td valign=\"top\">Return the constraints between the variables of a quaternion object<br>
      (shall be zero).</td>
  </tr>
  <tr><td valign=\"top\">w1 = <b>angularVelocity1</b>(Q, der_Q);</td>
      <td valign=\"top\">Return angular velocity resolved in frame 1 from
          quaternion object Q<br> and its derivative der_Q.
     </td>
  </tr>
  <tr><td valign=\"top\">w2 = <b>angularVelocity2</b>(Q, der_Q);</td>
      <td valign=\"top\">Return angular velocity resolved in frame 2 from
          quaternion object Q<br> and its derivative der_Q.
     </td>
  </tr>
  <tr><td valign=\"top\">v1 = <b>resolve1</b>(Q,v2);</td>
      <td valign=\"top\">Transform vector v2 from frame 2 to frame 1.
      </td>
  </tr>
  <tr><td valign=\"top\">v2 = <b>resolve2</b>(Q,v1);</td>
      <td valign=\"top\">Transform vector v1 from frame 1 to frame 2.
     </td>
  </tr>
  <tr><td valign=\"top\">[v1,w1] = <b>multipleResolve1</b>(Q, [v2,w2]);</td>
      <td valign=\"top\">Transform several vectors from frame 2 to frame 1.
      </td>
  </tr>
  <tr><td valign=\"top\">[v2,w2] = <b>multipleResolve2</b>(Q, [v1,w1]);</td>
      <td valign=\"top\">Transform several vectors from frame 1 to frame 2.
      </td>
  </tr>
  <tr><td valign=\"top\">Q = <b>nullRotation</b>()</td>
      <td valign=\"top\">Return quaternion object R that does not rotate a frame.
  </tr>
  <tr><td valign=\"top\">Q_inv = <b>inverseRotation</b>(Q);</td>
      <td valign=\"top\">Return inverse quaternion object.
      </td>
  </tr>
  <tr><td valign=\"top\">Q_rel = <b>relativeRotation</b>(Q1,Q2);</td>
      <td valign=\"top\">Return relative quaternion object from two absolute
          quaternion objects.
      </td>
  </tr>
  <tr><td valign=\"top\">Q2 = <b>absoluteRotation</b>(Q1,Q_rel);</td>
      <td valign=\"top\">Return absolute quaternion object from another
          absolute<br> and a relative quaternion object.
      </td>
  </tr>
  <tr><td valign=\"top\">Q = <b>planarRotation</b>(e, angle);</td>
      <td valign=\"top\">Return quaternion object of a planar rotation.
      </td>
  </tr>
  <tr><td valign=\"top\">phi = <b>smallRotation</b>(Q);</td>
      <td valign=\"top\">Return rotation angles phi valid for a small rotation.
      </td>
  </tr>
  <tr><td valign=\"top\">Q = <b>from_T</b>(T);</td>
      <td valign=\"top\">Return quaternion object Q from transformation matrix T.
      </td>
  </tr>
  <tr><td valign=\"top\">Q = <b>from_T_inv</b>(T_inv);</td>
      <td valign=\"top\">Return quaternion object Q from inverse transformation matrix T_inv.
      </td>
  </tr>
  <tr><td valign=\"top\">T = <b>to_T</b>(Q);</td>
      <td valign=\"top\">Return transformation matrix T from quaternion object Q.
  </tr>
  <tr><td valign=\"top\">T_inv = <b>to_T_inv</b>(Q);</td>
      <td valign=\"top\">Return inverse transformation matrix T_inv from quaternion object Q.
      </td>
  </tr>
</table>
</HTML>"));
          type Orientation "Orientation type defining rotation from a frame 1 into a frame 2 with quaternions {p1,p2,p3,p0}"
            extends Internal.QuaternionBase;
            annotation(Documentation(info = "<html>
<p>
This type describes the <b>rotation</b> to rotate a frame 1 into
a frame 2 using quaternions (also called <b>Euler parameters</b>)
according to the following definition:
</p>
<pre>
   Quaternions.Orientation Q;
   Real  n[3];
   Real  phi(unit=\"rad\");
   Q = [ n*sin(phi/2)
           cos(phi/2) ]
</pre>
<p>
where \"n\" is the <b>axis of rotation</b> to rotate frame 1 into
frame 2 and \"phi\" is the <b>rotation angle</b> for this rotation.
Vector \"n\" is either resolved in frame 1 or in frame 2
(the result is the same since the coordinates of \"n\" with respect to
frame 1 are identical to its coordinates with respect to frame 2).
<p>
<p>
The term \"quaternions\" is prefered over the historically
more reasonable \"Euler parameters\" in order to not get
confused with Modelica \"parameters\".
</p>
</html>
"));
            encapsulated function equalityConstraint "Return the constraint residues to express that two frames have the same quaternion orientation"
              import Modelica;
              import Modelica.Mechanics.MultiBody.Frames.Quaternions;
              extends Modelica.Icons.Function;
              input Quaternions.Orientation Q1 "Quaternions orientation object to rotate frame 0 into frame 1";
              input Quaternions.Orientation Q2 "Quaternions orientation object to rotate frame 0 into frame 2";
              output Real residue[3] "The half of the rotation angles around x-, y-, and z-axis of frame 1 to rotate frame 1 into frame 2 for a small rotation (shall be zero)";
              annotation(Inline = true);
            algorithm 
              residue:=[Q1[4],Q1[3], -Q1[2], -Q1[1]; -Q1[3],Q1[4],Q1[1], -Q1[2];Q1[2], -Q1[1],Q1[4], -Q1[3]] * Q2;
            end equalityConstraint;
          end Orientation;
          type der_Orientation = Real[4](each unit = "1/s") "First time derivative of Quaternions.Orientation";
          function orientationConstraint "Return residues of orientation constraints (shall be zero)"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            output Real residue[1] "Residue constraint (shall be zero)";
            annotation(Inline = true);
          algorithm 
            residue:={Q * Q - 1};
          end orientationConstraint;
          function angularVelocity1 "Compute angular velocity resolved in frame 1 from quaternion orientation object and its derivative"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            input der_Orientation der_Q "Derivative of Q";
            output Modelica.SIunits.AngularVelocity w[3] "Angular velocity resolved in frame 1";
            annotation(Inline = true);
          algorithm 
            w:=2 * [Q[4], -Q[3],Q[2], -Q[1];Q[3],Q[4], -Q[1], -Q[2]; -Q[2],Q[1],Q[4], -Q[3]] * der_Q;
          end angularVelocity1;
          function angularVelocity2 "Compute angular velocity resolved in frame 2 from quaternions orientation object and its derivative"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            input der_Orientation der_Q "Derivative of Q";
            output Modelica.SIunits.AngularVelocity w[3] "Angular velocity of frame 2 with respect to frame 1 resolved in frame 2";
            annotation(Inline = true);
          algorithm 
            w:=2 * [Q[4],Q[3], -Q[2], -Q[1]; -Q[3],Q[4],Q[1], -Q[2];Q[2], -Q[1],Q[4], -Q[3]] * der_Q;
          end angularVelocity2;
          function resolve1 "Transform vector from frame 2 to frame 1"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            input Real v2[3] "Vector in frame 2";
            output Real v1[3] "Vector in frame 1";
            annotation(Inline = true);
          algorithm 
            v1:=2 * ((Q[4] * Q[4] - 0.5) * v2 + Q[1:3] * v2 * Q[1:3] + Q[4] * cross(Q[1:3], v2));
          end resolve1;
          function resolve2 "Transform vector from frame 1 to frame 2"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            input Real v1[3] "Vector in frame 1";
            output Real v2[3] "Vector in frame 2";
            annotation(Inline = true);
          algorithm 
            v2:=2 * ((Q[4] * Q[4] - 0.5) * v1 + Q[1:3] * v1 * Q[1:3] - Q[4] * cross(Q[1:3], v1));
          end resolve2;
          function multipleResolve1 "Transform several vectors from frame 2 to frame 1"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            input Real v2[3,:] "Vectors in frame 2";
            output Real v1[3,size(v2, 2)] "Vectors in frame 1";
            annotation(Inline = true);
          algorithm 
            v1:=((2 * Q[4] * Q[4] - 1) * identity(3) + 2 * ([Q[1:3]] * transpose([Q[1:3]]) + Q[4] * skew(Q[1:3]))) * v2;
          end multipleResolve1;
          function multipleResolve2 "Transform several vectors from frame 1 to frame 2"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            input Real v1[3,:] "Vectors in frame 1";
            output Real v2[3,size(v1, 2)] "Vectors in frame 2";
            annotation(Inline = true);
          algorithm 
            v2:=((2 * Q[4] * Q[4] - 1) * identity(3) + 2 * ([Q[1:3]] * transpose([Q[1:3]]) - Q[4] * skew(Q[1:3]))) * v1;
          end multipleResolve2;
          function nullRotation "Return quaternions orientation object that does not rotate a frame"
            extends Modelica.Icons.Function;
            output Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            annotation(Inline = true);
          algorithm 
            Q:={0,0,0,1};
          end nullRotation;
          function inverseRotation "Return inverse quaternions orientation object"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            output Quaternions.Orientation Q_inv "Quaternions orientation object to rotate frame 2 into frame 1";
            annotation(Inline = true);
          algorithm 
            Q_inv:={ -Q[1], -Q[2], -Q[3],Q[4]};
          end inverseRotation;
          function relativeRotation "Return relative quaternions orientation object"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q1 "Quaternions orientation object to rotate frame 0 into frame 1";
            input Quaternions.Orientation Q2 "Quaternions orientation object to rotate frame 0 into frame 2";
            output Quaternions.Orientation Q_rel "Quaternions orientation object to rotate frame 1 into frame 2";
            annotation(Inline = true);
          algorithm 
            Q_rel:=[Q1[4],Q1[3], -Q1[2], -Q1[1]; -Q1[3],Q1[4],Q1[1], -Q1[2];Q1[2], -Q1[1],Q1[4], -Q1[3];Q1[1],Q1[2],Q1[3],Q1[4]] * Q2;
          end relativeRotation;
          function absoluteRotation "Return absolute quaternions orientation object from another absolute and a relative quaternions orientation object"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q1 "Quaternions orientation object to rotate frame 0 into frame 1";
            input Quaternions.Orientation Q_rel "Quaternions orientation object to rotate frame 1 into frame 2";
            output Quaternions.Orientation Q2 "Quaternions orientation object to rotate frame 0 into frame 2";
            annotation(Inline = true);
          algorithm 
            Q2:=[Q_rel[4],Q_rel[3], -Q_rel[2],Q_rel[1]; -Q_rel[3],Q_rel[4],Q_rel[1],Q_rel[2];Q_rel[2], -Q_rel[1],Q_rel[4],Q_rel[3]; -Q_rel[1], -Q_rel[2], -Q_rel[3],Q_rel[4]] * Q1;
          end absoluteRotation;
          function planarRotation "Return quaternions orientation object of a planar rotation"
            import Modelica.Math;
            extends Modelica.Icons.Function;
            input Real e[3](each final unit = "1") "Normalized axis of rotation (must have length=1)";
            input Modelica.SIunits.Angle angle "Rotation angle to rotate frame 1 into frame 2 along axis e";
            output Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2 along axis e";
            annotation(Inline = true);
          algorithm 
            Q:=vector([e * Math.sin(angle / 2);Math.cos(angle / 2)]);
          end planarRotation;
          function smallRotation "Return rotation angles valid for a small rotation"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            output Modelica.SIunits.Angle phi[3] "The rotation angles around x-, y-, and z-axis of frame 1 to rotate frame 1 into frame 2 for a small relative rotation";
            annotation(Inline = true);
          algorithm 
            phi:=2 * {Q[1],Q[2],Q[3]};
          end smallRotation;
          function from_T "Return quaternions orientation object Q from transformation matrix T"
            extends Modelica.Icons.Function;
            input Real T[3,3] "Transformation matrix to transform vector from frame 1 to frame 2 (v2=T*v1)";
            input Quaternions.Orientation Q_guess = nullRotation() "Guess value for Q (there are 2 solutions; the one close to Q_guess is used";
            output Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2 (Q and -Q have same transformation matrix)";
          protected 
            Real paux;
            Real paux4;
            Real c1;
            Real c2;
            Real c3;
            Real c4;
            constant Real p4limit = 0.1;
            constant Real c4limit = 4 * p4limit * p4limit;
          algorithm 
            c1:=1 + T[1,1] - T[2,2] - T[3,3];
            c2:=1 + T[2,2] - T[1,1] - T[3,3];
            c3:=1 + T[3,3] - T[1,1] - T[2,2];
            c4:=1 + T[1,1] + T[2,2] + T[3,3];
            if c4 > c4limit or c4 > c1 and c4 > c2 and c4 > c3 then 
                        paux:=sqrt(c4) / 2;
            paux4:=4 * paux;
            Q:={(T[2,3] - T[3,2]) / paux4,(T[3,1] - T[1,3]) / paux4,(T[1,2] - T[2,1]) / paux4,paux};
            elseif c1 > c2 and c1 > c3 and c1 > c4 then
              paux:=sqrt(c1) / 2;
              paux4:=4 * paux;
              Q:={paux,(T[1,2] + T[2,1]) / paux4,(T[1,3] + T[3,1]) / paux4,(T[2,3] - T[3,2]) / paux4};

            elseif c2 > c1 and c2 > c3 and c2 > c4 then
              paux:=sqrt(c2) / 2;
              paux4:=4 * paux;
              Q:={(T[1,2] + T[2,1]) / paux4,paux,(T[2,3] + T[3,2]) / paux4,(T[3,1] - T[1,3]) / paux4};


            else             paux:=sqrt(c3) / 2;
            paux4:=4 * paux;
            Q:={(T[1,3] + T[3,1]) / paux4,(T[2,3] + T[3,2]) / paux4,paux,(T[1,2] - T[2,1]) / paux4};

            end if;
            if Q * Q_guess < 0 then 
                        Q:= -Q;

            else 
            end if;
          end from_T;
          function from_T_inv "Return quaternions orientation object Q from inverse transformation matrix T_inv"
            extends Modelica.Icons.Function;
            input Real T_inv[3,3] "Inverse transformation matrix to transform vector from frame 2 to frame 1 (v1=T_inv*v2)";
            input Quaternions.Orientation Q_guess = nullRotation() "Guess value for output Q (there are 2 solutions; the one closer to Q_guess is used";
            output Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2 (Q and -Q have same transformation matrix)";
            annotation(Inline = true);
          algorithm 
            Q:=from_T(transpose(T_inv), Q_guess);
          end from_T_inv;
          function to_T "Return transformation matrix T from quaternion orientation object Q"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            output Real T[3,3] "Transformation matrix to transform vector from frame 1 to frame 2 (v2=T*v1)";
            annotation(Inline = true);
          algorithm 
            T:=[2 * (Q[1] * Q[1] + Q[4] * Q[4]) - 1,2 * (Q[1] * Q[2] + Q[3] * Q[4]),2 * (Q[1] * Q[3] - Q[2] * Q[4]);2 * (Q[2] * Q[1] - Q[3] * Q[4]),2 * (Q[2] * Q[2] + Q[4] * Q[4]) - 1,2 * (Q[2] * Q[3] + Q[1] * Q[4]);2 * (Q[3] * Q[1] + Q[2] * Q[4]),2 * (Q[3] * Q[2] - Q[1] * Q[4]),2 * (Q[3] * Q[3] + Q[4] * Q[4]) - 1];
          end to_T;
          function to_T_inv "Return inverse transformation matrix T_inv from quaternion orientation object Q"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            output Real T_inv[3,3] "Transformation matrix to transform vector from frame 2 to frame 1 (v1=T*v2)";
            annotation(Inline = true);
          algorithm 
            T_inv:=[2 * (Q[1] * Q[1] + Q[4] * Q[4]) - 1,2 * (Q[2] * Q[1] - Q[3] * Q[4]),2 * (Q[3] * Q[1] + Q[2] * Q[4]);2 * (Q[1] * Q[2] + Q[3] * Q[4]),2 * (Q[2] * Q[2] + Q[4] * Q[4]) - 1,2 * (Q[3] * Q[2] - Q[1] * Q[4]);2 * (Q[1] * Q[3] - Q[2] * Q[4]),2 * (Q[2] * Q[3] + Q[1] * Q[4]),2 * (Q[3] * Q[3] + Q[4] * Q[4]) - 1];
          end to_T_inv;
        end Quaternions;
        package TransformationMatrices "Functions for transformation matrices"
          extends Modelica.Icons.Library;
          type Orientation "Orientation type defining rotation from a frame 1 into a frame 2 with a transformation matrix"
            extends Internal.TransformationMatrix;
            annotation(Documentation(info = "<html>
<p>
This type describes the <b>rotation</b> from a <b>frame 1</b> into a <b>frame 2</b>.
An instance <b>R</b> of type <b>Orientation</b> has the following interpretation:
</p>
<pre>
   <b>T</b> = [<b>e</b><sub>x</sub>, <b>e</b><sub>y</sub>, <b>e</b><sub>z</sub>];
       e.g., <b>T</b> = [1,0,0; 0,1,0; 0,0,1]
</pre>
<p>
where <b>e</b><sub>x</sub>,<b>e</b><sub>y</sub>,<b>e</b><sub>z</sub>
are unit vectors in the direction of the x-axis, y-axis, and z-axis
of frame 1, resolved in frame 2, respectively. Therefore, if <b>v</b><sub>1</sub>
is vector <b>v</b> resolved in frame 1 and <b>v</b><sub>2</sub> is
vector <b>v</b> resolved in frame 2, the following relationship holds:
</p>
<pre>
    <b>v</b><sub>2</sub> = <b>T</b> * <b>v</b><sub>1</sub>
</pre>
</p>
The <b>inverse</b> orientation
<b>T_inv</b> = <b>T</b><sup>T</sup> describes the rotation
from frame 2 into frame 1.
</p>
<p>
Since the orientation is described by 9 variables, there are
6 constraints between these variables. These constraints
are defined in function <b>TransformationMatrices.orientationConstraint</b>.
</p>
<p>
Note, that in the MultiBody library the rotation object is
never directly accessed but only with the access functions provided
in package TransformationMatrices. As a consequence, other implementations of
Rotation can be defined by adapting this package correspondingly.
</p>
</html>
"));
            encapsulated function equalityConstraint "Return the constraint residues to express that two frames have the same orientation"
              import Modelica;
              import Modelica.Mechanics.MultiBody.Frames.TransformationMatrices;
              extends Modelica.Icons.Function;
              input TransformationMatrices.Orientation T1 "Orientation object to rotate frame 0 into frame 1";
              input TransformationMatrices.Orientation T2 "Orientation object to rotate frame 0 into frame 2";
              output Real residue[3] "The rotation angles around x-, y-, and z-axis of frame 1 to rotate frame 1 into frame 2 for a small rotation (should be zero)";
              annotation(Inline = true);
            algorithm 
              residue:={cross(T1[1,:], T1[2,:]) * T2[2,:], -cross(T1[1,:], T1[2,:]) * T2[1,:],T1[2,:] * T2[1,:]};
            end equalityConstraint;
          end Orientation;
          type der_Orientation = Real[3,3](each unit = "1/s") "New type defining the first time derivative of Orientation";
          function orientationConstraint "Return residues of orientation constraints (shall be zero)"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            output Real residue[6] "Residues of constraints between elements of orientation object (shall be zero)";
            annotation(Inline = true);
          algorithm 
            residue:={T[:,1] * T[:,1] - 1,T[:,2] * T[:,2] - 1,T[:,3] * T[:,3] - 1,T[:,1] * T[:,2],T[:,1] * T[:,3],T[:,2] * T[:,3]};
          end orientationConstraint;
          function angularVelocity1 "Return angular velocity resolved in frame 1 from orientation object and its derivative"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            input der_Orientation der_T "Derivative of T";
            output Modelica.SIunits.AngularVelocity w[3] "Angular velocity of frame 2 with respect to frame 1 resolved in frame 1";
            annotation(Inline = true);
          algorithm 
            w:={der_T[:,3] * T[:,2], -der_T[:,3] * T[:,1],der_T[:,2] * T[:,1]};
          end angularVelocity1;
          function angularVelocity2 "Return angular velocity resolved in frame 2 from orientation object and its derivative"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            input der_Orientation der_T "Derivative of T";
            output Modelica.SIunits.AngularVelocity w[3] "Angular velocity of frame 2 with respect to frame 1 resolved in frame 2";
            annotation(Inline = true);
          algorithm 
            w:={T[3,:] * der_T[2,:], -T[3,:] * der_T[1,:],T[2,:] * der_T[1,:]};
          end angularVelocity2;
          function resolve1 "Transform vector from frame 2 to frame 1"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            input Real v2[3] "Vector in frame 2";
            output Real v1[3] "Vector in frame 1";
            annotation(Inline = true);
          algorithm 
            v1:=transpose(T) * v2;
          end resolve1;
          function resolve2 "Transform vector from frame 1 to frame 2"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            input Real v1[3] "Vector in frame 1";
            output Real v2[3] "Vector in frame 2";
            annotation(Inline = true);
          algorithm 
            v2:=T * v1;
          end resolve2;
          function multipleResolve1 "Transform several vectors from frame 2 to frame 1"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            input Real v2[3,:] "Vectors in frame 2";
            output Real v1[3,size(v2, 2)] "Vectors in frame 1";
            annotation(Inline = true);
          algorithm 
            v1:=transpose(T) * v2;
          end multipleResolve1;
          function multipleResolve2 "Transform several vectors from frame 1 to frame 2"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            input Real v1[3,:] "Vectors in frame 1";
            output Real v2[3,size(v1, 2)] "Vectors in frame 2";
            annotation(Inline = true);
          algorithm 
            v2:=T * v1;
          end multipleResolve2;
          function resolveDyade1 "Transform second order tensor from frame 2 to frame 1"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            input Real D2[3,3] "Second order tensor resolved in frame 2";
            output Real D1[3,3] "Second order tensor resolved in frame 1";
            annotation(Inline = true);
          algorithm 
            D1:=transpose(T) * D2 * T;
          end resolveDyade1;
          function resolveDyade2 "Transform second order tensor from frame 1 to frame 2"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            input Real D1[3,3] "Second order tensor resolved in frame 1";
            output Real D2[3,3] "Second order tensor resolved in frame 2";
            annotation(Inline = true);
          algorithm 
            D2:=T * D1 * transpose(T);
          end resolveDyade2;
          function nullRotation "Return orientation object that does not rotate a frame"
            extends Modelica.Icons.Function;
            output TransformationMatrices.Orientation T "Orientation object such that frame 1 and frame 2 are identical";
            annotation(Inline = true);
          algorithm 
            T:=identity(3);
          end nullRotation;
          function inverseRotation "Return inverse orientation object"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            output TransformationMatrices.Orientation T_inv "Orientation object to rotate frame 2 into frame 1";
            annotation(Inline = true);
          algorithm 
            T_inv:=transpose(T);
          end inverseRotation;
          function relativeRotation "Return relative orientation object"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T1 "Orientation object to rotate frame 0 into frame 1";
            input TransformationMatrices.Orientation T2 "Orientation object to rotate frame 0 into frame 2";
            output TransformationMatrices.Orientation T_rel "Orientation object to rotate frame 1 into frame 2";
            annotation(Inline = true);
          algorithm 
            T_rel:=T2 * transpose(T1);
          end relativeRotation;
          function absoluteRotation "Return absolute orientation object from another absolute and a relative orientation object"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T1 "Orientation object to rotate frame 0 into frame 1";
            input TransformationMatrices.Orientation T_rel "Orientation object to rotate frame 1 into frame 2";
            output TransformationMatrices.Orientation T2 "Orientation object to rotate frame 0 into frame 2";
            annotation(Inline = true);
          algorithm 
            T2:=T_rel * T1;
          end absoluteRotation;
          function planarRotation "Return orientation object of a planar rotation"
            import Modelica.Math;
            extends Modelica.Icons.Function;
            input Real e[3](each final unit = "1") "Normalized axis of rotation (must have length=1)";
            input Modelica.SIunits.Angle angle "Rotation angle to rotate frame 1 into frame 2 along axis e";
            output TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            annotation(Inline = true);
          algorithm 
            T:=[e] * transpose([e]) + (identity(3) - [e] * transpose([e])) * Math.cos(angle) - skew(e) * Math.sin(angle);
          end planarRotation;
          function planarRotationAngle "Return angle of a planar rotation, given the rotation axis and the representations of a vector in frame 1 and frame 2"
            extends Modelica.Icons.Function;
            input Real e[3](each final unit = "1") "Normalized axis of rotation to rotate frame 1 around e into frame 2 (must have length=1)";
            input Real v1[3] "A vector v resolved in frame 1 (shall not be parallel to e)";
            input Real v2[3] "Vector v resolved in frame 2, i.e., v2 = resolve2(planarRotation(e,angle),v1)";
            output Modelica.SIunits.Angle angle "Rotation angle to rotate frame 1 into frame 2 along axis e in the range: -pi <= angle <= pi";
            annotation(Inline = true, Documentation(info = "<HTML>
<p>
A call to this function of the form
</p>
<pre>
    Real[3]                e, v1, v2;
    Modelica.SIunits.Angle angle;
  <b>equation</b>
    angle = <b>planarRotationAngle</b>(e, v1, v2);
</pre>
<p>
computes the rotation angle \"<b>angle</b>\" of a planar
rotation along unit vector <b>e</b>, rotating frame 1 into frame 2, given
the coordinate representations of a vector \"v\" in frame 1 (<b>v1</b>)
and in frame 2 (<b>v2</b>). Therefore, the result of this function
fulfills the following equation:
</p>
<pre>
    v2 = <b>resolve2</b>(<b>planarRotation</b>(e,angle), v1)
</pre>
<p>
The rotation angle is returned in the range
</p>
<pre>
    -<font face=\"Symbol\">p</font> &lt;= angle &lt;= <font face=\"Symbol\">p</font>
</pre>
<p>
This function makes the following assumptions on the input arguments
</p>
<ul>
<li> Vector <b>e</b> has length 1, i.e., length(e) = 1</li>
<li> Vector \"v\" is not parallel to <b>e</b>, i.e.,
     length(cross(e,v1)) &ne; 0</li>
</ul>
<p>
The function does not check the above assumptions. If these
assumptions are violated, a wrong result will be returned
and/or a division by zero will occur.
</p>
</HTML>"));
          algorithm 
            angle:=Modelica.Math.atan2( -cross(e, v1) * v2, v1 * v2 - e * v1 * e * v2);
          end planarRotationAngle;
          function axisRotation "Return rotation object to rotate around one frame axis"
            import Modelica.Math.*;
            extends Modelica.Icons.Function;
            input Integer axis(min = 1, max = 3) "Rotate around 'axis' of frame 1";
            input Modelica.SIunits.Angle angle "Rotation angle to rotate frame 1 into frame 2 along 'axis' of frame 1";
            output TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            annotation(Inline = true);
          algorithm 
            T:=if axis == 1 then [1,0,0;0,cos(angle),sin(angle);0, -sin(angle),cos(angle)] else if axis == 2 then [cos(angle),0, -sin(angle);0,1,0;sin(angle),0,cos(angle)] else [cos(angle),sin(angle),0; -sin(angle),cos(angle),0;0,0,1];
          end axisRotation;
          function axesRotations "Return rotation object to rotate in sequence around 3 axes"
            extends Modelica.Icons.Function;
            input Integer sequence[3](min = {1,1,1}, max = {3,3,3}) = {1,2,3} "Sequence of rotations from frame 1 to frame 2 along axis sequence[i]";
            input Modelica.SIunits.Angle angles[3] = {0,0,0} "Rotation angles around the axes defined in 'sequence'";
            output TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            annotation(Inline = true);
          algorithm 
            T:=absoluteRotation(absoluteRotation(axisRotation(sequence[1], angles[1]), axisRotation(sequence[2], angles[2])), axisRotation(sequence[3], angles[3]));
          end axesRotations;
          function axesRotationsAngles "Return the 3 angles to rotate in sequence around 3 axes to construct the given orientation object"
            import SI = Modelica.SIunits;
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            input Integer sequence[3](min = {1,1,1}, max = {3,3,3}) = {1,2,3} "Sequence of rotations from frame 1 to frame 2 along axis sequence[i]";
            input SI.Angle guessAngle1 = 0 "Select angles[1] such that |angles[1] - guessAngle1| is a minimum";
            output SI.Angle angles[3] "Rotation angles around the axes defined in 'sequence' such that T=TransformationMatrices.axesRotation(sequence,angles); -pi < angles[i] <= pi";
            annotation(Documentation(info = "<HTML>
<p>
A call to this function of the form
</p>
<pre>
    TransformationMatrices.Orientation     T;
    <b>parameter</b> Integer      sequence[3] = {1,2,3};
    Modelica.SIunits.Angle angles[3];
  <b>equation</b>
    angle = <b>axesRotationAngles</b>(T, sequence);
</pre>
<p>
computes the rotation angles \"<b>angles</b>[1:3]\" to rotate frame 1
into frame 2 along axes <b>sequence</b>[1:3], given the orientation
object <b>T</b> from frame 1 to frame 2. Therefore, the result of
this function fulfills the following equation:
</p>
<pre>
    T = <b>axesRotation</b>(sequence, angles)
</pre>
<p>
The rotation angles are returned in the range
</p>
<pre>
    -<font face=\"Symbol\">p</font> &lt;= angles[i] &lt;= <font face=\"Symbol\">p</font>
</pre>
<p>
There are <b>two solutions</b> for \"angles[1]\" in this range.
Via the third argument <b>guessAngle1</b> (default = 0) the
returned solution is selected such that |angles[1] - guessAngle1| is
minimal. The orientation object T may be in a singular configuration, i.e.,
there is an infinite number of angle values leading to the same T. The returned solution is
selected by setting angles[1] = guessAngle1. Then angles[2]
and angles[3] can be uniquely determined in the above range.
</p>
<p>
Note, that input argument <b>sequence</b> has the restriction that
only values 1,2,3 can be used and that sequence[1] &ne; sequence[2]
and sequence[2] &ne; sequence[3]. Often used values are:
</p>
<pre>
  sequence = <b>{1,2,3}</b>  // Cardan angle sequence
           = <b>{3,1,3}</b>  // Euler angle sequence
           = <b>{3,2,1}</b>  // Tait-Bryan angle sequence
</pre>
</HTML>"));
          protected 
            Real e1_1[3](each final unit = "1") "First rotation axis, resolved in frame 1";
            Real e2_1a[3](each final unit = "1") "Second rotation axis, resolved in frame 1a";
            Real e3_1[3](each final unit = "1") "Third rotation axis, resolved in frame 1";
            Real e3_2[3](each final unit = "1") "Third rotation axis, resolved in frame 2";
            Real A "Coefficient A in the equation A*cos(angles[1])+B*sin(angles[1]) = 0";
            Real B "Coefficient B in the equation A*cos(angles[1])+B*sin(angles[1]) = 0";
            SI.Angle angle_1a "Solution 1 for angles[1]";
            SI.Angle angle_1b "Solution 2 for angles[1]";
            TransformationMatrices.Orientation T_1a "Orientation object to rotate frame 1 into frame 1a";
          algorithm 
            assert(sequence[1] <> sequence[2] and sequence[2] <> sequence[3], "input argument 'sequence[1:3]' is not valid");
            e1_1:=if sequence[1] == 1 then {1,0,0} else if sequence[1] == 2 then {0,1,0} else {0,0,1};
            e2_1a:=if sequence[2] == 1 then {1,0,0} else if sequence[2] == 2 then {0,1,0} else {0,0,1};
            e3_1:=T[sequence[3],:];
            e3_2:=if sequence[3] == 1 then {1,0,0} else if sequence[3] == 2 then {0,1,0} else {0,0,1};
            A:=e2_1a * e3_1;
            B:=cross(e1_1, e2_1a) * e3_1;
            if abs(A) <= 1e-12 and abs(B) <= 1e-12 then 
                        angles[1]:=guessAngle1;

            else             angle_1a:=Modelica.Math.atan2(A,  -B);
            angle_1b:=Modelica.Math.atan2( -A, B);
            angles[1]:=if abs(angle_1a - guessAngle1) <= abs(angle_1b - guessAngle1) then angle_1a else angle_1b;

            end if;
            T_1a:=planarRotation(e1_1, angles[1]);
            angles[2]:=TransformationMatrices.planarRotationAngle(e2_1a, TransformationMatrices.resolve2(T_1a, e3_1), e3_2);
            angles[3]:=TransformationMatrices.planarRotationAngle(e3_2, e2_1a, TransformationMatrices.resolve2(T, TransformationMatrices.resolve1(T_1a, e2_1a)));
          end axesRotationsAngles;
          function smallRotation "Return rotation angles valid for a small rotation and optionally residues that should be zero"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            input Boolean withResidues = false "= false/true, if 'angles'/'angles and residues' are returned in phi";
            output Modelica.SIunits.Angle phi[if withResidues then 6 else 3] "The rotation angles around x-, y-, and z-axis of frame 1 to rotate frame 1 into frame 2 for a small rotation + optionally 3 residues that should be zero";
            annotation(Inline = true);
          algorithm 
            phi:=if withResidues then {T[2,3], -T[1,3],T[1,2],T[1,1] - 1,T[2,2] - 1,T[1,1] * T[2,2] - T[2,1] * T[1,2] - 1} else {T[2,3], -T[1,3],T[1,2]};
          end smallRotation;
          function from_nxy "Return orientation object from n_x and n_y vectors"
            extends Modelica.Icons.Function;
            input Real n_x[3](each final unit = "1") "Vector in direction of x-axis of frame 2, resolved in frame 1";
            input Real n_y[3](each final unit = "1") "Vector in direction of y-axis of frame 2, resolved in frame 1";
            output TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            annotation(Documentation(info = "<html>
<p>
It is assumed that the two input vectors n_x and n_y are
resolved in frame 1 and are directed along the x and y axis
of frame 2 (i.e., n_x and n_y are orthogonal to each other)
The function returns the orientation object T to rotate from
frame 1 to frame 2.
</p>
<p>
The function is robust in the sense that it returns always
an orientation object T, even if n_y is not orthogonal to n_x.
This is performed in the following way:
</p>
<p>
If n_x and n_y are not orthogonal to each other, first a unit
vector e_y is determined that is orthogonal to n_x and is lying
in the plane spanned by n_x and n_y. If n_x and n_y are parallel
or nearly parallel to each other, a vector e_y is selected
arbitrarily such that e_x and e_y are orthogonal to each other.
</p>
</html>"));
          protected 
            Real abs_n_x = sqrt(n_x * n_x);
            Real e_x[3](each final unit = "1") = if abs_n_x < 1e-10 then {1,0,0} else n_x / abs_n_x;
            Real n_z_aux[3](each final unit = "1") = cross(e_x, n_y);
            Real n_y_aux[3](each final unit = "1") = if n_z_aux * n_z_aux > 1e-06 then n_y else if abs(e_x[1]) > 1e-06 then {0,1,0} else {1,0,0};
            Real e_z_aux[3](each final unit = "1") = cross(e_x, n_y_aux);
            Real e_z[3](each final unit = "1") = e_z_aux / sqrt(e_z_aux * e_z_aux);
          algorithm 
            T:={e_x,cross(e_z, e_x),e_z};
          end from_nxy;
          function from_nxz "Return orientation object from n_x and n_z vectors"
            extends Modelica.Icons.Function;
            input Real n_x[3](each final unit = "1") "Vector in direction of x-axis of frame 2, resolved in frame 1";
            input Real n_z[3](each final unit = "1") "Vector in direction of z-axis of frame 2, resolved in frame 1";
            output TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            annotation(Documentation(info = "<html>
<p>
It is assumed that the two input vectors n_x and n_z are
resolved in frame 1 and are directed along the x and z axis
of frame 2 (i.e., n_x and n_z are orthogonal to each other)
The function returns the orientation object T to rotate from
frame 1 to frame 2.
</p>
<p>
The function is robust in the sense that it returns always
an orientation object T, even if n_z is not orthogonal to n_x.
This is performed in the following way:
</p>
<p>
If n_x and n_z are not orthogonal to each other, first a unit
vector e_z is determined that is orthogonal to n_x and is lying
in the plane spanned by n_x and n_z. If n_x and n_z are parallel
or nearly parallel to each other, a vector e_z is selected
arbitrarily such that n_x and e_z are orthogonal to each other.
</p>
</html>"));
          protected 
            Real abs_n_x = sqrt(n_x * n_x);
            Real e_x[3](each final unit = "1") = if abs_n_x < 1e-10 then {1,0,0} else n_x / abs_n_x;
            Real n_y_aux[3](each final unit = "1") = cross(n_z, e_x);
            Real n_z_aux[3](each final unit = "1") = if n_y_aux * n_y_aux > 1e-06 then n_z else if abs(e_x[1]) > 1e-06 then {0,0,1} else {1,0,0};
            Real e_y_aux[3](each final unit = "1") = cross(n_z_aux, e_x);
            Real e_y[3](each final unit = "1") = e_y_aux / sqrt(e_y_aux * e_y_aux);
          algorithm 
            T:={e_x,e_y,cross(e_x, e_y)};
          end from_nxz;
          function from_T "Return orientation object R from transformation matrix T"
            extends Modelica.Icons.Function;
            input Real T[3,3] "Transformation matrix to transform vector from frame 1 to frame 2 (v2=T*v1)";
            output TransformationMatrices.Orientation R "Orientation object to rotate frame 1 into frame 2";
            annotation(Inline = true);
          algorithm 
            R:=T;
          end from_T;
          function from_T_inv "Return orientation object R from inverse transformation matrix T_inv"
            extends Modelica.Icons.Function;
            input Real T_inv[3,3] "Inverse transformation matrix to transform vector from frame 2 to frame 1 (v1=T_inv*v2)";
            output TransformationMatrices.Orientation R "Orientation object to rotate frame 1 into frame 2";
            annotation(Inline = true);
          algorithm 
            R:=transpose(T_inv);
          end from_T_inv;
          function from_Q "Return orientation object T from quaternion orientation object Q"
            extends Modelica.Icons.Function;
            input Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            output TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            annotation(Inline = true);
          algorithm 
            T:=[2 * (Q[1] * Q[1] + Q[4] * Q[4]) - 1,2 * (Q[1] * Q[2] + Q[3] * Q[4]),2 * (Q[1] * Q[3] - Q[2] * Q[4]);2 * (Q[2] * Q[1] - Q[3] * Q[4]),2 * (Q[2] * Q[2] + Q[4] * Q[4]) - 1,2 * (Q[2] * Q[3] + Q[1] * Q[4]);2 * (Q[3] * Q[1] + Q[2] * Q[4]),2 * (Q[3] * Q[2] - Q[1] * Q[4]),2 * (Q[3] * Q[3] + Q[4] * Q[4]) - 1];
          end from_Q;
          function to_T "Return transformation matrix T from orientation object R"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation R "Orientation object to rotate frame 1 into frame 2";
            output Real T[3,3] "Transformation matrix to transform vector from frame 1 to frame 2 (v2=T*v1)";
            annotation(Inline = true);
          algorithm 
            T:=R;
          end to_T;
          function to_T_inv "Return inverse transformation matrix T_inv from orientation object R"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation R "Orientation object to rotate frame 1 into frame 2";
            output Real T_inv[3,3] "Inverse transformation matrix to transform vector from frame 2 into frame 1 (v1=T_inv*v2)";
            annotation(Inline = true);
          algorithm 
            T_inv:=transpose(R);
          end to_T_inv;
          function to_Q "Return quaternion orientation object Q from orientation object T"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            input Quaternions.Orientation Q_guess = Quaternions.nullRotation() "Guess value for output Q (there are 2 solutions; the one closer to Q_guess is used";
            output Quaternions.Orientation Q "Quaternions orientation object to rotate frame 1 into frame 2";
            annotation(Inline = true);
          algorithm 
            Q:=Quaternions.from_T(T, Q_guess);
          end to_Q;
          function to_vector "Map rotation object into vector"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            output Real vec[9] "Elements of T in one vector";
            annotation(Inline = true);
          algorithm 
            vec:={T[1,1],T[2,1],T[3,1],T[1,2],T[2,2],T[3,2],T[1,3],T[2,3],T[3,3]};
          end to_vector;
          function to_exy "Map rotation object into e_x and e_y vectors of frame 2, resolved in frame 1"
            extends Modelica.Icons.Function;
            input TransformationMatrices.Orientation T "Orientation object to rotate frame 1 into frame 2";
            output Real exy[3,2] "= [e_x, e_y] where e_x and e_y are axes unit vectors of frame 2, resolved in frame 1";
            annotation(Inline = true);
          algorithm 
            exy:=[T[1,:],T[2,:]];
          end to_exy;
          annotation(Documentation(info = "<HTML>
<p>
Package <b>Frames.TransformationMatrices</b> contains type definitions and
functions to transform rotational frame quantities using
transformation matrices.
</p>
<h4>Content</h4>
<p>In the table below an example is given for every function definition.
The used variables have the following declaration:
</p>
<pre>
   Orientation T, T1, T2, T_rel, T_inv;
   Real[3]     v1, v2, w1, w2, n_x, n_y, n_z, e, e_x, res_ori, phi;
   Real[6]     res_equal;
   Real        L, angle;
</pre>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Function/type</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\"><b>Orientation T;</b></td>
      <td valign=\"top\">New type defining an orientation object that describes<br>
          the rotation of frame 1 into frame 2.
      </td>
  </tr>
  <tr><td valign=\"top\"><b>der_Orientation</b> der_T;</td>
      <td valign=\"top\">New type defining the first time derivative
         of Frames.Orientation.
      </td>
  </tr>
  <tr><td valign=\"top\">res_ori = <b>orientationConstraint</b>(T);</td>
      <td valign=\"top\">Return the constraints between the variables of an orientation object<br>
      (shall be zero).</td>
  </tr>
  <tr><td valign=\"top\">w1 = <b>angularVelocity1</b>(T, der_T);</td>
      <td valign=\"top\">Return angular velocity resolved in frame 1 from
          orientation object T<br> and its derivative der_T.
     </td>
  </tr>
  <tr><td valign=\"top\">w2 = <b>angularVelocity2</b>(T, der_T);</td>
      <td valign=\"top\">Return angular velocity resolved in frame 2 from
          orientation object T<br> and its derivative der_T.
     </td>
  </tr>
  <tr><td valign=\"top\">v1 = <b>resolve1</b>(T,v2);</td>
      <td valign=\"top\">Transform vector v2 from frame 2 to frame 1.
      </td>
  </tr>
  <tr><td valign=\"top\">v2 = <b>resolve2</b>(T,v1);</td>
      <td valign=\"top\">Transform vector v1 from frame 1 to frame 2.
     </td>
  </tr>
  <tr><td valign=\"top\">[v1,w1] = <b>multipleResolve1</b>(T, [v2,w2]);</td>
      <td valign=\"top\">Transform several vectors from frame 2 to frame 1.
      </td>
  </tr>
  <tr><td valign=\"top\">[v2,w2] = <b>multipleResolve2</b>(T, [v1,w1]);</td>
      <td valign=\"top\">Transform several vectors from frame 1 to frame 2.
      </td>
  </tr>
  <tr><td valign=\"top\">D1 = <b>resolveDyade1</b>(T,D2);</td>
      <td valign=\"top\">Transform second order tensor D2 from frame 2 to frame 1.
      </td>
  </tr>
  <tr><td valign=\"top\">D2 = <b>resolveDyade2</b>(T,D1);</td>
      <td valign=\"top\">Transform second order tensor D1 from frame 1 to frame 2.
     </td>
  </tr>
  <tr><td valign=\"top\">T= <b>nullRotation</b>()</td>
      <td valign=\"top\">Return orientation object T that does not rotate a frame.
  </tr>
  <tr><td valign=\"top\">T_inv = <b>inverseRotation</b>(T);</td>
      <td valign=\"top\">Return inverse orientation object.
      </td>
  </tr>
  <tr><td valign=\"top\">T_rel = <b>relativeRotation</b>(T1,T2);</td>
      <td valign=\"top\">Return relative orientation object from two absolute
          orientation objects.
      </td>
  </tr>
  <tr><td valign=\"top\">T2 = <b>absoluteRotation</b>(T1,T_rel);</td>
      <td valign=\"top\">Return absolute orientation object from another
          absolute<br> and a relative orientation object.
      </td>
  </tr>
  <tr><td valign=\"top\">T = <b>planarRotation</b>(e, angle);</td>
      <td valign=\"top\">Return orientation object of a planar rotation.
      </td>
  </tr>
  <tr><td valign=\"top\">angle = <b>planarRotationAngle</b>(e, v1, v2);</td>
      <td valign=\"top\">Return angle of a planar rotation, given the rotation axis<br>
        and the representations of a vector in frame 1 and frame 2.
      </td>
  </tr>
  <tr><td valign=\"top\">T = <b>axisRotation</b>(i, angle);</td>
      <td valign=\"top\">Return orientation object T for rotation around axis i of frame 1.
      </td>
  </tr>
  <tr><td valign=\"top\">T = <b>axesRotations</b>(sequence, angles);</td>
      <td valign=\"top\">Return rotation object to rotate in sequence around 3 axes. Example:<br>
          T = axesRotations({1,2,3},{90,45,-90});
      </td>
  </tr>
  <tr><td valign=\"top\">angles = <b>axesRotationsAngles</b>(T, sequence);</td>
      <td valign=\"top\">Return the 3 angles to rotate in sequence around 3 axes to<br>
          construct the given orientation object.
      </td>
  </tr>
  <tr><td valign=\"top\">phi = <b>smallRotation</b>(T);</td>
      <td valign=\"top\">Return rotation angles phi valid for a small rotation.
      </td>
  </tr>
  <tr><td valign=\"top\">T = <b>from_nxy</b>(n_x, n_y);</td>
      <td valign=\"top\">Return orientation object from n_x and n_y vectors.
      </td>
  </tr>
  <tr><td valign=\"top\">T = <b>from_nxz</b>(n_x, n_z);</td>
      <td valign=\"top\">Return orientation object from n_x and n_z vectors.
      </td>
  </tr>
  <tr><td valign=\"top\">R = <b>from_T</b>(T);</td>
      <td valign=\"top\">Return orientation object R from transformation matrix T.
      </td>
  </tr>
  <tr><td valign=\"top\">R = <b>from_T_inv</b>(T_inv);</td>
      <td valign=\"top\">Return orientation object R from inverse transformation matrix T_inv.
      </td>
  </tr>
  <tr><td valign=\"top\">T = <b>from_Q</b>(Q);</td>
      <td valign=\"top\">Return orientation object T from quaternion orientation object Q.
      </td>
  </tr>
  <tr><td valign=\"top\">T = <b>to_T</b>(R);</td>
      <td valign=\"top\">Return transformation matrix T from orientation object R.
  </tr>
  <tr><td valign=\"top\">T_inv = <b>to_T_inv</b>(R);</td>
      <td valign=\"top\">Return inverse transformation matrix T_inv from orientation object R.
      </td>
  </tr>
  <tr><td valign=\"top\">Q = <b>to_Q</b>(T);</td>
      <td valign=\"top\">Return quaternione orientation object Q from orientation object T.
      </td>
  </tr>
  <tr><td valign=\"top\">exy = <b>to_exy</b>(T);</td>
      <td valign=\"top\">Return [e_x, e_y] matrix of an orientation object T, <br>
          with e_x and e_y vectors of frame 2, resolved in frame 1.
  </tr>
</table>
</HTML>"));
        end TransformationMatrices;
        package Internal "Internal definitions that may be removed or changed (do not use)"
          extends Modelica.Icons.Library;
          type TransformationMatrix = Real[3,3];
          type QuaternionBase = Real[4];
          function maxWithoutEvent "Maximum of the input arguments, without event and without warning message when differentiating"
            input Real u1;
            input Real u2;
            output Real y;
            annotation(Inline = false, derivative = maxWithoutEvent_d, Documentation(info = "<html>
<p>
Function <b>maxWithoutEvent</b> returns the maximum of its two
input arguments. This functions is used instead of the Modelica
built-in function \"max\" or an if-statement with \"noEvent(..)\",
because Dymola prints a warning message when differentiating
in these cases. For the special cases as used in the MultiBody
library, these warning messages are irrelevant but will potentially
irritate the user. The C-function \"maxWithoutEvent\" and its
derivatives provided as additional functions \"maxWithoutEvent_d\"
and \"maxWithoutEvent_dd\" will not lead to such warning
messages.
</p>
</html>"));
          protected 
            Integer dummy;
          algorithm 
            y:=if u1 > u2 then u1 else u2;
            dummy:=0;
          end maxWithoutEvent;
          function maxWithoutEvent_d "First derivative of function maxWithoutEvent(..)"
            input Real u1;
            input Real u2;
            input Real u1_d;
            input Real u2_d;
            output Real y_d;
            annotation(Inline = false, derivative(order = 2) = maxWithoutEvent_dd);
          protected 
            Integer dummy;
          algorithm 
            y_d:=if u1 > u2 then u1_d else u2_d;
            dummy:=0;
          end maxWithoutEvent_d;
          function maxWithoutEvent_dd "First derivative of function maxWithoutEvent_d(..)"
            input Real u1;
            input Real u2;
            input Real u1_d;
            input Real u2_d;
            input Real u1_dd;
            input Real u2_dd;
            output Real y_dd;
          algorithm 
            y_dd:=if u1 > u2 then u1_dd else u2_dd;
          end maxWithoutEvent_dd;
          function resolve1_der "Derivative of function Frames.resolve1(..)"
            import Modelica.Mechanics.MultiBody.Frames;
            extends Modelica.Icons.Function;
            input Orientation R "Orientation object to rotate frame 1 into frame 2";
            input Real v2[3] "Vector resolved in frame 2";
            input Real v2_der[3] "= der(v2)";
            output Real v1_der[3] "Derivative of vector v resolved in frame 1";
            annotation(Inline = true);
          algorithm 
            v1_der:=Frames.resolve1(R, v2_der + cross(R.w, v2));
          end resolve1_der;
          function resolve2_der "Derivative of function Frames.resolve2(..)"
            import Modelica.Mechanics.MultiBody.Frames;
            extends Modelica.Icons.Function;
            input Orientation R "Orientation object to rotate frame 1 into frame 2";
            input Real v1[3] "Vector resolved in frame 1";
            input Real v1_der[3] "= der(v1)";
            output Real v2_der[3] "Derivative of vector v resolved in frame 2";
            annotation(Inline = true);
          algorithm 
            v2_der:=Frames.resolve2(R, v1_der) - cross(R.w, Frames.resolve2(R, v1));
          end resolve2_der;
          function resolveRelative_der "Derivative of function Frames.resolveRelative(..)"
            import Modelica.Mechanics.MultiBody.Frames;
            extends Modelica.Icons.Function;
            input Real v1[3] "Vector in frame 1";
            input Orientation R1 "Orientation object to rotate frame 0 into frame 1";
            input Orientation R2 "Orientation object to rotate frame 0 into frame 2";
            input Real v1_der[3] "= der(v1)";
            output Real v2_der[3] "Derivative of vector v resolved in frame 2";
            annotation(Inline = true);
          algorithm 
            v2_der:=Frames.resolveRelative(v1_der + cross(R1.w, v1), R1, R2) - cross(R2.w, Frames.resolveRelative(v1, R1, R2));
          end resolveRelative_der;
        end Internal;
      end Frames;
      package Forces "Components that exert forces and/or torques between frames"
        import SI = Modelica.SIunits;
        extends Modelica.Icons.Library;
        annotation(Documentation(info = "<HTML>
<p>
This package contains components that exert forces and torques
between two frame connectors, e.g., between two parts.
</p>
<h4>Content</h4>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><b><i>Model</i></b></th><th><b><i>Description</i></b></th></tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Forces.WorldForce\">WorldForce</a></td>
      <td valign=\"top\"> External force acting at the frame to which this component
           is connected and defined by 3 input signals,
           that are interpreted as one vector resolved in frame world, frame_b or frame_resolve. <br>
           <IMG SRC=\"../Images/MultiBody/Forces/ArrowForce.png\"></td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Forces.WorldTorque\">WorldTorque</a></td>
      <td valign=\"top\"> External torque acting at the frame to which this component
           is connected and defined by 3 input signals,
           that are interpreted as one vector resolved in frame world, frame_b or frame_resolve. <br>
           <IMG SRC=\"../Images/MultiBody/Forces/ArrowTorque.png\"></td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Forces.WorldForceAndTorque\">WorldForceAndTorque</a></td>
      <td valign=\"top\"> External force and external torque acting at the frame
           to which this component
           is connected and defined by 3+3 input signals,
           that are interpreted as a force and as a torque vector
           resolved in frame world, frame_b or frame_resolve. <br>
           <IMG SRC=\"../Images/MultiBody/Forces/ArrowForce.png\"><br>
           <IMG SRC=\"../Images/MultiBody/Forces/ArrowTorque.png\">
      </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Forces.Force\">Force</a></td>
      <td valign=\"top\"> Force acting between two frames defined by 3 input signals
           resolved in frame world, frame_a, frame_b or in frame_resolve. <br>
           <IMG SRC=\"../Images/MultiBody/Forces/ArrowForce2.png\"></td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Forces.Torque\">Torque</a></td>
      <td valign=\"top\"> Torque acting between two frames defined by 3 input signals
           resolved in frame world, frame_a, frame_b or in frame_resolve. <br>
           <IMG SRC=\"../Images/MultiBody/Forces/ArrowTorque2.png\"></td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Forces.ForceAndTorque\">ForceAndTorque</a></td>
      <td valign=\"top\"> Force and torque acting between two frames defined by 3+3 input signals
           resolved in frame world, frame_a, frame_b or in frame_resolve. <br>
           <IMG SRC=\"../Images/MultiBody/Forces/ArrowForce2.png\"><br>
           <IMG SRC=\"../Images/MultiBody/Forces/ArrowTorque2.png\"></td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Forces.LineForceWithMass\">LineForceWithMass</a></td>
      <td valign=\"top\">  General line force component with an optional point mass
            on the connection line. The force law can be defined by a
            component of Modelica.Mechanics.Translational<br>
           <IMG SRC=\"../Images/MultiBody/Forces/LineForceWithMass.png\"></td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Forces.LineForceWithTwoMasses\">LineForceWithTwoMasses</a></td>
      <td valign=\"top\">  General line force component with two optional point masses
            on the connection line. The force law can be defined by a
            component of Modelica.Mechanics.Translational<br>
           <IMG SRC=\"../Images/MultiBody/Forces/LineForceWithTwoMasses.png\"></td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Forces.Spring\">Spring</a></td>
      <td valign=\"top\"> Linear translational spring with optional mass <br>
           <IMG SRC=\"../Images/MultiBody/Forces/Spring2.png\"></td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Forces.Damper\">Damper</a></td>
      <td valign=\"top\"> Linear (velocity dependent) damper <br>
           <IMG SRC=\"../Images/MultiBody/Forces/Damper2.png\"></td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Forces.SpringDamperParallel\">SpringDamperParallel</a></td>
      <td valign=\"top\"> Linear spring and damper in parallel connection </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.MultiBody.Forces.SpringDamperSeries\">SpringDamperSeries</a></td>
      <td valign=\"top\"> Linear spring and damper in series connection </td>
  </tr>
</table>
</HTML>"));
        model WorldForce "External force acting at frame_b, defined by 3 input signals and resolved in frame world, frame_b or frame_resolve"
          import SI = Modelica.SIunits;
          extends Interfaces.PartialOneFrame_b;
          Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.frame_resolve "The input signals are optionally resolved in this frame" annotation(Placement(transformation(origin = {0, -100}, extent = {{ -16, -16},{16,16}}, rotation = 270)));
          Modelica.Blocks.Interfaces.RealInput force[3](each final quantity = "Force", each final unit = "N") "x-, y-, z-coordinates of force resolved in frame defined by resolveInFrame" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.world "Frame in which input force is resolved (1: world, 2: frame_b, 3: frame_resolve)";
          parameter Real N_to_m(unit = "N/m") = world.defaultN_to_m "Force arrow scaling (length = force/N_to_m)" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter diameter = world.defaultArrowDiameter "Diameter of force arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.ForceColor "Color of arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(group = "if animation = true", enable = animation));
          annotation(defaultComponentName = "force", Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -89, -46},{91, -76}}, lineColor = {192,192,192}, textString = "resolve"),Polygon(points = {{ -100,10},{50,10},{50,31},{94,0},{50, -31},{50, -10},{ -100, -10},{ -100,10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -149,103},{136,42}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{0, -10},{0, -95}}, color = {95,95,95}, pattern = LinePattern.Dot)}), Documentation(info = "<HTML>

<p>
The <b>3</b> signals of the <b>force</b> connector are interpreted
as the x-, y- and z-coordinates of a <b>force</b> acting at the frame
connector to which frame_b of this component is attached.
Via parameter <b>resolveInFrame</b> it is defined, in which frame these
coordinates shall be resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve input force in world frame (= default)</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve input force in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve input force in frame_resolve (frame_resolve must be connected)</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameB.frame_resolve, the force coordinates
are with respect to the frame, that is connected to <b>frame_resolve</b>.
<p>

<p>
If force={100,0,0}, and for all parameters the default setting is used,
then the interpretation is that a force of 100 N is acting along the positive
x-axis of frame_b.
</p>

<p>
Note, the cut-torque in frame_b (frame_b.t) is always set to zero.
Conceptually, a force and torque acts on the world frame in such a way that
the force and torque balance between world.frame_b and frame_b is fulfilled.
For efficiency reasons, this reaction torque is, however, not computed.
</p>

<p>
This force component is by default visualized as an arrow
acting at the connector to which it is connected. The diameter
and color of the arrow can be defined via
variables <b>diameter</b> and <b>color</b>. The arrow
points in the direction defined by the
force signal. The length of the arrow is proportional
to the length of the force vector using parameter
<b>N_to_m</b> as scaling factor. For example, if N_to_m = 100 N/m,
then a force of 350 N is displayed as an arrow of length 3.5 m.
</p>
<p>
An example how to use this model is given in the
following figure:
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Forces/WorldForce1.png\">
</p>
<p>
This leads to the following animation
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Forces/WorldForce2.png\">
</p>

</HTML>
"));
        protected 
          SI.Position f_in_m[3] = frame_b.f / N_to_m "Force mapped from N to m for animation";
          Visualizers.Advanced.Arrow arrow(diameter = diameter, color = color, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = f_in_m, r_head =  -f_in_m) if world.enableAnimation and animation;
        public 
          Internal.BasicWorldForce basicWorldForce(resolveInFrame = resolveInFrame) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}})));
        protected 
          Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.frame_resolve annotation(Placement(transformation(extent = {{20, -40},{40, -20}})));

        equation 
          connect(basicWorldForce.frame_b,frame_b) annotation(Line(points = {{10,0},{100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicWorldForce.force,force) annotation(Line(points = {{ -12,0},{ -120,0}}, color = {0,0,127}, smooth = Smooth.None));
          connect(basicWorldForce.frame_resolve,frame_resolve) annotation(Line(points = {{0, -10},{0, -100}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve,basicWorldForce.frame_resolve) annotation(Line(points = {{20, -30},{0, -30},{0, -10}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
        end WorldForce;
        model WorldTorque "External torque acting at frame_b, defined by 3 input signals and resolved in frame world, frame_b or frame_resolve"
          extends Interfaces.PartialOneFrame_b;
          Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.frame_resolve "The input signals are optionally resolved in this frame" annotation(Placement(transformation(origin = {0,100}, extent = {{16, -16},{ -16,16}}, rotation = 270)));
          Modelica.Blocks.Interfaces.RealInput torque[3](each final quantity = "Torque", each final unit = "N.m") "x-, y-, z-coordiantes of torque resolved in frame defined by resolveInFrame" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.world "Frame in which input torque is resolved (1: world, 2: frame_b, 3: frame_resolve)";
          parameter Real Nm_to_m(unit = "N.m/m") = world.defaultNm_to_m "Torque arrow scaling (length = torque/Nm_to_m)" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter diameter = world.defaultArrowDiameter "Diameter of torque arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.TorqueColor "Color of arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(group = "if animation = true", enable = animation));
          annotation(defaultComponentName = "torque", Documentation(info = "<HTML>

<p>
The <b>3</b> signals of the <b>torque</b> connector are interpreted
as the x-, y- and z-coordinates of a <b>torque</b> acting at the frame
connector to which frame_b of this component is attached.
Via parameter <b>resolveInFrame</b> it is defined, in which frame these
coordinates shall be resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve input torque in world frame (= default)</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve input torque in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve input torque in frame_resolve (frame_resolve must be connected)</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameB.frame_resolve, the torque coordinates
are with respect to the frame, that is connected to <b>frame_resolve</b>.
<p>

<p>
If torque={100,0,0}, and for all parameters the default setting is used,
then the interpretation is that a torque of 100 N is acting along the positive
x-axis of frame_b.
</p>

<p>
Note, the cut-force in frame_b (frame_b.f) is always set to zero.
Conceptually, a force and torque acts on the world frame in such a way that
the force and torque balance between world.frame_b and frame_b is fulfilled.
For efficiency reasons, this reaction torque is, however, not computed.
</p>

<p>
This torque component is by default visualized as a <b>double arrow</b>
acting at the connector to which it is connected. The diameter
and color of the arrow can be defined via
variables <b>diameter</b> and <b>color</b>. The double arrow points
in the direction defined by the
torque vector. The length of the double arrow is proportional
to the length of the torque vector using parameter
<b>Nm_to_m</b> as scaling factor. For example, if Nm_to_m = 100 Nm/m,
then a torque of 350 Nm is displayed as an arrow of length 3.5 m.
</p>
<p>
An example how to use this model is given in the
following figure:
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Forces/WorldTorque1.png\">
</p>
<p>
This leads to the following animation
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Forces/WorldTorque2.png\">
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -61,64},{46,27}}, lineColor = {192,192,192}, textString = "resolve"),Text(extent = {{ -145, -28},{140, -89}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{0,95},{0,82}}, color = {95,95,95}, pattern = LinePattern.Dot),Line(points = {{ -100,0},{ -94,13},{ -86,28},{ -74,48},{ -65,60},{ -52,72},{ -35,81},{ -22,84},{ -8,84},{7,80},{19,73},{32,65},{44,55},{52,47},{58,40}}, color = {0,0,0}, thickness = 0.5),Polygon(points = {{94,10},{75,59},{41,24},{94,10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
        protected 
          SI.Position t_in_m[3] = frame_b.t / Nm_to_m "Torque mapped from Nm to m for animation";
          Visualizers.Advanced.DoubleArrow arrow(diameter = diameter, color = color, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = t_in_m, r_head =  -t_in_m) if world.enableAnimation and animation;
        public 
          Internal.BasicWorldTorque basicWorldTorque(resolveInFrame = resolveInFrame) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}})));
        protected 
          Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.frame_resolve annotation(Placement(transformation(extent = {{20,10},{40,30}})));

        equation 
          connect(basicWorldTorque.frame_b,frame_b) annotation(Line(points = {{10,0},{100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicWorldTorque.torque,torque) annotation(Line(points = {{ -12,0},{ -120,0}}, color = {0,0,127}, smooth = Smooth.None));
          connect(frame_resolve,basicWorldTorque.frame_resolve) annotation(Line(points = {{0,100},{0,10}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve,basicWorldTorque.frame_resolve) annotation(Line(points = {{20,20},{0,20},{0,10}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
        end WorldTorque;
        model WorldForceAndTorque "External force and torque acting at frame_b, defined by 3+3 input signals and resolved in frame world, frame_b or in frame_resolve"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Types;
          extends Interfaces.PartialOneFrame_b;
          Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.frame_resolve "The input signals are optionally resolved in this frame" annotation(Placement(transformation(origin = {0,100}, extent = {{16, -16},{ -16,16}}, rotation = 270)));
          Blocks.Interfaces.RealInput force[3](each final quantity = "Force", each final unit = "N") "x-, y-, z-coordinates of force resolved in frame defined by resolveInFrame" annotation(Placement(transformation(extent = {{ -140, -80},{ -100, -40}}, rotation = 0)));
          Blocks.Interfaces.RealInput torque[3](each final quantity = "Torque", each final unit = "N.m") "x-, y-, z-coordiantes of torque resolved in frame defined by resolveInFrame" annotation(Placement(transformation(extent = {{ -140,40},{ -100,80}}, rotation = 0)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.world "Frame in which input force and torque are resolved (1: world, 2: frame_b, 3: frame_resolve)";
          parameter Real N_to_m(unit = "N/m") = world.defaultN_to_m " Force arrow scaling (length = force/N_to_m)" annotation(Dialog(group = "if animation = true", enable = animation));
          parameter Real Nm_to_m(unit = "N.m/m") = world.defaultNm_to_m " Torque arrow scaling (length = torque/Nm_to_m)" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter forceDiameter = world.defaultArrowDiameter " Diameter of force arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter torqueDiameter = forceDiameter " Diameter of torque arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color forceColor = Modelica.Mechanics.MultiBody.Types.Defaults.ForceColor " Color of force arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color torqueColor = Modelica.Mechanics.MultiBody.Types.Defaults.TorqueColor " Color of torque arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(group = "if animation = true", enable = animation));
          annotation(defaultComponentName = "forceAndTorque", Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -63,56},{44,19}}, lineColor = {192,192,192}, textString = "resolve"),Text(extent = {{ -151, -73},{134, -134}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -100,60},{ -86,68},{ -80,72},{ -70,78},{ -64,82},{ -46,86},{ -34,88},{ -16,88},{ -2,86},{12,80},{24,74},{34,68},{46,58},{52,54},{58,48}}, color = {0,0,0}, thickness = 0.5),Polygon(points = {{89,17},{64,76},{30,41},{89,17}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{0,95},{0, -26}}, color = {95,95,95}, pattern = LinePattern.Dot),Line(points = {{0,0},{96,0}}, color = {95,95,95}, pattern = LinePattern.Dot),Polygon(points = {{ -104, -48},{54,0},{46,20},{96,0},{66, -42},{60, -22},{ -96, -72},{ -104, -48}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
<p>
The <b>3</b> signals of the <b>force</b> and <b>torque</b>
connector are interpreted
as the x-, y- and z-coordinates of a <b>force</b> and
<b>torque</b> acting at the frame
connector to which frame_b of this component is attached.
Via parameter <b>resolveInFrame</b> it is defined, in which frame these
coordinates shall be resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve input forceand torque in world frame (= default)</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve input force and torque in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve input force and torque in frame_resolve
                    (frame_resolve must be connected)</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameB.frame_resolve, the force and
torque coordinates
are with respect to the frame, that is connected to <b>frame_resolve</b>.
<p>

<p>
If force={100,0,0}, and for all parameters the default setting is used,
then the interpretation is that a force of 100 N is acting along the positive
x-axis of frame_b.
</p>

<p>
Conceptually, a force and torque acts on the world frame in such a way that
the force and torque balance between world.frame_b and frame_b is fulfilled.
For efficiency reasons, this reaction torque is, however, not computed.
</p>

<p>
The force and torque are by default visualized as an arrow (force)
and as a double arrow (torque) acting at the connector to which
they are connected. The diameters
and colors of the arrows can be defined via
variables <b>forceDiameter</b>, <b>torqueDiameter</b>,
<b>forceColor</b> and <b>torqueColor</b>. The arrows
point in the directions defined by the
force and torque vectors. The lengths of the arrows are proportional
to the length of the force and torque vectors, respectively, using parameters
<b>N_to_m</b> and <b>Nm_to_m</b> as scaling factors. For example, if N_to_m = 100 N/m,
then a force of 350 N is displayed as an arrow of length 3.5 m.
</p>
<p>
An example how to use this model is given in the
following figure:
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Forces/WorldForceAndTorque1.png\">
</p>
<p>
This leads to the following animation
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Forces/WorldForceAndTorque2.png\">
</p>
</HTML>
"));
          annotation(Coordsys(extent = [ -100, -100;100,100], grid = [1,1], component = [20,20]), Documentation(info = "
An external force element exerts the inport signal
as negative force on the connector frame (the force vector
is resolved in the world frame).
"), Icon(Text(extent = [ -132,99;128,39], string = "%name", lineColor = {0,0,255}), Polygon(points = [ -100,10;49,10;49,31;100,0;49, -31;49, -10; -100, -10; -100,10], style(color = 0, gradient = 0, fillColor = 0, fillPattern = 1), lineColor = {0,0,255})), Diagram(Polygon(points = [ -90,10;40,10;40,31;91,0;40, -31;40, -10; -90, -10; -90,10], style(color = 0, gradient = 0, fillColor = 0, fillPattern = 1), lineColor = {0,0,255})));
        protected 
          SI.Position f_in_m[3] = frame_b.f / N_to_m "Force mapped from N to m for animation";
          SI.Position t_in_m[3] = frame_b.t / Nm_to_m "Torque mapped from Nm to m for animation";
          Visualizers.Advanced.Arrow forceArrow(diameter = forceDiameter, color = forceColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = f_in_m, r_head =  -f_in_m) if world.enableAnimation and animation;
          Visualizers.Advanced.DoubleArrow torqueArrow(diameter = torqueDiameter, color = torqueColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = t_in_m, r_head =  -t_in_m) if world.enableAnimation and animation;
        public 
          Internal.BasicWorldForce basicWorldForce(resolveInFrame = resolveInFrame) annotation(Placement(transformation(extent = {{18, -50},{38, -70}})));
          Internal.BasicWorldTorque basicWorldTorque(resolveInFrame = resolveInFrame) annotation(Placement(transformation(extent = {{ -10,50},{10,70}})));
        protected 
          Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.frame_resolve annotation(Placement(transformation(extent = {{58,70},{78,90}})));

        equation 
          connect(basicWorldForce.frame_b,frame_b) annotation(Line(points = {{38, -60},{60, -60},{60,0},{100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicWorldForce.force,force) annotation(Line(points = {{16, -60},{ -120, -60}}, color = {0,0,127}, smooth = Smooth.None));
          connect(basicWorldTorque.frame_b,frame_b) annotation(Line(points = {{10,60},{60,60},{60,0},{100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicWorldTorque.torque,torque) annotation(Line(points = {{ -12,60},{ -120,60}}, color = {0,0,127}, smooth = Smooth.None));
          connect(basicWorldForce.frame_resolve,frame_resolve) annotation(Line(points = {{28, -50},{28,80},{0,80},{0,100}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(basicWorldTorque.frame_resolve,frame_resolve) annotation(Line(points = {{0,70},{0,100}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve,basicWorldTorque.frame_resolve) annotation(Line(points = {{58,80},{0,80},{0,70}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve,basicWorldForce.frame_resolve) annotation(Line(points = {{58,80},{40,80},{40, -40},{28, -40},{28, -50}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
        end WorldForceAndTorque;
        model Force "Force acting between two frames, defined by 3 input signals and resolved in frame world, frame_a, frame_b or frame_resolve"
          import SI = Modelica.SIunits;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
          Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve "The input signals are optionally resolved in this frame" annotation(Placement(transformation(origin = {40,100}, extent = {{ -16, -16},{16,16}}, rotation = 90)));
          Modelica.Blocks.Interfaces.RealInput force[3](each final quantity = "Force", each final unit = "N") "x-, y-, z-coordinates of force resolved in frame defined by resolveInFrame" annotation(Placement(transformation(origin = { -60,120}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_b "Frame in which input force is resolved (1: world, 2: frame_a, 3: frame_b, 4: frame_resolve)";
          parameter Real N_to_m(unit = "N/m") = world.defaultN_to_m " Force arrow scaling (length = force/N_to_m)" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter forceDiameter = world.defaultArrowDiameter " Diameter of force arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter connectionLineDiameter = forceDiameter " Diameter of line connecting frame_a and frame_b" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color forceColor = Modelica.Mechanics.MultiBody.Types.Defaults.ForceColor " Color of force arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color connectionLineColor = Modelica.Mechanics.MultiBody.Types.Defaults.SensorColor " Color of line connecting frame_a and frame_b" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(group = "if animation = true", enable = animation));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -98,99},{99, -98}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -92,61},{87,35}}, lineColor = {192,192,192}, textString = "resolve"),Text(extent = {{ -136, -52},{149, -113}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{40,100},{40,0}}, color = {95,95,95}, pattern = LinePattern.Dot),Polygon(points = {{ -94,0},{ -64,11},{ -64, -10},{ -94,0}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -60,100},{40,100}}, color = {95,95,95}, pattern = LinePattern.Dot),Polygon(points = {{94,0},{65,12},{65, -11},{94,0}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -64,0},{ -20,0}}, color = {0,0,0}),Line(points = {{20,0},{65,0}}, color = {0,0,0})}), Documentation(info = "<html>
<p>
The <b>3</b> signals of the <b>force</b> connector are interpreted
as the x-, y- and z-coordinates of a <b>force</b> acting at the frame
connector to which frame_b of this component is attached.
Via parameter <b>resolveInFrame</b> it is defined, in which frame these
coordinates shall be resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve input force in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve input force in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve input force in frame_b (= default)</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve input force in frame_resolve (frame_resolve must be connected)</td></tr>
</table>

<p>
If resolveInFrame = ResolveInFrameAB.frame_resolve, the force coordinates
are with respect to the frame, that is connected to <b>frame_resolve</b>.
<p>

<p>
If force={100,0,0}, and for all parameters the default setting is used,
then the interpretation is that a force of 100 N is acting along the positive
x-axis of frame_b.
</p>

<p>
Note, the cut-torque in frame_b (frame_b.t) is always set to zero.
Additionally, a force and torque acts on frame_a in such a way that
the force and torque balance between frame_a and frame_b is fulfilled.
</p>

<p>
An example how to use this model is given in the
following figure:
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Forces/Force1.png\">
</p>
<p>
This leads to the following animation (the yellow cylinder
characterizes the line between frame_a and frame_b of the
Force component, i.e., the force acts with negative sign
also on the opposite side of this cylinder, but for
clarity this is not shown in the animation):
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Forces/Force2.png\">
</p>
</html>
"));
        protected 
          SI.Position f_in_m[3] = frame_b.f / N_to_m "Force mapped from N to m for animation";
          Visualizers.Advanced.Arrow forceArrow(diameter = forceDiameter, color = forceColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = f_in_m, r_head =  -f_in_m) if world.enableAnimation and animation;
          Visualizers.Advanced.Shape connectionLine(shapeType = "cylinder", lengthDirection = basicForce.r_0, widthDirection = {0,1,0}, length = Modelica.Math.Vectors.length(basicForce.r_0), width = connectionLineDiameter, height = connectionLineDiameter, color = connectionLineColor, specularCoefficient = specularCoefficient, r = frame_a.r_0) if world.enableAnimation and animation;
        public 
          MultiBody.Forces.Internal.BasicForce basicForce(resolveInFrame = resolveInFrame) annotation(Placement(transformation(extent = {{0, -10},{20,10}})));
        protected 
          MultiBody.Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve annotation(Placement(transformation(extent = {{40,10},{60,30}})));

        equation 
          connect(basicForce.frame_a,frame_a) annotation(Line(points = {{0,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicForce.frame_b,frame_b) annotation(Line(points = {{20,0},{100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(force,basicForce.force) annotation(Line(points = {{ -60,120},{ -60,40},{4,40},{4,12}}, color = {0,0,127}, smooth = Smooth.None));
          connect(basicForce.frame_resolve,frame_resolve) annotation(Line(points = {{14,10},{14,40},{40,40},{40,100}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve,basicForce.frame_resolve) annotation(Line(points = {{40,20},{27,20},{27,10},{14,10}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
        end Force;
        model Torque "Torque acting between two frames, defined by 3 input signals and resolved in frame world, frame_a, frame_b or frame_resolve"
          import SI = Modelica.SIunits;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
          Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve "The input signals are optionally resolved in this frame" annotation(Placement(transformation(origin = {40,100}, extent = {{ -16, -16},{16,16}}, rotation = 90)));
          Modelica.Blocks.Interfaces.RealInput torque[3](each final quantity = "Torque", each final unit = "N.m") "x-, y-, z-coordiantes of torque resolved in frame defined by resolveInFrame" annotation(Placement(transformation(origin = { -60,120}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_b "Frame in which input force is resolved (1: world, 2: frame_a, 3: frame_b, 4: frame_resolve)";
          parameter Real Nm_to_m(unit = "N.m/m") = world.defaultNm_to_m " Torque arrow scaling (length = torque/Nm_to_m)" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter torqueDiameter = world.defaultArrowDiameter " Diameter of torque arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter connectionLineDiameter = torqueDiameter " Diameter of line connecting frame_a and frame_b" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color torqueColor = Modelica.Mechanics.MultiBody.Types.Defaults.TorqueColor " Color of torque arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color connectionLineColor = Modelica.Mechanics.MultiBody.Types.Defaults.SensorColor " Color of line connecting frame_a and frame_b" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(group = "if animation = true", enable = animation));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -98,99},{99, -98}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -59,55},{72,30}}, lineColor = {192,192,192}, textString = "resolve"),Text(extent = {{ -139, -27},{146, -88}}, textString = "%name", lineColor = {0,0,255}),Polygon(points = {{100,20},{84,52},{69,39},{100,20}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{40,100},{76,46}}, color = {95,95,95}, pattern = LinePattern.Dot),Polygon(points = {{ -99,20},{ -86,53},{ -70,42},{ -99,20}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -60,100},{40,100}}, color = {95,95,95}, pattern = LinePattern.Dot),Line(points = {{ -79,47},{ -70,61},{ -59,72},{ -45,81},{ -32,84},{ -20,85}}, color = {0,0,0}),Line(points = {{77,45},{66,60},{55,69},{49,74},{41,80},{31,84},{20,85}}, color = {0,0,0})}), Documentation(info = "<HTML>
<p>
The <b>3</b> signals of the <b>torque</b> connector are interpreted
as the x-, y- and z-coordinates of a <b>torque</b> acting at the frame
connector to which frame_b of this component is attached.
Via parameter <b>resolveInFrame</b> it is defined, in which frame these
coordinates shall be resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve input torque in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve input torque in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve input torque in frame_b (= default)</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve input torque in frame_resolve (frame_resolve must be connected)</td></tr>
</table>

<p>
If resolveInFrame = ResolveInFrameAB.frame_resolve, the torque coordinates
are with respect to the frame, that is connected to <b>frame_resolve</b>.
<p>

<p>
If torque={100,0,0}, and for all parameters the default setting is used,
then the interpretation is that a torque of 100 N.m is acting along the positive
x-axis of frame_b.
</p>

<p>
Note, the cut-forces in frame_a and frame_b (frame_a.f, frame_b.f) are
always set to zero and the cut-torque at frame_a (frame_a.t) is the same
as the cut-torque at frame_b (frame_b.t) but with opposite sign.
</p>

<p>
An example how to use this model is given in the
following figure:
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Forces/Torque1.png\">
</p>
<p>
This leads to the following animation (the yellow cylinder
characterizes the line between frame_a and frame_b of the
Torque component, i.e., the torque acts with negative sign
also on the opposite side of this cylinder, but for
clarity this is not shown in the animation):
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Forces/Torque2.png\">
</p>
</HTML>
"));
        protected 
          SI.Position t_in_m[3] = frame_b.t / Nm_to_m "Torque mapped from Nm to m for animation";
          Visualizers.Advanced.DoubleArrow torqueArrow(diameter = torqueDiameter, color = torqueColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = t_in_m, r_head =  -t_in_m) if world.enableAnimation and animation;
          Visualizers.Advanced.Shape connectionLine(shapeType = "cylinder", lengthDirection = basicTorque.r_0, widthDirection = {0,1,0}, length = Modelica.Math.Vectors.length(basicTorque.r_0), width = connectionLineDiameter, height = connectionLineDiameter, color = connectionLineColor, specularCoefficient = specularCoefficient, r = frame_a.r_0) if world.enableAnimation and animation;
        public 
          Internal.BasicTorque basicTorque(resolveInFrame = resolveInFrame) annotation(Placement(transformation(extent = {{ -8, -10},{12,10}})));
        protected 
          Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve annotation(Placement(transformation(extent = {{34,10},{54,30}})));

        equation 
          connect(basicTorque.frame_a,frame_a) annotation(Line(points = {{ -8,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicTorque.frame_b,frame_b) annotation(Line(points = {{12,0},{100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicTorque.torque,torque) annotation(Line(points = {{ -4,12},{ -4,60},{ -60,60},{ -60,120}}, color = {0,0,127}, smooth = Smooth.None));
          connect(basicTorque.frame_resolve,frame_resolve) annotation(Line(points = {{6,10},{6,60},{40,60},{40,100}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve,basicTorque.frame_resolve) annotation(Line(points = {{34,20},{20,20},{20,10},{6,10}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
        end Torque;
        model ForceAndTorque "Force and torque acting between two frames, defined by 3+3 input signals and resolved in frame world, frame_a, frame_b or frame_resolve"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Types;
          extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
          Blocks.Interfaces.RealInput force[3](each final quantity = "Force", each final unit = "N") "x-, y-, z-coordinates of force resolved in frame defined by resolveInFrame" annotation(Placement(transformation(origin = { -80,120}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
          Blocks.Interfaces.RealInput torque[3](each final quantity = "Torque", each final unit = "N.m") "x-, y-, z-coordiantes of torque resolved in frame defined by resolveInFrame" annotation(Placement(transformation(origin = {0,120}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
          Interfaces.Frame_resolve frame_resolve if resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve "The input signals are optionally resolved in this frame" annotation(Placement(transformation(origin = {80,100}, extent = {{ -16, -16},{16,16}}, rotation = 90)));
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_b "Frame in which input force and torque are resolved (1: world, 2: frame_a, 3: frame_b, 4: frame_resolve)";
          parameter Real N_to_m(unit = "N/m") = world.defaultN_to_m "Force arrow scaling (length = force/N_to_m)" annotation(Dialog(group = "if animation = true", enable = animation));
          parameter Real Nm_to_m(unit = "N.m/m") = world.defaultNm_to_m "Torque arrow scaling (length = torque/Nm_to_m)" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter forceDiameter = world.defaultArrowDiameter "Diameter of force arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter torqueDiameter = forceDiameter " Diameter of torque arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input SI.Diameter connectionLineDiameter = forceDiameter "Diameter of line connecting frame_a and frame_b" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color forceColor = Modelica.Mechanics.MultiBody.Types.Defaults.ForceColor "Color of force arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color torqueColor = Modelica.Mechanics.MultiBody.Types.Defaults.TorqueColor "Color of torque arrow" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.Color connectionLineColor = Modelica.Mechanics.MultiBody.Types.Defaults.SensorColor "Color of line connecting frame_a and frame_b" annotation(Dialog(group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(group = "if animation = true", enable = animation));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -98,99},{99, -98}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -59,55},{72,30}}, lineColor = {192,192,192}, textString = "resolve"),Text(extent = {{ -136, -52},{149, -113}}, textString = "%name", lineColor = {0,0,255}),Polygon(points = {{100,21},{84,55},{69,39},{100,21}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{80,100},{80,0}}, color = {95,95,95}, pattern = LinePattern.Dot),Polygon(points = {{ -95,1},{ -64,11},{ -64, -10},{ -95,1}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{ -100,20},{ -86,53},{ -70,42},{ -100,20}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -80,100},{80,100}}, color = {95,95,95}, pattern = LinePattern.Dot),Polygon(points = {{94,0},{65,12},{65, -11},{94,0}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -64,0},{ -20,0}}, color = {0,0,0}),Line(points = {{20,0},{65,0}}, color = {0,0,0}),Line(points = {{ -79,47},{ -70,61},{ -59,72},{ -45,81},{ -32,84},{ -20,85}}, color = {0,0,0}),Line(points = {{76,47},{66,60},{55,69},{49,74},{41,80},{31,84},{20,85}}, color = {0,0,0}),Text(extent = {{ -144,124},{ -106,102}}, lineColor = {0,0,0}, textString = "f"),Text(extent = {{20,124},{58,102}}, lineColor = {0,0,0}, textString = "t")}), Documentation(info = "<HTML>
The <b>3</b> signals of the <b>force</b> connector and the
<b>3</b> signals of the <b>torque</b> connector
are interpreted
as the x-, y- and z-coordinates of a <b>force</b> and of a
<b>torque</b> acting at the frame
connector to which frame_b of this component is attached.
Via parameter <b>resolveInFrame</b> it is defined, in which frame these
coordinates shall be resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve input force/torque in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve input force/torque in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve input force/torque in frame_b (= default)</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve input force/torque in frame_resolve (frame_resolve must be connected)</td></tr>
</table>

<p>
If resolveInFrame = ResolveInFrameAB.frame_resolve, the force and torque coordinates
are with respect to the frame, that is connected to <b>frame_resolve</b>.
<p>

<p>
If force={100,0,0}, and for all parameters the default setting is used,
then the interpretation is that a force of 100 N is acting along the positive
x-axis of frame_b.
</p>

<p>
Note, a force and torque acts on frame_a in such a way that
the force and torque balance between frame_a and frame_b is fulfilled.
</p>

<p>
An example how to use this model is given in the
following figure:
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Forces/ForceAndTorque1.png\">
</p>
<p>
This leads to the following animation (the yellow cylinder
characterizes the line between frame_a and frame_b of the
ForceAndTorque component, i.e., the force and torque acts with
negative sign
also on the opposite side of this cylinder, but for
clarity this is not shown in the animation):
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Forces/ForceAndTorque2.png\">
</p>
</HTML>
"));
        protected 
          SI.Position f_in_m[3] = frame_b.f / N_to_m "Force mapped from N to m for animation";
          SI.Position t_in_m[3] = frame_b.t / Nm_to_m "Torque mapped from Nm to m for animation";
          Visualizers.Advanced.Arrow forceArrow(diameter = forceDiameter, color = forceColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = f_in_m, r_head =  -f_in_m) if world.enableAnimation and animation;
          Visualizers.Advanced.DoubleArrow torqueArrow(diameter = torqueDiameter, color = torqueColor, specularCoefficient = specularCoefficient, R = frame_b.R, r = frame_b.r_0, r_tail = t_in_m, r_head =  -t_in_m) if world.enableAnimation and animation;
          Visualizers.Advanced.Shape connectionLine(shapeType = "cylinder", lengthDirection = basicForce.r_0, widthDirection = {0,1,0}, length = Modelica.Math.Vectors.length(basicForce.r_0), width = connectionLineDiameter, height = connectionLineDiameter, color = connectionLineColor, specularCoefficient = specularCoefficient, r = frame_a.r_0) if world.enableAnimation and animation;
        public 
          Internal.BasicForce basicForce(resolveInFrame = resolveInFrame) annotation(Placement(transformation(extent = {{ -84, -10},{ -64,10}})));
          Internal.BasicTorque basicTorque(resolveInFrame = resolveInFrame) annotation(Placement(transformation(extent = {{ -4,10},{16,30}})));
        protected 
          Interfaces.ZeroPosition zeroPosition if not resolveInFrame == Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_resolve annotation(Placement(transformation(extent = {{20,30},{40,50}})));

        equation 
          connect(basicForce.frame_a,frame_a) annotation(Line(points = {{ -84,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicForce.frame_b,frame_b) annotation(Line(points = {{ -64,0},{100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicTorque.frame_b,frame_b) annotation(Line(points = {{16,20},{68,20},{68,0},{100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicTorque.frame_a,frame_a) annotation(Line(points = {{ -4,20},{ -90,20},{ -90,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5, smooth = Smooth.None));
          connect(basicForce.force,force) annotation(Line(points = {{ -80,12},{ -80,120}}, color = {0,0,127}, smooth = Smooth.None));
          connect(basicTorque.torque,torque) annotation(Line(points = {{0,32},{0,120}}, color = {0,0,127}, smooth = Smooth.None));
          connect(basicTorque.frame_resolve,frame_resolve) annotation(Line(points = {{10,30},{10,80},{80,80},{80,100}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(basicForce.frame_resolve,frame_resolve) annotation(Line(points = {{ -70,10},{ -70,80},{80,80},{80,100}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve,basicTorque.frame_resolve) annotation(Line(points = {{20,40},{10,40},{10,30}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
          connect(zeroPosition.frame_resolve,basicForce.frame_resolve) annotation(Line(points = {{20,40},{ -70,40},{ -70,10}}, color = {95,95,95}, pattern = LinePattern.Dot, smooth = Smooth.None));
        end ForceAndTorque;
        model LineForceWithMass "General line force component with an optional point mass on the connection line"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Types;
          extends Interfaces.PartialTwoFrames;
          Modelica.Mechanics.Translational.Interfaces.Flange_a flange_b "1-dim. translational flange (connect force of Translational library between flange_a and flange_b)" annotation(Placement(transformation(origin = {60,100}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Mechanics.Translational.Interfaces.Flange_b flange_a "1-dim. translational flange (connect force of Translational library between flange_a and flange_b)" annotation(Placement(transformation(origin = { -60,100}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          parameter Boolean animateLine = true "= true, if a line shape between frame_a and frame_b shall be visualized";
          parameter Boolean animateMass = true "= true, if point mass shall be visualized as sphere provided m > 0";
          parameter SI.Mass m(min = 0) = 0 "Mass of point mass on the connetion line between the origin of frame_a and the origin of frame_b";
          parameter Real lengthFraction(unit = "1", min = 0, max = 1) = 0.5 "Location of point mass with respect to frame_a as a fraction of the distance from frame_a to frame_b";
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation"));
          parameter Types.ShapeType lineShapeType = "cylinder" " Type of shape visualizing the line from frame_a to frame_b" annotation(Dialog(tab = "Animation", group = "if animateLine = true", enable = animateLine));
          input SI.Length lineShapeWidth = world.defaultArrowDiameter " Width of shape" annotation(Dialog(tab = "Animation", group = "if animateLine = true", enable = animateLine));
          input SI.Length lineShapeHeight = lineShapeWidth " Height of shape" annotation(Dialog(tab = "Animation", group = "if animateLine = true", enable = animateLine));
          parameter Types.ShapeExtra lineShapeExtra = 0.0 " Extra parameter for shape" annotation(Dialog(tab = "Animation", group = "if animateLine = true", enable = animateLine));
          input Types.Color lineShapeColor = Modelica.Mechanics.MultiBody.Types.Defaults.SensorColor " Color of line shape" annotation(Dialog(tab = "Animation", group = "if animateLine = true", enable = animateLine));
          input Real massDiameter = world.defaultBodyDiameter " Diameter of point mass sphere" annotation(Dialog(tab = "Animation", group = "if animateMass = true", enable = animateMass));
          input Types.Color massColor = Modelica.Mechanics.MultiBody.Types.Defaults.BodyColor " Color of point mass" annotation(Dialog(tab = "Animation", group = "if animateMass = true", enable = animateMass));
          parameter SI.Position s_small = 1e-10 " Prevent zero-division if distance between frame_a and frame_b is zero" annotation(Dialog(tab = "Advanced"));
          SI.Distance length "Distance between the origin of frame_a and the origin of frame_b";
          SI.Position r_rel_0[3] "Position vector from frame_a to frame_b resolved in world frame";
          Real e_rel_0[3](each final unit = "1") "Unit vector in direction from frame_a to frame_b, resolved in world frame";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -95, -40},{ -15,40}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -85, -30},{ -25,30}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{15, -40},{95,40}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{23, -30},{83,29}}, lineColor = {128,128,128}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -145, -53},{145, -113}}, textString = "%name", lineColor = {0,0,255}),Rectangle(extent = {{ -40,41},{44, -40}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -70,15},{ -41, -13}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{40,14},{69, -14}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Line(points = {{ -56,0},{ -56,23},{ -30,23},{ -30,70},{ -60,70},{ -60,101}}, color = {0,0,0}),Line(points = {{55, -1},{55,20},{30,20},{30,70},{60,70},{60,100}}, color = {0,0,0}),Line(points = {{ -56,0},{55, -1}}, color = {0,0,0}, pattern = LinePattern.Dot),Ellipse(extent = {{ -8,8},{8, -8}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -60,80},{46,80}}, color = {0,0,255}),Polygon(points = {{60,80},{45,86},{45,74},{60,80}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -22,100},{20,76}}, textString = "length", lineColor = {0,0,255}),Ellipse(extent = {{ -100, -40},{ -20,40}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -90, -30},{ -30,30}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{20, -40},{100,40}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{31, -29},{91,30}}, lineColor = {128,128,128}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -50,39},{50, -41}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -74,15},{ -45, -13}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{45,15},{74, -13}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Line(points = {{ -60,0},{ -60,24},{ -40,24},{ -40,60},{ -60,60},{ -60,100}}, color = {0,0,0}),Line(points = {{60,1},{60,21},{40,21},{40,60},{60,60},{60,100}}, color = {0,0,0}),Line(points = {{ -60,0},{60,0}}, color = {0,0,0}, pattern = LinePattern.Dot),Ellipse(extent = {{ -8,8},{8, -8}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -60,0},{ -31,0}}, color = {0,0,255}),Polygon(points = {{ -19,0},{ -31,3},{ -31, -3},{ -19,0}}, lineColor = {0,0,255}),Line(points = {{ -60,16},{0,16}}, color = {0,0,255}),Line(points = {{0,0},{0,20}}, color = {0,0,255}),Text(extent = {{ -43, -8},{ -7, -33}}, lineColor = {0,0,0}, textString = "e_rel_0"),Polygon(points = {{0,16},{ -12,19},{ -12,13},{0,16}}, lineColor = {0,0,255}),Text(extent = {{ -50,35},{51,26}}, lineColor = {0,0,0}, textString = "length*lengthFraction"),Line(points = {{ -17,26},{ -26,16}}, pattern = LinePattern.Dot, color = {0,0,255}),Line(points = {{ -31, -13},{ -40,0}}, pattern = LinePattern.Dot, color = {0,0,255})}), Documentation(info = "<html>
<p>
This component is used to exert a <b>line force</b>
between the origin of frame_a and the origin of frame_b
by attaching components of the <b>1-dimensional translational</b>
mechanical library of Modelica (Modelica.Mechanics.Translational)
between the two flange connectors <b>flange_a</b> and
<b>flange_b</b>. Optionally, there is a <b>point mass</b> on the line
connecting the origin of frame_a and the origin of frame_b.
This point mass approximates the <b>mass</b> of the <b>force element</b>.
The distance of the point mass from frame_a as a fraction of the
distance between frame_a and frame_b is defined via
parameter <b>lengthFraction</b> (default is 0.5, i.e., the point
mass is in the middle of the line).
</p>
<p>
In the translational library there is the implicit assumption that
forces of components that have only one flange connector act with
opposite sign on the bearings of the component. This assumption
is also used in the LineForceWithMass component: If a connection
is present to only one of the flange connectors, then the force
in this flange connector acts implicitly with opposite sign also
in the other flange connector.
</p>
</html>"));
        protected 
          SI.Force fa "Force from flange_a";
          SI.Force fb "Force from flange_b";
          SI.Position r_CM_0[3](stateSelect = StateSelect.avoid) "Position vector from world frame to point mass, resolved in world frame";
          SI.Velocity v_CM_0[3](stateSelect = StateSelect.avoid) "First derivative of r_CM_0";
          SI.Acceleration ag_CM_0[3] "der(v_CM_0) - gravityAcceleration";
          Visualizers.Advanced.Shape lineShape(shapeType = lineShapeType, color = lineShapeColor, specularCoefficient = specularCoefficient, length = length, width = lineShapeWidth, height = lineShapeHeight, lengthDirection = e_rel_0, widthDirection = Frames.resolve1(frame_a.R, {0,1,0}), extra = lineShapeExtra, r = frame_a.r_0) if world.enableAnimation and animateLine;
          Visualizers.Advanced.Shape massShape(shapeType = "sphere", color = massColor, specularCoefficient = specularCoefficient, length = massDiameter, width = massDiameter, height = massDiameter, lengthDirection = e_rel_0, widthDirection = {0,1,0}, r_shape = e_rel_0 * (length * lengthFraction - massDiameter / 2), r = frame_a.r_0) if world.enableAnimation and animateMass and m > 0;

        equation 
          Connections.potentialRoot(frame_a.R, 100);
          Connections.potentialRoot(frame_b.R, 100);
          assert(noEvent(length > s_small), "
The distance between the origin of frame_a and the origin of frame_b
of a LineForceWithMass component became smaller as parameter s_small
(= a small number, defined in the \"Advanced\" menu). The distance is
set to s_small, although it is smaller, to avoid a division by zero
when computing the direction of the line force. Possible reasons
for this situation:
- At initial time the distance may already be zero: Change the initial
  positions of the bodies connected by this element.
- Hardware stops are not modeled or are modeled not stiff enough.
  Include stops, e.g., stiff springs, or increase the stiffness
  if already present.
- Another error in your model may lead to unrealistically large forces
  and torques that would in reality destroy the stops.
- The flange_b connector might be defined by a pre-defined motion,
  e.g., with Modelica.Mechanics.Translational.Position and the
  predefined flange_b.s is zero or negative.
");
          r_rel_0 = frame_b.r_0 - frame_a.r_0;
          length = Modelica.Math.Vectors.length(r_rel_0);
          flange_a.s = 0;
          flange_b.s = length;
          e_rel_0 = r_rel_0 / Frames.Internal.maxWithoutEvent(length, s_small);
          if cardinality(flange_a) > 0 and cardinality(flange_b) > 0 then
            fa = flange_a.f;
            fb = flange_b.f;

          elseif cardinality(flange_a) > 0 and cardinality(flange_b) == 0 then
            fa = flange_a.f;
            fb =  -fa;


          elseif cardinality(flange_a) == 0 and cardinality(flange_b) > 0 then
            fa =  -fb;
            fb = flange_b.f;


          else
            fa = 0;
            fb = 0;

          end if;
          if m > 0 then
            r_CM_0 = frame_a.r_0 + r_rel_0 * lengthFraction;
            v_CM_0 = der(r_CM_0);
            ag_CM_0 = der(v_CM_0) - world.gravityAcceleration(r_CM_0);
            frame_a.f = Frames.resolve2(frame_a.R, m * (1 - lengthFraction) * ag_CM_0 - e_rel_0 * fa);
            frame_b.f = Frames.resolve2(frame_b.R, m * lengthFraction * ag_CM_0 - e_rel_0 * fb);

          else
            r_CM_0 = zeros(3);
            v_CM_0 = zeros(3);
            ag_CM_0 = zeros(3);
            frame_a.f =  -Frames.resolve2(frame_a.R, e_rel_0 * fa);
            frame_b.f =  -Frames.resolve2(frame_b.R, e_rel_0 * fb);

          end if;
          if Connections.isRoot(frame_a.R) then
            frame_a.R = Frames.nullRotation();

          else
            frame_a.t = zeros(3);

          end if;
          if Connections.isRoot(frame_b.R) then
            frame_b.R = Frames.nullRotation();

          else
            frame_b.t = zeros(3);

          end if;
        end LineForceWithMass;
        model LineForceWithTwoMasses "General line force component with two optional point masses on the connection line"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Types;
          extends Interfaces.PartialTwoFrames;
          Modelica.Mechanics.Translational.Interfaces.Flange_a flange_b "1-dim. translational flange (connect force of Translational library between flange_a and flange_b)" annotation(Placement(transformation(origin = {60,110}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Mechanics.Translational.Interfaces.Flange_b flange_a "1-dim. translational flange (connect force of Translational library between flange_a and flange_b)" annotation(Placement(transformation(origin = { -60,110}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          parameter Boolean animate = true "= true, if animation shall be enabled";
          parameter Boolean animateMasses = true "= true, if point masses shall be visualized provided animate=true and m_a, m_b > 0";
          parameter SI.Mass m_a(min = 0) = 0 "Mass of point mass a on the connetion line between the origin of frame_a and the origin of frame_b";
          parameter SI.Mass m_b(min = 0) = 0 "Mass of point mass b on the connetion line between the origin of frame_a and the origin of frame_b";
          parameter SI.Position L_a = 0 "Distance between point mass a and frame_a (positive, if in direction of frame_b)";
          parameter SI.Position L_b = L_a "Distance between point mass b and frame_b (positive, if in direction of frame_a)";
          input SI.Diameter cylinderDiameter_a = world.defaultForceWidth " Diameter of cylinder at frame_a" annotation(Dialog(tab = "Animation", group = "Cylinder at frame_a if animation = true", enable = animate));
          parameter SI.Length cylinderLength_a = 2 * L_a " Length of cylinder at frame_a" annotation(Dialog(tab = "Animation", group = "Cylinder at frame_a if animation = true", enable = animate));
          input Types.Color color_a = {155,155,155} " Color of cylinder at frame_a" annotation(Dialog(tab = "Animation", group = "Cylinder at frame_a if animation = true", enable = animate));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "Cylinder at frame_a if animation = true", enable = animate));
          input Real diameterFraction = 0.8 " Diameter of cylinder at frame_b with respect to diameter of cylinder at frame_a" annotation(Dialog(tab = "Animation", group = "Cylinder at frame_b if animation = true", enable = animate));
          parameter SI.Length cylinderLength_b = 2 * L_b " Length of cylinder at frame_b" annotation(Dialog(tab = "Animation", group = "Cylinder at frame_b if animation = true", enable = animate));
          input Types.Color color_b = {100,100,100} " Color of cylinder at frame_b" annotation(Dialog(tab = "Animation", group = "Cylinder at frame_b if animation = true", enable = animate));
          input Real massDiameterFaction = 1.7 " Diameter of point mass spheres with respect to cylinderDiameter_a" annotation(Dialog(tab = "Animation", group = "if animation = true and animateMasses = true", enable = animate and animateMasses));
          input Types.Color massColor = Modelica.Mechanics.MultiBody.Types.Defaults.BodyColor " Color of point masses" annotation(Dialog(tab = "Animation", group = "if animation = true and animateMasses = true", enable = animate and animateMasses));
          parameter SI.Position s_small = 1e-10 " Prevent zero-division if distance between frame_a and frame_b is zero" annotation(Dialog(tab = "Advanced"));
          SI.Distance length "Distance between the origin of frame_a and the origin of frame_b";
          SI.Position r_rel_0[3] "Position vector from frame_a to frame_b resolved in world frame";
          Real e_rel_0[3](each final unit = "1") "Unit vector in direction from frame_a to frame_b, resolved in world frame";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -100, -40},{ -20,40}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -90, -30},{ -30,30}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{20, -40},{100,40}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{31, -29},{91,30}}, lineColor = {128,128,128}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -145, -53},{145, -113}}, textString = "%name", lineColor = {0,0,255}),Rectangle(extent = {{ -52,40},{48, -40}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -74,15},{ -45, -13}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{45,14},{74, -14}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Line(points = {{ -60,0},{ -60,23},{ -30,23},{ -30,70},{ -60,70},{ -60,101}}, color = {0,0,0}),Line(points = {{60,0},{60,20},{30,20},{30,70},{60,70},{60,100}}, color = {0,0,0}),Line(points = {{ -23,0},{25,0}}, color = {0,0,0}, pattern = LinePattern.Dot),Ellipse(extent = {{23,8},{39, -8}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -39,8},{ -23, -8}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -60,0},{ -29,0}}, color = {0,0,0}),Line(points = {{29,0},{60,0}}, color = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -60,80},{46,80}}, color = {0,0,255}),Polygon(points = {{60,80},{45,86},{45,74},{60,80}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -22,100},{20,76}}, textString = "length", lineColor = {0,0,255}),Ellipse(extent = {{ -100, -40},{ -20,40}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{ -90, -30},{ -30,30}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{20, -40},{100,40}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{31, -29},{91,30}}, lineColor = {128,128,128}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -49,39},{51, -41}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -74,15},{ -45, -13}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Ellipse(extent = {{45,15},{74, -13}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Line(points = {{ -60,0},{ -60,24},{ -40,24},{ -40,60},{ -60,60},{ -60,110}}, color = {0,0,0}),Line(points = {{60,1},{60,21},{40,21},{40,60},{60,60},{60,110}}, color = {0,0,0}),Line(points = {{ -60,0},{60,0}}, color = {0,0,0}, pattern = LinePattern.Dot),Ellipse(extent = {{20,8},{36, -8}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -18, -18},{11, -18}}, color = {0,0,255}),Polygon(points = {{23, -18},{11, -15},{11, -21},{23, -18}}, lineColor = {0,0,255}),Line(points = {{ -60,16},{ -37,16}}, color = {0,0,255}),Line(points = {{ -25,0},{ -25,20}}, color = {0,0,255}),Text(extent = {{ -16, -19},{20, -44}}, lineColor = {0,0,0}, textString = "e_rel_0"),Polygon(points = {{ -25,16},{ -37,19},{ -37,13},{ -25,16}}, lineColor = {0,0,255}),Text(extent = {{ -39,31},{ -22,21}}, lineColor = {0,0,0}, textString = "L_a"),Ellipse(extent = {{ -33,7},{ -17, -9}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{29,3},{29,22}}, color = {0,0,255}),Line(points = {{29,16},{60,16}}, color = {0,0,255}),Polygon(points = {{29,16},{41,19},{41,13},{29,16}}, lineColor = {0,0,255}),Text(extent = {{15,36},{32,26}}, lineColor = {0,0,0}, textString = "L_b"),Line(points = {{37,18},{30,27}}, pattern = LinePattern.Dot, color = {0,0,255})}), Documentation(info = "<html>
<p>
This component is used to exert a <b>line force</b>
between the origin of frame_a and the origin of frame_b
by attaching components of the <b>1-dimensional translational</b>
mechanical library of Modelica (Modelica.Mechanics.Translational)
between the two flange connectors <b>flange_a</b> and
<b>flange_b</b>. Optionally, there are <b>two point masses</b> on the line
connecting the origin of frame_a and the origin of frame_b.
These point masses approximate the <b>masses</b> of the <b>force element</b>.
The locations of the two point masses are defined by their
(fixed) distances of L_a relative to frame_a and of L_b relative
to frame_b, respectively.
</p>
<p>
In example
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Examples.Elementary.LineForceWithTwoMasses\">
MultiBody.Examples.Elementary.LineForceWithTwoMasses</a> the usage of this
line force element is shown and is compared with an alternative
implementation using a
<a href=\"Modelica://Modelica.Mechanics.MultiBody.Joints.Assemblies.JointUPS\">
MultiBody.Joints.Assemblies.JointUPS</a> component.
The composition diagram of this example
is displayed in the figure below.
</p>
<IMG SRC=\"../Images/MultiBody/Examples/Elementary/LineForceWithTwoMasses1.png\">
<p>
The animation view at time = 0 is shown in the next figure.
The system on the left side in the front is the animation with
the LineForceWithTwoMasses component whereas the system on the right
side in the back is the animation with the JointUPS component.
Both implementations yield the same result. However, the implementation
with the LineForceWithTwoMasses component is simpler.
</p>
<IMG SRC=\"../Images/MultiBody/Examples/Elementary/LineForceWithTwoMasses2.png\">
<p>
In the translational library there is the implicit assumption that
forces of components that have only one flange connector act with
opposite sign on the bearings of the component. This assumption
is also used in the LineForceWithTwoMasses component: If a connection
is present to only one of the flange connectors, then the force
in this flange connector acts implicitly with opposite sign also
in the other flange connector.
</p>
</html>"));
        protected 
          SI.Force fa "Force from flange_a";
          SI.Force fb "Force from flange_b";
          SI.Position r_CM1_0[3](stateSelect = StateSelect.avoid) "Position vector from world frame to point mass 1, resolved in world frame";
          SI.Position r_CM2_0[3](stateSelect = StateSelect.avoid) "Position vector from world frame to point mass 2, resolved in world frame";
          SI.Velocity v_CM1_0[3](stateSelect = StateSelect.avoid) "der(r_CM_1_0) - velocity of point mass 1";
          SI.Velocity v_CM2_0[3](stateSelect = StateSelect.avoid) "der(r_CM_2_0) - velocity of point mass 2";
          SI.Acceleration ag_CM1_0[3] "der(v_CM1_0) - gravityAcceleration(r_CM1_0)";
          SI.Acceleration ag_CM2_0[3] "der(v_CM2_0) - gravityAcceleration(r_CM2_0)";
          SI.Force aux1_0[3] "Auxiliary force 1";
          SI.Force aux2_0[3] "Auxiliary force 2";
          input SI.Length cylinderDiameter_b = cylinderDiameter_a * diameterFraction;
          input SI.Length massDiameter = cylinderDiameter_a * massDiameterFaction;
          parameter Boolean animateMasses2 = world.enableAnimation and animate and animateMasses and m_a > 0 and m_b > 0;
          Visualizers.Advanced.Shape cylinder_a(shapeType = "cylinder", color = color_a, specularCoefficient = specularCoefficient, length = cylinderLength_a, width = cylinderDiameter_a, height = cylinderDiameter_a, lengthDirection = e_rel_0, widthDirection = {0,1,0}, r = frame_a.r_0) if world.enableAnimation and animate;
          Visualizers.Advanced.Shape cylinder_b(shapeType = "cylinder", color = color_b, specularCoefficient = specularCoefficient, length = cylinderLength_b, width = cylinderDiameter_b, height = cylinderDiameter_b, lengthDirection =  -e_rel_0, widthDirection = {0,1,0}, r = frame_b.r_0) if world.enableAnimation and animate;
          Visualizers.Advanced.Shape sphere_a(shapeType = "sphere", color = massColor, specularCoefficient = specularCoefficient, length = massDiameter, width = massDiameter, height = massDiameter, lengthDirection = e_rel_0, widthDirection = {0,1,0}, r_shape = e_rel_0 * (L_a - massDiameter / 2), r = frame_a.r_0) if animateMasses2;
          Visualizers.Advanced.Shape sphere_b(shapeType = "sphere", color = massColor, specularCoefficient = specularCoefficient, length = massDiameter, width = massDiameter, height = massDiameter, lengthDirection =  -e_rel_0, widthDirection = {0,1,0}, r_shape =  -e_rel_0 * (L_b - massDiameter / 2), r = frame_b.r_0) if animateMasses2;

        equation 
          Connections.potentialRoot(frame_a.R, 100);
          Connections.potentialRoot(frame_b.R, 100);
          assert(noEvent(length > s_small), "
The distance between the origin of frame_a and the origin of frame_b
of a LineForceWithTwoMasses component became smaller as parameter s_small
(= a small number, defined in the \"Advanced\" menu). The distance is
set to s_small, although it is smaller, to avoid a division by zero
when computing the direction of the line force. Possible reasons
for this situation:
- At initial time the distance may already be zero: Change the initial
  positions of the bodies connected by this element.
- Hardware stops are not modeled or are modeled not stiff enough.
  Include stops, e.g., stiff springs, or increase the stiffness
  if already present.
- Another error in your model may lead to unrealistically large forces
  and torques that would in reality destroy the stops.
- The flange_b connector might be defined by a pre-defined motion,
  e.g., with Modelica.Mechanics.Translational.Position and the
  predefined flange_b.s is zero or negative.
");
          r_rel_0 = frame_b.r_0 - frame_a.r_0;
          length = Modelica.Math.Vectors.length(r_rel_0);
          flange_a.s = 0;
          flange_b.s = length;
          e_rel_0 = r_rel_0 / Frames.Internal.maxWithoutEvent(length, s_small);
          if cardinality(flange_a) > 0 and cardinality(flange_b) > 0 then
            fa = flange_a.f;
            fb = flange_b.f;

          elseif cardinality(flange_a) > 0 and cardinality(flange_b) == 0 then
            fa = flange_a.f;
            fb =  -fa;


          elseif cardinality(flange_a) == 0 and cardinality(flange_b) > 0 then
            fa =  -fb;
            fb = flange_b.f;


          else
            fa = 0;
            fb = 0;

          end if;
          if m_a > 0 or m_b > 0 then
            r_CM1_0 = frame_a.r_0 + e_rel_0 * L_a;
            r_CM2_0 = frame_b.r_0 - e_rel_0 * L_b;
            v_CM1_0 = der(r_CM1_0);
            v_CM2_0 = der(r_CM2_0);
            ag_CM1_0 = der(v_CM1_0) - world.gravityAcceleration(r_CM1_0);
            ag_CM2_0 = der(v_CM2_0) - world.gravityAcceleration(r_CM2_0);
            aux1_0 = ag_CM1_0 * (m_a * L_a) / length - ag_CM2_0 * (m_b * L_b) / length;
            aux2_0 = aux1_0 - aux1_0 * e_rel_0 * e_rel_0;
            frame_a.f = Frames.resolve2(frame_a.R, m_a * ag_CM1_0 - aux2_0 - e_rel_0 * fa);
            frame_b.f = Frames.resolve2(frame_b.R, m_b * ag_CM2_0 + aux2_0 - e_rel_0 * fb);

          else
            r_CM1_0 = zeros(3);
            r_CM2_0 = zeros(3);
            v_CM1_0 = zeros(3);
            v_CM2_0 = zeros(3);
            ag_CM1_0 = zeros(3);
            ag_CM2_0 = zeros(3);
            aux1_0 = zeros(3);
            aux2_0 = zeros(3);
            frame_a.f =  -Frames.resolve2(frame_a.R, e_rel_0 * fa);
            frame_b.f =  -Frames.resolve2(frame_b.R, e_rel_0 * fb);

          end if;
          if Connections.isRoot(frame_a.R) then
            frame_a.R = Frames.nullRotation();

          else
            frame_a.t = zeros(3);

          end if;
          if Connections.isRoot(frame_b.R) then
            frame_b.R = Frames.nullRotation();

          else
            frame_b.t = zeros(3);

          end if;
        end LineForceWithTwoMasses;
        model Spring "Linear translational spring with optional mass"
          import Modelica.Mechanics.MultiBody.Types;
          extends Interfaces.PartialTwoFrames;
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter Boolean showMass = true "= true, if point mass shall be visualized as sphere if animation=true and m>0";
          parameter SI.TranslationalSpringConstant c(final min = 0) "Spring constant";
          parameter SI.Length s_unstretched = 0 "Unstretched spring length";
          parameter SI.Mass m(min = 0) = 0 "Spring mass located on the connetion line between the origin of frame_a and the origin of frame_b";
          parameter Real lengthFraction(min = 0, max = 1) = 0.5 "Location of spring mass with respect to frame_a as a fraction of the distance from frame_a to frame_b (=0: at frame_a; =1: at frame_b)";
          input SI.Distance width = world.defaultForceWidth " Width of spring" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input SI.Distance coilWidth = width / 10 " Width of spring coil" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Integer numberOfWindings = 5 " Number of spring windings" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.SpringColor " Color of spring" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input SIunits.Diameter massDiameter = max(0, (width - 2 * coilWidth) * 0.9) " Diameter of mass point sphere" annotation(Dialog(tab = "Animation", group = "if animation = true and showMass = true", enable = animation and showMass));
          input Types.Color massColor = Modelica.Mechanics.MultiBody.Types.Defaults.BodyColor " Color of mass point" annotation(Dialog(tab = "Animation", group = "if animation = true and showMass = true", enable = animation and showMass));
          Forces.LineForceWithMass lineForce(animateLine = animation, animateMass = showMass, m = m, lengthFraction = lengthFraction, lineShapeType = "spring", lineShapeHeight = coilWidth * 2, lineShapeWidth = width, lineShapeExtra = numberOfWindings, lineShapeColor = color, specularCoefficient = specularCoefficient, massDiameter = massDiameter, massColor = massColor) annotation(Placement(transformation(extent = {{ -20, -20},{20,20}}, rotation = 0)));
          Modelica.Mechanics.Translational.Components.Spring spring(s_rel0 = s_unstretched, c = c) annotation(Placement(transformation(extent = {{ -8,40},{12,60}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -100,0},{ -58,0},{ -43, -30},{ -13,30},{17, -30},{47,30},{62,0},{100,0}}, color = {0,0,0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Text(extent = {{ -130,49},{132,109}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -141, -92},{125, -51}}, lineColor = {0,0,0}, textString = "c=%c"),Ellipse(extent = {{ -8,8},{8, -8}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<HTML>
<p>
<b>Linear spring</b> acting as line force between frame_a and frame_b.
A <b>force f</b> is exerted on the origin of frame_b and with opposite sign
on the origin of frame_a along the line from the origin of frame_a to the origin
of frame_b according to the equation:
</p>
<pre>
   f = c*(s - s_unstretched);
</pre>
<p>
where \"c\" and \"s_unstretched\" are parameters and \"s\" is the
distance between the origin of frame_a and the origin of frame_b.
</p>
<p>
Optionally, the mass of the spring is taken into account by a
point mass located on the line between frame_a and frame_b
(default: middle of the line). If the spring mass is zero, the
additional equations to handle the mass are removed.
</p>
<p>
In the following figure a typical animation of the
spring is shown. The blue sphere in the middle of the
spring characterizes the location of the point mass.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Examples/Elementary/SpringWithMass.png\"
ALT=\"model Examples.Elementary.SpringWithMass\">
</p>
</HTML>"));

        equation 
          connect(lineForce.frame_a,frame_a) annotation(Line(points = {{ -20,0},{ -100,0}}, color = {95,95,95}, thickness = 0.5));
          connect(lineForce.frame_b,frame_b) annotation(Line(points = {{20,0},{100,0}}, color = {95,95,95}, thickness = 0.5));
          connect(spring.flange_b,lineForce.flange_b) annotation(Line(points = {{12,50},{12,20}}, color = {0,191,0}));
          connect(spring.flange_a,lineForce.flange_a) annotation(Line(points = {{ -8,50},{ -12,50},{ -12,20}}, color = {0,191,0}));
        end Spring;
        model Damper "Linear (velocity dependent) damper"
          import Modelica.Mechanics.MultiBody.Types;
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter SI.TranslationalDampingConstant d(final min = 0, start = 0) "Damping constant";
          parameter SI.Distance length_a = world.defaultForceLength " Length of cylinder at frame_a side" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input SIunits.Diameter diameter_a = world.defaultForceWidth " Diameter of cylinder at frame_a side" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input SIunits.Diameter diameter_b = 0.6 * diameter_a " Diameter of cylinder at frame_b side" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color color_a = {100,100,100} " Color at frame_a" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation, colorSelector));
          input Types.Color color_b = {155,155,155} " Color at frame_b" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation, colorSelector));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          extends Interfaces.PartialLineForce;
          annotation(Documentation(info = "<HTML>
<p>
<b>Linear damper</b> acting as line force between frame_a and frame_b.
A <b>force f</b> is exerted on the origin of frame_b and with opposite sign
on the origin of frame_a along the line from the origin of frame_a to the origin
of frame_b according to the equation:
</p>
<pre>
   f = d*<b>der</b>(s);
</pre>
<p>
where \"d\" is a parameter, \"s\" is the
distance between the origin of frame_a and the origin of frame_b
and der(s) is the time derivative of \"s\".
</p>
<p>
In the following figure a typical animation is shown
where a mass is hanging on a damper.
</p>
<p align=\"center\">
<IMG SRC=\"../Images/MultiBody/Forces/Damper.png\">
</p>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -101,0},{ -60,0}}, color = {0,0,0}),Line(points = {{ -60, -30},{ -60,30}}, color = {0,0,0}),Line(points = {{ -60, -30},{60, -30}}, color = {0,0,0}),Line(points = {{ -60,30},{60,30}}, color = {0,0,0}),Rectangle(extent = {{ -60,30},{30, -30}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{30,0},{100,0}}, color = {0,0,0}),Text(extent = {{ -140,47},{145,105}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -156, -96},{152, -53}}, lineColor = {0,0,0}, textString = "d=%d")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -100,0},{ -60,0}}, color = {0,0,0}),Line(points = {{ -60, -30},{ -60,30}}, color = {0,0,0}),Line(points = {{ -60, -30},{60, -30}}, color = {0,0,0}),Line(points = {{ -60,30},{60,30}}, color = {0,0,0}),Rectangle(extent = {{ -60,30},{30, -30}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{30,0},{100,0}}, color = {0,0,0}),Line(points = {{ -50,60},{50,60}}, color = {128,128,128}),Text(extent = {{ -20,60},{10,85}}, lineColor = {128,128,128}, textString = "der(s)"),Polygon(points = {{64,60},{42,68},{42,52},{62,60},{64,60}}, lineColor = {128,128,128}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid)}));
        protected 
          SI.Position r0_b[3] = e_a * noEvent(min(length_a, s));
          Visualizers.Advanced.Shape shape_a(shapeType = "cylinder", color = color_a, specularCoefficient = specularCoefficient, length = noEvent(min(length_a, s)), width = diameter_a, height = diameter_a, lengthDirection = e_a, widthDirection = {0,1,0}, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;
          Visualizers.Advanced.Shape shape_b(shapeType = "cylinder", color = color_b, specularCoefficient = specularCoefficient, length = noEvent(max(s - length_a, 0)), width = diameter_b, height = diameter_b, lengthDirection = e_a, widthDirection = {0,1,0}, r_shape = r0_b, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;

        equation 
          f = d * der(s);
        end Damper;
        model SpringDamperParallel "Linear spring and linear damper in parallel"
          import SI = Modelica.SIunits;
          import Modelica.Mechanics.MultiBody.Types;
          parameter Boolean animation = true "= true, if animation shall be enabled";
          parameter SI.TranslationalSpringConstant c(final min = 0) "Spring constant";
          parameter SI.Length s_unstretched = 0 "Unstretched spring length";
          parameter SI.TranslationalDampingConstant d(final min = 0) = 0 "Damping constant";
          input SI.Distance width = world.defaultForceWidth " Width of spring" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input SI.Distance coilWidth = width / 10 " Width of spring coil" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          parameter Integer numberOfWindings = 5 " Number of spring windings" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.Color color = Modelica.Mechanics.MultiBody.Types.Defaults.SpringColor " Color of spring" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          input Types.SpecularCoefficient specularCoefficient = world.defaultSpecularCoefficient "Reflection of ambient light (= 0: light is completely absorbed)" annotation(Dialog(tab = "Animation", group = "if animation = true", enable = animation));
          extends Interfaces.PartialLineForce;
          annotation(Documentation(info = "<HTML>
<p>
<b>Linear spring</b> and <b>dinear damper</b>
in parallel acting as line force between frame_a and frame_b.
A <b>force f</b> is exerted on the origin of frame_b and with opposite sign
on the origin of frame_a along the line from the origin of frame_a to the origin
of frame_b according to the equation:
</p>
<pre>
   f = c*(s - s_unstretched) + d*<b>der</b>(s);
</pre>
<p>
where \"c\", \"s_unstretched\" and \"d\" are parameters, \"s\" is the
distance between the origin of frame_a and the origin of frame_b
and der(s) is the time derivative of s.
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -131, -166},{114, -108}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -80,40},{ -60,40},{ -45,10},{ -15,70},{15,10},{45,70},{60,40},{80,40}}, color = {0,0,0}),Line(points = {{ -80,40},{ -80, -70}}, color = {0,0,0}),Line(points = {{ -80, -70},{ -52, -70}}, color = {0,0,0}),Rectangle(extent = {{ -52, -40},{38, -100}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -52, -40},{68, -40}}, color = {0,0,0}),Line(points = {{ -52, -100},{68, -100}}, color = {0,0,0}),Line(points = {{38, -70},{80, -70}}, color = {0,0,0}),Line(points = {{80,40},{80, -70}}, color = {0,0,0}),Line(points = {{ -100,0},{ -80,0}}, color = {0,0,0}),Line(points = {{80,0},{100,0}}, color = {0,0,0}),Text(extent = {{ -140,72},{138,108}}, lineColor = {0,0,0}, textString = "c,d=%c,%d")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80,32},{ -58,32},{ -43,2},{ -13,62},{17,2},{47,62},{62,32},{80,32}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -68,32},{ -68,97}}, color = {128,128,128}),Line(points = {{72,32},{72,97}}, color = {128,128,128}),Line(points = {{ -68,92},{72,92}}, color = {128,128,128}),Polygon(points = {{62,95},{72,92},{62,89},{62,95}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{ -20,72},{20,97}}, lineColor = {0,0,255}, textString = "s"),Rectangle(extent = {{ -52, -20},{38, -80}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -52, -80},{68, -80}}, color = {0,0,0}),Line(points = {{ -52, -20},{68, -20}}, color = {0,0,0}),Line(points = {{38, -50},{80, -50}}, color = {0,0,0}),Line(points = {{ -80, -50},{ -52, -50}}, color = {0,0,0}),Line(points = {{ -80,32},{ -80, -50}}, color = {0,0,0}),Line(points = {{80,32},{80, -50}}, color = {0,0,0}),Line(points = {{ -100,0},{ -80,0}}, color = {0,0,0}),Line(points = {{100,0},{80,0}}, color = {0,0,0})}));
        protected 
          Visualizers.Advanced.Shape shape(shapeType = "spring", color = color, length = s, width = width, height = coilWidth * 2, lengthDirection = e_a, widthDirection = {0,1,0}, extra = numberOfWindings, r = frame_a.r_0, R = frame_a.R) if world.enableAnimation and animation;

        equation 
          f = c * (s - s_unstretched) + d * der(s);
        end SpringDamperParallel;
        model SpringDamperSeries "Linear spring and linear damper in series connection"
          import SI = Modelica.SIunits;
          parameter SI.TranslationalSpringConstant c(final min = 0) "Spring constant";
          parameter SI.Length s_unstretched = 0 "Unstretched spring length";
          parameter SI.TranslationalDampingConstant d(final min = 0) = 0 "Damping constant";
          parameter SI.Length s_damper_start = 0 "Initial length of damper";
          SI.Position s_damper(start = s_damper_start, fixed = true) "Actual length of damper (frame_a - damper - spring - frame_b)";
          extends Interfaces.PartialLineForce;
          annotation(Documentation(info = "<HTML>
<p>
<b>Linear spring</b> and <b>linear damper</b> in series connection
acting as line force between frame_a and frame_b:
</p>
<pre>
  frame_a --> damper ----> spring --> frame_b
          |              |
          |-- s_damper --|  (s_damper is the state variable of this system)
</pre>
<p>
A <b>force f</b> is exerted on the origin of frame_b and with opposite sign
on the origin of frame_a along the line from the origin of frame_a to the origin
of frame_b according to the equations:
</p>
<pre>
   f = c*(s - s_unstretched - s_damper);
   f = d*der(s_damper);
</pre>
<p>
where \"c\", \"s_unstretched\" and \"d\" are parameters, \"s\" is the
distance between the origin of frame_a and the origin of frame_b.
\"s_damper\" is the length of the damper (= an internal state of this
force element) and der(s_damper) is the time derivative of s_damper.
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -100,0},{ -15,0}}, color = {0,0,0}),Line(points = {{ -60, -30},{ -15, -30}}, color = {0,0,0}),Line(points = {{ -60,30},{ -15,30}}, color = {0,0,0}),Rectangle(extent = {{ -60,30},{ -30, -30}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -15,0},{ -5,0},{5, -30},{25,30},{45, -30},{65,30},{75,0},{100,0}}, color = {0,0,0}),Text(extent = {{ -140,47},{145,105}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -148, -96},{160, -53}}, lineColor = {0,0,0}, textString = "c,d=%c,%d")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -100,0},{ -15,0}}, color = {0,0,0}),Line(points = {{ -60, -30},{ -15, -30}}, color = {0,0,0}),Line(points = {{ -60,30},{ -15,30}}, color = {0,0,0}),Rectangle(extent = {{ -60,30},{ -30, -30}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -15,0},{ -5,0},{5, -30},{25,30},{45, -30},{65,30},{75,0},{99,0}}, color = {0,0,0}),Line(points = {{ -75,0},{ -75,85}}, color = {160,160,164}),Line(points = {{ -10,0},{ -10,65}}, color = {160,160,164}),Line(points = {{80,0},{80,85}}, color = {160,160,164}),Line(points = {{ -75,80},{80,80}}, color = {160,160,164}),Line(points = {{ -75,60},{ -10,60}}, color = {160,160,164}),Polygon(points = {{ -10,60},{ -20,65},{ -20,55},{ -10,60}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Polygon(points = {{80,80},{70,85},{70,75},{80,80}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Text(extent = {{ -72,63},{ -11,78}}, lineColor = {160,160,164}, textString = "s_damper"),Text(extent = {{0,80},{20,100}}, lineColor = {160,160,164}, textString = "s")}));

        equation 
          f = c * (s - s_unstretched - s_damper);
          d * der(s_damper) = f;
        end SpringDamperSeries;
        package Internal "Internal package, should not be used by user"
          model BasicForce "Force acting between two frames, defined by 3 input signals"
            import Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB;
            extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
            Interfaces.Frame_resolve frame_resolve "The input signals are optionally resolved in this frame" annotation(Placement(transformation(origin = {40,100}, extent = {{ -16, -16},{16,16}}, rotation = 90)));
            Modelica.Blocks.Interfaces.RealInput force[3](each final quantity = "Force", each final unit = "N") "x-, y-, z-coordinates of force resolved in frame defined by resolveInFrame" annotation(Placement(transformation(origin = { -60,120}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_b "Frame in which force is resolved (1: world, 2: frame_a, 3: frame_b, 4: frame_resolve)";
            Modelica.SIunits.Position r_0[3] "Position vector from origin of frame_a to origin of frame_b resolved in world frame";
            Modelica.SIunits.Force f_b_0[3] "frame_b.f resoved in world frame";
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -98,99},{99, -98}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -92,61},{87,35}}, lineColor = {192,192,192}, textString = "resolve"),Text(extent = {{ -136, -52},{149, -113}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{40,100},{40,0}}, color = {95,95,95}, pattern = LinePattern.Dot),Polygon(points = {{ -94,0},{ -64,11},{ -64, -10},{ -94,0}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -60,100},{40,100}}, color = {95,95,95}, pattern = LinePattern.Dot),Polygon(points = {{94,0},{65,12},{65, -11},{94,0}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -64,0},{ -20,0}}, color = {0,0,0}),Line(points = {{20,0},{65,0}}, color = {0,0,0})}), Documentation(info = "<HTML>
<p>
The <b>3</b> signals of the <b>force</b> connector are interpreted
as the x-, y- and z-coordinates of a <b>force</b> acting at the frame
connector to which frame_b of this component is attached.
Via parameter <b>resolveInFrame</b> it is defined, in which frame these
coordinates shall be resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve input force in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve input force in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve input force in frame_b (= default)</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve input force in frame_resolve (frame_resolve must be connected)</td></tr>
</table>

<p>
If resolveInFrame = ResolveInFrameAB.frame_resolve, the force coordinates
are with respect to the frame, that is connected to <b>frame_resolve</b>.
<p>

<p>
If resolveInFrame is not ResolveInFrameAB.frame_resolve, then the position
vector and the orientation object of frame_resolve must be set to constant
values from the outside in order that the model remains balanced
(these constant values are ignored).
</p>

</HTML>
"));

          equation 
            assert(cardinality(frame_resolve) > 0, "Connector frame_resolve must be connected at least once and frame_resolve.r_0/.R must be set");
            frame_resolve.f = zeros(3);
            frame_resolve.t = zeros(3);
            if resolveInFrame == ResolveInFrameAB.frame_a then
              f_b_0 =  -Frames.resolve1(frame_a.R, force);
              frame_b.f = Frames.resolve2(frame_b.R, f_b_0);

            elseif resolveInFrame == ResolveInFrameAB.frame_b then
              f_b_0 =  -Frames.resolve1(frame_b.R, force);
              frame_b.f =  -force;


            elseif resolveInFrame == ResolveInFrameAB.world then
              f_b_0 =  -force;
              frame_b.f = Frames.resolve2(frame_b.R, f_b_0);


            elseif resolveInFrame == ResolveInFrameAB.frame_resolve then
              f_b_0 =  -Frames.resolve1(frame_resolve.R, force);
              frame_b.f = Frames.resolve2(frame_b.R, f_b_0);


            else
              assert(false, "Wrong value for parameter resolveInFrame");
              f_b_0 = zeros(3);
              frame_b.f = zeros(3);

            end if;
            frame_b.t = zeros(3);
            r_0 = frame_b.r_0 - frame_a.r_0;
            zeros(3) = frame_a.f + Frames.resolve2(frame_a.R, f_b_0);
            zeros(3) = frame_a.t + Frames.resolve2(frame_a.R, cross(r_0, f_b_0));
          end BasicForce;
          model BasicTorque "Torque acting between two frames, defined by 3 input signals"
            import SI = Modelica.SIunits;
            import Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB;
            extends Modelica.Mechanics.MultiBody.Interfaces.PartialTwoFrames;
            Interfaces.Frame_resolve frame_resolve "The input signals are optionally resolved in this frame" annotation(Placement(transformation(origin = {40,100}, extent = {{ -16, -16},{16,16}}, rotation = 90)));
            Modelica.Blocks.Interfaces.RealInput torque[3](each final quantity = "Torque", each final unit = "N.m") "x-, y-, z-coordiantes of torque resolved in frame defined by resolveInFrame" annotation(Placement(transformation(origin = { -60,120}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameAB.frame_b "Frame in which torque is resolved (1: world, 2: frame_a, 3: frame_b, 4: frame_resolve)";
            SI.Position r_0[3] "Position vector from origin of frame_a to origin of frame_b resolved in world frame";
            SI.Torque t_b_0[3] "frame_b.t resoved in world frame";
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -98,99},{99, -98}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -59,55},{72,30}}, lineColor = {192,192,192}, textString = "resolve"),Text(extent = {{ -139, -27},{146, -88}}, textString = "%name", lineColor = {0,0,255}),Polygon(points = {{100,20},{84,52},{69,39},{100,20}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{40,100},{76,46}}, color = {95,95,95}, pattern = LinePattern.Dot),Polygon(points = {{ -99,20},{ -86,53},{ -70,42},{ -99,20}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -60,100},{40,100}}, color = {95,95,95}, pattern = LinePattern.Dot),Line(points = {{ -79,47},{ -70,61},{ -59,72},{ -45,81},{ -32,84},{ -20,85}}, color = {0,0,0}),Line(points = {{77,45},{66,60},{55,69},{49,74},{41,80},{31,84},{20,85}}, color = {0,0,0})}), Documentation(info = "<HTML>
<p>
The <b>3</b> signals of the <b>torque</b> connector are interpreted
as the x-, y- and z-coordinates of a <b>torque</b> acting at the frame
connector to which frame_b of this component is attached.
Via parameter <b>resolveInFrame</b> it is defined, in which frame these
coordinates shall be resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameAB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve input torque in world frame</td></tr>

<tr><td valign=\"top\">frame_a</td>
    <td valign=\"top\">Resolve input torque in frame_a</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve input torque in frame_b (= default)</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve input torque in frame_resolve (frame_resolve must be connected)</td></tr>
</table>

<p>
If resolveInFrame = ResolveInFrameAB.frame_resolve, the torque coordinates
are with respect to the frame, that is connected to <b>frame_resolve</b>.
<p>

<p>
If resolveInFrame is not ResolveInFrameAB.frame_resolve, then the position
vector and the orientation object of frame_resolve must be set to constant
values from the outside in order that the model remains balanced
(these constant values are ignored).
</p>
</HTML>
"));

          equation 
            assert(cardinality(frame_resolve) > 0, "Connector frame_resolve must be connected at least once and frame_resolve.r_0/.R must be set");
            frame_resolve.f = zeros(3);
            frame_resolve.t = zeros(3);
            r_0 = frame_b.r_0 - frame_a.r_0;
            frame_a.f = zeros(3);
            frame_b.f = zeros(3);
            if resolveInFrame == ResolveInFrameAB.frame_a then
              t_b_0 =  -Frames.resolve1(frame_a.R, torque);
              frame_b.t = Frames.resolve2(frame_b.R, t_b_0);

            elseif resolveInFrame == ResolveInFrameAB.frame_b then
              t_b_0 =  -Frames.resolve1(frame_b.R, torque);
              frame_b.t =  -torque;


            elseif resolveInFrame == ResolveInFrameAB.world then
              t_b_0 =  -torque;
              frame_b.t = Frames.resolve2(frame_b.R, t_b_0);


            elseif resolveInFrame == ResolveInFrameAB.frame_resolve then
              t_b_0 =  -Frames.resolve1(frame_resolve.R, torque);
              frame_b.t = Frames.resolve2(frame_b.R, t_b_0);


            else
              assert(false, "Wrong value for parameter resolveInFrame");
              t_b_0 = zeros(3);
              frame_b.t = zeros(3);

            end if;
            zeros(3) = frame_a.t + Frames.resolve2(frame_a.R, t_b_0);
          end BasicTorque;
          model BasicWorldForce "External force acting at frame_b, defined by 3 input signals"
            import SI = Modelica.SIunits;
            import Modelica.Mechanics.MultiBody.Types.ResolveInFrameB;
            extends Interfaces.PartialOneFrame_b;
            Interfaces.Frame_resolve frame_resolve "The input signals are optionally resolved in this frame" annotation(Placement(transformation(origin = {0, -100}, extent = {{ -16, -16},{16,16}}, rotation = 270)));
            Modelica.Blocks.Interfaces.RealInput force[3](each final quantity = "Force", each final unit = "N") "x-, y-, z-coordinates of force resolved in frame defined by resolveInFrame" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.world "Frame in which force is resolved (1: world, 2: frame_b, 3: frame_resolve)";
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -100,10},{50,10},{50,31},{97,0},{50, -31},{50, -10},{ -100, -10},{ -100,10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{0, -10},{0, -97}}, color = {95,95,95}, pattern = LinePattern.Dot)}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -89, -46},{91, -76}}, lineColor = {192,192,192}, textString = "resolve"),Polygon(points = {{ -100,10},{50,10},{50,31},{94,0},{50, -31},{50, -10},{ -100, -10},{ -100,10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -149,103},{136,42}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{0, -10},{0, -95}}, color = {95,95,95}, pattern = LinePattern.Dot)}), Documentation(info = "<HTML>
<p>
The 3 signals of the <b>force</b> connector are interpreted
as the x-, y- and z-coordinates of a <b>force</b> acting at the frame
connector to which this component is attached.
Via parameter <b>resolveInFrame</b> it is defined, in which frame these
coordinates shall be resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve input force in world frame (= default)</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve input force in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve input force in frame_resolve (frame_resolve must be connected)</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameB.frame_resolve, the force coordinates
are with respect to the frame, that is connected to <b>frame_resolve</b>.
<p>

<p>
If resolveInFrame is not Types.ResolveInFrameB.frame_resolve, then the position
vector and the orientation object of frame_resolve must be set to constant
values from the outside in order that the model remains balanced
(these constant values are ignored).
</p>

</HTML>
"));

          equation 
            assert(cardinality(frame_resolve) > 0, "Connector frame_resolve must be connected at least once and frame_resolve.r_0/.R must be set");
            frame_resolve.f = zeros(3);
            frame_resolve.t = zeros(3);
            if resolveInFrame == ResolveInFrameB.world then
              frame_b.f =  -Frames.resolve2(frame_b.R, force);

            elseif resolveInFrame == ResolveInFrameB.frame_b then
              frame_b.f =  -force;


            elseif resolveInFrame == ResolveInFrameB.frame_resolve then
              frame_b.f =  -Frames.resolveRelative(force, frame_resolve.R, frame_b.R);


            else
              assert(false, "Wrong value for parameter resolveInFrame");
              frame_b.f = zeros(3);

            end if;
            frame_b.t = zeros(3);
          end BasicWorldForce;
          model BasicWorldTorque "External torque acting at frame_b, defined by 3 input signals"
            import SI = Modelica.SIunits;
            import Modelica.Mechanics.MultiBody.Types.ResolveInFrameB;
            extends Interfaces.PartialOneFrame_b;
            Interfaces.Frame_resolve frame_resolve "The input signals are optionally resolved in this frame" annotation(Placement(transformation(origin = {0,100}, extent = {{16, -16},{ -16,16}}, rotation = 270)));
            Modelica.Blocks.Interfaces.RealInput torque[3](each final quantity = "Torque", each final unit = "N.m") "x-, y-, z-coordinates of torque resolved in frame defined by resolveInFrame" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
            parameter Modelica.Mechanics.MultiBody.Types.ResolveInFrameB resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.world "Frame in which torque is resolved (1: world, 2: frame_b, 3: frame_resolve)";
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{0,97},{0,82}}, color = {95,95,95}, pattern = LinePattern.Dot),Line(points = {{ -100,0},{ -94,13},{ -86,28},{ -74,48},{ -65,60},{ -52,72},{ -35,81},{ -22,84},{ -8,84},{7,80},{19,73},{32,65},{44,55},{52,47},{58,40}}, color = {0,0,0}, thickness = 0.5),Polygon(points = {{97,6},{75,59},{41,24},{97,6}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -61,64},{46,27}}, lineColor = {192,192,192}, textString = "resolve"),Text(extent = {{ -145, -28},{140, -89}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{0,95},{0,82}}, color = {95,95,95}, pattern = LinePattern.Dot),Line(points = {{ -100,0},{ -94,13},{ -86,28},{ -74,48},{ -65,60},{ -52,72},{ -35,81},{ -22,84},{ -8,84},{7,80},{19,73},{32,65},{44,55},{52,47},{58,40}}, color = {0,0,0}, thickness = 0.5),Polygon(points = {{94,10},{75,59},{41,24},{94,10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
<p>
The 3 signals of the <b>torque</b> connector are interpreted
as the x-, y- and z-coordinates of a <b>torque</b> acting at the frame
connector to which this component is attached.
Via parameter <b>resolveInFrame</b> it is defined, in which frame these
coordinates shall be resolved:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.ResolveInFrameB.</b></th><th><b>Meaning</b></th></tr>
<tr><td valign=\"top\">world</td>
    <td valign=\"top\">Resolve input torque in world frame (= default)</td></tr>

<tr><td valign=\"top\">frame_b</td>
    <td valign=\"top\">Resolve input torque in frame_b</td></tr>

<tr><td valign=\"top\">frame_resolve</td>
    <td valign=\"top\">Resolve input torque in frame_resolve (frame_resolve must be connected)</td></tr>
</table>

<p>
If resolveInFrame = Types.ResolveInFrameB.frame_resolve, the torque coordinates
are with respect to the frame, that is connected to <b>frame_resolve</b>.
<p>

<p>
If resolveInFrame is not Types.ResolveInFrameB.frame_resolve, then the position
vector and the orientation object of frame_resolve must be set to constant
values from the outside in order that the model remains balanced
(these constant values are ignored).
</p>

</HTML>
"));

          equation 
            assert(cardinality(frame_resolve) > 0, "Connector frame_resolve must be connected at least once and frame_resolve.r_0/.R must be set");
            frame_resolve.f = zeros(3);
            frame_resolve.t = zeros(3);
            if resolveInFrame == ResolveInFrameB.world then
              frame_b.t =  -Frames.resolve2(frame_b.R, torque);

            elseif resolveInFrame == ResolveInFrameB.frame_b then
              frame_b.t =  -torque;


            elseif resolveInFrame == ResolveInFrameB.frame_resolve then
              frame_b.t =  -Frames.resolveRelative(torque, frame_resolve.R, frame_b.R);


            else
              assert(false, "Wrong value for parameter resolveInFrame");
              frame_b.t = zeros(3);

            end if;
            frame_b.f = zeros(3);
          end BasicWorldTorque;
        end Internal;
      end Forces;
    end MultiBody;
    package Translational "Library to model 1-dimensional, translational mechanical systems"
      extends Modelica.Icons.Library2;
      import SI = Modelica.SIunits;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -84, -73},{66, -73}}, color = {0,0,0}),Rectangle(extent = {{ -81, -22},{ -8, -65}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Line(points = {{ -8, -43},{ -1, -43},{6, -64},{17, -23},{29, -65},{40, -23},{50, -44},{61, -44}}, color = {0,0,0}),Line(points = {{ -59, -73},{ -84, -93}}, color = {0,0,0}),Line(points = {{ -11, -73},{ -36, -93}}, color = {0,0,0}),Line(points = {{ -34, -73},{ -59, -93}}, color = {0,0,0}),Line(points = {{14, -73},{ -11, -93}}, color = {0,0,0}),Line(points = {{39, -73},{14, -93}}, color = {0,0,0}),Line(points = {{63, -73},{38, -93}}, color = {0,0,0})}), Documentation(info = "<html>
<p>
This package contains components to model <i>1-dimensional translational
mechanical</i> systems.
</p>
<p>
The <i>filled</i> and <i>non-filled green squares</i> at the left and
right side of a component represent <i>mechanical flanges</i>.
Drawing a line between such squares means that the corresponding
flanges are <i>rigidly attached</i> to each other. The components of this
library can be usually connected together in an arbitrary way. E.g. it is
possible to connect two springs or two sliding masses with inertia directly
together.
<p> The only <i>connection restriction</i> is that the Coulomb friction
elements (e.g. MassWithStopAndFriction) should be only connected
together provided a compliant element, such as a spring, is in between.
The reason is that otherwise the frictional force is not uniquely
defined if the elements are stuck at the same time instant (i.e., there
does not exist a unique solution) and some simulation systems may not be
able to handle this situation, since this leads to a singularity during
simulation. It can only be resolved in a \"clean way\" by combining the
two connected friction elements into
one component and resolving the ambiguity of the frictional force in the
stuck mode.
</p>
<p> Another restriction arises if the hard stops in model MassWithStopAndFriction are used, i. e.
the movement of the mass is limited by a stop at smax or smin.
<font color=\"#ff0000\"> <b>This requires the states Stop.s and Stop.v</b> </font>. If these states are eliminated during the index reduction
the model will not work. To avoid this any inertias should be connected via springs
to the Stop element, other sliding masses, dampers or hydraulic chambers must be avoided. </p>
<p>
In the <i>icon</i> of every component an <i>arrow</i> is displayed in grey
color. This arrow characterizes the coordinate system in which the vectors
of the component are resolved. It is directed into the positive
translational direction (in the mathematical sense).
In the flanges of a component, a coordinate system is rigidly attached
to the flange. It is called <i>flange frame</i> and is directed in parallel
to the component coordinate system. As a result, e.g., the positive
cut-force of a \"left\" flange (flange_a) is directed into the flange, whereas
the positive cut-force of a \"right\" flange (flange_b) is directed out of the
flange. A flange is described by a Modelica connector containing
the following variables:
</p>
<pre>
   Modelica.SIunits.Position s    \"Absolute position of flange\";
   <b>flow</b> Modelica.SIunits.Force f  \"Cut-force in the flange\";
</pre>

<p>
This library is designed in a fully object oriented way in order that
components can be connected together in every meaningful combination
(e.g. direct connection of two springs or two shafts with inertia).
As a consequence, most models lead to a system of
differential-algebraic equations of <i>index 3</i> (= constraint
equations have to be differentiated twice in order to arrive at
a state space representation) and the Modelica translator or
the simulator has to cope with this system representation.
According to our present knowledge, this requires that the
Modelica translator is able to symbolically differentiate equations
(otherwise it is e.g. not possible to provide consistent initial
conditions; even if consistent initial conditions are present, most
numerical DAE integrators can cope at most with index 2 DAEs).
</p>

<dl>
<dt><b>Library Officer</b>
<dd><a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a> <br>
    Deutsches Zentrum f&uuml;r Luft und Raumfahrt e.V. (DLR)<br>
    Institut f&uuml;r Robotik und Mechatronik (DLR-RM)<br>
    Abteilung Systemdynamik und Regelungstechnik<br>
    Postfach 1116<br>
    D-82230 Wessling<br>
    Germany<br>
    email: <A HREF=\"mailto:Martin.Otter@dlr.de\">Martin.Otter@dlr.de</A><br><br>
</dl>

<p>
<b>Contributors to this library:</b>
</p>

<ul>
<li> Main author until 2006:<br>
     Peter Beater <br>
     Universit&auml;t Paderborn, Abteilung Soest<br>
     Fachbereich Maschinenbau/Automatisierungstechnik<br>
     L&uuml;becker Ring 2 <br>
     D 59494 Soest <br>
     Germany <br>
     email: <A HREF=\"mailto:info@beater.de\">info@beater.de</A><br><br>
     </li>

<li> <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
     Technical Consulting &amp; Electrical Engineering<br>
     A-3423 St.Andrae-Woerdern, Austria<br>
     email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a><br><br></li>

<li> <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a> (DLR-RM)</li>
</ul>

<p>
Copyright &copy; 1998-2009, Modelica Association, Anton Haumer and Universit&auml;t Paderborn, FB 12.
</p>
<p>
<i>This Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p><br>

</HTML>
", revisions = "<html>
<ul>
<li><i>Version 1.0 (January 5, 2000)</i>
       by Peter Beater <br>
       Realized a first version based on Modelica library Mechanics.Rotational
       by Martin Otter and an existing Dymola library onedof.lib by Peter Beater.
       <br>
<li><i>Version 1.01 (July 18, 2001)</i>
       by Peter Beater <br>
       Assert statement added to \"Stop\", small bug fixes in examples.
       <br>
</li>
<li><i>Version 1.1.0 2007-11-16</i>
       by Anton Haumer<br>
       Redesign for Modelica 3.0-compliance<br>
       Added new components acording to Mechanics.Rotational library
       <br>
</li>
</ul>
</html>"));
      package Examples "Demonstration examples of the components of this package"
        extends Modelica.Icons.Library;
        annotation(Documentation(info = "<html>
<p>
This package contains example models to demonstrate the usage of the
Translational package. Open the models and
simulate them according to the provided description in the models.
</p>

</HTML>
"));
        model SignConvention "Examples for the used sign conventions."
          extends Modelica.Icons.Example;
          annotation(Documentation(info = "<html>
<p>
If all arrows point in the same direction a positive force
results in a positive acceleration a, velocity v and position s.
</p>
For a force of 1 N and a mass of 1 Kg this leads to
<pre>
        a = 1 m/s2
        v = 1 m/s after 1 s (SlidingMass1.v)
        s = 0.5 m after 1 s (SlidingMass1.s)
</pre>
The acceleration is not available for plotting.
<p>
</p>
System 1) and 2) are equivalent. It doesn't matter whether the
force pushes at flange_a in system 1 or pulls at flange_b in system 2.
</p><p>
It is of course possible to ignore the arrows and connect the models
in an arbitrary way. But then it is hard see in what direction the
force acts.
</p><p>
In the third system the two arrows are opposed which means that the
force acts in the opposite direction (in the same direction as in
the two other examples).
</p>

</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -100,80},{ -82,60}}, textString = "1)", lineColor = {0,0,255}),Text(extent = {{ -100,40},{ -82,20}}, textString = "2)", lineColor = {0,0,255}),Text(extent = {{ -100, -20},{ -82, -40}}, textString = "3)", lineColor = {0,0,255})}), experiment(StopTime = 1));
          Translational.Components.Mass mass1(L = 1, s(fixed = true), v(fixed = true), m = 1) annotation(Placement(transformation(extent = {{40,60},{60,80}}, rotation = 0)));
          Translational.Sources.Force force1 annotation(Placement(transformation(extent = {{ -4,60},{16,80}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant constant1(k = 1) annotation(Placement(transformation(extent = {{ -44,60},{ -24,80}}, rotation = 0)));
          Translational.Components.Mass mass2(L = 1, s(fixed = true), v(fixed = true), m = 1) annotation(Placement(transformation(extent = {{40,0},{60,20}}, rotation = 0)));
          Translational.Sources.Force force2 annotation(Placement(transformation(extent = {{ -4,20},{16,40}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant constant2(k = 1) annotation(Placement(transformation(extent = {{ -44,20},{ -24,40}}, rotation = 0)));
          Translational.Components.Mass mass3(L = 1, s(fixed = true), v(fixed = true), m = 1) annotation(Placement(transformation(extent = {{ -40, -40},{ -20, -20}}, rotation = 0)));
          Translational.Sources.Force force3(useSupport = true) annotation(Placement(transformation(extent = {{20, -40},{0, -20}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant constant3(k = 1) annotation(Placement(transformation(extent = {{60, -40},{40, -20}}, rotation = 0)));
          Translational.Components.Fixed fixed annotation(Placement(transformation(extent = {{0, -60},{20, -40}})));

        equation 
          connect(constant1.y,force1.f) annotation(Line(points = {{ -23,70},{ -6,70}}, color = {0,0,127}));
          connect(constant2.y,force2.f) annotation(Line(points = {{ -23,30},{ -6,30}}, color = {0,0,127}));
          connect(constant3.y,force3.f) annotation(Line(points = {{39, -30},{22, -30}}, color = {0,0,127}));
          connect(force1.flange,mass1.flange_a) annotation(Line(points = {{16,70},{40,70}}, color = {0,127,0}, smooth = Smooth.None));
          connect(force2.flange,mass2.flange_b) annotation(Line(points = {{16,30},{70,30},{70,10},{60,10}}, color = {0,127,0}, smooth = Smooth.None));
          connect(mass3.flange_b,force3.flange) annotation(Line(points = {{ -20, -30},{0, -30}}, color = {0,127,0}, smooth = Smooth.None));
          connect(fixed.flange,force3.support) annotation(Line(points = {{10, -50},{10, -40}}, color = {0,127,0}, smooth = Smooth.None));
        end SignConvention;
        model InitialConditions "Setting of initial conditions"
          extends Modelica.Icons.Example;
          annotation(Documentation(info = "<html>
<p>
There are several ways to set initial conditions.
In the first system the position of the mass m3 was defined
by using the modifier s(start=4.5), the position of m4 by s(start=12.5).
These positions were chosen such that the system is a rest. To calculate
these values start at the left (Fixed1) with a value of 1 m. The spring
has an unstreched length of 2 m and m3 an length of 3 m, which leads to
</p>

<pre>
        1   m (fixed1)
      + 2   m (spring s2)
      + 3/2 m (half of the length of mass m3)
      -------
        4,5 m = s(start = 4.5) for m3
      + 3/2 m (half of the length of mass m3)
      + 4   m (springDamper 2)
      + 5/2 m (half of length of mass m4)
      -------
       12,5 m = s(start = 12.5) for m4
</pre>

<p>
This selection of initial conditions has the effect that Dymola selects
those variables (m3.s and m4.s) as state variables.
In the second example the length of the springs are given as start values
but they cannot be used as state for pure springs (only for the spring/damper
combination). In this case the system is not at rest.
</p>

<p>
<IMG SRC=../Images/Translational/Fig.translational.examples.InitialConditions.png>
</p>


</html>
"), experiment(StopTime = 5), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          Translational.Components.Fixed fixed2(s0 = 1) annotation(Placement(transformation(extent = {{ -100,60},{ -80,80}}, rotation = 0)));
          Translational.Components.Spring s2(s_rel0 = 2, c = 1000.0) annotation(Placement(transformation(extent = {{ -60,60},{ -40,80}}, rotation = 0)));
          Translational.Components.Mass m3(L = 3, s(start = 4.5, fixed = true), v(fixed = true), m = 1) annotation(Placement(transformation(extent = {{ -20,60},{0,80}}, rotation = 0)));
          Translational.Components.SpringDamper sd2(s_rel0 = 4, c = 111, d = 1) annotation(Placement(transformation(extent = {{20,60},{40,80}}, rotation = 0)));
          Translational.Components.Mass m4(L = 5, s(start = 12.5, fixed = true), v(fixed = true), m = 1) annotation(Placement(transformation(extent = {{60,60},{80,80}}, rotation = 0)));
          Translational.Components.Fixed fixed1(s0 =  -1) annotation(Placement(transformation(extent = {{ -100, -20},{ -80,0}}, rotation = 0)));
          Translational.Components.Spring s1(s_rel0 = 1, c = 1000.0, s_rel(start = 1, fixed = true)) annotation(Placement(transformation(extent = {{ -58, -20},{ -38,0}}, rotation = 0)));
          Translational.Components.Mass m1(L = 1, v(fixed = true), m = 1) annotation(Placement(transformation(extent = {{ -20, -20},{0,0}}, rotation = 0)));
          Translational.Components.SpringDamper sd1(s_rel0 = 1, c = 111, s_rel(start = 1, fixed = true), v_rel(fixed = true), d = 1) annotation(Placement(transformation(extent = {{20, -20},{40,0}}, rotation = 0)));
          Translational.Components.Mass m2(L = 2, m = 1) annotation(Placement(transformation(extent = {{60, -20},{80,0}}, rotation = 0)));

        equation 
          connect(s2.flange_a,fixed2.flange) annotation(Line(points = {{ -60,70},{ -90,70}}, color = {0,127,0}, smooth = Smooth.None));
          connect(s1.flange_a,fixed1.flange) annotation(Line(points = {{ -58, -10},{ -90, -10}}, color = {0,127,0}, smooth = Smooth.None));
          connect(m1.flange_a,s1.flange_b) annotation(Line(points = {{ -20, -10},{ -38, -10}}, color = {0,127,0}, smooth = Smooth.None));
          connect(sd1.flange_a,m1.flange_b) annotation(Line(points = {{20, -10},{0, -10}}, color = {0,127,0}, smooth = Smooth.None));
          connect(m2.flange_a,sd1.flange_b) annotation(Line(points = {{60, -10},{40, -10}}, color = {0,127,0}, smooth = Smooth.None));
          connect(m4.flange_a,sd2.flange_b) annotation(Line(points = {{60,70},{40,70}}, color = {0,127,0}, smooth = Smooth.None));
          connect(sd2.flange_a,m3.flange_b) annotation(Line(points = {{20,70},{0,70}}, color = {0,127,0}, smooth = Smooth.None));
          connect(m3.flange_a,s2.flange_b) annotation(Line(points = {{ -20,70},{ -40,70}}, color = {0,127,0}, smooth = Smooth.None));
        end InitialConditions;
        model WhyArrows "Use of arrows in Mechanics.Translational"
          extends Modelica.Icons.Example;
          annotation(Documentation(info = "<html>
<p>
When using the models of the translational sublibrary
it is recommended to make sure that all arrows point in
the same direction because then all component have the
same reference system.
In the example the distance from flange_a of Rod1 to flange_b
of Rod2 is 2 m. The distance from flange_a of Rod1 to flange_b
of Rod3 is also 2 m though it is difficult to see that. Without
the arrows it would be almost impossible to notice.
That all arrows point in the same direction is a sufficient
condition for an easy use of the library. There are cases
where horizontally flipped models can be used without
problems.
</p>
</html>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -80,14},{90,0}}, lineColor = {0,0,255}, textString = "positionSensor2.s = positionSensor3.s"),Text(extent = {{ -84,4},{88, -16}}, lineColor = {0,0,255}, textString = "positionSensor3.s <>positionSensor1.s"),Text(extent = {{ -82, -80},{94, -92}}, textString = "Both systems are equivalent", lineColor = {0,0,255}),Line(points = {{ -90, -28},{90, -28}}, thickness = 0.5, color = {0,0,255})}), experiment(StopTime = 1));
          Translational.Components.Fixed fixed annotation(Placement(transformation(extent = {{ -20,20},{0,40}}, rotation = 0)));
          Translational.Components.Rod rod1(L = 1) annotation(Placement(transformation(extent = {{ -48,20},{ -28,40}}, rotation = 0)));
          Translational.Components.Rod rod2(L = 1) annotation(Placement(transformation(extent = {{20,20},{40,40}}, rotation = 0)));
          Translational.Components.Rod rod3(L = 1) annotation(Placement(transformation(extent = {{ -30,58},{ -50,78}}, rotation = 0)));
          Translational.Sensors.PositionSensor positionSensor2 annotation(Placement(transformation(extent = {{60,20},{80,40}}, rotation = 0)));
          Translational.Sensors.PositionSensor positionSensor1 annotation(Placement(transformation(extent = {{ -60,20},{ -80,40}}, rotation = 0)));
          Translational.Sensors.PositionSensor positionSensor3 annotation(Placement(transformation(extent = {{ -60,58},{ -80,78}}, rotation = 0)));
          Translational.Components.Fixed fixed1(s0 =  -1.9) annotation(Placement(transformation(extent = {{ -100, -60},{ -80, -40}}, rotation = 0)));
          Translational.Components.Spring spring1(s_rel0 = 2, c = 11) annotation(Placement(transformation(extent = {{ -80, -60},{ -60, -40}}, rotation = 0)));
          Translational.Components.Mass mass1(L = 2, s(fixed = true), v(fixed = true), m = 1) annotation(Placement(transformation(extent = {{ -50, -60},{ -30, -40}}, rotation = 0)));
          Translational.Components.Fixed fixed2(s0 =  -1.9) annotation(Placement(transformation(extent = {{0, -60},{20, -40}}, rotation = 0)));
          Translational.Components.Spring spring2(s_rel0 = 2, c = 11) annotation(Placement(transformation(extent = {{30, -60},{50, -40}}, rotation = 0)));
          Translational.Components.Mass inertia2(L = 2, m = 1, s(fixed = true), v(fixed = true)) annotation(Placement(transformation(extent = {{80, -60},{60, -40}}, rotation = 0)));

        equation 
          connect(spring1.flange_b,mass1.flange_b) annotation(Line(points = {{ -60, -50},{ -60, -72},{ -30, -72},{ -30, -50}}, color = {0,191,0}));
          connect(spring2.flange_b,inertia2.flange_b) annotation(Line(points = {{50, -50},{60, -50}}, color = {0,191,0}));
          connect(rod3.flange_b,positionSensor3.flange) annotation(Line(points = {{ -50,68},{ -60,68}}, color = {0,127,0}, smooth = Smooth.None));
          connect(rod1.flange_a,positionSensor1.flange) annotation(Line(points = {{ -48,30},{ -60,30}}, color = {0,127,0}, smooth = Smooth.None));
          connect(rod1.flange_b,fixed.flange) annotation(Line(points = {{ -28,30},{ -10,30}}, color = {0,127,0}, smooth = Smooth.None));
          connect(rod3.flange_a,fixed.flange) annotation(Line(points = {{ -30,68},{ -10,68},{ -10,30}}, color = {0,127,0}, smooth = Smooth.None));
          connect(fixed.flange,rod2.flange_a) annotation(Line(points = {{ -10,30},{20,30}}, color = {0,127,0}, smooth = Smooth.None));
          connect(rod2.flange_b,positionSensor2.flange) annotation(Line(points = {{40,30},{60,30}}, color = {0,127,0}, smooth = Smooth.None));
          connect(fixed1.flange,spring1.flange_a) annotation(Line(points = {{ -90, -50},{ -80, -50}}, color = {0,127,0}, smooth = Smooth.None));
          connect(fixed2.flange,spring2.flange_a) annotation(Line(points = {{10, -50},{30, -50}}, color = {0,127,0}, smooth = Smooth.None));
        end WhyArrows;
        model Accelerate "Use of model accelerate."
          extends Modelica.Icons.Example;
          Translational.Sources.Accelerate accelerate annotation(Placement(transformation(extent = {{ -40,20},{ -20,40}}, rotation = 0)));
          Translational.Components.Mass mass(L = 1, m = 1) annotation(Placement(transformation(extent = {{0,20},{20,40}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant constantAcc(k = 1) annotation(Placement(transformation(extent = {{ -80,20},{ -60,40}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>
Demonstrate usage of component Sources.Accelerate by moving a massing
with a predefined acceleration.
</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1));

        equation 
          connect(accelerate.flange,mass.flange_a) annotation(Line(points = {{ -20,30},{0,30}}, color = {0,127,0}, smooth = Smooth.None));
          connect(constantAcc.y,accelerate.a_ref) annotation(Line(points = {{ -59,30},{ -42,30}}, color = {0,0,127}, smooth = Smooth.None));
        end Accelerate;
        model Damper "Use of damper models."
          extends Modelica.Icons.Example;
          annotation(Documentation(info = "<html>
<p>
Demonstrate usage of damper components in different variants.
</p>
</html>"), experiment(StopTime = 1), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          Translational.Components.Mass mass1(L = 1, s(start = 3, fixed = true), v(start = 10, fixed = true), m = 1) annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}}, rotation = 0)));
          Translational.Components.Damper damper1(d = 25) annotation(Placement(transformation(extent = {{ -20,60},{0,80}}, rotation = 0)));
          Translational.Components.Fixed fixed1(s0 = 4.5) annotation(Placement(transformation(extent = {{22,60},{42,80}}, rotation = 0)));
          Translational.Components.Mass mass2(L = 1, s(start = 3, fixed = true), v(start = 10, fixed = true), m = 1) annotation(Placement(transformation(extent = {{ -80,0},{ -60,20}}, rotation = 0)));
          Translational.Components.Damper damper2(d = 25) annotation(Placement(transformation(extent = {{ -20,0},{0,20}}, rotation = 0)));
          Translational.Components.Fixed fixed2(s0 = 4.5) annotation(Placement(transformation(extent = {{20,0},{40,20}}, rotation = 0)));
          Translational.Components.Mass mass3(L = 1, s(start = 3, fixed = true), v(start = 10, fixed = true), m = 1) annotation(Placement(transformation(extent = {{ -80, -60},{ -60, -40}}, rotation = 0)));
          Translational.Components.Fixed fixed3(s0 = 4.5) annotation(Placement(transformation(extent = {{20, -60},{40, -40}}, rotation = 0)));
          Translational.Components.Spring spring2(s_rel0 = 1, c = 1) annotation(Placement(transformation(extent = {{ -20, -20},{0,0}}, rotation = 0)));
          Translational.Components.SpringDamper springDamper3(s_rel0 = 1, d = 25, c = 1) annotation(Placement(transformation(extent = {{ -20, -60},{0, -40}}, rotation = 0)));

        equation 
          connect(mass1.flange_b,damper1.flange_a) annotation(Line(points = {{ -60,70},{ -20,70}}, color = {0,191,0}));
          connect(mass2.flange_b,damper2.flange_a) annotation(Line(points = {{ -60,10},{ -20,10}}, color = {0,191,0}));
          connect(damper2.flange_b,spring2.flange_b) annotation(Line(points = {{0,10},{0, -10}}, color = {0,191,0}));
          connect(damper2.flange_a,spring2.flange_a) annotation(Line(points = {{ -20,10},{ -20, -10}}, color = {0,191,0}));
          connect(mass3.flange_b,springDamper3.flange_a) annotation(Line(points = {{ -60, -50},{ -20, -50}}, color = {0,191,0}));
          connect(damper1.flange_b,fixed1.flange) annotation(Line(points = {{0,70},{32,70}}, color = {0,127,0}, smooth = Smooth.None));
          connect(damper2.flange_b,fixed2.flange) annotation(Line(points = {{0,10},{30,10}}, color = {0,127,0}, smooth = Smooth.None));
          connect(springDamper3.flange_b,fixed3.flange) annotation(Line(points = {{0, -50},{30, -50}}, color = {0,127,0}, smooth = Smooth.None));
        end Damper;
        model Oscillator "Oscillator demonstrates the use of initial conditions."
          extends Modelica.Icons.Example;
          annotation(Documentation(info = "<html>
<p>
A spring - mass system is a mechanical oscillator. If no
damping is included and the system is excited at resonance
frequency infinite amplitudes will result.
The resonant frequency is given by
omega_res = sqrt(c / m)
with:
</p>

<pre>
      c spring stiffness
      m mass
</pre>

<p>
To make sure that the system is initially at rest the initial
conditions s(start=0) and v(start=0) for the SlindingMass
are set.
If damping is added the amplitudes are bounded.
</p>
</html>

"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), experiment(StopTime = 1));
          Translational.Components.Mass mass1(L = 1, s(start =  -0.5, fixed = true), v(start = 0, fixed = true), m = 1) annotation(Placement(transformation(extent = {{ -20,40},{0,60}}, rotation = 0)));
          Translational.Components.Spring spring1(s_rel0 = 1, c = 10000) annotation(Placement(transformation(extent = {{20,40},{40,60}}, rotation = 0)));
          Translational.Components.Fixed fixed1(s0 = 1) annotation(Placement(transformation(extent = {{60,40},{80,60}}, rotation = 0)));
          Translational.Sources.Force force1 annotation(Placement(transformation(extent = {{ -60,40},{ -40,60}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sine1(freqHz = 15.9155) annotation(Placement(transformation(extent = {{ -100,40},{ -80,60}}, rotation = 0)));
          Translational.Components.Mass mass2(L = 1, s(start =  -0.5, fixed = true), v(start = 0, fixed = true), m = 1) annotation(Placement(transformation(extent = {{ -20, -60},{0, -40}}, rotation = 0)));
          Translational.Components.Spring spring2(s_rel0 = 1, c = 10000) annotation(Placement(transformation(extent = {{20, -60},{40, -40}}, rotation = 0)));
          Translational.Components.Fixed fixed2(s0 = 1) annotation(Placement(transformation(extent = {{60, -60},{80, -40}}, rotation = 0)));
          Translational.Sources.Force force2 annotation(Placement(transformation(extent = {{ -60, -60},{ -40, -40}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sine2(freqHz = 15.9155) annotation(Placement(transformation(extent = {{ -100, -60},{ -80, -40}}, rotation = 0)));
          Translational.Components.Damper damper1(d = 10) annotation(Placement(transformation(extent = {{20, -36},{40, -16}}, rotation = 0)));

        equation 
          connect(mass1.flange_b,spring1.flange_a) annotation(Line(points = {{0,50},{20,50}}, color = {0,191,0}));
          connect(spring2.flange_a,damper1.flange_a) annotation(Line(points = {{20, -50},{20, -26}}, color = {0,191,0}));
          connect(mass2.flange_b,spring2.flange_a) annotation(Line(points = {{0, -50},{20, -50}}, color = {0,191,0}));
          connect(damper1.flange_b,spring2.flange_b) annotation(Line(points = {{40, -26},{40, -50}}, color = {0,191,0}));
          connect(sine1.y,force1.f) annotation(Line(points = {{ -79,50},{ -62,50}}, color = {0,0,127}));
          connect(sine2.y,force2.f) annotation(Line(points = {{ -79, -50},{ -62, -50}}, color = {0,0,127}));
          connect(spring1.flange_b,fixed1.flange) annotation(Line(points = {{40,50},{70,50}}, color = {0,127,0}, smooth = Smooth.None));
          connect(force2.flange,mass2.flange_a) annotation(Line(points = {{ -40, -50},{ -20, -50}}, color = {0,127,0}, smooth = Smooth.None));
          connect(force1.flange,mass1.flange_a) annotation(Line(points = {{ -40,50},{ -20,50}}, color = {0,127,0}, smooth = Smooth.None));
          connect(spring2.flange_b,fixed2.flange) annotation(Line(points = {{40, -50},{70, -50}}, color = {0,127,0}, smooth = Smooth.None));
        end Oscillator;
        model Sensors "Sensors for translational systems."
          extends Modelica.Icons.Example;
          annotation(Documentation(info = "<html>
<p>
These sensors measure
</p>

<pre>
   force f in N
   position s in m
   velocity v in m/s
   acceleration a in m/s2
</pre>

<p>
Dhe measured velocity and acceleration is independent on
the flange the sensor is connected to. The position
depends on the flange (flange_a or flange_b) and the
length L of the component.
Plot PositionSensor1.s, PositionSensor2.s and SlidingMass1.s
to see the difference.
</p>
</html>

"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), experiment(StopTime = 1));
          Translational.Sensors.ForceSensor forceSensor annotation(Placement(transformation(extent = {{ -20,40},{0,60}}, rotation = 0)));
          Translational.Sensors.SpeedSensor speedSensor1 annotation(Placement(transformation(extent = {{40, -40},{60, -20}}, rotation = 0)));
          Translational.Sensors.PositionSensor positionSensor1 annotation(Placement(transformation(extent = {{40,0},{60,20}}, rotation = 0)));
          Translational.Sensors.AccSensor accSensor1 annotation(Placement(transformation(extent = {{40, -80},{60, -60}}, rotation = 0)));
          Translational.Components.Mass mass(L = 1, s(fixed = true), v(fixed = true), m = 1) annotation(Placement(transformation(extent = {{20,40},{40,60}}, rotation = 0)));
          Translational.Sources.Force force annotation(Placement(transformation(extent = {{ -60,40},{ -40,60}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sineForce(amplitude = 10, freqHz = 4) annotation(Placement(transformation(extent = {{ -100,40},{ -80,60}}, rotation = 0)));
          Translational.Sensors.PositionSensor positionSensor2 annotation(Placement(transformation(extent = {{60,40},{80,60}}, rotation = 0)));

        equation 
          connect(forceSensor.flange_b,mass.flange_a) annotation(Line(points = {{0,50},{20,50}}, color = {0,191,0}));
          connect(sineForce.y,force.f) annotation(Line(points = {{ -79,50},{ -62,50}}, color = {0,0,127}));
          connect(forceSensor.flange_a,force.flange) annotation(Line(points = {{ -20,50},{ -40,50}}, color = {0,127,0}, smooth = Smooth.None));
          connect(mass.flange_a,positionSensor1.flange) annotation(Line(points = {{20,50},{20,10},{40,10}}, color = {0,127,0}, smooth = Smooth.None));
          connect(mass.flange_a,speedSensor1.flange) annotation(Line(points = {{20,50},{20, -30},{40, -30}}, color = {0,127,0}, smooth = Smooth.None));
          connect(mass.flange_a,accSensor1.flange) annotation(Line(points = {{20,50},{20, -70},{40, -70}}, color = {0,127,0}, smooth = Smooth.None));
          connect(mass.flange_b,positionSensor2.flange) annotation(Line(points = {{40,50},{60,50}}, color = {0,127,0}, smooth = Smooth.None));
        end Sensors;
        model Friction "Use of model Stop"
          extends Modelica.Icons.Example;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -100,80},{ -80,60}}, textString = "1)", lineColor = {0,0,255}),Text(extent = {{ -100,20},{ -80,0}}, textString = "2)", lineColor = {0,0,255}),Text(extent = {{ -100, -40},{ -80, -60}}, lineColor = {0,0,255}, textString = "3)")}), Documentation(info = "<html>
<ol>
<li> Simulate and then plot stop1.f as a function of stop1.v
     This gives the Stribeck curve.</li>
<li> The same model is also available by modeling the system with a Mass and
     a SupportFriction model. The SupportFriction model defines the friction characteristic
     with a table. The table is constructed with function
     Examples.Utilities.GenerateStribeckFrictionTable(..) to generate the
     same friction characteristic as with stop1.
     The simulation results of stop1 and of model mass are therefore identical.</li>
<li> Model stop2 gives an example for a hard stop. However there
     can arise some problems with the used modeling approach (use of
     <b>reinit</b>(..), convergence problems). In this case use the ElastoGap
     to model a stop (see example Preload).</li>
</ol>
</html>
"), experiment(StopTime = 5));
          Modelica.Mechanics.Translational.Components.MassWithStopAndFriction stop1(L = 1, s(fixed = true), v(fixed = true), smax = 25, smin =  -25, m = 1, F_prop = 1, F_Coulomb = 5, F_Stribeck = 10, fexp = 2) annotation(Placement(transformation(extent = {{20,60},{40,80}}, rotation = 0)));
          Translational.Sources.Force force annotation(Placement(transformation(extent = {{ -20,60},{0,80}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sineForce(amplitude = 25, freqHz = 0.25) annotation(Placement(transformation(extent = {{ -60,60},{ -40,80}}, rotation = 0)));
          Modelica.Mechanics.Translational.Components.MassWithStopAndFriction stop2(L = 1, smax = 0.9, smin =  -0.9, F_Coulomb = 3, F_Stribeck = 5, s(start = 0, fixed = true), m = 1, F_prop = 1, fexp = 2, v(start =  -5, fixed = true)) annotation(Placement(transformation(extent = {{42, -60},{62, -40}}, rotation = 0)));
          Translational.Components.Spring spring(s_rel0 = 1, c = 500) annotation(Placement(transformation(extent = {{2, -60},{22, -40}}, rotation = 0)));
          Translational.Components.Fixed fixed2(s0 =  -1.75) annotation(Placement(transformation(extent = {{ -40, -60},{ -20, -40}}, rotation = 0)));
          Translational.Sources.Force force2 annotation(Placement(transformation(extent = {{ -22,0},{ -2,20}}, rotation = 0)));
          Components.Mass mass(m = 1, L = 1, s(fixed = true), v(fixed = true)) annotation(Placement(transformation(extent = {{10,0},{30,20}})));
          Components.SupportFriction supportFriction(f_pos = Examples.Utilities.GenerateStribeckFrictionTable(F_prop = 1, F_Coulomb = 5, F_Stribeck = 10, fexp = 2, v_max = 12, nTable = 50)) annotation(Placement(transformation(extent = {{40,0},{60,20}})));

        equation 
          connect(spring.flange_b,stop2.flange_a) annotation(Line(points = {{22, -50},{42, -50}}, color = {0,191,0}));
          connect(sineForce.y,force.f) annotation(Line(points = {{ -39,70},{ -22,70}}, color = {0,0,127}));
          connect(spring.flange_a,fixed2.flange) annotation(Line(points = {{2, -50},{ -30, -50}}, color = {0,127,0}, smooth = Smooth.None));
          connect(force.flange,stop1.flange_a) annotation(Line(points = {{0,70},{20,70}}, color = {0,127,0}, smooth = Smooth.None));
          connect(force2.flange,mass.flange_a) annotation(Line(points = {{ -2,10},{10,10}}, color = {0,127,0}, smooth = Smooth.None));
          connect(mass.flange_b,supportFriction.flange_a) annotation(Line(points = {{30,10},{40,10}}, color = {0,127,0}, smooth = Smooth.None));
          connect(sineForce.y,force2.f) annotation(Line(points = {{ -39,70},{ -30,70},{ -30,10},{ -24,10}}, color = {0,0,127}, smooth = Smooth.None));
        end Friction;
        model PreLoad "Preload of a spool using ElastoGap models."
          extends Modelica.Icons.Example;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -98, -68},{102, -94}}, textString = "positive force => spool moves in positive direction ", lineColor = {0,0,255}),Text(extent = {{ -32, -46},{38, -62}}, textString = "Simulate for 100 s", lineColor = {0,0,255}),Text(extent = {{ -100, -54},{100, -80}}, lineColor = {0,0,255}, textString = "plot spool.s as a function of force.f")}), Documentation(info = "<html>
<p>
When designing hydraulic valves it is often necessary to hold the spool in
a certain position as long as an external force is below a threshold value.
If this force exceeds the treshold value a linear relation between force
and position is desired.
There are designs that need only one spring to accomplish this task. Using
the ElastoGap elements this design can be modelled easily.
Drawing of spool.
</p>

<p>
<<IMG SRC=../Images/Translational/PreLoad.png>
</p>

<p>
<IMG SRC=../Images/Translational/PreLoad3.png>
</p>

<p>
<IMG SRC=../Images/Translational/PreLoad4.png>
</p>

<p>
Spool position s as a function of working force f.
</p>

<p>
<IMG SRC=../Images/Translational/PreLoad2.png>
</p>
</html>

"), experiment(StopTime = 100));
          Translational.Components.ElastoGap innerContactA(c = 1000000.0, d = 250, s_rel0 = 0.001) annotation(Placement(transformation(extent = {{ -70,20},{ -50,40}}, rotation = 0)));
          Translational.Components.ElastoGap innerContactB(c = 1000000.0, d = 250, s_rel0 = 0.001) annotation(Placement(transformation(extent = {{50,20},{70,40}}, rotation = 0)));
          Translational.Components.Mass spool(L = 0.19, m = 0.15, s(start = 0.01475, fixed = true), v(fixed = true)) annotation(Placement(transformation(extent = {{0, -40},{40,0}}, rotation = 0)));
          Translational.Components.Fixed fixedLe(s0 =  -0.0955) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 270, origin = { -80,90})));
          Translational.Components.Mass springPlateA(m = 0.01, L = 0.002, s(start =  -0.093, fixed = true), v(fixed = true)) annotation(Placement(transformation(extent = {{ -38,60},{ -18,80}}, rotation = 0)));
          Translational.Components.Mass springPlateB(m = 0.01, s(start =  -0.06925, fixed = true), L = 0.002, v(fixed = true)) annotation(Placement(transformation(extent = {{20,60},{40,80}}, rotation = 0)));
          Translational.Components.Spring spring(c = 20000.0, s_rel0 = 0.025) annotation(Placement(transformation(extent = {{ -10,60},{10,80}}, rotation = 0)));
          Translational.Components.ElastoGap outerContactA(c = 1000000.0, d = 250, s_rel0 = 0.0015) annotation(Placement(transformation(extent = {{ -70,60},{ -50,80}}, rotation = 0)));
          Translational.Components.ElastoGap outerContactB(c = 1000000.0, d = 250, s_rel0 = 0.0015) annotation(Placement(transformation(extent = {{50,60},{70,80}}, rotation = 0)));
          Translational.Components.Rod rod1(L = 0.007) annotation(Placement(transformation(extent = {{ -40,30},{ -20,50}}, rotation = 0)));
          Translational.Components.Damper friction(d = 2500) annotation(Placement(transformation(origin = { -80,50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Translational.Sources.Force force annotation(Placement(transformation(extent = {{ -38, -30},{ -18, -10}}, rotation = 0)));
          Translational.Components.Rod housing(L = 0.0305) annotation(Placement(transformation(extent = {{ -10,80},{10,100}}, rotation = 0)));
          Translational.Components.Rod rod3(L = 0.00575) annotation(Placement(transformation(extent = {{ -40, -2},{ -20,18}}, rotation = 0)));
          Translational.Components.Rod rod4(L = 0.00575) annotation(Placement(transformation(extent = {{20, -2},{40,18}}, rotation = 0)));
          Translational.Components.Rod rod2(L = 0.007) annotation(Placement(transformation(extent = {{20,30},{40,50}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sineForce(amplitude = 150, freqHz = 0.01) annotation(Placement(transformation(extent = {{ -78, -30},{ -58, -10}}, rotation = 0)));

        equation 
          connect(outerContactA.flange_b,springPlateA.flange_a) annotation(Line(points = {{ -50,70},{ -38,70}}, color = {0,191,0}));
          connect(springPlateA.flange_b,spring.flange_a) annotation(Line(points = {{ -18,70},{ -18,70},{ -10,70}}, color = {0,191,0}));
          connect(spring.flange_b,springPlateB.flange_a) annotation(Line(points = {{10,70},{20,70}}, color = {0,191,0}));
          connect(springPlateB.flange_b,outerContactB.flange_a) annotation(Line(points = {{40,70},{40,70},{50,70}}, color = {0,191,0}));
          connect(outerContactB.flange_b,housing.flange_b) annotation(Line(points = {{70,70},{70,90},{10,90}}, color = {0,191,0}));
          connect(springPlateA.flange_b,rod1.flange_a) annotation(Line(points = {{ -18,70},{ -18,52},{ -40,52},{ -40,40}}, color = {0,191,0}));
          connect(innerContactA.flange_a,rod3.flange_a) annotation(Line(points = {{ -70,30},{ -80,30},{ -80,8},{ -40,8}}, color = {0,191,0}));
          connect(innerContactA.flange_b,rod1.flange_b) annotation(Line(points = {{ -50,30},{ -20,30},{ -20,40}}, color = {0,191,0}));
          connect(rod2.flange_a,innerContactB.flange_a) annotation(Line(points = {{20,40},{20,30},{50,30}}, color = {0,191,0}));
          connect(rod4.flange_b,innerContactB.flange_b) annotation(Line(points = {{40,8},{70,8},{70,30}}, color = {0,191,0}));
          connect(friction.flange_b,rod3.flange_a) annotation(Line(points = {{ -80,40},{ -80,8},{ -40,8}}, color = {0,191,0}));
          connect(rod3.flange_b,rod4.flange_a) annotation(Line(points = {{ -20,8},{20,8}}, color = {0,191,0}));
          connect(rod2.flange_b,springPlateB.flange_a) annotation(Line(points = {{40,40},{40,52},{20,52},{20,70}}, color = {0,191,0}));
          connect(spool.flange_a,rod4.flange_a) annotation(Line(points = {{0, -20},{0,8},{20,8}}, color = {0,191,0}));
          connect(sineForce.y,force.f) annotation(Line(points = {{ -57, -20},{ -46, -20},{ -40, -20}}, color = {0,0,127}));
          connect(force.flange,spool.flange_a) annotation(Line(points = {{ -18, -20},{0, -20}}, color = {0,127,0}, smooth = Smooth.None));
          connect(outerContactA.flange_a,fixedLe.flange) annotation(Line(points = {{ -70,70},{ -80,70},{ -80,90}}, color = {0,127,0}, smooth = Smooth.None));
          connect(housing.flange_a,fixedLe.flange) annotation(Line(points = {{ -10,90},{ -80,90}}, color = {0,127,0}, smooth = Smooth.None));
          connect(friction.flange_a,fixedLe.flange) annotation(Line(points = {{ -80,60},{ -80,90}}, color = {0,127,0}, smooth = Smooth.None));
        end PreLoad;
        model ElastoGap "Demonstrate usgae of ElastoGap"
          extends Modelica.Icons.Example;
          annotation(Documentation(info = "<html>
<p>
This model demonstrates the effect of ElastoGaps on eigenfrequency:<br>
Plot mass1.s and mass2.s as well as mass1.v and mass2.v<br><br>
mass1 is moved by both spring forces all the time.<br>
Since elastoGap1 lifts off at s &gt; -0.5 m and elastoGap2 lifts off s &lt; +0.5 m,
mass2 moves freely as long as -0.5 m &lt; s &lt; +0.5 m.
</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 10));
          Components.Fixed fixed annotation(Placement(transformation(extent = {{ -10, -10},{10,10}})));
          Components.Rod rod1(L = 2) annotation(Placement(transformation(extent = {{ -40, -10},{ -20,10}})));
          Components.Rod rod2(L = 2) annotation(Placement(transformation(extent = {{20, -10},{40,10}})));
          Components.SpringDamper springDamper1(c = 10, s_rel0 = 1, s_rel(fixed = false, start = 1), d = 1.5) annotation(Placement(transformation(extent = {{ -40,20},{ -20,40}})));
          Components.SpringDamper springDamper2(c = 10, s_rel0 = 1, s_rel(fixed = false, start = 1), d = 1.5) annotation(Placement(transformation(extent = {{20,20},{40,40}})));
          Components.Mass mass1(s(fixed = true, start = 2), L = 0, m = 1, v(fixed = true)) annotation(Placement(transformation(extent = {{ -10,20},{10,40}})));
          Components.ElastoGap elastoGap1(c = 10, s_rel(fixed = false, start = 1.5), s_rel0 = 1.5, d = 1.5) annotation(Placement(transformation(extent = {{ -40, -40},{ -20, -20}})));
          Components.ElastoGap elastoGap2(c = 10, s_rel(fixed = false, start = 1.5), s_rel0 = 1.5, d = 1.5) annotation(Placement(transformation(extent = {{20, -40},{40, -20}})));
          Components.Mass mass2(s(fixed = true, start = 2), L = 0, m = 1, v(fixed = true)) annotation(Placement(transformation(extent = {{ -10, -40},{10, -20}})));
          parameter SI.TranslationalDampingConstant d = 1.5 "damping constant";

        equation 
          connect(rod1.flange_b,fixed.flange) annotation(Line(points = {{ -20,0},{0,0}}, color = {0,127,0}, smooth = Smooth.None));
          connect(fixed.flange,rod2.flange_a) annotation(Line(points = {{0,0},{20,0}}, color = {0,127,0}, smooth = Smooth.None));
          connect(springDamper1.flange_a,rod1.flange_a) annotation(Line(points = {{ -40,30},{ -48,30},{ -48,0},{ -40,0}}, color = {0,127,0}, smooth = Smooth.None));
          connect(springDamper2.flange_b,rod2.flange_b) annotation(Line(points = {{40,30},{50,30},{50,0},{40,0}}, color = {0,127,0}, smooth = Smooth.None));
          connect(springDamper1.flange_b,mass1.flange_a) annotation(Line(points = {{ -20,30},{ -10,30}}, color = {0,127,0}, smooth = Smooth.None));
          connect(mass1.flange_b,springDamper2.flange_a) annotation(Line(points = {{10,30},{20,30}}, color = {0,127,0}, smooth = Smooth.None));
          connect(rod1.flange_a,elastoGap1.flange_a) annotation(Line(points = {{ -40,0},{ -48,0},{ -48, -30},{ -40, -30}}, color = {0,127,0}, smooth = Smooth.None));
          connect(rod2.flange_b,elastoGap2.flange_b) annotation(Line(points = {{40,0},{50,0},{50, -30},{40, -30}}, color = {0,127,0}, smooth = Smooth.None));
          connect(elastoGap1.flange_b,mass2.flange_a) annotation(Line(points = {{ -20, -30},{ -10, -30}}, color = {0,127,0}, smooth = Smooth.None));
          connect(mass2.flange_b,elastoGap2.flange_a) annotation(Line(points = {{10, -30},{20, -30}}, color = {0,127,0}, smooth = Smooth.None));
        end ElastoGap;
        model Brake "Demonstrate braking of a translational moving mass"
          extends Modelica.Icons.Example;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 2), Documentation(info = "<html>
<p>
This model consists of a mass with an initial velocity of 1 m/s.
After 0.1 s, a brake is activated and it is shown that the mass decelerates until
it arrives at rest and remains at rest. Two versions of this system are present,
one where the brake is implicitly grounded and one where it is explicitly grounded.
</p>

</html>"));
          Modelica.Mechanics.Translational.Components.Brake brake(fn_max = 1, useSupport = false) annotation(Placement(transformation(extent = {{6,40},{26,20}})));
          Modelica.Mechanics.Translational.Components.Mass mass1(m = 1, s(fixed = true), v(start = 1, fixed = true)) annotation(Placement(transformation(extent = {{ -34,20},{ -14,40}})));
          Modelica.Blocks.Sources.Step step(startTime = 0.1, height = 2) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0, origin = { -24, -10})));
          Modelica.Mechanics.Translational.Components.Brake brake1(fn_max = 1, useSupport = true) annotation(Placement(transformation(extent = {{6, -60},{26, -40}})));
          Modelica.Mechanics.Translational.Components.Mass mass2(m = 1, s(fixed = true), v(start = 1, fixed = true)) annotation(Placement(transformation(extent = {{ -34, -60},{ -14, -40}})));
          Modelica.Mechanics.Translational.Components.Fixed fixed annotation(Placement(transformation(extent = {{6, -80},{26, -60}})));

        equation 
          connect(mass1.flange_b,brake.flange_a) annotation(Line(points = {{ -14,30},{6,30}}, color = {0,127,0}, smooth = Smooth.None));
          connect(step.y,brake.f_normalized) annotation(Line(points = {{ -13, -10},{16, -10},{16,19}}, color = {0,0,127}, smooth = Smooth.None));
          connect(mass2.flange_b,brake1.flange_a) annotation(Line(points = {{ -14, -50},{6, -50}}, color = {0,127,0}, smooth = Smooth.None));
          connect(step.y,brake1.f_normalized) annotation(Line(points = {{ -13, -10},{16, -10},{16, -39}}, color = {0,0,127}, smooth = Smooth.None));
          connect(fixed.flange,brake1.support) annotation(Line(points = {{16, -70},{16, -60}}, color = {0,127,0}, smooth = Smooth.None));
        end Brake;
        package Utilities "Utility classes used by the Example models"
          extends Modelica.Icons.Library;
          function GenerateStribeckFrictionTable "Generate stribeck friction table for example Friction for the SupportFriction"
            input Real F_prop(final unit = "N.s/m", final min = 0) "Velocity dependent friction coefficient";
            input Modelica.SIunits.Force F_Coulomb "Constant friction: Coulomb force";
            input Modelica.SIunits.Force F_Stribeck "Stribeck effect";
            input Real fexp(final unit = "s/m", final min = 0) "Exponential decay";
            input Real v_max "Generate table from v=0 ... v_max";
            input Integer nTable(min = 2) = 100 "Number of table points";
            output Real table[nTable,2] "Friction table";
          algorithm 
            for i in 1:nTable loop
                          table[i,1]:=(v_max * (i - 1)) / (nTable - 1);
              table[i,2]:=F_Coulomb + F_prop * table[i,1] + F_Stribeck * exp( -fexp * table[i,1]);

            end for;
            annotation(Documentation(info = "<html>
<p>
Returns a table with the friction characteristic table[nTable,2] = [0, f1; ....; v_max, fn], where the first
column is the velocity v in the range 0..v_max and the second column is the friction force
according to the stribeck curve:
</p>
<pre>
  F_Coulomb + F_prop*v + F_Stribeck*exp(-fexp*v);
</pre>

</html>"));
          end GenerateStribeckFrictionTable;
        end Utilities;
      end Examples;
      package Components "Components for 1D translational mechanical drive trains"
        extends Modelica.Icons.Library2;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -67, -66},{44, -6}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {255,255,255})}), Documentation(info = "<html>
<p>
This package contains basic components 1D mechanical translational drive trains.
</p>
</html>"));
        model Fixed "Fixed flange"
          parameter SI.Position s0 = 0 "fixed offset position of housing";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -80, -40},{80, -40}}, color = {0,0,0}),Line(points = {{80, -40},{40, -80}}, color = {0,0,0}),Line(points = {{40, -40},{0, -80}}, color = {0,0,0}),Line(points = {{0, -40},{ -40, -80}}, color = {0,0,0}),Line(points = {{ -40, -40},{ -80, -80}}, color = {0,0,0}),Line(points = {{0, -40},{0, -10}}, color = {0,0,0}),Text(extent = {{0, -90},{0, -150}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
The <i>flange</i> of a 1D translational mechanical system <i>fixed</i>
at an position s0 in the <i>housing</i>. May be used:
</p>
<ul>
<li> to connect a compliant element, such as a spring or a damper,
     between a sliding mass and the housing.
<li> to fix a rigid element, such as a sliding mass, at a specific
     position.
</ul>

</HTML>
"));
          Interfaces.Flange_b flange annotation(Placement(transformation(origin = {0,0}, extent = {{ -10,10},{10, -10}}, rotation = 180)));

        equation 
          flange.s = s0;
        end Fixed;
        model Mass "Sliding mass with inertia"
          parameter SI.Mass m(min = 0, start = 1) "mass of the sliding mass";
          parameter StateSelect stateSelect = StateSelect.default "Priority to use s and v as states" annotation(Dialog(tab = "Advanced"));
          extends Translational.Interfaces.PartialRigid(L = 0, s(start = 0, stateSelect = stateSelect));
          SI.Velocity v(start = 0, stateSelect = stateSelect) "absolute velocity of component";
          SI.Acceleration a(start = 0) "absolute acceleration of component";
          annotation(Documentation(info = "<html>
<p>
Sliding mass with <i>inertia, without friction</i> and two rigidly connected flanges.
</p>
<p>
The sliding mass has the length L, the position coordinate s is in the middle.
Sign convention: A positive force at flange flange_a moves the sliding mass in the positive direction.
A negative force at flange flange_a moves the sliding mass to the negative direction.
</p>

</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -100,0},{ -55,0}}, color = {0,127,0}),Line(points = {{55,0},{100,0}}, color = {0,127,0}),Rectangle(extent = {{ -55, -30},{56,30}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {255,255,255}),Polygon(points = {{50, -90},{20, -80},{20, -100},{50, -90}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{ -60, -90},{20, -90}}, color = {0,0,0}),Text(extent = {{0,100},{0,40}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100, -40},{100, -80}}, lineColor = {0,0,255}, textString = "m=%m")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -100,0},{ -55,0}}, color = {0,127,0}),Line(points = {{55,0},{100,0}}, color = {0,127,0}),Rectangle(extent = {{ -55, -30},{55,30}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {255,255,255}),Polygon(points = {{50, -90},{20, -80},{20, -100},{50, -90}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{ -60, -90},{20, -90}}, color = {0,0,0}),Line(points = {{ -100, -29},{ -100, -61}}, color = {0,0,0}),Line(points = {{100, -61},{100, -28}}, color = {0,0,0}),Line(points = {{ -98, -60},{98, -60}}, color = {0,0,0}),Polygon(points = {{ -101, -60},{ -96, -59},{ -96, -61},{ -101, -60}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{100, -60},{95, -61},{95, -59},{100, -60}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -44, -41},{51, -57}}, textString = "Length L", lineColor = {0,0,255}),Line(points = {{0,30},{0,53}}, color = {0,0,0}),Line(points = {{ -72,40},{1,40}}, color = {0,0,0}),Polygon(points = {{ -7,42},{ -7,38},{ -1,40},{ -7,42}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -61,53},{ -9,42}}, textString = "Position s", lineColor = {0,0,255})}));

        equation 
          v = der(s);
          a = der(v);
          m * a = flange_a.f + flange_b.f;
        end Mass;
        model Rod "Rod without inertia"
          extends Translational.Interfaces.PartialRigid;
          annotation(Documentation(info = "<html>
<p>
Rod <i>without inertia</i> and two rigidly connected flanges.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -100,0},{ -55,0}}, color = {0,127,0}),Line(points = {{53,0},{99,0}}, color = {0,127,0}),Polygon(points = {{50, -90},{20, -80},{20, -100},{50, -90}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{ -60, -90},{20, -90}}, color = {0,0,0}),Rectangle(extent = {{ -55,10},{53, -10}}, lineColor = {160,160,164}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{0,100},{0,40}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100, -20},{100, -80}}, lineColor = {0,0,255}, textString = "L=%L")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -100,0},{ -55,0}}, color = {0,127,0}),Line(points = {{54,0},{100,0}}, color = {0,127,0}),Polygon(points = {{50, -90},{20, -80},{20, -100},{50, -90}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{ -60, -90},{20, -90}}, color = {0,0,0}),Rectangle(extent = {{ -55,3},{53, -4}}, lineColor = {160,160,164}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -100, -29},{ -100, -61}}, color = {0,0,0}),Line(points = {{100, -61},{100, -28}}, color = {0,0,0}),Line(points = {{ -98, -60},{98, -60}}, color = {0,0,0}),Polygon(points = {{ -101, -60},{ -96, -59},{ -96, -61},{ -101, -60}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{100, -60},{95, -61},{95, -59},{100, -60}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -44, -41},{51, -57}}, textString = "Length L", lineColor = {0,0,255})}));

        equation 
          0 = flange_a.f + flange_b.f;
        end Rod;
        model Spring "Linear 1D translational spring"
          extends Translational.Interfaces.PartialCompliant;
          parameter SI.TranslationalSpringConstant c(final min = 0, start = 1) "spring constant ";
          parameter SI.Distance s_rel0 = 0 "unstretched spring length";
          annotation(Documentation(info = "<html>
<p>
A <i>linear 1D translational spring</i>. The component can be connected either
between two sliding masses, or between
a sliding mass and the housing (model Fixed), to describe
a coupling of the sliding mass with the housing via a spring.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -60, -90},{20, -90}}, color = {0,0,0}),Polygon(points = {{50, -90},{20, -80},{20, -100},{50, -90}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{0,110},{0,50}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -86,0},{ -60,0},{ -44, -30},{ -16,30},{14, -30},{44,30},{60,0},{84,0}}, color = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -100,0},{ -100,65}}, color = {128,128,128}),Line(points = {{100,0},{100,65}}, color = {128,128,128}),Line(points = {{ -100,60},{100,60}}, color = {128,128,128}),Polygon(points = {{90,63},{100,60},{90,57},{90,63}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{ -22,62},{18,87}}, lineColor = {0,0,255}, textString = "s_rel"),Line(points = {{ -86,0},{ -60,0},{ -44, -30},{ -16,30},{14, -30},{44,30},{60,0},{84,0}}, color = {0,0,0})}));

        equation 
          f = c * (s_rel - s_rel0);
        end Spring;
        model Damper "Linear 1D translational damper"
          extends Translational.Interfaces.PartialCompliantWithRelativeStates;
          parameter SI.TranslationalDampingConstant d(final min = 0, start = 0) "damping constant";
          annotation(Documentation(info = "<html>
<p>
<i>Linear, velocity dependent damper</i> element. It can be either connected
between a sliding mass and the housing (model Fixed), or
between two sliding masses.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -90,0},{ -60,0}}, color = {0,0,0}),Line(points = {{ -60, -30},{ -60,30}}, color = {0,0,0}),Line(points = {{ -60, -30},{60, -30}}, color = {0,0,0}),Line(points = {{ -60,30},{60,30}}, color = {0,0,0}),Rectangle(extent = {{ -60,30},{30, -30}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{30,0},{90,0}}, color = {0,0,0}),Polygon(points = {{50, -90},{20, -80},{20, -100},{50, -90}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{ -60, -90},{20, -90}}, color = {0,0,0}),Text(extent = {{0,106},{0,46}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -90,0},{ -60,0}}, color = {0,0,0}),Line(points = {{ -60, -30},{ -60,30}}, color = {0,0,0}),Line(points = {{ -60, -30},{60, -30}}, color = {0,0,0}),Line(points = {{ -60,30},{60,30}}, color = {0,0,0}),Rectangle(extent = {{ -60,30},{30, -30}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{30,0},{90,0}}, color = {0,0,0}),Line(points = {{ -50,60},{50,60}}, color = {128,128,128}),Polygon(points = {{50,63},{60,60},{50,57},{50,63}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{ -40,68},{38,90}}, lineColor = {128,128,128}, textString = "der(s_rel)")}));

        equation 
          f = d * v_rel;
        end Damper;
        model SpringDamper "Linear 1D translational spring and damper in parallel"
          extends Translational.Interfaces.PartialCompliantWithRelativeStates;
          parameter SI.TranslationalSpringConstant c(final min = 0, start = 1) "spring constant";
          parameter SI.TranslationalDampingConstant d(final min = 0, start = 1) "damping constant";
          parameter SI.Position s_rel0 = 0 "unstretched spring length";
          annotation(Documentation(info = "<html>
<p>
A <i>spring and damper element connected in parallel</i>.
The component can be
connected either between two sliding masses to describe the elasticity
and damping, or between a sliding mass and the housing (model Fixed),
to describe a coupling of the sliding mass with the housing via a spring/damper.
<p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80,40},{ -60,40},{ -45,10},{ -15,70},{15,10},{45,70},{60,40},{80,40}}, color = {0,0,0}),Line(points = {{ -80,40},{ -80, -70}}, color = {0,0,0}),Line(points = {{ -80, -70},{ -52, -70}}, color = {0,0,0}),Rectangle(extent = {{ -52, -49},{38, -91}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -52, -49},{68, -49}}, color = {0,0,0}),Line(points = {{ -51, -91},{69, -91}}, color = {0,0,0}),Line(points = {{38, -70},{80, -70}}, color = {0,0,0}),Line(points = {{80,40},{80, -70}}, color = {0,0,0}),Line(points = {{ -90,0},{ -80,0}}, color = {0,0,0}),Line(points = {{80,0},{90,0}}, color = {0,0,0}),Polygon(points = {{53, -18},{23, -8},{23, -28},{53, -18}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{ -57, -18},{23, -18}}, color = {0,0,0}),Text(extent = {{1,140},{1,80}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80,32},{ -58,32},{ -43,2},{ -13,62},{17,2},{47,62},{62,32},{80,32}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -100,31},{ -100,96}}, color = {128,128,128}),Line(points = {{100,29},{100,94}}, color = {128,128,128}),Line(points = {{ -98,82},{100,82}}, color = {128,128,128}),Polygon(points = {{90,85},{100,82},{90,79},{90,85}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{ -21,61},{19,86}}, lineColor = {0,0,255}, textString = "s_rel"),Rectangle(extent = {{ -52, -28},{38, -72}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -51, -72},{69, -72}}, color = {0,0,0}),Line(points = {{ -52, -28},{68, -28}}, color = {0,0,0}),Line(points = {{38, -50},{80, -50}}, color = {0,0,0}),Line(points = {{ -80, -50},{ -52, -50}}, color = {0,0,0}),Line(points = {{ -80,32},{ -80, -50}}, color = {0,0,0}),Line(points = {{80,32},{80, -50}}, color = {0,0,0}),Line(points = {{ -90,0},{ -80,0}}, color = {0,0,0}),Line(points = {{90,0},{80,0}}, color = {0,0,0})}));

        equation 
          f = c * (s_rel - s_rel0) + d * v_rel;
        end SpringDamper;
        model ElastoGap "1D translational spring damper combination with gap"
          extends Modelica.Mechanics.Translational.Interfaces.PartialCompliantWithRelativeStates;
          parameter Real c(final unit = "N/m", final min = 0, start = 1) "Spring constant";
          parameter Real d(final unit = "N/ (m/s)", final min = 0, start = 1) "Damping constant";
          parameter Modelica.SIunits.Position s_rel0 = 0 "Unstretched spring length";
          parameter Real n(final min = 1) = 1 "Exponent of spring force ( f_c = -c*|s_rel-s_rel0|^n )";
          annotation(Window(x = 0.23, y = 0.04, width = 0.62, height = 0.69), Documentation(info = "<html>
<p>
A linear spring damper combination that can lift off.
The component can be connected between a sliding mass and the housing (model
<a href=\"Modelica://Modelica.Mechanics.Translational.Components.Fixed\">Fixed</a>),
to describe the contact of a sliding mass with the housing.
</p>

<p>
As long as s_rel &gt; s_rel0, no force is exerted (s_rel = flange_b.s - flange_a.s).
If s_rel &le; s_rel0, the contact force is basically computed with a linear
spring/damper characteristic. With parameter n&ge;1 (exponent of spring force),
a nonlinear spring force can be modeled:
</p>

<pre>
   desiredContactForce = c*|s_rel - s_rel0|^n + d*<b>der</b>(s_rel)
</pre>

<p>
Note, Hertzian contact is described by:
</p>
<ul>
<li> Contact between two metallic spheres: n=1.5</li>
<li> Contact between two metallic plates: n=1</li>
</ul>

<p>
The above force law leads to the following difficulties:
</p>

<ol>
<li> If the damper force becomes larger as the spring force and with opposite sign,
     the contact force would be \"pulling/sticking\" which is unphysical, since during
     contact only pushing forces can occur.</li>

<li> When contact occurs with a non-zero relative speed (which is the usual
     situation), the damping force has a non-zero value and therefore the contact
     force changes discontinuously at s_rel = s_rel0. Again, this is not physical
     because the force can only change continuously. (Note, this component is not an
     idealized model where a steep characteristic is approximated by a discontinuity,
     but it shall model the steep characteristic.)</li>
</ol>

<p>
In the literature there are several proposals to fix problem (2). Especially, often
the following model is used (see, e.g.,
Lankarani, Nikravesh: Continuous Contact Force Models for Impact
Analysis in Multibody Systems, Nonlinear Dynamics 5, pp. 193-207, 1994,
<a href=\"http://www.springerlink.com/content/h50x61270q06p65n/fulltext.pdf\">pdf-download</a>):
</p>

<pre>
   f = c*s_rel^n + (d*s_rel^n)*<b>der</b>(s_rel)
</pre>

<p>
However, this and other models proposed in literature violate
issue (1), i.e., unphysical pulling forces can occur (if d*<b>der</b>(s_rel)
becomes large enough). Note, if the force law is of the form \"f = f_c + f_d\", then a
necessary condition is that |f_d| &le; |f_c|, otherwise (1) and (2) are violated.
For this reason, the most simplest approach is used in the ElastoGap model
to fix both problems by using this necessary condition in the force law directly.
If s_rel0 = 0, the equations are:
</p>

<pre>
    <b>if</b> s_rel &ge; 0 <b>then</b>
       f = 0;    // contact force
    <b>else</b>
       f_c  = -c*|s_rel|^n;          // contact spring force (Hertzian contact force)
       f_d2 = d*<b>der</b>(s_rel);         // linear contact damper force
       f_d  = <b>if</b> f_d2 &lt;  f_c <b>then</b>  f_c <b>else</b>
              <b>if</b> f_d2 &gt; -f_c <b>then</b> -f_c <b>else</b> f_d2;  // bounded damper force
       f    = f_c + f_d;            // contact force
    <b>end if</b>;
</pre>

<p>
Note, since |f_d| &le; |f_c|, pulling forces cannot occur and the contact force
is always continuous, especially around the start of the penetration at s_rel = s_rel0.
</p>

<p>
In the next figure, a typical simulation with the ElastoGap model is shown
(<a href=\"Modelica://Modelica.Mechanics.Translational.Examples.ElastoGap\">Examples.ElastoGap</a>)
where the different effects are visualized:
</p>

<ol>
<li> Curve 1 (elastoGap1.f) is the unmodified contact force, i.e., the linear spring/damper
     characteristic. A pulling/sticking force is present at the end of the contact.</li>
<li> Curve 2 (elastoGap2.f) is the contact force, where the force is explicitly set to
     zero when pulling/sticking occurs. The contact force is discontinuous at being of
     the contact.</li>
<li> Curve 3 (elastoGap3.f) is the ElastoGap model of this library. No discontinuity and no
     pulling/sticking occurs.</li>
</ol>

<p align=\"center\">
<img src=\"../Images/Translational/ElastoGap1.png\">
</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -100,0},{ -50,0}}, color = {0,127,0}),Line(points = {{ -48,34},{ -48, -46}}, color = {0,0,0}, thickness = 1),Line(points = {{8,40},{8,2}}, color = {0,0,0}),Line(points = {{ -2,0},{38,0},{38,44},{ -2,44}}, color = {0,0,0}),Line(points = {{38,22},{72,22}}, color = {0,0,0}),Line(points = {{ -12, -38},{ -12,20}}, color = {0,0,0}, thickness = 1),Line(points = {{ -12,22},{8,22}}, color = {0,0,0}),Line(points = {{ -12, -38},{ -2, -38}}, color = {0,0,0}),Line(points = {{72,0},{100,0}}, color = {0,127,0}),Line(points = {{72,22},{72, -42}}, color = {0,0,0}),Line(points = {{ -2, -38},{10, -28},{22, -48},{38, -28},{50, -48},{64, -28},{72, -40}}, color = {0,0,0}),Rectangle(extent = {{8,44},{38,0}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -28, -80},{12, -55}}, lineColor = {0,0,255}, textString = "s_rel"),Line(points = {{ -100, -29},{ -100, -61}}, color = {0,0,0}),Line(points = {{100, -61},{100, -28}}, color = {0,0,0}),Line(points = {{ -98, -60},{98, -60}}, color = {0,0,0}),Polygon(points = {{ -101, -60},{ -96, -59},{ -96, -61},{ -101, -60}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{100, -60},{95, -61},{95, -59},{100, -60}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -98,0},{ -48,0}}, color = {0,127,0}),Line(points = {{ -48,34},{ -48, -46}}, color = {0,0,0}, thickness = 1),Line(points = {{8,40},{8,2}}, color = {0,0,0}),Line(points = {{ -2,0},{38,0},{38,44},{ -2,44}}, color = {0,0,0}),Line(points = {{38,22},{72,22}}, color = {0,0,0}),Line(points = {{ -12, -38},{ -12,20}}, color = {0,0,0}, thickness = 1),Line(points = {{ -12,22},{8,22}}, color = {0,0,0}),Line(points = {{ -12, -38},{ -2, -38}}, color = {0,0,0}),Line(points = {{72,0},{98,0}}, color = {0,127,0}),Line(points = {{72,22},{72, -42}}, color = {0,0,0}),Line(points = {{ -2, -38},{10, -28},{22, -48},{38, -28},{50, -48},{64, -28},{72, -40}}, color = {0,0,0}),Rectangle(extent = {{8,44},{38,0}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -60, -90},{20, -90}}, color = {0,0,0}),Polygon(points = {{50, -90},{20, -80},{20, -100},{50, -90}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{0,120},{0,60}}, textString = "%name", lineColor = {0,0,255})}), uses(Modelica(version = "2.2.2")));
          Boolean contact "=true, if contact, otherwise no contact";
        protected 
          Modelica.SIunits.Force f_c "Spring force";
          Modelica.SIunits.Force f_d2 "Linear damping force";
          Modelica.SIunits.Force f_d "Linear damping force which is limited by spring force (|f_d| <= |f_c|)";

        equation 
          contact = s_rel < s_rel0;
          f_c = smooth(1, noEvent(if contact then  -c * abs(s_rel - s_rel0) ^ n else 0));
          f_d2 = if contact then d * v_rel else 0;
          f_d = smooth(0, noEvent(if contact then if f_d2 < f_c then f_c else if f_d2 >  -f_c then  -f_c else f_d2 else 0));
          f = f_c + f_d;
        end ElastoGap;
        model SupportFriction "Coulomb friction in support"
          annotation(Documentation(info = "<html>
<p>
This element describes <b>Coulomb friction</b> in <b>support</b>,
i.e., a frictional force acting between a flange and the housing.
The positive sliding friction force \"f\" has to be defined
by table \"f_pos\" as function of the absolute velocity \"v\".
E.g.
<p>
<pre>
       v |   f
      ---+-----
       0 |   0
       1 |   2
       2 |   5
       3 |   8
</pre>
<p>
gives the following table:
</p>
<pre>
   f_pos = [0, 0; 1, 2; 2, 5; 3, 8];
</pre>
<p>
Currently, only linear interpolation in the table is supported.
Outside of the table, extrapolation through the last
two table entries is used. It is assumed that the negative
sliding friction force has the same characteristic with negative
values. Friction is modelled in the following way:
</p>
<p>
When the absolute velocity \"v\" is not zero, the friction force
is a function of v and of a constant normal force. This dependency
is defined via table f_pos and can be determined by measurements,
e.g. by driving the gear with constant velocity and measuring the
needed driving force (= friction force).
</p>
<p>
When the absolute velocity becomes zero, the elements
connected by the friction element become stuck, i.e., the absolute
position remains constant. In this phase the friction force is
calculated from a force balance due to the requirement, that
the absolute acceleration shall be zero.  The elements begin
to slide when the friction force exceeds a threshold value,
called the maximum static friction force, computed via:
</p>
<pre>
   maximum_static_friction = <b>peak</b> * sliding_friction(v=0)  (<b>peak</b> >= 1)
</pre>
<p>
This procedure is implemented in a \"clean\" way by state events and
leads to continuous/discrete systems of equations if friction elements
are dynamically coupled which have to be solved by appropriate
numerical methods. The method is described in:
</p>
<dl>
<dt>Otter M., Elmqvist H., and Mattsson S.E. (1999):
<dd><b>Hybrid Modeling in Modelica based on the Synchronous
    Data Flow Principle</b>. CACSD'99, Aug. 22.-26, Hawaii.
</dl>
<p>
More precise friction models take into account the elasticity of the
material when the two elements are \"stuck\", as well as other effects,
like hysteresis. This has the advantage that the friction element can
be completely described by a differential equation without events. The
drawback is that the system becomes stiff (about 10-20 times slower
simulation) and that more material constants have to be supplied which
requires more sophisticated identification. For more details, see the
following references, especially (Armstrong and Canudas de Witt 1996):
</p>
<dl>
<dt>Armstrong B. (1991):
<dd><b>Control of Machines with Friction</b>. Kluwer Academic
    Press, Boston MA.<br><br>
<dt>Armstrong B., and Canudas de Wit C. (1996):
<dd><b>Friction Modeling and Compensation.</b>
    The Control Handbook, edited by W.S.Levine, CRC Press,
    pp. 1369-1382.<br><br>
<dt>Canudas de Wit C., Olsson H., Astroem K.J., and Lischinsky P. (1995):
<dd><b>A new model for control of systems with friction.</b>
    IEEE Transactions on Automatic Control, Vol. 40, No. 3, pp. 419-425.<br><br>
</dl>


</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -90,10},{90, -10}}, lineColor = {0,127,0}, fillColor = {215,215,215}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -48, -10},{ -28, -30}}, lineColor = {95,95,95}, fillPattern = FillPattern.Sphere, fillColor = {175,175,175}),Ellipse(extent = {{ -10, -10},{10, -30}}, lineColor = {95,95,95}, fillPattern = FillPattern.Sphere, fillColor = {175,175,175}),Ellipse(extent = {{30, -10},{50, -30}}, lineColor = {95,95,95}, fillPattern = FillPattern.Sphere, fillColor = {175,175,175}),Polygon(points = {{ -60, -30},{60, -30},{60, -12},{80, -12},{80, -50},{ -80, -50},{ -80, -12},{ -60, -12},{ -60, -30}}, lineColor = {95,95,95}, fillPattern = FillPattern.Solid, smooth = Smooth.None, fillColor = {175,175,175}),Text(extent = {{ -150,100},{150,60}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -10, -85},{ -5, -80}}, color = {0,0,0}),Line(points = {{ -5, -90},{5, -80}}, color = {0,0,0}),Line(points = {{ -10, -90},{0, -80}}, color = {0,0,0}),Line(points = {{0, -90},{10, -80}}, color = {0,0,0}),Polygon(points = {{ -60, -50},{ -20, -80},{20, -80},{60, -50},{ -60, -50}}, lineColor = {95,95,95}, smooth = Smooth.None, fillColor = {175,175,175}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -50,30},{ -30,10}}, lineColor = {95,95,95}, fillPattern = FillPattern.Sphere, fillColor = {175,175,175}),Ellipse(extent = {{ -10,30},{10,10}}, lineColor = {95,95,95}, fillPattern = FillPattern.Sphere, fillColor = {175,175,175}),Ellipse(extent = {{30,30},{50,10}}, lineColor = {95,95,95}, fillPattern = FillPattern.Sphere, fillColor = {175,175,175}),Polygon(points = {{ -60,30},{60,30},{60,12},{80,12},{80,50},{ -80,50},{ -80,12},{ -60,12},{ -60,30}}, lineColor = {95,95,95}, fillPattern = FillPattern.Solid, smooth = Smooth.None, fillColor = {175,175,175})}));
          extends Modelica.Mechanics.Translational.Interfaces.PartialElementaryTwoFlangesAndSupport2;
          parameter Real f_pos[:,2] = [0,1] "[v, f] Positive sliding friction characteristic (v>=0)";
          parameter Real peak(final min = 1) = 1 "peak*f_pos[1,2] = Maximum friction force for v==0";
          extends Translational.Interfaces.PartialFriction;
          SI.Position s;
          SI.Force f "Friction force";
          SI.Velocity v "Absolute velocity of flange_a and flange_b";
          SI.Acceleration a "Absolute acceleration of flange_a and flange_b";

        equation 
          f0 = Modelica.Math.tempInterpol1(0, f_pos, 2);
          f0_max = peak * f0;
          free = false;
          s = s_a - s_support;
          s_a = s_b;
          v = der(s);
          a = der(v);
          v_relfric = v;
          a_relfric = a;
          flange_a.f + flange_b.f - f = 0;
          f = if locked then sa * unitForce else if startForward then Modelica.Math.tempInterpol1(v, f_pos, 2) else if startBackward then  -Modelica.Math.tempInterpol1( -v, f_pos, 2) else if pre(mode) == Forward then Modelica.Math.tempInterpol1(v, f_pos, 2) else  -Modelica.Math.tempInterpol1( -v, f_pos, 2);
        end SupportFriction;
        model Brake "Brake basend on Coulomb friction"
          annotation(Documentation(info = "<html>
<p>
This component models a <b>brake</b>, i.e., a component where a frictional
force is acting between the housing and a flange and a controlled normal
force presses the flange to the housing in order to increase friction.
The normal force fn has to be provided as input signal f_normalized in a normalized form
(0 &le; f_normalized &le; 1),
fn = fn_max*f_normalized, where fn_max has to be provided as parameter.
Friction in the brake is modelled in the following way:
</p>
<p>
When the absolute velocity \"v\" is not zero, the friction force
is a function of the velocity dependent friction coefficient  mue(v) , of
the normal force \"fn\", and of a geometry constant \"cgeo\" which takes into
account the geometry of the device and the assumptions on the friction
distributions:
</p>
<pre>
        frictional_force = <b>cgeo</b> * <b>mue</b>(v) * <b>fn</b>
</pre>
<p>
   Typical values of coefficients of friction:
</p>
<pre>
      dry operation   :  <b>mue</b> = 0.2 .. 0.4
      operating in oil:  <b>mue</b> = 0.05 .. 0.1
</pre>
<p>
    The positive part of the friction characteristic <b>mue</b>(v),
    v >= 0, is defined via table mue_pos (first column = v,
    second column = mue). Currently, only linear interpolation in
    the table is supported.
</p>
<p>
   When the absolute velocity becomes zero, the elements
   connected by the friction element become stuck, i.e., the absolute
   position remains constant. In this phase the friction force is
   calculated from a force balance due to the requirement, that
   the absolute acceleration shall be zero.  The elements begin
   to slide when the friction force exceeds a threshold value,
   called the  maximum static friction force, computed via:
</p>
<pre>
       frictional_force = <b>peak</b> * <b>cgeo</b> * <b>mue</b>(w=0) * <b>fn</b>   (<b>peak</b> >= 1)
</pre>
<p>
This procedure is implemented in a \"clean\" way by state events and
leads to continuous/discrete systems of equations if friction elements
are dynamically coupled. The method is described in:
</p>
<dl>
<dt>Otter M., Elmqvist H., and Mattsson S.E. (1999):
<dd><b>Hybrid Modeling in Modelica based on the Synchronous
    Data Flow Principle</b>. CACSD'99, Aug. 22.-26, Hawaii.
</dl>
<p>
More precise friction models take into account the elasticity of the
material when the two elements are \"stuck\", as well as other effects,
like hysteresis. This has the advantage that the friction element can
be completely described by a differential equation without events. The
drawback is that the system becomes stiff (about 10-20 times slower
simulation) and that more material constants have to be supplied which
requires more sophisticated identification. For more details, see the
following references, especially (Armstrong and Canudas de Witt 1996):
</p>
<dl>
<dt>Armstrong B. (1991):
<dd><b>Control of Machines with Friction</b>. Kluwer Academic
    Press, Boston MA.<br><br>
<dt>Armstrong B., and Canudas de Wit C. (1996):
<dd><b>Friction Modeling and Compensation.</b>
    The Control Handbook, edited by W.S.Levine, CRC Press,
    pp. 1369-1382.<br><br>
<dt>Canudas de Wit C., Olsson H., Astroem K.J., and Lischinsky P. (1995):
<dd><b>A new model for control of systems with friction.</b>
    IEEE Transactions on Automatic Control, Vol. 40, No. 3, pp. 419-425.<br><br>
</dl>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -90,10},{90, -10}}, lineColor = {0,127,0}, fillColor = {215,215,215}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -20,30},{20,20}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -20, -20},{20, -30}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{0, -30},{10, -50},{ -10, -50},{0, -30}}, lineColor = {0,0,127}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid),Polygon(points = {{10,50},{ -10,50},{0,30},{10,50}}, lineColor = {0,0,127}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid),Line(points = {{0,90},{0,50}}, color = {0,0,0}, smooth = Smooth.None),Rectangle(extent = {{20,28},{30,22}}, lineColor = {175,175,175}, fillColor = {175,175,175}, fillPattern = FillPattern.Solid),Rectangle(extent = {{20, -22},{30, -28}}, lineColor = {175,175,175}, fillColor = {175,175,175}, fillPattern = FillPattern.Solid),Rectangle(extent = {{30,28},{36, -102}}, lineColor = {175,175,175}, fillColor = {175,175,175}, fillPattern = FillPattern.Solid),Rectangle(extent = {{14, -96},{30, -102}}, lineColor = {175,175,175}, fillColor = {175,175,175}, fillPattern = FillPattern.Solid),Line(points = {{0, -50},{0, -60},{ -40, -50},{ -40,48},{0,60},{0,90}}, color = {0,0,0}, smooth = Smooth.None)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          extends Modelica.Mechanics.Translational.Interfaces.PartialElementaryTwoFlangesAndSupport2;
          parameter Real mue_pos[:,2] = [0,0.5] "[v, f] Positive sliding friction characteristic (v>=0)";
          parameter Real peak(final min = 1) = 1 "peak*mue_pos[1,2] = Maximum friction force for v==0";
          parameter Real cgeo(final min = 0) = 1 "Geometry constant containing friction distribution assumption";
          parameter SI.Force fn_max(final min = 0, start = 1) "Maximum normal force";
          extends Translational.Interfaces.PartialFriction;
          SI.Position s;
          SI.Force f "Brake friction force";
          SI.Velocity v "Absolute velocity of flange_a and flange_b";
          SI.Acceleration a "Absolute acceleration of flange_a and flange_b";
          Real mue0 "Friction coefficient for v=0 and forward sliding";
          SI.Force fn "Normal force (=fn_max*f_normalized)";
          Modelica.Blocks.Interfaces.RealInput f_normalized "Normalized force signal 0..1 (normal force = fn_max*f_normalized; brake is active if > 0)" annotation(Placement(transformation(origin = {0,110}, extent = {{20, -20},{ -20,20}}, rotation = 90)));

        equation 
          mue0 = Modelica.Math.tempInterpol1(0, mue_pos, 2);
          s = s_a;
          s = s_b;
          v = der(s);
          a = der(v);
          v_relfric = v;
          a_relfric = a;
          flange_a.f + flange_b.f - f = 0;
          fn = fn_max * f_normalized;
          f0 = mue0 * cgeo * fn;
          f0_max = peak * f0;
          free = fn <= 0;
          f = if locked then sa * unitForce else if free then 0 else cgeo * fn * (if startForward then Modelica.Math.tempInterpol1(v, mue_pos, 2) else if startBackward then  -Modelica.Math.tempInterpol1( -v, mue_pos, 2) else if pre(mode) == Forward then Modelica.Math.tempInterpol1(v, mue_pos, 2) else  -Modelica.Math.tempInterpol1( -v, mue_pos, 2));
        end Brake;
        model IdealGearR2T "Gearbox transforming rotational into translational motion\""
          extends Modelica.Mechanics.Rotational.Components.IdealGearR2T;
        end IdealGearR2T;
        model IdealRollingWheel "Simple 1-dim. model of an ideal rolling wheel without inertia"
          extends Modelica.Mechanics.Rotational.Components.IdealRollingWheel;
        end IdealRollingWheel;
        model InitializeFlange "Initializes a flange with pre-defined position, speed and acceleration (usually, this is reference data from a control bus)"
          extends Modelica.Blocks.Interfaces.BlockIcon;
          parameter Boolean use_s_start = true "= true, if initial position is defined by input s_start, otherwise not initialized";
          parameter Boolean use_v_start = true "= true, if initial speed is defined by input v_start, otherwise not initialized";
          parameter Boolean use_a_start = true "= true, if initial acceleration is defined by input a_start, otherwise not initialized";
          parameter StateSelect stateSelect = StateSelect.default "Priority to use flange angle and speed as states";
          Modelica.Blocks.Interfaces.RealInput s_start if use_s_start "Initial position of flange" annotation(Placement(transformation(extent = {{ -140,40},{ -100,80}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput v_start if use_v_start "Initial speed of flange" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput a_start if use_a_start "Initial angular acceleration of flange" annotation(Placement(transformation(extent = {{ -140, -80},{ -100, -40}}, rotation = 0)));
          Interfaces.Flange_b flange "Flange that is initialized" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.SIunits.Position s_flange(stateSelect = stateSelect) = flange.s "Flange position";
          Modelica.SIunits.Velocity v_flange(stateSelect = stateSelect) = der(s_flange) "= der(s_flange)";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -94,74},{68,46}}, lineColor = {0,0,0}, textString = "s_start"),Text(extent = {{ -94,16},{70, -14}}, lineColor = {0,0,0}, textString = "v_start"),Text(extent = {{ -94, -46},{66, -74}}, lineColor = {0,0,0}, textString = "a_start")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
This component is used to optionally initialize the position, speed,
and/or acceleration of the flange to which this component
is connected. Via parameters use_s_start, use_v_start, use_a_start
the corresponding input signals s_start, v_start, a_start are conditionally
activated. If an input is activated, the corresponding flange property
is initialized with the input value at start time.
</p>

<p>
For example, if \"use_s_start = true\", then flange.s is initialized
with the value of the input signal \"s_start\" at the start time.
</p>

<p>
Additionally, it is optionally possible to define the \"StateSelect\"
attribute of the flange position and the flange speed via paramater
\"stateSelection\".
</p>

<p>
This component is especially useful when the initial values of a flange
shall be set according to reference signals of a controller that are
provided via a signal bus.
</p>

</html>"));
        protected 
          encapsulated model Set_s_start "Set s_start"
            import Modelica;
            extends Modelica.Blocks.Interfaces.BlockIcon;
            Modelica.Blocks.Interfaces.RealInput s_start "Start position" annotation(HideResult = true, Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
            annotation(Diagram(graphics));
            Modelica.Mechanics.Translational.Interfaces.Flange_b flange annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));

          initial equation 
            flange.s = s_start;

          equation 
            flange.f = 0;
          end Set_s_start;
          encapsulated model Set_v_start "Set v_start"
            import Modelica;
            extends Modelica.Blocks.Interfaces.BlockIcon;
            Modelica.Blocks.Interfaces.RealInput v_start "Start velocity" annotation(HideResult = true, Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
            annotation(Diagram(graphics));
            Modelica.Mechanics.Translational.Interfaces.Flange_b flange annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));

          initial equation 
            der(flange.s) = v_start;

          equation 
            flange.f = 0;
          end Set_v_start;
          encapsulated model Set_a_start "Set a_start"
            import Modelica;
            extends Modelica.Blocks.Interfaces.BlockIcon;
            Modelica.Blocks.Interfaces.RealInput a_start "Start acceleration" annotation(HideResult = true, Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
            annotation(Diagram(graphics));
            Modelica.Mechanics.Translational.Interfaces.Flange_b flange(s(stateSelect = StateSelect.avoid)) annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
            Modelica.SIunits.Velocity v = der(flange.s) annotation(HideResult = true);

          initial equation 
            der(v) = a_start;

          equation 
            flange.f = 0;
          end Set_a_start;
          encapsulated model Set_flange_f "Set flange_f to zero"
            import Modelica;
            extends Modelica.Blocks.Interfaces.BlockIcon;
            Modelica.Mechanics.Translational.Interfaces.Flange_b flange annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));

          equation 
            flange.f = 0;
          end Set_flange_f;
        protected 
          Set_s_start set_s_start if use_s_start annotation(Placement(transformation(extent = {{ -20,50},{0,70}}, rotation = 0)));
          Set_v_start set_v_start if use_v_start annotation(Placement(transformation(extent = {{ -20, -10},{0,10}}, rotation = 0)));
          Set_a_start set_a_start if use_a_start annotation(Placement(transformation(extent = {{ -20, -70},{0, -50}}, rotation = 0)));
          Set_flange_f set_flange_f annotation(Placement(transformation(extent = {{20, -100},{40, -80}}, rotation = 0)));

        equation 
          connect(set_s_start.flange,flange) annotation(Line(points = {{0,60},{60,60},{60,0},{100,0}}, color = {0,0,0}, smooth = Smooth.None));
          connect(set_v_start.flange,flange) annotation(Line(points = {{0,0},{100,0}}, color = {0,0,0}, smooth = Smooth.None));
          connect(set_a_start.flange,flange) annotation(Line(points = {{0, -60},{60, -60},{60,0},{100,0}}, color = {0,0,0}, smooth = Smooth.None));
          connect(set_flange_f.flange,flange) annotation(Line(points = {{40, -90},{60, -90},{60,0},{100,0}}, color = {0,0,0}, smooth = Smooth.None));
          connect(s_start,set_s_start.s_start) annotation(Line(points = {{ -120,60},{ -22,60}}, color = {0,0,127}, smooth = Smooth.None));
          connect(v_start,set_v_start.v_start) annotation(Line(points = {{ -120,0},{ -22,0}}, color = {0,0,127}, smooth = Smooth.None));
          connect(a_start,set_a_start.a_start) annotation(Line(points = {{ -120, -60},{ -22, -60}}, color = {0,0,127}, smooth = Smooth.None));
        end InitializeFlange;
        model MassWithStopAndFriction "Sliding mass with hard stop and Stribeck friction"
          extends PartialFrictionWithStop;
          SI.Velocity v(start = 0, stateSelect = StateSelect.always) "Absolute velocity of flange_a and flange_b";
          SI.Acceleration a(start = 0) "Absolute acceleration of flange_a and flange_b";
          parameter Modelica.SIunits.Mass m(start = 1) "mass";
          parameter Real F_prop(final unit = "N.s/m", final min = 0, start = 1) "Velocity dependent friction";
          parameter Modelica.SIunits.Force F_Coulomb(start = 5) "Constant friction: Coulomb force";
          parameter Modelica.SIunits.Force F_Stribeck(start = 10) "Stribeck effect";
          parameter Real fexp(final unit = "s/m", final min = 0, start = 2) "Exponential decay";
          Integer stopped = if s <= smin + L / 2 then  -1 else if s >= smax - L / 2 then  +1 else 0;
          annotation(Documentation(info = "
<HTML>
<P>This element describes the <i>Stribeck friction characteristics</i> of a sliding mass,
i. e. the frictional force acting between the sliding mass and the support. Included is a
<i>hard stop</i> for the position. <BR>
The surface is fixed and there is friction between sliding mass and surface.
The frictional force f is given for positive velocity v by:</P>
<i><uL>
f = F_Coulomb + F_prop * v + F_Stribeck * exp (-fexp * v)</i> </ul><br>
<IMG SRC=../Images/Translational/Stribeck.png>
<br><br>
The distance between the left and the right connector is given by parameter L.
The position of the center of gravity, coordinate s, is in the middle between
the two flanges. </p>
<p>
There are hard stops at smax and smin, i. e. if <i><uL>
flange_a.s &gt;= smin
<ul>    and </ul>
flange_b.s &lt;= xmax </ul></i>
the sliding mass can move freely.</p>
<p>When the absolute velocity becomes zero, the sliding mass becomes stuck, i.e., the absolute position remains constant. In this phase the
friction force is calculated from a force balance due to the requirement that the
absolute acceleration shall be zero. The elements begin to slide when the friction
force exceeds a threshold value, called the maximum static friction force, computed via: </P>
<i><uL>
   maximum_static_friction =  F_Coulomb + F_Stribeck
</i> </ul>
<font color=\"#ff0000\"> <b>This requires the states Stop.s and Stop.v</b> </font>. If these states are eliminated during the index reduction
the model will not work. To avoid this any inertias should be connected via springs
to the Stop element, other sliding masses, dampers or hydraulic chambers must be avoided. </p>
<p>For more details of the used friction model see the following reference: <br> <br>
Beater P. (1999): <DD><a href=\"http://www.springer.de/cgi-bin/search_book.pl?isbn=3-540-65444-5\">
Entwurf hydraulischer Maschinen</a>. Springer Verlag Berlin Heidelberg New York.</DL></P>
<P>The friction model is implemented in a \"clean\" way by state events and leads to
continuous/discrete systems of equations which have to be solved by appropriate
numerical methods. The method is described in: </P>

<dl>
Otter M., Elmqvist H., and Mattsson S.E. (1999):
<i><DD>Hybrid Modeling in Modelica based on the Synchronous Data Flow Principle</i>. CACSD'99, Aug. 22.-26, Hawaii. </DD>
</DL>
<P>More precise friction models take into account the elasticity of the material when
the two elements are \"stuck\", as well as other effects, like hysteresis. This has
the advantage that the friction element can be completely described by a differential
equation without events. The drawback is that the system becomes stiff (about 10-20 times
slower simulation) and that more material constants have to be supplied which requires more
sophisticated identification. For more details, see the following references, especially
(Armstrong and Canudas de Witt 1996): </P>
<dl>
<dt>
Armstrong B. (1991):</dt>
<DD><i>Control of Machines with Friction</i>. Kluwer Academic Press, Boston MA.<BR>
</DD>
<DT>Armstrong B., and Canudas de Wit C. (1996): </DT>
<DD><i>Friction Modeling and Compensation.</i> The Control Handbook, edited by W.S.Levine, CRC Press, pp. 1369-1382.<BR>
</DD>
<DT>Canudas de Wit C., Olsson H., Astroem K.J., and Lischinsky P. (1995): </DT>
<DD>A<i> new model for control of systems with friction.</i> IEEE Transactions on Automatic Control, Vol. 40, No. 3, pp. 419-425.<BR>
</DD>
</DL>

</HTML>
", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>First Version from December 7, 1999 by P. Beater (based on Rotational.BearingFriction)</i> </li>
<li><i>July 14, 2001 by P. Beater, assert on initialization added, diagram modified </i> </li>
<li><i>October 11, 2001, by Hans Olsson, Dynasim, modified assert to handle start at stops,
modified event logic such if you have friction parameters equal to zero you do not get events
between the stops.</i> </li>
<li><i>June 10, 2002 by P. Beater, StateSelect.always for variables s and v (instead of fixed=true). </i> </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{80, -100},{50, -90},{50, -110},{80, -100}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{ -30, -100},{50, -100}}, color = {0,0,0}),Rectangle(extent = {{ -30,30},{35, -35}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {255,255,255}),Line(points = {{ -90,0},{ -30,0}}, color = {0,127,0}),Rectangle(extent = {{ -70, -45},{74, -60}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -63, -15},{ -55, -45}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{60, -16},{69, -45}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{35,0},{90,0}}, color = {0,127,0}),Text(extent = {{0,100},{0,40}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -50, -90},{ -30, -70}}, color = {0,0,0}),Line(points = {{ -30, -70},{30, -70}}, color = {0,0,0}),Line(points = {{ -30, -90},{ -10, -70}}, color = {0,0,0}),Line(points = {{ -10, -90},{10, -70}}, color = {0,0,0}),Line(points = {{10, -90},{30, -70}}, color = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{50, -75},{20, -65},{20, -85},{50, -75}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{ -60, -75},{20, -75}}, color = {0,0,0}),Rectangle(extent = {{ -30,26},{35, -9}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {255,255,255}),Line(points = {{ -90,0},{ -30,0}}, color = {0,127,0}),Line(points = {{35,0},{90,0}}, color = {0,127,0}),Rectangle(extent = {{ -68, -14},{76, -29}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -119,43},{ -111,17}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -111,43},{ -111,50}}, color = {0,0,0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Line(points = {{ -151,49},{ -113,49}}, color = {0,0,0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Text(extent = {{ -149,51},{ -126,60}}, textString = "s min", lineColor = {0,0,255}),Polygon(points = {{ -121,52},{ -111,49},{ -121,46},{ -121,52}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{124,42},{132,17}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{124,39},{124,87}}, color = {0,0,0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Line(points = {{ -19,78},{121,78}}, color = {0,0,0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Text(extent = {{ -17,83},{6,92}}, textString = "s max", lineColor = {0,0,255}),Polygon(points = {{114,81},{124,78},{114,75},{114,81}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{5,26},{5,63}}, color = {0,0,0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Line(points = {{ -77,58},{ -1,58}}, color = {0,0,0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Text(extent = {{ -75,60},{ -38,71}}, textString = "Position s", lineColor = {0,0,255}),Polygon(points = {{ -5,61},{5,58},{ -5,55},{ -5,61}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -100, -10},{ -100, -60}}, color = {0,0,0}),Line(points = {{100, -10},{100, -60}}, color = {0,0,0}),Polygon(points = {{90, -47},{100, -50},{90, -53},{90, -47}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{ -90, -47},{ -90, -53},{ -100, -50},{ -90, -47}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -90, -50},{92, -50}}, color = {0,0,0}),Text(extent = {{ -11, -46},{26, -36}}, textString = "Length L", lineColor = {0,0,255})}));
          encapsulated partial model PartialFrictionWithStop "Base model of Coulomb friction elements with stop"
            annotation(Documentation(info = "<html>
<p>
Basic model for Coulomb friction that models the stuck phase in a reliable way.<br>
Additionally, a left and right stop are handled.
</p>
</html>
"));
            import SI = Modelica.SIunits;
            import Modelica.Mechanics.Translational.Interfaces.PartialRigid;
            parameter SI.Position smax(start = 25) "Right stop for (right end of) sliding mass";
            parameter SI.Position smin(start =  -25) "Left stop for (left end of) sliding mass";
            parameter SI.Velocity v_small = 0.001 "Relative velocity near to zero (see model info text)" annotation(Dialog(tab = "Advanced"));
            SI.Velocity v_relfric "Relative velocity between frictional surfaces";
            SI.Acceleration a_relfric "Relative acceleration between frictional surfaces";
            SI.Force f "Friction force (positive, if directed in opposite direction of v_rel)";
            SI.Force f0 "Friction force for v=0 and forward sliding";
            SI.Force f0_max "Maximum friction force for v=0 and locked";
            Boolean free "true, if frictional element is not active";
            Real sa(unit = "1") "Path parameter of friction characteristic f = f(a_relfric)";
            Boolean startForward(start = false, fixed = true) "= true, if v_rel=0 and start of forward sliding or v_rel > v_small";
            Boolean startBackward(start = false, fixed = true) "= true, if v_rel=0 and start of backward sliding or v_rel < -v_small";
            Boolean locked(start = false) "true, if v_rel=0 and not sliding";
            extends PartialRigid(s(start = 0, stateSelect = StateSelect.always));
            constant Integer Unknown = 3 "Value of mode is not known";
            constant Integer Free = 2 "Element is not active";
            constant Integer Forward = 1 "v_rel > 0 (forward sliding)";
            constant Integer Stuck = 0 "v_rel = 0 (forward sliding, locked or backward sliding)";
            constant Integer Backward =  -1 "v_rel < 0 (backward sliding)";
            Integer mode(final min = Backward, final max = Unknown, start = Unknown, fixed = true);
          protected 
            constant SI.Acceleration unitAcceleration = 1 annotation(HideResult = true);
            constant SI.Force unitForce = 1 annotation(HideResult = true);

          equation 
            startForward = pre(mode) == Stuck and (sa > f0_max / unitForce and s < smax - L / 2 or pre(startForward) and sa > f0 / unitForce and s < smax - L / 2) or pre(mode) == Backward and v_relfric > v_small or initial() and v_relfric > 0;
            startBackward = pre(mode) == Stuck and (sa <  -f0_max / unitForce and s > smin + L / 2 or pre(startBackward) and sa <  -f0 / unitForce and s > smin + L / 2) or pre(mode) == Forward and v_relfric <  -v_small or initial() and v_relfric < 0;
            locked = not free and not (pre(mode) == Forward or startForward or pre(mode) == Backward or startBackward);
            a_relfric / unitAcceleration = if locked then 0 else if free then sa else if startForward then sa - f0_max / unitForce else if startBackward then sa + f0_max / unitForce else if pre(mode) == Forward then sa - f0_max / unitForce else sa + f0_max / unitForce;
            mode = if free then Free else if (pre(mode) == Forward or pre(mode) == Free or startForward) and v_relfric > 0 and s < smax - L / 2 then Forward else if (pre(mode) == Backward or pre(mode) == Free or startBackward) and v_relfric < 0 and s > smin + L / 2 then Backward else Stuck;
          end PartialFrictionWithStop;

        equation 
          f0 = F_Coulomb + F_Stribeck;
          f0_max = f0 * 1.001;
          free = f0 <= 0 and F_prop <= 0 and s > smin + L / 2 and s < smax - L / 2;
          v = der(s);
          a = der(v);
          v_relfric = v;
          a_relfric = a;
          0 = flange_a.f + flange_b.f - f - m * der(v);
          f = if locked then sa * unitForce else if free then 0 else if startForward then F_prop * v + F_Coulomb + F_Stribeck else if startBackward then F_prop * v - F_Coulomb - F_Stribeck else if pre(mode) == Forward then F_prop * v + F_Coulomb + F_Stribeck * exp( -fexp * abs(v)) else F_prop * v - F_Coulomb - F_Stribeck * exp( -fexp * abs(v));
        algorithm 
          when initial() then
                      assert(s > smin + L / 2 or s >= smin + L / 2 and v >= 0, "Error in initialization of hard stop. (s - L/2) must be >= smin ");
            assert(s < smax - L / 2 or s <= smax - L / 2 and v <= 0, "Error in initialization of hard stop. (s + L/2) must be <= smax ");
          
          end when;
          when stopped <> 0 then
                      reinit(s, if stopped < 0 then smin + L / 2 else smax - L / 2);
            if not initial() or stopped * v > 0 then 
                        reinit(v, 0);

            else 
            end if;
          
          end when;
        end MassWithStopAndFriction;
        model RelativeStates "Definition of relative state variables"
          extends Translational.Interfaces.PartialTwoFlanges;
          parameter StateSelect stateSelect = StateSelect.prefer "Priority to use the relative angle and relative speed as states";
          SI.Position s_rel(start = 0, stateSelect = StateSelect.prefer) "relative position used as state variable";
          SI.Velocity v_rel(start = 0, stateSelect = StateSelect.prefer) "relative velocity used as state variable";
          SI.Acceleration a_rel(start = 0) "relative angular acceleration";
          annotation(Documentation(info = "<html>
<p>
Usually, the absolute position and the absolute velocity of
Modelica.Mechanics.Translational.Inertia models are used as state variables.
In some circumstances, relative quantities are better suited, e.g.,
because it may be easier to supply initial values.
In such cases, model <b>RelativeStates</b> allows the definition of state variables
in the following way:
</p>
<ul>
<li> Connect an instance of this model between two flange connectors.</li>
<li> The <b>relative position</b> and the <b>relative velocity</b>
     between the two connectors are used as <b>state variables</b>.
</ul>
<p>
An example is given in the next figure
</p>
<IMG SRC=\"../Images/Translational/relativeStates2.png\" ALT=\"relativeStates2\">
<p>
Here, the relative position and the relative velocity between
the two masses are used as state variables. Additionally, the
simulator selects either the absolute position and absolute
velocity of model mass1 or of model mass2 as state variables.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Ellipse(extent = {{ -40,40},{40, -40}}, lineColor = {0,255,255}, fillColor = {0,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -40,40},{40, -40}}, textString = "S", lineColor = {0,0,255}),Line(points = {{ -92,0},{ -42,0}}, color = {0,0,0}, pattern = LinePattern.Dot),Line(points = {{40,0},{90,0}}, color = {0,0,0}, pattern = LinePattern.Dot),Text(extent = {{0,110},{0,50}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Ellipse(extent = {{ -40,40},{40, -40}}, lineColor = {0,255,255}, fillColor = {0,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -40,40},{40, -40}}, textString = "S", lineColor = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,0}, pattern = LinePattern.Dash),Line(points = {{ -100, -10},{ -100, -80}}, color = {160,160,164}),Line(points = {{100, -10},{100, -80}}, color = {160,160,164}),Polygon(points = {{80, -65},{80, -55},{100, -60},{80, -65}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Line(points = {{ -100, -60},{80, -60}}, color = {160,160,164}),Text(extent = {{ -30, -70},{30, -90}}, textString = "w_rel", lineColor = {0,0,255}),Line(points = {{ -76,80},{ -5,80}}, color = {128,128,128}),Polygon(points = {{14,80},{ -6,85},{ -6,75},{14,80}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{18,87},{86,74}}, lineColor = {128,128,128}, textString = "moving direction"),Line(points = {{ -90,0},{ -40,0}}, color = {0,0,0}, pattern = LinePattern.Dash)}));

        equation 
          s_rel = flange_b.s - flange_a.s;
          v_rel = der(s_rel);
          a_rel = der(v_rel);
          flange_a.f = 0;
          flange_b.f = 0;
        end RelativeStates;
      end Components;
      package Sources "Sources to drive 1D translational mechanical components"
        extends Modelica.Icons.Library2;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -100, -32},{10, -32},{10, -1},{80, -42},{10, -83},{10, -52},{ -100, -52},{ -100, -32}}, lineColor = {0,127,0}, fillColor = {0,127,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>
This package contains ideal sources to drive 1D mechanical translational drive trains.
</p>
</html>"));
        model Position "Forced movement of a flange according to a reference position"
          extends Modelica.Mechanics.Translational.Interfaces.PartialElementaryOneFlangeAndSupport2(s(stateSelect = if exact then StateSelect.default else StateSelect.prefer));
          parameter Boolean exact = false "true/false exact treatment/filtering the input signal";
          parameter SI.Frequency f_crit = 50 "if exact=false, critical frequency of filter to filter input signal" annotation(Dialog(enable = not exact));
          SI.Velocity v(start = 0, stateSelect = if exact then StateSelect.default else StateSelect.prefer) "If exact=false, absolute velocity of flange_b else dummy";
          SI.Acceleration a(start = 0) "If exact=false, absolute acceleration of flange_b else dummy";
          Modelica.Blocks.Interfaces.RealInput s_ref "reference position of flange as input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        protected 
          parameter Modelica.SIunits.AngularFrequency w_crit = 2 * Modelica.Constants.pi * f_crit "Critical frequency";
          constant Real af = 1.3617 "s coefficient of Bessel filter";
          constant Real bf = 0.618 "s*s coefficient of Bessel filter";
          annotation(Documentation(info = "<HTML>
<p>
The input signal <b>s_ref</b> defines the <b>reference
position</b> in [m]. Flange <b>flange_b</b> is <b>forced</b>
to move relative to the support connector according to this reference motion. According to parameter
<b>exact</b> (default = <b>false</b>), this is done in the following way:
<ol>
<li><b>exact=true</b><br>
    The reference position is treated <b>exactly</b>. This is only possible, if
    the input signal is defined by an analytical function which can be
    differentiated at least twice. If this prerequisite is fulfilled,
    the Modelica translator will differentiate the input signal twice
    in order to compute the reference acceleration of the flange.</li>
<li><b>exact=false</b><br>
    The reference position is <b>filtered</b> and the second derivative
    of the filtered curve is used to compute the reference acceleration
    of the flange. This second derivative is <b>not</b> computed by
    numerical differentiation but by an appropriate realization of the
    filter. For filtering, a second order Bessel filter is used.
    The critical frequency (also called cut-off frequency) of the
    filter is defined via parameter <b>f_crit</b> in [Hz]. This value
    should be selected in such a way that it is higher as the essential
    low frequencies in the signal.</li>
</ol>
<p>
The input signal can be provided from one of the signal generator
blocks of the block library Modelica.Blocks.Sources.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -40, -46},{ -140, -80}}, lineColor = {0,0,0}, textString = "s_ref"),Rectangle(extent = {{ -100,20},{100, -20}}, lineColor = {0,127,0}, fillColor = {215,215,215}, fillPattern = FillPattern.Solid),Text(extent = {{150,60},{ -150,100}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{0,52},{0,32}}, color = {0,0,0}),Line(points = {{ -29,32},{30,32}}, color = {0,0,0}),Line(points = {{ -30, -32},{30, -32}}, color = {0,0,0}),Line(points = {{0, -32},{0, -100}}, color = {0,0,0}),Line(points = {{30, -42},{20, -52}}, color = {0,0,0}),Line(points = {{30, -32},{10, -52}}, color = {0,0,0}),Line(points = {{20, -32},{0, -52}}, color = {0,0,0}),Line(points = {{10, -32},{ -10, -52}}, color = {0,0,0}),Line(points = {{0, -32},{ -20, -52}}, color = {0,0,0}),Line(points = {{ -10, -32},{ -30, -52}}, color = {0,0,0}),Line(points = {{ -20, -32},{ -30, -42}}, color = {0,0,0}),Text(extent = {{144, -30},{30, -60}}, lineColor = {0,0,0}, textString = "exact="),Text(extent = {{132, -68},{20, -96}}, lineColor = {0,0,0}, textString = "%exact")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));

        initial equation 
if not exact then
                      s = s_ref;
          end if;

        equation 
          if exact then
            s = s_ref;
            v = 0;
            a = 0;

          else
            v = der(s);
            a = der(v);
            a = ((s_ref - s) * w_crit - af * v) * w_crit / bf;

          end if;
        end Position;
        model Speed "Forced movement of a flange according to a reference speed"
          extends Modelica.Mechanics.Translational.Interfaces.PartialElementaryOneFlangeAndSupport2(s(start = 0, fixed = true, stateSelect = StateSelect.prefer));
          parameter Boolean exact = false "true/false exact treatment/filtering the input signal";
          parameter SI.Frequency f_crit = 50 "if exact=false, critical frequency of filter to filter input signal" annotation(Dialog(enable = not exact));
          SI.Velocity v(stateSelect = if exact then StateSelect.default else StateSelect.prefer) "Absolute velocity of flange_b";
          SI.Acceleration a "If exact=false, absolute acceleration of flange_b else dummy";
          Modelica.Blocks.Interfaces.RealInput v_ref "reference speed of flange as input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
          annotation(Documentation(info = "<HTML>
<p>
The input signal <b>v_ref</b> defines the <b>reference
speed</b> in [m/s]. Flange <b>flange_b</b> is <b>forced</b>
to move relative to the support connector  according to this reference motion. According to parameter
<b>exact</b> (default = <b>false</b>), this is done in the following way:
<ol>
<li><b>exact=true</b><br>
    The reference speed is treated <b>exactly</b>. This is only possible, if
    the input signal is defined by an analytical function which can be
    differentiated at least once. If this prerequisite is fulfilled,
    the Modelica translator will differentiate the input signal once
    in order to compute the reference acceleration of the flange.</li>
<li><b>exact=false</b><br>
    The reference speed is <b>filtered</b> and the first derivative
    of the filtered curve is used to compute the reference acceleration
    of the flange. This first derivative is <b>not</b> computed by
    numerical differentiation but by an appropriate realization of the
    filter. For filtering, a first order filter is used.
    The critical frequency (also called cut-off frequency) of the
    filter is defined via parameter <b>f_crit</b> in [Hz]. This value
    should be selected in such a way that it is higher as the essential
    low frequencies in the signal.</li>
</ol>
<p>
The input signal can be provided from one of the signal generator
blocks of the block library Modelica.Blocks.Sources.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -38, -48},{ -142, -80}}, lineColor = {0,0,0}, textString = "v_ref"),Line(points = {{ -30, -32},{30, -32}}, color = {0,0,0}),Line(points = {{0, -32},{0, -100}}, color = {0,0,0}),Line(points = {{30, -42},{20, -52}}, color = {0,0,0}),Line(points = {{30, -32},{10, -52}}, color = {0,0,0}),Line(points = {{20, -32},{0, -52}}, color = {0,0,0}),Line(points = {{10, -32},{ -10, -52}}, color = {0,0,0}),Line(points = {{0, -32},{ -20, -52}}, color = {0,0,0}),Line(points = {{ -10, -32},{ -30, -52}}, color = {0,0,0}),Line(points = {{ -20, -32},{ -30, -42}}, color = {0,0,0}),Rectangle(extent = {{ -100,20},{100, -20}}, lineColor = {0,127,0}, fillColor = {215,215,215}, fillPattern = FillPattern.Solid),Line(points = {{ -29,32},{30,32}}, color = {0,0,0}),Line(points = {{0,52},{0,32}}, color = {0,0,0}),Text(extent = {{150,60},{ -150,100}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{146, -38},{32, -64}}, lineColor = {0,0,0}, textString = "exact="),Text(extent = {{140, -76},{22, -102}}, lineColor = {0,0,0}, textString = "%exact")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));
        protected 
          parameter Modelica.SIunits.AngularFrequency w_crit = 2 * Modelica.Constants.pi * f_crit "Critical frequency";

        initial equation 
if not exact then
                      v = v_ref;
          end if;

        equation 
          v = der(s);
          if exact then
            v = v_ref;
            a = 0;

          else
            a = der(v);
            a = (v_ref - v) * w_crit;

          end if;
        end Speed;
        model Accelerate "Forced movement of a flange according to an acceleration signal"
          extends Modelica.Mechanics.Translational.Interfaces.PartialElementaryOneFlangeAndSupport2(s(start = 0, fixed = true, stateSelect = StateSelect.prefer));
          SI.Velocity v(start = 0, fixed = true, stateSelect = StateSelect.prefer) "Absolute velocity of flange_b";
          SI.Acceleration a "Absolute acceleration of flange_b";
          Modelica.Blocks.Interfaces.RealInput a_ref "absolute acceleration of flange as input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>
The input signal <b>a</b> in [m/s2] moves the 1D translational flange
connector flange_b with a predefined <i>acceleration</i>, i.e., the flange
is <i>forced</i> to move relative to the support connector  with this acceleration. The velocity and the
position of the flange are also predefined and are determined by
integration of the acceleration.
</p>
<p>
The acceleration \"a(t)\" can be provided from one of the signal generator
blocks of the block library Modelica.Blocks.Source.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -46, -42},{ -140, -70}}, lineColor = {0,0,0}, textString = "a_ref"),Line(points = {{ -30, -32},{30, -32}}, color = {0,0,0}),Line(points = {{0, -32},{0, -100}}, color = {0,0,0}),Line(points = {{30, -42},{20, -52}}, color = {0,0,0}),Line(points = {{30, -32},{10, -52}}, color = {0,0,0}),Line(points = {{20, -32},{0, -52}}, color = {0,0,0}),Line(points = {{10, -32},{ -10, -52}}, color = {0,0,0}),Line(points = {{0, -32},{ -20, -52}}, color = {0,0,0}),Line(points = {{ -10, -32},{ -30, -52}}, color = {0,0,0}),Line(points = {{ -20, -32},{ -30, -42}}, color = {0,0,0}),Rectangle(extent = {{ -100,20},{100, -20}}, lineColor = {0,127,0}, fillColor = {215,215,215}, fillPattern = FillPattern.Solid),Line(points = {{ -29,32},{30,32}}, color = {0,0,0}),Line(points = {{0,52},{0,32}}, color = {0,0,0}),Text(extent = {{150,60},{ -150,100}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));

        equation 
          v = der(s);
          a = der(v);
          a = a_ref;
        end Accelerate;
        model Move "Forced movement of a flange according to a position, velocity and acceleration signal"
          extends Modelica.Mechanics.Translational.Interfaces.PartialElementaryOneFlangeAndSupport2;
          Modelica.Blocks.Interfaces.RealInput u[3] "position, velocity and acceleration of flange as input signals" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>
Flange <b>flange_b</b> is <b>forced</b> to move relative to the support connector  with a predefined motion
according to the input signals:
</p>
<pre>
    u[1]: position of flange
    u[2]: velocity of flange
    u[3]: acceleration of flange
</pre>
<p>
The user has to guarantee that the input signals are consistent to each other,
i.e., that u[2] is the derivative of u[1] and that
u[3] is the derivative of u. There are, however,
also applications where by purpose these conditions do not hold. For example,
if only the position dependent terms of a mechanical system shall be
calculated, one may provide position = position(t) and set the velocity
and the acceleration to zero.
</p>
<p>
The input signals can be provided from one of the signal generator
blocks of the block library Modelica.Blocks.Sources.
</p>

</html>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -160, -44},{0, -76}}, lineColor = {0,0,0}, textString = "s,v,a"),Line(points = {{ -30, -32},{30, -32}}, color = {0,0,0}),Line(points = {{0, -32},{0, -100}}, color = {0,0,0}),Line(points = {{30, -42},{20, -52}}, color = {0,0,0}),Line(points = {{30, -32},{10, -52}}, color = {0,0,0}),Line(points = {{20, -32},{0, -52}}, color = {0,0,0}),Line(points = {{10, -32},{ -10, -52}}, color = {0,0,0}),Line(points = {{0, -32},{ -20, -52}}, color = {0,0,0}),Line(points = {{ -10, -32},{ -30, -52}}, color = {0,0,0}),Line(points = {{ -20, -32},{ -30, -42}}, color = {0,0,0}),Rectangle(extent = {{ -100,20},{100, -20}}, lineColor = {0,127,0}, fillColor = {215,215,215}, fillPattern = FillPattern.Solid),Line(points = {{0,52},{0,32}}, color = {0,0,0}),Line(points = {{ -29,32},{30,32}}, color = {0,0,0}),Text(extent = {{150,60},{ -150,100}}, textString = "%name", lineColor = {0,0,255})}));
        protected 
          function position
            input Real q_qd_qdd[3] "Required values for position, speed, acceleration";
            input Real dummy "Just to have one input signal that should be differentiated to avoid possible problems in the Modelica tool (is not used)";
            output Real q;
            annotation(derivative(noDerivative = q_qd_qdd) = position_der, __Dymola_InlineAfterIndexReduction = true);
          algorithm 
            q:=q_qd_qdd[1];
          end position;
          function position_der
            input Real q_qd_qdd[3] "Required values for position, speed, acceleration";
            input Real dummy "Just to have one input signal that should be differentiated to avoid possible problems in the Modelica tool (is not used)";
            input Real dummy_der;
            output Real qd;
            annotation(derivative(noDerivative = q_qd_qdd) = position_der2, __Dymola_InlineAfterIndexReduction = true);
          algorithm 
            qd:=q_qd_qdd[2];
          end position_der;
          function position_der2
            input Real q_qd_qdd[3] "Required values for position, speed, acceleration";
            input Real dummy "Just to have one input signal that should be differentiated to avoid possible problems in the Modelica tool (is not used)";
            input Real dummy_der;
            input Real dummy_der2;
            output Real qdd;
          algorithm 
            qdd:=q_qd_qdd[3];
          end position_der2;

        equation 
          s = position(u, time);
        end Move;
        model Force "External force acting on a drive train element as input signal"
          extends Modelica.Mechanics.Translational.Interfaces.PartialElementaryOneFlangeAndSupport2;
          Modelica.Blocks.Interfaces.RealInput f "driving force as input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>
The input signal \"f\" in [N] characterizes an <i>external
force</i> which acts (with positive sign) at a flange,
i.e., the component connected to the flange is driven by force f.
</p>
<p>
Input signal f can be provided from one of the signal generator
blocks of Modelica.Blocks.Source.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Polygon(points = {{ -100,10},{20,10},{20,41},{90,0},{20, -41},{20, -10},{ -100, -10},{ -100,10}}, lineColor = {0,127,0}, fillColor = {215,215,215}, fillPattern = FillPattern.Solid),Text(extent = {{ -100, -40},{ -47, -88}}, lineColor = {0,0,0}, textString = "f"),Text(extent = {{0,109},{0,49}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -30, -60},{30, -60}}, color = {0,0,0}),Line(points = {{0, -60},{0, -101}}, color = {0,0,0}),Line(points = {{ -30, -80},{ -10, -60}}, color = {0,0,0}),Line(points = {{ -10, -80},{10, -60}}, color = {0,0,0}),Line(points = {{10, -80},{30, -60}}, color = {0,0,0}),Polygon(points = {{ -61, -50},{ -30, -40},{ -30, -60},{ -61, -50}}, lineColor = {0,0,0}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{ -31, -50},{50, -50}}, color = {0,0,0}),Line(points = {{ -50, -80},{ -30, -60}}, color = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));

        equation 
          flange.f =  -f;
        end Force;
        model Force2 "Input signal acting as torque on two flanges"
          extends Translational.Interfaces.PartialTwoFlanges;
          Modelica.Blocks.Interfaces.RealInput f "driving force as input signal" annotation(Placement(transformation(extent = {{ -20, -20},{20,20}}, rotation = 270, origin = {0,60}), iconTransformation(extent = {{ -20, -20},{20,20}}, rotation = 270, origin = {0,40})));
          annotation(Documentation(info = "<html>
<p>
The input signal \"f\" in [N] characterizes an <i>external
force</i> which acts (with positive sign) at both flanges,
i.e., the components connected to these flanges are driven by force f.
</p>
<p>
Input signal s can be provided from one of the signal generator
blocks of Modelica.Blocks.Source.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{0, -40},{0, -100}}, textString = "%name", lineColor = {0,0,255}),Polygon(points = {{90,0},{60, -30},{60, -10},{10, -10},{10,10},{60,10},{60,31},{90,0}}, lineColor = {0,127,0}, smooth = Smooth.None, fillColor = {215,215,215}, fillPattern = FillPattern.Solid),Polygon(points = {{ -90,0},{ -60,30},{ -60,10},{ -10,10},{ -10, -10},{ -60, -10},{ -60, -30},{ -90,0}}, lineColor = {0,127,0}, smooth = Smooth.None, fillColor = {215,215,215}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));

        equation 
          flange_a.f = f;
          flange_b.f =  -f;
        end Force2;
        model LinearSpeedDependentForce "Linear dependency of force versus speed"
          extends Modelica.Mechanics.Translational.Interfaces.PartialForce;
          parameter Modelica.SIunits.Force f_nominal "Nominal force (if negative, force is acting as load)";
          parameter Boolean ForceDirection = true "Same direction of force in both directions of movement";
          parameter Modelica.SIunits.Velocity v_nominal(min = Modelica.Constants.eps) "Nominal speed";
          Modelica.SIunits.Velocity v "Velocity of flange with respect to support (= der(s))";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -100, -100},{100,100}}, color = {0,0,255})}), Documentation(info = "<HTML>
<p>
Model of force, linearly dependent on velocity of flange.<br>
Parameter ForceDirection chooses whether direction of force is the same in both directions of movement or not.
</p>
</HTML>"));

        equation 
          v = der(s);
          if ForceDirection then
            f =  -f_nominal * abs(v / v_nominal);

          else
            f =  -f_nominal * v / v_nominal;

          end if;
        end LinearSpeedDependentForce;
        model QuadraticSpeedDependentForce "Quadratic dependency of force versus speed"
          extends Modelica.Mechanics.Translational.Interfaces.PartialForce;
          parameter Modelica.SIunits.Force f_nominal "Nominal force (if negative, force is acting as load)";
          parameter Boolean ForceDirection = true "Same direction of force in both directions of movement";
          parameter Modelica.SIunits.Velocity v_nominal(min = Modelica.Constants.eps) "Nominal speed";
          Modelica.SIunits.Velocity v "Velocity of flange with respect to support (= der(s))";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -100, -100},{ -80, -98},{ -60, -92},{ -40, -82},{ -20, -68},{0, -50},{20, -28},{40, -2},{60,28},{80,62},{100,100}}, color = {0,0,255})}), Documentation(info = "<HTML>
<p>
Model of force, quadratic dependent on velocity of flange.<br>
Parameter ForceDirection chooses whether direction of force is the same in both directions of movement or not.
</p>
</HTML>"));

        equation 
          v = der(s);
          if ForceDirection then
            f =  -f_nominal * (v / v_nominal) ^ 2;

          else
            f =  -f_nominal * smooth(1, if v >= 0 then (v / v_nominal) ^ 2 else  -(v / v_nominal) ^ 2);

          end if;
        end QuadraticSpeedDependentForce;
        model ConstantForce "Constant force, not dependent on speed"
          extends Modelica.Mechanics.Translational.Interfaces.PartialForce;
          parameter Modelica.SIunits.Force f_constant "Nominal force (if negative, force is acting as load)";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -100,0},{98,0}}, color = {0,0,255}),Text(extent = {{ -118,58},{126,34}}, lineColor = {0,0,0}, textString = "%f_constant")}), Documentation(info = "<HTML>
<p>
Model of constant force, not dependent on velocity of flange.<br>
Positive force acts accelerating.
</p>
</HTML>"));

        equation 
          f =  -f_constant;
        end ConstantForce;
        model ConstantSpeed "Constant speed, not dependent on force"
          extends Modelica.Mechanics.Translational.Interfaces.PartialForce;
          parameter Modelica.SIunits.Velocity v_fixed "Fixed speed (if negative, force is acting as load)";
          Modelica.SIunits.Velocity v "Velocity of flange with respect to support (= der(s))";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{0, -100},{0,100}}, color = {0,0,255}),Text(extent = {{ -120,60},{124,36}}, lineColor = {0,0,0}, textString = "%v_fixed")}), Documentation(info = "<HTML>
<p>
Model of <b>fixed</b> verlocity of flange, not dependent on force.
</p>
</HTML>"));

        equation 
          v = der(s);
          v = v_fixed;
        end ConstantSpeed;
        model ForceStep "Constant force, not dependent on speed"
          extends Modelica.Mechanics.Translational.Interfaces.PartialForce;
          parameter Modelica.SIunits.Force stepForce(start = 1) "Height of force step (if negative, force is acting as load)";
          parameter Modelica.SIunits.Force offsetForce(start = 0) "Offset of force";
          parameter Modelica.SIunits.Time startTime = 0 "Force = offset for time < startTime";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -80, -60},{0, -60},{0,60},{80,60}}, color = {0,0,255}),Text(extent = {{0, -40},{100, -60}}, lineColor = {0,0,255}, textString = "time")}), Documentation(info = "<HTML>
<p>
Model of a force step at time .<br>
Positive force acts accelerating.
</p>
</HTML>"));

        equation 
          f =  -offsetForce - (if time < startTime then 0 else stepForce);
        end ForceStep;
      end Sources;
      package Sensors "Sensors for 1-dim. translational mechanical quantities"
        extends Modelica.Icons.Library2;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -76, -81},{64, -1}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -6, -61},{ -16, -37},{4, -37},{ -6, -61}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -6, -21},{ -6, -37}}, color = {0,0,0}),Line(points = {{ -76, -21},{ -6, -21}}, color = {0,0,0}),Line(points = {{ -56, -61},{ -56, -81}}, color = {0,0,0}),Line(points = {{ -36, -61},{ -36, -81}}, color = {0,0,0}),Line(points = {{ -16, -61},{ -16, -81}}, color = {0,0,0}),Line(points = {{4, -61},{4, -81}}, color = {0,0,0}),Line(points = {{24, -61},{24, -81}}, color = {0,0,0}),Line(points = {{44, -61},{44, -81}}, color = {0,0,0})}), Documentation(info = "<html>
<p>
This package contains ideal sensor components that provide
the connector variables as signals for further processing with the
Modelica.Blocks library.
</p>
</html>"));
        model PositionSensor "Ideal sensor to measure the absolute position"
          extends Translational.Interfaces.PartialAbsoluteSensor;
          Modelica.Blocks.Interfaces.RealOutput s "Absolute position of flange" annotation(Placement(transformation(extent = {{100, -11},{120,9}}, rotation = 0), iconTransformation(extent = {{100, -10},{120,10}})));
          annotation(Documentation(info = "<html>
<p>
Measures the <i>absolute position s</i> of a flange in an ideal way and provides the result as
output signals (to be further processed with blocks of the
Modelica.Blocks library).
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Text(extent = {{80, -28},{114, -62}}, lineColor = {0,0,0}, textString = "s")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          s = flange.s;
        end PositionSensor;
        model SpeedSensor "Ideal sensor to measure the absolute velocity"
          extends Translational.Interfaces.PartialAbsoluteSensor;
          Modelica.Blocks.Interfaces.RealOutput v "Absolute velocity of flange as output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>
Measures the <i>absolute velocity v</i> of a flange in an ideal way and provides the result as
output signals (to be further processed with blocks of the
Modelica.Blocks library).
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Text(extent = {{80, -28},{111, -61}}, lineColor = {0,0,0}, textString = "v")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          v = der(flange.s);
        end SpeedSensor;
        model AccSensor "Ideal sensor to measure the absolute acceleration"
          extends Translational.Interfaces.PartialAbsoluteSensor;
          SI.Velocity v "Absolute velocity of flange";
          Modelica.Blocks.Interfaces.RealOutput a "Absolute acceleration of flange as output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>
Measures the <i>absolute acceleration a</i>
of a flange in an ideal way and provides the result as
output signals (to be further processed with blocks of the
Modelica.Blocks library).
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Text(extent = {{80, -28},{115, -60}}, lineColor = {0,0,0}, textString = "a")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          v = der(flange.s);
          a = der(v);
        end AccSensor;
        model RelPositionSensor "Ideal sensor to measure the relative position"
          extends Translational.Interfaces.PartialRelativeSensor;
          Modelica.Blocks.Interfaces.RealOutput s_rel "Distance between two flanges (= flange_b.s - flange_a.s)" annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 270, origin = {0, -110})));
          annotation(Documentation(info = "<html>
<p>
Measures the <i>relative position s</i> of a flange in an ideal way and provides the result as
output signals (to be further processed with blocks of the
Modelica.Blocks library).
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{70.4,0},{100,0}}, color = {0,0,127}),Text(extent = {{8, -68},{42, -102}}, lineColor = {0,0,0}, textString = "s"),Line(points = {{0, -99},{0, -60}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          s_rel = flange_b.s - flange_a.s;
          0 = flange_a.f;
        end RelPositionSensor;
        model RelSpeedSensor "Ideal sensor to measure the relative speed"
          extends Translational.Interfaces.PartialRelativeSensor;
          SI.Position s_rel "distance between the two flanges (flange_b.s - flange_a.s)";
          Modelica.Blocks.Interfaces.RealOutput v_rel "Relative velocity between two flanges (= der(flange_b.s) - der(flange_a.s))" annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 270, origin = {0, -110})));
          annotation(Documentation(info = "<html>
<p>
Measures the <i>relative speed v</i> of a flange in an ideal way and provides the result as
output signals (to be further processed with blocks of the
Modelica.Blocks library).
</p>

</HTML>
", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>First Version from August 26, 1999 by P. Beater</i> </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{70.4,0},{100,0}}, color = {0,0,127}),Text(extent = {{8, -68},{42, -102}}, lineColor = {0,0,0}, textString = "v"),Line(points = {{0, -100},{0, -61}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          s_rel = flange_b.s - flange_a.s;
          v_rel = der(s_rel);
          0 = flange_a.f;
        end RelSpeedSensor;
        model RelAccSensor "Ideal sensor to measure the relative acceleration"
          extends Translational.Interfaces.PartialRelativeSensor;
          SI.Position s_rel "distance between the two flanges (flange_b.s - flange_a.s)";
          SI.Velocity v_rel "relative velocity between the two flanges (der(flange_b.s) - der(flange_a.s))";
          Modelica.Blocks.Interfaces.RealOutput a_rel "Relative acceleration between two flanges (= der(v_rel))" annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 270, origin = {0, -110})));
          annotation(Documentation(info = "<html>
<p>
Measures the <i>relative acceleration a</i> of a flange in an ideal way and provides the result as
output signals (to be further processed with blocks of the
Modelica.Blocks library).
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{70.4,0},{100,0}}, color = {0,0,127}),Text(extent = {{7, -68},{41, -102}}, lineColor = {0,0,0}, textString = "a"),Line(points = {{0, -99},{0, -60}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          s_rel = flange_b.s - flange_a.s;
          v_rel = der(s_rel);
          a_rel = der(v_rel);
          0 = flange_a.f;
        end RelAccSensor;
        model ForceSensor "Ideal sensor to measure the force between two flanges"
          extends Translational.Interfaces.PartialRelativeSensor;
          Modelica.Blocks.Interfaces.RealOutput f "force in flange_a and flange_b (f = flange_a.f = -flange_b.f)" annotation(Placement(transformation(origin = { -80, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          annotation(Documentation(info = "<html>
<p>
Measures the <i>cut-force between two flanges</i> in an ideal way
and provides the result as output signal (to be further processed
with blocks of the Modelica.Blocks library).
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -40, -70},{40, -120}}, lineColor = {0,0,0}, textString = "f"),Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{70,0},{90,0}}, color = {0,0,0}),Line(points = {{ -80, -100},{ -80,0}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          flange_a.s = flange_b.s;
          flange_a.f = f;
        end ForceSensor;
        model PowerSensor "Ideal sensor to measure the power between two flanges (= flange_a.f*der(flange_a.s))"
          extends Translational.Interfaces.PartialRelativeSensor;
          Modelica.Blocks.Interfaces.RealOutput power "Power in flange flange_a" annotation(Placement(transformation(origin = { -80, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          annotation(Documentation(info = "<html>
<p>
Measures the <b>power between two flanges</b> in an ideal way
and provides the result as output signal <b>power</b>
(to be further processed with blocks of the Modelica.Blocks library).
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -60, -70},{20, -120}}, lineColor = {0,0,0}, textString = "power"),Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{70,0},{90,0}}, color = {0,0,0}),Line(points = {{ -80, -100},{ -80,0}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          flange_a.s = flange_b.s;
          power = flange_a.f * der(flange_a.s);
        end PowerSensor;
      end Sensors;
      package Interfaces "Interfaces for 1-dim. translational mechanical components"
        extends Modelica.Icons.Library;
        annotation(Documentation(info = "<html>
<p>
This package contains connectors and partial models for 1-dim.
translational mechanical components. The components of this package can
only be used as basic building elements for models.
</p>

</html>
"));
        connector Flange_a "(left) 1D translational flange (flange axis directed INTO cut plane, e. g. from left to right)"
          annotation(defaultComponentName = "flange_a", Documentation(info = "<html>
This is a flange for 1D translational mechanical systems. In the cut plane of
the flange a unit vector n, called flange axis, is defined which is directed
INTO the cut plane, i. e. from left to right. All vectors in the cut plane are
resolved with respect to
this unit vector. E.g. force f characterizes a vector which is directed in
the direction of n with value equal to f. When this flange is connected to
other 1D translational flanges, this means that the axes vectors of the connected
flanges are identical.
</p>
<p>
The following variables are transported through this connector:
<pre>
  s: Absolute position of the flange in [m]. A positive translation
     means that the flange is translated along the flange axis.
  f: Cut-force in direction of the flange axis in [N].
</pre>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -100},{100,100}}, lineColor = {0,127,0}, fillColor = {0,127,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -40, -40},{40,40}}, lineColor = {0,127,0}, fillColor = {0,127,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -160,110},{40,50}}, lineColor = {0,127,0}, textString = "%name")}));
          SI.Position s "absolute position of flange";
          flow SI.Force f "cut force directed into flange";
        end Flange_a;
        connector Flange_b "right 1D translational flange (flange axis directed OUT OF cut plane)"
          SI.Position s "absolute position of flange";
          flow SI.Force f "cut force directed into flange";
          annotation(defaultComponentName = "flange_b", Documentation(info = "<html>
This is a flange for 1D translational mechanical systems. In the cut plane of
the flange a unit vector n, called flange axis, is defined which is directed
OUT OF the cut plane. All vectors in the cut plane are resolved with respect to
this unit vector. E.g. force f characterizes a vector which is directed in
the direction of n with value equal to f. When this flange is connected to
other 1D translational flanges, this means that the axes vectors of the connected
flanges are identical.
</p>
<p>
The following variables are transported through this connector:
<pre>
  s: Absolute position of the flange in [m]. A positive translation
     means that the flange is translated along the flange axis.
  f: Cut-force in direction of the flange axis in [N].
</pre>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100, -100},{100,100}}, lineColor = {0,127,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -40, -40},{40,40}}, lineColor = {0,127,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -40,110},{160,50}}, lineColor = {0,127,0}, textString = "%name")}));
        end Flange_b;
        connector Support "Support/housing 1D translational flange"
          SI.Position s "absolute position of flange";
          flow SI.Force f "cut force directed into flange";
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, fillColor = {175,175,175}, fillPattern = FillPattern.Solid, pattern = LinePattern.None),Text(extent = {{ -160,110},{40,50}}, lineColor = {0,127,0}, textString = "%name"),Rectangle(extent = {{ -40, -40},{40,40}}, lineColor = {0,127,0}, fillColor = {0,127,0}, fillPattern = FillPattern.Solid)}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -150,150},{150, -150}}, fillColor = {175,175,175}, fillPattern = FillPattern.Solid, pattern = LinePattern.None),Rectangle(extent = {{ -90, -90},{90,90}}, lineColor = {0,127,0}, fillColor = {0,127,0}, fillPattern = FillPattern.Solid)}));
        end Support;
        model InternalSupport "Adapter model to utilize conditional support connector"
          input SI.Force f "External support force (must be computed via force balance in model where InternalSupport is used; = flange.f)";
          SI.Position s "External support position (= flange.s)";
          Flange_a flange "Internal support flange (must be connected to the conditional support connector for useSupport=true and to conditional fixed model for useSupport=false)" annotation(Placement(transformation(extent = {{ -10, -10},{10,10}})));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -200,80},{200,40}}, lineColor = {0,0,255}, textString = "%name"),Rectangle(extent = {{ -20,20},{20, -20}}, lineColor = {0,127,0}, fillColor = {175,175,175}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>
This is an adapter model to utilize a conditional support connector
in an elementary component, i.e., where the component equations are
defined textually:
</p>

<ul>
<li> If <i>useSupport = true</i>, the flange has to be connected to the conditional
     support connector.</li>
<li> If <i>useSupport = false</i>, the flange has to be connected to the conditional
     fixed model.</li>
</ul>

<p>
Variable <b>f</b> is defined as <b>input</b> and must be provided when using
this component as a modifier (computed via a force balance in
the model where InternalSupport is used). Usually, model InternalSupport is
utilized via the partial models:
</p>

<blockquote>
<a href=\"Modelica://Modelica.Mechanics.Translational.Interfaces.PartialElementaryOneFlangeAndSupport\">
PartialElementaryOneFlangeAndSupport</a>,<br>
<a href=\"Modelica://Modelica.Mechanics.Translational.Interfaces.PartialElementaryTwoFlangesAndSupport\">
PartialElementaryTwoFlangesAndSupport</a>,<br>
<a href=\"Modelica://Modelica.Mechanics.Translational.Interfaces.PartialElementaryRotationalToTranslational\">
PartialElementaryRotationalToTranslational</a>.</li>
</blockquote>

<p>
Note, the support position can always be accessed as internalSupport.s, and
the support force can always be accessed as internalSupport.f.
</p>


</html>"));

        equation 
          flange.f = f;
          flange.s = s;
        end InternalSupport;
        partial model PartialTwoFlanges "Component with two translational 1D flanges "
          Flange_a flange_a "(left) driving flange (flange axis directed in to cut plane, e. g. from left to right)" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Flange_b flange_b "(right) driven flange (flange axis directed out of cut plane)" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>
This is a 1D translational component with two flanges.
It is used e.g. to built up parts of a drive train consisting
of several base components.
</p>

</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));
        end PartialTwoFlanges;
        partial model PartialOneFlangeAndSupport "Partial model for a component with one translational 1-dim. shaft flange and a support used for graphical modeling, i.e., the model is build up by drag-and-drop from elementary components"
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
          Flange_b flange "Flange of component" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Support support if useSupport "Support/housing of component" annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}})));
          annotation(Documentation(info = "<html>
<p>
This is a 1-dim. translational component with one flange and a support/housing.
It is used e.g. to build up parts of a drive train graphically consisting
of several components.
</p>

<p>
If <i>useSupport=true</i>, the support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupport=false</i>, the support connector is conditionally disabled
and instead the component is internally fixed to ground.
</p>

</html>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -38, -98},{ -6, -96}}, lineColor = {95,95,95}, textString = "(if useSupport)"),Text(extent = {{21, -95},{61, -96}}, lineColor = {95,95,95}, textString = "(if not useSupport)")}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(visible = not useSupport, points = {{ -50, -120},{ -30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -120},{ -10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -10, -120},{10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{10, -120},{30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -100},{30, -100}}, color = {0,0,0})}));
        protected 
          Support internalSupport "Internal support/housing of component (either connected to support, if useSupport=true, or connected to fixed, if useSupport=false)" annotation(Placement(transformation(extent = {{ -3, -83},{3, -77}})));
          Components.Fixed fixed if not useSupport "Fixed support/housing, if not useSupport" annotation(Placement(transformation(extent = {{10, -94},{30, -74}})));

        equation 
          connect(fixed.flange,internalSupport) annotation(Line(points = {{20, -84},{20, -80},{0, -80}}, color = {0,127,0}, smooth = Smooth.None));
          connect(internalSupport,support) annotation(Line(points = {{0, -80},{0, -100}}, pattern = LinePattern.None, smooth = Smooth.None));
        end PartialOneFlangeAndSupport;
        partial model PartialTwoFlangesAndSupport "Partial model for a component with two translational 1-dim. shaft flanges and a support used for graphical modeling, i.e., the model is build up by drag-and-drop from elementary components"
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
          Flange_a flange_a "Flange of left end" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Flange_b flange_b "Flange of right end" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Support support if useSupport "Support/housing of component" annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}})));
          annotation(Documentation(info = "<html>
<p>
This is a 1-dim. translational component with two flanges and a support/housing.
It is used e.g. to build up parts of a drive train graphically consisting
of several components.
</p>

<p>
If <i>useSupport=true</i>, the support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupport=false</i>, the support connector is conditionally disabled
and instead the component is internally fixed to ground.
</p>

</html>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -38, -98},{ -6, -96}}, lineColor = {95,95,95}, textString = "(if useSupport)"),Text(extent = {{24, -97},{64, -98}}, lineColor = {95,95,95}, textString = "(if not useSupport)")}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(visible = not useSupport, points = {{ -50, -120},{ -30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -120},{ -10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -10, -120},{10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{10, -120},{30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -100},{30, -100}}, color = {0,0,0})}));
        protected 
          Support internalSupport "Internal support/housing of component (either connected to support, if useSupport=true, or connected to fixed, if useSupport=false)" annotation(Placement(transformation(extent = {{ -3, -83},{3, -77}})));
          Components.Fixed fixed if not useSupport "Fixed support/housing, if not useSupport" annotation(Placement(transformation(extent = {{10, -97},{30, -77}})));

        equation 
          connect(fixed.flange,internalSupport) annotation(Line(points = {{20, -87},{20, -80},{0, -80}}, color = {0,127,0}, smooth = Smooth.None));
          connect(internalSupport,support) annotation(Line(points = {{0, -80},{0, -100}}, pattern = LinePattern.None, smooth = Smooth.None));
        end PartialTwoFlangesAndSupport;
        partial model PartialRigid "Rigid connection of two translational 1D flanges "
          SI.Position s "Absolute position of center of component (s = flange_a.s + L/2 = flange_b.s - L/2)";
          parameter SI.Length L(start = 0) "Length of component, from left flange to right flange (= flange_b.s - flange_a.s)";
          Flange_a flange_a "Left flange of translational component" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Flange_b flange_b "Right flange of translational component" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>
This is a 1-dim. translational component with two <i>rigidly</i> connected flanges.
The fixed distance between the left and the right flange is defined by parameter \"L\".
The forces at the right and left flange can be different.
It is used e.g. to built up sliding masses.
</p>
</html>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));

        equation 
          flange_a.s = s - L / 2;
          flange_b.s = s + L / 2;
        end PartialRigid;
        partial model PartialCompliant "Compliant connection of two translational 1D flanges"
          Flange_a flange_a "Left flange of compliant 1-dim. translational component" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Flange_b flange_b "Right flange of compliant 1-dim. translational component" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          SI.Distance s_rel(start = 0) "relative distance (= flange_b.s - flange_a.s)";
          SI.Force f "force between flanges (positive in direction of flange axis R)";
          annotation(Documentation(info = "<html>
<p>
This is a 1D translational component with a <i>compliant </i>connection of two
translational 1D flanges where inertial effects between the two
flanges are not included. The absolute value of the force at the left and the right
flange is the same. It is used to built up springs, dampers etc.
</p>

</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Polygon(points = {{50, -90},{20, -80},{20, -100},{50, -90}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{ -60, -90},{20, -90}}, color = {0,0,0})}));

        equation 
          s_rel = flange_b.s - flange_a.s;
          flange_b.f = f;
          flange_a.f =  -f;
        end PartialCompliant;
        partial model PartialCompliantWithRelativeStates "Base model for the compliant connection of two translational 1-dim. shaft flanges where the relative position and relative velocities are used as states"
          parameter StateSelect stateSelect = StateSelect.prefer "Priority to use phi_rel and w_rel as states" annotation(HideResult = true, Dialog(tab = "Advanced"));
          parameter SI.Distance s_nominal = 0.0001 "Nominal value of s_rel (used for scaling)" annotation(Dialog(tab = "Advanced"));
          SI.Distance s_rel(start = 0, stateSelect = stateSelect, nominal = s_nominal) "Relative distance (= flange_b.s - flange_a.s)";
          SI.Velocity v_rel(start = 0, stateSelect = stateSelect) "Relative velocity (= der(s_rel))";
          SI.Force f "Forces between flanges (= flange_b.f)";
          Translational.Interfaces.Flange_a flange_a "Left flange of compliant 1-dim. translational component" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Translational.Interfaces.Flange_b flange_b "Right flange of compliant 1-dim. transational component" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>
This is a 1-dim. translational component with a compliant connection of two
translational 1-dim. flanges where inertial effects between the two
flanges are neglected. The basic assumption is that the cut-forces
of the two flanges sum-up to zero, i.e., they have the same absolute value
but opposite sign: flange_a.f + flange_b.f = 0. This base class
is used to built up force elements such as springs, dampers, friction.
</p>

<p>
The difference to base classe \"PartialCompliant\" is that the relative
distance and the relative velocity are defined as preferred states.
The reason is that for a large class of drive trains,
the absolute position is quickly increasing during operation.
Numerically, it is better to use relative distances between drive train components
because they remain in a limited size. For this reason, StateSelect.prefer
is set for the relative distance of this component.
</p>

<p>
In order to improve the numerics, a nominal value for the relative distance
should be set, since drive train distances are in a small order and
then step size control of the integrator is practically switched off for
such a variable. A default nominal value of s_nominal = 1e-4 is defined.
This nominal value might also be computed from other values, such
as \"s_nominal = f_nominal / c\" for a spring, if f_nominal
and c have more meaningful values for the user.
</p>

</html>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));

        equation 
          s_rel = flange_b.s - flange_a.s;
          v_rel = der(s_rel);
          flange_b.f = f;
          flange_a.f =  -f;
        end PartialCompliantWithRelativeStates;
        partial model PartialElementaryOneFlangeAndSupport "Obsolete partial model. Use PartialElementaryOneFlangeAndSupport2."
          extends Modelica.Icons.ObsoleteModel;
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
          Modelica.SIunits.Length s = flange.s - internalSupport.s "distance between flange and support (= flange.s - support.s)";
          Flange_b flange "Flange of component" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>
This is a 1-dim. translational component with one flange and a support/housing.
It is used to build up elementary components of a drive train with
equations in the text layer.
</p>

<p>
If <i>useSupport=true</i>, the support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupport=false</i>, the support connector is conditionally disabled
and instead the component is internally fixed to ground.
</p>

</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -38, -98},{ -6, -96}}, lineColor = {95,95,95}, textString = "(if useSupport)"),Text(extent = {{24, -97},{64, -98}}, lineColor = {95,95,95}, textString = "(if not useSupport)")}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(visible = not useSupport, points = {{ -50, -120},{ -30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -120},{ -10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -10, -120},{10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{10, -120},{30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -100},{30, -100}}, color = {0,0,0})}));
        protected 
          InternalSupport internalSupport(f =  -flange.f) "Internal support/housing of component as a model with connector flange (flange is either connected to support, if useSupport=true, or connected to fixed, if useSupport=false)" annotation(Placement(transformation(extent = {{ -10, -90},{10, -70}})));
          Components.Fixed fixed if not useSupport "Fixed support/housing, if not useSupport" annotation(Placement(transformation(extent = {{10, -97},{30, -77}})));
        public 
          Support support if useSupport "Support/housing of component" annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}})));

        equation 
          connect(internalSupport.flange,support) annotation(Line(points = {{0, -80},{0, -100}}, color = {0,127,0}, smooth = Smooth.None));
          connect(fixed.flange,internalSupport.flange) annotation(Line(points = {{20, -87},{20, -80},{0, -80}}, color = {0,127,0}, smooth = Smooth.None));
        end PartialElementaryOneFlangeAndSupport;
        partial model PartialElementaryOneFlangeAndSupport2 "Partial model for a component with one translational 1-dim. shaft flange and a support used for textual modeling, i.e., for elementary models"
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
          Modelica.SIunits.Length s = flange.s - s_support "distance between flange and support (= flange.s - support.s)";
          Flange_b flange "Flange of component" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Support support(s = s_support, f =  -flange.f) if useSupport "Support/housing of component" annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}})));
        protected 
          Modelica.SIunits.Length s_support "Absolute position of support flange";

        equation 
if not useSupport then
                      s_support = 0;
          end if;
          annotation(Documentation(info = "<html>
<p>
This is a 1-dim. translational component with one flange and a support/housing.
It is used to build up elementary components of a drive train with
equations in the text layer.
</p>

<p>
If <i>useSupport=true</i>, the support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupport=false</i>, the support connector is conditionally disabled
and instead the component is internally fixed to ground.
</p>

</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(visible = not useSupport, points = {{ -50, -120},{ -30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -120},{ -10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -10, -120},{10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{10, -120},{30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -100},{30, -100}}, color = {0,0,0})}));
        end PartialElementaryOneFlangeAndSupport2;
        partial model PartialElementaryTwoFlangesAndSupport "Obsolete partial model. Use PartialElementaryTwoFlangesAndSupport2."
          extends Modelica.Icons.ObsoleteModel;
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
          Flange_a flange_a "Flange of left shaft" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Flange_b flange_b "Flange of right shaft" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.SIunits.Length s_a = flange_a.s - internalSupport.s "Distance between left flange and support";
          Modelica.SIunits.Length s_b = flange_b.s - internalSupport.s "Distance between right flange and support";
          annotation(Documentation(info = "<html>
<p>
This is a 1-dim. translational component with two flanges and an additional support.
It is used e.g. to build up elementary ideal gear components. The component
contains the force balance, i.e., the sum of the forces of the connectors
is zero (therefore, components that are based on PartialGear cannot have
a mass). The support connector needs to be connected
to avoid the unphysical behavior that the
support force is required to be zero (= the default value, if the
connector is not connected).
</p>

</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -38, -98},{ -6, -96}}, lineColor = {95,95,95}, textString = "(if useSupport)"),Text(extent = {{24, -97},{64, -98}}, lineColor = {95,95,95}, textString = "(if not useSupport)")}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(visible = not useSupport, points = {{ -50, -120},{ -30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -120},{ -10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -10, -120},{10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{10, -120},{30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -100},{30, -100}}, color = {0,0,0})}));
        protected 
          InternalSupport internalSupport(f =  -flange_a.f - flange_b.f) "Internal support/housing of component as a model with connector flange (flange is either connected to support, if useSupport=true, or connected to fixed, if useSupport=false)" annotation(Placement(transformation(extent = {{ -10, -90},{10, -70}})));
          Components.Fixed fixed if not useSupport "Fixed support/housing, if not useSupport" annotation(Placement(transformation(extent = {{10, -97},{30, -77}})));
        public 
          Support support if useSupport "Support/housing of component" annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}})));

        equation 
          connect(internalSupport.flange,support) annotation(Line(points = {{0, -80},{0, -100}}, color = {0,127,0}, smooth = Smooth.None));
          connect(fixed.flange,internalSupport.flange) annotation(Line(points = {{20, -87},{20, -80},{0, -80}}, color = {0,127,0}, smooth = Smooth.None));
        end PartialElementaryTwoFlangesAndSupport;
        partial model PartialElementaryTwoFlangesAndSupport2 "Partial model for a component with one translational 1-dim. shaft flange and a support used for textual modeling, i.e., for elementary models"
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
          Flange_a flange_a "Flange of left shaft" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Flange_b flange_b "Flange of right shaft" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Support support(s = s_support, f =  -flange_a.f - flange_b.f) if useSupport "Support/housing of component" annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}})));
          Modelica.SIunits.Length s_a = flange_a.s - s_support "Distance between left flange and support";
          Modelica.SIunits.Length s_b = flange_b.s - s_support "Distance between right flange and support";
        protected 
          Modelica.SIunits.Length s_support "Absolute position of support flange";

        equation 
if not useSupport then
                      s_support = 0;
          end if;
          annotation(Documentation(info = "<html>
<p>
This is a 1-dim. translational component with two flanges and an additional support.
It is used e.g. to build up elementary ideal gear components. The component
contains the force balance, i.e., the sum of the forces of the connectors
is zero (therefore, components that are based on PartialGear cannot have
a mass). The support connector needs to be connected
to avoid the unphysical behavior that the
support force is required to be zero (= the default value, if the
connector is not connected).
</p>

</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(visible = not useSupport, points = {{ -50, -120},{ -30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -120},{ -10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -10, -120},{10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{10, -120},{30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -100},{30, -100}}, color = {0,0,0})}));
        end PartialElementaryTwoFlangesAndSupport2;
        partial model PartialElementaryRotationalToTranslational
          extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryRotationalToTranslational;
        end PartialElementaryRotationalToTranslational;
        partial model PartialForce "Partial model of a force acting at the flange (accelerates the flange)"
          extends PartialElementaryOneFlangeAndSupport2;
          Modelica.SIunits.Force f = flange.f "Accelerating force acting at flange (= flange.f)";
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -96,96},{96, -96}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0, -60},{0, -100}}, color = {0,0,0}),Text(extent = {{ -150,140},{150,100}}, lineColor = {0,0,255}, textString = "%name"),Line(points = {{ -78,80},{51,80}}, color = {0,0,0}),Polygon(points = {{81,80},{51,90},{51,70},{81,80}}, lineColor = {0,0,0}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{ -52, -60},{77, -60}}, color = {0,0,0}),Polygon(points = {{ -82, -60},{ -51, -50},{ -51, -70},{ -82, -60}}, lineColor = {0,0,0}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(visible = not useSupport, points = {{ -50, -120},{ -30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -120},{ -10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -10, -120},{10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{10, -120},{30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -100},{30, -100}}, color = {0,0,0})}), Documentation(info = "<HTML>
<p>
Partial model of force that accelerates the flange.
</p>

<p>
If <i>useSupport=true</i>, the support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupport=false</i>, the support connector is conditionally disabled
and instead the component is internally fixed to ground.
</p>
</HTML>"));
        end PartialForce;
        partial model PartialAbsoluteSensor "Device to measure a single absolute flange variable"
          extends Modelica.Icons.TranslationalSensor;
          Interfaces.Flange_a flange "flange to be measured (flange axis directed in to cut plane, e. g. from left to right)" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>
This is the superclass of a 1D translational component with one flange and one
output signal in order to measure an absolute kinematic quantity in the flange
and to provide the measured signal as output signal for further processing
with the Modelica.Blocks blocks.
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -100, -90},{ -20, -90}}, color = {0,0,0}),Polygon(points = {{10, -90},{ -20, -80},{ -20, -100},{10, -90}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{70,0},{100,0}}, color = {0,0,127}),Text(extent = {{ -118,99},{118,40}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          0 = flange.f;
        end PartialAbsoluteSensor;
        partial model PartialRelativeSensor "Device to measure a single relative variable between two flanges"
          extends Modelica.Icons.TranslationalSensor;
          Interfaces.Flange_a flange_a "(left) driving flange (flange axis directed in to cut plane, e. g. from left to right)" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Interfaces.Flange_b flange_b "(right) driven flange (flange axis directed out of cut plane)" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>
This is a superclass for 1D translational components with two rigidly connected
flanges and one output signal in order to measure relative kinematic quantities
between the two flanges or the cut-force in the flange and
to provide the measured signal as output signal for further processing
with the Modelica.Blocks blocks.
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -51,34},{29,34}}, color = {0,0,0}),Polygon(points = {{59,34},{29,44},{29,24},{59,34}}, lineColor = {0,0,0}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{70,0},{90,0}}, color = {0,0,0}),Text(extent = {{ -117,116},{115,52}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          0 = flange_a.f + flange_b.f;
        end PartialRelativeSensor;
        partial model PartialFriction "Base model of Coulomb friction elements"
          annotation(Documentation(info = "<html>
<p>
Basic model for Coulomb friction that models the stuck phase in a reliable way.
</p>
</html>
"));
          parameter SI.Velocity v_small = 0.001 "Relative velocity near to zero (see model info text)" annotation(Dialog(tab = "Advanced"));
          SI.Velocity v_relfric "Relative velocity between frictional surfaces";
          SI.Acceleration a_relfric "Relative acceleration between frictional surfaces";
          SI.Force f0 "Friction force for v=0 and forward sliding";
          SI.Force f0_max "Maximum friction force for v=0 and locked";
          Boolean free "true, if frictional element is not active";
          Real sa(unit = "1") "Path parameter of friction characteristic f = f(a_relfric)";
          Boolean startForward(start = false, fixed = true) "true, if v_rel=0 and start of forward sliding";
          Boolean startBackward(start = false, fixed = true) "true, if v_rel=0 and start of backward sliding";
          Boolean locked(start = false) "true, if v_rel=0 and not sliding";
          constant Integer Unknown = 3 "Value of mode is not known";
          constant Integer Free = 2 "Element is not active";
          constant Integer Forward = 1 "v_rel > 0 (forward sliding)";
          constant Integer Stuck = 0 "v_rel = 0 (forward sliding, locked or backward sliding)";
          constant Integer Backward =  -1 "v_rel < 0 (backward sliding)";
          Integer mode(final min = Backward, final max = Unknown, start = Unknown, fixed = true);
        protected 
          constant SI.Acceleration unitAcceleration = 1 annotation(HideResult = true);
          constant SI.Force unitForce = 1 annotation(HideResult = true);

        equation 
          startForward = pre(mode) == Stuck and (sa > f0_max / unitForce or pre(startForward) and sa > f0 / unitForce) or pre(mode) == Backward and v_relfric > v_small or initial() and v_relfric > 0;
          startBackward = pre(mode) == Stuck and (sa <  -f0_max / unitForce or pre(startBackward) and sa <  -f0 / unitForce) or pre(mode) == Forward and v_relfric <  -v_small or initial() and v_relfric < 0;
          locked = not free and not (pre(mode) == Forward or startForward or pre(mode) == Backward or startBackward);
          a_relfric / unitAcceleration = if locked then 0 else if free then sa else if startForward then sa - f0_max / unitForce else if startBackward then sa + f0_max / unitForce else if pre(mode) == Forward then sa - f0_max / unitForce else sa + f0_max / unitForce;
          mode = if free then Free else if (pre(mode) == Forward or pre(mode) == Free or startForward) and v_relfric > 0 then Forward else if (pre(mode) == Backward or pre(mode) == Free or startBackward) and v_relfric < 0 then Backward else Stuck;
        end PartialFriction;
      end Interfaces;
    end Translational;
    package Rotational "Library to model 1-dimensional, rotational mechanical systems"
      extends Modelica.Icons.Library2;
      import SI = Modelica.SIunits;
      annotation(Documentation(info = "<html>

<p>
Library <b>Rotational</b> is a <b>free</b> Modelica package providing
1-dimensional, rotational mechanical components to model in a convenient way
drive trains with frictional losses. A typical, simple example is shown
in the next figure:
</p>

<p><img src=\"../Images/Rotational/driveExample.png\"></p>

<p>
For an introduction, have especially a look at:
</p>
<ul>
<li> <a href=\"Modelica://Modelica.Mechanics.Rotational.UsersGuide\">Rotational.UsersGuide</a>
     discusses the most important aspects how to use this library.</li>
<li> <a href=\"Modelica://Modelica.Mechanics.Rotational.Examples\">Rotational.Examples</a>
     contains examples that demonstrate the usage of this library.</li>
</ul>

<p>
In version 3.0 of the Modelica Standard Library, the basic design of the
library has changed: Previously, bearing connectors could or could not be connected.
In 3.0, the bearing connector is renamed to \"support\" and this connector
is enabled via parameter \"useSupport\". If the support connector is enabled,
it must be connected, and if it is not enabled, it must not be connected.
</p>

<p>
Copyright &copy; 1998-2009, Modelica Association and DLR.
</p>
<p>
<i>This Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p><br>
</html>
", revisions = ""), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -83, -66},{ -63, -66}}, color = {0,0,0}),Line(points = {{36, -68},{56, -68}}, color = {0,0,0}),Line(points = {{ -73, -66},{ -73, -91}}, color = {0,0,0}),Line(points = {{46, -68},{46, -91}}, color = {0,0,0}),Line(points = {{ -83, -29},{ -63, -29}}, color = {0,0,0}),Line(points = {{36, -32},{56, -32}}, color = {0,0,0}),Line(points = {{ -73, -9},{ -73, -29}}, color = {0,0,0}),Line(points = {{46, -12},{46, -32}}, color = {0,0,0}),Line(points = {{ -73, -91},{46, -91}}, color = {0,0,0}),Rectangle(extent = {{ -47, -17},{27, -80}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -87, -41},{ -47, -54}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{27, -42},{66, -56}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192})}));
      package UsersGuide "User's Guide of Rotational Library"
        annotation(__Dymola_DocumentationClass = true, Documentation(info = "<HTML>
<p>
Library <b>Rotational</b> is a <b>free</b> Modelica package providing
1-dimensional, rotational mechanical components to model in a convenient way
drive trains with frictional losses.
</p>

</HTML>"));
        class Overview "Overview"
          annotation(__Dymola_DocumentationClass = true, Documentation(info = "<HTML>

<p>
This package contains components to model <b>1-dimensional rotational
mechanical</b> systems, including different types of gearboxes,
shafts with inertia, external torques, spring/damper elements,
frictional elements, backlash, elements to measure angle, angular velocity,
angular acceleration and the cut-torque of a flange. In sublibrary
<b>Examples</b> several examples are present to demonstrate the usage of
the elements. Just open the corresponding example model and simulate
the model according to the provided description.
</p>
<p>
A unique feature of this library is the <b>component-oriented</b>
modeling of <b>Coulomb friction</b> elements, such as friction in bearings,
clutches, brakes, and gear efficiency. Even (dynamically) coupled
friction elements, e.g., as in automatic gearboxes, can be handeled
<b>without</b> introducing stiffness which leads to fast simulations.
The underlying theory is new and is based on the solution of mixed
continuous/discrete systems of equations, i.e., equations where the
<b>unknowns</b> are of type <b>Real</b>, <b>Integer</b> or <b>Boolean</b>.
Provided appropriate numerical algorithms for the solution of such types of
systems are available in the simulation tool, the simulation of
(dynamically) coupled friction elements of this library is
<b>efficient</b> and <b>reliable</b>.
</p>
<p><IMG SRC=\"../Images/Rotational/drive1.png\" ALT=\"drive1\"></p>
<p>
A simple example of the usage of this library is given in the
figure above. This drive consists of a shaft with inertia J1=0.2 which
is connected via an ideal gearbox with gear ratio=5 to a second shaft
with inertia J2=5. The left shaft is driven via an external,
sinusoidal torque.
The <b>filled</b> and <b>non-filled grey squares</b> at the left and
right side of a component represent <b>mechanical flanges</b>.
Drawing a line between such squares means that the corresponding
flanges are <b>rigidly attached</b> to each other.
By convention in this library, the connector characterized as a
<b>filled</b> grey square is called <b>flange_a</b> and placed at the
left side of the component in the \"design view\" and the connector
characterized as a <b>non-filled</b> grey square is called <b>flange_b</b>
and placed at the right side of the component in the \"design view\".
The two connectors are completely <b>identical</b>, with the only
exception that the graphical layout is a little bit different in order
to distinguish them for easier access of the connector variables.
For example, <tt>J1.flange_a.tau</tt> is the cut-torque in the connector
<tt>flange_a</tt> of component <tt>J1</tt>.
</p>
<p>
The components of this
library can be <b>connected</b> together in an <b>arbitrary</b> way. E.g., it is
possible to connect two springs or two shafts with inertia directly
together, see figure below.
</p>
<p><IMG SRC=\"../Images/Rotational/driveConnections1.png\" ALT=\"driveConnections1\"></p>
<p><IMG SRC=\"../Images/Rotational/driveConnections2.png\" ALT=\"driveConnections2\"></p>

</HTML>"));
        end Overview;
        class FlangeConnectors "Flange Connectors"
          annotation(__Dymola_DocumentationClass = true, Documentation(info = "<HTML>
<p>
A flange is described by the connector class
Interfaces.<b>Flange_a</b>
or Interfaces.<b>Flange_b</b>. As already noted, the two connector
classes are completely identical. There is only a difference in the icons,
in order to easier identify a flange variable in a diagram.
Both connector classes contain the following variables:
</p>
<pre>
   Modelica.SIunits.Angle       phi  \"Absolute rotation angle of flange\";
   <b>flow</b> Modelica.SIunits.Torque tau  \"Cut-torque in the flange\";
</pre>
<p>
If needed, the angular velocity <tt>w</tt> and the
angular acceleration <tt>a</tt> of a flange connector can be
determined by differentiation of the flange angle <tt>phi</tt>:
</p>
<pre>
     w = <b>der</b>(phi);    a = <b>der</b>(w);
</pre>
</HTML>"));
        end FlangeConnectors;
        class SupportTorques "Support Torques"
          annotation(__Dymola_DocumentationClass = true, Documentation(info = "<HTML>

<p>The following figure shows examples of components equipped with
a support flange (framed flange in the lower center), which can be used
to fix components on the ground or on other rotating elements or to combine
them with force elements. Via Boolean parameter <b>useSupport</b>, the
support torque is enabled or disabled. If it is enabled, it must be connected.
If it is disabled, it must not be connected.
Enabled support flanges offer, e.g., the possibility to model gearboxes mounted on
the ground via spring-damper-systems (cf. example
<a href=\"Modelica://Modelica.Mechanics.Rotational.Examples.ElasticBearing\">ElasticBearing</a>).
</p>

<p><IMG SRC=\"../Images/Rotational/bearing.png\" ALT=\"bearing\"></p>

<p>
Depending on the setting of <b>useSupport</b>, the icon of the corresponding
component is changing, to either show the support flange or a ground mounting.
For example, the two implementations in the following figure give
identical results.</p>

<p><IMG SRC=\"../Images/Rotational/bearing2.png\" ALT=\"bearing2\"></p>

</HTML>"));
        end SupportTorques;
        class SignConventions "Sign Conventions"
          annotation(__Dymola_DocumentationClass = true, Documentation(info = "<HTML>

<p>
The variables of a component of this library can be accessed in the
usual way. However, since most of these variables are basically elements
of <b>vectors</b>, i.e., have a direction, the question arises how the
signs of variables shall be interpreted. The basic idea is explained
at hand of the following figure:
</p>
<p><IMG SRC=\"../Images/Rotational/drive2.png\" ALT=\"drive2\"></p>
<p>
In the figure, three identical drive trains are shown. The only
difference is that the gear of the middle drive train and the
gear as well as the right inertia of the lower drive train
are horizontally flipped with regards to the upper drive train.
The signs of variables are now interpreted in the following way:
Due to the 1-dimensional nature of the model, all components are
basically connected together along one line (more complicated
cases are discussed below). First, one has to define
a <b>positive</b> direction of this line, called <b>axis of rotation</b>.
In the top part of the figure this is characterized by an arrow
defined as <tt>axis of rotation</tt>. The simple rule is now:
If a variable of a component is positive and can be interpreted as
the element of a vector (e.g. torque or angular velocity vector), the
corresponding vector is directed into the positive direction
of the axis of rotation. In the following figure, the right-most
inertias of the figure above are displayed with the positive
vector direction displayed according to this rule:
</p>
<p><IMG SRC=\"../Images/Rotational/drive3.png\" ALT=\"drive3\"></p>
<p>
The cut-torques <tt>J2.flange_a.tau, J4.flange_a.tau, J6.flange_b.tau</tt>
of the right inertias are all identical and are directed into the
direction of rotation if the values are positive. Similiarily,
the angular velocities <tt>J2.w, J4.w, J6.w</tt> of the right inertias
are all identical and are also directed into the
direction of rotation if the values are positive. Some special
cases are shown in the next figure:
</p>
<p><IMG SRC=\"../Images/Rotational/drive4.png\" ALT=\"drive4\"></p>
<p>
In the upper part of the figure, two variants of the connection of an
external torque and an inertia are shown. In both cases, a positive
signal input into the torque component accelerates the inertias
<tt>inertia1, inertia2</tt> into the positive axis of rotation,
i.e., the angular accelerations <tt>inertia1.a, inertia2.a</tt>
are positive and are directed along the \"axis of rotation\" arrow.
In the lower part of the figure the connection of inertias with
a planetary gear is shown. Note, that the three flanges of the
planetary gearbox are located along the axis of rotation and that
the axis direction determines the positive rotation along these
flanges. As a result, the positive rotation for <tt>inertia4, inertia6</tt>
is as indicated with the additional grey arrows.
</p>
</HTML>"));
        end SignConventions;
        class UserDefinedComponents "User Defined Components"
          annotation(__Dymola_DocumentationClass = true, Documentation(info = "<HTML>
<p>
In this section some hints are given to define your own
1-dimensional rotational components which are compatible with the
elements of this package.
It is convenient to define a new
component by inheritance from one of the following base classes,
which are defined in sublibrary Interfaces:
</p>
<table BORDER=1 CELLSPACING=0 CELLPADDING=2>
<tr><th>Name</th><th>Description</th></tr>
<tr>
  <td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.Rotational.Interfaces.PartialCompliant\">PartialCompliant</a>
  </td>
  <td valign=\"top\">Compliant connection of two rotational 1-dim. flanges
                   (used for force laws such as a spring or a damper).</td>
</tr>

<tr>
  <td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.Rotational.Interfaces.PartialCompliantWithRelativeStates\">PartialCompliantWithRelativeStates</a>
  </td>
  <td valign=\"top\"> Same as \"PartialCompliant\", but relative angle and relative speed are
                    defined as preferred states. Use this partial model if the force law
                    needs anyway the relative speed. The advantage is that it is usually better
                    to use relative angles between drive train components
                    as states, especially, if the angle is not limited (e.g., as for drive trains
                    in vehicles).
</td>
</tr>


<tr>
  <td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.Rotational.Interfaces.PartialElementaryTwoFlangesAndSupport2\">PartialElementaryTwoFlangesAndSupport2</a>
</td>
  <td valign=\"top\"> Partial model for a 1-dim. rotational gear consisting of the flange of
                    an input shaft, the flange of an output shaft and the support. 
  </td>
</tr>

<tr>
  <td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.Rotational.Interfaces.PartialTorque\">PartialTorque</a>
</td>
  <td valign=\"top\"> Partial model of a torque acting at the flange (accelerates the flange).
  </td>
</tr>

<tr>
  <td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.Rotational.Interfaces.PartialTwoFlanges\">PartialTwoFlanges</a>
</td>
  <td valign=\"top\">General connection of two rotational 1-dim. flanges.
  </td>
</tr>

<tr>
  <td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.Rotational.Interfaces.PartialAbsoluteSensor\">PartialAbsoluteSensor</a>
</td>
  <td valign=\"top\">Measure absolute flange variables.
  </td>
</tr>

<tr>
  <td valign=\"top\"><a href=\"Modelica://Modelica.Mechanics.Rotational.Interfaces.PartialRelativeSensor\">PartialRelativeSensor</a>
</td>
  <td valign=\"top\">Measure relative flange variables.
  </td>
</tr>
</table>

<p>
The difference between these base classes are the auxiliary
variables defined in the model and the relations between
the flange variables already defined in the base class.
For example, in model <b>PartialCompliant</b> there is no 
support flange, whereas in model 
<b>PartialElementaryTwoFlangesAndSupport2</b>
there is a support flange.
</p>
<p>
The equations of a mechanical component are vector equations, i.e.,
they need to be expressed in a common coordinate system.
Therefore, for a component a <b>local axis of rotation</b> has to be
defined. All vector quantities, such as cut-torques or angular
velocities have to be expressed according to this definition.
Examples for such a definition are given in the following figure
for an inertia component and a planetary gearbox:
</p>
<p><IMG SRC=\"../Images/Rotational/driveAxis.png\" ALT=\"driveAxis\"></p>
<p>
As can be seen, all vectors are directed into the direction
of the rotation axis. The angles in the flanges are defined
correspondingly. For example, the angle <tt>sun.phi</tt> in the
flange of the sun wheel of the planetary gearbox is positive,
if rotated in mathematical positive direction (= counter clock
wise) along the axis of rotation.
</p>
<p>
On first view, one may assume that the selected local
coordinate system has an influence on the usage of the
component. But this is not the case, as shown in the next figure:
</p>
<p><IMG SRC=\"../Images/Rotational/inertias.png\" ALT=\"inertias\"></p>
<p>
In the figure the <b>local</b> axes of rotation of the components
are shown. The connection of two inertias in the left and in the
right part of the figure are completely equivalent, i.e., the right
part is just a different drawing of the left part. This is due to the
fact, that by a connection, the two local coordinate systems are
made identical and the (automatically) generated connection equations
(= angles are identical, cut-torques sum-up to zero) are also
expressed in this common coordinate system. Therefore, even if in
the left figure it seems to be that the angular velocity vector of
<tt>J2</tt> goes from right to left, in reality it goes from
left to right as shown in the right part of the figure, where the
local coordinate systems are drawn such that they are aligned.
Note, that the simple rule stated in section 4 (Sign conventions)
also determines that
the angular velocity of <tt>J2</tt> in the left part of the
figure is directed from left to right.
</p>
<p>
To summarize, the local coordinate system selected for a component
is just necessary, in order that the equations of this component
are expressed correctly. The selection of the coordinate system
is arbitrary and has no influence on the usage of the component.
Especially, the actual direction of, e.g., a cut-torque is most
easily determined by the rule of section 4. A more strict determination
by aligning coordinate systems and then using the vector direction
of the local coordinate systems, often requires a re-drawing of the
diagram and is therefore less convenient to use.
</p>
</HTML>"));
        end UserDefinedComponents;
        class RequirementsForSimulationTool "Requirements for Simulation Tools"
          annotation(__Dymola_DocumentationClass = true, Documentation(info = "<HTML>

<p>
This library is designed in a fully object oriented way in order that
components can be connected together in every meaningful combination
(e.g. direct connection of two springs or two inertias).
As a consequence, most models lead to a system of
differential-algebraic equations of <b>index 3</b> (= constraint
equations have to be differentiated twice in order to arrive at
a state space representation) and the Modelica translator or
the simulator has to cope with this system representation.
According to our present knowledge, this requires that the
Modelica translator is able to symbolically differentiate equations
(otherwise it is e.g. not possible to provide consistent initial
conditions; even if consistent initial conditions are present, most
numerical DAE integrators can cope at most with index 2 DAEs).
</p>
</p>
The elements of this library can be connected together in an
arbitrary way. However, difficulties may occur, if the elements which can <b>lock</b> the
<b>relative motion</b> between two flanges are connected <b>rigidly</b>
together such that essentially the <b>same relative motion</b> can be locked.
The reason is
that the cut-torque in the locked phase is not uniquely defined if the
elements are locked at the same time instant (i.e., there does not exist a
unique solution) and some simulation systems may not be
able to handle this situation, since this leads to a singularity during
simulation. Currently, this type of problem can occur with the
Coulomb friction elements <b>BearingFriction, Clutch, Brake, LossyGear</b> when
the elements become stuck:
</p>
<p><IMG SRC=\"../Images/Rotational/driveConnections3.png\" ALT=\"driveConnections3\"></p>
<p>
In the figure above two typical situations are shown: In the upper part of
the figure, the series connection of rigidly attached BearingFriction and
Clutch components are shown. This does not hurt, because the BearingFriction
element can lock the relative motion between the element and the housing,
whereas the clutch element can lock the relative motion between the two
connected flanges. Contrary, the drive train in the lower part of the figure
may give rise to simulation problems, because the BearingFriction element
and the Brake element can lock the relative motion between a flange and
the housing and these flanges are rigidly connected together, i.e.,
essentially the same relative motion can be locked. These difficulties
may be solved by either introducing a compliance between these flanges
or by combining the BearingFriction and Brake element into
one component and resolving the ambiguity of the frictional torque in the
stuck mode. A tool may handle this situation also <b>automatically</b>,
by picking one solution of the infinitely many, e.g., the one where
the difference to the value of the previous time instant is as small
as possible.
</p>

</HTML>"));
        end RequirementsForSimulationTool;
        class Contact "Contact"
          annotation(Documentation(info = "<html>
<dl>
<dt><b>Library Officer</b>
<dd><a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a> <br>
    Deutsches Zentrum f&uuml;r Luft und Raumfahrt e.V. (DLR)<br>
    Institut f&uuml;r Robotik und Mechatronik (DLR-RM)<br>
    Abteilung Systemdynamik und Regelungstechnik<br>
    Postfach 1116<br>
    D-82230 Wessling<br>
    Germany<br>
    email: <A HREF=\"mailto:Martin.Otter@dlr.de\">Martin.Otter@dlr.de</A><br><br>
</dl>

<p>
<b>Contributors to this library:</b>
</p>

<ul>
<li> <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a> (DLR-RM)</li>
<li> Christian Schweiger (DLR-RM, until 2006).</li>
<li> <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
     Technical Consulting &amp; Electrical Engineering<br>
     A-3423 St.Andrae-Woerdern, Austria<br>
     email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a></li>
</ul>

</html>
"));
        end Contact;
      end UsersGuide;
      package Examples "Demonstration examples of the components of this package"
        extends Modelica.Icons.Library;
        annotation(Documentation(info = "<html>
<p>
This package contains example models to demonstrate the usage of the
Modelica.Mechanics.Rotational package. Open the models and
simulate them according to the provided description in the models.
</p>

</HTML>
"));
        model First "First example: simple drive train"
          import SI = Modelica.SIunits;
          extends Modelica.Icons.Example;
          parameter Modelica.SIunits.Torque amplitude = 10 "Amplitude of driving torque";
          parameter SI.Frequency freqHz = 5 "Frequency of driving torque";
          parameter SI.Inertia Jmotor(min = 0) = 0.1 "Motor inertia";
          parameter SI.Inertia Jload(min = 0) = 2 "Load inertia";
          parameter Real ratio = 10 "Gear ratio";
          parameter Real damping = 10 "Damping in bearing of gear";
          annotation(Documentation(info = "<html>
<p>The drive train consists of a motor inertia which is driven by
a sine-wave motor torque. Via a gearbox the rotational energy is
transmitted to a load inertia. Elasticity in the gearbox is modeled
by a spring element. A linear damper is used to model the
damping in the gearbox bearing.</p>
<p>Note, that a force component (like the damper of this example)
which is acting between a shaft and the housing has to be fixed
in the housing on one side via component Fixed.</p>
<p>Simulate for 1 second and plot the following variables:<br>
   angular velocities of inertias inertia2 and 3: inertia2.w, inertia3.w</p>

</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment);
          Rotational.Components.Fixed fixed annotation(Placement(transformation(extent = {{38, -48},{54, -32}}, rotation = 0)));
          Rotational.Sources.Torque torque(useSupport = true) annotation(Placement(transformation(extent = {{ -68, -8},{ -52,8}}, rotation = 0)));
          Rotational.Components.Inertia inertia1(J = Jmotor) annotation(Placement(transformation(extent = {{ -38, -8},{ -22,8}}, rotation = 0)));
          Rotational.Components.IdealGear idealGear(ratio = ratio, useSupport = true) annotation(Placement(transformation(extent = {{ -8, -8},{8,8}}, rotation = 0)));
          Rotational.Components.Inertia inertia2(J = 2, phi(fixed = true, start = 0), w(fixed = true)) annotation(Placement(transformation(extent = {{22, -8},{38,8}}, rotation = 0)));
          Rotational.Components.Spring spring(c = 10000.0, phi_rel(fixed = true)) annotation(Placement(transformation(extent = {{52, -8},{68,8}}, rotation = 0)));
          Rotational.Components.Inertia inertia3(J = Jload, w(fixed = true)) annotation(Placement(transformation(extent = {{82, -8},{98,8}}, rotation = 0)));
          Rotational.Components.Damper damper(d = damping) annotation(Placement(transformation(origin = {46, -22}, extent = {{ -8, -8},{8,8}}, rotation = 270)));
          Modelica.Blocks.Sources.Sine sine(amplitude = amplitude, freqHz = freqHz) annotation(Placement(transformation(extent = {{ -98, -8},{ -82,8}}, rotation = 0)));

        equation 
          connect(inertia1.flange_b,idealGear.flange_a) annotation(Line(points = {{ -22,0},{ -8,0}}, color = {0,0,0}));
          connect(idealGear.flange_b,inertia2.flange_a) annotation(Line(points = {{8,0},{22,0}}, color = {0,0,0}));
          connect(inertia2.flange_b,spring.flange_a) annotation(Line(points = {{38,0},{52,0}}, color = {0,0,0}));
          connect(spring.flange_b,inertia3.flange_a) annotation(Line(points = {{68,0},{82,0}}, color = {0,0,0}));
          connect(damper.flange_a,inertia2.flange_b) annotation(Line(points = {{46, -14},{46,0},{38,0}}, color = {0,0,0}));
          connect(damper.flange_b,fixed.flange) annotation(Line(points = {{46, -30},{46, -40}}, color = {0,0,0}));
          connect(sine.y,torque.tau) annotation(Line(points = {{ -81.2,0},{ -69.6,0}}, color = {0,0,127}));
          connect(torque.support,fixed.flange) annotation(Line(points = {{ -60, -8},{ -60, -40},{46, -40}}, color = {0,0,0}));
          connect(idealGear.support,fixed.flange) annotation(Line(points = {{0, -8},{0, -40},{46, -40}}, color = {0,0,0}));
          connect(torque.flange,inertia1.flange_a) annotation(Line(points = {{ -52,0},{ -38,0}}, color = {0,0,0}, smooth = Smooth.None));
        end First;
        model FirstGrounded "First example: simple drive train with grounded elments"
          import SI = Modelica.SIunits;
          extends Modelica.Icons.Example;
          parameter Modelica.SIunits.Torque amplitude = 10 "Amplitude of driving torque";
          parameter SI.Frequency freqHz = 5 "Frequency of driving torque";
          parameter SI.Inertia Jmotor(min = 0) = 0.1 "Motor inertia";
          parameter SI.Inertia Jload(min = 0) = 2 "Load inertia";
          parameter Real ratio = 10 "Gear ratio";
          parameter Real damping = 10 "Damping in bearing of gear";
          annotation(Documentation(info = "<html>
<p>The drive train consists of a motor inertia which is driven by
a sine-wave motor torque. Via a gearbox the rotational energy is
transmitted to a load inertia. Elasticity in the gearbox is modeled
by a spring element. A linear damper is used to model the
damping in the gearbox bearing.</p>
<p>Note, that a force component (like the damper of this example)
which is acting between a shaft and the housing has to be fixed
in the housing on one side via component Fixed.</p>
<p>Simulate for 1 second and plot the following variables:<br>
   angular velocities of inertias inertia2 and 3: inertia2.w, inertia3.w</p>

</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1));
          Rotational.Components.Fixed fixed annotation(Placement(transformation(extent = {{38, -48},{54, -32}}, rotation = 0)));
          Rotational.Sources.Torque torque(useSupport = false) annotation(Placement(transformation(extent = {{ -68, -8},{ -52,8}}, rotation = 0)));
          Rotational.Components.Inertia inertia1(J = Jmotor) annotation(Placement(transformation(extent = {{ -38, -8},{ -22,8}}, rotation = 0)));
          Rotational.Components.IdealGear idealGear(ratio = ratio, useSupport = false) annotation(Placement(transformation(extent = {{ -8, -8},{8,8}}, rotation = 0)));
          Rotational.Components.Inertia inertia2(J = 2, phi(fixed = true, start = 0), w(fixed = true)) annotation(Placement(transformation(extent = {{22, -8},{38,8}}, rotation = 0)));
          Rotational.Components.Spring spring(c = 10000.0, phi_rel(fixed = true)) annotation(Placement(transformation(extent = {{52, -8},{68,8}}, rotation = 0)));
          Rotational.Components.Inertia inertia3(J = Jload, w(fixed = true)) annotation(Placement(transformation(extent = {{82, -8},{98,8}}, rotation = 0)));
          Rotational.Components.Damper damper(d = damping) annotation(Placement(transformation(origin = {46, -22}, extent = {{ -8, -8},{8,8}}, rotation = 270)));
          Modelica.Blocks.Sources.Sine sine(amplitude = amplitude, freqHz = freqHz) annotation(Placement(transformation(extent = {{ -98, -8},{ -82,8}}, rotation = 0)));

        equation 
          connect(inertia1.flange_b,idealGear.flange_a) annotation(Line(points = {{ -22,0},{ -8,0}}, color = {0,0,0}));
          connect(idealGear.flange_b,inertia2.flange_a) annotation(Line(points = {{8,0},{22,0}}, color = {0,0,0}));
          connect(inertia2.flange_b,spring.flange_a) annotation(Line(points = {{38,0},{52,0}}, color = {0,0,0}));
          connect(spring.flange_b,inertia3.flange_a) annotation(Line(points = {{68,0},{82,0}}, color = {0,0,0}));
          connect(damper.flange_a,inertia2.flange_b) annotation(Line(points = {{46, -14},{46,0},{38,0}}, color = {0,0,0}));
          connect(damper.flange_b,fixed.flange) annotation(Line(points = {{46, -30},{46, -40}}, color = {0,0,0}));
          connect(sine.y,torque.tau) annotation(Line(points = {{ -81.2,0},{ -69.6,0}}, color = {0,0,127}));
          connect(torque.flange,inertia1.flange_a) annotation(Line(points = {{ -52,0},{ -38,0}}, color = {0,0,0}, smooth = Smooth.None));
        end FirstGrounded;
        model Friction "Drive train with clutch and brake"
          import Modelica.Constants.pi;
          import SI = Modelica.SIunits;
          extends Modelica.Icons.Example;
          parameter SI.Time startTime = 0.5 "Start time of step";
          output SI.Torque tMotor "driving torque of inertia3";
          output SI.Torque tClutch "friction torque of clutch";
          output SI.Torque tBrake "friction torque of brake";
          output SI.Torque tSpring "spring torque";
          annotation(Documentation(info = "<html>
<p>This drive train contains a frictional <b>clutch</b> and a <b>brake</b>.
Simulate the system for 1 second using the following initial
values (defined already in the model):</p>
<pre>   inertia1.w =  90 (or brake.w)
   inertia2.w =  90
   inertia3.w = 100
</pre>
<p>Plot the output signals</p>
<pre>   tMotor      Torque of motor
   tClutch     Torque in clutch
   tBrake      Torque in brake
   tSpring     Torque in spring
</pre>
<p>as well as the absolute angular velocities of the three inertia components
(inertia1.w, inertia2.w, inertia3.w).</p>

</HTML>"), Diagram(coordinateSystem(initialScale = 0.1, preserveAspectRatio = true, extent = {{ -180, -100},{120,100}}, grid = {2,2}), graphics), experiment(StopTime = 3));
          Rotational.Sources.Torque torque(useSupport = true) annotation(Placement(transformation(extent = {{ -90, -10},{ -70,10}}, rotation = 0)));
          Rotational.Components.Inertia inertia3(J = 1, phi(start = 0, fixed = true, displayUnit = "deg"), w(start = 100, fixed = true, displayUnit = "rad/s")) annotation(Placement(transformation(extent = {{ -60, -10},{ -40,10}}, rotation = 0)));
          Rotational.Components.Clutch clutch(fn_max = 160) annotation(Placement(transformation(extent = {{ -30, -10},{ -10,10}}, rotation = 0)));
          Rotational.Components.Inertia inertia2(J = 0.05, phi(start = 0, fixed = true), w(start = 90, fixed = true)) annotation(Placement(transformation(extent = {{0, -10},{20,10}}, rotation = 0)));
          Rotational.Components.SpringDamper spring(c = 160, d = 1) annotation(Placement(transformation(extent = {{30, -10},{50,10}}, rotation = 0)));
          Rotational.Components.Inertia inertia1(J = 1, phi(start = 0, fixed = true), w(start = 90, fixed = true)) annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Rotational.Components.Brake brake(fn_max = 1600, useSupport = true) annotation(Placement(transformation(extent = {{60, -10},{80,10}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant const(k = 1) annotation(Placement(transformation(origin = { -25,35}, extent = {{ -5, -5},{15,15}}, rotation = 270)));
          Modelica.Blocks.Sources.Step step(startTime = startTime) annotation(Placement(transformation(origin = {65,35}, extent = {{ -5, -5},{15,15}}, rotation = 270)));
          Modelica.Blocks.Sources.Step step2(height =  -1, offset = 1, startTime = startTime) annotation(Placement(transformation(extent = {{ -160, -30},{ -140, -10}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sine(amplitude = 200, freqHz = 50 / pi) annotation(Placement(transformation(extent = {{ -160,10},{ -140,30}}, rotation = 0)));
          Modelica.Blocks.Math.Product product annotation(Placement(transformation(extent = {{ -120, -10},{ -100,10}}, rotation = 0)));
          Rotational.Components.Fixed fixed annotation(Placement(transformation(extent = {{ -10, -30},{10, -10}}, rotation = 0)));

        equation 
          tMotor = torque.tau;
          tClutch = clutch.tau;
          tBrake = brake.tau;
          tSpring = spring.tau;
          connect(torque.flange,inertia3.flange_a) annotation(Line(points = {{ -70,0},{ -70,0},{ -60,0}}, color = {0,0,0}));
          connect(inertia3.flange_b,clutch.flange_a) annotation(Line(points = {{ -40,0},{ -30,0}}, color = {0,0,0}));
          connect(clutch.flange_b,inertia2.flange_a) annotation(Line(points = {{ -10,0},{0,0}}, color = {0,0,0}));
          connect(inertia2.flange_b,spring.flange_a) annotation(Line(points = {{20,0},{30,0}}, color = {0,0,0}));
          connect(spring.flange_b,brake.flange_a) annotation(Line(points = {{50,0},{60,0}}, color = {0,0,0}));
          connect(brake.flange_b,inertia1.flange_a) annotation(Line(points = {{80,0},{80,0},{90,0}}, color = {0,0,0}));
          connect(sine.y,product.u1) annotation(Line(points = {{ -139,20},{ -130,20},{ -130,6},{ -122,6}}, color = {0,0,127}));
          connect(step2.y,product.u2) annotation(Line(points = {{ -139, -20},{ -130, -20},{ -130, -6},{ -126, -6},{ -122, -6}}, color = {0,0,127}));
          connect(product.y,torque.tau) annotation(Line(points = {{ -99,0},{ -99,0},{ -92,0}}, color = {0,0,127}));
          connect(const.y,clutch.f_normalized) annotation(Line(points = {{ -20,19},{ -20,12.75},{ -20,11}}, color = {0,0,127}));
          connect(step.y,brake.f_normalized) annotation(Line(points = {{70,19},{70,16},{70,11}}, color = {0,0,127}));
          connect(torque.support,fixed.flange) annotation(Line(points = {{ -80, -10},{ -80, -20},{0, -20}}, color = {0,0,0}));
          connect(brake.support,fixed.flange) annotation(Line(points = {{70, -10},{70, -20},{0, -20}}, color = {0,0,0}));
        end Friction;
        model CoupledClutches "Drive train with 3 dynamically coupled clutches"
          import SI = Modelica.SIunits;
          extends Modelica.Icons.Example;
          parameter SI.Frequency freqHz = 0.2 "frequency of sine function to invoke clutch1";
          parameter SI.Time T2 = 0.4 "time when clutch2 is invoked";
          parameter SI.Time T3 = 0.9 "time when clutch3 is invoked";
          annotation(Documentation(info = "<html>
<p>This example demonstrates how variable structure
drive trains are handeled. The drive train consists
of 4 inertias and 3 clutches, where the clutches
are controlled by input signals. The system has
2^3=8 different configurations and 3^3 = 27
different states (every clutch may be in forward
sliding, backward sliding or locked mode when the
relative angular velocity is zero). By invoking the
clutches at different time instances, the switching
of the configurations can be studied.</p>
<p>Simulate the system for 1.2 seconds with the
following initial values:<br>
J1.w = 10.</p>
<p>Plot the following variables:<br>
angular velocities of inertias (J1.w, J2.w, J3.w,
J4.w), frictional torques of clutches (clutchX.tau),
frictional mode of clutches (clutchX.mode) where
mode = -1/0/+1 means backward sliding,
locked, forward sliding.</p>

</HTML>"), __Dymola_Commands(file = "Scripts/__Dymola_CoupledClutches.mos" "Simulate and Plot"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -140, -80},{140,80}}, grid = {2,2}), graphics), experiment(StopTime = 1.5));
          Rotational.Components.Inertia J1(J = 1, phi(fixed = true, start = 0), w(start = 10, fixed = true)) annotation(Placement(transformation(extent = {{ -70, -10},{ -50,10}}, rotation = 0)));
          Rotational.Sources.Torque torque(useSupport = true) annotation(Placement(transformation(extent = {{ -100, -10},{ -80,10}}, rotation = 0)));
          Rotational.Components.Clutch clutch1(peak = 1.1, fn_max = 20) annotation(Placement(transformation(extent = {{ -40, -10},{ -20,10}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sin1(amplitude = 10, freqHz = 5) annotation(Placement(transformation(extent = {{ -130, -10},{ -110,10}}, rotation = 0)));
          Modelica.Blocks.Sources.Step step1(startTime = T2) annotation(Placement(transformation(origin = {25,35}, extent = {{ -5, -5},{15,15}}, rotation = 270)));
          Rotational.Components.Inertia J2(J = 1, phi(fixed = true, start = 0), w(fixed = true)) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          Rotational.Components.Clutch clutch2(peak = 1.1, fn_max = 20) annotation(Placement(transformation(extent = {{20, -10},{40,10}}, rotation = 0)));
          Rotational.Components.Inertia J3(J = 1, phi(fixed = true, start = 0), w(fixed = true)) annotation(Placement(transformation(extent = {{50, -10},{70,10}}, rotation = 0)));
          Rotational.Components.Clutch clutch3(peak = 1.1, fn_max = 20) annotation(Placement(transformation(extent = {{80, -10},{100,10}}, rotation = 0)));
          Rotational.Components.Inertia J4(J = 1, phi(fixed = true, start = 0), w(fixed = true)) annotation(Placement(transformation(extent = {{110, -10},{130,10}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine sin2(amplitude = 1, freqHz = freqHz, phase = 1.57) annotation(Placement(transformation(origin = { -35,35}, extent = {{ -5, -5},{15,15}}, rotation = 270)));
          Modelica.Blocks.Sources.Step step2(startTime = T3) annotation(Placement(transformation(origin = {85,35}, extent = {{ -5, -5},{15,15}}, rotation = 270)));
          Rotational.Components.Fixed fixed annotation(Placement(transformation(extent = {{ -100, -30},{ -80, -10}}, rotation = 0)));

        equation 
          connect(torque.flange,J1.flange_a) annotation(Line(points = {{ -80,0},{ -70,0}}, color = {0,0,0}));
          connect(J1.flange_b,clutch1.flange_a) annotation(Line(points = {{ -50,0},{ -40,0}}, color = {0,0,0}));
          connect(clutch1.flange_b,J2.flange_a) annotation(Line(points = {{ -20,0},{ -10,0}}, color = {0,0,0}));
          connect(J2.flange_b,clutch2.flange_a) annotation(Line(points = {{10,0},{10,0},{20,0}}, color = {0,0,0}));
          connect(clutch2.flange_b,J3.flange_a) annotation(Line(points = {{40,0},{50,0}}, color = {0,0,0}));
          connect(J3.flange_b,clutch3.flange_a) annotation(Line(points = {{70,0},{80,0}}, color = {0,0,0}));
          connect(clutch3.flange_b,J4.flange_a) annotation(Line(points = {{100,0},{110,0}}, color = {0,0,0}));
          connect(sin1.y,torque.tau) annotation(Line(points = {{ -109,0},{ -102,0}}, color = {0,0,127}));
          connect(sin2.y,clutch1.f_normalized) annotation(Line(points = {{ -30,19},{ -30,19},{ -30,11}}, color = {0,0,127}));
          connect(step1.y,clutch2.f_normalized) annotation(Line(points = {{30,19},{30,19},{30,10},{30,11}}, color = {0,0,127}));
          connect(step2.y,clutch3.f_normalized) annotation(Line(points = {{90,19},{90,19},{90,11}}, color = {0,0,127}));
          connect(fixed.flange,torque.support) annotation(Line(points = {{ -90, -20},{ -90, -11},{ -90, -10}}, color = {0,0,0}));
        end CoupledClutches;
        model LossyGearDemo1 "Example to show that gear efficiency may lead to stuck motion"
          import SI = Modelica.SIunits;
          extends Modelica.Icons.Example;
          SI.Power PowerLoss "power lost in the gear";
          annotation(Documentation(info = "<html>
<p>
This model contains two inertias which are connected by an ideal
gear where the friction between the teeth of the gear is modeled in
a physical meaningful way (friction may lead to stuck mode which
locks the motion of the gear). The friction is defined by an
efficiency factor (= 0.5) for forward and backward driving condition leading
to a torque dependent friction loss. Simulate for about 0.5 seconds.
The friction in the gear will take all modes
(forward and backward rolling, as well as stuck).
</p>
<p>
You may plot:
</p>
<pre>
Inertia1.w,
Inertia2.w : angular velocities of inertias
powerLoss  : power lost in the gear
gear.mode  :  1 = forward rolling
              0 = stuck (w=0)
             -1 = backward rolling
</pre>
</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -80},{100,80}}), graphics), experiment(StopTime = 0.5));
          Rotational.Components.LossyGear gear(ratio = 2, lossTable = [0,0.5,0.5,0,0], useSupport = true) annotation(Placement(transformation(extent = {{ -10,0},{10,20}}, rotation = 0)));
          Rotational.Components.Inertia Inertia1(J = 1) annotation(Placement(transformation(extent = {{ -40,0},{ -20,20}}, rotation = 0)));
          Rotational.Components.Inertia Inertia2(J = 1.5, phi(fixed = true, start = 0), w(fixed = true)) annotation(Placement(transformation(extent = {{20,0},{40,20}}, rotation = 0)));
          Rotational.Sources.Torque torque1(useSupport = true) annotation(Placement(transformation(extent = {{ -70,0},{ -50,20}}, rotation = 0)));
          Rotational.Sources.Torque torque2(useSupport = true) annotation(Placement(transformation(extent = {{70,0},{50,20}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine DriveSine(amplitude = 10, freqHz = 1) annotation(Placement(transformation(extent = {{ -100,0},{ -80,20}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp load(height = 5, duration = 2, offset =  -10) annotation(Placement(transformation(extent = {{100,0},{80,20}}, rotation = 0)));
          Rotational.Components.Fixed fixed annotation(Placement(transformation(extent = {{ -10, -30},{10, -10}}, rotation = 0)));

        equation 
          connect(Inertia1.flange_b,gear.flange_a) annotation(Line(points = {{ -20,10},{ -10,10}}, color = {0,0,0}));
          connect(gear.flange_b,Inertia2.flange_a) annotation(Line(points = {{10,10},{20,10}}, color = {0,0,0}));
          connect(torque1.flange,Inertia1.flange_a) annotation(Line(points = {{ -50,10},{ -40,10}}, color = {0,0,0}));
          connect(torque2.flange,Inertia2.flange_b) annotation(Line(points = {{50,10},{40,10}}, color = {0,0,0}));
          PowerLoss = gear.flange_a.tau * der(gear.flange_a.phi) + gear.flange_b.tau * der(gear.flange_b.phi);
          connect(DriveSine.y,torque1.tau) annotation(Line(points = {{ -79,10},{ -72,10}}, color = {0,0,127}));
          connect(load.y,torque2.tau) annotation(Line(points = {{79,10},{72,10}}, color = {0,0,127}));
          connect(fixed.flange,gear.support) annotation(Line(points = {{0, -20},{0,0}}, color = {0,0,0}));
          connect(fixed.flange,torque1.support) annotation(Line(points = {{0, -20},{ -60, -20},{ -60,0}}, color = {0,0,0}));
          connect(fixed.flange,torque2.support) annotation(Line(points = {{0, -20},{60, -20},{60,0}}, color = {0,0,0}));
        end LossyGearDemo1;
        model LossyGearDemo2 "Example to show combination of LossyGear and BearingFriction"
          import SI = Modelica.SIunits;
          extends Modelica.Icons.Example;
          SI.Power PowerLoss "power lost in the gear";
          annotation(Documentation(info = "<html>
<p>
This model contains bearing friction and gear friction (= efficiency).
If both friction models are stuck, there is no unique solution.
Still a reliable Modelica simulator, such as Dymola, should
be able to handle this situation.
</p>
<p>
Simulate for about 0.5 seconds. The friction elements are
in all modes (forward and backward rolling, as well as stuck).
</p>
<p>
You may plot:
</p>
<pre>
Inertia1.w,
Inertia2.w          : angular velocities of inertias
powerLoss           : power lost in the gear
bearingFriction.mode:  1 = forward rolling
                       0 = stuck (w=0)
                      -1 = backward rolling
gear.mode           :  1 = forward rolling
                       0 = stuck (w=0)
                      -1 = backward rolling
</pre>
<p>Note: This combination of LossyGear and BearingFriction is not recommended to use,
as component LossyGear includes the functionality of component BearingFriction
(only <i>peak</i> not supported).</p>
</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -140, -80},{100,80}}), graphics), experiment(StopTime = 0.5));
          Rotational.Components.LossyGear gear(ratio = 2, lossTable = [0,0.5,0.5,0,0], useSupport = true) annotation(Placement(transformation(extent = {{ -20,0},{0,20}}, rotation = 0)));
          Rotational.Components.Inertia Inertia1(J = 1) annotation(Placement(transformation(extent = {{ -50,0},{ -30,20}}, rotation = 0)));
          Rotational.Components.Inertia Inertia2(J = 1.5, phi(fixed = true, start = 0), w(fixed = true)) annotation(Placement(transformation(extent = {{10,0},{30,20}}, rotation = 0)));
          Rotational.Sources.Torque torque1(useSupport = true) annotation(Placement(transformation(extent = {{ -110,0},{ -90,20}}, rotation = 0)));
          Rotational.Sources.Torque torque2(useSupport = true) annotation(Placement(transformation(extent = {{60,0},{40,20}}, rotation = 0)));
          Modelica.Blocks.Sources.Sine DriveSine(amplitude = 10, freqHz = 1) annotation(Placement(transformation(extent = {{ -140,0},{ -120,20}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp load(height = 5, duration = 2, offset =  -10) annotation(Placement(transformation(extent = {{90,0},{70,20}}, rotation = 0)));
          Rotational.Components.BearingFriction bearingFriction(tau_pos = [0,0.5;1,1], useSupport = true) annotation(Placement(transformation(extent = {{ -80,0},{ -60,20}}, rotation = 0)));
          Rotational.Components.Fixed fixed annotation(Placement(transformation(extent = {{ -20, -30},{0, -10}}, rotation = 0)));

        equation 
          PowerLoss = gear.flange_a.tau * der(gear.flange_a.phi) + gear.flange_b.tau * der(gear.flange_b.phi);
          connect(torque2.flange,Inertia2.flange_b) annotation(Line(points = {{40,10},{30,10}}, color = {0,0,0}));
          connect(Inertia2.flange_a,gear.flange_b) annotation(Line(points = {{10,10},{0,10}}, color = {0,0,0}));
          connect(gear.flange_a,Inertia1.flange_b) annotation(Line(points = {{ -20,10},{ -30,10}}, color = {0,0,0}));
          connect(Inertia1.flange_a,bearingFriction.flange_b) annotation(Line(points = {{ -50,10},{ -60,10}}, color = {0,0,0}));
          connect(bearingFriction.flange_a,torque1.flange) annotation(Line(points = {{ -80,10},{ -80,10},{ -90,10}}, color = {0,0,0}));
          connect(DriveSine.y,torque1.tau) annotation(Line(points = {{ -119,10},{ -119,10},{ -112,10}}, color = {0,0,127}));
          connect(load.y,torque2.tau) annotation(Line(points = {{69,10},{62,10}}, color = {0,0,127}));
          connect(gear.support,fixed.flange) annotation(Line(points = {{ -10,0},{ -10, -20}}, color = {0,0,0}));
          connect(fixed.flange,torque2.support) annotation(Line(points = {{ -10, -20},{50, -20},{50,0}}, color = {0,0,0}));
          connect(fixed.flange,bearingFriction.support) annotation(Line(points = {{ -10, -20},{ -70, -20},{ -70,0}}, color = {0,0,0}));
          connect(torque1.support,fixed.flange) annotation(Line(points = {{ -100,0},{ -100, -20},{ -10, -20}}, color = {0,0,0}));
        end LossyGearDemo2;
        model ElasticBearing "Example to show possible usage of support flange"
          extends Modelica.Icons.Example;
          Rotational.Components.Inertia shaft(phi(fixed = true, start = 0), w(fixed = true), J = 1) annotation(Placement(transformation(extent = {{ -20,40},{0,60}}, rotation = 0)));
          Rotational.Components.Inertia load(J = 50, w(fixed = true)) annotation(Placement(transformation(extent = {{70,40},{90,60}}, rotation = 0)));
          Rotational.Components.Spring spring(c = 1000.0, phi_rel(fixed = true)) annotation(Placement(transformation(extent = {{40,40},{60,60}}, rotation = 0)));
          Rotational.Components.Fixed fixed annotation(Placement(transformation(extent = {{10, -70},{30, -50}}, rotation = 0)));
          Rotational.Components.SpringDamper springDamper(c = 100000.0, d = 5, phi_rel(fixed = true), w_rel(fixed = true)) annotation(Placement(transformation(origin = {20, -30}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Rotational.Sources.Torque torque(useSupport = true) annotation(Placement(transformation(extent = {{ -50,40},{ -30,60}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp ramp(duration = 5, height = 100) annotation(Placement(transformation(extent = {{ -90,40},{ -70,60}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
This model demonstrates the usage of the bearing flange.
The gearbox is not connected rigidly to the ground, but by
a spring-damper-system. This allows examination of the gearbox
housing dynamics.</p>
<p>
Simulate for about 10 seconds and plot the angular velocities of the inertias <tt>housing.w</tt>,
<tt>shaft.w</tt> and <tt>load.w</tt>.</p>
</html>
"));
          Rotational.Components.IdealGear idealGear(ratio = 3, useSupport = true) annotation(Placement(transformation(extent = {{10,40},{30,60}}, rotation = 0)));
          Rotational.Components.Inertia housing(J = 5) annotation(Placement(transformation(origin = {20,10}, extent = {{ -10, -10},{10,10}}, rotation = 90)));

        equation 
          connect(torque.flange,shaft.flange_a) annotation(Line(points = {{ -30,50},{ -20,50}}, color = {0,0,0}));
          connect(spring.flange_b,load.flange_a) annotation(Line(points = {{60,50},{70,50}}, color = {0,0,0}));
          connect(springDamper.flange_a,fixed.flange) annotation(Line(points = {{20, -40},{20, -56},{20, -60}}, color = {0,0,0}));
          connect(shaft.flange_b,idealGear.flange_a) annotation(Line(points = {{0,50},{10,50}}, color = {0,0,0}));
          connect(idealGear.flange_b,spring.flange_a) annotation(Line(points = {{30,50},{40,50}}, color = {0,0,0}));
          connect(idealGear.support,housing.flange_b) annotation(Line(points = {{20,40},{20,20}}, color = {0,0,0}));
          connect(housing.flange_a,springDamper.flange_b) annotation(Line(points = {{20,0},{20, -20}}, color = {0,0,0}));
          connect(ramp.y,torque.tau) annotation(Line(points = {{ -69,50},{ -69,50},{ -52,50}}, color = {0,0,127}));
          connect(fixed.flange,torque.support) annotation(Line(points = {{20, -60},{ -40, -60},{ -40,40}}, color = {0,0,0}));
        end ElasticBearing;
        model Backlash "Example to demonstrate backlash"
          extends Modelica.Icons.Example;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
This model demonstrates the effect of a backlash on eigenfrequency, and
also that the damping torque does not lead to unphysical pulling torques
(since the ElastoBacklash model takes care of it).
</p>
</html>
"));
          Rotational.Components.Fixed fixed1 annotation(Placement(transformation(extent = {{ -50,50},{ -30,70}})));
          Rotational.Components.SpringDamper springDamper(c = 20000.0, d = 50, phi_nominal = 1) annotation(Placement(transformation(extent = {{ -20,50},{0,70}})));
          Rotational.Components.Inertia inertia1(J = 5, w(fixed = true, start = 0), phi(fixed = true, displayUnit = "deg", start = 1.5707963267949)) annotation(Placement(transformation(extent = {{20,50},{40,70}})));
          Rotational.Components.Fixed fixed2 annotation(Placement(transformation(extent = {{ -50, -50},{ -30, -30}})));
          Rotational.Components.ElastoBacklash elastoBacklash(c = 20000.0, d = 50, b(displayUnit = "deg") = 0.785398163397448, phi_nominal = 1) annotation(Placement(transformation(extent = {{ -20, -50},{0, -30}})));
          Rotational.Components.Inertia inertia2(J = 5, w(fixed = true, start = 0), phi(fixed = true, start = 1.5707963267949, displayUnit = "deg")) annotation(Placement(transformation(extent = {{20, -50},{40, -30}})));

        equation 
          connect(springDamper.flange_b,inertia1.flange_a) annotation(Line(points = {{0,60},{20,60}}, color = {0,0,0}, smooth = Smooth.None));
          connect(elastoBacklash.flange_b,inertia2.flange_a) annotation(Line(points = {{0, -40},{20, -40}}, color = {0,0,0}, smooth = Smooth.None));
          connect(fixed1.flange,springDamper.flange_a) annotation(Line(points = {{ -40,60},{ -20,60}}, color = {0,0,0}, smooth = Smooth.None));
          connect(fixed2.flange,elastoBacklash.flange_a) annotation(Line(points = {{ -40, -40},{ -20, -40}}, color = {0,0,0}, smooth = Smooth.None));
        end Backlash;
        model RollingWheel "Demonstrate coupling Rotational - Translational"
          extends Modelica.Icons.Example;
          Rotational.Components.IdealRollingWheel idealRollingWheel(radius = 1) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}})));
          Rotational.Components.Inertia inertia(J = 1) annotation(Placement(transformation(extent = {{ -40, -10},{ -20,10}})));
          Rotational.Sources.TorqueStep torqueStep(stepTorque = 10, offsetTorque = 0, startTime = 0.1, useSupport = false) annotation(Placement(transformation(extent = {{ -70, -10},{ -50,10}})));
          Translational.Components.Mass mass(L = 0, m = 1) annotation(Placement(transformation(extent = {{20, -10},{40,10}})));
          Translational.Sources.QuadraticSpeedDependentForce quadraticSpeedDependentForce(f_nominal =  -10, ForceDirection = false, v_nominal = 5) annotation(Placement(transformation(extent = {{72, -10},{52,10}})));

        equation 
          annotation(Documentation(info = "<html>
<p>
This model demonstrates the coupling between rotational and translational components:<br>
A torque (step) accelerates both the inertia (of the wheel) and the mass (of the vehicle).<br>
Du to a speed dependent force (like driving resistance), we find an eqilibrium at 5 m/s after approx. 5 s.
</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 5));
          connect(torqueStep.flange,inertia.flange_a) annotation(Line(points = {{ -50,0},{ -40,0}}, color = {0,0,0}, smooth = Smooth.None));
          connect(inertia.flange_b,idealRollingWheel.flangeR) annotation(Line(points = {{ -20,0},{ -10,0}}, color = {0,0,0}, smooth = Smooth.None));
          connect(idealRollingWheel.flangeT,mass.flange_a) annotation(Line(points = {{10,0},{20,0}}, color = {0,127,0}, smooth = Smooth.None));
          connect(quadraticSpeedDependentForce.flange,mass.flange_b) annotation(Line(points = {{52,0},{40,0}}, color = {0,127,0}, smooth = Smooth.None));
        end RollingWheel;
      end Examples;
      package Components "Components for 1D rotational mechanical drive trains"
        extends Modelica.Icons.Library2;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -58,8},{42, -92}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -100, -32},{ -58, -52}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{42, -32},{80, -52}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192})}), Documentation(info = "<html>
<p>
This package contains basic components 1D mechanical rotational drive trains.
</p>
</html>"));
        model Fixed "Flange fixed in housing at a given angle"
          parameter SI.Angle phi0 = 0 "Fixed offset angle of housing";
          Interfaces.Flange_b flange "(right) flange fixed in housing" annotation(Placement(transformation(extent = {{10, -10},{ -10,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -150, -90},{150, -130}}, lineColor = {0,0,255}, textString = "%name"),Line(points = {{ -80, -40},{80, -40}}, color = {0,0,0}),Line(points = {{80, -40},{40, -80}}, color = {0,0,0}),Line(points = {{40, -40},{0, -80}}, color = {0,0,0}),Line(points = {{0, -40},{ -40, -80}}, color = {0,0,0}),Line(points = {{ -40, -40},{ -80, -80}}, color = {0,0,0}),Line(points = {{0, -40},{0, -10}}, color = {0,0,0})}), Documentation(info = "<html>
<p>
The <b>flange</b> of a 1D rotational mechanical system is <b>fixed</b>
at an angle phi0 in the <b>housing</b>. May be used:
</p>
<ul>
<li> to connect a compliant element, such as a spring or a damper,
     between an inertia or gearbox component and the housing.
<li> to fix a rigid element, such as an inertia, with a specific
     angle to the housing.
</ul>

</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -80, -40},{80, -40}}, color = {0,0,0}),Line(points = {{80, -40},{40, -80}}, color = {0,0,0}),Line(points = {{40, -40},{0, -80}}, color = {0,0,0}),Line(points = {{0, -40},{ -40, -80}}, color = {0,0,0}),Line(points = {{ -40, -40},{ -80, -80}}, color = {0,0,0}),Line(points = {{0, -40},{0, -4}}, color = {0,0,0})}));

        equation 
          flange.phi = phi0;
        end Fixed;
        model Inertia "1D-rotational component with inertia"
          import SI = Modelica.SIunits;
          Rotational.Interfaces.Flange_a flange_a "Left flange of shaft" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Rotational.Interfaces.Flange_b flange_b "Right flange of shaft" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          parameter SI.Inertia J(min = 0, start = 1) "Moment of inertia";
          parameter StateSelect stateSelect = StateSelect.default "Priority to use phi and w as states" annotation(HideResult = true, Dialog(tab = "Advanced"));
          SI.Angle phi(stateSelect = stateSelect) "Absolute rotation angle of component" annotation(Dialog(group = "Initialization", __Dymola_initialDialog = true));
          SI.AngularVelocity w(stateSelect = stateSelect) "Absolute angular velocity of component (= der(phi))" annotation(Dialog(group = "Initialization", __Dymola_initialDialog = true));
          SI.AngularAcceleration a "Absolute angular acceleration of component (= der(w))" annotation(Dialog(group = "Initialization", __Dymola_initialDialog = true));
          annotation(Documentation(info = "<html>
<p>
Rotational component with <b>inertia</b> and two rigidly connected flanges.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,10},{ -50, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{50,10},{100, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Line(points = {{ -80, -25},{ -60, -25}}, color = {0,0,0}),Line(points = {{60, -25},{80, -25}}, color = {0,0,0}),Line(points = {{ -70, -25},{ -70, -70}}, color = {0,0,0}),Line(points = {{70, -25},{70, -70}}, color = {0,0,0}),Line(points = {{ -80,25},{ -60,25}}, color = {0,0,0}),Line(points = {{60,25},{80,25}}, color = {0,0,0}),Line(points = {{ -70,45},{ -70,25}}, color = {0,0,0}),Line(points = {{70,45},{70,25}}, color = {0,0,0}),Line(points = {{ -70, -70},{70, -70}}, color = {0,0,0}),Rectangle(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Text(extent = {{ -150,100},{150,60}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -150, -80},{150, -120}}, lineColor = {0,0,0}, textString = "J=%J")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          phi = flange_a.phi;
          phi = flange_b.phi;
          w = der(phi);
          a = der(w);
          J * a = flange_a.tau + flange_b.tau;
        end Inertia;
        model Disc "1-dim. rotational rigid component without inertia, where right flange is rotated by a fixed angle with respect to left flange"
          import SI = Modelica.SIunits;
          extends Rotational.Interfaces.PartialTwoFlanges;
          parameter SI.Angle deltaPhi = 0 "Fixed rotation of left flange with respect to right flange (= flange_b.phi - flange_a.phi)";
          SI.Angle phi "Absolute rotation angle of component";
          annotation(Documentation(info = "<html>
<p>
Rotational component with two rigidly connected flanges without <b>inertia</b>.
The right flange is rotated by the fixed angle \"deltaPhi\" with respect to the left
flange.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,10},{99, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -10,50},{10, -50}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Text(extent = {{ -150,100},{150,60}}, textString = "%name", lineColor = {0,0,255}),Rectangle(extent = {{ -30,50},{ -10,10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{10, -10},{30, -50}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Text(extent = {{ -160, -62},{160, -87}}, lineColor = {0,0,0}, textString = "deltaPhi = %deltaPhi")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          flange_a.phi = phi - deltaPhi / 2;
          flange_b.phi = phi + deltaPhi / 2;
          0 = flange_a.tau + flange_b.tau;
        end Disc;
        model Spring "Linear 1D rotational spring"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialCompliant;
          parameter SI.RotationalSpringConstant c(final min = 0, start = 100000.0) "Spring constant";
          parameter SI.Angle phi_rel0 = 0 "Unstretched spring angle";
          annotation(Documentation(info = "<html>
<p>
A <b>linear 1D rotational spring</b>. The component can be connected either
between two inertias/gears to describe the shaft elasticity, or between
a inertia/gear and the housing (component Fixed), to describe
a coupling of the element with the housing via a spring.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -150,80},{150,40}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -150, -40},{150, -80}}, lineColor = {0,0,0}, textString = "c=%c"),Line(points = {{ -100,0},{ -58,0},{ -43, -30},{ -13,30},{17, -30},{47,30},{62,0},{100,0}}, color = {0,0,0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -68,0},{ -68,65}}, color = {128,128,128}),Line(points = {{72,0},{72,65}}, color = {128,128,128}),Line(points = {{ -68,60},{72,60}}, color = {128,128,128}),Polygon(points = {{62,63},{72,60},{62,57},{62,63}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{ -22,62},{18,87}}, lineColor = {0,0,255}, textString = "phi_rel"),Line(points = {{ -96,0},{ -60,0},{ -42, -32},{ -12,30},{18, -30},{48,28},{62,0},{96,0}}, color = {0,0,255})}));

        equation 
          tau = c * (phi_rel - phi_rel0);
        end Spring;
        model Damper "Linear 1D rotational damper"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialCompliantWithRelativeStates;
          parameter SI.RotationalDampingConstant d(final min = 0, start = 0) "Damping constant";
          annotation(Documentation(info = "<html>
<p>
<b>Linear, velocity dependent damper</b> element. It can be either connected
between an inertia or gear and the housing (component Fixed), or
between two inertia/gear elements.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -90,0},{ -60,0}}, color = {0,0,0}),Line(points = {{ -60, -30},{ -60,30}}, color = {0,0,0}),Line(points = {{ -60, -30},{60, -30}}, color = {0,0,0}),Line(points = {{ -60,30},{60,30}}, color = {0,0,0}),Rectangle(extent = {{ -60,30},{30, -30}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{30,0},{90,0}}, color = {0,0,0}),Text(extent = {{ -150,80},{150,40}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -150, -50},{150, -90}}, lineColor = {0,0,0}, textString = "d=%d")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -96,0},{ -60,0}}, color = {0,0,0}),Line(points = {{ -60, -30},{ -60,30}}, color = {0,0,0}),Line(points = {{ -60, -30},{60, -30}}, color = {0,0,0}),Line(points = {{ -60,30},{60,30}}, color = {0,0,0}),Rectangle(extent = {{ -60,30},{30, -30}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{30,0},{96,0}}, color = {0,0,0}),Line(points = {{ -68,0},{ -68,65}}, color = {128,128,128}),Text(extent = {{ -22,62},{18,87}}, lineColor = {0,0,255}, textString = "phi_rel"),Line(points = {{ -68,60},{72,60}}, color = {128,128,128}),Line(points = {{72,0},{72,65}}, color = {128,128,128}),Polygon(points = {{62,63},{72,60},{62,57},{62,63}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid)}));

        equation 
          tau = d * w_rel;
        end Damper;
        model SpringDamper "Linear 1D rotational spring and damper in parallel"
          import SI = Modelica.SIunits;
          parameter SI.RotationalSpringConstant c(final min = 0, start = 100000.0) "Spring constant";
          parameter SI.RotationalDampingConstant d(final min = 0, start = 0) "Damping constant";
          parameter SI.Angle phi_rel0 = 0 "Unstretched spring angle";
          extends Modelica.Mechanics.Rotational.Interfaces.PartialCompliantWithRelativeStates;
          annotation(Documentation(info = "<html>
<p>
A <b>spring</b> and <b>damper</b> element <b>connected in parallel</b>.
The component can be
connected either between two inertias/gears to describe the shaft elasticity
and damping, or between an inertia/gear and the housing (component Fixed),
to describe a coupling of the element with the housing via a spring/damper.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80,40},{ -60,40},{ -45,10},{ -15,70},{15,10},{45,70},{60,40},{80,40}}, color = {0,0,0}),Line(points = {{ -80,40},{ -80, -40}}, color = {0,0,0}),Line(points = {{ -80, -40},{ -50, -40}}, color = {0,0,0}),Rectangle(extent = {{ -50, -10},{40, -70}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -50, -10},{70, -10}}, color = {0,0,0}),Line(points = {{ -50, -70},{70, -70}}, color = {0,0,0}),Line(points = {{40, -40},{80, -40}}, color = {0,0,0}),Line(points = {{80,40},{80, -40}}, color = {0,0,0}),Line(points = {{ -90,0},{ -80,0}}, color = {0,0,0}),Line(points = {{80,0},{90,0}}, color = {0,0,0}),Text(extent = {{ -150, -144},{150, -104}}, lineColor = {0,0,0}, textString = "d=%d"),Text(extent = {{ -190,110},{190,70}}, lineColor = {0,0,255}, textString = "%name"),Text(extent = {{ -150, -108},{150, -68}}, lineColor = {0,0,0}, textString = "c=%c")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80,32},{ -58,32},{ -43,2},{ -13,62},{17,2},{47,62},{62,32},{80,32}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -68,32},{ -68,97}}, color = {128,128,128}),Line(points = {{72,32},{72,97}}, color = {128,128,128}),Line(points = {{ -68,92},{72,92}}, color = {128,128,128}),Polygon(points = {{62,95},{72,92},{62,89},{62,95}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{ -20,72},{20,97}}, lineColor = {0,0,255}, textString = "phi_rel"),Rectangle(extent = {{ -50, -20},{40, -80}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -50, -80},{68, -80}}, color = {0,0,0}),Line(points = {{ -50, -20},{68, -20}}, color = {0,0,0}),Line(points = {{40, -50},{80, -50}}, color = {0,0,0}),Line(points = {{ -80, -50},{ -50, -50}}, color = {0,0,0}),Line(points = {{ -80,32},{ -80, -50}}, color = {0,0,0}),Line(points = {{80,32},{80, -50}}, color = {0,0,0}),Line(points = {{ -96,0},{ -80,0}}, color = {0,0,0}),Line(points = {{96,0},{80,0}}, color = {0,0,0})}));

        equation 
          tau = c * (phi_rel - phi_rel0) + d * w_rel;
        end SpringDamper;
        model ElastoBacklash "Backlash connected in series to linear spring and damper (backlash is modeled with elasticity)"
          import SI = Modelica.SIunits;
          parameter SI.RotationalSpringConstant c(final min = Modelica.Constants.small, start = 100000.0) "Spring constant (c > 0 required)";
          parameter SI.RotationalDampingConstant d(final min = 0, start = 0) "Damping constant";
          parameter SI.Angle b(final min = 0) = 0 "Total backlash";
          parameter SI.Angle phi_rel0 = 0 "Unstretched spring angle";
          extends Modelica.Mechanics.Rotational.Interfaces.PartialCompliantWithRelativeStates;
        protected 
          final parameter SI.Angle bMax = b / 2 "Backlash in range bMin <= phi_rel - phi_rel0 <= bMax";
          final parameter SI.Angle bMin =  -bMax "Backlash in range bMin <= phi_rel - phi_rel0 <= bMax";
          SI.Torque tau_c;
          SI.Torque tau_d;
          SI.Angle phi_diff = phi_rel - phi_rel0;
          constant SI.Angle bEps = 1e-10 "minimum backlash";
          annotation(Documentation(info = "<html>
<p>
This element consists of a <b>backlash</b> element <b>connected in series</b>
to a <b>spring</b> and <b>damper</b> element which are <b>connected in parallel</b>.
The spring constant shall be non-zero, otherwise the component cannot be used.
</p>

<p>
In combination with components IdealGear, the ElastoBacklash model
can be used to model a gear box with backlash, elasticity and damping.
</p>

<p>
During initialization, the backlash characteristic is replaced by a continuous
approximation in the backlash region, in order to reduce problems during
initialization, especially for inverse models.
</p>

<p>
If the backlash b is smaller as 1e-10 rad (especially, if b=0),
then the backlash is ignored and the component reduces to a spring/damper
element in parallel.
</p>

<p>
In the backlash region (-b/2 &le; flange_b.phi - flange_a.phi - phi_rel0 &le; b/2) no torque
is exerted (flange_b.tau = 0). Outside of this region, contact is present and
the contact torque is basically computed with a linear
spring/damper characteristic:
</p

<pre>
   desiredContactTorque = c*phi_contact + d*<b>der</b>(phi_contact)

            phi_contact = phi_rel - phi_rel0 - b/2 <b>if</b> phi_rel - phi_rel0 &gt;  b/2
                        = phi_rel - phi_rel0 + b/2 <b>if</b> phi_rel - phi_rel0 &lt; -b/2

            phi_rel     = flange_b.phi - flange_a.phi;
</pre>

<p>
This torque characteristic leads to the following difficulties:
</p>

<ol>
<li> If the damper torque becomes larger as the spring torque and with opposite sign,
     the contact torque would be \"pulling/sticking\" which is unphysical, since during
     contact only pushing torques can occur.</li>

<li> When contact occurs with a non-zero relative speed (which is the usual
     situation), the damping torque has a non-zero value and therefore the contact
     torque changes discontinuously at phi_rel = phi_rel0. Again, this is not physical
     because the torque can only change continuously. (Note, this component is not an
     idealized model where a steep characteristic is approximated by a discontinuity,
     but it shall model the steep characteristic.)</li>
</ol>

<p>
In the literature there are several proposals to fix problem (2). However, there
seems to be no proposal to avoid sticking. For this reason, the most simple
approach is used in the ElastoBacklash model, to fix both problems by slight changes
to the linear spring/damper characteristic:
</p>

<pre>
    // Torque characteristic when phi_rel > phi_rel0
    <b>if</b> phi_rel - phi_rel0 &lt; b/2 <b>then</b>
       tau_c = 0;          // spring torque
       tau_d = 0;          // damper torque
       flange_b.tau = 0;
    <b>else</b>
       tau_c = c*(phi_rel - phi_rel0);    // spring torque
       tau_d = d*<b>der</b>(phi_rel);            // damper torque
       flange_b.tau = <b>if</b> tau_c + tau_d &le; 0 <b>then</b> 0 <b>else</b> tau_c + <b>min</b>( tau_c, tau_d );
    <b>end if</b>;
</pre>

<p>
Note, when sticking would occur (tau_c + tau_d &le; 0), then the contact torque
is explicitly set to zero. The \"min(tau_c, tau_d)\" part in the if-expression,
limits the damping torque when it is pushing. This means that at the start of
the contact (phi_rel - phi_rel0 = b/2), the damping torque is zero and is continuous.
The effect of both modifications is that the absolute value of the damping torque
is always limited by the absolute value of the spring torque: |tau_d| &le; |tau_c|.
</p>

<p>
In the next figure, a typical simulation with the ElastoBacklash model is shown
(<a href=\"Modelica://Modelica.Mechanics.Rotational.Examples.Backlash\">Examples.Backlash</a>)
where the different effects are visualized:
</p>

<ol>
<li> Curve 1 (elastoBacklash1.tau) is the unmodified contact torque, i.e., the linear spring/damper
     characteristic. A pulling/sticking torque is present at the end of the contact.</li>
<li> Curve 2 (elastoBacklash2.tau) is the contact torque, where the torque is explicitly set to
     zero when pulling/sticking occurs. The contact torque is discontinuous at begin of
     contact.</li>
<li> Curve 3 (elastoBacklash3.tau) is the ElastoBacklash model of this library. No discontinuity and no
     pulling/sticking occurs.</li>
</ol>

<p align=\"center\">
<img src=\"../Images/Rotational/elastoBacklash1.png\">
</p>
</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80,32},{ -58,32},{ -48,0},{ -34,61},{ -20,0},{ -8,60},{0,30},{20,30}}, color = {0,0,0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Rectangle(extent = {{ -60, -10},{ -10, -50}}, lineColor = {0,0,0}, pattern = LinePattern.Solid, lineThickness = 0.25, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -60, -50},{0, -50}}, color = {0,0,0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Line(points = {{ -60, -10},{0, -10}}, color = {0,0,0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Line(points = {{ -10, -30},{20, -30}}, color = {0,0,0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Line(points = {{ -80, -30},{ -60, -30}}, color = {0,0,0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Line(points = {{ -80,32},{ -80, -30}}, color = {0,0,0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Line(points = {{20,30},{20, -30}}, color = {0,0,0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Line(points = {{ -90,0},{ -80,0}}, color = {0,0,0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Line(points = {{90,0},{80,0}}, color = {0,0,0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Line(points = {{20,0},{60,0},{60, -30}}, color = {0,0,0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Line(points = {{40, -12},{40, -40},{80, -40},{80,0}}, color = {0,0,0}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Text(extent = {{ -150, -130},{150, -90}}, lineColor = {0,0,0}, textString = "b=%b"),Text(extent = {{ -150,100},{150,60}}, lineColor = {0,0,255}, textString = "%name"),Text(extent = {{ -152, -92},{148, -52}}, lineColor = {0,0,0}, textString = "c=%c")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80,32},{ -58,32},{ -48,0},{ -34,60},{ -20,0},{ -8,60},{0,30},{20,30}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -68,32},{ -68,97}}, color = {128,128,128}),Line(points = {{80,0},{80,96}}, color = {128,128,128}),Line(points = {{ -68,92},{72,92}}, color = {128,128,128}),Polygon(points = {{70,95},{80,92},{70,89},{70,95}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{ -10,70},{30,95}}, lineColor = {128,128,128}, textString = "phi_rel"),Rectangle(extent = {{ -60, -20},{ -10, -80}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -52, -80},{0, -80}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -52, -20},{0, -20}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -10, -50},{20, -50}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -80, -50},{ -60, -50}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -80,32},{ -80, -50}}, color = {0,0,0}, thickness = 0.5),Line(points = {{20,30},{20, -50}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -96,0},{ -80,0}}, color = {0,0,0}),Line(points = {{96,0},{80,0}}, color = {0,0,0}, thickness = 0.5),Line(points = {{20,0},{60,0},{60, -30}}, color = {0,0,0}, thickness = 0.5),Line(points = {{40, -12},{40, -40},{80, -40},{80,0}}, color = {0,0,0}, thickness = 0.5),Line(points = {{30,0},{30,64}}, color = {128,128,128}),Line(points = {{30,60},{80,60}}, color = {128,128,128}),Polygon(points = {{70,63},{80,60},{70,57},{70,63}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{39,60},{68,46}}, lineColor = {160,160,164}, textString = "b")}));

        equation 
          if initial() then
            tau_c = if phi_diff > 1.5 * bMax then c * (phi_diff - bMax) else if phi_diff < 1.5 * bMin then c * (phi_diff - bMin) else c / 3 * phi_diff;
            tau_d = d * w_rel;
            tau = tau_c + tau_d;

          else
            tau_c = if abs(b) <= bEps then c * phi_diff else if phi_diff > bMax then c * (phi_diff - bMax) else if phi_diff < bMin then c * (phi_diff - bMin) else 0;
            tau_d = d * w_rel;
            tau = if abs(b) <= bEps then tau_c + tau_d else if phi_diff > bMax then smooth(0, noEvent(if tau_c + tau_d <= 0 then 0 else tau_c + min(tau_c, tau_d))) else if phi_diff < bMin then smooth(0, noEvent(if tau_c + tau_d >= 0 then 0 else tau_c + max(tau_c, tau_d))) else 0;

          end if;
        end ElastoBacklash;
        model BearingFriction "Coulomb friction in bearings "
          extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryTwoFlangesAndSupport2;
          parameter Real tau_pos[:,2] = [0,1] "[w,tau] Positive sliding friction characteristic (w>=0)";
          parameter Real peak(final min = 1) = 1 "peak*tau_pos[1,2] = Maximum friction torque for w==0";
          extends Rotational.Interfaces.PartialFriction;
          SI.Angle phi "Angle between shaft flanges (flange_a, flange_b) and support";
          SI.Torque tau "Friction torque";
          SI.AngularVelocity w "Absolute angular velocity of flange_a and flange_b";
          SI.AngularAcceleration a "Absolute angular acceleration of flange_a and flange_b";
          annotation(Documentation(info = "<html>
<p>
This element describes <b>Coulomb friction</b> in <b>bearings</b>,
i.e., a frictional torque acting between a flange and the housing.
The positive sliding friction torque \"tau\" has to be defined
by table \"tau_pos\" as function of the absolute angular velocity \"w\".
E.g.
<p>
<pre>
       w | tau
      ---+-----
       0 |   0
       1 |   2
       2 |   5
       3 |   8
</pre>
<p>
gives the following table:
</p>
<pre>
   tau_pos = [0, 0; 1, 2; 2, 5; 3, 8];
</pre>
<p>
Currently, only linear interpolation in the table is supported.
Outside of the table, extrapolation through the last
two table entries is used. It is assumed that the negative
sliding friction force has the same characteristic with negative
values. Friction is modelled in the following way:
</p>
<p>
When the absolute angular velocity \"w\" is not zero, the friction torque
is a function of w and of a constant normal force. This dependency
is defined via table tau_pos and can be determined by measurements,
e.g. by driving the gear with constant velocity and measuring the
needed motor torque (= friction torque).
</p>
<p>
When the absolute angular velocity becomes zero, the elements
connected by the friction element become stuck, i.e., the absolute
angle remains constant. In this phase the friction torque is
calculated from a torque balance due to the requirement, that
the absolute acceleration shall be zero.  The elements begin
to slide when the friction torque exceeds a threshold value,
called the maximum static friction torque, computed via:
</p>
<pre>
   maximum_static_friction = <b>peak</b> * sliding_friction(w=0)  (<b>peak</b> >= 1)
</pre>
<p>
This procedure is implemented in a \"clean\" way by state events and
leads to continuous/discrete systems of equations if friction elements
are dynamically coupled which have to be solved by appropriate
numerical methods. The method is described in:
</p>
<dl>
<dt>Otter M., Elmqvist H., and Mattsson S.E. (1999):
<dd><b>Hybrid Modeling in Modelica based on the Synchronous
    Data Flow Principle</b>. CACSD'99, Aug. 22.-26, Hawaii.
</dl>
<p>
More precise friction models take into account the elasticity of the
material when the two elements are \"stuck\", as well as other effects,
like hysteresis. This has the advantage that the friction element can
be completely described by a differential equation without events. The
drawback is that the system becomes stiff (about 10-20 times slower
simulation) and that more material constants have to be supplied which
requires more sophisticated identification. For more details, see the
following references, especially (Armstrong and Canudas de Witt 1996):
</p>
<dl>
<dt>Armstrong B. (1991):
<dd><b>Control of Machines with Friction</b>. Kluwer Academic
    Press, Boston MA.<br><br>
<dt>Armstrong B., and Canudas de Wit C. (1996):
<dd><b>Friction Modeling and Compensation.</b>
    The Control Handbook, edited by W.S.Levine, CRC Press,
    pp. 1369-1382.<br><br>
<dt>Canudas de Wit C., Olsson H., Astroem K.J., and Lischinsky P. (1995):
<dd><b>A new model for control of systems with friction.</b>
    IEEE Transactions on Automatic Control, Vol. 40, No. 3, pp. 419-425.<br><br>
</dl>


</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,10},{100, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -60, -10},{60, -60}}, lineColor = {0,0,0}),Rectangle(extent = {{ -60, -10},{60, -25}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -60, -45},{60, -61}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -50, -18},{50, -50}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{60, -60},{60, -70},{75, -70},{75, -80},{ -75, -80},{ -75, -70},{ -60, -70},{ -60, -60},{60, -60}}, lineColor = {0,0,0}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Line(points = {{ -75, -10},{ -75, -70}}, color = {0,0,0}),Line(points = {{75, -10},{75, -70}}, color = {0,0,0}),Rectangle(extent = {{ -60,60},{60,10}}, lineColor = {0,0,0}),Rectangle(extent = {{ -60,60},{60,45}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -60,25},{60,10}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -50,51},{50,19}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -75,70},{ -75,10}}, color = {0,0,0}),Polygon(points = {{60,60},{60,70},{75,70},{75,80},{ -75,80},{ -75,70},{ -60,70},{ -60,60},{60,60}}, lineColor = {0,0,0}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Line(points = {{75,70},{75,10}}, color = {0,0,0}),Text(extent = {{ -150,130},{150,90}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{0, -80},{0, -100}}, color = {0,0,0}, smooth = Smooth.None)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          tau0 = Modelica.Math.tempInterpol1(0, tau_pos, 2);
          tau0_max = peak * tau0;
          free = false;
          phi = flange_a.phi - phi_support;
          flange_b.phi = flange_a.phi;
          w = der(phi);
          a = der(w);
          w_relfric = w;
          a_relfric = a;
          flange_a.tau + flange_b.tau - tau = 0;
          tau = if locked then sa * unitTorque else if startForward then Modelica.Math.tempInterpol1(w, tau_pos, 2) else if startBackward then  -Modelica.Math.tempInterpol1( -w, tau_pos, 2) else if pre(mode) == Forward then Modelica.Math.tempInterpol1(w, tau_pos, 2) else  -Modelica.Math.tempInterpol1( -w, tau_pos, 2);
        end BearingFriction;
        model Brake "Brake based on Coulomb friction "
          extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryTwoFlangesAndSupport2;
          parameter Real mue_pos[:,2] = [0,0.5] "[w,mue] positive sliding friction coefficient (w_rel>=0)";
          parameter Real peak(final min = 1) = 1 "peak*mue_pos[1,2] = maximum value of mue for w_rel==0";
          parameter Real cgeo(final min = 0) = 1 "Geometry constant containing friction distribution assumption";
          parameter SI.Force fn_max(final min = 0, start = 1) "Maximum normal force";
          extends Rotational.Interfaces.PartialFriction;
          SI.Angle phi "Angle between shaft flanges (flange_a, flange_b) and support";
          SI.Torque tau "Brake friction torqu";
          SI.AngularVelocity w "Absolute angular velocity of flange_a and flange_b";
          SI.AngularAcceleration a "Absolute angular acceleration of flange_a and flange_b";
          Real mue0 "Friction coefficient for w=0 and forward sliding";
          SI.Force fn "Normal force (=fn_max*f_normalized)";
          Modelica.Blocks.Interfaces.RealInput f_normalized "Normalized force signal 0..1 (normal force = fn_max*f_normalized; brake is active if > 0)" annotation(Placement(transformation(origin = {0,110}, extent = {{20, -20},{ -20,20}}, rotation = 90)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Polygon(points = {{ -37, -55},{ -37, -90},{37, -90},{37, -55},{33, -55},{33, -86},{ -33, -86},{ -33, -55},{ -37, -55}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -100,10},{ -20, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -20,60},{20, -60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{20,10},{100, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Polygon(points = {{40, -40},{70, -30},{70, -50},{40, -40}}, lineColor = {0,0,127}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid),Rectangle(extent = {{30, -25},{40, -55}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{ -40, -40},{ -70, -30},{ -70, -50},{ -40, -40}}, lineColor = {0,0,127}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -40, -25},{ -30, -55}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{0,90},{80,70},{80, -40},{70, -40}}, color = {0,0,127}),Line(points = {{0,90},{ -80,70},{ -80, -40},{ -70, -40}}, color = {0,0,127}),Text(extent = {{ -150, -180},{150, -140}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<html>
<p>
This component models a <b>brake</b>, i.e., a component where a frictional
torque is acting between the housing and a flange and a controlled normal
force presses the flange to the housing in order to increase friction.
The normal force fn has to be provided as input signal f_normalized in a normalized form
(0 &le; f_normalized &le; 1),
fn = fn_max*f_normalized, where fn_max has to be provided as parameter.
Friction in the brake is modelled in the following way:
</p>
<p>
When the absolute angular velocity \"w\" is not zero, the friction torque
is a function of the velocity dependent friction coefficient  mue(w) , of
the normal force \"fn\", and of a geometry constant \"cgeo\" which takes into
account the geometry of the device and the assumptions on the friction
distributions:
</p>
<pre>
        frictional_torque = <b>cgeo</b> * <b>mue</b>(w) * <b>fn</b>
</pre>
<p>
   Typical values of coefficients of friction:
</p>
<pre>
      dry operation   :  <b>mue</b> = 0.2 .. 0.4
      operating in oil:  <b>mue</b> = 0.05 .. 0.1
</pre>
<p>
   When plates are pressed together, where  <b>ri</b>  is the inner radius,
   <b>ro</b> is the outer radius and <b>N</b> is the number of friction interfaces,
   the geometry constant is calculated in the following way under the
   assumption of a uniform rate of wear at the interfaces:
</p>
<pre>
         <b>cgeo</b> = <b>N</b>*(<b>r0</b> + <b>ri</b>)/2
</pre>
<p>
    The positive part of the friction characteristic <b>mue</b>(w),
    w >= 0, is defined via table mue_pos (first column = w,
    second column = mue). Currently, only linear interpolation in
    the table is supported.
</p>
<p>
   When the absolute angular velocity becomes zero, the elements
   connected by the friction element become stuck, i.e., the absolute
   angle remains constant. In this phase the friction torque is
   calculated from a torque balance due to the requirement, that
   the absolute acceleration shall be zero.  The elements begin
   to slide when the friction torque exceeds a threshold value,
   called the  maximum static friction torque, computed via:
</p>
<pre>
       frictional_torque = <b>peak</b> * <b>cgeo</b> * <b>mue</b>(w=0) * <b>fn</b>   (<b>peak</b> >= 1)
</pre>
<p>
This procedure is implemented in a \"clean\" way by state events and
leads to continuous/discrete systems of equations if friction elements
are dynamically coupled. The method is described in:
</p>
<dl>
<dt>Otter M., Elmqvist H., and Mattsson S.E. (1999):
<dd><b>Hybrid Modeling in Modelica based on the Synchronous
    Data Flow Principle</b>. CACSD'99, Aug. 22.-26, Hawaii.
</dl>
<p>
More precise friction models take into account the elasticity of the
material when the two elements are \"stuck\", as well as other effects,
like hysteresis. This has the advantage that the friction element can
be completely described by a differential equation without events. The
drawback is that the system becomes stiff (about 10-20 times slower
simulation) and that more material constants have to be supplied which
requires more sophisticated identification. For more details, see the
following references, especially (Armstrong and Canudas de Witt 1996):
</p>
<dl>
<dt>Armstrong B. (1991):
<dd><b>Control of Machines with Friction</b>. Kluwer Academic
    Press, Boston MA.<br><br>
<dt>Armstrong B., and Canudas de Wit C. (1996):
<dd><b>Friction Modeling and Compensation.</b>
    The Control Handbook, edited by W.S.Levine, CRC Press,
    pp. 1369-1382.<br><br>
<dt>Canudas de Wit C., Olsson H., Astroem K.J., and Lischinsky P. (1995):
<dd><b>A new model for control of systems with friction.</b>
    IEEE Transactions on Automatic Control, Vol. 40, No. 3, pp. 419-425.<br><br>
</dl>

</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));

        equation 
          mue0 = Modelica.Math.tempInterpol1(0, mue_pos, 2);
          phi = flange_a.phi - phi_support;
          flange_b.phi = flange_a.phi;
          w = der(phi);
          a = der(w);
          w_relfric = w;
          a_relfric = a;
          flange_a.tau + flange_b.tau - tau = 0;
          fn = fn_max * f_normalized;
          tau0 = mue0 * cgeo * fn;
          tau0_max = peak * tau0;
          free = fn <= 0;
          tau = if locked then sa * unitTorque else if free then 0 else cgeo * fn * (if startForward then Modelica.Math.tempInterpol1(w, mue_pos, 2) else if startBackward then  -Modelica.Math.tempInterpol1( -w, mue_pos, 2) else if pre(mode) == Forward then Modelica.Math.tempInterpol1(w, mue_pos, 2) else  -Modelica.Math.tempInterpol1( -w, mue_pos, 2));
        end Brake;
        model Clutch "Clutch based on Coulomb friction "
          extends Modelica.Mechanics.Rotational.Interfaces.PartialCompliantWithRelativeStates;
          parameter Real mue_pos[:,2] = [0,0.5] "[w,mue] positive sliding friction coefficient (w_rel>=0)";
          parameter Real peak(final min = 1) = 1 "peak*mue_pos[1,2] = maximum value of mue for w_rel==0";
          parameter Real cgeo(final min = 0) = 1 "Geometry constant containing friction distribution assumption";
          parameter SI.Force fn_max(final min = 0, start = 1) "Maximum normal force";
          Modelica.SIunits.AngularAcceleration a_rel(start = 0) "Relative angular acceleration (= der(w_rel))";
          extends Rotational.Interfaces.PartialFriction;
          Real mue0 "Friction coefficient for w=0 and forward sliding";
          SI.Force fn "Normal force (fn=fn_max*f_normalized)";
          Modelica.Blocks.Interfaces.RealInput f_normalized "Normalized force signal 0..1 (normal force = fn_max*f_normalized; clutch is engaged if > 0)" annotation(Placement(transformation(origin = {0,110}, extent = {{20, -20},{ -20,20}}, rotation = 90)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -30,40},{ -60,50},{ -60,30},{ -30,40}}, lineColor = {0,0,127}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid),Line(points = {{0,90},{ -90,70},{ -90,40},{ -30,40}}, color = {0,0,127}),Line(points = {{0,90},{90,70},{90,40},{30,40}}, color = {0,0,127}),Polygon(points = {{30,40},{60,50},{60,30},{30,40}}, lineColor = {0,0,127}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid),Rectangle(extent = {{10,60},{30, -60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -30,60},{ -10, -60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{30,10},{100, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -100,10},{ -30, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Text(extent = {{ -150, -110},{150, -70}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<html>
<p>
This component models a <b>clutch</b>, i.e., a component with
two flanges where friction is present between the two flanges
and these flanges are pressed together via a normal force.
The normal force fn has to be provided as input signal f_normalized in a normalized form
(0 &le; f_normalized &le; 1),
fn = fn_max*f_normalized, where fn_max has to be provided as parameter. Friction in the
clutch is modelled in the following way:
</p>
<p>
When the relative angular velocity is not zero, the friction torque is a
function of the velocity dependent friction coefficient  mue(w_rel) , of
the normal force \"fn\", and of a geometry constant \"cgeo\" which takes into
account the geometry of the device and the assumptions on the friction
distributions:
</p>
<pre>
        frictional_torque = <b>cgeo</b> * <b>mue</b>(w_rel) * <b>fn</b>
</pre>
<p>
   Typical values of coefficients of friction:
</p>
<pre>
      dry operation   :  <b>mue</b> = 0.2 .. 0.4
      operating in oil:  <b>mue</b> = 0.05 .. 0.1
</pre>
<p>
   When plates are pressed together, where  <b>ri</b>  is the inner radius,
   <b>ro</b> is the outer radius and <b>N</b> is the number of friction interfaces,
   the geometry constant is calculated in the following way under the
   assumption of a uniform rate of wear at the interfaces:
</p>
<pre>
         <b>cgeo</b> = <b>N</b>*(<b>r0</b> + <b>ri</b>)/2
</pre>
<p>
    The positive part of the friction characteristic <b>mue</b>(w_rel),
    w_rel >= 0, is defined via table mue_pos (first column = w_rel,
    second column = mue). Currently, only linear interpolation in
    the table is supported.
</p>
<p>
   When the relative angular velocity becomes zero, the elements
   connected by the friction element become stuck, i.e., the relative
   angle remains constant. In this phase the friction torque is
   calculated from a torque balance due to the requirement, that
   the relative acceleration shall be zero.  The elements begin
   to slide when the friction torque exceeds a threshold value,
   called the  maximum static friction torque, computed via:
</p>
<pre>
       frictional_torque = <b>peak</b> * <b>cgeo</b> * <b>mue</b>(w_rel=0) * <b>fn</b>   (<b>peak</b> >= 1)
</pre>
<p>
This procedure is implemented in a \"clean\" way by state events and
leads to continuous/discrete systems of equations if friction elements
are dynamically coupled. The method is described in:
</p>
<dl>
<dt>Otter M., Elmqvist H., and Mattsson S.E. (1999):
<dd><b>Hybrid Modeling in Modelica based on the Synchronous
    Data Flow Principle</b>. CACSD'99, Aug. 22.-26, Hawaii.
</dl>
<p>
More precise friction models take into account the elasticity of the
material when the two elements are \"stuck\", as well as other effects,
like hysteresis. This has the advantage that the friction element can
be completely described by a differential equation without events. The
drawback is that the system becomes stiff (about 10-20 times slower
simulation) and that more material constants have to be supplied which
requires more sophisticated identification. For more details, see the
following references, especially (Armstrong and Canudas de Witt 1996):
</p>
<dl>
<dt>Armstrong B. (1991):
<dd><b>Control of Machines with Friction</b>. Kluwer Academic
    Press, Boston MA.<br><br>
<dt>Armstrong B., and Canudas de Wit C. (1996):
<dd><b>Friction Modeling and Compensation.</b>
    The Control Handbook, edited by W.S.Levine, CRC Press,
    pp. 1369-1382.<br><br>
<dt>Canudas de Wit C., Olsson H., Astroem K.J., and Lischinsky P. (1995):
<dd><b>A new model for control of systems with friction.</b>
    IEEE Transactions on Automatic Control, Vol. 40, No. 3, pp. 419-425.<br><br>
</dl>
<br>

</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          mue0 = Modelica.Math.tempInterpol1(0, mue_pos, 2);
          w_relfric = w_rel;
          a_relfric = a_rel;
          fn = fn_max * f_normalized;
          free = fn <= 0;
          tau0 = mue0 * cgeo * fn;
          tau0_max = peak * tau0;
          tau = if locked then sa * unitTorque else if free then 0 else cgeo * fn * (if startForward then Modelica.Math.tempInterpol1(w_rel, mue_pos, 2) else if startBackward then  -Modelica.Math.tempInterpol1( -w_rel, mue_pos, 2) else if pre(mode) == Forward then Modelica.Math.tempInterpol1(w_rel, mue_pos, 2) else  -Modelica.Math.tempInterpol1( -w_rel, mue_pos, 2));
        end Clutch;
        model OneWayClutch "Series connection of freewheel and clutch"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialCompliantWithRelativeStates;
          parameter Real mue_pos[:,2] = [0,0.5] "[w,mue] positive sliding friction coefficient (w_rel>=0)";
          parameter Real peak(final min = 1) = 1 "peak*mue_pos[1,2] = maximum value of mue for w_rel==0";
          parameter Real cgeo(final min = 0) = 1 "Geometry constant containing friction distribution assumption";
          parameter SI.Force fn_max(final min = 0, start = 1) "Maximum normal force";
          parameter SI.AngularVelocity w_small = 10000000000.0 "Relative angular velocity near to zero if jumps due to a reinit(..) of the velocity can occur (set to low value only if such impulses can occur)" annotation(Dialog(tab = "Advanced"));
          Real u "normalized force input signal (0..1)";
          SI.Force fn "Normal force (fn=fn_max*inPort.signal)";
          Boolean startForward "true, if w_rel=0 and start of forward sliding or w_rel > w_small";
          Boolean locked "true, if w_rel=0 and not sliding";
          Boolean stuck(final start = false) "w_rel=0 (forward sliding or locked)";
        protected 
          SI.Torque tau0 "Friction torque for w=0 and sliding";
          SI.Torque tau0_max "Maximum friction torque for w=0 and locked";
          Real mue0 "Friction coefficient for w=0 and sliding";
          Boolean free "true, if frictional element is not active";
          Real sa(final unit = "1") "path parameter of tau = f(a_rel) Friction characteristic";
          constant Real eps0 = 0.0001 "Relative hysteresis epsilon";
          SI.Torque tau0_max_low "lowest value for tau0_max";
          parameter Real peak2 = max([peak,1 + eps0]);
          constant SI.AngularAcceleration unitAngularAcceleration = 1;
          constant SI.Torque unitTorque = 1;
        public 
          Modelica.Blocks.Interfaces.RealInput f_normalized "Normalized force signal 0..1 (normal force = fn_max*f_normalized; clutch is engaged if > 0)" annotation(Placement(transformation(origin = {0,110}, extent = {{20, -20},{ -20,20}}, rotation = 90)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -30,40},{ -60,50},{ -60,30},{ -30,40}}, lineColor = {0,0,127}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid),Line(points = {{0,90},{ -90,70},{ -90,40},{ -30,40}}, color = {0,0,127}),Line(points = {{0,90},{90,70},{90,40},{30,40}}, color = {0,0,127}),Polygon(points = {{30,40},{60,50},{60,30},{30,40}}, lineColor = {0,0,127}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid),Rectangle(extent = {{10,60},{30, -60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -30,60},{ -10, -60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{30,10},{100, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -100,10},{ -30, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Text(extent = {{ -150, -110},{150, -70}}, textString = "%name", lineColor = {0,0,255}),Polygon(points = {{ -10,30},{50,0},{ -10, -30},{ -10,30}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>
This component models a <b>one-way clutch</b>, i.e., a component with
two flanges where friction is present between the two flanges
and these flanges are pressed together via a normal force. These
flanges maybe sliding with respect to each other
Parallel connection of ClutchCombi and of FreeWheel.
                     The element is introduced to resolve the ambiguity
                     of the constraint torques of the elements.
<p>
A one-way-clutch is an element where a clutch is connected in parallel
to a free wheel. This special element is provided, because such
a parallel connection introduces an ambiguity into the model
(the constraint torques are not uniquely defined when both
elements are stuck) and this element resolves it by introducing
<b>one</b> constraint torque and not two.
</p>
<p>
Note, initial values have to be chosen for the model, such that the
relative speed of the one-way-clutch >= 0. Otherwise, the configuration
is physically not possible and an error occurs.
</p>
<p>
The normal force fn has to be provided as input signal f_normalized in a normalized form
(0 &le; f_normalized &le; 1),
fn = fn_max*f_normalized, where fn_max has to be provided as parameter. Friction in the
clutch is modelled in the following way:
</p>
<p>
When the relative angular velocity is positive, the friction torque is a
function of the velocity dependent friction coefficient  mue(w_rel) , of
the normal force \"fn\", and of a geometry constant \"cgeo\" which takes into
account the geometry of the device and the assumptions on the friction
distributions:
</p>
<pre>
        frictional_torque = <b>cgeo</b> * <b>mue</b>(w_rel) * <b>fn</b>
</pre>
<p>
   Typical values of coefficients of friction:
</p>
<pre>
      dry operation   :  <b>mue</b> = 0.2 .. 0.4
      operating in oil:  <b>mue</b> = 0.05 .. 0.1
</pre>
<p>
   When plates are pressed together, where  <b>ri</b>  is the inner radius,
   <b>ro</b> is the outer radius and <b>N</b> is the number of friction interfaces,
   the geometry constant is calculated in the following way under the
   assumption of a uniform rate of wear at the interfaces:
</p>
<pre>
         <b>cgeo</b> = <b>N</b>*(<b>r0</b> + <b>ri</b>)/2
</pre>
<p>
    The positive part of the friction characteristic <b>mue</b>(w_rel),
    w_rel >= 0, is defined via table mue_pos (first column = w_rel,
    second column = mue). Currently, only linear interpolation in
    the table is supported.
</p>
<p>
   When the relative angular velocity becomes zero, the elements
   connected by the friction element become stuck, i.e., the relative
   angle remains constant. In this phase the friction torque is
   calculated from a torque balance due to the requirement, that
   the relative acceleration shall be zero.  The elements begin
   to slide when the friction torque exceeds a threshold value,
   called the  maximum static friction torque, computed via:
</p>
<pre>
       frictional_torque = <b>peak</b> * <b>cgeo</b> * <b>mue</b>(w_rel=0) * <b>fn</b>   (<b>peak</b> >= 1)
</pre>
<p>
This procedure is implemented in a \"clean\" way by state events and
leads to continuous/discrete systems of equations if friction elements
are dynamically coupled. The method is described in:
</p>
<dl>
<dt>Otter M., Elmqvist H., and Mattsson S.E. (1999):
<dd><b>Hybrid Modeling in Modelica based on the Synchronous
    Data Flow Principle</b>. CACSD'99, Aug. 22.-26, Hawaii.
</dl>

</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          mue0 = Modelica.Math.tempInterpol1(0, mue_pos, 2);
          tau0_max_low = eps0 * mue0 * cgeo * fn_max;
          u = f_normalized;
          free = u <= 0;
          fn = if free then 0 else fn_max * u;
          tau0 = mue0 * cgeo * fn;
          tau0_max = if free then tau0_max_low else peak2 * tau0;
          startForward = pre(stuck) and (sa > tau0_max / unitTorque or pre(startForward) and sa > tau0 / unitTorque or w_rel > w_small) or initial() and w_rel > 0;
          locked = pre(stuck) and not startForward;
          a_rel = unitAngularAcceleration * (if locked then 0 else sa - tau0 / unitTorque);
          tau = if locked then sa * unitTorque else if free then 0 else cgeo * fn * Modelica.Math.tempInterpol1(w_rel, mue_pos, 2);
          stuck = locked or w_rel <= 0;
        end OneWayClutch;
        model IdealGear "Ideal gear without inertia"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryTwoFlangesAndSupport2;
          parameter Real ratio(start = 1) "Transmission ratio (flange_a.phi/flange_b.phi)";
          Modelica.SIunits.Angle phi_a "Angle between left shaft flange and support";
          Modelica.SIunits.Angle phi_b "Angle between right shaft flange and support";
          annotation(Documentation(info = "<html>
<p>
This element characterices any type of gear box which is fixed in the
ground and which has one driving shaft and one driven shaft.
The gear is <b>ideal</b>, i.e., it does not have inertia, elasticity, damping
or backlash. If these effects have to be considered, the gear has to be
connected to other elements in an appropriate way.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -40,20},{ -20, -20}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -40,100},{ -20,20}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{20,80},{40,39}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{20,40},{40, -40}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{40,10},{100, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -20,70},{20,50}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -100,10},{ -40, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Text(extent = {{ -153,145},{147,105}}, lineColor = {0,0,255}, textString = "%name"),Line(points = {{ -80,20},{ -60,20}}, color = {0,0,0}),Line(points = {{ -80, -20},{ -60, -20}}, color = {0,0,0}),Line(points = {{ -70, -20},{ -70, -85}}, color = {0,0,0}),Line(points = {{0,40},{0, -100}}, color = {0,0,0}),Line(points = {{ -10,40},{10,40}}, color = {0,0,0}),Line(points = {{ -10,80},{10,80}}, color = {0,0,0}),Line(points = {{60, -20},{80, -20}}, color = {0,0,0}),Line(points = {{60,20},{80,20}}, color = {0,0,0}),Line(points = {{70, -20},{70, -85}}, color = {0,0,0}),Line(points = {{70, -85},{ -70, -85}}, color = {0,0,0}),Text(extent = {{ -146, -49},{154, -79}}, lineColor = {0,0,0}, textString = "ratio=%ratio")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          phi_a = flange_a.phi - phi_support;
          phi_b = flange_b.phi - phi_support;
          phi_a = ratio * phi_b;
          0 = ratio * flange_a.tau + flange_b.tau;
        end IdealGear;
        model LossyGear "Gear with mesh efficiency and bearing friction (stuck/rolling possible)"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryTwoFlangesAndSupport2;
          parameter Real ratio(start = 1) "Transmission ratio (flange_a.phi/flange_b.phi)";
          parameter Real lossTable[:,5] = [0,1,1,0,0] "Array for mesh efficiencies and bearing friction depending on speed";
          Modelica.SIunits.Angle phi_a "Angle between left shaft flange and support";
          Modelica.SIunits.Angle phi_b "Angle between right shaft flange and support";
          Real sa(final unit = "1") "Path parameter for acceleration and torque loss";
          SI.AngularVelocity w_a "Angular velocity of flange_a";
          SI.AngularAcceleration a_a "Angular acceleration of flange_a";
          Real interpolation_result[1,size(lossTable, 2) - 1];
          Real eta_mf1;
          Real eta_mf2;
          Real tau_bf1;
          Real tau_bf2;
          Real quadrant1;
          Real quadrant2;
          Real quadrant3;
          Real quadrant4;
          SI.Torque tauLoss "Torque loss due to friction in the gear teeth and in the bearings";
          SI.Torque tauLossMax "Torque loss for positive speed";
          SI.Torque tauLossMin "Torque loss for negative speed";
          Boolean tau_aPos(start = true) "true, if torque of flange_a is not negative";
          Boolean startForward(start = false) "true, if starting to roll forward";
          Boolean startBackward(start = false) "true, if starting to roll backward";
          Boolean locked(start = false) "true, if gear is locked";
          Boolean ideal "true, if losses are neglected";
          constant Integer Unknown = 3 "Value of mode is not known";
          constant Integer Free = 2 "Element is not active";
          constant Integer Forward = 1 "w_a > 0 (forward rolling)";
          constant Integer Stuck = 0 "w_a = 0 (forward rolling, locked or backward rolling)";
          constant Integer Backward =  -1 "w_a < 0 (backward rolling)";
          Integer mode(final min = Backward, final max = Unknown, start = Free, fixed = true);
          annotation(Documentation(info = "<HTML>
<p>
This component models the gear ratio and the <b>losses</b> of
a standard gear box in a <b>reliable</b> way including the stuck phases
that may occur at zero speed. The gear boxes that can
be handeled are fixed in the ground, have one input and one
output shaft, and are essentially described by the equations:
</p>
<pre>     flange_a.phi  = i*flange_b.phi
   (-flange_b.tau) = i*(eta_mf*flange_a.tau - tau_bf)
</pre>
<p>
where
</p>
<ul>
<li><b>i</b> is the constant <b>gear ratio</b>,</li>
<li><b>eta_mf</b> = eta_mf(w) is the <b>mesh efficiency</b> due to the
   friction between the teeth of the gear wheels, </li>
<li><b>tau_bf</b> = tau_bf(w) is the <b>bearing friction torque</b>, and</li>
<li><b>w_a</b> = der(flange_a.phi) is the speed of flange_a</li>
</ul>
<p>The loss terms \"eta_mf\" and \"tau_bf\" are functions of the
<i>absolute value</i> of the input shaft speed w_a and of the energy
flow direction. They are defined by parameter <b>lossTable[:,5]
</b> where the columns of this table have the following
meaning:</p>
<p><table BORDER=1 CELLSPACING=0 CELLPADDING=2>
  <tbody>
    <tr>
      <td valign=\"top\">|w_a|</td>
      <td valign=\"top\">eta_mf1</td>
      <td valign=\"top\">eta_mf2</td>
      <td valign=\"top\">|tau_bf1|</td>
      <td valign=\"top\">|tau_bf2|</td>
    </tr>
    <tr>
      <td align=\"center\">...</td>
      <td align=\"center\">...</td>
      <td align=\"center\">...</td>
      <td align=\"center\">...</td>
      <td align=\"center\">...</td>
    </tr>
    <tr>
      <td align=\"center\">...</td>
      <td align=\"center\">...</td>
      <td align=\"center\">...</td>
      <td align=\"center\">...</td>
      <td align=\"center\">...</td>
    </tr>
  </tbody>
</table></p>
<p>with</p>
<p><table BORDER=1 CELLSPACING=0 CELLPADDING=2>
  <tbody>
    <tr>
      <td valign=\"top\">|w_a|</td>
      <td valign=\"top\">Absolute value of angular velocity of input shaft flange_a</td>
    </tr>
    <tr>
      <td valign=\"top\">eta_mf1</td>
      <td valign=\"top\">Mesh efficiency in case of input shaft driving</td>
    </tr>
    <tr>
      <td valign=\"top\">eta_mf2</td>
      <td valign=\"top\">Mesh efficiency in case of output shaft driving</td>
    </tr>
    <tr>
      <td valign=\"top\">|tau_bf1|</td>
      <td valign=\"top\">Absolute bearing friction torque in case of input shaft driving</td>
    </tr>
    <tr>
      <td valign=\"top\">|tau_bf2|</td>
      <td valign=\"top\">Absolute bearing friction torque in case of output shaft driving</td>
    </tr>
  </tbody>
</table></p>
<p>
With these variables, the mesh efficiency and the bearing friction
are formally defined as:
</p>
<pre>  <b>if</b> flange_a.tau*w_a > 0 <b>or</b> flange_a.tau==0 <b>and</b> w_a > 0 <b>then</b>
     eta_mf := eta_mf1
     tau_bf := tau_bf1
  <b>elseif</b> flange_a.tau*w_a &lt; 0 <b>or</b> flange_a.tau==0 <b>and</b> w_a &lt; 0 <b>then</b>
     eta_mf := 1/eta_mf2
     tau_bf := tau_bf2
  <b>else</b> // w_a == 0
     eta_mf and tau_bf are computed such that <b>der</b>(w_a) = 0
  <b>end if</b>;
</pre>
<p>Note, that the losses are modeled in a physically meaningful way taking
into account that at zero speed the movement may be locked due
to the friction in the gear teeth and/or in the bearings.
Due to this important property, this component can be used in
situations where the combination of the components
Modelica.Mechanics.Rotational.IdealGear and
Modelica.Mechanics.Rotational.GearEfficiency will fail because,
e.g., chattering occurs when using the
Modelica.Mechanics.Rotational.GearEfficiency model.
</p>
<p>
<b>Acknowledgement:</b> The essential idea to model efficiency
in this way is from Christoph Pelchen, ZF Friedrichshafen.
</p>
<p><b>For detailed information:</b></p>
<p>Pelchen C.,
<a href=\"http://www.robotic.dlr.de/Christian.Schweiger/\">Schweiger C.</a>,
and <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Otter M.</a>:
&quot;<a href=\"http://www.modelica.org/Conference2002/papers/p33_Pelchen.pdf\">Modeling
and Simulating the Efficiency of Gearboxes and of Planetary Gearboxes</A>,&quot; in
<I>Proceedings of the 2nd International Modelica Conference, Oberpfaffenhofen, Germany,</I>
pp. 257-266, The Modelica Association and Institute of Robotics and Mechatronics,
Deutsches Zentrum f&uuml;r Luft- und Raumfahrt e. V., March 18-19, 2002.</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -40,20},{ -20, -20}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{40,10},{100, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -100,10},{ -40, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Polygon(points = {{ -109,40},{ -80,40},{ -80,80},{ -90,80},{ -70,100},{ -50,80},{ -60,80},{ -60,20},{ -109,20},{ -109,40}}, lineColor = {0,0,0}, fillColor = {255,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -80,20},{ -60,20}}, color = {0,0,0}),Line(points = {{ -80, -20},{ -60, -20}}, color = {0,0,0}),Line(points = {{ -70, -20},{ -70, -84}}, color = {0,0,0}),Line(points = {{70, -85},{ -70, -85}}, color = {0,0,0}),Line(points = {{60,20},{80,20}}, color = {0,0,0}),Line(points = {{60, -20},{80, -20}}, color = {0,0,0}),Line(points = {{70, -20},{70, -85}}, color = {0,0,0}),Text(extent = {{ -148,145},{152,105}}, lineColor = {0,0,255}, textString = "%name"),Rectangle(extent = {{ -40,100},{ -20,20}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -20,70},{20,50}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Line(points = {{ -10,80},{10,80}}, color = {0,0,0}),Line(points = {{ -10,40},{10,40}}, color = {0,0,0}),Line(points = {{0,40},{0, -100}}, color = {0,0,0}),Rectangle(extent = {{20,80},{40,39}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{20,40},{40, -40}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Text(extent = {{ -146, -49},{154, -79}}, lineColor = {0,0,0}, textString = "ratio=%ratio")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
        protected 
          constant SI.AngularAcceleration unitAngularAcceleration = 1;
          constant SI.Torque unitTorque = 1;
          function equal "Compare whether two Real matrices are identical"
            extends Modelica.Icons.Function;
            input Real A[:,:];
            input Real B[:,:];
            input Real eps = Modelica.Constants.eps "two numbers r1, r2 are identical if abs(r1-r2) <= eps";
            output Boolean result;
          algorithm 
            result:=false;
            if size(A, 1) == size(B, 1) and size(A, 2) == size(B, 2) then 
                        result:=true;
            for i in 1:size(A, 1) loop
                          for j in 1:size(A, 2) loop
                              if abs(A[i,j] - B[i,j]) >= eps then 
                                result:=false;

                else 
                end if;

              end for;

            end for;

            else 
            end if;
            annotation(Documentation(info = "<HTML>
<p>
The function call
</p>
<pre>     equal(A1, A2);
</pre>
<p>
returns <b>true</b>, if the two Real matrices A1 and A2 have the
same dimensions and the same elements. Otherwise the function
returns <b>false</b>. Two elements r1 and r2 of A1 and A2 respectively
are checked on equality by the test 'eps >= abs(r1-r2)', where 'eps'
can be provided as third argument of the function (the default is
Modelica.Constants.eps).
</p>
</HTML>"));
          end equal;

        equation 
          assert(abs(ratio) > 0, "Error in initialization of LossyGear: ratio may not be zero");
          ideal = equal(lossTable, [0,1,1,0,0]);
          interpolation_result = if ideal then [1,1,0,0] else Modelica.Math.tempInterpol2(noEvent(abs(w_a)), lossTable, {2,3,4,5});
          eta_mf1 = interpolation_result[1,1];
          eta_mf2 = interpolation_result[1,2];
          tau_bf1 = noEvent(abs(interpolation_result[1,3]));
          tau_bf2 = noEvent(abs(interpolation_result[1,4]));
          phi_a = flange_a.phi - phi_support;
          phi_b = flange_b.phi - phi_support;
          phi_a = ratio * phi_b;
          0 = flange_b.tau + ratio * (flange_a.tau - tauLoss);
          w_a = der(phi_a);
          a_a = der(w_a);
          quadrant1 = (1 - eta_mf1) * flange_a.tau + tau_bf1;
          quadrant2 = (1 - 1 / eta_mf2) * flange_a.tau + tau_bf2;
          quadrant4 = (1 - 1 / eta_mf2) * flange_a.tau - tau_bf2;
          quadrant3 = (1 - eta_mf1) * flange_a.tau - tau_bf1;
          tau_aPos = ideal or flange_a.tau >= 0;
          tauLossMax = if tau_aPos then quadrant1 else quadrant2;
          tauLossMin = if tau_aPos then quadrant4 else quadrant3;
          startForward = pre(mode) == Stuck and sa > tauLossMax / unitTorque or initial() and w_a > 0;
          startBackward = pre(mode) == Stuck and sa < tauLossMin / unitTorque or initial() and w_a < 0;
          locked = not (ideal or pre(mode) == Forward or startForward or pre(mode) == Backward or startBackward);
          tauLoss = if ideal then 0 else if locked then sa * unitTorque else if startForward or pre(mode) == Forward then tauLossMax else tauLossMin;
          a_a = unitAngularAcceleration * (if locked then 0 else sa - tauLoss / unitTorque);
          mode = if ideal then Free else if (pre(mode) == Forward or startForward) and w_a > 0 then Forward else if (pre(mode) == Backward or startBackward) and w_a < 0 then Backward else Stuck;
        end LossyGear;
        model IdealPlanetary "Ideal planetary gear box"
          parameter Real ratio(start = 100 / 50) "number of ring_teeth/sun_teeth (e.g. ratio=100/50)";
          Interfaces.Flange_a sun "Flange of sun shaft" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Interfaces.Flange_a carrier "Flange of carrier shaft" annotation(Placement(transformation(extent = {{ -110,30},{ -90,50}}, rotation = 0)));
          Interfaces.Flange_b ring "Flange of ring shaft" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          annotation(Documentation(info = "<HTML>
<p>
The IdealPlanetary gear box is an ideal gear without inertia,
elasticity, damping or backlash consisting
of an inner <b>sun</b> wheel, an outer <b>ring</b> wheel and a
<b>planet</b> wheel located between sun and ring wheel. The bearing
of the planet wheel shaft is fixed in the planet <b>carrier</b>.
The component can be connected to other elements at the
sun, ring and/or carrier flanges. It is not possible to connect
to the planet wheel. If inertia shall not be neglected,
the sun, ring and carrier inertias can be easily added by attaching
inertias (= model Inertia) to the corresponding connectors.
The inertias of the planet wheels are always neglected.
</p>
<p>
The icon of the planetary gear signals that the sun and carrier
flanges are on the left side and the ring flange is on the right side
of the gear box. However, this component is generic and is valid
independantly how the flanges are actually placed (e.g. sun wheel
may be placed on the right side instead on the left side in reality).
</p>
<p>
The ideal planetary gearbox is uniquely defined by the ratio
of the number of ring teeth zr with respect to the number of
sun teeth zs. For example, if there are 100 ring teeth and
50 sun teeth then ratio = zr/zs = 2. The number of planet teeth
zp has to fulfill the following relationship:
</p>
<pre>
   <b>zp := (zr - zs) / 2</b>
</pre>
<p>
Therefore, in the above example zp = 25 is required.
</p>
<p>
According to the overall convention, the positive direction of all
vectors, especially the absolute angular velocities and cut-torques
in the flanges, are along the axis vector displayed in the icon.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{50,100},{10, -100}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -10,45},{ -50,85}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -10,30},{ -50, -30}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -50,10},{ -100, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{100,10},{50, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{50,100},{ -50,105}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Rectangle(extent = {{50, -100},{ -50, -105}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -80,70},{ -50,60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Line(points = {{ -90,40},{ -70,40}}, color = {0,0,0}),Line(points = {{ -80,50},{ -60,50}}, color = {0,0,0}),Line(points = {{ -70,50},{ -70,40}}, color = {0,0,0}),Line(points = {{ -80,80},{ -59,80}}, color = {0,0,0}),Line(points = {{ -70,100},{ -70,80}}, color = {0,0,0}),Text(extent = {{ -150,150},{150,110}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -150, -110},{150, -150}}, lineColor = {0,0,0}, textString = "ratio=%ratio")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{50,100},{10, -100}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -10,45},{ -50,85}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -10,30},{ -50, -30}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -50,10},{ -100, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{100,10},{50, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{50,100},{ -50,105}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Rectangle(extent = {{50, -100},{ -50, -105}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -80,70},{ -50,60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Line(points = {{ -90,40},{ -70,40}}, color = {0,0,0}),Line(points = {{ -80,50},{ -60,50}}, color = {0,0,0}),Line(points = {{ -70,50},{ -70,40}}, color = {0,0,0}),Line(points = {{ -80,80},{ -59,80}}, color = {0,0,0}),Line(points = {{ -70,90},{ -70,80}}, color = {0,0,0}),Line(points = {{ -26, -42},{ -32, -2}}, pattern = LinePattern.Dot, color = {0,0,255}),Line(points = {{36, -26},{64, -60}}, pattern = LinePattern.Dot, color = {0,0,255}),Text(extent = {{58, -66},{98, -78}}, textString = "ring gear", lineColor = {0,0,255}),Text(extent = {{ -112,111},{ -56,87}}, textString = "planet carrier ", lineColor = {0,0,255}),Text(extent = {{ -47, -42},{ -3, -56}}, textString = "sun gear", lineColor = {0,0,255}),Polygon(points = {{58,130},{28,140},{28,120},{58,130}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{ -52,130},{28,130}}, color = {0,0,0}),Line(points = {{ -92,93},{ -70,80}}, pattern = LinePattern.Dot, color = {0,0,255}),Polygon(points = {{ -7, -86},{ -27, -81},{ -27, -91},{ -7, -86}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{ -97, -86},{ -26, -86}}, color = {128,128,128}),Text(extent = {{ -96, -71},{ -28, -84}}, lineColor = {128,128,128}, textString = "rotation axis")}));

        equation 
          (1 + ratio) * carrier.phi = sun.phi + ratio * ring.phi;
          ring.tau = ratio * sun.tau;
          carrier.tau =  -(1 + ratio) * sun.tau;
        end IdealPlanetary;
        model Gearbox "Realistic model of a gearbox (based on LossyGear)"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialTwoFlangesAndSupport;
          parameter Real ratio(start = 1) "transmission ratio (flange_a.phi/flange_b.phi)";
          parameter Real lossTable[:,5] = [0,1,1,0,0] "Array for mesh efficiencies and bearing friction depending on speed (see docu of LossyGear)";
          parameter SI.RotationalSpringConstant c(final min = Modelica.Constants.small, start = 100000.0) "Gear elasticity (spring constant)";
          parameter SI.RotationalDampingConstant d(final min = 0, start = 0) "(relative) gear damping";
          parameter SI.Angle b(final min = 0) = 0 "Total backlash";
          parameter StateSelect stateSelect = StateSelect.prefer "Priority to use phi_rel and w_rel as states" annotation(HideResult = true, Dialog(tab = "Advanced"));
          Modelica.SIunits.Angle phi_rel(start = 0, stateSelect = stateSelect, nominal = 0.0001) "Relative rotation angle over gear elasticity (= flange_b.phi - lossyGear.flange_b.phi)";
          Modelica.SIunits.AngularVelocity w_rel(start = 0, stateSelect = stateSelect) "Relative angular velocity over gear elasticity (= der(phi_rel))";
          Modelica.SIunits.AngularAcceleration a_rel(start = 0) "Relative angular acceleration over gear elasticity (= der(w_rel))";
          annotation(Documentation(info = "<html>
<p>This component models the essential effects of a gearbox, in
particular</p>
<ul>
  <li>in component <b>lossyGear</b></li>
    <ul>
      <li>gear <b>efficiency</b> due to friction between the teeth</li>
      <li><b>bearing friction</b></li>
    </ul>
  <li>in component <b>elastoBacklash</b></li>
    <ul>
      <li>gear <b>elasticity</b></li>
      <li><b>damping</b></li>
      <li><b>backlash</b></li>
    </ul>
</ul>
<p>The inertia of the gear wheels is not modeled. If necessary,
inertia has to be taken into account by connecting components of
model Inertia to the left and/or the right flange of component
Gearbox.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,10},{ -60, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{60,10},{100, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -40,60},{40, -60}}, lineColor = {0,0,0}, pattern = LinePattern.Solid, lineThickness = 0.25, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Polygon(points = {{ -60,10},{ -60,20},{ -40,40},{ -40, -40},{ -60, -20},{ -60,10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {128,128,128}),Polygon(points = {{60,20},{40,40},{40, -40},{60, -20},{60,20}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,150},{150,110}}, lineColor = {0,0,255}, textString = "%name"),Text(extent = {{ -150,70},{150,100}}, lineColor = {0,0,0}, textString = "ratio=%ratio, c=%c"),Polygon(points = {{ -60, -80},{ -46, -80},{ -20, -20},{20, -20},{46, -80},{60, -80},{60, -90},{ -60, -90},{ -60, -80}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
          Rotational.Components.LossyGear lossyGear(final ratio = ratio, final lossTable = lossTable, final useSupport = true) annotation(Placement(transformation(extent = {{ -60, -20},{ -20,20}}, rotation = 0)));
          Rotational.Components.ElastoBacklash elastoBacklash(final b = b, final c = c, final phi_rel0 = 0, final d = d) annotation(Placement(transformation(extent = {{20, -20},{60,20}}, rotation = 0)));

        equation 
          phi_rel = flange_b.phi - lossyGear.flange_b.phi;
          w_rel = der(phi_rel);
          a_rel = der(w_rel);
          connect(flange_a,lossyGear.flange_a) annotation(Line(points = {{ -100,0},{ -60,0}}, color = {0,0,0}));
          connect(lossyGear.flange_b,elastoBacklash.flange_a) annotation(Line(points = {{ -20,0},{20,0}}, color = {0,0,0}));
          connect(elastoBacklash.flange_b,flange_b) annotation(Line(points = {{60,0},{100,0}}, color = {0,0,0}));
          connect(lossyGear.support,support) annotation(Line(points = {{ -40, -20},{ -40, -40},{0, -40},{0, -100}}, color = {0,0,0}, smooth = Smooth.None));
        end Gearbox;
        model IdealGearR2T "Gearbox transforming rotational into translational motion"
          extends Rotational.Interfaces.PartialElementaryRotationalToTranslational;
          parameter Real ratio(final unit = "rad/m", start = 1) "Transmission ratio (flange_a.phi/flange_b.s)";
          annotation(Documentation(info = "<html>
This is an ideal mass- and inertialess gearbox which transforms a
1D-rotational into a 1D-translational motion. If elasticity, damping
or backlash has to be considered, this ideal gearbox has to be
connected with corresponding elements.
This component defines the kinematic constraint:
</p>

<pre>
  (flangeR.phi - internalSupportR.phi) = ratio*(flangeT.s - internalSupportT.s);
</pre>

</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -70,40},{10, -40}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -40,10},{ -20, -10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -100,10},{ -70, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Text(extent = {{ -151,123},{149,83}}, lineColor = {0,0,255}, textString = "%name"),Polygon(points = {{ -74, -60},{ -54, -40},{ -34, -60},{ -14, -40},{6, -60},{26, -40},{46, -60},{66, -40},{86, -60},{ -74, -60}}, lineColor = {0,0,0}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Rectangle(extent = {{95, -10},{106, -60}}, lineColor = {0,0,0}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -74, -60},{106, -80}}, lineColor = {0,0,0}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Text(extent = {{ -149,78},{151,48}}, lineColor = {0,0,0}, textString = "ratio=%ratio"),Line(points = {{ -100,15},{ -80,15}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{ -100, -16},{ -80, -16}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{ -100, -16},{ -100, -100}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{100, -80},{100, -100}}, color = {0,0,0}, smooth = Smooth.None)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          flangeR.phi - internalSupportR.phi = ratio * (flangeT.s - internalSupportT.s);
          0 = ratio * flangeR.tau + flangeT.f;
        end IdealGearR2T;
        model IdealRollingWheel "Simple 1-dim. model of an ideal rolling wheel without inertia"
          extends Rotational.Interfaces.PartialElementaryRotationalToTranslational;
          parameter Modelica.SIunits.Distance radius(start = 0.3) "Wheel radius";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,10},{ -46, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Ellipse(extent = {{ -50,80},{10, -80}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {160,160,164}),Rectangle(extent = {{ -20,80},{12, -80}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {160,160,164}),Ellipse(extent = {{ -16,80},{44, -80}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {160,160,164}),Ellipse(extent = {{ -2,52},{34, -52}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Ellipse(extent = {{12,10},{20, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Rectangle(extent = {{16,10},{60, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Ellipse(extent = {{56,10},{64, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {192,192,192}),Text(extent = {{ -136,138},{128,76}}, textString = "%name", lineColor = {0,0,255}),Polygon(points = {{80,10},{80,26},{60,26},{60,20},{70,20},{70, -20},{60, -20},{60, -26},{80, -26},{80, -10},{90, -10},{90,10},{80,10}}, lineColor = {0,127,0}, fillColor = {0,127,0}, fillPattern = FillPattern.Solid),Line(points = {{ -100, -20},{ -60, -20}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{ -100, -20},{ -100, -100}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{ -100,20},{ -60,20}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{100, -90},{ -40, -90}}, color = {0,127,0}, smooth = Smooth.None),Line(points = {{70, -26},{70, -50},{100, -50},{100, -100}}, color = {0,127,0}, smooth = Smooth.None)}), Documentation(info = "<html>
<p>
A simple kinematic model of a rolling wheel which has no inertia and
no rolling resistance. This component defines the kinematic constraint:
</p>

<pre>
   (flangeR.phi - internalSupportR.phi)*wheelRadius = (flangeT.s - internalSupportT.s);
</pre>

</html>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          (flangeR.phi - internalSupportR.phi) * radius = flangeT.s - internalSupportT.s;
          0 = radius * flangeT.f + flangeR.tau;
        end IdealRollingWheel;
        model InitializeFlange "Initializes a flange with pre-defined angle, speed and angular acceleration (usually, this is reference data from a control bus)"
          extends Modelica.Blocks.Interfaces.BlockIcon;
          parameter Boolean use_phi_start = true "= true, if initial angle is defined by input phi_start, otherwise not initialized";
          parameter Boolean use_w_start = true "= true, if initial speed is defined by input w_start, otherwise not initialized";
          parameter Boolean use_a_start = true "= true, if initial angular acceleration is defined by input a_start, otherwise not initialized";
          parameter StateSelect stateSelect = StateSelect.default "Priority to use flange angle and speed as states";
          Modelica.Blocks.Interfaces.RealInput phi_start if use_phi_start "Initial angle of flange" annotation(Placement(transformation(extent = {{ -140,60},{ -100,100}}, rotation = 0), iconTransformation(extent = {{ -140,60},{ -100,100}})));
          Modelica.Blocks.Interfaces.RealInput w_start if use_w_start "Initial speed of flange" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput a_start if use_a_start "Initial angular acceleration of flange" annotation(Placement(transformation(extent = {{ -140, -100},{ -100, -60}}, rotation = 0), iconTransformation(extent = {{ -140, -100},{ -100, -60}})));
          Interfaces.Flange_b flange "Flange that is initialized" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.SIunits.Angle phi_flange(stateSelect = stateSelect) = flange.phi "Flange angle";
          Modelica.SIunits.AngularVelocity w_flange(stateSelect = stateSelect) = der(phi_flange) "= der(phi_flange)";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -94,94},{68,66}}, lineColor = {0,0,0}, textString = "phi_start"),Text(extent = {{ -94,16},{70, -14}}, lineColor = {0,0,0}, textString = "w_start"),Text(extent = {{ -92, -68},{68, -96}}, lineColor = {0,0,0}, textString = "a_start")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
This component is used to optionally initialize the angle, speed,
and/or angular acceleration of the flange to which this component
is connected. Via parameters use_phi_start, use_w_start, use_a_start
the corresponding input signals phi_start, w_start, a_start are conditionally
activated. If an input is activated, the corresponding flange property
is initialized with the input value at start time.
</p>

<p>
For example, if \"use_phi_start = true\", then flange.phi is initialized
with the value of the input signal \"phi_start\" at the start time.
</p>

<p>
Additionally, it is optionally possible to define the \"StateSelect\"
attribute of the flange angle and the flange speed via paramater
\"stateSelection\".
</p>

<p>
This component is especially useful when the initial values of a flange
shall be set according to reference signals of a controller that are
provided via a signal bus.
</p>

</html>"));
        protected 
          encapsulated model Set_phi_start "Set phi_start"
            import Modelica;
            extends Modelica.Blocks.Interfaces.BlockIcon;
            Modelica.Blocks.Interfaces.RealInput phi_start "Start angle" annotation(HideResult = true, Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
            annotation(Diagram(graphics));
            Modelica.Mechanics.Rotational.Interfaces.Flange_b flange annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));

          initial equation 
            flange.phi = phi_start;

          equation 
            flange.tau = 0;
          end Set_phi_start;
          encapsulated model Set_w_start "Set w_start"
            import Modelica;
            extends Modelica.Blocks.Interfaces.BlockIcon;
            Modelica.Blocks.Interfaces.RealInput w_start "Start angular velocity" annotation(HideResult = true, Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
            annotation(Diagram(graphics));
            Modelica.Mechanics.Rotational.Interfaces.Flange_b flange annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));

          initial equation 
            der(flange.phi) = w_start;

          equation 
            flange.tau = 0;
          end Set_w_start;
          encapsulated model Set_a_start "Set a_start"
            import Modelica;
            extends Modelica.Blocks.Interfaces.BlockIcon;
            Modelica.Blocks.Interfaces.RealInput a_start "Start angular acceleration" annotation(HideResult = true, Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
            Modelica.Mechanics.Rotational.Interfaces.Flange_b flange(phi(stateSelect = StateSelect.avoid)) annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
            Modelica.SIunits.AngularVelocity w = der(flange.phi) annotation(HideResult = true);

          initial equation 
            der(w) = a_start;

          equation 
            flange.tau = 0;
          end Set_a_start;
          encapsulated model Set_flange_tau "Set flange.tau to zero"
            import Modelica;
            extends Modelica.Blocks.Interfaces.BlockIcon;
            Modelica.Mechanics.Rotational.Interfaces.Flange_b flange annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));

          equation 
            flange.tau = 0;
          end Set_flange_tau;
        protected 
          Set_phi_start set_phi_start if use_phi_start annotation(Placement(transformation(extent = {{ -20,70},{0,90}}, rotation = 0)));
          Set_w_start set_w_start if use_w_start annotation(Placement(transformation(extent = {{ -20, -10},{0,10}}, rotation = 0)));
          Set_a_start set_a_start if use_a_start annotation(Placement(transformation(extent = {{ -20, -90},{0, -70}}, rotation = 0)));
          Set_flange_tau set_flange_tau annotation(Placement(transformation(extent = {{96, -90},{76, -70}}, rotation = 0)));

        equation 
          connect(set_phi_start.phi_start,phi_start) annotation(Line(points = {{ -22,80},{ -120,80}}, color = {0,0,127}, smooth = Smooth.None));
          connect(set_phi_start.flange,flange) annotation(Line(points = {{0,80},{60,80},{60,0},{100,0}}, color = {0,0,0}, smooth = Smooth.None));
          connect(set_w_start.flange,flange) annotation(Line(points = {{0,0},{100,0}}, color = {0,0,0}, smooth = Smooth.None));
          connect(set_w_start.w_start,w_start) annotation(Line(points = {{ -22,0},{ -120,0}}, color = {0,0,127}, smooth = Smooth.None));
          connect(set_a_start.a_start,a_start) annotation(Line(points = {{ -22, -80},{ -120, -80}}, color = {0,0,127}, smooth = Smooth.None));
          connect(set_a_start.flange,flange) annotation(Line(points = {{0, -80},{60, -80},{60,0},{100,0}}, color = {0,0,0}, smooth = Smooth.None));
          connect(set_flange_tau.flange,flange) annotation(Line(points = {{76, -80},{60, -80},{60,0},{100,0}}, color = {0,0,0}, smooth = Smooth.None));
        end InitializeFlange;
        model RelativeStates "Definition of relative state variables"
          extends Rotational.Interfaces.PartialTwoFlanges;
          parameter StateSelect stateSelect = StateSelect.prefer "Priority to use the relative angle and relative speed as states";
          SI.Angle phi_rel(start = 0, stateSelect = stateSelect) "Relative rotation angle used as state variable";
          SI.AngularVelocity w_rel(start = 0, stateSelect = stateSelect) "Relative angular velocity used as state variable";
          SI.AngularAcceleration a_rel(start = 0) "Relative angular acceleration";
          annotation(Documentation(info = "<html>
<p>
Usually, the absolute angle and the absolute angular velocity of
Modelica.Mechanics.Rotational.Components.Inertia models are used as state variables.
In some circumstances, relative quantities are better suited, e.g.,
because it may be easier to supply initial values.
In such cases, model <b>RelativeStates</b> allows the definition of state variables
in the following way:
</p>
<ul>
<li> Connect an instance of this model between two flange connectors.</li>
<li> The <b>relative rotation angle</b> and the <b>relative angular velocity</b>
     between the two connectors are used as <b>state variables</b>.
</ul>
<p>
An example is given in the next figure
</p>
<IMG SRC=\"../Images/Rotational/relativeStates.png\" ALT=\"relativeStates\">
<p>
Here, the relative angle and the relative angular velocity between
the two inertias are used as state variables. Additionally, the
simulator selects either the absolute angle and absolute angular
velocity of model inertia1 or of model inertia2 as state variables.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Ellipse(extent = {{ -40,40},{40, -40}}, lineColor = {0,255,255}, fillColor = {0,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -40,40},{40, -40}}, textString = "S", lineColor = {0,0,255}),Line(points = {{ -92,0},{ -42,0}}, color = {0,0,0}, pattern = LinePattern.Dot),Line(points = {{40,0},{90,0}}, color = {0,0,0}, pattern = LinePattern.Dot),Text(extent = {{ -150, -40},{150, -80}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Ellipse(extent = {{ -40,40},{40, -40}}, lineColor = {0,255,255}, fillColor = {0,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -40,40},{40, -40}}, textString = "S", lineColor = {0,0,255}),Line(points = {{40,0},{96,0}}, color = {0,0,0}, pattern = LinePattern.Dash),Line(points = {{ -100, -10},{ -100, -80}}, color = {160,160,164}),Line(points = {{100, -10},{100, -80}}, color = {160,160,164}),Polygon(points = {{80, -65},{80, -55},{100, -60},{80, -65}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Line(points = {{ -100, -60},{80, -60}}, color = {160,160,164}),Text(extent = {{ -30, -70},{30, -90}}, textString = "w_rel", lineColor = {0,0,255}),Line(points = {{ -76,80},{ -5,80}}, color = {128,128,128}),Polygon(points = {{14,80},{ -6,85},{ -6,75},{14,80}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{18,87},{86,74}}, lineColor = {128,128,128}, textString = "rotation axis"),Line(points = {{ -96,0},{ -40,0}}, color = {0,0,0}, pattern = LinePattern.Dash)}));

        equation 
          phi_rel = flange_b.phi - flange_a.phi;
          w_rel = der(phi_rel);
          a_rel = der(w_rel);
          flange_a.tau = 0;
          flange_b.tau = 0;
        end RelativeStates;
      end Components;
      package Sources "Sources to drive 1D rotational mechanical components"
        extends Modelica.Icons.Library2;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -98, -64},{ -74, -34},{ -46, -12},{ -12, -2},{18, -8},{38, -20},{54, -36},{66, -50},{76, -64}}, color = {0,0,0}, thickness = 0.5),Polygon(points = {{76, -64},{56, -6},{27, -37},{76, -64}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>
This package contains ideal sources to drive 1D mechanical rotational drive trains.
</p>
</html>"));
        model Position "Forced movement of a flange according to a reference angle signal"
          import SI = Modelica.SIunits;
          extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryOneFlangeAndSupport2;
          parameter Boolean exact = false "true/false exact treatment/filtering the input signal";
          parameter SI.Frequency f_crit = 50 "if exact=false, critical frequency of filter to filter input signal" annotation(Dialog(enable = not exact));
          SI.Angle phi(stateSelect = if exact then StateSelect.default else StateSelect.prefer) "Rotation angle of flange with respect to support";
          SI.AngularVelocity w(start = 0, stateSelect = if exact then StateSelect.default else StateSelect.prefer) "If exact=false, Angular velocity of flange with respect to support else dummy";
          SI.AngularAcceleration a(start = 0) "If exact=false, Angular acceleration of flange with respect to support else dummy";
          Modelica.Blocks.Interfaces.RealInput phi_ref(final quantity = "Angle", final unit = "rad", displayUnit = "deg") "Reference angle of flange with respect to support as input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
          annotation(Documentation(info = "<HTML>
<p>
The input signal <b>phi_ref</b> defines the <b>reference
angle</b> in [rad]. Flange <b>flange</b> is <b>forced</b>
to move according to this reference motion relative to flange support. According to parameter
<b>exact</b> (default = <b>false</b>), this is done in the following way:
<ol>
<li><b>exact=true</b><br>
    The reference angle is treated <b>exactly</b>. This is only possible, if
    the input signal is defined by an analytical function which can be
    differentiated at least twice. If this prerequisite is fulfilled,
    the Modelica translator will differentiate the input signal twice
    in order to compute the reference acceleration of the flange.</li>
<li><b>exact=false</b><br>
    The reference angle is <b>filtered</b> and the second derivative
    of the filtered curve is used to compute the reference acceleration
    of the flange. This second derivative is <b>not</b> computed by
    numerical differentiation but by an appropriate realization of the
    filter. For filtering, a second order Bessel filter is used.
    The critical frequency (also called cut-off frequency) of the
    filter is defined via parameter <b>f_crit</b> in [Hz]. This value
    should be selected in such a way that it is higher as the essential
    low frequencies in the signal.</li>
</ol>
<p>
The input signal can be provided from one of the signal generator
blocks of the block library Modelica.Blocks.Sources.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100,20},{100, -20}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Line(points = {{ -30, -32},{30, -32}}, color = {0,0,0}),Line(points = {{0,52},{0,32}}, color = {0,0,0}),Line(points = {{ -29,32},{30,32}}, color = {0,0,0}),Line(points = {{0, -32},{0, -100}}, color = {0,0,0}),Line(points = {{30, -42},{20, -52}}, color = {0,0,0}),Line(points = {{30, -32},{10, -52}}, color = {0,0,0}),Line(points = {{20, -32},{0, -52}}, color = {0,0,0}),Line(points = {{10, -32},{ -10, -52}}, color = {0,0,0}),Line(points = {{0, -32},{ -20, -52}}, color = {0,0,0}),Line(points = {{ -10, -32},{ -30, -52}}, color = {0,0,0}),Line(points = {{ -20, -32},{ -30, -42}}, color = {0,0,0}),Text(extent = {{ -56, -56},{ -172, -90}}, lineColor = {0,0,0}, textString = "phi_ref"),Text(extent = {{150,60},{ -150,100}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{146, -28},{30, -62}}, lineColor = {0,0,0}, textString = "exact="),Text(extent = {{146, -64},{30, -98}}, lineColor = {0,0,0}, textString = "%exact")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));
        protected 
          parameter Modelica.SIunits.AngularFrequency w_crit = 2 * Modelica.Constants.pi * f_crit "Critical frequency";
          constant Real af = 1.3617 "s coefficient of Bessel filter";
          constant Real bf = 0.618 "s*s coefficient of Bessel filter";

        initial equation 
if not exact then
                      phi = phi_ref;
          end if;

        equation 
          phi = flange.phi - phi_support;
          if exact then
            phi = phi_ref;
            w = 0;
            a = 0;

          else
            w = der(phi);
            a = der(w);
            a = ((phi_ref - phi) * w_crit - af * w) * w_crit / bf;

          end if;
        end Position;
        model Speed "Forced movement of a flange according to a reference angular velocity signal"
          import SI = Modelica.SIunits;
          extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryOneFlangeAndSupport2;
          parameter Boolean exact = false "true/false exact treatment/filtering the input signal";
          parameter SI.Frequency f_crit = 50 "if exact=false, critical frequency of filter to filter input signal";
          SI.Angle phi(start = 0, fixed = true, stateSelect = StateSelect.prefer) "Rotation angle of flange with respect to support";
          SI.AngularVelocity w(stateSelect = if exact then StateSelect.default else StateSelect.prefer) "Angular velocity of flange with respect to support";
          SI.AngularAcceleration a "If exact=false, angular acceleration of flange with respect to support else dummy";
          Modelica.Blocks.Interfaces.RealInput w_ref "Reference angular velocity of flange with respect to support as input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>
The input signal <b>w_ref</b> defines the <b>reference
speed</b> in [rad/s]. Flange <b>flange</b> is <b>forced</b>
to move relative to flange support according to this reference motion. According to parameter
<b>exact</b> (default = <b>false</b>), this is done in the following way:
<ol>
<li><b>exact=true</b><br>
    The reference speed is treated <b>exactly</b>. This is only possible, if
    the input signal is defined by an analytical function which can be
    differentiated at least once. If this prerequisite is fulfilled,
    the Modelica translator will differentiate the input signal once
    in order to compute the reference acceleration of the flange.</li>
<li><b>exact=false</b><br>
    The reference angle is <b>filtered</b> and the second derivative
    of the filtered curve is used to compute the reference acceleration
    of the flange. This second derivative is <b>not</b> computed by
    numerical differentiation but by an appropriate realization of the
    filter. For filtering, a first order filter is used.
    The critical frequency (also called cut-off frequency) of the
    filter is defined via parameter <b>f_crit</b> in [Hz]. This value
    should be selected in such a way that it is higher as the essential
    low frequencies in the signal.</li>
</ol>
<p>
The input signal can be provided from one of the signal generator
blocks of the block library Modelica.Blocks.Sources.
</p>

</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100,20},{100, -20}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Line(points = {{ -30, -32},{30, -32}}, color = {0,0,0}),Line(points = {{0,52},{0,32}}, color = {0,0,0}),Line(points = {{ -29,32},{30,32}}, color = {0,0,0}),Line(points = {{0, -32},{0, -100}}, color = {0,0,0}),Line(points = {{ -10, -32},{ -30, -52}}, color = {0,0,0}),Line(points = {{0, -32},{ -20, -52}}, color = {0,0,0}),Line(points = {{10, -32},{ -10, -52}}, color = {0,0,0}),Line(points = {{20, -32},{0, -52}}, color = {0,0,0}),Line(points = {{ -20, -32},{ -30, -42}}, color = {0,0,0}),Line(points = {{30, -32},{10, -52}}, color = {0,0,0}),Line(points = {{30, -42},{20, -52}}, color = {0,0,0}),Text(extent = {{ -54, -44},{ -158, -78}}, lineColor = {0,0,0}, textString = "w_ref"),Text(extent = {{0,120},{0,60}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{146, -26},{30, -60}}, lineColor = {0,0,0}, textString = "exact="),Text(extent = {{146, -62},{30, -96}}, lineColor = {0,0,0}, textString = "%exact")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));
        protected 
          parameter Modelica.SIunits.AngularFrequency w_crit = 2 * Modelica.Constants.pi * f_crit "Critical frequency";

        initial equation 
if not exact then
                      w = w_ref;
          end if;

        equation 
          phi = flange.phi - phi_support;
          w = der(phi);
          if exact then
            w = w_ref;
            a = 0;

          else
            a = der(w);
            a = (w_ref - w) * w_crit;

          end if;
        end Speed;
        model Accelerate "Forced movement of a flange according to an acceleration signal"
          import SI = Modelica.SIunits;
          extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryOneFlangeAndSupport2;
          SI.Angle phi(start = 0, fixed = true, stateSelect = StateSelect.prefer) "Rotation angle of flange with respect to support";
          SI.AngularVelocity w(start = 0, fixed = true, stateSelect = StateSelect.prefer) "Angular velocity of flange with respect to support";
          SI.AngularAcceleration a "Angular acceleration of flange with respect to support";
          Modelica.Blocks.Interfaces.RealInput a_ref "Absolute angular acceleration of flange with respect to support as input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>
The input signal <b>a</b> defines an <b>angular acceleration</b>
in [rad/s2]. Flange <b>flange</b> is <b>forced</b> to move relative to flange support with
this acceleration. The angular velocity <b>w</b> and the rotation angle
<b>phi</b> of the flange are automatically determined by integration of
the acceleration.
</p>
<p>
The input signal can be provided from one of the signal generator
blocks of the block library Modelica.Blocks.Sources.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100,20},{100, -20}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Line(points = {{ -30, -32},{30, -32}}, color = {0,0,0}),Line(points = {{0,52},{0,32}}, color = {0,0,0}),Line(points = {{ -29,32},{30,32}}, color = {0,0,0}),Line(points = {{0, -32},{0, -100}}, color = {0,0,0}),Line(points = {{30, -42},{20, -52}}, color = {0,0,0}),Line(points = {{30, -32},{10, -52}}, color = {0,0,0}),Line(points = {{20, -32},{0, -52}}, color = {0,0,0}),Line(points = {{10, -32},{ -10, -52}}, color = {0,0,0}),Line(points = {{0, -32},{ -20, -52}}, color = {0,0,0}),Line(points = {{ -10, -32},{ -30, -52}}, color = {0,0,0}),Line(points = {{ -20, -32},{ -30, -42}}, color = {0,0,0}),Text(extent = {{ -46, -50},{ -144, -86}}, lineColor = {0,0,0}, textString = "a_ref"),Text(extent = {{ -150,100},{150,60}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));

        equation 
          phi = flange.phi - phi_support;
          w = der(phi);
          a = der(w);
          a = a_ref;
        end Accelerate;
        model Move "Forced movement of a flange according to an angle, speed and angular acceleration signal"
          import SI = Modelica.SIunits;
          extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryOneFlangeAndSupport2;
          Modelica.SIunits.Angle phi "Rotation angle of flange with respect to support";
          Modelica.Blocks.Interfaces.RealInput u[3] "Angle, angular velocity and angular acceleration of flange with respect to support as input signals" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>
Flange <b>flange</b> is <b>forced</b> to move relative to flange support with a predefined motion
according to the input signals:
</p>
<pre>
    u[1]: angle of flange
    u[2]: angular velocity of flange
    u[3]: angular acceleration of flange
</pre>
<p>
The user has to guarantee that the input signals are consistent to each other,
i.e., that u[2] is the derivative of u[1] and that
u[3] is the derivative of u[2]. There are, however,
also applications where by purpose these conditions do not hold. For example,
if only the position dependent terms of a mechanical system shall be
calculated, one may provide angle = angle(t) and set the angular velocity
and the angular acceleration to zero.
</p>
<p>
The input signals can be provided from one of the signal generator
blocks of the block library Modelica.Blocks.Sources.
</p>

</html>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -80, -60},{ -80, -100}}, lineColor = {0,0,0}, textString = "phi,w,a"),Rectangle(extent = {{ -100,20},{100, -20}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Line(points = {{ -30, -32},{30, -32}}, color = {0,0,0}),Line(points = {{0,52},{0,32}}, color = {0,0,0}),Line(points = {{ -29,32},{30,32}}, color = {0,0,0}),Line(points = {{0, -32},{0, -100}}, color = {0,0,0}),Line(points = {{30, -42},{20, -52}}, color = {0,0,0}),Line(points = {{30, -32},{10, -52}}, color = {0,0,0}),Line(points = {{20, -32},{0, -52}}, color = {0,0,0}),Line(points = {{10, -32},{ -10, -52}}, color = {0,0,0}),Line(points = {{0, -32},{ -20, -52}}, color = {0,0,0}),Line(points = {{ -10, -32},{ -30, -52}}, color = {0,0,0}),Line(points = {{ -20, -32},{ -30, -42}}, color = {0,0,0}),Text(extent = {{ -150,100},{150,60}}, textString = "%name", lineColor = {0,0,255})}));
        protected 
          function position
            input Real q_qd_qdd[3] "Required values for position, speed, acceleration";
            input Real dummy "Just to have one input signal that should be differentiated to avoid possible problems in the Modelica tool (is not used)";
            output Real q;
            annotation(derivative(noDerivative = q_qd_qdd) = position_der, __Dymola_InlineAfterIndexReduction = true);
          algorithm 
            q:=q_qd_qdd[1];
          end position;
          function position_der
            input Real q_qd_qdd[3] "Required values for position, speed, acceleration";
            input Real dummy "Just to have one input signal that should be differentiated to avoid possible problems in the Modelica tool (is not used)";
            input Real dummy_der;
            output Real qd;
            annotation(derivative(noDerivative = q_qd_qdd) = position_der2, __Dymola_InlineAfterIndexReduction = true);
          algorithm 
            qd:=q_qd_qdd[2];
          end position_der;
          function position_der2
            input Real q_qd_qdd[3] "Required values for position, speed, acceleration";
            input Real dummy "Just to have one input signal that should be differentiated to avoid possible problems in the Modelica tool (is not used)";
            input Real dummy_der;
            input Real dummy_der2;
            output Real qdd;
          algorithm 
            qdd:=q_qd_qdd[3];
          end position_der2;

        equation 
          phi = flange.phi - phi_support;
          phi = position(u, time);
        end Move;
        model Torque "Input signal acting as external torque on a flange"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryOneFlangeAndSupport2;
          Modelica.Blocks.Interfaces.RealInput tau "Accelerating torque acting at flange (= -flange.tau)" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
          annotation(Documentation(info = "<HTML>
<p>
The input signal <b>tau</b> defines an external
torque in [Nm] which acts (with negative sign) at
a flange connector, i.e., the component connected to this
flange is driven by torque <b>tau</b>.</p>
<p>
The input signal can be provided from one of the signal generator
blocks of Modelica.Blocks.Sources.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -150,110},{150,70}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -62, -29},{ -141, -70}}, lineColor = {0,0,0}, textString = "tau"),Line(points = {{ -88,0},{ -64,30},{ -36,52},{ -2,62},{28,56},{48,44},{64,28},{76,14},{86,0}}, color = {0,0,0}, thickness = 0.5),Polygon(points = {{86,0},{66,58},{37,27},{86,0}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -30, -30},{30, -30}}, color = {0,0,0}),Line(points = {{0, -30},{0, -101}}, color = {0,0,0}),Line(points = {{ -30, -50},{ -10, -30}}, color = {0,0,0}),Line(points = {{ -10, -50},{10, -30}}, color = {0,0,0}),Line(points = {{10, -50},{30, -30}}, color = {0,0,0}),Line(points = {{ -54, -42},{ -38, -28},{ -16, -16},{4, -14},{22, -18},{36, -26},{48, -36},{56, -46},{64, -58}}, color = {0,0,0}),Polygon(points = {{ -61, -66},{ -44, -42},{ -58, -36},{ -61, -66}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{14,86},{82,73}}, lineColor = {128,128,128}, textString = "rotation axis"),Polygon(points = {{10,80},{ -10,85},{ -10,75},{10,80}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{ -80,80},{ -9,80}}, color = {128,128,128}),Line(points = {{ -88,0},{ -64,30},{ -36,52},{ -2,62},{28,56},{48,44},{64,28},{76,14},{80,10}}, color = {0,0,0}, thickness = 0.5),Polygon(points = {{86,0},{66,58},{38,28},{86,0}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}));

        equation 
          flange.tau =  -tau;
        end Torque;
        model Torque2 "Input signal acting as torque on two flanges"
          extends Rotational.Interfaces.PartialTwoFlanges;
          Modelica.Blocks.Interfaces.RealInput tau "Torque driving the two flanges (a positive value accelerates the flange)" annotation(Placement(transformation(origin = {0,40}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
          annotation(Documentation(info = "<HTML>
<p>
The input signal <b>tau</b> defines an external
torque in [Nm] which acts at both flange connectors,
i.e., the components connected to these flanges are driven by torque <b>tau</b>.</p>
<p>The input signal can be provided from one of the signal generator
blocks of Modelica.Blocks.Sources.</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -150, -40},{150, -80}}, textString = "%name", lineColor = {0,0,255}),Polygon(points = {{ -78,24},{ -69,17},{ -89,0},{ -78,24}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -74,20},{ -70,23},{ -65,26},{ -60,28},{ -56,29},{ -50,30},{ -41,30},{ -35,29},{ -31,28},{ -26,26},{ -21,23},{ -17,20},{ -13,15},{ -10,9}}, color = {0,0,0}, thickness = 0.5),Line(points = {{74,20},{70,23},{65,26},{60,28},{56,29},{50,30},{41,30},{35,29},{31,28},{26,26},{21,23},{17,20},{13,15},{10,9}}, color = {0,0,0}, thickness = 0.5),Polygon(points = {{89,0},{78,24},{69,17},{89,0}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{15, -71},{83, -84}}, lineColor = {128,128,128}, textString = "rotation axis"),Polygon(points = {{11, -77},{ -9, -72},{ -9, -82},{11, -77}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{ -79, -77},{ -8, -77}}, color = {128,128,128}),Line(points = {{ -75,20},{ -71,23},{ -66,26},{ -61,28},{ -57,29},{ -51,30},{ -42,30},{ -36,29},{ -32,28},{ -27,26},{ -22,23},{ -18,20},{ -14,15},{ -11,9}}, color = {0,0,0}, thickness = 0.5),Polygon(points = {{ -79,24},{ -70,17},{ -90,0},{ -79,24}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{73,20},{69,23},{64,26},{59,28},{55,29},{49,30},{40,30},{34,29},{30,28},{25,26},{20,23},{16,20},{12,15},{9,9}}, color = {0,0,0}, thickness = 0.5),Polygon(points = {{88,0},{77,24},{68,17},{88,0}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}));

        equation 
          flange_a.tau = tau;
          flange_b.tau =  -tau;
        end Torque2;
        model LinearSpeedDependentTorque "Linear dependency of torque versus speed"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialTorque;
          parameter Modelica.SIunits.Torque tau_nominal "Nominal torque (if negative, torque is acting as load)";
          parameter Boolean TorqueDirection = true "Same direction of torque in both directions of rotation";
          parameter Modelica.SIunits.AngularVelocity w_nominal(min = Modelica.Constants.eps) "Nominal speed";
          Modelica.SIunits.AngularVelocity w "Angular velocity of flange with respect to support (= der(phi))";
          Modelica.SIunits.Torque tau "Accelerating torque acting at flange (= -flange.tau)";
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -100, -100},{100,100}}, color = {0,0,255})}), Documentation(info = "<HTML>
<p>
Model of torque, linearly dependent on angular velocity of flange.<br>
Parameter TorqueDirection chooses whether direction of torque is the same in both directions of rotation or not.
</p>
</HTML>"));

        equation 
          w = der(phi);
          tau =  -flange.tau;
          if TorqueDirection then
            tau = tau_nominal * abs(w / w_nominal);

          else
            tau = tau_nominal * w / w_nominal;

          end if;
        end LinearSpeedDependentTorque;
        model QuadraticSpeedDependentTorque "Quadratic dependency of torque versus speed"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialTorque;
          parameter Modelica.SIunits.Torque tau_nominal "Nominal torque (if negative, torque is acting as load)";
          parameter Boolean TorqueDirection = true "Same direction of torque in both directions of rotation";
          parameter Modelica.SIunits.AngularVelocity w_nominal(min = Modelica.Constants.eps) "Nominal speed";
          Modelica.SIunits.AngularVelocity w "Angular velocity of flange with respect to support (= der(phi))";
          Modelica.SIunits.Torque tau "Accelerating torque acting at flange (= -flange.tau)";
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -100, -100},{ -80, -98},{ -60, -92},{ -40, -82},{ -20, -68},{0, -50},{20, -28},{40, -2},{60,28},{80,62},{100,100}}, color = {0,0,255})}), Documentation(info = "<HTML>
<p>
Model of torque, quadratic dependent on angular velocity of flange.<br>
Parameter TorqueDirection chooses whether direction of torque is the same in both directions of rotation or not.
</p>
</HTML>"));

        equation 
          w = der(phi);
          tau =  -flange.tau;
          if TorqueDirection then
            tau = tau_nominal * (w / w_nominal) ^ 2;

          else
            tau = tau_nominal * smooth(1, if w >= 0 then (w / w_nominal) ^ 2 else  -(w / w_nominal) ^ 2);

          end if;
        end QuadraticSpeedDependentTorque;
        model ConstantTorque "Constant torque, not dependent on speed"
          extends Rotational.Interfaces.PartialTorque;
          parameter Modelica.SIunits.Torque tau_constant "Constant torque (if negative, torque is acting as load)";
          Modelica.SIunits.Torque tau "Accelerating torque acting at flange (= -flange.tau)";
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -98,0},{100,0}}, color = {0,0,255}),Text(extent = {{ -124, -16},{120, -40}}, lineColor = {0,0,0}, textString = "%tau_constant")}), Documentation(info = "<HTML>
<p>
Model of constant torque, not dependent on angular velocity of flange.<br>
Positive torque acts accelerating.
</p>
</HTML>"));

        equation 
          tau =  -flange.tau;
          tau = tau_constant;
        end ConstantTorque;
        model ConstantSpeed "Constant speed, not dependent on torque"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialTorque;
          Modelica.SIunits.AngularVelocity w "Angular velocity of flange with respect to support (= der(phi))";
          parameter Modelica.SIunits.AngularVelocity w_fixed "Fixed speed";
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{0, -100},{0,100}}, color = {0,0,255}),Text(extent = {{ -116, -16},{128, -40}}, lineColor = {0,0,0}, textString = "%w_fixed")}), Documentation(info = "<HTML>
<p>
Model of <b>fixed</b> angular verlocity of flange, not dependent on torque.
</p>
</HTML>"));

        equation 
          w = der(phi);
          w = w_fixed;
        end ConstantSpeed;
        model TorqueStep "Constant torque, not dependent on speed"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialTorque;
          parameter Modelica.SIunits.Torque stepTorque(start = 1) "Height of torque step (if negative, torque is acting as load)";
          parameter Modelica.SIunits.Torque offsetTorque(start = 0) "Offset of torque";
          parameter Modelica.SIunits.Time startTime = 0 "Torque = offset for time < startTime";
          Modelica.SIunits.Torque tau "Accelerating torque acting at flange (= -flange.tau)";
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -80, -60},{0, -60},{0,60},{80,60}}, color = {0,0,255}),Text(extent = {{0, -40},{100, -60}}, lineColor = {0,0,255}, textString = "time")}), Documentation(info = "<HTML>
<p>
Model of a torque step at time .<br>
Positive torque acts accelerating.
</p>
</HTML>"));

        equation 
          tau =  -flange.tau;
          tau = offsetTorque + (if time < startTime then 0 else stepTorque);
        end TorqueStep;
      end Sources;
      package Sensors "Sensors to measure variables in 1D rotational mechanical components"
        extends Modelica.Icons.Library2;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -56, -61},{ -56, -81}}, color = {0,0,0}),Line(points = {{ -36, -61},{ -36, -81}}, color = {0,0,0}),Line(points = {{ -16, -61},{ -16, -81}}, color = {0,0,0}),Rectangle(extent = {{ -76, -81},{64, -1}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{4, -61},{4, -81}}, color = {0,0,0}),Line(points = {{24, -61},{24, -81}}, color = {0,0,0}),Line(points = {{44, -61},{44, -81}}, color = {0,0,0}),Polygon(points = {{ -6, -61},{ -16, -37},{4, -37},{ -6, -61}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -6, -21},{ -6, -37}}, color = {0,0,0}),Line(points = {{ -76, -21},{ -6, -21}}, color = {0,0,0}),Line(points = {{ -56, -61},{ -56, -81}}, color = {0,0,0}),Line(points = {{ -36, -61},{ -36, -81}}, color = {0,0,0}),Line(points = {{ -16, -61},{ -16, -81}}, color = {0,0,0})}), Documentation(info = "<html>
<p>
This package contains ideal sensor components that provide
the connector variables as signals for further processing with the
Modelica.Blocks library.
</p>
</html>"));
        model AngleSensor "Ideal sensor to measure the absolute flange angle"
          extends Rotational.Interfaces.PartialAbsoluteSensor;
          Modelica.Blocks.Interfaces.RealOutput phi "Absolute angle of flange" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>
Measures the <b>absolute angle phi</b> of a flange in an ideal
way and provides the result as output signal <b>phi</b>
(to be further processed with blocks of the Modelica.Blocks library).
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{70, -30},{120, -70}}, lineColor = {0,0,0}, textString = "phi")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          phi = flange.phi;
        end AngleSensor;
        model SpeedSensor "Ideal sensor to measure the absolute flange angular velocity"
          extends Rotational.Interfaces.PartialAbsoluteSensor;
          Modelica.Blocks.Interfaces.RealOutput w "Absolute angular velocity of flange" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>
Measures the <b>absolute angular velocity w</b> of a flange in an ideal
way and provides the result as output signal <b>w</b>
(to be further processed with blocks of the Modelica.Blocks library).
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{70, -30},{120, -70}}, lineColor = {0,0,0}, textString = "w")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          w = der(flange.phi);
        end SpeedSensor;
        model AccSensor "Ideal sensor to measure the absolute flange angular acceleration"
          extends Rotational.Interfaces.PartialAbsoluteSensor;
          SI.AngularVelocity w "Absolute angular velocity of flange";
          Modelica.Blocks.Interfaces.RealOutput a "Absolute angular acceleration of flange" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>
Measures the <b>absolute angular acceleration a</b> of a flange in an ideal
way and provides the result as output signal <b>a</b> (to be further processed with
blocks of the Modelica.Blocks library).
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{70, -30},{120, -70}}, lineColor = {0,0,0}, textString = "a")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          w = der(flange.phi);
          a = der(w);
        end AccSensor;
        model RelAngleSensor "Ideal sensor to measure the relative angle between two flanges"
          extends Rotational.Interfaces.PartialRelativeSensor;
          Modelica.Blocks.Interfaces.RealOutput phi_rel "Relative angle between two flanges (= flange_b.phi - flange_a.phi)" annotation(Placement(transformation(origin = {0, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          annotation(Documentation(info = "<html>
<p>
Measures the <b>relative angle phi_rel</b> between two flanges
in an ideal way and provides the result as output signal <b>phi_rel</b>
(to be further processed with blocks of the Modelica.Blocks library).
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{20, -84},{160, -114}}, lineColor = {0,0,0}, textString = "phi_rel"),Line(points = {{0, -100},{0, -70}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          phi_rel = flange_b.phi - flange_a.phi;
          0 = flange_a.tau;
        end RelAngleSensor;
        model RelSpeedSensor "Ideal sensor to measure the relative angular velocity between two flanges"
          extends Rotational.Interfaces.PartialRelativeSensor;
          SI.Angle phi_rel "Relative angle between two flanges (flange_b.phi - flange_a.phi)";
          Modelica.Blocks.Interfaces.RealOutput w_rel "Relative angular velocity between two flanges (= der(flange_b.phi) - der(flange_a.phi))" annotation(Placement(transformation(origin = {0, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          annotation(Documentation(info = "<html>
<p>
Measures the <b>relative angular velocity w_rel</b> between two flanges
in an ideal way and provides the result as output signal <b>w_rel</b>
(to be further processed with blocks of the Modelica.Blocks library).
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{20, -88},{160, -118}}, lineColor = {0,0,0}, textString = "w_rel"),Line(points = {{0, -100},{0, -70}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          phi_rel = flange_b.phi - flange_a.phi;
          w_rel = der(phi_rel);
          0 = flange_a.tau;
        end RelSpeedSensor;
        model RelAccSensor "Ideal sensor to measure the relative angular acceleration between two flanges"
          extends Rotational.Interfaces.PartialRelativeSensor;
          SI.Angle phi_rel "Relative angle between two flanges (flange_b.phi - flange_a.phi)";
          SI.AngularVelocity w_rel "Relative angular velocity between two flanges";
          Modelica.Blocks.Interfaces.RealOutput a_rel "Relative angular acceleration between two flanges" annotation(Placement(transformation(origin = {0, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          annotation(Documentation(info = "<html>
<p>
Measures the <b>relative angular acceleration a_rel</b> between two flanges
in an ideal way and provides the result as output signal <b>a_rel</b>
(to be further processed with blocks of the Modelica.Blocks library).
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{20, -86},{160, -116}}, lineColor = {0,0,0}, textString = "a_rel"),Line(points = {{0, -100},{0, -70}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          phi_rel = flange_b.phi - flange_a.phi;
          w_rel = der(phi_rel);
          a_rel = der(w_rel);
          0 = flange_a.tau;
        end RelAccSensor;
        model TorqueSensor "Ideal sensor to measure the torque between two flanges (= flange_a.tau)"
          extends Rotational.Interfaces.PartialRelativeSensor;
          Modelica.Blocks.Interfaces.RealOutput tau "Torque in flange flange_a and flange_b (tau = flange_a.tau = -flange_b.tau)" annotation(Placement(transformation(origin = { -80, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          annotation(Documentation(info = "<html>
<p>
Measures the <b>cut-torque between two flanges</b> in an ideal way
and provides the result as output signal <b>tau</b>
(to be further processed with blocks of the Modelica.Blocks library).
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -50, -80},{50, -120}}, lineColor = {0,0,0}, textString = "tau"),Line(points = {{ -80, -100},{ -80,0}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          flange_a.phi = flange_b.phi;
          flange_a.tau = tau;
        end TorqueSensor;
        model PowerSensor "Ideal sensor to measure the power between two flanges (= flange_a.tau*der(flange_a.phi))"
          extends Rotational.Interfaces.PartialRelativeSensor;
          Modelica.Blocks.Interfaces.RealOutput power "Power in flange flange_a" annotation(Placement(transformation(origin = { -80, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          annotation(Documentation(info = "<html>
<p>
Measures the <b>power between two flanges</b> in an ideal way
and provides the result as output signal <b>power</b>
(to be further processed with blocks of the Modelica.Blocks library).
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -50, -80},{100, -120}}, lineColor = {0,0,0}, textString = "power"),Line(points = {{ -80, -100},{ -80,0}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          flange_a.phi = flange_b.phi;
          power = flange_a.tau * der(flange_a.phi);
        end PowerSensor;
      end Sensors;
      package Interfaces "Connectors and partial models for 1D rotational mechanical components"
        extends Modelica.Icons.Library;
        annotation(Documentation(info = "<html>
<p>
This package contains connectors and partial models for 1-dim.
rotational mechanical components. The components of this package can
only be used as basic building elements for models.
</p>

</html>
"));
        connector Flange_a "1-dim. rotational flange of a shaft (filled square icon)"
          SI.Angle phi "Absolute rotation angle of flange";
          flow SI.Torque tau "Cut torque in the flange";
          annotation(defaultComponentName = "flange_a", Documentation(info = "<html>
<p>
This is a connector for 1-dim. rotational mechanical systems and models
the mechanical flange of a shaft. The following variables are defined in this connector:
</p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"> <b>phi</b></td>
      <td valign=\"top\"> Absolute rotation angle of theshaft flange in [rad] </td>
  </tr>
  <tr><td valign=\"top\"> <b>tau</b></td>
      <td valign=\"top\"> Cut-torque in the shaft flange in [Nm] </td>
  </tr>
</table>

<p>
There is a second connector for flanges: Flange_b. The connectors
Flange_a and Flange_b are completely identical. There is only a difference
in the icons, in order to easier identify a flange variable in a diagram.
For a discussion on the actual direction of the cut-torque tau and
of the rotation angle, see section
<a href=\"Modelica://Modelica.Mechanics.Rotational.UsersGuide.SignConventions\">Sign Conventions</a>
in the user's guide of Rotational.
</p>

<p>
If needed, the absolute angular velocity w and the
absolute angular acceleration a of the flange can be determined by
differentiation of the flange angle phi:
</p>
<pre>
     w = der(phi);    a = der(w)
</pre>

</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Ellipse(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -160,90},{40,50}}, lineColor = {0,0,0}, textString = "%name"),Ellipse(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,0}, fillColor = {135,135,135}, fillPattern = FillPattern.Solid)}));
        end Flange_a;
        connector Flange_b "1-dim. rotational flange of a shaft (non-filled square icon)"
          SI.Angle phi "Absolute rotation angle of flange";
          flow SI.Torque tau "Cut torque in the flange";
          annotation(defaultComponentName = "flange_b", Documentation(info = "<html>
<p>
This is a connector for 1-dim. rotational mechanical systems and models
the mechanical flange of a shaft. The following variables are defined in this connector:
</p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"> <b>phi</b></td>
      <td valign=\"top\"> Absolute rotation angle of the shaft flange in [rad] </td>
  </tr>
  <tr><td valign=\"top\"> <b>tau</b></td>
      <td valign=\"top\"> Cut-torque in the shaft flange in [Nm] </td>
  </tr>
</table>

<p>
There is a second connector for flanges: Flange_a. The connectors
Flange_a and Flange_b are completely identical. There is only a difference
in the icons, in order to easier identify a flange variable in a diagram.
For a discussion on the actual direction of the cut-torque tau and
of the rotation angle, see section
<a href=\"Modelica://Modelica.Mechanics.Rotational.UsersGuide.SignConventions\">Sign Conventions</a>
in the user's guide of Rotational.
</p>

<p>
If needed, the absolute angular velocity w and the
absolute angular acceleration a of the flange can be determined by
differentiation of the flange angle phi:
</p>
<pre>
     w = der(phi);    a = der(w)
</pre>

</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Ellipse(extent = {{ -98,100},{102, -100}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Ellipse(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -40,90},{160,50}}, lineColor = {0,0,0}, textString = "%name")}));
        end Flange_b;
        connector Support "Support/housing of a 1-dim. rotational shaft"
          SI.Angle phi "Absolute rotation angle of the support/housing";
          flow SI.Torque tau "Reaction torque in the support/housing";
          annotation(Documentation(info = "<html>
<p>
This is a connector for 1-dim. rotational mechanical systems and models
the support or housing of a shaft. The following variables are defined in this connector:
</p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"> <b>phi</b></td>
      <td valign=\"top\"> Absolute rotation angle of the support/housing in [rad] </td>
  </tr>
  <tr><td valign=\"top\"> <b>tau</b></td>
      <td valign=\"top\"> Reaction torque in the support/housing in [Nm] </td>
  </tr>
</table>

<p>
The support connector is usually defined as conditional connector.
It is most convenient to utilize it
</p>

<ul>
<li> For models to be build graphically (i.e. the model is build up by drag-and-drop
     from elementary components):<br>
     <a href=\"Modelica://Modelica.Mechanics.Rotational.Interfaces.PartialOneFlangeAndSupport\">
     PartialOneFlangeAndSupport</a>,<br>
     <a href=\"Modelica://Modelica.Mechanics.Rotational.Interfaces.PartialTwoFlangesAndSupport\">
     PartialTwoFlangesAndSupport</a>, <br> &nbsp; </li>

<li> For models to be build textually (i.e. elementary models):<br>
     <a href=\"Modelica://Modelica.Mechanics.Rotational.Interfaces.PartialElementaryOneFlangeAndSupport\">
     PartialElementaryOneFlangeAndSupport</a>,<br>
     <a href=\"Modelica://Modelica.Mechanics.Rotational.Interfaces.PartialElementaryTwoFlangesAndSupport\">
     PartialElementaryTwoFlangesAndSupport</a>,<br>
     <a href=\"Modelica://Modelica.Mechanics.Rotational.Interfaces.PartialElementaryRotationalToTranslational\">
     PartialElementaryRotationalToTranslational</a>.</li>
</ul>

</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}, initialScale = 0.1), graphics = {Ellipse(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -150,150},{150, -150}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}, initialScale = 0.1), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -160,100},{40,60}}, lineColor = {0,0,0}, textString = "%name"),Ellipse(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,0}, fillColor = {135,135,135}, fillPattern = FillPattern.Solid)}));
        end Support;
        model InternalSupport "Adapter model to utilize conditional support connector"
          input Modelica.SIunits.Torque tau "External support torque (must be computed via torque balance in model where InternalSupport is used; = flange.tau)";
          Modelica.SIunits.Angle phi "External support angle (= flange.phi)";
          Flange_a flange "Internal support flange (must be connected to the conditional support connector for useSupport=true and to conditional fixed model for useSupport=false)" annotation(Placement(transformation(extent = {{ -10, -10},{10,10}})));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -20,20},{20, -20}}, lineColor = {135,135,135}, fillColor = {175,175,175}, fillPattern = FillPattern.Solid),Text(extent = {{ -200,80},{200,40}}, lineColor = {0,0,255}, textString = "%name")}), Documentation(info = "<html>
<p>
This is an adapter model to utilize a conditional support connector
in an elementary component, i.e., where the component equations are
defined textually:
</p>

<ul>
<li> If <i>useSupport = true</i>, the flange has to be connected to the conditional
     support connector.</li>
<li> If <i>useSupport = false</i>, the flange has to be connected to the conditional
     fixed model.</li>
</ul>

<p>
Variable <b>tau</b> is defined as <b>input</b> and must be provided when using
this component as a modifier (computed via a torque balance in
the model where InternalSupport is used). Usually, model InternalSupport is
utilized via the partial models:
</p>

<blockquote>
<a href=\"Modelica://Modelica.Mechanics.Rotational.Interfaces.PartialElementaryOneFlangeAndSupport\">
PartialElementaryOneFlangeAndSupport</a>,<br>
<a href=\"Modelica://Modelica.Mechanics.Rotational.Interfaces.PartialElementaryTwoFlangesAndSupport\">
PartialElementaryTwoFlangesAndSupport</a>,<br>
<a href=\"Modelica://Modelica.Mechanics.Rotational.Interfaces.PartialElementaryRotationalToTranslational\">
PartialElementaryRotationalToTranslational</a>.</li>
</blockquote>

<p>
Note, the support angle can always be accessed as internalSupport.phi, and
the support torque can always be accessed as internalSupport.tau.
</p>


</html>"));

        equation 
          flange.tau = tau;
          flange.phi = phi;
        end InternalSupport;
        partial model PartialTwoFlanges "Partial model for a component with two rotational 1-dim. shaft flanges"
          Flange_a flange_a "Flange of left shaft" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Flange_b flange_b "Flange of right shaft" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>
This is a 1-dim. rotational component with two flanges.
It is used e.g. to build up parts of a drive train consisting
of several components.
</p>

</html>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));
        end PartialTwoFlanges;
        partial model PartialOneFlangeAndSupport "Partial model for a component with one rotational 1-dim. shaft flange and a support used for graphical modeling, i.e., the model is build up by drag-and-drop from elementary components"
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
          Flange_b flange "Flange of shaft" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Support support if useSupport "Support/housing of component" annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}})));
          annotation(Documentation(info = "<html>
<p>
This is a 1-dim. rotational component with one flange and a support/housing.
It is used e.g. to build up parts of a drive train graphically consisting
of several components.
</p>

<p>
If <i>useSupport=true</i>, the support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupport=false</i>, the support connector is conditionally disabled
and instead the component is internally fixed to ground.
</p>

</html>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -38, -98},{ -6, -96}}, lineColor = {95,95,95}, textString = "(if useSupport)"),Text(extent = {{21, -95},{61, -96}}, lineColor = {95,95,95}, textString = "(if not useSupport)")}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(visible = not useSupport, points = {{ -50, -120},{ -30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -120},{ -10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -10, -120},{10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{10, -120},{30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -100},{30, -100}}, color = {0,0,0})}));
        protected 
          Support internalSupport "Internal support/housing of component (either connected to support, if useSupport=true, or connected to fixed, if useSupport=false)" annotation(Placement(transformation(extent = {{ -3, -83},{3, -77}})));
          Components.Fixed fixed if not useSupport "Fixed support/housing, if not useSupport" annotation(Placement(transformation(extent = {{10, -94},{30, -74}})));

        equation 
          connect(support,internalSupport) annotation(Line(points = {{0, -100},{0, -80}}, color = {0,0,0}, smooth = Smooth.None));
          connect(internalSupport,fixed.flange) annotation(Line(points = {{0, -80},{20, -80},{20, -84}}, color = {0,0,0}, smooth = Smooth.None));
        end PartialOneFlangeAndSupport;
        partial model PartialTwoFlangesAndSupport "Partial model for a component with two rotational 1-dim. shaft flanges and a support used for graphical modeling, i.e., the model is build up by drag-and-drop from elementary components"
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
          Flange_a flange_a "Flange of left shaft" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Flange_b flange_b "Flange of right shaft" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Support support if useSupport "Support/housing of component" annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}})));
          annotation(Documentation(info = "<html>
<p>
This is a 1-dim. rotational component with two flanges and a support/housing.
It is used e.g. to build up parts of a drive train graphically consisting
of several components.
</p>

<p>
If <i>useSupport=true</i>, the support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupport=false</i>, the support connector is conditionally disabled
and instead the component is internally fixed to ground.
</p>

</html>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -38, -98},{ -6, -96}}, lineColor = {95,95,95}, textString = "(if useSupport)"),Text(extent = {{24, -97},{64, -98}}, lineColor = {95,95,95}, textString = "(if not useSupport)")}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(visible = not useSupport, points = {{ -50, -120},{ -30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -120},{ -10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -10, -120},{10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{10, -120},{30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -100},{30, -100}}, color = {0,0,0})}));
        protected 
          Support internalSupport "Internal support/housing of component (either connected to support, if useSupport=true, or connected to fixed, if useSupport=false)" annotation(Placement(transformation(extent = {{ -3, -83},{3, -77}})));
          Components.Fixed fixed if not useSupport "Fixed support/housing, if not useSupport" annotation(Placement(transformation(extent = {{10, -97},{30, -77}})));

        equation 
          connect(support,internalSupport) annotation(Line(points = {{0, -100},{0, -80}}, color = {0,0,0}, smooth = Smooth.None));
          connect(internalSupport,fixed.flange) annotation(Line(points = {{0, -80},{20, -80},{20, -87}}, color = {0,0,0}, smooth = Smooth.None));
        end PartialTwoFlangesAndSupport;
        partial model PartialCompliant "Partial model for the compliant connection of two rotational 1-dim. shaft flanges"
          Modelica.SIunits.Angle phi_rel(start = 0) "Relative rotation angle (= flange_b.phi - flange_a.phi)";
          Modelica.SIunits.Torque tau "Torque between flanges (= flange_b.tau)";
          Flange_a flange_a "Left flange of compliant 1-dim. rotational component" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Flange_b flange_b "Right flange of compliant 1-dim. rotational component" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>
This is a 1-dim. rotational component with a compliant connection of two
rotational 1-dim. flanges where inertial effects between the two
flanges are neglected. The basic assumption is that the cut-torques
of the two flanges sum-up to zero, i.e., they have the same absolute value
but opposite sign: flange_a.tau + flange_b.tau = 0. This base class
is used to built up force elements such as springs, dampers, friction.
</p>

</html>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));

        equation 
          phi_rel = flange_b.phi - flange_a.phi;
          flange_b.tau = tau;
          flange_a.tau =  -tau;
        end PartialCompliant;
        partial model PartialCompliantWithRelativeStates "Partial model for the compliant connection of two rotational 1-dim. shaft flanges where the relative angle and speed are used as preferred states"
          Modelica.SIunits.Angle phi_rel(start = 0, stateSelect = stateSelect, nominal = phi_nominal) "Relative rotation angle (= flange_b.phi - flange_a.phi)";
          Modelica.SIunits.AngularVelocity w_rel(start = 0, stateSelect = stateSelect) "Relative angular velocity (= der(phi_rel))";
          Modelica.SIunits.AngularAcceleration a_rel(start = 0) "Relative angular acceleration (= der(w_rel))";
          Modelica.SIunits.Torque tau "Torque between flanges (= flange_b.tau)";
          Flange_a flange_a "Left flange of compliant 1-dim. rotational component" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Flange_b flange_b "Right flange of compliant 1-dim. rotational component" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          parameter SI.Angle phi_nominal(displayUnit = "rad") = 0.0001 "Nominal value of phi_rel (used for scaling)" annotation(Dialog(tab = "Advanced"));
          parameter StateSelect stateSelect = StateSelect.prefer "Priority to use phi_rel and w_rel as states" annotation(HideResult = true, Dialog(tab = "Advanced"));
          annotation(Documentation(info = "<html>
<p>
This is a 1-dim. rotational component with a compliant connection of two
rotational 1-dim. flanges where inertial effects between the two
flanges are neglected. The basic assumption is that the cut-torques
of the two flanges sum-up to zero, i.e., they have the same absolute value
but opposite sign: flange_a.tau + flange_b.tau = 0. This base class
is used to built up force elements such as springs, dampers, friction.
</p>

<p>
The relative angle and the relative speed are defined as preferred states.
The reason is that for some drive trains, such as drive
trains in vehicles, the absolute angle is quickly increasing during operation.
Numerically, it is better to use relative angles between drive train components
because they remain in a limited size. For this reason, StateSelect.prefer
is set for the relative angle of this component.
</p>

<p>
In order to improve the numerics, a nominal value for the relative angle
can be provided via parameter <b>phi_nominal</b> in the Advanced menu.
The default ist 1e-4 rad since relative angles are usually
in this order and the step size control of an integrator would be
practically switched off, if a default of 1 rad would be used.
This nominal value might also be computed from other values, such
as \"phi_nominal = tau_nominal / c\" for a rotational spring, if tau_nominal
and c are more meaningful for the user.
</p>

</html>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));

        equation 
          phi_rel = flange_b.phi - flange_a.phi;
          w_rel = der(phi_rel);
          a_rel = der(w_rel);
          flange_b.tau = tau;
          flange_a.tau =  -tau;
        end PartialCompliantWithRelativeStates;
        partial model PartialElementaryOneFlangeAndSupport "Obsolete partial model. Use PartialElementaryOneFlangeAndSupport2."
          extends Modelica.Icons.ObsoleteModel;
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
          Flange_b flange "Flange of shaft" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Support support if useSupport "Support/housing of component" annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}})));
          annotation(Documentation(info = "<html>
<p>
This is a 1-dim. rotational component with one flange and a support/housing.
It is used to build up elementary components of a drive train with
equations in the text layer.
</p>

<p>
If <i>useSupport=true</i>, the support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupport=false</i>, the support connector is conditionally disabled
and instead the component is internally fixed to ground.
</p>

</html>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{25, -97},{65, -98}}, lineColor = {95,95,95}, textString = "(if not useSupport)"),Text(extent = {{ -38, -98},{ -6, -96}}, lineColor = {95,95,95}, textString = "(if useSupport)")}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(visible = not useSupport, points = {{ -50, -120},{ -30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -120},{ -10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -10, -120},{10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{10, -120},{30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -100},{30, -100}}, color = {0,0,0})}));
        protected 
          Rotational.Interfaces.InternalSupport internalSupport(tau =  -flange.tau) "Internal support/housing of component as a model with connector flange (flange is either connected to support, if useSupport=true, or connected to fixed, if useSupport=false)" annotation(Placement(transformation(extent = {{ -10, -90},{10, -70}})));
          Rotational.Components.Fixed fixed if not useSupport "Fixed support/housing, if not useSupport" annotation(Placement(transformation(extent = {{10, -96},{30, -76}})));

        equation 
          connect(internalSupport.flange,support) annotation(Line(points = {{0, -80},{0, -100}}, color = {0,0,0}, smooth = Smooth.None));
          connect(internalSupport.flange,fixed.flange) annotation(Line(points = {{0, -80},{20, -80},{20, -86}}, color = {0,0,0}, smooth = Smooth.None));
        end PartialElementaryOneFlangeAndSupport;
        partial model PartialElementaryOneFlangeAndSupport2 "Partial model for a component with one rotational 1-dim. shaft flange and a support used for textual modeling, i.e., for elementary models"
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
          Flange_b flange "Flange of shaft" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Support support(phi = phi_support, tau =  -flange.tau) if useSupport "Support/housing of component" annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}})));
        protected 
          Modelica.SIunits.Angle phi_support "Absolute angle of support flange";

        equation 
if not useSupport then
                      phi_support = 0;
          end if;
          annotation(Documentation(info = "<html>
<p>
This is a 1-dim. rotational component with one flange and a support/housing.
It is used to build up elementary components of a drive train with
equations in the text layer.
</p>

<p>
If <i>useSupport=true</i>, the support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupport=false</i>, the support connector is conditionally disabled
and instead the component is internally fixed to ground.
</p>

</html>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(visible = not useSupport, points = {{ -50, -120},{ -30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -120},{ -10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -10, -120},{10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{10, -120},{30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -100},{30, -100}}, color = {0,0,0})}));
        end PartialElementaryOneFlangeAndSupport2;
        partial model PartialElementaryTwoFlangesAndSupport "Obsolete partial model. Use PartialElementaryTwoFlangesAndSupport2."
          extends Modelica.Icons.ObsoleteModel;
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
          Flange_a flange_a "Flange of left shaft" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Flange_b flange_b "Flange of right shaft" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Support support if useSupport "Support/housing of component" annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}})));
          annotation(Documentation(info = "<html>
<p>
This is a 1-dim. rotational component with two flanges and a support/housing.
It is used to build up elementary components of a drive train with
equations in the text layer.
</p>

<p>
If <i>useSupport=true</i>, the support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupport=false</i>, the support connector is conditionally disabled
and instead the component is internally fixed to ground.
</p>

</html>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{24, -97},{64, -98}}, lineColor = {95,95,95}, textString = "(if not useSupport)"),Text(extent = {{ -38, -98},{ -6, -96}}, lineColor = {95,95,95}, textString = "(if useSupport)")}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(visible = not useSupport, points = {{ -50, -120},{ -30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -120},{ -10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -10, -120},{10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{10, -120},{30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -100},{30, -100}}, color = {0,0,0})}));
        protected 
          Rotational.Interfaces.InternalSupport internalSupport(tau =  -flange_a.tau - flange_b.tau) "Internal support/housing of component as a model with connector flange (flange is either connected to support, if useSupport=true, or connected to fixed, if useSupport=false)" annotation(Placement(transformation(extent = {{ -10, -90},{10, -70}})));
          Rotational.Components.Fixed fixed if not useSupport "Fixed support/housing, if not useSupport" annotation(Placement(transformation(extent = {{10, -97},{30, -77}})));

        equation 
          connect(internalSupport.flange,support) annotation(Line(points = {{0, -80},{0, -100}}, color = {0,0,0}, smooth = Smooth.None));
          connect(internalSupport.flange,fixed.flange) annotation(Line(points = {{0, -80},{20, -80},{20, -87}}, color = {0,0,0}, smooth = Smooth.None));
        end PartialElementaryTwoFlangesAndSupport;
        partial model PartialElementaryTwoFlangesAndSupport2 "Partial model for a component with two rotational 1-dim. shaft flanges and a support used for textual modeling, i.e., for elementary models"
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
          Flange_a flange_a "Flange of left shaft" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Flange_b flange_b "Flange of right shaft" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Support support(phi = phi_support, tau =  -flange_a.tau - flange_b.tau) if useSupport "Support/housing of component" annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}})));
        protected 
          Modelica.SIunits.Angle phi_support "Absolute angle of support flange";

        equation 
if not useSupport then
                      phi_support = 0;
          end if;
          annotation(Documentation(info = "<html>
<p>
This is a 1-dim. rotational component with two flanges and a support/housing.
It is used to build up elementary components of a drive train with
equations in the text layer.
</p>

<p>
If <i>useSupport=true</i>, the support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupport=false</i>, the support connector is conditionally disabled
and instead the component is internally fixed to ground.
</p>

</html>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(visible = not useSupport, points = {{ -50, -120},{ -30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -120},{ -10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -10, -120},{10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{10, -120},{30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -100},{30, -100}}, color = {0,0,0})}));
        end PartialElementaryTwoFlangesAndSupport2;
        partial model PartialElementaryRotationalToTranslational "Partial model to transform rotational into translational motion"
          parameter Boolean useSupportR = false "= true, if rotational support flange enabled, otherwise implicitly grounded" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
          parameter Boolean useSupportT = false "= true, if translational support flange enabled, otherwise implicitly grounded" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
          Rotational.Interfaces.Flange_a flangeR "Flange of rotational shaft" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Modelica.Mechanics.Translational.Interfaces.Flange_b flangeT "Flange of translational rod" annotation(Placement(transformation(extent = {{90,10},{110, -10}}, rotation = 0)));
          Rotational.Interfaces.Support supportR if useSupportR "Rotational support/housing of component" annotation(Placement(transformation(extent = {{ -110, -110},{ -90, -90}}, rotation = 0), iconTransformation(extent = {{ -110, -110},{ -90, -90}})));
          Translational.Interfaces.Support supportT if useSupportT "Translational support/housing of component" annotation(Placement(transformation(extent = {{110, -110},{90, -90}}, rotation = 0), iconTransformation(extent = {{90, -110},{110, -90}})));
          annotation(Documentation(info = "<html>

<p>
This is a 1-dim. rotational component with
</p>

<ul>
<li> one rotational flange, </li>
<li> one rotational support/housing, </li>
<li> one translational flange, and </li>
<li> one translatinal support/housing </li>
</ul>

<p>
This model is used to build up elementary components of a drive train
transforming rotational into translational motion with
equations in the text layer.
</p>

<p>
If <i>useSupportR=true</i>, the rotational support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupportR=false</i>, the rotational support connector is conditionally disabled
and instead the rotational part is internally fixed to ground.<br>
If <i>useSupportT=true</i>, the translational support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupportT=false</i>, the translational support connector is conditionally disabled
and instead the translational part is internally fixed to ground.
</p>


</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(visible = not useSupportT, points = {{85, -110},{95, -100}}, color = {0,0,0}),Line(visible = not useSupportT, points = {{95, -110},{105, -100}}, color = {0,0,0}),Line(visible = not useSupportT, points = {{105, -110},{115, -100}}, color = {0,0,0}),Line(visible = not useSupportT, points = {{85, -100},{115, -100}}, color = {0,0,0}),Line(visible = not useSupportR, points = {{ -115, -110},{ -105, -100}}, color = {0,0,0}),Line(visible = not useSupportR, points = {{ -105, -110},{ -95, -100}}, color = {0,0,0}),Line(visible = not useSupportR, points = {{ -95, -110},{ -85, -100}}, color = {0,0,0}),Line(visible = not useSupportR, points = {{ -115, -100},{ -85, -100}}, color = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
        protected 
          Rotational.Interfaces.InternalSupport internalSupportR(tau =  -flangeR.tau) annotation(Placement(transformation(extent = {{ -110, -90},{ -90, -70}})));
          Translational.Interfaces.InternalSupport internalSupportT(f =  -flangeT.f) annotation(Placement(transformation(extent = {{90, -90},{110, -70}})));
          Rotational.Components.Fixed fixedR if not useSupportR annotation(Placement(transformation(extent = {{ -90, -90},{ -70, -70}})));
          Translational.Components.Fixed fixedT if not useSupportT annotation(Placement(transformation(extent = {{70, -90},{90, -70}})));

        equation 
          connect(internalSupportR.flange,supportR) annotation(Line(points = {{ -100, -80},{ -100, -100}}, color = {0,0,0}, smooth = Smooth.None));
          connect(internalSupportR.flange,fixedR.flange) annotation(Line(points = {{ -100, -80},{ -80, -80}}, color = {0,0,0}, smooth = Smooth.None));
          connect(fixedT.flange,internalSupportT.flange) annotation(Line(points = {{80, -80},{100, -80}}, color = {0,127,0}, smooth = Smooth.None));
          connect(internalSupportT.flange,supportT) annotation(Line(points = {{100, -80},{100, -100}}, color = {0,127,0}, smooth = Smooth.None));
        end PartialElementaryRotationalToTranslational;
        partial model PartialTorque "Partial model of a torque acting at the flange (accelerates the flange)"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryOneFlangeAndSupport2;
          Modelica.SIunits.Angle phi "Angle of flange with respect to support (= flange.phi - support.phi)";
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -96,96},{96, -96}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0, -62},{0, -100}}, color = {0,0,0}),Line(points = {{ -92,0},{ -76,36},{ -54,62},{ -30,80},{ -14,88},{10,92},{26,90},{46,80},{64,62}}, color = {0,0,0}),Text(extent = {{ -150,140},{150,100}}, lineColor = {0,0,255}, textString = "%name"),Polygon(points = {{94,16},{80,74},{50,52},{94,16}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -58, -82},{ -42, -68},{ -20, -56},{0, -54},{18, -56},{34, -62},{44, -72},{54, -82},{60, -94}}, color = {0,0,0}),Polygon(points = {{ -65, -98},{ -46, -80},{ -58, -72},{ -65, -98}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(visible = not useSupport, points = {{ -50, -120},{ -30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -120},{ -10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -10, -120},{10, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{10, -120},{30, -100}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -30, -100},{30, -100}}, color = {0,0,0})}), Documentation(info = "<HTML>
<p>
Partial model of torque that accelerates the flange.
</p>

<p>
If <i>useSupport=true</i>, the support connector is conditionally enabled
and needs to be connected.<br>
If <i>useSupport=false</i>, the support connector is conditionally disabled
and instead the component is internally fixed to ground.
</p>

</html>"));

        equation 
          phi = flange.phi - phi_support;
        end PartialTorque;
        partial model PartialAbsoluteSensor "Partial model to measure a single absolute flange variable"
          Flange_a flange "Flange of shaft from which sensor information shall be measured" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>
This is a partial model of a 1-dim. rotational component with one flange of a shaft
in order to measure an absolute kinematic quantity in the flange
and to provide the measured signal as output signal for further processing
with the blocks of package Modelica.Blocks.
</p>

</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{70,0},{100,0}}, color = {0,0,127}),Text(extent = {{150,80},{ -150,120}}, textString = "%name", lineColor = {0,0,255}),Ellipse(extent = {{ -70,70},{70, -70}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,70},{0,40}}, color = {0,0,0}),Line(points = {{22.9,32.8},{40.2,57.3}}, color = {0,0,0}),Line(points = {{ -22.9,32.8},{ -40.2,57.3}}, color = {0,0,0}),Line(points = {{37.6,13.7},{65.8,23.9}}, color = {0,0,0}),Line(points = {{ -37.6,13.7},{ -65.8,23.9}}, color = {0,0,0}),Line(points = {{0,0},{9.02,28.6}}, color = {0,0,0}),Polygon(points = {{ -0.48,31.6},{18,26},{18,57.2},{ -0.48,31.6}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -5,5},{5, -5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          0 = flange.tau;
        end PartialAbsoluteSensor;
        partial model PartialRelativeSensor "Partial model to measure a single relative variable between two flanges"
          Flange_a flange_a "Left flange of shaft" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Flange_b flange_b "Right flange of shaft" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>
This is a partial model for 1-dim. rotational components with two rigidly connected
flanges in order to measure relative kinematic quantities
between the two flanges or the cut-torque in the flange and
to provide the measured signal as output signal for further processing
with the blocks of package Modelica.Blocks.
</p>

</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{70,0},{90,0}}, color = {0,0,0}),Text(extent = {{ -150,73},{150,113}}, textString = "%name", lineColor = {0,0,255}),Ellipse(extent = {{ -70,70},{70, -70}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,70},{0,40}}, color = {0,0,0}),Line(points = {{22.9,32.8},{40.2,57.3}}, color = {0,0,0}),Line(points = {{ -22.9,32.8},{ -40.2,57.3}}, color = {0,0,0}),Line(points = {{37.6,13.7},{65.8,23.9}}, color = {0,0,0}),Line(points = {{ -37.6,13.7},{ -65.8,23.9}}, color = {0,0,0}),Line(points = {{0,0},{9.02,28.6}}, color = {0,0,0}),Polygon(points = {{ -0.48,31.6},{18,26},{18,57.2},{ -0.48,31.6}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -5,5},{5, -5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          0 = flange_a.tau + flange_b.tau;
        end PartialRelativeSensor;
        partial model PartialFriction "Partial model of Coulomb friction elements"
          annotation(Documentation(info = "<html>
<p>
Basic model for Coulomb friction that models the stuck phase in a reliable way.
</p>
</html>"));
          parameter SI.AngularVelocity w_small = 10000000000.0 "Relative angular velocity near to zero if jumps due to a reinit(..) of the velocity can occur (set to low value only if such impulses can occur)" annotation(Dialog(tab = "Advanced"));
          SI.AngularVelocity w_relfric "Relative angular velocity between frictional surfaces";
          SI.AngularAcceleration a_relfric "Relative angular acceleration between frictional surfaces";
          SI.Torque tau0 "Friction torque for w=0 and forward sliding";
          SI.Torque tau0_max "Maximum friction torque for w=0 and locked";
          Boolean free "true, if frictional element is not active";
          Real sa(final unit = "1") "Path parameter of friction characteristic tau = f(a_relfric)";
          Boolean startForward(start = false, fixed = true) "true, if w_rel=0 and start of forward sliding";
          Boolean startBackward(start = false, fixed = true) "true, if w_rel=0 and start of backward sliding";
          Boolean locked(start = false) "true, if w_rel=0 and not sliding";
          constant Integer Unknown = 3 "Value of mode is not known";
          constant Integer Free = 2 "Element is not active";
          constant Integer Forward = 1 "w_rel > 0 (forward sliding)";
          constant Integer Stuck = 0 "w_rel = 0 (forward sliding, locked or backward sliding)";
          constant Integer Backward =  -1 "w_rel < 0 (backward sliding)";
          Integer mode(final min = Backward, final max = Unknown, start = Unknown, fixed = true);
        protected 
          constant SI.AngularAcceleration unitAngularAcceleration = 1 annotation(HideResult = true);
          constant SI.Torque unitTorque = 1 annotation(HideResult = true);

        equation 
          startForward = pre(mode) == Stuck and (sa > tau0_max / unitTorque or pre(startForward) and sa > tau0 / unitTorque) or pre(mode) == Backward and w_relfric > w_small or initial() and w_relfric > 0;
          startBackward = pre(mode) == Stuck and (sa <  -tau0_max / unitTorque or pre(startBackward) and sa <  -tau0 / unitTorque) or pre(mode) == Forward and w_relfric <  -w_small or initial() and w_relfric < 0;
          locked = not free and not (pre(mode) == Forward or startForward or pre(mode) == Backward or startBackward);
          a_relfric / unitAngularAcceleration = if locked then 0 else if free then sa else if startForward then sa - tau0_max / unitTorque else if startBackward then sa + tau0_max / unitTorque else if pre(mode) == Forward then sa - tau0_max / unitTorque else sa + tau0_max / unitTorque;
          mode = if free then Free else if (pre(mode) == Forward or pre(mode) == Free or startForward) and w_relfric > 0 then Forward else if (pre(mode) == Backward or pre(mode) == Free or startBackward) and w_relfric < 0 then Backward else Stuck;
        end PartialFriction;
      end Interfaces;
    end Rotational;
  end Mechanics;
  package Math "Library of mathematical functions (e.g., sin, cos) and of functions operating on vectors and matrices"
    import SI = Modelica.SIunits;
    extends Modelica.Icons.Library2;
    annotation(Invisible = true, Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -59, -9},{42, -56}}, lineColor = {0,0,0}, textString = "f(x)")}), Documentation(info = "<HTML>
<p>
This package contains <b>basic mathematical functions</b> (such as sin(..)),
as well as functions operating on <b>vectors</b> and <b>matrices</b>.
</p>

<dl>
<dt><b>Main Author:</b>
<dd><a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a><br>
    Deutsches Zentrum f&uuml;r Luft und Raumfahrt e.V. (DLR)<br>
    Institut f&uuml;r Robotik und Mechatronik<br>
    Postfach 1116<br>
    D-82230 Wessling<br>
    Germany<br>
    email: <A HREF=\"mailto:Martin.Otter@dlr.de\">Martin.Otter@dlr.de</A><br>
</dl>

<p>
Copyright &copy; 1998-2009, Modelica Association and DLR.
</p>
<p>
<i>This Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p><br>
</HTML>
", revisions = "<html>
<ul>
<li><i>October 21, 2002</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>
       and <a href=\"http://www.robotic.dlr.de/Christian.Schweiger/\">Christian Schweiger</a>:<br>
       Function tempInterpol2 added.</li>
<li><i>Oct. 24, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Icons for icon and diagram level introduced.</li>
<li><i>June 30, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized.</li>
</ul>

</html>"));
    package Vectors "Library of functions operating on vectors"
      extends Modelica.Icons.Library;
      annotation(preferedView = "info", Documentation(info = "<HTML>
<h4>Library content</h4>
<p>
This library provides functions operating on vectors:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><i>Function</i></th>
      <th><i>Description</i></th>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Math.Vectors.isEqual\">isEqual</a>(v1, v2)</td>
      <td valign=\"top\">Determines whether two vectors have the same size and elements</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Math.Vectors.norm\">norm</a>(v,p)</td>
      <td valign=\"top\">p-norm of vector v</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Math.Vectors.length\">length</a>(v)</td>
      <td valign=\"top\">Length of vector v (= norm(v,2), but inlined and therefore usable in
          symbolic manipulations) </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Math.Vectors.normalize\">normalize</a>(v)</td>
      <td valign=\"top\">Return normalized vector such that length = 1 and prevent
          zero-division for zero vector</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Math.Vectors.reverse\">reverse</a>(v)</td>
      <td valign=\"top\">Reverse vector elements</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Math.Vectors.sort\">sort</a>(v)</td>
      <td valign=\"top\">Sort elements of vector in ascending or descending order</td>
  </tr>
</table>
<h4>See also</h4>
<a href=\"Modelica://Modelica.Math.Matrices\">Matrices</a>
</HTML>"));
      function isEqual "Determine if two Real vectors are numerically identical"
        extends Modelica.Icons.Function;
        input Real v1[:] "First vector";
        input Real v2[:] "Second vector (may have different length as v1";
        input Real eps(min = 0) = 0 "Two elements e1 and e2 of the two vectors are identical if abs(e1-e2) <= eps";
        output Boolean result "= true, if vectors have the same length and the same elements";
        annotation(Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Vectors.<b>isEqual</b>(v1, v2);
Vectors.<b>isEqual</b>(v1, v2, eps=0);
</pre></blockquote>
<h4>Description</h4>
<p>
The function call \"<code>Vectors.isEqual(v1, v2)</code>\" returns <b>true</b>,
if the two Real vectors v1 and v2 have the same dimensions and
the same elements. Otherwise the function
returns <b>false</b>. Two elements e1 and e2 of the two vectors
are checked on equality by the test \"abs(e1-e2) &le; eps\", where \"eps\"
can be provided as third argument of the function. Default is \"eps = 0\".
</p>Modelica.Utilities.Strings.isEqual
<h4>Example</h4>
<blockquote><pre>
  Real v1[3] = {1, 2, 3};
  Real v2[3] = {1, 2, 3, 4};
  Real v3[3] = {1, 2, 3.0001};
  Boolean result;
<b>algorithm</b>
  result := Vectors.isEqual(v1,v2);     // = <b>false</b>
  result := Vectors.isEqual(v1,v3);     // = <b>false</b>
  result := Vectors.isEqual(v1,v1);     // = <b>true</b>
  result := Vectors.isEqual(v1,v3,0.1); // = <b>true</b>
</pre></blockquote>
<h4>See also</h4>
<a href=\"Modelica://Modelica.Math.Matrices.isEqual\">Matrices.isEqual</a>,
<a href=\"Modelica://Modelica.Utilities.Strings.isEqual\">Strings.isEqual</a>
</HTML>"));
      protected 
        Integer n = size(v1, 1) "Dimension of vector v1";
        Integer i = 1;
      algorithm 
        result:=false;
        if size(v2, 1) == n then 
                result:=true;
        while (i <= n) loop
                  if abs(v1[i] - v2[i]) > eps then 
                    result:=false;
          i:=n;

          else 
          end if;
          i:=i + 1;

        end while;

        else 
        end if;
      end isEqual;
      function norm "Return the p-norm of a vector"
        extends Modelica.Icons.Function;
        input Real v[:] "Vector";
        input Real p(min = 1) = 2 "Type of p-norm (often used: 1, 2, or Modelica.Constants.inf)";
        output Real result "p-norm of vector v";
        annotation(Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Vectors.<b>norm</b>(v);
Vectors.<b>norm</b>(v,p=2);   // 1 &le; p &le; &#8734;
</pre></blockquote>
<h4>Description</h4>
<p>
The function call \"<code>Vectors.<b>norm</b>(v)</code>\" returns the
<b>Euclidean norm</b> \"<code>sqrt(v*v)</code>\" of vector v.
With the optional
second argument \"p\", any other p-norm can be computed:
</p>
<center>
<IMG SRC=\"../Images/Math/vectorNorm.png\" ALT=\"function Vectors.norm\">
</center>
<p>
Besides the Euclidean norm (p=2), also the 1-norm and the
infinity-norm are sometimes used:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>1-norm</b></td>
      <td valign=\"top\">= sum(abs(v))</td>
      <td valign=\"top\"><b>norm</b>(v,1)</td>
  </tr>
  <tr><td valign=\"top\"><b>2-norm</b></td>
      <td valign=\"top\">= sqrt(v*v)</td>
      <td valign=\"top\"><b>norm</b>(v) or <b>norm</b>(v,2)</td>
  </tr>
  <tr><td valign=\"top\"><b>infinity-norm</b></td>
      <td valign=\"top\">= max(abs(v))</td>
      <td valign=\"top\"><b>norm</b>(v,Modelica.Constants.<b>inf</b>)</td>
  </tr>
</table>
<p>
Note, for any vector norm the following inequality holds:
</p>
<blockquote><pre>
<b>norm</b>(v1+v2,p) &le; <b>norm</b>(v1,p) + <b>norm</b>(v2,p)
</pre></blockquote>
<h4>Example</h4>
<blockquote><pre>
  v = {2, -4, -2, -1};
  <b>norm</b>(v,1);    // = 9
  <b>norm</b>(v,2);    // = 5
  <b>norm</b>(v);      // = 5
  <b>norm</b>(v,10.5); // = 4.00052597412635
  <b>norm</b>(v,Modelica.Constants.inf);  // = 4
</pre></blockquote>
<h4>See also</h4>
<a href=\"Modelica://Modelica.Math.Matrices.norm\">Matrices.norm</a>
</HTML>"));
      algorithm 
        if p == 2 then 
                result:=sqrt(v * v);
        elseif p == Modelica.Constants.inf then
          result:=max(abs(v));

        elseif p == 1 then
          result:=sum(abs(v));


        else         result:=sum(abs(v[i]) ^ p for i in 1:size(v, 1)) ^ (1 / p);

        end if;
      end norm;
      function length "Return length of a vectorReturn length of a vector (better as norm(), if further symbolic processing is performed)"
        extends Modelica.Icons.Function;
        input Real v[:] "Vector";
        output Real result "Length of vector v";
        annotation(Inline = true, Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Vectors.<b>length</b>(v);
</pre></blockquote>
<h4>Description</h4>
<p>
The function call \"<code>Vectors.<b>length</b>(v)</code>\" returns the
<b>Euclidean length</b> \"<code>sqrt(v*v)</code>\" of vector v.
The function call is equivalent to Vectors.norm(v). The advantage of
length(v) over norm(v)\"is that function length(..) is implemented
in one statement and therefore the function is usually automatically
inlined. Further symbolic processing is therefore possible, which is
not the case with function norm(..).
</p>
<h4>Example</h4>
<blockquote><pre>
  v = {2, -4, -2, -1};
  <b>length</b>(v);  // = 5
</pre></blockquote>
<h4>See also</h4>
<a href=\"Modelica://Modelica.Math.Vectors.norm\">Vectors.norm</a>
</html>"));
      algorithm 
        result:=sqrt(v * v);
      end length;
      function normalize "Return normalized vector such that length = 1Return normalized vector such that length = 1 and prevent zero-division for zero vector"
        extends Modelica.Icons.Function;
        input Real v[:] "Vector";
        input Real eps = 100 * Modelica.Constants.eps "if |v| < eps then result = v/eps";
        output Real result[size(v, 1)] "Input vector v normalized to length=1";
        annotation(Inline = true, Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Vectors.<b>normalize</b>(v);
Vectors.<b>normalize</b>(v,eps=100*Modelica.Constants.eps);
</pre></blockquote>
<h4>Description</h4>
<p>
The function call \"<code>Vectors.<b>normalize</b>(v)</code>\" returns the
<b>unit vector</b> \"<code>v/length(v)</code>\" of vector v.
If length(v) is close to zero (more precisely, if length(v) &lt; eps),
v/eps is returned in order to avoid
a division by zero. For many applications this is useful, because
often the unit vector <b>e</b> = <b>v</b>/length(<b>v</b>) is used to compute
a vector x*<b>e</b>, where the scalar x is in the order of length(<b>v</b>),
i.e., x*<b>e</b> is small, when length(<b>v</b>) is small and then
it is fine to replace <b>e</b> by <b>v</b> to avoid a division by zero.
</p>
<p>
Since the function is implemented in one statement,
it is usually inlined and therefore symbolic processing is
possible.
</p>
<h4>Example</h4>
<blockquote><pre>
  <b>normalize</b>({1,2,3});  // = {0.267, 0.534, 0.802}
  <b>normalize</b>({0,0,0});  // = {0,0,0}
</pre></blockquote>
<h4>See also</h4>
<a href=\"Modelica://Modelica.Math.Vectors.length\">Vectors.length</a>
</html>"));
      algorithm 
        result:=smooth(0, if length(v) >= eps then v / length(v) else v / eps);
      end normalize;
      function reverse "Reverse vector elements (e.g. v[1] becomes last element)"
        extends Modelica.Icons.Function;
        input Real v[:] "Vector";
        output Real result[size(v, 1)] "Elements of vector v in reversed order";
        annotation(Inline = true, Documentation(info = "<html>
<h4>Syntax</h4>
<blockquote><pre>
Vectors.<b>reverse</b>(v);
</pre></blockquote>
<h4>Description</h4>
<p>
The function call \"<code>Vectors.<b>reverse</b>(v)</code>\" returns the
vector elements in reverse order.
</p>
<h4>Example</h4>
<blockquote><pre>
  <b>reverse</b>({1,2,3,4});  // = {4,3,2,1}
</pre></blockquote>
</html>"));
      algorithm 
        result:=array(v[end - i + 1] for i in 1:size(v, 1));
      end reverse;
      function sort "Sort elements of vector in ascending or descending order"
        extends Modelica.Icons.Function;
        input Real v[:] "Vector to be sorted";
        input Boolean ascending = true "= true if ascending order, otherwise descending order";
        output Real sorted_v[size(v, 1)] = v "Sorted vector";
        output Integer indices[size(v, 1)] = 1:size(v, 1) "sorted_v = v[indices]";
        annotation(Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
           sorted_v = Vectors.<b>sort</b>(v);
(sorted_v, indices) = Vectors.<b>sort</b>(v, ascending=true);
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>sort</b>(..) sorts a Real vector v
in ascending order and returns the result in sorted_v.
If the optional argument \"ascending\" is <b>false</b>, the vector
is sorted in descending order. In the optional second
output argument the indices of the sorted vector with respect
to the original vector are given, such that sorted_v = v[indices].
</p>
<h4>Example</h4>
<blockquote><pre>
  (v2, i2) := Vectors.sort({-1, 8, 3, 6, 2});
       -> v2 = {-1, 2, 3, 6, 8}
          i2 = {1, 5, 3, 4, 2}
</pre></blockquote>
</HTML>"));
      protected 
        Integer gap;
        Integer i;
        Integer j;
        Real wv;
        Integer wi;
        Integer nv = size(v, 1);
        Boolean swap;
      algorithm 
        gap:=div(nv, 2);
        while (gap > 0) loop
                  i:=gap;
          while (i < nv) loop
                      j:=i - gap;
            if j >= 0 then 
                        if ascending then 
                        swap:=sorted_v[j + 1] > sorted_v[j + gap + 1];

            else             swap:=sorted_v[j + 1] < sorted_v[j + gap + 1];

            end if;

            else             swap:=false;

            end if;
            while (swap) loop
                          wv:=sorted_v[j + 1];
              wi:=indices[j + 1];
              sorted_v[j + 1]:=sorted_v[j + gap + 1];
              sorted_v[j + gap + 1]:=wv;
              indices[j + 1]:=indices[j + gap + 1];
              indices[j + gap + 1]:=wi;
              j:=j - gap;
              if j >= 0 then 
                            if ascending then 
                            swap:=sorted_v[j + 1] > sorted_v[j + gap + 1];

              else               swap:=sorted_v[j + 1] < sorted_v[j + gap + 1];

              end if;

              else               swap:=false;

              end if;

            end while;
            i:=i + 1;

          end while;
          gap:=div(gap, 2);

        end while;
      end sort;
    end Vectors;
    package Matrices "Library of functions operating on matrices"
      extends Modelica.Icons.Library;
      annotation(Documentation(info = "<HTML>
<h4>Library content</h4>
<p>
This library provides functions operating on matrices:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><th><i>Function</i></th>
      <th><i>Description</i></th>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Math.Matrices.isEqual\">isEqual</a>(M1, M2)</td>
      <td valign=\"top\">Determines whether two matrices have the same size and elements</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Math.Matrices.norm\">norm</a>(A)</td>
      <td valign=\"top\">1-, 2- and infinity-norm of matrix A</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Math.Matrices.sort\">sort</a>(M)</td>
      <td valign=\"top\">Sort rows or columns of matrix in ascending or descending order</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Math.Matrices.solve\">solve</a>(A,b)</td>
      <td valign=\"top\">Solve real system of linear equations A*x=b with a b vector</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Math.Matrices.solve2\">solve2</a>(A,B)</td>
      <td valign=\"top\">Solve real system of linear equations A*X=B with a B matrix</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Math.Matrices.leastSquares\">leastSquares</a>(A,b)</td>
      <td valign=\"top\">Solve overdetermined or underdetermined real system of <br>
          linear equations A*x=b in a least squares sense</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Math.Matrices.equalityLeastSquares\">equalityLeastSquares</a>(A,a,B,b)</td>
      <td valign=\"top\">Solve a linear equality constrained least squares problem:<br>
          min|A*x-a|^2 subject to B*x=b</td>
  </tr>
  <tr><td valign=\"top\">(LU,p,info) = <a href=\"Modelica://Modelica.Math.Matrices.LU\">LU</a>(A)</td>
      <td valign=\"top\">LU decomposition of square or rectangular matrix</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Math.Matrices.LU_solve\">LU_solve</a>(LU,p,b)</td>
      <td valign=\"top\">Solve real system of linear equations P*L*U*x=b with a<br>
          b vector and an LU decomposition from \"LU(..)\"</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Math.Matrices.LU_solve2\">LU_solve2</a>(LU,p,B)</td>
      <td valign=\"top\">Solve real system of linear equations P*L*U*X=B with a<br>
          B matrix and an LU decomposition from \"LU(..)\"</td>
  </tr>
  <tr><td valign=\"top\">(Q,R,p) = <a href=\"Modelica://Modelica.Math.Matrices.QR\">QR</a>(A)</td>
      <td valign=\"top\"> QR decomposition with column pivoting of rectangular matrix (Q*R = A[:,p]) </td>
  </tr>
  <tr><td valign=\"top\">eval = <a href=\"Modelica://Modelica.Math.Matrices.eigenValues\">eigenValues</a>(A)<br>
          (eval,evec) = <a href=\"Modelica://Modelica.Math.Matrices.eigenValues\">eigenValues</a>(A)</td>
      <td valign=\"top\"> Compute eigenvalues and optionally eigenvectors<br>
           for a real, nonsymmetric matrix </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Math.Matrices.eigenValueMatrix\">eigenValueMatrix</a>(eigen)</td>
      <td valign=\"top\"> Return real valued block diagonal matrix J of eigenvalues of
            matrix A (A=V*J*Vinv) </td>
  </tr>
  <tr><td valign=\"top\">sigma = <a href=\"Modelica://Modelica.Math.Matrices.singularValues\">singularValues</a>(A)<br>
      (sigma,U,VT) = <a href=\"Modelica://Modelica.Math.Matrices.singularValues\">singularValues</a>(A)</td>
      <td valign=\"top\"> Compute singular values and optionally left and right singular vectors </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Math.Matrices.det\">det</a>(A)</td>
      <td valign=\"top\"> Determinant of a matrix (do <b>not</b> use; use rank(..))</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Math.Matrices.inv\">inv</a>(A)</td>
      <td valign=\"top\"> Inverse of a matrix </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Math.Matrices.rank\">rank</a>(A)</td>
      <td valign=\"top\"> Rank of a matrix </td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Math.Matrices.balance\">balance</a>(A)</td>
      <td valign=\"top\">Balance a square matrix to improve the condition</td>
  </tr>
  <tr><td valign=\"top\"><a href=\"Modelica://Modelica.Math.Matrices.exp\">exp</a>(A)</td>
      <td valign=\"top\"> Compute the exponential of a matrix by adaptive Taylor series<br>
           expansion with scaling and balancing</td>
  </tr>
  <tr><td valign=\"top\">(P, G) = <a href=\"Modelica://Modelica.Math.Matrices.integralExp\">integralExp</a>(A,B)</td>
      <td valign=\"top\"> Compute the exponential of a matrix and its integral</td>
  </tr>
  <tr><td valign=\"top\">(P, G, GT) = <a href=\"Modelica://Modelica.Math.Matrices.integralExpT\">integralExpT</a>(A,B)</td>
      <td valign=\"top\"> Compute the exponential of a matrix and two integrals</td>
  </tr>
</table>

<p>
Most functions are solely an interface to the external LAPACK library
(<a href=\"http://www.netlib.org/lapack\">http://www.netlib.org/lapack</a>).
The details of this library are described in:
</p>

<dl>
<dt>Anderson E., Bai Z., Bischof C., Blackford S., Demmel J., Dongarra J.,
    Du Croz J., Greenbaum A., Hammarling S., McKenney A., and Sorensen D.:</dt>
<dd> <b>Lapack Users' Guide</b>.
     Third Edition, SIAM, 1999.</dd>
</dl>

<h4>See also</h4>
<a href=\"Modelica://Modelica.Math.Vectors\">Vectors</a>

</HTML>
"));
      function isEqual "Compare whether two Real matrices are identical"
        extends Modelica.Icons.Function;
        input Real M1[:,:] "First matrix";
        input Real M2[:,:] "Second matrix (may have different size as M1";
        input Real eps(min = 0) = 0 "Two elements e1 and e2 of the two matrices are identical if abs(e1-e2) <= eps";
        output Boolean result "= true, if matrices have the same size and the same elements";
        annotation(Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Matrices.<b>isEqual</b>(M1, M2);
Matrices.<b>isEqual</b>(M1, M2, eps=0);
</pre></blockquote>
<h4>Description</h4>
<p>
The function call \"<code>Matrices.isEqual(M1, M2)</code>\" returns <b>true</b>,
if the two Real matrices M1 and M2 have the same dimensions and
the same elements. Otherwise the function
returns <b>false</b>. Two elements e1 and e2 of the two matrices
are checked on equality by the test \"abs(e1-e2) &le; eps\", where \"eps\"
can be provided as third argument of the function. Default is \"eps = 0\".
</p>
<h4>Example</h4>
<blockquote><pre>
  Real A1[2,2] = [1,2; 3,4];
  Real A2[3,2] = [1,2; 3,4; 5,6];
  Real A3[2,2] = [1,2, 3,4.0001];
  Boolean result;
<b>algorithm</b>
  result := Matrices.isEqual(M1,M2);     // = <b>false</b>
  result := Matrices.isEqual(M1,M3);     // = <b>false</b>
  result := Matrices.isEqual(M1,M1);     // = <b>true</b>
  result := Matrices.isEqual(M1,M3,0.1); // = <b>true</b>
</pre></blockquote>
<h4>See also</h4>
<a href=\"Modelica://Modelica.Math.Vectors.isEqual\">Vectors.isEqual</a>,
<a href=\"Modelica://Modelica.Utilities.Strings.isEqual\">Strings.isEqual</a>
</HTML>"));
      protected 
        Integer nrow = size(M1, 1) "Number of rows of matrix M1";
        Integer ncol = size(M1, 2) "Number of columns of matrix M1";
        Integer i = 1;
        Integer j;
      algorithm 
        result:=false;
        if size(M2, 1) == nrow and size(M2, 2) == ncol then 
                result:=true;
        while (i <= nrow) loop
                  j:=1;
          while (j <= ncol) loop
                      if abs(M1[i,j] - M2[i,j]) > eps then 
                        result:=false;
            i:=nrow;
            j:=ncol;

            else 
            end if;
            j:=j + 1;

          end while;
          i:=i + 1;

        end while;

        else 
        end if;
      end isEqual;
      function norm "Returns the norm of a matrix"
        extends Modelica.Icons.Function;
        input Real A[:,:] "Input matrix";
        input Real p(min = 1) = 2 "Type of p-norm (only allowed: 1, 2 or Modelica.Constants.inf)";
        output Real result = 0.0 "p-norm of matrix A";
        annotation(Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Matrices.<b>norm</b>(A);
Matrices.<b>norm</b>(A, p=2);
</pre></blockquote>
<h4>Description</h4>
<p>
The function call \"<code>Matrices.norm(A)</code>\" returns the
2-norm of matrix A, i.e., the largest singular value of A.<br>
The function call \"<code>Matrices.norm(A, p)</code>\" returns the
p-norm of matrix A. The only allowed values for p are</p>
<ul>
<li> \"p=1\": the largest column sum of A</li>
<li> \"p=2\": the largest singular value of A</li>
<li> \"p=Modelica.Constants.inf\": the largest row sum of A</li>
</ul>
<p>
Note, for any matrices A1, A2 the following inequality holds:
</p>
<blockquote><pre>
Matrices.<b>norm</b>(A1+A2,p) &le; Matrices.<b>norm</b>(A1,p) + Matrices.<b>norm</b>(A2,p)
</pre></blockquote>
<p>
Note, for any matrix A and vector v the following inequality holds:
</p>
<blockquote><pre>
Vectors.<b>norm</b>(A*v,p) &le; Matrices.<b>norm</b>(A,p)*Vectors.<b>norm</b>(A,p)
</pre></blockquote>
</HTML>"));
      algorithm 
        if p == 1 then 
                for i in 1:size(A, 2) loop
                  result:=max(result, sum(abs(A[:,i])));

        end for;
        elseif p == 2 then
          result:=max(singularValues(A));

        elseif p == Modelica.Constants.inf then
          for i in 1:size(A, 1) loop
                      result:=max(result, sum(abs(A[i,:])));

          end for;


        else         assert(false, "Optional argument \"p\" of function \"norm\" must be
1, 2 or Modelica.Constants.inf");

        end if;
      end norm;
      function sort "Sort rows or columns of matrix in ascending or descending order"
        extends Modelica.Icons.Function;
        input Real M[:,:] "Matrix to be sorted";
        input Boolean sortRows = true "= true if rows are sorted, otherwise columns";
        input Boolean ascending = true "= true if ascending order, otherwise descending order";
        output Real sorted_M[size(M, 1),size(M, 2)] = M "Sorted matrix";
        output Integer indices[if sortRows then size(M, 1) else size(M, 2)] "sorted_M = if sortRows then M[indices,:] else M[:,indices]";
        annotation(Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
           sorted_M = Matrices.<b>sort</b>(M);
(sorted_M, indices) = Matrices.<b>sort</b>(M, sortRows=true, ascending=true);
</pre></blockquote>
<h4>Description</h4>
<p>
Function <b>sort</b>(..) sorts the rows of a Real matrix M
in ascending order and returns the result in sorted_M.
If the optional argument \"sortRows\" is <b>false</b>, the columns
of the matrix are sorted.
If the optional argument \"ascending\" is <b>false</b>, the rows or
columns are sorted in descending order. In the optional second
output argument, the indices of the sorted rows or columns with respect
to the original matrix are given, such that
</p>
<pre>
   sorted_M = <b>if</b> sortedRow <b>then</b> M[indices,:] <b>else</b> M[:,indices];
</pre>
<h4>Example</h4>
<blockquote><pre>
  (M2, i2) := Matrices.sort([2, 1,  0;
                             2, 0, -1]);
       -> M2 = [2, 0, -1;
                2, 1, 0 ];
          i2 = {2,1};
</pre></blockquote>
</HTML>"));
      protected 
        Integer gap;
        Integer i;
        Integer j;
        Real wM2[size(M, 2)];
        Integer wi;
        Integer nM1 = size(M, 1);
        Boolean swap;
        Real sorted_MT[size(M, 2),size(M, 1)];
        encapsulated function greater "Compare whether vector v1 > v2"
          import Modelica;
          extends Modelica.Icons.Function;
          import Modelica.Utilities.Types.Compare;
          input Real v1[:];
          input Real v2[size(v1, 1)];
          output Boolean result;
        protected 
          Integer n = size(v1, 1);
          Integer i = 1;
        algorithm 
          result:=false;
          while (i <= n) loop
                      if v1[i] > v2[i] then 
                        result:=true;
            i:=n;
            elseif v1[i] < v2[i] then
              i:=n;


            else 
            end if;
            i:=i + 1;

          end while;
        end greater;
        encapsulated function less "Compare whether vector v1 < v2"
          import Modelica;
          extends Modelica.Icons.Function;
          import Modelica.Utilities.Types.Compare;
          input Real v1[:];
          input Real v2[size(v1, 1)];
          output Boolean result;
        protected 
          Integer n = size(v1, 1);
          Integer i = 1;
        algorithm 
          result:=false;
          while (i <= n) loop
                      if v1[i] < v2[i] then 
                        result:=true;
            i:=n;
            elseif v1[i] > v2[i] then
              i:=n;


            else 
            end if;
            i:=i + 1;

          end while;
        end less;
      algorithm 
        if not sortRows then 
                (sorted_MT,indices):=sort(transpose(M), ascending = ascending);
        sorted_M:=transpose(sorted_MT);

        else         indices:=1:size(M, 1);
        gap:=div(nM1, 2);
        while (gap > 0) loop
                  i:=gap;
          while (i < nM1) loop
                      j:=i - gap;
            if j >= 0 then 
                        if ascending then 
                        swap:=greater(sorted_M[j + 1,:], sorted_M[j + gap + 1,:]);

            else             swap:=less(sorted_M[j + 1,:], sorted_M[j + gap + 1,:]);

            end if;

            else             swap:=false;

            end if;
            while (swap) loop
                          wM2:=sorted_M[j + 1,:];
              wi:=indices[j + 1];
              sorted_M[j + 1,:]:=sorted_M[j + gap + 1,:];
              sorted_M[j + gap + 1,:]:=wM2;
              indices[j + 1]:=indices[j + gap + 1];
              indices[j + gap + 1]:=wi;
              j:=j - gap;
              if j >= 0 then 
                            if ascending then 
                            swap:=greater(sorted_M[j + 1,:], sorted_M[j + gap + 1,:]);

              else               swap:=less(sorted_M[j + 1,:], sorted_M[j + gap + 1,:]);

              end if;

              else               swap:=false;

              end if;

            end while;
            i:=i + 1;

          end while;
          gap:=div(gap, 2);

        end while;

        end if;
      end sort;
      function solve "Solve real system of linear equations A*x=b with a b vector (Gaussian elemination with partial pivoting)"
        extends Modelica.Icons.Function;
        input Real A[:,size(A, 1)] "Matrix A of A*x = b";
        input Real b[size(A, 1)] "Vector b of A*x = b";
        output Real x[size(b, 1)] "Vector x such that A*x = b";
        annotation(Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Matrices.<b>solve</b>(A,b);
</pre></blockquote>
<h4>Description</h4>
<p>
This function call returns the
solution <b>x</b> of the linear system of equations
</p>
<blockquote>
<p>
<b>A</b>*<b>x</b> = <b>b</b>
</p>
</blockquote>
<p>
If a unique solution <b>x</b> does not exist (since <b>A</b> is singular),
an exception is raised.
</p>
<p>
Note, the solution is computed with the LAPACK function \"dgesv\",
i.e., by Gaussian elemination with partial pivoting.
</p>
<h4>Example</h4>
<blockquote><pre>
  Real A[3,3] = [1,2,3;
                 3,4,5;
                 2,1,4];
  Real b[3] = {10,22,12};
  Real x[3];
<b>algorithm</b>
  x := Matrices.solve(A,b);  // x = {3,2,1}
</pre></blockquote>
<h4>See also</h4>
<a href=\"Modelica://Modelica.Math.Matrices.LU\">Matrices.LU</a>,
<a href=\"Modelica://Modelica.Math.Matrices.LU_solve\">Matrices.LU_solve</a>
</HTML>"));
      protected 
        Integer info;
      algorithm 
        (x,info):=LAPACK.dgesv_vec(A, b);
        assert(info == 0, "Solving a linear system of equations with function
\"Matrices.solve\" is not possible, because the system has either
no or infinitely many solutions (A is singular).");
      end solve;
      function solve2 "Solve real system of linear equations A*X=B with a B matrix (Gaussian elemination with partial pivoting)"
        extends Modelica.Icons.Function;
        input Real A[:,size(A, 1)] "Matrix A of A*X = B";
        input Real B[size(A, 1),:] "Matrix B of A*X = B";
        output Real X[size(B, 1),size(B, 2)] "Matrix X such that A*X = B";
        annotation(Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Matrices.<b>solve2</b>(A,b);
</pre></blockquote>
<h4>Description</h4>
<p>
This function call returns the
solution <b>X</b> of the linear system of equations
</p>
<blockquote>
<p>
<b>A</b>*<b>X</b> = <b>B</b>
</p>
</blockquote>
<p>
If a unique solution <b>X</b> does not exist (since <b>A</b> is singular),
an exception is raised.
</p>
<p>
Note, the solution is computed with the LAPACK function \"dgesv\",
i.e., by Gaussian elemination with partial pivoting.
</p>
<h4>Example</h4>
<blockquote><pre>
  Real A[3,3] = [1,2,3;
                 3,4,5;
                 2,1,4];
  Real B[3,2] = [10, 20;
                 22, 44;
                 12, 24];
  Real X[3,2];
<b>algorithm</b>
  (LU, pivots) := Matrices.LU(A);
  X := Matrices.solve2(A, B1);  /* X = [3, 6;
                                        2, 4;
                                        1, 2] */
</pre></blockquote>

<h4>See also</h4>
<a href=\"Modelica://Modelica.Math.Matrices.LU\">Matrices.LU</a>,
<a href=\"Modelica://Modelica.Math.Matrices.LU_solve2\">Matrices.LU_solve2</a>
</HTML>"));
      protected 
        Integer info;
      algorithm 
        (X,info):=LAPACK.dgesv(A, B);
        assert(info == 0, "Solving a linear system of equations with function
\"Matrices.solve2\" is not possible, because the system has either
no or infinitely many solutions (A is singular).");
      end solve2;
      function leastSquares "Solve overdetermined or underdetermined real system of linear equations A*x=b in a least squares sense (A may be rank deficient)"
        extends Modelica.Icons.Function;
        input Real A[:,:] "Matrix A";
        input Real b[size(A, 1)] "Vector b";
        output Real x[size(A, 2)] "Vector x such that min|A*x-b|^2 if size(A,1) >= size(A,2) or min|x|^2 and A*x=b, if size(A,1) < size(A,2)";
        annotation(Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
x = Matrices.<b>leastSquares</b>(A,b);
</pre></blockquote>
<h4>Description</h4>
<p>
A linear system of equations A*x = b has no solutions or infinitely
many solutions if A is not square. Function \"leastSquares\" returns
a solution in a least squarse sense:
</p>
<pre>
  size(A,1) &gt; size(A,2):  returns x such that |A*x - b|^2 is a minimum
  size(A,1) = size(A,2):  returns x such that A*x = b
  size(A,1) &lt; size(A,2):  returns x such that |x|^2 is a minimum for all
                          vectors x that fulfill A*x = b
</pre>
<p>
Note, the solution is computed with the LAPACK function \"dgelsx\",
i.e., QR or LQ factorization of A with column pivoting.
If A does not have full rank,
the solution is not unique and from the infinitely many solutions
the one is selected that minimizes both |x|^2 and |A*x - b|^2.
</p>
</HTML>"));
      protected 
        Integer info;
        Integer rank;
        Real xx[max(size(A, 1), size(A, 2))];
      algorithm 
        (xx,info,rank):=LAPACK.dgelsx_vec(A, b, 100 * Modelica.Constants.eps);
        x:=xx[1:size(A, 2)];
        assert(info == 0, "Solving an overdetermined or underdetermined linear system of
equations with function \"Matrices.leastSquares\" failed.");
      end leastSquares;
      function equalityLeastSquares "Solve a linear equality constrained least squares problem"
        extends Modelica.Icons.Function;
        input Real A[:,:] "Minimize |A*x - a|^2";
        input Real a[size(A, 1)];
        input Real B[:,size(A, 2)] "subject to B*x=b";
        input Real b[size(B, 1)];
        output Real x[size(A, 2)] "solution vector";
        annotation(Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
x = Matrices.<b>equalityLeastSquares</b>(A,a,B,b);
</pre></blockquote>
<h4>Description</h4>
<p>
This function returns the
solution <b>x</b> of the linear equality-constrained least squares problem:
</p>
<blockquote>
<p>
min|<b>A</b>*<b>x</b> - <b>a</b>|^2 over <b>x</b>, subject to <b>B</b>*<b>x</b> = <b>b</b>
</p>
</blockquote>

<p>
It is required that the dimensions of A and B fulfill the following
relationship:
</p>

<blockquote>
size(B,1) &le; size(A,2) &le; size(A,1) + size(B,1)
</blockquote>

<p>
Note, the solution is computed with the LAPACK function \"dgglse\"
using the generalized RQ factorization under the assumptions that
B has full row rank (= size(B,1)) and the matrix [A;B] has
full column rank (= size(A,2)). In this case, the problem
has a unique solution.
</p>
</HTML>"));
      protected 
        Integer info;
      algorithm 
        assert(size(A, 2) >= size(B, 1) and size(A, 2) <= size(A, 1) + size(B, 1), "It is required that size(B,1) <= size(A,2) <= size(A,1) + size(B,1)\n" + "This relationship is not fulfilled, since the matrices are declared as:\n" + "  A[" + String(size(A, 1)) + "," + String(size(A, 2)) + "], B[" + String(size(B, 1)) + "," + String(size(B, 2)) + "]\n");
        (x,info):=LAPACK.dgglse_vec(A, a, B, b);
        assert(info == 0, "Solving a linear equality-constrained least squares problem
with function \"Matrices.equalityLeastSquares\" failed.");
      end equalityLeastSquares;
      function LU "LU decomposition of square or rectangular matrix"
        extends Modelica.Icons.Function;
        input Real A[:,:] "Square or rectangular matrix";
        output Real LU[size(A, 1),size(A, 2)] = A "L,U factors (used with LU_solve(..))";
        output Integer pivots[min(size(A, 1), size(A, 2))] "pivot indices (used with LU_solve(..))";
        output Integer info "Information";

        external "FORTRAN 77" dgetrf(size(A, 1),size(A, 2),LU,size(A, 1),pivots,info)         annotation(Library = "Lapack");         annotation(Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
(LU, pivots)       = Matrices.<b>LU</b>(A);
(LU, pivots, info) = Matrices.<b>LU</b>(A);
</pre></blockquote>
<h4>Description</h4>
<p>
This function call returns the
LU decomposition of a \"Real[m,n]\" matrix A, i.e.,
</p>
<blockquote>
<p>
<b>P</b>*<b>L</b>*<b>U</b> = <b>A</b>
</p>
</blockquote>
<p>
where <b>P</b> is a permutation matrix (implicitely
defined by vector <code>pivots</code>),
<b>L</b> is a lower triangular matrix with unit
diagonal elements (lower trapezoidal if m &gt; n), and
<b>U</b> is an upper triangular matrix (upper trapezoidal if m &lt; n).
Matrices <b>L</b> and <b>U</b> are stored in the returned
matrix <code>LU</code> (the diagonal of <b>L</b> is not stored).
With the companion function
<a href=\"Modelica://Modelica.Math.Matrices.LU_solve\">Matrices.LU_solve</a>,
this decomposition can be used to solve
linear systems (<b>P</b>*<b>L</b>*<b>U</b>)*<b>x</b> = <b>b</b> with different right
hand side vectors <b>b</b>. If a linear system of equations with
just one right hand side vector <b>b</b> shall be solved, it is
more convenient to just use the function
<a href=\"Modelica://Modelica.Math.Matrices.solve\">Matrices.solve</a>.
</p>
<p>
The optional third (Integer) output argument has the following meaning:
<table border=0 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\">info = 0:</td
      <td valign=\"top\">successful exit</td></tr>
  <tr><td valign=\"top\">info &gt; 0:</td>
      <td valign=\"top\">if info = i, U[i,i] is exactly zero. The factorization
          has been completed, <br>
          but the factor U is exactly
          singular, and division by zero will occur<br> if it is used
          to solve a system of equations.</td></tr>
</table>
</p>
<p>
The LU factorization is computed
with the LAPACK function \"dgetrf\",
i.e., by Gaussian elemination using partial pivoting
with row interchanges. Vector \"pivots\" are the
pivot indices, i.e., for 1 &le; i &le; min(m,n), row i of
matrix A was interchanged with row pivots[i].
</p>
<h4>Example</h4>
<blockquote><pre>
  Real A[3,3] = [1,2,3;
                 3,4,5;
                 2,1,4];
  Real b1[3] = {10,22,12};
  Real b2[3] = { 7,13,10};
  Real    LU[3,3];
  Integer pivots[3];
  Real    x1[3];
  Real    x2[3];
<b>algorithm</b>
  (LU, pivots) := Matrices.LU(A);
  x1 := Matrices.LU_solve(LU, pivots, b1);  // x1 = {3,2,1}
  x2 := Matrices.LU_solve(LU, pivots, b2);  // x2 = {1,0,2}
</pre></blockquote>
<h4>See also</h4>
<a href=\"Modelica://Modelica.Math.Matrices.LU_solve\">Matrices.LU_solve</a>,
<a href=\"Modelica://Modelica.Math.Matrices.solve\">Matrices.solve</a>,
</HTML>"));
      end LU;
      function LU_solve "Solve real system of linear equations P*L*U*x=b with a b vector and an LU decomposition (from LU(..))"
        extends Modelica.Icons.Function;
        input Real LU[:,size(LU, 1)] "L,U factors of Matrices.LU(..) for a square matrix";
        input Integer pivots[size(LU, 1)] "Pivots indices of Matrices.LU(..)";
        input Real b[size(LU, 1)] "Right hand side vector of P*L*U*x=b";
        output Real x[size(b, 1)] "Solution vector such that P*L*U*x = b";
        annotation(Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Matrices.<b>LU_solve</b>(LU, pivots, b);
</pre></blockquote>
<h4>Description</h4>
<p>
This function call returns the
solution <b>x</b> of the linear systems of equations
</p>
<blockquote>
<p>
<b>P</b>*<b>L</b>*<b>U</b>*<b>x</b> = <b>b</b>;
</p>
</blockquote>
<p>
where <b>P</b> is a permutation matrix (implicitely
defined by vector <code>pivots</code>),
<b>L</b> is a lower triangular matrix with unit
diagonal elements (lower trapezoidal if m &gt; n), and
<b>U</b> is an upper triangular matrix (upper trapezoidal if m &lt; n).
The matrices of this decomposition are computed with function
<a href=\"Modelica://Modelica.Math.Matrices.LU\">Matrices.LU</a> that
returns arguments <code>LU</code> and <code>pivots</code>
used as input arguments of <code>Matrices.LU_solve</code>.
With <code>Matrices.LU</code> and <code>Matrices.LU_solve</code>
it is possible to efficiently solve linear systems
with different right hand side vectors. If a linear system of equations with
just one right hand side vector shall be solved, it is
more convenient to just use the function
<a href=\"Modelica://Modelica.Math.Matrices.solve\">Matrices.solve</a>.
</p>
<p>
If a unique solution <b>x</b> does not exist (since the
LU decomposition is singular), an exception is raised.
</p>
<p>
The LU factorization is computed
with the LAPACK function \"dgetrf\",
i.e., by Gaussian elemination using partial pivoting
with row interchanges. Vector \"pivots\" are the
pivot indices, i.e., for 1 &le; i &le; min(m,n), row i of
matrix A was interchanged with row pivots[i].
</p>
<h4>Example</h4>
<blockquote><pre>
  Real A[3,3] = [1,2,3;
                 3,4,5;
                 2,1,4];
  Real b1[3] = {10,22,12};
  Real b2[3] = { 7,13,10};
  Real    LU[3,3];
  Integer pivots[3];
  Real    x1[3];
  Real    x2[3];
<b>algorithm</b>
  (LU, pivots) := Matrices.LU(A);
  x1 := Matrices.LU_solve(LU, pivots, b1);  // x1 = {3,2,1}
  x2 := Matrices.LU_solve(LU, pivots, b2);  // x2 = {1,0,2}
</pre></blockquote>
<h4>See also</h4>
<a href=\"Modelica://Modelica.Math.Matrices.LU\">Matrices.LU</a>,
<a href=\"Modelica://Modelica.Math.Matrices.solve\">Matrices.solve</a>,
</HTML>"));
      algorithm 
        for i in 1:size(LU, 1) loop
                  assert(LU[i,i] <> 0, "Solving a linear system of equations with function
\"Matrices.LU_solve\" is not possible, since the LU decomposition
is singular, i.e., no unique solution exists.");

        end for;
        x:=LAPACK.dgetrs_vec(LU, pivots, b);
      end LU_solve;
      function LU_solve2 "Solve real system of linear equations P*L*U*X=B with a B matrix and an LU decomposition (from LU(..))"
        extends Modelica.Icons.Function;
        input Real LU[:,size(LU, 1)] "L,U factors of Matrices.LU(..) for a square matrix";
        input Integer pivots[size(LU, 1)] "Pivots indices of Matrices.LU(..)";
        input Real B[size(LU, 1),:] "Right hand side matrix of P*L*U*X=B";
        output Real X[size(B, 1),size(B, 2)] "Solution matrix such that P*L*U*X = B";
        annotation(Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Matrices.<b>LU_solve</b>(LU, pivots, B);
</pre></blockquote>
<h4>Description</h4>
<p>
This function call returns the
solution <b>X</b> of the linear systems of equations
</p>
<blockquote>
<p>
<b>P</b>*<b>L</b>*<b>U</b>*<b>X</b> = <b>B</b>;
</p>
</blockquote>
<p>
where <b>P</b> is a permutation matrix (implicitely
defined by vector <code>pivots</code>),
<b>L</b> is a lower triangular matrix with unit
diagonal elements (lower trapezoidal if m &gt; n), and
<b>U</b> is an upper triangular matrix (upper trapezoidal if m &lt; n).
The matrices of this decomposition are computed with function
<a href=\"Modelica://Modelica.Math.Matrices.LU\">Matrices.LU</a> that
returns arguments <code>LU</code> and <code>pivots</code>
used as input arguments of <code>Matrices.LU_solve2</code>.
With <code>Matrices.LU</code> and <code>Matrices.LU_solve2</code>
it is possible to efficiently solve linear systems
with different right hand side <b>matrices</b>. If a linear system of equations with
just one right hand side matrix shall be solved, it is
more convenient to just use the function
<a href=\"Modelica://Modelica.Math.Matrices.solve2\">Matrices.solve2</a>.
</p>
<p>
If a unique solution <b>X</b> does not exist (since the
LU decomposition is singular), an exception is raised.
</p>
<p>
The LU factorization is computed
with the LAPACK function \"dgetrf\",
i.e., by Gaussian elemination using partial pivoting
with row interchanges. Vector \"pivots\" are the
pivot indices, i.e., for 1 &le; i &le; min(m,n), row i of
matrix A was interchanged with row pivots[i].
</p>
<h4>Example</h4>
<blockquote><pre>
  Real A[3,3] = [1,2,3;
                 3,4,5;
                 2,1,4];
  Real B1[3] = [10, 20;
                22, 44;
                12, 24];
  Real B2[3] = [ 7, 14;
                13, 26;
                10, 20];
  Real    LU[3,3];
  Integer pivots[3];
  Real    X1[3,2];
  Real    X2[3,2];
<b>algorithm</b>
  (LU, pivots) := Matrices.LU(A);
  X1 := Matrices.LU_solve2(LU, pivots, B1);  /* X1 = [3, 6;
                                                      2, 4;
                                                      1, 2] */
  X2 := Matrices.LU_solve2(LU, pivots, B2);  /* X2 = [1, 2;
                                                      0, 0;
                                                      2, 4] */
</pre></blockquote>
<h4>See also</h4>
<a href=\"Modelica://Modelica.Math.Matrices.LU\">Matrices.LU</a>,
<a href=\"Modelica://Modelica.Math.Matrices.solve2\">Matrices.solve2</a>,
</HTML>"));
      algorithm 
        for i in 1:size(LU, 1) loop
                  assert(LU[i,i] <> 0, "Solving a linear system of equations with function
\"Matrices.LU_solve\" is not possible, since the LU decomposition
is singular, i.e., no unique solution exists.");

        end for;
        X:=Modelica.Math.Matrices.LAPACK.dgetrs(LU, pivots, B);
      end LU_solve2;
      function QR "QR decomposition of a square matrix with column pivoting (A(:,p) = Q*R)"
        extends Modelica.Icons.Function;
        input Real A[:,:] "Rectangular matrix with size(A,1) >= size(A,2)";
        output Real Q[size(A, 1),size(A, 2)] "Rectangular matrix with orthonormal columns such that Q*R=A[:,p]";
        output Real R[size(A, 2),size(A, 2)] "Square upper triangular matrix";
        output Integer p[size(A, 2)] "Column permutation vector";
        annotation(Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
(Q,R,p) = Matrices.<b>QR</b>(A);
</pre></blockquote>
<h4>Description</h4>
<p>
This function returns the QR decomposition of
a rectangular matrix <b>A</b> (the number of columns of <b>A</b>
must be less than or equal to the number of rows):
</p>
<blockquote>
<p>
<b>Q</b>*<b>R</b> = <b>A</b>[:,<b>p</b>]
</p>
</blockquote>
<p>
where <b>Q</b> is a rectangular matrix that has orthonormal columns and
has the same size as A (<b>Q</b><sup>T</sup><b>Q</b>=<b>I</b>),
<b>R</b> is a square, upper triangular matrix and <b>p</b> is a permutation
vector. Matrix <b>R</b> has the following important properties:
</p>
<ul>
<li> The absolute value of a diagonal element of <b>R</b> is the largest
     value in this row, i.e.,
     abs(R[i,i]) &ge; abs(R[i,j]).</li>
<li> The diagonal elements of <b>R</b> are sorted according to size, such that
     the largest absolute value is abs(R[1,1]) and
     abs(R[i,i]) &ge; abs(R[j,j]) with i &lt; j. </li>
</ul>
<p>
This means that if abs(R[i,i]) &le; &epsilon; then abs(R[j,k]) &le; &epsilon;
for j &ge; i, i.e., the i-th row up to the last row of <b>R</b> have
small elements and can be treated as being zero.
This allows to, e.g., estimate the row-rank
of <b>R</b> (which is the same row-rank as <b>A</b>). Furthermore,
<b>R</b> can be partitioned in two parts
</p>
<blockquote>
<pre>
   <b>A</b>[:,<b>p</b>] = <b>Q</b> * [<b>R</b><sub>1</sub>, <b>R</b><sub>2</sub>;
                 <b>0</b>,  <b>0</b>]
</pre>
</blockquote>
<p>
where <b>R</b><sub>1</sub> is a regular, upper triangular matrix.
</p>
<p>
Note, the solution is computed with the LAPACK functions \"dgeqpf\"
and \"dorgqr\", i.e., by Housholder transformations with
column pivoting. If <b>Q</b> is not needed, the function may be
called as: <code>(,R,p) = QR(A)</code>.
</p>
<h4>Example</h4>
<blockquote><pre>
  Real A[3,3] = [1,2,3;
                 3,4,5;
                 2,1,4];
  Real R[3,3];
<b>algorithm</b>
  (,R) := Matrices.QR(A);  // R = [-7.07.., -4.24.., -3.67..;
                                    0     , -1.73.., -0.23..;
                                    0     ,  0     ,  0.65..];
</pre></blockquote>
</HTML>"));
      protected 
        Integer nrow = size(A, 1);
        Integer ncol = size(A, 2);
        Real tau[ncol];
      algorithm 
        assert(nrow >= ncol, "\nInput matrix A[" + String(nrow) + "," + String(ncol) + "] has more columns as rows.
This is not allowed when calling Modelica.Matrices.QR(A).");
        (Q,tau,p):=LAPACK.dgeqpf(A);
        R:=zeros(ncol, ncol);
        for i in 1:ncol loop
                  for j in i:ncol loop
                      R[i,j]:=Q[i,j];

          end for;

        end for;
        Q:=LAPACK.dorgqr(Q, tau);
      end QR;
      function eigenValues "Compute eigenvalues and eigenvectors for a real, nonsymmetric matrix"
        extends Modelica.Icons.Function;
        input Real A[:,size(A, 1)] "Matrix";
        output Real eigenvalues[size(A, 1),2] "Eigenvalues of matrix A (Re: first column, Im: second column)";
        output Real eigenvectors[size(A, 1),size(A, 2)] "Real-valued eigenvector matrix";
        annotation(Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
                eigenvalues = Matrices.<b>eigenValues</b>(A);
(eigenvalues, eigenvectors) = Matrices.<b>eigenValues</b>(A);
</pre></blockquote>
<h4>Description</h4>
<p>
This function call returns the eigenvalues and
optionally the (right) eigenvectors of a square matrix
<b>A</b>. The first column of \"eigenvalues\" contains the real and the
second column contains the imaginary part of the eigenvalues.
If the i-th eigenvalue has no imaginary part, then eigenvectors[:,i] is
the corresponding real eigenvector. If the i-th eigenvalue
has an imaginary part, then eigenvalues[i+1,:] is the conjugate complex
eigenvalue and eigenvectors[:,i] is the real and eigenvectors[:,i+1] is the
imaginary part of the eigenvector of the i-th eigenvalue.
With function
<a href=\"Modelica://Modelica.Math.Matrices.eigenValueMatrix\">Matrices.eigenValueMatrix</a>,
a real block diagonal matrix is constructed from the eigenvalues
such that
</p>
<blockquote>
<pre>
A = eigenvectors * eigenValueMatrix(eigenvalues) * inv(eigenvectors)
</pre>
</blockquote>
<p>
provided the eigenvector matrix \"eigenvectors\" can be inverted
(an inversion is possible, if all eigenvalues are different;
in some cases, an inversion is also possible if some eigenvalues are
the same).
</p>
<h4>Example</h4>
<blockquote><pre>
  Real A[3,3] = [1,2,3;
                 3,4,5;
                 2,1,4];
  Real eval;
<b>algorithm</b>
  eval := Matrices.eigenValues(A);  // eval = [-0.618, 0;
                                    //          8.0  , 0;
                                    //          1.618, 0];
</pre>
</blockquote>
<p>
i.e., matrix A has the 3 real eigenvalues -0.618, 8, 1.618.
</p>
</pre></blockquote>
<h4>See also</h4>
<a href=\"Modelica://Modelica.Math.Matrices.eigenValueMatrix\">Matrices.eigenValueMatrix</a>,
<a href=\"Modelica://Modelica.Math.Matrices.singularValues\">Matrices.singularValues</a>
</HTML>
"));
      protected 
        Integer info;
        Boolean onlyEigenvalues = false;
      algorithm 
        if size(A, 1) > 0 then 
                if onlyEigenvalues then 
                (eigenvalues[:,1],eigenvalues[:,2],info):=LAPACK.dgeev_eigenValues(A);
        eigenvectors:=zeros(size(A, 1), size(A, 1));

        else         (eigenvalues[:,1],eigenvalues[:,2],eigenvectors,info):=LAPACK.dgeev(A);

        end if;
        assert(info == 0, "Calculating the eigen values with function
\"Matrices.eigenvalues\" is not possible, since the
numerical algorithm does not converge.");

        else 
        end if;
      end eigenValues;
      function eigenValueMatrix "Return real valued block diagonal matrix J of eigenvalues of matrix A (A=V*J*Vinv)"
        extends Modelica.Icons.Function;
        input Real eigenValues[:,2] "Eigen values from function eigenValues(..) (Re: first column, Im: second column)";
        output Real J[size(eigenValues, 1),size(eigenValues, 1)] "Real valued block diagonal matrix with eigen values (Re: 1x1 block, Im: 2x2 block)";
        annotation(Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Matrices.<b>eigenValueMatrix</b>(eigenvalues);
</pre></blockquote>
<h4>Description</h4>
<p>
The function call returns a block diagonal matrix <b>J</b>
from the the two-column matrix <code>eigenvalues</code>
(computed by function
<a href=\"Modelica://Modelica.Math.Matrices.eigenValues\">Matrices.eigenValues</a>).
Matrix <code>eigenvalues</code> must have the real part of the
eigenvalues in the first column and the imaginary part in the
second column. If an eigenvalue i has a vanishing imaginary
part, then <b>J</b>[i,i] = eigenvalues[i,1], i.e., the diagonal
element of <b>J</b> is the real eigenvalue.
Otherwise, eigenvalue i and conjugate complex eigenvalue i+1
are used to construct a 2 by 2 diagonal block of <b>J</b>:
</p>
<blockquote>
<pre>
  J[i  , i]   := eigenvalues[i,1];
  J[i  , i+1] := eigenvalues[i,2];
  J[i+1, i]   := eigenvalues[i+1,2];
  J[i+1, i+1] := eigenvalues[i+1,1];
</pre>
</blockquote>
<h4>See also</h4>
<a href=\"Modelica://Modelica.Math.Matrices.eigenValues\">Matrices.eigenValues</a>
</HTML>"));
      protected 
        Integer n = size(eigenValues, 1);
        Integer i;
      algorithm 
        J:=zeros(n, n);
        i:=1;
        while (i <= n) loop
                  if eigenValues[i,2] == 0 then 
                    J[i,i]:=eigenValues[i,1];
          i:=i + 1;

          else           J[i,i]:=eigenValues[i,1];
          J[i,i + 1]:=eigenValues[i,2];
          J[i + 1,i]:=eigenValues[i + 1,2];
          J[i + 1,i + 1]:=eigenValues[i + 1,1];
          i:=i + 2;

          end if;

        end while;
      end eigenValueMatrix;
      function singularValues "Compute singular values and left and right singular vectors"
        extends Modelica.Icons.Function;
        input Real A[:,:] "Matrix";
        output Real sigma[min(size(A, 1), size(A, 2))] "Singular values";
        output Real U[size(A, 1),size(A, 1)] = identity(size(A, 1)) "Left orthogonal matrix";
        output Real VT[size(A, 2),size(A, 2)] = identity(size(A, 2)) "Transposed right orthogonal matrix ";
        annotation(Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
         sigma = Matrices.<b>singularValues</b>(A);
(sigma, U, VT) = Matrices.<b>singularValues</b>(A);
</pre></blockquote>
<h4>Description</h4>
<p>
This function computes the singular values and optionally the
singular vectors of matrix A. Basically the singular
value decomposition of A is computed, i.e.,
</p>
<blockquote><pre>
<b>A</b> = <b>U</b> <b><font face=\"Symbol\">S</font></b> <b>V</b><sup>T</sup>
  = U*Sigma*VT
</blockquote></pre>
<p>
where <b>U </b>and <b>V</b> are orthogonal matrices (<b>UU</b><sup>T</sup>=<b>I,
</b><b>VV</b><sup>T</sup>=<b>I</b>). <b><font face=\"Symbol\">S
</font></b> = diag(<font face=\"Symbol\">s</font><sub>i</sub>)
has the same size as matrix A with nonnegative diagonal elements
in decreasing order and with all other elements zero
(<font face=\"Symbol\">s</font><sub>1</sub> is the largest element). The function
returns the singular values <font face=\"Symbol\">s</font><sub>i</sub>
in vector <tt>sigma</tt> and the orthogonal matrices in
matrices <tt>U</tt> and <tt>V</tt>.
</p>
<h4>Example</h4>
<blockquote><pre>
  A = [1, 2,  3,  4;
       3, 4,  5, -2;
      -1, 2, -3,  5];
  (sigma, U, VT) = singularValues(A);
  results in:
     sigma = {8.33, 6.94, 2.31};
  i.e.
     Sigma = [8.33,    0,    0, 0;
                 0, 6.94,    0, 0;
                 0,    0, 2.31, 0]
</pre></blockquote>
<h4>See also</h4>
<a href=\"Modelica://Modelica.Math.Matrices.eigenValues\">Matrices.eigenValues</a>
</HTML>"));
      protected 
        Integer info;
        Integer n = min(size(A, 1), size(A, 2)) "Number of singular values";
      algorithm 
        if n > 0 then 
                (sigma,U,VT,info):=Modelica.Math.Matrices.LAPACK.dgesvd(A);
        assert(info == 0, "The numerical algorithm to compute the
singular value decomposition did not converge");

        else 
        end if;
      end singularValues;
      function det "Determinant of a matrix (computed by LU decomposition)"
        extends Modelica.Icons.Function;
        input Real A[:,size(A, 1)];
        output Real result "Determinant of matrix A";
      protected 
        Real LU[size(A, 1),size(A, 1)];
        Integer pivots[size(A, 1)];
        annotation(Documentation(info = "<HTML>
<h4>Syntax</h4>
<blockquote><pre>
Matrices.<b>det</b>(A);
</pre></blockquote>
<h4>Description</h4>
<p>
This function call returns the determinant of matrix A
computed by a LU decomposition.
Usally, this function should never be used, because
there are nearly always better numerical algorithms
as by computing the determinant. E.g., use function
<a href=\"Modelica://Modelica.Math.Matrices.rank\">Matrices.rank</a>
to compute the rank of a matrix.
<h4>See also</h4>
<a href=\"Modelica://Modelica.Math.Matrices.rank\">Matrices.rank</a>,
<a href=\"Modelica://Modelica.Math.Matrices.solve\">Matrices.solve</a>
</HTML>"));
      algorithm 
        (LU,pivots):=Matrices.LU(A);
        result:=product(LU[i,i] for i in 1:size(A, 1)) * product(if pivots[i] == i then 1 else  -1 for i in 1:size(pivots, 1));
      end det;
      function inv "Inverse of a matrix (try to avoid, use function solve(..) instead)"
        extends Modelica.Icons.Function;
        input Real A[:,size(A, 1)];
        output Real invA[size(A, 1),size(A, 2)] "Inverse of matrix A";
      protected 
        Integer info;
        Integer pivots[size(A, 1)] "Pivot vector";
        Real LU[size(A, 1),size(A, 2)] "LU factors of A";
      algorithm 
        (LU,pivots,info):=LAPACK.dgetrf(A);
        assert(info == 0, "Calculating an inverse matrix with function
\"Matrices.inv\" is not possible, since matrix A is singular.");
        invA:=LAPACK.dgetri(LU, pivots);
        annotation(Documentation(info = "<html>

</html>"));
      end inv;
      function rank "Rank of a matrix (computed with singular values)"
        extends Modelica.Icons.Function;
        input Real A[:,:] "Matrix";
        input Real eps = 0 "If eps > 0, the singular values are checked against eps; otherwise eps=max(size(A))*norm(A)*Modelica.Constants.eps is used";
        output Integer result "Rank of matrix A";
        annotation(Documentation(info = "<html>

</html>"));
      protected 
        Integer n = min(size(A, 1), size(A, 2));
        Integer i = n;
        Real sigma[n];
        Real eps2;
      algorithm 
        result:=0;
        if n > 0 then 
                sigma:=Modelica.Math.Matrices.singularValues(A);
        eps2:=if eps > 0 then eps else max(size(A)) * sigma[1] * Modelica.Constants.eps;
        while (i > 0) loop
                  if sigma[i] > eps2 then 
                    result:=i;
          i:=0;

          else 
          end if;
          i:=i - 1;

        end while;

        else 
        end if;
      end rank;
      function balance "Balancing of matrix A to improve the condition of A"
        extends Modelica.Icons.Function;
        input Real A[:,size(A, 1)];
        output Real D[size(A, 1)] "diagonal(D)=T is transformation matrix, such that
          T*A*inv(T) has smaller condition as A";
        output Real B[size(A, 1),size(A, 1)] "Balanced matrix (= diagonal(D)*A*inv(diagonal(D)))";
      protected 
        Integer na = size(A, 1);
        Integer radix = 2 "Radix of exponent representation must be 'radix'
          or a multiple of 'radix'";
        Integer radix2 = radix * radix;
        Boolean noconv = true;
        Integer i = 1;
        Integer j = 1;
        Real CO;
        Real RO;
        Real G;
        Real F;
        Real S;
        annotation(Documentation(info = "<HTML>
<p>
The function transformates the matrix A, so that the norm of the i-th column
is nearby the i-th row. (D,B)=Matrices.balance(A) returns a vector D, such
that B=inv(diagonal(D))*A*diagonal(D) has better condition. The elements of D
are multiples of 2. Balancing attempts to make the norm of each row equal to the
norm of the belonging column. <br>
Balancing is used to minimize roundoff errors inducted
through large matrix calculations like Taylor-series approximation
or computation of eigenvalues.
</p>
<b>Example:</b><br><br>
<pre>       - A = [1, 10,  1000; .01,  0,  10; .005,  .01,  10]
       - Matrices.norm(A, 1);
         = 1020.0
       - (T,B)=Matrices.balance(A)
       - T
         = {256, 16, 0.5}
       - B
         =  [1,     0.625,   1.953125;
             0.16,  0,       0.3125;
             2.56,  0.32,   10.0]
       - Matrices.norm(B, 1);
         = 12.265625
</pre>
<p>
The Algorithm is taken from
<dl>
<dt>H. D. Joos, G. Grbel:
<dd><b>RASP'91 Regulator Analysis and Synthesis Programs</b><br>
    DLR - Control Systems Group 1991
</dl>
which based on the balanc function from EISPACK.
</p>

</HTML>
", revisions = "<html>
<p><b>Release Notes:</b></p>
<li><i>July 5, 2002</i>
       by H. D. Joos and Nico Walther<br>
       Implemented.
</li>
</html>"));
      algorithm 
        D:=ones(na);
        B:=A;
        while (noconv) loop
                  noconv:=false;
          for i in 1:na loop
                      CO:=sum(abs(B[:,i])) - abs(B[i,i]);
            RO:=sum(abs(B[i,:])) - abs(B[i,i]);
            G:=RO / radix;
            F:=1;
            S:=CO + RO;
            while (not (CO >= G or CO == 0)) loop
                          F:=F * radix;
              CO:=CO * radix2;

            end while;
            G:=RO * radix;
            while (not (CO < G or RO == 0)) loop
                          F:=F / radix;
              CO:=CO / radix2;

            end while;
            if not (CO + RO) / F >= 0.95 * S then 
                        G:=1 / F;
            D[i]:=D[i] * F;
            B[i,:]:=B[i,:] * G;
            B[:,i]:=B[:,i] * F;
            noconv:=true;

            else 
            end if;

          end for;

        end while;
      end balance;
      function exp "Compute the exponential of a matrix by adaptive Taylor series expansion with scaling and balancing"
        extends Modelica.Icons.Function;
        input Real A[:,size(A, 1)];
        input Real T = 1;
        output Real phi[size(A, 1),size(A, 1)] "= exp(A*T)";
        annotation(Documentation(info = "<HTML>
<p>This function computes</p>
<pre>                            (<b>A</b>T)^2   (<b>A</b>T)^3
     <font size=4> <b>&Phi;</b></font> = e^(<b>A</b>T) = <b>I</b> + <b>A</b>T + ------ + ------ + ....
                              2!       3!
</pre>
<p>where e=2.71828..., <b>A</b> is an n x n matrix with real elements and T is a real number,
e.g., the sampling time.
<b>A</b> may be singular. With the exponential of a matrix it is, e.g., possible
to compute the solution of a linear system of differential equations</p>
<pre>    der(<b>x</b>) = <b>A</b>*<b>x</b>   ->   <b>x</b>(t0 + T) = e^(<b>A</b>T)*x(t0)
</pre>
<p>
The function is called as
<pre>     Phi = Matrices.exp(A,T);</pre>
or
<pre>       M = Matrices.exp(A);
</pre>
what calculates M as the exponential of matrix A.
</p>
<p><b>Algorithmic details:</b></p>
<p>The algorithm is taken from </p>
<dl>
<dt>H. D. Joos, G. Gruebel:
<dd><b>RASP'91 Regulator Analysis and Synthesis Programs</b><br>
    DLR - Control Systems Group 1991
</dl>
<p>The following steps are performed to calculate the exponential of A:</p>
<ol>
  <li>Matrix <b>A</b> is balanced <br>
  (= is transformed with a diagonal matrix <b>D</b>, such that inv(<b>D</b>)*<b>A</b>*<b>D</b>
  has a smaller condition as <b>A</b>).</li>
  <li>The scalar T is divided by a multiple of 2 such that norm(
       inv(<b>D</b>)*<b>A</b>*<b>D</b>*T/2^k ) &lt; 0.5. Note, that (1) and (2) are implemented such that no round-off errors
  are introduced.</li>
  <li>The matrix from (2) is approximated by explicitly performing the Taylor
  series expansion with a variable number of terms.
  Truncation occurs if a new term does no longer contribute to the value of <b>&Phi;</b>
  from the previous iteration.</li>
  <li>The resulting matrix is transformed back, by reverting the steps of (2)
  and (1).</li>
</ol>
<p>In several sources it is not recommended to use Taylor series expansion to
calculate the exponential of a matrix, such as in 'C.B. Moler and C.F. Van Loan:
Nineteen dubious ways to compute the exponential of a matrix. SIAM Review 20,
pp. 801-836, 1979' or in the documentation of m-file expm2 in Matlab version 6
(http://www.MathWorks.com) where it is stated that 'As a practical numerical
method, this is often slow and inaccurate'. These statements are valid for a
direct implementation of the Taylor series expansion, but <i>not</i> for the
implementation variant used in this function.
</p>

</HTML>
", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>July 5, 2002</i>
       by H. D. Joos and Nico Walther<br>
       Implemented.
</li>
</ul>
</html>"));
      protected 
        parameter Integer nmax = 21;
        parameter Integer na = size(A, 1);
        Integer j = 1;
        Integer k = 0;
        Boolean done = false;
        Real Anorm;
        Real Tscaled = 1;
        Real Atransf[na,na];
        Real D[na,na];
        Real M[na,na];
        Real Diag[na];
        encapsulated function columnNorm "Returns the column norm of a matrix"
          input Real A[:,:] "Input matrix";
          output Real result = 0.0 "1-norm of matrix A";
        algorithm 
          for i in 1:size(A, 2) loop
                      result:=max(result, sum(abs(A[:,i])));

          end for;
        end columnNorm;
      algorithm 
        (Diag,Atransf):=balance(A);
        Tscaled:=T;
        Anorm:=columnNorm(Atransf);
        Anorm:=Anorm * T;
        while (Anorm >= 0.5) loop
                  Anorm:=Anorm / 2;
          Tscaled:=Tscaled / 2;
          k:=k + 1;

        end while;
        M:=identity(na);
        D:=M;
        while (j < nmax and not done) loop
                  M:=(Atransf * M * Tscaled) / j;
          if columnNorm(D + M - D) == 0 then 
                    done:=true;

          else           D:=M + D;
          j:=j + 1;

          end if;

        end while;
        for i in 1:k loop
                  D:=D * D;

        end for;
        for j in 1:na loop
                  for k in 1:na loop
                      phi[j,k]:=(D[j,k] * Diag[j]) / Diag[k];

          end for;

        end for;
      end exp;
      function integralExp "Computation of the transition-matrix phi and its integral gamma"
        extends Modelica.Icons.Function;
        input Real A[:,size(A, 1)];
        input Real B[size(A, 1),:];
        input Real T = 1;
        output Real phi[size(A, 1),size(A, 1)] "= exp(A*T)";
        output Real gamma[size(A, 1),size(B, 2)] "= integral(phi)*B";
      protected 
        parameter Integer nmax = 21;
        parameter Integer na = size(A, 1);
        Integer j = 2;
        Integer k = 0;
        Boolean done = false;
        Real Anorm;
        Real Tscaled = 1;
        Real Atransf[na,na];
        Real Psi[na,na];
        Real M[na,na];
        Real Diag[na];
        annotation(Documentation(info = "<HTML>
<p>
The function uses a Taylor series expansion with Balancing and
scaling/squaring to approximate the integral <b>&Psi;</b> of the matrix
exponential <b>&Phi;</b>=e^(AT):
</p>
<pre>                                 AT^2   A^2 * T^3          A^k * T^(k+1)
        <b>&Psi;</b> = int(e^(As))ds = IT + ---- + --------- + ... + --------------
                                  2!        3!                (k+1)!
</pre>
<p>
<b>&Phi;</b> is calculated through <b>&Phi;</b> = I + A*<b>&Psi;</b>, so A may be singular. <b>&Gamma;</b> is
simple <b>&Psi;</b>*B.
</p>
<p>The algorithm runs in the following steps: </p>
<ol>
  <li>Balancing</li>
  <li>Scaling</li>
  <li>Taylor series expansion</li>
  <li>Re-scaling</li>
  <li>Re-Balancing</li>
</ol>
<p>Balancing put the bad condition of a square matrix <i>A</i> into a diagonal
transformation matrix <i>D</i>. This reduce the effort of following calculations.
Afterwards the result have to be re-balanced by transformation D*A<small>transf</small>
 *inv(D).<br>
Scaling halfen T&nbsp; k-times, until the norm of A*T is less than 0.5. This
garantees minumum rounding errors in the following series
expansion. The re-scaling based on the equation&nbsp; exp(A*2T) = exp(AT)^2.
The needed re-scaling formula for psi thus becomes:</p>
<pre>         <b>&Phi;</b> = <b>&Phi;</b>'*<b>&Phi;</b>'
   I + A*<b>&Psi;</b> = I + 2A*<b>&Psi;</b>' + A^2*<b>&Psi;</b>'^2
         <b>&Psi;</b> = A*<b>&Psi;</b>'^2 + 2*<b>&Psi;</b>'
</pre>
<p>
where psi' is the scaled result from the series expansion while psi is the
re-scaled matrix.
</p>
<p>
The function is normally used to discretize a state-space system as the
zero-order-hold equivalent:
</p>
<pre>      x(k+1) = <b>&Phi;</b>*x(k) + <b>&Gamma;</b>*u(k)
        y(k) = C*x(k) + D*u(k)
</pre>
<p>
The zero-order-hold sampling, also known as step-invariant method, gives
exact values of the state variables, under the assumption that the control
signal u is constant between the sampling instants. Zero-order-hold sampling
is discribed in
</p>
<dl>
<dt>K. J. Astroem, B. Wittenmark:
<dd><b>Computer Controlled Systems - Theory and Design</b><br>
    Third Edition, p. 32
</dl>
<pre><b>Syntax:</b>
      (phi,gamma) = Matrices.expIntegral(A,B,T)
                       A,phi: [n,n] square matrices
                     B,gamma: [n,m] input matrix
                           T: scalar, e.g. sampling time
</pre>
<p>
The Algorithm to calculate psi is taken from
<dl>
<dt>H. D. Joos, G. Gruebel:
<dd><b>RASP'91 Regulator Analysis and Synthesis Programs</b><br>
    DLR - Control Systems Group 1991
</dl>
</p>

</HTML>
", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>July 5, 2002</i>
       by H. D. Joos and Nico Walther<br>
       Implemented.
</li>
</ul>
</html>"));
        encapsulated function columnNorm "Returns the column norm of a matrix"
          input Real A[:,:] "Input matrix";
          output Real result = 0.0 "1-norm of matrix A";
        algorithm 
          for i in 1:size(A, 2) loop
                      result:=max(result, sum(abs(A[:,i])));

          end for;
        end columnNorm;
      algorithm 
        (Diag,Atransf):=balance(A);
        Tscaled:=T;
        Anorm:=columnNorm(Atransf);
        Anorm:=Anorm * T;
        while (Anorm >= 0.5) loop
                  Anorm:=Anorm / 2;
          Tscaled:=Tscaled / 2;
          k:=k + 1;

        end while;
        M:=identity(na) * Tscaled;
        Psi:=M;
        while (j < nmax and not done) loop
                  M:=(Atransf * M * Tscaled) / j;
          if columnNorm(Psi + M - Psi) == 0 then 
                    done:=true;

          else           Psi:=M + Psi;
          j:=j + 1;

          end if;

        end while;
        for j in 1:k loop
                  Psi:=Atransf * Psi * Psi + 2 * Psi;

        end for;
        for j in 1:na loop
                  for k in 1:na loop
                      Psi[j,k]:=(Psi[j,k] * Diag[j]) / Diag[k];

          end for;

        end for;
        gamma:=Psi * B;
        phi:=A * Psi + identity(na);
      end integralExp;
      function integralExpT "Computation of the transition-matrix phi and the integral gamma and gamma1"
        extends Modelica.Icons.Function;
        input Real A[:,size(A, 1)];
        input Real B[size(A, 1),:];
        input Real T = 1;
        output Real phi[size(A, 1),size(A, 1)] "= exp(A*T)";
        output Real gamma[size(A, 1),size(B, 2)] "= integral(phi)*B";
        output Real gamma1[size(A, 1),size(B, 2)] "= integral((T-t)*exp(A*t))*B";
      protected 
        Integer nmax = 200;
        parameter Integer na = size(A, 1);
        parameter Integer nb = size(B, 2);
        Integer j = 1;
        Boolean done = false;
        Real F[na + 2 * nb,na + 2 * nb];
        annotation(Documentation(info = "<HTML>
<p>
The function calculates the matrices phi,gamma,gamma1 through the equation:
</p>
<pre>                                 [ A B 0 ]
[phi gamma gamma1] = [I 0 0]*exp([ 0 0 I ]*T)
                                 [ 0 0 0 ]
</pre>
<pre>
<b>Syntax:</b><br>
      (phi,gamma,gamma1) = Matrices.ExpIntegral2(A,B,T)
                     A,phi: [n,n] square matrices
            B,gamma,gamma1: [n,m] matrices
                         T: scalar, e.g. sampling time
</pre>
<p>
The matrices define the discretized first-order-hold equivalent of
a state-space system:
<pre>      x(k+1) = phi*x(k) + gamma*u(k) + gamma1/T*(u(k+1) - u(k))
</pre>
The first-order-hold sampling, also known as ramp-invariant method, gives
more smooth control signals as the ZOH equivalent. First-order-hold sampling
is discribed in
<dl>
<dt>K. J. Astroem, B. Wittenmark:
<dd><b>Computer Controlled Systems - Theory and Design</b><br>
    Third Edition, p. 256
</dl>
</p>

</HTML>
", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>July 31, 2002</i>
       by Nico Walther<br>
       Realized.
</li>
</ul>
</html>"));
      algorithm 
        F:=[A,B,zeros(na, nb);zeros(2 * nb, na),zeros(2 * nb, nb),[identity(nb);zeros(nb, nb)]];
        F:=exp(F, T);
        phi:=F[1:na,1:na];
        gamma:=F[1:na,na + 1:na + nb];
        gamma1:=F[1:na,na + nb + 1:na + 2 * nb];
      end integralExpT;
      package LAPACK "Interface to LAPACK library (should usually not directly be used but only indirectly via Modelica.Math.Matrices)"
        extends Modelica.Icons.Library;
        function dgeev "Compute eigenvalues and (right) eigenvectors for real nonsymmetrix matrix A"
          extends Modelica.Icons.Function;
          input Real A[:,size(A, 1)];
          output Real eigenReal[size(A, 1)] "Real part of eigen values";
          output Real eigenImag[size(A, 1)] "Imaginary part of eigen values";
          output Real eigenVectors[size(A, 1),size(A, 1)] "Right eigen vectors";
          output Integer info;
        protected 
          Integer n = size(A, 1);
          Integer lwork = 12 * n;
          Real Awork[n,n] = A;
          Real work[lwork];
          annotation(Documentation(info = "Lapack documentation
    Purpose
    =======
    DGEEV computes for an N-by-N real nonsymmetric matrix A, the
    eigenvalues and, optionally, the left and/or right eigenvectors.
    The right eigenvector v(j) of A satisfies
                     A * v(j) = lambda(j) * v(j)
    where lambda(j) is its eigenvalue.
    The left eigenvector u(j) of A satisfies
                  u(j)**H * A = lambda(j) * u(j)**H
    where u(j)**H denotes the conjugate transpose of u(j).
    The computed eigenvectors are normalized to have Euclidean norm
    equal to 1 and largest component real.
    Arguments
    =========
    JOBVL   (input) CHARACTER*1
            = 'N': left eigenvectors of A are not computed;
            = 'V': left eigenvectors of A are computed.
    JOBVR   (input) CHARACTER*1
            = 'N': right eigenvectors of A are not computed;
            = 'V': right eigenvectors of A are computed.
    N       (input) INTEGER
            The order of the matrix A. N >= 0.
    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the N-by-N matrix A.
            On exit, A has been overwritten.
    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,N).
    WR      (output) DOUBLE PRECISION array, dimension (N)
    WI      (output) DOUBLE PRECISION array, dimension (N)
            WR and WI contain the real and imaginary parts,
            respectively, of the computed eigenvalues.  Complex
            conjugate pairs of eigenvalues appear consecutively
            with the eigenvalue having the positive imaginary part
            first.
    VL      (output) DOUBLE PRECISION array, dimension (LDVL,N)
            If JOBVL = 'V', the left eigenvectors u(j) are stored one
            after another in the columns of VL, in the same order
            as their eigenvalues.
            If JOBVL = 'N', VL is not referenced.
            If the j-th eigenvalue is real, then u(j) = VL(:,j),
            the j-th column of VL.
            If the j-th and (j+1)-st eigenvalues form a complex
            conjugate pair, then u(j) = VL(:,j) + i*VL(:,j+1) and
            u(j+1) = VL(:,j) - i*VL(:,j+1).
    LDVL    (input) INTEGER
            The leading dimension of the array VL.  LDVL >= 1; if
            JOBVL = 'V', LDVL >= N.
    VR      (output) DOUBLE PRECISION array, dimension (LDVR,N)
            If JOBVR = 'V', the right eigenvectors v(j) are stored one
            after another in the columns of VR, in the same order
            as their eigenvalues.
            If JOBVR = 'N', VR is not referenced.
            If the j-th eigenvalue is real, then v(j) = VR(:,j),
            the j-th column of VR.
            If the j-th and (j+1)-st eigenvalues form a complex
            conjugate pair, then v(j) = VR(:,j) + i*VR(:,j+1) and
            v(j+1) = VR(:,j) - i*VR(:,j+1).
    LDVR    (input) INTEGER
            The leading dimension of the array VR.  LDVR >= 1; if
            JOBVR = 'V', LDVR >= N.
    WORK    (workspace/output) DOUBLE PRECISION array, dimension (LWORK)

            On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
    LWORK   (input) INTEGER
            The dimension of the array WORK.  LWORK >= max(1,3*N), and
            if JOBVL = 'V' or JOBVR = 'V', LWORK >= 4*N.  For good
            performance, LWORK must generally be larger.
    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value.
            > 0:  if INFO = i, the QR algorithm failed to compute all the
                  eigenvalues, and no eigenvectors have been computed;
                  elements i+1:N of WR and WI contain eigenvalues which
                  have converged.
"));

          external "Fortran 77" dgeev("N","V",n,Awork,n,eigenReal,eigenImag,eigenVectors,n,eigenVectors,n,work,size(work, 1),info)           annotation(Library = "Lapack"); 
        end dgeev;
        function dgeev_eigenValues "Compute eigenvalues for real nonsymmetrix matrix A"
          extends Modelica.Icons.Function;
          input Real A[:,size(A, 1)];
          output Real EigenReal[size(A, 1)];
          output Real EigenImag[size(A, 1)];
          output Integer info;
        protected 
          Integer lwork = 8 * size(A, 1);
          Real Awork[size(A, 1),size(A, 1)] = A;
          Real work[lwork];
          Real EigenvectorsL[size(A, 1),size(A, 1)] = zeros(size(A, 1), size(A, 1));
          annotation(Documentation(info = "Lapack documentation
    Purpose
    =======
    DGEEV computes for an N-by-N real nonsymmetric matrix A, the
    eigenvalues and, optionally, the left and/or right eigenvectors.
    The right eigenvector v(j) of A satisfies
                     A * v(j) = lambda(j) * v(j)
    where lambda(j) is its eigenvalue.
    The left eigenvector u(j) of A satisfies
                  u(j)**H * A = lambda(j) * u(j)**H
    where u(j)**H denotes the conjugate transpose of u(j).
    The computed eigenvectors are normalized to have Euclidean norm
    equal to 1 and largest component real.
    Arguments
    =========
    JOBVL   (input) CHARACTER*1
            = 'N': left eigenvectors of A are not computed;
            = 'V': left eigenvectors of A are computed.
    JOBVR   (input) CHARACTER*1
            = 'N': right eigenvectors of A are not computed;
            = 'V': right eigenvectors of A are computed.
    N       (input) INTEGER
            The order of the matrix A. N >= 0.
    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the N-by-N matrix A.
            On exit, A has been overwritten.
    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,N).
    WR      (output) DOUBLE PRECISION array, dimension (N)
    WI      (output) DOUBLE PRECISION array, dimension (N)
            WR and WI contain the real and imaginary parts,
            respectively, of the computed eigenvalues.  Complex
            conjugate pairs of eigenvalues appear consecutively
            with the eigenvalue having the positive imaginary part
            first.
    VL      (output) DOUBLE PRECISION array, dimension (LDVL,N)
            If JOBVL = 'V', the left eigenvectors u(j) are stored one
            after another in the columns of VL, in the same order
            as their eigenvalues.
            If JOBVL = 'N', VL is not referenced.
            If the j-th eigenvalue is real, then u(j) = VL(:,j),
            the j-th column of VL.
            If the j-th and (j+1)-st eigenvalues form a complex
            conjugate pair, then u(j) = VL(:,j) + i*VL(:,j+1) and
            u(j+1) = VL(:,j) - i*VL(:,j+1).
    LDVL    (input) INTEGER
            The leading dimension of the array VL.  LDVL >= 1; if
            JOBVL = 'V', LDVL >= N.
    VR      (output) DOUBLE PRECISION array, dimension (LDVR,N)
            If JOBVR = 'V', the right eigenvectors v(j) are stored one
            after another in the columns of VR, in the same order
            as their eigenvalues.
            If JOBVR = 'N', VR is not referenced.
            If the j-th eigenvalue is real, then v(j) = VR(:,j),
            the j-th column of VR.
            If the j-th and (j+1)-st eigenvalues form a complex
            conjugate pair, then v(j) = VR(:,j) + i*VR(:,j+1) and
            v(j+1) = VR(:,j) - i*VR(:,j+1).
    LDVR    (input) INTEGER
            The leading dimension of the array VR.  LDVR >= 1; if
            JOBVR = 'V', LDVR >= N.
    WORK    (workspace/output) DOUBLE PRECISION array, dimension (LWORK)

            On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
    LWORK   (input) INTEGER
            The dimension of the array WORK.  LWORK >= max(1,3*N), and
            if JOBVL = 'V' or JOBVR = 'V', LWORK >= 4*N.  For good
            performance, LWORK must generally be larger.
    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value.
            > 0:  if INFO = i, the QR algorithm failed to compute all the
                  eigenvalues, and no eigenvectors have been computed;
                  elements i+1:N of WR and WI contain eigenvalues which
                  have converged.
"));

          external "Fortran 77" dgeev("N","N",size(A, 1),Awork,size(A, 1),EigenReal,EigenImag,EigenvectorsL,size(EigenvectorsL, 1),EigenvectorsL,size(EigenvectorsL, 1),work,size(work, 1),info)           annotation(Library = "Lapack"); 
        end dgeev_eigenValues;
        function dgegv "Compute generalized eigenvalues and eigenvectors for a (A,B) system"
          extends Modelica.Icons.Function;
          input Real A[:,size(A, 1)];
          input Real B[size(A, 1),size(A, 1)];
          output Real alphaReal[size(A, 1)] "Real part of alpha (eigenvalue=(alphaReal+i*alphaImag)/beta)";
          output Real alphaImag[size(A, 1)] "Imaginary part of alpha";
          output Real beta[size(A, 1)] "Denominator of eigenvalue";
          output Integer info;
        protected 
          Integer n = size(A, 1);
          Integer lwork = 12 * n;
          Real Awork[n,n] = A;
          Real Bwork[n,n] = B;
          Real work[lwork];
          Real dummy1[1,1];
          Real dummy2[1,1];
          annotation(Documentation(info = "Purpose
=======

For a pair of N-by-N real nonsymmetric matrices A, B:
   compute the generalized eigenvalues (alphar +/- alphai*i, beta)
   compute the left and/or right generalized eigenvectors
           (VL and VR)
The second action is optional -- see the description of JOBVL and
JOBVR below.
A generalized eigenvalue for a pair of matrices (A,B) is, roughly
speaking, a scalar w or a ratio  alpha/beta = w, such that  A - w*B
is singular.  It is usually represented as the pair (alpha,beta),
as there is a reasonable interpretation for beta=0, and even for
both being zero.  A good beginning reference is the book, \"Matrix
Computations\", by G. Golub &amp; C. van Loan (Johns Hopkins U. Press)
A right generalized eigenvector corresponding to a generalized
eigenvalue  w  for a pair of matrices (A,B) is a vector  r  such
that  (A - w B) r = 0 .  A left generalized eigenvector is a vector
                       H
l  such that  (A - w B) l = 0 .
Note: this routine performs \"full balancing\" on A and B -- see
\"Further Details\", below.
Arguments
=========
JOBVL   (input) CHARACTER*1
        = 'N':  do not compute the left generalized eigenvectors;
        = 'V':  compute the left generalized eigenvectors.
JOBVR   (input) CHARACTER*1
        = 'N':  do not compute the right generalized eigenvectors;
        = 'V':  compute the right generalized eigenvectors.
N       (input) INTEGER
        The number of rows and columns in the matrices A, B, VL, and
        VR.  N >= 0.
A       (input/workspace) DOUBLE PRECISION array, dimension (LDA, N)
        On entry, the first of the pair of matrices whose
        generalized eigenvalues and (optionally) generalized
        eigenvectors are to be computed.
        On exit, the contents will have been destroyed.  (For a
        description of the contents of A on exit, see \"Further
        Details\", below.)
LDA     (input) INTEGER
        The leading dimension of A.  LDA >= max(1,N).
B       (input/workspace) DOUBLE PRECISION array, dimension (LDB, N)
        On entry, the second of the pair of matrices whose
        generalized eigenvalues and (optionally) generalized
        eigenvectors are to be computed.
        On exit, the contents will have been destroyed.  (For a
        description of the contents of B on exit, see \"Further
        Details\", below.)
LDB     (input) INTEGER
        The leading dimension of B.  LDB >= max(1,N).
ALPHAR  (output) DOUBLE PRECISION array, dimension (N)
ALPHAI  (output) DOUBLE PRECISION array, dimension (N)
BETA    (output) DOUBLE PRECISION array, dimension (N)
        On exit, (ALPHAR(j) + ALPHAI(j)*i)/BETA(j), j=1,...,N, will
        be the generalized eigenvalues.  If ALPHAI(j) is zero, then
        the j-th eigenvalue is real; if positive, then the j-th and
        (j+1)-st eigenvalues are a complex conjugate pair, with
        ALPHAI(j+1) negative.
        Note: the quotients ALPHAR(j)/BETA(j) and ALPHAI(j)/BETA(j)
        may easily over- or underflow, and BETA(j) may even be zero.
        Thus, the user should avoid naively computing the ratio
        alpha/beta.  However, ALPHAR and ALPHAI will be always less
        than and usually comparable with norm(A) in magnitude, and
        BETA always less than and usually comparable with norm(B).
VL      (output) DOUBLE PRECISION array, dimension (LDVL,N)
        If JOBVL = 'V', the left generalized eigenvectors.  (See
        \"Purpose\", above.)  Real eigenvectors take one column,
        complex take two columns, the first for the real part and
        the second for the imaginary part.  Complex eigenvectors
        correspond to an eigenvalue with positive imaginary part.
        Each eigenvector will be scaled so the largest component
        will have abs(real part) + abs(imag. part) = 1, *except*
        that for eigenvalues with alpha=beta=0, a zero vector will
        be returned as the corresponding eigenvector.
        Not referenced if JOBVL = 'N'.
LDVL    (input) INTEGER
        The leading dimension of the matrix VL. LDVL >= 1, and
        if JOBVL = 'V', LDVL >= N.
VR      (output) DOUBLE PRECISION array, dimension (LDVR,N)
        If JOBVL = 'V', the right generalized eigenvectors.  (See
        \"Purpose\", above.)  Real eigenvectors take one column,
        complex take two columns, the first for the real part and
        the second for the imaginary part.  Complex eigenvectors
        correspond to an eigenvalue with positive imaginary part.
        Each eigenvector will be scaled so the largest component
        will have abs(real part) + abs(imag. part) = 1, *except*
        that for eigenvalues with alpha=beta=0, a zero vector will
        be returned as the corresponding eigenvector.
        Not referenced if JOBVR = 'N'.
LDVR    (input) INTEGER
        The leading dimension of the matrix VR. LDVR >= 1, and
        if JOBVR = 'V', LDVR >= N.
WORK    (workspace/output) DOUBLE PRECISION array, dimension (LWORK)
        On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
LWORK   (input) INTEGER
        The dimension of the array WORK.  LWORK >= max(1,8*N).
        For good performance, LWORK must generally be larger.
        To compute the optimal value of LWORK, call ILAENV to get
        blocksizes (for DGEQRF, DORMQR, and DORGQR.)  Then compute:
        NB  -- MAX of the blocksizes for DGEQRF, DORMQR, and DORGQR;
        The optimal LWORK is:
            2*N + MAX( 6*N, N*(NB+1) ).
INFO    (output) INTEGER
        = 0:  successful exit
        < 0:  if INFO = -i, the i-th argument had an illegal value.
        = 1,...,N:
              The QZ iteration failed.  No eigenvectors have been
              calculated, but ALPHAR(j), ALPHAI(j), and BETA(j)
              should be correct for j=INFO+1,...,N.
        > N:  errors that usually indicate LAPACK problems:
              =N+1: error return from DGGBAL
              =N+2: error return from DGEQRF
              =N+3: error return from DORMQR
              =N+4: error return from DORGQR
              =N+5: error return from DGGHRD
              =N+6: error return from DHGEQZ (other than failed
                                              iteration)
              =N+7: error return from DTGEVC
              =N+8: error return from DGGBAK (computing VL)
              =N+9: error return from DGGBAK (computing VR)
              =N+10: error return from DLASCL (various calls)
Further Details
===============
Balancing
---------
This driver calls DGGBAL to both permute and scale rows and columns
of A and B.  The permutations PL and PR are chosen so that PL*A*PR
and PL*B*R will be upper triangular except for the diagonal blocks
A(i:j,i:j) and B(i:j,i:j), with i and j as close together as
possible.  The diagonal scaling matrices DL and DR are chosen so
that the pair  DL*PL*A*PR*DR, DL*PL*B*PR*DR have entries close to
one (except for the entries that start out zero.)
After the eigenvalues and eigenvectors of the balanced matrices
have been computed, DGGBAK transforms the eigenvectors back to what
they would have been (in perfect arithmetic) if they had not been
balanced.
Contents of A and B on Exit
-------- -- - --- - -- ----
If any eigenvectors are computed (either JOBVL='V' or JOBVR='V' or
both), then on exit the arrays A and B will contain the real Schur
form[*] of the \"balanced\" versions of A and B.  If no eigenvectors
are computed, then only the diagonal blocks will be correct.
[*] See DHGEQZ, DGEGS, or read the book \"Matrix Computations\",
"));

          external "Fortran 77" dgegv("N","N",n,Awork,n,Bwork,n,alphaReal,alphaImag,beta,dummy1,1,dummy2,1,work,size(work, 1),info)           annotation(Library = "Lapack"); 
        end dgegv;
        function dgels_vec "Solves overdetermined or underdetermined real linear equations A*x=b with a b vector"
          extends Modelica.Icons.Function;
          input Real A[:,:];
          input Real b[size(A, 1)];
          output Real x[nx] = cat(1, b, zeros(nx - nrow)) "solution is in first size(A,2) rows";
          output Integer info;
        protected 
          Integer nrow = size(A, 1);
          Integer ncol = size(A, 2);
          Integer nx = max(nrow, ncol);
          Integer lwork = min(nrow, ncol) + nx;
          Real work[lwork];
          Real Awork[nrow,ncol] = A;

          external "FORTRAN 77" dgels("N",nrow,ncol,1,Awork,nrow,x,nx,work,lwork,info)           annotation(Library = "Lapack");           annotation(Documentation(info = "Lapack documentation
  Purpose
  =======

  DGELS solves overdetermined or underdetermined real linear systems
  involving an M-by-N matrix A, or its transpose, using a QR or LQ
  factorization of A.  It is assumed that A has full rank.

  The following options are provided:

  1. If TRANS = 'N' and m >= n:  find the least squares solution of
     an overdetermined system, i.e., solve the least squares problem
                  minimize || B - A*X ||.

  2. If TRANS = 'N' and m < n:  find the minimum norm solution of
     an underdetermined system A * X = B.

  3. If TRANS = 'T' and m >= n:  find the minimum norm solution of
     an undetermined system A**T * X = B.

  4. If TRANS = 'T' and m < n:  find the least squares solution of
     an overdetermined system, i.e., solve the least squares problem
                  minimize || B - A**T * X ||.

  Several right hand side vectors b and solution vectors x can be
  handled in a single call; they are stored as the columns of the
  M-by-NRHS right hand side matrix B and the N-by-NRHS solution
  matrix X.

  Arguments
  =========

  TRANS   (input) CHARACTER
          = 'N': the linear system involves A;
          = 'T': the linear system involves A**T.

  M       (input) INTEGER
          The number of rows of the matrix A.  M >= 0.

  N       (input) INTEGER
          The number of columns of the matrix A.  N >= 0.

  NRHS    (input) INTEGER
          The number of right hand sides, i.e., the number of
          columns of the matrices B and X. NRHS >=0.

  A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
          On entry, the M-by-N matrix A.
          On exit,
            if M >= N, A is overwritten by details of its QR
                       factorization as returned by DGEQRF;
            if M <  N, A is overwritten by details of its LQ
                       factorization as returned by DGELQF.

  LDA     (input) INTEGER
          The leading dimension of the array A.  LDA >= max(1,M).

  B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
          On entry, the matrix B of right hand side vectors, stored
          columnwise; B is M-by-NRHS if TRANS = 'N', or N-by-NRHS
          if TRANS = 'T'.
          On exit, B is overwritten by the solution vectors, stored
          columnwise:  if TRANS = 'N' and m >= n, rows 1 to n of B
          contain the least squares solution vectors; the residual
          sum of squares for the solution in each column is given by
          the sum of squares of elements N+1 to M in that column;
          if TRANS = 'N' and m < n, rows 1 to N of B contain the
          minimum norm solution vectors;
          if TRANS = 'T' and m >= n, rows 1 to M of B contain the
          minimum norm solution vectors;
          if TRANS = 'T' and m < n, rows 1 to M of B contain the
          least squares solution vectors; the residual sum of squares
          for the solution in each column is given by the sum of
          squares of elements M+1 to N in that column.

  LDB     (input) INTEGER
          The leading dimension of the array B. LDB >= MAX(1,M,N).

  WORK    (workspace) DOUBLE PRECISION array, dimension (LWORK)
          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.

  LWORK   (input) INTEGER
          The dimension of the array WORK.
          LWORK >= min(M,N) + MAX(1,M,N,NRHS).
          For optimal performance,
          LWORK >= min(M,N) + MAX(1,M,N,NRHS) * NB
          where NB is the optimum block size.

  INFO    (output) INTEGER
          = 0:  successful exit
          < 0:  if INFO = -i, the i-th argument had an illegal value
                                                                          "));
        end dgels_vec;
        function dgelsx_vec "Computes the minimum-norm solution to a real linear least squares problem with rank deficient A"
          extends Modelica.Icons.Function;
          input Real A[:,:];
          input Real b[size(A, 1)];
          input Real rcond = 0.0 "Reciprocal condition number to estimate rank";
          output Real x[max(nrow, ncol)] = cat(1, b, zeros(max(nrow, ncol) - nrow)) "solution is in first size(A,2) rows";
          output Integer info;
          output Integer rank "Effective rank of A";
        protected 
          Integer nrow = size(A, 1);
          Integer ncol = size(A, 2);
          Integer nx = max(nrow, ncol);
          Integer lwork = max(min(nrow, ncol) + 3 * ncol, 2 * min(nrow, ncol) + 1);
          Real work[lwork];
          Real Awork[nrow,ncol] = A;
          Integer jpvt[ncol] = zeros(ncol);

          external "FORTRAN 77" dgelsx(nrow,ncol,1,Awork,nrow,x,nx,jpvt,rcond,rank,work,lwork,info)           annotation(Library = "Lapack");           annotation(Documentation(info = "Lapack documentation
  Purpose
  =======

  DGELSX computes the minimum-norm solution to a real linear least
  squares problem:
      minimize || A * X - B ||
  using a complete orthogonal factorization of A.  A is an M-by-N
  matrix which may be rank-deficient.

  Several right hand side vectors b and solution vectors x can be
  handled in a single call; they are stored as the columns of the
  M-by-NRHS right hand side matrix B and the N-by-NRHS solution
  matrix X.

  The routine first computes a QR factorization with column pivoting:
      A * P = Q * [ R11 R12 ]
                  [  0  R22 ]
  with R11 defined as the largest leading submatrix whose estimated
  condition number is less than 1/RCOND.  The order of R11, RANK,
  is the effective rank of A.

  Then, R22 is considered to be negligible, and R12 is annihilated
  by orthogonal transformations from the right, arriving at the
  complete orthogonal factorization:
     A * P = Q * [ T11 0 ] * Z
                 [  0  0 ]
  The minimum-norm solution is then
     X = P * Z' [ inv(T11)*Q1'*B ]
                [        0       ]
  where Q1 consists of the first RANK columns of Q.

  Arguments
  =========

  M       (input) INTEGER
          The number of rows of the matrix A.  M >= 0.

  N       (input) INTEGER
          The number of columns of the matrix A.  N >= 0.

  NRHS    (input) INTEGER
          The number of right hand sides, i.e., the number of
          columns of matrices B and X. NRHS >= 0.

  A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
          On entry, the M-by-N matrix A.
          On exit, A has been overwritten by details of its
          complete orthogonal factorization.

  LDA     (input) INTEGER
          The leading dimension of the array A.  LDA >= max(1,M).

  B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
          On entry, the M-by-NRHS right hand side matrix B.
          On exit, the N-by-NRHS solution matrix X.
          If m >= n and RANK = n, the residual sum-of-squares for
          the solution in the i-th column is given by the sum of
          squares of elements N+1:M in that column.

  LDB     (input) INTEGER
          The leading dimension of the array B. LDB >= max(1,M,N).

  JPVT    (input/output) INTEGER array, dimension (N)
          On entry, if JPVT(i) .ne. 0, the i-th column of A is an
          initial column, otherwise it is a free column.  Before
          the QR factorization of A, all initial columns are
          permuted to the leading positions; only the remaining
          free columns are moved as a result of column pivoting
          during the factorization.
          On exit, if JPVT(i) = k, then the i-th column of A*P
          was the k-th column of A.

  RCOND   (input) DOUBLE PRECISION
          RCOND is used to determine the effective rank of A, which
          is defined as the order of the largest leading triangular
          submatrix R11 in the QR factorization with pivoting of A,
          whose estimated condition number < 1/RCOND.

  RANK    (output) INTEGER
          The effective rank of A, i.e., the order of the submatrix
          R11.  This is the same as the order of the submatrix T11
          in the complete orthogonal factorization of A.

  WORK    (workspace) DOUBLE PRECISION array, dimension
                      (max( min(M,N)+3*N, 2*min(M,N)+NRHS )),

  INFO    (output) INTEGER
          = 0:  successful exit
          < 0:  if INFO = -i, the i-th argument had an illegal value    "));
        end dgelsx_vec;
        function dgesv "Solve real system of linear equations A*X=B with a B matrix"
          extends Modelica.Icons.Function;
          input Real A[:,size(A, 1)];
          input Real B[size(A, 1),:];
          output Real X[size(A, 1),size(B, 2)] = B;
          output Integer info;
        protected 
          Real Awork[size(A, 1),size(A, 1)] = A;
          Integer ipiv[size(A, 1)];
          annotation(Documentation(info = "Lapack documentation:
    Purpose
    =======
    DGESV computes the solution to a real system of linear equations
       A * X = B,
    where A is an N-by-N matrix and X and B are N-by-NRHS matrices.
    The LU decomposition with partial pivoting and row interchanges is
    used to factor A as
       A = P * L * U,
    where P is a permutation matrix, L is unit lower triangular, and U is

    upper triangular.  The factored form of A is then used to solve the
    system of equations A * X = B.
    Arguments
    =========
    N       (input) INTEGER
            The number of linear equations, i.e., the order of the
            matrix A.  N >= 0.
    NRHS    (input) INTEGER
            The number of right hand sides, i.e., the number of columns
            of the matrix B.  NRHS >= 0.
    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the N-by-N coefficient matrix A.
            On exit, the factors L and U from the factorization
            A = P*L*U; the unit diagonal elements of L are not stored.
    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,N).
    IPIV    (output) INTEGER array, dimension (N)
            The pivot indices that define the permutation matrix P;
            row i of the matrix was interchanged with row IPIV(i).
    B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
            On entry, the N-by-NRHS matrix of right hand side matrix B.
            On exit, if INFO = 0, the N-by-NRHS solution matrix X.
    LDB     (input) INTEGER
            The leading dimension of the array B.  LDB >= max(1,N).
    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value
            > 0:  if INFO = i, U(i,i) is exactly zero.  The factorization

                  has been completed, but the factor U is exactly
                  singular, so the solution could not be computed.
"));

          external "FORTRAN 77" dgesv(size(A, 1),size(B, 2),Awork,size(A, 1),ipiv,X,size(A, 1),info)           annotation(Library = "Lapack"); 
        end dgesv;
        function dgesv_vec "Solve real system of linear equations A*x=b with a b vector"
          extends Modelica.Icons.Function;
          input Real A[:,size(A, 1)];
          input Real b[size(A, 1)];
          output Real x[size(A, 1)] = b;
          output Integer info;
        protected 
          Real Awork[size(A, 1),size(A, 1)] = A;
          Integer ipiv[size(A, 1)];
          annotation(Documentation(info = "
Same as function LAPACK.dgesv, but right hand side is a vector and not a matrix.
For details of the arguments, see documentation of dgesv.
"));

          external "FORTRAN 77" dgesv(size(A, 1),1,Awork,size(A, 1),ipiv,x,size(A, 1),info)           annotation(Library = "Lapack"); 
        end dgesv_vec;
        function dgglse_vec "Solve a linear equality constrained least squares problem"
          extends Modelica.Icons.Function;
          input Real A[:,:] "Minimize |A*x - c|^2";
          input Real c[size(A, 1)];
          input Real B[:,size(A, 2)] "subject to B*x=d";
          input Real d[size(B, 1)];
          output Real x[size(A, 2)] "solution vector";
          output Integer info;
        protected 
          Integer nrow_A = size(A, 1);
          Integer nrow_B = size(B, 1);
          Integer ncol_A = size(A, 2) "(min=nrow_B,max=nrow_A+nrow_B) required";
          Real Awork[nrow_A,ncol_A] = A;
          Real Bwork[nrow_B,ncol_A] = B;
          Real cwork[nrow_A] = c;
          Real dwork[nrow_B] = d;
          Integer lwork = ncol_A + nrow_B + max(nrow_A, max(ncol_A, nrow_B)) * 5;
          Real work[lwork];

          external "FORTRAN 77" dgglse(nrow_A,ncol_A,nrow_B,Awork,nrow_A,Bwork,nrow_B,cwork,dwork,x,work,lwork,info)           annotation(Library = "Lapack");           annotation(Documentation(info = "Lapack documentation

  Purpose
  =======

  DGGLSE solves the linear equality constrained least squares (LSE)
  problem:

          minimize || A*x - c ||_2   subject to B*x = d

  using a generalized RQ factorization of matrices A and B, where A is
  M-by-N, B is P-by-N, assume P <= N <= M+P, and ||.||_2 denotes vector
  2-norm. It is assumed that

                       rank(B) = P                                  (1)

  and the null spaces of A and B intersect only trivially, i.e.,

   intersection of Null(A) and Null(B) = {0} <=> rank( ( A ) ) = N  (2)
                                                     ( ( B ) )

  where N(A) denotes the null space of matrix A. Conditions (1) and (2)
  ensure that the problem LSE has a unique solution.

  Arguments
  =========

  M       (input) INTEGER
          The number of rows of the matrix A.  M >= 0.

  N       (input) INTEGER
          The number of columns of the matrices A and B. N >= 0.
          Assume that P <= N <= M+P.

  P       (input) INTEGER
          The number of rows of the matrix B.  P >= 0.

  A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
          On entry, the P-by-M matrix A.
          On exit, A is destroyed.

  LDA     (input) INTEGER
          The leading dimension of the array A. LDA >= max(1,M).

  B       (input/output) DOUBLE PRECISION array, dimension (LDB,N)
          On entry, the P-by-N matrix B.
          On exit, B is destroyed.

  LDB     (input) INTEGER
          The leading dimension of the array B. LDB >= max(1,P).

  C       (input/output) DOUBLE PRECISION array, dimension (M)
          On entry, C contains the right hand side vector for the
          least squares part of the LSE problem.
          On exit, the residual sum of squares for the solution
          is given by the sum of squares of elements N-P+1 to M of
          vector C.

  D       (input/output) DOUBLE PRECISION array, dimension (P)
          On entry, D contains the right hand side vector for the
          constrained equation.
          On exit, D is destroyed.

  X       (output) DOUBLE PRECISION array, dimension (N)
          On exit, X is the solution of the LSE problem.

  WORK    (workspace) DOUBLE PRECISION array, dimension (LWORK)
          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.

  LWORK   (input) INTEGER
          The dimension of the array WORK. LWORK >= N+P+max(N,M,P).
          For optimum performance LWORK >=
          N+P+max(M,P,N)*max(NB1,NB2), where NB1 is the optimal
          blocksize for the QR factorization of M-by-N matrix A.
          NB2 is the optimal blocksize for the RQ factorization of
          P-by-N matrix B.

  INFO    (output) INTEGER
          = 0:  successful exit.
          < 0:  if INFO = -i, the i-th argument had an illegal value.
"));
        end dgglse_vec;
        function dgtsv "Solve real system of linear equations A*X=B with B matrix and tridiagonal A"
          extends Modelica.Icons.Function;
          input Real superdiag[:];
          input Real diag[size(superdiag, 1) + 1];
          input Real subdiag[size(superdiag, 1)];
          input Real B[size(diag, 1),:];
          output Real X[size(B, 1),size(B, 2)] = B;
          output Integer info;
        protected 
          Real superdiagwork[size(superdiag, 1)] = superdiag;
          Real diagwork[size(diag, 1)] = diag;
          Real subdiagwork[size(subdiag, 1)] = subdiag;
          annotation(Documentation(info = "Lapack documentation:
    Purpose
    =======
    DGTSV  solves the equation
       A*X = B,
    where A is an N-by-N tridiagonal matrix, by Gaussian elimination with

    partial pivoting.
    Note that the equation  A'*X = B  may be solved by interchanging the

    order of the arguments DU and DL.
    Arguments
    =========
    N       (input) INTEGER
            The order of the matrix A.  N >= 0.
    NRHS    (input) INTEGER
            The number of right hand sides, i.e., the number of columns
            of the matrix B.  NRHS >= 0.
    DL      (input/output) DOUBLE PRECISION array, dimension (N-1)
            On entry, DL must contain the (n-1) subdiagonal elements of
            A.
            On exit, DL is overwritten by the (n-2) elements of the
            second superdiagonal of the upper triangular matrix U from
            the LU factorization of A, in DL(1), ..., DL(n-2).
    D       (input/output) DOUBLE PRECISION array, dimension (N)
            On entry, D must contain the diagonal elements of A.
            On exit, D is overwritten by the n diagonal elements of U.
    DU      (input/output) DOUBLE PRECISION array, dimension (N-1)
            On entry, DU must contain the (n-1) superdiagonal elements
            of A.
            On exit, DU is overwritten by the (n-1) elements of the first

            superdiagonal of U.
    B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
            On entry, the N-by-NRHS right hand side matrix B.
            On exit, if INFO = 0, the N-by-NRHS solution matrix X.
    LDB     (input) INTEGER
            The leading dimension of the array B.  LDB >= max(1,N).
    INFO    (output) INTEGER
            = 0:  successful exit
            < 0:  if INFO = -i, the i-th argument had an illegal value
            > 0:  if INFO = i, U(i,i) is exactly zero, and the solution
                  has not been computed.  The factorization has not been

                  completed unless i = N.
"));

          external "FORTRAN 77" dgtsv(size(diag, 1),size(B, 2),subdiagwork,diagwork,superdiagwork,X,size(B, 1),info)           annotation(Library = "Lapack"); 
        end dgtsv;
        function dgtsv_vec "Solve real system of linear equations A*x=b with b vector and tridiagonal A"
          extends Modelica.Icons.Function;
          input Real superdiag[:];
          input Real diag[size(superdiag, 1) + 1];
          input Real subdiag[size(superdiag, 1)];
          input Real b[size(diag, 1)];
          output Real x[size(b, 1)] = b;
          output Integer info;
        protected 
          Real superdiagwork[size(superdiag, 1)] = superdiag;
          Real diagwork[size(diag, 1)] = diag;
          Real subdiagwork[size(subdiag, 1)] = subdiag;
          annotation(Documentation(info = "
Same as function LAPACK.dgtsv, but right hand side is a vector and not a matrix.
For details of the arguments, see documentation of dgtsv.
"));

          external "FORTRAN 77" dgtsv(size(diag, 1),1,subdiagwork,diagwork,superdiagwork,x,size(b, 1),info)           annotation(Library = "Lapack"); 
        end dgtsv_vec;
        function dgbsv "Solve real system of linear equations A*X=B with a B matrix"
          extends Modelica.Icons.Function;
          input Integer n "Number of equations";
          input Integer kLower "Number of lower bands";
          input Integer kUpper "Number of upper bands";
          input Real A[2 * kLower + kUpper + 1,n];
          input Real B[n,:];
          output Real X[n,size(B, 2)] = B;
          output Integer info;
        protected 
          Real Awork[size(A, 1),size(A, 2)] = A;
          Integer ipiv[n];
          annotation(Documentation(info = "Lapack documentation:
Purpose
=======
DGBSV computes the solution to a real system of linear equations
A * X = B, where A is a band matrix of order N with KL subdiagonals
and KU superdiagonals, and X and B are N-by-NRHS matrices.
The LU decomposition with partial pivoting and row interchanges is
used to factor A as A = L * U, where L is a product of permutation
and unit lower triangular matrices with KL subdiagonals, and U is
upper triangular with KL+KU superdiagonals.  The factored form of A
is then used to solve the system of equations A * X = B.
Arguments
=========
N       (input) INTEGER
        The number of linear equations, i.e., the order of the
        matrix A.  N >= 0.
KL      (input) INTEGER
        The number of subdiagonals within the band of A.  KL >= 0.
KU      (input) INTEGER
        The number of superdiagonals within the band of A.  KU >= 0.
NRHS    (input) INTEGER
        The number of right hand sides, i.e., the number of columns
        of the matrix B.  NRHS >= 0.
AB      (input/output) DOUBLE PRECISION array, dimension (LDAB,N)
        On entry, the matrix A in band storage, in rows KL+1 to
        2*KL+KU+1; rows 1 to KL of the array need not be set.
        The j-th column of A is stored in the j-th column of the
        array AB as follows:
        AB(KL+KU+1+i-j,j) = A(i,j) for max(1,j-KU)<=i<=min(N,j+KL)
        On exit, details of the factorization: U is stored as an
        upper triangular band matrix with KL+KU superdiagonals in
        rows 1 to KL+KU+1, and the multipliers used during the
        factorization are stored in rows KL+KU+2 to 2*KL+KU+1.
        See below for further details.
LDAB    (input) INTEGER
        The leading dimension of the array AB.  LDAB >= 2*KL+KU+1.
IPIV    (output) INTEGER array, dimension (N)
        The pivot indices that define the permutation matrix P;
        row i of the matrix was interchanged with row IPIV(i).
B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
        On entry, the N-by-NRHS right hand side matrix B.
        On exit, if INFO = 0, the N-by-NRHS solution matrix X.
LDB     (input) INTEGER
        The leading dimension of the array B.  LDB >= max(1,N).
INFO    (output) INTEGER
        = 0:  successful exit
        < 0:  if INFO = -i, the i-th argument had an illegal value
        > 0:  if INFO = i, U(i,i) is exactly zero.  The factorization
              has been completed, but the factor U is exactly
              singular, and the solution has not been computed.
Further Details
===============
The band storage scheme is illustrated by the following example, when
M = N = 6, KL = 2, KU = 1:
On entry:                       On exit:
    *    *    *    +    +    +       *    *    *   u14  u25  u36
    *    *    +    +    +    +       *    *   u13  u24  u35  u46
    *   a12  a23  a34  a45  a56      *   u12  u23  u34  u45  u56
   a11  a22  a33  a44  a55  a66     u11  u22  u33  u44  u55  u66
   a21  a32  a43  a54  a65   *      m21  m32  m43  m54  m65   *
   a31  a42  a53  a64   *    *      m31  m42  m53  m64   *    *
Array elements marked * are not used by the routine; elements marked
+ need not be set on entry, but are required by the routine to store
elements of U because of fill-in resulting from the row interchanges."));

          external "FORTRAN 77" dgbsv(n,kLower,kUpper,size(B, 2),Awork,size(Awork, 1),ipiv,X,n,info)           annotation(Library = "Lapack"); 
        end dgbsv;
        function dgbsv_vec "Solve real system of linear equations A*x=b with a b vector"
          extends Modelica.Icons.Function;
          input Integer n "Number of equations";
          input Integer kLower "Number of lower bands";
          input Integer kUpper "Number of upper bands";
          input Real A[2 * kLower + kUpper + 1,n];
          input Real b[n];
          output Real x[n] = b;
          output Integer info;
        protected 
          Real Awork[size(A, 1),size(A, 2)] = A;
          Integer ipiv[n];
          annotation(Documentation(info = "Lapack documentation:
"));

          external "FORTRAN 77" dgbsv(n,kLower,kUpper,1,Awork,size(Awork, 1),ipiv,x,n,info)           annotation(Library = "Lapack"); 
        end dgbsv_vec;
        function dgesvd "Determine singular value decomposition"
          extends Modelica.Icons.Function;
          input Real A[:,:];
          output Real sigma[min(size(A, 1), size(A, 2))];
          output Real U[size(A, 1),size(A, 1)] = zeros(size(A, 1), size(A, 1));
          output Real VT[size(A, 2),size(A, 2)] = zeros(size(A, 2), size(A, 2));
          output Integer info;
        protected 
          Real Awork[size(A, 1),size(A, 2)] = A;
          Integer lwork = 5 * size(A, 1) + 5 * size(A, 2);
          Real work[lwork];
          annotation(Documentation(info = "Lapack documentation:
    Purpose
    =======
    DGESVD computes the singular value decomposition (SVD) of a real
    M-by-N matrix A, optionally computing the left and/or right singular

    vectors. The SVD is written
         A = U * SIGMA * transpose(V)
    where SIGMA is an M-by-N matrix which is zero except for its
    min(m,n) diagonal elements, U is an M-by-M orthogonal matrix, and
    V is an N-by-N orthogonal matrix.  The diagonal elements of SIGMA
    are the singular values of A; they are real and non-negative, and
    are returned in descending order.  The first min(m,n) columns of
    U and V are the left and right singular vectors of A.
    Note that the routine returns V**T, not V.
    Arguments
    =========
    JOBU    (input) CHARACTER*1
            Specifies options for computing all or part of the matrix U:

            = 'A':  all M columns of U are returned in array U:
            = 'S':  the first min(m,n) columns of U (the left singular
                    vectors) are returned in the array U;
            = 'O':  the first min(m,n) columns of U (the left singular
                    vectors) are overwritten on the array A;
            = 'N':  no columns of U (no left singular vectors) are
                    computed.
    JOBVT   (input) CHARACTER*1
            Specifies options for computing all or part of the matrix
            V**T:
            = 'A':  all N rows of V**T are returned in the array VT;
            = 'S':  the first min(m,n) rows of V**T (the right singular
                    vectors) are returned in the array VT;
            = 'O':  the first min(m,n) rows of V**T (the right singular
                    vectors) are overwritten on the array A;
            = 'N':  no rows of V**T (no right singular vectors) are
                    computed.
            JOBVT and JOBU cannot both be 'O'.
    M       (input) INTEGER
            The number of rows of the input matrix A.  M >= 0.
    N       (input) INTEGER
            The number of columns of the input matrix A.  N >= 0.
    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the M-by-N matrix A.
            On exit,
            if JOBU = 'O',  A is overwritten with the first min(m,n)
                            columns of U (the left singular vectors,
                            stored columnwise);
            if JOBVT = 'O', A is overwritten with the first min(m,n)
                            rows of V**T (the right singular vectors,
                            stored rowwise);
            if JOBU .ne. 'O' and JOBVT .ne. 'O', the contents of A
                            are destroyed.
    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,M).
    S       (output) DOUBLE PRECISION array, dimension (min(M,N))
            The singular values of A, sorted so that S(i) >= S(i+1).
    U       (output) DOUBLE PRECISION array, dimension (LDU,UCOL)
            (LDU,M) if JOBU = 'A' or (LDU,min(M,N)) if JOBU = 'S'.
            If JOBU = 'A', U contains the M-by-M orthogonal matrix U;
            if JOBU = 'S', U contains the first min(m,n) columns of U
            (the left singular vectors, stored columnwise);
            if JOBU = 'N' or 'O', U is not referenced.
    LDU     (input) INTEGER
            The leading dimension of the array U.  LDU >= 1; if
            JOBU = 'S' or 'A', LDU >= M.
    VT      (output) DOUBLE PRECISION array, dimension (LDVT,N)
            If JOBVT = 'A', VT contains the N-by-N orthogonal matrix
            V**T;
            if JOBVT = 'S', VT contains the first min(m,n) rows of
            V**T (the right singular vectors, stored rowwise);
            if JOBVT = 'N' or 'O', VT is not referenced.
    LDVT    (input) INTEGER
            The leading dimension of the array VT.  LDVT >= 1; if
            JOBVT = 'A', LDVT >= N; if JOBVT = 'S', LDVT >= min(M,N).
    WORK    (workspace/output) DOUBLE PRECISION array, dimension (LWORK)

            On exit, if INFO = 0, WORK(1) returns the optimal LWORK;
            if INFO > 0, WORK(2:MIN(M,N)) contains the unconverged
            superdiagonal elements of an upper bidiagonal matrix B
            whose diagonal is in S (not necessarily sorted). B
            satisfies A = U * B * VT, so it has the same singular values

            as A, and singular vectors related by U and VT.
    LWORK   (input) INTEGER
            The dimension of the array WORK. LWORK >= 1.
            LWORK >= MAX(3*MIN(M,N)+MAX(M,N),5*MIN(M,N)-4).
            For good performance, LWORK should generally be larger.
    INFO    (output) INTEGER
            = 0:  successful exit.
            < 0:  if INFO = -i, the i-th argument had an illegal value.
            > 0:  if DBDSQR did not converge, INFO specifies how many
                  superdiagonals of an intermediate bidiagonal form B
                  did not converge to zero. See the description of WORK
                  above for details.
"));

          external "Fortran 77" dgesvd("A","A",size(A, 1),size(A, 2),Awork,size(A, 1),sigma,U,size(A, 1),VT,size(A, 2),work,lwork,info)           annotation(Library = "Lapack"); 
        end dgesvd;
        function dgesvd_sigma "Determine singular values"
          extends Modelica.Icons.Function;
          input Real A[:,:];
          output Real sigma[min(size(A, 1), size(A, 2))];
          output Integer info;
        protected 
          Real Awork[size(A, 1),size(A, 2)] = A;
          Real U[size(A, 1),size(A, 1)];
          Real VT[size(A, 2),size(A, 2)];
          Integer lwork = 5 * size(A, 1) + 5 * size(A, 2);
          Real work[lwork];
          annotation(Documentation(info = "Lapack documentation:
    Purpose
    =======
    DGESVD computes the singular value decomposition (SVD) of a real
    M-by-N matrix A, optionally computing the left and/or right singular

    vectors. The SVD is written
         A = U * SIGMA * transpose(V)
    where SIGMA is an M-by-N matrix which is zero except for its
    min(m,n) diagonal elements, U is an M-by-M orthogonal matrix, and
    V is an N-by-N orthogonal matrix.  The diagonal elements of SIGMA
    are the singular values of A; they are real and non-negative, and
    are returned in descending order.  The first min(m,n) columns of
    U and V are the left and right singular vectors of A.
    Note that the routine returns V**T, not V.
    Arguments
    =========
    JOBU    (input) CHARACTER*1
            Specifies options for computing all or part of the matrix U:

            = 'A':  all M columns of U are returned in array U:
            = 'S':  the first min(m,n) columns of U (the left singular
                    vectors) are returned in the array U;
            = 'O':  the first min(m,n) columns of U (the left singular
                    vectors) are overwritten on the array A;
            = 'N':  no columns of U (no left singular vectors) are
                    computed.
    JOBVT   (input) CHARACTER*1
            Specifies options for computing all or part of the matrix
            V**T:
            = 'A':  all N rows of V**T are returned in the array VT;
            = 'S':  the first min(m,n) rows of V**T (the right singular
                    vectors) are returned in the array VT;
            = 'O':  the first min(m,n) rows of V**T (the right singular
                    vectors) are overwritten on the array A;
            = 'N':  no rows of V**T (no right singular vectors) are
                    computed.
            JOBVT and JOBU cannot both be 'O'.
    M       (input) INTEGER
            The number of rows of the input matrix A.  M >= 0.
    N       (input) INTEGER
            The number of columns of the input matrix A.  N >= 0.
    A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
            On entry, the M-by-N matrix A.
            On exit,
            if JOBU = 'O',  A is overwritten with the first min(m,n)
                            columns of U (the left singular vectors,
                            stored columnwise);
            if JOBVT = 'O', A is overwritten with the first min(m,n)
                            rows of V**T (the right singular vectors,
                            stored rowwise);
            if JOBU .ne. 'O' and JOBVT .ne. 'O', the contents of A
                            are destroyed.
    LDA     (input) INTEGER
            The leading dimension of the array A.  LDA >= max(1,M).
    S       (output) DOUBLE PRECISION array, dimension (min(M,N))
            The singular values of A, sorted so that S(i) >= S(i+1).
    U       (output) DOUBLE PRECISION array, dimension (LDU,UCOL)
            (LDU,M) if JOBU = 'A' or (LDU,min(M,N)) if JOBU = 'S'.
            If JOBU = 'A', U contains the M-by-M orthogonal matrix U;
            if JOBU = 'S', U contains the first min(m,n) columns of U
            (the left singular vectors, stored columnwise);
            if JOBU = 'N' or 'O', U is not referenced.
    LDU     (input) INTEGER
            The leading dimension of the array U.  LDU >= 1; if
            JOBU = 'S' or 'A', LDU >= M.
    VT      (output) DOUBLE PRECISION array, dimension (LDVT,N)
            If JOBVT = 'A', VT contains the N-by-N orthogonal matrix
            V**T;
            if JOBVT = 'S', VT contains the first min(m,n) rows of
            V**T (the right singular vectors, stored rowwise);
            if JOBVT = 'N' or 'O', VT is not referenced.
    LDVT    (input) INTEGER
            The leading dimension of the array VT.  LDVT >= 1; if
            JOBVT = 'A', LDVT >= N; if JOBVT = 'S', LDVT >= min(M,N).
    WORK    (workspace/output) DOUBLE PRECISION array, dimension (LWORK)

            On exit, if INFO = 0, WORK(1) returns the optimal LWORK;
            if INFO > 0, WORK(2:MIN(M,N)) contains the unconverged
            superdiagonal elements of an upper bidiagonal matrix B
            whose diagonal is in S (not necessarily sorted). B
            satisfies A = U * B * VT, so it has the same singular values

            as A, and singular vectors related by U and VT.
    LWORK   (input) INTEGER
            The dimension of the array WORK. LWORK >= 1.
            LWORK >= MAX(3*MIN(M,N)+MAX(M,N),5*MIN(M,N)-4).
            For good performance, LWORK should generally be larger.
    INFO    (output) INTEGER
            = 0:  successful exit.
            < 0:  if INFO = -i, the i-th argument had an illegal value.
            > 0:  if DBDSQR did not converge, INFO specifies how many
                  superdiagonals of an intermediate bidiagonal form B
                  did not converge to zero. See the description of WORK
                  above for details.
"));

          external "Fortran 77" dgesvd("N","N",size(A, 1),size(A, 2),Awork,size(A, 1),sigma,U,size(A, 1),VT,size(A, 2),work,lwork,info)           annotation(Library = "Lapack"); 
        end dgesvd_sigma;
        function dgetrf "Compute LU factorization of square or rectangular matrix A (A = P*L*U)"
          extends Modelica.Icons.Function;
          input Real A[:,:] "Square or rectangular matrix";
          output Real LU[size(A, 1),size(A, 2)] = A;
          output Integer pivots[min(size(A, 1), size(A, 2))] "Pivot vector";
          output Integer info "Information";
          annotation(Documentation(info = "Lapack documentation:
  SUBROUTINE DGETRF( M, N, A, LDA, IPIV, INFO )
-- LAPACK routine (version 1.1) --
   Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
   Courant Institute, Argonne National Lab, and Rice University
   March 31, 1993
   .. Scalar Arguments ..
   INTEGER            INFO, LDA, M, N
   ..
   .. Array Arguments ..
   INTEGER            IPIV( * )
   DOUBLE PRECISION   A( LDA, * )
   ..
Purpose
=======
DGETRF computes an LU factorization of a general M-by-N matrix A
using partial pivoting with row interchanges.
The factorization has the form
   A = P * L * U
where P is a permutation matrix, L is lower triangular with unit
diagonal elements (lower trapezoidal if m > n), and U is upper
triangular (upper trapezoidal if m < n).
This is the right-looking Level 3 BLAS version of the algorithm.
Arguments
=========
M       (input) INTEGER
        The number of rows of the matrix A.  M >= 0.
N       (input) INTEGER
        The number of columns of the matrix A.  N >= 0.
A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
        On entry, the M-by-N matrix to be factored.
        On exit, the factors L and U from the factorization
        A = P*L*U; the unit diagonal elements of L are not stored.
LDA     (input) INTEGER
        The leading dimension of the array A.  LDA >= max(1,M).
IPIV    (output) INTEGER array, dimension (min(M,N))
        The pivot indices; for 1 <= i <= min(M,N), row i of the
        matrix was interchanged with row IPIV(i).
INFO    (output) INTEGER
        = 0:  successful exit
        < 0:  if INFO = -i, the i-th argument had an illegal value
        > 0:  if INFO = i, U(i,i) is exactly zero. The factorization
              has been completed, but the factor U is exactly
              singular, and division by zero will occur if it is used
              to solve a system of equations.
"));

          external "FORTRAN 77" dgetrf(size(A, 1),size(A, 2),LU,size(A, 1),pivots,info)           annotation(Library = "Lapack"); 
        end dgetrf;
        function dgetrs "Solves a system of linear equations with the LU decomposition from dgetrf(..)"
          extends Modelica.Icons.Function;
          input Real LU[:,size(LU, 1)] "LU factorization of dgetrf of a square matrix";
          input Integer pivots[size(LU, 1)] "Pivot vector of dgetrf";
          input Real B[size(LU, 1),:] "Right hand side matrix B";
          output Real X[size(B, 1),size(B, 2)] = B "Solution matrix X";
          annotation(Documentation(info = "Lapack documentation:
  SUBROUTINE DGETRS( TRANS, N, NRHS, A, LDA, IPIV, B, LDB, INFO )
-- LAPACK routine (version 1.1) --
   Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
   Courant Institute, Argonne National Lab, and Rice University
   March 31, 1993
   .. Scalar Arguments ..
   CHARACTER          TRANS
   INTEGER            INFO, LDA, LDB, N, NRHS
   ..
   .. Array Arguments ..
   INTEGER            IPIV( * )
   DOUBLE PRECISION   A( LDA, * ), B( LDB, * )
   ..
Purpose
=======
DGETRS solves a system of linear equations
   A * X = B  or  A' * X = B
with a general N-by-N matrix A using the LU factorization computed
by DGETRF.
Arguments
=========
TRANS   (input) CHARACTER*1
        Specifies the form of the system of equations:
        = 'N':  A * X = B  (No transpose)
        = 'T':  A'* X = B  (Transpose)
        = 'C':  A'* X = B  (Conjugate transpose = Transpose)
N       (input) INTEGER
        The order of the matrix A.  N >= 0.
NRHS    (input) INTEGER
        The number of right hand sides, i.e., the number of columns
        of the matrix B.  NRHS >= 0.
A       (input) DOUBLE PRECISION array, dimension (LDA,N)
        The factors L and U from the factorization A = P*L*U
        as computed by DGETRF.
LDA     (input) INTEGER
        The leading dimension of the array A.  LDA >= max(1,N).
IPIV    (input) INTEGER array, dimension (N)
        The pivot indices from DGETRF; for 1<=i<=N, row i of the
        matrix was interchanged with row IPIV(i).
B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
        On entry, the right hand side matrix B.
        On exit, the solution matrix X.
LDB     (input) INTEGER
        The leading dimension of the array B.  LDB >= max(1,N).
INFO    (output) INTEGER
        = 0:  successful exit
        < 0:  if INFO = -i, the i-th argument had an illegal value
"));
        protected 
          Real work[size(LU, 1),size(LU, 1)] = LU;
          Integer info;

          external "FORTRAN 77" dgetrs("N",size(LU, 1),size(B, 2),work,size(LU, 1),pivots,X,size(B, 1),info)           annotation(Library = "Lapack"); 
        end dgetrs;
        function dgetrs_vec "Solves a system of linear equations with the LU decomposition from dgetrf(..)"
          extends Modelica.Icons.Function;
          input Real LU[:,size(LU, 1)] "LU factorization of dgetrf of a square matrix";
          input Integer pivots[size(LU, 1)] "Pivot vector of dgetrf";
          input Real b[size(LU, 1)] "Right hand side vector b";
          output Real x[size(b, 1)] = b;
          annotation(Documentation(info = "Lapack documentation:
  SUBROUTINE DGETRS( TRANS, N, NRHS, A, LDA, IPIV, B, LDB, INFO )
-- LAPACK routine (version 1.1) --
   Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
   Courant Institute, Argonne National Lab, and Rice University
   March 31, 1993
   .. Scalar Arguments ..
   CHARACTER          TRANS
   INTEGER            INFO, LDA, LDB, N, NRHS
   ..
   .. Array Arguments ..
   INTEGER            IPIV( * )
   DOUBLE PRECISION   A( LDA, * ), B( LDB, * )
   ..
Purpose
=======
DGETRS solves a system of linear equations
   A * X = B  or  A' * X = B
with a general N-by-N matrix A using the LU factorization computed
by DGETRF.
Arguments
=========
TRANS   (input) CHARACTER*1
        Specifies the form of the system of equations:
        = 'N':  A * X = B  (No transpose)
        = 'T':  A'* X = B  (Transpose)
        = 'C':  A'* X = B  (Conjugate transpose = Transpose)
N       (input) INTEGER
        The order of the matrix A.  N >= 0.
NRHS    (input) INTEGER
        The number of right hand sides, i.e., the number of columns
        of the matrix B.  NRHS >= 0.
A       (input) DOUBLE PRECISION array, dimension (LDA,N)
        The factors L and U from the factorization A = P*L*U
        as computed by DGETRF.
LDA     (input) INTEGER
        The leading dimension of the array A.  LDA >= max(1,N).
IPIV    (input) INTEGER array, dimension (N)
        The pivot indices from DGETRF; for 1<=i<=N, row i of the
        matrix was interchanged with row IPIV(i).
B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
        On entry, the right hand side matrix B.
        On exit, the solution matrix X.
LDB     (input) INTEGER
        The leading dimension of the array B.  LDB >= max(1,N).
INFO    (output) INTEGER
        = 0:  successful exit
        < 0:  if INFO = -i, the i-th argument had an illegal value
"));
        protected 
          Real work[size(LU, 1),size(LU, 1)] = LU;
          Integer info;

          external "FORTRAN 77" dgetrs("N",size(LU, 1),1,work,size(LU, 1),pivots,x,size(b, 1),info)           annotation(Library = "Lapack"); 
        end dgetrs_vec;
        function dgetri "Computes the inverse of a matrix using the LU factorization from dgetrf(..)"
          extends Modelica.Icons.Function;
          input Real LU[:,size(LU, 1)] "LU factorization of dgetrf of a square matrix";
          input Integer pivots[size(LU, 1)] "Pivot vector of dgetrf";
          output Real inv[size(LU, 1),size(LU, 2)] = LU "Inverse of matrix P*L*U";
          annotation(Documentation(info = "Lapack documentation:
   SUBROUTINE DGETRI( N, A, LDA, IPIV, WORK, LWORK, INFO )
-- LAPACK routine (version 1.1) --
   Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
   Courant Institute, Argonne National Lab, and Rice University
   March 31, 1993
   .. Scalar Arguments ..
   INTEGER            INFO, LDA, LWORK, N
   ..
   .. Array Arguments ..
   INTEGER            IPIV( * )
   DOUBLE PRECISION   A( LDA, * ), WORK( LWORK )
   ..
Purpose
=======
DGETRI computes the inverse of a matrix using the LU factorization
computed by DGETRF.
This method inverts U and then computes inv(A) by solving the system
inv(A)*L = inv(U) for inv(A).
Arguments
=========
N       (input) INTEGER
        The order of the matrix A.  N >= 0.
A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
        On entry, the factors L and U from the factorization
        A = P*L*U as computed by DGETRF.
        On exit, if INFO = 0, the inverse of the original matrix A.
LDA     (input) INTEGER
        The leading dimension of the array A.  LDA >= max(1,N).
IPIV    (input) INTEGER array, dimension (N)
        The pivot indices from DGETRF; for 1<=i<=N, row i of the
        matrix was interchanged with row IPIV(i).
WORK    (workspace) DOUBLE PRECISION array, dimension (LWORK)
        On exit, if INFO=0, then WORK(1) returns the optimal LWORK.
LWORK   (input) INTEGER
        The dimension of the array WORK.  LWORK >= max(1,N).
        For optimal performance LWORK >= N*NB, where NB is
        the optimal blocksize returned by ILAENV.
INFO    (output) INTEGER
        = 0:  successful exit
        < 0:  if INFO = -i, the i-th argument had an illegal value
        > 0:  if INFO = i, U(i,i) is exactly zero; the matrix is
              singular and its inverse could not be computed."));
        protected 
          Integer lwork = min(10, size(LU, 1)) * size(LU, 1) "Length of work array";
          Real work[lwork];
          Integer info;

          external "FORTRAN 77" dgetri(size(LU, 1),inv,size(LU, 1),pivots,work,lwork,info)           annotation(Library = "Lapack"); 
        end dgetri;
        function dgeqpf "Compute QR factorization of square or rectangular matrix A with column pivoting (A(:,p) = Q*R)"
          extends Modelica.Icons.Function;
          input Real A[:,:] "Square or rectangular matrix";
          output Real QR[size(A, 1),size(A, 2)] = A "QR factorization in packed format";
          output Real tau[min(size(A, 1), size(A, 2))] "The scalar factors of the elementary reflectors of Q";
          output Integer p[size(A, 2)] = zeros(size(A, 2)) "Pivot vector";
          annotation(Documentation(info = "Lapack documentation:
   SUBROUTINE DGEQPF( M, N, A, LDA, JPVT, TAU, WORK, INFO )
-- LAPACK test routine (version 1.1) --
   Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
   Courant Institute, Argonne National Lab, and Rice University
   March 31, 1993
   .. Scalar Arguments ..
   INTEGER            INFO, LDA, M, N
   ..
   .. Array Arguments ..
   INTEGER            JPVT( * )
   DOUBLE PRECISION   A( LDA, * ), TAU( * ), WORK( * )
   ..
Purpose
=======
DGEQPF computes a QR factorization with column pivoting of a
real M-by-N matrix A: A*P = Q*R.
Arguments
=========
M       (input) INTEGER
        The number of rows of the matrix A. M >= 0.
N       (input) INTEGER
        The number of columns of the matrix A. N >= 0
A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
        On entry, the M-by-N matrix A.
        On exit, the upper triangle of the array contains the
        min(M,N)-by-N upper triangular matrix R; the elements
        below the diagonal, together with the array TAU,
        represent the orthogonal matrix Q as a product of
        min(m,n) elementary reflectors.
LDA     (input) INTEGER
        The leading dimension of the array A. LDA >= max(1,M).
JPVT    (input/output) INTEGER array, dimension (N)
        On entry, if JPVT(i) .ne. 0, the i-th column of A is permuted
        to the front of A*P (a leading column); if JPVT(i) = 0,
        the i-th column of A is a free column.
        On exit, if JPVT(i) = k, then the i-th column of A*P
        was the k-th column of A.
TAU     (output) DOUBLE PRECISION array, dimension (min(M,N))
        The scalar factors of the elementary reflectors.
WORK    (workspace) DOUBLE PRECISION array, dimension (3*N)
INFO    (output) INTEGER
        = 0:  successful exit
        < 0:  if INFO = -i, the i-th argument had an illegal value
Further Details
===============
The matrix Q is represented as a product of elementary reflectors
   Q = H(1) H(2) . . . H(n)
Each H(i) has the form
   H = I - tau * v * v'
where tau is a real scalar, and v is a real vector with
v(1:i-1) = 0 and v(i) = 1; v(i+1:m) is stored on exit in A(i+1:m,i).
The matrix P is represented in jpvt as follows: If
   jpvt(j) = i
then the jth column of P is the ith canonical unit vector."));
        protected 
          Integer info;
          Integer ncol = size(A, 2) "Column dimension of A";
          Real work[3 * ncol] "work array";

          external "FORTRAN 77" dgeqpf(size(A, 1),ncol,QR,size(A, 1),p,tau,work,info)           annotation(Library = {"Lapack"}); 
        end dgeqpf;
        function dorgqr "Generates a Real orthogonal matrix Q which is defined as the product of elementary reflectors as returned from dgeqpf"
          extends Modelica.Icons.Function;
          input Real QR[:,:] "QR from dgeqpf";
          input Real tau[min(size(QR, 1), size(QR, 2))] "The scalar factors of the elementary reflectors of Q";
          output Real Q[size(QR, 1),size(QR, 2)] = QR "Orthogonal matrix Q";
          annotation(Documentation(info = "Lapack documentation:
   SUBROUTINE DORGQR( M, N, K, A, LDA, TAU, WORK, LWORK, INFO )
-- LAPACK routine (version 1.1) --
   Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
   Courant Institute, Argonne National Lab, and Rice University
   March 31, 1993
   .. Scalar Arguments ..
   INTEGER            INFO, K, LDA, LWORK, M, N
   ..
   .. Array Arguments ..
   DOUBLE PRECISION   A( LDA, * ), TAU( * ), WORK( LWORK )
   ..
Purpose
=======
DORGQR generates an M-by-N real matrix Q with orthonormal columns,
which is defined as the first N columns of a product of K elementary
reflectors of order M
      Q  =  H(1) H(2) . . . H(k)
as returned by DGEQRF.
Arguments
=========
M       (input) INTEGER
        The number of rows of the matrix Q. M >= 0.
N       (input) INTEGER
        The number of columns of the matrix Q. M >= N >= 0.
K       (input) INTEGER
        The number of elementary reflectors whose product defines the
        matrix Q. N >= K >= 0.
A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
        On entry, the i-th column must contain the vector which
        defines the elementary reflector H(i), for i = 1,2,...,k, as
        returned by DGEQRF in the first k columns of its array
        argument A.
        On exit, the M-by-N matrix Q.
LDA     (input) INTEGER
        The first dimension of the array A. LDA >= max(1,M).
TAU     (input) DOUBLE PRECISION array, dimension (K)
        TAU(i) must contain the scalar factor of the elementary
        reflector H(i), as returned by DGEQRF.
WORK    (workspace) DOUBLE PRECISION array, dimension (LWORK)
        On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
LWORK   (input) INTEGER
        The dimension of the array WORK. LWORK >= max(1,N).
        For optimum performance LWORK >= N*NB, where NB is the
        optimal blocksize.
INFO    (output) INTEGER
        = 0:  successful exit
        < 0:  if INFO = -i, the i-th argument has an illegal value
"));
        protected 
          Integer info;
          Integer lwork = min(10, size(QR, 2)) * size(QR, 2) "Length of work array";
          Real work[lwork];

          external "FORTRAN 77" dorgqr(size(QR, 1),size(QR, 2),size(tau, 1),Q,size(Q, 1),tau,work,lwork,info)           annotation(Library = {"Lapack"}); 
        end dorgqr;
        annotation(Documentation(info = "<html>
<p>
This package contains external Modelica functions as interface to the
LAPACK library
(<a href=\"http://www.netlib.org/lapack\">http://www.netlib.org/lapack</a>)
that provides FORTRAN subroutines to solve linear algebra
tasks. Usually, these functions are not directly called, but only via
the much more convenient interface of
<a href=\"Modelica://Modelica.Math.Matrices\">Modelica.Math.Matrices</a>.
The documentation of the LAPACK functions is a copy of the original
FORTRAN code.
</p>

<p>
The details of LAPACK are described in:
</p>

<dl>
<dt>Anderson E., Bai Z., Bischof C., Blackford S., Demmel J., Dongarra J.,
    Du Croz J., Greenbaum A., Hammarling S., McKenney A., and Sorensen D.:</dt>
<dd> <b>Lapack Users' Guide</b>.
     Third Edition, SIAM, 1999.</dd>
</dl>

<p>
This package contains a direct interface to the LAPACK subroutines
</p>

</html>"));
      end LAPACK;
    end Matrices;
    function sin "Sine"
      extends baseIcon1;
      input SI.Angle u;
      output Real y;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,0},{ -68.7,34.2},{ -61.5,53.1},{ -55.1,66.4},{ -49.4,74.6},{ -43.8,79.1},{ -38.2,79.8},{ -32.6,76.6},{ -26.9,69.7},{ -21.3,59.4},{ -14.9,44.1},{ -6.83,21.2},{10.1, -30.8},{17.3, -50.2},{23.7, -64.2},{29.3, -73.1},{35, -78.4},{40.6, -80},{46.2, -77.6},{51.9, -71.5},{57.5, -61.9},{63.9, -47.2},{72, -24.8},{80,0}}, color = {0,0,0}),Text(extent = {{12,84},{84,36}}, lineColor = {192,192,192}, textString = "sin")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -100,0},{84,0}}, color = {95,95,95}),Polygon(points = {{100,0},{84,6},{84, -6},{100,0}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80,0},{ -68.7,34.2},{ -61.5,53.1},{ -55.1,66.4},{ -49.4,74.6},{ -43.8,79.1},{ -38.2,79.8},{ -32.6,76.6},{ -26.9,69.7},{ -21.3,59.4},{ -14.9,44.1},{ -6.83,21.2},{10.1, -30.8},{17.3, -50.2},{23.7, -64.2},{29.3, -73.1},{35, -78.4},{40.6, -80},{46.2, -77.6},{51.9, -71.5},{57.5, -61.9},{63.9, -47.2},{72, -24.8},{80,0}}, color = {0,0,255}, thickness = 0.5),Text(extent = {{ -105,72},{ -85,88}}, textString = "1", lineColor = {0,0,255}),Text(extent = {{70,25},{90,5}}, textString = "2*pi", lineColor = {0,0,255}),Text(extent = {{ -103, -72},{ -83, -88}}, textString = "-1", lineColor = {0,0,255}),Text(extent = {{82, -6},{102, -26}}, lineColor = {95,95,95}, textString = "u"),Line(points = {{ -80,80},{ -28,80}}, color = {175,175,175}, smooth = Smooth.None),Line(points = {{ -80, -80},{50, -80}}, color = {175,175,175}, smooth = Smooth.None)}), Documentation(info = "<html>
<p>
This function returns y = sin(u), with -&infin; &lt; u &lt; &infin;:
</p>

<p>
<img src=\"../Images/Math/sin.png\">
</p>
</html>"));

      external "C" y = sin(u) ;      annotation(Library = "ModelicaExternalC");
    end sin;
    function cos "Cosine"
      extends baseIcon1;
      input SI.Angle u;
      output Real y;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,80},{ -74.4,78.1},{ -68.7,72.3},{ -63.1,63},{ -56.7,48.7},{ -48.6,26.6},{ -29.3, -32.5},{ -22.1, -51.7},{ -15.7, -65.3},{ -10.1, -73.8},{ -4.42, -78.8},{1.21, -79.9},{6.83, -77.1},{12.5, -70.6},{18.1, -60.6},{24.5, -45.7},{32.6, -23},{50.3,31.3},{57.5,50.7},{63.9,64.6},{69.5,73.4},{75.2,78.6},{80,80}}, color = {0,0,0}),Text(extent = {{ -36,82},{36,34}}, lineColor = {192,192,192}, textString = "cos")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -103,72},{ -83,88}}, textString = "1", lineColor = {0,0,255}),Text(extent = {{ -103, -72},{ -83, -88}}, textString = "-1", lineColor = {0,0,255}),Text(extent = {{70,25},{90,5}}, textString = "2*pi", lineColor = {0,0,255}),Line(points = {{ -100,0},{84,0}}, color = {95,95,95}),Polygon(points = {{98,0},{82,6},{82, -6},{98,0}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80,80},{ -74.4,78.1},{ -68.7,72.3},{ -63.1,63},{ -56.7,48.7},{ -48.6,26.6},{ -29.3, -32.5},{ -22.1, -51.7},{ -15.7, -65.3},{ -10.1, -73.8},{ -4.42, -78.8},{1.21, -79.9},{6.83, -77.1},{12.5, -70.6},{18.1, -60.6},{24.5, -45.7},{32.6, -23},{50.3,31.3},{57.5,50.7},{63.9,64.6},{69.5,73.4},{75.2,78.6},{80,80}}, color = {0,0,255}, thickness = 0.5),Text(extent = {{78, -6},{98, -26}}, lineColor = {95,95,95}, textString = "u"),Line(points = {{ -80, -80},{18, -80}}, color = {175,175,175}, smooth = Smooth.None)}), Documentation(info = "<html>
<p>
This function returns y = cos(u), with -&infin; &lt; u &lt; &infin;:
</p>

<p>
<img src=\"../Images/Math/cos.png\">
</p>
</html>"));

      external "C" y = cos(u) ;      annotation(Library = "ModelicaExternalC");
    end cos;
    function tan "Tangent (u shall not be -pi/2, pi/2, 3*pi/2, ...)"
      extends baseIcon2;
      input SI.Angle u;
      output Real y;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -78.4, -68.4},{ -76.8, -59.7},{ -74.4, -50},{ -71.2, -40.9},{ -67.1, -33},{ -60.7, -24.8},{ -51.1, -17.2},{ -35.8, -9.98},{ -4.42, -1.07},{33.4,9.12},{49.4,16.2},{59.1,23.2},{65.5,30.6},{70.4,39.1},{73.6,47.4},{76,56.1},{77.6,63.8},{80,80}}, color = {0,0,0}),Text(extent = {{ -90,72},{ -18,24}}, lineColor = {192,192,192}, textString = "tan")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -37, -72},{ -17, -88}}, textString = "-5.8", lineColor = {0,0,255}),Text(extent = {{ -33,86},{ -13,70}}, textString = " 5.8", lineColor = {0,0,255}),Text(extent = {{68, -13},{88, -33}}, textString = "1.4", lineColor = {0,0,255}),Line(points = {{ -100,0},{84,0}}, color = {95,95,95}),Polygon(points = {{98,0},{82,6},{82, -6},{98,0}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -78.4, -68.4},{ -76.8, -59.7},{ -74.4, -50},{ -71.2, -40.9},{ -67.1, -33},{ -60.7, -24.8},{ -51.1, -17.2},{ -35.8, -9.98},{ -4.42, -1.07},{33.4,9.12},{49.4,16.2},{59.1,23.2},{65.5,30.6},{70.4,39.1},{73.6,47.4},{76,56.1},{77.6,63.8},{80,80}}, color = {0,0,255}, thickness = 0.5),Text(extent = {{82,22},{102,2}}, lineColor = {95,95,95}, textString = "u"),Line(points = {{0,80},{86,80}}, color = {175,175,175}, smooth = Smooth.None),Line(points = {{80,88},{80, -16}}, color = {175,175,175}, smooth = Smooth.None)}), Documentation(info = "<html>
<p>
This function returns y = tan(u), with -&infin; &lt; u &lt; &infin;
(if u is a multiple of (2n-1)*pi/2, y = tan(u) is +/- infinity).
</p>

<p>
<img src=\"../Images/Math/tan.png\">
</p>
</html>"));

      external "C" y = tan(u) ;      annotation(Library = "ModelicaExternalC");
    end tan;
    function asin "Inverse sine (-1 <= u <= 1)"
      extends baseIcon2;
      input Real u;
      output SI.Angle y;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -79.2, -72.8},{ -77.6, -67.5},{ -73.6, -59.4},{ -66.3, -49.8},{ -53.5, -37.3},{ -30.2, -19.7},{37.4,24.8},{57.5,40.8},{68.7,52.7},{75.2,62.2},{77.6,67.5},{80,80}}, color = {0,0,0}),Text(extent = {{ -88,78},{ -16,30}}, lineColor = {192,192,192}, textString = "asin")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -40, -72},{ -15, -88}}, textString = "-pi/2", lineColor = {0,0,255}),Text(extent = {{ -38,88},{ -13,72}}, textString = " pi/2", lineColor = {0,0,255}),Text(extent = {{68, -9},{88, -29}}, textString = "+1", lineColor = {0,0,255}),Text(extent = {{ -90,21},{ -70,1}}, textString = "-1", lineColor = {0,0,255}),Line(points = {{ -100,0},{84,0}}, color = {95,95,95}),Polygon(points = {{98,0},{82,6},{82, -6},{98,0}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -79.2, -72.8},{ -77.6, -67.5},{ -73.6, -59.4},{ -66.3, -49.8},{ -53.5, -37.3},{ -30.2, -19.7},{37.4,24.8},{57.5,40.8},{68.7,52.7},{75.2,62.2},{77.6,67.5},{80,80}}, color = {0,0,255}, thickness = 0.5),Text(extent = {{82,24},{102,4}}, lineColor = {95,95,95}, textString = "u"),Line(points = {{0,80},{86,80}}, color = {175,175,175}, smooth = Smooth.None),Line(points = {{80,86},{80, -10}}, color = {175,175,175}, smooth = Smooth.None)}), Documentation(info = "<html>
<p>
This function returns y = asin(u), with -1 &le; u &le; +1:
</p>

<p>
<img src=\"../Images/Math/asin.png\">
</p>
</html>"));

      external "C" y = asin(u) ;      annotation(Library = "ModelicaExternalC");
    end asin;
    function acos "Inverse cosine (-1 <= u <= 1)"
      extends baseIcon2;
      input Real u;
      output SI.Angle y;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -90, -80},{68, -80}}, color = {192,192,192}),Polygon(points = {{90, -80},{68, -72},{68, -88},{90, -80}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,80},{ -79.2,72.8},{ -77.6,67.5},{ -73.6,59.4},{ -66.3,49.8},{ -53.5,37.3},{ -30.2,19.7},{37.4, -24.8},{57.5, -40.8},{68.7, -52.7},{75.2, -62.2},{77.6, -67.5},{80, -80}}, color = {0,0,0}),Text(extent = {{ -86, -14},{ -14, -62}}, lineColor = {192,192,192}, textString = "acos")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -100, -80},{84, -80}}, color = {95,95,95}),Polygon(points = {{98, -80},{82, -74},{82, -86},{98, -80}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80,80},{ -79.2,72.8},{ -77.6,67.5},{ -73.6,59.4},{ -66.3,49.8},{ -53.5,37.3},{ -30.2,19.7},{37.4, -24.8},{57.5, -40.8},{68.7, -52.7},{75.2, -62.2},{77.6, -67.5},{80, -80}}, color = {0,0,255}, thickness = 0.5),Text(extent = {{ -30,88},{ -5,72}}, textString = " pi", lineColor = {0,0,255}),Text(extent = {{ -94, -57},{ -74, -77}}, textString = "-1", lineColor = {0,0,255}),Text(extent = {{60, -81},{80, -101}}, textString = "+1", lineColor = {0,0,255}),Text(extent = {{82, -56},{102, -76}}, lineColor = {95,95,95}, textString = "u"),Line(points = {{ -2,80},{84,80}}, color = {175,175,175}, smooth = Smooth.None),Line(points = {{80,82},{80, -86}}, color = {175,175,175}, smooth = Smooth.None)}), Documentation(info = "<html>
<p>
This function returns y = acos(u), with -1 &le; u &le; +1:
</p>

<p>
<img src=\"../Images/Math/acos.png\">
</p>
</html>"));

      external "C" y = acos(u) ;      annotation(Library = "ModelicaExternalC");
    end acos;
    function atan "Inverse tangent"
      extends baseIcon2;
      input Real u;
      output SI.Angle y;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -52.7, -75.2},{ -37.4, -69.7},{ -26.9, -63},{ -19.7, -55.2},{ -14.1, -45.8},{ -10.1, -36.4},{ -6.03, -23.9},{ -1.21, -5.06},{5.23,21},{9.25,34.1},{13.3,44.2},{18.1,52.9},{24.5,60.8},{33.4,67.6},{47,73.6},{69.5,78.6},{80,80}}, color = {0,0,0}),Text(extent = {{ -86,68},{ -14,20}}, lineColor = {192,192,192}, textString = "atan")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -100,0},{84,0}}, color = {95,95,95}),Polygon(points = {{96,0},{80,6},{80, -6},{96,0}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -52.7, -75.2},{ -37.4, -69.7},{ -26.9, -63},{ -19.7, -55.2},{ -14.1, -45.8},{ -10.1, -36.4},{ -6.03, -23.9},{ -1.21, -5.06},{5.23,21},{9.25,34.1},{13.3,44.2},{18.1,52.9},{24.5,60.8},{33.4,67.6},{47,73.6},{65,77},{80,78}}, color = {0,0,255}, thickness = 0.5),Text(extent = {{ -34,87},{ -14,74}}, textString = "pi/2", lineColor = {0,0,255}),Text(extent = {{ -32, -71},{ -12, -91}}, textString = "-pi/2", lineColor = {0,0,255}),Text(extent = {{84, -4},{104, -24}}, lineColor = {95,95,95}, textString = "u"),Line(points = {{ -2,80},{84,80}}, color = {175,175,175}, smooth = Smooth.None)}), Documentation(info = "<html>
<p>
This function returns y = atan(u), with -&infin; &lt; u &lt; &infin;:
</p>

<p>
<img src=\"../Images/Math/atan.png\">
</p>
</html>"));

      external "C" y = atan(u) ;      annotation(Library = "ModelicaExternalC");
    end atan;
    function atan2 "Four quadrant inverse tangent"
      extends baseIcon2;
      input Real u1;
      input Real u2;
      output SI.Angle y;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{0, -80},{8.93, -67.2},{17.1, -59.3},{27.3, -53.6},{42.1, -49.4},{69.9, -45.8},{80, -45.1}}, color = {0,0,0}),Line(points = {{ -80, -34.9},{ -46.1, -31.4},{ -29.4, -27.1},{ -18.3, -21.5},{ -10.3, -14.5},{ -2.03, -3.17},{7.97,11.6},{15.5,19.4},{24.3,25},{39,30},{62.1,33.5},{80,34.9}}, color = {0,0,0}),Line(points = {{ -80,45.1},{ -45.9,48.7},{ -29.1,52.9},{ -18.1,58.6},{ -10.2,65.8},{ -1.82,77.2},{0,80}}, color = {0,0,0}),Text(extent = {{ -90, -46},{ -18, -94}}, lineColor = {192,192,192}, textString = "atan2")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -100,0},{84,0}}, color = {95,95,95}),Polygon(points = {{96,0},{80,6},{80, -6},{96,0}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{0, -80},{8.93, -67.2},{17.1, -59.3},{27.3, -53.6},{42.1, -49.4},{69.9, -45.8},{80, -45.1}}, color = {0,0,255}, thickness = 0.5),Line(points = {{ -80, -34.9},{ -46.1, -31.4},{ -29.4, -27.1},{ -18.3, -21.5},{ -10.3, -14.5},{ -2.03, -3.17},{7.97,11.6},{15.5,19.4},{24.3,25},{39,30},{62.1,33.5},{80,34.9}}, color = {0,0,255}, thickness = 0.5),Line(points = {{ -80,45.1},{ -45.9,48.7},{ -29.1,52.9},{ -18.1,58.6},{ -10.2,65.8},{ -1.82,77.2},{0,80}}, color = {0,0,255}, thickness = 0.5),Text(extent = {{ -32,89},{ -10,74}}, textString = "pi", lineColor = {0,0,255}),Text(extent = {{ -32, -72},{ -4, -88}}, textString = "-pi", lineColor = {0,0,255}),Text(extent = {{0,55},{20,42}}, textString = "pi/2", lineColor = {0,0,255}),Line(points = {{0,40},{ -8,40}}, color = {192,192,192}),Line(points = {{0, -40},{ -8, -40}}, color = {192,192,192}),Text(extent = {{0, -23},{20, -42}}, textString = "-pi/2", lineColor = {0,0,255}),Text(extent = {{62, -4},{94, -26}}, lineColor = {95,95,95}, textString = "u1, u2"),Line(points = {{ -88,40},{86,40}}, color = {175,175,175}, smooth = Smooth.None),Line(points = {{ -86, -40},{86, -40}}, color = {175,175,175}, smooth = Smooth.None)}), Documentation(info = "<HTML>
This function returns y = atan2(u1,u2) such that tan(y) = u1/u2 and
y is in the range -pi &lt; y &le; pi. u2 may be zero, provided
u1 is not zero. Usually u1, u2 is provided in such a form that
u1 = sin(y) and u2 = cos(y):
</p>

<p>
<img src=\"../Images/Math/atan2.png\">
</p>

</HTML>
"));

      external "C" y = atan2(u1,u2) ;      annotation(Library = "ModelicaExternalC");
    end atan2;
    function atan3 "Four quadrant inverse tangens (select solution that is closest to given angle y0)"
      import Modelica.Math;
      extends Modelica.Math.baseIcon2;
      input Real u1;
      input Real u2;
      input Modelica.SIunits.Angle y0 = 0 "y shall be in the range: -pi < y-y0 < pi";
      output Modelica.SIunits.Angle y;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{0, -80},{8.93, -67.2},{17.1, -59.3},{27.3, -53.6},{42.1, -49.4},{69.9, -45.8},{80, -45.1}}, color = {0,0,0}),Line(points = {{ -80, -34.9},{ -46.1, -31.4},{ -29.4, -27.1},{ -18.3, -21.5},{ -10.3, -14.5},{ -2.03, -3.17},{7.97,11.6},{15.5,19.4},{24.3,25},{39,30},{62.1,33.5},{80,34.9}}, color = {0,0,0}),Line(points = {{ -80,45.1},{ -45.9,48.7},{ -29.1,52.9},{ -18.1,58.6},{ -10.2,65.8},{ -1.82,77.2},{0,80}}, color = {0,0,0}),Text(extent = {{ -90, -46},{ -18, -94}}, lineColor = {192,192,192}, textString = "atan3")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -100, -86},{84, -86}}, color = {95,95,95}),Polygon(points = {{98, -86},{82, -80},{82, -92},{98, -86}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{0, -80},{8.93, -67.2},{17.1, -59.3},{27.3, -53.6},{42.1, -49.4},{69.9, -45.8},{80, -45.1}}, color = {0,0,255}, thickness = 0.5),Line(points = {{ -80, -34.9},{ -46.1, -31.4},{ -29.4, -27.1},{ -18.3, -21.5},{ -10.3, -14.5},{ -2.03, -3.17},{7.97,11.6},{15.5,19.4},{24.3,25},{39,30},{62.1,33.5},{80,34.9}}, color = {0,0,255}, thickness = 0.5),Line(points = {{ -80,45.1},{ -45.9,48.7},{ -29.1,52.9},{ -18.1,58.6},{ -10.2,65.8},{ -1.82,77.2},{0,80}}, color = {0,0,255}, thickness = 0.5),Text(extent = {{ -56,82},{ -12,72}}, textString = "(2*N-1)*pi", lineColor = {0,0,255}),Text(extent = {{ -52, -72},{ -10, -88}}, textString = "(2*N-3)*pi", lineColor = {0,0,255}),Line(points = {{0,40},{ -8,40}}, color = {192,192,192}),Line(points = {{0, -40},{ -8, -40}}, color = {192,192,192}),Text(extent = {{38, -68},{78, -84}}, lineColor = {95,95,95}, textString = "u1, u2, y0"),Line(points = {{ -84,40},{88,40}}, color = {175,175,175}, smooth = Smooth.None),Line(points = {{ -84, -40},{88, -40}}, color = {175,175,175}, smooth = Smooth.None)}), Documentation(info = "<HTML>
<p>
This function returns y = <b>atan3</b>(u1,u2,y0) such that
<b>tan</b>(y) = u1/u2 and
y is in the range: -pi &lt; y-y0 &lt; pi.<br>
u2 may be zero, provided u1 is not zero. The difference to
Modelica.Math.atan2(..) is the optional third argument y0 that
allows to specify which of the infinite many solutions
shall be returned:
</p>

<p>
<img src=\"../Images/Math/atan3.png\">
</p>

</HTML>
"));
    protected 
      Real pi = Modelica.Constants.pi;
      Real w;
    algorithm 
      w:=Math.atan2(u1, u2);
      y:=w + 2 * pi * div(abs(w - y0) + pi, 2 * pi) * (if y0 > w then  +1 else  -1);
    end atan3;
    function sinh "Hyperbolic sine"
      extends baseIcon2;
      input Real u;
      output Real y;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -76, -65.4},{ -71.2, -51.4},{ -65.5, -38.8},{ -59.1, -28.1},{ -51.1, -18.7},{ -41.4, -11.4},{ -27.7, -5.5},{ -4.42, -0.653},{24.5,4.57},{39,10.1},{49.4,17.2},{57.5,25.9},{63.9,35.8},{69.5,47.4},{74.4,60.4},{78.4,73.8},{80,80}}, color = {0,0,0}),Text(extent = {{ -88,80},{ -16,32}}, lineColor = {192,192,192}, textString = "sinh")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -100,0},{84,0}}, color = {95,95,95}),Polygon(points = {{98,0},{82,6},{82, -6},{98,0}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -76, -65.4},{ -71.2, -51.4},{ -65.5, -38.8},{ -59.1, -28.1},{ -51.1, -18.7},{ -41.4, -11.4},{ -27.7, -5.5},{ -4.42, -0.653},{24.5,4.57},{39,10.1},{49.4,17.2},{57.5,25.9},{63.9,35.8},{69.5,47.4},{74.4,60.4},{78.4,73.8},{80,80}}, color = {0,0,255}, thickness = 0.5),Text(extent = {{ -31,72},{ -11,88}}, textString = "27", lineColor = {0,0,255}),Text(extent = {{ -35, -88},{ -15, -72}}, textString = "-27", lineColor = {0,0,255}),Text(extent = {{68, -7},{88, -27}}, textString = "4", lineColor = {0,0,255}),Text(extent = {{ -98,21},{ -78,1}}, textString = "-4", lineColor = {0,0,255}),Text(extent = {{80,26},{100,6}}, lineColor = {95,95,95}, textString = "u"),Line(points = {{0,80},{86,80}}, color = {175,175,175}, smooth = Smooth.None),Line(points = {{80,84},{80, -6}}, color = {175,175,175}, smooth = Smooth.None)}), Documentation(info = "<html>
<p>
This function returns y = sinh(u), with -&infin; &lt; u &lt; &infin;:
</p>

<p>
<img src=\"../Images/Math/sinh.png\">
</p>
</html>"));

      external "C" y = sinh(u) ;      annotation(Library = "ModelicaExternalC");
    end sinh;
    function cosh "Hyperbolic cosine"
      extends baseIcon2;
      input Real u;
      output Real y;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -90, -86.083},{68, -86.083}}, color = {192,192,192}),Polygon(points = {{90, -86.083},{68, -78.083},{68, -94.083},{90, -86.083}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,80},{ -77.6,61.1},{ -74.4,39.3},{ -71.2,20.7},{ -67.1,1.29},{ -63.1, -14.6},{ -58.3, -29.8},{ -52.7, -43.5},{ -46.2, -55.1},{ -39, -64.3},{ -30.2, -71.7},{ -18.9, -77.1},{ -4.42, -79.9},{10.9, -79.1},{23.7, -75.2},{34.2, -68.7},{42.2, -60.6},{48.6, -51.2},{54.3, -40},{59.1, -27.5},{63.1, -14.6},{67.1,1.29},{71.2,20.7},{74.4,39.3},{77.6,61.1},{80,80}}, color = {0,0,0}),Text(extent = {{4,66},{66,20}}, lineColor = {192,192,192}, textString = "cosh")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -100, -84.083},{84, -84.083}}, color = {95,95,95}),Polygon(points = {{98, -84.083},{82, -78.083},{82, -90.083},{98, -84.083}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80,80},{ -77.6,61.1},{ -74.4,39.3},{ -71.2,20.7},{ -67.1,1.29},{ -63.1, -14.6},{ -58.3, -29.8},{ -52.7, -43.5},{ -46.2, -55.1},{ -39, -64.3},{ -30.2, -71.7},{ -18.9, -77.1},{ -4.42, -79.9},{10.9, -79.1},{23.7, -75.2},{34.2, -68.7},{42.2, -60.6},{48.6, -51.2},{54.3, -40},{59.1, -27.5},{63.1, -14.6},{67.1,1.29},{71.2,20.7},{74.4,39.3},{77.6,61.1},{80,80}}, color = {0,0,255}, thickness = 0.5),Text(extent = {{ -31,72},{ -11,88}}, textString = "27", lineColor = {0,0,255}),Text(extent = {{64, -83},{84, -103}}, textString = "4", lineColor = {0,0,255}),Text(extent = {{ -94, -63},{ -74, -83}}, textString = "-4", lineColor = {0,0,255}),Text(extent = {{80, -60},{100, -80}}, lineColor = {95,95,95}, textString = "u"),Line(points = {{0,80},{88,80}}, color = {175,175,175}, smooth = Smooth.None),Line(points = {{80,84},{80, -90}}, color = {175,175,175}, smooth = Smooth.None)}), Documentation(info = "<html>
<p>
This function returns y = cosh(u), with -&infin; &lt; u &lt; &infin;:
</p>

<p>
<img src=\"../Images/Math/cosh.png\">
</p>
</html>"));

      external "C" y = cosh(u) ;      annotation(Library = "ModelicaExternalC");
    end cosh;
    function tanh "Hyperbolic tangent"
      extends baseIcon2;
      input Real u;
      output Real y;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {0.5,0.5}), graphics = {Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -47.8, -78.7},{ -35.8, -75.7},{ -27.7, -70.6},{ -22.1, -64.2},{ -17.3, -55.9},{ -12.5, -44.3},{ -7.64, -29.2},{ -1.21, -4.82},{6.83,26.3},{11.7,42},{16.5,54.2},{21.3,63.1},{26.9,69.9},{34.2,75},{45.4,78.4},{72,79.9},{80,80}}, color = {0,0,0}),Text(extent = {{ -88,72},{ -16,24}}, lineColor = {192,192,192}, textString = "tanh")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {0.5,0.5}), graphics = {Line(points = {{ -100,0},{84,0}}, color = {95,95,95}),Polygon(points = {{96,0},{80,6},{80, -6},{96,0}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80.5},{ -47.8, -79.2},{ -35.8, -76.2},{ -27.7, -71.1},{ -22.1, -64.7},{ -17.3, -56.4},{ -12.5, -44.8},{ -7.64, -29.7},{ -1.21, -5.32},{6.83,25.8},{11.7,41.5},{16.5,53.7},{21.3,62.6},{26.9,69.4},{34.2,74.5},{45.4,77.9},{72,79.4},{80,79.5}}, color = {0,0,255}, thickness = 0.5),Text(extent = {{ -29,72},{ -9,88}}, textString = "1", lineColor = {0,0,255}),Text(extent = {{3, -72},{23, -88}}, textString = "-1", lineColor = {0,0,255}),Text(extent = {{82, -2},{102, -22}}, lineColor = {95,95,95}, textString = "u"),Line(points = {{0,80},{88,80}}, color = {175,175,175}, smooth = Smooth.None)}), Documentation(info = "<html>
<p>
This function returns y = tanh(u), with -&infin; &lt; u &lt; &infin;:
</p>

<p>
<img src=\"../Images/Math/tanh.png\">
</p>
</html>"));

      external "C" y = tanh(u) ;      annotation(Library = "ModelicaExternalC");
    end tanh;
    function asinh "Inverse of sinh (area hyperbolic sine)"
      extends Modelica.Math.baseIcon2;
      input Real u;
      output Real y;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -56.7, -68.4},{ -39.8, -56.8},{ -26.9, -44.7},{ -17.3, -32.4},{ -9.25, -19},{9.25,19},{17.3,32.4},{26.9,44.7},{39.8,56.8},{56.7,68.4},{80,80}}, color = {0,0,0}),Text(extent = {{ -90,80},{ -6,26}}, lineColor = {192,192,192}, textString = "asinh")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -100,0},{84,0}}, color = {95,95,95}),Polygon(points = {{98,0},{82,6},{82, -6},{98,0}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -56.7, -68.4},{ -39.8, -56.8},{ -26.9, -44.7},{ -17.3, -32.4},{ -9.25, -19},{9.25,19},{17.3,32.4},{26.9,44.7},{39.8,56.8},{56.7,68.4},{80,80}}, color = {0,0,255}, thickness = 0.5),Text(extent = {{ -31,72},{ -11,88}}, textString = "2.31", lineColor = {0,0,255}),Text(extent = {{ -35, -88},{ -15, -72}}, textString = "-2.31", lineColor = {0,0,255}),Text(extent = {{72, -13},{92, -33}}, textString = "5", lineColor = {0,0,255}),Text(extent = {{ -96,21},{ -76,1}}, textString = "-5", lineColor = {0,0,255}),Text(extent = {{80,22},{100,2}}, lineColor = {95,95,95}, textString = "u"),Line(points = {{0,80},{88,80}}, color = {175,175,175}, smooth = Smooth.None),Line(points = {{80,86},{80, -12}}, color = {175,175,175}, smooth = Smooth.None)}), Documentation(info = "<html>
<p>
The function returns the area hyperbolic sine of its
input argument u. This inverse of sinh(..) is unique
and there is no restriction on the input argument u of
asinh(u) (-&infin; &lt; u &lt; &infin;):
</p>

<p>
<img src=\"../Images/Math/asinh.png\">
</p>
</html>"));
    algorithm 
      y:=Modelica.Math.log(u + sqrt(u * u + 1));
    end asinh;
    function acosh "Inverse of cosh (area hyperbolic cosine)"
      import Modelica.Utilities.Streams.*;
      extends Modelica.Math.baseIcon1;
      input Real u;
      output Real y;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -90, -80},{68, -80}}, color = {192,192,192}),Polygon(points = {{90, -80},{68, -72},{68, -88},{90, -80}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -66, -80},{ -65.2, -66},{ -64.4, -60.3},{ -62.8, -52.2},{ -60.4, -43.4},{ -56.4, -32.4},{ -49.9, -19.3},{ -41.1, -5.65},{ -29,8.8},{ -12.9,23.8},{7.97,39.2},{35.3,55},{69.9,70.8},{94,80}}, color = {0,0,0}),Text(extent = {{ -14,2},{76, -54}}, lineColor = {192,192,192}, textString = "arcosh")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -100, -70},{84, -70}}, color = {95,95,95}),Polygon(points = {{100, -70},{84, -64},{84, -76},{100, -70}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -60, -70},{ -59.2, -56},{ -58.4, -50.3},{ -56.8, -42.2},{ -54.4, -33.4},{ -50.4, -22.4},{ -43.9, -9.3},{ -35.1,4.35},{ -23,18.8},{ -6.9,33.8},{13.97,49.2},{41.3,65},{75.9,80.8},{100,90}}, color = {0,0,255}, thickness = 0.5),Text(extent = {{ -80,66},{ -58,78}}, textString = "2.29", lineColor = {0,0,255}),Text(extent = {{ -73, -86},{ -50, -70}}, textString = "1.0", lineColor = {0,0,255}),Text(extent = {{64, -77},{84, -92}}, textString = "5", lineColor = {0,0,255}),Text(extent = {{ -96, -55},{ -76, -70}}, textString = "0", lineColor = {0,0,255}),Text(extent = {{74, -44},{94, -64}}, lineColor = {95,95,95}, textString = "u"),Line(points = {{ -80,80},{100,80}}, color = {175,175,175}, smooth = Smooth.None),Line(points = {{74, -78},{74,88}}, color = {175,175,175}, smooth = Smooth.None)}), Documentation(info = "<html>
<p>
This function returns the area hyperbolic cosine of its
input argument u. The valid range of u is
</p>
<pre>
  +1 &le; u &lt; +&infin;
</pre>
<p>
If the function is called with u &lt; 1, an error occurs.
The function cosh(u) has two inverse functions (the curve
looks similar to a sqrt(..) function). acosh(..) returns
the inverse that is positive. At u=1, the derivative dy/du is infinite.
Therefore, this function should not be used in a model, if u
can become close to 1:
</p>

<p>
<img src=\"../Images/Math/acosh.png\">
</p>
</html>"));
    algorithm 
      assert(u >= 1.0, "Input argument u (= " + String(u) + ") of acosh(u) must be >= 1.0");
      y:=Modelica.Math.log(u + sqrt(u * u - 1));
    end acosh;
    function exp "Exponential, base e"
      extends baseIcon2;
      input Real u;
      output Real y;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -90, -80.3976},{68, -80.3976}}, color = {192,192,192}),Polygon(points = {{90, -80.3976},{68, -72.3976},{68, -88.3976},{90, -80.3976}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -31, -77.9},{ -6.03, -74},{10.9, -68.4},{23.7, -61},{34.2, -51.6},{43, -40.3},{50.3, -27.8},{56.7, -13.5},{62.3,2.23},{67.1,18.6},{72,38.2},{76,57.6},{80,80}}, color = {0,0,0}),Text(extent = {{ -86,50},{ -14,2}}, lineColor = {192,192,192}, textString = "exp")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -100, -80.3976},{84, -80.3976}}, color = {95,95,95}),Polygon(points = {{98, -80.3976},{82, -74.3976},{82, -86.3976},{98, -80.3976}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -31, -77.9},{ -6.03, -74},{10.9, -68.4},{23.7, -61},{34.2, -51.6},{43, -40.3},{50.3, -27.8},{56.7, -13.5},{62.3,2.23},{67.1,18.6},{72,38.2},{76,57.6},{80,80}}, color = {0,0,255}, thickness = 0.5),Text(extent = {{ -31,72},{ -11,88}}, textString = "20", lineColor = {0,0,255}),Text(extent = {{ -92, -81},{ -72, -101}}, textString = "-3", lineColor = {0,0,255}),Text(extent = {{66, -81},{86, -101}}, textString = "3", lineColor = {0,0,255}),Text(extent = {{2, -69},{22, -89}}, textString = "1", lineColor = {0,0,255}),Text(extent = {{78, -54},{98, -74}}, lineColor = {95,95,95}, textString = "u"),Line(points = {{0,80},{88,80}}, color = {175,175,175}, smooth = Smooth.None),Line(points = {{80,84},{80, -84}}, color = {175,175,175}, smooth = Smooth.None)}), Documentation(info = "<html>
<p>
This function returns y = exp(u), with -&infin; &lt; u &lt; &infin;:
</p>

<p>
<img src=\"../Images/Math/exp.png\">
</p>
</html>"));

      external "C" y = exp(u) ;      annotation(Library = "ModelicaExternalC");
    end exp;
    function log "Natural (base e) logarithm (u shall be > 0)"
      extends baseIcon1;
      input Real u;
      output Real y;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -79.2, -50.6},{ -78.4, -37},{ -77.6, -28},{ -76.8, -21.3},{ -75.2, -11.4},{ -72.8, -1.31},{ -69.5,8.08},{ -64.7,17.9},{ -57.5,28},{ -47,38.1},{ -31.8,48.1},{ -10.1,58},{22.1,68},{68.7,78.1},{80,80}}, color = {0,0,0}),Text(extent = {{ -6, -24},{66, -72}}, lineColor = {192,192,192}, textString = "log")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -100,0},{84,0}}, color = {95,95,95}),Polygon(points = {{100,0},{84,6},{84, -6},{100,0}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -78, -80},{ -77.2, -50.6},{ -76.4, -37},{ -75.6, -28},{ -74.8, -21.3},{ -73.2, -11.4},{ -70.8, -1.31},{ -67.5,8.08},{ -62.7,17.9},{ -55.5,28},{ -45,38.1},{ -29.8,48.1},{ -8.1,58},{24.1,68},{70.7,78.1},{82,80}}, color = {0,0,255}, thickness = 0.5),Text(extent = {{ -105,72},{ -85,88}}, textString = "3", lineColor = {0,0,255}),Text(extent = {{60, -3},{80, -23}}, textString = "20", lineColor = {0,0,255}),Text(extent = {{ -78, -7},{ -58, -27}}, textString = "1", lineColor = {0,0,255}),Text(extent = {{84,26},{104,6}}, lineColor = {95,95,95}, textString = "u"),Text(extent = {{ -100,9},{ -80, -11}}, textString = "0", lineColor = {0,0,255}),Line(points = {{ -80,80},{84,80}}, color = {175,175,175}, smooth = Smooth.None),Line(points = {{82,82},{82, -6}}, color = {175,175,175}, smooth = Smooth.None)}), Documentation(info = "<html>
<p>
This function returns y = log(10) (the natural logarithm of u),
with u &gt; 0:
</p>

<p>
<img src=\"../Images/Math/log.png\">
</p>
</html>"));

      external "C" y = log(u) ;      annotation(Library = "ModelicaExternalC");
    end log;
    function log10 "Base 10 logarithm (u shall be > 0)"
      extends baseIcon1;
      input Real u;
      output Real y;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -79.8, -80},{ -79.2, -50.6},{ -78.4, -37},{ -77.6, -28},{ -76.8, -21.3},{ -75.2, -11.4},{ -72.8, -1.31},{ -69.5,8.08},{ -64.7,17.9},{ -57.5,28},{ -47,38.1},{ -31.8,48.1},{ -10.1,58},{22.1,68},{68.7,78.1},{80,80}}, color = {0,0,0}),Text(extent = {{ -30, -22},{60, -70}}, lineColor = {192,192,192}, textString = "log10")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -100,0},{84,0}}, color = {95,95,95}),Polygon(points = {{98,0},{82,6},{82, -6},{98,0}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -77.8, -80},{ -77.2, -50.6},{ -76.4, -37},{ -75.6, -28},{ -74.8, -21.3},{ -73.2, -11.4},{ -70.8, -1.31},{ -67.5,8.08},{ -62.7,17.9},{ -55.5,28},{ -45,38.1},{ -29.8,48.1},{ -8.1,58},{24.1,68},{70.7,78.1},{82,80}}, color = {0,0,255}, thickness = 0.5),Text(extent = {{66, -13},{86, -33}}, textString = "20", lineColor = {0,0,255}),Text(extent = {{ -78, -1},{ -58, -21}}, textString = "1", lineColor = {0,0,255}),Text(extent = {{ -83,62},{ -63,78}}, textString = " 1.3", lineColor = {0,0,255}),Text(extent = {{80,24},{100,4}}, lineColor = {95,95,95}, textString = "u"),Text(extent = {{ -100,9},{ -80, -11}}, textString = "0", lineColor = {0,0,255}),Line(points = {{ -80,80},{86,80}}, color = {175,175,175}, smooth = Smooth.None),Line(points = {{80,92},{80, -12}}, color = {175,175,175}, smooth = Smooth.None)}), Documentation(info = "<html>
<p>
This function returns y = log10(u),
with u &gt; 0:
</p>

<p>
<img src=\"../Images/Math/log10.png\">
</p>
</html>"));

      external "C" y = log10(u) ;      annotation(Library = "ModelicaExternalC");
    end log10;
    partial function baseIcon1 "Basic icon for mathematical function with y-axis on left side"
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -80,68}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,150},{150,110}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -80,80},{ -88,80}}, color = {95,95,95}),Line(points = {{ -80, -80},{ -88, -80}}, color = {95,95,95}),Line(points = {{ -80, -90},{ -80,84}}, color = {95,95,95}),Text(extent = {{ -75,104},{ -55,84}}, lineColor = {95,95,95}, textString = "y"),Polygon(points = {{ -80,98},{ -86,82},{ -74,82},{ -80,98}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>
Icon for a mathematical function, consisting of an y-axis on the left side.
It is expected, that an x-axis is added and a plot of the function.
</p>
</html>"));
    end baseIcon1;
    partial function baseIcon2 "Basic icon for mathematical function with y-axis in middle"
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0, -80},{0,68}}, color = {192,192,192}),Polygon(points = {{0,90},{ -8,68},{8,68},{0,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,150},{150,110}}, textString = "%name", lineColor = {0,0,255})}), Diagram(graphics = {Line(points = {{0,80},{ -8,80}}, color = {95,95,95}),Line(points = {{0, -80},{ -8, -80}}, color = {95,95,95}),Line(points = {{0, -90},{0,84}}, color = {95,95,95}),Text(extent = {{5,104},{25,84}}, lineColor = {95,95,95}, textString = "y"),Polygon(points = {{0,98},{ -6,82},{6,82},{0,98}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid)}), Documentation(revisions = "<html>
<p>
Icon for a mathematical function, consisting of an y-axis in the middle.
It is expected, that an x-axis is added and a plot of the function.
</p>
</html>"));
    end baseIcon2;
    function tempInterpol1 "Temporary function for linear interpolation (will be removed)"
      input Real u "input value (first column of table)";
      input Real table[:,:] "table to be interpolated";
      input Integer icol "column of table to be interpolated";
      output Real y "interpolated input value (icol column of table)";
    protected 
      Integer i;
      Integer n "number of rows of table";
      Real u1;
      Real u2;
      Real y1;
      Real y2;
    algorithm 
      n:=size(table, 1);
      if n <= 1 then 
            y:=table[1,icol];

      else       if u <= table[1,1] then 
            i:=1;

      else       i:=2;
      while (i < n and u >= table[i,1]) loop
              i:=i + 1;

      end while;
      i:=i - 1;

      end if;
      u1:=table[i,1];
      u2:=table[i + 1,1];
      y1:=table[i,icol];
      y2:=table[i + 1,icol];
      assert(u2 > u1, "Table index must be increasing");
      y:=y1 + ((y2 - y1) * (u - u1)) / (u2 - u1);

      end if;
      annotation(Documentation(info = "<html>

</html>"));
    end tempInterpol1;
    function tempInterpol2 "Temporary function for vectorized linear interpolation (will be removed)"
      input Real u "input value (first column of table)";
      input Real table[:,:] "table to be interpolated";
      input Integer icol[:] "column(s) of table to be interpolated";
      output Real y[1,size(icol, 1)] "interpolated input value(s) (column(s) icol of table)";
    protected 
      Integer i;
      Integer n "number of rows of table";
      Real u1;
      Real u2;
      Real y1[1,size(icol, 1)];
      Real y2[1,size(icol, 1)];
    algorithm 
      n:=size(table, 1);
      if n <= 1 then 
            y:=transpose([table[1,icol]]);

      else       if u <= table[1,1] then 
            i:=1;

      else       i:=2;
      while (i < n and u >= table[i,1]) loop
              i:=i + 1;

      end while;
      i:=i - 1;

      end if;
      u1:=table[i,1];
      u2:=table[i + 1,1];
      y1:=transpose([table[i,icol]]);
      y2:=transpose([table[i + 1,icol]]);
      assert(u2 > u1, "Table index must be increasing");
      y:=y1 + ((y2 - y1) * (u - u1)) / (u2 - u1);

      end if;
      annotation(Documentation(info = "<html>

</html>"));
    end tempInterpol2;
  end Math;
  package Magnetic "Library of magnetic models"
    extends Modelica.Icons.Library2;
    annotation(Documentation(info = "<html>
<p>
This library contains magnetic components to build especially
electro-magnetic devices.
</p>

</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -78, -24},{ -86, -16}}, lineColor = {0,0,0}),Line(points = {{ -64, -20},{ -78, -20}}, color = {0,0,0}),Line(points = {{ -64, -32},{ -32, -40}}, color = {0,0,0}),Line(points = {{ -64, -20},{ -32, -28}}, color = {0,0,0}),Ellipse(extent = {{ -78, -60},{ -86, -52}}, lineColor = {0,0,0}),Line(points = {{ -64, -56},{ -78, -56}}, color = {0,0,0}),Line(points = {{ -64, -44},{ -32, -52}}, color = {0,0,0}),Line(points = {{ -64, -56},{ -32, -64}}, color = {0,0,0}),Line(points = {{20, -2},{ -60, -2},{ -60, -88},{20, -88},{20, -66},{ -36, -66},{ -36, -24},{20, -24},{20, -2}}, color = {0,0,0}),Rectangle(extent = {{36, -2},{60, -88}}, lineColor = {0,0,0})}));
    package FluxTubes "Library for modelling of electromagnetic devices with lumped magnetic networks"
      annotation(Window(x = 0.45, y = 0.01, width = 0.44, height = 0.65, library = 1, autolayout = 1), Documentation(info = "<html>
<p>
This library contains components for modelling of electromagnetic devices with lumped magnetic networks. Those models are suited for both rough design of the magnetic subsystem of a device as well as for efficient dynamic simulation at system level together with neighbouring subsystems. At present, components and examples for modelling of <i>translatory</i> electromagnetic and electrodynamic actuators are provided. If needed, these components can be adapted to network modelling of <i>rotational</i> electrical machines.
</p>
<p>
<a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide\">User's Guide</a> gives a short introduction to the underlying concept of <b>magnetic flux tubes</b>, summarizes the calculation of magnetic <b>reluctance forces</b> from lumped magnetic network models and lists <b>reference literature</b>.
</p>
<p>
<a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples\">Examples</a> illustrates the usage of magnetic network models with simple models from different fields of application.
</p>

<p>
Copyright &copy; 2005-2009, Thomas B&ouml;drich.
</p>
<p>
<i>This Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p><br>

</html>
", revisions = "<html>
<table border=1>

<thead>
<tr><td>Version</td> <td>Date</td>  <td>Author(s)</td> <td>Comments</td></tr>
</thead>

<tbody>
<tr><td>1.2</td>  <td>2009-08-11</td>
    <td>Christian&nbsp;Kral<br>
        Anton&nbsp;Haumer<br>
        Thomas&nbsp;B&ouml;drich&nbsp;<br>
        Martin&nbsp;Otter</td> <td>Update and improvement for inclusion in the
        Modelica Standard Library.</td> </tr>
<tr><td>1.1</td>  <td>2009-05-19</td>  <td>Thomas B&ouml;drich </td>
    <td> Coupling coefficient in Basic.ElectroMagneticConverter Removed.<br>
         Basic.EddyCurrent added .<br>
         Example MovingCoilActuator, especially PermeanceModel, completely revised. <br>
         Leakage coefficient replaced by coupling coefficient in Basic.LeakageWithCoefficient. <br>
         Utilities.CoilDesign: parameter U renamed to V_op,CoilDesign moved to Utilities.<br>
         Reference direction for magnetic flux added in all sources.<br>
         degC replaced by K for compatibility with Modelica 3.0.<br>
         redeclare in Sensors for compatibility with Modelica 3.0 removed.<br>
         Partial flux tube components moved to Interfaces and basic elements moved to new package Basic. </td> </tr>
<tr><td>1.0</td>  <td>2007-10-11</td>  <td>Thomas B&ouml;drich </td> <td>Release of version 1.0 of the library </td> </tr>
<tr><td></td>     <td>2005</td>        <td>Thomas B&ouml;drich </td> <td>First release of a Modelica magnetic library</td> </tr>
</tbody>

</table>

</html>"), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{36,2},{60, -84}}, lineColor = {0,0,0}),Line(points = {{20,2},{ -60,2},{ -60, -84},{20, -84},{20, -62},{ -36, -62},{ -36, -20},{20, -20},{20,2}}, color = {0,0,0}),Line(points = {{ -64, -28},{ -32, -36}}, color = {0,0,0}),Line(points = {{ -64, -52},{ -32, -60}}, color = {0,0,0}),Line(points = {{ -64, -16},{ -32, -24}}, color = {0,0,0}),Line(points = {{ -64, -16},{ -78, -16}}, color = {0,0,0}),Line(points = {{ -64, -52},{ -78, -52}}, color = {0,0,0}),Ellipse(extent = {{ -78, -20},{ -86, -12}}, lineColor = {0,0,0}),Ellipse(extent = {{ -78, -56},{ -86, -48}}, lineColor = {0,0,0}),Line(points = {{ -64, -40},{ -32, -48}}, color = {0,0,0})}));
      import SI = Modelica.SIunits;
      import Modelica.Constants.pi;
      import mu_0 = Modelica.Constants.mue_0;
      extends Modelica.Icons.Library2;
      package UsersGuide "User's Guide"
        annotation(DocumentationClass = true, Documentation(info = "<html>
<p>
This library contains components for modelling of electromagnetic devices with lumped magnetic networks. Those models are suited for both rough design of the magnetic subsystem of a device as well as for efficient dynamic simulation at system level together with neighbouring subsystems. At present, components and examples for modelling of <i>translatory</i> electromagnetic and electrodynamic actuators are provided. If needed, these components can be adapted to network modellling of <i>rotational</i> electrical machines.
</p>
<p>
This user's guide gives a short introduction to the underlying concept of <b>magnetic flux tubes</b>, summarizes the calculation of magnetic <b>reluctance forces</b> from lumped magnetic network models and lists <b>reference literature</b>.
</p>
<p>
<a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples\">Examples</a> illustrates the usage of magnetic network models with simple models from different fields of application.
</html>"));
        class FluxTubeConcept "Flux tube concept"
          annotation(Documentation(info = "<html>
<h3><font color=\"#008000\" size=5>Overview of the Conept of Magnetic Flux Tubes</font></h3>
<p>
Following below, the concept of magnetic flux tubes is outlined in short. For a detailed description of flux tube elements, please have a look at the listed literature. Magnetic flux tubes enable for modeling of magnetic fields with lumped networks. The figure below and the following equations illustrate the transition from the original magnetic field quantities described by <i>Maxwell</i>'s equations to network elements with a flow variable and an across variable:
</p>

<p><IMG SRC=\"../Images/Magnetic/FluxTubes/UsersGuide/FluxTubeConcept/magnetic_flux_tube_schematic.png\" ALT=\"Magnetic flux tube\"></p>

<p>
For a region with an approximately homogeneous distribution of the magnetic field strength <b>H</b> and the magnetic flux density <b>B</b> through cross sectional area <i>A</i> at each length coordinate <i>s</i>  (<i>A</i> perpendicular to the direction of the magnetic field lines), a magnetic reluctance <i>R<sub>m</sub></i> can be defined:</p>

<p><IMG SRC=\"../Images/Magnetic/FluxTubes/UsersGuide/FluxTubeConcept/eq_transition_reluctance_flowAcross_IntegralQuantities.png\" ALT=\"Transition from field quantities to flow- and across variables\"></p>

<p>
With the definition of the magnetic potential difference <i>V<sub>m</sub></i> as an across variable and the magnetic flux <i>&Phi;</i> as flow variable, a reluctance element <i>R<sub>m</sub></i> can be defined similar to resistive network elements in other physical domains. Using <i>Maxwell</i>'s constitutive equation
</p>

<p><IMG SRC=\"../Images/Magnetic/FluxTubes/UsersGuide/FluxTubeConcept/eq_MaxwellConstitutive.png\" ALT=\"Maxwell's constitutive equation\"></p>

<p>the general formula for the calculation of a magnetic reluctance <i>R<sub>m</sub></i> from its geometric and material properties is:</p>

<p><IMG SRC=\"../Images/Magnetic/FluxTubes/UsersGuide/FluxTubeConcept/eq_reluctance_general.png\" ALT=\"General formula for calculation of a magnetic reluctance\"></p>

<P>
For a prismatic or cylindrical volume of length <i>l</i> and cross sectional area <i>A</i> with the magnetic flux entering and leaving the region through its end planes, the above equation simplifies to:</p>

<p><IMG SRC=\"../Images/Magnetic/FluxTubes/UsersGuide/FluxTubeConcept/eq_reluctance_prismatic.png\" ALT=\"Magnetic reluctance of a prismatic or cylindrical volume\"></p>

<p>
Similar equations can be derived for other geometries. In cases where a direct integration is not possible, the reluctance can be calclulated on base of average length, average cross sectional area and volume <i>V</i> respectively:
</p>

<p><IMG SRC=\"../Images/Magnetic/FluxTubes/UsersGuide/FluxTubeConcept/eq_reluctanceFromAverageGeometry.png\" ALT=\"Reluctance calculation from average geometric quantities\"></p>

<p>
Network elements for sources of a magnetic potential difference or magnetomotive force, i.e. coils or permanent magnets can be formulated as well. The resulting magnetic network models of actuators reflect the main dimensions of these devices as well as the normally nonlinear characteristics of their magnetically active materials.
</p>

</html>
"));
        end FluxTubeConcept;
        class ReluctanceForceCalculation "Reluctance forces"
          annotation(Documentation(info = "<html>
<h3><font color=\"#008000\" size=5>Calculation of reluctance forces from lumped magnetic network models</font></h3>

<p>
Generally, the thrust <i>F</i> developed by a translatory electro-magneto-mechanical actuator (similar for the rotational case with torque and angular position) is equal to the change of magnetic co-energy <i>W<sub>m</sub><sup>*</sup></i> with armature position <i>x</i> according to

<p><IMG SRC=\"../Images/Magnetic/FluxTubes/UsersGuide/ReluctanceForceCalculation/eq_CoEnergy_general.png\" ALT=\"Equation for force calculation from change of magnetic co-energy with armature position\"></p>

(<i>&Psi;</i> flux linkage, <i>i</i> actuator current). In lumped magnetic network models, the above equation simplifies to

<p><IMG SRC=\"../Images/Magnetic/FluxTubes/UsersGuide/ReluctanceForceCalculation/eq_forceFromPermeance_network.png\" ALT=\"Equation for force calculation in lumped magnetic network models\"></p>

<p>
where <i>n<sub>linear</sub></i> is the number of flux tube elements with constant relative permeability that change its permeance <i>G<sub>m i</sub></i> with armature position (index <i>i</i>), <i>V<sub>m i</sub></i> the magnetic voltage across each respective flux tube and <i>dG<sub>m i</sub>/dx</i> the derivative of the respective permeances with respect to armature position. Transition from the general formula based on magnetic co-energy to the latter one is outlined in <a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ka08]</a> for the reciprocal of the permeance, i.e. for the magnetic reluctance <i>R<sub>m</sub></i>. Note that
</p>


<p><IMG SRC=\"../Images/Magnetic/FluxTubes/UsersGuide/ReluctanceForceCalculation/eq_transition_forceReluctancePermeance.png\" ALT=\"Transition from force calculation based on reluctance to calculation based on permeance\"></p>

<p>with <i>&Phi;<sub>i</sub></i> being the magnetic flux through each respective flux tube element.</p>

<p>
Flux tube elements with <i>non-linear</i> material characteristics <i>&mu;<sub>r</sub></i>(<i>B</i>) in magnetic network models do not restrict the usability of the above equation. However, it is required that these nonlinear flux tube elements do not change its shape with armature motion (e.g. portion of a solenoid plunger where the magnetic flux passes through in axial direction). This limitation is not a strong one, since the permeance of nonlinear, but highly permeable ferromagnetic flux tube elements and its change with armature position compared to that of air gap flux tubes can be neglected in most cases. Because of this constraint, the dimensions of possibly nonlinear flux tube elements in sub-package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.FixedShape\">Shapes.FixedShape</a> are fixed, whereas the dimension in direction of motion of the linear flux tube elements in sub-package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.Force\">Shapes.Force</a> can vary during simulation. For the flux tubes defined in this package with their rather simple shapes, the derivative <i>dG<sub>m</sub>/dx</i> is given analytically. For more complex shapes and variations of dimensions with armature motion, it must be provided analytically during model development, preferably by extending the partial model <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Interfaces.PartialForce\">Interfaces.PartialForce</a>.
</p>

<p>
The sub-package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.Leakage\">Shapes.Leakage</a> contains flux tube shapes typical for leakage flux around prismatic or cylindrical poles. Since the permeance of these flux tubes does not change with armature position, they do not contribute to a reluctance actuator's thrust.
</p>


</html>
"));
        end ReluctanceForceCalculation;
        class Literature "Literature"
          annotation(Documentation(info = "<html>
<h3><font color=\"#008000\" size=5>Literature</font></h3>
<ul>
<li>
Usage of the Modelica Magnetic library is described in:
<dl>
<dt>[B&ouml;08] B&ouml;drich, T.:</dt>
<dd> <b>Electromagnetic Actuator Modelling with the Extended Modelica Magnetic Library</b>.
     Modelica 2008 Conference, Bielefeld, Germany,
     pp. 221-227, March 3-4, 2008.
     Download from:
     <a href=\"http://www.modelica.org/events/modelica2008/Proceedings/sessions/session2d2.pdf\">http://www.modelica.org/events/modelica2008/Proceedings/sessions/session2d2.pdf</a>
     </dd>
</dl>
</li>
<li>
The method of magnetic flux tubes as well as derivation of the permeance of many flux tube shapes is explained in detail in:
<dl>
<dt>[Ro41] Roters, H.:</dt>
<dd> <b>Electromagnetic Devices</b>.
New York: John Wiley &amp; Sons 1941 (8th Printing 1961)
</dd>
</dl>
</li>
<li> Structure, properties, applications and design of electromagnetic (reluctance type) actuators are thoroughly described in:
<dl>
<dt>[Ka08] Kallenbach, E.; Eick, R.; Quendt, P.; Str&ouml;hla, T.; Feindt, K.; Kallenbach, M.:</dt>
<dd><b>Elektromagnete: Grundlagen, Berechnung, Entwurf und Anwendung</b>.
3rd ed., Wiesbaden: Vieweg Teubner 2008 (in German)
<br>&nbsp;</dd>
<dt>[Ro00] Roschke, T.:</dt
<dd><b>Entwurf geregelter elektromagnetischer Antriebe f&uuml;r Luftsch&uuml;tze</b>.
    Fortschritt-Berichte VDI, Reihe 21, Nr. 293, D&uuml;sseldorf: VDI-Verlag 2000 (in German)</dd>
</dl>
</li>
<li>
Application of the method of magnetic flux tubes to the design of rotational electrical machines is explained for example in:
<dl>
<dt>[HM94] Hendershot, J.R. Jr.; Miller, T.J.E.:</dt>
<dd> <b>Design of Brushless Permanent-Magnet Motors</b>.
Magna Physics Publishing and Oxford University Press 1994
</dd>
</dl>
</li>

</ul>

</html>
"));
        end Literature;
        class Contact "Contact"
          annotation(Documentation(info = "<html>
<h3><font color=\"#008000\" size=5>Contact</font></h3>

<dl>
<dt>
<b>Main Author:</b></dt>
<br>
<dd>
    <a href=\"http://www.ifte.de/mitarbeiter/boedrich.html\">Thomas B&ouml;drich</a><br>
    Dresden University of Technology<br>
    Institute of Electromechanical and Electronic Design<br>
    01062 Dresden, Germany<br>
    Phone: +49 - 351 - 463 36296<br>
    Fax: +49 - 351 - 463 37183<br>
    email: <A HREF=\"mailto:Thomas.Boedrich@mailbox.tu-dresden.de\">Thomas.Boedrich@mailbox.tu-dresden.de</A><br></dd>
</dl>
<br>
<p><b>Acknowledgements:</b></p>
<ul>

<li>
The magnetisation characteristics of the included soft magnetic materials were compiled and measured respectively by Thomas Roschke, now with Johnson Electric. Provision of this data is highly appreciated. He also formulated the approximation function used for description of the magnetisation characteristics of these materials.
</li>

<li>
Andr&eacute; Klick of then Dresden University of Technology, Dresden, Germany gave valuable support on the implementation of this library. His contribution is highly appreciated, too.
</li>

</ul>

</html>
"));
        end Contact;
      end UsersGuide;
      package Examples "Illustration of component usage with simple models of various devices"
        model SaturatedInductor "Inductor with saturation in the ferromagnetic core"
          Modelica.Magnetic.FluxTubes.Basic.Ground ground_m annotation(Placement(transformation(extent = {{50, -30},{70, -10}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SineVoltage source(freqHz = 50, phase = pi / 2, V = 230 * sqrt(2)) "Voltage applied to inductor" annotation(Placement(transformation(origin = { -80,10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Resistor r(R = 7.5) "Inductor coil resistance" annotation(Placement(transformation(extent = {{ -61,10},{ -41,30}}, rotation = 0)));
          Modelica.Magnetic.FluxTubes.Basic.ElectroMagneticConverter coil(N = 600, i(fixed = true)) "Inductor coil" annotation(Placement(transformation(extent = {{ -30,0},{ -10,20}}, rotation = 0)));
          Basic.ConstantReluctance r_mLeak(R_m = 1200000.0) "Constant leakage reluctance" annotation(Placement(transformation(origin = {10,10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Magnetic.FluxTubes.Shapes.FixedShape.Cuboid r_mAirPar(a = 0.025, b = 0.025, nonLinearPermeability = false, mu_rConst = 1, l = 0.0001) "Reluctance of small parasitic air gap (ferromagnetic core packeted from single sheets)" annotation(Placement(transformation(extent = {{26,10},{46,30}}, rotation = 0)));
          Modelica.Magnetic.FluxTubes.Shapes.FixedShape.Cuboid r_mFe(mu_rConst = 1000, a = 0.025, b = 0.025, nonLinearPermeability = true, l = 4 * 0.065, material = Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.ElectricSheet.M350_50A()) "Reluctance of ferromagnetic inductor core" annotation(Placement(transformation(origin = {60,10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(Placement(transformation(extent = {{ -90, -30},{ -70, -10}}, rotation = 0)));
          annotation(experiment(StopTime = 0.1, Tolerance = 1e-07), Documentation(info = "<html>
<p>
This model demonstrates the effects of non-linear magnetisation characteristics of soft magnetic materials (hysteresis neglected). A sinusoidal voltage is applied to an inductor with a closed ferromagnetic core of rectangular shape. Set the <b>tolerance</b> to <b>1e-7</b>, <b>simulate for 0.1 s</b> and plot for example:
</p>

<pre>
    coil.i vs. time           // non-harmonic current due to saturation of the core material
    r_mFe.mu_r vs. r_mFe.B    // relative permeability vs. flux density inside core
    r_mFe.B vs. r_mFe.H       // magnetisation curve B(H); hysteresis neglected
</pre>

<p>
The magnetisation characteristics of the flux tube element representing the ferromagnetic core can easily be changed from simplified linear behaviour (nonLinearPermeability set to false and R_mFe.mu_rConst set to a positive value, preferably mu_rConst >> 1) to non-linear behaviour (e.g. selection of one of the electric sheets in <a href=\"Material.SoftMagnetic\">Material.SoftMagnetic</a> with nonLinearPermeability set to true). This enables for convenient inital design of magnetic circuits with linear material characteristics prior to simulation with non-linear behaviour.
</p>

<h4>Note</h4>

<p>
If the supply voltage has a zero-crossing when applied to the inductor at time t=0 (i.e. source.phase set to zero instead of pi/2), then the inrush current that is typical for switching of inductive loads can be observed.
</p>
</html>"), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100, -100},{80,50}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -100,50},{ -80,70},{100,70},{80,50},{ -100,50}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{100,70},{100, -80},{80, -100},{80,50},{100,70}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -120,130},{120,70}}, lineColor = {0,0,255}, textString = "%name"),Text(extent = {{ -100,30},{80, -80}}, lineColor = {0,0,255}, textString = "1")}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));

        equation 
          connect(source.p,r.p) annotation(Line(points = {{ -80,20},{ -61,20}}, color = {0,0,255}));
          connect(r.n,coil.p) annotation(Line(points = {{ -41,20},{ -30,20},{ -30,16}}, color = {0,0,255}));
          connect(source.n,coil.n) annotation(Line(points = {{ -80,0},{ -30,0},{ -30,4}}, color = {0,0,255}));
          connect(coil.port_p,r_mLeak.port_p) annotation(Line(points = {{ -10,16},{ -10,20},{10,20}}, color = {255,127,0}));
          connect(r_mLeak.port_p,r_mAirPar.port_p) annotation(Line(points = {{10,20},{26,20}}, color = {255,127,0}));
          connect(r_mAirPar.port_n,r_mFe.port_p) annotation(Line(points = {{46,20},{54,20},{54,20},{60,20}}, color = {255,127,0}));
          connect(r_mFe.port_n,r_mLeak.port_n) annotation(Line(points = {{60,0},{47.5,0},{47.5,5.55112e-16},{35,5.55112e-16},{35,0},{10,0}}, color = {255,127,0}));
          connect(r_mFe.port_n,coil.port_n) annotation(Line(points = {{60,0},{ -10,0},{ -10,4}}, color = {255,127,0}));
          connect(ground.p,source.n) annotation(Line(points = {{ -80, -10},{ -80,0}}, color = {0,0,255}, smooth = Smooth.None));
          connect(ground_m.port,r_mFe.port_n) annotation(Line(points = {{60, -10},{60,0},{60,0}}, color = {255,127,0}, smooth = Smooth.None));
        end SaturatedInductor;
        extends Modelica.Icons.Library;
        package MovingCoilActuator "Two translatory electrodynamic actuator models of different modelling depth and their comparison"
          annotation(Documentation(info = "<html>
<p>
Moving coil actuators are often called electrodynamic actuators and a proportional behaviour between force and current is expressed by a converter constant (see <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.ConstantActuator\">ConstantActuator</a>). However, in a simple moving coil actuator as presented in this example there is an additional non-linear force component that is due to the increase of the inductance when the armature coil moves into the ferromagnetic stator. A simple <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.PermeanceActuator\">PermeanceActuator</a> can be used to describe this non-linear force component.
</p>
</html>"), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -100},{80,50}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -100,50},{ -80,70},{100,70},{80,50},{ -100,50}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{100,70},{100, -80},{80, -100},{80,50},{100,70}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -120,130},{120,70}}, lineColor = {0,0,255}, textString = "%name"),Text(extent = {{ -100,32},{80, -78}}, lineColor = {0,0,255}, textString = "2")}));
          model ForceCurrentBehaviour "Comparison of the force-current characteristics of both converter models with armature blocked at mid-position"
            extends Modelica.Icons.Example;
            annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -98,96},{100,86}}, lineColor = {0,0,255}, textString = "Comparison of the force-current characteristics of both converter models"),Text(extent = {{ -98,88},{2,78}}, lineColor = {0,0,255}, textString = "with armature blocked at mid-position")}), experiment(StopTime = 6, Tolerance = 1e-07), Documentation(info = "<html>
<p>
Have a look at <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.ConstantActuator\">ConstantActuator</a> and at <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.PermeanceActuator\">PermeanceActuator</a> for an explanation of both converter models. <br>
</p>
<p>
Simulation of the force-current characteristics of both converter models with the armature blocked at mid-position x=0 reveals the difference between the two models. In the <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.ConstantActuator\">ConstantActuator</a>, force is proportional to current. In the simple <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.PermeanceActuator\">PermeanceActuator</a> there is an additional non-linear force component that is due to the dependency of the inductance on the armature position. Comparison with FEA results validates the higher accuracy of the <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.PermeanceActuator\">PermeanceActuator</a>. In the FEA model, the relative permeability of the stator iron was set to mu_rFe=const.=1000 in order to avoid additional non-linear force components due to saturation. <b>Simulate for 6 s</b> and <b>plot vs. current (e.g. iSensor.i)</b></p>
<pre>
    pmFixedPos.flange_b.f       // force of permeance model
    cFixedPos.flange_b.f        // force of converter constant model
    comparisonWithFEA.y[1]      // force of FEA model for comparison
</pre>
</html>"));
            Modelica.Electrical.Analog.Basic.Ground pmGround annotation(Placement(transformation(extent = {{ -70, -100},{ -50, -80}}, rotation = 0)));
            Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.PermeanceActuator pmActuator(x(start = 0), material = Modelica.Magnetic.FluxTubes.Material.HardMagnetic.PlasticNdFeB()) "Moving coil actuator described with permeance model" annotation(Placement(transformation(extent = {{ -20, -70},{0, -50}}, rotation = 0)));
            Modelica.Mechanics.Translational.Components.Fixed pmFixedPos(s0 = 0) "Fixed armature position" annotation(Placement(transformation(extent = {{10, -70},{30, -50}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.RampCurrent pmRampCurrent(I =  -6, duration = 6, offset = 3) "Ideal current source " annotation(Placement(transformation(origin = { -60, -60}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.Electrical.Analog.Basic.Ground cGround annotation(Placement(transformation(extent = {{ -70, -20},{ -50,0}}, rotation = 0)));
            Modelica.Mechanics.Translational.Components.Fixed cFixedPos(s0 = 0) "Fixed armature position" annotation(Placement(transformation(extent = {{10,10},{30,30}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.RampCurrent cRampCurrent(I =  -6, duration = 6, offset = 3) "Ideal current source " annotation(Placement(transformation(origin = { -60,20}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.ConstantActuator cActuator "Moving coil actuator described with converter constant" annotation(Placement(transformation(extent = {{ -20,10},{0,30}}, rotation = 0)));
            Modelica.Blocks.Tables.CombiTable1Ds comparisonWithFEA(table = [ -3, -9.65653; -2.5, -8.28587; -2, -6.82002; -1.5, -5.25898; -1, -3.60274; -0.5, -1.85131;0, -0.00468;0.5,1.93714;1,3.97415;1.5,6.10636;2,8.33376;2.5,10.65636;3,13.07415]) "Column 1: current, col. 2: force; mu_rFe=const.=1000 in FEA model" annotation(Placement(transformation(extent = {{40,50},{60,70}}, rotation = 0)));
            Modelica.Electrical.Analog.Sensors.CurrentSensor iSensor "Input value for look-up table with FEA results" annotation(Placement(transformation(extent = {{ -50,40},{ -30,20}}, rotation = 0)));

          equation 
            connect(pmFixedPos.flange,pmActuator.flange) annotation(Line(points = {{20, -60},{0, -60}}, color = {0,127,0}));
            connect(pmRampCurrent.p,pmActuator.p) annotation(Line(points = {{ -60, -50},{ -30, -50},{ -30, -54},{ -20, -54}}, color = {0,0,255}));
            connect(pmActuator.n,pmRampCurrent.n) annotation(Line(points = {{ -20, -66},{ -30, -66},{ -30, -70},{ -60, -70}}, color = {0,0,255}));
            connect(pmGround.p,pmRampCurrent.n) annotation(Line(points = {{ -60, -80},{ -60, -70}}, color = {0,0,255}));
            connect(cGround.p,cRampCurrent.n) annotation(Line(points = {{ -60,0},{ -60,10}}, color = {0,0,255}));
            connect(cActuator.flange,cFixedPos.flange) annotation(Line(points = {{0,20},{20,20}}, color = {0,127,0}));
            connect(cRampCurrent.n,cActuator.n) annotation(Line(points = {{ -60,10},{ -30,10},{ -30,14},{ -20,14}}, color = {0,0,255}));
            connect(cRampCurrent.p,iSensor.p) annotation(Line(points = {{ -60,30},{ -50,30}}, color = {0,0,255}));
            connect(cActuator.p,iSensor.n) annotation(Line(points = {{ -20,26},{ -30,26},{ -30,30}}, color = {0,0,255}));
            connect(iSensor.i,comparisonWithFEA.u) annotation(Line(points = {{ -40,40},{ -40,60},{38,60}}, color = {0,0,127}));
          end ForceCurrentBehaviour;
          model ForceStrokeBehaviour "Force-stroke characteristic of the permeance model at constant current"
            extends Modelica.Icons.Example;
            annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StartTime =  -4, StopTime = 4, Tolerance = 1e-07), Documentation(info = "<html>
<p>
Have a look at <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.ConstantActuator\">ConstantActuator</a> and at <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.PermeanceActuator\">PermeanceActuator</a> for an explanation of both converter models. <br>
</p>
<p>
Simulation of the force-stroke characteristic of the <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.PermeanceActuator\">PermeanceActuator</a> with a constant current I=3A and a forced armature movement (similar to measurements in reality) shows the dependency of both force-generating permeances G_ma and G_mb as well as inductance L on armature position x. <b>Simulate for 8 s</b> and <b>plot vs.</b> armature position <b>feedX.flange_b.s</b> (same physical quantities together in a common diagram for comparison):</p>
<pre>
    feedX.flange_b.f            // force of permeance model (permeance of stator iron neglegted in this model)
    comparisonWithFEA.y[1]      // force of FEA model with non-linear stator iron 1.0718
    comparisonWithFEA.y[2]      // force of FEA model with mu_rFe=const.=1000
    actuator.g_ma.G_m           // permeance G_ma
    actuator.g_mb.G_m           // permeance G_mb
    actuator.L                  // inductance of permeance model
    comparisonWithFEA.y[3]      // inductance of FEA model for comparison (mu_rFe=const.=1000).
</pre>
</html>"));
            Modelica.Electrical.Analog.Basic.Ground ground annotation(Placement(transformation(extent = {{ -70, -30},{ -50, -10}}, rotation = 0)));
            Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.PermeanceActuator actuator(x(start = 0), material = Modelica.Magnetic.FluxTubes.Material.HardMagnetic.PlasticNdFeB()) annotation(Placement(transformation(extent = {{ -30,0},{ -10,20}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.ConstantCurrent source(I = 3) annotation(Placement(transformation(origin = { -60,10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.Blocks.Sources.Ramp sweepX(height = 0.00799, duration = 8, offset =  -0.003995, startTime =  -4) annotation(Placement(transformation(extent = {{70,0},{50,20}}, rotation = 0)));
            Modelica.Mechanics.Translational.Sources.Position feedX(exact = true) annotation(Placement(transformation(extent = {{30,0},{10,20}}, rotation = 0)));
            Modelica.Blocks.Tables.CombiTable1Ds comparisonWithFEA(table = [ -0.004, -8.8729, -9.07503,0.00332; -0.0035, -9.05239, -9.25042,0.00352; -0.003, -9.1915, -9.38558,0.00371; -0.0025, -9.28247, -9.47266,0.0039; -0.002, -9.3587, -9.54503,0.00409; -0.0015, -9.41568, -9.59782,0.00429; -0.001, -9.45496, -9.6331,0.00448; -0.0005, -9.47427, -9.64839,0.00467;0, -9.48639, -9.65616,0.00486;0.0005, -9.48623, -9.65174,0.00505;0.001, -9.4732, -9.63435,0.00524;0.0015, -9.44143, -9.59825,0.00543;0.002, -9.39915, -9.55226,0.00562;0.0025, -9.33166, -9.47988,0.00581;0.003, -9.23707, -9.38112,0.006;0.0035, -9.09497, -9.23417,0.00619;0.004, -8.91839, -9.05337,0.00638]) "Column 1: position, col.2: force with non-linear stator iron, col.3: force with mu_rFe=const.=1000, col.4: inductance with mu_rFe=const.=1000 " annotation(Placement(transformation(extent = {{50,40},{70,60}}, rotation = 0)));

          equation 
            connect(ground.p,source.n) annotation(Line(points = {{ -60, -10},{ -60,0}}, color = {0,0,255}));
            connect(source.n,actuator.n) annotation(Line(points = {{ -60,0},{ -46,0},{ -46,4},{ -30,4}}, color = {0,0,255}));
            connect(source.p,actuator.p) annotation(Line(points = {{ -60,20},{ -46,20},{ -46,16},{ -30,16}}, color = {0,0,255}));
            connect(sweepX.y,feedX.s_ref) annotation(Line(points = {{49,10},{32,10}}, color = {0,0,127}));
            connect(feedX.flange,actuator.flange) annotation(Line(points = {{10,10},{ -10,10}}, color = {0,127,0}));
            connect(feedX.s_ref,comparisonWithFEA.u) annotation(Line(points = {{32,10},{32,50},{48,50}}, color = {0,0,127}));
          end ForceStrokeBehaviour;
          model ArmatureStroke "Armature stroke of both moving coil actuator models after a voltage step at time t=0"
            extends Modelica.Icons.Example;
            annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -98,98},{48,86}}, lineColor = {0,0,255}, textString = "Comparison of a pull-in stroke of both converter models"),Text(extent = {{ -98,90},{ -30,80}}, lineColor = {0,0,255}, textString = "after a voltage step at t=0")}), experiment(StopTime = 0.05, Tolerance = 1e-07), Documentation(info = "<html>
<p>
Have a look at <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.ConstantActuator\">ConstantActuator</a> and at <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.PermeanceActuator\">PermeanceActuator</a>
for an explanation of both actuator models.</p>

<p>
A voltage step at time t=0 is applied to both actuator models. In each model, the armature and an attached load mass perform a stroke between the two stoppers included in cActuator.armature and pmActuator.armature respectively. <b>Simulate for 0.05 s</b> and plot vs. time (same physical quantities together in a common diagram for comparison):
</p>
<pre>
    cActuator.p.i                     // input current to converter constant model
    pmActuator.p.i                    // input current to permeance model
    cActuator.armature.flange_a.f     // actuator force of converter constant model
    pmActuator.armature.flange_a.f    // actuator force of permeance model
    cActuator.x                       // armature position of converter constant model
    pmActuator.x                      // armature position of permeance model
    cActuator.L                       // inductance of converter constant model
    pmActuator.L                      // inductance of permeance model
</pre>
<p>
The initial current rise in both actuator models is due to the inductance of the actuator coil. After acceleration of the armature and the load, the current decreases due to the motion-induced back-emf. Bouncing occurs when the armatures of both models arrive at the stopper at maximum armature position. The bouncing is rather intense due to the absence of any kind of external friction in this simple example (apart from the nonlinear damping in the stopper elements). After decay of the bouncing, both actuators operate under conditions valid for a blocked armature.
</p>
<p>
Whereas the steady state current is the same in both models, the steady state actuator force is not due to the neglection of the non-linear force component in the converter constant model. Differences in the current rise of both models are due to the neglection of the coil inductance variation in the converter constant model.
</p>
</html>"));
            Modelica.Electrical.Analog.Basic.Ground pmGround annotation(Placement(transformation(extent = {{ -80, -70},{ -60, -50}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.StepVoltage pmSource(startTime = 0, V = pmActuator.R * 1.5) "Steady state current 1.5A" annotation(Placement(transformation(origin = { -70, -30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.PermeanceActuator pmActuator(material = Modelica.Magnetic.FluxTubes.Material.HardMagnetic.PlasticNdFeB(), x(start = pmActuator.x_min, fixed = true), armature(v(fixed = true)), coil(i(fixed = true))) "Moving coil actuator described with permeance model" annotation(Placement(transformation(extent = {{ -40, -40},{ -20, -20}}, rotation = 0)));
            Modelica.Mechanics.Translational.Components.Mass pmLoad(m = 0.05) "Load to be moved in addition to the armature mass" annotation(Placement(transformation(extent = {{0, -40},{20, -20}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground cGround annotation(Placement(transformation(extent = {{ -80,0},{ -60,20}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.StepVoltage cSource(startTime = 0, V = cActuator.R * 1.5) "Steady state current 1.5A" annotation(Placement(transformation(origin = { -70,40}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.ConstantActuator cActuator(x(start = cActuator.x_min, fixed = true), armature(v(fixed = true)), l(i(start = 0, fixed = true))) "Moving coil actuator described with converter constant" annotation(Placement(transformation(extent = {{ -40,30},{ -20,50}}, rotation = 0)));
            Modelica.Mechanics.Translational.Components.Mass cLoad(m = 0.05) "Load to be moved in addition to the armature mass" annotation(Placement(transformation(extent = {{0,30},{20,50}}, rotation = 0)));

          equation 
            connect(pmLoad.flange_a,pmActuator.flange) annotation(Line(points = {{0, -30},{ -20, -30}}, color = {0,127,0}));
            connect(cGround.p,cSource.n) annotation(Line(points = {{ -70,20},{ -70,30}}, color = {0,0,255}));
            connect(cLoad.flange_a,cActuator.flange) annotation(Line(points = {{0,40},{ -20,40}}, color = {0,127,0}));
            connect(cSource.p,cActuator.p) annotation(Line(points = {{ -70,50},{ -56,50},{ -56,46},{ -40,46}}, color = {0,0,255}));
            connect(cSource.n,cActuator.n) annotation(Line(points = {{ -70,30},{ -56,30},{ -56,34},{ -40,34}}, color = {0,0,255}));
            connect(pmSource.n,pmGround.p) annotation(Line(points = {{ -70, -40},{ -70, -50}}, color = {0,0,255}));
            connect(pmSource.n,pmActuator.n) annotation(Line(points = {{ -70, -40},{ -54, -40},{ -54, -36},{ -40, -36}}, color = {0,0,255}));
            connect(pmSource.p,pmActuator.p) annotation(Line(points = {{ -70, -20},{ -55, -20},{ -55, -24},{ -40, -24}}, color = {0,0,255}));
          end ArmatureStroke;
          package Components "Components to be used in examples"
            model PermeanceActuator "Detailed actuator model for rough magnetic design of actuator and system simulation"
              parameter Real N = 140 "Number of turns";
              parameter SI.Resistance R = 2.86 "Coil resistance";
              parameter SI.Radius r_core = 0.0125 "Radius of ferromagnetic stator core";
              parameter SI.Length l_PM = 0.0035 "Radial thickness of permanent magnet ring";
              parameter SI.Length t = 0.02 "Axial length of permanent magnet ring and air gap respectively";
              parameter SI.Length l_air = 0.003 "Total radial length of armature air gap";
              parameter SI.Length l_FeOut = 0.004 "Radial thickness of outer back iron (for estimation of leakage permeance)";
              parameter Modelica.Magnetic.FluxTubes.Material.HardMagnetic.BaseData material = Modelica.Magnetic.FluxTubes.Material.HardMagnetic.BaseData() "Ferromagnetic material characteristics" annotation(choicesAllMatching = true, Dialog(group = "Material"));
              parameter SI.Mass m_a = 0.012 "Mass of armature" annotation(Dialog(group = "Armature and stopper"));
              parameter SI.TranslationalSpringConstant c = 100000000000.0 "Spring stiffness between impact partners" annotation(Dialog(group = "Armature and stopper"));
              parameter SI.TranslationalDampingConstant d = 400 "Damping coefficient between impact partners" annotation(Dialog(group = "Armature and stopper"));
              parameter SI.Position x_min =  -0.004 "Position of stopper at minimum armature position" annotation(Dialog(group = "Armature and stopper"));
              parameter SI.Position x_max = 0.004 "Position of stopper at maximum armature position" annotation(Dialog(group = "Armature and stopper"));
              SI.Position x(start = x_min, stateSelect = StateSelect.prefer) "Armature position, alias for flange position";
              SI.Inductance L "Coil inductance";
              Modelica.Magnetic.FluxTubes.Sources.ConstantMagneticPotentialDifference mmf_PM(final V_m = material.H_cB * l_PM) "Permanent magnet's magnetomotive force" annotation(Placement(transformation(origin = {10,20}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              annotation(__Dymola_Images(Parameters(source = "../Images/Magnetic/FluxTubes/Examples/MovingCoilActuator/MovingCoilActuator_dimensions.png")), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
In the <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.ConstantActuator\">ConstantActuator</a> model the force F is strictly proportional to the current i as indicated by the converter constant c. However, there is an additional non-linear force component in such an actuator that is due to the dependency of the coil inductance L on the armature position x. The inductance increases as the armature moves into the stator. The total force is
</p>

<pre>
        1  2 dL
    F = - i  --  + c i
        2    dx
</pre>

<p>
Both force components are properly considered with a simple permeance model as shown in the figures below. Figure (a) illustrates the dimensions of the axisymmetric moving coil actuator that are needed in the permeance model. Figure (b) shows partitioning into flux tubes and the permanent magnetic field without current. G_ma and G_mb both are the permeances resulting from a series connection of the permaent magnet and air gap sections. The field plot of the coil-imposed mmf is shown in figure (c) without the permanent magnetic mmf (H_cB=0). The placement of the magnetic network components in figure (d) retains the geometric structure of the actuator. In figure (e), the permeance model is restructured and thus simplified.
</p>

<p><IMG SRC=\"../Images/Magnetic/FluxTubes/Examples/MovingCoilActuator/MovingCoilActuator_PermeanceModel.png\" ALT=\"Structure, assigned flux tubes and field plots of the moving coil actuator\"></p>
</html>"), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -90,100},{90, -100}}, lineColor = {255,128,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -90,100},{ -50, -100}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{90,80},{ -90,100}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{90, -80},{ -90, -100}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{70,34},{ -90, -34}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{90,52},{ -16,64}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {255,213,170}, fillPattern = FillPattern.Solid),Rectangle(extent = {{90, -64},{ -12, -52}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {255,213,170}, fillPattern = FillPattern.Solid),Text(extent = {{0,160},{0,120}}, lineColor = {0,0,255}, textString = "%name")}));
              Modelica.Magnetic.FluxTubes.Examples.Utilities.TranslatoryArmatureAndStopper armature(final L = 0, final m = m_a, final c = c, final d = d, n = 2, final x_max = x_max, final x_min = x_min) "Inertia of moving coil + coil carrier; stoppers at end of stroke range" annotation(Placement(transformation(extent = {{60, -10},{80,10}}, rotation = 0)));
              Modelica.Electrical.Analog.Basic.Resistor r(final R = R) annotation(Placement(transformation(origin = { -80,30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Basic.Ground ground annotation(Placement(transformation(extent = {{30, -38},{50, -18}}, rotation = 0)));
              Modelica.Magnetic.FluxTubes.Basic.ElectroMagneticConverter coil(final N = N) annotation(Placement(transformation(origin = {0, -20}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Shapes.Force.HollowCylinderRadialFlux g_ma(final dlBydx = 1, final l = t / 2 + x, final r_i = r_core, final r_o = r_core + l_air + l_PM, final mu_r = 1.05, final useSupport = false) annotation(Placement(transformation(origin = { -20,0}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Magnetic.FluxTubes.Shapes.Force.HollowCylinderRadialFlux g_mb(final r_i = r_core, final r_o = r_core + l_air + l_PM, final dlBydx =  -1, final l = t / 2 - x, final mu_r = 1.05, final useSupport = false) annotation(Placement(transformation(origin = {30,0}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Magnetic.FluxTubes.Shapes.Leakage.CoaxCylindersEndFaces g_mLeak1(final r_1 = r_core + l_air + l_PM, final r_2 = r_core + l_air + l_PM + l_FeOut, final r_0 = r_core) "Leakgage between coaxial end planes of ferromagnetic stator core and outer back iron" annotation(Placement(transformation(origin = { -60,10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Magnetic.FluxTubes.Shapes.Leakage.HalfCylinder g_mLeak2(final l = 2 * pi * (r_core + (l_air + l_PM) / 2)) "Leakgage between edges of ferromagnetic stator core and outer back iron" annotation(Placement(transformation(origin = { -40,10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.PositivePin p "Electrical connector" annotation(Placement(transformation(extent = {{ -110,50},{ -90,70}}, rotation = 0)));
              Modelica.Electrical.Analog.Interfaces.NegativePin n "Electrical connector" annotation(Placement(transformation(extent = {{ -90, -70},{ -110, -50}}, rotation = 0)));
              Modelica.Mechanics.Translational.Interfaces.Flange_b flange "Flange of component" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));

            equation 
              x = flange.s;
              L = coil.N ^ 2 * (g_ma.G_m + g_mLeak1.G_m + g_mLeak2.G_m);
              connect(armature.flange_b,flange) annotation(Line(points = {{80,0},{100,0}}, color = {0,127,0}, pattern = LinePattern.None));
              connect(r.p,p) annotation(Line(points = {{ -80,40},{ -80,60},{ -100,60}}, color = {0,0,255}));
              connect(armature.flange_a,g_mb.flange) annotation(Line(points = {{60,0},{40,0},{40, -1.83697e-15}}, color = {0,127,0}));
              connect(g_mb.flange,g_ma.flange) annotation(Line(points = {{40, -1.83697e-15},{40,40},{ -10,40},{ -10, -1.83697e-15}}, color = {0,127,0}));
              connect(g_mLeak1.port_n,g_ma.port_n) annotation(Line(points = {{ -60,0},{ -60, -10},{ -20, -10}}, color = {255,127,0}));
              connect(g_mLeak2.port_n,g_ma.port_n) annotation(Line(points = {{ -40,0},{ -40, -10},{ -20, -10}}, color = {255,127,0}));
              connect(g_ma.port_n,coil.port_p) annotation(Line(points = {{ -20, -10},{ -18, -10},{ -18, -12},{ -14, -12},{ -14, -10},{ -6, -10}}, color = {255,127,0}));
              connect(coil.port_n,g_mb.port_n) annotation(Line(points = {{6, -10},{30, -10}}, color = {255,127,0}));
              connect(g_mb.port_n,mmf_PM.port_p) annotation(Line(points = {{30, -10},{50, -10},{50,30},{10,30}}, color = {255,127,0}));
              connect(mmf_PM.port_p,g_mLeak2.port_p) annotation(Line(points = {{10,30},{ -40,30},{ -40,20}}, color = {255,127,0}));
              connect(mmf_PM.port_p,g_mLeak1.port_p) annotation(Line(points = {{10,30},{ -60,30},{ -60,20}}, color = {255,127,0}));
              connect(g_ma.port_p,mmf_PM.port_n) annotation(Line(points = {{ -20,10},{10,10}}, color = {255,127,0}));
              connect(mmf_PM.port_n,g_mb.port_p) annotation(Line(points = {{10,10},{30,10}}, color = {255,127,0}));
              connect(r.n,coil.p) annotation(Line(points = {{ -80,20},{ -80, -30},{ -6, -30}}, color = {0,0,255}));
              connect(coil.n,n) annotation(Line(points = {{6, -30},{6, -60},{ -100, -60}}, color = {0,0,255}));
              connect(ground.port,g_mb.port_n) annotation(Line(points = {{40, -18},{40, -10},{30, -10}}, color = {255,127,0}, smooth = Smooth.None));
            end PermeanceActuator;
            model ConstantActuator "Simple behavioural actuator model for system simulation"
              parameter SI.ElectricalForceConstant k = 3.88 "Converter constant";
              parameter SI.Resistance R = 2.86 "Coil resistance";
              parameter SI.Inductance L = 0.0051 "Coil inductance at mid-stroke";
              parameter SI.Mass m_a = 0.012 "Armature mass" annotation(Dialog(group = "Armature and stopper"));
              parameter Modelica.SIunits.TranslationalSpringConstant c = 100000000000.0 "Spring stiffness between impact partners" annotation(Dialog(group = "Armature and stopper"));
              parameter Modelica.SIunits.TranslationalDampingConstant d = 400 "Damping coefficient between impact partners" annotation(Dialog(group = "Armature and stopper"));
              parameter SI.Position x_min =  -0.004 "Minimum armature position" annotation(Dialog(group = "Armature and stopper"));
              parameter SI.Position x_max = 0.004 "Maximum armature position" annotation(Dialog(group = "Armature and stopper"));
              SI.Position x(start = x_min, stateSelect = StateSelect.prefer) "Armature position, alias for flange position";
              Modelica.Electrical.Analog.Basic.Resistor r(final R = R) "Coil resistance" annotation(Placement(transformation(extent = {{ -90,50},{ -70,70}}, rotation = 0)));
              Modelica.Magnetic.FluxTubes.Examples.Utilities.TranslatoryArmatureAndStopper armature(final m = m_a, final x_max = x_max, final x_min = x_min, final L = 0, final c = c, final d = d, n = 2) "Armature inertia with stoppers at end of stroke range" annotation(Placement(transformation(extent = {{60, -10},{80,10}}, rotation = 0)));
              Modelica.Electrical.Analog.Basic.Inductor l(final L = L) "Coil inductance" annotation(Placement(transformation(extent = {{ -60,50},{ -40,70}}, rotation = 0)));
              Modelica.Electrical.Analog.Basic.TranslationalEMF electroTranslationalConverter(final k = k) annotation(Placement(transformation(extent = {{ -40, -10},{ -20,10}}, rotation = 0)));
              Modelica.Electrical.Analog.Interfaces.PositivePin p "Electrical connector" annotation(Placement(transformation(extent = {{ -110,50},{ -90,70}}, rotation = 0)));
              Modelica.Electrical.Analog.Interfaces.NegativePin n "Electrical connector" annotation(Placement(transformation(extent = {{ -90, -70},{ -110, -50}}, rotation = 0)));
              Modelica.Mechanics.Translational.Interfaces.Flange_b flange "Flange of component" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));

            equation 
              flange.s = x;
              annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -80,100},{80, -100}}, lineColor = {255,128,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -70,80},{72, -80}}, lineColor = {255,128,0}, textString = "c"),Line(points = {{ -90,60},{ -80,60}}, color = {0,0,255}),Line(points = {{ -90, -60},{ -80, -60}}, color = {0,0,255}),Line(points = {{80,0},{90,0}}, color = {0,127,0}),Text(extent = {{0,160},{0,120}}, lineColor = {0,0,255}, textString = "%name")}), Documentation(info = "<html>
<p>
Similar to rotational DC-Motors, the electro-mechanical energy conversion of translatory electrodynamic actuators and generators of moving coil and moving magnet type can be described with the following two converter equations:
</p>

<pre>
      F = c * i
    V_i = c * v
</pre>

<p>
with electrodynamic or <i>Lorentz</i> force F, converter constant c, current i, induced back-emf V_i and armature velocity v. The model is very similar to the well-known behavioural model of a rotational one-phase DC-Machine, except that it is for translatory motion. For a moving coil actuator with a coil inside an air gap with flux density B and a total wire length l inside the magnetic field, the converter constant becomes
</p>

<pre>
    c = B * l.
</pre>

<p>
The converter constant c as well as coil resistance R and inductance L are assumed to be known, e.g. from measurements or catalogue data. Hence this model is well-suited for system simulation together with neighbouring subsystems, but not for actuator design, where the motor constant is to be found on base of the magnetic circuit's geometry, material properties and winding data. See <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.Components.PermeanceActuator\">PermeanceActuator</a> for a more accurate model of this actuator that is based on a magnetic network. Due to identical connectors, both models can be used in system simulation, e.g. to simulate a stroke as demonstrated in <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.MovingCoilActuator.ArmatureStroke\">ArmatureStroke</a>.
</p>
</html>"));
              connect(r.p,p) annotation(Line(points = {{ -90,60},{ -100,60}}, color = {0,0,255}));
              connect(l.p,r.n) annotation(Line(points = {{ -60,60},{ -70,60}}, color = {0,0,255}));
              connect(armature.flange_b,flange) annotation(Line(points = {{80,0},{85,0},{85,1.16573e-15},{90,1.16573e-15},{90,0},{100,0}}, color = {0,127,0}));
              connect(l.n,electroTranslationalConverter.p) annotation(Line(points = {{ -40,60},{ -30,60},{ -30,10}}, color = {0,0,255}));
              connect(n,electroTranslationalConverter.n) annotation(Line(points = {{ -100, -60},{ -30, -60},{ -30, -10}}, color = {0,0,255}));
              connect(electroTranslationalConverter.flange,armature.flange_a) annotation(Line(points = {{ -20,0},{0,0},{0, -6.10623e-16},{20, -6.10623e-16},{20,0},{60,0}}, color = {0,127,0}));
            end ConstantActuator;
          end Components;
        end MovingCoilActuator;
        annotation(Documentation(info = "<html>
</html>"));
        package SolenoidActuator "Two models of a reluctance actuator of different modelling depth and their comparison and usage"
          model ComparisonQuasiStationary "Slow forced armature motion of both solenoid models so that electromagnetic field and current are quasi-stationary"
            extends Modelica.Icons.Example;
            parameter SI.Voltage v_step = 12 "Applied voltage";
            annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 10, Tolerance = 1e-07), Documentation(info = "<html>
<p>
Have a look at <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator\">SolenoidActuator</a> for general comments and at <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.Components.SimpleSolenoid\">SimpleSolenoid</a> and <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.Components.AdvancedSolenoid\">AdvancedSolenoid</a> for a detailed description of both magnetic network models.
</p>

<p>
Similar to static force-stroke measurements on real actuators, the armatures of both actuator models are forced to move slowly here. Hence, the dynamics of the electrical subsystems due to coil inductance and armature motion can be neglected and the static force-stroke characteristics are obtained. To illustrate the accuracy to be expected from the lumped magnetic network models, results obtained with stationary FEA are included as reference (position-dependent force, armature flux and actuator inductance). Note that these reference values are valid for the default supply voltage v_step=12V DC only!
</p>

<p>
Set the <b>tolerance</b> to <b>1e-7</b> and <b>simulate for 10 s</b>. Plot in one common window the electromagnetic force of the two magnetic network models and the FEA reference <b>vs. armature position x_set.y</b>:
</p>

<pre>
    simpleSolenoid.armature.flange_a.f     // electromagnetic force of simple magnetic network model
    advancedSolenoid.armature.flange_a.f   // electromagnetic force of advaned magnetic network model
    comparisonWithFEA.y[1]                 // electromagnetic force obtained with FEA as reference
</pre>

<p>
Electromagnetic or reluctance forces always act towards a decrease of air gap lengths. With the defined armature position coordinate x, the forces of the models are negative.
</p>

<p>
The magnetic flux through the armature and the actuator's static inductance both illustrate the differences between the two magnetic network models. Similar to the forces, compare these quantities in one common plot window for each variable (plot vs. armature position x_set.y):
</p>

<pre>
    simpleSolenoid.G_mFeArm.Phi            // magnetic flux through armature of simple magnetic network model
    advancedSolenoid.G_mFeArm.Phi          // magnetic flux through armature of advanced magnetic network model
    comparisonWithFEA.y[2]                 // magnetic flux obtained with FEA as reference


    simpleSolenoid.coil.L_stat             // static inductance of simple magnetic network model
    advancedSolenoid.L_statTot             // series connection of both partial coils of advanced network model
    comparisonWithFEA.y[3]                 // static inductance obtained with FEA as reference
</pre>

<p>
As mentioned in the description of both magnetic network models, one can tell the higher armature flux and inductance of the advanced solenoid model at large air gaps compared to that of the simple model. The effect of this difference on dynamic model behaviour can be analysed in <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.ComparisonPullInStroke\">ComparisonPullInStroke</a>.
</p>
</html>"));
            Modelica.Blocks.Sources.Ramp x_set(duration = 10, height =  -(advancedSolenoid.x_max - advancedSolenoid.x_min), offset = advancedSolenoid.x_max) "Prescribed armature position, slow enforced motion from x_max to x_min" annotation(Placement(transformation(extent = {{80, -10},{60,10}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground advancedGround annotation(Placement(transformation(extent = {{ -80,10},{ -60,30}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.StepVoltage advancedSource(V = v_step) annotation(Placement(transformation(origin = { -70,50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.Components.AdvancedSolenoid advancedSolenoid annotation(Placement(transformation(extent = {{ -40,40},{ -20,60}}, rotation = 0)));
            Modelica.Mechanics.Translational.Sources.Position advancedFeed_x(f_crit = 1000, exact = true) annotation(Placement(transformation(origin = {0,50}, extent = {{ -10, -10},{10,10}}, rotation = 180)));
            Modelica.Electrical.Analog.Basic.Ground simpleGround annotation(Placement(transformation(extent = {{ -80, -90},{ -60, -70}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.StepVoltage simpleSource(V = v_step) annotation(Placement(transformation(origin = { -70, -50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.Components.SimpleSolenoid simpleSolenoid annotation(Placement(transformation(extent = {{ -40, -60},{ -20, -40}}, rotation = 0)));
            Modelica.Mechanics.Translational.Sources.Position simpleFeed_x(f_crit = 1000, exact = false) annotation(Placement(transformation(origin = {0, -50}, extent = {{ -10, -10},{10,10}}, rotation = 180)));
            Modelica.Blocks.Tables.CombiTable1Ds comparisonWithFEA(table = [0.00025, -85.8619,0.00014821,0.11954;0.0005, -59.9662,0.00013931,0.11004;0.00075, -41.0806,0.0001277,0.098942;0.001, -28.88,0.00011587,0.088425;0.00125, -21.4113,0.00010643,0.08015;0.0015, -16.8003,9.9406e-05,0.073992;0.00175, -13.6942,9.3416e-05,0.068792;0.002, -11.1188,8.8564e-05,0.064492;0.00225, -9.6603,8.4505e-05,0.060917;0.0025, -8.4835,8.1215e-05,0.058017;0.00275, -7.4658,7.7881e-05,0.055125;0.003, -6.5591,7.5197e-05,0.052733;0.00325, -5.9706,7.2447e-05,0.05035;0.0035, -5.5013,7.0342e-05,0.048525;0.00375, -5.0469,6.8527e-05,0.046867;0.004, -4.6573,6.6526e-05,0.045158;0.00425, -4.2977,6.4425e-05,0.043442;0.0045, -4.0912,6.2747e-05,0.04205;0.00475, -3.7456,6.1231e-05,0.040733;0.005, -3.5869,5.9691e-05,0.039467]) "Valid for u_source=12V only; column 1: position, col.2: force, col.3: armature flux, col.4: inductance" annotation(Placement(transformation(extent = {{60,60},{80,80}}, rotation = 0)));

          equation 
            connect(advancedGround.p,advancedSource.n) annotation(Line(points = {{ -70,30},{ -70,40}}, color = {0,0,255}));
            connect(x_set.y,advancedFeed_x.s_ref) annotation(Line(points = {{59,0},{20,0},{20,50},{12,50}}, color = {0,0,127}));
            connect(simpleSolenoid.p,simpleSource.p) annotation(Line(points = {{ -40, -44},{ -50, -44},{ -50, -40},{ -70, -40}}, color = {0,0,255}));
            connect(simpleSolenoid.n,simpleSource.n) annotation(Line(points = {{ -40, -56},{ -50, -56},{ -50, -60},{ -70, -60}}, color = {0,0,255}));
            connect(simpleSolenoid.flange,simpleFeed_x.flange) annotation(Line(points = {{ -20, -50},{ -10, -50}}, color = {0,127,0}));
            connect(advancedSolenoid.n,advancedSource.n) annotation(Line(points = {{ -40,44},{ -50,44},{ -50,40},{ -70,40}}, color = {0,0,255}));
            connect(simpleFeed_x.s_ref,x_set.y) annotation(Line(points = {{12, -50},{20, -50},{20,0},{59,0}}, color = {0,0,127}));
            connect(x_set.y,comparisonWithFEA.u) annotation(Line(points = {{59,0},{50,0},{50,70},{58,70}}, color = {0,0,127}));
            connect(advancedFeed_x.flange,advancedSolenoid.flange) annotation(Line(points = {{ -10,50},{ -20,50}}, color = {0,127,0}));
            connect(advancedSource.p,advancedSolenoid.p) annotation(Line(points = {{ -70,60},{ -50,60},{ -50,56},{ -40,56}}, color = {0,0,255}));
            connect(simpleGround.p,simpleSource.n) annotation(Line(points = {{ -70, -70},{ -70, -60}}, color = {0,0,255}));
          end ComparisonQuasiStationary;
          model ComparisonPullInStroke "Pull-in stroke of both solenoid models after a voltage step at time t=0"
            extends Modelica.Icons.Example;
            parameter SI.Voltage v_step = 12 "Applied voltage";
            annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 0.05, Tolerance = 1e-07), Documentation(info = "<html>
<p>
Have a look at <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator\">SolenoidActuator</a> for general comments and at <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.Components.SimpleSolenoid\">SimpleSolenoid</a> and <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.Components.AdvancedSolenoid\">AdvancedSolenoid</a> for a detailed description of both magnetic network models.
</p>

<p>
A voltage step is applied to both solenoid models at time t=0. The armatures of both models and therewith connected loads are pulled from their rest position at maximum air gap length to their minimum position that is due to a stopper. As a reference, simulation results obtained with a dynamic model based on stationary FEA look-up tables (not part of this library) are included. Note that these reference results are valid for the default supply voltage v_step=12V DC and the default load mass m_load=0.01kg only!
</p>

<p>
Set the <b>tolerance</b> to <b>1e-7</b> and <b>simulate for 0.05 s</b>. Plot actuator current, force and position of the two magnetic network models and the FEA-based reference <b>vs. time</b> (each quantity in one common plot window):
</p>

<pre>
Plot window for current:
    simpleSolenoid.p.i          // rapid current rise indicates low inductance of simple network model
    advancedSolenoid.p.i        // current rise slower, better match with FEA reference
    comparisonWithFEA.y[1]      // current obtained from dynamic model based on stationary FEA look-up tables

Plot window for force:
    simpleSolenoid.armature.flange_a.f       // reluctance force of simple actuator model
    advancedSolenoid.armature.flange_a.f     // reluctance force of advanced actuator model
    comparisonWithFEA.y[2]      // force obtained from dynamic model based on stationary FEA look-up tables

Plot window for position:
    simpleSolenoid.x            // armature position of simple actuator model
    advancedSolenoid.x          // armature position of advanced actuator model
    comparisonWithFEA.y[3]      // position obtained from dynamic model based on stationary FEA look-up tables
</pre>

<p>
The characteristic current drop during pull-in is due to both armature motion and increasing inductance with decreasing air gap length. Bouncing occurs when  armature and load of each model arrive at the stopper at minimum position. Although the pull-in times of the two magnetic network models are relatively close to the time obtained with the reference model, the accuracy of the advanced solenoid model is better, as one can tell from a comparison of the current rise at the beginning of the stroke.
</p>

</html>"));
            Modelica.Electrical.Analog.Basic.Ground advancedGround annotation(Placement(transformation(extent = {{ -70, -10},{ -50,10}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.StepVoltage advancedSource(V = v_step) annotation(Placement(transformation(origin = { -60,30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.Components.AdvancedSolenoid advancedSolenoid(x(fixed = true)) annotation(Placement(transformation(extent = {{ -20,20},{0,40}}, rotation = 0)));
            Modelica.Mechanics.Translational.Components.Mass advancedLoad(m = 0.01) "translatory load to be pulled horizontally" annotation(Placement(transformation(extent = {{20,20},{40,40}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground simpleGround annotation(Placement(transformation(extent = {{ -70, -90},{ -50, -70}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.StepVoltage simpleSource(V = v_step) annotation(Placement(transformation(origin = { -60, -50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.Components.SimpleSolenoid simpleSolenoid(x(fixed = true)) annotation(Placement(transformation(extent = {{ -20, -60},{0, -40}}, rotation = 0)));
            Modelica.Mechanics.Translational.Components.Mass simpleLoad(m = 0.01) "translatory load to be pulled horizontally" annotation(Placement(transformation(extent = {{20, -60},{40, -40}}, rotation = 0)));
            Modelica.Blocks.Sources.CombiTimeTable comparisonWithFEA(table = [0,0,0,0.005;2.61165e-07,7.93537e-05, -1.97914e-05,0.005;2.61165e-07,7.93537e-05, -1.97914e-05,0.005;0.0001,0.0300045, -0.00748335,0.005;0.0002,0.05926, -0.0147799,0.005;0.0003,0.0877841, -0.021894,0.00499999;0.0004,0.115593, -0.036608,0.00499997;0.0005,0.142707, -0.0568957,0.00499994;0.0006,0.169143, -0.076676,0.00499988;0.0007,0.194915, -0.0959614,0.0049998;0.0008,0.220042, -0.124763,0.00499968;0.0009,0.244539, -0.155317,0.00499951;0.001,0.26842, -0.185107,0.00499928;0.0011,0.291701, -0.214153,0.00499898;0.0012,0.314394, -0.249655,0.0049986;0.0013,0.336514, -0.288306,0.00499812;0.0014,0.358074, -0.325991,0.00499754;0.0015,0.379086, -0.362735,0.00499682;0.0016,0.399562, -0.398563,0.00499597;0.0017,0.419514, -0.44324,0.00499496;0.0018,0.438955, -0.487015,0.00499378;0.0019,0.457893, -0.529698,0.00499242;0.002,0.47634, -0.571317,0.00499085;0.0021,0.494305, -0.611901,0.00498906;0.0022,0.511799, -0.657374,0.00498704;0.0023,0.528832, -0.704491,0.00498476;0.0024,0.545412, -0.750434,0.00498221;0.0025,0.561548, -0.795237,0.00497937;0.0026,0.577248, -0.83893,0.00497623;0.0027,0.592521, -0.881543,0.00497277;0.0028,0.607375, -0.926803,0.00496896;0.0029,0.62182, -0.974598,0.0049648;0.003,0.63586, -1.02121,0.00496027;0.0031,0.649503, -1.06667,0.00495534;0.0032,0.662756, -1.11102,0.00495;0.0033,0.675625, -1.15428,0.00494424;0.0034,0.688119, -1.19648,0.00493803;0.0035,0.700242, -1.23778,0.00493136;0.0036,0.712005, -1.28391,0.00492421;0.0037,0.72341, -1.32891,0.00491657;0.0038,0.734463, -1.3728,0.00490842;0.0039,0.74517, -1.41563,0.00489974;0.004,0.755536, -1.45743,0.00489052;0.0041,0.765568, -1.49822,0.00488074;0.0042,0.775269, -1.53803,0.00487038;0.0043,0.784646, -1.57689,0.00485943;0.0044,0.793704, -1.61483,0.00484787;0.0045,0.80245, -1.65314,0.00483569;0.0046,0.810888, -1.69366,0.00482288;0.0047,0.81902, -1.7332,0.00480941;0.0048,0.826851, -1.77179,0.00479528;0.0049,0.834387, -1.80945,0.00478046;0.005,0.841631, -1.84622,0.00476495;0.0051,0.84859, -1.88259,0.00474873;0.0052,0.855304, -1.92429,0.00473179;0.0053,0.861739, -1.96564,0.0047141;0.0054,0.8679, -2.00668,0.00469566;0.0055,0.873791, -2.04743,0.00467645;0.0056,0.879419, -2.08794,0.00465645;0.0057,0.884782, -2.1282,0.00463565;0.0058,0.889885, -2.16824,0.00461403;0.0059,0.894731, -2.20808,0.00459157;0.006,0.899322, -2.24774,0.00456827;0.0061,0.903661, -2.28927,0.0045441;0.0062,0.907752, -2.33091,0.00451905;0.0063,0.911603, -2.37014,0.0044931;0.0064,0.915232, -2.40274,0.00446624;0.0065,0.91862, -2.43469,0.00443846;0.0066,0.92177, -2.466,0.00440974;0.0067,0.924686, -2.49668,0.00438007;0.0068,0.927368, -2.52672,0.00434945;0.0069,0.929822, -2.55615,0.00431785;0.007,0.93205, -2.58498,0.00428527;0.0071,0.934052, -2.61318,0.00425169;0.0072,0.935241, -2.64973,0.00421711;0.0073,0.936164, -2.68643,0.00418151;0.0074,0.936854, -2.7228,0.00414488;0.0075,0.937309, -2.7588,0.0041072;0.0076,0.937532, -2.7944,0.00406845;0.0077,0.937522, -2.82958,0.00402864;0.0078,0.937411, -2.866,0.00398773;0.0079,0.937385, -2.90613,0.00394572;0.008,0.937133, -2.94589,0.0039026;0.0081,0.936656, -2.98525,0.00385834;0.0082,0.935953, -3.02414,0.00381293;0.0083,0.935024, -3.06251,0.00376636;0.0084,0.934308, -3.10824,0.00371862;0.0085,0.933608, -3.15783,0.00366967;0.0086,0.93269, -3.20708,0.00361952;0.0087,0.931553, -3.25592,0.00356812;0.0088,0.930194, -3.30427,0.00351548;0.0089,0.928473, -3.35247,0.00346157;0.009,0.926467, -3.40014,0.00340636;0.0091,0.924232, -3.44698,0.00334985;0.0092,0.921766, -3.49289,0.00329202;0.0093,0.918579, -3.53879,0.00323283;0.0094,0.913925, -3.5856,0.00317229;0.0095,0.909004, -3.63034,0.00311037;0.0096,0.903809, -3.67275,0.00304706;0.0097,0.89859, -3.72881,0.00298233;0.0098,0.893783, -3.82589,0.00291616;0.0099,0.888707, -3.92096,0.00284852;0.01,0.883343, -4.01357,0.00277938;0.0101,0.876979, -4.10734,0.00270869;0.0102,0.869783, -4.19987,0.00263642;0.0103,0.862246, -4.28752,0.00256254;0.0104,0.854574, -4.37627,0.00248701;0.0105,0.847614, -4.49154,0.00240979;0.0106,0.840302, -4.60102,0.00233085;0.0107,0.832625, -4.70399,0.00225014;0.0108,0.822938, -4.82647,0.00216761;0.0109,0.812813, -4.93752,0.00208323;0.011,0.802204, -5.04175,0.00199695;0.0111,0.78997, -5.30274,0.00190873;0.0112,0.777197, -5.54515,0.00181846;0.0113,0.763521, -5.78149,0.00172606;0.0114,0.748272, -6.039,0.00163144;0.0115,0.73235, -6.25778,0.0015345;0.0116,0.715211, -6.57852,0.00143514;0.0117,0.696998, -6.91971,0.00133326;0.0118,0.677065, -7.30735,0.00122872;0.0119,0.652791, -7.88085,0.00112136;0.012,0.62734, -8.29718,0.00101097;0.0121,0.597125, -9.13179,0.000897364;0.0122,0.564919, -9.82427,0.000780251;0.0123,0.527838, -11.1684,0.000659331;0.0124,0.487477, -12.1609,0.000534142;0.0125,0.436631, -14.9103,0.000404205;0.0126,0.379243, -16.2449,0.000268616;0.0126134,0.371242, -16.2777,0.00025;0.0126134,0.371242, -16.2777,0.00025;0.0126868,0.350822, -16.2554,0.000198624;0.0126868,0.350822, -16.2554,0.000198624;0.0127,0.351869, -16.3218,0.000199455;0.0128,0.37695, -17.0338,0.000241587;0.0128157,0.381787, -17.1198,0.00025;0.0128157,0.381787, -17.1198,0.00025;0.0129,0.406591, -17.48,0.000292352;0.013,0.433421, -17.8191,0.000336402;0.0131,0.457261, -17.8337,0.000373609;0.0132,0.477911, -17.6706,0.000403962;0.0133,0.495294, -17.4605,0.00042752;0.0134,0.509353, -17.3988,0.000444358;0.0135,0.520015, -17.4878,0.0004545;0.0136,0.527192, -17.7433,0.000457911;0.0136003,0.527207, -17.7443,0.000457911;0.0136003,0.527207, -17.7443,0.000457911;0.0137,0.530748, -18.1997,0.000454491;0.0138,0.530517, -18.8646,0.000444064;0.0139,0.526294, -19.7142,0.000426376;0.014,0.517828, -20.6871,0.000401101;0.0141,0.504836, -21.6765,0.000367869;0.0142,0.487037, -22.6627,0.000326301;0.0143,0.464073, -23.4017,0.000276025;0.0143458,0.451744, -23.5657,0.00025;0.0143458,0.451744, -23.5657,0.00025;0.0144,0.439383, -23.6302,0.000223375;0.0144518,0.438001, -23.8106,0.00021654;0.0144518,0.438001, -23.8106,0.00021654;0.0145,0.442437, -24.0882,0.000220288;0.0146,0.459291, -24.7355,0.000241352;0.014643,0.466338, -24.9736,0.00025;0.014643,0.466338, -24.9736,0.00025;0.0147,0.47417, -25.2545,0.000258795;0.0148,0.483493, -25.7045,0.000266567;0.0148288,0.485111, -25.8323,0.00026698;0.0148288,0.485111, -25.8323,0.00026698;0.0149,0.486998, -26.1506,0.000264454;0.015,0.484444, -26.5924,0.000252282;0.0150127,0.483671, -26.6456,0.00025;0.0150127,0.483671, -26.6456,0.00025;0.0151,0.477935, -26.9803,0.000233764;0.0151954,0.478678, -27.3825,0.000227777;0.0151954,0.478678, -27.3825,0.000227777;0.0152,0.478896, -27.404,0.000227786;0.0153,0.486112, -27.9096,0.000231723;0.0154,0.494618, -28.4114,0.000237745;0.0154716,0.499054, -28.7526,0.000239402;0.0154716,0.499054, -28.7526,0.000239402;0.0155,0.500242, -28.8872,0.000239151;0.0156,0.502893, -29.3755,0.000235871;0.0157,0.505639, -29.8643,0.000232816;0.0158,0.509736, -30.3772,0.000231912;0.0158118,0.51029, -30.4396,0.000231905;0.0158118,0.51029, -30.4396,0.000231905;0.0159,0.514622, -30.9065,0.000232198;0.016,0.519654, -31.4343,0.000232755;0.016048,0.521947, -31.6846,0.000232849;0.016048,0.521947, -31.6846,0.000232849;0.0161,0.524291, -31.9527,0.000232753;0.0162,0.528618, -32.4638,0.000232328;0.0163,0.53296, -32.9726,0.000231976;0.0164,0.537374, -33.4793,0.000231787;0.0165,0.541801, -33.9827,0.000231672;0.0166,0.546199, -34.4828,0.000231561;0.0167,0.550555, -34.9795,0.000231435;0.0168,0.554875, -35.4729,0.0002313;0.0169,0.559164, -35.9631,0.000231166;0.017,0.56344, -36.4518,0.000231035;0.0171,0.567726, -36.9417,0.000230906;0.0172,0.571982, -37.4284,0.000230779;0.0173,0.576209, -37.9119,0.000230653;0.0174,0.580407, -38.3923,0.000230528;0.0175,0.584575, -38.8695,0.000230405;0.0176,0.588716, -39.3436,0.000230284;0.0177,0.593137, -39.8493,0.000230163;0.0178,0.59757, -40.357,0.000230038;0.0179,0.601967, -40.8716,0.000229911;0.018,0.60633, -41.3953,0.000229783;0.0181,0.610659, -41.9153,0.000229654;0.0182,0.614955, -42.4317,0.000229526;0.0183,0.619218, -42.9441,0.0002294;0.0184,0.623441, -43.452,0.000229276;0.0185,0.627634, -43.9562,0.000229154;0.0186,0.631795, -44.4569,0.000229034;0.0187,0.635926, -44.954,0.000228915;0.0188,0.640026, -45.4476,0.000228797;0.0189,0.644096, -45.9377,0.000228681;0.019,0.648136, -46.4242,0.000228566;0.0191,0.652146, -46.9074,0.000228453;0.0192,0.656126, -47.387,0.000228341;0.0193,0.660077, -47.8633,0.000228231;0.0194,0.663999, -48.3362,0.000228122;0.0195,0.667892, -48.8057,0.000228014;0.0196,0.671756, -49.2718,0.000227908;0.0197,0.675592, -49.7347,0.000227802;0.0198,0.67979, -50.2404,0.000227697;0.0199,0.684118, -50.7623,0.000227586;0.02,0.688404, -51.2799,0.000227471;0.0201,0.692654, -51.7933,0.000227355;0.0202,0.696868, -52.3025,0.000227241;0.0203,0.701047, -52.8002,0.00022713;0.0204,0.705193, -53.2717,0.000227022;0.0205,0.709307, -53.7394,0.000226918;0.0206,0.713479, -54.2135,0.000226817;0.0207,0.717635, -54.686,0.000226716;0.0208,0.721755, -55.1544,0.000226615;0.0209,0.725839, -55.619,0.000226515;0.021,0.729888, -56.0796,0.000226416;0.0211,0.733903, -56.5364,0.000226319;0.0212,0.737883, -56.9893,0.000226222;0.0213,0.741829, -57.4383,0.000226127;0.0214,0.745732, -57.8827,0.000226033;0.0215,0.749587, -58.3217,0.000225941;0.0216,0.75341, -58.7569,0.00022585;0.0217,0.757199, -59.1885,0.00022576;0.0218,0.760956, -59.6164,0.000225671;0.0219,0.764681, -60.0407,0.000225583;0.022,0.768373, -60.4614,0.000225497;0.0221,0.772034, -60.8786,0.000225411;0.0222,0.775663, -61.2922,0.000225326;0.0223,0.779579, -61.7378,0.000225242;0.0224,0.784355, -62.2802,0.000225151;0.0225,0.789065, -62.8168,0.000225046;0.0226,0.793716, -63.3474,0.000224938;0.0227,0.798315, -63.8721,0.000224831;0.0228,0.802863, -64.3256,0.000224728;0.0229,0.80737, -64.7356,0.000224637;0.023,0.811833, -65.1406,0.000224555;0.0231,0.816247, -65.541,0.000224477;0.0232,0.820611, -65.9369,0.000224399;0.0233,0.824909, -66.3269,0.000224322;0.0234,0.829106, -66.7079,0.000224246;0.0235,0.833258, -67.0845,0.000224172;0.0236,0.837362, -67.457,0.000224099;0.0237,0.84142, -67.8252,0.000224027;0.0238,0.845433, -68.1893,0.000223957;0.0239,0.8494, -68.5494,0.000223887;0.024,0.853323, -68.9053,0.000223818;0.0241,0.857201, -69.2573,0.00022375;0.0242,0.861036, -69.6053,0.000223683;0.0243,0.864828, -69.9494,0.000223617;0.0244,0.868577, -70.2896,0.000223552;0.0245,0.873541, -70.7381,0.000223484;0.0246,0.878506, -71.1879,0.000223404;0.0247,0.883389, -71.6312,0.00022332;0.0248,0.888198, -72.0678,0.000223236;0.0249,0.892935, -72.4978,0.000223154;0.025,0.8976, -72.9212,0.000223074;0.0251,0.902194, -73.2832,0.000222997;0.0252,0.906729, -73.5797,0.00022293;0.0253,0.911203, -73.8711,0.000222873;0.0254,0.915611, -74.1579,0.00022282;0.0255,0.919953, -74.4404,0.000222768;0.0256,0.924227, -74.7185,0.000222716;0.0257,0.928436, -74.9923,0.000222665;0.0258,0.932872, -75.2805,0.000222615;0.0259,0.937419, -75.5759,0.000222563;0.026,0.941886, -75.8664,0.000222509;0.0261,0.946276, -76.1519,0.000222456;0.0262,0.950592, -76.4326,0.000222404;0.0263,0.954834, -76.7084,0.000222354;0.0264,0.959005, -76.9795,0.000222304;0.0265,0.963104, -77.246,0.000222255;0.0266,0.967134, -77.5079,0.000222207;0.0267,0.971094, -77.7654,0.00022216;0.0268,0.974988, -78.0184,0.000222114;0.0269,0.978815, -78.2671,0.000222068;0.027,0.982577, -78.5115,0.000222024;0.0271,0.986275, -78.7518,0.00022198;0.0272,0.98991, -78.9879,0.000221937;0.0273,0.993484, -79.2201,0.000221895;0.0274,0.996996, -79.4482,0.000221854;0.0275,1.00082, -79.6845,0.000221813;0.0276,1.00486, -79.8903,0.000221773;0.0277,1.00883, -80.0919,0.000221735;0.0278,1.01272, -80.2892,0.000221699;0.0279,1.01653, -80.4824,0.000221665;0.028,1.02026, -80.6717,0.000221631;0.0281,1.02392, -80.8572,0.000221597;0.0282,1.0275, -81.0389,0.000221565;0.0283,1.03101, -81.2168,0.000221533;0.0284,1.03445, -81.3911,0.000221501;0.0285,1.03781, -81.5619,0.000221471;0.0286,1.04111, -81.7292,0.000221441;0.0287,1.04434, -81.893,0.000221412;0.0288,1.04751, -82.0535,0.000221383;0.0289,1.05061, -82.2107,0.000221355;0.029,1.05365, -82.3647,0.000221328;0.0291,1.05663, -82.5155,0.000221301;0.0292,1.05954, -82.6633,0.000221275;0.0293,1.0624, -82.808,0.000221249;0.0294,1.0652, -82.9498,0.000221224;0.0295,1.06794, -83.0887,0.000221199;0.0296,1.07063, -83.2248,0.000221175;0.0297,1.07326, -83.3581,0.000221151;0.0298,1.07584, -83.4886,0.000221128;0.0299,1.07836, -83.6165,0.000221106;0.03,1.08088, -83.7439,0.000221083;0.0301,1.08376, -83.8895,0.000221061;0.0302,1.08657, -84.0316,0.000221037;0.0303,1.08931, -84.1703,0.000221012;0.0304,1.09198, -84.3057,0.000220988;0.0305,1.09459, -84.4378,0.000220965;0.0306,1.09714, -84.5667,0.000220942;0.0307,1.09962, -84.6924,0.00022092;0.0308,1.10205, -84.7987,0.000220899;0.0309,1.10442, -84.8994,0.00022088;0.031,1.10673, -84.9975,0.000220862;0.0311,1.10898, -85.0932,0.000220846;0.0312,1.11119, -85.1866,0.000220829;0.0313,1.11333, -85.2778,0.000220813;0.0314,1.11543, -85.3668,0.000220798;0.0315,1.11748, -85.4536,0.000220782;0.0316,1.11947, -85.5382,0.000220768;0.0317,1.12142, -85.6209,0.000220753;0.0318,1.12332, -85.7015,0.000220739;0.0319,1.12518, -85.7802,0.000220725;0.032,1.12699, -85.857,0.000220712;0.0321,1.12875, -85.9319,0.000220699;0.0322,1.13048, -86.005,0.000220686;0.0323,1.13216, -86.0763,0.000220673;0.0324,1.1338, -86.1459,0.000220661;0.0325,1.1354, -86.2138,0.000220649;0.0326,1.13696, -86.28,0.000220638;0.0327,1.13849, -86.3447,0.000220627;0.0328,1.13997, -86.4078,0.000220616;0.0329,1.14143, -86.4693,0.000220605;0.033,1.14284, -86.5294,0.000220594;0.0331,1.14423, -86.588,0.000220584;0.0332,1.14558, -86.6452,0.000220574;0.0333,1.14689, -86.701,0.000220564;0.0334,1.14818, -86.7555,0.000220555;0.0335,1.14943, -86.8086,0.000220546;0.0336,1.15065, -86.8605,0.000220537;0.0337,1.15185, -86.9111,0.000220528;0.0338,1.15301, -86.9605,0.000220519;0.0339,1.15415, -87.0086,0.000220511;0.034,1.15526, -87.0556,0.000220503;0.0341,1.15634, -87.1015,0.000220495;0.0342,1.1574, -87.1463,0.000220487;0.0343,1.15843, -87.19,0.000220479;0.0344,1.15943, -87.2326,0.000220472;0.0345,1.16041, -87.2742,0.000220465;0.0346,1.16137, -87.3148,0.000220458;0.0347,1.16231, -87.3544,0.000220451;0.0348,1.16322, -87.3931,0.000220444;0.0349,1.16411, -87.4308,0.000220438;0.035,1.16498, -87.4676,0.000220431;0.0351,1.16582, -87.5035,0.000220425;0.0352,1.16665, -87.5385,0.000220419;0.0353,1.16746, -87.5727,0.000220413;0.0354,1.16824, -87.6061,0.000220407;0.0355,1.16901, -87.6386,0.000220402;0.0356,1.16976, -87.6704,0.000220396;0.0357,1.17049, -87.7014,0.000220391;0.0358,1.17121, -87.7316,0.000220386;0.0359,1.1719, -87.7612,0.00022038;0.036,1.17258, -87.79,0.000220375;0.0361,1.17325, -87.8181,0.000220371;0.0362,1.1739, -87.8455,0.000220366;0.0363,1.17453, -87.8722,0.000220361;0.0364,1.17514, -87.8984,0.000220357;0.0365,1.17574, -87.9238,0.000220352;0.0366,1.17633, -87.9487,0.000220348;0.0367,1.1769, -87.9729,0.000220344;0.0368,1.17746, -87.9966,0.00022034;0.0369,1.17801, -88.0197,0.000220336;0.037,1.17858, -88.0441,0.000220332;0.0371,1.17922, -88.0712,0.000220328;0.0372,1.17985, -88.0975,0.000220323;0.0373,1.18045, -88.123,0.000220319;0.0374,1.18103, -88.1477,0.000220314;0.0375,1.1816, -88.1717,0.00022031;0.0376,1.18215, -88.195,0.000220306;0.0377,1.18268, -88.2176,0.000220302;0.0378,1.1832, -88.2395,0.000220299;0.0379,1.1837, -88.2607,0.000220295;0.038,1.18419, -88.2814,0.000220291;0.0381,1.18466, -88.3014,0.000220288;0.0382,1.18512, -88.3208,0.000220284;0.0383,1.18556, -88.3396,0.000220281;0.0384,1.18599, -88.3578,0.000220278;0.0385,1.18641, -88.3756,0.000220275;0.0386,1.18682, -88.3928,0.000220272;0.0387,1.18721, -88.4094,0.000220269;0.0388,1.18759, -88.4256,0.000220266;0.0389,1.18796, -88.4413,0.000220264;0.039,1.18832, -88.4565,0.000220261;0.0391,1.18867, -88.4713,0.000220258;0.0392,1.18901, -88.4856,0.000220256;0.0393,1.18934, -88.4995,0.000220253;0.0394,1.18965, -88.513,0.000220251;0.0395,1.18996, -88.5261,0.000220249;0.0396,1.19026, -88.5388,0.000220247;0.0397,1.19055, -88.5511,0.000220245;0.0398,1.19084, -88.563,0.000220242;0.0399,1.19111, -88.5746,0.00022024;0.04,1.19137, -88.5859,0.000220239;0.0401,1.19163, -88.5968,0.000220237;0.0402,1.19188, -88.6074,0.000220235;0.0403,1.19212, -88.6176,0.000220233;0.0404,1.19236, -88.6276,0.000220231;0.0405,1.19259, -88.6373,0.00022023;0.0406,1.19281, -88.6466,0.000220228;0.0407,1.19302, -88.6557,0.000220226;0.0408,1.19323, -88.6646,0.000220225;0.0409,1.19343, -88.6731,0.000220223;0.041,1.19363, -88.6814,0.000220222;0.0411,1.19382, -88.6895,0.000220221;0.0412,1.19401, -88.6973,0.000220219;0.0413,1.19418, -88.7049,0.000220218;0.0414,1.19436, -88.7122,0.000220217;0.0415,1.19453, -88.7194,0.000220215;0.0416,1.19469, -88.7263,0.000220214;0.0417,1.19485, -88.733,0.000220213;0.0418,1.195, -88.7395,0.000220212;0.0419,1.19515, -88.7459,0.000220211;0.042,1.1953, -88.752,0.00022021;0.0421,1.19544, -88.7579,0.000220209;0.0422,1.19557, -88.7637,0.000220208;0.0423,1.19571, -88.7693,0.000220207;0.0424,1.19583, -88.7747,0.000220206;0.0425,1.19596, -88.78,0.000220205;0.0426,1.19608, -88.7851,0.000220204;0.0427,1.1962, -88.7901,0.000220203;0.0428,1.19631, -88.7949,0.000220202;0.0429,1.19642, -88.7996,0.000220202;0.043,1.19653, -88.8041,0.000220201;0.0431,1.19663, -88.8085,0.0002202;0.0432,1.19673, -88.8127,0.000220199;0.0433,1.19683, -88.8169,0.000220199;0.0434,1.19692, -88.8209,0.000220198;0.0435,1.19702, -88.8248,0.000220197;0.0436,1.1971, -88.8286,0.000220197;0.0437,1.19719, -88.8322,0.000220196;0.0438,1.19728, -88.8358,0.000220195;0.0439,1.19736, -88.8392,0.000220195;0.044,1.19744, -88.8426,0.000220194;0.0441,1.19751, -88.8458,0.000220194;0.0442,1.19759, -88.8489,0.000220193;0.0443,1.19766, -88.852,0.000220192;0.0444,1.19773, -88.855,0.000220192;0.0445,1.1978, -88.8578,0.000220191;0.0446,1.19786, -88.8606,0.000220191;0.0447,1.19793, -88.8633,0.000220191;0.0448,1.19799, -88.8659,0.00022019;0.0449,1.19805, -88.8685,0.00022019;0.045,1.19811, -88.871,0.000220189;0.0451,1.19816, -88.8734,0.000220189;0.0452,1.19822, -88.8757,0.000220188;0.0453,1.19827, -88.8779,0.000220188;0.0454,1.19832, -88.8801,0.000220188;0.0455,1.19837, -88.8822,0.000220187;0.0456,1.19842, -88.8843,0.000220187;0.0457,1.19847, -88.8863,0.000220187;0.0458,1.19851, -88.8882,0.000220186;0.0459,1.19856, -88.8901,0.000220186;0.046,1.1986, -88.8919,0.000220186;0.0461,1.19864, -88.8937,0.000220185;0.0462,1.19868, -88.8954,0.000220185;0.0463,1.19872, -88.897,0.000220185;0.0464,1.19876, -88.8987,0.000220184;0.0465,1.1988, -88.9002,0.000220184;0.0466,1.19883, -88.9017,0.000220184;0.0467,1.19887, -88.9032,0.000220184;0.0468,1.1989, -88.9046,0.000220183;0.0469,1.19893, -88.906,0.000220183;0.047,1.19897, -88.9074,0.000220183;0.0471,1.199, -88.9087,0.000220183;0.0472,1.19903, -88.91,0.000220182;0.0473,1.19906, -88.9112,0.000220182;0.0474,1.19908, -88.9124,0.000220182;0.0475,1.19911, -88.9135,0.000220182;0.0476,1.19914, -88.9146,0.000220182;0.0477,1.19916, -88.9157,0.000220181;0.0478,1.19919, -88.9168,0.000220181;0.0479,1.19921, -88.9178,0.000220181;0.048,1.19924, -88.9188,0.000220181;0.0481,1.19926, -88.9198,0.000220181;0.0482,1.19928, -88.9207,0.000220181;0.0483,1.1993, -88.9216,0.00022018;0.0484,1.19932, -88.9225,0.00022018;0.0485,1.19934, -88.9233,0.00022018;0.0486,1.19936, -88.9242,0.00022018;0.0487,1.19938, -88.925,0.00022018;0.0488,1.1994, -88.9258,0.00022018;0.0489,1.19942, -88.9265,0.00022018;0.049,1.19944, -88.9273,0.000220179;0.0491,1.19945, -88.928,0.000220179;0.0492,1.19947, -88.9287,0.000220179;0.0493,1.19948, -88.9293,0.000220179;0.0494,1.1995, -88.93,0.000220179;0.0495,1.19951, -88.9306,0.000220179;0.0496,1.19953, -88.9312,0.000220179;0.0497,1.19954, -88.9318,0.000220179;0.0498,1.19956, -88.9324,0.000220179;0.0499,1.19957, -88.933,0.000220178;0.05,1.19958, -88.9335,0.000220178;0.05,1.19958, -88.9335,0.000220178], tableOnFile = false, columns = 2:4, extrapolation = Modelica.Blocks.Types.Extrapolation.HoldLastPoint) "Valid for u_source=12VDC and m_load=0.01kg only; column 2: current, col.3: force, col.4: position" annotation(Placement(transformation(extent = {{ -40,60},{ -20,80}}, rotation = 0)));

          equation 
            connect(advancedGround.p,advancedSource.n) annotation(Line(points = {{ -60,10},{ -60,20}}, color = {0,0,255}));
            connect(advancedSource.p,advancedSolenoid.p) annotation(Line(points = {{ -60,40},{ -40,40},{ -40,36},{ -20,36}}, color = {0,0,255}));
            connect(advancedSolenoid.n,advancedSource.n) annotation(Line(points = {{ -20,24},{ -40,24},{ -40,20},{ -60,20}}, color = {0,0,255}));
            connect(advancedSolenoid.flange,advancedLoad.flange_a) annotation(Line(points = {{0,30},{20,30}}, color = {0,127,0}));
            connect(simpleGround.p,simpleSource.n) annotation(Line(points = {{ -60, -70},{ -60, -60}}, color = {0,0,255}));
            connect(simpleSource.p,simpleSolenoid.p) annotation(Line(points = {{ -60, -40},{ -40, -40},{ -40, -44},{ -20, -44}}, color = {0,0,255}));
            connect(simpleSolenoid.n,simpleSource.n) annotation(Line(points = {{ -20, -56},{ -40, -56},{ -40, -60},{ -60, -60}}, color = {0,0,255}));
            connect(simpleSolenoid.flange,simpleLoad.flange_a) annotation(Line(points = {{0, -50},{20, -50}}, color = {0,127,0}));
          end ComparisonPullInStroke;
          annotation(Documentation(info = "<html>
<p>
In electromagnetic or reluctance actuators, a thrust or reluctance force is generated due to a non-zero gradient of the relative magnetic permeability mu_r at surfaces between regions of different permeability (non-saturated ferromagnetic material: mu_r>>1, adjacent air: mu_r=1). In lumped magnetic network models, this force can be calculated as shortly outlined in <a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide.ReluctanceForceCalculation\">Reluctance Forces</a> of the User's Guide.
</p>

<p>
As an example of a reluctance actuator, a simple axisymmetric lifting magnet with planar end planes of armature and pole is shown. Often, a <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.Components.SimpleSolenoid\">SimpleSolenoid</a> model is sufficient for initial rough design of such an actuator's magnetic subsystem. Higher accuracy can be gained from an <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.Components.AdvancedSolenoid\">AdvancedSolenoid</a> model where the coil-imposed magnetomotive force is split and the leakage flux between armature and yoke is accounted for more precisely.
</p>

<p>
The differences between these two models in static behaviour can be analysed and compared to results obtained with a more accurate finite element analysis (FEA) in <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.ComparisonQuasiStationary\">ComparisonQuasiStationary</a>. The resulting differences in dynamic behaviour can be analysed and compared to FEA results with simulation of a pull-in stroke in <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.ComparisonPullInStroke\">ComparisonPullInStroke</a>.
</p>
</html>"), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -100},{80,50}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -100,50},{ -80,70},{100,70},{80,50},{ -100,50}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{100,70},{100, -80},{80, -100},{80,50},{100,70}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -120,130},{120,70}}, lineColor = {0,0,255}, textString = "%name"),Text(extent = {{ -100,30},{80, -80}}, lineColor = {0,0,255}, textString = "3")}));
          package Components "Components to be used in examples"
            model SimpleSolenoid "Simple network model of a lifting magnet with planar armature end face"
              parameter SI.Resistance R = 10 "Armature coil resistance";
              parameter Real N = 957 "Number of turns";
              parameter SI.Radius r_yokeOut = 0.015 "Outer yoke radius";
              parameter SI.Radius r_yokeIn = 0.0135 "Inner yoke radius";
              parameter SI.Length l_yoke = 0.035 "Axial yoke length";
              parameter SI.Length t_yokeBot = 0.0035 "Axial thickness of yoke bottom";
              parameter SI.Length l_pole = 0.0065 "Axial length of pole";
              parameter SI.Length t_poleBot = 0.0035 "Axial thickness of bottom at pole side";
              parameter SI.Length t_airPar = 0.00065 "Radial thickness of parasitic air gap due to slide guiding";
              parameter Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData material = Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.Steel.Steel_9SMnPb28() "Ferromagnetic material characteristics" annotation(choicesAllMatching = true, Dialog(group = "Material"));
              parameter SI.Radius r_arm = 0.005 "Armature radius = pole radius" annotation(Dialog(group = "Armature and stopper"));
              parameter SI.Length l_arm = 0.026 "Armature length" annotation(Dialog(group = "Armature and stopper"));
              parameter Modelica.SIunits.TranslationalSpringConstant c = 100000000000.0 "Spring stiffness between impact partners" annotation(Dialog(group = "Armature and stopper"));
              parameter Modelica.SIunits.TranslationalDampingConstant d = 400 "Damping coefficient between impact partners" annotation(Dialog(group = "Armature and stopper"));
              parameter SI.Position x_min = 0.00025 "Stopper at minimum armature position" annotation(Dialog(group = "Armature and stopper"));
              parameter SI.Position x_max = 0.005 "Stopper at maximum armature position" annotation(Dialog(group = "Armature and stopper"));
              SI.Position x(start = x_max, stateSelect = StateSelect.prefer) "Armature position, alias for flange position (identical with length of working air gap)";
            protected 
              parameter SI.Density rho_steel = 7853 "Density for calculation of armature mass from geometry";
              annotation(__Dymola_Images(Parameters(source = "../Images/Magnetic/FluxTubes/Examples/SolenoidActuator/Solenoid_dimensions.png")), Window(x = 0.16, y = 0.15, width = 0.6, height = 0.6), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -90,100},{90, -100}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{90, -30},{ -4,30}}, lineColor = {255,128,0}, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -40, -30},{ -90,30}}, lineColor = {255,128,0}, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -80, -100},{ -90,100}}, lineColor = {255,128,0}, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{90,90},{ -90,100}}, lineColor = {255,128,0}, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{90, -100},{ -90, -90}}, lineColor = {255,128,0}, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{90,40},{80,100}}, lineColor = {255,128,0}, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{90, -100},{80, -40}}, lineColor = {255,128,0}, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -70,80},{70,40}}, lineColor = {255,213,170}, fillColor = {255,213,170}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -70, -40},{70, -80}}, lineColor = {255,213,170}, fillColor = {255,213,170}, fillPattern = FillPattern.Solid),Text(extent = {{0,160},{0,120}}, lineColor = {0,0,255}, textString = "%name")}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
Please refer to the <b>Parameters</b> section for a schematic drawing of this axisymmetric lifting magnet.
In the half-section below, the flux tube elements of the actuator's magnetic circuit are superimposed on a field plot obtained with FEA. The magnetomotive force imposed by the coil is modelled as one lumped element. As a result, the radial leakage flux between armature and yoke that occurs especially at large working air gaps can not be considered properly. This leads to a a higher total reluctance and lower inductance respectively compared to FEA for large working air gaps (i.e. armature close to x_max). Please have a look at the comments associated with the individual model components for a short explanation of their purpose in the model.
</p>

<p><IMG SRC=\"../Images/Magnetic/FluxTubes/Examples/SolenoidActuator/SimpleSolenoidModel_fluxTubePartitioning.png\" ALT=\"Field lines and assigned flux tubes of the simple solenoid model\"></p>

<p>
The coupling coefficient c_coupl in the coil is set to 1 in this example, since leakage flux is accounted for explicitly with the flux tube element G_mLeakWork. Although this leakage model is rather simple, it describes the reluctance force due to the leakage field sufficiently, especially at large air gaps. With decreasing air gap length, the influence of the leakage flux on the actuator's net reluctance force decreases due to the increasing influence of the main working air gap G_mAirWork.
</p>

<p>
During model-based actuator design, the radii and lengths of the flux tube elements (and hence their cross-sectional areas and flux densities) should be assigned with parametric equations so that common design rules are met (e.g. allowed flux density in ferromagnetic parts, allowed current density and required cross-sectional area of winding). For simplicity, those equations are omitted in the example. Instead, the found values are assigned to the model elements directly.
</p>
</html>"));
            public 
              Modelica.Magnetic.FluxTubes.Basic.Ground ground annotation(Placement(transformation(extent = {{50,10},{70,30}}, rotation = 0)));
              Modelica.Magnetic.FluxTubes.Basic.ElectroMagneticConverter coil(final N = N, i(fixed = true)) "Electro-magnetic converter" annotation(Placement(transformation(origin = {0,20}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
              Modelica.Electrical.Analog.Basic.Resistor r(final R = R) "Coil resistance" annotation(Placement(transformation(extent = {{ -70, -30},{ -50, -10}}, rotation = 0)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderAxialFlux g_mFeYokeSide(final nonLinearPermeability = true, final material = material, final l = l_yoke - (t_poleBot + t_yokeBot) / 2, final r_i = r_yokeIn, final r_o = r_yokeOut) "Permeance of of hollow cylindric section of ferromagnetic yoke" annotation(Placement(transformation(extent = {{ -10,80},{10,100}}, rotation = 0)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderAxialFlux g_mFeArm(final nonLinearPermeability = true, final material = material, final l = l_yoke - (t_yokeBot + t_poleBot) / 2 - l_pole - (x_max + x_min) / 2, final r_i = 0, final r_o = r_arm) "Permeance of ferfomagnetic armature" annotation(Placement(transformation(origin = {16,40}, extent = {{ -10, -10},{10,10}}, rotation = 180)));
              Modelica.Magnetic.FluxTubes.Shapes.Force.HollowCylinderAxialFlux g_mAirWork(final mu_r = 1, final dlBydx = 1, final r_i = 0, final r_o = r_arm, final useSupport = false, final l = flange.s) "Permeance of working air gap (between armature and pole end faces)" annotation(Placement(transformation(origin = { -30,30}, extent = {{ -10,10},{10, -10}}, rotation = 180)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderRadialFlux g_mFeYokeBot(final nonLinearPermeability = true, final material = material, final l = t_yokeBot, final r_i = r_arm + t_airPar, final r_o = r_yokeIn) "Permeance of bottom side of ferromagnetic yoke" annotation(Placement(transformation(origin = {80,80}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderRadialFlux g_mAirPar(final nonLinearPermeability = false, final mu_rConst = 1, final l = t_yokeBot, final r_i = r_arm, final r_o = r_arm + t_airPar) "Permeance of parasitic radial air gap due to slide guiding" annotation(Placement(transformation(origin = {80,50}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderRadialFlux g_mFePoleBot(final l = t_poleBot, final r_i = r_arm, final r_o = r_yokeIn, final nonLinearPermeability = true, final material = material) "Permeance of bottom side of pole" annotation(Placement(transformation(origin = { -72,80}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderAxialFlux g_mFePole(final nonLinearPermeability = true, final material = material, final l = l_pole, final r_i = 0, final r_o = r_arm) "Permeance of ferromagnetic pole" annotation(Placement(transformation(origin = { -72,40}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
              Modelica.Magnetic.FluxTubes.Examples.Utilities.TranslatoryArmatureAndStopper armature(final m = rho_steel * l_arm * pi * r_arm ^ 2, final x_max = x_max, final x_min = x_min, final L = 0, final c = c, final d = d, n = 2, v(fixed = true)) "Inertia of armature and stoppers at end of stroke range" annotation(Placement(transformation(extent = {{64, -10},{84,10}}, rotation = 0)));
              Modelica.Magnetic.FluxTubes.Shapes.Leakage.QuarterCylinder g_mLeak1(l = 2 * pi * (r_arm + t_airPar / 2)) "Leakage pereance between inner edge of yoke bore and armature side face" annotation(Placement(transformation(origin = {60,50}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Shapes.Leakage.QuarterHollowCylinder g_mLeak2(final l = 2 * pi * r_arm, final ratio = 8) "Leakage permeance between inner side of yoke bottom and armature side (r_i = t_airPar)" annotation(Placement(transformation(origin = {40,50}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Shapes.Force.LeakageAroundPoles g_mLeakWork(final mu_r = 1, final dlBydx = 1, final w = 2 * pi * (r_arm + 0.0015), final r = 0.003, final l = flange.s, final useSupport = false) "Permeance of leakage air gap around working air gap (between armature and pole side faces)" annotation(Placement(transformation(origin = { -30,70}, extent = {{ -10,10},{10, -10}}, rotation = 180)));
              Modelica.Electrical.Analog.Interfaces.PositivePin p "Electrical connector" annotation(Placement(transformation(extent = {{ -110,50},{ -90,70}}, rotation = 0)));
              Modelica.Electrical.Analog.Interfaces.NegativePin n "Electrical connector" annotation(Placement(transformation(extent = {{ -90, -70},{ -110, -50}}, rotation = 0)));
              Modelica.Mechanics.Translational.Interfaces.Flange_b flange "Flange of component" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));

            equation 
              x = flange.s;
              connect(r.p,p) annotation(Line(points = {{ -70, -20},{ -90, -20},{ -90,60},{ -100,60}}, color = {0,0,255}));
              connect(armature.flange_b,flange) annotation(Line(points = {{84,0},{88,0},{88,1.16573e-15},{92,1.16573e-15},{92,0},{100,0}}, color = {0,127,0}));
              connect(armature.flange_a,g_mAirWork.flange) annotation(Line(points = {{64,0},{34,0},{34,52},{ -30,52},{ -30,40}}, color = {0,127,0}));
              connect(g_mAirWork.flange,g_mLeakWork.flange) annotation(Line(points = {{ -30,40},{ -30,52},{34,52},{34,80},{ -30,80}}, color = {0,127,0}));
              connect(r.n,coil.p) annotation(Line(points = {{ -50, -20},{ -6, -20},{ -6,10}}, color = {0,0,255}));
              connect(coil.n,n) annotation(Line(points = {{6,10},{6, -60},{ -100, -60}}, color = {0,0,255}));
              connect(coil.port_p,g_mAirWork.port_p) annotation(Line(points = {{ -6,30},{ -20,30}}, color = {255,127,0}));
              connect(g_mAirWork.port_p,g_mLeakWork.port_p) annotation(Line(points = {{ -20,30},{ -20,70}}, color = {255,127,0}));
              connect(g_mAirWork.port_n,g_mLeakWork.port_n) annotation(Line(points = {{ -40,30},{ -40,70}}, color = {255,127,0}));
              connect(g_mFePole.port_p,g_mAirWork.port_n) annotation(Line(points = {{ -72,30},{ -40,30}}, color = {255,127,0}));
              connect(g_mFePoleBot.port_p,g_mFePole.port_n) annotation(Line(points = {{ -72,70},{ -72,50}}, color = {255,127,0}));
              connect(g_mFePoleBot.port_n,g_mFeYokeSide.port_p) annotation(Line(points = {{ -72,90},{ -10,90}}, color = {255,127,0}));
              connect(g_mFeYokeSide.port_n,g_mFeYokeBot.port_n) annotation(Line(points = {{10,90},{80,90}}, color = {255,127,0}));
              connect(g_mFeYokeBot.port_p,g_mAirPar.port_n) annotation(Line(points = {{80,70},{80,60}}, color = {255,127,0}));
              connect(g_mFeArm.port_p,g_mLeak2.port_p) annotation(Line(points = {{26,40},{40,40}}, color = {255,127,0}));
              connect(g_mLeak2.port_p,g_mLeak1.port_p) annotation(Line(points = {{40,40},{60,40}}, color = {255,127,0}));
              connect(g_mLeak1.port_p,g_mAirPar.port_p) annotation(Line(points = {{60,40},{80,40}}, color = {255,127,0}));
              connect(g_mLeak2.port_n,g_mLeak1.port_n) annotation(Line(points = {{40,60},{60,60}}, color = {255,127,0}));
              connect(g_mLeak1.port_n,g_mAirPar.port_n) annotation(Line(points = {{60,60},{80,60}}, color = {255,127,0}));
              connect(g_mFeArm.port_n,coil.port_n) annotation(Line(points = {{6,40},{6,35},{6,30},{6,30}}, color = {255,127,0}));
              connect(ground.port,g_mLeak1.port_p) annotation(Line(points = {{60,30},{60,40}}, color = {255,127,0}, smooth = Smooth.None));
            end SimpleSolenoid;
            model AdvancedSolenoid "Advanced network model of a lifting magnet with planar armature end face, split magnetomotive force"
              parameter Real N = 957 "Number of turns";
              parameter SI.Resistance R = 5 "Coil resistance";
              parameter SI.Resistance R_par = 100000.0 "Resistance parallel to the coil, in series to C_par";
              parameter SI.Capacitance C_par = 1e-09 "Capacitance parallel to the coil, in series to R_par";
              parameter SI.Radius r_yokeOut = 0.015 "Outer yoke radius";
              parameter SI.Radius r_yokeIn = 0.0135 "Inner yoke radius";
              parameter SI.Length l_yoke = 0.035 "Axial yoke length";
              parameter SI.Length t_yokeBot = 0.0035 "Axial thickness of yoke bottom";
              parameter SI.Length l_pole = 0.0065 "Axial length of pole";
              parameter SI.Length t_poleBot = 0.0035 "Axial thickness of bottom at pole side";
              parameter SI.Length t_airPar = 0.00065 "Radial thickness of parasitic air gap due to slide guiding";
              parameter Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData material = Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.Steel.Steel_9SMnPb28() "Ferromagnetic material characteristics" annotation(choicesAllMatching = true, Dialog(group = "Material"));
              parameter SI.Radius r_arm = 0.005 "Armature radius = pole radius" annotation(Dialog(group = "Armature and stopper"));
              parameter SI.Length l_arm = 0.026 "Armature length" annotation(Dialog(group = "Armature and stopper"));
              parameter Modelica.SIunits.TranslationalSpringConstant c = 100000000000.0 "Spring stiffness between impact partners" annotation(Dialog(group = "Armature and stopper"));
              parameter Modelica.SIunits.TranslationalDampingConstant d = 400 "Damping coefficient between impact partners" annotation(Dialog(group = "Armature and stopper"));
              parameter SI.Position x_min = 0.00025 "Stopper at minimum armature position" annotation(Dialog(group = "Armature and stopper"));
              parameter SI.Position x_max = 0.005 "Stopper at maximum armature position" annotation(Dialog(group = "Armature and stopper"));
              SI.Position x(start = x_max, stateSelect = StateSelect.prefer) "Armature position";
              SI.MagneticFlux Psi_tot "Total flux linkage for information only";
              SI.Inductance L_statTot "Total static inductance for information only";
            protected 
              parameter SI.Density rho_steel = 7853 "Density for calculation of armature mass from geometry";
              annotation(__Dymola_Images(Parameters(source = "../Images/Magnetic/FluxTubes/Examples/SolenoidActuator/Solenoid_dimensions.png")), Window(x = 0.16, y = 0.15, width = 0.6, height = 0.6), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -90,100},{90, -100}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{90, -30},{ -4,30}}, lineColor = {255,128,0}, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -40, -30},{ -90,30}}, lineColor = {255,128,0}, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -80, -100},{ -90,100}}, lineColor = {255,128,0}, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{90,90},{ -90,100}}, lineColor = {255,128,0}, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{90, -100},{ -90, -90}}, lineColor = {255,128,0}, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{90,40},{80,100}}, lineColor = {255,128,0}, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{90, -100},{80, -40}}, lineColor = {255,128,0}, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -70,80},{70,40}}, lineColor = {255,213,170}, fillColor = {255,213,170}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -70, -40},{70, -80}}, lineColor = {255,213,170}, fillColor = {255,213,170}, fillPattern = FillPattern.Solid),Line(points = {{4,30},{4,32},{2,38},{ -4,48},{ -14,60},{ -22,72},{ -24,80},{ -24,90}}, color = {255,128,0}),Line(points = {{22,30},{22,32},{20,38},{14,48},{4,60},{ -4,72},{ -6,80},{ -6,90}}, color = {255,128,0}),Line(points = {{40,30},{40,32},{38,38},{32,48},{22,60},{14,72},{12,80},{12,90}}, color = {255,128,0}),Text(extent = {{0,160},{0,120}}, lineColor = {0,0,255}, textString = "%name")}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
Please have a look at <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.Components.SimpleSolenoid\">SimpleSolenoid</a> for a general description of this actuator. Unlike in that simple magnetic network model, the coil is split into two lumped elements here. This enables for more realistic modelling of the radial leakage flux between armature and yoke (leakage permeance G_mLeakRad). Especially for large air gaps, the influence of this leakage flux on the actuator's inductance and its electromagnetic force is rather strong. Please have a look at <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Examples.SolenoidActuator.ComparisonQuasiStationary\">ComparisonQuasiStationary</a> for a comparison of both models with FEA-based results included as reference.
</p>
<p><IMG SRC=\"../Images/Magnetic/FluxTubes/Examples/SolenoidActuator/AdvancedSolenoidModel_fluxTubePartitioning.png\" ALT=\"Assigned flux tubes and field plot of the solenoid actuator\"></p>

<p>
The parasitic capacitances c_par1 and c_par2 accross both partial coils assure that the voltages across these coils are well-defined during simulation.
</p>
</html>"));
            public 
              Modelica.Magnetic.FluxTubes.Basic.Ground ground annotation(Placement(transformation(extent = {{42,2},{62,22}}, rotation = 0)));
              Modelica.Magnetic.FluxTubes.Basic.ElectroMagneticConverter coil1(final N = N / 2, i(fixed = true)) "Electro-magnetic conversion in first half of coil" annotation(Placement(transformation(origin = { -46,20}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
              Modelica.Electrical.Analog.Basic.Resistor r_1(final R = R) "Resistance of first half of coil" annotation(Placement(transformation(extent = {{ -84, -30},{ -64, -10}}, rotation = 0)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderAxialFlux g_mFeYokeSide1(final l = l_yoke / 2 - t_poleBot / 2, final r_i = r_yokeIn, final r_o = r_yokeOut, final nonLinearPermeability = true, final material = material) "Permeance of of first half of yoke's hollow cylindric section" annotation(Placement(transformation(extent = {{ -50,70},{ -30,90}}, rotation = 0)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderAxialFlux g_mFeArm(final r_i = 0, final l = l_yoke - (t_yokeBot + t_poleBot) / 2 - l_pole - (x_max + x_min) / 2, final r_o = r_arm, final nonLinearPermeability = true, final material = material) "Permeance of ferfomagnetic armature" annotation(Placement(transformation(origin = {10,30}, extent = {{ -10,10},{10, -10}}, rotation = 180)));
              Modelica.Magnetic.FluxTubes.Shapes.Force.HollowCylinderAxialFlux g_mAirWork(final r_o = r_arm, final useSupport = false, final mu_r = 1, final dlBydx = 1, final r_i = 0, final l = flange.s) "Permeance of working air gap (between armature and pole end faces)" annotation(Placement(transformation(origin = { -20,30}, extent = {{ -10,10},{10, -10}}, rotation = 180)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderRadialFlux g_mFeYokeBot(final l = t_yokeBot, final r_i = r_arm + t_airPar, final r_o = r_yokeIn, final nonLinearPermeability = true, final material = material) "Permeance of bottom side of ferromagnetic yoke" annotation(Placement(transformation(origin = {74,70}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderRadialFlux g_mAirPar(final l = t_yokeBot, final r_i = r_arm, final r_o = r_arm + t_airPar, final nonLinearPermeability = false, final mu_rConst = 1) "Permeance of parasitic radial air gap due to slide guiding" annotation(Placement(transformation(origin = {74,40}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderRadialFlux g_mFePoleBot(final l = t_poleBot, final r_i = r_arm, final r_o = r_yokeIn, final nonLinearPermeability = true, final material = material) "Permeance of bottom side of pole" annotation(Placement(transformation(origin = { -78,56}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderAxialFlux g_mFePole(final l = l_pole, final r_o = r_arm, final nonLinearPermeability = true, final material = material) "Permeance of ferromagnetic pole" annotation(Placement(transformation(origin = { -68,30}, extent = {{ -10,10},{10, -10}}, rotation = 180)));
              Modelica.Magnetic.FluxTubes.Examples.Utilities.TranslatoryArmatureAndStopper armature(final x_max = x_max, final x_min = x_min, final m = rho_steel * l_arm * pi * r_arm ^ 2, final L = 0, final c = c, final d = d, n = 2, v(fixed = true)) "Inertia of armature and stoppers at end of stroke range" annotation(Placement(transformation(extent = {{64, -10},{84,10}}, rotation = 0)));
              Modelica.Magnetic.FluxTubes.Shapes.Leakage.QuarterCylinder g_mLeak1(final l = 2 * pi * (r_arm + t_airPar / 2)) "Leakage pereance between inner edge of yoke bore and armature side face" annotation(Placement(transformation(origin = {60,40}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Shapes.Leakage.QuarterHollowCylinder g_mLeak2(final ratio = 8, final l = 2 * pi * r_arm) "Leakage permeance between inner side of yoke bottom and armature side (r_i = t_airPar)" annotation(Placement(transformation(origin = {46,40}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Basic.ElectroMagneticConverter coil2(final N = N / 2, i(fixed = true)) "Electro-magnetic conversion in first half of coil" annotation(Placement(transformation(origin = {30,20}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
              Modelica.Electrical.Analog.Basic.Capacitor c_par1(final C = C_par, v(start = 0, fixed = true)) "Parasitic capacitance assigned to first half of coil" annotation(Placement(transformation(extent = {{ -60, -50},{ -40, -30}}, rotation = 0)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderRadialFlux G_mLeakRad(final mu_rConst = 1, final r_i = r_arm, final r_o = r_yokeIn, final l = l_yoke / 4, final nonLinearPermeability = false) "Permeance of radial leakage flux tube between armature side and yoke side" annotation(Placement(transformation(origin = {0,56}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Shapes.FixedShape.HollowCylinderAxialFlux g_mFeYokeSide2(final l = l_yoke / 2 - t_yokeBot / 2, final r_i = r_yokeIn, r_o = r_yokeOut, final nonLinearPermeability = true, final material = material) "Permeance of of second half of yoke's hollow cylindric section" annotation(Placement(transformation(extent = {{20,70},{40,90}}, rotation = 0)));
              Modelica.Electrical.Analog.Basic.Capacitor c_par2(final C = C_par, v(start = 0, fixed = true)) "Parasitic capacitance assigned to second half of coil" annotation(Placement(transformation(extent = {{16, -50},{36, -30}}, rotation = 0)));
              Modelica.Electrical.Analog.Basic.Resistor r_par1(final R = R_par) "Parasitic resistance assigned to first half of coil" annotation(Placement(transformation(extent = {{ -84, -50},{ -64, -30}}, rotation = 0)));
              Modelica.Electrical.Analog.Basic.Resistor r_par2(final R = R_par) "Parasitic resistance assigned to second half of coil" annotation(Placement(transformation(extent = {{ -8, -50},{12, -30}}, rotation = 0)));
              Modelica.Electrical.Analog.Basic.Resistor r_2(final R = R) "Resistance of second half of coil" annotation(Placement(transformation(extent = {{ -8, -30},{12, -10}}, rotation = 0)));
              Modelica.Magnetic.FluxTubes.Shapes.Leakage.QuarterCylinder g_mLeak3(final l = 2 * pi * (r_arm + t_airPar / 2)) "Leakage pereance between outer edge of yoke bore and armature side face" annotation(Placement(transformation(origin = {88,40}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
              Modelica.Magnetic.FluxTubes.Shapes.Force.LeakageAroundPoles g_mLeakWork(final w = 2 * pi * (r_arm + 0.0015), final r = 0.003, final useSupport = false, final mu_r = 1, final dlBydx = 1, final l = flange.s) "Permeance of leakage air gap around working air gap (between armature and pole side faces)" annotation(Placement(transformation(origin = { -20,64}, extent = {{ -10,10},{10, -10}}, rotation = 180)));
              Modelica.Electrical.Analog.Interfaces.PositivePin p "Electrical connector" annotation(Placement(transformation(extent = {{ -110,50},{ -90,70}}, rotation = 0)));
              Modelica.Electrical.Analog.Interfaces.NegativePin n "Electrical connector" annotation(Placement(transformation(extent = {{ -90, -70},{ -110, -50}}, rotation = 0)));
              Modelica.Mechanics.Translational.Interfaces.Flange_b flange "Flange of component" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));

            equation 
              x = flange.s;
              Psi_tot = coil1.Psi + coil2.Psi;
              L_statTot = coil1.L_stat + coil2.L_stat;
              connect(armature.flange_b,flange) annotation(Line(points = {{84,0},{88,0},{88,1.16573e-15},{92,1.16573e-15},{92,0},{100,0}}, color = {0,127,0}));
              connect(r_par1.n,c_par1.p) annotation(Line(points = {{ -64, -40},{ -60, -40}}, color = {0,0,255}));
              connect(r_par1.p,r_1.p) annotation(Line(points = {{ -84, -40},{ -84, -20}}, color = {0,0,255}));
              connect(c_par2.p,r_par2.n) annotation(Line(points = {{16, -40},{12, -40}}, color = {0,0,255}));
              connect(r_par2.p,r_2.p) annotation(Line(points = {{ -8, -40},{ -8, -20}}, color = {0,0,255}));
              connect(r_1.p,p) annotation(Line(points = {{ -84, -20},{ -92, -20},{ -92,60},{ -100,60}}, color = {0,0,255}));
              connect(g_mLeakWork.flange,g_mAirWork.flange) annotation(Line(points = {{ -20,74},{ -6,74},{ -6,40},{ -20,40}}, color = {0,127,0}));
              connect(g_mAirWork.flange,armature.flange_a) annotation(Line(points = {{ -20,40},{ -6,40},{ -6,0},{64,0}}, color = {0,127,0}));
              connect(n,c_par2.n) annotation(Line(points = {{ -100, -60},{36, -60},{36, -40}}, color = {0,0,255}));
              connect(coil2.port_p,g_mFeArm.port_p) annotation(Line(points = {{24,30},{20,30}}, color = {255,127,0}));
              connect(G_mLeakRad.port_p,g_mFeArm.port_n) annotation(Line(points = {{ -6.12323e-16,46},{0,46},{0,30}}, color = {255,127,0}));
              connect(g_mAirWork.port_p,g_mFeArm.port_n) annotation(Line(points = {{ -10,30},{0,30}}, color = {255,127,0}));
              connect(coil1.port_n,g_mAirWork.port_n) annotation(Line(points = {{ -40,30},{ -30,30}}, color = {255,127,0}));
              connect(g_mAirWork.port_n,g_mLeakWork.port_n) annotation(Line(points = {{ -30,30},{ -30,64}}, color = {255,127,0}));
              connect(g_mLeakWork.port_p,g_mAirWork.port_p) annotation(Line(points = {{ -10,64},{ -10,30}}, color = {255,127,0}));
              connect(coil1.port_p,g_mFePole.port_p) annotation(Line(points = {{ -52,30},{ -58,30}}, color = {255,127,0}));
              connect(g_mFePole.port_n,g_mFePoleBot.port_p) annotation(Line(points = {{ -78,30},{ -78,46}}, color = {255,127,0}));
              connect(g_mFePoleBot.port_n,g_mFeYokeSide1.port_p) annotation(Line(points = {{ -78,66},{ -78,80},{ -50,80}}, color = {255,127,0}));
              connect(g_mFeYokeSide1.port_n,G_mLeakRad.port_n) annotation(Line(points = {{ -30,80},{6.12323e-16,80},{6.12323e-16,66}}, color = {255,127,0}));
              connect(g_mFeYokeSide1.port_n,g_mFeYokeSide2.port_p) annotation(Line(points = {{ -30,80},{20,80}}, color = {255,127,0}));
              connect(g_mFeYokeSide2.port_n,g_mFeYokeBot.port_n) annotation(Line(points = {{40,80},{74,80}}, color = {255,127,0}));
              connect(coil2.port_n,g_mLeak2.port_p) annotation(Line(points = {{36,30},{46,30}}, color = {255,127,0}));
              connect(g_mLeak2.port_p,g_mLeak1.port_p) annotation(Line(points = {{46,30},{60,30}}, color = {255,127,0}));
              connect(g_mLeak1.port_p,g_mAirPar.port_p) annotation(Line(points = {{60,30},{74,30}}, color = {255,127,0}));
              connect(g_mAirPar.port_p,g_mLeak3.port_p) annotation(Line(points = {{74,30},{88,30}}, color = {255,127,0}));
              connect(g_mLeak2.port_n,g_mLeak1.port_n) annotation(Line(points = {{46,50},{60,50}}, color = {255,127,0}));
              connect(g_mLeak1.port_n,g_mAirPar.port_n) annotation(Line(points = {{60,50},{74,50}}, color = {255,127,0}));
              connect(g_mAirPar.port_n,g_mLeak3.port_n) annotation(Line(points = {{74,50},{88,50}}, color = {255,127,0}));
              connect(g_mFeYokeBot.port_p,g_mAirPar.port_n) annotation(Line(points = {{74,60},{74,50}}, color = {255,127,0}));
              connect(coil2.p,r_2.n) annotation(Line(points = {{24,10},{24, -20},{12, -20}}, color = {0,0,255}));
              connect(coil2.n,c_par2.n) annotation(Line(points = {{36,10},{36, -40}}, color = {0,0,255}));
              connect(coil1.n,c_par1.n) annotation(Line(points = {{ -40,10},{ -40, -40}}, color = {0,0,255}));
              connect(coil1.n,r_2.p) annotation(Line(points = {{ -40,10},{ -40, -20},{ -8, -20}}, color = {0,0,255}));
              connect(r_1.n,coil1.p) annotation(Line(points = {{ -64, -20},{ -52, -20},{ -52,10}}, color = {0,0,255}));
              connect(ground.port,g_mLeak2.port_p) annotation(Line(points = {{52,22},{52,30},{46,30}}, color = {255,127,0}, smooth = Smooth.None));
            end AdvancedSolenoid;
          end Components;
        end SolenoidActuator;
        package Utilities "Utilities to be used in examples"
          model TranslatoryArmatureAndStopper "Mass with free travel between two stoppers"
            parameter SI.Length L(start = 0) "Length of component from left flange to right flange (= flange_b.s - flange_a.s)";
            parameter SI.Mass m(start = 1) "Armature mass";
            parameter Modelica.SIunits.TranslationalSpringConstant c(start = 100000000000.0) "Spring stiffness between impact partners";
            parameter Modelica.SIunits.TranslationalDampingConstant d(start = 20000000.0) "Damping coefficient between impact partners";
            parameter Real n(final min = 1) = 2 "Exponent of spring forces (f_c = c*|s_rel|^n)" annotation(Evaluate = true);
            parameter SI.Position x_max(start = 0.01) "Position of stopper at maximum armature position";
            parameter SI.Position x_min(start = 0) "Position of stopper at minimum armature position";
            Modelica.SIunits.Position s(start = 0) "Absolute position of center of component (= flange_a.s + L/2)";
            Modelica.SIunits.Velocity v(start = 0) "Absolute velocity of components (= der(s))";
            Modelica.SIunits.Acceleration a(start = 0) "Absolute acceleration of components (= der(v))";
            Modelica.Mechanics.Translational.Components.Mass mass(final L = L, final m = m) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
            Modelica.Mechanics.Translational.Interfaces.Flange_a flange_a annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
            Modelica.Mechanics.Translational.Interfaces.Flange_b flange_b annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
            Modelica.Mechanics.Translational.Components.Fixed limit_xMin(s0 = x_min) annotation(Placement(transformation(extent = {{ -80, -50},{ -60, -30}}, rotation = 0)));
            Modelica.Mechanics.Translational.Components.Fixed limit_xMax(s0 = x_max) annotation(Placement(transformation(extent = {{60, -50},{80, -30}}, rotation = 0)));
            Modelica.Mechanics.Translational.Components.ElastoGap stopper_xMax(final c = c, final d = d, final n = n, final s_rel0 = 0) annotation(Placement(transformation(extent = {{50, -30},{70, -10}}, rotation = 0)));
            Modelica.Mechanics.Translational.Components.ElastoGap stopper_xMin(final c = c, final d = d, final n = n, final s_rel0 = 0) annotation(Placement(transformation(extent = {{ -70, -30},{ -50, -10}}, rotation = 0)));
            annotation(defaultComponentName = "armature", Window(x = 0.21, y = 0.35, width = 0.53, height = 0.73), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -80, -60},{ -80, -80}}, color = {160,160,164}, thickness = 0.5),Line(points = {{ -80, -76},{ -40, -76}}, color = {160,160,164}, thickness = 0.5, arrow = {Arrow.None,Arrow.Filled}),Text(extent = {{ -84, -82},{ -76, -92}}, lineColor = {160,160,164}, pattern = LinePattern.Dash, lineThickness = 0.5, textString = "0"),Text(extent = {{ -46, -82},{ -38, -92}}, lineColor = {160,160,164}, pattern = LinePattern.Dash, lineThickness = 0.5, textString = "x"),Ellipse(extent = {{ -82, -78},{ -78, -74}}, lineColor = {175,175,175}, fillColor = {175,175,175}, fillPattern = FillPattern.Solid)}), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -90,10},{90, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255,127,0}),Text(extent = {{ -120,140},{120,100}}, lineColor = {0,0,255}, textString = "%name"),Rectangle(extent = {{ -50,60},{50, -60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255,127,0}),Rectangle(extent = {{ -80, -20},{ -88, -80}}, lineColor = {255,128,0}, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{88, -20},{80, -80}}, lineColor = {255,128,0}, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -80,80},{ -88,20}}, lineColor = {255,128,0}, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{88,80},{80,20}}, lineColor = {255,128,0}, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -100, -100},{100, -140}}, lineColor = {0,0,0}, textString = "m=%m"),Line(points = {{ -50, -80},{30, -80}}, color = {0,0,0}),Polygon(points = {{60, -80},{30, -70},{30, -90},{60, -80}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>
In translatory actuators with limited stroke, the armature with its inertia can travel between two stoppers.
</p>
</html>"));

          equation 
            mass.s = s;
            mass.v = v;
            mass.a = a;
            connect(mass.flange_a,stopper_xMin.flange_b) annotation(Line(points = {{ -10,0},{ -40,0},{ -40, -20},{ -50, -20}}, color = {0,127,0}));
            connect(limit_xMax.flange,stopper_xMax.flange_b) annotation(Line(points = {{70, -40},{70, -20}}, color = {0,191,0}));
            connect(stopper_xMax.flange_a,mass.flange_b) annotation(Line(points = {{50, -20},{40, -20},{40,0},{10,0}}, color = {0,127,0}));
            connect(mass.flange_a,flange_a) annotation(Line(points = {{ -10,0},{ -32.5,0},{ -32.5,1.16573e-15},{ -55,1.16573e-15},{ -55,0},{ -100,0}}, color = {0,127,0}));
            connect(limit_xMin.flange,stopper_xMin.flange_a) annotation(Line(points = {{ -70, -40},{ -70, -20}}, color = {0,127,0}));
            connect(flange_b,mass.flange_b) annotation(Line(points = {{100,0},{10,0}}, color = {0,127,0}));
          end TranslatoryArmatureAndStopper;
        public 
          record CoilDesign "Calculation of winding parameters (wire diameter, number of turns et al.) and recalculation with optionally chosen parameters; to be adapted to particular design tasks"
            extends Modelica.Icons.Record;
            parameter SI.Resistivity rho_20 = 1.78e-08 "Resistivity of conductor material at 20�C (default: Copper)";
            parameter Modelica.SIunits.LinearTemperatureCoefficient alpha_20 = 0.0039 "Temperature coefficient of conductor material's resistivity at 20degC (default: Copper)";
            parameter SI.Temperature T_op = 293.15 "Operating temperature of winding";
            final parameter SI.Resistivity rho = rho_20 * (1 + alpha_20 * (T_op - 20 + 273.15)) "Resistivity at operating temperature";
            parameter SI.Length h_w "Height of winding cross-section";
            parameter SI.Length b_w "Width of winding cross-section";
            final parameter SI.Area A_w = h_w * b_w "Cross-section area of winding";
            parameter SI.Length l_avg "Average length of one turn";
            parameter SI.Voltage V_op "Operating voltage (nominal/ minimum/ maximum voltage depending on design objective)";
            parameter SI.CurrentDensity J_desired = 4000000.0 "DESIRED current density at operating temperature and voltage resp.";
            parameter Real c_condFillChosen = 0.6 "CHOSEN conductor filling factor = total conductor area without insulation/ total winding area";
            final parameter Real N_calculated = V_op / (rho * l_avg * J_desired) "CALCULATED number of turns";
            final parameter SI.Diameter d_wireCalculated = sqrt((4 * A_w * c_condFillChosen) / (pi * N_calculated)) "CALCULATED wire diameter (without insulation)";
            final parameter SI.Area A_wireCalculated = (pi * d_wireCalculated ^ 2) / 4 "Calculated wire cross-section area";
            final parameter SI.Resistance R_calculated = (rho * N_calculated * l_avg) / A_wireCalculated "Winding resistance at operating temperature and voltage resp. with CALCULATED number of turns and wire diameter";
            final parameter SI.Power P_calculated = V_op ^ 2 / R_calculated "Winding's ohmic losses at operating temperature and voltage resp. with CALCULATED number of turns and wire diameter";
            parameter SI.Diameter d_wireChosen = d_wireCalculated "CHOSEN available wire diameter (without insulation)" annotation(Dialog(group = "Chosen feasible parameters (optional)"));
            parameter Real N_chosen = N_calculated "CHOSEN number of turns" annotation(Dialog(group = "Chosen feasible parameters (optional)"));
            final parameter SI.Area A_wireChosen = (pi * d_wireChosen ^ 2) / 4 "Wire cross-section area resulting from CHOSEN wire diameter";
            final parameter SI.Resistance R_actual = (rho * N_chosen * l_avg) / A_wireChosen "Winding resistance at operating temperature and voltage resp. resulting from CHOSEN number of turns and wire diameter";
            final parameter SI.Power P_actual = V_op ^ 2 / R_actual "Winding's ohmic losses at operating temperature and voltage resp. resulting from CHOSEN number of turns and wire diameter";
            final parameter SI.CurrentDensity J_actual = (V_op * 4) / (R_actual * pi * d_wireChosen ^ 2) "Current density at operating temperature and voltage resp. resulting from CHOSEN number of turns and wire diameter";
            final parameter Real c_condFillActual = (N_chosen * pi * d_wireChosen ^ 2) / (4 * A_w) "Conductor filling factor resulting from CHOSEN number of turns and wire diameter";
            annotation(Icon(graphics), Diagram(graphics), Documentation(info = "<html>
<p>
This model exemplarily shows dimensioning of a winding (wire diameter, number of turns) based on desired operating conditions (voltage, temperature, current density, conductor filling factor) for a given cross-section area of the winding. It can be modified according to the parameters given and sought after for a particular design project.
</p>

<p>
The calculated winding resistance and number of turns can be used as input parameters to the electrical subsystem
of a device to be modelled. Operating voltage V_op can be minimum, nominal and maximum voltage respectively as specified for a particular design project. In conjunction with the setting of the operating temperature T_op, this enables for analysis of the device under worst-case conditions (e.g. minimum required magnetomotive force, maximum allowed ohmic losses, minimum and maximum force respectively).
</p>

<p>
For manufacturing of a winding, the obtained wire diameter d_wireCalculated must be rounded to that of an available wire. In order to analyse the influence of this rounding, one can enter the chosen wire diameter d_wireChosen and number of turns N_chosen as optional input. Calculation of the resulting winding parameters enables for comparison with the ones obtained otherwise.
</p>
</html>"));
          end CoilDesign;
        end Utilities;
      end Examples;
      package Basic "Basic elements of magnetic network models"
        model Ground "Zero magnetic potential"
          Modelica.Magnetic.FluxTubes.Interfaces.PositiveMagneticPort port annotation(Placement(transformation(extent = {{ -10,110},{10,90}}, rotation =  -0)));
          annotation(Documentation(info = "<html>
<p>
The magnetic potential at the magnetic ground node is zero. Every magnetic network model must contain at least one magnetic ground object.
</p>
</html>
"), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -60,50},{60,50}}, color = {255,127,0}),Line(points = {{ -40,30},{40,30}}, color = {255,127,0}),Line(points = {{ -20,10},{20,10}}, color = {255,127,0}),Line(points = {{0,90},{0,50}}, color = {255,127,0}),Text(extent = {{ -150, -40},{150,0}}, lineColor = {0,0,255}, textString = "%name")}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -60,50},{60,50}}, color = {255,127,0}, thickness = 0.5),Line(points = {{ -40,30},{40,30}}, color = {255,127,0}, thickness = 0.5),Line(points = {{ -20,10},{20,10}}, color = {255,127,0}, thickness = 0.5),Line(points = {{0,100},{0,50}}, color = {255,127,0}, thickness = 0.5),Text(extent = {{ -40, -40},{40,20}}, lineColor = {0,0,255}, textString = "port.V_m = 0")}), Window(x = 0.23, y = 0.23, width = 0.59, height = 0.63));

        equation 
          port.V_m = 0;
        end Ground;
        model ElectroMagneticConverter "Electro-magnetic energy conversion"
          Modelica.Magnetic.FluxTubes.Interfaces.PositiveMagneticPort port_p "Positive magnetic port" annotation(Placement(transformation(extent = {{90,50},{110,70}}, rotation = 0)));
          Modelica.Magnetic.FluxTubes.Interfaces.NegativeMagneticPort port_n "Negative magnetic port" annotation(Placement(transformation(extent = {{110, -70},{90, -50}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.PositivePin p "Positive electric pin" annotation(Placement(transformation(extent = {{ -90,50},{ -110,70}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin n "Negative electric pin" annotation(Placement(transformation(extent = {{ -110, -70},{ -90, -50}}, rotation = 0)));
          SI.Voltage v "Voltage";
          SI.Current i(start = 0, stateSelect = StateSelect.prefer) "Current";
          SI.MagneticPotentialDifference V_m "Magnetic potential difference";
          SI.MagneticFlux Phi "Magnetic flux coupled into magnetic circuit";
          parameter Real N(start = 1) "Number of turns";
          SI.MagneticFlux Psi "Flux linkage for information only";
          SI.Inductance L_stat "Static inductance abs(Psi/i) for information only";
          annotation(defaultComponentName = "converter", Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Polygon(points = {{ -134,63},{ -124,60},{ -134,57},{ -134,63}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Line(points = {{ -150,60},{ -125,60}}, color = {160,160,164}),Polygon(points = {{141, -57},{151, -60},{141, -63},{141, -57}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Line(points = {{125, -60},{150, -60}}, color = {160,160,164}),Text(extent = {{128, -56},{144, -41}}, lineColor = {160,160,164}, textString = "Phi"),Text(extent = {{128,64},{145,79}}, lineColor = {0,0,0}, textString = "Phi"),Line(points = {{ -150, -59},{ -125, -59}}, color = {160,160,164}),Polygon(points = {{ -140, -56},{ -150, -59},{ -140, -62},{ -140, -56}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Text(extent = {{ -141, -56},{ -124, -41}}, lineColor = {160,160,164}, textString = "i"),Text(extent = {{ -150,63},{ -133,78}}, lineColor = {160,160,164}, textString = "i"),Line(points = {{124,61},{149,61}}, color = {160,160,164}),Polygon(points = {{134,64},{124,61},{134,58},{134,64}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid)}), Window(x = 0.26, y = 0.14, width = 0.58, height = 0.58), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -70,100},{70, -100}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -50,0},{ -30,20}}, lineColor = {0,0,255}),Line(points = {{ -40,60},{ -40,40}}, color = {0,0,255}),Ellipse(extent = {{ -50,20},{ -30,40}}, lineColor = {0,0,255}),Ellipse(extent = {{ -50, -20},{ -30,0}}, lineColor = {0,0,255}),Ellipse(extent = {{ -50, -40},{ -30, -20}}, lineColor = {0,0,255}),Line(points = {{ -40, -40},{ -40, -60}}, color = {0,0,255}),Rectangle(extent = {{ -54,40},{ -40, -40}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -40,60},{ -92,60}}, color = {0,0,255}),Line(points = {{ -40, -60},{ -90, -60}}, color = {0,0,255}),Line(points = {{0,100},{ -70,100}}, color = {0,0,255}, pattern = LinePattern.Dash),Line(points = {{ -70,100},{ -70, -100}}, color = {0,0,255}, pattern = LinePattern.Dash),Line(points = {{0, -100},{ -70, -100}}, color = {0,0,255}, pattern = LinePattern.Dash),Line(points = {{70,100},{0,100}}, color = {255,127,0}, pattern = LinePattern.Dash),Line(points = {{70, -100},{0, -100}}, color = {255,127,0}, pattern = LinePattern.Dash),Line(points = {{70,100},{70, -100}}, color = {255,127,0}, pattern = LinePattern.Dash),Ellipse(extent = {{ -4, -34},{64,34}}, lineColor = {255,127,0}),Line(points = {{30, -60},{30, -34}}, color = {255,127,0}),Line(points = {{18,0},{42,0}}, color = {255,127,0}),Line(points = {{42,10},{42, -12}}, color = {255,127,0}),Line(points = {{30,34},{30,60}}, color = {255,127,0}),Line(points = {{30,60},{100,60}}, color = {255,127,0}),Line(points = {{30, -60},{90, -60}}, color = {255,127,0}),Text(extent = {{ -150,150},{150,110}}, lineColor = {0,0,255}, textString = "%name"),Line(points = {{18,10},{18, -12}}, color = {255,127,0}),Line(points = {{ -90,30},{ -90, -30}}, color = {0,0,255}),Polygon(points = {{ -90, -30},{ -84, -10},{ -96, -10},{ -90, -30}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Line(points = {{90,30},{90, -30}}, color = {255,128,0}),Polygon(points = {{90, -30},{96, -10},{84, -10},{90, -30}}, lineColor = {255,128,0}, fillColor = {255,128,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>
The electro-magnetic energy conversion is given by <i>Ampere</i>'s law and <i>Faraday</i>'s law respectively:
</p>

<pre>
    V_m = i * N
    N * d&Phi;/dt = -v
</pre>

<p>
V_m is the magnetomotive force that is supplied to the connected magnetic circuit, &Phi; is the magnetic flux through the associated branch of this magnetic circuit. The negative sign of the induced voltage v is due to <i>Lenz</i>'s law.
</p>

<p>
The flux linkage &Psi; and the static inductance L_stat = |&Psi;/i| are calculated for information only. Note that L_stat is set to |&Psi;/eps| if |i| &lt; eps
(= 100*Modelica.Constants.eps).
</p>
</html>"));
        protected 
          Real eps = 100 * Modelica.Constants.eps;

        equation 
          v = p.v - n.v;
          0 = p.i + n.i;
          i = p.i;
          V_m = port_p.V_m - port_n.V_m;
          0 = port_p.Phi + port_n.Phi;
          Phi = port_p.Phi;
          V_m = i * N;
          N * der(Phi) =  -v;
          Psi = N * Phi;
          L_stat = noEvent(if abs(i) > eps then abs(Psi / i) else abs(Psi / eps));
        end ElectroMagneticConverter;
        model ConstantReluctance "Constant reluctance"
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialTwoPorts;
          parameter SI.Reluctance R_m(start = 1) "Magnetic reluctance";
          annotation(Window(x = 0.16, y = 0.15, width = 0.6, height = 0.6), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -70,30},{70, -30}}, lineColor = {255,128,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -70,0},{ -90,0}}, color = {255,128,0}),Line(points = {{70,0},{90,0}}, color = {255,128,0}),Text(extent = {{ -100, -100},{100, -62}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
This constant reluctance is provided for test purposes and simple magnetic network models. The reluctance is not calculated from geometry and permeability of a flux tube, but is provided as a parameter.
</p>
</html>"));

        equation 
          V_m = Phi * R_m;
        end ConstantReluctance;
        model LeakageWithCoefficient "Leakage reluctance with respect to the reluctance of a useful flux path (not for dynamic simulation of actuators)"
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialLeakage;
          parameter SI.CouplingCoefficient c_usefulFlux(start = 0.7) "Ratio useful flux/(leakage flux + useful flux) = useful flux/total flux";
          annotation(__Dymola_Images(Parameters(group = "Reference reluctance", source = "../Images/Magnetic/FluxTubes/Basic/LeakageWithCoefficient.png")), Documentation(info = "<html>
<p>
Differently from the flux tube elements of package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.Leakage\">Shapes.Leakage</a>
that are calculated from their geometry, this leakage reluctance is calculated with reference to the total reluctance of a useful flux path. Please refer to the <b>Parameters</b> section for an illustration of the resulting magnetic network. Exploiting <i>Kirchhoff</i>'s generalized current law, the leakage reluctance is calculated by means of a coupling coefficient c_usefulFlux.
</p>

<h4>Attention:</h4>

<p>
This element must <b>not</b> be used <b>for dynamic simulation of</b> electro-magneto-mechanical <b>actuators</b>, where the shape of at least one flux tube element with reluctance force generation in the useful flux path changes with armature motion (e.g. air gap). This change results in a non-zero derivative dG_m/dx of those elements permeance G_m with respect to armature position x, which in turn will lead to a non-zero derivative of the leakage permeance with respect to armature position. This would generate a reluctance force in the leakage element that is not accounted for properly. <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.Force.LeakageAroundPoles\">Shapes.Force.LeakageAroundPoles</a> provides a simple leakage reluctance with force generation.
</p>
</html>"), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
          input SI.Reluctance R_mUsefulTot "Total reluctance of useful flux path as reference" annotation(Dialog(group = "Reference reluctance"));

        equation 
          (1 - c_usefulFlux) * R_m = c_usefulFlux * R_mUsefulTot;
        end LeakageWithCoefficient;
        model EddyCurrent "For modelling of eddy current in a conductive magnetic flux tube"
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialTwoPorts;
          parameter SI.Resistivity rho(start = 9.8e-08) "Resistivity of flux tube material (default: Iron at 20degC)";
          parameter SI.Length l(start = 1) "Average length of eddy current path";
          parameter SI.Area A(start = 1) "Cross sectional area of eddy current path";
          final parameter SI.Resistance R = (rho * l) / A "Electrical resistance of eddy current path";
          annotation(Window(x = 0.16, y = 0.15, width = 0.6, height = 0.6), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -70,30},{70, -30}}, lineColor = {255,128,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -70,0},{ -90,0}}, color = {255,128,0}),Line(points = {{70,0},{90,0}}, color = {255,128,0}),Rectangle(extent = {{ -70,30},{70, -30}}, lineColor = {255,128,0}, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -100, -98},{100, -60}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
Eddy currents are induced in a conductive magnetic flux tube when the flux changes with time. This causes a magnetic voltage drop in addition to the voltage drop that is due to the reluctance of this flux tube. The eddy current component can be thought of as a short-circuited secundary winding of a transformer with only one turn. Its resistance is calculated from the geometry and resistivity of the eddy current path.
</p>

<p>
Partitioning of a solid conductive cylinder or prism into several hollow cylinders or separate nested prisms and modelling of each of these flux tubes connected in parallel with a series connection of a reluctance element and an eddy current component can model the delayed buildup of the magnetic field in the complete flux tube from the outer to the inner sections. Please refer to <a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ka08]</a> for an illustration.
</p>
</html>"));

        equation 
          V_m = 1 / R * der(Phi);
        end EddyCurrent;
      end Basic;
      package Shapes "Reluctance and permeance elements respectively based on geometric shapes"
        package FixedShape "Flux tubes with fixed shape during simulation and linear or non-linear material characteristics"
          model HollowCylinderAxialFlux "(Hollow) cylinder with axial flux; fixed shape; linear or non-linear material characteristics"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialFixedShape;
            parameter SI.Length l(start = 0.01) "Axial length (in direction of flux)" annotation(Dialog(group = "Fixed geometry"));
            parameter SI.Radius r_i(start = 0) "Inner radius of hollow cylinder (zero for cylinder)" annotation(Dialog(group = "Fixed geometry"));
            parameter SI.Radius r_o(start = 0.01) "Outer radius of (hollow) cylinder" annotation(Dialog(group = "Fixed geometry"));
            annotation(__Dymola_Images(Parameters(group = "Fixed geometry", source = "../Images/Magnetic/FluxTubes/Shapes/HollowCylinderAxialFlux.png")), Window(x = 0.16, y = 0.15, width = 0.6, height = 0.6), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.FixedShape\">FixedShape</a> for a description of all elements of this package and to <a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>

<p>
Set the inner radius r_i=0 for modelling of a solid cylindric flux tube.
</p>
</html>"));

          equation 
            A = pi * (r_o ^ 2 - r_i ^ 2);
            G_m = (mu_0 * mu_r * A) / l;
          end HollowCylinderAxialFlux;
          model HollowCylinderRadialFlux "Hollow cylinder with radial flux; fixed shape; linear or non-linear material characteristics"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialFixedShape;
            parameter SI.Length l(start = 0.01) "Width (orthogonal to flux direction)" annotation(Dialog(group = "Fixed geometry"));
            parameter SI.Radius r_i(start = 0.01) "Inner radius of hollow cylinder" annotation(Dialog(group = "Fixed geometry"));
            parameter SI.Radius r_o(start = 0.02) "Outer radius of hollow cylinder" annotation(Dialog(group = "Fixed geometry"));
            annotation(__Dymola_Images(Parameters(group = "Fixed geometry", source = "../Images/Magnetic/FluxTubes/Shapes/HollowCylinderRadialFlux.png")), Window(x = 0.16, y = 0.15, width = 0.6, height = 0.6), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.FixedShape\">FixedShape</a> for a description of all elements of this package and to <a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>

<p>
For hollow cylindric flux tubes with a radial magnetic flux, the flux density is a function of the radius. For that reason, the characteristic mu_r(B) is evaluated for the flux density at the flux tube's mean radius.
</p>

<p>
For those flux tube sections of a magnetic device that have a nonlinear material characteristic mu_r(B) and a large aspect ratio of outer to inner radius r_o/r_i, the section can be split up in a series connection of several hollow cylindric flux tubes with radial flux. This allows for more realistic modelling of the dependence of flux density on the radius compared to modelling with just one flux tube element.
</p>
</html>"));

          equation 
            A = l * pi * (r_o + r_i);
            G_m = (2 * pi * mu_0 * mu_r * l) / Modelica.Math.log(r_o / r_i);
          end HollowCylinderRadialFlux;
          annotation(Documentation(info = "<html>
<p>
Please have a look at <a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide.ReluctanceForceCalculation\">UsersGuide.ReluctanceForceCalculation</a> for an explanation of the different flux tube categories and resulting sub-packages.
</p>

<p>
Due to the restrictions on reluctance force calculation outlined there, flux tube elements with a possibly non-linear material characteristic mu_r(B) must have a fixed shape during simulation of converter motion. Hence, the dimensions of these flux tubes are defined as parameters in the model components that extend the base class <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Interfaces.PartialFixedShape\">Interfaces.PartialFixedShape</a>.  </p>

<p>
For initial design of magnetic circuits, the relative permeability of possibly non-linear flux tube elements can easily be set to a constant value mu_rConst (non-linearPermeability set to false). In some cases, this can simplify the rough geometric design of a device's magnetic circuit. Once an initial geometry is found, the magnetic subsystem can be simulated and fine-tuned with more realistic non-linear characteristics of ferromagnetic materials. Doing so requires setting of the parameter non-linearPermeability to true and selection of one of the soft magnetic materials of <a href=\"Material.SoftMagnetic\">Material.SoftMagnetic</a>.
</p>
</html>"));
          model Cuboid "Flux tube with rectangular cross-section; fixed shape; linear or non-linear material characteristics"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialFixedShape;
            parameter SI.Length l = 0.01 "Length in direction of flux" annotation(Dialog(group = "Fixed geometry"));
            parameter SI.Length a = 0.01 "Width of rectangular cross-section" annotation(Dialog(group = "Fixed geometry"));
            parameter SI.Length b = 0.01 "Height of rectangular cross-section" annotation(Dialog(group = "Fixed geometry"));
            annotation(__Dymola_Images(Parameters(group = "Fixed geometry", source = "../Images/Magnetic/FluxTubes/Shapes/CuboidParallelFlux.png")), Window(x = 0.16, y = 0.15, width = 0.6, height = 0.6), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.FixedShape\">FixedShape</a> for a description of all elements of this package and to <a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));

          equation 
            A = a * b;
            G_m = (mu_0 * mu_r * A) / l;
          end Cuboid;
        end FixedShape;
        package Force "Flux tubes with reluctance force generation; constant permeability"
          model HollowCylinderAxialFlux "(Hollow) cylinder with axial flux; constant permeability"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialForce;
            SI.Length l = s "Axial length (in direction of flux)" annotation(Dialog(group = "Variable geometry"));
            parameter SI.Radius r_i(start = 0) "Inner radius of (hollow) cylinder";
            parameter SI.Radius r_o(start = 0.01) "Outer radius of (hollow) cylinder";
            annotation(__Dymola_Images(Parameters(group = "Variable geometry", source = "../Images/Magnetic/FluxTubes/Shapes/HollowCylinderAxialFlux.png")), Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.Force\">Force</a> for a description of all elements of this package and to <a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
            SI.MagneticFluxDensity B "Homogeneous flux density";
          protected 
            parameter SI.Area A = pi * (r_o ^ 2 - r_i ^ 2) "Cross-sectional area orthogonal to direction of flux";

          equation 
            G_m = (mu_0 * mu_r * A) / l;
            dGmBydx =  -(1 * mu_0 * mu_r * A) / l ^ 2 * dlBydx;
            B = Phi / A;
          end HollowCylinderAxialFlux;
          annotation(Documentation(info = "<html>
<p>
Please have a look at <a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide.ReluctanceForceCalculation\">UsersGuide.ReluctanceForceCalculation</a> for an explanation of the different flux tube categories and resulting sub-packages.
</p>
Flux tube elements with generation of a reluctance force are intended for modelling of position-dependent air gap sections and permanent magnet sections respectively of translatory actuators. By default, the position co-ordinate of the mechanical connector flange.s is identical with the dimension l of the package's flux tube elements. l is the dimension changes with armature motion. If needed, the identity l=flange.s can be replaced by an actuator-specific equation, for example, when a flux tube length increases with decreasing armature position. The position co-ordinate of an element's translatory connector flange.s in turn will be identical with the armature position x in most cases, as the examples illustrate. </p>
<p>
The derivative of each element's permeance with respect to armature position dGmBydx is calculated from the derivative of the flux tube's permeance with respect to its varying dimension dGmBydl and the derivative of this dimension with respect to armature position dlBydx:</p>
<pre>
    dG_m   dG_m   dl
    ---- = ---- * --
     dx     dl    dx
</pre>
</p>
<p>
The parameter dlBydx must be set in each flux tube element to +1 or -1 according to the definition of the armature co-ordinate and the position of the element in a device's magnetic circuit. Proper match between armature motion and resulting variation of the flux tube length assures that the element's reluctance force acts in the right direction.
</p>
<p>
The shapes of the flux tubes defined in this package are rather simple. Only one dimenion varies with armature motion. Flux tubes with more complex variations of dimensions with armature motion can be defined by extending the base class <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Interfaces.PartialForce\">Interfaces.PartialForce</a>, if needed. Determination of the analytic derivative dGmBydl could become more complex for those flux tubes.
</p>
</html>"));
          model HollowCylinderRadialFlux "Hollow cylinder with radial flux; constant permeability"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialForce;
            SI.Length l = s "Axial length (orthogonal to direction of flux)" annotation(Dialog(group = "Variable geometry"));
            parameter SI.Radius r_i(start = 0.01) "Inner radius of hollow cylinder";
            parameter SI.Radius r_o(start = 0.015) "Outer radius of hollow cylinder";
            annotation(__Dymola_Images(Parameters(group = "Variable geometry", source = "../Images/Magnetic/FluxTubes/Shapes/HollowCylinderRadialFlux.png")), Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.Force\">Force</a> for a description of all elements of this package and to <a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));
            SI.MagneticFluxDensity B_avg "Average flux density (at arithmetic mean radius)";
          protected 
            SI.Area A_avg "Average cross-sectional area orthogonal to direction of flux (at arithmetic mean radius)";

          equation 
            G_m = (mu_0 * mu_r * 2 * pi * l) / Modelica.Math.log(r_o / r_i);
            dGmBydx = (mu_0 * mu_r * 2 * pi) / Modelica.Math.log(r_o / r_i) * dlBydx;
            A_avg = pi * (r_i + r_o) * l;
            B_avg = Phi / A_avg;
          end HollowCylinderRadialFlux;
          model CuboidParallelFlux "Cuboid with flux in direction of motion, e.g. air gap with rectangular cross-section; constant permeability"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialForce;
            SI.Length l = s "Axial length (in direction of flux)" annotation(Dialog(group = "Variable geometry"));
            parameter SI.Length a(start = 0.01) "Width of rectangular cross-section";
            parameter SI.Length b(start = 0.01) "Height of rectangular cross-section";
            annotation(__Dymola_Images(Parameters(group = "Variable geometry", source = "../Images/Magnetic/FluxTubes/Shapes/CuboidParallelFlux.png")), Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.Force\">Force</a> for a description of all elements of this package and to <a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));
            SI.MagneticFluxDensity B "Homogeneous flux density";
          protected 
            parameter SI.Area A = a * b "Cross-sectional area orthogonal to direction of flux";

          equation 
            G_m = (mu_0 * mu_r * A) / l;
            dGmBydx =  -(1 * mu_0 * mu_r * A) / l ^ 2 * dlBydx;
            B = Phi / A;
          end CuboidParallelFlux;
          model CuboidOrthogonalFlux "Cuboid with flux orthogonal to direction of motion; constant permeability"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialForce;
            SI.Length l = s "Length in direction of motion (orthogonal to flux)" annotation(Dialog(group = "Variable geometry"));
            parameter SI.Length a(start = 0.01) "Width of rectangular cross-section";
            parameter SI.Length b(start = 0.01) "Height of rectangular cross-section (in flux direction)";
            annotation(__Dymola_Images(Parameters(group = "Variable geometry", source = "../Images/Magnetic/FluxTubes/Shapes/CuboidOrthogonalFlux.png")), Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.Force\">Force</a> for a description of all elements of this package and to <a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));
            SI.MagneticFluxDensity B "Homogeneous flux density";
          protected 
            SI.Area A "Cross-sectional area orthogonal to direction of flux";

          equation 
            A = a * l;
            G_m = (mu_0 * mu_r * A) / b;
            dGmBydx = (mu_0 * mu_r * a) / b * dlBydx;
            B = Phi / A;
          end CuboidOrthogonalFlux;
          model LeakageAroundPoles "Leakage flux tube around cylindrical or prismatic poles"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialForce;
            SI.Length l = s "Axial length (in direction of flux)" annotation(Dialog(group = "Variable geometry"));
            annotation(__Dymola_Images(Parameters(group = "Variable geometry", source = "../Images/Magnetic/FluxTubes/Shapes/LeakageAroundPoles.png")), Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing sub-package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.Force\">Force</a> for a description of all elements of this package.
</p>

<p>
Leakage flux around a prismatic or cylindric air gap between to poles can be described with this model. Due to its constant radius of the leakage field r, the model is rather simple. Whereas in reality the leakage radius is approximately constant for air gap lengths l greater than this radius, it decreases with air gap lengths less than the leakage radius. This decrease for small air gaps is neglected here, since the influence of the leakage flux tube compared to that of the enclosed main air gap (connected in parallel) decreases for decreasing air gap length l.
</p>

<p>
Note that in <a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ka08]</a> the equation for G_m is accidentally swapped with that of a similar element.
</p>
</html>"));
            parameter SI.Length w(start = 0.1) "Width orthogonal to flux; mean circumference of flux tube in case of cylindrical poles";
            parameter SI.Radius r(start = 0.01) "Radius of leakage field";

          equation 
            G_m = (mu_0 * w) / pi * Modelica.Math.log(1 + (pi * r) / l);
            dGmBydx =  -(mu_0 * w * r * dlBydx) / (l ^ 2 * (1 + (pi * r) / l));
          end LeakageAroundPoles;
        end Force;
        package Leakage "Leakage flux tubes with position-independent permeance and hence no force generation; mu_r=1"
          annotation(Documentation(info = "<html>
<p>
Please have a look at <a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide.ReluctanceForceCalculation\">UsersGuide.ReluctanceForceCalculation</a> for an explanation of the different flux tube categories and resulting sub-packages.
</p>

<p>
The permeances of all elements of this package are calculated from their geometry. These flux tube elements are intended for modelling of leakage fields through vacuum, air and other media with a relative permeability mu_r=1. <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Basic.LeakageWithCoefficient\">Basic.LeakageWithCoefficient</a> accounts for leakage not by the geometry of flux tubes, but by a coupling coefficient c_usefulFlux.
</p>

<p>
All dimensions are defined as parameters. As a result, the shape of these elements will remain constant during dynamic simulation of actuators and reluctance forces will not be generated in these flux tube elements. A simple leakage flux tube with reluctance force generation is provided with the element <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.Force.LeakageAroundPoles\">Force.LeakageAroundPoles</a>. In cases where the accuracy of that element is not sufficient, the leakage elements of this package can be adapted and extended so that they are able to change their shape with armature motion and to generate reluctance forces. This requires an extension of the partial model <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Interfaces.PartialForce\">Interfaces.PartialForce</a>, a higher variability of the variables representing the flux tube's dimensions, definition of a relationship between armature position and these dimensions and determination of the analytic derivative dG_m/dx of the flux tube's permeance G_m with respect to armature position x.
</p>
</html>"));
          model QuarterCylinder "Leakage flux from one edge to the opposite plane through a quarter cylinder"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialLeakage;
            parameter SI.Length l(start = 0.1) "Axial length orthogonal to flux (=2*pi*r for cylindrical pole and r>>distance between edge and plane)";
            annotation(__Dymola_Images(Parameters(source = "../Images/Magnetic/FluxTubes/Shapes/Leakage/QuarterCylinder.png")), Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.Leakage\">Leakage</a> for a description of all elements of this package and to <a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));

          equation 
            G_m = mu_0 * 0.52 * l;
          end QuarterCylinder;
          model QuarterHollowCylinder "Leakage flux in circumferential direction through a quarter hollow cylinder"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialLeakage;
            parameter SI.Length l(start = 0.1) "Axial length orthogonal to flux (=2*pi*r for cylindrical pole and r>>r_i)";
            parameter Real ratio(start = 1) "Constant ratio t/r_i";
            annotation(__Dymola_Images(Parameters(source = "../Images/Magnetic/FluxTubes/Shapes/Leakage/QuarterHollowCylinder.png")), Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.Leakage\">Leakage</a> for a description of all elements of this package and to <a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));

          equation 
            G_m = (2 * mu_0 * l * Modelica.Math.log(1 + ratio)) / pi;
          end QuarterHollowCylinder;
          model HalfCylinder "Leakage flux through the edges of a half cylinder"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialLeakage;
            parameter SI.Length l(start = 0.1) "Axial length orthogonal to flux (=2*pi*r for cylindrical pole and r>>distance between edges)";
            annotation(__Dymola_Images(Parameters(source = "../Images/Magnetic/FluxTubes/Shapes/Leakage/HalfCylinder.png")), Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.Leakage\">Leakage</a> for a description of all elements of this package and to <a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));

          equation 
            G_m = mu_0 * 0.26 * l;
          end HalfCylinder;
          model HalfHollowCylinder "Leakage flux in circumferential direction through a half hollow cylinder"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialLeakage;
            parameter SI.Length l(start = 0.1) "Axial length orthogonal to flux (=2*pi*r for cylindrical pole and r>>r_i)";
            parameter Real ratio(start = 1) "Constant ratio t/r_i";
            annotation(__Dymola_Images(Parameters(source = "../Images/Magnetic/FluxTubes/Shapes/Leakage/HalfHollowCylinder.png")), Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.Leakage\">Leakage</a> for a description of all elements of this package and to <a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));

          equation 
            G_m = (mu_0 * l * Modelica.Math.log(1 + ratio)) / pi;
          end HalfHollowCylinder;
          model QuarterSphere "Leakage flux through the corners of a quarter sphere"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialLeakage;
            parameter SI.Radius r(start = 0.005) "Radius of quarter sphere";
            annotation(__Dymola_Images(Parameters(source = "../Images/Magnetic/FluxTubes/Shapes/Leakage/QuarterSphere.png")), Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.Leakage\">Leakage</a> for a description of all elements of this package and to <a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));

          equation 
            G_m = mu_0 * 0.077 * 2 * r;
          end QuarterSphere;
          model QuarterHollowSphere "Leakage flux through the edges of a qarter hollow sphere"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialLeakage;
            parameter SI.Length t(start = 0.01) "Thickness of sperical shell";
            annotation(__Dymola_Images(Parameters(source = "../Images/Magnetic/FluxTubes/Shapes/Leakage/QuarterHollowSphere.png")), Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.Leakage\">Leakage</a> for a description of all elements of this package and to <a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));

          equation 
            G_m = mu_0 * 0.25 * t;
          end QuarterHollowSphere;
          model EighthOfSphere "Leakage flux through one edge and the opposite plane of an eighth of a sphere"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialLeakage;
            parameter SI.Radius r(start = 0.01) "Radius of eighth of sphere";
            annotation(__Dymola_Images(Parameters(source = "../Images/Magnetic/FluxTubes/Shapes/Leakage/EighthOfSphere.png")), Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.Leakage\">Leakage</a> for a description of all elements of this package and to <a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));

          equation 
            G_m = mu_0 * 0.308 * r;
          end EighthOfSphere;
          model EighthOfHollowSphere "Leakage flux through one edge and the opposite plane of an eighth of a hollow sphere"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialLeakage;
            parameter SI.Length t(start = 0.01) "Thickness of sperical shell";
            annotation(__Dymola_Images(Parameters(source = "../Images/Magnetic/FluxTubes/Shapes/Leakage/EighthOfHollowSphere.png")), Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.Leakage\">Leakage</a> for a description of all elements of this package and to <a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));

          equation 
            G_m = mu_0 * 0.5 * t;
          end EighthOfHollowSphere;
          model CoaxCylindersEndFaces "Leakage flux between the end planes of a inner solid cylinder and a coaxial outer hollow cylinder"
            extends Modelica.Magnetic.FluxTubes.Interfaces.PartialLeakage;
            parameter SI.Radius r_0(start = 0.01) "Radius of inner solid cylinder";
            parameter SI.Radius r_1(start = 0.017) "Inner radius of outer hollow cylinder";
            parameter SI.Radius r_2(start = 0.02) "Outer radius of outer hollow cylinder";
            final parameter SI.Distance l_g = r_1 - r_0 "Radial gap length between both cylinders";
            final parameter SI.Length t = r_2 - r_1 "Radial thickness of outer hollow cylinder";
            annotation(__Dymola_Images(Parameters(source = "../Images/Magnetic/FluxTubes/Shapes/Leakage/CoaxCylindersEndFaces.png")), Documentation(info = "<html>
<p>
Please refer to the enclosing sub-package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.Leakage\">Leakage</a> for a description of all elements of this package and to <a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro41]</a> for derivation and/or coefficients of the equation for permeance G_m.
</p>
</html>"));

          equation 
            G_m = if t <= r_0 then 2 * mu_0 * (r_0 + l_g / 2) * Modelica.Math.log(1 + (2 * t) / l_g) else 2 * mu_0 * (r_0 + l_g / 2) * Modelica.Math.log(1 + (2 * r_0) / l_g);
          end CoaxCylindersEndFaces;
        end Leakage;
      end Shapes;
      package Material "Magnetisation characteristics of common soft magnetic and hard magnetic materials"
        package SoftMagnetic "Characteristics mu_r(B) of common soft magnetic materials; hysteresis neglected"
          record BaseData "Coefficients for approximation of soft magnetic materials"
            extends Modelica.Icons.Record;
            parameter SI.RelativePermeability mu_i = 1 "Initial relative permeability at B=0";
            parameter SI.MagneticFluxDensity B_myMax = 1 "Flux density at maximum relative permeability";
            parameter Real c_a = 1 "Coefficient of approximation function";
            parameter Real c_b = 1 "Coefficient of approximation function";
            parameter Real n = 1 "Exponent of approximation function";
            annotation(Documentation(info = "<html>
<p>
The parameters needed for <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.mu_rApprox\">approximation of the magnetisation characteristics</a> of included soft magnetic materials are declared in this record.
</p>
</html>"));
          end BaseData;
          package Steel "Various ferromagnetic steels"
            record Steel_9SMnPb28 "9SMnPb28 (1.0718)"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 400, B_myMax = 1.488, c_a = 1200, c_b = 3, n = 12.5);
              annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
</html>"));
            end Steel_9SMnPb28;
            record Steel_9SMn28K "9SMn28k (1.0715)"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 500, B_myMax = 1.036, c_a = 43414, c_b = 35.8, n = 14) annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
</html>"));
            end Steel_9SMn28K;
            record DC01 "DC01 (1.0330, previously St2)"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 5, B_myMax = 1.1, c_a = 6450, c_b = 3.65, n = 7.7) annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
</html>"));
            end DC01;
            record DC03 "DC03 (1.0347, previously St3)"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 0, B_myMax = 1.05, c_a = 27790, c_b = 16, n = 10.4) annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
</html>"));
            end DC03;
            record X6Cr17 "X6Cr17 (1.4016)"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 274, B_myMax = 1.1, c_a = 970, c_b = 1.2, n = 8.3) annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
</html>"));
            end X6Cr17;
            record AISI_1008 "AISI 1008 (1.0204)"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 200, B_myMax = 1.17, c_a = 8100, c_b = 2.59, n = 10) annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
</html>"));
            end AISI_1008;
            record AISI_12L14 "AISI 12L14 (1.0718)"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 10, B_myMax = 0.94, c_a = 5900, c_b = 4.19, n = 6.4) annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
</html>"));
            end AISI_12L14;
            annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
</html>"));
          end Steel;
          package ElectricSheet "Various electric sheets"
            extends Modelica.Icons.Library;
            record M330_50A "M330-50A (1.0809) @ 50Hz"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 500, B_myMax = 0.7, c_a = 24000, c_b = 9.38, n = 9.6) annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
<p>
Sample: complete core after machining and packet assembling<br>
</p>
</html>"));
            end M330_50A;
            record M350_50A "M350-50A (1.0810) @ 50Hz"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 1210, B_myMax = 1.16, c_a = 24630, c_b = 2.44, n = 14) annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
<p>
Sample: sheet strip<br>
Measurement: Epstein frame
</p>
</html>"));
            end M350_50A;
            record M530_50A "M530-50A (1.0813) @ 50Hz"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 2120, B_myMax = 1.25, c_a = 12400, c_b = 1.6, n = 13.5) annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
<p>
Sample: sheet strip<br>
Measurement: Epstein frame
</p>
</html>"));
            end M530_50A;
            record M700_100A "M700-100A (1.0826) @ 50Hz"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 1120, B_myMax = 1.2, c_a = 20750, c_b = 3.55, n = 13.15) annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
<p>
Sample: sheet strip<br>
Measurement: Epstein frame
</p>
</html>"));
            end M700_100A;
            record M940_100A "M940-100A @ 50Hz"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 680, B_myMax = 1.26, c_a = 17760, c_b = 3.13, n = 13.9) annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
<p>
Sample: sheet strip<br>
Measurement: Epstein frame
</p>
</html>"));
            end M940_100A;
            annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
</html>"));
          end ElectricSheet;
          package PureIron "Pure iron "
            record RFe80 "Hyperm 0 (RFe80)"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 123, B_myMax = 1.27, c_a = 44410, c_b = 6.4, n = 10) annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
<p>
Source of B(H) characteristics: Product catalogue <i>Magnequench</i>, 2000
</p>
</html>"));
            end RFe80;
            record VacoferS2 "VACOFER S2 (99.95% Fe)"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 2666, B_myMax = 1.15, c_a = 187000, c_b = 4.24, n = 19) annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
<p>
Source of B(H) characteristics:
<dd>
<p><i>Boll, R.</i>: Weichmagnetische Werkstoffe: Einf&uuml;hrung in den Magnetismus, VAC-Werkstoffe und ihre Anwendungen. 4th ed. Berlin, M&uuml;nchen: Siemens Aktiengesellschaft 1990</p>
</dd>
</p>
</html>"));
            end VacoferS2;
          end PureIron;
          extends Modelica.Icons.Library;
          package CobaltIron "Cobalt iron"
            record Vacoflux50 "Vacoflux 50 (50% CoFe)"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 3850, B_myMax = 1.75, c_a = 11790, c_b = 2.63, n = 15.02) annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
<p>
Source of B(H) characteristics: VACUUMSCHMELZE GmbH &amp; Co. KG, Germany
</p>
</html>"));
            end Vacoflux50;
          end CobaltIron;
          package NickelIron "Nickel iron"
            record MuMetall "MUMETALL (77% NiFe)"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 27300, B_myMax = 0.46, c_a = 1037500, c_b = 3.67, n = 10) annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
<p>
Source of B(H) characteristics:
<dd>
<p><i>Boll, R.</i>: Weichmagnetische Werkstoffe: Einf&uuml;hrung in den Magnetismus, VAC-Werkstoffe und ihre Anwendungen. 4th ed. Berlin, M&uuml;nchen: Siemens Aktiengesellschaft 1990</p>
</dd>
</p>
</html>"));
            end MuMetall;
            record Permenorm3601K3 "PERMENORM 3601 K3 (36% NiFe)"
              extends Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData(mu_i = 3000, B_myMax = 0.67, c_a = 50000, c_b = 2.39, n = 9.3) annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic\">SoftMagnetic</a> for a description of all soft magnetic material characteristics of this package.
</p>
<p>
Source of B(H) characteristics:
<dd>
<p><i>Boll, R.</i>: Weichmagnetische Werkstoffe: Einf&uuml;hrung in den Magnetismus, VAC-Werkstoffe und ihre Anwendungen. 4th ed. Berlin, M&uuml;nchen: Siemens Aktiengesellschaft 1990</p>
</dd>
</p>
</html>"));
            end Permenorm3601K3;
          end NickelIron;
          function mu_rApprox "Approximation of relative permeability mu_r as a function of flux density B for soft magnetic materials"
            extends Modelica.Icons.Function;
            input Modelica.SIunits.MagneticFluxDensity B "Flux density in ferromagnetic flux tube element";
            input Modelica.SIunits.RelativePermeability mu_i "Initial relative permeability at B=0";
            input Modelica.SIunits.MagneticFluxDensity B_myMax "Flux density at maximum relative permeability";
            input Real c_a "Coefficient of approximation function";
            input Real c_b "Coefficient of approximation function";
            input Real n "Exponent of approximation function";
            output Modelica.SIunits.RelativePermeability mu_r "Relative magnetic permeability of ferromagnetic flux tube element";
          protected 
            Real B_N "Flux density B normalized to flux density at maximum relative permeability B_myMax";
          algorithm 
            B_N:=abs(B / B_myMax);
            mu_r:=1 + (mu_i - 1 + c_a * B_N) / (1 + c_b * B_N + B_N ^ n);
            annotation(Documentation(info = "<html>
<p>
The relative permeability mu_r as a function of flux density B for all soft magnetic materials currently included in this library is approximated with the following function <a href=\"Modelica://Modelica.Magnetic.FluxTubes.UsersGuide.Literature\">[Ro00]</a>:
<br>
<dl>
<dd>
<p><IMG SRC=\"../Images/Magnetic/FluxTubes/Material/SoftMagnetic/eq_mu_rApprox.png\" ALT=\"Equation for approximation mu_r(B)\"></p>
</dd>
</dl>
<br>
Two of the five parameters of this equation have a physical meaning, namely the initial relative permeability mu_i at B=0 and the magnetic flux density at maximum permeability B_myMax. B_N is the flux density normalized to latter parameter.
</html>"));
          end mu_rApprox;
          annotation(Documentation(info = "<html>
<p>
The magnetisation characteristics mu_r(B) of all soft magnetic materials currently included in this library are approximated with a <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.mu_rApprox\">function</a>. Each material is characterised by the five parameters of this function. The approximated characteristics mu_r(B) for most of the ferromagnetic materials currently included are shown in the plots below (solid lines) together with the original data points compiled from measurements and literature.

<p><IMG SRC=\"../Images/Magnetic/FluxTubes/Material/SoftMagnetic/Steel.png\" ALT=\"Approximated magnetization characteristics of selected steels\"></p>
<p><IMG SRC=\"../Images/Magnetic/FluxTubes/Material/SoftMagnetic/Miscellaneous.png\" ALT=\"Approximated magnetization characteristics of miscellaneous soft magnetic materials\"></p>
<p><IMG SRC=\"../Images/Magnetic/FluxTubes/Material/SoftMagnetic/ElectricSheet.png\" ALT=\"Approximated magnetization characteristics of included electric sheets\"></p>

<p>
For the nonlinear curve fit, data points for high flux densities (approximately B>1T) have been weighted higher than the ones for low flux densities. This is due to the large impact of saturated ferromagnetic sections in a magnetic circuit compared to that of non-saturated sections with relative permeabilities mu_r>>1.
</p>

<p>
Note that the magnetisation characteristics largely depend on possible previous machining and on measurement conditions. A virgin material normally has a considerably higher permeability than the same material after machining (and packet assembling in case of electric sheets). This is indicated in the above plots by different magnetisation curves for similar materials. In most cases, the original data points represent commutating curves obtained with measurements at 50Hz.
</p>

<p>
Additional user-specific materials can be defined as needed. This requires determination of the approximation parameters from the original data points, preferably with a nonlinear curve fit.
</p>
</html>"));
        end SoftMagnetic;
        extends Modelica.Icons.Library;
        annotation(Documentation(info = "<html>
</html>"));
        package HardMagnetic "Characteristics of common permanent magnetic materials (temperature dependence considered)"
        public 
          record BaseData "Record for permanent magnetic material data"
            extends Modelica.Icons.Record;
            parameter SI.MagneticFieldStrength H_cBRef = 1 "Coercivity at reference temperature";
            parameter SI.MagneticFluxDensity B_rRef = 1 "Remanence at reference temperature";
            parameter SI.Temperature T_ref = 293.15 "Reference temperature";
            parameter Modelica.SIunits.LinearTemperatureCoefficient alpha_Br = 0 "Temperature coefficient of remanence at reference temperature";
            parameter SI.Temperature T_op = 293.15 "Operating temperature";
            final parameter SI.MagneticFluxDensity B_r = B_rRef * (1 + alpha_Br * (T_op - T_ref)) "Remanence at operating temperature";
            final parameter SI.MagneticFieldStrength H_cB = H_cBRef * (1 + alpha_Br * (T_op - T_ref)) "Coercivity at operating temperature";
            final parameter SI.RelativePermeability mu_r = B_r / (mu_0 * H_cB) "Relative permeability";
            annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.HardMagnetic\">HardMagnetic</a> for a description of all permanent magnetic material characteristics of this package.
</p>
</html>"));
          end BaseData;
          record NdFeB "NdFeB sintered; exemplary values"
            extends Modelica.Magnetic.FluxTubes.Material.HardMagnetic.BaseData(H_cBRef = 900000, B_rRef = 1.2, T_ref = 20 + 273.15, alpha_Br =  -0.001);
            annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.HardMagnetic\">HardMagnetic</a> for a description of all permanent magnetic material characteristics of this package.
</p>
</html>"));
          end NdFeB;
          record Sm2Co17 "Sm2Co17 sintered, exemplary values"
            extends Modelica.Magnetic.FluxTubes.Material.HardMagnetic.BaseData(H_cBRef = 750000, B_rRef = 1.02, T_ref = 20 + 273.15, alpha_Br =  -0.0003);
            annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.HardMagnetic\">HardMagnetic</a> for a description of all permanent magnetic material characteristics of this package.
</p>
</html>"));
          end Sm2Co17;
          record SmCo5 "SmCo5 sintered, exemplary values"
            extends Modelica.Magnetic.FluxTubes.Material.HardMagnetic.BaseData(H_cBRef = 720000, B_rRef = 0.95, T_ref = 20 + 273.15, alpha_Br =  -0.0004);
            annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.HardMagnetic\">HardMagnetic</a> for a description of all permanent magnetic material characteristics of this package.
</p>
</html>"));
          end SmCo5;
          record PlasticNdFeB "Plastic-bonded NdFeB, exemplary values"
            extends Modelica.Magnetic.FluxTubes.Material.HardMagnetic.BaseData(H_cBRef = 400000, B_rRef = 0.58, T_ref = 20 + 273.15, alpha_Br =  -0.001);
            annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.HardMagnetic\">HardMagnetic</a> for a description of all permanent magnetic material characteristics of this package.
</p>
</html>"));
          end PlasticNdFeB;
          record PlasticSmCo "Plastic-bonded Sm-Co, exemplary values"
            extends Modelica.Magnetic.FluxTubes.Material.HardMagnetic.BaseData(H_cBRef = 385000, B_rRef = 0.57, T_ref = 20 + 273.15, alpha_Br =  -0.0004);
            annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.HardMagnetic\">HardMagnetic</a> for a description of all permanent magnetic material characteristics of this package.
</p>
</html>"));
          end PlasticSmCo;
          record HardFerrite "Hard ferrite sintered, exemplary values"
            extends Modelica.Magnetic.FluxTubes.Material.HardMagnetic.BaseData(H_cBRef = 170000, B_rRef = 0.38, T_ref = 20 + 273.15, alpha_Br =  -0.002);
            annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.HardMagnetic\">HardMagnetic</a> for a description of all permanent magnetic material characteristics of this package.
</p>
</html>"));
          end HardFerrite;
          record PlasticHardFerrite "Plastic-bonded hard ferrite, exemplary values"
            extends Modelica.Magnetic.FluxTubes.Material.HardMagnetic.BaseData(H_cBRef = 130000, B_rRef = 0.21, T_ref = 20 + 273.15, alpha_Br =  -0.002);
            annotation(Documentation(info = "<html>
<p>
Please refer to the description of  the enclosing package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Material.HardMagnetic\">HardMagnetic</a> for a description of all permanent magnetic material characteristics of this package.
</p>
</html>"));
          end PlasticHardFerrite;
          annotation(Documentation(info = "<html>
<p>
Typical values for remanence, coercivity and the temperature coefficient of remanence are provided for the common permanent magnetic materials illustrated below.
<dl>
<dd>
<p><IMG SRC=\"../Images/Magnetic/FluxTubes/Material/HardMagnetic/HardMagneticMaterials.png\" ALT=\"Demagnetization characteristics of included permanent magnetic materials\"></p>
</dd>
</dl>
<p>
Linear demagnetization curves are modelled. The characteristic, temperature-dependent \"knee\" of many permanent magnetic materials is not considered, since proper design of permanent magnetic circuits should avoid operation of permanent magnets \"below\" that point due to partial demagnetization. As a result, the temperature coefficient of coercivity is not considered. Only the temperature coefficient of remanence alpha_Br is accounted for, since it describes the dependende of the demagnetization curve on the temperature sufficiently for the region \"above the knee-point\".
</p>
<p>
Additional user-specific materials can be defined as needed.
</p>
</html>"));
        end HardMagnetic;
      end Material;
      package Interfaces "Interfaces of magnetic network components"
        extends Modelica.Icons.Library;
        annotation(Window(x = 0, y = 0.48, width = 0.2, height = 0.21, library = 1, autolayout = 1), Documentation(info = "<html>
<p>
This package contains connectors for the magnetic domain and partial models for lumped magnetic network components.
</p>

</html>"));
        connector MagneticPort "Generic magnetic port"
          SI.MagneticPotentialDifference V_m "Magnetic potential at the port";
          flow SI.MagneticFlux Phi "Magnetic flux flowing into the port";
          annotation(defaultComponentName = "mag", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
        end MagneticPort;
        connector PositiveMagneticPort "Positive magnetic port"
          extends Modelica.Magnetic.FluxTubes.Interfaces.MagneticPort;
          annotation(defaultComponentName = "port_p", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {255,127,0}, fillColor = {255,127,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -100,160},{ -100,100}}, lineColor = {255,127,0}, textString = "%name"),Rectangle(extent = {{ -40,40},{40, -40}}, lineColor = {255,127,0}, fillColor = {255,127,0}, fillPattern = FillPattern.Solid)}));
        end PositiveMagneticPort;
        connector NegativeMagneticPort "Negative magnetic port"
          extends Modelica.Magnetic.FluxTubes.Interfaces.MagneticPort;
          annotation(defaultComponentName = "port_n", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {255,127,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{100,160},{100,100}}, lineColor = {255,127,0}, textString = "%name"),Rectangle(extent = {{ -40,40},{40, -40}}, lineColor = {255,127,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}));
        end NegativeMagneticPort;
        partial model PartialTwoPortsElementary "Partial component with two magnetic ports p and n for textual programming"
          Modelica.Magnetic.FluxTubes.Interfaces.PositiveMagneticPort port_p "Positive magnetic port" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Modelica.Magnetic.FluxTubes.Interfaces.NegativeMagneticPort port_n "Negative magnetic port" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>
Partial model of a flux tube component with two magnetic ports:
the positive port connector port_p, and the negative port
connector port_n.
</p>
</html>
"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Window(x = 0.33, y = 0.04, width = 0.63, height = 0.67), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));
        end PartialTwoPortsElementary;
        partial model PartialTwoPorts "Partial component with magnetic potential difference between two magnetic ports p and n and magnetic flux Phi from p to n"
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialTwoPortsElementary;
          SI.MagneticPotentialDifference V_m "Magnetic potential difference between both ports";
          SI.MagneticFlux Phi "Magnetic flux from port_p to port_n";

        equation 
          V_m = port_p.V_m - port_n.V_m;
          Phi = port_p.Phi;
          0 = port_p.Phi + port_n.Phi;
          annotation(Documentation(info = "<html>
<p>
It is assumed that the magnetic flux flowing into port_p is identical to the flux flowing out of port_n.
This magnetic flux is provided explicitly as flux Phi.
</p>
</html>"));
        end PartialTwoPorts;
        partial model PartialFixedShape "Base class for flux tubes with fixed shape during simulation; linear or non-linear material characteristics"
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialTwoPorts;
          parameter Boolean nonLinearPermeability = true "= true, if non-linear rel. permeability is used, otherwise constant rel. permeability" annotation(Dialog(group = "Material"), Evaluate = true);
          parameter SI.RelativePermeability mu_rConst = 1 "Constant relative permeability; used if nonLinearPermeability = false" annotation(Dialog(group = "Material", enable = not nonLinearPermeability));
          parameter Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData material = Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.BaseData() "Ferromagnetic material characteristics; used if nonLinearPermeability = true" annotation(choicesAllMatching = true, Dialog(group = "Material", enable = nonLinearPermeability));
          SI.Reluctance R_m "Magnetic reluctance";
          SI.Permeance G_m "Magnetic permeance";
          SI.MagneticFluxDensity B "Magnetic flux density";
          SI.CrossSection A "Cross-sectional area penetrated by magnetic flux";
          SI.MagneticFieldStrength H "Magnetic field strength";
          annotation(Window(x = 0.16, y = 0.15, width = 0.6, height = 0.6), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -70,30},{70, -30}}, lineColor = {255,128,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -70,0},{ -90,0}}, color = {255,128,0}),Line(points = {{70,0},{90,0}}, color = {255,128,0}),Text(extent = {{ -150, -100},{150, -60}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
Please refer to the description of  the sub-package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.FixedShape\">Shapes.FixedShape</a> for utilisation of this partial model.
</p>
</html>"));
          SI.RelativePermeability mu_r "Relative magnetic permeability";

        equation 
          mu_r = if nonLinearPermeability then Modelica.Magnetic.FluxTubes.Material.SoftMagnetic.mu_rApprox(B, material.mu_i, material.B_myMax, material.c_a, material.c_b, material.n) else mu_rConst;
          R_m = 1 / G_m;
          V_m = Phi * R_m;
          B = Phi / A;
          H = B / (mu_0 * mu_r);
        end PartialFixedShape;
        partial model PartialForce "Base class for flux tubes with reluctance force generation; constant permeability"
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialTwoPorts;
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(Evaluate = true, HideResult = true);
          parameter SI.RelativePermeability mu_r = 1 "Relative magnetic permeability";
          SI.Force F_m "Reluctance force";
          Modelica.SIunits.Length s = flange.s - s_support "Distance between flange and support";
          SI.Reluctance R_m "Magnetic reluctance";
          SI.Permeance G_m "Magnetic permeance";
          SI.Permeability dGmBydx "Derivative of permeance with respect to armature position";
          parameter Integer dlBydx = 1 "Derivative of flux tube's varying dimension with respect to armature position; set to +1 or -1";
          Modelica.Mechanics.Translational.Interfaces.Flange_b flange "Generated reluctance force at armature position" annotation(Placement(transformation(extent = {{ -10,90},{10,110}}, rotation = 0)));
          Modelica.Mechanics.Translational.Interfaces.Support support(s = s_support, f =  -flange.f) if useSupport "Support/housing of component" annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}}, rotation = 0)));
        protected 
          Modelica.SIunits.Length s_support "Absolute position of support flange";
          annotation(Window(x = 0.16, y = 0.15, width = 0.6, height = 0.6), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{30,30},{70, -30}}, lineColor = {255,128,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -70,0},{ -90,0}}, color = {255,128,0}),Line(points = {{70,0},{90,0}}, color = {255,128,0}),Text(extent = {{ -150, -80},{150, -40}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -10, -100},{ -30, -120}}, color = {0,0,0}),Line(points = {{ -30, -100},{ -50, -120}}, color = {0,0,0}),Line(points = {{ -30, -100},{30, -100}}, color = {0,0,0}),Line(points = {{10, -100},{ -10, -120}}, color = {0,0,0}),Line(points = {{30, -100},{10, -120}}, color = {0,0,0}),Rectangle(extent = {{ -70,30},{ -30, -30}}, lineColor = {255,128,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -10,80},{10, -30}}, lineColor = {0,0,0}, fillColor = {0,127,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
Please refer to the description of  the sub-package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.Force\">Shapes.Force</a> for utilisation of this partial model.
</p>
</html>"));

        equation 
          V_m = Phi * R_m;
          R_m = 1 / G_m;
          F_m = 0.5 * V_m ^ 2 * dGmBydx;
if not useSupport then
                      s_support = 0;
          end if;
          flange.f =  -F_m;
        end PartialForce;
        partial model PartialLeakage "Base class for leakage flux tubes with position-independent permeance and hence no force generation; mu_r=1"
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialTwoPorts;
          SI.Reluctance R_m "Magnetic reluctance";
          SI.Permeance G_m "Magnetic permeance";
          annotation(Window(x = 0.16, y = 0.15, width = 0.6, height = 0.6), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -70,30},{70, -30}}, lineColor = {255,128,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -70,0},{ -90,0}}, color = {255,128,0}),Line(points = {{70,0},{90,0}}, color = {255,128,0}),Text(extent = {{ -150, -100},{150, -60}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
Please refer to the description of  the sub-package <a href=\"Modelica://Modelica.Magnetic.FluxTubes.Shapes.Leakage\">Shapes.Leakage</a> for utilisation of this partial model.
</p>
</html>"));

        equation 
          V_m = Phi * R_m;
          R_m = 1 / G_m;
        end PartialLeakage;
      end Interfaces;
      package Sources "Sources of different complexity of magnetomotive force and magnetic flux"
        extends Modelica.Icons.Library;
        annotation(Window(x = 0.45, y = 0.01, width = 0.35, height = 0.49, library = 1, autolayout = 1), Documentation(info = "<html>
<p>
This package contains sources of a magnetic potential difference or a magnetic flux:
</p>
</html>"));
        model ConstantMagneticPotentialDifference "Constant magnetomotive force"
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialTwoPortsElementary;
          parameter SI.MagneticPotentialDifference V_m "Magnetic potential differnce";
          SI.MagneticFlux Phi "Magnetic flux from port_p to port_n";
          annotation(defaultComponentName = "constantSource", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Ellipse(extent = {{ -50, -50},{50,50}}, lineColor = {255,127,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{100,0},{50,0}}, color = {255,127,0}),Line(points = {{ -50,0},{ -100,0}}, color = {255,127,0}),Text(extent = {{ -80, -20},{ -80, -40}}, lineColor = {255,128,0}, textString = "+"),Text(extent = {{80, -20},{80, -40}}, lineColor = {255,128,0}, textString = "-"),Text(extent = {{ -150, -110},{150, -70}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -50,0},{50,0}}, color = {255,127,0})}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Window(x = 0.48, y = 0.25, width = 0.6, height = 0.6), Documentation(info = "<html>
<p>
Magnetic circuits under steady-state conditions, i.e. with stationary magnetic fields (change of magnetic flux  d&Phi;/dt = 0) can be described with constant sources of a magnetic potential difference or magnetomotive force (mmf). Constant magnetic potential differences are imposed by
</p>
<ul>
<li>coils with stationary current (di / dt = 0) and </li>
<li>permanent magnets modelled with <i>Th&eacute;venin</i>'s equivalent magnetic circuit. </li>
</ul>
<p>
For modelling of reluctance actuators with this source component it is assumed that the armature is fixed so that no motion-induced flux change d&Phi;/dt can occur.
</p>
</html>"));

        equation 
          V_m = port_p.V_m - port_n.V_m;
          Phi = port_p.Phi;
          0 = port_p.Phi + port_n.Phi;
        end ConstantMagneticPotentialDifference;
        model SignalMagneticPotentialDifference "Signal-controlled magnetomotive force"
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialTwoPortsElementary;
          Modelica.Blocks.Interfaces.RealInput V_m "Magnetic potential difference" annotation(Placement(transformation(origin = {0,90}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          SI.MagneticFlux Phi "Magnetic flux from port_p to port_n";
          annotation(defaultComponentName = "signalSource", Window(x = 0.27, y = 0.04, width = 0.61, height = 0.73), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -100,0},{ -50,0}}, color = {255,127,0}),Line(points = {{50,0},{100,0}}, color = {255,127,0}),Text(extent = {{ -80, -20},{ -80, -40}}, lineColor = {255,128,0}, textString = "+"),Text(extent = {{80, -20},{80, -40}}, lineColor = {255,128,0}, textString = "-"),Line(points = {{0,100},{0,50}}, color = {255,127,0}),Text(extent = {{ -150, -110},{150, -70}}, textString = "%name", lineColor = {0,0,255}),Ellipse(extent = {{ -50, -50},{50,50}}, lineColor = {255,127,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -50,0},{50,0}}, color = {255,127,0})}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
In electromagnetic devices, a change of a coil's magnetic flux linkage &Psi; reacts on the electrical subsystem in that a voltage v is induced due to <i>Faraday</i>'s law:
</p>
<pre>
    v = - d&Psi;/dt
</pre>
<p>This reaction can possibly be neglected for</p>
<ul>
<li>modelling ofelectromagnetic actuators under quasi-stationary conditions (slow current change, slow armature motion),
<li>modelling of current-controlled electromagnetic actuators (ideal current source) and</li>
<li>for system simulation where the system dynamics is not governed by an electromagnetic actuator, but by the surrounding subsystems.</li>
</ul>
<p>
In these cases, the magnetic potential difference or magnetomotive force imposed by a coil can easily be modelled with a signal-controlled source. Except for the neglected dynamics, steady-state actuator forces will be calculated properly in actuator models based on these sources.
</p>
</html>"));

        equation 
          V_m = port_p.V_m - port_n.V_m;
          Phi = port_p.Phi;
          0 = port_p.Phi + port_n.Phi;
        end SignalMagneticPotentialDifference;
        model ConstantMagneticFlux "Source of constant magnetic flux"
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialTwoPortsElementary;
          parameter SI.MagneticFlux Phi = 1 "Magnetic flux";
          SI.MagneticPotentialDifference V_m "Magnetic potential difference between both ports";
          annotation(defaultComponentName = "constantSource", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -150, -110},{150, -70}}, textString = "%name", lineColor = {0,0,255}),Polygon(points = {{80,0},{60,6},{60, -6},{80,0}}, lineColor = {255,128,0}, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Line(points = {{ -100,0},{ -50,0}}, color = {255,127,0}),Line(points = {{50,0},{100,0}}, color = {255,127,0}),Ellipse(extent = {{ -50, -50},{50,50}}, lineColor = {255,127,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,50},{0, -50}}, color = {255,127,0})}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -125,0},{ -115,0}}, color = {160,160,164}),Line(points = {{ -120, -5},{ -120,5}}, color = {160,160,164}),Line(points = {{115,0},{125,0}}, color = {160,160,164})}), Window(x = 0.48, y = 0.25, width = 0.6, height = 0.6), Documentation(info = "<html>
<p>
Sources of a constant magnetic flux are useful for modelling of permanent magnets with <i>Norton</i>'s magnetic equivalent circuit.
</p>
</html>"));

        equation 
          V_m = port_p.V_m - port_n.V_m;
          Phi = port_p.Phi;
          0 = port_p.Phi + port_n.Phi;
        end ConstantMagneticFlux;
        model SignalMagneticFlux "Signal-controlled magnetic flux source"
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialTwoPortsElementary;
          Modelica.Blocks.Interfaces.RealInput Phi "Magnetic flux" annotation(Placement(transformation(origin = {0,90}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          SI.MagneticPotentialDifference V_m "Magnetic potential difference between both ports";
          annotation(defaultComponentName = "signalSource", Window(x = 0.27, y = 0.04, width = 0.61, height = 0.73), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Polygon(points = {{80,0},{60,6},{60, -6},{80,0}}, lineColor = {255,128,0}, fillColor = {255,128,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -150, -110},{150, -70}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -100,0},{ -50,0}}, color = {255,127,0}),Line(points = {{50,0},{100,0}}, color = {255,127,0}),Line(points = {{0,100},{0,50}}, color = {255,127,0}),Ellipse(extent = {{ -50, -50},{50,50}}, lineColor = {255,127,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,50},{0, -50}}, color = {255,127,0})}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
This source of a magnetic flux is intended for test purposes, e.g. for simulation and subsequent plotting of a softmagnetic material's magnetisation characteristics if used together with a non-linear reluctance element.
</p>
</html>"));

        equation 
          V_m = port_p.V_m - port_n.V_m;
          Phi = port_p.Phi;
          0 = port_p.Phi + port_n.Phi;
        end SignalMagneticFlux;
      end Sources;
      package Sensors "Sensors to measure variables in magnetic networks"
        model MagneticPotentialDifferenceSensor "Sensor to measure magnetic potential difference"
          extends Modelica.Icons.RotationalSensor;
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialTwoPortsElementary;
          Modelica.Blocks.Interfaces.RealOutput V_m "Magnetic potential difference between ports p and n as output signal" annotation(Placement(transformation(origin = {0, -100}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          SI.MagneticFlux Phi "Magnetic flux from port_p to port_n";

        equation 
          V_m = port_p.V_m - port_n.V_m;
          Phi = port_p.Phi;
          Phi = 0;
          0 = port_p.Phi + port_n.Phi;
          annotation(Window(x = 0.28, y = 0.29, width = 0.6, height = 0.6), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -52,1},{48, -57}}, lineColor = {0,0,0}, textString = "V_m"),Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{70,0},{90,0}}, color = {0,0,0}),Line(points = {{0, -90},{0, -70}}, color = {0,0,0}),Text(extent = {{ -150,120},{150,80}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -70,0},{ -100,0}}, color = {255,128,0}),Line(points = {{70,0},{100,0}}, color = {255,128,0}),Line(points = {{0, -100},{0, -70}}, color = {0,0,0})}));
        end MagneticPotentialDifferenceSensor;
        model MagneticFluxSensor "Sensor to measure magnetic flux"
          extends Modelica.Magnetic.FluxTubes.Interfaces.PartialTwoPortsElementary;
          extends Modelica.Icons.RotationalSensor;
          Modelica.Blocks.Interfaces.RealOutput Phi "Magnetic flux from port p to port n as output signal" annotation(Placement(transformation(origin = {0, -100}, extent = {{10, -10},{ -10,10}}, rotation = 90)));

        equation 
          port_p.V_m = port_n.V_m;
          Phi = port_p.Phi;
          0 = port_p.Phi + port_n.Phi;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{0, -100},{0, -70}}, color = {0,0,0}),Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{70,0},{90,0}}, color = {0,0,0}),Text(extent = {{ -29, -11},{30, -70}}, textString = "Phi"),Text(extent = [ -150,120;150,80], textString = "%name", lineColor = {0,0,255}),Line(points = [0, -90;0, -70])}));
        end MagneticFluxSensor;
        annotation(Documentation(info = "<html>
<p>
For analysis of magnetic networks, only magnetic potential differences and magnetic flux are variables of interest. For that reason, a magnetic potential sensor is not provided.
</p>
</html>"));
      end Sensors;
    end FluxTubes;
  end Magnetic;
  package Fluid "Library of 1-dim. thermo-fluid flow models using the Modelica.Media media description"
    extends Modelica.Icons.Library;
    import SI = Modelica.SIunits;
    package UsersGuide "Users Guide"
      annotation(DocumentationClass = true, Documentation(info = "<HTML>

<p>
Library <b>Modelica.Fluid</b> is a <b>free</b> Modelica package providing components for
<b>1-dimensional thermo-fluid flow</b> in networks of pipes. A unique feature is that the
component equations and the media models
as well as pressure loss and heat transfer correlations are decoupled from each other.
All components are implemented such that they can be used for
media from the Modelica.Media library. This means especially that an
incompressible or compressible medium, a single or a multiple
substance medium with one or more phases might be used.
</p>
</HTML>"));
      class Overview "Overview"
        annotation(Documentation(info = "<HTML>
<p>
The Modelica.Fluid library provides basic interfaces and
components to model 1-dimensional thermo-fluid flow in networks of pipes.
It is not the intention that this library covers all
application cases because the fluid flow area is too large and
because for special applications it is possible to implement
libraries with simpler component interfaces.
Instead, the goal is that the Modelica.Fluid library provides
a <b>reasonable set of components</b> and that it <b>demonstrates</b>
how to implement components of a fluid flow library in Modelica,
in particular to cope with difficult issues such as connector
design, reversing flow and initialization. It is planned to
include more components in the future. User proposals are
welcome.
</p>
<p>
This library has the following main features:
</p>
<ul>
<li> The connectors Modelica.Fluid.Interfaces.FluidPort_a/_b are designed
     for one-dimensional flow of a <b>single substance</b>
     or of a <b>mixture of substances</b> with optional <b>multiple phases</b>.
     All media models from Modelica.Media can be utilized when
     connecting components. For one substance media, the additional arrays for
     multiple
     substance media have zero dimension and are therefore removed
     from the code during translation. The general connector definition
     therefore does not introduce an overhead for special cases.<br>&nbsp;</li>
<li> All the components of the Modelica.Fluid library are designed
     that they can be utilized for all media models from
     Modelica.Media if this is posssible. For example, all media can
     be utilized for the Modelica.Fluid.Sensors/Sources components.
     For some components only special media are possible, since additional
     functionality is required. For example,
     Modelica.Fluid.Components.Evaporator requires a two phase medium
     (extending from Modelica.Media.Interfaces.PartialTwoPhaseMedium).
     <br>&nbsp;</li>
<li> In order to simplify the initialization in the components,
     there is the restriction that only media models are supported
     that have T, (p,T), (p,h), (T,X), (p,T,X) or (p,h,X) as
     independent variables. Other media models would be possible,
     e.g., with (T,d) as independent variables. However, this requires
     to rewrite the code for the component initialization.
     (Note, T is temperature, p is pressure, d is density,
     h is specific enthalpy, and X is a mass fraction vector).
     <br>&nbsp;</li>
<li> All components work for <b>incompressible</b> and <b>compressible</b> media.
     This is implemented by a small change in the initialization of a
     component, if the medium is incompressible. Otherwise, the equations
     of the components are not influenced by this property.<br>&nbsp;</li>
<li> All components allow fluid flow in both directions, i.e.,
     <b>reversing flow</b> is supported. However, it is possible to declare that
     the flow through a component only has the design direction, in order to
     obtain faster simulation code.<br>&nbsp;</li>
<li> Two or more components can be connected together. This means that
     the pressures of all connected ports are equal and the mass flow rates
     sum up to zero. Specific enthalpy, mass fractions and trace substances are
     mixed according to the mass flow rates.<br>&nbsp;</li>
<li> The <b>momentum balance</b> and the <b>energy balance</b> are only fulfilled exactly if
     <b>two ports of equal diameter</b> are connected. In all other cases, the balances
     are approximated, because kinetic and friction effect are neglected. An explicit fitting
     or junction should be used if these are important for the specific problem at hand.
     In all circuits where friction dominates, or components such as pumps determine the flow rate,
     kinetic pressure is typically irrelevant. You can consider the
     <a href=\"Modelica://Modelica.Fluid.Examples.Explanatory.MomentumBalanceFittings\">
     Modelica.Fluid.Examples.Explanatory.MomentumBalanceFittings</a> model (and its documentation)
     to see one case where the momentum balance essentially depends on kinetic pressure,
     so it is necessary to use explicit fittings in order to obtain correct results.
     <br>&nbsp;</li>
<li> Given the above-mentioned limitations, there is no restriction how components can be connected
     together. The resulting simulation performance however often strongly depends on the
     model structure and modeling assumptions made. In particular the direct connection of
     fluid volumes generally results in high-index DAEs for the pressures. The direct
     connection of flow models generally results in systems of implicit nonlinear algebraic
     equations.<br>&nbsp;</li>
</ul>
</HTML>
"));
      end Overview;
      class GettingStarted "Getting started"
        annotation(Documentation(info = "<html>

<p>
Please explore the
<a href=\"Modelica://Modelica.Fluid.Examples\">Examples</a>,
which provide simple models for a broad variety of applications.
</p>
</html>
"));
      end GettingStarted;
      package ComponentDefinition "Component definition"
        annotation(Documentation(info = "<html>

<p>
In this section it is described how the components
of the Modelica.Fluid library are implemented.
If you would like to introduce new components either in
Modelica.Fluid or your own library, you should be aware
of the issues discussed in this section.
</p>
<p>
This section is partly based on the following paper:
</p>
<dl>
<dt> Elmqvist H., Tummescheit H., and Otter M.:</dt>
<dd> <b>Object-Oriented Modeling of Thermo-Fluid Systems</b>.
     Modelica 2003 Conference, Link&ouml;ping, Sweden,
     pp. 269-286, Nov. 3-4, 2003.
     Download from:
     <a href=\"http://www.modelica.org/Conference2003/papers/h40_Elmqvist_fluid.pdf\">http://www.modelica.org/Conference2003/papers/h40_Elmqvist_fluid.pdf</a>
     </dd>
</dl>
Please note that the design of the connectors has been changed with respect to the design presented in that paper.
</html>
"));
        class FluidConnectors "Fluid connectors"
          annotation(Documentation(info = "<html>
<p>
In this section the design of the fluid connectors is
explained. </p>
<p>Fluid connectors represent the points in a device (e.g. the
flanges) through which a fluid can flow into or out of the component, carrying its
thermodynamic properties; these flanges are assumed to be fixed in space. </p>
<p>
A major design goal is that components can be arbitrarily
connected and that the important balance equations are automatically
fulfilled when 2 or more components are connected together at
one point as shown in the next figure:
</p>
<p align=\"center\">
<img src=\"../Images/Fluid/UsersGuide/MixingConnections.png\">
</p>
<p>
In such a case the balance equations define <b>ideal mixing</b>,
i.e., the upstream discretization scheme of each component uses
values that result from ideal mixing in
an infinitely small time period. If more realistic modelling
is desired that takes into account mixing losses, an explicit
model has to be used in the connection point.
</p>
<h4><font color=\"#008000\">Single substance media</font></h4>
<p>
For a single substance medium, the connector definition in
Modelica.Fluid.Interfaces.FluidPort reduces to
</p>
<pre>
  <b>connector</b> FluidPort
     <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium
              \"Medium model of the fluid\";
     <b>flow</b> Medium.MassFlowRate m_flow;
              \"Mass flow rate from the connection point into the component\";
     Medium.AbsolutePressure p
              \"Thermodynamic pressure in the connection point\";
     <b>stream</b> Medium.SpecificEnthalpy h_outflow
               \"Specific thermodynamic enthalpy close to the connection point if m_flow &lt; 0\";
  <b>end</b> FluidPort;
</pre>
<p>
The first statement defines the Medium flowing through the connector.
In a medium, medium specific types such as \"Medium.AbsolutePressure\"
are defined that contain medium specific values for the min, max and
nominal attributes. Furthermore, Medium.MassFlowRate is defined as:
</p>
<pre>
   <b>type</b> MassFlowRate =
      Modelica.SIunits.MassFlowRate(quantity=\"MassFlowRate.\" + mediumName);
</pre>
<p>
With the current library design, it is necessary to explictly select the medium
model for each component in a circuit. This model is then propagated to the ports,
and a Modelica translator will check that the quantity and unit attributes
of connected interfaces are identical. Therefore, an error occurs,
if connected FluidPorts do not have a medium with the same medium name.
In the future, automatic propagation of fluid models through the ports will be
introduced, but this still not possible with Modelica 3.0.
</p>

<p>
The thermodynamic pressure is an <i>effort</i> variable, which means that the connection
of two or more ports states that the port pressures are the same.
</p>

<p>
The mass flow rate is a <i>flow </i>variable, which means that the connection of two or
more ports states that the sum of all flow rates is zero.
<p>

<p>
The last variable is a <i>stream</i> variable, i.e., a specific quantity carried by the
flow variable. The quantity on the connector always
corresponds to the value close to the connection point, assuming that the fluid is
flowing out of the connector, regardless of the actual direction of the flow. This helps
avoiding singularities when the mass flow goes through zero. The stream properties for the
other flow direction can be inquired with the built-in operator inStream(..), while the
value of the stream variable corresponding to the actual flow direction can be inquired
through the built-in operator actualStream(..).
</p>
<p>
The actual equations corresponding to these operators are introduced and solved automatically
by the tool. In principle, they correspond to the balance equation
sum(flow_variable) = 0 and sum(flow_variable*stream_variable_at_connection) = 0 applied to the set of connected ports. In this case the first equation is the mass balance sum(m_flow) = 0, and the second is the energy balance at the connection point sum(m_flow*h_connection) = 0.
</p>
<p>
In the simpler case of a one-to-one connections between port_a and port_b,
inStream(port_a.h_outflow) just returns port_b.h_outflow. For multiple-way connections,
mixing equations are generated, and special care is taken in order to avoid discontinuities
around zero flow rates. For more details, see this
<a href=\"Documentation/Fluid/Stream-Connectors-Overview-Rationale.pdf\">presentation</a>
which illustrates the stream concept rationale and the underlying technicalities.
</p>

<p>
A connector should have only the minimal number of variables to
describe the interface, otherwise there will be connection
restrictions in certain cases. Therefore, in the connector
no redundant variables are present, e.g., the temperature T
is not present because it can be computed from the connector
variables pressure p and specific enthalpy h.
</p>

<p>
Here are two simple examples to illustrate modeling with stream connectors. The first
one is a rigid adiabatic volume mixing two flows, where the kinetic and gravitational
terms in the energy balance are neglected for simplicity.
</p>

<pre>
model MixingVolume \"Volume that mixes two flows\"
  replaceable package Medium = Modelica.Media.Interfaces.PartialPureSubstance;
  FluidPort port_a, port_b;
  parameter Modelica.SIunits.Volume V \"Volume of device\";
  Modelica.SIunits.Mass             m \"Mass in device\";
  Modelica.SIunits.Energy           U \"Inner energy in device\";
  Medium.BaseProperties medium(preferredMediumStates=true) \"Medium in the device\";
equation
  // Definition of port variables
  port_a.p         = medium.p;
  port_b.p         = medium.p;
  port_a.h_outflow = medium.h;  // The stream variable always corresponds to the
  port_b.h_outflow = medium.h;  // properties of the fluid holdup (outgoing flow)

  // Total quantities
  m = V*medium.d;
  U = m*medium.u;
   // Mass and energy balance (actualStream(..) is a built-in operator for streams to
  // compute the right h, depending on the flow direction)
  der(m) = port_a.m_flow + port_b.m_flow;
  der(U) = port_a.m_flow*actualStream(port_a.h_outflow) +
           port_b.m_flow*actualStream(port_b.h_outflow);
end MixingVolume;
</pre>

<p>
The second example is the model of a component describing a lumped pressure loss
between two ports, with no energy storage and no heat transfer. An isenthalpic
transformation is assumed (changes in kinetic and potential energy between
inlet and outlet are neglected)
</p>
<pre>
model PressureLoss \"Pressure loss component\"
  replaceable package Medium=Modelica.Media.Interfaces.PartialPureSubstance;
  FluidPort port_a, port_b:
  Medium.ThermodynamicState port_a_state_inflow \"State at port_a if inflowing\";
  Medium.ThermodynamicState port_b_state_inflow \"State at port_b if inflowing\";
  Medium density d_a, d_b \"Density at ports a and b if inflowing\";
  replaceable function f \"Function to compute the mass flow rate\";
equation
  // Medium states for inflowing fluid
  port_a_state_inflow = Medium.setState_phX(port_a.p, inStream(port_a.h_outflow));
  port_b_state_inflow = Medium.setState_phX(port_b.p, inStream(port_b.h_outflow));
  // Mass balance
  0 = port_a.m_flow + port_b.m_flow;
  // Instantaneous propagation of enthalpy flow between the ports with
  // isenthalpic state transformation (no storage and no loss of energy)
  port_a.h_outflow = inStream(port_b.h_outflow);
  port_b.h_outflow = inStream(port_a.h_outflow);
  // (Regularized) Momentum balance
  port_a.m_flow = f(port_a.p, port_b.p, d_a, d_b);
end PressureLoss;
</pre>

<p>
If many such components are connected in series between two models with storage, the
specific enthalpies are propagated in both directions and available to all pressure
loss components, without problems when the mass flow goes through zero. The function
f then uses either d_a or d_b depending on the sign of port_a.p-port_b.p, with a
suitable regularization around zero to avoid discontinuities.
</p>

<p>
Please note that these models are highly idealized in order to explain the stream connector
concept. Device models in the library are much more complete, handling issues such as
initialization, steady vs. dynamic modelling, heat transfer from the outside, etc.
</p>

<h4><font color=\"#008000\">Multiple-substance media</font></h4>
<p>
Modelica.Fluid can handle models where the fluid contains multiple substances, so that its
composition can be characterized by mass fraction vectors.
</p>
<pre>
<b>connector</b> FluidPort
   <b>replaceable package</b> Medium = Modelica.Media.Interfaces.PartialMedium
      \"Medium model of the fluid\";
   <b>flow</b> Medium.MassFlowRate m_flow;
      \"Mass flow rate from the connection point into the component\"
   Medium.AbsolutePressure p
      \"Thermodynamic pressure in the connection point\";
   <b>stream</b> Medium.SpecificEnthalpy h_outflow
       \"Specific thermodynamic enthalpy close to the connection point if m_flow &lt; 0\";
   <b>stream</b> Medium.MassFraction Xi_outflow[Medium.nXi]
       \"Independent mixture mass fractions m_i/m close to the connection point if m_flow &lt; 0\";
   <b>stream</b> Medium.ExtraProperty C_outflow[Medium.nC]
       \"Properties c_i/m close to the connection point if m_flow &lt; 0\";
  <b>end</b> FluidPort;
</pre>
The mass fraction vectors Xi and C are also stream quantities, as they are carried by the mass
flow rate. The corresponding connection equations are sum(m_flow*Xi) and sum(m_flow*C), which correspond to mass balances for the single substances. The vector Xi contains the mass fractions
of the main components of the fluid, and is used together with p and h to determine the
thermodynamic state of the fluid. The vector C contains the mass fraction of the trace components,
which are accounted for in mass balances, but is ignored when computing the fluid properties. This
allows to easily declare and use medium models with trace components starting from existing medium
models (e.g. adding CO<sub>2</sub> traces to Moist Air for air conditioning models).

<h4><font color=\"#008000\">Approximations in balance equations at connection point</font></h4>
<p>
Summing up, when two or more ports of the type FluidPort are connected, the following
equations are generated by the tool:
</p>
<pre>
sum(port_j.m_flow) = 0;               // Total Mass balance
port_j = port_k;                      // Momentum balance
sum(port_j.m_flow*h_connection) = 0;  // Energy balance
sum(port_j.m_flow*Xi_connection) = 0; // Single component mass balances
sum(port_j.m_flow*C_connection) = 0;  // Trace components mass balances
</pre>
<p>
It is <b>very important</b> to bear in mind that
<ul>
<li> the mass balances are always exact; </li>
<li> the momentum and energy balance are only exact when two port with the same
diameter are connected, because there is no friction and no change in fluid velocity. </li>
</ul>
</p>
<p>
In all other cases, i.e., different port diameters and/or multple port connections:
<ul>
<li> The momentum balance does not consider friction effects and changes of pressure due to changes
in velocity. </li>
<li> There might thus be errors in the momentum balance of the order of magnitude
of the dynamic pressure &rho;v<sup>2</sup>/2.</li>
<li> The energy balance does not consider the kinetic terms (gravity terms cancel out due
to the infinitesimal size of the connection volume). There might thus be errors in the
</li> momentum balance of the order of magnitude of the kinetic energy v^2/2. </li>
</ul>
In many applications, where fluid speeds are low and thermal phenomena are mainly of interest,
these approximations are commonly made and lead to acceptable results.
In all other cases, explicit fitting and junction models should be used, that model explicitly
all the kinetic phenomena with the appropriate level of detail.
</p></html>"));
        end FluidConnectors;
        class BalanceEquations "Balance equations"
          annotation(Documentation(info = "<html>

<p>
For one-dimensional flow
along the coordinate \"x\", the following partial differential
equations hold
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td> Mass balance</td>
      <td> <img src=\"../Images/Fluid/UsersGuide/massBalance.png\"></td>
  </tr>
  <tr><td> Momentum balance</td>
      <td> <img src=\"../Images/Fluid/UsersGuide/momentumBalance.png\"></td>
  </tr>
  <tr><td> Energy balance 1</td>
      <td> <img src=\"../Images/Fluid/UsersGuide/energyBalance1.png\"></td>
  </tr>
  <tr><td> Pipe friction</td>
      <td> <img src=\"../Images/Fluid/UsersGuide/pipeFriction.png\"></td>
  </tr>
  <tr><td></td>
      <td>x: independent spatial coordinate (flow is along coordinate x)<br>
          t: time<br>
          v(x,t): mean velocity<br>
          p(x,t): mean pressure<br>
          T(x,t): mean temperature<br>
          &rho;(x,t): mean density<br>
          u(x,t): specific internal energy<br>
          z(x): height over ground<br>
          A(x): area perpendicular to direction x<br>
          g: gravity constant<br>
          f: Fanning friction factor<br>
          S: circumference
  </tr>
</table>
<p>
An alternative energy balance can be derived by multiplying
the momentum balance with \"v\" and substracting it
from the energy balance 1 above. This results in
the \"energy balance 2\":
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td> Energy balance 2</td>
      <td> <img src=\"../Images/Fluid/UsersGuide/energyBalance2.png\"></td>
  </tr>
</table>

<p>
This formulation separates the internal energy of the fluid from the kinetic energy of fluid flow.
The internal energy is treated by the energy balance 2, the kinetic energy is treated by the momentum balance equally well.
The evaluation of medium properties, which are independent of the kinetic energy, and the formulation of many fluid models is simplified with the energy balance 2.
The overall conservation of energy is achieved by considering the mutual dependencies of energy and momentum balance.
</p>
<p>
Some components in the library, like DynamicPipe, provide a rigorous implementation of mass,
momentum and energy balance, using the energy balance 2 equation. Other components, like Valves and
Fittings, neglect the impact of changes of the kinetic energy and potential energy on the energy
balance, because they are usually irrelevant compared to changes due to heat flows. The StaticPipe
component neglects the effect of kinetic energy, but includes the potential energy in the balance,
which might be substantial.
</p>
<p>
All modelling assumptions and simplifications are stated in the component documentation; please note that some of the assumptions might be stated in the base classes the
component inherits from.
</p>
</html>
"));
        end BalanceEquations;
        class UpstreamDiscretization "Upstream discretization"
          annotation(Documentation(info = "<html>

<p>
When implementing a Fluid component, the difficult arises that
the value of intensive quantities (such as p, T, &rho;)
shall be accessed from the
<b>upstream</b> volume. For example, if the fluid flows
from volume A to volume B, then the intensive quantities of
volume B have no influence on the fluid between the
two volumes. On the other hand, if the flow direction is reversed,
the intensive quantities of volume A have no influence
on the fluid between the two volumes.
</p>
<p>
In the Modelica.Fluid library, such a situation is handeled
with the following code fragment
(from Interfaces.PartialTwoPortTransport):
</p>
<pre>    <b>replaceable package</b> Medium =
                   Modelica.Media.Interfaces.PartialMedium
                   <b>annotation</b>(choicesAllMatching = <b>true</b>);

    Interfaces.FluidPort_a port_a(<b>redeclare package</b> Medium = Medium);
    Interfaces.FluidPort_b port_b(<b>redeclare package</b> Medium = Medium);

    Medium.ThermodynamicState port_a_state_inflow
                    \"Medium state close to port_a for inflowing mass flow\";
    Medium.ThermodynamicState port_b_state_inflow
                    \"Medium state close to port_b for inflowing mass flow\";

  <b>equation</b>
    // Isenthalpic state transformation (no storage and no loss of energy)
    port_a.h_outflow  = <b>inStream</b>(port_b.h_outflow);
    port_b.h_outflow  = <b>inStream</b>(port_a.h_outflow);

    port_a.Xi_outflow = <b>inStream</b>(port_b.Xi_outflow);
    port_b.Xi_outflow = <b>inStream</b>(port_a.Xi_outflow);

    // Mass balance
    port_a.m_flow + port_b.m_flow = 0;

    // Medium states for inflowing medium
    port_a_state_inflow = Medium.setState_phX(port_a.p, port_b.h_outflow, port_b.Xi_outflow);
    port_b_state_inflow = Medium.setState_phX(port_b.p, port_a.h_outflow, port_a.Xi_outflow);

    // Densities close to the parts when mass flows in to the respective port
    port_a_rho_inflow = Medium.density(port_a_state_inflow);
    port_b_rho_inflow = Medium.density(port_b_state_inflow);

    // Pressure drop correlation (k_ab, k_ba are the loss factors for the two flow
    // directions; e.g. for a circular device: k = 8*zeta/(pi*diameter)^2)^2)
    m_flow = Utilities.regRoot2(port_a.p - port_b.p, dp_small,
                                port_a_rho_inflow/k1, port_b_rho_inflow/k2);
</pre>
<p>
The medium states for inflowing media can be used to compute density and dynamic
viscosity which in turn can be use to formulate the pressure drop equation.
The standard pressure drop equation
</p>

<pre>
   dp = port_a - port_b;
   m_flow = sqrt(2/(zeta*diameter))*if dp >= 0 then  sqrt(dp)
                                               else -sqrt(-dp)
</pre>

<p>
cannot be used, since the function has an infinite derivative at dp=0.
Instead the region around zero mass flow rate must be regularized using
one of the regularization functions of Modelica.Fluid.Utilities.
This requires to have density and/or other medium properties for both
flow directions at the same time. These media properties can be computed
from the medium states of the inflowing fluid at the two ports.
</p>

<p>
If the above component is connected between two volumes, i.e.,
the independent medium variables in port_a and port_b are
usually states, then port_a.h and port_b.h are either states
(i.e., known quantities in the model) or are computed from
states. In either case they are \"known\". In such a situation,
all equations can be directly evaluated without any problems.
Zero or reversed mass flow rate does not pose any problems because
the medium properties are always computed for both flow directions
and are then used in the regularization function.
</p>

<p>
If 3 or more components are connected together, it can be shown
that a system of non-linear algebraic equations appear.
The equations are written by purpose in such a form, that
a tool can select mass flow rates and pressures as iteration
variables of this system. The advantage is that these iteration
variables are continuous and even often differentiable. The
alternative to use the medium states as iteration variables
is not good, because T,h,d are discontinuous for reversing flow
direction.
</p>
</html>
"));
        end UpstreamDiscretization;
        class RegularizingCharacteristics "Regularizing characteristics"
          annotation(Documentation(info = "<html>

<p>
Pressure drop equations and other fluid characteristics are usually
computed by <b>semi-empirical</b> equations. Unfortunately, the developers
of semi-empirical equations nearly never take into account that the
equation might be used in a simulation program. As a consequence, these
semi-empirical equations can nearly never be used blindly but must
be slightly modified or adapted in order that obvious
simulation problems are avoided. Below, examples are given to
demonstrate what problems occur and how to regularize the characteristics:
</p>

<h4><font color=\"#008000\">Square root function</font></h4>
<p>
In several empirical formulae, expressions of the following form
are present, e.g., for turbulent flow in a pipe:
</p>
<pre>   y = <b>if</b> x &lt; 0 <b>then</b> -<b>sqrt</b>( <b>abs</b>(x) ) <b>else</b> <b>sqrt</b>(x)
</pre>
<p>
A plot of this characteristic is shown in the next figure:
</p>
<p align=\"center\">
<img src=\"../Images/Fluid/UsersGuide/sqrt.png\">
</p>
<p>
The difficulty with this function is that the derivative at x=0 is infinity.
In reality, such a function does not exist. E.g., for pipe flow,
the flow becomes laminar for small velocities and therefore around zero the
sqrt() function is replaced by a linear function. Since the laminar region is
usually of not much practical interest, the above approximation is used.
</p>
<p>
The direct implementation above does not work in Modelica, because
an event is generated when x &lt; 0 changes sign. In order to detect
this event, an event iteration takes place. During the event iteration,
the active if-branch is not changed. For example, assume that x is positive
(= \"else\" branch) and shall become negative. During the event iteration
x is slightly negative and the else branch, i.e., sqrt(x), is evaluated.
Since this results in an imaginary number, an error occurs.
It would be possible to fix this, by using the <b>noEvent</b>() operator
to explicitly switch of an event:
</p>
<pre>   y = <b>if</b> <b>noEvent</b>(x &lt; 0) <b>then</b> -<b>sqrt</b>( <b>abs</b>(x) ) <b>else</b> <b>sqrt</b>(x)
</pre>
<p>
Still, it is highly likely that good integrators will not work well
around x=0, because they will recognize that the derivative changes very
sharply and will reduce the step size drastically.
</p>
<p>
There are several solutions around this problem: Around x=0, the sqrt() function
can be replaced by a polynomial of 3rd order which is determined in such a way
that it smoothly touches the sqrt() function, i.e., the whole function is continuous
and continuously differentiable. In the Modelica.Fluid library, implementations of
such critical functions are provided in sublibrary Modelica.Fluid.Utilities.
The above sqrt() type function is computed by function <b>Utilities.regRoot</b>().
This function is defined as:
</p>
<pre>     y := x/(x*x+delta*delta)^0.25;
</pre>
<p>
where \"delta\" is the size of the small region around zero where the
sqrt() function is approximated by another function. The plot of the
function above is practically identical to the one of the original function.
However, it has a finite derivative at x=0 and is differentiable upto
any order. With the default value of delta=0.01, the difference between
the function above and regRoot(x) is 16% around x=0.01, 0.25% around x=0.1
and 0.0025% around x=1.
</p>
</html>
"));
        end RegularizingCharacteristics;
        class WallFriction "Wall friction"
          annotation(Documentation(info = "<html>


<p>
One important special case for a pressure loss is the friction at the
wall of a pipe under the assumption of quasi steady state flow (i.e., the
mass flow rate varies only slowly). In this section it is explained how this case is
handeled in the Modelica.Fluid library for pipes with
<b>nonuniform roughness</b>, including the smooth pipe
as a special case (see
<a href=\"Modelica://Modelica.Fluid.Pipes.BaseClasses.WallFriction\">Pipes.BaseClasses.WallFriction</a>.
The treatment is non-standard in order to get a
numerically well-posed description.
</p>

<p>
For pipes with circular cross section the pressure drop is computed as:
</p>

<pre>
   dp = &lambda;(Re,<font face=\"Symbol\">D</font>)*(L/D)*&rho;*v*|v|/2
      = &lambda;(Re,<font face=\"Symbol\">D</font>)*8*L/(&pi;^2*D^5*&rho;)*m_flow*|m_flow|
      = &lambda;2(Re,<font face=\"Symbol\">D</font>)*k2*sign(m_flow);

with
   Re     = |v|*D*&rho;/&mu;
          = |m_flow|*4/(&pi;*D*&mu;)
   m_flow = A*v*&rho;
   A      = &pi;*(D/2)^2
   &lambda;2     = &lambda;*Re^2
   k2     = L*&mu;^2/(2*D^3*&rho;)
</pre>

<p>
where
</p>
<ul>
<li> L is the length of the pipe.</li>
<li> D is the diameter of the pipe. If the pipe has not a
     circular cross section, D = 4*A/P, where A is the cross section
     area and P is the wetted perimeter.</li>
<li> &lambda; = &lambda;(Re,<font face=\"Symbol\">D</font>) is the \"usual\" wall friction coefficient.</li>
<li> &lambda;2 = &lambda;*Re^2 is the used friction coefficient to get a numerically
     well-posed formulation.</li>
<li> Re = |v|*D*&rho;/&mu; is the Reynolds number.</li>
<li> <font face=\"Symbol\">D</font> = <font face=\"Symbol\">d</font>/D is the relative roughness where
     \"<font face=\"Symbol\">d</font>\" is
     the absolute \"roughness\", i.e., the averaged height of asperities in the pipe
     (<font face=\"Symbol\">d</font> may change over time due to growth of surface asperities during
      service, see <i>[Idelchick 1994, p. 85, Tables 2-1, 2-2])</i>.</li>
<li> &rho; is the upstream density.</li>
<li> &mu; is the upstream dynamic viscosity.</li>
<li> v is the mean velocity.</li>
</ul>
<p>
The first form with &lambda; is used and presented in textbooks,
see \"blue\" curve in the next figure:
</p>

<IMG SRC=\"../Images/Fluid/Components/PipeFriction1.png\" ALT=\"PipeFriction1\">

<p>
This form is not suited for a simulation program since
&lambda; = 64/Re if Re &lt; 2000, i.e., a division by zero occurs for
zero mass flow rate because Re = 0 in this case.
More useful for a simulation model is the friction coefficient
&lambda;2 = &lambda;*Re^2, because &lambda;2 = 64*Re if Re &lt; 2000 and
therefore no problems for zero mass flow rate occur.
The characteristic of &lambda;2 is shown in the next figure and is
used in Modelica.Fluid:
</p>

<IMG SRC=\"../Images/Fluid/Components/PipeFriction2.png\" ALT=\"PipeFriction2\">

<p>
The pressure loss characteristic is divided into three regions:
</p>

<ul>
<li> <b>Region 1</b>:
     For <b>Re &le; 2000</b>, the flow is <b>laminar</b> and the exact solution of the
     3-dim. Navier-Stokes equations (momentum and mass balance) is used under the
     assumptions of steady flow, constant pressure gradient and constant
     density and viscosity (= Hagen-Poiseuille flow) leading to &lambda;2 = 64*Re.
     Therefore:
     <pre> dp = 128*&mu;*L/(&pi;*D^4*&rho;)*m_flow </pre> <br>&nbsp;
</li>

<li> <b>Region 3</b>:
     For <b>Re &ge; 4000</b>, the flow is <b>turbulent</b>.
     Depending on the calculation direction (see \"inverse formulation\"
     below) either of two explicite equations are used. If the pressure drop dp
     is assumed to be known, &lambda;2 = |dp|/k2. The
     Colebrook-White equation
     <i>[Colebrook 1939; Idelchik 1994, p. 83, eq. (2-9)]</i>:
     <pre>1/sqrt(&lambda;) = -2*lg( 2.51/(Re*sqrt(&lambda;)) + 0.27*<font face=\"Symbol\">D</font>) </pre>
     gives an implicit relationship between Re and &lambda;.
     Inserting &lambda;2 = &lambda;*Re^2 allows to solve this equation analytically
     for Re: <pre>Re = -2*sqrt(&lambda;2)*lg(2.51/sqrt(&lambda;2) + 0.27*<font face=\"Symbol\">D</font>)</pre>
     Finally, the mass flow rate m_flow is computed from Re via
     m_flow = Re*&pi;*D*&mu;/4*sign(dp).
     These are the <b>red</b> curves in the diagrams above.<br>
     If the mass flow rate is assumed known (and therefore implicitly
     also the Reynolds number), then &lambda;2 is computed by an
     approximation of the inverse of the Colebrook-White equation
     <i>[Swamee and Jain 1976;
     Miller 1990, p. 191, eq.(8.4)]</i> adapted to &lambda;2:
     <pre> &lambda;2 = 0.25*(Re/lg(<font face=\"Symbol\">D</font>/3.7 + 5.74/Re^0.9))^2 </pre>
     The pressure drop is then computed as dp = k2*&lambda;2*sign(m_flow).
     These are the <b>blue</b> curves in the diagrams above.<br>&nbsp;</li>

<li> <b>Region 2</b>:
     For <b>2000 &le; Re &le; 4000</b> there is a transition region between laminar
     and turbulent flow. The value of &lambda;2 depends on more factors as just
     the Reynolds number and the relative roughness, therefore only crude
     approximations are possible in this area.<br>
     The deviation from the laminar region depends on the
     relative roughness. A laminar flow at Re=2000 is only reached for smooth pipes.
     The deviation Reynolds number Re1 is computed according to
     <i>[Samoilenko 1968; Idelchik 1994, p. 81, sect. 2.1.21]</i> as:
     <pre>Re1 = 745*e^(if <font face=\"Symbol\">D</font> &le; 0.0065 then 1 else 0.0065/<font face=\"Symbol\">D</font>)</pre>
     These are the <b>blue</b> curves in the diagrams above.<br>
     Between Re1=Re1(<font face=\"Symbol\">d</font>/D) and Re2=4000,
     &lambda;2 is approximated by a cubic
     polynomial in the \"lg(&lambda;2) - lg(Re)\" chart (see figures above) such that the
     first derivative is continuous at these two points. In order to avoid
     the solution of non-linear equations, two different cubic polynomials are used
     for the direct and the inverse formulation. This leads to some discrepancies
     in &lambda;2 (= differences between the red and the blue curves).
     This is acceptable, because the transition region is anyway not
     precisely known since the actual friction coefficient depends on
     additional factors and since the operating points are usually
     not in this region.</li>
</ul>
<p>
The absolute roughness <font face=\"Symbol\">d</font> has usually to
be estimated. In <i>[Idelchik 1994, pp. 105-109,
Table 2-5; Miller 1990, p. 190, Table 8-1]</i> many examples are given.
As a short summary:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td><b>Smooth pipes</b></td>
      <td>Drawn brass, coper, aluminium, glass, etc.</td>
      <td><font face=\"Symbol\">d</font> = 0.0025 mm</td>
  </tr>
  <tr><td rowspan=\"3\"><b>Steel pipes</b></td>
      <td>New smooth pipes</td>
      <td><font face=\"Symbol\">d</font> = 0.025 mm</td>
  </tr>
  <tr><td>Mortar lined, average finish</td>
      <td><font face=\"Symbol\">d</font> = 0.1 mm</td>
  </tr>
  <tr><td>Heavy rust</td>
      <td><font face=\"Symbol\">d</font> = 1 mm</td>
  </tr>
  <tr><td rowspan=\"3\"><b>Concrete pipes</b></td>
      <td>Steel forms, first class workmanship</td>
      <td><font face=\"Symbol\">d</font> = 0.025 mm</td>
  </tr>
  <tr><td>Steel forms, average workmanship</td>
      <td><font face=\"Symbol\">d</font> = 0.1 mm</td>
  </tr>
  <tr><td>Block linings</td>
      <td><font face=\"Symbol\">d</font> = 1 mm</td>
  </tr>
</table>
<p>
The equations above are valid for incompressible flow.
They can also be applied for <b>compressible</b> flow up to about <b>Ma = 0.6</b>
(Ma is the Mach number) with a maximum error in &lambda; of about 3 %.
The effect of gas compressibility in a wide region can be taken into
account by the following formula derived by Voronin
<i>[Voronin 1959; Idelchick 1994, p. 97, sect. 2.1.81]</i>:
</p>
<pre>
  &lambda;_comp = &lambda;*(1 + (&kappa;-1)/2 * Ma^2)^(-0.47)
</pre>
<p>
where &kappa; is the isentropic coefficient
(for ideal gases, &kappa; is the ratio of specific heat capacities cp/cv).
An appreciable decrease in the coefficent \"&lambda;_comp\" is observed
only in a narrow transonic region and also at supersonic flow velocities
by about 15% <i>[Idelchick 1994, p. 97, sect. 2.1.81]</i>.
This effect is not yet included in Modelica.Fluid.
Another restriction is that the pressure drop model is valid
only for steady state or slowly changing mass flow rate.
For large fluid acceleration, the pressure drop depends additionally
on the frequency of the changing mass flow rate.
</p>

<h4><font color=\"#008000\">Inverse formulation</font></h4>

<p>
In the \"Advanced menu\" it is possible via parameter
\"from_dp\" to define in which form the
pressure drop equation is actually evaluated (<b>default</b> is from_dp = <b>true</b>):
</p>
<pre>
   from_dp = <b>true</b>:   m_flow = f1(dp)
           = <b>false</b>:  dp     = f2(m_flow)
</pre>
<p>
\"from_dp\" can be useful to avoid nonlinear systems of equations
in cases where the inverse pressure loss function is needed.
</p>


<h4><font color=\"#008000\">Summary</font></h4>

<p>
A detailed pressure drop model for pipe wall friction is
provided in the form m_flow = f1(dp, <font face=\"Symbol\">D</font>) or
dp = f2(m_flow, <font face=\"Symbol\">D</font>).
These functions are continuous and differentiable,
are provided in an explicit form without solving non-linear equations,
and do behave well also at small mass flow rates. This pressure drop
model can be used stand-alone in a static momentum balance and in
a dynamic momentum balance as the friction pressure drop term.
It is valid for incompressible and compressible flow up to a Mach number of 0.6.
</p>

<h4><font color=\"#008000\">References</font></h4>

<dl><dt>Colebrook F. (1939):</dt>
    <dd><b>Turbulent flow in pipes with particular reference to the transition
         region between the smooth and rough pipe laws</b>.
         J. Inst. Civ. Eng. no. 4, 14-25.</dd>
    <dt>Idelchik I.E. (1994):</dt>
    <dd><a href=\"http://www.bookfinder.com/dir/i/Handbook_of_Hydraulic_Resistance/0849399084/\"><b>Handbook
        of Hydraulic Resistance</b></a>. 3rd edition, Begell House, ISBN
        0-8493-9908-4</dd>
    <dt>Miller D. S. (1990):</dt>
    <dd><b>Internal flow systems</b>.
    2nd edition. Cranfield:BHRA(Information Services).</dd>
    <dt>Samoilenko L.A. (1968):</dt>
    <dd><b>Investigation of the Hydraulic Resistance of Pipelines in the
        Zone of Transition from Laminar into Turbulent Motion</b>.
        Thesis (Cand. of Technical Science), Leningrad.</dd>
    <dt>Swamee P.K. and Jain A.K. (1976):</dt>
    <dd><b>Explicit equations for pipe-flow problems</b>.
         Proc. ASCE, J.Hydraul. Div., 102 (HY5), pp. 657-664.</dd>
    <dt>Voronin F.S. (1959):</dt>
    <dd><b>Effect of contraction on the friction coefficient in a
           turbulent gas flow</b>.
           Inzh. Fiz. Zh., vol. 2, no. 11, pp. 81-85.</dd>
</dl>

</html>
"));
        end WallFriction;
        class ValveCharacteristics "Valve characteristics"
          annotation(Documentation(info = "<html>
<p>
The control valves in
<a href=\"Modelica://Modelica.Fluid.Valves\">Modelica.Fluid.Valves</a>
have the parameters <b>Kv</b> and <b>Cv</b>. They are defined
as unit-less variables, but in the description text a unit
is given. The reason for this definition is the following:
</p>

<p>
The basic equation for valves is:
</p>

<pre>
  q = Av*sqrt(dp/rho)
</pre>

<p>
In SI units, [q] is m3/s, [dp] is Pascal, [rho] is [kg/m3], and Av is an area, thus [Av] = m2. Basically, the equation stems from Bernoulli's law. Av is roughly 1.4 times the area of the valve throat. Now, usually valves aren't so big that their throat area is of the order of magnitude of square meters - depending on the applications it is from a few square millimeters to a few square centimeters. Therefore, in the common engineering practice, the following equations are used:
</p>

<p>
Europe:
</p>

<pre>
  q = Kv sqrt(dp/(rho/rho0)) , with [q] = m3/h, [dp] = bar
</pre>

<p>
US:
</p>

<pre>
  q = Cv sqrt(dp/(rho/rho0)) , with [q] = USG/min, [dp] = psi
</pre>

<p>
In both cases rho0 is the density of cold water at 4 C, 999 kg/m3. Note that these equations use relative, not absolute densities.
</p>

<p>
It turns out that Kv = 1e6/27.7*Av and Cv = 1e6/24*Av, so both US and EU engineers get more or less the same numbers (just by sheer luck), with a range between a few units and a few hundred units for typical industrial applications, and everybody is happy.
</p>

<p>
Now, we've got two problems here. First, depending on the unit, we change the equation: with SI units, we use the density, with non-SI units, we use the relative density. So the quantities (not only the units!) of Av and Cv/Kv are different.
</p>

<p>
Second, the units of Kv and Cv are usually labelled \"m3/h\" and \"USG/min\", but as a matter of fact they are different, as can be seen from the equations above: they are actually
m3/(h*sqrt(bar)) and USG/(min*sqrt(psi)). If I have a valve with Kv = 10 m3/h, it means I get 10 m3/h \"for a pressure drop of 1 bar\". Unfortunately, this is not correct from the point of view of strict dimensional analysis, but nobody uses sqrt(Pa) or sqrt(bar).
</p>

<p>
You might think this is crazy (it is, expecially when you try to explain it), but as a matter of fact the valve coefficient is <b>never</b> given in square meters in any catalog or datasheet; Cv is still the most used (even in Europe), followed by Kv. So, it will be very inconvenient for users to type in Av in square meters.
</p>

<p>
The pragmatic approach used in Modelica.Fluid.ControlValves is to accept the fact that m3/h and USG/min are not the real units of Cv and Kv, so we can't use the general unit conversion mechanism, put them just as mnemonic labels in the comment, use non-dimensional coefficients in the interface, and then define properly dimensioned unit conversion within the model
</p>

</html>
"));
        end ValveCharacteristics;
      end ComponentDefinition;
      package BuildingSystemModels "Building system models"
        class SystemComponent "System component"
          annotation(Documentation(info = "<html>

<p>
The Modelica.Fluid library is designed so that each model of a system must
include an instance <tt>system</tt> of the <tt>System</tt> component at the top level, in the same way as the <tt>World</tt> model of the MultiBody Library. The System component contains the parameters that
describe the environment surrounding the components (ambient pressure and
temperature, gravity acceleration), and also provides default settings
for many parameters which are used consistently by the models in the library.
These parameters are then propagated to the individual components
using the inner/outer variable mechanism. In case the system model is structured
hieararchically, it is possible to either put a single System
component at the top level, or possibly to put many of them at different levels,
which will only influence the system components from that level down.
</p>
<p>All the parameters defined in the System model are used as default values for the parameters of the individual components of the system model. Note that it is always possible to ovverride these defaults locally by changing the value of the parameters in the specific component instance.
</p>
<ul>
<li>
The <i>General</i> tab of the System model allows to set the default enviroment variables (pressure, temperature and gravity)
used by all the components.
</li>
<li>
The <i>Assumptions</i> tab allows to change the default modelling assumptions
used by all the components (see the section <i>Customizing a system model later</i>)</li>
<li>
The <i>Initialization</i> tab allows to define default start values for mass flow rates, pressures and temperatures in the model; this can be useful to help nonlinear solver converge to the solution of any nonlinear system of equations that involves such variables, by providing meaningful guess values. </li>
<li>
The <i>Advanced</i> tab contains default values for parameters used in
the advanced settings of some components.</i>
</ul>
<p>
Remember to <b>always add a System component</b> at the top level of
your system model, otherwise you will get errors when compiling the model. The tool will automatically name it <tt>system</tt>, so that it
is recognised by all other components.
</p>
</html>"));
        end SystemComponent;
        class MediumDefinition "Definition of the medium models"
          annotation(Documentation(info = "<html>

<p>
All the models in Modelica.Fluid compute fluid properties by using medium
models defined by Modelica.Media packages. Custom fluid models can also be
used, provided they extend the interfaces defined in
Modelica.Media.Interfaces.
</p>
<p>
All the components in Modelica.Fluid use a <i>replaceable</i> medium package, called <tt>Medium</tt>: the model is written for a generic fluid, and a specific fluid model can then be specified when building a system model by redeclaring the package. This can be done in different ways:
<ul>
<li>
If several components use the same medium, it is possible to select
all of them within a GUI, and set them simultaneously (as they are
all named Medium).
</li>
<li>It is also possible to declare one or more (possibly replaceable) medium packages in the model, and then use them to set up the individual
components</li>.
</ul>
</html>"));
        end MediumDefinition;
        class CustomizingModel "Customizing a system model"
          annotation(Documentation(info = "<html>

<p>
Once a system model has been built, it is possible to obtain different approximations by
appropriately setting the defaults in the System component (and/or the settings of specific
components.
</p>
<p>
The Assumptions | allowFlowReversal parameter determines whether reversing flow conditions
(i.e. flow direction opposite to design direction) are modelled or not. By default,
reversing flow conditions are considered by the models, but this causes a significant increase
of complexity in the equations, due to the conditional equations depending on the flow direction.
If you know in advance that the flow in a certain component (or in the whole system) will always
be in the design direction, then setting this parameter to false will produce a much faster and
possibly more robust simulation code.
</p>
<p>
The flags in the Assumptions | Dynamics tab allow different degrees of approximation on
the mass, energy, and momentum equations of the components.
<ul>
<li>DynamicFreeInitial: dynamic equations are considered (nonzero storage), no
initial equations are provided, and the start values are used as guess values.</li>
<li>FixedInitial: dynamic equations are considered (nonzero storage) and initial
equations are included, fixing the states to the start values provided by the
component parameters.</li>
<li>SteadyStateInitial: dynamic equations are considered (nonzero storage), initial
equations are included, declaring that the state derivatives are zero (steady-state
initialization) and the start values are used as guess values for the nonlinear solver. </li>
<li>SteadyState: algebraic (or static) balance equations are considered (no storage)
and the start values are used as guess values for the nonlinear solver.</li>
</ul>
It is then possible to neglect the storage of mass, momentum, and energy in the whole system
(or just in parts of it) just by a few mouse clicks in a GUI, and also to change the type of
initialization when considering dynamic models. Please note that some combinations of the
options might be contradictory, and will therefore trigger compilation errors.

</p>
</html>"));
        end CustomizingModel;
        annotation(Documentation(info = "<html>

<p>
This section is a quick primer explaining how to build a system model using Modelica.Fluid.
It covers some key issues, such as the System component, the definition of medium models in the
system, and the typical customizations available in the Modelica.Fluid models.
</p>
</html>"));
      end BuildingSystemModels;
      class ReleaseNotes "Release notes"
        annotation(Documentation(info = "<HTML>

<h4><font color=\"#008000\">Version 1.1, 2009-06-21</font></h4>

<p>
The Modelica_Fluid library (revisionId = r2625) was included as Modelica.Fluid in the
Modelica Standard Library 3.1.
</p>

<h4><font color=\"#008000\">Version 1.0, 2009-01-28</font></h4>

<p>
Modelica_Fluid was refactored and finalized for the release:
</p>

<ul>
<li> Refactoring of the code<br>
     This became necessary as the previous release Modelica.Fluid Streams Beta3
     still reflected the long development history, while the basic concepts had been crystalized.
     Please consult the subversion control (SVN) logs for individual changes.</li>

<li> Device oriented package names<br>
     The former sub-packages Junctions and PressureLosses have been combined into the new subpackage Fittings.
     The former Pumps and Volumes.SweptVolume have become the initial version of fluid Machines.
     The former Volumes package is now called Vessels.</li>

<li> Complete implementation of one-dimenstional fluid flow<br>
     The balance equations as documented in
     <a href=\"Modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.BalanceEquations\">UsersGuide.ComponentDefinition.BalanceEquations</a>
     are now completely implemented. The implementations with generic boundary flow and source terms find in:
     <ul>
     <li><a href=\"Modelica://Modelica.Fluid.Interfaces.PartialDistributedVolume\">Interfaces.PartialDistributedVolume</a>,
         <a href=\"Modelica://Modelica.Fluid.Interfaces.PartialLumpedVolume\">Interfaces.PartialLumpedVolume</a>:
         Energy, Mass and Substance balances</li>
     <li><a href=\"Modelica://Modelica.Fluid.Interfaces.PartialDistributedFlow\">Interfaces.PartialDistributedFlow</a>,
         <a href=\"Modelica://Modelica.Fluid.Interfaces.PartialLumpedFlow\">Interfaces.PartialLumpedFlow</a>:
         Momentum balance</li>
     </ul>
     Specific models combine the balances and define the boundary flow and source terms as appropriate.
     For instance
     <ul>
     <li><a href=\"Modelica://Modelica.Fluid.Vessels.OpenTank\">Vessels.OpenTank</a> extends from
         <a href=\"Modelica://Modelica.Fluid.Interfaces.PartialLumpedVolume\">Interfaces.PartialLumpedVolume</a>,</li>
     <li><a href=\"Modelica://Modelica.Fluid.Fittings.SimpleGenericOrifice\">Fittings.SimpleGenericOrifice</a> extends from
         <a href=\"Modelica://Modelica.Fluid.Interfaces.PartialLumpedFlow\">Interfaces.PartialLumpedFlow</a>, besides
         <a href=\"Modelica://Modelica.Fluid.Interfaces.PartialTwoPortTransport\">Interfaces.PartialTwoPortTransport</a>,</li>
     <li><a href=\"Modelica://Modelica.Fluid.Pipes.DynamicPipe\">Pipes.DynamicPipe</a> is based on
         <a href=\"Modelica://Modelica.Fluid.Interfaces.PartialDistributedVolume\">Interfaces.PartialDistributedVolume</a> and
         <a href=\"Modelica://Modelica.Fluid.Interfaces.PartialDistributedFlow\">Interfaces.PartialDistributedFlow</a>,
         besides <a href=\"Modelica://Modelica.Fluid.Interfaces.PartialTwoPort\">Interfaces.PartialTwoPort</a>.</li>
     </ul>
     All non-trivial mass and energy balances of Vessels, Machines and Fittings have been replaced with PartialLumpedVolume.
     The mass and energy balances of Pipes are based on PartialDistributedVolume.<br>
     See <a href=\"Modelica://Modelica.Fluid.Examples.BranchingDynamicPipes\">Examples.BranchingDynamicPipes</a>
     for an example utilizing the complete balance equations.

<li> New approach for the connection of distributed flow models<br>
     The staggered grid approach offers different choices for the connection approach. So far the preferred modeling was to put full mass balances
     into the pipes and expose half momentum balances through the ports (ModelStructure a_v_b).
     This resulted in nonlinear equation systems for pressure/flow correlations in connection sets.
     A new default ModelStructure av_vb has been introduced putting full momentum balances into the models and
     exposing half mass balances through the ports (av_vb replaces the former avb). This way the nonlinear equation systems are avoided.
     High-index DAEs need to be treated instead in connection sets.
     Alternatively a Fitting like SuddenExpansion can be introduced to account for different cross flow areas of connected flow models.</li>

<li> New Vessels.BaseClasses.PartialLumpedVessel treating the ports, including hydraulic resistances, for ClosedVolume, SimpleTank and SweptVolume.</li>

<li> Clarification of modeling assumptions<br>
     The documentation has been extended to better explain the modeling assumptions made. In particular the section
     <a href=\"Modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.FluidConnectors\">UsersGuide.ComponentDefinition.FluidConnectors</a>
     now makes clear that the ports represent the thermodynamic enthalpy, as opposed to stagnation enthalpy,
     and thermodynamic or static pressure, as opposed to total pressure. An new package Explanatory has been added to the
     examples to show the difference beteen static pressure and total pressure and possible implications. See
     <a href=\"Modelica://Modelica.Fluid.Examples.Explanatory.MomentumBalanceFittings\">Examples.Explanatory.MomentumBalanceFittings</a>.

<li> System (former Ambient)<br>
     The use of the global System object has been extended towards common default values for
     modeling assumptions, initialization, and advanced settings that are different for each application of the library
     but should nevertheless provide default values for reasons of convenience.
     In particular steady-state initialization and complete steady-state simulation can now be specified system-wide.
     A new Types.Init.Dynamics has been introduced, combining steady-state and initial conditions.
     The former Types.Init has become obsolete.
     <br>See <a href=\"Modelica://Modelica.Fluid.Examples.HeatingSystem\">Examples.HeatingSystem</a></li>.

<li> Extension of pumps for better consideration of zero flow and heat transfer with environment<br>
     The simplified mass and energy balances have been replaced with a rigorous formulation.
     Moreover an optional heat transfer model can be configured for heat exchanged with the environment or the housing.<br>
     See <a href=\"Modelica://Modelica.Fluid.Machines.BaseClasses.PartialPump\">Machines.BaseClasses.PartialPump</a></li>

<li> Refinement of valves for flow reversal<br>
     All valves now use upstream discretization for reverting flow conditions.</li>

<li> Finalization of trace substrances<br>
     Modelica.Fluid now provides a sound implementation for trace substances,
     which can easily be added to existing Media models, in order to study their evolution in a fluid system.</li>
     <br>See <a href=\"Modelica://Modelica.Fluid.Examples.TraceSubstances.RoomCO2WithControls\">Examples.TraceSubstances.RoomCO2WithControls</a></li>.

<li> Vectorized ports for volumes<br>
     The ports of models that typically have large volumes, like Vessels and Sources,
     have been vectorized. Formerly the connection of multiple flow models to the same port
     of such volume models resulted in unintended mixing equations for stream variables
     in connection sets outside the volumes. The mixing takes place inside the volumes
     when using multiple ports. Moreover a
     <a href=\"Modelica://Modelica.Fluid.Fittings.MultiPort\">Fittings.MultiPort</a>
     has been introduced. It can be attached to components like pipes,
     which don't have vectorized ports on their own.</li>

<li> Inverse parameterization of flow models with nominal operational conditions<br>
     Flow models have been added or extended to support the parameterization with nominal values
     (Machines.ControlledPump, Orifices.SimpleGenericOrifice, Pipes.BaseClasses.FlowModels.NominalTurbulentFlow).
     They are intended for early phases of system modeling, if geometries and flow characteristics
     are of secondary interest. As these models use the same interfaces, base classes and naming conventions,
     they can easily be replaced with more detailed models
     as more information shall be taken into account later on.<br>
     See <a href=\"Modelica://Modelica.Fluid.Examples.InverseParameterization\">Examples.InverseParameterization</a></li>.

<li> Replaceable HeatTransfer models<br>
     The Vessels and the Machines now have replaceable HeatTransfer models,
     besides the Pipes. All HeatTransfer models are optional.
     The heat transfer models are parameterized with the Medium and the ThermodynamicState
     of involved flow segments.<br>
     See <a href=\"Modelica://Modelica.Fluid.Interfaces.PartialHeatTransfer\">Interfaces.PartialHeatTransfer</a>.

<li> All examples are working now (using Dymola 7.1).<br>
     The number of examples has been extended with the former critical test cases
     HeatingSystem and IncompressibleFluidNetwork. Moreover the HeatExchangers have been
     moved into Examples.</li>

</ul>


<h4><font color=\"#008000\">Version 1.0 Streams Beta 3, 2008-10-12</font></h4>

<p>
Modelica.Fluid was further improved:
</p>

<ul>
<li> Volumes, tanks, junctions<br>
     Added asserts to require that ports are connected at most once.
     If a user would perform more than one connection, ideal mixing
     takes place for the connected components and this is nearly never
     what the user would like to have </li>

<li> Ambient<br>
     Renamed Ambient to System, including adaptation of models.<br>
     Introduced default values system.flowDirection and
     as a comment system.initType. system.flowDirection is used in
     two port components as default.</li>

<li> GenericJunction<br>
     Corrected specification of flowDirection.<br>
     Added a HeatPort.</li>

<li> PartialDistributedFlow models<br>
     Adapted determination of velocities to usage of
     upstream properties at ports.<br>
     Corrected and unified initialization of p_start[*] values.<li>

<li> DistributedPipe models<br>
     Changed treatment of port densities and viscosities
     to the treatment of the lumped pipe model. This way events are
     avoided if the mass flow rate crosses or approaches zero.<br>
     Correct determination of Reynolds numbers.<br>
     Added test model DistributedPipeClosingValve.</li>

<li> ControlValves<br>
     Changed flowCharacteristic into valveCharacteristic<br>
     Removed parameter Kv and added dp_nom, m_flow_nom from linear
     and discrete valve interfaces. Added test cases.<br>
     Adapted Examples to new LinearValve and DiscreteValve,
     using nominal values instead of Kv. <br>
     Changed default flow coefficient selection to OpPoint<li>

<li> Fixed units for Kv and Cv in control valve models.</li>

<li> Updated tests for valves.</li>

<li> Bug in Modelica.Fluid.Test.TestComponents.Pumps.TestWaterPump2 corrected
     (complicated redeclaration issue).</li>

<li> Adapted AST_BatchPlant so that \"Check\" is sucessful.
     Simulation fails after 600 s.</li>

<li> Introduced
     density_pTX(Medium.p_default, Medium.T_default, Medium.X_default)
     as default value for nominal densities (previously it was a literal
     such as 1000).</li>

<li> Pumps<br>
     Updated energy balance equations for pumps (no division by zero anymore,
     fixed several bugs related to Np).<br>
     Added two more test cases for pumps.<br>
     Fixed pump initialization options.</li>

<li> PartialPump<br>
     Explanation for the energy balanced added as comment<br>
     \"h=0\" replaced by \"h=Medium.h_default\" since otherwise an assert is triggered
     if \"h=0\" is not in the medium range.<br>
     Fluid ports positioned in the middle line
     and using the same size as for all other components.</li>

<li> Pumps.Pump <br>
     Resized input connector, so that it has the same size as the
     standard input connectors.<br>
     Changed icon text to input connector to \"N_in [rpm]\".<br>
     Added unit 1/min to the external and internal input connector.</li>

<li> PartialValve<br>
     fillcolor=white added to icon<br>
     made line Thickness = Single,
     since icon does not look nice sometimes</li>

<li> All components<br>
     Changed %name color from black to blue
     (is a conversion bug, since Modelica 2 has blue as default
     color whereas Modelica 3 has black and Dymola is not
     taking care off this).</li>

<li> Sources<br>
     Made icon elements unvisible, if corresponding input is disabled.</li>

<li> Valves, Pipes, PressureLosses, HeatExchangers, two port senors<br>
     Added an arrow in the icon for the \"design flow direction\" from
     port_a to port_b.</li>

<li> Moved default initialization in \"System\" in to a comment, since no effect yet</li>

<li> Added the explanation from Francesco for Kv, Cv for valves in the
     users guide and added links in the corresponding valves to this description</li>
</ul>

<p>
\"Check\" for the library is successful. \"Check with Simulation\"
(i.e., simulating all test models in the library) is successful
with the exceptions:
</p>

<ul>
<li> Examples.AST_BatchPlant.BatchPlant_StandardWater<br>
     Need to be fixed in a later release (requires quite a lot of work).</li>
<li> Test.TestOverdeterminedSteadyStateInit.Test5<br>
     Test.TestOverdeterminedSteadyStateInit.Test6<br>
     These are test cases where too much initial conditions are given.
     The goal is to work on methods how this can be handled.
     So, this is a principal problem that these models do not simulate.</li>
</ul>



<h4><font color=\"#008000\">Version 1.0 Streams Beta 2, 2008-10-08</font></h4>

<p>
Modelica.Fluid was transformed to Modelica 3 and to Modelica Standard
library 3.0 (by automatic conversion). Further changes:
</p>

<ul>
<li> Emulated enumerations changed to real enumerations.</li>
<li> Improved ControlValves code</li>
<li> Introduced stream connectors with stream keyword (was previously an annotation)</li>
<li> Introduced inStream() instead of inflow() </li>
<li> Introduced m_flow*actualStream(h_outflow) instead of
     streamFlow() or semiLinear(m_flow, inStream(h_outflow), medium.h)</li>
<li> Removed Modelica.Fluid.Media and all references to it (since now available
     in Modelica.Media of MSL3.0).</li>
<li> Fixed PartialLumpedVolume for media with multiple substances</li>
<li> New function \"Utilities.RegFun3\" for regularization with static head</li>
<li> Fix density in static head models with the new RegFun3 functions
     (ticket 7)</li>
<li> Minor bug in MixingVolume corrected:<br>
     V_lumped and Wb_flow have been set as modifiers when extending from PartialLumpedVolume,
     although they are not declared as input. This is not allowed in Modelica 3.
     Fixed by replacing the modifiers by equations.</li>
<li> Modelica.Fluid.Sources.FixedBoundary<br>
     Introduced p_default, T_default, h_default as default values, since
     otherwise warnings will always be printed because parameter value is missing.</li>
<li> Modelica.Fluid.Sources.Boundary_pT<br>
     Modelica.Fluid.Sources.Boundary_ph<br>
     Modelica.Fluid.Sources.MassFlowSource_T<br>
     Changed default values of parameters reference_p, reference_T to
     p_default, T_default (some have been xx_default, some reference_xx,
     it seems best to always use the same approach)</li>
<li> Modelica.Fluid.Pipes.BaseClasses.PartialDistributedFlow<br>
     Added default value for parameter \"rho_nominal\" =
     Medium.density_pTX(Medium.p_default, Medium.T_default, Medium.X_default)
     in order to avoid unnecessary warning messages.
     Should be replaced by \"Medium.rho_default\", once available.</li>
<li> Modelica.Fluid.Pipes.DistributedPipe<br>
     Modelica.Fluid.Pipes.DistributedPipeSb<br>
     Modelica.Fluid.Pipes.DistributedPipeSa<br>
     Added default value for parameter \"mu_nominal\"
    (computed with default values of p,T,X from dynamicViscosity(..))</li>
<li> Modelica.Fluid.Pipes.BaseClasses.PartialDistributedFlowLumpedPressure<br>
     Replaced default value \"rho_nominal=0.01\" by
     Medium.density_pTX(Medium.p_default, Medium.T_default, Medium.X_default)</li>
<li> Modelica.Fluid.Volumes.OpenTank<br>
     Modelica.Fluid.Volumes.Tank<br>
     Corrected icons of ports (wrongly sized by automatic conversion from
     Modelica 2 to Modelica 3).</li>
<li> Examples.BranchingDistributedPipes<br>
     Modelica.Fluid.Test.TestComponents.Junctions.TestGenericJunction<br>
     Modelica.Fluid.Test.TestComponents.Pipes.TestDistributedPipe01<br>
     Parameters dp_nom, m_flow_nom are not defined in junction components.
     Values provided.</li>
<li> PressureLosses.BaseClasses.QuadraticTurbulent.BaseModel<br>
     No default or start values for \"parameter LossFactorData data\"
     Changed the model to \"partial model\" to avoid warning messages</li>
</ul>


<h4><font color=\"#008000\">Version 1.0 Streams Beta 1, 2008-05-02</font></h4>

<p>
Changed connectors to stream connectors and adapted the following sublibraries:
</p>

<ul>
<li> Volumes</li>
<li> PressureLosses </li>
<li> Sensors </li>
<li> Sources </li>
<li> ControlValves </li>
<li> HeatExchangers </li>
<li> Junctions </li>
<li> Pipes </li>
<li> Pumps </li>
<li> Test and Exampleas (most of the examples and tests are simulating)</li>
</ul>

<p>
Other changes:
</p>

<ul>
<li> Introduced HeatPorts with vectorized icon in Modelica.Fluid.Interfaces</li>
<li> Deleted Modelica.Fluid.WorkInProgress since it seems to be too much work
     to convert it to stream connectors</li>
<li> Added Modelica.Fluid.Media (contains ConstantLiquidWater
     medium because functions are missing in Modelica.Media),<lI>
<li> Added two additional test cases with LumpedPipes
    (to identify problems with hierarchically connected stream connectors).</li>
<li> Deleted TestPortVolumes since PortVolumes can no longer be implemented with
     stream connectors</li>
<li> Leakage flow introduced for valves</li>
<li> Drumboiler Example corrected</li>
<li> Regularization for sensors (T,h,...), in order that no discontinuity
     for bi-directional flow</li>
<li> Density computation in static head corrected</li>
<li> New functions Utilities.regUnitStep, regStep</li>
<li> New components (TestComponents.Sensors.TestOnePortSensors1/.TestOnePortSensors2l,
     TestRegStep)</li>
<li> PartialTwoPortTransport<br>
     <ul>
     <li> Introduced port_a.T, port_b.T (for plotting)</li>
     <li> Removed initialization menu</li>
     <li> Introduced dp_start, m_flow_start</li>
     <li> Removed previous start values of PartialTwoPortTransport in all models</li>
     </ul></li>
<li> PartialPump: Removed p_nom, since no longer needed (only dp_nom)</li>
<li> Made \"%name\" in the icons of all components unified (and better looking)</li>
<li> Changed default value of leackage flow of valves to zero.</li>
<li> Fixed Modelica.Fluid.Junctions.MassFlowRatio so that it compiles
     (inflow(..) currently only supported for scalars, not for vectors)</li>
<li> Added script libraryinfo.mos, in order that Modelica.Fluid appears in the
     Dymola library window automatically (provided library is in MODELICAPATH)</li>
<li> Replaced semiLinear(..) by streamFlow(..) (not yet at all places)</li>
<li> Introduced check-boxes in parameter menu of Sources (is more convenient to use)</li>
<li> TwoPortTransport<br>
     Computation of V_flow and optionally port_a_T, port_b_T.
     Error in temperature calculation corrected</li>
<li> Tank:<br>
     Default of bottom pipe diameter changed from 0 to 0.1, since
     otherwise a division by zero (if not connected and not changed).</li>
<li> Modelica.Fluid.ControlValves.ValveVaporizing:<br>
     Due to changes in PartialTwoPortTransport, port_a_T_inflow does no longer exist
     and the usage to it is removed.
     </li>
<li> Modelica.Fluid.Test.TestComponents.Sensors.TestTemperatureSensor:<br>
     Due to changes in PartialTwoPortTransport,
     p_start does no longer exist and the usage to it is removed.</li>
<li> VersionBuild introduced, as well as automatic update of
     VersionBuild/VersionDate</li>
</ul>



<h4><font color=\"#008000\">Version 1.0 Beta 4, 2008-04-26</font></h4>

<p>
Changes according to the Modelica Design Meetings since the
last beta version. This version is used to \"freeze\" the current
development, in order to change to a version with a new
connector design using stream variables.
</p>


<h4><font color=\"#008000\">Version 1.0 Beta 3, 2007-06-05</font></h4>

<p>
Changes according to the Modelica Design Meetings since the
Modelica'2006 conference, especially, improved initialization,
changed Source components (input connectors must be enabled),
improved tank component, moved test models from Examples to
new package Test, many more test models, etc.
This version is slightly non-backward compatible to version 1.0 Beta 2.
</p>


<h4><font color=\"#008000\">Version 1.0 Beta 2, 2006-08-28</font></h4>

<p>
Package considerably restructured and some new components added.
New examples (ControlledTankSystem, AST_BatchPlant).
</p>


<h4><font color=\"#008000\">Version 0.96, 2006-01-08</font></h4>

<ul>
<li> New package Modelica.Fluid.PressureLosses.</li>
<li> New package Modelica.Fluid.WorkInProgress.</li>
<li> New components in Modelica.Fluid.Components:<br>
     ShortPipe, OpenTank, ValveDiscrete, StaticHead.</li>
<li> New components in Modelica.Fluid.Examples.</li>
<li> Improved users guide.</li>
</ul>

<h4><font color=\"#008000\">Version 0.910, 2005-10-25</font></h4>
<ul>
<li> Changes as decided on 41th-45th Modelica Design Meetings
     (details, see minutes).
</ul>
<h4><font color=\"#008000\">Version 0.900, 2004-10-18</font></h4>
<ul>
<li> Changes as decided on 40th Modelica Design Meeting in Dresden
     (see also minutes)
</ul>
<h4><font color=\"#008000\">Version 0.794, 2004-05-31</font></h4>
<ul>
<li> Sensors.mo, Examples/DrumBoiler.mo: extend sensors with user choice
     for measurement unit.</li>
<li> Components.mo, Types.mo: moved components and types to
     package Examples.</li>
<li> Moved Examples from <b>file</b> Modelica.Fluid/package.mo to
     Modelica.Media/Examples <b>subdirectory</b> and created separate
     file per sub-package. This shall simplify the maintenance of
     examples by different authors</li>
<li> Moved Interfaces from file Modelica.Fluid/package.mo to
     Modelica.Fluid/Interfaces.mo</li>
</ul>
<h4><font color=\"#008000\">Version 0.793, 2004-05-18</font></h4>
<ul>
<li> Removed \"semiLinear\" function since available as
     Modelica 2.1 built-in operator in Dymola.</li>
<li> Minor bug in \"Components.ShortPipe\" corrected.</li>
<li> Bug in \"Components.Orifice\" corrected
     (dp was previously calculated in
      Interfaces.PartialTwoPortTransport,
      but this was removed and not updated in Orifice).</li>
</ul>
<h4><font color=\"#008000\">Version 0.792, 2003-11-07</font></h4>
<p>
This is the first consolidated version made up from
several changes for Modelica'2003.
Modelica.Fluid is still quite far away
from a library that could be included in the Modelica
standard library.
</p>
<h4><font color=\"#008000\">Previous Releases</font></h4>
<ul>
<li><i>Oct., 2003</i><br>
       by Martin Otter: Adapted to latest design of the Modelica.Media
       library.<br>
       by Ruediger Franke: Included sensor components and
       Modelica.Fluid.Examples.DrumBoiler example.</li>
<li><i>Sept., 2003</i><br>
       by Martin Otter: Changes according to the decisions of the
       Modelica design meeting in Dearborn, Sept. 2-4, 2003.
       Fluid library splitt in to two packages: Modelica.Media
       that contains the media models and Modelica.Fluid that
       contains fluid flow components. Modelica.Media is
       independent of Modelica.Fluid and my be used also from
       other packages that may have a different design as
       Modelica.Fluid.
<li><i>Aug., 2003</i><br>
       by Martin Otter: Improved documentation, PortVicinity (now called semiLinear)
       manually expanded, two different volume types,
       replaced number of massFractions from n to n-1 in order
       that usage of model for single substances is easier
       and in order that no special cases have to be treated
       in the equations (previously the massFraction equations had to
       be removed for single substance flow; now they are removed
       automatically, since the dimensions are zero, and not one
       as previously), included asserts to check the validity of
       the medium models, included the dynamic viscosity in the
       medium models, adapted the examples and medium models to the
       changes in Interfaces, improved menus according to the new
       features in Dymola 5.1. Added \"Components.ShortPipe\" that
       contains a detailed model of the frictional losses in pipes
       over a very wide range.</li>
<li><i>Feb., 2003</i><br>
       by Martin Otter: Included several elementary components and
       a model for moisted air. Some elementary components, such as
       FixedAmbient, are adapted versions from the SimpleFlow fluid library
       of Anton Haumer.</li>
<li><i>Dec., 2002</i><br>
       by Hubertus Tummescheit:
       Improved version of the high precision water model
       (Copy from ThermoFluid library, code reorganization,
       enhanced documentation, additional functions).</li>
<li><i>Nov. 30, 2002</i><br>
       by Martin Otter: Improved the design from the design meeting:
       Adapted to Modelica standard library 1.5,
       added \"choicesAllMatching=true\" annotation,
       added short documentation to \"Interfaces\",
       added packages \"Examples\" and \"Media\" (previously called \"Properties\")
       from previous versions and adapted them to the updated
       \"Interfaces\" package.</li>
<li><i>Nov. 20-21, 2002</i><br>
       by Hilding Elmqvist, Mike Tiller, Allan Watson, John Batteh, Chuck Newman,
       Jonas Eborn: Improved at the 32nd Modelica Design Meeting.</li>
<li><i>Nov. 11, 2002</i><br>
       by Hilding Elmqvist, Martin Otter: improved version.</li>
<li><i>Nov. 6, 2002</i><br>
       by Hilding Elmqvist: first version of the basic design.</li>
</ul>
</HTML>
"));
      end ReleaseNotes;
      class Contact "Contact"
        annotation(Documentation(info = "<html>

<dl>
<dt><b>Library Officers:</b><br>&nbsp;</dt>
<dd>
<table border=0 cellspacing=0 cellpadding=2>
<tr>
<td>
    Francesco Casella<br>
    Dipartimento di Elettronica e Informazione<br>
    Politecnico di Milano<br>
    Via Ponzio 34/5<br>
    I-20133 Milano, Italy<br>
    email: <A HREF=\"mailto:casella@elet.polimi.it\">casella@elet.polimi.it</A><br>&nbsp;
</td>
<td></td>
<td>and</td>
<td></td>
<td>
    R&uuml;diger Franke<br>
    ABB AG<br>
    PTSP-E22<br>
    Kallstadter Str. 1<br>
    D-68163, Germany<br>
    email: <A HREF=\"mailto:ruediger.franke@de.abb.com\">ruediger.franke@de.abb.com</A><br>&nbsp;
</td>
</tr>
</table>
</dd>
</dl>
<p><b>Acknowledgements:</b></p>
<p>
The development of this library has been a collaborative effort
and many have contributed.
</p>
<ul>
<li> The previous design of this library (until beginning of 2008) was based on the paper
     Elmqvist H., Tummescheit H., and Otter M.:
     <a href=\"http://www.modelica.org/events/Conference2003/papers/h40_Elmqvist_fluid.pdf\">
     Object-Oriented Modeling of Thermo-Fluid Systems</a>.
     Modelica 2003 Conference, Link&ouml;ping, Sweden, pp. 269-286, Nov. 3-4, 2003.<br>
     This design has been partly changed, especially by the introduction of the streams
     concept.</li>

<li> The Fluid library development was organized in 2002-2004 by Martin
     Otter, since 2004 it is organized by Francesco Casella, and
     since 2008 it is organized jointly by Francesco Casella and R&uuml;diger Franke.</li>

<li> Francesco Casella included several components of his ThermoPower
     library with some rewriting. The stream connector concept used in Modelica.Fluid is
     based on a similar concept developed by him for the ThermoPower library.</li>

<li> R&uuml;diger Franke initiated the stream connector concept as an extension
     and improved version of the ThermoPower concept. In Nov. 2008 - Jan. 2009 he
     greatly restructured and improved the library.<li>

<li> Michael Wetter introduced trace constituents in Modelica.Fluid consistently and
     provided corresponding examples under Examples.TraceSubstances.</li>

<li> The following people contributed to the fluid component models,
     examples, and the further design of the library
     (alphabetical list):<br>
     John Batteh,
     Francesco Casella, Jonas Eborn, Hilding Elmqvist,
     R&uuml;diger Franke, Manuel Gr&auml;ber, Henning Knigge,
     Sven Erik Mattsson, Chuck Newman, Hans Olsson,
     Martin Otter, Katrin Pr&ouml;l&szlig;,
     Christoph Richter, Michael Sielemann, Mike Tiller, Hubertus Tummescheit,
     Allan Watson, Michael Wetter.</li>
</ul>

<p>
  Partial financial support of ABB and DLR by BMBF (BMBF F&ouml;rderkennzeichen: 01IS07022F) for the further development
     of this library within the <a href=\"http://www.itea2.org\">ITEA</a> project
     <a href=\"http://www.itea2.org/public/project_leaflets/EUROSYSLIB_profile_oct-07.pdf\">EUROSYSLIB</a>
     is highly appreciated.</p>


</html>"));
      end Contact;
    end UsersGuide;
    annotation(preferedView = "info", __Dymola_classOrder = {"UsersGuide","Examples","System","Vessels","Pipes","Machines","Valves","Fittings","Sources","Sensors","Interfaces","Types","Utilities","Icons","*"}, Documentation(info = "<html>
<p>
Library <b>Modelica.Fluid</b> is a <b>free</b> Modelica package providing components for
<b>1-dimensional thermo-fluid flow</b> in networks of vessels, pipes, fluid machines, valves and fittings.
A unique feature is that the component equations and the media models
as well as pressure loss and heat transfer correlations are decoupled from each other.
All components are implemented such that they can be used for
media from the Modelica.Media library. This means especially that an
incompressible or compressible medium, a single or a multiple
substance medium with one or more phases might be used.
</p>

<p>
In the next figure, several features of the library are demonstrated with
a simple heating system with a closed flow cycle. By just changing one configuration parameter in the system object the equations are changed between steady-state and dynamic simulation with fixed or steady-state initial conditions.
</p>

<blockquote>
    <img src=\"../Images/Fluid/UsersGuide/HeatingSystem.png\" border=1>
</blockquote>

<p>
With respect to previous versions, the design
of the connectors has been changed in a non-backward compatible way,
using the recently developed concept
of stream connectors that results in much more reliable simulations
(see an overview and a rationale
<a href=\"Documentation/Fluid/Stream-Connectors-Overview-Rationale.pdf\">here</a>).
This extension was included in Modelica 3.1.
As of Jan. 2009, the stream concept is supported in Dymola 7.1.
It is recommended to use Dymola 7.2 (available since Feb. 2009), or a later Dymola version,
since this version supports a new annotation to connect very
conveniently to vectors of connectors.
Other tool vendors will support the stream concept as well.
</p>

<p>
The following parts are useful, when newly starting with this library:
</p>
<ul>
<li> <a href=\"Modelica://Modelica.Fluid.UsersGuide\">Modelica.Fluid.UsersGuide</a>.</li>
<li> <a href=\"Modelica://Modelica.Fluid.UsersGuide.ReleaseNotes\">Modelica.Fluid.UsersGuide.ReleaseNotes</a>
     summarizes the changes of the library releases.</li>
<li> <a href=\"Modelica://Modelica.Fluid.Examples\">Modelica.Fluid.Examples</a>
     contains examples that demonstrate the usage of this library.</li>
</ul>


<p>
<b>Licensed by the Modelica Association under the Modelica License 2</b><br>
Copyright &copy; 2002-2009, ABB, DLR, Dynasim, Modelon, TU Braunschweig, TU Hamburg-Harburg, Politecnico di Milano.
</p>

<p>
<i>This Modelica package is <u>free</u> software and
the use is completely at <u>your own risk</u>;
it can be redistributed and/or modified under the terms of the
Modelica license 2, see the license conditions (including the
disclaimer of warranty)
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a></u>
or at
<a href=\"http://www.Modelica.org/licenses/ModelicaLicense2\">
http://www.Modelica.org/licenses/ModelicaLicense2</a>.</i>
</p>

</html>"));
    package Examples "Demonstration of the usage of the library"
      annotation(preferedView = "info", __Dymola_classOrder = {"PumpingSystem","HeatingSystem","DrumBoiler","Tanks","ControlledTankSystem","AST_BatchPlant","IncompressibleFluidNetwork","BranchingDynamicPipes","HeatExchanger","TraceSubstances","InverseParameterization","Explanatory","*"}, Documentation(info = "<html>

</html>"));
      extends Modelica.Icons.Library;
      package TraceSubstances "Library demonstrating the usage of trace substances"
        extends Modelica.Icons.Library;
        model RoomCO2 "Demonstrates a room volume with CO2 accumulation"
          extends Modelica.Icons.Example;
          package Medium = Modelica.Media.Air.MoistAir(extraPropertiesNames = {"CO2"});
          Modelica.Blocks.Sources.Constant C(k = 0.3 * 0.001519) "substance concentration, raising to 1000 PPM CO2" annotation(Placement(transformation(extent = {{ -94, -28},{ -74, -8}})));
          Sources.FixedBoundary boundary4(nPorts = 1, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{80, -20},{60,0}})));
          Sensors.TraceSubstances traceVolume(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{0,20},{20,40}})));
          inner System system annotation(Placement(transformation(extent = {{52,36},{72,56}}, rotation = 0)));
          Sources.MassFlowSource_T boundary1(use_C_in = true, m_flow = 100 / 1.2 / 3600 * 5, redeclare package Medium = Medium, nPorts = 2, X = Medium.X_default) annotation(Placement(transformation(extent = {{ -60, -20},{ -40,0}})));
          Modelica.Fluid.Vessels.ClosedVolume volume(C_start = {0.001519}, V = 100, redeclare package Medium = Medium, nPorts = 2, X_start = {0.015,0.085}, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, use_portsData = false) annotation(Placement(transformation(extent = {{ -20,0},{0,20}})));
          Modelica.Fluid.Pipes.StaticPipe pipe(redeclare package Medium = Medium, length = 1, diameter = 0.15, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow(show_Res = true)) annotation(Placement(transformation(extent = {{20, -20},{40,0}})));
          Sensors.TraceSubstances traceSource(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -40,20},{ -20,40}})));

        equation 
          connect(C.y,boundary1.C_in[1]) annotation(Line(points = {{ -73, -18},{ -60, -18}}, color = {0,0,127}, smooth = Smooth.None));
          connect(pipe.port_b,boundary4.ports[1]) annotation(Line(points = {{40, -10},{60, -10}}, color = {0,127,255}, smooth = Smooth.None));
          connect(volume.ports[2],pipe.port_a) annotation(Line(points = {{ -8,0},{ -6,0},{ -6, -10},{20, -10}}, color = {0,127,255}, smooth = Smooth.None));
          connect(traceVolume.port,pipe.port_a) annotation(Line(points = {{10,20},{10, -10},{20, -10}}, color = {0,127,255}, smooth = Smooth.None));
          connect(boundary1.ports[1],volume.ports[1]) annotation(Line(points = {{ -40, -8},{ -12, -8},{ -12,0}}, color = {0,127,255}, smooth = Smooth.None));
          connect(boundary1.ports[2],traceSource.port) annotation(Line(points = {{ -40, -12},{ -30, -12},{ -30,20}}, color = {0,127,255}, smooth = Smooth.None));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 3600), Documentation(info = "<html>

<p>
This example consists of a volume with a carbon dioxide concentration that corresponds to about 1000 PPM.
There is a fresh air stream with a carbon dioxide concentration of about 300 PPM.
The fresh air stream is such that the air exchange rate is about 5 air changes per hour.
After 1 hour of ventilation, the volume's carbon dioxide concentration is close to the
concentration of the fresh air.
</p>

<p align=\"center\">
<img src=\"../Images/Fluid/Examples/RoomCO2.png\" border=\"1\">
</p>

</html>"), __Dymola_Commands(file(ensureSimulated = true) = "Scripts/Fluid/RoomCO2/plotConcentrations.mos" "plot concentrations"));
        end RoomCO2;
        model RoomCO2WithControls "Demonstrates a room volume with CO2 controls"
          extends Modelica.Icons.Example;
          package Medium = Modelica.Media.Air.MoistAir(extraPropertiesNames = {"CO2"});
          Modelica.Blocks.Sources.Constant CAtm(k = 0.3 * 0.001519) "Atmospheric trace substance concentration, corresponding to 300 PPM CO2" annotation(Placement(transformation(extent = {{ -100, -50},{ -80, -30}})));
          Sources.FixedBoundary boundary4(nPorts = 1, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{92, -40},{72, -20}})));
          Sensors.TraceSubstances traceVolume(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{0, -2},{20,18}})));
          inner System system annotation(Placement(transformation(extent = {{70,70},{90,90}}, rotation = 0)));
          Sources.MassFlowSource_T freshAir(use_C_in = true, redeclare package Medium = Medium, nPorts = 2, use_m_flow_in = true) annotation(Placement(transformation(extent = {{ -60, -42},{ -40, -22}})));
          Modelica.Fluid.Vessels.ClosedVolume volume(C_start = {0.001519}, V = 100, redeclare package Medium = Medium, nPorts = 3, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, use_portsData = false) annotation(Placement(transformation(extent = {{ -20, -22},{0, -2}})));
          Modelica.Fluid.Pipes.StaticPipe pipe(redeclare package Medium = Medium, length = 1, diameter = 0.15, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow(show_Res = true)) annotation(Placement(transformation(extent = {{32, -40},{52, -20}})));
          Sensors.TraceSubstances traceSource(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -40, -2},{ -20,18}})));
          Sources.MassFlowSource_T peopleSource(m_flow = 100 / 1.2 / 3600 * 5, redeclare package Medium = Medium, nPorts = 1, use_m_flow_in = true, use_C_in = false, C = {100}) "CO2 emitted by room occupants." annotation(Placement(transformation(extent = {{ -38, -98},{ -18, -78}})));
          Modelica.Blocks.Sources.CombiTimeTable NumberOfPeople(table = [0,0;9 * 3600,0;9 * 3600,10;11 * 3600,10;11 * 3600,2;13 * 3600,2;13 * 3600,15;15 * 3600,15;15 * 3600,5;18 * 3600,5;18 * 3600,0;24 * 3600,0]) "Time table for number of people in the room" annotation(Placement(transformation(extent = {{ -100, -90},{ -80, -70}})));
          Modelica.Blocks.Math.Gain gain(k = 8.18e-06 / 100) "CO2 mass flow rate, released per 100 person (there is another 100 factor in peopleSource)" annotation(Placement(transformation(extent = {{ -68, -90},{ -48, -70}})));
          Modelica.Blocks.Math.Gain gain1(k =  -(100 * 1.2) / 3600 * 5) "Nominal fresh air flow rate (for u=1)" annotation(Placement(transformation(extent = {{0,40},{20,60}})));
          Modelica.Blocks.Math.Gain gainSensor(k = 1 / 0.001519) "Gain to normalize CO2 measurement signal. y=1 corresponds to 1000 PPM" annotation(Placement(transformation(extent = {{40, -2},{60,18}})));
          Modelica.Blocks.Sources.Constant CO2Set(k = 1) "Normalized CO2 set point" annotation(Placement(transformation(extent = {{ -80,40},{ -60,60}})));
          Modelica.Blocks.Continuous.LimPID PID(controllerType = Modelica.Blocks.Types.SimpleController.PI, yMax = 0, yMin =  -1, Ti = 10, k = 10) annotation(Placement(transformation(extent = {{ -40,40},{ -20,60}})));

        equation 
          connect(CAtm.y,freshAir.C_in[1]) annotation(Line(points = {{ -79, -40},{ -60, -40}}, color = {0,0,127}, smooth = Smooth.None));
          connect(pipe.port_b,boundary4.ports[1]) annotation(Line(points = {{52, -30},{72, -30}}, color = {0,127,255}, smooth = Smooth.None));
          connect(volume.ports[2],pipe.port_a) annotation(Line(points = {{ -10, -22},{ -10, -30},{32, -30}}, color = {0,127,255}, smooth = Smooth.None));
          connect(traceVolume.port,pipe.port_a) annotation(Line(points = {{10, -2},{10, -30},{32, -30}}, color = {0,127,255}, smooth = Smooth.None));
          connect(freshAir.ports[1],volume.ports[1]) annotation(Line(points = {{ -40, -30},{ -12.6667, -30},{ -12.6667, -22}}, color = {0,127,255}, smooth = Smooth.None));
          connect(freshAir.ports[2],traceSource.port) annotation(Line(points = {{ -40, -34},{ -30, -34},{ -30, -2}}, color = {0,127,255}, smooth = Smooth.None));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 86400, Tolerance = 1e-06), __Dymola_Commands(file(ensureSimulated = true) = "Scripts/Fluid/RoomCO2WithControls/plotStatesWithControl.mos" "plot states and controls"), Documentation(info = "<html>
<p>
This example illustrates a room volume with a CO2 source and a fresh air supply with feedback
control.
The CO2 emission rate is proportional to the room occupancy, which is defined by a schedule.
The fresh air flow rate is controlled such that the room CO2
concentration does not exceed <tt>1000 PPM (=1.519E-3 kg/kg)</tt>.
The fresh air has a CO2 concentration of <tt>300 PPM</tt> which corresponds to a typical
CO2 concentration in the outside air.
</p>

<p>
The CO2 emission from the occupants is implemented as a mass flow source.
Depending on the activity and size, a person emits about <tt>8.18E-6 kg/s</tt> CO2. In the model,
this value is multiplied by the number of occupants.
Since the mass flow rate associate with the CO2 source model contributes to the volume's energy balance,
this mass flow rate should be kept small. Thus, in the source model, we set the
CO2 concentration to <tt>C={100} kg/kg</tt>, and scaled the mass flow rate using
</p>

<pre>
  m_flow = 1/100 * nPeo * 8.18E-6 kg/(s*person)
</pre>

<p>
where <tt>nPeo</tt> is the number of people in the room.
This results in a mass flow rate that is about 5 orders of magnitudes smaller than the supply air flow rate,
and hence its contribution to the volume's energy balance is negligible.
</p>

<p align=\"center\">
<img src=\"../Images/Fluid/Examples/RoomCO2WithControls.png\" border=\"1\">
</p>
</html>"));
          connect(NumberOfPeople.y[1],gain.u) annotation(Line(points = {{ -79, -80},{ -70, -80}}, color = {0,0,127}, smooth = Smooth.None));
          connect(gain.y,peopleSource.m_flow_in) annotation(Line(points = {{ -47, -80},{ -38, -80}}, color = {0,0,127}, smooth = Smooth.None));
          connect(peopleSource.ports[1],volume.ports[3]) annotation(Line(points = {{ -18, -88},{ -8, -88},{ -8, -22},{ -7.33333, -22}}, color = {0,127,255}, smooth = Smooth.None));
          connect(traceVolume.C,gainSensor.u) annotation(Line(points = {{21,8},{38,8}}, color = {0,0,127}, smooth = Smooth.None));
          connect(CO2Set.y,PID.u_s) annotation(Line(points = {{ -59,50},{ -42,50}}, color = {0,0,127}, smooth = Smooth.None));
          connect(gainSensor.y,PID.u_m) annotation(Line(points = {{61,8},{70,8},{70,30},{ -30,30},{ -30,38}}, color = {0,0,127}, smooth = Smooth.None));
          connect(PID.y,gain1.u) annotation(Line(points = {{ -19,50},{ -2,50}}, color = {0,0,127}, smooth = Smooth.None));
          connect(gain1.y,freshAir.m_flow_in) annotation(Line(points = {{21,50},{30,50},{30,70},{ -88,70},{ -88, -24},{ -60, -24}}, color = {0,0,127}, smooth = Smooth.None));
        end RoomCO2WithControls;
      end TraceSubstances;
      package Tanks "Library demonstrating the usage of the tank model"
        extends Modelica.Icons.Library;
        model ThreeTanks "Demonstrating the usage of SimpleTank"
          import Modelica.Fluid;
          extends Modelica.Icons.Example;
          replaceable package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater extends Modelica.Media.Interfaces.PartialMedium "Medium in the component" annotation(choicesAllMatching = true);
          Modelica.Fluid.Vessels.OpenTank tank1(crossArea = 1, redeclare package Medium = Medium, use_portsData = true, height = 12, level_start = 8, nPorts = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1)}) annotation(Placement(transformation(extent = {{ -80,20},{ -40,60}}, rotation = 0)));
          Modelica.Fluid.Vessels.OpenTank tank2(crossArea = 1, redeclare package Medium = Medium, use_portsData = true, height = 12, level_start = 3, nPorts = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1)}) annotation(Placement(transformation(extent = {{ -20,20},{20,60}}, rotation = 0)));
          inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial) annotation(Placement(transformation(extent = {{70, -90},{90, -70}}, rotation = 0)));
          Modelica.Fluid.Vessels.OpenTank tank3(crossArea = 1, redeclare package Medium = Medium, use_portsData = true, height = 12, level_start = 3, nPorts = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1)}) annotation(Placement(transformation(extent = {{40,10},{80,50}}, rotation = 0)));
          Modelica.Fluid.Pipes.StaticPipe pipe1(redeclare package Medium = Medium, allowFlowReversal = true, height_ab = 2, length = 2, diameter = 0.1) annotation(Placement(transformation(origin = { -60, -10}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Fluid.Pipes.StaticPipe pipe2(redeclare package Medium = Medium, allowFlowReversal = true, height_ab = 2, length = 2, diameter = 0.1) annotation(Placement(transformation(origin = {0, -10}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Fluid.Pipes.StaticPipe pipe3(redeclare package Medium = Medium, allowFlowReversal = true, height_ab =  -1, length = 2, diameter = 0.1) annotation(Placement(transformation(origin = {60, -20}, extent = {{ -10, -10},{10,10}}, rotation = 90)));

        equation 
          connect(pipe1.port_a,pipe2.port_a) annotation(Line(points = {{ -60, -20},{ -60, -40},{0, -40},{0, -30},{0, -20},{ -6.12323e-16, -20}}, color = {0,127,255}));
          connect(pipe2.port_a,pipe3.port_a) annotation(Line(points = {{ -6.12323e-16, -20},{0, -20},{0, -40},{60, -40},{60, -30}}, color = {0,127,255}));
          connect(pipe3.port_b,tank3.ports[1]) annotation(Line(points = {{60, -10},{60, -10},{60,10}}, color = {0,127,255}));
          connect(pipe1.port_b,tank1.ports[1]) annotation(Line(points = {{ -60,0},{ -60,10},{ -60,20}}, color = {0,127,255}));
          connect(pipe2.port_b,tank2.ports[1]) annotation(Line(points = {{6.12323e-16,0},{0,0},{0,20}}, color = {0,127,255}, smooth = Smooth.None));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 200), __Dymola_Commands(file = "Scripts/Fluid/ThreeTanks/plot level and port.m_flow.mos" "plot level and port.m_flow"), Documentation(info = "<html>

<p align=\"center\">
<img src=\"../Images/Fluid/Examples/ThreeTanks.png\" border=\"1\">
</p>

</html>"));
        end ThreeTanks;
        model TanksWithOverflow "Two tanks connected with pipes at different heights"
          extends Modelica.Icons.Example;
          import Modelica.Fluid;
          Modelica.Fluid.Vessels.OpenTank upperTank(redeclare package Medium = Modelica.Media.Water.StandardWater, height = 20, level_start = 2, crossArea = 0.2, nPorts = 3, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1),Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1),Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 10)}) annotation(Placement(transformation(extent = {{ -40,20},{0,60}}, rotation = 0)));
          Modelica.Fluid.Sources.MassFlowSource_T massFlowRate(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, m_flow = 0.2, use_m_flow_in = true) annotation(Placement(transformation(extent = {{ -60, -40},{ -40, -20}}, rotation = 0)));
          inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial) annotation(Placement(transformation(extent = {{ -150, -112},{ -130, -92}}, rotation = 0)));
          Modelica.Fluid.Sensors.Pressure pressure(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{40,16},{60,36}}, rotation = 0)));
          Modelica.Fluid.Pipes.StaticPipe pipe(redeclare package Medium = Modelica.Media.Water.StandardWater, diameter = 0.02, height_ab =  -20, length = 200) annotation(Placement(transformation(origin = {0, -30}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Modelica.Fluid.Vessels.OpenTank lowerTank(height = 20, redeclare package Medium = Modelica.Media.Water.StandardWater, level_start = 2, crossArea = 1, nPorts = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1),Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 10)}) annotation(Placement(transformation(extent = {{40, -60},{80, -20}}, rotation = 0)));
          Modelica.Blocks.Logical.Hysteresis hysteresis(uLow = 110000.0, uHigh = 250000.0, pre_y_start = true) "mass flow rate signal by pressure control" annotation(Placement(transformation(extent = {{ -140, -30},{ -120, -10}}, rotation = 0)));
          Modelica.Blocks.Logical.Switch switch1 annotation(Placement(transformation(extent = {{ -100, -30},{ -80, -10}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant m_flow_off(k = 0) annotation(Placement(transformation(extent = {{ -140,10},{ -120,30}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant m_flow_on(k = 2) annotation(Placement(transformation(extent = {{ -140, -60},{ -120, -40}}, rotation = 0)));
          Modelica.Fluid.Pipes.StaticPipe overflow(redeclare package Medium = Modelica.Media.Water.StandardWater, diameter = 0.02, length = 200, height_ab =  -20) annotation(Placement(transformation(origin = {20, -10}, extent = {{10, -10},{ -10,10}}, rotation = 90)));

        equation 
          connect(massFlowRate.ports[1],upperTank.ports[1]) annotation(Line(points = {{ -40, -30},{ -25.3333, -30},{ -25.3333,20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pressure.p,hysteresis.u) annotation(Line(points = {{61,26},{70,26},{70,80},{ -150,80},{ -150, -20},{ -142, -20}}, color = {0,0,127}, smooth = Smooth.None));
          connect(hysteresis.y,switch1.u2) annotation(Line(points = {{ -119, -20},{ -102, -20}}, color = {255,0,255}, smooth = Smooth.None));
          connect(m_flow_off.y,switch1.u1) annotation(Line(points = {{ -119,20},{ -119,5},{ -102,5},{ -102, -12}}, color = {0,0,127}, smooth = Smooth.None));
          connect(m_flow_on.y,switch1.u3) annotation(Line(points = {{ -119, -50},{ -110, -50},{ -110, -28},{ -102, -28}}, color = {0,0,127}, smooth = Smooth.None));
          connect(switch1.y,massFlowRate.m_flow_in) annotation(Line(points = {{ -79, -20},{ -70, -20},{ -70, -22},{ -60, -22}}, color = {0,0,127}, smooth = Smooth.None));
          connect(upperTank.ports[2],pipe.port_a) annotation(Line(points = {{ -20,20},{ -20,10},{0,10},{0, -20},{6.12323e-16, -20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipe.port_a,pressure.port) annotation(Line(points = {{6.12323e-16, -20},{0, -20},{0,10},{50,10},{50,16}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipe.port_b,lowerTank.ports[1]) annotation(Line(points = {{ -6.12323e-16, -40},{0, -40},{0, -70},{56, -70},{56, -60}}, color = {0,127,255}, smooth = Smooth.None));
          connect(upperTank.ports[3],overflow.port_a) annotation(Line(points = {{ -14.6667,20},{0,20},{0,40},{20,40},{20,0}}, color = {0,127,255}, smooth = Smooth.None));
          connect(overflow.port_b,lowerTank.ports[2]) annotation(Line(points = {{20, -20},{20, -40},{40, -40},{40, -60},{64, -60}}, color = {0,127,255}, smooth = Smooth.None));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -160, -120},{100,120}}, initialScale = 0.1), graphics), experiment(StopTime = 25000, NumberOfIntervals = 5000), __Dymola_Commands(file = "Scripts/Fluid/TanksWithOverflow/plot level and port.m_flow.mos" "plot level and port.m_flow"), Documentation(info = "<html>
<p align=justify>The mass flow rate to the upper tank is controlled by the static pressure at its bottom.
The fluid flows through a pipe and forced by different heights from the upper tank to the lower tank.
</p>
<p>
Additional fluid flows through an overflow pipe if the level of the upper tank exceeds 10m.
Initially the overflow enters the lower tank above its fluid level; later on the fluid level exceeds the overflow port.
</p>
<p>
Note that the number of solver intervals has been increased, accounting for the long simulation time horizon.
Otherwise the simulation may fail due to too large steps subject to events. Alternatively the
simulation accuracy could be increased in order to avoid errors.
</p>

<p align=\"center\">
<img src=\"../Images/Fluid/Examples/TanksWithOverflow.png\" border=\"1\">
</p>

</html>"));
        end TanksWithOverflow;
        model EmptyTanks "Show the treatment of empty tanks"
          extends Modelica.Icons.Example;
          Modelica.Fluid.Vessels.OpenTank tank1(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, nPorts = 1, crossArea = 1, level_start = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1)}, height = 1.1) annotation(Placement(transformation(extent = {{ -40,20},{0,60}}, rotation = 0)));
          Modelica.Fluid.Pipes.StaticPipe pipe(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, length = 1, diameter = 0.1, height_ab =  -1) annotation(Placement(transformation(origin = { -20, -20}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Fluid.Vessels.OpenTank tank2(crossArea = 1, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, nPorts = 1, height = 1.1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 0.5)}, level_start = 0) annotation(Placement(transformation(extent = {{0, -80},{40, -40}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{60,60},{80,80}}, rotation = 0)));

        equation 
          connect(tank1.ports[1],pipe.port_a) annotation(Line(points = {{ -20,20},{ -20,5},{ -20, -10},{ -20, -10}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipe.port_b,tank2.ports[1]) annotation(Line(points = {{ -20, -30},{ -20, -60},{0, -60},{0, -80},{20, -80}}, color = {0,127,255}, smooth = Smooth.None));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 50), __Dymola_Commands(file = "Scripts/Fluid/EmptyTanks/plot level and port.p.mos" "plot level and port.p"), Documentation(info = "<html>

<p align=\"center\">
<img src=\"../Images/Fluid/Examples/EmptyTanks.png\" border=\"1\">
</p>
</html>"));
        end EmptyTanks;
      end Tanks;
      model PumpingSystem "Model of a pumping system for drinking water"
        extends Modelica.Icons.Example;
        Modelica.Fluid.Sources.FixedBoundary source(nPorts = 1, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, use_T = true, T = Modelica.SIunits.Conversions.from_degC(20), p = system.p_ambient) annotation(Placement(transformation(extent = {{ -100, -80},{ -80, -60}}, rotation = 0)));
        Modelica.Fluid.Pipes.StaticPipe pipe(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, allowFlowReversal = true, length = 100, height_ab = 50, diameter = 0.3) annotation(Placement(transformation(origin = { -30, -51}, extent = {{ -9, -10},{11,10}}, rotation = 90)));
        Machines.PrescribedPump pumps(checkValve = true, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, N_nominal = 1200, redeclare function flowCharacteristic = Modelica.Fluid.Machines.BaseClasses.PumpCharacteristics.quadraticFlow(V_flow_nominal = {0,0.25,0.5}, head_nominal = {100,60,0}), V = 50 / 1000, T_start = Modelica.SIunits.Conversions.from_degC(20), use_N_in = true, nParallel = 1, energyDynamics = Modelica.Fluid.Types.Dynamics.DynamicFreeInitial, massDynamics = Modelica.Fluid.Types.Dynamics.DynamicFreeInitial) annotation(Placement(transformation(extent = {{ -68, -80},{ -48, -60}}, rotation = 0)));
        Modelica.Fluid.Vessels.OpenTank reservoir(massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, T_start = Modelica.SIunits.Conversions.from_degC(20), use_portsData = true, crossArea = 50, level_start = 2.2, height = 3, nPorts = 3, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.3),Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.3),Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.01)}) annotation(Placement(transformation(extent = {{ -20, -16},{0,4}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveLinear userValve(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, allowFlowReversal = false, dp_nominal = 200000, m_flow_nominal = 400) annotation(Placement(transformation(extent = {{58, -38},{74, -22}}, rotation = 0)));
        Modelica.Fluid.Sources.FixedBoundary sink(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, p = system.p_ambient, T = system.T_ambient, nPorts = 2) annotation(Placement(transformation(extent = {{100, -40},{80, -20}}, rotation = 0)));
        Modelica.Blocks.Sources.Step valveOpening(startTime = 200, offset = 1e-06) annotation(Placement(transformation(extent = {{56,0},{76,20}}, rotation = 0)));
        Modelica.Blocks.Sources.Constant RelativePressureSetPoint(k = 20000.0) annotation(Placement(transformation(extent = {{ -100,60},{ -80,80}}, rotation = 0)));
        Modelica.Blocks.Logical.OnOffController controller(bandwidth = 4000, pre_y_start = false) annotation(Placement(transformation(extent = {{ -40,60},{ -20,80}}, rotation = 0)));
        Modelica.Blocks.Logical.TriggeredTrapezoid PumpRPMGenerator(rising = 3, falling = 3, amplitude = 1200, offset = 0.001) annotation(Placement(transformation(extent = {{0,60},{20,80}}, rotation = 0)));
        Modelica.Fluid.Sensors.RelativePressure reservoirPressure(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater) annotation(Placement(transformation(extent = {{10, -12},{30, -32}}, rotation = 0)));
        Modelica.Blocks.Continuous.FirstOrder PT1(T = 2, initType = Modelica.Blocks.Types.Init.InitialState, y_start = 0) annotation(Placement(transformation(extent = {{40,60},{60,80}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Documentation(info = "<html>
Water is pumped from a source by a pump (fitted with check valves), through a pipe whose outlet is 50 m higher than the source, into a reservoir. The users are represented by an equivalent valve, connected to the reservoir.
<p>
The water controller is a simple on-off controller, regulating on the gauge pressure measured at the base of the tower; the output of the controller is the rotational speed of the pump, which is represented by the output of a first-order system. A small but nonzero rotational speed is used to represent the standby state of the pumps, in order to avoid singularities in the flow characteristic.
<p>
Simulate for 2000 s. When the valve is opened at time t=200, the pump starts turning on and off to keep the reservoir level around 2 meters, which roughly corresponds to a gauge pressure of 200 mbar
<p>
If using Dymola, turn off \"Equidistant time grid\" to avoid numerical errors.
</p>

<p align=\"center\">
<img src=\"../Images/Fluid/Examples/PumpingSystem.png\" border=\"1\">
</p>

</html>", revisions = "<html>
<ul>
<li><i>Jan 2009</i>
    by R&uuml;diger Franke:<br>
       Reduce diameters of pipe and reservoir ports; use separate port for measurement of reservoirPressure, avoiding disturbances due to pressure losses.</li>
<li><i>1 Oct 2007</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Parameters updated.</li>
<li><i>2 Nov 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Created.</li>
</ul>
</html>"), experiment(StopTime = 2000, NumberOfIntervals = 5000, Tolerance = 1e-06), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1})));
        inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{60, -96},{80, -76}}, rotation = 0)));

      equation 
        connect(userValve.port_b,sink.ports[1]) annotation(Line(points = {{74, -30},{77, -30},{77, -28},{80, -28}}, color = {0,127,255}));
        connect(source.ports[1],pumps.port_a) annotation(Line(points = {{ -80, -70},{ -74, -70},{ -68, -70}}, color = {0,127,255}));
        connect(valveOpening.y,userValve.opening) annotation(Line(points = {{77,10},{98,10},{98, -12},{66, -12},{66, -23.6}}, color = {0,0,127}));
        connect(RelativePressureSetPoint.y,controller.reference) annotation(Line(points = {{ -79,70},{ -60,70},{ -60,76},{ -42,76}}, color = {0,0,127}));
        connect(controller.y,PumpRPMGenerator.u) annotation(Line(points = {{ -19,70},{ -2,70}}, color = {255,0,255}));
        connect(reservoirPressure.p_rel,controller.u) annotation(Line(points = {{20, -13},{20,50},{ -52,50},{ -52,64},{ -42,64}}, color = {0,0,127}));
        connect(reservoirPressure.port_b,sink.ports[2]) annotation(Line(points = {{30, -22},{44, -22},{44, -48},{80, -48},{80, -32}}, color = {0,127,255}, pattern = LinePattern.Dot));
        connect(PumpRPMGenerator.y,PT1.u) annotation(Line(points = {{21,70},{38,70}}, color = {0,0,127}));
        connect(PT1.y,pumps.N_in) annotation(Line(points = {{61,70},{74,70},{74,30},{ -58,30},{ -58, -60}}, color = {0,0,127}));
        connect(pipe.port_a,pumps.port_b) annotation(Line(points = {{ -30, -60},{ -30, -70},{ -48, -70}}, color = {0,127,255}));
        connect(reservoir.ports[1],pipe.port_b) annotation(Line(points = {{ -12.6667, -16},{ -12.6667, -30},{ -30, -30},{ -30, -40}}, color = {0,127,255}, smooth = Smooth.None));
        connect(reservoir.ports[3],reservoirPressure.port_a) annotation(Line(points = {{ -7.33333, -16},{ -7, -16},{ -7, -22},{10, -22}}, color = {0,127,255}, smooth = Smooth.None, pattern = LinePattern.Dot));
        connect(reservoir.ports[2],userValve.port_a) annotation(Line(points = {{ -10, -16},{ -10, -30},{58, -30}}, color = {0,127,255}, smooth = Smooth.None));
      end PumpingSystem;
      model InverseParameterization "Demonstrates the parameterization of a pump and a pipe for given nominal values"
        extends Modelica.Icons.Example;
        replaceable package Medium = Modelica.Media.Water.StandardWater;
        Modelica.Fluid.Sources.Boundary_pT source(redeclare package Medium = Medium, nPorts = 1, use_p_in = false, p = 100000) annotation(Placement(transformation(extent = {{ -76,14},{ -64,26}}, rotation = 0)));
        Modelica.Fluid.Machines.ControlledPump pump(m_flow_nominal = 1, control_m_flow = false, use_p_set = true, redeclare package Medium = Medium, p_a_nominal = 100000, p_b_nominal = 200000) annotation(Placement(transformation(extent = {{ -40,10},{ -20,30}})));
        Modelica.Fluid.Fittings.SimpleGenericOrifice orifice(redeclare package Medium = Medium, diameter = 0.0254, m_flow_nominal = 1, use_zeta = false, zeta = 0, dp_nominal = 100000) annotation(Placement(transformation(extent = {{20,10},{40,30}}, rotation = 0)));
        Modelica.Fluid.Sources.Boundary_pT sink(nPorts = 1, redeclare package Medium = Medium, p = 100000) annotation(Placement(transformation(extent = {{76,14},{64,26}}, rotation = 0)));
        inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -90,50},{ -70,70}}, rotation = 0)));
        Modelica.Fluid.Pipes.StaticPipe pipe1(redeclare package Medium = Medium, diameter = 0.0254, length = 0, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.NominalTurbulentPipeFlow(m_flow_nominal = 1, show_Res = true, dp_nominal = 100000)) annotation(Placement(transformation(extent = {{20, -30},{40, -10}}, rotation = 0)));
        Modelica.Fluid.Sources.Boundary_pT sink1(nPorts = 1, redeclare package Medium = Medium, p = 200000) annotation(Placement(transformation(extent = {{76, -26},{64, -14}}, rotation = 0)));
        Modelica.Blocks.Sources.Ramp p_set(height = 20000.0, offset = 190000.0, duration = 8, startTime = 1) annotation(Placement(transformation(extent = {{ -50,40},{ -30,60}})));
        Modelica.Fluid.Pipes.StaticPipe pipe2(redeclare package Medium = Medium, diameter = 0.0254, length = 1000, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.NominalLaminarFlow(show_Res = true, dp_nominal = 100000, m_flow_nominal = 1)) annotation(Placement(transformation(extent = {{20, -70},{40, -50}}, rotation = 0)));
        Modelica.Fluid.Sources.Boundary_pT sink2(nPorts = 1, redeclare package Medium = Medium, p = 200000) annotation(Placement(transformation(extent = {{76, -66},{64, -54}}, rotation = 0)));

      equation 
        connect(orifice.port_b,sink.ports[1]) annotation(Line(points = {{40,20},{64,20}}, color = {0,127,255}, smooth = Smooth.None));
        connect(source.ports[1],pump.port_a) annotation(Line(points = {{ -64,20},{ -40,20}}, color = {0,127,255}, smooth = Smooth.None));
        connect(pump.port_b,orifice.port_a) annotation(Line(points = {{ -20,20},{20,20}}, color = {0,127,255}, smooth = Smooth.None));
        connect(pipe1.port_b,sink1.ports[1]) annotation(Line(points = {{40, -20},{64, -20}}, color = {0,127,255}, smooth = Smooth.None));
        connect(pipe1.port_a,pump.port_b) annotation(Line(points = {{20, -20},{0, -20},{0,20},{ -20,20}}, color = {0,127,255}, smooth = Smooth.None));
        connect(p_set.y,pump.p_set) annotation(Line(points = {{ -29,50},{ -25,50},{ -25,28.2}}, color = {0,0,127}, smooth = Smooth.None));
        annotation(__Dymola_Commands(file(ensureSimulated = true) = "Scripts/Fluid/InverseParameterization/plotResults.mos" "plotResults"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 10, NumberOfIntervals = 10000), Documentation(info = "<html>
<p>
A pump, an orifice and two pipes are parameterized with simple nominal values.
Note that pipe1 and pipe2 use the flowModel NominalTurbulentFlow and NominalLaminarFlow, respectively,
which do not require the specification of geometry data.
Instead pathLengths_nominal are obtained internally for given nominal pressure loss and nominal mass flow rate.
</p>
<p>
The pump controls a pressure ramp from 1.9 bar to 2.1 bar.
This causes an appropriate ramp on the mass flow rate of the orifice, which has a boundary pressure of 1 bar.
Flow reversal occurs in the pipes, which have a boundary pressure of 2 bar.
The Command plotResults can be used to see the pump speed N, which is controlled ideally to obtain the pressure ramp.
Moreover the internally obtained nominal design values that fulfill the nominal operating conditions as well as
the Reynolds number, m_flows_turbulent, and dps_fg_turbulent are plotted.
</p>
<p>
Note that the large value for pipe2.flowModel.pathLengths_nominal[1] is only meaningful under the made assumption of laminar flow,
 which is hardly possible for a real pipe.
</p>
<p>
Once the geometries have been designed, the NominalTurbulentPipeFlow correlations can easily be replaced with
TurbulentPipeFlow or DetailedPipeFlow correlations. Similarily the ControlledPump can be replaced with a PrescribedPump
to investigate a real controller or with a Pump with rotational shaft to investigate inertia effects.
</p>

<p align=\"center\">
<img src=\"../Images/Fluid/Examples/InverseParametrization.png\" border=\"1\">
</p>

</html>"));
        connect(pipe2.port_b,sink2.ports[1]) annotation(Line(points = {{40, -60},{64, -60}}, color = {0,127,255}, smooth = Smooth.None));
        connect(pump.port_b,pipe2.port_a) annotation(Line(points = {{ -20,20},{0,20},{0, -60},{20, -60}}, color = {0,127,255}, smooth = Smooth.None));
      end InverseParameterization;
      model IncompressibleFluidNetwork "Multi-way connections of pipes and incompressible medium model"
        extends Modelica.Icons.Example;
        replaceable package Medium = Modelica.Media.Incompressible.Examples.Essotherm650 extends Modelica.Media.Interfaces.PartialMedium;
        model Pipe = Modelica.Fluid.Pipes.DynamicPipe;
        Sources.Boundary_pT source(nPorts = 1, redeclare package Medium = Medium, p = 500000.0, T = 300) annotation(Placement(transformation(extent = {{ -98,4},{ -86,16}}, rotation = 0)));
        Pipe pipe1(use_T_start = true, length = 10, diameter = 0.025, redeclare package Medium = Medium, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow, p_a_start = 500000) annotation(Placement(transformation(extent = {{ -80,0},{ -60,20}}, rotation = 0)));
        Pipe pipe2(use_T_start = true, diameter = 0.025, redeclare package Medium = Medium, length = 0.5, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow, p_a_start = 500000) annotation(Placement(transformation(origin = { -50,30}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Pipe pipe3(use_T_start = true, diameter = 0.025, redeclare package Medium = Medium, length = 0.5, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow, p_a_start = 500000) annotation(Placement(transformation(origin = { -50, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        Pipe pipe4(use_T_start = true, diameter = 0.025, redeclare package Medium = Medium, length = 2, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow, p_a_start = 500000) annotation(Placement(transformation(extent = {{ -20, -40},{0, -20}}, rotation = 0)));
        Pipe pipe6(use_T_start = true, diameter = 0.025, redeclare package Medium = Medium, length = 20, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow, p_a_start = 500000) annotation(Placement(transformation(extent = {{20, -30},{40, -10}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valve1(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, m_flow_nominal = 1, rho_nominal = 1000, dp_nominal = 30000) annotation(Placement(transformation(extent = {{ -46,40},{ -26,60}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valve2(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, m_flow_nominal = 1, rho_nominal = 1000, dp_nominal = 30000) annotation(Placement(transformation(extent = {{ -46, -20},{ -26, -40}}, rotation = 0)));
        Pipe pipe7(use_T_start = true, length = 10, diameter = 0.025, redeclare package Medium = Medium, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow, p_a_start = 500000) annotation(Placement(transformation(extent = {{ -20,40},{0,60}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valve3(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, m_flow_nominal = 1, rho_nominal = 1000, dp_nominal = 30000) annotation(Placement(transformation(extent = {{60,0},{80,20}}, rotation = 0)));
        Sources.Boundary_pT sink(nPorts = 1, redeclare package Medium = Medium, T = 300, p = 100000.0) annotation(Placement(transformation(extent = {{98,4},{86,16}}, rotation = 0)));
        inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{70, -92},{90, -72}}, rotation = 0)));
        Modelica.Blocks.Sources.Step valveOpening1(height =  -0.2, offset = 1, startTime = 1) annotation(Placement(transformation(extent = {{ -80,70},{ -60,90}}, rotation = 0)));
        Modelica.Blocks.Sources.Step valveOpening2(height =  -0.2, offset = 1, startTime = 2) annotation(Placement(transformation(extent = {{ -80, -70},{ -60, -50}}, rotation = 0)));
        Modelica.Blocks.Sources.Step valveOpening3(height =  -0.2, offset = 1, startTime = 3) annotation(Placement(transformation(extent = {{40,70},{60,90}}, rotation = 0)));
        Pipe pipe8(use_T_start = true, length = 10, diameter = 0.025, redeclare package Medium = Medium, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow, p_a_start = 500000) annotation(Placement(transformation(origin = {10,30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        Pipe pipe9(use_T_start = true, length = 10, diameter = 0.025, redeclare package Medium = Medium, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow, p_a_start = 500000) annotation(Placement(transformation(extent = {{20,40},{40,60}}, rotation = 0)));
        Pipe pipe10(use_T_start = true, length = 10, diameter = 0.025, redeclare package Medium = Medium, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow, p_a_start = 500000) annotation(Placement(transformation(extent = {{20,0},{40,20}}, rotation = 0)));
        Pipe pipe5(use_T_start = true, diameter = 0.025, redeclare package Medium = Medium, length = 20, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow, p_a_start = 500000) annotation(Placement(transformation(extent = {{20, -60},{40, -40}}, rotation = 0)));

      equation 
        connect(source.ports[1],pipe1.port_a) annotation(Line(points = {{ -86,10},{ -80,10}}, color = {0,127,255}));
        connect(pipe1.port_b,pipe3.port_a) annotation(Line(points = {{ -60,10},{ -50,10},{ -50,0}}, color = {0,127,255}));
        connect(pipe1.port_b,pipe2.port_a) annotation(Line(points = {{ -60,10},{ -50,10},{ -50,20}}, color = {0,127,255}));
        connect(pipe2.port_b,valve1.port_a) annotation(Line(points = {{ -50,40},{ -50,50},{ -46,50}}, color = {0,127,255}));
        connect(valve2.port_b,pipe4.port_a) annotation(Line(points = {{ -26, -30},{ -26, -30},{ -20, -30}}, color = {0,127,255}));
        connect(pipe3.port_b,valve2.port_a) annotation(Line(points = {{ -50, -20},{ -50, -30},{ -46, -30}}, color = {0,127,255}));
        connect(valve1.port_b,pipe7.port_a) annotation(Line(points = {{ -26,50},{ -26,50},{ -20,50}}, color = {0,127,255}));
        connect(pipe6.port_b,valve3.port_a) annotation(Line(points = {{40, -20},{50, -20},{50,10},{60,10}}, color = {0,127,255}));
        connect(valve3.port_b,sink.ports[1]) annotation(Line(points = {{80,10},{80,10},{86,10}}, color = {0,127,255}));
        connect(valveOpening1.y,valve1.opening) annotation(Line(points = {{ -59,80},{ -36,80},{ -36,58}}, color = {0,0,127}));
        connect(valveOpening2.y,valve2.opening) annotation(Line(points = {{ -59, -60},{ -36, -60},{ -36, -38}}, color = {0,0,127}));
        connect(valveOpening3.y,valve3.opening) annotation(Line(points = {{61,80},{70,80},{70,18}}, color = {0,0,127}));
        connect(pipe7.port_b,pipe9.port_a) annotation(Line(points = {{0,50},{0,50},{20,50}}, color = {0,127,255}));
        connect(pipe7.port_b,pipe8.port_a) annotation(Line(points = {{0,50},{10,50},{10,40}}, color = {0,127,255}));
        connect(pipe9.port_b,valve3.port_a) annotation(Line(points = {{40,50},{50,50},{50,10},{60,10}}, color = {0,127,255}));
        connect(pipe8.port_b,pipe10.port_a) annotation(Line(points = {{10,20},{10,10},{20,10}}, color = {0,127,255}));
        connect(pipe10.port_b,valve3.port_a) annotation(Line(points = {{40,10},{60,10}}, color = {0,127,255}));
        connect(pipe4.port_b,pipe6.port_a) annotation(Line(points = {{0, -30},{10, -30},{10, -20},{20, -20}}, color = {0,127,255}, smooth = Smooth.None));
        connect(pipe8.port_b,pipe4.port_b) annotation(Line(points = {{10,20},{10, -30},{0, -30}}, color = {0,127,255}, smooth = Smooth.None));
        connect(pipe5.port_a,pipe4.port_b) annotation(Line(points = {{20, -50},{10, -50},{10, -30},{0, -30}}, color = {0,127,255}, smooth = Smooth.None));
        connect(pipe5.port_b,valve3.port_a) annotation(Line(points = {{40, -50},{50, -50},{50,10},{60,10}}, color = {0,127,255}, smooth = Smooth.None));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
This example demonstrates two aspects: the efficient treatment of multi-way connections
and the usage of an incompressible medium model.
<p>
Normally one would expect bad equation systems in multi-way connections
and possibly introduce mixing volumes to work around this.
Here the problem is treated with the the modelStructure=av_vb in the
<a href=\"Modelica://Modelica.Fluid.Pipes.DynamicPipe\">DynamicPipe</a> model.
Each pipe exposes the states of the outer fluid segments to the respective fluid ports.
Consequently the pressures of all connected pipe segments get lumped together into one mass balance
spanning the whole connection set. With the stream concept in the fluid ports, the energy and substance
balances remain independent in the connected pipe segments.
</p>
<p>
The model does not contain pressure dynamics as an incompressible medium is used (Essotherm650).
Pressure dynamics becomes present with a compressible medium model (e.g. StandardWater).
</p>

<p align=\"center\">
<img src=\"../Images/Fluid/Examples/IncompressibleFluidNetwork.png\" border=\"1\">
</p>

</html>"), experiment(StopTime = 5), __Dymola_Commands(file = "Scripts/Fluid/IncompressibleFluidNetwork/plotJunctionPressuresAndValves.mos" "Plot junction pressures and valves"));
      end IncompressibleFluidNetwork;
      model HeatingSystem "Simple model of a heating system"
        extends Modelica.Icons.Example;
        replaceable package Medium = Modelica.Media.Water.StandardWater extends Modelica.Media.Interfaces.PartialMedium;
        Modelica.Fluid.Vessels.OpenTank tank(redeclare package Medium = Medium, crossArea = 0.01, height = 2, level_start = 1, nPorts = 2, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, use_HeatTransfer = true, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.01),Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.01)}, redeclare model HeatTransfer = Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.IdealHeatTransfer(k = 10)) annotation(Placement(transformation(extent = {{ -80,30},{ -60,50}}, rotation = 0)));
        Machines.ControlledPump pump(redeclare package Medium = Medium, N_nominal = 1500, use_T_start = true, T_start = Modelica.SIunits.Conversions.from_degC(40), m_flow_start = 0.01, m_flow_nominal = 0.01, control_m_flow = false, allowFlowReversal = false, p_a_start = 110000, p_b_start = 130000, p_a_nominal = 110000, p_b_nominal = 130000) annotation(Placement(transformation(extent = {{ -50,10},{ -30,30}}, rotation = 0)));
        Modelica.Fluid.Valves.ValveIncompressible valve(redeclare package Medium = Medium, CvData = Modelica.Fluid.Types.CvTypes.OpPoint, m_flow_nominal = 0.01, show_T = true, allowFlowReversal = false, dp_nominal = 10000) annotation(Placement(transformation(extent = {{60, -80},{40, -60}}, rotation = 0)));
      protected 
        Modelica.Blocks.Interfaces.RealOutput m_flow annotation(Placement(transformation(extent = {{ -6,34},{6,46}}, rotation = 0)));
      public 
        Sensors.MassFlowRate sensor_m_flow(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -20,10},{0,30}}, rotation = 0)));
        Modelica.Thermal.HeatTransfer.Sources.FixedTemperature T_ambient(T = system.T_ambient) annotation(Placement(transformation(extent = {{ -14, -27},{0, -13}}, rotation = 0)));
        Modelica.Thermal.HeatTransfer.Components.ThermalConductor wall(G = 1600.0 / 20) annotation(Placement(transformation(origin = {10, -48}, extent = {{8, -10},{ -8,10}}, rotation = 90)));
        Modelica.Thermal.HeatTransfer.Sources.FixedHeatFlow burner(Q_flow = 1600.0, T_ref = 343.15, alpha =  -0.5) annotation(Placement(transformation(extent = {{16,30},{36,50}}, rotation = 0)));
        inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.SteadyStateInitial, m_flow_small = 0.0001) annotation(Placement(transformation(extent = {{ -90,70},{ -70,90}}, rotation = 0)));
        Pipes.DynamicPipe heater(redeclare package Medium = Medium, use_T_start = true, T_start = Modelica.SIunits.Conversions.from_degC(80), length = 2, redeclare model HeatTransfer = Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.IdealFlowHeatTransfer, diameter = 0.01, nNodes = 1, modelStructure = Modelica.Fluid.Types.ModelStructure.a_vb, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow, use_HeatTransfer = true) annotation(Placement(transformation(extent = {{30,10},{50,30}}, rotation = 0)));
        Pipes.DynamicPipe radiator(use_T_start = true, redeclare package Medium = Medium, length = 10, T_start = Modelica.SIunits.Conversions.from_degC(40), redeclare model HeatTransfer = Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.IdealFlowHeatTransfer, diameter = 0.01, nNodes = 1, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow, modelStructure = Modelica.Fluid.Types.ModelStructure.av_b, use_HeatTransfer = true) annotation(Placement(transformation(extent = {{20, -80},{0, -60}}, rotation = 0)));
      protected 
        Modelica.Blocks.Interfaces.RealOutput T_forward annotation(Placement(transformation(extent = {{74,34},{86,46}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput T_return annotation(Placement(transformation(extent = {{ -46, -56},{ -58, -44}}, rotation = 0)));
      public 
        Modelica.Fluid.Sensors.Temperature sensor_T_forward(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{50,30},{70,50}}, rotation = 0)));
        Modelica.Fluid.Sensors.Temperature sensor_T_return(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -20, -60},{ -40, -40}}, rotation = 0)));
      protected 
        Modelica.Blocks.Interfaces.RealOutput tankLevel annotation(Placement(transformation(extent = {{ -56,34},{ -44,46}}, rotation = 0)));
      public 
        Modelica.Blocks.Sources.Step handle(startTime = 2000, height = 0.9, offset = 0.1) annotation(Placement(transformation(extent = {{26, -27},{40, -13}}, rotation = 0)));
        Pipes.DynamicPipe pipe(redeclare package Medium = Medium, use_T_start = true, T_start = Modelica.SIunits.Conversions.from_degC(80), redeclare model HeatTransfer = Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.IdealFlowHeatTransfer, diameter = 0.01, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow, length = 10) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation =  -90, origin = {80, -20})));

      equation 
        tankLevel = tank.level;
        connect(sensor_m_flow.m_flow,m_flow) annotation(Line(points = {{ -10,31},{ -10,40},{0,40}}, color = {0,0,127}));
        connect(sensor_m_flow.port_b,heater.port_a) annotation(Line(points = {{0,20},{0,20},{30,20}}, color = {0,127,255}));
        connect(T_ambient.port,wall.port_a) annotation(Line(points = {{1.77636e-15, -20},{10, -20},{10, -40}}, color = {191,0,0}));
        connect(sensor_T_forward.T,T_forward) annotation(Line(points = {{67,40},{80,40}}, color = {0,0,127}));
        connect(radiator.port_a,valve.port_b) annotation(Line(points = {{20, -70},{20, -70},{40, -70}}, color = {0,127,255}));
        connect(sensor_T_return.port,radiator.port_b) annotation(Line(points = {{ -30, -60},{ -30, -70},{0, -70}}, color = {0,127,255}));
        connect(tank.ports[2],pump.port_a) annotation(Line(points = {{ -68,30},{ -68,20},{ -50,20}}, color = {0,127,255}, smooth = Smooth.None));
        connect(handle.y,valve.opening) annotation(Line(points = {{40.7, -20},{50, -20},{50, -62}}, color = {0,0,127}, smooth = Smooth.None));
        connect(pump.port_b,sensor_m_flow.port_a) annotation(Line(points = {{ -30,20},{ -20,20}}, color = {0,127,255}, smooth = Smooth.None));
        connect(sensor_T_return.T,T_return) annotation(Line(points = {{ -37, -50},{ -52, -50}}, color = {0,0,127}, smooth = Smooth.None));
        connect(burner.port,heater.heatPorts[1]) annotation(Line(points = {{36,40},{40.1,40},{40.1,24.4}}, color = {191,0,0}, smooth = Smooth.None));
        connect(wall.port_b,radiator.heatPorts[1]) annotation(Line(points = {{10, -56},{10, -65.6},{9.9, -65.6}}, color = {191,0,0}, smooth = Smooth.None));
        connect(sensor_T_forward.port,heater.port_b) annotation(Line(points = {{60,30},{60,20},{50,20}}, color = {0,127,255}, smooth = Smooth.None));
        connect(heater.port_b,pipe.port_a) annotation(Line(points = {{50,20},{80,20},{80, -10}}, color = {0,127,255}, smooth = Smooth.None));
        connect(pipe.port_b,valve.port_a) annotation(Line(points = {{80, -30},{80, -70},{60, -70}}, color = {0,127,255}, smooth = Smooth.None));
        connect(radiator.port_b,tank.ports[1]) annotation(Line(points = {{0, -70},{ -72, -70},{ -72,30}}, color = {0,127,255}, smooth = Smooth.None));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
Simple heating system with a closed flow cycle. It is set up for steady-state initial values.
After 2000s of simulation time the valve fully opens. A simple idealized control is embedded
into the respective components, so that the heating system can be regulated with the valve:
the pump controls the pressure, the burner controls the temperature.
</p>
<p>
One can investigate the temperatures and flows for different settings of <tt>system.energyDynamics</tt>
(see Assumptions tab of the system object).
With <tt>system.energyDynamics==Types.Dynamics.SteadyState</tt> all but one dynamic states are eliminated.
The left state <tt>tank.m</tt> is to account for the closed flow cycle. It is constant as outflow and inflow are equal
in a steady-state simulation.
</p>
<p>
Note that a closed flow cycle generally causes circular equalities for the mass flow rates and leaves the pressure undefined.
This is why the tank.massDynamics, i.e. the tank level determining the port pressure, is modified locally to Types.Dynamics.FixedInitial.
</p>
<p>
Also note that the tank is thermally isolated againts its ambient. This way the temperature of the tank is also
well defined for zero flow rate in the heating system, e.g. for valveOpening.offset=0 at the beginning of a simulation.
The pipe however is assumed to be perfectly isolated.
If steady-state values shall be obtained with the valve fully closed, then a thermal
coupling between the pipe and its ambient should be defined as well.
</p>
<p>
Moreover it is worth noting that the idealized direct connection between the heater and the pipe, resulting in equal port pressures,
is treated as high-index DAE, as opposed to a nonlinear equation system for connected pressure loss correlations. A pressure loss correlation
could be additionally introduced to model the fitting between the heater and the pipe, e.g. to adapt different diameters.
</p>

<p align=\"center\">
<img src=\"../Images/Fluid/Examples/HeatingSystem.png\" border=\"1\">
</p>

</html>
"), experiment(StopTime = 6000), __Dymola_Commands(file(ensureSimulated = true) = "Scripts/Fluid/HeatingSystem/plotResults.mos" "plotResults"));
      end HeatingSystem;
      package HeatExchanger "Demo of a heat exchanger model"
        model HeatExchangerSimulation "simulation for the heat exchanger model"
          extends Modelica.Icons.Example;
          replaceable package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater;
          Modelica.Fluid.Examples.HeatExchanger.BaseClasses.BasicHX HEX(c_wall = 500, use_T_start = true, nNodes = 20, length = 2, m_flow_start_1 = 0.2, m_flow_start_2 = 0.2, k_wall = 100, energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, massDynamics = Modelica.Fluid.Types.Dynamics.SteadyStateInitial, s_wall = 0.005, crossArea_1 = 0.00045, crossArea_2 = 0.00045, perimeter_1 = 0.075, perimeter_2 = 0.075, area_h_1 = 0.075 * 2 * 20, area_h_2 = 0.075 * 2 * 20, rho_wall = 900, redeclare package Medium_1 = Medium, redeclare package Medium_2 = Medium, redeclare model HeatTransfer_1 = Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.ConstantFlowHeatTransfer(alpha0 = 1000), redeclare model HeatTransfer_2 = Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.ConstantFlowHeatTransfer(alpha0 = 200), Twall_start = 300, dT = 10, T_start_1 = 304, T_start_2 = 300) annotation(Placement(transformation(extent = {{ -26, -14},{34,46}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT ambient2(nPorts = 1, p = 100000.0, T = 280, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{82, -28},{62, -8}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT ambient1(nPorts = 1, p = 100000.0, T = 300, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{82,24},{62,44}}, rotation = 0)));
          Modelica.Fluid.Sources.MassFlowSource_T massFlowRate2(nPorts = 1, m_flow = 0.2, T = 360, redeclare package Medium = Medium, use_m_flow_in = true, use_T_in = false, use_X_in = false) annotation(Placement(transformation(extent = {{ -66,24},{ -46,44}}, rotation = 0)));
          Modelica.Fluid.Sources.MassFlowSource_T massFlowRate1(nPorts = 1, T = 300, m_flow = 0.5, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -66, -10},{ -46,10}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 100, Tolerance = 1e-05), Documentation(info = "<html>
<p align=\"center\">
<img src=\"../Images/Fluid/Examples/HeatExchanger.png\" border=\"1\">
</p>
</html>"));
          Modelica.Blocks.Sources.Ramp Ramp1(startTime = 50, duration = 5, height =  -1, offset = 0.5) annotation(Placement(transformation(extent = {{ -98,24},{ -78,44}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{60,70},{80,90}}, rotation = 0)));

        equation 
          connect(massFlowRate1.ports[1],HEX.port_a1) annotation(Line(points = {{ -46,0},{ -40,0},{ -40,15.4},{ -29,15.4}}, color = {0,127,255}));
          connect(HEX.port_b1,ambient1.ports[1]) annotation(Line(points = {{37,15.4},{48.5,15.4},{48.5,34},{62,34}}, color = {0,127,255}));
          connect(Ramp1.y,massFlowRate2.m_flow_in) annotation(Line(points = {{ -77,34},{ -74,34},{ -74,42},{ -66,42}}, color = {0,0,127}));
          connect(massFlowRate2.ports[1],HEX.port_b2) annotation(Line(points = {{ -46,34},{ -40,34},{ -40,29.8},{ -29,29.8}}, color = {0,127,255}, smooth = Smooth.None));
          connect(HEX.port_a2,ambient2.ports[1]) annotation(Line(points = {{37,2.2},{42,2},{50,2},{50, -18},{62, -18}}, color = {0,127,255}, smooth = Smooth.None));
        end HeatExchangerSimulation;
        package BaseClasses "Additional models for heat exchangers"
          extends Modelica.Fluid.Icons.BaseClassLibrary;
          model BasicHX "Simple heat exchanger model"
            outer Modelica.Fluid.System system "System properties";
            parameter Integer nNodes(min = 1) = 2 "Spatial segmentation";
            replaceable package Medium_1 = Modelica.Media.Water.StandardWater extends Modelica.Media.Interfaces.PartialMedium "Fluid 1" annotation(choicesAllMatching, Dialog(tab = "General", group = "Fluid 1"));
            replaceable package Medium_2 = Modelica.Media.Water.StandardWater extends Modelica.Media.Interfaces.PartialMedium "Fluid 2" annotation(choicesAllMatching, Dialog(tab = "General", group = "Fluid 2"));
            parameter SI.Area crossArea_1 "Cross sectional area" annotation(Dialog(tab = "General", group = "Fluid 1"));
            parameter SI.Area crossArea_2 "Cross sectional area" annotation(Dialog(tab = "General", group = "Fluid 2"));
            parameter SI.Length perimeter_1 "Flow channel perimeter" annotation(Dialog(tab = "General", group = "Fluid 1"));
            parameter SI.Length perimeter_2 "Flow channel perimeter" annotation(Dialog(tab = "General", group = "Fluid 2"));
            parameter SI.Length length(min = 0) "Length of flow path for both fluids";
            parameter SI.Length s_wall(min = 0) "Wall thickness";
            parameter Boolean use_HeatTransfer = false "= true to use the HeatTransfer_1/_2 model";
            replaceable model HeatTransfer_1 = Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.IdealFlowHeatTransfer extends Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.PartialFlowHeatTransfer "Heat transfer model" annotation(choicesAllMatching, Dialog(tab = "General", group = "Fluid 1", enable = use_HeatTransfer));
            replaceable model HeatTransfer_2 = Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.IdealFlowHeatTransfer extends Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.PartialFlowHeatTransfer "Heat transfer model" annotation(choicesAllMatching, Dialog(tab = "General", group = "Fluid 2", enable = use_HeatTransfer));
            parameter SI.Area area_h_1 "Heat transfer area" annotation(Dialog(tab = "General", group = "Fluid 1"));
            parameter SI.Area area_h_2 "Heat transfer area" annotation(Dialog(tab = "General", group = "Fluid 2"));
            parameter SI.Density rho_wall "Density of wall material" annotation(Dialog(tab = "General", group = "Solid material properties"));
            parameter SI.SpecificHeatCapacity c_wall "Specific heat capacity of wall material" annotation(Dialog(tab = "General", group = "Solid material properties"));
            final parameter SI.Area area_h = (area_h_1 + area_h_2) / 2 "Heat transfer area";
            final parameter SI.Mass m_wall = rho_wall * area_h * s_wall "Wall mass";
            parameter SI.ThermalConductivity k_wall "Thermal conductivity of wall material" annotation(Dialog(group = "Solid material properties"));
            parameter Boolean allowFlowReversal = system.allowFlowReversal "allow flow reversal, false restricts to design direction (port_a -> port_b)" annotation(Dialog(tab = "Assumptions"), Evaluate = true);
            parameter Types.Dynamics energyDynamics = system.energyDynamics "Formulation of energy balance" annotation(Evaluate = true, Dialog(tab = "Assumptions", group = "Dynamics"));
            parameter Types.Dynamics massDynamics = system.massDynamics "Formulation of mass balance" annotation(Evaluate = true, Dialog(tab = "Assumptions", group = "Dynamics"));
            parameter Types.Dynamics momentumDynamics = system.momentumDynamics "Formulation of momentum balance, if pressureLoss options available" annotation(Evaluate = true, Dialog(tab = "Assumptions", group = "Dynamics"));
            parameter SI.Temperature Twall_start "Start value of wall temperature" annotation(Dialog(tab = "Initialization", group = "Wall"));
            parameter SI.Temperature dT "Start value for pipe_1.T - pipe_2.T" annotation(Dialog(tab = "Initialization", group = "Wall"));
            parameter Boolean use_T_start = true "Use T_start if true, otherwise h_start" annotation(Evaluate = true, Dialog(tab = "Initialization"));
            parameter Medium_1.AbsolutePressure p_a_start1 = Medium_1.p_default "Start value of pressure" annotation(Dialog(tab = "Initialization", group = "Fluid 1"));
            parameter Medium_1.AbsolutePressure p_b_start1 = Medium_1.p_default "Start value of pressure" annotation(Dialog(tab = "Initialization", group = "Fluid 1"));
            parameter Medium_1.Temperature T_start_1 = if use_T_start then Medium_1.T_default else Medium_1.temperature_phX((p_a_start1 + p_b_start1) / 2, h_start_1, X_start_1) "Start value of temperature" annotation(Evaluate = true, Dialog(tab = "Initialization", group = "Fluid 1", enable = use_T_start));
            parameter Medium_1.SpecificEnthalpy h_start_1 = if use_T_start then Medium_1.specificEnthalpy_pTX((p_a_start1 + p_b_start1) / 2, T_start_1, X_start_1[1:Medium_1.nXi]) else Medium_1.h_default "Start value of specific enthalpy" annotation(Evaluate = true, Dialog(tab = "Initialization", group = "Fluid 1", enable = not use_T_start));
            parameter Medium_1.MassFraction X_start_1[Medium_1.nX] = Medium_1.X_default "Start value of mass fractions m_i/m" annotation(Dialog(tab = "Initialization", group = "Fluid 1", enable = Medium_1.nXi > 0));
            parameter Medium_1.MassFlowRate m_flow_start_1 = system.m_flow_start "Start value of mass flow rate" annotation(Evaluate = true, Dialog(tab = "Initialization", group = "Fluid 1"));
            parameter Medium_2.AbsolutePressure p_a_start2 = Medium_2.p_default "Start value of pressure" annotation(Dialog(tab = "Initialization", group = "Fluid 2"));
            parameter Medium_2.AbsolutePressure p_b_start2 = Medium_2.p_default "Start value of pressure" annotation(Dialog(tab = "Initialization", group = "Fluid 2"));
            parameter Medium_2.Temperature T_start_2 = if use_T_start then Medium_2.T_default else Medium_2.temperature_phX((p_a_start2 + p_b_start2) / 2, h_start_2, X_start_2) "Start value of temperature" annotation(Evaluate = true, Dialog(tab = "Initialization", group = "Fluid 2", enable = use_T_start));
            parameter Medium_2.SpecificEnthalpy h_start_2 = if use_T_start then Medium_2.specificEnthalpy_pTX((p_a_start2 + p_b_start2) / 2, T_start_2, X_start_2[1:Medium_2.nXi]) else Medium_2.h_default "Start value of specific enthalpy" annotation(Evaluate = true, Dialog(tab = "Initialization", group = "Fluid 2", enable = not use_T_start));
            parameter Medium_2.MassFraction X_start_2[Medium_2.nX] = Medium_2.X_default "Start value of mass fractions m_i/m" annotation(Dialog(tab = "Initialization", group = "Fluid 2", enable = Medium_2.nXi > 0));
            parameter Medium_2.MassFlowRate m_flow_start_2 = system.m_flow_start "Start value of mass flow rate" annotation(Evaluate = true, Dialog(tab = "Initialization", group = "Fluid 2"));
            replaceable model FlowModel_1 = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow extends Modelica.Fluid.Pipes.BaseClasses.FlowModels.PartialStaggeredFlowModel "Characteristic of wall friction" annotation(choicesAllMatching, Dialog(tab = "General", group = "Fluid 1"));
            replaceable model FlowModel_2 = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow extends Modelica.Fluid.Pipes.BaseClasses.FlowModels.PartialStaggeredFlowModel "Characteristic of wall friction" annotation(choicesAllMatching, Dialog(tab = "General", group = "Fluid 2"));
            parameter SI.Length roughness_1 = 2.5e-05 "Absolute roughness of pipe (default = smooth steel pipe)" annotation(Dialog(tab = "General", group = "Fluid 1"));
            parameter SI.Length roughness_2 = 2.5e-05 "Absolute roughness of pipe (default = smooth steel pipe)" annotation(Dialog(tab = "General", group = "Fluid 2"));
            SI.HeatFlowRate Q_flow_1 "Total heat flow rate of pipe 1";
            SI.HeatFlowRate Q_flow_2 "Total heat flow rate of pipe 2";
            BaseClasses.WallConstProps wall(rho_wall = rho_wall, c_wall = c_wall, T_start = Twall_start, k_wall = k_wall, dT = dT, s = s_wall, energyDynamics = energyDynamics, n = nNodes, area_h = (crossArea_1 + crossArea_2) / 2) annotation(Placement(transformation(extent = {{ -29, -23},{9,35}}, rotation = 0)));
            Modelica.Fluid.Pipes.DynamicPipe pipe_1(redeclare final package Medium = Medium_1, final isCircular = false, final diameter = 0, final nNodes = nNodes, final allowFlowReversal = allowFlowReversal, final energyDynamics = energyDynamics, final massDynamics = massDynamics, final momentumDynamics = momentumDynamics, final length = length, final use_HeatTransfer = use_HeatTransfer, redeclare final model HeatTransfer = HeatTransfer_1, final use_T_start = use_T_start, final T_start = T_start_1, final h_start = h_start_1, final X_start = X_start_1, final m_flow_start = m_flow_start_1, final perimeter = perimeter_1, final crossArea = crossArea_1, final roughness = roughness_1, redeclare final model FlowModel = FlowModel_1) annotation(Placement(transformation(extent = {{ -40, -80},{20, -20}}, rotation = 0)));
            Modelica.Fluid.Pipes.DynamicPipe pipe_2(redeclare final package Medium = Medium_2, final nNodes = nNodes, final allowFlowReversal = allowFlowReversal, final energyDynamics = energyDynamics, final massDynamics = massDynamics, final momentumDynamics = momentumDynamics, final length = length, final isCircular = false, final diameter = 0, final use_HeatTransfer = use_HeatTransfer, redeclare final model HeatTransfer = HeatTransfer_2, final use_T_start = use_T_start, final T_start = T_start_2, final h_start = h_start_2, final X_start = X_start_2, final m_flow_start = m_flow_start_2, final perimeter = perimeter_2, final crossArea = crossArea_2, final p_a_start = p_a_start1, final p_b_start = p_b_start2, final roughness = roughness_2, redeclare final model FlowModel = FlowModel_2) annotation(Placement(transformation(extent = {{20,88},{ -40,28}}, rotation = 0)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100, -26},{100, -30}}, lineColor = {0,0,0}, fillColor = {95,95,95}, fillPattern = FillPattern.Forward),Rectangle(extent = {{ -100,30},{100,26}}, lineColor = {0,0,0}, fillColor = {95,95,95}, fillPattern = FillPattern.Forward),Rectangle(extent = {{ -100,60},{100,30}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,63,125}),Rectangle(extent = {{ -100, -30},{100, -60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,63,125}),Rectangle(extent = {{ -100,26},{100, -26}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,128,255}),Text(extent = {{ -150,110},{150,70}}, lineColor = {0,0,255}, textString = "%name"),Line(points = {{30, -85},{ -60, -85}}, color = {0,128,255}, smooth = Smooth.None),Polygon(points = {{20, -70},{60, -85},{20, -100},{20, -70}}, lineColor = {0,128,255}, smooth = Smooth.None, fillColor = {0,128,255}, fillPattern = FillPattern.Solid),Line(points = {{30,77},{ -60,77}}, color = {0,128,255}, smooth = Smooth.None),Polygon(points = {{ -50,92},{ -90,77},{ -50,62},{ -50,92}}, lineColor = {0,128,255}, smooth = Smooth.None, fillColor = {0,128,255}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
Simple model of a heat exchanger consisting of two pipes and one wall in between.
For both fluids geometry parameters, such as heat transfer area and cross section as well as heat transfer and pressure drop correlations may be chosen.
The flow scheme may be concurrent or counterflow, defined by the respective flow directions of the fluids entering the component.
The design flow direction with positive m_flow variables is counterflow.
</html>"));
            Modelica.Fluid.Interfaces.FluidPort_b port_b1(redeclare final package Medium = Medium_1) annotation(Placement(transformation(extent = {{100, -12},{120,8}}, rotation = 0)));
            Modelica.Fluid.Interfaces.FluidPort_a port_a1(redeclare final package Medium = Medium_1) annotation(Placement(transformation(extent = {{ -120, -12},{ -100,8}}, rotation = 0)));
            Modelica.Fluid.Interfaces.FluidPort_b port_b2(redeclare final package Medium = Medium_2) annotation(Placement(transformation(extent = {{ -120,36},{ -100,56}}, rotation = 0)));
            Modelica.Fluid.Interfaces.FluidPort_a port_a2(redeclare final package Medium = Medium_2) annotation(Placement(transformation(extent = {{100, -56},{120, -36}}, rotation = 0)));

          equation 
            Q_flow_1 = sum(pipe_1.heatTransfer.Q_flows);
            Q_flow_2 = sum(pipe_2.heatTransfer.Q_flows);
            connect(pipe_2.port_b,port_b2) annotation(Line(points = {{ -40,58},{ -76,58},{ -76,46},{ -110,46}}, color = {0,127,255}, thickness = 0.5));
            connect(pipe_1.port_b,port_b1) annotation(Line(points = {{20, -50},{42, -50},{42, -2},{110, -2}}, color = {0,127,255}, thickness = 0.5));
            connect(pipe_1.port_a,port_a1) annotation(Line(points = {{ -40, -50},{ -75.3, -50},{ -75.3, -2},{ -110, -2}}, color = {0,127,255}, thickness = 0.5));
            connect(pipe_2.port_a,port_a2) annotation(Line(points = {{20,58},{65,58},{65, -46},{110, -46}}, color = {0,127,255}, thickness = 0.5));
            connect(wall.heatPort_b,pipe_1.heatPorts) annotation(Line(points = {{ -10, -8.5},{ -10, -36.8},{ -9.7, -36.8}}, color = {191,0,0}, smooth = Smooth.None));
            connect(pipe_2.heatPorts[nNodes: -1:1],wall.heatPort_a[1:nNodes]) annotation(Line(points = {{ -10.3,44.8},{ -10.3,31.7},{ -10,31.7},{ -10,20.5}}, color = {127,0,0}, smooth = Smooth.None));
          end BasicHX;
          model WallConstProps "Pipe wall with capacitance, assuming 1D heat conduction and constant material properties"
            parameter Integer n(min = 1) = 1 "Segmentation perpendicular to heat conduction";
            parameter SI.Length s "Wall thickness";
            parameter SI.Area area_h "Heat transfer area";
            parameter SI.Density rho_wall "Density of wall material";
            parameter SI.SpecificHeatCapacity c_wall "Specific heat capacity of wall material";
            parameter SI.ThermalConductivity k_wall "Thermal conductivity of wall material";
            parameter SI.Mass[n] m = fill((rho_wall * area_h * s) / n, n) "Distribution of wall mass";
            outer Modelica.Fluid.System system;
            parameter Types.Dynamics energyDynamics = system.energyDynamics "Formulation of energy balance" annotation(Evaluate = true, Dialog(tab = "Assumptions", group = "Dynamics"));
            parameter SI.Temperature T_start "Wall temperature start value";
            parameter SI.Temperature dT "Start value for port_b.T - port_a.T";
            SI.Temperature[n] Tb(each start = T_start + 0.5 * dT);
            SI.Temperature[n] Ta(each start = T_start - 0.5 * dT);
            SI.Temperature[n] T(start = ones(n) * T_start, stateSelect = StateSelect.prefer) "Wall temperature";
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a[n] heatPort_a "Thermal port" annotation(Placement(transformation(extent = {{ -20,40},{20,60}}, rotation = 0)));
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a[n] heatPort_b "Thermal port" annotation(Placement(transformation(extent = {{ -20, -40},{20, -60}}, rotation = 0)));

          initial equation 
            if energyDynamics == Types.Dynamics.SteadyState then
              der(T) = zeros(n);

            elseif energyDynamics == Types.Dynamics.FixedInitial then
              T = ones(n) * T_start;


            else

            end if;

          equation 
            for i in 1:n loop
            assert(m[i] > 0, "Wall has negative dimensions");
            if energyDynamics == Types.Dynamics.SteadyState then
              0 = heatPort_a[i].Q_flow + heatPort_b[i].Q_flow;

            else
              c_wall * m[i] * der(T[i]) = heatPort_a[i].Q_flow + heatPort_b[i].Q_flow;

            end if;
            heatPort_a[i].Q_flow = (k_wall / s * (Ta[i] - T[i]) * area_h) / n;
            heatPort_b[i].Q_flow = (k_wall / s * (Tb[i] - T[i]) * area_h) / n;

            end for;
            Ta = heatPort_a.T;
            Tb = heatPort_b.T;
            annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,40},{100, -40}}, lineColor = {0,0,0}, fillColor = {95,95,95}, fillPattern = FillPattern.Forward),Text(extent = {{ -82,18},{76, -18}}, lineColor = {0,0,0}, textString = "%name")}), Documentation(revisions = "<html>
<ul>
<li><i>04 Mar 2006</i>
    by Katrin Pr&ouml;l&szlig;:<br>
       Model added to the Fluid library</li>
</ul>
</html>", info = "<html>
Simple model of circular (or any other closed shape) wall to be used for pipe (or duct) models. Heat conduction is regarded one dimensional, capacitance is lumped at the arithmetic mean temperature. The spatial discretization (parameter <tt>n</tt>) is meant to correspond to a connected fluid model discretization.
</html>"));
          end WallConstProps;
        end BaseClasses;
      end HeatExchanger;
      package Explanatory "A set of examples illustrating when special attention has to be paid"
        model MeasuringTemperature "Differences between using one port with
   and without explicit junction model and two port sensors for fluid temperature meassuring"
          extends Modelica.Icons.Example;
          Modelica.Fluid.Sensors.Temperature T_onePort(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{ -20,40},{0,60}}, rotation = 0)));
          Modelica.Fluid.Sensors.TemperatureTwoPort T_twoPort(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{ -20,0},{0, -20}}, rotation = 0)));
          inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial) annotation(Placement(transformation(extent = {{ -100,56},{ -80,76}}, rotation = 0)));
          Modelica.Fluid.Vessels.OpenTank openTankCold2(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, height = 2, crossArea = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05)}, level_start = 0.975, T_start = 293.15) annotation(Placement(transformation(extent = {{20,6},{40,26}}, rotation = 0)));
          Modelica.Fluid.Vessels.OpenTank openTankCold1(redeclare package Medium = Modelica.Media.Water.StandardWater, height = 2, crossArea = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05)}, nPorts = 1, level_start = 0.975, T_start = 293.15) annotation(Placement(transformation(extent = {{20,60},{40,80}}, rotation = 0)));
          Modelica.Fluid.Vessels.OpenTank openTankHot1(nPorts = 1, level_start = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, height = 2, crossArea = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05)}, T_start = 353.15) annotation(Placement(transformation(extent = {{60,40},{80,60}}, rotation = 0)));
          Modelica.Fluid.Vessels.OpenTank openTankHot2(nPorts = 1, level_start = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, height = 2, crossArea = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05)}, T_start = 353.15) annotation(Placement(transformation(extent = {{60, -10},{80,10}}, rotation = 0)));
          Modelica.Fluid.Sources.MassFlowSource_T mFlow1(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, use_m_flow_in = true, use_T_in = false, T = 323.15) annotation(Placement(transformation(extent = {{ -60,30},{ -40,50}}, rotation = 0)));
          Modelica.Fluid.Sources.MassFlowSource_T mFlow2(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, use_m_flow_in = true, T = 323.15) annotation(Placement(transformation(extent = {{ -60, -20},{ -40,0}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{42,52},{62,46}}, lineColor = {0,0,0}, textString = "T=80"),Text(extent = {{ -4,76},{18,70}}, lineColor = {0,0,0}, textString = "T=20"),Text(extent = {{ -62,20},{ -36,12}}, lineColor = {0,0,0}, textString = "T=50")}), Documentation(info = "<html>
<p>
This model demonstrates the differences that occur when using
one- and two-port temperature sensors with and without explicit junction models.
As shown in the next figure, the same system is shown in 3 different variations.
In all cases exactly the same fluid system is defined. The only difference is
how the temperature is measured:
</p>

<blockquote>
<img src=\"../Images/Fluid/Examples/MeasuringTemperature1.png\">
</blockquote>

<p>
A pre-defined mass flow rate is present so that fluid flows from the reservoir to the 
tanks and after 0.5 s the mass flows from the tanks to the reservoir.
The reservoir has a temperature of 50<sup>0</sup>C whereas the tanks have an 
initial temperatur of 20<sup>0</sup>C and of 80<sup>0</sup>C. The initial height of the
tanks is made in such a form that fluid always flows out of the cold tank.
When the fluid flows from the reservoir to the tanks, then it mixes with the
cold tank and enters the hot tank.
When the fluid flow from the tanks to the reservoir, then the cold and hot water
from the two tanks first mixes and the flows to the reservoir.
</p>

<p>
A one-port sensor measures the <em>mixing</em> temperature at a connection point.
Therefore T_onePort.T (the blue curve in the figure below) is the
temperature of the mixing point.
A two-port sensor measures the temperature at the <em>upstream</em> side. 
Therefore T_twoPort.T (the red curve in the figure below which is identical
to the green curve) shows first the temperature of the reservoir and then
the mixing temperature when fluid flows from the tanks to the reservoir.
The same is measured with T_junction.T (the green curve below), because
the one-port sensor is connected between the mass flow source and the junction
and since the mixing takes place in the junction, the same situation is
present as for T_twoPort.T.
</p>

<blockquote>
<img src=\"../Images/Fluid/Examples/MeasuringTemperature2.png\">
</blockquote>

</html>"));
          Modelica.Blocks.Sources.Sine sine(freqHz = 1) annotation(Placement(transformation(extent = {{ -100,10},{ -80,30}}, rotation = 0)));
          Modelica.Fluid.Sensors.Temperature T_junction(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{ -20, -80},{0, -60}}, rotation = 0)));
          Modelica.Fluid.Vessels.OpenTank openTankCold3(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, height = 2, crossArea = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05)}, level_start = 0.975, T_start = 293.15) annotation(Placement(transformation(extent = {{20, -60},{40, -40}}, rotation = 0)));
          Modelica.Fluid.Vessels.OpenTank openTankHot3(nPorts = 1, level_start = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, height = 2, crossArea = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05)}, T_start = 353.15) annotation(Placement(transformation(extent = {{60, -80},{80, -60}}, rotation = 0)));
          Modelica.Fluid.Sources.MassFlowSource_T mFlow3(nPorts = 1, redeclare package Medium = Modelica.Media.Water.StandardWater, use_m_flow_in = true, T = 323.15) annotation(Placement(transformation(extent = {{ -60, -90},{ -40, -70}}, rotation = 0)));
          Modelica.Fluid.Fittings.TeeJunctionIdeal junctionIdeal(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{20, -90},{40, -70}}, rotation = 0)));

        equation 
          connect(mFlow2.ports[1],T_twoPort.port_a) annotation(Line(points = {{ -40, -10},{ -20, -10}}, color = {0,127,255}, smooth = Smooth.None));
          connect(mFlow1.ports[1],T_onePort.port) annotation(Line(points = {{ -40,40},{ -10,40}}, color = {0,127,255}, smooth = Smooth.None));
          connect(sine.y,mFlow1.m_flow_in) annotation(Line(points = {{ -79,20},{ -70,20},{ -70,48},{ -60,48}}, color = {0,0,127}, smooth = Smooth.None));
          connect(sine.y,mFlow2.m_flow_in) annotation(Line(points = {{ -79,20},{ -70,20},{ -70, -2},{ -60, -2}}, color = {0,0,127}, smooth = Smooth.None));
          connect(mFlow3.ports[1],T_junction.port) annotation(Line(points = {{ -40, -80},{ -10, -80}}, color = {0,127,255}, smooth = Smooth.None));
          connect(sine.y,mFlow3.m_flow_in) annotation(Line(points = {{ -79,20},{ -70,20},{ -70, -72},{ -60, -72}}, color = {0,0,127}, smooth = Smooth.None));
          connect(T_junction.port,junctionIdeal.port_1) annotation(Line(points = {{ -10, -80},{20, -80}}, color = {0,127,255}, smooth = Smooth.None));
          connect(T_twoPort.port_b,openTankCold2.ports[1]) annotation(Line(points = {{0, -10},{0, -10},{30, -10},{30,6}}, color = {0,127,255}));
          connect(T_onePort.port,openTankHot1.ports[1]) annotation(Line(points = {{ -10,40},{ -10,40},{70,40}}, color = {0,127,255}));
          connect(T_twoPort.port_b,openTankHot2.ports[1]) annotation(Line(points = {{0, -10},{30, -10},{70, -10}}, color = {0,127,255}));
          connect(junctionIdeal.port_3,openTankCold3.ports[1]) annotation(Line(points = {{30, -70},{30, -65},{30, -60}}, color = {0,127,255}));
          connect(junctionIdeal.port_2,openTankHot3.ports[1]) annotation(Line(points = {{40, -80},{55.5, -80},{70, -80}}, color = {0,127,255}));
          connect(T_onePort.port,openTankCold1.ports[1]) annotation(Line(points = {{ -10,40},{30,40},{30,60}}, color = {0,127,255}, smooth = Smooth.None));
        end MeasuringTemperature;
        model MomentumBalanceFittings "Illustrating a case in which kinetic terms play a major role in the momentum balance"
          extends Modelica.Icons.Example;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
This example shows the use of a sudden expansion / contraction model, which is connected to two boundary conditions prescribing static pressure. Notice that the prescribed static pressure on the right boundary is higher than on the left one. Still, the fluid flows from left to right.
</p>
<p>
The reason for this is that the boundary conditions model infinite reservoirs with an infinite diameter and thus zero flow velocity. The sudden expansion model does however have two ends with finite diameters, and, as explained in the <a href=\"Modelica://Modelica.Fluid.UsersGuide.Overview\">Overview</a> of the Users' Guide, the momentum balance is not fulfilled exactly for this type of connections. Using a simple <code>connect()</code>-statement, the difference of the kinetic terms is neglected, which is not reasonable in the present model: At the left boundary condition it is zero, and on the left side of the sudden expansion it has a non-zero value. It is not reasonable to neglect it in the shown model, because there is little friction and therefore these kinetic effects dominate. Consequently, only modelling these effects explicitly leads to the correct results.
</p>
<p>
To do so, two additional sudden expansions / contractions are included in the model. The diameter is set to <code>inf</code> close to the boundaries and the proper values close to the original model. These additional components now introduce <i>exact</i> momentum balances and the results are as expected.
</p>
<p>
The total pressures offer an additional perspective on the model. After setting the parameter <code>show_totalPressures</code> on the Advanced tab of the <code>AbruptAdaptor</code>s to <code>true</code>, the total pressures are included in said models and may be plotted. This allows to confirm that the <b>total</b> pressure <i>always</i> reduces along the flow direction, even in the upper model.
</p>


<p align=\"center\">
<img src=\"../Images/Fluid/Examples/MomentumBalanceFittings.png\" border=\"1\">
</p>


</html>"), __Dymola_Commands(file = "Scripts/Fluid/MomentumBalanceFittings/Plot the model results.mos" "Plot the model results"));
          Modelica.Fluid.Sources.Boundary_pT leftBoundary1(redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, nPorts = 1, p = 100000) annotation(Placement(transformation(extent = {{ -80,20},{ -60,40}})));
          Modelica.Fluid.Sources.Boundary_pT rightBoundary1(redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, nPorts = 1, p = 110000) annotation(Placement(transformation(extent = {{80,20},{60,40}})));
          Modelica.Fluid.Fittings.AbruptAdaptor suddenExpansion1(diameter_a = 0.1, diameter_b = 0.2, redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, show_totalPressures = true, show_portVelocities = true) annotation(Placement(transformation(extent = {{ -12,20},{8,40}})));
          Modelica.Fluid.Sources.Boundary_pT leftBoundary2(redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, nPorts = 1, p = 100000) annotation(Placement(transformation(extent = {{ -90, -40},{ -70, -20}})));
          Modelica.Fluid.Sources.Boundary_pT rightBoundary2(redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, nPorts = 1, p = 110000) annotation(Placement(transformation(extent = {{90, -40},{70, -20}})));
          Modelica.Fluid.Fittings.AbruptAdaptor suddenExpansion2(diameter_a = 0.1, diameter_b = 0.2, redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, show_totalPressures = true, show_portVelocities = true) annotation(Placement(transformation(extent = {{ -10, -40},{10, -20}})));
          Modelica.Fluid.Fittings.AbruptAdaptor leftAdaptor(diameter_a = 0.1, redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, diameter_b = Modelica.Constants.inf) annotation(Placement(transformation(extent = {{ -40, -40},{ -60, -20}})));
          Modelica.Fluid.Fittings.AbruptAdaptor rightAdaptor(redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, diameter_a = 0.2, diameter_b = Modelica.Constants.inf) annotation(Placement(transformation(extent = {{40, -40},{60, -20}})));
          inner System system annotation(Placement(transformation(extent = {{ -92,56},{ -72,76}})));

        equation 
          connect(leftBoundary1.ports[1],suddenExpansion1.port_a) annotation(Line(points = {{ -60,30},{ -12,30}}, color = {0,127,255}, smooth = Smooth.None));
          connect(suddenExpansion1.port_b,rightBoundary1.ports[1]) annotation(Line(points = {{8,30},{60,30}}, color = {0,127,255}, smooth = Smooth.None));
          connect(leftAdaptor.port_b,leftBoundary2.ports[1]) annotation(Line(points = {{ -60, -30},{ -70, -30}}, color = {0,127,255}, smooth = Smooth.None));
          connect(leftAdaptor.port_a,suddenExpansion2.port_a) annotation(Line(points = {{ -40, -30},{ -10, -30}}, color = {0,127,255}, smooth = Smooth.None));
          connect(suddenExpansion2.port_b,rightAdaptor.port_a) annotation(Line(points = {{10, -30},{40, -30}}, color = {0,127,255}, smooth = Smooth.None));
          connect(rightAdaptor.port_b,rightBoundary2.ports[1]) annotation(Line(points = {{60, -30},{70, -30}}, color = {0,127,255}, smooth = Smooth.None));
        end MomentumBalanceFittings;
      end Explanatory;
      package DrumBoiler "Drum boiler example, see Franke, Rode, Krueger: On-line Optimization of Drum Boiler Startup, 3rd International Modelica Conference, Linkoping, 2003"
        model DrumBoiler "Complete drum boiler model, including evaporator and supplementary components"
          extends Modelica.Icons.Example;
          import Modelica.SIunits.Conversions.*;
          Modelica.Fluid.Examples.DrumBoiler.BaseClasses.EquilibriumDrumBoiler evaporator(m_D = 300000.0, cp_D = 500, V_t = 100, V_l_start = 67, redeclare package Medium = Modelica.Media.Water.StandardWater, energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, p_start = 100000) annotation(Placement(transformation(extent = {{ -46, -30},{ -26, -10}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -151,165},{138,102}}, lineColor = {0,0,255}, textString = "%name"),Text(extent = {{ -79,67},{67,21}}, lineColor = {0,0,0}, textString = "drum"),Text(extent = {{ -90, -14},{88, -64}}, lineColor = {0,0,0}, textString = "boiler")}), experiment(StopTime = 5400), Documentation(info = "<html>
<p align=\"center\">
<img src=\"../Images/Fluid/Examples/DrumBoiler.png\" border=\"1\">
</p>
</html>"));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow furnace annotation(Placement(transformation(origin = { -36, -53}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Fluid.Sources.FixedBoundary sink(nPorts = 1, p = from_bar(0.5), redeclare package Medium = Modelica.Media.Water.StandardWaterOnePhase, T = 500) annotation(Placement(transformation(origin = {90, -20}, extent = {{ -10, -10},{10,10}}, rotation = 180)));
          Modelica.Fluid.Sensors.MassFlowRate massFlowRate(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(origin = {30, -20}, extent = {{10,10},{ -10, -10}}, rotation = 180)));
          Modelica.Fluid.Sensors.Temperature temperature(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(origin = { -3, -1}, extent = {{10,10},{ -10, -10}}, rotation = 180)));
          Modelica.Fluid.Sensors.Pressure pressure(redeclare package Medium = Modelica.Media.Water.StandardWater) annotation(Placement(transformation(extent = {{10,14},{30,34}}, rotation = 0)));
          Modelica.Blocks.Continuous.PI controller(T = 120, k = 10) annotation(Placement(transformation(extent = {{ -51,23},{ -65,37}}, rotation = 0)));
          Modelica.Fluid.Sources.MassFlowSource_h pump(nPorts = 1, h = 500000.0, redeclare package Medium = Modelica.Media.Water.StandardWater, use_m_flow_in = true) annotation(Placement(transformation(extent = {{ -80, -30},{ -60, -10}}, rotation = 0)));
          Modelica.Blocks.Math.Feedback feedback annotation(Placement(transformation(extent = {{ -26,20},{ -46,40}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant levelSetPoint(k = 67) annotation(Placement(transformation(extent = {{ -43,50},{ -30,63}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput T_S annotation(Placement(transformation(extent = {{100,56},{108,64}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput p_S annotation(Placement(transformation(extent = {{100,20},{108,28}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput qm_S annotation(Placement(transformation(extent = {{100, -4},{108,4}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput V_l annotation(Placement(transformation(extent = {{100,88},{108,96}}, rotation = 0)));
        public 
          Modelica.Blocks.Math.Gain MW2W(k = 1000000.0) annotation(Placement(transformation(extent = {{ -60, -75.5},{ -50, -64.5}}, rotation = 0)));
          Modelica.Blocks.Math.Gain Pa2bar(k = 1e-05) annotation(Placement(transformation(extent = {{37,19},{47,29}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Celsius.FromKelvin K2degC annotation(Placement(transformation(extent = {{38,55},{48,65}}, rotation = 0)));
          Modelica.Blocks.Nonlinear.Limiter limiter(uMin = 0, uMax = 500) annotation(Placement(transformation(origin = { -78,30}, extent = {{ -7, -7},{7,7}}, rotation = 180)));
          Modelica.Fluid.Valves.ValveLinear SteamValve(redeclare package Medium = Modelica.Media.Water.StandardWater, dp_nominal = 9000000, m_flow_nominal = 180) annotation(Placement(transformation(extent = {{50, -10},{70, -30}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -90,70},{ -70,90}})));
          Modelica.Blocks.Sources.TimeTable q_F_Tab(table = [0,0;3600,400;7210,400]) annotation(Placement(transformation(extent = {{ -90, -80},{ -70, -60}}, rotation = 0)));
          Modelica.Blocks.Sources.TimeTable Y_Valve_Tab(table = [0,0;900,1;7210,1]) annotation(Placement(transformation(extent = {{20, -80},{40, -60}}, rotation = 0)));

        equation 
          connect(furnace.port,evaporator.heatPort) annotation(Line(points = {{ -36, -43},{ -36, -30}}, color = {191,0,0}));
          connect(controller.u,feedback.y) annotation(Line(points = {{ -49.6,30},{ -45,30}}, color = {0,0,127}));
          connect(feedback.u2,evaporator.V) annotation(Line(points = {{ -36,22},{ -36, -9},{ -26, -9}}, color = {0,0,127}));
          connect(levelSetPoint.y,feedback.u1) annotation(Line(points = {{ -29.35,56.5},{ -22,56.5},{ -22,30},{ -28,30}}, color = {0,0,127}));
          connect(massFlowRate.m_flow,qm_S) annotation(Line(points = {{30, -9},{30,0},{104,0}}, color = {0,0,127}));
          connect(evaporator.V,V_l) annotation(Line(points = {{ -26, -9},{ -26,11},{ -15,11},{ -15,92},{104,92}}, color = {0,0,127}));
          connect(MW2W.y,furnace.Q_flow) annotation(Line(points = {{ -49.5, -70},{ -36, -70},{ -36, -63}}, color = {0,0,127}));
          connect(pressure.p,Pa2bar.u) annotation(Line(points = {{31,24},{36,24}}, color = {0,0,127}));
          connect(Pa2bar.y,p_S) annotation(Line(points = {{47.5,24},{104,24}}, color = {0,0,127}));
          connect(K2degC.Celsius,T_S) annotation(Line(points = {{48.5,60},{104,60}}, color = {0,0,127}));
          connect(controller.y,limiter.u) annotation(Line(points = {{ -65.7,30},{ -69.6,30}}, color = {0,0,127}));
          connect(limiter.y,pump.m_flow_in) annotation(Line(points = {{ -85.7,30},{ -90,30},{ -90, -12},{ -80, -12}}, color = {0,0,127}));
          connect(temperature.T,K2degC.Kelvin) annotation(Line(points = {{4, -1},{4, -1},{10, -1},{10,60},{37,60}}, color = {0,0,127}));
          connect(pressure.port,massFlowRate.port_a) annotation(Line(points = {{20,14},{20, -20}}, color = {0,127,255}));
          connect(pump.ports[1],evaporator.port_a) annotation(Line(points = {{ -60, -20},{ -46, -20}}, color = {0,127,255}));
          connect(massFlowRate.port_b,SteamValve.port_a) annotation(Line(points = {{40, -20},{50, -20}}, color = {0,127,255}));
          connect(SteamValve.port_b,sink.ports[1]) annotation(Line(points = {{70, -20},{75, -20},{80, -20}}, color = {0,127,255}));
          connect(evaporator.port_b,massFlowRate.port_a) annotation(Line(points = {{ -26, -20},{20, -20}}, color = {0,127,255}));
          connect(temperature.port,massFlowRate.port_a) annotation(Line(points = {{ -3, -11},{ -3, -20},{20, -20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(q_F_Tab.y,MW2W.u) annotation(Line(points = {{ -69, -70},{ -61, -70}}, color = {0,0,127}, smooth = Smooth.None));
          connect(Y_Valve_Tab.y,SteamValve.opening) annotation(Line(points = {{41, -70},{60, -70},{60, -28}}, color = {0,0,127}, smooth = Smooth.None));
        end DrumBoiler;
        package BaseClasses "Additional components for drum boiler example"
          extends Modelica.Fluid.Icons.BaseClassLibrary;
          model EquilibriumDrumBoiler "Simple Evaporator with two states, see Astroem, Bell: Drum-boiler dynamics, Automatica 36, 2000, pp.363-378"
            extends Modelica.Fluid.Interfaces.PartialTwoPort(final port_a_exposesState = true, final port_b_exposesState = true, redeclare replaceable package Medium = Modelica.Media.Water.StandardWater extends Modelica.Media.Interfaces.PartialTwoPhaseMedium);
            import Modelica.SIunits.Conversions.*;
            import Modelica.Constants;
            import Modelica.Fluid.Types;
            parameter SI.Mass m_D "mass of surrounding drum metal";
            parameter Medium.SpecificHeatCapacity cp_D "specific heat capacity of drum metal";
            parameter SI.Volume V_t "total volume inside drum";
            parameter Medium.AbsolutePressure p_start = system.p_start "Start value of pressure" annotation(Dialog(tab = "Initialization"));
            parameter SI.Volume V_l_start = V_t / 2 "Start value of liquid volumeStart value of volume" annotation(Dialog(tab = "Initialization"));
            parameter Boolean allowFlowReversal = system.allowFlowReversal "allow flow reversal, false restricts to design direction (port_a -> port_b)" annotation(Dialog(tab = "Assumptions"), Evaluate = true);
            parameter Types.Dynamics energyDynamics = system.energyDynamics "Formulation of energy balance" annotation(Evaluate = true, Dialog(tab = "Assumptions", group = "Dynamics"));
            parameter Types.Dynamics massDynamics = system.massDynamics "Formulation of mass balance" annotation(Evaluate = true, Dialog(tab = "Assumptions", group = "Dynamics"));
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}}, rotation = 0)));
            Modelica.Blocks.Interfaces.RealOutput V "liquid volume" annotation(Placement(transformation(origin = {100,110}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Medium.SaturationProperties sat "State vector to compute saturation properties";
            Medium.AbsolutePressure p(start = p_start, stateSelect = StateSelect.prefer) "pressure inside drum boiler";
            Medium.Temperature T "temperature inside drum boiler";
            SI.Volume V_v "volume of vapour phase";
            SI.Volume V_l(start = V_l_start, stateSelect = StateSelect.prefer) "volumes of liquid phase";
            Medium.SpecificEnthalpy h_v = Medium.dewEnthalpy(sat) "specific enthalpy of vapour";
            Medium.SpecificEnthalpy h_l = Medium.bubbleEnthalpy(sat) "specific enthalpy of liquid";
            Medium.Density rho_v = Medium.dewDensity(sat) "density in vapour phase";
            Medium.Density rho_l = Medium.bubbleDensity(sat) "density in liquid phase";
            SI.Mass m "total mass of drum boiler";
            SI.Energy U "internal energy";
            Medium.Temperature T_D = heatPort.T "temperature of drum";
            SI.HeatFlowRate q_F = heatPort.Q_flow "heat flow rate from furnace";
            Medium.SpecificEnthalpy h_W = inStream(port_a.h_outflow) "Feed water enthalpy (specific enthalpy close to feedwater port when mass flows in to the boiler)";
            Medium.SpecificEnthalpy h_S = inStream(port_b.h_outflow) "steam enthalpy (specific enthalpy close to steam port when mass flows in to the boiler)";
            SI.MassFlowRate qm_W = port_a.m_flow "feed water mass flow rate";
            SI.MassFlowRate qm_S = port_b.m_flow "steam mass flow rate";

          equation 
            m = rho_v * V_v + rho_l * V_l + m_D "Total mass";
            U = rho_v * V_v * h_v + rho_l * V_l * h_l - p * V_t + m_D * cp_D * T_D "Total energy";
            if massDynamics == Types.Dynamics.SteadyState then
              0 = qm_W + qm_S "Steady state mass balance";

            else
              der(m) = qm_W + qm_S "Dynamic mass balance";

            end if;
            if energyDynamics == Types.Dynamics.SteadyState then
              0 = q_F + port_a.m_flow * actualStream(port_a.h_outflow) + port_b.m_flow * actualStream(port_b.h_outflow) "Steady state energy balance";

            else
              der(U) = q_F + port_a.m_flow * actualStream(port_a.h_outflow) + port_b.m_flow * actualStream(port_b.h_outflow) "Dynamic energy balance";

            end if;
            V_t = V_l + V_v;
            sat.psat = p;
            sat.Tsat = T;
            sat.Tsat = Medium.saturationTemperature(p);
            T_D = T;
            port_a.p = p;
            port_a.h_outflow = h_l;
            port_b.p = p;
            port_b.h_outflow = h_v;
            V = V_l;
            assert(p < Medium.fluidConstants[1].criticalPressure - 10000, "Evaporator model requires subcritical pressure");

          initial equation 
            if energyDynamics == Types.Dynamics.FixedInitial then
              p = p_start;

            elseif energyDynamics == Types.Dynamics.SteadyStateInitial then
              der(p) = 0;


            else

            end if;
            if massDynamics == Types.Dynamics.FixedInitial then
              V_l = V_l_start;

            elseif energyDynamics == Types.Dynamics.SteadyStateInitial then
              der(V_l) = 0;


            else

            end if;
            annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,64},{100, -64}}, lineColor = {0,0,0}, fillPattern = FillPattern.Backward, fillColor = {135,135,135}),Rectangle(extent = {{ -100,44},{100, -44}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,127,255}),Ellipse(extent = {{18,0},{48, -29}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -1,29},{29,0}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{43,31},{73,2}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -31,1},{ -1, -28}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{50,15},{80, -14}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -72,25},{ -42, -4}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{71, -11},{101, -40}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{72,28},{102, -1}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{71,40},{101,11}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,70},{150,110}}, lineColor = {0,0,255}, textString = "%name"),Line(points = {{0, -64},{0, -100}}, color = {191,0,0}),Line(points = {{100,100},{100,60}}, color = {0,0,127}),Ellipse(extent = {{58, -11},{88, -40}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{71,1},{101, -28}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Documentation(revisions = "<html>
<ul>
<li><i>Dec 2008</i>
    by R&uuml;diger Franke:<br>
     Adapt initialization to new Types.Dynamics</li>
<li><i>2 Nov 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
     Initialization options fixed</li>
<li><i>6 Sep 2005</i><br>
    Model by Ruediger Franke<br>
    See Franke, Rode, Krueger: On-line Optimization of Drum Boiler Startup, 3rd International Modelica Conference, Linkoping, 2003.<br>
    Modified after the 45th Design Meeting</li>
</ul>
</html>", info = "<html>
Model of a simple evaporator with two states. The model assumes two-phase equilibrium inside the component; saturated steam goes out of the steam outlet.
<p>
References: Astroem, Bell: Drum-boiler dynamics, Automatica 36, 2000, pp.363-378
</html>"));
          end EquilibriumDrumBoiler;
        end BaseClasses;
      end DrumBoiler;
      package ControlledTankSystem "Tank system with controller, start/stop/shut operation and diagram animation"
        model ControlledTanks "Demonstrating the controller of a tank filling/emptying system"
          extends Modelica.Icons.Example;
          package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater;
          Modelica.Fluid.Examples.ControlledTankSystem.Utilities.TankController tankController(waitTime = 50, maxLevel = 0.9 * tank1.height, minLevel = 0.01) annotation(Placement(transformation(extent = {{ -60, -20},{ -20,20}}, rotation = 0)));
          Modelica.Fluid.Examples.ControlledTankSystem.Utilities.RadioButton start(reset = {stop.on,shut.on}, buttonTimeTable = {20,280}) annotation(Placement(transformation(extent = {{ -100,20},{ -80,40}}, rotation = 0)));
          Modelica.Fluid.Examples.ControlledTankSystem.Utilities.RadioButton stop(reset = {start.on,shut.on}, buttonTimeTable = {220,650}) annotation(Placement(transformation(extent = {{ -100, -10},{ -80,10}}, rotation = 0)));
          Modelica.Fluid.Examples.ControlledTankSystem.Utilities.RadioButton shut(reset = {start.on,stop.on}, buttonTimeTable = {700}) annotation(Placement(transformation(extent = {{ -100, -40},{ -80, -20}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), experiment(StopTime = 900), Documentation(info = "<html>
<p>
With this example, the controller of a tank filling/emptying system
is demonstrated.
</p>

<p>
The basic operation is to fill and empty the two tanks:
</p>
<ol>
<li> Valve 1 is opened and tank 1 is filled.</li>
<li> When tank 1 reaches its fill level limit,
     valve 1 is closed. </li>
<li> After a waiting time, valve 2 is
     opened and the fluid flows from tank 1 into tank 2.</li>
<li> When tank 1 reaches its minimum level, valve 2 is closed. </li>
<li> After a waiting time, valve 3 is opened and
     the fluid flows out of tank 2</li>
<li> When tank 2 reaches its minimum level, valve 3 is closed</liI>
</ol>
<p>
The above \"normal\" process can be influenced by three
buttons:
</p>
<ul>
<li> Button <b>start</b> starts the above process.
     When this button is pressed after a \"stop\" or
     \"shut\" operation, the process operation continues.
     </li>.
<li> Button <b>stop</b> stops the above process by
     closing all valves. Then, the controller waits for
     further input (either \"start\" or \"shut\" operation).</li>
<li> Button <b>shut</b> is used to shutdown the process,
     by emptying at once both tanks by opening valve 2 and
     valve 3. When this is achieved,
     the process goes back to its start configuration
     where all 3 valves are closed.
     Clicking on \"start\", restarts the process.</li>
</ul>

<p>
The demo-run uses the following button presses:
</p>

<ul>
<li> Button <b>start</b> pressed at 20 s.</li>
<li> Button <b>stop</b> pressed at 220 s </li>
<li> Button <b>start</b> pressed at 280 s </li>
<li> Button <b>stop</b> pressed at 650 s </li>
<li> Button <b>shut</b> pressed at 700 s </li>
<li> Simulate for 900 s</li>
</ul>

<p>
This example is based on
</p>

<dl>
<dt>Dressler I. (2004):</dt>
<dd> <b>Code Generation From JGrafchart to Modelica</b>.
     Master thesis, supervisor: Karl-Erik Arzen,
     Department of Automatic Control, Lund Institute of Technology,
     Lund, Sweden, March 30, 2004<br>&nbsp;</dd>
</dl>

<p align=\"center\">
<img src=\"../Images/Fluid/Examples/ControlledTanks.png\" border=\"1\">
</p>

</html>"), __Dymola_Commands(file = "Scripts/Fluid/ControlledTanks/plot level and ports.m_flow.mos" "plot level and ports.m_flow"));
          Modelica.Fluid.Valves.ValveDiscrete valve1(redeclare package Medium = Medium, m_flow_nominal = 40, dp_nominal = 100000) annotation(Placement(transformation(origin = { -10,70}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
          Modelica.Fluid.Vessels.OpenTank tank1(level_start = 0.05, redeclare package Medium = Medium, crossArea = 6, height = 4, nPorts = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.2, height = 4, zeta_out = 0, zeta_in = 1),Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.2, height = 0, zeta_out = 0, zeta_in = 1)}) annotation(Placement(transformation(extent = {{10,30},{50,70}}, rotation = 0)));
          Modelica.Blocks.Sources.RealExpression level1(y = tank1.level) annotation(Placement(transformation(extent = {{ -90, -60},{ -55, -40}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveDiscrete valve2(redeclare package Medium = Medium, dp_nominal(displayUnit = "Pa") = 1, m_flow_nominal = 100) annotation(Placement(transformation(origin = {34,0}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Modelica.Fluid.Valves.ValveDiscrete valve3(redeclare package Medium = Medium, dp_nominal(displayUnit = "Pa") = 1, m_flow_nominal = 10) annotation(Placement(transformation(origin = {35, -80}, extent = {{10, -10},{ -10,10}}, rotation = 0)));
          Modelica.Fluid.Vessels.OpenTank tank2(level_start = 0.05, redeclare package Medium = Medium, height = 5, crossArea = 6, nPorts = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.2, height = 5, zeta_out = 0, zeta_in = 1),Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.2, height = 0, zeta_out = 0, zeta_in = 1)}) annotation(Placement(transformation(extent = {{50, -60},{90, -20}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT ambient1(redeclare package Medium = Medium, nPorts = 1, p = system.p_ambient, T = system.T_ambient) annotation(Placement(transformation(extent = {{ -10, -90},{10, -70}}, rotation = 0)));
          Modelica.Blocks.Sources.RealExpression level2(y = tank2.level) annotation(Placement(transformation(extent = {{ -70, -80},{ -33, -60}}, rotation = 0)));
          Modelica.Fluid.Sources.Boundary_pT source(redeclare package Medium = Medium, p = 2500000.0, nPorts = 1, T = system.T_ambient) annotation(Placement(transformation(origin = { -40,70}, extent = {{ -10, -10},{10,10}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -90,70},{ -70,90}}, rotation = 0)));

        equation 
          connect(shut.on,tankController.shut) annotation(Line(points = {{ -79, -30},{ -72, -30},{ -72, -12},{ -62, -12}}, color = {255,0,255}));
          connect(stop.on,tankController.stop) annotation(Line(points = {{ -79,0},{ -62,0}}, color = {255,0,255}));
          connect(start.on,tankController.start) annotation(Line(points = {{ -79,30},{ -70,30},{ -70,12},{ -62,12}}, color = {255,0,255}));
          connect(tankController.valve1,valve1.open) annotation(Line(points = {{ -19,12},{ -10,12},{ -10,62}}, color = {255,0,255}));
          connect(level1.y,tankController.level1) annotation(Line(points = {{ -53.25, -50},{ -52, -50},{ -52, -22}}, color = {0,0,127}));
          connect(tankController.valve2,valve2.open) annotation(Line(points = {{ -19,0},{ -5,0},{ -5,4.89859e-16},{26,4.89859e-16}}, color = {255,0,255}));
          connect(tankController.valve3,valve3.open) annotation(Line(points = {{ -19, -12},{ -10, -12},{ -10, -50},{35, -50},{35, -72}}, color = {255,0,255}));
          connect(level2.y,tankController.level2) annotation(Line(points = {{ -31.15, -70},{ -28, -70},{ -28, -22}}, color = {0,0,127}));
          connect(source.ports[1],valve1.port_a) annotation(Line(points = {{ -30,70},{ -20,70}}, color = {0,127,255}, smooth = Smooth.None));
          connect(valve3.port_b,ambient1.ports[1]) annotation(Line(points = {{25, -80},{10, -80}}, color = {0,127,255}, smooth = Smooth.None));
          connect(tank2.ports[2],valve3.port_a) annotation(Line(points = {{74, -60},{74, -80},{45, -80}}, color = {0,127,255}, smooth = Smooth.None));
          connect(valve2.port_b,tank2.ports[1]) annotation(Line(points = {{34, -10},{34, -20},{50, -20},{50, -60},{66, -60}}, color = {0,127,255}, smooth = Smooth.None));
          connect(valve1.port_b,tank1.ports[1]) annotation(Line(points = {{0,70},{10,70},{10,30},{26,30}}, color = {0,127,255}, smooth = Smooth.None));
          connect(tank1.ports[2],valve2.port_a) annotation(Line(points = {{34,30},{34,10}}, color = {0,127,255}, smooth = Smooth.None));
        end ControlledTanks;
        package Utilities
          model TankController "Controller for tank system"
            extends Modelica.StateGraph.Interfaces.PartialStateGraphIcon;
            parameter SI.Height maxLevel "Fill level of tank 1";
            parameter SI.Height minLevel "Lowest level of tank 1 and 2";
            parameter SI.Time waitTime "Wait time, between operations";
            Modelica.StateGraph.InitialStep s1(nIn = 2) annotation(Placement(transformation(extent = {{ -72,30},{ -52,50}}, rotation = 0)));
            Modelica.Fluid.Examples.ControlledTankSystem.Utilities.NormalOperation normal(maxLevel = maxLevel, minLevel = minLevel, waitTime = waitTime) annotation(Placement(transformation(extent = {{ -20,20},{20,60}}, rotation = 0)));
            Modelica.StateGraph.Transition T1(condition = start) annotation(Placement(transformation(extent = {{ -50,50},{ -30,30}}, rotation = 0)));
            Modelica.StateGraph.Transition T2(condition = level2 < minLevel) annotation(Placement(transformation(extent = {{27,50},{47,30}}, rotation = 0)));
            Modelica.StateGraph.Transition T3(condition = stop) annotation(Placement(transformation(origin = { -23, -1}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Modelica.StateGraph.Step s2(nOut = 2) annotation(Placement(transformation(extent = {{ -50, -60},{ -30, -40}}, rotation = 0)));
            Modelica.StateGraph.Transition T4(condition = start) annotation(Placement(transformation(origin = {10,0}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Modelica.StateGraph.Transition T5(condition = shut) annotation(Placement(transformation(extent = {{ -6, -60},{14, -40}}, rotation = 0)));
            Modelica.StateGraph.Step emptyTanks annotation(Placement(transformation(extent = {{20, -60},{40, -40}}, rotation = 0)));
            Modelica.StateGraph.Transition T6(condition = level1 < minLevel and level2 < minLevel) annotation(Placement(transformation(extent = {{45, -60},{65, -40}}, rotation = 0)));
            Modelica.Blocks.Interfaces.BooleanInput start annotation(Placement(transformation(extent = {{ -120,50},{ -100,70}}, rotation = 0)));
            Modelica.Blocks.Interfaces.BooleanInput stop annotation(Placement(transformation(extent = {{ -120, -10},{ -100,10}}, rotation = 0)));
            Modelica.Blocks.Interfaces.BooleanInput shut annotation(Placement(transformation(extent = {{ -120, -70},{ -100, -50}}, rotation = 0)));
            Modelica.Blocks.Interfaces.RealInput level1 annotation(Placement(transformation(origin = { -60, -110}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Modelica.Blocks.Interfaces.RealInput level2 annotation(Placement(transformation(origin = {60, -110}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Modelica.Blocks.Interfaces.BooleanOutput valve1 annotation(Placement(transformation(extent = {{100,55},{110,65}}, rotation = 0)));
            Modelica.Blocks.Interfaces.BooleanOutput valve2 annotation(Placement(transformation(extent = {{100, -5},{110,5}}, rotation = 0)));
            Modelica.Blocks.Interfaces.BooleanOutput valve3 annotation(Placement(transformation(extent = {{100, -65},{110, -55}}, rotation = 0)));
            Modelica.Blocks.Sources.BooleanExpression setValve1(y = normal.fillTank1.active) annotation(Placement(transformation(extent = {{20,73},{80,92}}, rotation = 0)));
            Modelica.Blocks.Sources.BooleanExpression setValve2(y = normal.fillTank2.active or emptyTanks.active and level1 > minLevel) annotation(Placement(transformation(extent = {{ -40, -85},{80, -64}}, rotation = 0)));
            Modelica.Blocks.Sources.BooleanExpression setValve3(y = normal.emptyTank2.active or emptyTanks.active and level2 > minLevel) annotation(Placement(transformation(extent = {{ -40, -103},{80, -83}}, rotation = 0)));

          equation 
            annotation(structurallyIncomplete, Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0})}), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -100,68},{ -12,54}}, lineColor = {0,0,0}, textString = "start"),Text(extent = {{ -99,6},{ -14, -9}}, lineColor = {0,0,0}, textString = "stop"),Text(extent = {{ -99, -54},{ -14, -69}}, lineColor = {0,0,0}, textString = "shut"),Text(extent = {{ -94, -82},{ -9, -96}}, lineColor = {0,0,0}, textString = "level1"),Text(extent = {{11, -83},{96, -98}}, lineColor = {0,0,0}, textString = "level2"),Text(extent = {{10,68},{99,54}}, lineColor = {0,0,0}, textString = "valve1"),Text(extent = {{7,10},{101, -5}}, lineColor = {0,0,0}, textString = "valve2"),Text(extent = {{2, -51},{102, -67}}, lineColor = {0,0,0}, textString = "valve3")}));
            connect(s1.outPort[1],T1.inPort) annotation(Line(points = {{ -51.5,40},{ -44,40}}, color = {0,0,0}));
            connect(T1.outPort,normal.inPort) annotation(Line(points = {{ -38.5,40},{ -21.3333,40}}, color = {0,0,0}));
            connect(normal.outPort,T2.inPort) annotation(Line(points = {{20.6667,40},{33,40}}, color = {0,0,0}));
            connect(T5.outPort,emptyTanks.inPort[1]) annotation(Line(points = {{5.5, -50},{19, -50}}, color = {0,0,0}));
            connect(emptyTanks.outPort[1],T6.inPort) annotation(Line(points = {{40.5, -50},{51, -50}}, color = {0,0,0}));
            connect(setValve1.y,valve1) annotation(Line(points = {{83,82.5},{90,82.5},{90,60},{105,60}}, color = {255,0,255}));
            connect(setValve2.y,valve2) annotation(Line(points = {{86, -74.5},{90, -74.5},{90,0},{105,0}}, color = {255,0,255}));
            connect(setValve3.y,valve3) annotation(Line(points = {{86, -93},{95, -93},{95, -60},{105, -60}}, color = {255,0,255}));
            connect(normal.suspend[1],T3.inPort) annotation(Line(points = {{ -10,19.3333},{ -10,12},{ -23,12},{ -23,3}}, color = {0,0,0}));
            connect(T3.outPort,s2.inPort[1]) annotation(Line(points = {{ -23, -2.5},{ -23, -20},{ -60, -20},{ -60, -50},{ -51, -50}}, color = {0,0,0}));
            connect(level1,normal.level1) annotation(Line(points = {{ -60, -110},{ -60, -80},{ -80, -80},{ -80,20},{ -30,20},{ -30,24},{ -22.6667,24}}, color = {0,0,255}));
            connect(s2.outPort[1],T5.inPort) annotation(Line(points = {{ -29.5, -49.75},{ -30, -49.75},{ -30, -50},{0, -50}}, color = {0,0,0}));
            connect(s2.outPort[2],T4.inPort) annotation(Line(points = {{ -29.5, -50.25},{ -29, -50},{ -8, -50},{ -8, -25},{10, -25},{10, -4}}, color = {0,0,0}));
            connect(T2.outPort,s1.inPort[1]) annotation(Line(points = {{38.5,40},{70,40},{70,70},{ -80,70},{ -80,40},{ -73,40},{ -73,40.5}}, color = {0,0,0}));
            connect(T6.outPort,s1.inPort[2]) annotation(Line(points = {{56.5, -50},{70, -50},{70,70},{ -80,70},{ -80,40},{ -74,40},{ -73,39.5}}, color = {0,0,0}));
            connect(T4.outPort,normal.resume[1]) annotation(Line(points = {{10,1.5},{10,10},{10.5,10},{10.5,18.6667},{10,18.6667}}, color = {0,0,0}));
          end TankController;
          model NormalOperation "Normal operation of tank system (button start pressed)"
            extends Modelica.StateGraph.PartialCompositeStep;
            parameter SI.Height maxLevel "Fill level of tank 1";
            parameter SI.Height minLevel "Lowest level of tank 1 and 2";
            parameter SI.Time waitTime "Wait time between operations";
            Modelica.Blocks.Interfaces.RealInput level1 annotation(Placement(transformation(extent = {{ -190, -140},{ -150, -100}}, rotation = 0)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -150, -150},{150,150}}), graphics), Documentation(info = "<html>

</html>"));
            Modelica.StateGraph.Step fillTank1 annotation(Placement(transformation(extent = {{ -140, -10},{ -120,10}}, rotation = 0)));
            Modelica.StateGraph.Transition T1(condition = level1 > maxLevel) annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
            Modelica.StateGraph.Step fillTank2 annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
            Modelica.StateGraph.Transition T3(condition = level1 < minLevel) annotation(Placement(transformation(extent = {{20, -10},{40,10}}, rotation = 0)));
            Modelica.StateGraph.Step emptyTank2 annotation(Placement(transformation(extent = {{120, -10},{140,10}}, rotation = 0)));
            Modelica.StateGraph.Step wait1 annotation(Placement(transformation(extent = {{ -80, -10},{ -60,10}}, rotation = 0)));
            Modelica.StateGraph.Transition T2(enableTimer = true, waitTime = waitTime) annotation(Placement(transformation(extent = {{ -50, -10},{ -30,10}}, rotation = 0)));
            Modelica.StateGraph.Step wait2 annotation(Placement(transformation(extent = {{54, -10},{74,10}}, rotation = 0)));
            Modelica.StateGraph.Transition T4(enableTimer = true, waitTime = waitTime) annotation(Placement(transformation(extent = {{82, -10},{102,10}}, rotation = 0)));

          equation 
            connect(fillTank1.inPort[1],inPort) annotation(Line(points = {{ -141,0},{ -160,0}}, color = {0,0,0}));
            connect(fillTank1.outPort[1],T1.inPort) annotation(Line(points = {{ -119.5,0},{ -104,0}}, color = {0,0,0}));
            connect(fillTank2.outPort[1],T3.inPort) annotation(Line(points = {{10.5,0},{26,0}}, color = {0,0,0}));
            connect(emptyTank2.outPort[1],outPort) annotation(Line(points = {{140.5,0},{155,0}}, color = {0,0,0}));
            connect(wait1.outPort[1],T2.inPort) annotation(Line(points = {{ -59.5,0},{ -44,0}}, color = {0,0,0}));
            connect(T2.outPort,fillTank2.inPort[1]) annotation(Line(points = {{ -38.5,0},{ -11,0}}, color = {0,0,0}));
            connect(T1.outPort,wait1.inPort[1]) annotation(Line(points = {{ -98.5,0},{ -81,0}}, color = {0,0,0}));
            connect(wait2.outPort[1],T4.inPort) annotation(Line(points = {{74.5,0},{88,0}}, color = {0,0,0}));
            connect(T3.outPort,wait2.inPort[1]) annotation(Line(points = {{31.5,0},{53,0}}, color = {0,0,0}));
            connect(T4.outPort,emptyTank2.inPort[1]) annotation(Line(points = {{93.5,0},{119,0}}, color = {0,0,0}));
          end NormalOperation;
          block RadioButton "Button that sets its output to true when pressed and is reset when an element of 'reset' becomes true"
            parameter Modelica.SIunits.Time buttonTimeTable[:] "Time instants where button is pressend";
            input Boolean reset[:] = {false} "Reset button to false, if an element of reset becomes true" annotation(Dialog(group = "Time varying expressions"));
            annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -100},{100,100}}, fillColor = DynamicSelect({192,192,192}, if on > 0.5 then {0,255,0} else {192,192,192}), fillPattern = DynamicSelect(FillPattern.Solid, if on > 0.5 then FillPattern.Solid else FillPattern.Solid), lineColor = {128,128,128}, lineThickness = 0.5),Text(extent = {{ -80, -40},{80,40}}, lineColor = {0,0,0}, textString = "%name")}, interaction = {OnMouseDownSetBoolean(on, true)}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>

</html>"));
            Modelica.Blocks.Interfaces.BooleanOutput on annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
          protected 
            Modelica.Blocks.Sources.BooleanTable table(table = buttonTimeTable);
          algorithm 
            when pre(reset) then
                          on:=false;
            
            end when;
            when change(table.y) then
                          on:=true;
            
            end when;
          end RadioButton;
        end Utilities;
      end ControlledTankSystem;
      model BranchingDynamicPipes "Multi-way connections of pipes with dynamic momentum balance, pressure wave and flow reversal"
        extends Modelica.Icons.Example;
        replaceable package Medium = Modelica.Media.Air.MoistAir;
        annotation(extent = [ -90, -86; -70, -66], Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
This model demonstrates the use of distributed pipe models with dynamic energy, mass and momentum balances.
At time=2s the pressure of boundary4 jumps, which causes a pressure wave and flow reversal.
</p>
<p>
Change system.momentumDynamics on the Assumptions tab of the system object from DynamicFreeInitial to SteadyState,
in order to assume a steady-state momentum balance. This is the default for all models of the library.
</p>
<p>
Change the Medium from MoistAir to StandardWater, in order to investigate a medium with significantly different density.
Note the static head caused by the elevation of the pipes.
</p>

<p>
Note, pipe4.modelStructure = av_b, i.e., the pipe has no volume at port_b.
It is not possible to have a volume at port_b, since otherwise the pressure of the volume is
defined by the connected boundary source. This in turn means that the
derivative of the pressure of the boundary source is needed, since the volume
requires this derivative. It is, however, not possible to compute this
derivative because the input pressure is changing disontinuously and its
derivative would be a dirac impulse.
</p>

<p align=\"center\">
<img src=\"../Images/Fluid/Examples/BranchingDynamicPipes.png\" border=\"1\">
</p>
</html>"));
        inner Modelica.Fluid.System system(momentumDynamics = Modelica.Fluid.Types.Dynamics.DynamicFreeInitial) annotation(Placement(transformation(extent = {{ -90,70},{ -70,90}}, rotation = 0)));
        Modelica.Fluid.Sources.Boundary_pT boundary1(nPorts = 1, redeclare package Medium = Medium, p = 150000) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 90, origin = {0, -80})));
        Modelica.Fluid.Pipes.DynamicPipe pipe1(redeclare package Medium = Medium, use_T_start = true, nNodes = 5, diameter = 0.0254, m_flow_start = 0.02, p_a_start = 150000, p_b_start = 130000, height_ab = 50, length = 50) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 90, origin = {0, -50})));
        Modelica.Fluid.Pipes.DynamicPipe pipe2(redeclare package Medium = Medium, use_T_start = true, nNodes = 5, redeclare model HeatTransfer = Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.LocalPipeFlowHeatTransfer, use_HeatTransfer = true, diameter = 0.0254, m_flow_start = 0.01, length = 50, height_ab = 25, p_a_start = 130000, p_b_start = 120000) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 90, origin = { -20, -10})));
        Modelica.Fluid.Pipes.DynamicPipe pipe3(redeclare package Medium = Medium, use_T_start = true, nNodes = 5, diameter = 0.0254, m_flow_start = 0.01, length = 25, p_a_start = 130000, p_b_start = 120000, height_ab = 25) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 90, origin = {20, -10})));
        Modelica.Fluid.Pipes.DynamicPipe pipe4(redeclare package Medium = Medium, use_T_start = true, nNodes = 5, modelStructure = Modelica.Fluid.Types.ModelStructure.av_b, diameter = 0.0254, m_flow_start = 0.02, p_a_start = 120000, p_b_start = 100000, height_ab = 50, length = 50) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 90, origin = {0,30})));
        Modelica.Fluid.Sources.Boundary_pT boundary4(nPorts = 1, redeclare package Medium = Medium, use_p_in = true, use_T_in = false, p = 100000) annotation(Placement(transformation(extent = {{10, -10},{ -10,10}}, rotation = 90, origin = {0,60})));
        Modelica.Blocks.Sources.Ramp ramp1(offset = 100000.0, duration = 0, startTime = 2, height = 100000.0) annotation(Placement(transformation(extent = {{ -40,70},{ -20,90}}, rotation = 0)));
        Modelica.Thermal.HeatTransfer.Sources.FixedHeatFlow[pipe2.nNodes] heat2(Q_flow = 200 * pipe2.dxs) annotation(Placement(transformation(extent = {{ -60, -20},{ -40,0}}, rotation = 0)));

      equation 
        connect(ramp1.y,boundary4.p_in) annotation(Line(points = {{ -19,80},{ -8,80},{ -8,72}}, color = {0,0,127}, thickness = 0.5));
        connect(boundary1.ports[1],pipe1.port_a) annotation(Line(points = {{6.66134e-16, -70},{0, -70},{0, -60},{ -6.12323e-16, -60}}, color = {0,127,255}, thickness = 0.5));
        connect(pipe1.port_b,pipe2.port_a) annotation(Line(points = {{6.12323e-16, -40},{0, -40},{0, -30},{ -20, -30},{ -20, -20}}, color = {0,127,255}, thickness = 0.5));
        connect(pipe1.port_b,pipe3.port_a) annotation(Line(points = {{6.12323e-16, -40},{0, -40},{0, -30},{20, -30},{20, -20}}, color = {0,127,255}, thickness = 0.5));
        connect(pipe2.port_b,pipe4.port_a) annotation(Line(points = {{ -20,0},{ -20,0},{ -20,10},{0,10},{0,16},{0,20},{ -6.12323e-16,20}}, color = {0,127,255}, thickness = 0.5));
        connect(pipe3.port_b,pipe4.port_a) annotation(Line(points = {{20,0},{20,0},{20,10},{0,10},{0,16},{0,20},{ -6.12323e-16,20}}, color = {0,127,255}, thickness = 0.5));
        connect(pipe4.port_b,boundary4.ports[1]) annotation(Line(points = {{6.12323e-16,40},{6.12323e-16,50},{ -8.88178e-16,50}}, color = {0,127,255}, thickness = 0.5));
        connect(heat2.port,pipe2.heatPorts) annotation(Line(points = {{ -40, -10},{ -24.4, -10},{ -24.4, -9.9}}, color = {191,0,0}, thickness = 0.5));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 10), __Dymola_Commands(file(ensureSimulated = true) = "Scripts/Fluid/BranchingDynamicPipes/plotResults.mos" "plotResults"), Documentation(info = "<html>
<p>
This model demonstrates the use of distributed pipe models with dynamic energy, mass and momentum balances.
At time=2s the pressure of boundary4 jumps, which causes a pressure wave and flow reversal.
</p>
<p>
Change system.momentumDynamics on the Assumptions tab of the system object from DynamicFreeInitial to SteadyState,
in order to assume a steady-state momentum balance. This is the default for all models of the library.
</p>
<p>
Change the Medium from MoistAir to StandardWater, in order to investigate a medium with sigificantly different density.
Note the static head caused by the elevation of the pipes.
</p>
</html>"));
      end BranchingDynamicPipes;
      package AST_BatchPlant "Model of the experimental batch plant at Process Control Laboratory at University of Dortmund (Prof. Engell)"
        annotation(preferedView = "info", Documentation(info = "<html>
<p>
The process under consideration is an evaporation plant for a
student lab at the Process Control Laboratory (AST) of the
University of Dortmund that evaporates a water sodium chloride
mixture so that a higher concentrated solution is produced.
The task of the students is to learn how to program the process
control system. A picture of the batch plant is shown in the figure
below.
</p>

<p align=\"center\">
<img src=\"../Images/Fluid/Examples/AST_BatchPlant1.jpg\" border=\"1\">
</p>

<p>
The flow sheet diagram is shown in the next figure.
</p>

<p align=\"center\">
<img src=\"../Images/Fluid/Examples/AST_BatchPlant2.png\" border=\"1\">
</p>

<p>
Pure water from tank B1 and concentrated sodium chloride
solution from tank B2 are mixed in a mixing tank B3.
After buffering in tank B4 the mixture flows to the
evaporator B5. Here the water sodium chloride mixture
is evaporated until the desired con-centration is reached.
The steam is condensed in the condenser K1 and cooled
afterwards in the cooling tank B6. The concentrated
 solution is also led to a cooling tank B7. The cooled
fluids are pumped back to the charging vessels by the
pumps P1 and P2. Be-tween the tanks several valves are
present that are regulated by a central control system.
</p>
</html>"));
        model BatchPlant_StandardWater
          extends Modelica.Icons.Example;
          replaceable package BatchMedium = Modelica.Media.Water.StandardWater extends Modelica.Media.Interfaces.PartialTwoPhaseMedium "Component media";
          parameter Modelica.SIunits.Length pipeDiameter = 0.01;
          BaseClasses.TankWith3InletOutletArraysWithEvaporatorCondensor B5(redeclare package Medium = BatchMedium, height = 0.5, n_SidePorts = 1, V0 = 0.001, n_BottomPorts = 1, bottom_pipeArea = {0.0001}, top_pipeArea = {0.0001}, n_TopPorts = 1, min_level_for_heating = 0.0001, level_start = 0.0009, crossArea = 0.05, initType = BaseClasses.Init.InitialValues, side_pipeArea = {0.0001}, redeclare model HeatTransfer = Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.IdealHeatTransfer(k = 4.9)) annotation(Placement(transformation(extent = {{ -110, -60},{ -30, -20}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveDiscrete V12(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(Placement(transformation(origin = { -90,2}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Fluid.Valves.ValveDiscrete V15(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(Placement(transformation(origin = { -90, -82}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow HeatB5 annotation(Placement(transformation(extent = {{ -150, -50},{ -130, -30}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow CoolingB7 annotation(Placement(transformation(extent = {{ -150, -130},{ -130, -110}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow CoolingB6 annotation(Placement(transformation(origin = {110, -60}, extent = {{ -10, -10},{10,10}}, rotation = 180)));
          Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.Controller controller(Transition3(enableTimer = true, waitTime = 60), Transition7(condition = true, enableTimer = true, waitTime = 300)) annotation(Placement(transformation(extent = {{60,38},{100,78}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -300, -300},{300,300}}), graphics), experiment(StopTime = 3600), __Dymola_Commands(file = "Scripts/Fluid/AST_BatchPlant_StandardWater/plot level.mos" "plot level"), Documentation(info = "<html>

<p align=\"center\">
<img src=\"../Images/Fluid/Examples/BatchPlant_StandardWater.png\" border=\"1\">
</p>

</html>"));
          Modelica.Fluid.Valves.ValveDiscrete V11(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(Placement(transformation(extent = {{ -50,80},{ -70,100}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveDiscrete V8(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(Placement(transformation(origin = { -90,160}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Modelica.Fluid.Valves.ValveDiscrete V9(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(Placement(transformation(origin = {70,160}, extent = {{10,10},{ -10, -10}}, rotation = 90)));
          Modelica.Fluid.Valves.ValveDiscrete V2(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(Placement(transformation(extent = {{ -60,230},{ -40,250}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveDiscrete V4(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(Placement(transformation(extent = {{40,230},{20,250}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveDiscrete V3(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(Placement(transformation(extent = {{ -150,210},{ -130,230}}, rotation = 0)));
          Fittings.TeeJunctionIdeal volume2(redeclare package Medium = BatchMedium) annotation(Placement(transformation(origin = { -180,220}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
          Modelica.Fluid.Valves.ValveDiscrete V6(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(Placement(transformation(extent = {{130,210},{110,230}}, rotation = 0)));
          Fittings.TeeJunctionIdeal volume8(redeclare package Medium = BatchMedium) annotation(Placement(transformation(origin = {160,220}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Fluid.Valves.ValveDiscrete V23(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(Placement(transformation(origin = { -110, -250}, extent = {{ -10,10},{10, -10}}, rotation = 180)));
          Modelica.Fluid.Valves.ValveDiscrete V1(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(Placement(transformation(origin = { -180,110}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
          Modelica.Fluid.Valves.ValveDiscrete V22(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(Placement(transformation(origin = { -180, -170}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
          Modelica.Fluid.Valves.ValveDiscrete V5(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(Placement(transformation(origin = {160,110}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
          Modelica.Fluid.Valves.ValveDiscrete V24(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(Placement(transformation(origin = {90, -250}, extent = {{10,10},{ -10, -10}}, rotation = 180)));
          Modelica.Fluid.Valves.ValveDiscrete V25(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(Placement(transformation(origin = {160, -170}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
          Modelica.Fluid.Valves.ValveDiscrete V20(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(Placement(transformation(origin = {60, -210}, extent = {{10,10},{ -10, -10}}, rotation = 90)));
          Modelica.Fluid.Valves.ValveDiscrete V19(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(Placement(transformation(origin = {10, -200}, extent = {{ -10,10},{10, -10}}, rotation = 180)));
          Modelica.Fluid.Valves.ValveDiscrete V10(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(Placement(transformation(origin = { -21, -170}, extent = {{10,10},{ -10, -10}}, rotation = 90)));
          Modelica.Fluid.Valves.ValveDiscrete V21(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(Placement(transformation(origin = {30, -250}, extent = {{10,10},{ -10, -10}}, rotation = 180)));
          Modelica.Fluid.Fittings.TeeJunctionVolume volume5(redeclare package Medium = BatchMedium, V = 0.001) annotation(Placement(transformation(extent = {{50, -260},{70, -240}}, rotation = 0)));
          Modelica.Fluid.Valves.ValveDiscrete V18(redeclare package Medium = BatchMedium, m_flow_nominal = 1, dp_nominal = 100) annotation(Placement(transformation(origin = { -50, -200}, extent = {{10,10},{ -10, -10}}, rotation = 180)));
          Machines.PrescribedPump P1(redeclare package Medium = BatchMedium, m_flow_start = 0.1, redeclare function flowCharacteristic = Modelica.Fluid.Machines.BaseClasses.PumpCharacteristics.quadraticFlow(V_flow_nominal = {0,0.001,0.0015}, head_nominal = {100,50,0}), N_nominal = 200, use_N_in = true, show_NPSHa = true, V(displayUnit = "ml") = 0.0001, checkValve = true, p_a_start = 100000, p_b_start = 100000, energyDynamics = Modelica.Fluid.Types.Dynamics.DynamicFreeInitial, massDynamics = Modelica.Fluid.Types.Dynamics.DynamicFreeInitial) annotation(Placement(transformation(extent = {{ -140, -260},{ -160, -240}}, rotation = 0)));
          Machines.PrescribedPump P2(redeclare package Medium = BatchMedium, m_flow_start = 0.1, redeclare function flowCharacteristic = Modelica.Fluid.Machines.BaseClasses.PumpCharacteristics.quadraticFlow(V_flow_nominal = {0,0.001,0.0015}, head_nominal = {100,50,0}), N_nominal = 200, use_N_in = true, show_NPSHa = true, V(displayUnit = "ml") = 0.0001, checkValve = true, p_a_start = 100000, p_b_start = 100000, energyDynamics = Modelica.Fluid.Types.Dynamics.DynamicFreeInitial, massDynamics = Modelica.Fluid.Types.Dynamics.DynamicFreeInitial) annotation(Placement(transformation(extent = {{120, -260},{140, -240}}, rotation = 0)));
          Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts B1(level_start = 0.2, redeclare package Medium = BatchMedium, height = 0.5, crossArea = 0.05, V0 = 0.0001, nTopPorts = 1, nPorts = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.011, height = 0)}, stiffCharacteristicForEmptyPort = false) annotation(Placement(transformation(extent = {{ -110,180},{ -70,220}}, rotation = 0)));
          inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{180,250},{200,270}}, rotation = 0)));
          Modelica.Blocks.Logical.TriggeredTrapezoid P1_on(amplitude = 100, rising = 0) annotation(Placement(transformation(extent = {{ -122, -230},{ -142, -210}}, rotation = 0)));
          Modelica.Blocks.Logical.TriggeredTrapezoid P2_on(amplitude = 50, rising = 0) annotation(Placement(transformation(extent = {{100, -230},{120, -210}}, rotation = 0)));
          Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts B2(level_start = 0.2, redeclare package Medium = BatchMedium, height = 0.5, crossArea = 0.05, V0 = 0.0001, nTopPorts = 1, nPorts = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.011, height = 0)}, stiffCharacteristicForEmptyPort = false) annotation(Placement(transformation(extent = {{50,180},{90,220}}, rotation = 0)));
          Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts B3(redeclare package Medium = BatchMedium, height = 0.5, crossArea = 0.05, V0 = 0.0001, nTopPorts = 2, nPorts = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.011, height = 0),Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.011, height = 0)}, level_start = 0.02, stiffCharacteristicForEmptyPort = false) annotation(Placement(transformation(extent = {{ -30,100},{10,140}}, rotation = 0)));
          Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts B4(redeclare package Medium = BatchMedium, height = 0.5, crossArea = 0.05, V0 = 0.0001, level_start = 0.015, nTopPorts = 1, nPorts = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.011, height = 0)}, stiffCharacteristicForEmptyPort = false) annotation(Placement(transformation(extent = {{ -110,30},{ -70,70}}, rotation = 0)));
          Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts B7(redeclare package Medium = BatchMedium, V0 = 0.0001, nTopPorts = 1, nPorts = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.011, height = 0)}, level_start = 0.009, height = 0.5, crossArea = 0.05, stiffCharacteristicForEmptyPort = false, use_HeatTransfer = true, T_start = 298, redeclare model HeatTransfer = Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.IdealHeatTransfer(k = 4.9)) annotation(Placement(transformation(extent = {{ -110, -140},{ -70, -100}}, rotation = 0)));
          Pipes.DynamicPipe pipeB1B2(redeclare package Medium = BatchMedium, length = 1, diameter = pipeDiameter, height_ab = 0) annotation(Placement(transformation(extent = {{0,230},{ -20,250}}, rotation = 0)));
          Pipes.StaticPipe pipeB1B3(redeclare package Medium = BatchMedium, length = 1, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow, diameter = pipeDiameter, height_ab =  -0.1) annotation(Placement(transformation(extent = {{ -70,134},{ -50,154}}, rotation = 0)));
          Pipes.StaticPipe pipeB2B3(redeclare package Medium = BatchMedium, length = 1, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow, diameter = pipeDiameter, height_ab =  -0.1) annotation(Placement(transformation(extent = {{50,134},{30,154}}, rotation = 0)));
          Pipes.StaticPipe pipeB1B1(redeclare package Medium = BatchMedium, diameter = pipeDiameter, length = 1, height_ab =  -0.5) annotation(Placement(transformation(origin = { -8,10}, extent = {{10,10},{ -10, -10}}, rotation = 90)));
          Pipes.StaticPipe pipeB6Pump(redeclare package Medium = BatchMedium, length = 0.5, diameter = pipeDiameter, height_ab =  -0.5) annotation(Placement(transformation(origin = {60, -110}, extent = {{10,10},{ -10, -10}}, rotation = 90)));
          Pipes.StaticPipe pipeB7Pump(redeclare package Medium = BatchMedium, length = 1, diameter = pipeDiameter, height_ab =  -0.1) annotation(Placement(transformation(origin = { -90, -170}, extent = {{10,10},{ -10, -10}}, rotation = 90)));
          Pipes.DynamicPipe pipePump1B1(redeclare package Medium = BatchMedium, diameter = pipeDiameter, height_ab = 3, length = 3) annotation(Placement(transformation(origin = { -180,10}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
          Pipes.DynamicPipe pipePump2B2(redeclare package Medium = BatchMedium, diameter = pipeDiameter, height_ab = 3, length = 3) annotation(Placement(transformation(origin = {160,10}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
          Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts B6(redeclare package Medium = BatchMedium, V0 = 0.0001, nTopPorts = 1, height = 0.5, crossArea = 0.05, level_start = 0.02, nPorts = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.011, height = 0)}, stiffCharacteristicForEmptyPort = false, use_HeatTransfer = true, T_start = 298, redeclare model HeatTransfer = Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.IdealHeatTransfer(k = 4.9)) annotation(Placement(transformation(extent = {{80, -80},{40, -40}}, rotation = 0)));
          Fittings.MultiPort multiPort(redeclare package Medium = BatchMedium, nPorts_b = 3) annotation(Placement(transformation(extent = {{4,10},{ -4, -10}}, rotation =  -90, origin = { -20, -228})));
          Modelica.Fluid.Fittings.TeeJunctionVolume volume4(redeclare package Medium = BatchMedium, V = 0.001) annotation(Placement(transformation(extent = {{ -30, -260},{ -10, -240}}, rotation = 0)));

        equation 
          controller.sensors.LIS_301 = B3.level;
          controller.sensors.QI_302 = 0;
          controller.sensors.LIS_501 = B5.level;
          controller.sensors.QIS_502 = 0;
          controller.sensors.TI_503 = B5.medium.T;
          controller.sensors.LIS_601 = B6.level;
          controller.sensors.TIS_602 = B6.medium.T;
          controller.sensors.LIS_701 = B7.level;
          controller.sensors.TIS_702 = B7.medium.T;
          P1_on.u = controller.actuators.P1;
          P2_on.u = controller.actuators.P2;
          V1.open = controller.actuators.V1;
          V2.open = controller.actuators.V2;
          V3.open = controller.actuators.V3;
          V4.open = controller.actuators.V4;
          V5.open = controller.actuators.V5;
          V6.open = controller.actuators.V6;
          V8.open = controller.actuators.V8;
          V9.open = controller.actuators.V9;
          V10.open = controller.actuators.V10;
          V11.open = controller.actuators.V11;
          V12.open = controller.actuators.V12;
          V15.open = controller.actuators.V15;
          V18.open = controller.actuators.V18;
          V19.open = controller.actuators.V19;
          V20.open = controller.actuators.V20;
          V21.open = controller.actuators.V21;
          V22.open = controller.actuators.V22;
          V23.open = controller.actuators.V23;
          V24.open = controller.actuators.V24;
          V25.open = controller.actuators.V25;
          HeatB5.Q_flow = if controller.actuators.T5_Heater then 20000 else 0;
          CoolingB6.Q_flow = if controller.actuators.T6_Cooling then  -2000 else 0;
          CoolingB7.Q_flow = if controller.actuators.T7_Cooling then  -2000 else 0;
          connect(V24.port_b,P2.port_a) annotation(Line(points = {{100, -250},{120, -250}}, color = {0,127,255}));
          connect(V15.port_a,B5.BottomFluidPort[1]) annotation(Line(points = {{ -90, -72},{ -90, -60.4}}, color = {0,127,255}));
          connect(P1_on.y,P1.N_in) annotation(Line(points = {{ -143, -220},{ -150, -220},{ -150, -240}}, color = {0,0,127}));
          connect(P2_on.y,P2.N_in) annotation(Line(points = {{121, -220},{130, -220},{130, -240}}, color = {0,0,127}));
          connect(B4.ports[1],V12.port_b) annotation(Line(points = {{ -90,29},{ -90,21},{ -90,21},{ -90,12}}, color = {0,127,255}));
          connect(CoolingB7.port,B7.heatPort) annotation(Line(points = {{ -130, -120},{ -110, -120}}, color = {191,0,0}));
          connect(pipeB1B2.port_a,V4.port_b) annotation(Line(points = {{0,240},{20,240}}, color = {0,127,255}));
          connect(B5.TopFluidPort[1],V12.port_a) annotation(Line(points = {{ -90, -19.6},{ -90, -8}}, color = {0,0,255}));
          connect(V15.port_b,B7.topPorts[1]) annotation(Line(points = {{ -90, -92},{ -90, -96},{ -90, -99},{ -90, -99}}, color = {0,127,255}));
          connect(pipePump1B1.port_a,V22.port_b) annotation(Line(points = {{ -180,0},{ -180, -160}}, color = {0,127,255}));
          connect(V25.port_b,pipePump2B2.port_a) annotation(Line(points = {{160, -160},{160,0}}, color = {0,127,255}));
          connect(B6.topPorts[1],B5.Condensed) annotation(Line(points = {{60, -39},{60, -28},{ -29.6, -28}}, color = {0,127,255}));
          connect(CoolingB6.port,B6.heatPort) annotation(Line(points = {{100, -60},{80, -60}}, color = {191,0,0}));
          connect(HeatB5.port,B5.heatPort) annotation(Line(points = {{ -130, -40},{ -110, -40}}, color = {191,0,0}));
          connect(V1.port_b,volume2.port_1) annotation(Line(points = {{ -180,120},{ -180,210}}, color = {0,127,255}, smooth = Smooth.None));
          connect(volume8.port_2,V4.port_a) annotation(Line(points = {{160,230},{160,240},{40,240}}, color = {0,127,255}, smooth = Smooth.None));
          connect(volume8.port_1,V5.port_b) annotation(Line(points = {{160,210},{160,120}}, color = {0,127,255}, smooth = Smooth.None));
          connect(volume5.port_1,V21.port_b) annotation(Line(points = {{50, -250},{40, -250}}, color = {0,127,255}, smooth = Smooth.None));
          connect(volume5.port_2,V24.port_a) annotation(Line(points = {{70, -250},{80, -250}}, color = {0,127,255}, smooth = Smooth.None));
          connect(P2.port_b,V25.port_a) annotation(Line(points = {{140, -250},{160, -250},{160, -180}}, color = {0,127,255}, smooth = Smooth.None));
          connect(V22.port_a,P1.port_b) annotation(Line(points = {{ -180, -180},{ -180, -250},{ -160, -250}}, color = {0,127,255}, smooth = Smooth.None));
          connect(V5.port_a,pipePump2B2.port_b) annotation(Line(points = {{160,100},{160,20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(V1.port_a,pipePump1B1.port_b) annotation(Line(points = {{ -180,100},{ -180,20}}, color = {0,127,255}, smooth = Smooth.None));
          connect(P1.port_a,V23.port_b) annotation(Line(points = {{ -140, -250},{ -120, -250}}, color = {0,127,255}, smooth = Smooth.None));
          connect(V23.port_a,volume4.port_1) annotation(Line(points = {{ -100, -250},{ -30, -250}}, color = {0,127,255}, smooth = Smooth.None));
          connect(volume4.port_2,V21.port_a) annotation(Line(points = {{ -10, -250},{20, -250}}, color = {0,127,255}, smooth = Smooth.None));
          connect(multiPort.port_a,volume4.port_3) annotation(Line(points = {{ -20, -232},{ -20, -240}}, color = {0,127,255}, smooth = Smooth.None));
          connect(multiPort.ports_b[3],V19.port_b) annotation(Line(points = {{ -17.3333, -224},{ -17.3333, -200},{0, -200}}, color = {0,127,255}, smooth = Smooth.None));
          connect(V18.port_b,multiPort.ports_b[1]) annotation(Line(points = {{ -40, -200},{ -22.6667, -200},{ -22.6667, -224}}, color = {0,127,255}, smooth = Smooth.None));
          connect(V10.port_b,multiPort.ports_b[2]) annotation(Line(points = {{ -21, -180},{ -20, -180},{ -20, -224}}, color = {0,127,255}, smooth = Smooth.None));
          connect(V11.port_b,B4.topPorts[1]) annotation(Line(points = {{ -70,90},{ -90,90},{ -90,71}}, color = {0,127,255}, smooth = Smooth.None));
          connect(B3.ports[1],V11.port_a) annotation(Line(points = {{ -12,99},{ -12,90},{ -50,90}}, color = {0,127,255}, smooth = Smooth.None));
          connect(V20.port_b,volume5.port_3) annotation(Line(points = {{60, -220},{60, -240}}, color = {0,127,255}, smooth = Smooth.None));
          connect(V2.port_b,pipeB1B2.port_b) annotation(Line(points = {{ -40,240},{ -20,240}}, color = {0,127,255}, smooth = Smooth.None));
          connect(volume2.port_2,V2.port_a) annotation(Line(points = {{ -180,230},{ -180,240},{ -60,240}}, color = {0,127,255}, smooth = Smooth.None));
          connect(volume2.port_3,V3.port_a) annotation(Line(points = {{ -170,220},{ -150,220}}, color = {0,127,255}, smooth = Smooth.None));
          connect(V3.port_b,B1.topPorts[1]) annotation(Line(points = {{ -130,220},{ -120,220},{ -120,230},{ -90,230},{ -90,221}}, color = {0,127,255}, smooth = Smooth.None));
          connect(volume8.port_3,V6.port_a) annotation(Line(points = {{150,220},{130,220}}, color = {0,127,255}, smooth = Smooth.None));
          connect(V6.port_b,B2.topPorts[1]) annotation(Line(points = {{110,220},{100,220},{100,230},{70,230},{70,221}}, color = {0,127,255}, smooth = Smooth.None));
          connect(B2.ports[1],V9.port_a) annotation(Line(points = {{70,179},{70,175},{70,175},{70,170}}, color = {0,127,255}, smooth = Smooth.None));
          connect(V9.port_b,pipeB2B3.port_a) annotation(Line(points = {{70,150},{70,144},{50,144}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipeB2B3.port_b,B3.topPorts[2]) annotation(Line(points = {{30,144},{ -8,144},{ -8,141},{ -8,141}}, color = {0,127,255}, smooth = Smooth.None));
          connect(B1.ports[1],V8.port_a) annotation(Line(points = {{ -90,179},{ -90,175},{ -90,175},{ -90,170}}, color = {0,127,255}, smooth = Smooth.None));
          connect(V8.port_b,pipeB1B3.port_a) annotation(Line(points = {{ -90,150},{ -90,144},{ -70,144}}, color = {0,127,255}, smooth = Smooth.None));
          connect(B3.topPorts[1],pipeB1B3.port_b) annotation(Line(points = {{ -12,141},{ -12,141},{ -12,144},{ -50,144}}, color = {0,127,255}, smooth = Smooth.None));
          connect(B7.ports[1],pipeB7Pump.port_a) annotation(Line(points = {{ -90, -141},{ -90, -150},{ -90, -150},{ -90, -160}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipeB7Pump.port_b,V18.port_a) annotation(Line(points = {{ -90, -180},{ -90, -200},{ -60, -200}}, color = {0,127,255}, smooth = Smooth.None));
          connect(B6.ports[1],pipeB6Pump.port_a) annotation(Line(points = {{60, -81},{60, -90},{60, -90},{60, -100}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipeB6Pump.port_b,V20.port_a) annotation(Line(points = {{60, -120},{60, -200}}, color = {0,127,255}, smooth = Smooth.None));
          connect(V19.port_a,pipeB6Pump.port_b) annotation(Line(points = {{20, -200},{30, -200},{30, -160},{60, -160},{60, -120}}, color = {0,127,255}, smooth = Smooth.None));
          connect(pipeB1B1.port_b,V10.port_a) annotation(Line(points = {{ -8,0},{ -8, -80},{ -21, -80},{ -21, -160}}, color = {0,127,255}, smooth = Smooth.None));
          connect(B3.ports[2],pipeB1B1.port_a) annotation(Line(points = {{ -8,99},{ -8,20},{ -8,20}}, color = {0,127,255}, smooth = Smooth.None));
        end BatchPlant_StandardWater;
        package BaseClasses
          extends Modelica.Fluid.Icons.BaseClassLibrary;
          block TriggeredTrapezoid "Triggered trapezoid generator"
            extends Modelica.Blocks.Interfaces.partialBooleanBlockIcon;
            parameter Real amplitude = 1 "Amplitude of trapezoid";
            parameter Modelica.SIunits.Time rising(final min = 0) = 0 "Rising duration of trapezoid";
            parameter Modelica.SIunits.Time falling(final min = 0) = rising "Falling duration of trapezoid";
            parameter Real offset = 0 "Offset of output signal";
            Modelica.Blocks.Interfaces.BooleanInput u "Connector of Boolean input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
            Modelica.Blocks.Interfaces.RealOutput y "Connector of Real output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
            annotation(Icon(graphics = {Line(points = {{ -60, -70},{ -60, -70},{ -30,40},{8,40},{40, -70},{40, -70}}),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -70},{ -60, -70},{ -60,24},{8,24},{8, -70},{60, -70}}, color = {255,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -80, -20},{ -60, -20},{ -30,40},{8,40},{40, -20},{60, -20}}),Line(points = {{ -90, -70},{82, -70}}, color = {0,0,0}),Line(points = {{ -80,68},{ -80, -80}}, color = {0,0,0}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -68},{ -60, -68},{ -60, -42},{8, -42},{8, -68},{60, -68}}, color = {255,0,255}),Line(points = {{ -60,40},{ -60, -42}}, color = {0,0,0}, pattern = LinePattern.Dot),Line(points = {{8, -42},{8,40}}, color = {0,0,0}, pattern = LinePattern.Dot),Line(points = {{ -58,40},{ -28,40}}, color = {0,0,0}),Line(points = {{8, -20},{40, -20}}, color = {0,0,0}),Line(points = {{ -20,40},{ -20, -20}}, color = {0,0,0}),Line(points = {{ -20, -20},{ -20, -70}}, color = {0,0,0}),Text(extent = {{ -42,48},{ -42,38}}, lineColor = {0,0,0}, textString = "rising"),Text(extent = {{24, -10},{24, -20}}, lineColor = {0,0,0}, textString = "falling"),Polygon(points = {{ -58,40},{ -54,42},{ -54,38},{ -58,40}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -30,40},{ -34,42},{ -34,38},{ -30,40}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{8, -20},{12, -18},{12, -22},{8, -20}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{40, -20},{36, -18},{36, -22},{40, -20}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -22, -24},{ -20, -20},{ -18, -24},{ -22, -24}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -18, -66},{ -22, -66},{ -20, -70},{ -18, -66}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -22,36},{ -20,40},{ -18,36},{ -22,36}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -18, -16},{ -22, -16},{ -20, -20},{ -18, -16}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -40,6},{0, -4}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -20,6},{ -20, -4}}, lineColor = {0,0,0}, textString = "amplitude"),Rectangle(extent = {{ -40, -48},{0, -58}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -20, -48},{ -20, -58}}, lineColor = {0,0,0}, textString = "offset"),Text(extent = {{60, -82},{94, -92}}, lineColor = {0,0,0}, textString = "time"),Text(extent = {{ -88, -4},{ -54, -14}}, lineColor = {0,0,0}, textString = "y"),Text(extent = {{ -88, -46},{ -54, -56}}, lineColor = {0,0,0}, textString = "u"),Polygon(points = {{40,60},{36,62},{36,58},{40,60}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -56,60},{36,60}}, color = {0,0,0}),Polygon(points = {{ -60,60},{ -56,62},{ -56,58},{ -60,60}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -16,72},{ -16,62}}, lineColor = {0,0,0}, textString = "y_high")}), Documentation(info = "<HTML>
<p>The block TriggeredTrapezoid has a boolean input and a real
output signal and requires the parameters <i>amplitude</i>,
<i>rising</i>, <i>falling</i> and <i>offset</i>. The
output signal <b>y</b> represents a trapezoidal signal dependent on the
input signal <b>u</b>.
</p>
<p>The behaviour is as follows: Assume the initial input to be false. In this
case, the output will be <i>offset</i>. After a rising edge (i.e. the input
changes from false to true), the output is rising during <i>rising</i> to the
sum of <i>offset</i> and <i>amplitude</i>. In contrast, after a falling
edge (i.e. the input changes from true to false), the output is falling
during <i>falling</i> to a value of <i>offset</i>.
</p>
<p>Note, that the case of edges before expiration of rising or falling is
handled properly.</p>
</HTML>
"));
          protected 
            discrete Real endValue "Value of y at time of recent edge";
            discrete Real rate "Current rising/falling rate";
            discrete Modelica.SIunits.Time T "Predicted time of output reaching endValue";
          public 
            Modelica.Blocks.Interfaces.BooleanOutput y_high annotation(Placement(transformation(extent = {{100, -90},{120, -70}}, rotation = 0)));

          initial equation 
            pre(y) = 0;

          equation 
            y_high = time < T;
            y = if y_high then endValue - (T - time) * rate else endValue;
            when {initial(),u,not u} then
                          endValue = if u then offset + amplitude else offset;
              rate = if u and rising > 0 then amplitude / rising else if not u and falling > 0 then  -amplitude / falling else 0;
              T = if u and not rising > 0 or not u and not falling > 0 or not abs(amplitude) > 0 or initial() then time else time + (endValue - pre(y)) / rate;
            
            end when;
          end TriggeredTrapezoid;
          block setReal "Set output signal to a time varying Real expression"
            Modelica.Blocks.Interfaces.RealInput u "Set value of Real input" annotation(Dialog(group = "Time varying input signal"), Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
            annotation(Window(x = 0.29, y = 0.23, width = 0.6, height = 0.6), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100,40},{100, -40}}, lineColor = {0,0,0}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised),Text(extent = {{ -96,15},{96, -15}}, lineColor = {0,0,0}, textString = "%u"),Text(extent = {{ -150,90},{140,50}}, textString = "%name")}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>

</html>"));
          end setReal;
          model TankWith3InletOutletArraysWithEvaporatorCondensor "Tank with Heating and Evaporation"
            import Modelica.SIunits.Conversions.*;
            import Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.Init;
            replaceable package Medium = Modelica.Media.Water.WaterIF97_ph extends Modelica.Media.Interfaces.PartialTwoPhaseMedium "Medium in the component" annotation(choicesAllMatching = true);
            parameter Modelica.SIunits.Area crossArea "Tank area";
            parameter SI.Area top_pipeArea[n_TopPorts] "Area of outlet pipe";
            parameter SI.Area side_pipeArea[n_SidePorts] "Area of outlet pipe";
            parameter SI.Area bottom_pipeArea[n_BottomPorts] "Area of outlet pipe";
            parameter Modelica.SIunits.Height height(min = 0) = 10 "Height of Tank";
            parameter SI.Volume V0 = 0 "Volume of the liquid when the level is zero";
            constant Modelica.SIunits.Acceleration g = Modelica.Constants.g_n;
            parameter Real side_heights[n_SidePorts] = zeros(n_SidePorts);
            parameter Real bottom_heights[n_BottomPorts] = zeros(n_BottomPorts);
            parameter Real top_heights[n_TopPorts] = fill(height, n_TopPorts);
            parameter SI.Height level_start(min = 0) "Initial tank level" annotation(Dialog(tab = "Initialization"));
            parameter Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.Init initType = Init.GuessValues "Initialization option" annotation(Dialog(tab = "Initialization"));
            parameter Boolean use_T_start = true "Use T_start if true, otherwise h_start" annotation(Dialog(tab = "Initialization"), Evaluate = true);
            parameter Medium.Temperature T_start = if use_T_start then 293.15 else Medium.temperature_phX(p_ambient, h_start, X_start) "Start value of temperature" annotation(Dialog(tab = "Initialization", enable = use_T_start));
            parameter Medium.SpecificEnthalpy h_start = if use_T_start then Medium.specificEnthalpy_pTX(p_ambient, T_start, X_start[1:Medium.nXi]) else 10000.0 "Start value of specific enthalpy" annotation(Dialog(tab = "Initialization", enable = not use_T_start));
            parameter Medium.MassFraction X_start[Medium.nX] = Medium.reference_X "Start value of mass fractions m_i/m" annotation(Dialog(tab = "Initialization", enable = Medium.nXi > 0));
            parameter Medium.AbsolutePressure p_ambient = 101325 "Tank surface pressure";
            parameter Medium.Temperature T_ambient = 293.15 "Tank surface Temperature";
            parameter Integer n_TopPorts = 1 "number of Top connectors";
            parameter Integer n_SidePorts = 1 "number of side connectors";
            parameter Integer n_BottomPorts = 1 "number of bootom connectors";
            Medium.BaseProperties medium(preferredMediumStates = true, p(start = p_ambient), T(start = T_start), Xi(start = X_start[1:Medium.nXi]));
            Modelica.SIunits.Height level(stateSelect = StateSelect.prefer, min = 0, max = height) "Level height of tank";
            SI.Volume V(stateSelect = StateSelect.never) "Actual tank volume";
            SI.Energy U "Internal energy of tank volume";
            Real m(quantity = Medium.mediumName, unit = "kg") "Mass of tank volume";
            Real mXi[Medium.nXi](quantity = Medium.substanceNames, each unit = "kg") "Component masses of the independent substances";
            Real H_flow_BottomPorts[n_BottomPorts];
            Real H_flow_SidePorts[n_SidePorts];
            Real H_flow_TopPorts[n_TopPorts];
            Real m_flow_BottomPorts[n_BottomPorts];
            Real m_flow_SidePorts[n_SidePorts];
            Real m_flow_TopPorts[n_TopPorts];
            Real m_flow_BottomPorts_pos[n_BottomPorts];
            Real m_flow_SidePorts_pos[n_SidePorts];
            Real m_flow_TopPorts_pos[n_TopPorts];
            Real m_flow_pos;
            Medium.MassFlowRate mXi_flow_topPorts[n_TopPorts,Medium.nXi];
            Medium.MassFlowRate mXi_flowBottomPorts[n_BottomPorts,Medium.nXi];
            Medium.MassFlowRate mXi_flow_sidePorts[n_SidePorts,Medium.nXi];
            Modelica.Fluid.Interfaces.FluidPort_b BottomFluidPort[n_BottomPorts](redeclare package Medium = Medium, m_flow(each start = 0)) annotation(Placement(transformation(origin = { -100, -102}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Modelica.Fluid.Interfaces.FluidPort_a TopFluidPort[n_TopPorts](redeclare package Medium = Medium, m_flow(each start = 0)) annotation(Placement(transformation(extent = {{ -110,92},{ -90,112}}, rotation = 0)));
            Modelica.Fluid.Interfaces.FluidPort_b SideFluidPort[n_SidePorts](redeclare package Medium = Medium, m_flow(each start = 0)) annotation(Placement(transformation(extent = {{0, -10},{20,10}}, rotation = 0)));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.InnerTank InnerTankTop[n_TopPorts](each h = medium.h, each p_ambient = p_ambient, each d = medium.d, each Xi = medium.Xi, aboveLevel = array(level - top_heights[i] for i in 1:n_TopPorts), pipeArea = array(top_pipeArea[i] for i in 1:n_TopPorts), redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -140,60},{ -120,80}}, rotation = 0)));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.InnerTank InnerTankSide[n_SidePorts](each h = medium.h, each p_ambient = p_ambient, each d = medium.d, each Xi = medium.Xi, aboveLevel = array(level - side_heights[i] for i in 1:n_SidePorts), pipeArea = array(side_pipeArea[i] for i in 1:n_SidePorts), redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -20,0},{0,20}}, rotation = 0)));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.InnerTank InnerTankBottom[n_BottomPorts](each h = medium.h, each p_ambient = p_ambient, each d = medium.d, each Xi = medium.Xi, aboveLevel = array(level - bottom_heights[i] for i in 1:n_BottomPorts), pipeArea = array(bottom_pipeArea[i] for i in 1:n_BottomPorts), redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -80, -80},{ -60, -60}}, rotation = 0)));
            Modelica.Fluid.Interfaces.FluidPort_b Condensed(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{192,50},{212,70}}, rotation = 0)));
            replaceable model HeatTransfer = Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.IdealHeatTransfer extends Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.PartialVesselHeatTransfer "Wall heat transfer" annotation(Dialog(tab = "Assumptions", group = "Heat transfer", enable = use_HeatTransfer), choicesAllMatching = true);
            HeatTransfer heatTransfer(redeclare final package Medium = Medium, final n = 1, final states = {medium.state}, surfaceAreas = {crossArea + 2 * sqrt(crossArea * Modelica.Constants.pi) * level}, final use_k = true) annotation(Placement(transformation(extent = {{ -10, -10},{30,30}}, rotation = 90, origin = { -50, -10})));
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort annotation(Placement(transformation(extent = {{ -210, -10},{ -190,10}})));
            parameter Real min_level_for_heating;
            Medium.SaturationProperties sat "State vector to compute saturation properties";
            Medium.SpecificEnthalpy h_v = Medium.dewEnthalpy(sat) "specific enthalpy of vapour";
            Medium.SpecificEnthalpy h_l = Medium.bubbleEnthalpy(sat) "specific enthalpy of liquid";
            Medium.SpecificEnthalpy h "'is'specific enthalpy of liquid";
            Medium.Density rho_v = Medium.dewDensity(sat) "density in vapour phase";
            Medium.Density rho_l = Medium.bubbleDensity(sat) "density in liquid phase";
            Medium.Density rho "'is' density in liquid phase";

          equation 
            H_flow_TopPorts = InnerTankTop.H_flow;
            m_flow_TopPorts = InnerTankTop.m_flow;
            mXi_flow_topPorts = InnerTankTop.mXi_flow;
            H_flow_SidePorts = InnerTankSide.H_flow;
            m_flow_SidePorts = InnerTankSide.m_flow;
            mXi_flow_sidePorts = InnerTankSide.mXi_flow;
            H_flow_BottomPorts = InnerTankBottom.H_flow;
            m_flow_BottomPorts = InnerTankBottom.m_flow;
            mXi_flowBottomPorts = InnerTankBottom.mXi_flow;
            for i in 1:n_BottomPorts loop
            m_flow_BottomPorts_pos[i] = if m_flow_BottomPorts[i] > 0 then m_flow_BottomPorts[i] else 0;

            end for;
            for i in 1:n_SidePorts loop
            m_flow_SidePorts_pos[i] = if m_flow_SidePorts[i] > 0 then m_flow_SidePorts[i] else 0;

            end for;
            for i in 1:n_TopPorts loop
            m_flow_TopPorts_pos[i] = if m_flow_TopPorts[i] > 0 then m_flow_TopPorts[i] else 0;

            end for;
            for i in 1:n_BottomPorts loop
            connect(InnerTankBottom[i].port,BottomFluidPort[i]) annotation(Line(points = {{ -70, -81},{ -70, -102},{ -100, -102}}, color = {0,0,255}));

            end for;
            for i in 1:n_TopPorts loop
            connect(InnerTankTop[i].port,TopFluidPort[i]) annotation(Line(points = {{ -130,59},{ -92,59},{ -92,102},{ -100,102}}, color = {0,0,255}));

            end for;
            for i in 1:n_SidePorts loop
            connect(InnerTankSide[i].port,SideFluidPort[i]) annotation(Line(points = {{ -10, -1},{24, -1},{24,0},{10,0}}, color = {0,0,255}));

            end for;
            medium.p = p_ambient;
            der(m) = sum(m_flow_BottomPorts) + sum(m_flow_SidePorts) + sum(m_flow_TopPorts) + Condensed.m_flow;
            U = m * medium.h - p_ambient * V "Internal energy of fluid";
            m = V * medium.d "Mass of fluid";
            V = crossArea * level + V0 "Volume of fluid";
            mXi = m * medium.Xi "Mass of fluid components";
            sat.psat = medium.p;
            sat.Tsat = Medium.saturationTemperature(medium.p);
            if noEvent(medium.T < sat.Tsat) then
              if Medium.singleState then
                der(U) = sum(H_flow_BottomPorts) + sum(H_flow_SidePorts) + sum(H_flow_TopPorts) + Condensed.m_flow * actualStream(Condensed.h_outflow) + heatTransfer.Q_flows[1] "Mechanical work is neglected";

              else
                der(U) = sum(H_flow_BottomPorts) + sum(H_flow_SidePorts) + sum(H_flow_TopPorts) + Condensed.m_flow * actualStream(Condensed.h_outflow) - p_ambient * der(V) + heatTransfer.Q_flows[1];

              end if;
              Condensed.h_outflow = h;
              Condensed.m_flow = 0;
              rho = medium.d;
              h = medium.h;

            else
              if Medium.singleState then
                der(U) = sum(H_flow_BottomPorts) + sum(H_flow_SidePorts) + sum(H_flow_TopPorts) + Condensed.m_flow * actualStream(Condensed.h_outflow) "Mechanical work is neglected";

              else
                der(U) = sum(H_flow_BottomPorts) + sum(H_flow_SidePorts) + sum(H_flow_TopPorts) + Condensed.m_flow * actualStream(Condensed.h_outflow) - p_ambient * der(V);

              end if;
              Condensed.h_outflow = h;
              Condensed.m_flow =  -heatTransfer.Q_flows[1] / (h_v - h_l);
              rho = rho_l;
              h = h_l;
if noEvent(heatPort.Q_flow > 0.0) then
                              assert(noEvent(abs(m_flow_pos) <= 0.01), "Attempt to fill tank while evaporating.");
              end if;

            end if;
            m_flow_pos = sum(m_flow_TopPorts_pos) + sum(m_flow_SidePorts_pos) + sum(m_flow_BottomPorts_pos);
            for i in 1:Medium.nXi loop
            der(mXi[i]) = sum(mXi_flowBottomPorts[:,i]) + sum(mXi_flow_sidePorts[:,i]) + sum(mXi_flow_topPorts[:,i]);

            end for;
            assert(level < height, "
    Tank is overflowing.
    ");
            assert(not (heatPort.Q_flow > 0.0 and level <= min_level_for_heating), "
    Attempting to heat an empty tank
  ");

          initial equation 
            if initType == Init.GuessValues then

            elseif initType == Init.InitialValues then
              level = level_start;
              if use_T_start then
                medium.T = T_start;

              else
                medium.h = h_start;

              end if;
              medium.Xi = X_start[1:Medium.nXi];


            elseif initType == Init.SteadyStateHydraulic then
              der(level) = 0;
              if use_T_start then
                medium.T = T_start;

              else
                medium.h = h_start;

              end if;
              medium.Xi = X_start[1:Medium.nXi];


            else
              assert(false, "Unsupported initialization option");

            end if;
            annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -200, -100},{200,100}}), graphics = {Rectangle(extent = {{ -200,100},{0, -100}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.VerticalCylinder),Rectangle(extent = DynamicSelect({{ -200, -100},{0,0}}, {{ -200, -100},{0, -100 + (200 * level) / height}}), lineColor = {0,0,0}, fillColor = {85,170,255}, fillPattern = FillPattern.VerticalCylinder),Line(points = {{ -200,100},{ -200, -100},{0, -100},{0,100}}, color = {0,0,0}),Text(extent = {{ -200,84},{0,48}}, lineColor = {0,0,255}, textString = "%name"),Polygon(points = {{0,100},{200,70},{200,50},{200,50},{0,80},{0,100}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,0,255}),Polygon(points = {{20,98},{30,74},{52,84},{66,72},{86,78},{98,66},{118,74},{130,60},{144,70},{152,60},{168,66},{180,54},{196,74},{190,76},{180,64},{170,70},{156,66},{148,76},{132,68},{120,80},{100,74},{88,88},{70,78},{50,92},{32,82},{28,100},{20,98},{20,98}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {170,255,255}),Text(extent = {{ -193,30},{ -3,10}}, lineColor = {0,0,0}, textString = "level ="),Text(extent = {{ -195, -38},{ -5, -58}}, lineColor = {0,0,0}, textString = DynamicSelect("%level_start", realString(level, 1, 2)))}), Documentation(info = "<HTML>
<p>
<p>This tank has the same geometric variables as TankWith3InletOutletArrays plus the feature of a HeatPort and the possibility of evaporation.
(Assumption: The gas is condensed emidiatly afterwards so that a liquid boiling fluid is created.)
<p>The tank can be initialized with the following options:
<ul>
<li>GuessValues: no explicit initial conditions
<li>InitialValues: initial values of temperature (or specific enthalpy), composition and level are specified
<li>SteadyStateHydraulic: initial values of temperature (or specific enthalpy) and composition are specified; the initial level is determined so that levels and pressure are at steady state.
</ul>
Full steady state initialization is not supported, because the corresponding intial equations for temperature/enthalpy are undetermined (the flow rate through the port at steady state is zero).
</p>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -200, -100},{200,100}}), graphics));

          equation 
            connect(heatPort,heatTransfer.heatPorts[1]) annotation(Line(points = {{ -200,0},{ -87,0},{ -87,8.88178e-16},{ -74,8.88178e-16}}, color = {191,0,0}, smooth = Smooth.None));
          end TankWith3InletOutletArraysWithEvaporatorCondensor;
          model InnerTank
            replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium in the component" annotation(choicesAllMatching = true);
            Modelica.Fluid.Interfaces.FluidPort_a port(redeclare package Medium = Medium) annotation(Placement(transformation(origin = {0, -110}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Boolean m_flow_negative(start = true) "true= massflow out of tank";
            constant Modelica.SIunits.Acceleration g = Modelica.Constants.g_n;
            input Real aboveLevel;
            input Real d;
            input Real p_ambient;
            input Real h;
            input Medium.MassFraction Xi[Medium.nXi] "Actual mass fractions of fluid in tank" annotation(Dialog);
            input Real pipeArea;
            output Real H_flow;
            output Real m_flow;
            output Medium.MassFlowRate mXi_flow[Medium.nXi] "= port.mXi_flow (used to transform vector of connectors in vector of Real numbers)";

          equation 
            m_flow_negative = pre(m_flow_negative) and not port.p > p_ambient or port.m_flow <  -1e-06;
            if noEvent(aboveLevel > 0) then
              port.p = aboveLevel * g * d + p_ambient - smooth(2, noEvent(if noEvent(m_flow < 0) then m_flow ^ 2 / (2 * d * pipeArea ^ 2) else 0));

            else
              if pre(m_flow_negative) then
                port.m_flow = 0;

              else
                port.p = p_ambient;

              end if;

            end if;
            H_flow = port.m_flow * actualStream(port.h_outflow);
            m_flow = port.m_flow;
            mXi_flow = port.m_flow * actualStream(port.Xi_outflow);
            port.h_outflow = h;
            port.Xi_outflow = Xi;
          end InnerTank;
          model Controller
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.ControllerUtilities.Port_Sensors sensors annotation(Placement(transformation(extent = {{ -280, -40},{ -200,40}}, rotation = 0)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -200, -200},{200,200}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -200, -200},{200,200}}), graphics = {Rectangle(extent = {{ -200,200},{200, -200}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -288,286},{262,208}}, lineColor = {0,0,255}, textString = "%name"),Line(points = {{ -48,0},{0,0}}, color = {0,0,0}),Rectangle(extent = {{ -170,60},{ -50, -60}}, lineColor = {0,0,0}),Line(points = {{0,40},{0, -40}}, color = {0,0,0}),Line(points = {{0,0},{26,0}}, color = {0,0,0}),Polygon(points = {{26,10},{50,0},{26, -10},{26,10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{50,60},{170, -60}}, lineColor = {0,0,0}),Polygon(points = {{ -24,10},{0,0},{ -24, -10},{ -24,10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.ControllerUtilities.Port_Actuators actuators annotation(Placement(transformation(extent = {{200, -20},{240,20}}, rotation = 0)));
            parameter Real w_dilution = 0.003;
            parameter Real w_concentrate = 0.005;
            parameter Real startTime = 1;
            parameter Real T5_batch_level = 0.211;
            Modelica.StateGraph.InitialStep InitialStep1 annotation(Placement(transformation(extent = {{ -180,90},{ -160,110}}, rotation = 0)));
            Modelica.StateGraph.Transition Transition1(enableTimer = true, waitTime = startTime) annotation(Placement(transformation(extent = {{ -150,90},{ -130,110}}, rotation = 0)));
            Modelica.StateGraph.Step Step1 annotation(Placement(transformation(extent = {{ -120,90},{ -100,110}}, rotation = 0)));
            Modelica.StateGraph.Transition Transition2(condition = LIS_301 >= 0.13) annotation(Placement(transformation(extent = {{ -90,90},{ -70,110}}, rotation = 0)));
            Modelica.StateGraph.Step Step2 annotation(Placement(transformation(extent = {{ -60,90},{ -40,110}}, rotation = 0)));
            Modelica.StateGraph.Transition Transition3(condition = true, enableTimer = true, waitTime = 500) annotation(Placement(transformation(extent = {{ -30,90},{ -10,110}}, rotation = 0)));
            Modelica.StateGraph.Step Step3 annotation(Placement(transformation(extent = {{0,90},{20,110}}, rotation = 0)));
            Modelica.StateGraph.Transition Transition4(condition = LIS_301 <= 0.01) annotation(Placement(transformation(extent = {{30,90},{50,110}}, rotation = 0)));
            Modelica.StateGraph.Step Step4 annotation(Placement(transformation(extent = {{60,90},{80,110}}, rotation = 0)));
            Modelica.StateGraph.Transition Transition5(condition = T5_idle) annotation(Placement(transformation(extent = {{90,90},{110,110}}, rotation = 0)));
            Modelica.StateGraph.Step Step5 annotation(Placement(transformation(extent = {{120,90},{140,110}}, rotation = 0)));
            Modelica.StateGraph.Transition Transition6(condition = LIS_501 >= T5_batch_level) annotation(Placement(transformation(extent = {{150,90},{170,110}}, rotation = 0)));
            Modelica.StateGraph.Step Step6 annotation(Placement(transformation(extent = {{ -120,30},{ -100,50}}, rotation = 0)));
            Modelica.StateGraph.Transition Transition7(condition = true, enableTimer = true, waitTime = 300) annotation(Placement(transformation(extent = {{ -90,30},{ -70,50}}, rotation = 0)));
            Modelica.StateGraph.Parallel Parallel1 annotation(Placement(transformation(extent = {{ -176, -100},{194,0}}, rotation = 0)));
            Modelica.StateGraph.Step Step7 annotation(Placement(transformation(extent = {{ -122, -80},{ -102, -60}}, rotation = 0)));
            Modelica.StateGraph.Step Step8 annotation(Placement(transformation(extent = {{ -62, -80},{ -42, -60}}, rotation = 0)));
            Modelica.StateGraph.Step Step9 annotation(Placement(transformation(extent = {{ -2, -80},{18, -60}}, rotation = 0)));
            Modelica.StateGraph.Step Step10 annotation(Placement(transformation(extent = {{58, -80},{78, -60}}, rotation = 0)));
            Modelica.StateGraph.Step Step11 annotation(Placement(transformation(extent = {{118, -80},{138, -60}}, rotation = 0)));
            Modelica.StateGraph.Step Step12 annotation(Placement(transformation(extent = {{ -62, -40},{ -42, -20}}, rotation = 0)));
            Modelica.StateGraph.Step Step13 annotation(Placement(transformation(extent = {{ -2, -40},{18, -20}}, rotation = 0)));
            Modelica.StateGraph.Step Step14 annotation(Placement(transformation(extent = {{58, -40},{78, -20}}, rotation = 0)));
            Modelica.StateGraph.Transition Transition8(condition = T7_idle) annotation(Placement(transformation(extent = {{ -92, -80},{ -72, -60}}, rotation = 0)));
            Modelica.StateGraph.Transition Transition9(condition = LIS_501 <= 0.01) annotation(Placement(transformation(extent = {{ -32, -80},{ -12, -60}}, rotation = 0)));
            Modelica.StateGraph.Transition Transition10(condition = TIS_702 <= 298) annotation(Placement(transformation(extent = {{28, -80},{48, -60}}, rotation = 0)));
            Modelica.StateGraph.Transition Transition11(condition = LIS_701 <= 0.01) annotation(Placement(transformation(extent = {{88, -80},{108, -60}}, rotation = 0)));
            Modelica.StateGraph.Transition Transition12(condition = TIS_602 <= 298) annotation(Placement(transformation(extent = {{ -32, -40},{ -12, -20}}, rotation = 0)));
            Modelica.StateGraph.Transition Transition13(condition = LIS_601 <= 0.01) annotation(Placement(transformation(extent = {{28, -40},{48, -20}}, rotation = 0)));
            Real LIS_301;
            Real LIS_501;
            Real LIS_601;
            Real LIS_701;
            Real QI_302;
            Real QIS_502;
            Real TIS_602;
            Real TIS_702;
            Boolean T5_idle;
            Boolean T7_idle;
            Modelica.StateGraph.TransitionWithSignal TransitionWithSignal1 annotation(Placement(transformation(origin = { -2, -150}, extent = {{ -10, -10},{10,10}}, rotation = 180)));
            Modelica.Blocks.Sources.BooleanExpression BooleanExpression1(y = time > 2500) annotation(Placement(transformation(extent = {{ -104, -148},{ -18, -116}}, rotation = 0)));

          equation 
            LIS_301 = sensors.LIS_301;
            LIS_501 = sensors.LIS_501;
            LIS_601 = sensors.LIS_601;
            LIS_701 = sensors.LIS_701;
            QI_302 = sensors.QI_302;
            QIS_502 = sensors.QIS_502;
            TIS_602 = sensors.TIS_602;
            TIS_702 = sensors.TIS_702;
            T5_idle = not actuators.V12 and not actuators.V15 and not actuators.T5_Heater and sensors.LIS_501 < 0.01;
            T7_idle = not actuators.V15 and not actuators.V18 and not actuators.T7_Cooling and sensors.LIS_701 < 0.01;
            actuators.P1 = Step10.active;
            actuators.P2 = Step13.active;
            actuators.T5_Heater = Step6.active;
            actuators.T7_Cooling = Step9.active;
            actuators.T6_Cooling = Step12.active;
            actuators.V1 = Step10.active;
            actuators.V2 = false;
            actuators.V3 = Step10.active;
            actuators.V4 = false;
            actuators.V5 = Step13.active;
            actuators.V6 = Step13.active;
            actuators.V8 = Step1.active;
            actuators.V9 = Step2.active;
            actuators.V10 = false;
            actuators.V11 = Step3.active;
            actuators.V12 = Step5.active;
            actuators.V15 = Step8.active;
            actuators.V18 = Step10.active;
            actuators.V19 = false;
            actuators.V20 = Step13.active;
            actuators.V21 = false;
            actuators.V22 = Step10.active;
            actuators.V23 = Step10.active;
            actuators.V25 = Step13.active;
            actuators.V24 = Step13.active;
            connect(InitialStep1.outPort[1],Transition1.inPort) annotation(Line(points = {{ -159.5,100},{ -144,100}}, color = {0,0,0}));
            connect(Transition1.outPort,Step1.inPort[1]) annotation(Line(points = {{ -138.5,100},{ -121,100}}, color = {0,0,0}));
            connect(Step1.outPort[1],Transition2.inPort) annotation(Line(points = {{ -99.5,100},{ -84,100}}, color = {0,0,0}));
            connect(Transition2.outPort,Step2.inPort[1]) annotation(Line(points = {{ -78.5,100},{ -61,100}}, color = {0,0,0}));
            connect(Step2.outPort[1],Transition3.inPort) annotation(Line(points = {{ -39.5,100},{ -24,100}}, color = {0,0,0}));
            connect(Transition3.outPort,Step3.inPort[1]) annotation(Line(points = {{ -18.5,100},{ -1,100}}, color = {0,0,0}));
            connect(Step3.outPort[1],Transition4.inPort) annotation(Line(points = {{20.5,100},{36,100}}, color = {0,0,0}));
            connect(Transition4.outPort,Step4.inPort[1]) annotation(Line(points = {{41.5,100},{59,100}}, color = {0,0,0}));
            connect(Step4.outPort[1],Transition5.inPort) annotation(Line(points = {{80.5,100},{96,100}}, color = {0,0,0}));
            connect(Transition5.outPort,Step5.inPort[1]) annotation(Line(points = {{101.5,100},{119,100}}, color = {0,0,0}));
            connect(Step5.outPort[1],Transition6.inPort) annotation(Line(points = {{140.5,100},{156,100}}, color = {0,0,0}));
            connect(Transition6.outPort,Step6.inPort[1]) annotation(Line(points = {{161.5,100},{184,100},{184,70},{ -160,70},{ -160,40},{ -121,40}}, color = {0,0,0}));
            connect(Step6.outPort[1],Transition7.inPort) annotation(Line(points = {{ -99.5,40},{ -84,40}}, color = {0,0,0}));
            connect(Step12.inPort[1],Parallel1.split[1]) annotation(Line(points = {{ -63, -30},{ -134.375, -30},{ -134.375, -25}}, color = {0,0,0}));
            connect(Step12.outPort[1],Transition12.inPort) annotation(Line(points = {{ -41.5, -30},{ -26, -30}}, color = {0,0,0}));
            connect(Transition12.outPort,Step13.inPort[1]) annotation(Line(points = {{ -20.5, -30},{ -3, -30}}, color = {0,0,0}));
            connect(Step13.outPort[1],Transition13.inPort) annotation(Line(points = {{18.5, -30},{34, -30}}, color = {0,0,0}));
            connect(Transition13.outPort,Step14.inPort[1]) annotation(Line(points = {{39.5, -30},{57, -30}}, color = {0,0,0}));
            connect(Step14.outPort[1],Parallel1.join[1]) annotation(Line(points = {{78.5, -30},{152.375, -30},{152.375, -25}}, color = {0,0,0}));
            connect(Step7.inPort[1],Parallel1.split[2]) annotation(Line(points = {{ -123, -70},{ -138, -70},{ -138, -75},{ -134.375, -75}}, color = {0,0,0}));
            connect(Step7.outPort[1],Transition8.inPort) annotation(Line(points = {{ -101.5, -70},{ -86, -70}}, color = {0,0,0}));
            connect(Transition8.outPort,Step8.inPort[1]) annotation(Line(points = {{ -80.5, -70},{ -63, -70}}, color = {0,0,0}));
            connect(Step8.outPort[1],Transition9.inPort) annotation(Line(points = {{ -41.5, -70},{ -26, -70}}, color = {0,0,0}));
            connect(Transition9.outPort,Step9.inPort[1]) annotation(Line(points = {{ -20.5, -70},{ -3, -70}}, color = {0,0,0}));
            connect(Step9.outPort[1],Transition10.inPort) annotation(Line(points = {{18.5, -70},{34, -70}}, color = {0,0,0}));
            connect(Transition10.outPort,Step10.inPort[1]) annotation(Line(points = {{39.5, -70},{57, -70}}, color = {0,0,0}));
            connect(Step10.outPort[1],Transition11.inPort) annotation(Line(points = {{78.5, -70},{94, -70}}, color = {0,0,0}));
            connect(Transition11.outPort,Step11.inPort[1]) annotation(Line(points = {{99.5, -70},{117, -70}}, color = {0,0,0}));
            connect(Step11.outPort[1],Parallel1.join[2]) annotation(Line(points = {{138.5, -70},{154, -70},{154, -75},{152.375, -75}}, color = {0,0,0}));
            connect(Transition7.outPort,Parallel1.inPort) annotation(Line(points = {{ -78.5,40},{ -40,40},{ -40,20},{ -190,20},{ -190, -50},{ -181.55, -50}}, color = {0,0,0}));
            connect(TransitionWithSignal1.inPort,Parallel1.outPort) annotation(Line(points = {{2, -150},{208, -150},{208, -50},{197.7, -50}}, color = {0,0,0}));
            connect(TransitionWithSignal1.outPort,InitialStep1.inPort[1]) annotation(Line(points = {{ -3.5, -150},{ -194, -150},{ -194,100},{ -181,100}}, color = {0,0,0}));
            connect(BooleanExpression1.y,TransitionWithSignal1.condition) annotation(Line(points = {{ -13.7, -132},{ -2, -132},{ -2, -138}}, color = {255,0,255}));
          end Controller;
          package ControllerUtilities
            extends Modelica.Icons.Library;
            class Adapter_Inference
              Port_IdleTanks idleTanks;
            end Adapter_Inference;
            class Adapter_Superposition
              Port_Actuators actuators;
            end Adapter_Superposition;
            class Block_Recipe_TBD
              parameter Real startTime;
              parameter Real w_dilution = 0.003;
              parameter Real w_concentrat = 0.005;
              parameter Real T3_batch_level = 0.1273;
              parameter Real T5_batch_level = 0.211;
              Boolean trig;
              Boolean S0(start = true);
              Boolean S1;
              Boolean S2;
              Boolean S3;
              Boolean S4;
              Boolean S5;
              Boolean S6;
              Boolean S7;
              Boolean S8;
              Boolean S9;
              Boolean S10;
              Boolean S11;
              Boolean S12;
              Boolean S13;
              Boolean S14;
              Boolean tr0;
              Boolean tr1;
              Boolean tr2;
              Boolean tr3;
              Boolean tr4;
              Boolean tr5;
              Boolean tr6;
              Boolean tr7;
              Boolean tr8;
              Boolean tr9;
              Boolean tr10;
              Boolean tr11;
              Boolean tr12;
              Boolean tr13;
              Port_Actuators act annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
            end Block_Recipe_TBD;
            class BlockMain
              Boolean trig;
              Port_Actuators actuators annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
              Block_Recipe_TBD Recipe1 annotation(Placement(transformation(extent = {{ -50,10},{ -10,50}}, rotation = 0)));
              Block_Recipe_TBD Recipe2 annotation(Placement(transformation(extent = {{10,10},{50,50}}, rotation = 0)));
              Adapter_Inference Inference annotation(Placement(transformation(extent = {{ -50, -50},{ -10, -10}}, rotation = 0)));
              Adapter_Superposition Superposition annotation(Placement(transformation(extent = {{10, -50},{50, -10}}, rotation = 0)));
            end BlockMain;
            class Buffer_Recipe_TBD
              Port_Actuators act;
              Boolean S0;
              Boolean S1;
              Boolean S2;
              Boolean S3;
              Boolean S4;
              Boolean S5;
              Boolean S6;
              Boolean S7;
              Boolean S8;
              Boolean S9;
              Boolean S10;
              Boolean S11;
              Boolean S12;
              Boolean S13;
              Boolean S14;
            end Buffer_Recipe_TBD;
            class BufferMain
              Buffer_Recipe_TBD Recipe1;
              Buffer_Recipe_TBD Recipe2;
            end BufferMain;
            connector Port_Actuators
              output Boolean P1;
              output Boolean P2;
              output Boolean T5_Heater;
              output Boolean T7_Cooling;
              output Boolean T6_Cooling;
              output Boolean V1;
              output Boolean V2;
              output Boolean V3;
              output Boolean V4;
              output Boolean V5;
              output Boolean V6;
              output Boolean V8;
              output Boolean V9;
              output Boolean V10;
              output Boolean V11;
              output Boolean V12;
              output Boolean V15;
              output Boolean V18;
              output Boolean V19;
              output Boolean V20;
              output Boolean V21;
              output Boolean V22;
              output Boolean V23;
              output Boolean V24;
              output Boolean V25;
              annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -100,100},{100,0},{ -100, -100},{ -100,100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{0,50},{100,0},{0, -50},{0,50}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}));
            end Port_Actuators;
            connector Port_IdleTanks
              Boolean T5_idle;
              Boolean T7_idle;
            end Port_IdleTanks;
            connector Port_Sensors
              input Real LIS_301;
              input Real QI_302;
              input Real LIS_501;
              input Real QIS_502;
              input Real TI_503;
              input Real LIS_601;
              input Real TIS_602;
              input Real LIS_701;
              input Real TIS_702;
              annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -100,100},{ -100, -100},{100,0},{ -100,100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {215,215,215}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{0,50},{0, -50},{100,0},{0,50}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {215,215,215}, fillPattern = FillPattern.Solid)}));
            end Port_Sensors;
          end ControllerUtilities;
          type Init = enumeration(GuessValues "GuessValues -- Guess values (not fixed) for p, T or h, X, C", InitialValues "InitialValues -- Initial values for p, T or h, X, C", SteadyStateMomentum "SteadyStateMomentum: Steady state momentum", SteadyStateHydraulic  "SteadyStateHydraulic -- Hydraulic steady state (der(p)=0), guess value for p, initial values for T or h, X, C", SteadyState  "SteadyState -- Steady state (guess values for p, T or h, X, C)") "Enumeration to define initialization options" annotation(Documentation(info = "<html>
<p>
Integer type that can have the following values
(to be selected via choices menu):
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Types.Init.</b></th><th><b>Meaning</b></th></tr>
<tr><td>GuessValues</td>
    <td>GuessValues -- Guess values (not fixed) for p, T or h, X, C</td></tr>

<tr><td>InitialValues</td>
    <td>Initial values for p, T or h, X, C</td></tr>

<tr><td>SteadyStateMomentum</td>
    <td>Steady state momentum</td></tr>

<tr><td>SteadyStateHydraulic</td>
    <td>Hydraulic steady state (der(p)=0), guess value for p,
        initial values for T or h, X, C</td></tr>

<tr><td>SteadyState</td>
    <td>Steady state (guess values for p, T or h, X, C)</td></tr>
</table>
</html>"));
          model TankWithTopPorts "Tank with inlet/outlet ports and with inlet ports at the top"
            import Modelica.Constants;
            import Modelica.Fluid.Fittings.BaseClasses.lossConstant_D_zeta;
            import Modelica.Fluid.Utilities.regRoot2;
            import Modelica.Fluid.Vessels.BaseClasses.VesselPortsData;
            SI.Height level(stateSelect = StateSelect.prefer, start = level_start) "Fluid level in the tank";
            parameter SI.Height height "Maximum level of tank before it overflows";
            parameter SI.Area crossArea "Area of tank";
            parameter SI.Volume V0 = 0 "Volume of the liquid when level = 0";
            parameter Medium.AbsolutePressure p_ambient = system.p_ambient "Tank surface pressure" annotation(Dialog(tab = "Assumptions", group = "Ambient"));
            parameter Medium.Temperature T_ambient = system.T_ambient "Tank surface Temperature" annotation(Dialog(tab = "Assumptions", group = "Ambient"));
            parameter SI.Height level_start(min = 0) = 0.5 * height "Start value of tank level" annotation(Dialog(tab = "Initialization"));
            extends Modelica.Fluid.Interfaces.PartialLumpedVolume(final fluidVolume = V, final initialize_p = false, final p_start = p_ambient);
            parameter Integer nTopPorts = 0 "Number of inlet ports above height (>= 1)" annotation(Dialog(__Dymola_connectorSizing = true));
            Vessels.BaseClasses.VesselFluidPorts_a topPorts[nTopPorts](redeclare package Medium = Medium, m_flow(each start = 0, each min = 0)) "Inlet ports over height at top of tank (fluid flows only from the port in to the tank)" annotation(Placement(transformation(extent = {{ -20,0},{20,10}}, origin = {0,100})));
            parameter Integer nPorts = 0 "Number of inlet/outlet ports (on bottom and on the side)" annotation(Dialog(__Dymola_connectorSizing = true));
            parameter Modelica.Fluid.Vessels.BaseClasses.VesselPortsData portsData[nPorts] "Data of inlet/outlet ports at side and bottom of tank";
            Vessels.BaseClasses.VesselFluidPorts_b ports[nPorts](redeclare package Medium = Medium, m_flow(each start = 0)) "inlet/outlet ports at bottom or side of tank (fluid flows in to or out of port; a port might be above the fluid level)" annotation(Placement(transformation(extent = {{ -20,0},{20, -10}}, origin = {0, -100})));
            parameter Boolean use_HeatTransfer = false "= true to use the HeatTransfer model" annotation(Dialog(tab = "Assumptions", group = "Heat transfer"));
            replaceable model HeatTransfer = Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.IdealHeatTransfer extends Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.PartialVesselHeatTransfer "Wall heat transfer" annotation(Dialog(tab = "Assumptions", group = "Heat transfer", enable = use_HeatTransfer), choicesAllMatching = true);
            HeatTransfer heatTransfer(redeclare final package Medium = Medium, final n = 1, final states = {medium.state}, surfaceAreas = {crossArea + 2 * sqrt(crossArea * Modelica.Constants.pi) * level}, final use_k = use_HeatTransfer) annotation(Placement(transformation(extent = {{ -10, -10},{30,30}}, rotation = 90, origin = { -50, -10})));
            Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort if use_HeatTransfer annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}})));
            parameter Real hysteresisFactor(min = 0) = 0.1 "Hysteresis for empty pipe = diameter*hysteresisFactor" annotation(Dialog(tab = "Advanced", group = "Port properties"));
            parameter Boolean stiffCharacteristicForEmptyPort = false "=true, if steep pressure loss characteristic for empty pipe port" annotation(Dialog(tab = "Advanced", group = "Port properties"), Evaluate = true);
            parameter Real zetaLarge(min = 0) = 100000.0 "Large pressure loss factor if mass flows out of empty pipe port" annotation(Dialog(tab = "Advanced", group = "Port properties", enable = stiffCharacteristicForEmptyPort));
            parameter SI.MassFlowRate m_flow_small(min = 0) = system.m_flow_small "Regularization range at zero mass flow rate" annotation(Dialog(tab = "Advanced", group = "Port properties", enable = stiffCharacteristicForEmptyPort));
            SI.Volume V(stateSelect = StateSelect.never) "Actual tank volume";
            Medium.EnthalpyFlowRate H_flow_top[nTopPorts] "Enthalpy flow rates from the top ports in to the tank";
            Medium.EnthalpyFlowRate port_b_H_flow_bottom[nPorts] "Enthalpy flow rates from the bottom ports in to the tank";
            Medium.MassFlowRate mXi_flow_top[nTopPorts,Medium.nXi] "Substance mass flow rates from the top ports into the tank";
            Medium.MassFlowRate port_b_mXi_flow_bottom[nPorts,Medium.nXi] "Substance mass flow rates from the bottom ports into the tank";
            Medium.MassFlowRate mC_flow_top[nTopPorts,Medium.nC] "Trace substance mass flow rates from the top ports into the tank";
            Medium.MassFlowRate port_b_mC_flow_bottom[nPorts,Medium.nC] "Trace substance mass flow rates from the bottom ports into the tank";
          protected 
            SI.Area bottomArea[nPorts];
            SI.Diameter ports_emptyPipeHysteresis[nPorts];
            SI.Length levelAbovePort[nPorts] "Height of fluid over bottom ports";
            Boolean ports_m_flow_out[nPorts](each start = true, each fixed = true);
            Boolean aboveLevel[nPorts] "= true, if level >= ports[i].height";
            Real zetas_out[nPorts];
            Modelica.Blocks.Interfaces.RealInput portsData_diameter[nPorts] = portsData.diameter if nPorts > 0;
            Modelica.Blocks.Interfaces.RealInput portsData_diameter2[nPorts];
            Modelica.Blocks.Interfaces.RealInput portsData_height[nPorts] = portsData.height if nPorts > 0;
            Modelica.Blocks.Interfaces.RealInput portsData_height2[nPorts];

          equation 
            assert(level <= height, "Tank starts to overflow (level = height = " + String(level) + ")");
            assert(m >= 0, "Mass in tank is zero");
            connect(portsData_diameter,portsData_diameter2);
            connect(portsData_height,portsData_height2);
            for i in 1:nPorts loop
            bottomArea[i] = Constants.pi * (portsData_diameter2[i] / 2) ^ 2;
            ports_emptyPipeHysteresis[i] = portsData_diameter2[i] * hysteresisFactor;

            end for;
            medium.p = p_ambient;
            V = crossArea * level + V0 "Volume of fluid";
            mb_flow = sum(topPorts.m_flow) + sum(ports.m_flow);
            for i in 1:Medium.nXi loop
            mbXi_flow[i] = sum(mXi_flow_top[:,i]) + sum(port_b_mXi_flow_bottom[:,i]);

            end for;
            for i in 1:Medium.nC loop
            mbC_flow[i] = sum(mC_flow_top[:,i]) + sum(port_b_mC_flow_bottom[:,i]);

            end for;
            Hb_flow = sum(H_flow_top) + sum(port_b_H_flow_bottom);
            Qb_flow = heatTransfer.Q_flows[1];
            if Medium.singleState or energyDynamics == Types.Dynamics.SteadyState then
              Wb_flow = 0 "Mechanical work is neglected, since also neglected in medium model (otherwise unphysical small temperature change, if tank level changes)";

            else
              Wb_flow =  -p_ambient * der(V);

            end if;
            for i in 1:nTopPorts loop
            H_flow_top[i] = topPorts[i].m_flow * actualStream(topPorts[i].h_outflow);
            mXi_flow_top[i,:] = topPorts[i].m_flow * actualStream(topPorts[i].Xi_outflow);
            mC_flow_top[i,:] = topPorts[i].m_flow * actualStream(topPorts[i].C_outflow);
            topPorts[i].p = p_ambient;
            topPorts[i].h_outflow = h_start;
            topPorts[i].Xi_outflow = X_start[1:Medium.nXi];
            topPorts[i].C_outflow = C_start;

            end for;
            for i in 1:nPorts loop
            port_b_H_flow_bottom[i] = ports[i].m_flow * actualStream(ports[i].h_outflow);
            port_b_mXi_flow_bottom[i,:] = ports[i].m_flow * actualStream(ports[i].Xi_outflow);
            port_b_mC_flow_bottom[i,:] = ports[i].m_flow * actualStream(ports[i].C_outflow);
            aboveLevel[i] = level >= portsData_height2[i] + ports_emptyPipeHysteresis[i] or pre(aboveLevel[i]) and level >= portsData_height2[i] - ports_emptyPipeHysteresis[i];
            levelAbovePort[i] = if aboveLevel[i] then level - portsData_height2[i] else 0;
            ports[i].h_outflow = medium.h;
            ports[i].Xi_outflow = medium.Xi;
            ports[i].C_outflow = C;
            if stiffCharacteristicForEmptyPort then
              zetas_out[i] = 1 + (if aboveLevel[i] then 0 else zetaLarge);
              ports[i].p = p_ambient + levelAbovePort[i] * system.g * medium.d + Modelica.Fluid.Utilities.regSquare2(ports[i].m_flow, m_flow_small, lossConstant_D_zeta(portsData_diameter2[i], 0.01) / medium.d, lossConstant_D_zeta(portsData_diameter2[i], zetas_out[i]) / medium.d);
              ports_m_flow_out[i] = false;

            else
              ports_m_flow_out[i] = pre(ports_m_flow_out[i]) and not ports[i].p > p_ambient or ports[i].m_flow <  -1e-06;
              if aboveLevel[i] then
                ports[i].p = p_ambient + levelAbovePort[i] * system.g * medium.d - smooth(2, noEvent(if ports[i].m_flow < 0 then ports[i].m_flow ^ 2 / (2 * medium.d * bottomArea[i] ^ 2) else 0));

              else
                if pre(ports_m_flow_out[i]) then
                  ports[i].m_flow = 0;

                else
                  ports[i].p = p_ambient;

                end if;

              end if;
              zetas_out[i] = 0;

            end if;

            end for;

          initial equation 
            for i in 1:nPorts loop
            pre(aboveLevel[i]) = level_start >= portsData_height2[i];

            end for;
            if massDynamics == Types.Dynamics.FixedInitial then
              level = level_start;

            elseif massDynamics == Types.Dynamics.SteadyStateInitial then
              der(level) = 0;


            else

            end if;
            annotation(defaultComponentName = "tank", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}, initialScale = 0.2), graphics = {Rectangle(extent = {{ -100, -100},{100,100}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.VerticalCylinder),Rectangle(extent = DynamicSelect({{ -100, -100},{100,0}}, {{ -100, -100},{100, -100 + (200 * level) / height}}), lineColor = {0,0,0}, fillColor = {85,170,255}, fillPattern = FillPattern.VerticalCylinder),Text(extent = {{ -94,90},{95,60}}, lineColor = {0,0,255}, textString = "%name"),Text(extent = {{ -95,41},{95,21}}, lineColor = {0,0,0}, textString = "level ="),Line(points = {{ -100,100},{ -100, -100},{100, -100},{100,100}}, color = {0,0,0}),Text(extent = {{ -95, -39},{95, -59}}, lineColor = {0,0,0}, textString = DynamicSelect("%level_start", realString(level, 1, 2)))}), Documentation(info = "<HTML>
<p>
Model of a tank that is open to the environment at the fixed pressure
<tt>p_ambient</tt>.
The tank is filled with a single or multiple-substance liquid,
assumed to have uniform temperature and mass fractions.
</p>

<p>
At the top of the tank over the maximal fill level <b>height</b>
a vector of FluidPorts, called <b>topPorts</b>, is present.
The assumption is made that fluid flows always in to the tank via these
ports (and never back in to the connector).
</p>

<p>
The vector of connectors <b>ports</b> are fluid ports at the bottom
and side of the tank at a defineable height. Fluid can flow either out
of or in to this port. The fluid level of the tank may be below
one of these ports. This case is approximated by introducing a
large pressure flow coefficient so that the mass flow rate
through this port is very small in this case.
</p>

<p>
If the tank starts to over flow (i.e., level > height), an
assertion is triggered.
</p>

<p>
When the diagram layer is open in the plot environment, the
level of the tank is dynamically visualized. Note, the speed
of the diagram animation in Dymola can be set via command
<b>animationSpeed</b>(), e.g., animationSpeed(speed = 10)
</p>
</HTML>", revisions = "<html>
<ul>
<li><i>Dec. 12, 2008</i> by Ruediger Franke: replace energy and mass balances with
   common definition in BaseClasses.PartialLumpedVolume</li>
<li><i>Dec. 8, 2008</i> by Michael Wetter (LBNL):<br>
Implemented trace substances and missing equation for outflow of multi substance media at top port.</li>
<li><i>Jul. 29, 2006</i> by Martin Otter (DLR):<br>
   Improved handling of ports that are above the fluid level and
   simpler implementation.</li>

<li><i>Jan. 6, 2006</i> by Katja Poschlad, Manuel Remelhe (AST Uni Dortmund),
   Martin Otter (DLR):<br>
   Implementation based on former tank model but with several improvements
   (top, bottom, side ports; correctly treating kinetic energy for outlet
   and total dissipation for inlet; ports can be above the fluid level).</li>
</ul>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}, initialScale = 0.2), graphics));

          equation 
            connect(heatPort,heatTransfer.heatPorts[1]) annotation(Line(points = {{ -100,0},{ -87,0},{ -87,8.88178e-16},{ -74,8.88178e-16}}, color = {191,0,0}, smooth = Smooth.None));
          end TankWithTopPorts;
        end BaseClasses;
        package Test "Test of used tank models"
          model OneTank "Tank with one time-varying top inlet mass flow rate and a bottom outlet into the ambient"
            import Modelica.SIunits.Conversions.from_bar;
            extends Modelica.Icons.Example;
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts tank(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, crossArea = 1, height = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 0)}, V0 = 0.1, nTopPorts = 1, nPorts = 1, level_start = 0) annotation(Placement(transformation(extent = {{0,0},{40,40}}, rotation = 0)));
            Sources.MassFlowSource_T flowSource(nPorts = 1, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, m_flow = 20, T = system.T_ambient, use_m_flow_in = true) annotation(Placement(transformation(extent = {{ -12,42},{8,62}}, rotation = 0)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 100), __Dymola_Commands(file = "Scripts/Fluid/OneTank/plot level and port.p.mos" "plot level and port.p", file = "Scripts/Fluid/OneTank/plot level, port.p and port.m_flow.mos" "plot level, port.p and port.m_flow"));
            inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{70,72},{90,92}}, rotation = 0)));
            Modelica.Fluid.Sources.Boundary_pT ambient_fixed(nPorts = 1, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, p = system.p_ambient, T = system.T_ambient) annotation(Placement(transformation(extent = {{ -14, -50},{6, -30}}, rotation = 0)));
            Modelica.Fluid.Pipes.StaticPipe pipe(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, length = 1, diameter = 0.1, height_ab =  -1) annotation(Placement(transformation(origin = {20, -18}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
            Modelica.Blocks.Sources.TimeTable timeTable(table = [0,0;10,0;10,40;20,40;20,10;50,10;50,0;60,0;60,20;70,20;80,55;80,0;100,0]) annotation(Placement(transformation(extent = {{ -60,60},{ -40,80}})));

          equation 
            connect(flowSource.ports[1],tank.topPorts[1]) annotation(Line(points = {{8,52},{20,52},{20,41}}, color = {0,127,255}));
            connect(tank.ports[1],pipe.port_a) annotation(Line(points = {{20, -1},{20, -8}}, color = {0,127,255}, smooth = Smooth.None));
            connect(pipe.port_b,ambient_fixed.ports[1]) annotation(Line(points = {{20, -28},{20, -40},{6, -40}}, color = {0,127,255}, smooth = Smooth.None));
            connect(timeTable.y,flowSource.m_flow_in) annotation(Line(points = {{ -39,70},{ -24,70},{ -24,60},{ -12,60}}, color = {0,0,127}, smooth = Smooth.None));
          end OneTank;
          model TwoTanks
            import Modelica.SIunits.Conversions.from_bar;
            extends Modelica.Icons.Example;
            parameter Boolean stiffCharacteristicForEmptyPort = true;
            annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 70), __Dymola_Commands(file = "Scripts/Fluid/TwoTanks/plot level and port.p.mos" "plot level and port.p"));
            inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{40,62},{60,82}}, rotation = 0)));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts tank1(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, stiffCharacteristicForEmptyPort = stiffCharacteristicForEmptyPort, crossArea = 1, height = 4, level_start = 3, T_start = Modelica.SIunits.Conversions.from_degC(50), nPorts = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 0)}) annotation(Placement(transformation(extent = {{ -80,0},{ -40,40}}, rotation = 0)));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts tank2(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, stiffCharacteristicForEmptyPort = stiffCharacteristicForEmptyPort, crossArea = 1, height = 4, level_start = 1, T_start = Modelica.SIunits.Conversions.from_degC(100), nPorts = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 0)}) annotation(Placement(transformation(extent = {{0,0},{40,40}}, rotation = 0)));
            Modelica.Fluid.Pipes.StaticPipe pipe(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, length = 1, diameter = 0.1) annotation(Placement(transformation(extent = {{ -30, -30},{ -10, -10}}, rotation = 0)));

          equation 
            connect(tank1.ports[1],pipe.port_a) annotation(Line(points = {{ -60, -1},{ -60, -20},{ -30, -20}}, color = {0,127,255}));
            connect(pipe.port_b,tank2.ports[1]) annotation(Line(points = {{ -10, -20},{20, -20},{20, -1}}, color = {0,127,255}));
          end TwoTanks;
          model TankWithEmptyingPipe1 "Demonstrates a tank with one constant top inlet mass flow rate and a bottom outlet into the ambient"
            import Modelica.SIunits.Conversions.from_bar;
            extends Modelica.Icons.Example;
            Sources.MassFlowSource_T flowSource(nPorts = 1, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, m_flow = 50, T = system.T_ambient) annotation(Placement(transformation(extent = {{ -20,40},{0,60}}, rotation = 0)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 35), __Dymola_Commands(file = "Scripts/Fluid/TankWithEmptyingPipe1/plot level and port.p.mos" "plot level and port.p", file = "Scripts/Fluid/TankWithEmptyingPipe1/plot level, port.p and port.m_flow.mos" "plot level, port.p and port.m_flow"));
            inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100,60},{ -80,80}}, rotation = 0)));
            Modelica.Fluid.Sources.Boundary_pT ambient_fixed(nPorts = 1, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, p = system.p_ambient, T = system.T_ambient) annotation(Placement(transformation(extent = {{ -60, -100},{ -40, -80}}, rotation = 0)));
            Modelica.Fluid.Valves.ValveDiscrete valveDiscrete(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, dp_nominal(displayUnit = "Pa") = 1, m_flow_nominal = 100) annotation(Placement(transformation(origin = { -20, -50}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Modelica.Blocks.Sources.BooleanConstant open(k = false) annotation(Placement(transformation(extent = {{ -60, -60},{ -40, -40}}, rotation = 0)));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts tank1(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, crossArea = 1, V0 = 0.1, height = 2, level_start = 0.1, nPorts = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05, height = 0),Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 1)}, stiffCharacteristicForEmptyPort = true, nTopPorts = 1) annotation(Placement(transformation(extent = {{ -40, -20},{0,20}}, rotation = 0)));
            Modelica.Fluid.Pipes.StaticPipe pipe(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, length = 1, diameter = 0.1, height_ab = 1) annotation(Placement(transformation(origin = {40,10}, extent = {{ -10, -10},{10,10}}, rotation = 90)));

          equation 
            connect(ambient_fixed.ports[1],valveDiscrete.port_a) annotation(Line(points = {{ -40, -90},{ -20, -90},{ -20, -60}}, color = {0,127,255}));
            connect(open.y,valveDiscrete.open) annotation(Line(points = {{ -39, -50},{ -28, -50}}, color = {255,0,255}));
            connect(flowSource.ports[1],pipe.port_b) annotation(Line(points = {{0,50},{40,50},{40,20}}, color = {0,127,255}));
            connect(valveDiscrete.port_b,tank1.ports[1]) annotation(Line(points = {{ -20, -40},{ -20, -30},{ -20, -21},{ -22, -21}}, color = {0,127,255}));
            connect(pipe.port_a,tank1.ports[2]) annotation(Line(points = {{40,0},{40, -28},{ -18, -28},{ -18, -20},{ -18, -20},{ -18, -21}}, color = {0,127,255}));
          end TankWithEmptyingPipe1;
          model TankWithEmptyingPipe2 "Demonstrates a tank with one constant top inlet mass flow rate and a bottom outlet into the ambient"
            import Modelica.SIunits.Conversions.from_bar;
            extends Modelica.Icons.Example;
            annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 35), __Dymola_Commands(file = "Scripts/Fluid/TankWithEmptyingPipe2/plot level and port.p.mos" "plot level and port.p"));
            inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100,60},{ -80,80}}, rotation = 0)));
            Modelica.Fluid.Sources.Boundary_pT ambient_fixed(nPorts = 1, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, p = system.p_ambient, T = system.T_ambient) annotation(Placement(transformation(extent = {{ -60, -100},{ -40, -80}}, rotation = 0)));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts tank1(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, crossArea = 1, V0 = 0.1, height = 2, nPorts = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05, height = 0),Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 1)}, level_start = 2, stiffCharacteristicForEmptyPort = true) annotation(Placement(transformation(extent = {{ -40, -20},{0,20}}, rotation = 0)));
            Modelica.Fluid.Pipes.StaticPipe pipe1(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, length = 1, diameter = 0.1, height_ab = 1) annotation(Placement(transformation(origin = { -20, -60}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Modelica.Fluid.Pipes.StaticPipe pipe2(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, length = 1, diameter = 0.1, height_ab = 1) annotation(Placement(transformation(origin = {30, -60}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Modelica.Fluid.Sources.Boundary_pT ambient_fixed1(nPorts = 1, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, p = system.p_ambient, T = system.T_ambient) annotation(Placement(transformation(extent = {{0, -100},{20, -80}}, rotation = 0)));

          equation 
            connect(tank1.ports[1],pipe1.port_b) annotation(Line(points = {{ -22, -21},{ -22, -35},{ -20, -35},{ -20, -50}}, color = {0,127,255}));
            connect(ambient_fixed.ports[1],pipe1.port_a) annotation(Line(points = {{ -40, -90},{ -20, -90},{ -20, -70}}, color = {0,127,255}));
            connect(tank1.ports[2],pipe2.port_b) annotation(Line(points = {{ -18, -21},{ -18, -21},{ -18, -40},{30, -40},{30, -50}}, color = {0,127,255}));
            connect(ambient_fixed1.ports[1],pipe2.port_a) annotation(Line(points = {{20, -90},{30, -90},{30, -70}}, color = {0,127,255}));
          end TankWithEmptyingPipe2;
          model TanksWithEmptyingPipe1 "Demonstrates a tank with one constant top inlet mass flow rate and a bottom outlet into the ambient"
            import Modelica.SIunits.Conversions.from_bar;
            extends Modelica.Icons.Example;
            annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 35), __Dymola_Commands(file = "Scripts/Fluid/TanksWithEmptyingPipe1/plot level, port.p and port.m_flow.mos" "plot level, port.p and port.m_flow"));
            inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100,60},{ -80,80}}, rotation = 0)));
            Modelica.Fluid.Sources.Boundary_pT ambient_fixed1(nPorts = 1, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, p = system.p_ambient, T = system.T_ambient) annotation(Placement(transformation(extent = {{ -100, -80},{ -80, -60}}, rotation = 0)));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts tank1(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, crossArea = 1, V0 = 0.1, height = 2, nPorts = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05, height = 0),Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 1)}, level_start = 2, stiffCharacteristicForEmptyPort = true) annotation(Placement(transformation(extent = {{ -80,0},{ -40,40}}, rotation = 0)));
            Modelica.Fluid.Pipes.StaticPipe pipe1(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, length = 1, diameter = 0.1, height_ab = 1) annotation(Placement(transformation(origin = { -60, -40}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Modelica.Fluid.Pipes.StaticPipe pipe2(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, length = 1, diameter = 0.1, height_ab = 1) annotation(Placement(transformation(origin = {40, -40}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Modelica.Fluid.Sources.Boundary_pT ambient_fixed2(nPorts = 1, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, p = system.p_ambient, T = system.T_ambient) annotation(Placement(transformation(extent = {{0, -80},{20, -60}}, rotation = 0)));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts tank2(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, crossArea = 1, V0 = 0.1, height = 2, nPorts = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05, height = 0),Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 0.5)}, level_start = 0.1, stiffCharacteristicForEmptyPort = true) annotation(Placement(transformation(extent = {{20,0},{60,40}}, rotation = 0)));
            Modelica.Fluid.Pipes.StaticPipe pipe3(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, length = 1, diameter = 0.1, height_ab =  -0.5) annotation(Placement(transformation(extent = {{ -20,10},{0,30}}, rotation = 0)));

          equation 
            connect(tank1.ports[1],pipe1.port_b) annotation(Line(points = {{ -62, -1},{ -62, -15},{ -60, -15},{ -60, -30}}, color = {0,127,255}));
            connect(ambient_fixed1.ports[1],pipe1.port_a) annotation(Line(points = {{ -80, -70},{ -60, -70},{ -60, -50}}, color = {0,127,255}));
            connect(ambient_fixed2.ports[1],pipe2.port_a) annotation(Line(points = {{20, -70},{40, -70},{40, -50}}, color = {0,127,255}));
            connect(tank2.ports[1],pipe2.port_b) annotation(Line(points = {{38, -1},{38, -15},{40, -15},{40, -30}}, color = {0,127,255}));
            connect(pipe3.port_a,tank1.ports[2]) annotation(Line(points = {{ -20,20},{ -30,20},{ -30, -10},{ -58, -10},{ -58,0},{ -58,0},{ -58, -1}}, color = {0,127,255}));
            connect(pipe3.port_b,tank2.ports[2]) annotation(Line(points = {{0,20},{10,20},{10, -8},{38, -8},{38,0},{42,0},{42, -1}}, color = {0,127,255}));
          end TanksWithEmptyingPipe1;
          model TanksWithEmptyingPipe2 "Demonstrates a tank with one constant top inlet mass flow rate and a bottom outlet into the ambient"
            parameter Boolean stiffCharacteristicForEmptyPort = true;
            import Modelica.SIunits.Conversions.from_bar;
            extends Modelica.Icons.Example;
            replaceable package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater extends Modelica.Media.Interfaces.PartialMedium "Medium in the component" annotation(choicesAllMatching = true);
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 300), __Dymola_Commands(file = "Scripts/Fluid/TanksWithEmptyingPipe2/plot level and port.m_flow.mos" "plot level and port.m_flow"));
            inner Modelica.Fluid.System system annotation(Placement(transformation(extent = {{ -100,60},{ -80,80}}, rotation = 0)));
            Modelica.Fluid.Sources.Boundary_pT ambient_fixed(nPorts = 1, redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, p = system.p_ambient, T = system.T_ambient) annotation(Placement(transformation(extent = {{ -16, -102},{ -36, -82}}, rotation = 0)));
            Modelica.Fluid.Valves.ValveDiscrete valveDiscrete(redeclare package Medium = Modelica.Media.Water.ConstantPropertyLiquidWater, dp_nominal(displayUnit = "Pa") = 1, m_flow_nominal = 100) annotation(Placement(transformation(origin = { -60, -78}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Modelica.Blocks.Sources.BooleanConstant open(k = false) annotation(Placement(transformation(extent = {{ -98, -88},{ -78, -68}}, rotation = 0)));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts tank3(redeclare package Medium = Medium, crossArea = 1, V0 = 0.1, height = 20, nPorts = 2, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05, height = 0),Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05, height = 6.5)}, level_start = 6, nTopPorts = 1, stiffCharacteristicForEmptyPort = stiffCharacteristicForEmptyPort) annotation(Placement(transformation(extent = {{ -80, -50},{ -40, -10}}, rotation = 0)));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts tank1(redeclare package Medium = Medium, crossArea = 1, V0 = 0.1, height = 10, nPorts = 1, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 0)}, level_start = 9, stiffCharacteristicForEmptyPort = stiffCharacteristicForEmptyPort) annotation(Placement(transformation(extent = {{50,50},{90,90}}, rotation = 0)));
            Modelica.Fluid.Examples.AST_BatchPlant.BaseClasses.TankWithTopPorts tank2(redeclare package Medium = Medium, crossArea = 1, V0 = 0.1, height = 10, nPorts = 3, portsData = {Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05, height = 0),Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.05, height = 2),Modelica.Fluid.Vessels.BaseClasses.VesselPortsData(diameter = 0.1, height = 3)}, level_start = 1, stiffCharacteristicForEmptyPort = stiffCharacteristicForEmptyPort) annotation(Placement(transformation(extent = {{ -20,10},{20,50}}, rotation = 0)));
            Pipes.StaticPipe pipe1(redeclare package Medium = Medium, height_ab = 2, length = 2, diameter = 0.1) annotation(Placement(transformation(origin = {70,30}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Pipes.StaticPipe pipe2(redeclare package Medium = Medium, height_ab = 2, length = 2, diameter = 0.1) annotation(Placement(transformation(origin = {0, -22}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            Pipes.StaticPipe pipe3(redeclare package Medium = Medium, height_ab = 2, length = 2, diameter = 0.1) annotation(Placement(transformation(origin = { -60,10}, extent = {{ -10, -10},{10,10}}, rotation = 90)));

          equation 
            connect(ambient_fixed.ports[1],valveDiscrete.port_a) annotation(Line(points = {{ -36, -92},{ -60, -92},{ -60, -88}}, color = {0,127,255}));
            connect(open.y,valveDiscrete.open) annotation(Line(points = {{ -77, -78},{ -68, -78}}, color = {255,0,255}));
            connect(valveDiscrete.port_b,tank3.ports[1]) annotation(Line(points = {{ -60, -68},{ -60, -59},{ -60, -51},{ -62, -51}}, color = {0,127,255}));
            connect(pipe1.port_b,tank1.ports[1]) annotation(Line(points = {{70,40},{70,45},{70,49}}, color = {0,127,255}));
            connect(pipe2.port_a,tank3.ports[2]) annotation(Line(points = {{ -6.12323e-16, -32},{ -6.12323e-16, -48},{0, -60},{ -58, -60},{ -58, -51}}, color = {0,127,255}));
            connect(pipe3.port_a,tank3.topPorts[1]) annotation(Line(points = {{ -60,0},{ -60, -5},{ -60, -9}}, color = {0,127,255}));
            connect(pipe3.port_b,tank2.ports[1]) annotation(Line(points = {{ -60,20},{ -60,26},{ -30,26},{ -30,0},{ -2,0},{ -2,9},{ -2.66667,9}}, color = {0,127,255}));
            connect(pipe1.port_a,tank2.ports[3]) annotation(Line(points = {{70,20},{70,0},{2,0},{2,9},{2.66667,9}}, color = {0,127,255}));
            connect(pipe2.port_b,tank2.ports[2]) annotation(Line(points = {{6.12323e-16, -12},{0, -12},{0,9},{2.22045e-16,9}}, color = {0,127,255}, smooth = Smooth.None));
          end TanksWithEmptyingPipe2;
        end Test;
      end AST_BatchPlant;
    end Examples;
    package Vessels "Devices for storing fluid"
      extends Modelica.Fluid.Icons.VariantLibrary;
      model ClosedVolume "Volume of fixed size, closed to the ambient, with inlet/outlet ports"
        import Modelica.Constants.pi;
        extends Modelica.Fluid.Vessels.BaseClasses.PartialLumpedVessel(final fluidVolume = V, vesselArea = pi * (3 / 4 * V) ^ (2 / 3), heatTransfer(surfaceAreas = {4 * pi * ((3 / 4 * V) / pi) ^ (2 / 3)}));
        parameter SI.Volume V "Volume";

      equation 
        Wb_flow = 0;
        for i in 1:nPorts loop
        vessel_ps_static[i] = medium.p;

        end for;
        annotation(defaultComponentName = "volume", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {170,213,255}),Text(extent = {{ -150,12},{150, -18}}, lineColor = {0,0,0}, textString = "V=%V")}), Documentation(info = "<html>
<p>
Ideally mixed volume of constant size with two fluid ports and one medium model.
The flow properties are computed from the upstream quantities, pressures are equal in both nodes and the medium model if <code>use_portsData=false</code>.
Heat transfer through a thermal port is possible, it equals zero if the port remains unconnected.
A spherical shape is assumed for the heat transfer area, with V=4/3*pi*r^3, A=4*pi*r^2.
Ideal heat transfer is assumed per default; the thermal port temperature is equal to the medium temperature.
</p>
<p>
If <code>use_portsData=true</code>, the port pressures represent the pressures just after the outlet (or just before the inlet) in the attached pipe.
The hydraulic resistances <tt>portsData.zeta_in</tt> and <tt>portsData.zeta_out</tt> determine the dissipative pressure drop between volume and port depending on
the direction of mass flow. See <a href=\"Modelica://Modelica.Fluid.Vessels.BaseClasses.VesselPortsData\">VesselPortsData</a> and <i>[Idelchik, Handbook of Hydraulic Resistance, 2004]</i>.
</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
      end ClosedVolume;
      model OpenTank "Simple tank with inlet/outlet ports"
        import Modelica.Constants.pi;
        SI.Height level(stateSelect = StateSelect.prefer, start = max(level_start, Modelica.Constants.eps)) "Level height of tank";
        SI.Volume V(stateSelect = StateSelect.never) "Actual tank volume";
        parameter SI.Height height "Height of tank";
        parameter SI.Area crossArea "Area of tank";
        parameter Medium.AbsolutePressure p_ambient = system.p_ambient "Tank surface pressure" annotation(Dialog(tab = "Assumptions", group = "Ambient"));
        parameter Medium.Temperature T_ambient = system.T_ambient "Tank surface Temperature" annotation(Dialog(tab = "Assumptions", group = "Ambient"));
        parameter SI.Height level_start(min = 0) = 0.5 * height "Start value of tank level" annotation(Dialog(tab = "Initialization"));
        extends Modelica.Fluid.Vessels.BaseClasses.PartialLumpedVessel(final fluidVolume = V, final fluidLevel = level, final fluidLevel_max = height, final vesselArea = crossArea, heatTransfer(surfaceAreas = {crossArea + 2 * sqrt(crossArea * pi) * level}), final initialize_p = false, final p_start = p_ambient);

      equation 
        V = crossArea * level "Volume of fluid";
        medium.p = p_ambient;
        if Medium.singleState or energyDynamics == Types.Dynamics.SteadyState then
          Wb_flow = 0 "Mechanical work is neglected, since also neglected in medium model (otherwise unphysical small temperature change, if tank level changes)";

        else
          Wb_flow =  -p_ambient * der(V);

        end if;
        for i in 1:nPorts loop
        vessel_ps_static[i] = max(0, level - portsData_height[i]) * system.g * medium.d + p_ambient;

        end for;

      initial equation 
        if massDynamics == Types.Dynamics.FixedInitial then
          level = level_start;

        elseif massDynamics == Types.Dynamics.SteadyStateInitial then
          der(level) = 0;


        else

        end if;
        annotation(defaultComponentName = "tank", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}, initialScale = 0.2), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.VerticalCylinder),Rectangle(extent = DynamicSelect({{ -100, -100},{100,10}}, {{ -100, -100},{100, -100 + (200 * level) / height}}), lineColor = {0,0,0}, fillColor = {85,170,255}, fillPattern = FillPattern.VerticalCylinder),Line(points = {{ -100,100},{ -100, -100},{100, -100},{100,100}}, color = {0,0,0}),Text(extent = {{ -95,60},{95,40}}, lineColor = {0,0,0}, textString = "level ="),Text(extent = {{ -95, -24},{95, -44}}, lineColor = {0,0,0}, textString = DynamicSelect("%level_start", realString(level, 1, 2)))}), Documentation(info = "<HTML>
<p>
Model of a tank that is open to the ambient at the fixed pressure
<tt>p_ambient</tt>.
</p>
<p>
The vector of connectors <b>ports</b> represents fluid ports at configurable heights, relative to the bottom of tank.
Fluid can flow either out of or in to each port.
</p>
The following assumptions are made:
<ul>
<li>The tank is filled with a single or multiple-substance medium having a density higher than the density of the ambient medium.</li>
<li>The fluid has uniform density, temperature and mass fractions</li>
<li>No liquid is leaving the tank through the open top; the simulation breaks with an assertion if the liquid level growths over the height.</li>
</ul>
<p>
The port pressures represent the pressures just after the outlet (or just before the inlet) in the attached pipe.
The hydraulic resistances <tt>portsData.zeta_in</tt> and <tt>portsData.zeta_out</tt> determine the dissipative pressure drop between tank and port depending on
the direction of mass flow. See <a href=\"Modelica://Modelica.Fluid.Vessels.BaseClasses.VesselPortsData\">VesselPortsData</a> and <i>[Idelchik, Handbook of Hydraulic Resistance, 2004]</i>.
</p>
<p>
With the setting <tt>use_portsData=false</tt>, the port pressure represents the static head
at the height of the respective port.
The relationship between pressure drop and mass flow rate at the port must then be provided by connected components;
Heights of ports as well as kinetic and potential energy of fluid enering or leaving are not taken into account anymore.
</p>
</HTML>", revisions = "<html>
<ul>
<li><i>Dec. 12, 2008</i> by Ruediger Franke: move port definitions
   to BaseClasses.PartialLumpedVessel; also use energy and mass balance from common base class</li>
<li><i>Dec. 8, 2008</i> by Michael Wetter (LBNL):<br>
Implemented trace substances.</li>
<li><i>Jan. 6, 2006</i> by Katja Poschlad, Manuel Remelhe (AST Uni Dortmund),
   Martin Otter (DLR):<br>
   Implementation based on former tank model.</li>
<li><i>Oct. 29, 2007</i> by Carsten Heinrich (ILK Dresden):<br>
Adapted to the new fluid library interfaces:
<ul> <li>FluidPorts_b is used instead of FluidPort_b (due to it is defined as an array of ports)</li>
    <li>Port name changed from port to ports</li></ul>Updated documentation.</li>
<li><i>Apr. 25, 2006</i> by Katrin Pr&ouml;l&szlig; (TUHH):<br>
Limitation to bottom ports only, added inlet and outlet loss factors.</li>
</ul>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}, initialScale = 0.2), graphics));
      end OpenTank;
      package BaseClasses "Base classes used in the Vessels package (only of interest to build new component models)"
        extends Modelica.Fluid.Icons.BaseClassLibrary;
        partial model PartialLumpedVessel "Lumped volume with a vector of fluid ports and replaceable heat transfer model"
          extends Modelica.Fluid.Interfaces.PartialLumpedVolume;
          parameter Integer nPorts = 0 "Number of ports" annotation(Evaluate = true, Dialog(__Dymola_connectorSizing = true, tab = "General", group = "Ports"));
          VesselFluidPorts_b ports[nPorts](redeclare each package Medium = Medium) "Fluid inlets and outlets" annotation(Placement(transformation(extent = {{ -40, -10},{40,10}}, origin = {0, -100})));
          parameter Boolean use_portsData = true "= false to neglect pressure loss and kinetic energy" annotation(Evaluate = true, Dialog(tab = "General", group = "Ports"));
          parameter Modelica.Fluid.Vessels.BaseClasses.VesselPortsData[nPorts] portsData if use_portsData "Data of inlet/outlet ports" annotation(Dialog(tab = "General", group = "Ports", enable = use_portsData));
          parameter SI.MassFlowRate m_flow_small(min = 0) = system.m_flow_small "Regularization range at zero mass flow rate" annotation(Dialog(tab = "Advanced", group = "Port properties", enable = stiffCharacteristicForEmptyPort));
          Medium.EnthalpyFlowRate ports_H_flow[nPorts];
          Medium.MassFlowRate ports_mXi_flow[nPorts,Medium.nXi];
          Medium.MassFlowRate[Medium.nXi] sum_ports_mXi_flow "Substance mass flows through ports";
          Medium.ExtraPropertyFlowRate ports_mC_flow[nPorts,Medium.nC];
          Medium.ExtraPropertyFlowRate[Medium.nC] sum_ports_mC_flow "Trace substance mass flows through ports";
          parameter Boolean use_HeatTransfer = false "= true to use the HeatTransfer model" annotation(Dialog(tab = "Assumptions", group = "Heat transfer"));
          replaceable model HeatTransfer = Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.IdealHeatTransfer extends Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.PartialVesselHeatTransfer "Wall heat transfer" annotation(Dialog(tab = "Assumptions", group = "Heat transfer", enable = use_HeatTransfer), choicesAllMatching = true);
          HeatTransfer heatTransfer(redeclare final package Medium = Medium, final n = 1, final states = {medium.state}, final use_k = use_HeatTransfer) annotation(Placement(transformation(extent = {{ -10, -10},{30,30}}, rotation = 90, origin = { -50, -10})));
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort if use_HeatTransfer annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}})));
          Medium.Density[nPorts] portDensities "densites of the fluid at the device boudary";
          SI.Velocity[nPorts] portVelocities "velocities of fluid flow at device boundary";
          SI.EnergyFlowRate[nPorts] ports_E_flow "flow of kinetic and potential energy at device boundary";
          Real[nPorts] s(each start = fluidLevel_max) "curve parameters for port flows vs. port pressures; for further details see, Modelica Tutorial: Ideal switching devices";
          Real[nPorts] ports_penetration "penetration of port with fluid, depending on fluid level and port diameter";
          SI.Area[nPorts] portAreas = array(Modelica.Constants.pi / 4 * portsData_diameter[i] ^ 2 for i in 1:nPorts);
          Medium.AbsolutePressure[nPorts] vessel_ps_static "static pressures inside the vessel at the height of the corresponding ports, zero flow velocity";
        protected 
          input SI.Height fluidLevel = 0 "level of fluid in the vessel for treating heights of ports";
          parameter SI.Height fluidLevel_max = 1 "maximum level of fluid in the vessel";
          parameter SI.Area vesselArea = Modelica.Constants.inf "Area of the vessel used to relate to cross flow area of ports";
          Modelica.Blocks.Interfaces.RealInput[nPorts] portsData_diameter_internal = portsData.diameter if use_portsData and nPorts > 0;
          Modelica.Blocks.Interfaces.RealInput[nPorts] portsData_height_internal = portsData.height if use_portsData and nPorts > 0;
          Modelica.Blocks.Interfaces.RealInput[nPorts] portsData_zeta_in_internal = portsData.zeta_in if use_portsData and nPorts > 0;
          Modelica.Blocks.Interfaces.RealInput[nPorts] portsData_zeta_out_internal = portsData.zeta_out if use_portsData and nPorts > 0;
          Modelica.Blocks.Interfaces.RealInput[nPorts] portsData_diameter;
          Modelica.Blocks.Interfaces.RealInput[nPorts] portsData_height;
          Modelica.Blocks.Interfaces.RealInput[nPorts] portsData_zeta_in;
          Modelica.Blocks.Interfaces.RealInput[nPorts] portsData_zeta_out;

        equation 
          mb_flow = sum(ports.m_flow);
          mbXi_flow = sum_ports_mXi_flow;
          mbC_flow = sum_ports_mC_flow;
          Hb_flow = sum(ports_H_flow) + sum(ports_E_flow);
          Qb_flow = heatTransfer.Q_flows[1];
          for i in 1:nPorts loop
          assert(cardinality(ports[i]) <= 1, "
each ports[i] of volume can at most be connected to one component.
If two or more connections are present, ideal mixing takes
place with these connections, which is usually not the intention
of the modeller. Increase nPorts to add an additional port.
");

          end for;
          assert(fluidLevel <= fluidLevel_max, "Vessel is overflowing (fluidLevel > fluidLevel_max = " + String(fluidLevel) + ")");
          assert(fluidLevel >  -1e-06 * fluidLevel_max, "Fluid level (= " + String(fluidLevel) + ") is below zero meaning that the solution failed.");
          connect(portsData_diameter,portsData_diameter_internal);
          connect(portsData_height,portsData_height_internal);
          connect(portsData_zeta_in,portsData_zeta_in_internal);
          connect(portsData_zeta_out,portsData_zeta_out_internal);
if not use_portsData then
                      portsData_diameter = zeros(nPorts);
            portsData_height = zeros(nPorts);
            portsData_zeta_in = zeros(nPorts);
            portsData_zeta_out = zeros(nPorts);
          end if;
          for i in 1:nPorts loop
          if use_portsData then
            portDensities[i] = noEvent(Medium.density(Medium.setState_phX(vessel_ps_static[i], actualStream(ports[i].h_outflow), actualStream(ports[i].Xi_outflow))));
            portVelocities[i] = smooth(0, ports[i].m_flow / portAreas[i] / portDensities[i]);
            ports_penetration[i] = Utilities.regStep(fluidLevel - portsData_height[i] - 0.1 * portsData_diameter[i], 1, 0.001, 0.1 * portsData_diameter[i]);

          else
            portDensities[i] = medium.d;
            portVelocities[i] = 0;
            ports_penetration[i] = 1;

          end if;
          if fluidLevel >= portsData_height[i] then
            if use_portsData then
              ports[i].p = vessel_ps_static[i] + 0.5 / portAreas[i] ^ 2 * Utilities.regSquare2(ports[i].m_flow, m_flow_small, (portsData_zeta_in[i] - 1 + portAreas[i] ^ 2 / vesselArea ^ 2) / portDensities[i] * ports_penetration[i], (portsData_zeta_out[i] + 1 - portAreas[i] ^ 2 / vesselArea ^ 2) / medium.d / ports_penetration[i]);

            else
              ports[i].p = vessel_ps_static[i];

            end if;
            s[i] = fluidLevel - portsData_height[i];

          elseif s[i] > 0 or portsData_height[i] >= fluidLevel_max then
            ports[i].p = vessel_ps_static[i];
            s[i] = ports[i].m_flow;


          else
            ports[i].m_flow = 0;
            s[i] = (ports[i].p - vessel_ps_static[i]) / Medium.p_default * (portsData_height[i] - fluidLevel);

          end if;
          ports[i].h_outflow = medium.h;
          ports[i].Xi_outflow = medium.Xi;
          ports[i].C_outflow = C;
          ports_H_flow[i] = ports[i].m_flow * actualStream(ports[i].h_outflow) "Enthalpy flow";
          ports_E_flow[i] = ports[i].m_flow * (0.5 * portVelocities[i] * portVelocities[i] + system.g * portsData_height[i]) "Flow of kinetic and potential energy";
          ports_mXi_flow[i,:] = ports[i].m_flow * actualStream(ports[i].Xi_outflow) "Component mass flow";
          ports_mC_flow[i,:] = ports[i].m_flow * actualStream(ports[i].C_outflow) "Trace substance mass flow";

          end for;
          for i in 1:Medium.nXi loop
          sum_ports_mXi_flow[i] = sum(ports_mXi_flow[:,i]);

          end for;
          for i in 1:Medium.nC loop
          sum_ports_mC_flow[i] = sum(ports_mC_flow[:,i]);

          end for;
          connect(heatPort,heatTransfer.heatPorts[1]) annotation(Line(points = {{ -100,0},{ -87,0},{ -87,8.88178e-16},{ -74,8.88178e-16}}, color = {191,0,0}, smooth = Smooth.None));
          annotation(Documentation(info = "<html>
<p>
This base class extends PartialLumpedVolume with a vector of fluid ports and a replaceable wall HeatTransfer model.
<p>
The following modeling assumption are made:
<ul>
<li>homogenous medium, i.e. phase seperation is not taken into account,</li>
<li>no kinetic energy in the fluid, i.e. kinetic energy dissipates into the internal energy,</li>
<li>pressure loss definitions at vessel ports assume incompressible fluid,</li>
<li>outflow of ambient media is prevented at each port assuming check valve behavior.
    If <tt> fluidlevel &lt; portsData_height[i] </tt>and &nbsp; <tt> ports[i].p &lt; vessel_ps_static[i]</tt> massflow at the port is set to 0.</li>
</ul>
</p>
Each port has a (hydraulic) diameter and a height above the bottom of the vessel, which can be configured using the &nbsp;<b><tt>portsData</tt></b> record.
Alternatively the impact of port geometries can be neglected with <tt>use_portsData=false</tt>. This might be useful for early
design studies. Note that this means to assume an infinite port diameter at the bottom of the vessel.
Pressure drops and heights of the ports as well as kinetic and potential energy fluid entering or leaving the vessel are neglected then.
<p>
The following variables need to be defined by an extending model:
<ul>
<li><tt>input fluidVolume</tt>, the volume of the fluid in the vessel,</li>
<li><tt>vessel_ps_static[nPorts]</tt>, the static pressures inside the vessel at the height of the corresponding ports, at zero flow velocity, and</li>
<li><tt>Wb_flow</tt>, work term of the energy balance, e.g. p*der(V) if the volume is not constant or stirrer power.</li>
</ul>
An extending model should define:
<ul>
<li><tt>parameter vesselArea</tt> (default: Modelica.Constants.inf m2), the area of the vessel, to be related to cross flow areas of the ports for the consideration of dynamic pressure effects.</li>
</ul>
Optionally the fluid level may vary in the vessel, which effects the flow through the ports at configurable <tt>portsData_height[nPorts]</tt>.
This is why an extending model with varying fluid level needs to define:
<ul>
<li><tt>input fluidLevel (default: 0m)</tt>, the level the fluid in the vessel, and</li>
<li><tt>parameter fluidLevel_max (default: 1m)</tt>, the maximum level that must not be exceeded. Ports at or above fluidLevel_max can only receive inflow.</li>
</ul>
An extending model should not access the <tt>portsData</tt> record defined in the configuration dialog,
as an access to <tt>portsData</tt> may fail for <tt>use_portsData=false</tt> or <tt>nPorts=0</tt>.
Instead the predefined variables
<ul>
<li><tt>portsData_diameter[nPorts]</tt></li>,
<li><tt>portsData_height[nPorts]</tt></li>,
<li><tt>portsData_zeta_in[nPorts]</tt></li>, and
<li><tt>portsData_zeta_out[nPorts]</tt></li>
</ul>
should be used if these values are needed.
</p>
</html>", revisions = "<html>
<ul>
<li><i>Jan. 2009</i> by R&uuml;diger Franke: extended with
   <ul><li>portsData record and threat configurable port heights,</li>
       <li>consideration of kinetic and potential energy of fluid entering or leaving in energy balance</li>
   </ul>
</li>
<li><i>Dec. 2008</i> by R&uuml;diger Franke: derived from OpenTank, in order to make general use of configurable port diameters</i>
</ul>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -150,110},{150,150}}, textString = "%name", lineColor = {0,0,255})}));
        end PartialLumpedVessel;
        package HeatTransfer "HeatTransfer models for vessels"
          partial model PartialVesselHeatTransfer "Base class for vessel heat transfer models"
            extends Modelica.Fluid.Interfaces.PartialHeatTransfer;
            annotation(Documentation(info = "<html>
Base class for vessel heat transfer models.
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -60,64},{60, -56}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {232,0,0}),Text(extent = {{ -38,26},{40, -14}}, lineColor = {0,0,0}, textString = "%name")}));
          end PartialVesselHeatTransfer;
          model IdealHeatTransfer "IdealHeatTransfer: Ideal heat transfer without thermal resistance"
            extends PartialVesselHeatTransfer;

          equation 
            Ts = heatPorts.T;
            annotation(Documentation(info = "<html>
Ideal heat transfer without thermal resistance.
</html>"));
          end IdealHeatTransfer;
          model ConstantHeatTransfer "ConstantHeatTransfer: Constant heat transfer coefficient"
            extends PartialVesselHeatTransfer;
            parameter SI.CoefficientOfHeatTransfer alpha0 "constant heat transfer coefficient";

          equation 
            Q_flows = array((alpha0 + k) * surfaceAreas[i] * (heatPorts[i].T - Ts[i]) for i in 1:n);
            annotation(Documentation(info = "<html>
Simple heat transfer correlation with constant heat transfer coefficient.
</html>"));
          end ConstantHeatTransfer;
          annotation(Documentation(info = "<html>
Heat transfer correlations for pipe models
</html>"));
        end HeatTransfer;
        record VesselPortsData "Data to describe inlet/outlet ports at vessels:
    diameter -- Inner (hydraulic) diameter of inlet/outlet port
    height -- Height over the bottom of the vessel
    zeta_out -- Hydraulic resistance out of vessel, default 0.5 for small diameter mounted flush with the wall
    zeta_in -- Hydraulic resistance into vessel, default 1.04 for small diameter mounted flush with the wall"
          extends Modelica.Icons.Record;
          parameter SI.Diameter diameter "Inner (hydraulic) diameter of inlet/outlet port";
          parameter SI.Height height = 0 "Height over the bottom of the vessel";
          parameter Real zeta_out(min = 0) = 0.5 "Hydraulic resistance out of vessel, default 0.5 for small diameter mounted flush with the wall";
          parameter Real zeta_in(min = 0) = 1.04 "Hydraulic resistance into vessel, default 1.04 for small diameter mounted flush with the wall";
          annotation(preferredView = "info", Documentation(info = "<html>
<h4><font color=\"#008000\" >Vessel Port Data</font></h4>
<p>
This record describes the <b>ports</b> of a <b>vessel</b>. The variables in it are mostly self-explanatory (see list below); only the &zeta;
loss factors are discussed further. All data is quoted from Idelchik (1994).
</p>

<h4><font color=\"#008000\">Outlet Coefficients</font></h4>

<p>
If a <b>straight pipe with constant cross section is mounted flush with the wall</b>, its outlet pressure loss coefficient will be <code>&zeta; = 0.5</code> (Idelchik, p. 160, Diagram 3-1, paragraph 2).
</p>
<p>
If a <b>straight pipe with constant cross section is mounted into a vessel such that the entrance into it is at a distance</b> <code>b</code> from the wall (inside) the following table can be used. Herein, &delta; is the tube wall thickness (Idelchik, p. 160, Diagram 3-1, paragraph 1).
</p>

<table border=\"1\" cellspacing=\"0\" cellpadding=\"2\">
  <caption align=\"bottom\">Pressure loss coefficients for outlets, entrance at a distance from wall</caption>
  <tr>
    <td></td> <td>   </td><th colspan=\"5\" align=\"center\"> b / D_hyd  </th>
  </tr>
  <tr>
    <td></td> <td>   </td><th> 0.000 </th><th> 0.005 </th><th> 0.020 </th><th> 0.100 </th><th> 0.500-&#8734; </th>
  </tr>
  <tr>
     <th rowspan=\"5\" valign=\"middle\">&delta; / D_hyd</th> <th> 0.000 </th><td> 0.50 </td><td> 0.63  </td><td> 0.73  </td><td> 0.86  </td><td>      1.00     </td>
  </tr>
  <tr>
              <th> 0.008 </th><td> 0.50 </td><td> 0.55  </td><td> 0.62  </td><td> 0.74  </td><td>      0.88     </td>
  </tr>
  <tr>
              <th> 0.016 </th><td> 0.50 </td><td> 0.51  </td><td> 0.55  </td><td> 0.64  </td><td>      0.77     </td>
  </tr>
  <tr>
              <th> 0.024 </th><td> 0.50 </td><td> 0.50  </td><td> 0.52  </td><td> 0.58  </td><td>      0.68     </td>
  </tr>
  <tr>
              <th> 0.040 </th><td> 0.50 </td><td> 0.50  </td><td> 0.51  </td><td> 0.51  </td><td>      0.54     </td>
  </tr>
</table>

<p>
If a <b>straight pipe with a circular bellmouth inlet (collector) without baffle is mounted flush with the wall</b> then its pressure loss coefficient can be established from the following table. Herein, r is the radius of the bellmouth inlet surface (Idelchik, p. 164 f., Diagram 3-4, paragraph b)
</p>

<table border=\"1\" cellspacing=\"0\" cellpadding=\"2\">
  <caption align=\"bottom\">Pressure loss coefficients for outlets, bellmouth flush with wall</caption>
  <tr>
    <td></td> <th colspan=\"6\" align=\"center\"> r / D_hyd  </th>
  </tr>
  <tr>
    <td></td> <th> 0.01 </th><th> 0.03 </th><th> 0.05 </th><th> 0.08 </th><th> 0.16 </th><th>&ge;0.20</th>
  </tr>
  <tr>
     <th>&zeta;</th> <td> 0.44 </td><td> 0.31 </td><td> 0.22  </td><td> 0.15  </td><td> 0.06  </td><td>      0.03     </td>
  </tr>
</table>

<p>
If a <b>straight pipe with a circular bellmouth inlet (collector) without baffle is mounted at a distance from a wall</b> then its pressure loss coefficient can be established from the following table. Herein, r is the radius of the bellmouth inlet surface (Idelchik, p. 164 f., Diagram 3-4, paragraph a)
</p>

<table border=\"1\" cellspacing=\"0\" cellpadding=\"2\">
  <caption align=\"bottom\">Pressure loss coefficients for outlets, bellmouth at a distance of wall</caption>
  <tr>
    <td></td> <th colspan=\"6\" align=\"center\"> r / D_hyd  </th>
  </tr>
  <tr>
    <td></td> <th> 0.01 </th><th> 0.03 </th><th> 0.05 </th><th> 0.08 </th><th> 0.16 </th><th>&ge;0.20</th>
  </tr>
  <tr>
     <th>&zeta;</th> <td> 0.87 </td><td> 0.61 </td><td> 0.40  </td><td> 0.20  </td><td> 0.06  </td><td>      0.03     </td>
  </tr>
</table>



<h4><font color=\"#008000\">Inlet Coefficients</font></h4>

<p>
If a <b>straight pipe with constant circular cross section is mounted flush with the wall</b>, its vessel inlet pressure loss coefficient will be according to the following table (Idelchik, p. 209 f., Diagram 4-2 with <code>A_port/A_vessel = 0</code> and Idelchik, p. 640, Diagram 11-1, graph a). According to the text, <code>m = 9</code> is appropriate for fully developed turbulent flow.
</p>

<table border=\"1\" cellspacing=\"0\" cellpadding=\"2\">
  <caption align=\"bottom\">Pressure loss coefficients for inlets, circular tube flush with wall</caption>
  <tr>
    <td></td> <th colspan=\"6\" align=\"center\"> m  </th>
  </tr>
  <tr>
    <td></td> <th> 1.0 </th><th> 2.0 </th><th> 3.0 </th><th> 4.0 </th><th> 7.0 </th><th>9.0</th>
  </tr>
  <tr>
     <th>&zeta;</th> <td> 2.70 </td><td> 1.50 </td><td> 1.25  </td><td> 1.15  </td><td> 1.06  </td><td>      1.04     </td>
  </tr>
</table>

<p>
For larger port diameters, relative to the area of the vessel, the inlet pressure loss coefficient will be according to the following table (Idelchik, p. 209 f., Diagram 4-2 with <code>m = 7</code>).
</p>

<table border=\"1\" cellspacing=\"0\" cellpadding=\"2\">
  <caption align=\"bottom\">Pressure loss coefficients for inlets, circular tube flush with wall</caption>
  <tr>
    <td></td> <th colspan=\"6\" align=\"center\"> A_port / A_vessel  </th>
  </tr>
  <tr>
    <td></td> <th> 0.0 </th><th> 0.1 </th><th> 0.2 </th><th> 0.4 </th><th> 0.6 </th><th>0.8</th>
  </tr>
  <tr>
     <th>&zeta;</th> <td> 1.04 </td><td> 0.84 </td><td> 0.67  </td><td> 0.39  </td><td> 0.18  </td><td>      0.06     </td>
  </tr>
</table>


<h4><font color=\"#008000\">References</font></h4>

<dl><dt>Idelchik I.E. (1994):</dt>
    <dd><a href=\"http://www.bookfinder.com/dir/i/Handbook_of_Hydraulic_Resistance/0849399084/\"><b>Handbook
        of Hydraulic Resistance</b></a>. 3rd edition, Begell House, ISBN
        0-8493-9908-4</dd>
</dl>
</html>"));
        end VesselPortsData;
        connector VesselFluidPorts_a "Fluid connector with filled, large icon to be used for horizontally aligned vectors of FluidPorts (vector dimensions must be added after dragging)"
          extends Interfaces.FluidPort;
          annotation(defaultComponentName = "ports_b", Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -50, -200},{50,200}}, grid = {1,1}, initialScale = 0.2), graphics = {Text(extent = {{ -75,130},{75,100}}, textString = "%name"),Rectangle(extent = {{ -25,100},{25, -100}}, lineColor = {0,0,0}),Ellipse(extent = {{ -22,100},{ -10, -100}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -6,100},{6, -100}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{10,100},{22, -100}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid)}), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -50, -200},{50,200}}, grid = {1,1}, initialScale = 0.2), graphics = {Rectangle(extent = {{ -50,200},{50, -200}}, lineColor = {0,127,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -44,200},{ -20, -200}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -12,200},{12, -200}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{20,200},{44, -200}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid)}));
        end VesselFluidPorts_a;
        connector VesselFluidPorts_b "Fluid connector with outlined, large icon to be used for horizontally aligned vectors of FluidPorts (vector dimensions must be added after dragging)"
          extends Interfaces.FluidPort;
          annotation(defaultComponentName = "ports_b", Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -50, -200},{50,200}}, grid = {1,1}, initialScale = 0.2), graphics = {Text(extent = {{ -75,130},{75,100}}, textString = "%name"),Rectangle(extent = {{ -25,100},{25, -100}}, lineColor = {0,0,0}),Ellipse(extent = {{ -22,100},{ -10, -100}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -20, -69},{ -12,69}}, lineColor = {0,127,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -6,100},{6, -100}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{10,100},{22, -100}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -4, -69},{4,69}}, lineColor = {0,127,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{12, -69},{20,69}}, lineColor = {0,127,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -50, -200},{50,200}}, grid = {1,1}, initialScale = 0.2), graphics = {Rectangle(extent = {{ -50,200},{50, -200}}, lineColor = {0,127,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -44,200},{ -20, -200}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -12,200},{12, -200}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{20,200},{44, -200}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -39, -118.5},{ -25,113}}, lineColor = {0,127,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -7, -118.5},{7,113}}, lineColor = {0,127,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{25, -117.5},{39,114}}, lineColor = {0,127,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}));
        end VesselFluidPorts_b;
      end BaseClasses;
      annotation(Documentation(info = "<html>

</html>"));
    end Vessels;
    package Valves "Components for the regulation and control of fluid flow"
      extends Modelica.Fluid.Icons.VariantLibrary;
      model ValveIncompressible "Valve for (almost) incompressible fluids"
        extends BaseClasses.PartialValve;
        import Modelica.Fluid.Types.CvTypes;

      initial equation 
if CvData == CvTypes.OpPoint then
                  m_flow_nominal = valveCharacteristic(opening_nominal) * Av * sqrt(rho_nominal) * Utilities.regRoot(dp_nominal, dp_small) "Determination of Av by the operating point";
        end if;

      equation 
        if checkValve then
          m_flow = valveCharacteristic(opening) * Av * sqrt(Medium.density(state_a)) * (if dp >= 0 then Utilities.regRoot(dp, dp_small) else 0);

        elseif not allowFlowReversal then
          m_flow = valveCharacteristic(opening) * Av * sqrt(Medium.density(state_a)) * Utilities.regRoot(dp, dp_small);


        else
          m_flow = valveCharacteristic(opening) * Av * smooth(0, Utilities.regRoot(dp, dp_small) * (if dp >= 0 then sqrt(Medium.density(state_a)) else sqrt(Medium.density(state_b))));

        end if;
        annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Documentation(info = "<HTML>
<p>Valve model according to the IEC 534/ISA S.75 standards for valve sizing, incompressible fluids.
<p>This model assumes that the fluid has a low compressibility, which is always the case for liquids.
It can also be used with gases, provided that the pressure drop is lower than 0.2 times the absolute pressure at the inlet, so that the fluid density does not change much inside the valve.</p>
<p>If <tt>checkValve</tt> is false, the valve supports reverse flow, with a symmetric flow characteric curve. Otherwise, reverse flow is stopped (check valve behaviour).</p>

<p>
The treatment of parameters <b>Kv</b> and <b>Cv</b> is
explained in detail in the
<a href=\"Modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.ValveCharacteristics\">Users Guide</a>.
</p>

</html>", revisions = "<html>
<ul>
<li><i>2 Nov 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Adapted from the ThermoPower library.</li>
</ul>
</html>"));
      end ValveIncompressible;
      model ValveVaporizing "Valve for possibly vaporizing (almost) incompressible fluids, accounts for choked flow conditions"
        import Modelica.Fluid.Types.CvTypes;
        extends BaseClasses.PartialValve(redeclare replaceable package Medium = Modelica.Media.Water.WaterIF97_ph extends Modelica.Media.Interfaces.PartialTwoPhaseMedium);
        parameter Real Fl_nominal = 0.9 "Liquid pressure recovery factor";
        replaceable function FlCharacteristic = Modelica.Fluid.Valves.BaseClasses.ValveCharacteristics.one extends Modelica.Fluid.Valves.BaseClasses.ValveCharacteristics.baseFun "Pressure recovery characteristic";
        Real Ff "Ff coefficient (see IEC/ISA standard)";
        Real Fl "Pressure recovery coefficient Fl (see IEC/ISA standard)";
        SI.Pressure dpEff "Effective pressure drop";
        Medium.Temperature T_in "Inlet temperature";
        Medium.AbsolutePressure p_sat "Saturation pressure";
        Medium.AbsolutePressure p_in "Inlet pressure";
        Medium.AbsolutePressure p_out "Outlet pressure";

      initial equation 
        assert(not CvData == CvTypes.OpPoint, "OpPoint option not supported for vaporizing valve");

      equation 
        p_in = port_a.p;
        p_out = port_b.p;
        T_in = Medium.temperature(state_a);
        p_sat = Medium.saturationPressure(T_in);
        Ff = 0.96 - 0.28 * sqrt(p_sat / Medium.fluidConstants[1].criticalPressure);
        Fl = Fl_nominal * FlCharacteristic(opening);
        dpEff = if p_out < (1 - Fl ^ 2) * p_in + Ff * Fl ^ 2 * p_sat then Fl ^ 2 * (p_in - Ff * p_sat) else dp "Effective pressure drop, accounting for possible choked conditions";
        if checkValve then
          m_flow = valveCharacteristic(opening) * Av * sqrt(Medium.density(state_a)) * (if dpEff >= 0 then Utilities.regRoot(dpEff, dp_small) else 0);

        elseif not allowFlowReversal then
          m_flow = valveCharacteristic(opening) * Av * sqrt(Medium.density(state_a)) * Utilities.regRoot(dpEff, dp_small);


        else
          m_flow = valveCharacteristic(opening) * Av * smooth(0, Utilities.regRoot(dpEff, dp_small) * (if dpEff >= 0 then sqrt(Medium.density(state_a)) else sqrt(Medium.density(state_b))));

        end if;
        annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<HTML>
<p>Valve model according to the IEC 534/ISA S.75 standards for valve sizing, incompressible fluid at the inlet, and possibly two-phase fluid at the outlet, including choked flow conditions. </p>
<p>The model operating range includes choked flow operation, which takes place for low outlet pressures due to flashing in the vena contracta; otherwise, non-choking conditions are assumed.
<p>This model requires a two-phase medium model, to describe the liquid and (possible) two-phase conditions.
<p>The default liquid pressure recovery coefficient <tt>Fl</tt> is constant and given by the parameter <tt>Fl_nominal</tt>. The relative change (per unit) of the recovery coefficient can be specified as a given function of the valve opening by replacing the <tt>FlCharacteristic</tt> function.
<p>If <tt>checkValve</tt> is false, the valve supports reverse flow, with a symmetric flow characteric curve. Otherwise, reverse flow is stopped (check valve behaviour).</p>

<p>
The treatment of parameters <b>Kv</b> and <b>Cv</b> is
explained in detail in the
<a href=\"Modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.ValveCharacteristics\">Users Guide</a>.
</p>

</HTML>", revisions = "<html>
<ul>
<li><i>2 Nov 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Adapted from the ThermoPower library.</li>
</ul>
</html>"));
      end ValveVaporizing;
      model ValveCompressible "Valve for compressible fluids, accounts for choked flow conditions"
        extends BaseClasses.PartialValve;
        import Modelica.Fluid.Types.CvTypes;
        parameter Medium.AbsolutePressure p_nominal "Nominal inlet pressure" annotation(Dialog(group = "Nominal operating point"));
        parameter Real Fxt_full = 0.5 "Fk*xt critical ratio at full opening";
        replaceable function xtCharacteristic = Modelica.Fluid.Valves.BaseClasses.ValveCharacteristics.one extends Modelica.Fluid.Valves.BaseClasses.ValveCharacteristics.baseFun "Critical ratio characteristic";
        Real Fxt;
        Real x "Pressure drop ratio";
        Real xs "Saturated pressure drop ratio";
        Real Y "Compressibility factor";
        Medium.AbsolutePressure p "Inlet pressure";
      protected 
        parameter Real Fxt_nominal(fixed = false) "Nominal Fxt";
        parameter Real x_nominal(fixed = false) "Nominal pressure drop ratio";
        parameter Real xs_nominal(fixed = false) "Nominal saturated pressure drop ratio";
        parameter Real Y_nominal(fixed = false) "Nominal compressibility factor";

      initial equation 
        if CvData == CvTypes.OpPoint then
          Fxt_nominal = Fxt_full * xtCharacteristic(opening_nominal);
          x_nominal = dp_nominal / p_nominal;
          xs_nominal = smooth(0, if x_nominal > Fxt_nominal then Fxt_nominal else x_nominal);
          Y_nominal = 1 - abs(xs_nominal) / (3 * Fxt_nominal);
          m_flow_nominal = valveCharacteristic(opening_nominal) * Av * Y_nominal * sqrt(rho_nominal) * Utilities.regRoot(p_nominal * xs_nominal, dp_small);

        else
          Fxt_nominal = 0;
          x_nominal = 0;
          xs_nominal = 0;
          Y_nominal = 0;

        end if;

      equation 
        p = noEvent(if dp >= 0 then port_a.p else port_b.p);
        Fxt = Fxt_full * xtCharacteristic(opening);
        x = dp / p;
        xs = smooth(0, if x <  -Fxt then  -Fxt else if x > Fxt then Fxt else x);
        Y = 1 - abs(xs) / (3 * Fxt);
        if checkValve then
          m_flow = valveCharacteristic(opening) * Av * Y * sqrt(Medium.density(state_a)) * (if xs >= 0 then Utilities.regRoot(p * xs, dp_small) else 0);

        elseif not allowFlowReversal then
          m_flow = valveCharacteristic(opening) * Av * sqrt(Medium.density(state_a)) * Utilities.regRoot(p * xs, dp_small);


        else
          m_flow = valveCharacteristic(opening) * Av * Y * smooth(0, Utilities.regRoot(p * xs, dp_small) * (if xs >= 0 then sqrt(Medium.density(state_a)) else sqrt(Medium.density(state_b))));

        end if;
        annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<HTML>
<p>Valve model according to the IEC 534/ISA S.75 standards for valve sizing, compressible fluid, no phase change, also covering choked-flow conditions. </p>
<p>This model can be used with gases and vapours, with arbitrary pressure ratio between inlet and outlet.</p>

<p>The product Fk*xt is given by the parameter <tt>Fxt_full</tt>, and is assumed constant by default. The relative change (per unit) of the xt coefficient with the valve opening can be specified by replacing the <tt>xtCharacteristic</tt> function.
<p>If <tt>checkValve</tt> is false, the valve supports reverse flow, with a symmetric flow characteric curve. Otherwise, reverse flow is stopped (check valve behaviour).</p>

<p>
The treatment of parameters <b>Kv</b> and <b>Cv</b> is
explained in detail in the
<a href=\"Modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.ValveCharacteristics\">Users Guide</a>.
</p>

</HTML>", revisions = "<html>
<ul>
<li><i>2 Nov 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Adapted from the ThermoPower library.</li>
</ul>
</html>"));
      end ValveCompressible;
      model ValveLinear "Valve for water/steam flows with linear pressure drop"
        extends Modelica.Fluid.Interfaces.PartialTwoPortTransport;
        parameter SI.AbsolutePressure dp_nominal "Nominal pressure drop at full opening";
        parameter Medium.MassFlowRate m_flow_nominal "Nominal mass flowrate at full opening";
        final parameter Types.HydraulicConductance k = m_flow_nominal / dp_nominal "Hydraulic conductance at full opening";
        Modelica.Blocks.Interfaces.RealInput opening(min = 0, max = 1) "=1: completely open, =0: completely closed" annotation(Placement(transformation(origin = {0,90}, extent = {{ -20, -20},{20,20}}, rotation = 270), iconTransformation(extent = {{ -20, -20},{20,20}}, rotation = 270, origin = {0,80})));

      equation 
        m_flow = opening * k * dp;
        port_a.h_outflow = inStream(port_b.h_outflow);
        port_b.h_outflow = inStream(port_a.h_outflow);
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{0,50},{0,0}}, color = {0,0,0}),Rectangle(extent = {{ -20,60},{20,50}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{ -100,50},{100, -50},{100,50},{0,0},{ -100, -50},{ -100,50}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = DynamicSelect({{ -100,0},{100, -0},{100,0},{0,0},{ -100, -0},{ -100,0}}, {{ -100,50 * opening},{ -100,50 * opening},{100, -50 * opening},{100,50 * opening},{0,0},{ -100, -50 * opening},{ -100,50 * opening}}), fillColor = {0,255,0}, lineColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -100,50},{100, -50},{100,50},{0,0},{ -100, -50},{ -100,50}}, lineColor = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Documentation(info = "<HTML>
<p>This very simple model provides a pressure drop which is proportional to the flowrate and to the <tt>opening</tt> input, without computing any fluid property. It can be used for testing purposes, when
a simple model of a variable pressure loss is needed.</p>
<p>A medium model must be nevertheless be specified, so that the fluid ports can be connected to other components using the same medium model.</p>
<p>The model is adiabatic (no heat losses to the ambient) and neglects changes in kinetic energy from the inlet to the outlet. </p>
</HTML>", revisions = "<html>
<ul>
<li><i>2 Nov 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Adapted from the ThermoPower library.</li>
</ul>
</html>"));
      end ValveLinear;
      model ValveDiscrete "Valve for water/steam flows with linear pressure drop"
        extends Modelica.Fluid.Interfaces.PartialTwoPortTransport;
        parameter SI.Pressure dp_nominal "Nominal pressure drop at full opening=1";
        parameter Medium.MassFlowRate m_flow_nominal "Nominal mass flowrate at full opening=1";
        final parameter Types.HydraulicConductance k = m_flow_nominal / dp_nominal "Hydraulic conductance at full opening=1";
        Modelica.Blocks.Interfaces.BooleanInput open annotation(Placement(transformation(origin = {0,80}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
        parameter Real opening_min(min = 0) = 0 "Remaining opening if closed, causing small leakage flow";

      equation 
        m_flow = if open then 1 * k * dp else opening_min * k * dp;
        port_a.h_outflow = inStream(port_b.h_outflow);
        port_b.h_outflow = inStream(port_a.h_outflow);
        annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{0,50},{0,0}}, color = {0,0,0}),Rectangle(extent = {{ -20,60},{20,50}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{ -100,50},{100, -50},{100,50},{0,0},{ -100, -50},{ -100,50}}, fillColor = DynamicSelect({255,255,255}, if open > 0.5 then {0,255,0} else {255,255,255}), lineColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<HTML>
<p>
This very simple model provides a (small) pressure drop which is proportional to the flowrate if the Boolean open signal is <b>true</b>. Otherwise, the mass flow rate is zero. If opening_min > 0, a small leakage mass flow rate occurs when open = <b>false</b>.
</p>
<p>This model can be used for simplified modelling of on-off valves, when it is not important to accurately describe the pressure loss when the valve is open. Although the medium model is not used to determine the pressure loss, it must be nevertheless be specified, so that the fluid ports can be connected to other components using the same medium model.</p>
<p>The model is adiabatic (no heat losses to the ambient) and neglects changes in kinetic energy from the inlet to the outlet. </p>
<p>
In a diagram animation, the valve is shown in \"green\", when
it is open.
</p>
</HTML>", revisions = "<html>
<ul>
<li><i>Nov 2005</i>
    by Katja Poschlad (based on ValveLinear).</li>
</ul>
</html>"));
      end ValveDiscrete;
      package BaseClasses "Base classes used in the Valves package (only of interest to build new component models)"
        extends Modelica.Fluid.Icons.BaseClassLibrary;
        partial model PartialValve "Base model for valves"
          import Modelica.Fluid.Types.CvTypes;
          extends Modelica.Fluid.Interfaces.PartialTwoPortTransport(dp_start = dp_nominal, m_flow_start = m_flow_nominal, m_flow_small = system.m_flow_small);
          parameter CvTypes CvData = CvTypes.OpPoint "Selection of flow coefficient" annotation(Dialog(group = "Flow Coefficient"));
          parameter SI.Area Av(fixed = if CvData == CvTypes.Av then true else false, start = m_flow_nominal / sqrt(rho_nominal * dp_nominal) * valveCharacteristic(opening_nominal)) = 0 "Av (metric) flow coefficient" annotation(Dialog(group = "Flow Coefficient", enable = CvData == Modelica.Fluid.Types.CvTypes.Av));
          parameter Real Kv = 0 "Kv (metric) flow coefficient [m3/h]" annotation(Dialog(group = "Flow Coefficient", enable = CvData == CvTypes.Kv));
          parameter Real Cv = 0 "Cv (US) flow coefficient [USG/min]" annotation(Dialog(group = "Flow Coefficient", enable = CvData == CvTypes.Cv));
          parameter SI.Pressure dp_nominal "Nominal pressure drop" annotation(Dialog(group = "Nominal operating point"));
          parameter Medium.MassFlowRate m_flow_nominal "Nominal mass flowrate" annotation(Dialog(group = "Nominal operating point"));
          parameter Medium.Density rho_nominal = Medium.density_pTX(Medium.p_default, Medium.T_default, Medium.X_default) "Nominal inlet density" annotation(Dialog(group = "Nominal operating point"));
          parameter Real opening_nominal = 1 "Nominal opening" annotation(Dialog(group = "Nominal operating point"));
          parameter Boolean checkValve = false "Reverse flow stopped" annotation(Dialog(tab = "Assumptions"));
          replaceable function valveCharacteristic = Modelica.Fluid.Valves.BaseClasses.ValveCharacteristics.linear extends Modelica.Fluid.Valves.BaseClasses.ValveCharacteristics.baseFun "Inherent flow characteristic" annotation(choicesAllMatching = true);
          parameter SI.Pressure dp_small = system.dp_small "Regularisation of zero flow" annotation(Dialog(tab = "Advanced"));
          constant SI.Area Kv2Av = 2.77e-05 "Conversion factor";
          constant SI.Area Cv2Av = 2.4e-05 "Conversion factor";
          Modelica.Blocks.Interfaces.RealInput opening(min = 0, max = 1) "Valve position in the range 0-1" annotation(Placement(transformation(origin = {0,90}, extent = {{ -20, -20},{20,20}}, rotation = 270), iconTransformation(extent = {{ -20, -20},{20,20}}, rotation = 270, origin = {0,80})));

        initial equation 
          if CvData == CvTypes.Kv then
            Av = Kv * Kv2Av "Unit conversion";

          elseif CvData == CvTypes.Cv then
            Av = Cv * Cv2Av "Unit conversion";


          else

          end if;

        equation 
          port_a.h_outflow = inStream(port_b.h_outflow);
          port_b.h_outflow = inStream(port_a.h_outflow);
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{0,52},{0,0}}, color = {0,0,0}),Rectangle(extent = {{ -20,60},{20,52}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{ -100,50},{100, -50},{100,50},{0,0},{ -100, -50},{ -100,50}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = DynamicSelect({{ -100,0},{100, -0},{100,0},{0,0},{ -100, -0},{ -100,0}}, {{ -100,50 * opening},{ -100,50 * opening},{100, -50 * opening},{100,50 * opening},{0,0},{ -100, -50 * opening},{ -100,50 * opening}}), fillColor = {0,255,0}, lineColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -100,50},{100, -50},{100,50},{0,0},{ -100, -50},{ -100,50}}, lineColor = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<HTML>
<p>This is the base model for the <tt>ValveIncompressible</tt>, <tt>ValveVaporizing</tt>, and <tt>ValveCompressible</tt> valve models. The model is based on the IEC 534 / ISA S.75 standards for valve sizing.</p>
<p>The model optionally supports reverse flow conditions (assuming symmetrical behaviour) or check valve operation, and has been suitably regularized, compared to the equations in the standard, in order to avoid numerical singularities around zero pressure drop operating conditions. </p>
<p>The model assumes adiabatic operation (no heat losses to the ambient); changes in kinetic energy
from inlet to outlet are neglected in the energy balance. </p>
<p><b>Modelling options</b></p>
<p>The following options are available to specify the valve flow coefficient in fully open conditions:
<ul><li><tt>CvData = Modelica.Fluid.Types.CvTypes.Av</tt>: the flow coefficient is given by the metric <tt>Av</tt> coefficient (m^2).
<li><tt>CvData = Modelica.Fluid.Types.CvTypes.Kv</tt>: the flow coefficient is given by the metric <tt>Kv</tt> coefficient (m^3/h).
<li><tt>CvData = Modelica.Fluid.Types.CvTypes.Cv</tt>: the flow coefficient is given by the US <tt>Cv</tt> coefficient (USG/min).
<li><tt>CvData = Modelica.Fluid.Types.CvTypes.OpPoint</tt>: the flow is computed from the nominal operating point specified by <tt>p_nominal</tt>, <tt>dp_nominal</tt>, <tt>m_flow_nominal</tt>, <tt>rho_nominal</tt>, <tt>opening_nominal</tt>.
</ul>
<p>The nominal pressure drop <tt>dp_nominal</tt> must always be specified; to avoid numerical singularities, the flow characteristic is modified for pressure drops less than <tt>b*dp_nominal</tt> (the default value is 1% of the nominal pressure drop). Increase this parameter if numerical problems occur in valves with very low pressure drops.
<p>If <tt>checkValve</tt> is true, then the flow is stopped when the outlet pressure is higher than the inlet pressure; otherwise, reverse flow takes place. Use this option only when neede, as it increases the numerical complexity of the problem.
<p>The valve opening characteristic <tt>valveCharacteristic</tt>, linear by default, can be replaced by any user-defined function. Quadratic and equal percentage with customizable rangeability are already provided by the library.
</p>

<p>
The treatment of parameters <b>Kv</b> and <b>Cv</b> is
explained in detail in the
<a href=\"Modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.ValveCharacteristics\">Users Guide</a>.
</p>

</HTML>", revisions = "<html>
<ul>
<li><i>2 Nov 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Adapted from the ThermoPower library.</li>
</ul>
</html>"));
        end PartialValve;
        package ValveCharacteristics "Functions for valve characteristics"
          partial function baseFun "Base class for valve characteristics"
            extends Modelica.Icons.Function;
            input Real pos "Opening position (per unit)";
            output Real rc "Relative flow coefficient (per unit)";
          end baseFun;
          function linear "Linear characteristic"
            extends baseFun;
          algorithm 
            rc:=pos;
          end linear;
          function one "Constant characteristic"
            extends baseFun;
          algorithm 
            rc:=1;
          end one;
          function quadratic "Quadratic characteristic"
            extends baseFun;
          algorithm 
            rc:=pos * pos;
          end quadratic;
          function equalPercentage "Equal percentage characteristic"
            extends baseFun;
            input Real rangeability = 20 "Rangeability";
            input Real delta = 0.01;
          algorithm 
            rc:=if pos > delta then rangeability ^ (pos - 1) else pos / delta * rangeability ^ (delta - 1);
            annotation(Documentation(info = "<html>
This characteristic is such that the relative change of the flow coefficient is proportional to the change in the opening position:
<p> d(rc)/d(pos) = k d(pos).
<p> The constant k is expressed in terms of the rangeability, i.e. the ratio between the maximum and the minimum useful flow coefficient:
<p> rangeability = exp(k) = rc(1.0)/rc(0.0).
<p> The theoretical characteristic has a non-zero opening when pos = 0; the implemented characteristic is modified so that the valve closes linearly when pos &lt delta.
</html>"));
          end equalPercentage;
        end ValveCharacteristics;
      end BaseClasses;
      annotation(Documentation(info = "<html>

</html>"));
    end Valves;
    package Utilities "Utility models to construct fluid components (should not be used directly) "
      extends Modelica.Icons.Library;
      function checkBoundary "Check whether boundary definition is correct"
        extends Modelica.Icons.Function;
        input String mediumName;
        input String substanceNames[:] "Names of substances";
        input Boolean singleState;
        input Boolean define_p;
        input Real X_boundary[:];
        input String modelName = "??? boundary ???";
      protected 
        Integer nX = size(X_boundary, 1);
        String X_str;
      algorithm 
        assert(not singleState or singleState and define_p, "
Wrong value of parameter define_p (= false) in model \"" + modelName + "\":
The selected medium \"" + mediumName + "\" has Medium.singleState=true.
Therefore, an boundary density cannot be defined and
define_p = true is required.
");
        for i in 1:nX loop
                  assert(X_boundary[i] >= 0.0, "
Wrong boundary mass fractions in medium \"" + mediumName + "\" in model \"" + modelName + "\":
The boundary value X_boundary(" + String(i) + ") = " + String(X_boundary[i]) + "
is negative. It must be positive.
");

        end for;
        if nX > 0 and abs(sum(X_boundary) - 1.0) > 1e-10 then 
                X_str:="";
        for i in 1:nX loop
                  X_str:=X_str + "   X_boundary[" + String(i) + "] = " + String(X_boundary[i]) + " \"" + substanceNames[i] + "\"\n";

        end for;
        Modelica.Utilities.Streams.error("The boundary mass fractions in medium \"" + mediumName + "\" in model \"" + modelName + "\"\n" + "do not sum up to 1. Instead, sum(X_boundary) = " + String(sum(X_boundary)) + ":\n" + X_str);

        else 
        end if;
      end checkBoundary;
      function regRoot "Anti-symmetric square root approximation with finite derivative in the origin"
        extends Modelica.Icons.Function;
        input Real x;
        input Real delta = 0.01 "Range of significant deviation from sqrt(abs(x))*sgn(x)";
        output Real y;
        annotation(derivative(zeroDerivative = delta) = Utilities.regRoot_der, Documentation(info = "<html>
This function approximates sqrt(abs(x))*sgn(x), such that the derivative is finite and smooth in x=0.
</p>
<p>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th>Function</th><th>Approximation</th><th>Range</th></tr>
<tr><td>y = regRoot(x)</td><td>y ~= sqrt(abs(x))*sgn(x)</td><td>abs(x) &gt;&gt delta</td></tr>
<tr><td>y = regRoot(x)</td><td>y ~= x/sqrt(delta)</td><td>abs(x) &lt;&lt  delta</td></tr>
</table>
<p>
With the default value of delta=0.01, the difference between sqrt(x) and regRoot(x) is 16% around x=0.01, 0.25% around x=0.1 and 0.0025% around x=1.
</p>
</html>", revisions = "<html>
<ul>
<li><i>15 Mar 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Created. </li>
</ul>
</html>"));
      algorithm 
        y:=x / (x * x + delta * delta) ^ 0.25;
        annotation(Documentation(info = "<html>
This function approximates sqrt(x)*sign(x), such that the derivative is finite and smooth in x=0.
</p>
<p>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th>Function</th><th>Approximation</th><th>Range</th></tr>
<tr><td>y = sqrtReg(x)</td><td>y ~= sqrt(abs(x))*sign(x)</td><td>abs(x) &gt;&gt delta</td></tr>
<tr><td>y = sqrtReg(x)</td><td>y ~= x/delta</td><td>abs(x) &lt;&lt  delta</td></tr>
</table>
<p>
With the default value of delta=0.01, the difference between sqrt(x) and sqrtReg(x) is 0.5% around x=0.1 and 0.005% around x=1.
</p>
</html>", revisions = "<html>
<ul>
<li><i>15 Mar 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Created. </li>
</ul>
</html>"));
      end regRoot;
      function regRoot_der "Derivative of regRoot"
        extends Modelica.Icons.Function;
        input Real x;
        input Real delta = 0.01 "Range of significant deviation from sqrt(x)";
        input Real dx "Derivative of x";
        output Real dy;
      algorithm 
        dy:=(dx * 0.5 * (x * x + 2 * delta * delta)) / (x * x + delta * delta) ^ 1.25;
        annotation(Documentation(info = "<html>
</html>", revisions = "<html>
<ul>
<li><i>15 Mar 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Created. </li>
</ul>
</html>"));
      end regRoot_der;
      function regSquare "Anti-symmetric square approximation with non-zero derivative in the origin"
        extends Modelica.Icons.Function;
        input Real x;
        input Real delta = 0.01 "Range of significant deviation from x^2*sgn(x)";
        output Real y;
        annotation(Documentation(info = "<html>
This function approximates x^2*sgn(x), such that the derivative is non-zero in x=0.
</p>
<p>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th>Function</th><th>Approximation</th><th>Range</th></tr>
<tr><td>y = regSquare(x)</td><td>y ~= x^2*sgn(x)</td><td>abs(x) &gt;&gt delta</td></tr>
<tr><td>y = regSquare(x)</td><td>y ~= x*delta</td><td>abs(x) &lt;&lt  delta</td></tr>
</table>
<p>
With the default value of delta=0.01, the difference between x^2 and regSquare(x) is 41% around x=0.01, 0.4% around x=0.1 and 0.005% around x=1.
</p>
</p>
</html>", revisions = "<html>
<ul>
<li><i>15 Mar 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Created. </li>
</ul>
</html>"));
      algorithm 
        y:=x * sqrt(x * x + delta * delta);
        annotation(Documentation(info = "<html>
This function approximates sqrt(x)*sign(x), such that the derivative is finite and smooth in x=0.
</p>
<p>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th>Function</th><th>Approximation</th><th>Range</th></tr>
<tr><td>y = sqrtReg(x)</td><td>y ~= sqrt(abs(x))*sign(x)</td><td>abs(x) &gt;&gt delta</td></tr>
<tr><td>y = sqrtReg(x)</td><td>y ~= x/delta</td><td>abs(x) &lt;&lt  delta</td></tr>
</table>
<p>
With the default value of delta=0.01, the difference between sqrt(x) and sqrtReg(x) is 0.5% around x=0.1 and 0.005% around x=1.
</p>
</html>", revisions = "<html>
<ul>
<li><i>15 Mar 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Created. </li>
</ul>
</html>"));
      end regSquare;
      function regPow "Anti-symmetric power approximation with non-zero derivative in the origin"
        extends Modelica.Icons.Function;
        input Real x;
        input Real a;
        input Real delta = 0.01 "Range of significant deviation from x^a*sgn(x)";
        output Real y;
        annotation(Documentation(info = "<html>
This function approximates abs(x)^a*sign(x), such that the derivative is positive, finite and smooth in x=0.
</p>
<p>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th>Function</th><th>Approximation</th><th>Range</th></tr>
<tr><td>y = regPow(x)</td><td>y ~= abs(x)^a*sgn(x)</td><td>abs(x) &gt;&gt delta</td></tr>
<tr><td>y = regPow(x)</td><td>y ~= x*delta^(a-1)</td><td>abs(x) &lt;&lt  delta</td></tr>
</table>
</html>", revisions = "<html>
<ul>
<li><i>15 Mar 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Created. </li>
</ul>
</html>"));
      algorithm 
        y:=x * (x * x + delta * delta) ^ ((a - 1) / 2);
        annotation(Documentation(info = "<html>
This function approximates sqrt(x)*sign(x), such that the derivative is finite and smooth in x=0.
</p>
<p>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th>Function</th><th>Approximation</th><th>Range</th></tr>
<tr><td>y = sqrtReg(x)</td><td>y ~= sqrt(abs(x))*sign(x)</td><td>abs(x) &gt;&gt delta</td></tr>
<tr><td>y = sqrtReg(x)</td><td>y ~= x/delta</td><td>abs(x) &lt;&lt  delta</td></tr>
</table>
<p>
With the default value of delta=0.01, the difference between sqrt(x) and sqrtReg(x) is 0.5% around x=0.1 and 0.005% around x=1.
</p>
</html>", revisions = "<html>
<ul>
<li><i>15 Mar 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Created. </li>
</ul>
</html>"));
      end regPow;
      function regRoot2 "Anti-symmetric approximation of square root with discontinuous factor so that the first derivative is finite and continuous"
        extends Modelica.Icons.Function;
        input Real x "abscissa value";
        input Real x_small(min = 0) = 0.01 "approximation of function for |x| <= x_small";
        input Real k1(min = 0) = 1 "y = if x>=0 then sqrt(k1*x) else -sqrt(k2*|x|)";
        input Real k2(min = 0) = 1 "y = if x>=0 then sqrt(k1*x) else -sqrt(k2*|x|)";
        input Boolean use_yd0 = false "= true, if yd0 shall be used";
        input Real yd0(min = 0) = 1 "Desired derivative at x=0: dy/dx = yd0";
        output Real y "ordinate value";
        annotation(smoothOrder = 2, Documentation(info = "<html>
<p>
Approximates the function
</p>
<pre>
   y = <b>if</b> x &ge; 0 <b>then</b> <b>sqrt</b>(k1*x) <b>else</b> -<b>sqrt</b>(k2*<b>abs</b>(x)), with k1, k2 > 0
</pre>
<p>
in such a way that within the region -x_small &le; x &le; x_small,
the function is described by two polynomials of third order
(one in the region -x_small .. 0 and one within the region 0 .. x_small)
such that
</p>
<ul>
<li> The derivative at x=0 is finite. </li>
<li> The overall function is continuous with a
     continuous first derivative everywhere.</li>
<li> If parameter use_yd0 = <b>false</b>, the two polynomials
     are constructed such that the second derivatives at x=0
     are identical. If use_yd0 = <b>true</b>, the derivative
     at x=0 is explicitly provided via the additional argument
     yd0. If necessary, the derivative yd0 is automatically
     reduced in order that the polynomials are strict monotonically
     increasing <i>[Fritsch and Carlson, 1980]</i>.</li>
</ul>
<p>
Typical screenshots for two different configurations
are shown below. The first one with k1=k2=1:
</p>
<p>
<img src=\"../Images/Fluid/Components/regRoot2_a.png\">
</p>
<p>
and the second one with k1=1 and k2=3:
</p>
<p>
<img src=\"../Images/Fluid/Components/regRoot2_b.png\">
</p>

<p>
The (smooth) derivative of the function with
k1=1, k2=3 is shown in the next figure:
<p>
<img src=\"../Images/Fluid/Components/regRoot2_c.png\">
</p>

<p>
<b>Literature</b>
</p>

<dl>
<dt> Fritsch F.N. and Carlson R.E. (1980):</dt>
<dd> <b>Monotone piecewise cubic interpolation</b>.
     SIAM J. Numerc. Anal., Vol. 17, No. 2, April 1980, pp. 238-246</dd>
</dl>
</html>", revisions = "<html>
<ul>
<li><i>Nov., 2005</i>
    by <a href=\"mailto:Martin.Otter@DLR.de\">Martin Otter</a>:<br>
    Designed and implemented.</li>
</ul>
</html>"));
      protected 
        encapsulated function regRoot2_utility "Interpolating with two 3-order polynomials with a prescribed derivative at x=0"
          import Modelica.Fluid.Utilities.evaluatePoly3_derivativeAtZero;
          input Real x;
          input Real x1 "approximation of function abs(x) < x1";
          input Real k1 "y = if x>=0 then sqrt(k1*x) else -sqrt(k2*|x|); k1 >= k2";
          input Real k2 "y = if x>=0 then sqrt(k1*x) else -sqrt(k2*|x|))";
          input Boolean use_yd0 "= true, if yd0 shall be used";
          input Real yd0(min = 0) "Desired derivative at x=0: dy/dx = yd0";
          output Real y;
          annotation(smoothOrder = 2);
        protected 
          Real x2;
          Real xsqrt1;
          Real xsqrt2;
          Real y1;
          Real y2;
          Real y1d;
          Real y2d;
          Real w;
          Real y0d;
          Real w1;
          Real w2;
        algorithm 
          x2:= -x1 * k2 / k1;
          if x <= x2 then 
                    y:= -sqrt(k2 * abs(x));

          else           y1:=sqrt(k1 * x1);
          y2:= -sqrt(k2 * abs(x2));
          y1d:=sqrt(k1 / x1) / 2;
          y2d:=sqrt(k2 / abs(x2)) / 2;
          if use_yd0 then 
                    y0d:=yd0;

          else           w:=x2 / x1;
          y0d:=((3 * y2 - x2 * y2d) / w - (3 * y1 - x1 * y1d) * w) / (2 * x1 * (1 - w));

          end if;
          w1:=sqrt((8.75 * k1) / x1);
          w2:=sqrt((8.75 * k2) / abs(x2));
          y0d:=min(y0d, 0.9 * min(w1, w2));
          y:=y1 * (if x >= 0 then evaluatePoly3_derivativeAtZero(x / x1, 1, 1, (y1d * x1) / y1, (y0d * x1) / y1) else evaluatePoly3_derivativeAtZero(x / x1, x2 / x1, y2 / y1, (y2d * x1) / y1, (y0d * x1) / y1));

          end if;
        end regRoot2_utility;
      algorithm 
        y:=smooth(2, if x >= x_small then sqrt(k1 * x) else if x <=  -x_small then  -sqrt(k2 * abs(x)) else if k1 >= k2 then regRoot2_utility(x, x_small, k1, k2, use_yd0, yd0) else  -regRoot2_utility( -x, x_small, k2, k1, use_yd0, yd0));
      end regRoot2;
      function regSquare2 "Anti-symmetric approximation of square with discontinuous factor so that the first derivative is non-zero and is continuous"
        extends Modelica.Icons.Function;
        input Real x "abscissa value";
        input Real x_small(min = 0) = 0.01 "approximation of function for |x| <= x_small";
        input Real k1(min = 0) = 1 "y = (if x>=0 then k1 else k2)*x*|x|";
        input Real k2(min = 0) = 1 "y = (if x>=0 then k1 else k2)*x*|x|";
        input Boolean use_yd0 = false "= true, if yd0 shall be used";
        input Real yd0(min = 0) = 1 "Desired derivative at x=0: dy/dx = yd0";
        output Real y "ordinate value";
        annotation(smoothOrder = 2, Documentation(info = "<html>
<p>
Approximates the function
</p>
<pre>
   y = <b>if</b> x &ge; 0 <b>then</b> k1*x*x <b>else</b> -k2*x*x, with k1, k2 > 0
</pre>
<p>
in such a way that within the region -x_small &le; x &le; x_small,
the function is described by two polynomials of third order
(one in the region -x_small .. 0 and one within the region 0 .. x_small)
such that
</p>

<ul>
<li> The derivative at x=0 is non-zero (in order that the
     inverse of the function does not have an infinite derivative). </li>
<li> The overall function is continuous with a
     continuous first derivative everywhere.</li>
<li> If parameter use_yd0 = <b>false</b>, the two polynomials
     are constructed such that the second derivatives at x=0
     are identical. If use_yd0 = <b>true</b>, the derivative
     at x=0 is explicitly provided via the additional argument
     yd0. If necessary, the derivative yd0 is automatically
     reduced in order that the polynomials are strict monotonically
     increasing <i>[Fritsch and Carlson, 1980]</i>.</li>
</ul>
</ul>
<p>
A typical screenshot for k1=1, k2=3 is shown in the next figure:
</p>
<p>
<img src=\"../Images/Fluid/Components/regSquare2_b.png\">
</p>

<p>
The (smooth, non-zero) derivative of the function with
k1=1, k2=3 is shown in the next figure:
</p>

<p>
<img src=\"../Images/Fluid/Components/regSquare2_c.png\">
</p>

<p>
<b>Literature</b>
</p>

<dl>
<dt> Fritsch F.N. and Carlson R.E. (1980):</dt>
<dd> <b>Monotone piecewise cubic interpolation</b>.
     SIAM J. Numerc. Anal., Vol. 17, No. 2, April 1980, pp. 238-246</dd>
</dl>
</html>", revisions = "<html>
<ul>
<li><i>Nov., 2005</i>
    by <a href=\"mailto:Martin.Otter@DLR.de\">Martin Otter</a>:<br>
    Designed and implemented.</li>
</ul>
</html>"));
      protected 
        encapsulated function regSquare2_utility "Interpolating with two 3-order polynomials with a prescribed derivative at x=0"
          import Modelica.Fluid.Utilities.evaluatePoly3_derivativeAtZero;
          input Real x;
          input Real x1 "approximation of function abs(x) < x1";
          input Real k1 "y = (if x>=0 then k1 else -k2)*x*|x|; k1 >= k2";
          input Real k2 "y = (if x>=0 then k1 else -k2)*x*|x|";
          input Boolean use_yd0 = false "= true, if yd0 shall be used";
          input Real yd0(min = 0) = 1 "Desired derivative at x=0: dy/dx = yd0";
          output Real y;
          annotation(smoothOrder = 2);
        protected 
          Real x2;
          Real y1;
          Real y2;
          Real y1d;
          Real y2d;
          Real w;
          Real w1;
          Real w2;
          Real y0d;
        algorithm 
          x2:= -x1;
          if x <= x2 then 
                    y:= -k2 * x ^ 2;

          else           y1:=k1 * x1 ^ 2;
          y2:= -k2 * x2 ^ 2;
          y1d:=k1 * 2 * x1;
          y2d:= -k2 * 2 * x2;
          if use_yd0 then 
                    y0d:=yd0;

          else           w:=x2 / x1;
          y0d:=((3 * y2 - x2 * y2d) / w - (3 * y1 - x1 * y1d) * w) / (2 * x1 * (1 - w));

          end if;
          w1:=sqrt(5) * k1 * x1;
          w2:=sqrt(5) * k2 * abs(x2);
          y0d:=min(y0d, 0.9 * min(w1, w2));
          y:=if x >= 0 then evaluatePoly3_derivativeAtZero(x, x1, y1, y1d, y0d) else evaluatePoly3_derivativeAtZero(x, x2, y2, y2d, y0d);

          end if;
        end regSquare2_utility;
      algorithm 
        y:=smooth(2, if x >= x_small then k1 * x ^ 2 else if x <=  -x_small then  -k2 * x ^ 2 else if k1 >= k2 then regSquare2_utility(x, x_small, k1, k2, use_yd0, yd0) else  -regSquare2_utility( -x, x_small, k2, k1, use_yd0, yd0));
      end regSquare2;
      function regStep "Approximation of a general step, such that the characteristic is continuous and differentiable"
        extends Modelica.Icons.Function;
        input Real x "Abscissa value";
        input Real y1 "Ordinate value for x > 0";
        input Real y2 "Ordinate value for x < 0";
        input Real x_small(min = 0) = 1e-05 "Approximation of step for -x_small <= x <= x_small; x_small > 0 required";
        output Real y "Ordinate value to approximate y = if x > 0 then y1 else y2";
        annotation(Documentation(revisions = "<html>
<ul>
<li><i>April 29, 2008</i>
    by <a href=\"mailto:Martin.Otter@DLR.de\">Martin Otter</a>:<br>
    Designed and implemented.</li>
<li><i>August 12, 2008</i>
    by <a href=\"mailto:Michael.Sielemann@dlr.de\">Michael Sielemann</a>:<br>
    Minor modification to cover the limit case <code>x_small -> 0</code> without division by zero.</li>
</ul>
</html>"));
      algorithm 
        y:=smooth(1, if x > x_small then y1 else if x <  -x_small then y2 else if abs(x_small) > 0 then (x / x_small * ((x / x_small) ^ 2 - 3) * (y2 - y1)) / 4 + (y1 + y2) / 2 else (y1 + y2) / 2);
      end regStep;
      function evaluatePoly3_derivativeAtZero "Evaluate polynomial of order 3 that passes the origin with a predefined derivative"
        extends Modelica.Icons.Function;
        input Real x "Value for which polynomial shall be evaluated";
        input Real x1 "Abscissa value";
        input Real y1 "y1=f(x1)";
        input Real y1d "First derivative at y1";
        input Real y0d "First derivative at f(x=0)";
        output Real y;
        annotation(smoothOrder = 3, Documentation(info = "<html>

</html>"));
      protected 
        Real a1;
        Real a2;
        Real a3;
        Real xx;
      algorithm 
        a1:=x1 * y0d;
        a2:=3 * y1 - x1 * y1d - 2 * a1;
        a3:=y1 - a2 - a1;
        xx:=x / x1;
        y:=xx * (a1 + xx * (a2 + xx * a3));
      end evaluatePoly3_derivativeAtZero;
      annotation(Documentation(info = "<html>

</html>"));
      function regFun3 "Co-monotonic and C1 smooth regularization function"
        input Real x "Abscissa value";
        input Real x0 "Lower abscissa value";
        input Real x1 "Upper abscissa value";
        input Real y0 "Ordinate value at lower ordinate value";
        input Real y1 "Ordinate value at upper ordinate value";
        input Real y0d "Derivative at lower abscissa value";
        input Real y1d "Derivative at upper abscissa value";
        output Real y "Ordinate value";
        output Real c "Slope of linear section between two cubic polynomials or dummy linear section slope if single cubic is used";
        annotation(smoothOrder = 1, Documentation(revisions = "<html>
<ul>
<li><i>May 2008</i>
    by <a href=\"mailto:Michael.Sielemann@dlr.de\">Michael Sielemann</a>:<br>
    Designed and implemented.</li>
</ul>
</html>", info = "<html>
<p>
Approximates a function in a region between <code>x0</code> and <code>x1</code>
such that
</p>
<ul>
<li> The overall function is continuous with a
     continuous first derivative everywhere.</li>
<li> The function is co-monotone with the given
     data points.</li>
</ul>
<p>
In this region, a continuation is constructed from the given points
<code>(x0, y0)</code>, <code>(x1, y1)</code> and the respective
derivatives. For this purpose, a single polynomial of third order or two
cubic polynomials with a linear section in between are used <i>[Gasparo
and Morandi, 1991]</i>. This algorithm was extended with two additional
conditions to avoid saddle points with zero/infinite derivative that lead to
integrator step size reduction to zero.
</p>
<p>
This function was developed for pressure loss correlations properly
addressing the static head on top of the established requirements
for monotonicity and smoothness. In this case, the present function
allows to implement the exact solution in the limit of
<code>x1-x0 -> 0</code> or <code>y1-y0 -> 0</code>.
</p>
<p>
Typical screenshots for two different configurations
are shown below. The first one illustrates five different settings of <code>xi</code> and <code>yid</code>:
</p>
<p>
<img src=\"../Images/Fluid/Components/regFun3_a.png\">
</p>
<p>
The second graph shows the continous derivative of this regularization function:
</p>
<p>
<img src=\"../Images/Fluid/Components/regFun3_b.png\">
</p>

<p>
<b>Literature</b>
</p>

<dl>
<dt> Gasparo M. G. and Morandi R. (1991):</dt>
<dd> <b>Piecewise cubic monotone interpolation with assigned slopes</b>.
     Computing, Vol. 46, Issue 4, December 1991, pp. 355 - 365.</dd>
</dl>
</html>"));
      protected 
        Real h0 "Width of interval i=0";
        Real Delta0 "Slope of secant on interval i=0";
        Real xstar "Inflection point of cubic polynomial S0";
        Real mu "Distance of inflection point and left limit x0";
        Real eta "Distance of right limit x1 and inflection point";
        Real omega "Slope of subic polynomial S0 at inflection point";
        Real rho "Weighting factor of eta and eta_tilde, mu and mu_tilde";
        Real theta0 "Slope metric";
        Real mu_tilde "Distance of start of linear section and left limit x0";
        Real eta_tilde "Distance of right limit x1 and end of linear section";
        Real xi1 "Start of linear section ";
        Real xi2 "End of linear section ";
        Real a1 "Leading coefficient of cubic on the left";
        Real a2 "Leading coefficient of cubic on the right";
        Real const12 "Integration constant of left cubic, linear section";
        Real const3 "Integration constant of right cubic";
        Real aux01;
        Real aux02;
        Boolean useSingleCubicPolynomial = false "Indicate to override further logic and use single cubic";
      algorithm 
        assert(x0 < x1, "regFun3(): Data points not sorted appropriately (x0 = " + String(x0) + " > x1 = " + String(x1) + "). Please flip arguments.");
        h0:=x1 - x0;
        Delta0:=(y1 - y0) / h0;
        if abs(Delta0) <= 0 then 
                y:=y0;

        else         if abs(y1d + y0d - 2 * Delta0) < 100 * Modelica.Constants.eps then 
                xstar:=(x1 - x0) * (2 * y0d + y1d - 3 * Delta0) * (if y0d + y1d - 2 * Delta0 >= 0 then 1 else  -1) * Modelica.Constants.inf;

        else         xstar:=(1 / 3 * ( -3 * x0 * y0d - 3 * x0 * y1d + 6 * x0 * Delta0 - 2 * h0 * y0d - h0 * y1d + 3 * h0 * Delta0)) / ( -y0d - y1d + 2 * Delta0);

        end if;
        mu:=xstar - x0;
        eta:=x1 - xstar;
        omega:=(3 * (y0d + y1d - 2 * Delta0) * (xstar - x0) ^ 2) / h0 ^ 2 + (2 * ( -2 * y0d - y1d + 3 * Delta0) * (xstar - x0)) / h0 + y0d;
        aux01:=0.25 * sign(Delta0) * min(abs(omega), abs(Delta0)) "Slope c if not using plain cubic S0";
        if abs(y0d - y1d) <= 100 * Modelica.Constants.eps then 
                aux02:=y0d;
        if y1 > y0 + y0d * (x1 - x0) then 
                useSingleCubicPolynomial:=true;

        else 
        end if;
        elseif abs(y1d + y0d - 2 * Delta0) < 100 * Modelica.Constants.eps then
          aux02:=(6 * Delta0 * (y1d + y0d - 3 / 2 * Delta0) - y1d * y0d - y1d ^ 2 - y0d ^ 2) * (if y1d + y0d - 2 * Delta0 >= 0 then 1 else  -1) * Modelica.Constants.inf;


        else         aux02:=(6 * Delta0 * (y1d + y0d - 3 / 2 * Delta0) - y1d * y0d - y1d ^ 2 - y0d ^ 2) / (3 * (y1d + y0d - 2 * Delta0));

        end if;
        if (mu > 0 and eta < h0 and Delta0 * omega <= 0 or abs(aux01) < abs(aux02) and aux02 * Delta0 >= 0 or abs(aux01) < abs(0.1 * Delta0)) and not useSingleCubicPolynomial then 
                c:=aux01;
        if abs(c) < abs(aux02) and aux02 * Delta0 >= 0 then 
                c:=aux02;

        else 
        end if;
        if abs(c) < abs(0.1 * Delta0) then 
                c:=0.1 * Delta0;

        else 
        end if;
        theta0:=(y0d * mu + y1d * eta) / h0;
        if abs(theta0 - c) < 1e-06 then 
                c:=(1 - 1e-06) * theta0;

        else 
        end if;
        rho:=(3 * (Delta0 - c)) / (theta0 - c);
        mu_tilde:=rho * mu;
        eta_tilde:=rho * eta;
        xi1:=x0 + mu_tilde;
        xi2:=x1 - eta_tilde;
        a1:=(y0d - c) / max(mu_tilde ^ 2, 100 * Modelica.Constants.eps);
        a2:=(y1d - c) / max(eta_tilde ^ 2, 100 * Modelica.Constants.eps);
        const12:=y0 - a1 / 3 * (x0 - xi1) ^ 3 - c * x0;
        const3:=y1 - a2 / 3 * (x1 - xi2) ^ 3 - c * x1;
        if x < xi1 then 
                y:=a1 / 3 * (x - xi1) ^ 3 + c * x + const12;
        elseif x < xi2 then
          y:=c * x + const12;


        else         y:=a2 / 3 * (x - xi2) ^ 3 + c * x + const3;

        end if;

        else         y:=((y0d + y1d - 2 * Delta0) * (x - x0) ^ 3) / h0 ^ 2 + (( -2 * y0d - y1d + 3 * Delta0) * (x - x0) ^ 2) / h0 + y0d * (x - x0) + y0;
        aux01:=(x0 + x1) / 2;
        c:=(3 * (y0d + y1d - 2 * Delta0) * (aux01 - x0) ^ 2) / h0 ^ 2 + (2 * ( -2 * y0d - y1d + 3 * Delta0) * (aux01 - x0)) / h0 + y0d;

        end if;

        end if;
      end regFun3;
      function cubicHermite "Evaluate a cubic Hermite spline"
        input Real x "Abscissa value";
        input Real x1 "Lower abscissa value";
        input Real x2 "Upper abscissa value";
        input Real y1 "Lower ordinate value";
        input Real y2 "Upper ordinate value";
        input Real y1d "Lower gradient";
        input Real y2d "Upper gradient";
        output Real y "Interpolated ordinate value";
        annotation(smoothOrder = 3, Documentation(revisions = "<html>
<ul>
<li><i>May 2008</i>
    by <a href=\"mailto:Michael.Sielemann@dlr.de\">Michael Sielemann</a>:<br>
    Designed and implemented.</li>
</ul>
</html>"));
      protected 
        Real h "Distance between x1 and x2";
        Real t "abscissa scaled with h, i.e. t=[0..1] within x=[x1..x2]";
        Real h00 "Basis function 00 of cubic Hermite spline";
        Real h10 "Basis function 10 of cubic Hermite spline";
        Real h01 "Basis function 01 of cubic Hermite spline";
        Real h11 "Basis function 11 of cubic Hermite spline";
        Real aux3 "t cube";
        Real aux2 "t square";
      algorithm 
        h:=x2 - x1;
        if abs(h) > 0 then 
                t:=(x - x1) / h;
        aux3:=t ^ 3;
        aux2:=t ^ 2;
        h00:=2 * aux3 - 3 * aux2 + 1;
        h10:=aux3 - 2 * aux2 + t;
        h01:= -2 * aux3 + 3 * aux2;
        h11:=aux3 - aux2;
        y:=y1 * h00 + h * y1d * h10 + y2 * h01 + h * y2d * h11;

        else         y:=(y1 + y2) / 2;

        end if;
      end cubicHermite;
      function cubicHermite_withDerivative "Evaluate a cubic Hermite spline, return value and derivative"
        input Real x "Abscissa value";
        input Real x1 "Lower abscissa value";
        input Real x2 "Upper abscissa value";
        input Real y1 "Lower ordinate value";
        input Real y2 "Upper ordinate value";
        input Real y1d "Lower gradient";
        input Real y2d "Upper gradient";
        output Real y "Interpolated ordinate value";
        output Real dy_dx "Derivative dy/dx at abscissa value x";
        annotation(smoothOrder = 3, Documentation(revisions = "<html>
<ul>
<li><i>May 2008</i>
    by <a href=\"mailto:Michael.Sielemann@dlr.de\">Michael Sielemann</a>:<br>
    Designed and implemented.</li>
</ul>
</html>"));
      protected 
        Real h "Distance between x1 and x2";
        Real t "abscissa scaled with h, i.e. t=[0..1] within x=[x1..x2]";
        Real h00 "Basis function 00 of cubic Hermite spline";
        Real h10 "Basis function 10 of cubic Hermite spline";
        Real h01 "Basis function 01 of cubic Hermite spline";
        Real h11 "Basis function 11 of cubic Hermite spline";
        Real h00d "d/dt h00";
        Real h10d "d/dt h10";
        Real h01d "d/dt h01";
        Real h11d "d/dt h11";
        Real aux3 "t cube";
        Real aux2 "t square";
      algorithm 
        h:=x2 - x1;
        if abs(h) > 0 then 
                t:=(x - x1) / h;
        aux3:=t ^ 3;
        aux2:=t ^ 2;
        h00:=2 * aux3 - 3 * aux2 + 1;
        h10:=aux3 - 2 * aux2 + t;
        h01:= -2 * aux3 + 3 * aux2;
        h11:=aux3 - aux2;
        h00d:=6 * (aux2 - t);
        h10d:=3 * aux2 - 4 * t + 1;
        h01d:=6 * (t - aux2);
        h11d:=3 * aux2 - 2 * t;
        y:=y1 * h00 + h * y1d * h10 + y2 * h01 + h * y2d * h11;
        dy_dx:=(y1 * h00d) / h + y1d * h10d + (y2 * h01d) / h + y2d * h11d;

        else         y:=(y1 + y2) / 2;
        dy_dx:=sign(y2 - y1) * Modelica.Constants.inf;

        end if;
      end cubicHermite_withDerivative;
    end Utilities;
    package Types "Common types for fluid models"
      annotation(preferedView = "info", Documentation(info = "<html>

</html>"));
      type HydraulicConductance = Real(final quantity = "HydraulicConductance", final unit = "kg/(s.Pa)") "Real type for hydraulic conductance";
      type HydraulicResistance = Real(final quantity = "HydraulicResistance", final unit = "Pa.s/kg") "Real type for hydraulic resistance";
      type Dynamics = enumeration(DynamicFreeInitial "DynamicFreeInitial -- Dynamic balance, Initial guess value", FixedInitial "FixedInitial -- Dynamic balance, Initial value fixed", SteadyStateInitial  "SteadyStateInitial -- Dynamic balance, Steady state initial with guess value", SteadyState  "SteadyState -- Steady state balance, Initial guess value") "Enumeration to define definition of balance equations" annotation(Documentation(info = "<html>
<p>
Enumeration to define the formulation of balance equations
(to be selected via choices menu):
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Dynamics.</b></th><th><b>Meaning</b></th></tr>
<tr><td>DynamicFreeInitial</td><td>Dynamic balance, Initial guess value</td></tr>

<tr><td>FixedInitial</td><td>Dynamic balance, Initial value fixed</td></tr>

<tr><td>SteadyStateInitial</td><td>Dynamic balance, Steady state initial with guess value</td></tr>

<tr><td>SteadyState</td><td>Steady state balance, Initial guess value</td></tr>
</table>

<p>
The enumeration \"Dynamics\" is used for the mass, energy and momentum balance equations
respectively. The exact meaning for the three balance equations is stated in the following
tables:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><td colspan=\"3\"><b>Mass balance</b> </td>
<tr><td><b>Dynamics.</b></td>
    <td><b>Balance equation</b></td>
    <td><b>Initial condition</b></td></tr>

<tr><td> DynamicFreeInitial</td>
    <td> no restrictions </td>
    <td> no initial conditions </td></tr>

<tr><td> FixedInitial</td>
    <td> no restrictions </td>
    <td> <b>if</b> Medium.singleState <b>then</b> <br>
         &nbsp;&nbsp;no initial condition<br>
         <b>else</b> p=p_start </td></tr>

<tr><td> SteadyStateInitial</td>
    <td> no restrictions </td>
    <td> <b>if</b> Medium.singleState <b>then</b> <br>
         &nbsp;&nbsp;no initial condition<br>
         <b>else</b> <b>der</b>(p)=0 </td></tr>

<tr><td> SteadyState</td>
    <td> <b>der</b>(m)=0  </td></tr>
    <td> no initial conditions </td></tr>
</table>

&nbsp;<br>


<table border=1 cellspacing=0 cellpadding=2>
<tr><td colspan=\"3\"><b>Energy balance</b> </td>
<tr><td><b>Dynamics.</b></td>
    <td><b>Balance equation</b></td>
    <td><b>Initial condition</b></td></tr>

<tr><td> DynamicFreeInitial</td>
    <td> no restrictions </td>
    <td> no initial conditions </td></tr>

<tr><td> FixedInitial</td>
    <td> no restrictions </td>
    <td> T=T_start or h=h_start </td></tr>

<tr><td> SteadyStateInitial</td>
    <td> no restrictions </td>
    <td> <b>der</b>(T)=0 or <b>der</b>(h)=0 </td></tr>

<tr><td> SteadyState</td>
    <td> <b>der</b>(U)=0  </td></tr>
    <td> no initial conditions </td></tr>
</table>

&nbsp;<br>



<table border=1 cellspacing=0 cellpadding=2>
<tr><td colspan=\"3\"><b>Momentum balance</b> </td>
<tr><td><b>Dynamics.</b></td>
    <td><b>Balance equation</b></td>
    <td><b>Initial condition</b></td></tr>

<tr><td> DynamicFreeInitial</td>
    <td> no restrictions </td>
    <td> no initial conditions </td></tr>

<tr><td> FixedInitial</td>
    <td> no restrictions </td>
    <td> m_flow = m_flow_start </td></tr>

<tr><td> SteadyStateInitial</td>
    <td> no restrictions </td>
    <td> <b>der</b>(m_flow)=0 </td></tr>

<tr><td> SteadyState</td>
    <td> <b>der</b>(m_flow)=0 </td></tr>
    <td> no initial conditions </td></tr>
</table>


<p>
In the tables above, the equations are given for one-substance fluids. For multiple-substance
fluids and for trace substances, equivalent equations hold.
</p>

<p>
Medium.singleState is a medium property and defines whether the medium is only
described by one state (+ the mass fractions in case of a multi-substance fluid). In such
a case one initial condition less must be provided. For example, incompressible
media have Medium.singleState = <b>true</b>.
</p>

</html>"));
      type CvTypes = enumeration(Av "Av (metric) flow coefficient", Kv "Kv (metric) flow coefficient", Cv  "Cv (US) flow coefficient", OpPoint  "Av defined by operating point") "Enumeration to define the choice of valve flow coefficient" annotation(Documentation(info = "<html>

<p>
Enumeration to define the choice of valve flow coefficient
(to be selected via choices menu):
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>CvTypes.</b></th>
    <th><b>Meaning</b></th></tr>

<tr><td>Av</td>
    <td>Av (metric) flow coefficient</td></tr>

<tr><td>Kv</td>
    <td>Kv (metric) flow coefficient</td></tr>

<tr><td>Cv</td>
    <td>Cv (US) flow coefficient</td></tr>

<tr><td>OpPoint</td>
    <td>Av defined by operating point</td></tr>

</table>

<p>
The details of the coefficients are explained in the
<a href=\"Modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.ValveCharacteristics\">
   Users Guide </a>.
</p>

</html>"));
      type PortFlowDirection = enumeration(Entering "Fluid flow is only entering", Leaving  "Fluid flow is only leaving", Bidirectional  "No restrictions on fluid flow (flow reversal possible)") "Enumeration to define whether flow reversal is allowed" annotation(Documentation(info = "<html>

<p>
Enumeration to define the assumptions on the model for the
direction of fluid flow at a port (to be selected via choices menu):
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>PortFlowDirection.</b></th>
    <th><b>Meaning</b></th></tr>

<tr><td>Entering</td>
    <td>Fluid flow is only entering the port from the outside</td></tr>

<tr><td>Leaving</td>
    <td>Fluid flow is only leaving the port to the outside</td></tr>

<tr><td>Bidirectional</td>
    <td>No restrictions on fluid flow (flow reversal possible)</td></tr>
</table>

<p>
The default is \"PortFlowDirection.Bidirectional\". If you are completely sure that
the flow is only in one direction, then the other settings may
make the simulation of your model faster.
</p>

</html>"));
      type ModelStructure = enumeration(av_vb "av_vb: port_a - volume - flow model - volume - port_b", a_v_b "a_v_b: port_a - flow model - volume - flow model - port_b", av_b  "av_b: port_a - volume - flow model - port_b", a_vb  "a_vb: port_a - flow model - volume - port_b") "Enumeration with choices for model structure in distributed pipe model" annotation(Documentation(info = "<html>

<p>
Enumeration to define the discretization structure of
distributed pipe models according to the staggered grid scheme:
</p>

<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>ModelStructure.</b></th>
    <th><b>Meaning</b></th></tr>

<tr><td>av_vb</td>
    <td>port_a - volume - flow model - volume - port_b</td></tr>

<tr><td>a_v_b</td>
    <td>port_a - flow model - volume - flow model - port_b</td></tr>

<tr><td>av_b</td>
    <td>port_a - volume - flow model - port_b</td></tr>

<tr><td>a_vb</td>
    <td>port_a - flow model - volume - port_b</td></tr>
</table>

<p>
The default is \"ModelStructure.av_vb\", i.e., the distributed pipe
has \"volumes\" at its both ends. The advantage is that connections
of the pipe to flow models (like fittings) lead to the desirable structure
of alternating volume and flow models, which means that no non-linear
algebraic equations occur.
</p>

<p>
Direct connections of distributed pipes with
this option means that two volumes are directly connected together. Due to the
stream concept this means that the pressures of the two connected volumes
are identical, but the temperatures are not set equal
(this corresponds to volumes that are connected together with a very
short distance and it needs some time until different volume temperatures
are equilibrated). Since the pressures of the volumes are identical, the number
of states is reduced and index reduction takes place (which means that medium
equations depending on pressure are differentiated and the number of required
initial conditions is reduced by one).
</p>

<p>
The default option \"av_vb\" cannot be used, if the dynamic pipe is connected to a model with non-differentiable pressure, like a Sources.Boundary_pT with prescribed jumping pressure. The modelStructure can be configured as appropriate in such situations, in order to place a momentum balance between a pressure state of the pipe and a non-differentiable boundary condition
(e.g. if the jumping pressure component is connected to port_a, use model structure
ModelStructure.a_vb).
</p>


</html>"));
    end Types;
    model System "System properties and default values (ambient, flow direction, initialization)"
      package Medium = Modelica.Media.Interfaces.PartialMedium "Medium model for default start values" annotation(choicesAllMatching = true);
      parameter Modelica.SIunits.AbsolutePressure p_ambient = 101325 "Default ambient pressure" annotation(Dialog(group = "Environment"));
      parameter Modelica.SIunits.Temperature T_ambient = 293.15 "Default ambient temperature" annotation(Dialog(group = "Environment"));
      parameter Modelica.SIunits.Acceleration g = Modelica.Constants.g_n "Constant gravity acceleration" annotation(Dialog(group = "Environment"));
      parameter Boolean allowFlowReversal = true "= false to restrict to design flow direction (port_a -> port_b)" annotation(Dialog(tab = "Assumptions"), Evaluate = true);
      parameter Modelica.Fluid.Types.Dynamics energyDynamics = Modelica.Fluid.Types.Dynamics.DynamicFreeInitial "Default formulation of energy balances" annotation(Evaluate = true, Dialog(tab = "Assumptions", group = "Dynamics"));
      parameter Modelica.Fluid.Types.Dynamics massDynamics = energyDynamics "Default formulation of mass balances" annotation(Evaluate = true, Dialog(tab = "Assumptions", group = "Dynamics"));
      final parameter Modelica.Fluid.Types.Dynamics substanceDynamics = massDynamics "Default formulation of substance balances" annotation(Evaluate = true, Dialog(tab = "Assumptions", group = "Dynamics"));
      final parameter Modelica.Fluid.Types.Dynamics traceDynamics = massDynamics "Default formulation of trace substance balances" annotation(Evaluate = true, Dialog(tab = "Assumptions", group = "Dynamics"));
      parameter Modelica.Fluid.Types.Dynamics momentumDynamics = Modelica.Fluid.Types.Dynamics.SteadyState "Default formulation of momentum balances, if options available" annotation(Evaluate = true, Dialog(tab = "Assumptions", group = "Dynamics"));
      parameter Modelica.SIunits.MassFlowRate m_flow_start = 0 "Default start value for mass flow rates" annotation(Dialog(tab = "Initialization"));
      parameter Modelica.SIunits.AbsolutePressure p_start = p_ambient "Default start value for pressures" annotation(Dialog(tab = "Initialization"));
      parameter Modelica.SIunits.Temperature T_start = T_ambient "Default start value for temperatures" annotation(Dialog(tab = "Initialization"));
      parameter Modelica.SIunits.MassFlowRate m_flow_small(min = 0) = 0.01 "Default small laminar mass flow rate for regularization of zero flow" annotation(Dialog(tab = "Advanced"));
      parameter Modelica.SIunits.AbsolutePressure dp_small(min = 0) = 1 "Default small pressure drop for regularization of laminar and zero flow" annotation(Dialog(tab = "Advanced"));
      annotation(defaultComponentName = "system", defaultComponentPrefixes = "inner", missingInnerMessage = "
Your model is using an outer \"system\" component but
an inner \"system\" component is not defined.
For simulation drag Modelica.Fluid.System into your model
to specify system properties. The default System setting
is used for the current simulation.
", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,150},{150,110}}, lineColor = {0,0,255}, textString = "%name"),Line(points = {{ -86, -30},{82, -30}}, color = {0,0,0}),Line(points = {{ -82, -68},{ -52, -30}}, color = {0,0,0}),Line(points = {{ -48, -68},{ -18, -30}}, color = {0,0,0}),Line(points = {{ -14, -68},{16, -30}}, color = {0,0,0}),Line(points = {{22, -68},{52, -30}}, color = {0,0,0}),Line(points = {{74,84},{74,14}}, color = {0,0,0}),Polygon(points = {{60,14},{88,14},{74, -18},{60,14}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{16,20},{60, -18}}, lineColor = {0,0,0}, textString = "g"),Text(extent = {{ -90,82},{74,50}}, lineColor = {0,0,0}, textString = "defaults"),Line(points = {{ -82,14},{ -42, -20},{2,30}}, color = {0,0,0}, thickness = 0.5),Ellipse(extent = {{ -10,40},{12,18}}, pattern = LinePattern.None, lineColor = {0,0,0}, fillColor = {255,0,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
 A system component is needed in each fluid model to provide system-wide settings, such as ambient conditions and overall modeling assumptions.
 The system settings are propagated to the fluid models using the inner/outer mechanism.
</p>
<p>
 A model should never directly use system parameters.
 Instead a local parameter should be declared, which uses the global setting as default.
 The only exception currently made is the gravity system.g.
</p>
</html>"));
    end System;
    package Sources "Define fixed or prescribed boundary conditions"
      extends Modelica.Fluid.Icons.VariantLibrary;
      model FixedBoundary "Boundary source component"
        extends Sources.BaseClasses.PartialSource;
        parameter Boolean use_p = true "select p or d" annotation(Evaluate = true, Dialog(group = "Boundary pressure or Boundary density"));
        parameter Medium.AbsolutePressure p = Medium.p_default "Boundary pressure" annotation(Dialog(group = "Boundary pressure or Boundary density", enable = use_p));
        parameter Medium.Density d = Medium.density_pTX(Medium.p_default, Medium.T_default, Medium.X_default) "Boundary density" annotation(Dialog(group = "Boundary pressure or Boundary density", enable = not use_p));
        parameter Boolean use_T = true "select T or h" annotation(Evaluate = true, Dialog(group = "Boundary temperature or Boundary specific enthalpy"));
        parameter Medium.Temperature T = Medium.T_default "Boundary temperature" annotation(Dialog(group = "Boundary temperature or Boundary specific enthalpy", enable = use_T));
        parameter Medium.SpecificEnthalpy h = Medium.h_default "Boundary specific enthalpy" annotation(Dialog(group = "Boundary temperature or Boundary specific enthalpy", enable = not use_T));
        parameter Medium.MassFraction X[Medium.nX](quantity = Medium.substanceNames) = Medium.X_default "Boundary mass fractions m_i/m" annotation(Dialog(group = "Only for multi-substance flow", enable = Medium.nXi > 0));
        parameter Medium.ExtraProperty C[Medium.nC](quantity = Medium.extraPropertiesNames) = fill(0, Medium.nC) "Boundary trace substances" annotation(Dialog(group = "Only for trace-substance flow", enable = Medium.nC > 0));
        annotation(defaultComponentName = "boundary", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Ellipse(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {0,127,255}),Text(extent = {{ -150,110},{150,150}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<html>
<p>
Model <b>FixedBoundary</b> defines constant values for boundary conditions:
</p>
<ul>
<li> Boundary pressure or boundary density.</li>
<li> Boundary temperature or boundary specific enthalpy.</li>
<li> Boundary composition (only for multi-substance or trace-substance flow).</li>
</ul>
<p>
Note, that boundary temperature, density, specific enthalpy,
mass fractions and trace substances have only an effect if the mass flow
is from the Boundary into the port. If mass is flowing from
the port into the boundary, the boundary definitions,
with exception of boundary pressure, do not have an effect.
</p>
</html>"));

      equation 
        Modelica.Fluid.Utilities.checkBoundary(Medium.mediumName, Medium.substanceNames, Medium.singleState, use_p, X, "FixedBoundary");
        if use_p or Medium.singleState then
          medium.p = p;

        else
          medium.d = d;

        end if;
        if use_T then
          medium.T = T;

        else
          medium.h = h;

        end if;
        medium.Xi = X[1:Medium.nXi];
        ports.C_outflow = fill(C, nPorts);
      end FixedBoundary;
      annotation(Documentation(info = "<html>
<p>
Package <b>Sources</b> contains generic sources for fluid connectors
to define fixed or prescribed ambient conditions.
</p>
</html>"));
      model Boundary_pT "Boundary with prescribed pressure, temperature, composition and trace substances"
        extends Sources.BaseClasses.PartialSource;
        parameter Boolean use_p_in = false "Get the pressure from the input connector" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
        parameter Boolean use_T_in = false "Get the temperature from the input connector" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
        parameter Boolean use_X_in = false "Get the composition from the input connector" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
        parameter Boolean use_C_in = false "Get the trace substances from the input connector" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
        parameter Medium.AbsolutePressure p = Medium.p_default "Fixed value of pressure" annotation(Evaluate = true, Dialog(enable = not use_p_in));
        parameter Medium.Temperature T = Medium.T_default "Fixed value of temperature" annotation(Evaluate = true, Dialog(enable = not use_T_in));
        parameter Medium.MassFraction X[Medium.nX] = Medium.X_default "Fixed value of composition" annotation(Evaluate = true, Dialog(enable = not use_X_in and Medium.nXi > 0));
        parameter Medium.ExtraProperty C[Medium.nC](quantity = Medium.extraPropertiesNames) = fill(0, Medium.nC) "Fixed values of trace substances" annotation(Evaluate = true, Dialog(enable = not use_C_in and Medium.nC > 0));
        Modelica.Blocks.Interfaces.RealInput p_in if use_p_in "Prescribed boundary pressure" annotation(Placement(transformation(extent = {{ -140,60},{ -100,100}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput T_in if use_T_in "Prescribed boundary temperature" annotation(Placement(transformation(extent = {{ -140,20},{ -100,60}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput X_in[Medium.nX] if use_X_in "Prescribed boundary composition" annotation(Placement(transformation(extent = {{ -140, -60},{ -100, -20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput C_in[Medium.nC] if use_C_in "Prescribed boundary trace substances" annotation(Placement(transformation(extent = {{ -140, -100},{ -100, -60}}, rotation = 0)));
      protected 
        Modelica.Blocks.Interfaces.RealInput p_in_internal "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput T_in_internal "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput X_in_internal[Medium.nX] "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput C_in_internal[Medium.nC] "Needed to connect to conditional connector";
        annotation(defaultComponentName = "boundary", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Ellipse(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {0,127,255}),Text(extent = {{ -150,110},{150,150}}, textString = "%name", lineColor = {0,0,255}),Line(visible = use_p_in, points = {{ -100,80},{ -58,80}}, color = {0,0,255}),Line(visible = use_T_in, points = {{ -100,40},{ -92,40}}, color = {0,0,255}),Line(visible = use_X_in, points = {{ -100, -40},{ -92, -40}}, color = {0,0,255}),Line(visible = use_C_in, points = {{ -100, -80},{ -60, -80}}, color = {0,0,255}),Text(visible = use_p_in, extent = {{ -152,134},{ -68,94}}, lineColor = {0,0,0}, textString = "p"),Text(visible = use_X_in, extent = {{ -164,4},{ -62, -36}}, lineColor = {0,0,0}, textString = "X"),Text(visible = use_C_in, extent = {{ -164, -90},{ -62, -130}}, lineColor = {0,0,0}, textString = "C"),Text(visible = use_T_in, extent = {{ -162,34},{ -60, -6}}, lineColor = {0,0,0}, textString = "T")}), Documentation(info = "<html>
<p>
Defines prescribed values for boundary conditions:
</p>
<ul>
<li> Prescribed boundary pressure.</li>
<li> Prescribed boundary temperature.</li>
<li> Boundary composition (only for multi-substance or trace-substance flow).</li>
</ul>
<p>If <tt>use_p_in</tt> is false (default option), the <tt>p</tt> parameter
is used as boundary pressure, and the <tt>p_in</tt> input connector is disabled; if <tt>use_p_in</tt> is true, then the <tt>p</tt> parameter is ignored, and the value provided by the input connector is used instead.</p>
<p>The same thing goes for the temperature, composition and trace substances.</p>
<p>
Note, that boundary temperature,
mass fractions and trace substances have only an effect if the mass flow
is from the boundary into the port. If mass is flowing from
the port into the boundary, the boundary definitions,
with exception of boundary pressure, do not have an effect.
</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));

      equation 
        Modelica.Fluid.Utilities.checkBoundary(Medium.mediumName, Medium.substanceNames, Medium.singleState, true, X_in_internal, "Boundary_pT");
        connect(p_in,p_in_internal);
        connect(T_in,T_in_internal);
        connect(X_in,X_in_internal);
        connect(C_in,C_in_internal);
if not use_p_in then
                  p_in_internal = p;
        end if;
if not use_T_in then
                  T_in_internal = T;
        end if;
if not use_X_in then
                  X_in_internal = X;
        end if;
if not use_C_in then
                  C_in_internal = C;
        end if;
        medium.p = p_in_internal;
        medium.T = T_in_internal;
        medium.Xi = X_in_internal[1:Medium.nXi];
        ports.C_outflow = fill(C_in_internal, nPorts);
      end Boundary_pT;
      model Boundary_ph "Boundary with prescribed pressure, specific enthalpy, composition and trace substances"
        extends Sources.BaseClasses.PartialSource;
        parameter Boolean use_p_in = false "Get the pressure from the input connector" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
        parameter Boolean use_h_in = false "Get the specific enthalpy from the input connector" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
        parameter Boolean use_X_in = false "Get the composition from the input connector" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
        parameter Boolean use_C_in = false "Get the trace substances from the input connector" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
        parameter Medium.AbsolutePressure p = Medium.p_default "Fixed value of pressure" annotation(Evaluate = true, Dialog(enable = not use_p_in));
        parameter Medium.SpecificEnthalpy h = Medium.h_default "Fixed value of specific enthalpy" annotation(Evaluate = true, Dialog(enable = not use_h_in));
        parameter Medium.MassFraction X[Medium.nX] = Medium.X_default "Fixed value of composition" annotation(Evaluate = true, Dialog(enable = not use_X_in and Medium.nXi > 0));
        parameter Medium.ExtraProperty C[Medium.nC](quantity = Medium.extraPropertiesNames) = fill(0, Medium.nC) "Fixed values of trace substances" annotation(Evaluate = true, Dialog(enable = not use_C_in and Medium.nC > 0));
        Modelica.Blocks.Interfaces.RealInput p_in if use_p_in "Prescribed boundary pressure" annotation(Placement(transformation(extent = {{ -140,60},{ -100,100}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput h_in if use_h_in "Prescribed boundary specific enthalpy" annotation(Placement(transformation(extent = {{ -140,20},{ -100,60}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput X_in[Medium.nX] if use_X_in "Prescribed boundary composition" annotation(Placement(transformation(extent = {{ -140, -60},{ -100, -20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput C_in[Medium.nC] if use_C_in "Prescribed boundary trace substances" annotation(Placement(transformation(extent = {{ -140, -100},{ -100, -60}}, rotation = 0)));
        annotation(defaultComponentName = "boundary", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Ellipse(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {0,127,255}),Text(extent = {{ -150,110},{150,150}}, textString = "%name", lineColor = {0,0,255}),Line(visible = use_p_in, points = {{ -100,80},{ -60,80}}, color = {0,0,255}),Line(visible = use_h_in, points = {{ -100,40},{ -92,40}}, color = {0,0,255}),Line(visible = use_X_in, points = {{ -100, -40},{ -92, -40}}, color = {0,0,255}),Line(visible = use_C_in, points = {{ -100, -80},{ -60, -80}}, color = {0,0,255}),Text(visible = use_p_in, extent = {{ -150,134},{ -72,94}}, lineColor = {0,0,0}, textString = "p"),Text(visible = use_h_in, extent = {{ -166,34},{ -64, -6}}, lineColor = {0,0,0}, textString = "h"),Text(visible = use_X_in, extent = {{ -164,4},{ -62, -36}}, lineColor = {0,0,0}, textString = "X"),Text(visible = use_C_in, extent = {{ -164, -90},{ -62, -130}}, lineColor = {0,0,0}, textString = "C")}), Documentation(info = "<html>
<p>
Defines prescribed values for boundary conditions:
</p>
<ul>
<li> Prescribed boundary pressure.</li>
<li> Prescribed boundary temperature.</li>
<li> Boundary composition (only for multi-substance or trace-substance flow).</li>
</ul>
<p>If <tt>use_p_in</tt> is false (default option), the <tt>p</tt> parameter
is used as boundary pressure, and the <tt>p_in</tt> input connector is disabled; if <tt>use_p_in</tt> is true, then the <tt>p</tt> parameter is ignored, and the value provided by the input connector is used instead.</p>
<p>The same thing goes for the specific enthalpy and composition</p>
<p>
Note, that boundary temperature,
mass fractions and trace substances have only an effect if the mass flow
is from the boundary into the port. If mass is flowing from
the port into the boundary, the boundary definitions,
with exception of boundary pressure, do not have an effect.
</p>
</html>"));
      protected 
        Modelica.Blocks.Interfaces.RealInput p_in_internal "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput h_in_internal "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput X_in_internal[Medium.nX] "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput C_in_internal[Medium.nC] "Needed to connect to conditional connector";

      equation 
        Modelica.Fluid.Utilities.checkBoundary(Medium.mediumName, Medium.substanceNames, Medium.singleState, true, X_in_internal, "Boundary_ph");
        connect(p_in,p_in_internal);
        connect(h_in,h_in_internal);
        connect(X_in,X_in_internal);
        connect(C_in,C_in_internal);
if not use_p_in then
                  p_in_internal = p;
        end if;
if not use_h_in then
                  h_in_internal = h;
        end if;
if not use_X_in then
                  X_in_internal = X;
        end if;
if not use_C_in then
                  C_in_internal = C;
        end if;
        medium.p = p_in_internal;
        medium.h = h_in_internal;
        medium.Xi = X_in_internal[1:Medium.nXi];
        ports.C_outflow = fill(C_in_internal, nPorts);
      end Boundary_ph;
      model MassFlowSource_T "Ideal flow source that produces a prescribed mass flow with prescribed temperature, mass fraction and trace substances"
        extends Sources.BaseClasses.PartialSource;
        parameter Boolean use_m_flow_in = false "Get the mass flow rate from the input connector" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
        parameter Boolean use_T_in = false "Get the temperature from the input connector" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
        parameter Boolean use_X_in = false "Get the composition from the input connector" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
        parameter Boolean use_C_in = false "Get the trace substances from the input connector" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
        parameter Medium.MassFlowRate m_flow = 0 "Fixed mass flow rate going out of the fluid port" annotation(Evaluate = true, Dialog(enable = not use_m_flow_in));
        parameter Medium.Temperature T = Medium.T_default "Fixed value of temperature" annotation(Evaluate = true, Dialog(enable = not use_T_in));
        parameter Medium.MassFraction X[Medium.nX] = Medium.X_default "Fixed value of composition" annotation(Evaluate = true, Dialog(enable = not use_X_in and Medium.nXi > 0));
        parameter Medium.ExtraProperty C[Medium.nC](quantity = Medium.extraPropertiesNames) = fill(0, Medium.nC) "Fixed values of trace substances" annotation(Evaluate = true, Dialog(enable = not use_C_in and Medium.nC > 0));
        Modelica.Blocks.Interfaces.RealInput m_flow_in if use_m_flow_in "Prescribed mass flow rate" annotation(Placement(transformation(extent = {{ -120,60},{ -80,100}}, rotation = 0), iconTransformation(extent = {{ -120,60},{ -80,100}})));
        Modelica.Blocks.Interfaces.RealInput T_in if use_T_in "Prescribed fluid temperature" annotation(Placement(transformation(extent = {{ -140,20},{ -100,60}}, rotation = 0), iconTransformation(extent = {{ -140,20},{ -100,60}})));
        Modelica.Blocks.Interfaces.RealInput X_in[Medium.nX] if use_X_in "Prescribed fluid composition" annotation(Placement(transformation(extent = {{ -140, -60},{ -100, -20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput C_in[Medium.nC] if use_C_in "Prescribed boundary trace substances" annotation(Placement(transformation(extent = {{ -120, -100},{ -80, -60}}, rotation = 0)));
      protected 
        Modelica.Blocks.Interfaces.RealInput m_flow_in_internal "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput T_in_internal "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput X_in_internal[Medium.nX] "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput C_in_internal[Medium.nC] "Needed to connect to conditional connector";
        annotation(defaultComponentName = "boundary", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{35,45},{100, -45}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,127,255}),Ellipse(extent = {{ -100,80},{60, -80}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -60,70},{60,0},{ -60, -68},{ -60,70}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -54,32},{16, -30}}, lineColor = {255,0,0}, textString = "m"),Text(extent = {{ -150,130},{150,170}}, textString = "%name", lineColor = {0,0,255}),Ellipse(extent = {{ -26,30},{ -18,22}}, lineColor = {255,0,0}, fillColor = {255,0,0}, fillPattern = FillPattern.Solid),Text(visible = use_m_flow_in, extent = {{ -185,132},{ -45,100}}, lineColor = {0,0,0}, textString = "m_flow"),Text(visible = use_T_in, extent = {{ -111,71},{ -71,37}}, lineColor = {0,0,0}, textString = "T"),Text(visible = use_X_in, extent = {{ -153, -44},{ -33, -72}}, lineColor = {0,0,0}, textString = "X"),Text(visible = use_C_in, extent = {{ -155, -98},{ -35, -126}}, lineColor = {0,0,0}, textString = "C")}), Window(x = 0.45, y = 0.01, width = 0.44, height = 0.65), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Documentation(info = "<html>
<p>
Models an ideal flow source, with prescribed values of flow rate, temperature, composition and trace substances:
</p>
<ul>
<li> Prescribed mass flow rate.</li>
<li> Prescribed temperature.</li>
<li> Boundary composition (only for multi-substance or trace-substance flow).</li>
</ul>
<p>If <tt>use_m_flow_in</tt> is false (default option), the <tt>m_flow</tt> parameter
is used as boundary pressure, and the <tt>m_flow_in</tt> input connector is disabled; if <tt>use_m_flow_in</tt> is true, then the <tt>m_flow</tt> parameter is ignored, and the value provided by the input connector is used instead.</p>
<p>The same thing goes for the temperature and composition</p>
<p>
Note, that boundary temperature,
mass fractions and trace substances have only an effect if the mass flow
is from the boundary into the port. If mass is flowing from
the port into the boundary, the boundary definitions,
with exception of boundary flow rate, do not have an effect.
</p>
</html>"));

      equation 
        Utilities.checkBoundary(Medium.mediumName, Medium.substanceNames, Medium.singleState, true, X_in_internal, "MassFlowSource_T");
        connect(m_flow_in,m_flow_in_internal);
        connect(T_in,T_in_internal);
        connect(X_in,X_in_internal);
        connect(C_in,C_in_internal);
if not use_m_flow_in then
                  m_flow_in_internal = m_flow;
        end if;
if not use_T_in then
                  T_in_internal = T;
        end if;
if not use_X_in then
                  X_in_internal = X;
        end if;
if not use_C_in then
                  C_in_internal = C;
        end if;
        sum(ports.m_flow) =  -m_flow_in_internal;
        medium.T = T_in_internal;
        medium.Xi = X_in_internal[1:Medium.nXi];
        ports.C_outflow = fill(C_in_internal, nPorts);
      end MassFlowSource_T;
      model MassFlowSource_h "Ideal flow source that produces a prescribed mass flow with prescribed specific enthalpy, mass fraction and trace substances"
        extends Sources.BaseClasses.PartialSource;
        parameter Boolean use_m_flow_in = false "Get the mass flow rate from the input connector" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
        parameter Boolean use_h_in = false "Get the specific enthalpy from the input connector" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
        parameter Boolean use_X_in = false "Get the composition from the input connector" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
        parameter Boolean use_C_in = false "Get the trace substances from the input connector" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
        parameter Medium.MassFlowRate m_flow = 0 "Fixed mass flow rate going out of the fluid port" annotation(Evaluate = true, Dialog(enable = not use_m_flow_in));
        parameter Medium.SpecificEnthalpy h = Medium.h_default "Fixed value of specific enthalpy" annotation(Evaluate = true, Dialog(enable = not use_h_in));
        parameter Medium.MassFraction X[Medium.nX] = Medium.X_default "Fixed value of composition" annotation(Evaluate = true, Dialog(enable = not use_X_in and Medium.nXi > 0));
        parameter Medium.ExtraProperty C[Medium.nC](quantity = Medium.extraPropertiesNames) = fill(0, Medium.nC) "Fixed values of trace substances" annotation(Evaluate = true, Dialog(enable = not use_C_in and Medium.nC > 0));
        Modelica.Blocks.Interfaces.RealInput m_flow_in if use_m_flow_in "Prescribed mass flow rate" annotation(Placement(transformation(extent = {{ -120,60},{ -80,100}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput h_in if use_h_in "Prescribed fluid specific enthalpy" annotation(Placement(transformation(extent = {{ -140,20},{ -100,60}}, rotation = 0), iconTransformation(extent = {{ -140,20},{ -100,60}})));
        Modelica.Blocks.Interfaces.RealInput X_in[Medium.nX] if use_X_in "Prescribed fluid composition" annotation(Placement(transformation(extent = {{ -140, -60},{ -100, -20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput C_in[Medium.nC] if use_C_in "Prescribed boundary trace substances" annotation(Placement(transformation(extent = {{ -120, -100},{ -80, -60}}, rotation = 0), iconTransformation(extent = {{ -120, -100},{ -80, -60}})));
      protected 
        Modelica.Blocks.Interfaces.RealInput m_flow_in_internal "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput h_in_internal "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput X_in_internal[Medium.nX] "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput C_in_internal[Medium.nC] "Needed to connect to conditional connector";
        annotation(defaultComponentName = "boundary", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{36,45},{100, -45}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,127,255}),Ellipse(extent = {{ -100,80},{60, -80}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -60,70},{60,0},{ -60, -68},{ -60,70}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -54,32},{16, -30}}, lineColor = {255,0,0}, textString = "m"),Ellipse(extent = {{ -26,30},{ -18,22}}, lineColor = {255,0,0}, fillColor = {255,0,0}, fillPattern = FillPattern.Solid),Text(visible = use_m_flow_in, extent = {{ -185,132},{ -45,100}}, lineColor = {0,0,0}, textString = "m_flow"),Text(visible = use_h_in, extent = {{ -113,72},{ -73,38}}, lineColor = {0,0,0}, textString = "h"),Text(visible = use_X_in, extent = {{ -153, -44},{ -33, -72}}, lineColor = {0,0,0}, textString = "X"),Text(visible = use_X_in, extent = {{ -155, -98},{ -35, -126}}, lineColor = {0,0,0}, textString = "C"),Text(extent = {{ -150,110},{150,150}}, textString = "%name", lineColor = {0,0,255})}), Window(x = 0.45, y = 0.01, width = 0.44, height = 0.65), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Documentation(info = "<html>
<p>
Models an ideal flow source, with prescribed values of flow rate, temperature and composition:
</p>
<ul>
<li> Prescribed mass flow rate.</li>
<li> Prescribed specific enthalpy.</li>
<li> Boundary composition (only for multi-substance or trace-substance flow).</li>
</ul>
<p>If <tt>use_m_flow_in</tt> is false (default option), the <tt>m_flow</tt> parameter
is used as boundary pressure, and the <tt>m_flow_in</tt> input connector is disabled; if <tt>use_m_flow_in</tt> is true, then the <tt>m_flow</tt> parameter is ignored, and the value provided by the input connector is used instead.</p>
<p>The same thing goes for the temperature and composition</p>
<p>
Note, that boundary temperature,
mass fractions and trace substances have only an effect if the mass flow
is from the boundary into the port. If mass is flowing from
the port into the boundary, the boundary definitions,
with exception of boundary flow rate, do not have an effect.
</p>
</html>"));

      equation 
        Utilities.checkBoundary(Medium.mediumName, Medium.substanceNames, Medium.singleState, true, X_in_internal, "MassFlowSource_h");
        connect(m_flow_in,m_flow_in_internal);
        connect(h_in,h_in_internal);
        connect(X_in,X_in_internal);
        connect(C_in,C_in_internal);
if not use_m_flow_in then
                  m_flow_in_internal = m_flow;
        end if;
if not use_h_in then
                  h_in_internal = h;
        end if;
if not use_X_in then
                  X_in_internal = X;
        end if;
if not use_C_in then
                  C_in_internal = C;
        end if;
        sum(ports.m_flow) =  -m_flow_in_internal;
        medium.h = h_in_internal;
        medium.Xi = X_in_internal[1:Medium.nXi];
        ports.C_outflow = fill(C_in_internal, nPorts);
      end MassFlowSource_h;
      package BaseClasses "Base classes used in the Sources package (only of interest to build new component models)"
        extends Modelica.Fluid.Icons.BaseClassLibrary;
        partial model PartialSource "Partial component source with one fluid connector"
          import Modelica.Constants;
          parameter Integer nPorts = 0 "Number of ports" annotation(Dialog(__Dymola_connectorSizing = true));
          replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium model within the source" annotation(choicesAllMatching = true);
          Medium.BaseProperties medium "Medium in the source";
          Interfaces.FluidPorts_b ports[nPorts](redeclare each package Medium = Medium, m_flow(each max = if flowDirection == Types.PortFlowDirection.Leaving then 0 else  +Constants.inf, each min = if flowDirection == Types.PortFlowDirection.Entering then 0 else  -Constants.inf)) annotation(Placement(transformation(extent = {{90,40},{110, -40}})));
        protected 
          parameter Types.PortFlowDirection flowDirection = Types.PortFlowDirection.Bidirectional "Allowed flow direction" annotation(Evaluate = true, Dialog(tab = "Advanced"));

        equation 
          for i in 1:nPorts loop
          assert(cardinality(ports[i]) <= 1, "
each ports[i] of boundary shall at most be connected to one component.
If two or more connections are present, ideal mixing takes
place with these connections, which is usually not the intention
of the modeller. Increase nPorts to add an additional port.
");
          ports[i].p = medium.p;
          ports[i].h_outflow = medium.h;
          ports[i].Xi_outflow = medium.Xi;

          end for;
          annotation(defaultComponentName = "boundary", Documentation(info = "<html>
<p>
Partial component to model the <b>volume interface</b> of a <b>source</b>
component, such as a mass flow source. The essential
features are:
</p>
<ul>
<li> The pressure in the connection port (= ports.p) is identical to the
     pressure in the volume.</li>
<li> The outflow enthalpy rate (= port.h_outflow) and the composition of the
     substances (= port.Xi_outflow) are identical to the respective values in the volume.</li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
        end PartialSource;
      end BaseClasses;
    end Sources;
    package Sensors "Ideal sensor components to extract signals from a fluid connector"
      extends Modelica.Fluid.Icons.VariantLibrary;
      annotation(preferedView = "info", Documentation(info = "<html>
<p align = justify>
Package <b>Sensors</b> consists of idealized sensor components that
provide variables of a medium model and/or fluid ports as
output signals. These signals can be, e.g., further processed
with components of the Modelica.Blocks library.
Also more realistic sensor models can be built, by further
processing (e.g., by attaching block Modelica.Blocks.FirstOrder to
model the time constant of the sensor).

</p>

<p align = justify>For the thermodynamic state variables temperature, specific entalpy, specific entropy and density
the fluid library provides two different types of sensors: <b>regular one port</b> and <b>two port</b> sensors. </p>

<ul>
<li>
The <b>regular one port</b> sensors have the advantage of easy introduction and removal from a model, as no connections have to be broken.
A potential drawback is that the obtained value jumps as flow reverts.
</li>

<li> The <b>two port</b> sensors offer the advantages of an adjustable regularized step function around zero flow.
Moreover the obtained result is restricted to the value flowing into port_a if allowFlowReversal is false.</li>
</ul>

<p>
<a href=\"Modelica://Modelica.Fluid.Examples.Explanatory.MeasuringTemperature\">Modelica.Fluid.Examples.Explanatory.MeasuringTemperature</a>
demonstrates the differences between one- and two-port sensor at hand of a
simple example.
</p>

</html>
", revisions = "<html>
<ul>
<li><i>22 Dec 2008</i>
    by R;uumldiger Franke<br>
    <ul>
    <li>flow sensors based on Interfaces.PartialTwoPort</li>
    <li>adapted docu to stream connectors, i.e. less need for two port sensors</li>
    </ul>
<li><i>4 Dec 2008</i>
    by Michael Wetter<br>
       included sensors for trace substance</li>
<li><i>31 Oct 2007</i>
    by Carsten Heinrich<br>
       updated sensor models, included one and two port sensors for thermodynamic state variables</li>
</ul>
</html>"));
      model Pressure "Ideal pressure sensor"
        extends Sensors.BaseClasses.PartialAbsoluteSensor;
        extends Modelica.Icons.RotationalSensor;
        Modelica.Blocks.Interfaces.RealOutput p(final quantity = "Pressure", final unit = "Pa", displayUnit = "bar", min = 0) "Pressure at port" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{70,0},{100,0}}, color = {0,0,127}),Line(points = {{0, -70},{0, -100}}, color = {0,127,255}),Text(extent = {{ -150,80},{150,120}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{151, -20},{57, -50}}, lineColor = {0,0,0}, textString = "p")}), Documentation(info = "<HTML>
<p>
This component monitors the absolute pressure at its fluid port. The sensor is
ideal, i.e., it does not influence the fluid.
</p>
</HTML>
"));

      equation 
        p = port.p;
      end Pressure;
      model Density "Ideal one port density sensor"
        extends Sensors.BaseClasses.PartialAbsoluteSensor;
        extends Modelica.Icons.RotationalSensor;
        Modelica.Blocks.Interfaces.RealOutput d(final quantity = "Density", final unit = "kg/m3", displayUnit = "g/cm3", min = 0) "Density in port medium" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(defaultComponentName = "density", Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{0, -70},{0, -100}}, color = {0,0,127}),Text(extent = {{ -150,80},{150,120}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{154, -31},{56, -61}}, lineColor = {0,0,0}, textString = "d"),Line(points = {{70,0},{100,0}}, color = {0,0,127})}), Documentation(info = "<HTML>
<p>
This component monitors the density of the fluid passing its port.
The sensor is ideal, i.e. it does not influence the fluid.
</p>
<p>If using the one port sensor please read the <a href = Modelica.Fluid.Sensors>Information</a>  first.</p>

</HTML>
"));

      equation 
        d = Medium.density(Medium.setState_phX(port.p, inStream(port.h_outflow), inStream(port.Xi_outflow)));
      end Density;
      model DensityTwoPort "Ideal two port density sensor"
        extends Sensors.BaseClasses.PartialFlowSensor;
        extends Modelica.Icons.RotationalSensor;
        Modelica.Blocks.Interfaces.RealOutput d(final quantity = "Density", final unit = "kg/m3", displayUnit = "g/cm3", min = 0) "Density of the passing fluid" annotation(Placement(transformation(origin = {0,110}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
        parameter Medium.MassFlowRate m_flow_small(min = 0) = system.m_flow_small "For bi-directional flow, density is regularized in the region |m_flow| < m_flow_small (m_flow_small > 0 required)" annotation(Dialog(tab = "Advanced"));
        annotation(defaultComponentName = "density", Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{102,124},{6,95}}, lineColor = {0,0,0}, textString = "d"),Line(points = {{0,100},{0,70}}, color = {0,0,127}),Line(points = {{ -100,0},{ -70,0}}, color = {0,128,255}),Line(points = {{70,0},{100,0}}, color = {0,128,255})}), Documentation(info = "<HTML>
<p>
This component monitors the density of the fluid flowing from port_a to port_b.
The sensor is ideal, i.e. it does not influence the fluid.
</p>
</HTML>
"));
      protected 
        Medium.Density rho_a_inflow "Density of inflowing fluid at port_a";
        Medium.Density rho_b_inflow "Density of inflowing fluid at port_b or rho_a_inflow, if uni-directional flow";

      equation 
        if allowFlowReversal then
          rho_a_inflow = Medium.density(Medium.setState_phX(port_b.p, port_b.h_outflow, port_b.Xi_outflow));
          rho_b_inflow = Medium.density(Medium.setState_phX(port_a.p, port_a.h_outflow, port_a.Xi_outflow));
          d = Modelica.Fluid.Utilities.regStep(port_a.m_flow, rho_a_inflow, rho_b_inflow, m_flow_small);

        else
          d = Medium.density(Medium.setState_phX(port_b.p, port_b.h_outflow, port_b.Xi_outflow));
          rho_a_inflow = d;
          rho_b_inflow = d;

        end if;
      end DensityTwoPort;
      model Temperature "Ideal one port temperature sensor"
        extends Sensors.BaseClasses.PartialAbsoluteSensor;
        Modelica.Blocks.Interfaces.RealOutput T(final quantity = "ThermodynamicTemperature", final unit = "K", displayUnit = "degC", min = 0) "Temperature in port medium" annotation(Placement(transformation(extent = {{60, -10},{80,10}}, rotation = 0)));
        annotation(defaultComponentName = "temperature", Documentation(info = "<HTML>
<p>
This component monitors the temperature of the fluid passing its port.
The sensor is ideal, i.e. it does not influence the fluid.
</p>
</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{0, -70},{0, -100}}, color = {0,0,127}),Ellipse(extent = {{ -20, -98},{20, -60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -12,40},{12, -68}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{ -12,40},{ -12,80},{ -10,86},{ -6,88},{0,90},{6,88},{10,86},{12,80},{12,40},{ -12,40}}, lineColor = {0,0,0}, lineThickness = 0.5),Line(points = {{ -12,40},{ -12, -64}}, color = {0,0,0}, thickness = 0.5),Line(points = {{12,40},{12, -64}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -40, -20},{ -12, -20}}, color = {0,0,0}),Line(points = {{ -40,20},{ -12,20}}, color = {0,0,0}),Line(points = {{ -40,60},{ -12,60}}, color = {0,0,0}),Line(points = {{12,0},{60,0}}, color = {0,0,127})}), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -20, -88},{20, -50}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -12,50},{12, -58}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{ -12,50},{ -12,90},{ -10,96},{ -6,98},{0,100},{6,98},{10,96},{12,90},{12,50},{ -12,50}}, lineColor = {0,0,0}, lineThickness = 0.5),Line(points = {{ -12,50},{ -12, -54}}, color = {0,0,0}, thickness = 0.5),Line(points = {{12,50},{12, -54}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -40, -10},{ -12, -10}}, color = {0,0,0}),Line(points = {{ -40,30},{ -12,30}}, color = {0,0,0}),Line(points = {{ -40,70},{ -12,70}}, color = {0,0,0}),Text(extent = {{126, -30},{6, -60}}, lineColor = {0,0,0}, textString = "T"),Text(extent = {{ -150,110},{150,150}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{12,0},{60,0}}, color = {0,0,127})}));

      equation 
        T = Medium.temperature(Medium.setState_phX(port.p, inStream(port.h_outflow), inStream(port.Xi_outflow)));
      end Temperature;
      model TemperatureTwoPort "Ideal two port temperature sensor"
        extends Sensors.BaseClasses.PartialFlowSensor;
        Modelica.Blocks.Interfaces.RealOutput T(final quantity = "ThermodynamicTemperature", final unit = "K", min = 0, displayUnit = "degC") "Temperature of the passing fluid" annotation(Placement(transformation(origin = {0,110}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
        parameter Medium.MassFlowRate m_flow_small(min = 0) = system.m_flow_small "For bi-directional flow, temperature is regularized in the region |m_flow| < m_flow_small (m_flow_small > 0 required)" annotation(Dialog(tab = "Advanced"));
        annotation(defaultComponentName = "temperature", Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{0,100},{0,50}}, color = {0,0,127}),Line(points = {{ -92,0},{100,0}}, color = {0,128,255}),Ellipse(extent = {{ -20, -68},{20, -30}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -12,50},{12, -34}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{ -12,50},{ -12,70},{ -10,76},{ -6,78},{0,80},{6,78},{10,76},{12,70},{12,50},{ -12,50}}, lineColor = {0,0,0}, lineThickness = 0.5),Line(points = {{ -12,50},{ -12, -35}}, color = {0,0,0}, thickness = 0.5),Line(points = {{12,50},{12, -34}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -40, -10},{ -12, -10}}, color = {0,0,0}),Line(points = {{ -40,20},{ -12,20}}, color = {0,0,0}),Line(points = {{ -40,50},{ -12,50}}, color = {0,0,0}),Text(extent = {{94,122},{0,92}}, lineColor = {0,0,0}, textString = "T")}), Documentation(info = "<HTML>
<p>
This component monitors the temperature of the passing fluid.
The sensor is ideal, i.e. it does not influence the fluid.
</p>
</HTML>
"));
      protected 
        Medium.Temperature T_a_inflow "Temperature of inflowing fluid at port_a";
        Medium.Temperature T_b_inflow "Temperature of inflowing fluid at port_b or T_a_inflow, if uni-directional flow";

      equation 
        if allowFlowReversal then
          T_a_inflow = Medium.temperature(Medium.setState_phX(port_b.p, port_b.h_outflow, port_b.Xi_outflow));
          T_b_inflow = Medium.temperature(Medium.setState_phX(port_a.p, port_a.h_outflow, port_a.Xi_outflow));
          T = Modelica.Fluid.Utilities.regStep(port_a.m_flow, T_a_inflow, T_b_inflow, m_flow_small);

        else
          T = Medium.temperature(Medium.setState_phX(port_b.p, port_b.h_outflow, port_b.Xi_outflow));
          T_a_inflow = T;
          T_b_inflow = T;

        end if;
      end TemperatureTwoPort;
      model SpecificEnthalpy "Ideal one port specific enthalpy sensor"
        extends Sensors.BaseClasses.PartialAbsoluteSensor;
        extends Modelica.Icons.RotationalSensor;
        Modelica.Blocks.Interfaces.RealOutput h_out(final quantity = "SpecificEnergy", final unit = "J/kg") "Specific enthalpy in port medium" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(defaultComponentName = "specificEnthalpy", Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{0, -70},{0, -100}}, color = {0,0,127}),Text(extent = {{ -150,80},{150,120}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{168, -30},{52, -60}}, lineColor = {0,0,0}, textString = "h"),Line(points = {{70,0},{100,0}}, color = {0,0,127})}), Documentation(info = "<HTML>
<p>
This component monitors the specific enthalpy of the fluid passing its port.
The sensor is ideal, i.e. it does not influence the fluid.
</p>
</HTML>
"));

      equation 
        h_out = inStream(port.h_outflow);
      end SpecificEnthalpy;
      model SpecificEnthalpyTwoPort "Ideal two port sensor for the specific enthalpy"
        extends Sensors.BaseClasses.PartialFlowSensor;
        extends Modelica.Icons.RotationalSensor;
        Modelica.Blocks.Interfaces.RealOutput h_out(final quantity = "SpecificEnergy", final unit = "J/kg") "Specific enthalpy of the passing fluid" annotation(Placement(transformation(origin = {0,110}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
        parameter Medium.MassFlowRate m_flow_small(min = 0) = system.m_flow_small "For bi-directional flow, specific enthalpy is regularized in the region |m_flow| < m_flow_small (m_flow_small > 0 required)" annotation(Dialog(tab = "Advanced"));
        annotation(defaultComponentName = "specificEnthalpy", Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{102,120},{0,90}}, lineColor = {0,0,0}, textString = "h"),Line(points = {{0,100},{0,70}}, color = {0,0,127}),Line(points = {{ -100,0},{ -70,0}}, color = {0,128,255}),Line(points = {{70,0},{100,0}}, color = {0,128,255})}), Documentation(info = "<HTML>
<p>
This component monitors the specific enthalpy of a passing fluid.
The sensor is ideal, i.e. it does not influence the fluid.
</p>
</HTML>
"));

      equation 
        if allowFlowReversal then
          h_out = Modelica.Fluid.Utilities.regStep(port_a.m_flow, port_b.h_outflow, port_a.h_outflow, m_flow_small);

        else
          h_out = port_b.h_outflow;

        end if;
      end SpecificEnthalpyTwoPort;
      model SpecificEntropy "Ideal one port specific entropy sensor"
        extends Sensors.BaseClasses.PartialAbsoluteSensor;
        extends Modelica.Icons.RotationalSensor;
        Modelica.Blocks.Interfaces.RealOutput s(final quantity = "SpecificEntropy", final unit = "J/(kg.K)") "Specific entropy in port medium" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(defaultComponentName = "specificEntropy", Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{0, -70},{0, -100}}, color = {0,0,127}),Text(extent = {{ -150,80},{150,120}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{156, -24},{54, -54}}, lineColor = {0,0,0}, textString = "s"),Line(points = {{70,0},{100,0}}, color = {0,0,127})}), Documentation(info = "<HTML>
<p>
This component monitors the specific entropy of the fluid passing its port.
The sensor is ideal, i.e. it does not influence the fluid.
</p>
</HTML>
"));

      equation 
        s = Medium.specificEntropy(Medium.setState_phX(port.p, inStream(port.h_outflow), inStream(port.Xi_outflow)));
      end SpecificEntropy;
      model SpecificEntropyTwoPort "Ideal two port sensor for the specific entropy"
        extends Sensors.BaseClasses.PartialFlowSensor;
        extends Modelica.Icons.RotationalSensor;
        Modelica.Blocks.Interfaces.RealOutput s(final quantity = "SpecificEntropy", final unit = "J/(kg.K)") "Specific entropy of the passing fluid" annotation(Placement(transformation(origin = {0,110}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
        parameter Medium.MassFlowRate m_flow_small(min = 0) = system.m_flow_small "For bi-directional flow, specific entropy is regularized in the region |m_flow| < m_flow_small (m_flow_small > 0 required)" annotation(Dialog(tab = "Advanced"));
        annotation(defaultComponentName = "specificEntropy", Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{120,120},{0,90}}, lineColor = {0,0,0}, textString = "s"),Line(points = {{0,100},{0,70}}, color = {0,0,127}),Line(points = {{ -100,0},{ -70,0}}, color = {0,128,255}),Line(points = {{70,0},{100,0}}, color = {0,128,255})}), Documentation(info = "<HTML>
<p>
This component monitors the specific entropy of the passing fluid.
The sensor is ideal, i.e. it does not influence the fluid.
</p>
</HTML>
"));
      protected 
        Medium.SpecificEntropy s_a_inflow "Specific entropy of inflowing fluid at port_a";
        Medium.SpecificEntropy s_b_inflow "Specific entropy of inflowing fluid at port_b or s_a_inflow, if uni-directional flow";

      equation 
        if allowFlowReversal then
          s_a_inflow = Medium.specificEntropy(Medium.setState_phX(port_b.p, port_b.h_outflow, port_b.Xi_outflow));
          s_b_inflow = Medium.specificEntropy(Medium.setState_phX(port_a.p, port_a.h_outflow, port_a.Xi_outflow));
          s = Modelica.Fluid.Utilities.regStep(port_a.m_flow, s_a_inflow, s_b_inflow, m_flow_small);

        else
          s = Medium.specificEntropy(Medium.setState_phX(port_b.p, port_b.h_outflow, port_b.Xi_outflow));
          s_a_inflow = s;
          s_b_inflow = s;

        end if;
      end SpecificEntropyTwoPort;
      model TraceSubstances "Ideal one port trace substances sensor"
        extends Sensors.BaseClasses.PartialAbsoluteSensor;
        extends Modelica.Icons.RotationalSensor;
        parameter String substanceName = "CO2" "Name of trace substance";
        Modelica.Blocks.Interfaces.RealOutput C "Trace substance in port medium" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(defaultComponentName = "traceSubstance", Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{0, -70},{0, -100}}, color = {0,0,127}),Text(extent = {{ -150,80},{150,120}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{160, -30},{60, -60}}, lineColor = {0,0,0}, textString = "C"),Line(points = {{70,0},{100,0}}, color = {0,0,127})}), Documentation(info = "<HTML>
<p>
This component monitors the trace substances contained in the fluid passing its port.
The sensor is ideal, i.e. it does not influence the fluid.
</p>
</HTML>
"));
      protected 
        parameter Integer ind(fixed = false) "Index of species in vector of auxiliary substances";
        Medium.ExtraProperty CVec[Medium.nC](quantity = Medium.extraPropertiesNames) "Trace substances vector, needed because indexed argument for the operator inStream is not supported";
      initial algorithm 
        ind:= -1;
        for i in 1:Medium.nC loop
                  if Modelica.Utilities.Strings.isEqual(Medium.extraPropertiesNames[i], substanceName) then 
                    ind:=i;

          else 
          end if;

        end for;
        assert(ind > 0, "Trace substance '" + substanceName + "' is not present in medium '" + Medium.mediumName + "'.\n" + "Check sensor parameter and medium model.");

      equation 
        CVec = inStream(port.C_outflow);
        C = CVec[ind];
      end TraceSubstances;
      model TraceSubstancesTwoPort "Ideal two port sensor for trace substance"
        extends Sensors.BaseClasses.PartialFlowSensor;
        extends Modelica.Icons.RotationalSensor;
        Modelica.Blocks.Interfaces.RealOutput C "Trace substance of the passing fluid" annotation(Placement(transformation(origin = {0,110}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
        parameter String substanceName = "CO2" "Name of trace substance";
        parameter Medium.MassFlowRate m_flow_small(min = 0) = system.m_flow_small "For bi-directional flow, trace substance is regularized in the region |m_flow| < m_flow_small (m_flow_small > 0 required)" annotation(Dialog(tab = "Advanced"));
        annotation(defaultComponentName = "traceSubstance", Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{82,122},{0,92}}, lineColor = {0,0,0}, textString = "C"),Line(points = {{0,100},{0,70}}, color = {0,0,127}),Line(points = {{ -100,0},{ -70,0}}, color = {0,128,255}),Line(points = {{70,0},{100,0}}, color = {0,128,255})}), Documentation(info = "<HTML>
<p>
This component monitors the trace substance of the passing fluid.
The sensor is ideal, i.e. it does not influence the fluid.
</p>
</HTML>
"));
      protected 
        parameter Integer ind(fixed = false) "Index of species in vector of auxiliary substances";
      initial algorithm 
        ind:= -1;
        for i in 1:Medium.nC loop
                  if Modelica.Utilities.Strings.isEqual(Medium.extraPropertiesNames[i], substanceName) then 
                    ind:=i;

          else 
          end if;

        end for;
        assert(ind > 0, "Trace substance '" + substanceName + "' is not present in medium '" + Medium.mediumName + "'.\n" + "Check sensor parameter and medium model.");

      equation 
        if allowFlowReversal then
          C = Modelica.Fluid.Utilities.regStep(port_a.m_flow, port_b.C_outflow[ind], port_a.C_outflow[ind], m_flow_small);

        else
          C = port_b.C_outflow[ind];

        end if;
      end TraceSubstancesTwoPort;
      model MassFlowRate "Ideal sensor for mass flow rate"
        extends Sensors.BaseClasses.PartialFlowSensor;
        extends Modelica.Icons.RotationalSensor;
        Modelica.Blocks.Interfaces.RealOutput m_flow(quantity = "MassFlowRate", final unit = "kg/s") "Mass flow rate from port_a to port_b" annotation(Placement(transformation(origin = {0,110}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{70,0},{100,0}}, color = {0,128,255}),Text(extent = {{162,120},{2,90}}, lineColor = {0,0,0}, textString = "m_flow"),Line(points = {{0,100},{0,70}}, color = {0,0,127}),Line(points = {{ -100,0},{ -70,0}}, color = {0,128,255})}), Documentation(info = "<HTML>
<p>
This component monitors the mass flow rate flowing from port_a to port_b.
The sensor is ideal, i.e., it does not influence the fluid.
</p>
</HTML>
"));

      equation 
        m_flow = port_a.m_flow;
      end MassFlowRate;
      model VolumeFlowRate "Ideal sensor for volume flow rate"
        extends Sensors.BaseClasses.PartialFlowSensor;
        extends Modelica.Icons.RotationalSensor;
        Modelica.Blocks.Interfaces.RealOutput V_flow(final quantity = "VolumeFlowRate", final unit = "m3/s") "Volume flow rate from port_a to port_b" annotation(Placement(transformation(origin = {0,110}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
        parameter Medium.MassFlowRate m_flow_small(min = 0) = system.m_flow_small "For bi-directional flow, density is regularized in the region |m_flow| < m_flow_small (m_flow_small > 0 required)" annotation(Dialog(tab = "Advanced"));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{160,120},{0,90}}, lineColor = {0,0,0}, textString = "V_flow"),Line(points = {{0,100},{0,70}}, color = {0,0,127}),Line(points = {{ -100,0},{ -70,0}}, color = {0,128,255}),Line(points = {{70,0},{100,0}}, color = {0,128,255})}), Documentation(info = "<HTML>
<p>
This component monitors the volume flow rate flowing from port_a to port_b.
The sensor is ideal, i.e. it does not influence the fluid.
</p>
</HTML>
"));
      protected 
        Medium.Density rho_a_inflow "Density of inflowing fluid at port_a";
        Medium.Density rho_b_inflow "Density of inflowing fluid at port_b or rho_a_inflow, if uni-directional flow";
        Medium.Density d "Density of the passing fluid";

      equation 
        if allowFlowReversal then
          rho_a_inflow = Medium.density(Medium.setState_phX(port_b.p, port_b.h_outflow, port_b.Xi_outflow));
          rho_b_inflow = Medium.density(Medium.setState_phX(port_a.p, port_a.h_outflow, port_a.Xi_outflow));
          d = Modelica.Fluid.Utilities.regStep(port_a.m_flow, rho_a_inflow, rho_b_inflow, m_flow_small);

        else
          d = Medium.density(Medium.setState_phX(port_b.p, port_b.h_outflow, port_b.Xi_outflow));
          rho_a_inflow = d;
          rho_b_inflow = d;

        end if;
        V_flow = port_a.m_flow / d;
      end VolumeFlowRate;
      model RelativePressure "Ideal relative pressure sensor"
        extends Modelica.Icons.TranslationalSensor;
        replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium in the sensor" annotation(choicesAllMatching = true);
        Modelica.Fluid.Interfaces.FluidPort_a port_a(m_flow(min = 0), redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
        Modelica.Fluid.Interfaces.FluidPort_b port_b(m_flow(min = 0), redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{110, -12},{90,8}}, rotation = 0), iconTransformation(extent = {{110, -10},{90,10}})));
        Modelica.Blocks.Interfaces.RealOutput p_rel(final quantity = "Pressure", final unit = "Pa", displayUnit = "bar") "Relative pressure signal" annotation(Placement(transformation(origin = {0, -90}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
        annotation(Icon(graphics = {Line(points = {{ -100,0},{ -70,0}}, color = {0,127,255}),Line(points = {{70,0},{100,0}}, color = {0,127,255}),Line(points = {{0, -30},{0, -80}}, color = {0,0,127}),Text(extent = {{ -150,40},{150,80}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{130, -70},{4, -100}}, lineColor = {0,0,0}, textString = "p_rel"),Line(points = {{32,3},{ -58,3}}, color = {0,128,255}, smooth = Smooth.None),Polygon(points = {{22,18},{62,3},{22, -12},{22,18}}, lineColor = {0,128,255}, smooth = Smooth.None, fillColor = {0,128,255}, fillPattern = FillPattern.Solid)}), Diagram(graphics), Documentation(info = "<HTML>
<p>
The relative pressure \"port_a.p - port_b.p\" is determined between
the two ports of this component and is provided as output signal. The
sensor should be connected in parallel with other equipment, no flow
through the sensor is allowed.
</p>
</HTML>
"));

      equation 
        port_a.m_flow = 0;
        port_b.m_flow = 0;
        port_a.h_outflow = 0;
        port_b.h_outflow = 0;
        port_a.Xi_outflow = zeros(Medium.nXi);
        port_b.Xi_outflow = zeros(Medium.nXi);
        port_a.C_outflow = zeros(Medium.nC);
        port_b.C_outflow = zeros(Medium.nC);
        p_rel = port_a.p - port_b.p;
      end RelativePressure;
      model RelativeTemperature "Ideal relative temperature sensor"
        extends Modelica.Icons.TranslationalSensor;
        replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium in the sensor" annotation(choicesAllMatching = true);
        Modelica.Fluid.Interfaces.FluidPort_a port_a(m_flow(min = 0), redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
        Modelica.Fluid.Interfaces.FluidPort_b port_b(m_flow(min = 0), redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{110, -10},{90,10}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput T_rel(final quantity = "ThermodynamicTemperature", final unit = "K", displayUnit = "degC", min = 0) "Relative temperature signal" annotation(Placement(transformation(origin = {0, -90}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
        annotation(Icon(graphics = {Line(points = {{ -100,0},{ -70,0}}, color = {0,127,255}),Line(points = {{70,0},{100,0}}, color = {0,127,255}),Line(points = {{0, -30},{0, -80}}, color = {0,0,127}),Text(extent = {{ -150,40},{150,80}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{128, -70},{10, -100}}, lineColor = {0,0,0}, textString = "T_rel"),Line(points = {{34,3},{ -56,3}}, color = {0,128,255}, smooth = Smooth.None),Polygon(points = {{24,18},{64,3},{24, -12},{24,18}}, lineColor = {0,128,255}, smooth = Smooth.None, fillColor = {0,128,255}, fillPattern = FillPattern.Solid)}), Diagram(graphics), Documentation(info = "<HTML>
<p>
The relative temperature \"T(port_a) - T(port_b)\" is determined between
the two ports of this component and is provided as output signal. The
sensor should be connected in parallel with other equipment, no flow
through the sensor is allowed.
</p>
</HTML>
"));

      equation 
        port_a.m_flow = 0;
        port_b.m_flow = 0;
        port_a.h_outflow = 0;
        port_b.h_outflow = 0;
        port_a.Xi_outflow = zeros(Medium.nXi);
        port_b.Xi_outflow = zeros(Medium.nXi);
        port_a.C_outflow = zeros(Medium.nC);
        port_b.C_outflow = zeros(Medium.nC);
        T_rel = Medium.temperature(Medium.setState_phX(port_a.p, inStream(port_a.h_outflow), inStream(port_a.Xi_outflow))) - Medium.temperature(Medium.setState_phX(port_b.p, inStream(port_b.h_outflow), inStream(port_b.Xi_outflow)));
      end RelativeTemperature;
      package BaseClasses "Base classes used in the Sensors package (only of interest to build new component models)"
        extends Modelica.Fluid.Icons.BaseClassLibrary;
        partial model PartialAbsoluteSensor "Partial component to model a sensor that measures a potential variable"
          replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium in the sensor" annotation(choicesAllMatching = true);
          Modelica.Fluid.Interfaces.FluidPort_a port(redeclare package Medium = Medium, m_flow(min = 0)) annotation(Placement(transformation(origin = {0, -100}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          annotation(Documentation(info = "<html>
<p>
Partial component to model an <b>absolute sensor</b>. Can be used for pressure sensor models.
Use for other properties such as temperature or density is discouraged, because the enthalpy at the connector can have different meanings, depending on the connection topology. Use <tt>PartialFlowSensor</tt> instead.
as signal.
</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          port.m_flow = 0;
          port.h_outflow = 0;
          port.Xi_outflow = zeros(Medium.nXi);
          port.C_outflow = zeros(Medium.nC);
        end PartialAbsoluteSensor;
        partial model PartialFlowSensor "Partial component to model sensors that measure flow properties"
          extends Modelica.Fluid.Interfaces.PartialTwoPort;
          annotation(Documentation(info = "<html>
<p>
Partial component to model a <b>sensor</b> that measures any intensive properties
of a flow, e.g., to get temperature or density in the flow
between fluid connectors.<br>
The model includes zero-volume balance equations. Sensor models inheriting from
this partial class should add a medium instance to calculate the measured property.
</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));

        equation 
          0 = port_a.m_flow + port_b.m_flow;
          port_a.p = port_b.p;
          port_a.h_outflow = inStream(port_b.h_outflow);
          port_b.h_outflow = inStream(port_a.h_outflow);
          port_a.Xi_outflow = inStream(port_b.Xi_outflow);
          port_b.Xi_outflow = inStream(port_a.Xi_outflow);
          port_a.C_outflow = inStream(port_b.C_outflow);
          port_b.C_outflow = inStream(port_a.C_outflow);
        end PartialFlowSensor;
      end BaseClasses;
    end Sensors;
    package Pipes "Devices for conveying fluid"
      extends Modelica.Fluid.Icons.VariantLibrary;
      model StaticPipe "Basic pipe flow model without storage of mass or energy"
        extends Modelica.Fluid.Pipes.BaseClasses.PartialStraightPipe;
        parameter Medium.AbsolutePressure p_a_start = system.p_start "Start value of pressure at port a" annotation(Dialog(tab = "Initialization"));
        parameter Medium.AbsolutePressure p_b_start = p_a_start "Start value of pressure at port b" annotation(Dialog(tab = "Initialization"));
        parameter Medium.MassFlowRate m_flow_start = system.m_flow_start "Start value for mass flow rate" annotation(Evaluate = true, Dialog(tab = "Initialization"));
        FlowModel flowModel(redeclare final package Medium = Medium, final n = 2, states = {Medium.setState_phX(port_a.p, inStream(port_a.h_outflow), inStream(port_a.Xi_outflow)),Medium.setState_phX(port_b.p, inStream(port_b.h_outflow), inStream(port_b.Xi_outflow))}, vs = {port_a.m_flow / Medium.density(flowModel.states[1]) / flowModel.crossAreas[1], -port_b.m_flow / Medium.density(flowModel.states[2]) / flowModel.crossAreas[2]} / nParallel, final momentumDynamics = Types.Dynamics.SteadyState, final allowFlowReversal = allowFlowReversal, final p_a_start = p_a_start, final p_b_start = p_b_start, final m_flow_start = m_flow_start, final nParallel = nParallel, final pathLengths = {length}, final crossAreas = {crossArea,crossArea}, final dimensions = {(4 * crossArea) / perimeter,(4 * crossArea) / perimeter}, final roughnesses = {roughness,roughness}, final dheights = {height_ab}, final g = system.g) "Flow model" annotation(Placement(transformation(extent = {{ -38, -18},{38,18}}, rotation = 0)));

      equation 
        port_a.m_flow = flowModel.m_flows[1];
        0 = port_a.m_flow + port_b.m_flow;
        port_a.Xi_outflow = inStream(port_b.Xi_outflow);
        port_b.Xi_outflow = inStream(port_a.Xi_outflow);
        port_a.C_outflow = inStream(port_b.C_outflow);
        port_b.C_outflow = inStream(port_a.C_outflow);
        port_b.h_outflow = inStream(port_a.h_outflow) - system.g * height_ab;
        port_a.h_outflow = inStream(port_b.h_outflow) + system.g * height_ab;
        annotation(defaultComponentName = "pipe", Documentation(info = "<html>
<p>Model of a straight pipe with constant cross section and with steady-state mass, momentum and energy balances, i.e. the model does not store mass or energy.
There exist two thermodynamic states, one at each fluid port. The momentum balance is formulated for the two states, taking into account
momentum flows, friction and gravity. The same result can be obtained by using <a href=\"Modelica://Modelica.Fluid.Pipes.DynamicPipe\">DynamicPipe</a> with
steady-state dynamic settings. The intended use is to provide simple connections of vessels or other devices with storage, as it is done in:
<ul>
<li><a href=\"Modelica://Modelica.Fluid.Examples.Tanks.EmptyTanks\">Examples.Tanks.EmptyTanks</a></li>
<li><a href=\"Modelica://Modelica.Fluid.Examples.InverseParameterization\">Examples.InverseParameterization</a></li>.
</ul>

<h4>Numerical Issues</h4>
With the stream connectors the thermodynamic states on the ports are generally defined by models with storage or by sources placed upstream and downstream of the static pipe.
Other non storage components in the flow path may yield to state transformation. Note that this generally leads to nonlinear equation systems if multiple static pipes,
or other flow models without storage, are directly connected.
<br><br><br><br>

</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
      end StaticPipe;
      model DynamicPipe "Dynamic pipe model with storage of mass and energy"
        import Modelica.Fluid.Types.ModelStructure;
        extends Modelica.Fluid.Pipes.BaseClasses.PartialStraightPipe(final port_a_exposesState = modelStructure == ModelStructure.av_b or modelStructure == ModelStructure.av_vb, final port_b_exposesState = modelStructure == ModelStructure.a_vb or modelStructure == ModelStructure.av_vb);
        extends BaseClasses.PartialTwoPortFlow(final lengths = if n == 1 then {length} else if modelStructure == ModelStructure.av_vb then cat(1, {length / (n - 1) / 2}, fill(length / (n - 1), n - 2), {length / (n - 1) / 2}) else if modelStructure == ModelStructure.av_b then cat(1, {length / n / 2}, fill((length * (1 - 1 / n / 2)) / (n - 1), n - 1)) else if modelStructure == ModelStructure.a_vb then cat(1, fill((length * (1 - 1 / n / 2)) / (n - 1), n - 1), {length / n / 2}) else fill(length / n, n), final crossAreas = fill(crossArea, n), final dimensions = fill((4 * crossArea) / perimeter, n), final roughnesses = fill(roughness, n), final dheights = height_ab * dxs);
        parameter Boolean use_HeatTransfer = false "= true to use the HeatTransfer model" annotation(Dialog(tab = "Assumptions", group = "Heat transfer"));
        replaceable model HeatTransfer = Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.IdealFlowHeatTransfer extends Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.PartialFlowHeatTransfer "Wall heat transfer" annotation(Dialog(tab = "Assumptions", group = "Heat transfer", enable = use_HeatTransfer), choicesAllMatching = true);
        Interfaces.HeatPorts_a[nNodes] heatPorts if use_HeatTransfer annotation(Placement(transformation(extent = {{ -10,45},{10,65}}), iconTransformation(extent = {{ -30,36},{32,52}})));
        HeatTransfer heatTransfer(redeclare each final package Medium = Medium, final n = n, final nParallel = nParallel, final surfaceAreas = perimeter * lengths, final lengths = lengths, final dimensions = dimensions, final roughnesses = roughnesses, final states = mediums.state, final vs = vs, final use_k = use_HeatTransfer) "Heat transfer model" annotation(Placement(transformation(extent = {{ -36,19},{ -14,41}}, rotation = 0)));
        final parameter Real[n] dxs = lengths / sum(lengths);

      equation 
        Qb_flows = heatTransfer.Q_flows;
        if n == 1 or useLumpedPressure then
          Wb_flows = dxs * vs * dxs * crossAreas * dxs * (port_b.p - port_a.p + sum(flowModel.dps_fg) - system.g * dheights * mediums.d) * nParallel;

        else
          if modelStructure == ModelStructure.av_vb or modelStructure == ModelStructure.av_b then
            Wb_flows[2:n - 1] = array(vs[i] * crossAreas[i] * ((mediums[i + 1].p - mediums[i - 1].p) / 2 + (flowModel.dps_fg[i - 1] + flowModel.dps_fg[i]) / 2 - system.g * dheights[i] * mediums[i].d) for i in 2:n - 1) * nParallel;

          else
            Wb_flows[2:n - 1] = array(vs[i] * crossAreas[i] * ((mediums[i + 1].p - mediums[i - 1].p) / 2 + (flowModel.dps_fg[i] + flowModel.dps_fg[i + 1]) / 2 - system.g * dheights[i] * mediums[i].d) for i in 2:n - 1) * nParallel;

          end if;
          if modelStructure == ModelStructure.av_vb then
            Wb_flows[1] = vs[1] * crossAreas[1] * ((mediums[2].p - mediums[1].p) / 2 + flowModel.dps_fg[1] / 2 - system.g * dheights[1] * mediums[1].d) * nParallel;
            Wb_flows[n] = vs[n] * crossAreas[n] * ((mediums[n].p - mediums[n - 1].p) / 2 + flowModel.dps_fg[n - 1] / 2 - system.g * dheights[n] * mediums[n].d) * nParallel;

          elseif modelStructure == ModelStructure.av_b then
            Wb_flows[1] = vs[1] * crossAreas[1] * ((mediums[2].p - mediums[1].p) / 2 + flowModel.dps_fg[1] / 2 - system.g * dheights[1] * mediums[1].d) * nParallel;
            Wb_flows[n] = vs[n] * crossAreas[n] * ((port_b.p - mediums[n - 1].p) / 1.5 + flowModel.dps_fg[n - 1] / 2 + flowModel.dps_fg[n] - system.g * dheights[n] * mediums[n].d) * nParallel;


          elseif modelStructure == ModelStructure.a_vb then
            Wb_flows[1] = vs[1] * crossAreas[1] * ((mediums[2].p - port_a.p) / 1.5 + flowModel.dps_fg[1] + flowModel.dps_fg[2] / 2 - system.g * dheights[1] * mediums[1].d) * nParallel;
            Wb_flows[n] = vs[n] * crossAreas[n] * ((mediums[n].p - mediums[n - 1].p) / 2 + flowModel.dps_fg[n] / 2 - system.g * dheights[n] * mediums[n].d) * nParallel;


          elseif modelStructure == ModelStructure.a_v_b then
            Wb_flows[1] = vs[1] * crossAreas[1] * ((mediums[2].p - port_a.p) / 1.5 + flowModel.dps_fg[1] + flowModel.dps_fg[2] / 2 - system.g * dheights[1] * mediums[1].d) * nParallel;
            Wb_flows[n] = vs[n] * crossAreas[n] * ((port_b.p - mediums[n - 1].p) / 1.5 + flowModel.dps_fg[n] / 2 + flowModel.dps_fg[n + 1] - system.g * dheights[n] * mediums[n].d) * nParallel;


          else
            assert(true, "Unknown model structure");

          end if;

        end if;
        connect(heatPorts,heatTransfer.heatPorts) annotation(Line(points = {{0,55},{0,54},{ -24,54},{ -24,37.7},{ -25,37.7}}, color = {191,0,0}));
        annotation(defaultComponentName = "pipe", Documentation(info = "<html>
<p>Model of a straight pipe with distributed mass, energy and momentum balances.
It provides the complete balance equations for one-dimensional fluid flow as formulated in
<a href=\"Modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.BalanceEquations\">UsersGuide.ComponentDefinition.BalanceEquations</a>.
</p>
<p>
The partial differential equations are treated with the finite volume method and a staggered grid scheme for momentum balances.
The pipe is split into nNodes equally spaced segments along the flow path. The default value is nNodes=2.
This results in two lumped mass and energy balances and one lumped momentum balance across the dynamic pipe.
</p>
<p>
Note that this generally leads to high-index DAEs for pressure states if dynamic pipes are directly connected to each other,
or generally to models with storage exposing a thermodynamic state through the port. This may not be valid
if the dynamic pipe is connected to a model with non-differentiable pressure, like a Sources.Boundary_pT with prescribed jumping pressure.
The <b><tt>modelStructure</tt></b> can be configured as appropriate in such situations,
in order to place a momentum balance between a pressure state of the pipe and a non-differentiable boundary condition.
</p>
<p>
The default <b><tt>modelStructure</tt></b> is <tt>av_vb</tt> (see Advanced tab).
The simplest possible alternative symetric configuration, avoiding potential high-index DAEs at the cost of the potential introduction
of nonlinear equation systems, is obtained with the setting <tt>nNodes=1, modelStructure=a_v_b</tt>.
Depending on the configured model structure, the first and the last pipe segment,
or the flow path length of the first and the last momentum balance, are of half size.
See the documentation of the base class
<a href=\"Modelica://Modelica.Fluid.Pipes.BaseClasses.PartialTwoPortFlow\">Pipes.BaseClasses.PartialTwoPortFlow</a>,
also covering asymmetric configurations.
</p>
<p>
The <b><tt>HeatTransfer</tt></b> component specifies the source term <tt>Qb_flows</tt> of the energy balance.
The default component uses a constant coefficient for the heat transfer between the bulk flow and the segment boundaries exposed through the <tt>heatPorts</tt>.
The <tt>HeatTransfer</tt> model is replaceable and can be exchanged with any model extended from
<a href=\"Modelica://Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.PartialFlowHeatTransfer\">BaseClasses.HeatTransfer.PartialFlowHeatTransfer</a>.
</p>
<p>
The intended use is for complex networks of pipes and other flow devices, like valves. See e.g.
<ul>
<li><a href=\"Modelica://Modelica.Fluid.Examples.BranchingDynamicPipes\">Examples.BranchingDynamicPipes</a>, or </li>
<li><a href=\"Modelica://Modelica.Fluid.Examples.IncompressibleFluidNetwork\">Examples.IncompressibleFluidNetwork</a>.</li>
</ul>
</p>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,44},{100, -44}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,127,255}),Ellipse(extent = {{ -72,10},{ -52, -10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{50,10},{70, -10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,60},{100,50}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Backward),Rectangle(extent = {{ -100, -50},{100, -60}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Backward),Line(points = {{100,45},{100,50}}, arrow = {Arrow.None,Arrow.Filled}, color = {0,0,0}, pattern = LinePattern.Dot),Line(points = {{0,45},{0,50}}, arrow = {Arrow.None,Arrow.Filled}, color = {0,0,0}, pattern = LinePattern.Dot),Line(points = {{100, -45},{100, -50}}, arrow = {Arrow.None,Arrow.Filled}, color = {0,0,0}, pattern = LinePattern.Dot),Line(points = {{0, -45},{0, -50}}, arrow = {Arrow.None,Arrow.Filled}, color = {0,0,0}, pattern = LinePattern.Dot),Line(points = {{ -50,60},{ -50,50}}, smooth = Smooth.None, color = {0,0,0}, pattern = LinePattern.Dot),Line(points = {{50,60},{50,50}}, smooth = Smooth.None, color = {0,0,0}, pattern = LinePattern.Dot),Line(points = {{0, -50},{0, -60}}, smooth = Smooth.None, color = {0,0,0}, pattern = LinePattern.Dot)}));
      end DynamicPipe;
      package BaseClasses "Base classes used in the Pipes package (only of interest to build new component models)"
        extends Modelica.Fluid.Icons.BaseClassLibrary;
        partial model PartialStraightPipe "Base class for straight pipe models"
          extends Modelica.Fluid.Interfaces.PartialTwoPort;
          parameter Real nParallel(min = 1) = 1 "Number of identical parallel pipes" annotation(Dialog(group = "Geometry"));
          parameter SI.Length length "Length" annotation(Dialog(tab = "General", group = "Geometry"));
          parameter Boolean isCircular = true "= true if cross sectional area is circular" annotation(Evaluate, Dialog(tab = "General", group = "Geometry"));
          parameter SI.Diameter diameter "Diameter of circular pipe" annotation(Dialog(group = "Geometry", enable = isCircular));
          parameter SI.Area crossArea = (Modelica.Constants.pi * diameter * diameter) / 4 "Inner cross section area" annotation(Dialog(tab = "General", group = "Geometry", enable = not isCircular));
          parameter SI.Length perimeter = Modelica.Constants.pi * diameter "Inner perimeter" annotation(Dialog(tab = "General", group = "Geometry", enable = not isCircular));
          parameter SI.Height roughness = 2.5e-05 "Average height of surface asperities (default: smooth steel pipe)" annotation(Dialog(group = "Geometry"));
          final parameter SI.Volume V = crossArea * length * nParallel "volume size";
          parameter SI.Length height_ab = 0 "Height(port_b) - Height(port_a)" annotation(Dialog(group = "Static head"));
          replaceable model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow extends Modelica.Fluid.Pipes.BaseClasses.FlowModels.PartialStaggeredFlowModel "Wall friction, gravity, momentum flow" annotation(Dialog(group = "Pressure loss"), choicesAllMatching = true);

        equation 
          assert(length >= height_ab, "Parameter length must be greater or equal height_ab.");
          annotation(defaultComponentName = "pipe", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,40},{100, -40}}, fillPattern = FillPattern.Solid, fillColor = {95,95,95}, pattern = LinePattern.None),Rectangle(extent = {{ -100,44},{100, -44}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,127,255})}), Documentation(info = "<html>
<p>
Base class for one dimensional flow models. It specializes a PartialTwoPort with a parameter interface and icon graphics.
</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
        end PartialStraightPipe;
        partial model PartialTwoPortFlow "Base class for distributed flow models"
          import Modelica.Fluid.Types.ModelStructure;
          extends Modelica.Fluid.Interfaces.PartialTwoPort(final port_a_exposesState = modelStructure == ModelStructure.av_b or modelStructure == ModelStructure.av_vb, final port_b_exposesState = modelStructure == ModelStructure.a_vb or modelStructure == ModelStructure.av_vb);
          extends Modelica.Fluid.Interfaces.PartialDistributedVolume(final n = nNodes, final fluidVolumes = array(crossAreas[i] * lengths[i] for i in 1:n));
          parameter Real nParallel(min = 1) = 1 "Number of identical parallel flow devices" annotation(Dialog(group = "Geometry"));
          parameter SI.Length[n] lengths "lengths of flow segments" annotation(Dialog(group = "Geometry"));
          parameter SI.Area[n] crossAreas "cross flow areas of flow segments" annotation(Dialog(group = "Geometry"));
          parameter SI.Length[n] dimensions "hydraulic diameters of flow segments" annotation(Dialog(group = "Geometry"));
          parameter SI.Height[n] roughnesses "Average heights of surface asperities" annotation(Dialog(group = "Geometry"));
          parameter SI.Length[n] dheights = zeros(n) "Differences in heigths of flow segments" annotation(Dialog(group = "Static head"), Evaluate = true);
          parameter Types.Dynamics momentumDynamics = system.momentumDynamics "Formulation of momentum balances" annotation(Evaluate = true, Dialog(tab = "Assumptions", group = "Dynamics"));
          parameter Medium.MassFlowRate m_flow_start = system.m_flow_start "Start value for mass flow rate" annotation(Evaluate = true, Dialog(tab = "Initialization"));
          parameter Integer nNodes(min = 1) = 2 "Number of discrete flow volumes" annotation(Dialog(tab = "Advanced"), Evaluate = true);
          parameter Types.ModelStructure modelStructure = Types.ModelStructure.av_vb "Determines whether flow or volume models are present at the ports" annotation(Dialog(tab = "Advanced"), Evaluate = true);
          parameter Boolean useLumpedPressure = false "=true to lump pressure states together" annotation(Dialog(tab = "Advanced"), Evaluate = true);
          final parameter Integer nFM = if useLumpedPressure then nFMLumped else nFMDistributed "number of flow models in flowModel";
          final parameter Integer nFMDistributed = if modelStructure == Types.ModelStructure.a_v_b then n + 1 else if modelStructure == Types.ModelStructure.a_vb or modelStructure == Types.ModelStructure.av_b then n else n - 1;
          final parameter Integer nFMLumped = if modelStructure == Types.ModelStructure.a_v_b then 2 else 1;
          final parameter Integer iLumped = integer(n / 2) + 1 "Index of control volume with representative state if useLumpedPressure" annotation(Evaluate = true);
          parameter Boolean useInnerPortProperties = false "=true to take port properties for flow models from internal control volumes" annotation(Dialog(tab = "Advanced"), Evaluate = true);
          Medium.ThermodynamicState state_a "state defined by volume outside port_a";
          Medium.ThermodynamicState state_b "state defined by volume outside port_b";
          Medium.ThermodynamicState[nFM + 1] statesFM "state vector for flowModel model";
          replaceable model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow extends Modelica.Fluid.Pipes.BaseClasses.FlowModels.PartialStaggeredFlowModel "Wall friction, gravity, momentum flow" annotation(Dialog(group = "Pressure loss"), choicesAllMatching = true);
          FlowModel flowModel(redeclare final package Medium = Medium, final n = nFM + 1, final states = statesFM, final vs = vsFM, final momentumDynamics = momentumDynamics, final allowFlowReversal = allowFlowReversal, final p_a_start = p_a_start, final p_b_start = p_b_start, final m_flow_start = m_flow_start, final nParallel = nParallel, final pathLengths = pathLengths, final crossAreas = crossAreasFM, final dimensions = dimensionsFM, final roughnesses = roughnessesFM, final dheights = dheightsFM, final g = system.g) "Flow model" annotation(Placement(transformation(extent = {{ -77, -38},{75, -20}}, rotation = 0)));
          Medium.MassFlowRate[n + 1] m_flows(each min = if allowFlowReversal then  -Modelica.Constants.inf else 0, each start = m_flow_start) "Mass flow rates of fluid across segment boundaries";
          Medium.MassFlowRate[n + 1,Medium.nXi] mXi_flows "Independent mass flow rates across segment boundaries";
          Medium.MassFlowRate[n + 1,Medium.nC] mC_flows "Trace substance mass flow rates across segment boundaries";
          Medium.EnthalpyFlowRate[n + 1] H_flows "Enthalpy flow rates of fluid across segment boundaries";
          SI.Velocity[n] vs = array((0.5 * (m_flows[i] + m_flows[i + 1])) / mediums[i].d / crossAreas[i] for i in 1:n) / nParallel "mean velocities in flow segments";
        protected 
          SI.Length[nFM] pathLengths "Lengths along flow path";
          SI.Length[nFM] dheightsFM "Differences in heights between flow segments";
          SI.Area[nFM + 1] crossAreasFM "Cross flow areas of flow segments";
          SI.Velocity[nFM + 1] vsFM "Mean velocities in flow segments";
          SI.Length[nFM + 1] dimensionsFM "Hydraulic diameters of flow segments";
          SI.Height[nFM + 1] roughnessesFM "Average heights of surface asperities";

        equation 
          assert(nNodes > 1 or modelStructure <> ModelStructure.av_vb, "nNodes needs to be at least 2 for modelStructure av_vb, as flow model disappears otherwise!");
          if useLumpedPressure then
            if modelStructure <> ModelStructure.a_v_b then
              pathLengths[1] = sum(lengths);
              dheightsFM[1] = sum(dheights);
              if n == 1 then
                crossAreasFM[1:2] = {crossAreas[1],crossAreas[1]};
                dimensionsFM[1:2] = {dimensions[1],dimensions[1]};
                roughnessesFM[1:2] = {roughnesses[1],roughnesses[1]};

              else
                crossAreasFM[1:2] = {sum(crossAreas[1:iLumped - 1]) / (iLumped - 1),sum(crossAreas[iLumped:n]) / (n - iLumped + 1)};
                dimensionsFM[1:2] = {sum(dimensions[1:iLumped - 1]) / (iLumped - 1),sum(dimensions[iLumped:n]) / (n - iLumped + 1)};
                roughnessesFM[1:2] = {sum(roughnesses[1:iLumped - 1]) / (iLumped - 1),sum(roughnesses[iLumped:n]) / (n - iLumped + 1)};

              end if;

            else
              if n == 1 then
                pathLengths[1:2] = {lengths[1] / 2,lengths[1] / 2};
                dheightsFM[1:2] = {dheights[1] / 2,dheights[1] / 2};
                crossAreasFM[1:3] = {crossAreas[1],crossAreas[1],crossAreas[1]};
                dimensionsFM[1:3] = {dimensions[1],dimensions[1],dimensions[1]};
                roughnessesFM[1:3] = {roughnesses[1],roughnesses[1],roughnesses[1]};

              else
                pathLengths[1:2] = {sum(lengths[1:iLumped - 1]),sum(lengths[iLumped:n])};
                dheightsFM[1:2] = {sum(dheights[1:iLumped - 1]),sum(dheights[iLumped:n])};
                crossAreasFM[1:3] = {sum(crossAreas[1:iLumped - 1]) / (iLumped - 1),sum(crossAreas) / n,sum(crossAreas[iLumped:n]) / (n - iLumped + 1)};
                dimensionsFM[1:3] = {sum(dimensions[1:iLumped - 1]) / (iLumped - 1),sum(dimensions) / n,sum(dimensions[iLumped:n]) / (n - iLumped + 1)};
                roughnessesFM[1:3] = {sum(roughnesses[1:iLumped - 1]) / (iLumped - 1),sum(roughnesses) / n,sum(roughnesses[iLumped:n]) / (n - iLumped + 1)};

              end if;

            end if;

          else
            if modelStructure == ModelStructure.av_vb then
              if n == 2 then
                pathLengths[1] = lengths[1] + lengths[2];
                dheightsFM[1] = dheights[1] + dheights[2];

              else
                pathLengths[1:n - 1] = cat(1, {lengths[1] + 0.5 * lengths[2]}, 0.5 * (lengths[2:n - 2] + lengths[3:n - 1]), {0.5 * lengths[n - 1] + lengths[n]});
                dheightsFM[1:n - 1] = cat(1, {dheights[1] + 0.5 * dheights[2]}, 0.5 * (dheights[2:n - 2] + dheights[3:n - 1]), {0.5 * dheights[n - 1] + dheights[n]});

              end if;
              crossAreasFM[1:n] = crossAreas;
              dimensionsFM[1:n] = dimensions;
              roughnessesFM[1:n] = roughnesses;

            elseif modelStructure == ModelStructure.av_b then
              pathLengths[1:n] = lengths;
              dheightsFM[1:n] = dheights;
              crossAreasFM[1:n + 1] = cat(1, crossAreas[1:n], {crossAreas[n]});
              dimensionsFM[1:n + 1] = cat(1, dimensions[1:n], {dimensions[n]});
              roughnessesFM[1:n + 1] = cat(1, roughnesses[1:n], {roughnesses[n]});


            elseif modelStructure == ModelStructure.a_vb then
              pathLengths[1:n] = lengths;
              dheightsFM[1:n] = dheights;
              crossAreasFM[1:n + 1] = cat(1, {crossAreas[1]}, crossAreas[1:n]);
              dimensionsFM[1:n + 1] = cat(1, {dimensions[1]}, dimensions[1:n]);
              roughnessesFM[1:n + 1] = cat(1, {roughnesses[1]}, roughnesses[1:n]);


            elseif modelStructure == ModelStructure.a_v_b then
              pathLengths[1:n + 1] = cat(1, {0.5 * lengths[1]}, 0.5 * (lengths[1:n - 1] + lengths[2:n]), {0.5 * lengths[n]});
              dheightsFM[1:n + 1] = cat(1, {0.5 * dheights[1]}, 0.5 * (dheights[1:n - 1] + dheights[2:n]), {0.5 * dheights[n]});
              crossAreasFM[1:n + 2] = cat(1, {crossAreas[1]}, crossAreas[1:n], {crossAreas[n]});
              dimensionsFM[1:n + 2] = cat(1, {dimensions[1]}, dimensions[1:n], {dimensions[n]});
              roughnessesFM[1:n + 2] = cat(1, {roughnesses[1]}, roughnesses[1:n], {roughnesses[n]});


            else
              assert(true, "Unknown model structure");

            end if;

          end if;
          for i in 1:n loop
          mb_flows[i] = m_flows[i] - m_flows[i + 1];
          mbXi_flows[i,:] = mXi_flows[i,:] - mXi_flows[i + 1,:];
          mbC_flows[i,:] = mC_flows[i,:] - mC_flows[i + 1,:];
          Hb_flows[i] = H_flows[i] - H_flows[i + 1];

          end for;
          for i in 2:n loop
          H_flows[i] = semiLinear(m_flows[i], mediums[i - 1].h, mediums[i].h);
          mXi_flows[i,:] = semiLinear(m_flows[i], mediums[i - 1].Xi, mediums[i].Xi);
          mC_flows[i,:] = semiLinear(m_flows[i], Cs[i - 1,:], Cs[i,:]);

          end for;
          H_flows[1] = semiLinear(port_a.m_flow, inStream(port_a.h_outflow), mediums[1].h);
          H_flows[n + 1] =  -semiLinear(port_b.m_flow, inStream(port_b.h_outflow), mediums[n].h);
          mXi_flows[1,:] = semiLinear(port_a.m_flow, inStream(port_a.Xi_outflow), mediums[1].Xi);
          mXi_flows[n + 1,:] =  -semiLinear(port_b.m_flow, inStream(port_b.Xi_outflow), mediums[n].Xi);
          mC_flows[1,:] = semiLinear(port_a.m_flow, inStream(port_a.C_outflow), Cs[1,:]);
          mC_flows[n + 1,:] =  -semiLinear(port_b.m_flow, inStream(port_b.C_outflow), Cs[n,:]);
          port_a.m_flow = m_flows[1];
          port_b.m_flow =  -m_flows[n + 1];
          port_a.h_outflow = mediums[1].h;
          port_b.h_outflow = mediums[n].h;
          port_a.Xi_outflow = mediums[1].Xi;
          port_b.Xi_outflow = mediums[n].Xi;
          port_a.C_outflow = Cs[1,:];
          port_b.C_outflow = Cs[n,:];
          if useInnerPortProperties and n > 0 then
            state_a = Medium.setState_phX(port_a.p, mediums[1].h, mediums[1].Xi);
            state_b = Medium.setState_phX(port_b.p, mediums[n].h, mediums[n].Xi);

          else
            state_a = Medium.setState_phX(port_a.p, inStream(port_a.h_outflow), inStream(port_a.Xi_outflow));
            state_b = Medium.setState_phX(port_b.p, inStream(port_b.h_outflow), inStream(port_b.Xi_outflow));

          end if;
          if useLumpedPressure then
            if modelStructure <> ModelStructure.av_vb then
              fill(mediums[1].p, n - 1) = mediums[2:n].p;

            elseif n > 2 then
              fill(mediums[1].p, iLumped - 2) = mediums[2:iLumped - 1].p;
              fill(mediums[n].p, n - iLumped) = mediums[iLumped:n - 1].p;


            else

            end if;
            if modelStructure == ModelStructure.av_vb then
              port_a.p = mediums[1].p;
              statesFM[1] = mediums[1].state;
              m_flows[iLumped] = flowModel.m_flows[1];
              statesFM[2] = mediums[n].state;
              port_b.p = mediums[n].p;

            elseif modelStructure == ModelStructure.av_b then
              port_a.p = mediums[1].p;
              statesFM[1] = mediums[iLumped].state;
              statesFM[2] = state_b;
              m_flows[n + 1] = flowModel.m_flows[1];


            elseif modelStructure == ModelStructure.a_vb then
              m_flows[1] = flowModel.m_flows[1];
              statesFM[1] = state_a;
              statesFM[2] = mediums[iLumped].state;
              port_b.p = mediums[n].p;


            elseif modelStructure == ModelStructure.a_v_b then
              m_flows[1] = flowModel.m_flows[1];
              statesFM[1] = state_a;
              statesFM[2] = mediums[iLumped].state;
              statesFM[3] = state_b;
              m_flows[n + 1] = flowModel.m_flows[2];


            else
              assert(true, "Unknown model structure");

            end if;
            if modelStructure <> ModelStructure.a_v_b then
              vsFM[1] = (vs[1:iLumped - 1] * lengths[1:iLumped - 1]) / sum(lengths[1:iLumped - 1]);
              vsFM[2] = (vs[iLumped:n] * lengths[iLumped:n]) / sum(lengths[iLumped:n]);

            else
              vsFM[1] = (vs[1:iLumped - 1] * lengths[1:iLumped - 1]) / sum(lengths[1:iLumped - 1]);
              vsFM[2] = (vs[2:n - 1] * lengths[2:n - 1]) / sum(lengths[2:n - 1]);
              vsFM[3] = (vs[iLumped:n] * lengths[iLumped:n]) / sum(lengths[iLumped:n]);

            end if;

          else
            if modelStructure == ModelStructure.av_vb then
              statesFM[1:n] = mediums[1:n].state;
              m_flows[2:n] = flowModel.m_flows[1:n - 1];
              vsFM[1:n] = vs;
              port_a.p = mediums[1].p;
              port_b.p = mediums[n].p;

            elseif modelStructure == ModelStructure.av_b then
              statesFM[1:n] = mediums[1:n].state;
              statesFM[n + 1] = state_b;
              m_flows[2:n + 1] = flowModel.m_flows[1:n];
              vsFM[1:n] = vs;
              vsFM[n + 1] = m_flows[n + 1] / Medium.density(state_b) / crossAreas[n];
              port_a.p = mediums[1].p;


            elseif modelStructure == ModelStructure.a_vb then
              statesFM[1] = state_a;
              statesFM[2:n + 1] = mediums[1:n].state;
              m_flows[1:n] = flowModel.m_flows[1:n];
              vsFM[1] = m_flows[1] / Medium.density(state_a) / crossAreas[1];
              vsFM[2:n + 1] = vs;
              port_b.p = mediums[n].p;


            elseif modelStructure == ModelStructure.a_v_b then
              statesFM[1] = state_a;
              statesFM[2:n + 1] = mediums[1:n].state;
              statesFM[n + 2] = state_b;
              m_flows[1:n + 1] = flowModel.m_flows[1:n + 1];
              vsFM[1] = m_flows[1] / Medium.density(state_a) / crossAreas[1];
              vsFM[2:n + 1] = vs;
              vsFM[n + 2] = m_flows[n + 1] / Medium.density(state_b) / crossAreas[n];


            else
              assert(true, "Unknown model structure");

            end if;

          end if;
          annotation(defaultComponentName = "pipe", Documentation(info = "<html>
<p>Base class for distributed flow models. The total volume is split into nNodes segments along the flow path.
The default value is nNodes=2.
</p>
<p><b>Mass and Energy balances</b></p>
The mass and energy balances are inherited from <a href=\"Modelica://Modelica.Fluid.Interfaces.PartialDistributedVolume\">Interfaces.PartialDistributedVolume</a>.
One total mass and one energy balance is formed across each segment according to the finite volume approach.
Substance mass balances are added if the medium contains more than one component.
<p>
An extending model needs to define the geometry and the difference in heights between the flow segments (static head).
Moreover it needs to define two vectors of source terms for the distributed energy balance:
<ul>
<li><tt><b>Qb_flows[nNodes]</b></tt>, the heat flow source terms, e.g. conductive heat flows across segment boundaries, and</li>
<li><tt><b>Wb_flows[nNodes]</b></tt>, the work source terms.</li>
</ul>
</p>

<p><b>Momentum balance</b></p>
The momentum balance is determined by the <b><tt>FlowModel</tt></b> component, which can be replaced with any model extended from
<a href=\"Modelica://Modelica.Fluid.Pipes.BaseClasses.FlowModels.PartialStaggeredFlowModel\">BaseClasses.FlowModels.PartialStaggeredFlowModel</a>.
The default setting is <a href=\"Modelica://Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow\">DetailedPipeFlow</a>.
This considers
<ul>
<li>pressure drop due to friction and other dissipative losses, and</li>
<li>gravity effects for non-horizontal devices.</li>
<li>variation of flow velocity along the flow path,
which occur due to changes in the cross sectional area or the fluid density, provided that <tt>flowModel.use_Ib_flows</tt> is true.
</ul>

<p><b>Model Structure</b></p>
The momentum balances are formulated across the segment boundaries along the flow path according to the staggered grid approach.
The configurable <b><tt>modelStructure</tt></b> determines the formulation of the boundary conditions at <tt>port_a</tt> and <tt>port_b</tt>.
The options include (default: av_vb):
<ul>
<li><tt>av_vb</tt>: Symmetric setting with nNodes-1 momentum balances between nNodes flow segments.
    The ports <tt>port_a</tt> and <tt>port_b</tt> expose the first and the last thermodynamic state, respectively.
    Connecting two or more flow devices therefore may result in high-index DAEs for the pressures of connected flow segments.
<li><tt>a_v_b</tt>: Alternative symmetric setting with nNodes+1 momentum balances across nNodes flow segments.
    Half momentum balances are placed between <tt>port_a</tt> and the first flow segment as well as between the last flow segment and <tt>port_b</tt>.
    Connecting two or more flow devices therefore results in algebraic pressures at the ports.
    The specification of good start values for the port pressures is essential for the solution of large nonlinear equation systems.</li>
<li><tt>av_b</tt>: Unsymmetric setting with nNodes momentum balances, one between nth volume and <tt>port_b</tt>, potential pressure state at <tt>port_a</tt></li>
<li><tt>a_vb</tt>: Unsymmetric setting with nNodes momentum balance, one between first volume and <tt>port_a</tt>, potential pressure state at <tt>port_b</tt></li>
</ul></p>

When connecting two components, e.g. two pipes, the momentum balance across the connection point reduces to
</p>
<pre>pipe1.port_b.p = pipe2.port_a.p</pre>
<p>
This is only true if the flow velocity remains the same on each side of the connection.
Consider using a fitting for any significant change in diameter or fluid density, if the resulting effects,
such as change in kinetic energy, cannot be neglected.
This also allows for taking into account friction losses with respect to the actual geometry of the connection point.
</p>

</html>", revisions = "<html>
<ul>
<li><i>5 Dec 2008</i>
    by Michael Wetter:<br>
       Modified mass balance for trace substances. With the new formulation, the trace substances masses <tt>mC</tt> are stored
       in the same way as the species <tt>mXi</tt>.</li>
<li><i>Dec 2008</i>
    by R&uuml;diger Franke:<br>
       Derived model from original DistributedPipe models
    <ul>
    <li>moved mass and energy balances to PartialDistributedVolume</li>
    <li>introduced replaceable pressure loss models</li>
    <li>combined all model structures and lumped pressure into one model</li>
    <li>new ModelStructure av_vb, replacing former avb</li>
    </ul></li>
<li><i>04 Mar 2006</i>
    by Katrin Pr&ouml;l&szlig;:<br>
       Model added to the Fluid library</li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -72,10},{ -52, -10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{50,10},{70, -10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -100, -50},{ -100,50},{100,60},{100, -60},{ -100, -50}}, smooth = Smooth.None, fillColor = {215,215,215}, fillPattern = FillPattern.Solid, pattern = LinePattern.None, lineColor = {0,0,0}),Polygon(points = {{ -50, -52},{ -50,52},{50,58},{50, -58},{ -50, -52}}, smooth = Smooth.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None, lineColor = {0,0,0}),Line(points = {{ -100, -50},{ -100,50}}, arrow = {Arrow.Filled,Arrow.Filled}, color = {0,0,0}, pattern = LinePattern.Dot),Text(extent = {{ -99,36},{ -69,30}}, lineColor = {0,0,255}, textString = "crossAreas[1]", pattern = LinePattern.None),Line(points = {{ -100,70},{ -50,70}}, arrow = {Arrow.Filled,Arrow.Filled}, color = {0,0,0}, pattern = LinePattern.Dot),Text(extent = {{0,36},{40,30}}, lineColor = {0,0,255}, textString = "crossAreas[2:n-1]", pattern = LinePattern.None),Line(points = {{100, -60},{100,60}}, arrow = {Arrow.Filled,Arrow.Filled}, color = {0,0,0}, pattern = LinePattern.Dot),Text(extent = {{100.5,36},{130.5,30}}, lineColor = {0,0,255}, textString = "crossAreas[n]", pattern = LinePattern.None),Line(points = {{ -50,52},{ -50, -53}}, smooth = Smooth.None, color = {0,0,0}, pattern = LinePattern.Dash),Line(points = {{50,57},{50, -58}}, smooth = Smooth.None, color = {0,0,0}, pattern = LinePattern.Dash),Line(points = {{50,70},{100,70}}, arrow = {Arrow.Filled,Arrow.Filled}, color = {0,0,0}, pattern = LinePattern.Dot),Line(points = {{ -50,70},{50,70}}, arrow = {Arrow.Filled,Arrow.Filled}, color = {0,0,0}, pattern = LinePattern.Dot),Text(extent = {{ -30,77},{30,71}}, lineColor = {0,0,255}, textString = "lengths[2:n-1]", pattern = LinePattern.None),Line(points = {{ -100, -70},{0, -70}}, arrow = {Arrow.None,Arrow.Filled}, color = {0,0,0}),Text(extent = {{ -80, -63},{ -20, -69}}, lineColor = {0,0,255}, pattern = LinePattern.None, textString = "flowModel.dps_fg[1]"),Line(points = {{0, -70},{100, -70}}, arrow = {Arrow.None,Arrow.Filled}, color = {0,0,0}),Text(extent = {{20.5, -63},{80, -69}}, lineColor = {0,0,255}, pattern = LinePattern.None, textString = "flowModel.dps_fg[2:n-1]"),Line(points = {{ -95,0},{ -5,0}}, arrow = {Arrow.None,Arrow.Filled}, color = {0,0,0}),Text(extent = {{ -49,7},{ -19,1}}, lineColor = {0,0,255}, pattern = LinePattern.None, textString = "m_flows[2]"),Line(points = {{5,0},{95,0}}, arrow = {Arrow.None,Arrow.Filled}, color = {0,0,0}),Text(extent = {{17,7},{47,1}}, lineColor = {0,0,255}, pattern = LinePattern.None, textString = "m_flows[3:n]"),Line(points = {{ -150,0},{ -105,0}}, arrow = {Arrow.None,Arrow.Filled}, color = {0,0,0}),Line(points = {{105,0},{150,0}}, arrow = {Arrow.None,Arrow.Filled}, color = {0,0,0}),Text(extent = {{ -140,7},{ -110,1}}, lineColor = {0,0,255}, textString = "m_flows[1]", pattern = LinePattern.None),Text(extent = {{111,7},{141,1}}, lineColor = {0,0,255}, textString = "m_flows[n+1]", pattern = LinePattern.None),Text(extent = {{35, -92},{100, -98}}, lineColor = {0,0,255}, pattern = LinePattern.None, textString = "(ModelStructure av_vb, n=3)"),Line(points = {{ -100, -50},{ -100, -86}}, smooth = Smooth.None, color = {0,0,0}, pattern = LinePattern.Dot),Line(points = {{0, -55},{0, -86}}, smooth = Smooth.None, color = {0,0,0}, pattern = LinePattern.Dot),Line(points = {{100, -60},{100, -86}}, smooth = Smooth.None, color = {0,0,0}, pattern = LinePattern.Dot),Ellipse(extent = {{ -5,5},{5, -5}}, pattern = LinePattern.None, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{3, -4},{33, -10}}, lineColor = {0,0,255}, pattern = LinePattern.None, textString = "states[2:n-1]"),Ellipse(extent = {{95,5},{105, -5}}, pattern = LinePattern.None, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{104, -4},{124, -10}}, lineColor = {0,0,255}, pattern = LinePattern.None, textString = "states[n]"),Ellipse(extent = {{ -105,5},{ -95, -5}}, pattern = LinePattern.None, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -96, -4},{ -76, -10}}, lineColor = {0,0,255}, pattern = LinePattern.None, textString = "states[1]"),Text(extent = {{ -99.5,30},{ -69.5,24}}, lineColor = {0,0,255}, pattern = LinePattern.None, textString = "dimensions[1]"),Text(extent = {{ -0.5,30},{40,24}}, lineColor = {0,0,255}, pattern = LinePattern.None, textString = "dimensions[2:n-1]"),Text(extent = {{100.5,30},{130.5,24}}, lineColor = {0,0,255}, pattern = LinePattern.None, textString = "dimensions[n]"),Line(points = {{ -50,73},{ -50,52}}, smooth = Smooth.None, color = {0,0,0}, pattern = LinePattern.Dot),Line(points = {{50,73},{50,57}}, smooth = Smooth.None, color = {0,0,0}, pattern = LinePattern.Dot),Line(points = {{ -100,50},{100,60}}, smooth = Smooth.None, color = {0,0,0}, thickness = 0.5),Line(points = {{ -100, -50},{100, -60}}, smooth = Smooth.None, color = {0,0,0}, thickness = 0.5),Line(points = {{ -100,73},{ -100,50}}, smooth = Smooth.None, color = {0,0,0}, pattern = LinePattern.Dot),Line(points = {{100,73},{100,60}}, smooth = Smooth.None, color = {0,0,0}, pattern = LinePattern.Dot),Line(points = {{0, -55},{0,55}}, arrow = {Arrow.Filled,Arrow.Filled}, color = {0,0,0}, pattern = LinePattern.Dot),Line(points = {{ -50,11},{50,11}}, arrow = {Arrow.None,Arrow.Filled}, color = {0,0,0}),Text(extent = {{5,18},{25,12}}, lineColor = {0,0,255}, pattern = LinePattern.None, textString = "vs[2:n-1]"),Text(extent = {{ -80,18},{ -70,12}}, lineColor = {0,0,255}, pattern = LinePattern.None, textString = "vs[1]"),Line(points = {{ -100,11},{ -50,11}}, arrow = {Arrow.None,Arrow.Filled}, color = {0,0,0}),Text(extent = {{70,18},{80,12}}, lineColor = {0,0,255}, pattern = LinePattern.None, textString = "vs[n]"),Line(points = {{50,11},{100,11}}, arrow = {Arrow.None,Arrow.Filled}, color = {0,0,0}),Text(extent = {{ -80, -75},{ -20, -81}}, lineColor = {0,0,255}, pattern = LinePattern.None, textString = "flowModel.pathLengths[1]"),Line(points = {{ -100, -82},{0, -82}}, arrow = {Arrow.Filled,Arrow.Filled}, color = {0,0,0}),Line(points = {{0, -82},{100, -82}}, arrow = {Arrow.Filled,Arrow.Filled}, color = {0,0,0}),Text(extent = {{15, -75},{85, -81}}, lineColor = {0,0,255}, pattern = LinePattern.None, textString = "flowModel.pathLengths[2:n-1]"),Text(extent = {{ -100,77},{ -50,71}}, lineColor = {0,0,255}, pattern = LinePattern.None, textString = "lengths[1]"),Text(extent = {{50,77},{100,71}}, lineColor = {0,0,255}, pattern = LinePattern.None, textString = "lengths[n]")}));
        end PartialTwoPortFlow;
        package FlowModels "Flow models for pipes, including wall friction, static head and momentum flow"
          partial model PartialStaggeredFlowModel "Base class for momentum balances in flow models"
            replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium in the component" annotation(Dialog(tab = "Internal Interface", enable = false));
            parameter Integer n = 2 "Number of discrete flow volumes" annotation(Dialog(tab = "Internal Interface", enable = false));
            input Medium.ThermodynamicState[n] states "Thermodynamic states along design flow";
            input Modelica.SIunits.Velocity[n] vs "Mean velocities of fluid flow";
            parameter Real nParallel "number of identical parallel flow devices" annotation(Dialog(tab = "Internal Interface", enable = false, group = "Geometry"));
            input SI.Area[n] crossAreas "Cross flow areas at segment boundaries";
            input SI.Length[n] dimensions "Characteristic dimensions for fluid flow (diameters for pipe flow)";
            input SI.Height[n] roughnesses "Average height of surface asperities";
            input SI.Length[n - 1] dheights "Height(states[2:n]) - Height(states[1:n-1])";
            parameter SI.Acceleration g = system.g "Constant gravity acceleration" annotation(Dialog(tab = "Internal Interface", enable = false, group = "Static head"));
            parameter Boolean allowFlowReversal = system.allowFlowReversal "= true to allow flow reversal, false restricts to design direction (states[1] -> states[n+1])" annotation(Dialog(tab = "Internal Interface", enable = false, group = "Assumptions"), Evaluate = true);
            parameter Modelica.Fluid.Types.Dynamics momentumDynamics = system.momentumDynamics "Formulation of momentum balance" annotation(Dialog(tab = "Internal Interface", enable = false, group = "Assumptions"), Evaluate = true);
            parameter Medium.MassFlowRate m_flow_start = system.m_flow_start "Start value of mass flow rates" annotation(Dialog(tab = "Internal Interface", enable = false, group = "Initialization"));
            parameter Medium.AbsolutePressure p_a_start "Start value for p[1] at design inflow" annotation(Dialog(tab = "Internal Interface", enable = false, group = "Initialization"));
            parameter Medium.AbsolutePressure p_b_start "Start value for p[n+1] at design outflow" annotation(Dialog(tab = "Internal Interface", enable = false, group = "Initialization"));
            extends Modelica.Fluid.Interfaces.PartialDistributedFlow(final m = n - 1);
            parameter Boolean useUpstreamScheme = true "= false to average upstream and downstream properties across flow segments" annotation(Dialog(group = "Advanced"), Evaluate = true);
            parameter Boolean use_Ib_flows = momentumDynamics <> Types.Dynamics.SteadyState "= true to consider differences in flow of momentum through boundaries" annotation(Dialog(group = "Advanced"), Evaluate = true);
            SI.Density[n] rhos = if use_rho_nominal then fill(rho_nominal, n) else Medium.density(states);
            SI.Density[n - 1] rhos_act "Actual density per segment";
            SI.DynamicViscosity[n] mus = if use_mu_nominal then fill(mu_nominal, n) else Medium.dynamicViscosity(states);
            SI.DynamicViscosity[n - 1] mus_act "Actual viscosity per segment";
            Modelica.SIunits.Pressure[n - 1] dps_fg(each start = (p_a_start - p_b_start) / (n - 1)) "pressure drop between states";
            parameter SI.ReynoldsNumber Re_turbulent = 4000 "Start of turbulent regime, depending on type of flow device";
            parameter Boolean show_Res = false "= true, if Reynolds numbers are included for plotting" annotation(Evaluate = true, Dialog(group = "Diagnostics"));
            SI.ReynoldsNumber[n] Res = Modelica.Fluid.Pipes.BaseClasses.CharacteristicNumbers.ReynoldsNumber(vs / nParallel, rhos, mus, dimensions) if show_Res "Reynolds numbers";
            Medium.MassFlowRate[n - 1] m_flows_turbulent = array(nParallel * Modelica.Constants.pi / 4 * 0.5 * (dimensions[i] + dimensions[i + 1]) * mus_act[i] * Re_turbulent for i in 1:n - 1) if show_Res "Start of turbulent flow";
          protected 
            parameter Boolean use_rho_nominal = false "= true, if rho_nominal is used, otherwise computed from medium" annotation(Dialog(group = "Advanced"), Evaluate = true);
            parameter SI.Density rho_nominal = Medium.density_pTX(Medium.p_default, Medium.T_default, Medium.X_default) "Nominal density (e.g. rho_liquidWater = 995, rho_air = 1.2)" annotation(Dialog(group = "Advanced", enable = use_rho_nominal));
            parameter Boolean use_mu_nominal = false "= true, if mu_nominal is used, otherwise computed from medium" annotation(Dialog(group = "Advanced"), Evaluate = true);
            parameter SI.DynamicViscosity mu_nominal = Medium.dynamicViscosity(Medium.setState_pTX(Medium.p_default, Medium.T_default, Medium.X_default)) "Nominal dynamic viscosity (e.g. mu_liquidWater = 1e-3, mu_air = 1.8e-5)" annotation(Dialog(group = "Advanced", enable = use_mu_nominal));

          equation 
            if not allowFlowReversal then
              rhos_act = rhos[1:n - 1];
              mus_act = mus[1:n - 1];

            elseif not useUpstreamScheme then
              rhos_act = 0.5 * (rhos[1:n - 1] + rhos[2:n]);
              mus_act = 0.5 * (mus[1:n - 1] + mus[2:n]);


            else
              for i in 1:n - 1 loop
              rhos_act[i] = noEvent(if m_flows[i] > 0 then rhos[i] else rhos[i + 1]);
              mus_act[i] = noEvent(if m_flows[i] > 0 then mus[i] else mus[i + 1]);

              end for;

            end if;
            if use_Ib_flows then
              Ib_flows = array(rhos[i] * vs[i] * vs[i] * crossAreas[i] - rhos[i + 1] * vs[i + 1] * vs[i + 1] * crossAreas[i + 1] for i in 1:n - 1);

            else
              Ib_flows = zeros(n - 1);

            end if;
            Fs_p = array(0.5 * (crossAreas[i] + crossAreas[i + 1]) * (Medium.pressure(states[i + 1]) - Medium.pressure(states[i])) for i in 1:n - 1);
            dps_fg = array((Fs_fg[i] * 2) / (crossAreas[i] + crossAreas[i + 1]) for i in 1:n - 1);
            annotation(Documentation(info = "<html>
<p>
This paratial model defines a common interface for <tt>m=n-1</tt> flow models between <tt>n</tt> device segments.
The flow models provide a steady-state or dynamic momentum balance using an upwind discretization scheme per default.
Extending models must add pressure loss terms for friction and gravity.
</p>
<p>
The fluid is specified in the interface with the thermodynamic <tt>states[n]</tt> for a given <tt>Medium</tt> model.
The geometry is specified with the <tt>pathLengths[n-1]</tt> between the device segments as well as
with the <tt>crossAreas[n]</tt> and the <tt>roughnesses[n]</tt> of the device segments.
Moreover the fluid flow is characterized for different types of devices by the characteristic <tt>dimensions[n]</tt>
and the average velocities <tt>vs[n]</tt> of fluid flow in the device segments.
See <a href=\"Modelica://Modelica.Fluid.Pipes.BaseClasses.CharacteristicNumbers.ReynoldsNumber\">Pipes.BaseClasses.CharacteristicNumbers.ReynoldsNumber</a>
for examplary definitions.
</p>
<p>
The parameter <tt>Re_turbulent</tt> can be specified for the least mass flow rate of the turbulent regime.
It defaults to 4000, which is appropriate for pipe flow.
The <tt>m_flows_turbulent[n-1]</tt> resulting from <tt>Re_turbulent</tt> can optionally be calculated together with the Reynolds numbers
<tt>Res[n]</tt> of the device segments (<tt>show_Res=true</tt>).
</p>
<p>
Using the thermodynamic states[n] of the device segments, the densities rhos[n] and the dynamic viscosities mus[n]
of the segments as well as the actual densities rhos_act[n-1] and the actual viscosities mus_act[n-1] of the flows are predefined
in this base model. Note that no events are raised on flow reversal. This needs to be treated by an extending model,
e.g. with numerical smoothing or by raising events as appropriate.
</p>
</html>"), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -80, -50},{ -80,50},{80, -50},{80,50}}, color = {0,0,255}, smooth = Smooth.None, thickness = 1),Text(extent = {{ -40, -50},{40, -90}}, lineColor = {0,0,0}, textString = "%name")}));
          end PartialStaggeredFlowModel;
          model NominalLaminarFlow "NominalLaminarFlow: Linear pressure loss for nominal values"
            extends Modelica.Fluid.Pipes.BaseClasses.FlowModels.PartialStaggeredFlowModel(use_mu_nominal = not show_Res);
            parameter SI.AbsolutePressure dp_nominal "Nominal pressure loss";
            parameter SI.MassFlowRate m_flow_nominal "Mass flow rate for dp_nominal";
            SI.Length[n - 1] pathLengths_nominal = array(((dp_nominal / (n - 1) - g * dheights[i]) * Modelica.Constants.pi * ((dimensions[i] + dimensions[i + 1]) / 2) ^ 4 * rhos_act[i]) / (128 * mus_act[i]) / m_flow_nominal / nParallel for i in 1:n - 1) if show_Res;

          equation 
            if not allowFlowReversal or use_rho_nominal or not useUpstreamScheme then
              dps_fg = array(g * dheights[i] * rhos_act[i] for i in 1:n - 1) + dp_nominal / (n - 1) / m_flow_nominal * m_flows;

            else
              dps_fg = array(g * dheights[i] * (if m_flows[i] > 0 then rhos[i] else rhos[i + 1]) for i in 1:n - 1) + dp_nominal / (n - 1) / m_flow_nominal * m_flows;

            end if;
            annotation(Documentation(info = "<html>
<p>
This model defines a simple lineaer pressure loss assuming laminar flow for
specified <tt>dp_nominal</tt> and <tt>m_flow_nominal</tt>.
</p>
<p>
Select <tt>show_Res = true</tt> to analyze the actual flow and the lengths of a pipe that would fulfill the
specified nominal values for given geometry parameters <tt>crossAreas</tt>, <tt>dimensions</tt> and <tt>roughnesses</tt>.
</p>
</html>"));
          end NominalLaminarFlow;
          partial model PartialGenericPipeFlow "GenericPipeFlow: Pipe flow pressure loss and gravity with replaceable WallFriction package"
            extends Modelica.Fluid.Pipes.BaseClasses.FlowModels.PartialStaggeredFlowModel(final Re_turbulent = 4000);
            replaceable package WallFriction = Modelica.Fluid.Pipes.BaseClasses.WallFriction.Detailed extends Modelica.Fluid.Pipes.BaseClasses.WallFriction.PartialWallFriction "Wall friction model" annotation(Dialog(group = "Wall friction"), choicesAllMatching = true, __Dymola_editButton = false);
            input SI.Length[n - 1] pathLengths_internal "pathLengths used internally; to be defined by extending class";
            parameter SI.AbsolutePressure dp_nominal "Nominal pressure loss (for nominal models)";
            parameter SI.MassFlowRate m_flow_nominal "Mass flow rate for dp_nominal (for nominal models)";
            parameter Boolean from_dp = momentumDynamics >= Types.Dynamics.SteadyStateInitial " = true, use m_flow = f(dp), otherwise dp = f(m_flow)" annotation(Evaluate = true);
            parameter SI.AbsolutePressure dp_small = system.dp_small "Within regularization if |dp| < dp_small (may be wider for large discontinuities in static head)" annotation(Dialog(enable = from_dp and WallFriction.use_dp_small));
            parameter SI.MassFlowRate m_flow_small = system.m_flow_small "Within regularization if |m_flows| < m_flow_small (may be wider for large discontinuities in static head)" annotation(Dialog(enable = not from_dp and WallFriction.use_m_flow_small));
            final parameter Boolean constantPressureLossCoefficient = use_rho_nominal and (use_mu_nominal or not WallFriction.use_mu) "= true if the pressure loss does not depend on fluid states" annotation(Evaluate = true);
            final parameter Boolean continuousFlowReversal = not useUpstreamScheme or constantPressureLossCoefficient or not allowFlowReversal "= true if the pressure loss is continuous around zero flow" annotation(Evaluate = true);
            SI.Length[n - 1] diameters = 0.5 * (dimensions[1:n - 1] + dimensions[2:n]) "mean diameters between segments";

          equation 
            for i in 1:n - 1 loop
            assert(m_flows[i] >  -m_flow_small or allowFlowReversal, "Reverting flow occurs even though allowFlowReversal is false");

            end for;
            if continuousFlowReversal then
              if from_dp and not WallFriction.dp_is_zero then
                m_flows = WallFriction.massFlowRate_dp(dps_fg - array(g * dheights[i] * rhos_act[i] for i in 1:n - 1), rhos_act, rhos_act, mus_act, mus_act, pathLengths_internal, diameters, (roughnesses[1:n - 1] + roughnesses[2:n]) / 2, dp_small) * nParallel;

              else
                dps_fg = WallFriction.pressureLoss_m_flow(m_flows / nParallel, rhos_act, rhos_act, mus_act, mus_act, pathLengths_internal, diameters, (roughnesses[1:n - 1] + roughnesses[2:n]) / 2, m_flow_small / nParallel) + array(g * dheights[i] * rhos_act[i] for i in 1:n - 1);

              end if;

            else
              if from_dp and not WallFriction.dp_is_zero then
                m_flows = WallFriction.massFlowRate_dp_staticHead(dps_fg, rhos[1:n - 1], rhos[2:n], mus[1:n - 1], mus[2:n], pathLengths_internal, diameters, g * dheights, (roughnesses[1:n - 1] + roughnesses[2:n]) / 2, dp_small / n) * nParallel;

              else
                dps_fg = WallFriction.pressureLoss_m_flow_staticHead(m_flows / nParallel, rhos[1:n - 1], rhos[2:n], mus[1:n - 1], mus[2:n], pathLengths_internal, diameters, g * dheights, (roughnesses[1:n - 1] + roughnesses[2:n]) / 2, m_flow_small / nParallel);

              end if;

            end if;
            annotation(Documentation(info = "<html>
<p>
This model describes pressure losses due to <b>wall friction</b> in a pipe
and due to <b>gravity</b>.
Correlations of different complexity and validity can be
seleted via the replaceable package <b>WallFriction</b> (see parameter menu below).
The details of the pipe wall friction model are described in the
<a href=\"Modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.WallFriction\">UsersGuide</a>.
Basically, different variants of the equation
</p>

<pre>
   dp = &lambda;(Re,<font face=\"Symbol\">D</font>)*(L/D)*&rho;*v*|v|/2.
</pre>

<p>

By default, the correlations are computed with media data at the actual time instant.
In order to reduce non-linear equation systems, the parameters
<b>use_mu_nominal</b> and <b>use_rho_nominal</b> provide the option
to compute the correlations with constant media values
at the desired operating point. This might speed-up the
simulation and/or might give a more robust simulation.
</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,64},{100, -64}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Backward),Rectangle(extent = {{ -100,50},{100, -49}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -60, -49},{ -60,50}}, color = {0,0,255}, arrow = {Arrow.Filled,Arrow.Filled}),Text(extent = {{ -50,16},{6, -10}}, lineColor = {0,0,255}, textString = "diameters"),Line(points = {{ -100,74},{100,74}}, color = {0,0,255}, arrow = {Arrow.Filled,Arrow.Filled}),Text(extent = {{ -32,93},{32,74}}, lineColor = {0,0,255}, textString = "pathLengths")}));
          end PartialGenericPipeFlow;
          model NominalTurbulentPipeFlow "NominalTurbulentPipeFlow: Quadratic turbulent pressure loss for nominal values"
            extends Modelica.Fluid.Pipes.BaseClasses.FlowModels.PartialGenericPipeFlow(redeclare package WallFriction = Modelica.Fluid.Pipes.BaseClasses.WallFriction.QuadraticTurbulent, use_mu_nominal = not show_Res, pathLengths_internal = pathLengths_nominal, useUpstreamScheme = false);
            import Modelica.Constants.pi;
            SI.Length[n - 1] pathLengths_nominal "pathLengths resulting from nominal pressure loss and geometry";
            Real[n - 1] ks_inv "coefficient for quadratic flow";
            Real[n - 1] zetas "coefficient for quadratic flow";
            Medium.AbsolutePressure[n - 1] dps_fg_turbulent = array((((mus_act[i] * diameters[i] * pi) / 4) ^ 2 * Re_turbulent ^ 2) / (ks_inv[i] * rhos_act[i]) for i in 1:n - 1) if show_Res "Start of turbulent flow";

          equation 
            for i in 1:n - 1 loop
            ks_inv[i] = (m_flow_nominal / nParallel) ^ 2 / (dp_nominal / (n - 1) - g * dheights[i] * rhos_act[i]) / rhos_act[i];
            zetas[i] = (pi * diameters[i] * diameters[i]) ^ 2 / (8 * ks_inv[i]);
            pathLengths_nominal[i] = zetas[i] * diameters[i] * (2 * Modelica.Math.log10(3.7 / (roughnesses[i] + roughnesses[i + 1]) / 2 / diameters[i])) ^ 2;

            end for;
            annotation(Documentation(info = "<html>
<p>
This model defines the pressure loss assuming turbulent flow for
specified <tt>dp_nominal</tt> and <tt>m_flow_nominal</tt>.
It takes into account the fluid density of each flow segment and
obtaines appropriate <tt>pathLengths_nominal</tt> values
for an inverse parameterization of the
<a href=\"Modelica://Modelica.Fluid.Pipes.BaseClasses.FlowModels.TurbulentPipeFlow\">
          TurbulentPipeFlow</a>
model. Per default the upstream and downstream densities are averaged with the setting <tt>useUpstreamScheme = false</tt>,
in order to avoid discontinuous <tt>pathLengths_nominal</tt> values in the case of flow reversal.
</p>
<p>
The geometry parameters <tt>crossAreas</tt>, <tt>diameters</tt> and <tt>roughnesses</tt> do
not effect simulation results of this nominal pressure loss model.
As the geometry is specified however, the optionally calculated Reynolds number as well as
<tt>m_flows_turbulent</tt> and <tt>dps_fg_turbulent</tt> become meaningful
and can be related to <tt>m_flow_small</tt> and <tt>dp_small</tt>.
</p>
<p>
<b>Optional Variables if show_Res</b>
</p>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Type</b></th><th><b>Name</b></th><th><b>Description</b></th></tr>
<tr><td>ReynoldsNumber</td><td>Res[n]</td>
    <td>Reynolds numbers of pipe flow per flow segment</td></tr>
<tr><td>MassFlowRate</td><td>m_flows_turbulent[n-1]</td>
    <td>mass flow rates at start of turbulent region for Re_turbulent=4000</td></tr>
<tr><td>AbsolutePressure</td><td>dps_fg_turbulent[n-1]</td>
    <td>pressure losses due to friction and gravity corresponding to m_flows_turbulent</td></tr>
</table>
</html>", revisions = "<html>
<ul>
<li><i>6 Dec 2008</i>
    by Ruediger Franke</a>:<br>
       Model added to the Fluid library</li>
</ul>
</html>"));
          end NominalTurbulentPipeFlow;
          model TurbulentPipeFlow "TurbulentPipeFlow: Pipe wall friction in the quadratic turbulent regime (simple characteristic, mu not used)"
            extends Modelica.Fluid.Pipes.BaseClasses.FlowModels.PartialGenericPipeFlow(redeclare package WallFriction = Modelica.Fluid.Pipes.BaseClasses.WallFriction.QuadraticTurbulent, use_mu_nominal = not show_Res, pathLengths_internal = pathLengths, dp_nominal = 1000.0 * dp_small, m_flow_nominal = 100.0 * m_flow_small);
            annotation(Documentation(info = "<html>
<p>
This model defines only the quadratic turbulent regime of wall friction:
dp = k*m_flow*|m_flow|, where \"k\" depends on density and the roughness
of the pipe and is not a function of the Reynolds number.
This relationship is only valid for large Reynolds numbers.
The turbulent pressure loss correlation might be useful to optimize models that are only facing turbular flow.
</p>


</html>"));
          end TurbulentPipeFlow;
          model DetailedPipeFlow "DetailedPipeFlow: Pipe wall friction in the laminar and turbulent regime (detailed characteristic)"
            extends Modelica.Fluid.Pipes.BaseClasses.FlowModels.PartialGenericPipeFlow(redeclare package WallFriction = Modelica.Fluid.Pipes.BaseClasses.WallFriction.Detailed, pathLengths_internal = pathLengths, dp_nominal = 1000.0 * dp_small, m_flow_nominal = 100.0 * m_flow_small);
            annotation(Documentation(info = "<html>
<p>
This component defines the complete regime of wall friction.
The details are described in the
<a href=\"Modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.WallFriction\">UsersGuide</a>.
The functional relationship of the friction loss factor &lambda; is
displayed in the next figure. Function massFlowRate_dp() defines the \"red curve\"
(\"Swamee and Jain\"), where as function pressureLoss_m_flow() defines the
\"blue curve\" (\"Colebrook-White\"). The two functions are inverses from
each other and give slightly different results in the transition region
between Re = 1500 .. 4000, in order to get explicit equations without
solving a non-linear equation.
</p>

<img src=\"../Images/Fluid/Components/PipeFriction1.png\">

<p>
Additionally to wall friction, this component properly implements static
head. With respect to the latter, two cases can be distinguished. In the case
shown next, the change of elevation with the path from a to b has the opposite
sign of the change of density.</p>

<img src=\"../Images/Fluid/Components/PipeFrictionStaticHead_case-a.PNG\">

<p>
In the case illustrated second, the change of elevation with the path from a to
b has the same sign of the change of density.</p>

<img src=\"../Images/Fluid/Components/PipeFrictionStaticHead_case-b.PNG\">

</html>"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,64},{100, -64}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Backward),Rectangle(extent = {{ -100,50},{100, -49}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -60, -49},{ -60,50}}, color = {0,0,255}, arrow = {Arrow.Filled,Arrow.Filled}),Text(extent = {{ -50,16},{6, -10}}, lineColor = {0,0,255}, textString = "diameters"),Line(points = {{ -100,74},{100,74}}, color = {0,0,255}, arrow = {Arrow.Filled,Arrow.Filled})}));
          end DetailedPipeFlow;
        end FlowModels;
        package HeatTransfer "Heat transfer for flow models"
          partial model PartialFlowHeatTransfer "base class for any pipe heat transfer correlation"
            extends Modelica.Fluid.Interfaces.PartialHeatTransfer;
            input SI.Velocity[n] vs "Mean velocities of fluid flow in segments";
            parameter Real nParallel "number of identical parallel flow devices" annotation(Dialog(tab = "Internal Interface", enable = false, group = "Geometry"));
            input SI.Length[n] lengths "Lengths along flow path";
            input SI.Length[n] dimensions "Characteristic dimensions for fluid flow (diameter for pipe flow)";
            input SI.Height[n] roughnesses "Average heights of surface asperities";
            annotation(Documentation(info = "<html>
Base class for heat transfer models of flow devices.
<p>
The geometry is specified in the interface with the <tt>surfaceAreas[n]</tt>, the <tt>roughnesses[n]</tt>
and the lengths[n] along the flow path.
Moreover the fluid flow is characterized for different types of devices by the characteristic <tt>dimensions[n+1]</tt>
and the average velocities <tt>vs[n+1]</tt> of fluid flow.
See <a href=\"Modelica://Modelica.Fluid.Pipes.BaseClasses.CharacteristicNumbers.ReynoldsNumber\">Pipes.BaseClasses.CharacteristicNumbers.ReynoldsNumber</a>
for examplary definitions.
</p>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -80,60},{80, -60}}, pattern = LinePattern.None, lineColor = {0,0,0}, fillColor = {255,0,0}, fillPattern = FillPattern.HorizontalCylinder),Text(extent = {{ -40,22},{38, -18}}, lineColor = {0,0,0}, textString = "%name")}));
          end PartialFlowHeatTransfer;
          model IdealFlowHeatTransfer "IdealHeatTransfer: Ideal heat transfer without thermal resistance"
            extends PartialFlowHeatTransfer;

          equation 
            Ts = heatPorts.T;
            annotation(Documentation(info = "<html>
Ideal heat transfer without thermal resistance.
</html>"));
          end IdealFlowHeatTransfer;
          model ConstantFlowHeatTransfer "ConstantHeatTransfer: Constant heat transfer coefficient"
            extends PartialFlowHeatTransfer;
            parameter SI.CoefficientOfHeatTransfer alpha0 "heat transfer coefficient";
            annotation(Documentation(info = "<html>
Simple heat transfer correlation with constant heat transfer coefficient, used as default component in <a distributed pipe models.
</html>"));

          equation 
            Q_flows = array(alpha0 * surfaceAreas[i] * (heatPorts[i].T - Ts[i]) * nParallel for i in 1:n);
          end ConstantFlowHeatTransfer;
          partial model PartialPipeFlowHeatTransfer "Base class for pipe heat transfer correlation in terms of Nusselt number heat transfer in a circular pipe for laminar and turbulent one-phase flow"
            extends PartialFlowHeatTransfer;
            parameter SI.CoefficientOfHeatTransfer alpha0 = 100 "guess value for heat transfer coefficients";
            SI.CoefficientOfHeatTransfer[n] alphas(each start = alpha0) "CoefficientOfHeatTransfer";
            Real[n] Res "Reynolds numbers";
            Real[n] Prs "Prandtl numbers";
            Real[n] Nus "Nusselt numbers";
            Medium.Density[n] ds "Densities";
            Medium.DynamicViscosity[n] mus "Dynamic viscosities";
            Medium.ThermalConductivity[n] lambdas "Thermal conductivity";
            SI.Length[n] diameters = dimensions "Hydraulic diameters for pipe flow";

          equation 
            ds = Medium.density(states);
            mus = Medium.dynamicViscosity(states);
            lambdas = Medium.thermalConductivity(states);
            Prs = Medium.prandtlNumber(states);
            Res = CharacteristicNumbers.ReynoldsNumber(vs / nParallel, ds, mus, diameters);
            Nus = CharacteristicNumbers.NusseltNumber(alphas, diameters, lambdas);
            Q_flows = array(alphas[i] * surfaceAreas[i] * (heatPorts[i].T - Ts[i]) * nParallel for i in 1:n);
            annotation(Documentation(info = "<html>
Base class for heat transfer models that are expressed in terms of the Nusselt number and which can be used in distributed pipe models.
</html>"));
          end PartialPipeFlowHeatTransfer;
          annotation(Documentation(info = "<html>
Heat transfer correlations for pipe models
</html>"));
          model LocalPipeFlowHeatTransfer "LocalPipeFlowHeatTransfer: Laminar and turbulent forced convection in pipes, local coefficients"
            extends PartialPipeFlowHeatTransfer;
          protected 
            Real[n] Nus_turb "Nusselt number for turbulent flow";
            Real[n] Nus_lam "Nusselt number for laminar flow";
            Real Nu_1;
            Real[n] Nus_2;
            Real[n] Xis;

          equation 
            Nu_1 = 3.66;
            for i in 1:n loop
            Nus_turb[i] = smooth(0, (Xis[i] / 8 * abs(Res[i]) * Prs[i]) / (1 + 12.7 * (Xis[i] / 8) ^ 0.5 * (Prs[i] ^ (2 / 3) - 1)) * (1 + 1 / 3 * (diameters[i] / lengths[i] / (if vs[i] >= 0 then i - 0.5 else n - i + 0.5)) ^ (2 / 3)));
            Xis[i] = (1.8 * Modelica.Math.log10(max(1e-10, Res[i])) - 1.5) ^ ( -2);
            Nus_lam[i] = (Nu_1 ^ 3 + 0.7 ^ 3 + (Nus_2[i] - 0.7) ^ 3) ^ (1 / 3);
            Nus_2[i] = smooth(0, 1.077 * ((abs(Res[i]) * Prs[i] * diameters[i]) / lengths[i] / (if vs[i] >= 0 then i - 0.5 else n - i + 0.5)) ^ (1 / 3));
            Nus[i] = spliceFunction(Nus_turb[i], Nus_lam[i], Res[i] - 6150, 3850);

            end for;
            annotation(Documentation(info = "<html>
Heat transfer model for laminar and turbulent flow in pipes. Range of validity:
<ul>
<li>fully developed pipe flow</li>
<li>forced convection</li>
<li>one phase Newtonian fluid</li>
<li>(spatial) constant wall temperature in the laminar region</li>
<li>0 &le; Re &le; 1e6, 0.6 &le; Pr &le; 100, d/L &le; 1</li>
<li>The correlation holds for non-circular pipes only in the turbulent region. Use diameter=4*crossArea/perimeter as characteristic length.</li>
</ul>
The correlation takes into account the spatial position along the pipe flow, which changes discontinuously at flow reversal. However, the heat transfer coefficient itself is continuous around zero flow rate, but not its derivative.
<h4><font color=\"#008000\">References</font></h4>

<dl><dt>Verein Deutscher Ingenieure (1997):</dt>
    <dd><b>VDI W&auml;rmeatlas</b>.
         Springer Verlag, Ed. 8, 1997.</dd>
</dl>
</html>"));
          end LocalPipeFlowHeatTransfer;
        end HeatTransfer;
        package CharacteristicNumbers "Functions to compute characteristic numbers"
          function ReynoldsNumber "Return Reynolds number from v, rho, mu, D"
            input SI.Velocity v "Mean velocity of fluid flow";
            input SI.Density rho "Fluid density";
            input SI.DynamicViscosity mu "Dynamic (absolute) viscosity";
            input SI.Length D "Characteristic dimension (hydraulic diameter of pipes)";
            output SI.ReynoldsNumber Re "Reynolds number";
          algorithm 
            Re:=(abs(v) * rho * D) / mu;
            annotation(Documentation(info = "<html>
<p>
Calculation of Reynolds Number
<pre>
   Re = |v|&rho;D/&mu;
</pre>
a measure of the relationship between inertial forces (v&rho;) and viscous forces (D/&mu;).
</p>
<p>
The following table gives examples for the characteristic dimension D and the velocity v for different fluid flow devices:
</p>
<table border=1 cellspacing=0 cellpadding=2>
<tr><th><b>Device Type</b></th><th><b>Characteristic Dimension D</b></th><th><b>Velocity v</b></th></tr>
<tr><td>Circular Pipe</td><td>diameter</td>
    <td>m_flow/&rho;/crossArea</td></tr>
<tr><td>Rectangular Duct</td><td>4*crossArea/perimeter</td>
    <td>m_flow/&rho;/crossArea</td></tr>
<tr><td>Wide Duct</td><td>distance between narrow, parallel walls</td>
    <td>m_flow/&rho;/crossArea</td></tr>
<tr><td>Packed Bed</td><td>diameterOfSpericalParticles/(1-fluidFractionOfTotalVolume)</td>
    <td>m_flow/&rho;/crossArea (without particles)</td></tr>
<tr><td>Device with rotating agitator</td><td>diameterOfRotor</td>
    <td>RotationalSpeed*diameterOfRotor</td></tr>
</table>
</html>"));
          end ReynoldsNumber;
          function ReynoldsNumber_m_flow "Return Reynolds number from m_flow, mu, D, A"
            input SI.MassFlowRate m_flow "Mass flow rate";
            input SI.DynamicViscosity mu "Dynamic viscosity";
            input SI.Length D "Characteristic dimension (hydraulic diameter of pipes or orifices)";
            input SI.Area A = Modelica.Constants.pi / 4 * D * D "Cross sectional area of fluid flow";
            output SI.ReynoldsNumber Re "Reynolds number";
          algorithm 
            Re:=(abs(m_flow) * D) / A / mu;
            annotation(Documentation(info = "<html>Simplified calculation of Reynolds Number for flow through pipes or orifices;
              using the mass flow rate <tt>m_flow</tt> instead of the velocity <tt>v</tt> to express inertial forces.
<pre>
  Re = |m_flow|*diameter/A/&mu;
with
  m_flow = v*&rho;*A
</pre>
See also <a href=\"Modelica://Modelica.Fluid.Pipes.BaseClasses.CharacteristicNumbers.ReynoldsNumber\">
          Pipes.BaseClasses.CharacteristicNumbers.ReynoldsNumber</a>.
</html>"));
          end ReynoldsNumber_m_flow;
          function NusseltNumber "Return Nusselt number"
            input SI.CoefficientOfHeatTransfer alpha "Coefficient of heat transfer";
            input SI.Length D "Characteristic dimension";
            input SI.ThermalConductivity lambda "Thermal conductivity";
            output SI.NusseltNumber Nu "Nusselt number";
          algorithm 
            Nu:=(alpha * D) / lambda;
            annotation(Documentation(info = "Nusselt number Nu = alpha*D/lambda"));
          end NusseltNumber;
        end CharacteristicNumbers;
        package WallFriction "Different variants for pressure drops due to pipe wall friction"
          partial package PartialWallFriction "Partial wall friction characteristic (base package of all wall friction characteristics)"
            annotation(Documentation(info = "<html>

</html>"));
            constant Boolean use_mu = true "= true, if mu_a/mu_b are used in function, otherwise value is not used";
            constant Boolean use_roughness = true "= true, if roughness is used in function, otherwise value is not used";
            constant Boolean use_dp_small = true "= true, if dp_small is used in function, otherwise value is not used";
            constant Boolean use_m_flow_small = true "= true, if m_flow_small is used in function, otherwise value is not used";
            constant Boolean dp_is_zero = false "= true, if no wall friction is present, i.e., dp = 0 (function massFlowRate_dp() cannot be used)";
            replaceable partial function massFlowRate_dp "Return mass flow rate m_flow as function of pressure loss dp, i.e., m_flow = f(dp), due to wall friction"
              extends Modelica.Icons.Function;
              input SI.Pressure dp "Pressure loss (dp = port_a.p - port_b.p)";
              input SI.Density rho_a "Density at port_a";
              input SI.Density rho_b "Density at port_b";
              input SI.DynamicViscosity mu_a "Dynamic viscosity at port_a (dummy if use_mu = false)";
              input SI.DynamicViscosity mu_b "Dynamic viscosity at port_b (dummy if use_mu = false)";
              input SI.Length length "Length of pipe";
              input SI.Diameter diameter "Inner (hydraulic) diameter of pipe";
              input SI.Length roughness(min = 0) = 2.5e-05 "Absolute roughness of pipe, with a default for a smooth steel pipe (dummy if use_roughness = false)";
              input SI.AbsolutePressure dp_small = 1 "Turbulent flow if |dp| >= dp_small (dummy if use_dp_small = false)";
              output SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
              annotation(Documentation(info = "<html>

</html>"));
            end massFlowRate_dp;
            replaceable partial function massFlowRate_dp_staticHead "Return mass flow rate m_flow as function of pressure loss dp, i.e., m_flow = f(dp), due to wall friction and static head"
              extends Modelica.Icons.Function;
              input SI.Pressure dp "Pressure loss (dp = port_a.p - port_b.p)";
              input SI.Density rho_a "Density at port_a";
              input SI.Density rho_b "Density at port_b";
              input SI.DynamicViscosity mu_a "Dynamic viscosity at port_a (dummy if use_mu = false)";
              input SI.DynamicViscosity mu_b "Dynamic viscosity at port_b (dummy if use_mu = false)";
              input SI.Length length "Length of pipe";
              input SI.Diameter diameter "Inner (hydraulic) diameter of pipe";
              input Real g_times_height_ab "Gravity times (Height(port_b) - Height(port_a))";
              input SI.Length roughness(min = 0) = 2.5e-05 "Absolute roughness of pipe, with a default for a smooth steel pipe (dummy if use_roughness = false)";
              input SI.AbsolutePressure dp_small = 1 "Turbulent flow if |dp| >= dp_small (dummy if use_dp_small = false)";
              output SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
              annotation(Documentation(info = "<html>

</html>"));
            end massFlowRate_dp_staticHead;
            replaceable partial function pressureLoss_m_flow "Return pressure loss dp as function of mass flow rate m_flow, i.e., dp = f(m_flow), due to wall friction"
              extends Modelica.Icons.Function;
              input SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
              input SI.Density rho_a "Density at port_a";
              input SI.Density rho_b "Density at port_b";
              input SI.DynamicViscosity mu_a "Dynamic viscosity at port_a (dummy if use_mu = false)";
              input SI.DynamicViscosity mu_b "Dynamic viscosity at port_b (dummy if use_mu = false)";
              input SI.Length length "Length of pipe";
              input SI.Diameter diameter "Inner (hydraulic) diameter of pipe";
              input SI.Length roughness(min = 0) = 2.5e-05 "Absolute roughness of pipe, with a default for a smooth steel pipe (dummy if use_roughness = false)";
              input SI.MassFlowRate m_flow_small = 0.01 "Turbulent flow if |m_flow| >= m_flow_small (dummy if use_m_flow_small = false)";
              output SI.Pressure dp "Pressure loss (dp = port_a.p - port_b.p)";
              annotation(Documentation(info = "<html>

</html>"));
            end pressureLoss_m_flow;
            replaceable partial function pressureLoss_m_flow_staticHead "Return pressure loss dp as function of mass flow rate m_flow, i.e., dp = f(m_flow), due to wall friction and static head"
              extends Modelica.Icons.Function;
              input SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
              input SI.Density rho_a "Density at port_a";
              input SI.Density rho_b "Density at port_b";
              input SI.DynamicViscosity mu_a "Dynamic viscosity at port_a (dummy if use_mu = false)";
              input SI.DynamicViscosity mu_b "Dynamic viscosity at port_b (dummy if use_mu = false)";
              input SI.Length length "Length of pipe";
              input SI.Diameter diameter "Inner (hydraulic) diameter of pipe";
              input Real g_times_height_ab "Gravity times (Height(port_b) - Height(port_a))";
              input SI.Length roughness(min = 0) = 2.5e-05 "Absolute roughness of pipe, with a default for a smooth steel pipe (dummy if use_roughness = false)";
              input SI.MassFlowRate m_flow_small = 0.01 "Turbulent flow if |m_flow| >= m_flow_small (dummy if use_m_flow_small = false)";
              output SI.Pressure dp "Pressure loss (dp = port_a.p - port_b.p)";
              annotation(Documentation(info = "<html>

</html>"));
            end pressureLoss_m_flow_staticHead;
          end PartialWallFriction;
          annotation(Documentation(info = "<html>
<p>
This package provides functions to compute
pressure losses due to <b>wall friction</b> in a pipe.
Every correlation is defined by a package that is derived
by inheritance from the package WallFriction.PartialWallFriction.
The details of the underlying pipe wall friction model are described in the
<a href=\"Modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.WallFriction\">UsersGuide</a>.
Basically, different variants of the equation
</p>

<pre>
   dp = &lambda;(Re,<font face=\"Symbol\">D</font>)*(L/D)*&rho;*v*|v|/2
</pre>

<p>
are used, where the friction loss factor &lambda; is shown
in the next figure:
</p>

<img src=\"../Images/Fluid/Components/PipeFriction1.png\">

</html>"));
          package NoFriction "No pipe wall friction, no static head"
            annotation(Documentation(info = "<html>
<p>
This component sets the pressure loss due to wall friction
to zero, i.e., it allows to switch off pipe wall friction.
</p>
</html>"));
            extends PartialWallFriction(final use_mu = false, final use_roughness = false, final use_dp_small = false, final use_m_flow_small = false, final dp_is_zero = true);
            function extends massFlowRate_dp "Return mass flow rate m_flow as function of pressure loss dp, i.e., m_flow = f(dp), due to wall friction"
              annotation(Documentation(info = "<html>

</html>"));
            algorithm 
              assert(false, "function massFlowRate_dp (option: from_dp=true)
cannot be used for WallFriction.NoFriction. Use instead
function pressureLoss_m_flow (option: from_dp=false)");
            end massFlowRate_dp;
            function extends pressureLoss_m_flow "Return pressure loss dp as function of mass flow rate m_flow, i.e., dp = f(m_flow), due to wall friction"
              annotation(Documentation(info = "<html>

</html>"));
            algorithm 
              dp:=0;
            end pressureLoss_m_flow;
            function extends massFlowRate_dp_staticHead "Return mass flow rate m_flow as function of pressure loss dp, i.e., m_flow = f(dp), due to wall friction and static head"
              annotation(Documentation(info = "<html>

</html>"));
            algorithm 
              assert(false, "function massFlowRate_dp (option: from_dp=true)
cannot be used for WallFriction.NoFriction. Use instead
function pressureLoss_m_flow (option: from_dp=false)");
            end massFlowRate_dp_staticHead;
            function extends pressureLoss_m_flow_staticHead "Return pressure loss dp as function of mass flow rate m_flow, i.e., dp = f(m_flow), due to wall friction and static head"
              annotation(Documentation(info = "<html>

</html>"));
            algorithm 
              dp:=0;
              assert(abs(g_times_height_ab) < Modelica.Constants.small, "WallFriction.NoFriction does not consider static head and cannot be used with height_ab<>0!");
            end pressureLoss_m_flow_staticHead;
          end NoFriction;
          package Laminar "Pipe wall friction in the laminar regime (linear correlation)"
            annotation(Documentation(info = "<html>
<p>
This component defines only the laminar region of wall friction:
dp = k*m_flow, where \"k\" depends on density and dynamic viscosity.
The roughness of the wall does not have an influence on the laminar
flow and therefore argument roughness is ignored.
Since this is a linear relationship, the occuring systems of equations
are usually much simpler (e.g. either linear instead of non-linear).
By using nominal values for density and dynamic viscosity, the
systems of equations can still further be reduced.
</p>

<p>
In <a href=\"Modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.WallFriction\">UsersGuide</a> the complete friction regime is illustrated.
This component describes only the <b>Hagen-Poiseuille</b> equation.
</p>
<br>

</html>"));
            extends PartialWallFriction(final use_mu = true, final use_roughness = false, final use_dp_small = false, final use_m_flow_small = false);
            function extends massFlowRate_dp "Return mass flow rate m_flow as function of pressure loss dp, i.e., m_flow = f(dp), due to wall friction"
              annotation(Documentation(info = "<html>

</html>"));
            algorithm 
              m_flow:=(dp * Modelica.Constants.pi * diameter ^ 4 * (rho_a + rho_b)) / (128 * length * (mu_a + mu_b));
            end massFlowRate_dp;
            function extends pressureLoss_m_flow "Return pressure loss dp as function of mass flow rate m_flow, i.e., dp = f(m_flow), due to wall friction"
              annotation(Documentation(info = "<html>

</html>"));
            algorithm 
              dp:=(m_flow * 128 * length * (mu_a + mu_b)) / (Modelica.Constants.pi * diameter ^ 4 * (rho_a + rho_b));
            end pressureLoss_m_flow;
            function extends massFlowRate_dp_staticHead "Return mass flow rate m_flow as function of pressure loss dp, i.e., m_flow = f(dp), due to wall friction and static head"
              annotation(Documentation(info = "<html>

</html>"));
            protected 
              Real k0inv = (Modelica.Constants.pi * diameter ^ 4) / (128 * length) "Constant factor";
              Real dp_grav_a = g_times_height_ab * rho_a "Static head if mass flows in design direction (a to b)";
              Real dp_grav_b = g_times_height_ab * rho_b "Static head if mass flows against design direction (b to a)";
              Real dm_flow_ddp_fric_a = (k0inv * rho_a) / mu_a "Slope of mass flow rate over dp if flow in design direction (a to b)";
              Real dm_flow_ddp_fric_b = (k0inv * rho_b) / mu_b "Slope of mass flow rate over dp if flow against design direction (b to a)";
              Real dp_a = max(dp_grav_a, dp_grav_b) + dp_small "Upper end of regularization domain of the m_flow(dp) relation";
              Real dp_b = min(dp_grav_a, dp_grav_b) - dp_small "Lower end of regularization domain of the m_flow(dp) relation";
              SI.MassFlowRate m_flow_a "Value at upper end of regularization domain";
              SI.MassFlowRate m_flow_b "Value at lower end of regularization domain";
              SI.MassFlowRate m_flow_zero = 0;
              SI.Pressure dp_zero = (dp_grav_a + dp_grav_b) / 2;
              Real dm_flow_ddp_fric_zero;
            algorithm 
              if dp >= dp_a then 
                            m_flow:=dm_flow_ddp_fric_a * (dp - dp_grav_a);
              elseif dp <= dp_b then
                m_flow:=dm_flow_ddp_fric_b * (dp - dp_grav_b);


              else               m_flow_a:=dm_flow_ddp_fric_a * (dp_a - dp_grav_a);
              m_flow_b:=dm_flow_ddp_fric_b * (dp_b - dp_grav_b);
              (m_flow,dm_flow_ddp_fric_zero):=Utilities.regFun3(dp_zero, dp_b, dp_a, m_flow_b, m_flow_a, dm_flow_ddp_fric_b, dm_flow_ddp_fric_a);
              if dp > dp_zero then 
                            m_flow:=Utilities.regFun3(dp, dp_zero, dp_a, m_flow_zero, m_flow_a, dm_flow_ddp_fric_zero, dm_flow_ddp_fric_a);

              else               m_flow:=Utilities.regFun3(dp, dp_b, dp_zero, m_flow_b, m_flow_zero, dm_flow_ddp_fric_b, dm_flow_ddp_fric_zero);

              end if;

              end if;
            end massFlowRate_dp_staticHead;
            function extends pressureLoss_m_flow_staticHead "Return pressure loss dp as function of mass flow rate m_flow, i.e., dp = f(m_flow), due to wall friction and static head"
              annotation(Documentation(info = "<html>

</html>"));
            protected 
              Real k0 = (128 * length) / (Modelica.Constants.pi * diameter ^ 4) "Constant factor";
              Real dp_grav_a = g_times_height_ab * rho_a "Static head if mass flows in design direction (a to b)";
              Real dp_grav_b = g_times_height_ab * rho_b "Static head if mass flows against design direction (b to a)";
              Real ddp_dm_flow_a = (k0 * mu_a) / rho_a "Slope of dp over mass flow rate if flow in design direction (a to b)";
              Real ddp_dm_flow_b = (k0 * mu_b) / rho_b "Slope of dp over mass flow rate if flow against design direction (b to a)";
              Real m_flow_a = if dp_grav_a >= dp_grav_b then m_flow_small else m_flow_small + (dp_grav_b - dp_grav_a) / ddp_dm_flow_a "Upper end of regularization domain of the dp(m_flow) relation";
              Real m_flow_b = if dp_grav_a >= dp_grav_b then  -m_flow_small else  -m_flow_small - (dp_grav_b - dp_grav_a) / ddp_dm_flow_b "Lower end of regularization domain of the dp(m_flow) relation";
              SI.Pressure dp_a "Value at upper end of regularization domain";
              SI.Pressure dp_b "Value at lower end of regularization domain";
              SI.MassFlowRate m_flow_zero = 0;
              SI.Pressure dp_zero = (dp_grav_a + dp_grav_b) / 2;
              Real ddp_dm_flow_zero;
            algorithm 
              if m_flow >= m_flow_a then 
                            dp:=ddp_dm_flow_a * m_flow + dp_grav_a;
              elseif m_flow <= m_flow_b then
                dp:=ddp_dm_flow_b * m_flow + dp_grav_b;


              else               dp_a:=ddp_dm_flow_a * m_flow_a + dp_grav_a;
              dp_b:=ddp_dm_flow_b * m_flow_b + dp_grav_b;
              (dp,ddp_dm_flow_zero):=Utilities.regFun3(m_flow_zero, m_flow_b, m_flow_a, dp_b, dp_a, ddp_dm_flow_b, ddp_dm_flow_a);
              if m_flow > m_flow_zero then 
                            dp:=Utilities.regFun3(m_flow, m_flow_zero, m_flow_a, dp_zero, dp_a, ddp_dm_flow_zero, ddp_dm_flow_a);

              else               dp:=Utilities.regFun3(m_flow, m_flow_b, m_flow_zero, dp_b, dp_zero, ddp_dm_flow_b, ddp_dm_flow_zero);

              end if;

              end if;
            end pressureLoss_m_flow_staticHead;
          end Laminar;
          package QuadraticTurbulent "Pipe wall friction in the quadratic turbulent regime (simple characteristic, mu not used)"
            annotation(Documentation(info = "<html>
<p>
This component defines only the quadratic turbulent regime of wall friction:
dp = k*m_flow*|m_flow|, where \"k\" depends on density and the roughness
of the pipe and is no longer a function of the Reynolds number.
This relationship is only valid for large Reynolds numbers.
</p>

<p>
In <a href=\"Modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.WallFriction\">UsersGuide</a> the complete friction regime is illustrated.
This component describes only the asymptotic behaviour for large
Reynolds numbers, i.e., the values at the right ordinate where
&lambda; is constant.
</p>
<br>

</html>"));
            extends PartialWallFriction(final use_mu = false, final use_roughness = true, final use_dp_small = true, final use_m_flow_small = true);
            function extends massFlowRate_dp "Return mass flow rate m_flow as function of pressure loss dp, i.e., m_flow = f(dp), due to wall friction"
              import Modelica.Math;
              annotation(smoothOrder = 1, Documentation(info = "<html>

</html>"));
            protected 
              constant Real pi = Modelica.Constants.pi;
              Real zeta;
              Real k_inv;
            algorithm 
              assert(roughness > 1e-10, "roughness > 0 required for quadratic turbulent wall friction characteristic");
              zeta:=length / diameter / (2 * Math.log10(3.7 / roughness / diameter)) ^ 2;
              k_inv:=(pi * diameter * diameter) ^ 2 / (8 * zeta);
              m_flow:=Modelica.Fluid.Utilities.regRoot2(dp, dp_small, rho_a * k_inv, rho_b * k_inv);
            end massFlowRate_dp;
            function extends pressureLoss_m_flow "Return pressure loss dp as function of mass flow rate m_flow, i.e., dp = f(m_flow), due to wall friction"
              import Modelica.Math;
              annotation(smoothOrder = 1, Documentation(info = "<html>

</html>"));
            protected 
              constant Real pi = Modelica.Constants.pi;
              Real zeta;
              Real k;
            algorithm 
              assert(roughness > 1e-10, "roughness > 0 required for quadratic turbulent wall friction characteristic");
              zeta:=length / diameter / (2 * Math.log10(3.7 / roughness / diameter)) ^ 2;
              k:=(8 * zeta) / (pi * diameter * diameter) ^ 2;
              dp:=Modelica.Fluid.Utilities.regSquare2(m_flow, m_flow_small, k / rho_a, k / rho_b);
            end pressureLoss_m_flow;
            function extends massFlowRate_dp_staticHead "Return mass flow rate m_flow as function of pressure loss dp, i.e., m_flow = f(dp), due to wall friction and static head"
              import Modelica.Math;
              annotation(smoothOrder = 1, Documentation(info = "<html>

</html>"));
            protected 
              constant Real pi = Modelica.Constants.pi;
              Real zeta = length / diameter / (2 * Math.log10(3.7 / roughness / diameter)) ^ 2;
              Real k_inv = (pi * diameter * diameter) ^ 2 / (8 * zeta);
              SI.Pressure dp_grav_a = g_times_height_ab * rho_a "Static head if mass flows in design direction (a to b)";
              SI.Pressure dp_grav_b = g_times_height_ab * rho_b "Static head if mass flows against design direction (b to a)";
              Real k1 = rho_a * k_inv "Factor in m_flow =  sqrt(k1*(dp-dp_grav_a))";
              Real k2 = rho_b * k_inv "Factor in m_flow = -sqrt(k2*|dp-dp_grav_b|)";
              Real dp_a = max(dp_grav_a, dp_grav_b) + dp_small "Upper end of regularization domain of the m_flow(dp) relation";
              Real dp_b = min(dp_grav_a, dp_grav_b) - dp_small "Lower end of regularization domain of the m_flow(dp) relation";
              SI.MassFlowRate m_flow_a "Value at upper end of regularization domain";
              SI.MassFlowRate m_flow_b "Value at lower end of regularization domain";
              SI.MassFlowRate dm_flow_ddp_fric_a "Derivative at upper end of regularization domain";
              SI.MassFlowRate dm_flow_ddp_fric_b "Derivative at lower end of regularization domain";
              SI.MassFlowRate m_flow_zero = 0;
              SI.Pressure dp_zero = (dp_grav_a + dp_grav_b) / 2;
              Real dm_flow_ddp_fric_zero;
            algorithm 
              assert(roughness > 1e-10, "roughness > 0 required for quadratic turbulent wall friction characteristic");
              if dp >= dp_a then 
                            m_flow:=sqrt(k1 * (dp - dp_grav_a));
              elseif dp <= dp_b then
                m_flow:= -sqrt(k2 * abs(dp - dp_grav_b));


              else               m_flow_a:=sqrt(k1 * (dp_a - dp_grav_a));
              m_flow_b:= -sqrt(k2 * abs(dp_b - dp_grav_b));
              dm_flow_ddp_fric_a:=k1 / (2 * sqrt(k1 * (dp_a - dp_grav_a)));
              dm_flow_ddp_fric_b:=k2 / (2 * sqrt(k2 * abs(dp_b - dp_grav_b)));
              (m_flow,dm_flow_ddp_fric_zero):=Utilities.regFun3(dp_zero, dp_b, dp_a, m_flow_b, m_flow_a, dm_flow_ddp_fric_b, dm_flow_ddp_fric_a);
              if dp > dp_zero then 
                            m_flow:=Utilities.regFun3(dp, dp_zero, dp_a, m_flow_zero, m_flow_a, dm_flow_ddp_fric_zero, dm_flow_ddp_fric_a);

              else               m_flow:=Utilities.regFun3(dp, dp_b, dp_zero, m_flow_b, m_flow_zero, dm_flow_ddp_fric_b, dm_flow_ddp_fric_zero);

              end if;

              end if;
            end massFlowRate_dp_staticHead;
            function extends pressureLoss_m_flow_staticHead "Return pressure loss dp as function of mass flow rate m_flow, i.e., dp = f(m_flow), due to wall friction and static head"
              import Modelica.Math;
              annotation(smoothOrder = 1, Documentation(info = "<html>

</html>"));
            protected 
              constant Real pi = Modelica.Constants.pi;
              Real zeta = length / diameter / (2 * Math.log10(3.7 / roughness / diameter)) ^ 2;
              Real k = (8 * zeta) / (pi * diameter * diameter) ^ 2;
              SI.Pressure dp_grav_a = g_times_height_ab * rho_a "Static head if mass flows in design direction (a to b)";
              SI.Pressure dp_grav_b = g_times_height_ab * rho_b "Static head if mass flows against design direction (b to a)";
              Real k1 = k / rho_a "If m_flow >= 0 then dp = k1*m_flow^2 + dp_grav_a";
              Real k2 = k / rho_b "If m_flow < 0 then dp = -k2*m_flow^2 + dp_grav_b";
              Real m_flow_a = if dp_grav_a >= dp_grav_b then m_flow_small else m_flow_small + sqrt((dp_grav_b - dp_grav_a) / k1) "Upper end of regularization domain of the dp(m_flow) relation";
              Real m_flow_b = if dp_grav_a >= dp_grav_b then  -m_flow_small else  -m_flow_small - sqrt((dp_grav_b - dp_grav_a) / k2) "Lower end of regularization domain of the dp(m_flow) relation";
              SI.Pressure dp_a "Value at upper end of regularization domain";
              SI.Pressure dp_b "Value at lower end of regularization domain";
              Real ddp_dm_flow_a "Derivative of pressure drop with mass flow rate at m_flow_a";
              Real ddp_dm_flow_b "Derivative of pressure drop with mass flow rate at m_flow_b";
              SI.MassFlowRate m_flow_zero = 0;
              SI.Pressure dp_zero = (dp_grav_a + dp_grav_b) / 2;
              Real ddp_dm_flow_zero;
            algorithm 
              assert(roughness > 1e-10, "roughness > 0 required for quadratic turbulent wall friction characteristic");
              if m_flow >= m_flow_a then 
                            dp:=k1 * m_flow ^ 2 + dp_grav_a;
              elseif m_flow <= m_flow_b then
                dp:= -k2 * m_flow ^ 2 + dp_grav_b;


              else               dp_a:=k1 * m_flow_a ^ 2 + dp_grav_a;
              ddp_dm_flow_a:=2 * k1 * m_flow_a;
              dp_b:= -k2 * m_flow_b ^ 2 + dp_grav_b;
              ddp_dm_flow_b:= -2 * k2 * m_flow_b;
              (dp,ddp_dm_flow_zero):=Utilities.regFun3(m_flow_zero, m_flow_b, m_flow_a, dp_b, dp_a, ddp_dm_flow_b, ddp_dm_flow_a);
              if m_flow > m_flow_zero then 
                            dp:=Utilities.regFun3(m_flow, m_flow_zero, m_flow_a, dp_zero, dp_a, ddp_dm_flow_zero, ddp_dm_flow_a);

              else               dp:=Utilities.regFun3(m_flow, m_flow_b, m_flow_zero, dp_b, dp_zero, ddp_dm_flow_b, ddp_dm_flow_zero);

              end if;

              end if;
            end pressureLoss_m_flow_staticHead;
          end QuadraticTurbulent;
          package LaminarAndQuadraticTurbulent "Pipe wall friction in the laminar and quadratic turbulent regime (simple characteristic)"
            annotation(Documentation(info = "<html>
<p>
This component defines the quadratic turbulent regime of wall friction:
dp = k*m_flow*|m_flow|, where \"k\" depends on density and the roughness
of the pipe and is no longer a function of the Reynolds number.
This relationship is only valid for large Reynolds numbers.
At Re=4000, a polynomial is constructed that approaches
the constant &lambda; (for large Reynolds-numbers) at Re=4000
smoothly and has a derivative at zero mass flow rate that is
identical to laminar wall friction.
</p>
</html>"));
            extends PartialWallFriction(final use_mu = true, final use_roughness = true, final use_dp_small = true, final use_m_flow_small = true);
            import ln = Modelica.Math.log;
            import Modelica.Math.log10;
            import Modelica.Math.exp;
            import Modelica.Constants.pi;
            function extends massFlowRate_dp "Return mass flow rate m_flow as function of pressure loss dp, i.e., m_flow = f(dp), due to wall friction"
              import Modelica.Math;
              annotation(smoothOrder = 1, Documentation(info = "<html>

</html>"));
            protected 
              constant Real pi = Modelica.Constants.pi;
              constant Real Re_turbulent = 4000 "Start of turbulent regime";
              Real zeta;
              Real k0;
              Real k_inv;
              Real yd0 "Derivative of m_flow=m_flow(dp) at zero";
              SI.AbsolutePressure dp_turbulent;
            algorithm 
              assert(roughness > 1e-10, "roughness > 0 required for quadratic turbulent wall friction characteristic");
              zeta:=length / diameter / (2 * Math.log10(3.7 / roughness / diameter)) ^ 2;
              k0:=(128 * length) / (pi * diameter ^ 4);
              k_inv:=(pi * diameter * diameter) ^ 2 / (8 * zeta);
              yd0:=(rho_a + rho_b) / (k0 * (mu_a + mu_b));
              dp_turbulent:=((((mu_a + mu_b) * diameter * pi) / 8) ^ 2 * Re_turbulent ^ 2) / (k_inv * (rho_a + rho_b)) / 2;
              m_flow:=Modelica.Fluid.Utilities.regRoot2(dp, dp_turbulent, rho_a * k_inv, rho_b * k_inv, use_yd0 = true, yd0 = yd0);
            end massFlowRate_dp;
            function extends pressureLoss_m_flow "Return pressure loss dp as function of mass flow rate m_flow, i.e., dp = f(m_flow), due to wall friction"
              import Modelica.Math;
              annotation(smoothOrder = 1, Documentation(info = "<html>

</html>"));
            protected 
              constant Real pi = Modelica.Constants.pi;
              constant Real Re_turbulent = 4000 "Start of turbulent regime";
              Real zeta;
              Real k0;
              Real k;
              Real yd0 "Derivative of dp = f(m_flow) at zero";
              SI.MassFlowRate m_flow_turbulent "The turbulent region is: |m_flow| >= m_flow_turbulent";
            algorithm 
              assert(roughness > 1e-10, "roughness > 0 required for quadratic turbulent wall friction characteristic");
              zeta:=length / diameter / (2 * Math.log10(3.7 / roughness / diameter)) ^ 2;
              k0:=(128 * length) / (pi * diameter ^ 4);
              k:=(8 * zeta) / (pi * diameter * diameter) ^ 2;
              yd0:=(k0 * (mu_a + mu_b)) / (rho_a + rho_b);
              m_flow_turbulent:=pi / 8 * diameter * (mu_a + mu_b) * Re_turbulent;
              dp:=Modelica.Fluid.Utilities.regSquare2(m_flow, m_flow_turbulent, k / rho_a, k / rho_b, use_yd0 = true, yd0 = yd0);
            end pressureLoss_m_flow;
            function extends massFlowRate_dp_staticHead "Return mass flow rate m_flow as function of pressure loss dp, i.e., m_flow = f(dp), due to wall friction and static head"
              import Modelica.Math;
              annotation(smoothOrder = 1, Documentation(info = "<html>

</html>"));
            protected 
              Real Delta = roughness / diameter "Relative roughness";
              SI.ReynoldsNumber Re1 = 745 * exp(if Delta <= 0.0065 then 1 else 0.0065 / Delta) "Boundary between laminar regime and transition";
              constant SI.ReynoldsNumber Re2 = 4000 "Boundary between transition and turbulent regime";
              SI.Pressure dp_a "Upper end of regularization domain of the m_flow(dp) relation";
              SI.Pressure dp_b "Lower end of regularization domain of the m_flow(dp) relation";
              SI.MassFlowRate m_flow_a "Value at upper end of regularization domain";
              SI.MassFlowRate m_flow_b "Value at lower end of regularization domain";
              SI.MassFlowRate dm_flow_ddp_fric_a "Derivative at upper end of regularization domain";
              SI.MassFlowRate dm_flow_ddp_fric_b "Derivative at lower end of regularization domain";
              SI.Pressure dp_grav_a = g_times_height_ab * rho_a "Static head if mass flows in design direction (a to b)";
              SI.Pressure dp_grav_b = g_times_height_ab * rho_b "Static head if mass flows against design direction (b to a)";
              SI.MassFlowRate m_flow_zero = 0;
              SI.Pressure dp_zero = (dp_grav_a + dp_grav_b) / 2;
              Real dm_flow_ddp_fric_zero;
            algorithm 
              assert(roughness > 1e-10, "roughness > 0 required for quadratic turbulent wall friction characteristic");
              dp_a:=max(dp_grav_a, dp_grav_b) + dp_small;
              dp_b:=min(dp_grav_a, dp_grav_b) - dp_small;
              if dp >= dp_a then 
                            m_flow:=Internal.m_flow_of_dp_fric(dp - dp_grav_a, rho_a, rho_b, mu_a, mu_b, length, diameter, Re1, Re2, Delta);
              elseif dp <= dp_b then
                m_flow:=Internal.m_flow_of_dp_fric(dp - dp_grav_b, rho_a, rho_b, mu_a, mu_b, length, diameter, Re1, Re2, Delta);


              else               (m_flow_a,dm_flow_ddp_fric_a):=Internal.m_flow_of_dp_fric(dp_a - dp_grav_a, rho_a, rho_b, mu_a, mu_b, length, diameter, Re1, Re2, Delta);
              (m_flow_b,dm_flow_ddp_fric_b):=Internal.m_flow_of_dp_fric(dp_b - dp_grav_b, rho_a, rho_b, mu_a, mu_b, length, diameter, Re1, Re2, Delta);
              (m_flow,dm_flow_ddp_fric_zero):=Utilities.regFun3(dp_zero, dp_b, dp_a, m_flow_b, m_flow_a, dm_flow_ddp_fric_b, dm_flow_ddp_fric_a);
              if dp > dp_zero then 
                            m_flow:=Utilities.regFun3(dp, dp_zero, dp_a, m_flow_zero, m_flow_a, dm_flow_ddp_fric_zero, dm_flow_ddp_fric_a);

              else               m_flow:=Utilities.regFun3(dp, dp_b, dp_zero, m_flow_b, m_flow_zero, dm_flow_ddp_fric_b, dm_flow_ddp_fric_zero);

              end if;

              end if;
            end massFlowRate_dp_staticHead;
            function extends pressureLoss_m_flow_staticHead "Return pressure loss dp as function of mass flow rate m_flow, i.e., dp = f(m_flow), due to wall friction and static head"
              import Modelica.Math;
              annotation(smoothOrder = 1, Documentation(info = "<html>

</html>"));
            protected 
              Real Delta = roughness / diameter "Relative roughness";
              SI.ReynoldsNumber Re1 = 745 * exp(if Delta <= 0.0065 then 1 else 0.0065 / Delta) "Boundary between laminar regime and transition";
              constant SI.ReynoldsNumber Re2 = 4000 "Boundary between transition and turbulent regime";
              SI.MassFlowRate m_flow_a "Upper end of regularization domain of the dp(m_flow) relation";
              SI.MassFlowRate m_flow_b "Lower end of regularization domain of the dp(m_flow) relation";
              SI.Pressure dp_a "Value at upper end of regularization domain";
              SI.Pressure dp_b "Value at lower end of regularization domain";
              SI.Pressure dp_grav_a = g_times_height_ab * rho_a "Static head if mass flows in design direction (a to b)";
              SI.Pressure dp_grav_b = g_times_height_ab * rho_b "Static head if mass flows against design direction (b to a)";
              Real ddp_dm_flow_a "Derivative of pressure drop with mass flow rate at m_flow_a";
              Real ddp_dm_flow_b "Derivative of pressure drop with mass flow rate at m_flow_b";
              SI.MassFlowRate m_flow_zero = 0;
              SI.Pressure dp_zero = (dp_grav_a + dp_grav_b) / 2;
              Real ddp_dm_flow_zero;
            algorithm 
              assert(roughness > 1e-10, "roughness > 0 required for quadratic turbulent wall friction characteristic");
              m_flow_a:=if dp_grav_a < dp_grav_b then Internal.m_flow_of_dp_fric(dp_grav_b - dp_grav_a, rho_a, rho_b, mu_a, mu_b, length, diameter, Re1, Re2, Delta) + m_flow_small else m_flow_small;
              m_flow_b:=if dp_grav_a < dp_grav_b then Internal.m_flow_of_dp_fric(dp_grav_a - dp_grav_b, rho_a, rho_b, mu_a, mu_b, length, diameter, Re1, Re2, Delta) - m_flow_small else  -m_flow_small;
              if m_flow >= m_flow_a then 
                            dp:=Internal.dp_fric_of_m_flow(m_flow, rho_a, rho_b, mu_a, mu_b, length, diameter, Re1, Re2, Delta) + dp_grav_a;
              elseif m_flow <= m_flow_b then
                dp:=Internal.dp_fric_of_m_flow(m_flow, rho_a, rho_b, mu_a, mu_b, length, diameter, Re1, Re2, Delta) + dp_grav_b;


              else               (dp_a,ddp_dm_flow_a):=Internal.dp_fric_of_m_flow(m_flow_a, rho_a, rho_b, mu_a, mu_b, length, diameter, Re1, Re2, Delta);
              dp_a:=dp_a + dp_grav_a "Adding dp_grav to dp_fric to get dp";
              (dp_b,ddp_dm_flow_b):=Internal.dp_fric_of_m_flow(m_flow_b, rho_a, rho_b, mu_a, mu_b, length, diameter, Re1, Re2, Delta);
              dp_b:=dp_b + dp_grav_b "Adding dp_grav to dp_fric to get dp";
              (dp,ddp_dm_flow_zero):=Utilities.regFun3(m_flow_zero, m_flow_b, m_flow_a, dp_b, dp_a, ddp_dm_flow_b, ddp_dm_flow_a);
              if m_flow > m_flow_zero then 
                            dp:=Utilities.regFun3(m_flow, m_flow_zero, m_flow_a, dp_zero, dp_a, ddp_dm_flow_zero, ddp_dm_flow_a);

              else               dp:=Utilities.regFun3(m_flow, m_flow_b, m_flow_zero, dp_b, dp_zero, ddp_dm_flow_b, ddp_dm_flow_zero);

              end if;

              end if;
            end pressureLoss_m_flow_staticHead;
            package Internal "Functions to calculate mass flow rate from friction pressure drop and vice versa"
              function m_flow_of_dp_fric "Calculate mass flow rate as function of pressure drop due to friction"
                input SI.Pressure dp_fric "Pressure loss due to friction (dp = port_a.p - port_b.p)";
                input SI.Density rho_a "Density at port_a";
                input SI.Density rho_b "Density at port_b";
                input SI.DynamicViscosity mu_a "Dynamic viscosity at port_a (dummy if use_mu = false)";
                input SI.DynamicViscosity mu_b "Dynamic viscosity at port_b (dummy if use_mu = false)";
                input SI.Length length "Length of pipe";
                input SI.Diameter diameter "Inner (hydraulic) diameter of pipe";
                input SI.ReynoldsNumber Re1 "Boundary between laminar regime and transition";
                input SI.ReynoldsNumber Re2 "Boundary between transition and turbulent regime";
                input Real Delta "Relative roughness";
                output SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
                output Real dm_flow_ddp_fric "Derivative of mass flow rate with dp_fric";
                annotation(smoothOrder = 1);
              protected 
                SI.DynamicViscosity mu "Upstream viscosity";
                SI.Density rho "Upstream density";
                Real zeta;
                Real k0;
                Real k_inv;
                Real dm_flow_ddp_laminar "Derivative of m_flow=m_flow(dp) in laminar regime";
                SI.AbsolutePressure dp_fric_turbulent "The turbulent region is: |dp_fric| >= dp_fric_turbulent, simple quadratic correlation";
                SI.AbsolutePressure dp_fric_laminar "The laminar region is: |dp_fric| <= dp_fric_laminar";
              algorithm 
                if dp_fric >= 0 then 
                                rho:=rho_a;
                mu:=mu_a;

                else                 rho:=rho_b;
                mu:=mu_b;

                end if;
                zeta:=length / diameter / (2 * log10(3.7 / Delta)) ^ 2;
                k_inv:=(pi * diameter * diameter) ^ 2 / (8 * zeta);
                dp_fric_turbulent:=(sign(dp_fric) * ((mu * diameter * pi) / 4) ^ 2 * Re2 ^ 2) / (k_inv * rho);
                k0:=(128 * length) / (pi * diameter ^ 4);
                dm_flow_ddp_laminar:=rho / (k0 * mu);
                dp_fric_laminar:=((sign(dp_fric) * pi * k0 * mu ^ 2) / rho * diameter) / 4 * Re1;
                if abs(dp_fric) > abs(dp_fric_turbulent) then 
                                m_flow:=sign(dp_fric) * sqrt(rho * k_inv * abs(dp_fric));
                dm_flow_ddp_fric:=0.5 * rho * k_inv * (rho * k_inv * abs(dp_fric)) ^ ( -0.5);
                elseif abs(dp_fric) < abs(dp_fric_laminar) then
                  m_flow:=dm_flow_ddp_laminar * dp_fric;
                  dm_flow_ddp_fric:=dm_flow_ddp_laminar;


                else                 (m_flow,dm_flow_ddp_fric):=Utilities.cubicHermite_withDerivative(dp_fric, dp_fric_laminar, dp_fric_turbulent, dm_flow_ddp_laminar * dp_fric_laminar, sign(dp_fric_turbulent) * sqrt(rho * k_inv * abs(dp_fric_turbulent)), dm_flow_ddp_laminar, if abs(dp_fric_turbulent) > 0 then 0.5 * rho * k_inv * (rho * k_inv * abs(dp_fric_turbulent)) ^ ( -0.5) else Modelica.Constants.inf);

                end if;
              end m_flow_of_dp_fric;
              function dp_fric_of_m_flow "Calculate pressure drop due to friction as function of mass flow rate"
                input SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
                input SI.Density rho_a "Density at port_a";
                input SI.Density rho_b "Density at port_b";
                input SI.DynamicViscosity mu_a "Dynamic viscosity at port_a (dummy if use_mu = false)";
                input SI.DynamicViscosity mu_b "Dynamic viscosity at port_b (dummy if use_mu = false)";
                input SI.Length length "Length of pipe";
                input SI.Diameter diameter "Inner (hydraulic) diameter of pipe";
                input SI.ReynoldsNumber Re1 "Boundary between laminar regime and transition";
                input SI.ReynoldsNumber Re2 "Boundary between transition and turbulent regime";
                input Real Delta "Relative roughness";
                output SI.Pressure dp_fric "Pressure loss due to friction (dp_fric = port_a.p - port_b.p - dp_grav)";
                output Real ddp_fric_dm_flow "Derivative of pressure drop with mass flow rate";
                annotation(smoothOrder = 1);
              protected 
                SI.DynamicViscosity mu "Upstream viscosity";
                SI.Density rho "Upstream density";
                Real zeta;
                Real k0;
                Real k;
                Real ddp_fric_dm_flow_laminar "Derivative of dp_fric = f(m_flow) at zero";
                SI.MassFlowRate m_flow_turbulent "The turbulent region is: |m_flow| >= m_flow_turbulent";
                SI.MassFlowRate m_flow_laminar "The laminar region is: |m_flow| <= m_flow_laminar";
              algorithm 
                if m_flow >= 0 then 
                                rho:=rho_a;
                mu:=mu_a;

                else                 rho:=rho_b;
                mu:=mu_b;

                end if;
                zeta:=length / diameter / (2 * log10(3.7 / Delta)) ^ 2;
                k:=(8 * zeta) / (pi * diameter * diameter) ^ 2;
                m_flow_turbulent:=sign(m_flow) * pi / 4 * diameter * mu * Re2;
                k0:=(128 * length) / (pi * diameter ^ 4);
                ddp_fric_dm_flow_laminar:=(k0 * mu) / rho;
                m_flow_laminar:=sign(m_flow) * pi / 4 * diameter * mu * Re1;
                if abs(m_flow) > abs(m_flow_turbulent) then 
                                dp_fric:=k / rho * m_flow * abs(m_flow);
                ddp_fric_dm_flow:=(2 * k) / rho * abs(m_flow);
                elseif abs(m_flow) < abs(m_flow_laminar) then
                  dp_fric:=ddp_fric_dm_flow_laminar * m_flow;
                  ddp_fric_dm_flow:=ddp_fric_dm_flow_laminar;


                else                 (dp_fric,ddp_fric_dm_flow):=Utilities.cubicHermite_withDerivative(m_flow, m_flow_laminar, m_flow_turbulent, ddp_fric_dm_flow_laminar * m_flow_laminar, k / rho * m_flow_turbulent * abs(m_flow_turbulent), ddp_fric_dm_flow_laminar, (2 * k) / rho * abs(m_flow_turbulent));

                end if;
              end dp_fric_of_m_flow;
            end Internal;
          end LaminarAndQuadraticTurbulent;
          package Detailed "Pipe wall friction in the whole regime (detailed characteristic)"
            annotation(Documentation(info = "<html>
<p>
This component defines the complete regime of wall friction.
The details are described in the
<a href=\"Modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.WallFriction\">UsersGuide</a>.
The functional relationship of the friction loss factor &lambda; is
displayed in the next figure. Function massFlowRate_dp() defines the \"red curve\"
(\"Swamee and Jain\"), where as function pressureLoss_m_flow() defines the
\"blue curve\" (\"Colebrook-White\"). The two functions are inverses from
each other and give slightly different results in the transition region
between Re = 1500 .. 4000, in order to get explicit equations without
solving a non-linear equation.
</p>

<img src=\"../Images/Fluid/Components/PipeFriction1.png\">

<p>
Additionally to wall friction, this component properly implements static
head. With respect to the latter, two cases can be distinguished. In the case
shown next, the change of elevation with the path from a to b has the opposite
sign of the change of density.</p>

<img src=\"../Images/Fluid/Components/PipeFrictionStaticHead_case-a.PNG\">

<p>
In the case illustrated second, the change of elevation with the path from a to
b has the same sign of the change of density.</p>

<img src=\"../Images/Fluid/Components/PipeFrictionStaticHead_case-b.PNG\">


</html>"));
            extends PartialWallFriction(final use_mu = true, final use_roughness = true, final use_dp_small = true, final use_m_flow_small = true);
            import ln = Modelica.Math.log;
            import Modelica.Math.log10;
            import Modelica.Math.exp;
            import Modelica.Constants.pi;
            function extends massFlowRate_dp "Return mass flow rate m_flow as function of pressure loss dp, i.e., m_flow = f(dp), due to wall friction"
              import Modelica.Math;
              annotation(smoothOrder = 1, Documentation(info = "<html>

</html>"));
            protected 
              constant Real pi = Modelica.Constants.pi;
              Real Delta = roughness / diameter "Relative roughness";
              SI.ReynoldsNumber Re1 = (745 * Math.exp(if Delta <= 0.0065 then 1 else 0.0065 / Delta)) ^ 0.97 "Re leaving laminar curve";
              SI.ReynoldsNumber Re2 = 4000 "Re entering turbulent curve";
              SI.DynamicViscosity mu "Upstream viscosity";
              SI.Density rho "Upstream density";
              SI.ReynoldsNumber Re "Reynolds number";
              Real lambda2 "Modified friction coefficient (= lambda*Re^2)";
              function interpolateInRegion2
                input Real Re_turbulent;
                input SI.ReynoldsNumber Re1;
                input SI.ReynoldsNumber Re2;
                input Real Delta;
                input Real lambda2;
                output SI.ReynoldsNumber Re;
                annotation(smoothOrder = 1);
              protected 
                Real x1 = Math.log10(64 * Re1);
                Real y1 = Math.log10(Re1);
                Real yd1 = 1;
                Real aux1 = 0.5 / Math.log(10) * 5.74 * 0.9;
                Real aux2 = Delta / 3.7 + 5.74 / Re2 ^ 0.9;
                Real aux3 = Math.log10(aux2);
                Real L2 = 0.25 * (Re2 / aux3) ^ 2;
                Real aux4 = 2.51 / sqrt(L2) + 0.27 * Delta;
                Real aux5 =  -2 * sqrt(L2) * Math.log10(aux4);
                Real x2 = Math.log10(L2);
                Real y2 = Math.log10(aux5);
                Real yd2 = 0.5 + 2.51 / Math.log(10) / (aux5 * aux4);
                Real diff_x = x2 - x1;
                Real m = (y2 - y1) / diff_x;
                Real c2 = (3 * m - 2 * yd1 - yd2) / diff_x;
                Real c3 = (yd1 + yd2 - 2 * m) / (diff_x * diff_x);
                Real lambda2_1 = 64 * Re1;
                Real dx;
              algorithm 
                dx:=Math.log10(lambda2 / lambda2_1);
                Re:=Re1 * (lambda2 / lambda2_1) ^ (1 + dx * (c2 + dx * c3));
              end interpolateInRegion2;
            algorithm 
              rho:=if dp >= 0 then rho_a else rho_b;
              mu:=if dp >= 0 then mu_a else mu_b;
              lambda2:=(abs(dp) * 2 * diameter ^ 3 * rho) / (length * mu * mu);
              Re:=lambda2 / 64;
              if Re > Re1 then 
                            Re:= -2 * sqrt(lambda2) * Math.log10(2.51 / sqrt(lambda2) + 0.27 * Delta);
              if Re < Re2 then 
                            Re:=interpolateInRegion2(Re, Re1, Re2, Delta, lambda2);

              else 
              end if;

              else 
              end if;
              m_flow:=(pi * diameter) / 4 * mu * (if dp >= 0 then Re else  -Re);
            end massFlowRate_dp;
            function extends pressureLoss_m_flow "Return pressure loss dp as function of mass flow rate m_flow, i.e., dp = f(m_flow), due to wall friction"
              import Modelica.Math;
              annotation(smoothOrder = 1, Documentation(info = "<html>

</html>"));
            protected 
              constant Real pi = Modelica.Constants.pi;
              Real Delta = roughness / diameter "Relative roughness";
              SI.ReynoldsNumber Re1 = 745 * Math.exp(if Delta <= 0.0065 then 1 else 0.0065 / Delta) "Re leaving laminar curve";
              SI.ReynoldsNumber Re2 = 4000 "Re entering turbulent curve";
              SI.DynamicViscosity mu "Upstream viscosity";
              SI.Density rho "Upstream density";
              SI.ReynoldsNumber Re "Reynolds number";
              Real lambda2 "Modified friction coefficient (= lambda*Re^2)";
              function interpolateInRegion2
                input SI.ReynoldsNumber Re;
                input SI.ReynoldsNumber Re1;
                input SI.ReynoldsNumber Re2;
                input Real Delta;
                output Real lambda2;
                annotation(smoothOrder = 1);
              protected 
                Real x1 = Math.log10(Re1);
                Real y1 = Math.log10(64 * Re1);
                Real yd1 = 1;
                Real aux1 = 0.5 / Math.log(10) * 5.74 * 0.9;
                Real aux2 = Delta / 3.7 + 5.74 / Re2 ^ 0.9;
                Real aux3 = Math.log10(aux2);
                Real L2 = 0.25 * (Re2 / aux3) ^ 2;
                Real aux4 = 2.51 / sqrt(L2) + 0.27 * Delta;
                Real aux5 =  -2 * sqrt(L2) * Math.log10(aux4);
                Real x2 = Math.log10(Re2);
                Real y2 = Math.log10(L2);
                Real yd2 = 2 + (4 * aux1) / (aux2 * aux3 * Re2 ^ 0.9);
                Real diff_x = x2 - x1;
                Real m = (y2 - y1) / diff_x;
                Real c2 = (3 * m - 2 * yd1 - yd2) / diff_x;
                Real c3 = (yd1 + yd2 - 2 * m) / (diff_x * diff_x);
                Real dx;
              algorithm 
                dx:=Math.log10(Re / Re1);
                lambda2:=64 * Re1 * (Re / Re1) ^ (1 + dx * (c2 + dx * c3));
              end interpolateInRegion2;
            algorithm 
              rho:=if m_flow >= 0 then rho_a else rho_b;
              mu:=if m_flow >= 0 then mu_a else mu_b;
              Re:=(4 / pi * abs(m_flow)) / (diameter * mu);
              lambda2:=if Re <= Re1 then 64 * Re else if Re >= Re2 then 0.25 * (Re / Math.log10(Delta / 3.7 + 5.74 / Re ^ 0.9)) ^ 2 else interpolateInRegion2(Re, Re1, Re2, Delta);
              dp:=(length * mu * mu) / (2 * rho * diameter * diameter * diameter) * (if m_flow >= 0 then lambda2 else  -lambda2);
            end pressureLoss_m_flow;
            function extends massFlowRate_dp_staticHead "Return mass flow rate m_flow as function of pressure loss dp, i.e., m_flow = f(dp), due to wall friction and static head"
              annotation(smoothOrder = 1);
            protected 
              Real Delta = roughness / diameter "Relative roughness";
              SI.ReynoldsNumber Re "Reynolds number";
              SI.ReynoldsNumber Re1 = (745 * exp(if Delta <= 0.0065 then 1 else 0.0065 / Delta)) ^ 0.97 "Boundary between laminar regime and transition";
              constant SI.ReynoldsNumber Re2 = 4000 "Boundary between transition and turbulent regime";
              SI.Pressure dp_a "Upper end of regularization domain of the m_flow(dp) relation";
              SI.Pressure dp_b "Lower end of regularization domain of the m_flow(dp) relation";
              SI.MassFlowRate m_flow_a "Value at upper end of regularization domain";
              SI.MassFlowRate m_flow_b "Value at lower end of regularization domain";
              SI.MassFlowRate dm_flow_ddp_fric_a "Derivative at upper end of regularization domain";
              SI.MassFlowRate dm_flow_ddp_fric_b "Derivative at lower end of regularization domain";
              SI.Pressure dp_grav_a = g_times_height_ab * rho_a "Static head if mass flows in design direction (a to b)";
              SI.Pressure dp_grav_b = g_times_height_ab * rho_b "Static head if mass flows against design direction (b to a)";
              SI.MassFlowRate m_flow_zero = 0;
              SI.Pressure dp_zero = (dp_grav_a + dp_grav_b) / 2;
              Real dm_flow_ddp_fric_zero;
            algorithm 
              dp_a:=max(dp_grav_a, dp_grav_b) + dp_small;
              dp_b:=min(dp_grav_a, dp_grav_b) - dp_small;
              if dp >= dp_a then 
                            m_flow:=Internal.m_flow_of_dp_fric(dp - dp_grav_a, rho_a, rho_b, mu_a, mu_b, length, diameter, Re1, Re2, Delta);
              elseif dp <= dp_b then
                m_flow:=Internal.m_flow_of_dp_fric(dp - dp_grav_b, rho_a, rho_b, mu_a, mu_b, length, diameter, Re1, Re2, Delta);


              else               (m_flow_a,dm_flow_ddp_fric_a):=Internal.m_flow_of_dp_fric(dp_a - dp_grav_a, rho_a, rho_b, mu_a, mu_b, length, diameter, Re1, Re2, Delta);
              (m_flow_b,dm_flow_ddp_fric_b):=Internal.m_flow_of_dp_fric(dp_b - dp_grav_b, rho_a, rho_b, mu_a, mu_b, length, diameter, Re1, Re2, Delta);
              (m_flow,dm_flow_ddp_fric_zero):=Utilities.regFun3(dp_zero, dp_b, dp_a, m_flow_b, m_flow_a, dm_flow_ddp_fric_b, dm_flow_ddp_fric_a);
              if dp > dp_zero then 
                            m_flow:=Utilities.regFun3(dp, dp_zero, dp_a, m_flow_zero, m_flow_a, dm_flow_ddp_fric_zero, dm_flow_ddp_fric_a);

              else               m_flow:=Utilities.regFun3(dp, dp_b, dp_zero, m_flow_b, m_flow_zero, dm_flow_ddp_fric_b, dm_flow_ddp_fric_zero);

              end if;

              end if;
            end massFlowRate_dp_staticHead;
            function extends pressureLoss_m_flow_staticHead "Return pressure loss dp as function of mass flow rate m_flow, i.e., dp = f(m_flow), due to wall friction and static head"
              annotation(smoothOrder = 1);
            protected 
              Real Delta = roughness / diameter "Relative roughness";
              SI.ReynoldsNumber Re1 = 745 * exp(if Delta <= 0.0065 then 1 else 0.0065 / Delta) "Boundary between laminar regime and transition";
              constant SI.ReynoldsNumber Re2 = 4000 "Boundary between transition and turbulent regime";
              SI.MassFlowRate m_flow_a "Upper end of regularization domain of the dp(m_flow) relation";
              SI.MassFlowRate m_flow_b "Lower end of regularization domain of the dp(m_flow) relation";
              SI.Pressure dp_a "Value at upper end of regularization domain";
              SI.Pressure dp_b "Value at lower end of regularization domain";
              SI.Pressure dp_grav_a = g_times_height_ab * rho_a "Static head if mass flows in design direction (a to b)";
              SI.Pressure dp_grav_b = g_times_height_ab * rho_b "Static head if mass flows against design direction (b to a)";
              Real ddp_dm_flow_a "Derivative of pressure drop with mass flow rate at m_flow_a";
              Real ddp_dm_flow_b "Derivative of pressure drop with mass flow rate at m_flow_b";
              SI.MassFlowRate m_flow_zero = 0;
              SI.Pressure dp_zero = (dp_grav_a + dp_grav_b) / 2;
              Real ddp_dm_flow_zero;
            algorithm 
              m_flow_a:=if dp_grav_a < dp_grav_b then Internal.m_flow_of_dp_fric(dp_grav_b - dp_grav_a, rho_a, rho_b, mu_a, mu_b, length, diameter, Re1, Re2, Delta) + m_flow_small else m_flow_small;
              m_flow_b:=if dp_grav_a < dp_grav_b then Internal.m_flow_of_dp_fric(dp_grav_a - dp_grav_b, rho_a, rho_b, mu_a, mu_b, length, diameter, Re1, Re2, Delta) - m_flow_small else  -m_flow_small;
              if m_flow >= m_flow_a then 
                            dp:=Internal.dp_fric_of_m_flow(m_flow, rho_a, rho_b, mu_a, mu_b, length, diameter, Re1, Re2, Delta) + dp_grav_a;
              elseif m_flow <= m_flow_b then
                dp:=Internal.dp_fric_of_m_flow(m_flow, rho_a, rho_b, mu_a, mu_b, length, diameter, Re1, Re2, Delta) + dp_grav_b;


              else               (dp_a,ddp_dm_flow_a):=Internal.dp_fric_of_m_flow(m_flow_a, rho_a, rho_b, mu_a, mu_b, length, diameter, Re1, Re2, Delta);
              dp_a:=dp_a + dp_grav_a "Adding dp_grav to dp_fric to get dp";
              (dp_b,ddp_dm_flow_b):=Internal.dp_fric_of_m_flow(m_flow_b, rho_a, rho_b, mu_a, mu_b, length, diameter, Re1, Re2, Delta);
              dp_b:=dp_b + dp_grav_b "Adding dp_grav to dp_fric to get dp";
              (dp,ddp_dm_flow_zero):=Utilities.regFun3(m_flow_zero, m_flow_b, m_flow_a, dp_b, dp_a, ddp_dm_flow_b, ddp_dm_flow_a);
              if m_flow > m_flow_zero then 
                            dp:=Utilities.regFun3(m_flow, m_flow_zero, m_flow_a, dp_zero, dp_a, ddp_dm_flow_zero, ddp_dm_flow_a);

              else               dp:=Utilities.regFun3(m_flow, m_flow_b, m_flow_zero, dp_b, dp_zero, ddp_dm_flow_b, ddp_dm_flow_zero);

              end if;

              end if;
            end pressureLoss_m_flow_staticHead;
            package Internal "Functions to calculate mass flow rate from friction pressure drop and vice versa"
              function m_flow_of_dp_fric "Calculate mass flow rate as function of pressure drop due to friction"
                input SI.Pressure dp_fric "Pressure loss due to friction (dp = port_a.p - port_b.p)";
                input SI.Density rho_a "Density at port_a";
                input SI.Density rho_b "Density at port_b";
                input SI.DynamicViscosity mu_a "Dynamic viscosity at port_a (dummy if use_mu = false)";
                input SI.DynamicViscosity mu_b "Dynamic viscosity at port_b (dummy if use_mu = false)";
                input SI.Length length "Length of pipe";
                input SI.Diameter diameter "Inner (hydraulic) diameter of pipe";
                input SI.ReynoldsNumber Re1 "Boundary between laminar regime and transition";
                input SI.ReynoldsNumber Re2 "Boundary between transition and turbulent regime";
                input Real Delta "Relative roughness";
                output SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
                output Real dm_flow_ddp_fric "Derivative of mass flow rate with dp_fric";
                annotation(smoothOrder = 1);
              protected 
                function interpolateInRegion2_withDerivative "Interpolation in log-log space using a cubic Hermite polynomial, where x=log10(lambda2), y=log10(Re)"
                  input Real lambda2 "Known independent variable";
                  input SI.ReynoldsNumber Re1 "Boundary between laminar regime and transition";
                  input SI.ReynoldsNumber Re2 "Boundary between transition and turbulent regime";
                  input Real Delta "Relative roughness";
                  input SI.Pressure dp_fric "Pressure loss due to friction (dp = port_a.p - port_b.p)";
                  output SI.ReynoldsNumber Re "Unknown return variable";
                  output Real dRe_ddp "Derivative of return value";
                  annotation(smoothOrder = 1);
                protected 
                  Real x1 = log10(64 * Re1);
                  Real y1 = log10(Re1);
                  Real y1d = 1;
                  Real aux2 = Delta / 3.7 + 5.74 / Re2 ^ 0.9;
                  Real aux3 = log10(aux2);
                  Real L2 = 0.25 * (Re2 / aux3) ^ 2;
                  Real aux4 = 2.51 / sqrt(L2) + 0.27 * Delta;
                  Real aux5 =  -2 * sqrt(L2) * log10(aux4);
                  Real x2 = log10(L2);
                  Real y2 = log10(aux5);
                  Real y2d = 0.5 + 2.51 / ln(10) / (aux5 * aux4);
                  Real x = log10(lambda2);
                  Real y;
                  Real dy_dx "Derivative in transformed space";
                algorithm 
                  (y,dy_dx):=Utilities.cubicHermite_withDerivative(x, x1, x2, y1, y2, y1d, y2d);
                  Re:=10 ^ y;
                  dRe_ddp:=Re / abs(dp_fric) * dy_dx;
                end interpolateInRegion2_withDerivative;
                SI.DynamicViscosity mu "Upstream viscosity";
                SI.Density rho "Upstream density";
                Real lambda2 "Modified friction coefficient (= lambda*Re^2)";
                SI.ReynoldsNumber Re "Reynolds number";
                Real dRe_ddp "dRe/ddp";
                Real aux1;
                Real aux2;
              algorithm 
                if dp_fric >= 0 then 
                                rho:=rho_a;
                mu:=mu_a;

                else                 rho:=rho_b;
                mu:=mu_b;

                end if;
                lambda2:=(abs(dp_fric) * 2 * diameter ^ 3 * rho) / (length * mu * mu) "Known as lambda2=f(dp)";
                aux1:=(2 * diameter ^ 3 * rho) / (length * mu ^ 2);
                Re:=lambda2 / 64 "Hagen-Poiseuille";
                dRe_ddp:=aux1 / 64 "Hagen-Poiseuille";
                if Re > Re1 then 
                                Re:= -2 * sqrt(lambda2) * log10(2.51 / sqrt(lambda2) + 0.27 * Delta) "Colebrook-White";
                aux2:=sqrt(aux1 * abs(dp_fric));
                dRe_ddp:=1 / ln(10) * ( -(2 * ln(2.51 / aux2 + 0.27 * Delta) * aux1) / (2 * aux2) + (2 * 2.51) / (2 * abs(dp_fric) * (2.51 / aux2 + 0.27 * Delta)));
                if Re < Re2 then 
                                (Re,dRe_ddp):=interpolateInRegion2_withDerivative(lambda2, Re1, Re2, Delta, dp_fric);

                else 
                end if;

                else 
                end if;
                m_flow:=(pi * diameter) / 4 * mu * (if dp_fric >= 0 then Re else  -Re);
                dm_flow_ddp_fric:=(pi * diameter * mu) / 4 * dRe_ddp;
              end m_flow_of_dp_fric;
              function dp_fric_of_m_flow "Calculate pressure drop due to friction as function of mass flow rate"
                input SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
                input SI.Density rho_a "Density at port_a";
                input SI.Density rho_b "Density at port_b";
                input SI.DynamicViscosity mu_a "Dynamic viscosity at port_a (dummy if use_mu = false)";
                input SI.DynamicViscosity mu_b "Dynamic viscosity at port_b (dummy if use_mu = false)";
                input SI.Length length "Length of pipe";
                input SI.Diameter diameter "Inner (hydraulic) diameter of pipe";
                input SI.ReynoldsNumber Re1 "Boundary between laminar regime and transition";
                input SI.ReynoldsNumber Re2 "Boundary between transition and turbulent regime";
                input Real Delta "Relative roughness";
                output SI.Pressure dp_fric "Pressure loss due to friction (dp_fric = port_a.p - port_b.p - dp_grav)";
                output Real ddp_fric_dm_flow "Derivative of pressure drop with mass flow rate";
                annotation(smoothOrder = 1);
              protected 
                function interpolateInRegion2 "Interpolation in log-log space using a cubic Hermite polynomial, where x=log10(Re), y=log10(lambda2)"
                  input SI.ReynoldsNumber Re "Known independent variable";
                  input SI.ReynoldsNumber Re1 "Boundary between laminar regime and transition";
                  input SI.ReynoldsNumber Re2 "Boundary between transition and turbulent regime";
                  input Real Delta "Relative roughness";
                  input SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
                  output Real lambda2 "Unknown return value";
                  output Real dlambda2_dm_flow "Derivative of return value";
                  annotation(smoothOrder = 1);
                protected 
                  Real x1 = log10(Re1);
                  Real y1 = log10(64 * Re1);
                  Real y1d = 1;
                  Real aux2 = Delta / 3.7 + 5.74 / Re2 ^ 0.9;
                  Real aux3 = log10(aux2);
                  Real L2 = 0.25 * (Re2 / aux3) ^ 2;
                  Real x2 = log10(Re2);
                  Real y2 = log10(L2);
                  Real y2d = 2 + (2 * 5.74 * 0.9) / (ln(aux2) * Re2 ^ 0.9 * aux2);
                  Real x = log10(Re);
                  Real y;
                  Real dy_dx "Derivative in transformed space";
                algorithm 
                  (y,dy_dx):=Utilities.cubicHermite_withDerivative(x, x1, x2, y1, y2, y1d, y2d);
                  lambda2:=10 ^ y;
                  dlambda2_dm_flow:=lambda2 / abs(m_flow) * dy_dx;
                end interpolateInRegion2;
                SI.DynamicViscosity mu "Upstream viscosity";
                SI.Density rho "Upstream density";
                SI.ReynoldsNumber Re "Reynolds number";
                Real lambda2 "Modified friction coefficient (= lambda*Re^2)";
                Real dlambda2_dm_flow "dlambda2/dm_flow";
                Real aux1;
                Real aux2;
              algorithm 
                if m_flow >= 0 then 
                                rho:=rho_a;
                mu:=mu_a;

                else                 rho:=rho_b;
                mu:=mu_b;

                end if;
                Re:=(4 / pi * abs(m_flow)) / (diameter * mu);
                aux1:=4 / (pi * diameter * mu);
                if Re <= Re1 then 
                                lambda2:=64 * Re "Hagen-Poiseuille";
                dlambda2_dm_flow:=64 * aux1 "Hagen-Poiseuille";
                elseif Re >= Re2 then
                  lambda2:=0.25 * (Re / log10(Delta / 3.7 + 5.74 / Re ^ 0.9)) ^ 2 "Swamee-Jain";
                  aux2:=Delta / 3.7 + 5.74 / (aux1 * abs(m_flow)) ^ 0.9;
                  dlambda2_dm_flow:=0.5 * aux1 * Re * ln(10) ^ 2 * (1 / ln(aux2) ^ 2 + (5.74 * 0.9) / (ln(aux2) ^ 3 * Re ^ 0.9 * aux2)) "Swamee-Jain";


                else                 (lambda2,dlambda2_dm_flow):=interpolateInRegion2(Re, Re1, Re2, Delta, m_flow);

                end if;
                dp_fric:=(length * mu * mu) / (2 * rho * diameter * diameter * diameter) * (if m_flow >= 0 then lambda2 else  -lambda2);
                ddp_fric_dm_flow:=(length * mu ^ 2) / (2 * diameter ^ 3 * rho) * dlambda2_dm_flow;
              end dp_fric_of_m_flow;
            end Internal;
          end Detailed;
          model TestWallFrictionAndGravity "Pressure loss in pipe due to wall friction and gravity (only for test purposes; if needed use Pipes.StaticPipe instead)"
            extends Modelica.Fluid.Interfaces.PartialTwoPortTransport;
            replaceable package WallFriction = Modelica.Fluid.Pipes.BaseClasses.WallFriction.QuadraticTurbulent extends Modelica.Fluid.Pipes.BaseClasses.WallFriction.PartialWallFriction "Characteristic of wall friction" annotation(choicesAllMatching = true);
            parameter SI.Length length "Length of pipe";
            parameter SI.Diameter diameter "Inner (hydraulic) diameter of pipe";
            parameter SI.Length height_ab = 0.0 "Height(port_b) - Height(port_a)" annotation(Evaluate = true);
            parameter SI.Length roughness(min = 0) = 2.5e-05 "Absolute roughness of pipe (default = smooth steel pipe)" annotation(Dialog(enable = WallFriction.use_roughness));
            parameter Boolean use_nominal = false "= true, if mu_nominal and rho_nominal are used, otherwise computed from medium" annotation(Evaluate = true);
            parameter SI.DynamicViscosity mu_nominal = Medium.dynamicViscosity(Medium.setState_pTX(Medium.p_default, Medium.T_default, Medium.X_default)) "Nominal dynamic viscosity (e.g. mu_liquidWater = 1e-3, mu_air = 1.8e-5)" annotation(Dialog(enable = use_nominal));
            parameter SI.Density rho_nominal = Medium.density_pTX(Medium.p_default, Medium.T_default, Medium.X_default) "Nominal density (e.g. rho_liquidWater = 995, rho_air = 1.2)" annotation(Dialog(enable = use_nominal));
            parameter Boolean show_Re = false "= true, if Reynolds number is included for plotting" annotation(Evaluate = true, Dialog(tab = "Advanced"));
            parameter Boolean from_dp = true " = true, use m_flow = f(dp), otherwise dp = f(m_flow)" annotation(Evaluate = true, Dialog(tab = "Advanced"));
            parameter SI.AbsolutePressure dp_small = system.dp_small "Within regularization if |dp| < dp_small (may be wider for large discontinuities in static head)" annotation(Dialog(tab = "Advanced", enable = from_dp and WallFriction.use_dp_small));
            SI.ReynoldsNumber Re = Modelica.Fluid.Pipes.BaseClasses.CharacteristicNumbers.ReynoldsNumber_m_flow(m_flow, noEvent(if m_flow > 0 then mu_a else mu_b), diameter) if show_Re "Reynolds number of pipe flow";
          protected 
            SI.DynamicViscosity mu_a = if not WallFriction.use_mu then 1e-10 else if use_nominal then mu_nominal else Medium.dynamicViscosity(state_a);
            SI.DynamicViscosity mu_b = if not WallFriction.use_mu then 1e-10 else if use_nominal then mu_nominal else Medium.dynamicViscosity(state_b);
            SI.Density rho_a = if use_nominal then rho_nominal else Medium.density(state_a);
            SI.Density rho_b = if use_nominal then rho_nominal else Medium.density(state_b);
            Real g_times_height_ab(final unit = "m2/s2") = system.g * height_ab "Gravitiy times height_ab = dp_grav/d";
            final parameter Boolean use_x_small_staticHead = false "Use dp_/m_flow_small_staticHead only if static head actually exists" annotation(Evaluate = true);
            SI.AbsolutePressure dp_small_staticHead = noEvent(max(dp_small, 0.015 * abs(g_times_height_ab * (rho_a - rho_b)))) "Heuristic for large discontinuities in static head";
            SI.MassFlowRate m_flow_small_staticHead = noEvent(max(m_flow_small, ( -(5.55e-07 * (rho_a + rho_b)) / 2 + 0.00055) * abs(g_times_height_ab * (rho_a - rho_b)))) "Heuristic for large discontinuities in static head";

          equation 
            if from_dp and not WallFriction.dp_is_zero then
              m_flow = WallFriction.massFlowRate_dp_staticHead(dp, rho_a, rho_b, mu_a, mu_b, length, diameter, g_times_height_ab, roughness, if use_x_small_staticHead then dp_small_staticHead else dp_small);

            else
              dp = WallFriction.pressureLoss_m_flow_staticHead(m_flow, rho_a, rho_b, mu_a, mu_b, length, diameter, g_times_height_ab, roughness, if use_x_small_staticHead then m_flow_small_staticHead else m_flow_small);

            end if;
            port_a.h_outflow = inStream(port_b.h_outflow) + system.g * height_ab;
            port_b.h_outflow = inStream(port_a.h_outflow) - system.g * height_ab;
            annotation(defaultComponentName = "pipeFriction", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,60},{100, -60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -100,44},{100, -45}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,127,255}),Text(extent = {{ -150,80},{150,120}}, lineColor = {0,0,255}, textString = "%name")}), Documentation(info = "<html>
<p>
This model describes pressure losses due to <b>wall friction</b> in a pipe
and due to gravity.
It is assumed that no mass or energy is stored in the pipe.
Correlations of different complexity and validity can be
seleted via the replaceable package <b>WallFriction</b> (see parameter menu below).
The details of the pipe wall friction model are described in the
<a href=\"Modelica://Modelica.Fluid.UsersGuide.ComponentDefinition.WallFriction\">UsersGuide</a>.
Basically, different variants of the equation
</p>

<pre>
   dp = &lambda;(Re,<font face=\"Symbol\">D</font>)*(L/D)*&rho;*v*|v|/2
</pre>

<p>
are used, where the friction loss factor &lambda; is shown
in the next figure:
</p>

<img src=\"../Images/Fluid/Components/PipeFriction1.png\">

<p>
By default, the correlations are computed with media data
at the actual time instant.
In order to reduce non-linear equation systems, parameter
<b>use_nominal</b> provides the option
to compute the correlations with constant media values
at the desired operating point. This might speed-up the
simulation and/or might give a more robust simulation.
</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,64},{100, -64}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Backward),Rectangle(extent = {{ -100,50},{100, -49}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -60, -49},{ -60,50}}, color = {0,0,255}, arrow = {Arrow.Filled,Arrow.Filled}),Text(extent = {{ -50,16},{6, -10}}, lineColor = {0,0,255}, textString = "diameter"),Line(points = {{ -100,74},{100,74}}, color = {0,0,255}, arrow = {Arrow.Filled,Arrow.Filled}),Text(extent = {{ -34,92},{34,74}}, lineColor = {0,0,255}, textString = "length")}));
          end TestWallFrictionAndGravity;
        end WallFriction;
      end BaseClasses;
      annotation(Documentation(info = "<html>

</html>"));
    end Pipes;
    package Machines "Devices for converting between energy held in a fluid and mechanical energy"
      extends Modelica.Fluid.Icons.VariantLibrary;
      model SweptVolume "varying cylindric volume depending on the postition of the piston"
        import Modelica.Constants.pi;
        parameter SI.Area pistonCrossArea "cross sectional area of pistion";
        parameter SI.Volume clearance "remaining volume at zero piston stroke";
        SI.Volume V "fluid volume";
        extends Modelica.Fluid.Vessels.BaseClasses.PartialLumpedVessel(final fluidVolume = V, heatTransfer(surfaceAreas = {pistonCrossArea + 2 * sqrt(pistonCrossArea * pi) * (flange.s + clearance / pistonCrossArea)}));
        Modelica.Mechanics.Translational.Interfaces.Flange_b flange "translation flange for piston" annotation(Placement(transformation(extent = {{ -10,90},{10,110}}, rotation = 0)));

      equation 
        assert(flange.s >= 0, "Piston stroke (given by flange.s) must not be smaller than zero!");
        V = clearance + flange.s * pistonCrossArea;
        0 = flange.f + (medium.p - system.p_ambient) * pistonCrossArea;
        Wb_flow = medium.p * pistonCrossArea * ( -der(flange.s));
        for i in 1:nPorts loop
        vessel_ps_static[i] = medium.p;

        end for;
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -50,36},{50, -90}}, lineColor = {0,0,255}, pattern = LinePattern.None, lineThickness = 1, fillColor = {170,213,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -52,62},{ -48,62},{ -48, -30},{ -52, -30},{ -52,62}}, lineColor = {95,95,95}, smooth = Smooth.None, fillColor = {135,135,135}, fillPattern = FillPattern.Backward),Polygon(points = {{48,60},{52,60},{52, -34},{48, -34},{48,60}}, lineColor = {95,95,95}, smooth = Smooth.None, fillColor = {135,135,135}, fillPattern = FillPattern.Backward),Rectangle(extent = {{ -48,40},{48,30}}, lineColor = {95,95,95}, fillColor = {135,135,135}, fillPattern = FillPattern.Forward),Rectangle(extent = {{ -6,92},{6,40}}, lineColor = {95,95,95}, fillColor = {135,135,135}, fillPattern = FillPattern.Forward),Polygon(points = {{ -48, -90},{48, -90},{48,70},{52,70},{52, -94},{ -52, -94},{ -52,70},{ -48,70},{ -48, -90}}, lineColor = {95,95,95}, smooth = Smooth.None, fillColor = {135,135,135}, fillPattern = FillPattern.Backward),Line(visible = use_HeatTransfer, points = {{ -100,0},{ -52,0}}, smooth = Smooth.None, color = {198,0,0})}), Documentation(info = "<html>
<p> Mixing volume with varying size. The size of the volume is given by:</p>
<ul>
  <li>cross sectional piston area</li>
  <li>piston stroke given by the flange position s</li>
  <li>clearance (volume at flang position = 0)</li>
</ul>
<p>Losses are neglected. The shaft power is completely converted into mechanical work on the fluid. </p>

<p> The flange position has to be equal or greater than zero. Otherwise the simulation stops. The force of the flange results from the pressure difference between medium and ambient pressure and the cross sectional piston area. For using the component, a top level instance of the ambient model with the inner attribute is needed.</p>
<p> The pressure at both fluid ports equals the medium pressure in the volume. No suction nor discharge valve is included in the model.</p>
<p>The thermal port is directly connected to the medium. The temperature of the thermal port equals the medium temperature. The heat capacity of the cylinder and the piston are not includes in the model.</p>
</html>", revisions = "<html>
<ul>
<li><i>29 Oct 2007</i>
    by Carsten Heinrich:<br>
       Model added to the Fluid library</li>
</ul>
</html>"));
      end SweptVolume;
      model Pump "Centrifugal pump with mechanical connector for the shaft"
        extends Modelica.Fluid.Machines.BaseClasses.PartialPump;
        SI.Angle phi "Shaft angle";
        SI.AngularVelocity omega "Shaft angular velocity";
        Modelica.Mechanics.Rotational.Interfaces.Flange_a shaft annotation(Placement(transformation(extent = {{ -10,90},{10,110}}, rotation = 0)));

      equation 
        phi = shaft.phi;
        omega = der(phi);
        N = Modelica.SIunits.Conversions.to_rpm(omega);
        W_single = omega * shaft.tau;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -10,100},{10,78}}, lineColor = {0,0,0}, fillPattern = FillPattern.VerticalCylinder, fillColor = {95,95,95})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<HTML>
<p>This model describes a centrifugal pump (or a group of <tt>nParallel</tt> pumps) with a mechanical rotational connector for the shaft, to be used when the pump drive has to be modelled explicitly. In the case of <tt>nParallel</tt> pumps, the mechanical connector is relative to a single pump.
<p>The model extends <tt>PartialPump</tt>
 </HTML>", revisions = "<html>
<ul>
<li><i>31 Oct 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Model added to the Fluid library</li>
</ul>
</html>"));
      end Pump;
      model ControlledPump "Centrifugal pump with ideally controlled mass flow rate"
        import Modelica.SIunits.Conversions.NonSIunits.AngularVelocity_rpm;
        extends Modelica.Fluid.Machines.BaseClasses.PartialPump(N_nominal = 1500, N(start = N_nominal), redeclare replaceable function flowCharacteristic = Modelica.Fluid.Machines.BaseClasses.PumpCharacteristics.quadraticFlow(V_flow_nominal = {0,V_flow_op,1.5 * V_flow_op}, head_nominal = {2 * head_op,head_op,0}));
        parameter Medium.AbsolutePressure p_a_nominal "Nominal inlet pressure for predefined pump characteristics";
        parameter Medium.AbsolutePressure p_b_nominal "Nominal outlet pressure, fixed if not control_m_flow and not use_p_set";
        parameter Medium.MassFlowRate m_flow_nominal "Nominal mass flow rate, fixed if control_m_flow and not use_m_flow_set";
        parameter Boolean control_m_flow = true "= false to control outlet pressure port_b.p instead of m_flow" annotation(Evaluate = true);
        parameter Boolean use_m_flow_set = false "= true to use input signal m_flow_set instead of m_flow_nominal" annotation(Dialog(enable = control_m_flow));
        parameter Boolean use_p_set = false "= true to use input signal p_set instead of p_b_nominal" annotation(Dialog(enable = not control_m_flow));
        final parameter SI.VolumeFlowRate V_flow_op = m_flow_nominal / rho_nominal "operational volume flow rate according to nominal values";
        final parameter SI.Height head_op = (p_b_nominal - p_a_nominal) / (rho_nominal * g) "operational pump head according to nominal values";
        Modelica.Blocks.Interfaces.RealInput m_flow_set if use_m_flow_set "Prescribed mass flow rate" annotation(Placement(transformation(extent = {{ -20, -20},{20,20}}, rotation =  -90, origin = { -50,82})));
        Modelica.Blocks.Interfaces.RealInput p_set if use_p_set "Prescribed outlet pressure" annotation(Placement(transformation(extent = {{ -20, -20},{20,20}}, rotation =  -90, origin = {50,82})));
      protected 
        Modelica.Blocks.Interfaces.RealInput m_flow_set_internal "Needed to connect to conditional connector";
        Modelica.Blocks.Interfaces.RealInput p_set_internal "Needed to connect to conditional connector";

      equation 
        if control_m_flow then
          m_flow = m_flow_set_internal;

        else
          dp_pump = p_set_internal - port_a.p;

        end if;
if not use_m_flow_set then
                  m_flow_set_internal = m_flow_nominal;
        end if;
if not use_p_set then
                  p_set_internal = p_b_nominal;
        end if;
        connect(m_flow_set,m_flow_set_internal);
        connect(p_set,p_set_internal);
        annotation(defaultComponentName = "pump", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(visible = use_p_set, extent = {{82,108},{176,92}}, textString = "p_set"),Text(visible = use_m_flow_set, extent = {{ -20,108},{170,92}}, textString = "m_flow_set")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<HTML>
<p>
This model describes a centrifugal pump (or a group of <tt>nParallel</tt> pumps)
with ideally controlled mass flow rate or pressure.
</p>
<p>
Nominal values are used to predefine an exemplary pump characteristics and to define the operation of the pump.
The input connectors <tt>m_flow_set</tt> or <tt>p_set</tt> can optionally be enabled to provide time varying set points.
</p>
<p>
Use this model if the pump characteristics is of secondary interest.
The actual characteristics can be configured later on for the appropriate rotational speed N.
Then the model can be replaced with a Pump with rotational shaft or with a PrescribedPump.
</p>
</HTML>", revisions = "<html>
<ul>
<li><i>15 Dec 2008</i>
    by Ruediger Franke</a>:<br>
       Model added to the Fluid library</li>
</ul>
</html>"));
      end ControlledPump;
      model PrescribedPump "Centrifugal pump with ideally controlled speed"
        extends Modelica.Fluid.Machines.BaseClasses.PartialPump;
        parameter Boolean use_N_in = false "Get the rotational speed from the input connector";
        parameter Modelica.SIunits.Conversions.NonSIunits.AngularVelocity_rpm N_const = N_nominal "Constant rotational speed" annotation(Dialog(enable = not use_N_in));
        Modelica.Blocks.Interfaces.RealInput N_in(unit = "1/min") if use_N_in "Prescribed rotational speed" annotation(Placement(transformation(extent = {{ -20, -20},{20,20}}, rotation =  -90, origin = {0,100}), iconTransformation(extent = {{ -20, -20},{20,20}}, rotation =  -90, origin = {0,100})));
        annotation(defaultComponentName = "pump", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(visible = use_N_in, extent = {{14,98},{178,82}}, textString = "N_in [rpm]")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<HTML>
<p>This model describes a centrifugal pump (or a group of <tt>nParallel</tt> pumps) with prescribed speed, either fixed or provided by an external signal.
<p>The model extends <tt>PartialPump</tt>
<p>If the <tt>N_in</tt> input connector is wired, it provides rotational speed of the pumps (rpm); otherwise, a constant rotational speed equal to <tt>n_const</tt> (which can be different from <tt>N_nominal</tt>) is assumed.</p>
</HTML>", revisions = "<html>
<ul>
<li><i>31 Oct 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Model added to the Fluid library</li>
</ul>
</html>"));
      protected 
        Modelica.Blocks.Interfaces.RealInput N_in_internal(unit = "1/min") "Needed to connect to conditional connector";

      equation 
        connect(N_in,N_in_internal);
if not use_N_in then
                  N_in_internal = N_const;
        end if;
        N = max(N_in_internal, 0.001) "Rotational speed";
      end PrescribedPump;
      package BaseClasses "Base classes used in the Machines package (only of interest to build new component models)"
        extends Modelica.Fluid.Icons.BaseClassLibrary;
        partial model PartialPump "Base model for centrifugal pumps"
          import Modelica.SIunits.Conversions.NonSIunits.*;
          import Modelica.Constants;
          extends Modelica.Fluid.Interfaces.PartialTwoPort(port_b_exposesState = energyDynamics <> Types.Dynamics.SteadyState or massDynamics <> Types.Dynamics.SteadyState, port_a(p(start = p_a_start), m_flow(start = m_flow_start, min = if allowFlowReversal and not checkValve then  -Constants.inf else 0)), port_b(p(start = p_b_start), m_flow(start =  -m_flow_start, max = if allowFlowReversal and not checkValve then  +Constants.inf else 0)));
          parameter Medium.AbsolutePressure p_a_start = system.p_start "Guess value for inlet pressure" annotation(Dialog(tab = "Initialization"));
          parameter Medium.AbsolutePressure p_b_start = p_a_start "Guess value for outlet pressure" annotation(Dialog(tab = "Initialization"));
          parameter Medium.MassFlowRate m_flow_start = 1 "Guess value of m_flow = port_a.m_flow" annotation(Dialog(tab = "Initialization"));
          parameter Integer nParallel(min = 1) = 1 "Number of pumps in parallel" annotation(Dialog(group = "Characteristics"));
          replaceable function flowCharacteristic = PumpCharacteristics.baseFlow "Head vs. V_flow characteristic at nominal speed and density" annotation(Dialog(group = "Characteristics"), choicesAllMatching = true);
          parameter AngularVelocity_rpm N_nominal "Nominal rotational speed for flow characteristic" annotation(Dialog(group = "Characteristics"));
          parameter Medium.Density rho_nominal = Medium.density_pTX(Medium.p_default, Medium.T_default, Medium.X_default) "Nominal fluid density for characteristic" annotation(Dialog(group = "Characteristics"));
          parameter Boolean use_powerCharacteristic = false "Use powerCharacteristic (vs. efficiencyCharacteristic)" annotation(Evaluate = true, Dialog(group = "Characteristics"));
          replaceable function powerCharacteristic = PumpCharacteristics.quadraticPower(V_flow_nominal = {0,0,0}, W_nominal = {0,0,0}) "Power consumption vs. V_flow at nominal speed and density" annotation(Dialog(group = "Characteristics", enable = use_powerCharacteristic), choicesAllMatching = true);
          replaceable function efficiencyCharacteristic = PumpCharacteristics.constantEfficiency(eta_nominal = 0.8) extends PumpCharacteristics.baseEfficiency "Efficiency vs. V_flow at nominal speed and density" annotation(Dialog(group = "Characteristics", enable = not use_powerCharacteristic), choicesAllMatching = true);
          parameter Boolean checkValve = false "= true to prevent reverse flow" annotation(Dialog(tab = "Assumptions"), Evaluate = true);
          parameter SI.Volume V = 0 "Volume inside the pump" annotation(Dialog(tab = "Assumptions"), Evaluate = true);
          extends Modelica.Fluid.Interfaces.PartialLumpedVolume(final fluidVolume = V, energyDynamics = Types.Dynamics.SteadyState, massDynamics = Types.Dynamics.SteadyState, final p_start = p_b_start);
          parameter Boolean use_HeatTransfer = false "= true to use a HeatTransfer model, e.g. for a housing" annotation(Dialog(tab = "Assumptions", group = "Heat transfer"));
          replaceable model HeatTransfer = Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.IdealHeatTransfer extends Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.PartialVesselHeatTransfer "Wall heat transfer" annotation(Dialog(tab = "Assumptions", group = "Heat transfer", enable = use_HeatTransfer), choicesAllMatching = true);
          HeatTransfer heatTransfer(redeclare final package Medium = Medium, final n = 1, surfaceAreas = {4 * Modelica.Constants.pi * ((3 / 4 * V) / Modelica.Constants.pi) ^ (2 / 3)}, final states = {medium.state}, final use_k = use_HeatTransfer) annotation(Placement(transformation(extent = {{ -10, -10},{30,30}}, rotation = 180, origin = {50, -10})));
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort if use_HeatTransfer annotation(Placement(transformation(extent = {{30, -70},{50, -50}})));
          final parameter SI.Acceleration g = system.g;
          Medium.Density rho = medium.d;
          SI.Pressure dp_pump = port_b.p - port_a.p "Pressure increase";
          SI.Height head = dp_pump / (rho * g) "Pump head";
          SI.MassFlowRate m_flow = port_a.m_flow "Mass flow rate (total)";
          SI.MassFlowRate m_flow_single = m_flow / nParallel "Mass flow rate (single pump)";
          SI.VolumeFlowRate V_flow = m_flow / rho "Volume flow rate (total)";
          SI.VolumeFlowRate V_flow_single(start = m_flow_start / rho_nominal / nParallel) = V_flow / nParallel "Volume flow rate (single pump)";
          AngularVelocity_rpm N(start = N_nominal) "Shaft rotational speed";
          SI.Power W_single "Power Consumption (single pump)";
          SI.Power W_total = W_single * nParallel "Power Consumption (total)";
          Real eta "Global Efficiency";
          Real s(start = m_flow_start) "Curvilinear abscissa for the flow curve in parametric form (either mass flow rate or head)";
          parameter Boolean show_NPSHa = false "= true to compute Net Positive Suction Head available" annotation(Dialog(tab = "Advanced", group = "Diagnostics"));
          Medium.ThermodynamicState state_a = Medium.setState_phX(port_a.p, inStream(port_a.h_outflow), inStream(port_a.Xi_outflow)) if show_NPSHa "state for medium inflowing through port_a";
          Medium.Density rho_in = Medium.density(state_a) if show_NPSHa "Liquid density at the inlet port_a";
          SI.Length NPSHa = NPSPa / (rho_in * system.g) if show_NPSHa "Net Positive Suction Head available";
          SI.Pressure NPSPa = assertPositiveDifference(port_a.p, Medium.saturationPressure(Medium.temperature(state_a)), "Cavitation occurs at the pump inlet") if show_NPSHa "Net Positive Suction Pressure available";
          SI.Pressure NPDPa = assertPositiveDifference(port_b.p, Medium.saturationPressure(medium.T), "Cavitation occurs in the pump") if show_NPSHa "Net Positive Discharge Pressure available";
        protected 
          constant SI.Height unitHead = 1;
          constant SI.MassFlowRate unitMassFlowRate = 1;

        equation 
          if not checkValve then
            head = (N / N_nominal) ^ 2 * flowCharacteristic(V_flow_single * N_nominal / N);
            s = 0;

          elseif s > 0 then
            head = (N / N_nominal) ^ 2 * flowCharacteristic(V_flow_single * N_nominal / N);
            V_flow_single = (s * unitMassFlowRate) / rho;


          else
            head = (N / N_nominal) ^ 2 * flowCharacteristic(0) - s * unitHead;
            V_flow_single = 0;

          end if;
          if use_powerCharacteristic then
            W_single = (N / N_nominal) ^ 3 * rho / rho_nominal * powerCharacteristic(V_flow_single * N_nominal / N);
            eta = (dp_pump * V_flow_single) / W_single;

          else
            eta = efficiencyCharacteristic(V_flow_single * N_nominal / N);
            W_single = (dp_pump * V_flow_single) / eta;

          end if;
          Wb_flow = W_total;
          Qb_flow = heatTransfer.Q_flows[1];
          Hb_flow = port_a.m_flow * actualStream(port_a.h_outflow) + port_b.m_flow * actualStream(port_b.h_outflow);
          port_a.h_outflow = medium.h;
          port_b.h_outflow = medium.h;
          port_b.p = medium.p "outlet pressure is equal to medium pressure, which includes Wb_flow";
          mb_flow = port_a.m_flow + port_b.m_flow;
          mbXi_flow = port_a.m_flow * actualStream(port_a.Xi_outflow) + port_b.m_flow * actualStream(port_b.Xi_outflow);
          port_a.Xi_outflow = medium.Xi;
          port_b.Xi_outflow = medium.Xi;
          mbC_flow = port_a.m_flow * actualStream(port_a.C_outflow) + port_b.m_flow * actualStream(port_b.C_outflow);
          port_a.C_outflow = C;
          port_b.C_outflow = C;
          connect(heatTransfer.heatPorts[1],heatPort) annotation(Line(points = {{40, -34},{40, -60}}, color = {127,0,0}, smooth = Smooth.None));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,46},{100, -46}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.HorizontalCylinder),Polygon(points = {{ -48, -60},{ -72, -100},{72, -100},{48, -60},{ -48, -60}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {0,0,0}, fillPattern = FillPattern.VerticalCylinder),Ellipse(extent = {{ -80,80},{80, -80}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {0,100,199}),Polygon(points = {{ -28,30},{ -28, -30},{50, -2},{ -28,30}}, lineColor = {0,0,0}, pattern = LinePattern.None, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255,255,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<HTML>
<p>This is the base model for pumps.
<p>The model describes a centrifugal pump, or a group of <tt>nParallel</tt> identical pumps. The pump model is based on the theory of kinematic similarity: the pump characteristics are given for nominal operating conditions (rotational speed and fluid density), and then adapted to actual operating condition, according to the similarity equations.

<p><b>Pump characteristics</b></p>
<p> The nominal hydraulic characteristic (head vs. volume flow rate) is given by the the replaceable function <tt>flowCharacteristic</tt>.
<p> The pump energy balance can be specified in two alternative ways:
<ul>
<li><tt>use_powerCharacteristic = false</tt> (default option): the replaceable function <tt>efficiencyCharacteristic</tt> (efficiency vs. volume flow rate in nominal conditions) is used to determine the efficiency, and then the power consumption.
    The default is a constant efficiency of 0.8.</li>
<li><tt>use_powerCharacteristic = true</tt>: the replaceable function <tt>powerCharacteristic</tt> (power consumption vs. volume flow rate in nominal conditions) is used to determine the power consumption, and then the efficiency.
    Use <tt>powerCharacteristic</tt> to specify a non-zero power consumption for zero flow rate.
</ul>
<p>
Several functions are provided in the package <tt>PumpCharacteristics</tt> to specify the characteristics as a function of some operating points at nominal conditions.
<p>Depending on the value of the <tt>checkValve</tt> parameter, the model either supports reverse flow conditions, or includes a built-in check valve to avoid flow reversal.
</p>
<p>It is possible to take into account the heat capacity of the fluid inside the pump by specifying its volume <tt>V</tt>;
this is necessary to avoid singularities in the computation of the outlet enthalpy in case of zero flow rate.
If zero flow rate conditions are always avoided, this dynamic effect can be neglected by leaving the default value <tt>V = 0</tt>, thus avoiding a fast state variable in the model.
</p>

<p><b>Dynamics options</b></p>
<p>
Steady-state mass and energy balances are assumed per default, neglecting the holdup of fluid in the pump.
Dynamic mass and energy balance can be used by setting the corresponding dynamic parameters.
This might be desirable if the pump is assembled together with valves before port_a and behind port_b.
If both valves are closed, then the fluid is useful to define the thermodynamic state and in particular the absolute pressure in the pump.
Note that the <tt>flowCharacteristic</tt> only specifies a pressure difference.
</p>

<p><b>Heat transfer</b></p>
<p>
The boolean paramter <tt>use_HeatTransfer</tt> can be set to true if heat exchanged with the environment
should be taken into account or to model a housing. This might be desirable if a pump with realistic
<tt>powerCharacteristic</tt> for zero flow operates while a valve prevents fluid flow.
</p>

<p><b>Diagnostics of Cavitation</b></p>
<p>The boolean parameter show_NPSHa can set true to compute the Net Positive Suction Head available and check for cavitation,
provided a two-phase medium model is used.
</p>
</HTML>", revisions = "<html>
<ul>
<li><i>Dec 2008</i>
    by R&uuml;diger Franke:<br>
    <ul>
    <li>Replaced simplified mass and energy balances with rigorous formulation (base class PartialLumpedVolume)</li>
    <li>Introduced optional HeatTransfer model defining Qb_flow</li>
    <li>Enabled events when the checkValve is operating to support the opening of a discrete valve before port_a</li>
    </ul></li>
<li><i>31 Oct 2005</i>
    by <a href=\"mailto:francesco.casella@polimi.it\">Francesco Casella</a>:<br>
       Model added to the Fluid library</li>
</ul>
</html>"));
        end PartialPump;
        package PumpCharacteristics "Functions for pump characteristics"
          import NonSI = Modelica.SIunits.Conversions.NonSIunits;
          partial function baseFlow "Base class for pump flow characteristics"
            extends Modelica.Icons.Function;
            input SI.VolumeFlowRate V_flow "Volumetric flow rate";
            output SI.Height head "Pump head";
          end baseFlow;
          partial function basePower "Base class for pump power consumption characteristics"
            extends Modelica.Icons.Function;
            input SI.VolumeFlowRate V_flow "Volumetric flow rate";
            output SI.Power consumption "Power consumption";
          end basePower;
          partial function baseEfficiency "Base class for efficiency characteristics"
            extends Modelica.Icons.Function;
            input SI.VolumeFlowRate V_flow "Volumetric flow rate";
            output Real eta "Efficiency";
          end baseEfficiency;
          function linearFlow "Linear flow characteristic"
            extends baseFlow;
            input SI.VolumeFlowRate V_flow_nominal[2] "Volume flow rate for two operating points (single pump)" annotation(Dialog);
            input SI.Height head_nominal[2] "Pump head for two operating points" annotation(Dialog);
          protected 
            Real c[2] = Modelica.Math.Matrices.solve([ones(2),V_flow_nominal], head_nominal) "Coefficients of linear head curve";
          algorithm 
            head:=c[1] + V_flow * c[2];
          end linearFlow;
          function quadraticFlow "Quadratic flow characteristic"
            extends baseFlow;
            input SI.VolumeFlowRate V_flow_nominal[3] "Volume flow rate for three operating points (single pump)" annotation(Dialog);
            input SI.Height head_nominal[3] "Pump head for three operating points" annotation(Dialog);
          protected 
            Real V_flow_nominal2[3] = {V_flow_nominal[1] ^ 2,V_flow_nominal[2] ^ 2,V_flow_nominal[3] ^ 2} "Squared nominal flow rates";
            Real c[3] = Modelica.Math.Matrices.solve([ones(3),V_flow_nominal,V_flow_nominal2], head_nominal) "Coefficients of quadratic head curve";
          algorithm 
            head:=c[1] + V_flow * c[2] + V_flow ^ 2 * c[3];
          end quadraticFlow;
          function polynomialFlow "Polynomial flow characteristic"
            extends baseFlow;
            input SI.VolumeFlowRate V_flow_nominal[:] "Volume flow rate for N operating points (single pump)" annotation(Dialog);
            input SI.Height head_nominal[:] "Pump head for N operating points" annotation(Dialog);
          protected 
            Integer N = size(V_flow_nominal, 1) "Number of nominal operating points";
            Real V_flow_nominal_pow[N,N] = array(array(V_flow_nominal[i] ^ (j - 1) for j in 1:N) for i in 1:N) "Rows: different operating points; columns: increasing powers";
            Real c[N] = Modelica.Math.Matrices.solve(V_flow_nominal_pow, head_nominal) "Coefficients of polynomial head curve";
          algorithm 
            head:=sum(V_flow ^ (i - 1) * c[i] for i in 1:N);
          end polynomialFlow;
          function constantEfficiency "Constant efficiency characteristic"
            extends baseEfficiency;
            input Real eta_nominal "Nominal efficiency" annotation(Dialog);
          algorithm 
            eta:=eta_nominal;
          end constantEfficiency;
          function linearPower "Linear power consumption characteristic"
            extends basePower;
            input SI.VolumeFlowRate V_flow_nominal[2] "Volume flow rate for two operating points (single pump)" annotation(Dialog);
            input SI.Power W_nominal[2] "Power consumption for two operating points" annotation(Dialog);
          protected 
            Real c[2] = Modelica.Math.Matrices.solve([ones(3),V_flow_nominal], W_nominal) "Coefficients of linear power consumption curve";
          algorithm 
            consumption:=c[1] + V_flow * c[2];
          end linearPower;
          function quadraticPower "Quadratic power consumption characteristic"
            extends basePower;
            input SI.VolumeFlowRate V_flow_nominal[3] "Volume flow rate for three operating points (single pump)" annotation(Dialog);
            input SI.Power W_nominal[3] "Power consumption for three operating points" annotation(Dialog);
          protected 
            Real V_flow_nominal2[3] = {V_flow_nominal[1] ^ 2,V_flow_nominal[2] ^ 2,V_flow_nominal[3] ^ 2} "Squared nominal flow rates";
            Real c[3] = Modelica.Math.Matrices.solve([ones(3),V_flow_nominal,V_flow_nominal2], W_nominal) "Coefficients of quadratic power consumption curve";
          algorithm 
            consumption:=c[1] + V_flow * c[2] + V_flow ^ 2 * c[3];
          end quadraticPower;
        end PumpCharacteristics;
        function assertPositiveDifference
          extends Modelica.Icons.Function;
          input SI.Pressure p;
          input SI.Pressure p_sat;
          input String message;
          output SI.Pressure dp;
        algorithm 
          dp:=p - p_sat;
          assert(p >= p_sat, message);
        end assertPositiveDifference;
      end BaseClasses;
      annotation(Documentation(info = "<html>

</html>"));
    end Machines;
    package Interfaces "Interfaces for steady state and unsteady, mixed-phase, multi-substance, incompressible and compressible flow"
      annotation(Documentation(info = "<html>

</html>", revisions = "<html>
<ul>
<li><i>June 9th, 2008</i>
       by Michael Sielemann: Introduced stream keyword after decision at 57th Design Meeting (Lund).</li>
<li><i>May 30, 2007</i>
       by Christoph Richter: moved everything back to its original position in Modelica.Fluid.</li>
<li><i>Apr. 20, 2007</i>
       by Christoph Richter: moved parts of the original package from Modelica.Fluid
       to the development branch of Modelica 2.2.2.</li>
<li><i>Nov. 2, 2005</i>
       by Francesco Casella: restructured after 45th Design Meeting.</li>
<li><i>Nov. 20-21, 2002</i>
       by Hilding Elmqvist, Mike Tiller, Allan Watson, John Batteh, Chuck Newman,
       Jonas Eborn: Improved at the 32nd Modelica Design Meeting.
<li><i>Nov. 11, 2002</i>
       by Hilding Elmqvist, Martin Otter: improved version.</li>
<li><i>Nov. 6, 2002</i>
       by Hilding Elmqvist: first version.</li>
<li><i>Aug. 11, 2002</i>
       by Martin Otter: Improved according to discussion with Hilding
       Elmqvist and Hubertus Tummescheit.<br>
       The PortVicinity model is manually
       expanded in the base models.<br>
       The Volume used for components is renamed
       PartialComponentVolume.<br>
       A new volume model \"Fluid.Components.PortVolume\"
       introduced that has the medium properties of the port to which it is
       connected.<br>
       Fluid.Interfaces.PartialTwoPortTransport is a component
       for elementary two port transport elements, whereas PartialTwoPort
       is a component for a container component.</li>
</li>
</ul>
</html>"));
      extends Modelica.Icons.Library;
      connector FluidPort "Interface for quasi one-dimensional fluid flow in a piping network (incompressible or compressible, one or more phases, one or more substances)"
        replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium model" annotation(choicesAllMatching = true);
        flow Medium.MassFlowRate m_flow "Mass flow rate from the connection point into the component";
        Medium.AbsolutePressure p "Thermodynamic pressure in the connection point";
        stream Medium.SpecificEnthalpy h_outflow "Specific thermodynamic enthalpy close to the connection point if m_flow < 0";
        stream Medium.MassFraction Xi_outflow[Medium.nXi] "Independent mixture mass fractions m_i/m close to the connection point if m_flow < 0";
        stream Medium.ExtraProperty C_outflow[Medium.nC] "Properties c_i/m close to the connection point if m_flow < 0";
      end FluidPort;
      connector FluidPort_a "Generic fluid connector at design inlet"
        extends FluidPort;
        annotation(defaultComponentName = "port_a", Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,110},{150,50}}, textString = "%name")}), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -100,100},{100, -100}}, lineColor = {0,127,255}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid)}));
      end FluidPort_a;
      connector FluidPort_b "Generic fluid connector at design outlet"
        extends FluidPort;
        annotation(defaultComponentName = "port_b", Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -30,30},{30, -30}}, lineColor = {0,127,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,110},{150,50}}, textString = "%name")}), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -100,100},{100, -100}}, lineColor = {0,127,255}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -80,80},{80, -80}}, lineColor = {0,127,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}));
      end FluidPort_b;
      connector FluidPorts_a "Fluid connector with filled, large icon to be used for vectors of FluidPorts (vector dimensions must be added after dragging)"
        extends FluidPort;
        annotation(defaultComponentName = "ports_a", Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -50, -200},{50,200}}, grid = {1,1}, initialScale = 0.2), graphics = {Text(extent = {{ -75,130},{75,100}}, textString = "%name"),Rectangle(extent = {{25, -100},{ -25,100}}, lineColor = {0,127,255}),Ellipse(extent = {{ -25,90},{25,40}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -25,25},{25, -25}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -25, -40},{25, -90}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid)}), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -50, -200},{50,200}}, grid = {1,1}, initialScale = 0.2), graphics = {Rectangle(extent = {{50, -200},{ -50,200}}, lineColor = {0,127,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -50,180},{50,80}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -50, -80},{50, -180}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid)}));
      end FluidPorts_a;
      connector FluidPorts_b "Fluid connector with outlined, large icon to be used for vectors of FluidPorts (vector dimensions must be added after dragging)"
        extends FluidPort;
        annotation(defaultComponentName = "ports_b", Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -50, -200},{50,200}}, grid = {1,1}, initialScale = 0.2), graphics = {Text(extent = {{ -75,130},{75,100}}, textString = "%name"),Rectangle(extent = {{ -25,100},{25, -100}}, lineColor = {0,0,0}),Ellipse(extent = {{ -25,90},{25,40}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -25,25},{25, -25}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -25, -40},{25, -90}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -15, -50},{15, -80}}, lineColor = {0,127,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -15,15},{15, -15}}, lineColor = {0,127,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -15,50},{15,80}}, lineColor = {0,127,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -50, -200},{50,200}}, grid = {1,1}, initialScale = 0.2), graphics = {Rectangle(extent = {{ -50,200},{50, -200}}, lineColor = {0,127,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -50,180},{50,80}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -50, -80},{50, -180}}, lineColor = {0,0,0}, fillColor = {0,127,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -30,30},{30, -30}}, lineColor = {0,127,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -30,100},{30,160}}, lineColor = {0,127,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -30, -100},{30, -160}}, lineColor = {0,127,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}));
      end FluidPorts_b;
      partial model PartialTwoPort "Partial component with two ports"
        import Modelica.Constants;
        outer Modelica.Fluid.System system "System wide properties";
        replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium in the component" annotation(choicesAllMatching = true);
        parameter Boolean allowFlowReversal = system.allowFlowReversal "= true to allow flow reversal, false restricts to design direction (port_a -> port_b)" annotation(Dialog(tab = "Assumptions"), Evaluate = true);
        Modelica.Fluid.Interfaces.FluidPort_a port_a(redeclare package Medium = Medium, m_flow(min = if allowFlowReversal then  -Constants.inf else 0)) "Fluid connector a (positive design flow direction is from port_a to port_b)" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
        Modelica.Fluid.Interfaces.FluidPort_b port_b(redeclare package Medium = Medium, m_flow(max = if allowFlowReversal then  +Constants.inf else 0)) "Fluid connector b (positive design flow direction is from port_a to port_b)" annotation(Placement(transformation(extent = {{110, -10},{90,10}}, rotation = 0), iconTransformation(extent = {{110, -10},{90,10}})));
      protected 
        parameter Boolean port_a_exposesState = false "= true if port_a exposes the state of a fluid volume";
        parameter Boolean port_b_exposesState = false "= true if port_b.p exposes the state of a fluid volume";
        parameter Boolean showDesignFlowDirection = true "= false to hide the arrow in the model icon";
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Documentation(info = "<html>
<p>
This partial model defines an interface for components with two ports.
The treatment of the design flow direction and of flow reversal are predefined based on the parameter <tt><b>allowFlowReversal</b></tt>.
The component may transport fluid and may have internal storage for a given fluid <tt><b>Medium</b></tt>.
</p>
<p>
An extending model providing direct access to internal storage of mass or energy through port_a or port_b
should redefine the protected parameters <tt><b>port_a_exposesState</b></tt> and <tt><b>port_b_exposesState</b></tt> appropriately.
This will be visualized at the port icons, in order to improve the understanding of fluid model diagrams.
</p>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{20, -70},{60, -85},{20, -100},{20, -70}}, lineColor = {0,128,255}, smooth = Smooth.None, fillColor = {0,128,255}, fillPattern = FillPattern.Solid, visible = showDesignFlowDirection),Polygon(points = {{20, -75},{50, -85},{20, -95},{20, -75}}, lineColor = {255,255,255}, smooth = Smooth.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, visible = allowFlowReversal),Line(points = {{55, -85},{ -60, -85}}, color = {0,128,255}, smooth = Smooth.None, visible = showDesignFlowDirection),Text(extent = {{ -149, -114},{151, -154}}, lineColor = {0,0,255}, textString = "%name"),Ellipse(extent = {{ -110,26},{ -90, -24}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid, visible = port_a_exposesState),Ellipse(extent = {{90,25},{110, -25}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid, visible = port_b_exposesState)}));
      end PartialTwoPort;
      partial model PartialTwoPortTransport "Partial element transporting fluid between two ports without storage of mass or energy"
        extends PartialTwoPort(final port_a_exposesState = false, final port_b_exposesState = false);
        parameter Medium.AbsolutePressure dp_start = 0.01 * system.p_start "Guess value of dp = port_a.p - port_b.p" annotation(Dialog(tab = "Advanced"));
        parameter Medium.MassFlowRate m_flow_start = system.m_flow_start "Guess value of m_flow = port_a.m_flow" annotation(Dialog(tab = "Advanced"));
        parameter Medium.MassFlowRate m_flow_small = system.m_flow_small "Small mass flow rate for regularization of zero flow" annotation(Dialog(tab = "Advanced"));
        parameter Boolean show_T = true "= true, if temperatures at port_a and port_b are computed" annotation(Dialog(tab = "Advanced", group = "Diagnostics"));
        parameter Boolean show_V_flow = true "= true, if volume flow rate at inflowing port is computed" annotation(Dialog(tab = "Advanced", group = "Diagnostics"));
        Medium.MassFlowRate m_flow(min = if allowFlowReversal then  -Modelica.Constants.inf else 0, start = m_flow_start) "Mass flow rate in design flow direction";
        Modelica.SIunits.Pressure dp(start = dp_start) "Pressure difference between port_a and port_b (= port_a.p - port_b.p)";
        Modelica.SIunits.VolumeFlowRate V_flow = m_flow / Modelica.Fluid.Utilities.regStep(m_flow, Medium.density(state_a), Medium.density(state_b), m_flow_small) if show_V_flow "Volume flow rate at inflowing port (positive when flow from port_a to port_b)";
        Medium.Temperature port_a_T = Modelica.Fluid.Utilities.regStep(port_a.m_flow, Medium.temperature(state_a), Medium.temperature(Medium.setState_phX(port_a.p, port_a.h_outflow, port_a.Xi_outflow)), m_flow_small) if show_T "Temperature close to port_a, if show_T = true";
        Medium.Temperature port_b_T = Modelica.Fluid.Utilities.regStep(port_b.m_flow, Medium.temperature(state_b), Medium.temperature(Medium.setState_phX(port_b.p, port_b.h_outflow, port_b.Xi_outflow)), m_flow_small) if show_T "Temperature close to port_b, if show_T = true";
      protected 
        Medium.ThermodynamicState state_a "state for medium inflowing through port_a";
        Medium.ThermodynamicState state_b "state for medium inflowing through port_b";

      equation 
        state_a = Medium.setState_phX(port_a.p, inStream(port_a.h_outflow), inStream(port_a.Xi_outflow));
        state_b = Medium.setState_phX(port_b.p, inStream(port_b.h_outflow), inStream(port_b.Xi_outflow));
        dp = port_a.p - port_b.p;
        m_flow = port_a.m_flow;
        assert(m_flow >  -m_flow_small or allowFlowReversal, "Reverting flow occurs even though allowFlowReversal is false");
        port_a.m_flow + port_b.m_flow = 0;
        port_a.Xi_outflow = inStream(port_b.Xi_outflow);
        port_b.Xi_outflow = inStream(port_a.Xi_outflow);
        port_a.C_outflow = inStream(port_b.C_outflow);
        port_b.C_outflow = inStream(port_a.C_outflow);
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Documentation(info = "<html>
<p>
This component transports fluid between its two ports, without storing mass or energy.
Energy may be exchanged with the environment though, e.g. in the form of work.
<tt>PartialTwoPortTransport</tt> is intended as base class for devices like orifices, valves and simple fluid machines.
<p>
Three equations need to be added by an extending class using this component:
<ul>
<li>the momentum balance specifying the relationship between the pressure drop <tt>dp</tt> and the mass flow rate <tt>m_flow</tt></li>,
<li><tt>port_b.h_outflow</tt> for flow in design direction, and</li>
<li><tt>port_a.h_outflow</tt> for flow in reverse direction.</li>
</ul>
</p>
</html>"), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1})));
      end PartialTwoPortTransport;
      connector HeatPorts_a "HeatPort connector with filled, large icon to be used for vectors of HeatPorts (vector dimensions must be added after dragging)"
        extends Modelica.Thermal.HeatTransfer.Interfaces.HeatPort;
        annotation(defaultComponentName = "heatPorts_a", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -200, -50},{200,50}}, grid = {1,1}, initialScale = 0.2), graphics = {Rectangle(extent = {{ -201,50},{200, -50}}, lineColor = {127,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -171,45},{ -83, -45}}, lineColor = {127,0,0}, fillColor = {127,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -45,45},{43, -45}}, lineColor = {127,0,0}, fillColor = {127,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{82,45},{170, -45}}, lineColor = {127,0,0}, fillColor = {127,0,0}, fillPattern = FillPattern.Solid)}));
      end HeatPorts_a;
      connector HeatPorts_b "HeatPort connector with filled, large icon to be used for vectors of HeatPorts (vector dimensions must be added after dragging)"
        extends Modelica.Thermal.HeatTransfer.Interfaces.HeatPort;
        annotation(defaultComponentName = "heatPorts_b", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -200, -50},{200,50}}, grid = {1,1}, initialScale = 0.2), graphics = {Rectangle(extent = {{ -200,50},{200, -51}}, lineColor = {127,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -170,44},{ -82, -46}}, lineColor = {127,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -44,46},{44, -44}}, lineColor = {127,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{82,45},{170, -45}}, lineColor = {127,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}));
      end HeatPorts_b;
      partial model PartialHeatTransfer "Common interface for heat transfer models"
        replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium in the component" annotation(Dialog(tab = "Internal Interface", enable = false));
        parameter Integer n = 1 "Number of heat transfer segments" annotation(Dialog(tab = "Internal Interface", enable = false), Evaluate = true);
        input Medium.ThermodynamicState[n] states "Thermodynamic states of flow segments";
        input SI.Area[n] surfaceAreas "Heat transfer areas";
        output SI.HeatFlowRate[n] Q_flows "Heat flow rates";
        parameter Boolean use_k = false "= true to use k value for thermal isolation" annotation(Dialog(tab = "Internal Interface", enable = false));
        parameter SI.CoefficientOfHeatTransfer k = 0 "Heat transfer coefficient to ambient" annotation(Dialog(group = "Ambient"), Evaluate = true);
        parameter SI.Temperature T_ambient = system.T_ambient "Ambient temperature" annotation(Dialog(group = "Ambient"));
        outer Modelica.Fluid.System system "System wide properties";
        Modelica.Fluid.Interfaces.HeatPorts_a[n] heatPorts "Heat port to component boundary" annotation(Placement(transformation(extent = {{ -10,60},{10,80}}, rotation = 0), iconTransformation(extent = {{ -20,60},{20,80}})));
        SI.Temperature[n] Ts = Medium.temperature(states) "Temperatures defined by fluid states";

      equation 
        if use_k then
          Q_flows = heatPorts.Q_flow + array(k * surfaceAreas[i] * (T_ambient - heatPorts[i].T) for i in 1:n);

        else
          Q_flows = heatPorts.Q_flow;

        end if;
        annotation(Documentation(info = "<html>
<p>
This component is a common interface for heat transfer models. The heat flow rates <tt>Q_flows[n]</tt> through the boundaries of n flow segments
are obtained as function of the thermodynamic <tt>states</tt> of the flow segments for a given fluid <tt>Medium</tt>,
the <tt>surfaceAreas[n]</tt> and the boundary temperatures <tt>heatPorts[n].T</tt>.
</p>
<p>
The heat loss coefficient <tt>k</tt> can be used to model a thermal isolation between <tt>heatPorts.T</tt> and <tt>T_ambient</tt>.
<p>
An extending model implementing this interface needs to define one equation: the relation between the predefined fluid temperatures <tt>Ts[n]</tt>,
the boundary temperatures <tt>heatPorts[n].T</tt>, and the heat flow rates <tt>Q_flows[n]</tt>.
</p>
</html>"));
      end PartialHeatTransfer;
      partial model PartialLumpedVolume "Lumped volume with mass and energy balance"
        import Modelica.Fluid.Types;
        import Modelica.Fluid.Types.Dynamics;
        outer Modelica.Fluid.System system "System properties";
        replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium in the component" annotation(choicesAllMatching = true);
        input SI.Volume fluidVolume "Volume";
        parameter Types.Dynamics energyDynamics = system.energyDynamics "Formulation of energy balance" annotation(Evaluate = true, Dialog(tab = "Assumptions", group = "Dynamics"));
        parameter Types.Dynamics massDynamics = system.massDynamics "Formulation of mass balance" annotation(Evaluate = true, Dialog(tab = "Assumptions", group = "Dynamics"));
        final parameter Types.Dynamics substanceDynamics = massDynamics "Formulation of substance balance" annotation(Evaluate = true, Dialog(tab = "Assumptions", group = "Dynamics"));
        final parameter Types.Dynamics traceDynamics = massDynamics "Formulation of trace substance balance" annotation(Evaluate = true, Dialog(tab = "Assumptions", group = "Dynamics"));
        parameter Medium.AbsolutePressure p_start = system.p_start "Start value of pressure" annotation(Dialog(tab = "Initialization"));
        parameter Boolean use_T_start = true "= true, use T_start, otherwise h_start" annotation(Dialog(tab = "Initialization"), Evaluate = true);
        parameter Medium.Temperature T_start = if use_T_start then system.T_start else Medium.temperature_phX(p_start, h_start, X_start) "Start value of temperature" annotation(Dialog(tab = "Initialization", enable = use_T_start));
        parameter Medium.SpecificEnthalpy h_start = if use_T_start then Medium.specificEnthalpy_pTX(p_start, T_start, X_start) else Medium.h_default "Start value of specific enthalpy" annotation(Dialog(tab = "Initialization", enable = not use_T_start));
        parameter Medium.MassFraction X_start[Medium.nX] = Medium.X_default "Start value of mass fractions m_i/m" annotation(Dialog(tab = "Initialization", enable = Medium.nXi > 0));
        parameter Medium.ExtraProperty C_start[Medium.nC](quantity = Medium.extraPropertiesNames) = fill(0, Medium.nC) "Start value of trace substances" annotation(Dialog(tab = "Initialization", enable = Medium.nC > 0));
        Medium.BaseProperties medium(preferredMediumStates = true, p(start = p_start), h(start = h_start), T(start = T_start), Xi(start = X_start[1:Medium.nXi]));
        SI.Energy U "Internal energy of fluid";
        SI.Mass m "Mass of fluid";
        SI.Mass[Medium.nXi] mXi "Masses of independent components in the fluid";
        SI.Mass[Medium.nC] mC "Masses of trace substances in the fluid";
        Medium.ExtraProperty C[Medium.nC] "Trace substance mixture content";
        SI.MassFlowRate mb_flow "Mass flows across boundaries";
        SI.MassFlowRate[Medium.nXi] mbXi_flow "Substance mass flows across boundaries";
        Medium.ExtraPropertyFlowRate[Medium.nC] mbC_flow "Trace substance mass flows across boundaries";
        SI.EnthalpyFlowRate Hb_flow "Enthalpy flow across boundaries or energy source/sink";
        SI.HeatFlowRate Qb_flow "Heat flow across boundaries or energy source/sink";
        SI.Power Wb_flow "Work flow across boundaries or source term";
      protected 
        parameter Boolean initialize_p = not Medium.singleState "= true to set up initial equations for pressure";

      equation 
        assert(not (energyDynamics <> Dynamics.SteadyState and massDynamics == Dynamics.SteadyState) or Medium.singleState, "Bad combination of dynamics options and Medium not conserving mass if fluidVolume is fixed.");
        m = fluidVolume * medium.d;
        mXi = m * medium.Xi;
        U = m * medium.u;
        mC = m * C;
        if energyDynamics == Dynamics.SteadyState then
          0 = Hb_flow + Qb_flow + Wb_flow;

        else
          der(U) = Hb_flow + Qb_flow + Wb_flow;

        end if;
        if massDynamics == Dynamics.SteadyState then
          0 = mb_flow;

        else
          der(m) = mb_flow;

        end if;
        if substanceDynamics == Dynamics.SteadyState then
          zeros(Medium.nXi) = mbXi_flow;

        else
          der(mXi) = mbXi_flow;

        end if;
        if traceDynamics == Dynamics.SteadyState then
          zeros(Medium.nC) = mbC_flow;

        else
          der(mC) = mbC_flow;

        end if;

      initial equation 
        if energyDynamics == Dynamics.FixedInitial then
          if use_T_start then
            medium.T = T_start;

          else
            medium.h = h_start;

          end if;

        elseif energyDynamics == Dynamics.SteadyStateInitial then
          if use_T_start then
            der(medium.T) = 0;

          else
            der(medium.h) = 0;

          end if;


        else

        end if;
        if massDynamics == Dynamics.FixedInitial then
if initialize_p then
                      medium.p = p_start;
          end if;

        elseif massDynamics == Dynamics.SteadyStateInitial then
if initialize_p then
                      der(medium.p) = 0;
          end if;


        else

        end if;
        if substanceDynamics == Dynamics.FixedInitial then
          medium.Xi = X_start[1:Medium.nXi];

        elseif substanceDynamics == Dynamics.SteadyStateInitial then
          der(medium.Xi) = zeros(Medium.nXi);


        else

        end if;
        if traceDynamics == Dynamics.FixedInitial then
          C = C_start[1:Medium.nC];

        elseif traceDynamics == Dynamics.SteadyStateInitial then
          der(C) = zeros(Medium.nC);


        else

        end if;
        annotation(Documentation(info = "<html>
Interface and base class for an ideally mixed fluid volume with the ability to store mass and energy.
The following boundary flow and source terms are part of the energy balance and must be specified in an extending class:
<ul>
<li><tt><b>Qb_flow</b></tt>, e.g. convective or latent heat flow rate across segment boundary, and</li>
<li><tt><b>Wb_flow</b></tt>, work term, e.g. p*der(fluidVolume) if the volume is not constant.</li>
</ul>
The component volume <tt><b>fluidVolume</b></tt> is an input that needs to be set in the extending class to complete the model.
<p>
Further source terms must be defined by an extending class for fluid flow across the segment boundary:
</p>
<ul>
<li><tt><b>Hb_flow</b></tt>, enthalpy flow,</li>
<li><tt><b>mb_flow</b></tt>, mass flow,</li>
<li><tt><b>mbXi_flow</b></tt>, substance mass flow, and</li>
<li><tt><b>mbC_flow</b></tt>, trace substance mass flow.</li>
</ul>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
      end PartialLumpedVolume;
      partial model PartialLumpedFlow "Base class for a lumped momentum balance"
        outer Modelica.Fluid.System system "System properties";
        replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium in the component";
        parameter Boolean allowFlowReversal = system.allowFlowReversal "= true to allow flow reversal, false restricts to design direction (m_flow >= 0)" annotation(Dialog(tab = "Assumptions"), Evaluate = true);
        input SI.Length pathLength "Length flow path";
        Medium.MassFlowRate m_flow(min = if allowFlowReversal then  -Modelica.Constants.inf else 0, start = m_flow_start, stateSelect = if momentumDynamics == Types.Dynamics.SteadyState then StateSelect.default else StateSelect.prefer) "mass flow rates between states";
        parameter Modelica.Fluid.Types.Dynamics momentumDynamics = system.momentumDynamics "Formulation of momentum balance" annotation(Dialog(tab = "Assumptions", group = "Dynamics"), Evaluate = true);
        parameter Medium.MassFlowRate m_flow_start = system.m_flow_start "Start value of mass flow rates" annotation(Dialog(tab = "Initialization"));
        SI.Momentum I "Momentums of flow segments";
        SI.Force Ib_flow "Flow of momentum across boudaries";
        SI.Force F_p "Pressure force";
        SI.Force F_fg "Friction and gravity force";

      equation 
        I = m_flow * pathLength;
        if momentumDynamics == Types.Dynamics.SteadyState then
          0 = Ib_flow - F_p - F_fg;

        else
          der(I) = Ib_flow - F_p - F_fg;

        end if;

      initial equation 
        if momentumDynamics == Types.Dynamics.FixedInitial then
          m_flow = m_flow_start;

        elseif momentumDynamics == Types.Dynamics.SteadyStateInitial then
          der(m_flow) = 0;


        else

        end if;
        annotation(Documentation(info = "<html>
<p>
Interface and base class for a momentum balance, defining the mass flow rate <tt><b>m_flow</b></tt>
of a given <tt>Medium</tt> in a flow model.
</p>
<p>
The following boundary flow and force terms are part of the momentum balance and must be specified in an extending model (to zero if not considered):
<ul>
<li><tt><b>Ib_flow</b></tt>, the flow of momentum across model boundaries,</li>
<li><tt><b>F_p[m]</b></tt>, pressure force, and</li>
<li><tt><b>F_fg[m]</b></tt>, friction and gravity forces.</li>
</ul>
The length of the flow path <tt><b>pathLength</b></tt> is an input that needs to be set in an extending class to complete the model.
</p>
</html>"));
      end PartialLumpedFlow;
      partial model PartialDistributedVolume "Base class for distributed volume models"
        import Modelica.Fluid.Types;
        import Modelica.Fluid.Types.Dynamics;
        outer Modelica.Fluid.System system "System properties";
        replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium in the component" annotation(choicesAllMatching = true);
        parameter Integer n = 2 "Number of discrete volumes";
        input SI.Volume[n] fluidVolumes "Discretized volume, determine in inheriting class";
        parameter Types.Dynamics energyDynamics = system.energyDynamics "Formulation of energy balances" annotation(Evaluate = true, Dialog(tab = "Assumptions", group = "Dynamics"));
        parameter Types.Dynamics massDynamics = system.massDynamics "Formulation of mass balances" annotation(Evaluate = true, Dialog(tab = "Assumptions", group = "Dynamics"));
        final parameter Types.Dynamics substanceDynamics = massDynamics "Formulation of substance balances" annotation(Evaluate = true, Dialog(tab = "Assumptions", group = "Dynamics"));
        final parameter Types.Dynamics traceDynamics = massDynamics "Formulation of trace substance balances" annotation(Evaluate = true, Dialog(tab = "Assumptions", group = "Dynamics"));
        parameter Medium.AbsolutePressure p_a_start = system.p_start "Start value of pressure at port a" annotation(Dialog(tab = "Initialization"));
        parameter Medium.AbsolutePressure p_b_start = p_a_start "Start value of pressure at port b" annotation(Dialog(tab = "Initialization"));
        final parameter Medium.AbsolutePressure[n] ps_start = if n > 1 then linspace(p_a_start, p_b_start, n) else {(p_a_start + p_b_start) / 2} "Start value of pressure";
        parameter Boolean use_T_start = true "Use T_start if true, otherwise h_start" annotation(Evaluate = true, Dialog(tab = "Initialization"));
        parameter Medium.Temperature T_start = if use_T_start then system.T_start else Medium.temperature_phX((p_a_start + p_b_start) / 2, h_start, X_start) "Start value of temperature" annotation(Evaluate = true, Dialog(tab = "Initialization", enable = use_T_start));
        parameter Medium.SpecificEnthalpy h_start = if use_T_start then Medium.specificEnthalpy_pTX((p_a_start + p_b_start) / 2, T_start, X_start) else Medium.h_default "Start value of specific enthalpy" annotation(Evaluate = true, Dialog(tab = "Initialization", enable = not use_T_start));
        parameter Medium.MassFraction X_start[Medium.nX] = Medium.X_default "Start value of mass fractions m_i/m" annotation(Dialog(tab = "Initialization", enable = Medium.nXi > 0));
        parameter Medium.ExtraProperty C_start[Medium.nC](quantity = Medium.extraPropertiesNames) = fill(0, Medium.nC) "Start value of trace substances" annotation(Dialog(tab = "Initialization", enable = Medium.nC > 0));
        SI.Energy[n] Us "Internal energy of fluid";
        SI.Mass[n] ms "Fluid mass";
        SI.Mass[n,Medium.nXi] mXis "Substance mass";
        SI.Mass[n,Medium.nC] mCs "Trace substance mass";
        Medium.ExtraProperty Cs[n,Medium.nC] "Trace substance mixture content";
        Medium.BaseProperties[n] mediums(each preferredMediumStates = true, p(start = ps_start), each h(start = h_start), each T(start = T_start), each Xi(start = X_start[1:Medium.nXi]));
        Medium.MassFlowRate[n] mb_flows "Mass flow rate, source or sink";
        Medium.MassFlowRate[n,Medium.nXi] mbXi_flows "Independent mass flow rates, source or sink";
        Medium.ExtraPropertyFlowRate[n,Medium.nC] mbC_flows "Trace substance mass flow rates, source or sink";
        SI.EnthalpyFlowRate[n] Hb_flows "Enthalpy flow rate, source or sink";
        SI.HeatFlowRate[n] Qb_flows "Heat flow rate, source or sink";
        SI.Power[n] Wb_flows "Mechanical power, p*der(V) etc.";
      protected 
        parameter Boolean initialize_p = not Medium.singleState "= true to set up initial equations for pressure";

      equation 
        assert(not (energyDynamics <> Dynamics.SteadyState and massDynamics == Dynamics.SteadyState) or Medium.singleState, "Bad combination of dynamics options and Medium not conserving mass if fluidVolumes are fixed.");
        for i in 1:n loop
        ms[i] = fluidVolumes[i] * mediums[i].d;
        mXis[i,:] = ms[i] * mediums[i].Xi;
        mCs[i,:] = ms[i] * Cs[i,:];
        Us[i] = ms[i] * mediums[i].u;

        end for;
        if energyDynamics == Dynamics.SteadyState then
          for i in 1:n loop
          0 = Hb_flows[i] + Wb_flows[i] + Qb_flows[i];

          end for;

        else
          for i in 1:n loop
          der(Us[i]) = Hb_flows[i] + Wb_flows[i] + Qb_flows[i];

          end for;

        end if;
        if massDynamics == Dynamics.SteadyState then
          for i in 1:n loop
          0 = mb_flows[i];

          end for;

        else
          for i in 1:n loop
          der(ms[i]) = mb_flows[i];

          end for;

        end if;
        if substanceDynamics == Dynamics.SteadyState then
          for i in 1:n loop
          zeros(Medium.nXi) = mbXi_flows[i,:];

          end for;

        else
          for i in 1:n loop
          der(mXis[i,:]) = mbXi_flows[i,:];

          end for;

        end if;
        if traceDynamics == Dynamics.SteadyState then
          for i in 1:n loop
          zeros(Medium.nC) = mbC_flows[i,:];

          end for;

        else
          for i in 1:n loop
          der(mCs[i,:]) = mbC_flows[i,:];

          end for;

        end if;

      initial equation 
        if energyDynamics == Dynamics.FixedInitial then
          if use_T_start then
            mediums.T = fill(T_start, n);

          else
            mediums.h = fill(h_start, n);

          end if;

        elseif energyDynamics == Dynamics.SteadyStateInitial then
          if use_T_start then
            der(mediums.T) = zeros(n);

          else
            der(mediums.h) = zeros(n);

          end if;


        else

        end if;
        if massDynamics == Dynamics.FixedInitial then
if initialize_p then
                      mediums.p = ps_start;
          end if;

        elseif massDynamics == Dynamics.SteadyStateInitial then
if initialize_p then
                      der(mediums.p) = zeros(n);
          end if;


        else

        end if;
        if substanceDynamics == Dynamics.FixedInitial then
          mediums.Xi = fill(X_start[1:Medium.nXi], n);

        elseif substanceDynamics == Dynamics.SteadyStateInitial then
          for i in 1:n loop
          der(mediums[i].Xi) = zeros(Medium.nXi);

          end for;


        else

        end if;
        if traceDynamics == Dynamics.FixedInitial then
          Cs = fill(C_start[1:Medium.nC], n);

        elseif traceDynamics == Dynamics.SteadyStateInitial then
          for i in 1:n loop
          der(mCs[i,:]) = zeros(Medium.nC);

          end for;


        else

        end if;
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
Interface and base class for <tt><b>n</b></tt> ideally mixed fluid volumes with the ability to store mass and energy.
It is inteded to model a one-dimensional spatial discretization of fluid flow according to the finite volume method.
The following boundary flow and source terms are part of the energy balance and must be specified in an extending class:
<ul>
<li><tt><b>Qb_flows[n]</b></tt>, heat flow term, e.g. conductive heat flows across segment boundaries, and</li>
<li><tt><b>Wb_flows[n]</b></tt>, work term.</li>
</ul>
The component volumes <tt><b>fluidVolumes[n]</b></tt> are an input that needs to be set in an extending class to complete the model.
<p>
Further source terms must be defined by an extending class for fluid flow across the segment boundary:
</p>
<ul>
<li><tt><b>Hb_flows[n]</b></tt>, enthalpy flow,</li>
<li><tt><b>mb_flows[n]</b></tt>, mass flow,</li>
<li><tt><b>mbXi_flows[n]</b></tt>, substance mass flow, and</li>
<li><tt><b>mbC_flows[n]</b></tt>, trace substance mass flow.</li>
</ul>
</html>"));
      end PartialDistributedVolume;
      partial model PartialDistributedFlow "Base class for a distributed momentum balance"
        outer Modelica.Fluid.System system "System properties";
        replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium in the component";
        parameter Boolean allowFlowReversal = system.allowFlowReversal "= true to allow flow reversal, false restricts to design direction (m_flows >= zeros(m))" annotation(Dialog(tab = "Assumptions"), Evaluate = true);
        parameter Integer m = 1 "Number of flow segments";
        input SI.Length[m] pathLengths "Lengths along flow path";
        Medium.MassFlowRate[m] m_flows(each min = if allowFlowReversal then  -Modelica.Constants.inf else 0, each start = m_flow_start, each stateSelect = if momentumDynamics == Types.Dynamics.SteadyState then StateSelect.default else StateSelect.prefer) "mass flow rates between states";
        parameter Modelica.Fluid.Types.Dynamics momentumDynamics = system.momentumDynamics "Formulation of momentum balance" annotation(Dialog(tab = "Assumptions", group = "Dynamics"), Evaluate = true);
        parameter Medium.MassFlowRate m_flow_start = system.m_flow_start "Start value of mass flow rates" annotation(Dialog(tab = "Initialization"));
        SI.Momentum[m] Is "Momentums of flow segments";
        SI.Force[m] Ib_flows "Flow of momentum across boudaries";
        SI.Force[m] Fs_p "Pressure forces";
        SI.Force[m] Fs_fg "Friction and gravity forces";

      equation 
        Is = array(m_flows[i] * pathLengths[i] for i in 1:m);
        if momentumDynamics == Types.Dynamics.SteadyState then
          zeros(m) = Ib_flows - Fs_p - Fs_fg;

        else
          der(Is) = Ib_flows - Fs_p - Fs_fg;

        end if;

      initial equation 
        if momentumDynamics == Types.Dynamics.FixedInitial then
          m_flows = fill(m_flow_start, m);

        elseif momentumDynamics == Types.Dynamics.SteadyStateInitial then
          der(m_flows) = zeros(m);


        else

        end if;
        annotation(Documentation(info = "<html>
<p>
Interface and base class for <tt><b>m</b></tt> momentum balances, defining the mass flow rates <tt><b>m_flows[m]</b></tt>
of a given <tt>Medium</tt> in <tt><b>m</b></tt> flow segments.
</p>
<p>
The following boundary flow and force terms are part of the momentum balances and must be specified in an extending model (to zero if not considered):
<ul>
<li><tt><b>Ib_flows[m]</b></tt>, the flows of momentum across segment boundaries,</li>
<li><tt><b>Fs_p[m]</b></tt>, pressure forces, and</li>
<li><tt><b>Fs_fg[m]</b></tt>, friction and gravity forces.</li>
</ul>
The lengths along the flow path <tt><b>pathLengths[m]</b></tt> are an input that needs to be set in an extending class to complete the model.
</p>
</html>"));
      end PartialDistributedFlow;
    end Interfaces;
    package Icons
      partial class VariantLibrary "Icon for a library that contains several variants of one component"
        annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -40, -40},{100,100}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -70, -70},{70,70}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -100, -100},{40,40}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -125,158},{115,103}}, lineColor = {255,0,0}, textString = "%name")}));
      end VariantLibrary;
      partial package BaseClassLibrary "Icon for library"
        annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100, -100},{80,50}}, lineColor = {175,175,175}, fillColor = {235,255,253}, fillPattern = FillPattern.Solid),Polygon(points = {{ -100,50},{ -80,70},{100,70},{80,50},{ -100,50}}, lineColor = {175,175,175}, fillColor = {235,255,253}, fillPattern = FillPattern.Solid),Polygon(points = {{100,70},{100, -80},{80, -100},{80,50},{100,70}}, lineColor = {175,175,175}, fillColor = {235,255,253}, fillPattern = FillPattern.Solid),Text(extent = {{ -94,15},{73, -16}}, lineColor = {0,0,255}, textString = "Library of"),Text(extent = {{ -120,122},{120,73}}, lineColor = {255,0,0}, textString = "%name"),Text(extent = {{ -92, -44},{73, -72}}, lineColor = {0,0,255}, textString = "Base classes")}));
      end BaseClassLibrary;
    end Icons;
    package Fittings "Adaptors for connections of fluid components and the regulation of fluid flow"
      extends Modelica.Fluid.Icons.VariantLibrary;
      annotation(Documentation(info = "<html>

</html>"));
      model SimpleGenericOrifice "Simple generic orifice defined by pressure loss coefficient and diameter (only for flow from port_a to port_b)"
        extends Modelica.Fluid.Interfaces.PartialTwoPortTransport;
        extends Modelica.Fluid.Interfaces.PartialLumpedFlow(final pathLength = 0, final momentumDynamics = Types.Dynamics.SteadyState);
        parameter SI.Diameter diameter "Diameter of orifice";
        parameter Real zeta "Loss factor for flow of port_a -> port_b" annotation(Dialog(enable = use_zeta));
        parameter Boolean use_zeta = true "= false to obtain zeta from dp_nominal and m_flow_nominal";
        parameter SI.AbsolutePressure dp_nominal = 1000.0 "Nominal pressure drop";
        parameter SI.MassFlowRate m_flow_nominal = 1 "Mass flow rate for dp_nominal";
        parameter Boolean from_dp = true "= true, use m_flow = f(dp) else dp = f(m_flow)" annotation(Evaluate = true, Dialog(tab = "Advanced"));
        parameter Medium.AbsolutePressure dp_small = system.dp_small "Turbulent flow if |dp| >= dp_small" annotation(Dialog(tab = "Advanced", enable = from_dp));
        Real zeta_nominal(start = zeta);
        Medium.Density d = 0.5 * (Medium.density(state_a) + Medium.density(state_b));
        Modelica.SIunits.Pressure dp_fg(start = dp_start) "pressure loss due to friction and gravity";
        Modelica.SIunits.Area A_mean = Modelica.Constants.pi / 4 * diameter ^ 2 "mean cross flow area";

      equation 
        if use_zeta then
          zeta_nominal = zeta;

        else
          dp_nominal = BaseClasses.lossConstant_D_zeta(diameter, zeta_nominal) / d * m_flow_nominal ^ 2;

        end if;
        Ib_flow = 0;
        F_p = A_mean * (Medium.pressure(state_b) - Medium.pressure(state_a));
        F_fg = A_mean * dp_fg;
        if from_dp then
          m_flow = Utilities.regRoot2(dp_fg, dp_small, Medium.density(state_a) / BaseClasses.lossConstant_D_zeta(diameter, zeta_nominal), Medium.density(state_b) / BaseClasses.lossConstant_D_zeta(diameter, zeta_nominal));

        else
          dp_fg = Utilities.regSquare2(m_flow, m_flow_small, BaseClasses.lossConstant_D_zeta(diameter, zeta_nominal) / Medium.density(state_a), BaseClasses.lossConstant_D_zeta(diameter, zeta_nominal) / Medium.density(state_b));

        end if;
        port_a.h_outflow = inStream(port_b.h_outflow);
        port_b.h_outflow = inStream(port_a.h_outflow);
        annotation(defaultComponentName = "orifice", Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -60, -50},{ -60,50},{60, -50},{60,50}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -60,0},{ -100,0}}, color = {0,127,255}),Line(points = {{60,0},{100,0}}, color = {0,127,255}),Text(extent = {{ -173,104},{175,62}}, lineColor = {0,0,0}, textString = "zeta=%zeta")}), Documentation(info = "<html>
<p>
This pressure drop component defines a
simple, generic orifice, where the loss factor &zeta; is provided
for one flow direction (e.g., from loss table of a book):
</p>

<pre>   &Delta;p = 0.5*&zeta;*&rho;*v*|v|
      = 8*&zeta;/(&pi;^2*D^4*&rho;) * m_flow*|m_flow|
</pre>

<p>
where
</p>
<ul>
<li> &Delta;p is the pressure drop: &Delta;p = port_a.p - port_b.p</li>
<li> D is the diameter of the orifice at the position where
     &zeta; is defined (either at port_a or port_b). If the orifice has not a
     circular cross section, D = 4*A/P, where A is the cross section
     area and P is the wetted perimeter.</li>
<li> &zeta; is the loss factor with respect to D
     that depends on the geometry of
     the orifice. In the turbulent flow regime, it is assumed that
     &zeta; is constant.<br>
     For small mass flow rates, the flow is laminar and is approximated
     by a polynomial that has a finite derivative for m_flow=0.</li>
<li> v is the mean velocity.</li>
<li> &rho; is the upstream density.</li>
</ul>

<p>
Since the pressure loss factor zeta is provided only for a mass flow
from port_a to port_b, the pressure loss is not correct when the
flow is reversing. If reversing flow only occurs in a short time interval,
this is most likely uncritical. If significant reversing flow
can appear, this component should not be used.
</p>
</html>"));
      end SimpleGenericOrifice;
      model SharpEdgedOrifice "Pressure drop due to sharp edged orifice (for both flow directions)"
        import NonSI = Modelica.SIunits.Conversions.NonSIunits;
        extends BaseClasses.QuadraticTurbulent.BaseModel(final data = BaseClasses.QuadraticTurbulent.LossFactorData.sharpEdgedOrifice(diameter, leastDiameter, length, alpha));
        parameter SI.Length length "Length of orifice";
        parameter SI.Diameter diameter "Inner diameter of pipe (= same at port_a and port_b)";
        parameter SI.Diameter leastDiameter "Smallest diameter of orifice";
        parameter NonSI.Angle_deg alpha "Angle of orifice";
        annotation(defaultComponentName = "orifice", Documentation(info = "<html>
</html>"), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,44},{100, -44}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,127,255}),Polygon(points = {{ -25,44},{ -25,7},{35,37},{35,44},{ -25,44}}, lineColor = {0,0,0}, fillPattern = FillPattern.Backward, fillColor = {175,175,175}),Polygon(points = {{ -25, -7},{ -25, -44},{35, -44},{35, -36},{ -25, -7}}, lineColor = {0,0,0}, fillColor = {175,175,175}, fillPattern = FillPattern.Backward)}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,60},{100, -60}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -30,60},{ -30,12},{30,50},{30,60},{ -30,60}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Backward),Polygon(points = {{ -30, -10},{ -30, -60},{30, -60},{30, -50},{ -30, -10}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Backward),Line(points = {{ -82, -60},{ -82,60}}, color = {0,0,255}, arrow = {Arrow.Filled,Arrow.Filled}),Text(extent = {{ -78,16},{ -44, -8}}, lineColor = {0,0,255}, textString = "diameter"),Line(points = {{ -30, -10},{ -30,12}}, color = {0,0,255}, arrow = {Arrow.Filled,Arrow.Filled}),Text(extent = {{ -24,14},{8, -10}}, lineColor = {0,0,255}, textString = "leastDiameter"),Text(extent = {{ -20,84},{18,70}}, lineColor = {0,0,255}, textString = "length"),Line(points = {{30,68},{ -30,68}}, color = {0,0,255}, arrow = {Arrow.Filled,Arrow.Filled}),Line(points = {{16,40},{32,18},{36, -2},{34, -20},{20, -42}}, color = {0,0,255}, arrow = {Arrow.Filled,Arrow.Filled}),Text(extent = {{38,8},{92, -6}}, lineColor = {0,0,255}, textString = "alpha")}));
      end SharpEdgedOrifice;
      model AbruptAdaptor "Pressure drop in pipe due to suddenly expanding or reducing area (for both flow directions)"
        extends BaseClasses.QuadraticTurbulent.BaseModelNonconstantCrossSectionArea(final data = BaseClasses.QuadraticTurbulent.LossFactorData.suddenExpansion(diameter_a, diameter_b));
        parameter SI.Diameter diameter_a "Inner diameter of pipe at port_a";
        parameter SI.Diameter diameter_b "Inner diameter of pipe at port_b";
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{0,40},{ -100,40},{ -100, -40},{0, -40},{0, -100},{100, -100},{100,100},{0,100},{0,40}}, color = {0,0,0}),Rectangle(extent = {{ -100,40},{0, -40}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{0,100},{100, -100}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,40},{ -100,40},{ -100, -40},{0, -40},{0, -100},{100, -100},{100,100},{0,100},{0,40}}, color = {0,0,0}),Line(points = {{ -60, -40},{ -60,40}}, color = {0,0,255}, arrow = {Arrow.Filled,Arrow.Filled}),Text(extent = {{ -50,16},{ -26, -10}}, lineColor = {0,0,255}, textString = "diameter_a"),Line(points = {{34, -100},{34,100}}, color = {0,0,255}, arrow = {Arrow.Filled,Arrow.Filled}),Text(extent = {{54,16},{78, -10}}, lineColor = {0,0,255}, textString = "diameter_b")}), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = DynamicSelect({{ -100,22},{0, -22}}, {{ -100,max(0.1, min(1, diameter_a / max(diameter_a, diameter_b))) * 60},{0, -max(0.1, min(1, diameter_a / max(diameter_a, diameter_b))) * 60}}), lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,127,255}),Rectangle(extent = DynamicSelect({{0,60},{100, -60}}, {{0,max(0.1, min(1, diameter_b / max(diameter_a, diameter_b))) * 60},{100, -max(0.1, min(1, diameter_b / max(diameter_a, diameter_b))) * 60}}), lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,127,255})}), Documentation(info = "<html>

</html>"));
      end AbruptAdaptor;
      model MultiPort "Multiply a port; useful if multiple connections shall be made to a port exposing a state"
        function positiveMax
          input Real x;
          output Real y;
        algorithm 
          y:=max(x, 1e-10);
        end positiveMax;
        import Modelica.Constants;
        replaceable package Medium = Modelica.Media.Interfaces.PartialMedium annotation(choicesAllMatching);
        parameter Integer nPorts_b = 0 "Number of outlet ports (mass is distributed evenly between the outlet ports" annotation(Dialog(__Dymola_connectorSizing = true));
        Modelica.Fluid.Interfaces.FluidPort_a port_a(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{ -50, -10},{ -30,10}}, rotation = 0)));
        Modelica.Fluid.Interfaces.FluidPorts_b ports_b[nPorts_b](redeclare each package Medium = Medium) annotation(Placement(transformation(extent = {{30,40},{50, -40}}, rotation = 0)));
        Medium.MassFraction ports_b_Xi_inStream[nPorts_b,Medium.nXi] "inStream mass fractions at ports_b";
        Medium.ExtraProperty ports_b_C_inStream[nPorts_b,Medium.nC] "inStream extra properties at ports_b";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -40, -100},{40,100}}), graphics = {Line(points = {{ -40,0},{40,0}}, color = {0,128,255}, thickness = 1),Line(points = {{ -40,0},{40,26}}, color = {0,128,255}, thickness = 1),Line(points = {{ -40,0},{40, -26}}, color = {0,128,255}, thickness = 1),Text(extent = {{ -150,100},{150,60}}, lineColor = {0,0,255}, textString = "%name")}), Documentation(info = "<html>
<p>
This model is useful if multiple connections shall be made to a port of a volume model exposing a state,
like a pipe with ModelStructure av_vb.
The mixing is shifted into the volume connected to port_a and the result is propageted back to each ports_b.
</p>
<p>
If multiple connections were directly made to the volume,
then ideal mixing would take place in the connection set, outside the volume. This is normally not intended.
</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -40, -100},{40,100}}), graphics));

      equation 
        for i in 1:nPorts_b loop
        assert(cardinality(ports_b[i]) <= 1, "
each ports_b[i] of boundary shall at most be connected to one component.
If two or more connections are present, ideal mixing takes
place with these connections, which is usually not the intention
of the modeller. Increase nPorts_b to add an additional port.
");

        end for;
        0 = port_a.m_flow + sum(ports_b.m_flow);
        ports_b.p = fill(port_a.p, nPorts_b);
        port_a.h_outflow = sum(array(positiveMax(ports_b[j].m_flow) * inStream(ports_b[j].h_outflow) for j in 1:nPorts_b)) / sum(array(positiveMax(ports_b[j].m_flow) for j in 1:nPorts_b));
        for j in 1:nPorts_b loop
        ports_b[j].h_outflow = inStream(port_a.h_outflow);
        ports_b[j].Xi_outflow = inStream(port_a.Xi_outflow);
        ports_b[j].C_outflow = inStream(port_a.C_outflow);
        ports_b_Xi_inStream[j,:] = inStream(ports_b[j].Xi_outflow);
        ports_b_C_inStream[j,:] = inStream(ports_b[j].C_outflow);

        end for;
        for i in 1:Medium.nXi loop
        port_a.Xi_outflow[i] = (positiveMax(ports_b.m_flow) * ports_b_Xi_inStream[:,i]) / sum(positiveMax(ports_b.m_flow));

        end for;
        for i in 1:Medium.nC loop
        port_a.C_outflow[i] = (positiveMax(ports_b.m_flow) * ports_b_C_inStream[:,i]) / sum(positiveMax(ports_b.m_flow));

        end for;
      end MultiPort;
      model TeeJunctionIdeal "Splitting/joining component with static balances for an infinitesimal control volume"
        extends Modelica.Fluid.Fittings.BaseClasses.PartialTeeJunction;

      equation 
        connect(port_1,port_2) annotation(Line(points = {{ -100,0},{100,0}}, color = {0,127,255}, smooth = Smooth.None));
        connect(port_1,port_3) annotation(Line(points = {{ -100,0},{0,0},{0,100}}, color = {0,127,255}, smooth = Smooth.None));
        annotation(Documentation(info = "<html>
  This model is the simplest implementation for a splitting/joining component for
  three flows. Its use is not required. It just formulates the balance
  equations in the same way that the connect symmantics would formulate them anyways.
  The main advantage of using this component is, that the user does not get
  confused when looking at the specific enthalpy at each port which might be confusing
  when not using a splitting/joining component. The reason for the confusion is that one exmanins the mixing
  enthalpy of the infinitesimal control volume introduced with the connect statement when
  looking at the specific enthalpy in the connector which
  might not be equal to the specific enthalpy at the port in the \"real world\".</html>"), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
      end TeeJunctionIdeal;
      model TeeJunctionVolume "Splitting/joining component with static balances for a dynamic control volume"
        extends Modelica.Fluid.Fittings.BaseClasses.PartialTeeJunction;
        extends Modelica.Fluid.Interfaces.PartialLumpedVolume(final fluidVolume = V);
        parameter SI.Volume V "Mixing volume inside junction";

      equation 
        assert(cardinality(port_1) <= 1, "
port_1 of volume can at most be connected to one component.
If two or more connections are present, ideal mixing takes
place with these connections which is usually not the intention
of the modeller.
");
        assert(cardinality(port_2) <= 1, "
port_2 of volume can at most be connected to one component.
If two or more connections are present, ideal mixing takes
place with these connections which is usually not the intention
of the modeller.
");
        assert(cardinality(port_3) <= 1, "
port_3 of volume can at most be connected to one component.
If two or more connections are present, ideal mixing takes
place with these connections which is usually not the intention
of the modeller.
");
        port_1.h_outflow = medium.h;
        port_2.h_outflow = medium.h;
        port_3.h_outflow = medium.h;
        port_1.Xi_outflow = medium.Xi;
        port_2.Xi_outflow = medium.Xi;
        port_3.Xi_outflow = medium.Xi;
        port_1.C_outflow = C;
        port_2.C_outflow = C;
        port_3.C_outflow = C;
        mb_flow = port_1.m_flow + port_2.m_flow + port_3.m_flow "Mass balance";
        mbXi_flow = port_1.m_flow * actualStream(port_1.Xi_outflow) + port_2.m_flow * actualStream(port_2.Xi_outflow) + port_3.m_flow * actualStream(port_3.Xi_outflow) "Component mass balances";
        mbC_flow = port_1.m_flow * actualStream(port_1.C_outflow) + port_2.m_flow * actualStream(port_2.C_outflow) + port_3.m_flow * actualStream(port_3.C_outflow) "Trace substance mass balances";
        port_1.p = medium.p;
        port_2.p = medium.p;
        port_3.p = medium.p;
        Hb_flow = port_1.m_flow * actualStream(port_1.h_outflow) + port_2.m_flow * actualStream(port_2.h_outflow) + port_3.m_flow * actualStream(port_3.h_outflow);
        Qb_flow = 0;
        Wb_flow = 0;
        annotation(Documentation(info = "<html>
  This model introduces a mixing volume into a junction.
  This might be useful to examine the non-ideal mixing taking place in a real junction.</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -9,10},{11, -10}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
      end TeeJunctionVolume;
      annotation(Documentation(info = "<html>
<p>
This sublibrary contains models and functions providing pressure
loss correlations. All models in this library have the property
that no mass and no energy is stored in the component. Therefore,
none of the models has a state. The basic correlations are implemented
with functions of sublibrary
<a href=\"Modelica://Modelica.Fluid.PressureLosses.Utilities\">PressureLosses.Utilities</a>
These functions might also be directly called
(e.g. in another component implementation).
</p>

<p>
All functions are continuous and have a finite, non-zero, smooth, first derivative.
The functions are all guaranteed to be strict monontonically increasing.
The mentioned properties guarantee that a unique inverse of every
function exists. Note, the usual quadratic pressure loss correlation
</p>

<ul>
<li> in the form m_flow = f(dp) has an infinite derivative at zero
     mass flow rate and is therefore problematic to use.</li>
<li> in the form dp = f(m_flow) has a zero derivative at zero mass flow rate
     and is therefore problematic to invert, since the inverse function has
     then an infinite derivative at zero mass flow rate.</li>
</ul>
<p>
The two mentioned problems are solved in this package by approximating
the characteristics around zero mass flow rates with appropriate
polynomials. The monotonicity is guaranteed using results from:
</p>

<dl>
<dt> Fritsch F.N. and Carlson R.E. (1980):</dt>
<dd> <b>Monotone piecewise cubic interpolation</b>.
     SIAM J. Numerc. Anal., Vol. 17, No. 2, April 1980, pp. 238-246</dd>
</dl>

</html>", revisions = "<html>
<ul>
<li><i>Jan. 3, 2006</i>
    by <a href=\"mailto:Martin.Otter@DLR.de\">Martin Otter</a>:<br>
    New design and implementation based on previous iterations.</li>
</ul>
</html>"));
      package BaseClasses "Base classes used in the Fittings package (only of interest to build new component models)"
        extends Modelica.Fluid.Icons.BaseClassLibrary;
        function lossConstant_D_zeta "Return the loss constant 8*zeta/(pi^2*D^4)"
          extends Modelica.Icons.Function;
          input SI.Diameter D "Diameter at port_a or port_b";
          input Real zeta "Constant pressure loss factor with respect to D (i.e., either port_a or port_b)";
          output Real k "Loss constant (= 8*zeta/(pi^2*D^4))";
          annotation(Documentation(info = "<html>

</html>"));
        algorithm 
          k:=(8 * zeta) / (Modelica.Constants.pi * Modelica.Constants.pi * D * D * D * D);
        end lossConstant_D_zeta;
        package QuadraticTurbulent "Pressure loss components that are mainly defined by a quadratic turbulent regime with constant loss factor data"
          record LossFactorData "Data structure defining constant loss factor data for dp = zeta*rho*v*|v|/2 and functions providing the data for some loss types"
            extends Modelica.Icons.Record;
            SI.Diameter diameter_a "Diameter at port_a" annotation(Dialog);
            SI.Diameter diameter_b "Diameter at port_b" annotation(Dialog);
            Real zeta1 "Loss factor for flow port_a -> port_b" annotation(Dialog);
            Real zeta2 "Loss factor for flow port_b -> port_a" annotation(Dialog);
            SI.ReynoldsNumber Re_turbulent "Loss factors suited for Re >= Re_turbulent" annotation(Dialog);
            SI.Diameter D_Re "Diameter used to compute Re" annotation(Dialog);
            Boolean zeta1_at_a = true "dp = zeta1*(if zeta1_at_a then rho_a*v_a^2/2 else rho_b*v_b^2/2)" annotation(Dialog);
            Boolean zeta2_at_a = false "dp = -zeta2*(if zeta2_at_a then rho_a*v_a^2/2 else rho_b*v_b^2/2)" annotation(Dialog);
            Boolean zetaLaminarKnown = false "= true, if zeta = c0/Re in laminar region" annotation(Dialog);
            Real c0 = 1 "zeta = c0/Re; dp = zeta*rho_Re*v_Re^2/2, Re=v_Re*D_Re*rho_Re/mu_Re)" annotation(Dialog(enable = zetaLaminarKnown));
            annotation(preferedView = "info", Documentation(info = "<html>
<p>
This record defines the pressure loss factors of a pipe
segment (orifice, bending etc.) with a minimum amount of data.
If available, data should be provided for <b>both flow directions</b>,
i.e., flow from port_a to port_b and from port_b to port_a,
as well as for the <b>laminar</b> and the <b>turbulent</b> region.
It is also an option to provide the loss factor <b>only</b> for the
<b>turbulent</b> region for a flow from port_a to port_b.
</p>
<p>
The following equations are used:
</p>
<pre>   &Delta;p = 0.5*&zeta;*&rho;*v*|v|
      = 0.5*&zeta;/A^2 * (1/&rho;) * m_flow*|m_flow|
      = 8*&zeta;/(&pi;^2*D^4*&rho;) * m_flow*|m_flow|
        Re = |v|*D*&rho;/&mu;
</pre>
<table border=1 cellspacing=0 cellpadding=2>
<tr><td><b>flow type</b></td>
    <td><b>&zeta;</b> = </td>
    <td><b>flow region</b></td></tr>
<tr><td>turbulent</td>
    <td><b>zeta1</b> = const.</td>
    <td>Re &ge;  Re_turbulent, v &ge; 0</td></tr>
<tr><td></td>
    <td><b>zeta2</b> = const.</td>
    <td>Re &ge; Re_turbulent, v &lt; 0</td></tr>
<tr><td>laminar</td>
    <td><b>c0</b>/Re</td>
    <td>both flow directions, Re small; c0 = const.</td></tr>
</table>
<p>
where
</p>
<ul>
<li> &Delta;p is the pressure drop: &Delta;p = port_a.p - port_b.p</li>
<li> v is the mean velocity.</li>
<li> &rho; is the density.</li>
<li> &zeta; is the loss factor that depends on the geometry of
     the pipe. In the turbulent flow regime, it is assumed that
     &zeta; is constant and is given by \"zeta1\" and
     \"zeta2\" depending on the flow direction.<br>
     When the Reynolds number Re is below \"Re_turbulent\", the
     flow is laminar for small flow velocities. For higher
     velocities there is a transition region from
     laminar to turbulent flow. The loss factor for
     laminar flow at small velocities is defined by the often occuring
     approximation c0/Re. If c0 is different for the two
     flow directions, the mean value has to be used
     (c0 = (c0_ab + c0_ba)/2).<li>
<li> The equation \"&Delta;p = 0.5*&zeta;*&rho;*v*|v|\" is either with
     respect to port_a or to port_b, depending on the definition
     of the particular loss factor &zeta; (in some references loss
     factors are defined with respect to port_a, in other references
     with respect to port_b).</li>

<li> Re = |v|*D_Re*&rho;/&mu; = |m_flow|*D_Re/(A_Re*&mu;)
     is the Reynolds number at the smallest cross
     section area. This is often at port_a or at port_b, but can
     also be between the two ports. In the record, the diameter
     D_Re of this smallest cross section area has to be provided, as
     well, as Re_turbulent, the absolute value of the
     Reynolds number at which
     the turbulent flow starts. If Re_turbulent is different for
     the two flow directions, use the smaller value as Re_turbulent.</li>
<li> D is the diameter of the pipe. If the pipe has not a
     circular cross section, D = 4*A/P, where A is the cross section
     area and P is the wetted perimeter.</li>
<li> A is the cross section area with A = &pi;(D/2)^2.
<li> &mu; is the dynamic viscosity.</li>
</ul>
<p>
The laminar and the transition region is usually of
not much technical interest because the operating point is
mostly in the turbulent regime. For simplification and for
numercial reasons, this whole region is described by two
polynomials of third order, one polynomial for m_flow &ge; 0
and one for m_flow &lt; 0. The polynomials start at
Re = |m_flow|*4/(&pi;*D_Re*&mu;), where D_Re is the
smallest diameter between port_a and port_b.
The common derivative
of the two polynomials at Re = 0 is
computed from the equation \"c0/Re\". Note, the pressure drop
equation above in the laminar region is always defined
with respect to the smallest diameter D_Re.
</p>
<p>
If no data for c0 is available, the derivative at Re = 0 is computed in such
a way, that the second derivatives of the two polynomials
are identical at Re = 0. The polynomials are constructed, such that
they smoothly touch the characteristic curves in the turbulent
regions. The whole characteristic is therefore <b>continuous</b>
and has a <b>finite</b>, <b>continuous first derivative everywhere</b>.
In some cases, the constructed polynomials would \"vibrate\". This is
avoided by reducing the derivative at Re=0 in such a way that
the polynomials are guaranteed to be monotonically increasing.
The used sufficient criteria for monotonicity follows from:
</p>

<dl>
<dt> Fritsch F.N. and Carlson R.E. (1980):</dt>
<dd> <b>Monotone piecewise cubic interpolation</b>.
     SIAM J. Numerc. Anal., Vol. 17, No. 2, April 1980, pp. 238-246</dd>
</dl>
</html>"));
            encapsulated function wallFriction "Return pressure loss data due to friction in a straight pipe with walls of nonuniform roughness (not useful for smooth pipes, since zeta is no function of Re)"
              import Modelica.Fluid.Fittings.BaseClasses.QuadraticTurbulent.LossFactorData;
              import lg = Modelica.Math.log10;
              import SI = Modelica.SIunits;
              input SI.Length length "Length of pipe" annotation(Dialog);
              input SI.Diameter diameter "Inner diameter of pipe" annotation(Dialog);
              input SI.Length roughness(min = 1e-10) "Absolute roughness of pipe (> 0 required, details see info layer)" annotation(Dialog);
              output LossFactorData data "Pressure loss factors for both flow directions";
              annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,50},{100, -50}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,64},{100, -64}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Backward),Rectangle(extent = {{ -100,50},{100, -49}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -60, -49},{ -60,50}}, color = {0,0,255}, arrow = {Arrow.Filled,Arrow.Filled}),Text(extent = {{ -50,16},{6, -10}}, lineColor = {0,0,255}, textString = "diameter"),Line(points = {{ -100,74},{100,74}}, color = {0,0,255}, arrow = {Arrow.Filled,Arrow.Filled}),Text(extent = {{ -34,92},{34,74}}, lineColor = {0,0,255}, textString = "length")}), Documentation(info = "<html>
<p>
Friction in straight pipe with walls of nonuniform roughness
(commercial pipes) in the region that does not depend on the Reynolds-number
</p>
<p>
The loss factors are given for mass flow rates from
port_a to port_b as:
</p>
<pre>
  turbulent flow (Idelchik 1994, diagram 2-5, p. 117)
     zeta = (L/D)/(2*lg(3.7 / &Delta;))^2, for Re >= 560/&Delta;
&nbsp;
     for Re &ge; 560/&Delta; the loss factor does not depend on the
     Reynolds number. For Re &ge; 4000, the flow is turbulent,
     but depends both on &Delta; and slightly on Re.
&nbsp;
  laminar flow (Idelchick 1994, diagram 2-1, p. 110):
     zeta = 64*(L/D)/Re
</pre>
<p>
where
</p>
<ul>
<li> D is the inner pipe diameter</li>
<li> L is the lenght of the pipe</li>
<li> &Delta; = &delta;/D is the relative roughness where &delta; is
     the absolute \"roughness\", i.e., the averaged height of asperities in the pipe.
     (&delta; may change over time due to growth of surface asperities during
      service, see [Idelchick 1994, p. 85, Tables 2-1, 2-2]).</li>
</ul>

<p>
Since the LossFactorData record can only describe loss factors that depend
on geometry (but, e.g., not on the Reynolds number), only the region
with Re &ge; 560/&Delta; is described by this data. Still, the turbulent
region with the above zeta is defined to start at Re=4000, since otherwise
the approximation for Re &lt; 560/&Delta; is too bad.
</p>

<p>
The absolute roughness <font face=\"Symbol\">d</font> has usually to
be estimated. In <i>[Idelchik 1994, pp. 105-109,
Table 2-5; Miller 1990, p. 190, Table 8-1]</i> many examples are given.
As a short summary:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td><b>Smooth pipes</b></td>
      <td>Drawn brass, coper, aluminium, glass, etc.</td>
      <td><font face=\"Symbol\">d</font> = 0.0025 mm</td>
  </tr>
  <tr><td rowspan=\"3\"><b>Steel pipes</b></td>
      <td>New smooth pipes</td>
      <td><font face=\"Symbol\">d</font> = 0.025 mm</td>
  </tr>
  <tr><td>Mortar lined, average finish</td>
      <td><font face=\"Symbol\">d</font> = 0.1 mm</td>
  </tr>
  <tr><td>Heavy rust</td>
      <td><font face=\"Symbol\">d</font> = 1 mm</td>
  </tr>
  <tr><td rowspan=\"3\"><b>Concrete pipes</b></td>
      <td>Steel forms, first class workmanship</td>
      <td><font face=\"Symbol\">d</font> = 0.025 mm</td>
  </tr>
  <tr><td>Steel forms, average workmanship</td>
      <td><font face=\"Symbol\">d</font> = 0.1 mm</td>
  </tr>
  <tr><td>Block linings</td>
      <td><font face=\"Symbol\">d</font> = 1 mm</td>
  </tr>
</table>
</html>"));
            protected 
              Real Delta = roughness / diameter "relative roughness";
            algorithm 
              data.diameter_a:=diameter;
              data.diameter_b:=diameter;
              data.zeta1:=length / diameter / (2 * lg(3.7 / Delta)) ^ 2;
              data.zeta2:=data.zeta1;
              data.Re_turbulent:=4000 ">= 560/Delta flow does not depend on Re, but interpolation is bad";
              data.D_Re:=diameter;
              data.zeta1_at_a:=true;
              data.zeta2_at_a:=false;
              data.zetaLaminarKnown:=true;
              data.c0:=64 * length / diameter;
            end wallFriction;
            encapsulated function suddenExpansion "Return pressure loss data for sudden expansion or contraction in a pipe (for both flow directions)"
              import Modelica.Fluid.Fittings.BaseClasses.QuadraticTurbulent.LossFactorData;
              import SI = Modelica.SIunits;
              input SI.Diameter diameter_a "Inner diameter of pipe at port_a" annotation(Dialog);
              input SI.Diameter diameter_b "Inner diameter of pipe at port_b" annotation(Dialog);
              output LossFactorData data "Pressure loss factors for both flow directions";
              annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,40},{0, -40}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{0,100},{100, -100}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,40},{ -100,40},{ -100, -40},{0, -40},{0, -100},{100, -100},{100,100},{0,100},{0,40}}, color = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{0,40},{ -100,40},{ -100, -40},{0, -40},{0, -100},{100, -100},{100,100},{0,100},{0,40}}, color = {0,0,0}),Rectangle(extent = {{ -100,40},{0, -40}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{0,100},{100, -100}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,40},{ -100,40},{ -100, -40},{0, -40},{0, -100},{100, -100},{100,100},{0,100},{0,40}}, color = {0,0,0}),Line(points = {{ -60, -40},{ -60,40}}, color = {0,0,255}, arrow = {Arrow.Filled,Arrow.Filled}),Text(extent = {{ -50,16},{ -26, -10}}, lineColor = {0,0,255}, textString = "diameter_a"),Line(points = {{34, -100},{34,100}}, color = {0,0,255}, arrow = {Arrow.Filled,Arrow.Filled}),Text(extent = {{54,16},{78, -10}}, lineColor = {0,0,255}, textString = "diameter_b")}), Documentation(info = "<html>
<p>
The loss factors are given for mass flow rates from
port_a to port_b as:
</p>
<pre>
   A_a &lt; A_b (Idelchik 1994, diagram 4-1, p. 208):
      zeta = dp/(rho_a*v_a^2/2)
           = (1 - A_a/A_b)^2 for Re_a &ge; 3.3e3 (turbulent flow)
      zeta = 30/Re           for Re_a &lt; 10    (laminar flow)
&nbsp;
   A_a &gt; A_b (Idelchik 1994, diagram 4-9, p. 216 and diagram 4-10, p. 217)
      zeta = dp/(rho_b*v_b^2/2)
           = 0.5*(1 - A_b/A_a)^0.75 for Re_b &ge; 1e4 (turbulent flow)
      zeta = 30/Re                  for Re_a &lt; 10  (laminar flow)
</pre>
</html>"));
            protected 
              Real A_rel;
            algorithm 
              data.diameter_a:=diameter_a;
              data.diameter_b:=diameter_b;
              data.Re_turbulent:=100;
              data.zetaLaminarKnown:=true;
              data.c0:=30;
              if diameter_a <= diameter_b then 
                            A_rel:=(diameter_a / diameter_b) ^ 2;
              data.zeta1:=(1 - A_rel) ^ 2;
              data.zeta2:=0.5 * (1 - A_rel) ^ 0.75;
              data.zeta1_at_a:=true;
              data.zeta2_at_a:=true;
              data.D_Re:=diameter_a;

              else               A_rel:=(diameter_b / diameter_a) ^ 2;
              data.zeta1:=0.5 * (1 - A_rel) ^ 0.75;
              data.zeta2:=(1 - A_rel) ^ 2;
              data.zeta1_at_a:=false;
              data.zeta2_at_a:=false;
              data.D_Re:=diameter_b;

              end if;
            end suddenExpansion;
            encapsulated function sharpEdgedOrifice "Return pressure loss data for sharp edged orifice (for both flow directions)"
              import NonSI = Modelica.SIunits.Conversions.NonSIunits;
              import Modelica.Fluid.Fittings.BaseClasses.QuadraticTurbulent.LossFactorData;
              import SI = Modelica.SIunits;
              input SI.Diameter diameter "Inner diameter of pipe (= same at port_a and port_b)" annotation(Dialog);
              input SI.Diameter leastDiameter "Smallest diameter of orifice" annotation(Dialog);
              input SI.Diameter length "Length of orifice" annotation(Dialog);
              input NonSI.Angle_deg alpha "Angle of orifice" annotation(Dialog);
              output LossFactorData data "Pressure loss factors for both flow directions";
              annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,60},{100, -60}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -30,60},{ -30,12},{30,50},{30,60},{ -30,60}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Backward),Polygon(points = {{ -30, -10},{ -30, -60},{30, -60},{30, -50},{ -30, -10}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Backward)}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,60},{100, -60}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -30,60},{ -30,12},{30,50},{30,60},{ -30,60}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Backward),Polygon(points = {{ -30, -10},{ -30, -60},{30, -60},{30, -50},{ -30, -10}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Backward),Line(points = {{ -82, -60},{ -82,60}}, color = {0,0,255}, arrow = {Arrow.Filled,Arrow.Filled}),Text(extent = {{ -78,16},{ -44, -8}}, lineColor = {0,0,255}, textString = "diameter"),Line(points = {{ -30, -10},{ -30,12}}, color = {0,0,255}, arrow = {Arrow.Filled,Arrow.Filled}),Text(extent = {{ -24,14},{8, -10}}, lineColor = {0,0,255}, textString = "leastDiameter"),Text(extent = {{ -20,84},{18,70}}, lineColor = {0,0,255}, textString = "length"),Line(points = {{30,68},{ -30,68}}, color = {0,0,255}, arrow = {Arrow.Filled,Arrow.Filled}),Line(points = {{16,40},{32,18},{36, -2},{34, -20},{20, -42}}, color = {0,0,255}, arrow = {Arrow.Filled,Arrow.Filled}),Text(extent = {{38,8},{92, -6}}, lineColor = {0,0,255}, textString = "alpha")}), Documentation(info = "<html>
<p>
Loss factor for mass flow rate from port_a to port_b
(Idelchik 1994, diagram 4-14, p. 221):
</p>
<pre>
   zeta = [(1-A0/A1) + 0.707*(1-A0/A1)^0.375]^2*(A1/A0)^2
          for Re(A0) >= 1e5,  independent of alpha
</pre>
<p>
Loss factor for mass flow rate from port_b to port_a
(Idelchik 1994, diagram 4-13, p. 220, with A2=A1):
</p>
<pre>
   zeta = k*(1 - A0/A1)^0.75 + (1 - A0/A1)^2 + 2*sqrt(k*(1-A0/A1)^0.375) + (1- A0/A1)
          k  = 0.13 + 0.34*10^(-(3.4*LD+88.4*LD^2.3))
               (there is a typing error in the formula in diagram 4-13, the above
                equation corresponds to table (a) in diagram 4-12)
          LD = L/D0
          for Re(A0) >= 1e4, 40 deg &le; alpha &le; 60 deg
                             for other values of alpha, k is given as table
                             in diagram 3-7 (this is not yet included in the function)
</pre
</html>"));
            protected 
              Real D_rel = leastDiameter / diameter;
              Real LD = length / leastDiameter;
              Real k = 0.13 + 0.34 * 10 ^ ( -(3.4 * LD + 88.4 * LD ^ 2.3));
            algorithm 
              data.diameter_a:=diameter;
              data.diameter_b:=diameter;
              data.zeta1:=(1 - D_rel + 0.707 * (1 - D_rel) ^ 0.375) ^ 2 * (1 / D_rel) ^ 2;
              data.zeta2:=k * (1 - D_rel) ^ 0.75 + (1 - D_rel) ^ 2 + 2 * sqrt(k * (1 - D_rel) ^ 0.375) + 1 - D_rel;
              data.Re_turbulent:=10000.0;
              data.D_Re:=leastDiameter;
              data.zeta1_at_a:=true;
              data.zeta2_at_a:=false;
              data.zetaLaminarKnown:=false;
              data.c0:=0;
            end sharpEdgedOrifice;
          end LossFactorData;
          function massFlowRate_dp "Return mass flow rate from constant loss factor data and pressure drop (m_flow = f(dp))"
            extends Modelica.Icons.Function;
            input SI.Pressure dp "Pressure drop (dp = port_a.p - port_b.p)";
            input SI.Density rho_a "Density at port_a";
            input SI.Density rho_b "Density at port_b";
            input LossFactorData data "Constant loss factors for both flow directions" annotation(choices(choice = Modelica.Fluid.Fittings.Utilities.QuadraticTurbulent.LossFactorData.wallFriction(), choice = Modelica.Fluid.Fittings.Utilities.QuadraticTurbulent.LossFactorData.suddenExpansion(), choice = Modelica.Fluid.Fittings.Utilities.QuadraticTurbulent.LossFactorData.sharpEdgedOrifice()));
            input SI.AbsolutePressure dp_small = 1 "Turbulent flow if |dp| >= dp_small";
            output SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
            annotation(smoothOrder = 1, Documentation(info = "<html>
<p>
Compute mass flow rate from constant loss factor and pressure drop (m_flow = f(dp)).
For small pressure drops (dp &lt; dp_small), the characteristic is approximated by
a polynomial in order to have a finite derivative at zero mass flow rate.
</p>
</html>"));
          protected 
            Real k1 = lossConstant_D_zeta(if data.zeta1_at_a then data.diameter_a else data.diameter_b, data.zeta1);
            Real k2 = lossConstant_D_zeta(if data.zeta2_at_a then data.diameter_a else data.diameter_b, data.zeta2);
          algorithm 
            m_flow:=Utilities.regRoot2(dp, dp_small, rho_a / k1, rho_b / k2);
          end massFlowRate_dp;
          annotation(Documentation(info = "<html>
<p>
This library provides pressure loss factors of a pipe
segment (orifice, bending etc.) with a minimum amount of data.
If available, data can be provided for <b>both flow directions</b>,
i.e., flow from port_a to port_b and from port_b to port_a,
as well as for the <b>laminar</b> and the <b>turbulent</b> region.
It is also an option to provide the loss factor <b>only</b> for the
<b>turbulent</b> region for a flow from port_a to port_b.
Basically, the pressure drop is defined by the following
equation:
</p>
<pre>   &Delta;p = 0.5*&zeta;*&rho;*v*|v|
      = 0.5*&zeta;/A^2 * (1/&rho;) * m_flow*|m_flow|
      = 8*&zeta;/(&pi;^2*D^4*&rho;) * m_flow*|m_flow|
</pre>
<p>
where
</p>
<ul>
<li> &Delta;p is the pressure drop: &Delta;p = port_a.p - port_b.p</li>
<li> v is the mean velocity.</li>
<li> &rho; is the density.</li>
<li> &zeta; is the loss factor that depends on the geometry of
     the pipe. In the turbulent flow regime, it is assumed that
     &zeta; is constant and is given by \"zeta1\" and
     \"zeta2\" depending on the flow direction.<li>
<li> D is the diameter of the pipe segment. If this is not a
     circular cross section, D = 4*A/P, where A is the cross section
     area and P is the wetted perimeter.</li>
</ul>

</html>"));
          function massFlowRate_dp_and_Re "Return mass flow rate from constant loss factor data, pressure drop and Re (m_flow = f(dp))"
            extends Modelica.Icons.Function;
            input SI.Pressure dp "Pressure drop (dp = port_a.p - port_b.p)";
            input SI.Density rho_a "Density at port_a";
            input SI.Density rho_b "Density at port_b";
            input SI.DynamicViscosity mu_a "Dynamic viscosity at port_a";
            input SI.DynamicViscosity mu_b "Dynamic viscosity at port_b";
            input LossFactorData data "Constant loss factors for both flow directions" annotation(choices(choice = BaseClasses.PressureLosses.QuadraticTurbulent.LossFactorData.wallFriction(), choice = BaseClasses.PressureLosses.QuadraticTurbulent.LossFactorData.suddenExpansion(), choice = BaseClasses.PressureLosses.QuadraticTurbulent.LossFactorData.sharpEdgedOrifice()));
            output SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
            annotation(smoothOrder = 1, Documentation(info = "<html>
<p>
Compute mass flow rate from constant loss factor and pressure drop (m_flow = f(dp)).
If the Reynolds-number Re &ge; data.Re_turbulent, the flow
is treated as a turbulent flow with constant loss factor zeta.
If the Reynolds-number Re &lt; data.Re_turbulent, the flow
is laminar and/or in a transition region between laminar and
turbulent. This region is approximated by two
polynomials of third order, one polynomial for m_flow &ge; 0
and one for m_flow &lt; 0.
The common derivative
of the two polynomials at Re = 0 is
computed from the equation \"data.c0/Re\".
</p>
<p>
If no data for c0 is available, the derivative at Re = 0 is computed in such
a way, that the second derivatives of the two polynomials
are identical at Re = 0. The polynomials are constructed, such that
they smoothly touch the characteristic curves in the turbulent
regions. The whole characteristic is therefore <b>continuous</b>
and has a <b>finite</b>, <b>continuous first derivative everywhere</b>.
In some cases, the constructed polynomials would \"vibrate\". This is
avoided by reducing the derivative at Re=0 in such a way that
the polynomials are guaranteed to be monotonically increasing.
The used sufficient criteria for monotonicity follows from:
</p>

<dl>
<dt> Fritsch F.N. and Carlson R.E. (1980):</dt>
<dd> <b>Monotone piecewise cubic interpolation</b>.
     SIAM J. Numerc. Anal., Vol. 17, No. 2, April 1980, pp. 238-246</dd>
</dl>
</html>"));
          protected 
            constant Real pi = Modelica.Constants.pi;
            Real k0 = (2 * data.c0) / (pi * data.D_Re ^ 3);
            Real k1 = lossConstant_D_zeta(if data.zeta1_at_a then data.diameter_a else data.diameter_b, data.zeta1);
            Real k2 = lossConstant_D_zeta(if data.zeta2_at_a then data.diameter_a else data.diameter_b, data.zeta2);
            Real yd0 "Derivative of m_flow=m_flow(dp) at zero, if data.zetaLaminarKnown";
            SI.AbsolutePressure dp_turbulent "The turbulent region is: |dp| >= dp_turbulent";
          algorithm 
            dp_turbulent:=(k1 + k2) / (rho_a + rho_b) * (((mu_a + mu_b) * data.D_Re * pi) / 8) ^ 2 * data.Re_turbulent ^ 2;
            yd0:=if data.zetaLaminarKnown then (rho_a + rho_b) / (k0 * (mu_a + mu_b)) else 0;
            m_flow:=Utilities.regRoot2(dp, dp_turbulent, rho_a / k1, rho_b / k2, data.zetaLaminarKnown, yd0);
          end massFlowRate_dp_and_Re;
          function pressureLoss_m_flow "Return pressure drop from constant loss factor and mass flow rate (dp = f(m_flow))"
            extends Modelica.Icons.Function;
            input SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
            input SI.Density rho_a "Density at port_a";
            input SI.Density rho_b "Density at port_b";
            input LossFactorData data "Constant loss factors for both flow directions" annotation(choices(choice = BaseClasses.PressureLosses.QuadraticTurbulent.LossFactorData.wallFriction(), choice = BaseClasses.PressureLosses.QuadraticTurbulent.LossFactorData.suddenExpansion(), choice = BaseClasses.PressureLosses.QuadraticTurbulent.LossFactorData.sharpEdgedOrifice()));
            input SI.MassFlowRate m_flow_small = 0.01 "Turbulent flow if |m_flow| >= m_flow_small";
            output SI.Pressure dp "Pressure drop (dp = port_a.p - port_b.p)";
            annotation(smoothOrder = 1, Documentation(info = "<html>
<p>
Compute pressure drop from constant loss factor and mass flow rate (dp = f(m_flow)).
For small mass flow rates(|m_flow| &lt; m_flow_small), the characteristic is approximated by
a polynomial in order to have a finite derivative at zero mass flow rate.
</p>
</html>"));
          protected 
            Real k1 = lossConstant_D_zeta(if data.zeta1_at_a then data.diameter_a else data.diameter_b, data.zeta1);
            Real k2 = lossConstant_D_zeta(if data.zeta2_at_a then data.diameter_a else data.diameter_b, data.zeta2);
          algorithm 
            dp:=Utilities.regSquare2(m_flow, m_flow_small, k1 / rho_a, k2 / rho_b);
          end pressureLoss_m_flow;
          function pressureLoss_m_flow_and_Re "Return pressure drop from constant loss factor, mass flow rate and Re (dp = f(m_flow))"
            extends Modelica.Icons.Function;
            input SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
            input SI.Density rho_a "Density at port_a";
            input SI.Density rho_b "Density at port_b";
            input SI.DynamicViscosity mu_a "Dynamic viscosity at port_a";
            input SI.DynamicViscosity mu_b "Dynamic viscosity at port_b";
            input LossFactorData data "Constant loss factors for both flow directions" annotation(choices(choice = BaseClasses.PressureLosses.QuadraticTurbulent.LossFactorData.wallFriction(), choice = BaseClasses.PressureLosses.QuadraticTurbulent.LossFactorData.suddenExpansion(), choice = BaseClasses.PressureLosses.QuadraticTurbulent.LossFactorData.sharpEdgedOrifice()));
            output SI.Pressure dp "Pressure drop (dp = port_a.p - port_b.p)";
            annotation(smoothOrder = 1, Documentation(info = "<html>
<p>
Compute pressure drop from constant loss factor and mass flow rate (dp = f(m_flow)).
If the Reynolds-number Re &ge; data.Re_turbulent, the flow
is treated as a turbulent flow with constant loss factor zeta.
If the Reynolds-number Re &lt; data.Re_turbulent, the flow
is laminar and/or in a transition region between laminar and
turbulent. This region is approximated by two
polynomials of third order, one polynomial for m_flow &ge; 0
and one for m_flow &lt; 0.
The common derivative
of the two polynomials at Re = 0 is
computed from the equation \"data.c0/Re\".
</p>
<p>
If no data for c0 is available, the derivative at Re = 0 is computed in such
a way, that the second derivatives of the two polynomials
are identical at Re = 0. The polynomials are constructed, such that
they smoothly touch the characteristic curves in the turbulent
regions. The whole characteristic is therefore <b>continuous</b>
and has a <b>finite</b>, <b>continuous first derivative everywhere</b>.
In some cases, the constructed polynomials would \"vibrate\". This is
avoided by reducing the derivative at Re=0 in such a way that
the polynomials are guaranteed to be monotonically increasing.
The used sufficient criteria for monotonicity follows from:
</p>

<dl>
<dt> Fritsch F.N. and Carlson R.E. (1980):</dt>
<dd> <b>Monotone piecewise cubic interpolation</b>.
     SIAM J. Numerc. Anal., Vol. 17, No. 2, April 1980, pp. 238-246</dd>
</dl>
</html>"));
          protected 
            constant Real pi = Modelica.Constants.pi;
            Real k0 = (2 * data.c0) / (pi * data.D_Re ^ 3);
            Real k1 = lossConstant_D_zeta(if data.zeta1_at_a then data.diameter_a else data.diameter_b, data.zeta1);
            Real k2 = lossConstant_D_zeta(if data.zeta2_at_a then data.diameter_a else data.diameter_b, data.zeta2);
            Real yd0 "Derivative of dp = f(m_flow) at zero, if data.zetaLaminarKnown";
            SI.MassFlowRate m_flow_turbulent "The turbulent region is: |m_flow| >= m_flow_turbulent";
          algorithm 
            m_flow_turbulent:=pi / 8 * data.D_Re * (mu_a + mu_b) * data.Re_turbulent;
            yd0:=if data.zetaLaminarKnown then (k0 * (mu_a + mu_b)) / (rho_a + rho_b) else 0;
            dp:=Utilities.regSquare2(m_flow, m_flow_turbulent, k1 / rho_a, k2 / rho_b, data.zetaLaminarKnown, yd0);
          end pressureLoss_m_flow_and_Re;
          partial model BaseModel "Generic pressure drop component with constant turbulent loss factor data and without an icon"
            extends Modelica.Fluid.Interfaces.PartialTwoPortTransport;
            extends Modelica.Fluid.Interfaces.PartialLumpedFlow(final pathLength = 0, final momentumDynamics = Types.Dynamics.SteadyState);
            parameter LossFactorData data "Loss factor data";
            parameter Boolean from_dp = true "= true, use m_flow = f(dp) else dp = f(m_flow)" annotation(Evaluate = true, Dialog(tab = "Advanced"));
            parameter Boolean use_Re = false "= true, if turbulent region is defined by Re, otherwise by dp_small or m_flow_small" annotation(Evaluate = true, Dialog(tab = "Advanced"));
            parameter Medium.AbsolutePressure dp_small = system.dp_small "Turbulent flow if |dp| >= dp_small" annotation(Dialog(tab = "Advanced", enable = not use_Re and from_dp));
            parameter Medium.MassFlowRate m_flow_small = system.m_flow_small "Turbulent flow if |m_flow| >= m_flow_small" annotation(Dialog(tab = "Advanced", enable = not from_dp));
            parameter Boolean show_Re = false "= true, if Reynolds number is included for plotting" annotation(Evaluate = true, Dialog(tab = "Advanced", group = "Diagnostics"));
            SI.ReynoldsNumber Re = Modelica.Fluid.Pipes.BaseClasses.CharacteristicNumbers.ReynoldsNumber_m_flow(m_flow, noEvent(if m_flow > 0 then Medium.dynamicViscosity(state_a) else Medium.dynamicViscosity(state_b)), data.D_Re) if show_Re "Reynolds number at diameter data.D_Re";
            Modelica.SIunits.Pressure dp_fg "pressure loss due to friction and gravity";
            Modelica.SIunits.Area A_mean = (Modelica.Constants.pi / 4 * (data.diameter_a ^ 2 + data.diameter_b ^ 2)) / 2 "mean cross flow area";

          equation 
            Ib_flow = 0;
            F_p = A_mean * (Medium.pressure(state_b) - Medium.pressure(state_a));
            F_fg = A_mean * dp_fg;
            if from_dp then
              m_flow = if use_Re then massFlowRate_dp_and_Re(dp_fg, Medium.density(state_a), Medium.density(state_b), Medium.dynamicViscosity(state_a), Medium.dynamicViscosity(state_b), data) else massFlowRate_dp(dp_fg, Medium.density(state_a), Medium.density(state_b), data, dp_small);

            else
              dp_fg = if use_Re then pressureLoss_m_flow_and_Re(m_flow, Medium.density(state_a), Medium.density(state_b), Medium.dynamicViscosity(state_a), Medium.dynamicViscosity(state_b), data) else pressureLoss_m_flow(m_flow, Medium.density(state_a), Medium.density(state_b), data, m_flow_small);

            end if;
            port_a.h_outflow = inStream(port_b.h_outflow);
            port_b.h_outflow = inStream(port_a.h_outflow);
            annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
This model computes the pressure loss of a pipe
segment (orifice, bending etc.) with a minimum amount of data
provided via parameter <b>data</b>.
If available, data should be provided for <b>both flow directions</b>,
i.e., flow from port_a to port_b and from port_b to port_a,
as well as for the <b>laminar</b> and the <b>turbulent</b> region.
It is also an option to provide the loss factor <b>only</b> for the
<b>turbulent</b> region for a flow from port_a to port_b.
</p>
<p>
The following equations are used:
</p>
<pre>   &Delta;p = 0.5*&zeta;*&rho;*v*|v|
      = 0.5*&zeta;/A^2 * (1/&rho;) * m_flow*|m_flow|
        Re = |v|*D*&rho;/&mu;
</pre>
<table border=1 cellspacing=0 cellpadding=2>
<tr><td><b>flow type</b></td>
    <td><b>&zeta;</b> = </td>
    <td><b>flow region</b></td></tr>
<tr><td>turbulent</td>
    <td><b>zeta1</b> = const.</td>
    <td>Re &ge;  Re_turbulent, v &ge; 0</td></tr>
<tr><td></td>
    <td><b>zeta2</b> = const.</td>
    <td>Re &ge; Re_turbulent, v &lt; 0</td></tr>
<tr><td>laminar</td>
    <td><b>c0</b>/Re</td>
    <td>both flow directions, Re small; c0 = const.</td></tr>
</table>
<p>
where
</p>
<ul>
<li> &Delta;p is the pressure drop: &Delta;p = port_a.p - port_b.p</li>
<li> v is the mean velocity.</li>
<li> &rho; is the density.</li>
<li> &zeta; is the loss factor that depends on the geometry of
     the pipe. In the turbulent flow regime, it is assumed that
     &zeta; is constant and is given by \"zeta1\" and
     \"zeta2\" depending on the flow direction.<br>
     When the Reynolds number Re is below \"Re_turbulent\", the
     flow is laminar for small flow velocities. For higher
     velocities there is a transition region from
     laminar to turbulent flow. The loss factor for
     laminar flow at small velocities is defined by the often occuring
     approximation c0/Re. If c0 is different for the two
     flow directions, the mean value has to be used
     (c0 = (c0_ab + c0_ba)/2).<li>
<li> The equation \"&Delta;p = 0.5*&zeta;*&rho;*v*|v|\" is either with
     respect to port_a or to port_b, depending on the definition
     of the particular loss factor &zeta; (in some references loss
     factors are defined with respect to port_a, in other references
     with respect to port_b).</li>

<li> Re = |v|*D_Re*&rho;/&mu; = |m_flow|*D_Re/(A_Re*&mu;)
     is the Reynolds number at the smallest cross
     section area. This is often at port_a or at port_b, but can
     also be between the two ports. In the record, the diameter
     D_Re of this smallest cross section area has to be provided, as
     well, as Re_turbulent, the absolute value of the
     Reynolds number at which
     the turbulent flow starts. If Re_turbulent is different for
     the two flow directions, use the smaller value as Re_turbulent.</li>
<li> D is the diameter of the pipe. If the pipe has not a
     circular cross section, D = 4*A/P, where A is the cross section
     area and P is the wetted perimeter.</li>
<li> A is the cross section area with A = &pi;(D/2)^2.
<li> &mu; is the dynamic viscosity.</li>
</ul>
<p>
The laminar and the transition region is usually of
not much technical interest because the operating point is
mostly in the turbulent regime. For simplification and for
numercial reasons, this whole region is described by two
polynomials of third order, one polynomial for m_flow &ge; 0
and one for m_flow &lt; 0. The polynomials start at
Re = |m_flow|*4/(&pi;*D_Re*&mu;), where D_Re is the
smallest diameter between port_a and port_b.
The common derivative
of the two polynomials at Re = 0 is
computed from the equation \"c0/Re\". Note, the pressure drop
equation above in the laminar region is always defined
with respect to the smallest diameter D_Re.
</p>
<p>
If no data for c0 is available, the derivative at Re = 0 is computed in such
a way, that the second derivatives of the two polynomials
are identical at Re = 0. The polynomials are constructed, such that
they smoothly touch the characteristic curves in the turbulent
regions. The whole characteristic is therefore <b>continuous</b>
and has a <b>finite</b>, <b>continuous first derivative everywhere</b>.
In some cases, the constructed polynomials would \"vibrate\". This is
avoided by reducing the derivative at Re=0 in such a way that
the polynomials are guaranteed to be monotonically increasing.
The used sufficient criteria for monotonicity follows from:
</p>

<dl>
<dt> Fritsch F.N. and Carlson R.E. (1980):</dt>
<dd> <b>Monotone piecewise cubic interpolation</b>.
     SIAM J. Numerc. Anal., Vol. 17, No. 2, April 1980, pp. 238-246</dd>
</dl>
</html>"));
          end BaseModel;
          model TestWallFriction "Pressure drop in pipe due to wall friction (only for test purposes; if needed use Pipes.StaticPipe instead)"
            extends BaseModel(final data = LossFactorData.wallFriction(length, diameter, roughness));
            parameter SI.Length length "Length of pipe";
            parameter SI.Diameter diameter "Inner diameter of pipe";
            parameter SI.Length roughness(min = 1e-10) "Absolute roughness of pipe (> 0 required, details see info layer)";
            annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -150,80},{150,120}}, lineColor = {0,0,0}, textString = "%name"),Rectangle(extent = {{ -100,60},{100, -60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Rectangle(extent = {{ -100,34},{100, -36}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,127,255}),Text(extent = {{ -134, -66},{130, -92}}, lineColor = {0,0,0}, textString = "quad. turbulent")}), Documentation(info = "<html>

</html>"));
          end TestWallFriction;
          partial model BaseModelNonconstantCrossSectionArea "Generic pressure drop component with constant turbulent loss factor data and without an icon, for non-constant cross section area"
            extends Modelica.Fluid.Interfaces.PartialTwoPortTransport;
            extends Modelica.Fluid.Interfaces.PartialLumpedFlow(final pathLength = 0, final momentumDynamics = Types.Dynamics.SteadyState);
            parameter LossFactorData data "Loss factor data";
            final parameter Boolean from_dp = false "= true, use m_flow = f(dp) else dp = f(m_flow)" annotation(Evaluate = true, Dialog(tab = "Advanced"));
            final parameter Boolean use_Re = false "= true, if turbulent region is defined by Re, otherwise by dp_small or m_flow_small" annotation(Evaluate = true, Dialog(tab = "Advanced"));
            parameter Medium.AbsolutePressure dp_small = system.dp_small "Turbulent flow if |dp| >= dp_small" annotation(Dialog(tab = "Advanced", enable = not use_Re and from_dp));
            parameter Medium.MassFlowRate m_flow_small = system.m_flow_small "Turbulent flow if |m_flow| >= m_flow_small" annotation(Dialog(tab = "Advanced", enable = not from_dp));
            parameter Boolean show_Re = false "= true, if Reynolds number is included for plotting" annotation(Evaluate = true, Dialog(tab = "Advanced", group = "Diagnostics"));
            SI.ReynoldsNumber Re = Modelica.Fluid.Pipes.BaseClasses.CharacteristicNumbers.ReynoldsNumber_m_flow(m_flow, noEvent(if m_flow > 0 then Medium.dynamicViscosity(state_a) else Medium.dynamicViscosity(state_b)), data.D_Re) if show_Re "Reynolds number at diameter data.D_Re";
            parameter Boolean show_totalPressures = false "= true, if total pressures are included for plotting" annotation(Evaluate = true, Dialog(tab = "Advanced", group = "Diagnostics"));
            SI.AbsolutePressure p_total_a = port_a.p + (0.5 * m_flow ^ 2) / ((Modelica.Constants.pi / 4 * data.diameter_a ^ 2) ^ 2 * noEvent(if port_a.m_flow > 0 then Medium.density(state_a) else Medium.density(state_b))) if show_totalPressures "Total pressure at port_a";
            SI.AbsolutePressure p_total_b = port_b.p + (0.5 * m_flow ^ 2) / ((Modelica.Constants.pi / 4 * data.diameter_b ^ 2) ^ 2 * noEvent(if port_b.m_flow > 0 then Medium.density(state_b) else Medium.density(state_a))) if show_totalPressures "Total pressure at port_a";
            parameter Boolean show_portVelocities = false "= true, if port velocities are included for plotting" annotation(Evaluate = true, Dialog(tab = "Advanced", group = "Diagnostics"));
            SI.Velocity v_a = port_a.m_flow / (Modelica.Constants.pi / 4 * data.diameter_a ^ 2 * noEvent(if port_a.m_flow > 0 then Medium.density(state_a) else Medium.density(state_b))) if show_portVelocities "Fluid velocity into port_a";
            SI.Velocity v_b = port_b.m_flow / (Modelica.Constants.pi / 4 * data.diameter_b ^ 2 * noEvent(if port_b.m_flow > 0 then Medium.density(state_b) else Medium.density(state_a))) if show_portVelocities "Fluid velocity into port_b";
            Modelica.SIunits.Pressure dp_fg "pressure loss due to friction and gravity";
            Modelica.SIunits.Area A_mean = (Modelica.Constants.pi / 4 * (data.diameter_a ^ 2 + data.diameter_b ^ 2)) / 2 "mean cross flow area";
            Medium.ThermodynamicState state_b_des "Thermodynamic state at port b for flow a -> b";
            Medium.ThermodynamicState state_a_nondes "Thermodynamic state at port a for flow a <- b";

          equation 
            Ib_flow = 0;
            F_p = A_mean * (Medium.pressure(state_b) - Medium.pressure(state_a));
            F_fg = A_mean * dp_fg;
            if from_dp then
              m_flow = if use_Re then massFlowRate_dp_and_Re(dp_fg, Medium.density(state_a), Medium.density(state_b), Medium.dynamicViscosity(state_a), Medium.dynamicViscosity(state_b), data) else massFlowRate_dp(dp_fg, Medium.density(state_a), Medium.density(state_b), data, dp_small);

            else
              dp_fg = if use_Re then pressureLoss_m_flow_and_Re(m_flow, Medium.density(state_a), Medium.density(state_b), Medium.dynamicViscosity(state_a), Medium.dynamicViscosity(state_b), data) else pressureLoss_m_flow_totalPressure(m_flow, Medium.density(state_a), Medium.density(state_b_des), Medium.density(state_b), Medium.density(state_a_nondes), data, m_flow_small);

            end if;
            port_a.h_outflow = inStream(port_b.h_outflow);
            port_b.h_outflow = inStream(port_a.h_outflow);
            state_b_des = Medium.setState_phX(port_b.p, inStream(port_a.h_outflow), inStream(port_a.Xi_outflow));
            state_a_nondes = Medium.setState_phX(port_a.p, inStream(port_b.h_outflow), inStream(port_b.Xi_outflow));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
This model computes the pressure loss of a pipe
segment (orifice, bending etc.) with a minimum amount of data
provided via parameter <b>data</b>.
If available, data should be provided for <b>both flow directions</b>,
i.e., flow from port_a to port_b and from port_b to port_a,
as well as for the <b>laminar</b> and the <b>turbulent</b> region.
It is also an option to provide the loss factor <b>only</b> for the
<b>turbulent</b> region for a flow from port_a to port_b.
</p>
<p>
The following equations are used:
</p>
<pre>   &Delta;p = 0.5*&zeta;*&rho;*v*|v|
      = 0.5*&zeta;/A^2 * (1/&rho;) * m_flow*|m_flow|
        Re = |v|*D*&rho;/&mu;
</pre>
<table border=1 cellspacing=0 cellpadding=2>
<tr><td><b>flow type</b></td>
    <td><b>&zeta;</b> = </td>
    <td><b>flow region</b></td></tr>
<tr><td>turbulent</td>
    <td><b>zeta1</b> = const.</td>
    <td>Re &ge;  Re_turbulent, v &ge; 0</td></tr>
<tr><td></td>
    <td><b>zeta2</b> = const.</td>
    <td>Re &ge; Re_turbulent, v &lt; 0</td></tr>
<tr><td>laminar</td>
    <td><b>c0</b>/Re</td>
    <td>both flow directions, Re small; c0 = const.</td></tr>
</table>
<p>
where
</p>
<ul>
<li> &Delta;p is the pressure drop: &Delta;p = port_a.p - port_b.p</li>
<li> v is the mean velocity.</li>
<li> &rho; is the density.</li>
<li> &zeta; is the loss factor that depends on the geometry of
     the pipe. In the turbulent flow regime, it is assumed that
     &zeta; is constant and is given by \"zeta1\" and
     \"zeta2\" depending on the flow direction.<br>
     When the Reynolds number Re is below \"Re_turbulent\", the
     flow is laminar for small flow velocities. For higher
     velocities there is a transition region from
     laminar to turbulent flow. The loss factor for
     laminar flow at small velocities is defined by the often occuring
     approximation c0/Re. If c0 is different for the two
     flow directions, the mean value has to be used
     (c0 = (c0_ab + c0_ba)/2).<li>
<li> The equation \"&Delta;p = 0.5*&zeta;*&rho;*v*|v|\" is either with
     respect to port_a or to port_b, depending on the definition
     of the particular loss factor &zeta; (in some references loss
     factors are defined with respect to port_a, in other references
     with respect to port_b).</li>

<li> Re = |v|*D_Re*&rho;/&mu; = |m_flow|*D_Re/(A_Re*&mu;)
     is the Reynolds number at the smallest cross
     section area. This is often at port_a or at port_b, but can
     also be between the two ports. In the record, the diameter
     D_Re of this smallest cross section area has to be provided, as
     well, as Re_turbulent, the absolute value of the
     Reynolds number at which
     the turbulent flow starts. If Re_turbulent is different for
     the two flow directions, use the smaller value as Re_turbulent.</li>
<li> D is the diameter of the pipe. If the pipe has not a
     circular cross section, D = 4*A/P, where A is the cross section
     area and P is the wetted perimeter.</li>
<li> A is the cross section area with A = &pi;(D/2)^2.
<li> &mu; is the dynamic viscosity.</li>
</ul>
<p>
The laminar and the transition region is usually of
not much technical interest because the operating point is
mostly in the turbulent regime. For simplification and for
numercial reasons, this whole region is described by two
polynomials of third order, one polynomial for m_flow &ge; 0
and one for m_flow &lt; 0. The polynomials start at
Re = |m_flow|*4/(&pi;*D_Re*&mu;), where D_Re is the
smallest diameter between port_a and port_b.
The common derivative
of the two polynomials at Re = 0 is
computed from the equation \"c0/Re\". Note, the pressure drop
equation above in the laminar region is always defined
with respect to the smallest diameter D_Re.
</p>
<p>
If no data for c0 is available, the derivative at Re = 0 is computed in such
a way, that the second derivatives of the two polynomials
are identical at Re = 0. The polynomials are constructed, such that
they smoothly touch the characteristic curves in the turbulent
regions. The whole characteristic is therefore <b>continuous</b>
and has a <b>finite</b>, <b>continuous first derivative everywhere</b>.
In some cases, the constructed polynomials would \"vibrate\". This is
avoided by reducing the derivative at Re=0 in such a way that
the polynomials are guaranteed to be monotonically increasing.
The used sufficient criteria for monotonicity follows from:
</p>

<dl>
<dt> Fritsch F.N. and Carlson R.E. (1980):</dt>
<dd> <b>Monotone piecewise cubic interpolation</b>.
     SIAM J. Numerc. Anal., Vol. 17, No. 2, April 1980, pp. 238-246</dd>
</dl>
</html>"));
          end BaseModelNonconstantCrossSectionArea;
          function pressureLoss_m_flow_totalPressure "Return pressure drop from constant loss factor and mass flow rate (dp = f(m_flow))"
            extends Modelica.Icons.Function;
            input SI.MassFlowRate m_flow "Mass flow rate from port_a to port_b";
            input SI.Density rho_a_des "Density at port_a, mass flow in design direction a -> b";
            input SI.Density rho_b_des "Density at port_b, mass flow in design direction a -> b";
            input SI.Density rho_b_nondes "Density at port_b, mass flow against design direction a <- b";
            input SI.Density rho_a_nondes "Density at port_a, mass flow against design direction a <- b";
            input LossFactorData data "Constant loss factors for both flow directions" annotation(choices(choice = BaseClasses.PressureLosses.QuadraticTurbulent.LossFactorData.wallFriction(), choice = BaseClasses.PressureLosses.QuadraticTurbulent.LossFactorData.suddenExpansion(), choice = BaseClasses.PressureLosses.QuadraticTurbulent.LossFactorData.sharpEdgedOrifice()));
            input SI.MassFlowRate m_flow_small = 0.01 "Turbulent flow if |m_flow| >= m_flow_small";
            output SI.Pressure dp "Pressure drop (dp = port_a.p - port_b.p)";
            annotation(smoothOrder = 1, Documentation(info = "<html>
<p>
Compute pressure drop from constant loss factor and mass flow rate (dp = f(m_flow)).
For small mass flow rates(|m_flow| &lt; m_flow_small), the characteristic is approximated by
a polynomial in order to have a finite derivative at zero mass flow rate.
</p>
</html>"));
          protected 
            SI.Area A_a = (Modelica.Constants.pi * data.diameter_a ^ 2) / 4 "Cross section area at port_a";
            SI.Area A_b = (Modelica.Constants.pi * data.diameter_b ^ 2) / 4 "Cross section area at port_b";
          algorithm 
            dp:=1 / 2 * m_flow ^ 2 * (if m_flow > 0 then data.zeta1 / (if data.zeta1_at_a then rho_a_des * A_a ^ 2 else rho_b_des * A_b ^ 2) - 1 / (rho_a_des * A_a ^ 2) + 1 / (rho_b_des * A_b ^ 2) else  -data.zeta2 / (if data.zeta2_at_a then rho_a_nondes * A_a ^ 2 else rho_b_nondes * A_b ^ 2) - 1 / (rho_a_nondes * A_a ^ 2) + 1 / (rho_b_nondes * A_b ^ 2));
          end pressureLoss_m_flow_totalPressure;
        end QuadraticTurbulent;
        partial model PartialTeeJunction "Base class for a splitting/joining component with three ports"
          import Modelica.Fluid.Types;
          import Modelica.Fluid.Types.PortFlowDirection;
          replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium in the component" annotation(choicesAllMatching = true);
          Modelica.Fluid.Interfaces.FluidPort_a port_1(redeclare package Medium = Medium, m_flow(min = if portFlowDirection_1 == PortFlowDirection.Entering then 0.0 else  -Modelica.Constants.inf, max = if portFlowDirection_1 == PortFlowDirection.Leaving then 0.0 else Modelica.Constants.inf)) annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Modelica.Fluid.Interfaces.FluidPort_b port_2(redeclare package Medium = Medium, m_flow(min = if portFlowDirection_2 == PortFlowDirection.Entering then 0.0 else  -Modelica.Constants.inf, max = if portFlowDirection_2 == PortFlowDirection.Leaving then 0.0 else Modelica.Constants.inf)) annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.Fluid.Interfaces.FluidPort_a port_3(redeclare package Medium = Medium, m_flow(min = if portFlowDirection_3 == PortFlowDirection.Entering then 0.0 else  -Modelica.Constants.inf, max = if portFlowDirection_3 == PortFlowDirection.Leaving then 0.0 else Modelica.Constants.inf)) annotation(Placement(transformation(extent = {{ -10,90},{10,110}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,44},{100, -44}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,127,255}),Text(extent = {{ -150, -89},{150, -129}}, lineColor = {0,0,255}, textString = "%name"),Rectangle(extent = {{ -44,100},{44,44}}, lineColor = {0,0,0}, fillPattern = FillPattern.VerticalCylinder, fillColor = {0,127,255}),Rectangle(extent = {{ -22,82},{21, -4}}, fillPattern = FillPattern.Solid, fillColor = {0,128,255}, pattern = LinePattern.None, lineColor = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
        protected 
          parameter PortFlowDirection portFlowDirection_1 = PortFlowDirection.Bidirectional "Flow direction for port_1" annotation(Dialog(tab = "Advanced"));
          parameter PortFlowDirection portFlowDirection_2 = PortFlowDirection.Bidirectional "Flow direction for port_2" annotation(Dialog(tab = "Advanced"));
          parameter PortFlowDirection portFlowDirection_3 = PortFlowDirection.Bidirectional "Flow direction for port_3" annotation(Dialog(tab = "Advanced"));
        end PartialTeeJunction;
      end BaseClasses;
    end Fittings;
  end Fluid;
  package Electrical "Library of electrical models (analog, digital, machines, multi-phase)"
    extends Modelica.Icons.Library2;
    annotation(Documentation(info = "<html>
<p>
This library contains electrical components to build up analog and digital circuits,
as well as machines to model electrical motors and generators,
especially three phase induction machines such as an asynchronous motor.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -29, -13},{3, -27}}, lineColor = {0,0,0}),Line(points = {{37, -58},{62, -58}}, color = {0,0,0}),Line(points = {{36, -49},{61, -49}}, color = {0,0,0}),Line(points = {{ -78, -50},{ -43, -50}}, color = {0,0,0}),Line(points = {{ -67, -55},{ -55, -55}}, color = {0,0,0}),Line(points = {{ -61, -50},{ -61, -20},{ -29, -20}}, color = {0,0,0}),Line(points = {{3, -20},{48, -20},{48, -49}}, color = {0,0,0}),Line(points = {{48, -58},{48, -78},{ -61, -78},{ -61, -55}}, color = {0,0,0})}));
    package Analog "Library for analog electrical models"
      import SI = Modelica.SIunits;
      extends Modelica.Icons.Library2;
      annotation(__Dymola_classOrder = {"Examples","*"}, Documentation(info = "<html>
<p>
This package contains packages for analog electrical components:
<ul>
<li>Basic: basic components (resistor, capacitor, conductor, inductor, transformer, gyrator)</li>
<li>Semiconductors: semiconductor devices (diode, bipolar and MOS transistors)</li>
<li>Lines: transmission lines (lossy and lossless)</li>
<li>Ideal: ideal elements (switches, diode, transformer, idle, short, ...)</li>
<li>Sources: time-dependend and controlled voltage and current sources</li>
<li>Sensors: sensors to measure potential, voltage, and current</li>
</ul>
</p>
<dl>
<dt>
<b>Main Authors:</b></dt>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden</dd>
</dl>


<p>
Copyright &copy; 1998-2009, Modelica Association and Fraunhofer-Gesellschaft.
</p>
<p>
<i>This Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p><br>
</HTML>
"));
      package Examples "Examples that demonstrate the usage of the Analog electrical components"
        extends Modelica.Icons.Library2;
        annotation(Documentation(info = "<html>
<p>This package contains examples that demonstrate the usage of the components of the Electrical.Analog library. </p>
<p>The examples are simple to understand. They will show a typical behavior of the components, and they will give hints to users.</p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden<br>
<p>
<dt>
<b>Copyright:</b>
<dd>
Copyright &copy; 1998-2006, Modelica Association and Fraunhofer-Gesellschaft.<br>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package
Modelica in file \"Modelica/package.mo\".</i><br>
<p>
</dl>
</html>"));
        package Utilities "Utility components used by package Examples"
          extends Modelica.Icons.Library2;
          annotation(Documentation(info = "<html>
<p>This package contains some utility components used by package examples. These components are auxiliary components that should not be used like true MLS components since they are designed the purpose of the examples only, not for common use. </p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden<br>
<p>
<dt>
<b>Copyright:</b>
<dd>
Copyright &copy; 1998-2006, Modelica Association and Fraunhofer-Gesellschaft.<br>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package
Modelica in file \"Modelica/package.mo\".</i><br>
<p>
</dl>
</html>"));
          model Transistor "Transistor with resistance an capacitance"
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -80,80},{80, -80}}, lineColor = {0,0,255}),Line(points = {{80,60},{100,60}}, color = {0,0,255}),Line(points = {{80, -60},{100, -60}}, color = {0,0,255}),Line(points = {{ -100,0},{ -80,0}}, color = {0,0,255}),Line(points = {{ -60,0},{ -10,0}}, color = {0,0,255}),Line(points = {{ -10,40},{ -10, -40}}, color = {0,0,255}),Line(points = {{60,60},{40,60},{ -10,10}}, color = {0,0,255}),Line(points = {{ -10, -10},{40, -60},{60, -60}}, color = {0,0,255})}), Documentation(info = "<html>
<p>Since the simple bipolar transistor model does not have base or collector resistances both are added in this component. Additionally, a capacity is added to the base pin. See the schematic for more detailes.  In such a way the transistor model can be enhanced to become more common. </p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -76,82},{ -2,54}}, lineColor = {0,0,255}, textString = "Transistor")}));
            Basic.Resistor rtb(R = 0.05) annotation(Placement(transformation(extent = {{ -80, -10},{ -60,10}}, rotation = 0)));
            Basic.Resistor rtc(R = 0.1) annotation(Placement(transformation(extent = {{40,0},{60,20}}, rotation = 0)));
            Basic.Capacitor ct(C = 1e-10) annotation(Placement(transformation(origin = { -40, -30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Semiconductors.NPN Tr(Bf = 50, Br = 0.1, Is = 1e-16, Vak = 0.02, Tauf = 1.2e-10, Taur = 5e-09, Ccs = 1e-12, Cje = 4e-13, Cjc = 5e-13, Phie = 0.8, Me = 0.4, Phic = 0.8, Mc = 0.333, Gbc = 1e-15, Gbe = 1e-15, Vt = 0.02585) annotation(Placement(transformation(extent = {{ -20, -20},{20,20}}, rotation = 0)));
            Basic.Ground Ground1 annotation(Placement(transformation(extent = {{ -50, -80},{ -30, -60}}, rotation = 0)));
            Interfaces.Pin c annotation(Placement(transformation(extent = {{90,50},{110,70}}, rotation = 0)));
            Interfaces.Pin b annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
            Interfaces.Pin e annotation(Placement(transformation(extent = {{90, -70},{110, -50}}, rotation = 0)));

          equation 
            connect(rtb.n,Tr.B) annotation(Line(points = {{ -60,0},{ -20,0}}));
            connect(rtb.n,ct.p) annotation(Line(points = {{ -60,0},{ -40,0},{ -40, -20}}));
            connect(ct.n,Ground1.p) annotation(Line(points = {{ -40, -40},{ -40, -60}}));
            connect(Tr.C,rtc.p) annotation(Line(points = {{20,10},{40,10}}));
            connect(rtc.n,c) annotation(Line(points = {{60,10},{80,10},{80,60},{100,60}}));
            connect(b,rtb.p) annotation(Line(points = {{ -100,0},{ -80,0}}));
            connect(Tr.E,e) annotation(Line(points = {{20, -10},{80, -10},{80, -60},{100, -60}}));
          end Transistor;
          model RealSwitch "Ideal switch with resistance"
            Ideal.ControlledIdealCommutingSwitch S(level = 2.5) annotation(Placement(transformation(extent = {{11.3333, -46},{58,0.6667}}, rotation = 0)));
            Basic.Resistor R(R = 0.01) annotation(Placement(transformation(extent = {{ -66, -48},{ -19.3333, -1.3333}}, rotation = 0)));
            Interfaces.Pin p annotation(Placement(transformation(extent = {{ -110, -34},{ -90, -14}}, rotation = 0)));
            Interfaces.Pin n1 annotation(Placement(transformation(extent = {{90, -14},{110,6}}, rotation = 0)));
            Interfaces.Pin n2 annotation(Placement(transformation(extent = {{90, -60},{110, -40}}, rotation = 0)));
            Interfaces.Pin control annotation(Placement(transformation(extent = {{24,26},{44,46}}, rotation = 0)));

          equation 
            connect(p,R.p) annotation(Line(points = {{ -100, -24},{ -66, -24.6667}}));
            connect(R.n,S.p) annotation(Line(points = {{ -19.3333, -24.6667},{11.3333, -22.6667}}));
            connect(n1,S.n1) annotation(Line(points = {{100, -4},{58, -11}}));
            connect(n2,S.n2) annotation(Line(points = {{100, -50},{58, -22.6667}}));
            connect(control,S.control) annotation(Line(points = {{34,36},{34.6666,0.6667}}));
            annotation(Documentation(info = "<html>
<p>This component is a special kind of a commuting switch which possesses an additional constant resistance. This resistance was necessary in an example. It is not designed for common use. </p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -90,86},{28,70}}, lineColor = {0,0,255}, textString = "Real Switch")}));
          end RealSwitch;
          model NonlinearResistor "Chua's resistor"
            extends Interfaces.OnePort;
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -70,30},{70, -30}}, lineColor = {0,0,255}),Line(points = {{ -90,0},{ -70,0}}, color = {0,0,255}),Line(points = {{70,0},{90,0}}, color = {0,0,255}),Line(points = {{ -50, -60},{50,60}}, color = {0,0,255}),Polygon(points = {{50,60},{38,52},{44,46},{50,60}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -70,30},{70, -30}}, lineColor = {0,0,255}),Line(points = {{ -90,0},{ -70,0}}, color = {0,0,255}),Line(points = {{70,0},{90,0}}, color = {0,0,255}),Line(points = {{ -50, -60},{50,60}}, color = {0,0,255}),Polygon(points = {{50,60},{38,52},{44,46},{50,60}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -100,100},{100,70}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<html>
<p>This is the only nonlinear component for Chua&apos;s circuit. It is a piecewise linear resistor with both an inner and an outer range, which includes the inner one. The slopes of both ranges are given by parameters. The resistance caracteristic is continuous. For Chua&apos;s circuit both slopes have to be chosen to be negative. </p>
</html>"));
            parameter SI.Conductance Ga "Conductance in inner voltage range";
            parameter SI.Conductance Gb "Conductance in outer voltage range";
            parameter SI.Voltage Ve "Inner voltage range limit";

          equation 
            i = if v <  -Ve then Gb * (v + Ve) - Ga * Ve else if v > Ve then Gb * (v - Ve) + Ga * Ve else Ga * v;
          end NonlinearResistor;
          model Nand "CMOS NAND Gate (see Tietze/Schenk, page 157)"
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -50,70},{50, -70}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Ellipse(extent = {{50,5},{60, -5}}, lineColor = {0,0,0}, pattern = LinePattern.None),Text(extent = {{ -16,50},{14,20}}, lineColor = {0,0,0}, textString = "&"),Line(points = {{60,0},{100,0}}, color = {0,0,255}),Line(points = {{0,70},{0,100}}, color = {0,0,255}),Line(points = {{ -50,50},{ -100,50}}, color = {0,0,255}),Line(points = {{ -50, -50},{ -100, -50}}, color = {0,0,255}),Ellipse(extent = {{50,5},{60, -5}}, lineColor = {0,0,0}),Text(extent = {{ -100,34},{ -90,24}}, textString = "x1", lineColor = {0,0,255}),Text(extent = {{ -100, -65},{ -90, -75}}, textString = "x2", lineColor = {0,0,255}),Text(extent = {{90, -15},{100, -25}}, textString = "y", lineColor = {0,0,255})}), Documentation(info = "<html>
<p>The nand gate is a basic CMOS building block. It consists of four CMOS transistors.  </p>
<p><b>Reference:</b> </p>
<p>Tietze, U.; Schenk, Ch.: Halbleiter-Schaltungstechnik. Springer-Verlag Berlin Heidelberg NewYork 1980, p. 157 </p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden<br>
<p>
<dt>
<b>Copyright:</b>
<dd>
Copyright &copy; 1998-2006, Modelica Association and Fraunhofer-Gesellschaft.<br>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package
Modelica in file \"Modelica/package.mo\".</i><br>
<p>
</dl>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -94,93},{ -44,70}}, lineColor = {0,0,255}, textString = "NAND")}));
            Semiconductors.PMOS TP1(W = 6.5e-06, L = 3.1e-06, Beta = 1.05e-05, Vt =  -1, K2 = 0.41, K5 = 0.8385, dW =  -2.5e-06, dL =  -2.1e-06) annotation(Placement(transformation(extent = {{10,60},{30,80}}, rotation = 0)));
            Semiconductors.PMOS TP2(W = 6.5e-06, L = 3.1e-06, Beta = 1.05e-05, Vt =  -1, K2 = 0.41, K5 = 0.8385, dW =  -2.5e-06, dL =  -2.1e-06) annotation(Placement(transformation(extent = {{ -40,45},{ -20,65}}, rotation = 0)));
            Semiconductors.NMOS TN1(W = 6.5e-06, L = 3.1e-06, Beta = 4.1e-05, Vt = 0.8, K2 = 1.144, K5 = 0.7311, dW =  -2.5e-06, dL =  -1.5e-06) annotation(Placement(transformation(extent = {{10,15},{30,35}}, rotation = 0)));
            Semiconductors.NMOS TN2(W = 6.5e-06, L = 3.1e-06, Beta = 4.1e-05, Vt = 0.8, K2 = 1.144, K5 = 0.7311, dW =  -2.5e-06, dL =  -1.5e-06) annotation(Placement(transformation(extent = {{10, -25},{30, -5}}, rotation = 0)));
            Basic.Capacitor C4(C = 4e-14) annotation(Placement(transformation(origin = {60,30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Basic.Capacitor C7(C = 4e-14) annotation(Placement(transformation(origin = {55, -50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Basic.Ground Gnd annotation(Placement(transformation(extent = {{40, -25},{50, -15}}, rotation = 0)));
            Basic.Ground Gnd2 annotation(Placement(transformation(extent = {{55,0},{65,10}}, rotation = 0)));
            Basic.Ground Gnd3 annotation(Placement(transformation(extent = {{50, -80},{60, -70}}, rotation = 0)));
            Basic.Ground Gnd6 annotation(Placement(transformation(extent = {{ -15,30},{ -5,40}}, rotation = 0)));
            Basic.Ground Gnd7 annotation(Placement(transformation(extent = {{40,15},{50,25}}, rotation = 0)));
            Basic.Ground Gnd8 annotation(Placement(transformation(extent = {{40,60},{50,70}}, rotation = 0)));
            Interfaces.Pin x1 annotation(Placement(transformation(extent = {{ -110,40},{ -90,60}}, rotation = 0)));
            Interfaces.Pin x2 annotation(Placement(transformation(extent = {{ -110, -60},{ -90, -40}}, rotation = 0)));
            Interfaces.Pin Vdd annotation(Placement(transformation(extent = {{ -10,90},{10,110}}, rotation = 0)));
            Interfaces.Pin y annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));

          equation 
            connect(TN1.S,TN2.D) annotation(Line(points = {{30,20},{30, -10}}));
            connect(TN2.B,Gnd.p) annotation(Line(points = {{30, -15},{38, -15},{38, -15},{45, -15}}));
            connect(TN2.D,C7.p) annotation(Line(points = {{30, -10},{55, -10},{55, -40}}));
            connect(TN2.S,Gnd.p) annotation(Line(points = {{30, -20},{38, -20},{38, -15},{45, -15}}));
            connect(TP1.S,TN1.D) annotation(Line(points = {{30,65},{30,30}}));
            connect(TP1.G,TN1.G) annotation(Line(points = {{10,65},{10,20}}));
            connect(C4.n,Gnd2.p) annotation(Line(points = {{60,20},{60,10}}));
            connect(C7.n,Gnd3.p) annotation(Line(points = {{55, -60},{55, -70}}));
            connect(TP2.B,Gnd6.p) annotation(Line(points = {{ -20,55},{ -10,55},{ -10,40}}));
            connect(TN1.B,Gnd7.p) annotation(Line(points = {{30,25},{45,25}}));
            connect(TP1.B,Gnd8.p) annotation(Line(points = {{30,70},{45,70}}));
            connect(TP2.G,x1) annotation(Line(points = {{ -40,50},{ -100,50}}));
            connect(TP2.G,TN2.G) annotation(Line(points = {{ -40,50},{ -40, -20},{10, -20}}));
            connect(TN1.G,x2) annotation(Line(points = {{10,20},{ -70,20},{ -70, -50},{ -100, -50}}));
            connect(TP2.S,TN1.D) annotation(Line(points = {{ -20,50},{30,50},{30,30}}));
            connect(TN1.D,C4.p) annotation(Line(points = {{30,30},{30,50},{60,50},{60,40}}));
            connect(Vdd,TP1.D) annotation(Line(points = {{0,100},{0,80},{30,80},{30,75}}));
            connect(TP2.D,Vdd) annotation(Line(points = {{ -20,60},{ -20,80},{0,80},{0,100}}));
            connect(C4.p,y) annotation(Line(points = {{60,40},{60,50},{80,50},{80,0},{100,0}}));
          end Nand;
        end Utilities;
        model ThyristorBehaviourTest "Thyristor demonstration example"
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 0.0002), Documentation(info = "<html>
<p>This is a simple testcircuit, to test the behavior of the thysistor model.  </p><p>Interesting values to plot are Cathode.v, Gate.v and sineVoltage.p.v. and in another plot window pulseCurrent.p.i</p><p>The simulation time should be trom 0 seconds to 2e-4 seconds. </p>
</html>"), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{100,72},{100, -78},{80, -98},{80,52},{100,72}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{ -100,52},{ -80,72},{100,72},{80,52},{ -100,52}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{ -100, -98},{80,52}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -96,5},{77, -47}}, lineColor = {0,0,255}, textString = "Example"),Text(extent = {{ -120,134},{120,75}}, lineColor = {255,0,0}, textString = "%name")}));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(Placement(transformation(extent = {{18, -40},{38, -20}})));
          Modelica.Electrical.Analog.Sources.SineVoltage sineVoltage(V = 30, freqHz = 10000) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 270, origin = { -72,4})));
          Modelica.Electrical.Analog.Basic.Resistor resistor(R = 10) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 270, origin = {8,20})));
          Modelica.Electrical.Analog.Sources.PulseCurrent pulseCurrent(width = 10, period = 0.0001, startTime = 2e-05, I = 0.005) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 180, origin = { -8,70})));
          Modelica.Electrical.Analog.Semiconductors.Thyristor thyristor_v4_1 annotation(Placement(transformation(extent = {{ -44,30},{ -24,50}})));
          Modelica.Electrical.Analog.Basic.Inductor inductor(L = 2e-06) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 270, origin = {8, -4})));

        equation 
          connect(sineVoltage.n,ground.p) annotation(Line(points = {{ -72, -6},{ -72, -20},{28, -20}}, color = {0,0,255}, smooth = Smooth.None));
          connect(sineVoltage.p,thyristor_v4_1.Anode) annotation(Line(points = {{ -72,14},{ -72,40},{ -43,40}}, color = {0,0,255}, smooth = Smooth.None));
          connect(resistor.p,thyristor_v4_1.Cathode) annotation(Line(points = {{8,30},{8,40},{ -25,40}}, color = {0,0,255}, smooth = Smooth.None));
          connect(pulseCurrent.n,thyristor_v4_1.Gate) annotation(Line(points = {{ -18,70},{ -26,70},{ -26,49},{ -27,49}}, color = {0,0,255}, smooth = Smooth.None));
          connect(resistor.p,pulseCurrent.p) annotation(Line(points = {{8,30},{8,70},{2,70}}, color = {0,0,255}, smooth = Smooth.None));
          connect(resistor.n,inductor.p) annotation(Line(points = {{8,10},{8,8},{8,6},{8,6}}, color = {0,0,255}, smooth = Smooth.None));
          connect(inductor.n,ground.p) annotation(Line(points = {{8, -14},{8, -20},{28, -20}}, color = {0,0,255}, smooth = Smooth.None));
        end ThyristorBehaviourTest;
        model SwitchWithArc "Comparison of switch models both with and without arc"
          annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -100,80},{100,60}}, lineColor = {0,0,255}, textString = "Compare voltage and current of inductor1 and inductor2")}), experiment(StopTime = 2, Interval = 0.00025, Tolerance = 1e-06), Documentation(info = "<html>
<p align=\"justify\"><br/><br/>This example is to compare the behaviour of switch models with and without an electric arc taking into consideration.</p>
<p>Simulate until T=2 s. </p>
<p><br/><br/>Plot in one windows: </p>
<p><br/><br/>switch1.i and switch2.i </p>
<p><br/><br/>The difference in the closing area shows that the simple arc model avoids the suddenly switching. </p>
</html>", revisions = "<html>
<ul>
<li><i>May, 2009   </i>
       by Anton Haumer<br> initially realized<br>
       </li>
</ul>
</html>"));
          extends Modelica.Icons.Example;
          Modelica.Blocks.Sources.BooleanPulse booleanPulse annotation(Placement(transformation(extent = {{ -80, -30},{ -60, -10}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground ground1 annotation(Placement(transformation(extent = {{20, -20},{40,0}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.ConstantVoltage constantVoltage1(V = 50) annotation(Placement(transformation(origin = { -20,20}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Inductor inductor1(L = 0.1) annotation(Placement(transformation(extent = {{40,30},{60,50}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor resistor1 annotation(Placement(transformation(origin = {80,20}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Ideal.IdealClosingSwitch switch1 annotation(Placement(transformation(extent = {{0,30},{20,50}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground ground2 annotation(Placement(transformation(extent = {{20, -100},{40, -80}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.ConstantVoltage constantVoltage2(V = 50) annotation(Placement(transformation(origin = { -20, -60}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Inductor inductor2(L = 0.1) annotation(Placement(transformation(extent = {{40, -50},{60, -30}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor resistor2 annotation(Placement(transformation(origin = {80, -60}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Ideal.CloserWithArc switch2 annotation(Placement(transformation(extent = {{0, -50},{20, -30}}, rotation = 0)));

        equation 
          connect(inductor1.n,resistor1.p) annotation(Line(points = {{60,40},{80,40},{80,30}}, color = {0,0,255}));
          connect(resistor1.n,ground1.p) annotation(Line(points = {{80,10},{80,0},{30,0}}, color = {0,0,255}));
          connect(constantVoltage1.n,ground1.p) annotation(Line(points = {{ -20,10},{ -20,0},{30,0}}, color = {0,0,255}));
          connect(switch1.n,inductor1.p) annotation(Line(points = {{20,40},{40,40}}, color = {0,0,255}));
          connect(constantVoltage1.p,switch1.p) annotation(Line(points = {{ -20,30},{ -20,40},{0,40}}, color = {0,0,255}));
          connect(inductor2.n,resistor2.p) annotation(Line(points = {{60, -40},{80, -40},{80, -50}}, color = {0,0,255}));
          connect(resistor2.n,ground2.p) annotation(Line(points = {{80, -70},{80, -80},{30, -80}}, color = {0,0,255}));
          connect(constantVoltage2.n,ground2.p) annotation(Line(points = {{ -20, -70},{ -20, -80},{30, -80}}, color = {0,0,255}));
          connect(switch2.n,inductor2.p) annotation(Line(points = {{20, -40},{40, -40}}, color = {0,0,255}));
          connect(constantVoltage2.p,switch2.p) annotation(Line(points = {{ -20, -50},{ -20, -40},{0, -40}}, color = {0,0,255}));
          connect(booleanPulse.y,switch1.control) annotation(Line(points = {{ -59, -20},{ -40, -20},{ -40,60},{10,60},{10,47}}, color = {255,0,255}));
          connect(booleanPulse.y,switch2.control) annotation(Line(points = {{ -59, -20},{10, -20},{10, -30}}, color = {255,0,255}));
        end SwitchWithArc;
        model ShowVariableResistor "Simple demo of a VariableResistor model"
          extends Modelica.Icons.Example;
          Modelica.Electrical.Analog.Basic.VariableResistor VariableResistor annotation(Placement(transformation(extent = {{ -20, -20},{0,0}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground Ground1 annotation(Placement(transformation(extent = {{ -100, -80},{ -80, -60}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground Ground2 annotation(Placement(transformation(extent = {{60, -40},{80, -20}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R1 annotation(Placement(transformation(extent = {{ -60,40},{ -40,60}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R2 annotation(Placement(transformation(extent = {{ -20,40},{0,60}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R3 annotation(Placement(transformation(extent = {{20,40},{40,60}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R4 annotation(Placement(transformation(extent = {{ -60, -20},{ -40,0}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R5 annotation(Placement(transformation(extent = {{20, -20},{40,0}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>It is a simple test circuit for the VariableResistor. The VariableResistor sould be compared with R2. </p>
<p>Simulate until T=1 s.  </p>
</html>", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Mai 6, 2004   </i>
       by Teresa Schlegel<br> realized<br>
       </li>
</ul>
</html>"), experiment(StopTime = 1), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -100,112},{80,40}}, lineColor = {0,0,255}, textString = "Example VariableResistor")}));
          annotation(Diagram);
          Modelica.Electrical.Analog.Sources.SineVoltage SineVoltage1 annotation(Placement(transformation(origin = { -90, -30}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Blocks.Sources.Ramp Ramp1(height = 5, offset = 2) annotation(Placement(transformation(origin = { -10,20}, extent = {{ -10, -10},{10,10}}, rotation = 270)));

        equation 
          connect(R1.n,R2.p) annotation(Line(points = {{ -40,50},{ -20,50}}, color = {0,0,255}));
          connect(R2.n,R3.p) annotation(Line(points = {{0,50},{20,50}}, color = {0,0,255}));
          connect(R4.n,VariableResistor.p) annotation(Line(points = {{ -40, -10},{ -20, -10}}, color = {0,0,255}));
          connect(VariableResistor.n,R5.p) annotation(Line(points = {{0, -10},{20, -10}}, color = {0,0,255}));
          connect(R3.n,Ground2.p) annotation(Line(points = {{40,50},{70,50},{70, -20}}, color = {0,0,255}));
          connect(Ground2.p,R5.n) annotation(Line(points = {{70, -20},{70, -10},{40, -10}}, color = {0,0,255}));
          connect(SineVoltage1.p,Ground1.p) annotation(Line(points = {{ -90, -40},{ -90, -60}}, color = {0,0,255}));
          connect(SineVoltage1.n,R1.p) annotation(Line(points = {{ -90, -20},{ -90,50},{ -60,50}}, color = {0,0,255}));
          connect(SineVoltage1.n,R4.p) annotation(Line(points = {{ -90, -20},{ -90, -10},{ -60, -10}}, color = {0,0,255}));
          connect(Ramp1.y,VariableResistor.R) annotation(Line(points = {{ -10,9},{ -10,4.5},{ -10,1},{ -10,1}}, color = {0,0,255}));
        end ShowVariableResistor;
        model ShowSaturatingInductor "Simple demo to show behaviour of SaturatingInductor component"
          extends Modelica.Icons.Example;
          parameter Modelica.SIunits.Inductance Lzer = 2 "Inductance near current=0";
          parameter Modelica.SIunits.Inductance Lnom = 1 "Nominal inductance at Nominal current";
          parameter Modelica.SIunits.Current Inom = 1 "Nominal current";
          parameter Modelica.SIunits.Inductance Linf = 0.5 "Inductance at large currents";
          parameter Modelica.SIunits.Voltage U = 1.25 "Source voltage (peak)";
          parameter Modelica.SIunits.Frequency f = 1 / (2 * Modelica.Constants.pi) "Source frequency";
          parameter Modelica.SIunits.Angle phase = Modelica.Constants.pi / 2 "Source voltage phase shift";
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -80,84},{70,38}}, lineColor = {0,0,255}, textString = "Show Saturating Inductor")}), experiment(StopTime = 6.2832, Interval = 0.01), Documentation(info = "<html>
<p>This simple circuit uses the saturating inductor, whis has a changing inductivity.</p>
<p>This circuit should be simulated until 1 s. Compare SaturatingInductance1.p.i with Inductance1.p.i to see the difference between saturating and ideal inductor. </p>
</html>"));
          Modelica.Electrical.Analog.Sources.SineVoltage SineVoltage1(V = U, phase = phase, freqHz = f) annotation(Placement(transformation(origin = { -60, -6}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground Ground1 annotation(Placement(transformation(extent = {{ -70, -36},{ -50, -16}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.SaturatingInductor SaturatingInductance1(Lzer = Lzer, Lnom = Lnom, Inom = Inom, Linf = Linf) annotation(Placement(transformation(origin = { -20,0}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Basic.Inductor Inductance1(L = Lnom) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 270, origin = {20,0})));

        equation 
          connect(SineVoltage1.n,Ground1.p) annotation(Line(points = {{ -60, -16},{ -60, -16}}, color = {0,0,255}));
          connect(SineVoltage1.n,SaturatingInductance1.n) annotation(Line(points = {{ -60, -16},{ -20, -16},{ -20, -10}}, color = {0,0,255}));
          connect(SaturatingInductance1.p,SineVoltage1.p) annotation(Line(points = {{ -20,10},{ -20,20},{ -60,20},{ -60,4}}, color = {0,0,255}));
          connect(Inductance1.p,SineVoltage1.p) annotation(Line(points = {{20,10},{20,20},{ -60,20},{ -60,4}}, color = {0,0,255}, smooth = Smooth.None));
          connect(Inductance1.n,SineVoltage1.n) annotation(Line(points = {{20, -10},{20, -16},{ -60, -16}}, color = {0,0,255}, smooth = Smooth.None));
        end ShowSaturatingInductor;
        model Rectifier "B6 diode bridge"
          extends Modelica.Icons.Example;
          import Modelica.Electrical.Analog.Ideal;
          parameter Modelica.SIunits.Voltage VAC = 400 "RMS line-to-line";
          parameter Modelica.SIunits.Frequency f = 50 "line frequency";
          parameter Modelica.SIunits.Inductance LAC = 6e-05 "line inductor";
          parameter Modelica.SIunits.Resistance Ron = 0.001 "diode forward resistance";
          parameter Modelica.SIunits.Conductance Goff = 0.001 "diode backward conductance";
          parameter Modelica.SIunits.Voltage Vknee = 2 "diode threshold voltage";
          parameter Modelica.SIunits.Capacitance CDC = 0.015 "DC capacitance";
          parameter Modelica.SIunits.Current IDC = 500 "load current";
          output Modelica.SIunits.Voltage uDC;
          output Modelica.SIunits.Current iAC[3];
          output Modelica.SIunits.Voltage uAC[3];
          output Modelica.SIunits.Power Losses;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -80,90},{80,70}}, lineColor = {0,0,0}, textString = "Rectifier"),Line(points = {{ -16,18},{ -16,2},{ -18,6},{ -14,6},{ -16,2}}, color = {0,0,0}),Line(points = {{ -30,22},{ -26,20},{ -30,18},{ -30,22}}, color = {0,0,0}),Line(points = {{32,30},{32, -30},{30, -26},{34, -26},{32, -30}}, color = {0,0,0}),Text(extent = {{ -38,16},{ -22,8}}, lineColor = {0,0,0}, textString = "iAC"),Text(extent = {{ -14,8},{2,0}}, lineColor = {0,0,0}, textString = "uAC"),Text(extent = {{22, -16},{38, -24}}, lineColor = {0,0,0}, textString = "uDC")}), experiment(StopTime = 0.1, Interval = 1e-05), Documentation(info = "<html>
<p>The rectifier example shows a B6 diode bridge fed by a three phase sinusoidal voltage, loaded by a DC current.</p><p>DC capacitors start at ideal no-load voltage, thus making easier initial transient. </p>
<p><br/><br/>Simulate until T=0.1 s. </p><p><br/><br/>Plot in separate windows:</p><p><br/><br/>uDC ... DC-voltage</p><p><br/><br/>iAC ... AC-currents 1..3</p><p><br/><br/>uAC ... AC-voltages 1..3 (distorted)</p><p><br/><br/>Try different load currents iDC = 0..approximately 500 A. </p>
<p>You may watch Losses (of the whole diode bridge) trying different diode parameters. </p>
</html>", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Mai 7, 2004   </i>
       by Anton Haumer<br> realized<br>
       </li>
</ul>
</html>"));
          Modelica.Electrical.Analog.Sources.SineVoltage SineVoltage1(freqHz = f, V = VAC * sqrt(2 / 3)) annotation(Placement(transformation(extent = {{ -70,10},{ -90,30}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SineVoltage SineVoltage2(freqHz = f, phase =  -2 / 3 * Modelica.Constants.pi, V = VAC * sqrt(2 / 3)) annotation(Placement(transformation(extent = {{ -70, -10},{ -90,10}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SineVoltage SineVoltage3(freqHz = f, phase =  -4 / 3 * Modelica.Constants.pi, V = VAC * sqrt(2 / 3)) annotation(Placement(transformation(extent = {{ -70, -30},{ -90, -10}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Inductor Inductor1(L = LAC) annotation(Placement(transformation(extent = {{ -60,10},{ -40,30}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Inductor Inductor2(L = LAC) annotation(Placement(transformation(extent = {{ -60, -10},{ -40,10}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Inductor Inductor3(L = LAC) annotation(Placement(transformation(extent = {{ -60, -30},{ -40, -10}}, rotation = 0)));
          Ideal.IdealDiode IdealDiode1(Ron = Ron, Goff = Goff, Vknee = Vknee) annotation(Placement(transformation(origin = { -20,40}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Ideal.IdealDiode IdealDiode2(Ron = Ron, Goff = Goff, Vknee = Vknee) annotation(Placement(transformation(origin = {0,40}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Ideal.IdealDiode IdealDiode3(Ron = Ron, Goff = Goff, Vknee = Vknee) annotation(Placement(transformation(origin = {20,40}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Ideal.IdealDiode IdealDiode4(Ron = Ron, Goff = Goff, Vknee = Vknee) annotation(Placement(transformation(origin = { -20, -40}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Ideal.IdealDiode IdealDiode5(Ron = Ron, Goff = Goff, Vknee = Vknee) annotation(Placement(transformation(origin = {0, -40}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Ideal.IdealDiode IdealDiode6(Ron = Ron, Goff = Goff, Vknee = Vknee) annotation(Placement(transformation(origin = {20, -40}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Electrical.Analog.Basic.Capacitor Capacitor1(C = 2 * CDC) annotation(Placement(transformation(origin = {40,40}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Capacitor Capacitor2(C = 2 * CDC) annotation(Placement(transformation(origin = {40, -40}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground Ground1 annotation(Placement(transformation(extent = {{40, -80},{60, -60}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SignalCurrent SignalCurrent1 annotation(Placement(transformation(origin = {60,0}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Blocks.Sources.Constant Constant1(k = IDC) annotation(Placement(transformation(extent = {{100, -10},{80,10}}, rotation = 0)));

        initial equation 
          Capacitor1.v = (VAC * sqrt(2)) / 2;
          Capacitor2.v = (VAC * sqrt(2)) / 2;

        equation 
          uDC = Capacitor1.v + Capacitor2.v;
          iAC = {Inductor1.i,Inductor2.i,Inductor3.i};
          uAC[1] = Inductor1.n.v - Inductor2.n.v;
          uAC[2] = Inductor2.n.v - Inductor3.n.v;
          uAC[3] = Inductor3.n.v - Inductor1.n.v;
          Losses = IdealDiode1.v * IdealDiode1.i + IdealDiode2.v * IdealDiode2.i + IdealDiode3.v * IdealDiode3.i + IdealDiode4.v * IdealDiode4.i + IdealDiode5.v * IdealDiode5.i + IdealDiode6.v * IdealDiode6.i;
          connect(SineVoltage1.n,SineVoltage2.n) annotation(Line(points = {{ -90,20},{ -90,0}}, color = {0,0,255}));
          connect(SineVoltage2.n,SineVoltage3.n) annotation(Line(points = {{ -90,0},{ -90, -20}}, color = {0,0,255}));
          connect(SineVoltage1.p,Inductor1.p) annotation(Line(points = {{ -70,20},{ -60,20}}, color = {0,0,255}));
          connect(SineVoltage2.p,Inductor2.p) annotation(Line(points = {{ -70,0},{ -60,0}}, color = {0,0,255}));
          connect(SineVoltage3.p,Inductor3.p) annotation(Line(points = {{ -70, -20},{ -60, -20}}, color = {0,0,255}));
          connect(IdealDiode1.p,IdealDiode4.n) annotation(Line(points = {{ -20,30},{ -20, -30}}, color = {0,0,255}));
          connect(IdealDiode2.p,IdealDiode5.n) annotation(Line(points = {{ -6.12323e-16,30},{ -6.12323e-16,16},{0,0},{0, -30},{6.12323e-16, -30}}, color = {0,0,255}));
          connect(IdealDiode3.p,IdealDiode6.n) annotation(Line(points = {{20,30},{20, -30}}, color = {0,0,255}));
          connect(IdealDiode1.n,IdealDiode2.n) annotation(Line(points = {{ -20,50},{6.12323e-16,50}}, color = {0,0,255}));
          connect(IdealDiode2.n,IdealDiode3.n) annotation(Line(points = {{6.12323e-16,50},{20,50}}, color = {0,0,255}));
          connect(IdealDiode4.p,IdealDiode5.p) annotation(Line(points = {{ -20, -50},{ -6.12323e-16, -50}}, color = {0,0,255}));
          connect(IdealDiode5.p,IdealDiode6.p) annotation(Line(points = {{ -6.12323e-16, -50},{20, -50}}, color = {0,0,255}));
          connect(Capacitor2.n,IdealDiode6.p) annotation(Line(points = {{40, -50},{20, -50}}, color = {0,0,255}));
          connect(IdealDiode3.n,Capacitor1.p) annotation(Line(points = {{20,50},{40,50}}, color = {0,0,255}));
          connect(Capacitor1.n,Capacitor2.p) annotation(Line(points = {{40,30},{40, -30}}, color = {0,0,255}));
          connect(Capacitor2.p,Ground1.p) annotation(Line(points = {{40, -30},{40,0},{50,0},{50, -60}}, color = {0,0,255}));
          connect(Capacitor1.p,SignalCurrent1.p) annotation(Line(points = {{40,50},{60,50},{60,10}}, color = {0,0,255}));
          connect(SignalCurrent1.n,Capacitor2.n) annotation(Line(points = {{60, -10},{60, -50},{40, -50}}, color = {0,0,255}));
          connect(Constant1.y,SignalCurrent1.i) annotation(Line(points = {{79,0},{79, -1.28588e-15},{67, -1.28588e-15}}, color = {0,0,255}));
          connect(Inductor1.n,IdealDiode1.p) annotation(Line(points = {{ -40,20},{ -20,20},{ -20,30}}, color = {0,0,255}));
          connect(Inductor2.n,IdealDiode2.p) annotation(Line(points = {{ -40,0},{ -6.12323e-16,0},{ -6.12323e-16,30}}, color = {0,0,255}));
          connect(Inductor3.n,IdealDiode3.p) annotation(Line(points = {{ -40, -20},{20, -20},{20,30}}, color = {0,0,255}));
        end Rectifier;
        model OvervoltageProtection "Example for Zener diodes"
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 0.4), Documentation(info = "<html>
<p>This example is a simple circuit for overvoltage protection. If the voltage zDiode_1.n.v is too high, the Diode zDiode_2 breaks through and the voltage gets down.</p><p>The simulation end time should be set to 0.4. To get the typical behaviour please plot sineVoltage.p.v, RL.p.v, zDiode_2.n.v and zDiode_1.n.i.  </p>
</html>", revisions = "<html>
<ul>
<li><i>February 02, 2009</i>
       by Kristin Majetta<br>
       documentation added</li>
<li><i> January 08, 2009   </i>
        by Matthias Franke <br>initially implemented
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -100},{80,50}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{ -100,50},{ -80,70},{100,70},{80,50},{ -100,50}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{100,70},{100, -80},{80, -100},{80,50},{100,70}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -96,3},{77, -49}}, lineColor = {0,0,255}, textString = "Example"),Text(extent = {{ -120,132},{120,73}}, lineColor = {255,0,0}, textString = "%name")}));
          Modelica.Electrical.Analog.Sources.SineVoltage sineVoltage(offset = 0, V = 10, freqHz = 5) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 270, origin = { -66,6})));
          Modelica.Electrical.Analog.Basic.Resistor Rv(R = 20) annotation(Placement(transformation(extent = {{ -56,32},{ -36,52}})));
          Modelica.Electrical.Analog.Basic.Resistor RL(R = 2000) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 270, origin = {26,4})));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(Placement(transformation(extent = {{ -30, -82},{ -10, -62}})));
          Modelica.Electrical.Analog.Basic.Capacitor CL(C = 1e-07) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 270, origin = {62,4})));
          Semiconductors.ZDiode zDiode annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 90, origin = { -20, -20})));
          Semiconductors.ZDiode zDiode1 annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 270, origin = { -20,22})));

        equation 
          connect(sineVoltage.p,Rv.p) annotation(Line(points = {{ -66,16},{ -66,42},{ -56,42}}, color = {0,0,255}, smooth = Smooth.None));
          connect(RL.p,CL.p) annotation(Line(points = {{26,14},{62,14}}, color = {0,0,255}, smooth = Smooth.None));
          connect(RL.n,CL.n) annotation(Line(points = {{26, -6},{62, -6}}, color = {0,0,255}, smooth = Smooth.None));
          connect(sineVoltage.n,zDiode.p) annotation(Line(points = {{ -66, -4},{ -66, -30},{ -20, -30}}, color = {0,0,255}, smooth = Smooth.None));
          connect(zDiode.p,ground.p) annotation(Line(points = {{ -20, -30},{ -20, -62}}, color = {0,0,255}, smooth = Smooth.None));
          connect(zDiode.p,RL.n) annotation(Line(points = {{ -20, -30},{26, -30},{26, -6}}, color = {0,0,255}, smooth = Smooth.None));
          connect(zDiode1.p,Rv.n) annotation(Line(points = {{ -20,32},{ -20,42},{ -36,42}}, color = {0,0,255}, smooth = Smooth.None));
          connect(RL.p,Rv.n) annotation(Line(points = {{26,14},{26,42},{ -36,42}}, color = {0,0,255}, smooth = Smooth.None));
          connect(zDiode1.n,zDiode.n) annotation(Line(points = {{ -20,12},{ -20,1},{ -20,1},{ -20, -10}}, color = {0,0,255}, smooth = Smooth.None));
        end OvervoltageProtection;
        encapsulated model NandGate "CMOS NAND Gate (see Tietze/Schenk, page 157)"
          import Modelica.Electrical.Analog.Basic;
          import Modelica.Electrical.Analog.Sources;
          import Modelica.Electrical.Analog.Examples.Utilities;
          import Modelica.Icons;
          extends Icons.Example;
          annotation(Icon(graphics), Documentation(info = "<html>
<p>The nand gate is a basic CMOS building block. It consists of four CMOS transistors. The output voltage Nand.y.v is low if and only if the two input voltages at Nand.x1.v and Nand.x2.v are both high. In this way the nand functionality is realized. </p>
<p>The simulation end time should be set to 1e-7. Please plot the input voltages Nand.x1.v, d Nand.x2.v, and the output voltage Nand.y.v.  </p>
<p><b>Reference:</b> </p>
<p>Tietze, U.; Schenk, Ch.: Halbleiter-Schaltungstechnik. Springer-Verlag Berlin Heidelberg NewYork 1980, p. 157 </p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden<br>
<p>
<dt>
<b>Copyright:</b>
<dd>
Copyright &copy; 1998-2006, Modelica Association and Fraunhofer-Gesellschaft.<br>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package
Modelica in file \"Modelica/package.mo\".</i><br>
<p>
</dl>
</html>"), experiment(StopTime = 1e-07), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -90,98},{ -12,66}}, lineColor = {0,0,255}, textString = "NAND Gate")}));
          Sources.TrapezoidVoltage VIN1(V = 3.5, startTime = 2e-08, rising = 1e-09, width = 1.9e-08, falling = 1e-09, period = 4e-08) annotation(Placement(transformation(origin = { -70,20}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Sources.TrapezoidVoltage VIN2(V = 3.5, startTime = 1e-08, rising = 1e-09, width = 1.9e-08, falling = 1e-09, period = 4e-08) annotation(Placement(transformation(origin = { -70, -30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Sources.RampVoltage VDD(V = 5, duration = 1e-09) annotation(Placement(transformation(origin = {50,70}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Basic.Ground Gnd1 annotation(Placement(transformation(extent = {{45,43},{55,53}}, rotation = 0)));
          Basic.Ground Gnd4 annotation(Placement(transformation(extent = {{ -75, -5},{ -65,5}}, rotation = 0)));
          Basic.Ground Gnd5 annotation(Placement(transformation(extent = {{ -75, -55},{ -65, -45}}, rotation = 0)));
          Utilities.Nand Nand annotation(Placement(transformation(extent = {{ -40, -45},{40,55}}, rotation = 0)));

        equation 
          connect(VDD.n,Gnd1.p) annotation(Line(points = {{50,60},{50,53}}));
          connect(VIN1.n,Gnd4.p) annotation(Line(points = {{ -70,10},{ -70,5}}));
          connect(VIN2.n,Gnd5.p) annotation(Line(points = {{ -70, -40},{ -70, -45}}));
          connect(Nand.Vdd,VDD.p) annotation(Line(points = {{0,55},{0,90},{50,90},{50,80}}));
          connect(VIN1.p,Nand.x1) annotation(Line(points = {{ -70,30},{ -40,30}}));
          connect(VIN2.p,Nand.x2) annotation(Line(points = {{ -70, -20},{ -40, -20}}));
        end NandGate;
        model HeatingResistor "Heating resistor"
          extends Modelica.Icons.Example;
          Basic.HeatingResistor heatingResistor(R_ref = 100, alpha = 0.001, T_ref = 293.15) annotation(Placement(transformation(extent = {{ -10,10},{10, -10}}, rotation =  -90, origin = { -32,34})));
          Modelica.Electrical.Analog.Basic.Ground G annotation(Placement(transformation(extent = {{ -80,0},{ -60,20}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SineVoltage SineVoltage1(V = 220, freqHz = 1) annotation(Placement(transformation(origin = { -70,40}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -94,102},{0,74}}, textString = "HeatingRectifier", lineColor = {0,0,255})}), Documentation(info = "<html>
<p>This is a very simple circuit consisting of a voltage source and a resistor. The loss power in the resistor is transported to the environment via its heatPort.  </p>
</html>"));
          Modelica.Thermal.HeatTransfer.Components.ThermalConductor thermalConductor(G = 50) annotation(Placement(transformation(origin = {4,34}, extent = {{ -10, -10},{10,10}}, rotation = 0)));
          Thermal.HeatTransfer.Celsius.FixedTemperature fixedTemperature(T = 20) annotation(Placement(transformation(extent = {{52,24},{32,44}})));

        equation 
          connect(SineVoltage1.n,G.p) annotation(Line(points = {{ -70,30},{ -70,20}}, color = {0,0,255}));
          connect(heatingResistor.heatPort,thermalConductor.port_a) annotation(Line(points = {{ -22,34},{ -6,34}}, color = {191,0,0}));
          annotation(Documentation(info = "<HTML>
<P>
The heating rectifier shows a heat flow always if the electrical capacitor is loaded.
</P>
<P>
Simulate until T=5 s.
</P>
<P>
Plot in separate windows:<br> <br>
SineVoltage1.v  and  Capacitor1.p.v <br>
HeatCapacitor1.port.T and HeatingDiode1.heatPort.T<br>
HeatingDiode1.heatPort.Q_flow<br>
</P>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Mai 6, 2004   </i>
       by Christoph Clauss<br> realized<br>
       </li>
</ul>
</HTML>
"), experiment(StopTime = 5));
          connect(SineVoltage1.p,heatingResistor.p) annotation(Line(points = {{ -70,50},{ -32,50},{ -32,44}}, color = {0,0,255}, smooth = Smooth.None));
          connect(G.p,heatingResistor.n) annotation(Line(points = {{ -70,20},{ -32,20},{ -32,24}}, color = {0,0,255}, smooth = Smooth.None));
          connect(thermalConductor.port_b,fixedTemperature.port) annotation(Line(points = {{14,34},{32,34}}, color = {191,0,0}, smooth = Smooth.None));
        end HeatingResistor;
        model HeatingRectifier "Heating rectifier"
          extends Modelica.Icons.Example;
          Modelica.Electrical.Analog.Semiconductors.HeatingDiode HeatingDiode1(useHeatPort = true) annotation(Placement(transformation(extent = {{ -20,40},{0,60}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground G annotation(Placement(transformation(extent = {{ -80,0},{ -60,20}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SineVoltage SineVoltage1(V = 1, freqHz = 1) annotation(Placement(transformation(origin = { -70,40}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -94,102},{0,74}}, textString = "HeatingRectifier", lineColor = {0,0,255})}), Documentation(info = "<html>
<p>The heating rectifier shows a heat flow always if the electrical capacitor is loaded. </p>
<p>Simulate until T=5 s.  </p>
<p><br/><br/>Plot in separate windows:</p><p><br/><br/>SineVoltage1.v and Capacitor1.p.v </p><p><br/><br/>HeatCapacitor1.port.T and HeatingDiode1.heatPort.T</p><p><br/><br/>HeatingDiode1.heatPort.Q_flow</p>
</html>", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Mai 6, 2004   </i>
       by Christoph Clauss<br> realized<br>
       </li>
</ul>
</html>"));
          Modelica.Electrical.Analog.Basic.Capacitor Capacitor1(C = 1) annotation(Placement(transformation(extent = {{20,40},{40,60}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor HeatCapacitor1(C = 1) annotation(Placement(transformation(origin = { -10, -50}, extent = {{ -10, -10},{10,10}}, rotation = 180)));
          Modelica.Thermal.HeatTransfer.Components.ThermalConductor ThermalConductor1(G = 10) annotation(Placement(transformation(origin = { -10, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Resistor R(R = 1) annotation(Placement(transformation(extent = {{20,70},{40,90}}, rotation = 0)));

        equation 
          connect(SineVoltage1.p,HeatingDiode1.p) annotation(Line(points = {{ -70,50},{ -20,50}}, color = {0,0,255}));
          connect(SineVoltage1.n,G.p) annotation(Line(points = {{ -70,30},{ -70,20}}, color = {0,0,255}));
          connect(Capacitor1.n,G.p) annotation(Line(points = {{40,50},{40,20},{ -70,20}}, color = {0,0,255}));
          connect(HeatingDiode1.n,Capacitor1.p) annotation(Line(points = {{0,50},{20,50}}, color = {0,0,255}));
          connect(HeatingDiode1.heatPort,ThermalConductor1.port_a) annotation(Line(points = {{ -10,40},{ -10,20},{ -10,0},{ -10,0}}, color = {191,0,0}));
          connect(ThermalConductor1.port_b,HeatCapacitor1.port) annotation(Line(points = {{ -10, -20},{ -10, -25.75},{ -10, -40},{ -10, -40}}, color = {191,0,0}));
          connect(R.p,Capacitor1.p) annotation(Line(points = {{20,80},{20,50}}, color = {0,0,255}));
          connect(R.n,Capacitor1.n) annotation(Line(points = {{40,80},{40,50}}, color = {0,0,255}));
          annotation(Documentation(info = "<HTML>
<P>
The heating rectifier shows a heat flow always if the electrical capacitor is loaded.
</P>
<P>
Simulate until T=5 s.
</P>
<P>
Plot in separate windows:<br> <br>
SineVoltage1.v  and  Capacitor1.p.v <br>
HeatCapacitor1.port.T and HeatingDiode1.heatPort.T<br>
HeatingDiode1.heatPort.Q_flow<br>
</P>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Mai 6, 2004   </i>
       by Christoph Clauss<br> realized<br>
       </li>
</ul>
</HTML>
"), experiment(StopTime = 5));
        end HeatingRectifier;
        model HeatingNPN_OrGate "Heating NPN Or Gate"
          extends Modelica.Icons.Example;
          constant Modelica.SIunits.Capacitance CapVal = 0;
          constant Modelica.SIunits.Time tauVal = 0;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -100,100},{ -6,72}}, textString = "Heating \"NPN or\" Gate", lineColor = {0,0,255})}), Documentation(info = "<html>
<p>The heating &QUOT;NPN or&QUOT; gate shows a heat flow always if a transistor is leading. </p>
<p>Simulate until T=200 s. </p>
<p><br/><br/>Plot in separate windows: </p><p><br/><br/>V1.v and V2.v and C2.v </p><p><br/><br/>HeatCapacitor1.port.T and T1.heatPort.T and T2.heatPort.T</p><p><br/><br/>T1.heatPort.Q_flow and T2.heatPort.Q_flow</p>
</html>", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Mai 6, 2004   </i>
       by Christoph Clauss<br> realized<br>
       </li>
</ul>
</html>"));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor HeatCapacitor1(C = 0.1) annotation(Placement(transformation(origin = {76, -84}, extent = {{ -10, -10},{10,10}}, rotation = 180)));
          Modelica.Thermal.HeatTransfer.Components.ThermalConductor TC1(G = 0.01) annotation(Placement(transformation(origin = {90, -50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Thermal.HeatTransfer.Components.ThermalConductor TC2(G = 0.01) annotation(Placement(transformation(origin = {60, -50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          annotation(Documentation(info = "<HTML>
<P>
The heating NPN nand gate shows a heat flow always if a transistor is leading.
</P>
<P>
Simulate until T=200 s.
</P>
<P>
Plot in separate windows:<br> <br>
V1.v and V2.v and C2.v <br>
HeatCapacitor1.port.T and T1.heatPort.T and T2.heatPort.T<br>
T1.heatPort.Q_flow and T2.heatPort.Q_flow<br>
</P>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Mai 6, 2004   </i>
       by Christoph Clauss<br> realized<br>
       </li>
</ul>
</HTML>
"), experiment(StopTime = 200));
          Modelica.Electrical.Analog.Sources.RampVoltage V(V = 6, duration = 5) annotation(Placement(transformation(origin = {90,38}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Sources.TrapezoidVoltage V1(V = 6, startTime = 55, rising = 5, width = 15, falling = 5, period = 50, nperiod = 10) annotation(Placement(transformation(origin = { -90,18}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Sources.TrapezoidVoltage V2(V = 6, startTime = 65, rising = 5, width = 15, falling = 5, period = 50, nperiod = 10) annotation(Placement(transformation(origin = { -50, -42}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Resistor R1(R = 1800) annotation(Placement(transformation(extent = {{ -90,48},{ -70,68}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R2(R = 1800) annotation(Placement(transformation(extent = {{ -36, -22},{ -16, -2}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor RI(R = 40) annotation(Placement(transformation(extent = {{60,58},{80,78}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground Gnd annotation(Placement(transformation(extent = {{80,0},{100,20}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground Gnd1 annotation(Placement(transformation(extent = {{ -100, -22},{ -80, -2}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground Gnd2 annotation(Placement(transformation(extent = {{ -60, -82},{ -40, -62}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground Gnd3 annotation(Placement(transformation(extent = {{ -10,16},{10,36}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground Gnd4 annotation(Placement(transformation(extent = {{30, -52},{50, -32}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C1(C = CapVal) annotation(Placement(transformation(origin = { -70,38}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Capacitor C2(C = CapVal) annotation(Placement(transformation(origin = {60,42}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Capacitor C3(C = CapVal) annotation(Placement(transformation(origin = { -16, -40}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground Gnd5 annotation(Placement(transformation(extent = {{ -26, -76},{ -6, -56}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground Gnd6 annotation(Placement(transformation(extent = {{50,6},{70,26}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground Gnd7 annotation(Placement(transformation(extent = {{ -80,2},{ -60,22}}, rotation = 0)));
          Semiconductors.HeatingNPN T1(Bf = 100, Br = 1, Is = 1e-14, Vak = 0, Tauf = tauVal, Taur = tauVal, Ccs = CapVal, Cje = CapVal, Cjc = CapVal, Phie = 1, Me = 0.5, Phic = 1, Mc = 0.5, Gbc = 1e-12, Gbe = 1e-12, EMax = 40, vt_t(start = 0.01, fixed = false), useHeatPort = true) annotation(Placement(transformation(extent = {{ -20,48},{0,68}}, rotation = 0)));
          Semiconductors.HeatingNPN T2(Bf = 100, Br = 1, Is = 1e-14, Vak = 0, Tauf = tauVal, Taur = tauVal, Ccs = CapVal, Cje = CapVal, Cjc = CapVal, Phie = 1, Me = 0.5, Phic = 1, Mc = 0.5, Gbc = 1e-12, Gbe = 1e-12, EMax = 40, vt_t(start = 0.01, fixed = false), useHeatPort = true) annotation(Placement(transformation(extent = {{20, -22},{40, -2}}, rotation = 0)));

        equation 
          connect(Gnd1.p,V1.n) annotation(Line(points = {{ -90, -2},{ -90,8}}));
          connect(V1.p,R1.p) annotation(Line(points = {{ -90,28},{ -90,58}}));
          connect(RI.n,V.p) annotation(Line(points = {{80,68},{90,68},{90,48}}));
          connect(Gnd.p,V.n) annotation(Line(points = {{90,20},{90,28}}));
          connect(V2.p,R2.p) annotation(Line(points = {{ -50, -32},{ -50, -12},{ -36, -12}}));
          connect(Gnd2.p,V2.n) annotation(Line(points = {{ -50, -62},{ -50, -52}}));
          connect(Gnd7.p,C1.n) annotation(Line(points = {{ -70,22},{ -70,28}}));
          connect(C2.p,RI.p) annotation(Line(points = {{60,52},{60,68}}));
          connect(Gnd6.p,C2.n) annotation(Line(points = {{60,26},{60,32}}));
          connect(C3.p,R2.n) annotation(Line(points = {{ -16, -30},{ -16, -21},{ -16, -21},{ -16, -12}}));
          connect(C1.p,R1.n) annotation(Line(points = {{ -70,48},{ -70,58}}));
          connect(Gnd5.p,C3.n) annotation(Line(points = {{ -16, -56},{ -16, -53},{ -16, -50},{ -16, -50}}));
          connect(T1.B,R1.n) annotation(Line(points = {{ -20,58},{ -70,58}}, color = {0,0,255}));
          connect(T1.E,Gnd3.p) annotation(Line(points = {{0,53},{0,36}}, color = {0,0,255}));
          connect(RI.p,T1.C) annotation(Line(points = {{60,68},{30,68},{30,63},{0,63}}, color = {0,0,255}));
          connect(T2.B,R2.n) annotation(Line(points = {{20, -12},{ -16, -12}}, color = {0,0,255}));
          connect(T2.E,Gnd4.p) annotation(Line(points = {{40, -17},{40, -32}}, color = {0,0,255}));
          connect(T2.C,RI.p) annotation(Line(points = {{40, -7},{40,68},{60,68}}, color = {0,0,255}));
          connect(TC1.port_b,HeatCapacitor1.port) annotation(Line(points = {{90, -60},{84, -60},{84, -74},{76, -74}}, color = {191,0,0}));
          connect(TC2.port_b,HeatCapacitor1.port) annotation(Line(points = {{60, -60},{68, -60},{68, -74},{76, -74}}, color = {191,0,0}));
          connect(TC2.port_a,T2.heatPort) annotation(Line(points = {{60, -40},{60, -22},{30, -22}}, color = {191,0,0}));
          connect(TC1.port_a,T1.heatPort) annotation(Line(points = {{90, -40},{90,2},{ -10,2},{ -10,48}}, color = {191,0,0}));
        end HeatingNPN_OrGate;
        model HeatingMOSInverter "Heating MOS Inverter"
          extends Modelica.Icons.Example;
          Modelica.Electrical.Analog.Basic.Ground G annotation(Placement(transformation(extent = {{ -80, -40},{ -60, -20}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SineVoltage Sin(V = 5) annotation(Placement(transformation(origin = { -70,0}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -100,100},{ -6,72}}, textString = "Heating MOS Inverter", lineColor = {0,0,255})}), Documentation(info = "<html>
<p>The heating MOS inverter shows a heat flow always if a transistor is leading. </p>
<p>Simulate until T=5 s. </p>
<p><br/><br/>Plot in separate windows: </p><p><br/><br/>Sin.p.v and Capacitor1.p.v </p><p><br/><br/>HeatCapacitor1.port.T and H_PMOS.heatPort.T and H_NMOS.heatPort.T</p><p><br/><br/>H_PMOS.heatPort.Q_flow and H_NMOS.heatPort.Q_flow</p>
</html>", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Mai 6, 2004   </i>
       by Christoph Clauss<br> realized<br>
       </li>
</ul>
</html>"));
          Modelica.Electrical.Analog.Basic.Capacitor Capacitor1(C = 1e-05) annotation(Placement(transformation(origin = {30,10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Thermal.HeatTransfer.Components.HeatCapacitor HeatCapacitor1(C = 0.01) annotation(Placement(transformation(origin = {70, -60}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Thermal.HeatTransfer.Components.ThermalConductor TC1(G = 0.01) annotation(Placement(transformation(extent = {{0, -50},{20, -30}}, rotation = 0)));
          Semiconductors.HeatingPMOS H_PMOS(useHeatPort = true) annotation(Placement(transformation(extent = {{ -40,40},{ -20,60}}, rotation = 0)));
          Semiconductors.HeatingNMOS H_NMOS(useHeatPort = true) annotation(Placement(transformation(extent = {{ -40,0},{ -20,20}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.RampVoltage V(V = 5, duration = 0.01) annotation(Placement(transformation(origin = {50,50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Thermal.HeatTransfer.Components.ThermalConductor TC2(G = 0.01) annotation(Placement(transformation(extent = {{0, -90},{20, -70}}, rotation = 0)));
          Modelica.Thermal.HeatTransfer.Sources.FixedTemperature FixedTemperature1(T = 300) annotation(Placement(transformation(origin = {90, -30}, extent = {{ -10, -10},{10,10}}, rotation = 180)));
          Modelica.Thermal.HeatTransfer.Components.ThermalConductor TC3(G = 0.01) annotation(Placement(transformation(extent = {{50, -40},{70, -20}}, rotation = 0)));

        equation 
          connect(Sin.n,G.p) annotation(Line(points = {{ -70, -10},{ -70, -20}}, color = {0,0,255}));
          connect(Capacitor1.n,G.p) annotation(Line(points = {{30,0},{30, -20},{ -70, -20}}, color = {0,0,255}));
          annotation(Documentation(info = "<HTML>
<P>
The heating MOS inverter shows a heat flow always if a transistor is leading.
</P>
<P>
Simulate until T=5 s.
</P>
<P>
Plot in separate windows:<br> <br>
Sin.p.v  and  Capacitor1.p.v <br>
HeatCapacitor1.port.T and H_PMOS.heatPort.T and H_NMOS.heatPort.T<br>
H_PMOS.heatPort.Q_flow and H_NMOS.heatPort.Q_flow<br>
</P>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Mai 6, 2004   </i>
       by Christoph Clauss<br> realized<br>
       </li>
</ul>
</HTML>
"), experiment(StopTime = 5));
          connect(H_NMOS.G,H_PMOS.G) annotation(Line(points = {{ -40,5},{ -40,45}}, color = {0,0,255}));
          connect(H_NMOS.G,Sin.p) annotation(Line(points = {{ -40,5},{ -40,10},{ -70,10}}, color = {0,0,255}));
          connect(H_PMOS.S,H_NMOS.D) annotation(Line(points = {{ -20,45},{ -20,15}}, color = {0,0,255}));
          connect(H_NMOS.D,Capacitor1.p) annotation(Line(points = {{ -20,15},{ -20,20},{30,20}}, color = {0,0,255}));
          connect(H_NMOS.B,H_NMOS.S) annotation(Line(points = {{ -20,10},{ -20,5}}, color = {0,0,255}));
          connect(H_NMOS.S,G.p) annotation(Line(points = {{ -20,5},{ -20, -20},{ -70, -20}}, color = {0,0,255}));
          connect(H_PMOS.B,H_PMOS.D) annotation(Line(points = {{ -20,50},{ -20,55}}, color = {0,0,255}));
          connect(V.p,H_PMOS.D) annotation(Line(points = {{50,60},{ -20,60},{ -20,55}}, color = {0,0,255}));
          connect(V.n,G.p) annotation(Line(points = {{50,40},{50, -20},{ -70, -20}}, color = {0,0,255}));
          connect(TC1.port_b,HeatCapacitor1.port) annotation(Line(points = {{20, -40},{40, -40},{40, -60},{60, -60}}, color = {191,0,0}));
          connect(TC2.port_b,HeatCapacitor1.port) annotation(Line(points = {{20, -80},{40, -80},{40, -60},{60, -60}}, color = {191,0,0}));
          connect(TC1.port_a,H_PMOS.heatPort) annotation(Line(points = {{0, -40},{ -10, -40},{ -10,40},{ -30,40}}, color = {191,0,0}));
          connect(TC2.port_a,H_NMOS.heatPort) annotation(Line(points = {{0, -80},{ -30, -80},{ -30,0}}, color = {191,0,0}));
          connect(TC3.port_b,FixedTemperature1.port) annotation(Line(points = {{70, -30},{80, -30}}, color = {191,0,0}));
          connect(TC3.port_a,HeatCapacitor1.port) annotation(Line(points = {{50, -30},{40, -30},{40, -60},{60, -60}}, color = {191,0,0}));
        end HeatingMOSInverter;
        encapsulated model DifferenceAmplifier "Simple NPN transistor amplifier circuit"
          import Modelica.Electrical.Analog.Basic;
          import Modelica.Electrical.Analog.Sources;
          import Modelica.Electrical.Analog.Examples.Utilities;
          import Modelica.Icons;
          extends Icons.Example;
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Documentation(info = "<html>
<p>It is a simple NPN transistor amplifier circuit. The voltage difference between R1.p and R3.n is amplified. The output signal is the voltage between R2.n and R4.n. In this example the voltage at V1 is amplified because R3.n is grounded.  </p>
<p>The simulation end time should be set to 1e- 8. Please plot the input voltage V1.v, and the output voltages R2.n.v, and R4.n.v. </p>
<p><b>Reference:</b> </p>
<p>Tietze, U.; Schenk, Ch.: Halbleiter-Schaltungstechnik. Springer-Verlag Berlin Heidelberg NewYork 1980, p. 59 </p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden<br>
<p>
<dt>
<b>Copyright:</b>
<dd>
Copyright &copy; 1998-2006, Modelica Association and Fraunhofer-Gesellschaft.<br>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package
Modelica in file \"Modelica/package.mo\".</i><br>
<p>
</dl>
</html>"), experiment(StopTime = 1), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -98,106},{22,60}}, lineColor = {0,0,255}, textString = "Difference Amplifier")}));
          Sources.ExpSineVoltage V1(V = 0.2, freqHz = 200000000.0, damping = 10000000.0) annotation(Placement(transformation(origin = { -90, -50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Sources.RampVoltage V2(V = 15, duration = 1e-09) annotation(Placement(transformation(origin = {80,50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Sources.RampCurrent I1(I = 0.16, duration = 1e-09) annotation(Placement(transformation(origin = { -10, -50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Basic.Resistor R1(R = 0.0001) annotation(Placement(transformation(extent = {{ -90, -20},{ -70,0}}, rotation = 0)));
          Basic.Resistor R2(R = 100) annotation(Placement(transformation(origin = { -40,30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Basic.Resistor R3(R = 0.0001) annotation(Placement(transformation(extent = {{70, -20},{90,0}}, rotation = 0)));
          Basic.Resistor R4(R = 100) annotation(Placement(transformation(origin = {40,30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Basic.Capacitor C1(C = 1e-10) annotation(Placement(transformation(origin = { -60, -50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Basic.Capacitor C4(C = 1e-10) annotation(Placement(transformation(extent = {{10, -14},{30,6}}, rotation = 0)));
          Basic.Capacitor C5(C = 1e-10) annotation(Placement(transformation(origin = {10, -50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Basic.Capacitor C2(C = 1e-10) annotation(Placement(transformation(extent = {{ -30, -14},{ -10,6}}, rotation = 0)));
          Basic.Capacitor C3(C = 1e-10) annotation(Placement(transformation(origin = {60, -50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Basic.Ground Gnd1 annotation(Placement(transformation(extent = {{ -95, -80},{ -85, -70}}, rotation = 0)));
          Basic.Ground Gnd9 annotation(Placement(transformation(extent = {{75,20},{85,30}}, rotation = 0)));
          Basic.Ground Gnd3 annotation(Placement(transformation(extent = {{ -15, -20},{ -5, -10}}, rotation = 0)));
          Basic.Ground Gnd2 annotation(Placement(transformation(extent = {{ -65, -80},{ -55, -70}}, rotation = 0)));
          Basic.Ground Gnd6 annotation(Placement(transformation(extent = {{5, -20},{15, -10}}, rotation = 0)));
          Basic.Ground Gnd7 annotation(Placement(transformation(extent = {{ -15, -80},{ -5, -70}}, rotation = 0)));
          Basic.Ground Gnd8 annotation(Placement(transformation(extent = {{5, -80},{15, -70}}, rotation = 0)));
          Basic.Ground Gnd5 annotation(Placement(transformation(extent = {{55, -80},{65, -70}}, rotation = 0)));
          Basic.Ground Gnd4 annotation(Placement(transformation(extent = {{85, -50},{95, -40}}, rotation = 0)));
          Utilities.Transistor Transistor1 annotation(Placement(transformation(extent = {{ -60, -20},{ -40,0}}, rotation = 0)));
          Utilities.Transistor Transistor2 annotation(Placement(transformation(extent = {{60, -20},{40,0}}, rotation = 0)));

        equation 
          connect(V1.n,Gnd1.p) annotation(Line(points = {{ -90, -60},{ -90, -70}}));
          connect(C1.n,Gnd2.p) annotation(Line(points = {{ -60, -60},{ -60, -70}}));
          connect(I1.n,Gnd7.p) annotation(Line(points = {{ -10, -60},{ -10, -65},{ -10, -65},{ -10, -70}}));
          connect(C5.n,Gnd8.p) annotation(Line(points = {{10, -60},{10, -70}}));
          connect(C3.n,Gnd5.p) annotation(Line(points = {{60, -60},{60, -70}}));
          connect(R3.n,Gnd4.p) annotation(Line(points = {{90, -10},{90, -40}}));
          connect(C2.n,Gnd3.p) annotation(Line(points = {{ -10, -4},{ -10, -10}}));
          connect(C4.p,Gnd6.p) annotation(Line(points = {{10, -4},{10, -10}}));
          connect(I1.p,C5.p) annotation(Line(points = {{ -10, -40},{10, -40}}));
          connect(R1.p,V1.p) annotation(Line(points = {{ -90, -10},{ -90, -40}}));
          connect(R2.p,V2.p) annotation(Line(points = {{ -40,40},{ -40,70},{80,70},{80,60}}));
          connect(R4.p,V2.p) annotation(Line(points = {{40,40},{40,70},{80,70},{80,60}}));
          connect(V2.n,Gnd9.p) annotation(Line(points = {{80,40},{80,30}}));
          connect(R1.n,Transistor1.b) annotation(Line(points = {{ -70, -10},{ -60, -10}}));
          connect(Transistor1.b,C1.p) annotation(Line(points = {{ -60, -10},{ -60, -40}}));
          connect(Transistor1.c,C2.p) annotation(Line(points = {{ -40, -4},{ -30, -4}}));
          connect(R2.n,Transistor1.c) annotation(Line(points = {{ -40,20},{ -40, -4}}));
          connect(Transistor1.e,I1.p) annotation(Line(points = {{ -40, -16},{ -30, -16},{ -30, -40},{ -10, -40}}));
          connect(Transistor2.b,R3.p) annotation(Line(points = {{60, -10},{70, -10}}));
          connect(Transistor2.b,C3.p) annotation(Line(points = {{60, -10},{60, -40}}));
          connect(C4.n,Transistor2.c) annotation(Line(points = {{30, -4},{40, -4}}));
          connect(R4.n,Transistor2.c) annotation(Line(points = {{40,20},{40, -4}}));
          connect(C5.p,Transistor2.e) annotation(Line(points = {{10, -40},{30, -40},{30, -16},{40, -16}}));
        end DifferenceAmplifier;
        model ControlledSwitchWithArc "Comparison of controlled switch models both with and without arc"
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -100,80},{100,60}}, lineColor = {0,0,255}, textString = "Compare voltage and current of inductor1 and inductor2")}), experiment(StopTime = 6, Interval = 0.00025, Tolerance = 1e-06), Documentation(info = "<html>
<p><br/><br/>This example is to compare the behaviour of switch models with and without an electric arc taking into consideration.</p>
<p>Simulate until T=2 s.  </p>
<p><br/><br/>Plot in one windows:</p><p><br/><br/>switch1.i and switch2.i </p><p><br/><br/>The difference in the closing area shows that the simple arc model avoids the suddenly switching. </p>
</html>", revisions = "<html>
<ul>
<li><i>May, 2009   </i>
       by Anton Haumer<br> initially realized<br>
       </li>
</ul>
</html>"));
          extends Modelica.Icons.Example;
          Modelica.Electrical.Analog.Basic.Ground ground1 annotation(Placement(transformation(extent = {{20, -20},{40,0}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.ConstantVoltage constantVoltage1(V = 50) annotation(Placement(transformation(origin = { -20,20}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Inductor inductor1(L = 0.1) annotation(Placement(transformation(extent = {{40,30},{60,50}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor resistor1 annotation(Placement(transformation(origin = {80,20}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Ideal.ControlledIdealClosingSwitch switch1 annotation(Placement(transformation(extent = {{0,30},{20,50}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground ground2 annotation(Placement(transformation(extent = {{20, -100},{40, -80}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.ConstantVoltage constantVoltage2(V = 50) annotation(Placement(transformation(origin = { -20, -60}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Inductor inductor2(L = 0.1) annotation(Placement(transformation(extent = {{40, -50},{60, -30}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor resistor2 annotation(Placement(transformation(origin = {80, -60}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Ideal.ControlledCloserWithArc switch2 annotation(Placement(transformation(extent = {{0, -50},{20, -30}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SineVoltage sineVoltage(V = 1, freqHz = 1) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 270, origin = { -70, -10})));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(Placement(transformation(extent = {{ -80, -60},{ -60, -40}})));

        equation 
          connect(inductor1.n,resistor1.p) annotation(Line(points = {{60,40},{80,40},{80,30}}, color = {0,0,255}));
          connect(resistor1.n,ground1.p) annotation(Line(points = {{80,10},{80,0},{30,0}}, color = {0,0,255}));
          connect(constantVoltage1.n,ground1.p) annotation(Line(points = {{ -20,10},{ -20,0},{30,0}}, color = {0,0,255}));
          connect(switch1.n,inductor1.p) annotation(Line(points = {{20,40},{40,40}}, color = {0,0,255}));
          connect(constantVoltage1.p,switch1.p) annotation(Line(points = {{ -20,30},{ -20,40},{0,40}}, color = {0,0,255}));
          connect(inductor2.n,resistor2.p) annotation(Line(points = {{60, -40},{80, -40},{80, -50}}, color = {0,0,255}));
          connect(resistor2.n,ground2.p) annotation(Line(points = {{80, -70},{80, -80},{30, -80}}, color = {0,0,255}));
          connect(constantVoltage2.n,ground2.p) annotation(Line(points = {{ -20, -70},{ -20, -80},{30, -80}}, color = {0,0,255}));
          connect(switch2.n,inductor2.p) annotation(Line(points = {{20, -40},{40, -40}}, color = {0,0,255}));
          connect(constantVoltage2.p,switch2.p) annotation(Line(points = {{ -20, -50},{ -20, -40},{0, -40}}, color = {0,0,255}));
          connect(sineVoltage.p,switch1.control) annotation(Line(points = {{ -70,0},{ -40,0},{ -40,50},{10,50}}, color = {0,0,255}, smooth = Smooth.None));
          connect(sineVoltage.p,switch2.control) annotation(Line(points = {{ -70,0},{ -40,0},{ -40, -30},{10, -30}}, color = {0,0,255}, smooth = Smooth.None));
          connect(sineVoltage.n,ground.p) annotation(Line(points = {{ -70, -20},{ -70, -40}}, color = {0,0,255}, smooth = Smooth.None));
        end ControlledSwitchWithArc;
        model CompareTransformers "Transformer circuit to show the magnetization facilities"
          annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -60, -80},{60, -100}}, lineColor = {0,0,255}, textString = "try considerMagnetization=false/true"),Text(extent = {{ -60,20},{60,0}}, lineColor = {0,0,255}, textString = "Basic.Transformer (mutual inductance)")}), experiment(StopTime = 50, Interval = 0.001), Documentation(revisions = "<html>
<ul>
<li><i>  </i>
       </li>
<li><i> 2009   </i>
       by Anton Haumer<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This example is o demonstrate the behaviour of transformer models. The Basic.Transformer, which consists of mutual coupled inductors,</p><p>is compared with the ideal transformer model. If the ideal model is used with considerMagnetization=true eakage inductances are taken </p><p>into account, otherwise not. </p>
<p>The example is constructed in such a way that the ideal transformer circuit with considerMagnetization=true shows the same behaviour as the basic transformer. </p>
<p><br/><br/>Simulate until T=50 s with both considerMagnetization=false and  </p><p><br/><br/>considerMagnetization=true of the ideal transformer.</p><p><br/><br/>Plot in separate windows for comparison:</p><p><br/><br/>basicTransformer.p1.v and idealTransformer.p1.v </p><p><br/><br/>basicTransformer.p1.i and idealTransformer.p1.i </p><p><br/><br/>basicTransformer.p2.v and idealTransformer.p2.v </p><p><br/><br/>basicTransformer.p2.i and idealTransformer.p2.i </p>
</html>"));
          extends Modelica.Icons.Example;
          constant Modelica.SIunits.Angle pi = Modelica.Constants.pi;
          parameter Modelica.SIunits.Voltage Vdc = 0.1 "DC offset of voltage source";
          parameter Modelica.SIunits.Voltage Vpeak = 0.1 "Peak voltage of voltage source";
          parameter Modelica.SIunits.Frequency f = 10 "Frequency of voltage source";
          parameter Modelica.SIunits.Angle phi0 = pi / 2 "Phase of voltage source";
          parameter Real n = 2 "Turns ratio primary:secondary voltage";
          parameter Modelica.SIunits.Resistance R1 = 0.01 "Primary resistance w.r.t. primary side";
          parameter Modelica.SIunits.Inductance L1sigma = 0.05 / (2 * pi * f) "Primary leakage inductance w.r.t. primary side";
          parameter Modelica.SIunits.Inductance Lm1 = 10.0 / (2 * pi * f) "Magnetizing inductance w.r.t. primary side";
          parameter Modelica.SIunits.Inductance L2sigma = 0.05 / (2 * pi * f) / n ^ 2 "Secondary leakage inductance w.r.t. secondary side";
          parameter Modelica.SIunits.Resistance R2 = 0.01 / n ^ 2 "Secondary resistance w.r.t. secondary side";
          parameter Modelica.SIunits.Resistance RL = 1 / n ^ 2 "Load resistance";
          final parameter Modelica.SIunits.Inductance L1 = L1sigma + M * n "Primary no-load inductance";
          final parameter Modelica.SIunits.Inductance L2 = L2sigma + M / n "Secondary no-load inductance";
          final parameter Modelica.SIunits.Inductance M = Lm1 / n "Mutual inductance";
          output Modelica.SIunits.Voltage v1B = resistor11.n.v "Primary voltage, basic transformer";
          output Modelica.SIunits.Current i1B = resistor11.i "Primary current, basic transformer";
          output Modelica.SIunits.Voltage v2B = resistor12.p.v "Secondary voltage, basic transformer";
          output Modelica.SIunits.Current i2B = resistor12.i "Secondary current, basic transformer";
          output Modelica.SIunits.Voltage v1I = resistor21.n.v "Primary voltage, basic transformer";
          output Modelica.SIunits.Current i1I = resistor21.i "Primary current, basic transformer";
          output Modelica.SIunits.Voltage v2I = resistor22.p.v "Secondary voltage, basic transformer";
          output Modelica.SIunits.Current i2I = resistor22.i "Secondary current, basic transformer";
          Modelica.Electrical.Analog.Sources.SineVoltage sineVoltage1(V = Vpeak, phase = phi0, freqHz = f, offset = Vdc) annotation(Placement(transformation(origin = { -80,40}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground ground11 annotation(Placement(transformation(extent = {{ -90,0},{ -70,20}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor resistor11(R = R1) annotation(Placement(transformation(extent = {{ -80,50},{ -60,70}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor resistor12(R = R2) annotation(Placement(transformation(extent = {{60,50},{80,70}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor load1(R = RL) annotation(Placement(transformation(origin = {80,40}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground ground12 annotation(Placement(transformation(extent = {{70,0},{90,20}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SineVoltage sineVoltage2(V = Vpeak, phase = phi0, freqHz = f, offset = Vdc) annotation(Placement(transformation(origin = { -80, -50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground ground21 annotation(Placement(transformation(extent = {{ -90, -90},{ -70, -70}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor resistor21(R = R1) annotation(Placement(transformation(extent = {{ -80, -40},{ -60, -20}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Inductor inductor21(L = L1sigma) annotation(Placement(transformation(extent = {{ -50, -40},{ -30, -20}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Inductor inductor22(L = L2sigma) annotation(Placement(transformation(extent = {{30, -40},{50, -20}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor resistor22(R = R2) annotation(Placement(transformation(extent = {{60, -40},{80, -20}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor load2(R = RL) annotation(Placement(transformation(origin = {80, -50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground ground22 annotation(Placement(transformation(extent = {{70, -90},{90, -70}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Transformer basicTransformer(L1 = L1, L2 = L2, M = M) annotation(Placement(transformation(extent = {{ -10,40},{10,60}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealTransformer idealTransformer(n = n, Lm1 = Lm1, considerMagnetization = false) annotation(Placement(transformation(extent = {{ -10, -50},{10, -30}}, rotation = 0)));

        equation 
          connect(sineVoltage1.n,ground11.p) annotation(Line(points = {{ -80,30},{ -80,20}}, color = {0,0,255}));
          connect(sineVoltage1.p,resistor11.p) annotation(Line(points = {{ -80,50},{ -80,60}}, color = {0,0,255}));
          connect(load1.n,ground12.p) annotation(Line(points = {{80,30},{80,20}}, color = {0,0,255}));
          connect(resistor12.n,load1.p) annotation(Line(points = {{80,60},{80,50}}, color = {0,0,255}));
          connect(sineVoltage2.n,ground21.p) annotation(Line(points = {{ -80, -60},{ -80, -70}}, color = {0,0,255}));
          connect(sineVoltage2.p,resistor21.p) annotation(Line(points = {{ -80, -40},{ -80, -30}}, color = {0,0,255}));
          connect(resistor21.n,inductor21.p) annotation(Line(points = {{ -60, -30},{ -50, -30}}, color = {0,0,255}));
          connect(inductor22.n,resistor22.p) annotation(Line(points = {{50, -30},{60, -30}}, color = {0,0,255}));
          connect(load2.n,ground22.p) annotation(Line(points = {{80, -60},{80, -70}}, color = {0,0,255}));
          connect(resistor22.n,load2.p) annotation(Line(points = {{80, -30},{80, -40}}, color = {0,0,255}));
          connect(ground11.p,basicTransformer.n1) annotation(Line(points = {{ -80,20},{ -10,20},{ -10,45}}, color = {0,0,255}));
          connect(basicTransformer.n2,ground12.p) annotation(Line(points = {{10,45},{10,20},{80,20}}, color = {0,0,255}));
          connect(basicTransformer.p1,resistor11.n) annotation(Line(points = {{ -10,55},{ -10,60},{ -60,60}}, color = {0,0,255}));
          connect(basicTransformer.p2,resistor12.p) annotation(Line(points = {{10,55},{10,60},{60,60}}, color = {0,0,255}));
          connect(ground21.p,idealTransformer.n1) annotation(Line(points = {{ -80, -70},{ -10, -70},{ -10, -45}}, color = {0,0,255}));
          connect(ground22.p,idealTransformer.n2) annotation(Line(points = {{80, -70},{10, -70},{10, -45}}, color = {0,0,255}));
          connect(idealTransformer.p1,inductor21.n) annotation(Line(points = {{ -10, -35},{ -10, -30},{ -30, -30}}, color = {0,0,255}));
          connect(idealTransformer.p2,inductor22.p) annotation(Line(points = {{10, -35},{10, -30},{30, -30}}, color = {0,0,255}));
        end CompareTransformers;
        encapsulated model ChuaCircuit "Chua's circuit, ns, V, A"
          import Modelica.Electrical.Analog.Basic;
          import Modelica.Electrical.Analog.Examples.Utilities;
          import Modelica.Icons;
          extends Icons.Example;
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Documentation(info = "<html>
<p>Chua&apos;s circuit is the most simple nonlinear circuit which shows chaotic behaviour. The circuit consists of linear basic elements (capacitors, resistor, conductor, inductor), and one nonlinear element, which is called Chua&apos;s diode. The chaotic behaviour is simulated. </p>
<p>The simulation end time should be set to 5e4. To get the chaotic behaviour please plot C1.v. Choose C2.v as the independent variable . </p>
<p><b>Reference:</b> </p>
<p>Kennedy, M.P.: Three Steps to Chaos - Part I: Evolution. IEEE Transactions on CAS I 40 (1993)10, 640-656  </p>
</html>", revisions = "<html>
<dt>
<b>Main Authors:</b>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden<br>
<p>
<dt>
<b>Copyright:</b>
<dd>
Copyright &copy; 1998-2006, Modelica Association and Fraunhofer-Gesellschaft.<br>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package
Modelica in file \"Modelica/package.mo\".</i><br>
<p>
</dl>
</html>"), experiment(StopTime = 1), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -98,104},{ -32,72}}, lineColor = {0,0,255}, textString = "Chua Circuit")}));
          Basic.Inductor L(L = 18) annotation(Placement(transformation(origin = { -75,38}, extent = {{ -25, -25},{25,25}}, rotation = 270)));
          Basic.Resistor Ro(R = 0.0125) annotation(Placement(transformation(origin = { -75, -17}, extent = {{ -25, -25},{25,25}}, rotation = 270)));
          Basic.Conductor G(G = 0.565) annotation(Placement(transformation(extent = {{ -25,38},{25,88}}, rotation = 0)));
          Basic.Capacitor C1(C = 10, v(start = 4)) annotation(Placement(transformation(origin = {25,3}, extent = {{ -25, -25},{25,25}}, rotation = 270)));
          Basic.Capacitor C2(C = 100) annotation(Placement(transformation(origin = { -25,3}, extent = {{ -25, -25},{25,25}}, rotation = 270)));
          Utilities.NonlinearResistor Nr(Ga(min =  -1) =  -0.757576, Gb(min =  -1) =  -0.409091, Ve = 1) annotation(Placement(transformation(origin = {75,3}, extent = {{ -25, -25},{25,25}}, rotation = 270)));
          Basic.Ground Gnd annotation(Placement(transformation(extent = {{ -25, -112},{25, -62}}, rotation = 0)));

        equation 
          connect(L.p,G.p) annotation(Line(points = {{ -75,63},{ -25,63}}));
          connect(G.n,Nr.p) annotation(Line(points = {{25,63},{75,63},{75,28}}));
          connect(Nr.n,Gnd.p) annotation(Line(points = {{75, -22},{75, -62},{0, -62}}));
          connect(C1.p,G.n) annotation(Line(points = {{25,28},{25,45.5},{25,63}}));
          connect(L.n,Ro.p) annotation(Line(points = {{ -75,13},{ -75,8}}));
          connect(G.p,C2.p) annotation(Line(points = {{ -25,63},{ -26,46},{ -26,28},{ -25,28}}));
          connect(C1.n,Gnd.p) annotation(Line(points = {{25, -22},{25, -62},{0, -62}}));
          connect(C2.n,Gnd.p) annotation(Line(points = {{ -25, -22},{ -25, -62},{0, -62}}));
          connect(Ro.n,Gnd.p) annotation(Line(points = {{ -75, -42},{ -75, -62},{0, -62}}));
        end ChuaCircuit;
        model CharacteristicThyristors "Characteristic of ideal thyristors"
          extends Modelica.Icons.Example;
          Modelica.Electrical.Analog.Ideal.IdealThyristor IdealThyristor1(Vknee = 5) annotation(Placement(transformation(extent = {{ -20,0},{0,20}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SineVoltage SineVoltage1(V = 10, offset = 0) annotation(Placement(transformation(origin = { -40, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground Ground1 annotation(Placement(transformation(extent = {{ -50, -60},{ -30, -40}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R3(R = 0.001) annotation(Placement(transformation(extent = {{40,0},{60,20}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -96,100},{98,60}}, textString = "Characteristic Thyristors", lineColor = {0,0,255})}), Documentation(info = "<html>
<p><br/><br/><br/>Two examples of thyristors are shown: </p><p><br/><br/><br/>the <b>ideal thyristor</b> </p><p><br/><br/><br/>and the <b>ideal GTO thyristor</b> with <i>Vknee=5</i> </p>
<p>Simulate until T=2 s. </p>
<p><br/><br/>Plot in separate windows:</p><p><br/><br/>IdealThyristor1.i and IdealGTOThyristor1.i </p><p><br/><br/>IdealThyristor1.v and IdealGTOThyristor1.v </p>
</html>", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Mai 7, 2004   </i>
       by Christoph Clauss<br> realized<br>
       </li>
</ul>
</html>"));
          Modelica.Blocks.Sources.BooleanStep BooleanStep1(startValue = false, startTime = 1.25) annotation(Placement(transformation(extent = {{ -60,40},{ -40,60}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealGTOThyristor IdealGTOThyristor1(Vknee = 0) annotation(Placement(transformation(extent = {{ -20, -30},{0, -10}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R1(R = 0.001) annotation(Placement(transformation(extent = {{40, -30},{60, -10}}, rotation = 0)));
          annotation(Documentation(info = "<HTML>
<P>
Two examples of thyristors are shown:<br><br>
the <b>ideal thyristor</b>  <br>
and the <b>ideal GTO thyristor</b>  with <i>Vknee=5</i>  <br><br>
</P>
<P>
Simulate until T=2 s.
</P>
<P>
Plot in separate windows:<br> <br>
IdealThyristor1.i and IdealGTOThyristor1.i <br>
IdealThyristor1.v and IdealGTOThyristor1.v <br>
</P>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Mai 7, 2004   </i>
       by Christoph Clauss<br> realized<br>
       </li>
</ul>
</HTML>
"), experiment(StopTime = 2));

        equation 
          connect(IdealThyristor1.n,R3.p) annotation(Line(points = {{0,10},{40,10}}, color = {0,0,255}));
          connect(Ground1.p,SineVoltage1.n) annotation(Line(points = {{ -40, -40},{ -40, -20}}, color = {0,0,255}));
          connect(SineVoltage1.p,IdealThyristor1.p) annotation(Line(points = {{ -40,0},{ -40,10},{ -20,10}}, color = {0,0,255}));
          connect(BooleanStep1.y,IdealThyristor1.fire) annotation(Line(points = {{ -39,50},{ -3,50},{ -3,21}}, color = {255,0,255}));
          connect(IdealGTOThyristor1.n,R1.p) annotation(Line(points = {{0, -20},{40, -20}}, color = {0,0,255}));
          connect(R3.n,R1.n) annotation(Line(points = {{60,10},{60, -20}}, color = {0,0,255}));
          connect(R1.n,Ground1.p) annotation(Line(points = {{60, -20},{60, -40},{ -40, -40}}, color = {0,0,255}));
          connect(IdealGTOThyristor1.p,IdealThyristor1.p) annotation(Line(points = {{ -20, -20},{ -20,10}}, color = {0,0,255}));
          connect(IdealGTOThyristor1.fire,IdealThyristor1.fire) annotation(Line(points = {{ -3, -9},{ -3, -2.5},{ -2, -2.5},{ -2,5},{ -3,5},{ -3,21}}, color = {255,0,255}));
        end CharacteristicThyristors;
        model CharacteristicIdealDiodes "Characteristic of ideal diodes"
          extends Modelica.Icons.Example;
          Modelica.Electrical.Analog.Ideal.IdealDiode Ideal(Ron = 0, Goff = 0) annotation(Placement(transformation(extent = {{0,40},{20,60}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealDiode With_Ron_Goff(Ron = 0.1, Goff = 0.1) annotation(Placement(transformation(extent = {{0,0},{20,20}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealDiode With_Ron_Goff_Vknee(Ron = 0.2, Goff = 0.2, Vknee = 5) annotation(Placement(transformation(extent = {{0, -40},{20, -20}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SineVoltage SineVoltage1(V = 10, offset =  -9) annotation(Placement(transformation(origin = { -40,0}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground Ground1 annotation(Placement(transformation(extent = {{ -50, -80},{ -30, -60}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -88,102},{92,48}}, textString = "Characteristic Ideal Diodes", lineColor = {0,0,255})}), Documentation(info = "<html>
<p><br/><br/>Three examples of ideal diodes are shown: </p><p><br/><br/>the <b>totally ideal diode</b> (Ideal) with all parameters to be zero </p><p><br/><br/>the <b>nearly ideal diode</b> with <i>Ron=0.1</i> and <i>Goff=0.1</i> </p><p><br/><br/>the nearly ideal but <b>displaced diode</b> with <i>Vknee=5</i> and <i>Ron=0.1</i> and <i>Goff=0.1</i> </p><p><br/><br/>The resistance and conductance are chosen untypically high since the slopes should be seen in the graphics. </p>
<p>Simulate until T=1 s. </p>
<p><br/><br/>Plot in separate windows:</p><p><br/><br/>Ideal.i versus Ideal.v </p><p><br/><br/>With_Ron_Goff.i versus With_Ron_Goff.v</p><p><br/><br/>With_Ron_Goff_Vknee.i versus With_Ron_Goff_Vknee.v</p>
</html>", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Mai 7, 2004   </i>
       by Christoph Clauss<br> realized<br>
       </li>
</ul>
</html>"));
          Modelica.Electrical.Analog.Basic.Resistor R1(R = 0.001) annotation(Placement(transformation(extent = {{60,40},{80,60}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R2(R = 0.001) annotation(Placement(transformation(extent = {{60,0},{80,20}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R3(R = 0.001) annotation(Placement(transformation(extent = {{60, -40},{80, -20}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SineVoltage SineVoltage2(V = 10, offset = 0) annotation(Placement(transformation(origin = { -60,40}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Sources.SineVoltage SineVoltage3(V = 10, offset = 0) annotation(Placement(transformation(origin = { -20, -40}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          annotation(Documentation(info = "<HTML>
<P>
Three examples of ideal diodes are shown:<br><br>
the <b>totally ideal diode</b> (Ideal) with all parameters to be zero <br>
the <b>nearly ideal diode</b> with <i>Ron=0.1</i> and <i>Goff=0.1</i> <br>
the nearly ideal but <b>displaced diode</b> with <i>Vknee=5</i> and <i>Ron=0.1</i> and
<i>Goff=0.1</i> <br><br>
The resistance and conductance are chosen untypically high since the slopes should
be seen in the graphics.
</P>
<P>
Simulate until T=1 s.
</P>
<P>
Plot in separate windows:<br> <br>
Ideal.i versus Ideal.v <br>
With_Ron_Goff.i versus With_Ron_Goff.v<br>
With_Ron_Goff_Vknee.i versus With_Ron_Goff_Vknee.v<br>
</P>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Mai 7, 2004   </i>
       by Christoph Clauss<br> realized<br>
       </li>
</ul>
</HTML>
"), experiment(StopTime = 1));

        equation 
          connect(Ground1.p,SineVoltage1.n) annotation(Line(points = {{ -40, -60},{ -40, -10}}, color = {0,0,255}));
          connect(Ideal.n,R1.p) annotation(Line(points = {{20,50},{60,50}}, color = {0,0,255}));
          connect(With_Ron_Goff.n,R2.p) annotation(Line(points = {{20,10},{60,10}}, color = {0,0,255}));
          connect(With_Ron_Goff_Vknee.n,R3.p) annotation(Line(points = {{20, -30},{60, -30}}, color = {0,0,255}));
          connect(R1.n,R2.n) annotation(Line(points = {{80,50},{80,10}}, color = {0,0,255}));
          connect(R2.n,R3.n) annotation(Line(points = {{80,10},{80, -30}}, color = {0,0,255}));
          connect(R3.n,Ground1.p) annotation(Line(points = {{80, -30},{80, -60},{ -40, -60}}, color = {0,0,255}));
          connect(SineVoltage2.p,Ideal.p) annotation(Line(points = {{ -60,50},{0,50}}, color = {0,0,255}));
          connect(SineVoltage2.n,Ground1.p) annotation(Line(points = {{ -60,30},{ -60, -60},{ -40, -60}}, color = {0,0,255}));
          connect(SineVoltage1.p,With_Ron_Goff.p) annotation(Line(points = {{ -40,10},{0,10}}, color = {0,0,255}));
          connect(With_Ron_Goff_Vknee.p,SineVoltage3.p) annotation(Line(points = {{0, -30},{ -20, -30}}, color = {0,0,255}));
          connect(SineVoltage3.n,Ground1.p) annotation(Line(points = {{ -20, -50},{ -20, -60},{ -40, -60}}, color = {0,0,255}));
        end CharacteristicIdealDiodes;
        model CauerLowPassSC "Cauer low-pass filter with operational amplifiers and switched capacitors"
          model Rn "Negative resistance"
            parameter Modelica.SIunits.Time clock = 1 "Clock";
            parameter Modelica.SIunits.Resistance R(min = Modelica.Constants.eps) = 1 "Resistance";
            Modelica.Blocks.Sources.BooleanPulse BooleanPulse1(period = clock) annotation(Placement(transformation(extent = {{ -10,50},{10,70}}, rotation = 0)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Bitmap(extent = {{ -62,6},{ -62,14}}, fileName = ""),Rectangle(extent = {{ -80,32},{80, -30}}, pattern = LinePattern.None, lineColor = {0,0,255}),Text(extent = {{ -30, -40},{30, -72}}, lineColor = {0,0,0}, textString = "R=%R"),Line(points = {{ -74,0},{ -80,0},{ -60,0}}, color = {85,255,85}),Line(points = {{ -80, -20},{ -60, -20},{ -60, -16},{ -40, -8},{ -4, -8}}, color = {85,255,85}),Line(points = {{80,0},{60,0}}, color = {85,255,85}),Line(points = {{ -60,2},{ -60, -4}}, color = {170,255,170}),Line(points = {{ -4,2},{ -4, -18}}, color = {85,255,85}),Line(points = {{4,2},{4, -18}}, color = {85,255,85}),Text(extent = {{ -60,80},{60,40}}, lineColor = {0,0,255}, textString = "%name"),Line(points = {{ -92,0},{ -80,0}}, color = {85,85,255}),Line(points = {{90,0},{80,0},{80,0}}, color = {85,85,255}),Line(points = {{4, -8},{40, -8},{60, -4}}, color = {85,255,85}),Line(points = {{60,0},{60, -4}}, color = {85,255,85}),Line(points = {{60, -16},{60, -20},{80, -20}}, color = {85,255,85})}), Documentation(info = "<html>
<p>This model is a<b> negative</b> resistor without thermal behavior which is described as a switched capacitor model (care for the schematic).</p>
<p>The clock source is inside the model, its frequency can be chosen by parameter. Also the resistance is a parameter, it has to be <b>positive</b>. The internal (switched) capacitor is parametrized in such a way that the total resistance is independently from the frequency equal to the negative value of the <b>negative</b> resistance parameter.</p>
</html>"));
            Modelica.Electrical.Analog.Basic.Capacitor Capacitor1(C = clock / R) annotation(Placement(transformation(extent = {{ -20, -20},{20,20}}, rotation = 0)));
            Modelica.Electrical.Analog.Ideal.IdealCommutingSwitch IdealCommutingSwitch1 annotation(Placement(transformation(origin = { -50,0}, extent = {{ -10, -10},{10,10}}, rotation = 180)));
            Modelica.Electrical.Analog.Ideal.IdealCommutingSwitch IdealCommutingSwitch2 annotation(Placement(transformation(extent = {{40, -10},{60,10}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground Ground1 annotation(Placement(transformation(extent = {{ -66, -32},{ -54, -20}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground Ground2 annotation(Placement(transformation(extent = {{54, -32},{66, -20}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin n1 annotation(Placement(transformation(extent = {{ -112, -10},{ -92,10}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin n2 annotation(Placement(transformation(extent = {{90, -8},{110,12}}, rotation = 0)));

          equation 
            connect(IdealCommutingSwitch1.p,Capacitor1.p) annotation(Line(points = {{ -40, -1.22465e-15},{ -42, -1.22465e-15},{ -44,0},{ -20,0}}, color = {0,0,255}));
            connect(Capacitor1.n,IdealCommutingSwitch2.p) annotation(Line(points = {{20,0},{40,0}}, color = {0,0,255}));
            connect(IdealCommutingSwitch2.control,BooleanPulse1.y) annotation(Line(points = {{50,8},{50,30},{20,30},{20,60},{11,60}}, color = {255,0,255}));
            connect(IdealCommutingSwitch1.control,BooleanPulse1.y) annotation(Line(points = {{ -50, -8},{ -50,30},{20,30},{20,60},{11,60}}, color = {255,0,255}));
            connect(Ground2.p,IdealCommutingSwitch2.n2) annotation(Line(points = {{60, -20},{60,0}}, color = {0,0,255}));
            connect(IdealCommutingSwitch2.n1,n2) annotation(Line(points = {{60,5},{80,5},{80,2},{100,2}}, color = {0,0,255}));
            connect(n1,IdealCommutingSwitch1.n2) annotation(Line(points = {{ -102,0},{ -81,0},{ -81,1.22465e-15},{ -60,1.22465e-15}}, color = {0,0,255}));
            connect(Ground1.p,IdealCommutingSwitch1.n1) annotation(Line(points = {{ -60, -20},{ -60, -5}}, color = {0,0,255}));
          end Rn;
          model Rp "Positive resistance"
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Bitmap(extent = {{ -62,4},{ -62,12}}, fileName = ""),Rectangle(extent = {{ -80,30},{80, -32}}, pattern = LinePattern.None, lineColor = {0,0,255}),Line(points = {{ -92,0},{ -80,0}}, color = {85,85,255}),Line(points = {{80,0},{92,0}}, color = {85,85,255}),Text(extent = {{ -40, -40},{32, -72}}, lineColor = {0,0,0}, textString = "R=%R"),Line(points = {{ -74,0},{ -80,0},{ -60,0}}, color = {85,255,85}),Line(points = {{ -80, -20},{ -60, -20},{ -60, -16},{ -40, -10},{ -4, -10}}, color = {85,255,85}),Line(points = {{4, -10},{40, -10},{60, -16},{60, -20},{80, -20}}, color = {85,255,85}),Line(points = {{80,0},{60,0}}, color = {85,255,85}),Line(points = {{60,0},{60, -4}}, color = {85,255,85}),Line(points = {{ -60,0},{ -60, -6}}, color = {170,255,170}),Line(points = {{ -4,0},{ -4, -20}}, color = {85,255,85}),Line(points = {{4,0},{4, -20}}, color = {85,255,85}),Text(extent = {{ -60,80},{60,40}}, lineColor = {0,0,255}, textString = "%name")}), Documentation(info = "<html>
<p>This model is a <b>positive</b> resistor without thermal behavior which is described as a switched capacitor model (care for the schematic).</p>
<p>The clock source is inside the model, its frequency can be chosen by parameter. Also the resistance is a parameter, it has to be <b>positive</b>. The internal (switched) capacitor is parametrized in such a way that the total resistance is independently from the frequency equal to the resistance parameter.</p>
</html>"));
            parameter Modelica.SIunits.Time clock = 1 "Clock";
            parameter Modelica.SIunits.Resistance R(min = Modelica.Constants.eps) = 1 "Resistance";
            Modelica.Blocks.Sources.BooleanPulse BooleanPulse1(period = clock) annotation(Placement(transformation(extent = {{ -10,50},{10,70}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Capacitor Capacitor1(C = clock / R) annotation(Placement(transformation(extent = {{ -20, -20},{20,20}}, rotation = 0)));
            Modelica.Electrical.Analog.Ideal.IdealCommutingSwitch IdealCommutingSwitch1 annotation(Placement(transformation(origin = { -50,0}, extent = {{ -10, -10},{10,10}}, rotation = 180)));
            Modelica.Electrical.Analog.Ideal.IdealCommutingSwitch IdealCommutingSwitch2 annotation(Placement(transformation(extent = {{40, -10},{60,10}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground Ground1 annotation(Placement(transformation(origin = { -60,46}, extent = {{ -6, -6},{6,6}}, rotation = 180)));
            Modelica.Electrical.Analog.Basic.Ground Ground2 annotation(Placement(transformation(extent = {{54, -44},{66, -32}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin n1 annotation(Placement(transformation(extent = {{ -112, -10},{ -92,10}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin n2 annotation(Placement(transformation(extent = {{90, -8},{110,12}}, rotation = 0)));

          equation 
            connect(IdealCommutingSwitch1.p,Capacitor1.p) annotation(Line(points = {{ -40, -1.22465e-15},{ -42, -1.22465e-15},{ -44,0},{ -20,0}}, color = {0,0,255}));
            connect(Capacitor1.n,IdealCommutingSwitch2.p) annotation(Line(points = {{20,0},{40,0}}, color = {0,0,255}));
            connect(IdealCommutingSwitch2.control,BooleanPulse1.y) annotation(Line(points = {{50,8},{50,30},{20,30},{20,60},{11,60}}, color = {255,0,255}));
            connect(IdealCommutingSwitch1.control,BooleanPulse1.y) annotation(Line(points = {{ -50, -8},{ -50,30},{20,30},{20,60},{11,60}}, color = {255,0,255}));
            connect(Ground1.p,IdealCommutingSwitch1.n2) annotation(Line(points = {{ -60,40},{ -60,1.22465e-15}}, color = {0,0,255}));
            connect(Ground2.p,IdealCommutingSwitch2.n2) annotation(Line(points = {{60, -32},{60, -32},{60,0}}, color = {0,0,255}));
            connect(IdealCommutingSwitch1.n1,n1) annotation(Line(points = {{ -60, -5},{ -99, -5},{ -99,0},{ -102,0}}, color = {0,0,255}));
            connect(IdealCommutingSwitch2.n1,n2) annotation(Line(points = {{60,5},{80,5},{80,2},{100,2}}, color = {0,0,255}));
          end Rp;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -250, -200},{250,200}}), graphics = {Text(extent = {{ -100,180},{94,140}}, textString = "CauerLowPassSC", lineColor = {0,0,255})}), experiment(StopTime = 60, NumberOfIntervals = 1500), Documentation(revisions = "<html>
<ul>
<li><i>January 13, 2006</i>
       by Christoph Clauss<br>
       included into Modelica Standard Library</li>
<li><i>September 15, 2005</i>
       by by Peter Trappe designed and by Teresa Schlegel<br>
       initially modelled.</li>
</ul>
</html>", info = "<html>
<p>The example CauerLowPassSC is a low-pass-filter of the fifth order. It is realized using an switched-capacitor network with operational amplifiers. The voltage source V is the input voltage (step), and the OP5.out.v is the filter output voltage. The pulse response is calculated. </p>
<p>This model is identical to the CauerLowPassAnalog example, but inverting. To get the same response as that of the CauerLowPassAnalog example, a negative voltage step is used as input. </p>
<p>This model is identical to the CauerLowPassOPV example. But the resistors are realized by switched capacitors. There are two such resistors Rp (of value +1), and Rn (of value -1). In this models the switching clock source is included. In a typical switched capacitor circuit there would be a central clock source. </p>
<p>The simulation end time should be 60. Please plot both V.v (which is the inverted input voltage) and OP5.p.v (output voltage). Compare this result with the CauerLowPassAnalog result. </p>
<p>Due to the recharging of the capacitances after switching the performance of simulation is not as good as in the CauerLowPassOPV example.  </p>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -250, -200},{250,200}}), graphics = {Rectangle(extent = {{ -100, -100},{80,50}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{ -100,50},{ -80,70},{100,70},{80,50},{ -100,50}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{100,70},{100, -80},{80, -100},{80,50},{100,70}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -96,3},{77, -49}}, lineColor = {0,0,255}, textString = "Example"),Text(extent = {{ -120,132},{120,73}}, lineColor = {255,0,0}, textString = "%name")}));
          parameter Modelica.SIunits.Capacitance l1 = 1.304 "filter coefficient i1";
          parameter Modelica.SIunits.Capacitance l2 = 0.8586 "filter coefficient i2";
          parameter Modelica.SIunits.Capacitance c1 = 1.072 "filter coefficient c1";
          parameter Modelica.SIunits.Capacitance c2 = 1 / (1.704992 ^ 2 * l1) "filter coefficient c2";
          parameter Modelica.SIunits.Capacitance c3 = 1.682 "filter coefficient c3";
          parameter Modelica.SIunits.Capacitance c4 = 1 / (1.179945 ^ 2 * l2) "filter coefficient c4";
          parameter Modelica.SIunits.Capacitance c5 = 0.7262 "filter coefficient c5";
          Modelica.Electrical.Analog.Basic.Capacitor C1(C = c1 + c2) annotation(Placement(transformation(extent = {{ -193,30},{ -173,50}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C2(C = c2) annotation(Placement(transformation(extent = {{ -241, -70},{ -221, -50}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C3(C = l1) annotation(Placement(transformation(extent = {{ -91,30},{ -71,50}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C4(C = c4) annotation(Placement(transformation(extent = {{ -31,50},{ -11,70}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C5(C = c2) annotation(Placement(transformation(extent = {{ -31, -100},{ -11, -80}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealOpAmp3Pin Op1 annotation(Placement(transformation(extent = {{ -201, -50},{ -161, -10}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground G annotation(Placement(transformation(extent = {{ -189, -76},{ -175, -60}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealOpAmp3Pin Op2 annotation(Placement(transformation(extent = {{ -101, -50},{ -61, -10}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealOpAmp3Pin Op3 annotation(Placement(transformation(extent = {{5, -50},{45, -10}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground G1 annotation(Placement(transformation(extent = {{ -88, -76},{ -74, -60}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C6(C = c2 + c3 + c4) annotation(Placement(transformation(extent = {{8,30},{28,50}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealOpAmp3Pin Op4 annotation(Placement(transformation(extent = {{99, -50},{139, -10}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealOpAmp3Pin Op5 annotation(Placement(transformation(extent = {{199, -50},{239, -10}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C7(C = l2) annotation(Placement(transformation(extent = {{109,30},{129,50}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C8(C = c4) annotation(Placement(transformation(extent = {{159, -130},{179, -110}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C9(C = c4 + c5) annotation(Placement(transformation(extent = {{207,10},{227,30}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n1 annotation(Placement(transformation(extent = {{ -215, -22},{ -211, -18}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n2 annotation(Placement(transformation(extent = {{ -215,18},{ -211,22}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n3 annotation(Placement(transformation(extent = {{ -215,38},{ -211,42}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n4 annotation(Placement(transformation(origin = { -161,40}, extent = {{2,2},{ -2, -2}}, rotation = 180)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n5 annotation(Placement(transformation(extent = {{ -115, -32},{ -111, -28}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.PositivePin p1 annotation(Placement(transformation(extent = {{ -155, -122},{ -149, -118}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n6 annotation(Placement(transformation(extent = {{ -63,38},{ -59,42}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n7 annotation(Placement(transformation(extent = {{ -7, -32},{ -3, -28}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n8 annotation(Placement(transformation(extent = {{ -7,18},{ -3,22}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.PositivePin p2 annotation(Placement(transformation(extent = {{ -7,38},{ -3,42}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.PositivePin out1 annotation(Placement(transformation(extent = {{43, -122},{47, -118}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.PositivePin p3 annotation(Placement(transformation(extent = {{57,78},{61,82}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n9 annotation(Placement(transformation(extent = {{85,18},{89,22}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n10 annotation(Placement(transformation(extent = {{85, -32},{89, -28}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n11 annotation(Placement(transformation(extent = {{137,38},{141,42}}, rotation = 0)));
        public 
          Modelica.Electrical.Analog.Basic.Ground G2 annotation(Placement(transformation(extent = {{11, -76},{25, -60}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n12 annotation(Placement(transformation(extent = {{237,58},{241,62}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n13 annotation(Placement(transformation(extent = {{237,18},{241,22}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.PositivePin p4 annotation(Placement(transformation(extent = {{187,18},{193,22}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n14 annotation(Placement(transformation(extent = {{187, -32},{193, -28}}, rotation = 0)));
        public 
          Modelica.Electrical.Analog.Basic.Ground G3 annotation(Placement(transformation(extent = {{112, -76},{126, -60}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground G4 annotation(Placement(transformation(extent = {{212, -76},{226, -60}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.StepVoltage V(startTime = 1) annotation(Placement(transformation(origin = { -241, -150}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Electrical.Analog.Basic.Ground Ground1 annotation(Placement(transformation(extent = {{ -247, -182},{ -235, -170}}, rotation = 0)));
          Rn R4(clock = 0.1) annotation(Placement(transformation(extent = {{ -140, -40},{ -120, -20}}, rotation = 0)));
          Rn R5(clock = 0.1) annotation(Placement(transformation(extent = {{ -140, -80},{ -120, -60}}, rotation = 0)));
          Rn R8(clock = 0.1) annotation(Placement(transformation(extent = {{60,10},{80,30}}, rotation = 0)));
          Rn R9(clock = 0.1) annotation(Placement(transformation(extent = {{60, -40},{80, -20}}, rotation = 0)));
          Rp R1(clock = 0.1) annotation(Placement(transformation(extent = {{ -240, -30},{ -220, -10}}, rotation = 0)));
          Rp R2(clock = 0.1) annotation(Placement(transformation(extent = {{ -240,10},{ -220,30}}, rotation = 0)));
          Rp R3(clock = 0.1) annotation(Placement(transformation(extent = {{ -200,70},{ -180,90}}, rotation = 0)));
          Rp Rp1(clock = 0.1) annotation(Placement(transformation(extent = {{ -32,10},{ -12,30}}, rotation = 0)));
          Rp R7(clock = 0.1) annotation(Placement(transformation(extent = {{ -32, -40},{ -12, -20}}, rotation = 0)));
          Rp R10(clock = 0.1) annotation(Placement(transformation(extent = {{160, -40},{180, -20}}, rotation = 0)));
          Rp R11(clock = 0.1) annotation(Placement(transformation(extent = {{208,50},{228,70}}, rotation = 0)));

        equation 
          connect(Op1.in_p,G.p) annotation(Line(points = {{ -201, -40},{ -201, -50},{ -181, -50},{ -181, -60},{ -182, -60}}, color = {0,0,255}));
          connect(G1.p,Op2.in_p) annotation(Line(points = {{ -81, -60},{ -81, -50},{ -101, -50},{ -101, -40}}, color = {0,0,255}));
          connect(n1,Op1.in_n) annotation(Line(points = {{ -213, -20},{ -201, -20}}, color = {0,0,255}));
          connect(C2.n,n1) annotation(Line(points = {{ -221, -60},{ -213, -60},{ -213, -20}}, color = {0,0,255}));
          connect(n1,n2) annotation(Line(points = {{ -213, -20},{ -213,20}}, color = {0,0,255}));
          connect(n2,n3) annotation(Line(points = {{ -213,20},{ -213,40}}, color = {0,0,255}));
          connect(n3,C1.p) annotation(Line(points = {{ -213,40},{ -193,40}}, color = {0,0,255}));
          connect(C1.n,n4) annotation(Line(points = {{ -173,40},{ -161,40}}, color = {0,0,255}));
          connect(n4,Op1.out) annotation(Line(points = {{ -161,40},{ -161, -30}}, color = {0,0,255}));
          connect(C5.p,Op1.out) annotation(Line(points = {{ -31, -90},{ -161, -90},{ -161, -30}}, color = {0,0,255}));
          connect(n5,Op2.in_n) annotation(Line(points = {{ -113, -30},{ -113, -20},{ -101, -20}}, color = {0,0,255}));
          connect(C3.p,n5) annotation(Line(points = {{ -91,40},{ -113,40},{ -113, -30}}, color = {0,0,255}));
          connect(C2.p,p1) annotation(Line(points = {{ -241, -60},{ -241, -120},{ -152, -120}}, color = {0,0,255}));
          connect(C3.n,n6) annotation(Line(points = {{ -71,40},{ -61,40}}, color = {0,0,255}));
          connect(n6,Op2.out) annotation(Line(points = {{ -61,40},{ -61, -30}}, color = {0,0,255}));
          connect(n7,Op3.in_n) annotation(Line(points = {{ -5, -30},{ -5, -20},{5, -20}}, color = {0,0,255}));
          connect(C5.n,n7) annotation(Line(points = {{ -11, -90},{ -5, -90},{ -5, -30}}, color = {0,0,255}));
          connect(n7,n8) annotation(Line(points = {{ -5, -30},{ -5,20}}, color = {0,0,255}));
          connect(C6.p,p2) annotation(Line(points = {{8,40},{ -5,40}}, color = {0,0,255}));
          connect(n8,p2) annotation(Line(points = {{ -5,20},{ -5,40}}, color = {0,0,255}));
          connect(C4.n,p2) annotation(Line(points = {{ -11,60},{ -5,60},{ -5,40}}, color = {0,0,255}));
          connect(C6.n,Op3.out) annotation(Line(points = {{28,40},{45,40},{45, -30}}, color = {0,0,255}));
          connect(Op3.out,out1) annotation(Line(points = {{45, -30},{45, -120}}, color = {0,0,255}));
          connect(p1,out1) annotation(Line(points = {{ -152, -120},{45, -120}}, color = {0,0,255}));
          connect(out1,C8.p) annotation(Line(points = {{45, -120},{159, -120}}, color = {0,0,255}));
          connect(C4.p,p3) annotation(Line(points = {{ -31,60},{ -41,60},{ -41,80},{59,80}}, color = {0,0,255}));
          connect(n9,n10) annotation(Line(points = {{87,20},{87, -30}}, color = {0,0,255}));
          connect(n10,Op4.in_n) annotation(Line(points = {{87, -30},{87, -20},{99, -20}}, color = {0,0,255}));
          connect(n9,C7.p) annotation(Line(points = {{87,20},{87,40},{109,40}}, color = {0,0,255}));
          connect(C7.n,n11) annotation(Line(points = {{129,40},{139,40}}, color = {0,0,255}));
          connect(n11,Op4.out) annotation(Line(points = {{139,40},{139, -30}}, color = {0,0,255}));
          connect(G2.p,Op3.in_p) annotation(Line(points = {{18, -60},{5, -60},{5, -40}}, color = {0,0,255}));
          connect(p3,n12) annotation(Line(points = {{59,80},{239,80},{239,60}}, color = {0,0,255}));
          connect(C9.n,n13) annotation(Line(points = {{227,20},{239,20}}, color = {0,0,255}));
          connect(n12,n13) annotation(Line(points = {{239,60},{239,20}}, color = {0,0,255}));
          connect(n13,Op5.out) annotation(Line(points = {{239,20},{239, -30}}, color = {0,0,255}));
          connect(C9.p,p4) annotation(Line(points = {{207,20},{190,20}}, color = {0,0,255}));
          connect(p4,n14) annotation(Line(points = {{190,20},{190, -30}}, color = {0,0,255}));
          connect(Op5.in_n,n14) annotation(Line(points = {{199, -20},{190, -20},{190, -30}}, color = {0,0,255}));
          connect(C8.n,n14) annotation(Line(points = {{179, -120},{190, -120},{190, -30}}, color = {0,0,255}));
          connect(Op4.in_p,G3.p) annotation(Line(points = {{99, -40},{99, -60},{119, -60}}, color = {0,0,255}));
          connect(Op5.in_p,G4.p) annotation(Line(points = {{199, -40},{199, -60},{219, -60}}, color = {0,0,255}));
          connect(V.p,Ground1.p) annotation(Line(points = {{ -241, -160},{ -241, -170}}, color = {0,0,255}));
          connect(R4.n2,n5) annotation(Line(points = {{ -120, -29.8},{ -116, -29.8},{ -116, -30},{ -113, -30}}, color = {0,0,255}));
          connect(Op1.out,R4.n1) annotation(Line(points = {{ -161, -30},{ -140.2, -30}}, color = {0,0,255}));
          connect(R5.n1,p1) annotation(Line(points = {{ -140.2, -70},{ -152, -70},{ -152, -120}}, color = {0,0,255}));
          connect(R5.n2,n5) annotation(Line(points = {{ -120, -69.8},{ -116, -69.8},{ -116, -70},{ -113, -70},{ -113, -30}}, color = {0,0,255}));
          connect(p3,R8.n1) annotation(Line(points = {{59,80},{59.8,80},{59.8,20}}, color = {0,0,255}));
          connect(R8.n2,n9) annotation(Line(points = {{80,20.2},{84,20.2},{84,20},{87,20}}, color = {0,0,255}));
          connect(Op3.out,R9.n1) annotation(Line(points = {{45, -30},{59.8, -30}}, color = {0,0,255}));
          connect(R9.n2,n10) annotation(Line(points = {{80, -29.8},{84, -29.8},{84, -30},{87, -30}}, color = {0,0,255}));
          connect(R1.n1,V.n) annotation(Line(points = {{ -240.2, -20},{ -250, -20},{ -250, -130},{ -241, -130},{ -241, -140}}, color = {0,0,255}));
          connect(R1.n2,n1) annotation(Line(points = {{ -220, -19.8},{ -216, -19.8},{ -216, -20},{ -213, -20}}, color = {0,0,255}));
          connect(R2.n2,n2) annotation(Line(points = {{ -220,20.2},{ -218,20.2},{ -218,20},{ -213,20}}, color = {0,0,255}));
          connect(R2.n1,n6) annotation(Line(points = {{ -240.2,20},{ -240,20},{ -240,100},{ -61,100},{ -61,40}}, color = {0,0,255}));
          connect(R3.n1,n3) annotation(Line(points = {{ -200.2,80},{ -213,80},{ -213,40}}, color = {0,0,255}));
          connect(R3.n2,n4) annotation(Line(points = {{ -180,80.2},{ -170,80.2},{ -170,80},{ -161,80},{ -161,40}}, color = {0,0,255}));
          connect(Rp1.n2,n8) annotation(Line(points = {{ -12,20.2},{ -8,20.2},{ -8,20},{ -5,20}}, color = {0,0,255}));
          connect(Rp1.n1,n11) annotation(Line(points = {{ -32.2,20},{ -52,20},{ -52,100},{139,100},{139,40}}, color = {0,0,255}));
          connect(Op2.out,R7.n1) annotation(Line(points = {{ -61, -30},{ -32.2, -30}}, color = {0,0,255}));
          connect(R7.n2,n7) annotation(Line(points = {{ -12, -29.8},{ -8, -29.8},{ -8, -30},{ -5, -30}}, color = {0,0,255}));
          connect(R10.n1,Op4.out) annotation(Line(points = {{159.8, -30},{139, -30}}, color = {0,0,255}));
          connect(R10.n2,n14) annotation(Line(points = {{180, -29.8},{186, -29.8},{186, -30},{190, -30}}, color = {0,0,255}));
          connect(R11.n2,n12) annotation(Line(points = {{228,60.2},{234,60.2},{234,60},{239,60}}, color = {0,0,255}));
          connect(R11.n1,p4) annotation(Line(points = {{207.8,60},{190,60},{190,20}}, color = {0,0,255}));
        end CauerLowPassSC;
        model CauerLowPassOPV "Cauer low pass filter with operational amplifiers"
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -250, -200},{250,200}}), graphics = {Text(extent = {{ -130,172},{80,120}}, textString = "CauerLowPassOPV", lineColor = {0,0,255})}), experiment(StopTime = 60), Documentation(revisions = "<html>
<ul>
<li><i>January 13, 2006</i>
       by Christoph Clauss<br>
       included into Modelica Standard Library</li>
<li><i>September 15, 2005</i>
       by by Peter Trappe designed and by Teresa Schlegel<br>
       initially modelled.</li>
</ul>
</html>", info = "<html>
<p>The example Cauer Filter is a low-pass-filter of the fifth order. It is realized using an analog network with operational amplifiers. The voltage source V is the input voltage (step), and the OP5.out.v is the filter output voltage. The pulse response is calculated. </p>
<p>This model is identical to the CauerLowPassAnalog example, but inverting. To get the same response as that of the CauerLowPassAnalog example, a negative voltage step is used as input. </p>
<p>The simulation end time should be 60. Please plot both V.v (which is the inverted input voltage) and OP5.p.v (output voltage). Compare this result with the CauerLowPassAnalog result. </p>
<p>During translation some warnings are issued concerning resistor values (Value=-1 not in range[0,1.e+100]). Do not worry about it. The negative values are o.k.  </p>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -250, -200},{250,200}}), graphics = {Rectangle(extent = {{ -100, -80},{80,70}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{ -100,70},{ -80,90},{100,90},{80,70},{ -100,70}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{100,90},{100, -60},{80, -80},{80,70},{100,90}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -96,23},{77, -29}}, lineColor = {0,0,255}, textString = "Example"),Text(extent = {{ -120,152},{120,93}}, lineColor = {255,0,0}, textString = "%name")}));
          parameter Modelica.SIunits.Capacitance l1 = 1.304 "filter coefficient i1";
          parameter Modelica.SIunits.Capacitance l2 = 0.8586 "filter coefficient i2";
          parameter Modelica.SIunits.Capacitance c1 = 1.072 "filter coefficient c1";
          parameter Modelica.SIunits.Capacitance c2 = 1 / (1.704992 ^ 2 * l1) "filter coefficient c2";
          parameter Modelica.SIunits.Capacitance c3 = 1.682 "filter coefficient c3";
          parameter Modelica.SIunits.Capacitance c4 = 1 / (1.179945 ^ 2 * l2) "filter coefficient c4";
          parameter Modelica.SIunits.Capacitance c5 = 0.7262 "filter coefficient c5";
          Modelica.Electrical.Analog.Basic.Capacitor C1(C = c1 + c2) annotation(Placement(transformation(extent = {{ -192,10},{ -172,30}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C2(C = c2) annotation(Placement(transformation(extent = {{ -240, -90},{ -220, -70}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C3(C = l1) annotation(Placement(transformation(extent = {{ -90,10},{ -70,30}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C4(C = c4) annotation(Placement(transformation(extent = {{ -30,30},{ -10,50}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C5(C = c2) annotation(Placement(transformation(extent = {{ -30, -120},{ -10, -100}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R1 annotation(Placement(transformation(extent = {{ -240, -50},{ -220, -30}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R2 annotation(Placement(transformation(extent = {{ -240, -10},{ -220,10}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R3 annotation(Placement(transformation(extent = {{ -193,50},{ -173,70}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealOpAmp3Pin Op1 annotation(Placement(transformation(extent = {{ -200, -70},{ -160, -30}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground G annotation(Placement(transformation(extent = {{ -188, -96},{ -174, -80}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R4(R =  -1) annotation(Placement(transformation(extent = {{ -140, -60},{ -120, -40}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R5(R =  -1) annotation(Placement(transformation(extent = {{ -140, -100},{ -120, -80}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealOpAmp3Pin Op2 annotation(Placement(transformation(extent = {{ -100, -70},{ -60, -30}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealOpAmp3Pin Op3 annotation(Placement(transformation(extent = {{6, -70},{46, -30}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground G1 annotation(Placement(transformation(extent = {{ -87, -96},{ -73, -80}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R6 annotation(Placement(transformation(extent = {{ -31, -10},{ -11,10}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R7 annotation(Placement(transformation(extent = {{ -31, -60},{ -11, -40}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C6(C = c2 + c3 + c4) annotation(Placement(transformation(extent = {{9,10},{29,30}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R8(R =  -1) annotation(Placement(transformation(extent = {{60, -10},{80,10}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R9(R =  -1) annotation(Placement(transformation(extent = {{60, -60},{80, -40}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R10 annotation(Placement(transformation(extent = {{160, -60},{180, -40}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealOpAmp3Pin Op4 annotation(Placement(transformation(extent = {{100, -70},{140, -30}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealOpAmp3Pin Op5 annotation(Placement(transformation(extent = {{200, -70},{240, -30}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C7(C = l2) annotation(Placement(transformation(extent = {{110,10},{130,30}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C8(C = c4) annotation(Placement(transformation(extent = {{160, -150},{180, -130}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C9(C = c4 + c5) annotation(Placement(transformation(extent = {{208, -10},{228,10}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R11 annotation(Placement(transformation(extent = {{209,30},{229,50}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n1 annotation(Placement(transformation(extent = {{ -214, -42},{ -210, -38}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n2 annotation(Placement(transformation(extent = {{ -214, -2},{ -210,2}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n3 annotation(Placement(transformation(extent = {{ -214,18},{ -210,22}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n4 annotation(Placement(transformation(origin = { -160,20}, extent = {{2,2},{ -2, -2}}, rotation = 180)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n5 annotation(Placement(transformation(extent = {{ -114, -52},{ -110, -48}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.PositivePin p1 annotation(Placement(transformation(extent = {{ -154, -142},{ -148, -138}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n6 annotation(Placement(transformation(extent = {{ -62,18},{ -58,22}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n7 annotation(Placement(transformation(extent = {{ -6, -52},{ -2, -48}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n8 annotation(Placement(transformation(extent = {{ -6, -2},{ -2,2}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.PositivePin p2 annotation(Placement(transformation(extent = {{ -6,18},{ -2,22}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.PositivePin out1 annotation(Placement(transformation(extent = {{44, -142},{48, -138}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.PositivePin p3 annotation(Placement(transformation(extent = {{58,58},{62,62}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n9 annotation(Placement(transformation(extent = {{86, -2},{90,2}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n10 annotation(Placement(transformation(extent = {{86, -52},{90, -48}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n11 annotation(Placement(transformation(extent = {{138,18},{142,22}}, rotation = 0)));
        public 
          Modelica.Electrical.Analog.Basic.Ground G2 annotation(Placement(transformation(extent = {{12, -96},{26, -80}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n12 annotation(Placement(transformation(extent = {{238,38},{242,42}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n13 annotation(Placement(transformation(extent = {{238, -2},{242,2}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.PositivePin p4 annotation(Placement(transformation(extent = {{188, -2},{194,2}}, rotation = 0)));
        protected 
          Modelica.Electrical.Analog.Interfaces.NegativePin n14 annotation(Placement(transformation(extent = {{188, -52},{194, -48}}, rotation = 0)));
        public 
          Modelica.Electrical.Analog.Basic.Ground G3 annotation(Placement(transformation(extent = {{113, -96},{127, -80}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground G4 annotation(Placement(transformation(extent = {{213, -96},{227, -80}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.StepVoltage V(startTime = 1) annotation(Placement(transformation(origin = { -240, -170}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Electrical.Analog.Basic.Ground Ground1 annotation(Placement(transformation(extent = {{ -246, -202},{ -234, -190}}, rotation = 0)));

        equation 
          connect(Op1.in_p,G.p) annotation(Line(points = {{ -200, -60},{ -200, -70},{ -180, -70},{ -180, -80},{ -181, -80}}, color = {0,0,255}));
          connect(G1.p,Op2.in_p) annotation(Line(points = {{ -80, -80},{ -80, -70},{ -100, -70},{ -100, -60}}, color = {0,0,255}));
          connect(R1.n,n1) annotation(Line(points = {{ -220, -40},{ -212, -40}}, color = {0,0,255}));
          connect(n1,Op1.in_n) annotation(Line(points = {{ -212, -40},{ -200, -40}}, color = {0,0,255}));
          connect(C2.n,n1) annotation(Line(points = {{ -220, -80},{ -212, -80},{ -212, -40}}, color = {0,0,255}));
          connect(R2.n,n2) annotation(Line(points = {{ -220,0},{ -212,0}}, color = {0,0,255}));
          connect(n1,n2) annotation(Line(points = {{ -212, -40},{ -212,0}}, color = {0,0,255}));
          connect(n2,n3) annotation(Line(points = {{ -212,0},{ -212,20}}, color = {0,0,255}));
          connect(n3,C1.p) annotation(Line(points = {{ -212,20},{ -192,20}}, color = {0,0,255}));
          connect(n3,R3.p) annotation(Line(points = {{ -212,20},{ -212,60},{ -193,60}}, color = {0,0,255}));
          connect(C1.n,n4) annotation(Line(points = {{ -172,20},{ -160,20}}, color = {0,0,255}));
          connect(R3.n,n4) annotation(Line(points = {{ -173,60},{ -160,60},{ -160,20}}, color = {0,0,255}));
          connect(n4,Op1.out) annotation(Line(points = {{ -160,20},{ -160, -50}}, color = {0,0,255}));
          connect(R4.p,Op1.out) annotation(Line(points = {{ -140, -50},{ -160, -50}}, color = {0,0,255}));
          connect(C5.p,Op1.out) annotation(Line(points = {{ -30, -110},{ -160, -110},{ -160, -50}}, color = {0,0,255}));
          connect(R4.n,n5) annotation(Line(points = {{ -120, -50},{ -112, -50}}, color = {0,0,255}));
          connect(n5,Op2.in_n) annotation(Line(points = {{ -112, -50},{ -112, -40},{ -100, -40}}, color = {0,0,255}));
          connect(C3.p,n5) annotation(Line(points = {{ -90,20},{ -112,20},{ -112, -50}}, color = {0,0,255}));
          connect(R5.n,n5) annotation(Line(points = {{ -120, -90},{ -112, -90},{ -112, -50}}, color = {0,0,255}));
          connect(R5.p,p1) annotation(Line(points = {{ -140, -90},{ -151, -90},{ -151, -140}}, color = {0,0,255}));
          connect(C2.p,p1) annotation(Line(points = {{ -240, -80},{ -240, -140},{ -151, -140}}, color = {0,0,255}));
          connect(C3.n,n6) annotation(Line(points = {{ -70,20},{ -60,20}}, color = {0,0,255}));
          connect(n6,Op2.out) annotation(Line(points = {{ -60,20},{ -60, -50}}, color = {0,0,255}));
          connect(R2.p,n6) annotation(Line(points = {{ -240,0},{ -240,80},{ -60,80},{ -60,20}}, color = {0,0,255}));
          connect(Op2.out,R7.p) annotation(Line(points = {{ -60, -50},{ -31, -50}}, color = {0,0,255}));
          connect(R7.n,n7) annotation(Line(points = {{ -11, -50},{ -4, -50}}, color = {0,0,255}));
          connect(n7,Op3.in_n) annotation(Line(points = {{ -4, -50},{ -4, -40},{6, -40}}, color = {0,0,255}));
          connect(C5.n,n7) annotation(Line(points = {{ -10, -110},{ -4, -110},{ -4, -50}}, color = {0,0,255}));
          connect(R6.n,n8) annotation(Line(points = {{ -11,0},{ -4,0}}, color = {0,0,255}));
          connect(n7,n8) annotation(Line(points = {{ -4, -50},{ -4,0},{ -4,0}}, color = {0,0,255}));
          connect(C6.p,p2) annotation(Line(points = {{9,20},{ -4,20}}, color = {0,0,255}));
          connect(n8,p2) annotation(Line(points = {{ -4,0},{ -4,20}}, color = {0,0,255}));
          connect(C4.n,p2) annotation(Line(points = {{ -10,40},{ -4,40},{ -4,20}}, color = {0,0,255}));
          connect(C6.n,Op3.out) annotation(Line(points = {{29,20},{46,20},{46, -50}}, color = {0,0,255}));
          connect(R9.p,Op3.out) annotation(Line(points = {{60, -50},{46, -50}}, color = {0,0,255}));
          connect(Op3.out,out1) annotation(Line(points = {{46, -50},{46, -140}}, color = {0,0,255}));
          connect(p1,out1) annotation(Line(points = {{ -151, -140},{46, -140}}, color = {0,0,255}));
          connect(out1,C8.p) annotation(Line(points = {{46, -140},{160, -140}}, color = {0,0,255}));
          connect(C4.p,p3) annotation(Line(points = {{ -30,40},{ -40,40},{ -40,60},{60,60}}, color = {0,0,255}));
          connect(p3,R8.p) annotation(Line(points = {{60,60},{60,0}}, color = {0,0,255}));
          connect(R8.n,n9) annotation(Line(points = {{80,0},{88,0}}, color = {0,0,255}));
          connect(n9,n10) annotation(Line(points = {{88,0},{88, -50}}, color = {0,0,255}));
          connect(R9.n,n10) annotation(Line(points = {{80, -50},{88, -50}}, color = {0,0,255}));
          connect(n10,Op4.in_n) annotation(Line(points = {{88, -50},{88, -40},{100, -40}}, color = {0,0,255}));
          connect(n9,C7.p) annotation(Line(points = {{88,0},{88,20},{110,20}}, color = {0,0,255}));
          connect(C7.n,n11) annotation(Line(points = {{130,20},{140,20}}, color = {0,0,255}));
          connect(R6.p,n11) annotation(Line(points = {{ -31,0},{ -48,0},{ -48,80},{140,80},{140,20}}, color = {0,0,255}));
          connect(n11,Op4.out) annotation(Line(points = {{140,20},{140, -50}}, color = {0,0,255}));
          connect(Op4.out,R10.p) annotation(Line(points = {{140, -50},{160, -50}}, color = {0,0,255}));
          connect(G2.p,Op3.in_p) annotation(Line(points = {{19, -80},{6, -80},{6, -60}}, color = {0,0,255}));
          connect(R11.n,n12) annotation(Line(points = {{229,40},{240,40}}, color = {0,0,255}));
          connect(p3,n12) annotation(Line(points = {{60,60},{240,60},{240,40}}, color = {0,0,255}));
          connect(C9.n,n13) annotation(Line(points = {{228,0},{240,0}}, color = {0,0,255}));
          connect(n12,n13) annotation(Line(points = {{240,40},{240,0}}, color = {0,0,255}));
          connect(n13,Op5.out) annotation(Line(points = {{240,0},{240, -50}}, color = {0,0,255}));
          connect(C9.p,p4) annotation(Line(points = {{208,0},{191,0}}, color = {0,0,255}));
          connect(R11.p,p4) annotation(Line(points = {{209,40},{191,40},{191,0}}, color = {0,0,255}));
          connect(R10.n,n14) annotation(Line(points = {{180, -50},{191, -50}}, color = {0,0,255}));
          connect(p4,n14) annotation(Line(points = {{191,0},{191, -50}}, color = {0,0,255}));
          connect(Op5.in_n,n14) annotation(Line(points = {{200, -40},{191, -40},{191, -50}}, color = {0,0,255}));
          connect(C8.n,n14) annotation(Line(points = {{180, -140},{191, -140},{191, -50}}, color = {0,0,255}));
          connect(Op4.in_p,G3.p) annotation(Line(points = {{100, -60},{100, -80},{120, -80}}, color = {0,0,255}));
          connect(Op5.in_p,G4.p) annotation(Line(points = {{200, -60},{200, -80},{220, -80}}, color = {0,0,255}));
          connect(V.p,Ground1.p) annotation(Line(points = {{ -240, -180},{ -240, -190}}, color = {0,0,255}));
          connect(V.n,R1.p) annotation(Line(points = {{ -240, -160},{ -250, -160},{ -250, -40},{ -240, -40}}, color = {0,0,255}));
        end CauerLowPassOPV;
        model CauerLowPassAnalog "Cauer low pass filter with analog components"
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -200, -100},{200,100}}), graphics = {Rectangle(extent = {{ -62,32},{ -58,28}}, lineColor = {0,0,255}, fillColor = {85,85,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -2,28},{2,32}}, lineColor = {0,0,255}, fillColor = {85,85,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{58,32},{62,28}}, lineColor = {0,0,255}, fillColor = {85,85,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{58, -48},{62, -52}}, lineColor = {0,0,255}, fillColor = {85,85,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -2, -48},{2, -52}}, lineColor = {0,0,255}, fillColor = {85,85,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -62, -48},{ -58, -52}}, lineColor = {0,0,255}, fillColor = {85,85,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -110,116},{100,64}}, textString = "CauerLowPassAnalog", lineColor = {0,0,255})}), experiment(StopTime = 60), Documentation(revisions = "<html>
<ul>
<li><i>January 13, 2006</i>
       by Christoph Clauss<br>
       included into Modelica Standard Library</li>
<li><i>September 15, 2005</i>
       by Peter Trappe designed and by Teresa Schlegel<br>
       initially modelled.</li>
</ul>
</html>", info = "<html>
<p>The example Cauer Filter is a low-pass-filter of the fifth order. It is realized using an analog network. The voltage source V is the input voltage (step), and the R2.p.v is the filter output voltage. The pulse response is calculated.  </p>
<p>The simulation end time should be 60. Please plot both V.p.v (input voltage) and R2.p.v (output voltage). </p>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -200, -100},{200,100}}), graphics = {Rectangle(extent = {{ -80, -90},{100,60}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{ -80,60},{ -60,80},{120,80},{100,60},{ -80,60}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{120,80},{120, -70},{100, -90},{100,60},{120,80}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -76,13},{97, -39}}, lineColor = {0,0,255}, textString = "Example"),Text(extent = {{ -100,142},{140,83}}, lineColor = {255,0,0}, textString = "%name")}));
          parameter Modelica.SIunits.Inductance l1 = 1.304 "filter coefficient I1";
          parameter Modelica.SIunits.Inductance l2 = 0.8586 "filter coefficient I2";
          parameter Modelica.SIunits.Capacitance c1 = 1.072 "filter coefficient c1";
          parameter Modelica.SIunits.Capacitance c2 = 1 / (1.704992 ^ 2 * l1) "filter coefficient c2";
          parameter Modelica.SIunits.Capacitance c3 = 1.682 "filter coefficient c3";
          parameter Modelica.SIunits.Capacitance c4 = 1 / (1.179945 ^ 2 * l2) "filter coefficient c4";
          parameter Modelica.SIunits.Capacitance c5 = 0.7262 "filter coefficient c5";
          Modelica.Electrical.Analog.Basic.Ground G annotation(Placement(transformation(extent = {{ -10, -90},{10, -70}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C1(C = c1) annotation(Placement(transformation(origin = { -60, -20}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Capacitor C2(C = c2) annotation(Placement(transformation(extent = {{ -40,20},{ -20,40}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C3(C = c3) annotation(Placement(transformation(origin = {0, -20}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Capacitor C4(C = c4) annotation(Placement(transformation(extent = {{20,20},{40,40}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Capacitor C5(C = c5) annotation(Placement(transformation(origin = {60, -20}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Inductor L1(L = l1) annotation(Placement(transformation(extent = {{ -40,60},{ -20,80}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Inductor L2(L = l2) annotation(Placement(transformation(extent = {{20,60},{40,80}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R1 annotation(Placement(transformation(extent = {{ -100,20},{ -80,40}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor R2 annotation(Placement(transformation(origin = {100, -20}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Sources.StepVoltage V(startTime = 1, offset = 0) annotation(Placement(transformation(origin = { -100, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));

        equation 
          connect(R1.n,C1.p) annotation(Line(points = {{ -80,30},{ -60,30},{ -60, -10}}, color = {0,0,255}));
          connect(C1.n,G.p) annotation(Line(points = {{ -60, -30},{ -60, -50},{0, -50},{0, -70}}, color = {0,0,255}));
          connect(R1.n,C2.p) annotation(Line(points = {{ -80,30},{ -40,30}}, color = {0,0,255}));
          connect(L1.p,C2.p) annotation(Line(points = {{ -40,70},{ -40,30}}, color = {0,0,255}));
          connect(L1.p,C1.p) annotation(Line(points = {{ -40,70},{ -40,30},{ -60,30},{ -60, -10}}, color = {0,0,255}));
          connect(L1.n,C2.n) annotation(Line(points = {{ -20,70},{ -20,30}}, color = {0,0,255}));
          connect(C2.n,C3.p) annotation(Line(points = {{ -20,30},{1.83697e-15,30},{1.83697e-15, -10}}, color = {0,0,255}));
          connect(C2.n,C4.p) annotation(Line(points = {{ -20,30},{20,30}}, color = {0,0,255}));
          connect(L1.n,C3.p) annotation(Line(points = {{ -20,70},{ -20,30},{1.83697e-15,30},{1.83697e-15, -10}}, color = {0,0,255}));
          connect(L1.n,C4.p) annotation(Line(points = {{ -20,70},{ -20,30},{20,30}}, color = {0,0,255}));
          connect(L2.p,C4.p) annotation(Line(points = {{20,70},{20,30}}, color = {0,0,255}));
          connect(C2.n,L2.p) annotation(Line(points = {{ -20,30},{20,30},{20,70}}, color = {0,0,255}));
          connect(C3.p,L2.p) annotation(Line(points = {{1.83697e-15, -10},{0, -10},{0,30},{20,30},{20,70}}, color = {0,0,255}));
          connect(L2.n,C4.n) annotation(Line(points = {{40,70},{40,30}}, color = {0,0,255}));
          connect(L2.n,C5.p) annotation(Line(points = {{40,70},{40,30},{60,30},{60, -10}}, color = {0,0,255}));
          connect(L2.n,R2.p) annotation(Line(points = {{40,70},{40,30},{100,30},{100, -10}}, color = {0,0,255}));
          connect(R2.n,G.p) annotation(Line(points = {{100, -30},{100, -50},{0, -50},{0, -70}}, color = {0,0,255}));
          connect(C4.n,C5.p) annotation(Line(points = {{40,30},{60,30},{60, -10}}, color = {0,0,255}));
          connect(C4.n,R2.p) annotation(Line(points = {{40,30},{100,30},{100, -10}}, color = {0,0,255}));
          connect(C3.n,G.p) annotation(Line(points = {{ -1.83697e-15, -30},{0, -30},{0, -70}}, color = {0,0,255}));
          connect(C5.n,G.p) annotation(Line(points = {{60, -30},{60, -50},{0, -50},{0, -70}}, color = {0,0,255}));
          connect(C1.n,C3.n) annotation(Line(points = {{ -60, -30},{ -60, -50},{0, -50},{0, -30},{ -1.83697e-15, -30}}, color = {0,0,255}));
          connect(C1.n,C5.n) annotation(Line(points = {{ -60, -30},{ -60, -50},{60, -50},{60, -30}}, color = {0,0,255}));
          connect(R2.n,C5.n) annotation(Line(points = {{100, -30},{100, -50},{60, -50},{60, -30}}, color = {0,0,255}));
          connect(R2.n,C3.n) annotation(Line(points = {{100, -30},{100, -50},{0, -50},{0, -30},{ -1.83697e-15, -30}}, color = {0,0,255}));
          connect(R2.n,C1.n) annotation(Line(points = {{100, -30},{100, -50},{ -60, -50},{ -60, -30}}, color = {0,0,255}));
          connect(C5.p,R2.p) annotation(Line(points = {{60, -10},{60,30},{100,30},{100, -10}}, color = {0,0,255}));
          connect(R1.p,V.p) annotation(Line(points = {{ -100,30},{ -100,0}}, color = {0,0,255}));
          connect(V.n,G.p) annotation(Line(points = {{ -100, -20},{ -100, -70},{0, -70}}, color = {0,0,255}));
        end CauerLowPassAnalog;
        model AmplifierWithOpAmpDetailed "Simple Amplifier circuit which uses OpAmpDetailed"
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 0.003, NumberOfIntervals = 2500, Tolerance = 2e-07), Documentation(revisions = "<html>
<ul>
<li><i>  </i>
       </li>
<li><i>Juni 17, 2009   </i>
       by Susann Wolf<br> realized<br>
       </li>
</ul>
</html>", info = "<html>
<p>With the test circuit AmplifierWithOpAmpDetailed a time domain analysis of the example arrangement with a sinusoidal input voltage (12 V amplitude, frequency 1 kHz) using the operational amplifier model OpAmpDetailed is carried out. The working voltages are 15 V and -15 V.  </p>
</html>"), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -98},{80,52}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{ -100,52},{ -80,72},{100,72},{80,52},{ -100,52}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{100,72},{100, -78},{80, -98},{80,52},{100,72}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -96,5},{77, -47}}, lineColor = {0,0,255}, textString = "Example"),Text(extent = {{ -120,134},{120,75}}, lineColor = {255,0,0}, textString = "%name")}));
          Modelica.Electrical.Analog.Basic.OpAmpDetailed opAmp annotation(Placement(transformation(extent = {{ -26, -4},{2,24}})));
          Modelica.Electrical.Analog.Basic.Resistor resistor(R = 10000) annotation(Placement(transformation(extent = {{ -74,8},{ -56,26}})));
          Modelica.Electrical.Analog.Basic.Resistor resistor1(R = 20000) annotation(Placement(transformation(extent = {{ -22,46},{ -4,64}})));
          Modelica.Electrical.Analog.Basic.Resistor resistor2(R = 10000) annotation(Placement(transformation(extent = {{ -9, -9},{9,9}}, rotation = 270, origin = {21, -23})));
          Modelica.Electrical.Analog.Sources.SineVoltage sineVoltage(V = 12, freqHz = 1000) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 270, origin = { -80, -28})));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(Placement(transformation(extent = {{ -20, -70},{0, -50}})));
          Modelica.Electrical.Analog.Sources.ConstantVoltage constantVoltage(V = 15) annotation(Placement(transformation(extent = {{48,10},{68,30}})));
          Modelica.Electrical.Analog.Sources.ConstantVoltage constantVoltage1(V =  -15) annotation(Placement(transformation(extent = {{48, -10},{68,10}})));

        equation 
          connect(resistor.n,opAmp.m) annotation(Line(points = {{ -56,17},{ -34,17},{ -34,17.07},{ -26.07,17.07}}, color = {0,0,255}, smooth = Smooth.None));
          connect(resistor1.n,resistor2.p) annotation(Line(points = {{ -4,55},{10,55},{10,54},{22,54},{22, -14},{21, -14}}, color = {0,0,255}, smooth = Smooth.None));
          connect(resistor.p,sineVoltage.p) annotation(Line(points = {{ -74,17},{ -80,17},{ -80, -18}}, color = {0,0,255}, smooth = Smooth.None));
          connect(resistor1.p,opAmp.m) annotation(Line(points = {{ -22,55},{ -26.07,55},{ -26.07,17.07}}, color = {0,0,255}, smooth = Smooth.None));
          connect(sineVoltage.n,ground.p) annotation(Line(points = {{ -80, -38},{ -10, -38},{ -10, -50}}, color = {0,0,255}, smooth = Smooth.None));
          connect(opAmp.p,ground.p) annotation(Line(points = {{ -26.07,2.93},{ -26.07, -20},{ -10, -20},{ -10, -50}}, color = {0,0,255}, smooth = Smooth.None));
          connect(resistor2.n,ground.p) annotation(Line(points = {{21, -32},{20, -32},{20, -40},{ -10, -40},{ -10, -50}}, color = {0,0,255}, smooth = Smooth.None));
          connect(opAmp.p_supply,constantVoltage.p) annotation(Line(points = {{ -11.93,20.29},{18.035,20.29},{18.035,20},{48,20}}, color = {0,0,255}, smooth = Smooth.None));
          connect(opAmp.m_supply,constantVoltage1.p) annotation(Line(points = {{ -12, -0.36},{18, -0.36},{18,0},{48,0}}, color = {0,0,255}, smooth = Smooth.None));
          connect(constantVoltage.n,constantVoltage1.n) annotation(Line(points = {{68,20},{80,20},{80,0},{68,0}}, color = {0,0,255}, smooth = Smooth.None));
          connect(constantVoltage1.n,ground.p) annotation(Line(points = {{68,0},{80,0},{80, -46},{ -10, -46},{ -10, -50}}, color = {0,0,255}, smooth = Smooth.None));
          connect(opAmp.outp,resistor2.p) annotation(Line(points = {{2,10},{21,10},{21, -14}}, color = {0,0,255}, smooth = Smooth.None));
        end AmplifierWithOpAmpDetailed;
      end Examples;
      package Sources "Time-dependend and controlled voltage and current sources"
        extends Modelica.Icons.Library;
        annotation(Documentation(info = "<html>
<p>This package contains time-dependend and controlled voltage and current sources. Most of the sources use the behavior modeled in the Modelica.Blocks.Sources package. All sources are ideal in the sense that <b>no</b> internal resistances are included.</p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden<br>
<p>
<dt>
<b>Copyright:</b>
<dd>
Copyright &copy; 1998-2006, Modelica Association and Fraunhofer-Gesellschaft.<br>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package
Modelica in file \"Modelica/package.mo\".</i><br>
<p>
</dl>
</html>"));
        model SignalVoltage "Generic voltage source using the input signal as source voltage"
          Interfaces.PositivePin p annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Interfaces.NegativePin n annotation(Placement(transformation(extent = {{110, -10},{90,10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput v "Voltage between pin p and n (= p.v - n.v) as input signal" annotation(Placement(transformation(origin = {0,70}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
          SI.Current i "Current flowing from pin p to pin n";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{ -50,0}}, color = {0,0,0}),Line(points = {{50,0},{90,0}}, color = {0,0,0}),Line(points = {{ -50,0},{50,0}}, color = {0,0,0}),Text(extent = {{ -150, -104},{150, -64}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -120,50},{ -20,0}}, lineColor = {0,0,255}, textString = "+"),Text(extent = {{20,50},{120,0}}, lineColor = {0,0,255}, textString = "-")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -96,0},{ -50,0}}, color = {0,0,0}),Line(points = {{50,0},{96,0}}, color = {0,0,0}),Line(points = {{ -50,0},{50,0}}, color = {0,0,0}),Line(points = {{ -109,20},{ -84,20}}, color = {160,160,164}),Polygon(points = {{ -94,23},{ -84,20},{ -94,17},{ -94,23}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Line(points = {{91,20},{116,20}}, color = {160,160,164}),Text(extent = {{ -109,25},{ -89,45}}, lineColor = {160,160,164}, textString = "i"),Polygon(points = {{106,23},{116,20},{106,17},{106,23}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Text(extent = {{91,45},{111,25}}, lineColor = {160,160,164}, textString = "i"),Line(points = {{ -119, -5},{ -119,5}}, color = {160,160,164}),Line(points = {{ -124,0},{ -114,0}}, color = {160,160,164}),Line(points = {{116,0},{126,0}}, color = {160,160,164})}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Martin Otter<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>The signall voltage source is a parameterless converter of real valued signals into a the source voltage. No further effects are modeled. The real valued signal has to be provided by components of the blocks library. It can be regarded as the &QUOT;Opposite&QUOT; of a voltage sensor.</p>
</html>"));

        equation 
          v = p.v - n.v;
          0 = p.i + n.i;
          i = p.i;
        end SignalVoltage;
        model ConstantVoltage "Source for constant voltage"
          parameter SI.Voltage V(start = 1) "Value of constant voltage";
          extends Interfaces.OnePort;
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -90,0},{ -10,0}}, color = {0,0,0}),Line(points = {{ -10,60},{ -10, -60}}, color = {0,0,0}),Line(points = {{0,30},{0, -30}}, color = {0,0,0}),Line(points = {{0,0},{90,0}}, color = {0,0,0}),Text(extent = {{ -150, -112},{150, -72}}, textString = "%name=%V", lineColor = {0,0,255}),Text(extent = {{ -120,50},{ -20,0}}, lineColor = {0,0,255}, textString = "+"),Text(extent = {{20,50},{120,0}}, lineColor = {0,0,255}, textString = "-")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -90,0},{ -10,0}}, color = {0,0,0}),Line(points = {{ -10,60},{ -10, -60}}, color = {0,0,0}),Line(points = {{0,30},{0, -30}}, color = {0,0,0}),Line(points = {{0,0},{90,0}}, color = {0,0,0})}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>The ConstantVoltage source is a simple source for an ideal constant voltage which is provided by a parameter. There is no internal resistance modeled. If it is used instead of a battery model it is not very realistic: This battery will never be unloaded.</p>
</html>"));

        equation 
          v = V;
        end ConstantVoltage;
        model StepVoltage "Step voltage source"
          parameter SI.Voltage V(start = 1) "Height of step";
          extends Interfaces.VoltageSource(redeclare Modelica.Blocks.Sources.Step signalSource(height = V));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70, -70},{0, -70},{0,70},{69,70}}, color = {192,192,192})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Line(points = {{ -80, -18},{0, -18},{0,50},{80,50}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{70, -80},{94, -100}}, lineColor = {160,160,164}, textString = "time"),Text(extent = {{ -21, -72},{25, -90}}, lineColor = {160,160,164}, textString = "startTime"),Line(points = {{0, -17},{0, -71}}, color = {192,192,192}, pattern = LinePattern.Dash),Text(extent = {{ -68, -36},{ -22, -54}}, lineColor = {160,160,164}, textString = "offset"),Line(points = {{ -13,50},{ -13, -17}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Polygon(points = {{2,50},{ -19,50},{2,50}}, lineColor = {192,192,192}, pattern = LinePattern.Dash),Polygon(points = {{ -13, -17},{ -16, -4},{ -10, -4},{ -13, -17},{ -13, -17}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -13,50},{ -16,37},{ -9,37},{ -13,50}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -68,26},{ -22,8}}, lineColor = {160,160,164}, textString = "V"),Polygon(points = {{ -13, -69},{ -16, -56},{ -10, -56},{ -13, -69},{ -13, -69}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -13, -18},{ -13, -70}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Polygon(points = {{ -13, -18},{ -16, -31},{ -9, -31},{ -13, -18}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -67,93},{ -2,67}}, lineColor = {160,160,164}, textString = "v = p.v - n.v")}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This voltage source uses the coresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end StepVoltage;
        model RampVoltage "Ramp voltage source"
          parameter SI.Voltage V(start = 1) "Height of ramp";
          parameter SI.Time duration(min = Modelica.Constants.small, start = 2) "Duration of ramp";
          extends Interfaces.VoltageSource(redeclare Modelica.Blocks.Sources.Ramp signalSource(final height = V, final duration = duration));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Line(points = {{ -80, -20},{ -20, -20},{50,50}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -40, -20},{ -42, -30},{ -37, -30},{ -40, -20}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -40, -20},{ -40, -70}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Polygon(points = {{ -40, -70},{ -43, -60},{ -38, -60},{ -40, -70},{ -40, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -80, -33},{ -41, -49}}, lineColor = {160,160,164}, textString = "offset"),Text(extent = {{ -40, -70},{6, -88}}, lineColor = {160,160,164}, textString = "startTime"),Text(extent = {{ -67,93},{ -2,67}}, lineColor = {160,160,164}, textString = "v = p.v - n.v"),Text(extent = {{70, -80},{94, -100}}, lineColor = {160,160,164}, textString = "time"),Line(points = {{ -20, -20},{ -20, -70}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -19, -20},{50, -20}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Line(points = {{50,50},{101,50}}, color = {0,0,0}, thickness = 0.5),Line(points = {{50,50},{50, -20}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Polygon(points = {{50, -20},{42, -18},{42, -22},{50, -20}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -20, -20},{ -11, -18},{ -11, -22},{ -20, -20}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{50,50},{48,40},{53,40},{50,50}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{50, -20},{47, -10},{52, -10},{50, -20},{50, -20}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{53,25},{82,7}}, lineColor = {160,160,164}, textString = "V"),Text(extent = {{0, -17},{35, -37}}, lineColor = {160,160,164}, textString = "duration")}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80, -60},{ -50, -60},{50,60},{80,60}}, color = {192,192,192})}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This voltage source uses the coresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end RampVoltage;
        model SineVoltage "Sine voltage source"
          parameter SI.Voltage V(start = 1) "Amplitude of sine wave";
          parameter SI.Angle phase = 0 "Phase of sine wave";
          parameter SI.Frequency freqHz(start = 1) "Frequency of sine wave";
          extends Interfaces.VoltageSource(redeclare Modelica.Blocks.Sources.Sine signalSource(amplitude = V, freqHz = freqHz, phase = phase));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -66,0},{ -56.2,29.9},{ -49.8,46.5},{ -44.2,58.1},{ -39.3,65.2},{ -34.3,69.2},{ -29.4,69.8},{ -24.5,67},{ -19.6,61},{ -14.6,52},{ -9,38.6},{ -1.98,18.6},{12.79, -26.9},{19.1, -44},{24.8, -56.2},{29.7, -64},{34.6, -68.6},{39.5, -70},{44.5, -67.9},{49.4, -62.5},{54.3, -54.1},{59.9, -41.3},{67, -21.7},{74,0}}, color = {192,192,192})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80, -90},{ -80,84}}, color = {192,192,192}),Polygon(points = {{ -80,100},{ -86,84},{ -74,84},{ -80,100}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -99, -40},{85, -40}}, color = {192,192,192}),Polygon(points = {{101, -40},{85, -34},{85, -46},{101, -40}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -40,0},{ -31.6,34.2},{ -26.1,53.1},{ -21.3,66.4},{ -17.1,74.6},{ -12.9,79.1},{ -8.64,79.8},{ -4.42,76.6},{ -0.201,69.7},{4.02,59.4},{8.84,44.1},{14.9,21.2},{27.5, -30.8},{33, -50.2},{37.8, -64.2},{42, -73.1},{46.2, -78.4},{50.5, -80},{54.7, -77.6},{58.9, -71.5},{63.1, -61.9},{67.9, -47.2},{74, -24.8},{80,0}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -41, -2},{ -80, -2}}, color = {0,0,0}, thickness = 0.5),Text(extent = {{ -106, -11},{ -60, -29}}, lineColor = {160,160,164}, textString = "offset"),Line(points = {{ -41, -2},{ -41, -40}}, color = {192,192,192}, pattern = LinePattern.Dash),Text(extent = {{ -60, -43},{ -14, -61}}, lineColor = {160,160,164}, textString = "startTime"),Text(extent = {{84, -52},{108, -72}}, lineColor = {160,160,164}, textString = "time"),Line(points = {{ -9,79},{43,79}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -42, -1},{50,0}}, color = {192,192,192}, pattern = LinePattern.Dash),Polygon(points = {{33,80},{30,67},{37,67},{33,80}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{37,57},{83,39}}, lineColor = {160,160,164}, textString = "V"),Polygon(points = {{33,1},{30,14},{36,14},{33,1},{33,1}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{33,79},{33,0}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Text(extent = {{ -69,109},{ -4,83}}, lineColor = {160,160,164}, textString = "v = p.v - n.v")}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This voltage source uses the coresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end SineVoltage;
        model ExpSineVoltage "Exponentially damped sine voltage source"
          parameter SI.Voltage V(start = 1) "Amplitude of sine wave";
          parameter SI.Frequency freqHz(start = 2) "Frequency of sine wave";
          parameter SI.Angle phase = 0 "Phase of sine wave";
          parameter SI.Damping damping(start = 1) "Damping coefficient of sine wave";
          extends Interfaces.VoltageSource(redeclare Modelica.Blocks.Sources.ExpSine signalSource(amplitude = V, freqHz = freqHz, phase = phase, damping = damping));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -64, -14},{ -59.2,18.3},{ -56,36.3},{ -52.7,50.5},{ -49.5,60.2},{ -46.3,65.3},{ -43.1,65.6},{ -39.9,61.3},{ -36.7,53.1},{ -32.6,38.2},{ -27,11.8},{ -19, -27.9},{ -14.2, -47.7},{ -10.1, -59.9},{ -6.1, -67.2},{ -2.1, -69.3},{1.9, -66.5},{5.9, -59.3},{10.77, -46.1},{24.44, -0.3},{29.3,12.4},{34.1,20.8},{38.1,24},{42.9,23.2},{47.8,17.8},{54.2,5.4},{67.1, -24.5},{73.5, -35.2},{79.1, -39.9},{84.7, -39.9},{91.2, -34.5},{96, -27.8}}, color = {192,192,192})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80, -90},{ -80,84}}, color = {192,192,192}),Polygon(points = {{ -80,100},{ -86,84},{ -74,84},{ -80,100}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -99, -40},{85, -40}}, color = {192,192,192}),Polygon(points = {{101, -40},{85, -34},{85, -46},{101, -40}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -50,0},{ -46.1,28.2},{ -43.5,44},{ -40.9,56.4},{ -38.2,64.9},{ -35.6,69.4},{ -33,69.6},{ -30.4,65.9},{ -27.8,58.7},{ -24.5,45.7},{ -19.9,22.5},{ -13.4, -12.2},{ -9.5, -29.5},{ -6.23, -40.1},{ -2.96, -46.5},{0.302, -48.4},{3.57, -45.9},{6.83, -39.6},{10.8, -28.1},{21.9,12},{25.8,23.1},{29.7,30.5},{33,33.3},{36.9,32.5},{40.8,27.8},{46,16.9},{56.5, -9.2},{61.7, -18.6},{66.3, -22.7},{70.9, -22.6},{76.1, -18},{80, -12.1}}, color = {0,0,0}, thickness = 0.5),Text(extent = {{ -78,1},{ -55, -19}}, lineColor = {160,160,164}, textString = "offset"),Text(extent = {{ -72, -36},{ -26, -54}}, lineColor = {160,160,164}, textString = "startTime"),Text(extent = {{84, -52},{108, -72}}, lineColor = {160,160,164}, textString = "time"),Line(points = {{ -50,0},{18,0}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -50,0},{ -81,0}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -50,77},{ -50,0}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{18, -1},{18,76}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{18,73},{ -50,73}}, color = {192,192,192}),Text(extent = {{ -42,88},{9,74}}, lineColor = {160,160,164}, textString = "1/freqHz"),Polygon(points = {{ -49,73},{ -40,75},{ -40,71},{ -49,73}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{18,73},{10,75},{10,71},{18,73}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -50, -61},{ -19, -61}}, color = {192,192,192}),Polygon(points = {{ -18, -61},{ -26, -59},{ -26, -63},{ -18, -61}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -51, -63},{ -27, -75}}, lineColor = {160,160,164}, textString = "t"),Text(extent = {{ -82, -67},{108, -96}}, lineColor = {160,160,164}, textString = "V*exp(-damping*t)*sin(2*pi*freqHz*t+phase)"),Line(points = {{ -50,0},{ -50, -40}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -50, -54},{ -50, -72}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{18, -76},{ -1, -48}}, color = {192,192,192}, pattern = LinePattern.Dash),Text(extent = {{ -77,105},{ -38,92}}, lineColor = {160,160,164}, textString = "v = p.v - n.v")}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This voltage source uses the coresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end ExpSineVoltage;
        model ExponentialsVoltage "Rising and falling exponential voltage source"
          parameter Real vMax(start = 1) "Upper bound for rising edge";
          parameter SI.Time riseTime(min = 0, start = 0.5) "Rise time";
          parameter SI.Time riseTimeConst(min = Modelica.Constants.small, start = 0.1) "Rise time constant";
          parameter SI.Time fallTimeConst(min = Modelica.Constants.small, start = riseTimeConst) "Fall time constant";
          extends Interfaces.VoltageSource(redeclare Modelica.Blocks.Sources.Exponentials signalSource(outMax = vMax, riseTime = riseTime, riseTimeConst = riseTimeConst, fallTimeConst = fallTimeConst));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -76, -59},{ -73.2, -44.3},{ -70.3, -31.1},{ -66.8, -16.6},{ -63.3, -4},{ -59.7,6.92},{ -55.5,18.18},{ -51.3,27.7},{ -46.3,37},{ -40.6,45.5},{ -34.3,53.1},{ -27.2,59.6},{ -18.7,65.3},{ -8.1,70.2},{ -6,71},{ -3.88,58.5},{ -1.05,43.7},{1.78,30.8},{4.606,19.45},{8.14,7.3},{11.68, -3},{15.9, -13.2},{20.2, -21.6},{25.1, -29.5},{30.8, -36.4},{37.1, -42.3},{44.9, -47.5},{54.8, -51.8},{64, -54.4}}, color = {192,192,192})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -100, -70},{84, -70}}, color = {192,192,192}),Polygon(points = {{100, -70},{84, -64},{84, -76},{100, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -40, -30},{ -37.2, -15.3},{ -34.3, -2.1},{ -30.8,12.4},{ -27.3,25},{ -23.7,35.92},{ -19.5,47.18},{ -15.3,56.7},{ -10.3,66},{ -4.6,74.5},{1.7,82.1},{8.8,88.6},{17.3,94.3},{27.9,99.2},{30,100},{32.12,87.5},{34.95,72.7},{37.78,59.8},{40.606,48.45},{44.14,36.3},{47.68,26},{51.9,15.8},{56.2,7.4},{61.1, -0.5},{66.8, -7.4},{73.1, -13.3},{80.9, -18.5},{90.8, -22.8},{100, -25.4}}, color = {0,0,0}, thickness = 0.5),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Text(extent = {{ -70,91},{ -29,71}}, lineColor = {160,160,164}, textString = "outPort"),Text(extent = {{ -78, -43},{ -46, -56}}, lineColor = {160,160,164}, textString = "offset"),Polygon(points = {{ -40, -70},{ -43, -60},{ -38, -60},{ -40, -70},{ -40, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -40, -29},{ -40, -60}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Polygon(points = {{ -40, -30},{ -42, -40},{ -37, -40},{ -40, -30}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -39, -30},{ -80, -30}}, color = {0,0,0}, thickness = 0.5),Text(extent = {{ -59, -71},{ -13, -89}}, lineColor = {160,160,164}, textString = "startTime"),Polygon(points = {{ -41, -30},{ -32, -28},{ -32, -32},{ -41, -30}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -40, -30},{29, -30}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Polygon(points = {{29, -30},{21, -28},{21, -32},{29, -30}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -26, -12},{19, -28}}, lineColor = {160,160,164}, textString = "riseTime"),Text(extent = {{78, -76},{102, -96}}, lineColor = {160,160,164}, textString = "time"),Line(points = {{ -100, -70},{84, -70}}, color = {192,192,192}),Polygon(points = {{100, -70},{84, -64},{84, -76},{100, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -40, -30},{ -37.2, -15.3},{ -34.3, -2.1},{ -30.8,12.4},{ -27.3,25},{ -23.7,35.92},{ -19.5,47.18},{ -15.3,56.7},{ -10.3,66},{ -4.6,74.5},{1.7,82.1},{8.8,88.6},{17.3,94.3},{27.9,99.2},{30,100},{32.12,87.5},{34.95,72.7},{37.78,59.8},{40.606,48.45},{44.14,36.3},{47.68,26},{51.9,15.8},{56.2,7.4},{61.1, -0.5},{66.8, -7.4},{73.1, -13.3},{80.9, -18.5},{90.8, -22.8},{100, -25.4}}, color = {0,0,0}, thickness = 0.5),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Text(extent = {{ -70,91},{ -29,71}}, lineColor = {160,160,164}, textString = "outPort"),Text(extent = {{ -78, -43},{ -46, -56}}, lineColor = {160,160,164}, textString = "offset"),Polygon(points = {{ -40, -70},{ -43, -60},{ -38, -60},{ -40, -70},{ -40, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -40, -29},{ -40, -60}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Polygon(points = {{ -40, -30},{ -42, -40},{ -37, -40},{ -40, -30}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -39, -30},{ -80, -30}}, color = {0,0,0}, thickness = 0.5),Text(extent = {{ -59, -71},{ -13, -89}}, lineColor = {160,160,164}, textString = "startTime"),Polygon(points = {{ -41, -30},{ -32, -28},{ -32, -32},{ -41, -30}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -40, -30},{29, -30}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Polygon(points = {{29, -30},{21, -28},{21, -32},{29, -30}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -26, -12},{19, -28}}, lineColor = {160,160,164}, textString = "riseTime"),Text(extent = {{78, -76},{102, -96}}, lineColor = {160,160,164}, textString = "time"),Line(points = {{30,100},{30, -34}}, color = {192,192,192}, pattern = LinePattern.Dash)}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This voltage source uses the coresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end ExponentialsVoltage;
        model PulseVoltage "Pulse voltage source"
          parameter SI.Voltage V(start = 1) "Amplitude of pulse";
          parameter Real width(final min = Modelica.Constants.small, final max = 100, start = 50) "Width of pulse in % of period";
          parameter SI.Time period(final min = Modelica.Constants.small, start = 1) "Time for one period";
          extends Interfaces.VoltageSource(redeclare Modelica.Blocks.Sources.Pulse signalSource(amplitude = V, width = width, period = period));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70, -70},{ -40, -70},{ -40,70},{0,70},{0, -70},{40, -70},{40,70},{80,70}}, color = {192,192,192})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -34,1},{ -37, -12},{ -30, -12},{ -34,1}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -34, -1},{ -34, -70}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Polygon(points = {{ -33, -70},{ -36, -57},{ -30, -57},{ -33, -70},{ -33, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -78, -24},{ -35, -36}}, lineColor = {160,160,164}, textString = "offset"),Text(extent = {{ -31, -69},{15, -87}}, lineColor = {160,160,164}, textString = "startTime"),Text(extent = {{70, -80},{94, -100}}, lineColor = {160,160,164}, textString = "time"),Line(points = {{ -10,0},{ -10, -70}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -80,0},{ -20,0},{ -21,50},{21,50},{20, -1},{41,0},{40,50},{80,51}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -19,88},{ -19,49}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{21,74},{21,50}}, color = {160,160,164}, pattern = LinePattern.Dash),Line(points = {{41,88},{41,50}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -19,83},{42,83}}, color = {192,192,192}),Line(points = {{ -19,69},{21,69}}, color = {192,192,192}),Text(extent = {{ -9,97},{37,85}}, lineColor = {160,160,164}, textString = "period"),Text(extent = {{ -18,81},{21,69}}, lineColor = {160,160,164}, textString = "width"),Line(points = {{ -52,50},{ -19,50}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -34,50},{ -34,1}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Text(extent = {{ -78,34},{ -37,20}}, lineColor = {160,160,164}, textString = "V"),Polygon(points = {{ -34,49},{ -37,36},{ -30,36},{ -34,49}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -34,1},{ -37,14},{ -31,14},{ -34,1},{ -34,1}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{80,51},{80,0},{90, -1}}, color = {0,0,0}, thickness = 0.5),Polygon(points = {{ -19,69},{ -10,71},{ -10,67},{ -19,69}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{21,69},{13,71},{13,67},{21,69}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -19,83},{ -10,85},{ -10,81},{ -19,83}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{41,83},{33,85},{33,81},{41,83}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -78,103},{ -24,89}}, lineColor = {160,160,164}, textString = "v = p.v - n.v")}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This voltage source uses the coresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end PulseVoltage;
        model SawToothVoltage "Saw tooth voltage source"
          parameter SI.Voltage V(start = 1) "Amplitude of saw tooth";
          parameter SI.Time period(start = 1) "Time for one period";
          extends Interfaces.VoltageSource(redeclare Modelica.Blocks.Sources.SawTooth signalSource(amplitude = V, period = period));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -85, -70},{ -65, -70},{ -5,71},{ -5, -70},{55,71},{55, -70}}, color = {192,192,192})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -34, -19},{ -37, -32},{ -30, -32},{ -34, -19}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -34, -20},{ -34, -70}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Polygon(points = {{ -34, -70},{ -37, -57},{ -31, -57},{ -34, -70},{ -34, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -78, -24},{ -35, -36}}, lineColor = {160,160,164}, textString = "offset"),Text(extent = {{ -31, -69},{15, -87}}, lineColor = {160,160,164}, textString = "startTime"),Text(extent = {{70, -80},{94, -100}}, lineColor = {160,160,164}, textString = "time"),Line(points = {{ -10, -20},{ -10, -70}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -10,88},{ -10, -20}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{30,88},{30,59}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -10,83},{30,83}}, color = {192,192,192}),Text(extent = {{ -12,97},{34,85}}, lineColor = {160,160,164}, textString = "period"),Line(points = {{ -44,60},{30,60}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -34,47},{ -34, -7}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Text(extent = {{ -78,34},{ -37,20}}, lineColor = {160,160,164}, textString = "V"),Polygon(points = {{ -34,60},{ -37,47},{ -30,47},{ -34,60}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -34, -20},{ -37, -7},{ -31, -7},{ -34, -20},{ -34, -20}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -10,83},{ -1,85},{ -1,81},{ -10,83}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{30,83},{22,85},{22,81},{30,83}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -20},{ -10, -20},{30,60},{30, -20},{72,60},{72, -20}}, color = {0,0,0}, thickness = 0.5),Text(extent = {{ -77,100},{ -27,88}}, lineColor = {160,160,164}, textString = "v = p.v - n.v")}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This voltage source uses the coresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end SawToothVoltage;
        model TrapezoidVoltage "Trapezoidal voltage source"
          parameter SI.Voltage V(start = 1) "Amplitude of trapezoid";
          parameter SI.Time rising(final min = 0, start = 0) "Rising duration of trapezoid";
          parameter SI.Time width(final min = 0, start = 0.5) "Width duration of trapezoid";
          parameter SI.Time falling(final min = 0, start = 0) "Falling duration of trapezoid";
          parameter SI.Time period(final min = Modelica.Constants.small, start = 1) "Time for one period";
          parameter Integer nperiod(start =  -1) "Number of periods (< 0 means infinite number of periods)";
          extends Interfaces.VoltageSource(redeclare Modelica.Blocks.Sources.Trapezoid signalSource(amplitude = V, rising = rising, width = width, falling = falling, period = period, nperiod = nperiod));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -81, -70},{ -60, -70},{ -30,70},{1,70},{30, -70},{51, -70},{80,70}}, color = {192,192,192})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -45, -30},{ -47, -41},{ -43, -41},{ -45, -30}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -45, -31},{ -45, -70}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Polygon(points = {{ -45, -70},{ -47, -60},{ -43, -60},{ -45, -70},{ -45, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -86, -43},{ -43, -55}}, lineColor = {160,160,164}, textString = "offset"),Text(extent = {{ -47, -69},{ -1, -87}}, lineColor = {160,160,164}, textString = "startTime"),Text(extent = {{70, -80},{94, -100}}, lineColor = {160,160,164}, textString = "time"),Line(points = {{ -30,81},{ -30, -70}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -10,59},{ -10,40}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{20,59},{20,39}}, color = {160,160,164}, pattern = LinePattern.Dash),Line(points = {{40,59},{40, -30}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -29,56},{40,56}}, color = {192,192,192}),Text(extent = {{ -8,70},{21,60}}, lineColor = {160,160,164}, textString = "width"),Line(points = {{ -42,40},{ -10,40}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -39,40},{ -39, -19}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Text(extent = {{ -77,14},{ -40,0}}, lineColor = {160,160,164}, textString = "V"),Polygon(points = {{ -29,56},{ -22,58},{ -22,54},{ -29,56}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -10,56},{ -17,58},{ -17,54},{ -10,56}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -30},{ -30, -30},{ -9,46},{21,46},{40, -30},{60, -30},{80,46},{100,46}}, color = {0,0,0}, thickness = 0.5),Polygon(points = {{ -39,40},{ -41,29},{ -37,29},{ -39,40}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -39, -29},{ -41, -19},{ -37, -19},{ -39, -29},{ -39, -29}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{60,81},{60, -30}}, color = {192,192,192}, pattern = LinePattern.Dash),Polygon(points = {{39,56},{32,58},{32,54},{39,56}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{20,56},{27,58},{27,54},{20,56}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{20,56},{13,58},{13,54},{20,56}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -12,56},{ -5,58},{ -5,54},{ -12,56}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -34,70},{ -5,60}}, lineColor = {160,160,164}, textString = "rising"),Text(extent = {{16,70},{45,60}}, lineColor = {160,160,164}, textString = "falling"),Text(extent = {{ -77,103},{ -23,91}}, lineColor = {160,160,164}, textString = "v = p.v - n.v"),Line(points = {{ -20,76},{61,76}}, color = {192,192,192}),Polygon(points = {{ -29,76},{ -20,78},{ -20,74},{ -29,76}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{61,76},{53,78},{53,74},{61,76}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -2,86},{25,77}}, lineColor = {160,160,164}, textString = "period")}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This voltage source uses the coresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end TrapezoidVoltage;
        model TableVoltage "Voltage source by linear interpolation in a table"
          parameter Real table[:,:] = [0,0;1,1;2,4] "Table matrix (time = first column, voltage = second column)";
          extends Interfaces.VoltageSource(redeclare Modelica.Blocks.Sources.TimeTable signalSource(table = table));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -58, -36},{ -58,84},{42,84},{42, -36},{ -58, -36},{ -58, -6},{42, -6},{42,24},{ -58,24},{ -58,54},{42,54},{42,84},{ -8,84},{ -8, -37}}, color = {192,192,192})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -20,90},{30, -30}}, lineColor = {255,255,255}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -20, -30},{ -20,90},{80,90},{80, -30},{ -20, -30},{ -20,0},{80,0},{80,30},{ -20,30},{ -20,60},{80,60},{80,90},{30,90},{30, -31}}, color = {0,0,0}),Text(extent = {{ -77, -42},{ -38, -58}}, lineColor = {160,160,164}, textString = "offset"),Polygon(points = {{ -31, -30},{ -33, -40},{ -28, -40},{ -31, -30}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -30, -70},{ -33, -60},{ -28, -60},{ -30, -70},{ -30, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -31, -31},{ -31, -70}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Line(points = {{ -20, -20},{ -20, -70}}, color = {192,192,192}, pattern = LinePattern.Dash),Text(extent = {{ -38, -70},{8, -88}}, lineColor = {160,160,164}, textString = "startTime"),Line(points = {{ -20, -30},{ -80, -30}}, color = {192,192,192}, pattern = LinePattern.Dash),Text(extent = {{66, -81},{91, -93}}, lineColor = {160,160,164}, textString = "time"),Text(extent = {{ -15,83},{24,68}}, lineColor = {0,0,0}, textString = "time"),Text(extent = {{33,83},{76,67}}, lineColor = {0,0,0}, textString = "v"),Text(extent = {{ -81,98},{ -31,85}}, lineColor = {160,160,164}, textString = "v = p.v - n.v")}), Documentation(info = "<html>
<p>This voltage source uses the coresponding signal source of the Modelica.Blocks.Sources package.  Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
<p><br/>This block generates a voltage source by <b>linear interpolation</b> in a table. The time points and voltage values are stored in a matrix <b>table[i,j]</b>, where the first column table[:,1] contains the time points and the second column contains the voltage to be interpolated. The table interpolation has the following proporties: </p>
<p><ul>
<li>The time points need to be <b>monotonically increasing</b>. </li>
<li><b>Discontinuities</b> are allowed, by providing the same time point twice in the table. </li>
<li>Values <b>outside</b> of the table range, are computed by <b>extrapolation</b> through the last or first two points of the table.</li>
<li>If the table has only <b>one row</b>, no interpolation is performed and the voltage value is just returned independantly of the actual time instant, i.e., this is a constant voltage source.</li>
<li>Via parameters <b>startTime</b> and <b>offset</b> the curve defined by the table can be shifted both in time and in the voltage. </li>
<li>The table is implemented in a numerically sound way by generating <b>time events</b> at interval boundaries, in order to not integrate over a discontinuous or not differentiable points. </li>
</ul></p>
<p>Example: </p>
<pre>   table = [0  0
            1  0
            1  1
            2  4
            3  9
            4 16]
If, e.g., time = 1.0, the voltage v =  0.0 (before event), 1.0 (after event)
    e.g., time = 1.5, the voltage v =  2.5,
    e.g., time = 2.0, the voltage v =  4.0,
    e.g., time = 5.0, the voltage v = 23.0 (i.e. extrapolation). </pre>
<p><br/>  Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"));
        end TableVoltage;
        model SignalCurrent "Generic current source using the input signal as source current"
          Interfaces.PositivePin p annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Interfaces.NegativePin n annotation(Placement(transformation(extent = {{110, -10},{90,10}}, rotation = 0)));
          SI.Voltage v "Voltage drop between the two pins (= p.v - n.v)";
          Modelica.Blocks.Interfaces.RealInput i "Current flowing from pin p to pin n as input signal" annotation(Placement(transformation(origin = {0,70}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{ -50,0}}, color = {0,0,0}),Line(points = {{50,0},{90,0}}, color = {0,0,0}),Line(points = {{0, -50},{0,50}}, color = {0,0,0}),Text(extent = {{ -150, -120},{150, -80}}, textString = "%name", lineColor = {0,0,255}),Polygon(points = {{90,0},{60,10},{60, -10},{90,0}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -96,0},{ -50,0}}, color = {0,0,0}),Line(points = {{50,0},{96,0}}, color = {0,0,0}),Line(points = {{0, -50},{0,50}}, color = {0,0,0})}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Martin Otter<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>The signall current source is a parameterless converter of real valued signals into a the source currrent. No further effects are modeled. The real valued signal has to be provided by components of the blocks library. It can be regarded as the &QUOT;Opposite&QUOT; of a current sensor.</p>
</html>"));

        equation 
          v = p.v - n.v;
          0 = p.i + n.i;
          i = p.i;
        end SignalCurrent;
        model ConstantCurrent "Source for constant current"
          parameter SI.Current I(start = 1) "Value of constant current";
          extends Interfaces.OnePort;
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{ -50,0}}, color = {0,0,0}),Line(points = {{50,0},{90,0}}, color = {0,0,0}),Line(points = {{0, -50},{0,50}}, color = {0,0,0}),Text(extent = {{ -150,80},{150,120}}, textString = "%name=%I", lineColor = {0,0,255}),Polygon(points = {{90,0},{60,10},{60, -10},{90,0}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -96,0},{ -50,0}}, color = {0,0,0}),Line(points = {{50,0},{96,0}}, color = {0,0,0}),Line(points = {{0, -50},{0,50}}, color = {0,0,0})}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>The ConstantCurrrent source is a simple source for an ideal constant current which is provided by a parameter. There is no internal resistance modeled. No further effects are modeled. Especially, th currend flow will never end.</p>
</html>"));

        equation 
          i = I;
        end ConstantCurrent;
        model StepCurrent "Step current source"
          parameter SI.Current I(start = 1) "Height of step";
          extends Interfaces.CurrentSource(redeclare Modelica.Blocks.Sources.Step signalSource(height = I));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -86, -70},{ -14, -70},{ -14,70},{57,70}}, color = {192,192,192})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Line(points = {{ -80, -18},{0, -18},{0,50},{80,50}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{70, -80},{94, -100}}, lineColor = {160,160,164}, textString = "time"),Text(extent = {{ -21, -72},{25, -90}}, lineColor = {160,160,164}, textString = "startTime"),Line(points = {{0, -17},{0, -71}}, color = {192,192,192}, pattern = LinePattern.Dash),Text(extent = {{ -68, -36},{ -22, -54}}, lineColor = {160,160,164}, textString = "offset"),Line(points = {{ -13,50},{ -13, -17}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Polygon(points = {{2,50},{ -19,50},{2,50}}, lineColor = {192,192,192}, pattern = LinePattern.Dash),Polygon(points = {{ -13, -17},{ -16, -4},{ -10, -4},{ -13, -17},{ -13, -17}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -13,50},{ -16,37},{ -9,37},{ -13,50}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -68,26},{ -22,8}}, lineColor = {160,160,164}, textString = "I"),Polygon(points = {{ -13, -69},{ -16, -56},{ -10, -56},{ -13, -69},{ -13, -69}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -13, -18},{ -13, -70}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Polygon(points = {{ -13, -18},{ -16, -31},{ -9, -31},{ -13, -18}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -73,75},{ -53,95}}, lineColor = {192,192,192}, textString = "i")}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This current source uses the coresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end StepCurrent;
        model RampCurrent "Ramp current source"
          parameter SI.Current I(start = 1) "Height of ramp";
          parameter SI.Time duration(min = Modelica.Constants.small, start = 2) "Duration of ramp";
          extends Interfaces.CurrentSource(redeclare Modelica.Blocks.Sources.Ramp signalSource(final height = I, final duration = duration));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Line(points = {{ -80, -20},{ -20, -20},{50,50}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -40, -20},{ -42, -30},{ -37, -30},{ -40, -20}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -40, -20},{ -40, -70}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Polygon(points = {{ -40, -70},{ -43, -60},{ -38, -60},{ -40, -70},{ -40, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -80, -33},{ -41, -49}}, lineColor = {160,160,164}, textString = "offset"),Text(extent = {{ -40, -70},{6, -88}}, lineColor = {160,160,164}, textString = "startTime"),Text(extent = {{70, -80},{94, -100}}, lineColor = {160,160,164}, textString = "time"),Line(points = {{ -20, -20},{ -20, -70}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -19, -20},{50, -20}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Line(points = {{50,50},{101,50}}, color = {0,0,0}, thickness = 0.5),Line(points = {{50,50},{50, -20}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Polygon(points = {{50, -20},{42, -18},{42, -22},{50, -20}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -20, -20},{ -11, -18},{ -11, -22},{ -20, -20}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{50,50},{48,40},{53,40},{50,50}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{50, -20},{47, -10},{52, -10},{50, -20},{50, -20}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{53,25},{82,7}}, lineColor = {160,160,164}, textString = "I"),Text(extent = {{0, -17},{35, -37}}, lineColor = {160,160,164}, textString = "duration"),Text(extent = {{ -73,75},{ -53,95}}, lineColor = {192,192,192}, textString = "i")}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80, -60},{ -50, -60},{50,60},{80,60}}, color = {192,192,192})}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This current source uses the coresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end RampCurrent;
        model SineCurrent "Sine current source"
          parameter SI.Current I(start = 1) "Amplitude of sine wave";
          parameter SI.Angle phase = 0 "Phase of sine wave";
          parameter SI.Frequency freqHz(start = 1) "Frequency of sine wave";
          extends Interfaces.CurrentSource(redeclare Modelica.Blocks.Sources.Sine signalSource(amplitude = I, freqHz = freqHz, phase = phase));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -60.2,29.9},{ -53.8,46.5},{ -48.2,58.1},{ -43.3,65.2},{ -38.3,69.2},{ -33.4,69.8},{ -28.5,67},{ -23.6,61},{ -18.6,52},{ -13,38.6},{ -5.98,18.6},{8.79, -26.9},{15.1, -44},{20.8, -56.2},{25.7, -64},{30.6, -68.6},{35.5, -70},{40.5, -67.9},{45.4, -62.5},{50.3, -54.1},{55.9, -41.3},{63, -21.7},{70,0}}, color = {192,192,192})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80, -90},{ -80,84}}, color = {192,192,192}),Polygon(points = {{ -80,100},{ -86,84},{ -74,84},{ -80,100}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -99, -40},{85, -40}}, color = {192,192,192}),Polygon(points = {{101, -40},{85, -34},{85, -46},{101, -40}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -40,0},{ -31.6,34.2},{ -26.1,53.1},{ -21.3,66.4},{ -17.1,74.6},{ -12.9,79.1},{ -8.64,79.8},{ -4.42,76.6},{ -0.201,69.7},{4.02,59.4},{8.84,44.1},{14.9,21.2},{27.5, -30.8},{33, -50.2},{37.8, -64.2},{42, -73.1},{46.2, -78.4},{50.5, -80},{54.7, -77.6},{58.9, -71.5},{63.1, -61.9},{67.9, -47.2},{74, -24.8},{80,0}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -41, -2},{ -80, -2}}, color = {0,0,0}, thickness = 0.5),Text(extent = {{ -106, -11},{ -60, -29}}, lineColor = {160,160,164}, textString = "offset"),Line(points = {{ -41, -2},{ -41, -40}}, color = {192,192,192}, pattern = LinePattern.Dash),Text(extent = {{ -60, -43},{ -14, -61}}, lineColor = {160,160,164}, textString = "startTime"),Text(extent = {{84, -52},{108, -72}}, lineColor = {160,160,164}, textString = "time"),Line(points = {{ -9,79},{43,79}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -42, -1},{50,0}}, color = {192,192,192}, pattern = LinePattern.Dash),Polygon(points = {{33,80},{30,67},{37,67},{33,80}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{37,57},{83,39}}, lineColor = {160,160,164}, textString = "I"),Polygon(points = {{33,1},{30,14},{36,14},{33,1},{33,1}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{33,79},{33,0}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Text(extent = {{ -73,82},{ -53,102}}, lineColor = {192,192,192}, textString = "i")}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This current source uses the coresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end SineCurrent;
        model ExpSineCurrent "Exponentially damped sine current source"
          parameter Real I(start = 1) "Amplitude of sine wave";
          parameter SI.Frequency freqHz(start = 2) "Frequency of sine wave";
          parameter SI.Angle phase = 0 "Phase of sine wave";
          parameter SI.Damping damping(start = 1) "Damping coefficient of sine wave";
          extends Interfaces.CurrentSource(redeclare Modelica.Blocks.Sources.ExpSine signalSource(amplitude = I, freqHz = freqHz, phase = phase, damping = damping));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80, -14},{ -75.2,18.3},{ -72,36.3},{ -68.7,50.5},{ -65.5,60.2},{ -62.3,65.3},{ -59.1,65.6},{ -55.9,61.3},{ -52.7,53.1},{ -48.6,38.2},{ -43,11.8},{ -35, -27.9},{ -30.2, -47.7},{ -26.1, -59.9},{ -22.1, -67.2},{ -18.1, -69.3},{ -14.1, -66.5},{ -10.1, -59.3},{ -5.23, -46.1},{8.44, -0.3},{13.3,12.4},{18.1,20.8},{22.1,24},{26.9,23.2},{31.8,17.8},{38.2,5.4},{51.1, -24.5},{57.5, -35.2},{63.1, -39.9},{68.7, -39.9},{75.2, -34.5},{80, -27.8}}, color = {192,192,192})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80, -90},{ -80,84}}, color = {192,192,192}),Polygon(points = {{ -80,100},{ -86,84},{ -74,84},{ -80,100}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -99, -40},{85, -40}}, color = {192,192,192}),Polygon(points = {{101, -40},{85, -34},{85, -46},{101, -40}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -50,0},{ -46.1,28.2},{ -43.5,44},{ -40.9,56.4},{ -38.2,64.9},{ -35.6,69.4},{ -33,69.6},{ -30.4,65.9},{ -27.8,58.7},{ -24.5,45.7},{ -19.9,22.5},{ -13.4, -12.2},{ -9.5, -29.5},{ -6.23, -40.1},{ -2.96, -46.5},{0.302, -48.4},{3.57, -45.9},{6.83, -39.6},{10.8, -28.1},{21.9,12},{25.8,23.1},{29.7,30.5},{33,33.3},{36.9,32.5},{40.8,27.8},{46,16.9},{56.5, -9.2},{61.7, -18.6},{66.3, -22.7},{70.9, -22.6},{76.1, -18},{80, -12.1}}, color = {0,0,0}, thickness = 0.5),Text(extent = {{ -78,1},{ -55, -19}}, lineColor = {160,160,164}, textString = "offset"),Text(extent = {{ -72, -36},{ -26, -54}}, lineColor = {160,160,164}, textString = "startTime"),Text(extent = {{84, -52},{108, -72}}, lineColor = {160,160,164}, textString = "time"),Line(points = {{ -50,0},{18,0}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -50,0},{ -81,0}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -50,77},{ -50,0}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{18, -1},{18,76}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{18,73},{ -50,73}}, color = {192,192,192}),Text(extent = {{ -42,88},{9,74}}, lineColor = {160,160,164}, textString = "1/freqHz"),Polygon(points = {{ -49,73},{ -40,75},{ -40,71},{ -49,73}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{18,73},{10,75},{10,71},{18,73}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -50, -61},{ -19, -61}}, color = {192,192,192}),Polygon(points = {{ -18, -61},{ -26, -59},{ -26, -63},{ -18, -61}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -51, -63},{ -27, -75}}, lineColor = {160,160,164}, textString = "t"),Text(extent = {{ -82, -67},{108, -96}}, lineColor = {160,160,164}, textString = "I*exp(-damping*t)*sin(2*pi*freqHz*t+phase)"),Line(points = {{ -50,0},{ -50, -40}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -50, -54},{ -50, -72}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{18, -76},{ -1, -48}}, color = {192,192,192}, pattern = LinePattern.Dash),Text(extent = {{ -74,83},{ -54,103}}, lineColor = {192,192,192}, textString = "i")}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This current source uses the coresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end ExpSineCurrent;
        model ExponentialsCurrent "Rising and falling exponential current source"
          parameter Real iMax(start = 1) "Upper bound for rising edge";
          parameter SI.Time riseTime(min = 0, start = 0.5) "Rise time";
          parameter SI.Time riseTimeConst(min = Modelica.Constants.small, start = 0.1) "Rise time constant";
          parameter SI.Time fallTimeConst(min = Modelica.Constants.small, start = riseTimeConst) "Fall time constant";
          extends Interfaces.CurrentSource(redeclare Modelica.Blocks.Sources.Exponentials signalSource(outMax = iMax, riseTime = riseTime, riseTimeConst = riseTimeConst, fallTimeConst = fallTimeConst));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -76, -59},{ -73.2, -44.3},{ -70.3, -31.1},{ -66.8, -16.6},{ -63.3, -4},{ -59.7,6.92},{ -55.5,18.18},{ -51.3,27.7},{ -46.3,37},{ -40.6,45.5},{ -34.3,53.1},{ -27.2,59.6},{ -18.7,65.3},{ -8.1,70.2},{ -6,71},{ -3.88,58.5},{ -1.05,43.7},{1.78,30.8},{4.606,19.45},{8.14,7.3},{11.68, -3},{15.9, -13.2},{20.2, -21.6},{25.1, -29.5},{30.8, -36.4},{37.1, -42.3},{44.9, -47.5},{54.8, -51.8},{64, -54.4}}, color = {192,192,192})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -100, -70},{84, -70}}, color = {192,192,192}),Polygon(points = {{100, -70},{84, -64},{84, -76},{100, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -40, -30},{ -37.2, -15.3},{ -34.3, -2.1},{ -30.8,12.4},{ -27.3,25},{ -23.7,35.92},{ -19.5,47.18},{ -15.3,56.7},{ -10.3,66},{ -4.6,74.5},{1.7,82.1},{8.8,88.6},{17.3,94.3},{27.9,99.2},{30,100},{32.12,87.5},{34.95,72.7},{37.78,59.8},{40.606,48.45},{44.14,36.3},{47.68,26},{51.9,15.8},{56.2,7.4},{61.1, -0.5},{66.8, -7.4},{73.1, -13.3},{80.9, -18.5},{90.8, -22.8},{100, -25.4}}, color = {0,0,0}, thickness = 0.5),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Polygon(points = {{ -40, -70},{ -43, -60},{ -38, -60},{ -40, -70},{ -40, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -40, -29},{ -40, -60}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Polygon(points = {{ -40, -30},{ -42, -40},{ -37, -40},{ -40, -30}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -39, -30},{ -80, -30}}, color = {0,0,0}, thickness = 0.5),Text(extent = {{ -59, -71},{ -13, -89}}, lineColor = {160,160,164}, textString = "startTime"),Polygon(points = {{ -41, -30},{ -32, -28},{ -32, -32},{ -41, -30}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -40, -30},{29, -30}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Polygon(points = {{29, -30},{21, -28},{21, -32},{29, -30}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -26, -12},{19, -28}}, lineColor = {160,160,164}, textString = "riseTime"),Line(points = {{ -39, -30},{ -80, -30}}, color = {0,0,0}, thickness = 0.5),Line(points = {{30,100},{30, -34}}, color = {192,192,192}, pattern = LinePattern.Dash),Text(extent = {{ -73,75},{ -53,95}}, lineColor = {192,192,192}, textString = "i")}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This current source uses the coresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end ExponentialsCurrent;
        model PulseCurrent "Pulse current source"
          parameter SI.Current I(start = 1) "Amplitude of pulse";
          parameter Real width(final min = Modelica.Constants.small, final max = 100, start = 50) "Width of pulse in % of period";
          parameter SI.Time period(final min = Modelica.Constants.small, start = 1) "Time for one period";
          extends Interfaces.CurrentSource(redeclare Modelica.Blocks.Sources.Pulse signalSource(amplitude = I, width = width, period = period));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70, -67},{ -40, -67},{ -40,73},{0,73},{0, -67},{40, -67},{40,73},{80,73}}, color = {192,192,192})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -34,1},{ -37, -12},{ -30, -12},{ -34,1}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -34, -1},{ -34, -70}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Polygon(points = {{ -33, -70},{ -36, -57},{ -30, -57},{ -33, -70},{ -33, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -78, -24},{ -35, -36}}, lineColor = {160,160,164}, textString = "offset"),Text(extent = {{ -31, -69},{15, -87}}, lineColor = {160,160,164}, textString = "startTime"),Text(extent = {{70, -80},{94, -100}}, lineColor = {160,160,164}, textString = "time"),Line(points = {{ -10,0},{ -10, -70}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -80,0},{ -20,0},{ -21,50},{21,50},{20, -1},{41,0},{40,50},{80,51}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -19,88},{ -19,49}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{21,74},{21,50}}, color = {160,160,164}, pattern = LinePattern.Dash),Line(points = {{41,88},{41,50}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -19,83},{42,83}}, color = {192,192,192}),Line(points = {{ -19,69},{21,69}}, color = {192,192,192}),Text(extent = {{ -9,97},{37,85}}, lineColor = {160,160,164}, textString = "period"),Text(extent = {{ -18,81},{21,69}}, lineColor = {160,160,164}, textString = "width"),Line(points = {{ -52,50},{ -19,50}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -34,50},{ -34,1}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Text(extent = {{ -78,34},{ -37,20}}, lineColor = {160,160,164}, textString = "I"),Polygon(points = {{ -34,49},{ -37,36},{ -30,36},{ -34,49}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -34,1},{ -37,14},{ -31,14},{ -34,1},{ -34,1}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{80,51},{80,0},{90, -1}}, color = {0,0,0}, thickness = 0.5),Polygon(points = {{ -19,69},{ -10,71},{ -10,67},{ -19,69}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{21,69},{13,71},{13,67},{21,69}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -19,83},{ -10,85},{ -10,81},{ -19,83}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{41,83},{33,85},{33,81},{41,83}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -73,75},{ -53,95}}, lineColor = {192,192,192}, textString = "i")}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This current source uses the coresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end PulseCurrent;
        model SawToothCurrent "Saw tooth current source"
          parameter SI.Current I(start = 1) "Amplitude of saw tooth";
          parameter SI.Time period(start = 1) "Time for one period";
          extends Interfaces.CurrentSource(redeclare Modelica.Blocks.Sources.SawTooth signalSource(amplitude = I, period = period));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70, -71},{ -50, -71},{10,70},{10, -71},{70,70},{70, -71}}, color = {192,192,192})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -34, -19},{ -37, -32},{ -30, -32},{ -34, -19}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -34, -20},{ -34, -70}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Polygon(points = {{ -34, -70},{ -37, -57},{ -31, -57},{ -34, -70},{ -34, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -78, -24},{ -35, -36}}, lineColor = {160,160,164}, textString = "offset"),Text(extent = {{ -31, -69},{15, -87}}, lineColor = {160,160,164}, textString = "startTime"),Text(extent = {{70, -80},{94, -100}}, lineColor = {160,160,164}, textString = "time"),Line(points = {{ -10, -20},{ -10, -70}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -10,88},{ -10, -20}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{30,88},{30,59}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -10,83},{30,83}}, color = {192,192,192}),Text(extent = {{ -12,97},{34,85}}, lineColor = {160,160,164}, textString = "period"),Line(points = {{ -44,60},{30,60}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -34,47},{ -34, -7}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Text(extent = {{ -78,34},{ -37,20}}, lineColor = {160,160,164}, textString = "I"),Polygon(points = {{ -34,60},{ -37,47},{ -30,47},{ -34,60}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -34, -20},{ -37, -7},{ -31, -7},{ -34, -20},{ -34, -20}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -10,83},{ -1,85},{ -1,81},{ -10,83}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{30,83},{22,85},{22,81},{30,83}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -20},{ -10, -20},{30,60},{30, -20},{72,60},{72, -20}}, color = {0,0,0}, thickness = 0.5),Text(extent = {{ -73,75},{ -53,95}}, lineColor = {192,192,192}, textString = "i")}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This current source uses the coresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end SawToothCurrent;
        model TrapezoidCurrent "Trapezoidal current source"
          parameter SI.Current I(start = 1) "Amplitude of trapezoid";
          parameter SI.Time rising(final min = 0, start = 0) "Rising duration of trapezoid";
          parameter SI.Time width(final min = 0, start = 0.5) "Width duration of trapezoid";
          parameter SI.Time falling(final min = 0, start = 0) "Falling duration of trapezoid";
          parameter SI.Time period(final min = Modelica.Constants.small, start = 1) "Time for one period";
          parameter Integer nperiod(start =  -1) "Number of periods (< 0 means infinite number of periods)";
          extends Interfaces.CurrentSource(redeclare Modelica.Blocks.Sources.Trapezoid signalSource(amplitude = I, rising = rising, width = width, falling = falling, period = period, nperiod = nperiod));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -81, -66},{ -60, -66},{ -30,74},{1,74},{30, -66},{51, -66},{80,74}}, color = {192,192,192})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -45, -30},{ -47, -41},{ -43, -41},{ -45, -30}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -45, -31},{ -45, -70}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Polygon(points = {{ -45, -70},{ -47, -60},{ -43, -60},{ -45, -70},{ -45, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -86, -43},{ -43, -55}}, lineColor = {160,160,164}, textString = "offset"),Text(extent = {{ -47, -69},{ -1, -87}}, lineColor = {160,160,164}, textString = "startTime"),Text(extent = {{70, -80},{94, -100}}, lineColor = {160,160,164}, textString = "time"),Line(points = {{ -29,79},{ -30, -70}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -10,59},{ -10,40}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{20,59},{20,39}}, color = {160,160,164}, pattern = LinePattern.Dash),Line(points = {{40,59},{40, -30}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -20,75},{60,75}}, color = {192,192,192}),Line(points = {{ -29,56},{40,56}}, color = {192,192,192}),Text(extent = {{ -9,88},{37,76}}, lineColor = {160,160,164}, textString = "period"),Text(extent = {{ -8,70},{21,60}}, lineColor = {160,160,164}, textString = "width"),Line(points = {{ -42,40},{ -10,40}}, color = {192,192,192}, pattern = LinePattern.Dash),Line(points = {{ -39,40},{ -39, -19}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Text(extent = {{ -77,14},{ -40,0}}, lineColor = {160,160,164}, textString = "I"),Polygon(points = {{ -29,56},{ -22,58},{ -22,54},{ -29,56}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -10,56},{ -17,58},{ -17,54},{ -10,56}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -29,75},{ -20,77},{ -20,73},{ -29,75}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{60,75},{52,77},{52,73},{60,75}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -30},{ -30, -30},{ -9,46},{21,46},{40, -30},{60, -30},{80,46},{100,46}}, color = {0,0,0}, thickness = 0.5),Polygon(points = {{ -39,40},{ -41,29},{ -37,29},{ -39,40}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -39, -29},{ -41, -19},{ -37, -19},{ -39, -29},{ -39, -29}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{60,80},{60, -30}}, color = {192,192,192}, pattern = LinePattern.Dash),Polygon(points = {{39,56},{32,58},{32,54},{39,56}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{20,56},{27,58},{27,54},{20,56}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{20,56},{13,58},{13,54},{20,56}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -12,56},{ -5,58},{ -5,54},{ -12,56}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -34,70},{ -5,60}}, lineColor = {160,160,164}, textString = "rising"),Text(extent = {{16,70},{45,60}}, lineColor = {160,160,164}, textString = "falling"),Text(extent = {{ -73,75},{ -53,95}}, lineColor = {192,192,192}, textString = "i")}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This current source uses the coresponding signal source of the Modelica.Blocks.Sources package. Care for the meaning of the parameters in the Blocks package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>"));
        end TrapezoidCurrent;
        model TableCurrent "Current source by linear interpolation in a table"
          parameter Real table[:,:] = [0,0;1,1;2,4] "Table matrix (time = first column, current = second column)";
          extends Interfaces.CurrentSource(redeclare Modelica.Blocks.Sources.TimeTable signalSource(table = table));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -66, -36},{ -66,84},{34,84},{34, -36},{ -66, -36},{ -66, -6},{34, -6},{34,24},{ -66,24},{ -66,54},{34,54},{34,84},{ -16,84},{ -16, -37}}, color = {192,192,192})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -20,90},{30, -30}}, lineColor = {255,255,255}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -20, -30},{ -20,90},{80,90},{80, -30},{ -20, -30},{ -20,0},{80,0},{80,30},{ -20,30},{ -20,60},{80,60},{80,90},{30,90},{30, -31}}, color = {0,0,0}),Text(extent = {{ -77, -42},{ -38, -58}}, lineColor = {160,160,164}, textString = "offset"),Polygon(points = {{ -31, -30},{ -33, -40},{ -28, -40},{ -31, -30}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -30, -70},{ -33, -60},{ -28, -60},{ -30, -70},{ -30, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -31, -31},{ -31, -70}}, color = {192,192,192}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Line(points = {{ -20, -20},{ -20, -70}}, color = {192,192,192}, pattern = LinePattern.Dash),Text(extent = {{ -38, -70},{8, -88}}, lineColor = {160,160,164}, textString = "startTime"),Line(points = {{ -20, -30},{ -80, -30}}, color = {192,192,192}, pattern = LinePattern.Dash),Text(extent = {{66, -81},{91, -93}}, lineColor = {160,160,164}, textString = "time"),Text(extent = {{ -15,83},{24,68}}, lineColor = {0,0,0}, textString = "time"),Text(extent = {{33,83},{76,67}}, lineColor = {0,0,0}, textString = "v"),Text(extent = {{ -73,75},{ -53,95}}, lineColor = {192,192,192}, textString = "i")}), Documentation(info = "<html>
<p>This current source uses the coresponding signal source of the Modelica.Blocks.Sources package. Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
<p><br/>This block generates a current source by <b>linear interpolation</b> in a table. The time points and current values are stored in a matrix <b>table[i,j]</b>, where the first column table[:,1] contains the time points and the second column contains the current to be interpolated. The table interpolation has the following proporties: </p>
<p><ul>
<li>The time points need to be <b>monotonically increasing</b>. </li>
<li><b>Discontinuities</b> are allowed, by providing the same time point twice in the table. </li>
<li>Values <b>outside</b> of the table range, are computed by <b>extrapolation</b> through the last or first two points of the table.</li>
<li>If the table has only <b>one row</b>, no interpolation is performed and the current value is just returned independantly of the actual time instant, i.e., this is a constant current source.</li>
<li>Via parameters <b>startTime</b> and <b>offset</b> the curve defined by the table can be shifted both in time and in the current. </li>
<li>The table is implemented in a numerically sound way by generating <b>time events</b> at interval boundaries, in order to not integrate over a discontinuous or not differentiable points. </li>
</ul></p>
<p>Example: </p>
<pre>   table = [0  0
            1  0
            1  1
            2  4
            3  9
            4 16]
If, e.g., time = 1.0, the current i =  0.0 (before event), 1.0 (after event)
    e.g., time = 1.5, the current i =  2.5,
    e.g., time = 2.0, the current i =  4.0,
    e.g., time = 5.0, the current i = 23.0 (i.e. extrapolation). </pre>
<p><br/> Furthermore, an offset parameter is introduced, which is added to the value calculated by the blocks source. The startTime parameter allows to shift the blocks source behavior on the time axis.</p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"));
        end TableCurrent;
      end Sources;
      package Sensors "Potential, voltage, current, and power sensors"
        extends Modelica.Icons.Library;
        annotation(Documentation(info = "<html>
<p>This package contains potential, voltage, and current sensors. The sensors can be used to convert voltages or currents into real signal values o be connected to components of the Blocks package. The sensors are designed in such a way that they do not influence the electrical behavior.</p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden<br>
<p>
<dt>
<b>Copyright:</b>
<dd>
Copyright &copy; 1998-2006, Modelica Association and Fraunhofer-Gesellschaft.<br>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package
Modelica in file \"Modelica/package.mo\".</i><br>
<p>
</dl>
</html>"));
        model PotentialSensor "Sensor to measure the potential"
          extends Modelica.Icons.RotationalSensor;
          Interfaces.PositivePin p "pin to be measured" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput phi "Absolute voltage potential as output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -29, -11},{30, -70}}, lineColor = {0,0,0}, textString = "V"),Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{100,0},{70,0}}, color = {0,0,255}),Text(extent = {{ -150,80},{150,120}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -96,0}}, color = {0,0,0}),Line(points = {{100,0},{70,0}}, color = {0,0,255})}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>The potential sensor converts the voltage of a node (with respect to the ground node) into a real valued signal. It does not influence the current sum at the node which voltage is measured, therefore, the electrical behavior is not influenced by the sensor.</p>
</html>"));

        equation 
          p.i = 0;
          phi = p.v;
        end PotentialSensor;
        model VoltageSensor "Sensor to measure the voltage between two pins"
          extends Modelica.Icons.RotationalSensor;
          Interfaces.PositivePin p "positive pin" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Interfaces.NegativePin n "negative pin" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput v "Voltage between pin p and n (= p.v - n.v) as output signal" annotation(Placement(transformation(origin = {0, -100}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -29, -11},{30, -70}}, lineColor = {0,0,0}, textString = "V"),Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{70,0},{90,0}}, color = {0,0,0}),Line(points = {{0, -90},{0, -70}}, color = {0,0,255}),Text(extent = {{ -150,80},{150,120}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -96,0}}, color = {0,0,0}),Line(points = {{70,0},{96,0}}, color = {0,0,0}),Line(points = {{0, -90},{0, -70}}, color = {0,0,255})}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>The voltage  sensor converts the voltage between the two connectors into a real valued signal. It does not influence the current sum at the nodes in between the voltage is measured, therefore, the electrical behavior is not influenced by the sensor.</p>
</html>"));

        equation 
          p.i = 0;
          n.i = 0;
          v = p.v - n.v;
        end VoltageSensor;
        model CurrentSensor "Sensor to measure the current in a branch"
          extends Modelica.Icons.RotationalSensor;
          Interfaces.PositivePin p "positive pin" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Interfaces.NegativePin n "negative pin" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput i "current in the branch from p to n as output signal" annotation(Placement(transformation(origin = {0, -100}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -29, -11},{30, -70}}, lineColor = {0,0,0}, textString = "A"),Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Text(extent = {{ -150,80},{150,120}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{70,0},{90,0}}, color = {0,0,0}),Line(points = {{0, -90},{0, -70}}, color = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -153,79},{147,119}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -70,0},{ -96,0}}, color = {0,0,0}),Line(points = {{70,0},{96,0}}, color = {0,0,0}),Line(points = {{0, -90},{0, -70}}, color = {0,0,255})}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>The current  sensor converts the current flowing between the two connectors into a real valued signal. The two connectors are in the sensor connected like a short cut. The sensor has to be placed within an electrical connection in series.  It does not influence the current sum at the connected nodes. Therefore, the electrical behavior is not influenced by the sensor.</p>
</html>"));

        equation 
          p.v = n.v;
          p.i = i;
          n.i =  -i;
        end CurrentSensor;
        model PowerSensor "Sensor to measure the power"
          Modelica.Electrical.Analog.Interfaces.PositivePin pc "Positive pin, current path" annotation(Placement(transformation(extent = {{ -90, -10},{ -110,10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin nc "Negative pin, current path" annotation(Placement(transformation(extent = {{110, -10},{90,10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.PositivePin pv "Positive pin, voltage path" annotation(Placement(transformation(extent = {{ -10,110},{10,90}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin nv "Negative pin, voltage path" annotation(Placement(transformation(extent = {{10, -110},{ -10, -90}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput power annotation(Placement(transformation(origin = { -80, -110}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Modelica.Electrical.Analog.Sensors.VoltageSensor voltageSensor annotation(Placement(transformation(origin = {0, -30}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Modelica.Electrical.Analog.Sensors.CurrentSensor currentSensor annotation(Placement(transformation(extent = {{ -50, -10},{ -30,10}}, rotation = 0)));
          Modelica.Blocks.Math.Product product annotation(Placement(transformation(origin = { -30, -50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Ellipse(extent = {{ -70,70},{70, -70}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,100},{0,70}}, color = {0,0,255}),Line(points = {{0, -70},{0, -100}}, color = {0,0,255}),Line(points = {{ -80, -100},{ -80,0}}, color = {0,0,255}),Line(points = {{ -100,0},{100,0}}, color = {0,0,255}),Text(extent = {{150,120},{ -150,160}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{0,70},{0,40}}, color = {0,0,0}),Line(points = {{22.9,32.8},{40.2,57.3}}, color = {0,0,0}),Line(points = {{ -22.9,32.8},{ -40.2,57.3}}, color = {0,0,0}),Line(points = {{37.6,13.7},{65.8,23.9}}, color = {0,0,0}),Line(points = {{ -37.6,13.7},{ -65.8,23.9}}, color = {0,0,0}),Line(points = {{0,0},{9.02,28.6}}, color = {0,0,0}),Polygon(points = {{ -0.48,31.6},{18,26},{18,57.2},{ -0.48,31.6}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -5,5},{5, -5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -29, -11},{30, -70}}, lineColor = {0,0,0}, textString = "P")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<pre>This power sensor measures instantaneous electrical power of a singlephase system and has a separated voltage and current path. The pins of the voltage path are pv and nv, the pins of the current path are pc and nc. The internal resistance of the current path is zero, the internal resistance of the voltage path is infinite.</pre>
</html>", revisions = "<html>
<ul>
<li><i> January 12, 2006   </i>
       by Anton Haumer<br> implemented<br>
       </li>
</ul>
</html>"));

        equation 
          connect(pv,voltageSensor.p) annotation(Line(points = {{0,100},{0, -20},{6.12323e-16, -20}}, color = {0,0,255}));
          connect(voltageSensor.n,nv) annotation(Line(points = {{ -6.12323e-16, -40},{ -6.12323e-16, -63},{0, -63},{0, -100}}, color = {0,0,255}));
          connect(pc,currentSensor.p) annotation(Line(points = {{ -100,0},{ -50,0}}, color = {0,0,255}));
          connect(currentSensor.n,nc) annotation(Line(points = {{ -30,0},{100,0}}, color = {0,0,255}));
          connect(currentSensor.i,product.u2) annotation(Line(points = {{ -40, -10},{ -40, -30},{ -36, -30},{ -36, -38}}, color = {0,0,127}));
          connect(voltageSensor.v,product.u1) annotation(Line(points = {{10, -30},{ -24, -30},{ -24, -38}}, color = {0,0,127}));
          connect(product.y,power) annotation(Line(points = {{ -30, -61},{ -30, -80},{ -80, -80},{ -80, -110}}, color = {0,0,127}));
        end PowerSensor;
      end Sensors;
      package Semiconductors "Semiconductor devices such as diode, MOS and bipolar transistor"
        extends Modelica.Icons.Library;
        import Modelica.SIunits;
        annotation(Documentation(info = "<html>
<p>This package contains semiconductor devices:  </p>
<p><ul>
<li>diode</li>
<li>MOS transistors</li>
<li>bipolar transistors</li>
</ul></p>
<p>All semiconductor devices contain a conditional heat port, which is not active by default. If it is active the loss power is calculated to be used in a thermal net. The heating variants of the semiconductor devices are provided to use the thermal pot temperature in the electric calculation. That means that for a true thermal electric interaction the heating device models have to be used. </p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden<br>
<p>
<dt>
</dl>
</html>"));
        model Diode "Simple diode"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter SIunits.Current Ids = 1e-06 "Saturation current";
          parameter SIunits.Voltage Vt = 0.04 "Voltage equivalent of temperature (kT/qn)";
          parameter Real Maxexp(final min = Modelica.Constants.small) = 15 "Max. exponent for linear continuation";
          parameter SIunits.Resistance R = 100000000.0 "Parallel ohmic resistance";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = 293.15);
          annotation(Documentation(info = "<html>
<p>The simple diode is a one port. It consists of the diode itself and an parallel ohmic resistance <i>R</i>. The diode formula is:  </p>
<pre>                v/vt
  i  =  ids ( e      - 1).</pre>
<p><br/>If the exponent <i>v/vt</i> reaches the limit <i>maxex</i>, the diode characterisic is linearly continued to avoid overflow. </p><p><br/><b>Please note:</b> In case of useHeatPort=true the temperature dependence of the electrical behavior is <b>not </b>modelled yet. The parameters are not temperature dependent. </p>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i> November 15, 2005   </i>
       by Christoph Clauss<br> smooth function added<br>
       </li>
<li><i> 1998   </i>
       by Christoph Clauss<br> implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{30,0},{ -30,40},{ -30, -40},{30,0}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{40,0}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255}),Line(points = {{30,40},{30, -40}}, color = {0,0,255}),Text(extent = {{ -150, -49},{149, -77}}, lineColor = {0,0,0}, textString = "Vt=%Vt"),Text(extent = {{ -154,100},{146,60}}, textString = "%name", lineColor = {0,0,255}),Line(visible = useHeatPort, points = {{0, -100},{0, -20}}, color = {127,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{30,0},{ -30,40},{ -30, -40},{30,0}}, lineColor = {0,0,0}, fillColor = {255,0,0}, fillPattern = FillPattern.None),Line(points = {{ -96,0},{40,0}}, color = {0,0,255}),Line(points = {{40,0},{96,0}}, color = {0,0,255}),Line(points = {{30,40},{30, -40}}, color = {0,0,255})}));

        equation 
          i = smooth(1, if v / Vt > Maxexp then Ids * (exp(Maxexp) * (1 + v / Vt - Maxexp) - 1) + v / R else Ids * (exp(v / Vt) - 1) + v / R);
          LossPower = v * i;
        end Diode;
        model ZDiode "Zener diode with 3 working areas"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter Modelica.SIunits.Current Ids = 1e-06 "Saturation current";
          parameter Modelica.SIunits.Voltage Vt = 0.04 "Voltage equivalent of temperature (kT/qn)";
          parameter Real Maxexp(final min = Modelica.Constants.small) = 30 "Max. exponent for linear continuation";
          parameter Modelica.SIunits.Resistance R = 100000000.0 "Parallel ohmic resistance";
          parameter Modelica.SIunits.Voltage Bv = 5.1 "Breakthrough voltage = Zener- or Z-voltage";
          parameter Modelica.SIunits.Current Ibv = 0.7 "Breakthrough knee current";
          parameter Real Nbv = 0.74 "Breakthrough emission coefficient";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = 293.15);
          Real maxexp = exp(Maxexp);
          annotation(__Dymola_structurallyIncomplete = true, Documentation(info = "<html>
<p>The simple zener diode is a one port. It consists of the diode itself and an parallel ohmic resistance <i>R</i>. The diode formula is: </p>
<pre>                v/Vt                -(v+Bv)/(Nbv*Vt)
  i  =  Ids ( e      - 1) - Ibv ( e                  ).</pre>
<p>If the exponent in one of the two branches reaches the limit <i>Maxexp</i>, the diode characterisic is linearly continued to avoid overflow. </p>
<p><br/>The zener diode model permits (in contrast to the simple diode model) current in reverse direction if the breakdown voltage Bv (also known zener knee voltage) is exceeded. </p>
<p>The thermal power is calculated by <i>i*v</i>. </p><p><b>Please note:</b> In case of useHeatPort=true the temperature dependence of the electrical behavior is <b>not </b>modelled yet. The parameters are not temperature dependent. </p>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>April 5, 2004   </i>
       by Christoph Clauss<br> implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{30,0},{ -30,40},{ -30, -40},{30,0}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{40,0}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255}),Line(points = {{30,40},{30, -40}}, color = {0,0,255}),Text(extent = {{ -152,114},{148,74}}, textString = "%name", lineColor = {0,0,255}),Line(visible = useHeatPort, points = {{0, -101},{0, -20}}, color = {127,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{30,0},{ -30,40},{ -30, -40},{30,0}}, lineColor = {0,0,0}, fillColor = {255,0,0}, fillPattern = FillPattern.None),Line(points = {{ -99,0},{39,0}}, color = {0,0,255}),Line(points = {{40,0},{96,0}}, color = {0,0,255}),Line(points = {{30,40},{30, -40}}, color = {0,0,255})}));

        equation 
          i = smooth(1, if v > Maxexp * Vt then Ids * (exp(Maxexp) * (1 + v / Vt - Maxexp) - 1) + v / R else if v + Bv <  -Maxexp * Nbv * Vt then  -Ids - Ibv * exp(Maxexp) * (1 - (v + Bv) / (Nbv * Vt) - Maxexp) + v / R else Ids * (exp(v / Vt) - 1) - Ibv * exp( -(v + Bv) / (Nbv * Vt)) + v / R);
          LossPower = v * i;
        end ZDiode;
        model PMOS "Simple MOS Transistor"
          Interfaces.Pin D "Drain" annotation(Placement(transformation(extent = {{90,40},{110,60}}, rotation = 0)));
          Interfaces.Pin G "Gate" annotation(Placement(transformation(extent = {{ -90, -40},{ -110, -60}}, rotation = 0)));
          Interfaces.Pin S "Source" annotation(Placement(transformation(extent = {{90, -40},{110, -60}}, rotation = 0)));
          Interfaces.Pin B "Bulk" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          parameter SIunits.Length W = 2e-05 "Width";
          parameter SIunits.Length L = 6e-06 "Length";
          parameter SIunits.Transconductance Beta = 1.05e-05 "Transconductance parameter";
          parameter SIunits.Voltage Vt =  -1.0 "Zero bias threshold voltage";
          parameter Real K2 = 0.41 "Bulk threshold parameter";
          parameter Real K5 = 0.839 "Reduction of pinch-off region";
          parameter SIunits.Length dW =  -2.5e-06 "Narrowing of channel";
          parameter SIunits.Length dL =  -2.1e-06 "Shortening of channel";
          parameter SIunits.Resistance RDS = 10000000.0 "Drain-Source-Resistance";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = 293.15);
        protected 
          Real v;
          Real uds;
          Real ubs;
          Real ugst;
          Real ud;
          Real us;
          Real id;
          Real gds;
          annotation(Documentation(info = "
<HTML>
<P>
The PMOS model is a simple model of a p-channel metal-oxide semiconductor
FET. It differs slightly from the device used in the SPICE simulator.
For more details please care for H. Spiro.
</P>
<P>
The model does not consider capacitances. A high drain-source resistance RDS
is included to avoid numerical difficulties.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled yet. The parameters are not temperature dependent.
</P>
<DL>
<DT><b>References:</b>
<DD>Spiro, H.: Simulation integrierter Schaltungen. R. Oldenbourg Verlag
  Muenchen Wien 1990.
</DL>
<P>
Some typical parameter sets are:
</P>
<PRE>
  W       L      Beta        Vt       K2       K5       DW         DL
  m       m      A/V^2       V        -        -        m          m
  50.e-6  8.e-6  .0085e-3   -.15     .41      .839    -3.8e-6    -4.0e-6
  20.e-6  6.e-6  .0105e-3  -1.0      .41      .839    -2.5e-6    -2.1e-6
  30.e-6  5.e-6  .0059e-3   -.3      .98     1.01      0         -3.9e-6
  30.e-6  5.e-6  .0152e-3   -.69     .104    1.1       -.8e-6     -.4e-6
  30.e-6  5.e-6  .0163e-3   -.69     .104    1.1       -.8e-6     -.4e-6
  30.e-6  5.e-6  .0182e-3   -.69     .086    1.06      -.1e-6     -.6e-6
  20.e-6  6.e-6  .0074e-3  -1.       .4       .59      0          0
</PRE>

</HTML>
", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>December 7, 2005   </i>
       by Christoph Clauss<br>
       error in RDS calculation deleted</li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -90, -50},{ -10, -50}}, color = {0,0,255}),Line(points = {{ -10, -50},{ -10,50}}, color = {0,0,255}),Line(points = {{10,70},{10,29}}, color = {0,0,255}),Line(points = {{10,20},{10, -21}}, color = {0,0,255}),Line(points = {{10, -30},{10, -71}}, color = {0,0,255}),Line(points = {{10,50},{91,50}}, color = {0,0,255}),Line(points = {{10,0},{90,0}}, color = {0,0,255}),Line(points = {{10, -50},{90, -50}}, color = {0,0,255}),Polygon(points = {{60,0},{40,5},{40, -5},{60,0}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -149,117},{151,77}}, textString = "%name", lineColor = {0,0,255}),Line(visible = useHeatPort, points = {{0, -90},{0,0}}, color = {127,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -96, -50},{ -10, -50}}, color = {0,0,255}),Line(points = {{ -10, -50},{ -10,50}}, color = {0,0,255}),Line(points = {{10,70},{10,29}}, color = {0,0,255}),Line(points = {{10,20},{10, -21}}, color = {0,0,255}),Line(points = {{10, -30},{10, -71}}, color = {0,0,255}),Line(points = {{10,50},{96,50}}, color = {0,0,255}),Line(points = {{10,0},{96,0}}, color = {0,0,255}),Line(points = {{10, -50},{96, -50}}, color = {0,0,255}),Polygon(points = {{60,0},{40,5},{40, -5},{60,0}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255})}));

        equation 
          assert(L + dL > 0, "PMOS: Effective length must be positive");
          assert(W + dW > 0, "PMOS: Effective width  must be positive");
          gds = if RDS < 1e-20 and RDS >  -1e-20 then 1e+20 else 1 / RDS;
          v = (Beta * (W + dW)) / (L + dL);
          ud = smooth(0, if D.v > S.v then S.v else D.v);
          us = smooth(0, if D.v > S.v then D.v else S.v);
          uds = ud - us;
          ubs = smooth(0, if B.v < us then 0 else B.v - us);
          ugst = (G.v - us - Vt + K2 * ubs) * K5;
          id = smooth(0, if ugst >= 0 then uds * gds else if ugst < uds then  -v * uds * (ugst - uds / 2) + uds * gds else  -(v * ugst * ugst) / 2 + uds * gds);
          G.i = 0;
          D.i = smooth(0, if D.v > S.v then  -id else id);
          S.i = smooth(0, if D.v > S.v then id else  -id);
          B.i = 0;
          LossPower = D.i * (D.v - S.v);
        end PMOS;
        model NMOS "Simple MOS Transistor"
          Interfaces.Pin D "Drain" annotation(Placement(transformation(extent = {{90,40},{110,60}}, rotation = 0)));
          Interfaces.Pin G "Gate" annotation(Placement(transformation(extent = {{ -90, -40},{ -110, -60}}, rotation = 0)));
          Interfaces.Pin S "Source" annotation(Placement(transformation(extent = {{90, -40},{110, -60}}, rotation = 0)));
          Interfaces.Pin B "Bulk" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          parameter SIunits.Length W = 2e-05 "Width";
          parameter SIunits.Length L = 6e-06 "Length";
          parameter SIunits.Transconductance Beta = 4.1e-05 "Transconductance parameter";
          parameter SIunits.Voltage Vt = 0.8 "Zero bias threshold voltage";
          parameter Real K2 = 1.144 "Bulk threshold parameter";
          parameter Real K5 = 0.7311 "Reduction of pinch-off region";
          parameter SIunits.Length dW =  -2.5e-06 "narrowing of channel";
          parameter SIunits.Length dL =  -1.5e-06 "shortening of channel";
          parameter SIunits.Resistance RDS = 10000000.0 "Drain-Source-Resistance";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = 293.15);
        protected 
          Real v;
          Real uds;
          Real ubs;
          Real ugst;
          Real ud;
          Real us;
          Real id;
          Real gds;
          annotation(Documentation(info = "
<HTML>
<P>
The NMos model is a simple model of a n-channel metal-oxide semiconductor
FET. It differs slightly from the device used in the SPICE simulator.
For more details please care for H. Spiro.
</P>
<P>
The model does not consider capacitances. A high drain-source resistance RDS
is included to avoid numerical difficulties.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled yet. The parameters are not temperature dependent.
</P>
<P>
<PRE>
  W       L      Beta        Vt       K2      K5       DW         DL
  m       m      A/V^2       V        -       -        m          m
  12.e-6  4.e-6  .062e-3   -4.5      .24     .61     -1.2e-6     -.9e-6      depletion
  60.e-6  3.e-6  .048e-3     .1      .08     .68     -1.2e-6     -.9e-6      enhancement
  12.e-6  4.e-6  .0625e-3   -.8      .21     .78     -1.2e-6     -.9e-6      zero
  50.e-6  8.e-6  .0299e-3    .24    1.144    .7311   -5.4e-6    -4.e-6
  20.e-6  6.e-6  .041e-3     .8     1.144    .7311   -2.5e-6    -1.5e-6
  30.e-6  9.e-6  .025e-3   -4.       .861    .878    -3.4e-6    -1.74e-6
  30.e-6  5.e-6  .031e-3     .6     1.5      .72      0         -3.9e-6
  50.e-6  6.e-6  .0414e-3  -3.8      .34     .8      -1.6e-6    -2.e-6       depletion
  50.e-6  5.e-6  .03e-3      .37     .23     .86     -1.6e-6    -2.e-6       enhancement
  50.e-6  6.e-6  .038e-3    -.9      .23     .707    -1.6e-6    -2.e-6       zero
  20.e-6  4.e-6  .06776e-3   .5409   .065    .71      -.8e-6     -.2e-6
  20.e-6  4.e-6  .06505e-3   .6209   .065    .71      -.8e-6     -.2e-6
  20.e-6  4.e-6  .05365e-3   .6909   .03     .8       -.3e-6     -.2e-6
  20.e-6  4.e-6  .05365e-3   .4909   .03     .8       -.3e-6     -.2e-6
  12.e-6  4.e-6  .023e-3   -4.5      .29     .6       0          0           depletion
  60.e-6  3.e-6  .022e-3     .1      .11     .65      0          0           enhancement
  12.e-6  4.e-6  .038e-3    -.8      .33     .6       0          0           zero
  20.e-6  6.e-6  .022e-3     .8     1        .66      0          0
</PRE>

<P>
<DL>
<DT><b>References:</b>
<DD>Spiro, H.: Simulation integrierter Schaltungen. R. Oldenbourg Verlag
Muenchen Wien 1990.
</DL>
</HTML>
", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>December 7, 2005   </i>
       by Christoph Clauss<br>
       error in RDS calculation deleted</li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -90, -50},{ -10, -50}}, color = {0,0,255}),Line(points = {{ -10, -50},{ -10,50}}, color = {0,0,255}),Line(points = {{10,70},{10,29}}, color = {0,0,255}),Line(points = {{10,20},{10, -21}}, color = {0,0,255}),Line(points = {{10, -30},{10, -71}}, color = {0,0,255}),Line(points = {{10,50},{91,50}}, color = {0,0,255}),Line(points = {{10,0},{90,0}}, color = {0,0,255}),Line(points = {{10, -50},{90, -50}}, color = {0,0,255}),Polygon(points = {{40,0},{60,5},{60, -5},{40,0}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -155,119},{145,79}}, textString = "%name", lineColor = {0,0,255}),Line(visible = useHeatPort, points = {{0, -100},{0,0}}, color = {127,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -96, -50},{ -10, -50}}, color = {0,0,255}),Line(points = {{ -10, -50},{ -10,50}}, color = {0,0,255}),Line(points = {{10,70},{10,29}}, color = {0,0,255}),Line(points = {{10,20},{10, -21}}, color = {0,0,255}),Line(points = {{10, -30},{10, -71}}, color = {0,0,255}),Line(points = {{10,50},{96,50}}, color = {0,0,255}),Line(points = {{10,0},{96,0}}, color = {0,0,255}),Line(points = {{10, -50},{96, -50}}, color = {0,0,255}),Polygon(points = {{40,0},{60,5},{60, -5},{40,0}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255})}));

        equation 
          assert(L + dL > 0, "NMOS: Effective length must be positive");
          assert(W + dW > 0, "NMOS: Effective width  must be positive");
          gds = if RDS < 1e-20 and RDS >  -1e-20 then 1e+20 else 1 / RDS;
          v = (Beta * (W + dW)) / (L + dL);
          ud = smooth(0, if D.v < S.v then S.v else D.v);
          us = if D.v < S.v then D.v else S.v;
          uds = ud - us;
          ubs = smooth(0, if B.v > us then 0 else B.v - us);
          ugst = (G.v - us - Vt + K2 * ubs) * K5;
          id = smooth(0, if ugst <= 0 then uds * gds else if ugst > uds then v * uds * (ugst - uds / 2) + uds * gds else (v * ugst * ugst) / 2 + uds * gds);
          G.i = 0;
          D.i = smooth(0, if D.v < S.v then  -id else id);
          S.i = smooth(0, if D.v < S.v then id else  -id);
          B.i = 0;
          LossPower = D.i * (D.v - S.v);
        end NMOS;
        model NPN "Simple BJT according to Ebers-Moll"
          parameter Real Bf = 50 "Forward beta";
          parameter Real Br = 0.1 "Reverse beta";
          parameter SIunits.Current Is = 1e-16 "Transport saturation current";
          parameter SIunits.InversePotential Vak = 0.02 "Early voltage (inverse), 1/Volt";
          parameter SIunits.Time Tauf = 1.2e-10 "Ideal forward transit time";
          parameter SIunits.Time Taur = 5e-09 "Ideal reverse transit time";
          parameter SIunits.Capacitance Ccs = 1e-12 "Collector-substrat(ground) cap.";
          parameter SIunits.Capacitance Cje = 4e-13 "Base-emitter zero bias depletion cap.";
          parameter SIunits.Capacitance Cjc = 5e-13 "Base-coll. zero bias depletion cap.";
          parameter SIunits.Voltage Phie = 0.8 "Base-emitter diffusion voltage";
          parameter Real Me = 0.4 "Base-emitter gradation exponent";
          parameter SIunits.Voltage Phic = 0.8 "Base-collector diffusion voltage";
          parameter Real Mc = 0.333 "Base-collector gradation exponent";
          parameter SIunits.Conductance Gbc = 1e-15 "Base-collector conductance";
          parameter SIunits.Conductance Gbe = 1e-15 "Base-emitter conductance";
          parameter SIunits.Voltage Vt = 0.02585 "Voltage equivalent of temperature";
          parameter Real EMin =  -100 "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax = 40 "if x > EMax, the exp(x) function is linearized";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = 293.15);
        protected 
          Real vbc;
          Real vbe;
          Real qbk;
          Real ibc;
          Real ibe;
          Real cbc;
          Real cbe;
          Real ExMin;
          Real ExMax;
          Real Capcje;
          Real Capcjc;
          function pow "Just a helper function for x^y"
            input Real x;
            input Real y;
            output Real z;
          algorithm 
            z:=x ^ y;
          end pow;
        public 
          Modelica.Electrical.Analog.Interfaces.Pin C "Collector" annotation(Placement(transformation(extent = {{90,40},{110,60}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin B "Base" annotation(Placement(transformation(extent = {{ -90, -10},{ -110,10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin E "Emitter" annotation(Placement(transformation(extent = {{90, -40},{110, -60}}, rotation = 0)));
          annotation(Documentation(info = "
<HTML>
<P>
This model is a simple model of a bipolar npn junction transistor according
to Ebers-Moll.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled yet. The parameters are not temperature dependent.
</P>
<P>
A typical parameter set is:
</P>
<PRE>
  Bf  Br  Is     Vak  Tauf    Taur  Ccs   Cje     Cjc     Phie  Me   PHic   Mc     Gbc    Gbe    Vt
  -   -   A      V    s       s     F     F       F       V     -    V      -      mS     mS     V
  50  0.1 1e-16  0.02 0.12e-9 5e-9  1e-12 0.4e-12 0.5e-12 0.8   0.4  0.8    0.333  1e-15  1e-15  0.02585
</PRE>

<P>
<DL>
<DT><b>References:</b>
<DD>Vlach, J.; Singal, K.: Computer methods for circuit analysis and design.
Van Nostrand Reinhold, New York 1983
on page 317 ff.
</DL>
<P>
</HTML>
", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>

<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -10,40},{ -10, -40}}, color = {0,0,255}),Line(points = {{ -10,0},{ -90,0}}, color = {0,0,255}),Line(points = {{91,50},{30,50}}, color = {0,0,255}),Line(points = {{30,50},{ -10,10}}, color = {0,0,255}),Line(points = {{ -10, -10},{30, -50}}, color = {0,0,255}),Line(points = {{30, -50},{91, -50}}, color = {0,0,255}),Polygon(points = {{30, -50},{24, -36},{16, -44},{30, -50}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -152,117},{148,77}}, textString = "%name", lineColor = {0,0,255}),Line(visible = useHeatPort, points = {{0, -100},{0, -20}}, color = {127,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -10,40},{ -10, -40}}, color = {0,0,255}),Line(points = {{ -10,0},{ -100,0}}, color = {0,0,255}),Line(points = {{30,50},{ -10,10}}, color = {0,0,255}),Line(points = {{ -10, -10},{30, -50}}, color = {0,0,255}),Line(points = {{30, -50},{100, -50}}, color = {0,0,255}),Line(points = {{100,50},{30,50}}, color = {0,0,255}),Polygon(points = {{30, -50},{24, -36},{16, -44},{30, -50}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255})}));

        equation 
          ExMin = exp(EMin);
          ExMax = exp(EMax);
          vbc = B.v - C.v;
          vbe = B.v - E.v;
          qbk = 1 - vbc * Vak;
          ibc = smooth(1, if vbc / Vt < EMin then Is * (ExMin * (vbc / Vt - EMin + 1) - 1) + vbc * Gbc else if vbc / Vt > EMax then Is * (ExMax * (vbc / Vt - EMax + 1) - 1) + vbc * Gbc else Is * (exp(vbc / Vt) - 1) + vbc * Gbc);
          ibe = smooth(1, if vbe / Vt < EMin then Is * (ExMin * (vbe / Vt - EMin + 1) - 1) + vbe * Gbe else if vbe / Vt > EMax then Is * (ExMax * (vbe / Vt - EMax + 1) - 1) + vbe * Gbe else Is * (exp(vbe / Vt) - 1) + vbe * Gbe);
          Capcjc = smooth(1, if vbc / Phic > 0 then Cjc * (1 + (Mc * vbc) / Phic) else Cjc * pow(1 - vbc / Phic,  -Mc));
          Capcje = smooth(1, if vbe / Phie > 0 then Cje * (1 + (Me * vbe) / Phie) else Cje * pow(1 - vbe / Phie,  -Me));
          cbc = smooth(1, if vbc / Vt < EMin then (Taur * Is) / Vt * ExMin * (vbc / Vt - EMin + 1) + Capcjc else if vbc / Vt > EMax then (Taur * Is) / Vt * ExMax * (vbc / Vt - EMax + 1) + Capcjc else (Taur * Is) / Vt * exp(vbc / Vt) + Capcjc);
          cbe = smooth(1, if vbe / Vt < EMin then (Tauf * Is) / Vt * ExMin * (vbe / Vt - EMin + 1) + Capcje else if vbe / Vt > EMax then (Tauf * Is) / Vt * ExMax * (vbe / Vt - EMax + 1) + Capcje else (Tauf * Is) / Vt * exp(vbe / Vt) + Capcje);
          C.i = (ibe - ibc) * qbk - ibc / Br - cbc * der(vbc) + Ccs * der(C.v);
          B.i = ibe / Bf + ibc / Br + cbc * der(vbc) + cbe * der(vbe);
          E.i =  -B.i - C.i + Ccs * der(C.v);
          LossPower = (C.v - E.v) * (ibe - ibc) * qbk + (vbc * ibc) / Br + (vbe * ibe) / Bf;
        end NPN;
        model PNP "Simple BJT according to Ebers-Moll"
          parameter Real Bf = 50 "Forward beta";
          parameter Real Br = 0.1 "Reverse beta";
          parameter SIunits.Current Is = 1e-16 "Transport saturation current";
          parameter SIunits.InversePotential Vak = 0.02 "Early voltage (inverse), 1/Volt";
          parameter SIunits.Time Tauf = 1.2e-10 "Ideal forward transit time";
          parameter SIunits.Time Taur = 5e-09 "Ideal reverse transit time";
          parameter SIunits.Capacitance Ccs = 1e-12 "Collector-substrat(ground) cap.";
          parameter SIunits.Capacitance Cje = 4e-13 "Base-emitter zero bias depletion cap.";
          parameter SIunits.Capacitance Cjc = 5e-13 "Base-coll. zero bias depletion cap.";
          parameter SIunits.Voltage Phie = 0.8 "Base-emitter diffusion voltage";
          parameter Real Me = 0.4 "Base-emitter gradation exponent";
          parameter SIunits.Voltage Phic = 0.8 "Base-collector diffusion voltage";
          parameter Real Mc = 0.333 "Base-collector gradation exponent";
          parameter SIunits.Conductance Gbc = 1e-15 "Base-collector conductance";
          parameter SIunits.Conductance Gbe = 1e-15 "Base-emitter conductance";
          parameter SIunits.Voltage Vt = 0.02585 "Voltage equivalent of temperature";
          parameter Real EMin =  -100 "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax = 40 "if x > EMax, the exp(x) function is linearized";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = 293.15);
        protected 
          Real vbc;
          Real vbe;
          Real qbk;
          Real ibc;
          Real ibe;
          Real cbc;
          Real cbe;
          Real ExMin;
          Real ExMax;
          Real Capcje;
          Real Capcjc;
          function pow "Just a helper function for x^y"
            input Real x;
            input Real y;
            output Real z;
          algorithm 
            z:=x ^ y;
          end pow;
        public 
          Modelica.Electrical.Analog.Interfaces.Pin C "Collector" annotation(Placement(transformation(extent = {{90,40},{110,60}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin B "Base" annotation(Placement(transformation(extent = {{ -90, -10},{ -110,10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin E "Emitter" annotation(Placement(transformation(extent = {{90, -40},{110, -60}}, rotation = 0)));
          annotation(Documentation(info = "
<HTML>
<P>
This model is a simple model of a bipolar pnp junction transistor according
to Ebers-Moll.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled yet. The parameters are not temperature dependent.
<P>
A typical parameter set is:
</P>
<PRE>
  Bf  Br  Is     Vak  Tauf    Taur  Ccs   Cje     Cjc     Phie  Me   PHic   Mc     Gbc    Gbe    Vt
  -   -   A      V    s       s     F     F       F       V     -    V      -      mS     mS     V
  50  0.1 1e-16  0.02 0.12e-9 5e-9  1e-12 0.4e-12 0.5e-12 0.8   0.4  0.8    0.333  1e-15  1e-15  0.02585
</PRE>

<P>
<DL>
<DT><b>References:</b>
<DD>Vlach, J.; Singal, K.: Computer methods for circuit analysis and design.
Van Nostrand Reinhold, New York 1983
on page 317 ff.
</DL>
<P>
</HTML>
", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>

<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -10,40},{ -10, -40}}, color = {0,0,255}),Line(points = {{ -10,0},{ -90,0}}, color = {0,0,255}),Line(points = {{91,50},{30,50}}, color = {0,0,255}),Line(points = {{30,50},{ -10,10}}, color = {0,0,255}),Line(points = {{ -10, -10},{30, -50}}, color = {0,0,255}),Line(points = {{30, -50},{91, -50}}, color = {0,0,255}),Polygon(points = {{ -10, -10},{5, -17},{ -3, -25},{ -10, -10}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -158,119},{142,79}}, textString = "%name", lineColor = {0,0,255}),Line(visible = useHeatPort, points = {{0, -100},{0, -20}}, color = {127,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -10,40},{ -10, -40}}, color = {0,0,255}),Line(points = {{ -10,0},{ -99,0}}, color = {0,0,255}),Line(points = {{100,50},{30,50}}, color = {0,0,255}),Line(points = {{30,50},{ -10,10}}, color = {0,0,255}),Line(points = {{ -10, -10},{30, -50}}, color = {0,0,255}),Line(points = {{30, -50},{99, -50}}, color = {0,0,255}),Polygon(points = {{ -10, -10},{5, -17},{ -3, -25},{ -10, -10}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255})}));

        equation 
          ExMin = exp(EMin);
          ExMax = exp(EMax);
          vbc = C.v - B.v;
          vbe = E.v - B.v;
          qbk = 1 - vbc * Vak;
          ibc = smooth(1, if vbc / Vt < EMin then Is * (ExMin * (vbc / Vt - EMin + 1) - 1) + vbc * Gbc else if vbc / Vt > EMax then Is * (ExMax * (vbc / Vt - EMax + 1) - 1) + vbc * Gbc else Is * (exp(vbc / Vt) - 1) + vbc * Gbc);
          ibe = smooth(1, if vbe / Vt < EMin then Is * (ExMin * (vbe / Vt - EMin + 1) - 1) + vbe * Gbe else if vbe / Vt > EMax then Is * (ExMax * (vbe / Vt - EMax + 1) - 1) + vbe * Gbe else Is * (exp(vbe / Vt) - 1) + vbe * Gbe);
          Capcjc = smooth(1, if vbc / Phic > 0 then Cjc * (1 + (Mc * vbc) / Phic) else Cjc * pow(1 - vbc / Phic,  -Mc));
          Capcje = smooth(1, if vbe / Phie > 0 then Cje * (1 + (Me * vbe) / Phie) else Cje * pow(1 - vbe / Phie,  -Me));
          cbc = smooth(1, if vbc / Vt < EMin then (Taur * Is) / Vt * ExMin * (vbc / Vt - EMin + 1) + Capcjc else if vbc / Vt > EMax then (Taur * Is) / Vt * ExMax * (vbc / Vt - EMax + 1) + Capcjc else (Taur * Is) / Vt * exp(vbc / Vt) + Capcjc);
          cbe = smooth(1, if vbe / Vt < EMin then (Tauf * Is) / Vt * ExMin * (vbe / Vt - EMin + 1) + Capcje else if vbe / Vt > EMax then (Tauf * Is) / Vt * ExMax * (vbe / Vt - EMax + 1) + Capcje else (Tauf * Is) / Vt * exp(vbe / Vt) + Capcje);
          C.i =  -((ibe - ibc) * qbk - ibc / Br - cbc * der(vbc) - Ccs * der(C.v));
          B.i =  -(ibe / Bf + ibc / Br + cbe * der(vbe) + cbc * der(vbc));
          E.i =  -B.i - C.i + Ccs * der(C.v);
          LossPower = (E.v - C.v) * (ibe - ibc) * qbk + (vbc * ibc) / Br + (vbe * ibe) / Bf;
        end PNP;
        model HeatingDiode "Simple diode with heating port"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter Modelica.SIunits.Current Ids = 1e-06 "Saturation current";
          parameter Real Maxexp(final min = Modelica.Constants.small) = 15 "Max. exponent for linear continuation";
          parameter Modelica.SIunits.Resistance R = 100000000.0 "Parallel ohmic resistance";
          parameter Real EG = 1.11 "activation energy";
          parameter Real N = 1 "Emission coefficient";
          parameter Modelica.SIunits.Temperature TNOM = 300.15 "Parameter measurement temperature";
          parameter Real XTI = 3 "Temperature exponent of saturation current";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(useHeatPort = true);
          Modelica.SIunits.Temperature vt_t "Temperature voltage";
          Modelica.SIunits.Current id "diode current";
        protected 
          Real k = 1.380662e-23 "Boltzmann's constant, J/K";
          Real q = 1.6021892e-19 "Electron charge, As";
          Modelica.SIunits.Temperature htemp "auxiliary temperature";
          Real aux;
          Real auxp;
          Real maxexp = exp(Maxexp);
          annotation(__Dymola_structurallyIncomplete = true, Documentation(info = "
<HTML>
<P>
The simple diode is an electrical one port, where a heat port is added, which is
defined in the Modelica.Thermal library. It consists of the diode itself and an parallel ohmic
resistance <i>R</i>. The diode formula is:
</P>
<PRE>
                v/vt_t
  i  =  ids ( e        - 1).
</PRE>
where vt_t depends on the temperature of the heat port:
<PRE>
  vt_t = k*temp/q
</PRE>
<P>
If the exponent <i>v/vt_t</i> reaches the limit <i>maxex</i>, the diode characterisic is linearly
continued to avoid overflow.<br>
The thermal power is calculated by <i>i*v</i>.
</P>
</HTML>
", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>April 5, 2004   </i>
       by Christoph Clauss<br> implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{30,0},{ -30,40},{ -30, -40},{30,0}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{40,0}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255}),Line(points = {{30,40},{30, -40}}, color = {0,0,255}),Text(extent = {{ -152,114},{148,74}}, textString = "%name", lineColor = {0,0,255}),Line(visible = useHeatPort, points = {{0, -101},{0, -20}}, color = {127,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{30,0},{ -30,40},{ -30, -40},{30,0}}, lineColor = {0,0,0}, fillColor = {255,0,0}, fillPattern = FillPattern.None),Line(points = {{ -99,0},{39,0}}, color = {0,0,255}),Line(points = {{40,0},{96,0}}, color = {0,0,255}),Line(points = {{30,40},{30, -40}}, color = {0,0,255})}));

        equation 
          assert(T_heatPort > 0, "temperature must be positive");
          htemp = T_heatPort;
          vt_t = (k * htemp) / q;
          id = exlin(v / (N * vt_t), Maxexp) - 1;
          aux = ((htemp / TNOM - 1) * EG) / (N * vt_t);
          auxp = exp(aux);
          i = Ids * id * pow(htemp / TNOM, XTI / N) * auxp + v / R;
          LossPower = i * v;
        end HeatingDiode;
        model HeatingNMOS "Simple MOS Transistor with heating port"
          Modelica.Electrical.Analog.Interfaces.Pin D "Drain" annotation(Placement(transformation(extent = {{90,40},{110,60}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin G "Gate" annotation(Placement(transformation(extent = {{ -90, -40},{ -110, -60}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin S "Source" annotation(Placement(transformation(extent = {{90, -40},{110, -60}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin B "Bulk" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          parameter Modelica.SIunits.Length W = 2e-05 "Width";
          parameter Modelica.SIunits.Length L = 6e-06 "Length";
          parameter Modelica.SIunits.Transconductance Beta = 4.1e-05 "Transconductance parameter";
          parameter Modelica.SIunits.Voltage Vt = 0.8 "Zero bias threshold voltage";
          parameter Real K2 = 1.144 "Bulk threshold parameter";
          parameter Real K5 = 0.7311 "Reduction of pinch-off region";
          parameter Modelica.SIunits.Length dW =  -2.5e-06 "narrowing of channel";
          parameter Modelica.SIunits.Length dL =  -1.5e-06 "shortening of channel";
          parameter Modelica.SIunits.Resistance RDS = 10000000.0 "Drain-Source-Resistance";
          parameter Modelica.SIunits.Temperature Tnom = 300.15 "Parameter measurement temperature";
          parameter Real kvt =  -0.00696 "fitting parameter for Vt";
          parameter Real kk2 = 0.0006 "fitting parameter for K22";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(useHeatPort = true);
        protected 
          Real v;
          Real uds;
          Real ubs;
          Real ugst;
          Real ud;
          Real us;
          Real id;
          Real gds;
          Real beta_t;
          Real vt_t;
          Real k2_t;
          annotation(__Dymola_structurallyIncomplete = true, Documentation(info = "<html>
<p>The NMos model is a simple model of a n-channel metal-oxide semiconductor FET. It differs slightly from the device used in the SPICE simulator. For more details please care for H. Spiro. </p>
<p>A heating port is added for thermal electric simulation. The heating port is defined in the Modelica.Thermal library.  </p>
<p>The model does not consider capacitances. A high drain-source resistance RDS is included to avoid numerical difficulties. </p>
<pre>  W       L      Beta        Vt       K2      K5       DW         DL
  m       m      A/V^2       V        -       -        m          m
  12.e-6  4.e-6  .062e-3   -4.5      .24     .61     -1.2e-6     -.9e-6      depletion
  60.e-6  3.e-6  .048e-3     .1      .08     .68     -1.2e-6     -.9e-6      enhancement
  12.e-6  4.e-6  .0625e-3   -.8      .21     .78     -1.2e-6     -.9e-6      zero
  50.e-6  8.e-6  .0299e-3    .24    1.144    .7311   -5.4e-6    -4.e-6
  20.e-6  6.e-6  .041e-3     .8     1.144    .7311   -2.5e-6    -1.5e-6
  30.e-6  9.e-6  .025e-3   -4.       .861    .878    -3.4e-6    -1.74e-6
  30.e-6  5.e-6  .031e-3     .6     1.5      .72      0         -3.9e-6
  50.e-6  6.e-6  .0414e-3  -3.8      .34     .8      -1.6e-6    -2.e-6       depletion
  50.e-6  5.e-6  .03e-3      .37     .23     .86     -1.6e-6    -2.e-6       enhancement
  50.e-6  6.e-6  .038e-3    -.9      .23     .707    -1.6e-6    -2.e-6       zero
  20.e-6  4.e-6  .06776e-3   .5409   .065    .71      -.8e-6     -.2e-6
  20.e-6  4.e-6  .06505e-3   .6209   .065    .71      -.8e-6     -.2e-6
  20.e-6  4.e-6  .05365e-3   .6909   .03     .8       -.3e-6     -.2e-6
  20.e-6  4.e-6  .05365e-3   .4909   .03     .8       -.3e-6     -.2e-6
  12.e-6  4.e-6  .023e-3   -4.5      .29     .6       0          0           depletion
  60.e-6  3.e-6  .022e-3     .1      .11     .65      0          0           enhancement
  12.e-6  4.e-6  .038e-3    -.8      .33     .6       0          0           zero
  20.e-6  6.e-6  .022e-3     .8     1        .66      0          0</pre>
<p><b>References:</b> </p>
<p>Spiro, H.: Simulation integrierter Schaltungen. R. Oldenbourg Verlag Muenchen Wien 1990. </p>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>December 7, 2005   </i>
       by Christoph Clauss<br>
       error in RDS calculation deleted</li>
<li><i>March 31, 2004   </i>
       by Christoph Clauss<br> implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -90, -50},{ -10, -50}}, color = {0,0,255}),Line(points = {{ -10, -50},{ -10,50}}, color = {0,0,255}),Line(points = {{10,70},{10,29}}, color = {0,0,255}),Line(points = {{10,20},{10, -21}}, color = {0,0,255}),Line(points = {{10, -30},{10, -71}}, color = {0,0,255}),Line(points = {{10,50},{91,50}}, color = {0,0,255}),Line(points = {{10,0},{90,0}}, color = {0,0,255}),Line(points = {{10, -50},{90, -50}}, color = {0,0,255}),Polygon(points = {{40,0},{60,5},{60, -5},{40,0}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -152,118},{148,78}}, textString = "%name", lineColor = {0,0,255}),Line(visible = useHeatPort, points = {{0, -100},{0,0}}, color = {127,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -96, -50},{ -10, -50}}, color = {0,0,255}),Line(points = {{ -10, -50},{ -10,50}}, color = {0,0,255}),Line(points = {{10,70},{10,29}}, color = {0,0,255}),Line(points = {{10,20},{10, -21}}, color = {0,0,255}),Line(points = {{10, -30},{10, -71}}, color = {0,0,255}),Line(points = {{10,50},{96,50}}, color = {0,0,255}),Line(points = {{10,0},{96,0}}, color = {0,0,255}),Line(points = {{10, -50},{96, -50}}, color = {0,0,255}),Polygon(points = {{40,0},{60,5},{60, -5},{40,0}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255})}));

        equation 
          assert(L + dL > 0, "Heating NMOS: Effective length must be positive");
          assert(W + dW > 0, "Heating NMOS: Effective width  must be positive");
          assert(T_heatPort > 0, "Heating NMOS: Temperature must be positive");
          gds = if RDS < 1e-20 and RDS >  -1e-20 then 1e+20 else 1 / RDS;
          v = (beta_t * (W + dW)) / (L + dL);
          ud = smooth(0, if D.v < S.v then S.v else D.v);
          us = smooth(0, if D.v < S.v then D.v else S.v);
          uds = ud - us;
          ubs = smooth(0, if B.v > us then 0 else B.v - us);
          ugst = (G.v - us - vt_t + k2_t * ubs) * K5;
          id = smooth(0, if ugst <= 0 then uds * gds else if ugst > uds then v * uds * (ugst - uds / 2) + uds * gds else (v * ugst * ugst) / 2 + uds * gds);
          beta_t = Beta * pow(T_heatPort / Tnom,  -1.5);
          vt_t = Vt * (1 + (T_heatPort - Tnom) * kvt);
          k2_t = K2 * (1 + (T_heatPort - Tnom) * kk2);
          G.i = 0;
          D.i = smooth(0, if D.v < S.v then  -id else id);
          S.i = smooth(0, if D.v < S.v then id else  -id);
          B.i = 0;
          LossPower = D.i * (D.v - S.v);
        end HeatingNMOS;
        model HeatingPMOS "Simple PMOS Transistor with heating port"
          Modelica.Electrical.Analog.Interfaces.Pin D "Drain" annotation(Placement(transformation(extent = {{90,40},{110,60}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin G "Gate" annotation(Placement(transformation(extent = {{ -90, -40},{ -110, -60}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin S "Source" annotation(Placement(transformation(extent = {{90, -40},{110, -60}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin B "Bulk" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          parameter Modelica.SIunits.Length W = 2e-05 "Width";
          parameter Modelica.SIunits.Length L = 6e-06 "Length";
          parameter Modelica.SIunits.Transconductance Beta = 1.05e-05 "Transconductance parameter";
          parameter Modelica.SIunits.Voltage Vt =  -1.0 "Zero bias threshold voltage";
          parameter Real K2 = 0.41 "Bulk threshold parameter";
          parameter Real K5 = 0.839 "Reduction of pinch-off region";
          parameter Modelica.SIunits.Length dW =  -2.5e-06 "Narrowing of channel";
          parameter Modelica.SIunits.Length dL =  -2.1e-06 "Shortening of channel";
          parameter Modelica.SIunits.Resistance RDS = 10000000.0 "Drain-Source-Resistance";
          parameter Modelica.SIunits.Temperature Tnom = 300.15 "Parameter measurement temperature";
          parameter Real kvt =  -0.0029 "fitting parameter for Vt";
          parameter Real kk2 = 0.00062 "fitting parameter for Kk2";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(useHeatPort = true);
        protected 
          Real v;
          Real uds;
          Real ubs;
          Real ugst;
          Real ud;
          Real us;
          Real id;
          Real gds;
          Real beta_t;
          Real vt_t;
          Real k2_t;
          annotation(__Dymola_structurallyIncomplete = true, Documentation(info = "<html>
<p>The PMOS model is a simple model of a p-channel metal-oxide semiconductor FET. It differs slightly from the device used in the SPICE simulator. For more details please care for H. Spiro. </p>
<p>A heating port is added for thermal electric simulation. The heating port is defined in the Modelica.Thermal library. </p>
<p>The model does not consider capacitances. A high drain-source resistance RDS is included to avoid numerical difficulties.  </p>
<dl><dt><b>References:</b> </dt>
<dd>Spiro, H.: Simulation integrierter Schaltungen. R. Oldenbourg Verlag Muenchen Wien 1990. </dd>
</dl><p>Some typical parameter sets are: </p>
<pre>  W       L      Beta        Vt       K2       K5       DW         DL
  m       m      A/V^2       V        -        -        m          m
  50.e-6  8.e-6  .0085e-3   -.15     .41      .839    -3.8e-6    -4.0e-6
  20.e-6  6.e-6  .0105e-3  -1.0      .41      .839    -2.5e-6    -2.1e-6
  30.e-6  5.e-6  .0059e-3   -.3      .98     1.01      0         -3.9e-6
  30.e-6  5.e-6  .0152e-3   -.69     .104    1.1       -.8e-6     -.4e-6
  30.e-6  5.e-6  .0163e-3   -.69     .104    1.1       -.8e-6     -.4e-6
  30.e-6  5.e-6  .0182e-3   -.69     .086    1.06      -.1e-6     -.6e-6
  20.e-6  6.e-6  .0074e-3  -1.       .4       .59      0          0 </pre>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>December 7, 2005   </i>
       by Christoph Clauss<br>
       error in RDS calculation deleted</li>
<li><i>March 31, 2004   </i>
       by Christoph Clauss<br> implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -90, -50},{ -10, -50}}, color = {0,0,255}),Line(points = {{ -10, -50},{ -10,50}}, color = {0,0,255}),Line(points = {{10,70},{10,29}}, color = {0,0,255}),Line(points = {{10,20},{10, -21}}, color = {0,0,255}),Line(points = {{10, -30},{10, -71}}, color = {0,0,255}),Line(points = {{10,50},{91,50}}, color = {0,0,255}),Line(points = {{10,0},{90,0}}, color = {0,0,255}),Line(points = {{10, -50},{90, -50}}, color = {0,0,255}),Polygon(points = {{60,0},{40,5},{40, -5},{60,0}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -155,120},{145,80}}, textString = "%name", lineColor = {0,0,255}),Line(visible = useHeatPort, points = {{0, -100},{0,0}}, color = {127,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -100, -50},{ -10, -50}}, color = {0,0,255}),Line(points = {{ -10, -50},{ -10,50}}, color = {0,0,255}),Line(points = {{10,70},{10,29}}, color = {0,0,255}),Line(points = {{10,20},{10, -21}}, color = {0,0,255}),Line(points = {{10, -30},{10, -71}}, color = {0,0,255}),Line(points = {{10,50},{96,50}}, color = {0,0,255}),Line(points = {{10,0},{101,0}}, color = {0,0,255}),Line(points = {{10, -50},{100, -50}}, color = {0,0,255}),Polygon(points = {{60,0},{40,5},{40, -5},{60,0}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255})}));

        equation 
          assert(L + dL > 0, "HeatingPMOS: Effective length must be positive");
          assert(W + dW > 0, "HeatingPMOS: Effective width  must be positive");
          assert(T_heatPort > 0, "HeatingPMOS: Temperature must be positive");
          gds = if RDS < 1e-20 and RDS >  -1e-20 then 1e+20 else 1 / RDS;
          v = (beta_t * (W + dW)) / (L + dL);
          ud = smooth(0, if D.v > S.v then S.v else D.v);
          us = smooth(0, if D.v > S.v then D.v else S.v);
          uds = ud - us;
          ubs = smooth(0, if B.v < us then 0 else B.v - us);
          ugst = (G.v - us - vt_t + k2_t * ubs) * K5;
          id = smooth(0, if ugst >= 0 then uds * gds else if ugst < uds then  -v * uds * (ugst - uds / 2) + uds * gds else  -(v * ugst * ugst) / 2 + uds * gds);
          beta_t = Beta * pow(T_heatPort / Tnom,  -1.5);
          vt_t = Vt * (1 + (T_heatPort - Tnom) * kvt);
          k2_t = K2 * (1 + (T_heatPort - Tnom) * kk2);
          G.i = 0;
          D.i = smooth(0, if D.v > S.v then  -id else id);
          S.i = smooth(0, if D.v > S.v then id else  -id);
          B.i = 0;
          LossPower = D.i * (D.v - S.v);
        end HeatingPMOS;
        model HeatingNPN "Simple NPN BJT according to Ebers-Moll with heating port"
          parameter Real Bf = 50 "Forward beta";
          parameter Real Br = 0.1 "Reverse beta";
          parameter Modelica.SIunits.Current Is = 1e-16 "Transport saturation current";
          parameter Modelica.SIunits.InversePotential Vak = 0.02 "Early voltage (inverse), 1/Volt";
          parameter Modelica.SIunits.Time Tauf = 1.2e-10 "Ideal forward transit time";
          parameter Modelica.SIunits.Time Taur = 5e-09 "Ideal reverse transit time";
          parameter Modelica.SIunits.Capacitance Ccs = 1e-12 "Collector-substrat(ground) cap.";
          parameter Modelica.SIunits.Capacitance Cje = 4e-13 "Base-emitter zero bias depletion cap.";
          parameter Modelica.SIunits.Capacitance Cjc = 5e-13 "Base-coll. zero bias depletion cap.";
          parameter Modelica.SIunits.Voltage Phie = 0.8 "Base-emitter diffusion voltage";
          parameter Real Me = 0.4 "Base-emitter gradation exponent";
          parameter Modelica.SIunits.Voltage Phic = 0.8 "Base-collector diffusion voltage";
          parameter Real Mc = 0.333 "Base-collector gradation exponent";
          parameter Modelica.SIunits.Conductance Gbc = 1e-15 "Base-collector conductance";
          parameter Modelica.SIunits.Conductance Gbe = 1e-15 "Base-emitter conductance";
          parameter Real EMin =  -100 "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax = 40 "if x > EMax, the exp(x) function is linearized";
          parameter Modelica.SIunits.Temperature Tnom = 300.15 "Parameter measurement temperature";
          parameter Real XTI = 3 "Temperature exponent for effect on Is";
          parameter Real XTB = 0 "Forward and reverse beta temperature exponent";
          parameter Real EG = 1.11 "Energy gap for temperature effect on Is";
          parameter Real NF = 1.0 "Forward current emission coefficient";
          parameter Real NR = 1.0 "Reverse current emission coefficient";
          parameter Real K = 1.3806226e-23 "Boltzmann's constant";
          parameter Real q = 1.6021918e-19 "Elementary electronic charge";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(useHeatPort = true);
          Real vbc;
          Real vbe;
          Real qbk;
          Real ibc;
          Real ibe;
          Real cbc;
          Real cbe;
          Real ExMin;
          Real ExMax;
          Real Capcje;
          Real Capcjc;
          Real is_t;
          Real br_t;
          Real bf_t;
          Real vt_t;
          Real hexp;
          Real htempexp;
        public 
          Modelica.Electrical.Analog.Interfaces.Pin C "Collector" annotation(Placement(transformation(extent = {{90,40},{110,60}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin B "Base" annotation(Placement(transformation(extent = {{ -90, -10},{ -110,10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin E "Emitter" annotation(Placement(transformation(extent = {{90, -40},{110, -60}}, rotation = 0)));
          annotation(__Dymola_structurallyIncomplete = true, Documentation(info = "<html>
<p>This model is a simple model of a bipolar npn junction transistor according to Ebers-Moll. </p>
<p>A heating port is added for thermal electric simulation. The heating port is defined in the Modelica.Thermal library. </p>
<p>A typical parameter set is (the parameter Vt is no longer used):  </p>
<pre>  Bf  Br  Is     Vak  Tauf    Taur  Ccs   Cje     Cjc     Phie  Me   PHic   Mc     Gbc    Gbe
  -   -   A      V    s       s     F     F       F       V     -    V      -      mS     mS
  50  0.1 1e-16  0.02 0.12e-9 5e-9  1e-12 0.4e-12 0.5e-12 0.8   0.4  0.8    0.333  1e-15  1e-15</pre>
<p><b>References:</b> </p>
<p>Vlach, J.; Singal, K.: Computer methods for circuit analysis and design. Van Nostrand Reinhold, New York 1983 on page 317 ff. </p>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>March 20, 2004   </i>
       by Christoph Clauss<br> implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -10,40},{ -10, -40}}, color = {0,0,255}),Line(points = {{ -10,0},{ -90,0}}, color = {0,0,255}),Line(points = {{91,50},{30,50}}, color = {0,0,255}),Line(points = {{30,50},{ -10,10}}, color = {0,0,255}),Line(points = {{ -10, -10},{30, -50}}, color = {0,0,255}),Line(points = {{30, -50},{91, -50}}, color = {0,0,255}),Polygon(points = {{30, -50},{24, -36},{16, -44},{30, -50}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -162,125},{138,85}}, textString = "%name", lineColor = {0,0,255}),Line(visible = useHeatPort, points = {{0, -100},{0, -20}}, color = {127,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -10,40},{ -10, -40}}, color = {0,0,255}),Line(points = {{ -10,0},{ -100,0}}, color = {0,0,255}),Line(points = {{30,50},{ -10,10}}, color = {0,0,255}),Line(points = {{ -10, -10},{30, -50}}, color = {0,0,255}),Line(points = {{30, -50},{100, -50}}, color = {0,0,255}),Line(points = {{100,50},{30,50}}, color = {0,0,255}),Polygon(points = {{30, -50},{24, -36},{16, -44},{30, -50}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255})}));

        equation 
          assert(T_heatPort > 0, "temperature must be positive");
          ExMin = exp(EMin);
          ExMax = exp(EMax);
          vbc = B.v - C.v;
          vbe = B.v - E.v;
          qbk = 1 - vbc * Vak;
          hexp = ((T_heatPort / Tnom - 1) * EG) / vt_t;
          htempexp = smooth(1, if hexp < EMin then ExMin * (hexp - EMin + 1) else if hexp > EMax then ExMax * (hexp - EMax + 1) else exp(hexp));
          is_t = Is * pow(T_heatPort / Tnom, XTI) * htempexp;
          br_t = Br * pow(T_heatPort / Tnom, XTB);
          bf_t = Bf * pow(T_heatPort / Tnom, XTB);
          vt_t = K / q * T_heatPort;
          ibc = smooth(1, if vbc / (NR * vt_t) < EMin then is_t * (ExMin * (vbc / (NR * vt_t) - EMin + 1) - 1) + vbc * Gbc else if vbc / (NR * vt_t) > EMax then is_t * (ExMax * (vbc / (NR * vt_t) - EMax + 1) - 1) + vbc * Gbc else is_t * (exp(vbc / (NR * vt_t)) - 1) + vbc * Gbc);
          ibe = smooth(1, if vbe / (NF * vt_t) < EMin then is_t * (ExMin * (vbe / (NF * vt_t) - EMin + 1) - 1) + vbe * Gbe else if vbe / (NF * vt_t) > EMax then is_t * (ExMax * (vbe / (NF * vt_t) - EMax + 1) - 1) + vbe * Gbe else is_t * (exp(vbe / (NF * vt_t)) - 1) + vbe * Gbe);
          Capcjc = smooth(1, if vbc / Phic > 0 then Cjc * (1 + (Mc * vbc) / Phic) else Cjc * pow(1 - vbc / Phic,  -Mc));
          Capcje = smooth(1, if vbe / Phie > 0 then Cje * (1 + (Me * vbe) / Phie) else Cje * pow(1 - vbe / Phie,  -Me));
          cbc = smooth(1, if vbc / (NR * vt_t) < EMin then (Taur * is_t) / (NR * vt_t) * ExMin * (vbc / (NR * vt_t) - EMin + 1) + Capcjc else if vbc / (NR * vt_t) > EMax then (Taur * is_t) / (NR * vt_t) * ExMax * (vbc / (NR * vt_t) - EMax + 1) + Capcjc else (Taur * is_t) / (NR * vt_t) * exp(vbc / (NR * vt_t)) + Capcjc);
          cbe = smooth(1, if vbe / (NF * vt_t) < EMin then (Tauf * is_t) / (NF * vt_t) * ExMin * (vbe / (NF * vt_t) - EMin + 1) + Capcje else if vbe / (NF * vt_t) > EMax then (Tauf * is_t) / (NF * vt_t) * ExMax * (vbe / (NF * vt_t) - EMax + 1) + Capcje else (Tauf * is_t) / (NF * vt_t) * exp(vbe / (NF * vt_t)) + Capcje);
          C.i = (ibe - ibc) * qbk - ibc / br_t - cbc * der(vbc) + Ccs * der(C.v);
          B.i = ibe / bf_t + ibc / br_t + cbc * der(vbc) + cbe * der(vbe);
          E.i =  -B.i - C.i + Ccs * der(C.v);
          LossPower = (vbc * ibc) / br_t + (vbe * ibe) / bf_t + (ibe - ibc) * qbk * (C.v - E.v);
        end HeatingNPN;
        model HeatingPNP "Simple PNP BJT according to Ebers-Moll with heating port"
          parameter Real Bf = 50 "Forward beta";
          parameter Real Br = 0.1 "Reverse beta";
          parameter Modelica.SIunits.Current Is = 1e-16 "Transport saturation current";
          parameter Modelica.SIunits.InversePotential Vak = 0.02 "Early voltage (inverse), 1/Volt";
          parameter Modelica.SIunits.Time Tauf = 1.2e-10 "Ideal forward transit time";
          parameter Modelica.SIunits.Time Taur = 5e-09 "Ideal reverse transit time";
          parameter Modelica.SIunits.Capacitance Ccs = 1e-12 "Collector-substrat(ground) cap.";
          parameter Modelica.SIunits.Capacitance Cje = 4e-13 "Base-emitter zero bias depletion cap.";
          parameter Modelica.SIunits.Capacitance Cjc = 5e-13 "Base-coll. zero bias depletion cap.";
          parameter Modelica.SIunits.Voltage Phie = 0.8 "Base-emitter diffusion voltage";
          parameter Real Me = 0.4 "Base-emitter gradation exponent";
          parameter Modelica.SIunits.Voltage Phic = 0.8 "Base-collector diffusion voltage";
          parameter Real Mc = 0.333 "Base-collector gradation exponent";
          parameter Modelica.SIunits.Conductance Gbc = 1e-15 "Base-collector conductance";
          parameter Modelica.SIunits.Conductance Gbe = 1e-15 "Base-emitter conductance";
          parameter Real EMin =  -100 "if x < EMin, the exp(x) function is linearized";
          parameter Real EMax = 40 "if x > EMax, the exp(x) function is linearized";
          parameter Modelica.SIunits.Temperature Tnom = 300.15 "Parameter measurement temperature";
          parameter Real XTI = 3 "Temperature exponent for effect on Is";
          parameter Real XTB = 0 "Forward and reverse beta temperature exponent";
          parameter Real EG = 1.11 "Energy gap for temperature effect on Is";
          parameter Real NF = 1.0 "Forward current emission coefficient";
          parameter Real NR = 1.0 "Reverse current emission coefficient";
          parameter Real K = 1.3806226e-23 "Boltzmann's constant";
          parameter Real q = 1.6021918e-19 "Elementary electronic charge";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(useHeatPort = true);
        protected 
          Real vcb;
          Real veb;
          Real qbk;
          Real icb;
          Real ieb;
          Real ccb;
          Real ceb;
          Real ExMin;
          Real ExMax;
          Real Capcje;
          Real Capcjc;
          Real is_t;
          Real br_t;
          Real bf_t;
          Real vt_t;
          Real hexp;
          Real htempexp;
        public 
          Modelica.Electrical.Analog.Interfaces.Pin C "Collector" annotation(Placement(transformation(extent = {{90,40},{110,60}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin B "Base" annotation(Placement(transformation(extent = {{ -90, -10},{ -110,10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin E "Emitter" annotation(Placement(transformation(extent = {{90, -40},{110, -60}}, rotation = 0)));
          annotation(__Dymola_structurallyIncomplete = true, Documentation(info = "<html>
<p>This model is a simple model of a bipolar pnp junction transistor according to Ebers-Moll. </p>
<p>A heating port is added for thermal electric simulation. The heating port is defined in the Modelica.Thermal library. </p>
<p>A typical parameter set is (the parameter Vt is no longer used):  </p>
<pre>  Bf  Br  Is     Vak  Tauf    Taur  Ccs   Cje     Cjc     Phie  Me   PHic   Mc     Gbc    Gbe
  -   -   A      V    s       s     F     F       F       V     -    V      -      mS     mS
  50  0.1 1e-16  0.02 0.12e-9 5e-9  1e-12 0.4e-12 0.5e-12 0.8   0.4  0.8    0.333  1e-15  1e-15</pre>
<p><b>References:</b> </p>
<p>Vlach, J.; Singal, K.: Computer methods for circuit analysis and design. Van Nostrand Reinhold, New York 1983 on page 317 ff. </p>
</html>", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>March 20, 2004   </i>
       by Christoph Clauss<br> implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -10,40},{ -10, -40}}, color = {0,0,255}),Line(points = {{ -10,0},{ -90,0}}, color = {0,0,255}),Line(points = {{91,50},{30,50}}, color = {0,0,255}),Line(points = {{30,50},{ -10,10}}, color = {0,0,255}),Line(points = {{ -10, -10},{30, -50}}, color = {0,0,255}),Line(points = {{30, -50},{91, -50}}, color = {0,0,255}),Polygon(points = {{ -10, -10},{5, -17},{ -3, -25},{ -10, -10}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -155,117},{145,77}}, textString = "%name", lineColor = {0,0,255}),Line(visible = useHeatPort, points = {{0, -100},{0, -20}}, color = {127,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -10,40},{ -10, -40}}, color = {0,0,255}),Line(points = {{ -10,0},{ -99,0}}, color = {0,0,255}),Line(points = {{96,50},{30,50}}, color = {0,0,255}),Line(points = {{30,50},{ -10,10}}, color = {0,0,255}),Line(points = {{ -10, -10},{30, -50}}, color = {0,0,255}),Line(points = {{30, -50},{96, -50}}, color = {0,0,255}),Polygon(points = {{ -10, -10},{5, -17},{ -3, -25},{ -10, -10}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255})}));

        equation 
          assert(T_heatPort > 0, "temperature must be positive");
          ExMin = exp(EMin);
          ExMax = exp(EMax);
          vcb = C.v - B.v;
          veb = E.v - B.v;
          qbk = 1 - vcb * Vak;
          hexp = ((T_heatPort / Tnom - 1) * EG) / vt_t;
          htempexp = smooth(1, if hexp < EMin then ExMin * (hexp - EMin + 1) else if hexp > EMax then ExMax * (hexp - EMax + 1) else exp(hexp));
          is_t = Is * pow(T_heatPort / Tnom, XTI) * htempexp;
          br_t = Br * pow(T_heatPort / Tnom, XTB);
          bf_t = Bf * pow(T_heatPort / Tnom, XTB);
          vt_t = K / q * T_heatPort;
          icb = smooth(1, if vcb / (NR * vt_t) < EMin then is_t * (ExMin * (vcb / (NR * vt_t) - EMin + 1) - 1) + vcb * Gbc else if vcb / (NR * vt_t) > EMax then is_t * (ExMax * (vcb / (NR * vt_t) - EMax + 1) - 1) + vcb * Gbc else is_t * (exp(vcb / (NR * vt_t)) - 1) + vcb * Gbc);
          ieb = smooth(1, if veb / (NF * vt_t) < EMin then is_t * (ExMin * (veb / (NF * vt_t) - EMin + 1) - 1) + veb * Gbe else if veb / (NF * vt_t) > EMax then is_t * (ExMax * (veb / (NF * vt_t) - EMax + 1) - 1) + veb * Gbe else is_t * (exp(veb / (NF * vt_t)) - 1) + veb * Gbe);
          Capcjc = smooth(1, if vcb / Phic > 0 then Cjc * (1 + (Mc * vcb) / Phic) else Cjc * pow(1 - vcb / Phic,  -Mc));
          Capcje = smooth(1, if veb / Phie > 0 then Cje * (1 + (Me * veb) / Phie) else Cje * pow(1 - veb / Phie,  -Me));
          ccb = smooth(1, if vcb / (NR * vt_t) < EMin then (Taur * is_t) / (NR * vt_t) * ExMin * (vcb / (NR * vt_t) - EMin + 1) + Capcjc else if vcb / (NR * vt_t) > EMax then (Taur * is_t) / (NR * vt_t) * ExMax * (vcb / (NR * vt_t) - EMax + 1) + Capcjc else (Taur * is_t) / (NR * vt_t) * exp(vcb / (NR * vt_t)) + Capcjc);
          ceb = smooth(1, if veb / (NF * vt_t) < EMin then (Tauf * is_t) / (NF * vt_t) * ExMin * (veb / (NF * vt_t) - EMin + 1) + Capcje else if veb / (NF * vt_t) > EMax then (Tauf * is_t) / (NF * vt_t) * ExMax * (veb / (NF * vt_t) - EMax + 1) + Capcje else (Tauf * is_t) / (NF * vt_t) * exp(veb / (NF * vt_t)) + Capcje);
          C.i = icb / br_t + ccb * der(vcb) + Ccs * der(C.v) + (icb - ieb) * qbk;
          B.i =  -ieb / bf_t - icb / br_t - ceb * der(veb) - ccb * der(vcb);
          E.i =  -B.i - C.i + Ccs * der(C.v);
          LossPower = (vcb * icb) / br_t + (veb * ieb) / bf_t + (icb - ieb) * qbk * (C.v - E.v);
        end HeatingPNP;
      protected 
        function pow "Just a helper function for x^y in order that a symbolic engine can apply some transformations more easily"
          input Real x;
          input Real y;
          output Real z;
        algorithm 
          z:=x ^ y;
        end pow;
        function exlin "Exponential function linearly continued for x > Maxexp"
          input Real x;
          input Real Maxexp;
          output Real z;
        algorithm 
          z:=if x > Maxexp then exp(Maxexp) * (1 + x - Maxexp) else exp(x);
        end exlin;
      public 
        model Thyristor "Simple Thyristor Model"
          parameter Modelica.SIunits.Voltage VDRM(final min = 0) = 100 "Forward breakthrough voltage";
          parameter Modelica.SIunits.Voltage VRRM(final min = 0) = 100 "Reverse breakthrough voltage";
          parameter Modelica.SIunits.Current IDRM = 0.1 "Saturation current";
          parameter Modelica.SIunits.Voltage VTM = 1.7 "Conducting voltage";
          parameter Modelica.SIunits.Current IH = 0.006 "Holding current";
          parameter Modelica.SIunits.Current ITM = 25 "Conducting current";
          parameter Modelica.SIunits.Voltage VGT = 0.7 "Gate trigger voltage";
          parameter Modelica.SIunits.Current IGT = 0.005 "Gate trigger current";
          parameter Modelica.SIunits.Time TON = 1e-06 "Switch on time";
          parameter Modelica.SIunits.Time TOFF = 1.5e-05 "Switch off time";
          parameter Modelica.SIunits.Voltage Vt = 0.04 "Voltage equivalent of temperature (kT/qn)";
          parameter Real Nbv = 0.74 "Reverse Breakthrough emission coefficient";
          Real iGK "gate current";
          Real vGK "voltage between gate and cathode";
          Real vAK "voltage between anode and cathode";
          Real vControl(start = 0);
          Real vContot;
          Real vConmain;
          annotation(Documentation(info = "<html>
<p><br/>This is a simple thyristor model with three pins: Anode, Cathode and Gate. There are three operating modes: </p><p><br/>conducting, blocking and reverse breakthrough.  </p>
<p><br/>As long as the thyristor is in blocking mode it behaves like a linear resistance Roff=VDRM^2/(VTM*IH). </p><p><br/>But if the voltage between anode and cathode exceeds VDRM or a positive gate current flows for a sufficient time the mode changes to conducting mode. </p><p><br/>The model stays in conducting mode until the anode current falls below the holding current IH. There is no way to switch off the thyristor via the gate. </p><p><br/>If the voltage between anode and cathode is negative, the model represents a diode (parameters Vt, Nbv) with reverse breakthrough voltage VRRM. </p>
<p>The dV/dt switch on is not taken into account in this model. The gate circuit is not influenced by the main circuit. </p>
</html>", revisions = "<html>
<ul>
<li><i>May 12, 2009   </i>
       by Matthias Franke
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{30,0},{ -30,40},{ -30, -40},{30,0}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{40,0}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255}),Line(points = {{30,40},{30, -40}}, color = {0,0,255}),Line(points = {{30,20},{70,60},{70,90}}, color = {0,0,255}),Line(points = {{40,50},{60,30}}, color = {0,0,255}),Text(extent = {{ -142, -59},{158, -99}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80,0},{80,0}}, color = {128,128,128}),Polygon(points = {{70,4},{80,0},{70, -4},{70,4}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{0,80},{0, -80}}, color = {128,128,128}),Polygon(points = {{ -4,70},{0,80},{4,70},{ -4,70}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{5,81},{15,71}}, lineColor = {128,128,128}, textString = "i"),Text(extent = {{70, -10},{80, -20}}, lineColor = {128,128,128}, textString = "v"),Line(points = {{20,9},{20,0}}, color = {128,128,128}, pattern = LinePattern.Dot),Text(extent = {{25,64},{45,54}}, lineColor = {128,128,128}, pattern = LinePattern.Dot, textString = "Ron"),Line(points = {{14,20},{25,73}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -57, -11},{55,9}}, color = {0,0,0}, smooth = Smooth.None, thickness = 0.5),Line(points = {{ -57, -11},{ -59, -13},{ -60, -18},{ -63, -65}}, color = {0,0,0}, thickness = 0.5, smooth = Smooth.None),Text(extent = {{ -67,11},{ -47,1}}, lineColor = {128,128,128}, pattern = LinePattern.Dot, textString = "VRRM"),Text(extent = {{44, -2},{64, -12}}, lineColor = {128,128,128}, pattern = LinePattern.Dot, textString = "VDRM"),Line(points = {{ -57,1},{ -57, -1}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{54,1},{54, -1}}, color = {0,0,0}, smooth = Smooth.None),Line(points = {{55,9},{54,11},{50,13},{17,17},{15,18},{14,20}}, color = {0,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot),Text(extent = {{ -28, -9},{ -8, -19}}, lineColor = {128,128,128}, pattern = LinePattern.Dot, textString = "Roff"),Line(points = {{2,20},{ -2,20}}, color = {0,0,0}, smooth = Smooth.None),Text(extent = {{ -14,23},{ -2,16}}, lineColor = {128,128,128}, pattern = LinePattern.Dot, textString = "IH")}));
        public 
          Modelica.Electrical.Analog.Interfaces.PositivePin Anode annotation(Placement(transformation(extent = {{ -95, -12},{ -75,8}}), iconTransformation(extent = {{ -100, -10},{ -80,10}})));
          Modelica.Electrical.Analog.Interfaces.NegativePin Cathode annotation(Placement(transformation(extent = {{80, -10},{100,10}}), iconTransformation(extent = {{80, -10},{100,10}})));
          Modelica.Electrical.Analog.Interfaces.PositivePin Gate annotation(Placement(transformation(extent = {{60,80},{80,100}}), iconTransformation(extent = {{60,80},{80,100}})));
        protected 
          parameter Modelica.SIunits.Voltage Von = 5;
          parameter Modelica.SIunits.Voltage Voff = 1.5;
          parameter Modelica.SIunits.Resistance Ron = (VTM - 0.7) / ITM "Forward conducting mode resistance";
          parameter Modelica.SIunits.Resistance Roff = VDRM ^ 2 / VTM / IH "Blocking mode resistance";

        equation 
          Anode.i + Gate.i + Cathode.i = 0;
          vGK = Gate.v - Cathode.v;
          vAK = Anode.v - Cathode.v;
          iGK = Gate.i;
          vGK = smooth(0, if vGK < 0.65 then VGT / IGT * iGK else 0.65 ^ 2 / VGT + (iGK * (VGT - 0.65)) / IGT);
          vContot = vConmain + smooth(0, if iGK < 0.95 * IGT then 0 else if iGK < 0.95 * IGT + 0.001 then 10000 * (iGK - 0.95 * IGT) * vAK else 10 * vAK);
          der(vControl) = (vContot - vControl) / (if vContot - vControl > 0 then 1.87 * TON else 0.638 * TOFF);
          Anode.i = smooth(1, if vAK <  -VRRM then  -VRRM / Roff * exp( -(vAK + VRRM) / (Nbv * Vt)) else if vControl < Voff then vAK / Roff else if vControl < Von then vAK / (sqrt(Ron * Roff) * (Ron / Roff) ^ ((3 * (2 * vControl - Von - Voff) / (2 * (Von - Voff)) - 4 * ((2 * vControl - Von - Voff) / (2 * (Von - Voff))) ^ 3) / 2)) else vAK / Ron);
          vConmain = if Anode.i > IH or vAK > VDRM then Von else 0;
        end Thyristor;
      end Semiconductors;
      package Lines "Lossy and lossless segmented transmission lines, and LC distributed line models"
        extends Modelica.Icons.Library;
        annotation(Documentation(info = "<html>
<p>This package contains lossy and lossless segmented transmission lines, and LC distributed line models. The line models do not yet possess a conditional heating port.  </p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    <a href=\"http://people.eas.iis.fhg.de/Joachim.Haase/\">Joachim Haase;</a>
    &lt;<a href=\"mailto:haase@eas.iis.fhg.de\">haase@eas.iis.fhg.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden<br>
<p>
<dt>
<b>Copyright:</b>
<dd>
Copyright &copy; 1998-2006, Modelica Association and Fraunhofer-Gesellschaft.<br>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package
Modelica in file \"Modelica/package.mo\".</i><br>
<p>
</dl>
</html>"));
        model OLine "Lossy Transmission Line"
          Interfaces.Pin p1 annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Interfaces.Pin p2 annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Interfaces.Pin p3 annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}}, rotation = 0)));
          SI.Voltage v13;
          SI.Voltage v23;
          SI.Current i1;
          SI.Current i2;
          parameter Real r(final min = Modelica.Constants.small, unit = "Ohm/m", start = 1) "Resistance per meter";
          parameter Real l(final min = Modelica.Constants.small, unit = "H/m", start = 1) "Inductance per meter";
          parameter Real g(final min = Modelica.Constants.small, unit = "S/m", start = 1) "Conductance per meter";
          parameter Real c(final min = Modelica.Constants.small, unit = "F/m", start = 1) "Capacitance per meter";
          parameter SI.Length length(final min = Modelica.Constants.small, start = 1) "Length of line";
          parameter Integer N(final min = 1, start = 1) "Number of lumped segments";
        protected 
          Basic.Resistor R[N + 1](R = fill((r * length) / (N + 1), N + 1));
          Basic.Inductor L[N + 1](L = fill((l * length) / (N + 1), N + 1));
          Basic.Capacitor C[N](C = fill((c * length) / N, N));
          Basic.Conductor G[N](G = fill((g * length) / N, N));
          annotation(Documentation(info = "<html>
<p>Lossy Transmission Line. As can be seen in the picture below, the lossy transmission line OLine consists of segments of lumped resistances and inductances in series and conductances and capacitances that are connected with the reference pin p3. The precision of the model depends on the number N of lumped segments.  </p>
<p>To get a symmetric line model, the first resistor and inductor are cut into two parts (R1 and R_Nplus1, L1 and L_Nplus1)</p><p>The two new resistors and inductors have the half of the resistance/inductance the original resistor/inductor had. </p><p>The resistor R1/inductor L1 are at the beginning of the line and R_Nplus1/L_Nplu1 are at the end of the line. </p>
<p align=\"center\"><img src=\"../Images/Electrical/Analog/OLine.png\"/> </p>
<p>The values of the capacitances are calculated with: C=c*length/N with c given by the user, &QUOT;lenght=length of line&QUOT; and &QUOT;N=number of segments&QUOT;.</p><p>The values of the conductances are calculated in the same way the capacitances are, but with g instead of c.</p><p>The values of the resistors and inductors are calculated with :R=r*length/(N+1) and L=l*length/(N+1).</p><p>For all capacitances, conductances, resistors and inductances the values of each segment are the same exept</p><p>of the first and last resistor and inductor, that only have the half of the value of the rest.</p><p>Note, this is different to the lumped line model of SPICE. </p>
<dl><dt><b>References:</b> </dt>
<dd>Johnson, B.; Quarles, T.; Newton, A. R.; Pederson, D. O.; Sangiovanni-Vincentelli, A.: SPICE3 Version 3e User&apos;s Manual (April 1, 1991). Department of Electrical Engineering and Computer Sciences, University of California, Berkley p. 12, p. 106 - 107 </dd>
</dl></html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{0, -60},{0, -90}}, color = {0,0,255}),Line(points = {{60,0},{90,0}}, color = {0,0,255}),Line(points = {{ -60,0},{ -90,0}}, color = {0,0,255}),Line(points = {{30,30},{ -30,30}}, color = {0,0,255}),Line(points = {{ -30,40},{ -30,20}}, color = {0,0,255}),Line(points = {{30,40},{30,20}}, color = {0,0,255}),Text(extent = {{ -155,112},{145,72}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,255}),Line(points = {{0, -60},{0, -96}}, color = {0,0,255}),Line(points = {{60,0},{96,0}}, color = {0,0,255}),Line(points = {{ -60,0},{ -96,0}}, color = {0,0,255}),Line(points = {{30,30},{ -30,30}}, color = {0,0,255}),Line(points = {{ -30,40},{ -30,20}}, color = {0,0,255}),Line(points = {{30,40},{30,20}}, color = {0,0,255})}));

        equation 
          v13 = p1.v - p3.v;
          v23 = p2.v - p3.v;
          i1 = p1.i;
          i2 = p2.i;
          connect(p1,R[1].p);
          for i in 1:N loop
          connect(R[i].n,L[i].p);
          connect(L[i].n,C[i].p);
          connect(L[i].n,G[i].p);
          connect(C[i].n,p3);
          connect(G[i].n,p3);
          connect(L[i].n,R[i + 1].p);

          end for;
          connect(R[N + 1].n,L[N + 1].p);
          connect(L[N + 1].n,p2);
        end OLine;
        model M_OLine "Multiple OLine"
          parameter Modelica.SIunits.Length length(final min = Modelica.Constants.small) = 0.1 "Length of line";
          parameter Integer N(final min = 1) = 5 "Number of lumped segments";
          parameter Integer lines(final min = 2) = 4 "Number of lines";
        protected 
          parameter Integer dim_vector_lgc = div(lines * (lines + 1), 2);
        public 
          parameter Real r[lines](final min = Modelica.Constants.small, unit = "Ohm/m") = {476000.0,172000.0,172000.0,172000.0} "Resistance per meter";
          parameter Real l[dim_vector_lgc](final min = Modelica.Constants.small, unit = "H/m") = {5.98e-07,4.44e-07,4.39e-07,3.99e-07,5.81e-07,4.09e-07,4.23e-07,5.96e-07,4.71e-07,6.06e-07} "Inductance per meter";
          parameter Real g[dim_vector_lgc](final min = Modelica.Constants.small, unit = "S/m") = {8.05e-06,3.42e-05,2 - 0.00091,1.76e-06,9.16e-06,7.12e-06,2.43e-05,5.93e-06,4.19e-05,6.64e-06} "Conductance per meter";
          parameter Real c[dim_vector_lgc](final min = Modelica.Constants.small, unit = "F/m") = {2.38e-11,1.01e-10,8.56e-11,5.09e-12,2.71e-11,2.09e-11,7.16e-11,1.83e-11,1.23e-10,2.07e-11} "Capacitance per meter";
          model segment "Multiple line segment model"
            parameter Integer lines(final min = 1) = 3 "Number of lines";
            parameter Integer dim_vector_lgc = div(lines * (lines + 1), 2) "Length of the vectors for l, g, c";
            Modelica.Electrical.Analog.Interfaces.PositivePin p[lines] "Positive pin" annotation(Placement(transformation(extent = {{ -60, -10},{ -40,10}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin n[lines] "Negative pin" annotation(Placement(transformation(extent = {{40, -10},{60,10}}, rotation = 0)));
            parameter Real Cl[dim_vector_lgc] = fill(1, dim_vector_lgc) "Capacitance matrix";
            parameter Real Rl[lines] = fill(7, lines) "Resistance matrix";
            parameter Real Ll[dim_vector_lgc] = fill(2, dim_vector_lgc) "Inductance matrix";
            parameter Real Gl[dim_vector_lgc] = fill(1, dim_vector_lgc) "Conductance matrix";
            Modelica.Electrical.Analog.Basic.Capacitor C[dim_vector_lgc](C = Cl);
            Modelica.Electrical.Analog.Basic.Resistor R[lines](R = Rl);
            Modelica.Electrical.Analog.Basic.Conductor G[dim_vector_lgc](G = Gl);
            Modelica.Electrical.Analog.Basic.M_Transformer inductance(N = lines, L = Ll);
            Modelica.Electrical.Analog.Basic.Ground M;

          equation 
            for j in 1:lines - 1 loop
            connect(R[j].p,p[j]);
            connect(R[j].n,inductance.p[j]);
            connect(inductance.n[j],n[j]);
            connect(inductance.n[j],C[1 + (j - 1) * lines - (j - 2) * (j - 1) * 0.5].p);
            connect(C[1 + (j - 1) * lines - (j - 2) * (j - 1) * 0.5].n,M.p);
            connect(inductance.n[j],G[1 + (j - 1) * lines - (j - 2) * (j - 1) * 0.5].p);
            connect(G[1 + (j - 1) * lines - (j - 2) * (j - 1) * 0.5].n,M.p);
            for i in j + 1:lines loop
            connect(inductance.n[j],C[1 + (j - 1) * lines - (j - 2) * (j - 1) * 0.5 + 1 + i - j + 1].p);
            connect(C[1 + (j - 1) * lines - (j - 2) * (j - 1) * 0.5 + 1 + i - j + 1].n,inductance.n[i]);
            connect(inductance.n[j],G[1 + (j - 1) * lines - (j - 2) * (j - 1) * 0.5 + 1 + i - j + 1].p);
            connect(G[1 + (j - 1) * lines - (j - 2) * (j - 1) * 0.5 + 1 + i - j + 1].n,inductance.n[i]);

            end for;

            end for;
            connect(R[lines].p,p[lines]);
            connect(R[lines].n,inductance.p[lines]);
            connect(inductance.n[lines],n[lines]);
            connect(inductance.n[lines],C[dim_vector_lgc].p);
            connect(C[dim_vector_lgc].n,M.p);
            connect(inductance.n[lines],G[dim_vector_lgc].p);
            connect(G[dim_vector_lgc].n,M.p);
            annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{40, -40},{ -40,40}}, lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>The segment model is part of the multiple line model. It describes one line segment as outlined in the M_Oline description. Using the loop possibilities of Modelica it is formulated by connecting components the number of which depends on the number of lines.</p>
</html>"));
          end segment;
          model segment_last "Multiple line last segment model"
            Modelica.Electrical.Analog.Interfaces.PositivePin p[lines] "Positive pin" annotation(Placement(transformation(extent = {{ -40, -10},{ -20,10}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin n[lines] "Negative pin" annotation(Placement(transformation(extent = {{20, -10},{40,10}}, rotation = 0)));
            parameter Integer lines(final min = 1) = 3 "Number of lines";
            parameter Integer dim_vector_lgc = div(lines * (lines + 1), 2) "Length of the vectors for l, g, c";
            parameter Real Rl[lines] = fill(1, lines) "Resistance matrix";
            parameter Real Ll[dim_vector_lgc] = fill(1, dim_vector_lgc) "Inductance matrix";
            Modelica.Electrical.Analog.Basic.Resistor R[lines](R = Rl);
            Modelica.Electrical.Analog.Basic.M_Transformer inductance(N = lines, L = Ll);
            Modelica.Electrical.Analog.Basic.Ground M;

          equation 
            for j in 1:lines - 1 loop
            connect(p[j],inductance.p[j]);
            connect(inductance.n[j],R[j].p);
            connect(R[j].n,n[j]);

            end for;
            connect(p[lines],inductance.p[lines]);
            connect(inductance.n[lines],R[lines].p);
            connect(R[lines].n,n[lines]);
            annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{20, -40},{ -20,40}}, lineColor = {0,0,255})}), Documentation(info = "<html>
<p>The segment_last model is part of the multiple line model. It describes the special  line segment which is used to get the line symmetrical as outlined in the M_Oline description. Using the loop possibilities of Modelica it is formulated by connecting components the number of which depends on the number of lines.</p>
</html>"));
          end segment_last;
          segment s[N - 1](lines = fill(lines, N - 1), dim_vector_lgc = fill(dim_vector_lgc, N - 1), Rl = fill((r * length) / N, N - 1), Ll = fill((l * length) / N, N - 1), Cl = fill((c * length) / N, N - 1), Gl = fill((g * length) / N, N - 1));
          segment s_first(lines = lines, dim_vector_lgc = dim_vector_lgc, Rl = (r * length) / (2 * N), Cl = (c * length) / N, Ll = (l * length) / (2 * N), Gl = (g * length) / N);
          segment_last s_last(lines = lines, Rl = (r * length) / (2 * N), Ll = (l * length) / (2 * N));
          Modelica.Electrical.Analog.Interfaces.PositivePin p[lines] "Positive pin" annotation(Placement(transformation(extent = {{ -100, -80},{ -80,80}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin n[lines] "Negative pin" annotation(Placement(transformation(extent = {{80, -80},{100,80}}, rotation = 0)));

        equation 
          connect(p,s_first.p);
          connect(s_first.n,s[1].p);
          for i in 1:N - 2 loop
          connect(s[i].n,s[i + 1].p);

          end for;
          connect(s[N - 1].n,s_last.p);
          connect(s_last.n,n);
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{60,80},{ -60, -80}}, lineColor = {0,0,255}, fillPattern = FillPattern.Solid, fillColor = {255,255,255}),Line(points = {{40,60},{40,40}}),Line(points = {{40,50},{ -40,50}}),Line(points = {{ -40,60},{ -40,40}}),Line(points = {{40, -40},{40, -60}}),Line(points = {{40, -50},{ -40, -50}}),Line(points = {{ -40, -40},{ -40, -60}}),Line(points = {{40,30},{40,10}}),Line(points = {{40,20},{ -40,20}}),Line(points = {{ -40,30},{ -40,10}}),Line(points = {{0,6},{0, -34}}, color = {0,0,255}, pattern = LinePattern.Dot),Text(extent = {{ -146,135},{154,95}}, textString = "%name", lineColor = {0,0,255})}), Documentation(revisions = "<HTML>
<ul>

<li><i> November 24, 2008   </i> docu added, cosmetic, K. Majetta
       </li>
<li><i>February 26, 2007</i>
       by Kristin Majetta<br>
       initially modelled.</li>
</ul>
</HTML>", info = "<html>
<p><br/><br/>The M_OLine is a multi line model which consists of several segements and several single lines. Each segement consists of resistors and inductors that are connected in series in each single line, and of capacitors and conductors both between the lines and to the ground. The inductors are coupled to each other like in the M_Inductor model. The following picture shows the schematic of a segment with four single lines (lines=4):  </p>
<p align=\"center\"><img src=\"../Images/Electrical/Analog/segment.png\"/> </p>
<p><br/><br/><br/>The complete multi line consists of N segments and an auxiliary segment_last:</p>
<p align=\"center\"><br/>-- segment_1 -- segment_2 -- ... -- segment_N -- segment_last -- </p>
<p><br/><br/><br/>In the picture of the segment can be seen, that a single segment is unsymmetric. Connecting such unsymmetric segments in a series forces also an unsymmetric multi line. To get a symmetric model which is useful for coupling and which guaranties the same pin properties, in the segment_1 only half valued resistors and inductors are used. The remaining resistors and inductors are at the other end of the line within the auxiliary segment_last. For the example with 4 lines the schematic of segment_last is like this:</p>
<p align=\"center\"><img src=\"../Images/Electrical/Analog/segment_last.png\"/> </p>
<p><br/>The number of the capacitors and conductors depends on the number of single lines that are used, because each line is connected to every other line by both a capacitor and a conductor. One line consists of <b>at least two segements</b>. Inside the model M_OLine the model <i>segment</i> is used. This model represents one segment which is build as described above. For modelling the inductances and their mutual couplings the model M_Transformer is used. To fill the resistance vector, resistance values as many as lines are needed, e.g. if there are four lines, four resistances are needed. For example for a microelectronic line of 0.1m lenght, a sensible resistance-vector would be R=[4.76e5, 1.72e5, 1.72e5, 1.72e5]. </p>
<p><br/><br/>Filling the matrixes of the inductances, capacitances and conductances is a bit more complicated, because those components occur also between two lines and not only (like the resistor) in one line. The entries of the matrices are given by the user in form of a vector. The vector length dim_vector_lgc is calculated by <b>dim_vector_lgc = lines*(lines+1)/2</b>. Inside the model a symmetric inductance matrix, a symmetric capacitance matrix and a symmetric conductance matrix are build out of the entries of the vectors given by the user. The way of building is the same for each matrix, so the approach for filling one of the matrices will be shown at an example: </p><p><br/><br/>The number of lines is assumed to be four. To build the matrix, the model needs the values from the main diagonal and from </p><p><br/><br/>the positions that are below the main diagonal. To get the following matrix </p>
<table cellspacing=\"2\" cellpadding=\"0\" border=\"0\"><tr>
<td></td>
<td></td>
<td><p><br/><br/><h4>1</h4></p></td>
<td><p>0.1</p></td>
<td><p>0.2</p></td>
<td><p>0.4</p></td>
</tr>
<tr>
<td><p>Matrix</p></td>
<td><p>=</p></td>
<td><p>0.1</p></td>
<td><p><h4>2</h4></p></td>
<td><p>0.3</p></td>
<td><p>0.5</p></td>
</tr>
<tr>
<td></td>
<td></td>
<td><p><br/><br/>0.2</p></td>
<td><p>0.3</p></td>
<td><p><h4>3</h4></p></td>
<td><p>0.6</p></td>
</tr>
<tr>
<td></td>
<td></td>
<td><p><br/><br/>0.4</p></td>
<td><p>0.5</p></td>
<td><p>0.6</p></td>
<td><p><h4>4</h4></p></td>
</tr>
</table>
<p><br/>the vector with dim_vector_lgc=4*5/2=10 has to appear in the following way: vector = [<b>1</b>, 0.1, 0.2, 0.4, <b>2</b>, 0.3 0.5, <b>3</b>, 0.6, <b>4</b>] </p>
<p>For the example of a microelectronic line of 0.1m lenght, which is used as default example for the M_OLine model, a sensible inductance-matrix would be </p>
<table cellspacing=\"2\" cellpadding=\"0\" border=\"0\"><tr>
<td></td>
<td></td>
<td><p><br/><br/>5.98e-7</p></td>
<td><p>4.44e-7</p></td>
<td><p>4.39e-7</p></td>
<td><p>3.99e-7</p></td>
</tr>
<tr>
<td><p>L</p></td>
<td><p>=</p></td>
<td><p>4.44e-7</p></td>
<td><p>5.81e-7</p></td>
<td><p>4.09e-7</p></td>
<td><p>4.23e-7</p></td>
</tr>
<tr>
<td></td>
<td></td>
<td><p><br/><br/>4.39e-7</p></td>
<td><p>4.09e-7</p></td>
<td><p>5.96e-7</p></td>
<td><p>4.71e-7</p></td>
</tr>
<tr>
<td></td>
<td></td>
<td><p><br/><br/>3.99e-7</p></td>
<td><p>4.23e-7</p></td>
<td><p>4.71e-7</p></td>
<td><p>6.06e-7</p></td>
</tr>
</table>
<p><br/>For the example of a microelectronic line of 0.1m lenght, which is used as default example for the M_OLine model, a sensible capacitance-matrix would be </p>
<table cellspacing=\"2\" cellpadding=\"0\" border=\"0\"><tr>
<td></td>
<td></td>
<td><p><br/><br/>2.38e-11</p></td>
<td><p>1.01e-10</p></td>
<td><p>8.56e-11</p></td>
<td><p>5.09e-12</p></td>
</tr>
<tr>
<td><p>C</p></td>
<td><p>=</p></td>
<td><p>1.01e-10</p></td>
<td><p>2.71e-11</p></td>
<td><p>2.09e-11</p></td>
<td><p>7.16e-11</p></td>
</tr>
<tr>
<td></td>
<td></td>
<td><p><br/><br/>8.56e-11</p></td>
<td><p>2.09e-11</p></td>
<td><p>1.83e-11</p></td>
<td><p>1.23e-10</p></td>
</tr>
<tr>
<td></td>
<td></td>
<td><p><br/><br/>5.09e-12</p></td>
<td><p>7.16e-11</p></td>
<td><p>1.23e-10</p></td>
<td><p>2.07e-11</p></td>
</tr>
</table>
<p><br/>For the example of a microelectronic line of 0.1m lenght, which is used as default example for the M_OLine model, a sensible conductance-matrix would be </p>
<table cellspacing=\"2\" cellpadding=\"0\" border=\"0\"><tr>
<td></td>
<td></td>
<td><p><br/><br/>8.05e-6</p></td>
<td><p>3.42e-5</p></td>
<td><p>2.91e-5</p></td>
<td><p>1.76e-5</p></td>
</tr>
<tr>
<td><p>G</p></td>
<td><p>=</p></td>
<td><p>3.42e-5</p></td>
<td><p>9.16e-6</p></td>
<td><p>5.93e-6</p></td>
<td><p>6.64e-6</p></td>
</tr>
<tr>
<td></td>
<td></td>
<td><p><br/><br/>2.90e-5</p></td>
<td><p>7.08e-6</p></td>
<td><p>5.93e-6</p></td>
<td><p>4.19e-5</p></td>
</tr>
<tr>
<td></td>
<td></td>
<td><p><br/><br/>1.71e-6</p></td>
<td><p>2.43e-5</p></td>
<td><p>4.18e-5</p></td>
<td><p>6.64e-6</p></td>
</tr>
</table>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1e-09, NumberOfIntervals = 2000, Tolerance = 1e-09));
        end M_OLine;
        model ULine "Lossy RC Line"
          Interfaces.Pin p1 annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Interfaces.Pin p2 annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Interfaces.Pin p3 annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}}, rotation = 0)));
          SI.Voltage v13;
          SI.Voltage v23;
          SI.Current i1;
          SI.Current i2;
          parameter Real r(final min = Modelica.Constants.small, unit = "Ohm/m", start = 1) "Resistance per meter";
          parameter Real c(final min = Modelica.Constants.small, unit = "F/m", start = 1) "Capacitance per meter";
          parameter SI.Length length(final min = Modelica.Constants.small, start = 1) "Length of line";
          parameter Integer N(final min = 1, start = 1) "Number of lumped segments";
        protected 
          Basic.Resistor R[N + 1](R = fill((r * length) / (N + 1), N + 1));
          Basic.Capacitor C[N](C = fill((c * length) / N, N));
          annotation(Documentation(info = "<html>
<p>As can be seen in the picture below, the lossy RC line ULine consists of segments of lumped series resistances and capacitances that are connected with the reference pin p3. </p><p>The precision of the model depends on the number N of lumped segments.  </p>
<p>To get a symmetric line model, the first Resistor is cut into two parts (R1 and R_Nplus1)</p><p>The two new resistors have the half of the resistance of the original Resistor had. The Resistor R1</p><p>is at the beginning of the line and R_Nplus1 is at the end of the line. </p>
<p align=\"center\"><img src=\"../Images/Electrical/Analog/ULine.png\"/> </p>
<p>The values of the capacitances are calculated with: C=c*length/N with c given by the user, &QUOT;lenght=length of line&QUOT; and &QUOT;N=number of segments&QUOT;.</p><p>The values of the resistors are calculated with :R=r*length/(N+1).</p><p>For all capacitances and resistors the values of each segment are the same exept</p><p>of the first and last resistor, that only hase the half of the value of the other resistors.</p><p>Note, this is different to the lumped line model of SPICE. </p>
<p><b>References</b> </p>
<dl><dt>Johnson, B.; Quarles, T.; Newton, A. R.; Pederson, D. O.; Sangiovanni-Vincentelli, A.</dt>
<dd>SPICE3 Version 3e User&apos;s Manual (April 1, 1991). Department of Electrical Engineering and Computer Sciences, University of California, Berkley p. 22, p. 124 </dd>
</dl></html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{0, -60},{0, -90}}, color = {0,0,255}),Line(points = {{60,0},{90,0}}, color = {0,0,255}),Line(points = {{ -60,0},{ -90,0}}, color = {0,0,255}),Line(points = {{30,30},{ -30,30}}, color = {0,0,255}),Line(points = {{ -30,40},{ -30,20}}, color = {0,0,255}),Line(points = {{30,40},{30,20}}, color = {0,0,255}),Text(extent = {{ -154,117},{146,77}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,255}),Line(points = {{0, -60},{0, -96}}, color = {0,0,255}),Line(points = {{60,0},{96,0}}, color = {0,0,255}),Line(points = {{ -60,0},{ -96,0}}, color = {0,0,255}),Line(points = {{30,30},{ -30,30}}, color = {0,0,255}),Line(points = {{ -30,40},{ -30,20}}, color = {0,0,255}),Line(points = {{30,40},{30,20}}, color = {0,0,255})}));

        equation 
          v13 = p1.v - p3.v;
          v23 = p2.v - p3.v;
          i1 = p1.i;
          i2 = p2.i;
          connect(p1,R[1].p);
          for i in 1:N loop
          connect(R[i].n,R[i + 1].p);

          end for;
          for i in 1:N loop
          connect(R[i].n,C[i].p);

          end for;
          for i in 1:N loop
          connect(C[i].n,p3);

          end for;
          connect(R[N + 1].n,p2);
        end ULine;
        model TLine1 "Lossless transmission line with characteristic impedance Z0 and transmission delay TD"
          extends Modelica.Electrical.Analog.Interfaces.TwoPort;
          parameter Modelica.SIunits.Resistance Z0(start = 1) "Characteristic impedance";
          parameter Modelica.SIunits.Time TD(start = 1) "Transmission delay";
        protected 
          Modelica.SIunits.Voltage er;
          Modelica.SIunits.Voltage es;
          annotation(Documentation(info = "<html>
<p>Lossless transmission line with characteristic impedance Z0 and transmission delay TD The lossless transmission line TLine1 is a two Port. Both port branches consist of a resistor with characteristic impedance Z0 and a controled voltage source that takes into consideration the transmission delay TD. For further details see Branin&apos;s article below. The model parameters can be derived from inductance and capacitance per length (L&apos; resp. C&apos;), i. e. Z0 = sqrt(L&apos;/C&apos;) and TD = sqrt(L&apos;*C&apos;)*length_of_line. Resistance R&apos; and conductance C&apos; per meter are assumed to be zero.  </p>
<p><b>References:</b> </p>
<dl><dt>Branin Jr., F. H.</dt>
<dd>Transient Analysis of Lossless Transmission Lines. Proceedings of the IEEE 55(1967), 2012 - 2013</dd>
<dt>Hoefer, E. E. E.; Nielinger, H.</dt>
<dd>SPICE : Analyseprogramm fuer elektronische Schaltungen. Springer-Verlag, Berlin, Heidelberg, New York, Tokyo, 1985. </dd>
</dl></html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Joachim Haase<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,255}, fillPattern = FillPattern.Solid, fillColor = {255,255,255}),Line(points = {{60, -50},{90, -50}}, color = {0,0,255}),Line(points = {{60,50},{90,50}}, color = {0,0,255}),Line(points = {{ -60,50},{ -90,50}}, color = {0,0,255}),Line(points = {{ -60, -50},{ -90, -50}}, color = {0,0,255}),Line(points = {{30,30},{ -30,30}}, color = {0,0,255}),Line(points = {{ -30,40},{ -30,20}}, color = {0,0,255}),Line(points = {{30,40},{30,20}}, color = {0,0,255}),Text(extent = {{ -50,0},{50, -20}}, textString = "TLine1", lineColor = {0,0,0}),Text(extent = {{ -152,122},{148,82}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,255}),Line(points = {{60, -50},{96, -50}}, color = {0,0,255}),Line(points = {{60,50},{96,50}}, color = {0,0,255}),Line(points = {{ -60,50},{ -96,50}}, color = {0,0,255}),Line(points = {{ -60, -50},{ -96, -50}}, color = {0,0,255}),Line(points = {{30,30},{ -30,30}}, color = {0,0,255}),Line(points = {{ -30,40},{ -30,20}}, color = {0,0,255}),Line(points = {{30,40},{30,20}}, color = {0,0,255}),Text(extent = {{ -100,100},{100,70}}, textString = "TLine1", lineColor = {0,0,255}),Text(extent = {{ -30,0},{31, -31}}, textString = "TLine1", lineColor = {0,0,255})}));

        equation 
          assert(Z0 > 0, "Z0 has to be positive");
          assert(TD > 0, "TD has to be positive");
          i1 = (v1 - es) / Z0;
          i2 = (v2 - er) / Z0;
          es = 2 * delay(v2, TD) - delay(er, TD);
          er = 2 * delay(v1, TD) - delay(es, TD);
        end TLine1;
        model TLine2 "Lossless transmission line with characteristic impedance Z0, frequency F and normalized length NL"
          extends Modelica.Electrical.Analog.Interfaces.TwoPort;
          parameter Modelica.SIunits.Resistance Z0(start = 1) "Characteristic impedance";
          parameter Modelica.SIunits.Frequency F(start = 1) "Frequency";
          parameter Real NL(start = 1) "Normalized length";
        protected 
          Modelica.SIunits.Voltage er;
          Modelica.SIunits.Voltage es;
          Modelica.SIunits.Time TD;
          annotation(Documentation(info = "<html>
<p>Lossless transmission line with characteristic impedance Z0, frequency F and normalized length NL The lossless transmission line TLine2 is a two Port. Both port branches consist of a resistor with the value of the characteristic impedance Z0 and a controled voltage source that takes into consideration the transmission delay. For further details see Branin&apos;s article below. Resistance R&apos; and conductance C&apos; per meter are assumed to be zero. The characteristic impedance Z0 can be derived from inductance and capacitance per length (L&apos; resp. C&apos;), i. e. Z0 = sqrt(L&apos;/C&apos;). The normalized length NL is equal to the length of the line divided by the wavelength corresponding to the frequency F, i. e. the transmission delay TD is the quotient of NL and F.  </p>
<p><b>References:</b> </p>
<dl><dt>Branin Jr., F. H.</dt>
<dd>Transient Analysis of Lossless Transmission Lines. Proceedings of the IEEE 55(1967), 2012 - 2013</dd>
<dt>Hoefer, E. E. E.; Nielinger, H.</dt>
<dd>SPICE : Analyseprogramm fuer elektronische Schaltungen. Springer-Verlag, Berlin, Heidelberg, New York, Tokyo, 1985. </dd>
</dl></html>", revisions = "<html>
<li><i> 1998   </i>
       by Joachim Haase<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,255}, fillPattern = FillPattern.Solid, fillColor = {255,255,255}),Line(points = {{60, -50},{90, -50}}, color = {0,0,255}),Line(points = {{60,50},{90,50}}, color = {0,0,255}),Line(points = {{ -60,50},{ -90,50}}, color = {0,0,255}),Line(points = {{ -60, -50},{ -90, -50}}, color = {0,0,255}),Line(points = {{30,30},{ -30,30}}, color = {0,0,255}),Line(points = {{ -30,40},{ -30,20}}, color = {0,0,255}),Line(points = {{30,40},{30,20}}, color = {0,0,255}),Text(extent = {{ -51, -10},{50, -31}}, textString = "TLine2", lineColor = {0,0,0}),Text(extent = {{ -148,119},{152,79}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,255}),Line(points = {{60, -50},{96, -50}}, color = {0,0,255}),Line(points = {{60,50},{96,50}}, color = {0,0,255}),Line(points = {{ -60,50},{ -96,50}}, color = {0,0,255}),Line(points = {{ -60, -50},{ -96, -50}}, color = {0,0,255}),Line(points = {{30,30},{ -30,30}}, color = {0,0,255}),Line(points = {{ -30,40},{ -30,20}}, color = {0,0,255}),Line(points = {{30,40},{30,20}}, color = {0,0,255}),Text(extent = {{ -100,100},{100,70}}, textString = "TLine2", lineColor = {0,0,255})}));

        equation 
          assert(Z0 > 0, "Z0 has to be positive");
          assert(NL > 0, "NL has to be positive");
          assert(F > 0, "F  has to be positive");
          TD = NL / F;
          i1 = (v1 - es) / Z0;
          i2 = (v2 - er) / Z0;
          es = 2 * delay(v2, TD) - delay(er, TD);
          er = 2 * delay(v1, TD) - delay(es, TD);
        end TLine2;
        model TLine3 "Lossless transmission line with characteristic impedance Z0 and frequency F"
          extends Modelica.Electrical.Analog.Interfaces.TwoPort;
          parameter Modelica.SIunits.Resistance Z0(start = 1) "Natural impedance";
          parameter Modelica.SIunits.Frequency F(start = 1) "Frequency";
        protected 
          Modelica.SIunits.Voltage er;
          Modelica.SIunits.Voltage es;
          Modelica.SIunits.Time TD;
          annotation(Documentation(info = "<html>
<p>Lossless transmission line with characteristic impedance Z0 and frequency F The lossless transmission line TLine3 is a two Port. Both port branches consist of a resistor with value of the characteristic impedance Z0 and a controled voltage source that takes into consideration the transmission delay. For further details see Branin&apos;s article below. Resistance R&apos; and conductance C&apos; per meter are assumed to be zero. The characteristic impedance Z0 can be derived from inductance and capacitance per length (L&apos; resp. C&apos;), i. e. Z0 = sqrt(L&apos;/C&apos;). The length of the line is equal to a quarter of the wavelength corresponding to the frequency F, i. e. the transmission delay is the quotient of 4 and F. In this case, the caracteristic impedance is called natural impedance.</p>
<p><b>References:</b> </p>
<dl><dt>Branin Jr., F. H.</dt>
<dd>Transient Analysis of Lossless Transmission Lines. Proceedings of the IEEE 55(1967), 2012 - 2013</dd>
<dt>Hoefer, E. E. E.; Nielinger, H.</dt>
<dd>SPICE : Analyseprogramm fuer elektronische Schaltungen. Springer-Verlag, Berlin, Heidelberg, New York, Tokyo, 1985. </dd>
</dl></html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Joachim Haase<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,255}, fillPattern = FillPattern.Solid, fillColor = {255,255,255}),Line(points = {{60, -50},{90, -50}}, color = {0,0,255}),Line(points = {{60,50},{90,50}}, color = {0,0,255}),Line(points = {{ -60,50},{ -90,50}}, color = {0,0,255}),Line(points = {{ -60, -50},{ -90, -50}}, color = {0,0,255}),Line(points = {{30,30},{ -30,30}}, color = {0,0,255}),Line(points = {{ -30,40},{ -30,20}}, color = {0,0,255}),Line(points = {{30,40},{30,20}}, color = {0,0,255}),Text(extent = {{ -50, -10},{51, -30}}, textString = "TLine3", lineColor = {0,0,0}),Text(extent = {{ -155,124},{145,84}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,255}),Line(points = {{60, -50},{96, -50}}, color = {0,0,255}),Line(points = {{60,50},{96,50}}, color = {0,0,255}),Line(points = {{ -60,50},{ -96,50}}, color = {0,0,255}),Line(points = {{ -60, -50},{ -96, -50}}, color = {0,0,255}),Line(points = {{30,30},{ -30,30}}, color = {0,0,255}),Line(points = {{ -30,40},{ -30,20}}, color = {0,0,255}),Line(points = {{30,40},{30,20}}, color = {0,0,255}),Text(extent = {{ -100,100},{100,70}}, textString = "TLine3", lineColor = {0,0,255})}));

        equation 
          assert(Z0 > 0, "Z0 has to be positive");
          assert(F > 0, "F  has to be positive");
          TD = 1 / F / 4;
          i1 = (v1 - es) / Z0;
          i2 = (v2 - er) / Z0;
          es = 2 * delay(v2, TD) - delay(er, TD);
          er = 2 * delay(v1, TD) - delay(es, TD);
        end TLine3;
      end Lines;
      package Interfaces "Connectors and partial models for Analog electrical components"
        extends Modelica.Icons.Library;
        annotation(Documentation(info = "<html>
<p>This package contains connectors and interfaces (partial models) for analog electrical components. The partial models contain typical combinations of pins, and internal variables which are often used. Furthermode, the thermal heat port is in this package which can be included by inheritance. </p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden<br>
<p>
<dt>
</dl>

<b>Copyright:</b>
<dl>
<dd>
Copyright &copy; 1998-2006, Modelica Association and Fraunhofer-Gesellschaft.<br>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package
Modelica in file \"Modelica/package.mo\".</i><br>
<p>
</dl>

<ul>
<li><i> 1998</i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"));
        connector Pin "Pin of an electrical component"
          SI.Voltage v "Potential at the pin";
          flow SI.Current i "Current flowing into the pin";
          annotation(defaultComponentName = "pin", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -160,110},{40,50}}, lineColor = {0,0,255}, textString = "%name")}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>Pin is the basic electric connector. It includes the voltage which consists between the pin and the ground node. The ground node is the node of (any) ground device (Modelica.Electrical.Basic.Ground). Furthermore, the pin includes the current, which is considered to be <b>positive</b> if it is flowing at the pin<b> into the device</b>. </p>
</html>"));
        end Pin;
        connector PositivePin "Positive pin of an electric component"
          SI.Voltage v "Potential at the pin";
          flow SI.Current i "Current flowing into the pin";
          annotation(defaultComponentName = "pin_p", Documentation(info = "<html>
<p>Connectors PositivePin and NegativePin are nearly identical. The only difference is that the icons are different in order to identify more easily the pins of a component. Usually, connector PositivePin is used for the positive and connector NegativePin for the negative pin of an electrical component. </p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -160,110},{40,50}}, lineColor = {0,0,255}, textString = "%name")}));
        end PositivePin;
        connector NegativePin "Negative pin of an electric component"
          SI.Voltage v "Potential at the pin";
          flow SI.Current i "Current flowing into the pin";
          annotation(defaultComponentName = "pin_n", Documentation(info = "<html>
<p>Connectors PositivePin and NegativePin are nearly identical. The only difference is that the icons are different in order to identify more easily the pins of a component. Usually, connector PositivePin is used for the positive and connector NegativePin for the negative pin of an electrical component. </p>
</html>", revisions = "<html>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -40,110},{160,50}}, textString = "%name", lineColor = {0,0,255})}));
        end NegativePin;
        partial model TwoPin "Component with two electrical pins"
          SI.Voltage v "Voltage drop between the two pins (= p.v - n.v)";
          PositivePin p "Positive pin Positive pin (potential p.v > n.v for positive voltage drop v)" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          NegativePin n "Negative pin" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -120,3},{ -110,0},{ -120, -3},{ -120,3}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Line(points = {{ -136,0},{ -111,0}}, color = {160,160,164}),Text(extent = {{ -134,5},{ -117,20}}, lineColor = {160,160,164}, textString = "p.i"),Line(points = {{110,0},{135,0}}, color = {160,160,164}),Polygon(points = {{120,3},{110,0},{120, -3},{120,3}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Text(extent = {{117,3},{134,18}}, lineColor = {160,160,164}, textString = "n.i")}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>TwoPin is a partial model with two pins and one internal variable for the voltage over the two pins. Internal currents are not defined. It is intended to be used in cases where the model which inherits TwoPin is composed by combining other components graphically, not by equations.</p>
</html>"));

        equation 
          v = p.v - n.v;
        end TwoPin;
        partial model OnePort "Component with two electrical pins p and n and current i from p to n"
          SI.Voltage v "Voltage drop between the two pins (= p.v - n.v)";
          SI.Current i "Current flowing from pin p to pin n";
          PositivePin p "Positive pin (potential p.v > n.v for positive voltage drop v)" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          NegativePin n "Negative pin" annotation(Placement(transformation(extent = {{110, -10},{90,10}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>Superclass of elements which have <b>two</b> electrical pins: the positive pin connector <i>p</i>, and the negative pin connector <i>n</i>. It is assumed that the current flowing into pin p is identical to the current flowing out of pin n. This current is provided explicitly as current i.  </p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -110,20},{ -85,20}}, color = {160,160,164}),Polygon(points = {{ -95,23},{ -85,20},{ -95,17},{ -95,23}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Line(points = {{90,20},{115,20}}, color = {160,160,164}),Line(points = {{ -125,0},{ -115,0}}, color = {160,160,164}),Line(points = {{ -120, -5},{ -120,5}}, color = {160,160,164}),Text(extent = {{ -110,25},{ -90,45}}, lineColor = {160,160,164}, textString = "i"),Polygon(points = {{105,23},{115,20},{105,17},{105,23}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Line(points = {{115,0},{125,0}}, color = {160,160,164}),Text(extent = {{90,45},{110,25}}, lineColor = {160,160,164}, textString = "i")}));

        equation 
          v = p.v - n.v;
          0 = p.i + n.i;
          i = p.i;
        end OnePort;
        partial model TwoPort "Component with two electrical ports, including current"
          SI.Voltage v1 "Voltage drop over the left port";
          SI.Voltage v2 "Voltage drop over the right port";
          SI.Current i1 "Current flowing from pos. to neg. pin of the left port";
          SI.Current i2 "Current flowing from pos. to neg. pin of the right port";
          PositivePin p1 "Positive pin of the left port (potential p1.v > n1.v for positive voltage drop v1)" annotation(Placement(transformation(extent = {{ -110,40},{ -90,60}}, rotation = 0)));
          NegativePin n1 "Negative pin of the left port" annotation(Placement(transformation(extent = {{ -90, -60},{ -110, -40}}, rotation = 0)));
          PositivePin p2 "Positive pin of the right port (potential p2.v > n2.v for positive voltage drop v2)" annotation(Placement(transformation(extent = {{110,40},{90,60}}, rotation = 0)));
          NegativePin n2 "Negative pin of the right port" annotation(Placement(transformation(extent = {{90, -60},{110, -40}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -120,53},{ -110,50},{ -120,47},{ -120,53}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Line(points = {{ -136,50},{ -111,50}}, color = {160,160,164}),Polygon(points = {{127, -47},{137, -50},{127, -53},{127, -47}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Line(points = {{111, -50},{136, -50}}, color = {160,160,164}),Text(extent = {{112, -44},{128, -29}}, lineColor = {160,160,164}, textString = "i2"),Text(extent = {{118,52},{135,67}}, lineColor = {0,0,0}, textString = "i2"),Polygon(points = {{120,53},{110,50},{120,47},{120,53}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {160,160,164}),Line(points = {{111,50},{136,50}}, color = {0,0,0}),Line(points = {{ -136, -49},{ -111, -49}}, color = {160,160,164}),Polygon(points = {{ -126, -46},{ -136, -49},{ -126, -52},{ -126, -46}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Text(extent = {{ -127, -46},{ -110, -31}}, lineColor = {160,160,164}, textString = "i1"),Text(extent = {{ -136,53},{ -119,68}}, lineColor = {160,160,164}, textString = "i1")}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>TwoPort is a partial model that consists of two ports. Like OnePort each port has two pins. It is assumed that the current flowing into the positive  pin   is identical to the current flowing out of pin n. This currents of each port are  provided explicitly as currents i1 and i2, the voltages respectively as v1 and v2.</p>
</html>"));

        equation 
          v1 = p1.v - n1.v;
          v2 = p2.v - n2.v;
          0 = p1.i + n1.i;
          0 = p2.i + n2.i;
          i1 = p1.i;
          i2 = p2.i;
        end TwoPort;
        partial model ConditionalHeatPort "Partial model to include a conditional HeatPort in order to describe the power loss via a thermal network"
          parameter Boolean useHeatPort = false "=true, if HeatPort is enabled" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
          parameter Modelica.SIunits.Temperature T = 293.15 "Fixed device temperature if useHeatPort = false" annotation(Dialog(enable = not useHeatPort));
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort(T(start = T) = T_heatPort, Q_flow =  -LossPower) if useHeatPort annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}}), iconTransformation(extent = {{ -10, -110},{10, -90}})));
          Modelica.SIunits.Power LossPower "Loss power leaving component via HeatPort";
          Modelica.SIunits.Temperature T_heatPort "Temperature of HeatPort";

        equation 
if not useHeatPort then
                      T_heatPort = T;
          end if;
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(revisions = "<html>
<ul>
<li><i> February 17, 2009   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>This partial model provides a conditional heating port for the connection to a thermal network. </p>
<p><ul>
<li>If <b>useHeatPort</b> is set to <b>false</b> (default), no heat port is available, and the thermal loss power flows internally to the ground. In this case, the parameter <b>T</b> specifies the fixed device temperature (the default for T = 20oC) </li>
<p>. </p>
<li>If <b>useHeatPort</b> is set to <b>true</b>, a heat port is available. </li>
</ul></p>
<p>If this model is used, the loss power has to be provided by an equation in the model which inherits from ConditionalHeatingPort model (<b>lossPower = ...</b>). As device temperature <b>T_heatPort</b> can be used to describe the influence of the device temperature on the model behaviour. </p>
</html>"));
        end ConditionalHeatPort;
        partial model AbsoluteSensor "Base class to measure the absolute value of a pin variable"
          extends Modelica.Icons.RotationalSensor;
          Interfaces.PositivePin p "Pin to be measured" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput y "Measured quantity as Real output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{70,0},{100,0}}, color = {0,0,255}),Text(extent = {{ -150,80},{150,120}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -96,0}}, color = {0,0,0}),Line(points = {{70,0},{100,0}}, color = {0,0,255})}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>The AbsoluteSensor is a partial model for converting values that can be calculated from one pin connector into a real valued signal. The special calculation has to be described in the model which inherits the AbsoluteSensor.  It is often used in sensor devices. To be a true sensor the modeller has to take care that the sensor model does not influence the electrical behavior to be measured.</p>
</html>"));
        end AbsoluteSensor;
        partial model RelativeSensor "Base class to measure a relative variable between two pins"
          extends Modelica.Icons.RotationalSensor;
          Interfaces.PositivePin p "Positive pin" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Interfaces.NegativePin n "Negative pin" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput y "Measured quantity as Real output signal" annotation(Placement(transformation(origin = {0, -100}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{70,0},{90,0}}, color = {0,0,0}),Line(points = {{0, -90},{0, -70}}, color = {0,0,255}),Text(extent = {{ -150,80},{150,120}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -70,0},{ -96,0}}, color = {0,0,0}),Line(points = {{0, -90},{0, -70}}, color = {0,0,255}),Line(points = {{70,0},{96,0}}, color = {0,0,0})}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>The RelaticeSensor is a partial model for converting values that can be calculated from two pin connectors into a real valued signal. The special calculation has to be described in the model which inherits the RelativeSensor.  It is often used in sensor devices. To be a true sensor the modeller has to take care that the sensor model does not influence the electrical behavior to be measured.</p>
</html>"));
        end RelativeSensor;
        partial model VoltageSource "Interface for voltage sources"
          extends OnePort;
          parameter SI.Voltage offset = 0 "Voltage offset";
          parameter SI.Time startTime = 0 "Time offset";
          replaceable Modelica.Blocks.Interfaces.SignalSource signalSource(final offset = offset, final startTime = startTime) annotation(Placement(transformation(extent = {{70,70},{90,90}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,80},{150,120}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -90,0},{90,0}}, color = {0,0,0}),Text(extent = {{ -120,50},{ -20,0}}, lineColor = {0,0,255}, textString = "+"),Text(extent = {{20,50},{120,0}}, lineColor = {0,0,255}, textString = "-")}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>The VoltageSource partial model prepares voltage sources by providing the pins, and the offset and startTime parameters, which are the same at all voltage sources. The source behavior is taken from Modelica.Blocks signal sources by inheritance and usage of the replacable possibilities.</p>
</html>"));

        equation 
          v = signalSource.y;
        end VoltageSource;
        partial model CurrentSource "Interface for current sources"
          extends OnePort;
          parameter SI.Current offset = 0 "Current offset";
          parameter SI.Time startTime = 0 "Time offset";
          replaceable Modelica.Blocks.Interfaces.SignalSource signalSource(final offset = offset, final startTime = startTime) annotation(Placement(transformation(extent = {{70,69},{91,89}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{ -50,0}}, color = {0,0,0}),Line(points = {{50,0},{90,0}}, color = {0,0,0}),Line(points = {{0, -50},{0,50}}, color = {0,0,0}),Text(extent = {{ -150,120},{150,80}}, textString = "%name", lineColor = {0,0,255}),Polygon(points = {{90,0},{60,10},{60, -10},{90,0}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid)}), Documentation(revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>", info = "<html>
<p>The CurrentSource partial model prepares current sources by providing the pins, and the offset and startTime parameters, which are the same at all current sources. The source behavior is taken from Modelica.Blocks signal sources by inheritance and usage of the replacable possibilities.</p>
</html>"));

        equation 
          i = signalSource.y;
        end CurrentSource;
      end Interfaces;
      package Ideal "Ideal electrical elements such as switches, diode, transformer, operational amplifier"
        extends Modelica.Icons.Library;
        annotation(Documentation(info = "<html>
<p>This package contains electrical components with idealized behaviour. To enable more realistic applications than it is possible with pure realistic behavior some components are improved by additional features. E.g. the switches have resistances for the open or close case which can be parametrized.</p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden<br>
<p>
<dt>
<b>Copyright:</b>
<dd>
Copyright &copy; 1998-2006, Modelica Association and Fraunhofer-Gesellschaft.<br>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package
Modelica in file \"Modelica/package.mo\".</i><br>
<p>
</dl>
</html>"));
        model IdealThyristor "Ideal thyristor"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter Modelica.SIunits.Resistance Ron(final min = 0) = 1e-05 "Closed thyristor resistance";
          parameter Modelica.SIunits.Conductance Goff(final min = 0) = 1e-05 "Opened thyristor conductance";
          parameter Modelica.SIunits.Voltage Vknee(final min = 0, start = 0) "Forward threshold voltage";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Boolean off(start = true) "Switching state";
          Modelica.Blocks.Interfaces.BooleanInput fire annotation(Placement(transformation(origin = {70,110}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
        protected 
          Real s(final unit = "1") "Auxiliary variable: if on then current, if opened then voltage";
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(HideResult = true);
          annotation(Documentation(info = "<html>
This is an ideal thyristor model which is<br><br>
<b>open </b>(off), if the voltage drop is less than 0 or fire is false<br>
<b>closed</b> (on), if the voltage drop is greater or equal 0  and fire is true.<br>
</P>
<p>
This is the behaviour if all parameters are exactly zero.<br><br>
Note, there are circuits, where this ideal description
with zero resistance and zero cinductance is not possible.
In order to prevent singularities during switching, the opened
thyristor has a small conductance <i>Goff</i> and the closed thyristor has a low
resistance <i>Ron</i> which is default.
</P>
<P>
The parameter <i>Vknee</i> which is the forward threshold voltage, allows to displace
the knee point <br>
along  the <i>Goff</i>-characteristic until <i>v = Vknee</i>.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</p>
</HTML>
", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>Mai 7, 2004   </i>
       by Christoph Clauss and Anton Haumer<br> Vknee added<br>
       </li>
<li><i>some years ago   </i>
       by Christoph Clauss<br> realized<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -150, -47},{150, -87}}, textString = "%name", lineColor = {0,0,255}),Polygon(points = {{30,0},{ -30,40},{ -30, -40},{30,0}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{40,0}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255}),Line(points = {{30,40},{30, -40}}, color = {0,0,255}),Line(points = {{30,20},{70,60},{70,90}}, color = {0,0,255}),Line(points = {{40,50},{60,30}}, color = {0,0,255}),Line(visible = useHeatPort, points = {{0, -100},{0, -20}}, color = {127,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80,0},{80,0}}, color = {128,128,128}),Polygon(points = {{70,4},{80,0},{70, -4},{70,4}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{0,80},{0, -80}}, color = {128,128,128}),Polygon(points = {{ -4,70},{0,80},{4,70},{ -4,70}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{10,80},{20,70}}, lineColor = {128,128,128}, textString = "i"),Text(extent = {{70, -10},{80, -20}}, lineColor = {128,128,128}, textString = "v"),Line(points = {{ -80, -40},{ -20, -10},{20,10},{40,70}}, color = {0,0,0}, thickness = 0.5),Line(points = {{20,9},{20,0}}, color = {128,128,128}, pattern = LinePattern.Dot),Text(extent = {{20,0},{40, -10}}, lineColor = {128,128,128}, pattern = LinePattern.Dot, textString = "Vknee"),Text(extent = {{20,70},{40,60}}, lineColor = {128,128,128}, pattern = LinePattern.Dot, textString = "Ron"),Text(extent = {{ -20,10},{0,0}}, lineColor = {128,128,128}, pattern = LinePattern.Dot, textString = "Goff"),Ellipse(extent = {{18,12},{22,8}}, pattern = LinePattern.Dot, lineColor = {0,0,255}),Line(points = {{20,10},{70,40}}, color = {0,0,0}, thickness = 0.5)}));

        equation 
          off = s < 0 or pre(off) and not fire;
          v = s * unitCurrent * (if off then 1 else Ron) + Vknee;
          i = s * unitVoltage * (if off then Goff else 1) + Goff * Vknee;
          LossPower = v * i;
        end IdealThyristor;
        model IdealGTOThyristor "Ideal GTO thyristor"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter Modelica.SIunits.Resistance Ron(final min = 0) = 1e-05 "Closed thyristor resistance";
          parameter Modelica.SIunits.Conductance Goff(final min = 0) = 1e-05 "Opened thyristor conductance";
          parameter Modelica.SIunits.Voltage Vknee(final min = 0, start = 0) "Forward threshold voltage";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Boolean off(start = true) "Switching state";
          Modelica.Blocks.Interfaces.BooleanInput fire annotation(Placement(transformation(origin = {70,110}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
        protected 
          Real s(final unit = "1") "Auxiliary variable: if on then current, if opened then voltage";
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(HideResult = true);
          annotation(Documentation(info = "<html>
This is an ideal GTO thyristor model which is<br><br>
<b>open </b>(off), if the voltage drop is less than 0 or fire is false<br>
<b>closed</b> (on), if the voltage drop is greater or equal 0  and fire is true.<br>
</P>
<p>
This is the behaviour if all parameters are exactly zero.<br><br>
Note, there are circuits, where this ideal description
with zero resistance and zero cinductance is not possible.
In order to prevent singularities during switching, the opened
thyristor has a small conductance <i>Goff</i> and the closed thyristor has a low
resistance <i>Ron</i> which is default.
</P>
<P>
The parameter <i>Vknee</i> which is the forward threshold voltage, allows to displace
the knee point <br>
along  the <i>Goff</i>-characteristic until <i>v = Vknee</i>.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</p>
</HTML>
", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>Mai 7, 2004   </i>
       by Christoph Clauss and Anton Haumer<br> Vknee added<br>
       </li>
<li><i>some years ago   </i>
       by Christoph Clauss<br> realized<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{30,0},{ -30,40},{ -30, -40},{30,0}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{40,0}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255}),Line(points = {{30,40},{30, -40}}, color = {0,0,255}),Line(points = {{30,20},{70,60},{70,90}}, color = {0,0,255}),Line(points = {{40,50},{60,30}}, color = {0,0,255}),Line(visible = useHeatPort, points = {{0, -100},{0, -20}}, color = {127,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot),Text(extent = {{ -149, -43},{151, -83}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80,0},{80,0}}, color = {128,128,128}),Polygon(points = {{70,4},{80,0},{70, -4},{70,4}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{0,80},{0, -80}}, color = {128,128,128}),Polygon(points = {{ -4,70},{0,80},{4,70},{ -4,70}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{10,80},{20,70}}, lineColor = {128,128,128}, textString = "i"),Text(extent = {{70, -10},{80, -20}}, lineColor = {128,128,128}, textString = "v"),Line(points = {{ -80, -40},{ -20, -10},{20,10},{40,70}}, color = {0,0,0}, thickness = 0.5),Line(points = {{20,9},{20,0}}, color = {128,128,128}, pattern = LinePattern.Dot),Text(extent = {{20,0},{40, -10}}, lineColor = {128,128,128}, pattern = LinePattern.Dot, textString = "Vknee"),Text(extent = {{20,70},{40,60}}, lineColor = {128,128,128}, pattern = LinePattern.Dot, textString = "Ron"),Text(extent = {{ -20,10},{0,0}}, lineColor = {128,128,128}, pattern = LinePattern.Dot, textString = "Goff"),Ellipse(extent = {{18,12},{22,8}}, pattern = LinePattern.Dot, lineColor = {0,0,255}),Line(points = {{20,10},{70,40}}, color = {0,0,0}, thickness = 0.5)}));

        equation 
          off = s < 0 or not fire;
          v = s * unitCurrent * (if off then 1 else Ron) + Vknee;
          i = s * unitVoltage * (if off then Goff else 1) + Goff * Vknee;
          LossPower = v * i;
        end IdealGTOThyristor;
        model IdealCommutingSwitch "Ideal commuting switch"
          parameter SI.Resistance Ron(final min = 0) = 1e-05 "Closed switch resistance";
          parameter SI.Conductance Goff(final min = 0) = 1e-05 "Opened switch conductance";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Interfaces.PositivePin p annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Interfaces.NegativePin n2 annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Interfaces.NegativePin n1 annotation(Placement(transformation(extent = {{90,40},{110,60}}, rotation = 0)));
          Modelica.Blocks.Interfaces.BooleanInput control "true => p--n2 connected, false => p--n1 connected" annotation(Placement(transformation(origin = {0,80}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
        protected 
          Real s1(final unit = "1");
          Real s2(final unit = "1") "Auxiliary variables";
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(HideResult = true);
          annotation(Documentation(info = "<HTML>
<P>
The commuting switch has a positive pin p and two negative pins n1 and n2.
The switching behaviour is controlled
by the inpug signal control. If control is true, the pin p is connected
with the negative pin n2. Otherwise, the pin p is connected to the negative pin n1.
</P>
<P>
In order to prevent singularities during switching, the opened
switch has a (very low) conductance Goff
and the closed switch has a (very low) resistance Ron.
The limiting case is also allowed, i.e., the resistance Ron of the
closed switch could be exactly zero and the conductance Goff of the
open switch could be also exactly zero. Note, there are circuits,
where a description with zero Ron or zero Goff is not possible.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</P>
</HTML>
", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -44,4},{ -36, -4}}, lineColor = {0,0,255}),Text(extent = {{ -80,0},{ -60,22}}, textString = "p", lineColor = {0,0,255}),Text(extent = {{60,50},{80,72}}, textString = "n1", lineColor = {0,0,255}),Text(extent = {{60,0},{80,22}}, textString = "n2", lineColor = {0,0,255}),Line(points = {{ -90,0},{ -44,0}}, color = {0,0,255}),Line(points = {{ -37,2},{40,50}}, color = {0,0,255}),Line(points = {{40,50},{90,50}}, color = {0,0,255}),Line(points = {{0,90},{0,25}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255}),Line(visible = useHeatPort, points = {{0, -100},{0,25}}, color = {127,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot),Text(extent = {{ -148, -22},{152, -62}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -44,4},{ -36, -4}}, lineColor = {0,0,255}),Line(points = {{ -96,0},{ -44,0}}, color = {0,0,255}),Line(points = {{ -37,2},{40,50}}, color = {0,0,255}),Line(points = {{40,50},{96,50}}, color = {0,0,255}),Line(points = {{0,60},{0,25}}, color = {0,0,255}),Line(points = {{40,0},{96,0}}, color = {0,0,255})}));

        equation 
          0 = p.i + n2.i + n1.i;
          p.v - n1.v = s1 * unitCurrent * (if control then 1 else Ron);
          n1.i =  -s1 * unitVoltage * (if control then Goff else 1);
          p.v - n2.v = s2 * unitCurrent * (if control then Ron else 1);
          n2.i =  -s2 * unitVoltage * (if control then 1 else Goff);
          LossPower = p.i * p.v + n1.i * n1.v + n2.i * n2.v;
        end IdealCommutingSwitch;
        model IdealIntermediateSwitch "Ideal intermediate switch"
          parameter SI.Resistance Ron(final min = 0) = 1e-05 "Closed switch resistance";
          parameter SI.Conductance Goff(final min = 0) = 1e-05 "Opened switch conductance";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Interfaces.PositivePin p1 annotation(Placement(transformation(extent = {{ -110,40},{ -90,60}}, rotation = 0)));
          Interfaces.PositivePin p2 annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Interfaces.NegativePin n1 annotation(Placement(transformation(extent = {{90,40},{110,60}}, rotation = 0)));
          Interfaces.NegativePin n2 annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.BooleanInput control "true => p1--n2, p2--n1 connected, otherwise p1--n1, p2--n2  connected" annotation(Placement(transformation(origin = {0,80}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
        protected 
          Real s1(final unit = "1");
          Real s2(final unit = "1");
          Real s3(final unit = "1");
          Real s4(final unit = "1") "Auxiliary variables";
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(HideResult = true);
          annotation(Documentation(info = "<HTML>
<P>
The intermediate switch has four switching contact pins p1, p2, n1, and n2.
The switching behaviour is controlled by the input signal control. If control
is true, the pin p1 is connected to pin n2, and the pin p2 is
connected to the pin n2. Otherwise, the pin p1 is connected to n1, and
p2 is connected to n2.
</P>
<IMG SRC=\"../Images/IdealIntermediateSwitch1.png\" ALT=\"IdealIntermediateSwitch1\">
<P>
In order to prevent singularities during switching, the opened
switch has a (very low) conductance Goff
and the closed switch has a (very low) resistance Ron.
</P>
<IMG SRC=\"../Images/IdealIntermediateSwitch2.png\" ALT=\"IdealIntermediateSwitch2\">
<P>
The limiting case is also allowed, i.e., the resistance Ron of the
closed switch could be exactly zero and the conductance Goff of the
open switch could be also exactly zero. Note, there are circuits,
where a description with zero Ron or zero Goff is not possible.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</P>
</HTML>
", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -4,30},{4,22}}, lineColor = {0,0,255}),Text(extent = {{ -80,50},{ -60,72}}, textString = "p1", lineColor = {0,0,255}),Text(extent = {{ -80,0},{ -60,22}}, textString = "p2", lineColor = {0,0,255}),Text(extent = {{60,50},{80,72}}, textString = "n1", lineColor = {0,0,255}),Text(extent = {{60,0},{80,22}}, textString = "n2", lineColor = {0,0,255}),Line(points = {{ -90,0},{ -44,0}}, color = {0,0,255}),Line(points = {{ -90,50},{ -44,50}}, color = {0,0,255}),Line(points = {{ -44,0},{40,50}}, color = {0,0,255}),Line(points = {{ -44,50},{40,0}}, color = {0,0,255}),Line(points = {{40,50},{90,50}}, color = {0,0,255}),Line(points = {{0,90},{0,25}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255}),Text(extent = {{ -151, -24},{149, -64}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -4,29},{4,21}}, lineColor = {0,0,255}),Line(points = {{ -96,0},{ -40,0}}, color = {0,0,255}),Line(points = {{ -96,50},{ -40,50}}, color = {0,0,255}),Line(points = {{ -40,0},{40,50}}, color = {0,0,255}),Line(points = {{ -40,50},{40,0}}, color = {0,0,255}),Line(points = {{40,50},{96,50}}, color = {0,0,255}),Line(points = {{0,60},{0,25}}, color = {0,0,255}),Line(points = {{40,0},{96,0}}, color = {0,0,255})}));

        equation 
          p1.v - n1.v = s1 * unitCurrent * (if control then 1 else Ron);
          p2.v - n2.v = s2 * unitCurrent * (if control then 1 else Ron);
          p1.v - n2.v = s3 * unitCurrent * (if control then Ron else 1);
          p2.v - n1.v = s4 * unitCurrent * (if control then Ron else 1);
          p1.i = if control then s1 * unitVoltage * Goff + s3 * unitCurrent else s1 * unitCurrent + s3 * unitVoltage * Goff;
          p2.i = if control then s2 * unitVoltage * Goff + s4 * unitCurrent else s2 * unitCurrent + s4 * unitVoltage * Goff;
          n1.i = if control then  -s1 * unitVoltage * Goff - s4 * unitCurrent else  -s1 * unitCurrent - s4 * unitVoltage * Goff;
          n2.i = if control then  -s2 * unitVoltage * Goff - s3 * unitCurrent else  -s2 * unitCurrent - s3 * unitVoltage * Goff;
          LossPower = p1.i * p1.v + p2.i * p2.v + n1.i * n1.v + n2.i * n2.v;
        end IdealIntermediateSwitch;
        model ControlledIdealCommutingSwitch "Controlled ideal commuting switch"
          parameter SI.Voltage level = 0.5 "Switch level";
          parameter SI.Resistance Ron(final min = 0) = 1e-05 "Closed switch resistance";
          parameter SI.Conductance Goff(final min = 0) = 1e-05 "Opened switch conductance";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Interfaces.PositivePin p annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Interfaces.NegativePin n2 annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Interfaces.NegativePin n1 annotation(Placement(transformation(extent = {{90,40},{110,60}}, rotation = 0)));
          Interfaces.Pin control "Control pin: if control.v > level p--n2 connected, otherwise p--n1 connected" annotation(Placement(transformation(origin = {0,100}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        protected 
          Real s1(final unit = "1");
          Real s2(final unit = "1") "Auxiliary variables";
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(HideResult = true);
          annotation(Documentation(info = "<HTML>
<P>
The commuting switch has a positive pin p and two negative pins n1 and n2.
The switching behaviour is controlled
by the control pin. If its voltage exceeds the value of the parameter level,
the pin p is connected with the negative pin n2. Otherwise, the pin p is
connected the negative pin n1.
</P>
<P>
In order to prevent singularities during switching, the opened
switch has a (very low) conductance Goff
and the closed switch has a (very low) resistance Ron.
The limiting case is also allowed, i.e., the resistance Ron of the
closed switch could be exactly zero and the conductance Goff of the
open switch could be also exactly zero. Note, there are circuits,
where a description with zero Ron or zero Goff is not possible.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</P>
</HTML>
", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -44,4},{ -36, -4}}, lineColor = {0,0,255}),Text(extent = {{ -80,0},{ -60,22}}, textString = "p", lineColor = {0,0,255}),Text(extent = {{60,50},{80,72}}, textString = "n1", lineColor = {0,0,255}),Text(extent = {{60,0},{80,22}}, textString = "n2", lineColor = {0,0,255}),Line(points = {{ -90,0},{ -44,0}}, color = {0,0,255}),Line(points = {{ -37,2},{40,50}}, color = {0,0,255}),Line(points = {{40,50},{90,50}}, color = {0,0,255}),Line(points = {{0,90},{0,25}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255}),Line(visible = useHeatPort, points = {{0, -99},{0,25}}, color = {127,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot),Text(extent = {{ -145, -21},{155, -61}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -44,4},{ -36, -4}}, lineColor = {0,0,255}),Line(points = {{ -96,0},{ -44,0}}, color = {0,0,255}),Line(points = {{ -37,2},{40,50}}, color = {0,0,255}),Line(points = {{40,50},{96,50}}, color = {0,0,255}),Line(points = {{0,96},{0,25}}, color = {0,0,255}),Line(points = {{40,0},{96,0}}, color = {0,0,255})}));

        equation 
          control.i = 0;
          0 = p.i + n2.i + n1.i;
          p.v - n1.v = s1 * unitCurrent * (if control.v > level then 1 else Ron);
          n1.i =  -s1 * unitVoltage * (if control.v > level then Goff else 1);
          p.v - n2.v = s2 * unitCurrent * (if control.v > level then Ron else 1);
          n2.i =  -s2 * unitVoltage * (if control.v > level then 1 else Goff);
          LossPower = p.i * p.v + n1.i * n1.v + n2.i * n2.v;
        end ControlledIdealCommutingSwitch;
        model ControlledIdealIntermediateSwitch "Controlled ideal intermediate switch"
          parameter SI.Voltage level = 0.5 "Switch level";
          parameter SI.Resistance Ron(final min = 0) = 1e-05 "Closed switch resistance";
          parameter SI.Conductance Goff(final min = 0) = 1e-05 "Opened switch conductance";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Interfaces.PositivePin p1 annotation(Placement(transformation(extent = {{ -110,40},{ -90,60}}, rotation = 0)));
          Interfaces.PositivePin p2 annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Interfaces.NegativePin n1 annotation(Placement(transformation(extent = {{90,40},{110,60}}, rotation = 0)));
          Interfaces.NegativePin n2 annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Interfaces.Pin control "Control pin: if control.v > level p1--n2, p2--n1 connected,
         otherwise p1--n1, p2--n2  connected" annotation(Placement(transformation(origin = {0,100}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        protected 
          Real s1(final unit = "1");
          Real s2(final unit = "1");
          Real s3(final unit = "1");
          Real s4(final unit = "1") "Auxiliary variables";
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(HideResult = true);
          annotation(Documentation(info = "<HTML>
<P>
The intermediate switch has four switching contact pins p1, p2, n1, and n2.
The switching behaviour is controlled
by the control pin. If its voltage exceeds the value of the
parameter level, the pin p1 is connected to pin n2, and the pin p2 is
connected to the pin n2. Otherwise, the pin p1 is connected to n1, and
p2 is connected to n2.
</P>
<IMG SRC=\"../Images/ControlledIdealIntermediateSwitch1.png\" ALT=\"ControlledIdealIntermediateSwitch1\">
<P>
In order to prevent singularities during switching, the opened
switch has a (very low) conductance Goff
and the closed switch has a (very low) resistance Ron.
</P>
<IMG SRC=\"../Images/ControlledIdealIntermediateSwitch2.png\" ALT=\"ControlledIdealIntermediateSwitch2\">
<P>
The limiting case is also allowed, i.e., the resistance Ron of the
closed switch could be exactly zero and the conductance Goff of the
open switch could be also exactly zero. Note, there are circuits,
where a description with zero Ron or zero Goff is not possible.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</P>
</HTML>
", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -4,30},{4,22}}, lineColor = {0,0,255}),Text(extent = {{ -80,50},{ -60,72}}, textString = "p1", lineColor = {0,0,255}),Text(extent = {{ -80,0},{ -60,22}}, textString = "p2", lineColor = {0,0,255}),Text(extent = {{60,50},{80,72}}, textString = "n1", lineColor = {0,0,255}),Text(extent = {{60,0},{80,22}}, textString = "n2", lineColor = {0,0,255}),Line(points = {{ -90,0},{ -40,0}}, color = {0,0,255}),Line(points = {{ -90,50},{ -39,50}}, color = {0,0,255}),Line(points = {{ -40,0},{40,50}}, color = {0,0,255}),Line(points = {{ -40,50},{40,0}}, color = {0,0,255}),Line(points = {{40,50},{90,50}}, color = {0,0,255}),Line(points = {{0,90},{0,25}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255}),Line(visible = useHeatPort, points = {{0, -100},{0,22}}, color = {127,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot),Text(extent = {{ -150, -23},{150, -63}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -4,29},{4,21}}, lineColor = {0,0,255}),Line(points = {{ -96,0},{ -40,0}}, color = {0,0,255}),Line(points = {{ -96,50},{ -40,50}}, color = {0,0,255}),Line(points = {{ -40,0},{40,50}}, color = {0,0,255}),Line(points = {{ -40,50},{40,0}}, color = {0,0,255}),Line(points = {{40,50},{96,50}}, color = {0,0,255}),Line(points = {{0,96},{0,25}}, color = {0,0,255}),Line(points = {{40,0},{96,0}}, color = {0,0,255})}));

        equation 
          control.i = 0;
          p1.v - n1.v = s1 * unitCurrent * (if control.v > level then 1 else Ron);
          p2.v - n2.v = s2 * unitCurrent * (if control.v > level then 1 else Ron);
          p1.v - n2.v = s3 * unitCurrent * (if control.v > level then Ron else 1);
          p2.v - n1.v = s4 * unitCurrent * (if control.v > level then Ron else 1);
          p1.i = if control.v > level then s1 * unitVoltage * Goff + s3 * unitCurrent else s1 * unitCurrent + s3 * unitVoltage * Goff;
          p2.i = if control.v > level then s2 * unitVoltage * Goff + s4 * unitCurrent else s2 * unitCurrent + s4 * unitVoltage * Goff;
          n1.i = if control.v > level then  -s1 * unitVoltage * Goff - s4 * unitCurrent else  -s1 * unitCurrent - s4 * unitVoltage * Goff;
          n2.i = if control.v > level then  -s2 * unitVoltage * Goff - s3 * unitCurrent else  -s2 * unitCurrent - s3 * unitVoltage * Goff;
          LossPower = p1.i * p1.v + p2.i * p2.v + n1.i * n1.v + n2.i * n2.v;
        end ControlledIdealIntermediateSwitch;
        model IdealOpAmp "Ideal operational amplifier (norator-nullator pair)"
          SI.Voltage v1 "Voltage drop over the left port";
          SI.Voltage v2 "Voltage drop over the right port";
          SI.Current i1 "Current flowing from pos. to neg. pin of the left port";
          SI.Current i2 "Current flowing from pos. to neg. pin of the right port";
          Interfaces.PositivePin p1 "Positive pin of the left port" annotation(Placement(transformation(extent = {{ -110, -60},{ -90, -40}}, rotation = 0)));
          Interfaces.NegativePin n1 "Negative pin of the left port" annotation(Placement(transformation(extent = {{ -110,40},{ -90,60}}, rotation = 0)));
          Interfaces.PositivePin p2 "Positive pin of the right port" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Interfaces.NegativePin n2 "Negative pin of the right port" annotation(Placement(transformation(origin = {0, -100}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
          annotation(Documentation(info = "<html>
<P>
The ideal OpAmp is a two-port. The left port is fixed to <i>v1=0</i> and <i>i1=0</i>
(nullator). At the right port both any voltage <i>v2</i> and any current <i>i2</i>
are possible (norator).
</P>
</HTML>
", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{60,0},{ -60,70},{ -60, -70},{60,0}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -90,50},{ -60,50}}, color = {0,0,255}),Line(points = {{ -90, -50},{ -60, -50}}, color = {0,0,255}),Line(points = {{60,0},{90,0}}, color = {0,0,255}),Line(points = {{0, -35},{0, -91}}, color = {0,0,255}),Line(points = {{ -48,32},{ -28,32}}, color = {0,0,255}),Line(points = {{ -39, -20},{ -39, -41}}, color = {0,0,255}),Line(points = {{ -50, -31},{ -28, -31}}, color = {0,0,255}),Text(extent = {{ -150,126},{150,86}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{60,0},{ -60,70},{ -60, -70},{60,0}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -96,50},{ -60,50}}, color = {0,0,255}),Line(points = {{ -96, -50},{ -60, -50}}, color = {0,0,255}),Line(points = {{60,0},{96,0}}, color = {0,0,255}),Line(points = {{0, -35},{0, -96}}, color = {0,0,255}),Line(points = {{ -55,50},{ -45,50}}, color = {0,0,255}),Line(points = {{ -50, -45},{ -50, -55}}, color = {0,0,255}),Line(points = {{ -55, -50},{ -45, -50}}, color = {0,0,255}),Text(extent = {{ -111, -39},{ -90, -19}}, lineColor = {160,160,164}, textString = "p1.i=0"),Polygon(points = {{120,3},{110,0},{120, -3},{120,3}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {160,160,164}),Line(points = {{111,0},{136,0}}, color = {0,0,0}),Text(extent = {{118,2},{135,17}}, lineColor = {0,0,0}, textString = "i2"),Text(extent = {{ -111,60},{ -90,80}}, lineColor = {160,160,164}, textString = "n1.i=0"),Line(points = {{18, -111},{18, -86}}, color = {160,160,164}),Polygon(points = {{21, -101},{18, -111},{15, -101},{21, -101}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Text(extent = {{22, -100},{39, -85}}, lineColor = {0,0,0}, textString = "i2")}));

        equation 
          v1 = p1.v - n1.v;
          v2 = p2.v - n2.v;
          0 = p1.i + n1.i;
          0 = p2.i + n2.i;
          i1 = p1.i;
          i2 = p2.i;
          v1 = 0;
          i1 = 0;
        end IdealOpAmp;
        model IdealOpAmp3Pin "Ideal operational amplifier (norator-nullator pair), but 3 pins"
          Interfaces.PositivePin in_p "Positive pin of the input port" annotation(Placement(transformation(extent = {{ -110, -60},{ -90, -40}}, rotation = 0)));
          Interfaces.NegativePin in_n "Negative pin of the input port" annotation(Placement(transformation(extent = {{ -110,40},{ -90,60}}, rotation = 0)));
          Interfaces.PositivePin out "Output pin" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<P>
The ideal OpAmp with three pins is of exactly the same behaviour as the ideal
OpAmp with four pins. Only the negative output pin is left out.
Both the input voltage and current are fixed to zero (nullator).
At the output pin both any voltage <i>v2</i> and any current <i>i2</i>
are possible.
</P>
</HTML>
", revisions = "<html>
<ul>
<li><i> 2002   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{60,0},{ -60,70},{ -60, -70},{60,0}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -90,50},{ -60,50}}, color = {0,0,255}),Line(points = {{ -90, -50},{ -60, -50}}, color = {0,0,255}),Line(points = {{60,0},{90,0}}, color = {0,0,255}),Line(points = {{ -48,32},{ -28,32}}, color = {0,0,255}),Line(points = {{ -39, -20},{ -39, -41}}, color = {0,0,255}),Line(points = {{ -50, -31},{ -28, -31}}, color = {0,0,255}),Text(extent = {{ -149,117},{151,77}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{60,0},{ -60,70},{ -60, -70},{60,0}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -96,50},{ -60,50}}, color = {0,0,255}),Line(points = {{ -96, -50},{ -60, -50}}, color = {0,0,255}),Line(points = {{60,0},{97,0}}, color = {0,0,255}),Line(points = {{ -55,50},{ -45,50}}, color = {0,0,255}),Line(points = {{ -50, -45},{ -50, -55}}, color = {0,0,255}),Line(points = {{ -55, -50},{ -45, -50}}, color = {0,0,255}),Text(extent = {{ -111, -39},{ -90, -19}}, lineColor = {160,160,164}, textString = "p1.i=0"),Polygon(points = {{120,3},{110,0},{120, -3},{120,3}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {160,160,164}),Line(points = {{111,0},{136,0}}, color = {0,0,0}),Text(extent = {{118,2},{135,17}}, lineColor = {0,0,0}, textString = "i2"),Text(extent = {{ -111,60},{ -90,80}}, lineColor = {160,160,164}, textString = "n1.i=0")}));

        equation 
          in_p.v = in_n.v;
          in_p.i = 0;
          in_n.i = 0;
        end IdealOpAmp3Pin;
        model IdealOpAmpLimited "Ideal operational amplifier with limitation"
          Interfaces.PositivePin in_p "Positive pin of the input port" annotation(Placement(transformation(extent = {{ -110, -60},{ -90, -40}}, rotation = 0)));
          Interfaces.NegativePin in_n "Negative pin of the input port" annotation(Placement(transformation(extent = {{ -110,40},{ -90,60}}, rotation = 0)));
          Interfaces.PositivePin out "Output pin" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Interfaces.PositivePin VMax "Positive output voltage limitation" annotation(Placement(transformation(extent = {{ -10,60},{10,80}}, rotation = 0)));
          Interfaces.NegativePin VMin "Negative output voltage limitation" annotation(Placement(transformation(extent = {{ -10, -80},{10, -60}}, rotation = 0)));
          SI.Voltage vin "input voltage";
        protected 
          Real s(final unit = "1") "Auxiliary variable";
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(HideResult = true);
          annotation(Documentation(info = "<HTML>
<P>
The ideal OpAmp with limitation behaves like an ideal OpAmp without limitation,
if the output voltage is within the limits (VMin < out.v < VMax). In this case
the input voltage vin=in_p.v - in_n.v is zero.
If the input voltage is vin < 0, the output voltage is out.v = VMin.
If the input voltage is vin > 0, the output voltage is out.v = VMax.
</P>
</HTML>
", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{60,0},{ -60,70},{ -60, -70},{60,0}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -45, -10},{ -10, -10},{ -10,10},{20,10}}, color = {0,0,255}),Line(points = {{0,35},{0,80}}, color = {0,0,255}),Line(points = {{0, -35},{0, -80}}, color = {0,0,255}),Line(points = {{ -90,50},{ -60,50}}, color = {0,0,255}),Line(points = {{ -90, -50},{ -60, -50}}, color = {0,0,255}),Line(points = {{60,0},{90,0}}, color = {0,0,255}),Line(points = {{ -48,32},{ -28,32}}, color = {0,0,255}),Line(points = {{ -39, -20},{ -39, -41}}, color = {0,0,255}),Line(points = {{ -50, -31},{ -28, -31}}, color = {0,0,255}),Text(extent = {{ -152,135},{148,95}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{60,0},{ -60,70},{ -60, -70},{60,0}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -45, -10},{ -10, -10},{ -10,10},{20,10}}, color = {0,0,255}),Line(points = {{0,35},{0,80}}, color = {0,0,255}),Line(points = {{0, -35},{0, -80}}, color = {0,0,255}),Line(points = {{ -96,50},{ -60,50}}, color = {0,0,255}),Line(points = {{ -96, -50},{ -60, -50}}, color = {0,0,255}),Line(points = {{60,0},{96,0}}, color = {0,0,255}),Line(points = {{ -55,50},{ -45,50}}, color = {0,0,255}),Line(points = {{ -50, -45},{ -50, -55}}, color = {0,0,255}),Line(points = {{ -55, -50},{ -45, -50}}, color = {0,0,255}),Text(extent = {{ -111, -39},{ -90, -19}}, lineColor = {160,160,164}, textString = "p1.i=0"),Polygon(points = {{120,3},{110,0},{120, -3},{120,3}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {160,160,164}),Line(points = {{111,0},{136,0}}, color = {0,0,0}),Text(extent = {{118,2},{135,17}}, lineColor = {0,0,0}, textString = "i2"),Text(extent = {{ -111,60},{ -90,80}}, lineColor = {160,160,164}, textString = "n1.i=0")}));

        equation 
          in_p.i = 0;
          in_n.i = 0;
          VMax.i = 0;
          VMin.i = 0;
          vin = in_p.v - in_n.v;
          in_p.v - in_n.v = unitVoltage * smooth(0, if s <  -1 then s + 1 else if s > 1 then s - 1 else 0);
          out.v = smooth(0, if s <  -1 then VMin.v else if s > 1 then VMax.v else ((VMax.v - VMin.v) * s) / 2 + (VMax.v + VMin.v) / 2);
        end IdealOpAmpLimited;
        model IdealDiode "Ideal diode"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter Modelica.SIunits.Resistance Ron(final min = 0) = 1e-05 "Forward state-on differential resistance (closed diode resistance)";
          parameter Modelica.SIunits.Conductance Goff(final min = 0) = 1e-05 "Backward state-off conductance (opened diode conductance)";
          parameter Modelica.SIunits.Voltage Vknee(final min = 0, start = 0) "Forward threshold voltage";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort;
          Boolean off(start = true) "Switching state";
        protected 
          Real s(final unit = "1") "Auxiliary variable for actual position on the ideal diode characteristic";
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(HideResult = true);
          annotation(Documentation(info = "<html>
<P>
This is an ideal switch which is<br><br>
<b>open </b>(off), if it is reversed biased (voltage drop less than 0)<br>
<b>closed</b> (on), if it is conducting (current > 0).<br>
</P>
<p>
This is the behaviour if all parameters are exactly zero.<br><br>
Note, there are circuits, where this ideal description
with zero resistance and zero cinductance is not possible.
In order to prevent singularities during switching, the opened
diode has a small conductance <i>Gon</i> and the closed diode has a low
resistance <i>Roff</i> which is default.
</P>
<P>
The parameter <i>Vknee</i> which is the forward threshold voltage, allows to displace
the knee point <br>
along  the <i>Gon</i>-characteristic until <i>v = Vknee</i>.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled.
</p>
</HTML>
", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>Mai 7, 2004   </i>
       by Christoph Clauss and Anton Haumer<br> Vknee added<br>
       </li>
<li><i>some years ago   </i>
       by Christoph Clauss<br> realized<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{30,0},{ -30,40},{ -30, -40},{30,0}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{40,0}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255}),Line(points = {{30,40},{30, -40}}, color = {0,0,255}),Line(visible = useHeatPort, points = {{0, -100},{0, -20}}, color = {127,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot),Text(extent = {{ -156,101},{144,61}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80,0},{80,0}}, color = {128,128,128}),Polygon(points = {{70,4},{80,0},{70, -4},{70,4}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Line(points = {{0,80},{0, -80}}, color = {128,128,128}),Polygon(points = {{ -4,70},{0,80},{4,70},{ -4,70}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Text(extent = {{10,80},{20,70}}, lineColor = {128,128,128}, textString = "i"),Text(extent = {{70, -10},{80, -20}}, lineColor = {128,128,128}, textString = "v"),Line(points = {{ -80, -40},{ -20, -10},{20,10},{40,70}}, color = {0,0,0}, thickness = 0.5),Line(points = {{20,9},{20,0}}, color = {128,128,128}, pattern = LinePattern.Dot),Text(extent = {{20,0},{40, -10}}, lineColor = {128,128,128}, pattern = LinePattern.Dot, textString = "Vknee"),Text(extent = {{20,70},{40,60}}, lineColor = {128,128,128}, pattern = LinePattern.Dot, textString = "Ron"),Text(extent = {{ -20,10},{0,0}}, lineColor = {128,128,128}, pattern = LinePattern.Dot, textString = "Goff"),Ellipse(extent = {{18,12},{22,8}}, pattern = LinePattern.Dot, lineColor = {0,0,255})}));

        equation 
          off = s < 0;
          v = s * unitCurrent * (if off then 1 else Ron) + Vknee;
          i = s * unitVoltage * (if off then Goff else 1) + Goff * Vknee;
          LossPower = v * i;
        end IdealDiode;
        model IdealTransformer "Ideal transformer core with or without magnetization"
          extends Modelica.Electrical.Analog.Interfaces.TwoPort;
          parameter Real n(start = 1) "Turns ratio primary:secondary voltage";
          parameter Boolean considerMagnetization = false "Choice of considering magnetization";
          parameter Modelica.SIunits.Inductance Lm1(start = 1) "Magnetization inductance w.r.t. primary side" annotation(Dialog(enable = considerMagnetization));
          annotation(Documentation(info = "<html>
<p>
The ideal transformer is a two-port circuit element;
in case of Boolean parameter <code>considerMagnetization = false</code> it is characterized by the following equations:
</p>
<pre><code>
 i2 = -i1*n;
 v2 =  v1/n;
</code></pre>
<p>
where <code>n</code> is a real number called the turns ratio.
Due to this equations, also DC voltages and currents are transformed - which is not the case for technical transformers.
</p>
<p>
In case of Boolean parameter <code>considerMagnetization = true</code> it is characterized by the following equations:
</p>
<pre><code>
 im1  = i1 + i2/n \"Magnetizing current w.r.t. primary side\";
 psim1= Lm1*im1   \"Magnetic flux w.r.t. primary side\";
 v1 = der(psim1)  \"Primary voltage\";
 v2 = v1/n        \"Secondary voltage\";
</code></pre>
<p>
where <code>Lm</code> denotes the magnetizing inductance.
Due to this equations, the DC offset of secondary voltages and currents decrement according to the time constant defined by the connected circuit.
</p>
<p>
Taking primary <code>L1sigma</code> and secondary <code>L2ssigma</code> leakage inductances into account,
compared with the <a href=\"Modelica://Modelica.Electrical.Analog.Basic.Transformer/\">basic transformer</a>
the following parameter conversion can be applied (which leads to identical results):
</p>
<pre><code>
  L1 = L1sigma + M*n \"Primary inductance at secondary no-load\";
  L2 = L2sigma + M/n \"Secondary inductance at primary no-load\";
  M  = Lm1/n         \"Mutual inductance\";
</code></pre>
<p>
For the backward conversion, one has to decide about the partitioning of the leakage to primary and secondary side.
</p>
</html>", revisions = "<html>
<ul>
<li><i>June 3, 2009   </i>
       magnetisation current added by Anton Haumer<br>
       </li>
<li><i>1998   </i>
       initially implemented by Christoph Clauss<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -45, -50},{ -20, -25}}),Ellipse(extent = {{ -45, -25},{ -20,0}}),Ellipse(extent = {{ -45,0},{ -20,25}}),Ellipse(extent = {{ -45,25},{ -20,50}}),Rectangle(extent = {{ -72, -60},{ -33,60}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -90,50},{ -32,50}}),Line(points = {{ -90, -50},{ -32, -50}}),Ellipse(extent = {{20, -50},{45, -25}}),Ellipse(extent = {{20, -25},{45,0}}),Ellipse(extent = {{20,0},{45,25}}),Ellipse(extent = {{20,25},{45,50}}),Rectangle(extent = {{33, -60},{72,60}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{32,50},{90,50}}),Line(points = {{32, -50},{90, -50}}),Text(extent = {{ -100, -80},{100, -100}}, textString = "n=%n"),Text(extent = {{ -100,10},{ -80, -10}}, lineColor = {0,0,255}, textString = "1"),Text(extent = {{80,10},{100, -10}}, lineColor = {0,0,255}, textString = "2"),Text(extent = {{ -146,115},{154,75}}, textString = "%name", lineColor = {0,0,255})}), Diagram(graphics = {Text(extent = {{ -100,10},{0, -10}}, lineColor = {0,0,255}, textString = "1=primary"),Text(extent = {{0,10},{100, -10}}, lineColor = {0,0,255}, textString = "2=secondary")}));
        protected 
          Modelica.SIunits.Current im1 "Magnetization current w.r.t. primary side";
          Modelica.SIunits.MagneticFlux psim1 "Magnetic flux w.r.t primary side";

        equation 
          im1 = i1 + i2 / n;
          if considerMagnetization then
            psim1 = Lm1 * im1;
            v1 = der(psim1);

          else
            psim1 = 0;
            im1 = 0;

          end if;
          v1 = n * v2;
        end IdealTransformer;
        model IdealGyrator "Ideal gyrator"
          extends Interfaces.TwoPort;
          parameter SI.Conductance G(start = 1) "Gyration conductance";
          annotation(Documentation(info = "<html>
<P>
A gyrator is an ideal two-port element defined by the following equations:
</P>
<PRE>
    i1 =  G * v2
    i2 = -G * v1
</PRE>
<P>
where the constant <i>G</i> is called the gyration conductance.
</P>
</HTML>
", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -70, -30},{ -10,30}}, lineColor = {0,0,255}),Rectangle(extent = {{ -80, -40},{ -41,40}}, lineColor = {255,255,255}, pattern = LinePattern.Solid, lineThickness = 0.25, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -90,50},{ -40,50},{ -40, -50},{ -90, -50}}, color = {0,0,255}),Line(points = {{ -30,60},{20,60}}, color = {0,0,255}),Polygon(points = {{20,63},{30,60},{20,57},{20,63}}, pattern = LinePattern.Solid, lineThickness = 0.25, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Ellipse(extent = {{10, -30},{70,30}}, lineColor = {0,0,255}),Rectangle(extent = {{80, -40},{40,40}}, lineColor = {255,255,255}, pattern = LinePattern.Solid, lineThickness = 0.25, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{90,50},{40,50},{40, -50},{90, -50}}, color = {0,0,255}),Text(extent = {{ -152,116},{148,76}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -70, -30},{ -10,30}}, lineColor = {0,0,255}),Rectangle(extent = {{ -80, -40},{ -41,40}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -96,50},{ -40,50},{ -40, -50},{ -96, -50}}, color = {0,0,255}),Line(points = {{ -30,60},{20,60}}, color = {0,0,255}),Polygon(points = {{20,63},{30,60},{20,57},{20,63}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Ellipse(extent = {{10, -30},{70,30}}, lineColor = {0,0,255}),Rectangle(extent = {{80, -40},{40,40}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{96,50},{40,50},{40, -50},{96, -50}}, color = {0,0,255})}));

        equation 
          i1 = G * v2;
          i2 =  -G * v1;
        end IdealGyrator;
        model Idle "Idle branch"
          extends Interfaces.OnePort;
          annotation(Documentation(info = "<html>
<p>The model Idle is a simple idle running branch. That means between both pins no current is running. This ideal device is of no influence on the circuit. Therefore, it can be neglected in each case. For purposes of completness this component is part of the MSL, as an opposite of the short cut.</p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -90,0},{ -41,0}}, color = {0,0,255}),Line(points = {{91,0},{40,0}}, color = {0,0,255}),Text(extent = {{ -153,112},{147,72}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,255}),Line(points = {{ -96,0},{ -41,0}}, color = {0,0,255}),Line(points = {{96,0},{40,0}}, color = {0,0,255})}));

        equation 
          i = 0;
        end Idle;
        model Short "Short cut branch"
          extends Interfaces.OnePort;
          annotation(Documentation(info = "<html>
<p>The model Short is a simple short cut branch. That means the voltage drop between both pins is zero. This device could be nelected if both pins are combined to one node. Besides connecting the nodes of both pins this device has no further function.</p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{91,0},{ -90,0}}, color = {0,0,255}),Text(extent = {{ -151,113},{149,73}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,255}),Line(points = {{96,0},{ -96,0}}, color = {0,0,255}),Text(extent = {{ -100,100},{100,70}}, textString = "Short", lineColor = {0,0,255})}));

        equation 
          v = 0;
        end Short;
        model IdealOpeningSwitch "Ideal electrical opener"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter SI.Resistance Ron(final min = 0) = 1e-05 "Closed switch resistance" annotation(Placement(transformation(extent = {{ -56.6667,10},{ -10,56.6667}}, rotation = 0)));
          parameter SI.Conductance Goff(final min = 0) = 1e-05 "Opened switch conductance" annotation(Placement(transformation(extent = {{10,10},{56.6667,56.6667}}, rotation = 0)));
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Modelica.Blocks.Interfaces.BooleanInput control "true => switch open, false => p--n connected" annotation(Placement(transformation(origin = {0,70}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
        protected 
          Real s(final unit = "1") "Auxiliary variable";
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(HideResult = true);
          annotation(Documentation(info = "<HTML>
<P>
The ideal opening switch has a positive pin p and a negative pin n.
The switching behaviour is controlled by the input signal control.
If control is true, pin p is not connected
with negative pin n. Otherwise, pin p is connected
with negative pin n.
</P>
<P>
In order to prevent singularities during switching, the opened
switch has a (very low) conductance Goff
and the closed switch has a (very low) resistance Ron.
The limiting case is also allowed, i.e., the resistance Ron of the
closed switch could be exactly zero and the conductance Goff of the
open switch could be also exactly zero. Note, there are circuits,
where a description with zero Ron or zero Goff is not possible.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</P>
</HTML>
", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -44,4},{ -36, -4}}, lineColor = {0,0,255}),Line(points = {{ -90,0},{ -44,0}}, color = {0,0,255}),Line(points = {{ -37,2},{40,50}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255}),Line(points = {{0,51},{0,26}}, color = {0,0,255}),Line(points = {{40,20},{40,0}}, color = {0,0,255}),Line(visible = useHeatPort, points = {{0, -100},{0,25}}, color = {127,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot),Text(extent = {{ -151, -21},{149, -61}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -44,4},{ -36, -4}}, lineColor = {0,0,255}),Line(points = {{ -96,0},{ -44,0}}, color = {0,0,255}),Line(points = {{ -37,2},{40,50}}, color = {0,0,255}),Line(points = {{40,0},{96,0}}, color = {0,0,255}),Text(extent = {{ -100, -40},{100, -79}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{0,51},{0,26}}, color = {0,0,255}),Line(points = {{40,20},{40,0}}, color = {0,0,255})}));

        equation 
          v = s * unitCurrent * (if control then 1 else Ron);
          i = s * unitVoltage * (if control then Goff else 1);
          LossPower = v * i;
        end IdealOpeningSwitch;
        model IdealClosingSwitch "Ideal electrical closer"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter SI.Resistance Ron(final min = 0) = 1e-05 "Closed switch resistance" annotation(Placement(transformation(extent = {{ -56.6667,10},{ -10,56.6667}}, rotation = 0)));
          parameter SI.Conductance Goff(final min = 0) = 1e-05 "Opened switch conductance" annotation(Placement(transformation(extent = {{10,10},{56.6667,56.6667}}, rotation = 0)));
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Modelica.Blocks.Interfaces.BooleanInput control "true => p--n connected, false => switch open" annotation(Placement(transformation(origin = {0,70}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
        protected 
          Real s(final unit = "1") "Auxiliary variable";
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(HideResult = true);
          annotation(Documentation(info = "<HTML>
<P>
The ideal closing switch has a positive pin p and a negative pin n.
The switching behaviour is controlled by input signal control.
If control is true, pin p is connected
with negative pin n. Otherwise, pin p is not connected
with negative pin n.
</P>
<P>
In order to prevent singularities during switching, the opened
switch has a (very low) conductance Goff
and the closed switch has a (very low) resistance Ron.
The limiting case is also allowed, i.e., the resistance Ron of the
closed switch could be exactly zero and the conductance Goff of the
open switch could be also exactly zero. Note, there are circuits,
where a description with zero Ron or zero Goff is not possible.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</P>
</HTML>
", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -44,4},{ -36, -4}}, lineColor = {0,0,255}),Line(points = {{ -90,0},{ -44,0}}, color = {0,0,255}),Line(points = {{ -37,2},{40,50}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255}),Line(points = {{0,51},{0,26}}, color = {0,0,255}),Line(visible = useHeatPort, points = {{0, -100},{0,25}}, color = {127,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot),Text(extent = {{ -152, -28},{148, -68}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -44,4},{ -36, -4}}, lineColor = {0,0,255}),Line(points = {{ -96,0},{ -44,0}}, color = {0,0,255}),Line(points = {{ -37,2},{40,50}}, color = {0,0,255}),Line(points = {{40,0},{96,0}}, color = {0,0,255}),Text(extent = {{ -100, -40},{100, -79}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{0,51},{0,26}}, color = {0,0,255})}));

        equation 
          v = s * unitCurrent * (if control then Ron else 1);
          i = s * unitVoltage * (if control then 1 else Goff);
          LossPower = v * i;
        end IdealClosingSwitch;
        model ControlledIdealOpeningSwitch "Controlled ideal electrical opener"
          parameter SI.Voltage level = 0.5 "Switch level" annotation(Placement(transformation(extent = {{ -56.6667,10},{ -10,56.6667}}, rotation = 0)));
          parameter SI.Resistance Ron(final min = 0) = 1e-05 "Closed switch resistance" annotation(Placement(transformation(extent = {{10,10},{56.6667,56.6667}}, rotation = 0)));
          parameter SI.Conductance Goff(final min = 0) = 1e-05 "Opened switch conductance" annotation(Placement(transformation(extent = {{ -56.6667, -56.6667},{ -10, -10}}, rotation = 0)));
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Interfaces.PositivePin p annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Interfaces.NegativePin n annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Interfaces.Pin control "Control pin: control.v > level switch open, otherwise p--n connected" annotation(Placement(transformation(origin = {0,100}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        protected 
          Real s(final unit = "1") "Auxiliary variable";
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(HideResult = true);
          annotation(Documentation(info = "
<HTML>
<P>
The ideal switch has a positive pin p and a negative pin n.
The switching behaviour is controlled by the control pin.
If its voltage exceeds the voltage of the parameter level,
pin p is not connected with negative pin n.
Otherwise, pin p is connected with negative pin n.
</P>
<P>
In order to prevent singularities during switching, the opened
switch has a (very low) conductance Goff
and the closed switch has a (very low) resistance Ron.
The limiting case is also allowed, i.e., the resistance Ron of the
closed switch could be exactly zero and the conductance Goff of the
open switch could be also exactly zero. Note, there are circuits,
where a description with zero Ron or zero Goff is not possible.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</P>
</HTML>
", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -44,4},{ -36, -4}}, lineColor = {0,0,255}),Line(points = {{ -90,0},{ -44,0}}, color = {0,0,255}),Line(points = {{ -37,2},{40,50}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255}),Line(points = {{0,90},{0,25}}, color = {0,0,255}),Line(points = {{40,20},{40,0}}, color = {0,0,255}),Line(visible = useHeatPort, points = {{0, -100},{0,25}}, color = {127,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot),Text(extent = {{ -150, -34},{150, -74}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -44,4},{ -36, -4}}, lineColor = {0,0,255}),Line(points = {{ -96,0},{ -44,0}}, color = {0,0,255}),Line(points = {{ -37,2},{40,50}}, color = {0,0,255}),Line(points = {{40,0},{96,0}}, color = {0,0,255}),Line(points = {{0,96},{0,25}}, color = {0,0,255}),Line(points = {{40,20},{40,0}}, color = {0,0,255})}));

        equation 
          control.i = 0;
          0 = p.i + n.i;
          p.v - n.v = s * unitCurrent * (if control.v > level then 1 else Ron);
          p.i = s * unitVoltage * (if control.v > level then Goff else 1);
          LossPower = (p.v - n.v) * p.i;
        end ControlledIdealOpeningSwitch;
        model ControlledIdealClosingSwitch "Controlled ideal electrical closer"
          parameter SI.Voltage level = 0.5 "Switch level" annotation(Placement(transformation(extent = {{ -56.6667,10},{ -10,56.6667}}, rotation = 0)));
          parameter SI.Resistance Ron(final min = 0) = 1e-05 "Closed switch resistance" annotation(Placement(transformation(extent = {{10,10},{56.6667,56.6667}}, rotation = 0)));
          parameter SI.Conductance Goff(final min = 0) = 1e-05 "Opened switch conductance" annotation(Placement(transformation(extent = {{ -56.6667, -56.6667},{ -10, -10}}, rotation = 0)));
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Modelica.Electrical.Analog.Interfaces.PositivePin p annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin n annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin control "Control pin: control.v > level switch closed, otherwise switch open" annotation(Placement(transformation(origin = {0,100}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        protected 
          Real s(final unit = "1") "Auxiliary variable";
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(HideResult = true);
          annotation(Documentation(info = "
<HTML>
<P>
The closing ideal switch has a positive pin p and a negative pin n.
The switching behaviour is controlled by the control pin.
If its voltage exceeds the voltage of the parameter level,
pin p is connected with negative pin n.
Otherwise, pin p is not connected with negative pin n.
</P>
<P>
In order to prevent singularities during switching, the opened
switch has a (very low) conductance Goff
and the closed switch has a (very low) resistance Ron.
The limiting case is also allowed, i.e., the resistance Ron of the
closed switch could be exactly zero and the conductance Goff of the
open switch could be also exactly zero. Note, there are circuits,
where a description with zero Ron or zero Goff is not possible.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</P>
</HTML>
", revisions = "<html>
<ul>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -44,4},{ -36, -4}}, lineColor = {0,0,255}),Line(points = {{ -90,0},{ -44,0}}, color = {0,0,255}),Line(points = {{ -37,2},{40,50}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255}),Line(points = {{0,90},{0,25}}, color = {0,0,255}),Line(visible = useHeatPort, points = {{0, -100},{0,25}}, color = {127,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot),Text(extent = {{ -149, -32},{151, -72}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -44,4},{ -36, -4}}, lineColor = {0,0,255}),Line(points = {{ -96,0},{ -44,0}}, color = {0,0,255}),Line(points = {{ -37,2},{40,50}}, color = {0,0,255}),Line(points = {{40,0},{96,0}}, color = {0,0,255}),Line(points = {{0,96},{0,25}}, color = {0,0,255})}));

        equation 
          control.i = 0;
          0 = p.i + n.i;
          p.v - n.v = s * unitCurrent * (if control.v > level then Ron else 1);
          p.i = s * unitVoltage * (if control.v > level then 1 else Goff);
          LossPower = (p.v - n.v) * p.i;
        end ControlledIdealClosingSwitch;
        model OpenerWithArc "Ideal opening switch with simple arc model"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter Modelica.SIunits.Resistance Ron = 1e-05 "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff = 1e-05 "Opened switch conductance";
          parameter Modelica.SIunits.Voltage V0 = 30 "Initial arc voltage";
          parameter Modelica.SIunits.VoltageSlope dVdt = 10000.0 "Arc voltage slope";
          parameter Modelica.SIunits.Voltage Vmax = 60 "Max. arc voltage";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Modelica.Blocks.Interfaces.BooleanInput control "false => p--n connected, true => switch open" annotation(Placement(transformation(origin = {0,100}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{40,50},{32,32},{48,28},{40,18}}, color = {255,0,0}),Ellipse(extent = {{ -44,4},{ -36, -4}}),Line(points = {{ -90,0},{ -44,0}}),Line(points = {{ -37,2},{40,50}}),Line(points = {{40,0},{90,0}}),Line(points = {{0,90},{0,26}}, color = {255,85,255}),Line(points = {{40,18},{40,0}}),Text(extent = {{ -150, -35},{150, -75}}, textString = "%name", lineColor = {0,0,255})}), Diagram(graphics = {Line(points = {{ -60,60},{ -60, -60},{60, -60}}, color = {0,0,255}),Line(points = {{ -60, -60},{ -40, -60},{ -40, -40},{ -20,40},{40,40}}, color = {0,0,0}),Text(extent = {{30, -60},{50, -70}}, lineColor = {0,0,0}, textString = "time"),Text(extent = {{ -60,60},{ -20,50}}, lineColor = {0,0,0}, textString = "voltage"),Text(extent = {{ -60, -30},{ -40, -40}}, lineColor = {0,0,0}, textString = "V0"),Text(extent = {{ -50,40},{ -30,30}}, lineColor = {0,0,0}, textString = "Vmax"),Text(extent = {{ -40,10},{ -20,0}}, lineColor = {0,0,0}, textString = "dVdt"),Polygon(points = {{ -60,60},{ -62,52},{ -58,52},{ -60,60}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{60, -60},{54, -58},{54, -62},{60, -60}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>
This model is an extension to the <a href=\"Modelica://Modelica.Electrical.Analog.Ideal.IdealOpeningSwitch\">IdealOpeningSwitch</a>.
</p>
<p>
The basic model interupts the current through the switch in an infinitesimal time span.
If an inductive circuit is connected, the voltage across the swicth is limited only by numerics.
In order to give a better idea for the voltage across the switch, a simple arc model is added:
</p>
<p>
When the Boolean input <code>control</code> signals to open the switch, a voltage across the opened switch is impressed.
This voltage starts with <code>V0</code> (simulating the voltage drop of the arc roots), then rising with slope <code>dVdt</code>
(simulating the rising voltage of an extending arc) until a maximum voltage <code>Vmax</code> is reached.
</p>
<pre><code>
     | voltage
Vmax |      +-----
     |     /
     |    /
V0   |   +
     |   |
     +---+-------- time
</pre></code>
<p>
This arc voltage tends to lower the current following through the switch; it depends on the connected circuit, when the arc is quenched.
Once the arc is quenched, i.e. the current flowing through the switch gets zero, the equation for the off-state is activated
<code>i=Goff*v</code>.
</p>
<p>
When the Boolean input <code>control</code> signals to close the switch again, the switch is closed immediately,
i.e. the equation for the on-state is activated <code>v=Ron*i</code>.
</p>
<p>
Please note: In an AC circuit, at least the arc quenches when the next natural zero-crossing of the current occurs.
In a DC circuit, the arc will not quench if the arc voltage is not sufficient that a zero-crossing of the current occurs.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</p>
</html>", revisions = "<html>
<ul>
<li><i>June, 2009   </i>
       by Christoph Clauss<br> adapted to OpenerWithArc<br>
       </li>
<li><i>May, 2009   </i>
       by Anton Haumer<br> CloserWithArc initially implemented<br>
       </li>
</ul>
</html>"));
        protected 
          Boolean off = control;
          Boolean on = not off;
          discrete Modelica.SIunits.Time tSwitch;
          Boolean quenched;

        equation 
          when edge(off) then
                      tSwitch = time;
          
          end when;
          quenched = off and (abs(i) <= abs(v) * Goff or pre(quenched));
          if on then
            v = Ron * i;

          else
            if quenched then
              i = Goff * v;

            else
              v = min(Vmax, V0 + dVdt * (time - tSwitch)) * sign(i);

            end if;

          end if;
          LossPower = v * i;
        end OpenerWithArc;
        model CloserWithArc "Ideal closing switch with simple arc model"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter Modelica.SIunits.Resistance Ron = 1e-05 "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff = 1e-05 "Opened switch conductance";
          parameter Modelica.SIunits.Voltage V0 = 30 "Initial arc voltage";
          parameter Modelica.SIunits.VoltageSlope dVdt = 10000.0 "Arc voltage slope";
          parameter Modelica.SIunits.Voltage Vmax = 60 "Max. arc voltage";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Modelica.Blocks.Interfaces.BooleanInput control "true => p--n connected, false => switch open" annotation(Placement(transformation(origin = {0,100}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{40,50},{32,24},{48,28},{40,0}}, color = {255,0,0}),Ellipse(extent = {{ -44,4},{ -36, -4}}),Line(points = {{ -90,0},{ -44,0}}),Line(points = {{ -37,2},{40,50}}),Line(points = {{40,0},{90,0}}),Line(points = {{0,90},{0,26}}, color = {255,85,255}),Text(extent = {{ -152, -29},{148, -69}}, textString = "%name", lineColor = {0,0,255})}), Diagram(graphics = {Line(points = {{ -60,60},{ -60, -60},{60, -60}}, color = {0,0,255}),Line(points = {{ -60, -60},{ -40, -60},{ -40, -40},{ -20,40},{40,40}}, color = {0,0,0}),Text(extent = {{30, -60},{50, -70}}, lineColor = {0,0,0}, textString = "time"),Text(extent = {{ -60,60},{ -20,50}}, lineColor = {0,0,0}, textString = "voltage"),Text(extent = {{ -60, -30},{ -40, -40}}, lineColor = {0,0,0}, textString = "V0"),Text(extent = {{ -50,40},{ -30,30}}, lineColor = {0,0,0}, textString = "Vmax"),Text(extent = {{ -40,10},{ -20,0}}, lineColor = {0,0,0}, textString = "dVdt"),Polygon(points = {{ -60,60},{ -62,52},{ -58,52},{ -60,60}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{60, -60},{54, -58},{54, -62},{60, -60}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>
This model is an extension to the <a href=\"Modelica://Modelica.Electrical.Analog.Ideal.IdealClosingSwitch\">IdealClosingSwitch</a>.
</p>
<p>
The basic model interupts the current through the switch in an infinitesimal time span.
If an inductive circuit is connected, the voltage across the swicth is limited only by numerics.
In order to give a better idea for the voltage across the switch, a simple arc model is added:
</p>
<p>
When the Boolean input <code>control</code> signals to open the switch, a voltage across the opened switch is impressed.
This voltage starts with <code>V0</code> (simulating the voltage drop of the arc roots), then rising with slope <code>dVdt</code>
(simulating the rising voltage of an extending arc) until a maximum voltage <code>Vmax</code> is reached.
</p>
<pre><code>
     | voltage
Vmax |      +-----
     |     /
     |    /
V0   |   +
     |   |
     +---+-------- time
</pre></code>
<p>
This arc voltage tends to lower the current following through the switch; it depends on the connected circuit, when the arc is quenched.
Once the arc is quenched, i.e. the current flowing through the switch gets zero, the equation for the off-state is activated
<code>i=Goff*v</code>.
</p>
<p>
When the Boolean input <code>control</code> signals to close the switch again, the switch is closed immediately,
i.e. the equation for the on-state is activated <code>v=Ron*i</code>.
</p>
<p>
Please note: In an AC circuit, at least the arc quenches when the next natural zero-crossing of the current occurs.
In a DC circuit, the arc will not quench if the arc voltage is not sufficient that a zero-crossing of the current occurs.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</p>
</html>", revisions = "<html>
<ul>
<li><i>May, 2009   </i>
       by Anton Haumer<br> initially implemented<br>
       </li>
</ul>
</html>"));
        protected 
          Boolean on = control;
          Boolean off = not on;
          discrete Modelica.SIunits.Time tSwitch;
          Boolean quenched;

        equation 
          when edge(off) then
                      tSwitch = time;
          
          end when;
          quenched = off and (abs(i) <= abs(v) * Goff or pre(quenched));
          if on then
            v = Ron * i;

          else
            if quenched then
              i = Goff * v;

            else
              v = min(Vmax, V0 + dVdt * (time - tSwitch)) * sign(i);

            end if;

          end if;
          LossPower = v * i;
        end CloserWithArc;
        model ControlledOpenerWithArc "Controlled ideal electrical opener with simple arc model"
          parameter Modelica.SIunits.Voltage level = 0.5 "Switch level" annotation(Placement(transformation(extent = {{ -56.6667,10},{ -10,56.6667}}, rotation = 0)));
          parameter Modelica.SIunits.Resistance Ron(final min = 0) = 1e-05 "Closed switch resistance" annotation(Placement(transformation(extent = {{10,10},{56.6667,56.6667}}, rotation = 0)));
          parameter Modelica.SIunits.Conductance Goff(final min = 0) = 1e-05 "Opened switch conductance" annotation(Placement(transformation(extent = {{ -56.6667, -56.6667},{ -10, -10}}, rotation = 0)));
          parameter Modelica.SIunits.Voltage V0 = 30 "Initial arc voltage";
          parameter Modelica.SIunits.VoltageSlope dVdt = 10000.0 "Arc voltage slope";
          parameter Modelica.SIunits.Voltage Vmax = 60 "Max. arc voltage";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Modelica.Electrical.Analog.Interfaces.PositivePin p annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin n annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin control "Control pin: control.v > level switch open, otherwise p--n connected" annotation(Placement(transformation(origin = {0,100}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.SIunits.Current i;
          Modelica.SIunits.Voltage v;
        protected 
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(HideResult = true);
          Boolean off = control.v > level;
          Boolean on = not off;
          discrete Modelica.SIunits.Time tSwitch;
          Boolean quenched;
          annotation(Documentation(info = "
<HTML>

<p>
This model is an extension to the <a href=\"Modelica://Modelica.Electrical.Analog.Ideal.IdealOpeningSwitch\">IdealOpeningSwitch</a>.
</p>


<p>
The basic model interupts the current through the switch in an infinitesimal time span.
If an inductive circuit is connected, the voltage across the swicth is limited only by numerics.
In order to give a better idea for the voltage across the switch, a simple arc model is added:
</p>
<p>
When the control pin voltage <code>control.v</code> signals to open the switch, a voltage across the opened switch is impressed.
This voltage starts with <code>V0</code> (simulating the voltage drop of the arc roots), then rising with slope <code>dVdt</code>
(simulating the rising voltage of an extending arc) until a maximum voltage <code>Vmax</code> is reached.
</p>
<pre><code>
     | voltage
Vmax |      +-----
     |     /
     |    /
V0   |   +
     |   |
     +---+-------- time
</pre></code>
<p>
This arc voltage tends to lower the current following through the switch; it depends on the connected circuit, when the arc is quenched.
Once the arc is quenched, i.e. the current flowing through the switch gets zero, the equation for the off-state is activated
<code>i=Goff*v</code>.
</p>
<p>
When the control pin <code>control.v</code> signals to close the switch again, the switch is closed immediately,
i.e. the equation for the on-state is activated <code>v=Ron*i</code>.
</p>
<p>
Please note: In an AC circuit, at least the arc quenches when the next natural zero-crossing of the current occurs.
In a DC circuit, the arc will not quench if the arc voltage is not sufficient that a zero-crossing of the current occurs.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</p>
</HTML>
", revisions = "<html>
<ul>
<li><i>  </i>
       </li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -44,4},{ -36, -4}}, lineColor = {0,0,255}),Line(points = {{ -90,0},{ -44,0}}, color = {0,0,255}),Line(points = {{ -37,2},{40,50}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255}),Line(points = {{0,90},{0,25}}, color = {0,0,255}),Line(points = {{40,20},{40,0}}, color = {0,0,255}),Line(points = {{40,50},{32,34},{48,30},{40,20}}, color = {255,0,0}),Text(extent = {{ -148, -34},{152, -74}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -44,4},{ -36, -4}}, lineColor = {0,0,255}),Line(points = {{ -96,0},{ -44,0}}, color = {0,0,255}),Line(points = {{ -37,2},{40,50}}, color = {0,0,255}),Line(points = {{40,0},{96,0}}, color = {0,0,255}),Line(points = {{0,96},{0,25}}, color = {0,0,255}),Line(points = {{40,20},{40,0}}, color = {0,0,255})}));

        equation 
          control.i = 0;
          0 = p.i + n.i;
          i = p.i;
          p.v - n.v = v;
          when edge(off) then
                      tSwitch = time;
          
          end when;
          quenched = off and (abs(i) <= abs(v) * Goff or pre(quenched));
          if on then
            v = Ron * i;

          else
            if quenched then
              i = Goff * v;

            else
              v = min(Vmax, V0 + dVdt * (time - tSwitch)) * sign(i);

            end if;

          end if;
          LossPower = v * i;
        end ControlledOpenerWithArc;
        model ControlledCloserWithArc "Controlled ideal electrical closer with simple arc model"
          parameter Modelica.SIunits.Voltage level = 0.5 "Switch level" annotation(Placement(transformation(extent = {{ -56.6667,10},{ -10,56.6667}}, rotation = 0)));
          parameter Modelica.SIunits.Resistance Ron(final min = 0) = 1e-05 "Closed switch resistance" annotation(Placement(transformation(extent = {{10,10},{56.6667,56.6667}}, rotation = 0)));
          parameter Modelica.SIunits.Conductance Goff(final min = 0) = 1e-05 "Opened switch conductance" annotation(Placement(transformation(extent = {{ -56.6667, -56.6667},{ -10, -10}}, rotation = 0)));
          parameter Modelica.SIunits.Voltage V0 = 30 "Initial arc voltage";
          parameter Modelica.SIunits.VoltageSlope dVdt = 10000.0 "Arc voltage slope";
          parameter Modelica.SIunits.Voltage Vmax = 60 "Max. arc voltage";
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(final T = 293.15);
          Modelica.Electrical.Analog.Interfaces.PositivePin p annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin n annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.Pin control "Control pin: control.v > level switch closed, otherwise switch open" annotation(Placement(transformation(origin = {0,100}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.SIunits.Current i;
          Modelica.SIunits.Voltage v;
        protected 
          constant Modelica.SIunits.Voltage unitVoltage = 1 annotation(HideResult = true);
          constant Modelica.SIunits.Current unitCurrent = 1 annotation(HideResult = true);
          Boolean off = control.v < level;
          Boolean on = not off;
          discrete Modelica.SIunits.Time tSwitch;
          Boolean quenched;
          annotation(Documentation(info = "
<HTML>

<p>
This model is an extension to the <a href=\"Modelica://Modelica.Electrical.Analog.Ideal.IdealClosingSwitch\">IdealClosingSwitch</a>.
</p>


<p>
The basic model interupts the current through the switch in an infinitesimal time span.
If an inductive circuit is connected, the voltage across the swicth is limited only by numerics.
In order to give a better idea for the voltage across the switch, a simple arc model is added:
</p>
<p>
When the control pin voltage <code>control.v</code> signals to open the switch, a voltage across the opened switch is impressed.
This voltage starts with <code>V0</code> (simulating the voltage drop of the arc roots), then rising with slope <code>dVdt</code>
(simulating the rising voltage of an extending arc) until a maximum voltage <code>Vmax</code> is reached.
</p>
<pre><code>
     | voltage
Vmax |      +-----
     |     /
     |    /
V0   |   +
     |   |
     +---+-------- time
</pre></code>
<p>
This arc voltage tends to lower the current following through the switch; it depends on the connected circuit, when the arc is quenched.
Once the arc is quenched, i.e. the current flowing through the switch gets zero, the equation for the off-state is activated
<code>i=Goff*v</code>.
</p>
<p>
When the control pin <code>control.v</code> signals to close the switch again, the switch is closed immediately,
i.e. the equation for the on-state is activated <code>v=Ron*i</code>.
</p>
<p>
Please note: In an AC circuit, at least the arc quenches when the next natural zero-crossing of the current occurs.
In a DC circuit, the arc will not quench if the arc voltage is not sufficient that a zero-crossing of the current occurs.
<br> <br>
<b>Please note:</b>
In case of useHeatPort=true the temperature dependence of the electrical
behavior is <b> not </b> modelled. The parameters are not temperature dependent.
</p>

</HTML>
", revisions = "<html>
<ul>
<li><i>  </i>
       </li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -44,4},{ -36, -4}}, lineColor = {0,0,255}),Line(points = {{ -90,0},{ -44,0}}, color = {0,0,255}),Line(points = {{ -37,2},{40,50}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255}),Line(points = {{0,90},{0,25}}, color = {0,0,255}),Text(extent = {{ -100, -70},{100, -100}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{40,50},{32,24},{48,28},{40,0}}, color = {255,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -44,4},{ -36, -4}}, lineColor = {0,0,255}),Line(points = {{ -96,0},{ -44,0}}, color = {0,0,255}),Line(points = {{ -37,2},{40,50}}, color = {0,0,255}),Line(points = {{40,0},{96,0}}, color = {0,0,255}),Line(points = {{0,96},{0,25}}, color = {0,0,255})}));

        equation 
          control.i = 0;
          0 = p.i + n.i;
          i = p.i;
          p.v - n.v = v;
          when edge(off) then
                      tSwitch = time;
          
          end when;
          quenched = off and (abs(i) <= abs(v) * Goff or pre(quenched));
          if on then
            v = Ron * i;

          else
            if quenched then
              i = Goff * v;

            else
              v = min(Vmax, V0 + dVdt * (time - tSwitch)) * sign(i);

            end if;

          end if;
          LossPower = v * i;
        end ControlledCloserWithArc;
      end Ideal;
      package Basic "Basic electrical components"
        extends Modelica.Icons.Library;
        annotation(Documentation(info = "<html>
<p>This package contains very basic analog electrical components such as resistor, conductor, condensator, inductor, and the ground (which is needed in each electrical circuit description. Furthermore, controled sources, coupling components, and some improved (but newertheless basic) are in this package. </p>
</html>", revisions = "<html>
<dl>
<dt>
<b>Main Authors:</b>
<dd>
Christoph Clau&szlig;
    &lt;<a href=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</a>&gt;<br>
    Andr&eacute; Schneider
    &lt;<a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>&gt;<br>
    Fraunhofer Institute for Integrated Circuits<br>
    Design Automation Department<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden<br>
<p>
</dl>
</html>"));
        model Ground "Ground node"
          Interfaces.Pin p annotation(Placement(transformation(origin = {0,100}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
          annotation(Documentation(info = "<html>
<p>Ground of an electrical circuit. The potential at the ground node is zero. Every electrical circuit has to contain at least one ground object.  </p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -60,50},{60,50}}, color = {0,0,255}),Line(points = {{ -40,30},{40,30}}, color = {0,0,255}),Line(points = {{ -20,10},{20,10}}, color = {0,0,255}),Line(points = {{0,90},{0,50}}, color = {0,0,255}),Text(extent = {{ -144, -19},{156, -59}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -60,50},{60,50}}, thickness = 0.5, color = {0,0,255}),Line(points = {{ -40,30},{40,30}}, thickness = 0.5, color = {0,0,255}),Line(points = {{ -20,10},{20,10}}, thickness = 0.5, color = {0,0,255}),Line(points = {{0,96},{0,50}}, thickness = 0.5, color = {0,0,255}),Text(extent = {{ -24, -38},{22, -6}}, textString = "p.v=0", lineColor = {0,0,255})}));

        equation 
          p.v = 0;
        end Ground;
        model Resistor "Ideal linear electrical resistor"
          parameter Modelica.SIunits.Resistance R(start = 1) "Resistance at temperature T_ref";
          parameter Modelica.SIunits.Temperature T_ref = 300.15 "Reference temperature";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha = 0 "Temperature coefficient of resistance (R_actual = R*(1 + alpha*(T_heatPort - T_ref))";
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = T_ref);
          Modelica.SIunits.Resistance R_actual "Actual resistance = R*(1 + alpha*(T_heatPort - T_ref))";
          annotation(Documentation(info = "<html>
<p>The linear resistor connects the branch voltage <i>v</i> with the branch current <i>i</i> by <i>i*R = v</i>. The Resistance <i>R</i> is allowed to be positive, zero, or negative.  </p>
</html>", revisions = "<html>
<ul>
<li><i> August 07, 2009   </i>
       by Anton Haumer<br> temperature dependency of resistance added<br>
       </li>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -70,30},{70, -30}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{ -70,0}}, color = {0,0,255}),Line(points = {{70,0},{90,0}}, color = {0,0,255}),Text(extent = {{ -144, -40},{142, -72}}, lineColor = {0,0,0}, textString = "R=%R"),Line(visible = useHeatPort, points = {{0, -100},{0, -30}}, color = {127,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot),Text(extent = {{ -152,87},{148,47}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -70,30},{70, -30}}, lineColor = {0,0,255}),Line(points = {{ -96,0},{ -70,0}}, color = {0,0,255}),Line(points = {{70,0},{96,0}}, color = {0,0,255})}));

        equation 
          assert(1 + alpha * (T_heatPort - T_ref) >= Modelica.Constants.eps, "Temperature outside scope of model!");
          R_actual = R * (1 + alpha * (T_heatPort - T_ref));
          v = R_actual * i;
          LossPower = v * i;
        end Resistor;
        model HeatingResistor "Temperature dependent electrical resistor"
          parameter Modelica.SIunits.Resistance R_ref(start = 1) "Resistance at temperature T_ref";
          parameter Modelica.SIunits.Temperature T_ref = 300.15 "Reference temperature";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha = 0 "Temperature coefficient of resistance (R = R_ref*(1 + alpha*(heatPort.T - T_ref))";
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = T_ref, useHeatPort = true);
          Modelica.SIunits.Resistance R "Resistance = R_ref*(1 + alpha*(T_heatPort - T_ref))";
          annotation(__Dymola_structurallyIncomplete = true, Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -110,20},{ -85,20}}, color = {160,160,164}),Polygon(points = {{ -95,23},{ -85,20},{ -95,17},{ -95,23}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Line(points = {{90,20},{115,20}}, color = {160,160,164}),Line(points = {{ -125,0},{ -115,0}}, color = {160,160,164}),Line(points = {{ -120, -5},{ -120,5}}, color = {160,160,164}),Text(extent = {{ -110,25},{ -90,45}}, lineColor = {160,160,164}, textString = "i"),Polygon(points = {{105,23},{115,20},{105,17},{105,23}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Line(points = {{115,0},{125,0}}, color = {160,160,164}),Text(extent = {{90,45},{110,25}}, lineColor = {160,160,164}, textString = "i")}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -90,0},{ -70,0}}, color = {0,0,255}),Line(points = {{70,0},{90,0}}, color = {0,0,255}),Rectangle(extent = {{ -70,30},{70, -30}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -52, -50},{48,50}}, color = {0,0,255}),Polygon(points = {{40,52},{50,42},{54,56},{40,52}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Line(visible = useHeatPort, points = {{0, -100},{0, -30}}, color = {127,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot),Text(extent = {{ -156,109},{144,69}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<html>
<p>This is a model for an electrical resistor where the generated heat is dissipated to the environment via connector <b>heatPort</b> and where the resistance R is temperature dependent according to the following equation: </p>
<pre>    R = R_ref*(1 + alpha*(heatPort.T - T_ref))</pre>
<p><b>alpha</b> is the <b>temperature coefficient of resistance</b>, which is often abbreviated as <b>TCR</b>. In resistor catalogues, it is usually defined as <b>X [ppm/K]</b> (parts per million, similarly to per centage) meaning <b>X*1.e-6 [1/K]</b>. Resistors are available for 1 .. 7000 ppm/K, i.e., alpha = 1e-6 .. 7e-3 1/K; </p>
<p>Via parameter <b>useHeatPort</b> the heatPort connector can be enabled and disabled (default = enabled). If it is disabled, the generated heat is transported implicitly to an internal temperature source with a fixed temperature of T_ref.</p><p>If the heatPort connector is enabled, it must be connected. </p>
</html>", revisions = "<html>
<ul>
<li><i> August 07, 2009   </i>
       by Anton Haumer<br> temperature dependency of resistance added<br>
       </li>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i> 2002   </i>
       by Anton Haumer<br> initially implemented<br>
       </li>
</ul>
</html>"));

        equation 
          assert(1 + alpha * (T_heatPort - T_ref) >= Modelica.Constants.eps, "Temperature outside scope of model!");
          R = R_ref * (1 + alpha * (T_heatPort - T_ref));
          v = R * i;
          LossPower = v * i;
        end HeatingResistor;
        model Conductor "Ideal linear electrical conductor"
          parameter Modelica.SIunits.Conductance G(start = 1) "Conductance at temperature T_ref";
          parameter Modelica.SIunits.Temperature T_ref = 300.15 "Reference temperature";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha = 0 "Temperature coefficient of conductance (G_actual = G_ref/(1 + alpha*(T_heatPort - T_ref))";
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = T_ref);
          Modelica.SIunits.Conductance G_actual "Actual conductance = G_ref/(1 + alpha*(T_heatPort - T_ref))";
          annotation(Documentation(info = "<html>
<p>The linear conductor connects the branch voltage <i>v</i> with the branch current <i>i</i> by <i>i = v*G</i>. The Conductance <i>G</i> is allowed to be positive, zero, or negative.  </p>
</html>", revisions = "<html>
<ul>
<li><i> August 07, 2009   </i>
       by Anton Haumer<br> temperature dependency of conductance added<br>
       </li>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -70,30},{70, -30}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{ -70,30},{70, -30}}, lineColor = {0,0,255}),Line(points = {{ -90,0},{ -70,0}}, color = {0,0,255}),Line(points = {{70,0},{90,0}}, color = {0,0,255}),Text(extent = {{ -136, -42},{142, -74}}, lineColor = {0,0,0}, pattern = LinePattern.None, textString = "G=%G"),Line(visible = useHeatPort, points = {{0, -100},{0, -30}}, color = {127,0,0}, smooth = Smooth.None, pattern = LinePattern.Dot),Text(extent = {{ -152,87},{148,47}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -144, -40},{142, -72}}, lineColor = {0,0,0}, textString = "G=%G")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -96,0},{ -70,0}}, color = {0,0,255}),Line(points = {{70,0},{96,0}}, color = {0,0,255}),Rectangle(extent = {{ -70,30},{70, -30}}, lineColor = {0,0,255})}));

        equation 
          assert(1 + alpha * (T_heatPort - T_ref) >= Modelica.Constants.eps, "Temperature outside scope of model!");
          G_actual = G / (1 + alpha * (T_heatPort - T_ref));
          i = G_actual * v;
          LossPower = v * i;
        end Conductor;
        model Capacitor "Ideal linear electrical capacitor"
          extends Interfaces.OnePort;
          parameter SI.Capacitance C(start = 1) "Capacitance";
          annotation(Documentation(info = "<html>
<p>The linear capacitor connects the branch voltage <i>v</i> with the branch current <i>i</i> by <i>i = C * dv/dt</i>. The Capacitance <i>C</i> is allowed to be positive, zero, or negative.  </p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -14,28},{ -14, -28}}, thickness = 0.5, color = {0,0,255}),Line(points = {{14,28},{14, -28}}, thickness = 0.5, color = {0,0,255}),Line(points = {{ -90,0},{ -14,0}}, color = {0,0,255}),Line(points = {{14,0},{90,0}}, color = {0,0,255}),Text(extent = {{ -136, -60},{136, -92}}, lineColor = {0,0,0}, textString = "C=%C"),Text(extent = {{ -150,85},{150,45}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -20,40},{ -20, -40}}, thickness = 0.5, color = {0,0,255}),Line(points = {{20,40},{20, -40}}, thickness = 0.5, color = {0,0,255}),Line(points = {{ -96,0},{ -20,0}}, color = {0,0,255}),Line(points = {{20,0},{96,0}}, color = {0,0,255})}));

        equation 
          i = C * der(v);
        end Capacitor;
        model Inductor "Ideal linear electrical inductor"
          extends Interfaces.OnePort;
          parameter SI.Inductance L(start = 1) "Inductance";
          annotation(Documentation(info = "<html>
<p>The linear inductor connects the branch voltage <i>v</i> with the branch current <i>i</i> by <i>v = L * di/dt</i>. The Inductance <i>L</i> is allowed to be positive, zero, or negative.  </p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Ellipse(extent = {{ -60, -15},{ -30,15}}, lineColor = {0,0,255}),Ellipse(extent = {{ -30, -15},{0,15}}, lineColor = {0,0,255}),Ellipse(extent = {{0, -15},{30,15}}, lineColor = {0,0,255}),Ellipse(extent = {{30, -15},{60,15}}, lineColor = {0,0,255}),Rectangle(extent = {{ -60, -30},{60,0}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{60,0},{90,0}}, color = {0,0,255}),Line(points = {{ -90,0},{ -60,0}}, color = {0,0,255}),Text(extent = {{ -138, -60},{144, -94}}, lineColor = {0,0,0}, textString = "L=%L"),Text(extent = {{ -152,79},{148,39}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Ellipse(extent = {{ -60, -15},{ -30,15}}, lineColor = {0,0,255}),Ellipse(extent = {{ -30, -15},{0,15}}, lineColor = {0,0,255}),Ellipse(extent = {{0, -15},{30,15}}, lineColor = {0,0,255}),Ellipse(extent = {{30, -15},{60,15}}, lineColor = {0,0,255}),Rectangle(extent = {{ -60, -30},{60,0}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{60,0},{96,0}}, color = {0,0,255}),Line(points = {{ -96,0},{ -60,0}}, color = {0,0,255})}));

        equation 
          L * der(i) = v;
        end Inductor;
        model SaturatingInductor "Simple model of an inductor with saturation"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          parameter Modelica.SIunits.Current Inom(start = 1) "Nominal current";
          parameter Modelica.SIunits.Inductance Lnom(start = 1) "Nominal inductance at Nominal current";
          parameter Modelica.SIunits.Inductance Lzer(start = 2 * Lnom) "Inductance near current=0";
          parameter Modelica.SIunits.Inductance Linf(start = Lnom / 2) "Inductance at large currents";
          Modelica.SIunits.Inductance Lact(start = Lzer);
          Modelica.SIunits.MagneticFlux Psi;
        protected 
          parameter Modelica.SIunits.Current Ipar(start = Inom / 10, fixed = false);
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -60, -15},{ -30,15}}, lineColor = {0,0,255}),Ellipse(extent = {{ -30, -15},{0,15}}, lineColor = {0,0,255}),Ellipse(extent = {{0, -15},{30,15}}, lineColor = {0,0,255}),Ellipse(extent = {{30, -15},{60,15}}, lineColor = {0,0,255}),Rectangle(extent = {{ -60, -20},{62,0}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{60,0},{90,0}}, color = {0,0,255}),Line(points = {{ -90,0},{ -60,0}}, color = {0,0,255}),Rectangle(extent = {{ -60, -10},{60, -20}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {0,0,255}),Text(extent = {{ -148, -50},{152, -80}}, lineColor = {0,0,0}, textString = "Lnom=%Lnom"),Text(extent = {{ -148,91},{152,51}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<html>
<p>This model approximates the behaviour of an inductor with the influence of saturation, i.e. the value of the inductance depends on the current flowing through the inductor. The inductance decreases as current increases. </p><p>The parameters are: </p>
<p><ul>
<li>Inom...nominal current</li>
<li>Lnom...nominal inductance at nominal current</li>
<li>Lzer...inductance near current = 0; Lzer has to be greater than Lnom</li>
<li>Linf...inductance at large currents; Linf has to be less than Lnom </li>
</ul></p>
</html>", revisions = "<html>
<dl>
<p>
  <dt><b>Main Author:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
</p>
<p>
  <dt><b>Release Notes:</b></dt>
  <dd>
  <ul>
  <li> May 27, 2004: Implemented by Anton Haumer</li>
  </ul>
  </dd>
</dl>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -60, -15},{ -30,15}}, lineColor = {0,0,255}),Ellipse(extent = {{ -30, -15},{0,15}}, lineColor = {0,0,255}),Ellipse(extent = {{0, -15},{30,15}}, lineColor = {0,0,255}),Ellipse(extent = {{30, -15},{60,15}}, lineColor = {0,0,255}),Rectangle(extent = {{ -60, -20},{62,0}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{60,0},{96,0}}, color = {0,0,255}),Line(points = {{ -96,0},{ -60,0}}, color = {0,0,255}),Rectangle(extent = {{ -60, -10},{60, -20}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {0,0,255})}));

        initial equation 
          Lnom - Linf = ((Lzer - Linf) * Ipar) / Inom * (Modelica.Constants.pi / 2 - Modelica.Math.atan(Ipar / Inom));

        equation 
          assert(Lzer > Lnom + Modelica.Constants.eps, "Lzer (= " + String(Lzer) + ") has to be > Lnom (= " + String(Lnom) + ")");
          assert(Linf < Lnom - Modelica.Constants.eps, "Linf (= " + String(Linf) + ") has to be < Lnom (= " + String(Lnom) + ")");
          ((Lact - Linf) * i) / Ipar = (Lzer - Linf) * noEvent(Modelica.Math.atan(i / Ipar));
          Psi = Lact * i;
          v = der(Psi);
        end SaturatingInductor;
        model Transformer "Transformer with two ports"
          extends Interfaces.TwoPort;
          parameter SI.Inductance L1(start = 1) "Primary inductance";
          parameter SI.Inductance L2(start = 1) "Secondary inductance";
          parameter SI.Inductance M(start = 1) "Coupling inductance";
          annotation(Documentation(info = "<html>
<p>The transformer is a two port. The left port voltage <i>v1</i>, left port current <i>i1</i>, right port voltage <i>v2</i> and right port current <i>i2</i> are connected by the following relation: </p>
<pre>         | v1 |         | L1   M  |  | i1&apos; |
         |    |    =    |         |  |     |
         | v2 |         | M    L2 |  | i2&apos; |</pre>
<p><i>L1</i>, <i>L2</i>, and <i>M</i> are the primary, secondary, and coupling inductances respectively. </p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -45, -50},{ -20, -25}}, lineColor = {0,0,255}),Ellipse(extent = {{ -45, -25},{ -20,0}}, lineColor = {0,0,255}),Ellipse(extent = {{ -45,0},{ -20,25}}, lineColor = {0,0,255}),Ellipse(extent = {{ -45,25},{ -20,50}}, lineColor = {0,0,255}),Rectangle(extent = {{ -72, -60},{ -33,60}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -90,50},{ -32,50}}, color = {0,0,255}),Line(points = {{ -90, -50},{ -32, -50}}, color = {0,0,255}),Ellipse(extent = {{20, -50},{45, -25}}, lineColor = {0,0,255}),Ellipse(extent = {{20, -25},{45,0}}, lineColor = {0,0,255}),Ellipse(extent = {{20,0},{45,25}}, lineColor = {0,0,255}),Ellipse(extent = {{20,25},{45,50}}, lineColor = {0,0,255}),Rectangle(extent = {{33, -60},{72,60}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{32,50},{90,50}}, color = {0,0,255}),Line(points = {{32, -50},{90, -50}}, color = {0,0,255}),Text(extent = {{ -103,16},{ -60, -10}}, textString = "L1", lineColor = {0,0,0}, pattern = LinePattern.Dot),Text(extent = {{64,15},{105, -9}}, textString = "L2", lineColor = {0,0,0}, pattern = LinePattern.Dot),Text(extent = {{ -27, -71},{33, -97}}, textString = "M", lineColor = {0,0,0}),Text(extent = {{ -153,113},{147,73}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -45, -50},{ -20, -25}}, lineColor = {0,0,255}),Ellipse(extent = {{ -45, -25},{ -20,0}}, lineColor = {0,0,255}),Ellipse(extent = {{ -45,0},{ -20,25}}, lineColor = {0,0,255}),Ellipse(extent = {{ -45,25},{ -20,50}}, lineColor = {0,0,255}),Rectangle(extent = {{ -72, -60},{ -33,60}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -96,50},{ -32,50}}, color = {0,0,255}),Line(points = {{ -96, -50},{ -32, -50}}, color = {0,0,255}),Ellipse(extent = {{20, -50},{45, -25}}, lineColor = {0,0,255}),Ellipse(extent = {{20, -25},{45,0}}, lineColor = {0,0,255}),Ellipse(extent = {{20,0},{45,25}}, lineColor = {0,0,255}),Ellipse(extent = {{20,25},{45,50}}, lineColor = {0,0,255}),Rectangle(extent = {{33, -60},{72,60}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{32,50},{96,50}}, color = {0,0,255}),Line(points = {{32, -50},{96, -50}}, color = {0,0,255})}));

        equation 
          v1 = L1 * der(i1) + M * der(i2);
          v2 = M * der(i1) + L2 * der(i2);
        end Transformer;
        model M_Transformer "Generic transformer with free number of inductors"
          parameter Integer N(final min = 1) = 3 "number of inductors";
        protected 
          parameter Integer dimL = div(N * (N + 1), 2);
        public 
          parameter Modelica.SIunits.Inductance L[dimL] = {1,0.1,0.2,2,0.3,3} "inductances and coupling inductances";
          Modelica.Electrical.Analog.Interfaces.PositivePin p[N] "Positive pin" annotation(extent = [ -80, -40; -62,40], Placement(transformation(extent = {{ -80, -40},{ -62,40}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin n[N] "Negative pin" annotation(extent = [62, -40;80,40], Placement(transformation(extent = {{62, -40},{80,40}}, rotation = 0)));
          Modelica.SIunits.Voltage v[N];
          Modelica.SIunits.Current i[N];
          Modelica.SIunits.Inductance Lm[N,N];
        algorithm 
          for s in 1:N loop
                      for z in 1:N loop
                          Lm[z,s]:=if z >= s then L[(s - 1) * N + z - div((s - 1) * s, 2)] else Lm[s,z];

            end for;

          end for;

        equation 
          for j in 1:N loop
          v[j] = p[j].v - n[j].v;
          0 = p[j].i + n[j].i;
          i[j] = p[j].i;

          end for;
          v = Lm * der(i);
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -36,24},{ -18,42}}),Ellipse(extent = {{18,24},{36,42}}),Ellipse(extent = {{0,24},{18,42}}),Ellipse(extent = {{ -18,24},{0,42}}),Rectangle(extent = {{42,24},{ -44,34}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -36,34},{ -62,34}}, color = {0,0,255}, pattern = LinePattern.None),Line(points = {{62,34},{36,34}}, color = {0,0,255}, pattern = LinePattern.None),Ellipse(extent = {{ -36,8},{ -18,26}}),Ellipse(extent = {{18,8},{36,26}}),Ellipse(extent = {{0,8},{18,26}}),Ellipse(extent = {{ -18,8},{0,26}}),Rectangle(extent = {{42,6},{ -44,18}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -36,18},{ -62,18}}, color = {0,0,255}, pattern = LinePattern.None),Line(points = {{62,18},{36,18}}, color = {0,0,255}, pattern = LinePattern.None),Line(points = {{ -36, -29},{ -62, -29}}, color = {0,0,255}, pattern = LinePattern.None),Ellipse(extent = {{ -36, -40},{ -18, -22}}),Ellipse(extent = {{ -18, -40},{0, -22}}),Ellipse(extent = {{0, -40},{18, -22}}),Ellipse(extent = {{18, -40},{36, -22}}),Line(points = {{62, -29},{36, -29}}, color = {0,0,255}, pattern = LinePattern.None),Rectangle(extent = {{42, -42},{ -44, -30}}, lineColor = {0,0,255}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,8},{0, -18}}, color = {0,0,255}, pattern = LinePattern.Dot),Text(extent = {{ -150,103},{150,63}}, textString = "%name", lineColor = {0,0,255})}, Ellipse(extent = [ -36,24; -18,42]), Ellipse(extent = [18,24;36,42]), Ellipse(extent = [0,24;18,42]), Ellipse(extent = [ -18,24;0,42]), Rectangle(extent = [42,24; -44,34], style(pattern = 0, fillColor = 7, rgbfillColor = {255,255,255})), Line(points = [ -36,34; -62,34], style(pattern = 0, fillColor = 7, rgbfillColor = {255,255,255}, fillPattern = 1)), Line(points = [62,34;36,34], style(pattern = 0, fillColor = 7, rgbfillColor = {255,255,255}, fillPattern = 1)), Ellipse(extent = [ -36,8; -18,26]), Ellipse(extent = [18,8;36,26]), Ellipse(extent = [0,8;18,26]), Ellipse(extent = [ -18,8;0,26]), Rectangle(extent = [42,6; -44,18], style(pattern = 0, fillColor = 7, rgbfillColor = {255,255,255})), Line(points = [ -36,18; -62,18], style(pattern = 0, fillColor = 7, rgbfillColor = {255,255,255}, fillPattern = 1)), Line(points = [62,18;36,18], style(pattern = 0, fillColor = 7, rgbfillColor = {255,255,255}, fillPattern = 1)), Line(points = [ -36, -29; -62, -29], style(pattern = 0, fillColor = 7, rgbfillColor = {255,255,255}, fillPattern = 1)), Ellipse(extent = [ -36, -40; -18, -22]), Ellipse(extent = [ -18, -40;0, -22]), Ellipse(extent = [0, -40;18, -22]), Ellipse(extent = [18, -40;36, -22]), Line(points = [62, -29;36, -29], style(pattern = 0, fillColor = 7, rgbfillColor = {255,255,255}, fillPattern = 1)), Rectangle(extent = [42, -42; -44, -30], style(pattern = 0, fillColor = 7, rgbfillColor = {255,255,255})), Line(points = [0,8;0, -18], style(pattern = 3, fillColor = 7, rgbfillColor = {255,255,255}, fillPattern = 1))), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p><br/><br/><br/><br/>The model <i>M_Transformer </i>is a model of a transformer with the posibility to choose the number of inductors. Inside the model, an inductance matrix is built based on the inductance of the inductors and the coupling inductances between the inductors given as a parameter vector from the user of the model. </p><p><br/><br/><br/><br/>An example shows that approach: </p><p><br/><br/><br/><br/>The user chooses a model with <b>three</b> inductors, that means the parameter <i><b>N </b></i>has to be <b>3</b>. Then he has to specify the inductances of the three inductors and the three coupling inductances. The coupling inductances are no real existing devices, but effects that occur between two inductors. The inductivities (main diagonal of the inductance matrix) and the coupling inductivities have to be specified in the parameter vector <i>L </i>. The length <i>dimL </i>of the parameter vector is calculated as follows: <i><b>dimL=(N*(N+1))/2 </b></i></p><p><br/><br/><br/><br/>The following example shows how the parameter vector is used to fill in the inductance matrix. For example: To specify the inductance matrix of a three inductances transformer (<i> N=3 </i>), e.g.</p>
<table cellspacing=\"2\" cellpadding=\"0\" border=\"0\"><tr>
<td></td>
<td></td>
<td><p><br/><br/>1</p></td>
<td><p>0.1</p></td>
<td><p>0.2</p></td>
</tr>
<tr>
<td><p>Lm</p></td>
<td><p>=</p></td>
<td><p>0.1</p></td>
<td><p>2</p></td>
<td><p>0.3</p></td>
</tr>
<tr>
<td></td>
<td></td>
<td><p><br/><br/>0.2</p></td>
<td><p>0.3</p></td>
<td><p>3</p></td>
</tr>
</table>
<p><br/><br/>the user has to allocate the parameter vector <i>L[6] </i>, since <i>Nv=(N*(N+1))/2=(3*(3+1))/2=6</i>. The parameter vector must be filled like this: <i>L=[1,0.1,0.2,2,0.3,3] </i>. </p><p><br/><br/>Inside the model, two loops are used to fill the inductance matrix to guarantee that it is filled in a symmetric way. </p>
</html>", revisions = "
<html>
<ul>
<li><i> November 24, 2008   </i> docu added, K. Majetta
       </li>
<li><i> September 16, 2008   </i>
       by Kristin Majetta<br> initially implemented<br>
       </li>
</ul>
</html>
"));
        end M_Transformer;
        model Gyrator "Gyrator"
          extends Interfaces.TwoPort;
          parameter SI.Conductance G1(start = 1) "Gyration conductance";
          parameter SI.Conductance G2(start = 1) "Gyration conductance";
          annotation(Documentation(info = "<html>
<p>A gyrator is a two-port element defined by the following equations:  </p>
<pre>    i1 =  G2 * v2
    i2 = -G1 * v1</pre>
<p>where the constants <i>G1</i>, <i>G2</i> are called the gyration conductance. </p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -90,50},{ -60,50}}, color = {0,0,255}),Line(points = {{ -90, -50},{ -60, -50}}, color = {0,0,255}),Line(points = {{60,50},{90,50}}, color = {0,0,255}),Line(points = {{60, -50},{90, -50}}, color = {0,0,255}),Line(points = {{ -40,30},{40,30}}, color = {0,0,255}),Line(points = {{ -20, -20},{20, -20}}, color = {0,0,255}),Polygon(points = {{30,34},{40,30},{30,26},{30,34}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{ -26, -19},{ -16, -15},{ -16, -23},{ -26, -19}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -5,10},{ -10, -10}}, color = {0,0,0}),Line(points = {{9,10},{4, -9}}, color = {0,0,0}),Line(points = {{ -14,9},{16,10}}, color = {0,0,0}),Text(extent = {{ -29,59},{30,30}}, textString = "G1", lineColor = {0,0,0}),Text(extent = {{ -29, -29},{29, -58}}, textString = "G2", lineColor = {0,0,0}),Text(extent = {{ -156,117},{144,77}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,255}),Line(points = {{ -96,50},{ -60,50}}, color = {0,0,255}),Line(points = {{ -96, -50},{ -60, -50}}, color = {0,0,255}),Line(points = {{60,50},{96,50}}, color = {0,0,255}),Line(points = {{60, -50},{96, -50}}, color = {0,0,255}),Line(points = {{ -40,30},{40,30}}, color = {0,0,255}),Line(points = {{ -21, -22},{19, -22}}, color = {0,0,255}),Polygon(points = {{30,34},{40,30},{30,26},{30,34}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{ -20, -30},{ -10, -26},{ -10, -34},{ -20, -30}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -4,5},{ -6, -5}}, thickness = 1, color = {0,0,255}),Line(points = {{3,5},{1, -5}}, thickness = 1, color = {0,0,255}),Line(points = {{ -8,5},{7,5}}, thickness = 1, color = {0,0,255}),Text(extent = {{ -20,50},{20,35}}, textString = "G1", lineColor = {0,0,255}),Text(extent = {{ -20, -35},{20, -50}}, textString = "G2", lineColor = {0,0,255})}));

        equation 
          i1 = G2 * v2;
          i2 =  -G1 * v1;
        end Gyrator;
        model EMF "Electromotoric force (electric/mechanic transformer)"
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
          parameter SI.ElectricalTorqueConstant k(start = 1) "Transformation coefficient";
          SI.Voltage v "Voltage drop between the two pins";
          SI.Current i "Current flowing from positive to negative pin";
          SI.Angle phi "Angle of shaft flange with respect to support (= flange.phi - support.phi)";
          SI.AngularVelocity w "Angular velocity of flange relative to support";
          Interfaces.PositivePin p annotation(Placement(transformation(origin = {0,100}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Interfaces.NegativePin n annotation(Placement(transformation(origin = {0, -100}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Mechanics.Rotational.Interfaces.Flange_b flange annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Mechanics.Rotational.Interfaces.Support support if useSupport "Support/housing of emf shaft" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}})));
          annotation(defaultComponentName = "emf", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -85,10},{ -36, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Line(points = {{0,90},{0,40}}, color = {0,0,255}),Rectangle(extent = {{35,10},{100, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Ellipse(extent = {{ -40,40},{40, -40}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{0, -90},{0, -40}}, color = {0,0,255}),Text(extent = {{0, -50},{199, -90}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{0,80},{189,46}}, lineColor = {160,160,164}, textString = "k=%k"),Line(visible = not useSupport, points = {{ -100, -30},{ -40, -30}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -100, -50},{ -80, -30}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -80, -50},{ -60, -30}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -60, -50},{ -40, -30}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -70, -30},{ -70, -10}}, color = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -17,95},{ -20,85},{ -23,95},{ -17,95}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Line(points = {{ -20,110},{ -20,85}}, color = {160,160,164}),Text(extent = {{ -40,110},{ -30,90}}, lineColor = {160,160,164}, textString = "i"),Line(points = {{9,75},{19,75}}, color = {192,192,192}),Line(points = {{ -20, -110},{ -20, -85}}, color = {160,160,164}),Polygon(points = {{ -17, -100},{ -20, -110},{ -23, -100},{ -17, -100}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Text(extent = {{ -40, -110},{ -30, -90}}, lineColor = {160,160,164}, textString = "i"),Line(points = {{8, -79},{18, -79}}, color = {192,192,192}),Line(points = {{14,80},{14,70}}, color = {192,192,192})}), Documentation(info = "<html>
<p>EMF transforms electrical energy into rotational mechanical energy. It is used as basic building block of an electrical motor. The mechanical connector flange can be connected to elements of the Modelica.Mechanics.Rotational library. flange.tau is the cut-torque, flange.phi is the angle at the rotational connection.  </p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Martin Otter<br> initially implemented<br>
       </li>
</ul>
</html>"));
        protected 
          Mechanics.Rotational.Components.Fixed fixed if not useSupport annotation(Placement(transformation(extent = {{ -90, -20},{ -70,0}})));
          Mechanics.Rotational.Interfaces.InternalSupport internalSupport(tau =  -flange.tau) annotation(Placement(transformation(extent = {{ -90, -10},{ -70,10}})));

        equation 
          v = p.v - n.v;
          0 = p.i + n.i;
          i = p.i;
          phi = flange.phi - internalSupport.phi;
          w = der(phi);
          k * w = v;
          flange.tau =  -k * i;
          connect(internalSupport.flange,support) annotation(Line(points = {{ -80,0},{ -100,0}}, color = {0,0,0}, smooth = Smooth.None));
          connect(internalSupport.flange,fixed.flange) annotation(Line(points = {{ -80,0},{ -80, -10}}, color = {0,0,0}, smooth = Smooth.None));
        end EMF;
        model TranslationalEMF "Electromotoric force (electric/mechanic transformer)"
          parameter Boolean useSupport = false "= true, if support flange enabled, otherwise implicitly grounded" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
          parameter Modelica.SIunits.ElectricalForceConstant k(start = 1) "Transformation coefficient";
          Modelica.SIunits.Voltage v "Voltage drop between the two pins";
          Modelica.SIunits.Current i "Current flowing from positive to negative pin";
          Modelica.SIunits.Position s "Position of flange relative to support";
          Modelica.SIunits.Velocity vel "Velocity of flange relative to support";
          annotation(defaultComponentName = "emf", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -90,51},{ -40, -50}}, lineColor = {135,135,135}, fillColor = {135,135,135}, fillPattern = FillPattern.HorizontalCylinder),Rectangle(extent = {{ -21,20},{90, -20}}, lineColor = {135,135,135}, fillColor = {135,135,135}, fillPattern = FillPattern.Solid),Text(extent = {{0,80},{148,44}}, lineColor = {160,160,164}, textString = "k=%k"),Line(points = {{ -30,49},{ -30,80},{0,80},{0,91}}, color = {0,0,255}),Line(points = {{20, -49},{20, -80},{0, -80},{0, -89},{0, -90}}, color = {0,0,255}),Ellipse(extent = {{ -21,50},{9, -50}}, lineColor = {0,0,255}),Ellipse(extent = {{2,50},{32, -50}}, lineColor = {0,0,255}),Ellipse(extent = {{ -43,50},{ -13, -50}}, lineColor = {0,0,255}),Rectangle(extent = {{ -4,20},{ -1, -20}}, lineColor = {135,135,135}, fillColor = {135,135,135}, fillPattern = FillPattern.Solid),Rectangle(extent = {{7,20},{10, -20}}, lineColor = {135,135,135}, fillColor = {135,135,135}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -14,20},{ -11, -20}}, lineColor = {135,135,135}, fillColor = {135,135,135}, fillPattern = FillPattern.Solid),Rectangle(extent = {{19,20},{44, -20}}, lineColor = {135,135,135}, fillColor = {135,135,135}, fillPattern = FillPattern.Solid),Line(visible = not useSupport, points = {{ -100, -70},{ -40, -70}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -100, -90},{ -80, -70}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -80, -90},{ -60, -70}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -60, -90},{ -40, -70}}, color = {0,0,0}),Line(visible = not useSupport, points = {{ -70, -70},{ -70, -50}}, color = {0,0,0}),Text(extent = {{0, -50},{199, -90}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -17,95},{ -20,85},{ -23,95},{ -17,95}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Line(points = {{ -20,110},{ -20,85}}, color = {160,160,164}),Text(extent = {{ -40,110},{ -30,90}}, lineColor = {160,160,164}, textString = "i"),Line(points = {{9,75},{19,75}}, color = {192,192,192}),Line(points = {{ -20, -110},{ -20, -85}}, color = {160,160,164}),Polygon(points = {{ -17, -100},{ -20, -110},{ -23, -100},{ -17, -100}}, lineColor = {160,160,164}, fillColor = {160,160,164}, fillPattern = FillPattern.Solid),Text(extent = {{ -40, -110},{ -30, -90}}, lineColor = {160,160,164}, textString = "i"),Line(points = {{8, -79},{18, -79}}, color = {192,192,192}),Line(points = {{14,80},{14,70}}, color = {192,192,192}),Polygon(points = {{140,3},{150,0},{140, -3},{140,3},{140,3}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Window(x = 0.21, y = 0, width = 0.65, height = 0.66), Documentation(info = "<html>
<p>EMF transforms electrical energy into translational mechanical energy. It is used as basic building block of an electrical linear motor. The mechanical connector flange can be connected to elements of the Modelica.Mechanics.Translational library. flange.f is the cut-force, flange.s is the distance at the translational connection.  </p>
</html>", revisions = "<html>
<ul>
<li><i>  </i>
       </li>
<li><i> 2009   </i>
       by Anton Haumer<br> initially implemented<br>
       </li>
</ul>
</html>"));
          Modelica.Electrical.Analog.Interfaces.PositivePin p annotation(Placement(transformation(origin = {0,100}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Electrical.Analog.Interfaces.NegativePin n annotation(Placement(transformation(origin = {0, -100}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Mechanics.Translational.Interfaces.Flange_b flange annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.Mechanics.Translational.Interfaces.Support support if useSupport "Support/housing" annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}})));
        protected 
          Modelica.Mechanics.Translational.Components.Fixed fixed if not useSupport annotation(Placement(transformation(extent = {{ -90, -20},{ -70,0}})));
          Modelica.Mechanics.Translational.Interfaces.InternalSupport internalSupport(f =  -flange.f) annotation(Placement(transformation(extent = {{ -90, -10},{ -70,10}})));

        equation 
          v = p.v - n.v;
          0 = p.i + n.i;
          i = p.i;
          s = flange.s - internalSupport.s;
          vel = der(s);
          k * vel = v;
          flange.f =  -k * i;
          connect(internalSupport.flange,support) annotation(Line(points = {{ -80,0},{ -90,0},{ -90,0},{ -100,0}}, color = {0,127,0}, smooth = Smooth.None));
          connect(internalSupport.flange,fixed.flange) annotation(Line(points = {{ -80,0},{ -80, -10}}, color = {0,127,0}, smooth = Smooth.None));
        end TranslationalEMF;
        model VCV "Linear voltage-controlled voltage source"
          extends Interfaces.TwoPort;
          parameter Real gain(start = 1) "Voltage gain";
          annotation(Documentation(info = "<html>
<p>The linear voltage-controlled voltage source is a TwoPort. The right port voltage v2 is controlled by the left port voltage v1 via </p>
<pre>    v2 = v1 * gain. </pre>
<p>The left port current is zero. Any voltage gain can be chosen. </p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -70,70},{70, -70}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -90,50},{ -30,50}}, color = {0,0,255}),Line(points = {{ -30, -50},{ -90, -50}}, color = {0,0,255}),Line(points = {{100,50},{30,50},{30, -50},{100, -50}}, color = {0,0,255}),Ellipse(extent = {{10,20},{50, -20}}, lineColor = {0,0,255}),Line(points = {{ -20,60},{20,60}}, color = {0,0,255}),Polygon(points = {{20,60},{10,63},{10,57},{20,60}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -148, -81},{152, -121}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -96,50},{ -31,50}}, color = {0,0,255}),Line(points = {{ -30, -50},{ -96, -50}}, color = {0,0,255}),Line(points = {{96,50},{30,50},{30, -50},{96, -50}}, color = {0,0,255}),Ellipse(extent = {{10,20},{50, -20}}, lineColor = {0,0,255}),Rectangle(extent = {{ -70,70},{70, -70}}, lineColor = {0,0,255}),Line(points = {{ -20,60},{20,60}}, color = {0,0,255}),Polygon(points = {{20,60},{10,63},{10,57},{20,60}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255})}));

        equation 
          v2 = v1 * gain;
          i1 = 0;
        end VCV;
        model VCC "Linear voltage-controlled current source"
          extends Interfaces.TwoPort;
          parameter SI.Conductance transConductance(start = 1) "Transconductance";
          annotation(Documentation(info = "<html>
<p>The linear voltage-controlled current source is a TwoPort. The right port current i2 is controlled by the left port voltage v1 via </p>
<pre>    i2 = v1 * transConductance. </pre>
<p>The left port current is zero. Any transConductance can be chosen. </p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -70,70},{70, -70}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -90,50},{ -30,50}}, color = {0,0,255}),Line(points = {{ -30, -50},{ -90, -50}}, color = {0,0,255}),Ellipse(extent = {{10,20},{50, -20}}, lineColor = {0,0,255}),Line(points = {{ -20,60},{20,60}}, color = {0,0,255}),Polygon(points = {{20,60},{10,63},{10,57},{20,60}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{90,50},{30,50},{30,20}}, color = {0,0,255}),Line(points = {{91, -50},{30, -50},{30, -20}}, color = {0,0,255}),Line(points = {{10,0},{50,0}}, color = {0,0,255}),Text(extent = {{ -143, -82},{157, -122}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -96,50},{ -30,50}}, color = {0,0,255}),Line(points = {{ -30, -50},{ -96, -50}}, color = {0,0,255}),Ellipse(extent = {{10,20},{50, -20}}, lineColor = {0,0,255}),Rectangle(extent = {{ -70,70},{70, -70}}, lineColor = {0,0,255}),Line(points = {{ -20,60},{20,60}}, color = {0,0,255}),Polygon(points = {{20,60},{10,63},{10,57},{20,60}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{96,50},{30,50},{30,20}}, color = {0,0,255}),Line(points = {{96, -50},{30, -50},{30, -20}}, color = {0,0,255}),Line(points = {{10,0},{50,0}}, color = {0,0,255})}));

        equation 
          i2 = v1 * transConductance;
          i1 = 0;
        end VCC;
        model CCV "Linear current-controlled voltage source"
          extends Interfaces.TwoPort;
          parameter SI.Resistance transResistance(start = 1) "Transresistance";
          annotation(Documentation(info = "<html>
<p>The linear current-controlled voltage source is a TwoPort. The right port voltage v2 is controlled by the left port current i1 via </p>
<pre>    v2 = i1 * transResistance. </pre>
<p>The left port voltage is zero. Any transResistance can be chosen. </p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -70,70},{70, -70}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{100,50},{30,50},{30, -50},{100, -50}}, color = {0,0,255}),Ellipse(extent = {{10,20},{50, -20}}, lineColor = {0,0,255}),Line(points = {{ -20,60},{20,60}}, color = {0,0,255}),Polygon(points = {{20,60},{10,63},{10,57},{20,60}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -90,50},{ -20,50},{ -20, -50},{ -90, -50}}, color = {0,0,255}),Text(extent = {{ -140, -82},{160, -122}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{10,20},{50, -20}}, lineColor = {0,0,255}),Rectangle(extent = {{ -70,70},{70, -70}}, lineColor = {0,0,255}),Line(points = {{ -20,60},{20,60}}, color = {0,0,255}),Polygon(points = {{20,60},{10,63},{10,57},{20,60}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{96,50},{30,50},{30, -50},{96, -50}}, color = {0,0,255}),Line(points = {{ -96,50},{ -30,50},{ -30, -50},{ -96, -50}}, color = {0,0,255})}));

        equation 
          v2 = i1 * transResistance;
          v1 = 0;
        end CCV;
        model CCC "Linear current-controlled current source"
          extends Interfaces.TwoPort;
          parameter Real gain(start = 1) "Current gain";
          annotation(Documentation(info = "<html>
<p>The linear current-controlled current source is a TwoPort. The right port current i2 is controlled by the left port current i1 via </p>
<pre>    i2 = i1 * gain. </pre>
<p>The left port voltage is zero. Any current gain can be chosen. </p>
</html>", revisions = "<html>
<ul>
<li><i> 1998   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -70,70},{70, -70}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -100,50},{ -30,50},{ -30, -50},{ -100, -50}}, color = {0,0,255}),Ellipse(extent = {{10,20},{50, -20}}, lineColor = {0,0,255}),Line(points = {{ -20,60},{20,60}}, color = {0,0,255}),Polygon(points = {{20,60},{10,63},{10,57},{20,60}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{90,50},{30,50},{30,20}}, color = {0,0,255}),Line(points = {{91, -50},{30, -50},{30, -20}}, color = {0,0,255}),Line(points = {{10,0},{50,0}}, color = {0,0,255}),Text(extent = {{ -146, -76},{154, -116}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{10,20},{50, -20}}, lineColor = {0,0,255}),Rectangle(extent = {{ -70,70},{70, -70}}, lineColor = {0,0,255}),Line(points = {{ -20,60},{20,60}}, color = {0,0,255}),Polygon(points = {{20,60},{10,63},{10,57},{20,60}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{96,50},{30,50},{30,20}}, color = {0,0,255}),Line(points = {{96, -50},{30, -50},{30, -20}}, color = {0,0,255}),Line(points = {{10,0},{50,0}}, color = {0,0,255}),Line(points = {{ -96,50},{ -30,50},{ -30, -50},{ -96, -50}}, color = {0,0,255})}));

        equation 
          i2 = i1 * gain;
          v1 = 0;
        end CCC;
        model OpAmp "Simple nonideal model of an OpAmp with limitation"
          parameter Real Slope(start = 1) "Slope of the out.v/vin characteristic at vin=0";
          Modelica.Electrical.Analog.Interfaces.PositivePin in_p "Positive pin of the input port" annotation(Placement(transformation(extent = {{ -110, -60},{ -90, -40}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin in_n "Negative pin of the input port" annotation(Placement(transformation(extent = {{ -90,40},{ -110,60}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.PositivePin out "Output pin" annotation(Placement(transformation(extent = {{110, -10},{90,10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.PositivePin VMax "Positive output voltage limitation" annotation(Placement(transformation(extent = {{ -10,60},{10,80}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin VMin "Negative output voltage limitation" annotation(Placement(transformation(extent = {{ -10, -80},{10, -60}}, rotation = 0)));
          SI.Voltage vin "input voltagae";
        protected 
          Real f "auxiliary variable";
          Real absSlope;
          annotation(Documentation(info = "<html>
<p>The OpAmp is a simle nonideal model with a smooth out.v = f(vin) characteristic, where &QUOT;vin = in_p.v - in_n.v&QUOT;. The characteristic is limited by VMax.v and VMin.v. Its slope at vin=0 is the parameter Slope, which must be positive. (Therefore, the absolute value of Slope is taken into calculation.)  </p>
</html>", revisions = "<html>
<ul>
<li><i> 2000   </i>
       by Christoph Clauss<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{60,0},{ -60,70},{ -60, -70},{60,0}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -45, -10},{ -20, -10},{ -14, -9},{ -11, -7},{ -9,7},{ -6,9},{0,10},{20,10}}, color = {0,0,255}),Line(points = {{0,35},{0,80}}, color = {0,0,255}),Line(points = {{0, -35},{0, -80}}, color = {0,0,255}),Line(points = {{ -90,50},{ -60,50}}, color = {0,0,255}),Line(points = {{ -90, -50},{ -60, -50}}, color = {0,0,255}),Line(points = {{60,0},{90,0}}, color = {0,0,255}),Line(points = {{ -48,32},{ -28,32}}, color = {0,0,255}),Line(points = {{ -39, -20},{ -39, -41}}, color = {0,0,255}),Line(points = {{ -50, -31},{ -28, -31}}, color = {0,0,255}),Text(extent = {{ -149,138},{151,98}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{60,0},{ -60,70},{ -60, -70},{60,0}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -45, -10},{ -20, -10},{ -14, -9},{ -11, -7},{ -9,7},{ -6,9},{0,10},{20,10}}, color = {0,0,255}),Line(points = {{0,35},{0,70}}, color = {0,0,255}),Line(points = {{0, -35},{0, -70}}, color = {0,0,255}),Line(points = {{ -96,50},{ -60,50}}, color = {0,0,255}),Line(points = {{ -96, -50},{ -60, -50}}, color = {0,0,255}),Line(points = {{60,0},{96,0}}, color = {0,0,255}),Line(points = {{ -55,50},{ -45,50}}, color = {0,0,255}),Line(points = {{ -50, -45},{ -50, -55}}, color = {0,0,255}),Line(points = {{ -55, -50},{ -45, -50}}, color = {0,0,255}),Text(extent = {{ -112, -93},{ -91, -73}}, lineColor = {160,160,164}, textString = "in_p.i=0"),Polygon(points = {{120,3},{110,0},{120, -3},{120,3}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {160,160,164}),Line(points = {{111,0},{136,0}}, color = {0,0,0}),Text(extent = {{118,2},{135,17}}, lineColor = {0,0,0}, textString = "i2"),Text(extent = {{ -111,60},{ -85,80}}, lineColor = {160,160,164}, textString = "in_n.i=0"),Line(points = {{ -100, -35},{ -100,23},{ -100,24}}, color = {160,160,164}, arrow = {Arrow.None,Arrow.Filled}),Text(extent = {{ -97, -16},{ -74,4}}, lineColor = {160,160,164}, textString = "vin")}));

        equation 
          in_p.i = 0;
          in_n.i = 0;
          VMax.i = 0;
          VMin.i = 0;
          vin = in_p.v - in_n.v;
          f = 2 / (VMax.v - VMin.v);
          absSlope = smooth(0, if Slope < 0 then  -Slope else Slope);
          out.v = (VMax.v + VMin.v) / 2 + (absSlope * vin) / (1 + absSlope * smooth(0, if f * vin < 0 then  -f * vin else f * vin));
        end OpAmp;
        model OpAmpDetailed "Detailed model of an operational amplifier"
          import SI = Modelica.SIunits;
          parameter SI.Resistance Rdm = 2000000.0 "Input resistance (differential input mode)";
          parameter SI.Resistance Rcm = 2000000000.0 "Input resistance (common mode)";
          parameter SI.Capacitance Cin = 1.4e-12 "Input capacitance";
          parameter SI.Voltage Vos = 0.001 "Input offset voltage";
          parameter SI.Current Ib = 8e-08 "Input bias current";
          parameter SI.Current Ios = 2e-08 "Input offset current";
          parameter SI.Voltage vcp = 0.0 "Correction value for limiting by p_supply";
          parameter SI.Voltage vcm = 0.0 "Correction value for limiting by msupply";
          parameter Real Avd0 = 106.0 "Differential amplifier [dB]";
          parameter Real CMRR = 90.0 "Common-mode rejection [dB]";
          parameter SI.Frequency fp1 = 5.0 "Dominant pole";
          parameter SI.Frequency fp2 = 2000000.0 "Pole frequency";
          parameter SI.Frequency fp3 = 20000000.0 "Pole frequency";
          parameter SI.Frequency fp4 = 100000000.0 "Pole frequency";
          parameter SI.Frequency fz = 5000000.0 "Zero frequency";
          parameter SI.VoltageSlope sr_p = 500000.0 "Slew rate for increase";
          parameter SI.VoltageSlope sr_m = 500000.0 "Slew rate for decrease";
          parameter SI.Resistance Rout = 75.0 "Output resistance";
          parameter SI.Current Imaxso = 0.025 "Maximal output current (source current)";
          parameter SI.Current Imaxsi = 0.025 "Maximal output current (sink current)";
          parameter SI.Time Ts = 1.2e-06 "sampling time";
          constant Real Pi = 3.141592654;
          final parameter SI.Voltage vcp_abs = abs(vcp) "Positive correction value for limiting by p_supply";
          final parameter SI.Voltage vcm_abs = abs(vcm) "Positive correction value for limiting by msupply";
          final parameter SI.Current I1 = Ib + Ios / 2.0 "Current of internal source I1";
          final parameter SI.Current I2 = Ib - Ios / 2.0 "Current of internal source I2";
          final parameter Real Avd0_val = 10.0 ^ (Avd0 / 20.0) "differential mode gain";
          final parameter Real Avcm_val = Avd0_val / 10.0 ^ (CMRR / 20.0) / 2.0 "common mode gain";
          final parameter SI.VoltageSlope sr_p_val = abs(sr_p) "Value of slew rate for increase";
          final parameter SI.VoltageSlope sr_m_val =  -abs(sr_m) "Negative alue of slew rate for increase";
          final parameter SI.Current Imaxso_val = abs(Imaxso) "Orientation out outp";
          final parameter SI.Current Imaxsi_val = abs(Imaxsi) "Orientation into outp";
          Modelica.Electrical.Analog.Interfaces.PositivePin p "Positive pin of the input port" annotation(Placement(transformation(extent = {{ -111, -61},{ -90, -40}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin m "Negative pin of the input port" annotation(Placement(transformation(extent = {{ -90,40},{ -111,61}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.PositivePin outp "Output pin" annotation(Placement(transformation(extent = {{110, -10},{90,10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.PositivePin p_supply "Positive output voltage limitation" annotation(Placement(transformation(extent = {{ -8,65},{9,82}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin m_supply "Negative output voltage limitation" annotation(Placement(transformation(extent = {{ -9, -83},{9, -65}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>The OpAmpDetailed model is a general operational amplifier model. The emphasis is on separating each important data sheet parameter into a sub-circuit independent of the other parameters. The model is broken down into five functional stages <b>input</b>, <b>frequency response</b>, <b>gain</b>, <b>slew rate</b> and an <b>output</b> stage. Each stage contains data sheet parameters to be modeled. This partitioning and the modelling of the separate submodels are based on the description in <b>[CP92]</b>.  </p>
<p>Using <b>[CP92]</b> Joachim Haase (Fraunhofer Institute for Integrated Circuits, Design Automation Division) transfered 2001 operational amplifier models into VHDL-AMS. Now one of these models, the model &QUOT;amp(macro)&QUOT; was transferred into Modelica. </p>
<dl><dt><b>Reference:</b> </dt>
<dd><b>[CP92]</b> Conelly, J.A.; Choi, P.: Macromodelling with SPICE. Englewood Cliffs: Prentice-Hall, 1992 </dd>
</dl></html>", revisions = "<html>
<ul>
<li><i>  </i>
       </li>
<li><i> Juni 17, 2009   </i>
       by Susann Wolf<br> initially implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{60,0},{ -60,70},{ -60, -70},{60,0}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{0,35},{0,80}}, color = {0,0,255}),Line(points = {{0, -35},{0, -80}}, color = {0,0,255}),Line(points = {{ -90,50},{ -60,50}}, color = {0,0,255}),Line(points = {{ -90, -50},{ -60, -50}}, color = {0,0,255}),Line(points = {{60,0},{90,0}}, color = {0,0,255}),Line(points = {{ -48,32},{ -28,32}}, color = {0,0,255}),Line(points = {{ -39, -20},{ -39, -41}}, color = {0,0,255}),Line(points = {{ -50, -31},{ -28, -31}}, color = {0,0,255}),Text(extent = {{ -151,141},{149,101}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{60,0},{ -60,70},{ -60, -70},{60,0}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{0,35},{0,70}}, color = {0,0,255}),Line(points = {{0, -35},{0, -70}}, color = {0,0,255}),Line(points = {{ -96,50},{ -60,50}}, color = {0,0,255}),Line(points = {{ -96, -50},{ -60, -50}}, color = {0,0,255}),Line(points = {{60,0},{96,0}}, color = {0,0,255}),Line(points = {{ -55,50},{ -45,50}}, color = {0,0,255}),Line(points = {{ -50, -45},{ -50, -55}}, color = {0,0,255}),Line(points = {{ -55, -50},{ -45, -50}}, color = {0,0,255}),Polygon(points = {{120,3},{110,0},{120, -3},{120,3}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {160,160,164}),Line(points = {{111,0},{136,0}}, color = {0,0,0}),Text(extent = {{114,2},{131,17}}, lineColor = {0,0,0}, textString = "i2"),Line(points = {{ -100, -35},{ -100,23},{ -100,24}}, color = {160,160,164}, arrow = {Arrow.None,Arrow.Filled}),Text(extent = {{ -97, -16},{ -74,4}}, lineColor = {160,160,164}, textString = "vin")}));
          SI.Voltage v_pos;
          SI.Voltage v_neg;
          Modelica.SIunits.Voltage v_vos;
          Modelica.SIunits.Voltage v_3;
          Modelica.SIunits.Voltage v_in;
          Modelica.SIunits.Voltage v_4;
          Modelica.SIunits.Current i_vos;
          Modelica.SIunits.Current i_3;
          Modelica.SIunits.Current i_r2;
          Modelica.SIunits.Current i_c3;
          Modelica.SIunits.Current i_4;
          Real q_fr1;
          Real q_fr2;
          Real q_fr3;
          SI.Voltage q_sum;
          SI.Voltage q_sum_help;
          SI.Voltage q_fp1;
          SI.Voltage v_source;
          SI.Voltage x "auxiliary variable for slew rate";
          Modelica.SIunits.Voltage v_out;
          Modelica.SIunits.Current i_out;
          function FCNiout_limit "Internal limitation function"
            input SI.Voltage v_source;
            input SI.Voltage v_out;
            input SI.Resistance Rout;
            input SI.Current Imaxsi_val;
            input SI.Current Imaxso_val;
            output SI.Current result;
          algorithm 
            if v_out > v_source + Rout * Imaxsi_val then 
                        result:=Imaxsi_val;
            elseif v_out < v_source - Rout * Imaxso_val then
              result:= -Imaxso_val;


            else             result:=(v_out - v_source) / Rout;

            end if;
return;
            annotation(Documentation(info = "<html>
<p>Internal limitation function, designed for OpAmpDetailed, not for purpose of external usage. </p>
</html>"));
          end FCNiout_limit;
          function FCNq_sum_limit "Internal limitation function"
            input SI.Voltage q_sum;
            input SI.Voltage q_sum_ltf;
            input SI.Voltage v_pos;
            input SI.Voltage v_neg;
            input SI.Voltage vcp;
            input SI.Voltage vcm;
            output SI.Voltage result;
          algorithm 
            if q_sum > v_pos - vcp and q_sum_ltf >= v_pos - vcp then 
                        result:=v_pos - vcp;
            elseif q_sum < v_neg + vcm and q_sum_ltf <= v_neg + vcm then
              result:=v_neg + vcm;


            else             result:=q_sum;

            end if;
return;
            annotation(Documentation(info = "<html>
<p>Internal limitation function, designed for OpAmpDetailed, not for purpose of external usage. </p>
</html>"));
          end FCNq_sum_limit;

        equation 
          assert(Rout > 0.0, "Rout must be > 0.0.");
          v_pos = p_supply.v;
          v_neg = m_supply.v;
          p.i = i_vos;
          m.i = i_4 - i_r2 - i_c3;
          0 = i_3 + i_r2 + i_c3 - i_vos;
          p.v - m.v = v_vos + v_in;
          v_4 = m.v;
          v_3 = p.v - v_vos;
          v_vos = Vos;
          i_3 = I1 + v_3 / Rcm;
          v_in = Rdm * i_r2;
          i_c3 = Cin * der(v_in);
          i_4 = I2 + v_4 / Rcm;
          der(q_fr1) = 2.0 * Pi * fp2 * (v_in - q_fr1);
          q_fr2 + 1.0 / (2.0 * Pi * fp3) * der(q_fr2) = q_fr1 + 1.0 / (2.0 * Pi * fz) * der(q_fr1);
          der(q_fr3) = 2.0 * Pi * fp4 * (q_fr2 - q_fr3);
          q_sum = Avd0_val * q_fr3 + Avcm_val * (v_3 + v_4);
          q_sum_help = FCNq_sum_limit(q_sum, q_fp1, v_pos, v_neg, vcp_abs, vcm_abs);
          der(q_fp1) = 2.0 * Pi * fp1 * (q_sum_help - q_fp1);
if initial() then
                      v_source = q_fp1;
            x = 0;
          end if;
          der(x) = (q_fp1 - v_source) / Ts;
          der(v_source) = smooth(0, noEvent(if der(x) > sr_p_val then sr_p_val else if der(x) < sr_m_val then sr_m_val else der(x)));
          v_out = outp.v;
          i_out = outp.i;
          i_out = FCNiout_limit(v_source, v_out, Rout, Imaxsi_val, Imaxso_val);
          p_supply.i = 0;
          m_supply.i = 0;
        end OpAmpDetailed;
        model VariableResistor "Ideal linear electrical resistor with variable resistance"
          parameter Modelica.SIunits.Temperature T_ref = 300.15 "Reference temperature";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha = 0 "Temperature coefficient of resistance (R_actual = R*(1 + alpha*(T_heatPort - T_ref))";
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = T_ref);
          Modelica.SIunits.Resistance R_actual "Actual resistance = R*(1 + alpha*(T_heatPort - T_ref))";
          Modelica.Blocks.Interfaces.RealInput R annotation(Placement(transformation(origin = {0,110}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
          annotation(Documentation(info = "<html>
<p><br/>The linear resistor connects the branch voltage <i>v</i> with the branch current <i>i</i> by  </p><p><br/><i><b>i*R = v</b></i> </p>
<p>The Resistance <i>R</i> is given as input signal. </p>
<p><br/><b>Attention!!!</b></p><p><br/>It is recommended that the R signal should not cross the zero value. Otherwise depending on the surrounding circuit the probability of singularities is high. </p>
</html>", revisions = "<html>
<ul>
<li><i> August 07, 2009   </i>
       by Anton Haumer<br> temperature dependency of resistance added<br>
       </li>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>June 7, 2004   </i>
       by Christoph Clauss<br>changed, docu added<br>
       </li>
<li><i>April 30, 2004</i>
       by Anton Haumer<br>implemented.
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -90,0},{ -70,0}}, color = {0,0,255}),Rectangle(extent = {{ -70,30},{70, -30}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{70,0},{90,0}}, color = {0,0,255}),Line(points = {{0,90},{0,30}}, color = {0,0,255}),Text(extent = {{ -148, -41},{152, -81}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -70,30},{70, -30}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -96,0},{ -70,0}}, color = {0,0,255}),Line(points = {{0,90},{0,30}}, color = {0,0,255}),Line(points = {{70,0},{96,0}}, color = {0,0,255})}));

        equation 
          assert(1 + alpha * (T_heatPort - T_ref) >= Modelica.Constants.eps, "Temperature outside scope of model!");
          R_actual = R * (1 + alpha * (T_heatPort - T_ref));
          v = R_actual * i;
          LossPower = v * i;
        end VariableResistor;
        model VariableConductor "Ideal linear electrical conductor with variable conductance"
          parameter Modelica.SIunits.Temperature T_ref = 300.15 "Reference temperature";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha = 0 "Temperature coefficient of conductance (G_actual = G/(1 + alpha*(T_heatPort - T_ref))";
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          extends Modelica.Electrical.Analog.Interfaces.ConditionalHeatPort(T = T_ref);
          Modelica.SIunits.Conductance G_actual "Actual conductance = G/(1 + alpha*(T_heatPort - T_ref))";
          Modelica.Blocks.Interfaces.RealInput G annotation(Placement(transformation(origin = {0,110}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
          annotation(Documentation(info = "<html>
<p><br/>The linear conductor connects the branch voltage <i>v</i> with the branch current <i>i</i> by  </p><p><br/><i><b>i = G*v</b></i> </p>
<p>The Conductance <i>G</i> is given as input signal. </p>
<p><br/><b>Attention!!!</b></p><p><br/>It is recommended that the G signal should not cross the zero value. Otherwise depending on the surrounding circuit the probability of singularities is high. </p>
</html>", revisions = "<html>
<ul>
<li><i> August 07, 2009   </i>
       by Anton Haumer<br> temperature dependency of conductance added<br>
       </li>
<li><i> March 11, 2009   </i>
       by Christoph Clauss<br> conditional heat port added<br>
       </li>
<li><i>June 7, 2004   </i>
       by Christoph Clauss<br> implemented<br>
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -90,0},{ -70,0}}, color = {0,0,255}),Rectangle(extent = {{ -70,30},{70, -30}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{70,0},{90,0}}, color = {0,0,255}),Line(points = {{0,90},{0,30}}, color = {0,0,255}),Text(extent = {{ -148, -41},{152, -81}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{0,90},{0,30}}, color = {0,0,255}),Line(points = {{ -96,0},{ -70,0}}, color = {0,0,255}),Line(points = {{70,0},{96,0}}, color = {0,0,255}),Rectangle(extent = {{ -70,30},{70, -30}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}));

        equation 
          assert(1 + alpha * (T_heatPort - T_ref) >= Modelica.Constants.eps, "Temperature outside scope of model!");
          G_actual = G / (1 + alpha * (T_heatPort - T_ref));
          i = G_actual * v;
          LossPower = v * i;
        end VariableConductor;
        model VariableCapacitor "Ideal linear electrical capacitor with variable capacitance"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          Modelica.Blocks.Interfaces.RealInput C annotation(Placement(transformation(origin = {0,110}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
          parameter Modelica.SIunits.Capacitance Cmin = Modelica.Constants.eps "lower bound for variable capacitance";
          Modelica.SIunits.ElectricCharge Q;
          annotation(Documentation(info = "<html>
<p><br/>The linear capacitor connects the branch voltage <i>v</i> with the branch current <i>i</i> by  </p><p><br/><i><b>i = dQ/dt</b></i> with <i><b>Q = C * v</b></i><b> </b>. </p>
<p>The capacitance <i>C</i> is given as input signal. </p>
<p>It is required that C &ge; 0, otherwise an assertion is raised. To avoid a variable index system,</p><p>C = Cmin, if 0 &le; C &LT; Cmin, where Cmin is a parameter with default value Modelica.Constants.eps. </p>
</html>", revisions = "<html>
<ul>
<li><i>June 7, 2004   </i>
       by Christoph Clauss<br>changed, docu added<br>
       </li>
<li><i>April 30, 2004</i>
       by Anton Haumer<br>implemented.
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -90,0},{ -14,0}}, color = {0,0,255}),Line(points = {{14,0},{90,0}}, color = {0,0,255}),Line(points = {{0,90},{0,30}}, color = {0,0,255}),Line(points = {{ -14,28},{ -14, -28}}, thickness = 0.5, color = {0,0,255}),Line(points = {{14,28},{14, -28}}, thickness = 0.5, color = {0,0,255}),Text(extent = {{ -144, -43},{156, -83}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -96,0},{ -14,0}}, color = {0,0,255}),Line(points = {{14,0},{96,0}}, color = {0,0,255}),Line(points = {{0,90},{0,30}}, color = {0,0,255}),Line(points = {{ -14,28},{ -14, -28}}, thickness = 0.5, color = {0,0,255}),Line(points = {{14,28},{14, -28}}, thickness = 0.5, color = {0,0,255})}));

        equation 
          assert(C >= 0, "Capacitance C (= " + String(C) + ") has to be >= 0!");
          Q = noEvent(max(C, Cmin)) * v;
          i = der(Q);
        end VariableCapacitor;
        model VariableInductor "Ideal linear electrical inductor with variable inductance"
          extends Modelica.Electrical.Analog.Interfaces.OnePort;
          Modelica.Blocks.Interfaces.RealInput L annotation(Placement(transformation(origin = {0,108}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
          Modelica.SIunits.MagneticFlux Psi;
          parameter Modelica.SIunits.Inductance Lmin = Modelica.Constants.eps "lower bound for variable inductance";
          annotation(Documentation(info = "<html>
<p><br/>The linear inductor connects the branch voltage <i>v</i> with the branch current <i>i</i> by  </p><p><br/><i><b>v = d Psi/dt </b></i>with <i><b>Psi = L * i </b></i>. </p>
<p>The inductance <i>L</i> is as input signal. </p>
<p>It is required that L &ge; 0, otherwise an assertion is raised. To avoid a variable index system,</p><p>L = Lmin, if 0 &le; L &LT; Lmin, where Lmin is a parameter with default value Modelica.Constants.eps. </p>
</html>", revisions = "<html>
<ul>
<li><i>June 7, 2004   </i>
       by Christoph Clauss<br>changed, docu added<br>
       </li>
<li><i>April 30, 2004</i>
       by Anton Haumer<br>implemented.
       </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -90,0},{ -60,0}}, color = {0,0,255}),Line(points = {{60,0},{90,0}}, color = {0,0,255}),Line(points = {{0,90},{0,8}}, color = {0,0,255}),Ellipse(extent = {{ -60, -15},{ -30,15}}, lineColor = {0,0,255}),Ellipse(extent = {{ -30, -15},{0,15}}, lineColor = {0,0,255}),Ellipse(extent = {{0, -15},{30,15}}, lineColor = {0,0,255}),Ellipse(extent = {{30, -15},{60,15}}, lineColor = {0,0,255}),Rectangle(extent = {{ -60, -30},{60,0}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -146, -47},{154, -87}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -96,0},{ -60,0}}, color = {0,0,255}),Line(points = {{0,90},{0,8}}, color = {0,0,255}),Ellipse(extent = {{ -60, -15},{ -30,15}}, lineColor = {0,0,255}),Ellipse(extent = {{ -30, -15},{0,15}}, lineColor = {0,0,255}),Ellipse(extent = {{0, -15},{30,15}}, lineColor = {0,0,255}),Ellipse(extent = {{30, -15},{60,15}}, lineColor = {0,0,255}),Rectangle(extent = {{ -60, -30},{62,0}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{60,0},{96,0}}, color = {0,0,255})}));

        equation 
          assert(L >= 0, "Inductance L_ (= " + String(L) + ") has to be >= 0!");
          Psi = noEvent(max(L, Lmin)) * i;
          v = der(Psi);
        end VariableInductor;
      end Basic;
    end Analog;
    package MultiPhase "Library for electrical components with 2, 3 or more phases"
      extends Modelica.Icons.Library2;
      annotation(Documentation(info = "<HTML>
<p>
This package contains packages for electrical multiphase components, based on Modelica.Electrical.Analog:
<ul>
<li>Basic: basic components (resistor, capacitor, inductor, ...)</li>
<li>Ideal: ideal elements (switches, diode, transformer, ...)</li>
<li>Sensors: sensors to measure potentials, voltages, and currents</li>
<li>Sources: time-dependend and controlled voltage and current sources</li>
</ul>
This package is intended to be used the same way as Modelica.Electrical.Analog
but to make design of multiphase models easier.<br>
The package is based on the plug: a composite connector containing m pins.<br>
It is possible to connect plugs to plugs or single pins of a plug to single pins.<br>
Potentials may be accessed as <tt>plug.pin[].v</tt>, currents may be accessed as <tt>plug.pin[].i</tt>.
</p>
<p>
Further development:
<ul>
<li>temperature-dependent resistor</li>
<li>lines (m-phase models)</li>
</ul>
</p>
<dl>
<p>
  <dt><b>Main Author:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
</p>
</dl>
<p>
Copyright &copy; 1998-2005, Modelica Association and Anton Haumer.
</p>
<p>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p><br>
</HTML>", revisions = "<html>
  <ul>
  <li>v1.0 2004/10/01 Anton Haumer</li>
  <li>v1.1 2006/01/12 Anton Haumer<br>
      added Sensors.PowerSensor</li>
  <li>v1.2 2006/07/05 Anton Haumer<br>
      removed annotation from pin of Interfaces.Plug<br>
      corrected usage of resistance/conductance</li>
  <li>v1.3.0 2007/01/23 Anton Haumer<br>
      improved some icons</li>
  <li>v1.3.1 2007/08/12 Anton Haumer<br>
      improved documentation</li>
  <li>v1.3.2 2007/08/24 Anton Haumer<br>
      removed redeclare type SignalType</li>
  <li>v1.4.0 2009/08/26 Anton Haumer<br>
      added conditional HeatPorts as Electrical.Analog<br>
      added switches with arc as Electrical.Analog</li>
  </ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -60,10},{40, -90}}, lineColor = {0,0,255}),Ellipse(extent = {{ -40, -14},{ -20, -34}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{0, -14},{20, -34}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -20, -54},{0, -74}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid)}));
      package Examples "Multiphase test examples"
        extends Modelica.Icons.Library2;
        annotation(Documentation(info = "<HTML>
<p>
This package contains test examples of analog electrical multiphase circuits.
</p>

</HTML>", revisions = "<html>
<dl>
<p>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
</p>
<p>
  <dt><b>Release Notes:</b></dt>
  <dd>
  <ul>
  <li> v1.0 2004/10/01 Anton Haumer</li>
  </ul>
  </dd>
<p>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2006, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</p>
</dl>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -60,12},{40, -88}}, lineColor = {135,135,135}),Polygon(points = {{ -30, -10},{ -30, -66},{28, -38},{ -30, -10}}, lineColor = {135,135,135}, fillColor = {135,135,135}, fillPattern = FillPattern.Solid)}));
        model TransformerYY "Test example with multiphase components"
          extends Modelica.Icons.Example;
          parameter Integer m = 3 "Number of phases";
          parameter Modelica.SIunits.Voltage V = 1 "Amplitude of Star-Voltage";
          parameter Modelica.SIunits.Frequency f = 5 "Frequency";
          parameter Modelica.SIunits.Inductance LT = 0.003 "Transformer stray inductance";
          parameter Modelica.SIunits.Resistance RT = 0.05 "Transformer resistance";
          parameter Modelica.SIunits.Resistance RL = 1 "Load Resistance";
          annotation(Documentation(info = "<HTML>
<p>
Test example with multiphase components:<br>
Star-connected voltage source feeds via a Y-Y-transformer with internal impedance (RT, LT) a load resistor RT.<br>
Using f=5 Hz LT=3mH defines nominal voltage drop of approximately 10 %.<br>
Simulate for 1 second (2 periods) and compare voltages and currents of source, transformer and load.
</p>
</HTML>"), Diagram(graphics));
          Sources.SineVoltage sineVoltage(V = fill(V, m), freqHz = fill(f, m), m = m) annotation(Placement(transformation(origin = { -80,20}, extent = {{ -10, -10},{10,10}}, rotation = 180)));
          Basic.Star starS(m = m) annotation(Placement(transformation(origin = { -90, -62}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground groundS annotation(Placement(transformation(extent = {{ -100, -100},{ -80, -80}}, rotation = 0)));
          Ideal.IdealTransformer idealTransformer(m = m) annotation(Placement(transformation(extent = {{ -40,0},{ -20,20}}, rotation = 0)));
          Basic.Star starT1(m = m) annotation(Placement(transformation(origin = { -40, -62}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Basic.Star starT2(m = m) annotation(Placement(transformation(origin = { -20, -62}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground groundT2 annotation(Placement(transformation(extent = {{ -30, -100},{ -10, -80}}, rotation = 0)));
          Basic.Resistor transformerR(m = m, R = fill(RT, m)) annotation(Placement(transformation(extent = {{0,10},{20,30}}, rotation = 0)));
          Basic.Inductor transformerL(m = m, L = fill(LT, m)) annotation(Placement(transformation(extent = {{30,10},{50,30}}, rotation = 0)));
          Basic.Resistor loadR(m = m, R = fill(RL, m)) annotation(Placement(transformation(extent = {{70,10},{90,30}}, rotation = 0)));
          Basic.Star starL(m = m) annotation(Placement(transformation(origin = {90, -62}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground groundT1 annotation(Placement(transformation(extent = {{ -50, -100},{ -30, -80}}, rotation = 0)));

        equation 
          connect(starS.pin_n,groundS.p) annotation(Line(points = {{ -90, -72},{ -90, -80}}, color = {0,0,255}));
          connect(starT1.pin_n,groundT1.p) annotation(Line(points = {{ -40, -72},{ -40, -80}}, color = {0,0,255}));
          connect(starT2.pin_n,groundT2.p) annotation(Line(points = {{ -20, -72},{ -20, -80}}, color = {0,0,255}));
          connect(starS.plug_p,sineVoltage.plug_n) annotation(Line(points = {{ -90, -52},{ -90,20}}, color = {0,0,255}));
          connect(sineVoltage.plug_p,idealTransformer.plug_p1) annotation(Line(points = {{ -70,20},{ -40,20}}, color = {0,0,255}));
          connect(idealTransformer.plug_n1,starT1.plug_p) annotation(Line(points = {{ -40,0},{ -40, -52}}, color = {0,0,255}));
          connect(starT2.plug_p,idealTransformer.plug_n2) annotation(Line(points = {{ -20, -52},{ -20,0}}, color = {0,0,255}));
          connect(idealTransformer.plug_p2,transformerR.plug_p) annotation(Line(points = {{ -20,20},{0,20}}, color = {0,0,255}));
          connect(transformerR.plug_n,transformerL.plug_p) annotation(Line(points = {{20,20},{30,20}}, color = {0,0,255}));
          connect(transformerL.plug_n,loadR.plug_p) annotation(Line(points = {{50,20},{70,20}}, color = {0,0,255}));
          connect(loadR.plug_n,starL.plug_p) annotation(Line(points = {{90,20},{90, -52}}, color = {0,0,255}));
        end TransformerYY;
        model TransformerYD "Test example with multiphase components"
          extends Modelica.Icons.Example;
          parameter Integer m = 3 "Number of phases";
          parameter Modelica.SIunits.Voltage V = 1 "Amplitude of Star-Voltage";
          parameter Modelica.SIunits.Frequency f = 5 "Frequency";
          parameter Modelica.SIunits.Inductance LT = 0.003 "Transformer stray inductance";
          parameter Modelica.SIunits.Resistance RT = 0.05 "Transformer resistance";
          parameter Modelica.SIunits.Resistance RL = 1 "Load Resistance";
          parameter Real nT = 1 / sqrt((1 - Modelica.Math.cos((2 * Modelica.Constants.pi) / m)) ^ 2 + Modelica.Math.sin((2 * Modelica.Constants.pi) / m) ^ 2) "Transformer ratio";
          annotation(Documentation(info = "<HTML>
<p>
Test example with multiphase components:<br>
Star-connected voltage source feeds via a Y-D-transformer with internal impedance (RT, LT) a load resistor RT.<br>
Using f=5 Hz LT=3mH defines nominal voltage drop of approximately 10 %.<br>
Simulate for 1 second (2 periods) and compare voltages and currents of source, transformer and load.
</p>
</HTML>"), Diagram(graphics));
          Sources.SineVoltage sineVoltage(V = fill(V, m), freqHz = fill(f, m), m = m) annotation(Placement(transformation(origin = { -80,20}, extent = {{ -10, -10},{10,10}}, rotation = 180)));
          Basic.Star starS(m = m) annotation(Placement(transformation(origin = { -90, -62}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground groundS annotation(Placement(transformation(extent = {{ -100, -100},{ -80, -80}}, rotation = 0)));
          Ideal.IdealTransformer idealTransformer(m = m, n = fill(nT, m)) annotation(Placement(transformation(extent = {{ -40,0},{ -20,20}}, rotation = 0)));
          Basic.Star starT(m = m) annotation(Placement(transformation(origin = { -40, -62}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Basic.Delta deltaT2(m = m) annotation(Placement(transformation(origin = {50,10}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground groundT annotation(Placement(transformation(extent = {{ -50, -100},{ -30, -80}}, rotation = 0)));
          Basic.Resistor transformerR(m = m, R = fill(RT / nT ^ 2, m)) annotation(Placement(transformation(extent = {{ -10,10},{10,30}}, rotation = 0)));
          Basic.Inductor transformerL(m = m, L = fill(LT / nT ^ 2, m)) annotation(Placement(transformation(extent = {{20,10},{40,30}}, rotation = 0)));
          Basic.Resistor loadR(m = m, R = fill(RL, m)) annotation(Placement(transformation(extent = {{70,10},{90,30}}, rotation = 0)));
          Basic.Star starL(m = m) annotation(Placement(transformation(origin = {90, -62}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground groundL annotation(Placement(transformation(extent = {{80, -100},{100, -80}}, rotation = 0)));

        equation 
          connect(groundS.p,starS.pin_n) annotation(Line(points = {{ -90, -80},{ -90, -72}}, color = {0,0,255}));
          connect(groundT.p,starT.pin_n) annotation(Line(points = {{ -40, -80},{ -40, -72}}, color = {0,0,255}));
          connect(starS.plug_p,sineVoltage.plug_n) annotation(Line(points = {{ -90, -52},{ -90,20}}, color = {0,0,255}));
          connect(sineVoltage.plug_p,idealTransformer.plug_p1) annotation(Line(points = {{ -70,20},{ -40,20}}, color = {0,0,255}));
          connect(idealTransformer.plug_n1,starT.plug_p) annotation(Line(points = {{ -40,0},{ -40, -52}}, color = {0,0,255}));
          connect(idealTransformer.plug_p2,transformerR.plug_p) annotation(Line(points = {{ -20,20},{ -10,20}}, color = {0,0,255}));
          connect(transformerR.plug_n,transformerL.plug_p) annotation(Line(points = {{10,20},{20,20}}, color = {0,0,255}));
          connect(transformerL.plug_n,deltaT2.plug_p) annotation(Line(points = {{40,20},{50,20}}, color = {0,0,255}));
          connect(deltaT2.plug_n,idealTransformer.plug_n2) annotation(Line(points = {{50,0},{ -20,0}}, color = {0,0,255}));
          connect(deltaT2.plug_p,loadR.plug_p) annotation(Line(points = {{50,20},{70,20}}, color = {0,0,255}));
          connect(loadR.plug_n,starL.plug_p) annotation(Line(points = {{90,20},{90, -52}}, color = {0,0,255}));
          connect(starL.pin_n,groundL.p) annotation(Line(points = {{90, -72},{90, -80}}, color = {0,0,255}));
        end TransformerYD;
        model Rectifier "Test example with multiphase components"
          extends Modelica.Icons.Example;
          parameter Integer m = 3 "Number of phases";
          parameter Modelica.SIunits.Voltage V = 1 "Amplitude of Star-Voltage";
          parameter Modelica.SIunits.Frequency f = 5 "Frequency";
          parameter Modelica.SIunits.Inductance L = 0.001 "Line Inductance";
          parameter Modelica.SIunits.Resistance RL = 2 "Load Resistance";
          parameter Modelica.SIunits.Capacitance C = 0.05 "Total DC-Capacitance";
          parameter Modelica.SIunits.Resistance RE = 1000000.0 "Earthing Resistance";
          annotation(Documentation(info = "<HTML>
<p>
Test example with multiphase components:<br>
Star-connected voltage source feeds via a line reactor a diode bridge rectifier with a DC burden.<br>
Using f=5 Hz, simulate for 1 second (2 periods) and compare voltages and currents of source and DC burden,
neglecting initial transient.
</p>
</HTML>"), Diagram(graphics));
          Sources.SineVoltage sineVoltage(m = m, V = fill(V, m), freqHz = fill(f, m)) annotation(Placement(transformation(extent = {{ -70,10},{ -90, -10}}, rotation = 0)));
          Basic.Star starS(m = m) annotation(Placement(transformation(origin = { -90, -50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Basic.Inductor supplyL(m = m, L = fill(L, m)) annotation(Placement(transformation(extent = {{ -52, -10},{ -32,10}}, rotation = 0)));
          Ideal.IdealDiode idealDiode1(m = m) annotation(Placement(transformation(origin = {10,20}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Basic.Star star1(m = m) annotation(Placement(transformation(origin = {10,50}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Ideal.IdealDiode idealDiode2(m = m) annotation(Placement(transformation(origin = {10, -20}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Basic.Star star2(m = m) annotation(Placement(transformation(origin = {10, -50}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Resistor loadR(R = RL) annotation(Placement(transformation(origin = {50,0}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Capacitor cDC1(C = 2 * C) annotation(Placement(transformation(origin = {70,30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Capacitor cDC2(C = 2 * C) annotation(Placement(transformation(origin = {70, -30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground groundDC annotation(Placement(transformation(extent = {{80, -80},{100, -60}}, rotation = 0)));

        equation 
          connect(cDC1.n,cDC2.p) annotation(Line(points = {{70,20},{70, -20}}, color = {0,0,255}));
          connect(cDC1.n,groundDC.p) annotation(Line(points = {{70,20},{70,0},{90,0},{90, -60}}, color = {0,0,255}));
          connect(starS.plug_p,sineVoltage.plug_n) annotation(Line(points = {{ -90, -40},{ -90,0}}, color = {0,0,255}));
          connect(sineVoltage.plug_p,supplyL.plug_p) annotation(Line(points = {{ -70,0},{ -52,0}}, color = {0,0,255}));
          connect(idealDiode1.plug_p,supplyL.plug_n) annotation(Line(points = {{10,10},{10,0},{ -32,0}}, color = {0,0,255}));
          connect(idealDiode2.plug_n,supplyL.plug_n) annotation(Line(points = {{10, -10},{10,0},{ -32,0}}, color = {0,0,255}));
          connect(idealDiode1.plug_n,star1.plug_p) annotation(Line(points = {{10,30},{10,40}}, color = {0,0,255}));
          connect(idealDiode2.plug_p,star2.plug_p) annotation(Line(points = {{10, -30},{10, -40}}, color = {0,0,255}));
          connect(star2.pin_n,loadR.n) annotation(Line(points = {{10, -60},{50, -60},{50, -10}}, color = {0,0,255}));
          connect(star2.pin_n,cDC2.n) annotation(Line(points = {{10, -60},{70, -60},{70, -40}}, color = {0,0,255}));
          connect(star1.pin_n,loadR.p) annotation(Line(points = {{10,60},{50,60},{50,10}}, color = {0,0,255}));
          connect(star1.pin_n,cDC1.p) annotation(Line(points = {{10,60},{70,60},{70,40}}, color = {0,0,255}));
        end Rectifier;
      end Examples;
      package Basic "Basic components for electrical multiphase models"
        extends Modelica.Icons.Library2;
        annotation(Documentation(info = "<HTML>
<p>
This package contains basic analog electrical multiphase components.
</p>

</HTML>", revisions = "<html>
<dl>
<p>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
</p>
<p>
  <dt><b>Release Notes:</b></dt>
  <dd>
  <ul>
  <li> v1.0 2004/10/01 Anton Haumer</li>
  </ul>
  </dd>
<p>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2006, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</p>
</dl>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -80, -10},{60, -70}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{60, -40},{80, -40}}, color = {0,0,255}),Line(points = {{ -100, -40},{ -80, -40}}, color = {0,0,255})}));
        model Star "Star-connection"
          parameter Integer m(final min = 1) = 3 "number of phases";
          Interfaces.PositivePlug plug_p(final m = m) annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin pin_n annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -150,60},{150,120}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{80,0},{0,0}}, thickness = 0.5, color = {0,0,255}),Line(points = {{0,0},{ -39,68}}, thickness = 0.5, color = {0,0,255}),Line(points = {{0,0},{ -38, -69}}, thickness = 0.5, color = {0,0,255}),Text(extent = {{ -100, -110},{100, -70}}, lineColor = {0,0,0}, textString = "m=%m"),Line(points = {{ -90,0},{ -40,0}}, color = {0,0,255}),Line(points = {{80,0},{90,0}}, color = {0,0,255})}), Documentation(info = "<HTML>
<p>
Connects all pins of plug_p to pin_n, thus establishing a so-called star-connection.
</p>
</HTML>"));

        equation 
          for j in 1:m loop
          connect(plug_p.pin[j],pin_n);

          end for;
        end Star;
        model Delta "Delta (polygon) connection"
          parameter Integer m(final min = 2) = 3 "number of phases";
          Interfaces.PositivePlug plug_p(final m = m) annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Interfaces.NegativePlug plug_n(final m = m) annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -150,60},{150,120}}, lineColor = {0,0,255}, textString = "%name"),Line(points = {{ -40,68},{ -40, -70},{79,0},{ -40,68},{ -40,67}}, thickness = 0.5, color = {0,0,255}),Text(extent = {{ -100, -110},{100, -70}}, lineColor = {0,0,0}, textString = "m=%m"),Line(points = {{ -90,0},{ -40,0}}, color = {0,0,255}),Line(points = {{80,0},{90,0}}, color = {0,0,255})}), Documentation(info = "<HTML>
<p>
Connects in a cyclic way plug_n.pin[j] to plug_p.pin[j+1],
thus establishing a so-called delta (or polygon) connection
when used in parallel to another component.
</p>
</HTML>"));

        equation 
          for j in 1:m loop
          if j < m then
            connect(plug_n.pin[j],plug_p.pin[j + 1]);

          else
            connect(plug_n.pin[j],plug_p.pin[1]);

          end if;

          end for;
        end Delta;
        model PlugToPin_p "Connect one (positive) Pin"
          parameter Integer m(final min = 1) = 3 "number of phases";
          parameter Integer k(final min = 1, final max = m, start = 1) "phase index";
          Interfaces.PositivePlug plug_p(final m = m) annotation(Placement(transformation(extent = {{ -30, -10},{ -10,10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.PositivePin pin_p annotation(Placement(transformation(extent = {{10, -10},{30,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -150,100},{150,40}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100, -60},{100, -100}}, lineColor = {0,0,0}, textString = "k = %k"),Line(points = {{ -20,20},{40,20},{40, -20},{ -20, -20}}, color = {0,0,0}),Rectangle(extent = {{ -20,20},{40, -20}}, pattern = LinePattern.None, fillColor = {215,215,215}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Ellipse(extent = {{ -40,20},{0, -20}}, lineColor = {0,0,0}, fillColor = {215,215,215}, fillPattern = FillPattern.Solid),Line(points = {{ -20,20},{40,20},{40, -20},{ -20, -20}}, color = {0,0,0})}), Documentation(info = "<HTML>
<p>
Connects pin <i>k</i> of plug_p to pin_p, leaving the other pins of plug_p unconnected.
</p>
</HTML>"), Diagram(graphics));

        equation 
          pin_p.v = plug_p.pin[k].v;
          for j in 1:m loop
          plug_p.pin[j].i = if j == k then  -pin_p.i else 0;

          end for;
        end PlugToPin_p;
        model PlugToPin_n "Connect one (negative) Pin"
          parameter Integer m(final min = 1) = 3 "number of phases";
          parameter Integer k(final min = 1, final max = m, start = 1) "phase index";
          Interfaces.NegativePlug plug_n(final m = m) annotation(Placement(transformation(extent = {{ -30, -10},{ -10,10}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin pin_n annotation(Placement(transformation(extent = {{10, -10},{30,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -150,100},{150,40}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100, -60},{100, -100}}, lineColor = {0,0,0}, textString = "k = %k"),Line(points = {{ -20,20},{40,20},{40, -20},{ -20, -20}}, color = {0,0,0}),Rectangle(extent = {{ -20,20},{40, -20}}, pattern = LinePattern.None, fillColor = {215,215,215}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Ellipse(extent = {{ -40,20},{0, -20}}, lineColor = {0,0,0}, fillColor = {215,215,215}, fillPattern = FillPattern.Solid),Line(points = {{ -20,20},{40,20},{40, -20},{ -20, -20}}, color = {0,0,0})}), Documentation(info = "<HTML>
<p>
Connects pin <i>k</i> of plug_n to pin_n, leaving the other pins of plug_n unconnected.
</p>
</HTML>"), Diagram(graphics));

        equation 
          pin_n.v = plug_n.pin[k].v;
          for j in 1:m loop
          plug_n.pin[j].i = if j == k then  -pin_n.i else 0;

          end for;
        end PlugToPin_n;
        model Resistor "Ideal linear electrical resistors"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Resistance R[m](start = fill(1, m)) "Resistances R_ref at temperatures T_ref";
          parameter Modelica.SIunits.Temperature T_ref[m] = fill(300.15, m) "Reference temperatures";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha[m] = zeros(m) "Temperature coefficients of resistances at reference temperatures";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, T = T_ref);
          Modelica.Electrical.Analog.Basic.Resistor resistor[m](final R = R, final T_ref = T_ref, final alpha = alpha, each final useHeatPort = useHeatPort, final T = T) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -70,30},{70, -30}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{ -70,0}}, color = {0,0,255}),Line(points = {{70,0},{90,0}}, color = {0,0,255}),Text(extent = {{ -150,40},{150,100}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100, -100},{100, -60}}, lineColor = {0,0,0}, textString = "m=%m")}), Documentation(info = "<HTML>
<p>
Contains m resistors (Modelica.Electrical.Analog.Basic.Resistor)
</p>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          connect(resistor.p,plug_p.pin) annotation(Line(points = {{ -10,0},{ -100,0}}, color = {0,0,255}));
          connect(resistor.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
          connect(resistor.heatPort,heatPort) annotation(Line(points = {{0, -10},{0, -100}}, color = {191,0,0}, smooth = Smooth.None));
        end Resistor;
        model Conductor "Ideal linear electrical conductors"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Conductance G[m](start = fill(1, m)) "Conductances G_ref at temperatures T_ref";
          parameter Modelica.SIunits.Temperature T_ref[m] = fill(300.15, m) "Reference temperatures";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha[m] = zeros(m) "Temperature coefficients of conductances at reference temperatures";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, T = T_ref);
          Modelica.Electrical.Analog.Basic.Conductor conductor[m](final G = G, final T_ref = T_ref, final alpha = alpha, each final useHeatPort = useHeatPort, final T = T) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -70,30},{70, -30}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{ -70,0}}, color = {0,0,255}),Line(points = {{70,0},{90,0}}, color = {0,0,255}),Text(extent = {{ -150,40},{150,100}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100, -100},{100, -60}}, lineColor = {0,0,0}, textString = "m=%m")}), Documentation(info = "<HTML>
<p>
Contains m conductors (Modelica.Electrical.Analog.Basic.Conductor)
</p>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          connect(plug_p.pin,conductor.p) annotation(Line(points = {{ -100,0},{ -10,0}}, color = {0,0,255}));
          connect(plug_n.pin,conductor.n) annotation(Line(points = {{100,0},{10,0}}, color = {0,0,255}));
          connect(conductor.heatPort,heatPort) annotation(Line(points = {{0, -10},{0, -100}}, color = {191,0,0}, smooth = Smooth.None));
        end Conductor;
        model Capacitor "Ideal linear electrical capacitors"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Capacitance C[m](start = fill(1, m)) "Capacitance";
          Modelica.Electrical.Analog.Basic.Capacitor capacitor[m](final C = C) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -150,40},{150,100}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -14,28},{ -14, -28}}, thickness = 0.5, color = {0,0,255}),Line(points = {{14,28},{14, -28}}, thickness = 0.5, color = {0,0,255}),Line(points = {{ -90,0},{ -14,0}}, color = {0,0,255}),Line(points = {{14,0},{90,0}}, color = {0,0,255}),Text(extent = {{ -100, -100},{100, -60}}, lineColor = {0,0,0}, textString = "m=%m")}), Documentation(info = "<HTML>
<p>
Contains m capacitors (Modelica.Electrical.Analog.Basic.Capacitor)
</p>
</HTML>"), Diagram(graphics));

        equation 
          connect(capacitor.p,plug_p.pin) annotation(Line(points = {{ -10,0},{ -100,0}}, color = {0,0,255}));
          connect(capacitor.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
        end Capacitor;
        model Inductor "Ideal linear electrical inductors"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Inductance L[m](start = fill(1, m)) "Inductance";
          Modelica.Electrical.Analog.Basic.Inductor inductor[m](final L = L) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -60, -15},{ -30,15}}, lineColor = {0,0,255}),Ellipse(extent = {{ -30, -15},{0,15}}, lineColor = {0,0,255}),Ellipse(extent = {{0, -15},{30,15}}, lineColor = {0,0,255}),Ellipse(extent = {{30, -15},{60,15}}, lineColor = {0,0,255}),Rectangle(extent = {{ -60, -30},{60,0}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{60,0},{90,0}}, color = {0,0,255}),Line(points = {{ -90,0},{ -60,0}}, color = {0,0,255}),Text(extent = {{ -150,40},{150,100}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100, -100},{100, -60}}, lineColor = {0,0,0}, textString = "m=%m")}), Documentation(info = "<HTML>
<p>
Contains m inductors (Modelica.Electrical.Analog.Basic.Inductor)
</p>
</HTML>"), Diagram(graphics));

        equation 
          connect(inductor.p,plug_p.pin) annotation(Line(points = {{ -10,0},{ -100,0}}, color = {0,0,255}));
          connect(inductor.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
        end Inductor;
        model SaturatingInductor "Simple model of inductors with saturation"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Current Inom[m](start = fill(1, m)) "Nominal current";
          parameter Modelica.SIunits.Inductance Lnom[m](start = fill(1, m)) "Nominal inductance at Nominal current";
          parameter Modelica.SIunits.Inductance Lzer[m](start = array(2 * Lnom[j] for j in 1:m)) "Inductance near current=0";
          parameter Modelica.SIunits.Inductance Linf[m](start = array(Lnom[j] / 2 for j in 1:m)) "Inductance at large currents";
          Modelica.Electrical.Analog.Basic.SaturatingInductor saturatingInductor[m](final Inom = Inom, final Lnom = Lnom, final Lzer = Lzer, final Linf = Linf) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -60, -15},{ -30,15}}, lineColor = {0,0,255}),Ellipse(extent = {{ -30, -15},{0,15}}, lineColor = {0,0,255}),Ellipse(extent = {{0, -15},{30,15}}, lineColor = {0,0,255}),Ellipse(extent = {{30, -15},{60,15}}, lineColor = {0,0,255}),Rectangle(extent = {{ -60, -20},{60,0}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{60,0},{90,0}}, color = {0,0,255}),Line(points = {{ -90,0},{ -60,0}}, color = {0,0,255}),Rectangle(extent = {{ -60, -10},{60, -20}}, lineColor = {0,0,0}, fillPattern = FillPattern.Sphere, fillColor = {0,0,255}),Text(extent = {{ -150,40},{150,100}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100, -100},{100, -60}}, lineColor = {0,0,0}, textString = "m=%m")}), Documentation(info = "<HTML>
<p>
Contains m saturating inductors (Modelica.Electrical.Analog.Basic.SaturatingInductor)
</p>
<p>
<b>Attention!!!</b><br>
Each element of the array of saturatingInductors is only dependent on the current flowing through this element.
</p>
</HTML>"));

        equation 
          connect(saturatingInductor.p,plug_p.pin) annotation(Line(points = {{ -10,0},{ -100,0}}, color = {0,0,255}));
          connect(saturatingInductor.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
        end SaturatingInductor;
        model Transformer "Multiphase Transformer"
          extends Interfaces.FourPlug;
          parameter Modelica.SIunits.Inductance L1[m](start = fill(1, m)) "Primary inductance";
          parameter Modelica.SIunits.Inductance L2[m](start = fill(1, m)) "Secondary inductance";
          parameter Modelica.SIunits.Inductance M[m](start = fill(1, m)) "Coupling inductance";
          Modelica.Electrical.Analog.Basic.Transformer transformer[m](final L1 = L1, final L2 = L2, final M = M) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -45, -50},{ -20, -25}}, lineColor = {0,0,255}),Ellipse(extent = {{ -45, -25},{ -20,0}}, lineColor = {0,0,255}),Ellipse(extent = {{ -45,0},{ -20,25}}, lineColor = {0,0,255}),Ellipse(extent = {{ -45,25},{ -20,50}}, lineColor = {0,0,255}),Rectangle(extent = {{ -72, -60},{ -33,60}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -90,50},{ -32,50}}, color = {0,0,255}),Line(points = {{ -90, -50},{ -32, -50}}, color = {0,0,255}),Ellipse(extent = {{20, -50},{45, -25}}, lineColor = {0,0,255}),Ellipse(extent = {{20, -25},{45,0}}, lineColor = {0,0,255}),Ellipse(extent = {{20,0},{45,25}}, lineColor = {0,0,255}),Ellipse(extent = {{20,25},{45,50}}, lineColor = {0,0,255}),Rectangle(extent = {{33, -60},{72,60}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{32,50},{90,50}}, color = {0,0,255}),Line(points = {{32, -50},{90, -50}}, color = {0,0,255}),Text(extent = {{ -100,20},{ -58, -20}}, textString = "L1", lineColor = {0,0,255}),Text(extent = {{60,20},{100, -20}}, textString = "L2", lineColor = {0,0,255}),Text(extent = {{ -20,20},{20, -20}}, textString = "M", lineColor = {0,0,255}),Text(extent = {{ -80, -100},{80, -60}}, lineColor = {0,0,0}, textString = "m=%m"),Text(extent = {{ -100,50},{100,110}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<HTML>
<p>
Contains m transformers (Modelica.Electrical.Analog.Basic.Transformer)
</p>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          connect(plug_p1.pin,transformer.p1) annotation(Line(points = {{ -100,100},{ -10,100},{ -10,5}}, color = {0,0,255}));
          connect(plug_p2.pin,transformer.p2) annotation(Line(points = {{100,100},{10,100},{10,5}}, color = {0,0,255}));
          connect(plug_n1.pin,transformer.n1) annotation(Line(points = {{ -100, -100},{ -10, -100},{ -10, -5}}, color = {0,0,255}));
          connect(plug_n2.pin,transformer.n2) annotation(Line(points = {{100, -100},{10, -100},{10, -5}}, color = {0,0,255}));
        end Transformer;
        model VariableResistor "Ideal linear electrical resistors with variable resistance"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Temperature T_ref[m] = fill(300.15, m) "Reference temperatures";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha[m] = zeros(m) "Temperature coefficients of resistances at reference temperatures";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, T = T_ref);
          Modelica.Blocks.Interfaces.RealInput R[m] annotation(Placement(transformation(origin = {0,100}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.VariableResistor variableResistor[m](final T_ref = T_ref, final alpha = alpha, each final useHeatPort = useHeatPort, final T = T) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -90,0},{ -70,0}}, color = {0,0,255}),Rectangle(extent = {{ -70,30},{70, -30}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{70,0},{90,0}}, color = {0,0,255}),Line(points = {{0,90},{0,30}}, color = {0,0,255}),Text(extent = {{ -150, -100},{150, -40}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100,40},{100,80}}, lineColor = {0,0,0}, textString = "m=%m")}), Documentation(info = "<HTML>
<p>
Contains m variable resistors (Modelica.Electrical.Analog.Basic.VariableResistor)
</p>
<P>
<b>Attention!!!</b><br>
  It is recomended that none of the R_Port signals should not cross the zero value.
  Otherwise depending on the surrounding circuit the probability of singularities is high.
</P>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          connect(variableResistor.p,plug_p.pin) annotation(Line(points = {{ -10,0},{ -100,0}}, color = {0,0,255}));
          connect(variableResistor.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
          connect(R,variableResistor.R) annotation(Line(points = {{0,100},{0,11}}, color = {0,0,255}));
          connect(variableResistor.heatPort,heatPort) annotation(Line(points = {{0, -10},{0, -100}}, color = {191,0,0}, smooth = Smooth.None));
        end VariableResistor;
        model VariableConductor "Ideal linear electrical conductors with variable conductance"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Temperature T_ref[m] = fill(300.15, m) "Reference temperatures";
          parameter Modelica.SIunits.LinearTemperatureCoefficient alpha[m] = zeros(m) "Temperature coefficients of conductances at reference temperatures";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, T = T_ref);
          Modelica.Blocks.Interfaces.RealInput G[m] annotation(Placement(transformation(origin = {0,100}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.VariableConductor variableConductor[m](final T_ref = T_ref, final alpha = alpha, each final useHeatPort = useHeatPort, final T = T) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -90,0},{ -70,0}}, color = {0,0,255}),Rectangle(extent = {{ -70,30},{70, -30}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{70,0},{90,0}}, color = {0,0,255}),Line(points = {{0,90},{0,30}}, color = {0,0,255}),Text(extent = {{ -150, -100},{150, -40}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100,40},{100,80}}, lineColor = {0,0,0}, textString = "m=%m")}), Documentation(info = "<HTML>
<p>
Contains m variable conductors (Modelica.Electrical.Analog.Basic.VariableConductor)
</p>
<P>
<b>Attention!!!</b><br>
  It is recomended that none of the G_Port signals should not cross the zero value.
  Otherwise depending on the surrounding circuit the probability of singularities is high.
</P>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          connect(variableConductor.p,plug_p.pin) annotation(Line(points = {{ -10,0},{ -100,0}}, color = {0,0,255}));
          connect(variableConductor.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
          connect(G,variableConductor.G) annotation(Line(points = {{0,100},{0,11}}, color = {0,0,255}));
          connect(variableConductor.heatPort,heatPort) annotation(Line(points = {{0, -10},{0, -100}}, color = {191,0,0}, smooth = Smooth.None));
        end VariableConductor;
        model VariableCapacitor "Ideal linear electrical capacitors with variable capacitance"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Capacitance Cmin[m] = fill(Modelica.Constants.eps, m) "minimum Capacitance";
          Modelica.Blocks.Interfaces.RealInput C[m] annotation(Placement(transformation(origin = {0,100}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.VariableCapacitor variableCapacitor[m](final Cmin = Cmin) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{0,90},{0,30}}, color = {0,0,255}),Text(extent = {{ -150, -100},{150, -40}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100,40},{100,80}}, lineColor = {0,0,0}, textString = "m=%m"),Line(points = {{ -90,0},{ -14,0}}, color = {0,0,255}),Line(points = {{14,0},{90,0}}, color = {0,0,255}),Line(points = {{ -14,28},{ -14, -28}}, thickness = 0.5, color = {0,0,255}),Line(points = {{14,28},{14, -28}}, thickness = 0.5, color = {0,0,255})}), Documentation(info = "<HTML>
<p>
Contains m variable capacitors (Modelica.Electrical.Analog.Basic.VariableCapacitor)
</p>
<P>
It is required that each C_Port.signal &ge; 0, otherwise an
assertion is raised. To avoid a variable index system,<br>
C = Cmin, if 0 &le; C_Port.signal &lt; Cmin, where
Cmin is a parameter with default value Modelica.Constants.eps.
</P>
</HTML>"), Diagram(graphics));

        equation 
          connect(variableCapacitor.p,plug_p.pin) annotation(Line(points = {{ -10,0},{ -100,0}}, color = {0,0,255}));
          connect(variableCapacitor.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
          connect(C,variableCapacitor.C) annotation(Line(points = {{0,100},{0,11}}, color = {0,0,255}));
        end VariableCapacitor;
        model VariableInductor "Ideal linear electrical inductors with variable inductance"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Inductance Lmin[m] = fill(Modelica.Constants.eps, m) "minimum Inductance";
          Modelica.Blocks.Interfaces.RealInput L[m] annotation(Placement(transformation(origin = {0,100}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.VariableInductor variableInductor[m](final Lmin = Lmin) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -150, -100},{150, -40}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100,40},{100,80}}, lineColor = {0,0,0}, textString = "m=%m"),Line(points = {{ -90,0},{ -60,0}}, color = {0,0,255}),Line(points = {{60,0},{90,0}}, color = {0,0,255}),Line(points = {{0,90},{0,8}}, color = {0,0,255}),Ellipse(extent = {{ -60, -15},{ -30,15}}, lineColor = {0,0,255}),Ellipse(extent = {{ -30, -15},{0,15}}, lineColor = {0,0,255}),Ellipse(extent = {{0, -15},{30,15}}, lineColor = {0,0,255}),Ellipse(extent = {{30, -15},{60,15}}, lineColor = {0,0,255}),Rectangle(extent = {{ -60, -30},{60,0}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
<p>
Contains m variable inductors (Modelica.Electrical.Analog.Basic.VariableInductor)
</p>
<P>
It is required that each L_Port.signal &ge; 0, otherwise an
assertion is raised. To avoid a variable index system,<br>
L = Lmin, if 0 &le; L_Port.signal &lt; Lmin, where
Lmin is a parameter with default value Modelica.Constants.eps.
</P>
</HTML>"), Diagram(graphics));

        equation 
          connect(variableInductor.p,plug_p.pin) annotation(Line(points = {{ -10,0},{ -100,0}}, color = {0,0,255}));
          connect(variableInductor.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
          connect(L,variableInductor.L) annotation(Line(points = {{0,100},{0,10.8}}, color = {0,0,255}));
        end VariableInductor;
      end Basic;
      package Ideal "Multiphase components with idealized behaviour"
        extends Modelica.Icons.Library2;
        annotation(Documentation(info = "<HTML>
<p>
This package contains analog electrical multiphase components with idealized behaviour,
like thyristor, diode, switch, transformer.
</p>

</HTML>", revisions = "<html>
<dl>
<p>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
</p>
<p>
  <dt><b>Release Notes:</b></dt>
  <dd>
  <ul>
  <li> v1.0 2004/10/01 Anton Haumer</li>
  </ul>
  </dd>
<p>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2006, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</p>
</dl>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -100, -40},{80, -40}}, color = {0,0,255}),Polygon(points = {{20, -40},{ -40,0},{ -40, -80},{20, -40}}, lineColor = {0,0,0}, fillColor = {255,255,255}),Line(points = {{20,0},{20, -80}}, color = {0,0,255})}));
        model IdealThyristor "Multiphase ideal thyristor"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Resistance Ron[m](final min = zeros(m), start = fill(1e-05, m)) "Closed thyristor resistance";
          parameter Modelica.SIunits.Conductance Goff[m](final min = zeros(m), start = fill(1e-05, m)) "Opened thyristor conductance";
          parameter Modelica.SIunits.Voltage Vknee[m](final min = zeros(m), start = zeros(m)) "Treshold voltage";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, final T = fill(293.15, m));
          Modelica.Blocks.Interfaces.BooleanInput fire[m] annotation(Placement(transformation(origin = {70,110}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
          Modelica.Electrical.Analog.Ideal.IdealThyristor idealThyristor[m](final Ron = Ron, final Goff = Goff, final Vknee = Vknee, each final useHeatPort = useHeatPort) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -150, -40},{150, -100}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -90,0},{40,0}}, color = {0,0,255}),Polygon(points = {{30,0},{ -30,40},{ -30, -40},{30,0}}, lineColor = {0,0,0}, fillColor = {255,255,255}),Line(points = {{30,40},{30, -40}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255}),Line(points = {{40,50},{60,30}}, color = {0,0,255}),Line(points = {{30,20},{70,60},{70,90}}, color = {0,0,255}),Text(extent = {{ -100,100},{60,60}}, lineColor = {0,0,0}, textString = "m=%m")}), Documentation(info = "<HTML>
<p>
Contains m ideal thyristors (Modelica.Electrical.Analog.Ideal.IdealThyristor).
</p>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          connect(plug_p.pin,idealThyristor.p) annotation(Line(points = {{ -100,0},{ -10,0}}, color = {0,0,255}));
          connect(idealThyristor.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
          connect(fire,idealThyristor.fire) annotation(Line(points = {{70,110},{70,80},{7,80},{7,11}}, color = {255,0,255}));
          connect(idealThyristor.heatPort,heatPort) annotation(Line(points = {{0, -10},{0, -100}}, color = {191,0,0}, pattern = LinePattern.None, smooth = Smooth.None));
        end IdealThyristor;
        model IdealGTOThyristor "Multiphase ideal GTO thyristor"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Resistance Ron[m](final min = zeros(m), start = fill(1e-05, m)) "Closed thyristor resistance";
          parameter Modelica.SIunits.Conductance Goff[m](final min = zeros(m), start = fill(1e-05, m)) "Opened thyristor conductance";
          parameter Modelica.SIunits.Voltage Vknee[m](final min = zeros(m), start = zeros(m)) "Treshold voltage";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, final T = fill(293.15, m));
          Modelica.Blocks.Interfaces.BooleanInput fire[m] annotation(Placement(transformation(origin = {70,110}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
          Modelica.Electrical.Analog.Ideal.IdealGTOThyristor idealGTOThyristor[m](final Ron = Ron, final Goff = Goff, final Vknee = Vknee, each final useHeatPort = useHeatPort) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -150, -40},{150, -100}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -90,0},{40,0}}, color = {0,0,255}),Polygon(points = {{30,0},{ -30,40},{ -30, -40},{30,0}}, lineColor = {0,0,0}, fillColor = {255,255,255}),Line(points = {{30,40},{30, -40}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255}),Line(points = {{40,50},{60,30}}, color = {0,0,255}),Line(points = {{30,20},{70,60},{70,90}}, color = {0,0,255}),Text(extent = {{ -100,100},{60,60}}, lineColor = {0,0,0}, textString = "m=%m")}), Documentation(info = "<HTML>
<p>
Contains m ideal GTO thyristors (Modelica.Electrical.Analog.Ideal.IdealGTOThyristor).
</p>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          connect(idealGTOThyristor.p,plug_p.pin) annotation(Line(points = {{ -10,0},{ -100,0}}, color = {0,0,255}));
          connect(idealGTOThyristor.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
          connect(fire,idealGTOThyristor.fire) annotation(Line(points = {{70,110},{70,80},{7,80},{7,11}}, color = {255,0,255}));
          connect(idealGTOThyristor.heatPort,heatPort) annotation(Line(points = {{0, -10},{0, -100}}, color = {191,0,0}, pattern = LinePattern.None, smooth = Smooth.None));
        end IdealGTOThyristor;
        model IdealCommutingSwitch "Multiphase ideal commuting switch"
          parameter Integer m(final min = 1) = 3 "number of phases";
          parameter Modelica.SIunits.Resistance Ron[m](final min = zeros(m), start = fill(1e-05, m)) "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff[m](final min = zeros(m), start = fill(1e-05, m)) "Opened switch conductance";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, final T = fill(293.15, m));
          Modelica.Blocks.Interfaces.BooleanInput control[m] "true => p--n2 connected, false => p--n1 connected" annotation(Placement(transformation(origin = {0,80}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
          Interfaces.PositivePlug plug_p(final m = m) annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Interfaces.NegativePlug plug_n2(final m = m) annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Interfaces.NegativePlug plug_n1(final m = m) annotation(Placement(transformation(extent = {{90,40},{110,60}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealCommutingSwitch idealCommutingSwitch[m](final Ron = Ron, final Goff = Goff, each final useHeatPort = useHeatPort) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -150, -40},{150, -100}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100,100},{ -20,60}}, lineColor = {0,0,0}, textString = "m="),Text(extent = {{20,100},{100,60}}, lineColor = {0,0,0}, textString = "%m"),Ellipse(extent = {{ -44,4},{ -36, -4}}, lineColor = {0,0,255}),Line(points = {{ -90,0},{ -44,0}}, color = {0,0,255}),Line(points = {{ -37,2},{40,50}}, color = {0,0,255}),Line(points = {{40,50},{90,50}}, color = {0,0,255}),Line(points = {{0,90},{0,25}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255})}), Documentation(info = "<HTML>
<p>
Contains m ideal commuting switches (Modelica.Electrical.Analog.Ideal.IdealCommutingSwitch).
</p>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          connect(plug_p.pin,idealCommutingSwitch.p) annotation(Line(points = {{ -100,0},{ -10,0}}, color = {0,0,255}));
          connect(idealCommutingSwitch.n2,plug_n2.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
          connect(idealCommutingSwitch.n1,plug_n1.pin) annotation(Line(points = {{10,5},{10,50},{100,50}}, color = {0,0,255}));
          connect(control,idealCommutingSwitch.control) annotation(Line(points = {{0,80},{0,8}}, color = {255,0,255}));
          connect(idealCommutingSwitch.heatPort,heatPort) annotation(Line(points = {{0, -10},{0, -100}}, color = {191,0,0}, pattern = LinePattern.None, smooth = Smooth.None));
        end IdealCommutingSwitch;
        model IdealIntermediateSwitch "Multiphase ideal intermediate switch"
          parameter Integer m(final min = 1) = 3 "number of phases";
          parameter Modelica.SIunits.Resistance Ron[m](final min = zeros(m), start = fill(1e-05, m)) "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff[m](final min = zeros(m), start = fill(1e-05, m)) "Opened switch conductance";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, final T = fill(293.15, m));
          Modelica.Blocks.Interfaces.BooleanInput control[m] "true => p1--n2, p2--n1 connected, otherwise p1--n1, p2--n2 connected" annotation(Placement(transformation(origin = {0,80}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
          Interfaces.PositivePlug plug_p1(final m = m) annotation(Placement(transformation(extent = {{ -110,40},{ -90,60}}, rotation = 0)));
          Interfaces.PositivePlug plug_p2(final m = m) annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Interfaces.NegativePlug plug_n2(final m = m) annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Interfaces.NegativePlug plug_n1(final m = m) annotation(Placement(transformation(extent = {{90,40},{110,60}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealIntermediateSwitch idealIntermediateSwitch[m](final Ron = Ron, final Goff = Goff, each final useHeatPort = useHeatPort) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -150, -40},{150, -100}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100,100},{ -20,60}}, lineColor = {0,0,0}, textString = "m="),Text(extent = {{20,100},{100,60}}, lineColor = {0,0,0}, textString = "%m"),Ellipse(extent = {{ -4,30},{4,22}}, lineColor = {0,0,255}),Line(points = {{ -90,0},{ -44,0}}, color = {0,0,255}),Line(points = {{ -90,50},{ -44,50}}, color = {0,0,255}),Line(points = {{ -44,0},{40,50}}, color = {0,0,255}),Line(points = {{ -44,50},{40,0}}, color = {0,0,255}),Line(points = {{40,50},{90,50}}, color = {0,0,255}),Line(points = {{0,90},{0,25}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255})}), Documentation(info = "<HTML>
<p>
Contains m ideal intermediate switches (Modelica.Electrical.Analog.Ideal.IdealIntermediateSwitch).
</p>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          connect(plug_p2.pin,idealIntermediateSwitch.p2) annotation(Line(points = {{ -100,0},{ -10,0}}, color = {0,0,255}));
          connect(idealIntermediateSwitch.n2,plug_n2.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
          connect(idealIntermediateSwitch.p1,plug_p1.pin) annotation(Line(points = {{ -10,5},{ -10,50},{ -100,50}}, color = {0,0,255}));
          connect(idealIntermediateSwitch.n1,plug_n1.pin) annotation(Line(points = {{10,5},{10,50},{100,50}}, color = {0,0,255}));
          connect(control,idealIntermediateSwitch.control) annotation(Line(points = {{0,80},{0,8}}, color = {255,0,255}));
          connect(idealIntermediateSwitch.heatPort,heatPort) annotation(Line(points = {{0, -10},{0, -100}}, color = {191,0,0}, pattern = LinePattern.None, smooth = Smooth.None));
        end IdealIntermediateSwitch;
        model IdealDiode "Multiphase ideal diode"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Resistance Ron[m](final min = zeros(m), start = fill(1e-05, m)) "Closed diode resistance";
          parameter Modelica.SIunits.Conductance Goff[m](final min = zeros(m), start = fill(1e-05, m)) "Opened diode conductance";
          parameter Modelica.SIunits.Voltage Vknee[m](final min = zeros(m), start = zeros(m)) "Treshold voltage";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, final T = fill(293.15, m));
          Modelica.Electrical.Analog.Ideal.IdealDiode idealDiode[m](final Ron = Ron, final Goff = Goff, final Vknee = Vknee, each final useHeatPort = useHeatPort) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -150, -40},{150, -100}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -90,0},{40,0}}, color = {0,0,255}),Polygon(points = {{30,0},{ -30,40},{ -30, -40},{30,0}}, lineColor = {0,0,0}, fillColor = {255,255,255}),Line(points = {{30,40},{30, -40}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255}),Text(extent = {{ -80,100},{80,60}}, lineColor = {0,0,0}, textString = "m=%m")}), Documentation(info = "<HTML>
<p>
Contains m ideal diodes (Modelica.Electrical.Analog.Ideal.IdealDiode).
</p>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          connect(plug_p.pin,idealDiode.p) annotation(Line(points = {{ -100,0},{ -10,0}}, color = {0,0,255}));
          connect(idealDiode.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
        end IdealDiode;
        model IdealTransformer "Multiphase ideal transformer"
          extends Interfaces.FourPlug;
          parameter Real n[m](start = fill(1, m)) "Turns ratio";
          parameter Boolean considerMagnetization = false;
          parameter SIunits.Inductance Lm1[m](start = fill(1, m)) "Magnetization inductances w.r.t. primary side";
          Modelica.Electrical.Analog.Ideal.IdealTransformer idealTransformer[m](final n = n, each final considerMagnetization = considerMagnetization, final Lm1 = Lm1) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -80,100},{80,60}}, lineColor = {0,0,0}, textString = "m=%m"),Text(extent = {{ -100, -100},{100, -40}}, lineColor = {0,0,255}, textString = "%name"),Ellipse(extent = {{ -80,50},{20, -50}}, lineColor = {0,0,255}),Ellipse(extent = {{ -20,50},{80, -50}}, lineColor = {0,0,255}),Text(extent = {{ -20,20},{20, -20}}, lineColor = {0,0,255}, textString = "="),Line(points = {{ -96,50},{ -30,50}}, color = {0,0,255}, smooth = Smooth.None),Line(points = {{30,50},{96,50}}, color = {0,0,255}, smooth = Smooth.None),Line(points = {{30, -50},{96, -50}}, color = {0,0,255}, smooth = Smooth.None),Line(points = {{ -96, -50},{ -30, -50}}, color = {0,0,255}, smooth = Smooth.None)}), Documentation(info = "<HTML>
<p>
Contains m ideal transformers (Modelica.Electrical.Analog.Ideal.IdealTransformer).
</p>
<p>
<b>Note:</b> Due to the above equations, also DC signals will be transformed!
</p>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          connect(plug_p1.pin,idealTransformer.p1) annotation(Line(points = {{ -100,100},{ -10,100},{ -10,5}}, color = {0,0,255}));
          connect(plug_p2.pin,idealTransformer.p2) annotation(Line(points = {{100,100},{10,100},{10,5}}, color = {0,0,255}));
          connect(plug_n1.pin,idealTransformer.n1) annotation(Line(points = {{ -100, -100},{ -10, -100},{ -10, -5}}, color = {0,0,255}));
          connect(plug_n2.pin,idealTransformer.n2) annotation(Line(points = {{100, -100},{10, -100},{10, -5}}, color = {0,0,255}));
        end IdealTransformer;
        model Idle "Multiphase idle branch"
          extends Interfaces.TwoPlug;
          Modelica.Electrical.Analog.Ideal.Idle idle[m] annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -150,50},{150,110}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100, -100},{100, -60}}, lineColor = {0,0,0}, textString = "m=%m"),Rectangle(extent = {{ -60,60},{60, -60}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -90,0},{ -41,0}}, color = {0,0,255}),Line(points = {{91,0},{40,0}}, color = {0,0,255})}), Documentation(info = "<HTML>
<p>
Contains m idles (Modelica.Electrical.Analog.Ideal.Idle)
</p>
</HTML>"), Diagram(graphics));

        equation 
          connect(plug_p.pin,idle.p) annotation(Line(points = {{ -100,0},{ -10,0}}, color = {0,0,255}));
          connect(idle.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
        end Idle;
        model Short "Multiphase short cut branch"
          extends Interfaces.TwoPlug;
          Modelica.Electrical.Analog.Ideal.Short short[m] annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -150,50},{150,110}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100, -100},{100, -60}}, lineColor = {0,0,0}, textString = "m=%m"),Rectangle(extent = {{ -60,60},{60, -60}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -90,0},{90,0}}, color = {0,0,255})}), Documentation(info = "<HTML>
<p>
Contains m short cuts (Modelica.Electrical.Analog.Ideal.Short)
</p>
</HTML>"), Diagram(graphics));

        equation 
          connect(plug_p.pin,short.p) annotation(Line(points = {{ -100,0},{ -10,0}}, color = {0,0,255}));
          connect(short.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
        end Short;
        model IdealOpeningSwitch "Multiphase ideal opener"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Resistance Ron[m](final min = zeros(m), start = fill(1e-05, m)) "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff[m](final min = zeros(m), start = fill(1e-05, m)) "Opened switch conductance";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, final T = fill(293.15, m));
          Modelica.Blocks.Interfaces.BooleanInput control[m] "true => switch open, false => p--n connected" annotation(Placement(transformation(origin = {0,70}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
          Modelica.Electrical.Analog.Ideal.IdealOpeningSwitch idealOpeningSwitch[m](final Ron = Ron, final Goff = Goff, each final useHeatPort = useHeatPort) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -150, -40},{150, -100}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -90,0},{ -44,0}}, color = {0,0,255}),Ellipse(extent = {{ -44,4},{ -36, -4}}, lineColor = {0,0,255}),Line(points = {{ -37,2},{40,50}}, color = {0,0,255}),Line(points = {{0,88},{0,26}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255}),Text(extent = {{ -100,100},{ -20,60}}, lineColor = {0,0,0}, textString = "m="),Text(extent = {{20,100},{100,60}}, lineColor = {0,0,0}, textString = "%m"),Line(points = {{40,20},{40,0}}, color = {0,0,255})}), Documentation(info = "<HTML>
<p>
Contains m ideal opening switches (Modelica.Electrical.Analog.Ideal.IdealOpeningSwitch).
</p>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          connect(plug_p.pin,idealOpeningSwitch.p) annotation(Line(points = {{ -100,0},{ -10,0}}, color = {0,0,255}));
          connect(idealOpeningSwitch.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
          connect(control,idealOpeningSwitch.control) annotation(Line(points = {{0,70},{0,7}}, color = {255,0,255}));
          connect(idealOpeningSwitch.heatPort,heatPort) annotation(Line(points = {{0, -10},{0, -100}}, color = {191,0,0}, pattern = LinePattern.None, smooth = Smooth.None));
        end IdealOpeningSwitch;
        model IdealClosingSwitch "Multiphase ideal closer"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Resistance Ron[m](final min = zeros(m), start = fill(1e-05, m)) "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff[m](final min = zeros(m), start = fill(1e-05, m)) "Opened switch conductance";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, final T = fill(293.15, m));
          Modelica.Blocks.Interfaces.BooleanInput control[m] "true => p--n connected, false => switch open" annotation(Placement(transformation(origin = {0,70}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
          Modelica.Electrical.Analog.Ideal.IdealClosingSwitch idealClosingSwitch[m](final Ron = Ron, final Goff = Goff, each final useHeatPort = useHeatPort) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -150, -40},{150, -100}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -90,0},{ -44,0}}, color = {0,0,255}),Ellipse(extent = {{ -44,4},{ -36, -4}}, lineColor = {0,0,255}),Line(points = {{ -37,2},{40,50}}, color = {0,0,255}),Line(points = {{0,88},{0,26}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255}),Text(extent = {{ -100,100},{ -20,60}}, lineColor = {0,0,0}, textString = "m="),Text(extent = {{20,100},{100,60}}, lineColor = {0,0,0}, textString = "%m")}), Documentation(info = "<HTML>
<p>
Contains m ideal closing switches (Modelica.Electrical.Analog.Ideal.IdealClosingSwitch).
</p><
/HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          connect(plug_p.pin,idealClosingSwitch.p) annotation(Line(points = {{ -100,0},{ -10,0}}, color = {0,0,255}));
          connect(idealClosingSwitch.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
          connect(control,idealClosingSwitch.control) annotation(Line(points = {{0,70},{0,7}}, color = {255,0,255}));
          connect(idealClosingSwitch.heatPort,heatPort) annotation(Line(points = {{0, -10},{0, -100}}, color = {191,0,0}, pattern = LinePattern.None, smooth = Smooth.None));
        end IdealClosingSwitch;
        model OpenerWithArc "Multiphase opener with arc"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Resistance Ron[m](final min = zeros(m), start = fill(1e-05, m)) "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff[m](final min = zeros(m), start = fill(1e-05, m)) "Opened switch conductance";
          parameter Modelica.SIunits.Voltage V0[m](start = fill(30, m)) "Initial arc voltage";
          parameter Modelica.SIunits.VoltageSlope dVdt[m](start = fill(10000.0, m)) "Arc voltage slope";
          parameter Modelica.SIunits.Voltage Vmax[m](start = fill(60, m)) "Max. arc voltage";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, final T = fill(293.15, m));
          Modelica.Blocks.Interfaces.BooleanInput control[m] "true => switch open, false => p--n connected" annotation(Placement(transformation(origin = {0,70}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
          Modelica.Electrical.Analog.Ideal.OpenerWithArc openerWithArc[m](final Ron = Ron, final Goff = Goff, final V0 = V0, final dVdt = dVdt, final Vmax = Vmax, each final useHeatPort = useHeatPort) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -150, -40},{150, -100}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -90,0},{ -44,0}}, color = {0,0,255}),Ellipse(extent = {{ -44,4},{ -36, -4}}, lineColor = {0,0,255}),Line(points = {{ -37,2},{40,50}}, color = {0,0,255}),Line(points = {{0,88},{0,26}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255}),Text(extent = {{ -100,100},{ -20,60}}, lineColor = {0,0,0}, textString = "m="),Text(extent = {{20,100},{100,60}}, lineColor = {0,0,0}, textString = "%m"),Line(points = {{40,20},{40,0}}, color = {0,0,255}),Line(points = {{40,50},{32,34},{48,30},{40,20}}, color = {255,0,0})}), Documentation(info = "<html>
<p>Contains m opening switches with arc (Modelica.Electrical.Analog.Ideal.OpenerWithArc). </p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          connect(plug_p.pin,openerWithArc.p) annotation(Line(points = {{ -100,0},{ -10,0}}, color = {0,0,255}));
          connect(openerWithArc.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
          connect(control,openerWithArc.control) annotation(Line(points = {{0,70},{0,10}}, color = {255,0,255}));
          connect(openerWithArc.heatPort,heatPort) annotation(Line(points = {{0, -10},{0, -100}}, color = {191,0,0}, pattern = LinePattern.None, smooth = Smooth.None));
        end OpenerWithArc;
        model CloserWithArc "Multiphase closer with arc"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Resistance Ron[m](final min = zeros(m), start = fill(1e-05, m)) "Closed switch resistance";
          parameter Modelica.SIunits.Conductance Goff[m](final min = zeros(m), start = fill(1e-05, m)) "Opened switch conductance";
          parameter Modelica.SIunits.Voltage V0[m](start = fill(30, m)) "Initial arc voltage";
          parameter Modelica.SIunits.VoltageSlope dVdt[m](start = fill(10000.0, m)) "Arc voltage slope";
          parameter Modelica.SIunits.Voltage Vmax[m](start = fill(60, m)) "Max. arc voltage";
          extends Modelica.Electrical.MultiPhase.Interfaces.ConditionalHeatPort(final mh = m, final T = fill(293.15, m));
          Modelica.Blocks.Interfaces.BooleanInput control[m] "true => switch open, false => p--n connected" annotation(Placement(transformation(origin = {0,70}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
          Modelica.Electrical.Analog.Ideal.CloserWithArc closerWithArc[m](final Ron = Ron, final Goff = Goff, final V0 = V0, final dVdt = dVdt, final Vmax = Vmax, each final useHeatPort = useHeatPort) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -150, -40},{150, -100}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -90,0},{ -44,0}}, color = {0,0,255}),Ellipse(extent = {{ -44,4},{ -36, -4}}, lineColor = {0,0,255}),Line(points = {{ -37,2},{40,50}}, color = {0,0,255}),Line(points = {{0,88},{0,26}}, color = {0,0,255}),Line(points = {{40,0},{90,0}}, color = {0,0,255}),Text(extent = {{ -100,100},{ -20,60}}, lineColor = {0,0,0}, textString = "m="),Text(extent = {{20,100},{100,60}}, lineColor = {0,0,0}, textString = "%m"),Line(points = {{40,50},{32,24},{48,28},{40,0}}, color = {255,0,0})}), Documentation(info = "<html>
<p>Contains m closing switches with arc (Modelica.Electrical.Analog.Ideal.CloserWithArc). </p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          connect(plug_p.pin,closerWithArc.p) annotation(Line(points = {{ -100,0},{ -10,0}}, color = {0,0,255}));
          connect(closerWithArc.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
          connect(control,closerWithArc.control) annotation(Line(points = {{0,70},{0,10}}, color = {255,0,255}));
          connect(closerWithArc.heatPort,heatPort) annotation(Line(points = {{0, -10},{0, -100}}, color = {191,0,0}, pattern = LinePattern.None, smooth = Smooth.None));
        end CloserWithArc;
      end Ideal;
      package Interfaces "Interfaces for electrical multiphase models"
        extends Modelica.Icons.Library2;
        annotation(Documentation(info = "<HTML>
<p>
This package contains connectors and interfaces (partial models) for
electrical multiphase components, based on Modelica.Electrical.Analog.
</p>

</HTML>", revisions = "<html>
<dl>
<p>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
</p>
<p>
  <dt><b>Release Notes:</b></dt>
  <dd>
  <ul>
  <li>v1.0 2004/10/01 Anton Haumer</li>
  <li>v1.2 2006/05/12 Anton Haumer<br>
      removed annotation from pin of Interfaces.Plug</li>
  </ul>
  </dd>
<p>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2006, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</p>
</dl>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -60,10},{40, -90}}, lineColor = {0,0,255}),Ellipse(extent = {{ -40, -14},{ -20, -34}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{0, -14},{20, -34}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -20, -54},{0, -74}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid)}));
        connector Plug "Plug with m pins for an electric component"
          parameter Integer m(final min = 1) = 3 "number of phases";
          Modelica.Electrical.Analog.Interfaces.Pin pin[m];
          annotation(Documentation(info = "<HTML>
<p>
Connectors PositivePlug and NegativePlug are nearly identical.
The only difference is that the icons are different in order
to identify more easily the plugs of a component.
Usually, connector PositivePlug is used for the positive and
connector NegativePlug for the negative plug of an electrical component.<br>
Connector Plug is a composite connector containing m Pins (Modelica.Electrical.Analog.Interfaces.Pin).
</p>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -100, -99},{100, -179}}, lineColor = {0,0,255}, textString = "%name")}));
        end Plug;
        connector PositivePlug "Positive plug with m pins"
          extends Plug;
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
<p>
Connectors PositivePlug and NegativePlug are nearly identical.
The only difference is that the icons are different in order
to identify more easily the plugs of a component.
Usually, connector PositivePlug is used for the positive and
connector NegativePlug for the negative plug of an electrical component.<br>
Connector Plug is a composite connector containing m Pins (Modelica.Electrical.Analog.Interfaces.Pin).
</p>
</HTML>"));
        end PositivePlug;
        connector NegativePlug "Negative plug with m pins"
          extends Plug;
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
<p>
Connectors PositivePlug and NegativePlug are nearly identical.
The only difference is that the icons are different in order
to identify more easily the plugs of a component.
Usually, connector PositivePlug is used for the positive and
connector NegativePlug for the negative plug of an electrical component.<br>
Connector Plug is a composite connector containing m Pins (Modelica.Electrical.Analog.Interfaces.Pin).
</p>
</HTML>"));
        end NegativePlug;
        partial model ConditionalHeatPort "Partial model to include conditional HeatPorts in order to describe the power loss via a thermal network"
          parameter Integer mh(min = 1) = 3 "Number of heatPorts=number of phases";
          parameter Boolean useHeatPort = false "=true, if all HeatPorts are enabled" annotation(Evaluate = true, HideResult = true, choices(__Dymola_checkBox = true));
          parameter Modelica.SIunits.Temperature T[mh] = fill(293.15, mh) "Fixed device temperatures if useHeatPort = false" annotation(Dialog(enable = not useHeatPort));
          Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort[mh] if useHeatPort annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}}), iconTransformation(extent = {{ -10, -110},{10, -90}})));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(revisions = "<html>
<p><ul>
<li><i>August 26, 2009 </i>by Anton Haumer initially implemented</li>
</ul></p>
</html>", info = "<html>
<p>
This partial model provides conditional heat ports for the connection to a thermal network.
</p>
<ul>
<li> If <b>useHeatPort</b> is set to <b>false</b> (default), no heat port is available, and the thermal
     loss power flows internally to the ground. In this case, the parameter <b>T</b> specifies
     the fixed device temperatures.</li>.
<li> If <b>useHeatPort</b> is set to <b>true</b>, all heat ports are available.</li>
</ul>
</html>"));
        end ConditionalHeatPort;
        partial model TwoPlug "Component with one m-phase electric port"
          parameter Integer m(min = 1) = 3 "number of phases";
          Modelica.SIunits.Voltage v[m] "Voltage drops between the two plugs";
          Modelica.SIunits.Current i[m] "Currents flowing into positive plugs";
          PositivePlug plug_p(final m = m) annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          NegativePlug plug_n(final m = m) annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          annotation(Documentation(info = "<HTML>
<p>
Superclass of elements which have <b>two</b> electrical plugs:
the positive plug connector <i>plug_p</i>, and the negative plug connector <i>plug_n</i>.
The currents flowing into plug_p are provided explicitly as currents i[m].
</p>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          v = plug_p.pin.v - plug_n.pin.v;
          i = plug_p.pin.i;
        end TwoPlug;
        partial model OnePort "Component with two electrical plugs and currents from plug_p to plug_n"
          extends TwoPlug;
          annotation(Documentation(info = "<HTML>
<p>
Superclass of elements which have <b>two</b> electrical plugs:
the positive plug connector <i>plug_p</i>, and the negative plug connector <i>plug_n</i>.
The currents flowing into plug_p are provided explicitly as currents i[m].
It is assumed that the currents flowing into plug_p are identical to the currents flowing out of plug_n.
</p>
</HTML>"));

        equation 
          plug_p.pin.i + plug_n.pin.i = zeros(m);
        end OnePort;
        partial model FourPlug "Component with two m-phase electric ports"
          parameter Integer m(final min = 1) = 3 "number of phases";
          Modelica.SIunits.Voltage v1[m] "Voltage drops over the left port";
          Modelica.SIunits.Voltage v2[m] "Voltage drops over the right port";
          Modelica.SIunits.Current i1[m] "Current flowing into positive plug of the left port";
          Modelica.SIunits.Current i2[m] "Current flowing into positive plug of the right port";
          PositivePlug plug_p1(final m = m) annotation(Placement(transformation(extent = {{ -110,90},{ -90,110}}, rotation = 0)));
          PositivePlug plug_p2(final m = m) annotation(Placement(transformation(extent = {{90,90},{110,110}}, rotation = 0)));
          NegativePlug plug_n1(final m = m) annotation(Placement(transformation(extent = {{ -110, -110},{ -90, -90}}, rotation = 0)));
          NegativePlug plug_n2(final m = m) annotation(Placement(transformation(extent = {{90, -110},{110, -90}}, rotation = 0)));
          annotation(Documentation(info = "<HTML>
<p>
Superclass of elements which have <b>four</b> electrical plugs.
</p>
</HTML>"));

        equation 
          v1 = plug_p1.pin.v - plug_n1.pin.v;
          v2 = plug_p2.pin.v - plug_n2.pin.v;
          i1 = plug_p1.pin.i;
          i2 = plug_p2.pin.i;
        end FourPlug;
        partial model TwoPort "Component with two m-phase electric ports, including currents"
          extends FourPlug;
          annotation(Documentation(info = "<HTML>
<p>
Superclass of elements which have <b>four</b> electrical plugs.
It is assumed that the currents flowing into plug_p1 are identical to the currents flowing out of plug_n1,
and that the currents flowing into plug_p2 are identical to the currents flowing out of plug_n2.
</p>
</HTML>"));

        equation 
          plug_p1.pin.i + plug_n1.pin.i = zeros(m);
          plug_p2.pin.i + plug_n2.pin.i = zeros(m);
        end TwoPort;
      end Interfaces;
      package Sensors "Multiphase potential, voltage and current Sensors"
        extends Modelica.Icons.Library2;
        annotation(Documentation(info = "<HTML>
<p>
This package contains multiphase potential, voltage, and current sensors.
</p>

</HTML>", revisions = "<html>
<dl>
<p>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
</p>
<p>
  <dt><b>Release Notes:</b></dt>
  <dd>
  <ul>
  <li>v1.0 2004/10/01 Anton Haumer</li>
  <li>v1.1 2006/01/12 Anton Haumer<br>
      added PowerSensor</li>
  </ul>
  </dd>
<p>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2006, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</p>
</dl>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -60,10},{40, -90}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -50, -16},{ -36, -25}}, color = {0,0,0}),Line(points = {{ -35,0},{ -25, -14}}, color = {0,0,0}),Line(points = {{ -10,7},{ -10, -10}}, color = {0,0,0}),Line(points = {{15,0},{5, -14}}, color = {0,0,0}),Line(points = {{30, -15},{16, -25}}, color = {0,0,0}),Ellipse(extent = {{ -15, -35},{ -5, -45}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -10, -40},{ -6, -26}}, color = {0,0,0}),Polygon(points = {{ -12, -24},{ -0.5, -27},{2,1.5},{ -12, -24}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}));
        model PotentialSensor "Multiphase potential sensor"
          extends Modelica.Icons.RotationalSensor;
          parameter Integer m(final min = 1) = 3 "number of phases";
          Interfaces.PositivePlug plug_p(final m = m) annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput phi[m] "Absolute voltage potential as output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
          Modelica.Electrical.Analog.Sensors.PotentialSensor potentialSensor[m] annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(graphics = {Line(points = {{70,0},{100,0}}, color = {0,0,255}),Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Text(extent = {{ -100, -70},{100, -110}}, lineColor = {0,0,0}, textString = "m=%m"),Text(extent = {{ -150,60},{150,120}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<HTML>
<p>
Contains m potential sensors (Modelica.Electrical.Analog.Sensors.PotentialSensor),
thus measuring the m potentials <i>phi[m]</i> of the m pins of plug_p.
</p>
</HTML>"), Diagram(graphics));

        equation 
          connect(potentialSensor.p,plug_p.pin) annotation(Line(points = {{ -10,0},{ -100,0}}, color = {0,0,255}));
          connect(potentialSensor.phi,phi) annotation(Line(points = {{11,0},{110,0}}, color = {0,0,255}));
        end PotentialSensor;
        model VoltageSensor "Multiphase voltage sensor"
          extends Modelica.Icons.RotationalSensor;
          parameter Integer m(final min = 1) = 3 "number of phases";
          Interfaces.PositivePlug plug_p(final m = m) annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Interfaces.NegativePlug plug_n(final m = m) annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput v[m] "Voltage between pin p and n (= p.v - n.v) as output signal" annotation(Placement(transformation(origin = {0, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Modelica.Electrical.Analog.Sensors.VoltageSensor voltageSensor[m] annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(graphics = {Text(extent = {{ -29, -11},{30, -70}}, lineColor = {0,0,0}, textString = "V"),Text(extent = {{ -100,60},{100,120}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100, -60},{ -20, -100}}, lineColor = {0,0,0}, textString = "m="),Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{70,0},{90,0}}, color = {0,0,0}),Line(points = {{0, -100},{0, -70}}, color = {0,0,255}),Text(extent = {{20, -60},{100, -100}}, lineColor = {0,0,0}, textString = "%m")}), Documentation(info = "<HTML>
<p>
Contains m voltage sensors (Modelica.Electrical.Analog.Sensors.VoltageSensor),
thus measuring the m potential differences <i>v[m]</i> between the m pins of plug_p and plug_n.
</p>
</HTML>"), Diagram(graphics));

        equation 
          connect(voltageSensor.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
          connect(voltageSensor.p,plug_p.pin) annotation(Line(points = {{ -10,0},{ -100,0}}, color = {0,0,255}));
          connect(voltageSensor.v,v) annotation(Line(points = {{0, -10},{0, -110}}, color = {0,0,255}));
        end VoltageSensor;
        model CurrentSensor "Multiphase current sensor"
          extends Modelica.Icons.RotationalSensor;
          parameter Integer m(final min = 1) = 3 "number of phases";
          Interfaces.PositivePlug plug_p(final m = m) annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Interfaces.NegativePlug plug_n(final m = m) annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput i[m] "current in the branch from p to n as output signal" annotation(Placement(transformation(origin = {0, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Modelica.Electrical.Analog.Sensors.CurrentSensor currentSensor[m] annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(graphics = {Text(extent = {{ -29, -11},{30, -70}}, lineColor = {0,0,0}, textString = "A"),Text(extent = {{ -150,60},{150,120}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100, -60},{ -20, -100}}, lineColor = {0,0,0}, textString = "m="),Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{70,0},{90,0}}, color = {0,0,0}),Line(points = {{0, -100},{0, -70}}, color = {0,0,255}),Text(extent = {{20, -60},{100, -100}}, lineColor = {0,0,0}, textString = "%m")}), Documentation(info = "<HTML>
<p>
Contains m current sensors (Modelica.Electrical.Analog.Sensors.CurrentSensor),
thus measuring the m currents <i>i[m]</i> flowing from the m pins of plug_p to the m pins of plug_n.
</p>
</HTML>"), Diagram(graphics));

        equation 
          connect(plug_p.pin,currentSensor.p) annotation(Line(points = {{ -100,0},{ -10,0}}, color = {0,0,255}));
          connect(currentSensor.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
          connect(currentSensor.i,i) annotation(Line(points = {{0, -10},{0, -110}}, color = {0,0,255}));
        end CurrentSensor;
        model PowerSensor "Multiphase instantaneous power sensor"
          parameter Integer m(min = 1) = 3 "number of phases";
          annotation(Icon(graphics = {Line(points = {{0,100},{0,70}}, color = {0,0,255}),Line(points = {{0, -70},{0, -100}}, color = {0,0,255}),Line(points = {{ -80, -100},{ -80,0}}, color = {0,0,255}),Text(extent = {{150,120},{ -150,160}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{0, -80},{140, -120}}, lineColor = {0,0,0}, textString = "m=%m"),Ellipse(extent = {{ -70,70},{70, -70}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,70},{0,40}}, color = {0,0,0}),Line(points = {{22.9,32.8},{40.2,57.3}}, color = {0,0,0}),Line(points = {{ -22.9,32.8},{ -40.2,57.3}}, color = {0,0,0}),Line(points = {{37.6,13.7},{65.8,23.9}}, color = {0,0,0}),Line(points = {{ -37.6,13.7},{ -65.8,23.9}}, color = {0,0,0}),Line(points = {{0,0},{9.02,28.6}}, color = {0,0,0}),Polygon(points = {{ -0.48,31.6},{18,26},{18,57.2},{ -0.48,31.6}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -5,5},{5, -5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -29, -11},{30, -70}}, lineColor = {0,0,0}, textString = "P"),Line(points = {{ -100,0},{100,0}}, color = {0,0,255})}), Diagram(graphics), Documentation(info = "<html><p>
This power sensor measures instantaneous electrical power of a multiphase system and has a separated voltage and current path. The plugs of the voltage path are <code>pv</code> and <code>nv</code>, the plugs of the current path are <code>pc</code> and <code>nc</code>. The internal resistance of each current path is zero, the internal resistance of each voltage path is infinite.
</p></html>"));
          MultiPhase.Interfaces.PositivePlug pc(final m = m) "Positive plug, current path" annotation(Placement(transformation(extent = {{ -110,10},{ -90, -10}}, rotation = 0)));
          MultiPhase.Interfaces.NegativePlug nc(final m = m) "Negative plug, current path" annotation(Placement(transformation(extent = {{90,10},{110, -10}}, rotation = 0)));
          MultiPhase.Interfaces.PositivePlug pv(final m = m) "Positive plug, voltage path" annotation(Placement(transformation(extent = {{ -10,90},{10,110}}, rotation = 0)));
          MultiPhase.Interfaces.NegativePlug nv(final m = m) "Negative plug, voltage path" annotation(Placement(transformation(extent = {{ -10, -90},{10, -110}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput power annotation(Placement(transformation(origin = { -80, -110}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          MultiPhase.Sensors.VoltageSensor voltageSensor(final m = m) annotation(Placement(transformation(origin = {0, -20}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          MultiPhase.Sensors.CurrentSensor currentSensor(final m = m) annotation(Placement(transformation(extent = {{ -50, -10},{ -30,10}}, rotation = 0)));
          Modelica.Blocks.Math.Product product[m] annotation(Placement(transformation(origin = { -30, -40}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Blocks.Math.Sum sum(final nin = m, final k = ones(m)) annotation(Placement(transformation(origin = { -30, -70}, extent = {{10, -10},{ -10,10}}, rotation = 90)));

        equation 
          connect(pc,currentSensor.plug_p) annotation(Line(points = {{ -100,0},{ -50,0}}, color = {0,0,255}));
          connect(currentSensor.plug_n,nc) annotation(Line(points = {{ -30,0},{100,0}}, color = {0,0,255}));
          connect(voltageSensor.plug_p,pv) annotation(Line(points = {{6.12303e-16, -10},{0, -10},{0,100}}, color = {0,0,255}));
          connect(voltageSensor.plug_n,nv) annotation(Line(points = {{ -6.12303e-16, -30},{0, -30},{0, -100}}, color = {0,0,255}));
          connect(currentSensor.i,product.u2) annotation(Line(points = {{ -40, -11},{ -40, -20},{ -36, -20},{ -36, -28}}, color = {0,0,127}));
          connect(product.u1,voltageSensor.v) annotation(Line(points = {{ -24, -28},{ -24, -20},{ -11, -20}}, color = {0,0,127}));
          connect(product.y,sum.u) annotation(Line(points = {{ -30, -51},{ -30, -58}}, color = {0,0,127}));
          connect(sum.y,power) annotation(Line(points = {{ -30, -81},{ -30, -90},{ -80, -90},{ -80, -110}}, color = {0,0,127}));
        end PowerSensor;
      end Sensors;
      package Sources "Multiphase voltage and current sources"
        extends Modelica.Icons.Library2;
        annotation(Documentation(info = "<HTML>
<p>
This package contains time-dependend and controlled multiphase voltage and current sources:
<ul>
<li>SignalVoltage: fed by Modelica.Blocks.Sources arbitrary waveforms of voltages are possible</li>
<li>SineVoltage : phase shift between consecutive voltages by default <tt>= pi/m</tt></li>
<li>SignalCurrent: fed by Modelica.Blocks.Sources arbitrary waveforms of currents are possible</li>
<li>SineCurrent : phase shift between consecutive currents by default <tt>= pi/m</tt></li>
</ul>
</p>

</HTML>", revisions = "<html>
<dl>
<p>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
</p>
<p>
  <dt><b>Release Notes:</b></dt>
  <dd>
  <ul>
  <li> v1.0 2004/10/01 Anton Haumer</li>
  </ul>
  </dd>
<p>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2006, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</p>
</dl>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -100, -40},{ -60, -40}}, color = {0,0,255}),Line(points = {{ -60, -40},{40, -40}}, color = {0,0,255}),Line(points = {{40, -40},{80, -40}}, color = {0,0,255}),Ellipse(extent = {{ -60,10},{40, -90}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}));
        model SignalVoltage "Multiphase signal voltage source"
          parameter Integer m(min = 1) = 3 "number of phases";
          Modelica.SIunits.Current i[m] "Currents flowing into positive plugs";
          Interfaces.PositivePlug plug_p(final m = m) annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Interfaces.NegativePlug plug_n(final m = m) annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput v[m] "Voltage between pin p and n (= p.v - n.v) as input signal" annotation(Placement(transformation(origin = {0,70}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
          Modelica.Electrical.Analog.Sources.SignalVoltage signalVoltage[m] annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(graphics = {Line(points = {{ -90,0},{ -50,0}}, color = {0,0,255}),Line(points = {{50,0},{90,0}}, color = {0,0,255}),Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -50,0},{50,0}}, color = {0,0,255}),Text(extent = {{ -150, -110},{150, -50}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100,100},{ -20,60}}, lineColor = {0,0,0}),Text(extent = {{20,100},{100,60}}, lineColor = {0,0,0}, textString = "%m"),Text(extent = {{30,60},{110,0}}, lineColor = {0,0,255}, textString = "-"),Text(extent = {{ -110,60},{ -30,0}}, lineColor = {0,0,255}, textString = "+")}), Documentation(info = "<HTML>
<p>
Contains m signal controlled voltage sources (Modelica.Electrical.Analog.Sources.SignalVoltage)
</p>
</HTML>"), Diagram(graphics));

        equation 
          i = plug_p.pin.i;
          connect(signalVoltage.p,plug_p.pin) annotation(Line(points = {{ -10,0},{ -100,0}}, color = {0,0,255}));
          connect(signalVoltage.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
          connect(v,signalVoltage.v) annotation(Line(points = {{0,70},{0,7}}, color = {0,0,255}));
        end SignalVoltage;
        model ConstantVoltage "Multiphase constant voltage source"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Voltage V[m](start = fill(1, m)) "Value of constant voltage";
          Modelica.Electrical.Analog.Sources.ConstantVoltage constantVoltage[m](final V = V) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(graphics = {Line(points = {{ -90,0},{ -50,0}}, color = {0,0,255}),Line(points = {{50,0},{90,0}}, color = {0,0,255}),Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -50,0},{50,0}}, color = {0,0,255}),Text(extent = {{ -150, -110},{150, -50}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100,100},{100,60}}, lineColor = {0,0,0}, textString = "m=%m"),Text(extent = {{30,60},{110,0}}, lineColor = {0,0,255}, textString = "-"),Text(extent = {{ -110,60},{ -30,0}}, lineColor = {0,0,255}, textString = "+")}), Documentation(info = "<HTML>
<p>
Contains m constant voltage sources (Modelica.Electrical.Analog.Sources.ConstantVoltage)
</p>
</HTML>"), Diagram(graphics));

        equation 
          connect(constantVoltage.p,plug_p.pin) annotation(Line(points = {{ -10,0},{ -100,0}}, color = {0,0,255}));
          connect(constantVoltage.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
        end ConstantVoltage;
        model SineVoltage "Multiphase sine voltage source"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Voltage V[m](start = fill(1, m)) "Amplitudes of sine waves";
          parameter Modelica.SIunits.Angle phase[m] =  -array((j - 1) / m * 2 * Modelica.Constants.pi for j in 1:m) "Phases of sine waves";
          parameter Modelica.SIunits.Frequency freqHz[m](start = fill(1, m)) "Frequencies of sine waves";
          parameter Modelica.SIunits.Voltage offset[m] = zeros(m) "Voltage offsets";
          parameter Modelica.SIunits.Time startTime[m] = zeros(m) "Time offsets";
          Modelica.Electrical.Analog.Sources.SineVoltage sineVoltage[m](final V = V, final phase = phase, final freqHz = freqHz, final offset = offset, final startTime = startTime) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -90,0},{ -50,0}}, color = {0,0,255}),Line(points = {{50,0},{90,0}}, color = {0,0,255}),Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -50,0},{50,0}}, color = {0,0,255}),Line(points = {{ -70,0},{ -60.2,29.9},{ -53.8,46.5},{ -48.2,58.1},{ -43.3,65.2},{ -38.3,69.2},{ -33.4,69.8},{ -28.5,67},{ -23.6,61},{ -18.6,52},{ -13,38.6},{ -5.98,18.6},{8.79, -26.9},{15.1, -44},{20.8, -56.2},{25.7, -64},{30.6, -68.6},{35.5, -70},{40.5, -67.9},{45.4, -62.5},{50.3, -54.1},{55.9, -41.3},{63, -21.7},{70,0}}, color = {192,192,192}),Text(extent = {{ -150, -110},{150, -50}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100,100},{100,60}}, lineColor = {0,0,0}, textString = "m=%m"),Text(extent = {{30,60},{110,0}}, lineColor = {0,0,255}, textString = "-"),Text(extent = {{ -110,60},{ -30,0}}, lineColor = {0,0,255}, textString = "+")}), Documentation(info = "<HTML>
<p>
Contains m sine voltage sources (Modelica.Electrical.Analog.Sources.SineVoltage)
with a default phase shift of -(j-1)/m * 2*pi for j in 1:m.
</p>
</HTML>"), Diagram(graphics));

        equation 
          connect(sineVoltage.p,plug_p.pin) annotation(Line(points = {{ -10,0},{ -100,0}}, color = {0,0,255}));
          connect(sineVoltage.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
        end SineVoltage;
        model SignalCurrent "Multiphase sine current source"
          parameter Integer m(min = 1) = 3 "number of phases";
          Modelica.SIunits.Voltage v[m] "Voltage drops between the two plugs";
          Interfaces.PositivePlug plug_p(final m = m) annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Interfaces.NegativePlug plug_n(final m = m) annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput i[m] "Current flowing from pin p to pin n as input signal" annotation(Placement(transformation(origin = {0,70}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
          Modelica.Electrical.Analog.Sources.SignalCurrent signalCurrent[m] annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(graphics = {Line(points = {{ -90,0},{ -50,0}}, color = {0,0,255}),Line(points = {{50,0},{90,0}}, color = {0,0,255}),Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,50},{0, -50}}, color = {0,0,255}),Text(extent = {{ -150, -110},{150, -50}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100,100},{ -20,60}}, lineColor = {0,0,0}, textString = "m="),Text(extent = {{20,100},{100,60}}, lineColor = {0,0,0}, textString = "%m"),Polygon(points = {{90,0},{60,10},{60, -10},{90,0}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
<p>
Contains m signal controlled current sources (Modelica.Electrical.Analog.Sources.SignalCurrent)
</p>
</HTML>"), Diagram(graphics));

        equation 
          v = plug_p.pin.v - plug_n.pin.v;
          connect(signalCurrent.p,plug_p.pin) annotation(Line(points = {{ -10,0},{ -100,0}}, color = {0,0,255}));
          connect(signalCurrent.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
          connect(i,signalCurrent.i) annotation(Line(points = {{0,70},{0,7}}, color = {0,0,255}));
        end SignalCurrent;
        model ConstantCurrent "Multiphase constant current source"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Current I[m](start = fill(1, m)) "Value of constant current";
          Modelica.Electrical.Analog.Sources.ConstantCurrent constantCurrent[m](final I = I) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(graphics = {Line(points = {{ -90,0},{ -50,0}}, color = {0,0,255}),Line(points = {{50,0},{90,0}}, color = {0,0,255}),Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,50},{0, -50}}, color = {0,0,255}),Text(extent = {{ -150, -110},{150, -50}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100,100},{100,60}}, lineColor = {0,0,0}, textString = "m=%m"),Polygon(points = {{90,0},{60,10},{60, -10},{90,0}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
<p>
Contains m constant current sources (Modelica.Electrical.Analog.Sources.ConstantCurrent)
</p>
</HTML>"), Diagram(graphics));

        equation 
          connect(constantCurrent.p,plug_p.pin) annotation(Line(points = {{ -10,0},{ -100,0}}, color = {0,0,255}));
          connect(constantCurrent.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
        end ConstantCurrent;
        model SineCurrent "Multiphase sine current source"
          extends Interfaces.TwoPlug;
          parameter Modelica.SIunits.Current I[m](start = fill(1, m)) "Amplitudes of sine waves";
          parameter Modelica.SIunits.Angle phase[m] =  -array((j - 1) / m * 2 * Modelica.Constants.pi for j in 1:m) "Phases of sine waves";
          parameter Modelica.SIunits.Frequency freqHz[m](start = fill(1, m)) "Frequencies of sine waves";
          parameter Modelica.SIunits.Current offset[m] = zeros(m) "Current offsets";
          parameter Modelica.SIunits.Time startTime[m] = zeros(m) "Time offsets";
          Modelica.Electrical.Analog.Sources.SineCurrent sineCurrent[m](final I = I, final phase = phase, final freqHz = freqHz, final offset = offset, final startTime = startTime) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          annotation(Icon(graphics = {Line(points = {{ -90,0},{ -50,0}}, color = {0,0,255}),Line(points = {{50,0},{90,0}}, color = {0,0,255}),Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,50},{0, -50}}, color = {0,0,255}),Line(points = {{ -70,0},{ -60.2,29.9},{ -53.8,46.5},{ -48.2,58.1},{ -43.3,65.2},{ -38.3,69.2},{ -33.4,69.8},{ -28.5,67},{ -23.6,61},{ -18.6,52},{ -13,38.6},{ -5.98,18.6},{8.79, -26.9},{15.1, -44},{20.8, -56.2},{25.7, -64},{30.6, -68.6},{35.5, -70},{40.5, -67.9},{45.4, -62.5},{50.3, -54.1},{55.9, -41.3},{63, -21.7},{70,0}}, color = {192,192,192}),Text(extent = {{ -150, -110},{150, -50}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{ -100,100},{100,60}}, lineColor = {0,0,0}, textString = "m=%m"),Polygon(points = {{90,0},{60,10},{60, -10},{90,0}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
<p>
Contains m sine current sources (Modelica.Electrical.Analog.Sources.SineCurrent)
with a default phase shift of -(j-1)/m * 2*pi for j in 1:m.
</p>
</HTML>"), Diagram(graphics));

        equation 
          connect(sineCurrent.p,plug_p.pin) annotation(Line(points = {{ -10,0},{ -100,0}}, color = {0,0,255}));
          connect(sineCurrent.n,plug_n.pin) annotation(Line(points = {{10,0},{100,0}}, color = {0,0,255}));
        end SineCurrent;
      end Sources;
    end MultiPhase;
    package Machines "Library for electric machines"
      extends Modelica.Icons.Library2;
      annotation(Settings(NewStateSelection = true, Evaluate = true), Documentation(info = "<HTML>
This package contains components to model electrical machines:
<ul>
<li>Examples: test examples</li>
<li>BasicMachines: basic machine models</li>
<li>Sensors: sensors, usefull when modelling machines</li>
<li>SpacePhasors: an independent library for using space phasors</li>
<li>Interfaces: Space phasor connector and partial machine models</li>
</ul>
<b>Limitations and assumptions:</b>
<ul>
<li>number of phases (of induction machines) is limited to 3, therefore definition as a constant m=3</li>
<li>phase symmetric windings as well as symmetry of the whole machine structure</li>
<li>all values are used in physical units, no scaling to p.u. is done</li>
<li>only basic harmonics (in space) are taken into account</li>
<li>waveform (with respect to time) of voltages and currents is not restricted</li>
<li>constant parameters, i.e. no saturation, no skin effect</li>
<li>no iron losses, eddy currents, friction losses;<br>
    only ohmic losses in stator and rotor winding</li>
</ul>
<p>
You may have a look at a short summary of space phasor theory at <a href=\"http://www.haumer.at/refimg/SpacePhasors.pdf\">http://www.haumer.at/refimg/SpacePhasors.pdf</a>
</p>
<b>Further development:</b>
<ul>
<li>generalizing space phasor theory to m phases with arbitrary spatial angle of the coils</li>
<li>generalizing space phasor theory to arbitrary number of windings and winding factor of the coils</li>
<li>MachineModels: other machine types</li>
<li>effects: saturation, skin-effect, other losses than ohmic, ...</li>
</ul>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
</dl>
<p>
Copyright &copy; 1998-2009, Modelica Association and Anton Haumer.
</p>
<p>
<i>This Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p>
</HTML>", revisions = "<HTML>
  <ul>
  <li> v1.00  2004/09/16 Anton Haumer<br>
       first stable release</li>
  <li> v1.01  2004/09/18 Anton Haumer<br>
       moved common equations from machine models to PartialMachine<br>
       improved MoveToRotational</li>
  <li> v1.02  2004/09/19 Anton Haumer<br>
       new package structure for machine types<br>
       added DC machine models</li>
  <li> v1.03  2004/09/24 Anton Haumer<br>
       added package Sensors<br>
       added DC machine with series excitation<br>
       debugged and improved MoveToRotational</li>
  <li> v1.1   2004/10/01 Anton Haumer<br>
       changed naming and structure<br>
       issued to Modelica Standard Library 2.1</li>
  <li> v1.2   2004/10/27 Anton Haumer<br>
       fixed a bug with support (formerly bearing)</li>
  <li> v1.3   2004/11/05 Anton Haumer<br>
       several improvements in SpacePhasors.Blocks</li>
  <li> v1.3.1 2004/11/06 Anton Haumer<br>
       small changes in Examples.Utilities.VfController</li>
  <li> v1.3.2 2004/11/10 Anton Haumer<br>
       ReluctanceRotor moved to SynchronousMachines</li>
  <li> v1.4   2004/11/11 Anton Haumer<br>
       removed mechanical flange support<br>
       to ease the implementation of a 3D-frame in a future release</li>
  <li> v1.51  2005/02/01 Anton Haumer<br>
       changed parameter polePairs to Integer</li>
  <li> v1.52  2005/10/12 Anton Haumer<br>
       added BasicMachines.SynchronousInductionMachines.SM_ElectricalExcited<br>
       using new basicMachines.Components.ElectricalExcitation<br>
       as well as a new exmaple.</li>
  <li> v1.53  2005/10/14 Anton Haumer<br>
       introduced unsymmetrical DamperCage for Synchronous Machines</li>
  <li> v1.60  2005/11/04 Anton Haumer<br>
       added SpacePhasors.Components.Rotator<br>
       corrected consistent naming of parameters and variables</li>
  <li> v1.6.1 2005/11/22 Anton Haumer<br>
       improved Transformation and Rotation in SpacePhasor.<br>
       introduced Examples.Utilities.TerminalBox</li>
  <li> v1.6.2 2005/10/23 Anton Haumer<br>
       selectable DamperCage for Synchronous Machines</li>
  <li> v1.6.3 2005/11/25 Anton Haumer<br>
       easier parametrisation of AsynchronousInductionMachines.AIM_SlipRing model</li>
  <li> v1.7.0 2005/12/15 Anton Haumer<br>
       back-changed the naming to ensure backward compatibility</li>
  <li> v1.7.1 2006/02/06 Anton Haumer<br>
       changed some naming of synchronous machines, not affecting existing models</li>
  <li> v1.8.0 2006/11/26 Anton Haumer<br>
       introduced package Transformers<br>
       moved common parameters and functionality to partial models,<br>
       keeping Interfaces.PartialBasicInductionMachines resp. PartialBasicDCMachine for compatibility reasons.<br>
       implemented support showing reaction torque if connected</li>
  <li> v1.8.1 2006/12/01 Anton Haumer<br>
       resolved a compatibility issue with airGap</li>
  <li> v1.8.2 2007/01/15 Anton Haumer<br>
       resolved a bug in electrical excited synchronous induction machine</li>
  <li> v1.8.3 2007/06/08 Anton Haumer<br>
       documentation update</li>
  <li> v1.8.4 2007/06/25 Anton Haumer<br>
       corrected some typos in documentation</li>
  <li> v1.8.5 2007/06/26 Anton Haumer<br>
       consistent parameters of DCSE</li>
  <li> v1.8.6 2007/08/12 Anton Haumer<br>
       improved documentation</li>
  <li> v1.8.7 2007/08/20 Anton Haumer<br>
       corrected typo in documentation</li>
  <li> v1.8.8 2007/08/20 Anton Haumer<br>
       improved documentation</li>
  <li> v1.9.0 2007/08/24 Anton Haumer<br>
       removed redeclare type SignalType</li>
  <li> v1.9.1 2007/10/15 Anton Haumer<br>
       solved a bug with replaceable airgap / partial machines</li>
  <li> v1.9.2 2007/10/15 Anton Haumer<br>
       changed some names to be more concise (see conversion script)</li>
  <li> v2.0.0 2007/11/13 Anton Haumer<br>
       removed replaceable from airgaps<br>
       removed cardinality from support, using a boolean parameter<br>
       removed all nonSIunits</li>
  <li> v2.1.0 2009/08/26 Anton Haumer<br>
       set all useHeatPort=false</li>
  </ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,128,255}),Rectangle(extent = {{ -60,60},{ -80, -60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {128,128,128}),Rectangle(extent = {{60,10},{80, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {95,95,95}),Rectangle(extent = {{ -60,70},{20,50}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{ -70, -90},{ -60, -90},{ -30, -20},{20, -20},{50, -90},{60, -90},{60, -100},{ -70, -100},{ -70, -90}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}));
      package Examples "Test examples"
        extends Modelica.Icons.Library2;
        annotation(Documentation(info = "<HTML>
This package contains test examples of electric machines.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2009, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.00 2004/09/16 Anton Haumer</li>
  <li> v1.01 2004/09/18 Anton Haumer<br>
       adapted to improved MoveToRotational</li>
  <li> v1.02 2004/09/19 Anton Haumer<br>
       added examples for DC machines</li>
  <li> v1.03 2004/09/24 Anton Haumer<br>
       usage of Sensors.CurrentRMSSensor<br>
       added example for DC machine with series excitation</li>
  <li> v1.1  2004/10/01 Anton Haumer<br>
       changed naming and structure<br>
       issued to Modelica Standard Library 2.1</li>
  <li> v1.3.1 2004/11/06 Anton Haumer<br>
       small changes in Utilities.VfController</li>
  <li> v1.52 2005/10/12 Anton Haumer<br>
       new example for electrical excited synchronous induction machine</li>
  <li> v1.6.1 2004/11/22 Anton Haumer<br>
       introduced Utilities.TerminalBox</li>
  </ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -80,44},{60, -96}}, lineColor = {95,95,95}),Polygon(points = {{ -40,36},{ -40, -88},{60, -26},{ -40,36}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid)}));
        model AIMC_DOL "Test example 1: AsynchronousInductionMachineSquirrelCage direct-on-line"
          extends Modelica.Icons.Example;
          constant Integer m = 3 "number of phases";
          parameter Modelica.SIunits.Voltage VNominal = 100 "nominal RMS voltage per phase";
          parameter Modelica.SIunits.Frequency fNominal = 50 "nominal frequency";
          parameter Modelica.SIunits.Time tStart1 = 0.1 "start time";
          parameter Modelica.SIunits.Torque TLoad = 161.4 "nominal load torque";
          parameter Modelica.SIunits.AngularVelocity wLoad(displayUnit = "1/min") = (1440.45 * 2 * Modelica.Constants.pi) / 60 "nominal load speed";
          parameter Modelica.SIunits.Inertia JLoad = 0.29 "load's moment of inertia";
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1.5, Interval = 0.001), Documentation(info = "<HTML>
<b>1st Test example: Asynchronous induction machine with squirrel cage - direct on line starting</b><br>
At start time tStart three phase voltage is supplied to the asynchronous induction machine with squirrel cage;
the machine starts from standstill, accelerating inertias against load torque quadratic dependent on speed, finally reaching nominal speed.<br>
Simulate for 1.5 seconds and plot (versus time):
<ul>
<li>CurrentRMSSensor1.I: stator current RMS</li>
<li>AIMC1.rpmMechanical: motor's speed</li>
<li>AIMC1.tauElectrical: motor's torque</li>
</ul>
Default machine parameters of model <i>AIM_SquirrelCage</i> are used.
</HTML>"));
          Machines.BasicMachines.AsynchronousInductionMachines.AIM_SquirrelCage aimc annotation(Placement(transformation(extent = {{ -20, -50},{0, -30}}, rotation = 0)));
          Machines.Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensor annotation(Placement(transformation(origin = {0,0}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Sources.SineVoltage sineVoltage(final m = m, freqHz = fill(fNominal, m), V = fill(sqrt(2 / 3) * VNominal, m)) annotation(Placement(transformation(origin = {0,60}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(Placement(transformation(extent = {{ -50,80},{ -70,100}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(Placement(transformation(origin = { -90,90}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Blocks.Sources.BooleanStep booleanStep[m](each startTime = tStart1) annotation(Placement(transformation(extent = {{ -80,30},{ -60,50}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Ideal.IdealClosingSwitch idealCloser(final m = m) annotation(Placement(transformation(origin = {0,30}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(Placement(transformation(extent = {{40, -50},{60, -30}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticLoadTorque(w_nominal = wLoad, TorqueDirection = false, tau_nominal =  -TLoad, useSupport = false) annotation(Placement(transformation(extent = {{90, -50},{70, -30}}, rotation = 0)));
          Machines.Utilities.TerminalBox TerminalBox1(terminalConnection = "D") annotation(Placement(transformation(extent = {{ -20, -30},{0, -10}}, rotation = 0)));

        equation 
          connect(star.pin_n,ground.p) annotation(Line(points = {{ -70,90},{ -80,90}}, color = {0,0,255}));
          connect(sineVoltage.plug_n,star.plug_p) annotation(Line(points = {{1.83697e-15,70},{1.83697e-15,90},{ -50,90}}, color = {0,0,255}));
          connect(sineVoltage.plug_p,idealCloser.plug_p) annotation(Line(points = {{ -1.83697e-15,50},{0,48},{1.22461e-15,46},{1.83697e-15,46},{1.83697e-15,40}}, color = {0,0,255}));
          connect(booleanStep.y,idealCloser.control) annotation(Line(points = {{ -59,40},{ -20,40},{ -20,30},{ -7,30}}, color = {255,0,255}));
          connect(idealCloser.plug_n,currentQuasiRMSSensor.plug_p) annotation(Line(points = {{ -1.83697e-15,20},{ -1.83697e-15,17},{1.83697e-15,17},{1.83697e-15,10}}, color = {0,0,255}));
          connect(TerminalBox1.plug_sn,aimc.plug_sn) annotation(Line(points = {{ -16, -30},{ -16, -30}}, color = {0,0,255}, smooth = Smooth.None));
          connect(TerminalBox1.plug_sp,aimc.plug_sp) annotation(Line(points = {{ -4, -30},{ -4, -30}}, color = {0,0,255}, smooth = Smooth.None));
          connect(TerminalBox1.plugSupply,currentQuasiRMSSensor.plug_n) annotation(Line(points = {{ -10, -28},{ -10, -20},{ -1.83697e-15, -20},{ -1.83697e-15, -10}}, color = {0,0,255}, smooth = Smooth.None));
          connect(loadInertia.flange_b,quadraticLoadTorque.flange) annotation(Line(points = {{60, -40},{70, -40}}, color = {0,0,0}, smooth = Smooth.None));
          connect(aimc.flange,loadInertia.flange_a) annotation(Line(points = {{0, -40},{40, -40}}, color = {0,0,0}, smooth = Smooth.None));
        end AIMC_DOL;
        model AIMC_YD "Test example 2: AsynchronousInductionMachineSquirrelCage Y-D"
          extends Modelica.Icons.Example;
          constant Integer m = 3 "number of phases";
          parameter Modelica.SIunits.Voltage VNominal = 100 "nominal RMS voltage per phase";
          parameter Modelica.SIunits.Frequency fNominal = 50 "nominal frequency";
          parameter Modelica.SIunits.Time tStart1 = 0.1 "start time";
          parameter Modelica.SIunits.Time tStart2 = 2.0 "2nd start time";
          parameter Modelica.SIunits.Torque TLoad = 161.4 "nominal load torque";
          parameter Modelica.SIunits.AngularVelocity wLoad(displayUnit = "1/min") = (1440.45 * 2 * Modelica.Constants.pi) / 60 "nominal load speed";
          parameter Modelica.SIunits.Inertia JLoad = 0.29 "load's moment of inertia";
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 2.5, Interval = 0.001), Documentation(info = "<HTML>
<b>2nd Test example: Asynchronous induction machine with squirrel cage - Y-D starting</b><br>
At start time tStart three phase voltage is supplied to the asynchronous induction machine with squirrel cage, first star-connected, then delta-connetced; the machine starts from standstill, accelerating inertias against load torque quadratic dependent on speed, finally reaching nominal speed.<br>
Simulate for 2.5 seconds and plot (versus time):
<ul>
<li>CurrentRMSSensor1.I: stator current RMS</li>
<li>AIMC1.rpmMechanical: motor's speed</li>
<li>AIMC1.tauElectrical: motor's torque</li>
</ul>
Default machine parameters of model <i>AIM_SquirrelCage</i> are used.
</HTML>"));
          Machines.BasicMachines.AsynchronousInductionMachines.AIM_SquirrelCage aimc annotation(Placement(transformation(extent = {{ -20, -50},{0, -30}}, rotation = 0)));
          Machines.Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensor annotation(Placement(transformation(origin = {0,0}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Sources.SineVoltage sineVoltage(final m = m, freqHz = fill(fNominal, m), V = fill(sqrt(2 / 3) * VNominal, m)) annotation(Placement(transformation(origin = {0,60}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(Placement(transformation(extent = {{ -50,80},{ -70,100}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(Placement(transformation(origin = { -90,90}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Blocks.Sources.BooleanStep booleanStep[m](each startTime = tStart1) annotation(Placement(transformation(extent = {{ -80,30},{ -60,50}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Ideal.IdealClosingSwitch idealCloser(final m = m) annotation(Placement(transformation(origin = {0,30}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Machines.Utilities.SwitchYD switchYD annotation(Placement(transformation(extent = {{ -20, -30},{0, -10}}, rotation = 0)));
          Modelica.Blocks.Sources.BooleanStep booleanStepYD[m](each startTime = tStart2) annotation(Placement(transformation(extent = {{ -80, -30},{ -60, -10}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(Placement(transformation(extent = {{40, -50},{60, -30}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticLoadTorque(w_nominal = wLoad, TorqueDirection = false, tau_nominal =  -TLoad, useSupport = false) annotation(Placement(transformation(extent = {{90, -50},{70, -30}}, rotation = 0)));

        equation 
          connect(star.pin_n,ground.p) annotation(Line(points = {{ -70,90},{ -80,90}}, color = {0,0,255}));
          connect(sineVoltage.plug_n,star.plug_p) annotation(Line(points = {{1.83697e-15,70},{1.83697e-15,90},{ -50,90}}, color = {0,0,255}));
          connect(sineVoltage.plug_p,idealCloser.plug_p) annotation(Line(points = {{ -1.83697e-15,50},{0,48},{1.22461e-15,46},{1.83697e-15,46},{1.83697e-15,40}}, color = {0,0,255}));
          connect(loadInertia.flange_b,quadraticLoadTorque.flange) annotation(Line(points = {{60, -40},{70, -40}}, color = {0,0,0}));
          connect(booleanStep.y,idealCloser.control) annotation(Line(points = {{ -59,40},{ -20,40},{ -20,30},{ -7,30}}, color = {255,0,255}));
          connect(booleanStepYD.y,switchYD.control) annotation(Line(points = {{ -59, -20},{ -21, -20}}, color = {255,0,255}));
          connect(idealCloser.plug_n,currentQuasiRMSSensor.plug_p) annotation(Line(points = {{ -1.83697e-15,20},{ -1.83697e-15,18},{1.83697e-15,18},{1.83697e-15,10}}, color = {0,0,255}));
          connect(switchYD.plug_sn,aimc.plug_sn) annotation(Line(points = {{ -16, -30},{ -16, -30}}, color = {0,0,255}, smooth = Smooth.None));
          connect(switchYD.plug_sp,aimc.plug_sp) annotation(Line(points = {{ -4, -30},{ -4, -30}}, color = {0,0,255}, smooth = Smooth.None));
          connect(switchYD.plugSupply,currentQuasiRMSSensor.plug_n) annotation(Line(points = {{ -10, -10},{ -1.83697e-15, -10}}, color = {0,0,255}, smooth = Smooth.None));
          connect(aimc.flange,loadInertia.flange_a) annotation(Line(points = {{0, -40},{40, -40}}, color = {0,0,0}, smooth = Smooth.None));
        end AIMC_YD;
        model AIMS_Start "Test example 3: AsynchronousInductionMachineSlipRing"
          extends Modelica.Icons.Example;
          constant Integer m = 3 "number of phases";
          parameter Modelica.SIunits.Voltage VNominal = 100 "nominal RMS voltage per phase";
          parameter Modelica.SIunits.Frequency fNominal = 50 "nominal frequency";
          parameter Modelica.SIunits.Time tStart1 = 0.1 "1st start time";
          parameter Modelica.SIunits.Resistance Rstart = 0.16 "starting resistance";
          parameter Modelica.SIunits.Time tStart2 = 1.0 "2nd start time";
          parameter Modelica.SIunits.Torque TLoad = 161.4 "nominal load torque";
          parameter Modelica.SIunits.AngularVelocity wLoad(displayUnit = "1/min") = (1440.45 * 2 * Modelica.Constants.pi) / 60 "nominal load speed";
          parameter Modelica.SIunits.Inertia JLoad = 0.29 "load's moment of inertia";
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1.5, Interval = 0.001), Documentation(info = "<HTML>
<b>3rd Test example: Asynchronous induction machine with slipring rotor - resistance starting</b><br>
At start time tStart1 three phase voltage is supplied to the asynchronous induction machine with sliprings;
the machine starts from standstill, accelerating inertias against load torque quadratic dependent on speed,
using a starting resistance. At time tStart2 external rotor resistance is shortened, finally reaching nominal speed.<br>
Simulate for 1.5 seconds and plot (versus time):
<ul>
<li>CurrentRMSSensor1.I: stator current RMS</li>
<li>AIMS1.rpmMechanical: motor's speed</li>
<li>AIMS1.tauElectrical: motor's torque</li>
</ul>
Default machine parameters of model <i>AIM_SlipRing</i> are used.
</HTML>"));
          Machines.BasicMachines.AsynchronousInductionMachines.AIM_SlipRing aims annotation(Placement(transformation(extent = {{ -20, -50},{0, -30}}, rotation = 0)));
          Machines.Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensor annotation(Placement(transformation(origin = {0,0}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Sources.SineVoltage sineVoltage(final m = m, freqHz = fill(fNominal, m), V = fill(sqrt(2 / 3) * VNominal, m)) annotation(Placement(transformation(origin = {0,60}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(Placement(transformation(extent = {{ -50,80},{ -70,100}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(Placement(transformation(origin = { -90,90}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Blocks.Sources.BooleanStep booleanStep[m](each startTime = tStart1) annotation(Placement(transformation(extent = {{ -80,30},{ -60,50}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Ideal.IdealClosingSwitch idealCloser(final m = m) annotation(Placement(transformation(origin = {0,30}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Basic.Star starRotor(final m = m) annotation(Placement(transformation(extent = {{ -50, -100},{ -70, -80}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground groundRotor annotation(Placement(transformation(origin = { -90, -90}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Ideal.IdealCommutingSwitch idealCommutingSwitch(m = m) annotation(Placement(transformation(origin = { -40, -40}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Basic.Resistor rotorResistor(m = m, R = fill(Rstart, m)) annotation(Placement(transformation(extent = {{ -50, -80},{ -70, -60}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Basic.Star starRotorResistor(final m = m) annotation(Placement(transformation(extent = {{ -80, -80},{ -100, -60}}, rotation = 0)));
          Modelica.Blocks.Sources.BooleanStep booleanStepRotor[m](each startTime = tStart2) annotation(Placement(transformation(extent = {{ -80, -50},{ -60, -30}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(Placement(transformation(extent = {{40, -50},{60, -30}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticLoadTorque(w_nominal = wLoad, TorqueDirection = false, tau_nominal =  -TLoad, useSupport = false) annotation(Placement(transformation(extent = {{90, -50},{70, -30}}, rotation = 0)));
          Machines.Utilities.TerminalBox terminalBox(terminalConnection = "D") annotation(Placement(transformation(extent = {{ -20, -30},{0, -10}}, rotation = 0)));

        equation 
          connect(star.pin_n,ground.p) annotation(Line(points = {{ -70,90},{ -80,90}}, color = {0,0,255}));
          connect(sineVoltage.plug_n,star.plug_p) annotation(Line(points = {{1.83697e-15,70},{1.83697e-15,90},{ -50,90}}, color = {0,0,255}));
          connect(sineVoltage.plug_p,idealCloser.plug_p) annotation(Line(points = {{ -1.83697e-15,50},{0,48},{1.22461e-15,46},{1.83697e-15,46},{1.83697e-15,40}}, color = {0,0,255}));
          connect(starRotor.pin_n,groundRotor.p) annotation(Line(points = {{ -70, -90},{ -80, -90}}, color = {0,0,255}));
          connect(starRotorResistor.plug_p,rotorResistor.plug_n) annotation(Line(points = {{ -80, -70},{ -70, -70}}, color = {0,0,255}));
          connect(rotorResistor.plug_p,idealCommutingSwitch.plug_n1) annotation(Line(points = {{ -50, -70},{ -45, -70},{ -45, -50}}, color = {0,0,255}));
          connect(aims.plug_rn,starRotor.plug_p) annotation(Line(points = {{ -20, -46},{ -20, -90},{ -50, -90}}, color = {0,0,255}));
          connect(idealCommutingSwitch.plug_n2,aims.plug_rn) annotation(Line(points = {{ -40, -50},{ -28, -50},{ -28, -46},{ -20, -46}}, color = {0,0,255}));
          connect(loadInertia.flange_b,quadraticLoadTorque.flange) annotation(Line(points = {{60, -40},{70, -40}}, color = {0,0,0}));
          connect(idealCommutingSwitch.plug_p,aims.plug_rp) annotation(Line(points = {{ -40, -30},{ -30, -30},{ -30, -34},{ -20, -34}}, color = {0,0,255}));
          connect(booleanStep.y,idealCloser.control) annotation(Line(points = {{ -59,40},{ -20,40},{ -20,30},{ -7,30}}, color = {255,0,255}));
          connect(booleanStepRotor.y,idealCommutingSwitch.control) annotation(Line(points = {{ -59, -40},{ -48, -40}}, color = {255,0,255}));
          connect(idealCloser.plug_n,currentQuasiRMSSensor.plug_p) annotation(Line(points = {{ -1.83697e-15,20},{ -1.83697e-15,16},{1.83697e-15,16},{1.83697e-15,10}}, color = {0,0,255}));
          connect(terminalBox.plugSupply,currentQuasiRMSSensor.plug_n) annotation(Line(points = {{ -10, -28},{ -10, -20},{ -1.83697e-15, -20},{ -1.83697e-15, -10}}, color = {0,0,255}, smooth = Smooth.None));
          connect(terminalBox.plug_sn,aims.plug_sn) annotation(Line(points = {{ -16, -30},{ -16, -30}}, color = {0,0,255}, smooth = Smooth.None));
          connect(terminalBox.plug_sp,aims.plug_sp) annotation(Line(points = {{ -4, -30},{ -4, -30}}, color = {0,0,255}, smooth = Smooth.None));
          connect(aims.flange,loadInertia.flange_a) annotation(Line(points = {{0, -40},{40, -40}}, color = {0,0,0}, smooth = Smooth.None));
        end AIMS_Start;
        model AIMC_Inverter "Test example 4: AsynchronousInductionMachineSquirrelCage with inverter"
          extends Modelica.Icons.Example;
          constant Integer m = 3 "number of phases";
          parameter Modelica.SIunits.Voltage VNominal = 100 "nominal RMS voltage per phase";
          parameter Modelica.SIunits.Frequency fNominal = 50 "nominal frequency";
          parameter Modelica.SIunits.Frequency f = 50 "actual frequency";
          parameter Modelica.SIunits.Time tRamp = 1 "frequency ramp";
          parameter Modelica.SIunits.Torque TLoad = 161.4 "nominal load torque";
          parameter Modelica.SIunits.Time tStep = 1.2 "time of load torque step";
          parameter Modelica.SIunits.Inertia JLoad = 0.29 "load's moment of inertia";
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1.5, Interval = 0.001), Documentation(info = "<HTML>
<b>4th Test example: Asynchronous induction machine with squirrel cage fed by an ideal inverter</b><br>
An ideal frequency inverter is modeled by using a VfController and a threephase SignalVoltage.<br>
Frequency is raised by a ramp, causing the asynchronous induction machine with squirrel cage to start,
and accelerating inertias.<br>At time tStep a load step is applied.<br>
Simulate for 1.5 seconds and plot (versus time):
<ul>
<li>CurrentRMSSensor1.I: stator current RMS</li>
<li>AIMC1.rpmMechanical: motor's speed</li>
<li>AIMC1.tauElectrical: motor's torque</li>
</ul>
Default machine parameters of model <i>AIM_SquirrelCage</i> are used.
</HTML>"));
          Machines.BasicMachines.AsynchronousInductionMachines.AIM_SquirrelCage aimc annotation(Placement(transformation(extent = {{ -20, -50},{0, -30}}, rotation = 0)));
          Machines.Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensor annotation(Placement(transformation(origin = {0,0}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Modelica.Blocks.Sources.Ramp ramp(height = f, duration = tRamp) annotation(Placement(transformation(extent = {{ -80,50},{ -60,70}}, rotation = 0)));
          Machines.Utilities.VfController vfController(final m = m, VNominal = VNominal, fNominal = fNominal) annotation(Placement(transformation(extent = {{ -40,50},{ -20,70}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Sources.SignalVoltage signalVoltage(final m = m) annotation(Placement(transformation(origin = {0,60}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(Placement(transformation(extent = {{ -50,80},{ -70,100}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(Placement(transformation(origin = { -90,90}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(Placement(transformation(extent = {{40, -50},{60, -30}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Sources.TorqueStep loadTorqueStep(startTime = tStep, stepTorque =  -TLoad, useSupport = false) annotation(Placement(transformation(extent = {{90, -50},{70, -30}}, rotation = 0)));
          Machines.Utilities.TerminalBox terminalBox annotation(Placement(transformation(extent = {{ -20, -30},{0, -10}}, rotation = 0)));

        equation 
          connect(signalVoltage.plug_n,star.plug_p) annotation(Line(points = {{1.83697e-15,70},{1.83697e-15,90},{ -50,90}}, color = {0,0,255}));
          connect(star.pin_n,ground.p) annotation(Line(points = {{ -70,90},{ -80,90}}, color = {0,0,255}));
          connect(ramp.y,vfController.u) annotation(Line(points = {{ -59,60},{ -42,60}}, color = {0,0,255}));
          connect(vfController.y,signalVoltage.v) annotation(Line(points = {{ -19,60},{7,60}}, color = {0,0,255}));
          connect(loadTorqueStep.flange,loadInertia.flange_b) annotation(Line(points = {{70, -40},{60, -40}}, color = {0,0,0}));
          connect(signalVoltage.plug_p,currentQuasiRMSSensor.plug_p) annotation(Line(points = {{ -1.83697e-15,50},{ -1.83697e-15,40},{6.12303e-16,40},{6.12303e-16,10},{1.83697e-15,10}}, color = {0,0,255}));
          connect(terminalBox.plugSupply,currentQuasiRMSSensor.plug_n) annotation(Line(points = {{ -10, -28},{ -10, -20},{0, -20},{0, -10},{ -1.83697e-15, -10}}, color = {0,0,255}, smooth = Smooth.None));
          connect(terminalBox.plug_sn,aimc.plug_sn) annotation(Line(points = {{ -16, -30},{ -16, -30}}, color = {0,0,255}, smooth = Smooth.None));
          connect(terminalBox.plug_sp,aimc.plug_sp) annotation(Line(points = {{ -4, -30},{ -4, -30}}, color = {0,0,255}, smooth = Smooth.None));
          connect(aimc.flange,loadInertia.flange_a) annotation(Line(points = {{0, -40},{40, -40}}, color = {0,0,0}, smooth = Smooth.None));
        end AIMC_Inverter;
        model SMR_Inverter "Test example 5: SynchronousInductionMachineReluctanceRotor with inverter"
          extends Modelica.Icons.Example;
          constant Integer m = 3 "number of phases";
          parameter Modelica.SIunits.Voltage VNominal = 100 "nominal RMS voltage per phase";
          parameter Modelica.SIunits.Frequency fNominal = 50 "nominal frequency";
          parameter Modelica.SIunits.Frequency f = 50 "actual frequency";
          parameter Modelica.SIunits.Time tRamp = 1 "frequency ramp";
          parameter Modelica.SIunits.Torque TLoad = 46 "nominal load torque";
          parameter Modelica.SIunits.Time tStep = 1.2 "time of load torque step";
          parameter Modelica.SIunits.Inertia JLoad = 0.29 "load's moment of inertia";
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1.5, Interval = 0.001), Documentation(info = "<HTML>
<b>5th Test example: Synchronous induction machine with reluctance rotor fed by an ideal inverter</b><br>
An ideal frequency inverter is modeled by using a VfController and a threephase SignalVoltage.<br>
Frequency is raised by a ramp, causing the reluctance machine to start,
and accelerating inertias.<br>At time tStep a load step is applied.<br>
Simulate for 1.5 seconds and plot (versus time):
<ul>
<li>CurrentRMSSensor1.I: stator current RMS</li>
<li>SMRD1.rpmMechanical: motor's speed</li>
<li>SMRD1.tauElectrical: motor's torque</li>
<li>RotorDisplacementAngle.rotorDisplacementAngle: rotor displacement angle</li>
</ul>
Default machine parameters of model <i>SM_ReluctanceRotor</i> are used.
</HTML>"));
          Machines.BasicMachines.SynchronousInductionMachines.SM_ReluctanceRotor smr annotation(Placement(transformation(extent = {{ -20, -50},{0, -30}}, rotation = 0)));
          Machines.Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensor annotation(Placement(transformation(origin = {0,30}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Machines.Sensors.RotorDisplacementAngle rotorDisplacementAngle(p = smr.p) annotation(Placement(transformation(origin = {20, -40}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Modelica.Blocks.Sources.Ramp ramp(height = f, duration = tRamp) annotation(Placement(transformation(extent = {{ -80,50},{ -60,70}}, rotation = 0)));
          Machines.Utilities.VfController vfController(final m = m, VNominal = VNominal, fNominal = fNominal) annotation(Placement(transformation(extent = {{ -40,50},{ -20,70}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Sources.SignalVoltage signalVoltage(final m = m) annotation(Placement(transformation(origin = {0,60}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(Placement(transformation(extent = {{ -50,80},{ -70,100}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(Placement(transformation(origin = { -90,90}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(Placement(transformation(extent = {{40, -50},{60, -30}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Sources.TorqueStep loadTorqueStep(startTime = tStep, stepTorque =  -TLoad, useSupport = false) annotation(Placement(transformation(extent = {{90, -50},{70, -30}}, rotation = 0)));
          Machines.Utilities.TerminalBox terminalBox annotation(Placement(transformation(extent = {{ -20, -30},{0, -10}}, rotation = 0)));

        equation 
          connect(signalVoltage.plug_n,star.plug_p) annotation(Line(points = {{1.83697e-15,70},{1.83697e-15,90},{ -50,90}}, color = {0,0,255}));
          connect(star.pin_n,ground.p) annotation(Line(points = {{ -70,90},{ -80,90}}, color = {0,0,255}));
          connect(ramp.y,vfController.u) annotation(Line(points = {{ -59,60},{ -42,60}}, color = {0,0,255}));
          connect(vfController.y,signalVoltage.v) annotation(Line(points = {{ -19,60},{7,60}}, color = {0,0,255}));
          connect(loadInertia.flange_b,loadTorqueStep.flange) annotation(Line(points = {{60, -40},{70, -40}}, color = {0,0,0}));
          connect(currentQuasiRMSSensor.plug_p,signalVoltage.plug_p) annotation(Line(points = {{1.83697e-15,40},{1.83697e-15,42.5},{6.12303e-16,42.5},{6.12303e-16,50},{ -1.83697e-15,50}}, color = {0,0,255}));
          connect(smr.plug_sn,rotorDisplacementAngle.plug_n) annotation(Line(points = {{ -16, -30},{ -16, -20},{26, -20},{26, -30}}, color = {0,0,255}));
          connect(smr.plug_sp,rotorDisplacementAngle.plug_p) annotation(Line(points = {{ -4, -30},{14, -30}}, color = {0,0,255}));
          connect(terminalBox.plugSupply,currentQuasiRMSSensor.plug_n) annotation(Line(points = {{ -10, -28},{ -10,0},{ -1.83697e-15,0},{ -1.83697e-15,20}}, color = {0,0,255}, smooth = Smooth.None));
          connect(terminalBox.plug_sp,smr.plug_sp) annotation(Line(points = {{ -4, -30},{ -4, -30}}, color = {0,0,255}, smooth = Smooth.None));
          connect(terminalBox.plug_sn,smr.plug_sn) annotation(Line(points = {{ -16, -30},{ -16, -30}}, color = {0,0,255}, smooth = Smooth.None));
          connect(smr.flange,rotorDisplacementAngle.flange) annotation(Line(points = {{0, -40},{10, -40}}, color = {0,0,0}, smooth = Smooth.None));
          connect(smr.flange,loadInertia.flange_a) annotation(Line(points = {{0, -40},{40, -40}}, color = {0,0,0}, smooth = Smooth.None));
        end SMR_Inverter;
        model SMPM_Inverter "Test example 6: PermanentMagnetSynchronousInductionMachine with inverter"
          extends Modelica.Icons.Example;
          constant Integer m = 3 "number of phases";
          parameter Modelica.SIunits.Voltage VNominal = 100 "nominal RMS voltage per phase";
          parameter Modelica.SIunits.Frequency fNominal = 50 "nominal frequency";
          parameter Modelica.SIunits.Frequency f = 50 "actual frequency";
          parameter Modelica.SIunits.Time tRamp = 1 "frequency ramp";
          parameter Modelica.SIunits.Torque TLoad = 181.4 "nominal load torque";
          parameter Modelica.SIunits.Time tStep = 1.2 "time of load torque step";
          parameter Modelica.SIunits.Inertia JLoad = 0.29 "load's moment of inertia";
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1.5, Interval = 0.001), Documentation(info = "<HTML>
<b>6th Test example: Permanent magnet synchronous induction machine fed by an ideal inverter</b><br>
An ideal frequency inverter is modeled by using a VfController and a threephase SignalVoltage.<br>
Frequency is raised by a ramp, causing the permanent magnet synchronous induction machine to start,
and accelerating inertias.<br>At time tStep a load step is applied.<br>
Simulate for 1.5 seconds and plot (versus time):
<ul>
<li>CurrentRMSSensor1.I: stator current RMS</li>
<li>PMSMD1.rpmMechanical: motor's speed</li>
<li>PMSMD1.tauElectrical: motor's torque</li>
<li>RotorDisplacementAngle.rotorDisplacementAngle: rotor displacement angle</li>
</ul>
Default machine parameters of model <i>SM_PermanentMagnet</i> are used.
<p>
<b>In practice it is nearly impossible to drive a PMSMD without current controller.</b>
</p>
</HTML>"));
          Machines.BasicMachines.SynchronousInductionMachines.SM_PermanentMagnet smpm annotation(Placement(transformation(extent = {{ -20, -50},{0, -30}}, rotation = 0)));
          Machines.Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensor annotation(Placement(transformation(origin = {0,30}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Machines.Sensors.RotorDisplacementAngle rotorDisplacementAngle(p = smpm.p) annotation(Placement(transformation(origin = {20, -40}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Modelica.Blocks.Sources.Ramp ramp(height = f, duration = tRamp) annotation(Placement(transformation(extent = {{ -80,50},{ -60,70}}, rotation = 0)));
          Machines.Utilities.VfController vfController(final m = m, VNominal = VNominal, fNominal = fNominal, BasePhase =  +Modelica.Constants.pi / 2) annotation(Placement(transformation(extent = {{ -40,50},{ -20,70}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Sources.SignalVoltage signalVoltage(final m = m) annotation(Placement(transformation(origin = {0,60}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(Placement(transformation(extent = {{ -50,80},{ -70,100}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(Placement(transformation(origin = { -90,90}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(Placement(transformation(extent = {{40, -50},{60, -30}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Sources.TorqueStep loadTorqueStep(startTime = tStep, stepTorque =  -TLoad, useSupport = false) annotation(Placement(transformation(extent = {{90, -50},{70, -30}}, rotation = 0)));
          Machines.Utilities.TerminalBox terminalBox annotation(Placement(transformation(extent = {{ -20, -30},{0, -10}}, rotation = 0)));

        equation 
          connect(signalVoltage.plug_n,star.plug_p) annotation(Line(points = {{1.83697e-15,70},{1.83697e-15,90},{ -50,90}}, color = {0,0,255}));
          connect(star.pin_n,ground.p) annotation(Line(points = {{ -70,90},{ -80,90}}, color = {0,0,255}));
          connect(ramp.y,vfController.u) annotation(Line(points = {{ -59,60},{ -42,60}}, color = {0,0,255}));
          connect(vfController.y,signalVoltage.v) annotation(Line(points = {{ -19,60},{7,60}}, color = {0,0,255}));
          connect(loadInertia.flange_b,loadTorqueStep.flange) annotation(Line(points = {{60, -40},{70, -40}}, color = {0,0,0}));
          connect(signalVoltage.plug_p,currentQuasiRMSSensor.plug_p) annotation(Line(points = {{ -1.83697e-15,50},{0,50},{0,40},{1.83697e-15,40}}, color = {0,0,255}));
          connect(rotorDisplacementAngle.plug_n,smpm.plug_sn) annotation(Line(points = {{26, -30},{26, -20},{ -16, -20},{ -16, -30}}, color = {0,0,255}));
          connect(rotorDisplacementAngle.plug_p,smpm.plug_sp) annotation(Line(points = {{14, -30},{ -4, -30}}, color = {0,0,255}));
          connect(terminalBox.plugSupply,currentQuasiRMSSensor.plug_n) annotation(Line(points = {{ -10, -28},{ -10,0},{ -1.83697e-15,0},{ -1.83697e-15,20}}, color = {0,0,255}, smooth = Smooth.None));
          connect(terminalBox.plug_sn,smpm.plug_sn) annotation(Line(points = {{ -16, -30},{ -16, -30}}, color = {0,0,255}, smooth = Smooth.None));
          connect(terminalBox.plug_sp,smpm.plug_sp) annotation(Line(points = {{ -4, -30},{ -4, -30}}, color = {0,0,255}, smooth = Smooth.None));
          connect(smpm.flange,rotorDisplacementAngle.flange) annotation(Line(points = {{0, -40},{10, -40}}, color = {0,0,0}, smooth = Smooth.None));
          connect(smpm.flange,loadInertia.flange_a) annotation(Line(points = {{0, -40},{40, -40}}, color = {0,0,0}, smooth = Smooth.None));
        end SMPM_Inverter;
        model SMEE_Generator "Test example 7: ElectricalExcitedSynchronousInductionMachine as Generator"
          extends Modelica.Icons.Example;
          constant Integer m = 3 "number of phases";
          parameter Modelica.SIunits.Voltage VNominal = 100 "nominal RMS voltage per phase";
          parameter Modelica.SIunits.Frequency fNominal = 50 "nominal frequency";
          parameter Modelica.SIunits.AngularVelocity wActual(displayUnit = "1/min") = (1499 * 2 * Modelica.Constants.pi) / 60 "actual speed";
          parameter Modelica.SIunits.Current Ie = 19 "excitation current";
          parameter Modelica.SIunits.Current Ie0 = 10 "initial excitation current";
          parameter Modelica.SIunits.Angle gamma0(displayUnit = "deg") = 0 "initial rotor displacement angle";
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 30, Interval = 0.005), Documentation(info = "<HTML>
<b>7th Test example: Electrical excited synchronous induction machine as generator</b><br>
An electrically excited synchronous generator is connected to the grid and driven with constant speed.
Since speed is slightly smaller than synchronous speed corresponding to mains frequency,
rotor angle is very slowly increased. This allows to see several charactersistics dependent on rotor angle.
Simulate for 30 seconds and plot (versus RotorDisplacementAngle1.rotorDisplacementAngle):
<ul>
<li>SMEED1.tauElectrical</li>
<li>CurrentRMSSensor1.I</li>
<li>ElectricalPowerSensor1.P</li>
<li>ElectricalPowerSensor1.Q</li>
<li>MechanicalPowerSensor1.P</li>
</ul>
Default machine parameters of model <i>SM_ElectricalExcited</i> are used.
</HTML>"));
          Machines.BasicMachines.SynchronousInductionMachines.SM_ElectricalExcited smee(useSupport = true, phiMechanical(start =  -(Modelica.Constants.pi + gamma0) / smee.p, fixed = true)) annotation(Placement(transformation(extent = {{ -20, -50},{0, -30}}, rotation = 0)));
          Machines.Sensors.RotorDisplacementAngle rotorDisplacementAngle(p = smee.p, useSupport = true) annotation(Placement(transformation(origin = {20, -40}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground groundExcitation annotation(Placement(transformation(origin = { -90, -50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Mechanics.Rotational.Sources.ConstantSpeed constantSpeed(final w_fixed = wActual) annotation(Placement(transformation(extent = {{90, -50},{70, -30}}, rotation = 0)));
          Sensors.MechanicalPowerSensor mechanicalPowerSensor(useSupport = true) annotation(Placement(transformation(extent = {{40, -50},{60, -30}}, rotation = 0)));
          Sensors.ElectricalPowerSensor electricalPowerSensor annotation(Placement(transformation(origin = {0,60}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Machines.Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensor annotation(Placement(transformation(origin = {0,30}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Sources.SineVoltage sineVoltage(final m = m, final V = fill(VNominal * sqrt(2), m), final freqHz = fill(fNominal, m)) annotation(Placement(transformation(extent = {{ -20,80},{ -40,100}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(Placement(transformation(extent = {{ -50,80},{ -70,100}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(Placement(transformation(origin = { -90,90}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Sources.RampCurrent rampCurrent(duration = 0.1, I = Ie - Ie0, offset = Ie0) annotation(Placement(transformation(origin = { -50, -40}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Machines.Utilities.TerminalBox terminalBox annotation(Placement(transformation(extent = {{ -20, -30},{0, -10}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Components.Fixed fixed annotation(Placement(transformation(extent = {{70, -70},{90, -50}})));

        equation 
          connect(rotorDisplacementAngle.plug_n,smee.plug_sn) annotation(Line(points = {{26, -30},{26, -20},{ -16, -20},{ -16, -30}}, color = {0,0,255}));
          connect(rotorDisplacementAngle.plug_p,smee.plug_sp) annotation(Line(points = {{14, -30},{ -4, -30}}, color = {0,0,255}));
          connect(star.pin_n,ground.p) annotation(Line(points = {{ -70,90},{ -80,90}}, color = {0,0,255}));
          connect(star.plug_p,sineVoltage.plug_n) annotation(Line(points = {{ -50,90},{ -40,90}}, color = {0,0,255}));
          connect(electricalPowerSensor.plug_ni,currentQuasiRMSSensor.plug_p) annotation(Line(points = {{ -1.83697e-15,50},{1.76911e-22,46},{1.83697e-15,46},{1.83697e-15,40}}, color = {0,0,255}));
          connect(mechanicalPowerSensor.flange_b,constantSpeed.flange) annotation(Line(points = {{60, -40},{70, -40}}, color = {0,0,0}));
          connect(sineVoltage.plug_p,electricalPowerSensor.plug_p) annotation(Line(points = {{ -20,90},{1.83697e-15,90},{1.83697e-15,70}}, color = {0,0,255}));
          connect(rampCurrent.p,groundExcitation.p) annotation(Line(points = {{ -50, -50},{ -80, -50}}, color = {0,0,255}));
          connect(rampCurrent.p,smee.pin_en) annotation(Line(points = {{ -50, -50},{ -40, -50},{ -40, -46},{ -20, -46}}, color = {0,0,255}));
          connect(rampCurrent.n,smee.pin_ep) annotation(Line(points = {{ -50, -30},{ -40, -30},{ -40, -34},{ -20, -34}}, color = {0,0,255}));
          connect(electricalPowerSensor.plug_nv,smee.plug_sn) annotation(Line(points = {{ -10,60},{ -16,60},{ -16, -30}}, color = {0,0,255}));
          connect(terminalBox.plugSupply,currentQuasiRMSSensor.plug_n) annotation(Line(points = {{ -10, -28},{ -10,0},{ -1.83697e-15,0},{ -1.83697e-15,20}}, color = {0,0,255}, smooth = Smooth.None));
          connect(terminalBox.plug_sn,smee.plug_sn) annotation(Line(points = {{ -16, -30},{ -16, -30}}, color = {0,0,255}, smooth = Smooth.None));
          connect(terminalBox.plug_sp,smee.plug_sp) annotation(Line(points = {{ -4, -30},{ -4, -30}}, color = {0,0,255}, smooth = Smooth.None));
          connect(constantSpeed.support,fixed.flange) annotation(Line(points = {{80, -50},{80, -60}}, color = {0,0,0}, smooth = Smooth.None));
          connect(mechanicalPowerSensor.support,fixed.flange) annotation(Line(points = {{50, -50},{50, -60},{80, -60}}, color = {0,0,0}, smooth = Smooth.None));
          connect(smee.support,fixed.flange) annotation(Line(points = {{0, -50},{0, -60},{80, -60}}, color = {0,0,0}, smooth = Smooth.None));
          connect(rotorDisplacementAngle.support,smee.support) annotation(Line(points = {{10, -50},{0, -50}}, color = {0,0,0}, smooth = Smooth.None));
          connect(smee.flange,rotorDisplacementAngle.flange) annotation(Line(points = {{0, -40},{10, -40}}, color = {0,0,0}, smooth = Smooth.None));
          connect(smee.flange,mechanicalPowerSensor.flange_a) annotation(Line(points = {{0, -40},{40, -40}}, color = {0,0,0}, smooth = Smooth.None));
        end SMEE_Generator;
        model DCPM_Start "Test example 8: DC with permanent magnet starting with voltage ramp"
          extends Modelica.Icons.Example;
          parameter Modelica.SIunits.Voltage Va = 100 "actual armature voltage";
          parameter Modelica.SIunits.Time tStart = 0.2 "armature voltage ramp";
          parameter Modelica.SIunits.Time tRamp = 0.8 "armature voltage ramp";
          parameter Modelica.SIunits.Torque TLoad = 63.66 "nominal load torque";
          parameter Modelica.SIunits.Time tStep = 1.5 "time of load torque step";
          parameter Modelica.SIunits.Inertia JLoad = 0.15 "load's moment of inertia";
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 2, Interval = 0.001), Documentation(info = "<HTML>
<b>8th Test example: Permanent magnet DC machine started with an armature voltage ramp</b><br>
A voltage ramp is applied to the armature, causing the DC machine to start,
and accelerating inertias.<br>At time tStep a load step is applied.<br>
Simulate for 2 seconds and plot (versus time):
<ul>
<li>DCPM1.ia: armature current</li>
<li>DCPM1.rpmMechanical: motor's speed</li>
<li>DCPM1.tauElectrical: motor's torque</li>
</ul>
Default machine parameters of model <i>DC_PermanentMagnet</i> are used.
</HTML>"));
          Machines.BasicMachines.DCMachines.DC_PermanentMagnet dcpm annotation(Placement(transformation(extent = {{ -20, -50},{0, -30}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp ramp(duration = tRamp, height = Va, startTime = tStart) annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SignalVoltage signalVoltage annotation(Placement(transformation(extent = {{0,30},{ -20,50}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(Placement(transformation(origin = { -70,40}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(Placement(transformation(extent = {{40, -50},{60, -30}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Sources.TorqueStep loadTorqueStep(startTime = tStep, stepTorque =  -TLoad, useSupport = false) annotation(Placement(transformation(extent = {{90, -50},{70, -30}}, rotation = 0)));

        equation 
          connect(ramp.y,signalVoltage.v) annotation(Line(points = {{ -59,70},{ -10,70},{ -10,47}}, color = {0,0,255}));
          connect(signalVoltage.p,dcpm.pin_ap) annotation(Line(points = {{0,40},{0, -20},{ -4, -20},{ -4, -30}}, color = {0,0,255}));
          connect(signalVoltage.n,ground.p) annotation(Line(points = {{ -20,40},{ -60,40}}, color = {0,0,255}));
          connect(dcpm.pin_an,signalVoltage.n) annotation(Line(points = {{ -16, -30},{ -16, -20},{ -20, -20},{ -20,40}}, color = {0,0,255}));
          connect(loadInertia.flange_b,loadTorqueStep.flange) annotation(Line(points = {{60, -40},{70, -40}}, color = {0,0,0}));
          connect(dcpm.flange,loadInertia.flange_a) annotation(Line(points = {{0, -40},{40, -40}}, color = {0,0,0}, smooth = Smooth.None));
        end DCPM_Start;
        model DCEE_Start "Test example 9: DC with electrical ecxitation starting with voltage ramp"
          extends Modelica.Icons.Example;
          parameter Modelica.SIunits.Voltage Va = 100 "actual armature voltage";
          parameter Modelica.SIunits.Time tStart = 0.2 "armature voltage ramp";
          parameter Modelica.SIunits.Time tRamp = 0.8 "armature voltage ramp";
          parameter Modelica.SIunits.Voltage Ve = 100 "actual excitation voltage";
          parameter Modelica.SIunits.Torque TLoad = 63.66 "nominal load torque";
          parameter Modelica.SIunits.Time tStep = 1.5 "time of load torque step";
          parameter Modelica.SIunits.Inertia JLoad = 0.15 "load's moment of inertia";
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 2, Interval = 0.001), Documentation(info = "<HTML>
<b>9th Test example: Electrically separate excited DC machine started with an armature voltage ramp</b><br>
A voltage ramp is applied to the armature, causing the DC machine to start,
and accelerating inertias.<br>At time tStep a load step is applied.<br>
Simulate for 2 seconds and plot (versus time):
<ul>
<li>DCEE1.ia: armature current</li>
<li>DCEE1.rpmMechanical: motor's speed</li>
<li>DCEE1.tauElectrical: motor's torque</li>
<li>DCEE1.ie: excitation current</li>
</ul>
Default machine parameters of model <i>DC_ElectricalExcited</i> are used.
</HTML>"));
          Machines.BasicMachines.DCMachines.DC_ElectricalExcited dcee annotation(Placement(transformation(extent = {{ -20, -50},{0, -30}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp ramp(duration = tRamp, height = Va, startTime = tStart) annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SignalVoltage signalVoltage annotation(Placement(transformation(extent = {{0,30},{ -20,50}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(Placement(transformation(origin = { -70,40}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Sources.ConstantVoltage constantVoltage(V = Ve) annotation(Placement(transformation(origin = { -40, -40}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground groundExcitation annotation(Placement(transformation(origin = { -70, -50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(Placement(transformation(extent = {{40, -50},{60, -30}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Sources.TorqueStep loadTorqueStep(startTime = tStep, stepTorque =  -TLoad, useSupport = false) annotation(Placement(transformation(extent = {{90, -50},{70, -30}}, rotation = 0)));

        equation 
          connect(ramp.y,signalVoltage.v) annotation(Line(points = {{ -59,70},{ -10,70},{ -10,47}}, color = {0,0,255}));
          connect(signalVoltage.p,dcee.pin_ap) annotation(Line(points = {{0,40},{0, -20},{ -4, -20},{ -4, -30}}, color = {0,0,255}));
          connect(signalVoltage.n,ground.p) annotation(Line(points = {{ -20,40},{ -60,40}}, color = {0,0,255}));
          connect(dcee.pin_an,ground.p) annotation(Line(points = {{ -16, -30},{ -16, -20},{ -20, -20},{ -20,40},{ -60,40}}, color = {0,0,255}));
          connect(constantVoltage.n,groundExcitation.p) annotation(Line(points = {{ -40, -50},{ -60, -50}}, color = {0,0,255}));
          connect(dcee.pin_ep,constantVoltage.p) annotation(Line(points = {{ -20, -34},{ -30, -34},{ -30, -30},{ -40, -30}}, color = {0,0,255}));
          connect(dcee.pin_en,constantVoltage.n) annotation(Line(points = {{ -20, -46},{ -30, -46},{ -30, -50},{ -40, -50}}, color = {0,0,255}));
          connect(loadInertia.flange_b,loadTorqueStep.flange) annotation(Line(points = {{60, -40},{70, -40}}, color = {0,0,0}));
          connect(dcee.flange,loadInertia.flange_a) annotation(Line(points = {{0, -40},{40, -40}}, color = {0,0,0}, smooth = Smooth.None));
        end DCEE_Start;
        model DCSE_Start "Test example 10: DC with serial excitation starting with voltage ramp"
          extends Modelica.Icons.Example;
          parameter Modelica.SIunits.Voltage Va = 100 "actual armature voltage";
          parameter Modelica.SIunits.Time tStart = 0.2 "armature voltage ramp";
          parameter Modelica.SIunits.Time tRamp = 0.8 "armature voltage ramp";
          parameter Modelica.SIunits.Torque TLoad = 63.66 "nominal load torque";
          parameter Modelica.SIunits.AngularVelocity wLoad(displayUnit = "1/min") = (1410 * 2 * Modelica.Constants.pi) / 60 "nominal load speed";
          parameter Modelica.SIunits.Inertia JLoad = 0.15 "load's moment of inertia";
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 2, Interval = 0.001), Documentation(info = "<HTML>
<b>10th Test example: Series excited DC machine started with an armature voltage ramp</b><br>
A voltage ramp is applied to the armature, causing the DC machine to start,
and accelerating inertiasagainst load torque quadratic dependent on speed, finally reaching nominal speed.<br>
Simulate for 2 seconds and plot (versus time):
<ul>
<li>DCSE1.ia: armature current</li>
<li>DCSE1.rpmMechanical: motor's speed</li>
<li>DCSE1.tauElectrical: motor's torque</li>
</ul>
Default machine parameters of model <i>DC_SeriesExcited</i> are used.
</HTML>"));
          Machines.BasicMachines.DCMachines.DC_SeriesExcited dcse annotation(Placement(transformation(extent = {{ -20, -50},{0, -30}}, rotation = 0)));
          Modelica.Blocks.Sources.Ramp ramp(duration = tRamp, height = Va, startTime = tStart) annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SignalVoltage signalVoltage annotation(Placement(transformation(extent = {{0,30},{ -20,50}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(Placement(transformation(origin = { -70,40}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(Placement(transformation(extent = {{40, -50},{60, -30}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticLoadTorque(w_nominal = wLoad, TorqueDirection = false, tau_nominal =  -TLoad, useSupport = false) annotation(Placement(transformation(extent = {{90, -50},{70, -30}}, rotation = 0)));

        equation 
          connect(ramp.y,signalVoltage.v) annotation(Line(points = {{ -59,70},{ -10,70},{ -10,47}}, color = {0,0,255}));
          connect(signalVoltage.p,dcse.pin_ap) annotation(Line(points = {{0,40},{0, -20},{ -4, -20},{ -4, -30}}, color = {0,0,255}));
          connect(signalVoltage.n,ground.p) annotation(Line(points = {{ -20,40},{ -60,40}}, color = {0,0,255}));
          connect(loadInertia.flange_b,quadraticLoadTorque.flange) annotation(Line(points = {{60, -40},{70, -40}}, color = {0,0,0}));
          connect(dcse.pin_an,dcse.pin_ep) annotation(Line(points = {{ -16, -30},{ -20, -30},{ -20, -34}}, color = {0,0,255}));
          connect(dcse.pin_en,signalVoltage.n) annotation(Line(points = {{ -20, -46},{ -30, -46},{ -30, -20},{ -20, -20},{ -20,40}}, color = {0,0,255}));
          connect(dcse.flange,loadInertia.flange_a) annotation(Line(points = {{0, -40},{40, -40}}, color = {0,0,0}, smooth = Smooth.None));
        end DCSE_Start;
        model TransformerTestbench "Transformer Testbench"
          extends Modelica.Icons.Example;
          parameter Modelica.SIunits.Resistance RL[3] = fill(1 / 3, 3) "Load resistance";
          annotation(Documentation(info = "<HTML>
Transformer testbench:<br>
You may choose different connections as well as vary the load (even not symmetrical).<br>
<b>Please pay attention</b> to proper grounding of the primary and secondary part of the whole circuit.<br>
The primary and secondary starpoint are available as connectors, if the connection is not delta (D or d).<br>
In some cases it may be necessary to ground the transformer's starpoint
even though the source's or load's starpoint are grounded; you may use a reasonable high earthing resistance.
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 0.1));
          Modelica.Electrical.MultiPhase.Sources.SineVoltage source(freqHz = fill(50, 3), V = fill(sqrt(2 / 3) * 100, 3)) annotation(Placement(transformation(origin = { -90, -10}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Basic.Star starS annotation(Placement(transformation(origin = { -90, -40}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground groundS annotation(Placement(transformation(extent = {{ -100, -80},{ -80, -60}}, rotation = 0)));
          Sensors.ElectricalPowerSensor electricalPowerSensorS annotation(Placement(transformation(extent = {{ -90,0},{ -70,20}}, rotation = 0)));
          Machines.Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensorS annotation(Placement(transformation(extent = {{ -60,20},{ -40,0}}, rotation = 0)));
          Machines.Sensors.VoltageQuasiRMSSensor voltageQuasiRMSSensorS annotation(Placement(transformation(origin = { -40, -10}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Basic.Delta deltaS annotation(Placement(transformation(origin = { -40, -30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Resistor earth(R = 1000000.0) annotation(Placement(transformation(origin = {0, -40}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground groundT annotation(Placement(transformation(extent = {{ -10, -80},{10, -60}}, rotation = 0)));
          Machines.Sensors.VoltageQuasiRMSSensor voltageRMSSensorL annotation(Placement(transformation(origin = {40, -10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Basic.Delta deltaL annotation(Placement(transformation(origin = {40, -30}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Machines.Sensors.CurrentQuasiRMSSensor currentQuasiRMSSensorL annotation(Placement(transformation(extent = {{40,20},{60,0}}, rotation = 0)));
          Sensors.ElectricalPowerSensor electricalPowerSensorL annotation(Placement(transformation(extent = {{70,0},{90,20}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Basic.Resistor load(R = RL) annotation(Placement(transformation(origin = {90, -10}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Basic.Star starL annotation(Placement(transformation(origin = {90, -40}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground groundL annotation(Placement(transformation(extent = {{80, -80},{100, -60}}, rotation = 0)));
          Machines.Utilities.TransformerData transformerData(C1 = Modelica.Utilities.Strings.substring(transformer.VectorGroup, 1, 1), C2 = Modelica.Utilities.Strings.substring(transformer.VectorGroup, 2, 2)) annotation(Placement(transformation(extent = {{ -10,40},{10,60}}, rotation = 0)));
          BasicMachines.Transformers.Yd.Yd01 transformer(n = transformerData.n, R1 = transformerData.R1, L1sigma = transformerData.L1sigma, R2 = transformerData.R2, L2sigma = transformerData.L2sigma) annotation(Placement(transformation(extent = {{ -20, -10},{20,30}}, rotation = 0)));

        equation 
          connect(starS.pin_n,groundS.p) annotation(Line(points = {{ -90, -50},{ -90, -60}}, color = {0,0,255}));
          connect(source.plug_n,starS.plug_p) annotation(Line(points = {{ -90, -20},{ -90, -30}}, color = {0,0,255}));
          connect(starL.pin_n,groundL.p) annotation(Line(points = {{90, -50},{90, -60}}, color = {0,0,255}));
          connect(load.plug_n,starL.plug_p) annotation(Line(points = {{90, -20},{90, -30}}, color = {0,0,255}));
          connect(earth.n,groundT.p) annotation(Line(points = {{ -1.83697e-15, -50},{0, -50},{0, -60}}, color = {0,0,255}));
          connect(electricalPowerSensorS.plug_nv,starS.plug_p) annotation(Line(points = {{ -80,0},{ -80, -30},{ -90, -30}}, color = {0,0,255}));
          connect(source.plug_p,electricalPowerSensorS.plug_p) annotation(Line(points = {{ -90, -1.77636e-15},{ -90,10}}, color = {0,0,255}));
          connect(electricalPowerSensorS.plug_ni,currentQuasiRMSSensorS.plug_p) annotation(Line(points = {{ -70,10},{ -60,10}}, color = {0,0,255}));
          connect(currentQuasiRMSSensorL.plug_n,electricalPowerSensorL.plug_p) annotation(Line(points = {{60,10},{70,10}}, color = {0,0,255}));
          connect(electricalPowerSensorL.plug_ni,load.plug_p) annotation(Line(points = {{90,10},{90, -1.77636e-15}}, color = {0,0,255}));
          connect(currentQuasiRMSSensorS.plug_n,voltageQuasiRMSSensorS.plug_p) annotation(Line(points = {{ -40,10},{ -40, -1.77636e-15}}, color = {0,0,255}));
          connect(currentQuasiRMSSensorL.plug_p,voltageRMSSensorL.plug_p) annotation(Line(points = {{40,10},{40,0}}, color = {0,0,255}));
          connect(electricalPowerSensorL.plug_nv,starL.plug_p) annotation(Line(points = {{80,0},{80, -30},{90, -30}}, color = {0,0,255}));
          connect(voltageQuasiRMSSensorS.plug_n,deltaS.plug_p) annotation(Line(points = {{ -40, -20},{ -40, -20}}, color = {0,0,255}));
          connect(deltaL.plug_p,voltageRMSSensorL.plug_n) annotation(Line(points = {{40, -20},{40, -20}}, color = {0,0,255}));
          connect(deltaS.plug_n,voltageQuasiRMSSensorS.plug_p) annotation(Line(points = {{ -40, -40},{ -50, -40},{ -50, -1.77636e-15},{ -40, -1.77636e-15}}, color = {0,0,255}));
          connect(deltaL.plug_n,voltageRMSSensorL.plug_p) annotation(Line(points = {{40, -40},{50, -40},{50,0},{40,0}}, color = {0,0,255}));
          connect(currentQuasiRMSSensorS.plug_n,transformer.plug1) annotation(Line(points = {{ -40,10},{ -20,10}}, color = {0,0,255}, smooth = Smooth.None));
          connect(transformer.plug2,currentQuasiRMSSensorL.plug_p) annotation(Line(points = {{20,10},{40,10}}, color = {0,0,255}, smooth = Smooth.None));
        end TransformerTestbench;
        model Rectifier6pulse "6-pulse rectifier with 1 transformer"
          extends Modelica.Icons.Example;
          constant Integer m = 3 "Number of phases";
          parameter Modelica.SIunits.Voltage V = 100 * sqrt(2 / 3) "Amplitude of star-voltage";
          parameter Modelica.SIunits.Frequency f = 50 "Frequency";
          parameter Modelica.SIunits.Resistance RL = 0.4 "Load resistance";
          parameter Modelica.SIunits.Capacitance C = 0.005 "Total DC-capacitance";
          parameter Modelica.SIunits.Voltage VC0 = sqrt(3) * V "Initial voltage of capacitance";
          annotation(Documentation(info = "<HTML>
Test example with multiphase components:<br>
Star-connected voltage source feeds via a transformer a diode bridge rectifier with a DC burden.<br>
Using f=50 Hz, simulate for 0.1 seconds (5 periods) and compare voltages and currents of source and DC burden,
neglecting initial transient.
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 0.1));
          Modelica.Electrical.MultiPhase.Sources.SineVoltage source(m = m, V = fill(V, m), freqHz = fill(f, m)) annotation(Placement(transformation(origin = { -90, -10}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Basic.Star starAC(m = m) annotation(Placement(transformation(origin = { -90, -40}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground groundAC annotation(Placement(transformation(extent = {{ -100, -80},{ -80, -60}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Sensors.CurrentSensor currentQuasiRMSSensor annotation(Placement(transformation(extent = {{ -80, -10},{ -60,10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Ideal.IdealDiode diode1(m = m) annotation(Placement(transformation(origin = { -20,60}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
          Modelica.Electrical.MultiPhase.Basic.Star star1(m = m) annotation(Placement(transformation(origin = {0,70}, extent = {{10,10},{ -10, -10}}, rotation = 180)));
          Modelica.Electrical.MultiPhase.Ideal.IdealDiode diode2(m = m) annotation(Placement(transformation(origin = { -20,20}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
          Modelica.Electrical.MultiPhase.Basic.Star star2(m = m) annotation(Placement(transformation(extent = {{ -10,0},{10,20}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor load(R = RL) annotation(Placement(transformation(origin = {50,0}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Capacitor cDC1(C = 2 * C, v(start = VC0 / 2)) annotation(Placement(transformation(origin = {70,20}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Capacitor cDC2(C = 2 * C, v(start = VC0 / 2)) annotation(Placement(transformation(origin = {70, -20}, extent = {{ -10,10},{10, -10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Ground groundDC annotation(Placement(transformation(extent = {{80, -20},{100,0}}, rotation = 0)));
          Machines.Utilities.TransformerData transformerData1(C1 = Modelica.Utilities.Strings.substring(transformer1.VectorGroup, 1, 1), C2 = Modelica.Utilities.Strings.substring(transformer1.VectorGroup, 2, 2)) annotation(Placement(transformation(extent = {{ -60,60},{ -40,80}}, rotation = 0)));
          BasicMachines.Transformers.Dy.Dy01 transformer1(n = transformerData1.n, R1 = transformerData1.R1, L1sigma = transformerData1.L1sigma, R2 = transformerData1.R2, L2sigma = transformerData1.L2sigma) annotation(Placement(transformation(extent = {{ -50,30},{ -30,50}}, rotation = 0)));

        equation 
          connect(cDC1.n,cDC2.p) annotation(Line(points = {{70,10},{70, -10}}, color = {0,0,255}));
          connect(cDC1.n,groundDC.p) annotation(Line(points = {{70,10},{70,0},{90,0}}, color = {0,0,255}));
          connect(starAC.plug_p,source.plug_n) annotation(Line(points = {{ -90, -30},{ -90, -20}}, color = {0,0,255}));
          connect(diode1.plug_n,star1.plug_p) annotation(Line(points = {{ -20,70},{ -10,70}}, color = {0,0,255}));
          connect(diode2.plug_p,star2.plug_p) annotation(Line(points = {{ -20,10},{ -10,10}}, color = {0,0,255}));
          connect(diode2.plug_n,diode1.plug_p) annotation(Line(points = {{ -20,30},{ -20,50}}, color = {0,0,255}));
          connect(starAC.pin_n,groundAC.p) annotation(Line(points = {{ -90, -50},{ -90, -60}}, color = {0,0,255}));
          connect(source.plug_p,currentQuasiRMSSensor.plug_p) annotation(Line(points = {{ -90, -1.77636e-15},{ -85, -1.77636e-15},{ -85,0},{ -80,0}}, color = {0,0,255}));
          connect(load.p,cDC1.p) annotation(Line(points = {{50,10},{50,30},{70,30}}, color = {0,0,255}));
          connect(load.n,cDC2.n) annotation(Line(points = {{50, -10},{50, -30},{70, -30}}, color = {0,0,255}));
          connect(star1.pin_n,cDC1.p) annotation(Line(points = {{10,70},{70,70},{70,30}}, color = {0,0,255}));
          connect(star2.pin_n,cDC2.n) annotation(Line(points = {{10,10},{20,10},{20, -70},{70, -70},{70, -30}}, color = {0,0,255}));
          connect(transformer1.plug1,currentQuasiRMSSensor.plug_n) annotation(Line(points = {{ -50,40},{ -60,40},{ -60,0}}, color = {0,0,255}, smooth = Smooth.None));
          connect(transformer1.plug2,diode1.plug_p) annotation(Line(points = {{ -30,40},{ -20,40},{ -20,50}}, color = {0,0,255}, smooth = Smooth.None));
        end Rectifier6pulse;
        model Rectifier12pulse "12-pulse rectifier with 2 transformers"
          extends Modelica.Icons.Example;
          extends Machines.Examples.Rectifier6pulse(RL = 0.2);
          annotation(Documentation(info = "<HTML>
Test example with multiphase components:<br>
Star-connected voltage source feeds via two transformers (Dd0 and Dy1) two diode bridge rectifiers with a single DC burden.<br>
Using f=50 Hz, simulate for 0.1 seconds (5 periods) and compare voltages and currents of source and DC burden,
neglecting initial transient.
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 0.1));
          Modelica.Electrical.MultiPhase.Ideal.IdealDiode diode3(m = m) annotation(Placement(transformation(origin = { -20, -20}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
          Modelica.Electrical.MultiPhase.Basic.Star star3(m = m) annotation(Placement(transformation(extent = {{ -10,0},{10, -20}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Ideal.IdealDiode diode4(m = m) annotation(Placement(transformation(origin = { -20, -60}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
          Modelica.Electrical.MultiPhase.Basic.Star star4(m = m) annotation(Placement(transformation(extent = {{ -10, -60},{10, -80}}, rotation = 0)));
          BasicMachines.Transformers.Dd.Dd00 transformer2(n = transformerData2.n, R1 = transformerData2.R1, L1sigma = transformerData2.L1sigma, R2 = transformerData2.R2, L2sigma = transformerData2.L2sigma) annotation(Placement(transformation(extent = {{ -50, -50},{ -30, -30}}, rotation = 0)));
          Machines.Utilities.TransformerData transformerData2(C1 = Modelica.Utilities.Strings.substring(transformer2.VectorGroup, 1, 1), C2 = Modelica.Utilities.Strings.substring(transformer2.VectorGroup, 2, 2)) annotation(Placement(transformation(extent = {{ -60, -80},{ -40, -60}}, rotation = 0)));

        equation 
          connect(diode3.plug_n,star3.plug_p) annotation(Line(points = {{ -20, -10},{ -10, -10}}, color = {0,0,255}));
          connect(diode4.plug_p,star4.plug_p) annotation(Line(points = {{ -20, -70},{ -10, -70}}, color = {0,0,255}));
          connect(diode4.plug_n,diode3.plug_p) annotation(Line(points = {{ -20, -50},{ -20, -30}}, color = {0,0,255}));
          connect(star4.pin_n,cDC2.n) annotation(Line(points = {{10, -70},{70, -70},{70, -30}}, color = {0,0,255}));
          connect(star3.pin_n,cDC1.p) annotation(Line(points = {{10, -10},{18, -10},{18, -8},{22, -8},{22, -10},{30, -10},{30,70},{70,70},{70,30}}, color = {0,0,255}));
          connect(transformer2.plug2,diode4.plug_n) annotation(Line(points = {{ -30, -40},{ -20, -40},{ -20, -50}}, color = {0,0,255}, smooth = Smooth.None));
          connect(transformer2.plug1,currentQuasiRMSSensor.plug_n) annotation(Line(points = {{ -50, -40},{ -60, -40},{ -60,0}}, color = {0,0,255}, smooth = Smooth.None));
        end Rectifier12pulse;
        model AIMC_Steinmetz "AsynchronousInductionMachineSquirrelCage Steinmetz-connection"
          extends Modelica.Icons.Example;
          constant Integer m = 3 "number of phases";
          parameter Modelica.SIunits.Voltage VNominal = 100 "nominal RMS voltage per phase";
          parameter Modelica.SIunits.Frequency fNominal = 50 "nominal frequency";
          parameter Modelica.SIunits.Time tStart1 = 0.1 "start time";
          parameter Modelica.SIunits.Capacitance Cr = 0.0035 "motor's running capacitor";
          parameter Modelica.SIunits.Capacitance Cs = 5 * Cr "motor's (additional) starting capacitor";
          parameter Modelica.SIunits.AngularVelocity wSwitch(displayUnit = "1/min") = (1350 * 2 * Modelica.Constants.pi) / 60 "speed for switching off the starting capacitor";
          parameter Modelica.SIunits.Torque TLoad = 2 / 3 * 161.4 "nominal load torque";
          parameter Modelica.SIunits.AngularVelocity wLoad(displayUnit = "1/min") = (1462.5 * 2 * Modelica.Constants.pi) / 60 "nominal load speed";
          parameter Modelica.SIunits.Inertia JLoad = 0.29 "load's moment of inertia";
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 1.5, Interval = 0.001), Documentation(info = "<HTML>
<b>Asynchronous induction machine with squirrel cage - Steinmetz-connection</b><br>
At start time tStart single phase voltage is supplied to the asynchronous induction machine with squirrel cage;
the machine starts from standstill, accelerating inertias against load torque quadratic dependent on speed, finally reaching nominal speed.<br>
Default machine parameters of model <i>AIM_SquirrelCage</i> are used.
</HTML>"));
          Machines.BasicMachines.AsynchronousInductionMachines.AIM_SquirrelCage aimc(useSupport = true) annotation(Placement(transformation(extent = {{ -20, -50},{0, -30}}, rotation = 0)));
          Modelica.Electrical.Analog.Sources.SineVoltage sineVoltage(freqHz = fNominal, V = sqrt(2) * VNominal) annotation(Placement(transformation(extent = {{ -50,100},{ -70,80}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Ground ground annotation(Placement(transformation(origin = { -90,90}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Blocks.Sources.BooleanStep booleanStep(startTime = tStart1) annotation(Placement(transformation(extent = {{ -50,60},{ -30,80}}, rotation = 0)));
          Modelica.Electrical.Analog.Ideal.IdealClosingSwitch idealCloser annotation(Placement(transformation(origin = { -20,90}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
          Modelica.Mechanics.Rotational.Components.Inertia loadInertia(J = JLoad) annotation(Placement(transformation(extent = {{40, -50},{60, -30}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Sources.QuadraticSpeedDependentTorque quadraticLoadTorque(w_nominal = wLoad, TorqueDirection = false, tau_nominal =  -TLoad) annotation(Placement(transformation(extent = {{90, -50},{70, -30}}, rotation = 0)));
          Machines.Utilities.TerminalBox TerminalBox1(terminalConnection = "D") annotation(Placement(transformation(extent = {{ -20, -30},{0, -10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Basic.PlugToPin_p plugToPin_p3(m = m, k = 3) annotation(Placement(transformation(origin = { -30, -10}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Electrical.MultiPhase.Basic.PlugToPin_p plugToPin_p2(m = m, k = 2) annotation(Placement(transformation(origin = { -10, -10}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Electrical.MultiPhase.Basic.PlugToPin_p plugToPin_p1(m = m, k = 1) annotation(Placement(transformation(origin = {10, -10}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Electrical.Analog.Basic.Capacitor cRun(C = Cr) annotation(Placement(transformation(origin = {10,20}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Basic.Capacitor cStart(C = Cs) annotation(Placement(transformation(origin = {30,20}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.Analog.Ideal.IdealOpeningSwitch idealOpener annotation(Placement(transformation(origin = {30,50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Blocks.Logical.GreaterThreshold greaterThreshold(threshold = wSwitch) annotation(Placement(transformation(origin = {50,20}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Mechanics.Rotational.Components.Fixed fixed annotation(Placement(transformation(extent = {{70, -70},{90, -50}})));
          Modelica.Mechanics.Rotational.Sensors.RelSpeedSensor relSpeedSensor annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 90, origin = {20, -50})));

        equation 
          connect(ground.p,sineVoltage.n) annotation(Line(points = {{ -80,90},{ -70,90}}, color = {0,0,255}));
          connect(sineVoltage.p,idealCloser.p) annotation(Line(points = {{ -50,90},{ -30,90}}, color = {0,0,255}));
          connect(booleanStep.y,idealCloser.control) annotation(Line(points = {{ -29,70},{ -20,70},{ -20,83}}, color = {255,0,255}));
          connect(plugToPin_p3.pin_p,sineVoltage.n) annotation(Line(points = {{ -30, -8},{ -70, -8},{ -70,90}}, color = {0,0,255}));
          connect(idealCloser.n,plugToPin_p2.pin_p) annotation(Line(points = {{ -10,90},{ -10, -8}}, color = {0,0,255}));
          connect(cRun.n,plugToPin_p1.pin_p) annotation(Line(points = {{10,10},{10, -8}}, color = {0,0,255}));
          connect(loadInertia.flange_b,quadraticLoadTorque.flange) annotation(Line(points = {{60, -40},{70, -40}}, color = {0,0,0}));
          connect(cRun.p,idealCloser.n) annotation(Line(points = {{10,30},{10,90},{ -10,90}}, color = {0,0,255}));
          connect(plugToPin_p1.pin_p,cStart.n) annotation(Line(points = {{10, -8},{30, -8},{30,10}}, color = {0,0,255}));
          connect(idealOpener.n,cStart.p) annotation(Line(points = {{30,40},{30,37.5},{30,37.5},{30,35},{30,30},{30,30}}, color = {0,0,255}));
          connect(idealOpener.p,idealCloser.n) annotation(Line(points = {{30,60},{30,90},{ -10,90}}, color = {0,0,255}));
          connect(greaterThreshold.y,idealOpener.control) annotation(Line(points = {{50,31},{50,50},{37,50}}, color = {255,0,255}));
          connect(TerminalBox1.plug_sn,aimc.plug_sn) annotation(Line(points = {{ -16, -30},{ -16, -30}}, color = {0,0,255}, smooth = Smooth.None));
          connect(TerminalBox1.plug_sp,aimc.plug_sp) annotation(Line(points = {{ -4, -30},{ -4, -30}}, color = {0,0,255}, smooth = Smooth.None));
          connect(TerminalBox1.plugSupply,plugToPin_p2.plug_p) annotation(Line(points = {{ -10, -28},{ -10, -12}}, color = {0,0,255}, smooth = Smooth.None));
          connect(TerminalBox1.plugSupply,plugToPin_p3.plug_p) annotation(Line(points = {{ -10, -28},{ -10, -20},{ -30, -20},{ -30, -12}}, color = {0,0,255}, smooth = Smooth.None));
          connect(TerminalBox1.plugSupply,plugToPin_p1.plug_p) annotation(Line(points = {{ -10, -28},{ -10, -20},{10, -20},{10, -12}}, color = {0,0,255}, smooth = Smooth.None));
          connect(quadraticLoadTorque.support,fixed.flange) annotation(Line(points = {{80, -50},{80, -60}}, color = {0,0,0}, smooth = Smooth.None));
          connect(relSpeedSensor.flange_a,fixed.flange) annotation(Line(points = {{20, -60},{80, -60}}, color = {0,0,0}, smooth = Smooth.None));
          connect(relSpeedSensor.w_rel,greaterThreshold.u) annotation(Line(points = {{31, -50},{30, -50},{30, -20},{50, -20},{50,8}}, color = {0,0,127}, smooth = Smooth.None));
          connect(aimc.support,fixed.flange) annotation(Line(points = {{0, -50},{0, -60},{80, -60}}, color = {0,0,0}, smooth = Smooth.None));
          connect(aimc.flange,relSpeedSensor.flange_b) annotation(Line(points = {{0, -40},{20, -40}}, color = {0,0,0}, smooth = Smooth.None));
          connect(aimc.flange,loadInertia.flange_a) annotation(Line(points = {{0, -40},{40, -40}}, color = {0,0,0}, smooth = Smooth.None));
        end AIMC_Steinmetz;
      end Examples;
      package BasicMachines "Basic machine models"
        extends Modelica.Icons.Library2;
        annotation(Documentation(info = "<HTML>
This package contains components for modeling electrical machines, specially threephase induction machines, based on space phasor theory:
<ul>
<li>package AsynchronousInductionMachines: models of three phase asynchronous induction machines</li>
<li>package SynchronousInductionMachines: models of three phase synchronous induction machines</li>
<li>package DCMachines: models of DC machines with different excitation</li>
<li>package Transformers: Threephase transformers (see detailled documentation in subpackage)</li>
<li>package Components: components for modeling machines and transformers</li>
</ul>
The induction machine models use package SpacePhasors.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2009, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.00  2004/09/16 Anton Haumer</li>
  <li> v1.01  2004/09/18 Anton Haumer<br>
       moved common equations from machine models to PartialMachine</li>
  <li> v1.02  2004/09/19 Anton Haumer<br>
       new package structure for machine types<br>
       added DCMachine models</li>
  <li> v1.03  2004/09/24 Anton Haumer<br>
       added DC machine with series excitation</li>
  <li> v1.1   2004/10/01 Anton Haumer<br>
       changed naming and structure<br>
       issued to Modelica Standard Library 2.1</li>
  <li> v1.2   2004/10/27 Anton Haumer<br>
       fixed a bug with support (formerly bearing)</li>
  <li> v1.3.2 2004/11/10 Anton Haumer<br>
       ReluctanceRotor moved to SynchronousMachines</li>
  <li> v1.4   2004/11/11 Anton Haumer<br>
       removed mechanical flange support<br>
       to ease the implementation of a 3D-frame in a future release</li>
  <li> v1.53  2005/10/14 Anton Haumer<br>
       introduced unsymmetrical DamperCage for Synchronous Machines</li>
  <li> v1.6.2 2005/10/23 Anton Haumer<br>
       selectable DamperCage for Synchronous Machines</li>
  <li> v1.6.3 2005/11/25 Anton Haumer<br>
       easier parametrisation of AsynchronousInductionMachines.AIM_SlipRing model</li>
  <li> v1.7.1 2006/02/06 Anton Haumer<br>
       changed some naming of synchronous machines, not affecting existing models</li>
  </ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,128,255}),Rectangle(extent = {{ -60,60},{ -80, -60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {128,128,128}),Rectangle(extent = {{60,10},{80, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {95,95,95}),Rectangle(extent = {{ -60,70},{20,50}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{ -70, -90},{ -60, -90},{ -30, -20},{20, -20},{50, -90},{60, -90},{60, -100},{ -70, -100},{ -70, -90}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}));
        package AsynchronousInductionMachines "Models of asynchronous induction machines"
          extends Modelica.Icons.Library;
          annotation(Documentation(info = "<HTML>
This package contains models of asynchronous induction machines, based on space phasor theory:
<ul>
<li>AIM_SquirrelCage: asynchronous induction machine with squirrel cage</li>
<li>AIM_SlipRing: asynchronous induction machine with wound rotor</li>
</ul>
These models use package SpacePhasors.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2009, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.02 2004/09/19 Anton Haumer</li>
  <li> v1.03 2004/09/24 Anton Haumer<br>
       consistent naming of inductors and resistors in machine models</li>
  <li> v1.1  2004/10/01 Anton Haumer<br>
       changed naming and structure<br>
       issued to Modelica Standard Library 2.1</li>
  <li> v1.2  2004/10/27 Anton Haumer<br>
       fixed a bug with support (formerly bearing)</li>
  <li> v1.3.2 2004/11/10 Anton Haumer<br>
       ReluctanceRotor moved to SynchronousMachines</li>
  <li> v1.4   2004/11/11 Anton Haumer<br>
       removed mechanical flange support<br>
       to ease the implementation of a 3D-frame in a future release</li>
  <li> v1.6.3 2005/11/25 Anton Haumer<br>
       easier parametrisation of SlipRing model</li>
  </ul>
</HTML>"));
          model AIM_SquirrelCage "Asynchronous induction machine with squirrel cage rotor"
            extends Machines.Interfaces.PartialBasicInductionMachine(final idq_ss = airGapS.i_ss, final idq_sr = airGapS.i_sr, final idq_rs = airGapS.i_rs, final idq_rr = airGapS.i_rr);
            Components.AirGapS airGapS(final p = p, final m = 3, final Lm = Lm) annotation(Placement(transformation(origin = {0,0}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            parameter Modelica.SIunits.Inductance Lm(start = (3 * sqrt(1 - 0.0667)) / (2 * pi * fsNominal)) "main field inductance" annotation(Dialog(group = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Inductance Lrsigma(start = (3 * (1 - sqrt(1 - 0.0667))) / (2 * pi * fsNominal)) "rotor stray inductance (equivalent three phase winding)" annotation(Dialog(group = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Resistance Rr(start = 0.04) "warm rotor resistance (equivalent three phase winding)" annotation(Dialog(group = "Nominal resistances and inductances"));
            Machines.BasicMachines.Components.SquirrelCage squirrelCageR(final Lrsigma = Lrsigma, final Rr = Rr) annotation(Placement(transformation(origin = {0, -30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            annotation(defaultComponentName = "aimc", Documentation(info = "<HTML>
<b>Model of a three phase asynchronous induction machine with squirrel cage.</b><br>
Resistance and stray inductance of stator is modeled directly in stator phases, then using space phasor transformation. Resistance and stray inductance of rotor's squirrel cage is modeled in two axis of the rotor-fixed ccordinate system. Both together connected via a stator-fixed <i>AirGap</i> model. Only losses in stator and rotor resistance are taken into account.
<br><b>Default values for machine's parameters (a realistic example) are:</b><br>
<table>
<tr>
<td valign=\"top\">number of pole pairs p</td>
<td valign=\"top\">2</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">rotor's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">nominal frequency fNominal</td>
<td valign=\"top\">50</td><td valign=\"top\">Hz</td>
</tr>
<tr>
<td valign=\"top\">nominal voltage per phase</td>
<td valign=\"top\">100</td><td valign=\"top\">V RMS</td>
</tr>
<tr>
<td valign=\"top\">nominal current per phase</td>
<td valign=\"top\">100</td><td valign=\"top\">A RMS</td>
</tr>
<tr>
<td valign=\"top\">nominal torque</td>
<td valign=\"top\">161.4</td><td valign=\"top\">Nm</td>
</tr>
<tr>
<td valign=\"top\">nominal speed</td>
<td valign=\"top\">1440.45</td><td valign=\"top\">rpm</td>
</tr>
<tr>
<td valign=\"top\">nominal mechanical output</td>
<td valign=\"top\">24.346</td><td valign=\"top\">kW</td>
</tr>
<tr>
<td valign=\"top\">efficiency</td>
<td valign=\"top\">92.7</td><td valign=\"top\">%</td>
</tr>
<tr>
<td valign=\"top\">power factor</td>
<td valign=\"top\">0.875</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator resistance</td>
<td valign=\"top\">0.03</td><td valign=\"top\">Ohm per phase in warm condition</td>
</tr>
<tr>
<td valign=\"top\">rotor resistance</td>
<td valign=\"top\">0.04</td><td valign=\"top\">Ohm in warm condition</td>
</tr>
<tr>
<td valign=\"top\">stator reactance Xs</td>
<td valign=\"top\">3</td><td valign=\"top\">Ohm per phase</td>
</tr>
<tr>
<td valign=\"top\">rotor reactance Xr</td>
<td valign=\"top\">3</td><td valign=\"top\">Ohm</td>
</tr>
<tr>
<td valign=\"top\">total stray coefficient sigma</td>
<td valign=\"top\">0.0667</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">These values give the following inductances, <br>assuming equal stator and rotor stray inductances:</td>
<td valign=\"top\"> </td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator stray inductance per phase</td>
<td valign=\"top\">Xs * (1 - sqrt(1-sigma))/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">rotor stray inductance</td>
<td valign=\"top\">Xr * (1 - sqrt(1-sigma))/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">main field inductance per phase</td>
<td valign=\"top\">sqrt(Xs*Xr * (1-sigma))/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
</table>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

          equation 
            connect(airGapS.spacePhasor_r,squirrelCageR.spacePhasor_r) annotation(Line(points = {{10, -10},{10, -12.5},{10, -12.5},{10, -15},{10, -20},{10, -20}}, color = {0,0,255}));
            connect(spacePhasorS.spacePhasor,airGapS.spacePhasor_s) annotation(Line(points = {{10,20},{10,15},{10,10},{10,10}}, color = {0,0,255}, smooth = Smooth.None));
            connect(airGapS.support,internalSupport) annotation(Line(points = {{ -10,1.83697e-15},{ -56,1.83697e-15},{ -56,0},{ -90,0},{ -90, -100},{20, -100}}, color = {0,0,0}, smooth = Smooth.None));
            connect(airGapS.flange,inertiaRotor.flange_a) annotation(Line(points = {{10, -1.83697e-15},{36, -1.83697e-15},{36,1.22465e-15},{60,1.22465e-15}}, color = {0,0,0}, smooth = Smooth.None));
          end AIM_SquirrelCage;
          model AIM_SlipRing "Asynchronous induction machine with slipring rotor"
            extends Machines.Interfaces.PartialBasicInductionMachine(final idq_ss = airGapS.i_ss, final idq_sr = airGapS.i_sr, final idq_rs = airGapS.i_rs, final idq_rr = airGapS.i_rr);
            Components.AirGapS airGapS(final p = p, final m = 3, final Lm = Lm) annotation(Placement(transformation(origin = {0,0}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            parameter Modelica.SIunits.Inductance Lm(start = (3 * sqrt(1 - 0.0667)) / (2 * pi * fsNominal)) "main field inductance" annotation(Dialog(group = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Inductance Lrsigma(start = (3 * (1 - sqrt(1 - 0.0667))) / (2 * pi * fsNominal)) "rotor stray inductance per phase" annotation(Dialog(group = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Resistance Rr(start = 0.04) "warm rotor resistance per phase" annotation(Dialog(group = "Nominal resistances and inductances"));
            parameter Boolean useTurnsRatio(start = true) "use turnsRatio or calculate from locked-rotor voltage?";
            parameter Real turnsRatio(final min = Modelica.Constants.small, start = 1) "(ws*xis) / (wr*xir)" annotation(Dialog(enable = useTurnsRatio));
            parameter Modelica.SIunits.Voltage VsNominal(start = 100) "nominal stator voltage per phase" annotation(Dialog(enable = not useTurnsRatio));
            parameter Modelica.SIunits.Voltage VrLockedRotor(start = (100 * 2 * pi * fsNominal * Lm) / sqrt(Rs ^ 2 + (2 * pi * fsNominal * (Lm + Lssigma)) ^ 2)) "locked-rotor voltage per phase" annotation(Dialog(enable = not useTurnsRatio));
            output Modelica.SIunits.Current i_0_r(stateSelect = StateSelect.prefer) = spacePhasorR.zero.i "rotor zero-sequence current";
            output Modelica.SIunits.Voltage vr[m] = plug_rp.pin.v - plug_rn.pin.v "rotor instantaneous voltages";
            output Modelica.SIunits.Current ir[m] = plug_rp.pin.i "rotor instantaneous currents";
          protected 
            final parameter Real internalTurnsRatio = if useTurnsRatio then turnsRatio else (VsNominal / VrLockedRotor * 2 * pi * fsNominal * Lm) / sqrt(Rs ^ 2 + (2 * pi * fsNominal * (Lm + Lssigma)) ^ 2);
          public 
            Machines.SpacePhasors.Components.SpacePhasor spacePhasorR(final turnsRatio = internalTurnsRatio) annotation(Placement(transformation(origin = {0, -30}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
            Modelica.Electrical.MultiPhase.Basic.Inductor lrsigma(final m = m, final L = fill(Lrsigma, m)) annotation(Placement(transformation(extent = {{30, -90},{10, -70}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Basic.Resistor rr(final m = m, final R = fill(Rr, m), final T_ref = fill(293.15, m), final alpha = zeros(m), final useHeatPort = false, final T = rr.T_ref) annotation(Placement(transformation(extent = {{60, -90},{40, -70}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_rp(final m = m) annotation(Placement(transformation(extent = {{ -110,70},{ -90,50}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_rn(final m = m) annotation(Placement(transformation(extent = {{ -110, -50},{ -90, -70}}, rotation = 0)));
            annotation(defaultComponentName = "aims", Documentation(info = "<HTML>
<b>Model of a three phase asynchronous induction machine with slipring rotor.</b><br>
Resistance and stray inductance of stator and rotor are modeled directly in stator respectively rotor phases, then using space phasor transformation and a stator-fixed <i>AirGap</i> model. Only losses in stator and rotor resistance are taken into account.
<br><b>Default values for machine's parameters (a realistic example) are:</b><br>
<table>
<tr>
<td valign=\"top\">number of pole pairs p</td>
<td valign=\"top\">2</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">rotor's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">nominal frequency fNominal</td>
<td valign=\"top\">50</td><td valign=\"top\">Hz</td>
</tr>
<tr>
<td valign=\"top\">nominal voltage per phase</td>
<td valign=\"top\">100</td><td valign=\"top\">V RMS</td>
</tr>
<tr>
<td valign=\"top\">nominal current per phase</td>
<td valign=\"top\">100</td><td valign=\"top\">A RMS</td>
</tr>
<tr>
<td valign=\"top\">nominal torque</td>
<td valign=\"top\">161.4</td><td valign=\"top\">Nm</td>
</tr>
<tr>
<td valign=\"top\">nominal speed</td>
<td valign=\"top\">1440.45</td><td valign=\"top\">rpm</td>
</tr>
<tr>
<td valign=\"top\">nominal mechanical output</td>
<td valign=\"top\">24.346</td><td valign=\"top\">kW</td>
</tr>
<tr>
<td valign=\"top\">efficiency</td>
<td valign=\"top\">92.7</td><td valign=\"top\">%</td>
</tr>
<tr>
<td valign=\"top\">power factor</td>
<td valign=\"top\">0.875</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator resistance</td>
<td valign=\"top\">0.03</td><td valign=\"top\">Ohm per phase in warm condition</td>
</tr>
<tr>
<td valign=\"top\">rotor resistance</td>
<td valign=\"top\">0.04</td><td valign=\"top\">Ohm per phase in warm condition</td>
</tr>
<tr>
<td valign=\"top\">stator reactance Xs</td>
<td valign=\"top\">3</td><td valign=\"top\">Ohm per phase</td>
</tr>
<tr>
<td valign=\"top\">rotor reactance Xr</td>
<td valign=\"top\">3</td><td valign=\"top\">Ohm per phase</td>
</tr>
<tr>
<td valign=\"top\">total stray coefficient sigma</td>
<td valign=\"top\">0.0667</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">turnsRatio</td>
<td valign=\"top\">1</td><td valign=\"top\">effective ratio of stator and rotor current (ws*xis) / (wr*xir)</td>
</tr>
<tr>
<td valign=\"top\">These values give the following inductances:</td>
<td valign=\"top\"> </td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator stray inductance per phase</td>
<td valign=\"top\">Xs * (1 - sqrt(1-sigma))/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">rotor stray inductance</td>
<td valign=\"top\">Xr * (1 - sqrt(1-sigma))/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">main field inductance per phase</td>
<td valign=\"top\">sqrt(Xs*Xr * (1-sigma))/(2*pi*f)</td><td valign=\"top\"> </td>
</tr>
</table>
<p>
Parameter turnsRatio could be obtained from the following relationship
at standstill with open rotor circuit at nominal voltage and nominal frequency, <br>
using the locked-rotor voltage VR, no-load stator current I0 and powerfactor PF0:<br>
turnsRatio * <u>V</u><sub>R</sub> = <u>V</u><sub>s</sub> - (R<sub>s</sub> + j X<sub>s,sigma</sub>) <u>I</u><sub>0</sub>
</p>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -100,50},{ -100,20},{ -60,20}}, color = {0,0,255}),Line(points = {{ -100, -50},{ -100, -20},{ -60, -20}}, color = {0,0,255})}));

          equation 
            connect(rr.plug_n,lrsigma.plug_p) annotation(Line(points = {{40, -80},{30, -80}}, color = {0,0,255}));
            connect(rr.plug_p,plug_rp) annotation(Line(points = {{60, -80},{60, -90},{ -80, -90},{ -80,60},{ -100,60}}, color = {0,0,255}));
            connect(spacePhasorR.ground,spacePhasorR.zero) annotation(Line(points = {{ -10, -20},{ -10, -14},{6.12323e-16, -14},{6.12323e-16, -20}}, color = {0,0,255}));
            connect(airGapS.spacePhasor_r,spacePhasorR.spacePhasor) annotation(Line(points = {{10, -10},{10, -15},{10, -15},{10, -20}}, color = {0,0,255}));
            connect(spacePhasorS.spacePhasor,airGapS.spacePhasor_s) annotation(Line(points = {{10,20},{10,15},{10,10},{10,10}}, color = {0,0,255}, smooth = Smooth.None));
            connect(airGapS.support,internalSupport) annotation(Line(points = {{ -10,1.83697e-15},{ -90,1.83697e-15},{ -90, -100},{20, -100}}, color = {0,0,0}, smooth = Smooth.None));
            connect(airGapS.flange,inertiaRotor.flange_a) annotation(Line(points = {{10, -1.83697e-15},{34, -1.83697e-15},{34,1.22465e-15},{60,1.22465e-15}}, color = {0,0,0}, smooth = Smooth.None));
            connect(lrsigma.plug_n,spacePhasorR.plug_p) annotation(Line(points = {{10, -80},{10, -40}}, color = {0,0,255}, smooth = Smooth.None));
            connect(spacePhasorR.plug_n,plug_rn) annotation(Line(points = {{ -10, -40},{ -10, -60},{ -100, -60}}, color = {0,0,255}, smooth = Smooth.None));
          end AIM_SlipRing;
        end AsynchronousInductionMachines;
        package SynchronousInductionMachines "Models of synchronous induction machines"
          extends Modelica.Icons.Library;
          annotation(Documentation(info = "<HTML>
This package contains models of synchronous induction machines, based on space phasor theory:
<ul>
<li>SM_PermanentMagnet: synchronous induction machine with permanent magnet excitation, with damper cage</li>
<li>SM_ElectricalExcited: synchronous induction machine with electrical excitation
    and damper cage</li>
<li>SM_ReluctanceRotor: induction machine with reluctance rotor and damper cage<br>
i.e. a squirrel cage rotor with magnetic poles due to different airgap width</li>
</ul>
These models use package SpacePhasors.
<br><b>Please keep in mind:</b><br>
<ul>
<li>We keep the same reference system as for motors, i.e.:<br>
    Positive RotorDisplacementAngle means acting as motor,<br>
    with positive electric power consumption and positive mechanical power output.
<li>ElectricalAngle = p * MechanicalAngle</li>
<li>real axis = d-axis<br>
    imaginary= q-axis</li>
<li>Voltage induced by the magnet wheel (d-axis) is located in the q-axis.</li>
</ul>
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2009, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.02  2004/09/19 Anton Haumer</li>
  <li> v1.03  2004/09/24 Anton Haumer<br>
       consistent naming of inductors and resistors in machine models</li>
  <li> v1.1   2004/10/01 Anton Haumer<br>
       changed naming and structure<br>
       issued to Modelica Standard Library 2.1</li>
  <li> v1.2   2004/10/27 Anton Haumer<br>
       fixed a bug with support (formerly bearing)</li>
  <li> v1.3.2 2004/11/10 Anton Haumer<br>
       ReluctanceRotor moved to SynchronousMachines</li>
  <li> v1.4   2004/11/11 Anton Haumer<br>
       removed mechanical flange support<br>
       to ease the implementation of a 3D-frame in a future release</li>
  <li> v1.52  2005/10/12 Anton Haumer<br>
       added SM_ElectricalExcited</li>
  <li> v1.53  2005/10/14 Anton Haumer<br>
       introduced unsymmetrical DamperCage for Synchronous Machines</li>
  <li> v1.6.2 2005/10/23 Anton Haumer<br>
       selectable DamperCage for Synchronous Machines</li>
  <li> v1.7.1 2006/02/06 Anton Haumer<br>
       changed some naming of synchronous machines, not affecting existing models</li>
  </ul>
</HTML>"));
          model SM_PermanentMagnet "Permanent magnet synchronous induction machine"
            extends Machines.Interfaces.PartialBasicInductionMachine(Lssigma(start = 0.1 / (2 * pi * fsNominal)), final idq_ss = airGapR.i_ss, final idq_sr = airGapR.i_sr, final idq_rs = airGapR.i_rs, final idq_rr = airGapR.i_rr);
            Components.AirGapR airGapR(final p = p, final m = 3, final Lmd = Lmd, final Lmq = Lmq) annotation(Placement(transformation(origin = {0,0}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            parameter Modelica.SIunits.Voltage VsOpenCircuit(start = 112.3) "open circuit RMS voltage per phase @ fNominal" annotation(Dialog(group = "Excitation"));
            parameter Modelica.SIunits.Inductance Lmd(start = 0.3 / (2 * pi * fsNominal)) "main field inductance in d-axis" annotation(Dialog(group = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Inductance Lmq(start = 0.3 / (2 * pi * fsNominal)) "main field inductance in q-axis" annotation(Dialog(group = "Nominal resistances and inductances"));
            parameter Boolean useDamperCage(start = true) "enable / disable damper cage" annotation(Dialog(group = "DamperCage"));
            parameter Modelica.SIunits.Inductance Lrsigmad(start = 0.05 / (2 * pi * fsNominal)) "damper stray inductance in d-axis" annotation(Dialog(group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Inductance Lrsigmaq = Lrsigmad "damper stray inductance in q-axis" annotation(Dialog(group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Resistance Rrd(start = 0.04) "warm damper resistance in d-axis" annotation(Dialog(group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Resistance Rrq = Rrd "warm damper resistance in q-axis" annotation(Dialog(group = "DamperCage", enable = useDamperCage));
            output Modelica.SIunits.Current idq_dr[2](each stateSelect = StateSelect.prefer) = damperCage.spacePhasor_r.i_ if useDamperCage "damper space phasor current / rotor fixed frame";
          protected 
            final parameter Modelica.SIunits.Current Ie = (sqrt(2) * VsOpenCircuit) / (Lmd * 2 * pi * fsNominal) "equivalent excitation current";
          public 
            Machines.BasicMachines.Components.PermanentMagnet permanentMagnet(Ie = Ie) annotation(Placement(transformation(origin = {0, -60}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Components.DamperCage damperCage(final Lrsigmad = Lrsigmad, final Lrsigmaq = Lrsigmaq, final Rrd = Rrd, final Rrq = Rrq) if useDamperCage annotation(Placement(transformation(origin = {0, -30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            annotation(defaultComponentName = "smpm", Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -130,10},{ -100, -10}}, lineColor = {0,0,0}, fillColor = {0,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -100,10},{ -70, -10}}, lineColor = {0,0,0}, fillColor = {255,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -134,34},{ -66, -34}}, lineColor = {0,0,255})}), Documentation(info = "<HTML>
<b>Model of a three phase permanent magnet synchronous induction machine.</b><br>
Resistance and stray inductance of stator is modeled directly in stator phases, then using space phasor transformation and a rotor-fixed <i>AirGap</i> model. Resistance and stray inductance of rotor's squirrel cage is modeled in two axis of the rotor-fixed ccordinate system. Permanent magnet excitation is modelled by a constant equivalent excitation current feeding the d-axis. Only losses in stator and damper resistance are taken into account.
<br>Whether a damper cage is present or not, can be selected with Boolean parameter useDamperCage (default = true).
<br><b>Default values for machine's parameters (a realistic example) are:</b><br>
<table>
<tr>
<td valign=\"top\">number of pole pairs p</td>
<td valign=\"top\">2</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">rotor's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">nominal frequency fNominal</td>
<td valign=\"top\">50</td><td valign=\"top\">Hz</td>
</tr>
<tr>
<td valign=\"top\">nominal voltage per phase</td>
<td valign=\"top\">100</td><td valign=\"top\">V RMS</td>
</tr>
<tr>
<td valign=\"top\">no-load voltage per phase</td>
<td valign=\"top\">112.3</td><td valign=\"top\">V RMS @ nominal speed</td>
</tr>
<tr>
<td valign=\"top\">nominal current per phase</td>
<td valign=\"top\">100</td><td valign=\"top\">A RMS</td>
</tr>
<tr>
<td valign=\"top\">nominal torque</td>
<td valign=\"top\">181.4</td><td valign=\"top\">Nm</td>
</tr>
<tr>
<td valign=\"top\">nominal speed</td>
<td valign=\"top\">1500</td><td valign=\"top\">rpm</td>
</tr>
<tr>
<td valign=\"top\">nominal mechanical output</td>
<td valign=\"top\">28.5</td><td valign=\"top\">kW</td>
</tr>
<tr>
<td valign=\"top\">nominal rotor angle</td>
<td valign=\"top\">20.75</td><td valign=\"top\">degree</td>
</tr>
<tr>
<td valign=\"top\">efficiency</td>
<td valign=\"top\">95.0</td><td valign=\"top\">%</td>
</tr>
<tr>
<td valign=\"top\">power factor</td>
<td valign=\"top\">0.98</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator resistance</td>
<td valign=\"top\">0.03</td><td valign=\"top\">Ohm per phase in warm condition</td>
</tr>
<tr>
<td valign=\"top\">stator reactance Xd</td>
<td valign=\"top\">0.4</td><td valign=\"top\">Ohm per phase in d-axis</td>
</tr>
<tr>
<td valign=\"top\">stator reactance Xq</td>
<td valign=\"top\">0.4</td><td valign=\"top\">Ohm per phase in q-axis</td>
</tr>
<tr>
<td valign=\"top\">stator stray reactance Xss</td>
<td valign=\"top\">0.1</td><td valign=\"top\">Ohm per phase</td>
</tr>
<tr>
<td valign=\"top\">damper resistance in d-axis</td>
<td valign=\"top\">0.04</td><td valign=\"top\">Ohm in warm condition</td>
</tr>
<tr>
<td valign=\"top\">damper resistance in q-axis</td>
<td valign=\"top\">same as d-axis</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">damper stray reactance in d-axis XDds</td>
<td valign=\"top\">0.05</td><td valign=\"top\">Ohm</td>
</tr>
<tr>
<td valign=\"top\">damper stray reactance in q-axis XDqs</td>
<td valign=\"top\">same as d-axis</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">These values give the following inductances:</td>
<td valign=\"top\"> </td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">main field inductance in d-axis</td>
<td valign=\"top\">(Xd - Xss)/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">main field inductance in q-axis</td>
<td valign=\"top\">(Xq - Xss)/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator stray inductance per phase</td>
<td valign=\"top\">Xss/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">damper stray inductance in d-axis</td>
<td valign=\"top\">XDds/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">damper stray inductance in q-axis</td>
<td valign=\"top\">XDqs/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
</table>
</HTML>"));

          equation 
            connect(airGapR.spacePhasor_r,damperCage.spacePhasor_r) annotation(Line(points = {{10, -10},{10, -12.5},{10, -12.5},{10, -15},{10, -20},{10, -20}}, color = {0,0,255}));
            connect(airGapR.spacePhasor_r,permanentMagnet.spacePhasor_r) annotation(Line(points = {{10, -10},{20, -10},{20, -50},{10, -50}}, color = {0,0,255}));
            connect(spacePhasorS.spacePhasor,airGapR.spacePhasor_s) annotation(Line(points = {{10,20},{10,15},{10,10},{10,10}}, color = {0,0,255}, smooth = Smooth.None));
            connect(airGapR.support,internalSupport) annotation(Line(points = {{ -10,1.83697e-15},{ -90,1.83697e-15},{ -90, -100},{20, -100}}, color = {0,0,0}, smooth = Smooth.None));
            connect(airGapR.flange,inertiaRotor.flange_a) annotation(Line(points = {{10, -1.83697e-15},{35, -1.83697e-15},{35,1.22465e-15},{60,1.22465e-15}}, color = {0,0,0}, smooth = Smooth.None));
          end SM_PermanentMagnet;
          model SM_ElectricalExcited "Electrical excited synchronous induction machine with damper cage"
            extends Machines.Interfaces.PartialBasicInductionMachine(Lssigma(start = 0.1 / (2 * pi * fsNominal)), final idq_ss = airGapR.i_ss, final idq_sr = airGapR.i_sr, final idq_rs = airGapR.i_rs, final idq_rr = airGapR.i_rr);
            Components.AirGapR airGapR(final p = p, final m = 3, final Lmd = Lmd, final Lmq = Lmq) annotation(Placement(transformation(origin = {0,0}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            parameter Modelica.SIunits.Inductance Lmd(start = 1.5 / (2 * pi * fsNominal)) "main field inductance in d-axis" annotation(Dialog(group = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Inductance Lmq(start = 1.5 / (2 * pi * fsNominal)) "main field inductance in q-axis" annotation(Dialog(group = "Nominal resistances and inductances"));
            parameter Boolean useDamperCage(start = true) "enable / disable damper cage" annotation(Dialog(group = "DamperCage"));
            parameter Modelica.SIunits.Inductance Lrsigmad(start = 0.05 / (2 * pi * fsNominal)) "damper stray inductance in d-axis" annotation(Dialog(group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Inductance Lrsigmaq = Lrsigmad "damper stray inductance in q-axis" annotation(Dialog(group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Resistance Rrd(start = 0.04) "warm damper resistance in d-axis" annotation(Dialog(group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Resistance Rrq = Rrd "warm damper resistance in q-axis" annotation(Dialog(group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Voltage VsNominal(start = 100) "nominal stator RMS voltage per phase" annotation(Dialog(group = "Excitation"));
            parameter Modelica.SIunits.Current IeOpenCircuit(start = 10) "open circuit excitation current @ nominal voltage and frequency" annotation(Dialog(group = "Excitation"));
            parameter Modelica.SIunits.Resistance Re(start = 2.5) "warm excitation resistance" annotation(Dialog(group = "Excitation"));
            parameter Real sigmae(min = 0, max = 1, start = 0.025) "stray fraction of total excitation inductance" annotation(Dialog(group = "Excitation"));
            output Modelica.SIunits.Current idq_dr[2](each stateSelect = StateSelect.prefer) = damperCage.spacePhasor_r.i_ if useDamperCage "damper space phasor current / rotor fixed frame";
            output Modelica.SIunits.Voltage ve = pin_ep.v - pin_en.v "excitation voltage";
            output Modelica.SIunits.Current ie = pin_ep.i "excitation current";
          protected 
            final parameter Real turnsRatio = (sqrt(2) * VsNominal) / (2 * pi * fsNominal * Lmd * IeOpenCircuit) "stator current / excitation current";
            final parameter Modelica.SIunits.Inductance Lesigma = ((Lmd * turnsRatio ^ 2 * 3) / 2 * sigmae) / (1 - sigmae);
          public 
            Components.DamperCage damperCage(final Lrsigmad = Lrsigmad, final Lrsigmaq = Lrsigmaq, final Rrd = Rrd, final Rrq = Rrq) if useDamperCage annotation(Placement(transformation(origin = {0, -30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            Components.ElectricalExcitation electricalExcitation(final turnsRatio = turnsRatio) annotation(Placement(transformation(origin = {0, -60}, extent = {{10,10},{ -10, -10}}, rotation = 90)));
            Modelica.Electrical.Analog.Basic.Resistor re(final R = Re, final T_ref = 293.15, final alpha = 0, final useHeatPort = false, final T = re.T_ref) annotation(Placement(transformation(origin = {50, -80}, extent = {{ -10,10},{10, -10}}, rotation = 180)));
            Modelica.Electrical.Analog.Basic.Inductor lesigma(final L = Lesigma) annotation(Placement(transformation(extent = {{30, -90},{10, -70}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.PositivePin pin_ep annotation(Placement(transformation(extent = {{ -110,70},{ -90,50}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin pin_en annotation(Placement(transformation(extent = {{ -90, -50},{ -110, -70}}, rotation = 0)));
            annotation(defaultComponentName = "smee", Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -134,34},{ -66, -34}}, lineColor = {0,0,255}),Line(points = {{ -100,50},{ -100,20},{ -130,20},{ -130, -4}}, color = {0,0,255}),Line(points = {{ -130, -4},{ -129,1},{ -125,5},{ -120,6},{ -115,5},{ -111,1},{ -110, -4}}, color = {0,0,255}),Line(points = {{ -110, -4},{ -109,1},{ -105,5},{ -100,6},{ -95,5},{ -91,1},{ -90, -4}}, color = {0,0,255}),Line(points = {{ -90, -4},{ -89,1},{ -85,5},{ -80,6},{ -75,5},{ -71,1},{ -70, -4}}, color = {0,0,255}),Line(points = {{ -100, -50},{ -100, -20},{ -70, -20},{ -70, -2}}, color = {0,0,255})}), Documentation(info = "<HTML>
<b>Model of a three phase electrical excited synchronous induction machine with damper cage.</b><br>
Resistance and stray inductance of stator is modeled directly in stator phases, then using space phasor transformation and a rotor-fixed <i>AirGap</i> model. Resistance and stray inductance of rotor's squirrel cage is modeled in two axis of the rotor-fixed ccordinate system. Electrical excitation is modelled by converting excitation current and voltage to d-axis space phasors. Only losses in stator, damper and excitation resistance are taken into account.
<br>Whether a damper cage is present or not, can be selected with Boolean parameter useDamperCage (default = true).
<br><b>Default values for machine's parameters (a realistic example) are:</b><br>
<table>
<tr>
<td valign=\"top\">number of pole pairs p</td>
<td valign=\"top\">2</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">rotor's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">nominal frequency fNominal</td>
<td valign=\"top\">50</td><td valign=\"top\">Hz</td>
</tr>
<tr>
<td valign=\"top\">nominal voltage per phase</td>
<td valign=\"top\">100</td><td valign=\"top\">V RMS</td>
</tr>
<tr>
<td valign=\"top\">no-load excitation current<br>
    @ nominal voltage and frequency</td>
<td valign=\"top\">10</td><td valign=\"top\">A DC</td>
</tr>
<tr>
<td valign=\"top\">warm excitation resistance</td>
<td valign=\"top\">2.5</td><td valign=\"top\">Ohm</td>
</tr>
<tr>
<td valign=\"top\">nominal current per phase</td>
<td valign=\"top\">100</td><td valign=\"top\">A RMS</td>
</tr>
<tr>
<td valign=\"top\">nominal apparent power</td>
<td valign=\"top\">-30000</td><td valign=\"top\">VA</td>
</tr>
<tr>
<td valign=\"top\">power factor</td>
<td valign=\"top\">-1.0</td><td valign=\"top\">ind./cap.</td>
</tr>
<tr>
<td valign=\"top\">nominal excitation current</td>
<td valign=\"top\">19</td><td valign=\"top\">A</td>
</tr>
<tr>
<td valign=\"top\">efficiency w/o excitation</td>
<td valign=\"top\">97.1</td><td valign=\"top\">%</td>
</tr>
<tr>
<td valign=\"top\">nominal torque</td>
<td valign=\"top\">-196.7</td><td valign=\"top\">Nm</td>
</tr>
<tr>
<td valign=\"top\">nominal speed</td>
<td valign=\"top\">1500</td><td valign=\"top\">rpm</td>
</tr>
<tr>
<td valign=\"top\">nominal rotor angle</td>
<td valign=\"top\">-57.23</td><td valign=\"top\">degree</td>
</tr>
<tr>
<td valign=\"top\">stator resistance</td>
<td valign=\"top\">0.03</td><td valign=\"top\">Ohm per phase in warm condition</td>
</tr>
<tr>
<td valign=\"top\">stator reactance Xd</td>
<td valign=\"top\">1.6</td><td valign=\"top\">Ohm per phase in d-axis</td>
</tr>
<tr>
<td valign=\"top\">giving Kc</td>
<td valign=\"top\">0.625</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator reactance Xq</td>
<td valign=\"top\">1.6</td><td valign=\"top\">Ohm per phase in q-axis</td>
</tr>
<tr>
<td valign=\"top\">stator stray reactance Xss</td>
<td valign=\"top\">0.1</td><td valign=\"top\">Ohm per phase</td>
</tr>
<tr>
<td valign=\"top\">damper resistance in d-axis</td>
<td valign=\"top\">0.04</td><td valign=\"top\">Ohm in warm condition</td>
</tr>
<tr>
<td valign=\"top\">damper resistance in q-axis</td>
<td valign=\"top\">same as d-axis</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">damper stray reactance in d-axis XDds</td>
<td valign=\"top\">0.1</td><td valign=\"top\">Ohm</td>
</tr>
<tr>
<td valign=\"top\">damper stray reactance in q-axis XDqs</td>
<td valign=\"top\">same as d-axis</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">excitation stray inductance</td>
<td valign=\"top\">2.5</td><td valign=\"top\">% of total excitation inductance</td>
</tr>
<tr>
<td valign=\"top\">These values give the following inductances:</td>
<td valign=\"top\"> </td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">main field inductance in d-axis</td>
<td valign=\"top\">(Xd - Xss)/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">main field inductance in q-axis</td>
<td valign=\"top\">(Xq - Xss)/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator stray inductance per phase</td>
<td valign=\"top\">Xss/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">damper stray inductance in d-axis</td>
<td valign=\"top\">XDds/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">damper stray inductance in q-axis</td>
<td valign=\"top\">XDqs/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
</table>
</HTML>"));

          equation 
            connect(electricalExcitation.pin_en,pin_en) annotation(Line(points = {{ -10, -70},{ -70, -70},{ -70, -60},{ -100, -60}}, color = {0,0,255}));
            connect(pin_ep,re.p) annotation(Line(points = {{ -100,60},{ -80,60},{ -80, -90},{60, -90},{60, -80}}, color = {0,0,255}));
            connect(lesigma.p,re.n) annotation(Line(points = {{30, -80},{40, -80}}, color = {0,0,255}));
            connect(lesigma.n,electricalExcitation.pin_ep) annotation(Line(points = {{10, -80},{10, -75},{10, -70},{10, -70}}, color = {0,0,255}));
            connect(airGapR.spacePhasor_r,damperCage.spacePhasor_r) annotation(Line(points = {{10, -10},{10, -12.5},{10, -12.5},{10, -15},{10, -20},{10, -20}}, color = {0,0,255}));
            connect(airGapR.spacePhasor_r,electricalExcitation.spacePhasor_r) annotation(Line(points = {{10, -10},{20, -10},{20, -50},{10, -50}}, color = {0,0,255}));
            connect(spacePhasorS.spacePhasor,airGapR.spacePhasor_s) annotation(Line(points = {{10,20},{10,15},{10,10},{10,10}}, color = {0,0,255}, smooth = Smooth.None));
            connect(airGapR.support,internalSupport) annotation(Line(points = {{ -10,1.83697e-15},{ -46,1.83697e-15},{ -46,0},{ -90,0},{ -90, -100},{20, -100}}, color = {0,0,0}, smooth = Smooth.None));
            connect(airGapR.flange,inertiaRotor.flange_a) annotation(Line(points = {{10, -1.83697e-15},{35, -1.83697e-15},{35,1.22465e-15},{60,1.22465e-15}}, color = {0,0,0}, smooth = Smooth.None));
          end SM_ElectricalExcited;
          model SM_ReluctanceRotor "Synchronous induction machine with reluctance rotor and damper cage"
            extends Machines.Interfaces.PartialBasicInductionMachine(Lssigma(start = 0.1 / (2 * pi * fsNominal)), final idq_ss = airGapR.i_ss, final idq_sr = airGapR.i_sr, final idq_rs = airGapR.i_rs, final idq_rr = airGapR.i_rr);
            Components.AirGapR airGapR(final p = p, final m = 3, final Lmd = Lmd, final Lmq = Lmq) annotation(Placement(transformation(origin = {0,0}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            parameter Modelica.SIunits.Inductance Lmd(start = 2.9 / (2 * pi * fsNominal)) "main field inductance in d-axis" annotation(Dialog(group = "Nominal resistances and inductances"));
            parameter Modelica.SIunits.Inductance Lmq(start = 0.9 / (2 * pi * fsNominal)) "main field inductance in q-axis" annotation(Dialog(group = "Nominal resistances and inductances"));
            parameter Boolean useDamperCage(start = true) "enable / disable damper cage" annotation(Dialog(group = "DamperCage"));
            parameter Modelica.SIunits.Inductance Lrsigmad(start = 0.05 / (2 * pi * fsNominal)) "damper stray inductance in d-axis" annotation(Dialog(group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Inductance Lrsigmaq = Lrsigmad "damper stray inductance in q-axis" annotation(Dialog(group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Resistance Rrd(start = 0.04) "warm damper resistance in d-axis" annotation(Dialog(group = "DamperCage", enable = useDamperCage));
            parameter Modelica.SIunits.Resistance Rrq = Rrd "warm damper resistance in q-axis" annotation(Dialog(group = "DamperCage", enable = useDamperCage));
            Components.DamperCage damperCage(final Lrsigmad = Lrsigmad, final Lrsigmaq = Lrsigmaq, final Rrd = Rrd, final Rrq = Rrq) if useDamperCage annotation(Placement(transformation(origin = {0, -30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            annotation(defaultComponentName = "smr", Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -130,10},{ -100, -10}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -100,10},{ -70, -10}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -134,34},{ -66, -34}}, lineColor = {0,0,255})}), Documentation(info = "<HTML>
<b>Model of a three phase synchronous induction machine with reluctance rotor and damper cage.</b><br>
Resistance and stray inductance of stator is modeled directly in stator phases, then using space phasor transformation. Resistance and stray inductance of rotor's squirrel cage is modeled in two axis of the rotor-fixed ccordinate system. Both together connected via a rotor-fixed <i>AirGap</i> model. Only losses in stator and rotor resistance are taken into account.
<br>Whether a damper cage is present or not, can be selected with Boolean parameter useDamperCage (default = true).
<br><b>Default values for machine's parameters (a realistic example) are:</b><br>
<table>
<tr>
<td valign=\"top\">number of pole pairs p</td>
<td valign=\"top\">2</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">rotor's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">nominal frequency fNominal</td>
<td valign=\"top\">50</td><td valign=\"top\">Hz</td>
</tr>
<tr>
<td valign=\"top\">nominal voltage per phase</td>
<td valign=\"top\">100</td><td valign=\"top\">V RMS</td>
</tr>
<tr>
<td valign=\"top\">nominal current per phase</td>
<td valign=\"top\">50</td><td valign=\"top\">A RMS</td>
</tr>
<tr>
<td valign=\"top\">nominal torque</td>
<td valign=\"top\"> 46</td><td valign=\"top\">Nm</td>
</tr>
<tr>
<td valign=\"top\">nominal speed</td>
<td valign=\"top\">1500</td><td valign=\"top\">rpm</td>
</tr>
<tr>
<td valign=\"top\">nominal mechanical output</td>
<td valign=\"top\"> 7.23</td><td valign=\"top\">kW</td>
</tr>
<tr>
<td valign=\"top\">efficiency</td>
<td valign=\"top\">96.98</td><td valign=\"top\">%</td>
</tr>
<tr>
<td valign=\"top\">power factor</td>
<td valign=\"top\">0.497</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator resistance</td>
<td valign=\"top\">0.03</td><td valign=\"top\">Ohm per phase in warm condition</td>
</tr>
<tr>
<td valign=\"top\">rotor resistance in d-axis</td>
<td valign=\"top\">0.04</td><td valign=\"top\">Ohm in warm condition</td>
</tr>
<tr>
<td valign=\"top\">rotor resistance in q-axis</td>
<td valign=\"top\">same as d-axis</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator reactance Xsd in d-axis</td>
<td valign=\"top\">3</td><td valign=\"top\">Ohm per phase</td>
</tr>
<tr>
<td valign=\"top\">stator reactance Xsq in q-axis</td>
<td valign=\"top\">1</td><td valign=\"top\">Ohm</td>
</tr>
<tr>
<td valign=\"top\">stator stray reactance Xss</td>
<td valign=\"top\">0.1</td><td valign=\"top\">Ohm per phase</td>
</tr>
<tr>
<td valign=\"top\">rotor stray reactance in d-axis Xrds</td>
<td valign=\"top\">0.1</td><td valign=\"top\">Ohm per phase</td>
</tr>
<tr>
<td valign=\"top\">rotor stray reactance in q-axis Xrqs</td>
<td valign=\"top\">same as d-axis</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">These values give the following inductances:</td>
<td valign=\"top\"> </td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">stator stray inductance per phase</td>
<td valign=\"top\">Xss/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">rotor stray inductance in d-axis</td>
<td valign=\"top\">Xrds/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">rotor stray inductance in q-axis</td>
<td valign=\"top\">Xrqs/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">main field inductance per phase in d-axis</td>
<td valign=\"top\">(Xsd-Xss)/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
<tr>
<td valign=\"top\">main field inductance per phase in q-axis</td>
<td valign=\"top\">(Xsq-Xss)/(2*pi*fNominal)</td><td valign=\"top\"> </td>
</tr>
</table>
</HTML>"));

          equation 
            connect(airGapR.spacePhasor_r,damperCage.spacePhasor_r) annotation(Line(points = {{10, -10},{10, -12.5},{10, -12.5},{10, -15},{10, -20},{10, -20}}, color = {0,0,255}));
            connect(spacePhasorS.spacePhasor,airGapR.spacePhasor_s) annotation(Line(points = {{10,20},{10,15},{10,10},{10,10}}, color = {0,0,255}, smooth = Smooth.None));
            connect(airGapR.support,internalSupport) annotation(Line(points = {{ -10,1.83697e-15},{ -90,1.83697e-15},{ -90, -100},{20, -100}}, color = {0,0,0}, smooth = Smooth.None));
            connect(airGapR.flange,inertiaRotor.flange_a) annotation(Line(points = {{10, -1.83697e-15},{34, -1.83697e-15},{34,1.22465e-15},{60,1.22465e-15}}, color = {0,0,0}, smooth = Smooth.None));
          end SM_ReluctanceRotor;
        end SynchronousInductionMachines;
        package DCMachines "Models of DC machines"
          extends Modelica.Icons.Library;
          annotation(Documentation(info = "<HTML>
This package contains models of DC machines:
<ul>
<li>DC_PermanentMagnet: DC machine with permanent magnet excitation</li>
<li>DC_ElectricalExcited: DC machine with electrical shunt or separate excitation</li>
<li>DC_SeriesExcited: DC machine with series excitation</li>
</ul>
</HTML>
", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2009, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.02 2004/09/19 Anton Haumer</li>
  <li> v1.03 2004/09/24 Anton Haumer<br>
       added DC machine with series excitation</li>
  <li> v1.1  2004/10/01 Anton Haumer<br>
       changed naming and structure<br>
       issued to Modelica Standard Library 2.1</li>
  <li> v1.2  2004/10/27 Anton Haumer<br>
       fixed a bug with support (formerly bearing)</li>
  <li> v1.4   2004/11/11 Anton Haumer<br>
       removed mechanical flange support<br>
       to ease the implementation of a 3D-frame in a future release</li>
  </ul>
</HTML>"));
          model DC_PermanentMagnet "Permanent magnet DC machine"
            extends Machines.Interfaces.PartialBasicDCMachine(final turnsRatio = (VaNominal - Ra * IaNominal) / (wNominal * Le * IeNominal));
            Components.AirGapDC airGapDC(final turnsRatio = turnsRatio, final Le = Le) annotation(Placement(transformation(origin = {0,0}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          protected 
            final parameter Modelica.SIunits.Inductance Le(start = 1) "total field excitation inductance" annotation(Dialog(group = "Excitation"));
            constant Modelica.SIunits.Current IeNominal = 1 "equivalent excitation current";
          public 
            Modelica.Electrical.Analog.Basic.Ground eGround annotation(Placement(transformation(extent = {{ -20, -70},{0, -50}}, rotation = 0)));
            Modelica.Electrical.Analog.Sources.ConstantCurrent ie(I = IeNominal) annotation(Placement(transformation(origin = {10, -40}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
            annotation(defaultComponentName = "dcpm", Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -130,10},{ -100, -10}}, lineColor = {0,0,0}, fillColor = {0,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -100,10},{ -70, -10}}, lineColor = {0,0,0}, fillColor = {255,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
<b>Model of a DC Machine with Permanent magnet.</b><br>
Armature resistance and inductance are modeled directly after the armature pins, then using a <i>AirGapDC</i> model. Permanent magnet excitation is modelled by a constant equivalent excitation current feeding AirGapDC. Only losses in armature resistance are taken into account. No saturation is modelled.
<br><b>Default values for machine's parameters (a realistic example) are:</b><br>
<table>
<tr>
<td valign=\"top\">stator's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">rotor's moment of inertia</td>
<td valign=\"top\">0.15</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">nominal armature voltage</td>
<td valign=\"top\">100</td><td valign=\"top\">V</td>
</tr>
<tr>
<td valign=\"top\">nominal armature current</td>
<td valign=\"top\">100</td><td valign=\"top\">A</td>
</tr>
<tr>
<td valign=\"top\">nominal speed</td>
<td valign=\"top\">1425</td><td valign=\"top\">rpm</td>
</tr>
<tr>
<td valign=\"top\">nominal torque</td>
<td valign=\"top\">63.66</td><td valign=\"top\">Nm</td>
</tr>
<tr>
<td valign=\"top\">nominal mechanical output</td>
<td valign=\"top\">9.5</td><td valign=\"top\">kW</td>
</tr>
<tr>
<td valign=\"top\">efficiency</td>
<td valign=\"top\">95.0</td><td valign=\"top\">%</td>
</tr>
<tr>
<td valign=\"top\">armature resistance</td>
<td valign=\"top\">0.05</td><td valign=\"top\">Ohm in warm condition</td>
</tr>
<tr>
<td valign=\"top\">armature inductance</td>
<td valign=\"top\">0.0015</td><td valign=\"top\">H</td>
</tr>
</table>
Armature resistance resp. inductance include resistance resp. inductance of commutating pole winding and compensation windig, if present.
</HTML>"));

          equation 
            assert(VaNominal > Ra * IaNominal, "VaNominal has to be > Ra*IaNominal");
            connect(eGround.p,ie.p) annotation(Line(points = {{ -10, -50},{10, -50}}, color = {0,0,255}));
            connect(airGapDC.pin_ep,ie.n) annotation(Line(points = {{10, -10},{10, -20},{10, -20},{10, -30}}, color = {0,0,255}));
            connect(airGapDC.pin_en,eGround.p) annotation(Line(points = {{ -10, -10},{ -10, -30},{ -10, -30},{ -10, -50}}, color = {0,0,255}));
            connect(airGapDC.pin_ap,la.n) annotation(Line(points = {{10,10},{10,35},{10,35},{10,60}}, color = {0,0,255}, smooth = Smooth.None));
            connect(airGapDC.pin_an,pin_an) annotation(Line(points = {{ -10,10},{ -12,10},{ -12,60},{ -60,60},{ -60,100}}, color = {0,0,255}, smooth = Smooth.None));
            connect(airGapDC.support,internalSupport) annotation(Line(points = {{ -10,1.83697e-15},{ -90,1.83697e-15},{ -90, -100},{20, -100}}, color = {0,0,0}, smooth = Smooth.None));
            connect(airGapDC.flange,inertiaRotor.flange_a) annotation(Line(points = {{10, -1.83697e-15},{36, -1.83697e-15},{36,1.22465e-15},{60,1.22465e-15}}, color = {0,0,0}, smooth = Smooth.None));
          end DC_PermanentMagnet;
          model DC_ElectricalExcited "Electrical shunt/separate excited linear DC machine"
            extends Machines.Interfaces.PartialBasicDCMachine(final turnsRatio = (VaNominal - Ra * IaNominal) / (wNominal * Le * IeNominal));
            Components.AirGapDC airGapDC(final turnsRatio = turnsRatio, final Le = Le) annotation(Placement(transformation(origin = {0,0}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            parameter Modelica.SIunits.Current IeNominal(start = 1) "nominal excitation current" annotation(Dialog(group = "Excitation"));
            parameter Modelica.SIunits.Resistance Re(start = 100) "warm field excitation resistance" annotation(Dialog(group = "Excitation"));
            parameter Modelica.SIunits.Inductance Le(start = 1) "total field excitation inductance" annotation(Dialog(group = "Excitation"));
            output Modelica.SIunits.Voltage ve = pin_ep.v - pin_en.v "Field excitation voltage";
            output Modelica.SIunits.Current ie = pin_ep.i "Field excitation current";
            Modelica.Electrical.Analog.Basic.Resistor re(final R = Re, final T_ref = 293.15, final alpha = 0, final useHeatPort = false, final T = re.T_ref) annotation(Placement(transformation(origin = { -50, -40}, extent = {{10,10},{ -10, -10}}, rotation = 180)));
            Modelica.Electrical.Analog.Interfaces.PositivePin pin_ep annotation(Placement(transformation(extent = {{ -110,70},{ -90,50}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin pin_en annotation(Placement(transformation(extent = {{ -90, -50},{ -110, -70}}, rotation = 0)));
            annotation(defaultComponentName = "dcee", Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -130, -4},{ -129,1},{ -125,5},{ -120,6},{ -115,5},{ -111,1},{ -110, -4}}, color = {0,0,255}),Line(points = {{ -110, -4},{ -109,1},{ -105,5},{ -100,6},{ -95,5},{ -91,1},{ -90, -4}}, color = {0,0,255}),Line(points = {{ -90, -4},{ -89,1},{ -85,5},{ -80,6},{ -75,5},{ -71,1},{ -70, -4}}, color = {0,0,255}),Line(points = {{ -100, -50},{ -100, -20},{ -70, -20},{ -70, -2}}, color = {0,0,255}),Line(points = {{ -100,50},{ -100,20},{ -130,20},{ -130, -4}}, color = {0,0,255})}), Documentation(info = "<HTML>
<b>Model of a DC Machine with Electrical shunt or separate excitation.</b><br>
Armature resistance and inductance are modeled directly after the armature pins, then using a <i>AirGapDC</i> model.<br>
Only losses in armature and excitation resistance are taken into account. No saturation is modelled.<br>
Shunt or separate excitation is defined by the user's external circuit.
<br><b>Default values for machine's parameters (a realistic example) are:</b><br>
<table>
<tr>
<td valign=\"top\">stator's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">rotor's moment of inertia</td>
<td valign=\"top\">0.15</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">nominal armature voltage</td>
<td valign=\"top\">100</td><td valign=\"top\">V</td>
</tr>
<tr>
<td valign=\"top\">nominal armature current</td>
<td valign=\"top\">100</td><td valign=\"top\">A</td>
</tr>
<tr>
<td valign=\"top\">nominal torque</td>
<td valign=\"top\">63.66</td><td valign=\"top\">Nm</td>
</tr>
<tr>
<td valign=\"top\">nominal speed</td>
<td valign=\"top\">1425</td><td valign=\"top\">rpm</td>
</tr>
<tr>
<td valign=\"top\">nominal mechanical output</td>
<td valign=\"top\">9.5</td><td valign=\"top\">kW</td>
</tr>
<tr>
<td valign=\"top\">efficiency</td>
<td valign=\"top\">95.0</td><td valign=\"top\">% only armature</td>
</tr>
<tr>
<td valign=\"top\">efficiency</td>
<td valign=\"top\">94.06</td><td valign=\"top\">% including excitation</td>
</tr>
<tr>
<td valign=\"top\">armature resistance</td>
<td valign=\"top\">0.05</td><td valign=\"top\">Ohm in warm condition</td>
</tr>
<tr>
<td valign=\"top\">armature inductance</td>
<td valign=\"top\">0.0015</td><td valign=\"top\">H</td>
</tr>
<tr>
<td valign=\"top\">nominal excitation voltage</td>
<td valign=\"top\">100</td><td valign=\"top\">V</td>
</tr>
<tr>
<td valign=\"top\">nominal excitation current</td>
<td valign=\"top\">1</td><td valign=\"top\">A</td>
</tr>
<tr>
<td valign=\"top\">excitation resistance</td>
<td valign=\"top\">100</td><td valign=\"top\">Ohm in warm condition</td>
</tr>
<tr>
<td valign=\"top\">excitation inductance</td>
<td valign=\"top\">1</td><td valign=\"top\">H</td>
</tr>
</table>
Armature resistance resp. inductance include resistance resp. inductance of commutating pole winding and
compensation windig, if present.<br>
Armature current does not cover excitation current of a shunt excitation; in this case total current drawn from the grid = armature current + excitation current.
</HTML>"));

          equation 
            assert(VaNominal > Ra * IaNominal, "VaNominal has to be > Ra*IaNominal");
            connect(re.p,pin_ep) annotation(Line(points = {{ -60, -40},{ -80, -40},{ -80,60},{ -100,60}}, color = {0,0,255}));
            connect(pin_en,airGapDC.pin_en) annotation(Line(points = {{ -100, -60},{ -10, -60},{ -10, -10}}, color = {0,0,255}));
            connect(re.n,airGapDC.pin_ep) annotation(Line(points = {{ -40, -40},{10, -40},{10, -10}}, color = {0,0,255}));
            connect(airGapDC.pin_ap,la.n) annotation(Line(points = {{10,10},{10,35},{10,35},{10,60}}, color = {0,0,255}, smooth = Smooth.None));
            connect(airGapDC.pin_an,pin_an) annotation(Line(points = {{ -10,10},{ -10,60},{ -60,60},{ -60,100}}, color = {0,0,255}, smooth = Smooth.None));
            connect(airGapDC.support,internalSupport) annotation(Line(points = {{ -10,1.83697e-15},{ -46,1.83697e-15},{ -46,0},{ -90,0},{ -90, -100},{20, -100}}, color = {0,0,0}, smooth = Smooth.None));
            connect(airGapDC.flange,inertiaRotor.flange_a) annotation(Line(points = {{10, -1.83697e-15},{35, -1.83697e-15},{35,1.22465e-15},{60,1.22465e-15}}, color = {0,0,0}, smooth = Smooth.None));
          end DC_ElectricalExcited;
          model DC_SeriesExcited "Series excited linear DC machine"
            extends Machines.Interfaces.PartialBasicDCMachine(wNominal(start = (1410 * 2 * pi) / 60), final turnsRatio = (VaNominal - (Ra + Re) * IaNominal) / (wNominal * Le * IaNominal));
            Components.AirGapDC airGapDC(final turnsRatio = turnsRatio, final Le = Le) annotation(Placement(transformation(origin = {0,0}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
            parameter Modelica.SIunits.Resistance Re(start = 0.01) "warm field excitation resistance" annotation(Dialog(group = "Excitation"));
            parameter Modelica.SIunits.Inductance Le(start = 0.0005) "total field excitation inductance" annotation(Dialog(group = "Excitation"));
            output Modelica.SIunits.Voltage ve = pin_ep.v - pin_en.v "Field excitation voltage";
            output Modelica.SIunits.Current ie = pin_ep.i "Field excitation current";
            Modelica.Electrical.Analog.Basic.Resistor re(final R = Re, final T_ref = 293.15, final alpha = 0, final useHeatPort = false, final T = re.T_ref) annotation(Placement(transformation(origin = { -50, -40}, extent = {{10,10},{ -10, -10}}, rotation = 180)));
            Modelica.Electrical.Analog.Interfaces.PositivePin pin_ep annotation(Placement(transformation(extent = {{ -110,70},{ -90,50}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin pin_en annotation(Placement(transformation(extent = {{ -90, -50},{ -110, -70}}, rotation = 0)));
            annotation(defaultComponentName = "dcse", Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -100, -10},{ -105, -9},{ -109, -5},{ -110,0},{ -109,5},{ -105,9},{ -100,10}}, color = {0,0,255}),Line(points = {{ -100, -30},{ -105, -29},{ -109, -25},{ -110, -20},{ -109, -15},{ -105, -11},{ -100, -10}}, color = {0,0,255}),Line(points = {{ -100,10},{ -105,11},{ -109,15},{ -110,20},{ -109,25},{ -105,29},{ -100,30}}, color = {0,0,255}),Line(points = {{ -100,50},{ -100,30}}, color = {0,0,255}),Line(points = {{ -100, -30},{ -100, -50}}, color = {0,0,255})}), Documentation(info = "<HTML>
<b>Model of a DC Machine with Series excitation.</b><br>
Armature resistance and inductance are modeled directly after the armature pins, then using a <i>AirGapDC</i> model.<br>
Only losses in armature and excitation resistance are taken into account. No saturation is modelled.<br>
Series excitation has to be connected by the user's external circuit.
<br><b>Default values for machine's parameters (a realistic example) are:</b><br>
<table>
<tr>
<td valign=\"top\">stator's moment of inertia</td>
<td valign=\"top\">0.29</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">rotor's moment of inertia</td>
<td valign=\"top\">0.15</td><td valign=\"top\">kg.m2</td>
</tr>
<tr>
<td valign=\"top\">nominal armature voltage</td>
<td valign=\"top\">100</td><td valign=\"top\">V</td>
</tr>
<tr>
<td valign=\"top\">nominal armature current</td>
<td valign=\"top\">100</td><td valign=\"top\">A</td>
</tr>
<tr>
<td valign=\"top\">nominal torque</td>
<td valign=\"top\">63.66</td><td valign=\"top\">Nm</td>
</tr>
<tr>
<td valign=\"top\">nominal speed</td>
<td valign=\"top\">1410</td><td valign=\"top\">rpm</td>
</tr>
<tr>
<td valign=\"top\">nominal mechanical output</td>
<td valign=\"top\">9.4</td><td valign=\"top\">kW</td>
</tr>
<tr>
<td valign=\"top\">efficiency</td>
<td valign=\"top\">94.0</td><td valign=\"top\">% only armature</td>
</tr>
<tr>
<td valign=\"top\">armature resistance</td>
<td valign=\"top\">0.05</td><td valign=\"top\">Ohm in warm condition</td>
</tr>
<tr>
<td valign=\"top\">armature inductance</td>
<td valign=\"top\">0.0015</td><td valign=\"top\">H</td>
</tr>
<tr>
<td valign=\"top\">excitation resistance</td>
<td valign=\"top\">0.01</td><td valign=\"top\">Ohm in warm condition</td>
</tr>
<tr>
<td valign=\"top\">excitation inductance</td>
<td valign=\"top\">0.0005</td><td valign=\"top\">H</td>
</tr>
</table>
Armature resistance resp. inductance include resistance resp. inductance of commutating pole winding and
compensation windig, if present.<br>
Parameter nominal armature voltage includes voltage drop of series excitation;<br>
but for output the voltage is splitted into:<br>
va = armature voltage without voltage drop of series excitation<br>
ve = voltage drop of series excitation
</HTML>"));

          equation 
            assert(VaNominal > (Ra + Re) * IaNominal, "VaNominal has to be > (Ra+Re)*IaNominal");
            connect(re.p,pin_ep) annotation(Line(points = {{ -60, -40},{ -80, -40},{ -80,60},{ -100,60}}, color = {0,0,255}));
            connect(pin_en,airGapDC.pin_en) annotation(Line(points = {{ -100, -60},{ -10, -60},{ -10, -10}}, color = {0,0,255}));
            connect(re.n,airGapDC.pin_ep) annotation(Line(points = {{ -40, -40},{10, -40},{10, -10}}, color = {0,0,255}));
            connect(airGapDC.pin_ap,la.n) annotation(Line(points = {{10,10},{10,35},{10,35},{10,60}}, color = {0,0,255}, smooth = Smooth.None));
            connect(airGapDC.pin_an,pin_an) annotation(Line(points = {{ -10,10},{ -10,60},{ -60,60},{ -60,100}}, color = {0,0,255}, smooth = Smooth.None));
            connect(airGapDC.support,internalSupport) annotation(Line(points = {{ -10,1.83697e-15},{ -46,1.83697e-15},{ -46,0},{ -90,0},{ -90, -100},{20, -100}}, color = {0,0,0}, smooth = Smooth.None));
            connect(airGapDC.flange,inertiaRotor.flange_a) annotation(Line(points = {{10, -1.83697e-15},{36, -1.83697e-15},{36,1.22465e-15},{60,1.22465e-15}}, color = {0,0,0}, smooth = Smooth.None));
          end DC_SeriesExcited;
        end DCMachines;
        package Transformers "Library for technical 3phase transformers"
          extends Modelica.Icons.Library2;
          annotation(Documentation(info = "<HTML>
This package contains components to model technical threephase transformers:
<ul>
<li>Transformer: transformer model to choose connection / vector group</li>
<li>Yy: Transformers with primary primary Y / secondary y</li>
<li>Yd: Transformers with primary primary Y / secondary d</li>
<li>Yz: Transformers with primary primary Y / secondary zig-zag</li>
<li>Dy: Transformers with primary primary D / secondary y</li>
<li>Dd: Transformers with primary D / secondary d</li>
<li>Dz: Transformers with primary D / secondary zig-zag</li>
</ul>
<p>
Transformers are modeled by an ideal transformer, adding primary and secondary winding resistances and stray inductances.<br>
All transformers extend from the base model <i>PartialTransformer</i>, adding the primary and secondary connection.<br>
<b>VectorGroup</b> defines the phase shift between primary and secondary voltages, expressed by a number phase shift/30 degree
(i.e. the hour on a clock face). Therefore each transformer is identified by two characters and a two-digit number,
e.g. Yd11 ... primary connection Y (star), secondfary connection d (delta), vector group 11 (phase shift 330 degree)<br>
With the \"supermodel\" <i>Tranformer</i>&nbsp; the user may choose primary and secondary connection as well as the vector group.<br>
It calculates winding ratio as well as primary and secondary winding resistances and stray inductances,
distributing them equally to primary and secondary winding, from the following parameters:
</p>
<ul>
<li>nominal frequency</li>
<li>primary voltage (RMS line-to-line)</li>
<li>secondary voltage (RMS line-to-line)</li>
<li>nominal apparent power</li>
<li>impedance voltage drop</li>
<li>short-circuit copper losses</li>
</ul>
The <b>impedance voltage drop</b> indicates the (absolute value of the) voltage drop at nominal load (current) as well as
the voltage we have to apply to the primary winding to achieve nominal current in the short-circuited secondary winding.
<p>
<b>Please pay attention</b> to proper grounding of the primary and secondary part of the whole circuit.<br>
The primary and secondary starpoint are available as connectors, if the connection is not delta (D or d).<br>
<b>In some cases (Yy or Yz) it may be necessary to ground one of the transformer's starpoints
even though the source's and/or load's starpoint are grounded; you may use a reasonable high earthing resistance.</b>
</p>
<b>Limitations and assumptions:</b><br>
<ul>
<li>number of phases is limited to 3, therefore definition as a constant m=3</li>
<li>symmetry of the 3 phases resp. limbs</li>
<li>temperature dependency is neglected, i.e. resistances are constant</li>
<li>saturation is neglected, i.e. inductances are constant</li>
<li>magnetizing current is neglected</li>
<li>magnetizing losses are neglected</li>
<li>additional (stray) losses are neglected</li>
</ul>
<b>Further development:</b>
<ul>
<li>modeling magnetizing current, including saturation</li>
<li>temperature dependency of winding resistances</li>
</ul>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
</dl>
<p>
Copyright &copy; 1998-2009, Modelica Association and Anton Haumer.
</p>
<p>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p>
</HTML>", revisions = "<HTML>
  <ul>
  <li> v1.0.0 2006/11/19 Anton Haumer<br>
       first stable release</li>
  </ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -100,30},{20, -90}}, lineColor = {0,0,255}),Ellipse(extent = {{ -40,30},{80, -90}}, lineColor = {0,0,255})}));
          package Yy "Transformers: primary Y / secondary y"
            extends Modelica.Icons.Library2;
            annotation(Documentation(info = "<HTML>
This package contains transformers primary Y connected / secondary y connected in all possbile vector groups.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2009, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.0.0 2006/11/19 Anton Haumer</li>
  </ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{0,40},{0, -100}}, lineColor = {0,0,255}, textString = "Yy")}));
            model Yy00 "Transformer Yy0"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Yy0
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Yy00");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(Placement(transformation(origin = { -10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(Placement(transformation(extent = {{ -60, -110},{ -40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(Placement(transformation(origin = {10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(Placement(transformation(extent = {{40, -110},{60, -90}}, rotation = 0)));
              annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

            equation 
              connect(star1.pin_n,starpoint1) annotation(Line(points = {{ -10, -90},{ -10, -100},{ -50, -100}}, color = {0,0,255}));
              connect(star2.pin_n,starpoint2) annotation(Line(points = {{10, -90},{10, -100},{50, -100}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(core.plug_n1,star1.plug_p) annotation(Line(points = {{ -10, -5},{ -10, -37.5},{ -10, -70},{ -10, -70}}, color = {0,0,255}));
              connect(core.plug_n2,core.plug_p3) annotation(Line(points = {{10,4},{10, -4}}, color = {0,0,255}));
              connect(core.plug_p2,l2sigma.plug_p) annotation(Line(points = {{10,10},{10,20},{50,20},{50,0}}, color = {0,0,255}));
              connect(core.plug_n3,star2.plug_p) annotation(Line(points = {{10, -10},{10, -70},{10, -70}}, color = {0,0,255}));
            end Yy00;
            model Yy02 "Transformer Yy2"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Yy2
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Yy02");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(Placement(transformation(origin = { -10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(Placement(transformation(extent = {{ -60, -110},{ -40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(Placement(transformation(origin = {10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(Placement(transformation(extent = {{40, -110},{60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot2(final m = m) annotation(Placement(transformation(extent = {{50, -30},{30, -10}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(star1.pin_n,starpoint1) annotation(Line(points = {{ -10, -90},{ -10, -100},{ -50, -100}}, color = {0,0,255}));
              connect(star2.pin_n,starpoint2) annotation(Line(points = {{10, -90},{10, -100},{50, -100}}, color = {0,0,255}));
              connect(l2sigma.plug_p,Rot2.plug_p) annotation(Line(points = {{50,0},{50, -20}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(core.plug_n2,core.plug_p3) annotation(Line(points = {{10,4},{10, -4},{10, -4}}, color = {0,0,255}));
              connect(core.plug_n1,star1.plug_p) annotation(Line(points = {{ -10, -5},{ -10, -70}}, color = {0,0,255}));
              connect(core.plug_n3,Rot2.plug_n) annotation(Line(points = {{10, -10},{10, -20},{30, -20}}, color = {0,0,255}));
              connect(core.plug_p2,star2.plug_p) annotation(Line(points = {{10,10},{20,10},{20, -70},{10, -70}}, color = {0,0,255}));
            end Yy02;
            model Yy04 "Transformer Yy4"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Yy4
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Yy04");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(Placement(transformation(origin = { -10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(Placement(transformation(extent = {{ -60, -110},{ -40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(Placement(transformation(origin = {10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(Placement(transformation(extent = {{40, -110},{60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot2(final m = m) annotation(Placement(transformation(extent = {{30,10},{50,30}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(star1.pin_n,starpoint1) annotation(Line(points = {{ -10, -90},{ -10, -100},{ -50, -100}}, color = {0,0,255}));
              connect(star2.pin_n,starpoint2) annotation(Line(points = {{10, -90},{10, -100},{50, -100}}, color = {0,0,255}));
              connect(Rot2.plug_n,l2sigma.plug_p) annotation(Line(points = {{50,20},{50,0}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(core.plug_n1,star1.plug_p) annotation(Line(points = {{ -10, -5},{ -10, -70}}, color = {0,0,255}));
              connect(core.plug_n2,core.plug_p3) annotation(Line(points = {{10,4},{10, -4}}, color = {0,0,255}));
              connect(core.plug_p2,Rot2.plug_p) annotation(Line(points = {{10,10},{10,20},{30,20}}, color = {0,0,255}));
              connect(core.plug_n3,star2.plug_p) annotation(Line(points = {{10, -10},{10, -70},{10, -70}}, color = {0,0,255}));
            end Yy04;
            model Yy06 "Transformer Yy6"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Yy6
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Yy06");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(Placement(transformation(origin = { -10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(Placement(transformation(extent = {{ -60, -110},{ -40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(Placement(transformation(origin = {10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(Placement(transformation(extent = {{40, -110},{60, -90}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(star1.pin_n,starpoint1) annotation(Line(points = {{ -10, -90},{ -10, -100},{ -50, -100}}, color = {0,0,255}));
              connect(star2.pin_n,starpoint2) annotation(Line(points = {{10, -90},{10, -100},{50, -100}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(core.plug_n1,star1.plug_p) annotation(Line(points = {{ -10, -5},{ -10, -70}}, color = {0,0,255}));
              connect(core.plug_n2,core.plug_p3) annotation(Line(points = {{10,4},{10, -4}}, color = {0,0,255}));
              connect(l2sigma.plug_p,core.plug_n3) annotation(Line(points = {{50,0},{50, -20},{10, -20},{10, -10}}, color = {0,0,255}));
              connect(core.plug_p2,star2.plug_p) annotation(Line(points = {{10,10},{20,10},{20, -70},{10, -70}}, color = {0,0,255}));
            end Yy06;
            model Yy08 "Transformer Yy8"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Yy8
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Yy08");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(Placement(transformation(origin = { -10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(Placement(transformation(extent = {{ -60, -110},{ -40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(Placement(transformation(origin = {10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(Placement(transformation(extent = {{40, -110},{60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot2(final m = m) annotation(Placement(transformation(extent = {{50,10},{30,30}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(star1.pin_n,starpoint1) annotation(Line(points = {{ -10, -90},{ -10, -100},{ -50, -100}}, color = {0,0,255}));
              connect(star2.pin_n,starpoint2) annotation(Line(points = {{10, -90},{10, -100},{50, -100}}, color = {0,0,255}));
              connect(Rot2.plug_p,l2sigma.plug_p) annotation(Line(points = {{50,20},{50,0}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(core.plug_n1,star1.plug_p) annotation(Line(points = {{ -10, -5},{ -10, -70}}, color = {0,0,255}));
              connect(core.plug_n2,core.plug_p3) annotation(Line(points = {{10,4},{10, -4}}, color = {0,0,255}));
              connect(core.plug_p2,Rot2.plug_n) annotation(Line(points = {{10,10},{10,20},{30,20}}, color = {0,0,255}));
              connect(core.plug_n3,star2.plug_p) annotation(Line(points = {{10, -10},{10, -70},{10, -70},{10, -70}}, color = {0,0,255}));
            end Yy08;
            model Yy10 "Transformer Yy10"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Yy10
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Yy10");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(Placement(transformation(origin = { -10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(Placement(transformation(extent = {{ -60, -110},{ -40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(Placement(transformation(origin = {10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(Placement(transformation(extent = {{40, -110},{60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot2(final m = m) annotation(Placement(transformation(extent = {{30, -30},{50, -10}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(star1.pin_n,starpoint1) annotation(Line(points = {{ -10, -90},{ -10, -100},{ -50, -100}}, color = {0,0,255}));
              connect(star2.pin_n,starpoint2) annotation(Line(points = {{10, -90},{10, -100},{50, -100}}, color = {0,0,255}));
              connect(l2sigma.plug_p,Rot2.plug_n) annotation(Line(points = {{50,0},{50, -20}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(core.plug_n1,star1.plug_p) annotation(Line(points = {{ -10, -5},{ -10, -70}}, color = {0,0,255}));
              connect(core.plug_n2,core.plug_p3) annotation(Line(points = {{10,4},{10, -4}}, color = {0,0,255}));
              connect(core.plug_n3,Rot2.plug_p) annotation(Line(points = {{10, -10},{10, -20},{30, -20}}, color = {0,0,255}));
              connect(core.plug_p2,star2.plug_p) annotation(Line(points = {{10,10},{20,10},{20, -70},{10, -70}}, color = {0,0,255}));
            end Yy10;
          end Yy;
          package Yd "Transformers: primary Y / secondary d"
            extends Modelica.Icons.Library2;
            annotation(Documentation(info = "<HTML>
This package contains transformers primary Y connected / secondary d connected in all possbile vector groups.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2009, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.0.0 2006/11/19 Anton Haumer</li>
  </ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{0,40},{0, -100}}, lineColor = {0,0,255}, textString = "Yd")}));
            model Yd01 "Transformer Yd1"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Yd1
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Yd01");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(Placement(transformation(origin = { -10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(Placement(transformation(extent = {{ -60, -110},{ -40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta2(final m = m) annotation(Placement(transformation(extent = {{50, -30},{30, -10}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(star1.pin_n,starpoint1) annotation(Line(points = {{ -10, -90},{ -10, -100},{ -50, -100}}, color = {0,0,255}));
              connect(Delta2.plug_p,r2.plug_n) annotation(Line(points = {{50, -20},{90, -20},{90,0}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(core.plug_n1,star1.plug_p) annotation(Line(points = {{ -10, -5},{ -10, -70},{ -10, -70}}, color = {0,0,255}));
              connect(core.plug_n2,core.plug_p3) annotation(Line(points = {{10,4},{10, -4}}, color = {0,0,255}));
              connect(core.plug_n3,Delta2.plug_n) annotation(Line(points = {{10, -10},{10, -20},{30, -20}}, color = {0,0,255}));
              connect(core.plug_p2,l2sigma.plug_p) annotation(Line(points = {{10,10},{10,20},{50,20},{50,0}}, color = {0,0,255}));
            end Yd01;
            model Yd03 "Transformer Yd3"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Yd3
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Yd03");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(Placement(transformation(origin = { -10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(Placement(transformation(extent = {{ -60, -110},{ -40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta2(final m = m) annotation(Placement(transformation(extent = {{30,10},{50,30}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot2(final m = m) annotation(Placement(transformation(extent = {{50, -30},{30, -10}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(star1.pin_n,starpoint1) annotation(Line(points = {{ -10, -90},{ -10, -100},{ -50, -100}}, color = {0,0,255}));
              connect(Delta2.plug_n,r2.plug_n) annotation(Line(points = {{50,20},{90,20},{90,0}}, color = {0,0,255}));
              connect(l2sigma.plug_p,Rot2.plug_p) annotation(Line(points = {{50,0},{50, -20}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(core.plug_n1,star1.plug_p) annotation(Line(points = {{ -10, -5},{ -10, -5},{ -10, -70}}, color = {0,0,255}));
              connect(core.plug_n2,core.plug_p3) annotation(Line(points = {{10,4},{10, -4}}, color = {0,0,255}));
              connect(core.plug_p2,Delta2.plug_p) annotation(Line(points = {{10,10},{10,20},{30,20}}, color = {0,0,255}));
              connect(core.plug_n3,Rot2.plug_n) annotation(Line(points = {{10, -10},{10, -20},{30, -20}}, color = {0,0,255}));
            end Yd03;
            model Yd05 "Transformer Yd5"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Yd5
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Yd05");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(Placement(transformation(origin = { -10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(Placement(transformation(extent = {{ -60, -110},{ -40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta2(final m = m) annotation(Placement(transformation(extent = {{30,10},{50,30}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(star1.pin_n,starpoint1) annotation(Line(points = {{ -10, -90},{ -10, -100},{ -50, -100}}, color = {0,0,255}));
              connect(Delta2.plug_n,r2.plug_n) annotation(Line(points = {{50,20},{90,20},{90,0}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(core.plug_n1,star1.plug_p) annotation(Line(points = {{ -10, -5},{ -10, -5},{ -10, -70}}, color = {0,0,255}));
              connect(core.plug_n2,core.plug_p3) annotation(Line(points = {{10,4},{10, -4}}, color = {0,0,255}));
              connect(core.plug_n3,l2sigma.plug_p) annotation(Line(points = {{10, -10},{10, -20},{50, -20},{50,0}}, color = {0,0,255}));
              connect(core.plug_p2,Delta2.plug_p) annotation(Line(points = {{10,10},{10,20},{30,20}}, color = {0,0,255}));
            end Yd05;
            model Yd07 "Transformer Yd7"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Yd7
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Yd07");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(Placement(transformation(origin = { -10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(Placement(transformation(extent = {{ -60, -110},{ -40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta2(final m = m) annotation(Placement(transformation(extent = {{50,10},{30,30}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(star1.pin_n,starpoint1) annotation(Line(points = {{ -10, -90},{ -10, -100},{ -50, -100}}, color = {0,0,255}));
              connect(Delta2.plug_p,r2.plug_n) annotation(Line(points = {{50,20},{90,20},{90,0}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(core.plug_n1,star1.plug_p) annotation(Line(points = {{ -10, -5},{ -10, -70}}, color = {0,0,255}));
              connect(core.plug_n2,core.plug_p3) annotation(Line(points = {{10,4},{10, -4}}, color = {0,0,255}));
              connect(core.plug_p2,Delta2.plug_n) annotation(Line(points = {{10,10},{10,20},{30,20}}, color = {0,0,255}));
              connect(l2sigma.plug_p,core.plug_n3) annotation(Line(points = {{50,0},{50, -20},{10, -20},{10, -10}}, color = {0,0,255}));
            end Yd07;
            model Yd09 "Transformer Yd9"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Yd9
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Yd09");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(Placement(transformation(origin = { -10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(Placement(transformation(extent = {{ -60, -110},{ -40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta2(final m = m) annotation(Placement(transformation(extent = {{30, -30},{50, -10}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot2(final m = m) annotation(Placement(transformation(extent = {{50,10},{30,30}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(star1.pin_n,starpoint1) annotation(Line(points = {{ -10, -90},{ -10, -100},{ -50, -100}}, color = {0,0,255}));
              connect(Rot2.plug_p,l2sigma.plug_p) annotation(Line(points = {{50,20},{50,0}}, color = {0,0,255}));
              connect(Delta2.plug_n,r2.plug_n) annotation(Line(points = {{50, -20},{90, -20},{90,0}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(core.plug_n1,star1.plug_p) annotation(Line(points = {{ -10, -5},{ -10, -70}}, color = {0,0,255}));
              connect(core.plug_n2,core.plug_p3) annotation(Line(points = {{10,4},{10, -4}}, color = {0,0,255}));
              connect(core.plug_p2,Rot2.plug_n) annotation(Line(points = {{10,10},{10,20},{30,20}}, color = {0,0,255}));
              connect(core.plug_n3,Delta2.plug_p) annotation(Line(points = {{10, -10},{10, -20},{30, -20}}, color = {0,0,255}));
            end Yd09;
            model Yd11 "Transformer Yd11"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Yd11
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Yd11");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(Placement(transformation(origin = { -10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(Placement(transformation(extent = {{ -60, -110},{ -40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta2(final m = m) annotation(Placement(transformation(extent = {{30, -30},{50, -10}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(star1.pin_n,starpoint1) annotation(Line(points = {{ -10, -90},{ -10, -100},{ -50, -100}}, color = {0,0,255}));
              connect(Delta2.plug_n,r2.plug_n) annotation(Line(points = {{50, -20},{90, -20},{90,0}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(core.plug_n1,star1.plug_p) annotation(Line(points = {{ -10, -5},{ -10, -5},{ -10, -70}}, color = {0,0,255}));
              connect(core.plug_n2,core.plug_p3) annotation(Line(points = {{10,4},{10, -4}}, color = {0,0,255}));
              connect(core.plug_n3,Delta2.plug_p) annotation(Line(points = {{10, -10},{10, -20},{30, -20}}, color = {0,0,255}));
              connect(core.plug_p2,l2sigma.plug_p) annotation(Line(points = {{10,10},{10,20},{50,20},{50,0}}, color = {0,0,255}));
            end Yd11;
          end Yd;
          package Yz "Transformers: primary Y / secondary zig-zag"
            extends Modelica.Icons.Library2;
            annotation(Documentation(info = "<HTML>
This package contains transformers primary Y connected / secondary zig-zag connected in all possbile vector groups.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2009, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.0.0 2006/11/19 Anton Haumer</li>
  </ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{0,40},{0, -100}}, lineColor = {0,0,255}, textString = "Yz")}));
            model Yz01 "Transformer Yz1"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Yz1
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Yz01");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(Placement(transformation(origin = { -10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(Placement(transformation(extent = {{ -60, -110},{ -40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(Placement(transformation(origin = {10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(Placement(transformation(extent = {{40, -110},{60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot21(final m = m) annotation(Placement(transformation(origin = {30,0}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot22(final m = m) annotation(Placement(transformation(origin = {40,20}, extent = {{ -10,10},{10, -10}}, rotation = 180)));
              annotation(Diagram(graphics));

            equation 
              connect(star1.pin_n,starpoint1) annotation(Line(points = {{ -10, -90},{ -10, -100},{ -50, -100}}, color = {0,0,255}));
              connect(star2.pin_n,starpoint2) annotation(Line(points = {{10, -90},{10, -100},{50, -100}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(core.plug_n1,star1.plug_p) annotation(Line(points = {{ -10, -5},{ -10, -70},{ -10, -70}}, color = {0,0,255}));
              connect(core.plug_p2,Rot21.plug_n) annotation(Line(points = {{10,10},{30,10}}, color = {0,0,255}));
              connect(core.plug_p3,Rot21.plug_p) annotation(Line(points = {{10, -4},{20, -4},{20, -10},{30, -10}}, color = {0,0,255}));
              connect(l2sigma.plug_p,Rot22.plug_p) annotation(Line(points = {{50,0},{50,20}}, color = {0,0,255}));
              connect(Rot22.plug_n,core.plug_n2) annotation(Line(points = {{30,20},{20,20},{20,4},{10,4}}, color = {0,0,255}));
              connect(core.plug_n3,star2.plug_p) annotation(Line(points = {{10, -10},{10, -70},{10, -70}}, color = {0,0,255}));
            end Yz01;
            model Yz03 "Transformer Yz3"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Yz3
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Yz03");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(Placement(transformation(origin = { -10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(Placement(transformation(extent = {{ -60, -110},{ -40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(Placement(transformation(origin = {10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(Placement(transformation(extent = {{40, -110},{60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot21(final m = m) annotation(Placement(transformation(origin = {30,0}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot22(final m = m) annotation(Placement(transformation(origin = {40,20}, extent = {{ -10,10},{10, -10}}, rotation = 180)));
              annotation(Diagram(graphics));

            equation 
              connect(star1.pin_n,starpoint1) annotation(Line(points = {{ -10, -90},{ -10, -100},{ -50, -100}}, color = {0,0,255}));
              connect(star2.pin_n,starpoint2) annotation(Line(points = {{10, -90},{10, -100},{50, -100}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(core.plug_n1,star1.plug_p) annotation(Line(points = {{ -10, -5},{ -10, -70}}, color = {0,0,255}));
              connect(core.plug_p2,Rot21.plug_p) annotation(Line(points = {{10,10},{30,10}}, color = {0,0,255}));
              connect(core.plug_p3,Rot21.plug_n) annotation(Line(points = {{10, -4},{20, -4},{20, -10},{30, -10}}, color = {0,0,255}));
              connect(l2sigma.plug_p,Rot22.plug_p) annotation(Line(points = {{50,0},{50,20}}, color = {0,0,255}));
              connect(Rot22.plug_n,core.plug_n2) annotation(Line(points = {{30,20},{20,20},{20,4},{10,4}}, color = {0,0,255}));
              connect(core.plug_n3,star2.plug_p) annotation(Line(points = {{10, -10},{10, -70},{10, -70}}, color = {0,0,255}));
            end Yz03;
            model Yz05 "Transformer Yz5"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Yz5
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Yz05");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(Placement(transformation(origin = { -10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(Placement(transformation(extent = {{ -60, -110},{ -40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(Placement(transformation(origin = {10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(Placement(transformation(extent = {{40, -110},{60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot21(final m = m) annotation(Placement(transformation(origin = {30,0}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
              annotation(Diagram(graphics));

            equation 
              connect(star1.pin_n,starpoint1) annotation(Line(points = {{ -10, -90},{ -10, -100},{ -50, -100}}, color = {0,0,255}));
              connect(star2.pin_n,starpoint2) annotation(Line(points = {{10, -90},{10, -100},{50, -100}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(core.plug_n1,star1.plug_p) annotation(Line(points = {{ -10, -5},{ -10, -5},{ -10, -70}}, color = {0,0,255}));
              connect(core.plug_p2,Rot21.plug_n) annotation(Line(points = {{10,10},{30,10}}, color = {0,0,255}));
              connect(core.plug_p3,Rot21.plug_p) annotation(Line(points = {{10, -4},{20, -4},{20, -10},{30, -10}}, color = {0,0,255}));
              connect(core.plug_n3,star2.plug_p) annotation(Line(points = {{10, -10},{10, -70}}, color = {0,0,255}));
              connect(l2sigma.plug_p,core.plug_n2) annotation(Line(points = {{50,0},{50,20},{20,20},{20,4},{10,4}}, color = {0,0,255}));
            end Yz05;
            model Yz07 "Transformer Yz7"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Yz7
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Yz07");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(Placement(transformation(origin = { -10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(Placement(transformation(extent = {{ -60, -110},{ -40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(Placement(transformation(origin = {10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(Placement(transformation(extent = {{40, -110},{60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot21(final m = m) annotation(Placement(transformation(origin = {30,0}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
              annotation(Diagram(graphics));

            equation 
              connect(star1.pin_n,starpoint1) annotation(Line(points = {{ -10, -90},{ -10, -100},{ -50, -100}}, color = {0,0,255}));
              connect(star2.pin_n,starpoint2) annotation(Line(points = {{10, -90},{10, -100},{50, -100}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(core.plug_n1,star1.plug_p) annotation(Line(points = {{ -10, -5},{ -10, -70}}, color = {0,0,255}));
              connect(core.plug_p2,Rot21.plug_p) annotation(Line(points = {{10,10},{30,10}}, color = {0,0,255}));
              connect(core.plug_p3,Rot21.plug_n) annotation(Line(points = {{10, -4},{20, -4},{20, -10},{30, -10}}, color = {0,0,255}));
              connect(l2sigma.plug_p,core.plug_n2) annotation(Line(points = {{50,0},{50,20},{20,20},{20,4},{10,4}}, color = {0,0,255}));
              connect(core.plug_n3,star2.plug_p) annotation(Line(points = {{10, -10},{10, -70}}, color = {0,0,255}));
            end Yz07;
            model Yz09 "Transformer Yz9"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Yz9
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Yz09");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(Placement(transformation(origin = { -10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(Placement(transformation(extent = {{ -60, -110},{ -40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(Placement(transformation(origin = {10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(Placement(transformation(extent = {{40, -110},{60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot21(final m = m) annotation(Placement(transformation(origin = {30,0}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot22(final m = m) annotation(Placement(transformation(origin = {40,20}, extent = {{10,10},{ -10, -10}}, rotation = 180)));
              annotation(Diagram(graphics));

            equation 
              connect(star1.pin_n,starpoint1) annotation(Line(points = {{ -10, -90},{ -10, -100},{ -50, -100}}, color = {0,0,255}));
              connect(star2.pin_n,starpoint2) annotation(Line(points = {{10, -90},{10, -100},{50, -100}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(core.plug_n1,star1.plug_p) annotation(Line(points = {{ -10, -5},{ -10, -70},{ -10, -70}}, color = {0,0,255}));
              connect(core.plug_p2,Rot21.plug_n) annotation(Line(points = {{10,10},{30,10}}, color = {0,0,255}));
              connect(core.plug_p3,Rot21.plug_p) annotation(Line(points = {{10, -4},{20, -4},{20, -10},{30, -10}}, color = {0,0,255}));
              connect(core.plug_n3,star2.plug_p) annotation(Line(points = {{10, -10},{10, -70},{10, -70}}, color = {0,0,255}));
              connect(l2sigma.plug_p,Rot22.plug_n) annotation(Line(points = {{50,0},{50,20},{50,20}}, color = {0,0,255}));
              connect(Rot22.plug_p,core.plug_n2) annotation(Line(points = {{30,20},{20,20},{20,4},{10,4}}, color = {0,0,255}));
            end Yz09;
            model Yz11 "Transformer Yz11"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Yz11
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Yz11");
              Modelica.Electrical.MultiPhase.Basic.Star star1(final m = m) annotation(Placement(transformation(origin = { -10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint1 annotation(Placement(transformation(extent = {{ -60, -110},{ -40, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(Placement(transformation(origin = {10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(Placement(transformation(extent = {{40, -110},{60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot21(final m = m) annotation(Placement(transformation(origin = {30,0}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot22(final m = m) annotation(Placement(transformation(origin = {40,20}, extent = {{10,10},{ -10, -10}}, rotation = 180)));
              annotation(Diagram(graphics));

            equation 
              connect(star1.pin_n,starpoint1) annotation(Line(points = {{ -10, -90},{ -10, -100},{ -50, -100}}, color = {0,0,255}));
              connect(star2.pin_n,starpoint2) annotation(Line(points = {{10, -90},{10, -100},{50, -100}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(core.plug_n1,star1.plug_p) annotation(Line(points = {{ -10, -5},{ -10, -70}}, color = {0,0,255}));
              connect(core.plug_p2,Rot21.plug_p) annotation(Line(points = {{10,10},{30,10}}, color = {0,0,255}));
              connect(core.plug_p3,Rot21.plug_n) annotation(Line(points = {{10, -4},{20, -4},{20, -10},{30, -10}}, color = {0,0,255}));
              connect(l2sigma.plug_p,Rot22.plug_n) annotation(Line(points = {{50,0},{50,20}}, color = {0,0,255}));
              connect(Rot22.plug_p,core.plug_n2) annotation(Line(points = {{30,20},{20,20},{20,4},{10,4}}, color = {0,0,255}));
              connect(core.plug_n3,star2.plug_p) annotation(Line(points = {{10, -10},{10, -70},{10, -70}}, color = {0,0,255}));
            end Yz11;
          end Yz;
          package Dy "Transformers: primary D / secondary y"
            extends Modelica.Icons.Library2;
            annotation(Documentation(info = "<HTML>
This package contains transformers primary D connected / secondary y connected in all possbile vector groups.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2009, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.0.0 2006/11/19 Anton Haumer</li>
  </ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{0,40},{0, -100}}, lineColor = {0,0,255}, textString = "Dy")}));
            model Dy01 "Transformer Dy1"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Dy1
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Dy01");
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(Placement(transformation(origin = {10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(Placement(transformation(extent = {{40, -110},{60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(Placement(transformation(extent = {{ -50, -30},{ -30, -10}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot2(final m = m) annotation(Placement(transformation(extent = {{50, -30},{30, -10}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(star2.pin_n,starpoint2) annotation(Line(points = {{10, -90},{10, -100},{50, -100}}, color = {0,0,255}));
              connect(l2sigma.plug_p,Rot2.plug_p) annotation(Line(points = {{50,0},{50, -20}}, color = {0,0,255}));
              connect(Delta1.plug_p,r1.plug_p) annotation(Line(points = {{ -50, -20},{ -90, -20},{ -90,0}}, color = {0,0,255}));
              connect(Delta1.plug_n,core.plug_n1) annotation(Line(points = {{ -30, -20},{ -10, -20},{ -10, -5}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(core.plug_n2,core.plug_p3) annotation(Line(points = {{10,4},{10, -4}}, color = {0,0,255}));
              connect(core.plug_n3,Rot2.plug_n) annotation(Line(points = {{10, -10},{10, -20},{30, -20}}, color = {0,0,255}));
              connect(core.plug_p2,star2.plug_p) annotation(Line(points = {{10,10},{20,10},{20, -70},{10, -70}}, color = {0,0,255}));
            end Dy01;
            model Dy03 "Transformer Dy3"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Dy3
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Dy03");
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(Placement(transformation(origin = {10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(Placement(transformation(extent = {{40, -110},{60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(Placement(transformation(extent = {{ -50, -30},{ -30, -10}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot2(final m = m) annotation(Placement(transformation(extent = {{30,10},{50,30}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(star2.pin_n,starpoint2) annotation(Line(points = {{10, -90},{10, -100},{50, -100}}, color = {0,0,255}));
              connect(Rot2.plug_n,l2sigma.plug_p) annotation(Line(points = {{50,20},{50,0}}, color = {0,0,255}));
              connect(Delta1.plug_p,r1.plug_p) annotation(Line(points = {{ -50, -20},{ -90, -20},{ -90,0}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(Delta1.plug_n,core.plug_n1) annotation(Line(points = {{ -30, -20},{ -10, -20},{ -10, -5}}, color = {0,0,255}));
              connect(core.plug_p2,Rot2.plug_p) annotation(Line(points = {{10,10},{10,20},{30,20}}, color = {0,0,255}));
              connect(core.plug_n2,core.plug_p3) annotation(Line(points = {{10,4},{10, -4}}, color = {0,0,255}));
              connect(core.plug_n3,star2.plug_p) annotation(Line(points = {{10, -10},{10, -70}}, color = {0,0,255}));
            end Dy03;
            model Dy05 "Transformer Dy5"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Dy5
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Dy05");
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(Placement(transformation(origin = {10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(Placement(transformation(extent = {{40, -110},{60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(Placement(transformation(extent = {{ -50, -30},{ -30, -10}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(star2.pin_n,starpoint2) annotation(Line(points = {{10, -90},{10, -100},{50, -100}}, color = {0,0,255}));
              connect(Delta1.plug_p,r1.plug_p) annotation(Line(points = {{ -50, -20},{ -90, -20},{ -90,0}}, color = {0,0,255}));
              connect(Delta1.plug_n,core.plug_n1) annotation(Line(points = {{ -30, -20},{ -10, -20},{ -10, -5}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(core.plug_n2,core.plug_p3) annotation(Line(points = {{10,4},{10, -4}}, color = {0,0,255}));
              connect(l2sigma.plug_p,core.plug_n3) annotation(Line(points = {{50,0},{50, -20},{10, -20},{10, -10}}, color = {0,0,255}));
              connect(core.plug_p2,star2.plug_p) annotation(Line(points = {{10,10},{20,10},{20, -70},{10, -70}}, color = {0,0,255}));
            end Dy05;
            model Dy07 "Transformer Dy7"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Dy7
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Dy07");
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(Placement(transformation(origin = {10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(Placement(transformation(extent = {{40, -110},{60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(Placement(transformation(extent = {{ -50, -30},{ -30, -10}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot2(final m = m) annotation(Placement(transformation(extent = {{50,10},{30,30}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(star2.pin_n,starpoint2) annotation(Line(points = {{10, -90},{10, -100},{50, -100}}, color = {0,0,255}));
              connect(r1.plug_p,Delta1.plug_p) annotation(Line(points = {{ -90,0},{ -90, -20},{ -50, -20}}, color = {0,0,255}));
              connect(Rot2.plug_p,l2sigma.plug_p) annotation(Line(points = {{50,20},{50,0}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(Delta1.plug_n,core.plug_n1) annotation(Line(points = {{ -30, -20},{ -10, -20},{ -10, -5}}, color = {0,0,255}));
              connect(core.plug_n2,core.plug_p3) annotation(Line(points = {{10,4},{10, -4}}, color = {0,0,255}));
              connect(core.plug_n3,star2.plug_p) annotation(Line(points = {{10, -10},{10, -70}}, color = {0,0,255}));
              connect(core.plug_p2,Rot2.plug_n) annotation(Line(points = {{10,10},{10,20},{30,20}}, color = {0,0,255}));
            end Dy07;
            model Dy09 "Transformer Dy9"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Dy9
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Dy09");
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(Placement(transformation(origin = {10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(Placement(transformation(extent = {{40, -110},{60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(Placement(transformation(extent = {{ -50, -30},{ -30, -10}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot2(final m = m) annotation(Placement(transformation(extent = {{30, -30},{50, -10}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(star2.pin_n,starpoint2) annotation(Line(points = {{10, -90},{10, -100},{50, -100}}, color = {0,0,255}));
              connect(r1.plug_p,Delta1.plug_p) annotation(Line(points = {{ -90,0},{ -90, -20},{ -50, -20}}, color = {0,0,255}));
              connect(l2sigma.plug_p,Rot2.plug_n) annotation(Line(points = {{50,0},{50, -20}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5},{ -10,5}}, color = {0,0,255}));
              connect(Delta1.plug_n,core.plug_n1) annotation(Line(points = {{ -30, -20},{ -10, -20},{ -10, -5}}, color = {0,0,255}));
              connect(core.plug_n2,core.plug_p3) annotation(Line(points = {{10,4},{10, -4}}, color = {0,0,255}));
              connect(core.plug_n3,Rot2.plug_p) annotation(Line(points = {{10, -10},{10, -20},{30, -20}}, color = {0,0,255}));
              connect(core.plug_p2,star2.plug_p) annotation(Line(points = {{10,10},{20,10},{20, -70},{10, -70}}, color = {0,0,255}));
            end Dy09;
            model Dy11 "Transformer Dy11"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Dy11
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Dy11");
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(Placement(transformation(origin = {10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(Placement(transformation(extent = {{40, -110},{60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(Placement(transformation(extent = {{ -50, -30},{ -30, -10}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(star2.pin_n,starpoint2) annotation(Line(points = {{10, -90},{10, -100},{50, -100}}, color = {0,0,255}));
              connect(r1.plug_p,Delta1.plug_p) annotation(Line(points = {{ -90,0},{ -90, -20},{ -50, -20}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(Delta1.plug_n,core.plug_n1) annotation(Line(points = {{ -30, -20},{ -10, -20},{ -10, -5}}, color = {0,0,255}));
              connect(core.plug_n2,core.plug_p3) annotation(Line(points = {{10,4},{10, -4}}, color = {0,0,255}));
              connect(core.plug_n3,star2.plug_p) annotation(Line(points = {{10, -10},{10, -70}}, color = {0,0,255}));
              connect(core.plug_p2,l2sigma.plug_p) annotation(Line(points = {{10,10},{10,20},{50,20},{50,0}}, color = {0,0,255}));
            end Dy11;
          end Dy;
          package Dd "Transformers: primary D / secondary d"
            extends Modelica.Icons.Library2;
            annotation(Documentation(info = "<HTML>
This package contains transformers primary D connected / secondary d connected in all possbile vector groups.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2009, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.0.0 2006/11/19 Anton Haumer</li>
  </ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{0,40},{0, -100}}, lineColor = {0,0,255}, textString = "Dd")}));
            model Dd00 "Transformer Dd0"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Dd0
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Dd00");
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(Placement(transformation(extent = {{ -50, -30},{ -30, -10}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta2(final m = m) annotation(Placement(transformation(extent = {{50, -30},{30, -10}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(r1.plug_p,Delta1.plug_p) annotation(Line(points = {{ -90,0},{ -90, -20},{ -50, -20}}, color = {0,0,255}));
              connect(Delta2.plug_p,r2.plug_n) annotation(Line(points = {{50, -20},{90, -20},{90,0}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(Delta1.plug_n,core.plug_n1) annotation(Line(points = {{ -30, -20},{ -10, -20},{ -10, -5}}, color = {0,0,255}));
              connect(core.plug_n2,core.plug_p3) annotation(Line(points = {{10,4},{10, -4}}, color = {0,0,255}));
              connect(core.plug_p2,l2sigma.plug_p) annotation(Line(points = {{10,10},{10,20},{50,20},{50,0}}, color = {0,0,255}));
              connect(core.plug_n3,Delta2.plug_n) annotation(Line(points = {{10, -10},{10, -20},{30, -20}}, color = {0,0,255}));
            end Dd00;
            model Dd02 "Transformer Dd2"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Dd2
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Dd02");
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(Placement(transformation(extent = {{ -50, -30},{ -30, -10}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta2(final m = m) annotation(Placement(transformation(extent = {{30,10},{50,30}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot2(final m = m) annotation(Placement(transformation(extent = {{50, -30},{30, -10}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(r1.plug_p,Delta1.plug_p) annotation(Line(points = {{ -90,0},{ -90, -20},{ -50, -20}}, color = {0,0,255}));
              connect(Delta2.plug_n,r2.plug_n) annotation(Line(points = {{50,20},{90,20},{90,0}}, color = {0,0,255}));
              connect(l2sigma.plug_p,Rot2.plug_p) annotation(Line(points = {{50,0},{50, -20}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(Delta1.plug_n,core.plug_n1) annotation(Line(points = {{ -30, -20},{ -10, -20},{ -10, -5}}, color = {0,0,255}));
              connect(core.plug_n2,core.plug_p3) annotation(Line(points = {{10,4},{10, -4}}, color = {0,0,255}));
              connect(core.plug_p2,Delta2.plug_p) annotation(Line(points = {{10,10},{10,20},{30,20}}, color = {0,0,255}));
              connect(core.plug_n3,Rot2.plug_n) annotation(Line(points = {{10, -10},{10, -20},{30, -20}}, color = {0,0,255}));
            end Dd02;
            model Dd04 "Transformer Dd4"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Dd4
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Dd04");
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(Placement(transformation(extent = {{ -50, -30},{ -30, -10}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta2(final m = m) annotation(Placement(transformation(extent = {{30,10},{50,30}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(r1.plug_p,Delta1.plug_p) annotation(Line(points = {{ -90,0},{ -90, -20},{ -50, -20}}, color = {0,0,255}));
              connect(Delta2.plug_n,r2.plug_n) annotation(Line(points = {{50,20},{90,20},{90,0}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(Delta1.plug_n,core.plug_n1) annotation(Line(points = {{ -30, -20},{ -10, -20},{ -10, -5}}, color = {0,0,255}));
              connect(core.plug_n2,core.plug_p3) annotation(Line(points = {{10,4},{10,1},{10,1},{10, -4}}, color = {0,0,255}));
              connect(core.plug_p2,Delta2.plug_p) annotation(Line(points = {{10,10},{10,20},{30,20}}, color = {0,0,255}));
              connect(core.plug_n3,l2sigma.plug_p) annotation(Line(points = {{10, -10},{10, -20},{50, -20},{50,0}}, color = {0,0,255}));
            end Dd04;
            model Dd06 "Transformer Dd6"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Dd6
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Dd06");
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(Placement(transformation(extent = {{ -50, -30},{ -30, -10}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta2(final m = m) annotation(Placement(transformation(extent = {{50,10},{30,30}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(Delta2.plug_p,r2.plug_n) annotation(Line(points = {{50,20},{90,20},{90,0}}, color = {0,0,255}));
              connect(Delta1.plug_p,r1.plug_p) annotation(Line(points = {{ -50, -20},{ -90, -20},{ -90,0}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(Delta1.plug_n,core.plug_n1) annotation(Line(points = {{ -30, -20},{ -10, -20},{ -10, -5}}, color = {0,0,255}));
              connect(core.plug_n2,core.plug_p3) annotation(Line(points = {{10,4},{10, -4}}, color = {0,0,255}));
              connect(Delta2.plug_n,core.plug_p2) annotation(Line(points = {{30,20},{10,20},{10,10}}, color = {0,0,255}));
              connect(core.plug_n3,l2sigma.plug_p) annotation(Line(points = {{10, -10},{10, -20},{50, -20},{50,0}}, color = {0,0,255}));
            end Dd06;
            model Dd08 "Transformer Dd8"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Dd8
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Dd08");
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(Placement(transformation(extent = {{ -50, -30},{ -30, -10}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot2(final m = m) annotation(Placement(transformation(extent = {{50,10},{30,30}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta2(final m = m) annotation(Placement(transformation(extent = {{30, -30},{50, -10}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(r1.plug_p,Delta1.plug_p) annotation(Line(points = {{ -90,0},{ -90, -20},{ -50, -20}}, color = {0,0,255}));
              connect(Delta2.plug_n,r2.plug_n) annotation(Line(points = {{50, -20},{90, -20},{90,0}}, color = {0,0,255}));
              connect(Rot2.plug_p,l2sigma.plug_p) annotation(Line(points = {{50,20},{50,0}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(core.plug_n1,Delta1.plug_n) annotation(Line(points = {{ -10, -5},{ -10, -20},{ -30, -20}}, color = {0,0,255}));
              connect(core.plug_n2,core.plug_p3) annotation(Line(points = {{10,4},{10, -4}}, color = {0,0,255}));
              connect(core.plug_p2,Rot2.plug_n) annotation(Line(points = {{10,10},{10,20},{30,20}}, color = {0,0,255}));
              connect(core.plug_n3,Delta2.plug_p) annotation(Line(points = {{10, -10},{10, -20},{30, -20}}, color = {0,0,255}));
            end Dd08;
            model Dd10 "Transformer Dd10"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Dd10
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Dd10");
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(Placement(transformation(extent = {{ -50, -30},{ -30, -10}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta2(final m = m) annotation(Placement(transformation(extent = {{30, -30},{50, -10}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(r1.plug_p,Delta1.plug_p) annotation(Line(points = {{ -90,0},{ -90, -20},{ -50, -20}}, color = {0,0,255}));
              connect(Delta2.plug_n,r2.plug_n) annotation(Line(points = {{50, -20},{90, -20},{90,0}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(Delta1.plug_n,core.plug_n1) annotation(Line(points = {{ -30, -20},{ -10, -20},{ -10, -5}}, color = {0,0,255}));
              connect(core.plug_n2,core.plug_p3) annotation(Line(points = {{10,4},{10, -4}}, color = {0,0,255}));
              connect(core.plug_p2,l2sigma.plug_p) annotation(Line(points = {{10,10},{10,20},{50,20},{50,0}}, color = {0,0,255}));
              connect(core.plug_n3,Delta2.plug_p) annotation(Line(points = {{10, -10},{10, -20},{30, -20}}, color = {0,0,255}));
            end Dd10;
          end Dd;
          package Dz "Transformers: primary D / secondary ziag-zag"
            extends Modelica.Icons.Library2;
            annotation(Documentation(info = "<HTML>
This package contains transformers primary D connected / secondary d connected in all possbile vector groups.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2009, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.0.0 2006/11/19 Anton Haumer</li>
  </ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{0,40},{0, -100}}, lineColor = {0,0,255}, textString = "Dz")}));
            model Dz00 "Transformer Dz0"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Dz0
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Dz00");
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(Placement(transformation(origin = {10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(Placement(transformation(extent = {{40, -110},{60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot21(final m = m) annotation(Placement(transformation(origin = {30,0}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot22(final m = m) annotation(Placement(transformation(origin = {40,20}, extent = {{ -10,10},{10, -10}}, rotation = 180)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(Placement(transformation(extent = {{ -50, -30},{ -30, -10}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(star2.pin_n,starpoint2) annotation(Line(points = {{10, -90},{10, -100},{50, -100}}, color = {0,0,255}));
              connect(r1.plug_p,Delta1.plug_p) annotation(Line(points = {{ -90,0},{ -90, -20},{ -50, -20}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5},{ -10,5}}, color = {0,0,255}));
              connect(Delta1.plug_n,core.plug_n1) annotation(Line(points = {{ -30, -20},{ -10, -20},{ -10, -5}}, color = {0,0,255}));
              connect(core.plug_p2,Rot21.plug_n) annotation(Line(points = {{10,10},{30,10}}, color = {0,0,255}));
              connect(core.plug_p3,Rot21.plug_p) annotation(Line(points = {{10, -4},{20, -4},{20, -10},{30, -10}}, color = {0,0,255}));
              connect(l2sigma.plug_p,Rot22.plug_p) annotation(Line(points = {{50,0},{50,20}}, color = {0,0,255}));
              connect(Rot22.plug_n,core.plug_n2) annotation(Line(points = {{30,20},{20,20},{20,4},{10,4}}, color = {0,0,255}));
              connect(core.plug_n3,star2.plug_p) annotation(Line(points = {{10, -10},{10, -70},{10, -70}}, color = {0,0,255}));
            end Dz00;
            model Dz02 "Transformer Dz2"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Dz2
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Dz02");
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(Placement(transformation(origin = {10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(Placement(transformation(extent = {{40, -110},{60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot21(final m = m) annotation(Placement(transformation(origin = {30,0}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot22(final m = m) annotation(Placement(transformation(origin = {40,20}, extent = {{ -10,10},{10, -10}}, rotation = 180)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(Placement(transformation(extent = {{ -50, -30},{ -30, -10}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(star2.pin_n,starpoint2) annotation(Line(points = {{10, -90},{10, -100},{50, -100}}, color = {0,0,255}));
              connect(r1.plug_p,Delta1.plug_p) annotation(Line(points = {{ -90,0},{ -90, -20},{ -50, -20}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(Delta1.plug_n,core.plug_n1) annotation(Line(points = {{ -30, -20},{ -10, -20},{ -10, -5}}, color = {0,0,255}));
              connect(core.plug_p2,Rot21.plug_p) annotation(Line(points = {{10,10},{30,10}}, color = {0,0,255}));
              connect(core.plug_p3,Rot21.plug_n) annotation(Line(points = {{10, -4},{20, -4},{20, -10},{30, -10}}, color = {0,0,255}));
              connect(l2sigma.plug_p,Rot22.plug_p) annotation(Line(points = {{50,0},{50,20}}, color = {0,0,255}));
              connect(Rot22.plug_n,core.plug_n2) annotation(Line(points = {{30,20},{20,20},{20,4},{10,4}}, color = {0,0,255}));
              connect(core.plug_n3,star2.plug_p) annotation(Line(points = {{10, -10},{10, -70}}, color = {0,0,255}));
            end Dz02;
            model Dz04 "Transformer Dz4"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Dz4
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Dz04");
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(Placement(transformation(origin = {10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(Placement(transformation(extent = {{40, -110},{60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot21(final m = m) annotation(Placement(transformation(origin = {30,0}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(Placement(transformation(extent = {{ -50, -30},{ -30, -10}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(star2.pin_n,starpoint2) annotation(Line(points = {{10, -90},{10, -100},{50, -100}}, color = {0,0,255}));
              connect(r1.plug_p,Delta1.plug_p) annotation(Line(points = {{ -90,0},{ -90, -20},{ -50, -20}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(Delta1.plug_n,core.plug_n1) annotation(Line(points = {{ -30, -20},{ -10, -20},{ -10, -5}}, color = {0,0,255}));
              connect(core.plug_p2,Rot21.plug_n) annotation(Line(points = {{10,10},{30,10}}, color = {0,0,255}));
              connect(core.plug_p3,Rot21.plug_p) annotation(Line(points = {{10, -4},{20, -4},{20, -10},{30, -10}}, color = {0,0,255}));
              connect(l2sigma.plug_p,core.plug_n2) annotation(Line(points = {{50,0},{50,20},{20,20},{20,4},{10,4}}, color = {0,0,255}));
              connect(core.plug_n3,star2.plug_p) annotation(Line(points = {{10, -10},{10, -70},{10, -70}}, color = {0,0,255}));
            end Dz04;
            model Dz06 "Transformer Dz6"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Dz6
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Dz06");
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(Placement(transformation(origin = {10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(Placement(transformation(extent = {{40, -110},{60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot21(final m = m) annotation(Placement(transformation(origin = {30,0}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(Placement(transformation(extent = {{ -50, -30},{ -30, -10}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(star2.pin_n,starpoint2) annotation(Line(points = {{10, -90},{10, -100},{50, -100}}, color = {0,0,255}));
              connect(r1.plug_p,Delta1.plug_p) annotation(Line(points = {{ -90,0},{ -90, -20},{ -50, -20}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(Delta1.plug_n,core.plug_n1) annotation(Line(points = {{ -30, -20},{ -10, -20},{ -10, -5}}, color = {0,0,255}));
              connect(core.plug_p2,Rot21.plug_p) annotation(Line(points = {{10,10},{30,10}}, color = {0,0,255}));
              connect(core.plug_p3,Rot21.plug_n) annotation(Line(points = {{10, -4},{20, -4},{20, -10},{30, -10}}, color = {0,0,255}));
              connect(core.plug_n3,star2.plug_p) annotation(Line(points = {{10, -10},{10, -70}}, color = {0,0,255}));
              connect(l2sigma.plug_p,core.plug_n2) annotation(Line(points = {{50,0},{50,20},{20,20},{20,4},{10,4}}, color = {0,0,255}));
            end Dz06;
            model Dz08 "Transformer Dz8"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Dz8
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Dz08");
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(Placement(transformation(origin = {10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(Placement(transformation(extent = {{40, -110},{60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot21(final m = m) annotation(Placement(transformation(origin = {30,0}, extent = {{ -10,10},{10, -10}}, rotation = 90)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot22(final m = m) annotation(Placement(transformation(origin = {40,20}, extent = {{10,10},{ -10, -10}}, rotation = 180)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(Placement(transformation(extent = {{ -50, -30},{ -30, -10}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(star2.pin_n,starpoint2) annotation(Line(points = {{10, -90},{10, -100},{50, -100}}, color = {0,0,255}));
              connect(r1.plug_p,Delta1.plug_p) annotation(Line(points = {{ -90,0},{ -90, -20},{ -50, -20}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5},{ -10,5}}, color = {0,0,255}));
              connect(Delta1.plug_n,core.plug_n1) annotation(Line(points = {{ -30, -20},{ -10, -20},{ -10, -5}}, color = {0,0,255}));
              connect(core.plug_p2,Rot21.plug_n) annotation(Line(points = {{10,10},{30,10}}, color = {0,0,255}));
              connect(core.plug_p3,Rot21.plug_p) annotation(Line(points = {{10, -4},{20, -4},{20, -10},{30, -10}}, color = {0,0,255}));
              connect(core.plug_n3,star2.plug_p) annotation(Line(points = {{10, -10},{10, -70},{10, -70}}, color = {0,0,255}));
              connect(l2sigma.plug_p,Rot22.plug_n) annotation(Line(points = {{50,0},{50,20},{50,20}}, color = {0,0,255}));
              connect(Rot22.plug_p,core.plug_n2) annotation(Line(points = {{30,20},{20,20},{20,4},{10,4}}, color = {0,0,255}));
            end Dz08;
            model Dz10 "Transformer Dz10"
              annotation(defaultComponentName = "transformer", Documentation(info = "<html>
Transformer Dz10
</html>"));
              extends Machines.BasicMachines.Components.BasicTransformer(final VectorGroup = "Dz10");
              Modelica.Electrical.MultiPhase.Basic.Star star2(final m = m) annotation(Placement(transformation(origin = {10, -80}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
              Modelica.Electrical.Analog.Interfaces.NegativePin starpoint2 annotation(Placement(transformation(extent = {{40, -110},{60, -90}}, rotation = 0)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot21(final m = m) annotation(Placement(transformation(origin = {30,0}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
              Modelica.Electrical.MultiPhase.Basic.Delta Rot22(final m = m) annotation(Placement(transformation(origin = {40,20}, extent = {{10,10},{ -10, -10}}, rotation = 180)));
              Modelica.Electrical.MultiPhase.Basic.Delta Delta1(final m = m) annotation(Placement(transformation(extent = {{ -50, -30},{ -30, -10}}, rotation = 0)));
              annotation(Diagram(graphics));

            equation 
              connect(star2.pin_n,starpoint2) annotation(Line(points = {{10, -90},{10, -100},{50, -100}}, color = {0,0,255}));
              connect(r1.plug_p,Delta1.plug_p) annotation(Line(points = {{ -90,0},{ -90, -20},{ -50, -20}}, color = {0,0,255}));
              connect(l1sigma.plug_n,core.plug_p1) annotation(Line(points = {{ -50,0},{ -50,20},{ -10,20},{ -10,5}}, color = {0,0,255}));
              connect(Delta1.plug_n,core.plug_n1) annotation(Line(points = {{ -30, -20},{ -10, -20},{ -10, -5}}, color = {0,0,255}));
              connect(core.plug_p2,Rot21.plug_p) annotation(Line(points = {{10,10},{30,10}}, color = {0,0,255}));
              connect(core.plug_p3,Rot21.plug_n) annotation(Line(points = {{10, -4},{20, -4},{20, -10},{30, -10}}, color = {0,0,255}));
              connect(core.plug_n3,star2.plug_p) annotation(Line(points = {{10, -10},{10, -70},{10, -70}}, color = {0,0,255}));
              connect(l2sigma.plug_p,Rot22.plug_n) annotation(Line(points = {{50,0},{50,20}}, color = {0,0,255}));
              connect(Rot22.plug_p,core.plug_n2) annotation(Line(points = {{30,20},{20,20},{20,4},{10,4}}, color = {0,0,255}));
            end Dz10;
          end Dz;
        end Transformers;
        package Components "Machine components like AirGaps"
          extends Modelica.Icons.Library;
          annotation(Documentation(info = "<HTML>
This package contains components for modeling electrical machines, specially threephase induction machines, based on space phasor theory.
These models use package SpacePhasors.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2009, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.00 2004/09/16 Anton Haumer</li>
  <li> v1.02 2004/09/19 Anton Haumer<br>
       added AirGapDC models</li>
  <li> v1.2  2004/10/27 Anton Haumer<br>
       fixed a bug with support (formerly bearing)</li>
  <li> v1.52 2005/10/12 Anton Haumer<br>
       added electrical excitation</li>
  <li> v1.53 Beta 2005/10/14 Anton Haumer<br>
       introduced unsymmetrical DamperCage for Synchronous Machines</li>
  </ul>
</HTML>"));
          partial model PartialAirGap "Partial airgap model"
            parameter Integer m = 3 "number of phases";
            parameter Integer p(min = 1) "number of pole pairs";
            output Modelica.SIunits.Torque tauElectrical;
            Modelica.SIunits.Angle gamma "Rotor displacement angle";
            Modelica.SIunits.Current i_ss[2] "Stator current space phasor with respect to the stator fixed frame";
            Modelica.SIunits.Current i_sr[2] "Stator current space phasor with respect to the rotor fixed frame";
            Modelica.SIunits.Current i_rs[2] "Rotor current space phasor with respect to the stator fixed frame";
            Modelica.SIunits.Current i_rr[2] "Rotor current space phasor with respect to the rotor fixed frame";
            Modelica.SIunits.MagneticFlux psi_ms[2] "Magnetizing flux phasor with respect to the stator fixed frame";
            Modelica.SIunits.MagneticFlux psi_mr[2] "Magnetizing flux phasor with respect to the rotor fixed frame";
            Real RotationMatrix[2,2] "matrix of rotation from rotor to stator";
          public 
            Modelica.Mechanics.Rotational.Interfaces.Flange_a flange annotation(Placement(transformation(extent = {{ -10,110},{10,90}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_a support "support at which the reaction torque is acting" annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}}, rotation = 0)));
            Machines.Interfaces.SpacePhasor spacePhasor_s annotation(Placement(transformation(extent = {{ -110,90},{ -90,110}}, rotation = 0)));
            Machines.Interfaces.SpacePhasor spacePhasor_r annotation(Placement(transformation(extent = {{90,90},{110,110}}, rotation = 0)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -90,90},{90, -92}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -80,80},{80, -80}}, lineColor = {0,0,255}),Rectangle(extent = {{ -10,90},{10, -80}}, lineColor = {0,0,0}, fillPattern = FillPattern.VerticalCylinder, fillColor = {128,128,128}),Text(extent = {{ -150, -90},{150, -150}}, lineColor = {0,0,255}, textString = "%name")}), Documentation(info = "<HTML>
Partial model of the airgap, using only equations.
</HTML>"));

          equation 
            gamma = p * (flange.phi - support.phi);
            RotationMatrix = {{ +cos(gamma), -sin(gamma)},{ +sin(gamma), +cos(gamma)}};
            i_ss = spacePhasor_s.i_;
            i_ss = RotationMatrix * i_sr;
            i_rr = spacePhasor_r.i_;
            i_rs = RotationMatrix * i_rr;
            spacePhasor_s.v_ = der(psi_ms);
            spacePhasor_r.v_ = der(psi_mr);
            tauElectrical = m / 2 * p * (spacePhasor_s.i_[2] * psi_ms[1] - spacePhasor_s.i_[1] * psi_ms[2]);
            flange.tau =  -tauElectrical;
            support.tau = tauElectrical;
          end PartialAirGap;
          model AirGapS "Airgap in stator-fixed coordinate system"
            parameter Modelica.SIunits.Inductance Lm "main field inductance";
            extends PartialAirGap;
            Modelica.SIunits.Current i_ms[2] "Magnetizing current space phasor with respect to the stator fixed frame";
          protected 
            parameter Modelica.SIunits.Inductance L[2,2] = {{Lm,0},{0,Lm}} "inductance matrix";
            annotation(defaultComponentName = "airGap", Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -80,40},{0, -40}}, lineColor = {0,0,0}, textString = "S"),Text(extent = {{ -150, -90},{150, -150}}, lineColor = {0,0,255}, textString = "%name")}), Documentation(info = "<HTML>
Model of the airgap in stator-fixed coordinate system, using only equations.
</HTML>"));

          equation 
            i_ms = i_ss + i_rs;
            psi_ms = L * i_ms;
            psi_mr = transpose(RotationMatrix) * psi_ms;
          end AirGapS;
          model AirGapR "Airgap in rotor-fixed coordinate system"
            parameter Modelica.SIunits.Inductance Lmd "main field inductance d-axis";
            parameter Modelica.SIunits.Inductance Lmq "main field inductance q-axis";
            extends PartialAirGap;
            Modelica.SIunits.Current i_mr[2] "Magnetizing current space phasor with respect to the rotor fixed frame";
          protected 
            parameter Modelica.SIunits.Inductance L[2,2] = {{Lmd,0},{0,Lmq}} "inductance matrix";
            annotation(defaultComponentName = "airGap", Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{0,40},{80, -40}}, lineColor = {0,0,0}, textString = "R"),Text(extent = {{ -150, -90},{150, -150}}, lineColor = {0,0,255}, textString = "%name")}), Documentation(info = "<HTML>
Model of the airgap in rotor-fixed coordinate system, using only equations.
</HTML>"));

          equation 
            i_mr = i_sr + i_rr;
            psi_mr = L * i_mr;
            psi_ms = RotationMatrix * psi_mr;
          end AirGapR;
          model SquirrelCage "Squirrel Cage"
            parameter Modelica.SIunits.Inductance Lrsigma "rotor stray inductance per phase translated to stator";
            parameter Modelica.SIunits.Resistance Rr "warm rotor resistance per phase translated to stator";
            Machines.Interfaces.SpacePhasor spacePhasor_r annotation(Placement(transformation(extent = {{ -110,90},{ -90,110}}, rotation = 0)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -60,45},{ -30,75}}, lineColor = {0,0,255}),Ellipse(extent = {{ -30,45},{0,75}}, lineColor = {0,0,255}),Ellipse(extent = {{0,45},{30,75}}, lineColor = {0,0,255}),Ellipse(extent = {{30,45},{60,75}}, lineColor = {0,0,255}),Rectangle(extent = {{ -60,30},{60,60}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -100,60},{ -60,60}}, color = {0,0,255}),Line(points = {{60,60},{80,60},{80,40}}, color = {0,0,255}),Rectangle(extent = {{60,40},{100, -40}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{80, -40},{80, -60}}, color = {0,0,255}),Line(points = {{60, -60},{100, -60}}, color = {0,0,255}),Line(points = {{70, -80},{90, -80}}, color = {0,0,255}),Line(points = {{66, -70},{94, -70}}, color = {0,0,255}),Text(extent = {{ -150, -90},{150, -150}}, lineColor = {0,0,255}, textString = "%name"),Line(points = {{ -100,90},{ -100,60}}, color = {0,0,255})}), Documentation(info = "<HTML>
Model of a squirrel cage / damper cage in two axis.
</HTML>"));

          equation 
            spacePhasor_r.v_ = Rr * spacePhasor_r.i_ + Lrsigma * der(spacePhasor_r.i_);
          end SquirrelCage;
          model DamperCage "Squirrel Cage"
            parameter Modelica.SIunits.Inductance Lrsigmad "stray inductance in d-axis per phase translated to stator";
            parameter Modelica.SIunits.Inductance Lrsigmaq "stray inductance in q-axis per phase translated to stator";
            parameter Modelica.SIunits.Resistance Rrd "warm resistance in d-axis per phase translated to stator";
            parameter Modelica.SIunits.Resistance Rrq "warm resistance in q-axis per phase translated to stator";
            Machines.Interfaces.SpacePhasor spacePhasor_r annotation(Placement(transformation(extent = {{ -110,90},{ -90,110}}, rotation = 0)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{60,60},{100,60}}, color = {0,0,255}),Line(points = {{70,40},{90,40}}, color = {0,0,255}),Line(points = {{66,50},{94,50}}, color = {0,0,255}),Text(extent = {{ -150, -90},{150, -150}}, lineColor = {0,0,255}, textString = "%name"),Ellipse(extent = {{ -60,90},{ -40,70}}, lineColor = {0,0,255}),Ellipse(extent = {{ -40,90},{ -20,70}}, lineColor = {0,0,255}),Ellipse(extent = {{ -20,90},{0,70}}, lineColor = {0,0,255}),Line(points = {{ -80,0},{ -80, -20}}, color = {0,0,255}),Rectangle(extent = {{20,90},{60,70}}, lineColor = {0,0,255}),Line(points = {{60,80},{80,80},{80,60}}, color = {0,0,255}),Line(points = {{ -100,90},{ -100,80},{ -60,80}}, color = {0,0,255}),Line(points = {{ -100,90},{ -100,80},{ -80,80},{ -80,60}}, color = {0,0,255}),Ellipse(extent = {{ -90,60},{ -70,40}}, lineColor = {0,0,255}),Ellipse(extent = {{ -90,40},{ -70,20}}, lineColor = {0,0,255}),Ellipse(extent = {{ -90,20},{ -70,0}}, lineColor = {0,0,255}),Rectangle(extent = {{ -80,0},{ -70,60}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,80},{20,80}}, color = {0,0,255}),Rectangle(extent = {{ -90, -20},{ -70, -60}}, lineColor = {0,0,255}),Line(points = {{ -100, -80},{ -60, -80}}, color = {0,0,255}),Line(points = {{ -90, -100},{ -70, -100}}, color = {0,0,255}),Line(points = {{ -94, -90},{ -66, -90}}, color = {0,0,255}),Line(points = {{ -80, -60},{ -80, -80}}, color = {0,0,255}),Rectangle(extent = {{0,70},{ -60,80}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
Model of an unsymmetrical damper cage cage in two axis.
</HTML>"));

          equation 
            spacePhasor_r.v_[1] = Rrd * spacePhasor_r.i_[1] + Lrsigmad * der(spacePhasor_r.i_[1]);
            spacePhasor_r.v_[2] = Rrq * spacePhasor_r.i_[2] + Lrsigmaq * der(spacePhasor_r.i_[2]);
          end DamperCage;
          model ElectricalExcitation "Electrical excitation"
            parameter Real turnsRatio(start = 1) "stator current / excitation current";
            Modelica.SIunits.Current ie "excitation current";
            Modelica.SIunits.Voltage ve "excitation voltage";
            Machines.Interfaces.SpacePhasor spacePhasor_r annotation(Placement(transformation(extent = {{ -110,90},{ -90,110}}, rotation = 0)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -150, -90},{150, -150}}, lineColor = {0,0,255}, textString = "%name"),Polygon(points = {{ -90,100},{ -70,106},{ -70,94},{ -90,100}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -70,40},{10, -40}}, lineColor = {0,0,255}),Ellipse(extent = {{ -10,40},{70, -40}}, lineColor = {0,0,255}),Line(points = {{ -30,0},{ -54,32},{ -50,20},{ -44,26},{ -54,32}}, color = {0,0,255}),Line(points = {{ -30,0},{ -54, -32},{ -50, -20},{ -44, -26},{ -54, -32}}, color = {0,0,255}),Line(points = {{ -54,32},{ -54,100},{ -70,100}}, color = {0,0,255}),Line(points = {{90,100},{30,100},{30,40}}, color = {0,0,255}),Line(points = {{30, -40},{30, -100},{90, -100}}, color = {0,0,255})}), Documentation(info = "<HTML>
Model of an electrical excitation, converting excitation to space phasor.
</HTML>"));
            Modelica.Electrical.Analog.Interfaces.PositivePin pin_ep annotation(Placement(transformation(extent = {{90,110},{110,90}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin pin_en annotation(Placement(transformation(extent = {{110, -110},{90, -90}}, rotation = 0)));

          equation 
            pin_ep.i + pin_en.i = 0;
            ie =  +pin_ep.i;
            ve = pin_ep.v - pin_en.v;
            spacePhasor_r.i_ = { -ie * turnsRatio,0};
            ve = (spacePhasor_r.v_[1] * turnsRatio * 3) / 2;
          end ElectricalExcitation;
          model PermanentMagnet "Permanent magnet excitation"
            parameter Modelica.SIunits.Current Ie "equivalent excitation current";
            Machines.Interfaces.SpacePhasor spacePhasor_r annotation(Placement(transformation(extent = {{ -110,90},{ -90,110}}, rotation = 0)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -60,60},{60,20}}, lineColor = {255,0,0}, fillColor = {255,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -60, -20},{60, -60}}, lineColor = {0,255,0}, fillColor = {0,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -60,40},{60,0}}, lineColor = {255,0,0}, fillColor = {255,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{ -90,100},{ -70,106},{ -70,94},{ -90,100}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -60,0},{60, -40}}, lineColor = {0,255,0}, pattern = LinePattern.None, lineThickness = 0.5, fillColor = {0,255,0}, fillPattern = FillPattern.Solid),Line(points = {{ -60, -40},{ -60,0}}, color = {0,255,0}),Line(points = {{60, -40},{60,0}}, color = {0,255,0}),Line(points = {{0,100},{ -70,100}}, color = {0,0,255}),Line(points = {{0,100},{0,60}}, color = {255,0,0}),Text(extent = {{ -150, -90},{150, -150}}, lineColor = {0,0,255}, textString = "%name")}), Documentation(info = "<HTML>
Model of a permanent magnet excitation, characterized by an equivalent excitation current.
</HTML>"));

          equation 
            spacePhasor_r.i_ = { -Ie,0};
          end PermanentMagnet;
          partial model PartialAirGapDC "Partial airgap model of a DC machine"
            parameter Real turnsRatio "ratio of armature turns over number of turns of the excitation winding";
            Modelica.SIunits.AngularVelocity w "Angluar velocity";
            Modelica.SIunits.Voltage vei "Voltage drop across field excitation inductance";
            Modelica.SIunits.Current ie "Excitation current";
            Modelica.SIunits.MagneticFlux psi_e "Excitation flux";
            Modelica.SIunits.Voltage vai "Induced armature voltage";
            Modelica.SIunits.Current ia "Armature current";
            output Modelica.SIunits.Torque tauElectrical;
            Modelica.Mechanics.Rotational.Interfaces.Flange_a flange annotation(Placement(transformation(extent = {{ -10,110},{10,90}}, rotation = 0)));
            Modelica.Mechanics.Rotational.Interfaces.Flange_a support "support at which the reaction torque is acting" annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.PositivePin pin_ap annotation(Placement(transformation(extent = {{ -110,110},{ -90,90}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.PositivePin pin_ep annotation(Placement(transformation(extent = {{90,110},{110,90}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin pin_an annotation(Placement(transformation(extent = {{ -110, -110},{ -90, -90}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin pin_en annotation(Placement(transformation(extent = {{90, -110},{110, -90}}, rotation = 0)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -90,90},{90, -92}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -80,80},{80, -80}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -10,90},{10, -80}}, lineColor = {0,0,0}, fillPattern = FillPattern.VerticalCylinder, fillColor = {128,128,128}),Text(extent = {{0,40},{80, -40}}, lineColor = {0,0,0}, textString = "E"),Text(extent = {{ -150, -100},{150, -160}}, lineColor = {0,0,255}, textString = "%name"),Text(extent = {{ -80,40},{0, -40}}, lineColor = {0,0,0}, textString = "A")}), Documentation(info = "<HTML>
Linear model of the airgap (without saturation effects) of a DC machine, using only equations.<br>
Induced excitation voltage is calculated from der(flux), where flux is defined by excitation inductance times excitation current.<br>
Induced armature voltage is calculated from flux times angular velocity.
</HTML>"));

          equation 
            vai = pin_ap.v - pin_an.v;
            ia =  +pin_ap.i;
            ia =  -pin_an.i;
            vei = pin_ep.v - pin_en.v;
            ie =  +pin_ep.i;
            ie =  -pin_en.i;
            vei = der(psi_e);
            w = der(flange.phi) - der(support.phi);
            vai = turnsRatio * psi_e * w;
            tauElectrical = turnsRatio * psi_e * ia;
            flange.tau =  -tauElectrical;
            support.tau = tauElectrical;
          end PartialAirGapDC;
          model AirGapDC "Linear airgap model of a DC machine"
            extends PartialAirGapDC;
            parameter Modelica.SIunits.Inductance Le "excitation inductance";
            annotation(defaultComponentName = "airGap", Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -150, -100},{150, -160}}, lineColor = {0,0,255}, textString = "%name")}), Documentation(info = "<HTML>
Linear model of the airgap (without saturation effects) of a DC machine, using only equations.<br>
Induced excitation voltage is calculated from der(flux), where flux is defined by excitation inductance times excitation current.<br>
Induced armature voltage is calculated from flux times angular velocity.
</HTML>"));

          equation 
            psi_e = Le * ie;
          end AirGapDC;
          partial model BasicTransformer "Partial model of threephase transformer"
            constant Integer m(min = 1) = 3 "Number of phases";
            constant String VectorGroup = "Yy00";
            parameter Real n(start = 1) "primary voltage (line-to-line) / secondary voltage (line-to-line)";
            parameter Modelica.SIunits.Resistance R1(start = 0.005 / (if C1 == "D" then 1 else 3)) "warm primary resistance per phase";
            parameter Modelica.SIunits.Inductance L1sigma(start = 7.8e-05 / (if C1 == "D" then 1 else 3)) "primary stray inductance per phase";
            parameter Modelica.SIunits.Resistance R2(start = 0.005 / (if C2 == "d" then 1 else 3)) "warm secondary resistance per phase";
            parameter Modelica.SIunits.Inductance L2sigma(start = 7.8e-05 / (if C2 == "d" then 1 else 3)) "secondary stray inductance per phase";
            output Modelica.SIunits.Voltage v1[m] = plug1.pin.v "Primary voltage";
            output Modelica.SIunits.Current i1[m] = plug1.pin.i "Primary current";
            output Modelica.SIunits.Voltage v2[m] = plug2.pin.v "Secondary voltage";
            output Modelica.SIunits.Current i2[m] = plug2.pin.i "Secondary current";
          protected 
            constant String C1 = Modelica.Utilities.Strings.substring(VectorGroup, 1, 1);
            constant String C2 = Modelica.Utilities.Strings.substring(VectorGroup, 2, 2);
            parameter Real ni = (n * (if C2 == "z" then sqrt(3) else 2) * (if C2 == "d" then 1 else sqrt(3))) / (if C1 == "D" then 1 else sqrt(3));
          public 
            Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug1(final m = m) annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug2(final m = m) annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Basic.Resistor r1(final m = m, final R = fill(R1, m), final T_ref = fill(293.15, m), final alpha = zeros(m), final useHeatPort = false, final T = r1.T_ref) annotation(Placement(transformation(extent = {{ -90, -10},{ -70,10}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Basic.Inductor l1sigma(final m = m, final L = fill(L1sigma, m)) annotation(Placement(transformation(extent = {{ -70, -10},{ -50,10}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Basic.Resistor r2(final m = m, final R = fill(R2, m), final T_ref = fill(293.15, m), final alpha = zeros(m), final T = r2.T_ref) annotation(Placement(transformation(extent = {{70, -10},{90,10}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Basic.Inductor l2sigma(final m = m, final L = fill(L2sigma, m)) annotation(Placement(transformation(extent = {{50, -10},{70,10}}, rotation = 0)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -90,60},{30, -60}}, lineColor = {0,0,255}),Ellipse(extent = {{ -30,60},{90, -60}}, lineColor = {0,0,255}),Text(extent = {{0,100},{0,60}}, lineColor = {0,0,255}, textString = "%name"),Text(extent = {{0,40},{0, -40}}, lineColor = {0,0,255}, textString = "%VectorGroup"),Text(extent = {{0, -60},{0, -100}}, lineColor = {0,0,255}, textString = "n=%n")}), Documentation(info = "<html>
Partialmodel of a threephase transformer, containing primary and secondary resistances and stray inductances, as well as the iron core.
Circuit layout (vector group) of primary and secondary windings have to be defined.
</html>"));
            IdealCore core(final m = m, final n12 = ni, final n13 = ni) annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));

          equation 
            connect(r1.plug_n,l1sigma.plug_p) annotation(Line(points = {{ -70,0},{ -70,0}}, color = {0,0,255}));
            connect(l2sigma.plug_n,r2.plug_p) annotation(Line(points = {{70,0},{70,0}}, color = {0,0,255}));
            connect(plug1,r1.plug_p) annotation(Line(points = {{ -100,0},{ -90,0}}, color = {0,0,255}));
            connect(r2.plug_n,plug2) annotation(Line(points = {{90,0},{100,0}}, color = {0,0,255}));
          end BasicTransformer;
          partial model PartialCore "Partial model of transformer core with 3 windings"
            parameter Integer m(final min = 1) = 3 "number of phases";
            parameter Real n12(start = 1) "turns ratio 1:2";
            parameter Real n13(start = 1) "turns ratio 1:3";
            Modelica.SIunits.Voltage v1[m] = plug_p1.pin.v - plug_n1.pin.v;
            Modelica.SIunits.Current i1[m] = plug_p1.pin.i;
            Modelica.SIunits.Voltage v2[m] = plug_p2.pin.v - plug_n2.pin.v;
            Modelica.SIunits.Current i2[m] = plug_p2.pin.i;
            Modelica.SIunits.Voltage v3[m] = plug_p3.pin.v - plug_n3.pin.v;
            Modelica.SIunits.Current i3[m] = plug_p3.pin.i;
            Modelica.SIunits.Current im[m] = i1 + i2 / n12 + i3 / n13 "Magnetizing current";
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -100,130},{100,110}}, textString = "%name", lineColor = {0,0,255}),Ellipse(extent = {{ -45, -50},{ -20, -25}}, lineColor = {0,0,255}),Ellipse(extent = {{ -45, -25},{ -20,0}}, lineColor = {0,0,255}),Ellipse(extent = {{ -45,0},{ -20,25}}, lineColor = {0,0,255}),Ellipse(extent = {{ -45,25},{ -20,50}}, lineColor = {0,0,255}),Rectangle(extent = {{ -46, -50},{ -34,50}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -90,50},{ -32,50}}, color = {0,0,255}),Line(points = {{ -90, -50},{ -32, -50}}, color = {0,0,255}),Ellipse(extent = {{20,40},{45,65}}, lineColor = {0,0,255}),Ellipse(extent = {{20,65},{45,90}}, lineColor = {0,0,255}),Rectangle(extent = {{36,40},{46,90}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{32,90},{90,90}}, color = {0,0,255}),Line(points = {{34,40},{92,40}}, color = {0,0,255}),Ellipse(extent = {{20, -90},{45, -65}}, lineColor = {0,0,255}),Ellipse(extent = {{20, -65},{45, -40}}, lineColor = {0,0,255}),Rectangle(extent = {{36, -90},{46, -40}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{32, -40},{90, -40}}, color = {0,0,255}),Line(points = {{34, -90},{92, -90}}, color = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
Partial model of transformer core with 3 windings; saturation function flux versus magentizing current has to be defined.
</html>"));
            Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_p1(final m = m) annotation(Placement(transformation(extent = {{ -110,40},{ -90,60}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_n1(final m = m) annotation(Placement(transformation(extent = {{ -110, -60},{ -90, -40}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_p2(final m = m) annotation(Placement(transformation(extent = {{90,90},{110,110}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_n2(final m = m) annotation(Placement(transformation(extent = {{90,30},{110,50}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_p3(final m = m) annotation(Placement(transformation(extent = {{90, -30},{110, -50}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_n3(final m = m) annotation(Placement(transformation(extent = {{90, -90},{110, -110}}, rotation = 0)));

          equation 
            plug_p1.pin.i + plug_n1.pin.i = zeros(m);
            plug_p2.pin.i + plug_n2.pin.i = zeros(m);
            plug_p3.pin.i + plug_n3.pin.i = zeros(m);
          end PartialCore;
          model IdealCore "Ideal transformer with 3 windings"
            annotation(defaultComponentName = "core", Diagram(graphics), Documentation(info = "<html>
Ideal transformer with 3 windings: no magnetizing current.
</html>"));
            extends PartialCore;

          equation 
            im = zeros(m);
            v1 = n12 * v2;
            v1 = n13 * v3;
          end IdealCore;
        end Components;
      end BasicMachines;
      package Sensors "Sensors for machine modelling"
        extends Modelica.Icons.Library2;
        annotation(Documentation(info = "<HTML>
This package contains sensors that are usefull when modelling machines.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2009, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.03 2004/09/24 Anton Haumer</li>
  <li> v1.1  2004/10/01 Anton Haumer<br>
       changed RotorDisplacementAngle</li>
  <li> v1.4   2004/11/11 Anton Haumer<br>
       removed mechanical flange support, also in sensor RotorDisplacementAngle<br>
       to ease the implementation of a 3D-frame in a future release</li>
  </ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -74,46},{66, -94}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -4,46},{ -4,16}}, color = {0,0,0}),Line(points = {{18.9,8.8},{36.2,33.3}}, color = {0,0,0}),Line(points = {{ -26.9,8.8},{ -44.2,33.3}}, color = {0,0,0}),Line(points = {{33.6, -10.3},{61.8, -0.1}}, color = {0,0,0}),Line(points = {{ -41.6, -10.3},{ -69.8, -0.1}}, color = {0,0,0}),Line(points = {{ -4, -24},{5.02,4.6}}, color = {0,0,0}),Polygon(points = {{ -4.48,7.6},{14,2},{14,33.2},{ -4.48,7.6}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -9, -19},{1, -29}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}));
        model VoltageQuasiRMSSensor "Length of spcae phasor -> RMS voltage"
          constant Integer m(final min = 1) = 3 "Number of phases";
          Modelica.Blocks.Interfaces.RealOutput V annotation(Placement(transformation(origin = {0, -110}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_p(final m = m) annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_n(final m = m) annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Sensors.VoltageSensor VoltageSensor1(final m = m) annotation(Placement(transformation(extent = {{ -10,40},{10,60}}, rotation = 0)));
          Modelica.Blocks.Math.Gain Gain1(final k = 1 / sqrt(2)) annotation(Placement(transformation(origin = {0, -70}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Machines.SpacePhasors.Blocks.ToSpacePhasor ToSpacePhasor1 annotation(Placement(transformation(origin = {0,10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Machines.SpacePhasors.Blocks.ToPolar ToPolar1 annotation(Placement(transformation(origin = {0, -30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -70,70},{70, -70}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,70},{0,40}}, color = {0,0,0}),Line(points = {{22.9,32.8},{40.2,57.3}}, color = {0,0,0}),Line(points = {{ -22.9,32.8},{ -40.2,57.3}}, color = {0,0,0}),Line(points = {{37.6,13.7},{65.8,23.9}}, color = {0,0,0}),Line(points = {{ -37.6,13.7},{ -65.8,23.9}}, color = {0,0,0}),Line(points = {{0,0},{9.02,28.6}}, color = {0,0,0}),Polygon(points = {{ -0.48,31.6},{18,26},{18,57.2},{ -0.48,31.6}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -5,5},{5, -5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{ -70,0}}, color = {0,0,255}),Line(points = {{70,0},{90,0}}, color = {0,0,255}),Line(points = {{0, -70},{0, -100}}, color = {0,0,255}),Text(extent = {{ -40, -20},{40, -60}}, lineColor = {0,0,255}, textString = "V RMS")}), Documentation(info = "<HTML>
Measured 3-phase instantaneous voltages are transformed to the corresponding space phasor; <br>
output is length of the space phasor divided by sqrt(2), thus giving in sinusoidal stationary state RMS voltage.
</HTML>"));

        equation 
          connect(plug_p,VoltageSensor1.plug_p) annotation(Line(points = {{ -100,0},{ -100,50},{ -10,50}}, color = {0,0,255}));
          connect(VoltageSensor1.plug_n,plug_n) annotation(Line(points = {{10,50},{100,50},{100,0}}, color = {0,0,255}));
          connect(VoltageSensor1.v,ToSpacePhasor1.u) annotation(Line(points = {{0,39},{0,22},{2.20436e-15,22}}, color = {0,0,255}));
          connect(ToSpacePhasor1.y,ToPolar1.u) annotation(Line(points = {{ -2.02067e-15, -1},{ -2.02067e-15, -8.5},{2.20436e-15, -8.5},{2.20436e-15, -18}}, color = {0,0,255}));
          connect(ToPolar1.y[1],Gain1.u) annotation(Line(points = {{ -0.5, -41},{ -0.5, -49.5},{2.20436e-15, -49.5},{2.20436e-15, -58}}, color = {0,0,255}));
          connect(Gain1.y,V) annotation(Line(points = {{ -2.02067e-15, -81},{ -2.02067e-15, -91.5},{0, -91.5},{0, -110}}, color = {0,0,255}));
        end VoltageQuasiRMSSensor;
        model CurrentQuasiRMSSensor "Length of spcae phasor -> RMS current"
          constant Integer m(final min = 1) = 3 "Number of phases";
          Modelica.Blocks.Interfaces.RealOutput I annotation(Placement(transformation(origin = {0, -110}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_p(final m = m) annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_n(final m = m) annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Sensors.CurrentSensor CurrentSensor1(final m = m) annotation(Placement(transformation(extent = {{ -10,40},{10,60}}, rotation = 0)));
          Modelica.Blocks.Math.Gain Gain1(final k = 1 / sqrt(2)) annotation(Placement(transformation(origin = {0, -70}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Machines.SpacePhasors.Blocks.ToSpacePhasor ToSpacePhasor1 annotation(Placement(transformation(origin = {0,10}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Machines.SpacePhasors.Blocks.ToPolar ToPolar1 annotation(Placement(transformation(origin = {0, -30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -70,70},{70, -70}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,70},{0,40}}, color = {0,0,0}),Line(points = {{22.9,32.8},{40.2,57.3}}, color = {0,0,0}),Line(points = {{ -22.9,32.8},{ -40.2,57.3}}, color = {0,0,0}),Line(points = {{37.6,13.7},{65.8,23.9}}, color = {0,0,0}),Line(points = {{ -37.6,13.7},{ -65.8,23.9}}, color = {0,0,0}),Line(points = {{0,0},{9.02,28.6}}, color = {0,0,0}),Polygon(points = {{ -0.48,31.6},{18,26},{18,57.2},{ -0.48,31.6}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -5,5},{5, -5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{ -70,0}}, color = {0,0,255}),Line(points = {{70,0},{90,0}}, color = {0,0,255}),Line(points = {{0, -70},{0, -100}}, color = {0,0,255}),Text(extent = {{ -40, -20},{40, -60}}, lineColor = {0,0,255}, textString = "A RMS")}), Documentation(info = "<HTML>
Measured 3-phase instantaneous currents are transformed to the corresponding space phasor; <br>
output is length of the space phasor divided by sqrt(2), thus giving in sinusoidal stationary state RMS current.
</HTML>"));

        equation 
          connect(plug_p,CurrentSensor1.plug_p) annotation(Line(points = {{ -100,0},{ -100,50},{ -10,50}}, color = {0,0,255}));
          connect(CurrentSensor1.plug_n,plug_n) annotation(Line(points = {{10,50},{100,50},{100,0}}, color = {0,0,255}));
          connect(CurrentSensor1.i,ToSpacePhasor1.u) annotation(Line(points = {{0,39},{0,22},{2.20436e-15,22}}, color = {0,0,255}));
          connect(ToSpacePhasor1.y,ToPolar1.u) annotation(Line(points = {{ -2.02067e-15, -1},{ -2.02067e-15, -8.5},{2.20436e-15, -8.5},{2.20436e-15, -18}}, color = {0,0,255}));
          connect(ToPolar1.y[1],Gain1.u) annotation(Line(points = {{ -0.5, -41},{ -0.5, -49.5},{2.20436e-15, -49.5},{2.20436e-15, -58}}, color = {0,0,255}));
          connect(Gain1.y,I) annotation(Line(points = {{ -2.02067e-15, -81},{ -2.02067e-15, -91.5},{0, -91.5},{0, -110}}, color = {0,0,255}));
        end CurrentQuasiRMSSensor;
        model ElectricalPowerSensor "Instantaneous power from spcae phasors"
          constant Integer m(final min = 1) = 3 "Number of phases";
          Modelica.Blocks.Interfaces.RealOutput P annotation(Placement(transformation(origin = { -50,110}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
          Modelica.Blocks.Interfaces.RealOutput Q annotation(Placement(transformation(origin = {50,110}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
          Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_p(final m = m) annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_ni(final m = m) annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_nv(final m = m) annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -70,70},{70, -70}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,70},{0,40}}, color = {0,0,0}),Line(points = {{22.9,32.8},{40.2,57.3}}, color = {0,0,0}),Line(points = {{ -22.9,32.8},{ -40.2,57.3}}, color = {0,0,0}),Line(points = {{37.6,13.7},{65.8,23.9}}, color = {0,0,0}),Line(points = {{ -37.6,13.7},{ -65.8,23.9}}, color = {0,0,0}),Line(points = {{0,0},{9.02,28.6}}, color = {0,0,0}),Polygon(points = {{ -0.48,31.6},{18,26},{18,57.2},{ -0.48,31.6}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -5,5},{5, -5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{ -70,0}}, color = {0,0,255}),Line(points = {{70,0},{90,0}}, color = {0,0,255}),Text(extent = {{ -40, -20},{40, -60}}, lineColor = {0,0,255}, textString = "P Q"),Line(points = {{0, -70},{0, -90}}, color = {0,0,255}),Line(points = {{ -10,70},{ -10,80},{ -50,80},{ -50,100}}, color = {0,0,255}),Line(points = {{10,70},{10,80},{50,80},{50,100}}, color = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<HTML>
3-phase instantaneous voltages (plug_p - plug_nv) and currents (plug_p - plug_ni) are transformed to the corresponding space phasors, <br>
which are used to calculate power quantities:<br>
P = instantaneous power, thus giving in stationary state active power.<br>
Q = giving in stationary state reactive power.<br>
</HTML>"));
        protected 
          Modelica.SIunits.Voltage v_[2];
          Modelica.SIunits.Current i_[2];

        equation 
          plug_p.pin.v = plug_ni.pin.v;
          plug_p.pin.i + plug_ni.pin.i = zeros(m);
          plug_nv.pin.i = zeros(m);
          v_ = Machines.SpacePhasors.Functions.ToSpacePhasor(plug_p.pin.v - plug_nv.pin.v);
          i_ = Machines.SpacePhasors.Functions.ToSpacePhasor(plug_p.pin.i);
          2 / 3 * P =  +v_[1] * i_[1] + v_[2] * i_[2];
          2 / 3 * Q =  -v_[1] * i_[2] + v_[2] * i_[1];
        end ElectricalPowerSensor;
        model MechanicalPowerSensor "Mechanical power = torque x speed"
          extends Modelica.Mechanics.Rotational.Interfaces.PartialTwoFlanges;
          parameter Boolean useSupport = false "use support or fixed housing" annotation(Evaluate = true);
          Modelica.Blocks.Interfaces.RealOutput P annotation(Placement(transformation(origin = {0,110}, extent = {{10, -10},{ -10,10}}, rotation = 270)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -70,70},{70, -70}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,70},{0,40}}, color = {0,0,0}),Line(points = {{22.9,32.8},{40.2,57.3}}, color = {0,0,0}),Line(points = {{ -22.9,32.8},{ -40.2,57.3}}, color = {0,0,0}),Line(points = {{37.6,13.7},{65.8,23.9}}, color = {0,0,0}),Line(points = {{ -37.6,13.7},{ -65.8,23.9}}, color = {0,0,0}),Line(points = {{0,0},{9.02,28.6}}, color = {0,0,0}),Polygon(points = {{ -0.48,31.6},{18,26},{18,57.2},{ -0.48,31.6}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -5,5},{5, -5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -70,0},{ -90,0}}, color = {0,0,0}),Line(points = {{70,0},{90,0}}, color = {0,0,0}),Line(points = {{0,70},{0,100}}, color = {0,0,255}),Text(extent = {{ -40, -20},{40, -60}}, lineColor = {0,0,255}, textString = "Pmech"),Rectangle(extent = {{ -20, -80},{20, -120}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(visible = not useSupport, points = {{ -20, -100},{20, -100}}, color = {0,0,0}, smooth = Smooth.None),Line(visible = not useSupport, points = {{ -10, -100},{ -20, -120}}, color = {0,0,0}, smooth = Smooth.None),Line(visible = not useSupport, points = {{0, -100},{ -10, -120}}, color = {0,0,0}, smooth = Smooth.None),Line(visible = not useSupport, points = {{10, -100},{0, -120}}, color = {0,0,0}, smooth = Smooth.None),Line(visible = not useSupport, points = {{20, -100},{10, -120}}, color = {0,0,0}, smooth = Smooth.None)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<HTML>
Calculates (mechanical) power from torque times angular speed.
</HTML>"));
          Modelica.Mechanics.Rotational.Sensors.TorqueSensor torqueSensor annotation(Placement(transformation(extent = {{20, -10},{40,10}}, rotation = 0)));
          Modelica.Blocks.Math.Product product annotation(Placement(transformation(origin = {0,30}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Mechanics.Rotational.Sensors.RelSpeedSensor relSpeedSensor annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 90, origin = { -30, -30})));
          Modelica.Mechanics.Rotational.Components.Fixed fixed if not useSupport annotation(Placement(transformation(extent = {{ -40, -100},{ -20, -80}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Interfaces.Flange_a support if useSupport "support at which the reaction torque is acting" annotation(Placement(transformation(extent = {{ -10, -110},{10, -90}}, rotation = 0)));

        equation 
          connect(flange_a,torqueSensor.flange_a) annotation(Line(points = {{ -100,0},{ -70,0},{ -40,0},{20,0}}, color = {0,0,0}));
          connect(torqueSensor.flange_b,flange_b) annotation(Line(points = {{40,0},{100,0}}, color = {0,0,0}));
          connect(product.y,P) annotation(Line(points = {{6.73556e-16,41},{6.73556e-16,52},{0,52},{0,110}}, color = {0,0,255}));
          connect(torqueSensor.tau,product.u2) annotation(Line(points = {{22, -11},{22, -20},{6, -20},{6,18}}, color = {0,0,127}, smooth = Smooth.None));
          connect(flange_a,relSpeedSensor.flange_b) annotation(Line(points = {{ -100,0},{ -30,0},{ -30, -20}}, color = {0,0,0}, smooth = Smooth.None));
          connect(relSpeedSensor.w_rel,product.u1) annotation(Line(points = {{ -19, -30},{ -6, -30},{ -6,18}}, color = {0,0,127}, smooth = Smooth.None));
          connect(relSpeedSensor.flange_a,fixed.flange) annotation(Line(points = {{ -30, -40},{ -30, -90}}, color = {0,0,0}, smooth = Smooth.None));
          connect(relSpeedSensor.flange_a,support) annotation(Line(points = {{ -30, -40},{ -30, -80},{0, -80},{0, -100}}, color = {0,0,0}, smooth = Smooth.None));
        end MechanicalPowerSensor;
        model RotorDisplacementAngle "Rotor lagging angle"
          constant Integer m = 3 "number of phases";
          parameter Integer p(min = 1) "number of pole pairs";
          parameter Boolean useSupport = false "use support or fixed housing" annotation(Evaluate = true);
          Modelica.Blocks.Interfaces.RealOutput rotorDisplacementAngle annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_p(final m = m) annotation(Placement(transformation(extent = {{ -110,50},{ -90,70}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_n(final m = m) annotation(Placement(transformation(extent = {{ -110, -70},{ -90, -50}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Sensors.VoltageSensor VoltageSensor1(final m = m) annotation(Placement(transformation(origin = { -80, -10}, extent = {{10, -10},{ -10,10}}, rotation = 90)));
          Machines.SpacePhasors.Blocks.ToSpacePhasor ToSpacePhasorVS annotation(Placement(transformation(extent = {{ -60, -20},{ -40,0}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Interfaces.Flange_a flange annotation(Placement(transformation(extent = {{ -10,90},{10,110}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Sensors.RelAngleSensor relativeAngleSensor annotation(Placement(transformation(extent = {{40,70},{20,90}}, rotation = 0)));
          Modelica.Blocks.Sources.Constant constant_(final k = Modelica.Constants.pi / 2) annotation(Placement(transformation(extent = {{ -50,30},{ -30,50}}, rotation = 0)));
          Modelica.Blocks.Math.Add add(final k2 = 1, final k1 = p) annotation(Placement(transformation(origin = { -10,20}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Machines.SpacePhasors.Blocks.Rotator rotatorVS2R annotation(Placement(transformation(extent = {{ -20,0},{0, -20}}, rotation = 0)));
          Machines.SpacePhasors.Blocks.ToPolar ToPolarVSR annotation(Placement(transformation(extent = {{20, -20},{40,0}}, rotation = 0)));
          Modelica.Blocks.Routing.DeMultiplex2 deMultiplex2(final n1 = 1, final n2 = 1) annotation(Placement(transformation(extent = {{60, -20},{80,0}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Interfaces.Flange_a support if useSupport "support at which the reaction torque is acting" annotation(Placement(transformation(extent = {{90,90},{110,110}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Components.Fixed fixed if not useSupport annotation(Placement(transformation(extent = {{90,70},{110,90}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -60,80},{60,40}}, lineColor = {0,255,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -60, -40},{60, -80}}, lineColor = {0,255,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -60,60},{60,40}}, lineColor = {0,255,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -60, -40},{60, -60}}, lineColor = {0,255,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -40,40},{40, -40}}, lineColor = {0,255,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,0},{ -80,80}}, color = {0,0,255}),Polygon(points = {{ -80,80},{ -68,76},{ -76,68},{ -80,80}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,0},{0,80}}, color = {0,255,0}),Polygon(points = {{0,84},{4,72},{ -4,72},{0,84}}, lineColor = {0,255,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{80,120},{120,80}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(visible = not useSupport, points = {{80,100},{120,100}}, color = {0,0,0}, smooth = Smooth.None),Line(visible = not useSupport, points = {{90,120},{80,100}}, color = {0,0,0}, smooth = Smooth.None),Line(visible = not useSupport, points = {{100,120},{90,100}}, color = {0,0,0}, smooth = Smooth.None),Line(visible = not useSupport, points = {{110,120},{100,100}}, color = {0,0,0}, smooth = Smooth.None),Line(visible = not useSupport, points = {{120,120},{110,100}}, color = {0,0,0}, smooth = Smooth.None)}), Documentation(info = "<HTML>
Calculates rotor lagging angle by measuring the stator phase voltages, transforming them to the correspondig space phasor in stator-fixed coordinate system, <br>
rotating the space phasor to the rotor-fixed coordinate system and calculating the angle of this space phasor.
<p>
The sensor's housing can be implicitely fixed (useSupport=false).<br>
If the machine's stator also implicitely fixed (useSupport=false), the angle at the flange
is equal to the angle of the machine's rotor against the stator.<br>
Otherwise, the sensor's support has to be connected to the machine's support.
</p>
</HTML>"));

        equation 
          connect(constant_.y,add.u2) annotation(Line(points = {{ -29,40},{ -16,40},{ -16,32}}, color = {0,0,255}));
          connect(add.y,rotatorVS2R.angle) annotation(Line(points = {{ -10,9},{ -10,5.5},{ -10,5.5},{ -10,2}}, color = {0,0,255}));
          connect(ToSpacePhasorVS.y,rotatorVS2R.u) annotation(Line(points = {{ -39, -10},{ -22, -10}}, color = {0,0,255}));
          connect(rotatorVS2R.y,ToPolarVSR.u) annotation(Line(points = {{1, -10},{18, -10}}, color = {0,0,255}));
          connect(ToPolarVSR.y,deMultiplex2.u) annotation(Line(points = {{41, -10},{58, -10}}, color = {0,0,255}));
          connect(plug_p,VoltageSensor1.plug_p) annotation(Line(points = {{ -100,60},{ -80,60},{ -80,0}}, color = {0,0,255}));
          connect(plug_n,VoltageSensor1.plug_n) annotation(Line(points = {{ -100, -60},{ -80, -60},{ -80, -20}}, color = {0,0,255}));
          connect(VoltageSensor1.v,ToSpacePhasorVS.u) annotation(Line(points = {{ -69, -10},{ -62, -10}}, color = {0,0,255}));
          connect(deMultiplex2.y2[1],rotorDisplacementAngle) annotation(Line(points = {{81, -16},{90, -16},{90,0},{110,0}}, color = {0,0,255}));
          connect(relativeAngleSensor.phi_rel,add.u1) annotation(Line(points = {{30,69},{30,40},{ -4,40},{ -4,32}}, color = {0,0,255}));
          connect(relativeAngleSensor.flange_b,flange) annotation(Line(points = {{20,80},{0,80},{0,100}}, color = {0,0,0}));
          connect(relativeAngleSensor.flange_a,support) annotation(Line(points = {{40,80},{60,80},{60,100},{100,100}}, color = {0,0,0}, smooth = Smooth.None));
          connect(relativeAngleSensor.flange_a,fixed.flange) annotation(Line(points = {{40,80},{100,80}}, color = {0,0,0}, smooth = Smooth.None));
        end RotorDisplacementAngle;
      end Sensors;
      package SpacePhasors "Library with space phasor-models"
        extends Modelica.Icons.Library2;
        annotation(Documentation(info = "<HTML>
This package contains components, blocks and functions to utilize space phasor theory.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2009, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.00 2004/09/16 Anton Haumer</li>
  <li> v1.30 2004/11/05 Anton Haumer<br>
       several improvements in SpacePhasors.Blocks</li>
  <li> v1.60 2005/11/03 Anton Haumer<br>
       added Components.Rotator</li>
  <li> v1.6.1 2005/11/10 Anton Haumer<br>
       improved Transformation and Rotation</li>
  </ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -8, -26},{64,46},{44,38},{56,26},{64,46}}, color = {0,0,255}, thickness = 0.5),Line(points = {{ -8, -26},{64, -98},{56, -78},{44, -90},{64, -98}}, color = {0,0,255}, thickness = 0.5)}));
        package Components "Basic space phasor models"
          extends Modelica.Icons.Library;
          annotation(Documentation(info = "<HTML>
This package contains basic space phasor models.<br>
Real and imaginary part of voltage space phasor are the potentials v_[2] of the space phasor connector; (implicit grounded).<br>
Real and imaginary part of current space phasor are the currents i_[2] at the space phasor connector;
a ground has to be used where necessary for currents flowing back.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2009, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.00 2004/09/16 Anton Haumer</li>
  <li> v1.60 2005/11/03 Anton Haumer<br>
       added Rotator</li>
  <li> v1.6.1 2005/11/10 Anton Haumer<br>
       improved Transformation and Rotation</li>
  </ul>
</HTML>"));
          model SpacePhasor "Physical transformation: three phase <-> space phasors"
            constant Integer m = 3 "number of phases";
            constant Real pi = Modelica.Constants.pi;
            parameter Real turnsRatio = 1;
            Modelica.SIunits.Voltage v[m] "instantaneous phase voltages";
            Modelica.SIunits.Current i[m] "instantaneous phase currents";
          protected 
            parameter Real TransformationMatrix[2,m] = 2 / m * {array(cos( +(k - 1) / m * 2 * pi) for k in 1:m),array( +sin( +(k - 1) / m * 2 * pi) for k in 1:m)};
            parameter Real InverseTransformation[m,2] = array({cos( -(k - 1) / m * 2 * pi), -sin( -(k - 1) / m * 2 * pi)} for k in 1:m);
          public 
            Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_p(final m = m) annotation(Placement(transformation(extent = {{ -110,90},{ -90,110}}, rotation = 0)));
            Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_n(final m = m) annotation(Placement(transformation(extent = {{ -110, -110},{ -90, -90}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.PositivePin zero annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
            Modelica.Electrical.Analog.Interfaces.NegativePin ground annotation(Placement(transformation(extent = {{90, -110},{110, -90}}, rotation = 0)));
            Modelica.Electrical.Analog.Basic.Ground gnd annotation(Placement(transformation(extent = {{60, -110},{80, -90}}, rotation = 0)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{0,0},{80,80},{60,72},{72,60},{80,80}}, color = {0,0,255}),Line(points = {{0,0},{80, -80},{72, -60},{60, -72},{80, -80}}, color = {0,0,255}),Line(points = {{ -80,0},{ -73.33,10},{ -66.67,17.32},{ -60,20},{ -53.33,17.32},{ -46.67,10},{ -40,0},{ -33.33, -10},{ -26.67, -17.32},{ -20, -20},{ -13.33, -17.32},{ -6.67, -10},{0,0}}, color = {0,0,255}),Line(points = {{ -90,0},{ -83.33,10},{ -76.67,17.32},{ -70,20},{ -63.33,17.32},{ -56.67,10},{ -50,0},{ -43.33, -10},{ -36.67, -17.32},{ -30, -20},{ -23.33, -17.32},{ -16.67, -10},{ -10,0}}, color = {0,0,255}),Line(points = {{ -70,0},{ -63.33,10},{ -56.67,17.32},{ -50,20},{ -43.33,17.32},{ -36.67,10},{ -30,0},{ -23.33, -10},{ -16.67, -17.32},{ -10, -20},{ -3.33, -17.32},{3.33, -10},{10,0}}, color = {0,0,255}),Text(extent = {{ -150, -100},{150, -160}}, lineColor = {0,0,255}, textString = "%name"),Text(extent = {{40,10},{90, -10}}, lineColor = {0,0,255}, textString = "zero"),Line(points = {{90, -100},{60, -100}}, color = {0,0,255}),Line(points = {{60, -84},{60, -116}}, color = {0,0,255}),Line(points = {{50, -90},{50, -110}}, color = {0,0,255}),Line(points = {{40, -96},{40, -104}}, color = {0,0,255})}), Documentation(info = "<HTML>
Physical transformation of voltages and currents: three phases &lt;-&gt; space phasors:<br>
x[k] = X0 + {cos(-(k - 1)/m*2*pi),-sin(-(k - 1)/m*2*pi) * X[Re,Im]<br>
and vice versa:<br>
X0 = sum(x[k])/m<br>
X[Re,Im] = sum(2/m*{cos((k - 1)/m*2*pi),sin((k - 1)/m*2*pi)}*x[k])<br>
were x designates three phase values, X[Re,Im] designates the space phasor and X0 designates the zero sequence system.<br>
<i>Physcial transformation</i> means that both voltages and currents are transformed in both directions.<br>
Zero-sequence voltage and current are present at pin zero. An additional zero-sequence impedance could be connected between pin zero and pin ground.
</HTML>"));
            Interfaces.SpacePhasor spacePhasor annotation(Placement(transformation(extent = {{90,90},{110,110}}, rotation = 0)));

          equation 
            v / turnsRatio = plug_p.pin.v - plug_n.pin.v;
            i * turnsRatio =  +plug_p.pin.i;
            i * turnsRatio =  -plug_n.pin.i;
            zero.v = 1 / m * sum(v);
            spacePhasor.v_ = TransformationMatrix * v;
             -zero.i = 1 / m * sum(i);
             -spacePhasor.i_ = TransformationMatrix * i;
            connect(gnd.p,ground) annotation(Line(points = {{70, -90},{100, -90},{100, -100}}, color = {0,0,255}));
          end SpacePhasor;
          model Rotator "Rotates space phasor"
            constant Real pi = Modelica.Constants.pi;
          protected 
            Real RotationMatrix[2,2] = {{ +cos( -angle), -sin( -angle)},{ +sin( -angle), +cos( -angle)}};
          public 
            Interfaces.SpacePhasor spacePhasor_a annotation(Placement(transformation(extent = {{ -110,90},{ -90,110}}, rotation = 0)));
            Interfaces.SpacePhasor spacePhasor_b annotation(Placement(transformation(extent = {{90,90},{110,110}}, rotation = 0)));
            Modelica.Blocks.Interfaces.RealInput angle annotation(Placement(transformation(origin = {0, -120}, extent = {{ -20, -20},{20,20}}, rotation = 90)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{50, -2},{ -2, -50}}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{0,0},{80,0},{60,10},{60, -10},{80,0}}, color = {0,0,255}),Line(points = {{0,0},{0,80},{ -10,60},{10,60},{0,80}}, color = {0,0,255}),Polygon(points = {{50,0},{42,14},{54,16},{50,0}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,255}),Rectangle(extent = {{ -50,50},{ -2, -50}}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{50, -2},{ -2, -50}}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255})}), Documentation(info = "<HTML>
Rotates space phasors of left connector to right connector by the angle provided by the input signal \"angle\" from one coordinate system into another.
</HTML>"));

          equation 
            spacePhasor_b.v_ = RotationMatrix * spacePhasor_a.v_;
            spacePhasor_b.i_ + RotationMatrix * spacePhasor_a.i_ = zeros(2);
          end Rotator;
        end Components;
        package Blocks "Blocks for space phasor transformation"
          extends Modelica.Icons.Library;
          annotation(Documentation(info = "<HTML>
This package contains space phasor transformation blocks for use in controllers:
<ul>
<li>ToSpacePhasor: transforms a set of threephase values to space phasor and zero sequence system</li>
<li>FromSpacePhasor: transforms a space phasor and zero sequence system to a set of threephase values</li>
<li>Rotator: rotates a space phasor (from one coordinate system into another)</li>
<li>ToPolar: Converts a space phasor from rectangular coordinates to polar coordinates</li>
<li>FromPolar: Converts a space phasor from polar coordinates to rectangular coordinates</li>
</ul>
</HTML>", revisions = "<HTML>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2009, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.00 2004/09/16 Anton Haumer</li>
  <li> v1.30 2004/11/05 Anton Haumer<br>
       several improvements in SpacePhasors.Blocks</li>
  <li> v1.6.1 2005/11/10 Anton Haumer<br>
       improved Transformation and Rotation</li>
  </ul>
</HTML>"));
          block ToSpacePhasor "Conversion: three phase -> space phasor"
            extends Modelica.Blocks.Interfaces.MIMO(final nin = m, final nout = 2);
            constant Integer m = 3 "number of phases";
            constant Real pi = Modelica.Constants.pi;
          protected 
            parameter Real TransformationMatrix[2,m] = 2 / m * {array(cos( +(k - 1) / m * 2 * pi) for k in 1:m),array( +sin( +(k - 1) / m * 2 * pi) for k in 1:m)};
            parameter Real InverseTransformation[m,2] = array({cos( -(k - 1) / m * 2 * pi), -sin( -(k - 1) / m * 2 * pi)} for k in 1:m);
          public 
            Modelica.Blocks.Interfaces.RealOutput zero annotation(Placement(transformation(extent = {{100, -70},{120, -90}}, rotation = 0)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{0,0},{80,80},{60,72},{72,60},{80,80}}, color = {0,0,255}),Line(points = {{0,0},{80, -80},{72, -60},{60, -72},{80, -80}}, color = {0,0,255}),Line(points = {{ -80,0},{ -73.33,10},{ -66.67,17.32},{ -60,20},{ -53.33,17.32},{ -46.67,10},{ -40,0},{ -33.33, -10},{ -26.67, -17.32},{ -20, -20},{ -13.33, -17.32},{ -6.67, -10},{0,0}}, color = {0,0,255}),Line(points = {{ -90,0},{ -83.33,10},{ -76.67,17.32},{ -70,20},{ -63.33,17.32},{ -56.67,10},{ -50,0},{ -43.33, -10},{ -36.67, -17.32},{ -30, -20},{ -23.33, -17.32},{ -16.67, -10},{ -10,0}}, color = {0,0,255}),Line(points = {{ -70,0},{ -63.33,10},{ -56.67,17.32},{ -50,20},{ -43.33,17.32},{ -36.67,10},{ -30,0},{ -23.33, -10},{ -16.67, -17.32},{ -10, -20},{ -3.33, -17.32},{3.33, -10},{10,0}}, color = {0,0,255}),Text(extent = {{ -12, -74},{64, -86}}, lineColor = {0,0,0}, textString = "zero")}), Documentation(info = "<HTML>
Transformation of threephase values (voltages or currents) to space phasor and zero sequence value.
</HTML>"));

          equation 
            zero = 1 / m * sum(u);
            y = TransformationMatrix * u;
          end ToSpacePhasor;
          block FromSpacePhasor "Conversion: space phasor -> three phase"
            extends Modelica.Blocks.Interfaces.MIMO(final nin = 2, final nout = m);
            constant Integer m = 3 "number of phases";
            constant Real pi = Modelica.Constants.pi;
          protected 
            parameter Real TransformationMatrix[2,m] = 2 / m * {array(cos( +(k - 1) / m * 2 * pi) for k in 1:m),array( +sin( +(k - 1) / m * 2 * pi) for k in 1:m)};
            parameter Real InverseTransformation[m,2] = array({cos( -(k - 1) / m * 2 * pi), -sin( -(k - 1) / m * 2 * pi)} for k in 1:m);
          public 
            Modelica.Blocks.Interfaces.RealInput zero annotation(Placement(transformation(extent = {{ -140, -60},{ -100, -100}}, rotation = 0)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{0,0},{ -80,80},{ -60,72},{ -72,60},{ -80,80}}, color = {0,0,255}),Line(points = {{0,0},{ -80, -80},{ -72, -60},{ -60, -72},{ -80, -80}}, color = {0,0,255}),Line(points = {{0,0},{6.67,10},{13.33,17.32},{20,20},{26.67,17.32},{33.33,10},{40,0},{46.67, -10},{53.33, -17.32},{60, -20},{66.67, -17.32},{73.33, -10},{80,0}}, color = {0,0,255}),Line(points = {{ -10,0},{ -3.33,10},{3.33,17.32},{10,20},{16.67,17.32},{23.33,10},{30,0},{36.67, -10},{43.33, -17.32},{50, -20},{56.67, -17.32},{63.33, -10},{70,0}}, color = {0,0,255}),Line(points = {{10,0},{16.67,10},{23.33,17.32},{30,20},{36.67,17.32},{43.33,10},{50,0},{56.67, -10},{63.33, -17.32},{70, -20},{76.67, -17.32},{83.33, -10},{90,0}}, color = {0,0,255}),Text(extent = {{ -62, -74},{14, -86}}, lineColor = {0,0,0}, textString = "zero")}), Documentation(info = "<HTML>
Transformation of space phasor and zero sequence value to threephase values (voltages or currents).
</HTML>"));

          equation 
            y = fill(zero, m) + InverseTransformation * u;
          end FromSpacePhasor;
          block Rotator "Rotates space phasor"
            extends Modelica.Blocks.Interfaces.MIMOs(final n = 2);
          protected 
            Real RotationMatrix[2,2] = {{ +cos( -angle), -sin( -angle)},{ +sin( -angle), +cos( -angle)}};
          public 
            Modelica.Blocks.Interfaces.RealInput angle annotation(Placement(transformation(origin = {0, -120}, extent = {{ -20, -20},{20,20}}, rotation = 90)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{0,0},{0,80},{ -10,60},{10,60},{0,80}}, color = {0,0,255}),Line(points = {{0,0},{80,0},{60,10},{60, -10},{80,0}}, color = {0,0,255}),Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,255}),Rectangle(extent = {{ -50,50},{ -2, -50}}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Rectangle(extent = {{50, -2},{ -2, -50}}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{50,0},{42,14},{54,16},{50,0}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -88, -72},{84, -92}}, lineColor = {0,0,0}, textString = "angle")}), Documentation(info = "<HTML>
Rotates a space phasor (voltage or current) by the angle provided by the input signal \"angle\" from one coordinate system into another.
</HTML>"));

          equation 
            y = RotationMatrix * u;
          end Rotator;
          block ToPolar "Converts a space phasor to polar coordinates"
            extends Modelica.Blocks.Interfaces.MIMOs(final n = 2);
            constant Real small = Modelica.Constants.small;
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -60,60},{ -60, -60},{60, -60}}, color = {0,0,255}),Line(points = {{ -100, -100},{100,100}}, color = {0,0,255}),Polygon(points = {{26,26},{14,20},{20,14},{26,26}}, lineColor = {0,0,255}),Line(points = {{ -18, -18},{ -14, -22},{ -10, -28},{ -6, -34},{ -2, -44},{0, -52},{0, -60}}, color = {0,0,255}),Polygon(points = {{ -18, -18},{ -14, -26},{ -10, -22},{ -18, -18}}, lineColor = {0,0,255}),Text(extent = {{ -100,100},{ -6, -6}}, lineColor = {0,0,255}, textString = "R"),Text(extent = {{6,6},{100, -100}}, lineColor = {0,0,255}, textString = "P")}), Documentation(info = "<HTML>
Converts a space phasor from rectangular coordinates to polar coordinates.
</HTML>"));

          equation 
            y[1] = sqrt(u[1] ^ 2 + u[2] ^ 2);
            y[2] = if noEvent(y[1] <= small) then 0 else Modelica.Math.atan2(u[2], u[1]);
          end ToPolar;
          block FromPolar "Converts a space phasor from polar coordinates"
            extends Modelica.Blocks.Interfaces.MIMOs(final n = 2);
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -60,60},{ -60, -60},{60, -60}}, color = {0,0,255}),Line(points = {{ -100, -100},{100,100}}, color = {0,0,255}),Polygon(points = {{26,26},{14,20},{20,14},{26,26}}, lineColor = {0,0,255}),Line(points = {{ -18, -18},{ -14, -22},{ -10, -28},{ -6, -34},{ -2, -44},{0, -52},{0, -60}}, color = {0,0,255}),Polygon(points = {{ -18, -18},{ -14, -26},{ -10, -22},{ -18, -18}}, lineColor = {0,0,255}),Text(extent = {{ -100,100},{ -6, -6}}, lineColor = {0,0,255}, textString = "P"),Text(extent = {{6,6},{100, -100}}, lineColor = {0,0,255}, textString = "R")}), Documentation(info = "<HTML>
Converts a space phasor from polar coordinates to rectangular coordinates.
</HTML>"));

          equation 
            y = u[1] * {cos(u[2]),sin(u[2])};
          end FromPolar;
        end Blocks;
        package Functions "Functions for space phasor transformation"
          extends Modelica.Icons.Library;
          annotation(Documentation(info = "<HTML>
This package contains space phasor transformation functions for use in calculations:
<ul>
<li>ToSpacePhasor: transforms a set of threephase values to space phasor and zero sequence system</li>
<li>FromSpacePhasor: transforms a space phasor and zero sequence system to a set of threephase values</li>
<li>Rotator: rotates a space phasor (from one coordinate system into another)</li>
<li>ToPolar: Converts a space phasor from rectangular coordinates to polar coordinates</li>
<li>FromPolar: Converts a space phasor from polar coordinates to rectangular coordinates</li>
</ul>
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>

  <dt><b>Release Notes:</b></dt>
  <dd>
  <ul>
  <li> v1.00 2004/09/16 Anton Haumer</li>
  <li> v1.6.1 2005/11/10 Anton Haumer<br>
       improved Transformation and Rotation</li>
  </ul>
  </dd>

  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2009, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
</HTML>"));
          function ToSpacePhasor "Conversion: three phase -> space phasor"
            extends Modelica.Icons.Function;
            constant Integer m = 3 "number of phases";
            constant Real pi = Modelica.Constants.pi;
            input Real x[3];
            output Real y[2];
            output Real y0;
            annotation(Documentation(info = "<HTML>
Transformation of three phase values (voltages or currents) to space phasor and zero sequence value:<br>
y[k] = X0 + {cos(-(k - 1)/m*2*pi),-sin(-(k - 1)/m*2*pi) * X[Re,Im]<br>
were y designates three phase values, X[Re,Im] designates the space phasor and X0 designates the zero sequence system.
</HTML>"));
          algorithm 
            y:=zeros(2);
            for k in 1:m loop
                          y:=y + 2 / m * { +cos((k - 1) / m * 2 * pi), +sin( +(k - 1) / m * 2 * pi)} * x[k];

            end for;
            y0:=1 / m * sum(x);
          end ToSpacePhasor;
          function FromSpacePhasor "Conversion: space phasor -> three phase"
            extends Modelica.Icons.Function;
            constant Integer m = 3 "number of phases";
            constant Real pi = Modelica.Constants.pi;
            input Real x[2];
            input Real x0;
            output Real y[3];
            annotation(Documentation(info = "<HTML>
Transformation of space phasor and zero sequence value to three phase values (voltages or currents):<br>
Y0 = sum(x[k])/m<br>
Y[Re,Im] = sum(2/m*{cos((k - 1)/m*2*pi),sin((k - 1)/m*2*pi)}*x[k])<br>
were x designates three phase values, Y[Re,Im] designates the space phasor and Y0 designates the zero sequence system.
</HTML>"));
          algorithm 
            for k in 1:m loop
                          y[k]:=x0 + {cos( -(k - 1) / m * 2 * pi), -sin( -(k - 1) / m * 2 * pi)} * x;

            end for;
          end FromSpacePhasor;
          function Rotator "Rotates space phasor"
            extends Modelica.Icons.Function;
            input Real x[2];
            input Modelica.SIunits.Angle angle;
            output Real y[2];
          protected 
            Real RotationMatrix[2,2] = {{ +cos( -angle), -sin( -angle)},{ +sin( -angle), +cos( -angle)}};
            annotation(Documentation(info = "<HTML>
Rotates a space phasor (voltage or current) by the angle provided by input argument \"angle\" from one coordinate system into another:<br>
y[Re,Im] := {{+cos(-angle),-sin(-angle)},{+sin(-angle),+cos(-angle)}}*x[Re,Im]<br>
where y[Re,Im] designates the space phasor in the new coordinate system (twisted by angle against old coordinate system) and y[Re,Im] designates the space phasor in the old coordinate system.
</HTML>"));
          algorithm 
            y:=RotationMatrix * x;
          end Rotator;
          function ToPolar "Converts a space phasor to polar coordinates"
            extends Modelica.Icons.Function;
            constant Real small = Modelica.Constants.small;
            input Real x[2];
            output Real absolute;
            output Modelica.SIunits.Angle angle;
            annotation(Documentation(info = "<HTML>
Converts a space phasor from rectangular coordinates to polar coordinates, providing angle=0 for {0,0}.
</HTML>"));
          algorithm 
            absolute:=sqrt(x[1] ^ 2 + x[2] ^ 2);
            angle:=if absolute <= small then 0 else Modelica.Math.atan2(x[2], x[1]);
          end ToPolar;
          function FromPolar "Converts a space phasor from polar coordinates"
            extends Modelica.Icons.Function;
            constant Real pi = Modelica.Constants.pi;
            constant Real small = Modelica.Constants.small;
            input Real absolute;
            input Modelica.SIunits.Angle angle;
            output Real x[2];
            annotation(Documentation(info = "<HTML>
Converts a space phasor from polar coordinates to rectangular coordinates.
</HTML>"));
          algorithm 
            x:=absolute * {cos(angle),sin(angle)};
          end FromPolar;
        end Functions;
      end SpacePhasors;
      package Interfaces "SpacePhasor connector and PartialMachines"
        extends Modelica.Icons.Library2;
        annotation(Documentation(info = "<HTML>
This package contains the space phasor connector and partial models for machine models.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2009, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.00 2004/09/16 Anton Haumer</li>
  <li> v1.01 2004/09/18 Anton Haumer<br>
       moved common equations from machine models to PartialMachine</li>
  <li> v1.02 2004/09/19 Anton Haumer<br>
       added PartialDCMachine</li>
  <li> v1.2  2004/10/27 Anton Haumer<br>
       fixed a bug with support (formerly bearing)</li>
  <li> v1.4   2004/11/11 Anton Haumer<br>
       removed mechanical flange support<br>
       to ease the implementation of a 3D-frame in a future release</li>
  <li> v1.51 Beta 2005/02/01 Anton Haumer<br>
       changed parameter polePairs to Integer</li>
  </ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -70, -30},{ -10,30},{50, -30},{ -10, -90},{ -70, -30}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid)}));
        connector SpacePhasor "Connector for Space Phasors"
          Modelica.SIunits.Voltage v_[2];
          flow Modelica.SIunits.Current i_[2];
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{0,100},{ -100,0},{0, -100},{100,0},{0,100}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -150, -90},{150, -150}}, lineColor = {0,0,255}, textString = "%name")}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{0,100},{ -100,0},{0, -100},{100,0},{0,100}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid)}), Documentation(info = "<HTML>
Connector for Space Phasors:
<ul>
<li>Voltage v_[2] ... Real and Imaginary part of voltage space phasor</li>
<li>Current i_[2] ... Real and Imaginary part of current space phasor</li>
</ul>
</HTML>"));
        end SpacePhasor;
        partial model PartialBasicMachine "Partial model for all machines"
          constant Modelica.SIunits.Angle pi = Modelica.Constants.pi;
          parameter Modelica.SIunits.Inertia Jr "rotor's moment of inertia";
          parameter Boolean useSupport = false "enable / disable (=fixed stator) support" annotation(Evaluate = true);
          parameter Modelica.SIunits.Inertia Js(start = Jr) "stator's moment of inertia" annotation(Dialog(enable = useSupport));
          output Modelica.SIunits.Angle phiMechanical = flange.phi - internalSupport.phi "mechanical angle of rotor against stator";
          output Modelica.SIunits.AngularVelocity wMechanical(displayUnit = "1/min") = der(phiMechanical) "mechanical angular velocity of rotor against stator";
          output Modelica.SIunits.Torque tauElectrical = inertiaRotor.flange_a.tau "electromagnetic torque";
          output Modelica.SIunits.Torque tauShaft =  -flange.tau "shaft torque";
          Modelica.Mechanics.Rotational.Interfaces.Flange_a flange annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Components.Inertia inertiaRotor(final J = Jr) annotation(Placement(transformation(origin = {70,0}, extent = {{10,10},{ -10, -10}}, rotation = 180)));
          Modelica.Mechanics.Rotational.Interfaces.Flange_a support if useSupport "support at which the reaction torque is acting" annotation(Placement(transformation(extent = {{90, -110},{110, -90}}, rotation = 0)));
          Modelica.Mechanics.Rotational.Components.Inertia inertiaStator(final J = Js) if useSupport annotation(Placement(transformation(origin = {70, -100}, extent = {{10,10},{ -10, -10}}, rotation = 180)));
          Modelica.Mechanics.Rotational.Components.Fixed fixed if not useSupport annotation(Placement(transformation(extent = {{30, -110},{50, -90}}, rotation = 0)));
        protected 
          Mechanics.Rotational.Interfaces.Support internalSupport annotation(Placement(transformation(extent = {{16, -104},{24, -96}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -40,60},{80, -60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {0,128,255}),Rectangle(extent = {{ -40,60},{ -60, -60}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {128,128,128}),Rectangle(extent = {{80,10},{100, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {95,95,95}),Rectangle(extent = {{ -40,70},{40,50}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{ -50, -90},{ -40, -90},{ -10, -20},{40, -20},{70, -90},{80, -90},{80, -100},{ -50, -100},{ -50, -90}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -150, -120},{150, -180}}, lineColor = {0,0,255}, textString = "%name"),Rectangle(extent = {{80, -80},{120, -120}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(visible = not useSupport, points = {{80, -100},{120, -100}}, color = {0,0,0}, smooth = Smooth.None),Line(visible = not useSupport, points = {{90, -100},{80, -120}}, color = {0,0,0}, smooth = Smooth.None),Line(visible = not useSupport, points = {{100, -100},{90, -120}}, color = {0,0,0}, smooth = Smooth.None),Line(visible = not useSupport, points = {{110, -100},{100, -120}}, color = {0,0,0}, smooth = Smooth.None),Line(visible = not useSupport, points = {{120, -100},{110, -120}}, color = {0,0,0}, smooth = Smooth.None)}), Documentation(info = "<HTML>
Base partial model DC machines:
<ul>
<li>main parts of the icon</li>
<li>mechanical shaft</li>
<li>mechanical support</li>
</ul>
Besides the mechanical connector <i>flange</i> (i.e. the shaft) the machines have a second mechanical connector <i>support</i>.<br>
If <i>useSupport</i> = false, it is assumed that the stator is fixed.<br>
Otherwise reaction torque (i.e. airGap torque, minus acceleration torque for stator's moment of inertia) can be measured at <i>support</i>.<br>
One may also fix the the shaft and let rotate the stator; parameter Js is only of importance when the stator is rotating.
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          connect(inertiaRotor.flange_b,flange) annotation(Line(points = {{80, -1.22465e-15},{92, -1.22465e-15},{92,0},{100,0}}, color = {0,0,0}));
          connect(inertiaStator.flange_b,support) annotation(Line(points = {{80, -100},{100, -100}}, color = {0,0,0}));
          connect(internalSupport,inertiaStator.flange_a) annotation(Line(points = {{20, -100},{20, -96},{50, -96},{50, -100},{60, -100}}, color = {0,0,0}, smooth = Smooth.None));
          connect(internalSupport,fixed.flange) annotation(Line(points = {{20, -100},{40, -100}}, color = {0,0,0}, smooth = Smooth.None));
        end PartialBasicMachine;
        partial model PartialBasicInductionMachine "Partial model for induction machine"
          extends PartialBasicMachine(Jr(start = 0.29));
          constant Integer m = 3 "number of phases";
          parameter Integer p(min = 1, start = 2) "number of pole pairs (Integer)";
          parameter Modelica.SIunits.Frequency fsNominal(start = 50) "nominal frequency";
          parameter Modelica.SIunits.Resistance Rs(start = 0.03) "warm stator resistance per phase" annotation(Dialog(group = "Nominal resistances and inductances"));
          parameter Modelica.SIunits.Inductance Lssigma(start = (3 * (1 - sqrt(1 - 0.0667))) / (2 * pi * fsNominal)) "stator stray inductance per phase" annotation(Dialog(group = "Nominal resistances and inductances"));
          output Modelica.SIunits.Voltage vs[m] = plug_sp.pin.v - plug_sn.pin.v "stator instantaneous voltages";
          output Modelica.SIunits.Current is[m] = plug_sp.pin.i "stator instantaneous currents";
          output Modelica.SIunits.Current i_0_s(stateSelect = StateSelect.prefer) = spacePhasorS.zero.i "stator zero-sequence current";
          input Modelica.SIunits.Current idq_ss[2] "stator space phasor current / stator fixed frame";
          input Modelica.SIunits.Current idq_sr[2](each stateSelect = StateSelect.prefer) "stator space phasor current / rotor fixed frame";
          input Modelica.SIunits.Current idq_rs[2] "rotor space phasor current / stator fixed frame";
          input Modelica.SIunits.Current idq_rr[2](each stateSelect = StateSelect.prefer) "rotor space phasor current / rotor fixed frame";
          Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_sp(final m = m) annotation(Placement(transformation(extent = {{50,90},{70,110}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_sn(final m = m) annotation(Placement(transformation(extent = {{ -70,90},{ -50,110}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Basic.Resistor rs(final m = m, final R = fill(Rs, m), final T_ref = fill(293.15, m), final alpha = zeros(m), final useHeatPort = false, final T = rs.T_ref) annotation(Placement(transformation(extent = {{60,50},{40,70}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Basic.Inductor lssigma(final m = m, final L = fill(Lssigma, m)) annotation(Placement(transformation(extent = {{30,50},{10,70}}, rotation = 0)));
          SpacePhasors.Components.SpacePhasor spacePhasorS(final turnsRatio = 1) annotation(Placement(transformation(origin = {0,30}, extent = {{10,10},{ -10, -10}}, rotation = 90)));
          annotation(Documentation(info = "<HTML>
Partial model for induction machine models
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -50,100},{ -20,100},{ -20,70}}, color = {0,0,255}),Line(points = {{50,100},{20,100},{20,70}}, color = {0,0,255})}));

        equation 
          connect(plug_sp,rs.plug_p) annotation(Line(points = {{60,100},{60,60}}, color = {0,0,255}, smooth = Smooth.None));
          connect(rs.plug_n,lssigma.plug_p) annotation(Line(points = {{40,60},{30,60}}, color = {0,0,255}, smooth = Smooth.None));
          connect(lssigma.plug_n,spacePhasorS.plug_p) annotation(Line(points = {{10,60},{10,40}}, color = {0,0,255}, smooth = Smooth.None));
          connect(spacePhasorS.plug_n,plug_sn) annotation(Line(points = {{ -10,40},{ -10,60},{ -60,60},{ -60,100}}, color = {0,0,255}, smooth = Smooth.None));
          connect(spacePhasorS.ground,spacePhasorS.zero) annotation(Line(points = {{ -10,20},{ -10,14},{ -6.12323e-16,14},{ -6.12323e-16,20}}, color = {0,0,255}));
        end PartialBasicInductionMachine;
        partial model PartialBasicDCMachine "Partial model for DC machine"
          extends PartialBasicMachine(Jr(start = 0.15));
          parameter Modelica.SIunits.Voltage VaNominal(start = 100) "nominal armature voltage" annotation(Dialog(group = "Nominal parameters"));
          parameter Modelica.SIunits.Current IaNominal(start = 100) "nominal armature current" annotation(Dialog(group = "Nominal parameters"));
          parameter Modelica.SIunits.AngularVelocity wNominal(displayUnit = "1/min", start = (1425 * 2 * pi) / 60) "nominal speed" annotation(Dialog(group = "Nominal parameters"));
          parameter Modelica.SIunits.Resistance Ra(start = 0.05) "warm armature resistance" annotation(Dialog(group = "Nominal resistances and inductances"));
          parameter Modelica.SIunits.Inductance La(start = 0.0015) "armature inductance" annotation(Dialog(group = "Nominal resistances and inductances"));
          parameter Real turnsRatio "ratio of armature turns over number of turns of the excitation winding";
          output Modelica.SIunits.Voltage va = pin_ap.v - pin_an.v "armature voltage";
          output Modelica.SIunits.Current ia = pin_ap.i "armature current";
          annotation(Documentation(info = "<HTML>
Partial model for DC machine models.
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -50,100},{ -20,100},{ -20,70}}, color = {0,0,255}),Line(points = {{50,100},{20,100},{20,70}}, color = {0,0,255})}));
          Modelica.Electrical.Analog.Interfaces.PositivePin pin_ap annotation(Placement(transformation(extent = {{50,110},{70,90}}, rotation = 0)));
          Modelica.Electrical.Analog.Interfaces.NegativePin pin_an annotation(Placement(transformation(extent = {{ -70,110},{ -50,90}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Resistor ra(final R = Ra, final T_ref = 293.15, final alpha = 0, final useHeatPort = false, final T = ra.T_ref) annotation(Placement(transformation(extent = {{60,50},{40,70}}, rotation = 0)));
          Modelica.Electrical.Analog.Basic.Inductor la(final L = La) annotation(Placement(transformation(extent = {{30,50},{10,70}}, rotation = 0)));

        equation 
          connect(la.p,ra.n) annotation(Line(points = {{30,60},{40,60}}, color = {0,0,255}));
          connect(pin_ap,ra.p) annotation(Line(points = {{60,100},{60,60}}, color = {0,0,255}));
        end PartialBasicDCMachine;
      end Interfaces;
      package Utilities "Library with auxiliary models for testing"
        extends Modelica.Icons.Library;
        annotation(Documentation(info = "<HTML>
This package contains utility components for testing examples.
</HTML>", revisions = "<HTML>
<dl>
  <dt><b>Main Authors:</b></dt>
  <dd>
  <a href=\"http://www.haumer.at/\">Anton Haumer</a><br>
  Technical Consulting &amp; Electrical Engineering<br>
  A-3423 St.Andrae-Woerdern<br>Austria<br>
  email: <a href=\"mailto:a.haumer@haumer.at\">a.haumer@haumer.at</a>
  </dd>
  <dt><b>Copyright:</b></dt>
  <dd>Copyright &copy; 1998-2009, Modelica Association and Anton Haumer.<br>
  <i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
  under the terms of the <b>Modelica license</b>, see the license conditions
  and the accompanying <b>disclaimer</b> in the documentation of package
  Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
  <ul>
  <li> v1.00 2004/09/16 Anton Haumer</li>
  <li> v1.1  2004/10/01 Anton Haumer<br>
       changed naming and structure<br>
       issued to Modelica Standard Library 2.1</li>
  <li> v1.3.1 2004/11/06 Anton Haumer<br>
       small changes in VfController</li>
  <li> v1.6.1 2004/11/22 Anton Haumer<br>
       introduced TerminalBox</li>
  </ul>
</HTML>"));
        block VfController "Voltage-Frequency-Controller"
          extends Modelica.Blocks.Interfaces.SIMO(final nout = m);
          constant Modelica.SIunits.Angle pi = Modelica.Constants.pi;
          parameter Integer m = 3 "number of phases";
          parameter Modelica.SIunits.Voltage VNominal "nominal RMS voltage per phase";
          parameter Modelica.SIunits.Frequency fNominal "nominal frequency";
          parameter Modelica.SIunits.Angle BasePhase = 0 "common phase shift";
          output Modelica.SIunits.Angle x(start = 0, fixed = true) "integrator state";
          output Modelica.SIunits.Voltage amplitude;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -100, -100},{0,60},{80,60}}, color = {0,0,255}),Line(points = {{ -70,0},{ -60.2,29.9},{ -53.8,46.5},{ -48.2,58.1},{ -43.3,65.2},{ -38.3,69.2},{ -33.4,69.8},{ -28.5,67},{ -23.6,61},{ -18.6,52},{ -13,38.6},{ -5.98,18.6},{8.79, -26.9},{15.1, -44},{20.8, -56.2},{25.7, -64},{30.6, -68.6},{35.5, -70},{40.5, -67.9},{45.4, -62.5},{50.3, -54.1},{55.9, -41.3},{63, -21.7},{70,0}}, color = {192,192,192}),Line(points = {{ -40,0},{ -30.2,29.9},{ -23.8,46.5},{ -18.2,58.1},{ -13.3,65.2},{ -8.3,69.2},{ -3.4,69.8},{1.5,67},{6.4,61},{11.4,52},{17,38.6},{24.02,18.6},{38.79, -26.9},{45.1, -44},{50.8, -56.2},{55.7, -64},{60.6, -68.6},{65.5, -70},{70.5, -67.9},{75.4, -62.5},{80.3, -54.1},{85.9, -41.3},{93, -21.7},{100,0}}, color = {192,192,192}),Line(points = {{ -100,0},{ -90.2,29.9},{ -83.8,46.5},{ -78.2,58.1},{ -73.3,65.2},{ -68.3,69.2},{ -63.4,69.8},{ -58.5,67},{ -53.6,61},{ -48.6,52},{ -43,38.6},{ -35.98,18.6},{ -21.21, -26.9},{ -14.9, -44},{ -9.2, -56.2},{ -4.3, -64},{0.6, -68.6},{5.5, -70},{10.5, -67.9},{15.4, -62.5},{20.3, -54.1},{25.9, -41.3},{33, -21.7},{40,0}}, color = {192,192,192})}), Documentation(info = "<HTML>
Simple Voltage-Frequency-Controller.<br>
Amplitude of voltage is linear dependent (VNominal/fNominal) on frequency (input signal \"u\"), but limited by VNominal (nominal RMS voltage per phase).<br>
m sine-waves with amplitudes as described above are provided as output signal \"y\".<br>
The sine-waves are intended to feed a m-phase SignalVoltage.<br>
Phase shifts between sine-waves may be choosen by the user; default values are <i>(k-1)/m*pi for k in 1:m</i>.
</HTML>"));

        equation 
          amplitude = sqrt(2) * VNominal * (if abs(u) < fNominal then abs(u) / fNominal else 1);
          der(x) = 2 * pi * u;
          y = array(amplitude * sin(x + BasePhase - ((k - 1) * 2) / m * pi) for k in 1:m);
        end VfController;
        model SwitchYD "Y-D-switch"
          constant Integer m = 3 "number of phases";
          Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plugSupply(final m = m) annotation(Placement(transformation(extent = {{ -10,90},{10,110}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_sp(final m = m) annotation(Placement(transformation(extent = {{50, -90},{70, -110}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_sn(final m = m) annotation(Placement(transformation(extent = {{ -70, -90},{ -50, -110}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -100, -60},{ -60, -100},{58, -100},{100, -60},{100,60},{60,100},{ -60,100},{ -100,60},{ -100, -60}}, lineColor = {175,175,175}, smooth = Smooth.None, fillColor = {215,215,215}, fillPattern = FillPattern.Solid),Line(points = {{ -80,52},{ -40,0},{ -2,52},{ -40,0},{ -40, -52}}, color = {0,0,255}, thickness = 0.5),Line(points = {{0,40},{0, -40},{80,0},{0,40}}, color = {0,0,255}, thickness = 0.5)}), Documentation(info = "<HTML>
Simple Star-Delta-switch.<br>
If <i>control</i> is false, plug_sp and plug_sn are star connected and plug_sp connected to the supply plug.<br>
If <i>control</i> is true, plug_sp and plug_sn are delta connected and they are connected to the supply plug.
</HTML>"));
          Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) annotation(Placement(transformation(extent = {{20, -50},{40, -30}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Basic.Delta delta(final m = m) annotation(Placement(transformation(extent = {{40, -80},{20, -60}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Ideal.IdealCommutingSwitch idealCommutingSwitch(final m = m) annotation(Placement(transformation(extent = {{ -10, -70},{10, -50}}, rotation = 0)));
          Modelica.Blocks.Interfaces.BooleanInput control[m] annotation(Placement(transformation(extent = {{ -120, -10},{ -100,10}}, rotation = 0)));

        equation 
          connect(delta.plug_p,plugSupply) annotation(Line(points = {{40, -70},{60, -70},{60,100},{0,100}}, color = {0,0,255}));
          connect(delta.plug_p,plug_sp) annotation(Line(points = {{40, -70},{60, -70},{60, -100}}, color = {0,0,255}));
          connect(idealCommutingSwitch.plug_n2,delta.plug_n) annotation(Line(points = {{10, -60},{10, -70},{20, -70}}, color = {0,0,255}));
          connect(idealCommutingSwitch.plug_n1,star.plug_p) annotation(Line(points = {{10, -55},{10, -40},{20, -40}}, color = {0,0,255}));
          connect(idealCommutingSwitch.plug_p,plug_sn) annotation(Line(points = {{ -10, -60},{ -60, -60},{ -60, -100}}, color = {0,0,255}));
          connect(control,idealCommutingSwitch.control) annotation(Line(points = {{ -110,0},{0,0},{0, -52}}, color = {255,0,255}));
        end SwitchYD;
        model TerminalBox "terminal box Y/D-connection"
          constant Integer m = 3 "number of phases";
          parameter String terminalConnection(start = "Y") "choose Y=star/D=delta" annotation(choices(choice = "Y" "Star connection", choice = "D" "Delta connection"));
          Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plug_sp(final m = m) annotation(Placement(transformation(extent = {{50, -90},{70, -110}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.NegativePlug plug_sn(final m = m) annotation(Placement(transformation(extent = {{ -70, -90},{ -50, -110}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Basic.Star star(final m = m) if terminalConnection <> "D" annotation(Placement(transformation(origin = { -70, -80}, extent = {{ -10,10},{10, -10}}, rotation = 180)));
          Modelica.Electrical.MultiPhase.Basic.Delta delta(final m = m) if terminalConnection == "D" annotation(Placement(transformation(extent = {{ -20, -70},{ -40, -50}}, rotation = 0)));
          Modelica.Electrical.MultiPhase.Interfaces.PositivePlug plugSupply(final m = m) annotation(Placement(transformation(extent = {{ -10, -70},{10, -90}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -80, -80},{ -80, -84},{ -80, -120},{ -40, -140},{40, -140},{80, -110},{80, -84},{76, -80},{ -80, -80}}, lineColor = {95,95,95}, fillColor = {135,135,135}, fillPattern = FillPattern.Solid),Text(extent = {{ -40, -90},{40, -130}}, lineColor = {0,0,0}, textString = "%terminalConnection")}), Documentation(info = "<html>
TerminalBox: at the bottom connected to both machine plugs, connect at the top to the grid as usual,<br>
choosing Y-connection (StarDelta=Y) or D-connection (StarDelta=D).
</html>"));
          Modelica.Electrical.Analog.Interfaces.NegativePin starpoint if terminalConnection <> "D" annotation(Placement(transformation(extent = {{ -100, -90},{ -80, -70}}, rotation = 0)));

        equation 
          connect(plug_sn,star.plug_p) annotation(Line(points = {{ -60, -100},{ -60, -80}}, color = {0,0,255}));
          connect(plug_sn,delta.plug_n) annotation(Line(points = {{ -60, -100},{ -40, -100},{ -40, -60}}, color = {0,0,255}));
          connect(delta.plug_p,plug_sp) annotation(Line(points = {{ -20, -60},{60, -60},{60, -100}}, color = {0,0,255}));
          connect(plug_sp,plugSupply) annotation(Line(points = {{60, -100},{0, -100},{0, -80}}, color = {0,0,255}));
          connect(star.pin_n,starpoint) annotation(Line(points = {{ -80, -80},{ -90, -80}}, color = {0,0,255}));
        end TerminalBox;
        record TransformerData "Calculates Impedances from nominal values"
          extends Modelica.Icons.Record;
          parameter Modelica.SIunits.Frequency f(start = 50) "nominal frequency";
          parameter Modelica.SIunits.Voltage V1(start = 100) "primary nominal line-to-line voltage (RMS)";
          parameter String C1(start = "Y") "choose primary connection" annotation(choices(choice = "Y" "Star connection", choice = "D" "Delta connection"));
          parameter Modelica.SIunits.Voltage V2(start = 100) "secondary open circuit line-to-line voltage (RMS) @ primary nominal voltage";
          parameter String C2(start = "y") "choose secondary connection" annotation(choices(choice = "y" "Star connection", choice = "d" "Delta connection", choice = "z" "zig-zag connection"));
          parameter Modelica.SIunits.ApparentPower SNominal(start = 30000.0) "nominal apparent power";
          parameter Real v_sc(final min = 0, final max = 1, start = 0.05) "impedance voltage drop pu";
          parameter Modelica.SIunits.Power P_sc(start = 300) "short-circuit (copper) losses";
          parameter Real n = V1 / V2 "primary voltage (line-to-line) / secondary voltage (line-to-line)" annotation(Dialog(tab = "Result", enable = false));
          final parameter Modelica.SIunits.Voltage V1ph = V1 / (if C1 == "D" then 1 else sqrt(3)) "primary phase voltage (RMS)";
          final parameter Modelica.SIunits.Current I1ph = SNominal / (3 * V1ph) "primary phase current (RMS)";
          final parameter Modelica.SIunits.Voltage V2ph = V2 / (if C2 == "d" then 1 else sqrt(3)) "secondary phase voltage (RMS)";
          final parameter Modelica.SIunits.Current I2ph = SNominal / (3 * V2ph) "secondary phase current (RMS)";
          final parameter Modelica.SIunits.Impedance Z1ph = (0.5 * v_sc * V1ph) / I1ph "primary impedance per phase";
          parameter Modelica.SIunits.Resistance R1 = (0.5 * P_sc) / (3 * I1ph ^ 2) "warm primary resistance per phase" annotation(Dialog(tab = "Result", enable = false));
          parameter Modelica.SIunits.Inductance L1sigma = sqrt(Z1ph ^ 2 - R1 ^ 2) / (2 * Modelica.Constants.pi * f) "primary stray inductance per phase" annotation(Dialog(tab = "Result", enable = false));
          final parameter Modelica.SIunits.Impedance Z2ph = (0.5 * v_sc * V2ph) / I2ph "secondary impedance per phase";
          parameter Modelica.SIunits.Resistance R2 = (0.5 * P_sc) / (3 * I2ph ^ 2) "warm secondary resistance per phase" annotation(Dialog(tab = "Result", enable = false));
          parameter Modelica.SIunits.Inductance L2sigma = sqrt(Z2ph ^ 2 - R2 ^ 2) / (2 * Modelica.Constants.pi * f) "secondary stray inductance per phase" annotation(Dialog(tab = "Result", enable = false));
        end TransformerData;
      end Utilities;
    end Machines;
    package Digital "Library for digital electrical components based on the VHDL standard with 9-valued logic and conversion to 2-,3-,4-valued logic"
      package UsersGuide "User's Guide"
        annotation(__Dymola_DocumentationClass = true, Documentation(info = "<html>
<p>
Library <b>Electrical.Digital</b> is a <b>free</b> Modelica package providing
components to model <b>digital</b> electronic
systems based on combinational and sequential logic in a convenient
way. This package contains the <b>User's Guide</b> for
the library and has the following content:
</p>
<ol>
<li><a href=\"Modelica://Modelica.Electrical.Digital.UsersGuide.ReleaseNotes\">Release Notes</a>
    summarizes the differences between different versions of this library.</li>
<li><a href=\"Modelica://Modelica.Electrical.Digital.UsersGuide.Literature\">Literature</a>
    provides references that have been used to design and implement this
    library.</li>
<li><a href=\"Modelica://Modelica.Electrical.Digital.UsersGuide.Contact\">Contact</a>
    provides information about the authors of the library as well as
    acknowledgments.</li>
</ol>
</html>"));
        class ReleaseNotes "Release notes"
          annotation(Documentation(info = "<html>
<h4>Version 1.0.8, 2009-10-01</h4>
<ul>
<li>Package Register with Flip-Flops and Latches added</li>
<li>InertialDealySensitiveVector added</li>
</ul>
<h4>Version 1.0.7, 2005-07-01</h4>
<ul>
<li>InertialDelaySensitive improved</li>
<li>Minor errors fixed in basic and delay models (final introduced)</li>
<li>Sources.Pulse rewritten to avoid a warning</li>
<li>Minor errors in default values of sources fixed</li>
</ul>
<h4>Version 1.0.6, 2004-10-18</h4>
<ul>
<li> Missing HTML tags added (problems with mismatched pre tags fixed).</li>
<li> CVS ID string deleted.</li>
</ul>
<h4>Version 1.0.5, 2004-10-01</h4>
<ul>
<li> Wrong identifiers x0 and Tdel in HalfAdder example fixed.</li>
<li> Experiment command in FlipFlop example deleted.</li>
<li> Known issue: Pulse source causes a warning in Dymola. It is recommended to use Clock source.</li>
</ul>
<h4>Version 1.0.4, 2004-09-30</h4>
<ul>
<li> Documentation improved.</li>
</ul>
<h4>Version 1.0.3, 2004-09-21</h4>
<ul>
<li> Table names changed from \"map\" to \"Table\".</li>
<li> Icons for converters modified.</li>
<li> LogicValueType renamed to Logic. For the Electrical.Digital library
     the type Logic has a fundamental meaning. Logic is similar to
     Real, Integer or Boolean in other packages. Names for converters
     are now more consistent (LogicToBoolean, RealToLogic etc.).</li>
<li> Icons for gates and sources improved.</li>
<li> New examples added.</li>
<li> Internal names for signals and ports unified.</li>
<li> Simple Clock source added in addition to Pulse source (for convenience reasons).</li>
</ul>
<h4>Version 1.0.2, 2004-09-13</h4>
<ul>
<li> First prerelease for discussions at the 40th Modelica Design Meeting.</li>
</ul>
<h4>Version 1.0.1, 2004-06-01</h4>
<ul>
<li> Packages Tables, Basic, and Gates implemented.</li>
<li> Transport and inertial delay implemented and successfully tested.</li>
</ul>
<h4>Version 1.0.0, 2003-05-01</h4>
<ul>
<li> A first version has been implemented for case studies.</li>
</ul>
</html>
"));
        end ReleaseNotes;
        class Literature "Literature"
          annotation(Documentation(info = "<html>
<p>
The Electrical.Digital library is based on the following references:
</p>
<dl>
<dt>Ashenden, P. J.:</dt>
<dd> <b>The Designer's Guide to VHDL.</b> San Francisco: Morgan Kaufmann, 1995, 688 p. ISBN 1-55860-270-4.
     <br>&nbsp;</dd>
</dl>
<dl>
<dt>IEEE 1076-1993:</dt>
<dd> <b>IEEE Standard VHDL Language Reference Manual (ANSI).</b> 288 p. ISBN 1-55937-376-8. IEEE Ref. SH16840-NYF.
     <br>&nbsp;</dd>
</dl>
<dl>
<dt>IEEE 1164-1993:</dt>
<dd> <b>IEEE Standard Multivalue Logic System for VHDL Model Interoperability (Std_logic_1164).</b> 24 p. ISBN 1-55937-299-0. IEEE Ref. SH16097-NYF.
     <br>&nbsp;</dd>
</dl>
<dl>
<dt>Lipsett, R.; Schaefer, C.; Ussery, C.:</dt>
<dd> <b>VHDL: Hardware Description and Design.</b> Boston: Kluwer, 1989, 299 p. ISBN 079239030X.
     <br>&nbsp;</dd>
</dl>
<dl>
<dt>Navabi, Z:</dt>
<dd> <b>VHDL: Analysis and Modeling of Digital Systems.</b> New York: McGraw-Hill, 1993, 375 p. ISBN 0070464723.
     <br>&nbsp;</dd>
</dl>
</html>
 "));
        end Literature;
        class Contact "Contact"
          annotation(Documentation(info = "<html>
<dl>
<dt><b>Main Authors:</b>
<dd>Christoph Clau&szlig;
    &lt;<A HREF=\"mailto:Christoph.Clauss@eas.iis.fraunhofer.de\">Christoph.Clauss@eas.iis.fraunhofer.de</A>&gt;<br>
    Andr&eacute; Schneider
    &lt;<A HREF=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</A>&gt;<br>
    <a href=\"http://people.eas.iis.fraunhofer.de/Ulrich.Donath/\">Ulrich Donath</a>
    &lt;<A HREF=\"mailto:Ulrich.Donath@eas.iis.fraunhofer.de\">Ulrich.Donath@eas.iis.fraunhofer.de</A>&gt;<br>
    Fraunhofer Institute for Integrated Circuits (IIS)<br>
    Design Automation Department (EAS)<br>
    Zeunerstra&szlig;e 38<br>
    D-01069 Dresden<br>
    Germany<br>
</dl>
<br>
<br>
<dl>
<dt><b>Acknowledgements:</b></dt>
<dd>
    We thank our students Teresa Schlegel and Enrico Weber for implementing
    and carefully testing many models and examples.<br>
</dd>
</dl>
</html>
"));
        end Contact;
      end UsersGuide;
      annotation(Documentation(info = "<html>
<p>
This library contains packages for digital electrical components. Both, type system
and models are based on the VHDL standard (IEEE Std 1076-1987 VHDL, IEEE Std 1076-1993 VHDL,
IEEE Std 1164 Multivalue Logic System):
<ul>
<li>Interfaces: Definition of signals and interfaces</li>
<li>Tables: All truth tables needed</li>
<li>Delay: Transport and inertial delay</li>
<li>Basic: Basic logic without delay</li>
<li>Gates: Basic gates composed by basic components and inertial delay</li>
<li>Tristate: (not yet available)</li>
<li>FlipFlops: D-Flip-Flops</li>
<li>Latches: D-Latches</li>
<li>TransferGates: (not yet available)</li>
<li>Multiplexers (not yet available)</li>
<li>Memory: Ram, Rom, (not yet available)</li>
<li>Sources: Time-dependend signal sources</li>
<li>Converters</li>
<li>Examples</li>
</ul>
</p>
<p>
The logic values are coded by integer values. The following code table is necessary
for both setting of input and interpreting the output values.
</p>
<p><b>Code Table:</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Logic value</b></td>
      <td valign=\"top\"><b>Integer code</b></td>
      <td valign=\"top\"><b>Meaning</b></td>
  </tr>

  <tr><td valign=\"top\">'U'</td> <td valign=\"top\">1</td> <td valign=\"top\">Uninitialized</td></tr>
  <tr><td valign=\"top\">'X'</td> <td valign=\"top\">2</td> <td valign=\"top\">Forcing Unknown</td></tr>
  <tr><td valign=\"top\">'0'</td> <td valign=\"top\">3</td> <td valign=\"top\">Forcing 0</td></tr>
  <tr><td valign=\"top\">'1'</td> <td valign=\"top\">4</td> <td valign=\"top\">Forcing 1</td></tr>
  <tr><td valign=\"top\">'Z'</td> <td valign=\"top\">5</td> <td valign=\"top\">High Impedance</td></tr>
  <tr><td valign=\"top\">'W'</td> <td valign=\"top\">6</td> <td valign=\"top\">Weak Unknown</td></tr>
  <tr><td valign=\"top\">'L'</td> <td valign=\"top\">7</td> <td valign=\"top\">Weak 0</td></tr>
  <tr><td valign=\"top\">'H'</td> <td valign=\"top\">8</td> <td valign=\"top\">Weak 1</td></tr>
  <tr><td valign=\"top\">'-'</td> <td valign=\"top\">9</td> <td valign=\"top\">Don't care</td></tr>
</table>

<p>
The library will be developed in two main steps. The first step contains the basic components and
the gates. In the next step the more complicated devices will be added. Currently the first step of
the library is implemented and released for public use.
</p>

<p>
Copyright &copy; 1998-2009, Modelica Association and Fraunhofer-Gesellschaft.
</p>
<p>
<i>This Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p><br>

</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Polygon(points = {{ -100, -100},{ -100,50},{80,50},{80, -100},{ -100, -100}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{ -100,50},{ -80,70},{100,70},{80,50},{ -100,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{100,70},{100, -80},{80, -100},{80,50},{100,70}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -120,122},{120,73}}, lineColor = {255,0,0}, textString = "%name"),Rectangle(extent = {{ -52,4},{ -32, -36}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{8, -46},{28, -86}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{8,4},{28, -36}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -72, -6},{ -52, -6}}, color = {0,0,0}),Line(points = {{ -72, -28},{ -52, -28}}, color = {0,0,0}),Ellipse(extent = {{ -32, -14},{ -28, -18}}, lineColor = {0,0,0}),Ellipse(extent = {{28, -14},{32, -18}}, lineColor = {0,0,0}),Line(points = {{ -28, -16},{8, -16}}, color = {0,0,0}),Line(points = {{ -28, -16},{ -12, -16},{ -12, -56},{8, -56}}, color = {0,0,0}),Line(points = {{8, -76},{ -72, -76}}, color = {0,0,0}),Line(points = {{32, -16},{48, -16}}, color = {0,0,0}),Line(points = {{28, -66},{48, -66}}, color = {0,0,0}),Text(extent = {{ -32, -8},{ -52,0}}, lineColor = {0,0,0}, textString = "&"),Text(extent = {{28, -10},{8, -2}}, lineColor = {0,0,0}, textString = "1"),Text(extent = {{28, -58},{8, -50}}, lineColor = {0,0,0}, textString = "="),Text(extent = {{ -90,40},{70,10}}, lineColor = {160,160,164}, textString = "Library")}), uses(Modelica(version = "3.1")));
      package Examples "Examples that demonstrate the usage of the Digital electrical components"
        model Multiplexer "4 to 1 Bit Multiplexer Example"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          annotation(Documentation(info = "<html>
<p>The multiplexer converts a parallel 4 bit signal in a sequential 1 bit stream. The multiplexer component is composed by basic gates. It can be found in the Utilities subpackage. The example is designed to test and demonstrate the basic gate components.</p>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,170}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), experiment(StopTime = 250));
          D.Sources.Clock CLK(period = 20) annotation(Placement(transformation(extent = {{ -80, -56},{ -60, -36}}, rotation = 0)));
          D.Sources.Table D0(y0 = 3, x = {4,3,4,3}, t = {50,100,145,200}) annotation(Placement(transformation(extent = {{ -80,58},{ -60,78}}, rotation = 0)));
          D.Sources.Table D1(y0 = 3, x = {4,3,4,3}, t = {22,140,150,180}) annotation(Placement(transformation(extent = {{ -80,32},{ -60,52}}, rotation = 0)));
          D.Examples.Utilities.MUX4 MUX annotation(Placement(transformation(extent = {{ -10,0},{70,80}}, rotation = 0)));
          D.Sources.Table D2(y0 = 3, x = {4,3,4,3}, t = {22,140,150,180}) annotation(Placement(transformation(extent = {{ -80,6},{ -60,26}}, rotation = 0)));
          D.Sources.Table D3(y0 = 3, x = {4,3,4,3}, t = {22,140,150,180}) annotation(Placement(transformation(extent = {{ -80, -20},{ -60,0}}, rotation = 0)));
          D.Examples.Utilities.JKFF FF annotation(Placement(transformation(extent = {{ -20, -62},{0, -42}}, rotation = 0)));
          D.Sources.Set Enable annotation(Placement(transformation(extent = {{ -80, -82},{ -60, -62}}, rotation = 0)));

        equation 
          connect(CLK.y,FF.clk) annotation(Line(points = {{ -60, -46},{ -36, -46},{ -36, -52},{ -20, -52}}, color = {127,0,127}));
          connect(Enable.y,FF.k) annotation(Line(points = {{ -60, -72},{ -30, -72},{ -30, -59},{ -20, -59}}, color = {127,0,127}));
          connect(Enable.y,FF.j) annotation(Line(points = {{ -60, -72},{ -30, -72},{ -30, -45},{ -20, -45}}, color = {127,0,127}));
          connect(CLK.y,MUX.a0) annotation(Line(points = {{ -60, -46},{ -36, -46},{ -36,22.4},{ -10,22.4}}, color = {127,0,127}));
          connect(D0.y,MUX.d0) annotation(Line(points = {{ -60,68},{ -10,68}}, color = {127,0,127}));
          connect(D1.y,MUX.d1) annotation(Line(points = {{ -60,42},{ -54,42},{ -54,57.6},{ -10,57.6}}, color = {127,0,127}));
          connect(D2.y,MUX.d2) annotation(Line(points = {{ -60,16},{ -50,16},{ -50,47.2},{ -10,47.2}}, color = {127,0,127}));
          connect(D3.y,MUX.d3) annotation(Line(points = {{ -60, -10},{ -46, -10},{ -46,36.8},{ -10,36.8}}, color = {127,0,127}));
          connect(FF.q,MUX.a1) annotation(Line(points = {{0, -45},{2, -45},{2, -22},{ -20, -22},{ -20,12},{ -10,12}}, color = {127,0,127}));
        end Multiplexer;
        model FlipFlop "Pulse Triggered Master Slave Flip-Flop"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          annotation(Documentation(info = "<html>
<p>A pulse-triggered master-slave flip-flop is demonstrated. The flipflop component is composed by basic gates. It can be found in the Utilities subpackage. The example is designed to test and demonstrate the basic gate components.</p>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,170}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), experiment(StopTime = 250));
          D.Examples.Utilities.JKFF FF annotation(Placement(transformation(extent = {{ -10, -40},{70,40}}, rotation = 0)));
          D.Sources.Clock CLK(period = 10) annotation(Placement(transformation(extent = {{ -80, -10},{ -60,10}}, rotation = 0)));
          D.Sources.Table J(y0 = 3, x = {4,3,4,3}, t = {50,100,145,200}) annotation(Placement(transformation(extent = {{ -80,18},{ -60,38}}, rotation = 0)));
          D.Sources.Table K(y0 = 3, x = {4,3,4,3}, t = {22,140,150,180}) annotation(Placement(transformation(extent = {{ -80, -38},{ -60, -18}}, rotation = 0)));

        equation 
          connect(J.y,FF.j) annotation(Line(points = {{ -60,28},{ -10,28}}, color = {127,0,127}));
          connect(CLK.y,FF.clk) annotation(Line(points = {{ -60,0},{ -10,0}}, color = {127,0,127}));
          connect(K.y,FF.k) annotation(Line(points = {{ -60, -28},{ -10, -28}}, color = {127,0,127}));
        end FlipFlop;
        model HalfAdder "Adding circuit for binary numbers without input carry bit"
          import Modelica.Electrical.Digital;
          annotation(Documentation(info = "<HTML>
<P>
This example demonstrates an adding circuit for binary numbers, which internally realizes the interconnection to
And and to Xor in the final sum.
<br>
<br>
1 + 0 = 1<br>
0 + 1 = 1<br>
1 + 1 = 10<br>
0 + 0 = 0
<br>
<br>
<b>a</b> + <b>b</b> = <b>s</b>
<br>(The carry of this adding is <b>c</b>.)
<br>
<br>and
<br>
<br>
<b>a</b> * <b>b</b> = <b>s</b>
<br>  (It is an interconnection to And.)
<br>
<br>
<b>a</b> * <b>b</b> + <b>a</b> * <b>b</b> = <b>a</b> Xor <b>b</b> = <b>c</b>
<br>(It is an interconnection to Xor.)
<br>
<br>
<pre>  <b>a</b>     <b>b</b>     <b>c</b>      <b>s</b>     <b>t</b></pre>

 <pre>  1     0     1      0     1</pre>
 <pre>  0     1     1      0     2</pre>
 <pre>  1     1     0      1     3</pre>
 <pre>  0     0     0      0     4</pre>

<br>
<br>
<b>t</b> is the pick-up instant of the next bit(s) in the simulation.
The simulation stop time should be 5 seconds.
</P>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -88,100},{90, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,170}, fillPattern = FillPattern.Solid),Line(points = {{40,54},{40,60},{ -40,60},{10,0},{ -40, -60},{40, -60},{40, -52}}, color = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), experiment(StopTime = 5));
          Modelica.Electrical.Digital.Sources.Table a(t = {1,2,3,4}, x = {4,3,4,3}, y0 = 3) annotation(Placement(transformation(extent = {{ -80,18},{ -60,38}}, rotation = 0)));
          Modelica.Electrical.Digital.Sources.Table b(x = {4,3}, t = {2,4}, y0 = 3) annotation(Placement(transformation(extent = {{ -80, -38},{ -60, -18}}, rotation = 0)));
          Modelica.Electrical.Digital.Examples.Utilities.HalfAdder Adder(delayTime = 0.3) annotation(Placement(transformation(extent = {{ -40, -40},{40,40}}, rotation = 0)));
          Modelica.Electrical.Digital.Converters.LogicToReal s annotation(Placement(transformation(extent = {{60,18},{80,38}}, rotation = 0)));
          Modelica.Electrical.Digital.Converters.LogicToReal c annotation(Placement(transformation(extent = {{60, -38},{80, -18}}, rotation = 0)));

        equation 
          connect(b.y,Adder.b) annotation(Line(points = {{ -60, -28},{ -40, -28}}, color = {127,0,127}));
          connect(a.y,Adder.a) annotation(Line(points = {{ -60,28},{ -40,28}}, color = {127,0,127}));
          connect(Adder.s,s.x[1]) annotation(Line(points = {{40,28},{65,28}}, color = {127,0,127}));
          connect(Adder.c,c.x[1]) annotation(Line(points = {{40, -28},{65, -28}}, color = {127,0,127}));
        end HalfAdder;
        model FullAdder "Full 1 Bit Adder Example"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          annotation(Documentation(info = "<HTML>
<P>
<br>It is an adding circuit for binary numbers with input carry bit, which consists of two HalfAdders.
<br>
<br>
<b>a</b>.y, <b>b</b>.y and <b>c</b>.y are the inputs of the FullAdder.
<br>
<b>c</b>out = <b>Or1</b>.y and <b>h</b>.s are the outputs of the Fulladder.
<br>
<br>
<b>t</b> is the pick-up instant of the next bit(s) in the simulation.
<pre>   <b>a</b>.y      <b>b</b>.y      <b>c</b>.y     <b>c</b>out        <b>h</b>.s        <b>t</b> </pre>

<pre>
     1        0        0        0          1        1
     0        1        0        0          1        2
     0        0        1        0          1        3
     1        1        0        1          0        4
     0        1        1        1          0        5
     1        0        1        1          0        6
     1        1        1        1          1        7
     0        0        0        0          0        8
</pre>
</P>
The simulation stop time should be 10 seconds.
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,170}, fillPattern = FillPattern.Solid),Line(points = {{40,54},{40,60},{ -40,60},{10,0},{ -40, -60},{40, -60},{40, -52}}, color = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), experiment(StopTime = 10));
          D.Examples.Utilities.FullAdder Adder1 annotation(Placement(transformation(extent = {{0, -30},{60,30}}, rotation = 0)));
          D.Converters.LogicToReal s annotation(Placement(transformation(extent = {{70,12},{90,32}}, rotation = 0)));
          D.Converters.LogicToReal c_out annotation(Placement(transformation(extent = {{70, -32},{90, -12}}, rotation = 0)));
          D.Examples.Utilities.Counter3 Counter annotation(Placement(transformation(extent = {{ -60, -18},{ -20,22}}, rotation = 0)));
          D.Sources.Set Enable(x = L.'1') annotation(Placement(transformation(extent = {{ -90,6},{ -70,26}}, rotation = 0)));
          D.Sources.Clock CLK annotation(Placement(transformation(extent = {{ -90, -22},{ -70, -2}}, rotation = 0)));

        equation 
          connect(Adder1.s,s.x[1]) annotation(Line(points = {{60.3,21},{68,21},{68,22},{75,22}}, color = {127,0,127}));
          connect(Adder1.c_out,c_out.x[1]) annotation(Line(points = {{60, -21},{68, -21},{68, -22},{75, -22}}, color = {127,0,127}));
          connect(CLK.y,Counter.count) annotation(Line(points = {{ -70, -12},{ -60, -12}}, color = {127,0,127}));
          connect(Enable.y,Counter.enable) annotation(Line(points = {{ -70,16},{ -60,16}}, color = {127,0,127}));
          connect(Counter.q2,Adder1.a) annotation(Line(points = {{ -20,16},{ -10,16},{ -10,21},{0,21}}, color = {127,0,127}));
          connect(Counter.q1,Adder1.b) annotation(Line(points = {{ -20,2},{ -10,2},{ -10,9},{0,9}}, color = {127,0,127}));
          connect(Counter.q0,Adder1.c_in) annotation(Line(points = {{ -20, -12},{ -10, -12},{ -10, -21},{0, -21}}, color = {127,0,127}));
        end FullAdder;
        model Adder4 "4 Bit Adder Example"
          import Modelica.Electrical.Digital;
          annotation(Documentation(info = "<HTML>
<P>
Four Fulladders are combined to built a four bit adder unit.
<br>
<br>
In dependence on time five additions are carried out:
<br>
<pre>
at t = 0                            at t = 1
 a       0 0 0 0                       a      1 1 1 0
 b    +  0 0 0 0                       b   +  1 0 1 1
 <b>s     0 0 0 0 0</b>                      <b>s     1 0 0 1 0</b>
at t = 2                             at t = 3
 a       0 1 1 0                       a      1 1 1 0
 b    +  0 0 1 1                       b   +  1 0 1 0
 <b>s     1 0 1 0 0</b>                      <b>s     0 0 0 1 1</b>

at t = 4
 a      1 1 0 0
 b   +  1 1 1 0
 <b>s    0 0 1 0 1</b>
 </pre>
To show the influence of delay a large delay time of 0.1s is choosen.
Furthermore, all signals are initialized with U, the unitialized value.
Please remember, that the nine logic values are coded by the numbers 1,...,9.
The summands a and b can be found at the output signals of the taba and tabb sources.
The result can be seen in the output signals of the Fulladders according to:
<pre>
      a                       <b>a4</b>.y      <b>a3</b>.y      <b>a2</b>.y      <b>a1</b>.y
      b                       <b>b4</b>.y      <b>b3</b>.y      <b>b2</b>.y      <b>b1</b>.y
      sum   <b>Adder4</b>.c_out  <b>Adder4.s</b>  <b>Adder3.s</b>  <b>Adder2.s</b>  <b>Adder1.s</b>
</pre>
The simulation stop time has to be 5s.

</P>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -200, -200},{200,200}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -200,200},{200, -200}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,170}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -200, -200},{200,200}}, grid = {2,2}), graphics), experiment(StopTime = 5));
          Modelica.Electrical.Digital.Sources.Table b4(y0 = 3, x = {4,3}, t = {1,3}) annotation(Placement(transformation(extent = {{70, -20},{110,20}}, rotation = 0)));
          Modelica.Electrical.Digital.Sources.Table b1(x = {4,3,4}, y0 = 3, t = {1,2,3}) annotation(Placement(transformation(extent = {{ -170, -20},{ -130,20}}, rotation = 0)));
          Modelica.Electrical.Digital.Sources.Table b2(y0 = 3, x = {4}, t = {4}) annotation(Placement(transformation(extent = {{ -90, -20},{ -50,20}}, rotation = 0)));
          Modelica.Electrical.Digital.Sources.Table b3(y0 = 3, x = {4}, t = {1}) annotation(Placement(transformation(extent = {{ -10, -20},{30,20}}, rotation = 0)));
          Modelica.Electrical.Digital.Sources.Table a1(y0 = 3, x = {4,3,4}, t = {1,2,3}) annotation(Placement(transformation(extent = {{ -170,40},{ -130,80}}, rotation = 0)));
          Modelica.Electrical.Digital.Sources.Table a2(y0 = 3, x = {4}, t = {1}) annotation(Placement(transformation(extent = {{ -90,40},{ -50,80}}, rotation = 0)));
          Modelica.Electrical.Digital.Sources.Table a3(y0 = 3, x = {4,3}, t = {1,4}) annotation(Placement(transformation(extent = {{ -8,40},{30,80}}, rotation = 0)));
          Modelica.Electrical.Digital.Sources.Table a4(y0 = 3, x = {3}, t = {1}) annotation(Placement(transformation(extent = {{70,40},{110,80}}, rotation = 0)));
          Modelica.Electrical.Digital.Sources.Set Set(x = 3) annotation(Placement(transformation(origin = { -150, -74}, extent = {{20,20},{ -20, -20}}, rotation = 180)));
          Modelica.Electrical.Digital.Examples.Utilities.FullAdder Adder1 annotation(Placement(transformation(extent = {{ -100, -80},{ -60, -40}}, rotation = 0)));
          Modelica.Electrical.Digital.Examples.Utilities.FullAdder Adder2 annotation(Placement(transformation(extent = {{ -20, -80},{20, -40}}, rotation = 0)));
          Modelica.Electrical.Digital.Examples.Utilities.FullAdder Adder3 annotation(Placement(transformation(extent = {{60, -80},{100, -40}}, rotation = 0)));
          Modelica.Electrical.Digital.Examples.Utilities.FullAdder Adder4 annotation(Placement(transformation(extent = {{140, -80},{180, -40}}, rotation = 0)));

        equation 
          connect(b1.y,Adder1.b) annotation(Line(points = {{ -130,0},{ -120,0},{ -120, -54},{ -100, -54}}, color = {127,0,127}));
          connect(a1.y,Adder1.a) annotation(Line(points = {{ -130,60},{ -110,60},{ -110, -46},{ -100, -46}}, color = {127,0,127}));
          connect(Set.y,Adder1.c_in) annotation(Line(points = {{ -130, -74},{ -100, -74}}, color = {127,0,127}));
          connect(Adder1.c_out,Adder2.c_in) annotation(Line(points = {{ -60, -74},{ -20, -74}}, color = {127,0,127}));
          connect(Adder2.c_out,Adder3.c_in) annotation(Line(points = {{20, -74},{60, -74}}, color = {127,0,127}));
          connect(Adder3.c_out,Adder4.c_in) annotation(Line(points = {{100, -74},{140, -74}}, color = {127,0,127}));
          connect(b2.y,Adder2.b) annotation(Line(points = {{ -50,0},{ -40,0},{ -40, -54},{ -20, -54}}, color = {127,0,127}));
          connect(a2.y,Adder2.a) annotation(Line(points = {{ -50,60},{ -30,60},{ -30, -46},{ -20, -46}}, color = {127,0,127}));
          connect(b3.y,Adder3.b) annotation(Line(points = {{30,0},{40,0},{40, -54},{60, -54}}, color = {127,0,127}));
          connect(a3.y,Adder3.a) annotation(Line(points = {{30,60},{50,60},{50, -46},{60, -46}}, color = {127,0,127}));
          connect(b4.y,Adder4.b) annotation(Line(points = {{110,0},{120,0},{120, -54},{140, -54}}, color = {127,0,127}));
          connect(a4.y,Adder4.a) annotation(Line(points = {{110,60},{130,60},{130, -46},{140, -46}}, color = {127,0,127}));
        end Adder4;
        model Counter3 "3 Bit Counter Example"
          import D = Modelica.Electrical.Digital;
          annotation(Documentation(info = "<html>
<p>The three bit counter example is built up by components of the Utilities package which uses components of the Gates library.</p>
<p>The counter counts the high-low slopes of the clock signal, if the enable signal is set to be true. Otherwise if it is set to zero the counter is not counting.</p>
<p>Plot Counter.count (clock signal), and Counter.enable, and the output bit signals Counter.q0, Counter.q1, and Counter.q2.</p>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,170}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), experiment(StopTime = 10));
          D.Sources.Step Enable annotation(Placement(transformation(extent = {{ -90,8},{ -50,48}}, rotation = 0)));
          D.Sources.Clock Clock annotation(Placement(transformation(extent = {{ -90, -48},{ -50, -8}}, rotation = 0)));
          D.Examples.Utilities.Counter3 Counter annotation(Placement(transformation(extent = {{ -30, -40},{50,40}}, rotation = 0)));

        equation 
          connect(Enable.y,Counter.enable) annotation(Line(points = {{ -50,28},{ -30,28}}, color = {127,0,127}));
          connect(Clock.y,Counter.count) annotation(Line(points = {{ -50, -28},{ -30, -28}}, color = {127,0,127}));
        end Counter3;
        model Counter "Generic N Bit Counter Example"
          import D = Modelica.Electrical.Digital;
          annotation(Documentation(info = "<html>
<p>The counter example is built up by components of the Utilities package which uses components of the Gates library. It demonstrates the generic counter model. The parameter n is the number of counting bits. In this example it is set to 4.</p>
<p>The counter counts the high-low slopes of the clock signal, if the enable signal is set to be true. Otherwise if it is set to zero the counter is not counting.</p>
<p>Plot Counter.count (clock signal), and Counter.enable, and the output bit signals Counter.q[0], Counter.q[1], Counter.q[2], and Counter.q[3]</p>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,170}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), experiment(StopTime = 100));
          D.Sources.Step Enable annotation(Placement(transformation(extent = {{ -90,8},{ -50,48}}, rotation = 0)));
          D.Sources.Clock Clock annotation(Placement(transformation(extent = {{ -90, -48},{ -50, -8}}, rotation = 0)));
          D.Examples.Utilities.Counter Counter(n = 4) annotation(Placement(transformation(extent = {{ -30, -40},{50,40}}, rotation = 0)));
          D.Converters.LogicToReal Q0 annotation(Placement(transformation(extent = {{66, -40},{86, -20}}, rotation = 0)));
          D.Converters.LogicToReal Q1 annotation(Placement(transformation(extent = {{66, -20},{86,0}}, rotation = 0)));
          D.Converters.LogicToReal Q2 annotation(Placement(transformation(extent = {{66,0},{86,20}}, rotation = 0)));
          D.Converters.LogicToReal Q3 annotation(Placement(transformation(extent = {{66,20},{86,40}}, rotation = 0)));

        equation 
          connect(Enable.y,Counter.enable) annotation(Line(points = {{ -50,28},{ -30,28}}, color = {127,0,127}));
          connect(Clock.y,Counter.count) annotation(Line(points = {{ -50, -28},{ -30, -28}}, color = {127,0,127}));
          connect(Q0.x[1],Counter.q[1]) annotation(Line(points = {{71, -30},{58, -30},{58, -24},{50, -24}}, color = {127,0,127}));
          connect(Q1.x[1],Counter.q[2]) annotation(Line(points = {{71, -10},{60, -10},{60, -8},{50, -8}}, color = {127,0,127}));
          connect(Q2.x[1],Counter.q[3]) annotation(Line(points = {{71,10},{60,10},{60,8},{50,8}}, color = {127,0,127}));
          connect(Q3.x[1],Counter.q[4]) annotation(Line(points = {{71,30},{60,30},{60,24},{50,24}}, color = {127,0,127}));
        end Counter;
        model Delay_test "Vector delay"
          Modelica.Electrical.Digital.Delay.InertialDelaySensitiveVector delay(final tHL = 1, final tLH = 2, final n = 3) annotation(Placement(transformation(extent = {{ -36, -28},{40,48}})));
          Modelica.Electrical.Digital.Sources.Table table(x = {3,4,3,4,3}, t = {0,1,5,7,8}) annotation(Placement(transformation(extent = {{ -96,40},{ -76,60}})));
          Modelica.Electrical.Digital.Sources.Table table1(x = {3,4}, t = {0,1}) annotation(Placement(transformation(extent = {{ -96,0},{ -76,20}})));
          Modelica.Electrical.Digital.Sources.Table table2(x = {3,4,3}, t = {0,1,6}) annotation(Placement(transformation(extent = {{ -96, -50},{ -76, -30}})));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -80,80},{80, -80}}, lineColor = {127,0,127}, fillColor = {255,213,170}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>This example is a simple test of the vector valued sensitive delay component. The delay times are chosen different from each other. To examine the rersults plot bothe the input vector x and the output vector y.</p>
</html>"));

        equation 
          connect(table.y,delay.x[1]) annotation(Line(points = {{ -76,50},{ -30.68,50},{ -30.68,6.13667}}, color = {127,0,127}, smooth = Smooth.None));
          connect(table1.y,delay.x[2]) annotation(Line(points = {{ -76,10},{ -30.68,10},{ -30.68,9.81}}, color = {127,0,127}, smooth = Smooth.None));
          connect(table2.y,delay.x[3]) annotation(Line(points = {{ -76, -40},{ -30.68, -40},{ -30.68,13.4833}}, color = {127,0,127}, smooth = Smooth.None));
        end Delay_test;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -100},{80,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{ -100,50},{ -80,70},{100,70},{80,50},{ -100,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{100,70},{100, -80},{80, -100},{80,50},{100,70}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -85,35},{65, -85}}, lineColor = {0,0,255}, textString = "Library"),Text(extent = {{ -120,122},{120,73}}, lineColor = {255,0,0}, textString = "%name")}), Documentation(info = "<html>
<p>This package contains examples that demonstrate the usage of the components of the Electrical.Digital library. </p>
<p>The examples are simple to understand. They will show a typical behavior of the components, and they will give hints to users.</p>
</html>"));
        model DFFREG_test "Pulse triggered D-Register-Bank, high active reset"
          Modelica.Electrical.Digital.Sources.Table clock(x = {3,4,3,4,3,4,3}, t = {0,7,8,10,11,15,16}) annotation(Placement(transformation(extent = {{ -86, -32},{ -66, -12}})));
          Modelica.Electrical.Digital.Sources.Table data_0(x = {4,3}, t = {0,12}) annotation(Placement(transformation(extent = {{ -86,8},{ -66,28}})));
          Modelica.Electrical.Digital.Sources.Table reset(x = {3,4,3}, t = {0,1,2}) annotation(Placement(transformation(extent = {{ -86, -70},{ -66, -50}})));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -80,80},{80, -80}}, lineColor = {127,33,107}, fillColor = {255,213,170}, fillPattern = FillPattern.Solid)}), experiment(StopTime = 25), experimentSetupOutput, Documentation(info = "<html>
<p>This example is a simple test of the Registers.DFFREG component. The data width is set to two. After simulation plot both the dataIn and the dataOut vectors. To verify the results compare the the truth table which is documented in the DFFREG component.</p>
</html>"));
          Modelica.Electrical.Digital.Sources.Table data_1(t = {0,10}, x = {8,2}) annotation(Placement(transformation(extent = {{ -86,40},{ -66,60}})));
          Modelica.Electrical.Digital.Registers.DFFREG dFFREG(n = 2, tHL = 5, tLH = 6) annotation(Placement(transformation(extent = {{ -24, -26},{70,68}})));

        equation 
          connect(clock.y,dFFREG.clock) annotation(Line(points = {{ -66, -22},{ -46, -22},{ -46,11.6},{ -20.24,11.6}}, color = {127,0,127}, smooth = Smooth.None));
          connect(reset.y,dFFREG.reset) annotation(Line(points = {{ -66, -60},{ -20.24, -60},{ -20.24, -7.2}}, color = {127,0,127}, smooth = Smooth.None));
          connect(data_0.y,dFFREG.dataIn[1]) annotation(Line(points = {{ -66,18},{ -44,18},{ -44,37.92},{ -20.24,37.92}}, color = {127,0,127}, smooth = Smooth.None));
          connect(data_1.y,dFFREG.dataIn[2]) annotation(Line(points = {{ -66,50},{ -44,50},{ -44,41.68},{ -20.24,41.68}}, color = {127,0,127}, smooth = Smooth.None));
        end DFFREG_test;
        model DFFREGL_test "Pulse triggered D-Register-Bank, low active reset"
          Modelica.Electrical.Digital.Sources.Table clock(x = {3,4,3,4,3,4,3}, t = {0,7,8,10,11,15,16}) annotation(Placement(transformation(extent = {{ -86, -30},{ -66, -10}})));
          Modelica.Electrical.Digital.Sources.Table data_0(x = {4,3}, t = {0,12}) annotation(Placement(transformation(extent = {{ -88,10},{ -68,30}})));
          Modelica.Electrical.Digital.Sources.Table reset(t = {0,1,2}, x = {4,3,4}) annotation(Placement(transformation(extent = {{ -86, -70},{ -66, -50}})));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -80,80},{80, -80}}, lineColor = {127,33,107}, fillColor = {255,213,170}, fillPattern = FillPattern.Solid)}), experiment(StopTime = 25), experimentSetupOutput, Documentation(info = "<html>
<p>This example is a simple test of the Registers.DFFREGL component. The data width is set to two. After simulation plot both the dataIn and the dataOut vectors. To verify the results compare the the truth table which is documented in the DFFREGL component.</p>
</html>"));
          Modelica.Electrical.Digital.Sources.Table data_1(t = {0,10}, x = {8,2}) annotation(Placement(transformation(extent = {{ -86,40},{ -66,60}})));
          Modelica.Electrical.Digital.Registers.DFFREGL dFFREGL(n = 2, tHL = 5, tLH = 6) annotation(Placement(transformation(extent = {{ -41, -39},{62,65}})));

        equation 
          connect(reset.y,dFFREGL.reset) annotation(Line(points = {{ -66, -60},{ -36.88, -60},{ -36.88, -18.2}}, color = {127,0,127}, pattern = LinePattern.None, smooth = Smooth.None));
          connect(clock.y,dFFREGL.clock) annotation(Line(points = {{ -66, -20},{ -62, -20},{ -62,2.6},{ -36.88,2.6}}, color = {127,0,127}, pattern = LinePattern.None, smooth = Smooth.None));
          connect(data_0.y,dFFREGL.dataIn[1]) annotation(Line(points = {{ -68,20},{ -52,20},{ -52,31.72},{ -36.88,31.72}}, color = {127,0,127}, pattern = LinePattern.None, smooth = Smooth.None));
          connect(data_1.y,dFFREGL.dataIn[2]) annotation(Line(points = {{ -66,50},{ -52,50},{ -52,35.88},{ -36.88,35.88}}, color = {127,0,127}, pattern = LinePattern.None, smooth = Smooth.None));
        end DFFREGL_test;
        model DFFREGSRH_test "Pulse triggered D-Register-Bank, high active set and reset"
          Modelica.Electrical.Digital.Sources.Table clock(x = {3,4,3}, t = {0,10,11}) annotation(Placement(transformation(extent = {{ -86, -32},{ -66, -12}})));
          Modelica.Electrical.Digital.Sources.Table data_0(x = {6}, t = {0}) annotation(Placement(transformation(extent = {{ -86,8},{ -66,28}})));
          Modelica.Electrical.Digital.Sources.Table reset(x = {3,4,3}, t = {0,1,2}) annotation(Placement(transformation(extent = {{ -86, -70},{ -66, -50}})));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -80,80},{80, -80}}, lineColor = {127,33,107}, fillColor = {255,213,170}, fillPattern = FillPattern.Solid)}), experiment(StopTime = 15), experimentSetupOutput, Documentation(info = "<html>
<p>This example is a simple test of the Registers.DFFREGSRH component. The data width is set to two. After simulation plot both the dataIn and the dataOut vectors. To verify the results compare the the truth table which is documented in the DFFREGSRH component.</p>
</html>"));
          Modelica.Electrical.Digital.Sources.Table data_1(x = {3}, t = {0}) annotation(Placement(transformation(extent = {{ -86,40},{ -66,60}})));
          Modelica.Electrical.Digital.Sources.Table set(x = {3,4,3}, t = {0,5,6}) annotation(Placement(transformation(extent = {{ -86,74},{ -66,94}})));
          Modelica.Electrical.Digital.Registers.DFFREGSRH dFFREGSRH(tHL = 2, tLH = 3, n = 2) annotation(Placement(transformation(extent = {{ -34, -37},{73,71}})));

        equation 
          connect(clock.y,dFFREGSRH.clock) annotation(Line(points = {{ -66, -22},{ -50, -22},{ -50,6.2},{ -29.72,6.2}}, color = {127,0,127}, smooth = Smooth.None));
          connect(set.y,dFFREGSRH.set) annotation(Line(points = {{ -66,84},{ -48,84},{ -48,60.2},{ -29.72,60.2}}, color = {127,0,127}, smooth = Smooth.None));
          connect(reset.y,dFFREGSRH.reset) annotation(Line(points = {{ -66, -60},{ -29.72, -60},{ -29.72, -15.4}}, color = {127,0,127}, smooth = Smooth.None));
          connect(data_0.y,dFFREGSRH.dataIn[1]) annotation(Line(points = {{ -66,18},{ -48,18},{ -48,36.44},{ -29.72,36.44}}, color = {127,0,127}, smooth = Smooth.None));
          connect(data_1.y,dFFREGSRH.dataIn[2]) annotation(Line(points = {{ -66,50},{ -48,50},{ -48,40.76},{ -29.72,40.76}}, color = {127,0,127}, smooth = Smooth.None));
        end DFFREGSRH_test;
        model DFFREGSRL_test "Pulse triggered D-Register-Bank, low active set and reset"
          Modelica.Electrical.Digital.Sources.Table clock(x = {3,4,3}, t = {0,10,11}) annotation(Placement(transformation(extent = {{ -86, -32},{ -66, -12}})));
          Modelica.Electrical.Digital.Sources.Table data_0(x = {6}, t = {0}) annotation(Placement(transformation(extent = {{ -86,8},{ -66,28}})));
          Modelica.Electrical.Digital.Sources.Table reset(t = {0,1,2}, x = {4,3,4}) annotation(Placement(transformation(extent = {{ -86, -70},{ -66, -50}})));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -80,80},{80, -80}}, lineColor = {127,33,107}, fillColor = {255,213,170}, fillPattern = FillPattern.Solid)}), experiment(StopTime = 15), experimentSetupOutput, Documentation(info = "<html>
<p>This example is a simple test of the Registers.DFFREGSRL component. The data width is set to two. After simulation plot both the dataIn and the dataOut vectors. To verify the results compare the the truth table which is documented in the DFFREGSRL component.</p>
</html>"));
          Modelica.Electrical.Digital.Sources.Table data_1(x = {3}, t = {0}) annotation(Placement(transformation(extent = {{ -86,40},{ -66,60}})));
          Modelica.Electrical.Digital.Sources.Table set(t = {0,5,6}, x = {4,3,4}) annotation(Placement(transformation(extent = {{ -86,74},{ -66,94}})));
          Modelica.Electrical.Digital.Registers.DFFREGSRL dFFREGSRL(tHL = 2, tLH = 3, n = 2) annotation(Placement(transformation(extent = {{ -45, -54},{81,72}})));

        equation 
          connect(reset.y,dFFREGSRL.reset) annotation(Line(points = {{ -66, -60},{ -39.96, -60},{ -39.96, -28.8}}, color = {127,0,127}, smooth = Smooth.None));
          connect(clock.y,dFFREGSRL.clock) annotation(Line(points = {{ -66, -22},{ -56, -22},{ -56, -3.6},{ -39.96, -3.6}}, color = {127,0,127}, smooth = Smooth.None));
          connect(set.y,dFFREGSRL.set) annotation(Line(points = {{ -66,84},{ -39.96,84},{ -39.96,59.4}}, color = {127,0,127}, smooth = Smooth.None));
          connect(data_0.y,dFFREGSRL.dataIn[1]) annotation(Line(points = {{ -66,18},{ -56,18},{ -56,31.68},{ -39.96,31.68}}, color = {127,0,127}, smooth = Smooth.None));
          connect(data_1.y,dFFREGSRL.dataIn[2]) annotation(Line(points = {{ -66,50},{ -56,50},{ -56,36.72},{ -39.96,36.72}}, color = {127,0,127}, smooth = Smooth.None));
        end DFFREGSRL_test;
        model DLATREG_test "Level sensitive D-Register-Bank, high active reset"
          Modelica.Electrical.Digital.Sources.Table enable(x = {3,4,3}, t = {0,10,18}) annotation(Placement(transformation(extent = {{ -86, -32},{ -66, -12}})));
          Modelica.Electrical.Digital.Sources.Table data_0(x = {6,4}, t = {0,15}) annotation(Placement(transformation(extent = {{ -86,8},{ -66,28}})));
          Modelica.Electrical.Digital.Sources.Table reset(x = {3,4,3,4,3}, t = {0,1,2,20,21}) annotation(Placement(transformation(extent = {{ -86, -70},{ -66, -50}})));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -80,80},{80, -80}}, lineColor = {127,33,107}, fillColor = {255,213,170}, fillPattern = FillPattern.Solid)}), experiment(StopTime = 25), experimentSetupOutput, Documentation(info = "<html>
<p>This example is a simple test of the Registers.DLATREG component. The data width is set to two. After simulation plot both the dataIn and the dataOut vectors. To verify the results compare the the truth table which is documented in the DLATREG component.</p>
</html>"));
          Modelica.Electrical.Digital.Sources.Table data_1(x = {3,4}, t = {0,16}) annotation(Placement(transformation(extent = {{ -86,40},{ -66,60}})));
          Modelica.Electrical.Digital.Registers.DLATREG dLATREG(n = 2, tHL = 2, tLH = 3) annotation(Placement(transformation(extent = {{ -45, -50},{84,79}})));

        equation 
          connect(reset.y,dLATREG.reset) annotation(Line(points = {{ -66, -60},{ -39.84, -60},{ -39.84, -24.2}}, color = {127,0,127}, smooth = Smooth.None));
          connect(enable.y,dLATREG.enable) annotation(Line(points = {{ -66, -22},{ -56, -22},{ -56,1.6},{ -39.84,1.6}}, color = {127,0,127}, smooth = Smooth.None));
          connect(data_0.y,dLATREG.dataIn[1]) annotation(Line(points = {{ -66,18},{ -56,18},{ -56,37.72},{ -39.84,37.72}}, color = {127,0,127}, smooth = Smooth.None));
          connect(data_1.y,dLATREG.dataIn[2]) annotation(Line(points = {{ -66,50},{ -56,50},{ -56,42.88},{ -39.84,42.88}}, color = {127,0,127}, pattern = LinePattern.None, smooth = Smooth.None));
        end DLATREG_test;
        model DLATREGL_test "Level sensitive D-Register-Bank, low active reset"
          Modelica.Electrical.Digital.Sources.Table enable(x = {3,4,3}, t = {0,10,18}) annotation(Placement(transformation(extent = {{ -86, -32},{ -66, -12}})));
          Modelica.Electrical.Digital.Sources.Table data_0(x = {6,4}, t = {0,15}) annotation(Placement(transformation(extent = {{ -86,8},{ -66,28}})));
          Modelica.Electrical.Digital.Sources.Table reset(t = {0,1,2,20,21}, x = {4,3,4,3,4}) annotation(Placement(transformation(extent = {{ -86, -70},{ -66, -50}})));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -80,80},{80, -80}}, lineColor = {127,33,107}, fillColor = {255,213,170}, fillPattern = FillPattern.Solid)}), experiment(StopTime = 25), experimentSetupOutput, Documentation(info = "<html>
<p>This example is a simple test of the Registers.DLATREGL component. The data width is set to two. After simulation plot both the dataIn and the dataOut vectors. To verify the results compare the the truth table which is documented in the DLATREGL component.</p>
</html>"));
          Modelica.Electrical.Digital.Sources.Table data_1(x = {3,4}, t = {0,16}) annotation(Placement(transformation(extent = {{ -86,40},{ -66,60}})));
          Modelica.Electrical.Digital.Registers.DLATREGL dLATREGL(tHL = 2, tLH = 3, n = 2) annotation(Placement(transformation(extent = {{ -45, -50},{84,79}})));

        equation 
          connect(reset.y,dLATREGL.reset) annotation(Line(points = {{ -66, -60},{ -39.84, -60},{ -39.84, -24.2}}, color = {127,0,127}, smooth = Smooth.None));
          connect(enable.y,dLATREGL.enable) annotation(Line(points = {{ -66, -22},{ -54, -22},{ -54,1.6},{ -39.84,1.6}}, color = {127,0,127}, smooth = Smooth.None));
          connect(data_0.y,dLATREGL.dataIn[1]) annotation(Line(points = {{ -66,18},{ -56,18},{ -56,37.72},{ -39.84,37.72}}, color = {127,0,127}, smooth = Smooth.None));
          connect(data_1.y,dLATREGL.dataIn[2]) annotation(Line(points = {{ -66,50},{ -56,50},{ -56,42.88},{ -39.84,42.88}}, color = {127,0,127}, pattern = LinePattern.None, smooth = Smooth.None));
        end DLATREGL_test;
        model DLATREGSRH_test "Level sensitive D-Register-Bank, high active set and reset"
          Modelica.Electrical.Digital.Sources.Table enable(x = {3,4,3}, t = {0,10,18}) annotation(Placement(transformation(extent = {{ -86, -32},{ -66, -12}})));
          Modelica.Electrical.Digital.Sources.Table data_0(x = {6,4}, t = {0,15}) annotation(Placement(transformation(extent = {{ -86,8},{ -66,28}})));
          Modelica.Electrical.Digital.Sources.Table reset(x = {3,4,3,4,3}, t = {0,1,2,20,21}) annotation(Placement(transformation(extent = {{ -86, -70},{ -66, -50}})));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -80,80},{80, -80}}, lineColor = {127,33,107}, fillColor = {255,213,170}, fillPattern = FillPattern.Solid)}), experiment(StopTime = 25), experimentSetupOutput, Documentation(info = "<html>
<p>This example is a simple test of the Registers.DLATREGSRH component. The data width is set to two. After simulation plot both the dataIn and the dataOut vectors. To verify the results compare the the truth table which is documented in the DLATREGSRH component.</p>
</html>"));
          Modelica.Electrical.Digital.Sources.Table data_1(x = {3,4}, t = {0,16}) annotation(Placement(transformation(extent = {{ -86,40},{ -66,60}})));
          Modelica.Electrical.Digital.Sources.Table set(x = {3,4,3}, t = {0,5,6}) annotation(Placement(transformation(extent = {{ -86,74},{ -66,94}})));
          Modelica.Electrical.Digital.Registers.DLATREGSRH dLATREGSRH(tHL = 2, tLH = 3, n = 2) annotation(Placement(transformation(extent = {{ -45, -42},{69,71}})));

        equation 
          connect(reset.y,dLATREGSRH.reset) annotation(Line(points = {{ -66, -60},{ -40.44, -60},{ -40.44, -19.4}}, color = {127,0,127}, pattern = LinePattern.None, smooth = Smooth.None));
          connect(enable.y,dLATREGSRH.enable) annotation(Line(points = {{ -66, -22},{ -52, -22},{ -52,3.2},{ -40.44,3.2}}, color = {127,0,127}, pattern = LinePattern.None, smooth = Smooth.None));
          connect(data_0.y,dLATREGSRH.dataIn[1]) annotation(Line(points = {{ -66,18},{ -52,18},{ -52,34.84},{ -40.44,34.84}}, color = {127,0,127}, pattern = LinePattern.None, smooth = Smooth.None));
          connect(data_1.y,dLATREGSRH.dataIn[2]) annotation(Line(points = {{ -66,50},{ -52,50},{ -52,39.36},{ -40.44,39.36}}, color = {127,0,127}, pattern = LinePattern.None, smooth = Smooth.None));
          connect(set.y,dLATREGSRH.set) annotation(Line(points = {{ -66,84},{ -40.44,84},{ -40.44,59.7}}, color = {127,0,127}, pattern = LinePattern.None, smooth = Smooth.None));
        end DLATREGSRH_test;
        model DLATREGSRL_test "Level sensitive D-Register-Bank, low active set and reset"
          Modelica.Electrical.Digital.Sources.Table enable(t = {0,10,18}, x = {3,4,3}) annotation(Placement(transformation(extent = {{ -86, -32},{ -66, -12}})));
          Modelica.Electrical.Digital.Sources.Table data_0(x = {6,4}, t = {0,15}) annotation(Placement(transformation(extent = {{ -86,8},{ -66,28}})));
          Modelica.Electrical.Digital.Sources.Table reset(t = {0,1,2,20,21}, x = {4,3,4,3,4}) annotation(Placement(transformation(extent = {{ -86, -70},{ -66, -50}})));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -80,80},{80, -80}}, lineColor = {127,33,107}, fillColor = {255,213,170}, fillPattern = FillPattern.Solid)}), experiment(StopTime = 25), experimentSetupOutput, Documentation(info = "<html>
<p>This example is a simple test of the Registers.DLATREGSRL component. The data width is set to two. After simulation plot both the dataIn and the dataOut vectors. To verify the results compare the the truth table which is documented in the DLATREGSRL component.</p>
</html>"));
          Modelica.Electrical.Digital.Sources.Table data_1(x = {3,4}, t = {0,16}) annotation(Placement(transformation(extent = {{ -86,40},{ -66,60}})));
          Modelica.Electrical.Digital.Sources.Table set(t = {0,5,6}, x = {4,3,4}) annotation(Placement(transformation(extent = {{ -86,74},{ -66,94}})));
          Modelica.Electrical.Digital.Registers.DLATREGSRL dLATREGSRL(tHL = 2, tLH = 3, n = 2) annotation(Placement(transformation(extent = {{ -45, -43},{69,71}})));

        equation 
          connect(reset.y,dLATREGSRL.reset) annotation(Line(points = {{ -66, -60},{ -40.44, -60},{ -40.44, -20.2}}, color = {127,0,127}, pattern = LinePattern.None, smooth = Smooth.None));
          connect(enable.y,dLATREGSRL.enable) annotation(Line(points = {{ -66, -22},{ -56, -22},{ -56,2.6},{ -40.44,2.6}}, color = {127,0,127}, pattern = LinePattern.None, smooth = Smooth.None));
          connect(data_0.y,dLATREGSRL.dataIn[1]) annotation(Line(points = {{ -66,18},{ -56,18},{ -56,34.52},{ -40.44,34.52}}, color = {127,0,127}, pattern = LinePattern.None, smooth = Smooth.None));
          connect(data_1.y,dLATREGSRL.dataIn[2]) annotation(Line(points = {{ -66,50},{ -56,50},{ -56,39.08},{ -40.44,39.08}}, color = {127,0,127}, pattern = LinePattern.None, smooth = Smooth.None));
          connect(set.y,dLATREGSRL.set) annotation(Line(points = {{ -66,84},{ -40.44,84},{ -40.44,59.6}}, color = {127,0,127}, pattern = LinePattern.None, smooth = Smooth.None));
        end DLATREGSRL_test;
        package Utilities "Utility components used by package Examples"
          model MUX4 "4 to 1 Bit Multiplexer"
            import D = Modelica.Electrical.Digital;
            import L = Modelica.Electrical.Digital.Interfaces.Logic;
            annotation(Documentation(info = "<html>
<p>MUX4 is a four bit multiplexer which is built up by And, Not, and Or gates according to the schematic. </p>
<p>The parameters delayTime and q0 are prepared but<b> not</b> yet used in the component. The MUX4 component uses standard values in its components.</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -90,100},{90, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,170}, fillPattern = FillPattern.Solid),Line(points = {{ -60,100},{ -60, -100}}, color = {0,0,0}, thickness = 0.5),Line(points = {{60,100},{60, -100}}, color = {0,0,0}, thickness = 0.5),Text(extent = {{ -86,80},{ -64,60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "D0"),Text(extent = {{64,12},{86, -8}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "D"),Text(extent = {{ -150, -100},{150, -160}}, lineColor = {0,0,255}, textString = "%name"),Text(extent = {{ -60,100},{60,40}}, lineColor = {0,0,0}, textString = "MUX"),Text(extent = {{ -86, -60},{ -64, -80}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "A1"),Text(extent = {{ -86,54},{ -64,34}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "D1"),Text(extent = {{ -86,28},{ -64,8}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "D2"),Text(extent = {{ -86,2},{ -64, -18}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "D3"),Text(extent = {{ -86, -36},{ -64, -56}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "A0")}));
            parameter Modelica.SIunits.Time delayTime = 0.001 "Delay time";
            parameter D.Interfaces.Logic q0 = L.'0' "Initial value";
            D.Interfaces.DigitalInput d0 annotation(Placement(transformation(extent = {{ -110,60},{ -90,80}}, rotation = 0)));
            D.Interfaces.DigitalInput d1 annotation(Placement(transformation(extent = {{ -110,34},{ -90,54}}, rotation = 0)));
            D.Interfaces.DigitalInput d2 annotation(Placement(transformation(extent = {{ -110,8},{ -90,28}}, rotation = 0)));
            D.Interfaces.DigitalInput d3 annotation(Placement(transformation(extent = {{ -110, -18},{ -90,2}}, rotation = 0)));
            D.Interfaces.DigitalInput a0 annotation(Placement(transformation(extent = {{ -110, -54},{ -90, -34}}, rotation = 0)));
            D.Interfaces.DigitalInput a1 annotation(Placement(transformation(extent = {{ -110, -80},{ -90, -60}}, rotation = 0)));
            D.Interfaces.DigitalOutput d annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
            D.Basic.Or Or1(n = 4) annotation(Placement(transformation(extent = {{50,20},{70,40}}, rotation = 0)));
            D.Basic.And And1(n = 3) annotation(Placement(transformation(extent = {{ -20,60},{0,80}}, rotation = 0)));
            D.Basic.And And2(n = 3) annotation(Placement(transformation(extent = {{ -20,34},{0,54}}, rotation = 0)));
            D.Basic.And And3(n = 3) annotation(Placement(transformation(extent = {{ -20,8},{0,28}}, rotation = 0)));
            D.Basic.And And4(n = 3) annotation(Placement(transformation(extent = {{ -20, -18},{0,2}}, rotation = 0)));
            D.Basic.Not Not1 annotation(Placement(transformation(extent = {{ -76, -54},{ -56, -34}}, rotation = 0)));
            D.Basic.Not Not2 annotation(Placement(transformation(extent = {{ -76, -80},{ -56, -60}}, rotation = 0)));

          equation 
            connect(a0,Not1.x) annotation(Line(points = {{ -100, -44},{ -72, -44}}, color = {127,0,127}));
            connect(a1,Not2.x) annotation(Line(points = {{ -100, -70},{ -72, -70}}, color = {127,0,127}));
            connect(d0,And1.x[2]) annotation(Line(points = {{ -100,70},{ -58,70},{ -58,70},{ -16,70}}, color = {127,0,127}));
            connect(d1,And2.x[2]) annotation(Line(points = {{ -100,44},{ -16,44}}, color = {127,0,127}));
            connect(d2,And3.x[2]) annotation(Line(points = {{ -100,18},{ -16,18}}, color = {127,0,127}));
            connect(d3,And4.x[2]) annotation(Line(points = {{ -100, -8},{ -58, -8},{ -58, -8},{ -16, -8}}, color = {127,0,127}));
            connect(And4.y,Or1.x[1]) annotation(Line(points = {{0, -8},{40, -8},{40,24},{54,24}}, color = {127,0,127}));
            connect(And3.y,Or1.x[2]) annotation(Line(points = {{0,18},{20,18},{20,28},{54,28}}, color = {127,0,127}));
            connect(And2.y,Or1.x[3]) annotation(Line(points = {{0,44},{20,44},{20,32},{54,32}}, color = {127,0,127}));
            connect(And1.y,Or1.x[4]) annotation(Line(points = {{0,70},{40,70},{40,36},{54,36}}, color = {127,0,127}));
            connect(Or1.y,d) annotation(Line(points = {{70,30},{80,30},{80,0},{100,0}}, color = {127,0,127}));
            connect(Not1.y,And1.x[3]) annotation(Line(points = {{ -56, -44},{ -50, -44},{ -50,75.3333},{ -16,75.3333}}, color = {127,0,127}));
            connect(Not1.y,And3.x[3]) annotation(Line(points = {{ -56, -44},{ -50, -44},{ -50,23.3333},{ -16,23.3333}}, color = {127,0,127}));
            connect(Not2.y,And1.x[1]) annotation(Line(points = {{ -56, -70},{ -40, -70},{ -40,64.6667},{ -16,64.6667}}, color = {127,0,127}));
            connect(Not2.y,And2.x[1]) annotation(Line(points = {{ -56, -70},{ -40, -70},{ -40,38.6667},{ -16,38.6667}}, color = {127,0,127}));
            connect(a0,And4.x[3]) annotation(Line(points = {{ -100, -44},{ -80, -44},{ -80, -2.66667},{ -16, -2.66667}}, color = {127,0,127}));
            connect(a0,And2.x[3]) annotation(Line(points = {{ -100, -44},{ -80, -44},{ -80,49.3333},{ -16,49.3333}}, color = {127,0,127}));
            connect(a1,And4.x[1]) annotation(Line(points = {{ -100, -70},{ -80, -70},{ -80, -90},{ -30, -90},{ -30, -13.3333},{ -16, -13.3333}}, color = {127,0,127}));
            connect(a1,And3.x[1]) annotation(Line(points = {{ -100, -70},{ -80, -70},{ -80, -90},{ -30, -90},{ -30,12.6667},{ -16,12.6667}}, color = {127,0,127}));
          end MUX4;
          model RS "Unclocked RS FlipFlop"
            import D = Modelica.Electrical.Digital;
            import L = Modelica.Electrical.Digital.Interfaces.Logic;
            annotation(Documentation(info = "<html>
<p>RS is a basic component for e.g. the RS (set-reset) flipflop, which is built up by Nor gates according to the schematic. To avoid a numerical loop a small transport delay is inserted which delay time is a parameter of the RS component. Also its initial value can be set by parameter.</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -90,100},{90, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,170}, fillPattern = FillPattern.Solid),Text(extent = {{ -100,100},{100,40}}, lineColor = {0,0,0}, textString = "RS"),Line(points = {{ -60,100},{ -60, -100}}, color = {0,0,0}, thickness = 0.5),Line(points = {{60,100},{60, -100}}, color = {0,0,0}, thickness = 0.5),Text(extent = {{ -86, -60},{ -64, -80}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "R"),Text(extent = {{ -86,80},{ -64,60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "S"),Text(extent = {{64,80},{86,60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "Q"),Text(extent = {{64, -60},{86, -80}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "QN"),Text(extent = {{ -150, -100},{150, -160}}, lineColor = {0,0,255}, textString = "%name")}));
            parameter Modelica.SIunits.Time delayTime = 0 "Delay time";
            parameter D.Interfaces.Logic q0 = L.'U' "Initial value of output";
            D.Basic.Nor Nor1 annotation(Placement(transformation(extent = {{ -40,42},{0,82}}, rotation = 0)));
            D.Basic.Nor Nor2 annotation(Placement(transformation(extent = {{ -40, -82},{0, -42}}, rotation = 0)));
            D.Interfaces.DigitalInput s annotation(Placement(transformation(extent = {{ -110,60},{ -90,80}}, rotation = 0)));
            D.Interfaces.DigitalInput r annotation(Placement(transformation(extent = {{ -110, -80},{ -90, -60}}, rotation = 0)));
            D.Interfaces.DigitalOutput q annotation(Placement(transformation(extent = {{90,60},{110,80}}, rotation = 0)));
            D.Interfaces.DigitalOutput qn annotation(Placement(transformation(extent = {{90, -80},{110, -60}}, rotation = 0)));
            D.Delay.TransportDelay TD1(delayTime = delayTime, y0 = q0) annotation(Placement(transformation(extent = {{ -60, -64},{ -40, -44}}, rotation = 0)));

          equation 
            connect(s,Nor1.x[2]) annotation(Line(points = {{ -100,70},{ -32,70}}, color = {127,0,127}));
            connect(r,Nor2.x[1]) annotation(Line(points = {{ -100, -70},{ -32, -70}}, color = {127,0,127}));
            connect(Nor2.y,Nor1.x[1]) annotation(Line(points = {{0, -62},{20, -62},{20, -20},{ -70,20},{ -70,54},{ -32,54}}, color = {127,0,127}));
            connect(Nor1.y,qn) annotation(Line(points = {{0,62},{50,62},{50, -70},{100, -70}}, color = {127,0,127}));
            connect(Nor2.y,q) annotation(Line(points = {{0, -62},{70, -62},{70,70},{100,70}}, color = {127,0,127}));
            connect(TD1.y,Nor2.x[2]) annotation(Line(points = {{ -40, -54},{ -32, -54}}, color = {127,0,127}));
            connect(TD1.x,Nor1.y) annotation(Line(points = {{ -56, -54},{ -70, -54},{ -70, -20},{20,20},{20,62},{0,62}}, color = {127,0,127}));
          end RS;
          model RSFF "Unclocked RS FlipFlop"
            import D = Modelica.Electrical.Digital;
            import L = Modelica.Electrical.Digital.Interfaces.Logic;
            annotation(Documentation(info = "<html>
<p>Basing on the RS component RSFF is a RS (set-reset) flipflop composed according the schematic. Its parameter delayTime is the delay time of the RS component transport delay, q0 is the initial value of that delay.</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -90,100},{90, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,170}, fillPattern = FillPattern.Solid),Text(extent = {{ -100,100},{100,40}}, lineColor = {0,0,0}, textString = "RS"),Line(points = {{ -60,100},{ -60, -100}}, color = {0,0,0}, thickness = 0.5),Line(points = {{60,100},{60, -100}}, color = {0,0,0}, thickness = 0.5),Text(extent = {{ -86, -60},{ -64, -80}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "R"),Text(extent = {{ -86,80},{ -64,60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "S"),Text(extent = {{64,80},{86,60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "Q"),Text(extent = {{64, -60},{86, -80}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "QN"),Line(points = {{ -90,20},{ -60,0},{ -90, -20}}, color = {0,0,0}),Text(extent = {{ -150, -100},{150, -160}}, lineColor = {0,0,255}, textString = "%name")}));
            parameter Modelica.SIunits.Time delayTime = 0.01 "Delay time";
            parameter D.Interfaces.Logic q0 = L.'U' "Initial value";
            D.Interfaces.DigitalInput s annotation(Placement(transformation(extent = {{ -110,60},{ -90,80}}, rotation = 0)));
            D.Interfaces.DigitalInput r annotation(Placement(transformation(extent = {{ -110, -80},{ -90, -60}}, rotation = 0)));
            D.Interfaces.DigitalOutput q annotation(Placement(transformation(extent = {{90,60},{110,80}}, rotation = 0)));
            D.Interfaces.DigitalOutput qn "not Q" annotation(Placement(transformation(extent = {{90, -80},{110, -60}}, rotation = 0)));
            D.Interfaces.DigitalInput clk annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
            D.Examples.Utilities.RS RS1(delayTime = delayTime, q0 = q0) annotation(Placement(transformation(extent = {{ -10, -40},{70,40}}, rotation = 0)));
            D.Basic.And And1 annotation(Placement(transformation(extent = {{ -70,8},{ -30,48}}, rotation = 0)));
            D.Basic.And And2 annotation(Placement(transformation(extent = {{ -70, -48},{ -30, -8}}, rotation = 0)));

          equation 
            connect(And2.y,RS1.r) annotation(Line(points = {{ -30, -28},{ -10, -28}}, color = {127,0,127}));
            connect(And1.y,RS1.s) annotation(Line(points = {{ -30,28},{ -10,28}}, color = {127,0,127}));
            connect(s,And1.x[2]) annotation(Line(points = {{ -100,70},{ -70,70},{ -70,36},{ -62,36}}, color = {127,0,127}));
            connect(clk,And1.x[1]) annotation(Line(points = {{ -100,0},{ -70,0},{ -70,20},{ -62,20}}, color = {127,0,127}));
            connect(clk,And2.x[2]) annotation(Line(points = {{ -100,0},{ -70,0},{ -70, -20},{ -62, -20}}, color = {127,0,127}));
            connect(r,And2.x[1]) annotation(Line(points = {{ -100, -70},{ -70, -70},{ -70, -36},{ -62, -36}}, color = {127,0,127}));
            connect(RS1.q,q) annotation(Line(points = {{70,28},{80,28},{80,70},{100,70}}, color = {127,0,127}));
            connect(RS1.qn,qn) annotation(Line(points = {{70, -28},{80, -28},{80, -70},{100, -70}}, color = {127,0,127}));
          end RSFF;
          model DFF "D FlipFlop"
            import D = Modelica.Electrical.Digital;
            import L = Modelica.Electrical.Digital.Interfaces.Logic;
            annotation(Documentation(info = "<html>
<p>Basing on the RS component DFF is a D  flipflop composed according the schematic. Its parameter delayTime is the delay time of the RS component transport delay, q0 is the initial value of that delay.</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -90,100},{90, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,170}, fillPattern = FillPattern.Solid),Line(points = {{ -60,100},{ -60, -100}}, color = {0,0,0}, thickness = 0.5),Line(points = {{60,100},{60, -100}}, color = {0,0,0}, thickness = 0.5),Text(extent = {{ -86,80},{ -64,60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "D"),Text(extent = {{64,80},{86,60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "Q"),Text(extent = {{64, -60},{86, -80}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "QN"),Line(points = {{ -90,20},{ -60,0},{ -90, -20}}, color = {0,0,0}),Text(extent = {{ -150, -100},{150, -160}}, lineColor = {0,0,255}, textString = "%name"),Text(extent = {{ -100,100},{100,40}}, lineColor = {0,0,0}, textString = "D")}));
            parameter Modelica.SIunits.Time Tdel = 0.01 "Delay time";
            parameter L QInit = L.'U' "Initial value";
            D.Interfaces.DigitalInput d annotation(Placement(transformation(extent = {{ -110,60},{ -90,80}}, rotation = 0)));
            D.Interfaces.DigitalOutput q annotation(Placement(transformation(extent = {{90,60},{110,80}}, rotation = 0)));
            D.Interfaces.DigitalOutput qn "not Q" annotation(Placement(transformation(extent = {{90, -80},{110, -60}}, rotation = 0)));
            D.Interfaces.DigitalInput clk annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
            D.Examples.Utilities.RSFF RSFF1 annotation(Placement(transformation(extent = {{ -10, -40},{70,40}}, rotation = 0)));
            D.Basic.Not Not1 annotation(Placement(transformation(extent = {{ -70, -48},{ -30, -8}}, rotation = 0)));

          equation 
            connect(RSFF1.q,q) annotation(Line(points = {{70,28},{80,28},{80,70},{100,70}}, color = {127,0,127}));
            connect(RSFF1.qn,qn) annotation(Line(points = {{70, -28},{80, -28},{80, -70},{100, -70}}, color = {127,0,127}));
            connect(Not1.y,RSFF1.r) annotation(Line(points = {{ -30, -28},{ -10, -28}}, color = {127,0,127}));
            connect(clk,RSFF1.clk) annotation(Line(points = {{ -100,0},{ -10,0}}, color = {127,0,127}));
            connect(d,Not1.x) annotation(Line(points = {{ -100,70},{ -80,70},{ -80, -28},{ -62, -28}}, color = {127,0,127}));
            connect(d,RSFF1.s) annotation(Line(points = {{ -100,70},{ -80,70},{ -80,28},{ -10,28}}, color = {127,0,127}));
          end DFF;
          model JKFF "JK FlipFlop"
            import D = Modelica.Electrical.Digital;
            import L = Modelica.Electrical.Digital.Interfaces.Logic;
            annotation(Documentation(info = "<html>
<p>Basing on the RS component JKFF is a J-K-flipflop composed according the schematic. Its parameter delayTime is the delay time of the RS component transport delay, q0 is the initial value of that delay.</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -90,100},{90, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,170}, fillPattern = FillPattern.Solid),Line(points = {{ -60,100},{ -60, -100}}, color = {0,0,0}, thickness = 0.5),Line(points = {{60,100},{60, -100}}, color = {0,0,0}, thickness = 0.5),Text(extent = {{ -86,80},{ -64,60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "J"),Text(extent = {{64,80},{86,60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "Q"),Text(extent = {{64, -60},{86, -80}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "QN"),Line(points = {{ -90,20},{ -60,0},{ -90, -20}}, color = {0,0,0}),Text(extent = {{ -150, -100},{150, -160}}, lineColor = {0,0,255}, textString = "%name"),Text(extent = {{ -100,100},{100,40}}, lineColor = {0,0,0}, textString = "JK"),Text(extent = {{ -86, -60},{ -64, -80}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "K")}));
            parameter Modelica.SIunits.Time delayTime = 0.001 "Delay time";
            parameter D.Interfaces.Logic q0 = L.'0' "Initial value";
            D.Interfaces.DigitalInput j annotation(Placement(transformation(extent = {{ -110,60},{ -90,80}}, rotation = 0)));
            D.Interfaces.DigitalOutput q annotation(Placement(transformation(extent = {{90,60},{110,80}}, rotation = 0)));
            D.Interfaces.DigitalOutput qn "not Q" annotation(Placement(transformation(extent = {{90, -80},{110, -60}}, rotation = 0)));
            D.Interfaces.DigitalInput clk annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
            D.Interfaces.DigitalInput k annotation(Placement(transformation(extent = {{ -110, -80},{ -90, -60}}, rotation = 0)));
            D.Examples.Utilities.RS RS1(delayTime = delayTime, q0 = q0) annotation(Placement(transformation(extent = {{30, -24},{70,16}}, rotation = 0)));
            D.Examples.Utilities.RS RS2(delayTime = delayTime, q0 = q0) annotation(Placement(transformation(extent = {{ -44, -20},{ -4,20}}, rotation = 0)));
            D.Basic.And And1(n = 3) annotation(Placement(transformation(extent = {{ -70,4},{ -50,24}}, rotation = 0)));
            D.Basic.And And2(n = 3) annotation(Placement(transformation(extent = {{ -70, -24},{ -50, -4}}, rotation = 0)));
            D.Basic.And And3 annotation(Placement(transformation(extent = {{4,0},{24,20}}, rotation = 0)));
            D.Basic.And And4 annotation(Placement(transformation(extent = {{4, -28},{24, -8}}, rotation = 0)));
            D.Basic.Not Not1 annotation(Placement(transformation(extent = {{ -34, -66},{ -14, -46}}, rotation = 0)));

          equation 
            connect(And2.y,RS2.r) annotation(Line(points = {{ -50, -14},{ -44, -14}}, color = {127,0,127}));
            connect(And1.y,RS2.s) annotation(Line(points = {{ -50,14},{ -44,14}}, color = {127,0,127}));
            connect(clk,And2.x[3]) annotation(Line(points = {{ -100,0},{ -74,0},{ -74, -8.66667},{ -66, -8.66667}}, color = {127,0,127}));
            connect(clk,And1.x[1]) annotation(Line(points = {{ -100,0},{ -74,0},{ -74,8.66667},{ -66,8.66667}}, color = {127,0,127}));
            connect(k,And2.x[2]) annotation(Line(points = {{ -100, -70},{ -74, -70},{ -74, -14},{ -66, -14}}, color = {127,0,127}));
            connect(And4.y,RS1.r) annotation(Line(points = {{24, -18},{30, -18}}, color = {127,0,127}));
            connect(And3.y,RS1.s) annotation(Line(points = {{24,10},{30,10}}, color = {127,0,127}));
            connect(RS2.qn,And4.x[2]) annotation(Line(points = {{ -4, -14},{8, -14}}, color = {127,0,127}));
            connect(RS2.q,And3.x[2]) annotation(Line(points = {{ -4,14},{8,14}}, color = {127,0,127}));
            connect(clk,Not1.x) annotation(Line(points = {{ -100,0},{ -80,0},{ -80, -56},{ -30, -56}}, color = {127,0,127}));
            connect(Not1.y,And3.x[1]) annotation(Line(points = {{ -14, -56},{2, -56},{2,6},{8,6}}, color = {127,0,127}));
            connect(Not1.y,And4.x[1]) annotation(Line(points = {{ -14, -56},{2, -56},{2, -22},{8, -22}}, color = {127,0,127}));
            connect(j,And1.x[2]) annotation(Line(points = {{ -100,70},{ -74,70},{ -74,14},{ -66,14}}, color = {127,0,127}));
            connect(RS1.q,And2.x[1]) annotation(Line(points = {{70,10},{80,10},{80, -36},{ -70, -36},{ -70, -19.3333},{ -66, -19.3333}}, color = {127,0,127}));
            connect(RS1.qn,And1.x[3]) annotation(Line(points = {{70, -18},{86, -18},{86,36},{ -70,36},{ -70,19.3333},{ -66,19.3333}}, color = {127,0,127}));
            connect(RS1.qn,q) annotation(Line(points = {{70, -18},{86, -18},{86,70},{100,70}}, color = {127,0,127}));
            connect(RS1.q,qn) annotation(Line(points = {{70,10},{80,10},{80, -70},{100, -70}}, color = {127,0,127}));
          end JKFF;
          model HalfAdder "Half adder"
            parameter Real delayTime = 0 "Delay time";
            Modelica.Electrical.Digital.Interfaces.DigitalInput b annotation(Placement(transformation(extent = {{ -110, -80},{ -90, -60}}, rotation = 0)));
            Modelica.Electrical.Digital.Interfaces.DigitalOutput s annotation(Placement(transformation(extent = {{90,60},{110,80}}, rotation = 0)));
            annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -90,100},{90, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,170}, fillPattern = FillPattern.Solid),Text(extent = {{ -90,80},{ -60,60}}, lineColor = {0,0,0}, textString = "a"),Text(extent = {{ -90, -60},{ -60, -80}}, lineColor = {0,0,0}, textString = "b"),Text(extent = {{60,80},{90,60}}, lineColor = {0,0,0}, textString = "s"),Text(extent = {{60, -60},{90, -80}}, lineColor = {0,0,0}, textString = "c"),Text(extent = {{ -150, -100},{150, -160}}, lineColor = {0,0,255}, textString = "%name"),Text(extent = {{ -100,100},{100,0}}, lineColor = {0,0,0}, textString = "+"),Line(points = {{ -60,100},{ -60, -100}}, color = {0,0,0}, thickness = 0.5),Line(points = {{60,100},{60, -100}}, color = {0,0,0}, thickness = 0.5)}), Documentation(info = "<html>
<p>HalfAdder is a two bit adder which is composed by Gates components. </p>
<p>Its logic behavior is like this: </p>
<p><b>HalfAdder behavior</b> </p>
<table cellspacing=\"0\" cellpadding=\"2\" border=\"1\"><tr>
<td valign=\"top\"><p><h4>input a</h4></p></td>
<td valign=\"top\"><p><h4>input b</h4></p></td>
<td valign=\"top\"><p><h4>sum s</h4></p></td>
<td valign=\"top\"><p><h4>carry c</h4></p></td>
</tr>
<tr>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>0</p></td>
</tr>
<tr>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
</tr>
<tr>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
</tr>
<tr>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
</tr>
</table>
<p><br/><br/>The parameter delayTime is the delay time (tLH=tHL) of both the components.</p>
</html>"));
            Modelica.Electrical.Digital.Interfaces.DigitalInput a annotation(Placement(transformation(extent = {{ -110,60},{ -90,80}}, rotation = 0)));
            Modelica.Electrical.Digital.Interfaces.DigitalOutput c annotation(Placement(transformation(extent = {{90, -80},{110, -60}}, rotation = 0)));
            Modelica.Electrical.Digital.Gates.AndGate AND(tLH = delayTime, tHL = delayTime) annotation(Placement(transformation(extent = {{ -20, -82},{20, -42}}, rotation = 0)));
            Modelica.Electrical.Digital.Gates.XorGate XOR(tLH = delayTime, tHL = delayTime) annotation(Placement(transformation(extent = {{ -20,42},{20,82}}, rotation = 0)));

          equation 
            connect(AND.y,c) annotation(Line(points = {{20, -62},{60, -62},{60, -70},{100, -70}}, color = {127,0,127}));
            connect(XOR.y,s) annotation(Line(points = {{20,62},{60,62},{60,70},{100,70}}, color = {127,0,127}));
            connect(b,AND.x[1]) annotation(Line(points = {{ -100, -70},{ -12, -70}}, color = {127,0,127}));
            connect(b,XOR.x[1]) annotation(Line(points = {{ -100, -70},{ -30, -70},{ -30,54},{ -12,54}}, color = {127,0,127}));
            connect(a,XOR.x[2]) annotation(Line(points = {{ -100,70},{ -12,70}}, color = {127,0,127}));
            connect(a,AND.x[2]) annotation(Line(points = {{ -100,70},{ -40,70},{ -40, -54},{ -12, -54}}, color = {127,0,127}));
          end HalfAdder;
          model FullAdder "Adding circuit for binary numbers with input carry bit"
            annotation(Documentation(info = "<html>
<p>FullAdder is a two bit adder with additional carry in bit which is composed by Gates components. </p>
<p>Its logic behavior is like this: </p>
<p><b>FullAdder behavior</b> </p>
<table cellspacing=\"0\" cellpadding=\"2\" border=\"1\"><tr>
<td valign=\"top\"><p><h4>input a</h4></p></td>
<td valign=\"top\"><p><h4>input b</h4></p></td>
<td valign=\"top\"><p><h4>input carry c_in</h4></p></td>
<td valign=\"top\"><p><h4>sum s</h4></p></td>
<td valign=\"top\"><p><h4>output carry c_out</h4></p></td>
</tr>
<tr>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>0</p></td>
</tr>
<tr>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
</tr>
<tr>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
</tr>
<tr>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
</tr>
<tr>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
</tr>
<tr>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
</tr>
<tr>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>0</p></td>
<td valign=\"top\"><p>1</p></td>
</tr>
<tr>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>1</p></td>
<td valign=\"top\"><p>1</p></td>
</tr>
</table>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -90,100},{90, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,170}, fillPattern = FillPattern.Solid),Text(extent = {{ -150, -96},{150, -151}}, lineColor = {0,0,255}, textString = "%name"),Text(extent = {{ -86,80},{ -64,60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "a"),Text(extent = {{ -86,40},{ -64,20}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "b"),Text(extent = {{ -86, -60},{ -64, -80}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "c_in"),Text(extent = {{60, -60},{90, -80}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "c_out"),Text(extent = {{64,80},{86,60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "s"),Text(extent = {{ -100,100},{100,0}}, lineColor = {0,0,0}, textString = "+"),Line(points = {{ -60,100},{ -60, -100}}, color = {0,0,0}, thickness = 0.5),Line(points = {{60,100},{60, -100}}, color = {0,0,0}, thickness = 0.5)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));
            HalfAdder Adder2(delayTime = 0.001) annotation(Placement(transformation(extent = {{10,36},{50,76}}, rotation = 0)));
            HalfAdder Adder1(delayTime = 0.001) annotation(Placement(transformation(extent = {{ -60,36},{ -20,76}}, rotation = 0)));
            Modelica.Electrical.Digital.Interfaces.DigitalInput a annotation(Placement(transformation(origin = { -100,70}, extent = {{ -10, -10},{10,10}}, rotation = 180)));
            Modelica.Electrical.Digital.Interfaces.DigitalInput b annotation(Placement(transformation(origin = { -100,30}, extent = {{ -10, -10},{10,10}}, rotation = 180)));
            Modelica.Electrical.Digital.Interfaces.DigitalInput c_in annotation(Placement(transformation(origin = { -100, -70}, extent = {{ -10, -10},{10,10}}, rotation = 180)));
            Modelica.Electrical.Digital.Interfaces.DigitalOutput s annotation(Placement(transformation(origin = {101,70}, extent = {{11, -10},{ -11,10}}, rotation = 180)));
            Modelica.Electrical.Digital.Interfaces.DigitalOutput c_out annotation(Placement(transformation(origin = {100, -70}, extent = {{10, -10},{ -10,10}}, rotation = 180)));
            Modelica.Electrical.Digital.Basic.Or OR annotation(Placement(transformation(extent = {{10, -90},{50, -50}}, rotation = 0)));

          equation 
            connect(c_out,OR.y) annotation(Line(points = {{100, -70},{50, -70}}, color = {127,0,127}));
            connect(Adder2.c,OR.x[2]) annotation(Line(points = {{50,42},{70,42},{70, -40},{10, -40},{10, -62},{18, -62}}, color = {127,0,127}));
            connect(Adder2.s,s) annotation(Line(points = {{50,70},{101,70}}, color = {127,0,127}));
            connect(Adder1.a,a) annotation(Line(points = {{ -60,70},{ -100,70}}, color = {127,0,127}));
            connect(b,Adder1.b) annotation(Line(points = {{ -100,30},{ -70,30},{ -70,42},{ -60,42}}, color = {127,0,127}));
            connect(Adder1.s,Adder2.a) annotation(Line(points = {{ -20,70},{10,70}}, color = {127,0,127}));
            connect(Adder1.c,OR.x[1]) annotation(Line(points = {{ -20,42},{ -10,42},{ -10, -78},{18, -78}}, color = {127,0,127}));
            connect(c_in,Adder2.b) annotation(Line(points = {{ -100, -70},{0, -70},{0,42},{10,42}}, color = {127,0,127}));
          end FullAdder;
          model Adder "Generic N Bit Adder"
            import Modelica.Electrical.Digital;
            annotation(Documentation(info = "<html>
<p>The Adder is a generic n bit adder which is composed as a chain of FullAdder components. n can be chosen by the user, a and b are the n bit input vectors, s is the sum vector, and c_out is the carry bit of the &QUOT;highes&QUOT; FullAdder.   All components are built up by Gate components. </p>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -90,100},{90, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,170}, fillPattern = FillPattern.Solid),Text(extent = {{68,80},{88,60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "S"),Text(extent = {{ -150, -100},{150, -160}}, lineColor = {0,0,255}, textString = "%name"),Text(extent = {{ -40,60},{40,20}}, lineColor = {0,0,0}, textString = "Adder"),Text(extent = {{48, -60},{88, -80}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "Cout"),Text(extent = {{ -90, -60},{ -50, -80}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "Cin"),Text(extent = {{ -88,80},{ -68,60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "A"),Text(extent = {{ -88,40},{ -68,20}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "B")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));
            parameter Integer n = 2 "Number of single adders";
            Modelica.Electrical.Digital.Examples.Utilities.FullAdder Adder[n] annotation(Placement(transformation(extent = {{ -20, -20},{20,20}}, rotation = 0)));
            Modelica.Electrical.Digital.Interfaces.DigitalInput a[n] annotation(Placement(transformation(extent = {{ -110,60},{ -90,80}}, rotation = 0)));
            Modelica.Electrical.Digital.Interfaces.DigitalInput b[n] annotation(Placement(transformation(extent = {{ -110,20},{ -90,40}}, rotation = 0)));
            Modelica.Electrical.Digital.Interfaces.DigitalInput c_in annotation(Placement(transformation(extent = {{ -110, -80},{ -90, -60}}, rotation = 0)));
            Modelica.Electrical.Digital.Interfaces.DigitalOutput s[n] annotation(Placement(transformation(extent = {{90,60},{110,80}}, rotation = 0)));
            Modelica.Electrical.Digital.Interfaces.DigitalOutput c_out annotation(Placement(transformation(extent = {{90, -80},{110, -60}}, rotation = 0)));

          equation 
            connect(c_in,Adder[1].c_in);
            for i in 1:n loop
            connect(a[i],Adder[i].a);
            connect(b[i],Adder[i].b);
            connect(Adder[i].a,s[i]);
if i > 1 then
                          connect(Adder[i - 1].c_out,Adder[i].c_in);
            end if;

            end for;
            connect(Adder[n].c_out,c_out);
          end Adder;
          model Counter3 "3 Bit Counter"
            import D = Modelica.Electrical.Digital;
            import L = Modelica.Electrical.Digital.Interfaces.Logic;
            annotation(Documentation(info = "<html>
<p>The Counter3 counts the high-low slopes of the count signal, if the enable signal is set to be true. It is composed by three JK flipflops. q0, q1, and q2 are the bits of the resulting number, where q0 is the lowest, and q2 the highest bit.</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -90,100},{90, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,170}, fillPattern = FillPattern.Solid),Text(extent = {{ -80,80},{ -40,60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "ENABLE"),Text(extent = {{64,80},{86,60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "Q2"),Text(extent = {{64, -60},{86, -80}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "Q0"),Text(extent = {{ -150, -100},{150, -160}}, lineColor = {0,0,255}, textString = "%name"),Text(extent = {{ -60,40},{60,0}}, lineColor = {0,0,0}, textString = "Counter3"),Text(extent = {{ -80, -60},{ -40, -80}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "COUNT"),Text(extent = {{62,8},{84, -12}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "Q1")}));
            D.Interfaces.DigitalInput enable annotation(Placement(transformation(extent = {{ -110,60},{ -90,80}}, rotation = 0)));
            D.Interfaces.DigitalOutput q2 annotation(Placement(transformation(extent = {{90,60},{110,80}}, rotation = 0)));
            D.Interfaces.DigitalInput count annotation(Placement(transformation(extent = {{ -110, -80},{ -90, -60}}, rotation = 0)));
            D.Examples.Utilities.JKFF FF1 annotation(Placement(transformation(extent = {{ -74, -20},{ -34,20}}, rotation = 0)));
            D.Examples.Utilities.JKFF FF2 annotation(Placement(transformation(extent = {{ -20, -20},{20,20}}, rotation = 0)));
            D.Examples.Utilities.JKFF FF3 annotation(Placement(transformation(extent = {{34, -20},{74,20}}, rotation = 0)));
            D.Interfaces.DigitalOutput q1 annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
            D.Interfaces.DigitalOutput q0 annotation(Placement(transformation(extent = {{90, -80},{110, -60}}, rotation = 0)));

          equation 
            connect(enable,FF1.j) annotation(Line(points = {{ -100,70},{ -80,70},{ -80,14},{ -74,14}}, color = {127,0,127}, fillPattern = FillPattern.Solid));
            connect(enable,FF1.k) annotation(Line(points = {{ -100,70},{ -80,70},{ -80, -14},{ -74, -14}}, color = {127,0,127}, fillPattern = FillPattern.Solid));
            connect(count,FF1.clk) annotation(Line(points = {{ -100, -70},{ -86, -70},{ -86,0},{ -74,0}}, color = {127,0,127}, fillPattern = FillPattern.Solid));
            connect(FF1.q,FF2.clk) annotation(Line(points = {{ -34,14},{ -30,14},{ -30,0},{ -20,0}}, color = {127,0,127}, fillPattern = FillPattern.Solid));
            connect(FF2.q,FF3.clk) annotation(Line(points = {{20,14},{24,14},{24,0},{34,0}}, color = {127,0,127}, fillPattern = FillPattern.Solid));
            connect(FF2.j,enable) annotation(Line(points = {{ -20,14},{ -26,14},{ -26,70},{ -100,70}}, color = {127,0,127}, fillPattern = FillPattern.Solid));
            connect(FF2.k,FF2.j) annotation(Line(points = {{ -20, -14},{ -26, -14},{ -26,14},{ -20,14}}, color = {127,0,127}, fillPattern = FillPattern.Solid));
            connect(FF3.k,FF3.j) annotation(Line(points = {{34, -14},{28, -14},{28,14},{34,14}}, color = {127,0,127}, fillPattern = FillPattern.Solid));
            connect(FF3.j,enable) annotation(Line(points = {{34,14},{28,14},{28,70},{ -100,70}}, color = {127,0,127}, fillPattern = FillPattern.Solid));
            connect(FF3.q,q2) annotation(Line(points = {{74,14},{80,14},{80,70},{100,70}}, color = {127,0,127}, fillPattern = FillPattern.Solid));
            connect(FF1.q,q0) annotation(Line(points = {{ -34,14},{ -30,14},{ -30, -70},{100, -70}}, color = {127,0,127}));
            connect(FF2.q,q1) annotation(Line(points = {{20,14},{24,14},{24, -50},{86, -50},{86,0},{100,0}}, color = {127,0,127}));
          end Counter3;
          model Counter "Generic N Bit Counter"
            import D = Modelica.Electrical.Digital;
            import L = Modelica.Electrical.Digital.Interfaces.Logic;
            annotation(Documentation(info = "<html>
<p>The Counter is a generic component, which counts the high-low slopes of the count signal, if the enable signal is set to be true. It is composed by  n JK flipflops. q is the resulting number, where q[0] is the lowest, and q[n] the highest bit.</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{90,80},{110, -80}}, lineColor = {127,0,127}, fillColor = {127,0,127}, fillPattern = FillPattern.Solid)}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{90,80},{110, -80}}, lineColor = {127,0,127}, fillColor = {127,0,127}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -90,100},{90, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,170}, fillPattern = FillPattern.Solid),Text(extent = {{ -80,80},{ -40,60}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "ENABLE"),Text(extent = {{66,8},{88, -12}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "Q"),Text(extent = {{ -150, -100},{150, -160}}, lineColor = {0,0,255}, textString = "%name"),Text(extent = {{ -40,40},{40,0}}, lineColor = {0,0,0}, textString = "Counter"),Text(extent = {{ -80, -60},{ -40, -80}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, textString = "COUNT")}));
            parameter Integer n = 3 "Number of bits";
            parameter Modelica.SIunits.Time delayTime = 0.001 "Delay of each JKFF";
            parameter D.Interfaces.Logic q0 = L.'0' "Initial value";
            D.Interfaces.DigitalInput enable annotation(Placement(transformation(extent = {{ -110,60},{ -90,80}}, rotation = 0)));
            D.Interfaces.DigitalInput count annotation(Placement(transformation(extent = {{ -110, -80},{ -90, -60}}, rotation = 0)));
            D.Examples.Utilities.JKFF FF[n](each delayTime = delayTime, each q0 = q0);
            D.Interfaces.DigitalOutput q[n] annotation(Placement(transformation(extent = {{90, -80},{110,80}}, rotation = 0)));

          equation 
            connect(enable,FF[1].j);
            connect(enable,FF[1].k);
            connect(count,FF[1].clk);
            connect(FF[1].q,q[1]);
            for i in 2:n loop
            connect(enable,FF[i].j);
            connect(enable,FF[i].k);
            connect(FF[i - 1].q,FF[i].clk);
            connect(FF[i].q,q[i]);

            end for;
          end Counter;
          annotation(Documentation(info = "<html>
<p>This package contains utility components used by package Examples. Each component is built up hierarchically by components of the Gates package. In this way the Gates components were tested, and their usage is demonstrated.</p>
</html>"));
        end Utilities;
      end Examples;
      package Interfaces "Basic definitions"
        type Logic = enumeration('U' "U  Uninitialized", 'X' "X  Forcing Unknown", '0' "0  Forcing 0", '1' "1  Forcing 1", 'Z' "Z  High Impedance", 'W' "W  Weak    Unknown", 'L' "L  Weak    0", 'H'  "H  Weak    1", '-'  "-  Don't care") "Logic values and their coding according to IEEE 1164 STD_ULOGIC type" annotation(Documentation(info = "<html>
<p><b>Code Table:</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Logic value</b></td>
      <td valign=\"top\"><b>Meaning</b></td>
  </tr>

  <tr><td valign=\"top\">'U'</td> <td valign=\"top\">Uninitialized</td></tr>
  <tr><td valign=\"top\">'X'</td> <td valign=\"top\">Forcing Unknown</td></tr>
  <tr><td valign=\"top\">'0'</td> <td valign=\"top\">Forcing 0</td></tr>
  <tr><td valign=\"top\">'1'</td> <td valign=\"top\">Forcing 1</td></tr>
  <tr><td valign=\"top\">'Z'</td> <td valign=\"top\">High Impedance</td></tr>
  <tr><td valign=\"top\">'W'</td> <td valign=\"top\">Weak Unknown</td></tr>
  <tr><td valign=\"top\">'L'</td> <td valign=\"top\">Weak 0</td></tr>
  <tr><td valign=\"top\">'H'</td> <td valign=\"top\">Weak 1</td></tr>
  <tr><td valign=\"top\">'-'</td> <td valign=\"top\">Don't care</td></tr>
</table>


</html>"));
        type Strength = enumeration('S_X01', 'S_X0H', 'S_XL1', 'S_X0Z', 'S_XZ1', 'S_WLH', 'S_WLZ', 'S_WZH', 'S_W0H' , 'S_WL1' ) "Output strengthes of registers" annotation(Documentation(info = "<html>

<p><b>Strength Table:</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Strenght</b></td>
      <td valign=\"top\"><b>Output conversion to</b></td>
  </tr>

  <tr><td valign=\"top\">'S_X01'</td> <td valign=\"top\">Forcing X, 0, 1</td></tr>
  <tr><td valign=\"top\">'S_X0H'</td> <td valign=\"top\">Forcing X, 0 and Weak 1</td></tr>
  <tr><td valign=\"top\">'S_XL1'</td> <td valign=\"top\">Forcing X, 1 and Weak 0</td></tr>
  <tr><td valign=\"top\">'S_X0Z'</td> <td valign=\"top\">Forcing X, 0 and High Impedance</td></tr>
  <tr><td valign=\"top\">'S_XZ1'</td> <td valign=\"top\">Forcing X, 1 and High Impedance</td></tr>
  <tr><td valign=\"top\">'S_WLH'</td> <td valign=\"top\">Weak X, 0, 1</td></tr>
  <tr><td valign=\"top\">'S_WLZ'</td> <td valign=\"top\">Weak X, 0 and High Impedance</td></tr>
  <tr><td valign=\"top\">'S_WZH'</td> <td valign=\"top\">Weak X, 1 and High Impedance</td></tr>
  <tr><td valign=\"top\">'S_W0H'</td> <td valign=\"top\">Weak X, 1 and Forcing 0</td></tr>
  <tr><td valign=\"top\">'S_WL1'</td> <td valign=\"top\">Weak X, 0 and Forcing 1</td></tr>
</table>
</html>"));
        connector DigitalSignal = Logic "Digital port (both input/output possible)" annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>DigitalSignal is the basic digital connector defintion. A direction (input, output) is not yet defined. DigitalSignal is of type Logic. It can have the logic values (U, X, 0, 1, ...) which are internally coded by integer values by using the enumeration (c.f. the definition of type Logic).</p>
</html>"));
        connector DigitalInput = input DigitalSignal "Input DigitalSignal as connector" annotation(defaultComponentName = "x", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -100},{100,100}}, lineColor = {127,0,127}, fillColor = {127,0,127}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -150, -100},{150, -160}}, lineColor = {127,0,127}, textString = "%name"),Rectangle(extent = {{ -100, -100},{100,100}}, lineColor = {127,0,127}, fillColor = {127,0,127}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>DigitalInput is the  digital input connector defintion. DigitalInput is of type Logic. It can have the logic values (U, X, 0, 1, ...) which are internally coded by integer values by using the enumeration (c.f. the definition of type Logic).</p>
</html>"));
        connector DigitalOutput = output DigitalSignal "Output DigitalSignal as connector" annotation(defaultComponentName = "y", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -100,100},{100,0},{ -100, -100},{ -100,100}}, lineColor = {127,0,127}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -100,100},{100,0},{ -100, -100},{ -100,100}}, lineColor = {127,0,127}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -150, -100},{150, -160}}, lineColor = {127,0,127}, textString = "%name")}), Documentation(info = "<html>
<p>DigitalOutput is the  digital output connector defintion. DigitalOutput  is of type Logic. It can have the logic values (U, X, 0, 1, ...) which are internally coded by integer values by using the enumeration (c.f. the definition of type Logic). The arrow shape symbolizes the signal flow direction.</p>
</html>"));
        partial block SISO "Single input, single output"
          import D = Modelica.Electrical.Digital;
          D.Interfaces.DigitalInput x "Connector of Digital input signal" annotation(Placement(transformation(extent = {{ -70, -10},{ -50,10}}, rotation = 0)));
          D.Interfaces.DigitalOutput y "Connector of Digital output signal" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -50,100},{50, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{50,0},{92,0}}, color = {127,0,127})}), Documentation(info = "<html>
<p>SISO is a partial model for the connection pattern whith <b>s</b>ingle (scalar) digital <b>i</b>nput and <b>s</b>ingle (scalar) digital <b>o</b>utput. Besides the connectors it provides a rectangel for the icon which can be filled in by the component which inherits the SISO model.</p>
</html>"));
        end SISO;
        partial block MISO "Multiple input - single output"
          import D = Modelica.Electrical.Digital;
          parameter Integer n(final min = 2) = 2 "Number of inputs";
          D.Interfaces.DigitalInput x[n] "Connector of Digital input signal vector" annotation(Placement(transformation(extent = {{ -70, -80},{ -50,80}}, rotation = 0)));
          D.Interfaces.DigitalOutput y "Connector of Digital output signal" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -50,100},{50, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{50,0},{90,0}}, color = {127,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>MISO is a partial model for the connection pattern whith <b>m</b>ultiple (vector) digital<b> i</b>nput and <b>s</b>ingle (scalar) digital <b>o</b>utput. Besides the connectors it provides a rectangel for the icon which can be filled in by the component which inherits the MISO model.</p>
</html>"));
        end MISO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -100},{80,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{ -100,50},{ -80,70},{100,70},{80,50},{ -100,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{100,70},{100, -80},{80, -100},{80,50},{100,70}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -85,35},{65, -85}}, lineColor = {0,0,255}, textString = "Library"),Text(extent = {{ -120,122},{120,73}}, lineColor = {255,0,0}, textString = "%name")}), Documentation(info = "<html>
<p>This package contains basic definitions: Type definitions of Logic and Strength,  interface definitions (connectors) for digital electrical components, and partial models for connection patterns which are often used.</p>
</html>"), DymolaStoredErrors);
        partial block MIMO "Multiple input - multiple output"
          import D = Modelica.Electrical.Digital;
          parameter Integer n(final min = 1) = 1 "Number of inputs = Number of outputs";
          D.Interfaces.DigitalInput x[n] "Connector of Digital input signal vector" annotation(Placement(transformation(extent = {{ -70, -80},{ -50,80}}, rotation = 0)));
          D.Interfaces.DigitalOutput y[n] "Connector of Digital output signal vector" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -50,100},{50, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{50,0},{90,0}}, color = {127,0,127}),Rectangle(extent = {{50,80},{70, -80}}, lineColor = {127,33,107}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>MIMO is a partial model for the connection pattern whith <b>m</b>ultiple (vector) digital<b> i</b>nput and <b>m</b>ultiple  (vector) digital <b>o</b>utput. Besides the connectors it provides a rectangel for the icon which can be filled in by the component which inherits the MISO model.</p>
</html>"));
        end MIMO;
      end Interfaces;
      package Tables "Truth tables"
        import D = Modelica.Electrical.Digital;
        import L = Modelica.Electrical.Digital.Interfaces.Logic;
        import S = Modelica.Electrical.Digital.Interfaces.Strength;
        constant D.Interfaces.Logic AndTable[L,L] = [L.'U',L.'U',L.'0',L.'U',L.'U',L.'U',L.'0',L.'U',L.'U';L.'U',L.'X',L.'0',L.'X',L.'X',L.'X',L.'0',L.'X',L.'X';L.'0',L.'0',L.'0',L.'0',L.'0',L.'0',L.'0',L.'0',L.'0';L.'U',L.'X',L.'0',L.'1',L.'X',L.'X',L.'0',L.'1',L.'X';L.'U',L.'X',L.'0',L.'X',L.'X',L.'X',L.'0',L.'X',L.'X';L.'U',L.'X',L.'0',L.'X',L.'X',L.'X',L.'0',L.'X',L.'X';L.'0',L.'0',L.'0',L.'0',L.'0',L.'0',L.'0',L.'0',L.'0';L.'U',L.'X',L.'0',L.'1',L.'X',L.'X',L.'0',L.'1',L.'X';L.'U',L.'X',L.'0',L.'X',L.'X',L.'X',L.'0',L.'X',L.'X'] "9-value logic for 'and'";
        constant D.Interfaces.Logic OrTable[L,L] = [L.'U',L.'U',L.'U',L.'1',L.'U',L.'U',L.'U',L.'1',L.'U';L.'U',L.'X',L.'X',L.'1',L.'X',L.'X',L.'X',L.'1',L.'X';L.'U',L.'X',L.'0',L.'1',L.'X',L.'X',L.'0',L.'1',L.'X';L.'1',L.'1',L.'1',L.'1',L.'1',L.'1',L.'1',L.'1',L.'1';L.'U',L.'X',L.'X',L.'1',L.'X',L.'X',L.'X',L.'1',L.'X';L.'U',L.'X',L.'X',L.'1',L.'X',L.'X',L.'X',L.'1',L.'X';L.'U',L.'X',L.'0',L.'1',L.'X',L.'X',L.'0',L.'1',L.'X';L.'1',L.'1',L.'1',L.'1',L.'1',L.'1',L.'1',L.'1',L.'1';L.'U',L.'X',L.'X',L.'1',L.'X',L.'X',L.'X',L.'1',L.'X'] "9-value logic for 'or'";
        constant D.Interfaces.Logic NotTable[L] = {L.'U',L.'X',L.'1',L.'0',L.'X',L.'X',L.'1',L.'0',L.'X'} "9-value logic for 'not'";
        constant D.Interfaces.Logic XorTable[L,L] = [L.'U',L.'U',L.'U',L.'U',L.'U',L.'U',L.'U',L.'U',L.'U';L.'U',L.'X',L.'X',L.'X',L.'X',L.'X',L.'X',L.'X',L.'X';L.'U',L.'X',L.'0',L.'1',L.'X',L.'X',L.'0',L.'1',L.'X';L.'U',L.'X',L.'1',L.'0',L.'X',L.'X',L.'1',L.'0',L.'X';L.'U',L.'X',L.'X',L.'X',L.'X',L.'X',L.'X',L.'X',L.'X';L.'U',L.'X',L.'X',L.'X',L.'X',L.'X',L.'X',L.'X',L.'X';L.'U',L.'X',L.'0',L.'1',L.'X',L.'X',L.'0',L.'1',L.'X';L.'U',L.'X',L.'1',L.'0',L.'X',L.'X',L.'1',L.'0',L.'X';L.'U',L.'X',L.'X',L.'X',L.'X',L.'X',L.'X',L.'X',L.'X'] "9-value logic for 'xor'";
        constant D.Interfaces.Logic X01Table[L] = {L.'X',L.'X',L.'0',L.'1',L.'X',L.'X',L.'0',L.'1',L.'X'};
        constant D.Interfaces.Logic X01ZTable[L] = {L.'X',L.'X',L.'0',L.'1',L.'Z',L.'X',L.'0',L.'1',L.'Z'};
        constant D.Interfaces.Logic UX01Table[L] = {L.'U',L.'X',L.'0',L.'1',L.'X',L.'X',L.'0',L.'1',L.'X'};
        constant Integer DelayTable[L,L] = [0,0, -1,1,0,0, -1,1,0;0,0, -1,1,0,0, -1,1,0;1,1,0,1,1,1,0,1,1; -1, -1, -1,0, -1, -1, -1,0, -1;0,0, -1,1,0,0, -1,1,0;0,0, -1,1,0,0, -1,1,0;1,1,0,1,1,1,0,1,1; -1, -1, -1,0, -1, -1, -1,0, -1;0,0, -1,1,0,0, -1,1,0] "Delay selection for signal change by [old_signal, new_signal] reading:
      -1: High-Low delay |
       0: no delay |
       1: Low-High delay";
        constant Integer ClockMap[L,L] = [0,0,0,2,0,0,0,2,0;0,0,0,2,0,0,0,2,0;2,2,0,1,2,2,0,1,2;0,0,0,0,0,0,0,0,0;0,0,0,2,0,0,0,2,0;0,0,0,2,0,0,0,2,0;2,2,0,1,2,2,0,1,0;0,0,0,0,0,0,0,0,0;0,0,0,2,0,0,0,2,0] "Edge detection by [pre(clock), clock] reading:
    0: 0-Transition |
    1: rising edge  |
    2: X-Transition";
        constant D.Interfaces.Logic StrengthMap[L,S] = [L.'U',L.'U',L.'U',L.'U',L.'U',L.'U',L.'U',L.'U',L.'U',L.'U';L.'X',L.'X',L.'X',L.'X',L.'X',L.'W',L.'W',L.'W',L.'W',L.'W';L.'0',L.'0',L.'L',L.'0',L.'Z',L.'L',L.'L',L.'Z',L.'0',L.'L';L.'1',L.'H',L.'1',L.'Z',L.'1',L.'H',L.'Z',L.'H',L.'H',L.'1';L.'X',L.'X',L.'X',L.'X',L.'X',L.'W',L.'W',L.'W',L.'W',L.'W';L.'X',L.'X',L.'X',L.'X',L.'X',L.'W',L.'W',L.'W',L.'W',L.'W';L.'0',L.'0',L.'L',L.'0',L.'Z',L.'L',L.'L',L.'Z',L.'0',L.'L';L.'1',L.'H',L.'1',L.'Z',L.'1',L.'H',L.'Z',L.'H',L.'H',L.'1';L.'X',L.'X',L.'X',L.'X',L.'X',L.'W',L.'W',L.'W',L.'W',L.'W'] "Output strength conversion by [signal, strength] reading";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -100},{80,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{ -100,50},{ -80,70},{100,70},{80,50},{ -100,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{100,70},{100, -80},{80, -100},{80,50},{100,70}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -85,35},{65, -85}}, lineColor = {0,0,255}, textString = "Library"),Text(extent = {{ -120,122},{120,73}}, lineColor = {255,0,0}, textString = "%name")}), Documentation(info = "<html>
<p>Tables contains several logic valued matrixes which are used for quickly calculating the result. Since the logic values are coded internally by the integers 1 ... 9, the code of the input signals can be taken to chose line and column of the matrix element where the result can be found. In this way the tables are used by some components.</p>
</html>"));
      end Tables;
      package Delay "Delay blocks"
        partial block DelayParams "Definition of delay parameters"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          parameter Modelica.SIunits.Time tLH(start = 0) "Rise inertial delay";
          parameter Modelica.SIunits.Time tHL(start = 0) "Fall inertial delay";
          parameter L y0 = L.'U' "Initial value of output";
          annotation(Documentation(info = "<html>
<p>DelayParams is a partial model for providing delay times and inertial values. It is used in components of the package Gates which need the same parameters. The partial model does not have any behavior or equations. </p>
</html>"));
        end DelayParams;
        model TransportDelay "Transport delay with initial parameter"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends D.Interfaces.SISO(x(start = L.'U', fixed = true));
          parameter Modelica.SIunits.Time delayTime(start = 0) "delay time";
          parameter D.Interfaces.Logic y0 = L.'U' "initial value of output";
        protected 
          D.Interfaces.Logic x_delayed;
          annotation(Documentation(info = "<HTML>
<P>
Provide the input as output exactly delayed by <i>Tdel</i>.
If time less than <i>Tdel</i> the initial value <i>initout</i> holds.
</P>
</HTML>
", revisions = "<HTML>
<ul>
<li><i>   </i>
       by ...<br>
       </li>
<li><i>August 11, 2003</i>
       by Christoph Clauss<br>
       initially modelled.</li>
</ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{152, -160},{ -148, -100}}, lineColor = {0,0,255}, textString = "%name"),Rectangle(extent = {{ -50,100},{50, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {213,170,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -6,60},{ -16,40},{4,40},{ -6,60}}, lineColor = {0,0,0}),Line(points = {{0,60},{20,60}}, color = {0,0,0}),Line(points = {{10,60},{10,40}}, color = {0,0,0}),Text(extent = {{ -50, -40},{50, -20}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "Transport"),Text(extent = {{ -50, -60},{50, -40}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "Delay")}));

        equation 
          x_delayed = integer(delay(x, delayTime));
          y = if delayTime > 0 then if time >= delayTime then x_delayed else y0 else pre(x);
        end TransportDelay;
        block InertialDelay "Inertial delay with initial parameter"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends D.Interfaces.SISO;
          parameter Modelica.SIunits.Time delayTime(start = 0) "Minimum time to hold value";
          parameter D.Interfaces.Logic y0 = L.'U' "Initial value of output y";
        protected 
          D.Interfaces.Logic y_auxiliary(start = y0, fixed = true);
          D.Interfaces.Logic x_old(start = y0, fixed = true);
          discrete Modelica.SIunits.Time t_next(start = delayTime, fixed = true);
          annotation(Documentation(info = "<HTML>
<P>
Provides the input as output delayed by <i>Tdel</i> if the input holds its value for a longer time than <i>Tdel</i>.
If time is less than <i>Tdel</i> the initial value <i>initout</i> holds.
</P>
</HTML>
", revisions = "<HTML>
<ul>
<li><i>   </i>
       by ...<br>
       </li>
<li><i>August 12, 2003   </i>
       by Christoph Clauss<br>
       reviced</li>
<li><i>March 19, 2003</i>
       by Martin Otter<br>
       initially modelled.</li>
</ul>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{152, -160},{ -148, -100}}, lineColor = {0,0,255}, textString = "%name"),Rectangle(extent = {{ -50,100},{50, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {213,170,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -6,60},{ -16,40},{4,40},{ -6,60}}, lineColor = {0,0,0}),Line(points = {{10,60},{10,40}}, color = {0,0,0}),Line(points = {{0,60},{20,60}}, color = {0,0,0}),Text(extent = {{ -50, -40},{50, -20}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "Inertial"),Text(extent = {{ -50, -60},{50, -40}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "Delay")}));
        algorithm 
          when delayTime > 0 and change(x) then
                      x_old:=x;
            t_next:=time + delayTime;
          elsewhen time >= t_next then
            y_auxiliary:=x;

          end when;
          y:=if delayTime > 0 then y_auxiliary else x;
        end InertialDelay;
        model InertialDelaySensitive "Provide the input as output if it holds its value for a specific amount of time"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends D.Interfaces.SISO(x(start = L.'U', fixed = true));
          parameter Modelica.SIunits.Time tLH(start = 0) "rise inertial delay";
          parameter Modelica.SIunits.Time tHL(start = 0) "fall inertial delay";
          parameter D.Interfaces.Logic y0 = L.'U' "initial value of output";
        protected 
          Integer delayTable[:,:] = D.Tables.DelayTable "specification of delay according to signal change";
          Modelica.SIunits.Time delayTime;
          D.Interfaces.Logic y_auxiliary(start = y0, fixed = true);
          D.Interfaces.Logic y_old(start = y0, fixed = true);
          Integer lh;
          discrete Modelica.SIunits.Time t_next;
          annotation(Documentation(info = "<HTML>
<P>
Provides the input as output delayed by <i>Tdel</i> if the input holds its value for a longer time than <i>Tdel</i>.
If the time is less than <i>Tdel</i> the initial value <i>initout</i> holds.<br>
The delay <i>Tdel</i> depends on the values of the signal change. To calculate <i>Tdel</i>, the delaymap specified in
Digital.Tables is used. If the corresponding value is 1, then <i>tLH</i> is used, if it is -1, then <i>tHL</i>
is used, if it is zero, the input is not delayed.
</P>
</HTML>
", revisions = "<HTML>
<ul>
<li><i>September 8, 2009  </i> pre(y) and x are used to select <i>tHL</i> or <i>tLH</i>
       by Ulrich Donath<br>
       </li>
<li><i>January 13, 2005  </i> improved when-conditions and declaration of delayTable
       by Dynasim<br>
       </li>
<li><i>September 15, 2004  </i> color changed, names changed
       by Christoph Clauss<br>
       </li>
<li><i>May 12, 2004  </i> test <i>if Tdel=0</i> replaced
       by Christoph Clauss<br>
       </li>
<li><i>February 5, 2004 </i> handling of <i>tHL=0</i> or <i>tLH=0</i> revised
       by Christoph Clauss<br>
       </li>
<li><i>October 12, 2003   </i>
       by Christoph Clauss<br>
       initially modelled</li>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{152, -160},{ -148, -100}}, lineColor = {0,0,255}, textString = "%name"),Rectangle(extent = {{ -50,100},{50, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {213,170,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -50, -40},{50, -20}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "Inertial"),Text(extent = {{ -50, -60},{50, -40}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "Delay"),Text(extent = {{ -50, -80},{50, -60}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "sensitive"),Polygon(points = {{ -6,60},{ -16,40},{4,40},{ -6,60}}, lineColor = {0,0,0}),Line(points = {{10,60},{10,40}}, color = {0,0,0}),Line(points = {{0,60},{20,60}}, color = {0,0,0})}));
        algorithm 
          when {initial(),(tLH > 0 or tHL > 0) and change(x) and not initial()} then
                      y_old:=if initial() or pre(y) == 0 then y0 else pre(y);
            lh:=delayTable[y_old,x];
            delayTime:=if lh > 0 then tLH else if lh < 0 then tHL else 0;
            t_next:=time + delayTime;
            if lh == 0 or abs(delayTime) < Modelica.Constants.small then 
                        y_auxiliary:=x;

            else 
            end if;
          elsewhen time >= t_next then
            y_auxiliary:=x;

          end when;
          y:=if tLH > 0 or tHL > 0 then y_auxiliary else x;
        end InertialDelaySensitive;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -100},{80,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{ -100,50},{ -80,70},{100,70},{80,50},{ -100,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{100,70},{100, -80},{80, -100},{80,50},{100,70}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -85,35},{65, -85}}, lineColor = {0,0,255}, textString = "Library"),Text(extent = {{ -120,122},{120,73}}, lineColor = {255,0,0}, textString = "%name")}), Documentation(info = "<html>
<p>The Delay package collects the delay blocks which are used in many components. Both transport and inertial delay are offered for scalar connectors. The most advanced component is the sensitive interial delay the delay time of which is chosen in dependency of the signal values. This component is also available for vector valued input.</p>
</html>"));
        model InertialDelaySensitiveVector "Delay of a vector of digital signals"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          import S = Modelica.Electrical.Digital.Interfaces.Strength;
          import T = Modelica.Electrical.Digital.Tables;
          parameter Modelica.SIunits.Time tHL = 0 "High->Low delay";
          parameter Modelica.SIunits.Time tLH = 0 "Low->High delay";
          parameter Integer n(min = 1) = 1 "Data width";
          D.Interfaces.DigitalInput x[n] annotation(Placement(transformation(extent = {{ -100, -12},{ -76,12}}), iconTransformation(extent = {{ -100, -15},{ -72,14}})));
          D.Interfaces.DigitalOutput y[n] annotation(Placement(transformation(extent = {{72, -14},{100,14}}), iconTransformation(extent = {{72, -14},{100,14}})));
          Digital.Delay.InertialDelaySensitive inertialDelaySensitive[n](each tLH = tLH, each tHL = tHL);

        equation 
          for i in 1:n loop
          connect(x[i],inertialDelaySensitive[i].x);
          connect(inertialDelaySensitive[i].y,y[i]);

          end for;
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -50,54},{48, -54}}, lineColor = {127,0,127}, lineThickness = 0.5, fillPattern = FillPattern.Solid, fillColor = {255,255,255}),Line(points = {{ -80,0},{ -50,0}}, color = {127,0,127}, thickness = 1, smooth = Smooth.None),Line(points = {{48,0},{80,0}}, color = {127,0,127}, thickness = 1, smooth = Smooth.None),Text(extent = {{ -40,52},{38, -20}}, lineColor = {127,33,107}, textString = "DELAY"),Text(extent = {{ -26,12},{18, -32}}, lineColor = {127,0,127}, textString = "signal "),Text(extent = {{ -32,0},{28, -62}}, lineColor = {127,0,127}, textString = "sensitive")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>

<p>
<pre>
The delay element <b>Inertial Delay Sensitive</b> is applied to a vector of n signals.
The parameters <i>tLH</i> and <i>tHL</i> are valid for each of the n signals.
</pre>
</p>
</html>", revisions = "<html>
<ul>
<pre>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</pre>
</ul>
</html>"));
        end InertialDelaySensitiveVector;
      end Delay;
      package Basic "Basic logic blocks without delays"
        model Not "Not logic component without delay"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends D.Interfaces.SISO;
        protected 
          D.Interfaces.Logic auxiliary(start = L.'0', fixed = true);
          annotation(Documentation(info = "<html>
<p>Not component with 1 input value, without delay. </p>
<p>According to the standard logic not table (Tables.NotTable) the output value is calculated.</p>
<p>To avoid loops in the numerical treatment, the pre operator is applied to the output.</p>
</html>", revisions = "<HTML>
<ul>
<li><i>   </i>
       by ...<br>
       </li>
<li><i>August 14, 2003</i>
       by Teresa Schlegel<br>
       initially modelled.</li>
</ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -50,40},{50,80}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "1"),Text(extent = {{152, -160},{ -148, -100}}, lineColor = {0,0,255}, textString = "%name"),Ellipse(extent = {{50,6},{62, -6}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          auxiliary = D.Tables.NotTable[x];
          y = pre(auxiliary);
        end Not;
        model And "And logic component with multiple input and one output"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends D.Interfaces.MISO;
        protected 
          D.Interfaces.Logic auxiliary[n](each start = L.'U', each fixed = true);
          annotation(Documentation(info = "<html>
<p>And component with multiple input values and one output. </p>
<p>According to the standard logic and table (Tables.AndTable) the output value is calculated.</p>
<p>To avoid loops in the numerical treatment, the pre operator is applied to the output.</p>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004</i> vector approach used for all fixed numbers of inputs
       by Christoph Clauss<br>
       </li>
<li><i>October 22, 2003</i>
       by  Teresa Schlegel<br>
       initially modelled.</li>
</ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -50,40},{50,80}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "&"),Text(extent = {{152, -160},{ -148, -100}}, lineColor = {0,0,255}, textString = "%name")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          auxiliary[1] = x[1];
          for i in 1:n - 1 loop
          auxiliary[i + 1] = D.Tables.AndTable[auxiliary[i],x[i + 1]];

          end for;
          y = pre(auxiliary[n]);
        end And;
        model Nand "Nand logic component with multiple input and one output"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends D.Interfaces.MISO;
        protected 
          D.Interfaces.Logic auxiliary[n](each start = L.'U', each fixed = true);
          annotation(Documentation(info = "<html>
<p>Nand component with multiple input values and one output. </p>
<p>According to the standard logic and table (Tables.AndTable) an intermediate value is calculated, to which the not table (Tables.NotTable) is applied.</p>
<p>To avoid loops in the numerical treatment, the pre operator is applied to the output.</p>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004</i> vector approach used for all fixed numbers of inputs
       by Christoph Clauss<br>
       </li>
<li><i>October 22, 2003</i>
       by  Teresa Schlegel<br>
       initially modelled.</li>
</ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -50,40},{50,80}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "&"),Text(extent = {{152, -160},{ -148, -100}}, lineColor = {0,0,255}, textString = "%name"),Ellipse(extent = {{50,6},{62, -6}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          auxiliary[1] = x[1];
          for i in 1:n - 1 loop
          auxiliary[i + 1] = D.Tables.AndTable[auxiliary[i],x[i + 1]];

          end for;
          y = pre(D.Tables.NotTable[auxiliary[n]]);
        end Nand;
        model Or "Or logic component with multiple input and one output"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends D.Interfaces.MISO;
        protected 
          D.Interfaces.Logic auxiliary[n](each start = L.'U', each fixed = true);
          annotation(Documentation(info = "<html>
<p>Or component with multiple input values and one output. </p>
<p>According to the standard logic or table (Tables.OrTable) the output value is calculated.</p>
<p>To avoid loops in the numerical treatment, the pre operator is applied to the output.</p>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004</i> vector approach used for all fixed numbers of inputs
       by Christoph Clauss<br>
       </li>
<li><i>October 22, 2003</i>
       by  Teresa Schlegel<br>
       initially modelled.</li>
</ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -50,40},{50,80}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = ">=1"),Text(extent = {{152, -160},{ -148, -100}}, lineColor = {0,0,255}, textString = "%name")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          auxiliary[1] = x[1];
          for i in 1:n - 1 loop
          auxiliary[i + 1] = D.Tables.OrTable[auxiliary[i],x[i + 1]];

          end for;
          y = pre(auxiliary[n]);
        end Or;
        model Nor "Nor logic component with multiple input and one output"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends D.Interfaces.MISO;
        protected 
          D.Interfaces.Logic auxiliary[n](each start = L.'U', each fixed = true);
          annotation(Documentation(info = "<html>
<p>Nor component with multiple input values and one output. </p>
<p>According to the standard logic or table (Tables.OrTable) an intermediate value is calculated, to which the not table (Tables.NotTable) is applied.</p>
<p>To avoid loops in the numerical treatment, the pre operator is applied to the output.</p>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004</i> vector approach used for all fixed numbers of inputs
       by Christoph Clauss<br>
       </li>
<li><i>October 22, 2003</i>
       by Liane Jacobi<br>
       initially modelled.</li>
</ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -50,40},{50,80}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = ">=1"),Text(extent = {{152, -160},{ -148, -100}}, lineColor = {0,0,255}, textString = "%name"),Ellipse(extent = {{50,6},{62, -6}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          auxiliary[1] = x[1];
          for i in 1:n - 1 loop
          auxiliary[i + 1] = D.Tables.OrTable[auxiliary[i],x[i + 1]];

          end for;
          y = pre(D.Tables.NotTable[auxiliary[n]]);
        end Nor;
        model Xor "Xor logic component with multiple input and one output"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends D.Interfaces.MISO;
        protected 
          D.Interfaces.Logic auxiliary[n](each start = L.'U', each fixed = true);
          annotation(Documentation(info = "<html>
<p>Xor component with multiple input values and one output. </p>
<p>According to the standard logic xor table (Tables.XorTable) the output value is calculated.</p>
<p>To avoid loops in the numerical treatment, the pre operator is applied to the output.</p>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004</i> vector approach used for all fixed numbers of inputs
       by Christoph Clauss<br>
       </li>
<li><i>October 22, 2003</i>
       by Liane Jacobi<br>
       initially modelled.</li>
</ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -50,40},{50,80}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "=1"),Text(extent = {{152, -160},{ -148, -100}}, lineColor = {0,0,255}, textString = "%name")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          auxiliary[1] = x[1];
          for i in 1:n - 1 loop
          auxiliary[i + 1] = D.Tables.XorTable[auxiliary[i],x[i + 1]];

          end for;
          y = pre(auxiliary[n]);
        end Xor;
        model Xnor "Xnor logic component with multiple input and one output"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          extends D.Interfaces.MISO;
        protected 
          D.Interfaces.Logic auxiliary[n](each start = L.'U', each fixed = true);
          annotation(Documentation(info = "<html>
<p>Xnor component with multiple input values and one output. </p>
<p>According to the standard logic xor table (Tables.XorTable)an intermediate value is calculated, to which the not table (Tables.NotTable) is applied.</p>
<p>To avoid loops in the numerical treatment, the pre operator is applied to the output.</p>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004</i> vector approach used for all fixed numbers of inputs
       by Christoph Clauss<br>
       </li>
<li><i>October 22, 2003</i>
       by Liane Jacobi<br>
       initially modelled.</li>
</ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -50,40},{50,80}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "="),Text(extent = {{152, -160},{ -148, -100}}, lineColor = {0,0,255}, textString = "%name")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          auxiliary[1] = x[1];
          for i in 1:n - 1 loop
          auxiliary[i + 1] = D.Tables.XorTable[auxiliary[i],x[i + 1]];

          end for;
          y = pre(D.Tables.NotTable[auxiliary[n]]);
        end Xnor;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -100},{80,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{ -100,50},{ -80,70},{100,70},{80,50},{ -100,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{100,70},{100, -80},{80, -100},{80,50},{100,70}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -85,35},{65, -85}}, lineColor = {0,0,255}, textString = "Library"),Text(extent = {{ -120,122},{120,73}}, lineColor = {255,0,0}, textString = "%name")}), Documentation(info = "<html>
<p>Basic contains the basic gates according to standard logic. The components of Basic calculate their results using the corresponding truth tables. They do not contain any delay components.</p>
</html>"));
      end Basic;
      package Gates "Logic gates including delays"
        model InvGate "InvGate with 1 input value, composed by Not and sensitive intertial delay"
          import D = Modelica.Electrical.Digital;
          extends D.Delay.DelayParams;
          extends D.Interfaces.SISO;
          D.Basic.Not G1 annotation(Placement(transformation(extent = {{ -60, -20},{ -20,20}}, rotation = 0)));
          D.Delay.InertialDelaySensitive G2(tLH = tLH, tHL = tHL) annotation(Placement(transformation(extent = {{20, -20},{60,20}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>The InvGate model has a single valued input, and a single valued output. It is composed by a Basic Not and an InertialDelaySensitive. Its parameters are the delay parameters (rise and fall intertial delay time, and intitial value).</p>
</html>", revisions = "<HTML>
<ul>
<li><i>   </i>
       by ...<br>
       </li>
<li><i>August 14, 2003</i>
       by Teresa Schlegel<br>
       initially modelled.</li>
</ul>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{152, -160},{ -148, -100}}, lineColor = {0,0,255}, textString = "%name"),Rectangle(extent = {{ -50,100},{50, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {213,170,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -50,80},{50,40}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,191,127}, fillPattern = FillPattern.Solid, textString = "1"),Text(extent = {{ -18, -60},{20, -100}}, lineColor = {0,0,0}, textString = "Gate"),Ellipse(extent = {{50,6},{62, -6}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}));

        equation 
          connect(G2.y,y) annotation(Line(points = {{60,0},{100,0}}, color = {127,0,127}));
          connect(G1.x,x) annotation(Line(points = {{ -52,0},{ -60,0}}, color = {127,0,127}));
          connect(G1.y,G2.x) annotation(Line(points = {{ -20,0},{28,0}}, color = {127,0,127}));
        end InvGate;
        model AndGate "AndGate with multiple input"
          import D = Modelica.Electrical.Digital;
          extends D.Interfaces.MISO;
          extends D.Delay.DelayParams;
          D.Basic.And G1(final n = n) annotation(Placement(transformation(extent = {{ -40, -20},{0,20}}, rotation = 0)));
          D.Delay.InertialDelaySensitive G2(tLH = tLH, tHL = tHL, y0 = y0) annotation(Placement(transformation(extent = {{20, -20},{60,20}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>The AndGate model has a multiple valued (n) input vector, and a single valued output. It is composed by a Basic And and an InertialDelaySensitive. Its parameters are the delay parameters (rise and fall intertial delay time, and intitial value).</p>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004</i> vector approach used for all fixed numbers of inputs
       by Christoph Clauss<br>
       </li>
<li><i>October 22, 2003</i>
       by Teresa Schlegel<br>
       initially modelled.</li>
</ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{152, -160},{ -148, -100}}, lineColor = {0,0,255}, textString = "%name"),Rectangle(extent = {{ -50,100},{50, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {213,170,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -50,80},{50,40}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "&"),Text(extent = {{ -20, -60},{20, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "Gate")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          connect(x,G1.x) annotation(Line(points = {{ -60,0},{ -32,0}}, color = {127,0,127}));
          connect(y,G2.y) annotation(Line(points = {{100,0},{60,0}}, color = {127,0,127}));
          connect(G1.y,G2.x) annotation(Line(points = {{0,0},{28,0}}, color = {127,0,127}));
        end AndGate;
        model NandGate "NandGate with multiple input"
          import D = Modelica.Electrical.Digital;
          extends D.Delay.DelayParams;
          extends D.Interfaces.MISO;
          D.Basic.Nand G1(final n = n) annotation(Placement(transformation(extent = {{ -40, -20},{0,20}}, rotation = 0)));
          D.Delay.InertialDelaySensitive G2(tLH = tLH, tHL = tHL, y0 = y0) annotation(Placement(transformation(extent = {{20, -20},{60,20}}, rotation = 0)));
          annotation(extent = [0, -20;40,20], Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{152, -160},{ -148, -100}}, lineColor = {0,0,255}, textString = "%name"),Rectangle(extent = {{ -50,100},{50, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {213,170,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -40,40},{40,80}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "&"),Text(extent = {{ -20, -60},{20, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "Gate"),Ellipse(extent = {{50,6},{62, -6}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Documentation(revisions = "<HTML>
<ul>
<li><i>September 15, 2004</i> vector approach used for all fixed numbers of inputs
       by Christoph Clauss<br>
       </li>
<li><i>October 22, 2003</i>
       by Teresa Schlegel<br>
       initially modelled.</li>
</ul>
</HTML>", info = "<html>
<p>The NandGate model has a multiple valued (n) input vector, and a single valued output. It is composed by a Basic Nand and an InertialDelaySensitive. Its parameters are the delay parameters (rise and fall intertial delay time, and intitial value).</p>
</html>"), Placement(transformation(extent = {{0, -20},{40,20}}, rotation = 0)));
          annotation(Documentation(info = "<HTML>
<P>
NandGate with n input values, composed by Nand and sensitive intertial delay.
</P>
<p><b>Release Notes:</b></p>
<ul>
<li><i>   </i>
       by ...<br>
       </li>
<li><i>October 22, 2003</i>
       by Liane Jacobi<br>
       realized.</li>
</ul>
</HTML>
"), Icon(Text(extent = [ -40,80;40,40], style(color = 0, thickness = 0.5), string = "&", lineColor = {0,0,255}), Text(extent = [ -20, -40;20, -80], style(color = 0, thickness = 0.5), string = "Gate", lineColor = {0,0,255}), Ellipse(extent = [60, -10;40,10], style(color = 0, thickness = 0.5), lineColor = {0,0,255})), Diagram);

        equation 
          connect(x,G1.x) annotation(Line(points = {{ -60,0},{ -32,0}}, color = {127,0,127}));
          connect(G1.y,G2.x) annotation(Line(points = {{0,0},{28,0}}, color = {127,0,127}));
          connect(G2.y,y) annotation(Line(points = {{60,0},{100,0}}, color = {127,0,127}));
        end NandGate;
        model OrGate "OrGate with multiple input"
          import D = Modelica.Electrical.Digital;
          extends D.Delay.DelayParams;
          extends D.Interfaces.MISO;
          D.Basic.Or G1(final n = n) annotation(Placement(transformation(extent = {{ -40, -20},{0,20}}, rotation = 0)));
          D.Delay.InertialDelaySensitive G2(tLH = tLH, tHL = tHL, y0 = y0) annotation(Placement(transformation(extent = {{20, -20},{60,20}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>The OrGate model has a multiple valued (n) input vector, and a single valued output. It is composed by a Basic Or and an InertialDelaySensitive. Its parameters are the delay parameters (rise and fall intertial delay time, and intitial value).</p>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004</i> vector approach used for all fixed numbers of inputs
       by Christoph Clauss<br>
       </li>
<li><i>October 22, 2003</i>
       by Teresa Schlegel<br>
       initially modelled.</li>
</ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{152, -160},{ -148, -100}}, lineColor = {0,0,255}, textString = "%name"),Rectangle(extent = {{ -50,100},{50, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {213,170,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -50,80},{50,40}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = ">=1"),Text(extent = {{ -20, -60},{20, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "Gate")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          connect(G1.y,G2.x) annotation(Line(points = {{0,0},{28,0}}, color = {127,0,127}));
          connect(x,G1.x) annotation(Line(points = {{ -60,0},{ -32,0}}, color = {127,0,127}));
          connect(y,G2.y) annotation(Line(points = {{100,0},{60,0}}, color = {127,0,127}));
        end OrGate;
        model NorGate "NorGate with multiple input"
          import D = Modelica.Electrical.Digital;
          extends D.Delay.DelayParams;
          extends D.Interfaces.MISO;
          D.Basic.Nor G1(final n = n) annotation(Placement(transformation(extent = {{ -40, -20},{0,20}}, rotation = 0)));
          D.Delay.InertialDelaySensitive G2(tLH = tLH, tHL = tHL, y0 = y0) annotation(Placement(transformation(extent = {{20, -20},{60,20}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>The NorGate model has a multiple valued (n) input vector, and a single valued output. It is composed by a Basic Nor and an InertialDelaySensitive. Its parameters are the delay parameters (rise and fall intertial delay time, and intitial value). </p>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004</i> vector approach used for all fixed numbers of inputs
       by Christoph Clauss<br>
       </li>
<li><i>October 22, 2003</i>
       by Liane Jacobi<br>
       initially modelled.</li>
</ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{152, -160},{ -148, -100}}, lineColor = {0,0,255}, textString = "%name"),Rectangle(extent = {{ -50,100},{50, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {213,170,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -50,80},{50,40}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = ">=1"),Text(extent = {{ -20, -60},{20, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "Gate"),Ellipse(extent = {{50,6},{62, -6}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          connect(G1.y,G2.x) annotation(Line(points = {{0,0},{28,0}}, color = {127,0,127}));
          connect(x,G1.x) annotation(Line(points = {{ -60,0},{ -32,0}}, color = {127,0,127}));
          connect(y,G2.y) annotation(Line(points = {{100,0},{60,0}}, color = {127,0,127}));
        end NorGate;
        model XorGate "XorGate with multiple input"
          import D = Modelica.Electrical.Digital;
          extends D.Delay.DelayParams;
          extends D.Interfaces.MISO;
          D.Basic.Xor G1(final n = n) annotation(Placement(transformation(extent = {{ -40, -20},{0,20}}, rotation = 0)));
          D.Delay.InertialDelaySensitive G2(tLH = tLH, tHL = tHL, y0 = y0) annotation(Placement(transformation(extent = {{20, -20},{60,20}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>The XorGate model has a multiple valued (n) input vector, and a single valued output. It is composed by a Basic Xor and an InertialDelaySensitive. Its parameters are the delay parameters (rise and fall intertial delay time, and intitial value). </p>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004</i> vector approach used for all fixed numbers of inputs
       by Christoph Clauss<br>
       </li>
<li><i>October 22, 2003</i>
       by Liane Jacobi<br>
       initially modelled.</li>
</ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{152, -160},{ -148, -100}}, lineColor = {0,0,255}, textString = "%name"),Rectangle(extent = {{ -50,100},{50, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {213,170,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -50,80},{50,40}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "=1"),Text(extent = {{ -20, -60},{20, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "Gate")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          connect(x,G1.x) annotation(Line(points = {{ -60,0},{ -32,0}}, color = {127,0,127}));
          connect(G1.y,G2.x) annotation(Line(points = {{0,0},{28,0}}, color = {127,0,127}));
          connect(G2.y,y) annotation(Line(points = {{60,0},{100,0}}, color = {127,0,127}));
        end XorGate;
        model XnorGate "XnorGate with multiple input"
          import D = Modelica.Electrical.Digital;
          extends D.Delay.DelayParams;
          extends D.Interfaces.MISO;
          D.Basic.Xnor G1(final n = n) annotation(Placement(transformation(extent = {{ -40, -20},{0,20}}, rotation = 0)));
          D.Delay.InertialDelaySensitive G2(tLH = tLH, tHL = tHL, y0 = y0) annotation(Placement(transformation(extent = {{20, -20},{60,20}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>The XnorGate model has a multiple valued (n) input vector, and a single valued output. It is composed by a Basic Xnor and an InertialDelaySensitive. Its parameters are the delay parameters (rise and fall intertial delay time, and intitial value). </p>
</html>", revisions = "<HTML>
<ul>
<li><i>September 15, 2004</i> vector approach used for all fixed numbers of inputs
       by Christoph Clauss<br>
       </li>
<li><i>October 22, 2003</i>
       by Liane Jacobi<br>
       initially modelled.</li>
</ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{152, -160},{ -148, -100}}, lineColor = {0,0,255}, textString = "%name"),Rectangle(extent = {{ -50,100},{50, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {213,170,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -40,80},{40,40}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "="),Text(extent = {{ -20, -60},{20, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "Gate")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          connect(x,G1.x) annotation(Line(points = {{ -60,0},{ -32,0}}, color = {127,0,127}));
          connect(G2.y,y) annotation(Line(points = {{60,0},{100,0}}, color = {127,0,127}));
          connect(G1.y,G2.x) annotation(Line(points = {{0,0},{28,0}}, color = {127,0,127}));
        end XnorGate;
        model BufGate "BufGate with 1 input value, composed by Not and sensitive intertial delay"
          import D = Modelica.Electrical.Digital;
          extends D.Delay.DelayParams;
          extends D.Interfaces.SISO;
          D.Delay.InertialDelaySensitive G1(tLH = tLH, tHL = tHL) annotation(Placement(transformation(extent = {{ -30, -20},{10,20}}, rotation = 0)));
          annotation(Documentation(info = "<html>
<p>The BufGate model has a single valued input, and a single valued output. It consists of an InertialDelaySensitive only. Its parameters are the delay parameters (rise and fall intertial delay time, and intitial value).</p>
</html>", revisions = "<HTML>
<ul>
<li><i>   </i>
       by ...<br>
       </li>
<li><i>September 21, 2004</i>
       by Andre Schneider<br>
       initially modelled.</li>
</ul>
</HTML>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{152, -160},{ -148, -100}}, lineColor = {0,0,255}, textString = "%name"),Rectangle(extent = {{ -50,100},{50, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {213,170,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -18, -60},{20, -100}}, lineColor = {0,0,0}, textString = "Gate"),Text(extent = {{ -50,80},{50,40}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,191,127}, fillPattern = FillPattern.Solid, textString = "1")}));

        equation 
          connect(G1.y,y) annotation(Line(points = {{10,0},{100,0}}, color = {127,0,127}));
          connect(G1.x,x) annotation(Line(points = {{ -22,0},{ -60,0}}, color = {127,0,127}));
        end BufGate;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -100},{80,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{ -100,50},{ -80,70},{100,70},{80,50},{ -100,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{100,70},{100, -80},{80, -100},{80,50},{100,70}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -85,35},{65, -85}}, lineColor = {0,0,255}, textString = "Library"),Text(extent = {{ -120,122},{120,73}}, lineColor = {255,0,0}, textString = "%name")}), Documentation(info = "<html>
<p>Gates contains the basic gates according to standard logic as they are provided in the Basic package. Additionally they contain an InertialDelaySensitive component They are composed graphically, not using any equations.</p>
</html>"));
      end Gates;
      package Sources "Time-dependend digital signal sources"
        block Set "Digital Set Source"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          parameter D.Interfaces.Logic x(start = L.'1') "Logic value to be set";
          D.Interfaces.DigitalOutput y annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          annotation(Documentation(info = "<HTML>
<P>
Sets a nine valued digital signal, which is specified by the <i>setval</i> parameter.
</P>
<P>
To specify <i>setval</i>, the integer code has to be used.
</P>
<p><b>Code Table</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Logic value</b></td>
      <td valign=\"top\"><b>Integer code</b></td>
      <td valign=\"top\"><b>Meaning</b></td>
  </tr>

  <tr><td valign=\"top\">'U'</td> <td valign=\"top\">1</td> <td valign=\"top\">Uninitialized</td></tr>
  <tr><td valign=\"top\">'X'</td> <td valign=\"top\">2</td> <td valign=\"top\">Forcing Unknown</td></tr>
  <tr><td valign=\"top\">'0'</td> <td valign=\"top\">3</td> <td valign=\"top\">Forcing 0</td></tr>
  <tr><td valign=\"top\">'1'</td> <td valign=\"top\">4</td> <td valign=\"top\">Forcing 1</td></tr>
  <tr><td valign=\"top\">'Z'</td> <td valign=\"top\">5</td> <td valign=\"top\">High Impedance</td></tr>
  <tr><td valign=\"top\">'W'</td> <td valign=\"top\">6</td> <td valign=\"top\">Weak Unknown</td></tr>
  <tr><td valign=\"top\">'L'</td> <td valign=\"top\">7</td> <td valign=\"top\">Weak 0</td></tr>
  <tr><td valign=\"top\">'H'</td> <td valign=\"top\">8</td> <td valign=\"top\">Weak 1</td></tr>
  <tr><td valign=\"top\">'-'</td> <td valign=\"top\">9</td> <td valign=\"top\">Don't care</td></tr>
</table>

<P>
If the logic values are imported by <br><b> import L = Digital.Interfaces.Logic; </b> <br>
they can be used to specify the parameter, e.g. <b>L.'0'</b> for forcing 0.
</P>
</HTML>
", revisions = "<HTML>
<ul>
<li><i>   </i>
       by ...<br>
       </li>
<li><i>August 20, 2003</i>
       by Teresa Schlegel<br>
       initially modelled.</li>
</ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -50,100},{50, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {213,255,170}, fillPattern = FillPattern.Solid),Text(extent = {{ -50,40},{50,80}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "SET"),Text(extent = {{152, -160},{ -148, -100}}, lineColor = {0,0,255}, textString = "%name"),Line(points = {{50,0},{90,0}}, color = {127,0,127})}));
        algorithm 
          y:=x;
        end Set;
        block Step "Digital Step Source"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          parameter D.Interfaces.Logic before(start = L.'0') "Logic value before step";
          parameter D.Interfaces.Logic after(start = L.'1') "Logic value after step";
          parameter Real stepTime(start = 1) "step time";
          D.Interfaces.DigitalOutput y annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          annotation(Documentation(info = "<HTML>
<P>
The step source output signal steps from the value <i>before</i> to the value <i>after</i>
at the time <i>stepTime</i>.
</P>
<P>
To specify the logic value parameters, the integer code has to be used.
</P>
<p><b>Code Table</b></p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Logic value</b></td>
      <td valign=\"top\"><b>Integer code</b></td>
      <td valign=\"top\"><b>Meaning</b></td>
  </tr>

  <tr><td valign=\"top\">'U'</td> <td valign=\"top\">1</td> <td valign=\"top\">Uninitialized</td></tr>
  <tr><td valign=\"top\">'X'</td> <td valign=\"top\">2</td> <td valign=\"top\">Forcing Unknown</td></tr>
  <tr><td valign=\"top\">'0'</td> <td valign=\"top\">3</td> <td valign=\"top\">Forcing 0</td></tr>
  <tr><td valign=\"top\">'1'</td> <td valign=\"top\">4</td> <td valign=\"top\">Forcing 1</td></tr>
  <tr><td valign=\"top\">'Z'</td> <td valign=\"top\">5</td> <td valign=\"top\">High Impedance</td></tr>
  <tr><td valign=\"top\">'W'</td> <td valign=\"top\">6</td> <td valign=\"top\">Weak Unknown</td></tr>
  <tr><td valign=\"top\">'L'</td> <td valign=\"top\">7</td> <td valign=\"top\">Weak 0</td></tr>
  <tr><td valign=\"top\">'H'</td> <td valign=\"top\">8</td> <td valign=\"top\">Weak 1</td></tr>
  <tr><td valign=\"top\">'-'</td> <td valign=\"top\">9</td> <td valign=\"top\">Don't care</td></tr>
</table>
<P>
If the logic values are imported by <br><b> import L = Digital.Interfaces.Logic; </b> <br>
they can be used to specify the parameter, e.g. <b>L.'0'</b> for forcing 0.
</P>
</HTML>
", revisions = "<HTML>
<ul>
<li><i>   </i>
       by ...<br>
       </li>
<li><i>August 20, 2003</i>
       by Teresa Schlegel<br>
       initially modelled.</li>
</ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -50,100},{50, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {213,255,170}, fillPattern = FillPattern.Solid),Text(extent = {{152, -160},{ -148, -100}}, lineColor = {0,0,255}, textString = "%name"),Line(points = {{ -30,40},{0,40},{0,80},{30,80}}, color = {0,0,0}),Line(points = {{50,0},{90,0}}, color = {127,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
        algorithm 
          when initial() then
                      y:=before;
          
          end when;
          if time >= stepTime then 
                    y:=after;

          else           y:=before;

          end if;
        end Step;
        block Table "Digital Tabular Source"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          parameter D.Interfaces.Logic x[:] = {1} "vector of values";
          parameter Real t[size(x, 1)] = {1} "vector of corresponding time points";
          parameter D.Interfaces.Logic y0 = L.'U' "initial output value";
          final parameter Integer n = size(x, 1) "table size";
          D.Interfaces.DigitalOutput y annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
          annotation(Documentation(info = "<HTML>
<P>
The table source output signal <i>y</i> steps to the values of the  <i>x</i> table at the corresponding
timepoints in the <i>t</i> table. <br>The initial value is specified by <i>y0</i>.
</P>
<P>
To specify the logic value parameters, the integer code has to be used.
</P>
<p><b>Code Table</b></p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Logic value</b></td>
      <td valign=\"top\"><b>Integer code</b></td>
      <td valign=\"top\"><b>Meaning</b></td>
  </tr>

  <tr><td valign=\"top\">'U'</td> <td valign=\"top\">1</td> <td valign=\"top\">Uninitialized</td></tr>
  <tr><td valign=\"top\">'X'</td> <td valign=\"top\">2</td> <td valign=\"top\">Forcing Unknown</td></tr>
  <tr><td valign=\"top\">'0'</td> <td valign=\"top\">3</td> <td valign=\"top\">Forcing 0</td></tr>
  <tr><td valign=\"top\">'1'</td> <td valign=\"top\">4</td> <td valign=\"top\">Forcing 1</td></tr>
  <tr><td valign=\"top\">'Z'</td> <td valign=\"top\">5</td> <td valign=\"top\">High Impedance</td></tr>
  <tr><td valign=\"top\">'W'</td> <td valign=\"top\">6</td> <td valign=\"top\">Weak Unknown</td></tr>
  <tr><td valign=\"top\">'L'</td> <td valign=\"top\">7</td> <td valign=\"top\">Weak 0</td></tr>
  <tr><td valign=\"top\">'H'</td> <td valign=\"top\">8</td> <td valign=\"top\">Weak 1</td></tr>
  <tr><td valign=\"top\">'-'</td> <td valign=\"top\">9</td> <td valign=\"top\">Don't care</td></tr>
</table>
<P>
If the logic values are imported by <br><b> import L = Digital.Interfaces.Logic; </b> <br>
they can be used to specify the parameter, e.g. <b>L.'0'</b> for forcing 0.
</P>
</HTML>
", revisions = "<HTML>
<ul>
<li><i>   </i>
       by ...<br>
       </li>
<li><i>August 20, 2003</i>
       by Teresa Schlegel<br>
       initially modelled.</li>
</ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -50,100},{50, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {213,255,170}, fillPattern = FillPattern.Solid),Text(extent = {{152, -160},{ -148, -100}}, lineColor = {0,0,255}, textString = "%name"),Line(points = {{50,0},{90,0}}, color = {127,0,127}),Rectangle(extent = {{ -30,80},{32,70}}, lineColor = {0,0,0}, fillColor = {215,215,215}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -30,70},{32,40}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,80},{0,40}}, color = {0,0,0}),Line(points = {{ -30,60},{32,60}}, color = {0,0,0}),Line(points = {{ -30,50},{32,50}}, color = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
        algorithm 
          if initial() then 
                    assert(n > 0, "Invalid size of table (n < 1)");
          for i in 1:n loop
          
          end for;

          else 
          end if;
          y:=y0;
          for i in 1:n loop
                      if time >= t[i] then 
                        y:=x[i];

            else 
            end if;

          end for;
        end Table;
        model Pulse "Digital Pulse Source"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          parameter Real width(final min = Modelica.Constants.small, final max = 100, start = 50) "Widths of pulses in % of periods";
          parameter Modelica.SIunits.Time period(final min = Modelica.Constants.small, start = 1) "Time for one period";
          parameter Modelica.SIunits.Time startTime(start = 0) "Output = quiet for time < startTime";
          parameter D.Interfaces.Logic pulse(start = L.'0') "pulsed value";
          parameter D.Interfaces.Logic quiet(start = L.'1') "quiet value";
          Modelica.SIunits.Time T0(final start = startTime) "Start time of current period";
          parameter Integer nperiod(start =  -1) "Number of periods (< 0 means infinite number of periods)";
          Integer np(start = 0, fixed = true);
          D.Interfaces.DigitalOutput y annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
        protected 
          Boolean sampling;
          annotation(Documentation(info = "<HTML>
<P>
The pulse source forms pulses between the  <i>quiet</i> value and the <i>pulse</i> value.
The pulse length <i>width</i> is specified in percent of the period length <i>period</i>.
The number of periods is specified by <i>nperiod</i>. If <i>nperiod</i> is less than zero,
the number of periods is unlimited.
</P>
<P>
To specify the logic value parameters, the integer code has to be used.
</P>
<p><b>Code Table</b></p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Logic value</b></td>
      <td valign=\"top\"><b>Integer code</b></td>
      <td valign=\"top\"><b>Meaning</b></td>
  </tr>

  <tr><td valign=\"top\">'U'</td> <td valign=\"top\">1</td> <td valign=\"top\">Uninitialized</td></tr>
  <tr><td valign=\"top\">'X'</td> <td valign=\"top\">2</td> <td valign=\"top\">Forcing Unknown</td></tr>
  <tr><td valign=\"top\">'0'</td> <td valign=\"top\">3</td> <td valign=\"top\">Forcing 0</td></tr>
  <tr><td valign=\"top\">'1'</td> <td valign=\"top\">4</td> <td valign=\"top\">Forcing 1</td></tr>
  <tr><td valign=\"top\">'Z'</td> <td valign=\"top\">5</td> <td valign=\"top\">High Impedance</td></tr>
  <tr><td valign=\"top\">'W'</td> <td valign=\"top\">6</td> <td valign=\"top\">Weak Unknown</td></tr>
  <tr><td valign=\"top\">'L'</td> <td valign=\"top\">7</td> <td valign=\"top\">Weak 0</td></tr>
  <tr><td valign=\"top\">'H'</td> <td valign=\"top\">8</td> <td valign=\"top\">Weak 1</td></tr>
  <tr><td valign=\"top\">'-'</td> <td valign=\"top\">9</td> <td valign=\"top\">Don't care</td></tr>
</table>
<P>
If the logic values are imported by <br><b> import L = Digital.Interfaces.Logic; </b> <br>
they can be used to specify the parameter, e.g. <b>L.'0'</b> for forcing 0.
</P>
</HTML>
", revisions = "<HTML>
<ul>
<li><i>   </i>
       by ...<br>
       </li>
<li><i>September 2, 2003</i>
       by Christoph Clauss<br>
       initially modelled.</li>
</ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -50,100},{50, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {213,255,170}, fillPattern = FillPattern.Solid),Text(extent = {{152, -160},{ -148, -100}}, lineColor = {0,0,255}, textString = "%name"),Line(points = {{50,0},{90,0}}, color = {127,0,127}),Line(points = {{ -36,40},{ -30,40},{ -30,40},{ -18,40},{ -18,40},{ -14,40},{ -14,80},{14,80},{14,40},{14,40},{14,40},{28,40},{30,40},{36,40}}, color = {0,0,0})}));

        equation 
          sampling = nperiod <> 0 and (nperiod >= pre(np) or nperiod < 0);
          when sampling and sample(startTime, period) then
                      T0 = time;
            np = if nperiod > 0 then pre(np) + 1 else pre(np);
          
          end when;
          if sampling then
            y = if time < startTime or time >= T0 + (width * period) / 100 then quiet else pulse;

          else
            y = quiet;

          end if;
        end Pulse;
        model Clock "Digital Clock Source"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          parameter Modelica.SIunits.Time startTime(start = 0) "Output = offset for time < startTime";
          parameter Modelica.SIunits.Time period(final min = Modelica.Constants.small, start = 1) "Time for one period";
          parameter Real width(final min = Modelica.Constants.small, final max = 100, start = 50) "Width of pulses in % of period";
          D.Interfaces.DigitalOutput y "Connector of Digital output signal" annotation(Placement(transformation(extent = {{90, -10},{110,10}}, rotation = 0)));
        protected 
          Modelica.SIunits.Time t_i(final start = startTime) "Start time of current period";
          Modelica.SIunits.Time t_width = (period * width) / 100;
          annotation(Documentation(info = "<HTML>
<P>
The clock source forms pulses between the  <i>'0'</i> value (forcing 0) and the <i>'1'</i> value (forcing 1).
The pulse length <i>width</i> is specified in percent of the period length <i>period</i>.
The number of periods is unlimited. The first pulse starts at startTime.
</P>
<P> The clock source is a special but often used variant of the pulse source.
</P>
</HTML>
", revisions = "<HTML>
<ul>
<li><i>   </i>
       by ...<br>
       </li>
<li><i>September 18, 2004</i>
       by Andre Schneider<br>
       initially modelled.</li>
</ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -50,100},{50, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {213,255,170}, fillPattern = FillPattern.Solid),Text(extent = {{152, -160},{ -148, -100}}, lineColor = {0,0,255}, textString = "%name"),Line(points = {{50,0},{90,0}}, color = {127,0,127}),Line(points = {{ -36,40},{ -30,40},{ -30,80},{ -18,80},{ -18,40},{ -6,40},{ -6,80},{6,80},{6,40},{18,40},{18,80},{30,80},{30,40},{36,40}}, color = {0,0,0})}));
        algorithm 
          when sample(startTime, period) then
                      t_i:=time;
          
          end when;
          y:=if not time >= startTime or time >= t_i + t_width then L.'0' else L.'1';
        end Clock;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -100},{80,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{ -100,50},{ -80,70},{100,70},{80,50},{ -100,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{100,70},{100, -80},{80, -100},{80,50},{100,70}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -85,35},{65, -85}}, lineColor = {0,0,255}, textString = "Library"),Text(extent = {{ -120,122},{120,73}}, lineColor = {255,0,0}, textString = "%name")}), Documentation(info = "<html>
<p>The sources are not taken from Standard Logic. They were added since they turned out to be quite useful, since such sources are often needed. For a better optical overview the colour green was chosen for the sources.</p>
</html>"));
      end Sources;
      package Converters "Converters between 2-,3-,4- and 9-valued logic"
        block LogicToXO1 "Conversion to XO1"
          import D = Modelica.Electrical.Digital;
          import T = Modelica.Electrical.Digital.Tables;
          D.Interfaces.DigitalInput x[n] annotation(Placement(transformation(extent = {{ -60, -10},{ -40,10}}, rotation = 0)));
          D.Interfaces.DigitalOutput y[n] annotation(Placement(transformation(extent = {{40, -10},{60,10}}, rotation = 0)));
          parameter Integer n(final min = 1, start = 1) "signal width";
          annotation(Documentation(info = "<HTML>
<P>
Conversion of a nine valued digital input into a X01 digital output without
any delay according to IEEE 1164 To_X01 function.
</P>
<p><b>Conversion Table:</b></p>
<pre>
                          input                  output
                          'U' (coded by 1)       'X'  (coded by 2)
                          'X' (coded by 2)       'X'  (coded by 2)
                          '0' (coded by 3)       '0'  (coded by 3)
                          '1' (coded by 4)       '1'  (coded by 4)
                          'Z' (coded by 5)       'X'  (coded by 2)
                          'W' (coded by 6)       'X'  (coded by 2)
                          'L' (coded by 7)       '0'  (coded by 3)
                          'H' (coded by 8)       '1'  (coded by 4)
                          '-' (coded by 9)       'X'  (coded by 2)
</pre>
<P>
If the signal width is greater than 1 this conversion is done for each signal.
</P>
</HTML>
", revisions = "<HTML>
<ul>
<li><i>September 15, 2004   </i>
       by Christoph Clauss name converted from cvt_to_x01 into LogicToX01<br>
       </li>
<li><i>November 5, 2003</i>
       by Christoph Clauss<br>
       initially modelled.</li>
</ul>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{152, -100},{ -148, -40}}, lineColor = {0,0,255}, textString = "%name"),Polygon(points = {{ -40, -40},{ -40,40},{40,40},{ -40, -40}}, lineColor = {127,0,127}, fillColor = {127,0,127}, fillPattern = FillPattern.Solid),Polygon(points = {{ -40, -40},{40, -40},{40,40},{ -40, -40}}, lineColor = {127,0,127}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{0, -20},{40, -40}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "X01"),Rectangle(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,0}, lineThickness = 0.5)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
        algorithm 
          for i in 1:n loop
                      y[i]:=T.X01Table[x[i]];

          end for;
        end LogicToXO1;
        block LogicToXO1Z "Conversion to XO1Z"
          import D = Modelica.Electrical.Digital;
          import T = Modelica.Electrical.Digital.Tables;
          D.Interfaces.DigitalInput x[n] annotation(Placement(transformation(extent = {{ -60, -10},{ -40,10}}, rotation = 0)));
          D.Interfaces.DigitalOutput y[n] annotation(Placement(transformation(extent = {{40, -10},{60,10}}, rotation = 0)));
          parameter Integer n(final min = 1, start = 1) "signal width";
          annotation(Documentation(info = "<HTML>
<P>
Conversion of a nine valued digital input into a X01Z digital output without
any delay according to IEEE 1164 To_X01Z function.
</P>
<p><b>Conversion Table:</b></p>
<pre>
                          input                  output
                          'U' (coded by 1)       'X'  (coded by 2)
                          'X' (coded by 2)       'X'  (coded by 2)
                          '0' (coded by 3)       '0'  (coded by 3)
                          '1' (coded by 4)       '1'  (coded by 4)
                          'Z' (coded by 5)       'Z'  (coded by 5)
                          'W' (coded by 6)       'X'  (coded by 2)
                          'L' (coded by 7)       '0'  (coded by 3)
                          'H' (coded by 8)       '1'  (coded by 4)
                          '-' (coded by 9)       'X'  (coded by 2)
</pre>
<P>
If the signal width is greater than 1 this conversion is done for each signal.
</P>
</HTML>
", revisions = "<HTML>
<ul>
<li><i>September 15, 2004   </i>
       by Christoph Clauss name converted from cvt_to_x01z into LogicToX01Z<br>
       </li>
<li><i>November 5, 2003</i>
       by Christoph Clauss<br>
       initially modelled.</li>
</ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{152, -100},{ -148, -40}}, lineColor = {0,0,255}, textString = "%name"),Polygon(points = {{ -40, -40},{ -40,40},{40,40},{ -40, -40}}, lineColor = {127,0,127}, fillColor = {127,0,127}, fillPattern = FillPattern.Solid),Polygon(points = {{ -40, -40},{40, -40},{40,40},{ -40, -40}}, lineColor = {127,0,127}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{0, -20},{40, -40}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "X01Z"),Rectangle(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,0}, lineThickness = 0.5)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
        algorithm 
          for i in 1:n loop
                      y[i]:=T.X01ZTable[x[i]];

          end for;
        end LogicToXO1Z;
        block LogicToUX01 "Conversion to UXO1"
          import D = Modelica.Electrical.Digital;
          import T = Modelica.Electrical.Digital.Tables;
          D.Interfaces.DigitalInput x[n] annotation(Placement(transformation(extent = {{ -60, -10},{ -40,10}}, rotation = 0)));
          D.Interfaces.DigitalOutput y[n] annotation(Placement(transformation(extent = {{40, -10},{60,10}}, rotation = 0)));
          parameter Integer n(final min = 1, start = 1) "signal width";
          annotation(Documentation(info = "<HTML>
<P>
Conversion of a nine valued digital input into a UX01 digital output without
any delay according to IEEE 1164 To_UX01 function.
</P>
<p><b>Conversion Table:</b></p>
<pre>
                          input                  output
                          'U' (coded by 1)       'U'  (coded by 1)
                          'X' (coded by 2)       'X'  (coded by 2)
                          '0' (coded by 3)       '0'  (coded by 3)
                          '1' (coded by 4)       '1'  (coded by 4)
                          'Z' (coded by 5)       'X'  (coded by 2)
                          'W' (coded by 6)       'X'  (coded by 2)
                          'L' (coded by 7)       '0'  (coded by 3)
                          'H' (coded by 8)       '1'  (coded by 4)
                          '-' (coded by 9)       'X'  (coded by 2)
</pre>
<P>
If the signal width is greater than 1 this conversion is done for each signal.
</P>
</HTML>
", revisions = "<HTML>
<ul>
<li><i>September 15, 2004   </i>
       by Christoph Clauss name converted from cvt_to_ux01 into LogicToUX01<br>
       </li>
<li><i>November 5, 2003</i>
       by Christoph Clauss<br>
       initially modelled.</li>
</ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{152, -100},{ -148, -40}}, lineColor = {0,0,255}, textString = "%name"),Polygon(points = {{ -40, -40},{ -40,40},{40,40},{ -40, -40}}, lineColor = {127,0,127}, fillColor = {127,0,127}, fillPattern = FillPattern.Solid),Polygon(points = {{ -40, -40},{40, -40},{40,40},{ -40, -40}}, lineColor = {127,0,127}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{0, -20},{40, -40}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "UX01"),Rectangle(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,0}, lineThickness = 0.5)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));
        algorithm 
          for i in 1:n loop
                      y[i]:=T.UX01Table[x[i]];

          end for;
        end LogicToUX01;
        block BooleanToLogic "Boolean to Logic converter"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          Modelica.Blocks.Interfaces.BooleanInput x[n] annotation(Placement(transformation(extent = {{ -60, -10},{ -40,10}}, rotation = 0)));
          parameter Integer n(final min = 1, start = 1) "signal width";
          Modelica.Electrical.Digital.Interfaces.DigitalOutput y[n] annotation(Placement(transformation(extent = {{40, -10},{60,10}}, rotation = 0)));
          annotation(extent = [40, -10;60,10]);
          annotation(Documentation(info = "<HTML>
<P>
Conversion of a Boolean input into a digital output without any delay according to:
</P>
<pre>
                          input      output
                          true       '1'  (coded by 4)
                          false      '0'  (coded by 3)
</pre>
<P>
If the signal width is greater than 1 this conversion is done for each signal.
</P>
</HTML>
", revisions = "<HTML>
<ul>
<li><i>September 15, 2004   </i>
       by Christoph Clauss colors changed<br>
       </li>
<li><i>November 4, 2003</i>
       by Christoph Clauss<br>
       initially modelled.</li>
</ul>
</HTML>"), extent = [ -60, -10; -40,10], Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -40, -40},{40,40},{40, -40},{ -40, -40}}, lineColor = {127,0,127}, fillColor = {127,0,127}, fillPattern = FillPattern.Solid),Polygon(points = {{ -40,40},{40,40},{ -40, -40},{ -40,40}}, lineColor = {255,0,255}, fillColor = {255,0,255}, fillPattern = FillPattern.Solid),Text(extent = {{152, -100},{ -148, -40}}, lineColor = {0,0,255}, textString = "%name"),Rectangle(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,0}, lineThickness = 0.5)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Placement(transformation(extent = {{ -60, -10},{ -40,10}}, rotation = 0)));
          annotation(Diagram);

        equation 
          for i in 1:n loop
          y[i] = if x[i] then L.'1' else L.'0';

          end for;
        end BooleanToLogic;
        block LogicToBoolean "Logic to Boolean converter"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          Modelica.Electrical.Digital.Interfaces.DigitalInput x[n] annotation(Placement(transformation(extent = {{ -60, -10},{ -40,10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.BooleanOutput y[n] annotation(Placement(transformation(extent = {{40, -10},{60,10}}, rotation = 0)));
          parameter Integer n(final min = 1, start = 2) "signal width";
          annotation(Documentation(info = "<HTML>
<P>
Conversion of a digital input into a Boolean output without any delay according to:
</P>
<pre>
                          input                 output
                          'U'  (coded by 1)     false
                          'X'  (coded by 2)     false
                          '0'  (coded by 3)     false
                          '1'  (coded by 4)     true
                          'Z'  (coded by 5)     false
                          'W'  (coded by 6)     false
                          'L'  (coded by 7)     false
                          'H'  (coded by 8)     true
                          '-'  (coded by 9)     false
</pre>
<P>
If the signal width is greater than 1 this conversion is done for each signal.
</P>
</HTML>
", revisions = "<HTML>
<ul>
<li><i>September 15, 2004   </i>
       by Christoph Clauss colors changed<br>
       </li>
<li><i>November 4, 2003</i>
       by Christoph Clauss<br>
       initially modelled.</li>
</ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -40,40},{40,40},{ -40, -40},{ -40,40}}, lineColor = {127,0,127}, fillColor = {127,0,127}, fillPattern = FillPattern.Solid),Text(extent = {{152, -100},{ -148, -40}}, lineColor = {0,0,255}, textString = "%name"),Polygon(points = {{ -40, -40},{40, -40},{40,40},{ -40, -40}}, lineColor = {255,0,255}, fillColor = {255,0,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,0}, lineThickness = 0.5)}));

        equation 
          for i in 1:n loop
          y[i] = if x[i] == 4 or x[i] == 8 then true else false;

          end for;
        end LogicToBoolean;
        block RealToLogic "Real to Logic converter"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          Modelica.Blocks.Interfaces.RealInput x[n] annotation(Placement(transformation(extent = {{ -60, -10},{ -40,10}}, rotation = 0)));
          Digital.Interfaces.DigitalOutput y[n] annotation(Placement(transformation(extent = {{40, -10},{60,10}}, rotation = 0)));
          parameter Integer n(final min = 1, start = 1) "signal width";
          parameter Real upper_limit(start = 1) "upper limit";
          parameter Real lower_limit(start = 0) "lower limit";
          parameter L upper_value(start = L.'1') "output if input > upper_limit";
          parameter L lower_value(start = L.'0') "output if input < lower_limit";
          parameter L middle_value(start = L.'X') "output else";
          annotation(extent = [40, -10;60,10]);
          annotation(Documentation(info = "<HTML>
<P>
Conversion of a real input into a digital output without any delay according to:
</P>
<pre>
                                 condition            output
      first check:               input greater upp    lupp
      second check:              input larger low     llow
                                 else                 lmid
</pre>
<P>
If the signal width is greater than 1 this conversion is done for each signal.
</P>
</HTML>
", revisions = "<HTML>
<ul>
<li><i>September 15, 2004   </i>
       by Christoph Clauss colors changed<br>
       </li>
<li><i>November 5, 2003</i>
       by Christoph Clauss<br>
       initially modelled.</li>
</ul>
</HTML>"), extent = [ -60, -10; -40,10], Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -40, -40},{40,40},{40, -40},{ -40, -40}}, lineColor = {127,0,127}, fillColor = {127,0,127}, fillPattern = FillPattern.Solid),Polygon(points = {{ -40,40},{40,40},{ -40, -40},{ -40,40}}, lineColor = {0,0,255}),Text(extent = {{148, -100},{ -152, -40}}, lineColor = {0,0,255}, textString = "%name"),Rectangle(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,0}, lineThickness = 0.5)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Placement(transformation(extent = {{ -60, -10},{ -40,10}}, rotation = 0)));
          annotation(Diagram);

        equation 
          for i in 1:n loop
          y[i] = if x[i] > upper_limit then upper_value else if x[i] < lower_limit then lower_value else middle_value;

          end for;
        end RealToLogic;
        block LogicToReal "Logic to Real converter"
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          Modelica.Electrical.Digital.Interfaces.DigitalInput x[n] annotation(Placement(transformation(extent = {{ -60, -10},{ -40,10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealOutput y[n] annotation(Placement(transformation(extent = {{40, -10},{60,10}}, rotation = 0)));
          parameter Integer n(final min = 1, start = 1) "signal width";
          parameter Real value_U(start = 0.5) "value for digital U (uninitialized)";
          parameter Real value_X(start = 0.5) "value for digital X (Forcing Unknown)";
          parameter Real value_0(start = 0) "value for digital 0   (Forcing 0)";
          parameter Real value_1(start = 1) "value for digital 1 (Forcing 1)";
          parameter Real value_Z(start = 0.5) "value for digital Z (High Impedance)";
          parameter Real value_W(start = 0.5) "value for digital W (Weak    Unknown)";
          parameter Real value_L(start = 0) "value for digital L (Weak    0)";
          parameter Real value_H(start = 1) "value for digital H (Weak    1)";
          parameter Real value_m(start = 0.5) "value for digital m (Dont care)";
          annotation(Documentation(info = "<HTML>
<P>
Conversion of a digital input into a Real output without any delay according to:
</P>
<pre>
                          input                 output
                          'U'  (coded by 1)     val_U
                          'X'  (coded by 2)     val_X
                          '0'  (coded by 3)     val_0
                          '1'  (coded by 4)     val_1
                          'Z'  (coded by 5)     val_Z
                          'W'  (coded by 6)     val_W
                          'L'  (coded by 7)     val_L
                          'H'  (coded by 8)     val_H
                          '-'  (coded by 9)     val_m
</pre>
<P>
The values val... are given by parameters.</P>
<P>If the signal width is greater than 1 this conversion is done for each signal.
</P>
</HTML>
", revisions = "<HTML>
<ul>
<li><i>September 15, 2004   </i>
       by Christoph Clauss colors changed<br>
       </li>
<li><i>November 5, 2003</i>
       by Christoph Clauss<br>
       initially modelled.</li>
</ul>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -40,40},{40,40},{ -40, -40},{ -40,40}}, lineColor = {127,0,127}, fillColor = {127,0,127}, fillPattern = FillPattern.Solid),Polygon(points = {{ -40, -40},{40, -40},{40,40},{ -40, -40}}, lineColor = {0,0,255}),Text(extent = {{152, -100},{ -148, -40}}, lineColor = {0,0,255}, textString = "%name"),Rectangle(extent = {{ -40,40},{40, -40}}, lineColor = {0,0,0}, lineThickness = 0.5)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          for i in 1:n loop
          y[i] = if x[i] == L.'U' then value_U else if x[i] == L.'X' then value_X else if x[i] == L.'0' then value_0 else if x[i] == L.'1' then value_1 else if x[i] == L.'Z' then value_Z else if x[i] == L.'W' then value_W else if x[i] == L.'L' then value_L else if x[i] == L.'H' then value_H else value_m;

          end for;
        end LogicToReal;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -100},{80,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{ -100,50},{ -80,70},{100,70},{80,50},{ -100,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{100,70},{100, -80},{80, -100},{80,50},{100,70}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -85,35},{65, -85}}, lineColor = {0,0,255}, textString = "Library"),Text(extent = {{ -120,122},{120,73}}, lineColor = {255,0,0}, textString = "%name")}), Documentation(info = "<html>
<p>The Converter BooleanToLogic, LogicToBoolean, RealToLogic, and LogicTo Real components are not standard logic components. They were designed to easily convert from or to Boolean or Real valued signals. The LogicToX01, LogicToX01Z and LogicTo UX01 converters correspond to standard logic functions. They transform 9-valued logic to 3- or 4-valued logic.</p>
</html>"));
      end Converters;
      package Registers "Registers with N-bit input data and output data"
        model DFFR "Edge triggered register bank with reset"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          import S = Modelica.Electrical.Digital.Interfaces.Strength;
          import T = Modelica.Electrical.Digital.Tables;
          parameter Integer ResetMap[9] = {1,4,3,2,4,4,3,2,4} "function selection, defaults for high active reset";
          parameter D.Interfaces.Strength strength = S.'S_X01' "output strength";
          parameter Integer n(min = 1) = 1 "data width";
          D.Interfaces.DigitalInput reset annotation(Placement(transformation(extent = {{ -10, -100},{10, -80}}), iconTransformation(extent = {{ -10, -100},{10, -80}})));
          D.Interfaces.DigitalInput clock annotation(Placement(transformation(extent = {{ -90, -30},{ -70, -10}}), iconTransformation(extent = {{ -90, -30},{ -70, -10}})));
          D.Interfaces.DigitalInput dataIn[n] annotation(Placement(transformation(extent = {{ -90,20},{ -70,40}}), iconTransformation(extent = {{ -90,20},{ -70,40}})));
          D.Interfaces.DigitalOutput dataOut[n] annotation(Placement(transformation(extent = {{68,20},{88,40}}), iconTransformation(extent = {{69,20},{89,40}})));
        protected 
          Integer clock_flag(start = 0);
          Integer reset_flag(start = 1);
        protected 
          D.Interfaces.Logic nextstate[n](start = fill(L.'U', n));
          D.Interfaces.Logic next_assign_val[n](start = fill(L.'U', n));
        algorithm 
          if change(clock) or change(reset) then 
                    if change(clock) then 
                    if initial() then 
                    clock_flag:=T.ClockMap[L.'U',clock];

          else           clock_flag:=T.ClockMap[pre(clock),clock];

          end if;

          else 
          end if;
          reset_flag:=ResetMap[reset];
          for i in 1:n loop
                      if reset_flag == 1 then 
                        nextstate[i]:=L.'U';
            elseif reset_flag == 2 then
              nextstate[i]:=T.StrengthMap[L.'0',strength];

            elseif reset_flag == 3 then
              if clock_flag == 0 then 
                            break;
              elseif clock_flag == 1 then
                nextstate[i]:=T.StrengthMap[dataIn[i],strength];


              else               if next_assign_val[i] == T.StrengthMap[dataIn[i],strength] or next_assign_val[i] == L.'U' then 
                            break;
              elseif dataIn[i] == L.'U' then
                nextstate[i]:=L.'U';


              else               nextstate[i]:=T.StrengthMap[L.'X',strength];

              end if;

              end if;

            elseif reset_flag == 4 then
              if next_assign_val[i] == T.StrengthMap[L.'0',strength] and (dataIn[i] == L.'0' or dataIn[i] == L.'L' or clock_flag == 0) then 
                            break;
              elseif (dataIn[i] == L.'0' or dataIn[i] == L.'L') and clock_flag == 1 then
                nextstate[i]:=T.StrengthMap[L.'0',strength];

              elseif next_assign_val[i] == L.'U' and not clock_flag == 1 or dataIn[i] == L.'U' and not clock_flag == 0 then
                nextstate[i]:=L.'U';


              else               nextstate[i]:=T.StrengthMap[L.'X',strength];

              end if;


            else 
            end if;

          end for;

          else 
          end if;
          next_assign_val:=nextstate;
          dataOut:=nextstate;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -50,60},{50, -60}}, lineColor = {127,33,107}, lineThickness = 0.5, fillPattern = FillPattern.Solid, fillColor = {255,255,255}),Line(points = {{ -70,30},{ -50,30}}, color = {127,33,107}, thickness = 1, smooth = Smooth.None),Line(points = {{50,30},{76,30}}, color = {127,33,107}, thickness = 1, smooth = Smooth.None),Line(points = {{ -80, -20},{ -50, -20}}, color = {127,33,107}, smooth = Smooth.None),Line(points = {{0, -84},{0, -60}}, color = {127,33,107}, smooth = Smooth.None),Line(points = {{ -50, -8},{ -34, -20},{ -50, -32}}, color = {127,33,107}, smooth = Smooth.None),Text(extent = {{ -30,54},{20,8}}, lineColor = {127,33,107}, textString = "DFFR"),Text(extent = {{ -32, -14},{ -6, -26}}, lineColor = {127,33,107}, textString = "clock"),Text(extent = {{ -14, -44},{12, -56}}, lineColor = {127,33,107}, textString = "reset")}), Documentation(info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities<p>
<p><b>Truth Table for high active reset:</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Clock</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
      <td valign=\"top\">Map</td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td > <td valign=\"top\">U</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td>  <td valign=\"top\">0</td> <td valign=\"top\">2</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0-Trns</td> <td valign=\"top\">0</td>  <td valign=\"top\">NC</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1-Trns</td> <td valign=\"top\">0</td>  <td valign=\"top\">DataIn</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X-Trns</td> <td valign=\"top\">0</td>  <td valign=\"top\">X or U or NC</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 0 or NC</td> <td valign=\"top\">4</td> </tr>
</table>

<p><b>Truth Table for low active reset:</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Clock</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
      <td valign=\"top\">Map</td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td > <td valign=\"top\">U</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">2</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0-Trns</td> <td valign=\"top\">1</td>  <td valign=\"top\">NC</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1-Trns</td> <td valign=\"top\">1</td>  <td valign=\"top\">DataIn</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X-Trns</td> <td valign=\"top\">1</td>  <td valign=\"top\">X or U or NC</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 0 or NC</td> <td valign=\"top\">4</td> </tr>
</table>

<p>
<PRE>
  *  = don't care
  U  = L.'U'
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-'
  NC = no change

Clock transition definitions:
  1-Trns: 0 -> 1
  0-Trns: ~ -> 0 or 1 -> * or X -> X|U or U -> X|U
  X-Trns: 0 -> X|U or X|U -> 1
</PRE>
</p>
<br>
</html>", revisions = "<html>
<ul>
<PRE>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</PRE>
</ul>
</html>"));
        end DFFR;
        model DFFREG "Edge triggered register bank with high active reset"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          import S = Modelica.Electrical.Digital.Interfaces.Strength;
          import T = Modelica.Electrical.Digital.Tables;
          parameter Modelica.SIunits.Time tHL = 0 "High->Low delay";
          parameter Modelica.SIunits.Time tLH = 0 "Low->High delay";
          parameter D.Interfaces.Strength strength = S.'S_X01' "output strength";
          parameter Integer n(min = 1) = 1 "data width";
        protected 
          constant Integer ResetMap[9] = {1,4,3,2,4,4,3,2,4};
        public 
          Modelica.Electrical.Digital.Delay.InertialDelaySensitiveVector delay(tHL = tHL, tLH = tLH, n = n) annotation(Placement(transformation(extent = {{22,11},{79,69}})));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -50,70},{50, -50}}, lineColor = {127,0,127}, lineThickness = 0.5, fillPattern = FillPattern.Solid, fillColor = {255,255,255}),Line(points = {{ -86,40},{ -50,40}}, color = {127,0,127}, thickness = 1, smooth = Smooth.None),Line(points = {{ -84, -20},{ -50, -20}}, color = {127,0,127}, smooth = Smooth.None),Line(points = {{ -84, -60},{0, -60},{0, -50}}, color = {127,0,127}, smooth = Smooth.None),Line(points = {{50,40},{84,40}}, color = {127,0,127}, smooth = Smooth.None, thickness = 1),Line(points = {{ -50, -10},{ -36, -20},{ -50, -30}}, color = {127,0,127}, smooth = Smooth.None),Text(extent = {{ -32,70},{30,10}}, lineColor = {127,33,107}, textString = "DFFREG"),Text(extent = {{ -14, -34},{12, -46}}, lineColor = {127,33,107}, textString = "reset"),Text(extent = {{ -28, -14},{ -2, -26}}, lineColor = {127,33,107}, textString = "clock"),Text(extent = {{ -26,44},{26,2}}, lineColor = {127,33,107}, textString = "high active")}), Documentation(info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd<p>
<p><b>Truth Table</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Clock</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td > <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td>  <td valign=\"top\">0</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0-Trns</td> <td valign=\"top\">0</td>  <td valign=\"top\">NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1-Trns</td> <td valign=\"top\">0</td>  <td valign=\"top\">DataIn</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X-Trns</td> <td valign=\"top\">0</td>  <td valign=\"top\">X or U or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 0 or NC</td> </tr>
</table>

<p>
<PRE>
  *  = don't care
  U  = L.'U'
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-'
  NC = no change

Clock transition definitions:
  1-Trns: 0 -> 1
  0-Trns: ~ -> 0 or 1 -> * or X -> X|U or U -> X|U
  X-Trns: 0 -> X|U or X|U -> 1
</PRE>
</p>
<br>
</html>", revisions = "<html>
<ul>
<PRE>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</PRE>
</ul>
</html>"));
          D.Interfaces.DigitalInput reset annotation(Placement(transformation(extent = {{ -100, -68},{ -84, -52}}), iconTransformation(extent = {{ -100, -68},{ -84, -52}})));
          D.Interfaces.DigitalInput clock annotation(Placement(transformation(extent = {{ -100, -28},{ -84, -12}})));
          D.Interfaces.DigitalInput dataIn[n] annotation(Placement(transformation(extent = {{ -100,32},{ -84,48}})));
          D.Interfaces.DigitalOutput dataOut[n] annotation(Placement(transformation(extent = {{84,32},{100,48}}), iconTransformation(extent = {{84,32},{100,48}})));
          D.Registers.DFFR dFFR(n = n, ResetMap = ResetMap, strength = strength) annotation(Placement(transformation(extent = {{ -78, -23},{18,74}})));

        equation 
          connect(dataOut,dataOut) annotation(Line(points = {{92,40},{92,40}}, color = {127,0,127}, smooth = Smooth.None));
          connect(delay.y,dataOut) annotation(Line(points = {{75.01,40},{92,40}}, color = {127,0,127}, smooth = Smooth.None, thickness = 1));
          connect(dataIn,dFFR.dataIn) annotation(Line(points = {{ -92,40},{ -68.4,40},{ -68.4,40.05}}, color = {127,0,127}, thickness = 1, smooth = Smooth.None));
          connect(dFFR.dataOut,delay.x) annotation(Line(points = {{7.92,40.05},{25.99,39.855}}, color = {127,0,127}, thickness = 1, smooth = Smooth.None));
          connect(clock,dFFR.clock) annotation(Line(points = {{ -92, -20},{ -68.4, -20},{ -68.4,15.8}}, color = {127,0,127}, smooth = Smooth.None));
          connect(reset,dFFR.reset) annotation(Line(points = {{ -92, -60},{ -30, -60},{ -30, -18.15}}, color = {127,0,127}, smooth = Smooth.None));
        end DFFREG;
        model DFFREGL "Edge triggered register bank with low active reset"
          extends DFFREG(final ResetMap = {1,4,2,3,4,4,2,3,4});
          annotation(Documentation(info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd<p>
<p><b>Truth Table</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Clock</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td > <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0-Trns</td> <td valign=\"top\">1</td>  <td valign=\"top\">NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1-Trns</td> <td valign=\"top\">1</td>  <td valign=\"top\">DataIn</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X-Trns</td> <td valign=\"top\">1</td>  <td valign=\"top\">X or U or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 0 or NC</td> </tr>
</table>

<p>
<PRE>
  *  = don't care
  U  = L.'U'
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-'
  NC = no change

Clock transition definitions:
  1-Trns: 0 -> 1
  0-Trns: ~ -> 0 or 1 -> * or X -> X|U or U -> X|U
  X-Trns: 0 -> X|U or X|U -> 1
</PRE>
</p>
<br>
</html>", revisions = "<html>
<ul>
<PRE>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</PRE>
</ul>
</html>"), Icon(graphics = {Rectangle(extent = {{ -30,30},{28,16}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None),Text(extent = {{ -28,48},{24, -1}}, lineColor = {127,33,107}, textString = "low active")}));
        end DFFREGL;
        model DFFSR "Edge triggered register bank with set and reset"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          import S = Modelica.Electrical.Digital.Interfaces.Strength;
          import T = Modelica.Electrical.Digital.Tables;
          parameter Integer ResetSetMap[9,9] = [1,1,1,1,1,1,1,1,1;1,4,7,2,4,4,7,2,4;1,5,8,2,5,5,8,2,5;1,6,3,2,6,6,3,2,6;1,4,7,2,4,4,7,2,4;1,4,7,2,4,4,7,2,4;1,5,8,2,5,5,8,2,5;1,6,3,2,6,6,3,2,6;1,4,7,2,4,4,7,2,4] "function selection by [reset, set] reading";
          parameter D.Interfaces.Strength strength = S.'S_X01' "output strength";
          parameter Integer n(min = 1) = 1 "data width";
          D.Interfaces.DigitalInput set annotation(Placement(transformation(extent = {{ -10,100},{10,80}})));
          D.Interfaces.DigitalInput reset annotation(Placement(transformation(extent = {{ -10, -100},{10, -80}}), iconTransformation(extent = {{ -10, -100},{10, -80}})));
          D.Interfaces.DigitalInput clock annotation(Placement(transformation(extent = {{ -90, -20},{ -70,0}}), iconTransformation(extent = {{ -90, -30},{ -70, -10}})));
          D.Interfaces.DigitalInput dataIn[n] annotation(Placement(transformation(extent = {{ -90,20},{ -70,40}}), iconTransformation(extent = {{ -90,20},{ -70,40}})));
          D.Interfaces.DigitalOutput dataOut[n] annotation(Placement(transformation(extent = {{68,20},{88,40}}), iconTransformation(extent = {{69,20},{89,40}})));
        protected 
          Integer clock_flag(start = 0);
          Integer reset_set_flag(start = 1);
          D.Interfaces.Logic nextstate[n](start = fill(L.'U', n));
          D.Interfaces.Logic next_assign_val[n](start = fill(L.'U', n));
        algorithm 
          if change(clock) or change(reset) or change(set) then 
                    if change(clock) then 
                    if initial() then 
                    clock_flag:=T.ClockMap[L.'U',clock];

          else           clock_flag:=T.ClockMap[pre(clock),clock];

          end if;

          else 
          end if;
          reset_set_flag:=ResetSetMap[reset,set];
          for i in 1:n loop
                      if reset_set_flag == 1 then 
                        nextstate[i]:=L.'U';
            elseif reset_set_flag == 2 then
              nextstate[i]:=T.StrengthMap[L.'1',strength];

            elseif reset_set_flag == 3 then
              nextstate[i]:=T.StrengthMap[L.'0',strength];

            elseif reset_set_flag == 4 then
              if next_assign_val[i] == L.'U' and clock_flag <> 1 or dataIn[i] == L.'U' and clock_flag <> 0 then 
                            nextstate[i]:=L.'U';

              else               nextstate[i]:=T.StrengthMap[L.'X',strength];

              end if;

            elseif reset_set_flag == 5 then
              if next_assign_val[i] == T.StrengthMap[L.'1',strength] and (dataIn[i] == L.'1' or dataIn[i] == L.'H' or clock_flag == 0) then 
                            break;
              elseif (dataIn[i] == L.'1' or dataIn[i] == L.'H') and clock_flag == 1 then
                nextstate[i]:=T.StrengthMap[L.'1',strength];

              elseif next_assign_val[i] == L.'U' and clock_flag <> 1 or dataIn[i] == L.'U' and clock_flag <> 0 then
                nextstate[i]:=L.'U';


              else               nextstate[i]:=T.StrengthMap[L.'X',strength];

              end if;

            elseif reset_set_flag == 6 then
              nextstate[i]:=T.StrengthMap[L.'X',strength];

            elseif reset_set_flag == 7 then
              if next_assign_val[i] == T.StrengthMap[L.'0',strength] and (dataIn[i] == L.'0' or dataIn[i] == L.'L' or clock_flag == 0) then 
                            break;
              elseif (dataIn[i] == L.'0' or dataIn[i] == L.'L') and clock_flag == 1 then
                nextstate[i]:=T.StrengthMap[L.'0',strength];

              elseif next_assign_val[i] == L.'U' and clock_flag <> 1 or dataIn[i] == L.'U' and clock_flag <> 0 then
                nextstate[i]:=L.'U';


              else               nextstate[i]:=T.StrengthMap[L.'X',strength];

              end if;

            elseif reset_set_flag == 8 then
              if clock_flag == 0 then 
                            break;
              elseif clock_flag == 1 then
                nextstate[i]:=T.StrengthMap[dataIn[i],strength];


              else               if next_assign_val[i] == T.StrengthMap[dataIn[i],strength] or next_assign_val[i] == L.'U' then 
                            break;
              elseif dataIn[i] == L.'U' then
                nextstate[i]:=L.'U';


              else               nextstate[i]:=T.StrengthMap[L.'X',strength];

              end if;

              end if;


            else 
            end if;

          end for;

          else 
          end if;
          next_assign_val:=nextstate;
          dataOut:=nextstate;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -50,60},{50, -60}}, lineColor = {127,0,127}, lineThickness = 0.5, fillPattern = FillPattern.Solid, fillColor = {255,255,255}),Line(points = {{ -80,30},{ -50,30}}, color = {127,0,127}, smooth = Smooth.None, thickness = 1),Line(points = {{50,30},{80,30}}, color = {127,33,107}, thickness = 1, smooth = Smooth.None),Line(points = {{ -80, -20},{ -50, -20}}, color = {127,33,107}, smooth = Smooth.None),Line(points = {{0, -84},{0, -60}}, color = {127,33,107}, smooth = Smooth.None),Line(points = {{ -50, -8},{ -34, -20},{ -50, -32}}, color = {127,33,107}, smooth = Smooth.None),Text(extent = {{ -36,66},{34, -8}}, lineColor = {127,33,107}, textString = "DFFSR"),Text(extent = {{ -28, -14},{ -2, -26}}, lineColor = {127,33,107}, textString = "clock"),Text(extent = {{ -14, -44},{12, -56}}, lineColor = {127,33,107}, textString = "reset"),Line(points = {{0,80},{0,60}}, color = {127,0,127}, smooth = Smooth.None),Text(extent = {{ -13,56},{13,44}}, lineColor = {127,33,107}, textString = "set")}), Documentation(info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd<p>
<p><b>Truth Table for high active set and reset</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Clock</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>Set</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
      <td valign=\"top\">Map</td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">U</td> <td valign=\"top\">U</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td>  <td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">1</td> <td valign=\"top\">1</td> <td valign=\"top\">2</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td>  <td valign=\"top\">0</td> <td valign=\"top\">0</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td>  <td valign=\"top\">X</td> <td valign=\"top\">X</td> <td valign=\"top\">6</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U</td> <td valign=\"top\">4</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 1 or NC</td> <td valign=\"top\">5</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">0</td> <td valign=\"top\">X or U or 0 or NC</td> <td valign=\"top\">7</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X-Trns</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">X or U or NC</td> <td valign=\"top\">8</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1-Trns</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">DataIn</td> <td valign=\"top\">8</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0-Trns</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">NC</td> <td valign=\"top\">8</td> </tr>

</table>

<p><b>Truth Table for low active set and reset </b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Clock</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>Set</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
      <td valign=\"top\">Map</td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">U</td> <td valign=\"top\">U</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td>  <td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">0</td> <td valign=\"top\">1</td> <td valign=\"top\">2</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td>  <td valign=\"top\">1</td> <td valign=\"top\">0</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td>  <td valign=\"top\">X</td> <td valign=\"top\">X</td> <td valign=\"top\">6</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U</td> <td valign=\"top\">4</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 1 or NC</td> <td valign=\"top\">5</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">1</td> <td valign=\"top\">X or U or 0 or NC</td> <td valign=\"top\">7</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X-Trns</td> <td valign=\"top\">1</td>  <td valign=\"top\">1</td> <td valign=\"top\">X or U or NC</td> <td valign=\"top\">8</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1-Trns</td> <td valign=\"top\">1</td>  <td valign=\"top\">1</td> <td valign=\"top\">DataIn</td> <td valign=\"top\">8</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0-Trns</td> <td valign=\"top\">1</td>  <td valign=\"top\">1</td> <td valign=\"top\">NC</td> <td valign=\"top\">8</td> </tr>

</table>

<p>
<PRE>
  *  = don't care
  ~  = not equal
  U  = L.'U'
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-'
  NC = no change

Clock transition definitions:
  1-Trns: 0 -> 1
  0-Trns: ~ -> 0 or 1 -> * or X -> X|U or U -> X|U
  X-Trns: 0 -> X|U or X|U -> 1
</PRE>
</p>
<br>
</html>", revisions = "<html>
<ul>
<PRE>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</PRE>
</ul>
</html>"));
        end DFFSR;
        model DFFREGSRH "Edge triggered register bank with high active set and reset"
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          import S = Modelica.Electrical.Digital.Interfaces.Strength;
          import T = Modelica.Electrical.Digital.Tables;
          parameter Modelica.SIunits.Time tHL = 0 "High->Low delay";
          parameter Modelica.SIunits.Time tLH = 0 "Low->High delay";
          parameter D.Interfaces.Strength strength = S.'S_X01' "output strength";
          parameter Integer n(min = 1) = 1 "data width";
        protected 
          constant Integer ResetSetMap[9,9] = [1,1,1,1,1,1,1,1,1;1,4,7,2,4,4,7,2,4;1,5,8,2,5,5,8,2,5;1,6,3,2,6,6,3,2,6;1,4,7,2,4,4,7,2,4;1,4,7,2,4,4,7,2,4;1,5,8,2,5,5,8,2,5;1,6,3,2,6,6,3,2,6;1,4,7,2,4,4,7,2,4];
        protected 
          D.Delay.InertialDelaySensitiveVector delay(tHL = tHL, tLH = tLH, n = n) annotation(Placement(transformation(extent = {{23,12},{79,68}})));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -50,70},{52, -50}}, lineColor = {127,0,127}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -86,40},{ -50,40}}, color = {127,0,127}, thickness = 1, smooth = Smooth.None),Line(points = {{ -84, -20},{ -50, -20}}, color = {127,0,127}, smooth = Smooth.None),Line(points = {{52,40},{84,40}}, color = {127,0,127}, smooth = Smooth.None, thickness = 1),Line(points = {{ -50, -10},{ -36, -20},{ -50, -30}}, color = {127,0,127}, smooth = Smooth.None),Text(extent = {{ -44,94},{38, -14}}, lineColor = {127,33,107}, textString = "DFFREGSR"),Text(extent = {{ -14, -32},{12, -44}}, lineColor = {127,33,107}, textString = "reset"),Text(extent = {{ -33, -13},{ -7, -25}}, lineColor = {127,33,107}, textString = "clock"),Line(points = {{ -84,80},{0,80},{0,70}}, color = {127,0,127}, smooth = Smooth.None),Line(points = {{ -84, -60},{0, -60},{0, -50}}, color = {127,0,127}, smooth = Smooth.None),Text(extent = {{ -14,66},{12,54}}, lineColor = {127,33,107}, textString = "set"),Text(extent = {{ -32,38},{26,10}}, lineColor = {127,0,127}, textString = "high active")}), Documentation(info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd<p>
<p><b>Truth Table</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Clock</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>Set</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">U</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td>  <td valign=\"top\">*</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">1</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td>  <td valign=\"top\">0</td> <td valign=\"top\">0</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td>  <td valign=\"top\">X</td> <td valign=\"top\">X</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 1 or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">0</td> <td valign=\"top\">X or U or 0 or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X-Trns</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">X or U or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1-Trns</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">DataIn</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0-Trns</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">NC</td> </tr>

</table>

<p>
<PRE>
  *  = don't care
  ~  = not equal
  U  = L.'U'
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-'
  NC = no change

Clock transition definitions:
  1-Trns: 0 -> 1
  0-Trns: ~ -> 0 or 1 -> * or X -> X|U or U -> X|U
  X-Trns: 0 -> X|U or X|U -> 1
</PRE>
</p>
<br>
</html>", revisions = "<html>
<ul>
<PRE>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</PRE>
</ul>
</html>"));
          D.Registers.DFFSR dFFSR(strength = strength, n = n, ResetSetMap = ResetSetMap) annotation(Placement(transformation(extent = {{ -80, -25},{20,75}})));
        public 
          D.Interfaces.DigitalInput set annotation(Placement(transformation(extent = {{ -100,72},{ -84,88}})));
          D.Interfaces.DigitalInput reset annotation(Placement(transformation(extent = {{ -100, -68},{ -84, -52}})));
          D.Interfaces.DigitalInput clock annotation(Placement(transformation(extent = {{ -100, -28},{ -84, -12}})));
          D.Interfaces.DigitalInput dataIn[n] annotation(Placement(transformation(extent = {{ -100,32},{ -84,48}})));
          D.Interfaces.DigitalOutput dataOut[n] annotation(Placement(transformation(extent = {{84,32},{100,48}})));

        equation 
          connect(dFFSR.dataOut,delay.x) annotation(Line(points = {{9.5,40},{26.92,40},{26.92,39.86}}, color = {127,0,127}, thickness = 1, smooth = Smooth.None));
          connect(set,dFFSR.set) annotation(Line(points = {{ -92,80},{ -30,80},{ -30,70}}, color = {127,0,127}, smooth = Smooth.None));
          connect(reset,dFFSR.reset) annotation(Line(points = {{ -92, -60},{ -30, -60},{ -30, -20}}, color = {127,0,127}, smooth = Smooth.None));
          connect(clock,dFFSR.clock) annotation(Line(points = {{ -92, -20},{ -70, -20},{ -70,15}}, color = {127,0,127}, smooth = Smooth.None));
          connect(dataIn,dFFSR.dataIn) annotation(Line(points = {{ -92,40},{ -70,40}}, color = {127,0,127}, thickness = 1, smooth = Smooth.None));
          connect(delay.y,dataOut) annotation(Line(points = {{75.08,40},{92,40}}, color = {127,0,127}, thickness = 1, smooth = Smooth.None));
        end DFFREGSRH;
        model DFFREGSRL "Edge triggered register bank with low active set and reset"
          extends Digital.Registers.DFFREGSRH(final ResetSetMap = [1,1,1,1,1,1,1,1,1;1,4,2,7,4,4,2,7,4;1,6,2,3,6,6,2,3,6;1,5,2,8,5,5,2,8,5;1,4,2,7,4,4,2,7,4;1,4,2,7,4,4,2,7,4;1,6,2,3,6,6,2,3,6;1,5,2,8,5,5,2,8,5;1,4,2,7,4,4,2,7,4]);
          annotation(Icon(graphics = {Rectangle(extent = {{ -36,30},{28,16}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, pattern = LinePattern.None),Text(extent = {{ -30,38},{24,12}}, lineColor = {127,0,127}, textString = "low active")}), Documentation(info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd<p>

<p><b>Truth Table</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Clock</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>Set</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">U</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td>  <td valign=\"top\">*</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">0</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td>  <td valign=\"top\">1</td> <td valign=\"top\">0</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td>  <td valign=\"top\">X</td> <td valign=\"top\">X</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 1 or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">1</td> <td valign=\"top\">X or U or 0 or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X-Trns</td> <td valign=\"top\">1</td>  <td valign=\"top\">1</td> <td valign=\"top\">X or U or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1-Trns</td> <td valign=\"top\">1</td>  <td valign=\"top\">1</td> <td valign=\"top\">DataIn</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0-Trns</td> <td valign=\"top\">1</td>  <td valign=\"top\">1</td> <td valign=\"top\">NC</td> </tr>

</table>

<p>
<PRE>
  *  = don't care
  ~  = not equal
  U  = L.'U'
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-'
  NC = no change

Clock transition definitions:
  1-Trns: 0 -> 1
  0-Trns: ~ -> 0 or 1 -> * or X -> X|U or U -> X|U
  X-Trns: 0 -> X|U or X|U -> 1
</PRE>
</p>
<br>
</html>", revisions = "<html>
<ul>
<PRE>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</PRE>
</ul>
</html>"));
        end DFFREGSRL;
        model DLATR "Level sensitive register bank with reset"
          annotation(Documentation(info = "<html>

<p> Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd <p>
<p><b>Truth Table for high active reset:</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Enable</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
      <td valign=\"top\">Map</td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td > <td valign=\"top\">U</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td>  <td valign=\"top\">0</td> <td valign=\"top\">2</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">0</td>  <td valign=\"top\">NC</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">0</td>  <td valign=\"top\">DataIn</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">0</td>  <td valign=\"top\">X or U or NC</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">~1</td> <td valign=\"top\">U</td> <td valign=\"top\">4</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 0 or NC</td> <td valign=\"top\">4</td> </tr>
</table>

<p><b>Truth Table for low active reset:</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Enable</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
      <td valign=\"top\">Map</td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">U</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">0</td> <td valign=\"top\">2</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">1</td> <td valign=\"top\">NC</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">1</td> <td valign=\"top\">DataIn</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">1</td> <td valign=\"top\">X or U or NC</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">~0</td> <td valign=\"top\">U</td> <td valign=\"top\">4</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 0 or NC</td> <td valign=\"top\">4</td> </tr>
</table>

<p>
<PRE>
  *  = don't care
  ~  = not equal
  U  = L.'U'
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-'
  NC = no change
</PRE>
</p>
<br>
</HTML>
", revisions = "<html>
<ul>
<PRE>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</PRE>
</ul>
</html>"));
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          import S = Modelica.Electrical.Digital.Interfaces.Strength;
          import T = Modelica.Electrical.Digital.Tables;
          parameter Integer ResetMap[9] = {1,4,3,2,4,4,3,2,4} "function selection, defaults for high active reset";
          parameter D.Interfaces.Strength strength = S.'S_X01' "output strength";
          parameter Integer n(min = 1) = 1 "data width";
          D.Interfaces.DigitalInput reset annotation(Placement(transformation(extent = {{ -10, -100},{10, -80}}), iconTransformation(extent = {{ -10, -100},{10, -80}})));
          D.Interfaces.DigitalInput enable annotation(Placement(transformation(extent = {{ -90, -20},{ -70,0}}), iconTransformation(extent = {{ -90, -30},{ -70, -10}})));
          D.Interfaces.DigitalInput dataIn[n] annotation(Placement(transformation(extent = {{ -90,20},{ -70,40}}), iconTransformation(extent = {{ -90,20},{ -70,40}})));
          D.Interfaces.DigitalOutput dataOut[n] annotation(Placement(transformation(extent = {{68,20},{88,40}}), iconTransformation(extent = {{69,20},{89,40}})));
        protected 
          Integer enable_flag(start = 0);
          Integer reset_flag(start = 1);
          D.Interfaces.Logic nextstate[n](start = fill(L.'U', n));
          D.Interfaces.Logic next_assign_val[n](start = fill(L.'U', n));
        algorithm 
          if enable == L.'1' or enable == L.'H' then 
                    enable_flag:=1;
          elseif enable == L.'0' or enable == L.'L' then
            enable_flag:=0;

          elseif enable == L.'U' then
            enable_flag:=3;


          else           enable_flag:=2;

          end if;
          reset_flag:=ResetMap[reset];
          for i in 1:n loop
                      if reset_flag == 1 then 
                        nextstate[i]:=L.'U';
            elseif reset_flag == 2 then
              nextstate[i]:=T.StrengthMap[L.'0',strength];

            elseif reset_flag == 3 then
              if enable_flag == 0 then 
                            break;
              elseif enable_flag == 3 then
                nextstate[i]:=L.'U';

              elseif enable_flag == 1 then
                nextstate[i]:=T.StrengthMap[dataIn[i],strength];


              else               if next_assign_val[i] == T.StrengthMap[dataIn[i],strength] or next_assign_val[i] == L.'U' then 
                            break;
              elseif dataIn[i] == L.'U' then
                nextstate[i]:=L.'U';


              else               nextstate[i]:=T.StrengthMap[L.'X',strength];

              end if;

              end if;

            elseif reset_flag == 4 then
              if enable_flag == 3 or next_assign_val[i] == L.'U' and enable_flag <> 1 or dataIn[i] == L.'U' and enable_flag <> 0 then 
                            nextstate[i]:=L.'U';
              elseif next_assign_val[i] == T.StrengthMap[L.'0',strength] and (dataIn[i] == L.'0' or dataIn[i] == L.'L' or enable_flag == 0) then
                break;

              elseif (dataIn[i] == L.'0' or dataIn[i] == L.'L') and enable_flag == 1 then
                nextstate[i]:=T.StrengthMap[L.'0',strength];


              else               nextstate[i]:=T.StrengthMap[L.'X',strength];

              end if;


            else 
            end if;

          end for;
          next_assign_val:=nextstate;
          dataOut:=nextstate;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -50,60},{50, -60}}, lineColor = {127,0,127}, lineThickness = 0.5, fillPattern = FillPattern.Solid, fillColor = {255,255,255}),Line(points = {{ -80,30},{ -50,30}}, color = {127,0,127}, smooth = Smooth.None, thickness = 1),Line(points = {{50,30},{80,30}}, color = {127,33,107}, thickness = 1, smooth = Smooth.None),Line(points = {{ -80, -20},{ -50, -20}}, color = {127,33,107}, smooth = Smooth.None),Line(points = {{0, -84},{0, -60}}, color = {127,33,107}, smooth = Smooth.None),Line(points = {{ -50, -8},{ -34, -20},{ -50, -32}}, color = {127,33,107}, smooth = Smooth.None),Text(extent = {{ -30,58},{26,2}}, lineColor = {127,33,107}, textString = "DLATR"),Text(extent = {{ -29, -8},{6, -32}}, lineColor = {127,33,107}, textString = "enable"),Text(extent = {{ -14, -44},{12, -56}}, lineColor = {127,33,107}, textString = "reset")}));
        end DLATR;
        model DLATREG "Level sensitive register bank with reset active high"
          annotation(Documentation(info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd<p>
<p><b>Truth Table</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Enable</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td > <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td>  <td valign=\"top\">0</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">0</td>  <td valign=\"top\">NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">0</td>  <td valign=\"top\">DataIn</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">0</td>  <td valign=\"top\">X or U or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">~1</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 0 or NC</td> </tr>
</table>

<p>
<PRE>
  *  = don't care
  ~  = not equal
  U  = L.'U'
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-'
  NC = no change
</PRE>
</p>
<br>
</HTML>
", revisions = "<html>
<ul>
<PRE>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</PRE>
</ul>
</html>"));
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          import S = Modelica.Electrical.Digital.Interfaces.Strength;
          import T = Modelica.Electrical.Digital.Tables;
          parameter Modelica.SIunits.Time tHL = 0 "High->Low delay";
          parameter Modelica.SIunits.Time tLH = 0 "Low->High delay";
          parameter D.Interfaces.Strength strength = S.'S_X01' "output strength";
          parameter Integer n(min = 1) = 1 "data width";
        protected 
          constant Integer ResetMap[9] = {1,4,3,2,4,4,3,2,4};
        public 
          D.Delay.InertialDelaySensitiveVector delay(tHL = tHL, tLH = tLH, n = n) annotation(Placement(transformation(extent = {{20,12},{76,68}})));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -50,70},{52, -50}}, lineColor = {127,0,127}, lineThickness = 0.5, fillPattern = FillPattern.Solid, fillColor = {255,255,255}),Line(points = {{ -86,40},{ -50,40}}, color = {127,0,127}, thickness = 1, smooth = Smooth.None),Line(points = {{ -84, -20},{ -50, -20}}, color = {127,0,127}, smooth = Smooth.None),Line(points = {{52,40},{84,40}}, color = {127,0,127}, smooth = Smooth.None, thickness = 1),Line(points = {{ -50, -10},{ -36, -20},{ -50, -30}}, color = {127,0,127}, smooth = Smooth.None),Text(extent = {{ -32,82},{32, -2}}, lineColor = {127,0,127}, textString = "DLATREG"),Text(extent = {{ -14, -33},{12, -45}}, lineColor = {127,33,107}, textString = "reset"),Text(extent = {{ -33, -7},{ -1, -32}}, lineColor = {127,33,107}, textString = "enable"),Line(points = {{ -84, -60},{0, -60},{0, -50}}, color = {127,0,127}, smooth = Smooth.None),Text(extent = {{ -26,44},{26,2}}, lineColor = {127,33,107}, textString = "high active")}));
          D.Interfaces.DigitalInput reset annotation(Placement(transformation(extent = {{ -100, -68},{ -84, -52}})));
          D.Interfaces.DigitalInput enable annotation(Placement(transformation(extent = {{ -100, -28},{ -84, -12}})));
          D.Interfaces.DigitalInput dataIn[n] annotation(Placement(transformation(extent = {{ -100,32},{ -84,48}})));
          D.Interfaces.DigitalOutput dataOut[n] annotation(Placement(transformation(extent = {{84,32},{100,48}}), iconTransformation(extent = {{84,32},{100,48}})));
          D.Registers.DLATR dLATR(n = n, strength = strength, ResetMap = ResetMap) annotation(Placement(transformation(extent = {{ -78, -23},{18,74}})));

        equation 
          connect(dataOut,dataOut) annotation(Line(points = {{92,40},{92,40}}, color = {127,0,127}, smooth = Smooth.None));
          connect(delay.y,dataOut) annotation(Line(points = {{72.08,40},{92,40}}, color = {127,0,127}, thickness = 1, smooth = Smooth.None));
          connect(dLATR.dataOut,delay.x) annotation(Line(points = {{7.92,40.05},{15.96,39.86},{23.92,39.86}}, color = {127,0,127}, thickness = 1, smooth = Smooth.None));
          connect(dataIn,dLATR.dataIn) annotation(Line(points = {{ -92,40},{ -80.2,40},{ -80.2,40.05},{ -68.4,40.05}}, color = {127,0,127}, thickness = 1, smooth = Smooth.None));
          connect(enable,dLATR.enable) annotation(Line(points = {{ -92, -20},{ -68.4, -20},{ -68.4,15.8}}, color = {127,0,127}, smooth = Smooth.None));
          connect(reset,dLATR.reset) annotation(Line(points = {{ -92, -60},{ -30, -60},{ -30, -18.15}}, color = {127,0,127}, smooth = Smooth.None));
        end DLATREG;
        model DLATREGL "Level sensitive register bank with reset active low"
          annotation(Documentation(info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd<p>
<p><b>Truth Table</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Enable</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">0</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">1</td> <td valign=\"top\">NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">1</td> <td valign=\"top\">DataIn</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">1</td> <td valign=\"top\">X or U or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">~0</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 0 or NC</td> </tr>
</table>

<p>
<PRE>
  *  = don't care
  ~  = not equal
  U  = L.'U'
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-'
  NC = no change
</PRE>
</p>
<br>
</HTML>
", revisions = "<html>
<ul>
<PRE>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</PRE>
</ul>
</html>"));
          extends DLATREG(final ResetMap = {1,4,2,3,4,4,2,3,4});
          annotation(Icon(graphics = {Rectangle(extent = {{ -28,30},{30,18}}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -26,48},{26, -1}}, lineColor = {127,33,107}, textString = "low active")}));
        end DLATREGL;
        model DLATSR "Level sensitive register bank with set and reset"
          annotation(Documentation(info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd<p>
<p><b>Truth Table for high active set and reset</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Enable</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>Set</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
      <td valign=\"top\">Map</td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">U</td> <td valign=\"top\">U</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td>  <td valign=\"top\">~1</td> <td valign=\"top\">U</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">1</td> <td valign=\"top\">1</td> <td valign=\"top\">2</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td>  <td valign=\"top\">0</td> <td valign=\"top\">0</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td>  <td valign=\"top\">X</td> <td valign=\"top\">X</td> <td valign=\"top\">6</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">~1</td> <td valign=\"top\">~1</td> <td valign=\"top\">U</td> <td valign=\"top\">4,5,7,8</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">X</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U</td> <td valign=\"top\">4</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">0</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 1 or NC</td> <td valign=\"top\">5</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">X</td> <td valign=\"top\">0</td> <td valign=\"top\">X or U or 0 or NC</td> <td valign=\"top\">7</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">X or U or NC</td> <td valign=\"top\">8</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">DataIn</td> <td valign=\"top\">8</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">NC</td> <td valign=\"top\">8</td> </tr>

</table>

<p><b>Truth Table for low active set and reset </b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Enable</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>Set</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
      <td valign=\"top\">Map</td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">U</td> <td valign=\"top\">U</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td>  <td valign=\"top\">~0</td> <td valign=\"top\">U</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">0</td> <td valign=\"top\">1</td> <td valign=\"top\">2</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td>  <td valign=\"top\">1</td> <td valign=\"top\">0</td> <td valign=\"top\">3</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td>  <td valign=\"top\">X</td> <td valign=\"top\">X</td> <td valign=\"top\">6</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">~0</td> <td valign=\"top\">~0</td> <td valign=\"top\">U</td> <td valign=\"top\">4,5,7,8</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">X</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U</td> <td valign=\"top\">4</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">1</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 1 or NC</td> <td valign=\"top\">5</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">X</td> <td valign=\"top\">1</td> <td valign=\"top\">X or U or 0 or NC</td> <td valign=\"top\">7</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">1</td>  <td valign=\"top\">1</td> <td valign=\"top\">X or U or NC</td> <td valign=\"top\">8</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">1</td>  <td valign=\"top\">1</td> <td valign=\"top\">DataIn</td> <td valign=\"top\">8</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">1</td>  <td valign=\"top\">1</td> <td valign=\"top\">NC</td> <td valign=\"top\">8</td> </tr>

</table>

<p>
<PRE>
  *  = don't care
  ~  = not equal
  U  = L.'U'
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-'
  NC = no change
</PRE>
</p>
<br>
</HTML>
", revisions = "<html>
<ul>
<PRE>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</PRE>
</ul>
</html>"));
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          import S = Modelica.Electrical.Digital.Interfaces.Strength;
          import T = Modelica.Electrical.Digital.Tables;
          parameter Integer ResetSetMap[9,9] = [1,1,1,1,1,1,1,1,1;1,4,7,2,4,4,7,2,4;1,5,8,2,5,5,8,2,5;1,6,3,2,6,6,3,2,6;1,4,7,2,4,4,7,2,4;1,4,7,2,4,4,7,2,4;1,5,8,2,5,5,8,2,5;1,6,3,2,6,6,3,2,6;1,4,7,2,4,4,7,2,4] "function selection by [reset, set] reading";
          parameter D.Interfaces.Strength strength = S.'S_X01' "output strength";
          parameter Integer n(min = 1) = 1 "data width";
          D.Interfaces.DigitalInput set annotation(Placement(transformation(extent = {{ -10,100},{10,80}})));
          D.Interfaces.DigitalInput reset annotation(Placement(transformation(extent = {{ -10, -100},{10, -80}}), iconTransformation(extent = {{ -10, -100},{10, -80}})));
          D.Interfaces.DigitalInput enable annotation(Placement(transformation(extent = {{ -90, -20},{ -70,0}}), iconTransformation(extent = {{ -90, -30},{ -70, -10}})));
          D.Interfaces.DigitalInput dataIn[n] annotation(Placement(transformation(extent = {{ -90,20},{ -70,40}}), iconTransformation(extent = {{ -90,20},{ -70,40}})));
          D.Interfaces.DigitalOutput dataOut[n] annotation(Placement(transformation(extent = {{68,20},{88,40}}), iconTransformation(extent = {{69,20},{89,40}})));
        protected 
          Integer enable_flag(start = 0);
          Integer reset_set_flag(start = 1);
          D.Interfaces.Logic nextstate[n](start = fill(L.'U', n));
          D.Interfaces.Logic next_assign_val[n](start = fill(L.'U', n));
        algorithm 
          if enable == L.'1' or enable == L.'H' then 
                    enable_flag:=1;
          elseif enable == L.'0' or enable == L.'L' then
            enable_flag:=0;

          elseif enable == L.'U' then
            enable_flag:=3;


          else           enable_flag:=2;

          end if;
          reset_set_flag:=ResetSetMap[reset,set];
          for i in 1:n loop
                      if reset_set_flag == 1 then 
                        nextstate[i]:=L.'U';
            elseif reset_set_flag == 2 then
              nextstate[i]:=T.StrengthMap[L.'1',strength];

            elseif reset_set_flag == 3 then
              nextstate[i]:=T.StrengthMap[L.'0',strength];

            elseif reset_set_flag == 4 then
              if enable_flag == 3 or next_assign_val[i] == L.'U' and enable_flag <> 1 or dataIn[i] == L.'U' and enable_flag <> 0 then 
                            nextstate[i]:=L.'U';

              else               nextstate[i]:=T.StrengthMap[L.'X',strength];

              end if;

            elseif reset_set_flag == 5 then
              if enable_flag == 3 or next_assign_val[i] == L.'U' and enable_flag <> 1 or dataIn[i] == L.'U' and enable_flag <> 0 then 
                            nextstate[i]:=L.'U';
              elseif next_assign_val[i] == T.StrengthMap[L.'1',strength] and (dataIn[i] == L.'1' or dataIn[i] == L.'H' or enable_flag == 0) then
                break;

              elseif (dataIn[i] == L.'1' or dataIn[i] == L.'H') and enable_flag == 1 then
                nextstate[i]:=T.StrengthMap[L.'1',strength];


              else               nextstate[i]:=T.StrengthMap[L.'X',strength];

              end if;

            elseif reset_set_flag == 6 then
              nextstate[i]:=T.StrengthMap[L.'X',strength];

            elseif reset_set_flag == 7 then
              if enable_flag == 3 or next_assign_val[i] == L.'U' and enable_flag <> 1 or dataIn[i] == L.'U' and enable_flag <> 0 then 
                            nextstate[i]:=L.'U';
              elseif next_assign_val[i] == T.StrengthMap[L.'0',strength] and (dataIn[i] == L.'0' or dataIn[i] == L.'L' or enable_flag == 0) then
                break;

              elseif (dataIn[i] == L.'0' or dataIn[i] == L.'L') and enable_flag == 1 then
                nextstate[i]:=T.StrengthMap[L.'0',strength];


              else               nextstate[i]:=T.StrengthMap[L.'X',strength];

              end if;

            elseif reset_set_flag == 8 then
              if enable_flag == 0 then 
                            break;
              elseif enable_flag == 3 then
                nextstate[i]:=L.'U';

              elseif enable_flag == 1 then
                nextstate[i]:=T.StrengthMap[dataIn[i],strength];


              else               if next_assign_val[i] == T.StrengthMap[dataIn[i],strength] or next_assign_val[i] == L.'U' then 
                            break;
              elseif dataIn[i] == L.'U' then
                nextstate[i]:=L.'U';


              else               nextstate[i]:=T.StrengthMap[L.'X',strength];

              end if;

              end if;


            else 
            end if;

          end for;
          next_assign_val:=nextstate;
          dataOut:=nextstate;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -50,60},{50, -60}}, lineColor = {127,0,127}, lineThickness = 0.5, fillPattern = FillPattern.Solid, fillColor = {255,255,255}),Line(points = {{ -80,30},{ -50,30}}, color = {127,0,127}, smooth = Smooth.None, thickness = 1),Line(points = {{50,30},{80,30}}, color = {127,33,107}, thickness = 1, smooth = Smooth.None),Line(points = {{ -80, -20},{ -50, -20}}, color = {127,33,107}, smooth = Smooth.None),Line(points = {{0, -84},{0, -60}}, color = {127,33,107}, smooth = Smooth.None),Line(points = {{ -50, -8},{ -34, -20},{ -50, -32}}, color = {127,33,107}, smooth = Smooth.None),Text(extent = {{ -34,60},{32, -6}}, lineColor = {127,33,107}, textString = "DLATSR"),Text(extent = {{ -29, -8},{5, -30}}, lineColor = {127,33,107}, textString = "enable"),Text(extent = {{ -14, -44},{12, -56}}, lineColor = {127,33,107}, textString = "reset"),Line(points = {{0,80},{0,60}}, color = {127,0,127}, smooth = Smooth.None),Text(extent = {{ -13,56},{13,44}}, lineColor = {127,33,107}, textString = "set")}));
        end DLATSR;
        model DLATREGSRH "Level sensitive register bank with set and reset, active high"
          annotation(Documentation(info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd<p>
<p><b>Truth Table:</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Enable</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>Set</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">U</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td>  <td valign=\"top\">~1</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">1</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td>  <td valign=\"top\">0</td> <td valign=\"top\">0</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">1</td>  <td valign=\"top\">X</td> <td valign=\"top\">X</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">~1</td> <td valign=\"top\">~1</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">X</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">0</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 1 or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">X</td> <td valign=\"top\">0</td> <td valign=\"top\">X or U or 0 or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">X or U or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">DataIn</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">0</td>  <td valign=\"top\">0</td> <td valign=\"top\">NC</td> </tr>

</table>

<p>
<PRE>
  *  = don't care
  ~  = not equal
  U  = L.'U'
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-'
  NC = no change
</PRE>
</p>
<br>
</HTML>
", revisions = "<html>
<ul>
<PRE>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</PRE>
</ul>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
          import D = Modelica.Electrical.Digital;
          import L = Modelica.Electrical.Digital.Interfaces.Logic;
          import S = Modelica.Electrical.Digital.Interfaces.Strength;
          import T = Modelica.Electrical.Digital.Tables;
          parameter Modelica.SIunits.Time tHL = 0 "High->Low delay";
          parameter Modelica.SIunits.Time tLH = 0 "Low->High delay";
          parameter D.Interfaces.Strength strength = S.'S_X01' "output strength";
          parameter Integer n(min = 1) = 1 "data width";
        protected 
          constant Integer ResetSetMap[9,9] = [1,1,1,1,1,1,1,1,1;1,4,7,2,4,4,7,2,4;1,5,8,2,5,5,8,2,5;1,6,3,2,6,6,3,2,6;1,4,7,2,4,4,7,2,4;1,4,7,2,4,4,7,2,4;1,5,8,2,5,5,8,2,5;1,6,3,2,6,6,3,2,6;1,4,7,2,4,4,7,2,4];
        public 
          D.Delay.InertialDelaySensitiveVector delay(tHL = tHL, tLH = tLH, n = n) annotation(Placement(transformation(extent = {{20,12},{76,68}})));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -50,70},{52, -50}}, lineColor = {127,0,127}, lineThickness = 0.5, fillPattern = FillPattern.Solid, fillColor = {255,255,255}),Line(points = {{ -86,40},{ -50,40}}, color = {127,0,127}, thickness = 1, smooth = Smooth.None),Line(points = {{ -84, -20},{ -50, -20}}, color = {127,0,127}, smooth = Smooth.None),Line(points = {{52,40},{84,40}}, color = {127,0,127}, smooth = Smooth.None, thickness = 1),Line(points = {{ -50, -10},{ -36, -20},{ -50, -30}}, color = {127,0,127}, smooth = Smooth.None),Text(extent = {{ -46,96},{46, -18}}, lineColor = {127,0,127}, textString = "DLATREGSR"),Text(extent = {{ -14, -32},{14, -45}}, lineColor = {127,33,107}, textString = "reset"),Text(extent = {{ -33, -7},{ -1, -32}}, lineColor = {127,33,107}, textString = "enable"),Line(points = {{ -84,80},{0,80},{0,70}}, color = {127,0,127}, smooth = Smooth.None),Line(points = {{ -84, -60},{0, -60},{0, -50}}, color = {127,0,127}, smooth = Smooth.None),Text(extent = {{ -14,67},{12,53}}, lineColor = {127,33,107}, textString = "set"),Text(extent = {{ -28,38},{30,10}}, lineColor = {127,0,127}, textString = "high active")}));
          D.Interfaces.DigitalInput set annotation(Placement(transformation(extent = {{ -100,72},{ -84,88}})));
          D.Interfaces.DigitalInput reset annotation(Placement(transformation(extent = {{ -100, -68},{ -84, -52}})));
          D.Interfaces.DigitalInput enable annotation(Placement(transformation(extent = {{ -100, -28},{ -84, -12}})));
          D.Interfaces.DigitalInput dataIn[n] annotation(Placement(transformation(extent = {{ -100,32},{ -84,48}})));
          D.Interfaces.DigitalOutput dataOut[n] annotation(Placement(transformation(extent = {{84,32},{100,48}}), iconTransformation(extent = {{84,32},{100,48}})));
          D.Registers.DLATSR dLATSR(n = n, ResetSetMap = ResetSetMap, strength = strength) annotation(Placement(transformation(extent = {{ -78, -23},{18,74}})));

        equation 
          connect(dataOut,dataOut) annotation(Line(points = {{92,40},{92,40}}, color = {127,0,127}, smooth = Smooth.None));
          connect(delay.y,dataOut) annotation(Line(points = {{72.08,40},{92,40}}, color = {127,0,127}, thickness = 1, smooth = Smooth.None));
          connect(set,dLATSR.set) annotation(Line(points = {{ -92,80},{ -30,80},{ -30,69.15}}, color = {127,0,127}, smooth = Smooth.None));
          connect(reset,dLATSR.reset) annotation(Line(points = {{ -92, -60},{ -30, -60},{ -30, -18.15}}, color = {127,0,127}, smooth = Smooth.None));
          connect(enable,dLATSR.enable) annotation(Line(points = {{ -92, -20},{ -68.4, -20},{ -68.4,15.8}}, color = {127,0,127}, smooth = Smooth.None));
          connect(dataIn,dLATSR.dataIn) annotation(Line(points = {{ -92,40},{ -80.2,40},{ -80.2,40.05},{ -68.4,40.05}}, color = {127,0,127}, thickness = 1, smooth = Smooth.None));
          connect(dLATSR.dataOut,delay.x) annotation(Line(points = {{7.92,40.05},{23.92,40.05},{23.92,39.86}}, color = {127,0,127}, thickness = 1, smooth = Smooth.None));
        end DLATREGSRH;
        model DLATREGSRL "Level sensitive register bank with set and reset, active low"
          annotation(Documentation(info = "<html>
<p>Description in VHDL is given by http://www.cs.sfu.ca/~ggbaker/reference/std_logic/src/std_logic_entities.vhd<p>
<p><b>Truth Table</b></p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>DataIn</b></td>
      <td valign=\"top\"><b>Enable</b></td>
      <td valign=\"top\"><b>Reset</b></td>
      <td valign=\"top\"><b>Set</b></td>
      <td valign=\"top\"><b>DataOut</b></td>
  </tr>

  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">U</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">U</td>  <td valign=\"top\">~0</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">*</td>  <td valign=\"top\">0</td> <td valign=\"top\">1</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td>  <td valign=\"top\">1</td> <td valign=\"top\">0</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">*</td> <td valign=\"top\">0</td>  <td valign=\"top\">X</td> <td valign=\"top\">X</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">U</td> <td valign=\"top\">~0</td> <td valign=\"top\">~0</td> <td valign=\"top\">U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">X</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">1</td> <td valign=\"top\">X</td> <td valign=\"top\">X or U or 1 or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">~U</td> <td valign=\"top\">X</td> <td valign=\"top\">1</td> <td valign=\"top\">X or U or 0 or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">X</td> <td valign=\"top\">1</td>  <td valign=\"top\">1</td> <td valign=\"top\">X or U or NC</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">1</td> <td valign=\"top\">1</td>  <td valign=\"top\">1</td> <td valign=\"top\">DataIn</td> </tr>
  <tr><td valign=\"top\">*</td> <td valign=\"top\">0</td> <td valign=\"top\">1</td>  <td valign=\"top\">1</td> <td valign=\"top\">NC</td> </tr>

</table>

<p>
<PRE>
  *  = don't care
  ~  = not equal
  U  = L.'U'
  0  = L.'0' or L.'L'
  1  = L.'1' or L.'H'
  X  = L.'X' or L.'W' or L.'Z' or L.'-'
  NC = no change
</PRE>
</p>
<br>
</HTML>
", revisions = "<html>
<ul>
<PRE>
<li><i>September 11, 2009 </i> created by Ulrich Donath<br>
       </li>
</PRE>
</ul>
</html>"));
          extends Digital.Registers.DLATREGSRH(final ResetSetMap = [1,1,1,1,1,1,1,1,1;1,4,2,7,4,4,2,7,4;1,6,2,3,5,5,2,3,6;1,5,2,8,6,6,2,8,5;1,4,2,7,4,4,2,7,4;1,4,2,7,4,4,2,7,4;1,6,2,3,5,5,2,3,6;1,5,2,8,6,6,2,8,5;1,4,2,7,4,4,2,7,4]);
          annotation(Icon(graphics = {Rectangle(extent = {{ -40,30},{40,16}}, pattern = LinePattern.None, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -30,30},{30,16}}, lineColor = {127,0,127}, textString = "low active")}));
        end DLATREGSRL;
        annotation(Documentation(info = "<html>
<p>Registers is a collection of flipflops and latches. In the opposite to the Examples.Utilities models the Register models are a series of assignments in the algorithm part of the model. The model text is taken nearly identical from the standard logic text.</p>
</html>"));
      end Registers;
    end Digital;
  end Electrical;
  package Blocks "Library of basic input/output control blocks (continuous, discrete, logical, table blocks)"
    import SI = Modelica.SIunits;
    extends Modelica.Icons.Library2;
    annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -32, -6},{16, -35}}, lineColor = {0,0,0}),Rectangle(extent = {{ -32, -56},{16, -85}}, lineColor = {0,0,0}),Line(points = {{16, -20},{49, -20},{49, -71},{16, -71}}, color = {0,0,0}),Line(points = {{ -32, -72},{ -64, -72},{ -64, -21},{ -32, -21}}, color = {0,0,0}),Polygon(points = {{16, -71},{29, -67},{29, -74},{16, -71}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{ -32, -21},{ -46, -17},{ -46, -25},{ -32, -21}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>
This library contains input/output blocks to build up block diagrams.
</p>

<dl>
<dt><b>Main Author:</b>
<dd><a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a><br>
    Deutsches Zentrum f&uuml;r Luft und Raumfahrt e. V. (DLR)<br>
    Oberpfaffenhofen<br>
    Postfach 1116<br>
    D-82230 Wessling<br>
    email: <A HREF=\"mailto:Martin.Otter@dlr.de\">Martin.Otter@dlr.de</A><br>
</dl>
<p>
Copyright &copy; 1998-2009, Modelica Association and DLR.
</p>
<p>
<i>This Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p>
</HTML>
", revisions = "<html>
<ul>
<li><i>June 23, 2004</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Introduced new block connectors and adapated all blocks to the new connectors.
       Included subpackages Continuous, Discrete, Logical, Nonlinear from
       package ModelicaAdditions.Blocks.
       Included subpackage ModelicaAdditions.Table in Modelica.Blocks.Sources
       and in the new package Modelica.Blocks.Tables.
       Added new blocks to Blocks.Sources and Blocks.Logical.
       </li>
<li><i>October 21, 2002</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>
       and <a href=\"http://www.robotic.dlr.de/Christian.Schweiger/\">Christian Schweiger</a>:<br>
       New subpackage Examples, additional components.
       </li>
<li><i>June 20, 2000</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a> and
       Michael Tiller:<br>
       Introduced a replaceable signal type into
       Blocks.Interfaces.RealInput/RealOutput:
<pre>
   replaceable type SignalType = Real
</pre>
       in order that the type of the signal of an input/output block
       can be changed to a physical type, for example:
<pre>
   Sine sin1(outPort(redeclare type SignalType=Modelica.SIunits.Torque))
</pre>
      </li>
<li><i>Sept. 18, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Renamed to Blocks. New subpackages Math, Nonlinear.
       Additional components in subpackages Interfaces, Continuous
       and Sources. </li>
<li><i>June 30, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized a first version, based on an existing Dymola library
       of Dieter Moormann and Hilding Elmqvist.</li>
</ul>
</html>"));
    package Examples "Library of examples to demonstrate the usage of package Blocks"
      extends Icons.Library;
      model PID_Controller "Demonstrates the usage of a Continuous.LimPID controller"
        extends Modelica.Icons.Example;
        parameter Modelica.SIunits.Angle driveAngle = 1.57 "Reference distance to move";
        Modelica.Blocks.Continuous.LimPID PI(k = 100, Ti = 0.1, yMax = 12, Ni = 0.1, initType = Modelica.Blocks.Types.Init.SteadyState, limitsAtInit = false, controllerType = Modelica.Blocks.Types.SimpleController.PI, Td = 0.1) annotation(Placement(transformation(extent = {{ -56, -20},{ -36,0}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia1(a(fixed = true), phi(fixed = true, start = 0), J = 1) annotation(Placement(transformation(extent = {{2, -20},{22,0}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -99,48},{ -32,8}}, lineColor = {255,0,0}),Text(extent = {{ -98,59},{ -31,51}}, lineColor = {255,0,0}, textString = "reference speed generation"),Text(extent = {{ -98, -46},{ -60, -52}}, lineColor = {255,0,0}, textString = "PI controller"),Line(points = {{ -76, -44},{ -57, -23}}, color = {255,0,0}, arrow = {Arrow.None,Arrow.Filled}),Rectangle(extent = {{ -25,6},{99, -50}}, lineColor = {255,0,0}),Text(extent = {{4,14},{71,7}}, lineColor = {255,0,0}, textString = "plant (simple drive train)")}), experiment(StopTime = 4), Documentation(info = "<html>

<p>
This is a simple drive train controlled by a PID controller:
</p>

<ul>
<li> The two blocks \"kinematic_PTP\" and \"integrator\" are used to generate
     the reference speed (= constant acceleration phase, constant speed phase,
     constant deceleration phase until inertia is at rest). To check
     whether the system starts in steady state, the reference speed is
     zero until time = 0.5 s and then follows the sketched trajectory.</li>

<li> The block \"PI\" is an instance of \"Blocks.Continuous.LimPID\" which is
     a PID controller where several practical important aspects, such as
     anti-windup-compensation has been added. In this case, the control block
     is used as PI controller.</li>

<li> The output of the controller is a torque that drives a motor inertia
     \"inertia1\". Via a compliant spring/damper component, the load
     inertia \"inertia2\" is attached. A constant external torque of 10 Nm
     is acting on the load inertia.</li>
</ul>

<p>
The PI controller settings included \"limitAtInit=false\", in order that
the controller output limits of 12 Nm are removed from the initialization
problem.
</p>

<p>
The PI controller is initialized in steady state (initType=SteadyState)
and the drive shall also be initialized in steady state.
However, it is not possible to initialize \"inertia1\" in SteadyState, because
\"der(inertia1.phi)=inertia1.w=0\" is an input to the PI controller that
defines that the derivative of the integrator state is zero (= the same
condition that was already defined by option SteadyState of the PI controller).
Furthermore, one initial condition is missing, because the absolute position
of inertia1 or inertia2 is not defined. The solution shown in this examples is
to initialize the angle and the angular acceleration of \"inertia1\".
</p>

<p>
In the following figure, results of a typical simulation are shown:
</p>

<p>
<img src=\"../Images/Blocks/PID_controller.png\"><br>
<img src=\"../Images/Blocks/PID_controller2.png\">
</p>

<p>
In the upper figure the reference speed (= integrator.y) and
the actual speed (= inertia1.w) are shown. As can be seen,
the system initializes in steady state, since no transients
are present. The inertia follows the reference speed quite good
until the end of the constant speed phase. Then there is a deviation.
In the lower figure the reason can be seen: The output of the
controller (PI.y) is in its limits. The anti-windup compensation
works reasonably, since the input to the limiter (PI.limiter.u)
is forced back to its limit after a transient phase.
</p>

</html>"));
        Modelica.Mechanics.Rotational.Sources.Torque torque annotation(Placement(transformation(extent = {{ -25, -20},{ -5,0}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.SpringDamper spring(c = 10000.0, d = 100, stateSelect = StateSelect.prefer, w_rel(fixed = true)) annotation(Placement(transformation(extent = {{32, -20},{52,0}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Components.Inertia inertia2(J = 2) annotation(Placement(transformation(extent = {{60, -20},{80,0}}, rotation = 0)));
        Modelica.Blocks.Sources.KinematicPTP kinematicPTP(startTime = 0.5, deltaq = {driveAngle}, qd_max = {1}, qdd_max = {1}) annotation(Placement(transformation(extent = {{ -92,20},{ -72,40}}, rotation = 0)));
        Modelica.Blocks.Continuous.Integrator integrator(initType = Modelica.Blocks.Types.Init.InitialState) annotation(Placement(transformation(extent = {{ -63,20},{ -43,40}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Sensors.SpeedSensor speedSensor annotation(Placement(transformation(extent = {{22, -50},{2, -30}}, rotation = 0)));
        Modelica.Mechanics.Rotational.Sources.ConstantTorque loadTorque(tau_constant = 10, useSupport = false) annotation(Placement(transformation(extent = {{98, -15},{88, -5}}, rotation = 0)));

      initial equation 
        der(spring.w_rel) = 0;

      equation 
        connect(spring.flange_b,inertia2.flange_a) annotation(Line(points = {{52, -10},{60, -10}}, color = {0,0,0}));
        connect(inertia1.flange_b,spring.flange_a) annotation(Line(points = {{22, -10},{32, -10}}, color = {0,0,0}));
        connect(torque.flange,inertia1.flange_a) annotation(Line(points = {{ -5, -10},{2, -10}}, color = {0,0,0}));
        connect(kinematicPTP.y[1],integrator.u) annotation(Line(points = {{ -71,30},{ -65,30}}, color = {0,0,127}));
        connect(speedSensor.flange,inertia1.flange_b) annotation(Line(points = {{22, -40},{22, -10}}, color = {0,0,0}));
        connect(loadTorque.flange,inertia2.flange_b) annotation(Line(points = {{88, -10},{80, -10}}, color = {0,0,0}));
        connect(PI.y,torque.tau) annotation(Line(points = {{ -35, -10},{ -27, -10}}, color = {0,0,127}));
        connect(speedSensor.w,PI.u_m) annotation(Line(points = {{1, -40},{ -46, -40},{ -46, -22}}, color = {0,0,127}));
        connect(integrator.y,PI.u_s) annotation(Line(points = {{ -42,30},{ -37,30},{ -37,11},{ -67,11},{ -67, -10},{ -58, -10}}, color = {0,0,127}));
      end PID_Controller;
      model InverseModel "Demonstrates the construction of an inverse model"
        extends Modelica.Icons.Example;
        Continuous.FirstOrder firstOrder1(k = 1, T = 0.3, initType = Modelica.Blocks.Types.Init.SteadyState) annotation(Placement(transformation(extent = {{20,20},{0,40}})));
        Sources.Sine sine(freqHz = 2, offset = 1, startTime = 0.2) annotation(Placement(transformation(extent = {{ -80,20},{ -60,40}})));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
This example demonstrates how to construct an inverse model in Modelica
(for more details see <a href=\"http://www.modelica.org/events/Conference2005/online_proceedings/Session3/Session3c3.pdf\">Looye, Th&uuml;mmel, Kurze, Otter, Bals: Nonlinear Inverse Models for Control</a>).
</p>

<p>
For a linear, single input, single output system
</p>

<pre>
   y = n(s)/d(s) * u   // plant model
</pre>

<p>
the inverse model is derived by simply exchanging the numerator and
the denominator polynomial:
</p>

<pre>
   u = d(s)/n(s) * y   // inverse plant model
</pre>

<p>
If the denominator polynomial d(s) has a higher degree as the
numerator polynomial n(s) (which is usually the case for plant models),
then the inverse model is no longer proper, i.e., it is not causal.
To avoid this, an approximate inverse model is constructed by adding
a sufficient number of poles to the denominator of the inverse model.
This can be interpreted as filtering the desired output signal y:
</p>

<pre>
   u = d(s)/(n(s)*f(s)) * y  // inverse plant model with filtered y
</pre>

<p>
With Modelica it is in principal possible to construct inverse models not only
for linear but also for non-linear models and in particular for every
Modelica model. The basic construction mechanism is explained at hand
of this example:
</p>

<p>
<img src=\"../Images/Blocks/InverseModelSchematic.png\">
</p>

<p>
Here the first order block \"firstOrder1\" shall be inverted. This is performed
by connecting its inputs and outputs with an instance of block
Modelica.Blocks.Math.<b>InverseBlockConstraints</b>. By this connection,
the inputs and outputs are exchanged. The goal is to compute the input of the
\"firstOrder1\" block so that its output follows a given sine signal.
For this, the sine signal \"sin\" is first filtered with a \"CriticalDamping\"
filter of order 1 and then the output of this filter is connected to the output
of the \"firstOrder1\" block (via the InverseBlockConstraints block, since
2 outputs cannot be connected directly together in a block diagram).
</p>

<p>
In order to check the inversion, the computed input of \"firstOrder1\" is used
as input in an identical block \"firstOrder2\". The output of \"firstOrder2\" should
be the given \"sine\" function. The difference is constructed with the \"feedback\"
block. Since the \"sine\" function is filtered, one cannot expect that this difference
is zero. The higher the cut-off frequency of the filter, the closer is the
agreement. A typical simulation result is shown in the next figure:
</p>

<p>
<img src=\"../Images/Blocks/InverseModel.png\">
</p>

</html>"));
        Math.InverseBlockConstraints inverseBlockConstraints annotation(Placement(transformation(extent = {{ -10,20},{30,40}})));
        Continuous.FirstOrder firstOrder2(k = 1, T = 0.3, initType = Modelica.Blocks.Types.Init.SteadyState) annotation(Placement(transformation(extent = {{20, -20},{0,0}})));
        Math.Feedback feedback annotation(Placement(transformation(extent = {{ -40,0},{ -60, -20}})));
        Continuous.CriticalDamping criticalDamping(n = 1, f = 50 * sine.freqHz) annotation(Placement(transformation(extent = {{ -40,20},{ -20,40}})));

      equation 
        connect(firstOrder1.y,inverseBlockConstraints.u2) annotation(Line(points = {{ -1,30},{ -6,30}}, color = {0,0,127}, smooth = Smooth.None));
        connect(inverseBlockConstraints.y2,firstOrder1.u) annotation(Line(points = {{27,30},{22,30}}, color = {0,0,127}, smooth = Smooth.None));
        connect(firstOrder2.y,feedback.u1) annotation(Line(points = {{ -1, -10},{ -42, -10}}, color = {0,0,127}, smooth = Smooth.None));
        connect(sine.y,criticalDamping.u) annotation(Line(points = {{ -59,30},{ -42,30}}, color = {0,0,127}, smooth = Smooth.None));
        connect(criticalDamping.y,inverseBlockConstraints.u1) annotation(Line(points = {{ -19,30},{ -12,30}}, color = {0,0,127}, smooth = Smooth.None));
        connect(sine.y,feedback.u2) annotation(Line(points = {{ -59,30},{ -50,30},{ -50, -2}}, color = {0,0,127}, smooth = Smooth.None));
        connect(inverseBlockConstraints.y1,firstOrder2.u) annotation(Line(points = {{31,30},{40,30},{40, -10},{22, -10}}, color = {0,0,127}, smooth = Smooth.None));
      end InverseModel;
      model ShowLogicalSources "Demonstrates the usage of logical sources together with their diagram animation"
        extends Modelica.Icons.Example;
        Sources.BooleanTable table(table = {2,4,6,8}) annotation(Placement(transformation(extent = {{ -60, -100},{ -40, -80}}, rotation = 0)));
        Sources.BooleanConstant const annotation(Placement(transformation(extent = {{ -60,60},{ -40,80}}, rotation = 0)));
        Sources.BooleanStep step(startTime = 4) annotation(Placement(transformation(extent = {{ -60,20},{ -40,40}}, rotation = 0)));
        Sources.BooleanPulse pulse(period = 1.5) annotation(Placement(transformation(extent = {{ -60, -20},{ -40,0}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 10), Documentation(info = "<html>
<p>
This simple example demonstrates the logical sources in
<a href=\"Modelica://Modelica.Blocks.Sources\">Modelica.Blocks.Sources</a> and demonstrate
their diagram animation (see \"small circle\" close to the output connector).
The \"booleanExpression\" source shows how a logical expression can be defined
in its parameter menu refering to variables available on this level of the
model.
</p>

</html>"));
        Sources.SampleTrigger sample(period = 0.5) annotation(Placement(transformation(extent = {{ -60, -60},{ -40, -40}}, rotation = 0)));
        Sources.BooleanExpression booleanExpression(y = pulse.y and step.y) annotation(Placement(transformation(extent = {{20,20},{80,40}}, rotation = 0)));
      end ShowLogicalSources;
      model LogicalNetwork1 "Demonstrates the usage of logical blocks"
        extends Modelica.Icons.Example;
        Sources.BooleanTable table2(table = {1,3,5,7}) annotation(Placement(transformation(extent = {{ -80, -20},{ -60,0}}, rotation = 0)));
        Sources.BooleanTable table1(table = {2,4,6,8}) annotation(Placement(transformation(extent = {{ -80,20},{ -60,40}}, rotation = 0)));
        Logical.Not Not1 annotation(Placement(transformation(extent = {{ -40, -20},{ -20,0}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 10), Documentation(info = "<html>
<p>
This example demonstrates a network of logical blocks. Note, that
the Boolean values of the input and output signals are visualized
in the diagram animation, by the small \"circles\" close to the connectors.
If a \"circle\" is \"white\", the signal is <b>false</b>. It a
\"circle\" is \"green\", the signal is <b>true</b>.
</p>
</html>"));
        Logical.And And1 annotation(Placement(transformation(extent = {{0, -20},{20,0}}, rotation = 0)));
        Logical.Or Or1 annotation(Placement(transformation(extent = {{40,20},{60,40}}, rotation = 0)));
        Logical.Pre Pre1 annotation(Placement(transformation(extent = {{ -40, -60},{ -20, -40}}, rotation = 0)));

      equation 
        connect(table2.y,Not1.u) annotation(Line(points = {{ -59, -10},{ -42, -10}}, color = {255,0,255}));
        connect(And1.y,Or1.u2) annotation(Line(points = {{21, -10},{28, -10},{28,22},{38,22}}, color = {255,0,255}));
        connect(table1.y,Or1.u1) annotation(Line(points = {{ -59,30},{38,30}}, color = {255,0,255}));
        connect(Not1.y,And1.u1) annotation(Line(points = {{ -19, -10},{ -2, -10}}, color = {255,0,255}));
        connect(Pre1.y,And1.u2) annotation(Line(points = {{ -19, -50},{ -10, -50},{ -10, -18},{ -2, -18}}, color = {255,0,255}));
        connect(Or1.y,Pre1.u) annotation(Line(points = {{61,30},{68,30},{68, -70},{ -60, -70},{ -60, -50},{ -42, -50}}, color = {255,0,255}));
      end LogicalNetwork1;
      annotation(Documentation(info = "<html>
<p>
This package contains example models to demonstrate the
usage of package blocks.
</p>
</HTML>
"));
      model BusUsage "Demonstrates the usage of a signal bus"
        extends Modelica.Icons.Example;
        annotation(Documentation(info = "<html>
<p><b>Signal bus concept</b></p>
<p>
In technical systems, such as vehicles, robots or satellites, many signals
are exchanged between components. In a simulation system, these signals
are  usually modelled by signal connections of input/output blocks.
Unfortunately, the signal connection structure may become very complicated,
especially for hierarchical models.
</p>

<p>
The same is also true for real technical systems. To reduce complexity
and get higher flexibility, many technical systems use data buses to
exchange data between components. For the same reasons, it is often better
to use a \"signal bus\" concept also in a Modelica model. This is demonstrated
at hand of this model (Modelica.Blocks.Examples.BusUsage):
</p>

<p align=\"center\">
<img src=\"../Images/Blocks/BusUsage.png\">
</p>

<ul>
<li> Connector instance \"controlBus\" is a hierarchical connector that is
     used to exchange signals between different components. It is
     defined as \"expandable connector\" in order that <b>no</b> central definition
     of the connector is needed but is automatically constructed by the
     signals connected to it (see also Modelica specification 2.2.1).</li>
<li> Input/output signals can be directly connected to the \"controlBus\".</li>
<li> A component, such as \"part\", can be directly connected to the \"controlBus\",
     provided it has also a bus connector, or the \"part\" connector is a
     sub-connector contained in the \"controlBus\". </li>
</ul>

<p>
The control and sub-control bus icons are provided within Modelica.Icons.
In <a href=\"Modelica://Modelica.Blocks.Examples.BusUsage_Utilities.Interfaces\">Modelica.Blocks.Examples.BusUsage_Utilities.Interfaces</a>
the buses for this example are defined. Both the \"ControlBus\" and the \"SubControlBus\" are
<b>expandable</b> connectors that do not define any variable. For example,
<a href=\"Modelica://Modelica.Blocks.Examples.BusUsage_Utilities.Interfaces.ControlBus#text\">Interfaces.ControlBus</a>
is defined as:
</p>
<pre>  <b>expandable connector</b> ControlBus
      <b>extends</b> Modelica.Icons.ControlBus;
      <b>annotation</b> (Icon(Rectangle(extent=[-20, 2; 22, -2],
                       style(rgbcolor={255,204,51}, thickness=0.5))));
  <b>end</b> ControlBus;
</pre>
<p>
Note, the \"annotation\" in the connector is important since the color
and thickness of a connector line are taken from the first
line element in the icon annotation of a connector class. Above, a small rectangle in the
color of the bus is defined (and therefore this rectangle is not
visible). As a result, when connecting from an instance of this
connector to another connector instance, the connecting line has
the color of the \"ControlBus\" with double width (due to \"thickness=0.5\").
</p>

<p>
An <b>expandable</b> connector is a connector where the content of the connector
is constructed by the variables connected to instances of this connector.
For example, if \"sine.y\" is connected to the \"controlBus\", the following
menu pops-up in Dymola:
</p>

<p align=\"center\">
<img src=\"../Images/Blocks/BusUsage2.png\">
</p>

<p>
The \"Add variable/New name\" field allows the user to define the name of the signal on
the \"controlBus\". When typing \"realSignal1\" as \"New name\", a connection of the form:
</p>

<pre>     <b>connect</b>(sine.y, controlBus.realSignal1)
</pre>

<p>
is generated and the \"controlBus\" contains the new signal \"realSignal1\". Modelica tools
may give more support in order to list potential signals for a connection.
For example, in Dymola all variables are listed in the menu that are contained in
connectors which are derived by inheritance from \"controlBus\". Therefore, in
<a href=\"Modelica://Modelica.Blocks.Examples.BusUsage_Utilities.Interfaces.InternalConnectors\">BusUsage_Utilities.Interfaces.InternalConnectors</a>
the expected implementation of the \"ControlBus\" and of the \"SubControlBus\" are given.
For example \"Internal.ControlBus\" is defined as:
</p>

<pre>  <b>expandable connector</b> StandardControlBus
    <b>extends</b> BusUsage_Utilities.Interfaces.ControlBus;

    <b>import</b> SI = Modelica.SIunits;
    SI.AngularVelocity    realSignal1   \"First Real signal\";
    SI.Velocity           realSignal2   \"Second Real signal\";
    Integer               integerSignal \"Integer signal\";
    Boolean               booleanSignal \"Boolean signal\";
    StandardSubControlBus subControlBus \"Combined signal\";
  <b>end</b> StandardControlBus;
</pre>

<p>
Consequently, when connecting now from \"sine.y\" to \"controlBus\", the menu
looks differently:
</p>
<p align=\"center\">
<img src=\"../Images/Blocks/BusUsage3.png\">
</p>
<p>
Note, even if the signals from \"Internal.StandardControlBus\" are listed, these are
just potential signals. The user might still add different signal names.
</p>

</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 2));
      public 
        Modelica.Blocks.Sources.IntegerStep integerStep(height = 1, offset = 2, startTime = 0.5) annotation(Placement(transformation(extent = {{ -60, -40},{ -40, -20}}, rotation = 0)));
        Modelica.Blocks.Sources.BooleanStep booleanStep(startTime = 0.5) annotation(Placement(transformation(extent = {{ -58,0},{ -38,20}}, rotation = 0)));
        Modelica.Blocks.Sources.Sine sine(freqHz = 1) annotation(Placement(transformation(extent = {{ -60,40},{ -40,60}}, rotation = 0)));
        Modelica.Blocks.Examples.BusUsage_Utilities.Part part annotation(Placement(transformation(extent = {{ -60, -80},{ -40, -60}}, rotation = 0)));
        Modelica.Blocks.Math.Gain gain(k = 1) annotation(Placement(transformation(extent = {{ -40,70},{ -60,90}}, rotation = 0)));
      protected 
        BusUsage_Utilities.Interfaces.ControlBus controlBus annotation(Placement(transformation(origin = {30,10}, extent = {{ -20,20},{20, -20}}, rotation = 90)));

      equation 
        connect(sine.y,controlBus.realSignal1) annotation(Line(points = {{ -39,50},{12,50},{12,14},{30,14},{30,10}}, color = {0,0,127}, smooth = Smooth.None));
        connect(booleanStep.y,controlBus.booleanSignal) annotation(Line(points = {{ -37,10},{30,10}}, color = {255,0,255}, smooth = Smooth.None));
        connect(integerStep.y,controlBus.integerSignal) annotation(Line(points = {{ -39, -30},{0, -30},{0,10},{30,10}}, color = {255,127,0}, smooth = Smooth.None));
        connect(part.subControlBus,controlBus.subControlBus) annotation(Line(points = {{ -40, -70},{30, -70},{30,10}}, color = {255,204,51}, thickness = 0.5, smooth = Smooth.None));
        connect(gain.u,controlBus.realSignal1) annotation(Line(points = {{ -38,80},{20,80},{20,10},{30,10}}, color = {0,0,127}, smooth = Smooth.None));
      end BusUsage;
      package BusUsage_Utilities "Utility models and connectors for example Modelica.Blocks.Examples.BusUsage"
        annotation(Documentation(info = "<html>
<p>
This package contains utility models and bus definitions needed for the
<a href=\"Modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a> example.
</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));
        package Interfaces "Interfaces specialised for this example"
          expandable connector ControlBus "Empty control bus that is adapted to the signals connected to it"
            extends Modelica.Icons.SignalBus;
            annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -20,2},{22, -2}}, lineColor = {255,204,51}, lineThickness = 0.5)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
This connector defines the \"expandable connector\" ControlBus that
is used as bus in the
<a href=\"Modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a> example.
Note, this connector is \"empty\". When using it, the actual content is
constructed by the signals connected to this bus.
</p>
</html>"));
          end ControlBus;
          expandable connector SubControlBus "Empty sub-control bus that is adapted to the signals connected to it"
            extends Modelica.Icons.SignalSubBus;
            annotation(defaultComponentPrefixes = "protected", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -20,2},{22, -2}}, lineColor = {255,204,51}, lineThickness = 0.5)}), Documentation(info = "<html>
<p>
This connector defines the \"expandable connector\" SubControlBus that
is used as sub-bus in the
<a href=\"Modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a> example.
Note, this connector is \"empty\". When using it, the actual content is
constructed by the signals connected to this bus.
</p>
</html>"));
          end SubControlBus;
          package InternalConnectors "Internal definitions that are usually not utilized (only indirectly via the expandable connectors)"
            expandable connector StandardControlBus "Used to build up the standard control bus (do not use this connector)"
              extends Modelica.Blocks.Examples.BusUsage_Utilities.Interfaces.ControlBus;
              import SI = Modelica.SIunits;
              SI.AngularVelocity realSignal1 "First Real signal (angular velocity)";
              SI.Velocity realSignal2 "Second Real signal";
              Integer integerSignal "Integer signal";
              Boolean booleanSignal "Boolean signal";
              StandardSubControlBus subControlBus "Combined signal";
              annotation(Icon(graphics = {Rectangle(extent = {{ -20,2},{22, -2}}, lineColor = {255,204,51}, lineThickness = 0.5)}), Diagram(graphics), Documentation(info = "<html>
<p>
This connector is used to show default signals that might be added
to the
<a href=\"Modelica://Modelica.Blocks.Examples.BusUsage_Utilities.Interfaces.ControlBus\">ControlBus</a>.
</p>
</html>"));
            end StandardControlBus;
            expandable connector StandardSubControlBus "Used to build up the standard sub-control bus (do not use this connector)"
              extends Modelica.Blocks.Examples.BusUsage_Utilities.Interfaces.SubControlBus;
              Real myRealSignal;
              Boolean myBooleanSignal;
              annotation(defaultComponentPrefixes = "protected", Icon(graphics = {Rectangle(extent = {{ -20,2},{22, -2}}, lineColor = {255,204,51}, lineThickness = 0.5)}), Documentation(info = "<html>
<p>
This connector is used to show default signals that might be added
to the
<a href=\"Modelica://Modelica.Blocks.Examples.BusUsage_Utilities.Interfaces.SubControlBus\">SubControlBus</a>.
</p>
</html>"));
            end StandardSubControlBus;
            annotation(Documentation(info = "<html>
<p>
This package contains the \"actual\" default bus definitions needed for the
<a href=\"Modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a> example.
The bus definitions in this package are the default definitions shown in the
bus menu when connecting a signal to an expandable connector (here: ControlBus
or SubControlBus). Usually, the connectors of this package should not be
utilized by a user.
</p>
</html>"));
          end InternalConnectors;
          annotation(Documentation(info = "<html>
<p>
This package contains the bus definitions needed for the
<a href=\"Modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a> example.
</p>
</html>"));
        end Interfaces;
        model Part "Component with sub-control bus"
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,60},{100, -60}}, fillColor = {159,159,223}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -106,124},{114,68}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
<p>
This model is used to demonstrate the bus usage in example
<a href=\"Modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a>.
</p>
</p>
</html>"));
          Interfaces.SubControlBus subControlBus annotation(Placement(transformation(origin = {100,0}, extent = {{ -20, -20},{20,20}}, rotation = 270)));
          Sources.RealExpression realExpression(y = time) annotation(Placement(transformation(extent = {{ -6,0},{20,20}}, rotation = 0)));
          Sources.BooleanExpression booleanExpression(y = time > 0.5) annotation(Placement(transformation(extent = {{ -6, -30},{20, -10}}, rotation = 0)));

        equation 
          connect(realExpression.y,subControlBus.myRealSignal) annotation(Line(points = {{21.3,10},{88,10},{88,0},{100,0}}, color = {0,0,127}, smooth = Smooth.None));
          connect(booleanExpression.y,subControlBus.myBooleanSignal) annotation(Line(points = {{21.3, -20},{60, -20},{60,0},{100,0}}, color = {255,0,255}, smooth = Smooth.None));
        end Part;
      end BusUsage_Utilities;
    end Examples;
    package Types "Library of constants and types with choices, especially to build menus"
      extends Modelica.Icons.Library;
      annotation(Documentation(info = "<HTML>
<p>
In this package <b>types</b> and <b>constants</b> are defined that are used
in library Modelica.Blocks. The types have additional annotation choices
definitions that define the menus to be built up in the graphical
user interface when the type is used as parameter in a declaration.
</p>
</HTML>"));
      type Smoothness = enumeration(LinearSegments  "Table points are linearly interpolated", ContinuousDerivative  "Table points are interpolated such that the first derivative is continuous") "Enumeration defining the smoothness of table interpolation" annotation(Documentation(info = "<html>

</html>"));
      type Extrapolation = enumeration(HoldLastPoint "Hold the last table point outside of the table scope", LastTwoPoints  "Extrapolate linearly through the last two table points outside of the table scope", Periodic  "Repeat the table scope periodically") "Enumeration defining the extrapolation of time table interpolation" annotation(Documentation(info = "<html>

</html>"));
      type Init = enumeration(NoInit "No initialization (start values are used as guess values with fixed=false)", SteadyState "Steady state initialization (derivatives of states are zero)", InitialState  "Initialization with initial states", InitialOutput  "Initialization with initial outputs (and steady state of the states if possibles)") "Enumeration defining initialization of a block" annotation(Evaluate = true, Documentation(info = "<html>

</html>"));
      type InitPID = enumeration(NoInit "No initialization (start values are used as guess values with fixed=false)", SteadyState "Steady state initialization (derivatives of states are zero)", InitialState "Initialization with initial states", InitialOutput  "Initialization with initial outputs (and steady state of the states if possibles)", DoNotUse_InitialIntegratorState  "Don't use, only for backward compatibility (initialize only integrator state)") "Enumeration defining initialization of PID and LimPID blocks" annotation(Documentation(info = "<html>
<p>
This initialization type is identical to Types.Init and has just one
additional option <b>DoNotUse_InitialIntegratorState</b>. This option
is introduced in order that the default initialization for the
Continuous.PID and Continuous.LimPID blocks are backward
compatible. In Modelica 2.2, the integrators have been initialized
with their given states where as the D-part has not been initialized.
The option \"DoNotUse_InitialIntegratorState\" leads to this
initialization definition.
</p>

</html>"), Evaluate = true);
      type SimpleController = enumeration(P "P controller", PI "PI controller", PD  "PD controller", PID  "PID controller") "Enumeration defining P, PI, PD, or PID simple controller type" annotation(Evaluate = true, Documentation(info = "<html>

</html>"));
    end Types;
    package Tables "Library of blocks to interpolate in one and two-dimensional tables"
      extends Modelica.Icons.Library;
      model CombiTable1D "Table look-up in one dimension (matrix/file) with n inputs and n outputs "
        import Modelica.Blocks.Types;
        parameter Boolean tableOnFile = false "true, if table is defined on file or in function usertab" annotation(Dialog(group = "table data definition"));
        parameter Real table[:,:] = fill(0.0, 0, 2) "table matrix (grid = first column; e.g., table=[0,2])" annotation(Dialog(group = "table data definition", enable = not tableOnFile));
        parameter String tableName = "NoName" "table name on file or in function usertab (see docu)" annotation(Dialog(group = "table data definition", enable = tableOnFile));
        parameter String fileName = "NoName" "file where matrix is stored" annotation(Dialog(group = "table data definition", enable = tableOnFile, __Dymola_loadSelector(filter = "Text files (*.txt);;Matlab files (*.mat)", caption = "Open file in which table is present")));
        parameter Integer columns[:] = 2:size(table, 2) "columns of table to be interpolated" annotation(Dialog(group = "table data interpretation"));
        parameter Modelica.Blocks.Types.Smoothness smoothness = Types.Smoothness.LinearSegments "smoothness of table interpolation" annotation(Dialog(group = "table data interpretation"));
        extends Modelica.Blocks.Interfaces.MIMOs(final n = size(columns, 1));
        annotation(Documentation(info = "<html>
<p>
<b>Linear interpolation</b> in <b>one</b> dimension of a <b>table</b>.
Via parameter <b>columns</b> it can be defined how many columns of the
table are interpolated. If, e.g., columns={2,4}, it is assumed that 2 input
and 2 output signals are present and that the first output interpolates
the first input via column 2 and the second output interpolates the
second input via column 4 of the table matrix.
</p>
<p>
The grid points and function values are stored in a matrix \"table[i,j]\",
where the first column \"table[:,1]\" contains the grid points and the
other columns contain the data to be interpolated. Example:
</p>
<pre>
   table = [0,  0;
            1,  1;
            2,  4;
            4, 16]
   If, e.g., the input u = 1.0, the output y =  1.0,
       e.g., the input u = 1.5, the output y =  2.5,
       e.g., the input u = 2.0, the output y =  4.0,
       e.g., the input u =-1.0, the output y = -1.0 (i.e. extrapolation).
</pre>
<ul>
<li> The interpolation is <b>efficient</b>, because a search for a new interpolation
     starts at the interval used in the last call.</li>
<li> If the table has only <b>one row</b>, the table value is returned,
     independent of the value of the input signal.</li>
<li> If the input signal <b>u[i]</b> is <b>outside</b> of the defined <b>interval</b>, i.e.,
     u[i] &gt; table[size(table,1),i+1] or u[i] &lt; table[1,1], the corresponding
     value is also determined by linear
     interpolation through the last or first two points of the table.</li>
<li> The grid values (first column) have to be <b>strict</b>
     monotonically increasing.</li>
</ul>
<p>
The table matrix can be defined in the following ways:
</p>
<ol>
<li> Explicitly supplied as <b>parameter matrix</b> \"table\",
     and the other parameters have the following values:
<pre>
   tableName is \"NoName\" or has only blanks,
   fileName  is \"NoName\" or has only blanks.
</pre></li>
<li> <b>Read</b> from a <b>file</b> \"fileName\" where the matrix is stored as
      \"tableName\". Both ASCII and binary file format is possible.
      (the ASCII format is described below).
      It is most convenient to generate the binary file from Matlab
      (Matlab 4 storage format), e.g., by command
<pre>
   save tables.mat tab1 tab2 tab3 -V4
</pre>
      when the three tables tab1, tab2, tab3 should be
      used from the model.</li>
<li>  Statically stored in function \"usertab\" in file \"usertab.c\".
      The matrix is identified by \"tableName\". Parameter
      fileName = \"NoName\" or has only blanks.</li>
</ol>
<p>
Table definition methods (1) and (3) do <b>not</b> allocate dynamic memory,
and do not access files, whereas method (2) does. Therefore (1) and (3)
are suited for hardware-in-the-loop simulation (e.g. with dSpace hardware).
When the constant \"NO_FILE\" is defined in \"usertab.c\", all parts of the
source code of method (2) are removed by the C-preprocessor, such that
no dynamic memory allocation and no access to files takes place.
</p>
<p>
If tables are read from an ASCII-file, the file need to have the
following structure (\"-----\" is not part of the file content):
</p>
<pre>
-----------------------------------------------------
#1
double tab1(5,2)   # comment line
  0   0
  1   1
  2   4
  3   9
  4  16
double tab2(5,2)   # another comment line
  0   0
  2   2
  4   8
  6  18
  8  32
-----------------------------------------------------
</pre>
<p>
Note, that the first two characters in the file need to be
\"#1\". Afterwards, the corresponding matrix has to be declared
with type, name and actual dimensions. Finally, in successive
rows of the file, the elements of the matrix have to be given.
Several matrices may be defined one after another.
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -60,40},{ -60, -40},{60, -40},{60,40},{30,40},{30, -40},{ -30, -40},{ -30,40},{ -60,40},{ -60,20},{60,20},{60,0},{ -60,0},{ -60, -20},{60, -20},{60, -40},{ -60, -40},{ -60,40},{60,40},{60, -40}}, color = {0,0,0}),Line(points = {{0,40},{0, -40}}, color = {0,0,0}),Rectangle(extent = {{ -60,40},{ -30,20}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -60,20},{ -30,0}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -60,0},{ -30, -20}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -60, -20},{ -30, -40}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -100,0},{ -58,0}}, color = {0,0,255}),Line(points = {{60,0},{100,0}}, color = {0,0,255}),Text(extent = {{ -100,100},{100,64}}, textString = "1 dimensional linear table interpolation", lineColor = {0,0,255}),Line(points = {{ -54,40},{ -54, -40},{54, -40},{54,40},{28,40},{28, -40},{ -28, -40},{ -28,40},{ -54,40},{ -54,20},{54,20},{54,0},{ -54,0},{ -54, -20},{54, -20},{54, -40},{ -54, -40},{ -54,40},{54,40},{54, -40}}, color = {0,0,0}),Line(points = {{0,40},{0, -40}}, color = {0,0,0}),Rectangle(extent = {{ -54,40},{ -28,20}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -54,20},{ -28,0}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -54,0},{ -28, -20}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -54, -20},{ -28, -40}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -50,54},{ -32,42}}, textString = "u[1]/[2]", lineColor = {0,0,255}),Text(extent = {{ -24,54},{0,42}}, textString = "y[1]", lineColor = {0,0,255}),Text(extent = {{ -2, -40},{30, -54}}, textString = "columns", lineColor = {0,0,255}),Text(extent = {{2,54},{26,42}}, textString = "y[2]", lineColor = {0,0,255})}));
      protected 
        Integer tableID;
        function tableInit "Initialize 1-dim. table defined by matrix (for details see: Modelica/C-Sources/ModelicaTables.h)"
          input String tableName;
          input String fileName;
          input Real table[:,:];
          input Modelica.Blocks.Types.Smoothness smoothness;
          output Integer tableID;

          external "C" tableID = ModelicaTables_CombiTable1D_init(tableName,fileName,table,size(table, 1),size(table, 2),smoothness) ;          annotation(Library = "ModelicaExternalC");
        end tableInit;
        function tableIpo "Interpolate 1-dim. table defined by matrix (for details see: Modelica/C-Sources/ModelicaTables.h)"
          input Integer tableID;
          input Integer icol;
          input Real u;
          output Real value;

          external "C" value = ModelicaTables_CombiTable1D_interpolate(tableID,icol,u) ;          annotation(Library = "ModelicaExternalC");
        end tableIpo;

      equation 
if tableOnFile then
                  assert(tableName <> "NoName", "tableOnFile = true and no table name given");
        end if;
if not tableOnFile then
                  assert(size(table, 1) > 0 and size(table, 2) > 0, "tableOnFile = false and parameter table is an empty matrix");
        end if;
        for i in 1:n loop
        y[i] = if not tableOnFile and size(table, 1) == 1 then table[1,columns[i]] else tableIpo(tableID, columns[i], u[i]);

        end for;
        when initial() then
                  tableID = tableInit(if tableOnFile then tableName else "NoName", if tableOnFile then fileName else "NoName", table, smoothness);
        
        end when;
      end CombiTable1D;
      model CombiTable1Ds "Table look-up in one dimension (matrix/file) with one input and n outputs"
        import Modelica.Blocks.Types;
        parameter Boolean tableOnFile = false "true, if table is defined on file or in function usertab" annotation(Dialog(group = "table data definition"));
        parameter Real table[:,:] = fill(0.0, 0, 2) "table matrix (grid = first column; e.g., table=[0,2])" annotation(Dialog(group = "table data definition", enable = not tableOnFile));
        parameter String tableName = "NoName" "table name on file or in function usertab (see docu)" annotation(Dialog(group = "table data definition", enable = tableOnFile));
        parameter String fileName = "NoName" "file where matrix is stored" annotation(Dialog(group = "table data definition", enable = tableOnFile, __Dymola_loadSelector(filter = "Text files (*.txt);;Matlab files (*.mat)", caption = "Open file in which table is present")));
        parameter Integer columns[:] = 2:size(table, 2) "columns of table to be interpolated" annotation(Dialog(group = "table data interpretation"));
        parameter Modelica.Blocks.Types.Smoothness smoothness = Types.Smoothness.LinearSegments "smoothness of table interpolation" annotation(Dialog(group = "table data interpretation"));
        extends Modelica.Blocks.Interfaces.SIMO(final nout = size(columns, 1));
        annotation(Documentation(info = "<html>
<p>
<b>Linear interpolation</b> in <b>one</b> dimension of a <b>table</b>.
Via parameter <b>columns</b> it can be defined how many columns of the
table are interpolated. If, e.g., icol={2,4}, it is assumed that one input
and 2 output signals are present and that the first output interpolates
via column 2 and the second output interpolates via column 4 of the
table matrix.
</p>
<p>
The grid points and function values are stored in a matrix \"table[i,j]\",
where the first column \"table[:,1]\" contains the grid points and the
other columns contain the data to be interpolated. Example:
</p>
<pre>
   table = [0,  0;
            1,  1;
            2,  4;
            4, 16]
   If, e.g., the input u = 1.0, the output y =  1.0,
       e.g., the input u = 1.5, the output y =  2.5,
       e.g., the input u = 2.0, the output y =  4.0,
       e.g., the input u =-1.0, the output y = -1.0 (i.e. extrapolation).
</pre>
<ul>
<li> The interpolation is <b>efficient</b>, because a search for a new interpolation
     starts at the interval used in the last call.</li>
<li> If the table has only <b>one row</b>, the table value is returned,
     independent of the value of the input signal.</li>
<li> If the input signal <b>u</b> is <b>outside</b> of the defined <b>interval</b>, i.e.,
     u &gt; table[size(table,1),1] or u &lt; table[1,1], the corresponding
     value is also determined by linear
     interpolation through the last or first two points of the table.</li>
<li> The grid values (first column) have to be <b>strict</b>
     monotonically increasing.</li>
</ul>
<p>
The table matrix can be defined in the following ways:
</p>
<ol>
<li> Explicitly supplied as <b>parameter matrix</b> \"table\",
     and the other parameters have the following values:
<pre>
   tableName is \"NoName\" or has only blanks,
   fileName  is \"NoName\" or has only blanks.
</pre></li>
<li> <b>Read</b> from a <b>file</b> \"fileName\" where the matrix is stored as
      \"tableName\". Both ASCII and binary file format is possible.
      (the ASCII format is described below).
      It is most convenient to generate the binary file from Matlab
      (Matlab 4 storage format), e.g., by command
<pre>
   save tables.mat tab1 tab2 tab3 -V4
</pre>
      when the three tables tab1, tab2, tab3 should be
      used from the model.</li>
<li>  Statically stored in function \"usertab\" in file \"usertab.c\".
      The matrix is identified by \"tableName\". Parameter
      fileName = \"NoName\" or has only blanks.</li>
</ol>
<p>
Table definition methods (1) and (3) do <b>not</b> allocate dynamic memory,
and do not access files, whereas method (2) does. Therefore (1) and (3)
are suited for hardware-in-the-loop simulation (e.g. with dSpace hardware).
When the constant \"NO_FILE\" is defined, all parts of the
source code of method (2) are removed by the C-preprocessor, such that
no dynamic memory allocation and no access to files takes place.
</p>
<p>
If tables are read from an ASCII-file, the file need to have the
following structure (\"-----\" is not part of the file content):
</p>
<pre>
-----------------------------------------------------
#1
double tab1(5,2)   # comment line
  0   0
  1   1
  2   4
  3   9
  4  16
double tab2(5,2)   # another comment line
  0   0
  2   2
  4   8
  6  18
  8  32
-----------------------------------------------------
</pre>
<p>
Note, that the first two characters in the file need to be
\"#1\". Afterwards, the corresponding matrix has to be declared
with type, name and actual dimensions. Finally, in successive
rows of the file, the elements of the matrix have to be given.
Several matrices may be defined one after another.
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -60,40},{ -60, -40},{60, -40},{60,40},{30,40},{30, -40},{ -30, -40},{ -30,40},{ -60,40},{ -60,20},{60,20},{60,0},{ -60,0},{ -60, -20},{60, -20},{60, -40},{ -60, -40},{ -60,40},{60,40},{60, -40}}, color = {0,0,0}),Line(points = {{0,40},{0, -40}}, color = {0,0,0}),Rectangle(extent = {{ -60,40},{ -30,20}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -60,20},{ -30,0}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -60,0},{ -30, -20}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -60, -20},{ -30, -40}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -100,0},{ -58,0}}, color = {0,0,255}),Line(points = {{60,0},{100,0}}, color = {0,0,255}),Text(extent = {{ -100,100},{100,64}}, textString = "1 dimensional linear table interpolation", lineColor = {0,0,255}),Line(points = {{ -54,40},{ -54, -40},{54, -40},{54,40},{28,40},{28, -40},{ -28, -40},{ -28,40},{ -54,40},{ -54,20},{54,20},{54,0},{ -54,0},{ -54, -20},{54, -20},{54, -40},{ -54, -40},{ -54,40},{54,40},{54, -40}}, color = {0,0,0}),Line(points = {{0,40},{0, -40}}, color = {0,0,0}),Rectangle(extent = {{ -54,40},{ -28,20}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -54,20},{ -28,0}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -54,0},{ -28, -20}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -54, -20},{ -28, -40}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -52,56},{ -34,44}}, textString = "u", lineColor = {0,0,255}),Text(extent = {{ -22,54},{2,42}}, textString = "y[1]", lineColor = {0,0,255}),Text(extent = {{4,54},{28,42}}, textString = "y[2]", lineColor = {0,0,255}),Text(extent = {{0, -40},{32, -54}}, textString = "columns", lineColor = {0,0,255})}));
      protected 
        Integer tableID;
        function tableInit "Initialize 1-dim. table defined by matrix (for details see: Modelica/C-Sources/ModelicaTables.h)"
          input String tableName;
          input String fileName;
          input Real table[:,:];
          input Modelica.Blocks.Types.Smoothness smoothness;
          output Integer tableID;

          external "C" tableID = ModelicaTables_CombiTable1D_init(tableName,fileName,table,size(table, 1),size(table, 2),smoothness) ;          annotation(Library = "ModelicaExternalC");
        end tableInit;
        function tableIpo "Interpolate 1-dim. table defined by matrix (for details see: Modelica/C-Sources/ModelicaTables.h)"
          input Integer tableID;
          input Integer icol;
          input Real u;
          output Real value;

          external "C" value = ModelicaTables_CombiTable1D_interpolate(tableID,icol,u) ;          annotation(Library = "ModelicaExternalC");
        end tableIpo;

      equation 
if tableOnFile then
                  assert(tableName <> "NoName", "tableOnFile = true and no table name given");
        end if;
if not tableOnFile then
                  assert(size(table, 1) > 0 and size(table, 2) > 0, "tableOnFile = false and parameter table is an empty matrix");
        end if;
        for i in 1:nout loop
        y[i] = if not tableOnFile and size(table, 1) == 1 then table[1,columns[i]] else tableIpo(tableID, columns[i], u);

        end for;
        when initial() then
                  tableID = tableInit(if tableOnFile then tableName else "NoName", if tableOnFile then fileName else "NoName", table, smoothness);
        
        end when;
      end CombiTable1Ds;
      model CombiTable2D "Table look-up in two dimensions (matrix/file) "
        import Modelica.Blocks.Types;
        extends Modelica.Blocks.Interfaces.SI2SO;
        parameter Boolean tableOnFile = false "true, if table is defined on file or in function usertab" annotation(Dialog(group = "table data definition"));
        parameter Real table[:,:] = fill(0.0, 0, 2) "table matrix (grid u1 = first column, grid u2 = first row; e.g. table=[0,0;0,1])" annotation(Dialog(group = "table data definition", enable = not tableOnFile));
        parameter String tableName = "NoName" "table name on file or in function usertab (see docu)" annotation(Dialog(group = "table data definition", enable = tableOnFile));
        parameter String fileName = "NoName" "file where matrix is stored" annotation(Dialog(group = "table data definition", enable = tableOnFile, __Dymola_loadSelector(filter = "Text files (*.txt);;Matlab files (*.mat)", caption = "Open file in which table is present")));
        parameter Modelica.Blocks.Types.Smoothness smoothness = Types.Smoothness.LinearSegments "smoothness of table interpolation" annotation(Dialog(group = "table data interpretation"));
        annotation(Documentation(info = "<html>
<p>
<b>Linear interpolation</b> in <b>two</b> dimensions of a <b>table</b>.
The grid points and function values are stored in a matrix \"table[i,j]\",
where:
</p>
<ul>
<li> the first column \"table[2:,1]\" contains the u[1] grid points,</li>
<li> the first row \"table[1,2:]\" contains the u[2] grid points,</li>
<li> the other rows and columns contain the data to be interpolated.</li>
</ul>
<p>
Example:
</p>
<pre>
           |       |       |       |
           |  1.0  |  2.0  |  3.0  |  // u2
       ----*-------*-------*-------*
       1.0 |  1.0  |  3.0  |  5.0  |
       ----*-------*-------*-------*
       2.0 |  2.0  |  4.0  |  6.0  |
       ----*-------*-------*-------*
     // u1
   is defined as
      table = [0.0,   1.0,   2.0,   3.0;
               1.0,   1.0,   3.0,   5.0;
               2.0,   2.0,   4.0,   6.0]
   If, e.g. the input u is [1.0;1.0], the output y is 1.0,
       e.g. the input u is [2.0;1.5], the output y is 3.0.
</pre>
<ul>
<li> The interpolation is <b>efficient</b>, because a search for a new interpolation
     starts at the interval used in the last call.</li>
<li> If the table has only <b>one element</b>, the table value is returned,
     independent of the value of the input signal.</li>
<li> If the input signal <b>u1</b> or <b>u2</b> is <b>outside</b> of the defined <b>interval</b>,
     the corresponding value is also determined by linear
     interpolation through the last or first two points of the table.</li>
<li> The grid values (first column and first row) have to be <b>strict</b>
     monotonically increasing.</li>
</ul>
<p>
The table matrix can be defined in the following ways:
</p>
<ol>
<li> Explicitly supplied as <b>parameter matrix</b> \"table\",
     and the other parameters have the following values:
<pre>
   tableName is \"NoName\" or has only blanks,
   fileName  is \"NoName\" or has only blanks.
</pre></li>
<li> <b>Read</b> from a <b>file</b> \"fileName\" where the matrix is stored as
      \"tableName\". Both ASCII and binary file format is possible.
      (the ASCII format is described below).
      It is most convenient to generate the binary file from Matlab
      (Matlab 4 storage format), e.g., by command
<pre>
   save tables.mat tab1 tab2 tab3 -V4
</pre>
      when the three tables tab1, tab2, tab3 should be
      used from the model.</li>
<li>  Statically stored in function \"usertab\" in file \"usertab.c\".
      The matrix is identified by \"tableName\". Parameter
      fileName = \"NoName\" or has only blanks.</li>
</ol>
<p>
Table definition methods (1) and (3) do <b>not</b> allocate dynamic memory,
and do not access files, whereas method (2) does. Therefore (1) and (3)
are suited for hardware-in-the-loop simulation (e.g. with dSpace hardware).
When the constant \"NO_FILE\" is defined, all parts of the
source code of method (2) are removed by the C-preprocessor, such that
no dynamic memory allocation and no access to files takes place.
</p>
<p>
If tables are read from an ASCII-file, the file need to have the
following structure (\"-----\" is not part of the file content):
</p>
<pre>
-----------------------------------------------------
#1
double table2D_1(3,4)   # comment line
0.0  1.0  2.0  3.0  # u[2] grid points
1.0  1.0  3.0  5.0
2.0  2.0  4.0  6.0

double table2D_2(4,4)   # comment line
0.0  1.0  2.0  3.0  # u[2] grid points
1.0  1.0  3.0  5.0
2.0  2.0  4.0  6.0
3.0  3.0  5.0  7.0
-----------------------------------------------------
</pre>
<p>
Note, that the first two characters in the file need to be
\"#1\". Afterwards, the corresponding matrix has to be declared
with type, name and actual dimensions. Finally, in successive
rows of the file, the elements of the matrix have to be given.
Several matrices may be defined one after another.
The matrix elements are interpreted in exactly the same way
as if the matrix is given as a parameter. For example, the first
column \"table2D_1[2:,1]\" contains the u[1] grid points,
and the first row \"table2D_1[1,2:]\" contains the u[2] grid points.
</p>

</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -60,40},{ -60, -40},{60, -40},{60,40},{30,40},{30, -40},{ -30, -40},{ -30,40},{ -60,40},{ -60,20},{60,20},{60,0},{ -60,0},{ -60, -20},{60, -20},{60, -40},{ -60, -40},{ -60,40},{60,40},{60, -40}}, color = {0,0,0}),Line(points = {{0,40},{0, -40}}, color = {0,0,0}),Rectangle(extent = {{ -60,20},{ -30,0}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -60,0},{ -30, -20}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -60, -20},{ -30, -40}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -30,40},{0,20}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{0,40},{30,20}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{30,40},{60,20}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Line(points = {{ -60,40},{ -30,20}}, color = {0,0,0}),Line(points = {{ -30,40},{ -60,20}}, color = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{60,0},{100,0}}, color = {0,0,255}),Text(extent = {{ -100,100},{100,64}}, textString = "2 dimensional linear table interpolation", lineColor = {0,0,255}),Line(points = {{ -54,40},{ -54, -40},{54, -40},{54,40},{28,40},{28, -40},{ -28, -40},{ -28,40},{ -54,40},{ -54,20},{54,20},{54,0},{ -54,0},{ -54, -20},{54, -20},{54, -40},{ -54, -40},{ -54,40},{54,40},{54, -40}}, color = {0,0,0}),Line(points = {{0,40},{0, -40}}, color = {0,0,0}),Rectangle(extent = {{ -54,20},{ -28,0}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -54,0},{ -28, -20}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -54, -20},{ -28, -40}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -28,40},{0,20}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{0,40},{28,20}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{28,40},{54,20}}, lineColor = {0,0,0}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Line(points = {{ -54,40},{ -28,20}}, color = {0,0,0}),Line(points = {{ -28,40},{ -54,20}}, color = {0,0,0}),Text(extent = {{ -54, -40},{ -30, -56}}, textString = "u1", lineColor = {0,0,255}),Text(extent = {{28,58},{52,44}}, textString = "u2", lineColor = {0,0,255}),Text(extent = {{ -2,12},{32, -22}}, textString = "y", lineColor = {0,0,255})}));
      protected 
        Integer tableID;
        function tableInit "Initialize 2-dim. table defined by matrix (for details see: Modelica/C-Sources/ModelicaTables.h)"
          input String tableName;
          input String fileName;
          input Real table[:,:];
          input Modelica.Blocks.Types.Smoothness smoothness;
          output Integer tableID;

          external "C" tableID = ModelicaTables_CombiTable2D_init(tableName,fileName,table,size(table, 1),size(table, 2),smoothness) ;          annotation(Library = "ModelicaExternalC");
        end tableInit;
        function tableIpo "Interpolate 2-dim. table defined by matrix (for details see: Modelica/C-Sources/ModelicaTables.h)"
          input Integer tableID;
          input Real u1;
          input Real u2;
          output Real value;

          external "C" value = ModelicaTables_CombiTable2D_interpolate(tableID,u1,u2) ;          annotation(Library = "ModelicaExternalC");
        end tableIpo;

      equation 
if tableOnFile then
                  assert(tableName <> "NoName", "tableOnFile = true and no table name given");
        end if;
if not tableOnFile then
                  assert(size(table, 1) > 0 and size(table, 2) > 0, "tableOnFile = false and parameter table is an empty matrix");
        end if;
        y = tableIpo(tableID, u1, u2);
        when initial() then
                  tableID = tableInit(if tableOnFile then tableName else "NoName", if tableOnFile then fileName else "NoName", table, smoothness);
        
        end when;
      end CombiTable2D;
      annotation(Documentation(info = "<html>
<p>
This package contains blocks for one- and two-dimensional
interpolation in tables.
</p>
</html>"));
    end Tables;
    package Sources "Library of signal source blocks generating Real and Boolean signals"
      block RealExpression "Set output signal to a time varying Real expression"
        Modelica.Blocks.Interfaces.RealOutput y = 0.0 "Value of Real output" annotation(Dialog(group = "Time varying output signal"), Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100,40},{100, -40}}, lineColor = {0,0,0}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised),Text(extent = {{ -96,15},{96, -15}}, lineColor = {0,0,0}, textString = "%y"),Text(extent = {{ -150,90},{140,50}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
The (time varying) Real output signal of this block can be defined in its
parameter menu via variable <b>y</b>. The purpose is to support the
easy definition of Real expressions in a block diagram. For example,
in the y-menu the definition \"if time &lt; 1 then 0 else 1\" can be given in order
to define that the output signal is one, if time &ge; 1 and otherwise
it is zero. Note, that \"time\" is a built-in variable that is always
accessible and represents the \"model time\" and that
Variable <b>y</b> is both a variable and a connector.
</p>
</html>"));
      end RealExpression;
      block IntegerExpression "Set output signal to a time varying Integer expression"
        Modelica.Blocks.Interfaces.IntegerOutput y = 0 "Value of Integer output" annotation(Dialog(group = "Time varying output signal"), Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100,40},{100, -40}}, lineColor = {0,0,0}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised),Text(extent = {{ -96,15},{96, -15}}, lineColor = {0,0,0}, textString = "%y"),Text(extent = {{ -150,90},{140,50}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
The (time varying) Integer output signal of this block can be defined in its
parameter menu via variable <b>y</b>. The purpose is to support the
easy definition of Integer expressions in a block diagram. For example,
in the y-menu the definition \"if time &lt; 1 then 0 else 1\" can be given in order
to define that the output signal is one, if time &ge; 1 and otherwise
it is zero. Note, that \"time\" is a built-in variable that is always
accessible and represents the \"model time\" and that
Variable <b>y</b> is both a variable and a connector.
</p>
</html>"));
      end IntegerExpression;
      block BooleanExpression "Set output signal to a time varying Boolean expression"
        Modelica.Blocks.Interfaces.BooleanOutput y = false "Value of Boolean output" annotation(Dialog(group = "Time varying output signal"), Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100,40},{100, -40}}, lineColor = {0,0,0}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised),Text(extent = {{ -96,15},{96, -15}}, lineColor = {0,0,0}, textString = "%y"),Text(extent = {{ -150,90},{140,50}}, textString = "%name", lineColor = {0,0,255}),Polygon(points = {{100,10},{120,0},{100, -10},{100,10}}, lineColor = DynamicSelect({255,0,255}, if y > 0.5 then {0,255,0} else {255,0,255}), fillColor = DynamicSelect({255,255,255}, if y > 0.5 then {0,255,0} else {255,255,255}), fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
The (time varying) Boolean output signal of this block can be defined in its
parameter menu via variable <b>y</b>. The purpose is to support the
easy definition of Boolean expressions in a block diagram. For example,
in the y-menu the definition \"time &gt;= 1 and time &lt;= 2\" can be given in order
to define that the output signal is <b>true</b> in the time interval
1 &le; time &le; 2 and otherwise it is <b>false</b>.
Note, that \"time\" is a built-in variable that is always
accessible and represents the \"model time\" and that
Variable <b>y</b> is both a variable and a connector.
</p>
</html>"));
      end BooleanExpression;
      import Modelica.Blocks.Interfaces;
      import Modelica.SIunits;
      extends Modelica.Icons.Library;
      annotation(Documentation(info = "<HTML>
<p>
This package contains <b>source</b> components, i.e., blocks which
have only output signals. These blocks are used as signal generators
for Real, Integer and Boolean signals.
</p>

<p>
All Real source signals (with the exception of the Constant source)
have at least the following two parameters:
</p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>offset</b></td>
      <td valign=\"top\">Value which is added to the signal</td>
  </tr>
  <tr><td valign=\"top\"><b>startTime</b></td>
      <td valign=\"top\">Start time of signal. For time &lt; startTime,
                the output y is set to offset.</td>
  </tr>
</table>

<p>
The <b>offset</b> parameter is especially useful in order to shift
the corresponding source, such that at initial time the system
is stationary. To determine the corresponding value of offset,
usually requires a trimming calculation.
</p>
</HTML>
", revisions = "<html>
<ul>
<li><i>October 21, 2002</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>
       and <a href=\"http://www.robotic.dlr.de/Christian.Schweiger/\">Christian Schweiger</a>:<br>
       Integer sources added. Step, TimeTable and BooleanStep slightly changed.</li>
<li><i>Nov. 8, 1999</i>
       by <a href=\"mailto:clauss@eas.iis.fhg.de\">Christoph Clau&szlig;</a>,
       <a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>,
       <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       New sources: Exponentials, TimeTable. Trapezoid slightly enhanced
       (nperiod=-1 is an infinite number of periods).</li>
<li><i>Oct. 31, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       <a href=\"mailto:clauss@eas.iis.fhg.de\">Christoph Clau&szlig;</a>,
       <a href=\"mailto:Andre.Schneider@eas.iis.fraunhofer.de\">Andre.Schneider@eas.iis.fraunhofer.de</a>,
       All sources vectorized. New sources: ExpSine, Trapezoid,
       BooleanConstant, BooleanStep, BooleanPulse, SampleTrigger.
       Improved documentation, especially detailed description of
       signals in diagram layer.</li>
<li><i>June 29, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized a first version, based on an existing Dymola library
       of Dieter Moormann and Hilding Elmqvist.</li>
</ul>
</html>"));
      block Clock "Generate actual time signal "
        parameter Modelica.SIunits.Time offset = 0 "Offset of output signal";
        parameter Modelica.SIunits.Time startTime = 0 "Output = offset for time < startTime";
        extends Interfaces.SO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -80,80},{80, -80}}, lineColor = {160,160,164}),Line(points = {{0,80},{0,60}}, color = {160,160,164}),Line(points = {{80,0},{60,0}}, color = {160,160,164}),Line(points = {{0, -80},{0, -60}}, color = {160,160,164}),Line(points = {{ -80,0},{ -60,0}}, color = {160,160,164}),Line(points = {{37,70},{26,50}}, color = {160,160,164}),Line(points = {{70,38},{49,26}}, color = {160,160,164}),Line(points = {{71, -37},{52, -27}}, color = {160,160,164}),Line(points = {{39, -70},{29, -51}}, color = {160,160,164}),Line(points = {{ -39, -70},{ -29, -52}}, color = {160,160,164}),Line(points = {{ -71, -37},{ -50, -26}}, color = {160,160,164}),Line(points = {{ -71,37},{ -54,28}}, color = {160,160,164}),Line(points = {{ -38,70},{ -28,51}}, color = {160,160,164}),Line(points = {{0,0},{ -50,50}}, color = {0,0,0}, thickness = 0.5),Line(points = {{0,0},{40,0}}, color = {0,0,0}, thickness = 0.5),Text(extent = {{ -150, -150},{150, -110}}, lineColor = {0,0,0}, textString = "startTime=%startTime")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -80,90},{ -85,68},{ -75,68},{ -80,90}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {95,95,95}),Line(points = {{ -80,0},{ -10,0},{60,70}}, color = {0,0,255}, thickness = 0.5),Line(points = {{ -90, -70},{82, -70}}, color = {95,95,95}),Polygon(points = {{90, -70},{68, -64},{68, -76},{90, -70}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{ -34,0},{ -37, -13},{ -30, -13},{ -34,0}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -34, -13},{ -34, -70}}, color = {95,95,95}),Polygon(points = {{ -34, -69},{ -37, -56},{ -31, -56},{ -34, -69},{ -34, -69}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Text(extent = {{ -77, -28},{ -35, -40}}, lineColor = {0,0,0}, textString = "offset"),Text(extent = {{ -30, -73},{18, -86}}, lineColor = {0,0,0}, textString = "startTime"),Text(extent = {{ -81,91},{ -40,71}}, lineColor = {0,0,0}, textString = "y"),Text(extent = {{63, -79},{94, -89}}, lineColor = {0,0,0}, textString = "time"),Line(points = {{ -10,0},{ -10, -70}}, color = {95,95,95}),Line(points = {{ -10,0},{50,0}}, color = {95,95,95}),Line(points = {{50,0},{50,60}}, color = {95,95,95}),Text(extent = {{35,33},{50,23}}, lineColor = {0,0,0}, textString = "1"),Text(extent = {{14,13},{32,1}}, lineColor = {0,0,0}, textString = "1")}), Documentation(info = "<html>
<p>
The Real output y is a clock signal:
</p>

<p>
<img src=\"../Images/Blocks/Sources/Clock.png\">
</p>
</html>"));

      equation 
        y = offset + (if time < startTime then 0 else time - startTime);
      end Clock;
      block Constant "Generate constant signal of type Real"
        parameter Real k(start = 1) "Constant output value";
        extends Interfaces.SO;
        annotation(defaultComponentName = "const", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,0},{80,0}}, color = {0,0,0}),Text(extent = {{ -150, -150},{150, -110}}, lineColor = {0,0,0}, textString = "k=%k")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Polygon(points = {{ -80,90},{ -86,68},{ -74,68},{ -80,90}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {95,95,95}),Line(points = {{ -80,0},{80,0}}, color = {0,0,255}, thickness = 0.5),Line(points = {{ -90, -70},{82, -70}}, color = {95,95,95}),Polygon(points = {{90, -70},{68, -64},{68, -76},{90, -70}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Text(extent = {{ -83,92},{ -30,74}}, lineColor = {0,0,0}, textString = "y"),Text(extent = {{70, -80},{94, -100}}, lineColor = {0,0,0}, textString = "time"),Text(extent = {{ -101,8},{ -81, -12}}, lineColor = {0,0,0}, textString = "k")}), Documentation(info = "<html>
<p>
The Real output y is a constant signal:
</p>

<p>
<img src=\"../Images/Blocks/Sources/Constant.png\">
</p>
</html>"));

      equation 
        y = k;
      end Constant;
      block Step "Generate step signal of type Real"
        parameter Real height = 1 "Height of step";
        extends Interfaces.SignalSource;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -70},{0, -70},{0,50},{80,50}}, color = {0,0,0}),Text(extent = {{ -150, -150},{150, -110}}, lineColor = {0,0,0}, textString = "startTime=%startTime")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -80,90},{ -86,68},{ -74,68},{ -80,90}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {95,95,95}),Line(points = {{ -80, -18},{0, -18},{0,50},{80,50}}, color = {0,0,255}, thickness = 0.5),Line(points = {{ -90, -70},{82, -70}}, color = {95,95,95}),Polygon(points = {{90, -70},{68, -64},{68, -76},{90, -70}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Text(extent = {{70, -80},{94, -100}}, lineColor = {0,0,0}, textString = "time"),Text(extent = {{ -21, -72},{25, -90}}, lineColor = {0,0,0}, textString = "startTime"),Line(points = {{0, -17},{0, -71}}, color = {95,95,95}),Text(extent = {{ -68, -36},{ -22, -54}}, lineColor = {0,0,0}, textString = "offset"),Line(points = {{ -13,50},{ -13, -17}}, color = {95,95,95}),Polygon(points = {{2,50},{ -19,50},{2,50}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{ -13, -17},{ -16, -4},{ -10, -4},{ -13, -17},{ -13, -17}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{ -13,50},{ -16,37},{ -9,37},{ -13,50}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Text(extent = {{ -68,26},{ -22,8}}, lineColor = {0,0,0}, textString = "height"),Polygon(points = {{ -13, -69},{ -16, -56},{ -10, -56},{ -13, -69},{ -13, -69}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -13, -18},{ -13, -70}}, color = {95,95,95}),Polygon(points = {{ -13, -18},{ -16, -31},{ -9, -31},{ -13, -18}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Text(extent = {{ -72,100},{ -31,80}}, lineColor = {0,0,0}, textString = "y")}), Documentation(info = "<html>
<p>
The Real output y is a step signal:
</p>

<p>
<img src=\"../Images/Blocks/Sources/Step.png\">
</p>

</html>"));

      equation 
        y = offset + (if time < startTime then 0 else height);
      end Step;
      block Ramp "Generate ramp signal"
        parameter Real height = 1 "Height of ramps";
        parameter Modelica.SIunits.Time duration(min = Modelica.Constants.small, start = 2) "Durations of ramp";
        parameter Real offset = 0 "Offset of output signal";
        parameter Modelica.SIunits.Time startTime = 0 "Output = offset for time < startTime";
        extends Interfaces.SO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -70},{ -40, -70},{31,38}}, color = {0,0,0}),Text(extent = {{ -150, -150},{150, -110}}, lineColor = {0,0,0}, textString = "duration=%duration"),Line(points = {{31,38},{86,38}}, color = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -80,90},{ -86,68},{ -74,68},{ -80,90}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {95,95,95}),Line(points = {{ -80, -20},{ -20, -20},{50,50}}, color = {0,0,255}, thickness = 0.5),Line(points = {{ -90, -70},{82, -70}}, color = {95,95,95}),Polygon(points = {{90, -70},{68, -64},{68, -76},{90, -70}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{ -40, -20},{ -42, -30},{ -37, -30},{ -40, -20}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -40, -20},{ -40, -70}}, color = {95,95,95}, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Polygon(points = {{ -40, -70},{ -43, -60},{ -38, -60},{ -40, -70},{ -40, -70}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Text(extent = {{ -72, -39},{ -34, -50}}, lineColor = {0,0,0}, textString = "offset"),Text(extent = {{ -38, -72},{6, -83}}, lineColor = {0,0,0}, textString = "startTime"),Text(extent = {{ -78,92},{ -37,72}}, lineColor = {0,0,0}, textString = "y"),Text(extent = {{70, -80},{94, -91}}, lineColor = {0,0,0}, textString = "time"),Line(points = {{ -20, -20},{ -20, -70}}, color = {95,95,95}),Line(points = {{ -19, -20},{50, -20}}, color = {95,95,95}, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Line(points = {{50,50},{101,50}}, color = {0,0,255}, thickness = 0.5),Line(points = {{50,50},{50, -20}}, color = {95,95,95}, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Polygon(points = {{50, -20},{42, -18},{42, -22},{50, -20}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{ -20, -20},{ -11, -18},{ -11, -22},{ -20, -20}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{50,50},{48,40},{53,40},{50,50}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{50, -20},{47, -10},{52, -10},{50, -20},{50, -20}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Text(extent = {{53,23},{82,10}}, lineColor = {0,0,0}, textString = "height"),Text(extent = {{ -2, -21},{37, -33}}, lineColor = {0,0,0}, textString = "duration")}), Documentation(info = "<html>
<p>
The Real output y is a ramp signal:
</p>

<p>
<img src=\"../Images/Blocks/Sources/Ramp.png\">
</p>
</html>"));

      equation 
        y = offset + (if time < startTime then 0 else if time < startTime + duration then ((time - startTime) * height) / duration else height);
      end Ramp;
      block Sine "Generate sine signal"
        parameter Real amplitude = 1 "Amplitude of sine wave";
        parameter SIunits.Frequency freqHz(start = 1) "Frequency of sine wave";
        parameter SIunits.Angle phase = 0 "Phase of sine wave";
        parameter Real offset = 0 "Offset of output signal";
        parameter SIunits.Time startTime = 0 "Output = offset for time < startTime";
        extends Interfaces.SO;
      protected 
        constant Real pi = Modelica.Constants.pi;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,0},{ -68.7,34.2},{ -61.5,53.1},{ -55.1,66.4},{ -49.4,74.6},{ -43.8,79.1},{ -38.2,79.8},{ -32.6,76.6},{ -26.9,69.7},{ -21.3,59.4},{ -14.9,44.1},{ -6.83,21.2},{10.1, -30.8},{17.3, -50.2},{23.7, -64.2},{29.3, -73.1},{35, -78.4},{40.6, -80},{46.2, -77.6},{51.9, -71.5},{57.5, -61.9},{63.9, -47.2},{72, -24.8},{80,0}}, color = {0,0,0}),Text(extent = {{ -147, -152},{153, -112}}, lineColor = {0,0,0}, textString = "freqHz=%freqHz")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80, -90},{ -80,84}}, color = {95,95,95}),Polygon(points = {{ -80,97},{ -84,81},{ -76,81},{ -80,97}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -99, -40},{85, -40}}, color = {95,95,95}),Polygon(points = {{97, -40},{81, -36},{81, -45},{97, -40}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -41, -2},{ -31.6,34.2},{ -26.1,53.1},{ -21.3,66.4},{ -17.1,74.6},{ -12.9,79.1},{ -8.64,79.8},{ -4.42,76.6},{ -0.201,69.7},{4.02,59.4},{8.84,44.1},{14.9,21.2},{27.5, -30.8},{33, -50.2},{37.8, -64.2},{42, -73.1},{46.2, -78.4},{50.5, -80},{54.7, -77.6},{58.9, -71.5},{63.1, -61.9},{67.9, -47.2},{74, -24.8},{80,0}}, color = {0,0,255}, thickness = 0.5),Line(points = {{ -41, -2},{ -80, -2}}, color = {0,0,255}, thickness = 0.5),Text(extent = {{ -87,12},{ -40,0}}, lineColor = {0,0,0}, textString = "offset"),Line(points = {{ -41, -2},{ -41, -40}}, color = {95,95,95}),Text(extent = {{ -60, -43},{ -14, -54}}, lineColor = {0,0,0}, textString = "startTime"),Text(extent = {{75, -47},{100, -60}}, lineColor = {0,0,0}, textString = "time"),Text(extent = {{ -80,99},{ -40,82}}, lineColor = {0,0,0}, textString = "y"),Line(points = {{ -9,79},{43,79}}, color = {95,95,95}),Line(points = {{ -41, -2},{50, -2}}, color = {95,95,95}),Polygon(points = {{33,79},{30,66},{37,66},{33,79}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Text(extent = {{37,57},{83,39}}, lineColor = {0,0,0}, textString = "amplitude"),Polygon(points = {{33, -2},{30,11},{36,11},{33, -2},{33, -2}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{33,77},{33, -2}}, color = {95,95,95})}), Documentation(info = "<html>
<p>
The Real output y is a sine signal:
</p>

<p>
<img src=\"../Images/Blocks/Sources/Sine.png\">
</p>
</html>"));

      equation 
        y = offset + (if time < startTime then 0 else amplitude * Modelica.Math.sin(2 * pi * freqHz * (time - startTime) + phase));
      end Sine;
      block ExpSine "Generate exponentially damped sine signal"
        parameter Real amplitude = 1 "Amplitude of sine wave";
        parameter SIunits.Frequency freqHz(start = 2) "Frequency of sine wave";
        parameter SIunits.Angle phase = 0 "Phase of sine wave";
        parameter SIunits.Damping damping(start = 1) "Damping coefficient of sine wave";
        parameter Real offset = 0 "Offset of output signal";
        parameter SIunits.Time startTime = 0 "Output = offset for time < startTime";
        extends Interfaces.SO;
      protected 
        constant Real pi = Modelica.Constants.pi;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,0},{ -75.2,32.3},{ -72,50.3},{ -68.7,64.5},{ -65.5,74.2},{ -62.3,79.3},{ -59.1,79.6},{ -55.9,75.3},{ -52.7,67.1},{ -48.6,52.2},{ -43,25.8},{ -35, -13.9},{ -30.2, -33.7},{ -26.1, -45.9},{ -22.1, -53.2},{ -18.1, -55.3},{ -14.1, -52.5},{ -10.1, -45.3},{ -5.23, -32.1},{8.44,13.7},{13.3,26.4},{18.1,34.8},{22.1,38},{26.9,37.2},{31.8,31.8},{38.2,19.4},{51.1, -10.5},{57.5, -21.2},{63.1, -25.9},{68.7, -25.9},{75.2, -20.5},{80, -13.8}}, color = {0,0,0}),Text(extent = {{ -147, -152},{153, -112}}, lineColor = {0,0,0}, textString = "freqHz=%freqHz")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -86, -90},{ -86,84}}, color = {95,95,95}),Polygon(points = {{ -86,98},{ -90,82},{ -82,82},{ -86,98}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -99, -40},{85, -40}}, color = {95,95,95}),Polygon(points = {{97, -40},{81, -35},{81, -44},{97, -40}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -50,0},{ -46.1,28.2},{ -43.5,44},{ -40.9,56.4},{ -38.2,64.9},{ -35.6,69.4},{ -33,69.6},{ -30.4,65.9},{ -27.8,58.7},{ -24.5,45.7},{ -19.9,22.5},{ -13.4, -12.2},{ -9.5, -29.5},{ -6.23, -40.1},{ -2.96, -46.5},{0.302, -48.4},{3.57, -45.9},{6.83, -39.6},{10.8, -28.1},{21.9,12},{25.8,23.1},{29.7,30.5},{33,33.3},{36.9,32.5},{40.8,27.8},{46,16.9},{56.5, -9.2},{61.7, -18.6},{66.3, -22.7},{70.9, -22.6},{76.1, -18},{80, -12.1}}, color = {0,0,255}, thickness = 0.5),Text(extent = {{ -80,17},{ -57, -3}}, lineColor = {0,0,0}, textString = "offset"),Text(extent = {{ -72, -43},{ -25, -53}}, lineColor = {0,0,0}, textString = "startTime"),Text(extent = {{77, -53},{101, -64}}, lineColor = {0,0,0}, textString = "time"),Text(extent = {{ -89,99},{ -49,82}}, lineColor = {0,0,0}, textString = "y"),Line(points = {{ -50,0},{18,0}}, color = {95,95,95}),Line(points = {{ -50,0},{ -86,0}}, color = {0,0,255}, thickness = 0.5),Line(points = {{ -50,77},{ -50,0}}, color = {95,95,95}),Line(points = {{18, -1},{18,76}}, color = {95,95,95}),Line(points = {{18,73},{ -50,73}}, color = {95,95,95}),Text(extent = {{ -42,83},{9,74}}, lineColor = {0,0,0}, textString = "1/freqHz"),Polygon(points = {{ -49,73},{ -40,75},{ -40,71},{ -49,73}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{18,73},{10,75},{10,71},{18,73}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -50, -61},{ -19, -61}}, color = {95,95,95}),Polygon(points = {{ -18, -61},{ -26, -59},{ -26, -63},{ -18, -61}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Text(extent = {{ -51, -63},{ -27, -75}}, lineColor = {0,0,0}, textString = "t"),Text(extent = {{ -83, -69},{100, -96}}, lineColor = {0,0,0}, textString = "amplitude*exp(-damping*t)*sin(2*pi*freqHz*t+phase)"),Line(points = {{ -50,0},{ -50, -40}}, color = {95,95,95}),Line(points = {{ -50, -54},{ -50, -72}}, color = {95,95,95}),Line(points = {{ -15, -77},{ -1, -48}}, color = {95,95,95})}), Documentation(info = "<html>
<p>
The Real output y is a sine signal with exponentially changing amplitude:
</p>

<p>
<img src=\"../Images/Blocks/Sources/ExpSine.png\">
</p>
</html>"));

      equation 
        y = offset + (if time < startTime then 0 else amplitude * Modelica.Math.exp( -(time - startTime) * damping) * Modelica.Math.sin(2 * pi * freqHz * (time - startTime) + phase));
      end ExpSine;
      model Exponentials "Generate a rising and falling exponential signal"
        parameter Real outMax = 1 "Height of output for infinite riseTime";
        parameter SIunits.Time riseTime(min = 0, start = 0.5) "Rise time";
        parameter SIunits.Time riseTimeConst(min = Modelica.Constants.small) = 0.1 "Rise time constant; rising is defined as outMax*(1-exp(-riseTime/riseTimeConst))";
        parameter SIunits.Time fallTimeConst(min = Modelica.Constants.small) = riseTimeConst "Fall time constant";
        parameter Real offset = 0 "Offset of output signal";
        parameter SIunits.Time startTime = 0 "Output = offset for time < startTime";
        extends Interfaces.SO;
      protected 
        Real y_riseTime;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -90, -70},{68, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -70},{ -77.2, -55.3},{ -74.3, -42.1},{ -70.8, -27.6},{ -67.3, -15},{ -63.7, -4.08},{ -59.5,7.18},{ -55.3,16.7},{ -50.3,26},{ -44.6,34.5},{ -38.3,42.1},{ -31.2,48.6},{ -22.7,54.3},{ -12.1,59.2},{ -10,60},{ -7.88,47.5},{ -5.05,32.7},{ -2.22,19.8},{0.606,8.45},{4.14, -3.7},{7.68, -14},{11.9, -24.2},{16.2, -32.6},{21.1, -40.5},{26.8, -47.4},{33.1, -53.3},{40.9, -58.5},{50.8, -62.8},{60, -65.4}}, color = {0,0,0}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Text(extent = {{ -150, -150},{150, -110}}, lineColor = {0,0,0}, textString = "riseTime=%riseTime")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -90, -74},{84, -74}}, color = {95,95,95}),Polygon(points = {{97, -74},{81, -70},{81, -78},{97, -74}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -40, -34},{ -37.2, -19.3},{ -34.3, -6.1},{ -30.8,8.4},{ -27.3,21},{ -23.7,31.92},{ -19.5,43.18},{ -15.3,52.7},{ -10.3,62},{ -4.6,70.5},{1.7,78.1},{8.8,84.6},{17.3,90.3},{27.9,95.2},{30,96},{32.12,83.5},{34.95,68.7},{37.78,55.8},{40.606,44.45},{44.14,32.3},{47.68,22},{51.9,11.8},{56.2,3.4},{61.1, -4.5},{66.8, -11.4},{73.1, -17.3},{80.9, -22.5},{90.8, -26.8},{100, -29.4}}, color = {0,0,255}, thickness = 0.5),Polygon(points = {{ -80,86},{ -86,64},{ -74,64},{ -80,86}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80,64},{ -80, -84}}, color = {95,95,95}),Text(extent = {{ -81,87},{ -40,67}}, lineColor = {95,95,95}, textString = "y"),Text(extent = {{ -71, -46},{ -38, -55}}, lineColor = {0,0,0}, textString = "offset"),Polygon(points = {{ -40, -74},{ -43, -64},{ -38, -64},{ -40, -74},{ -40, -74}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -40, -33},{ -40, -64}}, color = {95,95,95}),Polygon(points = {{ -40, -34},{ -42, -44},{ -37, -44},{ -40, -34}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -39, -34},{ -80, -34}}, color = {0,0,255}, thickness = 0.5),Text(extent = {{ -62, -76},{ -17, -85}}, lineColor = {0,0,0}, textString = "startTime"),Polygon(points = {{ -41, -34},{ -32, -32},{ -32, -36},{ -41, -34}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -40, -34},{29, -34}}, color = {95,95,95}),Polygon(points = {{29, -34},{21, -32},{21, -36},{29, -34}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Text(extent = {{ -26, -22},{19, -32}}, lineColor = {0,0,0}, textString = "riseTime"),Text(extent = {{75, -79},{98, -90}}, lineColor = {0,0,0}, textString = "time"),Line(points = {{30,96},{30, -38}}, color = {95,95,95})}), Documentation(info = "<html>
<p>
The Real output y is a rising exponential followed
by a falling exponential signal:
</p>

<p>
<img src=\"../Images/Blocks/Sources/Exponentials.png\">
</p>
</html>"));

      equation 
        y_riseTime = outMax * (1 - Modelica.Math.exp( -riseTime / riseTimeConst));
        y = offset + (if time < startTime then 0 else if time < startTime + riseTime then outMax * (1 - Modelica.Math.exp( -(time - startTime) / riseTimeConst)) else y_riseTime * Modelica.Math.exp( -(time - startTime - riseTime) / fallTimeConst));
      end Exponentials;
      block Pulse "Generate pulse signal of type Real"
        parameter Real amplitude = 1 "Amplitude of pulse";
        parameter Real width(final min = Modelica.Constants.small, final max = 100) = 50 "Width of pulse in % of period";
        parameter Modelica.SIunits.Time period(final min = Modelica.Constants.small, start = 1) "Time for one period";
        parameter Real offset = 0 "Offset of output signals";
        parameter Modelica.SIunits.Time startTime = 0 "Output = offset for time < startTime";
        extends Modelica.Blocks.Interfaces.SO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -70},{ -40, -70},{ -40,44},{0,44},{0, -70},{40, -70},{40,44},{79,44}}, color = {0,0,0}),Text(extent = {{ -147, -152},{153, -112}}, lineColor = {0,0,0}, textString = "period=%period")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -80,90},{ -85,68},{ -74,68},{ -80,90}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {95,95,95}),Line(points = {{ -90, -70},{82, -70}}, color = {95,95,95}),Polygon(points = {{90, -70},{68, -65},{68, -76},{90, -70}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{ -34,1},{ -37, -12},{ -30, -12},{ -34,1}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -34, -1},{ -34, -70}}, color = {95,95,95}),Polygon(points = {{ -34, -70},{ -37, -57},{ -31, -57},{ -34, -70},{ -34, -70}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Text(extent = {{ -78, -24},{ -35, -36}}, lineColor = {0,0,0}, textString = "offset"),Text(extent = {{ -30, -72},{16, -81}}, lineColor = {0,0,0}, textString = "startTime"),Text(extent = {{ -82,96},{ -49,79}}, lineColor = {0,0,0}, textString = "y"),Text(extent = {{66, -79},{87, -89}}, lineColor = {0,0,0}, textString = "time"),Line(points = {{ -10,0},{ -10, -70}}, color = {95,95,95}),Line(points = {{ -80,0},{ -10,0},{ -10,50},{30,50},{30,0},{50,0},{50,50},{90,50}}, color = {0,0,255}, thickness = 0.5),Line(points = {{ -10,88},{ -10,49}}, color = {95,95,95}),Line(points = {{30,74},{30,50}}, color = {95,95,95}),Line(points = {{50,88},{50,50}}, color = {95,95,95}),Line(points = {{ -10,83},{51,83}}, color = {95,95,95}),Line(points = {{ -10,69},{30,69}}, color = {95,95,95}),Text(extent = {{ -3,93},{39,84}}, lineColor = {0,0,0}, textString = "period"),Text(extent = {{ -7,78},{30,69}}, lineColor = {0,0,0}, textString = "width"),Line(points = {{ -43,50},{ -10,50}}, color = {95,95,95}),Line(points = {{ -34,50},{ -34,1}}, color = {95,95,95}),Text(extent = {{ -77,30},{ -37,21}}, lineColor = {0,0,0}, textString = "amplitude"),Polygon(points = {{ -34,49},{ -37,36},{ -30,36},{ -34,49}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{ -34,1},{ -37,14},{ -31,14},{ -34,1},{ -34,1}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{90,50},{90,0},{100,0}}, color = {0,0,255}, thickness = 0.5),Polygon(points = {{ -10,69},{ -1,71},{ -1,67},{ -10,69}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{30,69},{22,71},{22,67},{30,69}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{ -10,83},{ -1,85},{ -1,81},{ -10,83}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{50,83},{42,85},{42,81},{50,83}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>
The Real output y is a pulse signal:
</p>

<p>
<img src=\"../Images/Blocks/Sources/Pulse.png\">
</p>
</html>"));
      protected 
        Modelica.SIunits.Time T0(final start = startTime) "Start time of current period";
        Modelica.SIunits.Time T_width = (period * width) / 100;

      equation 
        when sample(startTime, period) then
                  T0 = time;
        
        end when;
        y = offset + (if time < startTime or time >= T0 + T_width then 0 else amplitude);
      end Pulse;
      block SawTooth "Generate saw tooth signal"
        parameter Real amplitude = 1 "Amplitude of saw tooth";
        parameter SIunits.Time period(final min = Modelica.Constants.small, start = 1) "Time for one period";
        parameter Real offset = 0 "Offset of output signals";
        parameter SIunits.Time startTime = 0 "Output = offset for time < startTime";
        extends Interfaces.SO;
      protected 
        SIunits.Time T0(final start = startTime) "Start time of current period";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -70},{ -60, -70},{0,40},{0, -70},{60,41},{60, -70}}, color = {0,0,0}),Text(extent = {{ -147, -152},{153, -112}}, lineColor = {0,0,0}, textString = "period=%period")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -80,90},{ -86,68},{ -74,68},{ -80,90}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {95,95,95}),Line(points = {{ -90, -70},{82, -70}}, color = {95,95,95}),Polygon(points = {{90, -70},{68, -65},{68, -75},{90, -70}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{ -34, -19},{ -37, -32},{ -30, -32},{ -34, -19}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -34, -20},{ -34, -70}}, color = {95,95,95}),Polygon(points = {{ -34, -70},{ -37, -57},{ -31, -57},{ -34, -70},{ -34, -70}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Text(extent = {{ -65, -39},{ -29, -47}}, lineColor = {0,0,0}, textString = "offset"),Text(extent = {{ -29, -72},{13, -80}}, lineColor = {0,0,0}, textString = "startTime"),Text(extent = {{ -82,92},{ -43,76}}, lineColor = {0,0,0}, textString = "y"),Text(extent = {{67, -78},{88, -87}}, lineColor = {0,0,0}, textString = "time"),Line(points = {{ -10, -20},{ -10, -70}}, color = {95,95,95}),Line(points = {{ -10,88},{ -10, -20}}, color = {95,95,95}),Line(points = {{30,88},{30,59}}, color = {95,95,95}),Line(points = {{ -10,83},{30,83}}, color = {95,95,95}),Text(extent = {{ -12,94},{34,85}}, lineColor = {0,0,0}, textString = "period"),Line(points = {{ -44,60},{30,60}}, color = {95,95,95}),Line(points = {{ -34,47},{ -34, -7}}, color = {95,95,95}),Text(extent = {{ -73,25},{ -36,16}}, lineColor = {0,0,0}, textString = "amplitude"),Polygon(points = {{ -34,60},{ -37,47},{ -30,47},{ -34,60}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{ -34, -20},{ -37, -7},{ -31, -7},{ -34, -20},{ -34, -20}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{ -10,83},{ -1,85},{ -1,81},{ -10,83}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{30,83},{22,85},{22,81},{30,83}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -20},{ -10, -20},{30,60},{30, -20},{72,60},{72, -20}}, color = {0,0,255}, thickness = 0.5)}), Documentation(info = "<html>
<p>
The Real output y is a saw tooth signal:
</p>

<p>
<img src=\"../Images/Blocks/Sources/SawTooth.png\">
</p>
</html>"));

      equation 
        when sample(startTime, period) then
                  T0 = time;
        
        end when;
        y = offset + (if time < startTime then 0 else amplitude / period * (time - T0));
      end SawTooth;
      block Trapezoid "Generate trapezoidal signal of type Real"
        parameter Real amplitude = 1 "Amplitude of trapezoid";
        parameter SIunits.Time rising(final min = 0) = 0 "Rising duration of trapezoid";
        parameter SIunits.Time width(final min = 0) = 0.5 "Width duration of trapezoid";
        parameter SIunits.Time falling(final min = 0) = 0 "Falling duration of trapezoid";
        parameter SIunits.Time period(final min = Modelica.Constants.small, start = 1) "Time for one period";
        parameter Integer nperiod =  -1 "Number of periods (< 0 means infinite number of periods)";
        parameter Real offset = 0 "Offset of output signal";
        parameter SIunits.Time startTime = 0 "Output = offset for time < startTime";
        extends Interfaces.SO;
      protected 
        parameter SIunits.Time T_rising = rising "End time of rising phase within one period";
        parameter SIunits.Time T_width = T_rising + width "End time of width phase within one period";
        parameter SIunits.Time T_falling = T_width + falling "End time of falling phase within one period";
        SIunits.Time T0(final start = startTime) "Start time of current period";
        Integer counter(start = nperiod) "Period counter";
        Integer counter2(start = nperiod);
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -147, -152},{153, -112}}, lineColor = {0,0,0}, textString = "period=%period"),Line(points = {{ -81, -70},{ -60, -70},{ -30,40},{9,40},{39, -70},{61, -70},{90,40}}, color = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -81,90},{ -87,68},{ -75,68},{ -81,90}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -81,68},{ -81, -80}}, color = {95,95,95}),Line(points = {{ -91, -70},{81, -70}}, color = {95,95,95}),Polygon(points = {{89, -70},{67, -65},{67, -76},{89, -70}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{ -46, -30},{ -48, -41},{ -44, -41},{ -46, -30}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -46, -31},{ -46, -70}}, color = {95,95,95}, pattern = LinePattern.Solid, arrow = {Arrow.None,Arrow.None}),Polygon(points = {{ -46, -70},{ -48, -60},{ -44, -60},{ -46, -70},{ -46, -70}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Text(extent = {{ -80, -46},{ -42, -55}}, lineColor = {0,0,0}, textString = "offset"),Text(extent = {{ -49, -71},{ -6, -81}}, lineColor = {0,0,0}, textString = "startTime"),Text(extent = {{ -80,95},{ -47,80}}, lineColor = {0,0,0}, textString = "y"),Text(extent = {{66, -78},{89, -89}}, lineColor = {0,0,0}, textString = "time"),Line(points = {{ -30,82},{ -31, -70}}, color = {95,95,95}, pattern = LinePattern.Dash),Line(points = {{ -11,59},{ -11,40}}, color = {95,95,95}, pattern = LinePattern.Dash),Line(points = {{19,59},{19,39}}, color = {95,95,95}, pattern = LinePattern.Dash),Line(points = {{39,59},{39, -30}}, color = {95,95,95}, pattern = LinePattern.Dash),Line(points = {{ -21,76},{60,76}}, color = {95,95,95}),Line(points = {{ -30,56},{39,56}}, color = {95,95,95}),Text(extent = {{ -3,86},{24,77}}, lineColor = {0,0,0}, textString = "period"),Text(extent = {{ -11,68},{18,59}}, lineColor = {0,0,0}, textString = "width"),Line(points = {{ -43,40},{ -11,40}}, color = {95,95,95}, pattern = LinePattern.Dash),Line(points = {{ -40,40},{ -40, -19}}, color = {95,95,95}, pattern = LinePattern.Solid, arrow = {Arrow.None,Arrow.None}),Text(extent = {{ -77,11},{ -44,1}}, lineColor = {0,0,0}, textString = "amplitude"),Polygon(points = {{ -30,56},{ -23,58},{ -23,54},{ -30,56}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{ -11,56},{ -18,58},{ -18,54},{ -11,56}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{ -30,76},{ -21,78},{ -21,74},{ -30,76}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{60,76},{52,78},{52,74},{60,76}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -81, -30},{ -31, -30},{ -11,40},{19,40},{39, -30},{59, -30},{79,40},{99,40}}, color = {0,0,255}, thickness = 0.5),Polygon(points = {{ -40,40},{ -42,29},{ -38,29},{ -40,40}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{ -40, -29},{ -42, -19},{ -38, -19},{ -40, -29},{ -40, -29}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{60,84},{59, -30}}, color = {95,95,95}, pattern = LinePattern.Dash),Polygon(points = {{38,56},{31,58},{31,54},{38,56}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{19,56},{26,58},{26,54},{19,56}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{19,56},{12,58},{12,54},{19,56}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{ -13,56},{ -6,58},{ -6,54},{ -13,56}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Text(extent = {{ -35,68},{ -6,60}}, lineColor = {0,0,0}, textString = "rising"),Text(extent = {{16,68},{44,60}}, lineColor = {0,0,0}, textString = "falling")}), Documentation(info = "<html>
<p>
The Real output y is a trapezoid signal:
</p>

<p>
<img src=\"../Images/Blocks/Sources/Trapezoid.png\">
</p>
</html>"));

      equation 
        when pre(counter2) <> 0 and sample(startTime, period) then
                  T0 = time;
          counter2 = pre(counter);
          counter = pre(counter) - (if pre(counter) > 0 then 1 else 0);
        
        end when;
        y = offset + (if time < startTime or counter2 == 0 or time >= T0 + T_falling then 0 else if time < T0 + T_rising then ((time - T0) * amplitude) / T_rising else if time < T0 + T_width then amplitude else ((T0 + T_falling - time) * amplitude) / (T_falling - T_width));
      end Trapezoid;
      block KinematicPTP "Move as fast as possible along a distance within given kinematic constraints"
        parameter Real deltaq[:] "Distance to move";
        parameter Real qd_max[:](each final min = Modelica.Constants.small) "Maximum velocities der(q)";
        parameter Real qdd_max[:](each final min = Modelica.Constants.small) "Maximum accelerations der(qd)";
        parameter SIunits.Time startTime = 0 "Time instant at which movement starts";
        extends Interfaces.MO(final nout = max([size(deltaq, 1);size(qd_max, 1);size(qdd_max, 1)]));
      protected 
        parameter Real p_deltaq[nout] = if size(deltaq, 1) == 1 then ones(nout) * deltaq[1] else deltaq;
        parameter Real p_qd_max[nout] = if size(qd_max, 1) == 1 then ones(nout) * qd_max[1] else qd_max;
        parameter Real p_qdd_max[nout] = if size(qdd_max, 1) == 1 then ones(nout) * qdd_max[1] else qdd_max;
        Real sd_max;
        Real sdd_max;
        Real sdd;
        Real aux1[nout];
        Real aux2[nout];
        SIunits.Time Ta1;
        SIunits.Time Ta2;
        SIunits.Time Tv;
        SIunits.Time Te;
        Boolean noWphase;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80,78},{ -80, -82}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,88},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{82,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,0},{ -70,0},{ -70,70},{ -30,70},{ -30,0},{20,0},{20, -70},{60, -70},{60,0},{68,0}}, color = {0,0,0}, thickness = 0.25),Text(extent = {{2,80},{80,20}}, lineColor = {192,192,192}, textString = "acc"),Text(extent = {{ -150, -150},{150, -110}}, lineColor = {0,0,0}, textString = "deltaq=%deltaq")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80,78},{ -80, -72}}, color = {95,95,95}),Polygon(points = {{ -80,91},{ -86,71},{ -75,71},{ -80,91},{ -80,91}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{82,0}}, color = {95,95,95}),Polygon(points = {{89,0},{68,5},{68, -5},{89,0}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80,0},{ -70,0},{ -70,70},{ -30,70},{ -30,0},{20,0},{20, -70},{60, -70},{60,0},{68,0}}, color = {0,0,255}, thickness = 0.5),Text(extent = {{ -73,95},{ -16,80}}, lineColor = {0,0,0}, textString = "acceleration"),Text(extent = {{66,20},{88,8}}, lineColor = {0,0,0}, textString = "time")}), Documentation(info = "<HTML>
<p>
The goal is to move as <b>fast</b> as possible along a distance
<b>deltaq</b>
under given <b>kinematical constraints</b>. The distance can be a positional or
angular range. In robotics such a movement is called <b>PTP</b> (Point-To-Point).
This source block generates the <b>acceleration</b> qdd of this signal
as output:
</p>

<p>
<img src=\"../Images/Blocks/Sources/KinematicPTP.png\">
</p>

<p>
After integrating the output two times, the position q is
obtained. The signal is constructed in such a way that it is not possible
to move faster, given the <b>maximally</b> allowed <b>velocity</b> qd_max and
the <b>maximally</b> allowed <b>acceleration</b> qdd_max.
</p>
<p>
If several distances are given (vector deltaq has more than 1 element),
an acceleration output vector is constructed such that all signals
are in the same periods in the acceleration, constant velocity
and deceleration phase. This means that only one of the signals
is at its limits whereas the others are sychnronized in such a way
that the end point is reached at the same time instant.
</p>

<p>
This element is useful to generate a reference signal for a controller
which controls a drive train or in combination with model
Modelica.Mechanics.Rotational.<b>Accelerate</b> to drive
a flange according to a given acceleration.
</p>

</HTML>
", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>June 27, 2001</i>
       by Bernhard Bachmann.<br>
       Bug fixed that element is also correct if startTime is not zero.</li>
<li><i>Nov. 3, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Vectorized and moved from Rotational to Blocks.Sources.</li>
<li><i>June 29, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       realized.</li>
</ul>
</html>"));

      equation 
        for i in 1:nout loop
        aux1[i] = p_deltaq[i] / p_qd_max[i];
        aux2[i] = p_deltaq[i] / p_qdd_max[i];

        end for;
        sd_max = 1 / max(abs(aux1));
        sdd_max = 1 / max(abs(aux2));
        Ta1 = sqrt(1 / sdd_max);
        Ta2 = sd_max / sdd_max;
        noWphase = Ta2 >= Ta1;
        Tv = if noWphase then Ta1 else 1 / sd_max;
        Te = if noWphase then Ta1 + Ta1 else Tv + Ta2;
        sdd = if time < startTime then 0 else if noWphase then if time < Ta1 + startTime then sdd_max else if time < Te + startTime then  -sdd_max else 0 else if time < Ta2 + startTime then sdd_max else if time < Tv + startTime then 0 else if time < Te + startTime then  -sdd_max else 0;
        y = p_deltaq * sdd;
      end KinematicPTP;
      block KinematicPTP2 "Move as fast as possible from start to end position within given kinematic constraints with output signals q, qd=der(q), qdd=der(qd)"
        import SI = Modelica.SIunits;
        parameter Real q_begin[:] = {0} "Start position";
        parameter Real q_end[:] "End position";
        parameter Real qd_max[:](each final min = Modelica.Constants.small) "Maximum velocities der(q)";
        parameter Real qdd_max[:](each final min = Modelica.Constants.small) "Maximum accelerations der(qd)";
        parameter Modelica.SIunits.Time startTime = 0 "Time instant at which movement starts";
        extends Modelica.Blocks.Interfaces.BlockIcon;
        final parameter Integer nout = max([size(q_begin, 1);size(q_end, 1);size(qd_max, 1);size(qdd_max, 1)]) "Number of output signals (= dimension of q, qd, qdd, moving)";
        output Modelica.SIunits.Time endTime "Time instant at which movement stops";
        Modelica.Blocks.Interfaces.RealOutput q[nout] "Reference position of path planning" annotation(Placement(transformation(extent = {{100,70},{120,90}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput qd[nout] "Reference speed of path planning" annotation(Placement(transformation(extent = {{100,20},{120,40}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput qdd[nout] "Reference acceleration of path planning" annotation(Placement(transformation(extent = {{100, -40},{120, -20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.BooleanOutput moving[nout] "= true, if end position not yet reached; = false, if end position reached or axis is completely at rest" annotation(Placement(transformation(extent = {{100, -90},{120, -70}}, rotation = 0)));
        annotation(defaultComponentName = "kinematicPTP", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -80,78},{ -80, -82}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,88},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{17,0}}, color = {192,192,192}),Line(points = {{ -80,0},{ -70,0},{ -70,70},{ -50,70},{ -50,0},{ -15,0},{ -15, -70},{5, -70},{5,0},{18,0}}, color = {0,0,0}, thickness = 0.25),Text(extent = {{34,96},{94,66}}, lineColor = {0,0,0}, textString = "q"),Text(extent = {{40,44},{96,14}}, lineColor = {0,0,0}, textString = "qd"),Text(extent = {{32, -18},{99, -44}}, lineColor = {0,0,0}, textString = "qdd"),Text(extent = {{ -32, -74},{97, -96}}, lineColor = {0,0,0}, textString = "moving")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -80,78},{ -80, -70}}, color = {95,95,95}),Polygon(points = {{ -80,94},{ -86,74},{ -74,74},{ -80,94},{ -80,94}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{82,0}}, color = {95,95,95}),Polygon(points = {{90,0},{68,6},{68, -6},{90,0}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80,0},{ -70,0},{ -70,70},{ -30,70},{ -30,0},{20,0},{20, -70},{60, -70},{60,0},{68,0}}, color = {0,0,255}, thickness = 0.5),Text(extent = {{ -72,96},{ -15,81}}, lineColor = {0,0,0}, textString = "acceleration"),Text(extent = {{69,18},{91,6}}, lineColor = {0,0,0}, textString = "time")}), Documentation(info = "<html>
<p>
The goal is to move as <b>fast</b> as possible from start position <b>q_begin</b>
to end position <b>q_end</b>
under given <b>kinematical constraints</b>. The positions can be translational or
rotational definitions (i.e., q_begin/q_end is given). In robotics such a movement is called <b>PTP</b> (Point-To-Point).
This source block generates the <b>position</b> q(t), the
<b>speed</b> qd(t) = der(q), and the <b>acceleration</b> qdd = der(qd)
as output. The signals are constructed in such a way that it is not possible
to move faster, given the <b>maximally</b> allowed <b>velocity</b> qd_max and
the <b>maximally</b> allowed <b>acceleration</b> qdd_max:
</p>

<p>
<img src=\"../Images/Blocks/Sources/KinematicPTP2.png\">
</p>

<p>
If vectors q_begin/q_end have more than 1 element,
the output vectors are constructed such that all signals
are in the same periods in the acceleration, constant velocity
and deceleration phase. This means that only one of the signals
is at its limits whereas the others are sychnronized in such a way
that the end point is reached at the same time instant.
</p>

<p>
This element is useful to generate a reference signal for a controller
which controls, e.g., a drive train, or to drive
a flange according to a given acceleration.
</p>

</html>
", revisions = "<html>
<ul>
<li><i>March 24, 2007</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Non-standard Modelica function \"constrain(..)\" replaced by standard
       Modelica implementation (via internal function position()).<br>
       New output signal \"moving\" added.</li>
<li><i>June 27, 2001</i>
       by Bernhard Bachmann.<br>
       Bug fixed that element is also correct if startTime is not zero.</li>
<li><i>Nov. 3, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Vectorized and moved from Rotational to Blocks.Sources.</li>
<li><i>June 29, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       realized.</li>
</ul>
</html>"));
      protected 
        parameter Real p_q_begin[nout] = if size(q_begin, 1) == 1 then ones(nout) * q_begin[1] else q_begin;
        parameter Real p_q_end[nout] = if size(q_end, 1) == 1 then ones(nout) * q_end[1] else q_end;
        parameter Real p_qd_max[nout] = if size(qd_max, 1) == 1 then ones(nout) * qd_max[1] else qd_max;
        parameter Real p_qdd_max[nout] = if size(qdd_max, 1) == 1 then ones(nout) * qdd_max[1] else qdd_max;
        parameter Real p_deltaq[nout] = p_q_end - p_q_begin;
        constant Real eps = 10 * Modelica.Constants.eps;
        Boolean motion_ref;
        Real sd_max_inv;
        Real sdd_max_inv;
        Real sd_max;
        Real sdd_max;
        Real sdd;
        Real aux1[nout];
        Real aux2[nout];
        SI.Time Ta1;
        SI.Time Ta2;
        SI.Time Tv;
        SI.Time Te;
        Boolean noWphase;
        SI.Time Ta1s;
        SI.Time Ta2s;
        SI.Time Tvs;
        SI.Time Tes;
        Real sd_max2;
        Real s1;
        Real s2;
        Real s3;
        Real s;
        Real sd;
        Real r_s;
        Real r_sd;
        Real r_sdd;
        function position
          input Real q_qd_qdd[3] "Required values for position, speed, acceleration";
          input Real dummy "Just to have one input signal that should be differentiated to avoid possible problems in the Modelica tool (is not used)";
          output Real q;
          annotation(derivative(noDerivative = q_qd_qdd) = position_der, __Dymola_InlineAfterIndexReduction = true);
        algorithm 
          q:=q_qd_qdd[1];
        end position;
        function position_der
          input Real q_qd_qdd[3] "Required values for position, speed, acceleration";
          input Real dummy "Just to have one input signal that should be differentiated to avoid possible problems in the Modelica tool (is not used)";
          input Real dummy_der;
          output Real qd;
          annotation(derivative(noDerivative = q_qd_qdd) = position_der2, __Dymola_InlineAfterIndexReduction = true);
        algorithm 
          qd:=q_qd_qdd[2];
        end position_der;
        function position_der2
          input Real q_qd_qdd[3] "Required values for position, speed, acceleration";
          input Real dummy "Just to have one input signal that should be differentiated to avoid possible problems in the Modelica tool (is not used)";
          input Real dummy_der;
          input Real dummy_der2;
          output Real qdd;
        algorithm 
          qdd:=q_qd_qdd[3];
        end position_der2;

      equation 
        for i in 1:nout loop
        aux1[i] = p_deltaq[i] / p_qd_max[i];
        aux2[i] = p_deltaq[i] / p_qdd_max[i];

        end for;
        sd_max_inv = max(abs(aux1));
        sdd_max_inv = max(abs(aux2));
        if sd_max_inv <= eps or sdd_max_inv <= eps then
          sd_max = 0;
          sdd_max = 0;
          Ta1 = 0;
          Ta2 = 0;
          noWphase = false;
          Tv = 0;
          Te = 0;
          Ta1s = 0;
          Ta2s = 0;
          Tvs = 0;
          Tes = 0;
          sd_max2 = 0;
          s1 = 0;
          s2 = 0;
          s3 = 0;
          r_sdd = 0;
          r_sd = 0;
          r_s = 0;

        else
          sd_max = 1 / max(abs(aux1));
          sdd_max = 1 / max(abs(aux2));
          Ta1 = sqrt(1 / sdd_max);
          Ta2 = sd_max / sdd_max;
          noWphase = Ta2 >= Ta1;
          Tv = if noWphase then Ta1 else 1 / sd_max;
          Te = if noWphase then Ta1 + Ta1 else Tv + Ta2;
          Ta1s = Ta1 + startTime;
          Ta2s = Ta2 + startTime;
          Tvs = Tv + startTime;
          Tes = Te + startTime;
          sd_max2 = sdd_max * Ta1;
          s1 = (sdd_max * (if noWphase then Ta1 * Ta1 else Ta2 * Ta2)) / 2;
          s2 = s1 + (if noWphase then sd_max2 * (Te - Ta1) - sdd_max / 2 * (Te - Ta1) ^ 2 else sd_max * (Tv - Ta2));
          s3 = s2 + sd_max * (Te - Tv) - sdd_max / 2 * (Te - Tv) * (Te - Tv);
          if time < startTime then
            r_sdd = 0;
            r_sd = 0;
            r_s = 0;

          elseif noWphase then
            if time < Ta1s then
              r_sdd = sdd_max;
              r_sd = sdd_max * (time - startTime);
              r_s = sdd_max / 2 * (time - startTime) * (time - startTime);

            elseif time < Tes then
              r_sdd =  -sdd_max;
              r_sd = sd_max2 - sdd_max * (time - Ta1s);
              r_s = s1 + sd_max2 * (time - Ta1s) - sdd_max / 2 * (time - Ta1s) * (time - Ta1s);


            else
              r_sdd = 0;
              r_sd = 0;
              r_s = s2;

            end if;


          elseif time < Ta2s then
            r_sdd = sdd_max;
            r_sd = sdd_max * (time - startTime);
            r_s = sdd_max / 2 * (time - startTime) * (time - startTime);


          elseif time < Tvs then
            r_sdd = 0;
            r_sd = sd_max;
            r_s = s1 + sd_max * (time - Ta2s);


          elseif time < Tes then
            r_sdd =  -sdd_max;
            r_sd = sd_max - sdd_max * (time - Tvs);
            r_s = s2 + sd_max * (time - Tvs) - sdd_max / 2 * (time - Tvs) * (time - Tvs);


          else
            r_sdd = 0;
            r_sd = 0;
            r_s = s3;

          end if;

        end if;
        qdd = p_deltaq * sdd;
        qd = p_deltaq * sd;
        q = p_q_begin + p_deltaq * s;
        endTime = Tes;
        s = position({r_s,r_sd,r_sdd}, time);
        sd = der(s);
        sdd = der(sd);
        motion_ref = time <= endTime;
        for i in 1:nout loop
        moving[i] = if abs(q_begin[i] - q_end[i]) > eps then motion_ref else false;

        end for;
      end KinematicPTP2;
      block TimeTable "Generate a (possibly discontinuous) signal by linear interpolation in a table"
        parameter Real table[:,2] "Table matrix (time = first column; e.g. table=[0, 0; 1, 1; 2, 4])";
        parameter Real offset = 0 "Offset of output signal";
        parameter SIunits.Time startTime = 0 "Output = offset for time < startTime";
        extends Interfaces.SO;
      protected 
        Real a "Interpolation coefficients a of actual interval (y=a*x+b)";
        Real b "Interpolation coefficients b of actual interval (y=a*x+b)";
        Integer last(start = 1) "Last used lower grid index";
        SIunits.Time nextEvent(start = 0, fixed = true) "Next event instant";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -48,70},{2, -50}}, lineColor = {255,255,255}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -48, -50},{ -48,70},{52,70},{52, -50},{ -48, -50},{ -48, -20},{52, -20},{52,10},{ -48,10},{ -48,40},{52,40},{52,70},{2,70},{2, -51}}, color = {0,0,0}),Text(extent = {{ -150, -150},{150, -110}}, lineColor = {0,0,0}, textString = "offset=%offset")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -80,90},{ -85,68},{ -74,68},{ -80,90}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {95,95,95}),Line(points = {{ -90, -70},{82, -70}}, color = {95,95,95}),Polygon(points = {{88, -70},{68, -65},{68, -74},{88, -70}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -20,90},{30, -30}}, lineColor = {255,255,255}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -20, -30},{ -20,90},{80,90},{80, -30},{ -20, -30},{ -20,0},{80,0},{80,30},{ -20,30},{ -20,60},{80,60},{80,90},{30,90},{30, -31}}, color = {0,0,0}),Text(extent = {{ -70, -42},{ -32, -54}}, lineColor = {0,0,0}, textString = "offset"),Polygon(points = {{ -31, -30},{ -33, -40},{ -28, -40},{ -31, -30}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{ -31, -70},{ -34, -60},{ -29, -60},{ -31, -70},{ -31, -70}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -31, -32},{ -31, -70}}, color = {95,95,95}),Line(points = {{ -20, -30},{ -20, -70}}, color = {95,95,95}),Text(extent = {{ -38, -73},{8, -83}}, lineColor = {0,0,0}, textString = "startTime"),Line(points = {{ -20, -30},{ -80, -30}}, color = {95,95,95}),Text(extent = {{ -76,93},{ -44,75}}, lineColor = {0,0,0}, textString = "y"),Text(extent = {{66, -78},{90, -88}}, lineColor = {0,0,0}, textString = "time"),Text(extent = {{ -15,83},{24,68}}, lineColor = {0,0,0}, textString = "time"),Text(extent = {{33,83},{76,67}}, lineColor = {0,0,0}, textString = "y")}), Documentation(info = "<HTML>
<p>
This block generates an output signal by <b>linear interpolation</b> in
a table. The time points and function values are stored in a matrix
<b>table[i,j]</b>, where the first column table[:,1] contains the
time points and the second column contains the data to be interpolated.
The table interpolation has the following proporties:
</p>
<ul>
<li>The time points need to be <b>monotonically increasing</b>. </li>
<li><b>Discontinuities</b> are allowed, by providing the same
    time point twice in the table. </li>
<li>Values <b>outside</b> of the table range, are computed by
    <b>extrapolation</b> through the last or first two points of the
    table.</li>
<li>If the table has only <b>one row</b>, no interpolation is performed and
    the function value is just returned independantly of the
    actual time instant.</li>
<li>Via parameters <b>startTime</b> and <b>offset</b> the curve defined
    by the table can be shifted both in time and in the ordinate value.
<li>The table is implemented in a numerically sound way by
    generating <b>time events</b> at interval boundaries,
    in order to not integrate over a discontinuous or not differentiable
    points.
</li>
</ul>
<p>
Example:
</p>
<pre>
   table = [0  0
            1  0
            1  1
            2  4
            3  9
            4 16]
If, e.g., time = 1.0, the output y =  0.0 (before event), 1.0 (after event)
    e.g., time = 1.5, the output y =  2.5,
    e.g., time = 2.0, the output y =  4.0,
    e.g., time = 5.0, the output y = 23.0 (i.e. extrapolation).
</pre>


<p>
<img src=\"../Images/Blocks/Sources/TimeTable.png\">
</p>

</HTML>
", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>Oct. 21, 2002</i>
       by <a href=\"http://www.robotic.dlr.de/Christian.Schweiger/\">Christian Schweiger</a>:<br>
       Corrected interface from
<pre>
    parameter Real table[:, :]=[0, 0; 1, 1; 2, 4];
</pre>
       to
<pre>
    parameter Real table[:, <b>2</b>]=[0, 0; 1, 1; 2, 4];
</pre>
       </li>
<li><i>Nov. 7, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized.</li>
</ul>
</html>"));
        function getInterpolationCoefficients "Determine interpolation coefficients and next time event"
          input Real table[:,2] "Table for interpolation";
          input Real offset "y-offset";
          input Real startTime "time-offset";
          input Real t "Actual time instant";
          input Integer last "Last used lower grid index";
          input Real TimeEps "Relative epsilon to check for identical time instants";
          output Real a "Interpolation coefficients a (y=a*x + b)";
          output Real b "Interpolation coefficients b (y=a*x + b)";
          output Real nextEvent "Next event instant";
          output Integer next "New lower grid index";
        protected 
          Integer columns = 2 "Column to be interpolated";
          Integer ncol = 2 "Number of columns to be interpolated";
          Integer nrow = size(table, 1) "Number of table rows";
          Integer next0;
          Real tp;
          Real dt;
        algorithm 
          next:=last;
          nextEvent:=t - TimeEps * abs(t);
          tp:=t + TimeEps * abs(t) - startTime;
          if tp < 0.0 then 
                    nextEvent:=startTime;
          a:=0;
          b:=offset;
          elseif nrow < 2 then
            a:=0;
            b:=offset + table[1,columns];


          else           while (next < nrow and tp >= table[next,1]) loop
                      next:=next + 1;

          end while;
          if next < nrow then 
                    nextEvent:=startTime + table[next,1];

          else 
          end if;
          next0:=next - 1;
          dt:=table[next,1] - table[next0,1];
          if dt <= TimeEps * abs(table[next,1]) then 
                    a:=0;
          b:=offset + table[next,columns];

          else           a:=(table[next,columns] - table[next0,columns]) / dt;
          b:=offset + table[next0,columns] - a * table[next0,1];

          end if;

          end if;
          b:=b - a * startTime;
        end getInterpolationCoefficients;
      algorithm 
        when {time >= pre(nextEvent),initial()} then
                  (a,b,nextEvent,last):=getInterpolationCoefficients(table, offset, startTime, time, last, 100 * Modelica.Constants.eps);
        
        end when;

      equation 
        y = a * time + b;
      end TimeTable;
      model CombiTimeTable "Table look-up with respect to time and linear/perodic extrapolation methods (data from matrix/file)"
        parameter Boolean tableOnFile = false "= true, if table is defined on file or in function usertab" annotation(Dialog(group = "table data definition"));
        parameter Real table[:,:] = fill(0.0, 0, 2) "Table matrix (time = first column; e.g. table=[0,2])" annotation(Dialog(group = "table data definition", enable = not tableOnFile));
        parameter String tableName = "NoName" "Table name on file or in function usertab (see docu)" annotation(Dialog(group = "table data definition", enable = tableOnFile));
        parameter String fileName = "NoName" "File where matrix is stored" annotation(Dialog(group = "table data definition", enable = tableOnFile, __Dymola_loadSelector(filter = "Text files (*.txt);;Matlab files (*.mat)", caption = "Open file in which table is present")));
        parameter Integer columns[:] = 2:size(table, 2) "Columns of table to be interpolated" annotation(Dialog(group = "table data interpretation"));
        parameter Modelica.Blocks.Types.Smoothness smoothness = Modelica.Blocks.Types.Smoothness.LinearSegments "Smoothness of table interpolation" annotation(Dialog(group = "table data interpretation"));
        parameter Modelica.Blocks.Types.Extrapolation extrapolation = Modelica.Blocks.Types.Extrapolation.LastTwoPoints "Extrapolation of data outside the definition range" annotation(Dialog(group = "table data interpretation"));
        parameter Real offset[:] = {0} "Offsets of output signals" annotation(Dialog(group = "table data interpretation"));
        parameter Modelica.SIunits.Time startTime = 0 "Output = offset for time < startTime" annotation(Dialog(group = "table data interpretation"));
        extends Modelica.Blocks.Interfaces.MO(final nout = max([size(columns, 1);size(offset, 1)]));
        final parameter Real t_min(fixed = false) "Minimum abscissa value defined in table";
        final parameter Real t_max(fixed = false) "Maximum abscissa value defined in table";
        annotation(Documentation(info = "<HTML>
<p>
This block generates an output signal y[:] by <b>linear interpolation</b> in
a table. The time points and function values are stored in a matrix
<b>table[i,j]</b>, where the first column table[:,1] contains the
time points and the other columns contain the data to be interpolated.
</p>

<p>
<img src=\"../Images/Blocks/Sources/CombiTimeTable.png\">
</p>

<p>
Via parameter <b>columns</b> it can be defined which columns of the
table are interpolated. If, e.g., columns={2,4}, it is assumed that
2 output signals are present and that the first output is computed
by interpolation of column 2 and the second output is computed
by interpolation of column 4 of the table matrix.
The table interpolation has the following properties:
</p>
<ul>
<li>The time points need to be <b>monotonically increasing</b>. </li>
<li><b>Discontinuities</b> are allowed, by providing the same
    time point twice in the table. </li>
<li>Values <b>outside</b> of the table range, are computed by
    extrapolation according to the setting of parameter
    <b>extrapolation</b>:
<pre>
  extrapolation = 0: hold the first or last value of the table,
                     if outside of the range.
                = 1: extrapolate through the last or first two
                     points of the table.
                = 2: periodically repeat the table data
                     (periodical function).
</pre></li>
<li>Via parameter <b>smoothness</b> it is defined how the data is interpolated:
<pre>
  smoothness = 0: linear interpolation
             = 1: smooth interpolation with Akima Splines such
                  that der(y) is continuous.
</pre></li>
<li>If the table has only <b>one row</b>, no interpolation is performed and
    the table values of this row are just returned.</li>
<li>Via parameters <b>startTime</b> and <b>offset</b> the curve defined
    by the table can be shifted both in time and in the ordinate value.
    The time instants stored in the table are therefore <b>relative</b>
    to <b>startTime</b>.
    If time &lt; startTime, no interpolation is performed and the offset
    is used as ordinate value for all outputs.
<li>The table is implemented in a numerically sound way by
    generating <b>time events</b> at interval boundaries,
    in order to not integrate over a discontinuous or not differentiable
    points.
<li>For special applications it is sometimes needed to know the minimum
    and maximum time instant defined in the table as a parameter. For this
    reason parameters <b>t_min</b> and <b>t_max</b> are provided and can be access from
    the outside of the table object.
</li>
</ul>
<p>
Example:
</p>
<pre>
   table = [0  0
            1  0
            1  1
            2  4
            3  9
            4 16]; extrapolation = 1 (default)
If, e.g., time = 1.0, the output y =  0.0 (before event), 1.0 (after event)
    e.g., time = 1.5, the output y =  2.5,
    e.g., time = 2.0, the output y =  4.0,
    e.g., time = 5.0, the output y = 23.0 (i.e. extrapolation via last 2 points).
</pre>
<p>
The table matrix can be defined in the following ways:
</p>
<ol>
<li> Explicitly supplied as <b>parameter matrix</b> \"table\",
     and the other parameters have the following values:
<pre>
   tableName is \"NoName\" or has only blanks,
   fileName  is \"NoName\" or has only blanks.
</pre></li>
<li> <b>Read</b> from a <b>file</b> \"fileName\" where the matrix is stored as
      \"tableName\". Both ASCII and binary file format is possible.
      (the ASCII format is described below).
      It is most convenient to generate the binary file from Matlab
      (Matlab 4 storage format), e.g., by command
<pre>
   save tables.mat tab1 tab2 tab3 -V4
</pre>
      when the three tables tab1, tab2, tab3 should be
      used from the model.</li>
<li>  Statically stored in function \"usertab\" in file \"usertab.c\".
      The matrix is identified by \"tableName\". Parameter
      fileName = \"NoName\" or has only blanks.</li>
</ol>
<p>
Table definition methods (1) and (3) do <b>not</b> allocate dynamic memory,
and do not access files, whereas method (2) does. Therefore (1) and (3)
are suited for hardware-in-the-loop simulation (e.g. with dSpace hardware).
When the constant \"NO_FILE\" is defined in \"usertab.c\", all parts of the
source code of method (2) are removed by the C-preprocessor, such that
no dynamic memory allocation and no access to files takes place.
</p>
<p>
If tables are read from an ASCII-file, the file need to have the
following structure (\"-----\" is not part of the file content):
</p>
<pre>
-----------------------------------------------------
#1
double tab1(6,2)   # comment line
  0   0
  1   0
  1   1
  2   4
  3   9
  4  16
double tab2(6,2)   # another comment line
  0   0
  2   0
  2   2
  4   8
  6  18
  8  32
-----------------------------------------------------
</pre>
<p>
Note, that the first two characters in the file need to be
\"#1\". Afterwards, the corresponding matrix has to be declared
with type, name and actual dimensions. Finally, in successive
rows of the file, the elements of the matrix have to be given.
Several matrices may be defined one after another.
</p>

</HTML>
", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>March 31, 2001</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Used CombiTableTime as a basis and added the
       arguments <b>extrapolation, columns, startTime</b>.
       This allows periodic function definitions. </li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -48,70},{2, -50}}, lineColor = {255,255,255}, fillColor = {255,255,0}, fillPattern = FillPattern.Solid),Line(points = {{ -48, -50},{ -48,70},{52,70},{52, -50},{ -48, -50},{ -48, -20},{52, -20},{52,10},{ -48,10},{ -48,40},{52,40},{52,70},{2,70},{2, -51}}, color = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {95,95,95}),Line(points = {{ -90, -70},{82, -70}}, color = {95,95,95}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -20,90},{20, -30}}, lineColor = {255,255,255}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -20, -30},{ -20,90},{80,90},{80, -30},{ -20, -30},{ -20,0},{80,0},{80,30},{ -20,30},{ -20,60},{80,60},{80,90},{20,90},{20, -30}}, color = {0,0,0}),Text(extent = {{ -71, -42},{ -32, -54}}, lineColor = {0,0,0}, textString = "offset"),Polygon(points = {{ -31, -30},{ -33, -40},{ -28, -40},{ -31, -30}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{ -31, -70},{ -34, -60},{ -29, -60},{ -31, -70},{ -31, -70}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -31, -31},{ -31, -70}}, color = {95,95,95}),Line(points = {{ -20, -30},{ -20, -70}}, color = {95,95,95}),Text(extent = {{ -42, -74},{6, -84}}, lineColor = {0,0,0}, textString = "startTime"),Line(points = {{ -20, -30},{ -80, -30}}, color = {95,95,95}),Text(extent = {{ -73,93},{ -44,74}}, lineColor = {0,0,0}, textString = "y"),Text(extent = {{66, -81},{92, -92}}, lineColor = {0,0,0}, textString = "time"),Text(extent = {{ -19,83},{20,68}}, lineColor = {0,0,0}, textString = "time"),Text(extent = {{21,82},{50,68}}, lineColor = {0,0,0}, textString = "y[1]"),Line(points = {{50,90},{50, -30}}, color = {0,0,0}),Line(points = {{80,0},{100,0}}, color = {0,0,255}),Text(extent = {{34, -30},{71, -42}}, textString = "columns", lineColor = {0,0,255}),Text(extent = {{51,82},{80,68}}, lineColor = {0,0,0}, textString = "y[2]")}));
      protected 
        final parameter Real p_offset[nout] = if size(offset, 1) == 1 then ones(nout) * offset[1] else offset;
        Integer tableID;
        function tableTimeInit "Initialize 1-dim. table where first column is time (for details see: Modelica/C-Sources/ModelicaTables.h)"
          input String tableName;
          input String fileName;
          input Real table[:,:];
          input Real startTime;
          input Modelica.Blocks.Types.Smoothness smoothness;
          input Modelica.Blocks.Types.Extrapolation extrapolation;
          output Integer tableID;

          external "C" tableID = ModelicaTables_CombiTimeTable_init(tableName,fileName,table,size(table, 1),size(table, 2),startTime,smoothness,extrapolation) ;          annotation(Library = "ModelicaExternalC");
        end tableTimeInit;
        function tableTimeIpo "Interpolate 1-dim. table where first column is time (for details see: Modelica/C-Sources/ModelicaTables.h)"
          input Integer tableID;
          input Integer icol;
          input Real timeIn;
          output Real value;

          external "C" value = ModelicaTables_CombiTimeTable_interpolate(tableID,icol,timeIn) ;          annotation(Library = "ModelicaExternalC");
        end tableTimeIpo;
        function tableTimeTmin "Return minimum time value of 1-dim. table where first column is time (for details see: Modelica/C-Sources/ModelicaTables.h)"
          input Integer tableID;
          output Real Tmin "minimum time value in table";

          external "C" Tmin = ModelicaTables_CombiTimeTable_minimumTime(tableID) ;          annotation(Library = "ModelicaExternalC");
        end tableTimeTmin;
        function tableTimeTmax "Return maximum time value of 1-dim. table where first column is time (for details see: Modelica/C-Sources/ModelicaTables.h)"
          input Integer tableID;
          output Real Tmax "maximum time value in table";

          external "C" Tmax = ModelicaTables_CombiTimeTable_maximumTime(tableID) ;          annotation(Library = "ModelicaExternalC");
        end tableTimeTmax;

      equation 
if tableOnFile then
                  assert(tableName <> "NoName", "tableOnFile = true and no table name given");
        end if;
if not tableOnFile then
                  assert(size(table, 1) > 0 and size(table, 2) > 0, "tableOnFile = false and parameter table is an empty matrix");
        end if;
        for i in 1:nout loop
        y[i] = p_offset[i] + tableTimeIpo(tableID, columns[i], time);

        end for;
        when initial() then
                  tableID = tableTimeInit(if not tableOnFile then "NoName" else tableName, if not tableOnFile then "NoName" else fileName, table, startTime, smoothness, extrapolation);
        
        end when;

      initial equation 
        t_min = tableTimeTmin(tableID);
        t_max = tableTimeTmax(tableID);
      end CombiTimeTable;
      block BooleanConstant "Generate constant signal of type Boolean"
        parameter Boolean k = true "Constant output value";
        extends Interfaces.partialBooleanSource;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -80,0},{80,0}}, color = {0,0,0}),Text(extent = {{ -150, -140},{150, -110}}, lineColor = {0,0,0}, textString = "%k")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -70,0},{80,0}}, color = {0,0,255}, thickness = 0.5),Text(extent = {{ -69,20},{ -49,0}}, lineColor = {0,0,0}, textString = "k"),Text(extent = {{ -96,6},{ -76, -4}}, lineColor = {0,0,0}, textString = "true"),Text(extent = {{ -98, -58},{ -72, -68}}, lineColor = {0,0,0}, textString = "false")}), Documentation(info = "<html>
<p>
The Boolean output y is a constant signal:
</p>

<p>
<img src=\"../Images/Blocks/Sources/BooleanConstant.png\">
</p>
</html>"));

      equation 
        y = k;
      end BooleanConstant;
      block BooleanStep "Generate step signal of type Boolean"
        parameter Modelica.SIunits.Time startTime = 0 "Time instant of step start";
        parameter Boolean startValue = false "Output before startTime";
        extends Interfaces.partialBooleanSource;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -80, -70},{0, -70},{0,50},{80,50}}, color = {0,0,0}),Text(extent = {{ -150, -140},{150, -110}}, lineColor = {0,0,0}, textString = "%startTime")}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -80, -70},{0, -70},{0,50},{80,50}}, color = {0,0,255}, thickness = 0.5),Text(extent = {{ -15, -74},{20, -82}}, lineColor = {0,0,0}, textString = "startTime"),Polygon(points = {{2,50},{ -80,50},{2,50}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Text(extent = {{ -66,62},{ -22,48}}, lineColor = {0,0,0}, textString = "not startValue"),Text(extent = {{ -68, -58},{ -36, -72}}, lineColor = {0,0,0}, textString = "startValue")}), Documentation(info = "<html>
<p>
The Boolean output y is a step signal:
</p>

<p>
<img src=\"../Images/Blocks/Sources/BooleanStep.png\">
</p>
</html>"));

      equation 
        y = if time >= startTime then not startValue else startValue;
      end BooleanStep;
      block BooleanPulse "Generate pulse signal of type Boolean"
        parameter Real width(final min = Modelica.Constants.small, final max = 100) = 50 "Width of pulse in % of period";
        parameter Modelica.SIunits.Time period(final min = Modelica.Constants.small, start = 1) "Time for one period";
        parameter Modelica.SIunits.Time startTime = 0 "Time instant of first pulse";
        extends Modelica.Blocks.Interfaces.partialBooleanSource;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -150, -140},{150, -110}}, lineColor = {0,0,0}, textString = "%period"),Line(points = {{ -80, -70},{ -40, -70},{ -40,44},{0,44},{0, -70},{40, -70},{40,44},{79,44}}, color = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -60, -74},{ -19, -82}}, lineColor = {0,0,0}, textString = "startTime"),Line(points = {{ -78, -70},{ -40, -70},{ -40,20},{20,20},{20, -70},{50, -70},{50,20},{100,20}}, color = {0,0,255}, thickness = 0.5),Line(points = {{ -40,61},{ -40,21}}, color = {95,95,95}),Line(points = {{20,44},{20,20}}, color = {95,95,95}),Line(points = {{50,58},{50,20}}, color = {95,95,95}),Line(points = {{ -40,53},{50,53}}, color = {95,95,95}),Line(points = {{ -40,35},{20,35}}, color = {95,95,95}),Text(extent = {{ -30,65},{16,55}}, lineColor = {0,0,0}, textString = "period"),Text(extent = {{ -33,47},{14,37}}, lineColor = {0,0,0}, textString = "width"),Line(points = {{ -70,20},{ -41,20}}, color = {95,95,95}),Polygon(points = {{ -40,35},{ -31,37},{ -31,33},{ -40,35}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{20,35},{12,37},{12,33},{20,35}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{ -40,53},{ -31,55},{ -31,51},{ -40,53}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{50,53},{42,55},{42,51},{50,53}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Text(extent = {{ -95,26},{ -66,17}}, lineColor = {0,0,0}, textString = "true"),Text(extent = {{ -96, -60},{ -75, -69}}, lineColor = {0,0,0}, textString = "false")}), Documentation(info = "<html>
<p>
The Boolean output y is a pulse signal:
</p>

<p>
<img src=\"../Images/Blocks/Sources/Pulse.png\">
</p>
</html>"));
      protected 
        parameter Modelica.SIunits.Time Twidth = (period * width) / 100 "width of one pulse" annotation(HideResult = true);
        discrete Modelica.SIunits.Time pulsStart "Start time of pulse" annotation(HideResult = true);

      initial equation 
        pulsStart = startTime;

      equation 
        when sample(startTime, period) then
                  pulsStart = time;
        
        end when;
        y = time >= pulsStart and time < pulsStart + Twidth;
      end BooleanPulse;
      block SampleTrigger "Generate sample trigger signal"
        parameter Modelica.SIunits.Time period(final min = Modelica.Constants.small, start = 0.01) "Sample period";
        parameter Modelica.SIunits.Time startTime = 0 "Time instant of first sample trigger";
        extends Interfaces.partialBooleanSource;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -60, -70},{ -60,70}}, color = {0,0,0}),Line(points = {{ -20, -70},{ -20,70}}, color = {0,0,0}),Line(points = {{20, -70},{20,70}}, color = {0,0,0}),Line(points = {{60, -70},{60,70}}, color = {0,0,0}),Text(extent = {{ -150, -140},{150, -110}}, lineColor = {0,0,0}, textString = "%period")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -51, -72},{ -11, -81}}, lineColor = {0,0,0}, textString = "startTime"),Line(points = {{ -30,47},{ -30,19}}, color = {95,95,95}),Line(points = {{0,47},{0,18}}, color = {95,95,95}),Line(points = {{ -30,41},{0,41}}, color = {95,95,95}),Text(extent = {{ -37,61},{9,49}}, lineColor = {0,0,0}, textString = "period"),Line(points = {{ -73,19},{ -30,19}}, color = {95,95,95}),Polygon(points = {{ -30,41},{ -21,43},{ -21,39},{ -30,41}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{0,41},{ -8,43},{ -8,39},{0,41}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Text(extent = {{ -91,23},{ -71,13}}, lineColor = {0,0,0}, textString = "true"),Text(extent = {{ -90, -59},{ -70, -68}}, lineColor = {0,0,0}, textString = "false"),Line(points = {{0, -70},{0,19}}, color = {0,0,255}, thickness = 0.5),Line(points = {{ -30, -70},{ -30,19}}, color = {0,0,255}, thickness = 0.5),Line(points = {{30, -70},{30,19}}, color = {0,0,255}, thickness = 0.5),Line(points = {{60, -70},{60,19}}, color = {0,0,255}, thickness = 0.5)}), Documentation(info = "<html>
<p>
The Boolean output y is a trigger signal where the output y is only <b>true</b>
at sample times (defined by parameter <b>period</b>) and is otherwise
<b>false</b>.
</p>

<p>
<img src=\"../Images/Blocks/Sources/SampleTrigger.png\">
</p>
</html>"));

      equation 
        y = sample(startTime, period);
      end SampleTrigger;
      block BooleanTable "Generate a Boolean output signal based on a vector of time instants"
        parameter Boolean startValue = false "Start value of y. At time = table[1], y changes to 'not startValue'";
        parameter Modelica.SIunits.Time table[:] "Vector of time points. At every time point, the output y gets its opposite value (e.g. table={0,1})";
        extends Interfaces.partialBooleanSource;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -18,70},{32, -50}}, lineColor = {255,255,255}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -18, -50},{ -18,70},{32,70},{32, -50},{ -18, -50},{ -18, -20},{32, -20},{32,10},{ -18,10},{ -18,40},{32,40},{32,70},{32,70},{32, -51}}, color = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -34,66},{16, -54}}, lineColor = {255,255,255}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -34, -54},{ -34,66},{16,66},{16, -54},{ -34, -54},{ -34, -24},{16, -24},{16,6},{ -34,6},{ -34,36},{16,36},{16,66},{16,66},{16, -55}}, color = {0,0,0}),Text(extent = {{ -29,59},{10,44}}, lineColor = {0,0,0}, textString = "time")}), Documentation(info = "<html>
<p>
The Boolean output y is a signal defined by parameter vector <b>table</b>.
In the vector time points are stored. At every time point, the output y
changes its value to the negated value of the previous one.
</p>

<p>
<img src=\"../Images/Blocks/Sources/BooleanTable.png\">
</p>
</html>"));
      protected 
        function getFirstIndex "Get first index of table and check table"
          input Real table[:] "Vector of time instants";
          input Modelica.SIunits.Time simulationStartTime "Simulation start time";
          input Boolean startValue "Value of y for y < table[1]";
          output Integer index "First index to be used";
          output Modelica.SIunits.Time nextTime "Time instant of first event";
          output Boolean y "Value of y at simulationStartTime";
        protected 
          Modelica.SIunits.Time t_last;
          Integer j;
          Integer n = size(table, 1) "Number of table points";
        algorithm 
          if size(table, 1) == 0 then 
                    index:=0;
          nextTime:= -Modelica.Constants.inf;
          y:=startValue;
          elseif size(table, 1) == 1 then
            index:=1;
            if table[1] > simulationStartTime then 
                        nextTime:=table[1];
            y:=startValue;

            else             nextTime:=simulationStartTime;
            y:=startValue;

            end if;


          else           t_last:=table[1];
          for i in 2:n loop
                      assert(table[i] > t_last, "Time values of table not strict monotonically increasing: table[" + String(i - 1) + "] = " + String(table[i - 1]) + "table[" + String(i) + "] = " + String(table[i]));

          end for;
          j:=1;
          y:=startValue;
          while (j < n and table[j] <= simulationStartTime) loop
                      y:=not y;
            j:=j + 1;

          end while;
          if j == 1 then 
                    nextTime:=table[1];
          y:=startValue;
          elseif j == n and table[n] <= simulationStartTime then
            nextTime:=simulationStartTime - 1;
            y:=not y;


          else           nextTime:=table[j];

          end if;
          index:=j;

          end if;
        end getFirstIndex;
        parameter Integer n = size(table, 1) "Number of table points";
        Modelica.SIunits.Time nextTime;
        Integer index "Index of actual table entry";
      initial algorithm 
        (index,nextTime,y):=getFirstIndex(table, time, startValue);
      algorithm 
        when time >= pre(nextTime) and n > 0 then
                  if index < n then 
                    index:=index + 1;
          nextTime:=table[index];
          y:=not y;
          elseif index == n then
            index:=index + 1;
            y:=not y;


          else 
          end if;
        
        end when;
      end BooleanTable;
      block IntegerConstant "Generate constant signal of type Integer"
        parameter Integer k(start = 1) "Constant output value";
        extends Interfaces.IntegerSO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,0},{80,0}}, color = {0,0,0}),Text(extent = {{ -150, -150},{150, -110}}, lineColor = {0,0,0}, textString = "k=%k")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -80,90},{ -86,68},{ -74,68},{ -80,90}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {95,95,95}),Line(points = {{ -80,0},{80,0}}, color = {0,0,255}, thickness = 0.5),Line(points = {{ -90, -70},{82, -70}}, color = {95,95,95}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Text(extent = {{ -81,92},{ -38,74}}, lineColor = {0,0,0}, textString = "y"),Text(extent = {{66, -82},{94, -94}}, lineColor = {0,0,0}, textString = "time"),Text(extent = {{ -101,8},{ -81, -12}}, lineColor = {0,0,0}, textString = "k")}), Documentation(info = "<html>
<p>
The Integer output y is a constant signal:
</p>

<p>
<img src=\"../Images/Blocks/Sources/IntegerConstant.png\">
</p>
</html>"));

      equation 
        y = k;
      end IntegerConstant;
      block IntegerStep "Generate step signal of type Integer"
        parameter Integer height = 1 "Height of step";
        extends Interfaces.IntegerSignalSource;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -70},{0, -70},{0,50},{80,50}}, color = {0,0,0}),Text(extent = {{ -150, -150},{150, -110}}, lineColor = {0,0,0}, textString = "startTime=%startTime")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -80,88},{ -86,68},{ -74,68},{ -80,88}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -80}}, color = {95,95,95}),Line(points = {{ -80, -18},{0, -18},{0,50},{80,50}}, color = {0,0,255}, thickness = 0.5),Line(points = {{ -90, -70},{82, -70}}, color = {95,95,95}),Polygon(points = {{90, -70},{70, -66},{70, -74},{90, -70}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Text(extent = {{66, -78},{92, -88}}, lineColor = {0,0,0}, textString = "time"),Text(extent = {{ -21, -76},{26, -88}}, lineColor = {0,0,0}, textString = "startTime"),Line(points = {{0, -17},{0, -71}}, color = {95,95,95}),Text(extent = {{ -60, -36},{ -12, -48}}, lineColor = {0,0,0}, textString = "offset"),Line(points = {{ -13,50},{ -13, -17}}, color = {95,95,95}),Polygon(points = {{2,50},{ -19,50},{2,50}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{ -13, -17},{ -16, -4},{ -10, -4},{ -13, -17},{ -13, -17}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Polygon(points = {{ -13,50},{ -16,37},{ -9,37},{ -13,50}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Text(extent = {{ -58,22},{ -12,10}}, lineColor = {0,0,0}, textString = "height"),Polygon(points = {{ -13, -69},{ -16, -56},{ -10, -56},{ -13, -69},{ -13, -69}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -13, -18},{ -13, -70}}, color = {95,95,95}),Polygon(points = {{ -13, -18},{ -16, -31},{ -9, -31},{ -13, -18}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Text(extent = {{ -78,96},{ -44,80}}, lineColor = {0,0,0}, textString = "y")}), Documentation(info = "<html>
<p>
The Integer output y is a step signal:
</p>

<p>
<img src=\"../Images/Blocks/Sources/IntegerStep.png\">
</p>
</html>"));

      equation 
        y = offset + (if time < startTime then 0 else height);
      end IntegerStep;
    end Sources;
    package Routing "Library of blocks to combine and extract signals"
      extends Icons.Library;
      block Replicator "Signal replicator"
        extends Modelica.Blocks.Interfaces.SIMO;
        annotation(Window(x = 0.15, y = 0.16, width = 0.63, height = 0.59), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -100,0},{ -6,0}}, color = {0,0,127}),Line(points = {{100,0},{10,0}}, color = {0,0,127}),Line(points = {{0,0},{100,10}}, color = {0,0,127}),Line(points = {{0,0},{100, -10}}, color = {0,0,127}),Ellipse(extent = {{ -14,16},{16, -14}}, lineColor = {0,0,0}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
This block replicates the input signal to an array of <code>nout</code> identical output signals.
</p>
</html>"));

      equation 
        y = fill(u, nout);
      end Replicator;
      block ExtractSignal "Extract signals from an input signal vector"
        extends Modelica.Blocks.Interfaces.MIMO;
        parameter Integer extract[nout] = 1:nout "Extracting vector";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -90,51},{ -50, -49}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,127}),Rectangle(extent = {{50,50},{90, -50}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,127}),Polygon(points = {{ -94.4104,1.90792},{ -94.4104, -2.09208},{ -90.4104, -0.0920762},{ -94.4104,1.90792}}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid, lineColor = {0,0,127}),Line(points = {{ -72,2},{ -60.1395,12.907},{ -49.1395,12.907}}, color = {0,0,127}),Line(points = {{ -73,4},{ -59,40},{ -49,40}}, color = {0,0,127}),Line(points = {{ -113,0},{ -76.0373, -0.0180176}}, color = {0,0,127}),Ellipse(extent = {{ -81.0437,4.59255},{ -71.0437, -4.90745}}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid, lineColor = {0,0,127}),Line(points = {{ -73, -5},{ -60, -40},{ -49, -40}}, color = {0,0,127}),Line(points = {{ -72, -2},{ -60.0698, -12.907},{ -49.0698, -12.907}}, color = {0,0,127}),Polygon(points = {{ -48.8808, -11},{ -48.8808, -15},{ -44.8808, -13},{ -48.8808, -11}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,127}),Line(points = {{ -46,13},{ -35,13},{35, -30},{45, -30}}, color = {0,0,127}),Line(points = {{ -45,40},{ -35,40},{35,0},{44,0}}, color = {0,0,127}),Line(points = {{ -45, -40},{ -34, -40},{35,30},{44,30}}, color = {0,0,127}),Polygon(points = {{ -49,42},{ -49,38},{ -45,40},{ -49,42}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,127}),Polygon(points = {{ -48.8728, -38.0295},{ -48.8728, -42.0295},{ -44.8728, -40.0295},{ -48.8728, -38.0295}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,127}),Polygon(points = {{ -48.9983,14.8801},{ -48.9983,10.8801},{ -44.9983,12.8801},{ -48.9983,14.8801}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,127}),Ellipse(extent = {{69.3052,4.12743},{79.3052, -5.37257}}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid, lineColor = {0,0,127}),Line(points = {{80,0},{100,0}}, color = {0,0,127}),Polygon(points = {{43.1618,32.3085},{43.1618,28.3085},{47.1618,30.3085},{43.1618,32.3085}}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid, lineColor = {0,0,127}),Polygon(points = {{43.2575,1.80443},{43.2575, -2.19557},{47.2575, -0.195573},{43.2575,1.80443}}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid, lineColor = {0,0,127}),Polygon(points = {{43.8805, -28.1745},{43.8805, -32.1745},{47.8805, -30.1745},{43.8805, -28.1745}}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid, lineColor = {0,0,127}),Line(points = {{48,0},{70,0}}, color = {0,0,127}),Line(points = {{47,30},{60,30},{73,3}}, color = {0,0,127}),Line(points = {{49, -30},{60, -30},{74, -4}}, color = {0,0,127}),Text(extent = {{ -150, -150},{150, -110}}, lineColor = {0,0,0}, textString = "extract=%extract")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -90,50},{ -50, -50}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{50,50},{90, -50}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -94.4104,1.90792},{ -94.4104, -2.09208},{ -90.4104, -0.0920762},{ -94.4104,1.90792}}, lineColor = {0,0,255}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -72,2},{ -60.1395,12.907},{ -49.1395,12.907}}, color = {0,0,255}),Line(points = {{ -73,4},{ -59,40},{ -49,40}}, color = {0,0,255}),Line(points = {{ -112,0},{ -75.0373, -0.0180176}}, color = {0,0,255}),Ellipse(extent = {{ -80.0437,4.59255},{ -70.0437, -4.90745}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Line(points = {{ -73, -5},{ -60, -40},{ -49, -40}}, color = {0,0,255}),Line(points = {{ -72, -2},{ -60.0698, -12.907},{ -49.0698, -12.907}}, color = {0,0,255}),Polygon(points = {{ -48.8808, -11},{ -48.8808, -15},{ -44.8808, -13},{ -48.8808, -11}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -46,13},{ -35,13},{35, -30},{45, -30}}, color = {0,0,255}),Line(points = {{ -45,40},{ -35,40},{35,0},{44,0}}, color = {0,0,255}),Line(points = {{ -45, -40},{ -34, -40},{35,30},{44,30}}, color = {0,0,255}),Polygon(points = {{ -49,42},{ -49,38},{ -45,40},{ -49,42}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -48.8728, -38.0295},{ -48.8728, -42.0295},{ -44.8728, -40.0295},{ -48.8728, -38.0295}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -48.9983,14.8801},{ -48.9983,10.8801},{ -44.9983,12.8801},{ -48.9983,14.8801}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{70.3052,4.12743},{80.3052, -5.37257}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Line(points = {{80,0},{105,0}}, color = {0,0,255}),Polygon(points = {{44.1618,32.3085},{44.1618,28.3085},{48.1618,30.3085},{44.1618,32.3085}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Polygon(points = {{44.2575,1.80443},{44.2575, -2.19557},{48.2575, -0.195573},{44.2575,1.80443}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Polygon(points = {{44.8805, -28.1745},{44.8805, -32.1745},{48.8805, -30.1745},{44.8805, -28.1745}}, lineColor = {0,0,255}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid),Line(points = {{48,0},{70,0}}, color = {0,0,255}),Line(points = {{47,30},{60,30},{73,3}}, color = {0,0,255}),Line(points = {{49, -30},{60, -30},{74, -4}}, color = {0,0,255}),Rectangle(extent = {{ -100,80},{100, -81}}, lineColor = {0,0,255})}), Documentation(info = "<HTML>
<p>Extract signals from the input connector and transfer them
to the output connector.</p>
<p>The extracting scheme is given by the integer vector 'extract'.
This vector specifies, which input signals are taken and in which
order they are transfered to the output vector. Note, that the
dimension of 'extract' has to match the number of outputs.
Additionally, the dimensions of the input connector signals and
the output connector signals have to be explicitly defined via the
parameters 'nin' and 'nout'.</p>
<p>Example:</p>
<pre>     nin = 7 \"Number of inputs\";
     nout = 4 \"Number of outputs\";
     extract[nout] = {6,3,3,2} \"Extracting vector\";
</pre>
<p>extracts four output signals (nout=4) from the seven elements of the
input vector (nin=7):</p>
<pre>   output no. 1 is set equal to input no. 6
   output no. 2 is set equal to input no. 3
   output no. 3 is set equal to input no. 3
   output no. 4 is set equal to input no. 2
</pre>
</HTML>
"));

      equation 
        for i in 1:nout loop
        y[i] = u[extract[i]];

        end for;
      end ExtractSignal;
      block Extractor "Extract scalar signal out of signal vector dependent on IntegerRealInput index"
        extends Modelica.Blocks.Interfaces.MISO;
        parameter Boolean allowOutOfRange = false "Index may be out of range";
        parameter Real outOfRangeValue = 10000000000.0 "Output signal if index is out of range";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -80,50},{ -40, -50}}, lineColor = {0,0,127}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -84.4104,1.9079},{ -84.4104, -2.09208},{ -80.4104, -0.09208},{ -84.4104,1.9079}}, lineColor = {0,0,127}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -62,2},{ -50.1395,12.907},{ -39.1395,12.907}}, color = {0,0,127}),Line(points = {{ -63,4},{ -49,40},{ -39,40}}, color = {0,0,127}),Line(points = {{ -102,0},{ -65.0373, -0.01802}}, color = {0,0,127}),Ellipse(extent = {{ -70.0437,4.5925},{ -60.0437, -4.90745}}, lineColor = {0,0,127}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid),Line(points = {{ -63, -5},{ -50, -40},{ -39, -40}}, color = {0,0,127}),Line(points = {{ -62, -2},{ -50.0698, -12.907},{ -39.0698, -12.907}}, color = {0,0,127}),Polygon(points = {{ -38.8808, -11},{ -38.8808, -15},{ -34.8808, -13},{ -38.8808, -11}}, lineColor = {0,0,127}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -39,42},{ -39,38},{ -35,40},{ -39,42}}, lineColor = {0,0,127}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -38.8728, -38.0295},{ -38.8728, -42.0295},{ -34.8728, -40.0295},{ -38.8728, -38.0295}}, lineColor = {0,0,127}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -38.9983,14.8801},{ -38.9983,10.8801},{ -34.9983,12.8801},{ -38.9983,14.8801}}, lineColor = {0,0,127}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -30,50},{30, -50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,127}),Line(points = {{100,0},{0,0}}, color = {0,0,127}),Line(points = {{0,2},{0, -104}}, color = {255,128,0}),Line(points = {{ -35,40},{ -20,40}}, color = {0,0,127}),Line(points = {{ -35,13},{ -20,13}}, color = {0,0,127}),Line(points = {{ -35, -13},{ -20, -13}}, color = {0,0,127}),Line(points = {{ -35, -40},{ -20, -40}}, color = {0,0,127}),Polygon(points = {{0,0},{ -20,13},{ -20,13},{0,0},{0,0}}, lineColor = {0,0,127}),Ellipse(extent = {{ -6,6},{6, -6}}, lineColor = {255,128,0}, fillColor = {255,128,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>This block extracts a scalar output signal out the
vector of input signals dependent on the Integer
value of the additional u index:</p>
<pre>    y = u [ index ] ;
</pre>
<p>where index is an additional Integer input signal.</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1})));
        Modelica.Blocks.Interfaces.IntegerInput index annotation(Placement(transformation(origin = {0, -120}, extent = {{ -20, -20},{20,20}}, rotation = 90)));
      protected 
        Real k[nin];

      equation 
        when {initial(),change(index)} then
                  for i in 1:nin loop
          k[i] = if index == i then 1 else 0;

          end for;
        
        end when;
        y = if not allowOutOfRange or index > 0 and index <= nin then k * u else outOfRangeValue;
      end Extractor;
      block Multiplex2 "Multiplexer block for two input connectors"
        extends Modelica.Blocks.Interfaces.BlockIcon;
        parameter Integer n1 = 1 "dimension of input signal connector 1";
        parameter Integer n2 = 1 "dimension of input signal connector 2";
        Modelica.Blocks.Interfaces.RealInput u1[n1] "Connector of Real input signals 1" annotation(Placement(transformation(extent = {{ -140,40},{ -100,80}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u2[n2] "Connector of Real input signals 2" annotation(Placement(transformation(extent = {{ -140, -80},{ -100, -40}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y[n1 + n2] "Connector of Real output signals" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<HTML>
<p>
The output connector is the <b>concatenation</b> of the two input connectors.
Note, that the dimensions of the input connector signals have to be
explicitly defined via parameters n1 and n2.
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{8,0},{102,0}}, color = {0,0,127}),Ellipse(extent = {{ -14,16},{16, -14}}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid, lineColor = {0,0,127}),Line(points = {{ -98,60},{ -60,60},{ -4,6}}, color = {0,0,127}),Line(points = {{ -98, -60},{ -60, -60},{ -4, -4}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -98,60},{ -60,60},{ -4,6}}, color = {0,0,255}),Line(points = {{ -98, -60},{ -60, -60},{ -4, -4}}, color = {0,0,255}),Line(points = {{8,0},{102,0}}, color = {0,0,255}),Ellipse(extent = {{ -14,16},{16, -14}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255})}));

      equation 
        [y] = [u1;u2];
      end Multiplex2;
      block Multiplex3 "Multiplexer block for three input connectors"
        extends Modelica.Blocks.Interfaces.BlockIcon;
        parameter Integer n1 = 1 "dimension of input signal connector 1";
        parameter Integer n2 = 1 "dimension of input signal connector 2";
        parameter Integer n3 = 1 "dimension of input signal connector 3";
        Modelica.Blocks.Interfaces.RealInput u1[n1] "Connector of Real input signals 1" annotation(Placement(transformation(extent = {{ -140,50},{ -100,90}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u2[n2] "Connector of Real input signals 2" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u3[n3] "Connector of Real input signals 3" annotation(Placement(transformation(extent = {{ -140, -90},{ -100, -50}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y[n1 + n2 + n3] "Connector of Real output signals" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<HTML>
<p>
The output connector is the <b>concatenation</b> of the three input connectors.
Note, that the dimensions of the input connector signals have to be
explicitly defined via parameters n1, n2 and n3.
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{8,0},{102,0}}, color = {0,0,127}),Line(points = {{ -100,70},{ -60,70},{ -4,6}}, color = {0,0,127}),Line(points = {{ -100,0},{ -12,0}}, color = {0,0,127}),Line(points = {{ -100, -70},{ -62, -70},{ -4, -4}}, color = {0,0,127}),Ellipse(extent = {{ -14,16},{16, -14}}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid, lineColor = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -100,70},{ -60,70},{ -4,6}}, color = {0,0,255}),Line(points = {{ -100, -70},{ -62, -70},{ -4, -4}}, color = {0,0,255}),Line(points = {{8,0},{102,0}}, color = {0,0,255}),Ellipse(extent = {{ -14,16},{16, -14}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -100,0},{ -12,0}}, color = {0,0,255})}));

      equation 
        [y] = [u1;u2;u3];
      end Multiplex3;
      block Multiplex4 "Multiplexer block for four input connectors"
        extends Modelica.Blocks.Interfaces.BlockIcon;
        parameter Integer n1 = 1 "dimension of input signal connector 1";
        parameter Integer n2 = 1 "dimension of input signal connector 2";
        parameter Integer n3 = 1 "dimension of input signal connector 3";
        parameter Integer n4 = 1 "dimension of input signal connector 4";
        Modelica.Blocks.Interfaces.RealInput u1[n1] "Connector of Real input signals 1" annotation(Placement(transformation(extent = {{ -140,70},{ -100,110}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u2[n2] "Connector of Real input signals 2" annotation(Placement(transformation(extent = {{ -140,10},{ -100,50}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u3[n3] "Connector of Real input signals 3" annotation(Placement(transformation(extent = {{ -140, -50},{ -100, -10}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u4[n4] "Connector of Real input signals 4" annotation(Placement(transformation(extent = {{ -140, -110},{ -100, -70}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y[n1 + n2 + n3 + n4] "Connector of Real output signals" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<HTML>
<p>
The output connector is the <b>concatenation</b> of the four input connectors.
Note, that the dimensions of the input connector signals have to be
explicitly defined via parameters n1, n2, n3 and n4.
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{8,0},{102,0}}, color = {0,0,127}),Line(points = {{ -100,90},{ -60,90},{ -3,4}}, color = {0,0,127}),Line(points = {{ -100,30},{ -60,30},{ -9,0}}, color = {0,0,127}),Line(points = {{ -99, -30},{ -59, -30},{ -10, -5}}, color = {0,0,127}),Line(points = {{ -100, -90},{ -60, -90},{ -5, -6}}, color = {0,0,127}),Ellipse(extent = {{ -15,15},{15, -15}}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid, lineColor = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -100,90},{ -60,90},{ -3,4}}, color = {0,0,255}),Line(points = {{ -100, -90},{ -60, -90},{ -5, -6}}, color = {0,0,255}),Line(points = {{8,0},{102,0}}, color = {0,0,255}),Ellipse(extent = {{ -15,15},{15, -15}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -100,30},{ -60,30},{ -9,0}}, color = {0,0,255}),Line(points = {{ -99, -30},{ -59, -30},{ -10, -5}}, color = {0,0,255})}));

      equation 
        [y] = [u1;u2;u3;u4];
      end Multiplex4;
      block Multiplex5 "Multiplexer block for five input connectors"
        extends Modelica.Blocks.Interfaces.BlockIcon;
        parameter Integer n1 = 1 "dimension of input signal connector 1";
        parameter Integer n2 = 1 "dimension of input signal connector 2";
        parameter Integer n3 = 1 "dimension of input signal connector 3";
        parameter Integer n4 = 1 "dimension of input signal connector 4";
        parameter Integer n5 = 1 "dimension of input signal connector 5";
        Modelica.Blocks.Interfaces.RealInput u1[n1] "Connector of Real input signals 1" annotation(Placement(transformation(extent = {{ -140,80},{ -100,120}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u2[n2] "Connector of Real input signals 2" annotation(Placement(transformation(extent = {{ -140,30},{ -100,70}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u3[n3] "Connector of Real input signals 3" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u4[n4] "Connector of Real input signals 4" annotation(Placement(transformation(extent = {{ -140, -70},{ -100, -30}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u5[n5] "Connector of Real input signals 5" annotation(Placement(transformation(extent = {{ -140, -120},{ -100, -80}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y[n1 + n2 + n3 + n4 + n5] "Connector of Real output signals" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<HTML>
<p>
The output connector is the <b>concatenation</b> of the five input connectors.
Note, that the dimensions of the input connector signals have to be
explicitly defined via parameters n1, n2, n3, n4 and n5.
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{8,0},{102,0}}, color = {0,0,255}),Line(points = {{ -100,100},{ -60,100},{ -4,6}}, color = {0,0,127}),Line(points = {{ -99,50},{ -60,50},{ -8,5}}, color = {0,0,127}),Line(points = {{ -100,0},{ -7,0}}, color = {0,0,127}),Line(points = {{ -99, -50},{ -60, -50},{ -9, -6}}, color = {0,0,127}),Line(points = {{ -100, -100},{ -60, -100},{ -4, -4}}, color = {0,0,127}),Ellipse(extent = {{ -15,15},{15, -15}}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid, lineColor = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -100,100},{ -60,100},{ -4,6}}, color = {0,0,255}),Line(points = {{ -100, -100},{ -60, -100},{ -4, -4}}, color = {0,0,255}),Line(points = {{8,0},{102,0}}, color = {0,0,255}),Ellipse(extent = {{ -15,15},{15, -15}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -99,50},{ -60,50},{ -8,5}}, color = {0,0,255}),Line(points = {{ -100,0},{ -7,0}}, color = {0,0,255}),Line(points = {{ -99, -50},{ -60, -50},{ -9, -6}}, color = {0,0,255})}));

      equation 
        [y] = [u1;u2;u3;u4;u5];
      end Multiplex5;
      block Multiplex6 "Multiplexer block for six input connectors"
        extends Modelica.Blocks.Interfaces.BlockIcon;
        parameter Integer n1 = 1 "dimension of input signal connector 1";
        parameter Integer n2 = 1 "dimension of input signal connector 2";
        parameter Integer n3 = 1 "dimension of input signal connector 3";
        parameter Integer n4 = 1 "dimension of input signal connector 4";
        parameter Integer n5 = 1 "dimension of input signal connector 5";
        parameter Integer n6 = 1 "dimension of input signal connector 6";
        Modelica.Blocks.Interfaces.RealInput u1[n1] "Connector of Real input signals 1" annotation(Placement(transformation(extent = {{ -124,73},{ -100,97}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u2[n2] "Connector of Real input signals 2" annotation(Placement(transformation(extent = {{ -124,39},{ -100,63}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u3[n3] "Connector of Real input signals 3" annotation(Placement(transformation(extent = {{ -124,5},{ -100,29}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u4[n4] "Connector of Real input signals 4" annotation(Placement(transformation(extent = {{ -124, -29},{ -100, -5}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u5[n5] "Connector of Real input signals 5" annotation(Placement(transformation(extent = {{ -124, -63},{ -100, -39}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u6[n6] "Connector of Real input signals 6" annotation(Placement(transformation(extent = {{ -124, -97},{ -100, -73}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y[n1 + n2 + n3 + n4 + n5 + n6] "Connector of Real output signals" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<HTML>
<p>
The output connector is the <b>concatenation</b> of the six input connectors.
Note, that the dimensions of the input connector signals have to be
explicitly defined via parameters n1, n2, n3, n4, n5 and n6.
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{8,0},{102,0}}, color = {0,0,127}),Ellipse(extent = {{ -15,15},{15, -15}}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid, lineColor = {0,0,127}),Line(points = {{ -99,85},{ -61,85},{ -3,11}}, color = {0,0,127}),Line(points = {{ -100,51},{ -61,51},{ -7,6}}, color = {0,0,127}),Line(points = {{ -101,17},{ -60,17},{ -9,2}}, color = {0,0,127}),Line(points = {{ -100, -18},{ -60, -18},{ -11, -4}}, color = {0,0,127}),Line(points = {{ -99, -50},{ -60, -50},{ -9, -6}}, color = {0,0,127}),Line(points = {{ -100, -85},{ -60, -85},{ -3, -10}}, color = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -99,85},{ -61,85},{ -3,11}}, color = {0,0,255}),Line(points = {{ -100, -85},{ -60, -85},{ -3, -10}}, color = {0,0,255}),Line(points = {{8,0},{102,0}}, color = {0,0,255}),Ellipse(extent = {{ -15,15},{15, -15}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -100,51},{ -61,51},{ -7,6}}, color = {0,0,255}),Line(points = {{ -99, -50},{ -60, -50},{ -9, -6}}, color = {0,0,255}),Line(points = {{ -101,17},{ -60,17},{ -9,2}}, color = {0,0,255}),Line(points = {{ -100, -18},{ -60, -18},{ -11, -4}}, color = {0,0,255})}));

      equation 
        [y] = [u1;u2;u3;u4;u5;u6];
      end Multiplex6;
      block DeMultiplex2 "DeMultiplexer block for two output connectors"
        extends Modelica.Blocks.Interfaces.BlockIcon;
        parameter Integer n1 = 1 "dimension of output signal connector 1";
        parameter Integer n2 = 1 "dimension of output signal connector 2";
        Modelica.Blocks.Interfaces.RealInput u[n1 + n2] "Connector of Real input signals" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y1[n1] "Connector of Real output signals 1" annotation(Placement(transformation(extent = {{100,50},{120,70}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y2[n2] "Connector of Real output signals 2" annotation(Placement(transformation(extent = {{100, -70},{120, -50}}, rotation = 0)));
        annotation(Documentation(info = "<HTML>
<p>
The input connector is <b>splitted</b> up into two output connectors.
Note, that the dimensions of the output connector signals have to be
explicitly defined via parameters n1 and n2.
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{100,60},{60,60},{10,8}}, color = {0,0,127}),Ellipse(extent = {{ -14,16},{16, -14}}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid, lineColor = {0,0,127}),Line(points = {{100, -60},{60, -60},{8, -8}}, color = {0,0,127}),Line(points = {{ -100,0},{ -6,0}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{100,60},{60,60},{10,8}}, color = {0,0,255}),Line(points = {{100, -60},{60, -60},{8, -8}}, color = {0,0,255}),Line(points = {{ -100,0},{ -6,0}}, color = {0,0,255}),Ellipse(extent = {{ -14,16},{16, -14}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255})}));

      equation 
        [u] = [y1;y2];
      end DeMultiplex2;
      block DeMultiplex3 "DeMultiplexer block for three output connectors"
        extends Modelica.Blocks.Interfaces.BlockIcon;
        parameter Integer n1 = 1 "dimension of output signal connector 1";
        parameter Integer n2 = 1 "dimension of output signal connector 2";
        parameter Integer n3 = 1 "dimension of output signal connector 3";
        Modelica.Blocks.Interfaces.RealInput u[n1 + n2 + n3] "Connector of Real input signals" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y1[n1] "Connector of Real output signals 1" annotation(Placement(transformation(extent = {{100,60},{120,80}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y2[n2] "Connector of Real output signals 2" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y3[n3] "Connector of Real output signals 3" annotation(Placement(transformation(extent = {{100, -80},{120, -60}}, rotation = 0)));
        annotation(Documentation(info = "<HTML>
<p>
The input connector is <b>splitted</b> up into three output connectors.
Note, that the dimensions of the output connector signals have to be
explicitly defined via parameters n1, n2 and n3.
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -14,16},{16, -14}}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -100,0},{ -6,0}}, color = {0,0,127}),Line(points = {{100,70},{60,70},{4,5}}, color = {0,0,127}),Line(points = {{0,0},{101,0}}, color = {0,0,127}),Line(points = {{100, -70},{61, -70},{5, -5}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{100,70},{60,70},{4,5}}, color = {0,0,255}),Line(points = {{100, -70},{61, -70},{5, -5}}, color = {0,0,255}),Line(points = {{ -100,0},{ -6,0}}, color = {0,0,255}),Ellipse(extent = {{ -16,15},{14, -15}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{0,0},{101,0}}, color = {0,0,255})}));

      equation 
        [u] = [y1;y2;y3];
      end DeMultiplex3;
      block DeMultiplex4 "DeMultiplexer block for four output connectors"
        extends Modelica.Blocks.Interfaces.BlockIcon;
        parameter Integer n1 = 1 "dimension of output signal connector 1";
        parameter Integer n2 = 1 "dimension of output signal connector 2";
        parameter Integer n3 = 1 "dimension of output signal connector 3";
        parameter Integer n4 = 1 "dimension of output signal connector 4";
        Modelica.Blocks.Interfaces.RealInput u[n1 + n2 + n3 + n4] "Connector of Real input signals" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y1[n1] "Connector of Real output signals 1" annotation(Placement(transformation(extent = {{100,80},{120,100}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y2[n2] "Connector of Real output signals 2" annotation(Placement(transformation(extent = {{100,20},{120,40}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y3[n3] "Connector of Real output signals 3" annotation(Placement(transformation(extent = {{100, -40},{120, -20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y4[n4] "Connector of Real output signals 4" annotation(Placement(transformation(extent = {{100, -100},{120, -80}}, rotation = 0)));
        annotation(Documentation(info = "<HTML>
<p>
The input connector is <b>splitted</b> up into four output connectors.
Note, that the dimensions of the output connector signals have to be
explicitly defined via parameters n1, n2, n3 and n4.
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -14,16},{16, -14}}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid, lineColor = {0,0,127}),Line(points = {{ -100,0},{ -6,0}}, color = {0,0,127}),Line(points = {{100,90},{60,90},{6,5}}, color = {0,0,127}),Line(points = {{100,30},{60,30},{9,2}}, color = {0,0,127}),Line(points = {{100, -30},{60, -30},{8, -4}}, color = {0,0,127}),Line(points = {{99, -90},{60, -90},{6, -6}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{100,90},{60,90},{6,5}}, color = {0,0,255}),Line(points = {{99, -90},{60, -90},{6, -6}}, color = {0,0,255}),Line(points = {{ -100,0},{ -6,0}}, color = {0,0,255}),Line(points = {{100,30},{60,30},{9,2}}, color = {0,0,255}),Line(points = {{100, -30},{60, -30},{8, -4}}, color = {0,0,255}),Ellipse(extent = {{ -16,15},{14, -15}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255})}));

      equation 
        [u] = [y1;y2;y3;y4];
      end DeMultiplex4;
      block DeMultiplex5 "DeMultiplexer block for five output connectors"
        extends Modelica.Blocks.Interfaces.BlockIcon;
        parameter Integer n1 = 1 "dimension of output signal connector 1";
        parameter Integer n2 = 1 "dimension of output signal connector 2";
        parameter Integer n3 = 1 "dimension of output signal connector 3";
        parameter Integer n4 = 1 "dimension of output signal connector 4";
        parameter Integer n5 = 1 "dimension of output signal connector 5";
        Modelica.Blocks.Interfaces.RealInput u[n1 + n2 + n3 + n4 + n5] "Connector of Real input signals" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y1[n1] "Connector of Real output signals 1" annotation(Placement(transformation(extent = {{100,70},{120,90}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y2[n2] "Connector of Real output signals 2" annotation(Placement(transformation(extent = {{100,30},{120,50}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y3[n3] "Connector of Real output signals 3" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y4[n4] "Connector of Real output signals 4" annotation(Placement(transformation(extent = {{100, -50},{120, -30}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y5[n5] "Connector of Real output signals 5" annotation(Placement(transformation(extent = {{100, -90},{120, -70}}, rotation = 0)));
        annotation(Documentation(info = "<HTML>
<p>
The input connector is <b>splitted</b> up into five output connectors.
Note, that the dimensions of the output connector signals have to be
explicitly defined via parameters n1, n2, n3, n4 and n5.
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -14,16},{16, -14}}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid, lineColor = {0,0,127}),Line(points = {{ -100,0},{ -6,0}}, color = {0,0,127}),Line(points = {{99,80},{60,80},{6,5}}, color = {0,0,127}),Line(points = {{100,40},{60,40},{10,3}}, color = {0,0,127}),Line(points = {{100,0},{10,0}}, color = {0,0,127}),Line(points = {{100, -40},{61, -40},{11, -7}}, color = {0,0,127}),Line(points = {{100, -80},{60, -80},{7, -5}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{99,80},{60,80},{6,5}}, color = {0,0,255}),Line(points = {{100, -80},{60, -80},{7, -5}}, color = {0,0,255}),Line(points = {{ -100,0},{ -6,0}}, color = {0,0,255}),Ellipse(extent = {{ -14,15},{16, -15}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{100,40},{60,40},{10,3}}, color = {0,0,255}),Line(points = {{100,0},{10,0}}, color = {0,0,255}),Line(points = {{100, -40},{61, -40},{11, -7}}, color = {0,0,255})}));

      equation 
        [u] = [y1;y2;y3;y4;y5];
      end DeMultiplex5;
      block DeMultiplex6 "DeMultiplexer block for six output connectors"
        extends Modelica.Blocks.Interfaces.BlockIcon;
        parameter Integer n1 = 1 "dimension of output signal connector 1";
        parameter Integer n2 = 1 "dimension of output signal connector 2";
        parameter Integer n3 = 1 "dimension of output signal connector 3";
        parameter Integer n4 = 1 "dimension of output signal connector 4";
        parameter Integer n5 = 1 "dimension of output signal connector 5";
        parameter Integer n6 = 1 "dimension of output signal connector 6";
        Modelica.Blocks.Interfaces.RealInput u[n1 + n2 + n3 + n4 + n5 + n6] "Connector of Real input signals" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y1[n1] "Connector of Real output signals 1" annotation(Placement(transformation(extent = {{100,80},{120,100}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y2[n2] "Connector of Real output signals 2" annotation(Placement(transformation(extent = {{100,44},{120,64}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y3[n3] "Connector of Real output signals 3" annotation(Placement(transformation(extent = {{100,8},{120,28}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y4[n4] "Connector of Real output signals 4" annotation(Placement(transformation(extent = {{100, -28},{120, -8}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y5[n5] "Connector of Real output signals 5" annotation(Placement(transformation(extent = {{100, -64},{120, -44}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y6[n6] "Connector of Real output signals 6" annotation(Placement(transformation(extent = {{100, -100},{120, -80}}, rotation = 0)));
        annotation(Documentation(info = "<HTML>
<p>
The input connector is <b>splitted</b> up into six output connectors.
Note, that the dimensions of the output connector signals have to be
explicitly defined via parameters n1, n2, n3, n4, n5 and n6.
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -14,16},{16, -14}}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid, lineColor = {0,0,127}),Line(points = {{ -100,0},{ -6,0}}, color = {0,0,127}),Line(points = {{99,90},{60,90},{5,10}}, color = {0,0,127}),Line(points = {{100,53},{60,53},{8,6}}, color = {0,0,127}),Line(points = {{100,18},{59,18},{7,2}}, color = {0,0,127}),Line(points = {{100, -19},{60, -19},{13, -2}}, color = {0,0,127}),Line(points = {{99, -54},{60, -54},{9, -1}}, color = {0,0,127}),Line(points = {{100, -91},{60, -91},{3, -7}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{99,90},{60,90},{5,10}}, color = {0,0,255}),Line(points = {{100, -91},{60, -91},{3, -7}}, color = {0,0,255}),Line(points = {{ -100,0},{ -6,0}}, color = {0,0,255}),Ellipse(extent = {{ -14,15},{16, -15}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{100,53},{60,53},{8,6}}, color = {0,0,255}),Line(points = {{99, -54},{60, -54},{9, -1}}, color = {0,0,255}),Line(points = {{100,18},{59,18},{7,2}}, color = {0,0,255}),Line(points = {{100, -19},{60, -19},{13, -2}}, color = {0,0,255})}));

      equation 
        [u] = [y1;y2;y3;y4;y5;y6];
      end DeMultiplex6;
      annotation(Documentation(info = "<html>
<p>
This package contains blocks to combine and extract signals.
</p>
</html>"));
      model RealPassThrough "Pass a Real signal through without modification"
        extends Modelica.Blocks.Interfaces.BlockIcon;
        Modelica.Blocks.Interfaces.RealInput u "Input signal" annotation(HideResult = true, Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y "Output signal" annotation(HideResult = true, Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -100,0},{100,0}}, color = {0,0,127})}), Documentation(info = "<html>
<p>
Passes a Real signal through without modification.  Enables signals to be read out of one bus, have their name changed and be sent back to a bus.
</p>
</html>"));

      equation 
        y = u;
      end RealPassThrough;
      model IntegerPassThrough "Pass a Integer signal through without modification"
        extends Modelica.Blocks.Interfaces.IntegerBlockIcon;
        Modelica.Blocks.Interfaces.IntegerInput u "Input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.IntegerOutput y "Output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -100,0},{100,0}}, color = {255,128,0})}), Documentation(info = "<html>
<p>Passes a Integer signal through without modification.  Enables signals to be read out of one bus, have their name changed and be sent back to a bus.</p>
</html>"));

      equation 
        y = u;
      end IntegerPassThrough;
      model BooleanPassThrough "Pass a Boolean signal through without modification"
        extends Modelica.Blocks.Interfaces.BooleanBlockIcon;
        Modelica.Blocks.Interfaces.BooleanInput u "Input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.BooleanOutput y "Output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>Passes a Boolean signal through without modification.  Enables signals to be read out of one bus, have their name changed and be sent back to a bus.</p>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -100,0},{100,0}}, color = {255,0,255})}));

      equation 
        y = u;
      end BooleanPassThrough;
    end Routing;
    package Nonlinear "Library of discontinuous or non-differentiable algebraic control blocks"
      import Modelica.Blocks.Interfaces;
      extends Modelica.Icons.Library;
      annotation(Documentation(info = "
<HTML>
<p>
This package contains <b>discontinuous</b> and
<b>non-differentiable, algebraic</b> input/output blocks.
</p>
</HTML>
", revisions = "<html>
<ul>
<li><i>October 21, 2002</i>
       by <a href=\"http://www.robotic.dlr.de/Christian.Schweiger/\">Christian Schweiger</a>:<br>
       New block VariableLimiter added.
<li><i>August 22, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized, based on an existing Dymola library
       of Dieter Moormann and Hilding Elmqvist.
</li>
</ul>
</html>
"));
      block Limiter "Limit the range of a signal"
        parameter Real uMax(start = 1) "Upper limits of input signals";
        parameter Real uMin =  -uMax "Lower limits of input signals";
        parameter Boolean limitsAtInit = true "= false, if limits are ignored during initializiation (i.e., y=u)";
        extends Interfaces.SISO;
        annotation(Documentation(info = "
<HTML>
<p>
The Limiter block passes its input signal as output signal
as long as the input is within the specified upper and lower
limits. If this is not the case, the corresponding limits are passed
as output.
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{0, -90},{0,68}}, color = {192,192,192}),Polygon(points = {{0,90},{ -8,68},{8,68},{0,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68, -8},{68,8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -70},{ -50, -70},{50,70},{80,70}}, color = {0,0,0}),Text(extent = {{ -150, -150},{150, -110}}, lineColor = {0,0,0}, textString = "uMax=%uMax"),Text(extent = {{ -150,150},{150,110}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{0, -60},{0,50}}, color = {192,192,192}),Polygon(points = {{0,60},{ -5,50},{5,50},{0,60}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -60,0},{50,0}}, color = {192,192,192}),Polygon(points = {{60,0},{50, -5},{50,5},{60,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -50, -40},{ -30, -40},{30,40},{50,40}}, color = {0,0,0}),Text(extent = {{46, -6},{68, -18}}, lineColor = {128,128,128}, textString = "u"),Text(extent = {{ -30,70},{ -5,50}}, lineColor = {128,128,128}, textString = "y"),Text(extent = {{ -58, -54},{ -28, -42}}, lineColor = {128,128,128}, textString = "uMin"),Text(extent = {{26,40},{66,56}}, lineColor = {128,128,128}, textString = "uMax")}));

      equation 
        assert(uMax >= uMin, "Limiter: Limits must be consistent. However, uMax (=" + String(uMax) + ") < uMin (=" + String(uMin) + ")");
        if initial() and not limitsAtInit then
          y = u;
          assert(u >= uMin - 0.01 * abs(uMin) and u <= uMax + 0.01 * abs(uMax), "Limiter: During initialization the limits have been ignored.\n" + "However, the result is that the input u is not within the required limits:\n" + "  u = " + String(u) + ", uMin = " + String(uMin) + ", uMax = " + String(uMax));

        else
          y = smooth(0, if u > uMax then uMax else if u < uMin then uMin else u);

        end if;
      end Limiter;
      block VariableLimiter "Limit the range of a signal with variable limits"
        extends Interfaces.SISO;
        parameter Boolean limitsAtInit = true "= false, if limits are ignored during initializiation (i.e., y=u)";
        Interfaces.RealInput limit1 "Connector of Real input signal used as maximum of input u" annotation(Placement(transformation(extent = {{ -140,60},{ -100,100}}, rotation = 0)));
        Interfaces.RealInput limit2 "Connector of Real input signal used as minimum of input u" annotation(Placement(transformation(extent = {{ -140, -100},{ -100, -60}}, rotation = 0)));
      protected 
        Real uMax;
        Real uMin;
        annotation(Documentation(info = "<html>
<p>
The Limiter block passes its input signal as output signal
as long as the input is within the upper and lower
limits specified by the two additional inputs limit1 and
limit2. If this is not the case, the corresponding limit
is passed as output.
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{0, -90},{0,68}}, color = {192,192,192}),Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68, -8},{68,8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -70},{ -50, -70},{50,70},{80,70}}, color = {0,0,0}),Text(extent = {{ -150,150},{150,110}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -100,80},{66,80},{66,70}}, color = {0,0,127}),Line(points = {{ -100, -80},{ -64, -80},{ -64, -70}}, color = {0,0,127}),Polygon(points = {{ -64, -70},{ -66, -74},{ -62, -74},{ -64, -70}}, lineColor = {0,0,127}),Polygon(points = {{66,70},{64,74},{68,74},{66,70}}, lineColor = {0,0,127}),Polygon(points = {{0,90},{ -8,68},{8,68},{0,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{0, -60},{0,50}}, color = {192,192,192}),Polygon(points = {{0,60},{ -5,50},{5,50},{0,60}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -60,0},{50,0}}, color = {192,192,192}),Polygon(points = {{60,0},{50, -5},{50,5},{60,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -50, -40},{ -30, -40},{30,40},{50,40}}, color = {0,0,0}),Text(extent = {{46, -6},{68, -18}}, lineColor = {128,128,128}, textString = "inPort"),Text(extent = {{ -30,70},{ -5,50}}, lineColor = {128,128,128}, textString = "outPort"),Text(extent = {{ -66, -40},{ -26, -20}}, lineColor = {128,128,128}, textString = "uMin"),Text(extent = {{30,20},{70,40}}, lineColor = {128,128,128}, textString = "uMax"),Line(points = {{ -100,80},{40,80},{40,40}}, color = {0,0,255}),Line(points = {{ -100, -80},{ -40, -80},{ -40, -40}}, color = {0,0,255}),Polygon(points = {{40,40},{35,50},{45,50},{40,40}}, lineColor = {0,0,255}),Polygon(points = {{ -40, -40},{ -45, -50},{ -35, -50},{ -40, -40}}, lineColor = {0,0,255})}));

      equation 
        uMax = max(limit1, limit2);
        uMin = min(limit1, limit2);
        if initial() and not limitsAtInit then
          y = u;
          assert(u >= uMin - 0.01 * abs(uMin) and u <= uMax + 0.01 * abs(uMax), "VariableLimiter: During initialization the limits have been ignored.\n" + "However, the result is that the input u is not within the required limits:\n" + "  u = " + String(u) + ", uMin = " + String(uMin) + ", uMax = " + String(uMax));

        else
          y = smooth(0, if u > uMax then uMax else if u < uMin then uMin else u);

        end if;
      end VariableLimiter;
      block DeadZone "Provide a region of zero output"
        parameter Real uMax(start = 1) "Upper limits of dead zones";
        parameter Real uMin =  -uMax "Lower limits of dead zones";
        parameter Boolean deadZoneAtInit = true "= false, if dead zone is ignored during initializiation (i.e., y=u)";
        extends Interfaces.SISO;
        annotation(Documentation(info = "
<HTML>
<p>
The DeadZone block defines a region of zero output.
</p>
<p>
If the input is within uMin ... uMax, the output
is zero. Outside of this zone, the output is a linear
function of the input with a slope of 1.
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{0, -90},{0,68}}, color = {192,192,192}),Polygon(points = {{0,90},{ -8,68},{8,68},{0,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68, -8},{68,8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -60},{ -20,0},{20,0},{80,60}}, color = {0,0,0}),Text(extent = {{ -150, -150},{150, -110}}, lineColor = {160,160,164}, textString = "uMax=%uMax"),Text(extent = {{ -150,150},{150,110}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{0, -60},{0,50}}, color = {192,192,192}),Polygon(points = {{0,60},{ -5,50},{5,50},{0,60}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -76,0},{74,0}}, color = {192,192,192}),Polygon(points = {{84,0},{74, -5},{74,5},{84,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -81, -40},{ -38,0},{40,0},{80,40}}, color = {0,0,0}),Text(extent = {{62, -7},{88, -25}}, lineColor = {128,128,128}, textString = "u"),Text(extent = {{ -36,72},{ -5,50}}, lineColor = {128,128,128}, textString = "y"),Text(extent = {{ -51,1},{ -28,19}}, lineColor = {128,128,128}, textString = "uMin"),Text(extent = {{27,21},{52,5}}, lineColor = {128,128,128}, textString = "uMax")}));

      equation 
        assert(uMax >= uMin, "DeadZone: Limits must be consistent. However, uMax (=" + String(uMax) + ") < uMin (=" + String(uMin) + ")");
        if initial() and not deadZoneAtInit then
          y = u;

        else
          y = smooth(0, if u > uMax then u - uMax else if u < uMin then u - uMin else 0);

        end if;
      end DeadZone;
      block FixedDelay "Delay block with fixed DelayTime"
        extends Modelica.Blocks.Interfaces.SISO;
        parameter SI.Time delayTime(start = 1) "Delay time of output with respect to input signal";
        annotation(Documentation(info = "<html>
<p>
The Input signal is delayed by a given time instant, or more precisely:
</p>
<pre>
   y = u(time - delayTime) for time &gt; time.start + delayTime
     = u(time.start)       for time &le; time.start + delayTime
</pre>
</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{8, -102},{8, -142}}, lineColor = {0,0,0}, textString = "delayTime=%delayTime"),Line(points = {{ -92,0},{ -80.7,34.2},{ -73.5,53.1},{ -67.1,66.4},{ -61.4,74.6},{ -55.8,79.1},{ -50.2,79.8},{ -44.6,76.6},{ -38.9,69.7},{ -33.3,59.4},{ -26.9,44.1},{ -18.83,21.2},{ -1.9, -30.8},{5.3, -50.2},{11.7, -64.2},{17.3, -73.1},{23, -78.4},{28.6, -80},{34.2, -77.6},{39.9, -71.5},{45.5, -61.9},{51.9, -47.2},{60, -24.8},{68,0}}, color = {0,0,127}),Line(points = {{ -62,0},{ -50.7,34.2},{ -43.5,53.1},{ -37.1,66.4},{ -31.4,74.6},{ -25.8,79.1},{ -20.2,79.8},{ -14.6,76.6},{ -8.9,69.7},{ -3.3,59.4},{3.1,44.1},{11.17,21.2},{28.1, -30.8},{35.3, -50.2},{41.7, -64.2},{47.3, -73.1},{53, -78.4},{58.6, -80},{64.2, -77.6},{69.9, -71.5},{75.5, -61.9},{81.9, -47.2},{90, -24.8},{98,0}}, color = {160,160,164})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -80,80},{ -88,80}}, color = {192,192,192}),Line(points = {{ -80, -80},{ -88, -80}}, color = {192,192,192}),Line(points = {{ -80, -88},{ -80,86}}, color = {192,192,192}),Text(extent = {{ -75,98},{ -46,78}}, lineColor = {0,0,255}, textString = "outPort"),Polygon(points = {{ -80,96},{ -86,80},{ -74,80},{ -80,96}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -100,0},{84,0}}, color = {192,192,192}),Polygon(points = {{100,0},{84,6},{84, -6},{100,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,0},{ -68.7,34.2},{ -61.5,53.1},{ -55.1,66.4},{ -49.4,74.6},{ -43.8,79.1},{ -38.2,79.8},{ -32.6,76.6},{ -26.9,69.7},{ -21.3,59.4},{ -14.9,44.1},{ -6.83,21.2},{10.1, -30.8},{17.3, -50.2},{23.7, -64.2},{29.3, -73.1},{35, -78.4},{40.6, -80},{46.2, -77.6},{51.9, -71.5},{57.5, -61.9},{63.9, -47.2},{72, -24.8},{80,0}}, color = {0,0,255}),Text(extent = {{ -24,98},{ -2,78}}, lineColor = {0,0,0}, textString = "inPort"),Line(points = {{ -64,0},{ -52.7,34.2},{ -45.5,53.1},{ -39.1,66.4},{ -33.4,74.6},{ -27.8,79.1},{ -22.2,79.8},{ -16.6,76.6},{ -10.9,69.7},{ -5.3,59.4},{1.1,44.1},{9.17,21.2},{26.1, -30.8},{33.3, -50.2},{39.7, -64.2},{45.3, -73.1},{51, -78.4},{56.6, -80},{62.2, -77.6},{67.9, -71.5},{73.5, -61.9},{79.9, -47.2},{88, -24.8},{96,0}}, color = {0,0,0}),Text(extent = {{67,22},{96,6}}, lineColor = {160,160,164}, textString = "time"),Line(points = {{ -64, -30},{ -64,0}}, color = {192,192,192}),Text(extent = {{ -58, -42},{ -58, -32}}, textString = "delayTime", lineColor = {0,0,255}),Line(points = {{ -94, -26},{ -80, -26}}, color = {192,192,192}),Line(points = {{ -64, -26},{ -50, -26}}, color = {192,192,192}),Polygon(points = {{ -80, -26},{ -88, -24},{ -88, -28},{ -80, -26}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -56, -24},{ -64, -26},{ -56, -28},{ -56, -24}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid)}));

      equation 
        y = delay(u, delayTime);
      end FixedDelay;
      block PadeDelay "Pade approximation of delay block with fixed DelayTime "
        extends Modelica.Blocks.Interfaces.SISO;
        parameter SI.Time delayTime(start = 1) "Delay time of output with respect to input signal";
        parameter Integer n(min = 1) = 1 "Order of pade approximation";
        parameter Integer m(min = 1, max = n) = n "Order of numerator";
      protected 
        Real x1dot "Derivative of first state of TransferFcn";
        Real xn "Highest order state of TransferFcn";
        Real a[n + 1];
        Real b[m + 1];
      public 
        final output Real x[n] "State of transfer function from controller canonical form";
        annotation(Documentation(info = "<html>
<p>
The Input signal is delayed by a given time instant, or more precisely:
</p>
<pre>
   y = u(time - delayTime) for time &gt; time.start + delayTime
     = u(time.start)       for time &le; time.start + delayTime
</pre>
<p>
The delay is approximated by a Pade approximation, i.e., by
a transfer function
</p>
<pre>
           b[1]*s^m + b[2]*s^[m-1] + ... + b[m+1]
   y(s) = --------------------------------------------- * u(s)
           a[1]*s^n + a[2]*s^[n-1] + ... + a[n+1]
</pre>
<p>
where the coefficients b[:] and a[:] are calculated such that the
coefficients of the Taylor expansion of the delay exp(-T*s) around s=0
are identical upto order n+m.
</p>
<p>
The main advantage of this approach is that the delay is
approximated by a linear differential equation system, which
is continuous and continuously differentiable. For example, it
is uncritical to linearize a system containing a Pade-approximated
delay.
</p>
<p>
The standard text book version uses order \"m=n\", which is
also the default setting of this block. The setting
\"m=n-1\" may yield a better approximation in certain cases.
<p>
<p>
Literature:<br>
Otto Foellinger: Regelungstechnik, 8. Auflage,
chapter 11.9, page 412-414, Huethig Verlag Heidelberg, 1994
</p>
</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{8, -102},{8, -142}}, lineColor = {0,0,0}, textString = "delayTime=%delayTime"),Line(points = {{ -94,0},{ -82.7,34.2},{ -75.5,53.1},{ -69.1,66.4},{ -63.4,74.6},{ -57.8,79.1},{ -52.2,79.8},{ -46.6,76.6},{ -40.9,69.7},{ -35.3,59.4},{ -28.9,44.1},{ -20.83,21.2},{ -3.9, -30.8},{3.3, -50.2},{9.7, -64.2},{15.3, -73.1},{21, -78.4},{26.6, -80},{32.2, -77.6},{37.9, -71.5},{43.5, -61.9},{49.9, -47.2},{58, -24.8},{66,0}}, color = {0,0,127}),Line(points = {{ -72,0},{ -60.7,34.2},{ -53.5,53.1},{ -47.1,66.4},{ -41.4,74.6},{ -35.8,79.1},{ -30.2,79.8},{ -24.6,76.6},{ -18.9,69.7},{ -13.3,59.4},{ -6.9,44.1},{1.17,21.2},{18.1, -30.8},{25.3, -50.2},{31.7, -64.2},{37.3, -73.1},{43, -78.4},{48.6, -80},{54.2, -77.6},{59.9, -71.5},{65.5, -61.9},{71.9, -47.2},{80, -24.8},{88,0}}, color = {160,160,164}),Text(extent = {{ -10,100},{100,38}}, lineColor = {160,160,164}, textString = "m=%m"),Text(extent = {{ -98, -34},{6, -96}}, lineColor = {160,160,164}, textString = "n=%n")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -80,80},{ -88,80}}, color = {192,192,192}),Line(points = {{ -80, -80},{ -88, -80}}, color = {192,192,192}),Line(points = {{ -80, -88},{ -80,86}}, color = {192,192,192}),Text(extent = {{ -75,98},{ -46,78}}, lineColor = {0,0,255}, textString = "outPort"),Polygon(points = {{ -80,96},{ -86,80},{ -74,80},{ -80,96}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -100,0},{84,0}}, color = {192,192,192}),Polygon(points = {{100,0},{84,6},{84, -6},{100,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,0},{ -68.7,34.2},{ -61.5,53.1},{ -55.1,66.4},{ -49.4,74.6},{ -43.8,79.1},{ -38.2,79.8},{ -32.6,76.6},{ -26.9,69.7},{ -21.3,59.4},{ -14.9,44.1},{ -6.83,21.2},{10.1, -30.8},{17.3, -50.2},{23.7, -64.2},{29.3, -73.1},{35, -78.4},{40.6, -80},{46.2, -77.6},{51.9, -71.5},{57.5, -61.9},{63.9, -47.2},{72, -24.8},{80,0}}, color = {0,0,255}),Text(extent = {{ -24,98},{ -2,78}}, lineColor = {0,0,0}, textString = "inPort"),Line(points = {{ -64,0},{ -52.7,34.2},{ -45.5,53.1},{ -39.1,66.4},{ -33.4,74.6},{ -27.8,79.1},{ -22.2,79.8},{ -16.6,76.6},{ -10.9,69.7},{ -5.3,59.4},{1.1,44.1},{9.17,21.2},{26.1, -30.8},{33.3, -50.2},{39.7, -64.2},{45.3, -73.1},{51, -78.4},{56.6, -80},{62.2, -77.6},{67.9, -71.5},{73.5, -61.9},{79.9, -47.2},{88, -24.8},{96,0}}, color = {0,0,0}),Text(extent = {{67,22},{96,6}}, lineColor = {160,160,164}, textString = "time"),Line(points = {{ -64, -30},{ -64,0}}, color = {192,192,192}),Text(extent = {{ -58, -42},{ -58, -32}}, textString = "delayTime", lineColor = {0,0,255}),Line(points = {{ -94, -26},{ -80, -26}}, color = {192,192,192}),Line(points = {{ -64, -26},{ -50, -26}}, color = {192,192,192}),Polygon(points = {{ -80, -26},{ -88, -24},{ -88, -28},{ -80, -26}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -56, -24},{ -64, -26},{ -56, -28},{ -56, -24}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid)}));
      protected 
        function padeCoefficients
          input Real T "delay time";
          input Integer n "order of denominator";
          input Integer m "order of numerator";
          output Real b[m + 1] "numerator coefficients of transfer function";
          output Real a[n + 1] "denominator coefficients of transfer function";
        protected 
          Real nm;
        algorithm 
          a[1]:=1;
          b[1]:=1;
          nm:=n + m;
          for i in 1:n loop
                      a[i + 1]:=a[i] * (T * (n - i + 1) / (nm - i + 1)) / i;
            if i <= m then 
                        b[i + 1]:= -b[i] * (T * (m - i + 1) / (nm - i + 1)) / i;

            else 
            end if;

          end for;
          b:=b[m + 1: -1:1];
          a:=a[n + 1: -1:1];
        end padeCoefficients;

      equation 
        (b,a) = padeCoefficients(delayTime, n, m);
        [der(x);xn] = [x1dot;x];
        [u] = transpose([a]) * [x1dot;x];
        [y] = transpose([zeros(n - m, 1);b]) * [x1dot;x];

      initial equation 
        x[n] = u;
      end PadeDelay;
      block VariableDelay "Delay block with variable DelayTime"
        extends Modelica.Blocks.Interfaces.SISO;
        parameter Real delayMax(min = 0, start = 1) "maximum delay time";
        annotation(Documentation(info = "<html>
<p>
The Input signal is delayed by a given time instant, or more precisely:
</p>
<pre>
   y = u(time - delayTime) for time &gt; time.start + delayTime
     = u(time.start)       for time &le; time.start + delayTime
</pre>
<p>
where delayTime is an additional input signal which must follow
the following relationship:
</p>
<pre>  0 &le; delayTime &le; delayMax
</pre>
</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -100, -108},{100, -148}}, lineColor = {0,0,0}, textString = "delayMax=%delayMax"),Line(points = {{ -92,0},{ -80.7,34.2},{ -73.5,53.1},{ -67.1,66.4},{ -61.4,74.6},{ -55.8,79.1},{ -50.2,79.8},{ -44.6,76.6},{ -38.9,69.7},{ -33.3,59.4},{ -26.9,44.1},{ -18.83,21.2},{ -1.9, -30.8},{5.3, -50.2},{11.7, -64.2},{17.3, -73.1},{23, -78.4},{28.6, -80},{34.2, -77.6},{39.9, -71.5},{45.5, -61.9},{51.9, -47.2},{60, -24.8},{68,0}}, color = {0,0,127}),Line(points = {{ -64,0},{ -52.7,34.2},{ -45.5,53.1},{ -39.1,66.4},{ -33.4,74.6},{ -27.8,79.1},{ -22.2,79.8},{ -16.6,76.6},{ -10.9,69.7},{ -5.3,59.4},{1.1,44.1},{9.17,21.2},{26.1, -30.8},{33.3, -50.2},{39.7, -64.2},{45.3, -73.1},{51, -78.4},{56.6, -80},{62.2, -77.6},{67.9, -71.5},{73.5, -61.9},{79.9, -47.2},{88, -24.8},{96,0}}, color = {0,0,0}),Polygon(points = {{6,4},{ -14, -2},{ -6, -12},{6,4}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -100, -60},{ -76, -60},{ -8, -6}}, color = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100, -100},{100,100}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -80,96},{ -86,80},{ -74,80},{ -80,96}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -69,98},{ -40,78}}, lineColor = {0,0,255}, textString = "outPort"),Line(points = {{ -64,0},{ -52.7,34.2},{ -45.5,53.1},{ -39.1,66.4},{ -33.4,74.6},{ -27.8,79.1},{ -22.2,79.8},{ -16.6,76.6},{ -10.9,69.7},{ -5.3,59.4},{1.1,44.1},{9.17,21.2},{26.1, -30.8},{33.3, -50.2},{39.7, -64.2},{45.3, -73.1},{51, -78.4},{56.6, -80},{62.2, -77.6},{67.9, -71.5},{73.5, -61.9},{79.9, -47.2},{88, -24.8},{96,0}}, color = {0,0,0}),Line(points = {{ -80,0},{ -68.7,34.2},{ -61.5,53.1},{ -55.1,66.4},{ -49.4,74.6},{ -43.8,79.1},{ -38.2,79.8},{ -32.6,76.6},{ -26.9,69.7},{ -21.3,59.4},{ -14.9,44.1},{ -6.83,21.2},{10.1, -30.8},{17.3, -50.2},{23.7, -64.2},{29.3, -73.1},{35, -78.4},{40.6, -80},{46.2, -77.6},{51.9, -71.5},{57.5, -61.9},{63.9, -47.2},{72, -24.8},{80,0}}, color = {0,0,255}),Line(points = {{ -100,0},{84,0}}, color = {192,192,192}),Polygon(points = {{100,0},{84,6},{84, -6},{100,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{67,22},{96,6}}, lineColor = {160,160,164}, textString = "time"),Text(extent = {{ -58, -42},{ -58, -32}}, textString = "delayTime", lineColor = {0,0,255}),Line(points = {{ -80, -88},{ -80,86}}, color = {192,192,192}),Text(extent = {{ -24,98},{ -2,78}}, lineColor = {0,0,0}, textString = "inPort"),Text(extent = {{ -24,98},{ -2,78}}, lineColor = {0,0,0}, textString = "inPort"),Polygon(points = {{ -80, -26},{ -88, -24},{ -88, -28},{ -80, -26}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -56, -24},{ -64, -26},{ -56, -28},{ -56, -24}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -64, -26},{ -50, -26}}, color = {192,192,192}),Line(points = {{ -94, -26},{ -80, -26}}, color = {192,192,192}),Text(extent = {{ -58, -42},{ -58, -32}}, textString = "delayTime", lineColor = {0,0,255}),Line(points = {{ -100, -60},{ -70, -60},{ -64, -44}}, arrow = {Arrow.None,Arrow.Filled}, color = {0,0,255}),Line(points = {{ -80, -88},{ -80,86}}, color = {192,192,192}),Line(points = {{ -100,0},{84,0}}, color = {192,192,192}),Line(points = {{ -64,0},{ -52.7,34.2},{ -45.5,53.1},{ -39.1,66.4},{ -33.4,74.6},{ -27.8,79.1},{ -22.2,79.8},{ -16.6,76.6},{ -10.9,69.7},{ -5.3,59.4},{1.1,44.1},{9.17,21.2},{26.1, -30.8},{33.3, -50.2},{39.7, -64.2},{45.3, -73.1},{51, -78.4},{56.6, -80},{62.2, -77.6},{67.9, -71.5},{73.5, -61.9},{79.9, -47.2},{88, -24.8},{96,0}}, color = {0,0,0}),Polygon(points = {{ -80,96},{ -86,80},{ -74,80},{ -80,96}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,0},{ -68.7,34.2},{ -61.5,53.1},{ -55.1,66.4},{ -49.4,74.6},{ -43.8,79.1},{ -38.2,79.8},{ -32.6,76.6},{ -26.9,69.7},{ -21.3,59.4},{ -14.9,44.1},{ -6.83,21.2},{10.1, -30.8},{17.3, -50.2},{23.7, -64.2},{29.3, -73.1},{35, -78.4},{40.6, -80},{46.2, -77.6},{51.9, -71.5},{57.5, -61.9},{63.9, -47.2},{72, -24.8},{80,0}}, color = {0,0,255}),Text(extent = {{ -69,98},{ -40,78}}, lineColor = {0,0,255}, textString = "outPort"),Text(extent = {{ -24,98},{ -2,78}}, lineColor = {0,0,0}, textString = "inPort"),Text(extent = {{67,22},{96,6}}, lineColor = {160,160,164}, textString = "time"),Polygon(points = {{100,0},{84,6},{84, -6},{100,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -64, -30},{ -64,0}}, color = {192,192,192})}));
        Modelica.Blocks.Interfaces.RealInput delayTime annotation(Placement(transformation(extent = {{ -140, -80},{ -100, -40}}, rotation = 0)));

      equation 
        y = delay(u, delayTime, delayMax);
      end VariableDelay;
    end Nonlinear;
    package Math "Library of mathematical functions as input/output blocks"
      import Modelica.SIunits;
      import Modelica.Blocks.Interfaces;
      extends Modelica.Icons.Library;
      annotation(Documentation(info = "
<HTML>
<p>
This package contains basic <b>mathematical operations</b>,
such as summation and multiplication, and basic <b>mathematical
functions</b>, such as <b>sqrt</b> and <b>sin</b>, as
input/output blocks. All blocks of this library can be either
connected with continuous blocks or with sampled-data blocks.
</p>
</HTML>
", revisions = "<html>
<ul>
<li><i>October 21, 2002</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>
       and <a href=\"http://www.robotic.dlr.de/Christian.Schweiger/\">Christian Schweiger</a>:<br>
       New blocks added: RealToInteger, IntegerToReal, Max, Min, Edge, BooleanChange, IntegerChange.</li>
<li><i>August 7, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized (partly based on an existing Dymola library
       of Dieter Moormann and Hilding Elmqvist).
</li>
</ul>
</html>"));
      encapsulated package UnitConversions "Conversion blocks to convert between SI and non-SI unit signals"
        import Modelica;
        import SI = Modelica.SIunits;
        import NonSI = Modelica.SIunits.Conversions.NonSIunits;
        block To_degC "Convert from Kelvin to Celsius"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "K"), y(unit = "degC"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "K"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "C")}), Documentation(info = "<html>
<p>
This block converts the input signal from Kelvin to Celsius and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.to_degC(u);
        end To_degC;
        block From_degC "Convert from Celsius to Kelvin"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "degC"), y(unit = "K"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "C"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "K")}), Documentation(info = "<html>
<p>
This block converts the input signal from Celsius to Kelvin and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.from_degC(u);
        end From_degC;
        block To_degF "Convert from Kelvin to Fahrenheit"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "K"), y(unit = "degF"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "K"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "F")}), Documentation(info = "<html>
<p>
This block converts the input signal from Kelvin to Fahrenheit and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.to_degF(u);
        end To_degF;
        block From_degF "Convert from Fahrenheit to Kelvin"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "degF"), y(unit = "K"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "F"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "K")}), Documentation(info = "<html>
<p>
This block converts the input signal from Fahrenheit to Kelvin and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.from_degF(u);
        end From_degF;
        block To_degRk "Convert from Kelvin to Rankine"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "K"), y(unit = "degRk"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "K"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "Rk")}), Documentation(info = "<html>
<p>
This block converts the input signal from Kelvin to Rankine and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.to_degRk(u);
        end To_degRk;
        block From_degRk "Convert from Rankine to Kelvin"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "degRk"), y(unit = "K"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "Rk"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "K")}), Documentation(info = "<html>
<p>
This block converts the input signal from Rankine to Kelvin and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.from_degRk(u);
        end From_degRk;
        block To_deg "Convert from radian to degree"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "rad"), y(unit = "deg"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "rad"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "deg")}), Documentation(info = "<html>
<p>
This block converts the input signal from radian to degree and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.to_deg(u);
        end To_deg;
        block From_deg "Convert from degree to radian"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "deg"), y(unit = "rad"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "deg"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "rad")}), Documentation(info = "<html>
<p>
This block converts the input signal from degree to radian and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.from_deg(u);
        end From_deg;
        block To_rpm "Convert from radian per second to revolutions per minute"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "rad/s"), y(unit = "1/min"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{26,82},{ -98,50}}, lineColor = {0,0,0}, textString = "rad/s"),Text(extent = {{100, -42},{ -62, -74}}, lineColor = {0,0,0}, textString = "1/min")}), Documentation(info = "<html>
<p>
This block converts the input signal from radian per second to revolutions per minute and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.to_rpm(u);
        end To_rpm;
        block From_rpm "Convert from revolutions per minute to radian per second"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "1/min"), y(unit = "rad/s"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{50,84},{ -94,56}}, lineColor = {0,0,0}, textString = "1/min"),Text(extent = {{94, -42},{ -26, -74}}, lineColor = {0,0,0}, textString = "rad/s")}), Documentation(info = "<html>
<p>
This block converts the input signal from revolutions per minute to radian per second and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.from_rpm(u);
        end From_rpm;
        block To_kmh "Convert from metre per second to kilometre per hour"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "m/s"), y(unit = "km/h"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{0,82},{ -96,42}}, lineColor = {0,0,0}, textString = "m/s"),Text(extent = {{92, -40},{ -14, -84}}, lineColor = {0,0,0}, textString = "km/h")}), Documentation(info = "<html>
<p>
This block converts the input signal from metre per second to kilometre per hour and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.to_kmh(u);
        end To_kmh;
        block From_kmh "Convert from kilometre per hour to metre per second"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "km/h"), y(unit = "m/s"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{26,80},{ -96,48}}, lineColor = {0,0,0}, textString = "km/h"),Text(extent = {{92, -46},{ -20, -82}}, lineColor = {0,0,0}, textString = "m/s")}), Documentation(info = "<html>
<p>
This block converts the input signal from kilometre per hour to metre per second and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.from_kmh(u);
        end From_kmh;
        block To_day "Convert from second to day"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "s"), y(unit = "d"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "s"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "day")}), Documentation(info = "<html>
<p>
This block converts the input signal from second to day and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.to_day(u);
        end To_day;
        block From_day "Convert from day to second"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "d"), y(unit = "s"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "day"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "s")}), Documentation(info = "<html>
<p>
This block converts the input signal from day to second and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.from_day(u);
        end From_day;
        block To_hour "Convert from second to hour"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "s"), y(unit = "h"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "s"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "hour")}), Documentation(info = "<html>
<p>
This block converts the input signal from second to hour and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.to_hour(u);
        end To_hour;
        block From_hour "Convert from hour to second"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "h"), y(unit = "s"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "hour"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "s")}), Documentation(info = "<html>
<p>
This block converts the input signal from hour to second and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.from_hour(u);
        end From_hour;
        block To_minute "Convert from second to minute"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "s"), y(unit = "min"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "s"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "minute")}), Documentation(info = "<html>
<p>
This block converts the input signal from second to minute and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.to_minute(u);
        end To_minute;
        block From_minute "Convert from minute to second"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "min"), y(unit = "s"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "minute"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "s")}), Documentation(info = "<html>
<p>
This block converts the input signal from minute to second and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.from_minute(u);
        end From_minute;
        block To_litre "Convert from cubic metre to litre"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "m3"), y(unit = "l"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "m3"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "litre")}), Documentation(info = "<html>
<p>
This block converts the input signal from metre to litre and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.to_litre(u);
        end To_litre;
        block From_litre "Convert from litre to cubic metre"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "l"), y(unit = "m3"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "litre"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "m3")}), Documentation(info = "<html>
<p>
This block converts the input signal from litre to cubic metre and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.from_litre(u);
        end From_litre;
        block To_kWh "Convert from Joule to kilo Watt hour"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "J"), y(unit = "kW.h"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "J"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "kW.h")}), Documentation(info = "<html>
<p>
This block converts the input signal from Joule to kilo Watt hour and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.to_kWh(u);
        end To_kWh;
        block From_kWh "Convert from kilo Watt hour to Joule"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "kW.h"), y(unit = "J"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "kW.h"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "J")}), Documentation(info = "<html>
<p>
This block converts the input signal from kilo Watt hour to Joule and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.from_kWh(u);
        end From_kWh;
        block To_bar "Convert from Pascal to bar"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "Pa"), y(unit = "bar"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "Pa"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "bar")}), Documentation(info = "<html>
<p>
This block converts the input signal from Pascal to bar and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.to_bar(u);
        end To_bar;
        block From_bar "Convert from bar to Pascal"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "bar"), y(unit = "Pa"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "bar"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "Pa")}), Documentation(info = "<html>
<p>
This block converts the input signal from bar to Pascal and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.from_bar(u);
        end From_bar;
        block To_gps "Convert from kilogram per second to gram per second"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "kg/s"), y(unit = "g/s"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "kg/s"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "g/s")}), Documentation(info = "<html>
<p>
This block converts the input signal from kilogram per second to gram per seconds and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.to_gps(u);
        end To_gps;
        block From_gps "Convert from gram per second to kilogram per second"
          extends Modelica.Blocks.Interfaces.PartialConversionBlock(u(unit = "g/s"), y(unit = "kg/s"));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "g/s"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "kg/s")}), Documentation(info = "<html>
<p>
This block converts the input signal from gram per second to kilogram per second and returns
the result as output signal.
</p>
</html>"));

        equation 
          y = SI.Conversions.from_gps(u);
        end From_gps;
        annotation(Documentation(info = "<html>
<p>
This package consists of blocks that convert an input signal
with a specific unit to an output signal in another unit
(e.g. conversion of an angle signal from \"deg\" to \"rad\").
Block \"ConvertAllUnits\" converts between a set of units that
can be selected in a pull-down menu of the parameter menu.
All other blocks convert exactly between two different units.
</p>

</html>"));
      end UnitConversions;
      block InverseBlockConstraints "Construct inverse model by requiring that two inputs and two outputs are identical (replaces the previously, unbalanced, TwoInputs and TwoOutputs blocks)"
        Modelica.Blocks.Interfaces.RealInput u1 "Input signal 1 (u1 = u2)" annotation(Placement(transformation(extent = {{ -240, -20},{ -200,20}}, rotation = 0), iconTransformation(extent = {{ -240, -20},{ -200,20}})));
        Modelica.Blocks.Interfaces.RealInput u2 "Input signal 2 (u1 = u2)" annotation(Placement(transformation(extent = {{ -140, -20},{ -180,20}}, rotation = 0), iconTransformation(extent = {{ -140, -20},{ -180,20}})));
        Modelica.Blocks.Interfaces.RealOutput y1 "Output signal 1 (y1 = y2)" annotation(Placement(transformation(extent = {{200, -10},{220,10}}, rotation = 0), iconTransformation(extent = {{200, -10},{220,10}})));
        Modelica.Blocks.Interfaces.RealOutput y2 "Output signal 2 (y2 = y2)" annotation(Placement(transformation(extent = {{10, -10},{ -10,10}}, rotation = 0, origin = {170,0}), iconTransformation(extent = {{180, -10},{160,10}})));
        annotation(__Dymola_structurallyIncomplete = true, Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -200, -120},{200,120}}), graphics), Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -200, -120},{200,120}}), graphics = {Line(points = {{180,0},{200,0}}, color = {0,0,127}, smooth = Smooth.None),Line(points = {{ -200,0},{ -180,0}}, color = {0,0,127}, smooth = Smooth.None),Rectangle(extent = {{ -190,120},{190, -120}}, lineColor = {135,135,135})}), Documentation(info = "<html>
<p>
Exchange input and ouput signals of a block, i.e., the previous
block inputs become block outputs and the previous block outputs become
block inputs. This block is used to construct inverse models.
Its usage is demonstrated in example:
<a href=\"Modelica://Modelica.Blocks.Examples.InverseModel\">Modelica.Blocks.Examples.InverseModel</a>.
</p>

<p>
Note, if a block shall be inverted that has several input and output blocks,
then this can be easily achieved by using a vector of InverseBlockConstraints
instances:
</p>

<pre>
   InverseBlockConstraint invert[3];  // Block to be inverted has 3 input signals
</pre>
</html>"));

      equation 
        u1 = u2;
        y1 = y2;
      end InverseBlockConstraints;
      block Gain "Output the product of a gain value with the input signal"
        parameter Real k(start = 1) "Gain value multiplied with input signal";
      public 
        Interfaces.RealInput u "Input signal connector" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Interfaces.RealOutput y "Output signal connector" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "
<HTML>
<p>
This block computes output <i>y</i> as
<i>product</i> of gain <i>k</i> with the
input <i>u</i>:
</p>
<pre>
    y = k * u;
</pre>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Polygon(points = {{ -100, -100},{ -100,100},{100,0},{ -100, -100}}, lineColor = {0,0,127}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -150, -140},{150, -100}}, lineColor = {0,0,0}, textString = "k=%k"),Text(extent = {{ -150,140},{150,100}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Polygon(points = {{ -100, -100},{ -100,100},{100,0},{ -100, -100}}, lineColor = {0,0,127}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -76,38},{0, -34}}, textString = "k", lineColor = {0,0,255})}));

      equation 
        y = k * u;
      end Gain;
      block MatrixGain "Output the product of a gain matrix with the input signal vector"
        parameter Real K[:,:] = [1,0;0,1] "Gain matrix which is multiplied with the input";
        extends Interfaces.MIMO(final nin = size(K, 2), final nout = size(K, 1));
        annotation(Documentation(info = "
<HTML>
<p>
This blocks computes output vector <b>y</b> as <i>product</i> of the
gain matrix <b>K</b> with the input signal vector <b>u</b>:
</p>
<pre>
    <b>y</b> = <b>K</b> * <b>u</b>;
</pre>
<p>
Example:
</p>
<pre>
   parameter: <b>K</b> = [0.12 2; 3 1.5]

   results in the following equations:

     | y[1] |     | 0.12  2.00 |   | u[1] |
     |      |  =  |            | * |      |
     | y[2] |     | 3.00  1.50 |   | u[2] |
</pre>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -90, -60},{90,60}}, lineColor = {160,160,164}, textString = "*K"),Text(extent = {{ -150,150},{150,110}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100, -100},{100,100}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -90, -60},{90,60}}, lineColor = {160,160,164}, textString = "*K")}));

      equation 
        y = K * u;
      end MatrixGain;
      block Sum "Output the sum of the elements of the input vector"
        extends Interfaces.MISO;
        parameter Real k[nin] = ones(nin) "Optional: sum coefficients";
        annotation(defaultComponentName = "sum1", Documentation(info = "
<HTML>
<p>
This blocks computes output <b>y</b> as
<i>sum</i> of the elements of the input signal vector
<b>u</b>:
</p>
<pre>
    <b>y</b> = <b>u</b>[1] + <b>u</b>[2] + ...;
</pre>
<p>
Example:
</p>
<pre>
     parameter:   nin = 3;

  results in the following equations:

     y = u[1] + u[2] + u[3];
</pre>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{26,42},{ -34,42},{6,2},{ -34, -38},{26, -38}}, color = {0,0,0}, thickness = 0.25),Text(extent = {{ -150,150},{150,110}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100, -100},{100,100}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{26,42},{ -34,42},{6,2},{ -34, -38},{26, -38}}, color = {0,0,0}, thickness = 0.25)}));

      equation 
        y = k * u;
      end Sum;
      block Feedback "Output difference between commanded and feedback input"
        input Interfaces.RealInput u1 annotation(Placement(transformation(extent = {{ -100, -20},{ -60,20}}, rotation = 0)));
        input Interfaces.RealInput u2 annotation(Placement(transformation(origin = {0, -80}, extent = {{ -20, -20},{20,20}}, rotation = 90)));
        output Interfaces.RealOutput y annotation(Placement(transformation(extent = {{80, -10},{100,10}}, rotation = 0)));
        annotation(Documentation(info = "
<HTML>
<p>
This blocks computes output <b>y</b> as <i>difference</i> of the
commanded input <b>u1</b> and the feedback
input <b>u2</b>:
</p>
<pre>
    <b>y</b> = <b>u1</b> - <b>u2</b>;
</pre>
<p>
Example:
</p>
<pre>
     parameter:   n = 2

  results in the following equations:

     y = u1 - u2
</pre>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Ellipse(extent = {{ -20,20},{20, -20}}, lineColor = {0,0,127}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid),Line(points = {{ -60,0},{ -20,0}}, color = {0,0,127}),Line(points = {{20,0},{80,0}}, color = {0,0,127}),Line(points = {{0, -20},{0, -60}}, color = {0,0,127}),Text(extent = {{ -14,0},{82, -94}}, lineColor = {0,0,0}, textString = "-"),Text(extent = {{ -150,94},{150,44}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Ellipse(extent = {{ -20,20},{20, -20}}, pattern = LinePattern.Solid, lineThickness = 0.25, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Line(points = {{ -60,0},{ -20,0}}, color = {0,0,255}),Line(points = {{20,0},{80,0}}, color = {0,0,255}),Line(points = {{0, -20},{0, -60}}, color = {0,0,255}),Text(extent = {{ -12,10},{84, -84}}, lineColor = {0,0,0}, textString = "-")}));

      equation 
        y = u1 - u2;
      end Feedback;
      block Add "Output the sum of the two inputs"
        extends Interfaces.SI2SO;
        parameter Real k1 =  +1 "Gain of upper input";
        parameter Real k2 =  +1 "Gain of lower input";
        annotation(Documentation(info = "
<HTML>
<p>
This blocks computes output <b>y</b> as <i>sum</i> of the
two input signals <b>u1</b> and <b>u2</b>:
</p>
<pre>
    <b>y</b> = k1*<b>u1</b> + k2*<b>u2</b>;
</pre>
<p>
Example:
</p>
<pre>
     parameter:   k1= +2, k2= -3

  results in the following equations:

     y = 2 * u1 - 3 * u2
</pre>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -98, -52},{7, -92}}, lineColor = {0,0,0}, textString = "%k2"),Text(extent = {{ -100,90},{5,50}}, lineColor = {0,0,0}, textString = "%k1"),Text(extent = {{ -150,150},{150,110}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -100,60},{ -40,60},{ -30,40}}, color = {0,0,255}),Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,255}),Line(points = {{ -100, -60},{ -40, -60},{ -30, -40}}, color = {0,0,255}),Line(points = {{ -15, -25.99},{15,25.99}}, color = {0,0,0}),Rectangle(extent = {{ -100, -100},{100,100}}, lineColor = {0,0,127}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{50,0},{100,0}}, color = {0,0,255}),Line(points = {{ -100,60},{ -74,24},{ -44,24}}, color = {0,0,127}),Line(points = {{ -100, -60},{ -74, -28},{ -42, -28}}, color = {0,0,127}),Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,127}),Line(points = {{50,0},{100,0}}, color = {0,0,127}),Text(extent = {{ -38,34},{38, -34}}, lineColor = {0,0,0}, textString = "+"),Text(extent = {{ -100,52},{5,92}}, lineColor = {0,0,0}, textString = "%k1"),Text(extent = {{ -100, -52},{5, -92}}, lineColor = {0,0,0}, textString = "%k2")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100, -100},{100,100}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -98, -52},{7, -92}}, lineColor = {0,0,0}, textString = "%k2"),Text(extent = {{ -100,90},{5,50}}, lineColor = {0,0,0}, textString = "%k1"),Line(points = {{ -100,60},{ -40,60},{ -30,40}}, color = {0,0,255}),Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,255}),Line(points = {{ -100, -60},{ -40, -60},{ -30, -40}}, color = {0,0,255}),Line(points = {{ -15, -25.99},{15,25.99}}, color = {0,0,0}),Rectangle(extent = {{ -100, -100},{100,100}}, lineColor = {0,0,127}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{50,0},{100,0}}, color = {0,0,255}),Line(points = {{ -100,60},{ -74,24},{ -44,24}}, color = {0,0,127}),Line(points = {{ -100, -60},{ -74, -28},{ -42, -28}}, color = {0,0,127}),Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,127}),Line(points = {{50,0},{100,0}}, color = {0,0,127}),Text(extent = {{ -38,34},{38, -34}}, lineColor = {0,0,0}, textString = "+"),Text(extent = {{ -100,52},{5,92}}, lineColor = {0,0,0}, textString = "k1"),Text(extent = {{ -100, -52},{5, -92}}, lineColor = {0,0,0}, textString = "k2")}));

      equation 
        y = k1 * u1 + k2 * u2;
      end Add;
      block Add3 "Output the sum of the three inputs"
        extends Interfaces.BlockIcon;
        parameter Real k1 =  +1 "Gain of upper input";
        parameter Real k2 =  +1 "Gain of middle input";
        parameter Real k3 =  +1 "Gain of lower input";
        input Interfaces.RealInput u1 "Connector 1 of Real input signals" annotation(Placement(transformation(extent = {{ -140,60},{ -100,100}}, rotation = 0)));
        input Interfaces.RealInput u2 "Connector 2 of Real input signals" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        input Interfaces.RealInput u3 "Connector 3 of Real input signals" annotation(Placement(transformation(extent = {{ -140, -100},{ -100, -60}}, rotation = 0)));
        output Interfaces.RealOutput y "Connector of Real output signals" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "
<HTML>
<p>
This blocks computes output <b>y</b> as <i>sum</i> of the
three input signals <b>u1</b>, <b>u2</b> and <b>u3</b>:
</p>
<pre>
    <b>y</b> = k1*<b>u1</b> + k2*<b>u2</b> + k3*<b>u3</b>;
</pre>
<p>
Example:
</p>
<pre>
     parameter:   k1= +2, k2= -3, k3=1;

  results in the following equations:

     y = 2 * u1 - 3 * u2 + u3;
</pre>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -100,50},{5,90}}, lineColor = {0,0,0}, textString = "%k1"),Text(extent = {{ -100, -20},{5,20}}, lineColor = {0,0,0}, textString = "%k2"),Text(extent = {{ -100, -50},{5, -90}}, lineColor = {0,0,0}, textString = "%k3"),Text(extent = {{2,36},{100, -44}}, lineColor = {0,0,0}, textString = "+")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100, -100},{100,100}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -100,50},{5,90}}, lineColor = {0,0,0}, textString = "%k1"),Text(extent = {{ -100, -20},{5,20}}, lineColor = {0,0,0}, textString = "%k2"),Text(extent = {{ -100, -50},{5, -90}}, lineColor = {0,0,0}, textString = "%k3"),Text(extent = {{2,36},{100, -44}}, lineColor = {0,0,0}, textString = "+"),Rectangle(extent = {{ -100, -100},{100,100}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -100,50},{5,90}}, lineColor = {0,0,0}, textString = "k1"),Text(extent = {{ -100, -20},{5,20}}, lineColor = {0,0,0}, textString = "k2"),Text(extent = {{ -100, -50},{5, -90}}, lineColor = {0,0,0}, textString = "k3"),Text(extent = {{2,36},{100, -44}}, lineColor = {0,0,0}, textString = "+")}));

      equation 
        y = k1 * u1 + k2 * u2 + k3 * u3;
      end Add3;
      block Product "Output product of the two inputs"
        extends Interfaces.SI2SO;
        annotation(Documentation(info = "
<HTML>
<p>
This blocks computes the output <b>y</b> (element-wise)
as <i>product</i> of the corresponding elements of
the two inputs <b>u1</b> and <b>u2</b>:
</p>
<pre>
    y = u1 * u2;
</pre>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -100,60},{ -40,60},{ -30,40}}, color = {0,0,127}),Line(points = {{ -100, -60},{ -40, -60},{ -30, -40}}, color = {0,0,127}),Line(points = {{50,0},{100,0}}, color = {0,0,127}),Line(points = {{ -30,0},{30,0}}, color = {0,0,0}),Line(points = {{ -15,25.99},{15, -25.99}}, color = {0,0,0}),Line(points = {{ -15, -25.99},{15,25.99}}, color = {0,0,0}),Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100, -100},{100,100}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -100,60},{ -40,60},{ -30,40}}, color = {0,0,255}),Line(points = {{ -100, -60},{ -40, -60},{ -30, -40}}, color = {0,0,255}),Line(points = {{50,0},{100,0}}, color = {0,0,255}),Line(points = {{ -30,0},{30,0}}, color = {0,0,0}),Line(points = {{ -15,25.99},{15, -25.99}}, color = {0,0,0}),Line(points = {{ -15, -25.99},{15,25.99}}, color = {0,0,0}),Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,255})}));

      equation 
        y = u1 * u2;
      end Product;
      block Division "Output first input divided by second input"
        extends Interfaces.SI2SO;
        annotation(Documentation(info = "
<HTML>
<p>
This block computes the output <b>y</b> (element-wise)
by <i>dividing</i> the corresponding elements of
the two inputs <b>u1</b> and <b>u2</b>:
</p>
<pre>
    y = u1 / u2;
</pre>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{50,0},{100,0}}, color = {0,0,127}),Line(points = {{ -30,0},{30,0}}, color = {0,0,0}),Ellipse(extent = {{ -5,20},{5,30}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -5, -20},{5, -30}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,127}),Text(extent = {{ -150,150},{150,110}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -100,60},{ -66,60},{ -40,30}}, color = {0,0,127}),Line(points = {{ -100, -60},{0, -60},{0, -50}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100, -100},{100,100}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{50,0},{100,0}}, color = {0,0,255}),Line(points = {{ -30,0},{30,0}}, color = {0,0,0}),Ellipse(extent = {{ -5,20},{5,30}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -5, -20},{5, -30}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -50,50},{50, -50}}, lineColor = {0,0,255}),Line(points = {{ -100,60},{ -66,60},{ -40,30}}, color = {0,0,255}),Line(points = {{ -100, -60},{0, -60},{0, -50}}, color = {0,0,255})}));

      equation 
        y = u1 / u2;
      end Division;
      block Abs "Output the absolute value of the input"
        extends Interfaces.SISO;
        annotation(defaultComponentName = "abs1", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Polygon(points = {{92,0},{70,8},{70, -8},{92,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,80},{0,0},{80,80}}, color = {0,0,0}),Line(points = {{0, -14},{0,68}}, color = {192,192,192}),Polygon(points = {{0,90},{ -8,68},{8,68},{0,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -34, -28},{38, -76}}, lineColor = {192,192,192}, textString = "abs"),Line(points = {{ -88,0},{76,0}}, color = {192,192,192})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -100,0},{76,0}}, color = {192,192,192}),Polygon(points = {{92,0},{76,6},{76, -6},{92,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,80},{0,0},{80,80}}, color = {0,0,0}),Line(points = {{0, -80},{0,68}}, color = {192,192,192}),Polygon(points = {{0,90},{ -8,68},{8,68},{0,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{0, -80},{0,68}}, color = {192,192,192}),Polygon(points = {{0,90},{ -8,68},{8,68},{0,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{7,98},{34,78}}, lineColor = {160,160,164}, textString = "u"),Text(extent = {{74, -8},{96, -28}}, lineColor = {160,160,164}, textString = "y"),Text(extent = {{52, -3},{72, -23}}, textString = "1", lineColor = {0,0,255}),Text(extent = {{ -86, -1},{ -66, -21}}, textString = "-1", lineColor = {0,0,255}),Text(extent = {{ -28,79},{ -8,59}}, textString = "1", lineColor = {0,0,255})}), Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b>
as <i>absolute value</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>abs</b>( u );
</pre>

</HTML>
"));

      equation 
        y = abs(u);
      end Abs;
      block Sign "Output the sign of the input"
        extends Interfaces.SISO;
        annotation(defaultComponentName = "sign1", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{0, -80}}, color = {0,0,0}),Polygon(points = {{0,90},{ -8,68},{8,68},{0,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{0, -80},{0,68}}, color = {192,192,192}),Text(extent = {{ -90,72},{ -18,24}}, lineColor = {192,192,192}, textString = "sign"),Line(points = {{0,80},{80,80}}, color = {0,0,0}),Rectangle(extent = {{ -2,2},{2, -4}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{0, -80}}, color = {0,0,0}),Line(points = {{ -0.01,0},{0.01,0}}, color = {0,0,0}),Line(points = {{0,80},{80,80}}, color = {0,0,0}),Rectangle(extent = {{ -2,2},{2, -4}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Polygon(points = {{0,100},{ -6,84},{6,84},{0,100}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{0, -90},{0,84}}, color = {192,192,192}),Text(extent = {{7,102},{32,82}}, lineColor = {160,160,164}, textString = "y"),Text(extent = {{70, -6},{94, -26}}, lineColor = {160,160,164}, textString = "u"),Text(extent = {{ -25,86},{ -5,70}}, textString = "1", lineColor = {0,0,255}),Text(extent = {{5, -72},{25, -88}}, textString = "-1", lineColor = {0,0,255})}), Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b>
as <b>sign</b> of the input <b>u</b>:
</p>
<pre>
         1  <b>if</b> u > 0
    y =  0  <b>if</b> u == 0
        -1  <b>if</b> u < 0
</pre>

</HTML>
"));

      equation 
        y = sign(u);
      end Sign;
      block Sqrt "Output the square root of the input (input >= 0 required)"
        extends Interfaces.SISO;
        annotation(defaultComponentName = "sqrt1", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -90, -80},{68, -80}}, color = {192,192,192}),Polygon(points = {{90, -80},{68, -72},{68, -88},{90, -80}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -79.2, -68.7},{ -78.4, -64},{ -76.8, -57.3},{ -73.6, -47.9},{ -67.9, -36.1},{ -59.1, -22.2},{ -46.2, -6.49},{ -28.5,10.7},{ -4.42,30},{27.7,51.3},{69.5,74.7},{80,80}}, color = {0,0,0}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -88},{ -80,68}}, color = {192,192,192}),Text(extent = {{ -8, -4},{64, -52}}, lineColor = {192,192,192}, textString = "sqrt")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -92, -80},{84, -80}}, color = {192,192,192}),Polygon(points = {{100, -80},{84, -74},{84, -86},{100, -80}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -79.2, -68.7},{ -78.4, -64},{ -76.8, -57.3},{ -73.6, -47.9},{ -67.9, -36.1},{ -59.1, -22.2},{ -46.2, -6.49},{ -28.5,10.7},{ -4.42,30},{27.7,51.3},{69.5,74.7},{80,80}}, color = {0,0,0}),Polygon(points = {{ -80,98},{ -86,82},{ -74,82},{ -80,98}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -90},{ -80,84}}, color = {192,192,192}),Text(extent = {{ -71,98},{ -44,78}}, lineColor = {160,160,164}, textString = "y"),Text(extent = {{60, -52},{84, -72}}, lineColor = {160,160,164}, textString = "u")}), Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b>
as <i>square root</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>sqrt</b>( u );
</pre>
<p>
All elements of the input vector shall be zero or positive.
Otherwise an error occurs.
</p>

</HTML>
"));

      equation 
        y = sqrt(u);
      end Sqrt;
      block Sin "Output the sine of the input"
        extends Interfaces.SISO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -80,68}}, color = {192,192,192}),Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Line(points = {{ -80,0},{ -68.7,34.2},{ -61.5,53.1},{ -55.1,66.4},{ -49.4,74.6},{ -43.8,79.1},{ -38.2,79.8},{ -32.6,76.6},{ -26.9,69.7},{ -21.3,59.4},{ -14.9,44.1},{ -6.83,21.2},{10.1, -30.8},{17.3, -50.2},{23.7, -64.2},{29.3, -73.1},{35, -78.4},{40.6, -80},{46.2, -77.6},{51.9, -71.5},{57.5, -61.9},{63.9, -47.2},{72, -24.8},{80,0}}, color = {0,0,0}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{12,84},{84,36}}, lineColor = {192,192,192}, textString = "sin")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -80,80},{ -88,80}}, color = {192,192,192}),Line(points = {{ -80, -80},{ -88, -80}}, color = {192,192,192}),Line(points = {{ -80, -90},{ -80,84}}, color = {192,192,192}),Text(extent = {{ -75,98},{ -46,78}}, lineColor = {160,160,164}, textString = "y"),Polygon(points = {{ -80,96},{ -86,80},{ -74,80},{ -80,96}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -100,0},{84,0}}, color = {192,192,192}),Polygon(points = {{100,0},{84,6},{84, -6},{100,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,0},{ -68.7,34.2},{ -61.5,53.1},{ -55.1,66.4},{ -49.4,74.6},{ -43.8,79.1},{ -38.2,79.8},{ -32.6,76.6},{ -26.9,69.7},{ -21.3,59.4},{ -14.9,44.1},{ -6.83,21.2},{10.1, -30.8},{17.3, -50.2},{23.7, -64.2},{29.3, -73.1},{35, -78.4},{40.6, -80},{46.2, -77.6},{51.9, -71.5},{57.5, -61.9},{63.9, -47.2},{72, -24.8},{80,0}}, color = {0,0,0}),Text(extent = {{ -105,72},{ -85,88}}, textString = "1", lineColor = {0,0,255}),Text(extent = {{70,25},{90,5}}, textString = "2*pi", lineColor = {0,0,255}),Text(extent = {{ -105, -72},{ -85, -88}}, textString = "-1", lineColor = {0,0,255}),Text(extent = {{76, -10},{98, -30}}, lineColor = {160,160,164}, textString = "u")}), Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b>
as <b>sine</b> of the input <b>u</b>:
</p>
<pre>
    y = <b>sin</b>( u );
</pre>

<p>
<img src=\"../Images/Math/sin.png\">
</p>

</HTML>
"));

      equation 
        y = Modelica.Math.sin(u);
      end Sin;
      block Cos "Output the cosine of the input"
        extends Interfaces.SISO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -80,68}}, color = {192,192,192}),Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,80},{ -74.4,78.1},{ -68.7,72.3},{ -63.1,63},{ -56.7,48.7},{ -48.6,26.6},{ -29.3, -32.5},{ -22.1, -51.7},{ -15.7, -65.3},{ -10.1, -73.8},{ -4.42, -78.8},{1.21, -79.9},{6.83, -77.1},{12.5, -70.6},{18.1, -60.6},{24.5, -45.7},{32.6, -23},{50.3,31.3},{57.5,50.7},{63.9,64.6},{69.5,73.4},{75.2,78.6},{80,80}}, color = {0,0,0}),Text(extent = {{ -36,82},{36,34}}, lineColor = {192,192,192}, textString = "cos")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -80,80},{ -88,80}}, color = {192,192,192}),Line(points = {{ -80, -80},{ -88, -80}}, color = {192,192,192}),Line(points = {{ -80, -90},{ -80,84}}, color = {192,192,192}),Text(extent = {{ -75,102},{ -48,82}}, lineColor = {160,160,164}, textString = "y"),Polygon(points = {{ -80,100},{ -86,84},{ -74,84},{ -80,100}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -103,72},{ -83,88}}, textString = "1", lineColor = {0,0,255}),Text(extent = {{ -79, -72},{ -59, -88}}, textString = "-1", lineColor = {0,0,255}),Text(extent = {{70,25},{90,5}}, textString = "2*pi", lineColor = {0,0,255}),Line(points = {{ -100,0},{84,0}}, color = {192,192,192}),Polygon(points = {{100,0},{84,6},{84, -6},{100,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,80},{ -74.4,78.1},{ -68.7,72.3},{ -63.1,63},{ -56.7,48.7},{ -48.6,26.6},{ -29.3, -32.5},{ -22.1, -51.7},{ -15.7, -65.3},{ -10.1, -73.8},{ -4.42, -78.8},{1.21, -79.9},{6.83, -77.1},{12.5, -70.6},{18.1, -60.6},{24.5, -45.7},{32.6, -23},{50.3,31.3},{57.5,50.7},{63.9,64.6},{69.5,73.4},{75.2,78.6},{80,80}}, color = {0,0,0}),Text(extent = {{74, -4},{98, -24}}, lineColor = {160,160,164}, textString = "u")}), Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b>
as <b>cos</b> of the input <b>u</b>:
</p>
<pre>
    y = <b>cos</b>( u );
</pre>

<p>
<img src=\"../Images/Math/cos.png\">
</p>

</HTML>
"));

      equation 
        y = Modelica.Math.cos(u);
      end Cos;
      block Tan "Output the tangent of the input"
        extends Interfaces.SISO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Polygon(points = {{0,90},{ -8,68},{8,68},{0,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{0, -80},{0,68}}, color = {192,192,192}),Line(points = {{ -80, -80},{ -78.4, -68.4},{ -76.8, -59.7},{ -74.4, -50},{ -71.2, -40.9},{ -67.1, -33},{ -60.7, -24.8},{ -51.1, -17.2},{ -35.8, -9.98},{ -4.42, -1.07},{33.4,9.12},{49.4,16.2},{59.1,23.2},{65.5,30.6},{70.4,39.1},{73.6,47.4},{76,56.1},{77.6,63.8},{80,80}}, color = {0,0,0}),Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -90,72},{ -18,24}}, lineColor = {192,192,192}, textString = "tan")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{0,80},{ -8,80}}, color = {192,192,192}),Line(points = {{0, -80},{ -8, -80}}, color = {192,192,192}),Line(points = {{0, -88},{0,86}}, color = {192,192,192}),Text(extent = {{11,100},{38,80}}, lineColor = {160,160,164}, textString = "y"),Polygon(points = {{0,102},{ -6,86},{6,86},{0,102}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -37, -72},{ -17, -88}}, textString = "-5.8", lineColor = {0,0,255}),Text(extent = {{ -33,86},{ -13,70}}, textString = " 5.8", lineColor = {0,0,255}),Text(extent = {{70,25},{90,5}}, textString = "1.4", lineColor = {0,0,255}),Line(points = {{ -100,0},{84,0}}, color = {192,192,192}),Polygon(points = {{100,0},{84,6},{84, -6},{100,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -78.4, -68.4},{ -76.8, -59.7},{ -74.4, -50},{ -71.2, -40.9},{ -67.1, -33},{ -60.7, -24.8},{ -51.1, -17.2},{ -35.8, -9.98},{ -4.42, -1.07},{33.4,9.12},{49.4,16.2},{59.1,23.2},{65.5,30.6},{70.4,39.1},{73.6,47.4},{76,56.1},{77.6,63.8},{80,80}}, color = {0,0,0}),Text(extent = {{70, -6},{94, -26}}, lineColor = {160,160,164}, textString = "u")}), Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b>
as <b>tan</b> of the input <b>u</b>:
</p>
<pre>
    y = <b>tan</b>( u );
</pre>

<p>
<img src=\"../Images/Math/tan.png\">
</p>

</HTML>
"));

      equation 
        y = Modelica.Math.tan(u);
      end Tan;
      block Asin "Output the arc sine of the input"
        extends Interfaces.SISO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Polygon(points = {{0,90},{ -8,68},{8,68},{0,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{0, -80},{0,68}}, color = {192,192,192}),Line(points = {{ -80, -80},{ -79.2, -72.8},{ -77.6, -67.5},{ -73.6, -59.4},{ -66.3, -49.8},{ -53.5, -37.3},{ -30.2, -19.7},{37.4,24.8},{57.5,40.8},{68.7,52.7},{75.2,62.2},{77.6,67.5},{80,80}}, color = {0,0,0}),Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -88,78},{ -16,30}}, lineColor = {192,192,192}, textString = "asin")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{0,80},{ -8,80}}, color = {192,192,192}),Line(points = {{0, -80},{ -8, -80}}, color = {192,192,192}),Line(points = {{0, -90},{0,84}}, color = {192,192,192}),Text(extent = {{9,102},{36,82}}, lineColor = {160,160,164}, textString = "y"),Polygon(points = {{0,100},{ -6,84},{6,84},{0,100}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -40, -72},{ -15, -88}}, textString = "-pi/2", lineColor = {0,0,255}),Text(extent = {{ -38,88},{ -13,72}}, textString = " pi/2", lineColor = {0,0,255}),Text(extent = {{70,25},{90,5}}, textString = "+1", lineColor = {0,0,255}),Text(extent = {{ -90,21},{ -70,1}}, textString = "-1", lineColor = {0,0,255}),Line(points = {{ -100,0},{84,0}}, color = {192,192,192}),Polygon(points = {{100,0},{84,6},{84, -6},{100,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -79.2, -72.8},{ -77.6, -67.5},{ -73.6, -59.4},{ -66.3, -49.8},{ -53.5, -37.3},{ -30.2, -19.7},{37.4,24.8},{57.5,40.8},{68.7,52.7},{75.2,62.2},{77.6,67.5},{80,80}}, color = {0,0,0}),Text(extent = {{74, -4},{100, -26}}, lineColor = {160,160,164}, textString = "u")}), Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>sine-inverse</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>asin</b>( u );
</pre>
<p>
The absolute values of the elements of the input <b>u</b> need to
be less or equal to one (<b>abs</b>( u ) <= 1).
Otherwise an error occurs.
</p>

<p>
<img src=\"../Images/Math/asin.png\">
</p>

</HTML>
"));

      equation 
        y = Modelica.Math.asin(u);
      end Asin;
      block Acos "Output the arc cosine of the input"
        extends Interfaces.SISO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Polygon(points = {{0,90},{ -8,68},{8,68},{0,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,80},{ -79.2,72.8},{ -77.6,67.5},{ -73.6,59.4},{ -66.3,49.8},{ -53.5,37.3},{ -30.2,19.7},{37.4, -24.8},{57.5, -40.8},{68.7, -52.7},{75.2, -62.2},{77.6, -67.5},{80, -80}}, color = {0,0,0}),Line(points = {{0, -88},{0,68}}, color = {192,192,192}),Line(points = {{ -90, -80},{68, -80}}, color = {192,192,192}),Polygon(points = {{90, -80},{68, -72},{68, -88},{90, -80}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -86, -14},{ -14, -62}}, lineColor = {192,192,192}, textString = "acos")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{0,80},{ -8,80}}, color = {192,192,192}),Line(points = {{0, -80},{ -8, -80}}, color = {192,192,192}),Line(points = {{0, -90},{0,84}}, color = {192,192,192}),Text(extent = {{13,100},{42,82}}, lineColor = {160,160,164}, textString = "y"),Polygon(points = {{0,100},{ -6,84},{6,84},{0,100}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -100, -80},{84, -80}}, color = {192,192,192}),Polygon(points = {{100, -80},{84, -74},{84, -86},{100, -80}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,80},{ -79.2,72.8},{ -77.6,67.5},{ -73.6,59.4},{ -66.3,49.8},{ -53.5,37.3},{ -30.2,19.7},{37.4, -24.8},{57.5, -40.8},{68.7, -52.7},{75.2, -62.2},{77.6, -67.5},{80, -80}}, color = {0,0,0}),Text(extent = {{ -30,88},{ -5,72}}, textString = " pi", lineColor = {0,0,255}),Text(extent = {{ -94, -57},{ -74, -77}}, textString = "-1", lineColor = {0,0,255}),Text(extent = {{80, -45},{100, -65}}, textString = "+1", lineColor = {0,0,255}),Text(extent = {{76, -84},{102, -102}}, lineColor = {160,160,164}, textString = "u")}), Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>cosine-inverse</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>acos</b>( u );
</pre>
<p>
The absolute values of the elements of the input <b>u</b> need to
be less or equal to one (<b>abs</b>( u ) <= 1).
Otherwise an error occurs.
</p>

<p>
<img src=\"../Images/Math/acos.png\">
</p>

</HTML>
"));

      equation 
        y = Modelica.Math.acos(u);
      end Acos;
      block Atan "Output the arc tangent of the input"
        extends Interfaces.SISO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Polygon(points = {{0,90},{ -8,68},{8,68},{0,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{0, -80},{0,68}}, color = {192,192,192}),Line(points = {{ -80, -80},{ -52.7, -75.2},{ -37.4, -69.7},{ -26.9, -63},{ -19.7, -55.2},{ -14.1, -45.8},{ -10.1, -36.4},{ -6.03, -23.9},{ -1.21, -5.06},{5.23,21},{9.25,34.1},{13.3,44.2},{18.1,52.9},{24.5,60.8},{33.4,67.6},{47,73.6},{69.5,78.6},{80,80}}, color = {0,0,0}),Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -86,68},{ -14,20}}, lineColor = {192,192,192}, textString = "atan")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{0,80},{ -8,80}}, color = {192,192,192}),Line(points = {{0, -80},{ -8, -80}}, color = {192,192,192}),Line(points = {{0, -90},{0,84}}, color = {192,192,192}),Text(extent = {{13,102},{42,82}}, lineColor = {160,160,164}, textString = "y"),Polygon(points = {{0,100},{ -6,84},{6,84},{0,100}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -100,0},{84,0}}, color = {192,192,192}),Polygon(points = {{100,0},{84,6},{84, -6},{100,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -52.7, -75.2},{ -37.4, -69.7},{ -26.9, -63},{ -19.7, -55.2},{ -14.1, -45.8},{ -10.1, -36.4},{ -6.03, -23.9},{ -1.21, -5.06},{5.23,21},{9.25,34.1},{13.3,44.2},{18.1,52.9},{24.5,60.8},{33.4,67.6},{47,73.6},{69.5,78.6},{80,80}}, color = {0,0,0}),Text(extent = {{ -32,91},{ -12,71}}, textString = "1.4", lineColor = {0,0,255}),Text(extent = {{ -32, -71},{ -12, -91}}, textString = "-1.4", lineColor = {0,0,255}),Text(extent = {{73,26},{93,10}}, textString = " 5.8", lineColor = {0,0,255}),Text(extent = {{ -103,20},{ -83,4}}, textString = "-5.8", lineColor = {0,0,255}),Text(extent = {{66, -8},{94, -28}}, lineColor = {160,160,164}, textString = "u")}), Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>tangent-inverse</i> of the input <b>u</b>:
</p>
<pre>
    y= <b>atan</b>( u );
</pre>

<p>
<img src=\"../Images/Math/atan.png\">
</p>

</HTML>
"));

      equation 
        y = Modelica.Math.atan(u);
      end Atan;
      block Atan2 "Output atan(u1/u2) of the inputs u1 and u2"
        extends Interfaces.SI2SO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -150,150},{150,110}}, textString = "%name", lineColor = {0,0,255}),Polygon(points = {{0,90},{ -8,68},{8,68},{0,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{0, -80},{0,68}}, color = {192,192,192}),Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -34.9},{ -46.1, -31.4},{ -29.4, -27.1},{ -18.3, -21.5},{ -10.3, -14.5},{ -2.03, -3.17},{7.97,11.6},{15.5,19.4},{24.3,25},{39,30},{62.1,33.5},{80,34.9}}, color = {0,0,0}),Line(points = {{ -80,45.1},{ -45.9,48.7},{ -29.1,52.9},{ -18.1,58.6},{ -10.2,65.8},{ -1.82,77.2},{0,80}}, color = {0,0,0}),Line(points = {{0, -80},{8.93, -67.2},{17.1, -59.3},{27.3, -53.6},{42.1, -49.4},{69.9, -45.8},{80, -45.1}}, color = {0,0,0}),Text(extent = {{ -90, -46},{ -18, -94}}, lineColor = {192,192,192}, textString = "atan2")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{0,80},{ -8,80}}, color = {192,192,192}),Line(points = {{0, -80},{ -8, -80}}, color = {192,192,192}),Line(points = {{0, -90},{0,84}}, color = {192,192,192}),Text(extent = {{11,98},{42,78}}, lineColor = {160,160,164}, textString = "y"),Polygon(points = {{0,100},{ -6,84},{6,84},{0,100}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -100,0},{84,0}}, color = {192,192,192}),Polygon(points = {{100,0},{84,6},{84, -6},{100,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{0, -80},{8.93, -67.2},{17.1, -59.3},{27.3, -53.6},{42.1, -49.4},{69.9, -45.8},{80, -45.1}}, color = {0,0,0}),Line(points = {{ -80, -34.9},{ -46.1, -31.4},{ -29.4, -27.1},{ -18.3, -21.5},{ -10.3, -14.5},{ -2.03, -3.17},{7.97,11.6},{15.5,19.4},{24.3,25},{39,30},{62.1,33.5},{80,34.9}}, color = {0,0,0}),Line(points = {{ -80,45.1},{ -45.9,48.7},{ -29.1,52.9},{ -18.1,58.6},{ -10.2,65.8},{ -1.82,77.2},{0,80}}, color = {0,0,0}),Text(extent = {{ -30,89},{ -10,70}}, textString = "pi", lineColor = {0,0,255}),Text(extent = {{ -30, -69},{ -10, -88}}, textString = "-pi", lineColor = {0,0,255}),Text(extent = {{ -30,49},{ -10,30}}, textString = "pi/2", lineColor = {0,0,255}),Line(points = {{0,40},{ -8,40}}, color = {192,192,192}),Line(points = {{0, -40},{ -8, -40}}, color = {192,192,192}),Text(extent = {{ -30, -31},{ -10, -50}}, textString = "-pi/2", lineColor = {0,0,255}),Text(extent = {{48,0},{100, -34}}, lineColor = {160,160,164}, textString = "u1 / u2")}), Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>tangent-inverse</i> of the input <b>u1</b> divided by
input <b>u2</b>:
</p>
<pre>
    y = <b>atan2</b>( u1, u2 );
</pre>
<p>
u1 and u2 shall not be zero at the same time instant.
<b>Atan2</b> uses the sign of u1 and u2 in order to construct
the solution in the range -180 deg &le; y &le; 180 deg, whereas
block <b>Atan</b> gives a solution in the range
-90 deg &le; y &le; 90 deg.
</p>

<p>
<img src=\"../Images/Math/atan2.png\">
</p>

</HTML>
"));

      equation 
        y = Modelica.Math.atan2(u1, u2);
      end Atan2;
      block Sinh "Output the hyperbolic sine of the input"
        extends Interfaces.SISO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Polygon(points = {{0,90},{ -8,68},{8,68},{0,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -86,80},{ -14,32}}, lineColor = {192,192,192}, textString = "sinh"),Line(points = {{0, -80},{0,68}}, color = {192,192,192}),Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Line(points = {{ -80, -80},{ -76, -65.4},{ -71.2, -51.4},{ -65.5, -38.8},{ -59.1, -28.1},{ -51.1, -18.7},{ -41.4, -11.4},{ -27.7, -5.5},{ -4.42, -0.653},{24.5,4.57},{39,10.1},{49.4,17.2},{57.5,25.9},{63.9,35.8},{69.5,47.4},{74.4,60.4},{78.4,73.8},{80,80}}, color = {0,0,0}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{0,80},{ -8,80}}, color = {192,192,192}),Line(points = {{0, -80},{ -8, -80}}, color = {192,192,192}),Line(points = {{0, -90},{0,84}}, color = {192,192,192}),Text(extent = {{9,98},{34,78}}, lineColor = {160,160,164}, textString = "y"),Polygon(points = {{0,100},{ -6,84},{6,84},{0,100}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -100,0},{84,0}}, color = {192,192,192}),Polygon(points = {{100,0},{84,6},{84, -6},{100,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -76, -65.4},{ -71.2, -51.4},{ -65.5, -38.8},{ -59.1, -28.1},{ -51.1, -18.7},{ -41.4, -11.4},{ -27.7, -5.5},{ -4.42, -0.653},{24.5,4.57},{39,10.1},{49.4,17.2},{57.5,25.9},{63.9,35.8},{69.5,47.4},{74.4,60.4},{78.4,73.8},{80,80}}, color = {0,0,0}),Text(extent = {{ -31,72},{ -11,88}}, textString = "27", lineColor = {0,0,255}),Text(extent = {{ -35, -88},{ -15, -72}}, textString = "-27", lineColor = {0,0,255}),Text(extent = {{70,25},{90,5}}, textString = "4", lineColor = {0,0,255}),Text(extent = {{ -98,21},{ -78,1}}, textString = "-4", lineColor = {0,0,255}),Text(extent = {{72, -6},{96, -26}}, lineColor = {160,160,164}, textString = "u")}), Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>hyperbolic sine</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>sinh</b>( u );
</pre>

<p>
<img src=\"../Images/Math/sinh.png\">
</p>

</HTML>
"));

      equation 
        y = Modelica.Math.sinh(u);
      end Sinh;
      block Cosh "Output the hyperbolic cosine of the input"
        extends Interfaces.SISO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Polygon(points = {{0,90},{ -8,68},{8,68},{0,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{0, -80},{0,68}}, color = {192,192,192}),Text(extent = {{4,66},{66,20}}, lineColor = {192,192,192}, textString = "cosh"),Line(points = {{ -80,80},{ -77.6,61.1},{ -74.4,39.3},{ -71.2,20.7},{ -67.1,1.29},{ -63.1, -14.6},{ -58.3, -29.8},{ -52.7, -43.5},{ -46.2, -55.1},{ -39, -64.3},{ -30.2, -71.7},{ -18.9, -77.1},{ -4.42, -79.9},{10.9, -79.1},{23.7, -75.2},{34.2, -68.7},{42.2, -60.6},{48.6, -51.2},{54.3, -40},{59.1, -27.5},{63.1, -14.6},{67.1,1.29},{71.2,20.7},{74.4,39.3},{77.6,61.1},{80,80}}, color = {0,0,0}),Line(points = {{ -90, -86.083},{68, -86.083}}, color = {192,192,192}),Polygon(points = {{90, -86.083},{68, -78.083},{68, -94.083},{90, -86.083}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{0,80},{ -8,80}}, color = {192,192,192}),Line(points = {{0, -80},{ -8, -80}}, color = {192,192,192}),Line(points = {{0, -90},{0,84}}, color = {192,192,192}),Text(extent = {{11,98},{38,78}}, lineColor = {160,160,164}, textString = "y"),Polygon(points = {{0,100},{ -6,84},{6,84},{0,100}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -100, -86.083},{84, -86.083}}, color = {192,192,192}),Polygon(points = {{100, -86.083},{84, -80.083},{84, -92.083},{100, -86.083}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,80},{ -77.6,61.1},{ -74.4,39.3},{ -71.2,20.7},{ -67.1,1.29},{ -63.1, -14.6},{ -58.3, -29.8},{ -52.7, -43.5},{ -46.2, -55.1},{ -39, -64.3},{ -30.2, -71.7},{ -18.9, -77.1},{ -4.42, -79.9},{10.9, -79.1},{23.7, -75.2},{34.2, -68.7},{42.2, -60.6},{48.6, -51.2},{54.3, -40},{59.1, -27.5},{63.1, -14.6},{67.1,1.29},{71.2,20.7},{74.4,39.3},{77.6,61.1},{80,80}}, color = {0,0,0}),Text(extent = {{ -31,72},{ -11,88}}, textString = "27", lineColor = {0,0,255}),Text(extent = {{62, -67},{82, -87}}, textString = "4", lineColor = {0,0,255}),Text(extent = {{ -104, -63},{ -84, -83}}, textString = "-4", lineColor = {0,0,255}),Text(extent = {{78, -54},{100, -74}}, lineColor = {160,160,164}, textString = "u")}), Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>hyperbolic cosine</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>cosh</b>( u );
</pre>

<p>
<img src=\"../Images/Math/cosh.png\">
</p>

</HTML>
"));

      equation 
        y = Modelica.Math.cosh(u);
      end Cosh;
      block Tanh "Output the hyperbolic tangent of the input"
        extends Interfaces.SISO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{0, -90},{0,84}}, color = {192,192,192}),Line(points = {{ -100,0},{84,0}}, color = {192,192,192}),Line(points = {{ -80, -80},{ -47.8, -78.7},{ -35.8, -75.7},{ -27.7, -70.6},{ -22.1, -64.2},{ -17.3, -55.9},{ -12.5, -44.3},{ -7.64, -29.2},{ -1.21, -4.82},{6.83,26.3},{11.7,42},{16.5,54.2},{21.3,63.1},{26.9,69.9},{34.2,75},{45.4,78.4},{72,79.9},{80,80}}, color = {0,0,0}),Polygon(points = {{0,90},{ -8,68},{8,68},{0,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -88,72},{ -16,24}}, lineColor = {192,192,192}, textString = "tanh"),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{0,80},{ -8,80}}, color = {192,192,192}),Line(points = {{0, -80},{ -8, -80}}, color = {192,192,192}),Line(points = {{0, -90},{0,84}}, color = {192,192,192}),Text(extent = {{9,98},{42,80}}, lineColor = {160,160,164}, textString = "y"),Polygon(points = {{0,100},{ -6,84},{6,84},{0,100}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -100,0},{84,0}}, color = {192,192,192}),Polygon(points = {{100,0},{84,6},{84, -6},{100,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -47.8, -78.7},{ -35.8, -75.7},{ -27.7, -70.6},{ -22.1, -64.2},{ -17.3, -55.9},{ -12.5, -44.3},{ -7.64, -29.2},{ -1.21, -4.82},{6.83,26.3},{11.7,42},{16.5,54.2},{21.3,63.1},{26.9,69.9},{34.2,75},{45.4,78.4},{72,79.9},{80,80}}, color = {0,0,0}),Text(extent = {{70,25},{90,5}}, textString = "4", lineColor = {0,0,255}),Text(extent = {{ -106,21},{ -86,1}}, textString = "-4", lineColor = {0,0,255}),Text(extent = {{ -29,72},{ -9,88}}, textString = "1", lineColor = {0,0,255}),Text(extent = {{3, -72},{23, -88}}, textString = "-1", lineColor = {0,0,255}),Text(extent = {{72, -8},{98, -26}}, lineColor = {160,160,164}, textString = "u")}), Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>hyperbolic tangent</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>tanh</b>( u );
</pre>

<p>
<img src=\"../Images/Math/tanh.png\">
</p>

</HTML>
"));

      equation 
        y = Modelica.Math.tanh(u);
      end Tanh;
      block Exp "Output the exponential (base e) of the input"
        extends Interfaces.SISO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{0, -80},{0,68}}, color = {192,192,192}),Polygon(points = {{0,90},{ -8,68},{8,68},{0,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -86,50},{ -14,2}}, lineColor = {192,192,192}, textString = "exp"),Line(points = {{ -80, -80},{ -31, -77.9},{ -6.03, -74},{10.9, -68.4},{23.7, -61},{34.2, -51.6},{43, -40.3},{50.3, -27.8},{56.7, -13.5},{62.3,2.23},{67.1,18.6},{72,38.2},{76,57.6},{80,80}}, color = {0,0,0}),Line(points = {{ -90, -80.3976},{68, -80.3976}}, color = {192,192,192}),Polygon(points = {{90, -80.3976},{68, -72.3976},{68, -88.3976},{90, -80.3976}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{0,80},{ -8,80}}, color = {192,192,192}),Line(points = {{0, -80},{ -8, -80}}, color = {192,192,192}),Line(points = {{0, -90},{0,84}}, color = {192,192,192}),Text(extent = {{9,100},{40,80}}, lineColor = {160,160,164}, textString = "y"),Polygon(points = {{0,100},{ -6,84},{6,84},{0,100}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -100, -80.3976},{84, -80.3976}}, color = {192,192,192}),Polygon(points = {{100, -80.3976},{84, -74.3976},{84, -86.3976},{100, -80.3976}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -31, -77.9},{ -6.03, -74},{10.9, -68.4},{23.7, -61},{34.2, -51.6},{43, -40.3},{50.3, -27.8},{56.7, -13.5},{62.3,2.23},{67.1,18.6},{72,38.2},{76,57.6},{80,80}}, color = {0,0,0}),Text(extent = {{ -31,72},{ -11,88}}, textString = "20", lineColor = {0,0,255}),Text(extent = {{ -92, -83},{ -72, -103}}, textString = "-3", lineColor = {0,0,255}),Text(extent = {{70, -83},{90, -103}}, textString = "3", lineColor = {0,0,255}),Text(extent = {{ -18, -53},{2, -73}}, textString = "1", lineColor = {0,0,255}),Text(extent = {{66, -52},{96, -72}}, lineColor = {160,160,164}, textString = "u")}), Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>exponential</i> (of base e) of the input <b>u</b>:
</p>
<pre>
    y = <b>exp</b>( u );
</pre>

<p>
<img src=\"../Images/Math/exp.png\">
</p>

</HTML>
"));

      equation 
        y = Modelica.Math.exp(u);
      end Exp;
      block Log "Output the natural (base e) logarithm of the input (input > 0 required)"
        extends Interfaces.SISO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -80, -80},{ -80,68}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -79.2, -50.6},{ -78.4, -37},{ -77.6, -28},{ -76.8, -21.3},{ -75.2, -11.4},{ -72.8, -1.31},{ -69.5,8.08},{ -64.7,17.9},{ -57.5,28},{ -47,38.1},{ -31.8,48.1},{ -10.1,58},{22.1,68},{68.7,78.1},{80,80}}, color = {0,0,0}),Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -6, -24},{66, -72}}, lineColor = {192,192,192}, textString = "log")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -80,80},{ -88,80}}, color = {192,192,192}),Line(points = {{ -80, -80},{ -88, -80}}, color = {192,192,192}),Line(points = {{ -80, -90},{ -80,84}}, color = {192,192,192}),Text(extent = {{ -67,98},{ -40,80}}, lineColor = {160,160,164}, textString = "y"),Polygon(points = {{ -80,100},{ -86,84},{ -74,84},{ -80,100}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -100,0},{84,0}}, color = {192,192,192}),Polygon(points = {{100,0},{84,6},{84, -6},{100,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -79.2, -50.6},{ -78.4, -37},{ -77.6, -28},{ -76.8, -21.3},{ -75.2, -11.4},{ -72.8, -1.31},{ -69.5,8.08},{ -64.7,17.9},{ -57.5,28},{ -47,38.1},{ -31.8,48.1},{ -10.1,58},{22.1,68},{68.7,78.1},{80,80}}, color = {0,0,0}),Text(extent = {{ -105,72},{ -85,88}}, textString = "3", lineColor = {0,0,255}),Text(extent = {{ -109, -88},{ -89, -72}}, textString = "-3", lineColor = {0,0,255}),Text(extent = {{70, -3},{90, -23}}, textString = "20", lineColor = {0,0,255}),Text(extent = {{ -78, -1},{ -58, -21}}, textString = "1", lineColor = {0,0,255}),Text(extent = {{68,28},{94,8}}, lineColor = {160,160,164}, textString = "u")}), Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>natural (base e) logarithm</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>log</b>( u );
</pre>
<p>
An error occurs if the elements of the input <b>u</b> are
zero or negative.
</p>

<p>
<img src=\"../Images/Math/log.png\">
</p>

</HTML>
"));

      equation 
        y = Modelica.Math.log(u);
      end Log;
      block Log10 "Output the base 10 logarithm of the input (input > 0 required)"
        extends Interfaces.SISO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Polygon(points = {{90,0},{68,8},{68, -8},{90,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{68,0}}, color = {192,192,192}),Line(points = {{ -79.8, -80},{ -79.2, -50.6},{ -78.4, -37},{ -77.6, -28},{ -76.8, -21.3},{ -75.2, -11.4},{ -72.8, -1.31},{ -69.5,8.08},{ -64.7,17.9},{ -57.5,28},{ -47,38.1},{ -31.8,48.1},{ -10.1,58},{22.1,68},{68.7,78.1},{80,80}}, color = {0,0,0}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -80,68}}, color = {192,192,192}),Text(extent = {{ -30, -22},{60, -70}}, lineColor = {192,192,192}, textString = "log10")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -80,80},{ -88,80}}, color = {192,192,192}),Line(points = {{ -80, -80},{ -88, -80}}, color = {192,192,192}),Line(points = {{ -80, -90},{ -80,84}}, color = {192,192,192}),Text(extent = {{ -65,96},{ -38,78}}, lineColor = {160,160,164}, textString = "y"),Polygon(points = {{ -80,100},{ -86,84},{ -74,84},{ -80,100}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -100,0},{84,0}}, color = {192,192,192}),Polygon(points = {{100,0},{84,6},{84, -6},{100,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -79.8, -80},{ -79.2, -50.6},{ -78.4, -37},{ -77.6, -28},{ -76.8, -21.3},{ -75.2, -11.4},{ -72.8, -1.31},{ -69.5,8.08},{ -64.7,17.9},{ -57.5,28},{ -47,38.1},{ -31.8,48.1},{ -10.1,58},{22.1,68},{68.7,78.1},{80,80}}, color = {0,0,0}),Text(extent = {{70, -3},{90, -23}}, textString = "20", lineColor = {0,0,255}),Text(extent = {{ -78, -1},{ -58, -21}}, textString = "1", lineColor = {0,0,255}),Text(extent = {{ -109,72},{ -89,88}}, textString = " 1.3", lineColor = {0,0,255}),Text(extent = {{ -109, -88},{ -89, -72}}, textString = "-1.3", lineColor = {0,0,255}),Text(extent = {{62,30},{90,10}}, lineColor = {160,160,164}, textString = "u")}), Documentation(info = "<HTML>
<p>
This blocks computes the output <b>y</b> as the
<i>base 10 logarithm</i> of the input <b>u</b>:
</p>
<pre>
    y = <b>log10</b>( u );
</pre>
<p>
An error occurs if the elements of the input <b>u</b> are
zero or negative.
</p>

<p>
<img src=\"../Images/Math/log10.png\">
</p>

</HTML>
"));

      equation 
        y = Modelica.Math.log10(u);
      end Log10;
      block RealToInteger "Convert Real to Integer signal"
        extends Interfaces.IntegerBlockIcon;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -100,60},{0, -60}}, textString = "R", lineColor = {0,0,255}),Text(extent = {{20,60},{120, -60}}, lineColor = {255,127,0}, textString = "I"),Polygon(points = {{50,0},{30,20},{30,10},{0,10},{0, -10},{30, -10},{30, -20},{50,0}}, lineColor = {255,127,0}, fillColor = {255,127,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>
This block computes the output <b>y</b>
as <i>nearest integer value</i> of the input <b>u</b>:
</p>
<pre>    y = <b>integer</b>( <b>floor</b>( u + 0.5 ) )  for  u &gt; 0;
    y = <b>integer</b>( <b>ceil </b>( u - 0.5 ) )  for  u &lt; 0;
</pre>
</html>
"));
      public 
        Interfaces.RealInput u "Connector of Real input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Interfaces.IntegerOutput y "Connector of Integer output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));

      equation 
        y = if u > 0 then integer(floor(u + 0.5)) else integer(ceil(u - 0.5));
      end RealToInteger;
      block IntegerToReal "Convert integer to real signals"
        extends Interfaces.BlockIcon;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -120,60},{ -20, -60}}, lineColor = {255,127,0}, textString = "I"),Text(extent = {{0,60},{100, -60}}, textString = "R", lineColor = {0,0,255}),Polygon(points = {{10,0},{ -10,20},{ -10,10},{ -40,10},{ -40, -10},{ -10, -10},{ -10, -20},{10,0}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255})}), Documentation(info = "<html>
<p>
This block computes the output <b>y</b>
as <i>Real equivalent</i> of the Integer input <b>u</b>:
</p>
<pre>    y = u;
</pre>
<p>where <b>u</b> is of Integer and <b>y</b> of Real type.</p>
</html>
"));
        Interfaces.IntegerInput u "Connector of Integer input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Interfaces.RealOutput y "Connector of Real output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));

      equation 
        y = u;
      end IntegerToReal;
      block BooleanToReal "Convert Boolean to Real signal"
        extends Interfaces.partialBooleanSI;
        parameter Real realTrue = 1.0 "Output signal for true Boolean input";
        parameter Real realFalse = 0.0 "Output signal for false Boolean input";
        Blocks.Interfaces.RealOutput y "Connector of Real output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<html>
<p>
This block computes the output <b>y</b>
as <i>Real equivalent</i> of the Boolean input <b>u</b>:
</p>
<pre>    y = <b>if</b> u <b>then</b> realTrue <b>else</b> realFalse;
</pre>
<p>where <b>u</b> is of Boolean and <b>y</b> of Real type,
and <b>realTrue</b> and <b>realFalse</b> are parameters.
</p>
</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -86,92},{ -6,10}}, lineColor = {255,0,255}, textString = "B"),Polygon(points = {{ -12, -46},{ -32, -26},{ -32, -36},{ -64, -36},{ -64, -56},{ -32, -56},{ -32, -66},{ -12, -46}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{8, -4},{92, -94}}, textString = "R", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));

      equation 
        y = if u then realTrue else realFalse;
      end BooleanToReal;
      block BooleanToInteger "Convert Boolean to Integer signal"
        extends Interfaces.partialBooleanSI;
        parameter Integer integerTrue = 1 "Output signal for true Boolean input";
        parameter Integer integerFalse = 0 "Output signal for false Boolean input";
        Blocks.Interfaces.IntegerOutput y "Connector of Integer output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<html>
<p>
This block computes the output <b>y</b>
as <i>Integer equivalent</i> of the Boolean input <b>u</b>:
</p>
<pre>    y = <b>if</b> u <b>then</b> integerTrue <b>else</b> integerFalse;
</pre>
<p>where <b>u</b> is of Boolean and <b>y</b> of Integer type,
and <b>integerTrue</b> and <b>integerFalse</b> are parameters.
</p>
</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -86,92},{ -6,10}}, lineColor = {255,0,255}, textString = "B"),Polygon(points = {{ -12, -46},{ -32, -26},{ -32, -36},{ -64, -36},{ -64, -56},{ -32, -56},{ -32, -66},{ -12, -46}}, lineColor = {255,170,85}, fillColor = {255,170,85}, fillPattern = FillPattern.Solid),Text(extent = {{8, -4},{92, -94}}, lineColor = {255,170,85}, textString = "I")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));

      equation 
        y = if u then integerTrue else integerFalse;
      end BooleanToInteger;
      block RealToBoolean "Convert Real to Boolean signal"
        Blocks.Interfaces.RealInput u "Connector of Real input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        extends Interfaces.partialBooleanSO;
        parameter Real threshold = 0.5 "Output signal y is true, if input u >= threshold";
        annotation(Documentation(info = "<html>
<p>
This block computes the Boolean output <b>y</b>
from the Real input <b> u</b> by the equation:
</p>

<pre>    y = u &ge; threshold;
</pre>

<p>
where <b>threshold</b> is a parameter.
</p>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -86,92},{ -6,10}}, lineColor = {0,0,255}, textString = "R"),Polygon(points = {{ -12, -46},{ -32, -26},{ -32, -36},{ -64, -36},{ -64, -56},{ -32, -56},{ -32, -66},{ -12, -46}}, lineColor = {255,0,255}, fillColor = {255,0,255}, fillPattern = FillPattern.Solid),Text(extent = {{8, -4},{92, -94}}, lineColor = {255,0,255}, textString = "B")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));

      equation 
        y = u >= threshold;
      end RealToBoolean;
      block IntegerToBoolean "Convert Integer to Boolean signal"
        Blocks.Interfaces.IntegerInput u "Connector of Integer input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        extends Interfaces.partialBooleanSO;
        parameter Integer threshold = 1 "Output signal y is true, if input u >= threshold";
        annotation(Documentation(info = "<html>
<p>
This block computes the Boolean output <b>y</b>
from the Integer input <b> u</b> by the equation:
</p>

<pre>    y = u &ge; threshold;
</pre>

<p>
where <b>threshold</b> is a parameter.
</p>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -86,92},{ -6,10}}, lineColor = {255,128,0}, textString = "I"),Polygon(points = {{ -12, -46},{ -32, -26},{ -32, -36},{ -64, -36},{ -64, -56},{ -32, -56},{ -32, -66},{ -12, -46}}, lineColor = {255,0,255}, fillColor = {255,0,255}, fillPattern = FillPattern.Solid),Text(extent = {{8, -4},{92, -94}}, lineColor = {255,0,255}, textString = "B")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));

      equation 
        y = u >= threshold;
      end IntegerToBoolean;
      block RectangularToPolar "Convert rectangular coordinates to polar coordinates"
        extends Modelica.Blocks.Interfaces.BlockIcon;
        Modelica.Blocks.Interfaces.RealInput u_re "Real part of rectangular representation" annotation(Placement(transformation(extent = {{ -140,40},{ -100,80}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u_im "Imaginary part of rectangular representation" annotation(Placement(transformation(extent = {{ -140, -80},{ -100, -40}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y_abs "Length of polar representation" annotation(Placement(transformation(extent = {{100,50},{120,70}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y_arg "Angle of polar representation" annotation(Placement(transformation(extent = {{100, -70},{120, -50}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(graphics = {Text(extent = {{ -90,80},{ -20,40}}, lineColor = {0,0,0}, textString = "re"),Text(extent = {{ -90, -40},{ -20, -80}}, lineColor = {0,0,0}, textString = "im"),Text(extent = {{20,80},{90,40}}, lineColor = {0,0,0}, textString = "abs"),Text(extent = {{20, -40},{90, -80}}, lineColor = {0,0,0}, textString = "arg")}), Documentation(info = "<html>
<p>
The input values of this block are the rectangular components
<code>u_re</code> and <code>u_im</code> of a phasor in two dimensions.
This block calculates the length <code>y_abs</code> and
the angle <code>y_arg</code> of the polar representation of this phasor.
</p>

<pre>
  y_abs = abs(u_re + j*u_im) = sqrt( u_re<sup>2</sup> + u_im<sup>2</sup> )
  y_arg = arg(u_re + j*u_im) = atan2(u_im, u_re)
</pre>
</html>"));

      equation 
        y_abs = sqrt(u_re * u_re + u_im * u_im);
        y_arg = Modelica.Math.atan2(u_im, u_re);
      end RectangularToPolar;
      block PolarToRectangular "Convert polar coordinates to rectangular coordinates"
        extends Modelica.Blocks.Interfaces.BlockIcon;
        Modelica.Blocks.Interfaces.RealInput u_abs "Length of polar representation" annotation(Placement(transformation(extent = {{ -140,40},{ -100,80}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u_arg "Angle of polar representation" annotation(Placement(transformation(extent = {{ -140, -80},{ -100, -40}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y_re "Real part of rectangular representation" annotation(Placement(transformation(extent = {{100,50},{120,70}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y_im "Imaginary part of rectangular representation" annotation(Placement(transformation(extent = {{100, -70},{120, -50}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Icon(graphics = {Text(extent = {{ -90,80},{ -20,40}}, lineColor = {0,0,0}, textString = "abs"),Text(extent = {{ -90, -40},{ -20, -80}}, lineColor = {0,0,0}, textString = "arg"),Text(extent = {{20,80},{90,40}}, lineColor = {0,0,0}, textString = "re"),Text(extent = {{20, -40},{90, -80}}, lineColor = {0,0,0}, textString = "im")}), Documentation(info = "<html>
<p>
The input values of this block are the polar components <code>uabs</code> and <code>uarg</code> of a phasor.
This block calculates the components <code>y_re</code> and <code>y_im</code> of the rectangular representation of this phasor.
</p>
<pre>
   y_re = u_abs * cos( u_arg )
   y_im = u_abs * sin( u_arg )
</pre>
</html>"));

      equation 
        y_re = u_abs * Modelica.Math.cos(u_arg);
        y_im = u_abs * Modelica.Math.sin(u_arg);
      end PolarToRectangular;
      block Max "Pass through the largest signal"
        extends Interfaces.SI2SO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -90,36},{90, -36}}, lineColor = {160,160,164}, textString = "max()")}), Documentation(info = "<html>
<p>
This block computes the output <b>y</b> as <i>maximum</i>
of the two Real inputs <b>u1</b> and <b>u2</b>:
</p>
<pre>    y = <b>max</b> ( u1 , u2 );
</pre>
</html>
"));

      equation 
        y = max(u1, u2);
      end Max;
      block Min "Pass through the smallest signal"
        extends Interfaces.SI2SO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -90,36},{90, -36}}, lineColor = {160,160,164}, textString = "min()")}), Documentation(info = "<html>
<p>
This block computes the output <b>y</b> as <i>minimum</i> of
the two Real inputs <b>u1</b> and <b>u2</b>:
</p>
<pre>    y = <b>min</b> ( u1 , u2 );
</pre>
</html>
"));

      equation 
        y = min(u1, u2);
      end Min;
      block Edge "Indicates rising edge of boolean signal"
        extends Interfaces.BooleanSISO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -90,36},{90, -36}}, lineColor = {160,160,164}, textString = "edge()")}), Documentation(info = "<html>
<p>
This block sets the Boolean output <b>y</b> to true,
when the Boolean input <b>u</b> shows a <i>rising edge</i>:
</p>
<pre>    y = <b>edge</b>( u );
</pre>
</html>
"));

      equation 
        y = edge(u);
      end Edge;
      block BooleanChange "Indicates boolean signal changing"
        extends Interfaces.BooleanSISO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -90,36},{90, -36}}, lineColor = {160,160,164}, textString = "change()")}), Documentation(info = "<html>
<p>
This block sets the Boolean output <b>y</b> to true, when the
Boolean input <b>u</b> shows a <i>rising or falling edge</i>,
i.e., when the signal changes:
</p>
<pre>    y = <b>change</b>( u );
</pre>
</html>
"));

      equation 
        y = change(u);
      end BooleanChange;
      block IntegerChange "Indicates integer signal changing"
        extends Interfaces.IntegerSIBooleanSO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -90,36},{90, -36}}, lineColor = {160,160,164}, textString = "change()")}), Documentation(info = "<html>
<p>
This block sets the Boolean output <b>y</b> to true, when the
Integer input <b>u</b> changes:
</p>
<pre>    y = <b>change</b>( u );
</pre>
</html>
"));

      equation 
        y = change(u);
      end IntegerChange;
    end Math;
    package Logical "Library of components with Boolean input and output signals"
      extends Modelica.Icons.Library;
      annotation(Documentation(info = "<html>
<p>
This package provides blocks with Boolean input and output signals
to describe logical networks. A typical example for a logical
network built with package Logical is shown in the next figure:
</p>
<p align=\"center\">
<img src=\"../Images/Blocks/LogicalNetwork1.png\">
</p>

<p>
The actual value of Boolean input and/or output signals is displayed
in the respective block icon as \"circle\", where \"white\" color means
value <b>false</b> and \"green\" color means value <b>true</b>. These
values are visualized in a diagram animation.
</p>
</html>"));
      model And "Logical 'and': y = u1 and u2"
        extends Blocks.Interfaces.partialBooleanSI2SO;
        annotation(defaultComponentName = "and1", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -90,40},{90, -40}}, lineColor = {0,0,0}, textString = "and")}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
The output is <b>true</b> if all inputs are <b>true</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));

      equation 
        y = u1 and u2;
      end And;
      model Or "Logical 'or': y = u1 or u2"
        extends Blocks.Interfaces.partialBooleanSI2SO;
        annotation(defaultComponentName = "or1", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -90,40},{90, -40}}, lineColor = {0,0,0}, textString = "or")}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
The output is <b>true</b> if at least one input is <b>true</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));

      equation 
        y = u1 or u2;
      end Or;
      model Xor "Logical 'xor': y = u1 xor u2"
        extends Blocks.Interfaces.partialBooleanSI2SO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -90,40},{90, -40}}, lineColor = {0,0,0}, textString = "xor")}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
The output is <b>true</b> if exactly one input is <b>true</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));

      equation 
        y = not (u1 and u2 or not u1 and not u2);
      end Xor;
      model Nor "Logical 'nor': y = not (u1 or u2)"
        extends Blocks.Interfaces.partialBooleanSI2SO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -90,40},{90, -40}}, lineColor = {0,0,0}, textString = "nor")}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
The output is <b>true</b> if none of the inputs is <b>true</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));

      equation 
        y = not (u1 or u2);
      end Nor;
      model Nand "Logical 'nand': y = not (u1 and u2)"
        extends Blocks.Interfaces.partialBooleanSI2SO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -90,40},{90, -40}}, lineColor = {0,0,0}, textString = "nand")}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
The output is <b>true</b> if at least one input is <b>false</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));

      equation 
        y = not (u1 and u2);
      end Nand;
      model Not "Logical 'not': y = not u"
        extends Blocks.Interfaces.partialBooleanSISO;
        annotation(defaultComponentName = "not1", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -90,40},{90, -40}}, lineColor = {0,0,0}, textString = "not")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
The output is <b>true</b> if the input is <b>false</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));

      equation 
        y = not u;
      end Not;
      model Pre "Breaks algebraic loops by an infinitesimal small time delay (y = pre(u): event iteration continues until u = pre(u))"
        parameter Boolean pre_u_start = false "Start value of pre(u) at initial time";
        extends Blocks.Interfaces.partialBooleanSISO;
        annotation(defaultComponentName = "pre1", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -90,40},{90, -40}}, lineColor = {0,0,0}, textString = "pre")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
This block delays the Boolean input by an infinitesimal small time delay and
therefore breaks algebraic loops. In a network of logical blocks, in every
\"closed connection loop\" at least one logical block must have a delay,
since algebraic systems of Boolean equations are not solveable.
</p>

<p>
The \"Pre\" block returns the value of the \"input\" signal from the
last \"event iteration\". The \"event iteration\" stops, once both
values are identical (u = pre(u)).
</p>
</html>"));

      initial equation 
        pre(u) = pre_u_start;

      equation 
        y = pre(u);
      end Pre;
      model Edge "Output y is true, if the input u has a rising edge (y = edge(u))"
        parameter Boolean pre_u_start = false "Start value of pre(u) at initial time";
        extends Blocks.Interfaces.partialBooleanSISO;
        annotation(defaultComponentName = "edge1", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -90,40},{90, -40}}, lineColor = {0,0,0}, textString = "edge")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
The output is <b>true</b> if the Boolean input has a rising edge
from <b>false</b> to <b>true</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));

      initial equation 
        pre(u) = pre_u_start;

      equation 
        y = edge(u);
      end Edge;
      model FallingEdge "Output y is true, if the input u has a falling edge (y = edge(not u))"
        parameter Boolean pre_u_start = false "Start value of pre(u) at initial time";
        extends Blocks.Interfaces.partialBooleanSISO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -90,40},{90, -40}}, lineColor = {0,0,0}, textString = "falling")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
The output is <b>true</b> if the Boolean input has a falling edge
from <b>true</b> to <b>false</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));
      protected 
        Boolean not_u = not u;

      initial equation 
        pre(not_u) = not pre_u_start;

      equation 
        y = edge(not_u);
      end FallingEdge;
      model Change "Output y is true, if the input u has a rising or falling edge (y = change(u))"
        parameter Boolean pre_u_start = false "Start value of pre(u) at initial time";
        extends Blocks.Interfaces.partialBooleanSISO;
        annotation(defaultComponentName = "change1", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -90,40},{90, -40}}, lineColor = {0,0,0}, textString = "change")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
The output is <b>true</b> if the Boolean input has either a rising edge
from <b>false</b> to <b>true</b> or a falling edge from
<b>true</b> to <b>false</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));

      initial equation 
        pre(u) = pre_u_start;

      equation 
        y = change(u);
      end Change;
      block GreaterThreshold "Output y is true, if input u is greater than threshold"
        extends Blocks.Interfaces.partialBooleanThresholdComparison;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -90, -40},{60,40}}, lineColor = {0,0,0}, textString = ">")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
The output is <b>true</b> if the Real input is greater than
parameter <b>threshold</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));

      equation 
        y = u > threshold;
      end GreaterThreshold;
      block GreaterEqualThreshold "Output y is true, if input u is greater or equal than threshold"
        extends Blocks.Interfaces.partialBooleanThresholdComparison;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -90, -40},{60,40}}, lineColor = {0,0,0}, textString = ">=")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
The output is <b>true</b> if the Real input is greater than or equal to
parameter <b>threshold</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));

      equation 
        y = u >= threshold;
      end GreaterEqualThreshold;
      block LessThreshold "Output y is true, if input u is less than threshold"
        extends Blocks.Interfaces.partialBooleanThresholdComparison;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -90, -40},{60,40}}, lineColor = {0,0,0}, textString = "<")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
The output is <b>true</b> if the Real input is less than
parameter <b>threshold</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));

      equation 
        y = u < threshold;
      end LessThreshold;
      block LessEqualThreshold "Output y is true, if input u is less or equal than threshold"
        extends Blocks.Interfaces.partialBooleanThresholdComparison;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -90, -40},{60,40}}, lineColor = {0,0,0}, textString = "<=")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
The output is <b>true</b> if the Real input is less than or equal to
parameter <b>threshold</b>, otherwise
the output is <b>false</b>.
</p>
</html>"));

      equation 
        y = u <= threshold;
      end LessEqualThreshold;
      block Greater "Output y is true, if input u1 is greater as input u2"
        extends Blocks.Interfaces.partialBooleanComparison;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{32,10},{52, -10}}, lineColor = {0,0,255}),Line(points = {{ -100, -80},{42, -80},{42,0}}, color = {0,0,255}),Text(extent = {{ -80, -60},{20,60}}, lineColor = {0,0,0}, textString = ">")}), Documentation(info = "<html>
<p>
The output is <b>true</b> if Real input u1 is greater than
Real input u2, otherwise the output is <b>false</b>.
</p>
</html>"));

      equation 
        y = u1 > u2;
      end Greater;
      block GreaterEqual "Output y is true, if input u1 is greater or equal as input u2"
        extends Blocks.Interfaces.partialBooleanComparison;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{32,10},{52, -10}}, lineColor = {0,0,255}),Line(points = {{ -100, -80},{42, -80},{42,0}}, color = {0,0,255}),Text(extent = {{ -80, -60},{20,60}}, lineColor = {0,0,0}, textString = ">=")}), Documentation(info = "<html>
<p>
The output is <b>true</b> if Real input u1 is greater than or equal to
Real input u2, otherwise the output is <b>false</b>.
</p>
</html>"));

      equation 
        y = u1 >= u2;
      end GreaterEqual;
      block Less "Output y is true, if input u1 is less as input u2"
        extends Blocks.Interfaces.partialBooleanComparison;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{32,10},{52, -10}}, lineColor = {0,0,255}),Line(points = {{ -100, -80},{42, -80},{42,0}}, color = {0,0,255}),Text(extent = {{ -80, -60},{20,60}}, lineColor = {0,0,0}, textString = "<")}), Documentation(info = "<html>
<p>
The output is <b>true</b> if Real input u1 is less than
Real input u2, otherwise the output is <b>false</b>.
</p>
</html>"));

      equation 
        y = u1 < u2;
      end Less;
      block LessEqual "Output y is true, if input u1 is less or equal as input u2"
        extends Blocks.Interfaces.partialBooleanComparison;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{32,10},{52, -10}}, lineColor = {0,0,255}),Line(points = {{ -100, -80},{42, -80},{42,0}}, color = {0,0,255}),Text(extent = {{ -80, -60},{20,60}}, lineColor = {0,0,0}, textString = "<=")}), Documentation(info = "<html>
<p>
The output is <b>true</b> if Real input u1 is less than or equal to
Real input u2, otherwise the output is <b>false</b>.
</p>
</html>"));

      equation 
        y = u1 <= u2;
      end LessEqual;
      block ZeroCrossing "Trigger zero crossing of input u"
        extends Blocks.Interfaces.partialBooleanSO;
        Blocks.Interfaces.RealInput u annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Blocks.Interfaces.BooleanInput enable "Zero input crossing is triggered if the enable input signal is true" annotation(Placement(transformation(origin = {0, -120}, extent = {{ -20, -20},{20,20}}, rotation = 90)));
        annotation(Documentation(info = "<HTML>
<p>
The output \"y\" is <b>true</b> at the
time instant when the input \"u\" becomes
zero, provided the input \"enable\" is
<b>true</b>. At all other time instants, the output \"y\" is <b>false</b>.
If the input \"u\" is zero at a time instant when the \"enable\"
input changes its value, then the output y is <b>false</b>.
</p>
<p>
Note, that in the plot window of a Modelica simulator, the output of
this block is usually identically to <b>false</b>, because the output
may only be <b>true</b> at an event instant, but not during
continuous integration. In order to check that this component is
actually working as expected, one should connect its output to, e.g.,
component <i>ModelicaAdditions.Blocks.Discrete.TriggeredSampler</i>.
</p>
</HTML>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -78,68},{ -78, -80}}, color = {192,192,192}),Polygon(points = {{ -78,90},{ -86,68},{ -70,68},{ -78,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -88,0},{70,0}}, color = {192,192,192}),Line(points = {{ -78,0},{ -73.2,32.3},{ -70,50.3},{ -66.7,64.5},{ -63.5,74.2},{ -60.3,79.3},{ -57.1,79.6},{ -53.9,75.3},{ -50.7,67.1},{ -46.6,52.2},{ -41,25.8},{ -33, -13.9},{ -28.2, -33.7},{ -24.1, -45.9},{ -20.1, -53.2},{ -16.1, -55.3},{ -12.1, -52.5},{ -8.1, -45.3},{ -3.23, -32.1},{10.44,13.7},{15.3,26.4},{20.1,34.8},{24.1,38},{28.9,37.2},{33.8,31.8},{40.2,19.4},{53.1, -10.5},{59.5, -21.2},{65.1, -25.9},{70.7, -25.9},{77.2, -20.5},{82, -13.8}}, color = {192,192,192}),Polygon(points = {{92,0},{70,8},{70, -8},{92,0}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -36, -59},{ -36,81}}, color = {255,0,255}),Line(points = {{6, -59},{6,81}}, color = {255,0,255}),Line(points = {{49, -59},{49,81}}, color = {255,0,255}),Line(points = {{ -78,0},{70,0}}, color = {255,0,255})}));
      protected 
        Boolean disable = not enable;
        Boolean u_pos;

      initial equation 
        pre(u_pos) = false;
        pre(enable) = false;
        pre(disable) = not pre(enable);

      equation 
        u_pos = enable and u >= 0;
        y = change(u_pos) and not edge(enable) and not edge(disable);
      end ZeroCrossing;
      block LogicalSwitch "Logical Switch"
        extends Blocks.Interfaces.partialBooleanSI3SO;
        annotation(Documentation(info = "<html>
<p>The LogicalSwitch switches, depending on the
Boolean u2 connector (the middle connector),
between the two possible input signals
u1 (upper connector) and u3 (lower connector).</p>
<p>If u2 is true, connector y is set equal to
u1, else it is set equal to u3.</p>
</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{12,0},{100,0}}, color = {255,0,255}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Line(points = {{ -100,0},{ -40,0}}, color = {255,0,255}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Line(points = {{ -100, -80},{ -40, -80},{ -40, -80}}, color = {255,0,255}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Line(points = {{ -40,12},{ -40, -10}}, color = {255,0,255}),Line(points = {{ -100,80},{ -40,80}}, color = {255,0,255}),Line(points = {{ -40,80},{8,2}}, color = {255,0,127}, thickness = 1),Ellipse(extent = {{2,8},{18, -6}}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid, lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));

      equation 
        y = if u2 then u1 else u3;
      end LogicalSwitch;
      block Switch "Switch between two Real signals"
        extends Blocks.Interfaces.partialBooleanBlockIcon;
        Blocks.Interfaces.RealInput u1 "Connector of first Real input signal" annotation(Placement(transformation(extent = {{ -140,60},{ -100,100}}, rotation = 0)));
        Blocks.Interfaces.BooleanInput u2 "Connector of Boolean input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Blocks.Interfaces.RealInput u3 "Connector of second Real input signal" annotation(Placement(transformation(extent = {{ -140, -100},{ -100, -60}}, rotation = 0)));
        Blocks.Interfaces.RealOutput y "Connector of Real output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(defaultComponentName = "switch1", Documentation(info = "<html>
<p>The Logical.Switch switches, depending on the
logical connector u2 (the middle connector)
between the two possible input signals
u1 (upper connector) and u3 (lower connector).</p>
<p>If u2 is <b>true</b>, the output signal y is set equal to
u1, else it is set equal to u3.</p>
</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{12,0},{100,0}}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}, color = {0,0,255}),Line(points = {{ -100,0},{ -40,0}}, color = {255,0,127}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}),Line(points = {{ -100, -80},{ -40, -80},{ -40, -80}}, pattern = LinePattern.Solid, thickness = 0.25, arrow = {Arrow.None,Arrow.None}, color = {0,0,255}),Line(points = {{ -40,12},{ -40, -12}}, color = {255,0,127}),Line(points = {{ -100,80},{ -38,80}}, color = {0,0,255}),Line(points = {{ -38,80},{6,2}}, thickness = 1, color = {0,0,255}),Ellipse(extent = {{2,8},{18, -6}}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid, lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));

      equation 
        y = if u2 then u1 else u3;
      end Switch;
      block Hysteresis "Transform Real to Boolean signal with Hysteresis"
        extends Blocks.Interfaces.partialBooleanBlockIcon;
        parameter Real uLow(start = 0) "if y=true and u<=uLow, switch to y=false";
        parameter Real uHigh(start = 1) "if y=false and u>=uHigh, switch to y=true";
        parameter Boolean pre_y_start = false "Value of pre(y) at initial time";
        Blocks.Interfaces.RealInput u annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Blocks.Interfaces.BooleanOutput y annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -65,89},{ -73,67},{ -57,67},{ -65,89}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -65,67},{ -65, -81}}, color = {192,192,192}),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{70, -80},{94, -100}}, lineColor = {160,160,164}, textString = "u"),Text(extent = {{ -65,93},{ -12,75}}, lineColor = {160,160,164}, textString = "y"),Line(points = {{ -80, -70},{30, -70}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -50,10},{80,10}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -50,10},{ -50, -70}}, color = {0,0,0}, thickness = 0.5),Line(points = {{30,10},{30, -70}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -10, -65},{0, -70},{ -10, -75}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -10,15},{ -20,10},{ -10,5}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -55, -20},{ -50, -30},{ -44, -20}}, color = {0,0,0}, thickness = 0.5),Line(points = {{25, -30},{30, -19},{35, -30}}, color = {0,0,0}, thickness = 0.5),Text(extent = {{ -99,2},{ -70,18}}, lineColor = {160,160,164}, textString = "true"),Text(extent = {{ -98, -87},{ -66, -73}}, lineColor = {160,160,164}, textString = "false"),Text(extent = {{19, -87},{44, -70}}, lineColor = {0,0,0}, textString = "uHigh"),Text(extent = {{ -63, -88},{ -38, -71}}, lineColor = {0,0,0}, textString = "uLow"),Line(points = {{ -69,10},{ -60,10}}, color = {160,160,164})}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80,68},{ -80, -29}}, color = {192,192,192}),Polygon(points = {{92, -29},{70, -21},{70, -37},{92, -29}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -79, -29},{84, -29}}, color = {192,192,192}),Line(points = {{ -79, -29},{41, -29}}, color = {0,0,0}),Line(points = {{ -15, -21},{1, -29},{ -15, -36}}, color = {0,0,0}),Line(points = {{41,51},{41, -29}}, color = {0,0,0}),Line(points = {{33,3},{41,22},{50,3}}, color = {0,0,0}),Line(points = {{ -49,51},{81,51}}, color = {0,0,0}),Line(points = {{ -4,59},{ -19,51},{ -4,43}}, color = {0,0,0}),Line(points = {{ -59,29},{ -49,11},{ -39,29}}, color = {0,0,0}),Line(points = {{ -49,51},{ -49, -29}}, color = {0,0,0}),Text(extent = {{ -92, -49},{ -9, -92}}, lineColor = {192,192,192}, textString = "%uLow"),Text(extent = {{2, -49},{91, -92}}, lineColor = {192,192,192}, textString = "%uHigh"),Rectangle(extent = {{ -91, -49},{ -8, -92}}, lineColor = {192,192,192}),Line(points = {{ -49, -29},{ -49, -49}}, color = {192,192,192}),Rectangle(extent = {{2, -49},{91, -92}}, lineColor = {192,192,192}),Line(points = {{41, -29},{41, -49}}, color = {192,192,192})}), Documentation(info = "<HTML>
<p>
This block transforms a <b>Real</b> input signal into a <b>Boolean</b>
output signal:
</p>
<ul>
<li> When the output was <b>false</b> and the input becomes
     <b>greater</b> than parameter <b>uHigh</b>, the output
     switches to <b>true</b>.</li>
<li> When the output was <b>true</b> and the input becomes
     <b>less</b> than parameter <b>uLow</b>, the output
     switches to <b>false</b>.</li>
</ul>
<p>
The start value of the output is defined via parameter
<b>pre_y_start</b> (= value of pre(y) at initial time).
The default value of this parameter is <b>false</b>.
</p>
</HTML>
"));

      initial equation 
        pre(y) = pre_y_start;

      equation 
        y = u > uHigh or pre(y) and u >= uLow;
      end Hysteresis;
      block OnOffController "On-off controller"
        extends Interfaces.partialBooleanBlockIcon;
        Blocks.Interfaces.RealInput reference "Connector of Real input signal used as reference signal" annotation(Placement(transformation(extent = {{ -140,80},{ -100,40}}, rotation = 0)));
        Blocks.Interfaces.RealInput u "Connector of Real input signal used as measurement signal" annotation(Placement(transformation(extent = {{ -140, -40},{ -100, -80}}, rotation = 0)));
        Blocks.Interfaces.BooleanOutput y "Connector of Real output signal used as actuator signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        parameter Real bandwidth(start = 0.1) "Bandwidth around reference signal";
        parameter Boolean pre_y_start = false "Value of pre(y) at initial time";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -92,74},{44,44}}, lineColor = {0,0,0}, lineThickness = 0.5, textString = "reference"),Text(extent = {{ -94, -52},{ -34, -74}}, lineColor = {0,0,0}, textString = "u"),Line(points = {{ -76, -32},{ -68, -6},{ -50,26},{ -24,40},{ -2,42},{16,36},{32,28},{48,12},{58, -6},{68, -28}}, color = {0,0,255}),Line(points = {{ -78, -2},{ -6,18},{82, -12}}, color = {255,0,0}),Line(points = {{ -78,12},{ -6,30},{82,0}}, color = {0,0,0}),Line(points = {{ -78, -16},{ -6,4},{82, -26}}, color = {0,0,0}),Line(points = {{ -82, -18},{ -56, -18},{ -56, -40},{64, -40},{64, -20},{90, -20}}, color = {255,0,255})}), Documentation(info = "<html>
<p>The block OnOffController sets the output signal <b>y</b> to <b>true</b> when
the input signal <b>u</b> falls below the <b>reference</b> signal minus half of
the bandwidth and sets the output signal <b>y</b> to <b>false</b> when the input
signal <b>u</b> exceeds the <b>reference</b> signal plus half of the bandwidth.</p>
</html>
"));

      initial equation 
        pre(y) = pre_y_start;

      equation 
        y = pre(y) and u < reference + bandwidth / 2 or u < reference - bandwidth / 2;
      end OnOffController;
      block TriggeredTrapezoid "Triggered trapezoid generator"
        extends Interfaces.partialBooleanBlockIcon;
        parameter Real amplitude = 1 "Amplitude of trapezoid";
        parameter Modelica.SIunits.Time rising(final min = 0) = 0 "Rising duration of trapezoid";
        parameter Modelica.SIunits.Time falling(final min = 0) = rising "Falling duration of trapezoid";
        parameter Real offset = 0 "Offset of output signal";
        Blocks.Interfaces.BooleanInput u "Connector of Boolean input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Blocks.Interfaces.RealOutput y "Connector of Real output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -60, -70},{ -60, -70},{ -30,40},{8,40},{40, -70},{40, -70}}, color = {0,0,255}),Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -70},{ -60, -70},{ -60,24},{8,24},{8, -70},{60, -70}}, color = {255,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -80, -20},{ -60, -20},{ -30,40},{8,40},{40, -20},{60, -20}}, color = {0,0,255}),Line(points = {{ -90, -70},{82, -70}}, color = {0,0,0}),Line(points = {{ -80,68},{ -80, -80}}, color = {0,0,0}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -68},{ -60, -68},{ -60, -42},{8, -42},{8, -68},{60, -68}}, color = {255,0,255}),Line(points = {{ -60,40},{ -60, -42}}, color = {0,0,0}, pattern = LinePattern.Dot),Line(points = {{8, -42},{8,40}}, color = {0,0,0}, pattern = LinePattern.Dot),Line(points = {{ -60,40},{ -30,40}}, color = {0,0,0}),Line(points = {{8, -20},{40, -20}}, color = {0,0,0}),Line(points = {{ -20,40},{ -20, -20}}, color = {0,0,0}),Line(points = {{ -20, -20},{ -20, -70}}, color = {0,0,0}),Text(extent = {{ -44,50},{ -44,40}}, lineColor = {0,0,0}, textString = "rising"),Text(extent = {{24, -10},{24, -20}}, lineColor = {0,0,0}, textString = "falling"),Polygon(points = {{ -60,40},{ -56,42},{ -56,38},{ -60,40}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -30,40},{ -34,42},{ -34,38},{ -30,40}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{8, -20},{12, -18},{12, -22},{8, -20}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{40, -20},{36, -18},{36, -22},{40, -20}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -22, -24},{ -20, -20},{ -18, -24},{ -22, -24}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -18, -66},{ -22, -66},{ -20, -70},{ -18, -66}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -22,36},{ -20,40},{ -18,36},{ -22,36}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -18, -16},{ -22, -16},{ -20, -20},{ -18, -16}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -40,6},{0, -4}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -20,6},{ -20, -4}}, lineColor = {0,0,0}, textString = "amplitude"),Rectangle(extent = {{ -40, -48},{0, -58}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -20, -48},{ -20, -58}}, lineColor = {0,0,0}, textString = "offset"),Text(extent = {{60, -82},{94, -92}}, lineColor = {0,0,0}, textString = "time"),Text(extent = {{ -88, -4},{ -54, -14}}, lineColor = {0,0,0}, textString = "y"),Text(extent = {{ -88, -46},{ -54, -56}}, lineColor = {0,0,0}, textString = "u")}), Documentation(info = "<HTML>
<p>The block TriggeredTrapezoid has a boolean input and a real
output signal and requires the parameters <i>amplitude</i>,
<i>rising</i>, <i>falling</i> and <i>offset</i>. The
output signal <b>y</b> represents a trapezoidal signal dependent on the
input signal <b>u</b>.
</p>
<p>The behaviour is as follows: Assume the initial input to be false. In this
case, the output will be <i>offset</i>. After a rising edge (i.e. the input
changes from false to true), the output is rising during <i>rising</i> to the
sum of <i>offset</i> and <i>amplitude</i>. In contrast, after a falling
edge (i.e. the input changes from true to false), the output is falling
during <i>falling</i> to a value of <i>offset</i>.
</p>
<p>Note, that the case of edges before expiration of rising or falling is
handled properly.</p>
</HTML>
"));
      protected 
        discrete Real endValue "Value of y at time of recent edge";
        discrete Real rate "Current rising/falling rate";
        discrete Modelica.SIunits.Time T "Predicted time of output reaching endValue";

      initial equation 
        pre(y) = 0;

      equation 
        y = if time < T then endValue - (T - time) * rate else endValue;
        when {initial(),u,not u} then
                  endValue = if u then offset + amplitude else offset;
          rate = if u and rising > 0 then amplitude / rising else if not u and falling > 0 then  -amplitude / falling else 0;
          T = if u and not rising > 0 or not u and not falling > 0 or not abs(amplitude) > 0 or initial() then time else time + (endValue - pre(y)) / rate;
        
        end when;
      end TriggeredTrapezoid;
      block Timer "Timer measuring the time from the time instant where the Boolean input became true"
        extends Interfaces.partialBooleanBlockIcon;
        Blocks.Interfaces.BooleanInput u "Connector of Boolean input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Blocks.Interfaces.RealOutput y "Connector of Real output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -90, -70},{82, -70}}, color = {192,192,192}),Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -70},{ -60, -70},{ -60, -26},{38, -26},{38, -70},{66, -70}}, color = {255,0,255}),Line(points = {{ -80,0},{ -62,0},{40,90},{40,0},{68,0}}, color = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -90, -70},{82, -70}}, color = {0,0,0}),Line(points = {{ -80,68},{ -80, -80}}, color = {0,0,0}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -68},{ -60, -68},{ -60, -40},{20, -40},{20, -68},{60, -68}}, color = {255,0,255}),Line(points = {{ -80, -20},{ -60, -20},{20,60},{20, -20},{60, -20},{60, -20}}, color = {0,0,255}),Text(extent = {{ -88,6},{ -54, -4}}, lineColor = {0,0,0}, textString = "y"),Text(extent = {{48, -80},{84, -88}}, lineColor = {0,0,0}, textString = "time"),Text(extent = {{ -88, -36},{ -54, -46}}, lineColor = {0,0,0}, textString = "u")}), Documentation(info = "<HTML>
<p> When the Boolean input \"u\" becomes <b>true</b>, the timer is started
and the output \"y\" is the time from the time instant where u became true.
The timer is stopped and the output is reset to zero, once the
input becomes false.
</p>
</HTML>
"));
      protected 
        discrete Modelica.SIunits.Time entryTime "Time instant when u became true";

      initial equation 
        pre(entryTime) = 0;

      equation 
        when u then
                  entryTime = time;
        
        end when;
        y = if u then time - entryTime else 0.0;
      end Timer;
      block TerminateSimulation "Terminate simulation if condition is fullfilled"
        Modelica.Blocks.Interfaces.BooleanOutput condition = false "Terminate simulation when condition becomes true" annotation(Dialog, Placement(transformation(extent = {{200, -10},{220,10}}, rotation = 0)));
        parameter String terminationText = "... End condition reached" "Text that will be displayed when simulation is terminated";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -200, -20},{200,20}}, grid = {2,2}, initialScale = 0.2), graphics = {Rectangle(extent = {{ -200,20},{200, -20}}, lineColor = {0,0,0}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised),Text(extent = {{ -166,15},{194, -15}}, lineColor = {0,0,0}, textString = "%condition"),Rectangle(extent = {{ -194,14},{ -168, -14}}, lineColor = {0,0,0}, fillColor = {255,0,0}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised),Text(extent = {{ -200,46},{200,22}}, lineColor = {0,0,255}, textString = "%name")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -200, -20},{200,20}}, grid = {2,2}, initialScale = 0.2), graphics), Documentation(info = "<html>
<p>
In the parameter menu, a <b>time varying</b> expression can be defined
via variable <b>condition</b>, for example \"condition = x &lt; 0\",
where \"x\" is a variable that is declared in the model in which the
\"TerminateSimulation\" block is present.
If this expression becomes <b>true</b>,
the simulation is (successfully) terminated. A termination message
explaining the reason for the termination can be given via
parameter \"terminationText\".
</p>

</html>"));

      equation 
        when condition then
                  terminate(terminationText);
        
        end when;
      end TerminateSimulation;
    end Logical;
    package Interfaces "Library of connectors and partial models for input/output blocks"
      import Modelica.SIunits;
      extends Modelica.Icons.Library;
      annotation(Documentation(info = "<HTML>
<p>
This package contains interface definitions for
<b>continuous</b> input/output blocks with Real,
Integer and Boolean signals. Furthermore, it contains
partial models for continuous and discrete blocks.
</p>

</HTML>
", revisions = "<html>
<ul>
<li><i>Oct. 21, 2002</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>
       and <a href=\"http://www.robotic.dlr.de/Christian.Schweiger/\">Christian Schweiger</a>:<br>
       Added several new interfaces. <a href=\"../Documentation/ChangeNotes1.5.html\">Detailed description</a> available.
<li><i>Oct. 24, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       RealInputSignal renamed to RealInput. RealOutputSignal renamed to
       output RealOutput. GraphBlock renamed to BlockIcon. SISOreal renamed to
       SISO. SOreal renamed to SO. I2SOreal renamed to M2SO.
       SignalGenerator renamed to SignalSource. Introduced the following
       new models: MIMO, MIMOs, SVcontrol, MVcontrol, DiscreteBlockIcon,
       DiscreteBlock, DiscreteSISO, DiscreteMIMO, DiscreteMIMOs,
       BooleanBlockIcon, BooleanSISO, BooleanSignalSource, MI2BooleanMOs.</li>
<li><i>June 30, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized a first version, based on an existing Dymola library
       of Dieter Moormann and Hilding Elmqvist.</li>
</ul>
</html>
"));
      connector RealInput = input Real "'input Real' as connector" annotation(defaultComponentName = "u", Icon(graphics = {Polygon(points = {{ -100,100},{100,0},{ -100, -100},{ -100,100}}, lineColor = {0,0,127}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid)}, coordinateSystem(extent = {{ -100, -100},{100,100}}, preserveAspectRatio = true, initialScale = 0.2)), Diagram(coordinateSystem(preserveAspectRatio = true, initialScale = 0.2, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{0,50},{100,0},{0, -50},{0,50}}, lineColor = {0,0,127}, fillColor = {0,0,127}, fillPattern = FillPattern.Solid),Text(extent = {{ -10,85},{ -10,60}}, lineColor = {0,0,127}, textString = "%name")}), Documentation(info = "<html>
<p>
Connector with one input signal of type Real.
</p>
</html>"));
      connector RealOutput = output Real "'output Real' as connector" annotation(defaultComponentName = "y", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -100,100},{100,0},{ -100, -100},{ -100,100}}, lineColor = {0,0,127}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -100,50},{0,0},{ -100, -50},{ -100,50}}, lineColor = {0,0,127}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{30,110},{30,60}}, lineColor = {0,0,127}, textString = "%name")}), Documentation(info = "<html>
<p>
Connector with one output signal of type Real.
</p>
</html>"));
      connector BooleanInput = input Boolean "'input Boolean' as connector" annotation(defaultComponentName = "u", Icon(graphics = {Polygon(points = {{ -100,100},{100,0},{ -100, -100},{ -100,100}}, lineColor = {255,0,255}, fillColor = {255,0,255}, fillPattern = FillPattern.Solid)}, coordinateSystem(extent = {{ -100, -100},{100,100}}, preserveAspectRatio = true, initialScale = 0.2)), Diagram(coordinateSystem(preserveAspectRatio = true, initialScale = 0.2, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{0,50},{100,0},{0, -50},{0,50}}, lineColor = {255,0,255}, fillColor = {255,0,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -10,85},{ -10,60}}, lineColor = {255,0,255}, textString = "%name")}), Documentation(info = "<html>
<p>
Connector with one input signal of type Boolean.
</p>
</html>"));
      connector BooleanOutput = output Boolean "'output Boolean' as connector" annotation(defaultComponentName = "y", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -100,100},{100,0},{ -100, -100},{ -100,100}}, lineColor = {255,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -100,50},{0,0},{ -100, -50},{ -100,50}}, lineColor = {255,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{30,110},{30,60}}, lineColor = {255,0,255}, textString = "%name")}), Documentation(info = "<html>
<p>
Connector with one output signal of type Boolean.
</p>
</html>"));
      connector IntegerInput = input Integer "'input Integer' as connector" annotation(defaultComponentName = "u", Icon(graphics = {Polygon(points = {{ -100,100},{100,0},{ -100, -100},{ -100,100}}, lineColor = {255,127,0}, fillColor = {255,127,0}, fillPattern = FillPattern.Solid)}, coordinateSystem(extent = {{ -100, -100},{100,100}}, preserveAspectRatio = true, initialScale = 0.2)), Diagram(coordinateSystem(preserveAspectRatio = true, initialScale = 0.2, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{0,50},{100,0},{0, -50},{0,50}}, lineColor = {255,127,0}, fillColor = {255,127,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -10,85},{ -10,60}}, lineColor = {255,127,0}, textString = "%name")}), Documentation(info = "<html>
<p>
Connector with one input signal of type Integer.
</p>
</html>"));
      connector IntegerOutput = output Integer "'output Integer' as connector" annotation(defaultComponentName = "y", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -100,100},{100,0},{ -100, -100},{ -100,100}}, lineColor = {255,127,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -100,50},{0,0},{ -100, -50},{ -100,50}}, lineColor = {255,127,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{30,110},{30,60}}, lineColor = {255,127,0}, textString = "%name")}), Documentation(info = "<html>
<p>
Connector with one output signal of type Integer.
</p>
</html>"));
      partial block BlockIcon "Basic graphical layout of input/output block"
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -100},{100,100}}, lineColor = {0,0,127}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,150},{150,110}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<html>
<p>
Block that has only the basic icon for an input/output
block (no declarations, no equations). Most blocks
of package Modelica.Blocks inherit directly or indirectly
from this block.
</p>
</html>"));
      end BlockIcon;
      partial block SO "Single Output continuous control block"
        extends BlockIcon;
        RealOutput y "Connector of Real output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
Block has one continuous Real output signal.
</p>
</html>"));
      end SO;
      partial block MO "Multiple Output continuous control block"
        extends BlockIcon;
        parameter Integer nout(min = 1) = 1 "Number of outputs";
        RealOutput y[nout] "Connector of Real output signals" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<html>
<p>
Block has one continuous Real output signal vector.
</p>
</html>"));
      end MO;
      partial block SISO "Single Input Single Output continuous control block"
        extends BlockIcon;
        RealInput u "Connector of Real input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        RealOutput y "Connector of Real output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<html>
<p>
Block has one continuous Real input and one continuous Real output signal.
</p>
</html>"));
      end SISO;
      partial block SI2SO "2 Single Input / 1 Single Output continuous control block"
        extends BlockIcon;
        RealInput u1 "Connector of Real input signal 1" annotation(Placement(transformation(extent = {{ -140,40},{ -100,80}}, rotation = 0)));
        RealInput u2 "Connector of Real input signal 2" annotation(Placement(transformation(extent = {{ -140, -80},{ -100, -40}}, rotation = 0)));
        RealOutput y "Connector of Real output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<html>
<p>
Block has two continuous Real input signals u1 and u2 and one
continuous Real output signal y.
</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));
      end SI2SO;
      partial block SIMO "Single Input Multiple Output continuous control block"
        extends BlockIcon;
        parameter Integer nout = 1 "Number of outputs";
        RealInput u "Connector of Real input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        RealOutput y[nout] "Connector of Real output signals" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<HTML>
<p> Block has one continuous Real input signal and a
    vector of continuous Real output signals.</p>

</HTML>
"));
      end SIMO;
      partial block MISO "Multiple Input Single Output continuous control block"
        extends BlockIcon;
        parameter Integer nin = 1 "Number of inputs";
        RealInput u[nin] "Connector of Real input signals" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        RealOutput y "Connector of Real output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<HTML>
<p>
Block has a vector of continuous Real input signals and
one continuous Real output signal.
</p>
</HTML>
"));
      end MISO;
      partial block MIMO "Multiple Input Multiple Output continuous control block"
        extends BlockIcon;
        parameter Integer nin = 1 "Number of inputs";
        parameter Integer nout = 1 "Number of outputs";
        RealInput u[nin] "Connector of Real input signals" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        RealOutput y[nout] "Connector of Real output signals" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<HTML>
<p>
Block has a continuous Real input and a continuous Real output signal vector.
The signal sizes of the input and output vector may be different.
</p>
</HTML>
"));
      end MIMO;
      partial block MIMOs "Multiple Input Multiple Output continuous control block with same number of inputs and outputs"
        extends BlockIcon;
        parameter Integer n = 1 "Number of inputs (= number of outputs)";
        RealInput u[n] "Connector of Real input signals" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        RealOutput y[n] "Connector of Real output signals" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<HTML>
<p>
Block has a continuous Real input and a continuous Real output signal vector
where the signal sizes of the input and output vector are identical.
</p>
</HTML>
"));
      end MIMOs;
      partial block MI2MO "2 Multiple Input / Multiple Output continuous control block"
        extends BlockIcon;
        parameter Integer n = 1 "Dimension of input and output vectors.";
        RealInput u1[n] "Connector 1 of Real input signals" annotation(Placement(transformation(extent = {{ -140,40},{ -100,80}}, rotation = 0)));
        RealInput u2[n] "Connector 2 of Real input signals" annotation(Placement(transformation(extent = {{ -140, -80},{ -100, -40}}, rotation = 0)));
        RealOutput y[n] "Connector of Real output signals" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<html>
<p>
Block has two continuous Real input vectors u1 and u2 and one
continuous Real output vector y.
All vectors have the same number of elements.
</p>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));
      end MI2MO;
      partial block SignalSource "Base class for continuous signal source"
        extends SO;
        parameter Real offset = 0 "Offset of output signal y";
        parameter SIunits.Time startTime = 0 "Output y = offset for time < startTime";
        annotation(Documentation(info = "<html>
<p>
Basic block for Real sources of package Blocks.Sources.
This component has one continuous Real output signal y
and two parameters (offset, startTime) to shift the
generated signal.
</p>
</html>"));
      end SignalSource;
      partial block SVcontrol "Single-Variable continuous controller"
        extends BlockIcon;
        RealInput u_s "Connector of setpoint input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        RealInput u_m "Connector of measurement input signal" annotation(Placement(transformation(origin = {0, -120}, extent = {{20, -20},{ -20,20}}, rotation = 270)));
        RealOutput y "Connector of actuator output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -102,34},{ -142,24}}, textString = "(setpoint)", lineColor = {0,0,255}),Text(extent = {{100,24},{140,14}}, textString = "(actuator)", lineColor = {0,0,255}),Text(extent = {{ -83, -112},{ -33, -102}}, textString = " (measurement)", lineColor = {0,0,255})}), Documentation(info = "<html>
<p>
Block has two continuous Real input signals and one
continuous Real output signal. The block is designed
to be used as base class for a corresponding controller.
</p>
</html>"));
      end SVcontrol;
      partial block MVcontrol "Multi-Variable continuous controller"
        extends BlockIcon;
        parameter Integer nu_s = 1 "Number of setpoint inputs";
        parameter Integer nu_m = 1 "Number of measurement inputs";
        parameter Integer ny = 1 "Number of actuator outputs";
        RealInput u_s[nu_s] "Connector of setpoint input signals" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        RealInput u_m[nu_m] "Connector of measurement input signals" annotation(Placement(transformation(origin = {0, -120}, extent = {{20, -20},{ -20,20}}, rotation = 270)));
        RealOutput y[ny] "Connector of actuator output signals" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -100,36},{ -140,26}}, textString = "(setpoint)", lineColor = {0,0,255}),Text(extent = {{102,24},{142,14}}, textString = "(actuator)", lineColor = {0,0,255}),Text(extent = {{ -75, -108},{ -25, -98}}, textString = " (measurement)", lineColor = {0,0,255})}), Documentation(info = "<html>
<p>
Block has two continuous Real input signal vectors and one
continuous Real output signal vector. The block is designed
to be used as base class for a corresponding controller.
</p>
</html>"));
      end MVcontrol;
      partial block DiscreteBlockIcon "Graphical layout of discrete block component icon"
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -100},{100,100}}, lineColor = {0,0,127}, fillColor = {223,223,159}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,150},{150,110}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<html>
<p>
Block that has only the basic icon for an input/output,
discrete block (no declarations, no equations), e.g.,
from Blocks.Discrete.
</p>
</html>"));
      end DiscreteBlockIcon;
      partial block DiscreteBlock "Base class of discrete control blocks"
        extends DiscreteBlockIcon;
        parameter SI.Time samplePeriod(min = 100 * Modelica.Constants.eps, start = 0.1) "Sample period of component";
        parameter SI.Time startTime = 0 "First sample time instant";
      protected 
        output Boolean sampleTrigger "True, if sample time instant";
        output Boolean firstTrigger "Rising edge signals first sample instant";

      equation 
        sampleTrigger = sample(startTime, samplePeriod);
        when sampleTrigger then
                  firstTrigger = time <= startTime + samplePeriod / 2;
        
        end when;
        annotation(Documentation(info = "<html>
<p>
Basic definitions of a discrete block of library
Blocks.Discrete.
</p>
</html>"));
      end DiscreteBlock;
      partial block DiscreteSISO "Single Input Single Output discrete control block"
        extends DiscreteBlock;
        Modelica.Blocks.Interfaces.RealInput u "Continuous input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y "Continuous output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<html>
<p>
Block has one continuous input and one continuous output signal
which are sampled due to the defined <b>samplePeriod</b> parameter.
</p>
</html>"));
      end DiscreteSISO;
      partial block DiscreteMIMO "Multiple Input Multiple Output discrete control block"
        extends DiscreteBlock;
        parameter Integer nin = 1 "Number of inputs";
        parameter Integer nout = 1 "Number of outputs";
        Modelica.Blocks.Interfaces.RealInput u[nin] "Continuous input signals" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y[nout] "Continuous output signals" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<html>
<p>
Block has a continuous input and a continuous output signal vector
which are sampled due to the defined <b>samplePeriod</b> parameter.
</p>
</HTML>
"));
      end DiscreteMIMO;
      partial block DiscreteMIMOs "Multiple Input Multiple Output discrete control block"
        parameter Integer n = 1 "Number of inputs (= number of outputs)";
        extends DiscreteBlock;
        Modelica.Blocks.Interfaces.RealInput u[n] "Continuous input signals" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y[n] "Continuous output signals" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<html>
<p>
Block has a continuous input and a continuous output signal vector
where the signal sizes of the input and output vector are identical.
These signals are sampled due to the defined <b>samplePeriod</b> parameter.
</p>
</HTML>
"));
      end DiscreteMIMOs;
      partial block SVdiscrete "Discrete Single-Variable controller"
        extends DiscreteBlock;
        Discrete.Sampler sampler_s(final samplePeriod = samplePeriod, final startTime = startTime) annotation(Placement(transformation(extent = {{ -100, -10},{ -80,10}}, rotation = 0)));
        Discrete.Sampler sampler_m(final samplePeriod = samplePeriod, final startTime = startTime) annotation(Placement(transformation(origin = {0, -90}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Blocks.Interfaces.RealInput u_s "Continuous scalar setpoint input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u_m "Continuous scalar measurement input signal" annotation(Placement(transformation(origin = {0, -120}, extent = {{20, -20},{ -20,20}}, rotation = 270)));
        Modelica.Blocks.Interfaces.RealOutput y "Continuous scalar actuator output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -100,34},{ -140,24}}, lineColor = {0,0,0}, textString = "(setpoint)"),Text(extent = {{100,22},{130,14}}, lineColor = {0,0,0}, textString = "(actuator)"),Text(extent = {{ -70, -112},{ -20, -102}}, lineColor = {0,0,0}, textString = " (measurement)")}), Documentation(info = "<html>
<p>
Block has two continuous Real input signals and one
continuous Real output signal
that are sampled due to the defined <b>samplePeriod</b> parameter.
The block is designed
to be used as base class for a corresponding controller.
</p>
</html>"));

      equation 
        connect(u_s,sampler_s.u) annotation(Line(points = {{ -120,0},{ -102,0}}));
        connect(u_m,sampler_m.u) annotation(Line(points = {{0, -120},{0, -111},{0, -102},{ -7.34788e-16, -102}}));
      end SVdiscrete;
      partial block MVdiscrete "Discrete Multi-Variable controller"
        extends DiscreteBlock;
        parameter Integer nu_s = 1 "Number of setpoint inputs";
        parameter Integer nu_m = 1 "Number of measurement inputs";
        parameter Integer ny = 1 "Number of actuator outputs";
        Discrete.Sampler sampler_s[nu_s](each final samplePeriod = samplePeriod, each final startTime = startTime) annotation(Placement(transformation(extent = {{ -90, -10},{ -70,10}}, rotation = 0)));
        Discrete.Sampler sampler_m[nu_m](each final samplePeriod = samplePeriod, each final startTime = startTime) annotation(Placement(transformation(origin = {0, -80}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
        Modelica.Blocks.Interfaces.RealInput u_s[nu_s] "Continuous setpoint input signals" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealInput u_m[nu_m] "Continuous measurement input signals" annotation(Placement(transformation(origin = {0, -120}, extent = {{20, -20},{ -20,20}}, rotation = 270)));
        Modelica.Blocks.Interfaces.RealOutput y[ny] "Continuous actuator output signals" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -100, -10},{ -80, -30}}, textString = "u_s", lineColor = {0,0,255}),Text(extent = {{ -98,34},{ -138,24}}, lineColor = {0,0,0}, textString = "(setpoint)"),Text(extent = {{98,24},{138,14}}, lineColor = {0,0,0}, textString = "(actuator)"),Text(extent = {{ -62, -110},{ -12, -100}}, lineColor = {0,0,0}, textString = " (measurement)")}), Documentation(info = "<html>
<p>
Block has two continuous Real input signal vectors and one
continuous Real output signal vector. The vector signals
are sampled due to the defined <b>samplePeriod</b> parameter.
The block is designed
to be used as base class for a corresponding controller.
</p>
</html>"));

      equation 
        connect(u_s,sampler_s.u) annotation(Line(points = {{ -120,0},{ -92,0}}));
        connect(u_m,sampler_m.u) annotation(Line(points = {{0, -120},{0, -106},{0, -92},{ -7.34788e-16, -92}}));
      end MVdiscrete;
      partial block BooleanBlockIcon "Basic graphical layout of Boolean block"
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -100},{100,100}}, lineColor = {255,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,150},{150,110}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<html>
<p>
Block that has only the basic icon for an input/output,
Boolean block (no declarations, no equations).
</p>
</html>"));
      end BooleanBlockIcon;
      partial block BooleanSISO "Single Input Single Output control block with signals of type Boolean"
        extends BooleanBlockIcon;
      public 
        BooleanInput u "Connector of Boolean input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        BooleanOutput y "Connector of Boolean output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<html>
<p>
Block has one continuous Boolean input and one continuous Boolean output signal.
</p>
</html>"));
      end BooleanSISO;
      partial block BooleanMIMOs "Multiple Input Multiple Output continuous control block with same number of inputs and outputs of boolean type"
        extends BooleanBlockIcon;
        parameter Integer n = 1 "Number of inputs (= number of outputs)";
        BooleanInput u[n] "Connector of Boolean input signals" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        BooleanOutput y[n] "Connector of Boolean output signals" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<HTML>
<p>
Block has a continuous Boolean input and a continuous Boolean output signal vector
where the signal sizes of the input and output vector are identical.
</p>
</HTML>
"));
      end BooleanMIMOs;
      partial block MI2BooleanMOs "2 Multiple Input / Boolean Multiple Output block with same signal lengths"
        extends BooleanBlockIcon;
        parameter Integer n = 1 "Dimension of input and output vectors.";
        RealInput u1[n] "Connector 1 of Boolean input signals" annotation(Placement(transformation(extent = {{ -140,40},{ -100,80}}, rotation = 0)));
        RealInput u2[n] "Connector 2 of Boolean input signals" annotation(Placement(transformation(extent = {{ -140, -80},{ -100, -40}}, rotation = 0)));
        BooleanOutput y[n] "Connector of Boolean output signals" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<html>
<p>Block has two Boolean input vectors u1 and u2 and one Boolean output
vector y. All vectors have the same number of elements.</p>
</html>
"));
      end MI2BooleanMOs;
      partial block SI2BooleanSO "2 Single Input / Boolean Single Output block"
        extends BooleanBlockIcon;
        input RealInput u1 "Connector 1 of Boolean input signals" annotation(Placement(transformation(extent = {{ -140,40},{ -100,80}}, rotation = 0)));
        input RealInput u2 "Connector 2 of Boolean input signals" annotation(Placement(transformation(extent = {{ -140, -80},{ -100, -40}}, rotation = 0)));
        output BooleanOutput y "Connector of Boolean output signals" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<html>
<p>
Block has two Boolean input signals u1 and u2 and one Boolean output signal y.
</p>
</html>
"));
      end SI2BooleanSO;
      partial block BooleanSignalSource "Base class for Boolean signal sources"
        extends BooleanBlockIcon;
        BooleanOutput y "Connector of Boolean output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -80,68},{ -80, -80}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90, -70},{68, -70}}, color = {192,192,192}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
Basic block for Boolean sources of package Blocks.Sources.
This component has one continuous Boolean output signal y.
</p>
</html>"));
      end BooleanSignalSource;
      partial block IntegerBlockIcon "Basic graphical layout of Integer block"
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -100},{100,100}}, lineColor = {255,127,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,150},{150,110}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<html>
<p>
Block that has only the basic icon for an input/output,
Integer block (no declarations, no equations).
</p>
</html>"));
      end IntegerBlockIcon;
      partial block IntegerSO "Single Integer Output continuous control block"
        extends IntegerBlockIcon;
        IntegerOutput y "Connector of Integer output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<html>
<p>
Block has one continuous Integer output signal.
</p>
</html>"));
      end IntegerSO;
      partial block IntegerMO "Multiple Integer Output continuous control block"
        extends IntegerBlockIcon;
        parameter Integer nout(min = 1) = 1 "Number of outputs";
        IntegerOutput y[nout] "Connector of Integer output signals" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<html>
<p>
Block has one continuous Integer output signal vector.
</p>
</html>"));
      end IntegerMO;
      partial block IntegerSignalSource "Base class for continuous Integer signal source"
        extends IntegerSO;
        parameter Integer offset = 0 "Offset of output signal y";
        parameter SI.Time startTime = 0 "Output y = offset for time < startTime";
        annotation(Documentation(info = "<html>
<p>
Basic block for Integer sources of package Blocks.Sources.
This component has one continuous Integer output signal y
and two parameters (offset, startTime) to shift the
generated signal.
</p>
</html>"));
      end IntegerSignalSource;
      partial block IntegerSIBooleanSO "Integer Input Boolean Output continuous control block"
        extends BooleanBlockIcon;
        IntegerInput u "Connector of Integer input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        BooleanOutput y "Connector of Boolean output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<HTML>
<p>
Block has a continuous Integer input and a continuous Boolean output signal.
</p>
</HTML>
"));
      end IntegerSIBooleanSO;
      partial block IntegerMIBooleanMOs "Multiple Integer Input Multiple Boolean Output continuous control block with same number of inputs and outputs"
        extends BooleanBlockIcon;
        parameter Integer n = 1 "Number of inputs (= number of outputs)";
        IntegerInput u[n] "Connector of Integer input signals" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        BooleanOutput y[n] "Connector of Boolean output signals" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Documentation(info = "<HTML>
<p>
Block has a continuous Integer input and a continuous Boolean output signal vector
where the signal sizes of the input and output vector are identical.
</p>
</HTML>
"));
      end IntegerMIBooleanMOs;
      partial block partialBooleanBlockIcon "Basic graphical layout of logical block"
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {210,210,210}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised),Text(extent = {{ -150,150},{150,110}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<html>
<p>
Block that has only the basic icon for an input/output,
Boolean block (no declarations, no equations) used especially
in the Blocks.Logical library.
</p>
</html>"));
      end partialBooleanBlockIcon;
      partial block partialBooleanSISO "Partial block with 1 input and 1 output Boolean signal"
        extends partialBooleanBlockIcon;
        Blocks.Interfaces.BooleanInput u "Connector of Boolean input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Blocks.Interfaces.BooleanOutput y "Connector of Boolean output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -71,7},{ -85, -7}}, lineColor = DynamicSelect({235,235,235}, if u > 0.5 then {0,255,0} else {235,235,235}), fillColor = DynamicSelect({235,235,235}, if u > 0.5 then {0,255,0} else {235,235,235}), fillPattern = FillPattern.Solid),Ellipse(extent = {{71,7},{85, -7}}, lineColor = DynamicSelect({235,235,235}, if y > 0.5 then {0,255,0} else {235,235,235}), fillColor = DynamicSelect({235,235,235}, if y > 0.5 then {0,255,0} else {235,235,235}), fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>
Block has one continuous Boolean input and one continuous Boolean output signal
with a 3D icon (e.g. used in Blocks.Logical library).
</p>
</html>"));
      end partialBooleanSISO;
      partial block partialBooleanSI2SO "Partial block with 2 input and 1 output Boolean signal"
        extends partialBooleanBlockIcon;
        Blocks.Interfaces.BooleanInput u1 "Connector of first Boolean input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Blocks.Interfaces.BooleanInput u2 "Connector of second Boolean input signal" annotation(Placement(transformation(extent = {{ -140, -100},{ -100, -60}}, rotation = 0)));
        Blocks.Interfaces.BooleanOutput y "Connector of Boolean output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -71,7},{ -85, -7}}, lineColor = DynamicSelect({235,235,235}, if u1 > 0.5 then {0,255,0} else {235,235,235}), fillColor = DynamicSelect({235,235,235}, if u1 > 0.5 then {0,255,0} else {235,235,235}), fillPattern = FillPattern.Solid),Ellipse(extent = {{ -71, -74},{ -85, -88}}, lineColor = DynamicSelect({235,235,235}, if u2 > 0.5 then {0,255,0} else {235,235,235}), fillColor = DynamicSelect({235,235,235}, if u2 > 0.5 then {0,255,0} else {235,235,235}), fillPattern = FillPattern.Solid),Ellipse(extent = {{71,7},{85, -7}}, lineColor = DynamicSelect({235,235,235}, if y > 0.5 then {0,255,0} else {235,235,235}), fillColor = DynamicSelect({235,235,235}, if y > 0.5 then {0,255,0} else {235,235,235}), fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>
Block has two continuous Boolean input and one continuous Boolean output signal
with a 3D icon (e.g. used in Blocks.Logical library).
</p>
</html>"));
      end partialBooleanSI2SO;
      partial block partialBooleanSI3SO "Partial block with 3 input and 1 output Boolean signal"
        extends partialBooleanBlockIcon;
        Blocks.Interfaces.BooleanInput u1 "Connector of first Boolean input signal" annotation(Placement(transformation(extent = {{ -140,60},{ -100,100}}, rotation = 0)));
        Blocks.Interfaces.BooleanInput u2 "Connector of second Boolean input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Blocks.Interfaces.BooleanInput u3 "Connector of third Boolean input signal" annotation(Placement(transformation(extent = {{ -140, -100},{ -100, -60}}, rotation = 0)));
        Blocks.Interfaces.BooleanOutput y "Connector of Boolean output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -71,74},{ -85,88}}, lineColor = DynamicSelect({235,235,235}, if u1 > 0.5 then {0,255,0} else {235,235,235}), fillColor = DynamicSelect({235,235,235}, if u1 > 0.5 then {0,255,0} else {235,235,235}), fillPattern = FillPattern.Solid),Ellipse(extent = {{ -71,7},{ -85, -7}}, lineColor = DynamicSelect({235,235,235}, if u2 > 0.5 then {0,255,0} else {235,235,235}), fillColor = DynamicSelect({235,235,235}, if u2 > 0.5 then {0,255,0} else {235,235,235}), fillPattern = FillPattern.Solid),Ellipse(extent = {{ -71, -74},{ -85, -88}}, lineColor = DynamicSelect({235,235,235}, if u3 > 0.5 then {0,255,0} else {235,235,235}), fillColor = DynamicSelect({235,235,235}, if u3 > 0.5 then {0,255,0} else {235,235,235}), fillPattern = FillPattern.Solid),Ellipse(extent = {{71,7},{85, -7}}, lineColor = DynamicSelect({235,235,235}, if y > 0.5 then {0,255,0} else {235,235,235}), fillColor = DynamicSelect({235,235,235}, if y > 0.5 then {0,255,0} else {235,235,235}), fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
Block has three continuous Boolean input and one continuous Boolean output signal
with a 3D icon (e.g. used in Blocks.Logical library).
</p>
</html>"));
      end partialBooleanSI3SO;
      partial block partialBooleanSI "Partial block with 1 input Boolean signal"
        extends partialBooleanBlockIcon;
        Blocks.Interfaces.BooleanInput u "Connector of Boolean input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -71,7},{ -85, -7}}, lineColor = DynamicSelect({235,235,235}, if u > 0.5 then {0,255,0} else {235,235,235}), fillColor = DynamicSelect({235,235,235}, if u > 0.5 then {0,255,0} else {235,235,235}), fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>
Block has one continuous Boolean input signal
with a 3D icon (e.g. used in Blocks.Logical library).
</p>
</html>"));
      end partialBooleanSI;
      partial block partialBooleanSO "Partial block with 1 output Boolean signal"
        Blocks.Interfaces.BooleanOutput y "Connector of Boolean output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        extends partialBooleanBlockIcon;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{71,7},{85, -7}}, lineColor = DynamicSelect({235,235,235}, if y > 0.5 then {0,255,0} else {235,235,235}), fillColor = DynamicSelect({235,235,235}, if y > 0.5 then {0,255,0} else {235,235,235}), fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>
Block has one continuous Boolean output signal
with a 3D icon (e.g. used in Blocks.Logical library).
</p>
</html>"));
      end partialBooleanSO;
      partial block partialBooleanSource "Partial source block (has 1 output Boolean signal and an appropriate default icon)"
        extends partialBooleanBlockIcon;
        Blocks.Interfaces.BooleanOutput y "Connector of Boolean output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -80,88},{ -88,66},{ -72,66},{ -80,88}}, lineColor = {255,0,255}, fillColor = {255,0,255}, fillPattern = FillPattern.Solid),Line(points = {{ -80,66},{ -80, -82}}, color = {255,0,255}),Line(points = {{ -90, -70},{72, -70}}, color = {255,0,255}),Polygon(points = {{90, -70},{68, -62},{68, -78},{90, -70}}, lineColor = {255,0,255}, fillColor = {255,0,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{71,7},{85, -7}}, lineColor = DynamicSelect({235,235,235}, if y > 0.5 then {0,255,0} else {235,235,235}), fillColor = DynamicSelect({235,235,235}, if y > 0.5 then {0,255,0} else {235,235,235}), fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -70,92},{ -76,70},{ -64,70},{ -70,92}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Line(points = {{ -70,70},{ -70, -88}}, color = {95,95,95}),Line(points = {{ -90, -70},{68, -70}}, color = {95,95,95}),Polygon(points = {{90, -70},{68, -64},{68, -76},{90, -70}}, lineColor = {95,95,95}, fillColor = {95,95,95}, fillPattern = FillPattern.Solid),Text(extent = {{54, -80},{106, -92}}, lineColor = {0,0,0}, textString = "time"),Text(extent = {{ -64,92},{ -46,74}}, lineColor = {0,0,0}, textString = "y")}), Documentation(info = "<html>
<p>
Basic block for Boolean sources of package Blocks.Sources.
This component has one continuous Boolean output signal y
and a 3D icon (e.g. used in Blocks.Logical library).
</p>
</html>"));
      end partialBooleanSource;
      partial block partialBooleanThresholdComparison "Partial block to compare the Real input u with a threshold and provide the result as 1 Boolean output signal"
        parameter Real threshold = 0 "Comparison with respect to threshold";
        Blocks.Interfaces.RealInput u "Connector of Boolean input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Blocks.Interfaces.BooleanOutput y "Connector of Boolean output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {210,210,210}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised),Text(extent = {{ -150, -140},{150, -110}}, lineColor = {0,0,0}, textString = "%threshold"),Ellipse(extent = {{71,7},{85, -7}}, lineColor = DynamicSelect({235,235,235}, if y > 0.5 then {0,255,0} else {235,235,235}), fillColor = DynamicSelect({235,235,235}, if y > 0.5 then {0,255,0} else {235,235,235}), fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
Block has one continuous Real input and one continuous Boolean output signal
as well as a 3D icon (e.g. used in Blocks.Logical library).
</p>
</html>"));
      end partialBooleanThresholdComparison;
      partial block partialBooleanComparison "Partial block with 2 Real input and 1 Boolean output signal (the result of a comparison of the two Real inputs"
        Blocks.Interfaces.RealInput u1 "Connector of first Boolean input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Blocks.Interfaces.RealInput u2 "Connector of second Boolean input signal" annotation(Placement(transformation(extent = {{ -140, -100},{ -100, -60}}, rotation = 0)));
        Blocks.Interfaces.BooleanOutput y "Connector of Boolean output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {210,210,210}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised),Ellipse(extent = {{73,7},{87, -7}}, lineColor = DynamicSelect({235,235,235}, if y > 0.5 then {0,255,0} else {235,235,235}), fillColor = DynamicSelect({235,235,235}, if y > 0.5 then {0,255,0} else {235,235,235}), fillPattern = FillPattern.Solid),Ellipse(extent = {{32,10},{52, -10}}, lineColor = {0,0,255}),Line(points = {{ -100, -80},{42, -80},{42,0}}, color = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>
<p>
Block has two continuous Real input and one continuous Boolean output signal
as a result of the comparision of the two input signals. The block
has a 3D icon (e.g. used in Blocks.Logical library).
</p>
</html>"));
      end partialBooleanComparison;
      package Adaptors "Obsolete package with components to send signals to a bus or receive signals from a bus (only for backward compatibility)"
        extends Modelica.Icons.Library;
        block SendReal "Obsolete block to send Real signal to bus"
          RealOutput toBus "Output signal to be connected to bus" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
          RealInput u "Input signal to be send to bus" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,40},{100, -40}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -144,96},{144,46}}, lineColor = {0,0,0}, textString = "%name"),Text(extent = {{ -100,30},{100, -30}}, lineColor = {0,0,255}, textString = "send")}), Documentation(info = "<html>
<p>
Obsolete block that was previously used to connect a Real signal
to a signal in a connector. This block is only provided for
backward compatibility.
</p>

<p>
It is much more convenient and more powerful to use \"expandable connectors\"
for signal buses, see example
<a href=\"Modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a>.
</p>
</html>
"));

        equation 
          toBus = u;
        end SendReal;
        block SendBoolean "Obsolete block to send Boolean signal to bus"
          BooleanOutput toBus "Output signal to be connected to bus" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
          BooleanInput u "Input signal to be send to bus" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,40},{100, -40}}, lineColor = {255,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -144,96},{144,46}}, lineColor = {0,0,0}, textString = "%name"),Text(extent = {{ -100,30},{100, -30}}, lineColor = {255,0,255}, textString = "send")}), Documentation(info = "<html>
<p>
Obsolete block that was previously used to connect a Boolean signal
to a signal in a connector. This block is only provided for
backward compatibility.
</p>

<p>
It is much more convenient and more powerful to use \"expandable connectors\"
for signal buses, see example
<a href=\"Modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a>.
</p>
</html>
"));

        equation 
          toBus = u;
        end SendBoolean;
        block SendInteger "Obsolete block to send Integer signal to bus"
          IntegerOutput toBus "Output signal to be connected to bus" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
          IntegerInput u "Input signal to be send to bus" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,40},{100, -40}}, lineColor = {255,127,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -144,96},{144,46}}, lineColor = {0,0,0}, textString = "%name"),Text(extent = {{ -100,30},{100, -30}}, lineColor = {255,127,0}, textString = "send")}), Documentation(info = "<html>
<p>
Obsolete block that was previously used to connect an Integer signal
to a signal in a connector. This block is only provided for
backward compatibility.
</p>

<p>
It is much more convenient and more powerful to use \"expandable connectors\"
for signal buses, see example
<a href=\"Modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a>.
</p>
</html>"));

        equation 
          toBus = u;
        end SendInteger;
        block ReceiveReal "Obsolete block to receive Real signal from bus"
          RealInput fromBus "To be connected with signal on bus" annotation(Placement(transformation(extent = {{ -120, -10},{ -100,10}}, rotation = 0)));
          RealOutput y "Output signal to be received from bus" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,40},{100, -40}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -100,30},{100, -30}}, lineColor = {0,0,255}, textString = "receive"),Text(extent = {{ -144,96},{144,46}}, lineColor = {0,0,0}, textString = "%name")}), Documentation(info = "<html>
<p>
Obsolete block that was previously used to connect a Real signal
in a connector to an input of a block. This block is only provided for
backward compatibility.
</p>

<p>
It is much more convenient and more powerful to use \"expandable connectors\"
for signal buses, see example
<a href=\"Modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a>.
</p>
</html>"));

        equation 
          y = fromBus;
        end ReceiveReal;
        block ReceiveBoolean "Obsolete block to receive Boolean signal from bus"
          BooleanInput fromBus "To be connected with signal on bus" annotation(Placement(transformation(extent = {{ -120, -10},{ -100,10}}, rotation = 0)));
          BooleanOutput y "Output signal to be received from bus" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,40},{100, -40}}, lineColor = {255,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -100,30},{100, -30}}, lineColor = {255,0,255}, textString = "receive"),Text(extent = {{ -144,96},{144,46}}, lineColor = {0,0,0}, textString = "%name")}), Documentation(info = "<html>
<p>
Obsolete block that was previously used to connect a Boolean signal
in a connector to an input of a block. This block is only provided for
backward compatibility.
</p>

<p>
It is much more convenient and more powerful to use \"expandable connectors\"
for signal buses, see example
<a href=\"Modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a>.
</p>
</html>"));

        equation 
          y = fromBus;
        end ReceiveBoolean;
        block ReceiveInteger "Obsolete block to receive Integer signal from bus"
          IntegerInput fromBus "To be connected with signal on bus" annotation(Placement(transformation(extent = {{ -120, -10},{ -100,10}}, rotation = 0)));
          IntegerOutput y "Output signal to be received from bus" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,40},{100, -40}}, lineColor = {255,127,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -100,30},{100, -30}}, lineColor = {255,127,0}, textString = "receive"),Text(extent = {{ -144,96},{144,46}}, lineColor = {0,0,0}, textString = "%name")}), Documentation(info = "<html>
<p>
Obsolete block that was previously used to connect an Integer signal
in a connector to an input of a block. This block is only provided for
backward compatibility.
</p>

<p>
It is much more convenient and more powerful to use \"expandable connectors\"
for signal buses, see example
<a href=\"Modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a>.
</p>
</html>
"));

        equation 
          y = fromBus;
        end ReceiveInteger;
        annotation(Documentation(info = "<html>
<p>
The components of this package should no longer be used.
They are only provided for backward compatibility.
It is much more convenient and more powerful to use \"expandable connectors\"
for signal buses, see example
<a href=\"Modelica://Modelica.Blocks.Examples.BusUsage\">BusUsage</a>.
</p>
</html>"));
      end Adaptors;
      partial block PartialConversionBlock "Partial block defining the interface for conversion blocks"
        RealInput u "Connector of Real input signal to be converted" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        RealOutput y "Connector of Real output signal containing input signal u in another unit" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,127}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{30,0}}, color = {191,0,0}),Polygon(points = {{90,0},{30,20},{30, -20},{90,0}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -115,155},{115,105}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<html>
<p>
This block defines the interface of a conversion block that
converts from one unit into another one.
</p>

</html>"));
      end PartialConversionBlock;
    end Interfaces;
    package Discrete "Library of discrete input/output blocks with fixed sample period"
      extends Modelica.Icons.Library;
      annotation(Documentation(info = "<html>
<p>
This package contains <b>discrete control blocks</b>
with <b>fixed sample period</b>.
Every component of this package is structured in the following way:
</p>
<ol>
<li> A component has <b>continuous real</b> input and output signals.</li>
<li> The <b>input</b> signals are <b>sampled</b> by the given sample period
     defined via parameter <b>samplePeriod</b>.
     The first sample instant is defined by parameter <b>startTime</b>.
<li> The <b>output</b> signals are computed from the sampled input signals.
</ol>
<p>
A <b>sampled data system</b> may consist of components of package <b>Discrete</b>
and of every other purely <b>algebraic</b> input/output block, such
as the components of packages <b>Modelica.Blocks.Math</b>,
<b>Modelica.Blocks.Nonlinear</b> or <b>Modelica.Blocks.Sources</b>.
</p>

</HTML>
", revisions = "<html>
<ul>
<li><i>October 21, 2002</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       New components TriggeredSampler and TriggeredMax added.</li>
<li><i>June 18, 2000</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized based on a corresponding library of Dieter Moormann and
       Hilding Elmqvist.</li>
</ul>
</html>"));
      block Sampler "Ideal sampling of continuous signals"
        extends Interfaces.DiscreteSISO;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -25, -10},{ -45,10}}, lineColor = {0,0,127}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{45, -10},{25,10}}, lineColor = {0,0,127}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -100,0},{ -45,0}}, color = {0,0,127}),Line(points = {{45,0},{100,0}}, color = {0,0,127}),Line(points = {{ -35,0},{30,35}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -25, -10},{ -45,10}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{45, -10},{25,10}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -100,0},{ -45,0}}, color = {0,0,255}),Line(points = {{45,0},{100,0}}, color = {0,0,255}),Line(points = {{ -35,0},{30,35}}, color = {0,0,255})}), Documentation(info = "<HTML>
<p>
Samples the continues input signal with a sampling rate defined
via parameter <b>samplePeriod</b>.
</p>
</HTML>
"));

      equation 
        when {sampleTrigger,initial()} then
                  y = u;
        
        end when;
      end Sampler;
      block ZeroOrderHold "Zero order hold of a sampled-data system"
        extends Interfaces.DiscreteSISO;
        output Real ySample(start = 0, fixed = true);
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -78, -42},{ -52, -42},{ -52,0},{ -26,0},{ -26,24},{ -6,24},{ -6,64},{18,64},{18,20},{38,20},{38,0},{44,0},{44,0},{62,0}}, color = {0,0,127})}), Documentation(info = "<HTML>
<p>
The output is identical to the sampled input signal at sample
time instants and holds the output at the value of the last
sample instant during the sample points.
</p>
</HTML>
"));

      equation 
        when {sampleTrigger,initial()} then
                  ySample = u;
        
        end when;
        y = pre(ySample);
      end ZeroOrderHold;
      block FirstOrderHold "First order hold of a sampled-data system"
        extends Interfaces.DiscreteSISO;
      protected 
        Real ySample;
        Modelica.SIunits.Time tSample;
        Real c;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -79, -41},{ -59, -33},{ -40,1},{ -20,9},{0,63},{21,20},{41,10},{60,20}}, color = {0,0,127}),Line(points = {{60,19},{81,10}}, color = {0,0,255})}), Documentation(info = "<HTML>
<p>
The output signal is the extrapolation through the
values of the last two sampled input signals.
</p>
</HTML>
"));

      equation 
        when sampleTrigger then
                  ySample = u;
          tSample = time;
          c = if firstTrigger then 0 else (ySample - pre(ySample)) / samplePeriod;
        
        end when;
        y = pre(ySample) + pre(c) * (time - tSample);
      end FirstOrderHold;
      block UnitDelay "Unit Delay Block"
        parameter Real y_start = 0 "Initial value of output signal";
        extends Interfaces.DiscreteSISO;
        annotation(Documentation(info = "<html>
<p>
This block describes a unit delay:
</p>
<pre>
          1
     y = --- * u
          z
</pre>
<p>
that is, the output signal y is the input signal u of the
previous sample instant. Before the second sample instant,
the output y is identical to parameter yStart.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -30,0},{30,0}}, color = {0,0,127}),Text(extent = {{ -90,10},{90,90}}, textString = "1", lineColor = {0,0,127}),Text(extent = {{ -90, -10},{90, -90}}, textString = "z", lineColor = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,255}),Text(extent = {{ -160,10},{ -140, -10}}, textString = "u", lineColor = {0,0,255}),Text(extent = {{115,10},{135, -10}}, textString = "y", lineColor = {0,0,255}),Line(points = {{ -100,0},{ -60,0}}, color = {0,0,255}),Line(points = {{60,0},{100,0}}, color = {0,0,255}),Line(points = {{40,0},{ -40,0}}, color = {0,0,0}),Text(extent = {{ -55,55},{55,5}}, lineColor = {0,0,0}, textString = "1"),Text(extent = {{ -55, -5},{55, -55}}, lineColor = {0,0,0}, textString = "z")}));

      equation 
        when sampleTrigger then
                  y = pre(u);
        
        end when;

      initial equation 
        y = y_start;
      end UnitDelay;
      block TransferFunction "Discrete Transfer Function block"
        parameter Real b[:] = {1} "Numerator coefficients of transfer function.";
        parameter Real a[:] "Denominator coefficients of transfer function.";
        extends Interfaces.DiscreteSISO;
        output Real x[size(a, 1) - 1](each start = 0, each fixed = true) "State of transfer function from controller canonical form";
      protected 
        parameter Integer nb = size(b, 1) "Size of Numerator of transfer function";
        parameter Integer na = size(a, 1) "Size of Denominator of transfer function";
        Real x1;
        Real xext[size(a, 1)];
        annotation(Documentation(info = "<html>
<p>The <b>discrete transfer function</b> block defines the
transfer function between the input signal u and the output
signal y. The numerator has the order nb-1, the denominator
has the order na-1.</p>
<pre>          b(1)*z^(nb-1) + b(2)*z^(nb-2) + ... + b(nb)
   y(z) = -------------------------------------------- * u(z)
          a(1)*z^(na-1) + a(2)*z^(na-2) + ... + a(na)
</pre>
<p>State variables <b>x</b> are defined according to
<b>controller canonical</b> form. Initial values of the
states can be set as start values of <b>x</b>.<p>
<p>Example:</p>
<pre>     Blocks.Discrete.TransferFunction g(b = {2,4}, a = {1,3});
</pre>
<p>results in the following transfer function:</p>
<pre>        2*z + 4
   y = --------- * u
         z + 3
</pre>

</HTML>
", revisions = "<html>
<p><b>Release Notes:</b></p>
<ul>
<li><i>November 15, 2000</i>
    by <a href=\"http://www.dynasim.se\">Hans Olsson</a>:<br>
    Converted to when-semantics of Modelica 1.4 with special
    care to avoid unnecessary algebraic loops.</li>
<li><i>June 18, 2000</i>
    by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
    Realized based on a corresponding model of Dieter Moormann
    and Hilding Elmqvist.</li>
</ul>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{82,0},{ -84,0}}, color = {0,0,127}),Text(extent = {{ -92,92},{86,12}}, lineColor = {0,0,127}, textString = "b(z)"),Text(extent = {{ -90, -12},{90, -90}}, lineColor = {0,0,127}, textString = "a(z)")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,255}),Line(points = {{40,0},{ -44,0}}, color = {0,0,0}, thickness = 0.5),Text(extent = {{ -54,54},{54,4}}, lineColor = {0,0,0}, textString = "b(z)"),Text(extent = {{ -54, -6},{56, -56}}, lineColor = {0,0,0}, textString = "a(z)"),Line(points = {{ -100,0},{ -60,0}}, color = {0,0,255}),Line(points = {{60,0},{100,0}}, color = {0,0,255})}));

      equation 
        when sampleTrigger then
                  x1 = (u - a[2:size(a, 1)] * pre(x)) / a[1];
          xext = vector([x1;pre(x)]);
          x = xext[1:size(x, 1)];
          y = vector([zeros(na - nb, 1);b]) * xext;
        
        end when;
      end TransferFunction;
      block StateSpace "Discrete State Space block"
        parameter Real A[:,size(A, 1)] "Matrix A of state space model";
        parameter Real B[size(A, 1),:] "Matrix B of state space model";
        parameter Real C[:,size(A, 1)] "Matrix C of state space model";
        parameter Real D[size(C, 1),size(B, 2)] = zeros(size(C, 1), size(B, 2)) "Matrix D of state space model";
        extends Interfaces.DiscreteMIMO(final nin = size(B, 2), final nout = size(C, 1));
        output Real x[size(A, 1)] "State vector";
        annotation(Documentation(info = "<html>
<p>
The <b>discrete state space</b> block defines the relation
between the input u=inPort.signal and the output
y=outPort.signal in state space form:
</p>
<pre>
    x = A * pre(x) + B * u
    y = C * pre(x) + D * u
</pre>
<p>
where pre(x) is the value of the discrete state x at
the previous sample time instant.
The input is a vector of length nu, the output is a vector
of length ny and nx is the number of states. Accordingly
</p>
<pre>
        A has the dimension: A(nx,nx),
        B has the dimension: B(nx,nu),
        C has the dimension: C(ny,nx),
        D has the dimension: D(ny,nu)
</pre>
<p>
Example:
</p>
<pre>
     parameter: A = [0.12, 2;3, 1.5]
     parameter: B = [2, 7;3, 1]
     parameter: C = [0.1, 2]
     parameter: D = zeros(ny,nu)

results in the following equations:
  [x[1]]   [0.12  2.00] [pre(x[1])]   [2.0  7.0] [u[1]]
  [    ] = [          ]*[         ] + [        ]*[    ]
  [x[2]]   [3.00  1.50] [pre(x[2])]   [0.1  2.0] [u[2]]
                             [pre(x[1])]            [u[1]]
       y[1]   = [0.1  2.0] * [         ] + [0  0] * [    ]
                             [pre(x[2])]            [u[2]]
</pre>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -90,15},{ -15,90}}, textString = "A", lineColor = {0,0,127}),Text(extent = {{15,15},{90,90}}, textString = "B", lineColor = {0,0,127}),Text(extent = {{ -52,28},{54, -20}}, textString = "z", lineColor = {0,0,127}),Text(extent = {{ -90, -15},{ -15, -90}}, textString = "C", lineColor = {0,0,127}),Text(extent = {{15, -15},{90, -90}}, textString = "D", lineColor = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,255}),Text(extent = {{ -54,50},{52, -10}}, lineColor = {0,0,0}, textString = "zx=Ax+Bu"),Text(extent = {{ -56,14},{54, -50}}, lineColor = {0,0,0}, textString = "  y=Cx+Du"),Line(points = {{ -102,0},{ -60,0}}, color = {0,0,255}),Line(points = {{60,0},{100,0}}, color = {0,0,255})}));

      equation 
        when sampleTrigger then
                  x = A * pre(x) + B * u;
          y = C * pre(x) + D * u;
        
        end when;
      end StateSpace;
      block TriggeredSampler "Triggered sampling of continuous signals"
        extends Interfaces.DiscreteBlockIcon;
        parameter Real y_start = 0 "initial value of output signal";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -25, -10},{ -45,10}}, lineColor = {0,0,127}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{45, -10},{25,10}}, lineColor = {0,0,127}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -100,0},{ -45,0}}, color = {0,0,127}),Line(points = {{45,0},{100,0}}, color = {0,0,127}),Line(points = {{0, -100},{0, -26}}, color = {255,0,255}),Line(points = {{ -35,0},{28, -48}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -25, -10},{ -45,10}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{45, -10},{25,10}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -100,0},{ -45,0}}, color = {0,0,255}),Line(points = {{45,0},{100,0}}, color = {0,0,255}),Line(points = {{ -35,0},{28, -48}}, color = {0,0,255}),Line(points = {{0, -100},{0, -26}}, color = {255,0,255})}), Documentation(info = "<HTML>
<p>
Samples the continuous input signal whenever the trigger input
signal is rising (i.e., trigger changes from <b>false</b> to
<b>true</b>) and provides the sampled input signal as output.
Before the first sampling, the output signal is equal to
the initial value defined via parameter <b>y0</b>.
</p>
</HTML>
"));
        Modelica.Blocks.Interfaces.RealInput u "Connector with a Real input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y "Connector with a Real output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        Modelica.Blocks.Interfaces.BooleanInput trigger annotation(Placement(transformation(origin = {0, -118}, extent = {{ -20, -20},{20,20}}, rotation = 90)));

      equation 
        when trigger then
                  y = u;
        
        end when;

      initial equation 
        y = y_start;
      end TriggeredSampler;
      block TriggeredMax "Compute maximum, absolute value of continuous signal at trigger instants"
        extends Interfaces.DiscreteBlockIcon;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -25, -10},{ -45,10}}, lineColor = {0,0,127}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{45, -10},{25,10}}, lineColor = {0,0,127}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -100,0},{ -45,0}}, color = {0,0,127}),Line(points = {{45,0},{100,0}}, color = {0,0,127}),Line(points = {{0, -100},{0, -26}}, color = {255,0,255}),Line(points = {{ -35,0},{28, -48}}, color = {0,0,127}),Text(extent = {{ -86,82},{82,24}}, lineColor = {0,0,0}, textString = "max")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -25, -10},{ -45,10}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Ellipse(extent = {{45, -10},{25,10}}, lineColor = {0,0,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -100,0},{ -45,0}}, color = {0,0,255}),Line(points = {{45,0},{100,0}}, color = {0,0,255}),Line(points = {{ -35,0},{28, -48}}, color = {0,0,255}),Line(points = {{0, -100},{0, -26}}, color = {255,0,255})}), Documentation(info = "<HTML>
<p>
Samples the continuous input signal whenever the trigger input
signal is rising (i.e., trigger changes from <b>false</b> to
<b>true</b>). The maximum, absolute value of the input signal
at the sampling point is provided as output signal.
</p>
</HTML>
"));
        Modelica.Blocks.Interfaces.RealInput u "Connector with a Real input signal" annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        Modelica.Blocks.Interfaces.RealOutput y "Connector with a Real output signal" annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
        Modelica.Blocks.Interfaces.BooleanInput trigger annotation(Placement(transformation(origin = {0, -118}, extent = {{ -20, -20},{20,20}}, rotation = 90)));

      equation 
        when trigger then
                  y = max(pre(y), abs(u));
        
        end when;

      initial equation 
        y = 0;
      end TriggeredMax;
    end Discrete;
    package Continuous "Library of continuous control blocks with internal states"
      import Modelica.Blocks.Interfaces;
      import Modelica.SIunits;
      extends Modelica.Icons.Library;
      annotation(Documentation(info = "<html>
<p>
This package contains basic <b>continuous</b> input/output blocks
described by differential equations.
</p>

<p>
All blocks of this package can be initialized in different
ways controlled by parameter <b>initType</b>. The possible
values of initType are defined in
<a href=\"Modelica://Modelica.Blocks.Types.Init\">Modelica.Blocks.Types.Init</a>:
</p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Name</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>

  <tr><td valign=\"top\"><b>Init.NoInit</b></td>
      <td valign=\"top\">no initialization (start values are used as guess values with fixed=false)</td></tr>

  <tr><td valign=\"top\"><b>Init.SteadyState</b></td>
      <td valign=\"top\">steady state initialization (derivatives of states are zero)</td></tr>

  <tr><td valign=\"top\"><b>Init.InitialState</b></td>
      <td valign=\"top\">Initialization with initial states</td></tr>

  <tr><td valign=\"top\"><b>Init.InitialOutput</b></td>
      <td valign=\"top\">Initialization with initial outputs (and steady state of the states if possibles)</td></tr>
</table>

<p>
For backward compatibility reasons the default of all blocks is
<b>Init.NoInit</b>, with the exception of Integrator and LimIntegrator
where the default is <b>Init.InitialState</b> (this was the initialization
defined in version 2.2 of the Modelica standard library).
</p>

<p>
In many cases, the most useful initial condition is
<b>Init.SteadyState</b> because initial transients are then no longer
present. The drawback is that in combination with a non-linear
plant, non-linear algebraic equations occur that might be
difficult to solve if appropriate guess values for the
iteration variables are not provided (i.e. start values with fixed=false).
However, it is often already useful to just initialize
the linear blocks from the Continuous blocks library in SteadyState.
This is uncritical, because only linear algebraic equations occur.
If Init.NoInit is set, then the start values for the states are
interpreted as <b>guess</b> values and are propagated to the
states with fixed=<b>false</b>.
</p>

<p>
Note, initialization with Init.SteadyState is usually difficult
for a block that contains an integrator
(Integrator, LimIntegrator, PI, PID, LimPID).
This is due to the basic equation of an integrator:
</p>

<pre>
  <b>initial equation</b>
     <b>der</b>(y) = 0;   // Init.SteadyState
  <b>equation</b>
     <b>der</b>(y) = k*u;
</pre>

<p>
The steady state equation leads to the condition that the input to the
integrator is zero. If the input u is already (directly or indirectly) defined
by another initial condition, then the initialization problem is <b>singular</b>
(has none or infinitely many solutions). This situation occurs often
for mechanical systems, where, e.g., u = desiredSpeed - measuredSpeed and
since speed is both a state and a derivative, it is always defined by
Init.InitialState or Init.SteadyState initializtion.
</p>

<p>
In such a case, <b>Init.NoInit</b> has to be selected for the integrator
and an additional initial equation has to be added to the system
to which the integrator is connected. E.g., useful initial conditions
for a 1-dim. rotational inertia controlled by a PI controller are that
<b>angle</b>, <b>speed</b>, and <b>acceleration</b> of the inertia are zero.
</p>

</html>
"));
      block Integrator "Output the integral of the input signal"
        import Modelica.Blocks.Types.Init;
        parameter Real k = 1 "Integrator gain";
        parameter Modelica.Blocks.Types.Init initType = Modelica.Blocks.Types.Init.InitialState "Type of initialization (1: no init, 2: steady state, 3,4: initial output)" annotation(Evaluate = true, Dialog(group = "Initialization"));
        parameter Real y_start = 0 "Initial or guess value of output (= state)" annotation(Dialog(group = "Initialization"));
        extends Interfaces.SISO(y(start = y_start));
        annotation(Documentation(info = "<html>
<p>
This blocks computes output <b>y</b> (element-wise) as
<i>integral</i> of the input <b>u</b> multiplied with
the gain <i>k</i>:
</p>
<pre>
         k
     y = - u
         s
</pre>

<p>
It might be difficult to initialize the integrator in steady state.
This is discussed in the description of package
<a href=\"Modelica://Modelica.Blocks.Continuous#info\">Continuous</a>.
</p>

</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -80,78},{ -80, -90}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90, -80},{82, -80}}, color = {192,192,192}),Polygon(points = {{90, -80},{68, -72},{68, -88},{90, -80}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{0, -10},{60, -70}}, lineColor = {192,192,192}, textString = "I"),Text(extent = {{ -150, -150},{150, -110}}, lineColor = {0,0,0}, textString = "k=%k"),Line(points = {{ -80, -80},{80,80}}, color = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,255}),Line(points = {{ -100,0},{ -60,0}}, color = {0,0,255}),Line(points = {{60,0},{100,0}}, color = {0,0,255}),Text(extent = {{ -36,60},{32,2}}, lineColor = {0,0,0}, textString = "k"),Text(extent = {{ -32,0},{36, -58}}, lineColor = {0,0,0}, textString = "s"),Line(points = {{ -46,0},{46,0}}, color = {0,0,0})}));

      initial equation 
        if initType == Init.SteadyState then
          der(y) = 0;

        elseif initType == Init.InitialState or initType == Init.InitialOutput then
          y = y_start;


        else

        end if;

      equation 
        der(y) = k * u;
      end Integrator;
      block LimIntegrator "Integrator with limited value of the output"
        import Modelica.Blocks.Types.Init;
        parameter Real k = 1 "Integrator gain";
        parameter Real outMax(start = 1) "Upper limit of output";
        parameter Real outMin =  -outMax "Lower limit of output";
        parameter Modelica.Blocks.Types.Init initType = Modelica.Blocks.Types.Init.InitialState "Type of initialization (1: no init, 2: steady state, 3/4: initial output)" annotation(Evaluate = true, Dialog(group = "Initialization"));
        parameter Boolean limitsAtInit = true "= false, if limits are ignored during initializiation (i.e., der(y)=k*u)" annotation(Evaluate = true, Dialog(group = "Initialization"));
        parameter Real y_start = 0 "Initial or guess value of output (must be in the limits outMin .. outMax)" annotation(Dialog(group = "Initialization"));
        extends Interfaces.SISO(y(start = y_start));
        annotation(Documentation(info = "<html>
<p>
This blocks computes <b>y</b> (element-wise) as <i>integral</i>
of the input <b>u</b> multiplied with the gain <i>k</i>. If the
integral reaches a given upper or lower <i>limit</i> and the
input will drive the integral outside of this bound, the
integration is halted and only restarted if the input drives
the integral away from the bounds.
</p>

<p>
It might be difficult to initialize the integrator in steady state.
This is discussed in the description of package
<a href=\"Modelica://Modelica.Blocks.Continuous#info\">Continuous</a>.
</p>

<p>
If parameter <b>limitAtInit</b> = <b>false</b>, the limits of the
integrator are removed from the initialization problem which
leads to a much simpler equation system. After initialization has been
performed, it is checked via an assert whether the output is in the
defined limits. For backward compatibility reasons
<b>limitAtInit</b> = <b>true</b>. In most cases it is best
to use <b>limitAtInit</b> = <b>false</b>.
</p>
</html>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -80,78},{ -80, -90}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90, -80},{82, -80}}, color = {192,192,192}),Polygon(points = {{90, -80},{68, -72},{68, -88},{90, -80}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{20,20},{80,20}}, color = {0,0,127}),Text(extent = {{0, -10},{60, -70}}, lineColor = {192,192,192}, textString = "I"),Text(extent = {{ -150, -150},{150, -110}}, lineColor = {0,0,0}, textString = "k=%k")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,255}),Text(extent = {{ -54,46},{ -4, -48}}, lineColor = {0,0,0}, textString = "lim"),Line(points = {{ -100,0},{ -60,0}}, color = {0,0,255}),Line(points = {{60,0},{100,0}}, color = {0,0,255}),Text(extent = {{ -8,60},{60,2}}, lineColor = {0,0,0}, textString = "k"),Text(extent = {{ -8, -2},{60, -60}}, lineColor = {0,0,0}, textString = "s"),Line(points = {{4,0},{46,0}}, color = {0,0,0})}));

      initial equation 
        if initType == Init.SteadyState then
          der(y) = 0;

        elseif initType == Init.InitialState or initType == Init.InitialOutput then
          y = y_start;


        else

        end if;

      equation 
        if initial() and not limitsAtInit then
          der(y) = k * u;
          assert(y >= outMin - 0.01 * abs(outMin) and y <= outMax + 0.01 * abs(outMax), "LimIntegrator: During initialization the limits have been ignored.\n" + "However, the result is that the output y is not within the required limits:\n" + "  y = " + String(y) + ", outMin = " + String(outMin) + ", outMax = " + String(outMax));

        else
          der(y) = if y < outMin and u < 0 or y > outMax and u > 0 then 0 else k * u;

        end if;
      end LimIntegrator;
      block Derivative "Approximated derivative block"
        import Modelica.Blocks.Types.Init;
        parameter Real k = 1 "Gains";
        parameter SIunits.Time T(min = Modelica.Constants.small) = 0.01 "Time constants (T>0 required; T=0 is ideal derivative block)";
        parameter Modelica.Blocks.Types.Init initType = Modelica.Blocks.Types.Init.NoInit "Type of initialization (1: no init, 2: steady state, 3: initial state, 4: initial output)" annotation(Evaluate = true, Dialog(group = "Initialization"));
        parameter Real x_start = 0 "Initial or guess value of state" annotation(Dialog(group = "Initialization"));
        parameter Real y_start = 0 "Initial value of output (= state)" annotation(Dialog(enable = initType == Init.InitialOutput, group = "Initialization"));
        extends Interfaces.SISO;
        output Real x(start = x_start) "State of block";
        annotation(Documentation(info = "
<HTML>
<p>
This blocks defines the transfer function between the
input u and the output y
(element-wise) as <i>approximated derivative</i>:
</p>
<pre>
             k * s
     y = ------------ * u
            T * s + 1
</pre>
<p>
If you would like to be able to change easily between different
transfer functions (FirstOrder, SecondOrder, ... ) by changing
parameters, use the general block <b>TransferFunction</b> instead
and model a derivative block with parameters<br>
b = {k,0}, a = {T, 1}.
</p>

<p>
If k=0, the block reduces to y=0.
</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -80,78},{ -80, -90}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90, -80},{82, -80}}, color = {192,192,192}),Polygon(points = {{90, -80},{68, -72},{68, -88},{90, -80}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -80,60},{ -70,17.95},{ -60, -11.46},{ -50, -32.05},{ -40, -46.45},{ -30, -56.53},{ -20, -63.58},{ -10, -68.51},{0, -71.96},{10, -74.37},{20, -76.06},{30, -77.25},{40, -78.07},{50, -78.65},{60, -79.06}}, color = {0,0,127}),Text(extent = {{ -30,14},{86,60}}, lineColor = {192,192,192}, textString = "DT1"),Text(extent = {{ -150, -150},{150, -110}}, lineColor = {0,0,0}, textString = "k=%k")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -54,52},{50,10}}, lineColor = {0,0,0}, textString = "k s"),Text(extent = {{ -54, -6},{52, -52}}, lineColor = {0,0,0}, textString = "T s + 1"),Line(points = {{ -50,0},{50,0}}, color = {0,0,0}),Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,255}),Line(points = {{ -100,0},{ -60,0}}, color = {0,0,255}),Line(points = {{60,0},{100,0}}, color = {0,0,255})}));
      protected 
        parameter Boolean zeroGain = abs(k) < Modelica.Constants.eps;

      initial equation 
        if initType == Init.SteadyState then
          der(x) = 0;

        elseif initType == Init.InitialState then
          x = x_start;


        elseif initType == Init.InitialOutput then
          if zeroGain then
            x = u;

          else
            y = y_start;

          end if;


        else

        end if;

      equation 
        der(x) = if zeroGain then 0 else (u - x) / T;
        y = if zeroGain then 0 else k / T * (u - x);
      end Derivative;
      block FirstOrder "First order transfer function block (= 1 pole)"
        import Modelica.Blocks.Types.Init;
        parameter Real k = 1 "Gain";
        parameter SIunits.Time T(start = 1) "Time Constant";
        parameter Modelica.Blocks.Types.Init initType = Modelica.Blocks.Types.Init.NoInit "Type of initialization (1: no init, 2: steady state, 3/4: initial output)" annotation(Evaluate = true, Dialog(group = "Initialization"));
        parameter Real y_start = 0 "Initial or guess value of output (= state)" annotation(Dialog(group = "Initialization"));
        extends Interfaces.SISO(y(start = y_start));
        annotation(Documentation(info = "<HTML>
<p>
This blocks defines the transfer function between the input u
and the output y (element-wise) as <i>first order</i> system:
</p>
<pre>
               k
     y = ------------ * u
            T * s + 1
</pre>
<p>
If you would like to be able to change easily between different
transfer functions (FirstOrder, SecondOrder, ... ) by changing
parameters, use the general block <b>TransferFunction</b> instead
and model a first order SISO system with parameters<br>
b = {k}, a = {T, 1}.
</p>
<pre>
Example:
   parameter: k = 0.3, T = 0.4
   results in:
             0.3
      y = ----------- * u
          0.4 s + 1.0
</pre>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -80,78},{ -80, -90}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,88},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90, -80},{82, -80}}, color = {192,192,192}),Polygon(points = {{90, -80},{68, -72},{68, -88},{90, -80}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -70, -45.11},{ -60, -19.58},{ -50, -0.9087},{ -40,12.75},{ -30,22.75},{ -20,30.06},{ -10,35.41},{0,39.33},{10,42.19},{20,44.29},{30,45.82},{40,46.94},{50,47.76},{60,48.36},{70,48.8},{80,49.12}}, color = {0,0,127}),Text(extent = {{0,0},{60, -60}}, lineColor = {192,192,192}, textString = "PT1"),Text(extent = {{ -150, -150},{150, -110}}, lineColor = {0,0,0}, textString = "T=%T")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -48,52},{50,8}}, lineColor = {0,0,0}, textString = "k"),Text(extent = {{ -54, -6},{56, -56}}, lineColor = {0,0,0}, textString = "T s + 1"),Line(points = {{ -50,0},{50,0}}, color = {0,0,0}),Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,255}),Line(points = {{ -100,0},{ -60,0}}, color = {0,0,255}),Line(points = {{60,0},{100,0}}, color = {0,0,255})}));

      initial equation 
        if initType == Init.SteadyState then
          der(y) = 0;

        elseif initType == Init.InitialState or initType == Init.InitialOutput then
          y = y_start;


        else

        end if;

      equation 
        der(y) = (k * u - y) / T;
      end FirstOrder;
      block SecondOrder "Second order transfer function block (= 2 poles)"
        import Modelica.Blocks.Types.Init;
        parameter Real k = 1 "Gain";
        parameter Real w(start = 1) "Angular frequency";
        parameter Real D(start = 1) "Damping";
        parameter Modelica.Blocks.Types.Init initType = Modelica.Blocks.Types.Init.NoInit "Type of initialization (1: no init, 2: steady state, 3/4: initial output)" annotation(Evaluate = true, Dialog(group = "Initialization"));
        parameter Real y_start = 0 "Initial or guess value of output (= state)" annotation(Dialog(group = "Initialization"));
        parameter Real yd_start = 0 "Initial or guess value of derivative of output (= state)" annotation(Dialog(group = "Initialization"));
        extends Interfaces.SISO(y(start = y_start));
        output Real yd(start = yd_start) "Derivative of y";
        annotation(Documentation(info = "<HTML>
<p>
This blocks defines the transfer function between the input u and
the output y (element-wise) as <i>second order</i> system:
</p>
<pre>
                             k
     y = ---------------------------------------- * u
            ( s / w )^2 + 2*D*( s / w ) + 1
</pre>
<p>
If you would like to be able to change easily between different
transfer functions (FirstOrder, SecondOrder, ... ) by changing
parameters, use the general model class <b>TransferFunction</b>
instead and model a second order SISO system with parameters<br>
b = {k}, a = {1/w^2, 2*D/w, 1}.
</p>
<pre>
Example:

   parameter: k =  0.3,  w = 0.5,  D = 0.4
   results in:
                  0.3
      y = ------------------- * u
          4.0 s^2 + 1.6 s + 1
</pre>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -80,78},{ -80, -90}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,88},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90, -80},{82, -80}}, color = {192,192,192}),Polygon(points = {{90, -80},{68, -72},{68, -88},{90, -80}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -72, -68.53},{ -64, -39.5},{ -56, -2.522},{ -48,32.75},{ -40,58.8},{ -32,71.51},{ -24,70.49},{ -16,58.45},{ -8,40.06},{0,20.55},{8,4.459},{16, -5.271},{24, -7.629},{32, -3.428},{40,5.21},{48,15.56},{56,25.03},{64,31.66},{72,34.5},{80,33.61}}, color = {0,0,127}),Text(extent = {{0, -10},{60, -70}}, lineColor = {192,192,192}, textString = "PT2"),Text(extent = {{ -150, -150},{150, -110}}, lineColor = {0,0,0}, textString = "w=%w")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,255}),Text(extent = {{ -60,60},{60,14}}, lineColor = {0,0,0}, textString = "k"),Text(extent = {{ -60,8},{ -32, -20}}, lineColor = {0,0,0}, textString = "s"),Line(points = {{ -100,0},{ -60,0}}, color = {0,0,255}),Line(points = {{60,0},{100,0}}, color = {0,0,255}),Line(points = {{ -50,14},{50,14}}, color = {0,0,0}),Line(points = {{ -54, -20},{ -38, -20}}, color = {0,0,0}),Text(extent = {{ -52, -26},{ -36, -48}}, lineColor = {0,0,0}, textString = "w"),Line(points = {{ -50,2},{ -56, -8},{ -56, -28},{ -52, -46}}, color = {0,0,0}),Line(points = {{ -40,2},{ -34, -10},{ -34, -30},{ -38, -46}}, color = {0,0,0}),Text(extent = {{ -34,8},{ -22, -10}}, lineColor = {0,0,0}, textString = "2"),Text(extent = {{ -34, -6},{6, -36}}, lineColor = {0,0,0}, textString = "+2D"),Text(extent = {{2,8},{30, -20}}, lineColor = {0,0,0}, textString = "s"),Line(points = {{8, -20},{24, -20}}, color = {0,0,0}),Text(extent = {{10, -26},{26, -48}}, lineColor = {0,0,0}, textString = "w"),Line(points = {{12,2},{6, -8},{6, -28},{10, -46}}, color = {0,0,0}),Line(points = {{22,2},{28, -10},{28, -30},{24, -46}}, color = {0,0,0}),Text(extent = {{30,2},{58, -42}}, lineColor = {0,0,0}, textString = "+1")}));

      initial equation 
        if initType == Init.SteadyState then
          der(y) = 0;
          der(yd) = 0;

        elseif initType == Init.InitialState or initType == Init.InitialOutput then
          y = y_start;
          yd = yd_start;


        else

        end if;

      equation 
        der(y) = yd;
        der(yd) = w * (w * (k * u - y) - 2 * D * yd);
      end SecondOrder;
      block PI "Proportional-Integral controller"
        import Modelica.Blocks.Types.Init;
        parameter Real k = 1 "Gain";
        parameter SIunits.Time T(start = 1, min = Modelica.Constants.small) "Time Constant (T>0 required)";
        parameter Modelica.Blocks.Types.Init initType = Modelica.Blocks.Types.Init.NoInit "Type of initialization (1: no init, 2: steady state, 3: initial state, 4: initial output)" annotation(Evaluate = true, Dialog(group = "Initialization"));
        parameter Real x_start = 0 "Initial or guess value of state" annotation(Dialog(group = "Initialization"));
        parameter Real y_start = 0 "Initial value of output" annotation(Dialog(enable = initType == Init.SteadyState or initType == Init.InitialOutput, group = "Initialization"));
        extends Interfaces.SISO;
        output Real x(start = x_start) "State of block";
        annotation(defaultComponentName = "PI", Documentation(info = "
<HTML>
<p>
This blocks defines the transfer function between the input u and
the output y (element-wise) as <i>PI</i> system:
</p>
<pre>
                 1
   y = k * (1 + ---) * u
                T*s
           T*s + 1
     = k * ------- * u
             T*s
</pre>
<p>
If you would like to be able to change easily between different
transfer functions (FirstOrder, SecondOrder, ... ) by changing
parameters, use the general model class <b>TransferFunction</b>
instead and model a PI SISO system with parameters<br>
b = {k*T, k}, a = {T, 0}.
</p>
<pre>
Example:

   parameter: k = 0.3,  T = 0.4

   results in:
               0.4 s + 1
      y = 0.3 ----------- * u
                 0.4 s
</pre>

<p>
It might be difficult to initialize the PI component in steady state
due to the integrator part.
This is discussed in the description of package
<a href=\"Modelica://Modelica.Blocks.Continuous#info\">Continuous</a>.
</p>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -80,78},{ -80, -90}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,88},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90, -80},{82, -80}}, color = {192,192,192}),Polygon(points = {{90, -80},{68, -72},{68, -88},{90, -80}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -80, -20},{60,80}}, color = {0,0,127}),Text(extent = {{0,6},{60, -56}}, lineColor = {192,192,192}, textString = "PI"),Text(extent = {{ -150, -150},{150, -110}}, lineColor = {0,0,0}, textString = "T=%T")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,255}),Text(extent = {{ -68,24},{ -24, -18}}, lineColor = {0,0,0}, textString = "k"),Text(extent = {{ -32,48},{60,0}}, lineColor = {0,0,0}, textString = "T s + 1"),Text(extent = {{ -30, -8},{52, -40}}, lineColor = {0,0,0}, textString = "T s"),Line(points = {{ -24,0},{54,0}}, color = {0,0,0}),Line(points = {{ -100,0},{ -60,0}}, color = {0,0,255}),Line(points = {{62,0},{100,0}}, color = {0,0,255})}));

      initial equation 
        if initType == Init.SteadyState then
          der(x) = 0;

        elseif initType == Init.InitialState then
          x = x_start;


        elseif initType == Init.InitialOutput then
          y = y_start;


        else

        end if;

      equation 
        der(x) = u / T;
        y = k * (x + u);
      end PI;
      block PID "PID-controller in additive description form"
        import Modelica.Blocks.Types.InitPID;
        extends Interfaces.SISO;
        parameter Real k = 1 "Gain";
        parameter SIunits.Time Ti(min = Modelica.Constants.small, start = 0.5) "Time Constant of Integrator";
        parameter SIunits.Time Td(min = 0, start = 0.1) "Time Constant of Derivative block";
        parameter Real Nd(min = Modelica.Constants.small) = 10 "The higher Nd, the more ideal the derivative block";
        parameter Modelica.Blocks.Types.InitPID initType = Modelica.Blocks.Types.InitPID.DoNotUse_InitialIntegratorState "Type of initialization (1: no init, 2: steady state, 3: initial state, 4: initial output)" annotation(Evaluate = true, Dialog(group = "Initialization"));
        parameter Real xi_start = 0 "Initial or guess value value for integrator output (= integrator state)" annotation(Dialog(group = "Initialization"));
        parameter Real xd_start = 0 "Initial or guess value for state of derivative block" annotation(Dialog(group = "Initialization"));
        parameter Real y_start = 0 "Initial value of output" annotation(Dialog(enable = initType == Init.InitialOutput, group = "Initialization"));
        annotation(defaultComponentName = "PID", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -80,78},{ -80, -90}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90, -80},{82, -80}}, color = {192,192,192}),Polygon(points = {{90, -80},{68, -72},{68, -88},{90, -80}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -80,50},{ -80, -20},{60,80}}, color = {0,0,127}),Text(extent = {{ -20, -20},{80, -60}}, lineColor = {192,192,192}, textString = "PID"),Text(extent = {{ -150, -150},{150, -110}}, lineColor = {0,0,0}, textString = "Ti=%Ti")}), Documentation(info = "<HTML>
<p>
This is the text-book version of a PID-controller.
For a more practically useful PID-controller, use
block LimPID.
</p>

<p>
The PID block can be initialized in different
ways controlled by parameter <b>initType</b>. The possible
values of initType are defined in
<a href=\"Modelica://Modelica.Blocks.Types.InitPID\">Modelica.Blocks.Types.InitPID</a>.
This type is identical to
<a href=\"Modelica://Modelica.Blocks.Types.Init\">Types.Init</a>,
with the only exception that the additional option
<b>DoNotUse_InitialIntegratorState</b> is added for
backward compatibility reasons (= integrator is initialized with
InitialState whereas differential part is initialized with
NoInit which was the initialization in version 2.2 of the Modelica
standard library).
</p>

<p>
Based on the setting of initType, the integrator (I) and derivative (D)
blocks inside the PID controller are initialized according to the following table:
</p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>initType</b></td>
      <td valign=\"top\"><b>I.initType</b></td>
      <td valign=\"top\"><b>D.initType</b></td></tr>

  <tr><td valign=\"top\"><b>NoInit</b></td>
      <td valign=\"top\">NoInit</td>
      <td valign=\"top\">NoInit</td></tr>

  <tr><td valign=\"top\"><b>SteadyState</b></td>
      <td valign=\"top\">SteadyState</td>
      <td valign=\"top\">SteadyState</td></tr>

  <tr><td valign=\"top\"><b>InitialState</b></td>
      <td valign=\"top\">InitialState</td>
      <td valign=\"top\">InitialState</td></tr>

  <tr><td valign=\"top\"><b>InitialOutput</b><br>
          and initial equation: y = y_start</td>
      <td valign=\"top\">NoInit</td>
      <td valign=\"top\">SteadyState</td></tr>

  <tr><td valign=\"top\"><b>DoNotUse_InitialIntegratorState</b></td>
      <td valign=\"top\">InitialState</td>
      <td valign=\"top\">NoInit</td></tr>
</table>

<p>
In many cases, the most useful initial condition is
<b>SteadyState</b> because initial transients are then no longer
present. If initType = InitPID.SteadyState, then in some
cases difficulties might occur. The reason is the
equation of the integrator:
</p>

<pre>
   <b>der</b>(y) = k*u;
</pre>

<p>
The steady state equation \"der(x)=0\" leads to the condition that the input u to the
integrator is zero. If the input u is already (directly or indirectly) defined
by another initial condition, then the initialization problem is <b>singular</b>
(has none or infinitely many solutions). This situation occurs often
for mechanical systems, where, e.g., u = desiredSpeed - measuredSpeed and
since speed is both a state and a derivative, it is natural to
initialize it with zero. As sketched this is, however, not possible.
The solution is to not initialize u or the variable that is used
to compute u by an algebraic equation.
</p>


</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics));
        Blocks.Math.Gain P(k = 1) "Proportional part of PID controller" annotation(Placement(transformation(extent = {{ -60,60},{ -20,100}}, rotation = 0)));
        Blocks.Continuous.Integrator I(k = 1 / Ti, y_start = xi_start, initType = if initType == InitPID.SteadyState then InitPID.SteadyState else if initType == InitPID.InitialState or initType == InitPID.DoNotUse_InitialIntegratorState then InitPID.InitialState else InitPID.NoInit) "Integral part of PID controller" annotation(Placement(transformation(extent = {{ -60, -20},{ -20,20}}, rotation = 0)));
        Blocks.Continuous.Derivative D(k = Td, T = max([Td / Nd,100 * Modelica.Constants.eps]), x_start = xd_start, initType = if initType == InitPID.SteadyState or initType == InitPID.InitialOutput then InitPID.SteadyState else if initType == InitPID.InitialState then InitPID.InitialState else InitPID.NoInit) "Derivative part of PID controller" annotation(Placement(transformation(extent = {{ -60, -100},{ -20, -60}}, rotation = 0)));
        Blocks.Math.Gain Gain(k = k) "Gain of PID controller" annotation(Placement(transformation(extent = {{60, -10},{80,10}}, rotation = 0)));
        Blocks.Math.Add3 Add annotation(Placement(transformation(extent = {{20, -10},{40,10}}, rotation = 0)));

      initial equation 
if initType == InitPID.InitialOutput then
                  y = y_start;
        end if;

      equation 
        connect(u,P.u) annotation(Line(points = {{ -120,0},{ -80,0},{ -80,80},{ -64,80}}, color = {0,0,127}));
        connect(u,I.u) annotation(Line(points = {{ -120,0},{ -64,0}}, color = {0,0,127}));
        connect(u,D.u) annotation(Line(points = {{ -120,0},{ -80,0},{ -80, -80},{ -64, -80}}, color = {0,0,127}));
        connect(P.y,Add.u1) annotation(Line(points = {{ -18,80},{0,80},{0,8},{18,8}}, color = {0,0,127}));
        connect(I.y,Add.u2) annotation(Line(points = {{ -18,0},{18,0}}, color = {0,0,127}));
        connect(D.y,Add.u3) annotation(Line(points = {{ -18, -80},{0, -80},{0, -8},{18, -8}}, color = {0,0,127}));
        connect(Add.y,Gain.u) annotation(Line(points = {{41,0},{58,0}}, color = {0,0,127}));
        connect(Gain.y,y) annotation(Line(points = {{81,0},{110,0}}, color = {0,0,127}));
      end PID;
      block LimPID "P, PI, PD, and PID controller with limited output, anti-windup compensation and setpoint weighting"
        import Modelica.Blocks.Types.InitPID;
        import Modelica.Blocks.Types.SimpleController;
        extends Interfaces.SVcontrol;
        output Real controlError = u_s - u_m "Control error (set point - measurement)";
        parameter Modelica.Blocks.Types.SimpleController controllerType = Modelica.Blocks.Types.SimpleController.PID "Type of controller";
        parameter Real k(min = 0) = 1 "Gain of controller";
        parameter SIunits.Time Ti(min = Modelica.Constants.small, start = 0.5) "Time constant of Integrator block" annotation(Dialog(enable = controllerType == SimpleController.PI or controllerType == SimpleController.PID));
        parameter SIunits.Time Td(min = 0, start = 0.1) "Time constant of Derivative block" annotation(Dialog(enable = controllerType == SimpleController.PD or controllerType == SimpleController.PID));
        parameter Real yMax(start = 1) "Upper limit of output";
        parameter Real yMin =  -yMax "Lower limit of output";
        parameter Real wp(min = 0) = 1 "Set-point weight for Proportional block (0..1)";
        parameter Real wd(min = 0) = 0 "Set-point weight for Derivative block (0..1)" annotation(Dialog(enable = controllerType == SimpleController.PD or controllerType == SimpleController.PID));
        parameter Real Ni(min = 100 * Modelica.Constants.eps) = 0.9 "Ni*Ti is time constant of anti-windup compensation" annotation(Dialog(enable = controllerType == SimpleController.PI or controllerType == SimpleController.PID));
        parameter Real Nd(min = 100 * Modelica.Constants.eps) = 10 "The higher Nd, the more ideal the derivative block" annotation(Dialog(enable = controllerType == SimpleController.PD or controllerType == SimpleController.PID));
        parameter Modelica.Blocks.Types.InitPID initType = Modelica.Blocks.Types.InitPID.DoNotUse_InitialIntegratorState "Type of initialization (1: no init, 2: steady state, 3: initial state, 4: initial output)" annotation(Evaluate = true, Dialog(group = "Initialization"));
        parameter Boolean limitsAtInit = true "= false, if limits are ignored during initializiation" annotation(Evaluate = true, Dialog(group = "Initialization", enable = controllerType == SimpleController.PI or controllerType == SimpleController.PID));
        parameter Real xi_start = 0 "Initial or guess value value for integrator output (= integrator state)" annotation(Dialog(group = "Initialization", enable = controllerType == SimpleController.PI or controllerType == SimpleController.PID));
        parameter Real xd_start = 0 "Initial or guess value for state of derivative block" annotation(Dialog(group = "Initialization", enable = controllerType == SimpleController.PD or controllerType == SimpleController.PID));
        parameter Real y_start = 0 "Initial value of output" annotation(Dialog(enable = initType == InitPID.InitialOutput, group = "Initialization"));
        annotation(defaultComponentName = "PID", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80,78},{ -80, -90}}, color = {192,192,192}),Polygon(points = {{ -80,90},{ -88,68},{ -72,68},{ -80,90}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90, -80},{82, -80}}, color = {192,192,192}),Polygon(points = {{90, -80},{68, -72},{68, -88},{90, -80}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -80,50},{ -80, -20},{30,60},{80,60}}, color = {0,0,127}),Text(extent = {{ -20, -20},{80, -60}}, lineColor = {192,192,192}, textString = "PID")}), Documentation(info = "<HTML>
<p>
Via parameter <b>controllerType</b> either <b>P</b>, <b>PI</b>, <b>PD</b>,
or <b>PID</b> can be selected. If, e.g., PI is selected, all components belonging to the
D-part are removed from the block (via conditional declarations).
The example model
<a href=\"Modelica://Modelica.Blocks.Examples.PID_Controller\">Modelica.Blocks.Examples.PID_Controller</a>
demonstrates the usage of this controller.
Several practical aspects of PID controller design are incorporated
according to chapter 3 of the book:
</p>

<dl>
<dt>&Aring;str&ouml;m K.J., and H&auml;gglund T.:</dt>
<dd> <b>PID Controllers: Theory, Design, and Tuning</b>.
     Instrument Society of America, 2nd edition, 1995.
</dd>
</dl>

<p>
Besides the additive <b>proportional, integral</b> and <b>derivative</b>
part of this controller, the following features are present:
</p>
<ul>
<li> The output of this controller is limited. If the controller is
     in its limits, anti-windup compensation is activated to drive
     the integrator state to zero. </li>
<li> The high-frequency gain of the derivative part is limited
     to avoid excessive amplification of measurement noise.</li>
<li> Setpoint weighting is present, which allows to weight
     the setpoint in the proportional and the derivative part
     independantly from the measurement. The controller will respond
     to load disturbances and measurement noise independantly of this setting
     (parameters wp, wd). However, setpoint changes will depend on this
     setting. For example, it is useful to set the setpoint weight wd
     for the derivative part to zero, if steps may occur in the
     setpoint signal.</li>
</ul>

<p>
The parameters of the controller can be manually adjusted by performing
simulations of the closed loop system (= controller + plant connected
together) and using the following strategy:
</p>

<ol>
<li> Set very large limits, e.g., yMax = Modelica.Constants.inf</li>
<li> Select a <b>P</b>-controller and manually enlarge parameter <b>k</b>
     (the total gain of the controller) until the closed-loop response
     cannot be improved any more.</li>
<li> Select a <b>PI</b>-controller and manually adjust parameters
     <b>k</b> and <b>Ti</b> (the time constant of the integrator).
     The first value of Ti can be selected, such that it is in the
     order of the time constant of the oscillations occuring with
     the P-controller. If, e.g., vibrations in the order of T=10 ms
     occur in the previous step, start with Ti=0.01 s.</li>
<li> If you want to make the reaction of the control loop faster
     (but probably less robust against disturbances and measurement noise)
     select a <b>PID</b>-Controller and manually adjust parameters
     <b>k</b>, <b>Ti</b>, <b>Td</b> (time constant of derivative block).</li>
<li> Set the limits yMax and yMin according to your specification.</li>
<li> Perform simulations such that the output of the PID controller
     goes in its limits. Tune <b>Ni</b> (Ni*Ti is the time constant of
     the anti-windup compensation) such that the input to the limiter
     block (= limiter.u) goes quickly enough back to its limits.
     If Ni is decreased, this happens faster. If Ni=infinity, the
     anti-windup compensation is switched off and the controller works bad.</li>
</ol>

<p>
<b>Initialization</b>
</p>

<p>
This block can be initialized in different
ways controlled by parameter <b>initType</b>. The possible
values of initType are defined in
<a href=\"Modelica://Modelica.Blocks.Types.InitPID\">Modelica.Blocks.Types.InitPID</a>.
This type is identical to
<a href=\"Modelica://Modelica.Blocks.Types.Init\">Types.Init</a>,
with the only exception that the additional option
<b>DoNotUse_InitialIntegratorState</b> is added for
backward compatibility reasons (= integrator is initialized with
InitialState whereas differential part is initialized with
NoInit which was the initialization in version 2.2 of the Modelica
standard library).
</p>

<p>
Based on the setting of initType, the integrator (I) and derivative (D)
blocks inside the PID controller are initialized according to the following table:
</p>

<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>initType</b></td>
      <td valign=\"top\"><b>I.initType</b></td>
      <td valign=\"top\"><b>D.initType</b></td></tr>

  <tr><td valign=\"top\"><b>NoInit</b></td>
      <td valign=\"top\">NoInit</td>
      <td valign=\"top\">NoInit</td></tr>

  <tr><td valign=\"top\"><b>SteadyState</b></td>
      <td valign=\"top\">SteadyState</td>
      <td valign=\"top\">SteadyState</td></tr>

  <tr><td valign=\"top\"><b>InitialState</b></td>
      <td valign=\"top\">InitialState</td>
      <td valign=\"top\">InitialState</td></tr>

  <tr><td valign=\"top\"><b>InitialOutput</b><br>
          and initial equation: y = y_start</td>
      <td valign=\"top\">NoInit</td>
      <td valign=\"top\">SteadyState</td></tr>

  <tr><td valign=\"top\"><b>DoNotUse_InitialIntegratorState</b></td>
      <td valign=\"top\">InitialState</td>
      <td valign=\"top\">NoInit</td></tr>
</table>

<p>
In many cases, the most useful initial condition is
<b>SteadyState</b> because initial transients are then no longer
present. If initType = InitPID.SteadyState, then in some
cases difficulties might occur. The reason is the
equation of the integrator:
</p>

<pre>
   <b>der</b>(y) = k*u;
</pre>

<p>
The steady state equation \"der(x)=0\" leads to the condition that the input u to the
integrator is zero. If the input u is already (directly or indirectly) defined
by another initial condition, then the initialization problem is <b>singular</b>
(has none or infinitely many solutions). This situation occurs often
for mechanical systems, where, e.g., u = desiredSpeed - measuredSpeed and
since speed is both a state and a derivative, it is natural to
initialize it with zero. As sketched this is, however, not possible.
The solution is to not initialize u_m or the variable that is used
to compute u_m by an algebraic equation.
</p>

<p>
If parameter <b>limitAtInit</b> = <b>false</b>, the limits at the
output of this controller block are removed from the initialization problem which
leads to a much simpler equation system. After initialization has been
performed, it is checked via an assert whether the output is in the
defined limits. For backward compatibility reasons
<b>limitAtInit</b> = <b>true</b>. In most cases it is best
to use <b>limitAtInit</b> = <b>false</b>.
</p>
</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics));
        Blocks.Math.Add addP(k1 = wp, k2 =  -1) annotation(Placement(transformation(extent = {{ -80,40},{ -60,60}}, rotation = 0)));
        Blocks.Math.Add addD(k1 = wd, k2 =  -1) if with_D annotation(Placement(transformation(extent = {{ -80, -10},{ -60,10}}, rotation = 0)));
        Blocks.Math.Gain P(k = 1) annotation(Placement(transformation(extent = {{ -40,40},{ -20,60}}, rotation = 0)));
        Blocks.Continuous.Integrator I(k = 1 / Ti, y_start = xi_start, initType = if initType == InitPID.SteadyState then InitPID.SteadyState else if initType == InitPID.InitialState or initType == InitPID.DoNotUse_InitialIntegratorState then InitPID.InitialState else InitPID.NoInit) if with_I annotation(Placement(transformation(extent = {{ -40, -60},{ -20, -40}}, rotation = 0)));
        Blocks.Continuous.Derivative D(k = Td, T = max([Td / Nd,1e-14]), x_start = xd_start, initType = if initType == InitPID.SteadyState or initType == InitPID.InitialOutput then InitPID.SteadyState else if initType == InitPID.InitialState then InitPID.InitialState else InitPID.NoInit) if with_D annotation(Placement(transformation(extent = {{ -40, -10},{ -20,10}}, rotation = 0)));
        Blocks.Math.Gain gainPID(k = k) annotation(Placement(transformation(extent = {{30, -10},{50,10}}, rotation = 0)));
        Blocks.Math.Add3 addPID annotation(Placement(transformation(extent = {{0, -10},{20,10}}, rotation = 0)));
        Blocks.Math.Add3 addI(k2 =  -1) if with_I annotation(Placement(transformation(extent = {{ -80, -60},{ -60, -40}}, rotation = 0)));
        Blocks.Math.Add addSat(k1 =  +1, k2 =  -1) if with_I annotation(Placement(transformation(origin = {80, -50}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        Blocks.Math.Gain gainTrack(k = 1 / (k * Ni)) if with_I annotation(Placement(transformation(extent = {{40, -80},{20, -60}}, rotation = 0)));
        Blocks.Nonlinear.Limiter limiter(uMax = yMax, uMin = yMin, limitsAtInit = limitsAtInit) annotation(Placement(transformation(extent = {{70, -10},{90,10}}, rotation = 0)));
      protected 
        parameter Boolean with_I = controllerType == SimpleController.PI or controllerType == SimpleController.PID annotation(Evaluate = true, HideResult = true);
        parameter Boolean with_D = controllerType == SimpleController.PD or controllerType == SimpleController.PID annotation(Evaluate = true, HideResult = true);
      public 
        Sources.Constant Dzero(k = 0) if not with_D annotation(Placement(transformation(extent = {{ -30,20},{ -20,30}}, rotation = 0)));
        Sources.Constant Izero(k = 0) if not with_I annotation(Placement(transformation(extent = {{10, -55},{0, -45}}, rotation = 0)));

      initial equation 
if initType == InitPID.InitialOutput then
                  y = y_start;
        end if;

      equation 
        assert(yMax >= yMin, "LimPID: Limits must be consistent. However, yMax (=" + String(yMax) + ") < yMin (=" + String(yMin) + ")");
if initType == InitPID.InitialOutput and (y_start < yMin or y_start > yMax) then
                  Modelica.Utilities.Streams.error("LimPID: Start value y_start (=" + String(y_start) + ") is outside of the limits of yMin (=" + String(yMin) + ") and yMax (=" + String(yMax) + ")");
        end if;
        assert(limitsAtInit or not limitsAtInit and y >= yMin and y <= yMax, "LimPID: During initialization the limits have been switched off.\n" + "After initialization, the output y (=" + String(y) + ") is outside of the limits of yMin (=" + String(yMin) + ") and yMax (=" + String(yMax) + ")");
        connect(u_s,addP.u1) annotation(Line(points = {{ -120,0},{ -96,0},{ -96,56},{ -82,56}}, color = {0,0,127}));
        connect(u_s,addD.u1) annotation(Line(points = {{ -120,0},{ -96,0},{ -96,6},{ -82,6}}, color = {0,0,127}));
        connect(u_s,addI.u1) annotation(Line(points = {{ -120,0},{ -96,0},{ -96, -42},{ -82, -42}}, color = {0,0,127}));
        connect(addP.y,P.u) annotation(Line(points = {{ -59,50},{ -42,50}}, color = {0,0,127}));
        connect(addD.y,D.u) annotation(Line(points = {{ -59,0},{ -42,0}}, color = {0,0,127}));
        connect(addI.y,I.u) annotation(Line(points = {{ -59, -50},{ -42, -50}}, color = {0,0,127}));
        connect(P.y,addPID.u1) annotation(Line(points = {{ -19,50},{ -10,50},{ -10,8},{ -2,8}}, color = {0,0,127}));
        connect(D.y,addPID.u2) annotation(Line(points = {{ -19,0},{ -2,0}}, color = {0,0,127}));
        connect(I.y,addPID.u3) annotation(Line(points = {{ -19, -50},{ -10, -50},{ -10, -8},{ -2, -8}}, color = {0,0,127}));
        connect(addPID.y,gainPID.u) annotation(Line(points = {{21,0},{28,0}}, color = {0,0,127}));
        connect(gainPID.y,addSat.u2) annotation(Line(points = {{51,0},{60,0},{60, -20},{74, -20},{74, -38}}, color = {0,0,127}));
        connect(gainPID.y,limiter.u) annotation(Line(points = {{51,0},{68,0}}, color = {0,0,127}));
        connect(limiter.y,addSat.u1) annotation(Line(points = {{91,0},{94,0},{94, -20},{86, -20},{86, -38}}, color = {0,0,127}));
        connect(limiter.y,y) annotation(Line(points = {{91,0},{110,0}}, color = {0,0,127}));
        connect(addSat.y,gainTrack.u) annotation(Line(points = {{80, -61},{80, -70},{42, -70}}, color = {0,0,127}));
        connect(gainTrack.y,addI.u3) annotation(Line(points = {{19, -70},{ -88, -70},{ -88, -58},{ -82, -58}}, color = {0,0,127}));
        connect(u_m,addP.u2) annotation(Line(points = {{0, -120},{0, -92},{ -92, -92},{ -92,44},{ -82,44}}, color = {0,0,127}, thickness = 0.5));
        connect(u_m,addD.u2) annotation(Line(points = {{0, -120},{0, -92},{ -92, -92},{ -92, -6},{ -82, -6}}, color = {0,0,127}, thickness = 0.5));
        connect(u_m,addI.u2) annotation(Line(points = {{0, -120},{0, -92},{ -92, -92},{ -92, -50},{ -82, -50}}, color = {0,0,127}, thickness = 0.5));
        connect(Dzero.y,addPID.u2) annotation(Line(points = {{ -19.5,25},{ -14,25},{ -14,0},{ -2,0}}, color = {0,0,127}));
        connect(Izero.y,addPID.u3) annotation(Line(points = {{ -0.5, -50},{ -10, -50},{ -10, -8},{ -2, -8}}, color = {0,0,127}));
      end LimPID;
      block TransferFunction "Linear transfer function"
        import Modelica.Blocks.Types.Init;
        extends Interfaces.SISO;
        parameter Real b[:] = {1} "Numerator coefficients of transfer function" annotation(Dialog(group = "y = (2*s+3)/(4*s^2+5*s+6)*u is defined as b={2,3}, a={4,5,6}"));
        parameter Real a[:] "Denominator coefficients of transfer function" annotation(Dialog(group = "y = (2*s+3)/(4*s^2+5*s+6)*u is defined as b={2,3}, a={4,5,6}"));
        parameter Modelica.Blocks.Types.Init initType = Modelica.Blocks.Types.Init.NoInit "Type of initialization (1: no init, 2: steady state, 3: initial state, 4: initial output)" annotation(Evaluate = true, Dialog(group = "Initialization"));
        parameter Real x_start[size(a, 1) - 1] = zeros(nx) "Initial or guess values of states" annotation(Dialog(group = "Initialization"));
        parameter Real y_start = 0 "Initial value of output (derivatives of y are zero upto nx-1-th derivative)" annotation(Dialog(enable = initType == Init.InitialOutput, group = "Initialization"));
        output Real x[size(a, 1) - 1](start = x_start) "State of transfer function from controller canonical form";
      protected 
        parameter Integer na = size(a, 1) "Size of Denominator of transfer function.";
        parameter Integer nb = size(b, 1) "Size of Numerator of transfer function.";
        parameter Integer nx = size(a, 1) - 1;
        parameter Real bb[:] = vector([zeros(max(0, na - nb), 1);b]);
        parameter Real d = bb[1] / a[1];
        parameter Real a_end = if a[end] > 100 * Modelica.Constants.eps * sqrt(a * a) then a[end] else 1.0;
        Real x_scaled[size(x, 1)] "Scaled vector x";
        annotation(Documentation(info = "<html>
<p>
This block defines the transfer function between the input
u and the output y
as (nb = dimension of b, na = dimension of a):
</p>
<pre>
           b[1]*s^[nb-1] + b[2]*s^[nb-2] + ... + b[nb]
   y(s) = --------------------------------------------- * u(s)
           a[1]*s^[na-1] + a[2]*s^[na-2] + ... + a[na]
</pre>
<p>
State variables <b>x</b> are defined according to <b>controller canonical</b>
form. Internally, vector <b>x</b> is scaled to improve the numerics (the states in versions before version 3.0 of the Modelica Standard Library have been not scaled). This scaling is
not visible from the outside of this block because the non-scaled vector <b>x</b>
is provided as output signal and the start value is with respect to the non-scaled
vector <b>x</b>.
Initial values of the states <b>x</b> can be set via parameter <b>x_start</b>.
</p>

<p>
Example:
</p>
<pre>
     TransferFunction g(b = {2,4}, a = {1,3});
</pre>
<p>
results in the following transfer function:
</p>
<pre>
        2*s + 4
   y = --------- * u
         s + 3
</pre>
</html>"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{ -80,0},{80,0}}, color = {0,0,127}),Text(extent = {{ -90,10},{90,90}}, textString = "b(s)", lineColor = {0,0,127}),Text(extent = {{ -90, -10},{90, -90}}, textString = "a(s)", lineColor = {0,0,127})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Line(points = {{40,0},{ -40,0}}, color = {0,0,0}),Text(extent = {{ -55,55},{55,5}}, lineColor = {0,0,0}, textString = "b(s)"),Text(extent = {{ -55, -5},{55, -55}}, lineColor = {0,0,0}, textString = "a(s)"),Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,255}),Line(points = {{ -100,0},{ -60,0}}, color = {0,0,255}),Line(points = {{60,0},{100,0}}, color = {0,0,255})}));

      initial equation 
        if initType == Init.SteadyState then
          der(x_scaled) = zeros(nx);

        elseif initType == Init.InitialState then
          x_scaled = x_start * a_end;


        elseif initType == Init.InitialOutput then
          y = y_start;
          der(x_scaled[2:nx]) = zeros(nx - 1);


        else

        end if;

      equation 
        assert(size(b, 1) <= size(a, 1), "Transfer function is not proper");
        if nx == 0 then
          y = d * u;

        else
          der(x_scaled[1]) = ( -a[2:na] * x_scaled + a_end * u) / a[1];
          der(x_scaled[2:nx]) = x_scaled[1:nx - 1];
          y = ((bb[2:na] - d * a[2:na]) * x_scaled) / a_end + d * u;
          x = x_scaled / a_end;

        end if;
      end TransferFunction;
      block StateSpace "Linear state space system"
        import Modelica.Blocks.Types.Init;
        parameter Real A[:,size(A, 1)] "Matrix A of state space model (e.g. A=[1, 0; 0, 1])";
        parameter Real B[size(A, 1),:] "Matrix B of state space model (e.g. B=[1; 1])";
        parameter Real C[:,size(A, 1)] "Matrix C of state space model (e.g. C=[1, 1])";
        parameter Real D[size(C, 1),size(B, 2)] = zeros(size(C, 1), size(B, 2)) "Matrix D of state space model";
        parameter Modelica.Blocks.Types.Init initType = Modelica.Blocks.Types.Init.NoInit "Type of initialization (1: no init, 2: steady state, 3: initial state, 4: initial output)" annotation(Evaluate = true, Dialog(group = "Initialization"));
        parameter Real x_start[nx] = zeros(nx) "Initial or guess values of states" annotation(Dialog(group = "Initialization"));
        parameter Real y_start[ny] = zeros(ny) "Initial values of outputs (remaining states are in steady state if possible)" annotation(Dialog(enable = initType == Init.InitialOutput, group = "Initialization"));
        extends Interfaces.MIMO(final nin = size(B, 2), final nout = size(C, 1));
        output Real x[size(A, 1)](start = x_start) "State vector";
        annotation(Documentation(info = "<HTML>
<p>
The State Space block defines the relation
between the input u and the output
y in state space form:
</p>
<pre>

    der(x) = A * x + B * u
        y  = C * x + D * u
</pre>
<p>
The input is a vector of length nu, the output is a vector
of length ny and nx is the number of states. Accordingly
</p>
<pre>
        A has the dimension: A(nx,nx),
        B has the dimension: B(nx,nu),
        C has the dimension: C(ny,nx),
        D has the dimension: D(ny,nu)
</pre>
<p>
Example:
</p>
<pre>
     parameter: A = [0.12, 2;3, 1.5]
     parameter: B = [2, 7;3, 1]
     parameter: C = [0.1, 2]
     parameter: D = zeros(ny,nu)
results in the following equations:
  [der(x[1])]   [0.12  2.00] [x[1]]   [2.0  7.0] [u[1]]
  [         ] = [          ]*[    ] + [        ]*[    ]
  [der(x[2])]   [3.00  1.50] [x[2]]   [0.1  2.0] [u[2]]
                             [x[1]]            [u[1]]
       y[1]   = [0.1  2.0] * [    ] + [0  0] * [    ]
                             [x[2]]            [u[2]]
</pre>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Text(extent = {{ -90,10},{ -10,90}}, textString = "A", lineColor = {0,0,127}),Text(extent = {{10,10},{90,90}}, textString = "B", lineColor = {0,0,127}),Text(extent = {{ -90, -10},{ -10, -90}}, textString = "C", lineColor = {0,0,127}),Text(extent = {{10, -10},{90, -90}}, textString = "D", lineColor = {0,0,127}),Line(points = {{0, -90},{0,90}}, color = {192,192,192}),Line(points = {{ -90,0},{90,0}}, color = {192,192,192})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,255}),Text(extent = {{ -60,40},{60,0}}, lineColor = {0,0,0}, textString = "sx=Ax+Bu"),Text(extent = {{ -60,0},{60, -40}}, lineColor = {0,0,0}, textString = " y=Cx+Du"),Line(points = {{ -100,0},{ -60,0}}, color = {0,0,255}),Line(points = {{60,0},{100,0}}, color = {0,0,255})}));
      protected 
        parameter Integer nx = size(A, 1) "number of states";
        parameter Integer ny = size(C, 1) "number of outputs";

      initial equation 
        if initType == Init.SteadyState then
          der(x) = zeros(nx);

        elseif initType == Init.InitialState then
          x = x_start;


        elseif initType == Init.InitialOutput then
          x = Modelica.Math.Matrices.equalityLeastSquares(A,  -B * u, C, y_start - D * u);


        else

        end if;

      equation 
        der(x) = A * x + B * u;
        y = C * x + D * u;
      end StateSpace;
      block Der "Derivative of input (= analytic differentations)"
        extends Interfaces.SISO;
        annotation(defaultComponentName = "der1", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -80,76},{80, -82}}, textString = "der()", lineColor = {0,0,127})}), Documentation(info = "<HTML>
<p>
Defines that the output y is the <i>derivative</i>
of the input u. Note, that Modelica.Blocks.Continuous.Derivative
computes the derivative in an approximate sense, where as this block computes
the derivative exactly. This requires that the input u is differentiated
by the Modelica translator, if this derivative is not yet present in
the model.
</p>
</HTML>"));

      equation 
        y = der(u);
      end Der;
      block LowpassButterworth "Output the input signal filtered with a low pass Butterworth filter of any order"
        import Modelica.Math.*;
        import Modelica.Blocks.Types.Init;
        extends Modelica.Blocks.Interfaces.SISO;
        parameter Integer n(min = 1) = 2 "Order of filter";
        parameter SI.Frequency f(start = 1) "Cut-off frequency";
        parameter Modelica.Blocks.Types.Init initType = Modelica.Blocks.Types.Init.NoInit "Type of initialization (1: no init, 2: steady state, 3: initial state, 4: initial output)" annotation(Evaluate = true, Dialog(group = "Initialization"));
        parameter Real x1_start[m] = zeros(m) "Initial or guess values of states 1 (der(x1)=x2))" annotation(Dialog(group = "Initialization"));
        parameter Real x2_start[m] = zeros(m) "Initial or guess values of states 2" annotation(Dialog(group = "Initialization"));
        parameter Real xr_start = 0.0 "Initial or guess value of real pole for uneven order otherwise dummy" annotation(Dialog(group = "Initialization"));
        parameter Real y_start = 0.0 "Initial value of output (states are initialized in steady state if possible)" annotation(Dialog(enable = initType == Init.InitialOutput, group = "Initialization"));
        output Real x1[m](start = x1_start) "states 1 of second order filters (der(x1) = x2)";
        output Real x2[m](start = x2_start) "states 2 of second order filters";
        output Real xr(start = xr_start) "state of real pole for uneven order otherwise dummy";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -80,78},{ -80, -90}}, color = {192,192,192}),Polygon(points = {{ -79.5584,91.817},{ -87.5584,69.817},{ -71.5584,69.817},{ -79.5584,89.817},{ -79.5584,91.817}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -80, -80},{ -72, -68.53},{ -64, -39.5},{ -56, -2.522},{ -48,32.75},{ -40,58.8},{ -32,71.51},{ -24,70.49},{ -16,58.45},{ -8,40.06},{0,20.55},{8,4.459},{16, -5.271},{24, -7.629},{32, -3.428},{40,5.21},{48,15.56},{56,25.03},{64,31.66},{72,34.5},{80,33.61}}, color = {0,0,127}),Line(points = {{ -90.9779, -80.7697},{81.0221, -80.7697}}, color = {192,192,192}),Polygon(points = {{91.3375, -79.8233},{69.3375, -71.8233},{69.3375, -87.8233},{91.3375, -79.8233}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -45.1735, -11.47},{92, -68}}, lineColor = {192,192,192}, textString = "LowpassButterworthFilter"),Text(extent = {{8, -106},{8, -146}}, lineColor = {0,0,0}, textString = "f=%f"),Text(extent = {{ -2,94},{94,48}}, lineColor = {192,192,192}, textString = "%n")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{40,0},{ -40,0}}, color = {0,0,0}),Text(extent = {{ -55,55},{55,5}}, lineColor = {0,0,0}, textString = "1"),Text(extent = {{ -55, -5},{55, -55}}, lineColor = {0,0,0}, textString = "a(s)"),Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,255}),Line(points = {{ -100,0},{ -60,0}}, color = {0,0,255}),Line(points = {{60,0},{100,0}}, color = {0,0,255})}), Documentation(info = "<html>
<p>
This block defines the transfer function between the input u
and the output y as an n-th order low pass filter with <i>Butterworth</i>
characteristics and cut-off frequency f. It is implemented as
a series of second order filters and a first order filter.
Butterworth filters have the feature that the amplitude at the
cut-off frequency f is 1/sqrt(2) (= 3 dB), i.e., they are
always \"normalized\". Step responses of the Butterworth filter of
different orders are shown in the next figure:
</p>

<p>
<img src=\"../Images/Blocks/Butterworth.png\">
</p>

<p>
If transients at the simulation start shall be avoided, the filter
should be initialized in steady state (e.g., using option
initType=Modelica.Blocks.Types.Init.SteadyState).
</p>


</html>
"));
      protected 
        constant Real pi = Modelica.Constants.pi;
        parameter Integer m = integer(n / 2);
        parameter Boolean evenOrder = 2 * m == n;
        parameter Real w = 2 * pi * f;
        Real z[m + 1];
        Real polereal[m];
        Real poleimag[m];
        Real realpol;
        Real k2[m];
        Real D[m];
        Real w0[m];
        Real k1;
        Real T;

      initial equation 
        if initType == Init.SteadyState then
          der(x1) = zeros(m);
          der(x2) = zeros(m);
if not evenOrder then
                      der(xr) = 0.0;
          end if;

        elseif initType == Init.InitialState then
          x1 = x1_start;
          x2 = x2_start;
if not evenOrder then
                      xr = xr_start;
          end if;


        elseif initType == Init.InitialOutput then
          y = y_start;
          der(x1) = zeros(m);
          if evenOrder then
if m > 1 then
                          der(x2[1:m - 1]) = zeros(m - 1);
            end if;

          else
            der(x1) = zeros(m);

          end if;


        else

        end if;

      equation 
        k2 = ones(m);
        k1 = 1;
        z[1] = u;
        for i in 1:m loop
        polereal[i] = cos(pi / 2 + pi / n * (i - 0.5));
        poleimag[i] = sin(pi / 2 + pi / n * (i - 0.5));
        w0[i] = (polereal[i] ^ 2 + poleimag[i] ^ 2) * w;
        D[i] =  -polereal[i] / w0[i] * w;

        end for;
        realpol = 1 * w;
        T = 1 / realpol;
        for i in 1:m loop
        der(x1[i]) = x2[i];
        der(x2[i]) = k2[i] * w0[i] ^ 2 * z[i] - 2 * D[i] * w0[i] * x2[i] - w0[i] ^ 2 * x1[i];
        z[i + 1] = x1[i];

        end for;
        if evenOrder then
          xr = 0;
          y = z[m + 1];

        else
          der(xr) = (k1 * z[m + 1] - xr) / T;
          y = xr;

        end if;
      end LowpassButterworth;
      block CriticalDamping "Output the input signal filtered with an n-th order filter with critical damping"
        import Modelica.Blocks.Types.Init;
        extends Modelica.Blocks.Interfaces.SISO;
        parameter Integer n = 2 "Order of filter";
        parameter Modelica.SIunits.Frequency f(start = 1) "Cut-off frequency";
        parameter Boolean normalized = true "= true, if amplitude at f_cut is 3 dB, otherwise unmodified filter";
        parameter Modelica.Blocks.Types.Init initType = Modelica.Blocks.Types.Init.NoInit "Type of initialization (1: no init, 2: steady state, 3: initial state, 4: initial output)" annotation(Evaluate = true, Dialog(group = "Initialization"));
        parameter Real x_start[n] = zeros(n) "Initial or guess values of states" annotation(Dialog(group = "Initialization"));
        parameter Real y_start = 0.0 "Initial value of output (remaining states are in steady state)" annotation(Dialog(enable = initType == Init.InitialOutput, group = "Initialization"));
        output Real x[n](start = x_start) "Filter states";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -80.6897,77.6256},{ -80.6897, -90.3744}}, color = {192,192,192}),Polygon(points = {{ -79.7044,90.6305},{ -87.7044,68.6305},{ -71.7044,68.6305},{ -79.7044,88.6305},{ -79.7044,90.6305}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Line(points = {{ -90, -80},{82, -80}}, color = {192,192,192}),Polygon(points = {{90, -80},{68, -72},{68, -88},{90, -80}}, lineColor = {192,192,192}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{0,0},{60, -60}}, lineColor = {192,192,192}, textString = "PTn"),Line(points = {{ -80.7599, -80.5082},{ -70.7599, -74.5082},{ -56, -60},{ -48, -42},{ -42, -18},{ -36,4},{ -26,20},{ -10.7599,34.9018},{ -0.759907,38.8218},{9.24009,41.6818},{19.2401,43.7818},{29.2401,45.3118},{39.2401,46.4318},{49.2401,47.2518},{59.2401,47.8518},{69.2401,48.2918},{79.2401,48.6118}}, color = {0,0,127}),Text(extent = {{ -70,94},{26,48}}, lineColor = {192,192,192}, textString = "%n"),Text(extent = {{8, -106},{8, -146}}, lineColor = {0,0,0}, textString = "f=%f")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{40,0},{ -40,0}}, color = {0,0,0}),Text(extent = {{ -55,55},{55,5}}, lineColor = {0,0,0}, textString = "1"),Rectangle(extent = {{ -60,60},{60, -60}}, lineColor = {0,0,255}),Line(points = {{ -100,0},{ -60,0}}, color = {0,0,255}),Line(points = {{60,0},{100,0}}, color = {0,0,255}),Text(extent = {{ -54, -6},{44, -56}}, lineColor = {0,0,0}, textString = "(s/w + 1)"),Text(extent = {{38, -10},{58, -30}}, lineColor = {0,0,0}, textString = "n")}), Documentation(info = "<html>
<p>This block defines the transfer function between the
input u and the output y
as an n-th order filter with <i>critical damping</i>
characteristics and cut-off frequency f. It is
implemented as a series of first order filters.
This filter type is especially useful to filter the input of an
inverse model, since the filter does not introduce any transients.
</p>

<p>
If parameter <b>normalized</b> = <b>true</b> (default), the filter
is normalized such that the amplitude of the filter transfer function
at the cut-off frequency f is 1/sqrt(2) (= 3 dB). Otherwise, the filter
is not normalized, i.e., it is unmodified. A normalized filter is usually
much better for applications, since filters of different orders are
\"comparable\", whereas non-normalized filters usually require to adapt the
cut-off frequency, when the order of the filter is changed.
Figures of the filter step responses are shown below.
Note, in versions before version 3.0 of the Modelica Standard library,
the CriticalDamping filter was provided only in non-normalzed form.
</p>

<p>If transients at the simulation start shall be avoided, the filter
should be initialized in steady state (e.g., using option
initType=Modelica.Blocks.Types.Init.SteadyState).
</p>

<p>
The critical damping filter is defined as
</p>

<pre>
    &alpha; = <b>if</b> normalized <b>then</b> <b>sqrt</b>(2^(1/n) - 1) <b>else</b> 1 // frequency correction factor
    &omega; = 2*&pi;*f/&alpha;
              1
    y = ------------- * u
         (s/w + 1)^n

</pre>

<p>
<img src=\"../Images/Blocks/CriticalDampingNormalized.png\">
</p>

<p>
<img src=\"../Images/Blocks/CriticalDampingNonNormalized.png\">
</p>

</html>
"));
      protected 
        parameter Real alpha = if normalized then sqrt(2 ^ (1 / n) - 1) else 1.0 "Frequency correction factor for normalized filter";
        parameter Real w = (2 * Modelica.Constants.pi * f) / alpha;

      initial equation 
        if initType == Init.SteadyState then
          der(x) = zeros(n);

        elseif initType == Init.InitialState then
          x = x_start;


        elseif initType == Init.InitialOutput then
          y = y_start;
          der(x[1:n - 1]) = zeros(n - 1);


        else

        end if;

      equation 
        der(x[1]) = (u - x[1]) * w;
        for i in 2:n loop
        der(x[i]) = (x[i - 1] - x[i]) * w;

        end for;
        y = x[n];
      end CriticalDamping;
    end Continuous;
  end Blocks;
  package StateGraph "Library of hierarchical state machine components to model discrete event and reactive systems"
    extends Modelica.Icons.Library2;
    annotation(Documentation(info = "<html>
<p>
Library <b>StateGraph</b> is a <b>free</b> Modelica package providing
components to model <b>discrete event</b> and <b>reactive</b>
systems in a convenient
way. It is based on the JGraphChart method and
takes advantage of Modelica features for
the \"action\" language. JGraphChart is a further development of
Grafcet to include elements of StateCharts that are not present
in Grafcet/Sequential Function Charts. Therefore, the StateGraph
library has a similar modeling power as StateCharts but avoids
some deficiences of StateCharts.
</p>
<p>
For an introduction, have especially a look at:
</p>
<ul>
<li> <a href=\"Modelica://Modelica.StateGraph.UsersGuide\">StateGraph.UsersGuide</a>
     discusses the most important aspects how to use this library.</li>
<li> <a href=\"Modelica://Modelica.StateGraph.Examples\">StateGraph.Examples</a>
     contains examples that demonstrate the usage of this library.</li>
</ul>
<p>
A typical model generated with this library is shown
in the next figure where on the left hand side a two-tank
system with a tank controller and on the right hand side the
top-level part of the tank controller as a StateGraph is shown:
</p>
<p>
<img src=\"../Images/StateGraph/Examples/ControlledTanks1_small.png\">
<img src=\"../Images/StateGraph/Examples/ControlledTanks2_small.png\">
</p>
<p>
The unique feature of the StateGraph library with respect to JGraphCharts,
Grafcet, Sequential Function Charts, and StateCharts, is Modelica's
\"single assignment rule\" that requires that every variable is defined
by exactly one equation. This leads to a different \"action\" definition
as in these formalisms. The advantage is that the translator can either
determine a useful evaluation sequence by equation sorting or
reports an error if this is not possible, e.g., because a model
would lead to a non-determinism or to a dead-lock. As a side effect,
this leads also to simpler and more easier to understand models and
global variables are no longer needed (whereas in JGraphCharts,
Grafcet, Sequential Function Charts and StateCharts global variables
are nearly always needed).
</p>
<p>
The StateGraph library is currently available in a beta release.
The available components will most likely not be changed for the
release version. It is planned to improve the convenience of
building models with the StateGraph library for the release version
(this may require to introduce some additional annotations).
It is planned to include the StateGraph library in the
Modelica standard library.
It is most useful to combine this libray with the Modelica libraries
</p>
<ul>
<li><b>Modelica.Blocks.Logical</b> that provides
    components available in PLCs (programmable logic controllers). </li>
<li><b>UserInteraction</b> that provides components to
    interactively communicate with models in a running simulation.</li>
</ul>

<p>
Copyright &copy; 1998-2009, Modelica Association and DLR
</p>
<p>
<i>This Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p><br>

</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -88, -20},{ -50, -54}}, lineColor = {0,0,0}),Line(points = {{ -50, -38},{ -24, -38}}, color = {0,0,0}),Polygon(points = {{ -24, -32},{ -12, -38},{ -24, -44},{ -24, -32}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -12, -6},{ -12, -76}}, color = {0,0,0}),Line(points = {{ -12, -38},{14, -38}}, color = {0,0,0}),Polygon(points = {{14, -32},{26, -38},{14, -44},{14, -32}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{26, -22},{64, -56}}, lineColor = {0,0,0})}));
    package UsersGuide "User's Guide of StateGraph Library"
      annotation(__Dymola_DocumentationClass = true, Documentation(info = "<html>
<p>
Library <b>StateGraph</b> is a <b>free</b> Modelica package providing
components to model <b>discrete event</b> and <b>reactive</b>
systems in a convenient
way. This package contains the <b>User's Guide</b> for
the library and has the following content:
</p>
<ol>
<li><a href=\"Modelica://Modelica.StateGraph.UsersGuide.OverView\">Overview of library</a>
     gives an overview of the library.</li>
<li> <a href=\"Modelica://Modelica.StateGraph.UsersGuide.FirstExample\">A first example</a>
     demonstrates at hand of a first example how to use this library.</li>
<li> <a href=\"Modelica://Modelica.StateGraph.UsersGuide.ApplicationExample\">An
     application example</a> demonstrates varies features at hand of the
     control of a two tank system.</li>
<li><a href=\"Modelica://Modelica.StateGraph.UsersGuide.ReleaseNotes\">Release Notes</a>
    summarizes the differences between different versions of this library.</li>
<li><a href=\"Modelica://Modelica.StateGraph.UsersGuide.Literature\">Literature</a>
    provides references that have been used to design and implement this
    library.</li>
<li><a href=\"Modelica://Modelica.StateGraph.UsersGuide.Contact\">Contact</a>
    provides information about the authors of the library as well as
    acknowledgments.</li>
</ol>
</html>"));
      class OverView "Overview of library"
        annotation(Documentation(info = "<html>
<p>
In this section, an overview of the most important features
of this library is given.
</p>
<h4><font color=\"#008000\">Steps and Transitions</font></h4>
<p>
A <b>StateGraph</b> is an enhanced finite state machine.
It is based on the JGraphChart method and
takes advantage of Modelica features for
the \"action\" language. JGraphChart is a further development of
Grafcet to include elements of StateCharts that are not present
in Grafcet/Sequential Function Charts. Therefore, the StateGraph
library has a similar modeling power as StateCharts but avoids
some deficiences of StateCharts.
</p>
<p>
The basic elements of StateGraphs are <b>steps</b> and <b>transitions</b>:
</p>
<p align=\"center\">
<img src=\"../Images/StateGraph/UsersGuide/StepAndTransition1.png\">
</p>
<p>
<b>Steps</b> represent the possible states a StateGraph can have.
If a step is active the Boolean variable <b>active</b> of
the step is <b>true</b>. If it is deactivated,
<b>active</b> = <b>false</b>. At the initial time, all \"usual\"
steps are deactivated. The <b>InitialStep</b> objects are steps
that are activated at the initial time. They are characterized
by a double box (see figure above).
</p>
<p>
<b>Transitions</b> are used to change the state of a StateGraph.
When the step connected to the input of a transition is active,
the step connected to the output of this transition is deactivated
and the transition condition becomes true, then the
transition fires. This means that the step connected to the input to the
transition is deactivated and the step connected to the output
of the transition is activated.
</p>
<p>
The transition <b>condition</b> is defined via the parameter menu
of the transition object. Clicking on object \"transition1\" in
the above figure, results in the following menu:
</p>
<p align=\"center\">
<img src=\"../Images/StateGraph/UsersGuide/StepAndTransition2.png\">
</p>
<p>
In the input field \"<b>condition</b>\", any type of time varying
Boolean expression can be given (in Modelica notation, this is
a modification of the time varying variable <b>condition</b>).
Whenever this condition is true, the transition can fire.
Additionally, it is possible to activate a timer, via
<b>enableTimer</b> (see menu above) and provide a
<b>waitTime</b>. In this case the firing of the transition
is delayed according to the provided waitTime. The transition
condition and the waitTime are displayed in the transition icon.
</p>
<p>
In the above example, the simulation starts at <b>initialStep</b>.
After 1 second, <b>transition1</b> fires and <b>step1</b> becomes
active. After another second <b>transition2</b> fires and
<b>initialStep</b> becomes again active. After a further
second <b>step1</b> becomes again active, and so on.
</p>
<p>
In JGrafcharts, Grafcet and Sequential Function Charts, the
network of steps and transitions is drawn from top to bottom.
In StateGraphs, no particular direction is defined, since
steps and transitions are blocks with input and output connectors
that can be arbitrarily placed and connected. Usually, it is
most practical to define the network from left to right,
as in the example above, since then it is easy to read the
labels on the icons.
</p>
<h4><font color=\"#008000\">Conditions and Actions</font></h4>
<p>
With the block <b>TransitionWithSignal</b>, the firing condition
can be provided as Boolean input signal, instead as entry in the
menu of the transition. An example is given in the next
figure:
</p>
<p align=\"center\">
<img src=\"../Images/StateGraph/UsersGuide/StepAndTransition3.png\">
</p>
<p>
Component \"step\" is an instance of \"StepWithSignal\" that is
a usual step where the active flag is available as Boolean
output signal. To this output, component \"Timer\" from
library \"Modelica.Blocks.Logical\" is connected. It measures the
time from the time instant where the Boolean input (i.e., the
active flag of the step) became true upto the current
time instant. The timer is connected to a comparison
component. The output is true, once the timer reaches
1 second. This signal is used as condition input of the
transition. As a result, \"transition2\" fires, once step
\"step\" has been active for 1 second.
Of course, any other
Modelica block with a Boolean output signal can be
connected to the condition input of such a transition block
as well.
</p>
<p>
Conditions of a transition can either be computed by
a network of logical blocks from the Logical library as
in the figure above, or via the \"SetBoolean\" component
any type of logical expression can be defined in textual
form, as shown in the next figure:
</p>
<p align=\"center\">
<img src=\"../Images/StateGraph/UsersGuide/StepAndTransition4.png\">
</p>
<p>
With the block \"<b>SetBoolean</b>\", a time varying expression
can be provided as modification to the output signal <b>y</b>
(see block with icon text \"timer.y > 1\" in the figure above).
The output signal can be in turn connected to the condition
input of a TransitionWithSignal block.
</p>
<p>
The \"<b>SetBoolean</b>\" block can also be used to
compute a Boolean signal depending on the active step.
In the figure above, the output of the block with the
icon text \"step.active\" is
true, when \"step\" is active, otherwise it is false
(note, the icon text of \"SetBoolean\" displays the modification
of the output signal \"y\").
This signal can then be used to compute desired <b>actions</b>
in the physical systems model. For example, if a <b>valve</b>
shall be open, when the StateGraph is in \"step1\" or
in \"step4\", a \"SetBoolean\" block may be connected to the
valve model using the following condition:
</p>
<pre>
    valve = step1.active <b>or</b> step2.active
</pre>
<p>
Via the Modelica operators <b>edge</b>(..) and <b>change</b>(..),
conditions depending on rising and falling edges of
Boolean expressions can be used when needed.
</p>
<p>
In JGrafcharts, Grafcet, Sequential Function Charts and StateCharts,
<b>actions</b> are formulated <b>within a step</b>. Such actions are
distinguished as <b>entry</b>, <b>normal</b>, <b>exit</b> and
<b>abort</b> actions. For example, a valve might be opened by
an entry action of a step and might be closed by an exit
action of the same step. In StateGraphs, this is (fortunately)
<b>not possible</b>
due to Modelicas \"single assignment rule\" that requires that every
variable is defined by exactly one equation. Instead, the
approach explained above is used. For example, via the
\"SetBoolean\" component, the valve variable is set to true
when the StateGraph is in particular steps.
</p>
<p>
This feature of a StateGraph is <b>very useful</b>, since it allows
a Modelica translator to <b>guarantee</b> that a given StateGraph
has always <b>deterministic</b> behaviour without conflicts.
In the other methodologies this is much more cumbersome. For example,
if two steps are executed in parallel and both step actions
modify the same variable, the result is either non-deterministic
or non-obvious rules have to be defined which action
takes priority. In a StateGraph, such a situation is detected by
the translator resulting in an error, since there are two equations
to compute one variable. Additional benefits of the StateGraph
approach are:
</p>
<ul>
<li> A JGrafchart or a StateChart need to potentially access
     variables in a step that are defined in
     higher hierarchical levels of a model. Therefore, mostly <b>global
     variables</b> are used in the whole network, even if the
     network is structured hierarchically. In StateGraphs this
     is not necessary, since the physical systems outside
     of a StateGraph might access the step or transition state
     via a hierarchical name. This means that <b>no global variables</b>
     are needed, because the local variables in the StateGraph
     are accessed from local variables outside of the StateGraph.
     </li>
<li> It is simpler for a user to understand the information that
     is provided in the non-graphical definition, since every
     variable is defined at exactly one place. In the other
     methodologies, the setting and re-setting of the same
     variable is cluttered within the whole network.
    </li>
</ul>
<p>
To emphasize this important difference between these methodologies,
consider the case that a state machine has the following
hierarchy:
</p>
<pre>
   stateMachine.superstate1.superstate2.step1
</pre>
<p>
Within \"step1\" a StateChart would, e.g., access variable
\"stateMachine.openValve\", say as \"entry action: openValve = true\".
This typical usage has the severe drawback that it is not possible
to use the hierarchical state \"superstate1\" as component in another
context, because \"step1\" references a particular name outside of this
component.
</p>
<p>
In a StateGraph, there would be typically a \"SetBoolean\" component
in the \"stateMachine\" component stating:
</p>
<pre>
    openValve = superstate1.superstate2.step1.active;
</pre>
<p>
As a result, the \"superstate1\" component can be used in
another context, because it does not depend on the environment
where it is used.
</p>
<h4><font color=\"#008000\">Execution Model</font></h4>
<p>
The execution model of a StateGraph follows from its
Modelica implementation: Given the states of all steps, i.e.,
whether a step is active or not active, the equations of all
steps, transitions, transition conditions, actions etc. are
sorted resulting in an execution sequence to compute
essentially the new values of the steps. If conflicts occur,
e.g., if there are more equations as variables, of if there
are algebraic loops between Boolean variables, an exception
is raised. Once all equations have been processed, the
<b>active</b> variable of all steps are updated to the newly
calculated values. Afterwards, the equations are again
evaluated. The iteration stops, once no step changes
its state anymore, i.e., once no transition fires anymore.
Then, simulation continuous until a new event is triggered,
(when a relation changes its value).
</p>
<p>
With the Modelica \"sampled(..)\" operator, a StateGraph might also
be executed within a discrete controller that is called
at regular time instants. In a future version of the StateGraph
library, this might be more directly supported.
</p>
<h4><font color=\"#008000\">Parallel and Alternative Execution</font></h4>
<p>
Parallel activities can be defined by
component <b>Parallel</b> and alternative activities
can be defined by component <b>Alternative</b>.
An example for both components is given in the next figure.
</p>
<p align=\"center\">
<img src=\"../Images/StateGraph/UsersGuide/Parallel1.png\">
</p>
<p>
Here, the branch from \"step2\" to \"step5\" is executed in parallel
to \"step1\". A transition connected to the output of a parallel
branch component can only fire if the final steps
in all parallel branches are active simultaneously.
The figure above is a screen-shot from the animation of the
StateGraph: Whenever a step is active or a transition can fire,
the corresponding component is marked in <b>green</b> color.
</p>
<p>
The three branches within \"step2\" to \"step5\" are
executed alternatively, depending which transition condition
of \"transition3\", \"transition4\", \"transition4a\" fires first.
Since all three transitions fire after 1 second, they are all
candidates for the active branch. If two or more transitions
would fire at the same time instant, a priority selection
is made: The alternative and parallel components have a
vector of connectors. Every branch has to be connected to
exactly one entry of the connector vector. The entry with
the lowest number has the highest priority.
</p>
<p>
Parallel, Alternative and Step components have vectors of
connectors. The dimensions of these vectors are set in the
corresponding parameter menu. E.g. in a \"Parallel\" component:
</p>
<p align=\"center\">
<img src=\"../Images/StateGraph/UsersGuide/Parallel2.png\">
</p>
<p>
Currently in Dymola the following menu pops up, when a branch
is connected to a vector of components in order to define
the vector index to which the component shall be connected:
</p>
<p align=\"center\">
<img src=\"../Images/StateGraph/UsersGuide/Parallel3.png\">
</p>
<h4><font color=\"#008000\">CompositeSteps, Suspend and Resume Port</font></h4>
<p>
A StateGraph can be hierarchically structured by using a <b>CompositeStep</b>.
This is a component that inherits from <b>PartialCompositeStep</b>.
An example is given in the next figure (from Examples.ControlledTanks):
</p>
<p align=\"center\">
<img src=\"../Images/StateGraph/UsersGuide/CompositeStep1.png\">
</p>
<p>
The CompositeStep component contains a local StateGraph that is
entered by one or more input transitions and that is left
by one or more output transitions. Also, other needed signals
may enter a CompositeStep. The CompositeStep has similiar properties
as a \"usual\" step: The CompositeStep is <b>active</b> once at least
one step within the CompositeStep is active. Variable <b>active</b>
defines the state of the CompositeStep.
</p>
<p>
Additionally, a CompositeStep has a <b>suspend</b> port. Whenever the
transition connected to this port fires, the CompositeStep is left
at once. When leaving the CompositeStep via the suspend port, the internal
state of the CompositeStep is saved, i.e., the active flags of all
steps within the CompositeStep. The CompositeStep might be entered via
its <b>resume</b> port. In this case the internal state from the
suspend transition is reconstructed and the CompositeStep continues
the execution that it had before the suspend transition fired
(this corresponds to the history ports of StateCharts or JGrafCharts).
</p>
<p>
A CompositeStep may contain other CompositeSteps. At every level,
a CompositeStep and all of its content can be left via its suspend ports
(actually, there
is a vector of suspend connectors, i.e., a CompositeStep might
be aborted due to different transitions).
</p>
</html>
"));
      end OverView;
      class FirstExample "A first example"
        annotation(Documentation(info = "<html>
<p>
A first example will be given here (not yet done).
</p>
</html>
"));
      end FirstExample;
      class ApplicationExample "An application example"
        annotation(Documentation(info = "<html>
<p>
In this section a more realistic, still simple, application example
is given, to demonstrate various features of the StateGraph library.
This example shows the control of a two tank system from the master thesis
of Isolde Dressler
(<a href=\"Modelica://Modelica.StateGraph.UsersGuide.Literature\">see literature</a>).
</p>
<p>
In the following figure the top level of the model is shown.
This model is available as StateGraph.Examples.ControlledTanks.
</p>
<p align=\"center\">
<img src=\"../Images/StateGraph/Examples/ControlledTanks1.png\">
</p>
<p>
In the right part of the figure, two tanks are shown. At the top part,
a large fluid source is present from which fluid can be filled in
<b>tank1</b> when <b>valve1</b> is open. Tank1 can be emptied via
<b>valve2</b> that is located in the bottom of tank2 and
fills a second <b>tank2</b> which in turn is emptied via
<b>valve3</b>. The actual levels of the tanks are measured
and are provided as signals <b>level1</b> and <b>level2</b>
to the <b>tankController</b>.
</p>
<p>
The <b>tankController</b> is controlled by three buttons,
<b>start</b>, <b>stop</b> and <b>shut</b> (for shutdown)
that are mutually exclusive. This means that whenever one button is
pressed (i.e., its state is <b>true</b>) then the other two
buttons are not pressed (i.e., their states are <b>false</b>).
When button <b>start</b> is pressed, the \"normal\" operation
to fill and to empty the two tanks is processed:
</p>
<ol>
<li> Valve 1 is opened and tank 1 is filled.</li>
<li> When tank 1 reaches its fill level limit,
     valve 1 is closed. </li>
<li> After a waiting time, valve 2 is
     opened and the fluid flows from tank 1 into tank 2.</li>
<li> When tank 1 is empty, valve 2 is closed. </li>
<li> After a waiting time, valve 3 is opened and
     the fluid flows out of tank 2</li>
<li> When tank 2 is empty, valve 3 is closed</liI>
</ol>
<p>
The above \"normal\" process can be influenced by the following
buttons:
</p>
<ul>
<li> Button <b>start</b> starts the above process.
     When this button is pressed after a \"stop\" or
     \"shut\" operation, the process operation continues.
     </li>.
<li> Button <b>stop</b> stops the above process by
     closing all valves. Then, the controller waits for
     further input (either \"start\" or \"shut\" operation).</li>
<li> Button <b>shut</b> is used to shutdown the process,
     by emptying at once both tanks. When this is achieved,
     the process goes back to its start configuration.
     Clicking on \"start\", restarts the process.</li>
</ul>
<p>
The implementation of the <b>tankController</b> is shown in
the next figure:
</p>
<p align=\"center\">
<img src=\"../Images/StateGraph/Examples/ControlledTanks2.png\">
</p>
<p>
When the \"<b>start</b>\" button is pressed, the stateGraph is
within the CompositeStep \"<b>makeProduct</b>\". During normal
operation this CompositeStep is only left, once tank2 is empty.
Afterwards, the CompositeStep is at once re-entered.
</p>
<p>
When the \"<b>stop</b>\" button is pressed, the \"makeProduct\"
CompositeStep is at once terminated via the \"<b>suspend</b>\" port
and the stateGraph waits in step \"<b>s2</b>\" for further
commands. When the \"<b>start</b>\" button is pressed, the CompositeStep
is re-entered via its <b>resume</b> port and the \"normal\"
operation continues at the state where it was aborted by the
suspend transition. If the \"<b>shut</b>\" button is pressed,
the stateGraph waits in the \"<b>emptyTanks</b>\" step, until
both tanks are empty and then waits at the initial step
\"<b>s1</b>\" for further input.
</p>
<p>
The opening and closing of valves is <b>not</b> directly
defined in the stateGraph. Instead via the \"<b>setValveX</b>\"
components, the Boolean state of the valves are computed.
For example, the output y of \"setValve2\" is computed as:
</p>
<pre>
  y = makeProduct.fillTank2.active or emptyTanks.active
</pre>
<p>
i.e., valve2 is open, when step \"makeProduct.fillTank2 or when
step \"emptyTanks\" is active. Otherwise, valve2 is closed.
</p>
</html>
"));
      end ApplicationExample;
      class ReleaseNotes "Release notes"
        annotation(Documentation(info = "<html>
<h4>Version 0.87, 2004-06-23</h4>
<ul>
<li> Included in Modelica standard library 2.0 Beta 1 with the new block connectors.
     Changed all the references to the block connectors and the Logical library
     correspondingly.</li>
</ul>
<h4>Version 0.86, 2004-06-20</h4>
<ul>
<li> New components \"Alternative\" and \"Parallel\" for alternative and
     parallel execution paths.</li>
<li> A step has now a vector of input and output connectors in order
     that multiple connections to and from a step are possible</li>
<li> Removed components \"AlternativeSplit\", \"AlternativeJoin\",
     \"ParallelSplit\" and \"ParallelJoin\" since the newly introduced
     components (\"Alternative\", \"Parallel\", vector connectors of steps)
     have the same modeling power but are safer and more convenient.</li>
<li> Removed the timer in a step (attach instead Logical.Timer to
     the \"active\" port of a \"StepWithSignal\" component). Note, that in
     most cases it is more convenient and more efficient to use the
     built-in timer of a transition.</li>
<li> Component \"StepInitial\" renamed to \"InitialStep\".</li>
<li> New component \"Timer\" within sublibrary Logical.</li>
<li> Updated and improved documentation of the library.</li>
</ul>
<h4>Version 0.85, 2004-06-17</h4>
<ul>
<li> Renamed \"MacroStep\" to \"CompositeStep\" and the ports of the MacroStep
     from \"abort\" to \"suspend\" and \"histoy\" to \"resume\".</li>
<li> Nested \"CompositeStep\" components are supported, based on the
     experimental feature of nested inner/outer components
     introduced by Dymola. This means that CompositeSteps can
     be suspended and resumed at every level.</li>
<li> New example \"Examples.ShowExceptions\" to demonstrate the new
     feature of nested CompositeSteps.</li>
<li> New package \"Logical\". It contains all components of
     ModelicaAdditions.Blocks.Logical, but with new block connectors
     and nicer icons. Additionally, logical blocks are also added.</li>
<li> Improved icons for several components of the StateGraph library.</li>
</ul>
<h4>Version 0.83, 2004-05-21</h4>
<ul>
<li> The \"abort\" and \"history\" connectors are no longer visible in the
     diagram layer of a CompositeStep since it is not allowed to connect
     to them in a CompositeStep.</li>
<li> Made the diagram/icon size of a CompositeStep smaller (from 200/-200 to
     150/-150).</li>
<li> Improved icons for \"SetBoolean/SetInteger/SetReal\" components.</li>
<li> Renamed \"ParameterReal\" to \"SetRealParameter\".</li>
</ul>
<h4>Version 0.82, 2004-05-18</h4>
<p>
Implemented a first version that is provided to other people.
</p>
</html>
"));
      end ReleaseNotes;
      class Literature "Literature"
        annotation(Documentation(info = "<html>
<p>
The StateGraph library is based on the following references:
</p>
<dl>
<dt>Arzen K.-E. (2004):</dt>
<dd> <b>JGrafchart User Manual. Version 1.5</b>.
     Department of Automatic Control, Lund Institute of Technology,
     Lund, Sweden, Feb. 13<br>&nbsp;</dd>
<dt>Dressler I. (2004):</dt>
<dd> <b>Code Generation From JGrafchart to Modelica</b>.
     Master thesis, supervisor: Karl-Erik Arzen,
     Department of Automatic Control, Lund Institute of Technology,
     Lund, Sweden, March 30<br>&nbsp;</dd>
<dt>Elmqvist H., Mattsson S.E., Otter M. (2001):</dt>
<dd> <b>Object-Oriented and Hybrid Modeling in Modelica</b>.
     Journal Europeen des systemes automatises (JESA),
     Volume 35 - n. 1.<br>&nbsp;</dd>
<dt>Mosterman P., Otter M., Elmqvist H. (1998):</dt>
<dd> <b>Modeling Petri Nets as Local Constraint Equations for
     Hybrid Systems using Modelica</b>.
     SCSC'98, Reno, Nevada, USA,
     Society for Computer Simulation International, pp. 314-319.
     </dd>
</dl>
</html>
"));
      end Literature;
      class Contact "Contact"
        annotation(Documentation(info = "<html>
<dl>
<dt><b>Main Author:</b>
<dd><a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a><br>
    Deutsches Zentrum f&uuml;r Luft und Raumfahrt e.V. (DLR)<br>
    Institut f&uuml;r Robotik und Mechatronik<br>
    Abteilung f&uuml;r Entwurfsorientierte Regelungstechnik<br>
    Postfach 1116<br>
    D-82230 Wessling<br>
    Germany<br>
    email: <A HREF=\"mailto:Martin.Otter@dlr.de\">Martin.Otter@dlr.de</A><br>
</dl>
<p><b>Acknowledgements:</b></p>
<ul>
<li> The development of this library was strongly motivated by the
     master thesis of Isolde Dressler
     (<a href=\"Modelica://Modelica.StateGraph.UsersGuide.Literature\">see literature</a>),
     in which
     a compiler from JGrafChart to Modelica was designed and
     implemented. This project was supervised by Karl-Erik Arzen
     from Departement of Automatic Control, Lund Institut of
     Technology, Lund, Sweden.</li>
<li> This library profits also from the experience gained
     in the focused research program (Schwerpunktprogramm)
     \"Continuous-Discrete Dynamic Systems\" (KONDISK), sponsored by the
     Deutsche Forschungsgemeinschaft under grants OT174/1-2 and EN152/22-2.
     This support is most gratefully acknowledged.
 </li>
<li> The implementation of the basic components of this library by describing
     finite state machines with equations is based on
     (Elmqvist, Mattsson and Otter, 2001),
     which in turn uses ideas from (Mosterman, Otter and Elmqvist, 1998),
     see <a href=\"Modelica://Modelica.StateGraph.UsersGuide.Literature\">literature</a></li>
</ul>
</html>
"));
      end Contact;
    end UsersGuide;
    package Examples "Examples to demonstrate the usage of the components of the StateGraph library"
      model FirstExample "A first simple StateGraph example"
        extends Modelica.Icons.Example;
        InitialStep initialStep annotation(Placement(transformation(extent = {{ -48,0},{ -28,20}}, rotation = 0)));
        Transition transition1(enableTimer = true, waitTime = 1) annotation(Placement(transformation(extent = {{ -20,0},{0,20}}, rotation = 0)));
        Step step annotation(Placement(transformation(extent = {{10,0},{30,20}}, rotation = 0)));
        Transition transition2(enableTimer = true, waitTime = 1) annotation(Placement(transformation(extent = {{40,0},{60,20}}, rotation = 0)));
        inner StateGraphRoot stateGraphRoot annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}})));

      equation 
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 5), Documentation(info = "<html>

</html>"));
        connect(initialStep.outPort[1],transition1.inPort) annotation(Line(points = {{ -27.5,10},{ -14,10}}, color = {0,0,0}));
        connect(transition1.outPort,step.inPort[1]) annotation(Line(points = {{ -8.5,10},{9,10}}, color = {0,0,0}));
        connect(step.outPort[1],transition2.inPort) annotation(Line(points = {{30.5,10},{46,10}}, color = {0,0,0}));
        connect(transition2.outPort,initialStep.inPort[1]) annotation(Line(points = {{51.5,10},{70,10},{70,32},{ -62,32},{ -62,10},{ -49,10}}, color = {0,0,0}));
      end FirstExample;
      model FirstExample_Variant2 "A variant of the first simple StateGraph example"
        extends Modelica.Icons.Example;
        InitialStep initialStep annotation(Placement(transformation(extent = {{ -70,0},{ -50,20}}, rotation = 0)));
        Transition transition1(enableTimer = true, waitTime = 1) annotation(Placement(transformation(extent = {{ -42,0},{ -22,20}}, rotation = 0)));
        StepWithSignal step annotation(Placement(transformation(extent = {{ -14,0},{6,20}}, rotation = 0)));
        TransitionWithSignal transition2 annotation(Placement(transformation(extent = {{52,0},{72,20}}, rotation = 0)));
        Modelica.Blocks.Logical.Timer timer annotation(Placement(transformation(extent = {{6, -40},{26, -20}}, rotation = 0)));
        Modelica.Blocks.Logical.GreaterEqualThreshold greaterEqual(threshold = 1) annotation(Placement(transformation(extent = {{36, -40},{56, -20}}, rotation = 0)));
        inner StateGraphRoot stateGraphRoot annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}})));

      equation 
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 5), Documentation(info = "<html>

</html>"));
        connect(initialStep.outPort[1],transition1.inPort) annotation(Line(points = {{ -49.5,10},{ -36,10}}, color = {0,0,0}));
        connect(transition1.outPort,step.inPort[1]) annotation(Line(points = {{ -30.5,10},{ -15,10}}, color = {0,0,0}));
        connect(step.active,timer.u) annotation(Line(points = {{ -4, -1},{ -4, -30},{4, -30}}, color = {255,0,255}));
        connect(step.outPort[1],transition2.inPort) annotation(Line(points = {{6.5,10},{58,10}}, color = {0,0,0}));
        connect(timer.y,greaterEqual.u) annotation(Line(points = {{27, -30},{34, -30}}, color = {0,0,255}));
        connect(greaterEqual.y,transition2.condition) annotation(Line(points = {{57, -30},{62, -30},{62, -2}}, color = {255,0,255}));
        connect(transition2.outPort,initialStep.inPort[1]) annotation(Line(points = {{63.5,10},{82,10},{82,32},{ -80,32},{ -80,10},{ -71,10}}, color = {0,0,0}));
      end FirstExample_Variant2;
      model FirstExample_Variant3 "A variant of the first simple StateGraph example"
        extends Modelica.Icons.Example;
        InitialStep initialStep annotation(Placement(transformation(extent = {{ -70,0},{ -50,20}}, rotation = 0)));
        Transition transition1(enableTimer = true, waitTime = 1) annotation(Placement(transformation(extent = {{ -42,0},{ -22,20}}, rotation = 0)));
        StepWithSignal step annotation(Placement(transformation(extent = {{ -14,0},{6,20}}, rotation = 0)));
        TransitionWithSignal transition2 annotation(Placement(transformation(extent = {{56,0},{76,20}}, rotation = 0)));
        Modelica.Blocks.Logical.Timer timer annotation(Placement(transformation(origin = { -4, -30}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
        Modelica.Blocks.Sources.BooleanExpression SetBoolean1(y = timer.y > 1) annotation(Placement(transformation(extent = {{28, -40},{60, -20}}, rotation = 0)));
        Modelica.Blocks.Sources.BooleanExpression SetBoolean2(y = step.active) annotation(Placement(transformation(extent = {{ -68, -40},{ -36, -20}}, rotation = 0)));
        inner StateGraphRoot stateGraphRoot annotation(Placement(transformation(extent = {{ -80,60},{ -60,80}})));

      equation 
        annotation(Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics), experiment(StopTime = 5), Documentation(info = "<html>

</html>"));
        connect(initialStep.outPort[1],transition1.inPort) annotation(Line(points = {{ -49.5,10},{ -36,10}}, color = {0,0,0}));
        connect(transition1.outPort,step.inPort[1]) annotation(Line(points = {{ -30.5,10},{ -15,10}}, color = {0,0,0}));
        connect(step.active,timer.u) annotation(Line(points = {{ -4, -1},{ -4, -9.5},{ -4, -18},{ -4, -18}}, color = {255,0,255}));
        connect(step.outPort[1],transition2.inPort) annotation(Line(points = {{6.5,10},{62,10}}, color = {0,0,0}));
        connect(transition2.outPort,initialStep.inPort[1]) annotation(Line(points = {{67.5,10},{82,10},{82,32},{ -80,32},{ -80,10},{ -71,10}}, color = {0,0,0}));
        connect(SetBoolean1.y,transition2.condition) annotation(Line(points = {{61.6, -30},{66, -30},{66, -2}}, color = {255,0,255}));
      end FirstExample_Variant3;
      extends Modelica.Icons.Library;
      model ExecutionPaths "Example to demonstrate parallel and alternative execution paths"
        extends Modelica.Icons.Example;
        annotation(Documentation(info = "<HTML>
<p>
This is an example to demonstrate in which way <b>parallel</b> activities
can be modelled by a StateGraph. When transition1 fires
(after 1 second), two branches are executed in parallel.
After 6 seconds the two branches are synchronized in order to arrive
at step6.
</p>
<p>
Before simulating the model, try to figure out whether which branch
of the alternative sequence is executed. Note, that alternatives
have priorities according to the port index (alternative.split[1]
has a higher priority to fire as alternative.split[2]).
</p>
</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -160, -160},{160,160}}, grid = {2,2}), graphics), experiment(StopTime = 15));
        InitialStep step0 annotation(Placement(transformation(extent = {{ -140, -100},{ -120, -80}}, rotation = 0)));
        Transition transition1(enableTimer = true, waitTime = 1) annotation(Placement(transformation(extent = {{ -100, -100},{ -80, -80}}, rotation = 0)));
        Step step1 annotation(Placement(transformation(extent = {{ -10, -40},{10, -20}}, rotation = 0)));
        Transition transition2(enableTimer = true, waitTime = 1) annotation(Placement(transformation(extent = {{90, -100},{110, -80}}, rotation = 0)));
        Step step6 annotation(Placement(transformation(extent = {{120, -100},{140, -80}}, rotation = 0)));
        Step step2 annotation(Placement(transformation(extent = {{ -98,40},{ -78,60}}, rotation = 0)));
        Transition transition3(enableTimer = true, waitTime = 1) annotation(Placement(transformation(extent = {{ -42,80},{ -22,100}}, rotation = 0)));
        Transition transition4(enableTimer = true, waitTime = 1) annotation(Placement(transformation(extent = {{ -42,40},{ -22,60}}, rotation = 0)));
        Step step3 annotation(Placement(transformation(extent = {{ -8,80},{12,100}}, rotation = 0)));
        Step step4 annotation(Placement(transformation(extent = {{ -8,40},{12,60}}, rotation = 0)));
        Transition transition5(enableTimer = true, waitTime = 1) annotation(Placement(transformation(extent = {{26,80},{46,100}}, rotation = 0)));
        Transition transition6(enableTimer = true, waitTime = 1) annotation(Placement(transformation(extent = {{26,40},{46,60}}, rotation = 0)));
        Step step5 annotation(Placement(transformation(extent = {{80,40},{100,60}}, rotation = 0)));
        Modelica.Blocks.Sources.RealExpression setReal(y = time) annotation(Placement(transformation(extent = {{21, -160},{41, -140}}, rotation = 0)));
        TransitionWithSignal transition7 annotation(Placement(transformation(extent = {{9, -134},{ -11, -114}}, rotation = 0)));
        Modelica.Blocks.Sources.BooleanExpression setCondition(y = time > 7) annotation(Placement(transformation(extent = {{ -77, -160},{ -19, -140}}, rotation = 0)));
        Transition transition4a(enableTimer = true, waitTime = 1) annotation(Placement(transformation(extent = {{ -42,0},{ -22,20}}, rotation = 0)));
        Step step4a annotation(Placement(transformation(extent = {{ -8,0},{12,20}}, rotation = 0)));
        Transition transition6a(enableTimer = true, waitTime = 2) annotation(Placement(transformation(extent = {{26,0},{46,20}}, rotation = 0)));
        StateGraph.Temporary.NumericValue NumericValue1 annotation(Placement(transformation(extent = {{61, -160},{81, -140}}, rotation = 0)));
        Alternative alternative(nBranches = 3) annotation(Placement(transformation(extent = {{ -70, -10},{72,110}}, rotation = 0)));
        Parallel Parallel1 annotation(Placement(transformation(extent = {{ -154, -50},{152,120}}, rotation = 0)));
        inner StateGraphRoot stateGraphRoot annotation(Placement(transformation(extent = {{ -160,120},{ -140,140}})));

      equation 
        connect(transition3.outPort,step3.inPort[1]) annotation(Line(points = {{ -30.5,90},{ -9,90}}, color = {0,0,0}));
        connect(step3.outPort[1],transition5.inPort) annotation(Line(points = {{12.5,90},{32,90}}, color = {0,0,0}));
        connect(transition4.outPort,step4.inPort[1]) annotation(Line(points = {{ -30.5,50},{ -9,50}}, color = {0,0,0}));
        connect(step4.outPort[1],transition6.inPort) annotation(Line(points = {{12.5,50},{32,50}}, color = {0,0,0}));
        connect(transition7.outPort,step0.inPort[1]) annotation(Line(points = {{ -2.5, -124},{ -149, -124},{ -149, -90},{ -141, -90}}, color = {0,0,0}));
        connect(step6.outPort[1],transition7.inPort) annotation(Line(points = {{140.5, -90},{150, -90},{150, -124},{3, -124}}, color = {0,0,0}));
        connect(transition4a.outPort,step4a.inPort[1]) annotation(Line(points = {{ -30.5,10},{ -9,10}}, color = {0,0,0}));
        connect(step4a.outPort[1],transition6a.inPort) annotation(Line(points = {{12.5,10},{32,10}}, color = {0,0,0}));
        connect(setCondition.y,transition7.condition) annotation(Line(points = {{ -16.1, -150},{ -1, -150},{ -1, -136}}, color = {255,0,255}));
        connect(setReal.y,NumericValue1.Value) annotation(Line(points = {{42, -150},{59, -150}}, color = {0,0,255}));
        connect(transition3.inPort,alternative.split[1]) annotation(Line(points = {{ -36,90},{ -55.09,90}}, color = {0,0,0}));
        connect(transition4.inPort,alternative.split[2]) annotation(Line(points = {{ -36,50},{ -55.09,50}}, color = {0,0,0}));
        connect(transition4a.inPort,alternative.split[3]) annotation(Line(points = {{ -36,10},{ -45.0125,10},{ -45.0125,10},{ -55.09,10}}, color = {0,0,0}));
        connect(transition5.outPort,alternative.join[1]) annotation(Line(points = {{37.5,90},{57.09,90}}, color = {0,0,0}));
        connect(transition6.outPort,alternative.join[2]) annotation(Line(points = {{37.5,50},{57.09,50}}, color = {0,0,0}));
        connect(transition6a.outPort,alternative.join[3]) annotation(Line(points = {{37.5,10},{46.7625,10},{46.7625,10},{57.09,10}}, color = {0,0,0}));
        connect(step2.outPort[1],alternative.inPort) annotation(Line(points = {{ -77.5,50},{ -72.13,50}}, color = {0,0,0}));
        connect(alternative.outPort,step5.inPort[1]) annotation(Line(points = {{73.42,50},{79,50}}, color = {0,0,0}));
        connect(step2.inPort[1],Parallel1.split[1]) annotation(Line(points = {{ -99,50},{ -118,50},{ -118,78},{ -119.575,78},{ -119.575,77.5}}, color = {0,0,0}));
        connect(step1.outPort[1],Parallel1.join[2]) annotation(Line(points = {{10.5, -30},{118, -30},{118, -7.5},{117.575, -7.5}}, color = {0,0,0}));
        connect(step0.outPort[1],transition1.inPort) annotation(Line(points = {{ -119.5, -90},{ -94, -90}}, color = {0,0,0}));
        connect(transition2.outPort,step6.inPort[1]) annotation(Line(points = {{101.5, -90},{119, -90}}, color = {0,0,0}));
        connect(transition1.outPort,Parallel1.inPort) annotation(Line(points = {{ -88.5, -90},{ -70, -90},{ -70, -64},{ -174, -64},{ -174,35},{ -158.59,35}}, color = {0,0,0}));
        connect(Parallel1.outPort,transition2.inPort) annotation(Line(points = {{155.06,35},{168,35},{168, -60},{80, -60},{80, -90},{96, -90}}, color = {0,0,0}));
        connect(step5.outPort[1],Parallel1.join[1]) annotation(Line(points = {{100.5,50},{116,50},{116,77.5},{117.575,77.5}}, color = {0,0,0}));
        connect(Parallel1.split[2],step1.inPort[1]) annotation(Line(points = {{ -119.575, -7.5},{ -118, -8},{ -118, -30},{ -11, -30}}, color = {0,0,0}));
      end ExecutionPaths;
      model ShowCompositeStep "Example to demonstrate parallel activities described by a StateGraph"
        extends Modelica.Icons.Example;
        annotation(Documentation(info = "<HTML>
<p>
This is the same example as \"ExecutionPaths\". The only difference
is that the alternative paths are included in a \"CompositeStep\".
</p>
</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), experiment(StopTime = 15));
        Utilities.CompositeStep compositeStep annotation(Placement(transformation(extent = {{ -10,5},{20,35}}, rotation = 0)));
        InitialStep step0 annotation(Placement(transformation(extent = {{ -89, -10},{ -69,10}}, rotation = 0)));
        Transition transition1(enableTimer = true, waitTime = 1) annotation(Placement(transformation(extent = {{ -59, -10},{ -39,10}}, rotation = 0)));
        Step step1 annotation(Placement(transformation(extent = {{ -4, -30},{16, -10}}, rotation = 0)));
        Transition transition2(enableTimer = true, waitTime = 1) annotation(Placement(transformation(extent = {{45, -10},{65,10}}, rotation = 0)));
        Step step6 annotation(Placement(transformation(extent = {{71, -10},{91,10}}, rotation = 0)));
        TransitionWithSignal transition7 annotation(Placement(transformation(extent = {{10, -70},{ -10, -50}}, rotation = 0)));
        Parallel Parallel1 annotation(Placement(transformation(extent = {{ -30, -40},{36,40}}, rotation = 0)));
        Modelica.Blocks.Sources.BooleanExpression setCondition(y = time > 7) annotation(Placement(transformation(extent = {{ -40, -90},{ -10, -70}}, rotation = 0)));
        inner StateGraphRoot stateGraphRoot annotation(Placement(transformation(extent = {{ -90,50},{ -70,70}})));

      equation 
        connect(step0.outPort[1],transition1.inPort) annotation(Line(points = {{ -68.5,0},{ -53,0}}, color = {0,0,0}));
        connect(transition7.outPort,step0.inPort[1]) annotation(Line(points = {{ -1.5, -60},{ -98, -60},{ -98,0},{ -90,0}}, color = {0,0,0}));
        connect(step6.outPort[1],transition7.inPort) annotation(Line(points = {{91.5,0},{96,0},{96, -60},{4, -60}}, color = {0,0,0}));
        connect(transition2.outPort,step6.inPort[1]) annotation(Line(points = {{56.5,0},{70,0}}, color = {0,0,0}));
        connect(transition1.outPort,Parallel1.inPort) annotation(Line(points = {{ -47.5,0},{ -30.99,0}}, color = {0,0,0}));
        connect(Parallel1.outPort,transition2.inPort) annotation(Line(points = {{36.66,0},{51,0}}, color = {0,0,0}));
        connect(compositeStep.inPort,Parallel1.split[1]) annotation(Line(points = {{ -11,20},{ -22.575,20}}, color = {0,0,0}));
        connect(compositeStep.outPort,Parallel1.join[1]) annotation(Line(points = {{20.5,20},{28.575,20}}, color = {0,0,0}));
        connect(step1.inPort[1],Parallel1.split[2]) annotation(Line(points = {{ -5, -20},{ -22.575, -20}}, color = {0,0,0}));
        connect(step1.outPort[1],Parallel1.join[2]) annotation(Line(points = {{16.5, -20},{28.575, -20}}, color = {0,0,0}));
        connect(setCondition.y,transition7.condition) annotation(Line(points = {{ -8.5, -80},{0, -80},{0, -72}}, color = {255,0,255}));
      end ShowCompositeStep;
      model ShowExceptions "Example to demonstrate how a hierarchically structured StateGraph can suspend and resume actions on different levels"
        extends Modelica.Icons.Example;
        annotation(Documentation(info = "<HTML>
<p>
CompositeStep \"compositeStep\" is a hierarchical StateGraph consisting of
two other subgraphs. Whenever component \"compositeStep\" is suspended,
all steps with in \"compositeStep\" are deactivated. By entering \"compositeStep\"
via its \"resume\" port, all steps within \"compositeStep\" are activated
according to their setting before leaving the \"compositeStep\" via its
\"suspend\" port.
</p>
</HTML>
"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), experiment(StopTime = 20));
        Utilities.CompositeStep1 compositeStep annotation(Placement(transformation(extent = {{ -20,25},{10,55}}, rotation = 0)));
        InitialStep initialStep annotation(Placement(transformation(extent = {{ -80,30},{ -60,50}}, rotation = 0)));
        Transition transition1(enableTimer = true, waitTime = 1) annotation(Placement(transformation(extent = {{ -50,30},{ -30,50}}, rotation = 0)));
        Transition transition2(enableTimer = true, waitTime = 1) annotation(Placement(transformation(extent = {{20,30},{40,50}}, rotation = 0)));
        Transition transition3(enableTimer = true, waitTime = 2) annotation(Placement(transformation(extent = {{ -55, -30},{ -35, -10}}, rotation = 0)));
        Step step1 annotation(Placement(transformation(extent = {{ -24, -30},{ -4, -10}}, rotation = 0)));
        Transition transition4(enableTimer = true, waitTime = 1) annotation(Placement(transformation(extent = {{10, -30},{30, -10}}, rotation = 0)));
        inner StateGraphRoot stateGraphRoot annotation(Placement(transformation(extent = {{ -90, -80},{ -70, -60}})));

      equation 
        connect(transition1.outPort,compositeStep.inPort) annotation(Line(points = {{ -38.5,40},{ -21,40}}, color = {0,0,0}));
        connect(initialStep.outPort[1],transition1.inPort) annotation(Line(points = {{ -59.5,40},{ -44,40}}, color = {0,0,0}));
        connect(compositeStep.outPort,transition2.inPort) annotation(Line(points = {{10.5,40},{26,40}}, color = {0,0,0}));
        connect(transition2.outPort,initialStep.inPort[1]) annotation(Line(points = {{31.5,40},{46,40},{46,80},{ -90,80},{ -90,40},{ -81,40}}, color = {0,0,0}));
        connect(compositeStep.suspend[1],transition3.inPort) annotation(Line(points = {{ -12.5,24.5},{ -12.5,10},{ -60,10},{ -60, -20},{ -49, -20}}, color = {0,0,0}));
        connect(transition3.outPort,step1.inPort[1]) annotation(Line(points = {{ -43.5, -20},{ -25, -20}}, color = {0,0,0}));
        connect(step1.outPort[1],transition4.inPort) annotation(Line(points = {{ -3.5, -20},{16, -20}}, color = {0,0,0}));
        connect(transition4.outPort,compositeStep.resume[1]) annotation(Line(points = {{21.5, -20},{40, -20},{40,10},{2.5,10},{2.5,24}}, color = {0,0,0}));
      end ShowExceptions;
      model ControlledTanks "Demonstrating the controller of a tank filling/emptying system"
        extends Modelica.Icons.Example;
        Utilities.TankController tankController annotation(Placement(transformation(extent = {{ -50, -20},{ -10,20}}, rotation = 0)));
        StateGraph.Temporary.RadioButton start(reset = {stop.on,shut.on}, buttonTimeTable = {1,13,15,19}) annotation(Placement(transformation(extent = {{ -90,20},{ -70,40}}, rotation = 0)));
        StateGraph.Temporary.RadioButton stop(reset = {start.on,shut.on}, buttonTimeTable = {13,15,19,21}) annotation(Placement(transformation(extent = {{ -90, -10},{ -70,10}}, rotation = 0)));
        StateGraph.Temporary.RadioButton shut(reset = {start.on,stop.on}, buttonTimeTable = {21,100}) annotation(Placement(transformation(extent = {{ -90, -40},{ -70, -20}}, rotation = 0)));
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {0.5,0.5}), graphics), experiment(StopTime = 100), Documentation(info = "<html>
<p>
With this example the controller of a tank filling/emptying system
is demonstrated. This example is from Dressler (2004),
see <a href=\"Modelica://Modelica.StateGraph.UsersGuide.Literature\">Literature</a>.
The basic operation is to fill and empty the two tanks:
</p>
<ol>
<li> Valve 1 is opened and tank 1 is filled.</li>
<li> When tank 1 reaches its fill level limit,
     valve 1 is closed. </li>
<li> After a waiting time, valve 2 is
     opened and the fluid flows from tank 1 into tank 2.</li>
<li> When tank 1 is empty, valve 2 is closed. </li>
<li> After a waiting time, valve 3 is opened and
     the fluid flows out of tank 2</li>
<li> When tank 3 is empty, valve 3 is closed</liI>
</ol>
<p>
The above \"normal\" process can be influenced by three
buttons:
</p>
<ul>
<li> Button <b>start</b> starts the above process.
     When this button is pressed after a \"stop\" or
     \"shut\" operation, the process operation continues.
     </li>.
<li> Button <b>stop</b> stops the above process by
     closing all valves. Then, the controller waits for
     further input (either \"start\" or \"shut\" operation).</li>
<li> Button <b>shut</b> is used to shutdown the process,
     by emptying at once both tanks. When this is achieved,
     the process goes back to its start configuration.
     Clicking on \"start\", restarts the process.</li>
</ul>

</html>"));
        Utilities.Tank tank1 annotation(Placement(transformation(extent = {{10,20},{60,70}}, rotation = 0)));
        Utilities.Tank tank2 annotation(Placement(transformation(extent = {{34, -60},{84, -10}}, rotation = 0)));
        Utilities.valve valve1 annotation(Placement(transformation(origin = {22.5,72}, extent = {{ -5.5, -5.5},{5.5,5.5}}, rotation = 270)));
        Utilities.Source source annotation(Placement(transformation(extent = {{12.5,80.5},{32.5,100.5}}, rotation = 0)));
        Utilities.valve valve2 annotation(Placement(transformation(origin = {46.5,13}, extent = {{ -7, -8},{7,8}}, rotation = 270)));
        Utilities.valve valve3 annotation(Placement(transformation(origin = {73.5, -77}, extent = {{ -7, -8},{7,8}}, rotation = 270)));
        inner StateGraphRoot stateGraphRoot annotation(Placement(transformation(extent = {{ -90,75},{ -70,95}})));

      equation 
        connect(tank1.outflow1,valve2.outflow1) annotation(Line(points = {{50,33.75},{50,26.875},{46.5,26.875},{46.5,16.5}}, color = {0,0,0}, thickness = 0.5));
        connect(tank2.inflow1,valve2.inflow1) annotation(Line(points = {{46.5, -18.75},{46.5,9.5}}, color = {0,0,0}, thickness = 0.5));
        connect(tank2.outflow1,valve3.outflow1) annotation(Line(points = {{74, -46.25},{74, -73.5},{73.5, -73.5}}, color = {0,0,0}, thickness = 0.5));
        connect(tank1.inflow1,valve1.inflow1) annotation(Line(points = {{22.5,61.25},{22.5,69.25}}, color = {0,0,0}, thickness = 0.5));
        connect(shut.on,tankController.shut) annotation(Line(points = {{ -69, -30},{ -62, -30},{ -62, -12},{ -52, -12}}, color = {255,0,255}));
        connect(stop.on,tankController.stop) annotation(Line(points = {{ -69,0},{ -52,0}}, color = {255,0,255}));
        connect(start.on,tankController.start) annotation(Line(points = {{ -69,30},{ -60,30},{ -60,12},{ -52,12}}, color = {255,0,255}));
        connect(tank1.levelSensor,tankController.level1) annotation(Line(points = {{17.25,40},{ -30,40},{ -30,60},{ -97,60},{ -97, -50},{ -42, -50},{ -42, -22}}, color = {0,0,255}));
        connect(tank2.levelSensor,tankController.level2) annotation(Line(points = {{41.25, -40},{ -18, -40},{ -18, -22}}, color = {0,0,255}));
        connect(tankController.valve1,valve1.valveControl) annotation(Line(points = {{ -9,12},{10,12},{10,72},{18.1,72}}, color = {255,0,255}));
        connect(tankController.valve2,valve2.valveControl) annotation(Line(points = {{ -9,0},{30,0},{30,13},{40.1,13}}, color = {255,0,255}));
        connect(tankController.valve3,valve3.valveControl) annotation(Line(points = {{ -9, -12},{23, -12},{23, -77},{67.1, -77}}, color = {255,0,255}));
        connect(source.outflow1,valve1.outflow1) annotation(Line(points = {{22.5,85.5},{22.5,74.75}}, color = {0,0,0}, thickness = 0.5));
      end ControlledTanks;
      package Utilities "Utility components for the examples"
        model TankController "Controller for tank system"
          extends StateGraph.Interfaces.PartialStateGraphIcon;
          parameter Temporary.SetRealParameter limit = 0.98 "Limit level of tank 1" annotation(Placement(transformation(extent = {{ -64,76},{ -44,96}}, rotation = 0)));
          parameter Temporary.SetRealParameter waitTime = 3 "Wait time" annotation(Placement(transformation(extent = {{ -91,76},{ -71,96}}, rotation = 0)));
          InitialStep s1(nIn = 2) annotation(Placement(transformation(extent = {{ -72,30},{ -52,50}}, rotation = 0)));
          MakeProduct makeProduct(limit = limit, waitTime = waitTime) annotation(Placement(transformation(extent = {{ -20,25},{10,55}}, rotation = 0)));
          Transition T1(condition = start) annotation(Placement(transformation(extent = {{ -50,50},{ -30,30}}, rotation = 0)));
          Transition T2(condition = level2 < 0.001) annotation(Placement(transformation(extent = {{27,50},{47,30}}, rotation = 0)));
          Transition T3(condition = stop) annotation(Placement(transformation(origin = { -23, -1}, extent = {{ -10, -10},{10,10}}, rotation = 270)));
          Step s2(nOut = 2) annotation(Placement(transformation(extent = {{ -50, -60},{ -30, -40}}, rotation = 0)));
          Transition T4(condition = start) annotation(Placement(transformation(origin = {10, -1}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Transition T5(condition = shut) annotation(Placement(transformation(extent = {{ -6, -60},{14, -40}}, rotation = 0)));
          Step emptyTanks annotation(Placement(transformation(extent = {{22, -60},{42, -40}}, rotation = 0)));
          Transition T6(condition = level1 + level2 < 0.001) annotation(Placement(transformation(extent = {{45, -60},{65, -40}}, rotation = 0)));
          Modelica.Blocks.Interfaces.BooleanInput start annotation(Placement(transformation(extent = {{ -120,50},{ -100,70}}, rotation = 0)));
          Modelica.Blocks.Interfaces.BooleanInput stop annotation(Placement(transformation(extent = {{ -120, -10},{ -100,10}}, rotation = 0)));
          Modelica.Blocks.Interfaces.BooleanInput shut annotation(Placement(transformation(extent = {{ -120, -70},{ -100, -50}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput level1 annotation(Placement(transformation(origin = { -60, -110}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Blocks.Interfaces.RealInput level2 annotation(Placement(transformation(origin = {60, -110}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
          Modelica.Blocks.Interfaces.BooleanOutput valve1 annotation(Placement(transformation(extent = {{100,55},{110,65}}, rotation = 0)));
          Modelica.Blocks.Interfaces.BooleanOutput valve2 annotation(Placement(transformation(extent = {{100, -5},{110,5}}, rotation = 0)));
          Modelica.Blocks.Interfaces.BooleanOutput valve3 annotation(Placement(transformation(extent = {{100, -65},{110, -55}}, rotation = 0)));
          Modelica.Blocks.Sources.BooleanExpression setValve1(y = makeProduct.fillTank1.active) annotation(Placement(transformation(extent = {{20,73},{80,92}}, rotation = 0)));
          Modelica.Blocks.Sources.BooleanExpression setValve2(y = makeProduct.fillTank2.active or emptyTanks.active) annotation(Placement(transformation(extent = {{ -25, -89},{80, -68}}, rotation = 0)));
          Modelica.Blocks.Sources.BooleanExpression setValve3(y = makeProduct.emptyTank2.active or emptyTanks.active) annotation(Placement(transformation(extent = {{ -26, -100},{80, -80}}, rotation = 0)));

        equation 
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0})}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -100,68},{ -32,54}}, lineColor = {0,0,0}, textString = "start"),Text(extent = {{ -100,6},{ -32, -8}}, lineColor = {0,0,0}, textString = "stop"),Text(extent = {{ -100, -54},{ -32, -68}}, lineColor = {0,0,0}, textString = "shut"),Text(extent = {{ -94, -82},{ -18, -96}}, lineColor = {0,0,0}, textString = "level1"),Text(extent = {{24, -84},{96, -98}}, lineColor = {0,0,0}, textString = "level2"),Text(extent = {{31,68},{99,54}}, lineColor = {0,0,0}, textString = "valve1"),Text(extent = {{33,9},{101, -5}}, lineColor = {0,0,0}, textString = "valve2"),Text(extent = {{34, -53},{102, -67}}, lineColor = {0,0,0}, textString = "valve3")}));
          connect(s1.outPort[1],T1.inPort) annotation(Line(points = {{ -51.5,40},{ -44,40}}, color = {0,0,0}));
          connect(T1.outPort,makeProduct.inPort) annotation(Line(points = {{ -38.5,40},{ -21,40}}, color = {0,0,0}));
          connect(makeProduct.outPort,T2.inPort) annotation(Line(points = {{10.5,40},{33,40}}, color = {0,0,0}));
          connect(T5.outPort,emptyTanks.inPort[1]) annotation(Line(points = {{5.5, -50},{21, -50}}, color = {0,0,0}));
          connect(emptyTanks.outPort[1],T6.inPort) annotation(Line(points = {{42.5, -50},{51, -50}}, color = {0,0,0}));
          connect(setValve1.y,valve1) annotation(Line(points = {{83,82.5},{90,82.5},{90,60},{105,60}}, color = {255,0,255}));
          connect(setValve2.y,valve2) annotation(Line(points = {{85.25, -78.5},{90, -78.5},{90,0},{105,0}}, color = {255,0,255}));
          connect(setValve3.y,valve3) annotation(Line(points = {{85.3, -90},{95, -90},{95, -60},{105, -60}}, color = {255,0,255}));
          connect(makeProduct.suspend[1],T3.inPort) annotation(Line(points = {{ -12.5,24.5},{ -12.5,12},{ -23,12},{ -23,3}}, color = {0,0,0}));
          connect(T3.outPort,s2.inPort[1]) annotation(Line(points = {{ -23, -2.5},{ -23, -20},{ -66, -20},{ -66, -50},{ -51, -50}}, color = {0,0,0}));
          connect(T4.outPort,makeProduct.resume[1]) annotation(Line(points = {{10,0.5},{10,15},{2.5,15},{2.5,24}}, color = {0,0,0}));
          connect(level1,makeProduct.level1) annotation(Line(points = {{ -60, -110},{ -60, -80},{ -80, -80},{ -80,20},{ -30,20},{ -30,28},{ -22,28}}, color = {0,0,255}));
          connect(s2.outPort[1],T5.inPort) annotation(Line(points = {{ -29.5, -49.75},{ -30, -49.75},{ -30, -50},{0, -50}}, color = {0,0,0}));
          connect(s2.outPort[2],T4.inPort) annotation(Line(points = {{ -29.5, -50.25},{ -29, -50},{ -8, -50},{ -8, -25},{10, -25},{10, -5}}, color = {0,0,0}));
          connect(T2.outPort,s1.inPort[1]) annotation(Line(points = {{38.5,40},{70,40},{70,70},{ -84,70},{ -84,40},{ -73,40},{ -73,40.5}}, color = {0,0,0}));
          connect(T6.outPort,s1.inPort[2]) annotation(Line(points = {{56.5, -50},{70, -50},{70,70},{ -84,70},{ -84,40},{ -74,40},{ -73,39.5}}, color = {0,0,0}));
        end TankController;
        model MakeProduct "State machine defining the time instants when to fill or empty a tank"
          extends StateGraph.PartialCompositeStep;
          parameter StateGraph.Temporary.SetRealParameter limit = 0.98 "Limit level of tank 1" annotation(Placement(transformation(extent = {{ -60,40},{ -20,60}}, rotation = 0)));
          parameter StateGraph.Temporary.SetRealParameter waitTime = 3 "Wait time" annotation(Placement(transformation(extent = {{ -120,40},{ -80,60}}, rotation = 0)));
          Modelica.Blocks.Interfaces.RealInput level1 annotation(Placement(transformation(extent = {{ -190, -140},{ -150, -100}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -150, -150},{150,150}}), graphics));
          Step fillTank1 annotation(Placement(transformation(extent = {{ -140, -10},{ -120,10}}, rotation = 0)));
          Transition T1(condition = level1 > limit) annotation(Placement(transformation(extent = {{ -110, -10},{ -90,10}}, rotation = 0)));
          Step fillTank2 annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          Transition T3(condition = level1 < 0.001) annotation(Placement(transformation(extent = {{20, -10},{40,10}}, rotation = 0)));
          Step emptyTank2 annotation(Placement(transformation(extent = {{120, -10},{140,10}}, rotation = 0)));
          Step wait1 annotation(Placement(transformation(extent = {{ -80, -10},{ -60,10}}, rotation = 0)));
          Transition T2(enableTimer = true, waitTime = waitTime) annotation(Placement(transformation(extent = {{ -50, -10},{ -30,10}}, rotation = 0)));
          Step wait2 annotation(Placement(transformation(extent = {{54, -10},{74,10}}, rotation = 0)));
          Transition T4(enableTimer = true, waitTime = waitTime) annotation(Placement(transformation(extent = {{82, -10},{102,10}}, rotation = 0)));

        equation 
          connect(fillTank1.inPort[1],inPort) annotation(Line(points = {{ -141,0},{ -160,0}}, color = {0,0,0}));
          connect(fillTank1.outPort[1],T1.inPort) annotation(Line(points = {{ -119.5,0},{ -104,0}}, color = {0,0,0}));
          connect(fillTank2.outPort[1],T3.inPort) annotation(Line(points = {{10.5,0},{26,0}}, color = {0,0,0}));
          connect(emptyTank2.outPort[1],outPort) annotation(Line(points = {{140.5,0},{155,0}}, color = {0,0,0}));
          connect(wait1.outPort[1],T2.inPort) annotation(Line(points = {{ -59.5,0},{ -44,0}}, color = {0,0,0}));
          connect(T2.outPort,fillTank2.inPort[1]) annotation(Line(points = {{ -38.5,0},{ -11,0}}, color = {0,0,0}));
          connect(T1.outPort,wait1.inPort[1]) annotation(Line(points = {{ -98.5,0},{ -81,0}}, color = {0,0,0}));
          connect(wait2.outPort[1],T4.inPort) annotation(Line(points = {{74.5,0},{88,0}}, color = {0,0,0}));
          connect(T3.outPort,wait2.inPort[1]) annotation(Line(points = {{31.5,0},{53,0}}, color = {0,0,0}));
          connect(T4.outPort,emptyTank2.inPort[1]) annotation(Line(points = {{93.5,0},{119,0}}, color = {0,0,0}));
        end MakeProduct;
        connector inflow1 "Inflow connector (this is a copy from Isolde Dressler's master thesis project)"
          import Units = Modelica.SIunits;
          input Units.VolumeFlowRate Fi "inflow";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -100, -100},{0,100},{100, -100},{ -100, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}));
        end inflow1;
        connector inflow2 "Inflow connector (this is a copy from Isolde Dressler's master thesis project)"
          import Units = Modelica.SIunits;
          output Units.VolumeFlowRate Fi "inflow";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -100, -100},{0,100},{100, -100},{ -100, -100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}));
        end inflow2;
        connector outflow1 "Outflow connector (this is a copy from Isolde Dressler's master thesis project)"
          import Units = Modelica.SIunits;
          output Units.VolumeFlowRate Fo "outflow";
          input Boolean open "valve open";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -100,100},{0, -100},{100,100},{ -100,100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}));
        end outflow1;
        connector outflow2 "Outflow connector (this is a copy from Isolde Dressler's master thesis project)"
          import Units = Modelica.SIunits;
          input Units.VolumeFlowRate Fo "outflow";
          output Boolean open "valve open";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -100,100},{0, -100},{100,100},{ -100,100}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}));
        end outflow2;
        model valve "Simple valve model (this is a copy from Isolde Dressler's master thesis project)"
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{0, -60},{0,0}}, color = {255,0,255})}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{20,20},{20,20}}, color = {0,0,0}, thickness = 0.5),Text(extent = {{ -131,125},{136,67}}, lineColor = {0,0,255}, textString = "%name"),Line(points = {{0,0},{0, -60}}, color = {255,0,255})}));
          Modelica.Blocks.Interfaces.BooleanInput valveControl annotation(Placement(transformation(origin = {0, -80}, extent = {{ -20, -20},{20,20}}, rotation = 90)));
          Modelica.StateGraph.Examples.Utilities.inflow2 inflow1 annotation(Placement(transformation(origin = {50,0}, extent = {{ -50, -50},{50,50}}, rotation = 90)));
          Modelica.StateGraph.Examples.Utilities.outflow2 outflow1 annotation(Placement(transformation(origin = { -50,0}, extent = {{ -50, -50},{50,50}}, rotation = 90)));

        equation 
          outflow1.Fo = inflow1.Fi;
          outflow1.open = valveControl;
        end valve;
        model Tank "Simple tank model (this is a copy from Isolde Dressler's master thesis project)"
          Modelica.Blocks.Interfaces.RealOutput levelSensor annotation(Placement(transformation(extent = {{ -61, -30},{ -81, -10}}, rotation = 0)));
          Modelica.StateGraph.Examples.Utilities.inflow1 inflow1 annotation(Placement(transformation(extent = {{ -55,60},{ -45,70}}, rotation = 0)));
          Modelica.StateGraph.Examples.Utilities.outflow1 outflow1 annotation(Placement(transformation(extent = {{55, -50},{65, -40}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -122, -82},{88, -42}}, textString = "%name", lineColor = {0,0,255}),Rectangle(extent = {{ -60,60},{80, -40}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = DynamicSelect({{ -60, -40},{ -60, -40}}, {{ -60, -40},{80, -40 + level * 100}}), lineColor = {0,0,0}, lineThickness = 0.5, fillPattern = FillPattern.HorizontalCylinder, fillColor = {191,0,95})}));
          Real level "Tank level in % of max height";
          parameter Real A = 1 "ground area of tank in m²";
          parameter Real a = 0.2 "area of drain hole in m²";
          parameter Real hmax = 1 "max height of tank in m";
          constant Real g = Modelica.Constants.g_n;

        equation 
          der(level) = (inflow1.Fi - outflow1.Fo) / (hmax * A);
          if outflow1.open then
            outflow1.Fo = sqrt(max(0, 2 * g * hmax * level)) * a;

          else
            outflow1.Fo = 0;

          end if;
          levelSensor = level;
        end Tank;
        model Source "Simple source model (this is a copy from Isolde Dressler's master thesis project)"
          Modelica.StateGraph.Examples.Utilities.outflow1 outflow1 annotation(Placement(transformation(extent = {{ -10, -60},{10, -40}}, rotation = 0)));
          parameter Real maxflow = 1 "maximal flow out of source";
          annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -80,40},{80, -40}}, lineColor = {0,0,0}, lineThickness = 0.5, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -144,54},{152,114}}, textString = "%name", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics));

        equation 
          if outflow1.open then
            outflow1.Fo = maxflow;

          else
            outflow1.Fo = 0;

          end if;
        end Source;
        model CompositeStep "State machine demonstrating a composite step (used in StateGraph.Examples.ShowCompositeStep)"
          extends StateGraph.PartialCompositeStep;
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -150, -150},{150,150}}), graphics));
          Transition transition3(enableTimer = true, waitTime = 1) annotation(Placement(transformation(extent = {{ -64,50},{ -44,70}}, rotation = 0)));
          Transition transition4(enableTimer = true, waitTime = 1) annotation(Placement(transformation(extent = {{ -64, -10},{ -44,10}}, rotation = 0)));
          Step step3 annotation(Placement(transformation(extent = {{ -10,50},{10,70}}, rotation = 0)));
          Step step4 annotation(Placement(transformation(extent = {{ -10, -10},{10,10}}, rotation = 0)));
          Transition transition5(enableTimer = true, waitTime = 2) annotation(Placement(transformation(extent = {{36,50},{56,70}}, rotation = 0)));
          Transition transition6(enableTimer = true, waitTime = 2) annotation(Placement(transformation(extent = {{36, -10},{56,10}}, rotation = 0)));
          Transition transition4a(enableTimer = true, waitTime = 1) annotation(Placement(transformation(extent = {{ -64, -70},{ -44, -50}}, rotation = 0)));
          Step step4a annotation(Placement(transformation(extent = {{ -10, -70},{10, -50}}, rotation = 0)));
          Transition transition6a(enableTimer = true, waitTime = 2) annotation(Placement(transformation(extent = {{36, -70},{56, -50}}, rotation = 0)));
          Step initStep annotation(Placement(transformation(extent = {{ -140, -10},{ -120,10}}, rotation = 0)));
          Step exitStep annotation(Placement(transformation(extent = {{120, -10},{140,10}}, rotation = 0)));
          Alternative Alternative1(nBranches = 3) annotation(Placement(transformation(extent = {{ -100, -90},{96,90}}, rotation = 0)));

        equation 
          connect(transition3.outPort,step3.inPort[1]) annotation(Line(points = {{ -52.5,60},{ -11,60}}, color = {0,0,0}));
          connect(step3.outPort[1],transition5.inPort) annotation(Line(points = {{10.5,60},{42,60}}, color = {0,0,0}));
          connect(transition4.outPort,step4.inPort[1]) annotation(Line(points = {{ -52.5,0},{ -11,0}}, color = {0,0,0}));
          connect(step4.outPort[1],transition6.inPort) annotation(Line(points = {{10.5,0},{42,0}}, color = {0,0,0}));
          connect(transition4a.outPort,step4a.inPort[1]) annotation(Line(points = {{ -52.5, -60},{ -11, -60}}, color = {0,0,0}));
          connect(step4a.outPort[1],transition6a.inPort) annotation(Line(points = {{10.5, -60},{42, -60}}, color = {0,0,0}));
          connect(initStep.inPort[1],inPort) annotation(Line(points = {{ -141,0},{ -160,0}}, color = {0,0,0}));
          connect(exitStep.outPort[1],outPort) annotation(Line(points = {{140.5,0},{155,0}}, color = {0,0,0}));
          connect(initStep.outPort[1],Alternative1.inPort) annotation(Line(points = {{ -119.5,0},{ -102.94,0}}, color = {0,0,0}));
          connect(Alternative1.outPort,exitStep.inPort[1]) annotation(Line(points = {{97.96,0},{119,0}}, color = {0,0,0}));
          connect(transition3.inPort,Alternative1.split[1]) annotation(Line(points = {{ -58,60},{ -79.42,60}}, color = {0,0,0}));
          connect(transition4.inPort,Alternative1.split[2]) annotation(Line(points = {{ -58,0},{ -67.975,0},{ -67.975, -7.10543e-15},{ -79.42, -7.10543e-15}}, color = {0,0,0}));
          connect(transition4a.inPort,Alternative1.split[3]) annotation(Line(points = {{ -58, -60},{ -67.975, -60},{ -67.975, -60},{ -79.42, -60}}, color = {0,0,0}));
          connect(transition5.outPort,Alternative1.join[1]) annotation(Line(points = {{47.5,60},{75.42,60}}, color = {0,0,0}));
          connect(transition6.outPort,Alternative1.join[2]) annotation(Line(points = {{47.5,0},{60.725,0},{60.725, -7.10543e-15},{75.42, -7.10543e-15}}, color = {0,0,0}));
          connect(transition6a.outPort,Alternative1.join[3]) annotation(Line(points = {{47.5, -60},{60.725, -60},{60.725, -60},{75.42, -60}}, color = {0,0,0}));
        end CompositeStep;
        model CompositeStep1 "Composite step used to demonstrate exceptions (in StateGraph.Examples.ShowExceptions)"
          extends PartialCompositeStep;
          Transition transition1(enableTimer = false, waitTime = 0, condition = time > 8) annotation(Placement(transformation(extent = {{ -60,20},{ -40,40}}, rotation = 0)));
          Step initStep annotation(Placement(transformation(extent = {{ -140, -10},{ -120,10}}, rotation = 0)));
          Step exitStep annotation(Placement(transformation(extent = {{110, -10},{130,10}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -150, -150},{150,150}}, grid = {1,1}), graphics));
          CompositeStep2 compositeStep11(waitTime = 3) annotation(Placement(transformation(extent = {{ -20,15},{10,45}}, rotation = 0)));
          CompositeStep2 compositeStep12(waitTime = 2) annotation(Placement(transformation(extent = {{ -20, -45},{10, -15}}, rotation = 0)));
          Transition transition2(condition = time > 4, enableTimer = false, waitTime = 0) annotation(Placement(transformation(extent = {{ -61, -40},{ -41, -20}}, rotation = 0)));
          Transition transition3(enableTimer = false, waitTime = 0) annotation(Placement(transformation(extent = {{29,20},{49,40}}, rotation = 0)));
          Transition transition4(enableTimer = false, waitTime = 0) annotation(Placement(transformation(extent = {{29, -40},{49, -20}}, rotation = 0)));
          Alternative Alternative1 annotation(Placement(transformation(extent = {{ -100, -60},{89,60}}, rotation = 0)));

        equation 
          connect(exitStep.outPort[1],outPort) annotation(Line(points = {{130.5,0},{155,0}}, color = {0,0,0}));
          connect(initStep.inPort[1],inPort) annotation(Line(points = {{ -141,0},{ -160,0}}, color = {0,0,0}));
          connect(transition1.outPort,compositeStep11.inPort) annotation(Line(points = {{ -48.5,30},{ -21,30}}, color = {0,0,0}));
          connect(transition2.outPort,compositeStep12.inPort) annotation(Line(points = {{ -49.5, -30},{ -21, -30}}, color = {0,0,0}));
          connect(compositeStep11.outPort,transition3.inPort) annotation(Line(points = {{10.5,30},{35,30}}, color = {0,0,0}));
          connect(compositeStep12.outPort,transition4.inPort) annotation(Line(points = {{10.5, -30},{35, -30}}, color = {0,0,0}));
          connect(initStep.outPort[1],Alternative1.inPort) annotation(Line(points = {{ -119.5,0},{ -102.835,0}}, color = {0,0,0}));
          connect(Alternative1.outPort,exitStep.inPort[1]) annotation(Line(points = {{90.89,0},{109,0}}, color = {0,0,0}));
          connect(transition1.inPort,Alternative1.split[1]) annotation(Line(points = {{ -54,30},{ -80.155,30}}, color = {0,0,0}));
          connect(transition2.inPort,Alternative1.split[2]) annotation(Line(points = {{ -55, -30},{ -80.155, -30}}, color = {0,0,0}));
          connect(transition3.outPort,Alternative1.join[1]) annotation(Line(points = {{40.5,30},{69.155,30}}, color = {0,0,0}));
          connect(transition4.outPort,Alternative1.join[2]) annotation(Line(points = {{40.5, -30},{69.155, -30}}, color = {0,0,0}));
        end CompositeStep1;
        model CompositeStep2 "Composite step used to demonstrate exceptions (in StateGraph.Examples.ShowExceptions)"
          extends PartialCompositeStep;
          Transition transition(enableTimer = true, waitTime = waitTime) annotation(Placement(transformation(extent = {{ -30, -10},{ -10,10}}, rotation = 0)));
          Step initStep annotation(Placement(transformation(extent = {{ -140, -10},{ -120,10}}, rotation = 0)));
          Step exitStep annotation(Placement(transformation(extent = {{110, -10},{130,10}}, rotation = 0)));
          annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -150, -150},{150,150}}, grid = {1,1}), graphics));
          parameter Temporary.SetRealParameter waitTime = 2 "waiting time in this composite step" annotation(Placement(transformation(extent = {{ -80,30},{ -50,50}}, rotation = 0)));

        equation 
          connect(exitStep.outPort[1],outPort) annotation(Line(points = {{130.5,0},{155,0}}, color = {0,0,0}));
          connect(initStep.inPort[1],inPort) annotation(Line(points = {{ -141,0},{ -160,0}}, color = {0,0,0}));
          connect(initStep.outPort[1],transition.inPort) annotation(Line(points = {{ -119.5,0},{ -24,0}}, color = {0,0,0}));
          connect(transition.outPort,exitStep.inPort[1]) annotation(Line(points = {{ -18.5,0},{109,0}}, color = {0,0,0}));
        end CompositeStep2;
      end Utilities;
      annotation(Documentation(info = "<html>

</html>"));
    end Examples;
    package Interfaces "Connectors and partial models"
      extends Modelica.Icons.Library;
      connector Step_in "Input port of a step"
        output Boolean occupied "true, if step is active" annotation(HideResult = true);
        input Boolean set "true, if transition fires and step is activated" annotation(HideResult = true);
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -100,100},{100,0},{ -100, -100},{ -100,100}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{0,50},{100,0},{0, -50},{0,50}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -141,100},{100,60}}, lineColor = {0,0,0}, textString = "%name")}), Documentation(info = "<html>

</html>"));
      end Step_in;
      connector Step_out "Output port of a step"
        output Boolean available "true, if step is active" annotation(HideResult = true);
        input Boolean reset "true, if transition fires and step is deactivated" annotation(HideResult = true);
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,50},{0, -50}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -100,100},{186,58}}, lineColor = {0,0,0}, textString = "%name")}), Documentation(info = "<html>

</html>"));
      end Step_out;
      connector Transition_in "Input port of a transition"
        input Boolean available "true, if step connected to the transition input is active" annotation(HideResult = true);
        output Boolean reset "true, if transition fires and the step connected to the transition input is deactivated" annotation(HideResult = true);
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{ -100,100},{100,0},{ -100, -100},{ -100,100}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Polygon(points = {{0,50},{100,0},{0, -50},{0,50}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -141,100},{100,60}}, lineColor = {0,0,0}, textString = "%name")}), Documentation(info = "<html>

</html>"));
      end Transition_in;
      connector Transition_out "Output port of a transition"
        input Boolean occupied "true, if step connected to the transition output is active" annotation(HideResult = true);
        output Boolean set "true, if transition fires and step connected to the transition output becomes active" annotation(HideResult = true);
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -100,100},{146,60}}, lineColor = {0,0,0}, textString = "%name"),Rectangle(extent = {{ -100,50},{0, -50}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>

</html>"));
      end Transition_out;
      connector CompositeStep_resume "Input port of a step (used for resume connector of a CompositeStep)"
        output Boolean occupied "true, if step is active" annotation(HideResult = true);
        input Boolean set "true, if transition fires and step is activated" annotation(HideResult = true);
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Polygon(points = {{ -100,100},{100,0},{ -100, -100},{ -100,100}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{0,0},{0,0}}, lineColor = {255,255,255}, pattern = LinePattern.None)}), Documentation(info = "<html>

</html>"));
      end CompositeStep_resume;
      connector CompositeStep_suspend "Output port of a step (used for suspend connector of a CompositeStep)"
        output Boolean available "true, if step is active" annotation(HideResult = true);
        input Boolean reset "true, if transition fires and step is deactived" annotation(HideResult = true);
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{0,0},{0,0}}, lineColor = {255,255,255}, pattern = LinePattern.None)}), Documentation(info = "<html>

</html>"));
      end CompositeStep_suspend;
      connector CompositeStepStatePort_in "Communication port between a CompositeStep and the ordinary steps within the CompositeStep (suspend/resume are inputs)"
        annotation(Documentation(info = "<html>

</html>"));
        input Boolean suspend "= true, if suspend transition of CompositeStep fires";
        input Boolean resume "= true, if resume transition of CompositeStep fires";
        Real activeStepsDummy "Dummy variable in order that connector fulfills restriction of connector" annotation(HideResult = true);
        flow Real activeSteps "Number of active steps in the CompositeStep";
      end CompositeStepStatePort_in;
      connector CompositeStepStatePort_out "Communication port between a CompositeStep and the ordinary steps within the CompositeStep (suspend/resume are outputs)"
        output Boolean suspend "= true, if suspend transition of CompositeStep fires";
        output Boolean resume "= true, if resume transition of CompositeStep fires";
        Real activeStepsDummy "Dummy variable in order that connector fulfills restriction of connector" annotation(HideResult = true);
        flow Real activeSteps "Number of active steps in the CompositeStep";
        annotation(Documentation(info = "<html>

</html>"));
      end CompositeStepStatePort_out;
      partial block PartialStep "Partial step with one input and one output transition port"
        annotation(Documentation(info = "<html>

</html>"));
        parameter Integer nIn(min = 0) = 1 "Number of input connections";
        parameter Integer nOut(min = 0) = 1 "Number of output connections";
        output Boolean localActive "= true if step is active, otherwise the step is not active" annotation(HideResult = true);
        Interfaces.Step_in inPort[nIn] "Vector of step input connectors" annotation(Placement(transformation(extent = {{ -120,10},{ -100, -10}}, rotation = 0)));
        Interfaces.Step_out outPort[nOut] "Vector of step output connectors" annotation(Placement(transformation(extent = {{100,5},{110, -5}}, rotation = 0)));
      protected 
        outer Interfaces.CompositeStepState stateGraphRoot;
        model OuterStatePort
          CompositeStepStatePort_in subgraphStatePort;
        end OuterStatePort;
        OuterStatePort outerStatePort;
        Boolean newActive "Value of active in the next iteration" annotation(HideResult = true);
        Boolean oldActive "Value of active when CompositeStep was aborted";

      initial equation 
        pre(newActive) = pre(localActive);
        pre(oldActive) = pre(localActive);

      equation 
        connect(outerStatePort.subgraphStatePort,stateGraphRoot.subgraphStatePort);
        for i in 1:nIn loop
        assert(cardinality(inPort[i]) <= 1, "Connector is connected to more than one transition (this is not allowed)");

        end for;
        for i in 1:nOut loop
        assert(cardinality(outPort[i]) <= 1, "Connector is connected to more than one transition (this is not allowed)");

        end for;
        localActive = pre(newActive);
        newActive = if outerStatePort.subgraphStatePort.resume then oldActive else (StateGraph.Temporary.anyTrue(inPort.set) or localActive and not StateGraph.Temporary.anyTrue(outPort.reset)) and not outerStatePort.subgraphStatePort.suspend;
        when outerStatePort.subgraphStatePort.suspend then
                  oldActive = localActive;
        
        end when;
        outerStatePort.subgraphStatePort.activeSteps = if localActive then 1.0 else 0.0;
        for i in 1:nIn loop
        inPort[i].occupied = if i == 1 then localActive else inPort[i - 1].occupied or inPort[i - 1].set;

        end for;
        for i in 1:nOut loop
        outPort[i].available = if i == 1 then localActive else outPort[i - 1].available and not outPort[i - 1].reset;

        end for;
        for i in 1:nIn loop
if cardinality(inPort[i]) == 0 then
                  inPort[i].set = false;
        end if;

        end for;
        for i in 1:nOut loop
if cardinality(outPort[i]) == 0 then
                  outPort[i].reset = false;
        end if;

        end for;
        annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Coordsys(grid = [1,1], component = [20,20]));
      end PartialStep;
      partial block PartialTransition "Partial transition with input and output connections"
        input Boolean localCondition "= true, if transition may fire" annotation(HideResult = true);
        parameter Boolean enableTimer = false "= true, if timer is enabled" annotation(Evaluate = true, Dialog(group = "Timer"));
        parameter Modelica.SIunits.Time waitTime(min = 0) = 0 "Wait time before transition fires" annotation(Dialog(group = "Timer", enable = enableTimer));
        output Modelica.SIunits.Time t "= actual waiting time (transition will fire when t > waitTime)";
        output Boolean enableFire "= true, if all firing conditions are true";
        output Boolean fire "= true, if transition fires" annotation(HideResult = true);
        StateGraph.Interfaces.Transition_in inPort "Vector of transition input connectors" annotation(Placement(transformation(extent = {{ -50, -10},{ -30,10}}, rotation = 0)));
        StateGraph.Interfaces.Transition_out outPort "Vector of transition output connectors" annotation(Placement(transformation(extent = {{10, -5},{20,5}}, rotation = 0)));
      protected 
        Modelica.SIunits.Time t_start "Time instant at which the transition would fire, if waitTime would be zero";
        Real t_dummy;

      initial equation 
        pre(enableFire) = false;

      equation 
        assert(cardinality(inPort) == 1, "Connector inPort is not connected to exactly one other connector");
        assert(cardinality(outPort) == 1, "Connector outPort is not connected to exactly one other connector");
        if enableTimer then
          when enableFire then
                      t_start = time;
          
          end when;
          t_dummy = time - t_start;
          t = if enableFire then t_dummy else 0;
          fire = enableFire and time >= t_start + waitTime;

        else
          t_start = 0;
          t_dummy = 0;
          t = 0;
          fire = enableFire;

        end if;
        enableFire = localCondition and inPort.available and not outPort.occupied;
        inPort.reset = fire;
        outPort.set = fire;
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Documentation(info = "<html>

</html>"));
      end PartialTransition;
      partial block PartialStateGraphIcon "Icon for a StateGraph object"
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{160,110},{ -160,150}}, textString = "%name", lineColor = {0,0,255})}), Documentation(info = "<html>

</html>"));
      end PartialStateGraphIcon;
      model CompositeStepState "Communication channel between CompositeSteps and steps in the CompositeStep"
        output Boolean suspend = false;
        output Boolean resume = false;
        CompositeStepStatePort_out subgraphStatePort;
        annotation(defaultComponentName = "stateGraphRoot", defaultComponentPrefixes = "inner", missingInnerMessage = "A \"stateGraphRoot\" component was automatically introduced.", Documentation(info = "<html>

</html>"));

      equation 
        suspend = subgraphStatePort.suspend;
        resume = subgraphStatePort.resume;
        subgraphStatePort.activeStepsDummy = 0;
      end CompositeStepState;
      annotation(Documentation(info = "<html>

</html>"));
    end Interfaces;
    block InitialStep "Initial step (= step that is active when simulation starts)"
      output Boolean active "= true if step is active, otherwise the step is not active";
      extends Interfaces.PartialStep;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -200,110},{200,150}}, textString = "%name", lineColor = {0,0,255}),Rectangle(extent = {{ -100,100},{100, -100}}, fillColor = DynamicSelect({255,255,255}, if active > 0.5 then {0,255,0} else {255,255,255}), lineColor = {0,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -80,80},{80, -80}}, lineColor = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}),Rectangle(extent = {{ -80,80},{80, -80}}, lineColor = {0,0,0})}), Documentation(info = "<html>

</html>"));

    initial equation 
      active = true;

    equation 
      active = localActive;
    end InitialStep;
    block InitialStepWithSignal "Initial step (= step that is active when simulation starts). Connector 'active' is true when the step is active"
      extends Interfaces.PartialStep;
      annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}),Rectangle(extent = {{ -80,80},{80, -80}}, lineColor = {0,0,0})}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -200,110},{200,150}}, textString = "%name", lineColor = {0,0,255}),Rectangle(extent = {{ -100,100},{100, -100}}, fillColor = DynamicSelect({255,255,255}, if active > 0.5 then {0,255,0} else {255,255,255}), lineColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -92, -50},{94, -68}}, lineColor = {0,0,0}, textString = "active"),Rectangle(extent = {{ -80,80},{80, -80}}, lineColor = {0,0,0})}), Documentation(info = "<html>

</html>"));
      Modelica.Blocks.Interfaces.BooleanOutput active annotation(Placement(transformation(origin = {0, -110}, extent = {{ -10, -10},{10,10}}, rotation = 270)));

    initial equation 
      active = true;

    equation 
      active = localActive;
    end InitialStepWithSignal;
    block Step "Ordinary step (= step that is not active when simulation starts)"
      output Boolean active "= true if step is active, otherwise the step is not active";
      extends Interfaces.PartialStep;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{ -200,110},{200,150}}, textString = "%name", lineColor = {0,0,255}),Rectangle(extent = {{ -100,100},{100, -100}}, fillColor = DynamicSelect({255,255,255}, if active > 0.5 then {0,255,0} else {255,255,255}), lineColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0})}), Documentation(info = "<html>

</html>"));

    initial equation 
      active = false;

    equation 
      active = localActive;
    end Step;
    block StepWithSignal "Ordinary step (= step that is not active when simulation starts). Connector 'active' is true when the step is active"
      extends Interfaces.PartialStep;
      annotation(Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0})}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -200,110},{200,150}}, textString = "%name", lineColor = {0,0,255}),Rectangle(extent = {{ -100,100},{100, -100}}, fillColor = DynamicSelect({255,255,255}, if active > 0.5 then {0,255,0} else {255,255,255}), lineColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -92, -74},{94, -92}}, lineColor = {0,0,0}, textString = "active")}), Documentation(info = "<html>

</html>"));
      Modelica.Blocks.Interfaces.BooleanOutput active annotation(Placement(transformation(origin = {0, -110}, extent = {{ -10, -10},{10,10}}, rotation = 270)));

    initial equation 
      active = false;

    equation 
      active = localActive;
    end StepWithSignal;
    block Transition "Transition where the fire condition is set by a modification of variable condition"
      input Boolean condition = true "= true, if transition may fire (time varying expression)" annotation(Dialog(group = "Fire condition"));
      extends Interfaces.PartialTransition(final localCondition = condition);
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -10,100},{10, -100}}, fillColor = DynamicSelect({0,0,0}, if enableFire > 0.5 then {0,255,0} else {0,0,0}), lineColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -30,0},{ -10,0}}, color = {0,0,0}),Text(extent = {{200,110},{ -200,150}}, textString = "%name", lineColor = {0,0,255}),Text(extent = {{20,20},{200,45}}, lineColor = DynamicSelect({0,0,0}, if enableTimer < 0.5 then {255,255,255} else {0,0,0}), textString = "%waitTime"),Text(extent = {{ -200, -120},{200, -145}}, lineColor = DynamicSelect({0,0,0}, if condition > 0.5 then {0,255,0} else {0,0,0}), textString = "%condition")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -31,0},{ -11,0}}, color = {0,0,0}),Rectangle(extent = {{ -10,100},{10, -100}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>

</html>"));
    end Transition;
    block TransitionWithSignal "Transition where the fire condition is set by a Boolean input signal"
      Modelica.Blocks.Interfaces.BooleanInput condition annotation(Placement(transformation(origin = {0, -120}, extent = {{ -20, -20},{20,20}}, rotation = 90)));
      extends Interfaces.PartialTransition(final localCondition = condition);
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Text(extent = {{20,20},{200,45}}, lineColor = DynamicSelect({0,0,0}, if enableTimer < 0.5 then {255,255,255} else {0,0,0}), textString = "%waitTime"),Rectangle(extent = {{ -10,100},{10, -100}}, fillColor = DynamicSelect({0,0,0}, if enableFire > 0.5 then {0,255,0} else {0,0,0}), lineColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -30,0},{ -10,0}}, color = {0,0,0}),Text(extent = {{200,110},{ -200,150}}, textString = "%name", lineColor = {0,0,255}),Ellipse(extent = {{7, -81},{ -7, -95}}, lineColor = DynamicSelect({0,0,0}, if condition > 0.5 then {0,255,0} else {0,0,0}), fillColor = DynamicSelect({0,0,0}, if condition > 0.5 then {0,255,0} else {0,0,0}), fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -31,0},{ -11,0}}, color = {0,0,0}),Rectangle(extent = {{ -10,100},{10, -100}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>

</html>"));
    end TransitionWithSignal;
    block Alternative "Alternative splitting of execution path (use component between two steps)"
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -80,100},{80,100}}, color = {0,0,0}, pattern = LinePattern.Dot),Line(points = {{ -80, -100},{80, -100}}, color = {0,0,0}, pattern = LinePattern.Dot),Line(points = {{ -100,0},{ -80,0}}, color = {0,0,0}),Line(points = {{80,0},{100,0}}, color = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -100,0},{ -80,0}}, color = {0,0,0}),Line(points = {{80,0},{100,0}}, color = {0,0,0})}), Documentation(info = "<html>

</html>"));
      parameter Integer nBranches(min = 1) = 2 "Number of alternative branches";
      Interfaces.Transition_in inPort annotation(Placement(transformation(extent = {{ -106, -3},{ -100,3}}, rotation = 0)));
      Interfaces.Transition_out outPort annotation(Placement(transformation(extent = {{100, -2},{104,2}}, rotation = 0)));
      Step_in_forAlternative join[nBranches] annotation(Placement(transformation(extent = {{78,100},{80, -100}}, rotation = 0)));
      Step_out_forAlternative split[nBranches] annotation(Placement(transformation(extent = {{ -78,100},{ -80, -100}}, rotation = 0)));
    protected 
      connector Step_in_forAlternative "Input port of a step (has special icon for usage in component 'Alternative')"
        output Boolean occupied "true, if step is active" annotation(HideResult = true);
        input Boolean set "true, if transition fires and step is activated" annotation(HideResult = true);
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {175,175,175}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {175,175,175}, fillPattern = FillPattern.Solid)}));
      end Step_in_forAlternative;
      connector Step_out_forAlternative "Output port of a step (has special icon for usage in component 'Alternative')"
        output Boolean available "true, if step is active" annotation(HideResult = true);
        input Boolean reset "true, if transition fires and step is deactivated" annotation(HideResult = true);
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {175,175,175}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {175,175,175}, fillPattern = FillPattern.Solid)}));
      end Step_out_forAlternative;

    equation 
      assert(cardinality(inPort) == 1, "Connector inPort is not connected to exactly one other connector");
      assert(cardinality(outPort) == 1, "Connector outPort is not connected to exactly one other connector");
      for i in 1:nBranches loop
      assert(cardinality(split[i]) == 1, "Connector is not connected to exactly one other connector");
      assert(cardinality(join[i]) == 1, "Connector is not connected to exactly one other connector");

      end for;
      for i in 1:nBranches loop
      split[i].available = if i == 1 then inPort.available else split[i - 1].available and not split[i - 1].reset;

      end for;
      join.occupied = fill(outPort.occupied, nBranches);
      inPort.reset = StateGraph.Temporary.anyTrue(split.reset);
      outPort.set = StateGraph.Temporary.anyTrue(join.set);
    end Alternative;
    block Parallel "Parallel splitting of execution path (use component between two transitions)"
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -100,0},{ -80,0}}, color = {0,0,0}),Line(points = {{80,0},{100,0}}, color = {0,0,0}),Line(points = {{ -80,100},{80,100}}, color = {0,0,0}, pattern = LinePattern.Dot),Line(points = {{ -80, -100},{80, -100}}, color = {0,0,0}, pattern = LinePattern.Dot)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Line(points = {{ -100,0},{ -80,0}}, color = {0,0,0}),Line(points = {{80,0},{100,0}}, color = {0,0,0})}), Documentation(info = "<html>

</html>"));
      parameter Integer nBranches(min = 1) = 2 "Number of parallel branches that are executed in parallel";
      Interfaces.Step_in inPort annotation(Placement(transformation(extent = {{ -106, -3},{ -100,3}}, rotation = 0)));
      Interfaces.Step_out outPort annotation(Placement(transformation(extent = {{100, -2},{104,2}}, rotation = 0)));
      Transition_in_forParallel join[nBranches] annotation(Placement(transformation(extent = {{75,100},{80, -100}}, rotation = 0)));
      Transition_out_forParallel split[nBranches] annotation(Placement(transformation(extent = {{ -75,100},{ -80, -100}}, rotation = 0)));
    protected 
      connector Transition_in_forParallel "Input port of a transition (has special icon for usage in component 'Parallel')"
        input Boolean available "true, if step connected to the transition input is active" annotation(HideResult = true);
        output Boolean reset "true, if transition fires and the step connected to the transition input is deactivated" annotation(HideResult = true);
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -100,100},{ -100, -100}}, color = {0,0,0}, thickness = 0.5),Line(points = {{100,100},{100, -100}}, color = {0,0,0}, thickness = 0.5)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -100,100},{ -100, -100}}, color = {0,0,0}, thickness = 0.5),Line(points = {{100,100},{100, -100}}, color = {0,0,0}, thickness = 0.5)}));
      end Transition_in_forParallel;
      connector Transition_out_forParallel "Output port of a transition (has special icon for usage in component 'Parallel')"
        input Boolean occupied "true, if step connected to the transition output is active" annotation(HideResult = true);
        output Boolean set "true, if transition fires and step connected to the transition output becomes active" annotation(HideResult = true);
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -100,100},{ -100, -100}}, color = {0,0,0}, thickness = 0.5),Line(points = {{100,100},{100, -100}}, color = {0,0,0}, thickness = 0.5)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {255,255,255}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -100,100},{ -100, -100}}, color = {0,0,0}, thickness = 0.5),Line(points = {{100,100},{100, -100}}, color = {0,0,0}, thickness = 0.5)}));
      end Transition_out_forParallel;

    equation 
      assert(cardinality(inPort) == 1, "Connector inPort is not connected to exactly one other connector");
      assert(cardinality(outPort) == 1, "Connector outPort is not connected to exactly one other connector");
      for i in 1:nBranches loop
      assert(cardinality(split[i]) == 1, "Connector is not connected to exactly one other connector");
      assert(cardinality(join[i]) == 1, "Connector is not connected to exactly one other connector");

      end for;
      split.set = fill(inPort.set, nBranches);
      join.reset = fill(outPort.reset, nBranches);
      inPort.occupied = StateGraph.Temporary.anyTrue(split.occupied);
      outPort.available = StateGraph.Temporary.allTrue(join.available);
    end Parallel;
    partial model PartialCompositeStep "Superclass of a subgraph, i.e., a composite step that has internally a StateGraph"
      parameter Integer nSuspend = 1 "Number of suspend ports" annotation(Dialog(group = "Exception connections"));
      parameter Integer nResume = 1 "Number of resume ports" annotation(Dialog(group = "Exception connections"));
      inner outer StateGraph.Interfaces.CompositeStepState stateGraphRoot(suspend = StateGraph.Temporary.anyTrue(suspend.reset) or outerState.subgraphStatePort.suspend, resume = StateGraph.Temporary.anyTrue(resume.set) or outerState.subgraphStatePort.resume) "Communication port between the CompositeStep and the steps within the CompositeStep";
      output Boolean active "= true if step is active, otherwise the step is not active";
      StateGraph.Interfaces.Step_in inPort annotation(Placement(transformation(extent = {{ -170,10},{ -150, -10}}, rotation = 0)));
      StateGraph.Interfaces.Step_out outPort annotation(Placement(transformation(extent = {{150,5},{160, -5}}, rotation = 0)));
      StateGraph.Interfaces.CompositeStep_suspend suspend[nSuspend] annotation(Placement(transformation(origin = { -75, -155}, extent = {{ -5,5},{5, -5}}, rotation = 270)));
      StateGraph.Interfaces.CompositeStep_resume resume[nResume] annotation(Placement(transformation(origin = {75, -160}, extent = {{ -10, -10},{10,10}}, rotation = 90)));
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -150, -150},{150,150}}, grid = {1,1}), graphics = {Text(extent = {{ -250,160},{250,200}}, textString = "%name", lineColor = {0,0,255}),Rectangle(extent = {{ -150,150},{150, -150}}, fillColor = DynamicSelect({255,255,255}, if active > 0.5 then {0,255,0} else {255,255,255}), lineColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{4, -115},{145, -130}}, lineColor = {0,0,0}, textString = "resume"),Text(extent = {{ -144, -114},{ -3, -129}}, lineColor = {0,0,0}, textString = "suspend")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -150, -150},{150,150}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -150,150},{150, -150}}, lineColor = {0,0,0})}), Documentation(info = "<html>

</html>"));
      model OuterState "Block containing the port that is connected to the outer stateGraphRoot"
        Interfaces.CompositeStepStatePort_in subgraphStatePort "Port connected to outer stateGraphRoot";
      end OuterState;
      OuterState outerState;
    protected 
      model InnerState
        outer Interfaces.CompositeStepState stateGraphRoot;
      end InnerState;
      InnerState innerState;
      Boolean newActive "Value of active in the next iteration" annotation(HideResult = true);
      Integer activeSteps "Number of active steps within the CompositeStep";

    initial equation 
      pre(newActive) = pre(active);

    equation 
      connect(outerState.subgraphStatePort,stateGraphRoot.subgraphStatePort);
      outerState.subgraphStatePort.activeSteps = if active then 1.0 else 0.0;
      activeSteps =  -integer(innerState.stateGraphRoot.subgraphStatePort.activeSteps);
      active = pre(newActive);
      newActive = activeSteps > 0 and not StateGraph.Temporary.anyTrue(suspend.reset) and not outerState.subgraphStatePort.suspend or StateGraph.Temporary.anyTrue(resume.set) or outerState.subgraphStatePort.resume;
      for i in 1:nResume loop
      resume[i].occupied = if i == 1 then active else resume[i - 1].occupied or resume[i - 1].set;

      end for;
      for i in 1:nSuspend loop
      suspend[i].available = if i == 1 then active else suspend[i - 1].available and not suspend[i - 1].reset;

      end for;
      for i in 1:nSuspend loop
      assert(cardinality(suspend[i]) <= 1, "Connector suspend[" + String(i) + "] of the CompositeStep is connected
to more than one transition");
if cardinality(suspend[i]) == 0 then
              suspend[i].reset = false;
      end if;

      end for;
      for i in 1:nResume loop
      assert(cardinality(resume[i]) <= 1, "Connector resume[" + String(i) + "] of the CompositeStep is connected
to more than one transition");
if cardinality(resume[i]) == 0 then
              resume[i].set = false;
      end if;

      end for;
if cardinality(inPort) < 2 then
              inPort.occupied = false;
        inPort.set = false;
      end if;
if cardinality(outPort) < 2 then
              outPort.available = false;
        outPort.reset = false;
      end if;
      assert(cardinality(inPort) <= 2, "Connector inPort of the CompositeStep has more than 2 connections.
It should have only one connection from the outside to the
inPort and one connection to a step inside the CompositeStep.");
      assert(cardinality(outPort) <= 2, "Connector outPort of the CompositeStep has more than 2 connections.
It should have only one connection from the outPort to the
outside to the CompositeStep and one connection from a step
inside the CompositeStep to the outPort connector.");
    end PartialCompositeStep;
    model StateGraphRoot "Root of a StateGraph (has to be present on the highest level of a StateGraph)"
      extends StateGraph.Interfaces.CompositeStepState;
      output Integer activeSteps "Number of active steps within the stategraph";
      annotation(defaultComponentName = "stateGraphRoot", defaultComponentPrefixes = "inner", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -200,110},{200,150}}, textString = "%name", lineColor = {0,0,255}),Rectangle(extent = {{ -100,100},{100, -100}}, fillColor = DynamicSelect({255,255,255}, if active > 0.5 then {0,255,0} else {255,255,255}), lineColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -92,78},{96,34}}, lineColor = {0,0,0}, textString = "root"),Rectangle(extent = {{ -82, -6},{ -44, -40}}, lineColor = {0,0,0}),Line(points = {{0,10},{0, -60}}, color = {0,0,0}),Rectangle(extent = {{48, -6},{86, -40}}, lineColor = {0,0,0}),Polygon(points = {{ -12, -16},{0, -22},{ -12, -28},{ -12, -16}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{ -44, -22},{ -12, -22}}, color = {0,0,0}),Polygon(points = {{36, -16},{48, -22},{36, -28},{36, -16}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{0, -22},{36, -22}}, color = {0,0,0})}), Documentation(info = "<html>
<p>
On the highest level of a StateGraph, an instance of StateGraphRoot
has to be present. If it is not within in a model, it is automatically
included by a Modelica translator due to an appropriate annotation.
Practically, this means that it need not be present in a
StateGraph model.
</p>
<p>
The StateGraphRoot object is needed, since all Step objects have
an \"outer\" reference to communicate with the \"nearest\" CompositeStep
(which inherits from PartialCompositeStep), especially to abort
a CompositeStep via the \"suspend\" port. Even if no \"CompositeStep\" is present,
on highest level a corresponding \"inner\" definition is needed
and is provided by the StateGraphRoot object.
</p>
</html>"));

    equation 
      activeSteps =  -integer(subgraphStatePort.activeSteps);
    end StateGraphRoot;
    package Temporary "Components that will be provided by other libraries in the future"
      type SetRealParameter = Real "Define Real parameter (GUI not yet satisfactory)" annotation(Dialog, defaultComponentName = "name", defaultComponentPrefixes = "parameter", Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -100,40},{100, -40}}, lineColor = {0,0,0}, fillColor = {245,245,245}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised),Text(extent = {{ -150,90},{150,50}}, textString = "%name ="),Text(extent = {{ -96,15},{96, -15}}, lineColor = {0,0,0}, textString = "%value")}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics), Documentation(info = "<html>
<p>
This is an <b>experimental component</b> to define a
<b>Real parameter</b>
in the <b>diagram layer</b>. The idea is to drag the icon from the
package browser into the diagram layer. Then a window pops
up in which the properties of this parameter can be defined
(such as the default value). The name and default value of the
parameter are displayed in the icon of this component. Whenever
clicking on it, the dialog to change parameter settings pops-up.
</p>
<p>
In Dymola, the described property is not fully available.
Currently, when dragging this component in the diagram layer,
a dialog pops up in which the properties of the parameter
can be defined. However, afterwards, the parameter is not
visible in the diagram layer. Making it visible requires to
go into the text layer and add an annotation with the
component size, resulting for example in:
</p>
<pre>
  <b>parameter</b> StateGraph.SetRealParameter name = 2
                       <b>annotation</b>(extent=[-10,-10; 10,10]);
</pre>
<p>
This change makes the parameter icon visible in the
diagram layer. However, clicking on this icon has no
effect. Changing parameter properties, such as the default
value, still requires to go in to the text layer.
</p>
</html>"));
      extends Modelica.Icons.Library;
      function anyTrue "Returns true, if at least on element of the Boolean input vector is true"
        extends Modelica.Icons.Function;
        input Boolean b[:];
        output Boolean result;
      algorithm 
        result:=false;
        for i in 1:size(b, 1) loop
                  result:=result or b[i];

        end for;
        annotation(Documentation(info = "<html>

</html>"));
      end anyTrue;
      function allTrue "Returns true, if all elements of the Boolean input vector are true"
        extends Modelica.Icons.Function;
        input Boolean b[:];
        output Boolean result;
      algorithm 
        result:=true;
        for i in 1:size(b, 1) loop
                  result:=result and b[i];

        end for;
        annotation(Documentation(info = "<html>

</html>"));
      end allTrue;
      block RadioButton "Button that sets its output to true when pressed and is reset when an element of 'reset' becomes true"
        parameter Modelica.SIunits.Time buttonTimeTable[:] = {0} "Time instants where button is pressend and released";
        input Boolean reset[:] = {false} "Reset button to false, if an element of reset becomes true" annotation(Dialog(group = "Time varying expressions"));
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -100},{100,100}}, fillColor = DynamicSelect({192,192,192}, if on > 0.5 then {0,255,0} else {192,192,192}), fillPattern = DynamicSelect(FillPattern.Solid, if on > 0.5 then FillPattern.Solid else FillPattern.Solid), lineColor = {128,128,128}, lineThickness = 0.5),Text(extent = {{ -80, -40},{80,40}}, lineColor = {0,0,0}, textString = "%name")}, interaction = {OnMouseDownSetBoolean(on, true)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics), Documentation(info = "<html>

</html>"));
        Modelica.Blocks.Interfaces.BooleanOutput on annotation(Placement(transformation(extent = {{100, -10},{120,10}}, rotation = 0)));
      protected 
        Modelica.Blocks.Sources.BooleanTable table(table = buttonTimeTable);
      algorithm 
        on:=table.y;
        when pre(reset) then
                  on:=false;
        
        end when;
      end RadioButton;
      model NumericValue "Show value of Real input signal dynamically"
        parameter Integer precision(min = 0) = 3 "Number of significant digits to be shown";
        parameter Boolean hideConnector = false "= true, if connector is not shown in the dynamic object diagram";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{100,50},{ -100, -50}}, lineColor = {0,0,255}, fillColor = {236,233,216}, fillPattern = FillPattern.Solid, borderPattern = BorderPattern.Raised),Text(extent = {{ -90, -46},{90,34}}, textString = DynamicSelect(" ", realString(Value, 1, integer(precision))), lineColor = {0,0,255})}), Diagram(graphics), Documentation(info = "<html>

</html>"));
        Modelica.Blocks.Interfaces.RealInput Value "Real value to be shown in icon" annotation(Dialog(enable = hideConnector), Placement(visible = not boolean(hideConnector), transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
      end NumericValue;
      model IndicatorLamp "Dynamically show Boolean input signal (false/true = white/green color)"
        Modelica.Blocks.Interfaces.BooleanInput u annotation(Placement(transformation(extent = {{ -140, -20},{ -100,20}}, rotation = 0)));
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Ellipse(extent = {{ -100, -100},{100,100}}, fillColor = DynamicSelect({235,235,235}, if u > 0.5 then {0,255,0} else {235,235,235}), lineColor = {0,0,0}, pattern = LinePattern.None, fillPattern = FillPattern.Sphere),Text(extent = {{ -150,150},{150,110}}, textString = "%name", lineColor = {0,0,255})}), Diagram(graphics), Documentation(info = "<html>

</html>"));
      end IndicatorLamp;
      annotation(Documentation(info = "<html>
<p>
This library is just temporarily present. The components of
this library will be present in the future in the Modelica
standard library (with the new block connectors) and in the
UserInteraction library that is currently under development.
</p>
</html>"));
    end Temporary;
  end StateGraph;
  package SIunits "Library of type and unit definitions based on SI units according to ISO 31-1992"
    package UsersGuide "User's Guide of SIunits Library"
      annotation(__Dymola_DocumentationClass = true, Documentation(info = "<HTML>
<p>
Library <b>SIunits</b> is a <b>free</b> Modelica package providing
predefined types, such as <i>Mass</i>,
<i>Length</i>, <i>Time</i>, based on the international standard
on units.</p>

</HTML>"));
      class HowToUseSIunits "How to use SIunits"
        annotation(__Dymola_DocumentationClass = true, Documentation(info = "<html>
<p>
When implementing a Modelica model, every variable needs to
be declared. Physical variables should be declared with a unit.
The basic approach in Modelica is that the unit attribute of
a variable is the <b>unit</b> in which the <b>equations</b> are <b>written</b>,
for example:
</p>

<pre>   <b>model</b> MassOnGround
     <b>parameter</b> Real m(quantity=\"Mass\", unit=\"kg\") \"Mass\";
     <b>parameter</b> Real f(quantity=\"Force\", unit=\"N\") \"Driving force\";
     Real s(unit=\"m\") \"Position of mass\";
     Real v(unit=\"m/s\") \"Velocity of mass\";
   <b>equation</b>
     <b>der</b>(s) = v;
     m*<b>der</b>(v) = f;
   <b>end</b> MassOnGround;
</pre>

<p>
This means that the equations in the equation section are only correct
for the specified units. A different issue is the user interface, i.e.,
in which unit the variable is presented to the user in graphical
user interfaces, both for input (e.g., parameter menu), as well as
for output (e.g., in the plot window). Preferably, the Modelica tool
should provide a list of units from which the user can select, e.g.,
\"m\", \"cm\", \"km\", \"inch\" for quantity \"Length\". When storing the value in
the model as a Modelica modifier, it has to be converted to the unit defined
in the declaration. Additionally, the unit used in the graphical
user interface has to be stored. In order to have a standardized way
of doing this, Modelica provides the following three attributs
for a variable of type Real:
</p>

<ul>
<li><b>quantity</b> to define the physical quantity (e.g. \"Length\", or \"Energy\").</li>
<li><b>unit</b> to define the unit that has to be used
    in order that the equations are correct (e.g. \"N.m\").</li>
<li><b>displayUnit</b> to define the unit used in the graphical
    user interface as default display unit for input and/or output.</li>
</ul>

<p>
Note, a unit, such as \"N.m\", is not sufficient to define uniquely the
physical quantity, since, e.g., \"N.m\" might be either \"torque\" or
\"energy\". The \"quantity\" attribute might therefore be used by a tool
to select the corresponding menu from which the user can select
a unit for the corresponding variable.
</p>

<p>
For example, after providing a value for \"m\" and \"f\" in a parameter
menu of an instance of MassOnGround, a tool might generate the following code:
</p>

<pre>
   MassOnGround myObject(m(displayUnit=\"g\")=2, f=3);
</pre>

<p>
The meaning is that in the equations a value of \"2\" is used
and that in the graphical user interface a value of \"2000\" should be used,
together with the unit \"g\" from the unit set \"Mass\" (= the quantity name).
Note, according to the Modelica specification
a tool might ignore the \"displayUnit\" attribute.
</p>

<p>
In order to help the Modelica model developer, the Modelica.SIunits
library provides about 450 predefined type names,
together with values for the attributes <b>quantity</b>, <b>unit</b> and sometimes
<b>displayUnit</b> and <b>min</b>. The unit is always selected as SI-unit according to the
ISO standard. The type and the quantity names are the
quantity names used in the ISO standard. \"quantity\" and \"unit\" are defined
as \"<b>final</b>\" in order that they cannot be modified. Attributes \"displayUnit\"
and \"min\" can, however, be changed in a model via a modification. The example above,
might therefore be alternatively also defined as:
</p>

<pre>   <b>model</b> MassOnGround
     <b>parameter</b> Modelica.SIunits.Mass  m \"Mass\";
     <b>parameter</b> Modelica.SIunits.Force f \"Driving force\";
     ...
   <b>end</b> MassOnGround;
</pre>

<p>
or in a short hand notation as
</p>

<pre>
   <b>model</b> MassOnGround
     <b>import</b> SI = Modelica.SIunits;
     <b>parameter</b> SI.Mass  m \"Mass\";
     <b>parameter</b> SI.Force f \"Driving force\";
     ...
   <b>end</b> MassOnGround;
</pre>

<p>
For some often
used Non SI-units (like hour), some additional type definitions are
present as Modelica.SIunits.Conversions.NonSIunits. If this is not sufficient,
the user has to define its own types or use the attributes directly
in the declaration as in the example at the beginning.
</p>

</html>"));
      end HowToUseSIunits;
      class Conventions "Conventions"
        annotation(__Dymola_DocumentationClass = true, Documentation(info = "<html>
<p>The following conventions are used in package SIunits:</p>
<ul>
<li>Modelica quantity names are defined according to the recommendations
    of ISO 31. Some of these name are rather long, such as
    \"ThermodynamicTemperature\". Shorter alias names are defined, e.g.,
    \"type Temp_K = ThermodynamicTemperature;\".</li>
<li>Modelica units are defined according to the SI base units without
    multiples (only exception \"kg\").</li>
<li>For some quantities, more convenient units for an engineer are
    defined as \"displayUnit\", i.e., the default unit for display
    purposes (e.g., displayUnit=\"deg\" for quantity=\"Angle\").</li>
<li>The type name is identical to the quantity name, following
    the convention of type names.</li>
<li>All quantity and unit attributes are defined as final in order
    that they cannot be redefined to another value.</li>
<li>Similiar quantities, such as \"Length, Breadth, Height, Thickness,
    Radius\" are defined as the same quantity (here: \"Length\").</li>
<li>The ordering of the type declarations in this package follows ISO 31:
<pre>
  Chapter  1: <b>Space and Time</b>
  Chapter  2: <b>Periodic and Related Phenomena</b>
  Chapter  3: <b>Mechanics</b>
  Chapter  4: <b>Heat</b>
  Chapter  5: <b>Electricity and Magnetism</b>
  Chapter  6: <b>Light and Related Electro-Magnetic Radiations</b>
  Chapter  7: <b>Acoustics</b>
  Chapter  8: <b>Physical Chemistry</b>
  Chapter  9: <b>Atomic and Nuclear Physics</b>
  Chapter 10: <b>Nuclear Reactions and Ionizing Radiations</b>
  Chapter 11: (not defined in ISO 31-1992)
  Chapter 12: <b>Characteristic Numbers</b>
  Chapter 13: <b>Solid State Physics</b>
</pre>
</li>
<li>Conversion functions between SI and non-SI units are available in subpackage
    <b>Conversions</b>.</li>
</ul>
</html>"));
      end Conventions;
      class Literature "Literature"
        annotation(Documentation(info = "<html>
<p> This package is based on the following references
</p>

<dl>
<dt>ISO 31-1992:</dt>
<dd> <b>General principles concerning
    quantities, units and symbols</b>.<br>&nbsp;</dd>

<dt>ISO 1000-1992:</dt>
<dd> <b>SI units and recommendations for the use
    of their multiples and of certain other units</b>.<br>&nbsp;</dd>

<dt>Cardarelli F.:</dt>
<dd> <b>Scientific Unit Conversion - A Practical
     Guide to Metrication</b>. Springer 1997.</dd>
</dl>

</html>
"));
      end Literature;
      class Contact "Contact"
        annotation(Documentation(info = "<html>
<dl>
<dt><b>Main Author:</b>
<dd><a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a><br>
    Deutsches Zentrum f&uuml;r Luft und Raumfahrt e.V. (DLR)<br>
    Institut f&uuml;r Robotik und Mechatronik<br>
    Abteilung f&uuml;r Entwurfsorientierte Regelungstechnik<br>
    Postfach 1116<br>
    D-82230 Wessling<br>
    Germany<br>
    email: <A HREF=\"mailto:Martin.Otter@dlr.de\">Martin.Otter@dlr.de</A><br>
</dl>


<p><b>Acknowledgements:</b></p>
<ul>
<li> Astrid Jaschinski, Hubertus Tummescheit and Christian Schweiger
     contributed to the implementation of this package.</li>
</ul>
</html>
"));
      end Contact;
    end UsersGuide;
    extends Modelica.Icons.Library2;
    annotation(Invisible = true, Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -63, -13},{45, -67}}, lineColor = {0,0,0}, textString = "[kg.m2]")}), Documentation(info = "<html>
<p>This package provides predefined types, such as <i>Mass</i>,
<i>Angle</i>, <i>Time</i>, based on the international standard
on units, e.g.,
</p>

<pre>   <b>type</b> Angle = Real(<b>final</b> quantity = \"Angle\",
                     <b>final</b> unit     = \"rad\",
                     displayUnit    = \"deg\");
</pre>

<p>
as well as conversion functions from non SI-units to SI-units
and vice versa in subpackage
<a href=\"Modelica://Modelica.SIunits.Conversions\">Conversions</a>.
</p>

<p>
For an introduction how units are used in the Modelica standard library
with package SIunits, have a look at:
<a href=\"Modelica://Modelica.SIunits.UsersGuide.HowToUseSIunits\">How to use SIunits</a>.
</p>

<p>
Copyright &copy; 1998-2009, Modelica Association and DLR.
</p>
<p>
<i>This Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p>

</html>", revisions = "<html>
<ul>
<li><i>Dec. 14, 2005</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Add User's Guide and removed \"min\" values for Resistance and Conductance.</li>
<li><i>October 21, 2002</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>
       and <a href=\"http://www.robotic.dlr.de/Christian.Schweiger/\">Christian Schweiger</a>:<br>
       Added new package <b>Conversions</b>. Corrected typo <i>Wavelenght</i>.</li>
<li><i>June 6, 2000</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Introduced the following new types<br>
       type Temperature = ThermodynamicTemperature;<br>
       types DerDensityByEnthalpy, DerDensityByPressure,
       DerDensityByTemperature, DerEnthalpyByPressure,
       DerEnergyByDensity, DerEnergyByPressure<br>
       Attribute \"final\" removed from min and max values
       in order that these values can still be changed to narrow
       the allowed range of values.<br>
       Quantity=\"Stress\" removed from type \"Stress\", in order
       that a type \"Stress\" can be connected to a type \"Pressure\".</li>
<li><i>Oct. 27, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       New types due to electrical library: Transconductance, InversePotential,
       Damping.</li>
<li><i>Sept. 18, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Renamed from SIunit to SIunits. Subpackages expanded, i.e., the
       SIunits package, does no longer contain subpackages.</li>
<li><i>Aug 12, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Type \"Pressure\" renamed to \"AbsolutePressure\" and introduced a new
       type \"Pressure\" which does not contain a minimum of zero in order
       to allow convenient handling of relative pressure. Redefined
       BulkModulus as an alias to AbsolutePressure instead of Stress, since
       needed in hydraulics.</li>
<li><i>June 29, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Bug-fix: Double definition of \"Compressibility\" removed
       and appropriate \"extends Heat\" clause introduced in
       package SolidStatePhysics to incorporate ThermodynamicTemperature.</li>
<li><i>April 8, 1998</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>
       and Astrid Jaschinski:<br>
       Complete ISO 31 chapters realized.</li>
<li><i>Nov. 15, 1997</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>
       and <a href=\"http://www.control.lth.se/~hubertus/\">Hubertus Tummescheit</a>:<br>
       Some chapters realized.</li>
</ul>
</html>"), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{169,86},{349,236}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{169,236},{189,256},{369,256},{349,236},{169,236}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{369,256},{369,106},{349,86},{349,236},{369,256}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{179,226},{339,196}}, lineColor = {160,160,164}, textString = "Library"),Text(extent = {{206,173},{314,119}}, lineColor = {0,0,0}, textString = "[kg.m2]"),Text(extent = {{163,320},{406,264}}, lineColor = {255,0,0}, textString = "Modelica.SIunits")}));
    package Conversions "Conversion functions to/from non SI units and type definitions of non SI units"
      extends Modelica.Icons.Library2;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -33, -7},{ -92, -67}}, lineColor = {0,0,0}, lineThickness = 1, textString = "C"),Text(extent = {{82, -7},{22, -67}}, lineColor = {0,0,0}, textString = "K"),Line(points = {{ -26, -36},{6, -36}}, color = {0,0,0}),Polygon(points = {{6, -28},{6, -45},{26, -37},{6, -28}}, pattern = LinePattern.None, fillColor = {0,0,0}, fillPattern = FillPattern.Solid, lineColor = {0,0,255})}), Documentation(info = "<HTML>
<p>This package provides conversion functions from the non SI Units
defined in package Modelica.SIunits.Conversions.NonSIunits to the
corresponding SI Units defined in package Modelica.SIunits and vice
versa. It is recommended to use these functions in the following
way (note, that all functions have one Real input and one Real output
argument):</p>
<pre>
  <b>import</b> SI = Modelica.SIunits;
  <b>import</b> Modelica.SIunits.Conversions.*;
     ...
  <b>parameter</b> SI.Temperature     T   = from_degC(25);   // convert 25 degree Celsius to Kelvin
  <b>parameter</b> SI.Angle           phi = from_deg(180);   // convert 180 degree to radian
  <b>parameter</b> SI.AngularVelocity w   = from_rpm(3600);  // convert 3600 revolutions per minutes
                                                      // to radian per seconds
</pre>

</HTML>
"));
      package NonSIunits "Type definitions of non SI units"
        extends Modelica.Icons.Library2;
        type Temperature_degC = Real(final quantity = "ThermodynamicTemperature", final unit = "degC") "Absolute temperature in degree Celsius (for relative temperature use SIunits.TemperatureDifference)" annotation(__Dymola_absoluteValue = true);
        type Temperature_degF = Real(final quantity = "ThermodynamicTemperature", final unit = "degF") "Absolute temperature in degree Fahrenheit (for relative temperature use SIunits.TemperatureDifference)" annotation(__Dymola_absoluteValue = true);
        type Temperature_degRk = Real(final quantity = "ThermodynamicTemperature", final unit = "degRk") "Absolute temperature in degree Rankine (for relative temperature use SIunits.TemperatureDifference)" annotation(__Dymola_absoluteValue = true);
        type Angle_deg = Real(final quantity = "Angle", final unit = "deg") "Angle in degree";
        type AngularVelocity_rpm = Real(final quantity = "AngularVelocity", final unit = "1/min") "Angular velocity in revolutions per minute. Alias unit names that are outside of the SI system: rpm, r/min, rev/min" annotation(Documentation(info = "<html>
<p>

</html>"));
        type Velocity_kmh = Real(final quantity = "Velocity", final unit = "km/h") "Velocity in kilometers per hour";
        type Time_day = Real(final quantity = "Time", final unit = "d") "Time in days";
        type Time_hour = Real(final quantity = "Time", final unit = "h") "Time in hours";
        type Time_minute = Real(final quantity = "Time", final unit = "min") "Time in minutes";
        type Volume_litre = Real(final quantity = "Volume", final unit = "l") "Volume in litres";
        type Energy_kWh = Real(final quantity = "Energy", final unit = "kW.h") "Energy in kilo watt hours";
        type Pressure_bar = Real(final quantity = "Pressure", final unit = "bar") "Absolute pressure in bar";
        type MassFlowRate_gps = Real(final quantity = "MassFlowRate", final unit = "g/s") "Mass flow rate in gramm per second";
        annotation(Documentation(info = "<HTML>
<p>
This package provides predefined types, such as <b>Angle_deg</b> (angle in
degree), <b>AngularVelocity_rpm</b> (angular velocity in revolutions per
minute) or <b>Temperature_degF</b> (temperature in degree Fahrenheit),
which are in common use but are not part of the international standard on
units according to ISO 31-1992 \"General principles concerning quantities,
units and symbols\" and ISO 1000-1992 \"SI units and recommendations for
the use of their multiples and of certain other units\".</p>
<p>If possible, the types in this package should not be used. Use instead
types of package Modelica.SIunits. For more information on units, see also
the book of Francois Cardarelli <b>Scientific Unit Conversion - A
Practical Guide to Metrication</b> (Springer 1997).</p>
<p>Some units, such as <b>Temperature_degC/Temp_C</b> are both defined in
Modelica.SIunits and in Modelica.Conversions.NonSIunits. The reason is that these
definitions have been placed erroneously in Modelica.SIunits although they
are not SIunits. For backward compatibility, these type definitions are
still kept in Modelica.SIunits.</p>
</HTML>
"), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -66, -13},{52, -67}}, lineColor = {0,0,0}, textString = "[km/h]")}));
      end NonSIunits;
      function to_degC "Convert from Kelvin to Celsius"
        extends ConversionIcon;
        input Temperature Kelvin "Kelvin value";
        output NonSIunits.Temperature_degC Celsius "Celsius value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "K"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "C")}));
      algorithm 
        Celsius:=Kelvin + Modelica.Constants.T_zero;
      end to_degC;
      function from_degC "Convert from Celsius to Kelvin"
        extends ConversionIcon;
        input NonSIunits.Temperature_degC Celsius "Celsius value";
        output Temperature Kelvin "Kelvin value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "C"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "K")}));
      algorithm 
        Kelvin:=Celsius - Modelica.Constants.T_zero;
      end from_degC;
      function to_degF "Convert from Kelvin to Fahrenheit"
        extends ConversionIcon;
        input Temperature Kelvin "Kelvin value";
        output NonSIunits.Temperature_degF Fahrenheit "Fahrenheit value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "K"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "F")}));
      algorithm 
        Fahrenheit:=(Kelvin + Modelica.Constants.T_zero) * 9 / 5 + 32;
      end to_degF;
      function from_degF "Convert from Fahrenheit to Kelvin"
        extends ConversionIcon;
        input NonSIunits.Temperature_degF Fahrenheit "Fahrenheit value";
        output Temperature Kelvin "Kelvin value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "F"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "K")}));
      algorithm 
        Kelvin:=(Fahrenheit - 32) * 5 / 9 - Modelica.Constants.T_zero;
      end from_degF;
      function to_degRk "Convert from Kelvin to Rankine"
        extends ConversionIcon;
        input Temperature Kelvin "Kelvin value";
        output NonSIunits.Temperature_degRk Rankine "Rankine value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "K"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "Rk")}));
      algorithm 
        Rankine:=9 / 5 * Kelvin;
      end to_degRk;
      function from_degRk "Convert from Rankine to Kelvin"
        extends ConversionIcon;
        input NonSIunits.Temperature_degRk Rankine "Rankine value";
        output Temperature Kelvin "Kelvin value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "Rk"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "K")}));
      algorithm 
        Kelvin:=5 / 9 * Rankine;
      end from_degRk;
      function to_deg "Convert from radian to degree"
        extends ConversionIcon;
        input Angle radian "radian value";
        output NonSIunits.Angle_deg degree "degree value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "rad"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "deg")}));
      algorithm 
        degree:=180.0 / Modelica.Constants.pi * radian;
      end to_deg;
      function from_deg "Convert from degree to radian"
        extends ConversionIcon;
        input NonSIunits.Angle_deg degree "degree value";
        output Angle radian "radian value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "deg"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "rad")}));
      algorithm 
        radian:=Modelica.Constants.pi / 180.0 * degree;
      end from_deg;
      function to_rpm "Convert from radian per second to revolutions per minute"
        extends ConversionIcon;
        input AngularVelocity rs "radian per second value";
        output NonSIunits.AngularVelocity_rpm rpm "revolutions per minute value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "rad/s"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "1/min")}));
      algorithm 
        rpm:=30 / Modelica.Constants.pi * rs;
      end to_rpm;
      function from_rpm "Convert from revolutions per minute to radian per second"
        extends ConversionIcon;
        input NonSIunits.AngularVelocity_rpm rpm "revolutions per minute value";
        output AngularVelocity rs "radian per second value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "1/min"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "rad/s")}));
      algorithm 
        rs:=Modelica.Constants.pi / 30 * rpm;
      end from_rpm;
      function to_kmh "Convert from metre per second to kilometre per hour"
        extends ConversionIcon;
        input Velocity ms "metre per second value";
        output NonSIunits.Velocity_kmh kmh "kilometre per hour value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "m/s"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "km/h")}));
      algorithm 
        kmh:=3.6 * ms;
      end to_kmh;
      function from_kmh "Convert from kilometre per hour to metre per second"
        extends ConversionIcon;
        input NonSIunits.Velocity_kmh kmh "kilometre per hour value";
        output Velocity ms "metre per second value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "km/h"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "m/s")}));
      algorithm 
        ms:=kmh / 3.6;
      end from_kmh;
      function to_day "Convert from second to day"
        extends ConversionIcon;
        input Time s "second value";
        output NonSIunits.Time_day day "day value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "s"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "day")}));
      algorithm 
        day:=s / 86400;
      end to_day;
      function from_day "Convert from day to second"
        extends ConversionIcon;
        input NonSIunits.Time_day day "day value";
        output Time s "second value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "day"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "s")}));
      algorithm 
        s:=86400 * day;
      end from_day;
      function to_hour "Convert from second to hour"
        extends ConversionIcon;
        input Time s "second value";
        output NonSIunits.Time_hour hour "hour value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "s"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "hour")}));
      algorithm 
        hour:=s / 3600;
      end to_hour;
      function from_hour "Convert from hour to second"
        extends ConversionIcon;
        input NonSIunits.Time_hour hour "hour value";
        output Time s "second value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "hour"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "s")}));
      algorithm 
        s:=3600 * hour;
      end from_hour;
      function to_minute "Convert from second to minute"
        extends ConversionIcon;
        input Time s "second value";
        output NonSIunits.Time_minute minute "minute value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "s"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "minute")}));
      algorithm 
        minute:=s / 60;
      end to_minute;
      function from_minute "Convert from minute to second"
        extends ConversionIcon;
        input NonSIunits.Time_minute minute "minute value";
        output Time s "second value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "minute"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "s")}));
      algorithm 
        s:=60 * minute;
      end from_minute;
      function to_litre "Convert from cubic metre to litre"
        extends ConversionIcon;
        input Volume m3 "cubic metre value";
        output NonSIunits.Volume_litre litre "litre value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "m3"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "litre")}));
      algorithm 
        litre:=1000 * m3;
      end to_litre;
      function from_litre "Convert from litre to cubic metre"
        extends ConversionIcon;
        input NonSIunits.Volume_litre litre "litre value";
        output Volume m3 "cubic metre value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "litre"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "m3")}));
      algorithm 
        m3:=litre / 1000;
      end from_litre;
      function to_kWh "Convert from Joule to kilo Watt hour"
        extends ConversionIcon;
        input Energy J "Joule value";
        output NonSIunits.Energy_kWh kWh "kWh value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "J"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "kWh")}));
      algorithm 
        kWh:=J / 3600000.0;
      end to_kWh;
      function from_kWh "Convert from kilo Watt hour to Joule"
        extends ConversionIcon;
        input NonSIunits.Energy_kWh kWh "kWh value";
        output Energy J "Joule value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "kWh"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "J")}));
      algorithm 
        J:=3600000.0 * kWh;
      end from_kWh;
      function to_bar "Convert from Pascal to bar"
        extends ConversionIcon;
        input Pressure Pa "Pascal value";
        output NonSIunits.Pressure_bar bar "bar value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "Pa"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "bar")}));
      algorithm 
        bar:=Pa / 100000.0;
      end to_bar;
      function from_bar "Convert from bar to Pascal"
        extends ConversionIcon;
        input NonSIunits.Pressure_bar bar "bar value";
        output Pressure Pa "Pascal value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "bar"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "Pa")}));
      algorithm 
        Pa:=100000.0 * bar;
      end from_bar;
      function to_gps "Convert from kilogram per second to gram per second"
        extends ConversionIcon;
        input MassFlowRate kgps "kg/s value";
        output NonSIunits.MassFlowRate_gps gps "g/s value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "kg/s"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "g/s")}));
      algorithm 
        gps:=1000 * kgps;
      end to_gps;
      function from_gps "Convert from gram per second to kilogram per second"
        extends ConversionIcon;
        input NonSIunits.MassFlowRate_gps gps "g/s value";
        output MassFlowRate kgps "kg/s value";
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "g/s"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "kg/s")}));
      algorithm 
        kgps:=gps / 1000;
      end from_gps;
      partial function ConversionIcon "Base icon for conversion functions"
        annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {191,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{ -90,0},{30,0}}, color = {191,0,0}),Polygon(points = {{90,0},{30,20},{30, -20},{90,0}}, lineColor = {191,0,0}, fillColor = {191,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -115,155},{115,105}}, textString = "%name", lineColor = {0,0,255})}));
      end ConversionIcon;
      function from_Hz "Convert from Hz to rad/s"
        extends Modelica.SIunits.Conversions.ConversionIcon;
        input Modelica.SIunits.Frequency f "frequency";
        output Modelica.SIunits.AngularVelocity w "angular velocity";
        annotation(Icon(graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "Hz"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "1/s")}));
      algorithm 
        w:=2 * Modelica.Constants.pi * f;
      end from_Hz;
      function to_Hz "Convert from rad/s to Hz"
        extends Modelica.SIunits.Conversions.ConversionIcon;
        input Modelica.SIunits.AngularVelocity w "angular velocity";
        output Modelica.SIunits.Frequency f "frequency";
        annotation(Icon(graphics = {Text(extent = {{ -20,100},{ -100,20}}, lineColor = {0,0,0}, textString = "1/s"),Text(extent = {{100, -20},{20, -100}}, lineColor = {0,0,0}, textString = "Hz")}));
      algorithm 
        f:=w / (2 * Modelica.Constants.pi);
      end to_Hz;
    end Conversions;
    type Angle = Real(final quantity = "Angle", final unit = "rad", displayUnit = "deg");
    type SolidAngle = Real(final quantity = "SolidAngle", final unit = "sr");
    type Length = Real(final quantity = "Length", final unit = "m");
    type PathLength = Length;
    type Position = Length;
    type Distance = Length(min = 0);
    type Breadth = Length(min = 0);
    type Height = Length(min = 0);
    type Thickness = Length(min = 0);
    type Radius = Length(min = 0);
    type Diameter = Length(min = 0);
    type Area = Real(final quantity = "Area", final unit = "m2");
    type Volume = Real(final quantity = "Volume", final unit = "m3");
    type Time = Real(final quantity = "Time", final unit = "s");
    type Duration = Time;
    type AngularVelocity = Real(final quantity = "AngularVelocity", final unit = "rad/s");
    type AngularAcceleration = Real(final quantity = "AngularAcceleration", final unit = "rad/s2");
    type Velocity = Real(final quantity = "Velocity", final unit = "m/s");
    type Acceleration = Real(final quantity = "Acceleration", final unit = "m/s2");
    type Period = Real(final quantity = "Time", final unit = "s");
    type Frequency = Real(final quantity = "Frequency", final unit = "Hz");
    type AngularFrequency = Real(final quantity = "AngularFrequency", final unit = "rad/s");
    type Wavelength = Real(final quantity = "Wavelength", final unit = "m");
    type Wavelenght = Wavelength;
    type WaveNumber = Real(final quantity = "WaveNumber", final unit = "m-1");
    type CircularWaveNumber = Real(final quantity = "CircularWaveNumber", final unit = "rad/m");
    type AmplitudeLevelDifference = Real(final quantity = "AmplitudeLevelDifference", final unit = "dB");
    type PowerLevelDifference = Real(final quantity = "PowerLevelDifference", final unit = "dB");
    type DampingCoefficient = Real(final quantity = "DampingCoefficient", final unit = "s-1");
    type LogarithmicDecrement = Real(final quantity = "LogarithmicDecrement", final unit = "1/S");
    type AttenuationCoefficient = Real(final quantity = "AttenuationCoefficient", final unit = "m-1");
    type PhaseCoefficient = Real(final quantity = "PhaseCoefficient", final unit = "m-1");
    type PropagationCoefficient = Real(final quantity = "PropagationCoefficient", final unit = "m-1");
    type Damping = DampingCoefficient;
    type Mass = Real(quantity = "Mass", final unit = "kg", min = 0);
    type Density = Real(final quantity = "Density", final unit = "kg/m3", displayUnit = "g/cm3", min = 0);
    type RelativeDensity = Real(final quantity = "RelativeDensity", final unit = "1", min = 0);
    type SpecificVolume = Real(final quantity = "SpecificVolume", final unit = "m3/kg", min = 0);
    type LinearDensity = Real(final quantity = "LinearDensity", final unit = "kg/m", min = 0);
    type SurfaceDensity = Real(final quantity = "SurfaceDensity", final unit = "kg/m2", min = 0);
    type Momentum = Real(final quantity = "Momentum", final unit = "kg.m/s");
    type Impulse = Real(final quantity = "Impulse", final unit = "N.s");
    type AngularMomentum = Real(final quantity = "AngularMomentum", final unit = "kg.m2/s");
    type AngularImpulse = Real(final quantity = "AngularImpulse", final unit = "N.m.s");
    type MomentOfInertia = Real(final quantity = "MomentOfInertia", final unit = "kg.m2");
    type Inertia = MomentOfInertia;
    type Force = Real(final quantity = "Force", final unit = "N");
    type TranslationalSpringConstant = Real(final quantity = "TranslationalSpringConstant", final unit = "N/m");
    type TranslationalDampingConstant = Real(final quantity = "TranslationalDampingConstant", final unit = "N.s/m");
    type Weight = Force;
    type Torque = Real(final quantity = "Torque", final unit = "N.m");
    type ElectricalTorqueConstant = Real(final quantity = "ElectricalTorqueConstant", final unit = "N.m/A");
    type MomentOfForce = Torque;
    type RotationalSpringConstant = Real(final quantity = "RotationalSpringConstant", final unit = "N.m/rad");
    type RotationalDampingConstant = Real(final quantity = "RotationalDampingConstant", final unit = "N.m.s/rad");
    type Pressure = Real(final quantity = "Pressure", final unit = "Pa", displayUnit = "bar");
    type AbsolutePressure = Pressure(min = 0);
    type BulkModulus = AbsolutePressure;
    type Stress = Real(final unit = "Pa");
    type NormalStress = Stress;
    type ShearStress = Stress;
    type Strain = Real(final quantity = "Strain", final unit = "1");
    type LinearStrain = Strain;
    type ShearStrain = Strain;
    type VolumeStrain = Real(final quantity = "VolumeStrain", final unit = "1");
    type PoissonNumber = Real(final quantity = "PoissonNumber", final unit = "1");
    type ModulusOfElasticity = Stress;
    type ShearModulus = Stress;
    type SecondMomentOfArea = Real(final quantity = "SecondMomentOfArea", final unit = "m4");
    type SecondPolarMomentOfArea = SecondMomentOfArea;
    type SectionModulus = Real(final quantity = "SectionModulus", final unit = "m3");
    type CoefficientOfFriction = Real(final quantity = "CoefficientOfFriction", final unit = "1");
    type DynamicViscosity = Real(final quantity = "DynamicViscosity", final unit = "Pa.s", min = 0);
    type KinematicViscosity = Real(final quantity = "KinematicViscosity", final unit = "m2/s", min = 0);
    type SurfaceTension = Real(final quantity = "SurfaceTension", final unit = "N/m");
    type Work = Real(final quantity = "Work", final unit = "J");
    type Energy = Real(final quantity = "Energy", final unit = "J");
    type EnergyDensity = Real(final quantity = "EnergyDensity", final unit = "J/m3");
    type PotentialEnergy = Energy;
    type KineticEnergy = Energy;
    type Power = Real(final quantity = "Power", final unit = "W");
    type EnergyFlowRate = Power;
    type EnthalpyFlowRate = Real(final quantity = "EnthalpyFlowRate", final unit = "W");
    type Efficiency = Real(final quantity = "Efficiency", final unit = "1", min = 0);
    type MassFlowRate = Real(quantity = "MassFlowRate", final unit = "kg/s");
    type VolumeFlowRate = Real(final quantity = "VolumeFlowRate", final unit = "m3/s");
    type MomentumFlux = Real(final quantity = "MomentumFlux", final unit = "N");
    type AngularMomentumFlux = Real(final quantity = "AngularMomentumFlux", final unit = "N.m");
    type ThermodynamicTemperature = Real(final quantity = "ThermodynamicTemperature", final unit = "K", min = 0, displayUnit = "degC") "Absolute temperature (use type TemperatureDifference for relative temperatures)" annotation(__Dymola_absoluteValue = true);
    type Temp_K = ThermodynamicTemperature;
    type Temperature = ThermodynamicTemperature;
    type TemperatureDifference = Real(final quantity = "ThermodynamicTemperature", final unit = "K") annotation(__Dymola_absoluteValue = false);
    type Temp_C = Modelica.SIunits.Conversions.NonSIunits.Temperature_degC;
    type TemperatureSlope = Real(final quantity = "TemperatureSlope", final unit = "K/s");
    type LinearTemperatureCoefficient = Real(final quantity = "LinearTemperatureCoefficient", final unit = "1/K");
    type QuadraticTemperatureCoefficient = Real(final quantity = "QuadraticTemperatureCoefficient", final unit = "1/K2");
    type LinearExpansionCoefficient = Real(final quantity = "LinearExpansionCoefficient", final unit = "1/K");
    type CubicExpansionCoefficient = Real(final quantity = "CubicExpansionCoefficient", final unit = "1/K");
    type RelativePressureCoefficient = Real(final quantity = "RelativePressureCoefficient", final unit = "1/K");
    type PressureCoefficient = Real(final quantity = "PressureCoefficient", final unit = "Pa/K");
    type Compressibility = Real(final quantity = "Compressibility", final unit = "1/Pa");
    type IsothermalCompressibility = Compressibility;
    type IsentropicCompressibility = Compressibility;
    type Heat = Real(final quantity = "Energy", final unit = "J");
    type HeatFlowRate = Real(final quantity = "Power", final unit = "W");
    type HeatFlux = Real(final quantity = "HeatFlux", final unit = "W/m2");
    type DensityOfHeatFlowRate = Real(final quantity = "DensityOfHeatFlowRate", final unit = "W/m2");
    type ThermalConductivity = Real(final quantity = "ThermalConductivity", final unit = "W/(m.K)");
    type CoefficientOfHeatTransfer = Real(final quantity = "CoefficientOfHeatTransfer", final unit = "W/(m2.K)");
    type SurfaceCoefficientOfHeatTransfer = CoefficientOfHeatTransfer;
    type ThermalInsulance = Real(final quantity = "ThermalInsulance", final unit = "m2.K/W");
    type ThermalResistance = Real(final quantity = "ThermalResistance", final unit = "K/W");
    type ThermalConductance = Real(final quantity = "ThermalConductance", final unit = "W/K");
    type ThermalDiffusivity = Real(final quantity = "ThermalDiffusivity", final unit = "m2/s");
    type HeatCapacity = Real(final quantity = "HeatCapacity", final unit = "J/K");
    type SpecificHeatCapacity = Real(final quantity = "SpecificHeatCapacity", final unit = "J/(kg.K)");
    type SpecificHeatCapacityAtConstantPressure = SpecificHeatCapacity;
    type SpecificHeatCapacityAtConstantVolume = SpecificHeatCapacity;
    type SpecificHeatCapacityAtSaturation = SpecificHeatCapacity;
    type RatioOfSpecificHeatCapacities = Real(final quantity = "RatioOfSpecificHeatCapacities", final unit = "1");
    type IsentropicExponent = Real(final quantity = "IsentropicExponent", final unit = "1");
    type Entropy = Real(final quantity = "Entropy", final unit = "J/K");
    type EntropyFlowRate = Real(final quantity = "EntropyFlowRate", final unit = "J/(K.s)");
    type SpecificEntropy = Real(final quantity = "SpecificEntropy", final unit = "J/(kg.K)");
    type InternalEnergy = Heat;
    type Enthalpy = Heat;
    type HelmholtzFreeEnergy = Heat;
    type GibbsFreeEnergy = Heat;
    type SpecificEnergy = Real(final quantity = "SpecificEnergy", final unit = "J/kg");
    type SpecificInternalEnergy = SpecificEnergy;
    type SpecificEnthalpy = SpecificEnergy;
    type SpecificHelmholtzFreeEnergy = SpecificEnergy;
    type SpecificGibbsFreeEnergy = SpecificEnergy;
    type MassieuFunction = Real(final quantity = "MassieuFunction", final unit = "J/K");
    type PlanckFunction = Real(final quantity = "PlanckFunction", final unit = "J/K");
    type DerDensityByEnthalpy = Real(final unit = "kg.s2/m5");
    type DerDensityByPressure = Real(final unit = "s2/m2");
    type DerDensityByTemperature = Real(final unit = "kg/(m3.K)");
    type DerEnthalpyByPressure = Real(final unit = "J.m.s2/kg2");
    type DerEnergyByDensity = Real(final unit = "J.m3/kg");
    type DerEnergyByPressure = Real(final unit = "J.m.s2/kg");
    type ElectricCurrent = Real(final quantity = "ElectricCurrent", final unit = "A");
    type Current = ElectricCurrent;
    type CurrentSlope = Real(final quantity = "CurrentSlope", final unit = "A/s");
    type ElectricCharge = Real(final quantity = "ElectricCharge", final unit = "C");
    type Charge = ElectricCharge;
    type VolumeDensityOfCharge = Real(final quantity = "VolumeDensityOfCharge", final unit = "C/m3", min = 0);
    type SurfaceDensityOfCharge = Real(final quantity = "SurfaceDensityOfCharge", final unit = "C/m2", min = 0);
    type ElectricFieldStrength = Real(final quantity = "ElectricFieldStrength", final unit = "V/m");
    type ElectricPotential = Real(final quantity = "ElectricPotential", final unit = "V");
    type Voltage = ElectricPotential;
    type PotentialDifference = ElectricPotential;
    type ElectromotiveForce = ElectricPotential;
    type VoltageSlope = Real(final quantity = "VoltageSlope", final unit = "V/s");
    type ElectricFluxDensity = Real(final quantity = "ElectricFluxDensity", final unit = "C/m2");
    type ElectricFlux = Real(final quantity = "ElectricFlux", final unit = "C");
    type Capacitance = Real(final quantity = "Capacitance", final unit = "F", min = 0);
    type Permittivity = Real(final quantity = "Permittivity", final unit = "F/m", min = 0);
    type PermittivityOfVacuum = Permittivity;
    type RelativePermittivity = Real(final quantity = "RelativePermittivity", final unit = "1");
    type ElectricSusceptibility = Real(final quantity = "ElectricSusceptibility", final unit = "1");
    type ElectricPolarization = Real(final quantity = "ElectricPolarization", final unit = "C/m2");
    type Electrization = Real(final quantity = "Electrization", final unit = "V/m");
    type ElectricDipoleMoment = Real(final quantity = "ElectricDipoleMoment", final unit = "C.m");
    type CurrentDensity = Real(final quantity = "CurrentDensity", final unit = "A/m2");
    type LinearCurrentDensity = Real(final quantity = "LinearCurrentDensity", final unit = "A/m");
    type MagneticFieldStrength = Real(final quantity = "MagneticFieldStrength", final unit = "A/m");
    type MagneticPotential = Real(final quantity = "MagneticPotential", final unit = "A");
    type MagneticPotentialDifference = Real(final quantity = "MagneticPotential", final unit = "A");
    type MagnetomotiveForce = Real(final quantity = "MagnetomotiveForce", final unit = "A");
    type CurrentLinkage = Real(final quantity = "CurrentLinkage", final unit = "A");
    type MagneticFluxDensity = Real(final quantity = "MagneticFluxDensity", final unit = "T");
    type MagneticFlux = Real(final quantity = "MagneticFlux", final unit = "Wb");
    type MagneticVectorPotential = Real(final quantity = "MagneticVectorPotential", final unit = "Wb/m");
    type Inductance = Real(final quantity = "Inductance", final unit = "H");
    type SelfInductance = Inductance(min = 0);
    type MutualInductance = Inductance;
    type CouplingCoefficient = Real(final quantity = "CouplingCoefficient", final unit = "1");
    type LeakageCoefficient = Real(final quantity = "LeakageCoefficient", final unit = "1");
    type Permeability = Real(final quantity = "Permeability", final unit = "H/m");
    type PermeabilityOfVacuum = Permeability;
    type RelativePermeability = Real(final quantity = "RelativePermeability", final unit = "1");
    type MagneticSusceptibility = Real(final quantity = "MagneticSusceptibility", final unit = "1");
    type ElectromagneticMoment = Real(final quantity = "ElectromagneticMoment", final unit = "A.m2");
    type MagneticDipoleMoment = Real(final quantity = "MagneticDipoleMoment", final unit = "Wb.m");
    type Magnetization = Real(final quantity = "Magnetization", final unit = "A/m");
    type MagneticPolarization = Real(final quantity = "MagneticPolarization", final unit = "T");
    type ElectromagneticEnergyDensity = Real(final quantity = "EnergyDensity", final unit = "J/m3");
    type PoyntingVector = Real(final quantity = "PoyntingVector", final unit = "W/m2");
    type Resistance = Real(final quantity = "Resistance", final unit = "Ohm");
    type Resistivity = Real(final quantity = "Resistivity", final unit = "Ohm.m");
    type Conductivity = Real(final quantity = "Conductivity", final unit = "S/m");
    type Reluctance = Real(final quantity = "Reluctance", final unit = "H-1");
    type Permeance = Real(final quantity = "Permeance", final unit = "H");
    type PhaseDifference = Real(final quantity = "Angle", final unit = "rad", displayUnit = "deg");
    type Impedance = Resistance;
    type ModulusOfImpedance = Resistance;
    type Reactance = Resistance;
    type QualityFactor = Real(final quantity = "QualityFactor", final unit = "1");
    type LossAngle = Real(final quantity = "Angle", final unit = "rad", displayUnit = "deg");
    type Conductance = Real(final quantity = "Conductance", final unit = "S");
    type Admittance = Conductance;
    type ModulusOfAdmittance = Conductance;
    type Susceptance = Conductance;
    type InstantaneousPower = Real(final quantity = "Power", final unit = "W");
    type ActivePower = Real(final quantity = "Power", final unit = "W");
    type ApparentPower = Real(final quantity = "Power", final unit = "VA");
    type ReactivePower = Real(final quantity = "Power", final unit = "var");
    type PowerFactor = Real(final quantity = "PowerFactor", final unit = "1");
    type Transconductance = Real(final quantity = "Transconductance", final unit = "A/V2");
    type InversePotential = Real(final quantity = "InversePotential", final unit = "1/V");
    type ElectricalForceConstant = Real(final quantity = "ElectricalForceConstant", final unit = "N/A");
    type RadiantEnergy = Real(final quantity = "Energy", final unit = "J");
    type RadiantEnergyDensity = Real(final quantity = "EnergyDensity", final unit = "J/m3");
    type SpectralRadiantEnergyDensity = Real(final quantity = "SpectralRadiantEnergyDensity", final unit = "J/m4");
    type RadiantPower = Real(final quantity = "Power", final unit = "W");
    type RadiantEnergyFluenceRate = Real(final quantity = "RadiantEnergyFluenceRate", final unit = "W/m2");
    type RadiantIntensity = Real(final quantity = "RadiantIntensity", final unit = "W/sr");
    type Radiance = Real(final quantity = "Radiance", final unit = "W/(sr.m2)");
    type RadiantExtiance = Real(final quantity = "RadiantExtiance", final unit = "W/m2");
    type Irradiance = Real(final quantity = "Irradiance", final unit = "W/m2");
    type Emissivity = Real(final quantity = "Emissivity", final unit = "1");
    type SpectralEmissivity = Real(final quantity = "SpectralEmissivity", final unit = "1");
    type DirectionalSpectralEmissivity = Real(final quantity = "DirectionalSpectralEmissivity", final unit = "1");
    type LuminousIntensity = Real(final quantity = "LuminousIntensity", final unit = "cd");
    type LuminousFlux = Real(final quantity = "LuminousFlux", final unit = "lm");
    type QuantityOfLight = Real(final quantity = "QuantityOfLight", final unit = "lm.s");
    type Luminance = Real(final quantity = "Luminance", final unit = "cd/m2");
    type LuminousExitance = Real(final quantity = "LuminousExitance", final unit = "lm/m2");
    type Illuminance = Real(final quantity = "Illuminance", final unit = "lx");
    type LightExposure = Real(final quantity = "LightExposure", final unit = "lx.s");
    type LuminousEfficacy = Real(final quantity = "LuminousEfficacy", final unit = "lm/W");
    type SpectralLuminousEfficacy = Real(final quantity = "SpectralLuminousEfficacy", final unit = "lm/W");
    type LuminousEfficiency = Real(final quantity = "LuminousEfficiency", final unit = "1");
    type SpectralLuminousEfficiency = Real(final quantity = "SpectralLuminousEfficiency", final unit = "1");
    type CIESpectralTristimulusValues = Real(final quantity = "CIESpectralTristimulusValues", final unit = "1");
    type ChromaticityCoordinates = Real(final quantity = "CromaticityCoordinates", final unit = "1");
    type SpectralAbsorptionFactor = Real(final quantity = "SpectralAbsorptionFactor", final unit = "1");
    type SpectralReflectionFactor = Real(final quantity = "SpectralReflectionFactor", final unit = "1");
    type SpectralTransmissionFactor = Real(final quantity = "SpectralTransmissionFactor", final unit = "1");
    type SpectralRadianceFactor = Real(final quantity = "SpectralRadianceFactor", final unit = "1");
    type LinearAttenuationCoefficient = Real(final quantity = "AttenuationCoefficient", final unit = "m-1");
    type LinearAbsorptionCoefficient = Real(final quantity = "LinearAbsorptionCoefficient", final unit = "m-1");
    type MolarAbsorptionCoefficient = Real(final quantity = "MolarAbsorptionCoefficient", final unit = "m2/mol");
    type RefractiveIndex = Real(final quantity = "RefractiveIndex", final unit = "1");
    type StaticPressure = Real(final quantity = "Pressure", final unit = "Pa", displayUnit = "bar", min = 0);
    type SoundPressure = StaticPressure;
    type SoundParticleDisplacement = Real(final quantity = "Length", final unit = "m");
    type SoundParticleVelocity = Real(final quantity = "Velocity", final unit = "m/s");
    type SoundParticleAcceleration = Real(final quantity = "Acceleration", final unit = "m/s2");
    type VelocityOfSound = Real(final quantity = "Velocity", final unit = "m/s");
    type SoundEnergyDensity = Real(final quantity = "EnergyDensity", final unit = "J/m3");
    type SoundPower = Real(final quantity = "Power", final unit = "W");
    type SoundIntensity = Real(final quantity = "SoundIntensity", final unit = "W/m2");
    type AcousticImpedance = Real(final quantity = "AcousticImpedance", final unit = "Pa.s/m3");
    type SpecificAcousticImpedance = Real(final quantity = "SpecificAcousticImpedance", final unit = "Pa.s/m");
    type MechanicalImpedance = Real(final quantity = "MechanicalImpedance", final unit = "N.s/m");
    type SoundPressureLevel = Real(final quantity = "SoundPressureLevel", final unit = "dB");
    type SoundPowerLevel = Real(final quantity = "SoundPowerLevel", final unit = "dB");
    type DissipationCoefficient = Real(final quantity = "DissipationCoefficient", final unit = "1");
    type ReflectionCoefficient = Real(final quantity = "ReflectionCoefficient", final unit = "1");
    type TransmissionCoefficient = Real(final quantity = "TransmissionCoefficient", final unit = "1");
    type AcousticAbsorptionCoefficient = Real(final quantity = "AcousticAbsorptionCoefficient", final unit = "1");
    type SoundReductionIndex = Real(final quantity = "SoundReductionIndex", final unit = "dB");
    type EquivalentAbsorptionArea = Real(final quantity = "Area", final unit = "m2");
    type ReverberationTime = Real(final quantity = "Time", final unit = "s");
    type LoundnessLevel = Real(final quantity = "LoundnessLevel", final unit = "phon");
    type Loundness = Real(final quantity = "Loundness", final unit = "sone");
    type RelativeAtomicMass = Real(final quantity = "RelativeAtomicMass", final unit = "1");
    type RelativeMolecularMass = Real(final quantity = "RelativeMolecularMass", final unit = "1");
    type NumberOfMolecules = Real(final quantity = "NumberOfMolecules", final unit = "1");
    type AmountOfSubstance = Real(final quantity = "AmountOfSubstance", final unit = "mol", min = 0);
    type MolarMass = Real(final quantity = "MolarMass", final unit = "kg/mol", min = 0);
    type MolarVolume = Real(final quantity = "MolarVolume", final unit = "m3/mol", min = 0);
    type MolarInternalEnergy = Real(final quantity = "MolarInternalEnergy", final unit = "J/mol");
    type MolarHeatCapacity = Real(final quantity = "MolarHeatCapacity", final unit = "J/(mol.K)");
    type MolarEntropy = Real(final quantity = "MolarEntropy", final unit = "J/(mol.K)");
    type MolarFlowRate = Real(final quantity = "MolarFlowRate", final unit = "mol/s");
    type NumberDensityOfMolecules = Real(final quantity = "NumberDensityOfMolecules", final unit = "m-3");
    type MolecularConcentration = Real(final quantity = "MolecularConcentration", final unit = "m-3");
    type MassConcentration = Real(final quantity = "MassConcentration", final unit = "kg/m3");
    type MassFraction = Real(final quantity = "MassFraction", final unit = "1");
    type Concentration = Real(final quantity = "Concentration", final unit = "mol/m3");
    type VolumeFraction = Real(final quantity = "VolumeFraction", final unit = "1");
    type MoleFraction = Real(final quantity = "MoleFraction", final unit = "1");
    type ChemicalPotential = Real(final quantity = "ChemicalPotential", final unit = "J/mol");
    type AbsoluteActivity = Real(final quantity = "AbsoluteActivity", final unit = "1");
    type PartialPressure = Real(final quantity = "Pressure", final unit = "Pa", displayUnit = "bar", min = 0);
    type Fugacity = Real(final quantity = "Fugacity", final unit = "Pa");
    type StandardAbsoluteActivity = Real(final quantity = "StandardAbsoluteActivity", final unit = "1");
    type ActivityCoefficient = Real(final quantity = "ActivityCoefficient", final unit = "1");
    type ActivityOfSolute = Real(final quantity = "ActivityOfSolute", final unit = "1");
    type ActivityCoefficientOfSolute = Real(final quantity = "ActivityCoefficientOfSolute", final unit = "1");
    type StandardAbsoluteActivityOfSolute = Real(final quantity = "StandardAbsoluteActivityOfSolute", final unit = "1");
    type ActivityOfSolvent = Real(final quantity = "ActivityOfSolvent", final unit = "1");
    type OsmoticCoefficientOfSolvent = Real(final quantity = "OsmoticCoefficientOfSolvent", final unit = "1");
    type StandardAbsoluteActivityOfSolvent = Real(final quantity = "StandardAbsoluteActivityOfSolvent", final unit = "1");
    type OsmoticPressure = Real(final quantity = "Pressure", final unit = "Pa", displayUnit = "bar", min = 0);
    type StoichiometricNumber = Real(final quantity = "StoichiometricNumber", final unit = "1");
    type Affinity = Real(final quantity = "Affinity", final unit = "J/mol");
    type MassOfMolecule = Real(final quantity = "Mass", final unit = "kg");
    type ElectricDipoleMomentOfMolecule = Real(final quantity = "ElectricDipoleMomentOfMolecule", final unit = "C.m");
    type ElectricPolarizabilityOfAMolecule = Real(final quantity = "ElectricPolarizabilityOfAMolecule", final unit = "C.m2/V");
    type MicrocanonicalPartitionFunction = Real(final quantity = "MicrocanonicalPartitionFunction", final unit = "1");
    type CanonicalPartitionFunction = Real(final quantity = "CanonicalPartitionFunction", final unit = "1");
    type GrandCanonicalPartitionFunction = Real(final quantity = "GrandCanonicalPartitionFunction", final unit = "1");
    type MolecularPartitionFunction = Real(final quantity = "MolecularPartitionFunction", final unit = "1");
    type StatisticalWeight = Real(final quantity = "StatisticalWeight", final unit = "1");
    type MeanFreePath = Length;
    type DiffusionCoefficient = Real(final quantity = "DiffusionCoefficient", final unit = "m2/s");
    type ThermalDiffusionRatio = Real(final quantity = "ThermalDiffusionRatio", final unit = "1");
    type ThermalDiffusionFactor = Real(final quantity = "ThermalDiffusionFactor", final unit = "1");
    type ThermalDiffusionCoefficient = Real(final quantity = "ThermalDiffusionCoefficient", final unit = "m2/s");
    type ElementaryCharge = Real(final quantity = "ElementaryCharge", final unit = "C");
    type ChargeNumberOfIon = Real(final quantity = "ChargeNumberOfIon", final unit = "1");
    type FaradayConstant = Real(final quantity = "FaradayConstant", final unit = "C/mol");
    type IonicStrength = Real(final quantity = "IonicStrength", final unit = "mol/kg");
    type DegreeOfDissociation = Real(final quantity = "DegreeOfDissociation", final unit = "1");
    type ElectrolyticConductivity = Real(final quantity = "ElectrolyticConductivity", final unit = "S/m");
    type MolarConductivity = Real(final quantity = "MolarConductivity", final unit = "S.m2/mol");
    type TransportNumberOfIonic = Real(final quantity = "TransportNumberOfIonic", final unit = "1");
    type ProtonNumber = Real(final quantity = "ProtonNumber", final unit = "1");
    type NeutronNumber = Real(final quantity = "NeutronNumber", final unit = "1");
    type NucleonNumber = Real(final quantity = "NucleonNumber", final unit = "1");
    type AtomicMassConstant = Real(final quantity = "Mass", final unit = "kg");
    type MassOfElectron = Real(final quantity = "Mass", final unit = "kg");
    type MassOfProton = Real(final quantity = "Mass", final unit = "kg");
    type MassOfNeutron = Real(final quantity = "Mass", final unit = "kg");
    type HartreeEnergy = Real(final quantity = "Energy", final unit = "J");
    type MagneticMomentOfParticle = Real(final quantity = "MagneticMomentOfParticle", final unit = "A.m2");
    type BohrMagneton = MagneticMomentOfParticle;
    type NuclearMagneton = MagneticMomentOfParticle;
    type GyromagneticCoefficient = Real(final quantity = "GyromagneticCoefficient", final unit = "A.m2/(J.s)");
    type GFactorOfAtom = Real(final quantity = "GFactorOfAtom", final unit = "1");
    type GFactorOfNucleus = Real(final quantity = "GFactorOfNucleus", final unit = "1");
    type LarmorAngularFrequency = Real(final quantity = "AngularFrequency", final unit = "s-1");
    type NuclearPrecessionAngularFrequency = Real(final quantity = "AngularFrequency", final unit = "s-1");
    type CyclotronAngularFrequency = Real(final quantity = "AngularFrequency", final unit = "s-1");
    type NuclearQuadrupoleMoment = Real(final quantity = "NuclearQuadrupoleMoment", final unit = "m2");
    type NuclearRadius = Real(final quantity = "Length", final unit = "m");
    type ElectronRadius = Real(final quantity = "Length", final unit = "m");
    type ComptonWavelength = Real(final quantity = "Length", final unit = "m");
    type MassExcess = Real(final quantity = "Mass", final unit = "kg");
    type MassDefect = Real(final quantity = "Mass", final unit = "kg");
    type RelativeMassExcess = Real(final quantity = "RelativeMassExcess", final unit = "1");
    type RelativeMassDefect = Real(final quantity = "RelativeMassDefect", final unit = "1");
    type PackingFraction = Real(final quantity = "PackingFraction", final unit = "1");
    type BindingFraction = Real(final quantity = "BindingFraction", final unit = "1");
    type MeanLife = Real(final quantity = "Time", final unit = "s");
    type LevelWidth = Real(final quantity = "LevelWidth", final unit = "J");
    type Activity = Real(final quantity = "Activity", final unit = "Bq");
    type SpecificActivity = Real(final quantity = "SpecificActivity", final unit = "Bq/kg");
    type DecayConstant = Real(final quantity = "DecayConstant", final unit = "s-1");
    type HalfLife = Real(final quantity = "Time", final unit = "s");
    type AlphaDisintegrationEnergy = Real(final quantity = "Energy", final unit = "J");
    type MaximumBetaParticleEnergy = Real(final quantity = "Energy", final unit = "J");
    type BetaDisintegrationEnergy = Real(final quantity = "Energy", final unit = "J");
    type ReactionEnergy = Real(final quantity = "Energy", final unit = "J");
    type ResonanceEnergy = Real(final quantity = "Energy", final unit = "J");
    type CrossSection = Real(final quantity = "Area", final unit = "m2");
    type TotalCrossSection = Real(final quantity = "Area", final unit = "m2");
    type AngularCrossSection = Real(final quantity = "AngularCrossSection", final unit = "m2/sr");
    type SpectralCrossSection = Real(final quantity = "SpectralCrossSection", final unit = "m2/J");
    type SpectralAngularCrossSection = Real(final quantity = "SpectralAngularCrossSection", final unit = "m2/(sr.J)");
    type MacroscopicCrossSection = Real(final quantity = "MacroscopicCrossSection", final unit = "m-1");
    type TotalMacroscopicCrossSection = Real(final quantity = "TotalMacroscopicCrossSection", final unit = "m-1");
    type ParticleFluence = Real(final quantity = "ParticleFluence", final unit = "m-2");
    type ParticleFluenceRate = Real(final quantity = "ParticleFluenceRate", final unit = "s-1.m2");
    type EnergyFluence = Real(final quantity = "EnergyFluence", final unit = "J/m2");
    type EnergyFluenceRate = Real(final quantity = "EnergyFluenceRate", final unit = "W/m2");
    type CurrentDensityOfParticles = Real(final quantity = "CurrentDensityOfParticles", final unit = "m-2.s-1");
    type MassAttenuationCoefficient = Real(final quantity = "MassAttenuationCoefficient", final unit = "m2/kg");
    type MolarAttenuationCoefficient = Real(final quantity = "MolarAttenuationCoefficient", final unit = "m2/mol");
    type AtomicAttenuationCoefficient = Real(final quantity = "AtomicAttenuationCoefficient", final unit = "m2");
    type HalfThickness = Real(final quantity = "Length", final unit = "m");
    type TotalLinearStoppingPower = Real(final quantity = "TotalLinearStoppingPower", final unit = "J/m");
    type TotalAtomicStoppingPower = Real(final quantity = "TotalAtomicStoppingPower", final unit = "J.m2");
    type TotalMassStoppingPower = Real(final quantity = "TotalMassStoppingPower", final unit = "J.m2/kg");
    type MeanLinearRange = Real(final quantity = "Length", final unit = "m");
    type MeanMassRange = Real(final quantity = "MeanMassRange", final unit = "kg/m2");
    type LinearIonization = Real(final quantity = "LinearIonization", final unit = "m-1");
    type TotalIonization = Real(final quantity = "TotalIonization", final unit = "1");
    type Mobility = Real(final quantity = "Mobility", final unit = "m2/(V.s)");
    type IonNumberDensity = Real(final quantity = "IonNumberDensity", final unit = "m-3");
    type RecombinationCoefficient = Real(final quantity = "RecombinationCoefficient", final unit = "m3/s");
    type NeutronNumberDensity = Real(final quantity = "NeutronNumberDensity", final unit = "m-3");
    type NeutronSpeed = Real(final quantity = "Velocity", final unit = "m/s");
    type NeutronFluenceRate = Real(final quantity = "NeutronFluenceRate", final unit = "s-1.m-2");
    type TotalNeutronSourceDensity = Real(final quantity = "TotalNeutronSourceDesity", final unit = "s-1.m-3");
    type SlowingDownDensity = Real(final quantity = "SlowingDownDensity", final unit = "s-1.m-3");
    type ResonanceEscapeProbability = Real(final quantity = "ResonanceEscapeProbability", final unit = "1");
    type Lethargy = Real(final quantity = "Lethargy", final unit = "1");
    type SlowingDownArea = Real(final quantity = "Area", final unit = "m2");
    type DiffusionArea = Real(final quantity = "Area", final unit = "m2");
    type MigrationArea = Real(final quantity = "Area", final unit = "m2");
    type SlowingDownLength = Real(final quantity = "SLength", final unit = "m");
    type DiffusionLength = Length;
    type MigrationLength = Length;
    type NeutronYieldPerFission = Real(final quantity = "NeutronYieldPerFission", final unit = "1");
    type NeutronYieldPerAbsorption = Real(final quantity = "NeutronYieldPerAbsorption", final unit = "1");
    type FastFissionFactor = Real(final quantity = "FastFissionFactor", final unit = "1");
    type ThermalUtilizationFactor = Real(final quantity = "ThermalUtilizationFactor", final unit = "1");
    type NonLeakageProbability = Real(final quantity = "NonLeakageProbability", final unit = "1");
    type Reactivity = Real(final quantity = "Reactivity", final unit = "1");
    type ReactorTimeConstant = Real(final quantity = "Time", final unit = "s");
    type EnergyImparted = Real(final quantity = "Energy", final unit = "J");
    type MeanEnergyImparted = Real(final quantity = "Energy", final unit = "J");
    type SpecificEnergyImparted = Real(final quantity = "SpecificEnergy", final unit = "Gy");
    type AbsorbedDose = Real(final quantity = "AbsorbedDose", final unit = "Gy");
    type DoseEquivalent = Real(final quantity = "DoseEquivalent", final unit = "Sv");
    type AbsorbedDoseRate = Real(final quantity = "AbsorbedDoseRate", final unit = "Gy/s");
    type LinearEnergyTransfer = Real(final quantity = "LinearEnergyTransfer", final unit = "J/m");
    type Kerma = Real(final quantity = "Kerma", final unit = "Gy");
    type KermaRate = Real(final quantity = "KermaRate", final unit = "Gy/s");
    type MassEnergyTransferCoefficient = Real(final quantity = "MassEnergyTransferCoefficient", final unit = "m2/kg");
    type Exposure = Real(final quantity = "Exposure", final unit = "C/kg");
    type ExposureRate = Real(final quantity = "ExposureRate", final unit = "C/(kg.s)");
    type ReynoldsNumber = Real(final quantity = "ReynoldsNumber", final unit = "1");
    type EulerNumber = Real(final quantity = "EulerNumber", final unit = "1");
    type FroudeNumber = Real(final quantity = "FroudeNumber", final unit = "1");
    type GrashofNumber = Real(final quantity = "GrashofNumber", final unit = "1");
    type WeberNumber = Real(final quantity = "WeberNumber", final unit = "1");
    type MachNumber = Real(final quantity = "MachNumber", final unit = "1");
    type KnudsenNumber = Real(final quantity = "KnudsenNumber", final unit = "1");
    type StrouhalNumber = Real(final quantity = "StrouhalNumber", final unit = "1");
    type FourierNumber = Real(final quantity = "FourierNumber", final unit = "1");
    type PecletNumber = Real(final quantity = "PecletNumber", final unit = "1");
    type RayleighNumber = Real(final quantity = "RayleighNumber", final unit = "1");
    type NusseltNumber = Real(final quantity = "NusseltNumber", final unit = "1");
    type BiotNumber = NusseltNumber;
    type StantonNumber = Real(final quantity = "StantonNumber", final unit = "1");
    type FourierNumberOfMassTransfer = Real(final quantity = "FourierNumberOfMassTransfer", final unit = "1");
    type PecletNumberOfMassTransfer = Real(final quantity = "PecletNumberOfMassTransfer", final unit = "1");
    type GrashofNumberOfMassTransfer = Real(final quantity = "GrashofNumberOfMassTransfer", final unit = "1");
    type NusseltNumberOfMassTransfer = Real(final quantity = "NusseltNumberOfMassTransfer", final unit = "1");
    type StantonNumberOfMassTransfer = Real(final quantity = "StantonNumberOfMassTransfer", final unit = "1");
    type PrandtlNumber = Real(final quantity = "PrandtlNumber", final unit = "1");
    type SchmidtNumber = Real(final quantity = "SchmidtNumber", final unit = "1");
    type LewisNumber = Real(final quantity = "LewisNumber", final unit = "1");
    type MagneticReynoldsNumber = Real(final quantity = "MagneticReynoldsNumber", final unit = "1");
    type AlfvenNumber = Real(final quantity = "AlfvenNumber", final unit = "1");
    type HartmannNumber = Real(final quantity = "HartmannNumber", final unit = "1");
    type CowlingNumber = Real(final quantity = "CowlingNumber", final unit = "1");
    type BraggAngle = Angle;
    type OrderOfReflexion = Real(final quantity = "OrderOfReflexion", final unit = "1");
    type ShortRangeOrderParameter = Real(final quantity = "RangeOrderParameter", final unit = "1");
    type LongRangeOrderParameter = Real(final quantity = "RangeOrderParameter", final unit = "1");
    type DebyeWallerFactor = Real(final quantity = "DebyeWallerFactor", final unit = "1");
    type CircularWavenumber = Real(final quantity = "CircularWavenumber", final unit = "m-1");
    type FermiCircularWavenumber = Real(final quantity = "FermiCircularWavenumber", final unit = "m-1");
    type DebyeCircularWavenumber = Real(final quantity = "DebyeCircularWavenumber", final unit = "m-1");
    type DebyeCircularFrequency = Real(final quantity = "AngularFrequency", final unit = "s-1");
    type DebyeTemperature = ThermodynamicTemperature;
    type SpectralConcentration = Real(final quantity = "SpectralConcentration", final unit = "s/m3");
    type GrueneisenParameter = Real(final quantity = "GrueneisenParameter", final unit = "1");
    type MadelungConstant = Real(final quantity = "MadelungConstant", final unit = "1");
    type DensityOfStates = Real(final quantity = "DensityOfStates", final unit = "J-1/m-3");
    type ResidualResistivity = Real(final quantity = "ResidualResistivity", final unit = "Ohm.m");
    type LorenzCoefficient = Real(final quantity = "LorenzCoefficient", final unit = "V2/K2");
    type HallCoefficient = Real(final quantity = "HallCoefficient", final unit = "m3/C");
    type ThermoelectromotiveForce = Real(final quantity = "ThermoelectromotiveForce", final unit = "V");
    type SeebeckCoefficient = Real(final quantity = "SeebeckCoefficient", final unit = "V/K");
    type PeltierCoefficient = Real(final quantity = "PeltierCoefficient", final unit = "V");
    type ThomsonCoefficient = Real(final quantity = "ThomsonCoefficient", final unit = "V/K");
    type RichardsonConstant = Real(final quantity = "RichardsonConstant", final unit = "A/(m2.K2)");
    type FermiEnergy = Real(final quantity = "Energy", final unit = "eV");
    type GapEnergy = Real(final quantity = "Energy", final unit = "eV");
    type DonorIonizationEnergy = Real(final quantity = "Energy", final unit = "eV");
    type AcceptorIonizationEnergy = Real(final quantity = "Energy", final unit = "eV");
    type FermiTemperature = ThermodynamicTemperature;
    type ElectronNumberDensity = Real(final quantity = "ElectronNumberDensity", final unit = "m-3");
    type HoleNumberDensity = Real(final quantity = "HoleNumberDensity", final unit = "m-3");
    type IntrinsicNumberDensity = Real(final quantity = "IntrinsicNumberDensity", final unit = "m-3");
    type DonorNumberDensity = Real(final quantity = "DonorNumberDensity", final unit = "m-3");
    type AcceptorNumberDensity = Real(final quantity = "AcceptorNumberDensity", final unit = "m-3");
    type EffectiveMass = Mass;
    type MobilityRatio = Real(final quantity = "MobilityRatio", final unit = "1");
    type RelaxationTime = Time;
    type CarrierLifeTime = Time;
    type ExchangeIntegral = Real(final quantity = "Energy", final unit = "eV");
    type CurieTemperature = ThermodynamicTemperature;
    type NeelTemperature = ThermodynamicTemperature;
    type LondonPenetrationDepth = Length;
    type CoherenceLength = Length;
    type LandauGinzburgParameter = Real(final quantity = "LandauGinzburgParameter", final unit = "1");
    type FluxiodQuantum = Real(final quantity = "FluxiodQuantum", final unit = "Wb");
  end SIunits;
  package Icons "Library of icons"
    annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100, -100},{80,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{ -100,50},{ -80,70},{100,70},{80,50},{ -100,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{100,70},{100, -80},{80, -100},{80,50},{100,70}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -120,135},{120,70}}, lineColor = {255,0,0}, textString = "%name"),Text(extent = {{ -90,40},{70,10}}, lineColor = {160,160,164}, textString = "Library"),Rectangle(extent = {{ -100, -100},{80,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{ -100,50},{ -80,70},{100,70},{80,50},{ -100,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{100,70},{100, -80},{80, -100},{80,50},{100,70}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -90,40},{70,10}}, lineColor = {160,160,164}, textString = "Library"),Polygon(points = {{ -64, -20},{ -50, -4},{50, -4},{36, -20},{ -64, -20},{ -64, -20}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -64, -20},{36, -84}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid),Text(extent = {{ -60, -24},{32, -38}}, lineColor = {128,128,128}, textString = "Library"),Polygon(points = {{50, -4},{50, -70},{36, -84},{36, -20},{50, -4}}, lineColor = {0,0,0}, fillColor = {192,192,192}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>
This package contains definitions for the graphical layout of
components which may be used in different libraries.
The icons can be utilized by inheriting them in the desired class
using \"extends\" or by directly copying the \"icon\" layer.
</p>

<dl>
<dt><b>Main Author:</b>
<dd><a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a><br>
    Deutsches Zentrum fuer Luft und Raumfahrt e.V. (DLR)<br>
    Oberpfaffenhofen<br>
    Postfach 1116<br>
    D-82230 Wessling<br>
    email: <A HREF=\"mailto:Martin.Otter@dlr.de\">Martin.Otter@dlr.de</A><br>
</dl>

<p>
Copyright &copy; 1998-2009, Modelica Association and DLR.
</p>
<p>
<i>This Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p><br>
</HTML>
", revisions = "<html>
<ul>
<li><i>October 21, 2002</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>
       and <a href=\"http://www.robotic.dlr.de/Christian.Schweiger/\">Christian Schweiger</a>:<br>
       Added new icons <b>Function</b>, <b>Enumerations</b> and <b>Record</b>.</li>
<li><i>June 6, 2000</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Replaced <b>model</b> keyword by <b>package</b> if the main
       usage is for inheriting from a package.<br>
       New icons <b>GearIcon</b> and <b>MotorIcon</b>.</li>
<li><i>Sept. 18, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Renaming package Icon to Icons.
       Model Advanced removed (icon not accepted on the Modelica meeting).
       New model Library2, which is the Library icon with enough place
       to add library specific elements in the icon. Icon also used in diagram
       level for models Info, TranslationalSensor, RotationalSensor.</li>
<li><i>July 15, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Model Caution renamed to Advanced, model Sensor renamed to
       TranslationalSensor, new model RotationalSensor.</li>
<li><i>June 30, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized a first version.</li>
</ul>
<br>
</html>"));
    partial model Info "Icon for an information class"
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,80},{80, -90}}, lineColor = {0,0,255}),Polygon(points = {{ -100,80},{ -90,90},{90,90},{80,80},{ -100,80}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{80, -90},{90, -80},{90,90},{80,80},{80, -90}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -80,70},{60, -70}}, textString = "Info", lineColor = {0,0,255})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,80},{80, -90}}, lineColor = {0,0,255}),Polygon(points = {{ -100,80},{ -90,90},{90,90},{80,80},{ -100,80}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{80, -90},{90, -80},{90,90},{80,80},{80, -90}}, fillColor = {0,0,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -80,70},{60, -70}}, textString = "Info", lineColor = {0,0,255})}), Documentation(info = "<html>
<p>
This icon is designed for an <b>information</b> class.
</p>
</html>"));
    end Info;
    partial package Library "Icon for library"
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100, -100},{80,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{ -100,50},{ -80,70},{100,70},{80,50},{ -100,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{100,70},{100, -80},{80, -100},{80,50},{100,70}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -85,35},{65, -85}}, lineColor = {0,0,255}, textString = "Library"),Text(extent = {{ -120,122},{120,73}}, lineColor = {255,0,0}, textString = "%name")}), Documentation(info = "<html>
<p>
This icon is designed for a <b>library</b>.
</p>
</html>"));
    end Library;
    partial package Library2 "Icon for library where additional icon elements shall be added"
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100, -100},{80,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{ -100,50},{ -80,70},{100,70},{80,50},{ -100,50}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{100,70},{100, -80},{80, -100},{80,50},{100,70}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -120,125},{120,70}}, lineColor = {255,0,0}, textString = "%name"),Text(extent = {{ -90,40},{70,10}}, lineColor = {160,160,164}, textString = "Library")}), Documentation(info = "<html>
<p>
This icon is designed for a <b>package</b> where a package
specific graphic is additionally included in the icon.
</p>
</html>"));
    end Library2;
    partial model Example "Icon for an example model"
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100, -100},{80,50}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{ -100,50},{ -80,70},{100,70},{80,50},{ -100,50}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{100,70},{100, -80},{80, -100},{80,50},{100,70}}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -96,3},{77, -49}}, lineColor = {0,0,255}, textString = "Example"),Text(extent = {{ -120,132},{120,73}}, lineColor = {255,0,0}, textString = "%name")}), Documentation(info = "<html>
<p>
This icon is designed for an <b>Example package</b>,
i.e. a package containing executable demo models.
</p>
</html>"));
    end Example;
    partial function Function "Icon for a function"
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Text(extent = {{ -140,162},{136,102}}, textString = "%name", lineColor = {0,0,255}),Ellipse(extent = {{ -100,100},{100, -100}}, lineColor = {255,127,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Text(extent = {{ -100,100},{100, -100}}, lineColor = {255,127,0}, textString = "f")}), Documentation(info = "<html>
<p>
This icon is designed for a <b>function</b>
</p>
</html>"));
    end Function;
    partial record Record "Icon for a record"
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,50},{100, -100}}, fillColor = {255,255,127}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{ -127,115},{127,55}}, textString = "%name", lineColor = {0,0,255}),Line(points = {{ -100, -50},{100, -50}}, color = {0,0,0}),Line(points = {{ -100,0},{100,0}}, color = {0,0,0}),Line(points = {{0,50},{0, -100}}, color = {0,0,0})}), Documentation(info = "<html>
<p>
This icon is designed for a <b>record</b>
</p>
</html>"));
    end Record;
    type TypeReal "Icon for a Real type"
      extends Real;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {181,181,181}, fillPattern = FillPattern.Solid),Text(extent = {{ -94,94},{94, -94}}, lineColor = {0,0,0}, textString = "R")}), Documentation(info = "<html>
<p>
This icon is designed for a <b>Real</b> type.
</p>
</html>"));
    end TypeReal;
    type TypeInteger "Icon for an Integer type"
      extends Integer;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {181,181,181}, fillPattern = FillPattern.Solid),Text(extent = {{ -94,94},{94, -94}}, lineColor = {0,0,0}, textString = "I")}), Documentation(info = "<html>
<p>
This icon is designed for an <b>Integer</b> type.
</p>
</html>"));
    end TypeInteger;
    type TypeBoolean "Icon for a Boolean type"
      extends Boolean;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {181,181,181}, fillPattern = FillPattern.Solid),Text(extent = {{ -94,94},{94, -94}}, lineColor = {0,0,0}, textString = "B")}), Documentation(info = "<html>
<p>
This icon is designed for a <b>Boolean</b> type.
</p>
</html>"));
    end TypeBoolean;
    type TypeString "Icon for a String type"
      extends String;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -100,100},{100, -100}}, lineColor = {0,0,0}, fillColor = {181,181,181}, fillPattern = FillPattern.Solid),Text(extent = {{ -94,94},{94, -94}}, lineColor = {0,0,0}, textString = "S")}), Documentation(info = "<html>
<p>
This icon is designed for a <b>String</b> type.
</p>
</html>"));
    end TypeString;
    partial model TranslationalSensor "Icon representing translational measurement device"
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -70, -60},{70,20}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Polygon(points = {{0, -40},{ -10, -16},{10, -16},{0, -40}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Line(points = {{0,0},{0, -16}}, color = {0,0,0}),Line(points = {{ -70,0},{0,0}}, color = {0,0,0}),Line(points = {{ -50, -40},{ -50, -60}}, color = {0,0,0}),Line(points = {{ -30, -40},{ -30, -60}}, color = {0,0,0}),Line(points = {{ -10, -40},{ -10, -60}}, color = {0,0,0}),Line(points = {{10, -40},{10, -60}}, color = {0,0,0}),Line(points = {{30, -40},{30, -60}}, color = {0,0,0}),Line(points = {{50, -40},{50, -60}}, color = {0,0,0})}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Documentation(info = "<html>
<p>
This icon is designed for a <b>translational sensor</b> model.
</p>
</html>"));
    end TranslationalSensor;
    partial model RotationalSensor "Icon representing rotational measurement device"
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Ellipse(extent = {{ -70,70},{70, -70}}, lineColor = {0,0,0}, fillColor = {255,255,255}, fillPattern = FillPattern.Solid),Line(points = {{0,70},{0,40}}, color = {0,0,0}),Line(points = {{22.9,32.8},{40.2,57.3}}, color = {0,0,0}),Line(points = {{ -22.9,32.8},{ -40.2,57.3}}, color = {0,0,0}),Line(points = {{37.6,13.7},{65.8,23.9}}, color = {0,0,0}),Line(points = {{ -37.6,13.7},{ -65.8,23.9}}, color = {0,0,0}),Line(points = {{0,0},{9.02,28.6}}, color = {0,0,0}),Polygon(points = {{ -0.48,31.6},{18,26},{18,57.2},{ -0.48,31.6}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -5,5},{5, -5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics), Documentation(info = "<html>
<p>
This icon is designed for a <b>rotational sensor</b> model.
</p>
</html>"));
    end RotationalSensor;
    partial model GearIcon "Icon for gearbox"
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}), graphics = {Rectangle(extent = {{ -90,10},{ -60, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Polygon(points = {{ -60,10},{ -60,20},{ -40,40},{ -40, -40},{ -60, -20},{ -60,10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {128,128,128}),Rectangle(extent = {{ -40,60},{40, -60}}, lineColor = {0,0,0}, pattern = LinePattern.Solid, lineThickness = 0.25, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Polygon(points = {{60,20},{40,40},{40, -40},{60, -20},{60,20}}, lineColor = {128,128,128}, fillColor = {128,128,128}, fillPattern = FillPattern.Solid),Rectangle(extent = {{60,10},{90, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192}),Polygon(points = {{ -60, -90},{ -50, -90},{ -20, -30},{20, -30},{48, -90},{60, -90},{60, -100},{ -60, -100},{ -60, -90}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Documentation(info = "<html>
<p>
This icon is designed for a <b>gearbox</b> model.
</p>
</html>"));
    end GearIcon;
    partial model MotorIcon "Icon for electrical motor"
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {1,1}), graphics = {Rectangle(extent = {{ -100,50},{30, -50}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {255,0,0}),Polygon(points = {{ -100, -90},{ -90, -90},{ -60, -20},{ -10, -20},{20, -90},{30, -90},{30, -100},{ -100, -100},{ -100, -90}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{30,10},{90, -10}}, lineColor = {0,0,0}, fillPattern = FillPattern.HorizontalCylinder, fillColor = {192,192,192})}), Documentation(info = "<html>
<p>
This icon is designed for an <b>electrical motor</b> model.
</p>
</html>"));
    end MotorIcon;
    connector SignalBus "Icon for signal bus"
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}, initialScale = 0.2), graphics = {Rectangle(extent = {{ -20,2},{20, -2}}, lineColor = {255,204,51}, lineThickness = 0.5),Polygon(points = {{ -80,50},{80,50},{100,30},{80, -40},{60, -50},{ -60, -50},{ -80, -40},{ -100,30},{ -80,50}}, lineColor = {0,0,0}, fillColor = {255,204,51}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -65,25},{ -55,15}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -5,25},{5,15}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{55,25},{65,15}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -35, -15},{ -25, -25}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{25, -15},{35, -25}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}, initialScale = 0.2), graphics = {Polygon(points = {{ -40,25},{40,25},{50,15},{40, -20},{30, -25},{ -30, -25},{ -40, -20},{ -50,15},{ -40,25}}, lineColor = {0,0,0}, fillColor = {255,204,51}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -32.5,7.5},{ -27.5,12.5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -2.5,12.5},{2.5,7.5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{27.5,12.5},{32.5,7.5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -17.5, -7.5},{ -12.5, -12.5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{12.5, -7.5},{17.5, -12.5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,70},{150,40}}, lineColor = {0,0,0}, textString = "%name")}), Documentation(info = "<html>
<p>
This icon is designed for a <b>signal bus</b> connector.
</p>
</html>"));
    end SignalBus;
    connector SignalSubBus "Icon for signal sub-bus"
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}, initialScale = 0.1), graphics = {Line(points = {{ -16,2},{16,2}}, color = {255,204,51}, thickness = 0.5),Rectangle(extent = {{ -10,8},{8,0}}, lineColor = {255,204,51}, lineThickness = 0.5),Polygon(points = {{ -80,50},{80,50},{100,30},{80, -40},{60, -50},{ -60, -50},{ -80, -40},{ -100,30},{ -80,50}}, lineColor = {0,0,0}, fillColor = {255,204,51}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -55,25},{ -45,15}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{45,25},{55,15}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -5, -15},{5, -25}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Rectangle(extent = {{ -20,4},{20,0}}, lineColor = {255,204,51}, lineThickness = 0.5)}), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}, grid = {2,2}, initialScale = 0.1), graphics = {Polygon(points = {{ -40,25},{40,25},{50,15},{40, -20},{30, -25},{ -30, -25},{ -40, -20},{ -50,15},{ -40,25}}, lineColor = {0,0,0}, fillColor = {255,204,51}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -22.5,7.5},{ -17.5,12.5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{17.5,12.5},{22.5,7.5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Ellipse(extent = {{ -2.5, -7.5},{2.5, -12.5}}, lineColor = {0,0,0}, fillColor = {0,0,0}, fillPattern = FillPattern.Solid),Text(extent = {{ -150,70},{150,40}}, lineColor = {0,0,0}, textString = "%name")}), Documentation(info = "<html>
<p>
This icon is designed for a <b>sub-bus</b> in a signal connector.
</p>
</html>"));
    end SignalSubBus;
    partial model ObsoleteModel "Icon for an obsolete model (use only for this case)"
      annotation(__Dymola_obsolete = "Only used to mark an obsolete model. Do not use otherwise.", Icon(coordinateSystem(preserveAspectRatio = false, extent = {{ -100, -100},{100,100}}), graphics = {Rectangle(extent = {{ -102,102},{102, -102}}, lineColor = {255,0,0}, pattern = LinePattern.Dash, lineThickness = 0.5)}), Documentation(info = "<html>
<p>
This partial model is intended to provide a <u>default icon
for an obsolete model</u> that will be removed from the
corresponding library in a future release.
<p>
</html>"));
    end ObsoleteModel;
  end Icons;
  package Constants "Library of mathematical constants and constants of nature (e.g., pi, eps, R, sigma)"
    import SI = Modelica.SIunits;
    import NonSI = Modelica.SIunits.Conversions.NonSIunits;
    extends Modelica.Icons.Library2;
    final constant Real e = Modelica.Math.exp(1.0);
    final constant Real pi = 2 * Modelica.Math.asin(1.0);
    final constant Real D2R = pi / 180 "Degree to Radian";
    final constant Real R2D = 180 / pi "Radian to Degree";
    final constant Real eps = 1e-15 "Biggest number such that 1.0 + eps = 1.0";
    final constant Real small = 1e-60 "Smallest number such that small and -small are representable on the machine";
    final constant Real inf = 1e+60 "Biggest Real number such that inf and -inf are representable on the machine";
    final constant Integer Integer_inf = 1073741823 "Biggest Integer number such that Integer_inf and -Integer_inf are representable on the machine";
    final constant SI.Velocity c = 299792458 "Speed of light in vacuum";
    final constant SI.Acceleration g_n = 9.80665 "Standard acceleration of gravity on earth";
    final constant Real G(final unit = "m3/(kg.s2)") = 6.6742e-11 "Newtonian constant of gravitation";
    final constant SI.FaradayConstant F = 96485.3399 "Faraday constant, C/mol";
    final constant Real h(final unit = "J.s") = 6.6260693e-34 "Planck constant";
    final constant Real k(final unit = "J/K") = 1.3806505e-23 "Boltzmann constant";
    final constant Real R(final unit = "J/(mol.K)") = 8.314472 "Molar gas constant";
    final constant Real sigma(final unit = "W/(m2.K4)") = 5.6704e-08 "Stefan-Boltzmann constant";
    final constant Real N_A(final unit = "1/mol") = 6.0221415e+23 "Avogadro constant";
    final constant Real mue_0(final unit = "N/A2") = 4 * pi * 1e-07 "Magnetic constant";
    final constant Real epsilon_0(final unit = "F/m") = 1 / (mue_0 * c * c) "Electric constant";
    final constant NonSI.Temperature_degC T_zero =  -273.15 "Absolute zero temperature";
    annotation(Documentation(info = "<html>
<p>
This package provides often needed constants from mathematics, machine
dependent constants and constants from nature. The latter constants
(name, value, description) are from the following source:
</p>

<dl>
<dt>Peter J. Mohr and Barry N. Taylor (1999):</dt>
<dd><b>CODATA Recommended Values of the Fundamental Physical Constants: 1998</b>.
    Journal of Physical and Chemical Reference Data, Vol. 28, No. 6, 1999 and
    Reviews of Modern Physics, Vol. 72, No. 2, 2000. See also <a href=
\"http://physics.nist.gov/cuu/Constants/\">http://physics.nist.gov/cuu/Constants/</a></dd>
</dl>

<p>CODATA is the Committee on Data for Science and Technology.</p>

<dl>
<dt><b>Main Author:</b></dt>
<dd><a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a><br>
    Deutsches Zentrum f&uuml;r Luft und Raumfahrt e. V. (DLR)<br>
    Oberpfaffenhofen<br>
    Postfach 11 16<br>
    D-82230 We&szlig;ling<br>
    email: <a href=\"mailto:Martin.Otter@dlr.de\">Martin.Otter@dlr.de</a></dd>
</dl>


<p>
Copyright &copy; 1998-2009, Modelica Association and DLR.
</p>
<p>
<i>This Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b>
<a href=\"Modelica://Modelica.UsersGuide.ModelicaLicense2\">here</a>.</i>
</p><br>
</html>
", revisions = "<html>
<ul>
<li><i>Nov 8, 2004</i>
       by <a href=\"http://www.robotic.dlr.de/Christian.Schweiger/\">Christian Schweiger</a>:<br>
       Constants updated according to 2002 CODATA values.</li>
<li><i>Dec 9, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Constants updated according to 1998 CODATA values. Using names, values
       and description text from this source. Included magnetic and
       electric constant.</li>
<li><i>Sep 18, 1999</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Constants eps, inf, small introduced.</li>
<li><i>Nov 15, 1997</i>
       by <a href=\"http://www.robotic.dlr.de/Martin.Otter/\">Martin Otter</a>:<br>
       Realized.</li>
</ul>
</html>"), Invisible = true, Icon(coordinateSystem(preserveAspectRatio = true, extent = {{ -100, -100},{100,100}}), graphics = {Line(points = {{ -34, -38},{12, -38}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -20, -38},{ -24, -48},{ -28, -56},{ -34, -64}}, color = {0,0,0}, thickness = 0.5),Line(points = {{ -2, -38},{2, -46},{8, -56},{14, -64}}, color = {0,0,0}, thickness = 0.5)}), Diagram(graphics = {Rectangle(extent = {{200,162},{380,312}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{200,312},{220,332},{400,332},{380,312},{200,312}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Polygon(points = {{400,332},{400,182},{380,162},{380,312},{400,332}}, fillColor = {235,235,235}, fillPattern = FillPattern.Solid, lineColor = {0,0,255}),Text(extent = {{210,302},{370,272}}, lineColor = {160,160,164}, textString = "Library"),Line(points = {{266,224},{312,224}}, color = {0,0,0}, thickness = 1),Line(points = {{280,224},{276,214},{272,206},{266,198}}, color = {0,0,0}, thickness = 1),Line(points = {{298,224},{302,216},{308,206},{314,198}}, color = {0,0,0}, thickness = 1),Text(extent = {{152,412},{458,334}}, lineColor = {255,0,0}, textString = "Modelica.Constants")}));
  end Constants;
end Modelica;
